/* 
 *
 * Sample userintf.c for Linux
 * 
 * This is $Revision: 1.3 $
 *
 * $Log: userintf.c,v $
 * Revision 1.3  2001/05/29 09:13:36  cyeoh
 * Merge for consistency
 *
 * Revision 1.2  2001/05/29 08:07:28  cyeoh
 * Update for UNIX98 mode
 *
 * Revision 1.1  2000/11/09 05:33:35  cyeoh
 * Merge of userintf.c features needed for both vsthlite and vsx-pcts
 *
 *
 */

#ifdef __linux__

/* modify the following for different mountable filesystem types */

#define __USERINTF_FSTYPE "ext2"

#endif

/*
*      SCCS:   @(#)common/vport(userintf.top)	4.2 (04/22/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*/
/***********************************************************************

NAME:		userintf.c - user-modifiable interface routines

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	This file contains functions which perform test-setup tasks in
	an implementation-defined manner.  The contents of this template
	vary according to the subsets selected by the user.  Only
	the functions needed by testsets in the selected subsets are
	included.

	As supplied these routines make use of system calls commonly
	found on many UNIX systems.  If these are not appropriate the
	routines should be modified as necessary before the VSX test
	suite is installed.

	Descriptions of the individual interfaces may be found
	with the corresponding function code below.

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	December 1996
MODIFICATIONS:

************************************************************************/

#ifndef __linux__

#undef _POSIX_SOURCE	/* don't want namespace restrictions in here */
#undef _POSIX_C_SOURCE
#undef _XOPEN_SOURCE

#define _ALL_SOURCE	/* change to suitable feature test macro */

#else

/*
 * for Linux switch on extensions at least to POSIX and X/Open
 * compile 
 */

#define _POSIX_SOURCE	

#ifndef _XOPEN_SOURCE
#define _XOPEN_SOURCE  500
#endif

#endif

#include <std.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <setprv.h>
#include <xnfs.h>

/*
 *	Setprv() provides the current process with "appropriate privileges".
 *	The argument specifies what privilege is being requested.
 *	It returns 0 for success, -1 for failure.
 */

public int
setprv(prvtype)
int prvtype;
{
	switch (prvtype)
	{
	case PRV_DEVICE :	/* access device files */
		/* no privilege required */
		return 0;

	case PRV_SETID :	/* change to any uid/gid */
		/* fallthrough */

	case PRV_MOUNT :	/* mount and unmount file systems */
		/* fallthrough */

	case PRV_LINKDIR :	/* create links to directories */
		/* fallthrough */

	case PRV_ACCESS :	/* unrestricted access to files */
		/* fallthrough */

	case PRV_CHOWN :	/* change owner/group/mode of ANY file */
		/* fallthrough */

	case PRV_SETGRPS :	/* set supplementary group IDs */
		/* fallthrough */

	case PRV_NEWROOT :	/* set root directory of process */
		/* fallthrough */

	case PRV_NICE :		/* change process priority */
		/* fallthrough */

	case PRV_KILL :		/* send signal to any process */
		/* fallthrough */

	case PRV_IPC :		/* allow unrestricted IPC access */
		/* fallthrough */

	case PRV_ULIMIT :	/* increase file size limit */
		/* fallthrough */

	case PRV_LIMITS :	/* privileged setrlimit() */
		/* fallthrough */

	case PRV_MEMLOCK :	/* memory locking */
		/* fallthrough */

	case PRV_SETTIME :	/* set (real-time) clocks */
		/* fallthrough */

	case PRV_SETRTSCHED :	/* set (real-time) process scheduling params */
		/* fallthrough */

	case PRV_GETRTSCHED :	/* get (real-time) process scheduling params */
		/* fallthrough */

	case PRV_SETTHRSCHED :	/* set threads scheduling params */
		/* fallthrough */

	case PRV_GETTHRSCHED :	/* get threads scheduling params */
		/* fallthrough */

	case PRV_ASSIGN :	/* assign privileges to an executable file */

		if (geteuid() == 0)
			return 0;
		break;
	
	default :
		(void) fprintf(stderr,
			"Error in vport/userintf: unknown privilege request (%d) in setprv()\n",
			prvtype);
		break;
	}

	return -1;
}

/*
 *	Unsetprv() removes the specified privilege from the current process.
 *	The argument specifies what privilege is to be removed.
 *	It returns 0 for success, -1 for failure.  (If the process already
 *	does not have the privilege, this is considered success).
 */

public int
unsetprv(prvtype)
int prvtype;
{
	switch (prvtype)
	{
	case PRV_DEVICE :	/* access device files */
		/* no privilege required */
		return 0;

	case PRV_SETID :	/* change to any uid/gid */
		/* fallthrough */

	case PRV_MOUNT :	/* mount and unmount file systems */
		/* fallthrough */

	case PRV_LINKDIR :	/* create links to directories */
		/* fallthrough */

	case PRV_ACCESS :	/* unrestricted access to files */
		/* fallthrough */

	case PRV_CHOWN :	/* change owner/group/mode of ANY file */
		/* fallthrough */

	case PRV_SETGRPS :	/* set supplementary group IDs */
		/* fallthrough */

	case PRV_NEWROOT :	/* set root directory of process */
		/* fallthrough */

	case PRV_NICE :		/* change process priority */
		/* fallthrough */

	case PRV_KILL :		/* send signal to any process */
		/* fallthrough */

	case PRV_IPC :		/* allow unrestricted IPC access */
		/* fallthrough */

	case PRV_ULIMIT :	/* increase file size limit */
		/* fallthrough */

	case PRV_LIMITS :	/* privileged setrlimit() */
		/* fallthrough */

	case PRV_MEMLOCK :	/* memory locking */
		/* fallthrough */

	case PRV_SETTIME :	/* set (real-time) clocks */
		/* fallthrough */

	case PRV_SETRTSCHED :	/* set (real-time) process scheduling params */
		/* fallthrough */

	case PRV_GETRTSCHED :	/* get (real-time) process scheduling params */
		/* fallthrough */

	case PRV_SETTHRSCHED :	/* set threads scheduling params */
		/* fallthrough */

	case PRV_GETTHRSCHED :	/* get threads scheduling params */
		/* fallthrough */

	case PRV_ASSIGN :	/* assign privileges to an executable file */

		if (setuid(getuid()) != -1)
			return 0;
		break;
	
	default :
		(void) fprintf(stderr,
			"Error in vport/userintf: unknown privilege request (%d) in unsetprv()\n",
			prvtype);
		break;
	}

	return -1;
}

/*
 *	Prv_assign() assigns "appropriate privileges" to an executable file.
 *	The arguments are the file name and a zero-terminated array of the
 *	privileges to be assigned.
 *	It returns 0 for success, -1 for failure.
 */

public int
prv_assign(file, prvtypes)
char *file;
int *prvtypes;
{
	int prvtype, ret = 0;

	while ((prvtype = *prvtypes++) != 0)
	{
		switch (prvtype)
		{
		case PRV_DEVICE :	/* access device files */
			/* no privilege required */
			break;

		case PRV_SETID :	/* change to any uid/gid */
			/* fallthrough */

		case PRV_MOUNT :	/* mount and unmount file systems */
			/* fallthrough */

		case PRV_LINKDIR :	/* create links to directories */
			/* fallthrough */

		case PRV_ACCESS :	/* unrestricted access to files */
			/* fallthrough */

		case PRV_CHOWN :	/* change owner/group/mode of ANY file */
			/* fallthrough */

		case PRV_SETGRPS :	/* set supplementary group IDs */
			/* fallthrough */

		case PRV_NEWROOT :	/* set root directory of process */
			/* fallthrough */

		case PRV_NICE :		/* change process priority */
			/* fallthrough */

		case PRV_KILL :		/* send signal to any process */
			/* fallthrough */

		case PRV_IPC :		/* allow unrestricted IPC access */
			/* fallthrough */

		case PRV_ULIMIT :	/* increase file size limit */
			/* fallthrough */

		case PRV_LIMITS :	/* privileged setrlimit() */
			/* fallthrough */

		case PRV_MEMLOCK :	/* memory locking */
			/* fallthrough */

		case PRV_SETTIME :	/* set (real-time) clocks */
			/* fallthrough */

		case PRV_SETRTSCHED :	/* set (real-time) process scheduling params */
			/* fallthrough */

		case PRV_GETRTSCHED :	/* get (real-time) process scheduling params */
			/* fallthrough */

		case PRV_SETTHRSCHED :	/* set threads scheduling params */
			/* fallthrough */

		case PRV_GETTHRSCHED :	/* get threads scheduling params */
		    /* fallthrough */

		case PRV_ASSIGN :	/* assign privileges to an executable file */

			break;
		
		default :
			(void) fprintf(stderr,
				"Error in vport/userintf: unknown privilege request (%d) in prv_assign()\n",
				prvtype);
			ret = -1;
			break;
		}
	}


	return ret;
}

#ifdef TET_POSIX_THREADS


/******	User editable routines for VSTH
	The Open Group Threads Test Suite
	@(#)vsth	1.8							******/

#include <pthread.h>

/******	Set the process concurrency level to the maximum allowed without
	requiring appropriate privilege.				******/

void
vsth_setconcurrency()
{

#if	( TEST_XPG > 4 )

/*
	int		level;
	static int	done;

	level = 8;

	if ( ! done )
	{
		done = 1;
		tet_printf("setting concurrency level to %d", level);
	}
	(void) pthread_setconcurrency( level );
*/

#else

#endif
}

/******	An atomic increment - not currently used but may be introduced in a
	future release.

	Increment *arg in an atomic manner and return *arg		******/

int
vsth_autoinc( int *arg )
{
	/* example only - must be atomic */
	return(++*arg);
}

#endif /* TET_POSIX_THREADS*/
/*
*      SCCS:   @(#)install/userintf/gen_mnt	1.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*/
/***********************************************************************

NAME:		gen_mnt - user-modifiable mount/unmount routines
PROJECT:	VSX (X/OPEN Validation Suite)
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	December 1996
MODIFICATIONS:

************************************************************************/

/*
 *	Mnt_rw() mounts the file system specified by "spec" onto the
 *	directory "dir" for reading and writing.
 *	It returns 0 for success, -1 for failure.
 */

#ifdef __linux__
#include <sys/mount.h>
#endif

public int
mnt_rw(spec, dir)
char *spec;
char *dir;
{
	int ret;

#ifdef __linux__
        /* file system independent mount for filesystem */
        ret = mount(spec, dir, (char *) __USERINTF_FSTYPE, MS_MGC_VAL, 0);
#else
	ret = mount(spec, dir, 0);
#endif
	if (ret == -1 && errno == EBUSY)
	{
		(void) unmnt(spec, dir);
		errno = 0;
#ifdef __linux__
        /* file system independent mount for filesystem */
        ret = mount(spec, dir, __USERINTF_FSTYPE, MS_MGC_VAL, 0);
#else
		ret = mount(spec, dir, 0);
#endif
	}

	return ret;
}

/*
 *	Mnt_ro() mounts the file system specified by "spec" onto the
 *	directory "dir" for reading only.
 *	It returns 0 for success, -1 for failure.
 */

public int
mnt_ro(spec, dir)
char *spec;
char *dir;
{
	int ret;

#ifdef __linux__
        /* file system independent mount for filesystem */
        ret = mount(spec, dir, __USERINTF_FSTYPE, MS_MGC_VAL|MS_RDONLY, 0);
#else
	ret = mount(spec, dir, 1);
#endif
	if (ret == -1 && errno == EBUSY)
	{
		(void) unmnt(spec, dir);
		errno = 0;
#ifdef __linux__
        ret = mount(spec, dir, __USERINTF_FSTYPE, MS_MGC_VAL|MS_RDONLY, 0);
#else
		ret = mount(spec, dir, 1);
#endif
	}

	return ret;
}

/*
 *	Unmnt() unmounts the file system specified by "spec" from the
 *	directory "dir" where it has previously been mounted.
 *	It returns 0 for success, -1 for failure.
 */

/* ARGSUSED */
public int
unmnt(spec, dir)
char *spec;
char *dir;
{
	umount(dir);
	return 0;
}

/*
*      SCCS:   @(#)install/userintf/gen_termios	1.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*/
/***********************************************************************

NAME:		gen_termios - user-modifiable terminal interface routines
PROJECT:	VSX (X/OPEN Validation Suite)
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	December 1996
MODIFICATIONS:

************************************************************************/

/*
 *	Openctl() opens the terminal device "spec" with the specified flags.
 *	This then becomes the controlling terminal for the current process.
 *	If it cannot become the controlling terminal, openctl() must still
 *	open the device as an ordinary terminal.
 *	Openctl() returns the open file descriptor to the terminal,
 *	or -1 on failure.
 */
		
public int
openctl(spec, oflag)
char *spec;
int  oflag;
{
	/* Note: setsid() will have been called before this routine */

	int fd;

	/* allow for modem line delay if there has just been a close() */
	(void) sleep(1);

	fd = open(spec, oflag, 0);

	/* Make sure failure is not just because the device cannot
	   become the controlling terminal */
	
	if (fd < 0 && errno != EINTR)
		fd = open(spec, oflag|O_NOCTTY, 0);

	return fd;
}

/*
 *	Openpty() opens the master and slave sides of a pseudo-terminal.
 *	Both device names are supplied, but if master pty's are obtained
 *	from a clone device the slave name is a dummy which must be
 *	overwritten with the real device name.  If cntrl is true
 *	the slave must be opened as a controlling terminal by calling
 *	openctl() instead of plain open().  Both devices must be opened
 *	for reading and writing.  On many systems master devices can
 *	only be opened once, giving EBUSY for example on subsequent
 *	opens.  If openpty() encounters a busy master device, it must
 *	simply open the slave.
 *
 *	The open file descriptors are returned via the pointers mfdp and
 *	sfdp.  Openpty() returns 0 for success or -1 for failure.
 *
 *	This routine is only called if the VSX_MASTER_TTY and
 *	VSX_MASTER_LOOP parameters are set.
 */

/* these defined on for Linux using UNIX98 pseudo ttys */

#ifdef __linux__
#define CLONE_MPTY
#define STREAMS_PTY
#endif

/* #define CLONE_MPTY	/* uncomment if master pty's are obtained from a
		 	   clone device (e.g. /dev/ptmx) */
/* #define STREAMS_PTY	/* uncomment if pty's use STREAMS */

#ifdef STREAMS_PTY
#include <sys/stropts.h>
#endif

public int
openpty(master, slave, mfdp, sfdp, cntrl)
char *master;
char *slave;
int *mfdp;
int *sfdp;
int cntrl;
{
	*mfdp = open(master, O_RDWR|O_NOCTTY);
	if (*mfdp < 0 && errno != EBUSY)
		return -1;

#ifdef CLONE_MPTY
	if (*mfdp >= 0)
	{
		char *newslave;
		size_t len;
		static struct { char *ptr; size_t len; } slavelen[2];
		extern char *ptsname();

		(void) grantpt(*mfdp);   /* change permission of slave */
		(void) unlockpt(*mfdp);  /* unlock slave */
		newslave=ptsname(*mfdp); /* get a slave */
		if (slavelen[0].ptr == NULL)
		{
			/* remember length of original dummy name
			   to be used if called again */
			slavelen[0].ptr = slave;
			slavelen[0].len = strlen(slave);
		}
		else if (slave != slavelen[0].ptr && slavelen[1].ptr == NULL)
		{
			/* remember length of original dummy name
			   to be used if called again */
			slavelen[1].ptr = slave;
			slavelen[1].len = strlen(slave);
		}
		if (slave == slavelen[0].ptr)
			len = slavelen[0].len;
		else
			len = slavelen[1].len;
		if (strlen(newslave) > len)
		{
			(void) fprintf(stderr,
				"Error in vport/userintf: dummy slave name too short to take \"%s\" in openpty()\n",
				newslave);
			return -1;
		}
		(void) strcpy(slave, newslave);
	}
#endif

	if (cntrl)
		*sfdp = openctl(slave, O_RDWR);
	else
		*sfdp = open(slave, O_RDWR|O_NOCTTY);

	if (*sfdp < 0)
		return -1;

#ifdef STREAMS_PTY
	if (*mfdp >= 0)
	{
		(void) ioctl(*sfdp, I_PUSH, "ptem");   /* pseudo tty emulator */
		(void) ioctl(*sfdp, I_PUSH, "ldterm"); /* line discipline */
	}
#endif
	
	return 0;
}

/*
 *	Ptygetattr() obtains terminal attributes for the slave pseudo-
 *	terminal corresponding to the master device addressed by mfd.
 *	It returns 0 for success, -1 for failure.
 */

public int
ptygetattr(mfd, termios_p)
int mfd;
struct termios *termios_p;
{
#ifdef STREAMS_PTY
	extern char *ptsname();
	char *slave;
	int ret, sfd = -1;
	
	/* Obtain slave device name and call tcgetattr() on the slave */
	slave = ptsname(mfd);
	if (slave && (sfd = open(slave, O_RDWR)) >= 0)
		ret = tcgetattr(sfd, termios_p);
	else
		ret = -1;
	if (sfd >= 0)
		(void) close(sfd);
	return ret;
#else
	/* This code assumes tcgetattr() on the master returns the
	   settings of the slave (true for BSD-style pseudo-terminals) */
	return tcgetattr(mfd, termios_p);
#endif
}

/*
*      SCCS:   @(#)install/userintf/gen_newroot	1.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*/
/***********************************************************************

NAME:		gen_newroot - user-modifiable newroot() routine
PROJECT:	VSX (X/OPEN Validation Suite)
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	December 1996
MODIFICATIONS:

************************************************************************/

/*
 *	Newroot() causes "path" to become the root directory for the
 *	current process.
 *	Newroot() is only called after setprv(PRV_NEWROOT) since setting
 *	the root directory is usually a privileged operation.
 *	It returns 0 for success, -1 for failure.
 */

public int
newroot(path)
char *path;
{
	return chroot(path);
}

/*
*      SCCS:   @(#)install/userintf/gen_setlimit	1.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*/
/***********************************************************************

NAME:		gen_setlimit - user-modifiable setlimit() routine
PROJECT:	VSX (X/OPEN Validation Suite)
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	December 1996
MODIFICATIONS:

************************************************************************/

/*
 *	Setlimit() is used to reduce the output file size limit for 
 *	the process.  The argument is the value to be set in units of
 *	512 byte blocks, and will always be >= VSX_ULIMIT_BLKS.
 *	If the system supports the SIGXFSZ signal, setlimit() should
 *	set it to be ignored.
 *	Setlimit() returns the new limit, or -1L for failure.
 */

#include <signal.h>
#include <ulimit.h>

public long
setlimit(lim)
long lim;
{
#ifdef SIGXFSZ
	struct sigaction sact;
	sact.sa_handler = SIG_IGN;
	sact.sa_flags = 0;
	(void) sigemptyset(&sact.sa_mask);
	if (sigaction(SIGXFSZ, &sact, (struct sigaction *)0) == -1)
		return -1L;
#endif
	return ulimit(UL_SETFSIZE, lim);
}

/*
*      SCCS:   @(#)install/userintf/gen_pathres	1.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*/
/***********************************************************************

NAME:		gen_pathres - user-modifiable pathname resolution routines
PROJECT:	VSX (X/OPEN Validation Suite)
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	December 1996
MODIFICATIONS:

************************************************************************/

/*
 *	Pathdepth() is used to determine the depth of pathnames
 *	beginning with "//" below the root directory of the process.
 *	The return value is such that a path equivalent to "/usr"
 *	gives a value of 1, one equivalent to "/usr/bin" gives 2, etc.
 *	The default code supplied is suitable for systems which use
 *	"//" to refer to a directory level above the root, i.e. when
 *	the root directory is referred to by "//somename".
 */

public int
pathdepth(path)
char *path;
{
	char *p;
	int dircnt = 0;

	/* count '/' characters after the initial "//" */
	for (p = &path[2]; *p != '\0'; p++)
	{
		if (*p == '/')
		{
			++dircnt;
			/* skip multiple '/'s */
			while (*(p+1) == '/')
				++p;
			/* watch out for trailing '/' */
			if (*(p+1) == '\0')
				--dircnt;
		}
	
	}

	return dircnt;
}

/*
*      SCCS:   @(#)install/userintf/base_setgrps	Rel 4.4.1 (06/09/97)
*
*	UniSoft Ltd., London, England
*/
/***********************************************************************

NAME:		base_setgrps - user-modifiable setgrps() routine
PROJECT:	VSX (X/OPEN Validation Suite)
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	December 1996
MODIFICATIONS:

************************************************************************/

/*
 *	Setgrps() sets the current groups list to the group id's in
 *	"grparray".  This array should contain "ngrps" entries.
 *	Setgrps() is only called after setprv(PRV_SETGRPS) since setting
 *	groups is usually a privileged operation.
 *	It returns 0 for success, -1 for failure.
 *
 *	If the implementation does not support setting the supplementary
 *	groups list, then setgrps() must return -1 with errno set to ENOSYS.
 */

public int
setgrps(ngrps, grparray)
int   ngrps;
gid_t grparray[];
{
	/* If setting supplementary groups is not supported replace
	   the line below with:

		errno = ENOSYS;
		return -1;
	*/

	return setgroups(ngrps, grparray);

	/* If setgroups() takes an array of integers of a different
	   size to gid_t, the gid_t array will need to be copied.
	   In this case, remove the line above and the #if/#endif
	   around the following, and alter the typedef as necesary:
	 */

#if 0
	typedef int GID_T;
	GID_T *gcopy, gdummy;
	int i, rval;

	if (ngrps > 0)
	{
		gcopy = (GID_T *)malloc((size_t)(ngrps * sizeof(GID_T)));
		if (gcopy == NULL)
			return -1;
	}
	else
		gcopy = &gdummy;

	for (i=0; i<ngrps; i++)
		gcopy[i] = grparray[i];

	rval = setgroups(ngrps, gcopy);

	if (ngrps > 0)
		free((void *)gcopy);

	return rval;
#endif /* 0 */
}

/*
*      SCCS:   @(#)install/userintf/base_cputime	Rel 4.4.3 (07/20/98)
*
*	UniSoft Ltd., London, England
*/
/***********************************************************************

NAME:		base_cputime - user-modifiable use-CPU-time routines
PROJECT:	VSX (X/OPEN Validation Suite)
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	December 1996
MODIFICATIONS:

	Geoff Clare, October 1997
		Increase default amount of processing performed.

************************************************************************/

/*
 *	Sys_call() makes one or more calls to an interface which is
 *	known to be a system call.  I.e. after several calls the system
 *	CPU time for the process will have increased measurably.
 *	It returns 0 for success, -1 for failure.
 */

public int
sys_call()
{
	int i, ret;

	for (i = 0; i < 10; i++)
		ret = access("/", F_OK);

	return ret;
}

/*
 *	Use_time() performs processing which will accumulate
 *	user CPU time.  The number of iterations may be	adjusted
 *	if necessary.
 */

public void
use_time()
{
	int i;
	unsigned int dummy = 0;

	for (i = 0; i < 1000; i++)
		dummy = dummy + i / (dummy + 1) * (dummy % 3);
}

