/*
*	SCCS:	@(#)	INC/dbug.h		4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************
 *
 *  FILE
 *
 *	dbug.h    user include file for programs using the dbug package
 *
 *  SYNOPSIS
 *
 *	#include <dbug.h>
 *
 *  DESCRIPTION
 *
 *	Programs which use the dbug package must include this file.
 *	It contains the appropriate macros to call support routines
 *	in the dbug runtime library.
 *
 *	All externally visible symbol names follow the pattern
 *	"db_xxx..xx_" to minimize the possibility of a dbug package
 *	symbol colliding with a user defined symbol.
 *	
 *  AUTHOR
 *
 *	Fred Fish, UniSoft Systems, Berkeley, Ca.
 *
 *  MODIFICATIONS
 *
 *	Extensively modified for VSX.
 *
 *	Steve Ratcliffe, 26 Feb 1990
 *	Add prototypes for varadic functions.
 *
 *	Geoff Clare, 5 June 1990
 *	Change names from "_db_" to "db_" because "_db_" is in the
 *	namespace reserved for the implementation
 *
 *	Geoff Clare, 26 Nov 1990
 *	Add path trace macros and globok declaration
 *
 *	Geoff Clare, 17 Dec 1991
 *	Rename Er_rpt, In_rpt, Xs_rpt and Xx_rpt to
 *	allow for case-insensitive linkers.
 *
 *	Geoff Clare, 30 Oct 1992
 *	Add DBUG_TET_EXIT
 *
 *	Geoff Clare, 4 Feb 1993
 *	Move sigsetjmp() call from DBUG_SETJMP macro into db_setjmp().
 *	Move siglongjmp() call from DBUG_LONGJMP macro into db_longjmp().
 *
 *	Geoff Clare, 20 Dec 1993
 *	Change DBUG_EXIT() to use tet_exit() for DTET2.
 *	DBUG_OFF compiler flag no longer supported.
 *
 *	Geoff Clare, 20 June 1994
 *	Ensure sigsetjmp() is used in line with ANSI C constraints.

 *	Geoff Clare, 24 May 1996
 *	TenDRA printf format string checking.
 */

#ifdef __TenDRA__
#pragma TenDRA type db_printf_fmt for ... printf
#else
#define db_printf_fmt char *
#endif

/*
 *	Internally used dbug variables which must be global.
 */

extern int db_line;
extern int db_on;			/* TRUE if debug currently enabled */
extern int db_lev;			/* Level of debug output */
extern char *db_process;		/* Name of current process */
extern int db_keyword ();		/* Accept/reject keyword */
extern void db_push ();			/* Push state, set up new state */
extern void db_pop ();			/* Pop previous debug state */
extern void db_enter ();		/* New user function entered */
extern void db_return ();		/* User function return */
#if TEST_ANSI
extern void db_printf (char*, db_printf_fmt, ...); /* Print debug output */
#else
extern void db_printf ();		/* Print debug output */
#endif
extern void db_vprintf ();		/* Vprint debug output */
extern void db_setjmp ();		/* Save debugger environment */
extern void db_longjmp ();		/* Restore debugger environment */

/*
 *	These macros provide a user interface into functions in the
 *	dbug runtime support library.  They isolate users from changes
 *	in the MACROS and/or runtime support.
 *
 *	The symbols "__LINE__" and "__FILE__" are expanded by the
 *	preprocessor to the current source file line number and file
 *	name respectively.
 *
 *	WARNING ---  Because the DBUG_ENTER macro allocates space on
 *	the user function's stack, it must precede any executable
 *	statements in the user function.
 *
 */

#define DBUG_DECL \
	char *db_func, *db_file; \
	int db_level, db_oldlev;
#define DBUG_ENTER(a) \
	DBUG_DECL \
	db_oldlev = db_lev; db_lev = 1; \
	db_enter (a,__FILE__,__LINE__,&db_func,&db_file,&db_level)
#define DBUG_LENTER(a,lev) \
	DBUG_DECL \
	 db_oldlev = db_lev; db_lev = lev; \
	 db_enter (a,__FILE__,__LINE__,&db_func,&db_file,&db_level)
#define DBUG_LEAVE \
	(db_return (__LINE__, &db_func, &db_file, &db_level), db_lev = db_oldlev)
#define DBUG_RETURN(a1) return (DBUG_LEAVE, (a1))
#define DBUG_VOID_RETURN {DBUG_LEAVE; return;}
#if TEST_XNFS
#  ifndef DTET1
#    define DBUG_EXIT(a1) (DBUG_LEAVE, tet_exit(a1))
     /* for use in children of tet_fork() only - not plain fork() */
#  else
#    define DBUG_EXIT(a1) (DBUG_LEAVE, exit(a1))
     /* tet_fork() in old DTET does not make separate connection for child */
#  endif
#  define DBUG_TET_EXIT(a1) (DBUG_LEAVE, tet_exit(a1))
   /* for use in processes executed with tet_exec() */
#else
#  define DBUG_EXIT(a1) (DBUG_LEAVE, exit(a1))
#  define DBUG_TET_EXIT(a1) DBUG_EXIT(a1)
#endif
#define DBUG_EXECUTE(keyword,a1) \
	if (db_on) {if (db_keyword (keyword)) { a1 }}
#define DBUG_PRINTF \
	db_line = __LINE__, db_printf
#define DBUG_PUSH(a1) db_push (a1)
#define DBUG_POP() db_pop ()
#define DBUG_PROCESS(a1) (db_process = a1)
#define DBUG_SETJMP(a1) sigsetjmp(a1, (db_setjmp(),1))
#define DBUG_LONGJMP(a1,a2) siglongjmp(a1, (db_longjmp(),a2))

/* Reporting functions need to include line number for debugging */
#define xs_rpt()		db_line = __LINE__, xs_dbrpt()
#define xx_rpt(code)	db_line = __LINE__, xx_dbrpt(code)
#define er_rpt(reason)	db_line = __LINE__, er_dbrpt(reason)
#define in_rpt		db_line = __LINE__, in_dbrpt
#if TEST_ANSI
extern void	in_dbrpt(db_printf_fmt, ...);
#else
extern void	in_dbrpt();
#endif
extern void	er_dbrpt();
extern void	xs_dbrpt();
extern void	xx_dbrpt();

/* path tracing macros and declarations */

extern	int	globok;	/* set by library functions before return */

#define	PATH_TRACE	(pathok++, \
		DBUG_PRINTF("trace", "path counter %d", pathok))
#define	JUMP_TRACE	(jumpok++, \
		DBUG_PRINTF("trace", "jump counter %d", jumpok))
#define	PATH_XS_RPT(nn)	\
{ \
	int ExP_CnT = (nn); \
	if (pathok == ExP_CnT) \
		xs_rpt(); \
	else \
	{ \
		xx_rpt(DELETION); \
		in_rpt("path tracing error: path counter %d, expected %d", \
			pathok, ExP_CnT); \
	} \
}
#define	JUMP_XS_RPT(nn)	\
{ \
	int ExP_CnT = (nn); \
	if (jumpok == ExP_CnT) \
		xs_rpt(); \
	else \
	{ \
		xx_rpt(DELETION); \
		in_rpt("path tracing error: jump counter %d, expected %d", \
			jumpok, ExP_CnT); \
	} \
}

/* use after a function call to test and clear globok
	this macro always leaves globok set to 0; it is written this way so as
	to avoid lint "null effect" messages */
#define PATH_FUNC_TRACE \
	(globok = (globok == 1 ? (PATH_TRACE, 0) : \
	    (DBUG_PRINTF("trace", "path check failed in function call"), 0)))

/* use in a library function to set globok according to whether or not the
	function's path check was successful */
#define PATH_FUNC_RPT(nn)	(globok = (pathok == (nn)) ? 1 : 0)

/* use in a setup function to set globok according to whether or not the
	setup path check was successful */
#define PATH_SETUP_RPT(nn)	(setupok = (pathok == (nn)) ? 0 : -1)
