/*
 *      SCCS: @(#)  INC/intf.h 4.1 (01/21/97)  VSXgen release 1.4
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

/***********************************************************************

NAME:		intf.h

PROJECT:	XNFS (X/OPEN NFS tests)

DESCRIPTION:	Declare interface functions for XNFS protocol tests

AUTHOR:		Matthew Newman, UniSoft Ltd.
DATE CREATED:
MODIFICATIONS:
		Geoff Clare, 15 June 1993
			Add declarations of xnfs_lckename(), xnfs_lckname()
			and xnfs_nlm_svc().

************************************************************************/
#include <setjmp.h>

#ifndef XNFS
#define XNFS	void
#endif

#ifndef XNFSSVC
#define XNFSSVC	void
#endif

#define	IPPROTO_TCP	6
#define IPPROTO_UDP	17

/*
 * char *xnfs_strerror(XNFS *cl, char *str)
 *
 * returns string "str : error", used to obtain reason for RPC call
 * failing.
 */
extern char *xnfs_strerror();

/*
 * char *xnfs_bind_strerror(char *str)
 *
 * returns string "str : error", used to obtain reason for RPC
 * bind failure.
 */
extern char *xnfs_bind_strerror();

/*
 * void xnfs_gethostname( char *buffer, u_int buf_len)
 *
 * Returns the textual network hostname for this node.
 */
extern int xnfs_gethostname();

/*
 * XNFS *xnfs_bind( char *server, u_int prog, u_int vers, char *prot)
 *
 * Creates and binds RPC path to (prog,vers) on server via prot.
 *
 * prot, one of "datagram_v" or "circuit_v".
 */
extern XNFS *xnfs_bind();

/*
 * void xnfs_unbind( XNFS *cl )
 *
 * Destroys client handle returned by xnfs_bind()
 */
extern void xnfs_unbind();

/*
 * XNFS *xnfs_mnt_bind( char *server, char *prot)
 *
 * Creates and binds RPC path to (MOUNTPROG, MOUNTVERS) on server via prot.
 *
 * prot, one of "datagram_v" or "circuit_v".
 *
 */
extern XNFS *xnfs_mnt_bind();

/*
 * XNFS *xnfs_nfs_bind( char *server, char *prot)
 *
 * Creates and binds RPC path to (NFS_PROGRAM, NFS_VERSION) on server via prot.
 *
 * prot, one of "datagram_v" or "circuit_v".
 */
extern XNFS *xnfs_nfs_bind();

/*
 * int xnfs_svc_listen( XNFSSVC *transp, u_int timeout)
 *
 * Listens to network for incoming RPC requests until 'timeout' seconds
 * has expired.
 *
 * A callback function may do a longjmp(xnfs_service_env,n) to force end of
 * service phase. Returns 'n' or -1 for a timeout.
 */
extern jmp_buf	xnfs_service_env;
extern int xnfs_svc_listen();

/*
 * void *xnfs_svc_create(	void (*dispatch)(),
 *				u_int prog, u_int vers, char *prot)
 *
 * prot, one or "datagram_v" or "circuit_v".
 */
extern void *xnfs_svc_create();
extern void *xnfs_server();
extern void *xnfs_pserver();
extern void *xnfs_nlm_svc();

/*
 * char *xnfs_mntename(int e)		
 * char *xnfs_nfsename(int e)		
 * char *xnfs_ftype(int e)		
 * char *xnfs_nsmename(int e)		
 * char *xnfs_lckename(int e);
 * char *xnfs_lckname(int e);
 *
 * e is an error number, which is converted into an error name.
 */

extern char *xnfs_mntename();
extern char *xnfs_nfsename();
extern char *xnfs_ftype();
extern char *xnfs_nsmename();
extern char *xnfs_lckename();
extern char *xnfs_lckname();
