/*
 *      SCCS: @(#)  INC/nfs.h 4.2 (05/07/98)  VSXgen release 1.4
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

/***********************************************************************

NAME:		nfs.h

PROJECT:	XNFS (X/OPEN NFS tests)

DESCRIPTION:	RPC Header file for NFS protocol - generated by rpcgen

		Requires prior inclusion of <rpcincs.h>

AUTHOR:		Matthew Newman, UniSoft Ltd.
DATE CREATED:
MODIFICATIONS:
		Geoff Clare, 4 Jan 1993
			Removed extraneous commas from last enum members.

		Geoff Clare, 14 Jan 1993
			New configurable header <rpcincs.h>

		Geoff Clare, 7 May 1998
			Added NFSERR_INVAL.

************************************************************************/

/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user or with the express written consent of
 * Sun Microsystems, Inc.
 *
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 *
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 *
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 *
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 *
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */
/*
 * Copyright (c) 1987, 1990 by Sun Microsystems, Inc.
 */

/* from @(#)nfs.h	4.2 98/05/07 */
/* from @(#)nfs_prot.x	1.3 91/03/11 TIRPC 1.0 */
#ifndef _xnfs_nfs_h
#define _xnfs_nfs_h
#define NFS_MAXDATA 8192
#define NFS_MAXPATHLEN 1024
#define NFS_MAXNAMLEN 255
#define NFS_COOKIESIZE 4
#define NFS_FHSIZE 32

enum nfs_stat {
	NFS_OK = 0,
	NFSERR_PERM = 1,
	NFSERR_NOENT = 2,
	NFSERR_IO = 5,
	NFSERR_NXIO = 6,
	NFSERR_ACCES = 13,
	NFSERR_EXIST = 17,
	NFSERR_NODEV = 19,
	NFSERR_NOTDIR = 20,
	NFSERR_ISDIR = 21,
	NFSERR_INVAL = 22,
	NFSERR_FBIG = 27,
	NFSERR_NOSPC = 28,
	NFSERR_ROFS = 30,
	NFSERR_NAMETOOLONG = 63,
	NFSERR_NOTEMPTY = 66,
	NFSERR_DQUOT = 69,
	NFSERR_STALE = 70
};
typedef enum nfs_stat nfs_stat;
bool_t xdr_nfs_stat();

enum ftype {
	NFNON = 0,
	NFREG = 1,
	NFDIR = 2,
	NFBLK = 3,
	NFCHR = 4,
	NFLNK = 5
};
typedef enum ftype ftype;
bool_t xdr_ftype();
typedef void *opaque;

typedef char nfscookie[NFS_COOKIESIZE];
bool_t xdr_nfscookie();

#ifndef _fhandle
#define _fhandle
typedef char fhandle[NFS_FHSIZE];
bool_t xdr_fhandle();
#endif

struct nfs_timeval {
	unsigned int seconds;
	unsigned int useconds;
};
typedef struct nfs_timeval nfs_timeval;
bool_t xdr_nfs_timeval();

struct fattr {
	ftype type;
	unsigned int mode;
	unsigned int nlink;
	unsigned int uid;
	unsigned int gid;
	unsigned int size;
	unsigned int blocksize;
	unsigned int rdev;
	unsigned int blocks;
	unsigned int fsid;
	unsigned int fileid;
	nfs_timeval atime;
	nfs_timeval mtime;
	nfs_timeval ctime;
};
typedef struct fattr fattr;
bool_t xdr_fattr();

struct diropok {
	fhandle file;
	fattr attributes;
};
typedef struct diropok diropok;
bool_t xdr_diropok();

struct sattr {
	unsigned int mode;
	unsigned int uid;
	unsigned int gid;
	unsigned int size;
	nfs_timeval atime;
	nfs_timeval mtime;
};
typedef struct sattr sattr;
bool_t xdr_sattr();

typedef char *filename;
bool_t xdr_filename();

typedef char *path;
bool_t xdr_path();

struct attrstat {
	nfs_stat status;
	union {
		fattr attributes;
	} attrstat_u;
};
typedef struct attrstat attrstat;
bool_t xdr_attrstat();

struct diropargs {
	fhandle dir;
	filename name;
};
typedef struct diropargs diropargs;
bool_t xdr_diropargs();

struct diropres {
	nfs_stat status;
	union {
		struct diropok diropres;
	} diropres_u;
};
typedef struct diropres diropres;
bool_t xdr_diropres();

struct sattrargs {
	fhandle file;
	sattr attributes;
};
typedef struct sattrargs sattrargs;
bool_t xdr_sattrargs();

struct readlinkres {
	nfs_stat status;
	union {
		path data;
	} readlinkres_u;
};
typedef struct readlinkres readlinkres;
bool_t xdr_readlinkres();

struct readargs {
	fhandle file;
	unsigned int offset;
	unsigned int count;
	unsigned int totalcount;
};
typedef struct readargs readargs;
bool_t xdr_readargs();

struct readresok {
	fattr attributes;
	struct {
		unsigned int data_len;
		char *data_val;
	} data;
};
typedef struct readresok readresok;
bool_t xdr_readresok();

struct readres {
	nfs_stat status;
	union {
		struct readresok ok;
	} readres_u;
};
typedef struct readres readres;
bool_t xdr_readres();

struct writeargs {
	fhandle file;
	unsigned int beginoffset;
	unsigned int offset;
	unsigned int totalcount;
	struct {
		unsigned int data_len;
		char *data_val;
	} data;
};
typedef struct writeargs writeargs;
bool_t xdr_writeargs();

struct createargs {
	diropargs where;
	sattr attributes;
};
typedef struct createargs createargs;
bool_t xdr_createargs();

struct renameargs {
	diropargs from;
	diropargs to;
};
typedef struct renameargs renameargs;
bool_t xdr_renameargs();

struct linkargs {
	fhandle from;
	diropargs to;
};
typedef struct linkargs linkargs;
bool_t xdr_linkargs();

struct symlinkargs {
	diropargs from;
	path to;
	sattr attributes;
};
typedef struct symlinkargs symlinkargs;
bool_t xdr_symlinkargs();

struct readdirargs {
	fhandle dir;
	nfscookie cookie;
	unsigned int count;
};
typedef struct readdirargs readdirargs;
bool_t xdr_readdirargs();

struct entry {
	unsigned int fileid;
	filename name;
	nfscookie cookie;
	struct entry *nextentry;
};
typedef struct entry entry;
bool_t xdr_entry();

struct readdirok {
	entry *entries;
	bool_t eof;
};
typedef struct readdirok readdirok;
bool_t xdr_readdirok();

struct readdirres {
	nfs_stat status;
	union {
		struct readdirok readdirok;
	} readdirres_u;
};
typedef struct readdirres readdirres;
bool_t xdr_readdirres();

struct statfsresok {
	unsigned int tsize;
	unsigned int bsize;
	unsigned int blocks;
	unsigned int bfree;
	unsigned int bavail;
};
typedef struct statfsresok statfsresok;
bool_t xdr_statfsresok();

struct statfsres {
	nfs_stat status;
	union {
		struct statfsresok ok;
	} statfsres_u;
};
typedef struct statfsres statfsres;
bool_t xdr_statfsres();

#define NFS_PROGRAM ((unsigned long)100003)
#define NFS_VERSION ((unsigned long)2)
#define NFSPROC_NULL ((unsigned long)0)
extern void *nfsproc_null_2();
#define NFSPROC_GETATTR ((unsigned long)1)
extern attrstat *nfsproc_getattr_2();
#define NFSPROC_SETATTR ((unsigned long)2)
extern attrstat *nfsproc_setattr_2();
#define NFSPROC_ROOT ((unsigned long)3)
extern void *nfsproc_root_2();
#define NFSPROC_LOOKUP ((unsigned long)4)
extern diropres *nfsproc_lookup_2();
#define NFSPROC_READLINK ((unsigned long)5)
extern readlinkres *nfsproc_readlink_2();
#define NFSPROC_READ ((unsigned long)6)
extern readres *nfsproc_read_2();
#define NFSPROC_WRITECACHE ((unsigned long)7)
extern void *nfsproc_writecache_2();
#define NFSPROC_WRITE ((unsigned long)8)
extern attrstat *nfsproc_write_2();
#define NFSPROC_CREATE ((unsigned long)9)
extern diropres *nfsproc_create_2();
#define NFSPROC_REMOVE ((unsigned long)10)
extern nfs_stat *nfsproc_remove_2();
#define NFSPROC_RENAME ((unsigned long)11)
extern nfs_stat *nfsproc_rename_2();
#define NFSPROC_LINK ((unsigned long)12)
extern nfs_stat *nfsproc_link_2();
#define NFSPROC_SYMLINK ((unsigned long)13)
extern nfs_stat *nfsproc_symlink_2();
#define NFSPROC_MKDIR ((unsigned long)14)
extern diropres *nfsproc_mkdir_2();
#define NFSPROC_RMDIR ((unsigned long)15)
extern nfs_stat *nfsproc_rmdir_2();
#define NFSPROC_READDIR ((unsigned long)16)
extern readdirres *nfsproc_readdir_2();
#define NFSPROC_STATFS ((unsigned long)17)
extern statfsres *nfsproc_statfs_2();
#endif /*!_xnfs_nfs_h*/
