/*
*	SCCS:	@(#)	INC/timeout.h		4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/
/***********************************************************************

NAME:		timeout.h - macros for timing out code sections

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Code to be timed out should be bracketed by

		SET_TIMEOUT(waittime)
	and
		CLEAR_ALARM

	The external variable alrm_flag will be greater than zero if a
	time-out occurred (see alrm(genlib)).  In addition CLEAR_ALARM
	preserves the value of errno.

	Any existing alarm setting is honoured.

REQUIRES PRIOR INCLUSION OF:
	<signal.h>
AUTHOR:
	Geoff Clare, UniSoft Ltd.
DATE CREATED:
	13 Dec 1988
MODIFICATIONS:
	Geoff Clare, 6 Sept 1996
	Replaced errno declaration with #include <errno.h>

************************************************************************/

#include <errno.h>

#ifndef alarm
extern	unsigned int	alarm();
#endif
extern	void		alrm();

extern	int	alrm_flag;

/*
 * These two macros define a code block - this means you can't
 * do things like:  if (...) { CLEAR_ALARM; return -1; }
 */

#define SET_TIMEOUT(waittime) \
	{ \
		unsigned to_unslept, to_alrm, to_time; \
		int to_err, to_flag; \
		struct sigaction to_alrm_sa, to_old_sa; \
		to_time = (waittime); \
		to_flag = 0; \
		alrm_flag = 0; \
		to_alrm_sa.sa_handler = alrm; \
		to_alrm_sa.sa_flags = 0; \
		(void) sigemptyset(&to_alrm_sa.sa_mask); \
		to_alrm = alarm(0); \
		if (sigaction(SIGALRM, &to_alrm_sa, &to_old_sa) == SYSERROR) \
		    vsxerror("SET_TIMEOUT: sigaction(SIGALRM, ...) failed"); \
		if (to_alrm != 0) { \
			if (to_time < to_alrm) { \
				++to_flag; \
				to_alrm -= to_time; \
			} else { \
				--to_flag; \
				to_time = to_alrm; \
				to_alrm = 0; \
				(void) sigaction(SIGALRM, &to_old_sa, NULLSA); \
			} \
		} \
		(void) alarm(to_time);


#define CLEAR_ALARM \
		to_err = errno; \
		to_unslept = alarm(0); \
		if (to_flag >= 0) \
			(void) sigaction(SIGALRM, &to_old_sa, NULLSA); \
		if (to_flag > 0 || (to_flag < 0 && to_unslept != 0)) \
			(void) alarm(to_alrm + to_unslept); \
		errno = to_err; \
	}
