/*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
* Developed for X/Open by Applied Testing and Technology, Inc.
*
* Project: VSRT
*
* File: SRC/INC/vsrt/vsrt.h
*
* Modifications:
* $Log: vsrt.h,v $
* Revision 1.1  2001/06/01 02:00:15  cyeoh
* Merges for lsb-os test suite
*
* Revision 11.3  2000/04/19 10:05:57  vsx
* set wait status in VSRT_KROF2 macro
*
* Revision 11.2  2000/04/06 11:14:05  vsx
* added vsrt_open_shm_np()
*
* Revision 11.1  2000/04/05 09:15:45  vsx
* report fork() failures
*
* Revision 11.0  1999/03/22 19:46:38  andy
* Branch point for Release 5.0.3
*
* Revision 10.0  1999/02/08 16:44:11  andy
* Branch point for Release 5.0.3b1
*
* Revision 9.0  1998/07/15 18:57:32  andy
* Branch point for Release 5.0.2
*
* Revision 8.0  1998/06/18 22:21:38  andy
* Branch point for Release 5.0.2b2
*
* Revision 7.0  1998/05/25 01:53:25  andy
* Branch point for Release 5.0.2b1
*
* Revision 6.0  1998/01/04 02:33:34  tbr
* Branch point for Release vsrt 5.0.1A
*
* Revision 5.1  1997/12/23 05:40:31  vsx0
* Removed VSRT_SET_EVENT and underlying function - no longer used.
*
* Revision 5.0  1997/11/26 21:44:15  andy
* Branch point for Release 5.0.1
*
* Revision 4.1  1997/11/20 22:31:25  vsx0
* Added additional parameter (npages) to vsrt_open_file.
*
* Revision 4.0  1997/10/22 15:42:50  dave
* Branch point for Release 5-0-0b1
*
* Revision 3.3  1997/10/09 23:58:51  vsx0
* Moved includes common to most tests into here
*
* Revision 3.2  1997/09/19 17:20:06  andy
* Upped VSRT_AIO_ITERATIONS to 40
*
* Revision 3.1  1997/09/19  04:43:53  andy
* Added VSRT_AIO_ITERATIONS
*
* Revision 3.0  1997/07/27  18:16:11  andy
* Branch point for Release 5.0.0
*
* Revision 2.0  1997/06/15  19:59:26  andy
* Branch point for Release 5.0.0b1
*
* Revision 1.11  1997/06/15  00:08:42  andy
* created WITH_ERROR versions of alarm macros
*
* Revision 1.10  1997/06/12  22:08:42  srini
* added prototypes for vsrtlib/semaphores.c functions
*
* Revision 1.9  1997/06/12  01:05:05  andy
* More aio
*
* Revision 1.7  1997/06/11  00:55:15  andy
* More aio mods
*
* Revision 1.6  1997/06/10  21:02:48  andy
* Added library functions for aio
*
* Revision 1.5  1997/06/04  19:08:15  srini
* Added prototypes for message passing library routines
*
* Revision 1.4  1997/05/29  17:21:25  andy
* Removed vsrt_mmapsig_fn
*
* Revision 1.3  1997/05/28  18:17:56  andy
* Added VSRT_RT_SIGNAL and use of debug returns
*
* Revision 1.1  1997/05/27  18:18:34  andy
* Initial revision
*
*/

#ifndef _VSRT_H_
#define _VSRT_H_

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <setjmp.h>

#include <std.h>
#include <dbug.h>
#include <exitcodes.h>
#include <pathres.h>
#include <sysdep.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>

/*from mkmmap.c*/
struct vsrt_exec_resource {
	int     fd;
	off_t     offset;
};

#define	VSRT_PROT_ALL	(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

/*prototypes for vsrtlib functions*/
/*ch_stime.c*/
void vsrt_ck_stime(int, int, time_t, struct stat *, time_t, struct stat *);

/*signal.c*/
void    vsrt_lsah(int);
int	vsrt_signal(int, void (*)(int));
int	vsrt_rt_signal(int, void (*)(int, siginfo_t *, void *));
int	vsrt_set_alarm(int);
int	vsrt_clear_alarm(void);
int	vsrt_expect_alarm(void);
int	vsrt_ignore_alarm(void);

/*waitfor.c*/
int	vsrt_wait_for(pid_t, int);

/*event.c*/
int	vsrt_alloc_sem(void);
void	vsrt_free_sem(void);
int	vsrt_wait_event(int, int);
int	vsrt_post_event(int);
int	vsrt_get_event(int);

/*mkmmap*/
int vsrt_mmap_setup(void);
int vsrt_mmap_setup(void);
int vsrt_open_file(char *, int, mode_t, int);
int vsrt_open_shm(char *, int, mode_t);
int vsrt_open_shm_np(char *, int, mode_t, int);
int vsrt_create_exec_file(char *, int, mode_t, struct vsrt_exec_resource *);
int vsrt_create_exec_shm(char *, int, mode_t, struct vsrt_exec_resource *);
void *vsrt_map3(int, int, int, int, int, int, int);


/*misc.c*/
void vsrt_check_support(char *, int *);
char **vsrt_nametoolng(void);
void vsrt_clntoolng(void);
void vsrt_sysfail(char *);

/*sharedmem.c*/
int vsrt_get_shm_name(char *, char *, size_t);

/*mqueue.c*/
int vsrt_get_mqueue_name(char *, char *, size_t);
char **vsrt_mq_nametoolng(void);
void vsrt_mq_clntoolng(void);

/*semaphores.c*/
int vsrt_get_sem_name(char *, char *, size_t);
char **vsrt_sem_nametoolng(void);
void vsrt_sem_clntoolng(void);

/*aio.c*/
int vsrt_init_ptys(char *, int *, int *);
int vsrt_init_ttys(char *, char *, int *, int *);
int vsrt_forkatest(int (*)(int, int), int, int);
void	vsrt_aio_test_file(int (*)(int, int));
void	vsrt_aio_test_tty(int (*)(int, int));
void	vsrt_aio_test_pty(int (*)(int, int));
void	vsrt_aio_test_pty_write(int (*)(int, int));
void	vsrt_aio_test_pipe(int (*)(int, int));
void	vsrt_aio_test_all(int (*)(int, int));
void	vsrt_aio_test_allbutfile(int (*)(int, int));
void	vsrt_aio_test_all_write(int (*)(int, int));
void	vsrt_aio_test_allbutfile_write(int (*)(int, int));

/*macros for forking child processes*/
#define MAX_TIME_TO_RUN	10
#define WAIT_FOR_TIME	2

#define VSRT_KROFV(pid) \
			_exit(EXIT_SUCCESS); \
		} \
		(void)vsrt_wait_for(pid, MAX_TIME_TO_RUN);

/* N.B. VSRT_FORK() is not thread-safe.  It must only be used in
   single-threaded processes.  In processes with multiple threads,
   use tet_fork() instead. */
#define VSRT_FORK(pid) \
		if ((pid = fork()) == 0) { \
			tet_setcontext();

#define VSRT_KROF(pid) \
			exit(0); \
		} else if (pid == -1) { \
			tet_infoline("ERROR: fork() failed"); \
			tet_result(TET_UNRESOLVED); \
		} else { \
			tet_setblock(); \
			(void)vsrt_wait_for(pid, MAX_TIME_TO_RUN); \
		}

#define VSRT_LKROF(pid,wait_time) \
			exit(0); \
		} else if (pid == -1) { \
			tet_infoline("ERROR: fork() failed"); \
			tet_result(TET_UNRESOLVED); \
		} else { \
			tet_setblock(); \
			(void)vsrt_wait_for(pid, wait_time); \
		}

#define VSRT_EFORK \
			exit(0); \
		} \
		tet_setblock();

#define VSRT_KROF2(pid,status)\
			exit(0); \
		} else if (pid == -1) { \
			tet_infoline("ERROR: fork() failed"); \
			tet_result(TET_UNRESOLVED); \
			status = -1; \
		} else { \
			tet_setblock(); \
			status = vsrt_wait_for(pid, MAX_TIME_TO_RUN); \
		}

/*macros for semaphore between processes*/

#define	VSRT_BEGIN_EVENTS		if (vsrt_alloc_sem() == -1)\
						return;
#define	VSRT_END_EVENTS			vsrt_free_sem()
#define	VSRT_WAIT_EVENT(x)		vsrt_wait_event((x), MAX_TIME_TO_RUN)
#define	VSRT_WAIT_EVENT2(x)		vsrt_wait_event((x), MAX_TIME_TO_RUN*2)
#define	VSRT_WAIT_EVENTN(x,t)		vsrt_wait_event((x), t)
#define	VSRT_POST_EVENT(x)		vsrt_post_event(x)
#define	VSRT_GET_EVENT(x)		vsrt_get_event(x)

#define	VSRT_WAIT_FOR_PID(pid)		vsrt_wait_for(pid, MAX_TIME_TO_RUN)

/*macros for signals and alarms*/

extern int	got_sigalrm;
extern unsigned	unused_time;
extern struct sigaction Sigaction;

#define VSRT_SIGNAL(signal, handler) {		\
	if(vsrt_signal(signal, handler) == -1) {	\
		DBUG_VOID_RETURN				\
	}					\
}

#define VSRT_RT_SIGNAL(signal, handler) {		\
	if(vsrt_rt_signal(signal, handler) == -1) {	\
		DBUG_VOID_RETURN				\
	}					\
}

#define VSRT_SET_ALARM(time)				\
	if(vsrt_set_alarm(time) == -1) {		\
		DBUG_VOID_RETURN				\
	}					\

#define VSRT_SET_ALARM_WITH_ERROR(time)		\
	if(vsrt_set_alarm(time) == -1) {		\
		DBUG_RETURN(-1);			\
	}					\

#define VSRT_CLEAR_ALARM_WITH_ERROR			\
	if(vsrt_clear_alarm()) {			\
		DBUG_RETURN(-1);			\
	}					\

#define VSRT_CLEAR_ALARM				\
	if(vsrt_clear_alarm()) {			\
		DBUG_VOID_RETURN				\
	}					\

#define VSRT_EXPECT_ALARM_WITH_ERROR				\
	if(vsrt_expect_alarm()) {		\
		DBUG_RETURN(-1);				\
	}					\

#define VSRT_EXPECT_ALARM				\
	if(vsrt_expect_alarm()) {		\
		DBUG_VOID_RETURN				\
	}					\

#define VSRT_IGNORE_ALARM				\
	if(vsrt_ignore_alarm()) {		\
		DBUG_VOID_RETURN				\
	}					\

#define VSRT_IGNORE_ALARM_WITH_ERROR				\
	if(vsrt_ignore_alarm()) {		\
		DBUG_RETURN(-1);				\
	}					\

/*  Constants used for time-related field checking  */
#define CHK_ATIME	1
#define CHK_MTIME	2
#define CHK_CTIME	3

/*timeout for vsrt_forkatest*/
#define VSRT_AIO_TO     120

/*number of loop iterations in waiting for a test's requests to complete*/
/*causes sleep(2)*VSRT_AIO_ITERATIONS maximum wait*/
/*must fit within VSRT_AIO_TO with room to spare*/
#define VSRT_AIO_ITERATIONS	40

#endif
