
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth.h	1.29
*
*************************************************************************/

#if	( TEST_XPG <= 4 )	/* POSIX 96 mode only */
#include <sys/types.h>
#endif

#include <pthread.h>
#include <time.h>

#include "std.h"

#define	VSTH_NMAX		999
#define	VSTH_RESULT(e,f)	((f)>0?TET_FAIL:((e)>0?TET_UNRESOLVED:TET_PASS))
#define	VSTH_NOSYNC		0
#define	VSTH_SYNC		1
#define	VSTH_DEFAULT_TIMEOUT	30
#define	VSTH_TIMEOUT		Vsth_timeout


/* must match the further information provided result code for VSXgen */
#define VSTH_TET_WARNING	101
#define VSTH_TET_FIP		102

/*	a sync point	*/
typedef	struct {
	long		p;	/* predicate		*/
	pthread_cond_t	c;	/* condition variable	*/
	pthread_mutex_t	m;	/* mutex		*/
} VSTH_SP, *VSTH_PSP;

typedef	struct {
	int		policy;
	int		priority;
	int		scope;
	int		inheritsched;
	int		detachstate;
	void		*(*func)(void *);
	void		*arg;
	VSTH_SP		sp;
	pthread_t	tid;
	int		status;
	void		*exitval;
} VSTH_TSET, *VSTH_PTSET;

extern int	Vsth_timeout;
extern int	Vsth_debug;

/* informational convenience functions */
void	vsth_error	( char *s, int rval, int *error );
void	vsth_surprise	( char *msg, int eval, int rval, int *fail );
void	vsth_badconfig	( char *var, int *errors );

/*	sync point methods	*/
int	vsth_sp_init	( VSTH_PSP psp );
int	vsth_sp_destroy	( VSTH_PSP psp );
int	vsth_sp_signal	( VSTH_PSP psp, long val );
int	vsth_sp_bcast	( VSTH_PSP psp, long val );
int	vsth_sp_wait	( VSTH_PSP psp, long val );
int	vsth_sp_set	( VSTH_PSP psp, long val );
int	vsth_sp_clear	( VSTH_PSP psp );

/* pthread_attr_t init convenience function */
int	vsth_attr_init(
			pthread_attr_t *attr,
			int detachstate,
			int policy,
			int priority,
			int inheritsched,
			int scope );

/* some pthread_*() wrappers for debugging purposes */
int	vsth_pthread_create(	pthread_t *ptid,
				pthread_attr_t *pa,
				void * (*start)(void *),
				void *arg );
int	vsth_pthread_join( pthread_t tid, void **val );
int	vsth_pthread_cancel( pthread_t tid );

/* some utility functions */
void	vsth_getprio( int policy, int *min, int *max, int *errors);
void	vsth_rr_get_interval( pid_t pid, struct timespec *interval, int *errors);
int	vsth_newthread(	int		policy,
			int		priority,
			int		scope,
			int		detachstate,
			void		*(*func)(void *),
			void		*arg,
			pthread_t	*ptid,
			int		*errors,
			int		*fails );
int	vsth_spa_init( VSTH_PSP psp, int nsp );
int	vsth_spa_destroy( VSTH_PSP psp, int nsp );
int	vsth_ts_create(	VSTH_PTSET pts,
			int nthreads,
			int syncflag,
			int *errors,
			int *fails );
void	vsth_ts_cancel( VSTH_PTSET pts, int nthreads, int detachflag );

int	vsth_ts_join( VSTH_PTSET pts, int nthreads, int *errors, int *fails );
int	vsth_ts_sync( VSTH_PTSET pts, int nthreads, int *errors, int *fails );

void	vsth_weave(	VSTH_PTSET	ptset,
			int		nthreads,
			void		*(*func)(void *),
			pthread_mutex_t	*pmx,
			pthread_cond_t	*pcv,
			int		policy,
			int		prio,
			int		prioflag,
			int		*errors,
			int		*fails );

int	vsth_spinwait(	pthread_mutex_t *pmx,
			int *flag,
			long val,
			int tout,
			int *errors );

void	vsth_startup(char *testname);
void	vsth_cleanup(char *testname);
int	vsth_bwait( pthread_cond_t *pcv, pthread_mutex_t *pmx, int *flag );
void	*vsth_null_thread( void *arg );
int	vsth_sched_yield();
void	vsth_setconcurrency();		/* in SRC/install/userintf/vsth	*/
int	vsth_autoinc( int *arg );	/* ditto, not currently used	*/

/* from vsxgen */
char	*execpath( char *progname );

