/*
*	SCCS:	@(#)	INC/vsx_signal.h		4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		vsx_signal.h - vsx signal definitions

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	These definitions are for use with sig_array[], sig_name[]
	and sig_type[].
	See vlib/sigvars.c and tsetlib/vsx_sigs.c for details.

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Thu Aug 14 10:15:33 PDT 1986
MODIFICATIONS:
		Geoff Clare, 8 March 1989
			Update for XPG3

		Geoff Clare, 15 Jan 1997
			Update for XSH5

************************************************************************/

#define	VNOSIG		-1		/* non-existant signal */

/* Indices for sig_array[] and sig_name[] */

#define	VNULLSIG	0		/* the null signal */
#define	VSIGABRT	1
#define	VSIGALRM	2
#define	VSIGFPE		3
#define	VSIGHUP		4
#define	VSIGILL		5
#define	VSIGINT		6
#define	VSIGKILL	7
#define	VSIGPIPE	8
#define	VSIGQUIT	9
#define	VSIGSEGV	10
#define	VSIGTERM	11
#define	VSIGUSR1	12
#define	VSIGUSR2	13
#define	VSIGCHLD	14

/* insert extra mode-dependent signals before Job Control signals */
/* and set VNOJSIG to number of non-job-control signals */

#if TEST_UNIX

/* UNIX95 and UNIX98 modes (and later) */
#    define VSIGBUS	15
#    define VSIGPOLL	16
#    define VSIGPROF	17
#    define VSIGSYS	18
#    define VSIGTRAP	19
#    define VSIGURG	20
#    define VSIGVTALRM	21
#    define VSIGXCPU	22
#    define VSIGXFSZ	23

#    define VNOJSIG	23

#else /* !TEST_UNIX */
#  if _POSIX_C_SOURCE >= 199506

/* POSIX96 mode (and later POSIX modes) */
#    define VSIGBUS	15

#    define VNOJSIG	15

#  else /* !POSIX96 */

/* POSIX90, FIPS, XPG3 and XPG4 modes */
#    define VNOJSIG	14

#  endif /* !POSIX96 */
#endif /* !TEST_UNIX */

#define	VSIGCONT	(VNOJSIG+1)
#define	VSIGSTOP	(VNOJSIG+2)
#define	VSIGTSTP	(VNOJSIG+3)
#define	VSIGTTIN	(VNOJSIG+4)
#define	VSIGTTOU	(VNOJSIG+5)
#define	VSIGINVAL	(VNOJSIG+6)	/* the invalid signal */

#define VNUMSIG		(VNOJSIG+6)	/* number of VSX defined signals */

/* Values for use in sig_type[] giving default actions for each signal */
#define TYPE_ABRT	1		/* abort on receiving signal */
#define TYPE_IGNR	2		/* ignore the signal */
#define TYPE_STOP	3		/* stop on receiving signal */
#define TYPE_CONT	4		/* continue on receiving signal */

extern	int	sig_array[VNUMSIG + 1];
extern	char	*sig_name[VNUMSIG + 1];
extern	int	sig_type[NSIG];
