/*
 *      SCCS: @(#)  INC/xnfs.h  4.1 (01/21/97)  VSXgen release 1.4
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

/***********************************************************************

NAME:		xnfs.h

PROJECT:	XNFS (X/OPEN NFS tests)

DESCRIPTION:	A header file for use with XNFS tests

AUTHOR:		Andrew Dingwall, UniSoft Ltd.

DATE CREATED:	27 August 1992

MODIFICATIONS:
		Geoff Clare, 8 Jan 1993
			Change xnfs_anon_uid from int to uid_t.

************************************************************************/

#if TEST_XNFS

#include <sys/types.h>

/*
**	common definitions for use with sync points
**
**	sync points may occur within a test, or within a subroutine called by
**	a test
**
**	at present, sync point numbers are +ve int values
**	
**	an XNFS sync point number consists of:
**		7 bits test number
**		4 bits per-test sync point number
**		4 bits per-subroutine sync point number
**
**	this imposes some limits which might be a bit tight:
**		max 127 tests per testset
**		max 16 syncs per test
**		max 16 syncs per subroutine
**
**	if you need more than these, you will have to change the type of
**	the tet_sync() syncptno parameter from int to long, so as to allow
**	this scheme to work on 16 bit machines
*/

/* generate a sync point number for immediate use or to pass to common
	subroutines - may be used in startup functions and test functions */
#define MK_SPNO(spno)	((tet_thistest << 8) | (((spno) & 017) << 4))
/* same as above but for use in cleanup functions */
#define MK_CSPNO(spno)	(((no_tests + 1) << 8) | (((spno) & 017) << 4))

/* sync point number to indicate to some library routines not to sync */
#define SP_NOSYNC	-1

/* sync point number used by init_pathres() */
#define	SP_PATHRES	7

/* sync timeout */
#define SYNC_TIMEOUT	30

/* child process timeout */
#define CHILD_TIMEOUT	60

/*
**	declarations relating to common setup functions
*/

/* this is the user ID that is mapped to XNFS_ANON_UID */
#define	XNFS_MAPPED_UID	0

/* xnfs_uids() sets xnfs_anon_uid from the parameter XNFS_ANON_UID */
extern int xnfs_uids();
extern uid_t xnfs_anon_uid;


/*
**	declarations related to xnfs_mount() and xnfs_unmount()
*/

extern int xnfs_mount(), xnfs_unmount();
extern int usr_mount(), usr_unmount();	/* the user-supplied functions */

/* mount flag values 			** MountedFileSystem attribute */
#define XNFS_MOUNT_RW		000	/* Mode=ReadWrite */
#define XNFS_MOUNT_RO		001	/* Mode=ReadOnly */
#define XNFS_MOUNT_GRPID	002	/* GrpID=True (default False) */
#define XNFS_MOUNT_SETUID	004	/* SetUID=True (default False) */
#define XNFS_MOUNT_SOFT		010	/* RetrySemantics=Soft (default Hard)*/
#define XNFS_MOUNT_INTR		020	/* Intr=True (default False) */

/* interfaces from mnt_rw(), mnt_ro() and unmnt() */
#define mnt_rw(spec, dir) xnfs_mount(spec, dir, XNFS_MOUNT_RW|XNFS_MOUNT_INTR)
#define mnt_ro(spec, dir) xnfs_mount(spec, dir, XNFS_MOUNT_RO|XNFS_MOUNT_INTR)
#define unmnt(spec, dir)  xnfs_unmount(spec, dir)


/*
**	declarations related to xnfs_export() and xnfs_unexport()
*/

extern int xnfs_export_c(), xnfs_export_s();
extern int xnfs_unexport_c(), xnfs_unexport_s();
extern int usr_export(), usr_unexport(); /* the user-supplied functions */

/* export flag values			** ExportedFileSystem attribute */
#define XNFS_EXPORT_RW		000	/* Mode=ReadWrite */
#define XNFS_EXPORT_RO		001	/* Mode=ReadOnly */
#define XNFS_EXPORT_ROOT	002	/* Root=XNFS_CLIENT */
#define XNFS_EXPORT_NOACCESS	004	/* access not granted to XNFS_CLIENT */

/* export sync points */
#define SP_EXPORT_START		0
#define SP_EXPORT_END		1

/* unexport sync points */
#define SP_UNEXPORT_START	0
#define SP_UNEXPORT_END		1


/*
**	declarations related to common ESTALE tests
*/

extern void t_estale_rd_c();
extern void t_estale_wr_c();
extern void t_estale_flush_c();
extern void t_estale_s();

/* estale sync points */
#define SP_ESTALE_RM_START	0
#define SP_ESTALE_RM_END	1


/*
**	declarations related to common EACCES tests
*/

extern void t_eacces_rd_c();
extern void t_eacces_wr_c();
extern void t_eacces_flush_c();
extern void t_eacces_s();

/* eacces sync points */
#define SP_EACCES_CR_START		0
#define SP_EACCES_CR_END		1
#define SP_EACCES_SETPERMS_START	2
#define SP_EACCES_SETPERMS_END		3


/*
**	declarations related to xnfs_rtime()
*/

extern int xnfs_rtime_c(), xnfs_rtime_s();

/* remote time sync points */
#define SP_RTIME_START		0
#define SP_RTIME_END		1


/*
**	declarations related to data transfer functions
*/

extern int xnfs_puts_s(), xnfs_putsv_s(), xnfs_putsv_c();
extern char **xnfs_getsv_c();
extern char **xnfs_getsv_s();
extern void xnfs_cln_getsv();

/* data transfer sync points */
#define SP_GETPUT_START		0
#define SP_GETPUT_END		1


/*
**	declarations of common server test parts
*/

extern void t_success_s();
extern void t_export_s();
extern void t_exportsu_s();
extern void t_nosearchdir_s();
extern void t_nosrchdir_d_s();
extern void t_nosrchdir_f_s();
extern void t_nowritedir_s();
extern void t_nowrtdir_d_s();
extern void t_nowrtdir_f_s();
extern void t_dir_s();
extern void t_do_access_s();
extern void t_nowritefile_s();
extern void t_nametoolng_s();
extern void t_pathres_s();
extern void t_exportnoacc_s();


/*
**	declarations of functions used within the xnfs library
*/

extern char **xnfs_getsv();
extern int  xnfs_putsv();
extern char *xnfs_sfname_c();
extern char *xnfs_sfname_s();
extern char *xnfs_syncfname();
extern void xnfs_doroot_c();
extern void xnfs_doroot_s();


#endif /* TEST_XNFS */

