/*
 *      SCCS:   @(#)common/drivers/Clocal.c	4.2 (02/11/97)  VSXgen release 1.4
 *
 *       NIXDORF COMPUTER AG, Paderborn, FRG
 *
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
 */
static	char sccsid[] = "@(#)common/drivers/driver.C VSXgen release 1.4";

/***********************************************************************

NAME:           Clocal.c - local routines/data for driver.C

PROJECT:        VSX (X/OPEN Validation Suite) C Languge Test

PARAMETER:      file:   pathname of sourcefile
		status: exit status of test

RETURNS:        pointer to excluded comments, wich may include
		'\n' characters.

ASSUMPTION:     the source file has the following layout:

		Heading line(s) enclosed in
			"\n@ " and "@" or star/slash
		special error line(s) enclosed in
			"\n@<exitcode> " and "@" or star/slash

		all constitute a comment

AUTHORS:        U. Krapp, D. Pfeifer, H. Pavel, all NCAG

MODIFICATIONS:
		-mhr: Mon Oct 20 14:32:25 BST 1986
			o re-wrote getstring as "state machine"
			o reformatted code

		Stuart Boutell, 13 Nov 1990
			Upgraded to become part of shared C/hdr driver

		Stuart Boutell, 29 Nov 1990
			Added C_driver flag;

		Stuart Boutell, 11 Mar 1991
			Added C_warn flag to indicate to the reporting
			mechanism that the test ought to return a WARNING
			as opposed to a fail. Changed return value from
			void to int to return the flag value.
		
		Geoff Clare, 12 Nov 1991
			Added ftmaclist[] array.

		Geoff Clare, 11 Feb 1997
			Renamed vsx_cflags_var, added vsx_libs_var.

************************************************************************/

#include <std.h>
#include <hdr.h>

#include <stdio.h>
#include <sys/types.h>
#include <dbug.h>
#include <ctype.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <ts_arc.h>
#include <report.h>
#include <stdlib.h>
#include <tet_api.h>

#include <sysdep.h>

#define BLKSIZE 1024

#define	END		(-1)
#define	START		0		/* states for getstring */
#define	FSLASH		1
#define	COMMENT		2
#define	FSTAR		3
#define	VALUE		4
#define	INSTRING	5
#define	BACKSLASH	6
#define	QUOTE		7

private	int	getstring();
private int	getline();

private int	clinno;
private int     state;
private int     end_ptr;
private int     buf_ptr;
private char	puffer[BLKSIZE];

public	int	C_driver = 1;	/* This is the C driver */

public	char	*vsx_cflags_var = "VSX_CFLAGS";
public	char	*vsx_libs_var = "VSX_LIBS";

public	char	*ftmaclist[] = {
#ifdef _XOPEN_SOURCE
	"-D_XOPEN_SOURCE",
#else
	"-D_POSIX_SOURCE",
#endif
	0
};


/*
 *	read the return value of a test and a string describing the test
 *	from the source.  The string can have the following format:
 *		/\*.*(@[0-9]*(\\.|[^@]|'([^\n]|\\.)*[\n'])*)*\*\/
 *	in english that means the string must be inside
 *	a C comment, start with a @ with an optional number that is the
 *	expected return value of the program, terminated by a *\/. @'s may
 *	be escaped by enclosing it (and any other string) with quote ('),
 *	a quote is also ended by a \n.
 */

public int
cfile_rpt()
{
	int		C_warn;
	int		test_contents;
	int		n, fd, ilink;
	char		str[BLKSIZE];
	char		*filname;

	DBUG_ENTER("exclude");
	buf_ptr = BLKSIZE + 1;
	end_ptr = 0;
	state = START;
	ilink = 0;

	filname = datafiles[ilink];

	test_contents = 0;
	C_warn = T_FAIL;	

	fd = open(filname, O_RDONLY, 0);
	if (fd != SYSERROR)
	{
		clinno = 1;
		in_rpt("Test Contents:");
		do
		{
			n = getstring(fd, str);
			if (n == 0 || n == cstatus)
			{
				in_rpt("%s", str);
				if (strncmp(str, " Warning", 8) == 0)
					C_warn = T_WARN;
				test_contents++;
			}
		} while (n != END);
		if (test_contents == 0)
			in_rpt("<none>");
		in_rpt("Test Code:");
		state = START;
		do {
			n = getline(fd, str);
			if (str[0] != NUL)
				in_rpt("%2d:  %s", clinno, str);
			clinno++;
		} while (n != END);
		(void) close(fd);
	}
	while((++ilink) <= test[test_index]->tc_multifiles )
	{
		clinno = 1;
		filname = datafiles[ilink];
		fd = open(filname, O_RDONLY, 0);
		if (fd != SYSERROR)
		{
			in_rpt("Test Code for file %s:", filname);
			state = START;
			do {
				n = getline(fd, str);
				if (str[0] != NUL)
					in_rpt("%2d:  %s", clinno, str);
				clinno++;
			} while (n != END);
			(void) close(fd);
		}
	}
	
	DBUG_RETURN(C_warn); /* Return whether a failure or warning should be
				given */
}

/*
 *	function to read a string from file descriptor fd, see comment for
 *	exclude.
 */
private int
getstring(fd, str)
	int     fd;
	char    *str;
{
	int		cnt;
	int		ret_val;
	int		leave;
	char		c;
	int		oldstate;

	DBUG_ENTER("getstring");
	str[0] = NUL;
	if (state == END)
		DBUG_RETURN(END);
	cnt = 0;
	leave = 0;
	ret_val = 0;
	while (!leave)
	{
		if (buf_ptr > end_ptr)
		{
			end_ptr = read(fd, puffer, (unsigned) BLKSIZE);
			if (end_ptr <= 0)
			{
				(void) lseek(fd, (off_t)0, SEEK_SET);
				leave = 1;
				state = END;
				break;
			}
			end_ptr--;
			buf_ptr = 0;
		}
		c = puffer[buf_ptr];
		if (c == '\n')
			clinno++;
		switch (state)
		{
		case START:	/* start state */
			if (c == '/')
				state = FSLASH;
			break;

		case FSLASH:	/* first slash (maybe start comment) */
			if (c == '*')
				state = COMMENT;
			else if (c != '/')
				state = START;
			break;

		case COMMENT:	/* inside comment */
			if (c == '*')
			{
				oldstate = COMMENT;
				state = FSTAR;
			}
			else if (c == '@')
				state = VALUE;
			break;

		case FSTAR:	/* first start (maybe end comment) */
			if (c == '/')
			{
				if (oldstate == INSTRING)
					str[cnt - 1] = NUL;
				leave = 1;
				state = END;
			}
			else
			{
				if (oldstate == INSTRING)
				{
					str[cnt] = c;
					cnt = cnt + 1;
				}
				if (c != '*')
					state = oldstate;
			}
			break;

		case VALUE:	/* look for a number */
			if (isdigit(c))
			{
				ret_val = (ret_val * 10) + c - '0';
				break;
			}
			state = INSTRING;
			cnt = 0;
			/* fall through */

		case INSTRING:	/* get the string */
			str[cnt] = c;
			if (c == '\\')
			{
				oldstate = INSTRING;
				state = BACKSLASH;
			}
			else if (c == '@')
			{
				str[cnt] = NUL;
				leave = 1;
				state = VALUE;
			}
			else if (c == '\'')
				state = QUOTE;
			else if (c == '*')
			{
				oldstate = INSTRING;
				state = FSTAR;
			}
			cnt = cnt + 1;
			break;

		case BACKSLASH:	/* got a backslash (\) */
			str[cnt] = c;
			cnt = cnt + 1;
			state = oldstate;
			break;

		case QUOTE:	/* got a single quote (') */
			if (c == '\\')
			{
				oldstate = QUOTE;
				state = BACKSLASH;
			}
			else if (c == '\'' || c == '\n')
				state = INSTRING;
			str[cnt] = c;
			cnt = cnt + 1;
			break;
		}
		buf_ptr = buf_ptr + 1;
	}

	/* strip off trailing white spaces */

	while (cnt > 0 && (isspace(str[cnt - 1]) || str[cnt-1] == NUL))
		cnt = cnt - 1;
	str[cnt] = NUL;
	if (state == END && cnt == 0 && ret_val == 0)
		ret_val = END;

	DBUG_PRINTF("getstring", "getstring returning (%s)%d", str, ret_val);
	DBUG_RETURN(ret_val);
}

/*
 *	function to read a line of code from file descripor fd.
 */
private int
getline(fd, str)
	int     fd;
	char    *str;
{
	int		cnt;
	int		leave;
	char		c;

	DBUG_ENTER("getline");
	str[0] = NUL;
	if (state == END)
		DBUG_RETURN(END);
	cnt = 0;
	leave = 0;
	while (!leave)
	{
		if (buf_ptr > end_ptr)
		{
			end_ptr = read(fd, puffer, (unsigned) BLKSIZE);
			if (end_ptr <= 0)
			{
				leave = 1;
				state = END;
				break;
			}
			end_ptr--;
			buf_ptr = 0;
		}
		c = puffer[buf_ptr++];
		if (c == '\n')
			leave = 1;
		else
			str[cnt++] = c;
	}

	/* strip off trailing white spaces */

	while (cnt > 0 && (isspace(str[cnt - 1]) || str[cnt-1] == NUL))
		cnt = cnt - 1;
	str[cnt] = NUL;

	DBUG_RETURN(state);
}

