/*
*      SCCS:   @(#)SRC/common/drivers/cleanup.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		cleanup.c - cleanup after test completion

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	cleanup()

SYNOPSIS:
	void
	cleanup()


DESCRIPTION:
	the main task of cleanup() is to remove any test detritus,
	id est, objectfile and resultsfile
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Paul Hackney
DATE CREATED:	Wed Jul 30 13:39:20 BST 1986

MODIFICATIONS:
		Stuart Boutell, 5 Nov 1990
			#ifdef'ed out vcleanup

		Stuart Boutell, 6 Nov 1990
			deleted vcleanup

************************************************************************/

#include	<std.h>

#include	<sys/types.h>
#include	<errno.h>
#include	<dbug.h>
#include	<hdr.h>
#include	<signal.h>
#include	<stdlib.h>
#include	<tet_api.h>

#include	<sysdep.h>

public void
cleanup()
	/*NOPARAMETERS*/
{
	int	tmp;

	DBUG_ENTER( "cleanup" );

	/* try aour best to removed files */

	if ( unlink( objectfile ) == SYSERROR )
	{
		if ( errno != ENOENT )
		{
			sys_error( "cleanup", objectfile, "unlink" );
		}
	}
	if ( unlink( resultsfile ) == SYSERROR )
	{
		if ( errno != ENOENT )
		{
			sys_error( "cleanup", resultsfile, "unlink" );
		}
	}
	tmp = 0;
	do
	{
		DBUG_PRINTF("cleanup","unlink %s", cc_ofiles[tmp]);
		if ( unlink( cc_ofiles[tmp] ) == SYSERROR )
		{
			if ( errno != ENOENT )
			{
				sys_error( "cleanup", cc_ofiles[tmp], "unlink" );
			}
		}
		DBUG_PRINTF("cleanup","unlink %s", datafiles[tmp]);
		if ( unlink( datafiles[tmp] ) == SYSERROR )
		{
			if ( errno != ENOENT )
			{
				sys_error( "cleanup", datafiles[tmp], "unlink");
			}
		}
		(void)	free((void *)cc_ofiles[tmp]);
		cc_ofiles[tmp] = NULL;
		(void)	free((void *)datafiles[tmp]);
		datafiles[tmp] = NULL;
	} while(datafiles[++tmp] != NULL);
	DBUG_VOID_RETURN;
}

