/*
*      SCCS:   %ZSRC/common/drivers(compile.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		compile.c - compile "datafile"

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	cc()

SYNOPSIS:
	int	
	cc( flags )
		char	*flags;

DESCRIPTION:
	cc() invokes VSX_CC with std{err,out} redirected to
	"resultsfile" with VSX_CFLAGS and local flag defined by inc;
	cc() will return only SUCCEED or FAIL

METHODS:
ASSUMES:
	extern char *datafile points to valid file name
	extern char *resultsfile points to valid file name

CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Paul Hackney
DATE CREATED:	Wed Jul 30 13:39:20 BST 1986

MODIFICATIONS:
		Geoff Clare, 11 Sep 1989
			Increase NARGS and add bounds check for argv[]

		Stuart Boutell, 2 Nov 1990
			Command line option mechanism rewritten.

		Stuart Boutell, 11 Mar 1991
			Tidied up cc()

		Geoff Clare, 18 June 1992
			Added -DHDRTEST to compiler flags.

		Geoff Clare, 23 June 1994
			Added #include <stdio.h> (for sprintf decl)

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<ts_arc.h>
#include	<errno.h>
#include	<dbug.h>
#include	<hdr.h>
#include	<fcntl.h>
#include	<unistd.h>

#include	<sysdep.h>

#define	OFLAG	(O_RDWR|O_APPEND)

extern	char	**environ;
private	char	command[MAX_STR_LEN];	/* effective CC invocation line */
public	char	*argv[NARGS];		/* argument vector for CC */

private	int	split();
private	void	buildcom();
private	int	ctype();
public	void	ccexec();
extern	char	charof();
extern	char	*errname();

public	int 
cc( inc )
char	*inc;
{
	int	tmp=0;
	DBUG_ENTER( "compile" );

	cstatus = 0;
	while(datafiles[tmp] != NULL)
	{
		buildcom( datafiles[tmp] , inc);

		DBUG_PRINTF( "compile", "command = %s", command );

		if (split() != SUCCEED) /* split it into args */
			DBUG_RETURN( FAIL );

		cstatus	|= tet_fork(ccexec,NULLFN,(160*SPEEDFACTOR),~0);
		tmp ++;
	}
	DBUG_RETURN( cstatus == 0 ?
		SUCCEED : FAIL );
}
	
private	void
buildcom( datafile , i)
char	*datafile;
char	*i;
{
	if(!is_string(i))
	{
		(void) sprintf( command, "%s %s %s -DHDRTEST -c %s",
			vsx_cc, vsx_cflags, ftmacs, datafile );
	}
	else
	{
		(void) sprintf( command, "%s %s %s -DHDRTEST -I%s%s -c %s",
			vsx_cc, vsx_cflags, ftmacs, vsx_c_isep, i, datafile);
	}
}

private	int
split()
{
	/*
	 * split command line in command[] into arguments and store
	 * pointers to arguments in *argv[]
	 */
	char	*s	= command;	/* scan command line pointers */
	char	*t	= command;	/* scan command line pointers */
	char	**u	= argv;		/* argument vector pointer */
	int	state	= 0;		/* starting state */
	static	int	next[4][3] =	/* next state table */
	{
		{ -1, 0, 1 },		/* 0 */
		{ 2, 3, 1 },		/* 1 */
		{ -1, -1, -1 },		/* 2 */
		{ -1, 0, 1 }		/* 3 */
	};

	do {
		switch( ( state = next[ state ][ ctype( *s ) ] ) )
		{
		    case -1: *u = NULL; break;

		    case 0: t = ++s; break;

		    case 1: s++; break;

		    case 2: *u++ = t; break;

		    case 3: *s = NUL; *u++ = t; t = ++s; break;
		}
		if (u - argv == NARGS)
		{
			vsxerror( "NARGS exceeded in split() - %s", __FILE__ );
			return( FAIL );
		}
	} while ( state != -1 );

	return( SUCCEED );
}


private	int
ctype( c )
	char	c;
{
	if ( c == NUL )
		return 0;
	else
		return ( c == ' ' ) ? 1 : 2;
}

public void
ccexec()
{
	private	int	fildes;
	DBUG_ENTER("ccexec");
	if ( ( fildes = open( resultsfile, OFLAG ) ) == SYSERROR )
	{
		DBUG_PRINTF("ccexec","open(resultsfile) failed errno %d(%s)",
			errno, errname(errno));
		DBUG_EXIT(-1);
	}
	(void)	close(STDOUT_FILENO);
	(void)	close(STDERR_FILENO);
	if (fcntl(fildes, F_DUPFD, STDOUT_FILENO) != STDOUT_FILENO ||
		fcntl(fildes, F_DUPFD, STDERR_FILENO) != STDERR_FILENO)
	{
		DBUG_PRINTF("ccexec", "fcntl(%d) failed errno %d(%s)",
			fildes, errno, errname(errno));
		DBUG_EXIT(-1);
	}
	(void)	execvp(argv[0],argv);
	/* an error has occured if the next line is reached!! */
	vsxerror("execvp() failed - error %d(%s)", errno, errname(errno));
	DBUG_PRINTF("ccexec","execvp() failed - error %d(%s)", errno, errname(errno));
	DBUG_EXIT(-1);
}

