/*
*      SCCS:   @(#)common/drivers/driver.c	4.2 (02/11/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};

/***********************************************************************

NAME:		driver.c	- main driver code for C and hdr testsets

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		testfunc()
		prolog()
SYNOPSIS:
		void testfunc();
		void prolog();

DESCRIPTION:
	Each testset starts execution in this module ( API will call
	the "prolog" subroutine as an entry into the program,and testfunc
	for each test case). 
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
	VSX3 SRC/common/driver.hdr/*
	VSX3 SRC/common/driver.C/*
	AUTHORS:	Geoff Clare, Paul Hackney, U. Krapp,
			H.Pavel, D. Pfeifer, Mike Rendell
SOURCE:

AUTHOR:		Stuart Boutell, UniSoft Ltd
DATE CREATED:	Thu Oct 18 13:32:46 BST 1990
MODIFICATIONS:
		Stuart Boutell, 25 Oct 1990
			Resilience added to testfunc, prolog

		Stuart Boutel, 5 Nov 1990
			Removed child = -1 declaration
		
		Geoff Clare, 14 Feb 1991
			Remove redundant (and buggy) extractenv() routine
			and calls to it.

		Stuart Boutell, 2 Apr 1991
			Corrected vsx_ld default to vsx_cc (from "/bin/cc")
		
		Geoff Clare, 2 Oct 1991
			Removed use of toupper() and tolower().
		
		Geoff Clare, 12 Nov 1991
			Repeat each test for all combinations of
			feature test macros in ftmaclist[].
		
		Geoff Clare, 1 Apr 1992
			Added code to recognise VSX_DBUG_FILE.
		
		Geoff Clare, 20 June 1994
			Fixed some incorrect arguments to sys_error().

		Geoff Clare, 11 Feb 1997
			Renamed vsx_cflags_var, added vsx_libs_var.

************************************************************************/

/* *** driver.c #include's *** */
#include	<std.h>

#include	<dbug.h>
#include	<hdr.h>
#include	<setjmp.h>
#include	<string.h>
#include	<ts_arc.h>
#include	<tet_api.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<exitcodes.h>

#include	<sysdep.h>


#ifndef getenv
extern	char	*getenv();
#endif

extern	void	in_rptfail();
extern	void	in_rptrfi();
extern	void	in_rptpass();
extern	void	in_rptwarn();
extern	void	in_rptuntes();
extern	void	in_rptunsup();
extern	void	in_rptnotinuse();
extern	void	in_rptuninit();
extern	void	in_rptbadcode();
extern	char	*get_pname();
extern	char	*arc_errname();
extern	void	htabandon();
extern	void	set_pname();
extern	int	setup();

extern	char	*vsx_cflags_var;
extern	char	*vsx_libs_var;
extern	char	*ftmaclist[];

private	void	testfunc();
private	void	prolog();

private	int	setupok = -1;

#define	TLI	testfunc, 1
#define	TLI10	TLI, TLI, TLI, TLI, TLI, TLI, TLI, TLI, TLI, TLI
#define	TLINULL	NULL, 1

public	struct tet_testlist tet_testlist[] = {
	TLI10 , TLI10 ,
	TLI10 , TLI10 ,
	TLI10 , TLI10 ,
	TLI10 , TLI10 ,
	TLI10 , TLI10 ,
	TLINULL };

public	void	(*tet_startup)() = prolog;
public	void	(*tet_cleanup)() = NULL;

private char	*testvars[] = {
#include "testvars.h"
NULL };

extern	char	**environ;
extern	char	*vg_file;
extern	char	*vg_dbug;


int	getint(a)
char	a;
{
	char t[2];
	t[0] = a;
	t[1] = '\0';
	return(atoi(t));
}

private void
prolog()
{
	int	i, len;
	char	*s;
	char	*p;
	char	*dbfile;
	char	arcname[_POSIX_PATH_MAX];
	char	**filenames;
	char	**tmp;
	int	pathok=0;

	DBUG_ENTER( "prolog" );

	if ( is_string( tet_pname ) )
	{
		set_pname( vg_file = tet_pname );
		set_ident( tet_pname );
	}

	if ( is_string( s = tet_getvar( "VSX_DBUG_FLAGS" ) ) )
	{
		dbfile = NULL;

		/* If no "o,file" in flags, add one for VSX_DBUG_FILE */
		if (s[0] != 'o')
		{
			for (p = s; p[1] != '\0'; p++)
				if (p[0] == ':' && p[1] == 'o')
					break;
			if (p[1] != 'o')
				dbfile = tet_getvar("VSX_DBUG_FILE");
		}
		if (dbfile == NULL)
			dbfile = "";

		len = strlen(s) + strlen(dbfile) + 3; /* 3 for ":o," */
		vg_dbug = (char *) malloc((size_t)(len + 3)); /* 3 for "-D\0" */
		if (vg_dbug != NULL)
		{
			(void) strcpy(vg_dbug, "-D");
			(void) strcat(vg_dbug, s);
			if (*dbfile != '\0')
			{
				(void) strcat(vg_dbug, ":o,");
				(void) strcat(vg_dbug, dbfile);
			}
			DBUG_PUSH(&vg_dbug[2]);
		}
		else
		{
			DBUG_PUSH(s);
			DBUG_PRINTF("prolog", "malloc() for vg_dbug failed");
		}
		debug = TRUE;
	}

	s = tet_getvar("VSX_PATH" );
	if ( !is_string( s ) )
		s = ":/bin:/usr/bin";
	if ((p = (char *)malloc((size_t)(strlen(s) + 6))) == NULL)
	{
		sys_error( "prolog", "", "malloc" );
		htabandon();
	}
	else
		PATH_TRACE;
	(void) strcat(strcpy(p, "PATH="), s);
	if ( vputenv( &environ, p ) == SYSERROR )
	{
		vsxerror( "vputenv(&environ, \"%s\") failed", p );
		sys_error( "prolog", "", "vputenv failed" );
		htabandon();
	}
	else
		PATH_TRACE;

	if ( !is_string( ( vsx_cc = tet_getvar( "VSX_CC" ) ) ) )
		vsx_cc = VSX_CC;

	if ( !is_string( ( vsx_cflags = tet_getvar( vsx_cflags_var ) ) ) )
		vsx_cflags = VSX_CFLAGS;

	if ( !is_string( ( vsx_libs = tet_getvar( vsx_libs_var ) ) ) )
		vsx_libs = VSX_LIBS;

	if ( !is_string( ( vsx_ld = tet_getvar( "VSX_LD" ) ) ) )
		vsx_ld = vsx_cc;

	if ( !is_string( ( vsx_c_link = tet_getvar( "VSX_C_LINK" ) ) ) )
		vsx_c_link = VSX_C_LINK;

	if ( !is_string( ( vsx_c_nolink = tet_getvar( "VSX_C_NOLINK" ) ) ) )
		vsx_c_nolink = VSX_C_NOLINK;

	if ( !is_string( ( vsx_c_out = tet_getvar( "VSX_C_OUT" ) ) ) )
		vsx_c_out = VSX_C_OUT;

	if ( !is_string( ( vsx_c_isep = tet_getvar( "VSX_C_ISEP" ) ) ) )
		vsx_c_isep = VSX_C_ISEP;

	if ( !is_string( ( vsx_c_suffix = tet_getvar( "VSX_C_SUFFIX" ) ) ) )
		vsx_c_suffix = VSX_C_SUFFIX;

	for(tmp=testvars;*tmp!=NULL;tmp++)
	{
		s=tet_getvar(*tmp);
		if(s!=NULL)
		{
			if ((p = (char *)malloc((size_t)(strlen(*tmp)+strlen(s) + 2))) == NULL)
			{
				sys_error( "prolog", "", "malloc" );
				htabandon();
			}
			(void) strcat(strcat(strcpy(p, *tmp ), "="),s);
			DBUG_PRINTF("prolog","%s",p);
			if ( vputenv( &environ, p ) == SYSERROR )
			{
				vsxerror( "vputenv(&environ, \"%s\") failed", p );
			}
		}
	}
	

	DBUG_PRINTF( "prolog", "pname = %s, ident = %s", get_pname(), get_ident() );
	DBUG_PRINTF( "prolog", "vsx_cc = %s, vsx_cflags = %s, vsx_libs = %s",
		vsx_cc, vsx_cflags, vsx_libs );
	DBUG_PRINTF( "prolog", "vsx_ld = %s", vsx_ld);
	DBUG_PRINTF( "prolog", "vg_file %s, vg_dbug %s", vg_file, vg_dbug);


	(void)	strcat( data_dir, "." );

	/* extract files from testset archive */

	(void) strcat( strcpy( arcname, "L."), get_ident() );

	ntests = arc_extr(arcname, data_dir, &filenames);

	if (ntests == SYSERROR)
	{
		vsxerror("error extracting files from testset archive");
		vsxerror("%s",  arc_errname(arcname));
		htabandon();
	}
	else
		PATH_TRACE;

	/* prolog the test[] array */
	if (ntests > 0)
	{
		test = (struct tcntrl **)
			malloc( (size_t) (ntests * sizeof(struct tcntrl *)) );
		if ( test == NULL )
		{
			sys_error( "prolog", "", "malloc" );
			htabandon();
		}
		else
			PATH_TRACE;
		DBUG_PRINTF("prolog", "filenames=%#x, ntests=%d", filenames, ntests);
		for (i = 0 ; i < ntests ; i++)
		{
			DBUG_PRINTF("prolog", "i %d, (%d)(%s)", i, ntests, filenames[i]);

			test[i] = (struct tcntrl *)
				malloc( (size_t) sizeof( struct tcntrl ) );
			if ( test[i] == NULL )
			{
				sys_error( "prolog", "", "malloc" );
				htabandon();
			}
			else
				PATH_TRACE;

			test[i]->tc_flag = 0;
			test[i]->tc_xx = (char *) NULL;
			test[i]->tc_execute = (filenames[i][TS_COMP] == 'e' ||
				filenames[i][TS_COMP] == 'E');
			test[i]->tc_link = !(filenames[i][TS_COMP] == 'c' ||
				filenames[i][TS_COMP] == 'C');
			test[i]->tc_warning = (filenames[i][TS_NLINK] == 'w' ||
				filenames[i][TS_NLINK] == 'W');
/* Extensions: */
			test[i]->tc_include = isupper(filenames[i][TS_COMP]);
			test[i]->tc_negtest = (filenames[i][TS_CLASS] == 'd');
			test[i]->tc_multifiles = (isdigit(filenames[i][TS_NLINK])? (getint(filenames[i][TS_NLINK])) : -1);

			filenames[i][TS_NUMBER+2]='\0';
			test[i]->tc_number = atoi(&(filenames[i][TS_NUMBER]));

			DBUG_PRINTF("prolog",
"test struct: tnum:%d link:%d, exec:%d, warn:%d, incl:%d, negt:%d, mult:%d",
test[i]->tc_number, test[i]->tc_link, test[i]->tc_execute, test[i]->tc_warning,
test[i]->tc_include, test[i]->tc_negtest, test[i]->tc_multifiles);

			if(test[i]->tc_negtest && test[i]->tc_execute)
				tet_delete( test[i]->tc_number, "Execute and Negative test options [cd##e_.c files] are incompatible");
			free((void *)filenames[i]);
		}
		free((void *) filenames);
	}

	tet_testlist[ntests].testfunc = NULL;

	/* switch to data directory */

	if ( chdir( data_dir ) == SYSERROR )
	{
		sys_error( "prolog", data_dir, "chdir" );
		htabandon();
	}
	else
		PATH_TRACE;

	if(system("cp ${TET_EXECUTE:?}/INC/* .")!=0)
	{
		for(i=0;i<ntests;i++)
				tet_delete( test[i]->tc_number, "failed to copy xxx.h files from $TET_EXECUTE/INC to test directory");
	
	}
	else
		PATH_TRACE;

	if(pathok == (ntests+6))
		setupok = 0;
	else
		setupok = -1;

	DBUG_PRINTF("prolog","setupok = %d",setupok);

	DBUG_VOID_RETURN;
}

private void
testfunc()
{
	char	**ftmacp;
	int	t, ret, fail = 0;
	int	pathok = setupok;
	DBUG_ENTER("testfunc");

	
	test_index = -1;
	for(t=0;t<ntests;t++)
		if(test[t]->tc_number == tet_thistest)
		{
			PATH_TRACE;
			test_index = t;
			break;
		}

	if(test_index==-1)
	{
		in_rptuninit();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( setup() == SUCCEED )
	{
		for (ftmacp = ftmaclist; *ftmacp != NULL; ftmacp++)
		{
			ftmacs = *ftmacp;
			globok = 0;
			ret = perform();

			switch (ret)
			{
				/* these have no output */
				case T_PASS:
				case T_UNSUP:
				case T_UNTES:
				case T_NOTINUSE:
					break;
				
				default:
					if (*ftmacs != '\0' && !C_driver)
						in_rpt("Feature test macros: %s",
							ftmacs);
					break;
			}

			switch (ret)
			{
				case T_FAIL:
					fail++;
					if(cfile_rpt()==T_FAIL)
						in_rptfail();
					else
						in_rptwarn();
					break;

				case T_RFI:
					fail++;
					(void) cfile_rpt();
					in_rptrfi();
					break;

				case T_WARN:
					fail++;
					(void) cfile_rpt();
					in_rptwarn();
					break;

				case T_UNSUP:
					fail++;
					in_rptunsup();
					break;

				case T_UNTES:
					fail++;
					in_rptuntes();
					break;

				case T_NOTINUSE:
					fail++;
					in_rptnotinuse();
					cleanup();
					DBUG_VOID_RETURN;
					break;

				case T_PASS:
					if (ftmacp == ftmaclist)
						PATH_FUNC_TRACE;
					break;

				default:
					fail++;
					in_rptbadcode();
					break;
			}
		}
		cleanup();
	}
	if (fail == 0)
		in_rptpass(3,pathok);
	DBUG_VOID_RETURN;
}
