/*
*      SCCS:   @(#)SRC/common/drivers/error.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		error.c - error handling routines

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	sys_error()
	htabandon()

SYNOPSIS:
	void
	sys_error( func, subj, name )
		char	*func;
		char	*subj;
		char	*name;

	void
	htabandon()

DESCRIPTION:
	sys_error called as a result of a UNIX system call error;
	formats a standard VSX error report containing function
	name (func), the subject of the failure (subj) and 
	system call name (name) or subroutine call name.
	htabandon() displays a standard message and causes the testset
	to exit with a code TE_ABANDON.


METHODS:
ASSUMES:
	extern int errno contains the error code

CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Paul Hackney
DATE CREATED:	Wed Jul 30 13:39:20 BST 1986

MODIFICATIONS:

************************************************************************/

#include	<std.h>
#include	<dbug.h>
#include	<hdr.h>
#include	<stdlib.h>
#include	<sysdep.h>

extern	char	*errname();

public void
sys_error( func, subj, name )
	char	*func;			/* function name */
	char	*subj;			/* subject of error */
	char	*name;			/* syscall name */

{

	DBUG_ENTER( "sys_error" );

	er_rpt( "System Call error:" );
	in_rpt( "%s: \"%s\" [%s(%d)], %s failed",
		func, subj, errname( errno), errno, name );

	DBUG_VOID_RETURN;

}


public 	void
htabandon()
	/*NOPARAMETERS*/
{

	DBUG_ENTER( "htabandon" );

	/* kludge to avoid htabandon->cleanup->syserror->htabandon loop
	 when thereis no data_dir */

	if ( running )
		cleanup();
	er_rpt( "Abandoning Testset:" );
	exit( TE_ABANDON );

	DBUG_VOID_RETURN;
}
