/*
*      SCCS:   @(#)common/drivers/hdr.h	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		hdr.h - driver include file

PROJECT:	VSX (X/OPEN Validation Suite)
DESCRIPTION:	defines for driver.C, driver.hdr, driver.hdr2
NOTE:		Some of the defines below are duplicated in HEADER.h;
		any change to this file must be reflected in HEADER.h as well.
AUTHOR:		Paul Hackney
DATE CREATED:	Wed Jul 30 12:45:53 BST 1986
MODIFICATIONS:
		Geoff Clare, 2 Nov 1989
			Provide for warning on compilation failure
		
		Geoff Clare, 13 Feb 1990
			Remove unused VSX_SH definition
		
		Geoff Clare, 27 Apr 1990
			Add NOTINUSE exit code

		Stuart Boutell, 5 Nov 1990
			Redundant code eliminated

		Geoff Clare, 12 Nov 1991
			Added ftmacs.

************************************************************************/

#include	<errno.h>
/* os.hdr testset exit codes ( 10-99 ) */

#define	TE_ABANDON	(10)		/* testset abandoned */

/* test status return codes - must agree with HEADER.h */

#define	T_PASS		(0)
#define	T_FAIL		(1)
#define	T_RFI		(2)
#define	T_WARN		(3)
#define	T_UNSUP		(4)
#define	T_UNTES		(5)
#define	T_NOTINUSE	(6)

#define	FATAL	(1)			/* force exit failure */

/* definitions that may go into a VSX wide file */

#define	IDENT_SIZE	(32)		/* testset identifier length */
#define	LOCAL_PATH_LEN	(32)		/* for test specific files */
#define	MAX_STR_LEN	(256)		/* maximum string length */
#define	PATH_LEN	(64)		/* general path length */

/* os.hdr definitions */

#define	VSX_CC		"/bin/cc"	/* default C compiler */
#define	VSX_CFLAGS	""		/* default C compiler flags */
#define	VSX_LIBS	"-lm"		/* default C compiler libraries */
#define	VSX_C_LINK	""
#define	VSX_C_NOLINK	"-c"
#define	VSX_C_LINK	""
#define	VSX_C_OUT	"-o"
#define	VSX_C_ISEP	""
#define	VSX_C_SUFFIX	".o"

/* compiler flags */
#define	INCLUDE		"./incl"

#define	NOLOAD	(0)			/* don't call loader */
#define	LOAD	(1)			/* call loader */

#define	NARGS	20			/* Max number of command line args */

/*
 * tcntrl - test control structure
 *
 * each testset has an array of tcntrl structure coincident with the
 * number of tests
 */

struct tcntrl {
	int	tc_flag;	/* test flag - bitmask */
#define DELETED		(1)
#define NEXTONE		(2)
	char	*tc_xx;		/* deletion reason */
	int	tc_execute;	/* TRUE if test is to be executed */
	int	tc_warning;	/* TRUE if test produces WARNING only */
/* Extensions */
	int	tc_link;	/* TRUE if test is to be compiled & linked only */
	int	tc_include;	/* TRUE if test requires compile line -I */
	int	tc_multifiles;	/* -1 for single file; 0-9 for multifiles */
	int	tc_negtest;	/* TRUE for a negative test class */
	int	tc_number;	/* The test number */
};

/* is...() macros to aid clarity! */

#define	is_set(x)	( (x) == TRUE )
#define	is_string(x)	( (x) != NULL && *(x) != NUL )
#define is_dltd(x)	( test[(x)-1]->tc_flag & DELETED )

/* extern variable declarations - declared and set in the respective local
   source files for each driver. */
extern	int	C_driver;

/* extern variable declarations - declared in globals.c */

extern	int	cstatus;	/* child process status */
extern	int	debug;		/* true for "-D" flag */
extern	int	ntests;		/* number of tests in testset */
extern	int	test_index; 		/* Index into test struct for current test */
#ifdef DELETING
extern	int	testno;		/* current test number */
#endif
extern	int	ntests;		/* number of tests in testset */
extern	int	running;	/* testset is running */

extern	char	*vsx_cc;	/* C compiler path */
extern	char	*vsx_cflags;	/* C compiler flags */
extern	char	*ftmacs;	/* Feature test macros */
extern	char	*vsx_libs;	/* C compiler libraries */
extern	char	*vsx_ld;	/* Linker path */
extern	char	*vsx_c_out;	/* linker output specifiere option */
extern	char	*vsx_c_nolink;	/* No link compiler option */
extern	char	*vsx_c_link;	/* Force link compiler option */
extern	char	*vsx_c_isep;	/* -I separator */
extern	char	*vsx_c_suffix;	/* C object suffix */

extern	char	*incl;		/* Include directory; */

extern	char	*(cc_ofiles[]);	/* cc object file names */
extern	char	*(datafiles[]);	/* current DATAFILE names */
extern	char	objectfile[];	/* current OBJECTFILE name */
extern	char	resultsfile[];	/* current RESULTSFILE name */
extern	char	data_dir[];	/* data directory (d.xxx) */

/* extern variable declarations - declared in testsetname.c */

extern	struct	tcntrl	**test;	/* test control structure */

/* extern library (hdrlib) procedure declarations */

extern	char	*get_ident();	/* returns testset ident (eg argv[1] ) */
extern	void	cleanup();	/* cleanup after test */
extern	int	no_output();	/* no_output in resutlsfile? */
extern	int	perform();	/* perform test */
extern	void	set_ident();	/* sets testset ident */
extern	int	setup();	/* test entry preparation */
extern	void	sys_error();	/* syscall error */
extern	int	cfile_rpt();	/* Include C source file in report as nec.
				   The ret. val. indicates warn(1) or fail(0) */

