/*
*      SCCS:   @(#)SRC/common/drivers/link.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		link.c - link "datafile"

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	ld()

SYNOPSIS:
	void
	ld( flag )
		char	*flag;

DESCRIPTION:
	ld() invokes VSX_CC with std{err,out} redirected to
	"resultsfile" with VSX_CFLAGS and local flag "flag";
	ld() will return only SUCCEED or FAIL

METHODS:
ASSUMES:
	extern char *datafile points to valid file name
	extern char *resultsfile points to valid file name

CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
		compile.c
SOURCE:

AUTHOR:		Stuart Boutell, UniSoft Ltd
DATE CREATED:	Fri Oct 26 12:59:29 BST 1990

MODIFIED:
		Geoff Clare, 23 June 1994
			Added #include <stdio.h> (for sprintf decl)

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<ts_arc.h>
#include	<errno.h>
#include	<dbug.h>
#include	<hdr.h>
#include	<fcntl.h>
#include	<unistd.h>

#include	<sysdep.h>

private	char	command[MAX_STR_LEN];	/* effective CC invocation line */
extern	char	*argv[];		/* argument vector for CC */

extern	char	*errname();
private	int	split();
private	void	buildcom();
private	int	ctype();
extern	void	ccexec();
extern	char	charof();

public	int 
ld()
	/*NOPARAMETERS*/
{
	int	tmp1=0;
	int	tmp2=0;
	char	*ofiles;
	DBUG_ENTER( "ld" );

	while(cc_ofiles[tmp1] != NULL)
	{
		tmp1++;
	}

	if( (ofiles = (char *)malloc( (size_t)(10*tmp1))) == NULL)
	{
		sys_error("ld","ofiles","malloc failed");
		DBUG_RETURN(FAIL);
	}
	*ofiles = '\0';

	while (tmp2 < tmp1)
	{
		(void)strcat(ofiles, cc_ofiles[tmp2++]);
		(void)strcat(ofiles, " ");
	}

	buildcom( ofiles );

	DBUG_PRINTF( "ld", "command = %s", command );

	if (split() != SUCCEED) /* split it into args */
		DBUG_RETURN( FAIL );

	cstatus	= tet_fork(ccexec,NULLFN,(160*SPEEDFACTOR),~0);
	DBUG_RETURN( cstatus == 0 ?
		SUCCEED : FAIL );
}
	
private	void
buildcom( cc_ofile )
char	*cc_ofile;
{
	(void) sprintf( command, "%s %s %s %s %s %s",
		vsx_ld, vsx_c_link, vsx_c_out, objectfile, cc_ofile, vsx_libs );
}

private	int
split()
{
	/*
	 * split command line in command[] into arguments and store
	 * pointers to arguments in *argv[]
	 */
	char	*s	= command;	/* scan command line pointers */
	char	*t	= command;	/* scan command line pointers */
	char	**u	= argv;		/* argument vector pointer */
	int	state	= 0;		/* starting state */
	static	int	next[4][3] =	/* next state table */
	{
		{ -1, 0, 1 },		/* 0 */
		{ 2, 3, 1 },		/* 1 */
		{ -1, -1, -1 },		/* 2 */
		{ -1, 0, 1 }		/* 3 */
	};

	do {
		switch( ( state = next[ state ][ ctype( *s ) ] ) )
		{
		    case -1: *u = NULL; break;

		    case 0: t = ++s; break;

		    case 1: s++; break;

		    case 2: *u++ = t; break;

		    case 3: *s = NUL; *u++ = t; t = ++s; break;
		}
		if (u - argv == NARGS)
		{
			vsxerror( "NARGS exceeded in split() - %s", __FILE__ );
			return( FAIL );
		}
	} while ( state != -1 );

	return( SUCCEED );
}


private	int
ctype( c )
	char	c;
{
	if ( c == NUL )
		return 0;
	else
		return ( c == ' ' ) ? 1 : 2;
}

