/*
*      SCCS:   @(#)common/drivers/perform.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		perform.c - perform the current test

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	perform()

SYNOPSIS:
	int
	perform()

DESCRIPTION:
	perform calls the test strategy routine in structure tcntrl
	(tc_strategy); 
METHODS:
ASSUMES:
	extern int test_index
		contains the current test structure index
	extern char *(datafiles[]);
	extern char *objectfile;
	extern char *resultsfile;
		point to valid filenames

CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Paul Hackney
DATE CREATED:	Wed Jul 30 13:39:20 BST 1986

MODIFICATIONS:
		Geoff Clare, 2 Nov 1989
			Provide for warning on compilation failure
		
		Geoff Clare, 27 Apr 1990
			Add NOTINUSE exit code

		Stuart Boutell, 24 Oct 1990
			Removed sdata,adata

		Stuart Boutell, 13 Nov 1990
			Added driver.C extentions for reporting.
			Removed result side effect, - return result.
			Added C_driver conditional section

		Stuart Boutell, 25 Feb 1991
			Changed to allow an execute/warn
			test that fails to compile to return a warn
			result code.
		
		Geoff Clare, 27 Apr 1990
			Change return from perform() on failed
			link-before-execute from RFI to FAIL

************************************************************************/

#include	<std.h>
#include	<dbug.h>
#include	<hdr.h>
#include	<stdlib.h>
#include	<tet_api.h>
#include	<sysdep.h>

#define	C_L_E		1
#define	C_L_ONLY	2
#define	C_ONLY		3

extern	int	ld();
extern	int	cc();

public int
perform()
	/*NOPARAMETERS*/
{
	int	cres,lres,eres;
	int	mode,warn,negt;
	char	*include;
	int	pathok = 0;
	DBUG_ENTER( "perform" );

	include = test[test_index]->tc_include?incl:(char *)NULL;
	warn = (test[test_index]->tc_warning);
	negt = (test[test_index]->tc_negtest);
	mode =  test[test_index]->tc_execute ?
		 C_L_E : (test[test_index]->tc_link ? C_L_ONLY : C_ONLY);


	cres = cc( include );	
	DBUG_PRINTF("perform","COMPILE PHASE COMPLETE");
	if (mode == C_ONLY) 
	{
		PATH_TRACE;
		if(negt)  /* Expecting a negative test result */
		{
			PATH_TRACE;
			DBUG_PRINTF("perform","Checking for expected FAIL");
			if (cres == SUCCEED)
			{
				PATH_TRACE;
				DBUG_PRINTF("perform","Compile suceeded");
				in_rpt("Compilation exited with zero return value when expected to fail");
				if (warn)
				{
					DBUG_PRINTF("perform","result =T_WARN");
					PATH_FUNC_RPT(3);
					DBUG_RETURN(T_WARN);
				}
				else
				{
					DBUG_PRINTF("perform","result=T_FAIL");
					PATH_FUNC_RPT(3);
					DBUG_RETURN(T_FAIL);
				}
			}
			else
			{
				DBUG_PRINTF("perform","Compile failed");
				DBUG_PRINTF("perform","result=T_PASS");
				PATH_FUNC_RPT(2);
				DBUG_RETURN(T_PASS);
			}
		}
		else
		{	
			PATH_TRACE;
			DBUG_PRINTF("perform","Checking for expected PASS");
			if (cres == SUCCEED)
			{
				PATH_TRACE;
				DBUG_PRINTF("perform","Compile suceeded");
				if(!no_output())
				{
					DBUG_PRINTF("perform","Compiler rfi'd");
					DBUG_PRINTF("perform","result=T_RFI");
					in_rpt("Compilation exited with zero return value, but unexpected output was generated");
					PATH_FUNC_RPT(3);
					DBUG_RETURN(T_RFI);
				}	
				else
				{
					DBUG_PRINTF("perform","result=T_PASS");
					PATH_FUNC_RPT(3);
					DBUG_RETURN(T_PASS);
				}
			}
			else
			{
				PATH_TRACE;
				DBUG_PRINTF("perform","Compile failed");
				in_rpt("Compilation exited with non-zero value when expected to succeed");
				if (warn)
				{
					DBUG_PRINTF("perform","result=T_WARN");
					PATH_FUNC_RPT(3);
					DBUG_RETURN(T_WARN);
				}
				else
				{
					DBUG_PRINTF("perform","result=T_FAIL");
					PATH_FUNC_RPT(3);
					DBUG_RETURN(T_FAIL);
				}
			}
		}
	}
	else
	{	/* Was going on to link */
		PATH_TRACE;
		if (cres != SUCCEED)
		{
			PATH_TRACE;
			DBUG_PRINTF("perform","Compile failed");
			/* Always an error, but could be a warning */
			in_rpt("Compilation exited with non-zero value when expected to succeed");
			if (warn)
			{
				DBUG_PRINTF("perform","result=T_WARN");
				PATH_FUNC_RPT(2);
				DBUG_RETURN(T_WARN);
			}
			else
			{
				DBUG_PRINTF("perform","result=T_FAIL");
				PATH_FUNC_RPT(2);
				DBUG_RETURN(T_FAIL);
			}
		}
	}


	lres = ld();

	DBUG_PRINTF("perform","LINK PHASE COMPLETE");


	if (mode == C_L_ONLY) 
	{
		PATH_TRACE;
		if(negt)  /* Expecting a negative test result */
		{
			PATH_TRACE;
			DBUG_PRINTF("perform","Checking for desired FAIL");
			if (lres == SUCCEED)
			{
				PATH_TRACE;
				DBUG_PRINTF("perform","Link suceeded");
				in_rpt("Linker exited with zero return value when expected to fail");
				if (warn)
				{
					DBUG_PRINTF("perform","result=T_WARN");
					PATH_FUNC_RPT(4);
					DBUG_RETURN(T_WARN);
				}
				else
				{
					DBUG_PRINTF("perform","result=T_FAIL");
					PATH_FUNC_RPT(4);
					DBUG_RETURN(T_FAIL);
				}
			}
			else
			{
				DBUG_PRINTF("perform","Link failed");
				DBUG_PRINTF("perform","result=T_PASS");
				PATH_FUNC_RPT(3);
				DBUG_RETURN(T_PASS);
			}
		}
		else
		{	
			PATH_TRACE;
			DBUG_PRINTF("perform","Checking for desired PASS");
			if (cres == SUCCEED)
			{
				PATH_TRACE;
				DBUG_PRINTF("perform","Link succeeded");
				if(!no_output())
				{
					DBUG_PRINTF("perform","Linker rfi'd");
					DBUG_PRINTF("perform","result=T_RFI");
					in_rpt("Linker exited with zero return value, but unexpected output was generated");
					PATH_FUNC_RPT(4);
					DBUG_RETURN(T_RFI);
				}	
				else
				{
					DBUG_PRINTF("perform","result=T_PASS");
					PATH_FUNC_RPT(4);
					DBUG_RETURN(T_PASS);
				}
			}
			else
			{
				PATH_TRACE;
				DBUG_PRINTF("perform","Link failed");
				in_rpt("Linker exited with non-zero value when expected to succeed");
				if (warn)
				{
					DBUG_PRINTF("perform","result=T_WARN");
					PATH_FUNC_RPT(4);
					DBUG_RETURN(T_WARN);
				}
				else
				{
					DBUG_PRINTF("perform","result=T_FAIL");
					PATH_FUNC_RPT(4);
					DBUG_RETURN(T_FAIL);
				}
			}
		}
	}
	else
	{	/* Was going on to execute */
		PATH_TRACE;
		if (lres != SUCCEED)
		{
			DBUG_PRINTF("perform","Link failed");
			DBUG_PRINTF("perform","result=T_FAIL");
			in_rpt("Linker exited with non-zero value when expected to succeed");
			PATH_FUNC_RPT(2);
			DBUG_RETURN(T_FAIL);
		}
	}
	
	eres = execute();
	DBUG_PRINTF("perform","EXECUTE PHASE COMPLETE");
	if (eres == SUCCEED)
	{
		PATH_TRACE;
		DBUG_PRINTF("perform","execute succeeded");
		if( no_output() )
		{
			DBUG_PRINTF("perform","result=T_PASS");
			PATH_FUNC_RPT(3);
			DBUG_RETURN(T_PASS);
		}
		else
		{
			DBUG_PRINTF("perform","test rfi'd (!no_output)");
			DBUG_PRINTF("perform","result = T_RFI");
			in_rpt("execution of test succeeded, but unexpected output was generated");
			PATH_FUNC_RPT(3);
			DBUG_RETURN(T_RFI);
		}
	}
	else /* execute failed */
	{
		PATH_TRACE;
		DBUG_PRINTF( "perform", "execute failed" );
		if(C_driver)
		{
			DBUG_PRINTF( "perform", "C language driver test result %d (FAIL)",cstatus);
			DBUG_PRINTF( "perform", "result=T_FAIL");
			PATH_FUNC_RPT(3);
			DBUG_RETURN(T_FAIL);
		}
		else
		{
			DBUG_PRINTF( "perform", "Header driver test result %d (FAIL)",cstatus);
			switch (cstatus)
			{
			case T_RFI:
				DBUG_PRINTF( "perform", "test rfi (return RFI)" );
				DBUG_PRINTF( "perform","result=T_RFI");
				PATH_FUNC_RPT(3);
				DBUG_RETURN(T_RFI);
				/* NOTREACHED */
				break;
			case T_WARN:
				DBUG_PRINTF( "perform", "test warning (return WARN)" );
				DBUG_PRINTF( "perform", "result=T_WARN");
				PATH_FUNC_RPT(3);
				DBUG_RETURN(T_WARN);
				/* NOTREACHED */
				break;
			case T_UNSUP:
				DBUG_PRINTF( "perform", "test unsupported (return UNSUP)" );
				DBUG_PRINTF( "perform", "result=T_UNSUP");
				PATH_FUNC_RPT(3);
				DBUG_RETURN(T_UNSUP);
				/* NOTREACHED */
				break;
			case T_UNTES:
				DBUG_PRINTF( "perform", "test untested (return UNTES)" );
				DBUG_PRINTF( "perform", "result=T_UNTES");
				PATH_FUNC_RPT(3);
				DBUG_RETURN(T_UNTES);
				/* NOTREACHED */
				break;
			case T_NOTINUSE:
				DBUG_PRINTF( "perform", "test not in use (return NOTINUSE)" );
				DBUG_PRINTF( "perform", "result=T_NOTINUSE");
				PATH_FUNC_RPT(3);
				DBUG_RETURN(T_NOTINUSE);
				/* NOTREACHED */
				break;
			case T_FAIL:
				DBUG_PRINTF( "perform", "test failed (return FAIL)" );
				DBUG_PRINTF("perform","result=T_FAIL");
				PATH_FUNC_RPT(3);
				DBUG_RETURN(T_FAIL);
				/* NOTREACHED */
				break;
			default:
				DBUG_PRINTF( "perform", "test returned an unknown exit code %d", cstatus);
				DBUG_PRINTF("perform","result=-1");
				PATH_FUNC_RPT(3);
				DBUG_RETURN(-1);
				/* NOTREACHED */
				break;
			}
		}
	}
	/*NOTREACHED*/
	DBUG_PRINTF("perform","DRIVER FAILED! REACHED UNREACHABLE CODE");
	PATH_FUNC_RPT(99999); /* Ought never execute this ! */
	DBUG_RETURN(T_PASS);  /* Return pass to ensure pathtrace error occurs */
}
