/*
*      SCCS:   @(#)common/drivers/report.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		report.c - os.hdr specific report routines

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	in_file()
	in_rptpass()
	in_rptfail()
	in_rptrfi()
	in_rptnotinuse()
	in_rptuninit()
	in_rptbadcode()

SYNOPSIS:
	void in_file()
	void in_rptpass()
	void in_rptfail()
	void in_rptrfi()
	void in_rptnotinuse()
	void in_rptuninit()
	void in_rptbadcode()

DESCRIPTION:
	in_file() displays resultsfile using in_rpt
	in_rpt{pass,fail,rfi}() display the result
	in VSX standard form
METHODS:
ASSUMES:
	extern char *resultsfile points to valid file name
	extern int  test_index is index into test structure
AUTHOR:		Paul Hackney
DATE CREATED:	Wed Jul 30 13:39:20 BST 1986

MODIFICATIONS:
		Geoff Clare, 27 Apr 1990
			Add in_rptnotinuse().

		Stuart Boutell, 24 Oct 1990
			Add in_rptuninit().

		Stuart Boutell, 25 Oct 1990
			Added resilience to in_rptpass().

		Stuart Boutell, 12 Mar 1991
			Added in_rptbadcode() to report bad exit codes
			from tests.

		Geoff Clare, 12 Nov 1991
			Truncate results file after reporting.

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#include	<string.h>
#include	<dbug.h>
#include	<hdr.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<tet_api.h>
#include	<sysdep.h>

#ifndef	RPT_DEST
#define	RPT_DEST	stdout
#endif	/* RPT_DEST */


private	void
in_file()
	/*NOPARAMETERS*/
{

	FILE	*file;			/* resultsfile */
	int	fd;
	char	buf[MAX_STR_LEN];	/* yes it's a buffer */

	DBUG_ENTER( "in_file" );

	in_rpt("Compiler or run-time messages or results:");

	if ( no_output() )
	{
		in_rpt("<NIL> (no results or messages produced)");
	}
	else
	{
		if ( ( file = fopen( resultsfile, "r" ) ) == (FILE *)NULL )
		{
			sys_error( "in_file", resultsfile, "fopen" );
		}
		else
		{
			while ( fgets( buf, MAX_STR_LEN, file ) != (char *)NULL )
			{
				if ( strlen( buf ) < MAX_STR_LEN - 1 )
					/* remove trailing '\n' */
					buf[strlen( buf ) - 1] = NUL;
				in_rpt( "%s", buf );
			}
			(void)fclose( file );

			/* Truncate file for re-use */
			fd = creat(resultsfile, (mode_t)0);
			if (fd >= 0)
				(void) close(fd);
		}
	}

	DBUG_VOID_RETURN;

}

public	void
in_rptpass(exp,act)
int	exp;
int	act;
{
	int	pathok;
	DBUG_ENTER( "in_rptpass" );

	pathok = act;
	PATH_XS_RPT(exp);

	DBUG_VOID_RETURN;

}

public	void
in_rptfail()
	/*NOPARAMETERS*/
{

	DBUG_ENTER( "in_rptfail" );

	xx_rpt( FAILURE );
	in_file();

	DBUG_VOID_RETURN;

}

public	void
in_rptwarn()
	/*NOPARAMETERS*/
{

	DBUG_ENTER( "in_rptwarn" );

	xx_rpt( WARNING );
	in_file();

	DBUG_VOID_RETURN;

}

public	void
in_rptuntes()
	/*NOPARAMETERS*/
{

	DBUG_ENTER( "in_rptuntes" );

	xx_rpt( UNTESTED );

	DBUG_VOID_RETURN;

}

public	void
in_rptunsup()
	/*NOPARAMETERS*/
{

	DBUG_ENTER( "in_rptunsup" );

	xx_rpt( UNSUPPORTED );

	DBUG_VOID_RETURN;

}

public	void
in_rptrfi()
	/*NOPARAMETERS*/
{

	DBUG_ENTER( "in_rptrfi" );

	xx_rpt( RFI );
	in_rpt( "C compiler return status: %d", cstatus );
	in_file();

	DBUG_VOID_RETURN;

}


public	void
in_rptbadcode()
	/*NOPARAMETERS*/
{

	DBUG_ENTER( "in_rptbadcode" );

	xx_rpt( DELETION );
	in_rpt( "Test returned unexpected exit code %d",
			cstatus );
	DBUG_VOID_RETURN;

}

public	void
in_rptnotinuse()
	/*NOPARAMETERS*/
{

	DBUG_ENTER( "in_rptnotinuse" );

	xx_rpt( NOTINUSE );

	DBUG_VOID_RETURN;

}

public	void
in_rptuninit()
	/*NOPARAMETERS*/
{

	DBUG_ENTER( "in_rptuninit" );

	in_rpt("No files associated with test number");
	xx_rpt( DELETION );

	DBUG_VOID_RETURN;
}
