/*
*      SCCS:   @(#)common/drivers/setup.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		setup.c - perform test preparation chores

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	setup()

SYNOPSIS:
	int
	setup()

DESCRIPTION:
	the main task of setup() is to prepare the test environment
	for the test; if DATAFILE cannot be accessed the test is deleted.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Paul Hackney
DATE CREATED:	Wed Jul 30 13:39:20 BST 1986

MODIFICATIONS:
		Geoff Clare, 2 Nov 1989
			Provide for warning on compilation failure

		Stuart Boutell, 25 Nov 1990
			Changed way files referenced.

************************************************************************/

#include	<std.h>
#include	<sys/types.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<unistd.h>
#include	<hdr.h>
#include	<sys/stat.h>
#include	<sysdep.h>

#define	MODE	(S_IWUSR|S_IRUSR)	/* -rw------- */
#ifndef	RPT_DEST
#define	RPT_DEST	stdout
#endif	/* RPT_DEST */

extern	char	*errname();
extern	void	htabandon();
extern	char	charof();

public	int 
setup()
	/*NOPARAMETERS*/
{
	int	count;
	char	*tset;			/* temporary */
	int	fildes;			/* temporary */

	DBUG_ENTER( "setup" );

	/* generate file names */

	tset = get_ident();

	DBUG_PRINTF( "setup", "test  = %d", test[test_index]->tc_number );
	DBUG_PRINTF( "setup", "test_index = %d", test_index);

	count = 0;
	do
	{
	
	 	if((datafiles[count] = (char *)malloc((size_t)9))==NULL)
		{
			sys_error("setup", "datafiles", "malloc failed");
			DBUG_RETURN( FAIL );
		}
	 	if((cc_ofiles[count] = (char *)malloc((size_t)9))==NULL)
		{
			sys_error("setup", "cc_ofiles", "malloc failed");
			DBUG_RETURN( FAIL );
		}
	
	
		(void)sprintf( datafiles[count], "c%c%02.2d%c%c.c", 
			test[test_index]->tc_negtest ? 'd' : 'c' ,
			test[test_index]->tc_number,
			test[test_index]->tc_execute ? 
				(test[test_index]->tc_include? 'E':'e') :
				(test[test_index]->tc_link ? 
				(test[test_index]->tc_include? 'L':'l') :
				(test[test_index]->tc_include? 'C':'c') ) ,
			test[test_index]->tc_warning ? 'w' : 
				(test[test_index]->tc_multifiles == -1 ? 's' : charof(count)));
	
		/* cc_ofile - same as DATAFILE but ".o" extension */
	
		cc_ofiles[count][0] = '\0';
		(void)strncat( cc_ofiles[count], datafiles[count], strlen(datafiles[count]) -2 );
		(void)strcat( cc_ofiles[count], vsx_c_suffix);
	
		DBUG_PRINTF( "setup", "datafiles[%d] = %s", count, datafiles[count] );
		DBUG_PRINTF( "setup", "cc_ofiles[%d] = %s", count, cc_ofiles[count] );
	
		/* check for existence of DATAFILE  */
		if ( access( datafiles[count], R_OK ) == SYSERROR )
		{
			xx_rpt( CANCELLED );
			if ( errno == ENOENT )
			{
				sys_error( "setup", datafiles[count], "access" );
				DBUG_RETURN( FAIL );
			}
			else
			{
				sys_error( "setup", datafiles[count], "access" );
				htabandon();
			}
		}
	
	} while((++count) <= test[test_index]->tc_multifiles);

	datafiles[count] = (char *) NULL;
	cc_ofiles[count] = (char *) NULL;

	/* OBJECTFILE - xxx-o.0 */

	(void)sprintf( objectfile, "%s-o.%-d", tset, test[test_index]->tc_number);

	/* RESULTSFILE - xxx-r.0 */

	(void)sprintf( resultsfile, "%s-r.%-d", tset, test[test_index]->tc_number);

	DBUG_PRINTF( "setup", "objectfile = %s", objectfile );
	DBUG_PRINTF( "setup", "resultsfile = %s", resultsfile );

	if ( ( fildes = creat( resultsfile, MODE ) ) == SYSERROR )
	{
		xx_rpt( CANCELLED );
		sys_error( "setup", resultsfile, "creat" );
		htabandon();
	}
	else
	{
		(void)close( fildes );
	}

	DBUG_RETURN( SUCCEED );

}
