/*
*      SCCS:   @(#)common/genlib(asc_to_psl.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		asc_to_psl - Convert to Pseudo Language characters from
				C-locale-printable form.

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	asc_to_psl()
	nomet_to_psl()
SYNOPSIS:
	char *asc_to_psl(str, langnum)
		char	*str;
		int	langnum;

	char *nomet_to_psl(str, langnum)
		char	*str;
		int	langnum;
DESCRIPTION:
	asc_to_psl converts from a printable representation of pseudo
	language to the correct pseudo language characters for language
	langnum.  The conversion is done IN PLACE, so the routine should
	not be called more than once for the same string.

	Nomet_to_psl is a historical left-over, and is now equivalent
	to asc_to_psl().

SOURCE:		
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	Tue Nov  3 11:27:48 1987
MODIFICATIONS:
		Geoff Clare, 23 Aug 1988: Add nomet_to_psl()

		Geoff Clare, 29 May 1992
			Add '#' and '$' to INFO1LANG and INFO2LANG

		Geoff Clare, 22 March 1995
			Use codes from <pslcodes.h>

************************************************************************/

#include	<std.h>
#include	<nlstest.h>
#include	<stdio.h>	
#include	<dbug.h>
#include	<pslcodes.h>
#include	<sysdep.h>

char *
asc_to_psl(str, langnum)
char *str;
int  langnum;
{
	register unsigned char *s, *t;

	DBUG_LENTER("asc_to_psl", 2);
	DBUG_PRINTF("args", "asc_to_psl(%s, %d)", str, langnum);

	if (langnum != VALINFO1LANG && langnum != VALINFO2LANG
		&& langnum != VALINFO3LANG && langnum != VALINFO4LANG)
	{
		DBUG_RETURN(str);
	}

	for (s = t = (unsigned char *)str; *s != '\0'; s++, t++)
	{
		if (langnum == VALINFO1LANG || langnum == VALINFO2LANG)
		{
			switch (*s) {
			case '&' : *t = PSL12_eszet; continue;
			case '~' : *t = PSL12_cedilla; continue;
			}
			if (s[1] == '"')
			{
				switch (*s) {
				case 'A' : *t = PSL12_A_umlaut; s++; continue;
				case 'O' : *t = PSL12_O_umlaut; s++; continue;
				case 'U' : *t = PSL12_U_umlaut; s++; continue;
				case 'a' : *t = PSL12_a_umlaut; s++; continue;
				case 'o' : *t = PSL12_o_umlaut; s++; continue;
				case 'u' : *t = PSL12_u_umlaut; s++; continue;
				}
			}
			*t = *s;
			continue;
		}
		else
		{
			switch(*s) {
			case '&' :
				*t = PSL34_eszet; continue;
			case 'A' :
				switch(s[1]) {
				case '`':  *t = PSL34_A_grave; s++; continue;
				case '\'': *t = PSL34_A_acute; s++; continue;
				case '^':  *t = PSL34_A_circumflex; s++; continue;
				case '~':  *t = PSL34_A_tilde; s++; continue;
				case '"':  *t = PSL34_A_umlaut; s++; continue;
				case '0':  *t = PSL34_A_ring; s++; continue;
				default:   *t = *s; continue;
				}
			case 'C' :
				switch(s[1]) {
				case '~':  *t = PSL34_C_cedilla; s++; continue;
				default:   *t = *s; continue;
				}
			case 'E' :
				switch(s[1]) {
				case '`':  *t = PSL34_E_grave; s++; continue;
				case '\'': *t = PSL34_E_acute; s++; continue;
				case '^':  *t = PSL34_E_circumflex; s++; continue;
				case '"':  *t = PSL34_E_umlaut; s++; continue;
				default:   *t = *s; continue;
				}
			case 'I' :
				switch(s[1]) {
				case '`':  *t = PSL34_I_grave; s++; continue;
				case '\'': *t = PSL34_I_acute; s++; continue;
				case '^':  *t = PSL34_I_circumflex; s++; continue;
				case '"':  *t = PSL34_I_umlaut; s++; continue;
				default:   *t = *s; continue;
				}
			case 'O' :
				switch(s[1]) {
				case '`':  *t = PSL34_O_grave; s++; continue;
				case '\'': *t = PSL34_O_acute; s++; continue;
				case '^':  *t = PSL34_O_circumflex; s++; continue;
				case '~':  *t = PSL34_O_tilde; s++; continue;
				case '"':  *t = PSL34_O_umlaut; s++; continue;
				default:   *t = *s; continue;
				}
			case 'U' :
				switch(s[1]) {
				case '`':  *t = PSL34_U_grave; s++; continue;
				case '\'': *t = PSL34_U_acute; s++; continue;
				case '^':  *t = PSL34_U_circumflex; s++; continue;
				case '"':  *t = PSL34_U_umlaut; s++; continue;
				default:   *t = *s; continue;
				}
			case 'Y' :
				switch(s[1]) {
				case '\'': *t = PSL34_Y_acute; s++; continue;
				default:   *t = *s; continue;
				}
			case 'a' :
				switch(s[1]) {
				case '`':  *t = PSL34_a_grave; s++; continue;
				case '\'': *t = PSL34_a_acute; s++; continue;
				case '^':  *t = PSL34_a_circumflex; s++; continue;
				case '~':  *t = PSL34_a_tilde; s++; continue;
				case '"':  *t = PSL34_a_umlaut; s++; continue;
				case '0':  *t = PSL34_a_ring; s++; continue;
				default:   *t = *s; continue;
				}
			case 'c' :
				switch(s[1]) {
				case '~':  *t = PSL34_c_cedilla; s++; continue;
				default:   *t = *s; continue;
				}
			case 'e' :
				switch(s[1]) {
				case '`':  *t = PSL34_e_grave; s++; continue;
				case '\'': *t = PSL34_e_acute; s++; continue;
				case '^':  *t = PSL34_e_circumflex; s++; continue;
				case '"':  *t = PSL34_e_umlaut; s++; continue;
				default:   *t = *s; continue;
				}
			case 'i' :
				switch(s[1]) {
				case '`':  *t = PSL34_i_grave; s++; continue;
				case '\'': *t = PSL34_i_acute; s++; continue;
				case '^':  *t = PSL34_i_circumflex; s++; continue;
				case '"':  *t = PSL34_i_umlaut; s++; continue;
				default:   *t = *s; continue;
				}
			case 'o' :
				switch(s[1]) {
				case '`':  *t = PSL34_o_grave; s++; continue;
				case '\'': *t = PSL34_o_acute; s++; continue;
				case '^':  *t = PSL34_o_circumflex; s++; continue;
				case '~':  *t = PSL34_o_tilde; s++; continue;
				case '"':  *t = PSL34_o_umlaut; s++; continue;
				default:   *t = *s; continue;
				}
			case 'u' :
				switch(s[1]) {
				case '`':  *t = PSL34_u_grave; s++; continue;
				case '\'': *t = PSL34_u_acute; s++; continue;
				case '^':  *t = PSL34_u_circumflex; s++; continue;
				case '"':  *t = PSL34_u_umlaut; s++; continue;
				default:   *t = *s; continue;
				}
			case 'y' :
				switch(s[1]) {
				case '\'': *t = PSL34_y_acute; s++; continue;
				default:   *t = *s; continue;
				}
			default :
				*t = *s; continue;
			}
		}
	}

	if (*t != '\0')
		*t = '\0';

	DBUG_RETURN(str);
}

char *
nomet_to_psl(str, langnum)
char *str;
int  langnum;
{
	return asc_to_psl(str, langnum);
}
