/*
*      SCCS:   @(#)common/genlib/clean_dir.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		clean_dir.c - remove everything from a directory

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	clean_dir()
SYNOPSIS:
	int clean_dir(path)
		char *path;

DESCRIPTION:
	  Attempts to delete everything in directory "path".  If
	successful SUCCEED is returned otherwise FAIL is returned.

CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
	Errno may be changed
RELATED DOCUMENTS:
SOURCE:
AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Mon Oct 20 16:30:36 BST 1986
MODIFICATIONS:

************************************************************************/

#include	<std.h>
#include	<stdio.h>	
#include	<dbug.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<string.h>
#include	<dirent.h>
#include	<errno.h>
#include	<limits.h>
#include	<string.h>
#include	<sysdep.h>

#define	DOT	"."
#define	DOTDOT	".."

private	int	do_dclean();

public int
clean_dir(path)
	char	*path;
{
	struct stat	statb;
	int	tmp;

	DBUG_LENTER("clean_dir", 2);
	DBUG_PRINTF("args", "clean_dir(%s)", path);

	if (path == (char *) NULL
		|| stat(path, &statb) == SYSERROR
		|| !S_ISDIR(statb.st_mode))
			DBUG_RETURN(FAIL);
	(void) chmod(path, S_IRWXU|S_IRWXG|S_IRWXO);

	tmp = do_dclean(path);
	DBUG_PRINTF("return", "clean_dir = %d", tmp);
	DBUG_RETURN(tmp);
}


private	int
do_dclean(path)
	char	*path;
{
	int		ret = SUCCEED;
	DIR		*dir;
	struct dirent	*dp;
	struct stat	statb;
	char		nbuf[_POSIX_PATH_MAX];

	if ((dir = opendir(path)) == (DIR *) NULL)
		return FAIL;
	while ((dp = readdir(dir)) != (struct dirent *) NULL)
	{
		if (strncmp(dp->d_name, DOT, 2) == 0
			|| strncmp(dp->d_name, DOTDOT, 3) == 0)
				continue;
		(void) strncat(strcat(strcpy(nbuf, path), "/"), dp->d_name, _POSIX_NAME_MAX);
		(void) chmod(nbuf, S_IRWXU|S_IRWXG|S_IRWXO);
		if (stat(nbuf, &statb) == SYSERROR)
			ret = FAIL;
		else if (S_ISDIR(statb.st_mode))
		{
			if (do_dclean(nbuf) != SUCCEED || rmdir(nbuf) == FAIL)
					ret = FAIL;
		}
		else if (unlink(nbuf) == SYSERROR)
			ret = FAIL;
	}
	closedir(dir);

	return ret;
}
