/*
*      SCCS:   @(#)common/genlib/crdir.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		crdir.c - make a directory with mode, uid, gui

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	crdir()
SYNOPSIS:
	int crdir(path, mode, uid, gid)
		char *path;
		mode_t mode;
		int uid;
		int gid;

DESCRIPTION:
	  crdir attempts to make the directory "path" with mode "mode",
	owner "gid" and group "gid".  If the mkdir fails
	it then trys to find out why.  If the directory already exists
	then the chmod, chown and chgrp are carried out.  If "path" exists
	and is a regular file, crdir will attempt to remove it and make
	the directory again, if this fails then EEXIST is returned.
	If crdir fails for other reasons such as EACCES, EPERM, EFAULT,
	EROFS, or ENOSPC the correct error is returned.  If one of
	chmod, chown, chgrp fails then the directory is removed and
	FAIL is returned (with errno set appropriatly).

CONDITIONAL COMPILE PARAMS:

SIDE EFFECTS:
	Errno will be changed, file "path" may be removed if it is
	an ordinary file.
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Thu Aug  7 15:33:40 PDT 1986
MODIFICATIONS:	chown is not attempted if it is already own directory.
		Manuch Ghezel-Ayagh, UniSoft Ltd., Tue Jan  6 15:42:44 WET 1987

		Geoff Clare, 17 July 1992
			Change mode argument from int to mode_t.

		Geoff Clare, 13 Oct 1992
			Allow for NFS user-id mapping.

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<errno.h>
#include	<sysdep.h>

public int
crdir(path, mode, uid, gid)

	char	*path;
	mode_t	mode;
	int	uid;
	int	gid;
{
	int	err;
	struct	stat	statbuf;

	DBUG_LENTER("crdir", 2);
	DBUG_PRINTF("args", "crdir(%s, 0%o, %d, %d)", path, mode, uid, gid);

	if (path == (char *) NULL)
	{
		errno = EFAULT;
		DBUG_PRINTF("return", "crdir = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}

	if (vxmkdir(path) == FAIL)
	{
		DBUG_PRINTF("return", "crdir = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}

	if (chmod(path, mode) == SYSERROR)
	{
		err = errno;
		(void) rmdir(path);
		errno = err;
		DBUG_PRINTF("return", "crdir = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}

	if ((stat(path, &statbuf) == SYSERROR) ||
	    (statbuf.st_uid != uid) || (statbuf.st_gid != gid))
	{
#if TEST_XNFS
		/* Allow for NFS user-id mapping */
		if (uid == 0 && geteuid() == 0)
			uid = statbuf.st_uid;
#endif
		if (chown(path, (uid_t)uid, (gid_t)gid) == SYSERROR)
		{
			err = errno;
			(void) rmdir(path);
			errno = err;
			DBUG_PRINTF("return", "crdir = %d", SYSERROR);
			DBUG_RETURN(SYSERROR);
		}
	}

	DBUG_PRINTF("return", "crdir = %d", SUCCEED);
	DBUG_RETURN(SUCCEED);
}
