/*
*      SCCS:   @(#)common/genlib/crfile.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		crfile.c - create file with given name, mode, uid & gid;

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		crfile()
USAGE:
	int crfile(path, mode, uid, gid)
		char	*path;
		mode_t	mode;
		int	uid;
		int	gid;
DESCRIPTION:
	Crfile tries to creat a file with the given path, mode, uid and
	gid. If successful, it will return the file descriptor of the
	created file; if not, SYSERROR will be returned.
	Any file already called "path" will be truncated, unless permission
	was denied.
SIDE EFFECTS:	Sets globok to 1 iff path checking worked, 0 otherwise.
		This assumed to be defined in the source of the caller.
RELATED DOCUMENTS:
SOURCE:
AUTHOR:		Hamish Reid, UniSoft Ltd.
DATE CREATED:	Wed Jun 25 15:43:21 BST 1986
MODIFICATIONS:	chown() is not attempted if it is own file.
		Manuch Ghezel-Ayagh, UniSoft Ltd., Tue Jan  6 15:42:44 WET 1987

		Geoff Clare, 17 July 1992
			Change mode argument from int to mode_t.

		Geoff Clare, 13 Oct 1992
			Allow for NFS user-id mapping.

************************************************************************/

#include	<std.h>
#include	<stdio.h>	
#include	<dbug.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<sysdep.h>

int
crfile(path, mode, uid, gid)
	char	*path;	/* file name */
	mode_t	mode;	/* creation mode */
	int	uid;	/* uid for new file, if possible */
	int	gid;	/* gid for new file, if possible */
{
	/* create new file with path, mode, uid and gid as given
	above; return file descriptor for file,
	or SYSERROR if any part unsuccesful, leaving
	errno untouched (could be chmod fail, for instance).
	*/

	int	fd;
	struct	stat	statbuf;

	DBUG_LENTER("crfile", 2);
	DBUG_PRINTF("args", "crfile(%s, 0%o, %d, %d)", path, mode, uid, gid);

	if ((fd = creat(path, mode)) == SYSERROR)
	{
		DBUG_PRINTF("return", "crfile = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}
	else
	{
		/* the chmod() is not as silly as it looks - if the
		file already existed, it keeps its old mode, therefore
		we chmod() to be sure.
		*/

		if (chmod(path, mode) == SYSERROR)
		{
			(void)close(fd);
			DBUG_PRINTF("return", "crfile = %d", SYSERROR);
			DBUG_RETURN(SYSERROR);
		}

		if ((stat(path, &statbuf) == SYSERROR) ||
		    (statbuf.st_uid != uid) || (statbuf.st_gid != gid))
		{
#if TEST_XNFS
			/* Allow for NFS user-id mapping */
			if (uid == 0 && geteuid() == 0)
				uid = statbuf.st_uid;
#endif
			if (chown(path, (uid_t)uid, (gid_t)gid) == SYSERROR)
			{
				(void)close(fd);
				DBUG_PRINTF("return", "crfile = %d",SYSERROR);
				DBUG_RETURN (SYSERROR);
			}
		}
	}

	DBUG_PRINTF("return", "crfile = %d", fd);
	DBUG_RETURN(fd);
}
