/*
*      SCCS:   @(#)common/genlib/eaccess.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		eaccess.c - an effective uid access

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	eaccess()
SYNOPSIS:
	int eaccess(path, mode)
		char	*path;
		int	mode;

DESCRIPTION:
	   eaccess() attempts to do the equivelent of an access,
	using the euid instead of uid.  the return values are SUCCEED
	and SYSERROR, if SYSERROR is returned errno will be set appropriatly.

CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
	Errno will be changed.
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Thu Aug  7 15:33:40 PDT 1986
MODIFICATIONS:

************************************************************************/

#include	<std.h>
#include	<errno.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<unistd.h>
#include	<sysdep.h>

#define	SUPER_USER	0		/* ie: root's uid */
#define	USERBITS	6		/* where the user mode bits start */
#define	GROUPBITS	3		/*           group                */
#define	OTHERBITS	0		/*           other                */
#define	MODEMASK	07		/* mask for one group of bits in mode */

public int
eaccess(path, mode)

	char	*path;
	int	mode;
{
	int	euid;
	int	egid;
	int	shift;
	struct	stat statb;

	DBUG_LENTER("eaccess", 2);
	DBUG_PRINTF("args", "eaccess(%s, %d)", path, mode);

	if (path == (char *) NULL)
	{
		errno = EFAULT;
		DBUG_PRINTF("return", "eaccess = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}

	if (stat(path, &statb) == SYSERROR)
	{
		DBUG_PRINTF("return", "eaccess = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}

	if ((euid = geteuid()) == SUPER_USER)
	{
		DBUG_PRINTF("return", "eaccess = %d", SUCCEED);
		DBUG_RETURN(SUCCEED);
	}

	egid = getegid();
	shift = statb.st_uid == euid ? USERBITS :
		(statb.st_gid == egid ? GROUPBITS :
			OTHERBITS);
	mode &= MODEMASK;
	if ((mode & (statb.st_mode >> shift)) == mode)
	{
		DBUG_PRINTF("return", "eaccess = %d", SUCCEED);
		DBUG_RETURN(SUCCEED);
	}

	errno = EACCES;
	DBUG_PRINTF("return", "eaccess = %d", SYSERROR);
	DBUG_RETURN(SYSERROR);
}
