/*
*      SCCS:   @(#)common/genlib/errname.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		errname.c - return error name for error number

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		errname()
SYNOPSIS:
		char	*errname(number)
				int	number;
DESCRIPTION:
	errname() returns a pointer to a static string containing the
	error name (from <errno.h>) corresponding to the error number
	in "number"; if none exists, the string "UNKNOWN" is returned.
	Errno = 0 returns "NO ERROR".
	NOTE: this procedure only considers those error names and numbers
	defined in the XPG.
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:
AUTHOR:		Hamish Reid, UniSoft Ltd.
DATE CREATED:	Thu Jul  3 17:32:38 BST 1986
MODIFICATIONS:
		Geoff Clare, 4 Apr 1989:  XPG3 errno values

		Geoff Clare, 3 Nov 1989
			Protect non-POSIX errnos with #ifdef _XOPEN_SOURCE

		Geoff Clare, 19 Oct 1992
			Only include ENOTBLK in XPG3 mode.
			Add EILSEQ in XPG4 mode.
			Add ESTALE if testing XNFS.

		Geoff Clare, 10 Jan 1997
			Add new errno values from XSH5.

************************************************************************/

#include	<std.h>
#include	<dbug.h>
#include	<errno.h>
#include	<sysdep.h>

private	struct	nament	{
	int	eno;
	char	*ename;
} name[] = {
	NOERROR,	"NO ERROR",
	E2BIG,		"E2BIG",
	EACCES,		"EACCES",
#ifdef EADDRINUSE
	EADDRINUSE,	"EADDRINUSE",
#endif
#ifdef EADDRNOTAVAIL
	EADDRNOTAVAIL,	"EADDRNOTAVAIL",
#endif
#ifdef EAFNOSUPPORT
	EAFNOSUPPORT,	"EAFNOSUPPORT",
#endif
	EAGAIN,		"EAGAIN",
#ifdef EALREADY
	EALREADY,	"EALREADY",
#endif
	EBADF,		"EBADF",
#ifdef EBADMSG
	EBADMSG,	"EBADMSG",
#endif
	EBUSY,		"EBUSY",
#ifdef ECANCELED
	ECANCELED,	"ECANCELED",
#endif
	ECHILD,		"ECHILD",
#ifdef ECONNABORTED
	ECONNABORTED,	"ECONNABORTED",
#endif
#ifdef ECONNREFUSED
	ECONNREFUSED,	"ECONNREFUSED",
#endif
#ifdef ECONNRESET
	ECONNRESET,	"ECONNRESET",
#endif
	EDEADLK,	"EDEADLK",
#ifdef EDESTADDRREQ
	EDESTADDRREQ,	"EDESTADDRREQ",
#endif
	EDOM,		"EDOM",
#ifdef EDQUOT
	EDQUOT,		"EDQUOT",
#endif
	EEXIST,		"EEXIST",
	EFAULT,		"EFAULT",
	EFBIG,		"EFBIG",
#ifdef EHOSTUNREACH
	EHOSTUNREACH,	"EHOSTUNREACH",
#endif
#ifdef EIDRM
	EIDRM,		"EIDRM",
#endif
#ifdef EILSEQ
	EILSEQ,		"EILSEQ",
#endif
#ifdef EINPROGRESS
	EINPROGRESS,	"EINPROGRESS",
#endif
	EINTR,		"EINTR",
	EINVAL,		"EINVAL",
	EIO,		"EIO",
#ifdef EISCONN
	EISCONN,	"EISCONN",
#endif
	EISDIR,		"EISDIR",
#ifdef ELOOP
	ELOOP,		"ELOOP",
#endif
	EMFILE,		"EMFILE",
	EMLINK,		"EMLINK",
#ifdef EMSGSIZE
	EMSGSIZE,	"EMSGSIZE",
#endif
#ifdef EMULTIHOP
	EMULTIHOP,	"EMULTIHOP",
#endif
	ENAMETOOLONG,	"ENAMETOOLONG",
#ifdef ENETDOWN
	ENETDOWN,	"ENETDOWN",
#endif
#ifdef ENETUNREACH
	ENETUNREACH,	"ENETUNREACH",
#endif
	ENFILE,		"ENFILE",
#ifdef ENOBUFS
	ENOBUFS,	"ENOBUFS",
#endif
#ifdef ENODATA
	ENODATA,	"ENODATA",
#endif
	ENODEV,		"ENODEV",
	ENOENT,		"ENOENT",
	ENOEXEC,	"ENOEXEC",
	ENOLCK,		"ENOLCK",
#ifdef ENOLINK
	ENOLINK,	"ENOLINK",
#endif
	ENOMEM,		"ENOMEM",
#ifdef ENOMSG
	ENOMSG,		"ENOMSG",
#endif
#ifdef ENOPROTOOPT
	ENOPROTOOPT,	"ENOPROTOOPT",
#endif
	ENOSPC,		"ENOSPC",
#ifdef ENOSR
	ENOSR,		"ENOSR",
#endif
#ifdef ENOSTR
	ENOSTR,		"ENOSTR",
#endif
	ENOSYS,		"ENOSYS",
#if TEST_XPG == 3
	ENOTBLK,	"ENOTBLK",
#endif
#ifdef ENOTCONN
	ENOTCONN,	"ENOTCONN",
#endif
	ENOTDIR,	"ENOTDIR",
	ENOTEMPTY,	"ENOTEMPTY",
#ifdef ENOTSOCK
	ENOTSOCK,	"ENOTSOCK",
#endif
#ifdef ENOTSUP
	ENOTSUP,	"ENOTSUP",
#endif
	ENOTTY,		"ENOTTY",
	ENXIO,		"ENXIO",
#ifdef EOPNOTSUPP
	EOPNOTSUPP,	"EOPNOTSUPP",
#endif
#ifdef EOVERFLOW
	EOVERFLOW,	"EOVERFLOW",
#endif
	EPERM,		"EPERM",
	EPIPE,		"EPIPE",
#ifdef EPROTO
	EPROTO,		"EPROTO",
#endif
#ifdef EPROTONOSUPPORT
	EPROTONOSUPPORT,"EPROTONOSUPPORT",
#endif
#ifdef EPROTOTYPE
	EPROTOTYPE,	"EPROTOTYPE",
#endif
	ERANGE,		"ERANGE",
	EROFS,		"EROFS",
	ESPIPE,		"ESPIPE",
	ESRCH,		"ESRCH",
#ifdef ESTALE
	ESTALE,		"ESTALE",
#endif
#ifdef ETIME
	ETIME,		"ETIME",
#endif
#ifdef ETIMEDOUT
	ETIMEDOUT,	"ETIMEDOUT",
#endif
#ifdef ETXTBSY
	ETXTBSY,	"ETXTBSY",
#endif
#ifdef EWOULDBLOCK
	EWOULDBLOCK,	"EWOULDBLOCK",
#endif
	EXDEV,		"EXDEV",
 };

char *
errname(number)
	int	number;
{
	int	i;

	DBUG_LENTER("errname", 2);
	DBUG_PRINTF("args", "errname(%d)", number);

	 for (i = 0; i < ((sizeof name)/(sizeof(struct nament))); i++)
	 {
		if (name[i].eno == number)
		{
			DBUG_PRINTF("return", "errname = %s", name[i].ename);
			DBUG_RETURN(name[i].ename);
		}
	 }
	 DBUG_PRINTF("return", "errname = UNKNOWN");
	 DBUG_RETURN("UNKNOWN");
}
