/*
*      SCCS:   @(#)common/genlib/execpath.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		execpath - Return path to executable subprograms

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	execpath()
SYNOPSIS:
	char *execpath(progname)
		char *progname;

DESCRIPTION:
	execpath() returns a path name which can be used to refer to
	the executable subprogram specified by progname.   This is
	located in the directory where the testset was originally
	executed.  When testing XNFS this directory was saved in
	the global startdir[] by the (*tet_startup)() function,
	otherwise it is just assumed to be the parent of the
	current directory.

SOURCE:		
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	10 Sept 1992
MODIFICATIONS:	

************************************************************************/

#include <std.h>
#include <string.h>
#include <limits.h>
#include <dbug.h>
#include <sysdep.h>

#if TEST_XNFS
extern	char	startdir[];
#endif

private char	retpath[_POSIX_PATH_MAX+1];

public char *
execpath(progname)
char *progname;
{
	size_t len;

	DBUG_LENTER("execpath", 2);
	DBUG_PRINTF("args", "execpath(%s)", progname);

#if TEST_XNFS
	(void) strcpy(retpath, startdir);
#else
	(void) strcpy(retpath, "..");
#endif

	len = strlen(retpath);
	retpath[len++] = '/';
	retpath[len] = '\0';
	(void) strncat(&retpath[len], progname, (size_t)(_POSIX_PATH_MAX-len));

	DBUG_RETURN(retpath);
}
