/*
*      SCCS:   @(#)common/genlib/itoa.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		itoa.c - integer to "ascii" routine.

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	itoa()
SYNOPSIS:
	char *itoa(num)
		int num;
DESCRIPTION:
	itoa() returns a pointer to a static character array which
	holds a printable representation of num.

SOURCE:		
AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Mon Aug 18 13:50:14 PDT 1986
MODIFICATIONS:
		Geoff Clare, 21 Nov 1996
		Calculate required buffer size.
		Avoid overflow when num is INT_MIN.

************************************************************************/

#include	<std.h>
#include	<stdio.h>	
#include	<limits.h>	
#include	<dbug.h>
#include	<sysdep.h>

public char *
itoa(num)
int num;
{
	static char nbuf[2 + (sizeof(num)*CHAR_BIT*10 + 32)/33];
	char *p;
	unsigned int magn;
	int minus = 0;

	DBUG_LENTER("itoa", 2);
	DBUG_PRINTF("args", "itoa(%d)", num);

	p = &nbuf[sizeof(nbuf)];
	*--p = '\0';

	if (num == 0)
		*--p = '0';
	else
	{
		if (num < 0)
		{
			minus = 1;
			/* cast needed because -num can overflow */
			magn = -(unsigned int)num;
		}
		else
			magn = num;

		while (magn != 0)
		{
			*--p = "0123456789"[magn % 10];
			magn /= 10;
		}

		if (minus)
			*--p = '-';
	}

	DBUG_PRINTF("return", "itoa = %s", p);
	DBUG_RETURN(p);
}
