/*
*      SCCS:   @(#)common/genlib/killchild.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		killchild.c - kill a child and wait for it to return.

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	killchild()
SYNOPSIS:
	int killchild(child, timeout)
		int		child;
		unsigned	timeout;

DESCRIPTION:
	Killchild() kill()s "child" with SIGKILL and then waitpid()'s
	for the child to return.  If the wait times out it will retry
	the kill() and waitpid() a fixed number of times.  If the
	wait fails for any other reason A_WAITFAIL is returned.
	If the child exits or was not there (ECHILD) then SUCCEED is
	returned.  If the kill() fails and errno is ESRCH the wait is
	done anyway.

	Note that, because of the retries, the time spent in this routine
	may be much longer than the timeout given.

CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
	- Errno may be changed.
RELATED DOCUMENTS:
SOURCE:
AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Thu Aug 28 11:36:30 BST 1986
MODIFICATIONS:
		Geoff Clare, 15 Dec 1988
			Use waitpid() instead of wait().
			Allow retries.

		Geoff Clare, 17 July 1992
			Change child argument from int to pid_t.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>	
#include	<sys/wait.h>	
#include	<dbug.h>
#include	<errno.h>
#include	<signal.h>
#include	<timeout.h>
#include	<exitcodes.h>
#include	<sysdep.h>

#define	RETRIES	5

extern	pid_t	waitpid();

int
killchild(child, timeout)
	pid_t		child;
	unsigned	timeout;
{
	pid_t		pid;
	int		ret = SUCCEED;
	int		count, status;

	DBUG_LENTER("killchild", 2);
	DBUG_PRINTF("args", "killchild(%d, %u)", child, timeout);

	for (count = 0; count < RETRIES; count++)
	{
		if (kill(child, SIGKILL) == SYSERROR && errno != ESRCH)
		{
			ret = A_KILLFAIL;
			break;
		}

		SET_TIMEOUT(timeout)
		pid = waitpid(child, &status, 0);
		CLEAR_ALARM

		if (pid == SYSERROR && alrm_flag == 0)
		{
			if (errno != ECHILD)
				ret = A_WAITFAIL;
			break;
		}
	}
	if (count == RETRIES)
		ret = A_WAITFAIL;

	DBUG_PRINTF("return", "killchild = %d", ret);
	DBUG_RETURN(ret);
}
