/*
*      SCCS:   @(#)common/genlib/pr_char.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		pr_char.c - char to printable acsii string library routine

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	pr_char()
SYNOPSIS:
	char *pr_char(c)
		char c;

	char *pr_nchars(buffer,nitem)
		char *buffer;
		int nitem;
DESCRIPTION:
	pr_char() returns a pointer to a (static) string containing
	a printable string for the character.

	pr_nchars() returns a pointer to a (malloc'ed) string containing
	a printable string for the sequence of nitem bytes pointed to
	in buf.

SOURCE:		
AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Mon Aug 11 15:55:53 PDT 1986
MODIFICATIONS:
		Geoff Clare, 22 May 1989
			Use isprint() rather than assuming ASCII, print
			in 'C' string literal notation instead of "^A" etc.

		Stuart Boutell, 2 May 1991
			Added pr_nchars(), and moved common code to internal
			library function.

************************************************************************/

#include	<std.h>
#include	<stdio.h>	
#include	<stdlib.h>
#include	<ctype.h>	
#include	<dbug.h>
#include	<sysdep.h>

private	int
process_char(ch,t)
	unsigned char	ch;
	char	*t;
{
	int	i=0;

	switch (ch)
	{
	case '\n' : i+=2; *t++ = '\\'; *t++ = 'n'; break;
	case '\t' : i+=2; *t++ = '\\'; *t++ = 't'; break;
	case '\v' : i+=2; *t++ = '\\'; *t++ = 'v'; break;
	case '\b' : i+=2; *t++ = '\\'; *t++ = 'b'; break;
	case '\r' : i+=2; *t++ = '\\'; *t++ = 'r'; break;
	case '\f' : i+=2; *t++ = '\\'; *t++ = 'f'; break;
	case '\\' : i+=2; *t++ = '\\'; *t++ = '\\'; break;
	default :   
		if (isprint((int)ch))
		{
			i++; *t++ = ch;
		}
		else
		{
			i+=4;
			*t++ = '\\';
			*t++ = '0' + ((ch >> 6) & 07);
			*t++ = '0' + ((ch >> 3) & 07);
			*t++ = '0' + (ch & 07);
		}
	}

	*t = '\0';

	return i;
}

public  char *
pr_nchars(buffer,nitems)
	char *buffer;
	int   nitems;
{
	static	char *fail_msg="ERROR:malloc() failed in pr_nchars()";
	char	cbuf[5];	/* buffer to hold character processing in */
	char	*cbufp;
	int	length;		/* Length of string to return  */
	char	*dp, *result;	
	int	done;


/* calculate the length of the return string */
	length = 1;
	cbufp = cbuf;
	for(dp = buffer, done = 0; done < nitems; dp++, done++)
		length += process_char((unsigned char)*dp, cbufp);

	result = (char *) malloc( (size_t) length);

	if(result == NULL)
		return fail_msg;

	cbufp = result;
	for(dp = buffer, done = 0; done < nitems; dp++, done++)
		cbufp += process_char((unsigned char)*dp, cbufp);

	*cbufp = '\0';
	
	return result;
}

public	char *
pr_char(c)
	char	c;
{
	static char	buf[5];
	char		*t = buf;

	(void)	process_char((unsigned  char)c,t);

	return	buf;
}


