/*
*      SCCS:   @(#)common/genlib/pr_wchar.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		pr_wchar.c - return printable representation of wide characters

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	pr_wchar()
	pr_wstr()
SYNOPSIS:
	char *pr_wchar(wc)
		wchar_t wc;

	char *pr_wstr(ws)
		wchar_t *ws;
DESCRIPTION:
	pr_wchar() returns a pointer to a (static) string containing
	a printable representation of the wide character.

	pr_wstr() returns a pointer to a (malloc'ed) string containing
	a printable representation of the wide character string.

SIDE EFFECTS:
	pr_wchar() may call pr_char() thus overwiting the internal
	static buffer used by pr_char().

AUTHOR:		Neil Moses, UniSoft Ltd.
DATE CREATED:	02/09/91
MODIFICATIONS:
		Geoff Clare, 4 Feb 1992
			Remove #include <wchar.h>

************************************************************************/

#include <std.h>

#if defined(_XOPEN_SOURCE) && TEST_XPG > 3

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <sysdep.h>

extern char *pr_char();
extern char *pr_nchars();

char *
pr_wchar(wc)
wchar_t wc;
{
	static char buf[50];

	if (wc >= 0 && wc <= CHAR_MAX)
		return pr_char((int)wc);

	(void) sprintf(buf, "0x%lx", (unsigned long)wc);
	return buf;
}


char *
pr_wstr(pwcs)
wchar_t *pwcs;
{
	char	*buf, *ptr;
	int	len, i, normal;

	normal = 1;
	for(len = 0; *(pwcs + len) != '\0'; len++)
	{
		if (*(pwcs + len) < 0 || *(pwcs + len) > CHAR_MAX)
			normal = 0;
	}

	if (normal)
		buf = (char *) malloc((size_t)(len + 1));
	else
		buf = (char *) malloc((size_t)
			(((1 + sizeof(wchar_t) * CHAR_BIT / 4) * len) + 4));
	if (buf == NULL)
		return "malloc() failed in pr_wstr()";
		
	ptr = buf;
	for(i = 0; i < len; i++)
	{
		if (normal)
			*ptr++ = *pwcs++;
		else
		{
			if(i == 0)
			{
				*ptr++ = '0';
				*ptr++ = 'x';
			}
			(void) sprintf(ptr, " %lx", (unsigned long)*pwcs++);
			ptr += strlen(ptr);
		}
	}
	*ptr = '\0';

	if (normal)
	{
		ptr = pr_nchars(buf, len);
		free((void *)buf);
		return ptr;
	}
	else
		return buf;
}

#else /* defined(_XOPEN_SOURCE) && TEST_XPG > 3 */
static int dummy;
#endif
