/*
*      SCCS:   @(#)common/genlib/psl_to_asc.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		psl_to_asc - Convert Pseudo Language characters to a
				C-locale-printable form

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	psl_to_asc()
SYNOPSIS:
	char *psl_to_asc(str, langnum)
		char	*str;
		int	langnum;
DESCRIPTION:
	psl_to_asc converts from a pseudo language string to a
	representation that is printable in the C locale.

SOURCE:		
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	Tue Nov 10 16:42:22 WET 1987
MODIFICATIONS:
		Geoff Clare, 29 May 1992
			Add '#' and '$' to INFO1LANG and INFO2LANG

		Geoff Clare, 22 March 1995
			Use codes from <pslcodes.h>

************************************************************************/

#include	<std.h>
#include	<stdlib.h>
#include	<nlstest.h>
#include	<nlsclass.h>
#include	<stdio.h>	
#include	<string.h>	
#include	<sys/types.h>	
#include	<dbug.h>
#include	<pslcodes.h>
#include	<sysdep.h>

#define BUFLEN 1024

#define OCTAL(t,s)	{ *t = '\\'; \
			  *++t = '0' + ((*s >> 6) & 07); \
			  *++t = '0' + ((*s >> 3) & 07); \
			  *++t = '0' + (*s & 07); }

char *
psl_to_asc(str, langnum)
char *str;
int  langnum;
{
	register unsigned char *s, *t;
	unsigned char buf[BUFLEN];

	if (*str == '\0')
		return str;

	for (s = (unsigned char *)str, t = buf;
			*s != '\0' && t < &buf[BUFLEN-5]; s++, t++)
	{
		switch(*s)
		{
		case '\n': *t = '\\'; *++t = 'n'; continue;
		case '\t': *t = '\\'; *++t = 't'; continue;
		case '\v': *t = '\\'; *++t = 'v'; continue;
		case '\b': *t = '\\'; *++t = 'b'; continue;
		case '\r': *t = '\\'; *++t = 'r'; continue;
		case '\f': *t = '\\'; *++t = 'f'; continue;
		case '\\': *t = '\\'; *++t = '\\'; continue;
		}

		*t = '\0';	/* mark as not yet assigned */

		if (langnum == VALINFO1LANG || langnum == VALINFO2LANG)
		{
			switch(*s)
			{
			case PSL12_A_umlaut: *t = 'A'; *++t = '"';break;
			case PSL12_O_umlaut: *t = 'O'; *++t = '"';break;
			case PSL12_U_umlaut: *t = 'U'; *++t = '"';break;
			case PSL12_cedilla:  *t = '~';break;
			case PSL12_a_umlaut: *t = 'a'; *++t = '"';break;
			case PSL12_o_umlaut: *t = 'o'; *++t = '"';break;
			case PSL12_u_umlaut: *t = 'u'; *++t = '"';break;
			case PSL12_eszet:    *t = '&';break;
			}
		}
		else if (langnum == VALINFO3LANG || langnum == VALINFO4LANG)
		{
			switch(*s)
			{
			case PSL34_A_grave:      *t = 'A'; *++t = '`';break;	
			case PSL34_A_acute:      *t = 'A'; *++t = '\'';break;	
			case PSL34_A_circumflex: *t = 'A'; *++t = '^';break;	
			case PSL34_A_tilde:      *t = 'A'; *++t = '~';break;	
			case PSL34_A_umlaut:     *t = 'A'; *++t = '"';break;	
			case PSL34_A_ring:       *t = 'A'; *++t = '0';break;	
			case PSL34_C_cedilla:    *t = 'C'; *++t = '~';break;	
			case PSL34_E_grave:      *t = 'E'; *++t = '`';break;	
			case PSL34_E_acute:      *t = 'E'; *++t = '\'';break;	
			case PSL34_E_circumflex: *t = 'E'; *++t = '^';break;	
			case PSL34_E_umlaut:     *t = 'E'; *++t = '"';break;	
			case PSL34_I_grave:      *t = 'I'; *++t = '`';break;	
			case PSL34_I_acute:      *t = 'I'; *++t = '\'';break;	
			case PSL34_I_circumflex: *t = 'I'; *++t = '^';break;	
			case PSL34_I_umlaut:     *t = 'I'; *++t = '"';break;	
			case PSL34_O_grave:      *t = 'O'; *++t = '`';break;	
			case PSL34_O_acute:      *t = 'O'; *++t = '\'';break;	
			case PSL34_O_circumflex: *t = 'O'; *++t = '^';break;	
			case PSL34_O_tilde:      *t = 'O'; *++t = '~';break;	
			case PSL34_O_umlaut:     *t = 'O'; *++t = '"';break;	
			case PSL34_U_grave:      *t = 'U'; *++t = '`';break;	
			case PSL34_U_acute:      *t = 'U'; *++t = '\'';break;	
			case PSL34_U_circumflex: *t = 'U'; *++t = '^';break;	
			case PSL34_U_umlaut:     *t = 'U'; *++t = '"';break;	
			case PSL34_Y_acute:      *t = 'Y'; *++t = '\'';break;	
			case PSL34_eszet:        *t = '&';break;
			case PSL34_a_grave:      *t = 'a'; *++t = '`';break;	
			case PSL34_a_acute:      *t = 'a'; *++t = '\'';break;	
			case PSL34_a_circumflex: *t = 'a'; *++t = '^';break;	
			case PSL34_a_tilde:      *t = 'a'; *++t = '~';break;	
			case PSL34_a_umlaut:     *t = 'a'; *++t = '"';break;	
			case PSL34_a_ring:       *t = 'a'; *++t = '0';break;	
			case PSL34_c_cedilla:    *t = 'c'; *++t = '~';break;	
			case PSL34_e_grave:      *t = 'e'; *++t = '`';break;	
			case PSL34_e_acute:      *t = 'e'; *++t = '\'';break;	
			case PSL34_e_circumflex: *t = 'e'; *++t = '^';break;	
			case PSL34_e_umlaut:     *t = 'e'; *++t = '"';break;	
			case PSL34_i_grave:      *t = 'i'; *++t = '`';break;	
			case PSL34_i_acute:      *t = 'i'; *++t = '\'';break;	
			case PSL34_i_circumflex: *t = 'i'; *++t = '^';break;	
			case PSL34_i_umlaut:     *t = 'i'; *++t = '"';break;	
			case PSL34_o_grave:      *t = 'o'; *++t = '`';break;	
			case PSL34_o_acute:      *t = 'o'; *++t = '\'';break;	
			case PSL34_o_circumflex: *t = 'o'; *++t = '^';break;	
			case PSL34_o_tilde:      *t = 'o'; *++t = '~';break;	
			case PSL34_o_umlaut:     *t = 'o'; *++t = '"';break;	
			case PSL34_u_grave:      *t = 'u'; *++t = '`';break;	
			case PSL34_u_acute:      *t = 'u'; *++t = '\'';break;	
			case PSL34_u_circumflex: *t = 'u'; *++t = '^';break;	
			case PSL34_u_umlaut:     *t = 'u'; *++t = '"';break;	
			case PSL34_y_acute:      *t = 'y'; *++t = '\'';break;	
			}
		}

		if (*t != '\0')
			continue;

		/* ensure lang0_table[] has been initialised */
		if (lang0_table[0][0] == 0)
			(void) pslclass();

		/* use '\ddd' notation for nonprinting chars */
		if (*s < 256 && (lang0_table[*s][0] & DEFPRINT))
			*t = *s;
		else
			OCTAL(t, s)
	}

	*t = '\0';

	/* return the string in a malloc()'ed buffer so subsequent
	   calls don't overwrite it */
	if (*s != '\0')
		return "string too long for buffer in psl_to_asc()";
	else {
		t = (unsigned char *)malloc((size_t)(1+strlen((char *)buf)));
		if (t == NULL)
			return "malloc() failed in psl_to_asc()";
		else
			return strcpy((char *)t, (char *)buf);
	}
}
