/*
*      SCCS:   @(#)common/genlib(pslanginfo.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/****************************************************************************

NAME:  		pslanginfo() - get VSX Pseudo-language information
		
MODULE CONTAINS:
       	pslanginfo()
SYNOPSIS:
	char *pslanginfo(item, langnum)
		int item;
		int langnum;

DESCRIPTION:
	Pslanginfo() returns a pointer to a string containing the
	requested item of Native Language information for the
	specified VSX pseudo-language.

AUTHOR:		Geoff Clare
DATE CREATED:	24 May 1989

MODIFICATIONS:
		Geoff Clare, 6 Oct 1989: don't convert CRNCYSTR

		Geoff Clare, 14 Nov 1989
			Do not pass string literals to asc_to_psl()
			Add debug calls

		Geoff Clare, 13 April 1995
			Different D_T_FMT for C locale in XPG4.

****************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<nlstest.h>
#include	<nlsinfo.h>
#include	<sysdep.h>


#define NUMLANG		6
#define NUMITEMS	48
#define D_FMT_ITEMNO	38
#define T_FMT_ITEMNO	39
#define D_T_FMT_ITEMNO	40
#define CRNCY_ITEMNO	47
#define ITEMLEN		25

static int	nlitems[NUMITEMS] = {
	VSX_DAY_1, VSX_DAY_2, VSX_DAY_3, VSX_DAY_4, VSX_DAY_5, VSX_DAY_6, 
	VSX_DAY_7, VSX_ABDAY_1, VSX_ABDAY_2, VSX_ABDAY_3, VSX_ABDAY_4, 
	VSX_ABDAY_5, VSX_ABDAY_6, VSX_ABDAY_7,
	VSX_MON_1, VSX_MON_2, VSX_MON_3, VSX_MON_4, VSX_MON_5, VSX_MON_6,
	VSX_MON_7, VSX_MON_8, VSX_MON_9, VSX_MON_10, VSX_MON_11, VSX_MON_12,
	VSX_ABMON_1, VSX_ABMON_2, VSX_ABMON_3, VSX_ABMON_4, VSX_ABMON_5, 
	VSX_ABMON_6, VSX_ABMON_7, VSX_ABMON_8, VSX_ABMON_9, VSX_ABMON_10, 
	VSX_ABMON_11, VSX_ABMON_12,
	VSX_D_FMT, VSX_T_FMT, VSX_D_T_FMT, VSX_AM_STR, VSX_PM_STR,
	VSX_RADIXCHAR, VSX_THOUSEP, VSX_YESSTR, VSX_NOSTR, VSX_CRNCYSTR,
};	

static char nlinfo[NUMLANG][NUMITEMS][ITEMLEN] = {
	{ /* 0 is CLANG */
		"Sunday", "Monday", "Tuesday", "Wednesday",
		"Thursday", "Friday", "Saturday",
		"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat",
		"January", "February", "March", "April",
		"May", "June", "July", "August",
		"September", "October", "November", "December",
		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
		"%m/%d/%y", "%H:%M:%S",
#if TEST_XPG > 3
		"%a %b %e %H:%M:%S %Y",
#else
		"%a %b %d %H:%M:%S %Y",
#endif
		"AM", "PM",
		".", "", "yes", "no", "",
	},
	{ /* 1 is NOINFOLANG */
		"", "", "", "", "", "", "",
		"", "", "", "", "", "", "",
		"", "", "", "", "", "", "", "", "", "", "", "",
		"", "", "", "", "", "", "", "", "", "", "", "",
		"", "", "", "", "",
		"", "", "", "", "",
	},
	{ /* 2 is INFO1LANG */
		"Sonntag", "Montag", "Dienstag", "Mittwoch",
		"Donnerstag", "Freitag", "Samstag",
		"Son", "Mon", "Die", "Mit", "Don", "Fre", "Sam",
		"Januar", "Februar", "Ma\"rz", "April",
		"Mai", "Juni", "Juli", "August",
		"September", "Oktober", "November", "Dezember",
		"Jan", "Feb", "Ma\"rz", "Apr", "Mai", "Juni",
		"Juli", "Aug", "Sept", "Okt", "Nov", "Dez",
		"%m/%d/%y", "%I.%M.%S %p", "%I.%M.%S %p %m/%d/%y", "VM", "NM",
		",", ".", "j", "n", "-Dm",
	},
	{ /* 3 is INFO2LANG */
		"dimanche", "lundi", "mardi", "mercredi",
		"jeudi", "vendredi", "samedi",
		"dim", "lun", "mar", "mer", "jeu", "ven", "sam",
		"janvier", "fe'vrier", "mars", "avril",
		"mai", "juin", "juillet", "aou^t",
		"septembre", "octobre", "novembre", "de'cembre",
		"jan", "fe'v", "mars", "avr", "mai", "juin",
		"juil", "aou^t", "sept", "oct", "nov", "de'c",
		"%d %m %Y", "%H:%M:%S", "%d %m %Y %H:%M:%S", "am", "pm",
		"I", ",", "o", "n", "+Fr"
	},
	{ /* 4 is INFO3LANG */
		"Sonntag", "Montag", "Dienstag", "Mittwoch",
		"Donnerstag", "Freitag", "Samstag",
		"Son", "Mon", "Die", "Mit", "Don", "Fre", "Sam",
		"Januar", "Februar", "Ma\"rz", "April",
		"Mai", "Juni", "Juli", "August",
		"September", "Oktober", "November", "Dezember",
		"Jan", "Feb", "Ma\"rz", "Apr", "Mai", "Juni",
		"Juli", "Aug", "Sept", "Okt", "Nov", "Dez",
		"%m/%d/%y", "%I.%M.%S %p", "%I.%M.%S %p %m/%d/%y", "VM", "NM",
		",", ".", "j", "n", "-Dm",
	},
	{ /* 5 is INFO3LANG@dict */
		"dimanche", "lundi", "mardi", "mercredi",
		"jeudi", "vendredi", "samedi",
		"dim", "lun", "mar", "mer", "jeu", "ven", "sam",
		"janvier", "fe'vrier", "mars", "avril",
		"mai", "juin", "juillet", "aou^t",
		"septembre", "octobre", "novembre", "de'cembre",
		"jan", "fe'v", "mars", "avr", "mai", "juin",
		"juil", "aou^t", "sept", "oct", "nov", "de'c",
		"%d %m %Y", "%H:%M:%S", "%d %m %Y %H:%M:%S", "am", "pm",
		"I", ",", "o", "n", "+Fr"
	}
	/* UNDEFLANG and NONASCIILANG will give "" */
};

extern char *asc_to_psl();

public char *
pslanginfo(item, langnum)
int item;
int langnum;
{
	int i, x;
	int *p;
	char (*s)[ITEMLEN];
	static int converted = 0;

	DBUG_LENTER("pslanginfo", 2);
	DBUG_PRINTF("args", "pslanginfo(%d, %d)", item, langnum);

	/* check if ASCII converted to Psuedo Language, if not,
	   convert tables for INFO1LANG and INFO2LANG */

	if (!converted) {
		for (x = VALINFO1LANG; x < NUMLANG; x++)
		{
		    for (s = nlinfo[x], i = 0; i < NUMITEMS; s++, i++)
			if (i != D_FMT_ITEMNO && i != T_FMT_ITEMNO &&
			    i != D_T_FMT_ITEMNO && i != CRNCY_ITEMNO)
				(void) asc_to_psl(*s, x);
		}
		converted = 1;
	}

	/* find item name */
	for (i = 0, p = nlitems; *p != item && i < NUMITEMS; i++, p++)
		;
	if (i >= NUMITEMS)
		DBUG_RETURN("");

	/* If unknown language return info for "C" locale */
	if (langnum < 0 || langnum >= NUMLANG)
		langnum = 0;

	/* If no info return corresponding string for "C" locale */
	if (nlinfo[langnum][i][0] == '\0')
		langnum = 0;

	DBUG_RETURN(nlinfo[langnum][i]);
}
