/*
*      SCCS:   @(#)common/genlib(pslclass.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1995 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/****************************************************************************

NAME:  		pslclass() - initialise VSX pseudo-language character
			     class tables
		
MODULE CONTAINS:
       	pslclass()
	lang0_table[], lang1_table[], lang3_table[]

SYNOPSIS:
	int pslclass(void)

DESCRIPTION:
	Pslclass() initialises the character class tables lang0_table[]
	etc. using the portable character set and the character encodings
	in the user-configurable files <ctrlcodes.h> and <pslcodes.h>.
	It returns 0 for success, non-zero for failure.

AUTHOR:		Geoff Clare
DATE CREATED:	March 1995

MODIFICATIONS:

****************************************************************************/

#include	<std.h>
#include	<dbug.h>
#include	<locale.h>
#include	<nlsclass.h>
#include	<ctrlcodes.h>
#include	<pslcodes.h>
#include	<sysdep.h>

public int	lang0_table[256][2]; /* a.k.a. asc_table   */
public int	lang1_table[256][2]; /* a.k.a. lang2_table */
public int	lang3_table[256][2]; /* a.k.a. lang4_table */

struct chlist {
	int ch;
	int defs;
	int mapto;
};

private struct chlist portchars[] = {
	'\0', DEFCNTRL,							'\0',
#if TEST_ANSI
	'\a', DEFCNTRL,							'\a',
#endif
	'\b', DEFCNTRL,							'\b',
	'\f', DEFCNTRL|DEFSPACE,					'\f',
	'\n', DEFCNTRL|DEFSPACE,					'\n',
	'\r', DEFCNTRL|DEFSPACE,					'\r',
	'\t', DEFCNTRL|DEFSPACE,					'\t',
	'\v', DEFCNTRL|DEFSPACE,					'\v',
	' ',  DEFSPACE|DEFPRINT,					' ',
	'!',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'!',
	'"',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'"',
	'#',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'#',
	'$',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'$',
	'%',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'%',
	'&',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'&',
	'\'', DEFPUNCT|DEFPRINT|DEFGRAPH, 				'\'',
	'(',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'(',
	')',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				')',
	'*',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'*',
	'+',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'+',
	',',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				',',
	'-',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'-',
	'.',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'.',
	'/',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'/',
	'0',  DEFDIGIT|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 		'0',
	'1',  DEFDIGIT|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 		'1',
	'2',  DEFDIGIT|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 		'2',
	'3',  DEFDIGIT|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 		'3',
	'4',  DEFDIGIT|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 		'4',
	'5',  DEFDIGIT|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 		'5',
	'6',  DEFDIGIT|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 		'6',
	'7',  DEFDIGIT|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 		'7',
	'8',  DEFDIGIT|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 		'8',
	'9',  DEFDIGIT|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 		'9',
	':',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				':',
	';',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				';',
	'<',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'<',
	'=',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'=',
	'>',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'>',
	'?',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'?',
	'@',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'@',
	'A',  DEFALPHA|DEFUPPER|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 	'a',
	'B',  DEFALPHA|DEFUPPER|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 	'b',
	'C',  DEFALPHA|DEFUPPER|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 	'c',
	'D',  DEFALPHA|DEFUPPER|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 	'd',
	'E',  DEFALPHA|DEFUPPER|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 	'e',
	'F',  DEFALPHA|DEFUPPER|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 	'f',
	'G',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'g',
	'H',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'h',
	'I',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'i',
	'J',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'j',
	'K',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'k',
	'L',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'l',
	'M',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'm',
	'N',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'n',
	'O',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'o',
	'P',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'p',
	'Q',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'q',
	'R',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'r',
	'S',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		's',
	'T',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		't',
	'U',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'u',
	'V',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'v',
	'W',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'w',
	'X',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'x',
	'Y',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'y',
	'Z',  DEFALPHA|DEFUPPER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'z',
	'[',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'[',
	'\\', DEFPUNCT|DEFPRINT|DEFGRAPH, 				'\\',
	']',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				']',
	'^',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'^',
	'_',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'_',
	'`',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'`',
	'a',  DEFALPHA|DEFLOWER|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 	'A',
	'b',  DEFALPHA|DEFLOWER|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 	'B',
	'c',  DEFALPHA|DEFLOWER|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 	'C',
	'd',  DEFALPHA|DEFLOWER|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 	'D',
	'e',  DEFALPHA|DEFLOWER|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 	'E',
	'f',  DEFALPHA|DEFLOWER|DEFXDIGIT|DEFALNUM|DEFPRINT|DEFGRAPH, 	'F',
	'g',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'G',
	'h',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'H',
	'i',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'I',
	'j',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'J',
	'k',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'K',
	'l',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'L',
	'm',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'M',
	'n',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'N',
	'o',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'O',
	'p',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'P',
	'q',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'Q',
	'r',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'R',
	's',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'S',
	't',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'T',
	'u',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'U',
	'v',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'V',
	'w',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'W',
	'x',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'X',
	'y',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'Y',
	'z',  DEFALPHA|DEFLOWER|DEFALNUM|DEFPRINT|DEFGRAPH, 		'Z',
	'{',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'{',
	'|',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'|',
	'}',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'}',
	'~',  DEFPUNCT|DEFPRINT|DEFGRAPH, 				'~',
};

private int	lang0_ctrl[] = {
	PSL0_ACK,  PSL0_BEL,  PSL0_BS,   PSL0_CAN,  PSL0_CR,   PSL0_DC1,
	PSL0_DC2,  PSL0_DC3,  PSL0_DC4,  PSL0_DEL,  PSL0_DLE,  PSL0_EM,
	PSL0_ENQ,  PSL0_EOT,  PSL0_ESC,  PSL0_ETB,  PSL0_ETX,  PSL0_FF,
	PSL0_FS,   PSL0_GS,   PSL0_HT,   PSL0_IS4,  PSL0_IS3,  PSL0_IS2,
	PSL0_IS1,  PSL0_LF,   PSL0_NAK,  PSL0_RS,   PSL0_SI,   PSL0_SO,
	PSL0_SOH,  PSL0_STX,  PSL0_SUB,  PSL0_SYN,  PSL0_US,   PSL0_VT,
};

private int	lang14_ctrl[] = {
	PSL14_ACK, PSL14_BEL, PSL14_BS,  PSL14_CAN, PSL14_CR,  PSL14_DC1,
	PSL14_DC2, PSL14_DC3, PSL14_DC4, PSL14_DEL, PSL14_DLE, PSL14_EM,
	PSL14_ENQ, PSL14_EOT, PSL14_ESC, PSL14_ETB, PSL14_ETX, PSL14_FF,
	PSL14_FS,  PSL14_GS,  PSL14_HT,  PSL14_IS4, PSL14_IS3, PSL14_IS2,
	PSL14_IS1, PSL14_LF,  PSL14_NAK, PSL14_RS,  PSL14_SI,  PSL14_SO,
	PSL14_SOH, PSL14_STX, PSL14_SUB, PSL14_SYN, PSL14_US,  PSL14_VT,
};

private struct chlist lang12_list[] = {
	/* NB: DEFPRINT and DEFGRAPH assumed */
	PSL12_A_umlaut, DEFUPPER|DEFALPHA|DEFALNUM, PSL12_a_umlaut,
	PSL12_O_umlaut, DEFUPPER|DEFALPHA|DEFALNUM, PSL12_o_umlaut,
	PSL12_U_umlaut, DEFUPPER|DEFALPHA|DEFALNUM, PSL12_u_umlaut,
	PSL12_cedilla,  DEFPUNCT, PSL12_cedilla,
	PSL12_a_umlaut, DEFLOWER|DEFALPHA|DEFALNUM, PSL12_A_umlaut,
	PSL12_o_umlaut, DEFLOWER|DEFALPHA|DEFALNUM, PSL12_O_umlaut,
	PSL12_u_umlaut, DEFLOWER|DEFALPHA|DEFALNUM, PSL12_U_umlaut,
	PSL12_eszet,    DEFLOWER|DEFALPHA|DEFALNUM, PSL12_eszet,
};

private struct chlist lang34_list[] = {
	/* NB: DEFALPHA, DEFALNUM, DEFPRINT and DEFGRAPH assumed */
	PSL34_A_grave,      DEFUPPER, PSL34_a_grave,
	PSL34_A_acute,      DEFUPPER, PSL34_a_acute,
	PSL34_A_circumflex, DEFUPPER, PSL34_a_circumflex,
	PSL34_A_tilde,      DEFUPPER, PSL34_a_tilde,
	PSL34_A_umlaut,     DEFUPPER, PSL34_a_umlaut,
	PSL34_A_ring,       DEFUPPER, PSL34_a_ring,
	PSL34_C_cedilla,    DEFUPPER, PSL34_c_cedilla,
	PSL34_E_grave,      DEFUPPER, PSL34_e_grave,
	PSL34_E_acute,      DEFUPPER, PSL34_e_acute,
	PSL34_E_circumflex, DEFUPPER, PSL34_e_circumflex,
	PSL34_E_umlaut,     DEFUPPER, PSL34_e_umlaut,
	PSL34_I_grave,      DEFUPPER, PSL34_i_grave,
	PSL34_I_acute,      DEFUPPER, PSL34_i_acute,
	PSL34_I_circumflex, DEFUPPER, PSL34_i_circumflex,
	PSL34_I_umlaut,     DEFUPPER, PSL34_i_umlaut,
	PSL34_O_grave,      DEFUPPER, PSL34_o_grave,
	PSL34_O_acute,      DEFUPPER, PSL34_o_acute,
	PSL34_O_circumflex, DEFUPPER, PSL34_o_circumflex,
	PSL34_O_tilde,      DEFUPPER, PSL34_o_tilde,
	PSL34_O_umlaut,     DEFUPPER, PSL34_o_umlaut,
	PSL34_U_grave,      DEFUPPER, PSL34_u_grave,
	PSL34_U_acute,      DEFUPPER, PSL34_u_acute,
	PSL34_U_circumflex, DEFUPPER, PSL34_u_circumflex,
	PSL34_U_umlaut,     DEFUPPER, PSL34_u_umlaut,
	PSL34_Y_acute,      DEFUPPER, PSL34_y_acute,
	PSL34_eszet,        DEFLOWER, PSL34_eszet,
	PSL34_a_grave,      DEFLOWER, PSL34_A_grave,
	PSL34_a_acute,      DEFLOWER, PSL34_A_acute,
	PSL34_a_circumflex, DEFLOWER, PSL34_A_circumflex,
	PSL34_a_tilde,      DEFLOWER, PSL34_A_tilde,
	PSL34_a_umlaut,     DEFLOWER, PSL34_A_umlaut,
	PSL34_a_ring,       DEFLOWER, PSL34_A_ring,
	PSL34_c_cedilla,    DEFLOWER, PSL34_C_cedilla,
	PSL34_e_grave,      DEFLOWER, PSL34_E_grave,
	PSL34_e_acute,      DEFLOWER, PSL34_E_acute,
	PSL34_e_circumflex, DEFLOWER, PSL34_E_circumflex,
	PSL34_e_umlaut,     DEFLOWER, PSL34_E_umlaut,
	PSL34_i_grave,      DEFLOWER, PSL34_I_grave,
	PSL34_i_acute,      DEFLOWER, PSL34_I_acute,
	PSL34_i_circumflex, DEFLOWER, PSL34_I_circumflex,
	PSL34_i_umlaut,     DEFLOWER, PSL34_I_umlaut,
	PSL34_o_grave,      DEFLOWER, PSL34_O_grave,
	PSL34_o_acute,      DEFLOWER, PSL34_O_acute,
	PSL34_o_circumflex, DEFLOWER, PSL34_O_circumflex,
	PSL34_o_tilde,      DEFLOWER, PSL34_O_tilde,
	PSL34_o_umlaut,     DEFLOWER, PSL34_O_umlaut,
	PSL34_u_grave,      DEFLOWER, PSL34_U_grave,
	PSL34_u_acute,      DEFLOWER, PSL34_U_acute,
	PSL34_u_circumflex, DEFLOWER, PSL34_U_circumflex,
	PSL34_u_umlaut,     DEFLOWER, PSL34_U_umlaut,
	PSL34_y_acute,      DEFLOWER, PSL34_Y_acute,
};

public int
pslclass()
{
	int i, ch;

	DBUG_LENTER("pslclass", 2);

	/* Reset all tables */
	for (ch = 0; ch < 256; ch++)
	{
		lang0_table[ch][0] = 0;
		lang0_table[ch][1] = ch;
		lang1_table[ch][0] = 0;
		lang1_table[ch][1] = ch;
		lang3_table[ch][0] = 0;
		lang3_table[ch][1] = ch;
	}

	/* Set iscntrl for control characters */
	for (i = 0; i < sizeof(lang0_ctrl)/sizeof(lang0_ctrl[0]); i++)
	{
		ch = lang0_ctrl[i];
		if (ch < 0 || ch >= 256)
		{
			in_rpt("Invalid PSL0_xxx control character encoding in ctrlcodes.h: 0x%x", ch);
			DBUG_RETURN(-1);
		}
		lang0_table[ch][0] = DEFCNTRL;
	}
	for (i = 0; i < sizeof(lang14_ctrl)/sizeof(lang14_ctrl[0]); i++)
	{
		ch = lang14_ctrl[i];
		if (ch < 0 || ch >= 256)
		{
			in_rpt("Invalid PSL14_xxx control character encoding in ctrlcodes.h: 0x%x", ch);
			DBUG_RETURN(-1);
		}
		lang1_table[ch][0] = DEFCNTRL;
		lang3_table[ch][0] = DEFCNTRL;
	}

	/* Set known classes for the portable character set */
	for (i = 0; i < sizeof(portchars)/sizeof(portchars[0]); i++)
	{
		ch = portchars[i].ch;
		if (ch < 0 || ch >= 256)
		{
			in_rpt("Portable character set encodings are not in the range 0-255 !!");
			DBUG_RETURN(-1);
		}
		lang0_table[ch][0] = portchars[i].defs;
		lang0_table[ch][1] = portchars[i].mapto;
		lang1_table[ch][0] = portchars[i].defs;
		lang1_table[ch][1] = portchars[i].mapto;
		lang3_table[ch][0] = portchars[i].defs;
		lang3_table[ch][1] = portchars[i].mapto;
	}

	/* Set classes (other than cntrl) for extra PSL characters */
	for (i = 0; i < sizeof(lang12_list)/sizeof(lang12_list[0]); i++)
	{
		ch = lang12_list[i].ch;
		if (ch < 0 || ch >= 256)
		{
			in_rpt("Invalid PSL12_xxx character encoding in pslcodes.h: 0x%x", ch);
			DBUG_RETURN(-1);
		}
		lang1_table[ch][0] = lang12_list[i].defs | DEFPRINT | DEFGRAPH;
		lang1_table[ch][1] = lang12_list[i].mapto;
	}
	for (i = 0; i < sizeof(lang34_list)/sizeof(lang34_list[0]); i++)
	{
		ch = lang34_list[i].ch;
		if (ch < 0 || ch >= 256)
		{
			in_rpt("Invalid PSL34_xxx character encoding in pslcodes.h: 0x%x", ch);
			DBUG_RETURN(-1);
		}
		lang3_table[ch][0] = lang34_list[i].defs |
			DEFALPHA | DEFALNUM | DEFPRINT | DEFGRAPH;
		lang3_table[ch][1] = lang34_list[i].mapto;
	}

	/* Set isascii for chars 0-127 */
	for (ch = 0; ch < 128; ch++)
	{
		lang0_table[ch][0] |= DEFASCII;
		lang1_table[ch][0] |= DEFASCII;
		lang3_table[ch][0] |= DEFASCII;
	}

	DBUG_RETURN(0);
}
