/*
*      SCCS:   @(#)common/genlib(pslocale.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/****************************************************************************

NAME:  		pslocale() - get VSX Pseudo-language information
		
MODULE CONTAINS:
       	pslocale()
SYNOPSIS:
	struct	lconv	*pslocale(langnum)
		int langnum;

DESCRIPTION:
	Pslocale() returns a pointer to a struct lconv containing the
	requested lconv data for the specified VSX pseudo-language.

AUTHOR:		J.A. Nave
DATE CREATED:	Thu Dec 13 12:13:35 GMT 1990

MODIFICATIONS:
		Stuart Boutell, 25 April 1991
			Recoded so that each member is set individually.
		
		Geoff Clare, 21 Aug 1992
			Change n_cs_precedes from 1 to 0 for VSX4L2/L3@dict.
		
		Geoff Clare, 31 March 1995
			New character encodings for VSX4L1 and VSX4L2.
			Use symbolic constants for language numbers.

****************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<nlstest.h>
#include	<locale.h>
#include	<limits.h>
#include	<sysdep.h>

#if TEST_XPG > 3

#define NUMLANG		6

static struct	lconv	pslconvinfo[NUMLANG];

private	int	pinit = 0;

private	void
pslc_init()
{
	DBUG_LENTER("pslc_init",3);

	pslconvinfo[VALCLANG].decimal_point	= ".";
	pslconvinfo[VALCLANG].thousands_sep	= "";
	pslconvinfo[VALCLANG].grouping		= "";
	pslconvinfo[VALCLANG].int_curr_symbol	= "";
	pslconvinfo[VALCLANG].currency_symbol	= "";
	pslconvinfo[VALCLANG].mon_decimal_point = "";
	pslconvinfo[VALCLANG].mon_thousands_sep = "";
	pslconvinfo[VALCLANG].mon_grouping	= "";
	pslconvinfo[VALCLANG].positive_sign	= "";
	pslconvinfo[VALCLANG].negative_sign	= "";
	pslconvinfo[VALCLANG].int_frac_digits	= CHAR_MAX;
	pslconvinfo[VALCLANG].frac_digits	= CHAR_MAX;
	pslconvinfo[VALCLANG].p_cs_precedes	= CHAR_MAX;
	pslconvinfo[VALCLANG].p_sep_by_space	= CHAR_MAX;
	pslconvinfo[VALCLANG].n_cs_precedes	= CHAR_MAX;
	pslconvinfo[VALCLANG].n_sep_by_space	= CHAR_MAX;
	pslconvinfo[VALCLANG].p_sign_posn	= CHAR_MAX;
	pslconvinfo[VALCLANG].n_sign_posn	= CHAR_MAX;

	pslconvinfo[VALNOINFOLANG] = pslconvinfo[VALCLANG];

	pslconvinfo[VALINFO1LANG].decimal_point	=	",";
	pslconvinfo[VALINFO1LANG].thousands_sep	=	".";
	pslconvinfo[VALINFO1LANG].grouping	=	"";
	pslconvinfo[VALINFO1LANG].int_curr_symbol =	"DDM ";
	pslconvinfo[VALINFO1LANG].currency_symbol =	"Dm";
	pslconvinfo[VALINFO1LANG].mon_decimal_point =	"^";
	pslconvinfo[VALINFO1LANG].mon_thousands_sep =	" ";
	pslconvinfo[VALINFO1LANG].mon_grouping	=	"\3";
	pslconvinfo[VALINFO1LANG].positive_sign	=	"";
	pslconvinfo[VALINFO1LANG].negative_sign	=	"C";
	pslconvinfo[VALINFO1LANG].int_frac_digits =	2;
	pslconvinfo[VALINFO1LANG].frac_digits	=	3;
	pslconvinfo[VALINFO1LANG].p_cs_precedes	=	1;
	pslconvinfo[VALINFO1LANG].p_sep_by_space =	0;
	pslconvinfo[VALINFO1LANG].n_cs_precedes	=	1;
	pslconvinfo[VALINFO1LANG].n_sep_by_space =	0;
	pslconvinfo[VALINFO1LANG].p_sign_posn	=	3;
	pslconvinfo[VALINFO1LANG].n_sign_posn	=	3;

	pslconvinfo[VALINFO2LANG].decimal_point	=	"I";
	pslconvinfo[VALINFO2LANG].thousands_sep	=	",";
	pslconvinfo[VALINFO2LANG].grouping	=	"\1\2\3\4";
	pslconvinfo[VALINFO2LANG].int_curr_symbol =	"FRF ";
	pslconvinfo[VALINFO2LANG].currency_symbol =	"Fr";
	pslconvinfo[VALINFO2LANG].mon_decimal_point =	"*";
	pslconvinfo[VALINFO2LANG].mon_thousands_sep =	"-";
	pslconvinfo[VALINFO2LANG].mon_grouping	=	"\4";
	pslconvinfo[VALINFO2LANG].positive_sign	=	"+";
	pslconvinfo[VALINFO2LANG].negative_sign	=	"-";
	pslconvinfo[VALINFO2LANG].int_frac_digits =	1;
	pslconvinfo[VALINFO2LANG].frac_digits	=	2;
	pslconvinfo[VALINFO2LANG].p_cs_precedes	=	0;
	pslconvinfo[VALINFO2LANG].p_sep_by_space =	0;
	pslconvinfo[VALINFO2LANG].n_cs_precedes	=	0;
	pslconvinfo[VALINFO2LANG].n_sep_by_space =	1;
	pslconvinfo[VALINFO2LANG].p_sign_posn	=	4;
	pslconvinfo[VALINFO2LANG].n_sign_posn	=	2;

	pslconvinfo[VALINFO3LANG] = pslconvinfo[VALINFO1LANG];

	pslconvinfo[VALINFO4LANG] = pslconvinfo[VALINFO2LANG];

	pinit++;

	DBUG_VOID_RETURN;
}


public struct	lconv	*
pslocale(langnum)
int langnum;
{
	DBUG_LENTER("pslocale", 2);
	DBUG_PRINTF("args", "pslocale(%d)", langnum);

	if(!pinit)
	{
		DBUG_PRINTF("pslocale","Initialising pslconvinfo");
		pslc_init();
	}

	/* If unknown language return info for "C" locale */
	if (langnum < 0 || langnum >= NUMLANG)
		langnum = 0;

	DBUG_RETURN(&pslconvinfo[langnum]);
}

#else /* TEST_XPG > 3 */

static int dummy;

#endif /* TEST_XPG > 3 */
