/*
*      SCCS:   @(#)common/genlib/uids.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		uids.c - testset uid/gid routines

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		uids()
		vguid0, vguid1, vguid2,
		vggid0, vggid1, vggid2,

SYNOPSIS:
		int	uids(void);
		int	vguid0, vguid1, vguid2;
		int	vggid0, vggid1, vggid2;

DESCRIPTION:

	Uids() sets the global variables vguid0, vggid0, etc, from the
	config parameters VSX_UID0, VSX_GID0, etc.

	Uids() returns -1 and reports via tet_infoline() if one or more
	parameters are not set, otherwise it returns 0.

SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	12 Nov 1990
MODIFICATIONS:
		Geoff Clare, 21 Sept 1992
			Change default uid/gid values from -2 since
			this value is significant for NFS.

************************************************************************/

#include	<std.h>
#include	<string.h>	
#include	<dbug.h>
#include	<tet_api.h>
#include	<sysdep.h>


int	vguid0 = -999;
int	vguid1 = -999;
int	vguid2 = -999;

int	vggid0 = -999;
int	vggid1 = -999;
int	vggid2 = -999;

private int
set(varp, param)
int *varp;
char *param;
{
	char	*cp;
	char	buf[40];

	if ((cp = tet_getvar(param)) == NULL || *cp == '\0')
	{
		(void) strcpy(buf, "parameter not set: ");
		(void) strcat(buf, param);
		tet_infoline(buf);
		return 1;
	}

	*varp = atoi(cp);
	return 0;
}

int
uids()
{
	int ret = 0;

	DBUG_LENTER("uids", 2);

	ret += set(&vguid0, "VSX_UID0");
	ret += set(&vguid1, "VSX_UID1");
	ret += set(&vguid2, "VSX_UID2");
	ret += set(&vggid0, "VSX_GID0");
	ret += set(&vggid1, "VSX_GID1");
	ret += set(&vggid2, "VSX_GID2");

	if (ret > 0)
		ret = -1;

	DBUG_PRINTF("return", "uids = %d", ret);
	DBUG_RETURN(ret);
}
