/*
*      SCCS:   @(#)common/genlib/uidtoa.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		uidtoa.c - uid_t to "ascii" routine.

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	uidtoa()
SYNOPSIS:
	char *uidtoa(num)
		uid_t num;
DESCRIPTION:
	uidtoa() returns a pointer to a static character array which
	holds a printable representation of num.

SOURCE:		
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	8 Jan 1993
MODIFICATIONS:
		Geoff Clare, 21 Nov 1996
		New buffer size calculation.
		Avoid overflow when negating num.

************************************************************************/

#include	<std.h>
#include	<sys/types.h>
#include	<stdio.h>
#include	<limits.h>
#include	<dbug.h>
#include	<sysdep.h>

public char *
uidtoa(num)
uid_t num;
{
	static char nbuf[2 + (sizeof(num)*CHAR_BIT*10 + 32)/33];
	char *p;
	unsigned long magn;
	int minus = 0;

	DBUG_LENTER("uidtoa", 2);
	DBUG_PRINTF("args", "uidtoa(%ld)", (long)num);

	p = &nbuf[sizeof(nbuf)];
	*--p = '\0';

	if (num == 0)
		*--p = '0';
	else
	{
		if (num < 0)
		{
			minus = 1;
			/* cast needed because -num can overflow */
			magn = -(unsigned long)num;
		}
		else
			magn = num;

		while (magn != 0)
		{
			*--p = "0123456789"[magn % 10];
			magn /= 10;
		}

		if (minus)
			*--p = '-';
	}

	DBUG_PRINTF("return", "uidtoa = %s", p);
	DBUG_RETURN(p);
}
