/*
*      SCCS:   @(#)common/genlib/vxmkdir.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		vxmkdir.c - make a directory

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	vxmkdir()
SYNOPSIS:
	int vxmkdir(path)
		char *path;

DESCRIPTION:
	Vxmkdir() attempts to make the directory "path".
	If the directory already exists then no error is returned.
	If "path" exists and is a regular file, vxmkdir will attempt
	to remove it before making the directory.

CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
	Errno will be changed, file "path" may be removed if it is
	an ordinary file.
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Thu Aug  7 15:33:40 PDT 1986
MODIFICATIONS:
		Geoff Clare, 12 Oct 1988
			Changed to use mkdir(2) which is now provided in 
			the vport library when necessary.

************************************************************************/

#include	<std.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<sysdep.h>

#define MODEANY	(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

public int
vxmkdir(path)
	char *path;
{
	int rval, err;
	struct stat statb;

	DBUG_LENTER("vxmkdir", 2);
	DBUG_PRINTF("args", "vxmkdir(%s)", path);

	if (path == (char *) NULL)
	{
		DBUG_PRINTF("return", "vxmkdir = %d", FAIL);
		DBUG_RETURN((errno = EFAULT, FAIL));
	}

	if (stat(path, &statb) != SYSERROR)
	{
		if (S_ISDIR(statb.st_mode))
		{
			if (eaccess(path, W_OK | X_OK) == 0)
			{
				DBUG_PRINTF("return", "vxmkdir = %d", SUCCEED);
				DBUG_RETURN(SUCCEED);
			}
		}
		else
			(void) unlink(path);
	}

	rval = mkdir(path, MODEANY);
	err = errno;

	DBUG_PRINTF("return", "vxmkdir = %d", rval);
	DBUG_RETURN((errno = err, rval));
}
