/*
*      SCCS:   @(#)common/genlib(xx_all.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		xx_all - failure report for all tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	xx_all()
SYNOPSIS:
	void xx_all(result, reason)
		char result;
		char *reason;

DESCRIPTION:
	xx_all() specifies a failure result for all tests with a reason
	string.  When each test is executed it will call xx_rpt() with
	the specified result code and in_rpt() with the reason string.
	The behaviour is much the same as calling tet_delete() for all
	tests except that the result code can be specified (tet_delete()
	leads to an UNINITIATED result).  xx_all() is normally used
	from setup functions, but can also be used in a test function
	to cause all remaining tests to report the specified result.

	The reason argument must point to static data.

SIDE EFFECTS:
	xx_all() modifies the tet_testlist[] array to cause its own
	reporting function to be called in place of the original
	test function.

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	15 Jan 1992
MODIFICATIONS:
		Geoff Clare, 2 Feb 1993
			Use tet_delete() where appropriate to avoid
			sync errors from the other system.

************************************************************************/

#include <std.h>
#include <stdlib.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>

private char	xx_result = 0;
private char	*xx_reason = NULL;

extern struct tet_testlist tet_testlist[];

private void
xx_func()
{
	DBUG_LENTER("xx_func", 2);

	if (xx_result != 0)
		xx_rpt(xx_result);
	if (xx_reason != NULL)
		in_rpt("%s", xx_reason);

	DBUG_VOID_RETURN;
}

public void
xx_all(result, reason)
char result;
char *reason;
{
	struct tet_testlist *tp;
	int i;

	DBUG_LENTER("xx_all", 2);
	DBUG_PRINTF("args", "xx_all(%c, %s)", result, reason);

	if (result == DELETION || result == CANCELLED)
	{
		/* OK to use tet_delete() */

		for (i=1, tp = tet_testlist; tp->testfunc != NULL; tp++, i++)
			tet_delete(i, reason);
	}
	else
	{
		/* Can't use tet_delete() */
		xx_result = result;
		xx_reason = reason;

		for (tp = tet_testlist; tp->testfunc != NULL; tp++)
			tp->testfunc = xx_func;
	}

	DBUG_VOID_RETURN;
}
