/*
*      SCCS:   @(#)common/purefile/purefile.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

static	char sccsid[] = "@(#)common/purefile VSXgen release 1.4";
			/* SEE ALSO pr_sccs(), below */
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};

/***********************************************************************

NAME:		purefile.c - A pure file to be used with testset programs

PROJECT:	VSX (X/OPEN Validation Suite)
MODULE CONTAINS:
		main()
SYNOPSIS:
		purefile fd timeout [-Doptions]

		 or
		
		purefile
		
		main(argc, argv)
			int	argc;
			char	**argv;
DESCRIPTION:
	Purefile is exec()'ed by testset programs to test operations on
	busy pure files (eg: test if open() for writting fails with
	ETXTBUSY, etc).   The argument fd is assumed to be a valid
	file descriptor opened for writing and is used for syncronization.
	The timeout option is that number of seconds to wait before exiting
	if test parent process does not respond.  The optional -Doptions
	argument is for the debugging package.

RELATED DOCUMENTS:
AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Mon Oct  6 12:17:20 BST 1986
MODIFICATIONS:

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<errno.h>
#include	<string.h>
#include	<sysdep.h>

#define	WAITTIME	(SPEEDFACTOR * 10)
/*
 *	Note: If this value is changed, then the value in
 *	tsetlib/pure_run.c should also be changed to the
 *	same value
 */
#define	SYNCSIZE	4

extern	char *		errname();
extern	void		set_pname();
extern	int		sendsync();


public	char		*vg_file = (char *) NULL;
public	char		*vg_dbug = (char *) NULL;

public	int
main(argc, argv)
	int	argc;
	char	*argv[];
{
	int	fd;
	int	timeout = 0;
	char	syncbuf[SYNCSIZE];

	DBUG_ENTER("main(purefile)");
	if (argc >= 1 && argv[0][0] != NUL)
		set_pname(vg_file = argv[0]);
	else
		set_pname(vg_file = "purefile");

	/*
	 *	If not called by exec testset, then tell the parent
	 *	that we are running
	 */
	if (argc != 1)
	{
		if (argc != 3 && argc != 4)
		{
			vsxerror("bad argument count: %d, should be 3 (or 4)\n",
				argc);
			exit(FAIL);
		}
		fd = atoi(argv[1]);
		timeout = atoi(argv[2]);
		if (timeout <= 0)
			timeout = WAITTIME;

		if (argc == 4 && strncmp(argv[3], "-D", 2) == 0)
			DBUG_PUSH(vg_dbug = &argv[3][2]);

		if (vg_dbug != (char *) NULL)
			DBUG_PRINTF("purefile", "fd=%d, timeout=%d, vg_dug=%s",
				fd, timeout, vg_dbug);

		/* tell the parent that we are running */
		if (sendsync(fd, syncbuf, sizeof(syncbuf), (unsigned)timeout)
								== SYSERROR)
		{
			if (errno == EINTR)
				vsxerror("cannot syncronize with parent");
			else
				vsxerror("write(%d, ...) failed: errno = %d(%s)",
					fd, errno, errname(errno));
			exit(FAIL);
		}
	}

	(void) sleep((unsigned) timeout);
	exit(0);

	/* this for lint */
	DBUG_RETURN(0);
}
