/*
*      SCCS:   @(#)common/tsetlib(atocp.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		atocp.c - ascii string to (char *) library routine

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	atocp()
SYNOPSIS:
	char *atocp(str)
		char *str;
DESCRIPTION:
	Atocp() takes a pointer to an ascii string as its argument, and
	converts the Hex, Octal, or Decimal number represented by that
	string to a character pointer - i.e.
		cp = atocp("0xABCD")
	places the address value ABCD(hex) into cp.

	Hex strings start with "0x" or "0X", and can be upper or lower
	case (or mixed).

	Octal strings start with "0"; Decimal with any digit 1-9.

	The string to be converted MUST start at *str - there is no
	whitespace/garbage skipping.

	Atocp stops as soon as it sees a non Hex/Oct/Dec character.

	Strings containing garbage are bound to cause amusement; no
	written or implied guarantee is expressed for this misuse.

	If "str" is (char *) NULL, then the value NULL is returned.
NOTE:
	This routine may not be totally portable; on those machines
	where sizeof(long) < sizeof(char *), and where str pts at a
	string containing a valid address that is larger than MAXLONG,
	there will be (insoluble) problems.
	Also will not work on non-ASCII machines.

	Typedef "ltype" below must be set to "unsigned long" if at all
	possible; otherwise "long" will do, with half the precision...

	Routines below spell it all out in baroque glory (when a few
	clever one-liners and combined functions might have done the trick)
	for portability.

AUTHOR:		Hamish Reid, UniSoft Ltd.
DATE CREATED:	Tue Jun 24 16:12:55 BST 1986
MODIFICATIONS:

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifndef strtol
extern long strtol();
#endif

char	*
atocp(str)
	register char	*str;
{
	char	*rval;

	DBUG_LENTER("atocp", 2);
	DBUG_PRINTF("args", "atocp(%s)", str);

	if (str == (char *) NULL)
	{
		DBUG_PRINTF("return", "atocp = NULL");
		DBUG_RETURN((char *) NULL);
	}

	if (*str == '0')
	{
		/* could be Hex or Octal	*/

		if ((str[1] == 'X') || (str[1] == 'x'))
		{
			rval = (char *) strtol(str, (char **) NULL, 16);
			DBUG_PRINTF("return", "atocp = 0x%lx",(long)rval);
			DBUG_RETURN (rval);
		}
		else
		{
			rval = (char *) strtol(str, (char **) NULL, 8);
			DBUG_PRINTF("return", "atocp = 0x%lx", (long)rval);
			DBUG_RETURN (rval);
		}
	}
	else
	{
		rval = (char *) strtol(str, (char **) NULL, 10);
		DBUG_PRINTF("return", "atocp = 0x%lx", (long)rval);
		DBUG_RETURN (rval);
	}
}
