/*
*      SCCS:   @(#)common/tsetlib(mlmathtype.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		mlmathtype.c - return math error name for math error type

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		mathtype()
SYNOPSIS:
		char	*mathtype(number)
			int	number;
DESCRIPTION:
	mathtype() returns a pointer to a static string containing the
	math error name (from <math.h>) corresponding to the math error type
	in "number"; if none exists, the string "UNKNOWN" is returned.
	Math error = 0 returns "NO ERROR".
	NOTE: this procedure only considers those error names and numbers
	defined in the XPG.
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:
AUTHOR:		Hamish Reid, UniSoft Ltd.
DATE CREATED:	Thu Jul	 3 17:32:38 BST 1986
MODIFICATIONS:
		Geoff Clare, 14 Oct 1992
			Changes for XNFS.

************************************************************************/

#include	<std.h>

#if !TEST_XNFS
#include	<math.h>
#include	<stdio.h>
#include	<dbug.h>
#include <tet_api.h>
#include	<sysdep.h>

char *
mathtype(number)
	int	number;
{
	static	struct	nament	{
				int	eno;
				char	*ename;
		}	name[] =
	{
		{ 0,		"NO ERROR" },
#ifdef	DOMAIN
		{ DOMAIN,	"DOMAIN" },
#endif
#ifdef	SING
		{ SING,		"SING" },
#endif
#ifdef	OVERFLOW
		{ OVERFLOW,	"OVERFLOW" },
#endif
#ifdef	UNDERFLOW
		{ UNDERFLOW,	"UNDERFLOW" },
#endif
#ifdef	TLOSS
		{ TLOSS,	"TLOSS" },
#endif
#ifdef	PLOSS
		{ PLOSS,	"PLOSS" },
#endif
	 };

	 register int	i;
	
	DBUG_LENTER("mlmathtype", 2);
	DBUG_PRINTF("args", "mlmathtype(%d)", number);
	for (i = 0; i < ((sizeof name)/(sizeof(struct nament))); i++)
	{
		if (name[i].eno == number)
		{
			DBUG_PRINTF("return", "mlmathtype = %s", name[i].ename);
			DBUG_RETURN( name[i].ename);
		}
	}
	DBUG_PRINTF("return", "mlmathtype = UNKNOWN");
	DBUG_RETURN ("UNKNOWN");
}
#else /* !TEST_XNFS */
static int dummy;
#endif /* !TEST_XNFS */
