/*
*      SCCS:   @(#)common/tsetlib(mlvalrpt.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		mlvalrpt.c - report observed and expected return & errno values

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		mlvalrpt()
SYNOPSIS:
		void mlvalrpt(exrval, orval, exerrno, oerrno)
			double	exrval;
			double	orval;
			int	exerrno;
			int	oerrno;
DESCRIPTION:
	Mlvalrpt() reports (using in_rpt()) the expected and observed
	return values and errnos from a maths library call, in the format:
		RETURN VALUES: expected: X, observed: Y
		ERRNO VALUES: expected: A (EEEEEE), observed: B (EEEEEE)
	where X is the value of exrval, Y the value of orval,
	A the value of exerrno, B the value of oerrno,
	C the value of extype, D the value of otype, EEEEE
	is the symbolic name for the expected errno, as determined by
	errname(), and FFFFFF is the symbolic name for the math error type
	as determined by mathtype(),

	If exrval == orval, no RETURN VALUES line is printed;
	if exerno == oerrno, no ERRNO VALUES line is printed;

	Mlvalrpt() should be used after xx_rpt() has been called to flag
	the failure/deletion/RFI that caused this information to be
	output, and before the next test is announced.
SIDE EFFECTS:
RELATED DOCUMENTS:
AUTHOR:		Hamish Reid, UniSoft Ltd.
DATE CREATED:	Mon Jun 23 15:04:11 BST 1986
MODIFICATIONS:
		Geoff Clare, 15 May 1992
			Check explicitly for NaN's.

		Geoff Clare, 14 Oct 1992
			Changes for XNFS.

************************************************************************/

#include	<std.h>

#if !TEST_XNFS
#include	<sys/types.h>
#include	<stdio.h>
#include	<dbug.h>
#include <tet_api.h>
#include	<sysdep.h>

extern	char	*errname();	/* from tsetlib */

void
mlvalrpt(exrval, orval, exerrno, oerrno)

	double	exrval;		/* expected rtn value */
	double	orval;		/* observed rtn value */
	int	exerrno;	/* expected errno */
	int	oerrno;		/* observed errno */
{
	char	repval[(4*sizeof(double))+1];
	char	*dblchar;
	int	i;

	if ( (exrval != orval && !isnan(exrval) && !isnan(orval)) ||
	     (isnan(exrval) && !isnan(orval)) ||
	     (!isnan(exrval) && isnan(orval)) )
	{
		in_rpt("RETURN VALUES: expected: %g, observed: %g",
			exrval, orval);
		dblchar = (char *) &exrval;
		for (i=0; i<sizeof(double); i++)
		{
			repval[4*i] = '\\';
			repval[(4*i)+1] = '0' + ((dblchar[i]&0300)/0100);
			repval[(4*i)+2] = '0' + ((dblchar[i]&0070)/0010);
			repval[(4*i)+3] = '0' + (dblchar[i]&0007);
		}
		repval[4*i] = '\0';
		in_rpt("    Bit Representation: expected value: %s", repval);
		dblchar = (char *) &orval;
		for (i=0; i<sizeof(double); i++)
		{
			repval[4*i] = '\\';
			repval[(4*i)+1] = '0' + ((dblchar[i]&0300)/0100);
			repval[(4*i)+2] = '0' + ((dblchar[i]&0070)/0010);
			repval[(4*i)+3] = '0' + (dblchar[i]&0007);
		}
		repval[4*i] = '\0';
		in_rpt("    Bit Representation: observed value: %s", repval);
	}

	if (exerrno != oerrno)
	{
		in_rpt("ERRNO VALUES: expected: %d (%s), observed: %d (%s)",
			exerrno, errname(exerrno), oerrno, errname(oerrno));
	}
}
#else /* !TEST_XNFS */
static int dummy;
#endif /* !TEST_XNFS */
