/*
*      SCCS:   @(#)common/tsetlib(mountfs.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		mountfs.c - mount a file system

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	mountfs()
	umntfs()

SYNOPSIS:

DESCRIPTION:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Mon Oct  6 13:54:37 BST 1986
MODIFICATIONS:
		Geoff Clare, 14 Oct 1988
			Use user-configurable interfaces from vport
			instead of calling mount/umount directly.

		Geoff Clare, 9 May 1990
			Remove junk files left from aborted ENOSPC tests

		David Sawyer
		Tue Jul 17 16:20:35 BST 1990
		Added resillience code.

                John-Paul, 23 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		Geoff Clare, 13 Oct 1992
			Added #include <xnfs.h>

		John-Paul, 3 December 1992
			Changed crdir() to vxmkdir() to circumvent problems
			with chown() over NFS.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<xnfs.h>
#include	<tet_api.h>
#include	<sysdep.h>

#define VSX_MOUNT_DEV	"VSX_MOUNT_DEV"



private	char	*mnt_dev = NULL;

int
mountfs(dirname, mode)
char *dirname;
mode_t   mode;
{
	char *cp;
	int	pathok = 0;

	DBUG_LENTER("mountfs", 2);
	DBUG_PRINTF("args", "mountfs(%s, %d)", dirname, mode);

	if (mnt_dev != NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: mountfs() called when device \"%s\" already mounted",
			mnt_dev);
		vsxerror("mountfs() must not be called twice without an intervening umntfs()");
		DBUG_PRINTF("return", "mountfs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if ((mnt_dev = tet_getvar(VSX_MOUNT_DEV)) == NULL ||
			*mnt_dev == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter %s is not set",
			VSX_MOUNT_DEV);
		DBUG_PRINTF("return", "mountfs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (setprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_MOUNT) failed");
		DBUG_PRINTF("return", "mountfs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (vxmkdir(dirname) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can't make directory %s", dirname);
		DBUG_PRINTF("return", "mountfs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (mnt_rw(mnt_dev, dirname) < 0)
	{
		(void) rmdir(dirname);
		xx_rpt(DELETION);
		in_rpt("deletion reason: mnt_rw(%s, %s) failed", mnt_dev, dirname);
		DBUG_PRINTF("return", "mountfs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	/* Remove any junk files left from aborted ENOSPC tests */

	cp = (char *)malloc((size_t)(strlen(dirname) + sizeof("/fill.tmpX")));
	if (cp != NULL)
	{
		(void) strcat(strcpy(cp, dirname), "/fill.tmp1");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.tmp2");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.tmp3");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.dir");
		(void) rmdir(cp);
		free((void *)cp);
	}

	PATH_FUNC_RPT(5);
	DBUG_PRINTF("return", "mountfs = 0");
	DBUG_RETURN(0);
}

int
umntfs(dirname)
char *dirname;
{
	DBUG_LENTER("umntfs", 2);
	DBUG_PRINTF("args", "umntfs(%s)", dirname);

	if (mnt_dev == NULL || dirname == NULL)
	{
		DBUG_PRINTF("return", "umntfs = 1");
		DBUG_RETURN(1);
	}

	(void) unmnt(mnt_dev, dirname);
	(void) rmdir(dirname);
	mnt_dev = NULL;

	DBUG_PRINTF("return", "umntfs = 0");
	DBUG_RETURN(0);
}
