/*
*      SCCS:   @(#)common/tsetlib(n_open.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		n_open.c - determine number of open file descriptors

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	n_open_fds()
SYNOPSIS:
	int n_open_fds();
	extern	int	globok;

DESCRIPTION:
AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Mon Sep  1 18:23:02 BST 1986
MODIFICATIONS:
		Geoff Clare, 1 Dec 1988
			Use sysconf(_SC_OPEN_MAX) instead of OPEN_MAX

		J.A.Nave	18/05/90
			Added resilinece testing
		
		Geoff Clare, 25 May 1990
			Allow for unlimited OPEN_MAX

		J.A.Nave	06/06/90
			Removed resilinece testing

************************************************************************/

#include	<std.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<stdio.h>
#include	<errno.h>
#include	<dbug.h>
#include	<pcts_max.h>
#include <tet_api.h>
#include	<sysdep.h>

public	int
n_open_fds()
{
	int		fd;
	int		open_max;
	int		count;
	struct stat	statb;

	DBUG_LENTER("n_open_fds", 2);

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0)
	{
		if (errno != 0)
			vsxerror("sysconf(_SC_OPEN_MAX) failed in n_open_fds()");
		open_max = 2 * PCTS_OPEN_MAX;
	}

	count = 0;
	for (fd = 0; fd < open_max; fd++)
	{
		if (fstat(fd, &statb) != SYSERROR)
			count++;
	}

	DBUG_PRINTF("return", "open_max = %d", open_max);
	DBUG_PRINTF("return", "n_open_fds = %d", count);
	DBUG_RETURN(count);
}
