/*
*      SCCS:   @(#)common/tsetlib(setupnospc.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		setupnospc.c - set up a full file system

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	setupnospc()
	tidynospc()

SYNOPSIS:
	int setupnospc(char *base)
	void tidynospc(char *base, int nfile)

DESCRIPTION:
	setupnospc mounts the filsystem given by VSX_NOSPC_DEV on directory
	base, and copies the file filler to fill.tmp[1-3].  It also creates
	the directory fill.dir.

	tidyupnospc() undoes the actions of setupnospc().  It also deletes
	nfile temporary files of the the form fill.tmpN

AUTHOR:		Steve Ratcliffe, Unisoft Ltd.
DATE CREATED:	Wed May 09 10:06:45 BST 1990
MODIFICATIONS:
		jan
		Resiliance.

		Anthony Lee, 11 July 1990
		Retrofitted resillience macros.

		Stuart Boutell, 3 Apr 1991
			Added mkdir return value check
		
		Geoff Clare, 8 May 1991
			Remove files from subdirectory in tidynospc().
		
		Geoff Clare, 30 Jan 1992
			Change mkdir to vxmkdir.

		Geoff Clare, 13 Oct 1992
			Added #include <xnfs.h>

		Dave Cater, 3 December 1992
			Don't use crfile() and crdir() because
			chown() can fail over NFS.
		
		Geoff Clare, 21 June 1994
			Change unlink() calls to remove() in tidynospc().
		
		Geoff Clare, 24 May 1996
			<std.h> must come before <dbug.h>

************************************************************************/

#include <std.h>
#include <dbug.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <report.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <setprv.h>
#include <xnfs.h>
#include <tet_api.h>
#include <sysdep.h>

extern	char	*itoa();
extern	char	*errname();

private int 	testfail;

int 	setupnospc();
void 	tidynospc();
private int 	mntspcfs();
private int 	unmntspcfs();

#define	BUFLEN	512

#define MODEANY	(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

int
setupnospc(base)
char	*base;
{
	char	*name;
	char	buf[BUFLEN];
	int 	fillerd;
	int 	err;
	int 	fd;
	int 	i, n;
	int	pathok = 0;

	DBUG_LENTER("setupnospc", 2);

	testfail = 0;
	globok = 0;
	if (mntspcfs(base))
	{
		DBUG_RETURN(1);
	}
	else
		PATH_FUNC_TRACE;

	name = (char*)malloc(strlen(base)+sizeof("/fill.tmp0000"));
	if (name == NULL)
	{
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("Out of memory");
		tidynospc(base, 0);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	/*
	 * First check that things have been set up by checking
	 * for the presence of the file filler.
	 * Then copy it to fill.tmp[1-3] this is supposed to fill up
	 * the file system.  It is not clear how the necessary conditions
	 * can be set up unless the ENOSPC error is returned correctly in the
	 * first place however.
	 */
	(void) strcat(strcpy(name, base), "/filler");

	if ((fillerd = open(name, O_RDONLY)) == SYSERROR)
	{
		if (errno == ENOENT)
		{
			if(!testfail++)
				xx_rpt(DELETION);
			in_rpt("File system not set up correctly for ENOSPC tests");
		}
		else
		{
			err = errno;
			if(!testfail++)
				xx_rpt(DELETION);
			in_rpt("Open failed on filler errno %s (%d)", errname(err), err);
		}
		tidynospc(base, 0);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	/*
	 * Create directory for use in the rmdir test
	 */
	if (vxmkdir(strcat(strcpy(name, base), "/fill.dir")) != 0)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("vxmkdir() failed in setupnospc - errno %s (%d)",
			errname(err), err);
		tidynospc(base,0);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	err = 0;
	for (i = 1; !testfail && i <=3; i++)
	{

		(void) strcat(strcpy(name, base), "/fill.tmp");
		(void) strcat(name, itoa(i));

		DBUG_PRINTF("setupnospc", "create %s", name);
		if ((fd = creat(name, MODEANY)) == SYSERROR)
		{
			if(!testfail++)
				xx_rpt(DELETION);
			in_rpt("creat failed in setupnospc");
			tidynospc(base, 3);
		}
		else
			PATH_TRACE;

		(void) lseek(fillerd, 0, SEEK_SET);

		while (!testfail && (n = read(fillerd, buf, BUFLEN)) > 0)
		{
			if (write(fd, buf, (unsigned)n) == SYSERROR)
			{
				if (errno == ENOSPC)
					break;
				err = errno;
				if(!testfail++)
					xx_rpt(DELETION);
				in_rpt("Write failed in setupnospc errno %s (%d)", errname(err), err);
				tidynospc(base, 3);
			}

		}

		if (!testfail)
			(void) close(fd);

	}

	globok = 0;
	PATH_FUNC_RPT(7);

	(void) close(fillerd);
	free((void*)name);

	DBUG_RETURN(testfail);

}

void
tidynospc(base, nfile)
char	*base;
{
	int 	i;
	char	*name;

	DBUG_LENTER("tidynospc", 2);

	name = (char*)malloc(strlen(base)+sizeof("/fill.dir/fill.tmp0000"));
	if (name == NULL)
	{
		vsxerror("Out of memory in tidynospc()");
		DBUG_VOID_RETURN;
	}

	/*
	 * Delete all files that (may or may not) have been created
	 */
	for (i = nfile; i > 0; i--)
	{
		(void) strcat(strcpy(name, base), "/fill.tmp");
		(void) strcat(name, itoa(i));
		(void) remove(name);

		(void) strcat(strcpy(name, base), "/fill.dir/fill.tmp");
		(void) strcat(name, itoa(i));
		(void) remove(name);
	}

	(void) rmdir(strcat(strcpy(name, base), "/fill.dir"));

	(void) unmntspcfs(base);
	free((void*)name);

	DBUG_VOID_RETURN;
}


char	*nospcdev;

#define	VSX_NOSPC_DEV	"VSX_NOSPC_DEV"

private int
mntspcfs(base)
char	*base;
{
	int	pathok = 0;

	DBUG_LENTER("mountenospfs", 2);

	if ((nospcdev = tet_getvar(VSX_NOSPC_DEV)) == NULL || *nospcdev == '\0')
	{
		if(!testfail++)
			xx_rpt(UNTESTED);
		in_rpt("parameter %s is not set", VSX_NOSPC_DEV);
		DBUG_PRINTF("return", "mntspcfs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (setprv(PRV_MOUNT) != 0)
	{
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_MOUNT) failed");
		DBUG_PRINTF("return", "mntspcfs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (vxmkdir(base) == SYSERROR)
	{
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: can't make directory %s", base);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (mnt_rw(nospcdev, base) < 0)
	{
		(void) unmnt(nospcdev, base);
		(void) rmdir(base);
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: mnt_rw(%s, %s) failed", nospcdev, base);
		DBUG_PRINTF("return", "mntspcfs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	globok = 0;
	PATH_FUNC_RPT(4); 

	DBUG_PRINTF("return", "mntspcfs = 0");
	DBUG_RETURN(0);
}

private int
unmntspcfs(base)
char *base;
{
	DBUG_LENTER("unmntspcfs", 2);
	DBUG_PRINTF("args", "unmntspcfs(%s)", base);

	if (nospcdev == (char *) 0)
	{
		DBUG_PRINTF("return", "unmntspcfs = 1");
		DBUG_RETURN(1);
	}

	if (unmnt(nospcdev, base) == SYSERROR)
	{
		DBUG_PRINTF("unmntspcfs", "unmnt failed errno %s", errname(errno));
	}

	(void) rmdir(base);
	DBUG_PRINTF("return", "unmntspcfs = 0");
	DBUG_RETURN(0);
}
