/*
*      SCCS:   @(#)common/tsetlib(setuprofs.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		setuprofs.c - set up a read only file system

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	setuprofs()

SYNOPSIS:

DESCRIPTION:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Mon Oct  6 13:54:37 BST 1986
MODIFICATIONS:
		Geoff Clare, 14 Oct 1988
			Use user-configurable interfaces from vport
			instead of calling mount/umount directly.
		
		Geoff Clare, 11 Apr 1989
			S_IFDIR set in "mode" argument indicates
			"filename" is to be created as a directory
			instead of a plain file.
		
		Geoff Clare, 15 Nov 1989
			Add filetype argument for use instead of S_IFDIR
		
		Geoff Clare, 9 May 1990
			Remove junk files left from aborted ENOSPC tests
		
		Geoff Clare, 7 June 1990
			UNSUPPORTED result if VSX_ROFS=unsup
		
		Geoff Clare, 8 Nov 1991
			Implement signal catcher for SIGTERM to clean
			up if timed out.

		John-Paul, 23 July 1992
			mode variables changed from 'int' to 'mode_t'.
		
		Geoff Clare, 13 Oct 1992
			Added #include <xnfs.h>

		John-Paul, 2 December 1992
			Don't use crfile() and crdir() because
			chown() can fail over NFS.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<signal.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<setprv.h>
#include	<xnfs.h>
#include	<tet_api.h>
#include	<sysdep.h>

#define VSX_ROFS	"VSX_ROFS"
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();
extern	void	(*vsignal())();

private	char	*rofs;
private	char	*tidydir;
private	char	*tidyfile;
private	void	(*oldhdlr)();

private void
sigtidy(sig)
int sig;
{
	(void) clnuprofs(tidydir, tidyfile);

	if (oldhdlr == SIG_DFL)
		_exit(255);
	else if (oldhdlr != SIG_IGN)
		(*oldhdlr)(sig);
}

int
setuprofs(dirname, filename, filetype, mode)
char *dirname, *filename;
int  filetype;
mode_t mode;
{
	int	fd;
	char	*cp;
	int	pathok = 0;
	int	err;

	DBUG_LENTER("setuprofs", 2);
	DBUG_PRINTF("args", "setuprofs(%s, %s, '%c', %#o)", dirname,
		(filename == NULL ? "NULL" : filename), filetype, mode);

	if ((rofs = tet_getvar(VSX_ROFS)) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter %s is not set", VSX_ROFS);
		DBUG_PRINTF("return", "setuprofs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;
	if (strncmp(rofs, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter %s is set to \"unsup\"", VSX_ROFS);
		DBUG_PRINTF("return", "setuprofs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (setprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_MOUNT) failed");
		DBUG_PRINTF("return", "setuprofs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	(void) rmdir(dirname);
	if (vxmkdir(dirname) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: can't make directory %s - errno %d (%s)", dirname, err, errname(err));
		DBUG_PRINTF("return", "setuprofs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (mnt_rw(rofs, dirname) < 0)
	{
		(void) rmdir(dirname);
		xx_rpt(DELETION);
		in_rpt("deletion reason: mnt_rw(%s, %s) failed", rofs, dirname);
		DBUG_PRINTF("return", "setuprofs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	/* Remove any junk files left from aborted ENOSPC tests */

	cp = (char *)malloc((size_t)(strlen(dirname) + sizeof("/fill.tmpX")));
	if (cp != NULL)
	{
		(void) strcat(strcpy(cp, dirname), "/fill.tmp1");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.tmp2");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.tmp3");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.dir");
		(void) rmdir(cp);
		free((void *)cp);
	}

	/* install signal handler to tidy up if timed out */
	tidydir = dirname;
	tidyfile = filename;
	oldhdlr = vsignal(SIGTERM, sigtidy);

	/* create requested file/directory */

	if (filename != NULL)
	{
		if (filetype == 'd')
		{
			(void) rmdir(filename);
			if (vxmkdir(filename) == SYSERROR)
			{
				err = errno;
				(void) unmnt(rofs, dirname);
				(void) rmdir(dirname);
				xx_rpt(DELETION);
				in_rpt("deletion reason: can't create directory %s - errno %d (%s)", filename, err, errname(err));
				DBUG_PRINTF("return", "setuprofs = 1");
				DBUG_RETURN(1);
			}
			else
				PATH_TRACE;
		}
		else
		{
			(void) unlink(filename);
			if ((fd = creat(filename, mode)) == SYSERROR)
			{
				err = errno;
				(void) unmnt(rofs, dirname);
				(void) rmdir(dirname);
				xx_rpt(DELETION);
				in_rpt("deletion reason: can't create file %s - errno %d (%s)", filename, err, errname(err));
				DBUG_PRINTF("return", "setuprofs = 1");
				DBUG_RETURN(1);
			}
			else
				PATH_TRACE;
			(void) close (fd);
		}
	}
	else
		PATH_TRACE;

	if (unmnt(rofs, dirname) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: unmnt(%s, %s) failed", rofs, dirname);
		if (filename != NULL)
			(void) remove(filename);
		(void) rmdir(dirname);
		DBUG_PRINTF("return", "setuprofs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (mnt_ro(rofs, dirname) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: mnt_ro(%s, %s) failed", rofs, dirname);
		(void) clnuprofs(dirname, filename);
		DBUG_PRINTF("return", "setuprofs = 1");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(8);

	DBUG_PRINTF("return", "setuprofs = 0");
	DBUG_RETURN(0);
}

int
clnuprofs(dirname, filename)
char *dirname, *filename;
{
	DBUG_LENTER("clnuprofs", 2);
	DBUG_PRINTF("args", "clnuprofs(%s, %s)", dirname, filename);

	if (oldhdlr != NULL)
		(void) vsignal(SIGTERM, SIG_IGN);

	if (rofs == (char *) 0 || dirname == (char *) 0)
	{
		DBUG_PRINTF("return", "clnuprofs = 1");
		DBUG_RETURN(1);
	}
	if (filename != NULL)
	{
		(void) unmnt(rofs, dirname);
		if (mnt_rw(rofs, dirname) < 0)
		{
			(void) rmdir(dirname);
			DBUG_PRINTF("return", "clnuprofs = 1");
			DBUG_RETURN(1);
		}
	
		(void) remove(filename);
	}

	(void) unmnt(rofs, dirname);
	(void) rmdir(dirname);

	if (oldhdlr != NULL)
		(void) vsignal(SIGTERM, oldhdlr);

	DBUG_PRINTF("return", "clnuprofs = 0");
	DBUG_RETURN(0);
}
