/*
*      SCCS:  @(#)common/tsetlib(stat_child.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		status_child.c - report child status

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:	sig_child() - 
		Return a null terminated string which contains the response
		of a process to a signal being generated.

		status_child() - 
		Return a null terminated string which contains the response
		of a process when no signal has necessarily been generated.

		The function wait_child() must have been called to set the 
		values of ex_code and sg_code, and stat_child must be the 
		value returned by wait_child() which must be either 
		CHLD_EXIT
		CHLD_ABRT
		or
		CHLD_STOP

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Fri Mar 10 11:51:53 GMT 1989
MODIFICATIONS:

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<sys/wait.h>
#include	<stdio.h>
#include	<errno.h>
#include	<wait_child.h>
#include <tet_api.h>
#include	<sysdep.h>

#define	BUFLEN	128		/* length of internal buffers */

extern	int	ex_code;	/* defined in wait_child.c */
extern	int	sg_code;	/* defined in wait_child.c */

public char *
sig_child(stat_child)
int	stat_child;	/* Status of child process */
{
	private char rcv_buf[BUFLEN];
	private char *str_receive;

	switch(stat_child) {
	case CHLD_STOP:	
		(void) sprintf(rcv_buf, "signal stopped process");
		break;
	case CHLD_EXIT:
		if(ex_code == 0)
			(void) sprintf(rcv_buf, "process ignored signal");
		else  {
			(void) sprintf(rcv_buf, "process caught signal %d", ex_code);
		}
		break;
	case CHLD_ABRT:
		(void) sprintf(rcv_buf, "signal %d terminated process", sg_code);
		break;
	default:
		(void) sprintf(rcv_buf, "internal error - bad child status");
		break;
	}
	str_receive = rcv_buf;
	return(str_receive);
}

public char *
status_child(stat_child)
int	stat_child;	/* Status of child process */
{
	private char rcv_buf[BUFLEN];
	private char *str_receive;

	switch(stat_child) {
	case CHLD_STOP:	
		(void) sprintf(rcv_buf, "signal stopped process");
		break;
	case CHLD_EXIT:
		(void) sprintf(rcv_buf, "process exitted - exit code %d", ex_code);
		break;
	case CHLD_ABRT:
		(void) sprintf(rcv_buf, "signal %d terminated process", sg_code);
		break;
	case CHLD_LIVE:
		(void) sprintf(rcv_buf, "process still active");
		break;
	default:
		(void) sprintf(rcv_buf, "internal error - bad child status");
		break;
	}
	str_receive = rcv_buf;
	return(str_receive);
}
