/*
*      SCCS:  @(#)common/tsetlib(statdevind.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/



/***********************************************************************

NAME:

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	statdevinod()
SYNOPSIS:
	int statdevinod(path, inode, dev)
		char	*path;
		ino_t	*inode;
		dev_t	*dev;


DESCRIPTION:
	statdevinod() attempts to stat "path".  If successful the
	inode and device number of "path" will be placed in "*inode"
	and *dev respectivly, and SUCCEED will be returned.  If the
	stat fails, then SYSERROR will be returned and errno will
	be set (*inode will not be modified).

	fstatdevinod() likewise attempts to stat file descriptor fdes.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
	Errno may be changed
RELATED DOCUMENTS:
SOURCE:
AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Thu Aug  7 11:27:01 PDT 1986
MODIFICATIONS:
	HR - Mon Aug 18 14:13:25 PDT 1986 - added fstatdevinod();

************************************************************************/
#include <std.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <dbug.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include <sysdep.h>

extern	char	*errname();

public int
statdevinod(path, inode, dev)
	char	*path;
	ino_t	*inode;
	dev_t	*dev;
{
	int	err;
	struct stat statb;

	DBUG_LENTER("statdevinod", 2);
	DBUG_PRINTF("args", "statdevinod(%s, %lu, %ld)", path, (unsigned long) inode, (long) dev);

	if (stat(path, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(%s) failed", path);
		in_rpt("               : errno %d (%s)", err, errname(err));
		DBUG_PRINTF("return", "statdevinod = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}
	*inode = statb.st_ino;
	*dev = statb.st_dev;
	DBUG_PRINTF("return", "statdevinod = %d", SUCCEED);
	DBUG_RETURN(SUCCEED);
}

public int
fstatdevinod(fdes, inode, dev)
	int	fdes;
	ino_t	*inode;
	dev_t	*dev;
{
	int	err;
	struct stat statb;

	DBUG_LENTER("fstatdevinod", 2);
	DBUG_PRINTF("args", "fstatdevinod(%d, %lu, %ld)", fdes, (unsigned long) inode, (long) dev);

	if (fstat(fdes, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat(%d) failed", fdes);
		in_rpt("               : errno %d (%s)", err, errname(err));
		DBUG_PRINTF("return", "fstatdevinod = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}
	*inode = statb.st_ino;
	*dev = statb.st_dev;
	DBUG_PRINTF("return", "fstatdevinod = %d", SUCCEED);
	DBUG_RETURN(SUCCEED);
}
