/*
*      SCCS:  @(#)common/tsetlib(statmode.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/



/***********************************************************************

NAME:		statmode.c

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	statmode()
	fstatmode()
SYNOPSIS:
	int statmode(path, mode)
		char	*path;
		mode_t	*mode;

	int fstatmode(fdes, mode)
		int	fdes;
		mode_t	*mode;
DESCRIPTION:
	statmode() attempts to stat "path".  If successful the
	mode of "path" will be placed in *mode and SUCCEED will be returned. 
	If the stat fails, then SYSERROR will be returned and errno will
	be set (*mode will not be modified).

	fstatmode() likewise attempts to stat file descriptor fdes.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
	Errno may be changed
RELATED DOCUMENTS:
SOURCE:
AUTHOR:		Hamish Reid, UniSoft Ltd.
DATE CREATED:	Tue Aug 19 10:20:04 PDT 1986
MODIFICATIONS:

************************************************************************/
#include <std.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include	<dbug.h>
#include <tet_api.h>
#include	<sysdep.h>

public int
statmode(path, mode)
	char	*path;
	mode_t	*mode;
{
	struct stat statb;

	DBUG_LENTER("statmode", 2);
	DBUG_PRINTF("args", "statmode(%s, %u)", path, *mode);

	if (stat(path, &statb) == SYSERROR)
	{
		DBUG_PRINTF("return", "statmode = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}
	*mode = statb.st_mode;
	DBUG_PRINTF("return", "statmode = %d", SUCCEED);
	DBUG_RETURN(SUCCEED);
}

public int
fstatmode(fdes, mode)
	int	fdes;
	mode_t	*mode;
{
	struct stat statb;

	DBUG_LENTER("fstatmode", 2);
	DBUG_PRINTF("args", "statmode(%d, %u)", fdes, *mode);

	if (fstat(fdes, &statb) == SYSERROR)
	{
		DBUG_PRINTF("return", "fstatmode = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}
	*mode = statb.st_mode;
	DBUG_PRINTF("return", "fstatmode = %d", SUCCEED);
	DBUG_RETURN(SUCCEED);
}
