/*
*      SCCS:  @(#)common/tsetlib(statugid.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/



/***********************************************************************

NAME:		statugid.c

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	statugid()
	fstatugid()
SYNOPSIS:
	int statugid(path, uid, gid)
		char	*path;
		uid_t	*uid;
		gid_t	*gid;

	int fstatugid(fdes, uid, gid)
		int	fdes;
		uid_t	*uid;
		gid_t	*gid;
DESCRIPTION:
	statmode() attempts to stat "path".  If successful the
	uid and gid of "path" will be placed in *uid and *gid
	and SUCCEED will be returned. 
	If the stat fails, then SYSERROR will be returned and errno will
	be set (*uid and *gid will not be modified).

	fstatugid() likewise attempts to stat file descriptor fdes.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
	Errno may be changed
RELATED DOCUMENTS:
SOURCE:		Any resemblance to statmode() is pure coincidence!
AUTHOR:		Manucher Ghezel-Ayagh, UniSoft Ltd.
DATE CREATED:	Fri Aug 22 13:20:10 BST 1986
MODIFICATIONS:

************************************************************************/
#include <std.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include	<dbug.h>
#include <tet_api.h>
#include <sysdep.h>


public int
statugid(path, uid, gid)
	char	*path;
	uid_t	*uid;
	gid_t	*gid;
{
	struct stat statb;

	DBUG_LENTER("statugid", 2);
	DBUG_PRINTF("args", "statugid(%s, %u, %u)", path, *uid, *gid);

	if (stat(path, &statb) == SYSERROR)
	{
		DBUG_PRINTF("return", "statugid = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}
	*uid = statb.st_uid;
	*gid = statb.st_gid;
	DBUG_PRINTF("return", "statugid = %d", SUCCEED);
	DBUG_RETURN(SUCCEED);
}

public int
fstatugid(fdes, uid, gid)
	int	fdes;
	uid_t	*uid;
	gid_t	*gid;
{
	struct stat statb;

	DBUG_LENTER("fstatugid", 2);
	DBUG_PRINTF("args", "fstatugid(%d, %u, %u)", fdes, *uid, *gid);

	if (fstat(fdes, &statb) == SYSERROR)
	{
		DBUG_PRINTF("return", "fstatugid = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}
	*uid = statb.st_uid;
	*gid = statb.st_gid;
	DBUG_PRINTF("return", "fstatugid = %d", SUCCEED);
	DBUG_RETURN(SUCCEED);
}
