/*
*      SCCS:   @(#)common/tsetlib(valrpt.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		valrpt.c - report observed and expected return and errno values

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		valrpt()
SYNOPSIS:
		void valrpt(exrval, orval, exerrno, oerrno)
			int	exrval;
			int	orval;
			int	exerrno;
			int	oerrno;
DESCRIPTION:
	Valrpt() reports (using in_rpt()) the expected and observed
	return values and errnos from a system call or other type of
	test call, in the format:
		RETURN VALUES: expected: X, observed: Y
		ERRNO VALUES: expected: A (EEEEEE), observed: B (EEEEEE)
	where X is the value of exrval, Y the value of orval,
	A the value of exerrno, B the value of oerrno, and EEEEE
	is the symbolic name for the expected errno, as determined by
	errname().

	If exrval == orval, no RETURN VALUES line is printed;
	similarly, if exerno = oerrno, no ERRNO VALUES line is printed.

	Valrpt() should be used after xx_rpt() has been called to flag
	the failure/deletion/RFI that caused this information to be
	output, and before the next test is announced.
SIDE EFFECTS:
RELATED DOCUMENTS:
AUTHOR:		Hamish Reid, UniSoft Ltd.
DATE CREATED:	Mon Jun 23 15:04:11 BST 1986
MODIFICATIONS:

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include <tet_api.h>
#include	<sysdep.h>

extern	char	*errname();	/* from tsetlib */

void
valrpt(exrval, orval, exerrno, oerrno)

	int	exrval;		/* expected rtn value */
	int	orval;		/* observed rtn value */
	int	exerrno;	/* expected errno */
	int	oerrno;		/* observed errno */
{
	if (exrval != orval)
	{
		in_rpt("RETURN VALUES: expected: %d, observed: %d",
			exrval, orval);
	}

	if (exerrno != oerrno)
	{
		in_rpt("ERRNO VALUES: expected: %d (%s), observed: %d (%s)",
			exerrno, errname(exerrno), oerrno, errname(oerrno));
	}
}
