/*
*      SCCS:   @(#)common/tsetlib(vsx_sigs.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		vsx_sigs.c - find signal values and save them

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	vsx_sigs()
SYNOPSIS:
	void	vsx_sigs()

DESCRIPTION:
	vsx_sigs() initialises sig_array[VSIGINVAL] to the value of
	VSX_INVAL_SIG parameter.  If this parameter is not set then
	sig_array[VSIGINVAL] is left as VNOSIG.
	It also fills in the entries in sig_type[] for X/Open signals.
	The default actions for these signals are hard-coded into the
	initialisation.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Aug 13 18:12:55 PDT 1986
MODIFICATIONS:
		Geoff Clare, 9 Mar 1989
			sig_array[] and sig_name[] moved to vsignal.c
			in huge signal rationalisation.
			Added sig_type[] initialisation.

		Anthony Lee, 11 July 1990
			Retrofitted resillience code.	

		Geoff Clare, 15 Jan 1997
			Update for XSH5.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <signal.h>
#include <vsx_signal.h>
#include <dbug.h>
#include <tet_api.h>
#include <sysdep.h>


public	int	sig_type[NSIG];

public	void
vsx_sigs()
{
	int i, signum;
	char	*sinval;
	int	pathok = 0;

	DBUG_LENTER("vsx_sigs", 2);

	globok = 0;

	if ((sinval = tet_getvar("VSX_INVAL_SIG")) != 0 && *sinval != '\0')
		sig_array[VSIGINVAL] = atoi(sinval);
	
	sig_type[0] = TYPE_IGNR;

	for (i = 1; i < VNUMSIG; i++)
	{
		PATH_TRACE;

		signum = sig_array[i];
		if (signum <= 0 || signum >= NSIG)
			continue;

		/* don't use switch: job control signals may not be unique */

		if (signum == SIGCHLD)
			sig_type[signum] = TYPE_IGNR;
		else if (signum == SIGCONT)
			sig_type[signum] = TYPE_CONT;
		else if (signum == SIGSTOP || signum == SIGTSTP ||
			 signum == SIGTTIN || signum == SIGTTOU)
			sig_type[signum] = TYPE_STOP;
#if TEST_XPG >= 500
		else if (signum == SIGURG)
			sig_type[signum] = TYPE_IGNR;
#endif
		else
			sig_type[signum] = TYPE_ABRT;
	}

	PATH_FUNC_RPT(VNUMSIG - 1);

	DBUG_VOID_RETURN;
}
