/*
*      SCCS:   @(#)common/tsetlib(wait_child.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME: wait_child.c - wait for a child process

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:	wait_child() returns a value to indicate the status of a 
		child process whose process number should be in "child".
		The value will be one of the following:
		CHLD_EXIT	Child process exitted.
				In this case, ex_code is the exit code of child.
		CHLD_ABRT	Child process terminated on receiving signal.
				In this case, sg_code is the signal number.
		CHLD_LIVE	The child process is alive - waitpid timed out.
		CHLD_STOP	The child process is stopped.
		CHLD_ERR	Error in waitpid().
		

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Fri Mar 10 11:51:53 GMT 1989
MODIFICATIONS:

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<signal.h>
#include	<sys/wait.h>
#include	<timeout.h>
#include	<errno.h>
#include	<wait_child.h>
#include <tet_api.h>
#include	<sysdep.h>

public	int	ex_code;
public	int	sg_code;

int wait_child(waittime)
int	waittime;
{
	int	stat_child;	/* child process status */
	int	status;		/* wait return status */
	pid_t	rval;		/* return value of waitpid */

	errno = NOERROR;

	SET_TIMEOUT(waittime)
	rval = waitpid(tet_child, &status, WUNTRACED);
	CLEAR_ALARM
	if(rval == SYSERROR) {
		if(errno == EINTR && alrm_flag) {
			stat_child = CHLD_LIVE;
		} else {
			stat_child = CHLD_ERR;
		}
	} else if(WIFEXITED(status)) {
		stat_child = CHLD_EXIT;
		ex_code = WEXITSTATUS(status);
	} else if(WIFSIGNALED(status)) {
		stat_child = CHLD_ABRT;
		sg_code = WTERMSIG(status);
	} else if(WIFSTOPPED(status)) {
		stat_child = CHLD_STOP;
	} else {
		/* SHOULD NOT GET HERE - waitpid() WRONGLY IMPLEMENTED */
		/* errno will be set to NOERROR if this is the case */
		stat_child = CHLD_ERR;
	}

	return(stat_child);
}
