/*
*      SCCS:   @(#)common/tsetlib(wcharcfg.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		wcharcfg.c - read the wide character binary file

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	wcharcfg()
SYNOPSIS:
	int wcharcfg(cfg)
	struct wchar *cfg;
DESCRIPTION:
	wcharcfg() reads a binary wide character file generated by wcc.
	The name of the file that is read is as follows:
	<value of TET_EXECUTE in env>/TESTROOT/BIN/wchars.bin
	As each entry is read in the corresponding entry in the wchar
	structure is allocated memory and assigned the value of the
	entry. No format checking of the binary file is done.


NOTE:

AUTHOR:		David G. Sawyer, UniSoft Ltd.
DATE CREATED:	Fri Dec 06 21:20:06 GMT 1991
MODIFICATIONS:
		Geoff Clare, 14 Oct 1992
			Changes for XNFS.

		Geoff Clare, 24 Feb 1994
			Change buffer[] to be declared static.

************************************************************************/

#include <std.h>
#if TEST_XPG > 3 && !TEST_XNFS
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <limits.h>
#include <wchar_cfg.h>
#include <sysdep.h>

#define READ_IN_MAX	4096		/* The max number of bytes to be read */
#define FILE_DIR	"/BIN"		/* The dir below TET_EXECUTE */
#define FILE_NAME	"/wchars.bin"	/* The actual file itself */

#define	READ_CFG(_val)							\
		len = strlen(ptr);				\
		_val = (unsigned char *) malloc( (size_t) (len + 1));	\
		if( _val == NULL)					\
			return(-1);					\
		(void) strcpy((char *)_val, ptr);			\
		ptr = &(ptr[len+1])


int
wcharcfg(cfg)
struct wchar *cfg;
{
	int	len, ctr = 0;
	FILE	*fptr;
	char	path[_POSIX_PATH_MAX];
	char	*str, *ptr;
	char	*getenv(), *envptr;
	static char buffer[READ_IN_MAX+1]; /* must be static so that end
					      of input is marked by nulls */

	envptr = getenv("TET_EXECUTE");
	if( envptr != NULL)
		(void) sprintf(path, "%s%s%s", envptr, FILE_DIR, FILE_NAME);
	else
		(void) sprintf(path,"%s", FILE_NAME);

	if( (fptr = fopen(path, "r")) == (FILE *) NULL)
		return(-1);

	if( read(fileno(fptr), buffer, READ_IN_MAX) == -1)
		return(-1);

	ptr = buffer;

	READ_CFG(cfg->locale1);
	READ_CFG(cfg->locale2);
	READ_CFG(cfg->iswcntrl[0]);
	READ_CFG(cfg->iswcntrl[1]);
	READ_CFG(cfg->iswcntrl[2]);
	READ_CFG(cfg->notcntrl[0]);
	READ_CFG(cfg->notcntrl[1]);
	READ_CFG(cfg->notcntrl[2]);
	READ_CFG(cfg->iswgraph[0]);
	READ_CFG(cfg->iswgraph[1]);
	READ_CFG(cfg->iswgraph[2]);
	READ_CFG(cfg->notgraph[0]);
	READ_CFG(cfg->notgraph[1]);
	READ_CFG(cfg->notgraph[2]);
	READ_CFG(cfg->iswlower[0]);
	READ_CFG(cfg->iswlower[1]);
	READ_CFG(cfg->iswlower[2]);
	READ_CFG(cfg->notlower[0]);
	READ_CFG(cfg->notlower[1]);
	READ_CFG(cfg->notlower[2]);
	READ_CFG(cfg->iswpunct[0]);
	READ_CFG(cfg->iswpunct[1]);
	READ_CFG(cfg->iswpunct[2]);
	READ_CFG(cfg->notpunct[0]);
	READ_CFG(cfg->notpunct[1]);
	READ_CFG(cfg->notpunct[2]);
	READ_CFG(cfg->iswspace[0]);
	READ_CFG(cfg->iswspace[1]);
	READ_CFG(cfg->iswspace[2]);
	READ_CFG(cfg->notspace[0]);
	READ_CFG(cfg->notspace[1]);
	READ_CFG(cfg->notspace[2]);
	READ_CFG(cfg->iswupper[0]);
	READ_CFG(cfg->iswupper[1]);
	READ_CFG(cfg->iswupper[2]);
	READ_CFG(cfg->notupper[0]);
	READ_CFG(cfg->notupper[1]);
	READ_CFG(cfg->notupper[2]);
	READ_CFG(cfg->towupper[0][0]);
	READ_CFG(cfg->towupper[0][1]);
	READ_CFG(cfg->towupper[1][0]);
	READ_CFG(cfg->towupper[1][1]);
	READ_CFG(cfg->towupper[2][0]);
	READ_CFG(cfg->towupper[2][1]);
	READ_CFG(cfg->towlower[0][0]);
	READ_CFG(cfg->towlower[0][1]);
	READ_CFG(cfg->towlower[1][0]);
	READ_CFG(cfg->towlower[1][1]);
	READ_CFG(cfg->towlower[2][0]);
	READ_CFG(cfg->towlower[2][1]);
	READ_CFG(cfg->width0[0]);
	READ_CFG(cfg->width0[1]);
	READ_CFG(cfg->width0[2]);
	READ_CFG(cfg->width1[0]);
	READ_CFG(cfg->width1[1]);
	READ_CFG(cfg->width1[2]);
	READ_CFG(cfg->width2[0]);
	READ_CFG(cfg->width2[1]);
	READ_CFG(cfg->width2[2]);
	READ_CFG(cfg->width3[0]);
	READ_CFG(cfg->width3[1]);
	READ_CFG(cfg->width3[2]);
	READ_CFG(cfg->dontcare[0]);
	READ_CFG(cfg->dontcare[1]);
	READ_CFG(cfg->dontcare[2]);
	READ_CFG(cfg->primary[0][0]);
	READ_CFG(cfg->primary[0][1]);
	READ_CFG(cfg->primary[1][0]);
	READ_CFG(cfg->primary[1][1]);
	READ_CFG(cfg->primary[2][0]);
	READ_CFG(cfg->primary[2][1]);
	READ_CFG(cfg->primary[3][0]);
	READ_CFG(cfg->primary[3][1]);
	READ_CFG(cfg->primary[4][0]);
	READ_CFG(cfg->primary[4][1]);
	READ_CFG(cfg->primary[5][0]);
	READ_CFG(cfg->primary[5][1]);
	READ_CFG(cfg->primary[6][0]);
	READ_CFG(cfg->primary[6][1]);
	READ_CFG(cfg->primary[7][0]);
	READ_CFG(cfg->primary[7][1]);
	READ_CFG(cfg->primary[8][0]);
	READ_CFG(cfg->primary[8][1]);
	READ_CFG(cfg->primary[9][0]);
	READ_CFG(cfg->primary[9][1]);
	READ_CFG(cfg->secondary[0][0]);
	READ_CFG(cfg->secondary[0][1]);
	READ_CFG(cfg->secondary[1][0]);
	READ_CFG(cfg->secondary[1][1]);
	READ_CFG(cfg->secondary[2][0]);
	READ_CFG(cfg->secondary[2][1]);
	READ_CFG(cfg->secondary[3][0]);
	READ_CFG(cfg->secondary[3][1]);
	READ_CFG(cfg->secondary[4][0]);
	READ_CFG(cfg->secondary[4][1]);
	READ_CFG(cfg->secondary[5][0]);
	READ_CFG(cfg->secondary[5][1]);
	READ_CFG(cfg->secondary[6][0]);
	READ_CFG(cfg->secondary[6][1]);
	READ_CFG(cfg->secondary[7][0]);
	READ_CFG(cfg->secondary[7][1]);
	READ_CFG(cfg->secondary[8][0]);
	READ_CFG(cfg->secondary[8][1]);
	READ_CFG(cfg->secondary[9][0]);
	READ_CFG(cfg->secondary[9][1]);
	READ_CFG(cfg->primary2_1[0][0]);
	READ_CFG(cfg->primary2_1[0][1]);
	READ_CFG(cfg->primary2_1[0][2]);
	READ_CFG(cfg->primary2_1[1][0]);
	READ_CFG(cfg->primary2_1[1][1]);
	READ_CFG(cfg->primary2_1[1][2]);
	READ_CFG(cfg->primary2_1[2][0]);
	READ_CFG(cfg->primary2_1[2][1]);
	READ_CFG(cfg->primary2_1[2][2]);
	READ_CFG(cfg->primary2_1[3][0]);
	READ_CFG(cfg->primary2_1[3][1]);
	READ_CFG(cfg->primary2_1[3][2]);
	READ_CFG(cfg->primary2_1[4][0]);
	READ_CFG(cfg->primary2_1[4][1]);
	READ_CFG(cfg->primary2_1[4][2]);
	READ_CFG(cfg->primary2_1[5][0]);
	READ_CFG(cfg->primary2_1[5][1]);
	READ_CFG(cfg->primary2_1[5][2]);
	READ_CFG(cfg->secondary2_1[0][0]);
	READ_CFG(cfg->secondary2_1[0][1]);
	READ_CFG(cfg->secondary2_1[0][2]);
	READ_CFG(cfg->secondary2_1[1][0]);
	READ_CFG(cfg->secondary2_1[1][1]);
	READ_CFG(cfg->secondary2_1[1][2]);
	READ_CFG(cfg->secondary2_1[2][0]);
	READ_CFG(cfg->secondary2_1[2][1]);
	READ_CFG(cfg->secondary2_1[2][2]);
	READ_CFG(cfg->secondary2_1[3][0]);
	READ_CFG(cfg->secondary2_1[3][1]);
	READ_CFG(cfg->secondary2_1[3][2]);
	READ_CFG(cfg->secondary2_1[4][0]);
	READ_CFG(cfg->secondary2_1[4][1]);
	READ_CFG(cfg->secondary2_1[4][2]);
	READ_CFG(cfg->secondary2_1[5][0]);
	READ_CFG(cfg->secondary2_1[5][1]);
	READ_CFG(cfg->secondary2_1[5][2]);
	READ_CFG(cfg->equal1_2[0][0]);
	READ_CFG(cfg->equal1_2[0][1]);
	READ_CFG(cfg->equal1_2[0][2]);
	READ_CFG(cfg->equal1_2[1][0]);
	READ_CFG(cfg->equal1_2[1][1]);
	READ_CFG(cfg->equal1_2[1][2]);
	READ_CFG(cfg->equal1_2[2][0]);
	READ_CFG(cfg->equal1_2[2][1]);
	READ_CFG(cfg->equal1_2[2][2]);
	READ_CFG(cfg->equal1_2[3][0]);
	READ_CFG(cfg->equal1_2[3][1]);
	READ_CFG(cfg->equal1_2[3][2]);
	READ_CFG(cfg->equal1_2[4][0]);
	READ_CFG(cfg->equal1_2[4][1]);
	READ_CFG(cfg->equal1_2[4][2]);
	READ_CFG(cfg->equal1_2[5][0]);
	READ_CFG(cfg->equal1_2[5][1]);
	READ_CFG(cfg->equal1_2[5][2]);
	READ_CFG(cfg->decpoint1);
	READ_CFG(cfg->decpoint2);
	READ_CFG(cfg->month1[0]);
	READ_CFG(cfg->month1[1]);
	READ_CFG(cfg->month1[2]);
	READ_CFG(cfg->month1[3]);
	READ_CFG(cfg->month1[4]);
	READ_CFG(cfg->month1[5]);
	READ_CFG(cfg->month1[6]);
	READ_CFG(cfg->month1[7]);
	READ_CFG(cfg->month1[8]);
	READ_CFG(cfg->month1[9]);
	READ_CFG(cfg->month1[10]);
	READ_CFG(cfg->month1[11]);
	READ_CFG(cfg->abmon1[0]);
	READ_CFG(cfg->abmon1[1]);
	READ_CFG(cfg->abmon1[2]);
	READ_CFG(cfg->abmon1[3]);
	READ_CFG(cfg->abmon1[4]);
	READ_CFG(cfg->abmon1[5]);
	READ_CFG(cfg->abmon1[6]);
	READ_CFG(cfg->abmon1[7]);
	READ_CFG(cfg->abmon1[8]);
	READ_CFG(cfg->abmon1[9]);
	READ_CFG(cfg->abmon1[10]);
	READ_CFG(cfg->abmon1[11]);
	READ_CFG(cfg->day1[0]);
	READ_CFG(cfg->day1[1]);
	READ_CFG(cfg->day1[2]);
	READ_CFG(cfg->day1[3]);
	READ_CFG(cfg->day1[4]);
	READ_CFG(cfg->day1[5]);
	READ_CFG(cfg->day1[6]);
	READ_CFG(cfg->abday1[0]);
	READ_CFG(cfg->abday1[1]);
	READ_CFG(cfg->abday1[2]);
	READ_CFG(cfg->abday1[3]);
	READ_CFG(cfg->abday1[4]);
	READ_CFG(cfg->abday1[5]);
	READ_CFG(cfg->abday1[6]);
	READ_CFG(cfg->am_str1);
	READ_CFG(cfg->pm_str1);
	READ_CFG(cfg->month2[0]);
	READ_CFG(cfg->month2[1]);
	READ_CFG(cfg->month2[2]);
	READ_CFG(cfg->month2[3]);
	READ_CFG(cfg->month2[4]);
	READ_CFG(cfg->month2[5]);
	READ_CFG(cfg->month2[6]);
	READ_CFG(cfg->month2[7]);
	READ_CFG(cfg->month2[8]);
	READ_CFG(cfg->month2[9]);
	READ_CFG(cfg->month2[10]);
	READ_CFG(cfg->month2[11]);
	READ_CFG(cfg->abmon2[0]);
	READ_CFG(cfg->abmon2[1]);
	READ_CFG(cfg->abmon2[2]);
	READ_CFG(cfg->abmon2[3]);
	READ_CFG(cfg->abmon2[4]);
	READ_CFG(cfg->abmon2[5]);
	READ_CFG(cfg->abmon2[6]);
	READ_CFG(cfg->abmon2[7]);
	READ_CFG(cfg->abmon2[8]);
	READ_CFG(cfg->abmon2[9]);
	READ_CFG(cfg->abmon2[10]);
	READ_CFG(cfg->abmon2[11]);
	READ_CFG(cfg->day2[0]);
	READ_CFG(cfg->day2[1]);
	READ_CFG(cfg->day2[2]);
	READ_CFG(cfg->day2[3]);
	READ_CFG(cfg->day2[4]);
	READ_CFG(cfg->day2[5]);
	READ_CFG(cfg->day2[6]);
	READ_CFG(cfg->abday2[0]);
	READ_CFG(cfg->abday2[1]);
	READ_CFG(cfg->abday2[2]);
	READ_CFG(cfg->abday2[3]);
	READ_CFG(cfg->abday2[4]);
	READ_CFG(cfg->abday2[5]);
	READ_CFG(cfg->abday2[6]);
	READ_CFG(cfg->am_str2);
	READ_CFG(cfg->pm_str2);
	READ_CFG(cfg->mbcurmax1);
	READ_CFG(cfg->mbcurmax2);
	READ_CFG(cfg->invalid_mb);

	len = strlen(ptr);
	str = (char *) malloc( (size_t) (len + 1));
	if( str == NULL)
		return(-1);
	(void) strcpy(str, ptr);
	ptr = &(ptr[len+1]);
	cfg->invalid_wc = (int) strtol(str, (char **) NULL, 0);
	free((void *) str);

	/* check for any properties */
	cfg->properties = NULL;
	len = strlen(ptr);
	while( len != 0)
	{
		cfg->properties = ( struct wc_prop *) realloc((void *) cfg->properties, (size_t) (sizeof(struct wc_prop) * (ctr+1)));
		if(cfg->properties == NULL)
			return(-1);

		/* first the name */
		cfg->properties[ctr].name = (unsigned char *) malloc( (size_t) (len + 1));
		if( cfg->properties[ctr].name == NULL)
			return(-1);
		(void) strcpy((char *) cfg->properties[ctr].name, ptr);
		ptr = &(ptr[len+1]);

		/* the IS bit */
		len = strlen(ptr);
		cfg->properties[ctr].is[0] = (unsigned char *) malloc( (size_t) (len + 1));
		if( cfg->properties[ctr].is[0] == NULL)
			return(-1);
		(void) strcpy((char *) cfg->properties[ctr].is[0], ptr);
		ptr = &(ptr[len+1]);

		len = strlen(ptr);
		cfg->properties[ctr].is[1] = (unsigned char *) malloc( (size_t) (len + 1));
		if( cfg->properties[ctr].is[1] == NULL)
			return(-1);
		(void) strcpy((char *) cfg->properties[ctr].is[1], ptr);
		ptr = &(ptr[len+1]);

		len = strlen(ptr);
		cfg->properties[ctr].is[2] = (unsigned char *) malloc( (size_t) (len + 1));
		if( cfg->properties[ctr].is[2] == NULL)
			return(-1);
		(void) strcpy((char *) cfg->properties[ctr].is[2], ptr);
		ptr = &(ptr[len+1]);

		/* the NOT bit */
		len = strlen(ptr);
		cfg->properties[ctr].not[0] = (unsigned char *) malloc( (size_t) (len + 1));
		if( cfg->properties[ctr].not[0] == NULL)
			return(-1);
		(void) strcpy((char *) cfg->properties[ctr].not[0], ptr);
		ptr = &(ptr[len+1]);

		len = strlen(ptr);
		cfg->properties[ctr].not[1] = (unsigned char *) malloc( (size_t) (len + 1));
		if( cfg->properties[ctr].not[1] == NULL)
			return(-1);
		(void) strcpy((char *) cfg->properties[ctr].not[1], ptr);
		ptr = &(ptr[len+1]);

		len = strlen(ptr);
		cfg->properties[ctr].not[2] = (unsigned char *) malloc( (size_t) (len + 1));
		if( cfg->properties[ctr].not[2] == NULL)
			return(-1);
		(void) strcpy((char *) cfg->properties[ctr].not[2], ptr);
		ptr = &(ptr[len+1]);

		len = strlen(ptr);
		++ctr;
	}
	cfg->properties = ( struct wc_prop *) realloc((void *) cfg->properties, (size_t) (sizeof(struct wc_prop) * (ctr+1)));
	if(cfg->properties == NULL)
		return(-1);

	/* this is checked for in the tests */
	cfg->properties[ctr].name = NULL;

	(void ) fclose(fptr);

	return(0);
}
#else /* TEST_XPG > 3 && !TEST_XNFS */
static int dummy;
#endif
