/*
*      SCCS:   @(#)common/tsetlib(wcharstr.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		wcharstr.c - generate a wide char string library routine

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	wcharstr()
SYNOPSIS:
	int wcharstr(locale, wcstr)
	char **locale;
	char **wcstr;
DESCRIPTION:
	wcharstr() takes the address of two charcter pointers as parameters.
	The first is allocated memory and assigned the name of the locale1
	from the wide character configuration file.
	The second is allocated memory and assigned a string consisting of
	wide characters built up from entries in the wide character
	configuration file.
	The locale is saved before the function changes to locale1 specified
	in the wide character configuration file. The wide character string
	is then built up by converting entries from the config file using
	mbtowc(). Once the string has been built and assigned the locale is
	restored to its original value.


NOTE:

AUTHOR:		David G. Sawyer, UniSoft Ltd.
DATE CREATED:	Fri Dec 06 21:20:06 GMT 1991
MODIFICATIONS:
		Geoff Clare, 31 July 1992
			Add decimal point character to string.

		Geoff Clare, 14 Oct 1992
			Changes for XNFS.

************************************************************************/

#include <std.h>
#if TEST_XPG > 3 && !TEST_XNFS
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <limits.h>
#include <locale.h>
#include <wchar_cfg.h>
#include <sysdep.h>

#define BUILD_STR(_val)							\
			if(mbtowc(wcptr,(char *)_val,(size_t) 50) != 0)	\
			{						\
				if( *wcptr != 0)			\
				{					\
					if( wcstr == NULL)		\
						wcstr = (wchar_t *) malloc(sizeof(wchar_t));								\
					else				\
						wcstr = (wchar_t *) realloc((void *) wcstr, (size_t) (sizeof(wchar_t)*(num+1)));			\
					if(wcstr == NULL)		\
						return(-1);		\
					wcstr[num] = *wcptr;		\
					num++;				\
				}					\
			}



wcharstr(locale, str)
char **locale;
wchar_t **str;
{
	struct	wchar cfg;
	wchar_t	*wcstr = NULL;
	int	num = 0, ctr;
	wchar_t wcptr[2];
	char *ptr;
	char current[50];

	/* Get a copy of the current locale */
	if((ptr = setlocale(LC_CTYPE, NULL)) == (char *)NULL)
		return(-1);

	(void) strcpy(current, ptr);

	/* Get the wchar structure */
	if(wcharcfg(&cfg) == -1)
		return(-1);
			
	/* Set up the locale */
	if(setlocale(LC_CTYPE, (char *)cfg.locale1) == (char *)NULL)
		return(-1);

	if((*locale = (char *) malloc((size_t) (strlen((char *) cfg.locale1)+1))) == NULL)
		return(-1);

	/* put a copy of the locale name into the supplied parameter */
	(void) strcpy(*locale, (char *) cfg.locale1);

	/* build up the array */
	BUILD_STR(cfg.iswcntrl[0]);
	BUILD_STR(cfg.iswcntrl[1]);
	BUILD_STR(cfg.iswcntrl[2]);
	BUILD_STR(cfg.notcntrl[0]);
	BUILD_STR(cfg.notcntrl[1]);
	BUILD_STR(cfg.notcntrl[2]);
	BUILD_STR(cfg.iswgraph[0]);
	BUILD_STR(cfg.iswgraph[1]);
	BUILD_STR(cfg.iswgraph[2]);
	BUILD_STR(cfg.notgraph[0]);
	BUILD_STR(cfg.notgraph[1]);
	BUILD_STR(cfg.notgraph[2]);
	BUILD_STR(cfg.iswlower[0]);
	BUILD_STR(cfg.iswlower[1]);
	BUILD_STR(cfg.iswlower[2]);
	BUILD_STR(cfg.notlower[0]);
	BUILD_STR(cfg.notlower[1]);
	BUILD_STR(cfg.notlower[2]);
	BUILD_STR(cfg.iswpunct[0]);
	BUILD_STR(cfg.iswpunct[1]);
	BUILD_STR(cfg.iswpunct[2]);
	BUILD_STR(cfg.notpunct[0]);
	BUILD_STR(cfg.notpunct[1]);
	BUILD_STR(cfg.notpunct[2]);
	BUILD_STR(cfg.iswspace[0]);
	BUILD_STR(cfg.iswspace[1]);
	BUILD_STR(cfg.iswspace[2]);
	BUILD_STR(cfg.notspace[0]);
	BUILD_STR(cfg.notspace[1]);
	BUILD_STR(cfg.notspace[2]);
	BUILD_STR(cfg.iswupper[0]);
	BUILD_STR(cfg.iswupper[1]);
	BUILD_STR(cfg.iswupper[2]);
	BUILD_STR(cfg.notupper[0]);
	BUILD_STR(cfg.notupper[1]);
	BUILD_STR(cfg.notupper[2]);
	BUILD_STR(cfg.width0[0]);
	BUILD_STR(cfg.width0[1]);
	BUILD_STR(cfg.width0[2]);
	BUILD_STR(cfg.width1[0]);
	BUILD_STR(cfg.width1[1]);
	BUILD_STR(cfg.width1[2]);
	BUILD_STR(cfg.width2[0]);
	BUILD_STR(cfg.width2[1]);
	BUILD_STR(cfg.width2[2]);
	BUILD_STR(cfg.width3[0]);
	BUILD_STR(cfg.width3[1]);
	BUILD_STR(cfg.width3[2]);
	BUILD_STR(cfg.decpoint1);
	BUILD_STR(cfg.mbcurmax1);

	if( num != 0)
	{
		*str = (wchar_t *) malloc( (size_t) (sizeof(wchar_t) * (num+1)));
		if( *str == NULL)
			return(-1);
	}

	for( ctr = 0; ctr < num; ctr++)
		(*str)[ctr] = wcstr[ctr];
	(*str)[ctr] = (wchar_t) 0;

	/* Restore the locale */
	if(setlocale(LC_CTYPE, current) == NULL)
		return(-1);

	return(0);
}
#else /* TEST_XPG > 3 && !TEST_XNFS */
static int dummy;
#endif
