/*
*      SCCS:   @(#)common/vlib(error.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/
/***********************************************************************

NAME:		error.c - print error string and return
PROJECT:	VSX (X/OPEN Validation Suite)
MODULE CONTAINS: vsxerror()
SYNOPSIS:
	#include <varargs.h>

	void
	vsxerror(format, arg1, ... argn)
		char	*format;
		args -> depend on format, like printf()
DESCRIPTION:
	prints out, on stderr,

	PPPPP error: XXXXXXXX

	- where PPPP = progname from get_pname(), XXXXX =
	format string and optional args as form printf().
NOTE:
	progname PPPPP is sensible only if it has been set
	by set_pname() prior to the call to get_pname();
ASSUMES:
	file descriptor 2 (STDERR) open;
	varargs functions properly on the host system;

AUTHOR:	Hamish REID, UniSoft Ltd.
DATE CREATED:	Tue Apr  8 17:06:19 BST 1986
MODIFICATIONS:
	Tue Sep 16 10:29:09 BST 1986	 -mhr
		added ifndef's for systems with out vfprintf().
	Mon Mar  7 16:01:03 WET 1988	 -gwc
		removed ifndef's since vprintf() now supplied in vport
	Mon Feb 26 13:58:46 GMT 1990
		Did varargs/stdarg conversion - sr

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#if TEST_ANSI
#include	<stdarg.h>
#else
#include	<varargs.h>
#endif
#include 	<dbug.h>
#include	<sysdep.h>


extern	char	*get_pname();

/*VARARGS0*/
void
#if TEST_ANSI
vsxerror(char *format, ...)
#else
vsxerror(format, va_alist)
	char *format;
	va_dcl
#endif
{
	/* call as vsxerror(format, arg1, ... argn)	*/

	va_list	args;

#if TEST_ANSI
	va_start(args, format);
#else
	va_start(args);
#endif

	(void) fprintf(stderr, "%s error: ", get_pname());
	(void) vfprintf(stderr, format, args);
	(void) fprintf(stderr, "\n");
	(void) fflush(stderr);

	va_end(args);
}
