/*
*      SCCS:   @(#)common/vlib(setsigs.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/
/***********************************************************************

NAME:		setsigs.c - set up signal handlers
PROJECT:	VSX (X/OPEN Validation Suite)
MODULE CONTAINS:
		setsigs()
SYNOPSIS:
	#include <signal.h>

	void setsigs(func)
		void (*func)();

DESCRIPTION:

	Setsigs() sets all signals except SIGKILL, SIGSTOP, SIGCHLD,
	and any signals already being ignored, to be handled by "func".
	Honouring of ignored signals is done so that only "vtest" needs
	to process VSX_SIG_IGN - all its children inherit the ignored
	signals.

AUTHOR:		Stuart Boutell, UniSoft Ltd.
DATE CREATED:	9 Nov 1990 (split out from vsignal.c)
MODIFICATIONS:	

************************************************************************/

#include	<std.h>
#include	<string.h>
#include	<stdio.h>
#include	<errno.h>
#include	<dbug.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<tet_api.h>
#include	<sysdep.h>

private int	sig_leave[NSIG] = { FALSE };

private void
leave_init()
{
	char	*list, *sname;
	int	snum, i;

	sig_leave[0] = TRUE;
	for (i = 1; i < NSIG; i++)
		sig_leave[i] = FALSE;

	list = tet_getvar("TET_SIG_LEAVE");
	if (list == NULL)
		return;

	for (sname = strtok(list, ", "); sname != NULL;
					sname = strtok((char *) NULL, ", "))
	{
		snum = atoi(sname);

		if (snum < 1 || snum >= NSIG)
		{
			vsxerror("warning: bad entry \"%s\" in VSX_SIG_LEAVE ignored",
					sname);
			continue;
		}

		/* Check it's not a "known" signal */
		for (i = 0; i <= VNUMSIG; i++)
			if (sig_array[i] == snum)
				break;
		if (i <= VNUMSIG)
		{
			vsxerror("warning: illegal entry \"%s\" in VSX_SIG_LEAVE ignored",
					sname);
			continue;
		}

		sig_leave[snum] = TRUE;
	}
}

public	void
setsigs(func)
void	(*func)();
{
	/*
	 * Sets all signals except SIGKILL, SIGSTOP and SIGCHLD
	 * to be caught by "func", except that signals already being
	 * ignored continue to be ignored and signals specified in
	 * VSX_SIG_LEAVE are left alone.
	 */

	int i;
	struct sigaction sig;

	DBUG_LENTER("setsigs", 2);
	DBUG_PRINTF("args", "setsigs(0x%lx)", (long) func);

	/* sig_leave[0] indicates whether remainder of array initialised */
	if (!sig_leave[0])
		leave_init();

	for (i = 1; i < NSIG; i++)
	{
		if (i == SIGKILL || i == SIGSTOP || i == SIGCHLD
			|| sig_leave[i])
			continue;
		
		if (sigaction(i, NULLSA, &sig) == SYSERROR ||
			sig.sa_handler != SIG_IGN)
		{
			sig.sa_handler = func;
			sig.sa_flags = 0;
			(void) sigemptyset(&sig.sa_mask);

			(void) sigaction(i, &sig, NULLSA);
			/* return can be safely ignored: only way
			   of failing is EINVAL */
		}
	}

	DBUG_VOID_RETURN;
}
