/*
*      SCCS:   @(#)common/vlib(signame.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/
/***********************************************************************

NAME:		signame.c - look up signal name from number
PROJECT:	VSX (X/OPEN Validation Suite)
MODULE CONTAINS:
		signame()
SYNOPSIS:

	char *signame(sig)
		int sig;
	
DESCRIPTION:
	Signame() is used to obtain signal names.  It returns
	"non-X/Open signal" for signals not defined in the XPG.


AUTHOR:		Stuart Boutell, UniSoft Ltd.
DATE CREATED:	22 Nov 1990 (split out from vsignal.c)
MODIFICATIONS:	

	Geoff Clare, 10 Jan 1997
		Rewritten to be independent of sig_name[] (which
		has test-mode-dependent contents).
			
************************************************************************/

#include	<std.h>
#include	<dbug.h>
#include	<signal.h>
#include	<sysdep.h>

private	struct	nament	{
	int	sno;
	char	*sname;
} name[] = {
	0,		"0",
	SIGABRT,	"SIGABRT",
	SIGALRM,	"SIGALRM",
	SIGFPE,		"SIGFPE",
	SIGHUP,		"SIGHUP",
	SIGILL,		"SIGILL",
	SIGINT,		"SIGINT",
	SIGKILL,	"SIGKILL",
	SIGPIPE,	"SIGPIPE",
	SIGQUIT,	"SIGQUIT",
	SIGSEGV,	"SIGSEGV",
	SIGTERM,	"SIGTERM",
	SIGUSR1,	"SIGUSR1",
	SIGUSR2,	"SIGUSR2",
	SIGCHLD,	"SIGCHLD",
	SIGCONT,	"SIGCONT",
	SIGSTOP,	"SIGSTOP",
	SIGTSTP,	"SIGTSTP",
	SIGTTIN,	"SIGTTIN",
	SIGTTOU,	"SIGTTOU",
#ifdef SIGBUS
	SIGBUS,		"SIGBUS",
#endif
#ifdef SIGPOLL
	SIGPOLL,	"SIGPOLL",
#endif
#ifdef SIGPROF
	SIGPROF,	"SIGPROF",
#endif
#ifdef SIGSYS
	SIGSYS,		"SIGSYS",
#endif
#ifdef SIGTRAP
	SIGTRAP,	"SIGTRAP",
#endif
#ifdef SIGURG
	SIGURG,		"SIGURG",
#endif
#ifdef SIGVTALRM
	SIGVTALRM,	"SIGVTALRM",
#endif
#ifdef SIGXCPU
	SIGXCPU,	"SIGXCPU",
#endif
#ifdef SIGXFSZ
	SIGXFSZ,	"SIGXFSZ",
#endif
};


char *
signame(number)
int number;
{
	int	i;
	char	*s = 0;

	DBUG_LENTER("signame", 2);
	DBUG_PRINTF("args", "signame(%d)", number);

	for (i = 0; i < ((sizeof name)/(sizeof(struct nament))); i++)
	{
		if (name[i].sno == number)
		{
			s = name[i].sname;
			break;
		}
	}

	if (s == 0)
		s = "non-X/Open signal";

	DBUG_PRINTF("return", "signame = %s", s);
	DBUG_RETURN(s);
}
