/*
*      SCCS:   @(#)common/vlib(sigvars.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/
/***********************************************************************

NAME:		sigvars.c - signal global variables.
PROJECT:	VSX (X/OPEN Validation Suite)
MODULE CONTAINS:
		sig_array[]
		sig_name[]
SYNOPSIS:
	#include <signal.h>
	#include <vsx_signal.h>

	extern int sig_array[VNUMSIG+1];
	extern char *sig_name[VNUMSIG+1];

DESCRIPTION:
	These routines provide for a simple and uniform way of
	handling signals by testset drivers and subprograms.

	Sig_array[] gives the signal numbers of the signals valid
	for the current test mode.  It is indexed by VSIGHUP, etc. 
	It contains room for an extra entry VSIGINVAL, which if it
	is needed may be initialised by calling vsx_sigs().

	Sig_name[] gives the names of the signals in sig_array[],
	indexed by VSIGHUP etc.  This is provided for convenience
	when using sig_array[].  In other cases, signal names should
	be obtained from signame().

AUTHOR:		Stuart Boutell, UniSoft Ltd.
DATE CREATED:	21 Nov 1990 (split out from vsignal.c)
MODIFICATIONS:	
	
	Geoff Clare, 15 Jan 1997
		Update for XSH5.

************************************************************************/

#include	<std.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<sysdep.h>

public	int	sig_array[VNUMSIG + 1] = {
	0,
	SIGABRT,
	SIGALRM,
	SIGFPE,
	SIGHUP,
	SIGILL,
	SIGINT,
	SIGKILL,
	SIGPIPE,
	SIGQUIT,
	SIGSEGV,
	SIGTERM,
	SIGUSR1,
	SIGUSR2,
	SIGCHLD,
#ifdef VSIGBUS
	SIGBUS,
#endif
#ifdef VSIGPOLL
	SIGPOLL,
#endif
#ifdef VSIGPROF
	SIGPROF,
#endif
#ifdef VSIGSYS
	SIGSYS,
#endif
#ifdef VSIGTRAP
	SIGTRAP,
#endif
#ifdef VSIGURG
	SIGURG,
#endif
#ifdef VSIGVTALRM
	SIGVTALRM,
#endif
#ifdef VSIGXCPU
	SIGXCPU,
#endif
#ifdef VSIGXFSZ
	SIGXFSZ,
#endif
	SIGCONT,
	SIGSTOP,
	SIGTSTP,
	SIGTTIN,
	SIGTTOU,
	VNOSIG
};

public	char	*sig_name[VNUMSIG + 1] = {
	"0",
	"SIGABRT",
	"SIGALRM",
	"SIGFPE",
	"SIGHUP",
	"SIGILL",
	"SIGINT",
	"SIGKILL",
	"SIGPIPE",
	"SIGQUIT",
	"SIGSEGV",
	"SIGTERM",
	"SIGUSR1",
	"SIGUSR2",
	"SIGCHLD",
#ifdef VSIGBUS
	"SIGBUS",
#endif
#ifdef VSIGPOLL
	"SIGPOLL",
#endif
#ifdef VSIGPROF
	"SIGPROF",
#endif
#ifdef VSIGSYS
	"SIGSYS",
#endif
#ifdef VSIGTRAP
	"SIGTRAP",
#endif
#ifdef VSIGURG
	"SIGURG",
#endif
#ifdef VSIGVTALRM
	"SIGVTALRM",
#endif
#ifdef VSIGXCPU
	"SIGXCPU",
#endif
#ifdef VSIGXFSZ
	"SIGXFSZ",
#endif
	"SIGCONT",
	"SIGSTOP",
	"SIGTSTP",
	"SIGTTIN",
	"SIGTTOU",
	"invalid signal"
};

