/*
*      SCCS:   @(#)common/vlib(vsignal.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/
/***********************************************************************

NAME:		vsignal.c - signal handling routines
PROJECT:	VSX (X/OPEN Validation Suite)
MODULE CONTAINS:
		vsignal()

	#include <signal.h>

	void (*vsignal(sig, func))()
		int sig;
		void (*func)();


DESCRIPTION:
	Vsignal() emulates signal() using sigaction() thus allowing
	painless transformation from signal() to sigaction().

AUTHOR:		Manuch Ghezel, UniSoft Ltd.
DATE CREATED:	TUE May 27
MODIFICATIONS:	
		Geoff Clare, 9 Mar 1989
			Rationalised the plethora of signal facilities
			in vlib and tsetlib into vlib/vsignal.c.
			Converted to XPG3 (signal -> sigaction, etc.).
		
		Geoff Clare, 13 Nov 1989
			New VSX_SIG_LEAVE parameter
		
		Geoff Clare, 22 Mar 1990
			Provide SIG_ERR if not defined (in POSIX-only mode)

		Stuart Boutell, 9 Nov 1990
			Split vsignal.c into setsigs.c, signame.c, sigvars.c
			and vsignal.c.   Removed all but vsignal and SIG_ERR.

************************************************************************/

#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<signal.h>
#include	<sysdep.h>

#ifndef SIG_ERR
/*
 * This is to provide a value for use as SIG_ERR.
 * Note that (void (*)())-1 is not portable.
 */

extern void panic();

public void
SIG_ERR()
{
	panic("SIG_ERR() got called! (see vlib/vsignal.c)");
}
#endif /* SIG_ERR */

public void
(*vsignal(sig, func))()
int sig;
void (*func)();
{
	struct sigaction new, old;

	DBUG_LENTER("vsignal", 2);
	DBUG_PRINTF("args", "vsignal(%d, 0x%lx)", sig, (long) func);

	if (func == SIG_ERR)
	{
		DBUG_PRINTF("return", "vsignal = SIG_ERR");
		DBUG_RETURN(SIG_ERR);
	}

	new.sa_handler = func;
	new.sa_flags = 0;
	(void) sigemptyset(&new.sa_mask);

	if (sigaction(sig, &new, &old) == SYSERROR)
	{
		int err = errno;
		DBUG_PRINTF("return", "vsignal = SIG_ERR");
		DBUG_RETURN((errno = err, SIG_ERR));
	}
	else
	{
		DBUG_PRINTF("return", "vsignal = 0x%lx", (long) old.sa_handler);
		DBUG_RETURN(old.sa_handler);
	}
}
