/*
*	SCCS:	@(#)common/vprog/input.c	4.3 (03/25/98)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		input.c - vrpt input routine

PROJECT:	VSX (X/OPEN Validation Suite)

AUTHOR:			David McCardle
DATE CREATED:		Wed Apr 30 10:53:47 BST 1986
MODIFICATIONS:
		6/4/88   Geoff Clare:	-M and -T options
		9/6/88,  Geoff Clare:	Double pass: vrptsum.awk, vrptdet.awk
		19/9/88, Geoff Clare:	Recognise %..td, %..ts

		J.A. Nave	Thu Oct 25 17:03:11 BST 1990
		Modified to accept phoenix style journal files instead
		of old vsx style journal files

		Geoff Clare, 11 Feb 1991
		Fix core dumping on build journal problem.
		Tidy up.

		Geoff Clare, 21 June 1991
		Add -t option.
		Add FIP Sign-off section

		Geoff Clare, 6 Nov 1991
		Change getcnam() to account for new pathnames ("/tset/...")

		Geoff Clare, 14 Nov 1991
		Change td_rpt() and ts_rpt() to work with macro tests.

		Stuart Boutell, 21 Nov 1991
		Add -p option.

		Geoff Clare, 20 Oct 1992
		Changes for XNFS.

		Geoff Clare, 21 Nov 1996
		Prevent line[] buffer overflow in getcnam().

		Geoff Clare, 15 July 1997
		Added journal code 5 (system info) to list of valid codes.

		Geoff Clare, 25 March 1998
		Allow for long testset names in td_rpt() and ts_rpt().

************************************************************************/

#include <std.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <limits.h>
#include <dbug.h>
#include <vrpt.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <tet_api.h>
#include <sysdep.h>

#define	JOU_NUM_LEN	11
#define VSX_TET_FIP	102

#ifndef _XOPEN_SOURCE
#undef popen
extern	FILE	*popen();
#endif
extern char	*newstr();
extern char	*vsxalloc();
extern	char	*arc_errname();

public void	ctrl_rpt();
public	int	pcheck();
public	char	*getcnam();

private FILE	*com_pipe;
private int	reported = FALSE;

private	int	file_inp();
private	void	prt_220();
private	void	ign_rpt();
private void	ts_rpt();
private void	td_rpt();


public int
inp_pipe(awkprog, arc_proc, n_files, t_file, argc, argv, opts)
char *awkprog;
int	arc_proc;
int n_files;
char *t_file[];
int argc;
char *argv[];
struct opt *opts;
{
      int i, err;
      time_t tloc;
      char *cmd;
      char *path;
      FILE *tf;

      static char curdir[_POSIX_PATH_MAX + 1] = "";

      DBUG_ENTER ("inp_pipe");

      err = 0;
      if ((path = getenv("VSXBIN")) != NULL && *path != '\0')
      {
	 cmd = vsxalloc((unsigned)(strlen(path) + strlen(awkprog) + 10));
	 (void) sprintf(cmd, "awk -f %s/%s", path, awkprog);
      }
      else if ((path = getenv("HOME")) != NULL && *path != '\0')
      {
	 /* No VSXBIN - use $HOME/BIN */
	 cmd = vsxalloc((unsigned)(strlen(path) + strlen(awkprog) + 15));
	 (void) sprintf(cmd, "awk -f %s/BIN/%s", path, awkprog);
      }
      else
      {
	 vsxerror("No HOME in environment!");
         DBUG_RETURN(201);
      }

      DBUG_PRINTF("inp_pipe0", "cmd :%s:", cmd);

      if ((com_pipe = popen(cmd, "w")) == NULL)
      {
	 vsxerror("cannot open pipe to \"%s\"", cmd);
         DBUG_RETURN(201);
      }

      free((void *)cmd);

      /* Send parameters down pipe */
      (void) time(&tloc);
      (void) fprintf(com_pipe, "%s", ctime(&tloc));
      DBUG_PRINTF("inp_pipe0", "ctime(&tloc) :%s:", ctime(&tloc));
      (void) fprintf(com_pipe, "%s", argv[0]);
      DBUG_PRINTF("inp_pipe0", "argv[0] :%s:", argv[0]);
      for (i = 1; i < argc; i++)
      {
	 if (strchr(argv[i], ' ') == NULL && strchr(argv[i], '\t') == NULL)
         {
	      (void) fprintf(com_pipe, " %s", argv[i]);
              DBUG_PRINTF("inp_pipe0", "argv[i] :%s:", argv[i]);
         }
	 else
         {
	      (void) fprintf(com_pipe, " \"%s\"", argv[i]);
              DBUG_PRINTF("inp_pipe0", "argv[i] :%s:", argv[i]);
         }
      }
      (void) fprintf(com_pipe, "\n%s\n%s\n%s\n%s\n%s\n%d\n%d\n%s\n%s\n%d\n%s\n",
	  opts->l, opts->r, opts->d, opts->H, opts->P,
	  opts->L, opts->W, opts->n, opts->v, opts->t,
          opts->p);
      DBUG_PRINTF("inp_pipe0", "opts \n%s\n%s\n%s\n%s\n%s\n%d\n%d\n%s\n%s\n%d\n%s\n",
	  opts->l, opts->r, opts->d, opts->H, opts->P,
	  opts->L, opts->W, opts->n, opts->v, opts->t,
          opts->p);

      DBUG_PRINTF("inp_pipe2", "n_files :%d:", n_files);

      for (i = 0; i < n_files && err == 0; i++)
      {
	 DBUG_PRINTF("inp_pipe3", "t_file[%d] :%s:", i, t_file[i]);

	 tf = fopen(t_file[i], "r");
	 if (tf == NULL)
         {
	    vsxerror("can't open journal file \"%s\"", t_file[i]);
	    err = 202;
         }
	 else
	 {
	    DBUG_PRINTF("inp_pipe4", "t_file[%d] :%s:", i, t_file[i]);

	    if (t_file[i][0] == '/')
	       (void) fprintf(com_pipe, "-1|%s|\n", t_file[i]);
	    else
	    {
	       if (curdir[0] == '\0')
		  (void) getcwd(curdir, _POSIX_PATH_MAX);
	       (void) fprintf(com_pipe, "-1|%s/%s|\n", curdir, t_file[i]);
               (void) fflush(com_pipe);
	    }

	    err = file_inp(tf, arc_proc);
	    (void) fclose(tf);
	 }
      } /* for (i = 0; i < n_files; i++) */

      (void) pclose(com_pipe);

      /* Don't repeat warnings on second pass */
      reported = TRUE;

      DBUG_RETURN (err);

} /* inp_pipe() */

private int
file_inp(test_file, arc_proc)

FILE *test_file;
int	arc_proc;
{

	int	err;
	char	line[BUFSIZE+1];
	long	n_lines;
	char	*tc_name = NULL;

	DBUG_ENTER ("file_inp");
	DBUG_PRINTF("file_inp1", "arc_proc :%d:", arc_proc);

	err = 0;
	n_lines = 0;
	while (fgets(line, BUFSIZE+1, test_file) != (char *)NULL)
	{

		DBUG_PRINTF("file_inp2", "line :%s:", line);

		n_lines++;
		if (n_lines == 1 && strncmp(line, "0|", (size_t)2) != 0)
			ctrl_rpt("start");

		DBUG_PRINTF("file_inp3", "n_lines :%ld:", n_lines);
		DBUG_PRINTF("file_inp4", "line :%s:", line);

		if (pcheck(n_lines, line) == 0)
		{
			if (fputs(line, com_pipe) == EOF)
			{
				vsxerror("I/O error writing to pipe");
				err = 210;
				break;
			}

			/* Extract test case name from TC Start line */
			if (strncmp(line, "10|", (size_t)3) == 0)
			{
				if ((tc_name = getcnam(line)) ==
					(char *)NULL)
				{
					vsxerror("getcnam() failed");
					err = 210;
					break;
				}
			}

			/* For TP Result lines, add test
			   descriptions and strategies */
			if (tc_name != NULL &&
			    strncmp(line, "220|", (size_t)4) == 0)
				prt_220(tc_name, line, arc_proc);

			DBUG_PRINTF("file_inp7", "line :%s:", line);
		}
	} 

   /* Check we got "0|" at the start and "900|" just before EOF */
   if (n_lines == 0)
	ctrl_rpt("start");
   else if (strncmp(line, "900|", (size_t)4) != 0)
	ctrl_rpt("end");

   DBUG_RETURN (err);
} /* file_inp(test_file) */


public void
ctrl_rpt(s)
char *s;
{
      if (!reported)
	 (void) fprintf(stderr, "%s %s %s\n",
	    "warning: journal file does not", s,
	    "with correct control sequence");
}

public	int
pcheck(line_n, line)
long	line_n;
char	line[];
{
	char	ph_jou_no[JOU_NUM_LEN];
	int	journal_no;
	int	pipe_found;
	int	i,	j;

   	DBUG_ENTER ("pcheck()");
	pipe_found = 0;

	for (i=0, j=0; line[i] != '\n' && line[i] != '\0' && j<JOU_NUM_LEN;
		i++, j++)
	{
		if (line[i] == '|')
		{	pipe_found++;
			break;
		}
		if (line[0] == '-' || isdigit(line[i]))
			ph_jou_no[j] = line[i];
	}

	if (!pipe_found)
	{
		ign_rpt(line_n, line);
      		DBUG_PRINTF("pcheck() failed", "pipe symbol not found");
		DBUG_RETURN(-1);
	}

	ph_jou_no[j] = '\0';

	journal_no = atoi(ph_jou_no);

	switch(journal_no)
	{
		case	-1:
		case	0:
		case	5:
		case	10:
		case	15:
		case	20:
		case	30:
		case	40:
		case	50:
		case	70:
		case	80:
		case	90:
		case	110:
		case	130:
		case	200:
		case	220:
		case	300:
		case	320:
		case	400:
		case	410:
		case	510:
		case	520:
		case	800:
		case	820:
		case	900:
		break;
		default:
		{
			ign_rpt(line_n, line);
      			DBUG_PRINTF("pcheck() failed", "invalid jounal number");
			DBUG_RETURN(-1);
		}
	}

      	DBUG_PRINTF("pcheck() successful", "line correctly validated");
	DBUG_RETURN(0);
}

private void
ign_rpt(line_n, line)
long line_n;
char	*line;
{

      if (!reported)
	 (void) fprintf(stderr,
	    "warning - line %ld ignored:%s:\n", line_n, line);
}

public char *
mkarcname(tc_name, suffix)
char *tc_name;
char *suffix;
{
	/*
	 * Produce test description/strategy archive name from
	 * test case name and specified suffix ("t" or "s").
	 *
	 * tc_name contains ".../sect/area/[M]tdir/T.tset"
	 * returned name is ".../sect/area/tdir/L.tset_x" (x is suffix)
	 */
	char		*endarea, *tdir, *endtdir, *tset;
	static char	name[_POSIX_PATH_MAX];

   	DBUG_ENTER ("mkarcname");

	endarea = strrchr(tc_name, '/');
	for (--endarea; *endarea != '/'; endarea--)
		;
	*endarea = '\0';
	tdir = endarea+1;
	if (*tdir == 'M')
		tdir++;
	endtdir = strchr(tdir, '/');
	*endtdir = '\0';
	tset = endtdir+3;
	(void) sprintf(name, "%s/%s/L.%s_%s", tc_name, tdir, tset, suffix);
	*endarea = '/';
	*endtdir = '/';

	DBUG_RETURN(name);
}

private void
td_rpt(tc_name, testnum, fcode)
char	*tc_name;
int	testnum;
int	fcode;
{
	/*
	 * Print test description from L.<test>_t -
	 * if test is successful just print POSIX reference, otherwise
	 * print whole description.  The POSIX reference is in a separate
	 * archive entry "<testname><testnum>R".
	 */

	char	fname[_POSIX_PATH_MAX+1];
	char	prefix[30];
	char	*testnam;
	char	*arcname;

   	DBUG_ENTER ("td_rpt()");

	arcname = mkarcname(tc_name, "t");
	testnam = strrchr(tc_name, '/') +3;
	(void) sprintf(fname, "%.*s%d", _POSIX_PATH_MAX-4, testnam, testnum);
	(void) sprintf(prefix, "-3|%d|", testnum);

	DBUG_PRINTF("td_rpt", "arcname :%s:\tfname :%s:", arcname, fname);

	if (fcode != TET_PASS)
	{
		if (arc_print(prefix, com_pipe, arcname, fname) == -1)
			(void) fprintf(com_pipe, "Error: %s\n", arc_errname(arcname));
	}
	(void) strcat(fname, "R");
	if (arc_print(prefix, com_pipe, arcname, fname) == -1)
		(void) fprintf(com_pipe, "Error: %s\n", arc_errname(arcname));

	DBUG_VOID_RETURN;
}

private void
ts_rpt(tc_name, testnum)
char *tc_name;
int testnum;
{
	/* Print test strategy from L.<test>_s */

	char	fname[_POSIX_PATH_MAX+1];
	char	prefix[30];
	char	*testnam;
	char	*arcname;

   	DBUG_ENTER ("ts_rpt()");

	arcname = mkarcname(tc_name, "s");
	testnam = strrchr(tc_name, '/') +3;
	(void) sprintf(fname, "%.*s%d", _POSIX_PATH_MAX-4, testnam, testnum);
	(void) sprintf(prefix, "-2|%d|", testnum);

	DBUG_PRINTF("ts_rpt", "arcname :%s:\tfname :%s:", arcname, fname);

	if (arc_print(prefix, com_pipe, arcname, fname) == -1)
		(void) fprintf(com_pipe, "Error: %s\n", arc_errname(arcname));

	DBUG_VOID_RETURN;
}


public	char *
getcnam(tmp)
char	*tmp;
{
	char	*path, *p, *r;
	char	line[_POSIX_PATH_MAX];
	static	char	buf[_POSIX_PATH_MAX];

   	DBUG_ENTER ("getcnam()");

	/* set p to point to the start of the pathname & null terminate it */

	*line = '\0';
	(void) strncat(line, tmp, sizeof(line));
	for (p = strchr(line, '|'); *p != ' ' && *p != '\0'; p++)
		;
	p++;
   	DBUG_PRINTF("getcnam()", "p :%s:", p);

	r = p;
	for(r++; *r != ' ' && *r != '\0'; r++)
		;
	*r = '\0';
   	DBUG_PRINTF("getcnam()", "p :%s:", p);

	if ((path = getenv("HOME")) == NULL || *path == '\0')
	{
		vsxerror("No HOME in environment!");
		DBUG_RETURN((char *)NULL);
	}

	(void) strcat( strcat( strcpy(buf, path), "/MAN"), p);

   	DBUG_PRINTF("getcnam()", "buf :%s:", buf);

	DBUG_RETURN(buf);
}

private	void
prt_220(testnam, line, arc_proc)
char	*testnam;
char	*line;
int	arc_proc;
{
	int	i;
	char	*p;
	int	tpnum;
	int	fcode;
	char	*tmp;

   	DBUG_ENTER ("prt_220()");

	/* extract the TPnumber and fcode */

	if ((tmp = newstr(line)) == (char *)NULL)
		DBUG_VOID_RETURN;

	for (i = 0, p = strtok(tmp, " "); i < 4;
		p = strtok((char *)NULL, " "), i++)
	{
		if (i == 1)
			tpnum = atoi(p);
		if (i == 2)
			fcode = atoi(p);
	}

	/*
	 * arc_proc is true if test descriptions and strategies are to be
	 * added to the output.  If false, only add test descriptions for
	 * FIP results.
	 */

	if (arc_proc || fcode == VSX_TET_FIP)
		td_rpt(testnam, tpnum, fcode);
	if (arc_proc && fcode == TET_FAIL)
		ts_rpt(testnam, tpnum);
	free((void *) tmp);

	DBUG_VOID_RETURN;
}

