/*
*	SCCS:	@(#)common/vprog/prpt.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

static	char sccsid[] = "@(#)common/vprog/prpt VSXgen release 1.4";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};

/***********************************************************************

NAME:		prpt.c - main entry for prpt(1)

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		prpt()
AUTHOR:		J.A. Nave, UniSoft Ltd.
MODIFICATIONS:
		Geoff Clare, 11 Feb 1991
			Fix core dumping on unset tc_name problem.
			Tidy up.

		Geoff Clare, 21 June 1991
			Catch SIGPIPE

		Geoff Clare, 14 Nov 1991
			Change posix_prt() to work with macro tests.

		Stuart Boutell, 21 Nov 1991
			Added page of page numbering.

************************************************************************/

#include <std.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <dbug.h>
#include <vrpt.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include <limits.h>
#include <time.h>
#include <signal.h>
#include <sysdep.h>

#ifndef _XOPEN_SOURCE
#undef popen
extern	FILE	*popen();
#endif
#ifndef optind
extern int optind;
#endif
#ifndef opterr
extern int opterr;
#endif
#ifndef optarg
extern char *optarg;
#endif

extern	void	set_pname();
extern	void	ctrl_rpt();
extern	char	*newstr();
extern	char	*vsxalloc();
extern	int	pcheck();
extern	char	*getcnam();
extern	char	*mkarcname();
extern	char	*arc_errname();
extern void	(*vsignal())();

private int	opt_get();
private int	out_pipe();
private int	pipe_err();
private	int	inp_pipe(); /* Input driver to awk command (local version) */
private	int	file_inp(); 
private	void	posix_prt(); 

private	FILE	*com_pipe;
static	char	*t_file[100];
static	int	n_files = 0;

static	char	yes[] = "YES";
static	char	no[]  = "NO";

static void
sigpipe()
{
	vsxerror("received SIGPIPE");
	exit(128+SIGPIPE);
}

int
main(argc, argv)

   int argc;
   char *argv[];
{
   int ix, i, err;
   struct opt option;

   DBUG_ENTER("main");
   DBUG_PROCESS(argv[0]);

   set_pname(argv[0]);

   for (ix = 1; ix < argc && argv[ix][0] == '-'; ix++)
   {
      switch (argv[ix][1])
      {
         case '#':
            DBUG_PUSH(&(argv[ix][2]));
            if ((ix + 1) < argc)
               for (i = ix + 1; i < argc; i++)
                  argv[i - 1] = argv[i];
             else
               argv[ix] = (char *) NULL;
            argc--;
            ix--;
            break;
      }
   } /* for (ix = 1; ix < argc && argv[ix][0] == '-'; ix++) */

   /* Set default options */
   option.P = yes;
   option.p = yes;
   option.H = yes;
   option.d = no;
   option.n = no;
   option.l = "POSIX";
   option.r = "all";
   option.f = "none";
   option.v = no;
   option.L = 66;
   option.W = 80;
   err = 0;

   if ((err = opt_get(argc, argv, &option)) != 0)
   {
      DBUG_PRINTF ("opt_get", "err :%d:", err);

      (void) fprintf(stderr, "usage: prpt [-H] [-L len] [-W wid] [file ...]\n");
      exit(1);
   }

   while (optind < argc)
   {
      t_file[n_files] = vsxalloc((unsigned) strlen(argv[optind]) + 1);
      (void) strcpy(t_file[n_files++], argv[optind++]);
   }

   if (n_files == 0)
   {
      (void) fprintf(stderr, "prpt: must specify at least one input file\n");
      exit(1);
   }

   DBUG_PRINTF ("option", "l :%s:", option.l);
   DBUG_PRINTF ("option", "r :%s:", option.r);
   DBUG_PRINTF ("option", "f :%s:", option.f);
   DBUG_PRINTF ("option", "d :%s:", option.d);
   DBUG_PRINTF ("option", "H :%s:", option.H);
   DBUG_PRINTF ("option", "P :%s:", option.P);
   DBUG_PRINTF ("option", "p :%s:", option.p);
   DBUG_PRINTF ("option", "L :%d:", option.L);
   DBUG_PRINTF ("option", "W :%d:", option.W);
   DBUG_PRINTF ("option", "v :%s:", option.v);

   (void) vsignal(SIGPIPE, sigpipe);

   out_pipe();

   err = inp_pipe(n_files, t_file, &option);
   if (err != 0)
   {
      DBUG_PRINTF ("inp_pipe", "err :%d:", err);
      exit(2);
   }

   if (fclose(stdout) >= 0)
      (void) wait((int *)0);

   DBUG_RETURN(0);
} /* main(argc, argv) */

private int
opt_get(opt_argc, opt_argv, option)

int opt_argc;
char *opt_argv[];
struct opt *option;
{
   int c, len, err;
   
   err = 0;
   opterr = 0;
   while ((c = getopt(opt_argc, opt_argv, "H:L:W:")) != EOF)
   {
      switch (c)
      {
         case 'H':
            option->H = no;
            break;
   
         case 'L':
            if ((len = atoi(optarg)) <= 0)
               err = 107;
	    else
	       option->L = len;
            break;
   
         case 'W':
            if ((len = atoi(optarg)) <= 0)
               err = 107;
	    else if (len <= MAXCOL)
		option->W = len;
	    else
		option->W = MAXCOL;
            break;

         case '?':
               err = 108;
               break;
      } /* switch (c) */
      if (err != 0)
         break;
   } /* while ((c = getopt(...)) != EOF) */

   return(err);
} /* opt_get(opt_argc, opt_argv, option) */


private int
out_pipe()
{
	/* reopen stdout as pipe to paging program */

	char		*path;
	int		pid, pipefd[2];
	static char	*cmd = "vrpt.page";	/* Use PATH if no VSXBIN */

	DBUG_ENTER("out_pipe");

	/* set path to paging program */
	if ((path = getenv("VSXBIN")) != NULL && *path != '\0')
	{
	   cmd = vsxalloc((unsigned) strlen(path) + 12);
	   (void) strcat(strcpy(cmd, path), "/vrpt.page");
	}
	DBUG_PRINTF ("out_pipe0", "cmd :%s:", cmd);

	if (pipe(pipefd) != 0)
		pipe_err("pipe()");
	
	if ((pid = fork()) < 0)
		pipe_err("fork()");
	else if (pid == 0) {
		/* child - execute paging program */
DBUG_PRINTF ("out_pipe0", "cmd :%s:", cmd);
		(void) close(0);
		if (dup(pipefd[0]) != 0)
			pipe_err("dup()");
		(void) close(pipefd[0]);
		(void) close(pipefd[1]);
		(void) execlp(cmd, cmd, (char *)0);
		pipe_err("execlp()");
		exit(2);
	}
	else {
		/* parent - attach stdout to pipe */
		(void) close(1);
		if (dup(pipefd[1]) != 1)
			pipe_err("dup()");
		(void) close(pipefd[0]);
		(void) close(pipefd[1]);
	}
	
	DBUG_VOID_RETURN;
}

public char *
vsxalloc(size)
unsigned size;
{
	char *buf;

	if ((buf = (char *)malloc((size_t)size)) == NULL)
	{
           vsxerror("malloc() failed");
	   exit(1);
	}
	
	return buf;
}

private int
pipe_err(func)
char *func;
{
	/* exit on failure to create pipe to paging command */

	vsxerror("could not create pipe to vrpt.page (%s failed)", func);
	exit(1);
}

private int
inp_pipe(no_files, tm_file, opts)
int no_files;
char *tm_file[];
struct opt *opts;
{
	int i, err;
	char *tmp;
	char *cmd;
	char *path;
	FILE *tf;
	char	*cmdfmt =
			"(	\
				awk -f %s/prpt.awk1 tmpfile=%s - &&	\
				sort -t. +1n -2 +2n -3 +0 -1 +3n -4 +4n -5 \
				%s %s/prpt.static |sed 's/\\./ /'	\
			) | awk -f %s/prpt.awk2";
	static char curdir[_POSIX_PATH_MAX + 1] = "";
	char	tfile[_POSIX_PATH_MAX + 1];

	DBUG_ENTER ("inp_pipe");

	err = 0;
	if ((path = getenv("VSXBIN")) == NULL || *path == '\0')
	{
		if ((tmp = getenv("HOME")) != NULL && *tmp != '\0')
		{
			path = vsxalloc((unsigned) (1 + strlen(tmp) +
				strlen("/BIN")));
			/* No VSXBIN - use $HOME/BIN */
			(void) strcat(strcpy(path, tmp), "/BIN");
		}
		else
		{
			vsxerror("No HOME in environment!");
			DBUG_RETURN(201);
		}
	}

	if (tmpnam(tfile) == (char *)NULL)
	{
		vsxerror("Unable to obtain tmporary filename");
		DBUG_RETURN(-1);
	}

	DBUG_PRINTF("inp_pipe0", "path :%s:", path);
	DBUG_PRINTF("inp_pipe0", "tfile :%s:", tfile);

	cmd = vsxalloc((unsigned) (3*strlen(path) + 2*strlen(tfile) 
		+ strlen(cmdfmt)));

	(void) sprintf(cmd, cmdfmt, path, tfile, tfile, path, path);

	DBUG_PRINTF("inp_pipe0", "cmd :%s:", cmd);

	if ((com_pipe = popen(cmd, "w")) == NULL)
	{
		vsxerror("cannot open pipe to \"%s\"", cmd);
		DBUG_RETURN(201);
	}

	/* Send parameters down pipe */
	DBUG_PRINTF("inp_pipe0", "opts \n%s\n%s\n%d\n%d\n\n",
		opts->H, opts->l, opts->L, opts->W);
	(void) printf("%s\n%s\n%d\n%d\n\n\n\n%s\n",
		opts->H, opts->l, opts->L, opts->W, opts->p);
	(void) fflush(stdout);

	DBUG_PRINTF("inp_pipe2", "no_files :%d:", no_files);

	for (i = 0; i < no_files && err == 0; i++)
	{
		DBUG_PRINTF("inp_pipe3", "tm_file[%d] :%s:", i, tm_file[i]);

		tf = fopen(tm_file[i], "r");
		if (tf == NULL)
		{
			vsxerror("can't open journal file \"%s\"",
					tm_file[i]);
			err = 202;
		}
		else
		{
			DBUG_PRINTF("inp_pipe4", "tm_file[%d] :%s:",
				i, tm_file[i]);

			if (tm_file[i][0] == '/')
			{
				(void) fprintf(com_pipe, "-1|%s|\n",
					tm_file[i]);
				(void) fflush(com_pipe);
				DBUG_PRINTF("inp_pipe5", "-1|%s|\n",
					tm_file[i]);
			}
			else
			{
				if (curdir[0] == '\0')
					(void) getcwd(curdir, _POSIX_PATH_MAX);
				(void) fprintf(com_pipe, "-1|%s/%s|\n",
					curdir, tm_file[i]);
				(void) fflush(com_pipe);
				DBUG_PRINTF("inp_pipe5", "-1|%s/%s|\n",
					curdir, tm_file[i]);
			}

			err = file_inp(tf);
			(void) fclose(tf);
		}
	} 

	(void) pclose(com_pipe);
	free((void *)cmd);
	(void) unlink(tfile);

	DBUG_RETURN (err);

}

private int
file_inp(test_file)

FILE *test_file;
{

	int	err;
	char	line[BUFSIZE+1];
	long	n_lines;
	char	*tc_name = NULL;

	DBUG_ENTER ("file_inp");

	err = 0;
	n_lines = 0;
	while (fgets(line, BUFSIZE+1, test_file) != (char *)NULL)
	{

		DBUG_PRINTF("file_inp2", "line :%s:", line);

		n_lines++;
		if (n_lines == 1 && strncmp(line, "0|", (size_t)2) != 0)
			ctrl_rpt("start");

		DBUG_PRINTF("file_inp3", "n_lines :%d:", n_lines);
		DBUG_PRINTF("file_inp4", "line :%s:", line);

		if (pcheck(n_lines, line) == 0)
		{
			/* Extract test case name from TC Start line */
			if (strncmp(line, "10|", (size_t)3) == 0)
			{
				if ((tc_name = getcnam(line)) == (char *)NULL)
				{
					vsxerror("getcnam() failed");
					err = 210;
					break;
				}
			}

			/* For TP Result lines, insert POSIX ref. number */
			if (tc_name != NULL &&
			    strncmp(line, "220|", (size_t)4) == 0)
				posix_prt(tc_name, line);

			DBUG_PRINTF("file_inp7", "line :%s:", line);
			if (fputs(line, com_pipe) == EOF)
			{
				vsxerror("I/O error writing to pipe");
				err = 210;
				break;
			}
		}
	} 

   /* Check we got "01|" at the start and "900|" before EOF */
   if (n_lines == 0)
	ctrl_rpt("start");
   else if (strncmp(line, "900|", (size_t)4) != 0)
	ctrl_rpt("end");

   DBUG_RETURN (err);
} /* file_inp(test_file) */


private	void
posix_prt(tc_name, line)
char	*tc_name;
char	*line;
{
	int	i;
	char	*p;
	int	tpnum;
	char	*tmp;
	char	*testnam;
	char	*arcname;
	char	fname[_POSIX_NAME_MAX+1];
	char	prefix[30];

	/*
	 * The POSIX reference is in a separate
	 * archive entry "<testname><tpnum>R".
	 */

   	DBUG_ENTER ("posix_prt()");

	/* extract the TPnumber */

	if ((tmp = newstr(line)) == (char *)NULL)
		DBUG_VOID_RETURN;

	for (i = 0, p = strtok(tmp, " "); i < 4;
		p = strtok((char *)NULL, " "), i++)
	{
		if (i == 1)
			tpnum = atoi(p);
	}

	arcname = mkarcname(tc_name, "t");
	testnam = strrchr(tc_name, '/') +3;
	(void) sprintf(fname, "%.*s%dR", _POSIX_NAME_MAX-4, testnam, tpnum);
	(void) sprintf(prefix, "-3|%d|", tpnum);

	DBUG_PRINTF("posix_prt", "arcname :%s:\tfname :%s:", arcname, fname);

	if (arc_print(prefix, com_pipe, arcname, fname) == -1)
		(void) fprintf(com_pipe, "Error: %s\n", arc_errname(arcname));

	free((void *) tmp);

	DBUG_VOID_RETURN;
}

