/*
*      SCCS:   @(#)common/vprog/vbuild.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

char	sccsid[] = "@(#)common/vprog/vbuild VSXgen release 1.4";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};

/***********************************************************************

NAME:		vbuild.c

PROJECT:	VSX (X/OPEN Validation Suite)
MODULE CONTAINS:
SYNOPSIS	Executes $HOME/BIN/vmake.sh INSTALL if tet_pname
		is "vbuild" and $HOME/BIN/vmake.sh CLOBBER if
		tet_pname is "vclean".  When testing XNFS the
		targets are INSTALL_xnfs and CLOBBER_xnfs.
			
DESCRIPTION:
RELATED DOCUMENTS:
AUTHOR:			Calum Laing, Unisoft Ltd.
DATE CREATED:		2nd November, 1990.
MODIFICATIONS:
		Geoff Clare, 8 Jan 1992
			Check for null return from tet_getvar().

		Geoff Clare, 10 Mar 1992
			Ignore SIGHUP, SIGINT and SIGQUIT.

		Geoff Clare, 22 Sept 1992
			Changes for testing XNFS.

		Geoff Clare, 10 Jan 1997
			Remove most entries from env_vars[], as vmake.sh
			obtains them direct from vsxparams instead of
			from the environment.

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<sys/wait.h>
#include	<signal.h>
#include	<tet_api.h>
#include	<sysdep.h>

#define 	CMDBUFSZ	256
#define		BUILD_NAME	"vbuild"
#define		CLEAN_NAME	"vclean"
#if TEST_XNFS
#define		INSTALL_TARGET	"INSTALL_xnfs"
#define		CLOBBER_TARGET	"CLOBBER_xnfs"
#else
#define		INSTALL_TARGET	"INSTALL"
#define		CLOBBER_TARGET	"CLOBBER"
#endif

private	char	*env_vars[] = {
		"VSXDIR",
		"PATH",
		NULL };   /* Must be terminated by a NULL */


private	char	cmdbuf[CMDBUFSZ];	/* Command used to execute vmake.sh 		*/

private	char	*resfile;		/* Name of file containing tool output		*/
extern	char	*vgetenv();
extern	void	(*vsignal())();
extern	char	**environ;
#ifndef getenv
extern	char	*getenv();
#endif

private	void	startfunc();
private	void	toolfunc();
private void	tool_config();

public	void	(*tet_startup)() = startfunc;
public	void	(*tet_cleanup)() = NULL;

public	struct tet_testlist tet_testlist[] = {
	toolfunc, 0,
	NULL, 0
};

private void
startfunc()
{
	char 	*nameptr;

	(void) vsignal(SIGHUP, SIG_IGN);
	(void) vsignal(SIGINT, SIG_IGN);
	(void) vsignal(SIGQUIT, SIG_IGN);
	tet_nosigreset = 1;

	/* In case it's a full path */
	if((nameptr = strrchr(tet_pname, '/')) != NULL)
		++nameptr;
	else
		nameptr = tet_pname;	
	
	if(strcmp(nameptr, BUILD_NAME) == 0) {
		tool_config(INSTALL_TARGET);
	}
	else if(strcmp(nameptr, CLEAN_NAME ) == 0) {
		tool_config(CLOBBER_TARGET);
	} 
	else  {
		tet_delete(1,"Tool file name not set correctly.");
		return;
	}
}

private void
toolfunc()
{
	int		status = 0;
	FILE	*log;
	char	fbuf[512-50];

	(void) fflush(stdout);
	(void) fflush(stderr);

	status = system(cmdbuf);

	if (WIFEXITED(status) && WEXITSTATUS(status) == 0)
		tet_result(TET_PASS);
	else
	{
		if( (log = fopen( resfile, "r")) == NULL) {
			(void)sprintf(fbuf,
				"could not open output file %s for reading",
				resfile);
			tet_infoline(fbuf);
		} else {
			
			while( fgets( fbuf, sizeof fbuf, log) != NULL)
				tet_infoline(fbuf);
			(void) fclose(log);
		}

		tet_result(TET_FAIL);
	}
}

private void
tool_config(target)
	char	*target;
{
	char	*s;
	char	*cvar;
	char	*ep;
	char	**ev;

	int		space;
	
	for(ev = env_vars; *ev != NULL ; ev++ ) {

		ep = *ev;
		cvar = tet_getvar(ep);
		if (cvar == NULL)
			cvar = "";

		if ( (s = (char *) malloc( (size_t) ( strlen(ep) + 3 + strlen(cvar) )))  == NULL ) {
			tet_delete(1,"malloc() failed.");
			return;
		}
	
		(void) sprintf(s,"%s=%s",ep,cvar);
	
		if( vputenv(&environ,s) != 0) {
			tet_delete(1,"vputenv() failed.");
			return;
		}
	}

/* Construct the name of the results file */
/* Length of "vmake."is 7 */

	if ( (resfile = (char *) malloc( (size_t) ( strlen(target) + 7 ))) == NULL) {
		tet_delete(1,"malloc() failed.");
		return;
	}
	
	(void) sprintf(resfile,"vmake.%s", target);

/* Construct the shell command */

	if(( s = getenv("HOME")) == NULL || *s == '\0') {
		tet_delete(1,"no HOME in environment.");
		return;
	}
	
	(void) strcpy(cmdbuf, s);
	space = CMDBUFSZ - (int) strlen(s);

	(void) strncat(cmdbuf, s= "/BIN/vmake.sh ", (size_t)space); 
	(void) strncat(cmdbuf, target,    (size_t)space); space -= (int) strlen(target);
	(void) strncat(cmdbuf, s=" > ",    (size_t)space); space -= (int) strlen(s);
	(void) strncat(cmdbuf, resfile,    (size_t)space); space -= (int) strlen(resfile);
	(void) strncat(cmdbuf, s=" 2>&1",    (size_t)space); space -= (int) strlen(s);
	
	if (space <= 0) {
		tet_delete(1,"make command too long for internal buffer");
		return;
	}
}
