/*
*	SCCS:	@(#)common/vprog/vrpt.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

static	char sccsid[] = "@(#)common/vprog/vrpt VSXgen release 1.4";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};

/***********************************************************************

NAME:		vrpt.c - main entry for vrpt(1)

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		vrpt()
AUTHOR:		David McCardle, UniSoft Ltd.
MODIFICATIONS:
		7/1/88, Geoff Clare:	Add -v option
		11/1/88, Geoff Clare:	Add -W option
		6/4/88, Geoff Clare:	Add -M and -T options,
					General tidy up
		9/6/88, Geoff Clare:	Double pass: vrptsum.awk, vrptdet.awk
		25/9/89, Geoff Clare:	Remove extern declarations.

		J.A. Nave	Thu Oct 25 17:03:11 BST 1990
		Modified to accept TET style journal files instead
		of old vsx style journal files

		Geoff Clare, 11 Feb 1991
		Don't fetch description and strategy for build journals

		Geoff Clare, 21 June 1991
			Catch SIGPIPE; add -t option

		Stuart Boutell, 21 November 1991
			Add -p option

		Geoff Clare, 30 Mar 1992
			Add forward declarations of static functions.

		Geoff Clare, 21 Apr 1992
			Remove -d from usage message.

************************************************************************/

#include <std.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <dbug.h>
#include <vrpt.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include <signal.h>
#include <sysdep.h>

#ifndef optind
extern int optind;
#endif
#ifndef opterr
extern int opterr;
#endif
#ifndef optarg
extern char *optarg;
#endif

extern int	inp_pipe(); /* Input driver to awk command */
extern void	set_pname();
extern char	*vsxalloc();
extern void	(*vsignal())();

private int opt_get();
private char *getfile();
private int out_pipe();
private int out_pipe();
private int pipe_err();

private char	*t_file[100];
private int	n_files = 0;

private int	vmake_jnl = FALSE;

private char	yes[] = "YES";
private char	no[]  = "NO";

private void
sigpipe()
{
	vsxerror("received SIGPIPE");
	exit(128+SIGPIPE);
}

int
main(argc, argv)

   int argc;
   char *argv[];
{
   int ix, i, err;
   struct opt option;

   DBUG_ENTER("main");
   DBUG_PROCESS(argv[0]);

   set_pname(argv[0]);

   for (ix = 1; ix < argc && argv[ix][0] == '-'; ix++)
   {
      switch (argv[ix][1])
      {
         case '#':
            DBUG_PUSH(&(argv[ix][2]));
            if ((ix + 1) < argc)
               for (i = ix + 1; i < argc; i++)
                  argv[i - 1] = argv[i];
             else
               argv[ix] = (char *) NULL;
            argc--;
            ix--;
            break;
      }
   } /* for (ix = 1; ix < argc && argv[ix][0] == '-'; ix++) */

   /* Set default options */
   option.P = yes;
   option.p = yes;
   option.H = yes;
   option.d = no;
   option.n = no;
   option.l = "Test-Set";
   option.r = "all";
   option.f = "none";
   option.v = no;
   option.L = 66;
   option.W = 80;
   option.t = 0;
   err = 0;

   if ((err = opt_get(argc, argv, &option)) != 0)
   {
      DBUG_PRINTF ("opt_get", "err :%d:", err);

      /* -d is obsolete - don't include in usage message */
      (void) fprintf(stderr, "usage: vrpt [-l level] [-r coverage] ");
      (void) fprintf(stderr, "[-f file] [-v]\n");
      (void) fprintf(stderr, "            [-H] [-p] [-P] [-L len] [-W wid] ");
      (void) fprintf(stderr, "[-t lines] [file ...]\n");
      exit(1);
   }

   while (optind < argc)
   {
      t_file[n_files] = vsxalloc((unsigned) strlen(argv[optind]) + 1);
      (void) strcpy(t_file[n_files++], argv[optind++]);
   }

   if (n_files == 0)
   {
      (void) fprintf(stderr, "vrpt: must specify at least one input file\n");
      exit(1);
   }

   DBUG_PRINTF ("option", "l :%s:", option.l);
   DBUG_PRINTF ("option", "r :%s:", option.r);
   DBUG_PRINTF ("option", "f :%s:", option.f);
   DBUG_PRINTF ("option", "d :%s:", option.d);
   DBUG_PRINTF ("option", "H :%s:", option.H);
   DBUG_PRINTF ("option", "P :%s:", option.P);
   DBUG_PRINTF ("option", "p :%s:", option.p);
   DBUG_PRINTF ("option", "L :%d:", option.L);
   DBUG_PRINTF ("option", "W :%d:", option.W);
   DBUG_PRINTF ("option", "v :%s:", option.v);
   DBUG_PRINTF ("option", "t :%d:", option.t);

   (void) vsignal(SIGPIPE, sigpipe);

   out_pipe();

   if (!vmake_jnl && option.P == yes)
   {
      err = inp_pipe("vrptsum.awk", FALSE, n_files, t_file, argc, argv,
                      &option);
      if (err != 0)
      {
	 DBUG_PRINTF ("inp_pipe", "err :%d:", err);
	 exit(2);
      }
   }

   err = inp_pipe("vrptdet.awk", !vmake_jnl, n_files, t_file, argc, argv,
		   &option);
   if (err != 0)
   {
      DBUG_PRINTF ("inp_pipe", "err :%d:", err);
      exit(2);
   }

   if (fclose(stdout) >= 0)
      (void) wait((int *)0);

   DBUG_RETURN(0);
} /* main(argc, argv) */

private int
opt_get(opt_argc, opt_argv, option)

int opt_argc;
char *opt_argv[];
struct opt *option;
{
   int c, len, err;
   char *range, *filerange;
   char *getfile();
   
   range = filerange = "";
   err = 0;
   opterr = 0;
   while ((c = getopt(opt_argc, opt_argv, "HpPdvl:r:f:L:W:t:")) != EOF)
   {
      switch (c)
      {
         case 'P':
            option->P = no;
            option->p = no;
            break;

         case 'p':
            option->p = no;
            break;
   
         case 'H':
            option->H = no;
            break;
   
         case 'd':
            option->d = yes;
            break;
   
         case 'n':
            option->n = yes;
            break;
   
         case 'v':
            option->v = yes;
            break;
   
         case 'l':
            if (strcmp(optarg, "sect") == 0)
               option->l = "Section";
            else if (strcmp(optarg, "area") == 0)
               option->l = "Area";
            else if (strcmp(optarg, "tset") == 0)
               option->l = "Test-Set";
            else
	       err = 102;
            break;
   
         case 'r':
            range = optarg;
            break;
   
         case 'f':
            option->f = optarg;
            if ((filerange = getfile(optarg)) == NULL)
               err = 105;
            break;
   
         case 'L':
            if ((len = atoi(optarg)) <= 0)
               err = 107;
	    else
	       option->L = len;
            break;
   
         case 'W':
            if ((len = atoi(optarg)) <= 0)
               err = 107;
	    else if (len <= MAXCOL)
		option->W = len;
	    else
		option->W = MAXCOL;
            break;

         case 't':
            if ((len = atoi(optarg)) < 0)
               err = 107;
	    else
		option->t = len;
            break;

         case '?':
               err = 108;
               break;
      } /* switch (c) */
      if (err != 0)
         break;
   } /* while ((c = getopt(...)) != EOF) */

   if (err == 0)
   {
      if (*range != '\0' && *filerange != '\0')
      {
	 option->r = vsxalloc((unsigned)(strlen(range) + strlen(filerange) + 2));
         (void) sprintf(option->r, "%s %s", range, filerange);
      }
      else if (*range != '\0')
	 option->r = range;
      else if (*filerange != '\0')
	 option->r = filerange;
   }

   return(err);
} /* opt_get(opt_argc, opt_argv, option) */

private char *
getfile(file_arg)

   char *file_arg;
{

   static char range[BUFSIZE];
   char line[MAXCOL+1], *fgets();
   FILE *file_des, *fopen();
   int len;

   if ((file_des = fopen(file_arg, "r")) == (FILE *) NULL)
   {
      vsxerror("cannot open file \"%s\"", file_arg);
      return((char *) NULL);
   }
   if (fgets(line, MAXCOL+1, file_des)  != (char *) NULL)
   {
      len = strlen(line);
      line[len - 1] = '\0';
      (void) strcpy(range, line);
   }
   while (fgets(line, MAXCOL+1, file_des)  != (char *) NULL)
   {
      len = strlen(line);
      line[len - 1] = '\0';
      (void) strncat(range, " ", BUFSIZE);
      (void) strncat(range, line, BUFSIZE);
   }
   (void) fclose(file_des);
   return (range);
} /* getfile(file_arg) */

private int
out_pipe()
{
	/* reopen stdout as pipe to paging program */

	char		*path;
	int		pid, pipefd[2];
	static char	*cmd = "vrpt.page";	/* Use PATH if no VSXBIN */

	DBUG_ENTER("out_pipe");

	/* set path to paging program */
	if ((path = getenv("VSXBIN")) != NULL && *path != '\0')
	{
	   cmd = vsxalloc((unsigned) strlen(path) + 12);
	   (void) strcat(strcpy(cmd, path), "/vrpt.page");
	}
	DBUG_PRINTF ("out_pipe0", "cmd :%s:", cmd);

	if (pipe(pipefd) != 0)
		pipe_err("pipe()");
	
	if ((pid = fork()) < 0)
		pipe_err("fork()");
	else if (pid == 0) {
		/* child - execute paging program */
		DBUG_PRINTF ("out_pipe0", "cmd :%s:", cmd);
		(void) close(0);
		if (dup(pipefd[0]) != 0)
			pipe_err("dup()");
		(void) close(pipefd[0]);
		(void) close(pipefd[1]);
		(void) execlp(cmd, cmd, (char *)0);
		pipe_err("execlp()");
		exit(2);
	}
	else {
		/* parent - attach stdout to pipe */
		(void) close(1);
		if (dup(pipefd[1]) != 1)
			pipe_err("dup()");
		(void) close(pipefd[0]);
		(void) close(pipefd[1]);
	}
	
	DBUG_VOID_RETURN;
}

public char *
vsxalloc(size)
unsigned size;
{
	char *buf;

	if ((buf = (char *)malloc((size_t)size)) == NULL)
	{
           vsxerror("malloc() failed");
	   exit(1);
	}
	
	return buf;
}

private int
pipe_err(func)
char *func;
{
	/* exit on failure to create pipe to paging command */

	vsxerror("could not create pipe to vrpt.page (%s failed)", func);
	exit(1);
}
