/*
*	SCCS:	@(#)common/vprog/vrpt.page.c	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

static	char sccsid[] = "@(#)common/vprog/vrpt.page VSXgen release 1.4";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};

/***********************************************************************

NAME:		vrpt.page.c - vrpt post-awk pager

PROJECT:	VSX (X/OPEN Validation Suite)


AUTHOR:		David McCardle UniSoft Ltd.
MODIFICATIONS:
		Geoff Clare, 8/1/88:	General re-write
		Geoff Clare, 8/1/88:	Perform line wrapping
		Geoff Clare, 16/6/88:	Change Header/Footer layout
		Stuart Boutell, 21/11/91:	Add pageofpage numbering
		Geoff Clare, 20/7/92:	Change page_num from long to int
		Geoff Clare, 16/1/97:	Changes for VSXgen

************************************************************************/

#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dbug.h>
#include <string.h>
#include <vrpt.h>
#include <sysdep.h>

#ifndef _XOPEN_SOURCE
#undef memset
extern	void	*memset();
#endif

#define public
#define private	static

#define MAX_INDENT_CHARS	MAXCOL
#define TABSTOP			8

#define INCPOS(pos,ch)		if (ch == '\t') \
				    while ((++pos) % TABSTOP != 0) \
					; \
				else \
				    ++pos;

private int col, max, number, rep_len, header, pageofpage, pagesize;
private int page_num;
private char *cmd, rep_dte[50], foot[MAXCOL+1], part[50], summary[MAXCOL+1];
private char s[BUFSIZE];
private FILE *output;
private	char outfile[L_tmpnam];

private void init_pageofpage();
private void do_pageofpage();
private void do_pageno();

main(argc, argv)
int argc;
char **argv;
{
	int ix;

	DBUG_ENTER("main");
	DBUG_PROCESS(argv[0]);

	for (ix = 1; ix < argc && argv[ix][0] == '-'; ix++)
	    if (argv[ix][1] == '#')
	    {
		DBUG_PUSH(&(argv[ix][2]));
	    }
	
	cmd = argv[0];

	get_params();

	init_pageofpage();	

	page_num = 1;
	number = 0;

	while (gets(s) != NULL)
	    proc_line();

	do_pageofpage();

	DBUG_RETURN(0);
}

#define NPARAM 8

get_params()
{
	/* read and process parameters (first NPARAM lines of input) */

	char		*param[NPARAM];
	int		i, len;
	char		space[MAXCOL+1];

	DBUG_ENTER("get_params");

	for (i = 0; i < NPARAM; i++)
	    if (gets(s) != NULL)
	    {
		if ((param[i] = (char *)malloc((size_t)(strlen(s)+1))) == NULL)
		{
			(void) fprintf(stderr, "%s: malloc() failed\n", cmd);
			exit(1);
		}
		(void) strcpy(param[i], s);
		DBUG_PRINTF("first", "param[%d] :%s: s :%s:", i, param[i], s);
	    }
	    else
		exit(0);
	
	header = (strncmp(param[0], "YES", 3) == 0);
	if (header) {
		pageofpage = (strncmp(param[7], "YES", 3) == 0);
	} else {
		pageofpage = 0;	/* no headers -> no pageofpage numbering */
	}


	pagesize = atoi(param[2]);
	col = atoi(param[3]);
	max = pagesize - 12;
	DBUG_PRINTF("init", "header = %c pagesize = %d col = %d" ,
		header ? 'Y' : 'N', pagesize, col);
	DBUG_PRINTF("init", "pageofpage = %c" ,
		pageofpage ? 'Y' : 'N');
	
	if (col > MAXCOL)
	{
		(void) fprintf(stderr, "%s: invalid page width (%d)\n", cmd, col);
		col = 80;
	}

	(void) sprintf(part, "%s Summary", param[1]);
	len = 2 * strlen(part);
	if (len >= col)
		len = col - 1;
	(void) memset((void *)space, ' ', (size_t)(col - len));
	space[col - len] = '\0';
	(void) sprintf(summary, "%s%s%s\n", part, space, part);

	len = strlen(param[4]) + strlen(param[5]);
	if (len >= col)
		len = col - 1;
	(void) memset((void *)space, ' ', (size_t)(col - len));
	space[col - len] = '\0';
	(void) strcpy(foot, param[4]);
	(void) strcat(foot, space);
	(void) strcat(foot, param[5]);

	(void) strcpy(rep_dte, param[6]);
	rep_len = strlen(rep_dte);


	DBUG_PRINTF("val1", "max = %d", max);
	DBUG_PRINTF("val2", "page_num = %d part :%s:", page_num, part);
	DBUG_PRINTF("val3", "space :%s: summary :%s:", space, summary);
	DBUG_PRINTF("val4", "foot :%s: rep_dte :%s:", foot, rep_dte);
	DBUG_PRINTF("val5", "rep_len = %d len = %d", rep_len, len);
	
	for (i = 0; i < NPARAM; i++)
	    free((void *)param[i]);

	DBUG_VOID_RETURN;
}

proc_line()
{
	int	len, remain, first_part;

	private char	indent[MAX_INDENT_CHARS];
	register char	*ip, *p;

	DBUG_ENTER("proc_line");

	DBUG_PRINTF("at top", "s :%s: number = %d", s, number);

	if (strncmp(s, ".bp", 3) == 0)
	{
	    if (number > 0)	/* don't bother if already at top of page */
	    {
		for ( ; number < max; number++)
		    (void) putc('\n',output);
		put_footer();
		number = 0;
	    }
	    DBUG_VOID_RETURN;
	}

	/* save indentation to use after wrapping */
	ip = indent;
	for (p = s; (*p == ' ' || *p == '\t') &&
		*p != '\0' && ip < &indent[MAX_INDENT_CHARS - 1]; p++, ip++)
	    *ip = *p;
	*ip++ = '\t';
	*ip = '\0';

	remain = strlen(s);
	if (strlen(indent) > 1)
	    len = find_wrap_pt(s, p) + (p - s);
	else
	    len = remain;	/* don't wrap if no indent! */
	p = s;
	first_part = 1;

	while (remain > 0 || first_part)
	{
	    if (number == 0)
		put_header();

	    if (!first_part)
	        (void) fputs(indent, output);
	    (void) fwrite((void *)p, (size_t)len, (size_t)1, output);
	    (void) putc('\n', output);
	    first_part = 0;

	    if (++number >= max)
	    {
	        put_footer();
	        number = 0;
	    }

	    remain -= len;
	    if (remain > 0)
	    {
		p += len;
		while (*p == ' ' || *p == '\t')
			++p, --remain;
		len = find_wrap_pt(indent, p);
	    }
	} /* while (remain > 0) */

	DBUG_VOID_RETURN;
}

put_header()
{
	static char middle[] = "X/OPEN Verification Suite";
	register int n;

	DBUG_ENTER("put_header");

	if (header)
	{
	    n = col - (sizeof(middle) - 1);
	    (void) fprintf(output, "\n\n%*s%s%*s\n%s\n\n",
			  n/2, " ", middle, n - n/2, " ", summary);
	}
	else
	    (void) fprintf(output, "\n\n\n\n\n");

	DBUG_VOID_RETURN;
}

put_footer()
{
	DBUG_ENTER("put_footer");

	if (header)
	{
	    (void) fprintf(output, "\n\n%s\n", foot);
            do_pageno(page_num, 9999, output);
	    (void) fprintf(output, "\n\n");
	    if (pageofpage && (page_num % 25 == 0)) {
		(void) fprintf(stderr, "%s: completed page %d\n", cmd, page_num);
	    }
	    page_num++;
	}
	else /* (header) */
	    (void) fprintf(output, "\n\n\n\n\n\n");

	DBUG_VOID_RETURN;
}

private void
do_pageno(page, total, stream) 
int page;
int total;
FILE *stream;
{
    private char	page_str[25];
    int i, num;
    DBUG_ENTER("do_pageno");

    (void) fprintf(stream, "%s", rep_dte);
    if (pageofpage) {
        (void) sprintf(page_str, "Page %d of %d", page, total);
    } else {
        (void) sprintf(page_str, "Page %d", page);
    }
    num = col - (rep_len + strlen(page_str));
    for (i = 0; i < num; i++)
        (void) putc(' ', stream);
    (void) fprintf(stream, "%s\n", page_str);

    DBUG_VOID_RETURN;
}

find_wrap_pt(indstr, str)
register char *indstr, *str;
{
	/* return length of str which can be printed following indstr
	   so as to fit on one line */
	
	int      last_space, indentpos;
	register linepos, len;

	DBUG_ENTER("find_wrap_pt");

	/* find length of indent ON SCREEN (i.e. expand tabs) */
	for (linepos = 0; *indstr == ' ' || *indstr == '\t'; indstr++)
	    INCPOS(linepos, *indstr);

	indentpos = linepos;

	last_space = 0;
	for (len = 0; linepos < col && *str != '\0'; len++, str++)
	{
	    INCPOS(linepos, *str);
	    
	    if (*str == ' ' || *str == '\t')
	    {
		last_space = len;
		indstr = str;	/* indstr not needed now, so save str in it */
	    }
	}

	if (*str != '\0' && last_space > 0)
	{
	    /* if the word after the wrap point won't fit on the next line,
	       might as well break it here (i.e. don't wrap) */
	    ++indstr;
	    ++indentpos;
	    for (linepos = indentpos; linepos < col && *indstr != '\0' &&
		    *indstr != ' ' && *indstr != '\t'; indstr++)
		INCPOS(linepos, *indstr);
	    if (linepos < col)
		DBUG_RETURN(last_space);
	}
	DBUG_RETURN(len);
}

private void
init_pageofpage()
{
	DBUG_ENTER("init_pageofpage");
	if (pageofpage) {
		(void) tmpnam(outfile);
		output = fopen(outfile, "w+");
		if (output == (FILE *)NULL) {
			(void) fprintf(stderr,
				"%s: fopen of temporary file failed\n", cmd);
			exit(1);
		}
		(void) unlink(outfile); /* Remove so that temporary file */
					/* doesn't remain if program aborts */
	} else {
		output = stdout;
	}

	DBUG_VOID_RETURN;
}


private void
do_pageofpage()
{
	int pagectr, linectr, linebrk;

	DBUG_ENTER("do_pageofpage");

	if (pageofpage) {

		page_num--;
		linebrk = pagesize - 3;

		rewind(output);

		pagectr = 1;
		linectr = 0;
		while( fgets(s, BUFSIZE, output) != NULL ) {
			if ((linectr % pagesize) == linebrk) {
				do_pageno(pagectr, page_num, stdout);
				pagectr++;
			} else {
				(void) printf("%s",s);
			}
			linectr++;
		}

		(void) fclose(output);

	}

	DBUG_VOID_RETURN;
}
