/*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
* Project: LSB-OS
*
* File: SRC/common/vsrtlib/ck_stime.c
*
* Modifications:
* $Log: ck_stime.c,v $
* Revision 1.2  2001/10/02 11:59:05  ajosey
* clean up the various header comment blocks for lsb-os
*
* Revision 1.1  2001/09/28 08:21:02  cyeoh
* Move of lsb-os specific libraries to lsb-os test set
*
* Revision 1.1  2001/06/01 02:00:15  cyeoh
* Merges for lsb-os test suite
*
*
*/

#include <vsrt/vsrt.h>
#include <time.h>


/*	Verify input specified time related field of struct stat	*/

void vsrt_ck_stime( int check, int req, time_t s_time, struct stat *st_start,
    time_t e_time, struct stat *st_end)
{
	if(check == CHK_ATIME) {
		time_t	e_atime = st_end->st_atime;

		/* Want atime to lie between s_time and e_time */
	  	if (req) {
			if (e_atime < s_time) {
				in_rpt("st_atime is %s", ctime(&e_atime));
				in_rpt("expected greater than %s", ctime(&s_time));
				xx_rpt(FAILURE);
				return;
			}
	
			if (e_atime > e_time) {
				in_rpt("st_atime is %s", ctime(&e_atime));
				in_rpt("expected less than or equal to %s", ctime(&e_time));
				xx_rpt(FAILURE);
				return;
			}
		} else {
			time_t	s_atime = st_start->st_atime;

			if (s_atime != e_atime) {
				in_rpt("st_atime changed from %s", ctime(&s_atime));
				in_rpt("to %s", ctime(&e_atime));
				xx_rpt(FAILURE);
				return;
			}
		}
	} else if(check == CHK_MTIME) {
		time_t	e_mtime = st_end->st_mtime;

		/* Want mtime to lie between s_time and e_time */
	  	if (req) {
			if (e_mtime < s_time) {
				in_rpt("st_mtime is %s", ctime(&e_mtime));
				in_rpt("expected greater than %s", ctime(&s_time));
				xx_rpt(FAILURE);
				return;
			}
	
			if (e_mtime > e_time) {
				in_rpt("st_mtime is %s", ctime(&e_mtime));
				in_rpt("expected less than or equal to %s", ctime(&e_time));
				xx_rpt(FAILURE);
				return;
			}
		} else {
			time_t	s_mtime = st_start->st_mtime;

			if (s_mtime != e_mtime) {
				in_rpt("st_mtime changed from %s", ctime(&s_mtime));
				in_rpt("to %s", ctime(&e_mtime));
				xx_rpt(FAILURE);
				return;
			}
		}
	} else if(check == CHK_CTIME) {
		time_t	e_ctime = st_end->st_ctime;

		/* Want ctime to lie between s_time and e_time */
	  	if (req) {
			if (e_ctime < s_time) {
				in_rpt("st_ctime is %s", ctime(&e_ctime));
				in_rpt("expected greater than %s", ctime(&s_time));
				xx_rpt(FAILURE);
				return;
			}
	
			if (e_ctime > e_time) {
				in_rpt("st_ctime is %s", ctime(&e_ctime));
				in_rpt("expected less than or equal to %s", ctime(&e_ctime));
				xx_rpt(FAILURE);
				return;
			}
		} else {
			time_t	s_ctime = st_start->st_ctime;

			if (s_ctime != e_ctime) {
				in_rpt("st_ctime changed from %s", ctime(&s_ctime));
				in_rpt("to %s", ctime(&e_ctime));
				xx_rpt(FAILURE);
				return;
			}
		}
	}
}
