/*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
* Project: LSB-OS
*
* File: SRC/common/vsrtlib/misc.c
*
* Modifications:
* $Log: misc.c,v $
* Revision 1.2  2001/10/02 11:59:05  ajosey
* clean up the various header comment blocks for lsb-os
*
* Revision 1.1  2001/09/28 08:21:02  cyeoh
* Move of lsb-os specific libraries to lsb-os test set
*
* Revision 1.1  2001/06/01 02:00:15  cyeoh
* Merges for lsb-os test suite
*
*/

#include <vsrt/vsrt.h>
#include <pcts_max.h>

private char    *names[2] = { NULL, NULL };
extern  char    *errname();

/*check for support of a function based on tetexec.cfg variables*/
void vsrt_check_support(char *, int *);
void vsrt_check_support(char *fname, int *supports)
{
	char	buf[128];
	char	*tgvret;

	memset(buf, 0, sizeof(buf));

	strcpy(buf, "VSRT_SUPPORTS_");
	strcat(buf, fname);
	tgvret = tet_getvar(buf);
	if ((tgvret == 0) || (strlen(tgvret) == 0) || (*tgvret == 'n')) {
		*supports = 0;
		return;
	}
	if (*tgvret == 'y') {
		*supports = 1;
		return;
	}
	*supports = 0;
	return;
}

static void vsrt_makename(char *, int);
static void vsrt_makename(char *filenam, int len)
{
	/* construct filename of len bytes */
	int i;

	for (i = 0; i < len; i++)
		filenam[i] = '0' + (i+1)%10;
	filenam[i] = '\0';
}

/*Analagous to the VSX nametoolng routine but uses the implementation's
location for shared memory objects rather than "." in determing
NAME_MAX*/
char **vsrt_nametoolng(void);
char **vsrt_nametoolng(void)
{
	char	*tshm;
	int	len1, len2;

	DBUG_LENTER("vsrt_nametoolng", 2);

	tshm = tet_getvar("VSRT_SHM_PREFIX");
	if ((tshm == 0) || (strlen(tshm) == 0)) {
		xx_rpt(FAILURE);
		in_rpt("vsrt_nametoolng: Parameter VSRT_SHM_PREFIX is not set in tetexec.cfg");
		DBUG_RETURN((char **)NULL);
	}
	if (pathconf(tshm, _PC_NO_TRUNC) < 0) {
		if (errno == 0) {
			xx_rpt(UNSUPPORTED);
			in_rpt("_POSIX_NO_TRUNC is not in effect");
			DBUG_RETURN((char **)NULL);
		}
		else {
			in_rpt("pathconf(\"VSRT_SHM_PREFIX\", _PC_NO_TRUNC) failed in vsrt_nametoolng(), errno = %d (%s)", errno, errname(errno));
			DBUG_PRINTF("return", "vsrt_nametoolng = NULL");
			DBUG_RETURN((char **)NULL);
		}
	}
	errno = 0;
	len1 = pathconf(tshm, _PC_NAME_MAX);
	if ((len1 == -1) && (errno != 0)) {
		in_rpt("pathconf(\"VSRT_SHM_PREFIX\", _PC_NAME_MAX) failed - errno %d (%s)", errno, errname(errno));
		DBUG_PRINTF("return", "vsrt_nametoolng = NULL");
		DBUG_RETURN((char **)NULL);
	}
	else if ((len1 != -1) && (len1 <= 0)) {
		in_rpt("pathconf(\"VSRT_SHM_PREFIX\", _PC_NAME_MAX) returned %d", len1);
		DBUG_PRINTF("return", "vsrt_nametoolng = NULL");
		DBUG_RETURN((char **)NULL);
	}

	if (len1 == -1 || len1 > PCTS_NAME_MAX) {
		len1 = PCTS_NAME_MAX;
		len2 = -1;
	}
	else
		len2 = len1 + 1;

	names[0] = (char *)malloc((size_t)(len1 + 1));
	if (len2 > 0)
		names[1] = (char *)malloc((size_t)(len2 + 1));
	if (names[0] == NULL || (len2 > 0 && names[1] == NULL)) {
		in_rpt("malloc() failed in vsrt_nametoolng()");
		DBUG_PRINTF("return", "vsrt_nametoolng = NULL");
		DBUG_RETURN((char **)NULL);
	}

	vsrt_makename(names[0], len1);
	if (len2 > 0)
		vsrt_makename(names[1], len2);

	DBUG_PRINTF("return", "vsrt_nametoolng = 0x%lx", (long)names);
	DBUG_RETURN(names);
}

void vsrt_clntoolng(void);
void vsrt_clntoolng(void)
{
       if (names[0] != NULL) {
		free((void *)names[0]);
		names[0] = NULL;
	}
	if (names[1] != NULL) {
		free((void *)names[1]);
		names[1] = NULL;
	}
}

void vsrt_sysfail(api)
char *api;
{
	int err;

	err = errno;
	in_rpt("%s failed, errno = %d (%s)", api, err, errname(err));
	xx_rpt(FAILURE);
}
