/*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
* Project: LSB-OS
*
* File: SRC/common/vsrtlib/mqueue.c
*
* Modifications:
* $Log: mqueue.c,v $
* Revision 1.4  2001/10/02 11:59:05  ajosey
* clean up the various header comment blocks for lsb-os
*
* Revision 1.3  2001/09/28 08:52:21  cyeoh
* restores file I shouldn't have deleted
*
* Revision 1.1  2001/06/01 02:00:15  cyeoh
* Merges for lsb-os test suite
*
*
*/

#include <vsrt/vsrt.h>
#include <pcts_max.h>

private char    *mq_names[2] = { NULL, NULL };
extern  char    *errname();

/*create a message queue name by prepending the used-defined path*/
/*name supplied must be at most 14 characters*/
int vsrt_get_mqueue_name(char *, char *, size_t);

int vsrt_get_mqueue_name(char *buf, char *name, size_t size_max)
{
	char	*tmqueue;

	DBUG_LENTER("vsrt_get_mqueue", 2);
	
	memset(buf, 0, size_max);
	tmqueue = tet_getvar("VSRT_MQUEUE_PREFIX");
	if ((tmqueue == 0) || (strlen(tmqueue) == 0)) {
		xx_rpt(FAILURE);
		in_rpt("vsrt_get_mqueue_name: Parameter VSRT_MQUEUE_PREFIX is not set in tetexec.cfg");
		DBUG_RETURN(-1);
	}

	if (strlen(tmqueue) + strlen(name) + 2 > size_max) {
		xx_rpt(FAILURE);
		in_rpt("vsrt_get_mqueue_name: VSRT_MQUEUE_PREFIX + name is too large\n");
		DBUG_RETURN(-1);
	}

	strcpy(buf, tmqueue);

	if (buf[strlen(buf)-1] != '/')
		strcat(buf, "/");

	strcat(buf, name);

	DBUG_RETURN(0);
}

static void vsrt_mq_makename(char *, int);
static void vsrt_mq_makename(char *filenam, int len)
{
	/* construct filename of len bytes */
	int i;

	for (i = 0; i < len; i++)
		filenam[i] = '0' + (i+1)%10;
	filenam[i] = '\0';
}

/*Analagous to the VSX nametoolng routine but uses the implementation's
location for message queues rather than "." in determing
NAME_MAX*/
char **vsrt_mq_nametoolng(void);
char **vsrt_mq_nametoolng(void)
{
	char	*tmq;
	int	len1, len2;

	DBUG_LENTER("vsrt_mq_nametoolng", 2);

	tmq = tet_getvar("VSRT_MQUEUE_PREFIX");
	if ((tmq == 0) || (strlen(tmq) == 0)) {
		xx_rpt(FAILURE);
		in_rpt("vsrt_get_mqueue_name: Parameter VSRT_MQUEUE_PREFIX is not set in tetexec.cfg");
		DBUG_RETURN((char **)NULL);
	}
	if (pathconf(tmq, _PC_NO_TRUNC) < 0) {
		if (errno == 0) {
			xx_rpt(UNSUPPORTED);
			in_rpt("_POSIX_NO_TRUNC is not in effect");
			DBUG_RETURN((char **)NULL);
		}
		else {
			in_rpt("pathconf(\"VSRT_MQUEUE_PREFIX\", _PC_NO_TRUNC) failed in vsrt_mq_nametoolng(), errno = %d (%s)", errno, errname(errno));
			DBUG_PRINTF("return", "vsrt_mq_nametoolng = NULL");
			DBUG_RETURN((char **)NULL);
		}
	}
	errno = 0;
	len1 = pathconf(tmq, _PC_NAME_MAX);
	if ((len1 == -1) && (errno != 0)) {
		in_rpt("pathconf(\"VSRT_MQUEUE_PREFIX\", _PC_NAME_MAX) failed - errno %d (%s)", errno, errname(errno));
		DBUG_PRINTF("return", "vsrt_mq_nametoolng = NULL");
		DBUG_RETURN((char **)NULL);
	}
	else if ((len1 != -1) && (len1 <= 0)) {
		in_rpt("pathconf(\"VSRT_MQUEUE_PREFIX\", _PC_NAME_MAX) returned %d", len1);
		DBUG_PRINTF("return", "vsrt_mq_nametoolng = NULL");
		DBUG_RETURN((char **)NULL);
	}

	if (len1 == -1 || len1 > PCTS_NAME_MAX) {
		len1 = PCTS_NAME_MAX;
		len2 = -1;
	}
	else
		len2 = len1 + 1;

	mq_names[0] = (char *)malloc((size_t)(len1 + 1));
	if (len2 > 0)
		mq_names[1] = (char *)malloc((size_t)(len2 + 1));
	if (mq_names[0] == NULL || (len2 > 0 && mq_names[1] == NULL)) {
		in_rpt("malloc() failed in vsrt_mq_nametoolng()");
		DBUG_PRINTF("return", "vsrt_mq_nametoolng = NULL");
		DBUG_RETURN((char **)NULL);
	}

	vsrt_mq_makename(mq_names[0], len1);
	if (len2 > 0)
		vsrt_mq_makename(mq_names[1], len2);

	DBUG_PRINTF("return", "vsrt_mq_nametoolng = 0x%lx", (long)mq_names);
	DBUG_RETURN(mq_names);
}

void vsrt_mq_clntoolng(void);
void vsrt_mq_clntoolng(void)
{
       if (mq_names[0] != NULL) {
		free((void *)mq_names[0]);
		mq_names[0] = NULL;
	}
	if (mq_names[1] != NULL) {
		free((void *)mq_names[1]);
		mq_names[1] = NULL;
	}
}
