/*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
* Project: LSB-OS
*
* File: SRC/common/vsrtlib/waitfor.c
*
* Modifications:
* $Log: signals.c,v $
* Revision 1.2  2001/10/02 11:59:05  ajosey
* clean up the various header comment blocks for lsb-os
*
* Revision 1.1  2001/09/28 08:21:02  cyeoh
* Move of lsb-os specific libraries to lsb-os test set
*
* Revision 1.1  2001/06/01 02:00:15  cyeoh
* Merges for lsb-os test suite
*
*
*/

#include <vsrt/vsrt.h>

extern     char    *errname(int);

int              got_sigalrm;
unsigned         unused_time;
struct sigaction Sigaction;

void vsrt_lsah(int sig)
{
	got_sigalrm++;
}


int vsrt_signal(int sig, void (*handler)(int))
{
	int	rval;

        rval = sigemptyset(&(Sigaction.sa_mask));
	if (rval == -1) {
		in_rpt("vsrt_signal: sigemptyset failed, errno = %s", errname(errno));
		xx_rpt(FAILURE);
		return -1;
	}
        Sigaction.sa_handler = handler;
        Sigaction.sa_flags = 0;
        rval = sigaction(sig,&Sigaction,(struct sigaction *)NULL);
	if (rval == -1) {
		in_rpt("vsrt_signal: sigaction failed for signal %d, errno = %s", sig, errname(errno));
		xx_rpt(FAILURE);
		return -1;
	}
	return 0;
}


int vsrt_rt_signal(int sig, void (*handler)(int, siginfo_t *, void *))
{
	int	rval;

        rval = sigemptyset(&(Sigaction.sa_mask));
	if (rval == -1) {
		in_rpt("vsrt_signal: sigemptyset failed, errno = %s", errname(errno));
		xx_rpt(FAILURE);
		return -1;
	}
        Sigaction.sa_sigaction = handler;
        Sigaction.sa_flags = SA_SIGINFO;
        rval = sigaction(sig,&Sigaction,(struct sigaction *)NULL);
	if (rval == -1) {
		in_rpt("vsrt_signal: sigaction failed for signal %d, errno = %s", sig, errname(errno));
		xx_rpt(FAILURE);
		return -1;
	}
	return 0;
}

int vsrt_set_alarm(int my_time)
{

	got_sigalrm = 0;
        if(vsrt_signal(SIGALRM, vsrt_lsah) == -1) {
		return -1;
	}
	return alarm((unsigned int)my_time);
}


int vsrt_clear_alarm(void)
{

	unused_time = alarm(0);
	if(vsrt_signal(SIGALRM, SIG_DFL) == -1) {
		return -1;
	}
	
	if (got_sigalrm != 0) {
		in_rpt("vsrt_clear_alarm: alarm expired awaiting completion");
		xx_rpt(FAILURE);
		return(-1);
	} 
	return 0;
}


int vsrt_expect_alarm(void)
{

	unused_time = alarm(0);
	if(vsrt_signal(SIGALRM,SIG_DFL) == -1) {
		return -1;
	}
	if (got_sigalrm == 0) {
		in_rpt("vsrt_expect_alarm: Alarm did not expire as expected");
		xx_rpt(FAILURE);
		return -1;
	} 
	return 0;
}


int vsrt_ignore_alarm(void)
{
	unused_time = alarm(0);
	return vsrt_signal(SIGALRM,SIG_DFL);                
}
