/*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
* Project: LSB-OS
*
* File: SRC/common/vsrtlib/waitfor.c
*
* Modifications:
* $Log: waitfor.c,v $
* Revision 1.2  2001/10/02 11:59:05  ajosey
* clean up the various header comment blocks for lsb-os
*
* Revision 1.1  2001/09/28 08:21:02  cyeoh
* Move of lsb-os specific libraries to lsb-os test set
*
* Revision 1.1  2001/06/01 02:00:15  cyeoh
* Merges for lsb-os test suite
*
*/

#include <vsrt/vsrt.h>

/*
** Wait for a child process to exit, then return child's process status.
**
** Arguments
**	pid		Process ID of the child we are waiting for.
**	timeout		Maximum time (in seconds) to wait.
*/

int vsrt_wait_for(pid_t pid, int timeout)
{
	int	child, save_errno;
	pid_t	ret_val;

	if (vsrt_set_alarm(timeout) == -1) {
		return -1;
	}

	errno=0;
	ret_val = waitpid(pid, &child, 0);
	save_errno = errno;

	unused_time = alarm(0);

	if(vsrt_signal(SIGALRM, SIG_DFL) == -1) {
		return -1;
	}

	if (got_sigalrm) {
		(void) kill(pid, SIGKILL);

		sleep(2);

		(void) waitpid(pid, &child, WNOHANG);
		in_rpt("vsrt_wait_for: Timed-out waiting for child to exit");
		xx_rpt(FAILURE);
		return (-1);
	}

	if (ret_val == (pid_t) -1) {
		in_rpt("vsrt_wait_for: wait returned -1, errno=%d", save_errno);
		xx_rpt(FAILURE);
		return (-1);
	}

	return (child);
}
