
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_cp.c	1.18
*
*************************************************************************/


#include <stdio.h>
#include <unistd.h>

#include "std.h"

#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"
#include "vsth/vsth_cp.h"

/******	A short note on the rational for the way we do CP testing ...

	The POSIX standard, when specifying the set of functions which
	must/must_not provide a cancellation point, does not specify
	a well defined subset of possible invocations for which CP condition
	testing must be performed. As a result, we conclude that implementors
	must cover all possible invocations and all possible paths thru the
	function under test, including those leading to error returns, when
	determining cancellability.

	As a result, we often employ error paths thru the function under
	test to determine whether they provide a cancellation point.	******/


VSTH_RT	Vsth_cp6[] = {
#if	defined(_POSIX_ASYNCHRONOUS_IO)
	{	aio_suspend_csf,	aio_suspend_ctf,	aio_suspend_cef,	"aio_suspend"	},
#endif
	{	close_csf,	close_ctf,	close_cef,	"close"	},
	{	creat_csf,	creat_ctf,	creat_cef,	"creat"	},
	{	fcntl_csf,	fcntl_ctf,	fcntl_cef,	"fcntl"	},
#if	defined(_POSIX_FSYNC)
	{	fsync_csf,	fsync_ctf,	fsync_cef,	"fsync"	},
#endif
#if	defined(_POSIX_MESSAGE_PASSING)
	{	mq_receive_csf,	mq_receive_ctf,	mq_receive_cef,	"mq_receive"	},
#endif
#if	defined(_POSIX_MESSAGE_PASSING)
	{	mq_send_csf,	mq_send_ctf,	mq_send_cef,	"mq_send"	},
#endif
#if	defined(_POSIX_MAPPED_FILES) && defined(_POSIX_SYNCHRONIZED_IO)
	{	msync_csf,	msync_ctf,	msync_cef,	"msync"	},
#endif
#if	defined(_POSIX_TIMERS)
	{	nanosleep_csf,	nanosleep_ctf,	nanosleep_cef,	"nanosleep"	},
#endif
	{	open_csf,	open_ctf,	open_cef,	"open"	},
	{	pause_csf,	pause_ctf,	pause_cef,	"pause"	},
#if	defined(_POSIX_THREADS)
	{	pthread_cond_timedwait_csf,	pthread_cond_timedwait_ctf,	pthread_cond_timedwait_cef,	"pthread_cond_timedwait"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_cond_wait_csf,	pthread_cond_wait_ctf,	pthread_cond_wait_cef,	"pthread_cond_wait"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_join_csf,	pthread_join_ctf,	pthread_join_cef,	"pthread_join"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_testcancel_csf,	pthread_testcancel_ctf,	pthread_testcancel_cef,	"pthread_testcancel"	},
#endif
	{	read_csf,	read_ctf,	read_cef,	"read"	},
#if	defined(_POSIX_SEMAPHORES)
	{	sem_wait_csf,	sem_wait_ctf,	sem_wait_cef,	"sem_wait"	},
#endif
	{	sigsuspend_csf,	sigsuspend_ctf,	sigsuspend_cef,	"sigsuspend"	},
#ifdef	_POSIX_REALTIME_SIGNALS
	{	sigtimedwait_csf,	sigtimedwait_ctf,	sigtimedwait_cef,	"sigtimedwait"	},
#endif
	{	sigwait_csf,	sigwait_ctf,	sigwait_cef,	"sigwait"	},
#ifdef	_POSIX_REALTIME_SIGNALS
	{	sigwaitinfo_csf,	sigwaitinfo_ctf,	sigwaitinfo_cef,	"sigwaitinfo"	},
#endif
	{	sleep_csf,	sleep_ctf,	sleep_cef,	"sleep"	},
	{	system_csf,	system_ctf,	system_cef,	"system"	},
	{	tcdrain_csf,	tcdrain_ctf,	tcdrain_cef,	"tcdrain"	},
	{	wait_csf,	wait_ctf,	wait_cef,	"wait"	},
	{	waitpid_csf,	waitpid_ctf,	waitpid_cef,	"waitpid"	},
	{	write_csf,	write_ctf,	write_cef,	"write"	},
	{	NULL,	NULL,	NULL,	NULL	}
};

VSTH_RT	Vsth_cp7[] = {
	{	_exit_csf,	NULL,	_exit_cef,	"_exit"	},
	{	abort_csf,	NULL,	abort_cef,	"abort"	},
	{	abs_csf,	abs_ctf,	abs_cef,	"abs"	},
	{	access_csf,	access_ctf,	access_cef,	"access"	},
	{	acos_csf,	acos_ctf,	acos_cef,	"acos"	},
#if	defined(_POSIX_ASYNCHRONOUS_IO)
	{	aio_cancel_csf,	aio_cancel_ctf,	aio_cancel_cef,	"aio_cancel"	},
#endif
#if	defined(_POSIX_ASYNCHRONOUS_IO)
	{	aio_error_csf,	aio_error_ctf,	aio_error_cef,	"aio_error"	},
#endif
#if	defined(_POSIX_ASYNCHRONOUS_IO) && defined(_POSIX_SYNCHRONIZED_IO)
	{	aio_fsync_csf,	aio_fsync_ctf,	aio_fsync_cef,	"aio_fsync"	},
#endif
#if	defined(_POSIX_ASYNCHRONOUS_IO)
	{	aio_read_csf,	aio_read_ctf,	aio_read_cef,	"aio_read"	},
#endif
#if	defined(_POSIX_ASYNCHRONOUS_IO)
	{	aio_return_csf,	aio_return_ctf,	aio_return_cef,	"aio_return"	},
#endif
#if	defined(_POSIX_ASYNCHRONOUS_IO)
	{	aio_write_csf,	aio_write_ctf,	aio_write_cef,	"aio_write"	},
#endif
	{	alarm_csf,	alarm_ctf,	alarm_cef,	"alarm"	},
	{	asctime_csf,	asctime_ctf,	asctime_cef,	"asctime"	},
	{	asin_csf,	asin_ctf,	asin_cef,	"asin"	},
	{	assert_csf,	assert_ctf,	assert_cef,	"assert"	},
	{	atan_csf,	atan_ctf,	atan_cef,	"atan"	},
	{	atan2_csf,	atan2_ctf,	atan2_cef,	"atan2"	},
	{	atexit_csf,	atexit_ctf,	atexit_cef,	"atexit"	},
	{	atof_csf,	atof_ctf,	atof_cef,	"atof"	},
	{	atoi_csf,	atoi_ctf,	atoi_cef,	"atoi"	},
	{	atol_csf,	atol_ctf,	atol_cef,	"atol"	},
	{	bsearch_csf,	bsearch_ctf,	bsearch_cef,	"bsearch"	},
	{	calloc_csf,	calloc_ctf,	calloc_cef,	"calloc"	},
	{	ceil_csf,	ceil_ctf,	ceil_cef,	"ceil"	},
	{	cfgetispeed_csf,	cfgetispeed_ctf,	cfgetispeed_cef,	"cfgetispeed"	},
	{	cfgetospeed_csf,	cfgetospeed_ctf,	cfgetospeed_cef,	"cfgetospeed"	},
	{	cfsetispeed_csf,	cfsetispeed_ctf,	cfsetispeed_cef,	"cfsetispeed"	},
	{	cfsetospeed_csf,	cfsetospeed_ctf,	cfsetospeed_cef,	"cfsetospeed"	},
	{	chdir_csf,	chdir_ctf,	chdir_cef,	"chdir"	},
	{	chmod_csf,	chmod_ctf,	chmod_cef,	"chmod"	},
	{	chown_csf,	chown_ctf,	chown_cef,	"chown"	},
	{	clearerr_csf,	clearerr_ctf,	clearerr_cef,	"clearerr"	},
	{	clock_csf,	clock_ctf,	clock_cef,	"clock"	},
#if	defined(_POSIX_TIMERS)
	{	clock_getres_csf,	clock_getres_ctf,	clock_getres_cef,	"clock_getres"	},
#endif
#if	defined(_POSIX_TIMERS)
	{	clock_gettime_csf,	clock_gettime_ctf,	clock_gettime_cef,	"clock_gettime"	},
#endif
#if	defined(_POSIX_TIMERS)
	{	clock_settime_csf,	clock_settime_ctf,	clock_settime_cef,	"clock_settime"	},
#endif
	{	cos_csf,	cos_ctf,	cos_cef,	"cos"	},
	{	cosh_csf,	cosh_ctf,	cosh_cef,	"cosh"	},
	{	ctime_csf,	ctime_ctf,	ctime_cef,	"ctime"	},
#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)
	{	ctime_r_csf,	ctime_r_ctf,	ctime_r_cef,	"ctime_r"	},
#endif
	{	difftime_csf,	difftime_ctf,	difftime_cef,	"difftime"	},
	{	div_csf,	div_ctf,	div_cef,	"div"	},
	{	dup_csf,	dup_ctf,	dup_cef,	"dup"	},
	{	dup2_csf,	dup2_ctf,	dup2_cef,	"dup2"	},
	{	execl_csf,	execl_ctf,	execl_cef,	"execl"	},
	{	execle_csf,	execle_ctf,	execle_cef,	"execle"	},
	{	execlp_csf,	execlp_ctf,	execlp_cef,	"execlp"	},
	{	execv_csf,	execv_ctf,	execv_cef,	"execv"	},
	{	execve_csf,	execve_ctf,	execve_cef,	"execve"	},
	{	execvp_csf,	execvp_ctf,	execvp_cef,	"execvp"	},
	{	exit_csf,	NULL,	exit_cef,	"exit"	},
	{	exp_csf,	exp_ctf,	exp_cef,	"exp"	},
	{	fabs_csf,	fabs_ctf,	fabs_cef,	"fabs"	},
	{	fchmod_csf,	fchmod_ctf,	fchmod_cef,	"fchmod"	},
#if	defined(_POSIX_SYNCHRONIZED_IO)
	{	fdatasync_csf,	fdatasync_ctf,	fdatasync_cef,	"fdatasync"	},
#endif
	{	fdopen_csf,	fdopen_ctf,	fdopen_cef,	"fdopen"	},
	{	feof_csf,	feof_ctf,	feof_cef,	"feof"	},
	{	ferror_csf,	ferror_ctf,	ferror_cef,	"ferror"	},
	{	fgetpos_csf,	fgetpos_ctf,	fgetpos_cef,	"fgetpos"	},
	{	fileno_csf,	fileno_ctf,	fileno_cef,	"fileno"	},
#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)
	{	flockfile_csf,	flockfile_ctf,	flockfile_cef,	"flockfile"	},
#endif
	{	floor_csf,	floor_ctf,	floor_cef,	"floor"	},
	{	fmod_csf,	fmod_ctf,	fmod_cef,	"fmod"	},
	{	fork_csf,	fork_ctf,	fork_cef,	"fork"	},
	{	fpathconf_csf,	fpathconf_ctf,	fpathconf_cef,	"fpathconf"	},
	{	free_csf,	free_ctf,	free_cef,	"free"	},
	{	frexp_csf,	frexp_ctf,	frexp_cef,	"frexp"	},
	{	fsetpos_csf,	fsetpos_ctf,	fsetpos_cef,	"fsetpos"	},
	{	fstat_csf,	fstat_ctf,	fstat_cef,	"fstat"	},
	{	ftruncate_csf,	ftruncate_ctf,	ftruncate_cef,	"ftruncate"	},
#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)
	{	ftrylockfile_csf,	ftrylockfile_ctf,	ftrylockfile_cef,	"ftrylockfile"	},
#endif
#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)
	{	funlockfile_csf,	funlockfile_ctf,	funlockfile_cef,	"funlockfile"	},
#endif
	{	getegid_csf,	getegid_ctf,	getegid_cef,	"getegid"	},
	{	getenv_csf,	getenv_ctf,	getenv_cef,	"getenv"	},
	{	geteuid_csf,	geteuid_ctf,	geteuid_cef,	"geteuid"	},
	{	getgid_csf,	getgid_ctf,	getgid_cef,	"getgid"	},
	{	getgroups_csf,	getgroups_ctf,	getgroups_cef,	"getgroups"	},
	{	getpgrp_csf,	getpgrp_ctf,	getpgrp_cef,	"getpgrp"	},
	{	getpid_csf,	getpid_ctf,	getpid_cef,	"getpid"	},
	{	getppid_csf,	getppid_ctf,	getppid_cef,	"getppid"	},
	{	getuid_csf,	getuid_ctf,	getuid_cef,	"getuid"	},
	{	gmtime_csf,	gmtime_ctf,	gmtime_cef,	"gmtime"	},
#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)
	{	gmtime_r_csf,	gmtime_r_ctf,	gmtime_r_cef,	"gmtime_r"	},
#endif
	{	isalnum_csf,	isalnum_ctf,	isalnum_cef,	"isalnum"	},
	{	isalpha_csf,	isalpha_ctf,	isalpha_cef,	"isalpha"	},
	{	isascii_csf,	isascii_ctf,	isascii_cef,	"isascii"	},
	{	isatty_csf,	isatty_ctf,	isatty_cef,	"isatty"	},
	{	iscntrl_csf,	iscntrl_ctf,	iscntrl_cef,	"iscntrl"	},
	{	isdigit_csf,	isdigit_ctf,	isdigit_cef,	"isdigit"	},
	{	isgraph_csf,	isgraph_ctf,	isgraph_cef,	"isgraph"	},
	{	islower_csf,	islower_ctf,	islower_cef,	"islower"	},
	{	isprint_csf,	isprint_ctf,	isprint_cef,	"isprint"	},
	{	ispunct_csf,	ispunct_ctf,	ispunct_cef,	"ispunct"	},
	{	isspace_csf,	isspace_ctf,	isspace_cef,	"isspace"	},
	{	isupper_csf,	isupper_ctf,	isupper_cef,	"isupper"	},
	{	kill_csf,	kill_ctf,	kill_cef,	"kill"	},
	{	labs_csf,	labs_ctf,	labs_cef,	"labs"	},
	{	ldexp_csf,	ldexp_ctf,	ldexp_cef,	"ldexp"	},
	{	ldiv_csf,	ldiv_ctf,	ldiv_cef,	"ldiv"	},
	{	link_csf,	link_ctf,	link_cef,	"link"	},
#if	defined(_POSIX_ASYNCHRONOUS_IO)
	{	lio_listio_csf,	lio_listio_ctf,	lio_listio_cef,	"lio_listio"	},
#endif
	{	localtime_csf,	localtime_ctf,	localtime_cef,	"localtime"	},
#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)
	{	localtime_r_csf,	localtime_r_ctf,	localtime_r_cef,	"localtime_r"	},
#endif
	{	log_csf,	log_ctf,	log_cef,	"log"	},
	{	log10_csf,	log10_ctf,	log10_cef,	"log10"	},
	{	longjmp_csf,	NULL,	longjmp_cef,	"longjmp"	},
	{	malloc_csf,	malloc_ctf,	malloc_cef,	"malloc"	},
	{	mblen_csf,	mblen_ctf,	mblen_cef,	"mblen"	},
	{	mbtowc_csf,	mbtowc_ctf,	mbtowc_cef,	"mbtowc"	},
	{	memchr_csf,	memchr_ctf,	memchr_cef,	"memchr"	},
	{	memcmp_csf,	memcmp_ctf,	memcmp_cef,	"memcmp"	},
	{	memcpy_csf,	memcpy_ctf,	memcpy_cef,	"memcpy"	},
	{	memmove_csf,	memmove_ctf,	memmove_cef,	"memmove"	},
	{	memset_csf,	memset_ctf,	memset_cef,	"memset"	},
	{	mkdir_csf,	mkdir_ctf,	mkdir_cef,	"mkdir"	},
	{	mkfifo_csf,	mkfifo_ctf,	mkfifo_cef,	"mkfifo"	},
	{	mktime_csf,	mktime_ctf,	mktime_cef,	"mktime"	},
#if	defined(_POSIX_MEMLOCK_RANGE)
	{	mlock_csf,	mlock_ctf,	mlock_cef,	"mlock"	},
#endif
#if	defined(_POSIX_MEMLOCK)
	{	mlockall_csf,	mlockall_ctf,	mlockall_cef,	"mlockall"	},
#endif
#if	defined(_POSIX_MAPPED_FILES) || defined(_POSIX_SHARED_MEMORY_OBJECTS)
	{	mmap_csf,	mmap_ctf,	mmap_cef,	"mmap"	},
#endif
	{	modf_csf,	modf_ctf,	modf_cef,	"modf"	},
#if	defined(_POSIX_MEMORY_PROTECTION)
	{	mprotect_csf,	mprotect_ctf,	mprotect_cef,	"mprotect"	},
#endif
#if	defined(_POSIX_MESSAGE_PASSING)
	{	mq_close_csf,	mq_close_ctf,	mq_close_cef,	"mq_close"	},
#endif
#if	defined(_POSIX_MESSAGE_PASSING)
	{	mq_getattr_csf,	mq_getattr_ctf,	mq_getattr_cef,	"mq_getattr"	},
#endif
#if	defined(_POSIX_MESSAGE_PASSING) && defined(_POSIX_REALTIME_SIGNALS)
	{	mq_notify_csf,	mq_notify_ctf,	mq_notify_cef,	"mq_notify"	},
#endif
#if	defined(_POSIX_MESSAGE_PASSING)
	{	mq_open_csf,	mq_open_ctf,	mq_open_cef,	"mq_open"	},
#endif
#if	defined(_POSIX_MESSAGE_PASSING)
	{	mq_setattr_csf,	mq_setattr_ctf,	mq_setattr_cef,	"mq_setattr"	},
#endif
#if	defined(_POSIX_MESSAGE_PASSING)
	{	mq_unlink_csf,	mq_unlink_ctf,	mq_unlink_cef,	"mq_unlink"	},
#endif
#if	defined(_POSIX_MEMLOCK_RANGE)
	{	munlock_csf,	munlock_ctf,	munlock_cef,	"munlock"	},
#endif
#if	defined(_POSIX_MEMLOCK)
	{	munlockall_csf,	munlockall_ctf,	munlockall_cef,	"munlockall"	},
#endif
#if	defined(_POSIX_MAPPED_FILES) || defined(_POSIX_SHARED_MEMORY_OBJECTS)
	{	munmap_csf,	munmap_ctf,	munmap_cef,	"munmap"	},
#endif
	{	pathconf_csf,	pathconf_ctf,	pathconf_cef,	"pathconf"	},
	{	pipe_csf,	pipe_ctf,	pipe_cef,	"pipe"	},
	{	pow_csf,	pow_ctf,	pow_cef,	"pow"	},
#if	defined(_POSIX_THREADS)
	{	pthread_atfork_csf,	pthread_atfork_ctf,	pthread_atfork_cef,	"pthread_atfork"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_attr_destroy_csf,	pthread_attr_destroy_ctf,	pthread_attr_destroy_cef,	"pthread_attr_destroy"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_attr_getdetachstate_csf,	pthread_attr_getdetachstate_ctf,	pthread_attr_getdetachstate_cef,	"pthread_attr_getdetachstate"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)
	{	pthread_attr_getinheritsched_csf,	pthread_attr_getinheritsched_ctf,	pthread_attr_getinheritsched_cef,	"pthread_attr_getinheritsched"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_attr_getschedparam_csf,	pthread_attr_getschedparam_ctf,	pthread_attr_getschedparam_cef,	"pthread_attr_getschedparam"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)
	{	pthread_attr_getschedpolicy_csf,	pthread_attr_getschedpolicy_ctf,	pthread_attr_getschedpolicy_cef,	"pthread_attr_getschedpolicy"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)
	{	pthread_attr_getscope_csf,	pthread_attr_getscope_ctf,	pthread_attr_getscope_cef,	"pthread_attr_getscope"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_ATTR_STACKADDR)
	{	pthread_attr_getstackaddr_csf,	pthread_attr_getstackaddr_ctf,	pthread_attr_getstackaddr_cef,	"pthread_attr_getstackaddr"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_ATTR_STACKSIZE)
	{	pthread_attr_getstacksize_csf,	pthread_attr_getstacksize_ctf,	pthread_attr_getstacksize_cef,	"pthread_attr_getstacksize"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_attr_init_csf,	pthread_attr_init_ctf,	pthread_attr_init_cef,	"pthread_attr_init"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_attr_setdetachstate_csf,	pthread_attr_setdetachstate_ctf,	pthread_attr_setdetachstate_cef,	"pthread_attr_setdetachstate"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)
	{	pthread_attr_setinheritsched_csf,	pthread_attr_setinheritsched_ctf,	pthread_attr_setinheritsched_cef,	"pthread_attr_setinheritsched"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_attr_setschedparam_csf,	pthread_attr_setschedparam_ctf,	pthread_attr_setschedparam_cef,	"pthread_attr_setschedparam"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)
	{	pthread_attr_setschedpolicy_csf,	pthread_attr_setschedpolicy_ctf,	pthread_attr_setschedpolicy_cef,	"pthread_attr_setschedpolicy"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)
	{	pthread_attr_setscope_csf,	pthread_attr_setscope_ctf,	pthread_attr_setscope_cef,	"pthread_attr_setscope"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_ATTR_STACKADDR)
	{	pthread_attr_setstackaddr_csf,	pthread_attr_setstackaddr_ctf,	pthread_attr_setstackaddr_cef,	"pthread_attr_setstackaddr"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_ATTR_STACKSIZE)
	{	pthread_attr_setstacksize_csf,	pthread_attr_setstacksize_ctf,	pthread_attr_setstacksize_cef,	"pthread_attr_setstacksize"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_cancel_csf,	pthread_cancel_ctf,	pthread_cancel_cef,	"pthread_cancel"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_cleanup_pop_csf,	pthread_cleanup_pop_ctf,	pthread_cleanup_pop_cef,	"pthread_cleanup_pop"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_cleanup_push_csf,	pthread_cleanup_push_ctf,	pthread_cleanup_push_cef,	"pthread_cleanup_push"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_cond_broadcast_csf,	pthread_cond_broadcast_ctf,	pthread_cond_broadcast_cef,	"pthread_cond_broadcast"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_cond_destroy_csf,	pthread_cond_destroy_ctf,	pthread_cond_destroy_cef,	"pthread_cond_destroy"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_cond_init_csf,	pthread_cond_init_ctf,	pthread_cond_init_cef,	"pthread_cond_init"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_cond_signal_csf,	pthread_cond_signal_ctf,	pthread_cond_signal_cef,	"pthread_cond_signal"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_condattr_destroy_csf,	pthread_condattr_destroy_ctf,	pthread_condattr_destroy_cef,	"pthread_condattr_destroy"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PROCESS_SHARED)
	{	pthread_condattr_getpshared_csf,	pthread_condattr_getpshared_ctf,	pthread_condattr_getpshared_cef,	"pthread_condattr_getpshared"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_condattr_init_csf,	pthread_condattr_init_ctf,	pthread_condattr_init_cef,	"pthread_condattr_init"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PROCESS_SHARED)
	{	pthread_condattr_setpshared_csf,	pthread_condattr_setpshared_ctf,	pthread_condattr_setpshared_cef,	"pthread_condattr_setpshared"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_create_csf,	pthread_create_ctf,	pthread_create_cef,	"pthread_create"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_detach_csf,	pthread_detach_ctf,	pthread_detach_cef,	"pthread_detach"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_equal_csf,	pthread_equal_ctf,	pthread_equal_cef,	"pthread_equal"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_exit_csf,	pthread_exit_ctf,	pthread_exit_cef,	"pthread_exit"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)
	{	pthread_getschedparam_csf,	pthread_getschedparam_ctf,	pthread_getschedparam_cef,	"pthread_getschedparam"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_getspecific_csf,	pthread_getspecific_ctf,	pthread_getspecific_cef,	"pthread_getspecific"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_key_create_csf,	pthread_key_create_ctf,	pthread_key_create_cef,	"pthread_key_create"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_key_delete_csf,	pthread_key_delete_ctf,	pthread_key_delete_cef,	"pthread_key_delete"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_kill_csf,	pthread_kill_ctf,	pthread_kill_cef,	"pthread_kill"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_mutex_destroy_csf,	pthread_mutex_destroy_ctf,	pthread_mutex_destroy_cef,	"pthread_mutex_destroy"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIO_PROTECT)
	{	pthread_mutex_getprioceiling_csf,	pthread_mutex_getprioceiling_ctf,	pthread_mutex_getprioceiling_cef,	"pthread_mutex_getprioceiling"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_mutex_init_csf,	pthread_mutex_init_ctf,	pthread_mutex_init_cef,	"pthread_mutex_init"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_mutex_lock_csf,	pthread_mutex_lock_ctf,	pthread_mutex_lock_cef,	"pthread_mutex_lock"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIO_PROTECT)
	{	pthread_mutex_setprioceiling_csf,	pthread_mutex_setprioceiling_ctf,	pthread_mutex_setprioceiling_cef,	"pthread_mutex_setprioceiling"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_mutex_trylock_csf,	pthread_mutex_trylock_ctf,	pthread_mutex_trylock_cef,	"pthread_mutex_trylock"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_mutex_unlock_csf,	pthread_mutex_unlock_ctf,	pthread_mutex_unlock_cef,	"pthread_mutex_unlock"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_mutexattr_destroy_csf,	pthread_mutexattr_destroy_ctf,	pthread_mutexattr_destroy_cef,	"pthread_mutexattr_destroy"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIO_PROTECT)
	{	pthread_mutexattr_getprioceiling_csf,	pthread_mutexattr_getprioceiling_ctf,	pthread_mutexattr_getprioceiling_cef,	"pthread_mutexattr_getprioceiling"	},
#endif
#if	defined(_POSIX_THREADS) && (defined(_POSIX_THREAD_PRIO_INHERIT) || defined(_POSIX_THREAD_PRIO_PROTECT))
	{	pthread_mutexattr_getprotocol_csf,	pthread_mutexattr_getprotocol_ctf,	pthread_mutexattr_getprotocol_cef,	"pthread_mutexattr_getprotocol"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PROCESS_SHARED)
	{	pthread_mutexattr_getpshared_csf,	pthread_mutexattr_getpshared_ctf,	pthread_mutexattr_getpshared_cef,	"pthread_mutexattr_getpshared"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_mutexattr_init_csf,	pthread_mutexattr_init_ctf,	pthread_mutexattr_init_cef,	"pthread_mutexattr_init"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIO_PROTECT)
	{	pthread_mutexattr_setprioceiling_csf,	pthread_mutexattr_setprioceiling_ctf,	pthread_mutexattr_setprioceiling_cef,	"pthread_mutexattr_setprioceiling"	},
#endif
#if	defined(_POSIX_THREADS) && (defined(_POSIX_THREAD_PRIO_INHERIT) || defined(_POSIX_THREAD_PRIO_PROTECT))
	{	pthread_mutexattr_setprotocol_csf,	pthread_mutexattr_setprotocol_ctf,	pthread_mutexattr_setprotocol_cef,	"pthread_mutexattr_setprotocol"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PROCESS_SHARED)
	{	pthread_mutexattr_setpshared_csf,	pthread_mutexattr_setpshared_ctf,	pthread_mutexattr_setpshared_cef,	"pthread_mutexattr_setpshared"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_once_csf,	pthread_once_ctf,	pthread_once_cef,	"pthread_once"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_self_csf,	pthread_self_ctf,	pthread_self_cef,	"pthread_self"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_setcancelstate_csf,	pthread_setcancelstate_ctf,	pthread_setcancelstate_cef,	"pthread_setcancelstate"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_setcanceltype_csf,	pthread_setcanceltype_ctf,	pthread_setcanceltype_cef,	"pthread_setcanceltype"	},
#endif
#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)
	{	pthread_setschedparam_csf,	pthread_setschedparam_ctf,	pthread_setschedparam_cef,	"pthread_setschedparam"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_setspecific_csf,	pthread_setspecific_ctf,	pthread_setspecific_cef,	"pthread_setspecific"	},
#endif
#if	defined(_POSIX_THREADS)
	{	pthread_sigmask_csf,	pthread_sigmask_ctf,	pthread_sigmask_cef,	"pthread_sigmask"	},
#endif
	{	qsort_csf,	qsort_ctf,	qsort_cef,	"qsort"	},
	{	raise_csf,	raise_ctf,	raise_cef,	"raise"	},
	{	rand_csf,	rand_ctf,	rand_cef,	"rand"	},
#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)
	{	rand_r_csf,	rand_r_ctf,	rand_r_cef,	"rand_r"	},
#endif
	{	readdir_csf,	readdir_ctf,	readdir_cef,	"readdir"	},
#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)
	{	readdir_r_csf,	readdir_r_ctf,	readdir_r_cef,	"readdir_r"	},
#endif
	{	realloc_csf,	realloc_ctf,	realloc_cef,	"realloc"	},
	{	rmdir_csf,	rmdir_ctf,	rmdir_cef,	"rmdir"	},
#if	defined(_POSIX_PRIORITY_SCHEDULING)
	{	sched_get_priority_max_csf,	sched_get_priority_max_ctf,	sched_get_priority_max_cef,	"sched_get_priority_max"	},
#endif
#if	defined(_POSIX_PRIORITY_SCHEDULING)
	{	sched_get_priority_min_csf,	sched_get_priority_min_ctf,	sched_get_priority_min_cef,	"sched_get_priority_min"	},
#endif
#if	defined(_POSIX_PRIORITY_SCHEDULING)
	{	sched_getparam_csf,	sched_getparam_ctf,	sched_getparam_cef,	"sched_getparam"	},
#endif
#if	defined(_POSIX_PRIORITY_SCHEDULING)
	{	sched_getscheduler_csf,	sched_getscheduler_ctf,	sched_getscheduler_cef,	"sched_getscheduler"	},
#endif
#if	defined(_POSIX_PRIORITY_SCHEDULING)
	{	sched_rr_get_interval_csf,	sched_rr_get_interval_ctf,	sched_rr_get_interval_cef,	"sched_rr_get_interval"	},
#endif
#if	defined(_POSIX_PRIORITY_SCHEDULING)
	{	sched_setparam_csf,	sched_setparam_ctf,	sched_setparam_cef,	"sched_setparam"	},
#endif
#if	defined(_POSIX_PRIORITY_SCHEDULING)
	{	sched_setscheduler_csf,	sched_setscheduler_ctf,	sched_setscheduler_cef,	"sched_setscheduler"	},
#endif
#if	defined(_POSIX_THREADS) || defined(_POSIX_PRIORITY_SCHEDULING)
	{	sched_yield_csf,	sched_yield_ctf,	sched_yield_cef,	"sched_yield"	},
#endif
#if	defined(_POSIX_SEMAPHORES)
	{	sem_close_csf,	sem_close_ctf,	sem_close_cef,	"sem_close"	},
#endif
#if	defined(_POSIX_SEMAPHORES)
	{	sem_destroy_csf,	sem_destroy_ctf,	sem_destroy_cef,	"sem_destroy"	},
#endif
#if	defined(_POSIX_SEMAPHORES)
	{	sem_getvalue_csf,	sem_getvalue_ctf,	sem_getvalue_cef,	"sem_getvalue"	},
#endif
#if	defined(_POSIX_SEMAPHORES)
	{	sem_init_csf,	sem_init_ctf,	sem_init_cef,	"sem_init"	},
#endif
#if	defined(_POSIX_SEMAPHORES)
	{	sem_open_csf,	sem_open_ctf,	sem_open_cef,	"sem_open"	},
#endif
#if	defined(_POSIX_SEMAPHORES)
	{	sem_post_csf,	sem_post_ctf,	sem_post_cef,	"sem_post"	},
#endif
#if	defined(_POSIX_SEMAPHORES)
	{	sem_trywait_csf,	sem_trywait_ctf,	sem_trywait_cef,	"sem_trywait"	},
#endif
#if	defined(_POSIX_SEMAPHORES)
	{	sem_unlink_csf,	sem_unlink_ctf,	sem_unlink_cef,	"sem_unlink"	},
#endif
	{	setbuf_csf,	setbuf_ctf,	setbuf_cef,	"setbuf"	},
	{	setgid_csf,	setgid_ctf,	setgid_cef,	"setgid"	},
	{	setjmp_csf,	setjmp_ctf,	setjmp_cef,	"setjmp"	},
	{	setlocale_csf,	setlocale_ctf,	setlocale_cef,	"setlocale"	},
#if	defined(_POSIX_JOB_CONTROL)
	{	setpgid_csf,	setpgid_ctf,	setpgid_cef,	"setpgid"	},
#endif
	{	setsid_csf,	setsid_ctf,	setsid_cef,	"setsid"	},
	{	setuid_csf,	setuid_ctf,	setuid_cef,	"setuid"	},
	{	setvbuf_csf,	setvbuf_ctf,	setvbuf_cef,	"setvbuf"	},
#if	defined(_POSIX_SHARED_MEMORY_OBJECTS)
	{	shm_open_csf,	shm_open_ctf,	shm_open_cef,	"shm_open"	},
#endif
#if	defined(_POSIX_SHARED_MEMORY_OBJECTS)
	{	shm_unlink_csf,	shm_unlink_ctf,	shm_unlink_cef,	"shm_unlink"	},
#endif
	{	sigaction_csf,	sigaction_ctf,	sigaction_cef,	"sigaction"	},
	{	sigaddset_csf,	sigaddset_ctf,	sigaddset_cef,	"sigaddset"	},
	{	sigdelset_csf,	sigdelset_ctf,	sigdelset_cef,	"sigdelset"	},
	{	sigemptyset_csf,	sigemptyset_ctf,	sigemptyset_cef,	"sigemptyset"	},
	{	sigfillset_csf,	sigfillset_ctf,	sigfillset_cef,	"sigfillset"	},
	{	sigismember_csf,	sigismember_ctf,	sigismember_cef,	"sigismember"	},
	{	siglongjmp_csf,	NULL,	siglongjmp_cef,	"siglongjmp"	},
	{	signal_csf,	signal_ctf,	signal_cef,	"signal"	},
	{	sigpending_csf,	sigpending_ctf,	sigpending_cef,	"sigpending"	},
	{	sigprocmask_csf,	sigprocmask_ctf,	sigprocmask_cef,	"sigprocmask"	},
#if	defined(_POSIX_REALTIME_SIGNALS)
	{	sigqueue_csf,	sigqueue_ctf,	sigqueue_cef,	"sigqueue"	},
#endif
	{	sigsetjmp_csf,	sigsetjmp_ctf,	sigsetjmp_cef,	"sigsetjmp"	},
	{	sin_csf,	sin_ctf,	sin_cef,	"sin"	},
	{	sinh_csf,	sinh_ctf,	sinh_cef,	"sinh"	},
	{	sprintf_csf,	sprintf_ctf,	sprintf_cef,	"sprintf"	},
	{	sqrt_csf,	sqrt_ctf,	sqrt_cef,	"sqrt"	},
	{	srand_csf,	srand_ctf,	srand_cef,	"srand"	},
	{	sscanf_csf,	sscanf_ctf,	sscanf_cef,	"sscanf"	},
	{	stat_csf,	stat_ctf,	stat_cef,	"stat"	},
	{	strcat_csf,	strcat_ctf,	strcat_cef,	"strcat"	},
	{	strchr_csf,	strchr_ctf,	strchr_cef,	"strchr"	},
	{	strcmp_csf,	strcmp_ctf,	strcmp_cef,	"strcmp"	},
	{	strcoll_csf,	strcoll_ctf,	strcoll_cef,	"strcoll"	},
	{	strcpy_csf,	strcpy_ctf,	strcpy_cef,	"strcpy"	},
	{	strcspn_csf,	strcspn_ctf,	strcspn_cef,	"strcspn"	},
	{	strerror_csf,	strerror_ctf,	strerror_cef,	"strerror"	},
	{	strftime_csf,	strftime_ctf,	strftime_cef,	"strftime"	},
	{	strlen_csf,	strlen_ctf,	strlen_cef,	"strlen"	},
	{	strncat_csf,	strncat_ctf,	strncat_cef,	"strncat"	},
	{	strncmp_csf,	strncmp_ctf,	strncmp_cef,	"strncmp"	},
	{	strncpy_csf,	strncpy_ctf,	strncpy_cef,	"strncpy"	},
	{	strpbrk_csf,	strpbrk_ctf,	strpbrk_cef,	"strpbrk"	},
	{	strrchr_csf,	strrchr_ctf,	strrchr_cef,	"strrchr"	},
	{	strspn_csf,	strspn_ctf,	strspn_cef,	"strspn"	},
	{	strstr_csf,	strstr_ctf,	strstr_cef,	"strstr"	},
	{	strtod_csf,	strtod_ctf,	strtod_cef,	"strtod"	},
	{	strtok_csf,	strtok_ctf,	strtok_cef,	"strtok"	},
#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)
	{	strtok_r_csf,	strtok_r_ctf,	strtok_r_cef,	"strtok_r"	},
#endif
	{	strtol_csf,	strtol_ctf,	strtol_cef,	"strtol"	},
	{	strtoul_csf,	strtoul_ctf,	strtoul_cef,	"strtoul"	},
	{	strxfrm_csf,	strxfrm_ctf,	strxfrm_cef,	"strxfrm"	},
	{	sysconf_csf,	sysconf_ctf,	sysconf_cef,	"sysconf"	},
	{	tan_csf,	tan_ctf,	tan_cef,	"tan"	},
	{	tanh_csf,	tanh_ctf,	tanh_cef,	"tanh"	},
	{	tcflow_csf,	tcflow_ctf,	tcflow_cef,	"tcflow"	},
	{	tcflush_csf,	tcflush_ctf,	tcflush_cef,	"tcflush"	},
	{	tcgetattr_csf,	tcgetattr_ctf,	tcgetattr_cef,	"tcgetattr"	},
#if	defined(_POSIX_JOB_CONTROL)
	{	tcgetpgrp_csf,	tcgetpgrp_ctf,	tcgetpgrp_cef,	"tcgetpgrp"	},
#endif
	{	tcsendbreak_csf,	tcsendbreak_ctf,	tcsendbreak_cef,	"tcsendbreak"	},
	{	tcsetattr_csf,	tcsetattr_ctf,	tcsetattr_cef,	"tcsetattr"	},
#if	defined(_POSIX_JOB_CONTROL)
	{	tcsetpgrp_csf,	tcsetpgrp_ctf,	tcsetpgrp_cef,	"tcsetpgrp"	},
#endif
	{	time_csf,	time_ctf,	time_cef,	"time"	},
#if	defined(_POSIX_TIMERS)
	{	timer_create_csf,	timer_create_ctf,	timer_create_cef,	"timer_create"	},
#endif
#if	defined(_POSIX_TIMERS)
	{	timer_delete_csf,	timer_delete_ctf,	timer_delete_cef,	"timer_delete"	},
#endif
#if	defined(_POSIX_TIMERS)
	{	timer_getoverrun_csf,	timer_getoverrun_ctf,	timer_getoverrun_cef,	"timer_getoverrun"	},
#endif
#if	defined(_POSIX_TIMERS)
	{	timer_gettime_csf,	timer_gettime_ctf,	timer_gettime_cef,	"timer_gettime"	},
#endif
#if	defined(_POSIX_TIMERS)
	{	timer_settime_csf,	timer_settime_ctf,	timer_settime_cef,	"timer_settime"	},
#endif
	{	times_csf,	times_ctf,	times_cef,	"times"	},
	{	tmpnam_csf,	tmpnam_ctf,	tmpnam_cef,	"tmpnam"	},
	{	tolower_csf,	tolower_ctf,	tolower_cef,	"tolower"	},
	{	toupper_csf,	toupper_ctf,	toupper_cef,	"toupper"	},
	{	tzset_csf,	tzset_ctf,	tzset_cef,	"tzset"	},
	{	umask_csf,	umask_ctf,	umask_cef,	"umask"	},
	{	uname_csf,	uname_ctf,	uname_cef,	"uname"	},
	{	utime_csf,	utime_ctf,	utime_cef,	"utime"	},
	{	vfprintf_csf,	vfprintf_ctf,	vfprintf_cef,	"vfprintf"	},
	{	vprintf_csf,	vprintf_ctf,	vprintf_cef,	"vprintf"	},
	{	vsprintf_csf,	vsprintf_ctf,	vsprintf_cef,	"vsprintf"	},
	{	wcstombs_csf,	wcstombs_ctf,	wcstombs_cef,	"wcstombs"	},
	{	wctomb_csf,	wctomb_ctf,	wctomb_cef,	"wctomb"	},
	{	NULL,	NULL,	NULL,	NULL	}
};

VSTH_RT	Vsth_cp8[] = {
#if	( TEST_XPG > 4 )
	{	getmsg_csf,	getmsg_ctf,	getmsg_cef,	"getmsg"	},
	{	getpmsg_csf,	getpmsg_ctf,	getpmsg_cef,	"getpmsg"	},
	{	lockf_csf,	lockf_ctf,	lockf_cef,	"lockf"	},
	{	msgrcv_csf,	msgrcv_ctf,	msgrcv_cef,	"msgrcv"	},
	{	msgsnd_csf,	msgsnd_ctf,	msgsnd_cef,	"msgsnd"	},
	{	poll_csf,	poll_ctf,	poll_cef,	"poll"	},
	{	pread_csf,	pread_ctf,	pread_cef,	"pread"	},
	{	putmsg_csf,	putmsg_ctf,	putmsg_cef,	"putmsg"	},
	{	putpmsg_csf,	putpmsg_ctf,	putpmsg_cef,	"putpmsg"	},
	{	pwrite_csf,	pwrite_ctf,	pwrite_cef,	"pwrite"	},
	{	readv_csf,	readv_ctf,	readv_cef,	"readv"	},
	{	select_csf,	select_ctf,	select_cef,	"select"	},
	{	sigpause_csf,	sigpause_ctf,	sigpause_cef,	"sigpause"	},
	{	usleep_csf,	usleep_ctf,	usleep_cef,	"usleep"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	wait3_csf,	wait3_ctf,	wait3_cef,	"wait3"	},
#endif
	{	waitid_csf,	waitid_ctf,	waitid_cef,	"waitid"	},
	{	writev_csf,	writev_ctf,	writev_cef,	"writev"	},
#endif
	{	NULL,	NULL,	NULL,	NULL	}
};

VSTH_RT	Vsth_cp9[] = {
#if	( TEST_XPG > 4 )
	{	_longjmp_csf,	NULL,	_longjmp_cef,	"_longjmp"	},
	{	_setjmp_csf,	NULL,	_setjmp_cef,	"_setjmp"	},
	{	_tolower_csf,	NULL,	_tolower_cef,	"_tolower"	},
	{	_toupper_csf,	NULL,	_toupper_cef,	"_toupper"	},
	{	a64l_csf,	a64l_ctf,	a64l_cef,	"a64l"	},
	{	acosh_csf,	acosh_ctf,	acosh_cef,	"acosh"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	advance_csf,	NULL,	advance_cef,	"advance"	},
#endif
	{	asinh_csf,	asinh_ctf,	asinh_cef,	"asinh"	},
	{	atanh_csf,	atanh_ctf,	atanh_cef,	"atanh"	},
	{	basename_csf,	basename_ctf,	basename_cef,	"basename"	},
	{	bcmp_csf,	bcmp_ctf,	bcmp_cef,	"bcmp"	},
	{	bcopy_csf,	bcopy_ctf,	bcopy_cef,	"bcopy"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	brk_csf,	NULL,	brk_cef,	"brk"	},
#endif
	{	bsd_signal_csf,	bsd_signal_ctf,	bsd_signal_cef,	"bsd_signal"	},
	{	btowc_csf,	btowc_ctf,	btowc_cef,	"btowc"	},
	{	bzero_csf,	bzero_ctf,	bzero_cef,	"bzero"	},
	{	cbrt_csf,	cbrt_ctf,	cbrt_cef,	"cbrt"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	chroot_csf,	chroot_ctf,	chroot_cef,	"chroot"	},
#endif
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	compile_csf,	NULL,	compile_cef,	"compile"	},
#endif
	{	confstr_csf,	confstr_ctf,	confstr_cef,	"confstr"	},
#if	defined(_XOPEN_CRYPT) && (_XOPEN_CRYPT != -1)
	{	crypt_csf,	crypt_ctf,	crypt_cef,	"crypt"	},
#endif
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	cuserid_csf,	cuserid_ctf,	cuserid_cef,	"cuserid"	},
#endif
	{	dbm_clearerr_csf,	dbm_clearerr_ctf,	dbm_clearerr_cef,	"dbm_clearerr"	},
	{	dbm_error_csf,	dbm_error_ctf,	dbm_error_cef,	"dbm_error"	},
	{	dbm_firstkey_csf,	dbm_firstkey_ctf,	dbm_firstkey_cef,	"dbm_firstkey"	},
	{	dirname_csf,	dirname_ctf,	dirname_cef,	"dirname"	},
	{	dlerror_csf,	dlerror_ctf,	dlerror_cef,	"dlerror"	},
	{	dlsym_csf,	NULL,	dlsym_cef,	"dlsym"	},
	{	drand48_csf,	drand48_ctf,	drand48_cef,	"drand48"	},
	{	ecvt_csf,	ecvt_ctf,	ecvt_cef,	"ecvt"	},
#if	defined(_XOPEN_CRYPT) && (_XOPEN_CRYPT != -1)
	{	encrypt_csf,	encrypt_ctf,	encrypt_cef,	"encrypt"	},
#endif
	{	erand48_csf,	erand48_ctf,	erand48_cef,	"erand48"	},
	{	erf_csf,	erf_ctf,	erf_cef,	"erf"	},
	{	erfc_csf,	erfc_ctf,	erfc_cef,	"erfc"	},
	{	expm1_csf,	expm1_ctf,	expm1_cef,	"expm1"	},
	{	fattach_csf,	fattach_ctf,	fattach_cef,	"fattach"	},
	{	fchdir_csf,	fchdir_ctf,	fchdir_cef,	"fchdir"	},
	{	fchown_csf,	fchown_ctf,	fchown_cef,	"fchown"	},
	{	fcvt_csf,	fcvt_ctf,	fcvt_cef,	"fcvt"	},
	{	fdetach_csf,	fdetach_ctf,	fdetach_cef,	"fdetach"	},
	{	ffs_csf,	ffs_ctf,	ffs_cef,	"ffs"	},
	{	fmtmsg_csf,	fmtmsg_ctf,	fmtmsg_cef,	"fmtmsg"	},
	{	fnmatch_csf,	fnmatch_ctf,	fnmatch_cef,	"fnmatch"	},
	{	fstatvfs_csf,	fstatvfs_ctf,	fstatvfs_cef,	"fstatvfs"	},
	{	ftime_csf,	ftime_ctf,	ftime_cef,	"ftime"	},
	{	ftok_csf,	ftok_ctf,	ftok_cef,	"ftok"	},
	{	fwide_csf,	fwide_ctf,	fwide_cef,	"fwide"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	gamma_csf,	gamma_ctf,	gamma_cef,	"gamma"	},
#endif
	{	gcvt_csf,	gcvt_ctf,	gcvt_cef,	"gcvt"	},
	{	getcontext_csf,	getcontext_ctf,	getcontext_cef,	"getcontext"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	getdtablesize_csf,	getdtablesize_ctf,	getdtablesize_cef,	"getdtablesize"	},
#endif
	{	gethostid_csf,	gethostid_ctf,	gethostid_cef,	"gethostid"	},
	{	getitimer_csf,	getitimer_ctf,	getitimer_cef,	"getitimer"	},
	{	getopt_csf,	getopt_ctf,	getopt_cef,	"getopt"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	getpagesize_csf,	getpagesize_ctf,	getpagesize_cef,	"getpagesize"	},
#endif
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	getpass_csf,	getpass_ctf,	getpass_cef,	"getpass"	},
#endif
	{	getpgid_csf,	getpgid_ctf,	getpgid_cef,	"getpgid"	},
	{	getpriority_csf,	getpriority_ctf,	getpriority_cef,	"getpriority"	},
	{	getrlimit_csf,	getrlimit_ctf,	getrlimit_cef,	"getrlimit"	},
	{	getrusage_csf,	getrusage_ctf,	getrusage_cef,	"getrusage"	},
	{	getsid_csf,	getsid_ctf,	getsid_cef,	"getsid"	},
	{	getsubopt_csf,	getsubopt_ctf,	getsubopt_cef,	"getsubopt"	},
	{	gettimeofday_csf,	gettimeofday_ctf,	gettimeofday_cef,	"gettimeofday"	},
	{	grantpt_csf,	grantpt_ctf,	grantpt_cef,	"grantpt"	},
	{	hcreate_csf,	hcreate_ctf,	hcreate_cef,	"hcreate"	},
	{	hdestroy_csf,	hdestroy_ctf,	hdestroy_cef,	"hdestroy"	},
	{	hsearch_csf,	hsearch_ctf,	hsearch_cef,	"hsearch"	},
	{	hypot_csf,	hypot_ctf,	hypot_cef,	"hypot"	},
	{	iconv_csf,	iconv_ctf,	iconv_cef,	"iconv"	},
	{	ilogb_csf,	ilogb_ctf,	ilogb_cef,	"ilogb"	},
	{	index_csf,	index_ctf,	index_cef,	"index"	},
	{	initstate_csf,	initstate_ctf,	initstate_cef,	"initstate"	},
	{	insque_csf,	insque_ctf,	insque_cef,	"insque"	},
	{	isastream_csf,	isastream_ctf,	isastream_cef,	"isastream"	},
	{	isnan_csf,	isnan_ctf,	isnan_cef,	"isnan"	},
	{	iswalnum_csf,	iswalnum_ctf,	iswalnum_cef,	"iswalnum"	},
	{	iswalpha_csf,	iswalpha_ctf,	iswalpha_cef,	"iswalpha"	},
	{	iswcntrl_csf,	iswcntrl_ctf,	iswcntrl_cef,	"iswcntrl"	},
	{	iswdigit_csf,	iswdigit_ctf,	iswdigit_cef,	"iswdigit"	},
	{	iswgraph_csf,	iswgraph_ctf,	iswgraph_cef,	"iswgraph"	},
	{	iswlower_csf,	iswlower_ctf,	iswlower_cef,	"iswlower"	},
	{	iswprint_csf,	iswprint_ctf,	iswprint_cef,	"iswprint"	},
	{	iswpunct_csf,	iswpunct_ctf,	iswpunct_cef,	"iswpunct"	},
	{	iswspace_csf,	iswspace_ctf,	iswspace_cef,	"iswspace"	},
	{	iswupper_csf,	iswupper_ctf,	iswupper_cef,	"iswupper"	},
	{	iswxdigit_csf,	iswxdigit_ctf,	iswxdigit_cef,	"iswxdigit"	},
	{	iswctype_csf,	iswctype_ctf,	iswctype_cef,	"iswctype"	},
	{	j0_csf,	j0_ctf,	j0_cef,	"j0"	},
	{	j1_csf,	j1_ctf,	j1_cef,	"j1"	},
	{	jn_csf,	jn_ctf,	jn_cef,	"jn"	},
	{	jrand48_csf,	jrand48_ctf,	jrand48_cef,	"jrand48"	},
	{	killpg_csf,	killpg_ctf,	killpg_cef,	"killpg"	},
	{	l64a_csf,	l64a_ctf,	l64a_cef,	"l64a"	},
	{	lchown_csf,	lchown_ctf,	lchown_cef,	"lchown"	},
	{	lcong48_csf,	lcong48_ctf,	lcong48_cef,	"lcong48"	},
	{	lfind_csf,	lfind_ctf,	lfind_cef,	"lfind"	},
	{	lgamma_csf,	lgamma_ctf,	lgamma_cef,	"lgamma"	},
	{	localeconv_csf,	localeconv_ctf,	localeconv_cef,	"localeconv"	},
	{	log1p_csf,	log1p_ctf,	log1p_cef,	"log1p"	},
	{	logb_csf,	logb_ctf,	logb_cef,	"logb"	},
	{	lrand48_csf,	lrand48_ctf,	lrand48_cef,	"lrand48"	},
	{	lsearch_csf,	lsearch_ctf,	lsearch_cef,	"lsearch"	},
	{	lstat_csf,	lstat_ctf,	lstat_cef,	"lstat"	},
	{	makecontext_csf,	makecontext_ctf,	makecontext_cef,	"makecontext"	},
	{	mbrlen_csf,	mbrlen_ctf,	mbrlen_cef,	"mbrlen"	},
	{	mbrtowc_csf,	mbrtowc_ctf,	mbrtowc_cef,	"mbrtowc"	},
	{	mbsinit_csf,	mbsinit_ctf,	mbsinit_cef,	"mbsinit"	},
	{	mbsrtowcs_csf,	mbsrtowcs_ctf,	mbsrtowcs_cef,	"mbsrtowcs"	},
	{	mbstowcs_csf,	mbstowcs_ctf,	mbstowcs_cef,	"mbstowcs"	},
	{	memccpy_csf,	memccpy_ctf,	memccpy_cef,	"memccpy"	},
	{	mknod_csf,	mknod_ctf,	mknod_cef,	"mknod"	},
	{	mktemp_csf,	mktemp_ctf,	mktemp_cef,	"mktemp"	},
	{	mrand48_csf,	mrand48_ctf,	mrand48_cef,	"mrand48"	},
	{	msgctl_csf,	msgctl_ctf,	msgctl_cef,	"msgctl"	},
	{	msgget_csf,	msgget_ctf,	msgget_cef,	"msgget"	},
	{	nextafter_csf,	nextafter_ctf,	nextafter_cef,	"nextafter"	},
	{	nice_csf,	nice_ctf,	nice_cef,	"nice"	},
	{	nl_langinfo_csf,	nl_langinfo_ctf,	nl_langinfo_cef,	"nl_langinfo"	},
	{	nrand48_csf,	nrand48_ctf,	nrand48_cef,	"nrand48"	},
	{	pthread_attr_getguardsize_csf,	pthread_attr_getguardsize_ctf,	pthread_attr_getguardsize_cef,	"pthread_attr_getguardsize"	},
	{	pthread_attr_setguardsize_csf,	pthread_attr_setguardsize_ctf,	pthread_attr_setguardsize_cef,	"pthread_attr_setguardsize"	},
	{	pthread_getconcurrency_csf,	pthread_getconcurrency_ctf,	pthread_getconcurrency_cef,	"pthread_getconcurrency"	},
	{	pthread_mutexattr_gettype_csf,	pthread_mutexattr_gettype_ctf,	pthread_mutexattr_gettype_cef,	"pthread_mutexattr_gettype"	},
	{	pthread_mutexattr_settype_csf,	pthread_mutexattr_settype_ctf,	pthread_mutexattr_settype_cef,	"pthread_mutexattr_settype"	},
	{	pthread_rwlock_destroy_csf,	pthread_rwlock_destroy_ctf,	pthread_rwlock_destroy_cef,	"pthread_rwlock_destroy"	},
	{	pthread_rwlock_init_csf,	pthread_rwlock_init_ctf,	pthread_rwlock_init_cef,	"pthread_rwlock_init"	},
	{	pthread_rwlock_rdlock_csf,	pthread_rwlock_rdlock_ctf,	pthread_rwlock_rdlock_cef,	"pthread_rwlock_rdlock"	},
	{	pthread_rwlock_tryrdlock_csf,	pthread_rwlock_tryrdlock_ctf,	pthread_rwlock_tryrdlock_cef,	"pthread_rwlock_tryrdlock"	},
	{	pthread_rwlock_trywrlock_csf,	pthread_rwlock_trywrlock_ctf,	pthread_rwlock_trywrlock_cef,	"pthread_rwlock_trywrlock"	},
	{	pthread_rwlock_unlock_csf,	pthread_rwlock_unlock_ctf,	pthread_rwlock_unlock_cef,	"pthread_rwlock_unlock"	},
	{	pthread_rwlock_wrlock_csf,	pthread_rwlock_wrlock_ctf,	pthread_rwlock_wrlock_cef,	"pthread_rwlock_wrlock"	},
	{	pthread_rwlockattr_destroy_csf,	pthread_rwlockattr_destroy_ctf,	pthread_rwlockattr_destroy_cef,	"pthread_rwlockattr_destroy"	},
	{	pthread_rwlockattr_getpshared_csf,	pthread_rwlockattr_getpshared_ctf,	pthread_rwlockattr_getpshared_cef,	"pthread_rwlockattr_getpshared"	},
	{	pthread_rwlockattr_init_csf,	pthread_rwlockattr_init_ctf,	pthread_rwlockattr_init_cef,	"pthread_rwlockattr_init"	},
	{	pthread_rwlockattr_setpshared_csf,	pthread_rwlockattr_setpshared_ctf,	pthread_rwlockattr_setpshared_cef,	"pthread_rwlockattr_setpshared"	},
	{	pthread_setconcurrency_csf,	pthread_setconcurrency_ctf,	pthread_setconcurrency_cef,	"pthread_setconcurrency"	},
	{	ptsname_csf,	ptsname_ctf,	ptsname_cef,	"ptsname"	},
	{	putenv_csf,	putenv_ctf,	putenv_cef,	"putenv"	},
	{	random_csf,	random_ctf,	random_cef,	"random"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	re_comp_csf,	re_comp_ctf,	re_comp_cef,	"re_comp"	},
#endif
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	re_exec_csf,	re_exec_ctf,	re_exec_cef,	"re_exec"	},
#endif
	{	readlink_csf,	readlink_ctf,	readlink_cef,	"readlink"	},
	{	realpath_csf,	realpath_ctf,	realpath_cef,	"realpath"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	regcmp_csf,	regcmp_ctf,	regcmp_cef,	"regcmp"	},
#endif
	{	regcomp_csf,	regcomp_ctf,	regcomp_cef,	"regcomp"	},
	{	regerror_csf,	regerror_ctf,	regerror_cef,	"regerror"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	regex_csf,	regex_ctf,	regex_cef,	"regex"	},
#endif
	{	regexec_csf,	regexec_ctf,	regexec_cef,	"regexec"	},
	{	regfree_csf,	regfree_ctf,	regfree_cef,	"regfree"	},
	{	remainder_csf,	remainder_ctf,	remainder_cef,	"remainder"	},
	{	remque_csf,	remque_ctf,	remque_cef,	"remque"	},
	{	rindex_csf,	rindex_ctf,	rindex_cef,	"rindex"	},
	{	rint_csf,	rint_ctf,	rint_cef,	"rint"	},
	{	sbrk_csf,	NULL,	sbrk_cef,	"sbrk"	},
	{	scalb_csf,	scalb_ctf,	scalb_cef,	"scalb"	},
	{	seed48_csf,	seed48_ctf,	seed48_cef,	"seed48"	},
	{	semctl_csf,	semctl_ctf,	semctl_cef,	"semctl"	},
	{	semget_csf,	semget_ctf,	semget_cef,	"semget"	},
	{	setcontext_csf,	NULL,	setcontext_cef,	"setcontext"	},
	{	setitimer_csf,	setitimer_ctf,	setitimer_cef,	"setitimer"	},
#if	defined(_XOPEN_CRYPT) && (_XOPEN_CRYPT != -1)
	{	setkey_csf,	setkey_ctf,	setkey_cef,	"setkey"	},
#endif
	{	setlogmask_csf,	setlogmask_ctf,	setlogmask_cef,	"setlogmask"	},
	{	setpgrp_csf,	setpgrp_ctf,	setpgrp_cef,	"setpgrp"	},
	{	setpriority_csf,	setpriority_ctf,	setpriority_cef,	"setpriority"	},
	{	setregid_csf,	setregid_ctf,	setregid_cef,	"setregid"	},
	{	setreuid_csf,	setreuid_ctf,	setreuid_cef,	"setreuid"	},
	{	setrlimit_csf,	setrlimit_ctf,	setrlimit_cef,	"setrlimit"	},
	{	setstate_csf,	setstate_ctf,	setstate_cef,	"setstate"	},
	{	shmat_csf,	shmat_ctf,	shmat_cef,	"shmat"	},
	{	shmctl_csf,	shmctl_ctf,	shmctl_cef,	"shmctl"	},
	{	shmdt_csf,	shmdt_ctf,	shmdt_cef,	"shmdt"	},
	{	shmget_csf,	shmget_ctf,	shmget_cef,	"shmget"	},
	{	sigaltstack_csf,	sigaltstack_ctf,	sigaltstack_cef,	"sigaltstack"	},
	{	sighold_csf,	sighold_ctf,	sighold_cef,	"sighold"	},
	{	sigignore_csf,	sigignore_ctf,	sigignore_cef,	"sigignore"	},
	{	siginterrupt_csf,	siginterrupt_ctf,	siginterrupt_cef,	"siginterrupt"	},
	{	sigrelse_csf,	sigrelse_ctf,	sigrelse_cef,	"sigrelse"	},
	{	sigset_csf,	sigset_ctf,	sigset_cef,	"sigset"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	sigstack_csf,	sigstack_ctf,	sigstack_cef,	"sigstack"	},
#endif
	{	snprintf_csf,	snprintf_ctf,	snprintf_cef,	"snprintf"	},
	{	srand48_csf,	srand48_ctf,	srand48_cef,	"srand48"	},
	{	srandom_csf,	srandom_ctf,	srandom_cef,	"srandom"	},
	{	statvfs_csf,	statvfs_ctf,	statvfs_cef,	"statvfs"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	step_csf,	NULL,	step_cef,	"step"	},
#endif
	{	strcasecmp_csf,	strcasecmp_ctf,	strcasecmp_cef,	"strcasecmp"	},
	{	strdup_csf,	strdup_ctf,	strdup_cef,	"strdup"	},
	{	strfmon_csf,	strfmon_ctf,	strfmon_cef,	"strfmon"	},
	{	strncasecmp_csf,	strncasecmp_ctf,	strncasecmp_cef,	"strncasecmp"	},
	{	strptime_csf,	strptime_ctf,	strptime_cef,	"strptime"	},
	{	swab_csf,	swab_ctf,	swab_cef,	"swab"	},
	{	swapcontext_csf,	NULL,	swapcontext_cef,	"swapcontext"	},
	{	swprintf_csf,	swprintf_ctf,	swprintf_cef,	"swprintf"	},
	{	swscanf_csf,	swscanf_ctf,	swscanf_cef,	"swscanf"	},
	{	symlink_csf,	symlink_ctf,	symlink_cef,	"symlink"	},
	{	sync_csf,	sync_ctf,	sync_cef,	"sync"	},
	{	tcgetsid_csf,	tcgetsid_ctf,	tcgetsid_cef,	"tcgetsid"	},
	{	tdelete_csf,	tdelete_ctf,	tdelete_cef,	"tdelete"	},
	{	telldir_csf,	telldir_ctf,	telldir_cef,	"telldir"	},
	{	tempnam_csf,	tempnam_ctf,	tempnam_cef,	"tempnam"	},
	{	tfind_csf,	tfind_ctf,	tfind_cef,	"tfind"	},
	{	toascii_csf,	toascii_ctf,	toascii_cef,	"toascii"	},
	{	towctrans_csf,	towctrans_ctf,	towctrans_cef,	"towctrans"	},
	{	towlower_csf,	towlower_ctf,	towlower_cef,	"towlower"	},
	{	towupper_csf,	towupper_ctf,	towupper_cef,	"towupper"	},
	{	truncate_csf,	truncate_ctf,	truncate_cef,	"truncate"	},
	{	tsearch_csf,	tsearch_ctf,	tsearch_cef,	"tsearch"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	ttyslot_csf,	ttyslot_ctf,	ttyslot_cef,	"ttyslot"	},
#endif
	{	twalk_csf,	twalk_ctf,	twalk_cef,	"twalk"	},
	{	ualarm_csf,	ualarm_ctf,	ualarm_cef,	"ualarm"	},
	{	ulimit_csf,	ulimit_ctf,	ulimit_cef,	"ulimit"	},
	{	unlockpt_csf,	unlockpt_ctf,	unlockpt_cef,	"unlockpt"	},
	{	utimes_csf,	utimes_ctf,	utimes_cef,	"utimes"	},
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	valloc_csf,	valloc_ctf,	valloc_cef,	"valloc"	},
#endif
	{	vfork_csf,	NULL,	vfork_cef,	"vfork"	},
	{	vsnprintf_csf,	vsnprintf_ctf,	vsnprintf_cef,	"vsnprintf"	},
	{	vswprintf_csf,	vswprintf_ctf,	vswprintf_cef,	"vswprintf"	},
	{	wcrtomb_csf,	wcrtomb_ctf,	wcrtomb_cef,	"wcrtomb"	},
	{	wcscat_csf,	wcscat_ctf,	wcscat_cef,	"wcscat"	},
	{	wcschr_csf,	wcschr_ctf,	wcschr_cef,	"wcschr"	},
	{	wcscmp_csf,	wcscmp_ctf,	wcscmp_cef,	"wcscmp"	},
	{	wcscoll_csf,	wcscoll_ctf,	wcscoll_cef,	"wcscoll"	},
	{	wcscpy_csf,	wcscpy_ctf,	wcscpy_cef,	"wcscpy"	},
	{	wcscspn_csf,	wcscspn_ctf,	wcscspn_cef,	"wcscspn"	},
	{	wcsftime_csf,	wcsftime_ctf,	wcsftime_cef,	"wcsftime"	},
	{	wcslen_csf,	wcslen_ctf,	wcslen_cef,	"wcslen"	},
	{	wcsncat_csf,	wcsncat_ctf,	wcsncat_cef,	"wcsncat"	},
	{	wcsncmp_csf,	wcsncmp_ctf,	wcsncmp_cef,	"wcsncmp"	},
	{	wcsncpy_csf,	wcsncpy_ctf,	wcsncpy_cef,	"wcsncpy"	},
	{	wcspbrk_csf,	wcspbrk_ctf,	wcspbrk_cef,	"wcspbrk"	},
	{	wcsrchr_csf,	wcsrchr_ctf,	wcsrchr_cef,	"wcsrchr"	},
	{	wcsrtombs_csf,	wcsrtombs_ctf,	wcsrtombs_cef,	"wcsrtombs"	},
	{	wcsspn_csf,	wcsspn_ctf,	wcsspn_cef,	"wcsspn"	},
	{	wcsstr_csf,	wcsstr_ctf,	wcsstr_cef,	"wcsstr"	},
	{	wcstod_csf,	wcstod_ctf,	wcstod_cef,	"wcstod"	},
	{	wcstok_csf,	wcstok_ctf,	wcstok_cef,	"wcstok"	},
	{	wcstol_csf,	wcstol_ctf,	wcstol_cef,	"wcstol"	},
	{	wcstoul_csf,	wcstoul_ctf,	wcstoul_cef,	"wcstoul"	},
	{	wcswcs_csf,	wcswcs_ctf,	wcswcs_cef,	"wcswcs"	},
	{	wcswidth_csf,	wcswidth_ctf,	wcswidth_cef,	"wcswidth"	},
	{	wcsxfrm_csf,	wcsxfrm_ctf,	wcsxfrm_cef,	"wcsxfrm"	},
	{	wctob_csf,	wctob_ctf,	wctob_cef,	"wctob"	},
	{	wctrans_csf,	wctrans_ctf,	wctrans_cef,	"wctrans"	},
	{	wctype_csf,	wctype_ctf,	wctype_cef,	"wctype"	},
	{	wcwidth_csf,	wcwidth_ctf,	wcwidth_cef,	"wcwidth"	},
	{	wmemchr_csf,	wmemchr_ctf,	wmemchr_cef,	"wmemchr"	},
	{	wmemcmp_csf,	wmemcmp_ctf,	wmemcmp_cef,	"wmemcmp"	},
	{	wmemcpy_csf,	wmemcpy_ctf,	wmemcpy_cef,	"wmemcpy"	},
	{	wmemmove_csf,	wmemmove_ctf,	wmemmove_cef,	"wmemmove"	},
	{	wmemset_csf,	wmemset_ctf,	wmemset_cef,	"wmemset"	},
	{	wordexp_csf,	wordexp_ctf,	wordexp_cef,	"wordexp"	},
	{	y0_csf,	y0_ctf,	y0_cef,	"y0"	},
	{	y1_csf,	y1_ctf,	y1_cef,	"y1"	},
	{	yn_csf,	yn_ctf,	yn_cef,	"yn"	},
#endif
	{	NULL,	NULL,	NULL,	NULL	}
};

