
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_cpg.c	1.12
*
*************************************************************************/


#include "std.h"


#if	( TEST_XPG > 4 )
#include <glob.h>
#endif

#include <grp.h>
#include <limits.h>
#include <math.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>

#if	( TEST_XPG > 4 )
#include <stropts.h>
#include <sys/resource.h>
#include <sys/time.h>
#endif

#include <sys/types.h>
#include <time.h>

#if	( TEST_XPG > 4 )
#include <ucontext.h>
#endif

#include <unistd.h>

#if	( TEST_XPG > 4 )
#include <utmpx.h>
#include <wchar.h>
#endif


#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"


#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for gamma()					 ******/

#ifdef	gamma
#undef	gamma
#endif

int
gamma_csf()
{
	return(0);
}

int
gamma_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) gamma(0.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
gamma_cef()
{
	return(0);
}

#endif
#endif

#if	( TEST_XPG > 4 )

/****** Testset for gcvt()					 ******/

#ifdef	gcvt
#undef	gcvt
#endif

int
gcvt_csf()
{
	return(0);
}

int
gcvt_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) gcvt(0.0, 0, &buf[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
gcvt_cef()
{
	return(0);
}

#endif


/****** Testset for getc()					 ******/

#ifdef	getc
#undef	getc
#endif

static FILE	*Getc_fp;

int
getc_csf()
{
	int	rval;

	rval = 0;

	Getc_fp = fopen("./nonesuch", "w+");
	if ( Getc_fp != NULL )
	{
		rval = close(fileno(Getc_fp));
	}
	else
	{
		++rval;
	}
	return(rval);
}

int
getc_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getc(Getc_fp);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getc_cef()
{
	(void) fclose(Getc_fp);
	(void) remove("./nonesuch");
	return(0);
}



/****** Testset for getc_unlocked()				 ******/

#ifdef	getc_unlocked
#undef	getc_unlocked
#endif

static FILE	*Getc_unlocked_fp;

int
getc_unlocked_csf()
{
	int	rval;

	rval = 0;

	Getc_unlocked_fp = fopen("./nonesuch", "w+");
	if ( Getc_unlocked_fp != NULL )
	{
		rval = close(fileno(Getc_unlocked_fp));
	}
	else
	{
		tet_infoline("fopen failed");
		++rval;
	}
	return(rval);
}

int
getc_unlocked_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getc_unlocked(Getc_unlocked_fp);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getc_unlocked_cef()
{
	(void) fclose(Getc_unlocked_fp);
	(void) remove("./nonesuch");
	return(0);
}



/****** Testset for getchar()					 ******/


#ifdef	getchar
#undef	getchar
#endif

int
getchar_csf()
{
	return(0);
}

int
getchar_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getchar_cef()
{
	return(0);
}



/****** Testset for getchar_unlocked()				 ******/

#ifdef	getchar_unlocked
#undef	getchar_unlocked
#endif

int
getchar_unlocked_csf()
{
	return(0);
}

int
getchar_unlocked_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getchar_unlocked_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for getcontext()				 ******/

#ifdef	getcontext
#undef	getcontext
#endif

int
getcontext_csf()
{
	return(0);
}

int
getcontext_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	ucontext_t	context;

	errs	= 0;
	fails	= 0;

	(void) getcontext(&context);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getcontext_cef()
{
	return(0);
}

#endif


/****** Testset for getcwd()					 ******/

#ifdef	getcwd
#undef	getcwd
#endif

int
getcwd_csf()
{
	return(0);
}

int
getcwd_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) getcwd(&buf[0], 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getcwd_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for getdate()					 ******/

#ifdef	getdate
#undef	getdate
#endif

int
getdate_csf()
{
	return(0);
}

int
getdate_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getdate("");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getdate_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for getdtablesize()				 ******/

#ifdef	getdtablesize
#undef	getdtablesize
#endif

int
getdtablesize_csf()
{
	return(0);
}

int
getdtablesize_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getdtablesize();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getdtablesize_cef()
{
	return(0);
}

#endif
#endif


/****** Testset for getegid()					 ******/

#ifdef	getegid
#undef	getegid
#endif

int
getegid_csf()
{
	return(0);
}

int
getegid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getegid();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getegid_cef()
{
	return(0);
}



/****** Testset for getenv()					 ******/

#ifdef	getenv
#undef	getenv
#endif

int
getenv_csf()
{
	return(0);
}

int
getenv_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getenv("");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getenv_cef()
{
	return(0);
}



/****** Testset for geteuid()					 ******/

#ifdef	geteuid
#undef	geteuid
#endif

int
geteuid_csf()
{
	return(0);
}

int
geteuid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) geteuid();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
geteuid_cef()
{
	return(0);
}



/****** Testset for getgid()					 ******/

#ifdef	getgid
#undef	getgid
#endif

int
getgid_csf()
{
	return(0);
}

int
getgid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getgid();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getgid_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for getgrent()					 ******/

#ifdef	getgrent
#undef	getgrent
#endif

int
getgrent_csf()
{
	return(0);
}

int
getgrent_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getgrent();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getgrent_cef()
{
	return(0);
}

#endif


/****** Testset for getgrgid()					 ******/

#ifdef	getgrgid
#undef	getgrgid
#endif

int
getgrgid_csf()
{
	return(0);
}

int
getgrgid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getgrgid((gid_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getgrgid_cef()
{
	return(0);
}


#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for getgrgid_r()				 ******/

#ifdef	getgrgid_r
#undef	getgrgid_r
#endif

int
getgrgid_r_csf()
{
	return(0);
}

int
getgrgid_r_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct group	grp;
	struct group	*result;
	char		buf[1];

	errs	= 0;
	fails	= 0;

	(void) getgrgid_r( (gid_t) 0, &grp, &buf[0], 0, &result);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getgrgid_r_cef()
{
	return(0);
}

#endif


/****** Testset for getgrnam()					 ******/

#ifdef	getgrnam
#undef	getgrnam
#endif

int
getgrnam_csf()
{
	return(0);
}

int
getgrnam_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getgrnam("");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getgrnam_cef()
{
	return(0);
}


#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for getgrnam_r()				 ******/

#ifdef	getgrnam_r
#undef	getgrnam_r
#endif

int
getgrnam_r_csf()
{
	return(0);
}

int
getgrnam_r_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	char		buf[1];
	struct group	grp;
	struct group	*result;

	errs	= 0;
	fails	= 0;


	(void) getgrnam_r("", &grp, &buf[0], 0, &result);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getgrnam_r_cef()
{
	return(0);
}

#endif


/****** Testset for getgroups()					 ******/

#ifdef	getgroups
#undef	getgroups
#endif

int
getgroups_csf()
{
	return(0);
}

int
getgroups_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	gid_t		groups[1];

	errs	= 0;
	fails	= 0;

	(void) getgroups(0, &groups[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getgroups_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for gethostid()					 ******/

#ifdef	gethostid
#undef	gethostid
#endif

int
gethostid_csf()
{
	return(0);
}

int
gethostid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) gethostid();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
gethostid_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for getitimer()					 ******/

#ifdef	getitimer
#undef	getitimer
#endif

int
getitimer_csf()
{
	return(0);
}

int
getitimer_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getitimer_cef()
{
	return(0);
}

#endif


/****** Testset for getlogin()					 ******/

#ifdef	getlogin
#undef	getlogin
#endif

int
getlogin_csf()
{
	return(0);
}

int
getlogin_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getlogin_cef()
{
	return(0);
}


#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for getlogin_r()				 ******/

#ifdef	getlogin_r
#undef	getlogin_r
#endif

int
getlogin_r_csf()
{
	return(0);
}

int
getlogin_r_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getlogin_r_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for getmsg()					 ******/

#ifdef	getmsg
#undef	getmsg
#endif

int
getmsg_csf()
{
	return(0);
}

int
getmsg_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	int		flags;
	char		buf[1];
	struct strbuf	ctl;
	struct strbuf	data;

	errs	= 0;
	fails	= 0;

	ctl.buf		= &buf[0];
	ctl.maxlen	= 1;

	data.buf	= &buf[0];
	data.maxlen	= 1;

	flags = 0;

	(void) getmsg(-1, &ctl, &data, &flags);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getmsg_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for getopt()					 ******/

#ifdef	getopt
#undef	getopt
#endif

int
getopt_csf()
{
	return(0);
}

int
getopt_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getopt_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for getpagesize()				 ******/

#ifdef	getpagesize
#undef	getpagesize
#endif

int
getpagesize_csf()
{
	return(0);
}

int
getpagesize_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getpagesize_cef()
{
	return(0);
}

#endif
#endif

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for getpass()					 ******/

#ifdef	getpass
#undef	getpass
#endif

int
getpass_csf()
{
	return(0);
}

int
getpass_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getpass_cef()
{
	return(0);
}

#endif
#endif

#if	( TEST_XPG > 4 )

/****** Testset for getpgid()					 ******/

#ifdef	getpgid
#undef	getpgid
#endif

int
getpgid_csf()
{
	return(0);
}

int
getpgid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getpgid_cef()
{
	return(0);
}

#endif


/****** Testset for getpgrp()					 ******/

#ifdef	getpgrp
#undef	getpgrp
#endif

int
getpgrp_csf()
{
	return(0);
}

int
getpgrp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getpgrp_cef()
{
	return(0);
}



/****** Testset for getpid()					 ******/

#ifdef	getpid
#undef	getpid
#endif

int
getpid_csf()
{
	return(0);
}

int
getpid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getpid_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for getpmsg()					 ******/

#ifdef	getpmsg
#undef	getpmsg
#endif

int
getpmsg_csf()
{
	return(0);
}

int
getpmsg_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	int		flags;
	int		band;
	char		buf[1];
	struct strbuf	ctl;
	struct strbuf	data;

	errs	= 0;
	fails	= 0;

	ctl.buf		= &buf[0];
	ctl.maxlen	= 1;
	data.buf	= &buf[0];
	data.maxlen	= 1;

	flags		= MSG_BAND;
	band		= 0;

	(void) getpmsg(-1, &ctl, &data, &band, &flags);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getpmsg_cef()
{
	return(0);
}

#endif


/****** Testset for getppid()					 ******/

#ifdef	getppid
#undef	getppid
#endif

int
getppid_csf()
{
	return(0);
}

int
getppid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getppid();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getppid_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for getpriority()				 ******/

#ifdef	getpriority
#undef	getpriority
#endif

int
getpriority_csf()
{
	return(0);
}

int
getpriority_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getpriority(PRIO_PROCESS, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getpriority_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for getpwent()					 ******/

#ifdef	getpwent
#undef	getpwent
#endif

int
getpwent_csf()
{
	return(0);
}

int
getpwent_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getpwent();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getpwent_cef()
{
	return(0);
}

#endif


/****** Testset for getpwnam()					 ******/

#ifdef	getpwnam
#undef	getpwnam
#endif

int
getpwnam_csf()
{
	return(0);
}

int
getpwnam_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getpwnam("");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getpwnam_cef()
{
	return(0);
}


#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for getpwnam_r()				 ******/

#ifdef	getpwnam_r
#undef	getpwnam_r
#endif

int
getpwnam_r_csf()
{
	return(0);
}

int
getpwnam_r_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	char		buf[1];
	struct passwd	pwd;
	struct passwd	*result;

	errs	= 0;
	fails	= 0;

	(void) getpwnam_r("", &pwd, &buf[0], 0, &result);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getpwnam_r_cef()
{
	return(0);
}

#endif


/****** Testset for getpwuid()					 ******/

#ifdef	getpwuid
#undef	getpwuid
#endif

int
getpwuid_csf()
{
	return(0);
}

int
getpwuid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getpwuid((uid_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getpwuid_cef()
{
	return(0);
}


#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for getpwuid_r()				 ******/

#ifdef	getpwuid_r
#undef	getpwuid_r
#endif

int
getpwuid_r_csf()
{
	return(0);
}

int
getpwuid_r_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	char		buf[1];
	struct passwd	pwd;
	struct passwd	*result;

	errs	= 0;
	fails	= 0;

	(void) getpwuid_r((uid_t) 0, &pwd, &buf[0], 0, &result);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getpwuid_r_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for getrlimit()					 ******/

#ifdef	getrlimit
#undef	getrlimit
#endif

int
getrlimit_csf()
{
	return(0);
}

int
getrlimit_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct rlimit	rl;


	errs	= 0;
	fails	= 0;

	(void) getrlimit(RLIMIT_CPU, &rl);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getrlimit_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for getrusage()					 ******/

#ifdef	getrusage
#undef	getrusage
#endif

int
getrusage_csf()
{
	return(0);
}

int
getrusage_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct rusage	rl;

	errs	= 0;
	fails	= 0;

	(void) getrusage(RUSAGE_SELF, &rl);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getrusage_cef()
{
	return(0);
}

#endif


/****** Testset for gets()					 ******/

#ifdef	gets
#undef	gets
#endif

int
gets_csf()
{
	return(0);
}

int
gets_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
gets_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for getsid()					 ******/

#ifdef	getsid
#undef	getsid
#endif

int
getsid_csf()
{
	return(0);
}

int
getsid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getsid((pid_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getsid_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for getsubopt()					 ******/

#ifdef	getsubopt
#undef	getsubopt
#endif

int
getsubopt_csf()
{
	return(0);
}

int
getsubopt_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	opts[4];
	char	*p;
	char	*toks[2];
	char	*vp;

	errs	= 0;
	fails	= 0;

	toks[0]	= "x";
	toks[1]	= NULL;
	p	= &opts[0];

	strcpy(&opts[0], "a=b");
	(void) getsubopt(&p, &toks[0], &vp);

	return(0);
}

int
getsubopt_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for gettimeofday()				 ******/

#ifdef	gettimeofday
#undef	gettimeofday
#endif

int
gettimeofday_csf()
{
	return(0);
}

int
gettimeofday_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct timeval	tv;

	errs	= 0;
	fails	= 0;

	tv.tv_sec	= 0;
	tv.tv_usec	= 0;

	(void) gettimeofday(&tv, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
gettimeofday_cef()
{
	return(0);
}

#endif


/****** Testset for getuid()					 ******/

#ifdef	getuid
#undef	getuid
#endif

int
getuid_csf()
{
	return(0);
}

int
getuid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getuid();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getuid_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for getutxent()					 ******/

#ifdef	getutxent
#undef	getutxent
#endif

int
getutxent_csf()
{
	return(0);
}

int
getutxent_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getutxent_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for getutxid()					 ******/

#ifdef	getutxid
#undef	getutxid
#endif

int
getutxid_csf()
{
	return(0);
}

int
getutxid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getutxent();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getutxid_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for getutxline()				 ******/

#ifdef	getutxline
#undef	getutxline
#endif

int
getutxline_csf()
{
	return(0);
}

int
getutxline_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct utmpx	line;

	errs	= 0;
	fails	= 0;

	(void) getutxline(&line);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getutxline_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for getw()					 ******/

#ifdef	getw
#undef	getw
#endif

static FILE	*Getw_fp;

int
getw_csf()
{
	int	rval;

	rval = 0;

	Getw_fp = fopen("./nonesuch", "w+");
	if ( Getw_fp != NULL )
	{
		rval = close(fileno(Getw_fp));
	}
	else
	{
		++rval;
	}
	return(rval);
}

int
getw_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getw(Getw_fp);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getw_cef()
{
	(void) fclose(Getw_fp);
	(void) remove("./nonesuch");
	return(0);
}

#endif
#endif

#if	( TEST_XPG > 4 )

/****** Testset for getwc()					 ******/

#ifdef	getwc
#undef	getwc
#endif

static FILE	*Getwc_fp;

int
getwc_csf()
{
	int	rval;

	rval = 0;

	Getwc_fp = fopen("./nonesuch", "w+");
	if ( Getwc_fp != NULL )
	{
		rval = close(fileno(Getwc_fp));
	}
	else
	{
		++rval;
	}
	return(rval);
}

int
getwc_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getwc(Getwc_fp);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getwc_cef()
{
	(void) fclose(Getwc_fp);
	(void) remove("./nonesuch");
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for getwchar()					 ******/

#ifdef	getwchar
#undef	getwchar
#endif

int
getwchar_csf()
{
	return(0);
}

int
getwchar_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getwchar_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for getwd()					 ******/

#ifdef	getwd
#undef	getwd
#endif

char	*Getwd_buf;

int
getwd_csf()
{
	Getwd_buf = (char *) malloc((pathconf(".", _PC_PATH_MAX) + 1) * sizeof(char));
	if ( Getwd_buf == NULL)
		return(1);
	else
		return(0);
}

int
getwd_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) getwd(Getwd_buf);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getwd_cef()
{
	if (Getwd_buf != NULL)
		(void) free(Getwd_buf);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for glob()					 ******/

#ifdef	glob
#undef	glob
#endif

static int
glob_func(const char *epath, int errno)
{
	return(1);
}

int
glob_csf()
{
	return(0);
}

int
glob_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	glob_t	g;

	errs	= 0;
	fails	= 0;

	(void) glob("", GLOB_ERR, glob_func, &g);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
glob_cef()
{
	return(0);
}

#endif


/****** Testset for gmtime()					 ******/

#ifdef	gmtime
#undef	gmtime
#endif

int
gmtime_csf()
{
	return(0);
}

int
gmtime_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	time_t	t;

	errs	= 0;
	fails	= 0;

	t = (time_t) 0;
	(void) gmtime(&t);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
gmtime_cef()
{
	return(0);
}


#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for gmtime_r()					 ******/

#ifdef	gmtime_r
#undef	gmtime_r
#endif

int
gmtime_r_csf()
{
	return(0);
}

int
gmtime_r_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	time_t		t;
	struct tm	tm;

	errs	= 0;
	fails	= 0;

	t = (time_t) 0;
	(void) gmtime_r(&t, &tm);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
gmtime_r_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for grantpt()					 ******/

#ifdef	grantpt
#undef	grantpt
#endif

int
grantpt_csf()
{
	return(0);
}

int
grantpt_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) grantpt(-1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
grantpt_cef()
{
	return(0);
}

#endif


