
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_cpi.c	1.14
*
*************************************************************************/


#include "std.h"

#include <ctype.h>

#if	( TEST_XPG > 4 )
#include <iconv.h>
#endif

#include <math.h>

#if	( TEST_XPG > 4 )
#include <search.h>
#endif

#include <stdio.h>
#include <stdlib.h>

#if	( TEST_XPG > 4 )
#include <strings.h>
#include <stropts.h>
#endif

#include <sys/types.h>
#include <unistd.h>

#if	( TEST_XPG > 4 )
#include <wctype.h>
#endif


#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"


#if	( TEST_XPG > 4 )

/****** Testset for iconv()					 ******/

#ifdef	iconv
#undef	iconv
#endif

static iconv_t	Iconv_desc;

int
iconv_csf()
{
	int	rval;
	char	*p;
	char	*q;

	rval = 1;

	p = tet_getvar("VSTH_ICONV_CODESET1");
	q = tet_getvar("VSTH_ICONV_CODESET2");
	if ( p && *p  && q && *q)
	{
		Iconv_desc = iconv_open(p, q);
		if ( Iconv_desc != (iconv_t) -1 )
		{
			rval = 0;
		}
	}

	return(rval);
}

int
iconv_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	size_t	inleft;
	size_t	outleft;
	char	inbuf[2];
	char	outbuf[2];
	char	*in;
	char	*out;

	errs	= 0;
	fails	= 0;

	in	= &inbuf[0];
	out	= &outbuf[0];
	inleft	= 0;
	(void) iconv(Iconv_desc, &in, &inleft, &out, &outleft);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iconv_cef()
{
	(void) iconv_close(Iconv_desc);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iconv_close()				 ******/

#ifdef	iconv_close
#undef	iconv_close
#endif

iconv_t		Iconv_close_desc;

int
iconv_close_csf()
{
	int	rval;
	char	*p;

	rval = 1;

	p = tet_getvar("VSTH_ICONV_CODESET");
	if ( p && *p )
	{
		Iconv_close_desc = iconv_open(p, p);
		if ( Iconv_close_desc != (iconv_t) -1 )
		{
			rval = 0;
		}
	}

	return(rval);
}

int
iconv_close_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iconv_close(Iconv_close_desc);

	return(0);
}

int
iconv_close_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iconv_open()				 ******/

#ifdef	iconv_open
#undef	iconv_open
#endif

static iconv_t	Iconv_open_desc;
char		*Iconv_open_codeset;

int
iconv_open_csf()
{
	int	rval;

	rval = 1;

	Iconv_open_codeset = tet_getvar("VSTH_ICONV_CODESET");
	if ( Iconv_open_codeset  && *Iconv_open_codeset )
	{
		rval = 0;
	}

	return(rval);
}

int
iconv_open_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;

	Iconv_open_desc = iconv_open(Iconv_open_codeset, Iconv_open_codeset);

	if ( Iconv_open_desc != (iconv_t) -1 )
	{
		rval = 0;
	}

	return(rval);
}

int
iconv_open_cef()
{
	(void) iconv_close(Iconv_open_desc);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for ilogb()					 ******/

#ifdef	ilogb
#undef	ilogb
#endif

int
ilogb_csf()
{
	return(0);
}

int
ilogb_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) ilogb(0.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ilogb_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for index()					 ******/

#ifdef	index
#undef	index
#endif

int
index_csf()
{
	return(0);
}

int
index_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) index("", 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
index_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for initstate()					 ******/

#ifdef	initstate
#undef	initstate
#endif

int
initstate_csf()
{
	return(0);
}

int
initstate_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[2];

	errs	= 0;
	fails	= 0;

	(void) initstate(0, &buf[0], 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
initstate_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for insque()					 ******/

#ifdef	insque
#undef	insque
#endif

struct qe {
	struct qe	*next;
	struct qe	*last;
	int		a;
};

int
insque_csf()
{
	return(0);
}

int
insque_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct qe	first;
	struct qe	second;

	errs	= 0;
	fails	= 0;

	first.next = (struct qe *) NULL;
	first.last = (struct qe *) NULL;

	(void) insque((void *) &second, (void *) &first);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
insque_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for ioctl()					 ******/

#ifdef	ioctl
#undef	ioctl
#endif

int
ioctl_csf()
{
	return(0);
}

int
ioctl_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	val;

	errs	= 0;
	fails	= 0;

	(void) ioctl(-1, I_GETCLTIME, &val);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ioctl_cef()
{
	return(0);
}

#endif


/****** Testset for isalnum()					 ******/

#ifdef	isalnum
#undef	isalnum
#endif

int
isalnum_csf()
{
	return(0);
}

int
isalnum_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) isalnum('X');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
isalnum_cef()
{
	return(0);
}



/****** Testset for isalpha()					 ******/

#ifdef	isalpha
#undef	isalpha
#endif

int
isalpha_csf()
{
	return(0);
}

int
isalpha_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) isalpha('X');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
isalpha_cef()
{
	return(0);
}



/****** Testset for isascii()					 ******/

#ifdef	isascii
#undef	isascii
#endif

int
isascii_csf()
{
	return(0);
}

int
isascii_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) isascii('X');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
isascii_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for isastream()					 ******/

#ifdef	isastream
#undef	isastream
#endif

int
isastream_csf()
{
	return(0);
}

int
isastream_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) isastream(-1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
isastream_cef()
{
	return(0);
}

#endif


/****** Testset for isatty()					 ******/

#ifdef	isatty
#undef	isatty
#endif

int
isatty_csf()
{
	return(0);
}

int
isatty_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) isatty(-1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
isatty_cef()
{
	return(0);
}



/****** Testset for iscntrl()					 ******/

#ifdef	iscntrl
#undef	iscntrl
#endif

int
iscntrl_csf()
{
	return(0);
}

int
iscntrl_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iscntrl('X');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iscntrl_cef()
{
	return(0);
}



/****** Testset for isdigit()					 ******/

#ifdef	isdigit
#undef	isdigit
#endif

int
isdigit_csf()
{
	return(0);
}

int
isdigit_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) isdigit('X');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
isdigit_cef()
{
	return(0);
}



/****** Testset for isgraph()					 ******/

#ifdef	isgraph
#undef	isgraph
#endif

int
isgraph_csf()
{
	return(0);
}

int
isgraph_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) isgraph('X');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
isgraph_cef()
{
	return(0);
}



/****** Testset for islower()					 ******/

#ifdef	islower
#undef	islower
#endif

int
islower_csf()
{
	return(0);
}

int
islower_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) islower('X');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
islower_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for isnan()					 ******/

#ifdef	isnan
#undef	isnan
#endif

int
isnan_csf()
{
	return(0);
}

int
isnan_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) isnan(0.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
isnan_cef()
{
	return(0);
}

#endif


/****** Testset for isprint()					 ******/

#ifdef	isprint
#undef	isprint
#endif

int
isprint_csf()
{
	return(0);
}

int
isprint_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) isprint('X');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
isprint_cef()
{
	return(0);
}



/****** Testset for ispunct()					 ******/

#ifdef	ispunct
#undef	ispunct
#endif

int
ispunct_csf()
{
	return(0);
}

int
ispunct_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) ispunct('X');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ispunct_cef()
{
	return(0);
}



/****** Testset for isspace()					 ******/

#ifdef	isspace
#undef	isspace
#endif

int
isspace_csf()
{
	return(0);
}

int
isspace_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) isspace('X');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
isspace_cef()
{
	return(0);
}



/****** Testset for isupper()					 ******/

#ifdef	isupper
#undef	isupper
#endif

int
isupper_csf()
{
	return(0);
}

int
isupper_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) isupper('X');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
isupper_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for iswalnum()					 ******/

#ifdef	iswalnum
#undef	iswalnum
#endif

int
iswalnum_csf()
{
	return(0);
}

int
iswalnum_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iswalnum((wint_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iswalnum_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iswalpha()					 ******/

#ifdef	iswalpha
#undef	iswalpha
#endif

int
iswalpha_csf()
{
	return(0);
}

int
iswalpha_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iswalpha((wint_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iswalpha_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iswcntrl()					 ******/

#ifdef	iswcntrl
#undef	iswcntrl
#endif

int
iswcntrl_csf()
{
	return(0);
}

int
iswcntrl_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iswcntrl((wint_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iswcntrl_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iswdigit()					 ******/

#ifdef	iswdigit
#undef	iswdigit
#endif

int
iswdigit_csf()
{
	return(0);
}

int
iswdigit_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iswdigit((wint_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iswdigit_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iswgraph()					 ******/

#ifdef	iswgraph
#undef	iswgraph
#endif

int
iswgraph_csf()
{
	return(0);
}

int
iswgraph_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iswgraph((wint_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iswgraph_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iswlower()					 ******/

#ifdef	iswlower
#undef	iswlower
#endif

int
iswlower_csf()
{
	return(0);
}

int
iswlower_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iswlower((wint_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iswlower_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iswprint()					 ******/

#ifdef	iswprint
#undef	iswprint
#endif

int
iswprint_csf()
{
	return(0);
}

int
iswprint_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iswprint((wint_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iswprint_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iswpunct()					 ******/

#ifdef	iswpunct
#undef	iswpunct
#endif

int
iswpunct_csf()
{
	return(0);
}

int
iswpunct_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iswpunct((wint_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iswpunct_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iswspace()					 ******/

#ifdef	iswspace
#undef	iswspace
#endif

int
iswspace_csf()
{
	return(0);
}

int
iswspace_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iswspace((wint_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iswspace_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iswupper()					 ******/

#ifdef	iswupper
#undef	iswupper
#endif

int
iswupper_csf()
{
	return(0);
}

int
iswupper_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iswupper((wint_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iswupper_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iswxdigit()					 ******/

#ifdef	iswxdigit
#undef	iswxdigit
#endif

int
iswxdigit_csf()
{
	return(0);
}

int
iswxdigit_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iswxdigit((wint_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iswxdigit_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for iswctype()					 ******/

#ifdef	iswctype
#undef	iswctype
#endif

int
iswctype_csf()
{
	return(0);
}

int
iswctype_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) iswctype((wint_t) 0, wctype("print"));

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
iswctype_cef()
{
	return(0);
}

#endif


