
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_cpl.c	1.8
*
*
* CHANGE HISTORY
*	12 May 1999 - Incorporate SR#104, to require ifdefs around aio.h
*************************************************************************/


#include "std.h"

#include <unistd.h>
#if	defined(_POSIX_ASYNCHRONOUS_IO)
#include <aio.h>
#endif
#include <locale.h>
#include <math.h>

#if	( TEST_XPG > 4 )
#include <search.h>
#endif

#include <setjmp.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>

#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"


#if	( TEST_XPG > 4 )

/****** Testset for l64a()					 ******/

#ifdef	l64a
#undef	l64a
#endif

int
l64a_csf()
{
	return(0);
}

int
l64a_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) l64a(0L);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
l64a_cef()
{
	return(0);
}

#endif


/****** Testset for labs()					 ******/

#ifdef	labs
#undef	labs
#endif

int
labs_csf()
{
	return(0);
}

int
labs_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) labs(0L);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
labs_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for lchown()					 ******/

#ifdef	lchown
#undef	lchown
#endif

int
lchown_csf()
{
	return(0);
}

int
lchown_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) lchown("", (uid_t) 0, (gid_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lchown_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for lcong48()					 ******/

#ifdef	lcong48
#undef	lcong48
#endif

int
lcong48_csf()
{
	return(0);
}

int
lcong48_ctf()
{
	int			rval;
	int			errs;
	int			fails;
	unsigned short int	param[7];

	errs	= 0;
	fails	= 0;

	(void) lcong48(&param[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lcong48_cef()
{
	return(0);
}

#endif


/****** Testset for ldexp()					 ******/

#ifdef	ldexp
#undef	ldexp
#endif

int
ldexp_csf()
{
	return(0);
}

int
ldexp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) ldexp(0.0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ldexp_cef()
{
	return(0);
}



/****** Testset for ldiv()					 ******/

#ifdef	ldiv
#undef	ldiv
#endif

int
ldiv_csf()
{
	return(0);
}

int
ldiv_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) ldiv(1, 1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ldiv_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for lfind()					 ******/

#ifdef	lfind
#undef	lfind
#endif

static int
lfind_func( const void *p1, const void *p2)
{
	return(0);
}

int
lfind_csf()
{
	return(0);
}

int
lfind_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	size_t	nelem;
	char	*buf[2];

	errs	= 0;
	fails	= 0;

	nelem	= 0;
	buf[0]	= NULL;
	(void) lfind((void *) "", (void *) &buf[0], &nelem, sizeof(char *), lfind_func);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lfind_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for lgamma()					 ******/

#ifdef	lgamma
#undef	lgamma
#endif

int
lgamma_csf()
{
	return(0);
}

int
lgamma_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) lgamma(0.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lgamma_cef()
{
	return(0);
}

#endif


/****** Testset for link()					 ******/

#ifdef	link
#undef	link
#endif

int
link_csf()
{
	return(0);
}

int
link_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) link("./nonesuch", "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
link_cef()
{
	return(0);
}


#if	defined(_POSIX_ASYNCHRONOUS_IO)

/****** Testset for lio_listio()				 ******/

#ifdef	lio_listio
#undef	lio_listio
#endif

int
lio_listio_csf()
{
	return(0);
}

int
lio_listio_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	char		buf[1];
	struct aiocb	op;
	struct aiocb	*list[1];

	errs	= 0;
	fails	= 0;

	op.aio_fildes		= -1;
	op.aio_offset		= 0;
	op.aio_buf		= &buf[0];
	op.aio_nbytes		= 0;
	op.aio_lio_opcode	= LIO_READ;

	list[0]			= &op;

	(void) lio_listio(LIO_WAIT, &list[0], 0, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lio_listio_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for localeconv()					 ******/

#ifdef	localeconv
#undef	localeconv
#endif

int
localeconv_csf()
{
	return(0);
}

int
localeconv_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) localeconv();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
localeconv_cef()
{
	return(0);
}

#endif


/****** Testset for localtime()					 ******/

#ifdef	localtime
#undef	localtime
#endif

static time_t	Localtime_timer;

int
localtime_csf()
{
	Localtime_timer = time(NULL);
	return(0);
}

int
localtime_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) localtime(&Localtime_timer);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
localtime_cef()
{
	return(0);
}


#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for localtime_r()				 ******/

#ifdef	localtime_r
#undef	localtime_r
#endif

static time_t	Localtime_r_timer;

int
localtime_r_csf()
{
	Localtime_r_timer = time(NULL);
	return(0);
}

int
localtime_r_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct tm	tm;

	errs	= 0;
	fails	= 0;

	(void) localtime_r(&Localtime_r_timer, &tm);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
localtime_r_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for lockf()					 ******/

#ifdef	lockf
#undef	lockf
#endif

int
lockf_csf()
{
	return(0);
}

int
lockf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) lockf(-1, F_TEST, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lockf_cef()
{
	return(0);
}

#endif


/****** Testset for log()					 ******/

#ifdef	log
#undef	log
#endif

int
log_csf()
{
	return(0);
}

int
log_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) log(0.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
log_cef()
{
	return(0);
}



/****** Testset for log10()					 ******/

#ifdef	log10
#undef	log10
#endif

int
log10_csf()
{
	return(0);
}

int
log10_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) log10(0.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
log10_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for log1p()					 ******/

#ifdef	log1p
#undef	log1p
#endif

int
log1p_csf()
{
	return(0);
}

int
log1p_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) log1p(0.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
log1p_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for logb()					 ******/

#ifdef	logb
#undef	logb
#endif

int
logb_csf()
{
	return(0);
}

int
logb_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) logb(0.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
logb_cef()
{
	return(0);
}

#endif


/****** Testset for longjmp()					 ******/

#ifdef	longjmp
#undef	longjmp
#endif

int
longjmp_csf()
{
	return(0);
}

int
longjmp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
longjmp_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for lrand48()					 ******/

#ifdef	lrand48
#undef	lrand48
#endif

int
lrand48_csf()
{
	return(0);
}

int
lrand48_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) lrand48();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lrand48_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for lsearch()					 ******/

#ifdef	lsearch
#undef	lsearch
#endif

static int
lsearch_func( const void *p1, const void *p2 )
{
	return(0);
}

int
lsearch_csf()
{
	return(0);
}

int
lsearch_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	size_t	nelem;
	char	*buf[2];

	errs	= 0;
	fails	= 0;

	nelem	= 1;
	buf[0]	= "";
	buf[1]	= NULL;

	(void) lsearch((const void *) "", (void *) &buf[0], &nelem, sizeof(char *), lsearch_func); 

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lsearch_cef()
{
	return(0);
}

#endif


/****** Testset for lseek()					 ******/

#ifdef	lseek
#undef	lseek
#endif

int
lseek_csf()
{
	return(0);
}

int
lseek_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) lseek(-1, 0, SEEK_CUR);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lseek_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for lstat()					 ******/

#ifdef	lstat
#undef	lstat
#endif

int
lstat_csf()
{
	return(0);
}

int
lstat_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct stat	sb;

	errs	= 0;
	fails	= 0;

	(void) lstat("./nonesuch", &sb);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lstat_cef()
{
	return(0);
}

#endif


