
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_cpm.c	1.14
*
*
* CHANGE HISTORY
*	12 May 1999 - Incorporate SR#104, to require ifdefs around mqueue.h
*************************************************************************/


#include "std.h"

#include <errno.h>
#include <fcntl.h>
#include <math.h>
#include <unistd.h>

#if	defined(_POSIX_MESSAGE_PASSING)
#include <mqueue.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>

#if	( TEST_XPG > 4 )
#include <sys/msg.h>
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>

#if	( TEST_XPG > 4 )
#include <ucontext.h>
#endif


#if	( TEST_XPG > 4 )
#include <wchar.h>
#endif


#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"


#if	( TEST_XPG > 4 )

/****** Testset for makecontext()				 ******/

#ifdef	makecontext
#undef	makecontext
#endif

static int
makecontext_func()
{
	return(0);
}

int
makecontext_csf()
{
	return(0);
}

int
makecontext_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
makecontext_cef()
{
	return(0);
}

#endif


/****** Testset for malloc()					 ******/

#ifdef	malloc
#undef	malloc
#endif

static char	*Malloc_addr;

int
malloc_csf()
{
	return(0);
}

int
malloc_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	Malloc_addr = (char *) malloc(1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
malloc_cef()
{
	if (Malloc_addr != NULL)
	{
		(void) free(Malloc_addr);
	}
	return(0);
}



/****** Testset for mblen()					 ******/

#ifdef	mblen
#undef	mblen
#endif

int
mblen_csf()
{
	return(0);
}

int
mblen_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) mblen(&buf[0], 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mblen_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for mbrlen()					 ******/

#ifdef	mbrlen
#undef	mbrlen
#endif

int
mbrlen_csf()
{
	return(0);
}

int
mbrlen_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) mbrlen(&buf[0], 0, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mbrlen_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for mbrtowc()					 ******/

#ifdef	mbrtowc
#undef	mbrtowc
#endif

int
mbrtowc_csf()
{
	return(0);
}

int
mbrtowc_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	wchar_t	wbuf[1];
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) mbrtowc(&wbuf[0], &buf[0], 0, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mbrtowc_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for mbsinit()					 ******/

#ifdef	mbsinit
#undef	mbsinit
#endif

int
mbsinit_csf()
{
	return(0);
}

int
mbsinit_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) mbsinit(NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mbsinit_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for mbsrtowcs()					 ******/

#ifdef	mbsrtowcs
#undef	mbsrtowcs
#endif

int
mbsrtowcs_csf()
{
	return(0);
}

int
mbsrtowcs_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	wchar_t	wbuf[1];
	char	buf[1];
	char	*ps;

	errs	= 0;
	fails	= 0;

	ps = &buf[0];
	(void)mbsrtowcs(&wbuf[0], (const char **) &ps, 0, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mbsrtowcs_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for mbstowcs()					 ******/

#ifdef	mbstowcs
#undef	mbstowcs
#endif

int
mbstowcs_csf()
{
	return(0);
}

int
mbstowcs_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mbstowcs_cef()
{
	return(0);
}

#endif


/****** Testset for mbtowc()					 ******/

#ifdef	mbtowc
#undef	mbtowc
#endif

int
mbtowc_csf()
{
	return(0);
}

int
mbtowc_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	wchar_t	wbuf[1];
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) mbtowc(&wbuf[0], &buf[0], 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mbtowc_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for memccpy()					 ******/

#ifdef	memccpy
#undef	memccpy
#endif

int
memccpy_csf()
{
	return(0);
}

int
memccpy_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf1[1];
	char	buf2[1];

	errs	= 0;
	fails	= 0;

	(void) memccpy(&buf1[0], &buf2[0], 0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
memccpy_cef()
{
	return(0);
}

#endif


/****** Testset for memchr()					 ******/

#ifdef	memchr
#undef	memchr
#endif

int
memchr_csf()
{
	return(0);
}

int
memchr_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) memchr(&buf[0], 0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
memchr_cef()
{
	return(0);
}



/****** Testset for memcmp()					 ******/

#ifdef	memcmp
#undef	memcmp
#endif

int
memcmp_csf()
{
	return(0);
}

int
memcmp_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf1[1];
	char	buf2[1];

	errs	= 0;
	fails	= 0;

	(void) memcmp(&buf1[0], &buf2[0], 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
memcmp_cef()
{
	return(0);
}



/****** Testset for memcpy()					 ******/

#ifdef	memcpy
#undef	memcpy
#endif

int
memcpy_csf()
{
	return(0);
}

int
memcpy_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf1[1];
	char	buf2[1];

	errs	= 0;
	fails	= 0;

	(void) memcpy(&buf1[0], &buf2[0], 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
memcpy_cef()
{
	return(0);
}



/****** Testset for memmove()					 ******/

#ifdef	memmove
#undef	memmove
#endif

int
memmove_csf()
{
	return(0);
}

int
memmove_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf1[1];
	char	buf2[1];

	errs	= 0;
	fails	= 0;

	(void) memmove(&buf1[0], &buf2[0], 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
memmove_cef()
{
	return(0);
}



/****** Testset for memset()					 ******/

#ifdef	memset
#undef	memset
#endif

int
memset_csf()
{
	return(0);
}

int
memset_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) memset(&buf[0], 0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
memset_cef()
{
	return(0);
}



/****** Testset for mkdir()					 ******/

#ifdef	mkdir
#undef	mkdir
#endif

int
mkdir_csf()
{
	return(0);
}

int
mkdir_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) mkdir(".", S_IRWXU);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mkdir_cef()
{
	return(0);
}



/****** Testset for mkfifo()					 ******/

#ifdef	mkfifo
#undef	mkfifo
#endif

int
mkfifo_csf()
{
	return(0);
}

int
mkfifo_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) mkfifo(".", S_IRWXU);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mkfifo_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for mknod()					 ******/

#ifdef	mknod
#undef	mknod
#endif

int
mknod_csf()
{
	return(0);
}

int
mknod_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) mknod(".", S_IFIFO | S_IRWXU, (dev_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mknod_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for mkstemp()					 ******/

#ifdef	mkstemp
#undef	mkstemp
#endif

static int	Mkstemp_fd;
static char	Mkstemp_buf[8];

int
mkstemp_csf()
{
	return(0);
}

int
mkstemp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	Mkstemp_buf[0] = 'a';
	Mkstemp_buf[1] = 'X';
	Mkstemp_buf[2] = 'X';
	Mkstemp_buf[3] = 'X';
	Mkstemp_buf[4] = 'X';
	Mkstemp_buf[5] = 'X';
	Mkstemp_buf[6] = 'X';
	Mkstemp_buf[7] = '\0';

	Mkstemp_fd = mkstemp(&Mkstemp_buf[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mkstemp_cef()
{
	(void) close(Mkstemp_fd);
	(void) remove(&Mkstemp_buf[0]);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for mktemp()					 ******/

#ifdef	mktemp
#undef	mktemp
#endif

int
mktemp_csf()
{
	return(0);
}

int
mktemp_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[8];

	errs	= 0;
	fails	= 0;

	buf[0]	= 'a';
	buf[1]	= 'X';
	buf[2]	= 'X';
	buf[3]	= 'X';
	buf[4]	= 'X';
	buf[5]	= 'X';
	buf[6]	= 'X';
	buf[7]	= '\0';

	(void) mktemp(&buf[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mktemp_cef()
{
	return(0);
}

#endif


/****** Testset for mktime()					 ******/

#ifdef	mktime
#undef	mktime
#endif

static struct tm	*Mktime_ptm;

int
mktime_csf()
{
	time_t	t;

	t = time(NULL);
	Mktime_ptm = localtime(&t);
	return(0);
}

int
mktime_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) mktime(Mktime_ptm);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mktime_cef()
{
	return(0);
}


#if	defined(_POSIX_MEMLOCK_RANGE)

/****** Testset for mlock()					 ******/

#ifdef	mlock
#undef	mlock
#endif

static void	*Mlock_addr;
static int	Mlock_fd;

int
mlock_csf()
{
	int	rval;

	rval = 0;

	Mlock_fd = open("./nonesuch", O_RDWR | O_CREAT, S_IRWXU);
	if ( Mlock_fd >= 0 )
	{
		Mlock_addr = mmap(NULL, sizeof(int), PROT_READ | PROT_WRITE, MAP_PRIVATE, Mlock_fd, 0);
		if ( Mlock_addr == MAP_FAILED )
		{
			++rval;
			tet_printf("mmap failed, errno == %d", errno);
		}
	}
	else
	{
		++rval;
		tet_printf("open failed, errno == %d", errno);
	}
	return(rval);
}

int
mlock_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) mlock(Mlock_addr, sizeof(int));

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mlock_cef()
{
	if ( Mlock_addr != MAP_FAILED )
	{
		(void) munlock(Mlock_addr, sizeof(int));
	}
	(void) close(Mlock_fd);
	(void) remove("./nonesuch");
	return(0);
}

#endif

#if	defined(_POSIX_MEMLOCK)

/****** Testset for mlockall()					 ******/

#ifdef	mlockall
#undef	mlockall
#endif

int
mlockall_csf()
{
	return(0);
}

int
mlockall_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) mlockall(MCL_CURRENT);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mlockall_cef()
{
	(void) munlockall();
	return(0);
}

#endif

#if	defined(_POSIX_MAPPED_FILES) || defined(_POSIX_SHARED_MEMORY_OBJECTS)

/****** Testset for mmap()					 ******/

#ifdef	mmap
#undef	mmap
#endif

static void	*Mmap_addr;
static int	Mmap_fd;

int
mmap_csf()
{
	int	rval;

	rval = 0;

	Mmap_fd = open("./nonesuch", O_RDWR | O_CREAT, S_IRWXU);
	if ( Mmap_fd < 0 )
	{
		++rval;
	}
	return(rval);
}

int
mmap_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	Mmap_addr = mmap(NULL, sizeof(int), PROT_READ | PROT_WRITE, MAP_PRIVATE, Mmap_fd, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mmap_cef()
{
	(void) remove("./nonesuch");

	if ( Mmap_addr != MAP_FAILED )
	{
		munmap(Mmap_addr, sizeof(int));
	}
	return(0);
}

#endif


/****** Testset for modf()					 ******/

#ifdef	modf
#undef	modf
#endif

int
modf_csf()
{
	return(0);
}

int
modf_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	double	val;

	errs	= 0;
	fails	= 0;

	(void) modf(0.0, &val);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
modf_cef()
{
	return(0);
}


#if	defined(_POSIX_MEMORY_PROTECTION)

/****** Testset for mprotect()					 ******/

#ifdef	mprotect
#undef	mprotect
#endif

static void	*Mprotect_addr;
static int	Mprotect_fd;

int
mprotect_csf()
{
	int	rval;

	rval = 0;

	Mprotect_fd = open("./nonesuch", O_RDWR | O_CREAT, S_IRWXU);
	if ( Mprotect_fd >= 0 )
	{
		Mprotect_addr = mmap(0, sizeof(int), PROT_READ | PROT_WRITE, MAP_PRIVATE, Mprotect_fd, 0);
		if ( Mprotect_addr == MAP_FAILED )
		{
			++rval;
			tet_printf("mmap failed, errno == %d", errno);
		}
	}
	else
	{
		++rval;
		tet_printf("open failed, errno == %d", errno);
	}
	return(rval);
}

int
mprotect_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) mprotect(Mprotect_addr, sizeof(int), PROT_READ | PROT_WRITE);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mprotect_cef()
{
	if ( Mprotect_addr != MAP_FAILED )
	{
		munmap(Mprotect_addr, sizeof(int));
	}
	if ( Mprotect_fd >= 0 )
	{
		(void) close(Mprotect_fd);
	}

	(void) remove("./nonesuch");

	return(0);
}

#endif

#if	defined(_POSIX_MESSAGE_PASSING)

/****** Testset for mq_close()					 ******/

#ifdef	mq_close
#undef	mq_close
#endif

int
mq_close_csf()
{
	return(0);
}

int
mq_close_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) mq_close((mqd_t) -1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mq_close_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_MESSAGE_PASSING)

/****** Testset for mq_getattr()				 ******/

#ifdef	mq_getattr
#undef	mq_getattr
#endif

int
mq_getattr_csf()
{
	return(0);
}

int
mq_getattr_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct mq_attr	ma;

	errs	= 0;
	fails	= 0;

	(void) mq_getattr((mqd_t) -1, &ma);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mq_getattr_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_MESSAGE_PASSING) && defined(_POSIX_REALTIME_SIGNALS)

/****** Testset for mq_notify()					 ******/

#ifdef	mq_notify
#undef	mq_notify
#endif

int
mq_notify_csf()
{
	return(0);
}

int
mq_notify_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) mq_notify((mqd_t) -1, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mq_notify_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_MESSAGE_PASSING)

/****** Testset for mq_open()					 ******/

#ifdef	mq_open
#undef	mq_open
#endif

static mqd_t	Mq_open_mqd;

int
mq_open_csf()
{
	return(0);
}

int
mq_open_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	Mq_open_mqd = mq_open("./nonesuch", O_RDONLY);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mq_open_cef()
{
	if ( Mq_open_mqd != (mqd_t) -1)
	{
		(void) mq_close(Mq_open_mqd);
	}
	return(0);
}

#endif

#if	defined(_POSIX_MESSAGE_PASSING)

/****** Testset for mq_receive()				 ******/

#ifdef	mq_receive
#undef	mq_receive
#endif

int
mq_receive_csf()
{
	return(0);
}

int
mq_receive_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	unsigned int	prio;
	char		buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	(void) mq_receive((mqd_t) -1, &buf[0], 0, &prio );

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mq_receive_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_MESSAGE_PASSING)

/****** Testset for mq_send()					 ******/

#ifdef	mq_send
#undef	mq_send
#endif

int
mq_send_csf()
{
	return(0);
}

int
mq_send_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) mq_send((mqd_t) -1, &buf[0], 0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mq_send_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_MESSAGE_PASSING)

/****** Testset for mq_setattr()				 ******/

#ifdef	mq_setattr
#undef	mq_setattr
#endif

int
mq_setattr_csf()
{
	return(0);
}

int
mq_setattr_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct mq_attr	set;
	struct mq_attr	get;

	errs	= 0;
	fails	= 0;

	(void) mq_setattr((mqd_t) -1, &set, &get);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mq_setattr_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_MESSAGE_PASSING)

/****** Testset for mq_unlink()					 ******/

#ifdef	mq_unlink
#undef	mq_unlink
#endif

int
mq_unlink_csf()
{
	return(0);
}

int
mq_unlink_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) mq_unlink("./nonesuch");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mq_unlink_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for mrand48()					 ******/

#ifdef	mrand48
#undef	mrand48
#endif

int
mrand48_csf()
{
	return(0);
}

int
mrand48_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) mrand48();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
mrand48_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for msgctl()					 ******/

#ifdef	msgctl
#undef	msgctl
#endif

int
msgctl_csf()
{
	return(0);
}

int
msgctl_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct msqid_ds	buf;

	errs	= 0;
	fails	= 0;

	(void) msgctl(-1, IPC_STAT, &buf);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
msgctl_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for msgget()					 ******/

#ifdef	msgget
#undef	msgget
#endif

static int	Msgget_id;

int
msgget_csf()
{
	return(0);
}

int
msgget_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	Msgget_id = msgget(IPC_PRIVATE, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
msgget_cef()
{
	(void) msgctl(Msgget_id, IPC_RMID, NULL);

	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for msgrcv()					 ******/

#ifdef	msgrcv
#undef	msgrcv
#endif

int
msgrcv_csf()
{
	return(0);
}

int
msgrcv_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) msgrcv(-1, (void *) &buf[0], (size_t) 0, 0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
msgrcv_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for msgsnd()					 ******/

#ifdef	msgsnd
#undef	msgsnd
#endif

int
msgsnd_csf()
{
	return(0);
}

int
msgsnd_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) msgsnd(-1, (const void *) &buf[0], 0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
msgsnd_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_MAPPED_FILES) && defined(_POSIX_SYNCHRONIZED_IO)

/****** Testset for msync()					 ******/

#ifdef	msync
#undef	msync
#endif

static int	Msync_fd;
void		*Msync_addr;

int
msync_csf()
{
	int	rval;

	rval = 0;

	Msync_fd = open( "./nonesuch", O_RDWR | O_CREAT | O_TRUNC, S_IRWXU );
	if ( Msync_fd >= 0 )
	{
		rval = ftruncate(Msync_fd, sizeof(void *));
		if ( rval == 0 )
		{
			Msync_addr = mmap( NULL, sizeof(int), PROT_READ | PROT_WRITE,
					MAP_PRIVATE, Msync_fd, 0);
			if ( Msync_addr == (void *) MAP_FAILED )
			{
				++rval;
			}
		}
	}
	else
	{
		++rval;
	}

	return(rval);
}

int
msync_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) msync(Msync_addr, 0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
msync_cef()
{
	if ( Msync_addr != MAP_FAILED)
	{
		(void) munmap(Msync_addr, sizeof(int));
	}
	if ( Msync_fd >= 0 )
	{
		(void) close(Msync_fd);
	}
	(void) remove("./nonesuch");
	return(0);
}

#endif

#if	defined(_POSIX_MEMLOCK_RANGE)

/****** Testset for munlock()					 ******/

#ifdef	munlock
#undef	munlock
#endif

static void	*Munlock_addr;
static int	Munlock_fd;

int
munlock_csf()
{
	int	rval;

	rval = 0;

	Munlock_fd = open("./nonesuch", O_RDWR | O_CREAT, S_IRWXU);
	if ( Munlock_fd >= 0 )
	{
		rval = ftruncate(Munlock_fd, 4);
		if ( rval == 0 )
		{

			Munlock_addr = mmap(	0,
						sizeof(int),
						PROT_READ | PROT_WRITE,
						MAP_PRIVATE,
						Munlock_fd,
						0);
			if ( Munlock_addr != MAP_FAILED )
			{
				rval = mlock(Munlock_addr, sizeof(int));
			}
			else
			{
				++rval;
				tet_printf("mmap failed, errno == %d", errno);
			}
		}
	}
	return(rval);
}

int
munlock_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) munlock(Munlock_addr, sizeof(int));

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
munlock_cef()
{
	if ( Munlock_addr != MAP_FAILED )
	{
		(void) munmap(Munlock_addr, sizeof(int));
	}
	if ( Munlock_fd >= 0 )
	{
		(void) close(Munlock_fd);
	}
	(void) remove("./nonesuch");

	return(0);
}

#endif

#if	defined(_POSIX_MEMLOCK)

/****** Testset for munlockall()				 ******/

#ifdef	munlockall
#undef	munlockall
#endif

int
munlockall_csf()
{
	return(0);
}

int
munlockall_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) munlockall();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
munlockall_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_MAPPED_FILES) || defined(_POSIX_SHARED_MEMORY_OBJECTS)

/****** Testset for munmap()					 ******/

#ifdef	munmap
#undef	munmap
#endif

static int	Munmap_fd;
static void	*Munmap_addr;

int
munmap_csf()
{
	int	rval;

	rval = 0;

	Munmap_fd = open("./nonesuch", O_RDWR | O_CREAT, S_IRWXU);
	if ( Munmap_fd >= 0 )
	{
		Munmap_addr = mmap(	0,
					sizeof(int),
					PROT_READ | PROT_WRITE,
					MAP_PRIVATE,
					Munmap_fd,
					0 );

		if ( Munmap_addr == MAP_FAILED )
		{
			++rval;
			tet_printf("mmap failed, errno == %d", errno);
		}
	}
	return(rval);
}

int
munmap_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) munmap(Munmap_addr, sizeof(int));

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
munmap_cef()
{
	if ( Munmap_fd >= 0 )
	{
		(void) close(Munmap_fd);
	}
	(void) remove("./nonesuch");
	return(0);
}

#endif


