
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_cpp.c	1.17
*
*************************************************************************/


#include "std.h"

#include <errno.h>
#include <math.h>

#if	( TEST_XPG > 4 )
#include <poll.h>
#endif

#include <limits.h>
#include <pthread.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>

#if	( TEST_XPG > 4 )
#include <stropts.h>
#endif

#include <sys/types.h>
#include <unistd.h>

#if	( TEST_XPG > 4 )
#include <utmpx.h>
#include <wchar.h>
#endif


#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"



/****** Testset for pathconf()					 ******/

#ifdef	pathconf
#undef	pathconf
#endif

int
pathconf_csf()
{
	return(0);
}

int
pathconf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pathconf(".", _PC_NAME_MAX);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pathconf_cef()
{
	return(0);
}



/****** Testset for pause()					 ******/

#ifdef	pause
#undef	pause
#endif

int
pause_csf()
{
	return(0);
}

int
pause_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pause();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pause_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for pclose()					 ******/

#ifdef	pclose
#undef	pclose
#endif

static FILE	*Pclose_fp;

int
pclose_csf()
{
	Pclose_fp = popen("echo", "r");
	return(0);
}

int
pclose_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pclose(Pclose_fp);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pclose_cef()
{
	return(0);
}

#endif


/****** Testset for perror()					 ******/

#ifdef	perror
#undef	perror
#endif

int
perror_csf()
{
	return(0);
}

int
perror_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	errno = 0;
	(void) perror(NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
perror_cef()
{
	return(0);
}



/****** Testset for pipe()					 ******/

#ifdef	pipe
#undef	pipe
#endif

static int	Pipe_fd[2];

int
pipe_csf()
{
	return(0);
}

int
pipe_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pipe(Pipe_fd);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pipe_cef()
{
	(void) close(Pipe_fd[0]);
	(void) close(Pipe_fd[1]);
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for poll()					 ******/

#ifdef	poll
#undef	poll
#endif

int
poll_csf()
{
	return(0);
}

int
poll_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct pollfd	fds[1];

	errs	= 0;
	fails	= 0;

	fds[0].fd	= -1;
	fds[0].events	= POLLIN;
	fds[0].revents	= POLLOUT;

	(void) poll( &fds[0], 0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
poll_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for popen()					 ******/

#ifdef	popen
#undef	popen
#endif

static FILE	*Popen_fp;

int
popen_csf()
{
	return(0);
}

int
popen_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	Popen_fp = popen("echo", "r");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
popen_cef()
{
	(void) pclose(Popen_fp);
	return(0);
}

#endif


/****** Testset for pow()					 ******/

#ifdef	pow
#undef	pow
#endif

int
pow_csf()
{
	return(0);
}

int
pow_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pow(0.0, 0.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pow_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for pread()					 ******/

#ifdef	pread
#undef	pread
#endif

int
pread_csf()
{
	return(0);
}

int
pread_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) pread(-1, (void *) &buf[0], 0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pread_cef()
{
	return(0);
}

#endif


/****** Testset for printf()					 ******/

#ifdef	printf
#undef	printf
#endif

int
printf_csf()
{
	return(0);
}

int
printf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) printf("");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
printf_cef()
{
	return(0);
}


#if	defined(_POSIX_THREADS)

/****** Testset for pthread_atfork()				 ******/

#ifdef	pthread_atfork
#undef	pthread_atfork
#endif

int
pthread_atfork_csf()
{
	return(0);
}

int
pthread_atfork_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_atfork(NULL, NULL, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_atfork_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_attr_destroy()			 ******/

#ifdef	pthread_attr_destroy
#undef	pthread_attr_destroy
#endif

static pthread_attr_t	Pad_attr;

int
pthread_attr_destroy_csf()
{
	int	rval;
	rval = pthread_attr_init(&Pad_attr);
	return(rval);
}

int
pthread_attr_destroy_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_destroy(&Pad_attr);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_destroy_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_attr_getdetachstate()		 ******/

#ifdef	pthread_attr_getdetachstate
#undef	pthread_attr_getdetachstate
#endif

static pthread_attr_t	Pagds_attr;

int
pthread_attr_getdetachstate_csf()
{
	int	rval;

	rval = pthread_attr_init(&Pagds_attr);
	return(rval);
}

int
pthread_attr_getdetachstate_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	state;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_getdetachstate(&Pagds_attr, &state);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_getdetachstate_cef()
{
	(void) pthread_attr_destroy(&Pagds_attr);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_attr_getguardsize()			 ******/

#ifdef	pthread_attr_getguardsize
#undef	pthread_attr_getguardsize
#endif

static pthread_attr_t	Paggs_attr;

int
pthread_attr_getguardsize_csf()
{
	int	rval;

	rval = pthread_attr_init(&Paggs_attr);
	return(rval);
}

int
pthread_attr_getguardsize_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	size_t	size;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_getguardsize(&Paggs_attr, &size);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_getguardsize_cef()
{
	(void) pthread_attr_destroy(&Paggs_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)

/****** Testset for pthread_attr_getinheritsched()		 ******/

#ifdef	pthread_attr_getinheritsched
#undef	pthread_attr_getinheritsched
#endif

static pthread_attr_t	Pagis_attr;

int
pthread_attr_getinheritsched_csf()
{
	int	rval;

	rval = pthread_attr_init(&Pagis_attr);
	return(rval);
}

int
pthread_attr_getinheritsched_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	ihs;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_getinheritsched(&Pagis_attr, &ihs);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_getinheritsched_cef()
{
	(void) pthread_attr_destroy(&Pagis_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_attr_getschedparam()		 ******/

#ifdef	pthread_attr_getschedparam
#undef	pthread_attr_getschedparam
#endif

static pthread_attr_t	Pagsp_attr;

int
pthread_attr_getschedparam_csf()
{
	int	rval;

	rval = pthread_attr_init(&Pagsp_attr);
	return(rval);
}

int
pthread_attr_getschedparam_ctf()
{
	int			rval;
	int			errs;
	int			fails;
	struct sched_param	params;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_getschedparam(&Pagsp_attr, &params);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_getschedparam_cef()
{
	(void) pthread_attr_destroy(&Pagsp_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)

/****** Testset for pthread_attr_getschedpolicy()		 ******/

#ifdef	pthread_attr_getschedpolicy
#undef	pthread_attr_getschedpolicy
#endif

static pthread_attr_t	Pagspol_attr;

int
pthread_attr_getschedpolicy_csf()
{
	int	rval;

	rval = pthread_attr_init(&Pagspol_attr);
	return(rval);
}

int
pthread_attr_getschedpolicy_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	policy;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_getschedpolicy(&Pagspol_attr, &policy);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_getschedpolicy_cef()
{
	(void) pthread_attr_destroy(&Pagspol_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)

/****** Testset for pthread_attr_getscope()			 ******/

#ifdef	pthread_attr_getscope
#undef	pthread_attr_getscope
#endif

static pthread_attr_t	Pags_attr;

int
pthread_attr_getscope_csf()
{
	int	rval;

	rval = pthread_attr_init(&Pags_attr);
	return(rval);
}

int
pthread_attr_getscope_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	scope;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_getscope(&Pags_attr, &scope);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_getscope_cef()
{
	(void) pthread_attr_destroy(&Pags_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_ATTR_STACKADDR)

/****** Testset for pthread_attr_getstackaddr()			 ******/

#ifdef	pthread_attr_getstackaddr
#undef	pthread_attr_getstackaddr
#endif

static pthread_attr_t	Pagsa_attr;

int
pthread_attr_getstackaddr_csf()
{
	int	rval;

	rval = pthread_attr_init(&Pagsa_attr);
	return(rval);
}

int
pthread_attr_getstackaddr_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	void	*addr;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_getstackaddr(&Pagsa_attr, &addr);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_getstackaddr_cef()
{
	(void) pthread_attr_destroy(&Pagsa_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_ATTR_STACKSIZE)

/****** Testset for pthread_attr_getstacksize()			 ******/

#ifdef	pthread_attr_getstacksize
#undef	pthread_attr_getstacksize
#endif

static pthread_attr_t	Pagsz_attr;

int
pthread_attr_getstacksize_csf()
{
	int	rval;

	rval = pthread_attr_init(&Pagsz_attr);
	return(rval);
}

int
pthread_attr_getstacksize_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	size_t	size;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_getstacksize(&Pagsz_attr, &size);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_getstacksize_cef()
{
	(void) pthread_attr_destroy(&Pagsz_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_attr_init()				 ******/

#ifdef	pthread_attr_init
#undef	pthread_attr_init
#endif

static pthread_attr_t	Pai_attr;

int
pthread_attr_init_csf()
{
	return(0);
}

int
pthread_attr_init_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_init(&Pai_attr);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_init_cef()
{
	(void) pthread_attr_destroy(&Pai_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_attr_setdetachstate()		 ******/

#ifdef	pthread_attr_setdetachstate
#undef	pthread_attr_setdetachstate
#endif

static pthread_attr_t	Pasds_attr;

int
pthread_attr_setdetachstate_csf()
{
	int	rval;

	rval = pthread_attr_init(&Pasds_attr);
	return(rval);
}

int
pthread_attr_setdetachstate_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_setdetachstate(&Pasds_attr, PTHREAD_CREATE_JOINABLE);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_setdetachstate_cef()
{
	(void) pthread_attr_destroy(&Pasds_attr);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_attr_setguardsize()			 ******/

#ifdef	pthread_attr_setguardsize
#undef	pthread_attr_setguardsize
#endif

static pthread_attr_t	Pasgz_attr;

int
pthread_attr_setguardsize_csf()
{
	int	rval;

	rval = pthread_attr_init(&Pasgz_attr);
	return(rval);
}

int
pthread_attr_setguardsize_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_setguardsize(&Pasgz_attr, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_setguardsize_cef()
{
	(void) pthread_attr_destroy(&Pasgz_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)

/****** Testset for pthread_attr_setinheritsched()		 ******/

#ifdef	pthread_attr_setinheritsched
#undef	pthread_attr_setinheritsched
#endif

static pthread_attr_t	Pasihs_attr;

int
pthread_attr_setinheritsched_csf()
{
	int	rval;

	rval = pthread_attr_init(&Pasihs_attr);
	return(rval);
}

int
pthread_attr_setinheritsched_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_setinheritsched(&Pasihs_attr, PTHREAD_EXPLICIT_SCHED);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_setinheritsched_cef()
{
	(void) pthread_attr_destroy(&Pasihs_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_attr_setschedparam()		 ******/

#ifdef	pthread_attr_setschedparam
#undef	pthread_attr_setschedparam
#endif

static pthread_attr_t	Passp_attr;

int
pthread_attr_setschedparam_csf()
{
	int	rval;

	rval = pthread_attr_init(&Passp_attr);
	return(rval);
}

int
pthread_attr_setschedparam_ctf()
{
	int			rval;
	int			errs;
	int			fails;
	struct sched_param	params;

	errs	= 0;
	fails	= 0;

	params.sched_priority = 0;
	(void) pthread_attr_setschedparam(&Passp_attr, &params);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_setschedparam_cef()
{
	(void) pthread_attr_destroy(&Passp_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)

/****** Testset for pthread_attr_setschedpolicy()		 ******/

#ifdef	pthread_attr_setschedpolicy
#undef	pthread_attr_setschedpolicy
#endif

static pthread_attr_t	Passp_attr;

int
pthread_attr_setschedpolicy_csf()
{
	int	rval;

	rval = pthread_attr_init(&Passp_attr);
	return(rval);
}

int
pthread_attr_setschedpolicy_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_setschedpolicy(&Passp_attr, SCHED_RR);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_setschedpolicy_cef()
{
	(void) pthread_attr_destroy(&Passp_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)

/****** Testset for pthread_attr_setscope()			 ******/

#ifdef	pthread_attr_setscope
#undef	pthread_attr_setscope
#endif

static pthread_attr_t	Pass_attr;

int
pthread_attr_setscope_csf()
{
	int	rval;

	rval = pthread_attr_init(&Pass_attr);
	return(rval);
}

int
pthread_attr_setscope_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_setscope(&Pass_attr, PTHREAD_SCOPE_PROCESS);
	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_setscope_cef()
{
	(void) pthread_attr_destroy(&Pass_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_ATTR_STACKADDR)

/****** Testset for pthread_attr_setstackaddr()			 ******/

#ifdef	pthread_attr_setstackaddr
#undef	pthread_attr_setstackaddr
#endif

static pthread_attr_t	Passa_attr;
static void		*Passa_addr;


int
pthread_attr_setstackaddr_csf()
{
	int	rval;
	long	size;

	size = sysconf(_SC_THREAD_STACK_MIN) + 1;

	rval = pthread_attr_init(&Passa_attr);
	if ( rval == 0 )
	{
		Passa_addr = malloc((size_t) size * sizeof(char));
		if ( Passa_addr == NULL )
		{
			tet_infoline("malloc failed");
			++rval;
		}
	}

	return(rval);
}

int
pthread_attr_setstackaddr_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_setstackaddr(&Passa_attr, Passa_addr);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_setstackaddr_cef()
{
	(void) pthread_attr_destroy(&Passa_attr);
	if ( Passa_addr != NULL )
	{
		(void) free(Passa_addr);
	}
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_ATTR_STACKSIZE)

/****** Testset for pthread_attr_setstacksize()			 ******/

#ifdef	pthread_attr_setstacksize
#undef	pthread_attr_setstacksize
#endif

static pthread_attr_t	Passz_attr;

int
pthread_attr_setstacksize_csf()
{
	int	rval;

	rval = pthread_attr_init(&Passz_attr);
	return(rval);
}

int
pthread_attr_setstacksize_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_attr_setstacksize(	&Passz_attr,
						(size_t) sysconf(_SC_THREAD_STACK_MIN));

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_attr_setstacksize_cef()
{
	(void) pthread_attr_destroy(&Passz_attr);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_cancel()				 ******/

#ifdef	pthread_cancel
#undef	pthread_cancel
#endif

static pthread_t	Ptc_tid;

int
pthread_cancel_csf()
{
	int	rval;

	rval = pthread_create(&Ptc_tid, NULL, vsth_null_thread, NULL);

	return(rval);
}

int
pthread_cancel_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_cancel(Ptc_tid);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_cancel_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_cleanup_pop()			 ******/

static void
ptpop_func( void *arg )
{
}

int
pthread_cleanup_pop_csf()
{
	return(0);
}

int
pthread_cleanup_pop_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* must use in pairs in same lexical scope */
#ifndef	PUSHPOP_BUGS
	pthread_cleanup_push(ptpop_func, NULL);
	pthread_cleanup_pop(0);
#endif

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_cleanup_pop_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_cleanup_push()			 ******/

static void
ptpush_func( void *arg )
{
}

int
pthread_cleanup_push_csf()
{
	return(0);
}

int
pthread_cleanup_push_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* must use in pairs in same lexical scope */
#ifdef	PUSHPOP_BUGS
	pthread_cleanup_push(ptpush_func, NULL);
	pthread_cleanup_pop(0);
#endif

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_cleanup_push_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_cond_broadcast()			 ******/

#ifdef	pthread_cond_broadcast
#undef	pthread_cond_broadcast
#endif

static pthread_mutex_t	Pcb_mx = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t	Pcb_cv = PTHREAD_COND_INITIALIZER;

int
pthread_cond_broadcast_csf()
{
	return(0);
}

int
pthread_cond_broadcast_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_cond_broadcast(&Pcb_cv);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_cond_broadcast_cef()
{
	(void) pthread_mutex_destroy(&Pcb_mx);
	(void) pthread_cond_destroy(&Pcb_cv);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_cond_destroy()			 ******/

#ifdef	pthread_cond_destroy
#undef	pthread_cond_destroy
#endif

static pthread_cond_t	Pcd_cv = PTHREAD_COND_INITIALIZER;

int
pthread_cond_destroy_csf()
{
	return(0);
}

int
pthread_cond_destroy_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_cond_destroy(&Pcd_cv);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_cond_destroy_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_cond_init()				 ******/

#ifdef	pthread_cond_init
#undef	pthread_cond_init
#endif

static pthread_cond_t	Pci_cv;

int
pthread_cond_init_csf()
{
	return(0);
}

int
pthread_cond_init_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_cond_init(&Pci_cv, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_cond_init_cef()
{
	(void) pthread_cond_destroy(&Pci_cv);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_cond_signal()			 ******/

#ifdef	pthread_cond_signal
#undef	pthread_cond_signal
#endif

static pthread_cond_t	Pcs_cv = PTHREAD_COND_INITIALIZER;

int
pthread_cond_signal_csf()
{
	return(0);
}

int
pthread_cond_signal_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_cond_signal(&Pcs_cv);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_cond_signal_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_cond_timedwait()			 ******/

#ifdef	pthread_cond_timedwait
#undef	pthread_cond_timedwait
#endif

static pthread_mutex_t	Vsth_pctw_mx;
static pthread_cond_t	Vsth_pctw_cv;


int
pthread_cond_timedwait_csf()
{
	int	rval;

	rval = pthread_mutex_init(&Vsth_pctw_mx, NULL);
	if ( rval == 0 )
	{
		rval = pthread_cond_init (&Vsth_pctw_cv, NULL);
	}

	return(rval);
}

int
pthread_cond_timedwait_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct timespec	ts;

	errs	= 0;
	fails	= 0;

	ts.tv_sec	= 1;
	ts.tv_nsec	= 0;

	(void) pthread_cond_timedwait(&Vsth_pctw_cv, &Vsth_pctw_mx, &ts);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_cond_timedwait_cef()
{
	(void) pthread_mutex_destroy( &Vsth_pctw_mx );
	(void) pthread_cond_destroy ( &Vsth_pctw_cv );
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_cond_wait()				 ******/

#ifdef	pthread_cond_wait
#undef	pthread_cond_wait
#endif

static pthread_mutex_t	Pcw_mx = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t	Pcw_cv = PTHREAD_COND_INITIALIZER;

int
pthread_cond_wait_csf()
{
	return(0);
}

int
pthread_cond_wait_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_cond_wait(&Pcw_cv, &Pcw_mx);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_cond_wait_cef()
{
	(void) pthread_mutex_destroy(&Pcw_mx);
	(void) pthread_cond_destroy(&Pcw_cv);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_condattr_destroy()			 ******/

#ifdef	pthread_condattr_destroy
#undef	pthread_condattr_destroy
#endif

static pthread_condattr_t	Pcad_ca;

int
pthread_condattr_destroy_csf()
{
	int	rval;

	rval = pthread_condattr_init(&Pcad_ca);
	return(rval);
}

int
pthread_condattr_destroy_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_condattr_destroy(&Pcad_ca);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_condattr_destroy_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PROCESS_SHARED)

/****** Testset for pthread_condattr_getpshared()		 ******/

#ifdef	pthread_condattr_getpshared
#undef	pthread_condattr_getpshared
#endif

static pthread_condattr_t	Pcagps_ca;

int
pthread_condattr_getpshared_csf()
{
	int	rval;

	rval = pthread_condattr_init(&Pcagps_ca);
	return(rval);
}

int
pthread_condattr_getpshared_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	pshared;

	errs	= 0;
	fails	= 0;

	(void) pthread_condattr_getpshared(&Pcagps_ca, &pshared);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_condattr_getpshared_cef()
{
	(void) pthread_condattr_destroy(&Pcagps_ca);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_condattr_init()			 ******/

#ifdef	pthread_condattr_init
#undef	pthread_condattr_init
#endif

static pthread_condattr_t	Pcai_ca;

int
pthread_condattr_init_csf()
{
	return(0);
}

int
pthread_condattr_init_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_condattr_init(&Pcai_ca);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_condattr_init_cef()
{
	(void) pthread_condattr_destroy(&Pcai_ca);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PROCESS_SHARED)

/****** Testset for pthread_condattr_setpshared()		 ******/

#ifdef	pthread_condattr_setpshared
#undef	pthread_condattr_setpshared
#endif

static pthread_condattr_t	Pcasps_ca;

int
pthread_condattr_setpshared_csf()
{
	int	rval;

	rval = pthread_condattr_init(&Pcasps_ca);
	return(rval);
}

int
pthread_condattr_setpshared_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_condattr_setpshared(&Pcasps_ca, PTHREAD_PROCESS_PRIVATE);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_condattr_setpshared_cef()
{
	(void) pthread_condattr_destroy(&Pcasps_ca);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_create()				 ******/

#ifdef	pthread_create
#undef	pthread_create
#endif

static pthread_t	Pc_tid;

int
pthread_create_csf()
{
	return(0);
}

int
pthread_create_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_create(&Pc_tid, NULL, vsth_null_thread, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_create_cef()
{
	void	*tval;

	(void) pthread_join(Pc_tid, &tval);

	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_detach()				 ******/

#ifdef	pthread_detach
#undef	pthread_detach
#endif

static pthread_t	Pd_tid;

int
pthread_detach_csf()
{
	int	rval;

	rval = pthread_create(&Pd_tid, NULL, vsth_null_thread, NULL);

	return(rval);
}

int
pthread_detach_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_detach(Pd_tid);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_detach_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_equal()				 ******/

#ifdef	pthread_equal
#undef	pthread_equal
#endif

static pthread_t	Pe_tid;

int
pthread_equal_csf()
{
	Pe_tid = pthread_self();
	return(0);
}

int
pthread_equal_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_equal(Pe_tid, Pe_tid);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_equal_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_exit()				 ******/

#ifdef	pthread_exit
#undef	pthread_exit
#endif

int
pthread_exit_csf()
{
	return(0);
}

int
pthread_exit_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_exit(NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_exit_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_getconcurrency()			 ******/

#ifdef	pthread_getconcurrency
#undef	pthread_getconcurrency
#endif

int
pthread_getconcurrency_csf()
{
	return(0);
}

int
pthread_getconcurrency_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	(void) pthread_getconcurrency();

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_getconcurrency_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)

/****** Testset for pthread_getschedparam()			 ******/

#ifdef	pthread_getschedparam
#undef	pthread_getschedparam
#endif

static pthread_t	Pgsp_tid;

int
pthread_getschedparam_csf()
{
	Pgsp_tid = pthread_self();
	return(0);
}

int
pthread_getschedparam_ctf()
{
	int			rval;
	int			errs;
	int			fails;
	int			policy;
	struct sched_param	params;

	errs	= 0;
	fails	= 0;

	(void) pthread_getschedparam(Pgsp_tid, &policy, &params);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_getschedparam_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_getspecific()			 ******/

#ifdef	pthread_getspecific
#undef	pthread_getspecific
#endif

static pthread_key_t	Pgs_key;

int
pthread_getspecific_csf()
{
	int	rval;

	rval = pthread_key_create(&Pgs_key, NULL);

	return(rval);
}

int
pthread_getspecific_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_getspecific(Pgs_key);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_getspecific_cef()
{
	(void) pthread_key_delete(Pgs_key);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_join()				 ******/

#ifdef	pthread_join
#undef	pthread_join
#endif

static pthread_t	Vsth_ptj_tid;

int
pthread_join_csf()
{
	int	rval;

	rval = pthread_create(&Vsth_ptj_tid, NULL, vsth_null_thread, NULL);

	return(rval);
}

int
pthread_join_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	void	*tval;

	errs	= 0;
	fails	= 0;

	(void) pthread_join(Vsth_ptj_tid, &tval);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_join_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_key_create()			 ******/

#ifdef	pthread_key_create
#undef	pthread_key_create
#endif

static pthread_key_t	Pkc_key;

int
pthread_key_create_csf()
{
	return(0);
}

int
pthread_key_create_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_key_create(&Pkc_key, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_key_create_cef()
{
	(void) pthread_key_delete(Pkc_key);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_key_delete()			 ******/

#ifdef	pthread_key_delete
#undef	pthread_key_delete
#endif

static pthread_key_t	Pkd_key;

int
pthread_key_delete_csf()
{
	int	rval;

	rval = pthread_key_create(&Pkd_key, NULL);

	return(rval);
}

int
pthread_key_delete_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_key_delete(Pkd_key);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_key_delete_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_kill()				 ******/

#ifdef	pthread_kill
#undef	pthread_kill
#endif

static pthread_t	Pk_tid;

int
pthread_kill_csf()
{
	Pk_tid = pthread_self();
	return(0);
}

int
pthread_kill_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_kill(Pk_tid, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_kill_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_mutex_destroy()			 ******/

#ifdef	pthread_mutex_destroy
#undef	pthread_mutex_destroy
#endif

static pthread_mutex_t	Pmd_mx = PTHREAD_MUTEX_INITIALIZER;

int
pthread_mutex_destroy_csf()
{
	return(0);
}

int
pthread_mutex_destroy_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutex_destroy(&Pmd_mx);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutex_destroy_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIO_PROTECT)

/****** Testset for pthread_mutex_getprioceiling()		 ******/

#ifdef	pthread_mutex_getprioceiling
#undef	pthread_mutex_getprioceiling
#endif

static pthread_mutex_t	Pmgpc_mx = PTHREAD_MUTEX_INITIALIZER;

int
pthread_mutex_getprioceiling_csf()
{
	return(0);
}

int
pthread_mutex_getprioceiling_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	ceiling;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutex_getprioceiling(&Pmgpc_mx, &ceiling);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutex_getprioceiling_cef()
{
	(void) pthread_mutex_destroy(&Pmgpc_mx);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_mutex_init()			 ******/

#ifdef	pthread_mutex_init
#undef	pthread_mutex_init
#endif

static pthread_mutex_t	Pmi_mx;

int
pthread_mutex_init_csf()
{
	return(0);
}

int
pthread_mutex_init_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutex_init(&Pmi_mx, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutex_init_cef()
{
	(void) pthread_mutex_destroy(&Pmi_mx);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_mutex_lock()			 ******/

#ifdef	pthread_mutex_lock
#undef	pthread_mutex_lock
#endif

static pthread_mutex_t	Pml_mx = PTHREAD_MUTEX_INITIALIZER;

int
pthread_mutex_lock_csf()
{
	return(0);
}

int
pthread_mutex_lock_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutex_lock(&Pml_mx);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutex_lock_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIO_PROTECT)

/****** Testset for pthread_mutex_setprioceiling()		 ******/

#ifdef	pthread_mutex_setprioceiling
#undef	pthread_mutex_setprioceiling
#endif

static pthread_mutex_t	Pmspc_mx = PTHREAD_MUTEX_INITIALIZER;

int
pthread_mutex_setprioceiling_csf()
{
	return(0);
}

int
pthread_mutex_setprioceiling_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutex_setprioceiling(&Pmspc_mx, 0, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutex_setprioceiling_cef()
{
	(void) pthread_mutex_destroy(&Pmspc_mx);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_mutex_trylock()			 ******/

#ifdef	pthread_mutex_trylock
#undef	pthread_mutex_trylock
#endif

static pthread_mutex_t	Pmtl_mx = PTHREAD_MUTEX_INITIALIZER;

int
pthread_mutex_trylock_csf()
{
	return(0);
}

int
pthread_mutex_trylock_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutex_trylock(&Pmtl_mx);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutex_trylock_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_mutex_unlock()			 ******/

#ifdef	pthread_mutex_unlock
#undef	pthread_mutex_unlock
#endif

static pthread_mutex_t	Pmul_mx = PTHREAD_MUTEX_INITIALIZER;

int
pthread_mutex_unlock_csf()
{
	return(0);
}

int
pthread_mutex_unlock_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	oldstate;

	errs	= 0;
	fails	= 0;

	rval = pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &oldstate);
	if ( rval == 0 )
	{
		rval = pthread_mutex_lock( &Pmul_mx );
		if ( rval == 0 )
		{
			rval = pthread_setcancelstate( PTHREAD_CANCEL_ENABLE, &oldstate );
			if ( rval == 0 )
			{
				(void) pthread_mutex_unlock(&Pmul_mx);
			}
			else
			{
				++errs;
			}
		}
		else
		{
			++errs;
		}
	}
	else
	{
		++errs;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutex_unlock_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_mutexattr_destroy()			 ******/

#ifdef	pthread_mutexattr_destroy
#undef	pthread_mutexattr_destroy
#endif

static pthread_mutexattr_t	Pmad_ma;

int
pthread_mutexattr_destroy_csf()
{
	int	rval;

	rval = pthread_mutexattr_init(&Pmad_ma);

	return(rval);
}

int
pthread_mutexattr_destroy_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutexattr_destroy(&Pmad_ma);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutexattr_destroy_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIO_PROTECT)

/****** Testset for pthread_mutexattr_getprioceiling()		 ******/

#ifdef	pthread_mutexattr_getprioceiling
#undef	pthread_mutexattr_getprioceiling
#endif

static pthread_mutexattr_t	Pmagpc_ma;

int
pthread_mutexattr_getprioceiling_csf()
{
	int	rval;

	rval = pthread_mutexattr_init(&Pmagpc_ma);

	return(rval);
}

int
pthread_mutexattr_getprioceiling_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	ceiling;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutexattr_getprioceiling(&Pmagpc_ma, &ceiling);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutexattr_getprioceiling_cef()
{
	(void) pthread_mutexattr_destroy(&Pmagpc_ma);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && (defined(_POSIX_THREAD_PRIO_INHERIT) || defined(_POSIX_THREAD_PRIO_PROTECT))

/****** Testset for pthread_mutexattr_getprotocol()		 ******/

#ifdef	pthread_mutexattr_getprotocol
#undef	pthread_mutexattr_getprotocol
#endif

static pthread_mutexattr_t	Pmagp_ma;

int
pthread_mutexattr_getprotocol_csf()
{
	int	rval;

	rval = pthread_mutexattr_init(&Pmagp_ma);

	return(rval);
}

int
pthread_mutexattr_getprotocol_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	protocol;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutexattr_getprotocol(&Pmagp_ma, &protocol);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutexattr_getprotocol_cef()
{
	(void) pthread_mutexattr_destroy(&Pmagp_ma);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PROCESS_SHARED)

/****** Testset for pthread_mutexattr_getpshared()		 ******/

#ifdef	pthread_mutexattr_getpshared
#undef	pthread_mutexattr_getpshared
#endif

static pthread_mutexattr_t	Pmagps_ma;

int
pthread_mutexattr_getpshared_csf()
{
	int	rval;

	rval = pthread_mutexattr_init(&Pmagps_ma);

	return(rval);
}

int
pthread_mutexattr_getpshared_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	pshared;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutexattr_getpshared(&Pmagps_ma, &pshared);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutexattr_getpshared_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_mutexattr_gettype()			 ******/

#ifdef	pthread_mutexattr_gettype
#undef	pthread_mutexattr_gettype
#endif

static pthread_mutexattr_t	Pmagt_ma;

int
pthread_mutexattr_gettype_csf()
{
	int	rval;

	rval = pthread_mutexattr_init(&Pmagt_ma);

	return(rval);
}

int
pthread_mutexattr_gettype_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	type;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutexattr_gettype(&Pmagt_ma, &type);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutexattr_gettype_cef()
{
	(void) pthread_mutexattr_destroy(&Pmagt_ma);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_mutexattr_init()			 ******/

#ifdef	pthread_mutexattr_init
#undef	pthread_mutexattr_init
#endif

static pthread_mutexattr_t	Pmai_ma;

int
pthread_mutexattr_init_csf()
{
	return(0);
}

int
pthread_mutexattr_init_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutexattr_init(&Pmai_ma);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutexattr_init_cef()
{
	(void) pthread_mutexattr_destroy(&Pmai_ma);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIO_PROTECT)

/****** Testset for pthread_mutexattr_setprioceiling()		 ******/

#ifdef	pthread_mutexattr_setprioceiling
#undef	pthread_mutexattr_setprioceiling
#endif

static pthread_mutexattr_t	Pmaspc_ma;

int
pthread_mutexattr_setprioceiling_csf()
{
	int	rval;

	rval = pthread_mutexattr_init(&Pmaspc_ma);

	return(rval);
}

int
pthread_mutexattr_setprioceiling_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutexattr_setprioceiling(&Pmaspc_ma, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutexattr_setprioceiling_cef()
{
	(void) pthread_mutexattr_destroy(&Pmaspc_ma);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && (defined(_POSIX_THREAD_PRIO_INHERIT) || defined(_POSIX_THREAD_PRIO_PROTECT))

/****** Testset for pthread_mutexattr_setprotocol()		 ******/

#ifdef	pthread_mutexattr_setprotocol
#undef	pthread_mutexattr_setprotocol
#endif

static pthread_mutexattr_t	Pmasp_ma;

int
pthread_mutexattr_setprotocol_csf()
{
	int	rval;

	rval = pthread_mutexattr_init(&Pmasp_ma);

	return(rval);
}

int
pthread_mutexattr_setprotocol_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutexattr_setprotocol(&Pmasp_ma, PTHREAD_PRIO_PROTECT);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutexattr_setprotocol_cef()
{
	(void) pthread_mutexattr_destroy(&Pmasp_ma);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PROCESS_SHARED)

/****** Testset for pthread_mutexattr_setpshared()		 ******/

#ifdef	pthread_mutexattr_setpshared
#undef	pthread_mutexattr_setpshared
#endif

static pthread_mutexattr_t	Pmasps_ma;

int
pthread_mutexattr_setpshared_csf()
{
	int	rval;

	rval = pthread_mutexattr_init(&Pmasps_ma);

	return(rval);
}

int
pthread_mutexattr_setpshared_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutexattr_setpshared(&Pmasps_ma, PTHREAD_PROCESS_SHARED);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutexattr_setpshared_cef()
{
	(void) pthread_mutexattr_destroy(&Pmasps_ma);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_mutexattr_settype()			 ******/

#ifdef	pthread_mutexattr_settype
#undef	pthread_mutexattr_settype
#endif

static pthread_mutexattr_t	Pmast_ma;

int
pthread_mutexattr_settype_csf()
{
	int	rval;

	rval = pthread_mutexattr_init(&Pmast_ma);

	return(rval);
}

int
pthread_mutexattr_settype_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_mutexattr_settype(&Pmast_ma, PTHREAD_MUTEX_NORMAL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_mutexattr_settype_cef()
{
	(void) pthread_mutexattr_destroy(&Pmast_ma);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_once()				 ******/

#ifdef	pthread_once
#undef	pthread_once
#endif

static pthread_once_t	Po_once = PTHREAD_ONCE_INIT;

static void
po_func()
{
}

int
pthread_once_csf()
{
	return(0);
}

int
pthread_once_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_once(&Po_once, po_func);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_once_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_rwlock_destroy()			 ******/

#ifdef	pthread_rwlock_destroy
#undef	pthread_rwlock_destroy
#endif

static pthread_rwlock_t	Prwld_rwl;

int
pthread_rwlock_destroy_csf()
{
	int	rval;

	rval = pthread_rwlock_init(&Prwld_rwl, NULL);

	return(rval);
}

int
pthread_rwlock_destroy_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_rwlock_destroy(&Prwld_rwl);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_rwlock_destroy_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_rwlock_init()			 ******/

#ifdef	pthread_rwlock_init
#undef	pthread_rwlock_init
#endif

static pthread_rwlock_t	Prwli_rwl;

int
pthread_rwlock_init_csf()
{
	return(0);
}

int
pthread_rwlock_init_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_rwlock_init(&Prwli_rwl, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_rwlock_init_cef()
{
	(void) pthread_rwlock_destroy(&Prwli_rwl);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_rwlock_rdlock()			 ******/

#ifdef	pthread_rwlock_rdlock
#undef	pthread_rwlock_rdlock
#endif

static pthread_rwlock_t	Prwlrl_rwl;

int
pthread_rwlock_rdlock_csf()
{
	int	rval;

	rval = pthread_rwlock_init(&Prwlrl_rwl, NULL);

	return(rval);
}

int
pthread_rwlock_rdlock_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_rwlock_rdlock(&Prwlrl_rwl);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_rwlock_rdlock_cef()
{
	(void) pthread_rwlock_unlock(&Prwlrl_rwl);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_rwlock_tryrdlock()			 ******/

#ifdef	pthread_rwlock_tryrdlock
#undef	pthread_rwlock_tryrdlock
#endif

static pthread_rwlock_t	Prwltrl_rwl;

int
pthread_rwlock_tryrdlock_csf()
{
	int	rval;

	rval = pthread_rwlock_init(&Prwltrl_rwl, NULL);

	return(rval);
}

int
pthread_rwlock_tryrdlock_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_rwlock_tryrdlock(&Prwltrl_rwl);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_rwlock_tryrdlock_cef()
{
	(void) pthread_rwlock_destroy(&Prwltrl_rwl);

	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_rwlock_trywrlock()			 ******/

#ifdef	pthread_rwlock_trywrlock
#undef	pthread_rwlock_trywrlock
#endif

static pthread_rwlock_t	Prwltwl_rwl;

int
pthread_rwlock_trywrlock_csf()
{
	int	rval;

	rval = pthread_rwlock_init(&Prwltwl_rwl, NULL);

	return(rval);
}

int
pthread_rwlock_trywrlock_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_rwlock_trywrlock(&Prwltwl_rwl);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_rwlock_trywrlock_cef()
{
	(void) pthread_rwlock_unlock(&Prwltwl_rwl);
	(void) pthread_rwlock_destroy(&Prwltwl_rwl);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_rwlock_unlock()			 ******/

#ifdef	pthread_rwlock_unlock
#undef	pthread_rwlock_unlock
#endif

static pthread_rwlock_t	Prwlu_rwl;

int
pthread_rwlock_unlock_csf()
{
	int	rval;

	rval = pthread_rwlock_init(&Prwlu_rwl, NULL);
	if ( rval == 0 )
	{
		rval = pthread_rwlock_rdlock(&Prwlu_rwl);
	}

	return(rval);
}

int
pthread_rwlock_unlock_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_rwlock_unlock(&Prwlu_rwl);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_rwlock_unlock_cef()
{
	(void) pthread_rwlock_destroy(&Prwlu_rwl);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_rwlock_wrlock()			 ******/

#ifdef	pthread_rwlock_wrlock
#undef	pthread_rwlock_wrlock
#endif

static pthread_rwlock_t	Prwlwl_rwl;

int
pthread_rwlock_wrlock_csf()
{
	int	rval;

	rval = pthread_rwlock_init(&Prwlwl_rwl, NULL);

	return(rval);
}

int
pthread_rwlock_wrlock_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_rwlock_wrlock(&Prwlwl_rwl);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_rwlock_wrlock_cef()
{
	(void) pthread_rwlock_unlock(&Prwlwl_rwl);
	(void) pthread_rwlock_destroy(&Prwlwl_rwl);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_rwlockattr_destroy()		 ******/

#ifdef	pthread_rwlockattr_destroy
#undef	pthread_rwlockattr_destroy
#endif

static pthread_rwlockattr_t	Prwlad_rwla;

int
pthread_rwlockattr_destroy_csf()
{
	int	rval;

	rval = pthread_rwlockattr_init(&Prwlad_rwla);

	return(rval);
}

int
pthread_rwlockattr_destroy_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_rwlockattr_destroy(&Prwlad_rwla);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_rwlockattr_destroy_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_rwlockattr_getpshared()		 ******/

#ifdef	pthread_rwlockattr_getpshared
#undef	pthread_rwlockattr_getpshared
#endif

static pthread_rwlockattr_t	Prwlagps_rwla;

int
pthread_rwlockattr_getpshared_csf()
{
	int	rval;

	rval = pthread_rwlockattr_init(&Prwlagps_rwla);

	return(rval);
}

int
pthread_rwlockattr_getpshared_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	pshared;

	errs	= 0;
	fails	= 0;

	(void) pthread_rwlockattr_getpshared(&Prwlagps_rwla, &pshared);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_rwlockattr_getpshared_cef()
{
	(void) pthread_rwlockattr_destroy(&Prwlagps_rwla);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_rwlockattr_init()			 ******/

#ifdef	pthread_rwlockattr_init
#undef	pthread_rwlockattr_init
#endif

static pthread_rwlockattr_t	Prwlai_rwla;

int
pthread_rwlockattr_init_csf()
{
	return(0);
}

int
pthread_rwlockattr_init_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_rwlockattr_init(&Prwlai_rwla);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_rwlockattr_init_cef()
{
	(void) pthread_rwlockattr_destroy(&Prwlai_rwla);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_rwlockattr_setpshared()		 ******/

#ifdef	pthread_rwlockattr_setpshared
#undef	pthread_rwlockattr_setpshared
#endif

static pthread_rwlockattr_t	Prwlasps_rwla;

int
pthread_rwlockattr_setpshared_csf()
{
	int	rval;

	rval = pthread_rwlockattr_init(&Prwlasps_rwla);

	return(rval);
}

int
pthread_rwlockattr_setpshared_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_rwlockattr_setpshared(&Prwlasps_rwla, PTHREAD_PROCESS_PRIVATE);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_rwlockattr_setpshared_cef()
{
	(void) pthread_rwlockattr_destroy(&Prwlasps_rwla);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_self()				 ******/

#ifdef	pthread_self
#undef	pthread_self
#endif

int
pthread_self_csf()
{
	return(0);
}

int
pthread_self_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	pthread_t	tid;

	errs	= 0;
	fails	= 0;

	tid = pthread_self();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_self_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_setcancelstate()			 ******/

#ifdef	pthread_setcancelstate
#undef	pthread_setcancelstate
#endif

int
pthread_setcancelstate_csf()
{
	return(0);
}

int
pthread_setcancelstate_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	oldstate;

	errs	= 0;
	fails	= 0;

	(void) pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &oldstate);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_setcancelstate_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_setcanceltype()			 ******/

#ifdef	pthread_setcanceltype
#undef	pthread_setcanceltype
#endif

int
pthread_setcanceltype_csf()
{
	return(0);
}

int
pthread_setcanceltype_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	oldtype;

	errs	= 0;
	fails	= 0;

	(void) pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &oldtype);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_setcanceltype_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pthread_setconcurrency()			 ******/

#ifdef	pthread_setconcurrency
#undef	pthread_setconcurrency
#endif

int
pthread_setconcurrency_csf()
{
	return(0);
}

int
pthread_setconcurrency_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_setconcurrency(0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_setconcurrency_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) && defined(_POSIX_THREAD_PRIORITY_SCHEDULING)

/****** Testset for pthread_setschedparam()			 ******/

#ifdef	pthread_setschedparam
#undef	pthread_setschedparam
#endif

static pthread_t		Pssp_tid;
static int			Pssp_policy;
static struct sched_param	Pssp_params;

int
pthread_setschedparam_csf()
{
	int	rval;

	Pssp_tid = pthread_self();
	rval = pthread_getschedparam(Pssp_tid, &Pssp_policy, &Pssp_params);

	return(rval);
}

int
pthread_setschedparam_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_setschedparam(Pssp_tid, Pssp_policy, &Pssp_params);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_setschedparam_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_setspecific()			 ******/

#ifdef	pthread_setspecific
#undef	pthread_setspecific
#endif

pthread_key_t	Pss_key;

int
pthread_setspecific_csf()
{
	int	rval;

	rval = pthread_key_create(&Pss_key, NULL);

	return(rval);
}

int
pthread_setspecific_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_setspecific(Pss_key, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_setspecific_cef()
{
	(void) pthread_key_delete(Pss_key);
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_sigmask()				 ******/

#ifdef	pthread_sigmask
#undef	pthread_sigmask
#endif

static sigset_t	Psm_set;

int
pthread_sigmask_csf()
{
	(void) sigemptyset(&Psm_set);
	return(0);
}

int
pthread_sigmask_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_sigmask(SIG_BLOCK, &Psm_set, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_sigmask_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS)

/****** Testset for pthread_testcancel()			 ******/

#ifdef	pthread_testcancel
#undef	pthread_testcancel
#endif

int
pthread_testcancel_csf()
{
	return(0);
}

int
pthread_testcancel_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) pthread_testcancel();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pthread_testcancel_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for ptsname()					 ******/

#ifdef	ptsname
#undef	ptsname
#endif

int
ptsname_csf()
{
	return(0);
}

int
ptsname_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) ptsname(-1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ptsname_cef()
{
	return(0);
}

#endif


/****** Testset for putc()					 ******/

#ifdef	putc
#undef	putc
#endif

static FILE	*Putc_fp;

int
putc_csf()
{
	int	rval;

	rval = 0;

	Putc_fp = fopen("./nonesuch", "w+");
	if (Putc_fp == NULL)
	{
		++rval;
	}

	return(rval);
}

int
putc_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) putc('X', Putc_fp);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
putc_cef()
{
	(void) fclose(Putc_fp);
	(void) remove("./nonesuch");
	return(0);
}



/****** Testset for putc_unlocked()				 ******/

#ifdef	putc_unlocked
#undef	putc_unlocked
#endif

static FILE	*Putc_unlocked_fp;

int
putc_unlocked_csf()
{
	int	rval;

	rval = 0;

	Putc_unlocked_fp = fopen("./nonesuch", "w+");
	if ( Putc_unlocked_fp == NULL )
	{
		++rval;
	}

	return(rval);
}

int
putc_unlocked_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) putc_unlocked('X', Putc_unlocked_fp);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
putc_unlocked_cef()
{
	(void) fclose(Putc_unlocked_fp);
	(void) remove("./nonesuch");
	return(0);
}



/****** Testset for putchar()					 ******/

#ifdef	putchar
#undef	putchar
#endif

int
putchar_csf()
{
	return(0);
}

int
putchar_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) putchar('\0');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
putchar_cef()
{
	return(0);
}



/****** Testset for putchar_unlocked()				 ******/

#ifdef	putchar_unlocked
#undef	putchar_unlocked
#endif

int
putchar_unlocked_csf()
{
	return(0);
}

int
putchar_unlocked_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) putchar_unlocked('\0');

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
putchar_unlocked_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for putenv()					 ******/

#ifdef	putenv
#undef	putenv
#endif

int
putenv_csf()
{
	return(0);
}

int
putenv_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) putenv("");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
putenv_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for putmsg()					 ******/

#ifdef	putmsg
#undef	putmsg
#endif

int
putmsg_csf()
{
	return(0);
}

int
putmsg_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	int		flags;
	char		buf[1];
	struct strbuf	ctl;
	struct strbuf	data;

	errs	= 0;
	fails	= 0;

	ctl.buf		= &buf[0];
	ctl.maxlen	= 0;

	data.buf	= &buf[0];
	data.maxlen	= 0;

	flags		= 0;

	(void) putmsg(-1, &ctl, &data, flags);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
putmsg_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for putpmsg()					 ******/

#ifdef	putpmsg
#undef	putpmsg
#endif

int
putpmsg_csf()
{
	return(0);
}

int
putpmsg_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	int		flags;
	int		band;
	char		buf[1];
	struct strbuf	ctl;
	struct strbuf	data;


	errs	= 0;
	fails	= 0;

	ctl.buf		= &buf[0];
	ctl.maxlen	= 0;

	data.buf	= &buf[0];
	data.maxlen	= 0;

	flags		= MSG_BAND;
	band		= 0;

	(void) putpmsg(-1, &ctl, &data, band, flags);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
putpmsg_cef()
{
	return(0);
}

#endif


/****** Testset for puts()					 ******/

#ifdef	puts
#undef	puts
#endif

int
puts_csf()
{
	return(0);
}

int
puts_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) puts("");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
puts_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for pututxline()				 ******/

#ifdef	pututxline
#undef	pututxline
#endif

static struct utmpx	*Pututxline_u;

int
pututxline_csf()
{
	Pututxline_u = getutxent();
	return(0);
}

int
pututxline_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct utmpx	u;

	errs	= 0;
	fails	= 0;

	(void) pututxline(Pututxline_u);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pututxline_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for putw()					 ******/

#ifdef	putw
#undef	putw
#endif

static FILE	*Putw_fp;

int
putw_csf()
{
	int	rval;

	rval = 0;

	Putw_fp = fopen("./nonesuch", "w+");
	if ( Putw_fp == NULL )
	{
		++rval;
	}

	return(rval);
}

int
putw_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) putw(0, Putw_fp);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
putw_cef()
{
	(void) fclose(Putw_fp);
	(void) remove("./nonesuch");
	return(0);
}

#endif
#endif

#if	( TEST_XPG > 4 )

/****** Testset for putwc()					 ******/

#ifdef	putwc
#undef	putwc
#endif

static FILE	*Putwc_fp;

int
putwc_csf()
{
	int	rval;

	rval = 0;

	Putwc_fp = fopen("./nonesuch", "w+");
	if ( Putwc_fp == NULL )
	{
		++rval;
	}

	return(rval);
}

int
putwc_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) putwc((wchar_t) 0, Putwc_fp);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
putwc_cef()
{
	(void) fclose(Putwc_fp);
	(void) remove("./nonesuch");
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for putwchar()					 ******/

#ifdef	putwchar
#undef	putwchar
#endif

int
putwchar_csf()
{
	return(0);
}

int
putwchar_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) putwchar((wchar_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
putwchar_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for pwrite()					 ******/

#ifdef	pwrite
#undef	pwrite
#endif

int
pwrite_csf()
{
	return(0);
}

int
pwrite_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) pwrite(-1, &buf[0], 0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
pwrite_cef()
{
	return(0);
}

#endif


