
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_cps.c	1.24
*
*
* CHANGE HISTORY
*	12 May 1999 - Incorporate SR#104, to require ifdefs around semaphore.h
*************************************************************************/



#include "std.h"

#include <dirent.h>
#include <fcntl.h>
#include <grp.h>
#include <limits.h>
#include <locale.h>
#include <math.h>

#if	( TEST_XPG > 4 )
#include <monetary.h>
#endif

#include <pwd.h>
#include <unistd.h>
#include <sched.h>
#if	defined(_POSIX_SEMAPHORES)
#include <semaphore.h>
#endif
#include <setjmp.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if	( TEST_XPG > 4 )
#include <strings.h>
#include <sys/ipc.h>
#endif

#include <sys/mman.h>

#if	( TEST_XPG > 4 )
#include <sys/resource.h>
#include <sys/sem.h>
#include <sys/shm.h>
#endif

#include <sys/stat.h>

#if	( TEST_XPG > 4 )
#include <sys/statvfs.h>
#include <sys/time.h>
#endif

#include <sys/types.h>

#if	( TEST_XPG > 4 )
#include <syslog.h>
#endif

#include <time.h>

#if	( TEST_XPG > 4 )
#include <ucontext.h>
#endif


#if	( TEST_XPG > 4 )
#include <utmpx.h>
#include <wchar.h>
#endif


#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"


#if	( TEST_XPG > 4 )

/****** Testset for sbrk()					 ******/

#ifdef	sbrk
#undef	sbrk
#endif

int
sbrk_csf()
{
	return(0);
}

int
sbrk_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* legacy, not tested */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sbrk_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for scalb()					 ******/

#ifdef	scalb
#undef	scalb
#endif

int
scalb_csf()
{
	return(0);
}

int
scalb_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) scalb(1.0, 1.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
scalb_cef()
{
	return(0);
}

#endif


/****** Testset for scanf()					 ******/

#ifdef	scanf
#undef	scanf
#endif

int
scanf_csf()
{
	return(0);
}

int
scanf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) scanf("");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
scanf_cef()
{
	return(0);
}


#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_get_priority_max()			 ******/

#ifdef	sched_get_priority_max
#undef	sched_get_priority_max
#endif

int
sched_get_priority_max_csf()
{
	return(0);
}

int
sched_get_priority_max_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sched_get_priority_max(SCHED_RR);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_get_priority_max_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_get_priority_min()			 ******/

#ifdef	sched_get_priority_min
#undef	sched_get_priority_min
#endif

int
sched_get_priority_min_csf()
{
	return(0);
}

int
sched_get_priority_min_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sched_get_priority_min(SCHED_RR);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_get_priority_min_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_getparam()				 ******/

#ifdef	sched_getparam
#undef	sched_getparam
#endif

static pid_t	Sched_getparam_pid;

int
sched_getparam_csf()
{
	Sched_getparam_pid = getpid();
	return(0);
}

int
sched_getparam_ctf()
{
	int			rval;
	int			errs;
	int			fails;
	struct sched_param	params;

	errs	= 0;
	fails	= 0;

	(void) sched_getparam(Sched_getparam_pid, &params);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_getparam_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_getscheduler()			 ******/

#ifdef	sched_getscheduler
#undef	sched_getscheduler
#endif

static pid_t	Sched_getscheduler_pid;

int
sched_getscheduler_csf()
{
	Sched_getscheduler_pid = getpid();
	return(0);
}

int
sched_getscheduler_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sched_getscheduler(Sched_getscheduler_pid);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_getscheduler_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_rr_get_interval()			 ******/

#ifdef	sched_rr_get_interval
#undef	sched_rr_get_interval
#endif

static pid_t	Sched_rr_get_interval_pid;

int
sched_rr_get_interval_csf()
{
	Sched_rr_get_interval_pid = getpid();
	return(0);
}

int
sched_rr_get_interval_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct timespec	interval;

	errs	= 0;
	fails	= 0;

	(void) sched_rr_get_interval( Sched_rr_get_interval_pid, &interval);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_rr_get_interval_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_setparam()				 ******/

#ifdef	sched_setparam
#undef	sched_setparam
#endif

static pid_t	Sched_setparam_pid;

int
sched_setparam_csf()
{
	Sched_setparam_pid = getpid();
	return(0);
}

int
sched_setparam_ctf()
{
	int			rval;
	int			errs;
	int			fails;
	struct sched_param	params;

	errs	= 0;
	fails	= 0;

	params.sched_priority = 0;
	(void) sched_setparam(Sched_setparam_pid, &params);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_setparam_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_setscheduler()			 ******/

#ifdef	sched_setscheduler
#undef	sched_setscheduler
#endif

static pid_t	Sched_setscheduler_pid;

int
sched_setscheduler_csf()
{
	Sched_setscheduler_pid = getpid();
	return(0);
}

int
sched_setscheduler_ctf()
{
	int			rval;
	int			errs;
	int			fails;
	struct sched_param	params;

	errs	= 0;
	fails	= 0;

	params.sched_priority = 0;
	(void) sched_setscheduler(Sched_setscheduler_pid, SCHED_RR, &params);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_setscheduler_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) || defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_yield()				 ******/

#ifdef	sched_yield
#undef	sched_yield
#endif

int
sched_yield_csf()
{
	return(0);
}

int
sched_yield_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sched_yield();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_yield_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for seed48()					 ******/

#ifdef	seed48
#undef	seed48
#endif

int
seed48_csf()
{
	return(0);
}

int
seed48_ctf()
{
	int			rval;
	int			errs;
	int			fails;
	unsigned short int	x[3];

	errs	= 0;
	fails	= 0;

	(void) seed48(&x[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
seed48_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for seekdir()					 ******/

#ifdef	seekdir
#undef	seekdir
#endif

static DIR	*Seekdir_dirp;
long		Seekdir_pos;

int
seekdir_csf()
{
	int	rval;

	rval = 0;

	Seekdir_dirp = opendir(".");
	if (Seekdir_dirp != (DIR *) NULL)
	{
		Seekdir_pos = telldir(Seekdir_dirp);
	}
	else
	{
		++rval;
	}

	return(rval);
}

int
seekdir_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) seekdir(Seekdir_dirp, Seekdir_pos);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
seekdir_cef()
{
	(void) closedir(Seekdir_dirp);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for select()					 ******/

#ifdef	select
#undef	select
#endif

static fd_set	Vsth_select_fdset;

int
select_csf()
{
	FD_ZERO(&Vsth_select_fdset);
	return(0);
}

int
select_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct timeval	to;

	errs	= 0;
	fails	= 0;

	to.tv_sec	= 0;
	to.tv_usec	= 0;

	(void) select(-1, &Vsth_select_fdset, NULL, NULL, &to);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
select_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_close()					 ******/

#ifdef	sem_close
#undef	sem_close
#endif

static sem_t	*Sem_close_sem;
static	char	Vsth_sem_close_buf[L_tmpnam + 1];

int
sem_close_csf()
{
	int	rval;
	char	*p;

	rval = 0;

	p = tmpnam(Vsth_sem_close_buf);
	if ( p == Vsth_sem_close_buf )
	{
		Sem_close_sem = sem_open(Vsth_sem_close_buf, O_CREAT, S_IRWXU, 0);
	}
	else
	{
		++rval;
	}

	return(rval);
}

int
sem_close_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sem_close(Sem_close_sem);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_close_cef()
{
	(void) sem_unlink(Vsth_sem_close_buf);
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_destroy()				 ******/

#ifdef	sem_destroy
#undef	sem_destroy
#endif

sem_t	Sem_destroy_sem;

int
sem_destroy_csf()
{
	int	rval;

	rval = sem_init(&Sem_destroy_sem, 0, 0);

	return(rval);
}

int
sem_destroy_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sem_destroy(&Sem_destroy_sem);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_destroy_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_getvalue()				 ******/

#ifdef	sem_getvalue
#undef	sem_getvalue
#endif

static sem_t	Sem_getvalue_sem;

int
sem_getvalue_csf()
{
	int	rval;

	rval = sem_init(&Sem_getvalue_sem, 0, 0);

	return(rval);
}

int
sem_getvalue_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	val;

	errs	= 0;
	fails	= 0;

	(void) sem_getvalue(&Sem_getvalue_sem, &val);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_getvalue_cef()
{
	(void) sem_destroy(&Sem_getvalue_sem);
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_init()					 ******/

#ifdef	sem_init
#undef	sem_init
#endif

static sem_t	Sem_init_sem;

int
sem_init_csf()
{
	return(0);
}

int
sem_init_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sem_init(&Sem_init_sem, 0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_init_cef()
{
	(void) sem_destroy(&Sem_init_sem);
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_open()					 ******/

#ifdef	sem_open
#undef	sem_open
#endif

static sem_t	*Sem_open_sem;
static	char	Vsth_sem_open_buf[L_tmpnam + 1];

int
sem_open_csf()
{
	int	rval;
	char	*p;

	rval = 0;

	p = tmpnam(Vsth_sem_open_buf);
	if ( p != Vsth_sem_open_buf )
	{
		++rval;
	}
	return(rval);
}

int
sem_open_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	Sem_open_sem = sem_open(Vsth_sem_open_buf, O_CREAT, S_IRWXU, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_open_cef()
{
	(void) sem_close(Sem_open_sem);
	(void) sem_unlink(Vsth_sem_open_buf);
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_post()					 ******/

#ifdef	sem_post
#undef	sem_post
#endif

int
sem_post_csf()
{
	return(0);
}

int
sem_post_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_post_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_trywait()				 ******/

#ifdef	sem_trywait
#undef	sem_trywait
#endif

static sem_t	Sem_trywait_sem;

int
sem_trywait_csf()
{
	int	rval;

	rval = sem_init(&Sem_trywait_sem, 0, 0);

	return(rval);
}

int
sem_trywait_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sem_trywait(&Sem_trywait_sem);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_trywait_cef()
{
	(void) sem_post(&Sem_trywait_sem);
	(void) sem_destroy(&Sem_trywait_sem);
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_unlink()				 ******/

#ifdef	sem_unlink
#undef	sem_unlink
#endif

static sem_t	*Sem_unlink_sem;
static	char	Vsth_sem_unlink_buf[L_tmpnam + 1];

int
sem_unlink_csf()
{
	int	rval;
	char	*p,*q,*r;

	rval = 0;

	p = tmpnam(Vsth_sem_unlink_buf);

/* 
	TSD5TH.00059: we need to strip any slash characters from the
	pathname other than the leading slash, as sem_open() only
	is guaranteed to operate on a filename containing a single
	leading slash 
*/
	q = strchr(p, (int) '/');
    if (q != NULL) {
		/* ignore the leading slash */
    	r = strchr (p+1,(int)  '/');
        if (r != NULL)
        	*r='z';
        q = strchr (r+1,(int)  '/');
        if (q != NULL)
        	*q='z';   
	}

	if ( p == Vsth_sem_unlink_buf )
	{
		Sem_unlink_sem = sem_open(Vsth_sem_unlink_buf, O_CREAT, S_IRWXU, 0);
		if (Sem_unlink_sem != SEM_FAILED )
		{
			rval = sem_close(Sem_unlink_sem);
		}
		else
		{
			++rval;
		}
	}
	else
	{
		++rval;
	}

	return(rval);
}

int
sem_unlink_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sem_unlink(Vsth_sem_unlink_buf);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_unlink_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_wait()					 ******/

#ifdef	sem_wait
#undef	sem_wait
#endif

static sem_t	Vsth_sem_wait_sem;

int
sem_wait_csf()
{
	int	rval;

	rval = sem_init(&Vsth_sem_wait_sem, 0, 0);
	return(rval);
}

int
sem_wait_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sem_wait(&Vsth_sem_wait_sem);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_wait_cef()
{
	(void) sem_destroy(&Vsth_sem_wait_sem);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for semctl()					 ******/

#ifdef	semctl
#undef	semctl
#endif

int
semctl_csf()
{
	return(0);
}

int
semctl_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) semctl(-1, 0, GETPID);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
semctl_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for semget()					 ******/

#ifdef	semget
#undef	semget
#endif

static int	Semget_semid;

int
semget_csf()
{
	return(0);
}

int
semget_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	Semget_semid = semget(IPC_PRIVATE, 1, IPC_CREAT);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
semget_cef()
{
	(void) semctl(Semget_semid, 0, IPC_RMID);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for semop()					 ******/

#ifdef	semop
#undef	semop
#endif

int
semop_csf()
{
	return(0);
}

int
semop_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct sembuf	ops[1];

	errs	= 0;
	fails	= 0;

	(void) semop(-1, &ops[0], 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
semop_cef()
{
	return(0);
}

#endif


/****** Testset for setbuf()					 ******/

#ifdef	setbuf
#undef	setbuf
#endif

int
setbuf_csf()
{
	return(0);
}

int
setbuf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setbuf(stderr, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setbuf_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for setcontext()				 ******/

#ifdef	setcontext
#undef	setcontext
#endif

int
setcontext_csf()
{
	return(0);
}

int
setcontext_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not currently tested due to difficulty level */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setcontext_cef()
{
	return(0);
}

#endif


/****** Testset for setgid()					 ******/

#ifdef	setgid
#undef	setgid
#endif

static gid_t	Setgid_gid;

int
setgid_csf()
{
	Setgid_gid = getgid();
	return(0);
}

int
setgid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setgid(Setgid_gid);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setgid_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for setgrent()					 ******/

#ifdef	setgrent
#undef	setgrent
#endif

int
setgrent_csf()
{
	return(0);
}

int
setgrent_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setgrent();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setgrent_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setitimer()					 ******/

#ifdef	setitimer
#undef	setitimer
#endif

static int	Setitimer_timer;

int
setitimer_csf()
{
	return(0);
}

int
setitimer_ctf()
{
	int			rval;
	int			errs;
	int			fails;
	struct itimerval	value;
	struct itimerval	oldval;

	errs	= 0;
	fails	= 0;

	value.it_interval.tv_sec	= 0;
	value.it_interval.tv_usec	= 0;
	value.it_value.tv_sec		= (time_t) INT_MAX;
	value.it_value.tv_usec		= 0;

	(void) setitimer(ITIMER_VIRTUAL, &value, &oldval);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setitimer_cef()
{
	return(0);
}

#endif


/****** Testset for setjmp()					 ******/

int
setjmp_csf()
{
	return(0);
}

int
setjmp_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	jmp_buf	buf;

	errs	= 0;
	fails	= 0;

	(void) setjmp(buf);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setjmp_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_CRYPT) && (_XOPEN_CRYPT != -1)

/****** Testset for setkey()					 ******/

#ifdef	setkey
#undef	setkey
#endif

static unsigned char	Setkey_key[64];
int
setkey_csf()
{
	(void) memset(&Setkey_key[0], 0, 64);
	return(0);
}

int
setkey_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	setkey((const char *) &Setkey_key[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setkey_cef()
{
	return(0);
}

#endif
#endif


/****** Testset for setlocale()					 ******/

#ifdef	setlocale
#undef	setlocale
#endif

int
setlocale_csf()
{
	return(0);
}

int
setlocale_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setlocale(LC_ALL, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setlocale_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for setlogmask()				 ******/

#ifdef	setlogmask
#undef	setlogmask
#endif

int
setlogmask_csf()
{
	return(0);
}

int
setlogmask_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setlogmask(0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setlogmask_cef()
{
	return(0);
}

#endif

#if	defined(_POSIX_JOB_CONTROL)

/****** Testset for setpgid()					 ******/

#ifdef	setpgid
#undef	setpgid
#endif

int
setpgid_csf()
{
	return(0);
}

int
setpgid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setpgid(0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setpgid_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setpgrp()					 ******/

#ifdef	setpgrp
#undef	setpgrp
#endif

int
setpgrp_csf()
{
	return(0);
}

int
setpgrp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setpgrp();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setpgrp_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setpriority()				 ******/

#ifdef	setpriority
#undef	setpriority
#endif

int
setpriority_csf()
{
	return(0);
}

int
setpriority_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setpriority(PRIO_PROCESS, 0, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setpriority_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setpwent()					 ******/

#ifdef	setpwent
#undef	setpwent
#endif

int
setpwent_csf()
{
	return(0);
}

int
setpwent_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setpwent();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setpwent_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setregid()					 ******/

#ifdef	setregid
#undef	setregid
#endif

int
setregid_csf()
{
	return(0);
}

int
setregid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setregid((gid_t) -1, (gid_t) -1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setregid_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setreeid()					 ******/

#ifdef	setreuid
#undef	setreuid
#endif

int
setreuid_csf()
{
	return(0);
}

int
setreuid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setreuid((uid_t) -1, (uid_t) -1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setreuid_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setrlimit()					 ******/

#ifdef	setrlimit
#undef	setrlimit
#endif

int
setrlimit_csf()
{
	return(0);
}

int
setrlimit_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct rlimit	limit;

	errs	= 0;
	fails	= 0;

	limit.rlim_cur = RLIM_INFINITY;
	limit.rlim_max = RLIM_INFINITY;

	(void) setrlimit(RLIMIT_CPU, &limit);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setrlimit_cef()
{
	return(0);
}

#endif


/****** Testset for setsid()					 ******/

#ifdef	setsid
#undef	setsid
#endif

int
setsid_csf()
{
	return(0);
}

int
setsid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setsid();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setsid_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for setstate()					 ******/

#ifdef	setstate
#undef	setstate
#endif

static char	Setstate_state[8];

int
setstate_csf()
{
	(void) initstate((unsigned int) 0, &Setstate_state[0], 8);

	return(0);
}

int
setstate_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setstate(&Setstate_state[8]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setstate_cef()
{
	return(0);
}

#endif


/****** Testset for setuid()					 ******/

#ifdef	setuid
#undef	setuid
#endif

static uid_t	Setuid_uid;

int
setuid_csf()
{
	Setuid_uid = getuid();

	return(0);
}

int
setuid_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

/*	(void)  chown ( ".", getuid(), getgid()); */

	(void) setuid(Setuid_uid);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setuid_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for setutxent()					 ******/

#ifdef	setutxent
#undef	setutxent
#endif

int
setutxent_csf()
{
	return(0);
}

int
setutxent_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setutxent();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setutxent_cef()
{
	return(0);
}

#endif


/****** Testset for setvbuf()					 ******/

#ifdef	setvbuf
#undef	setvbuf
#endif

int
setvbuf_csf()
{
	return(0);
}

int
setvbuf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) setvbuf(stderr, NULL, _IOFBF, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setvbuf_cef()
{
	return(0);
}


#if	defined(_POSIX_SHARED_MEMORY_OBJECTS)

/****** Testset for shm_open()					 ******/

#ifdef	shm_open
#undef	shm_open
#endif

static int	Shm_open_id;
static	char	Vsth_shm_open_buf[L_tmpnam + 1];

int
shm_open_csf()
{
	int	rval;
	char	*p;

	rval = 0;

	p = tmpnam(Vsth_shm_open_buf);
	if ( p != Vsth_shm_open_buf )
	{
		++rval;
	}

	return(rval);
}

int
shm_open_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	Shm_open_id = shm_open(Vsth_shm_open_buf, O_RDWR | O_CREAT, S_IRWXU);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
shm_open_cef()
{
	(void) shm_unlink(Vsth_shm_open_buf);
	return(0);
}

#endif

#if	defined(_POSIX_SHARED_MEMORY_OBJECTS)

/****** Testset for shm_unlink()				 ******/

#ifdef	shm_unlink
#undef	shm_unlink
#endif

static int	Shm_unlink_id;
static	char	Vsth_shm_unlink_buf[L_tmpnam + 1];

int
shm_unlink_csf()
{

	int	rval;
	char	*p;

	rval = 0;

	p = tmpnam(Vsth_shm_unlink_buf);
	if ( p == Vsth_shm_unlink_buf )
	{
		Shm_unlink_id = shm_open(Vsth_shm_unlink_buf, O_RDWR | O_CREAT, S_IRWXU);
	}
	else
	{
		++rval;
	}

	return(rval);
}

int
shm_unlink_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) shm_unlink(Vsth_shm_unlink_buf);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
shm_unlink_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for shmat()					 ******/

#ifdef	shmat
#undef	shmat
#endif

static int	Shmat_id;
static void	*Shmat_addr;

int
shmat_csf()
{
	int	rval;

	rval = 0;

	Shmat_id = shmget((key_t) IPC_PRIVATE, sizeof(int), 0);
	if ( Shmat_id == -1 )
	{
		++rval;
	}

	return(rval);
}

int
shmat_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	Shmat_addr = shmat(Shmat_id, NULL, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
shmat_cef()
{
	if (Shmat_addr != (void *) -1)
	{
		(void) shmdt(Shmat_addr);
	}

	if ( Shmat_id != -1 )
	{
		(void) shmctl(Shmat_id, IPC_RMID, NULL);
	}

	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for shmctl()					 ******/

#ifdef	shmctl
#undef	shmctl
#endif

static int	Shmctl_id;

int
shmctl_csf()
{
	int	rval;

	rval = 0;

	Shmctl_id = shmget((key_t)IPC_PRIVATE, sizeof(int), 0);
	if ( Shmctl_id == -1 )
	{
		++rval;
	}
	return(rval);
}

int
shmctl_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) shmctl(Shmctl_id, IPC_RMID, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
shmctl_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for shmdt()					 ******/

#ifdef	shmdt
#undef	shmdt
#endif

static int	Shmdt_id;
static void	*Shmdt_addr;


int
shmdt_csf()
{
	int	rval;
/* 
   	Changed 21 Oct 1998.  
   	Added shmflag=0644, so that shmat has permission 
   	to attach to the shared memory segment. Previously
   	it was mode 0
*/
	int	shmflag=0644;  

	rval = 0;

	Shmdt_id = shmget((key_t) IPC_PRIVATE, sizeof(int), shmflag);
	if (Shmdt_id != -1 )
	{
		Shmdt_addr = shmat(Shmdt_id, NULL, 0);
		if ( Shmdt_addr == (void *) -1 )
		{
			++rval;
		}
	}
	else
	{
		++rval;
	}

	return(rval);
}

int
shmdt_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) shmdt(Shmdt_addr);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
shmdt_cef()
{
/*
   	Changed 21 Oct 1998.  
   	Add shmctl() line to remove the shared memory segment.
*/
	(void) shmctl(Shmdt_id, IPC_RMID, 0);  
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for shmget()					 ******/

#ifdef	shmget
#undef	shmget
#endif

static int	Shmget_id;

int
shmget_csf()
{
	return(0);
}

int
shmget_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	Shmget_id = shmget(IPC_PRIVATE, sizeof(int), 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
shmget_cef()
{
	(void) shmctl(Shmget_id, IPC_RMID, NULL);
	return(0);
}

#endif


/****** Testset for sigaction()					 ******/

#ifdef	sigaction
#undef	sigaction
#endif

int
sigaction_csf()
{
	return(0);
}

int
sigaction_ctf()
{
	int			rval;
	int			errs;
	int			fails;
	struct sigaction	act;

	errs	= 0;
	fails	= 0;

	(void) sigaction(SIGINT, NULL, &act);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigaction_cef()
{
	return(0);
}



/****** Testset for sigaddset()					 ******/

#ifdef	sigaddset
#undef	sigaddset
#endif

static sigset_t	Sigaddset_set;

int
sigaddset_csf()
{
	(void) sigemptyset(&Sigaddset_set);
	return(0);
}

int
sigaddset_ctf()
{
	int		rval;
	int		errs;
	int		fails;

	errs	= 0;
	fails	= 0;

	(void) sigaddset(&Sigaddset_set, SIGPIPE);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigaddset_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for sigaltstack()				 ******/

#ifdef	sigaltstack
#undef	sigaltstack
#endif

int
sigaltstack_csf()
{
	return(0);
}

int
sigaltstack_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	stack_t	stack;

	errs	= 0;
	fails	= 0;

	(void) sigaltstack(NULL, &stack);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigaltstack_cef()
{
	return(0);
}

#endif


/****** Testset for sigdelset()					 ******/

#ifdef	sigdelset
#undef	sigdelset
#endif

static sigset_t	Sigdelset_set;

int
sigdelset_csf()
{
	(void) sigfillset(&Sigdelset_set);
	return(0);
}

int
sigdelset_ctf()
{
	int		rval;
	int		errs;
	int		fails;

	errs	= 0;
	fails	= 0;

	(void) sigdelset(&Sigdelset_set, SIGPIPE);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigdelset_cef()
{
	return(0);
}



/****** Testset for sigemptyset()				 ******/

#ifdef	sigemptyset
#undef	sigemptyset
#endif

int
sigemptyset_csf()
{
	return(0);
}

int
sigemptyset_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	sigset_t	set;

	errs	= 0;
	fails	= 0;

	(void) sigemptyset(&set);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigemptyset_cef()
{
	return(0);
}



/****** Testset for sigfillset()				 ******/

#ifdef	sigfillset
#undef	sigfillset
#endif

int
sigfillset_csf()
{
	return(0);
}

int
sigfillset_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	sigset_t	set;

	errs	= 0;
	fails	= 0;

	(void) sigfillset(&set);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigfillset_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for sighold()					 ******/

#ifdef	sighold
#undef	sighold
#endif

int
sighold_csf()
{
	return(0);
}

int
sighold_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sighold(-1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sighold_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for sigignore()					 ******/

#ifdef	sigignore
#undef	sigignore
#endif

int
sigignore_csf()
{
	return(0);
}

int
sigignore_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sigignore(-1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigignore_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for siginterrupt()				 ******/

#ifdef	siginterrupt
#undef	siginterrupt
#endif

int
siginterrupt_csf()
{
	return(0);
}

int
siginterrupt_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) siginterrupt(-1, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
siginterrupt_cef()
{
	return(0);
}

#endif


/****** Testset for sigismember()				 ******/

#ifdef	sigismember
#undef	sigismember
#endif

static sigset_t	Sigismember_set;

int
sigismember_csf()
{
	(void) sigemptyset(&Sigismember_set);
	return(0);
}

int
sigismember_ctf()
{
	int		rval;
	int		errs;
	int		fails;

	errs	= 0;
	fails	= 0;

	(void) sigismember(&Sigismember_set, -1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigismember_cef()
{
	return(0);
}



/****** Testset for siglongjmp()				 ******/

#ifdef	siglongjmp
#undef	siglongjmp
#endif

int
siglongjmp_csf()
{
	return(0);
}

int
siglongjmp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
siglongjmp_cef()
{
	return(0);
}



/****** Testset for signal()					 ******/

#ifdef	signal
#undef	signal
#endif

int
signal_csf()
{
	return(0);
}

int
signal_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) signal(-1, SIG_IGN);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
signal_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for sigpause()					 ******/

#ifdef	sigpause
#undef	sigpause
#endif

int
sigpause_csf()
{
	return(0);
}

int
sigpause_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sigpause( -1 );

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigpause_cef()
{
	return(0);
}

#endif


/****** Testset for sigpending()				 ******/

#ifdef	sigpending
#undef	sigpending
#endif

int
sigpending_csf()
{
	return(0);
}

int
sigpending_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	sigset_t	set;

	errs	= 0;
	fails	= 0;

	(void) sigpending(&set);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigpending_cef()
{
	return(0);
}



/****** Testset for sigprocmask()				 ******/

#ifdef	sigprocmask
#undef	sigprocmask
#endif

int
sigprocmask_csf()
{
	return(0);
}

int
sigprocmask_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	sigset_t	set;

	errs	= 0;
	fails	= 0;

	(void) sigprocmask(SIG_SETMASK, NULL, &set);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigprocmask_cef()
{
	return(0);
}


#if	defined(_POSIX_REALTIME_SIGNALS)

/****** Testset for sigqueue()					 ******/

#ifdef	sigqueue
#undef	sigqueue
#endif

static pid_t	Sigqueue_pid;

int
sigqueue_csf()
{
	Sigqueue_pid = getpid();
	return(0);
}

int
sigqueue_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	union sigval	val;

	errs	= 0;
	fails	= 0;

	val.sival_int = 0;
	(void) sigqueue(Sigqueue_pid, 0, val);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigqueue_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for sigrelse()					 ******/

#ifdef	sigrelse
#undef	sigrelse
#endif

int
sigrelse_csf()
{
	return(0);
}

int
sigrelse_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sigrelse(-1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigrelse_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for sigset()					 ******/

#ifdef	sigset
#undef	sigset
#endif

int
sigset_csf()
{
	return(0);
}

int
sigset_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sigset(-1, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigset_cef()
{
	return(0);
}

#endif


/****** Testset for sigsetjmp()					 ******/

int
sigsetjmp_csf()
{
	return(0);
}

int
sigsetjmp_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	sigjmp_buf	buf;

	errs	= 0;
	fails	= 0;

	(void) sigsetjmp(buf, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigsetjmp_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for sigstack()					 ******/

#ifdef	sigstack
#undef	sigstack
#endif

int
sigstack_csf()
{
	return(0);
}

int
sigstack_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct sigstack	ostack;

	errs	= 0;
	fails	= 0;

	(void) sigstack(NULL, &ostack);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigstack_cef()
{
	return(0);
}

#endif
#endif


/****** Testset for sigsuspend()				 ******/

#ifdef	sigsuspend
#undef	sigsuspend
#endif

static sigset_t		Vsth_sigsuspend_set;

int
sigsuspend_csf()
{
	int	rval;

	rval = sigemptyset(&Vsth_sigsuspend_set);
	return(rval);
}

int
sigsuspend_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sigsuspend(&Vsth_sigsuspend_set);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigsuspend_cef()
{
	return(0);
}


#ifdef	_POSIX_REALTIME_SIGNALS

/****** Testset for sigtimedwait()				 ******/

#ifdef	sigtimedwait
#undef	sigtimedwait
#endif

static sigset_t		Vsth_sigtimedwait_set;

int
sigtimedwait_csf()
{
	int	rval;

	rval = sigemptyset(&Vsth_sigtimedwait_set);
	return(rval);
}

int
sigtimedwait_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct timespec	to;

	errs	= 0;
	fails	= 0;

	to.tv_sec	= 0;
	to.tv_nsec	= 0;

	(void) sigtimedwait(&Vsth_sigtimedwait_set, NULL, &to);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigtimedwait_cef()
{
	return(0);
}

#endif	/* _POSIX_REALTIME_SIGNALS */


/****** Testset for sigwait()					 ******/

#ifdef	sigwait
#undef	sigwait
#endif

static sigset_t	Vsth_sigwait_set;

int
sigwait_csf()
{
	int	rval;

	rval = sigemptyset(&Vsth_sigwait_set);
	return(rval);
}

int
sigwait_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	sig;

	errs	= 0;
	fails	= 0;

	(void) sigwait( &Vsth_sigwait_set, &sig );

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigwait_cef()
{
	return(0);
}



#ifdef	_POSIX_REALTIME_SIGNALS

/****** Testset for sigwaitinfo()				 ******/

#ifdef	sigwaitinfo
#undef	sigwaitinfo
#endif

static sigset_t	Vsth_sigwaitinfo_set;

int
sigwaitinfo_csf()
{
	int	rval;

	rval = sigemptyset(&Vsth_sigwaitinfo_set);
	return(rval);
}

int
sigwaitinfo_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sigwaitinfo( &Vsth_sigwaitinfo_set, NULL );

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigwaitinfo_cef()
{
	return(0);
}

#endif	/* _POSIX_REALTIME_SIGNALS */

/****** Testset for sin()					 ******/

#ifdef	sin
#undef	sin
#endif

int
sin_csf()
{
	return(0);
}

int
sin_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sin(0.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sin_cef()
{
	return(0);
}



/****** Testset for sinh()					 ******/

#ifdef	sinh
#undef	sinh
#endif

int
sinh_csf()
{
	return(0);
}

int
sinh_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sinh(0.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sinh_cef()
{
	return(0);
}



/****** Testset for sleep()					 ******/

#ifdef	sleep
#undef	sleep
#endif

int
sleep_csf()
{
	return(0);
}

int
sleep_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sleep(1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sleep_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for snprintf()					 ******/

#ifdef	snprintf
#undef	snprintf
#endif

int
snprintf_csf()
{
	return(0);
}

int
snprintf_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) snprintf(&buf[0], 0, "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
snprintf_cef()
{
	return(0);
}

#endif


/****** Testset for sprintf()					 ******/

#ifdef	sprintf
#undef	sprintf
#endif

int
sprintf_csf()
{
	return(0);
}

int
sprintf_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	sprintf(&buf[0], "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sprintf_cef()
{
	return(0);
}



/****** Testset for sqrt()					 ******/

#ifdef	sqrt
#undef	sqrt
#endif

int
sqrt_csf()
{
	return(0);
}

int
sqrt_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sqrt(0.0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sqrt_cef()
{
	return(0);
}



/****** Testset for srand()					 ******/

#ifdef	srand
#undef	srand
#endif

int
srand_csf()
{
	return(0);
}

int
srand_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) srand(0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
srand_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for srand48()					 ******/

#ifdef	srand48
#undef	srand48
#endif

int
srand48_csf()
{
	return(0);
}

int
srand48_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) srand48(0L);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
srand48_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for srandom()					 ******/

#ifdef	srandom
#undef	srandom
#endif

int
srandom_csf()
{
	return(0);
}

int
srandom_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) srandom(0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
srandom_cef()
{
	return(0);
}

#endif


/****** Testset for sscanf()					 ******/

#ifdef	sscanf
#undef	sscanf
#endif

int
sscanf_csf()
{
	return(0);
}

int
sscanf_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) sscanf(&buf[0], "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sscanf_cef()
{
	return(0);
}



/****** Testset for stat()					 ******/

#ifdef	stat
#undef	stat
#endif

int
stat_csf()
{
	return(0);
}

int
stat_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct stat	sb;

	errs	= 0;
	fails	= 0;

	(void) stat("./nonesuch", &sb);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
stat_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for statvfs()					 ******/

#ifdef	statvfs
#undef	statvfs
#endif

int
statvfs_csf()
{
	return(0);
}

int
statvfs_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct statvfs	sb;

	errs	= 0;
	fails	= 0;

	(void) statvfs("./nonesuch", &sb);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
statvfs_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for step()					 ******/

#ifdef	step
#undef	step
#endif

int
step_csf()
{
	return(0);
}

int
step_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* legacy, not tested */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
step_cef()
{
	return(0);
}

#endif
#endif

#if	( TEST_XPG > 4 )

/****** Testset for strcasecmp()				 ******/

#ifdef	strcasecmp
#undef	strcasecmp
#endif

int
strcasecmp_csf()
{
	return(0);
}

int
strcasecmp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strcasecmp("", "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strcasecmp_cef()
{
	return(0);
}

#endif


/****** Testset for strcat()					 ******/

#ifdef	strcat
#undef	strcat
#endif

int
strcat_csf()
{
	return(0);
}

int
strcat_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	buf[0] = '\0';
	strcat(&buf[0], "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strcat_cef()
{
	return(0);
}



/****** Testset for strchr()					 ******/

#ifdef	strchr
#undef	strchr
#endif

int
strchr_csf()
{
	return(0);
}

int
strchr_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strchr("", 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strchr_cef()
{
	return(0);
}



/****** Testset for strcmp()					 ******/

#ifdef	strcmp
#undef	strcmp
#endif

int
strcmp_csf()
{
	return(0);
}

int
strcmp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strcmp("", "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strcmp_cef()
{
	return(0);
}



/****** Testset for strcoll()					 ******/

#ifdef	strcoll
#undef	strcoll
#endif

int
strcoll_csf()
{
	return(0);
}

int
strcoll_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strcoll("", "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strcoll_cef()
{
	return(0);
}



/****** Testset for strcpy()					 ******/

#ifdef	strcpy
#undef	strcpy
#endif

int
strcpy_csf()
{
	return(0);
}

int
strcpy_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	buf[0] = '\0';
	strcpy(&buf[0], "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strcpy_cef()
{
	return(0);
}



/****** Testset for strcspn()					 ******/

#ifdef	strcspn
#undef	strcspn
#endif

int
strcspn_csf()
{
	return(0);
}

int
strcspn_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strcspn("", "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strcspn_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for strdup()					 ******/

#ifdef	strdup
#undef	strdup
#endif

static char	*Strdup_p;

int
strdup_csf()
{
	return(0);
}

int
strdup_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	Strdup_p = strdup("");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strdup_cef()
{
	if ( Strdup_p != NULL )
	{
		(void) free(Strdup_p);
	}
	return(0);
}

#endif


/****** Testset for strerror()					 ******/

#ifdef	strerror
#undef	strerror
#endif

int
strerror_csf()
{
	return(0);
}

int
strerror_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strerror(0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strerror_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for strfmon()					 ******/

#ifdef	strfmon
#undef	strfmon
#endif

int
strfmon_csf()
{
	return(0);
}

int
strfmon_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) strfmon(&buf[0], 0, "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strfmon_cef()
{
	return(0);
}

#endif


/****** Testset for strftime()					 ******/

#ifdef	strftime
#undef	strftime
#endif

static struct tm	*Strftime_ptm;

int
strftime_csf()
{
	time_t	t;

	t = time(NULL);
	Strftime_ptm = localtime(&t);
	return(0);
}

int
strftime_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) strftime(&buf[0], 0, "", Strftime_ptm);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strftime_cef()
{
	return(0);
}



/****** Testset for strlen()					 ******/

#ifdef	strlen
#undef	strlen
#endif

int
strlen_csf()
{
	return(0);
}

int
strlen_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strlen("");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strlen_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for strncasecmp()				 ******/

#ifdef	strncasecmp
#undef	strncasecmp
#endif

int
strncasecmp_csf()
{
	return(0);
}

int
strncasecmp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strncasecmp("", "", 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strncasecmp_cef()
{
	return(0);
}

#endif


/****** Testset for strncat()					 ******/

#ifdef	strncat
#undef	strncat
#endif

int
strncat_csf()
{
	return(0);
}

int
strncat_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	buf[0]	= '\0';
	strncat(&buf[0], "", 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strncat_cef()
{
	return(0);
}



/****** Testset for strncmp()					 ******/

#ifdef	strncmp
#undef	strncmp
#endif

int
strncmp_csf()
{
	return(0);
}

int
strncmp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strncmp("", "", 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strncmp_cef()
{
	return(0);
}



/****** Testset for strncpy()					 ******/

#ifdef	strncpy
#undef	strncpy
#endif

int
strncpy_csf()
{
	return(0);
}

int
strncpy_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	buf[0]	= 0;
	strncpy(&buf[0], "", 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strncpy_cef()
{
	return(0);
}



/****** Testset for strpbrk()					 ******/

#ifdef	strpbrk
#undef	strpbrk
#endif

int
strpbrk_csf()
{
	return(0);
}

int
strpbrk_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strpbrk("", "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strpbrk_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for strptime()					 ******/

#ifdef	strptime
#undef	strptime
#endif

static struct tm	*Strptime_ptm;

int
strptime_csf()
{
	time_t	t;

	t = time(NULL);
	Strptime_ptm = localtime(&t);

	return(0);
}

int
strptime_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	buf[0]	= '\0';
	strptime(&buf[0], "", Strptime_ptm);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strptime_cef()
{
	return(0);
}

#endif


/****** Testset for strrchr()					 ******/

#ifdef	strrchr
#undef	strrchr
#endif

int
strrchr_csf()
{
	return(0);
}

int
strrchr_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strrchr("", 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strrchr_cef()
{
	return(0);
}



/****** Testset for strspn()					 ******/

#ifdef	strspn
#undef	strspn
#endif

int
strspn_csf()
{
	return(0);
}

int
strspn_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strspn("", "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strspn_cef()
{
	return(0);
}



/****** Testset for strstr()					 ******/

#ifdef	strstr
#undef	strstr
#endif

int
strstr_csf()
{
	return(0);
}

int
strstr_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strstr("", "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strstr_cef()
{
	return(0);
}



/****** Testset for strtod()					 ******/

#ifdef	strtod
#undef	strtod
#endif

int
strtod_csf()
{
	return(0);
}

int
strtod_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	*p;

	errs	= 0;
	fails	= 0;

	(void) strtod("0.0", &p);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strtod_cef()
{
	return(0);
}



/****** Testset for strtok()					 ******/

#ifdef	strtok
#undef	strtok
#endif

int
strtok_csf()
{
	return(0);
}

int
strtok_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	buf[0] = '\0';
	(void) strtok(&buf[0], "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strtok_cef()
{
	return(0);
}


#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for strtok_r()					 ******/

#ifdef	strtok_r
#undef	strtok_r
#endif

int
strtok_r_csf()
{
	return(0);
}

int
strtok_r_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];
	char	*p;

	errs	= 0;
	fails	= 0;

	buf[0] = '\0';
	(void) strtok_r(&buf[0], "", &p);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strtok_r_cef()
{
	return(0);
}

#endif


/****** Testset for strtol()					 ******/

#ifdef	strtol
#undef	strtol
#endif

int
strtol_csf()
{
	return(0);
}

int
strtol_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strtol("0", NULL, 10);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strtol_cef()
{
	return(0);
}



/****** Testset for strtoul()					 ******/

#ifdef	strtoul
#undef	strtoul
#endif

int
strtoul_csf()
{
	return(0);
}

int
strtoul_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) strtoul("0", NULL, 10);
	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strtoul_cef()
{
	return(0);
}



/****** Testset for strxfrm()					 ******/

#ifdef	strxfrm
#undef	strxfrm
#endif

int
strxfrm_csf()
{
	return(0);
}

int
strxfrm_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	buf[0] = '\0';
	(void) strxfrm(&buf[0], "", 0);


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strxfrm_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for swab()					 ******/

#ifdef	swab
#undef	swab
#endif

int
swab_csf()
{
	return(0);
}

int
swab_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf1[1];
	char	buf2[1];

	errs	= 0;
	fails	= 0;

	buf1[0] = '\0';
	buf2[0] = '\0';

	(void) swab(&buf1[0], &buf2[0], 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
swab_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for swapcontext()				 ******/

#ifdef	swapcontext
#undef	swapcontext
#endif

int
swapcontext_csf()
{
	return(0);
}

int
swapcontext_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
swapcontext_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for swprintf()					 ******/

#ifdef	swprintf
#undef	swprintf
#endif

static wchar_t	Swprintf_buf[16];

int
swprintf_csf()
{
	int	rval;

	rval = mbstowcs(&Swprintf_buf[0], "", 16);
	return( rval == -1 ? rval : 0);
}

int
swprintf_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	wchar_t	buf[16];

	errs	= 0;
	fails	= 0;

	(void) swprintf(&buf[0], 16, &Swprintf_buf[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
swprintf_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for swscanf()					 ******/

#ifdef	swscanf
#undef	swscanf
#endif

int
swscanf_csf()
{
	return(0);
}

int
swscanf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;


	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
swscanf_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for symlink()					 ******/

#ifdef	symlink
#undef	symlink
#endif

int
symlink_csf()
{
	return(0);
}

int
symlink_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) symlink("./nonesuch", "./nonesuch_link");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
symlink_cef()
{
	(void) remove("./nonesuch");
	(void) remove("./nonesuch_link");
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for sync()					 ******/

#ifdef	sync
#undef	sync
#endif

int
sync_csf()
{
	return(0);
}

int
sync_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sync();

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sync_cef()
{
	return(0);
}

#endif


/****** Testset for sysconf()					 ******/

#ifdef	sysconf
#undef	sysconf
#endif

int
sysconf_csf()
{
	return(0);
}

int
sysconf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) sysconf(_SC_OPEN_MAX);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sysconf_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for syslog()					 ******/

#ifdef	syslog
#undef	syslog
#endif

int
syslog_csf()
{
	return(0);
}

int
syslog_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) syslog(LOG_USER, "vsth-ignore", NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
syslog_cef()
{
	return(0);
}

#endif


/****** Testset for system()					 ******/

#ifdef	system
#undef	system
#endif

int
system_csf()
{
	return(0);
}

int
system_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	system("echo");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
system_cef()
{
	return(0);
}



