
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_cpu.c	1.6
*
*************************************************************************/


#include "std.h"

#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>

#if	( TEST_XPG > 4 )
#include <sys/time.h>
#endif

#include <sys/types.h>
#include <sys/utsname.h>

#if	( TEST_XPG > 4 )
#include <ulimit.h>
#endif

#include <unistd.h>
#include <utime.h>

#if	( TEST_XPG > 4 )
#include <wchar.h>
#endif


#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"


#if	( TEST_XPG > 4 )

/****** Testset for ualarm()					 ******/

#ifdef	ualarm
#undef	ualarm
#endif

int
ualarm_csf()
{
	return(0);
}

int
ualarm_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) ualarm((useconds_t) INT_MAX, (useconds_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ualarm_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for ulimit()					 ******/

#ifdef	ulimit
#undef	ulimit
#endif

int
ulimit_csf()
{
	return(0);
}

int
ulimit_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) ulimit(UL_GETFSIZE);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ulimit_cef()
{
	return(0);
}

#endif


/****** Testset for umask()					 ******/

#ifdef	umask
#undef	umask
#endif

static mode_t	Umask_mask;

int
umask_csf()
{
	Umask_mask = umask(0);
	return(0);
}

int
umask_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) umask(Umask_mask);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
umask_cef()
{
	return(0);
}



/****** Testset for uname()					 ******/

#ifdef	uname
#undef	uname
#endif

int
uname_csf()
{
	return(0);
}

int
uname_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	struct utsname	u;

	errs	= 0;
	fails	= 0;

	(void) uname(&u);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
uname_cef()
{
	return(0);
}



/****** Testset for ungetc()					 ******/

#ifdef	ungetc
#undef	ungetc
#endif

static FILE	*Ungetc_fp;

int
ungetc_csf()
{
	int	rval;

	rval = 0;

	Ungetc_fp = fopen("./nonesuch", "w+");
	if ( Ungetc_fp == NULL )
	{
		++rval;
	}

	return(rval);
}

int
ungetc_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) ungetc('X', Ungetc_fp);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ungetc_cef()
{
	if ( Ungetc_fp != NULL )
	{
		(void) fclose(Ungetc_fp);
	}
	(void) remove("./nonesuch");
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for ungetwc()					 ******/

#ifdef	ungetwc
#undef	ungetwc
#endif

static FILE	*Ungetwc_fp;

int
ungetwc_csf()
{
	int	rval;

	rval = 0;

	Ungetwc_fp = fopen("./nonesuch", "w+");
	if ( Ungetwc_fp == NULL )
	{
		++rval;
	}

	return(rval);
}

int
ungetwc_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) ungetwc((wint_t) 'X', Ungetwc_fp);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ungetwc_cef()
{
	if ( Ungetwc_fp != NULL )
	{
		(void) fclose(Ungetwc_fp);
	}
	(void) remove("./nonesuch");
	return(0);
}

#endif


/****** Testset for unlink()					 ******/

#ifdef	unlink
#undef	unlink
#endif

int
unlink_csf()
{
	return(0);
}

int
unlink_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) unlink("./nonesuch");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
unlink_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for unlockpt()					 ******/

#ifdef	unlockpt
#undef	unlockpt
#endif

int
unlockpt_csf()
{
	return(0);
}

int
unlockpt_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) unlockpt(-1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
unlockpt_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for usleep()					 ******/

#ifdef	usleep
#undef	usleep
#endif

int
usleep_csf()
{
	return(0);
}

int
usleep_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) usleep(0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
usleep_cef()
{
	return(0);
}

#endif


/****** Testset for utime()					 ******/

#ifdef	utime
#undef	utime
#endif

int
utime_csf()
{
	return(0);
}

int
utime_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) utime("./nonesuch", NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
utime_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for utimes()					 ******/

#ifdef	utimes
#undef	utimes
#endif

int
utimes_csf()
{
	return(0);
}

int
utimes_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) utimes("./nonesuch", NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
utimes_cef()
{
	return(0);
}

#endif


