
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_cpv.c	1.8
*
*************************************************************************/


#include "std.h"

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#if	( TEST_XPG > 4 )
#include <wchar.h>
#endif


#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"


#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for valloc()					 ******/

#ifdef	valloc
#undef	valloc
#endif

static void	*Valloc_addr;

int
valloc_csf()
{
	return(0);
}

int
valloc_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	Valloc_addr = valloc(1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
valloc_cef()
{
	if ( Valloc_addr != NULL )
	{
		(void) free(Valloc_addr);
	}
	return(0);
}

#endif
#endif

#if	( TEST_XPG > 4 )

/****** Testset for vfork()					 ******/

#ifdef	vfork
#undef	vfork
#endif

int
vfork_csf()
{
	return(0);
}

int
vfork_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not tested, behavior is undefined if calling function returns */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
vfork_cef()
{
	return(0);
}

#endif


/****** Testset for vfprintf()					 ******/

#ifdef	vfprintf
#undef	vfprintf
#endif

static int
vfprintf_func(FILE *stream, const char *format, ... )
{
	int	rval;
	va_list args;

	va_start(args, format);
	rval = vfprintf(stream, format, args);
	va_end(args);
	return(rval);
}

int
vfprintf_csf()
{
	return(0);
}

int
vfprintf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) vfprintf_func(stdout, "%s", "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
vfprintf_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for vfwprintf()					 ******/

#ifdef	vfwprintf
#undef	vfwprintf
#endif

static wchar_t	Vfwprintf_format[32];

static int
vfwprintf_func(FILE *stream, const wchar_t *format, ... )
{
	int	rval;
	va_list args;

	va_start(args, format);
	rval = vfwprintf(stream, format, args);
	va_end(args);
	return(rval);
}

int
vfwprintf_csf()
{
	int	rval;

	rval = mbstowcs(&Vfwprintf_format[0], "%s", 32);

	return(rval == -1 ? 1 : 0);
}

int
vfwprintf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) vfwprintf_func(stdout, &Vfwprintf_format[0], "", NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
vfwprintf_cef()
{
	return(0);
}

#endif


/****** Testset for vprintf()					 ******/

#ifdef	vprintf
#undef	vprintf
#endif

int
vprintf_csf()
{
	return(0);
}

static int
vprintf_func(const char *format, ... )
{
	int	rval;
	va_list args;

	va_start(args, format);
	rval = vprintf(format, args);
	va_end(args);
	return(rval);
}

int
vprintf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) vprintf_func("%s", "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
vprintf_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for vsnprintf()					 ******/

#ifdef	vsnprintf
#undef	vsnprintf
#endif

void
vsnprintf_func(char *buf, int len, char *fmt, ...)
{
	va_list	args;

	va_start(args, fmt);

	(void) vsnprintf(buf, len, fmt, args);

	va_end(args);
}

int
vsnprintf_csf()
{
	return(0);
}

int
vsnprintf_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) vsnprintf_func(&buf[0], 0, "%s", "", NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
vsnprintf_cef()
{
	return(0);
}

#endif


/****** Testset for vsprintf()					 ******/

#ifdef	vsprintf
#undef	vsprintf
#endif

void
vsprintf_func(char *buf, char *fmt, ...)
{
	va_list	args;

	va_start(args, fmt);

	(void) vsprintf(buf, fmt, args);

	va_end(args);
}

int
vsprintf_csf()
{
	return(0);
}

int
vsprintf_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) vsprintf_func(&buf[0], "%s", "", NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
vsprintf_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for vswprintf()					 ******/

#ifdef	vswprintf
#undef	vswprintf
#endif

static wchar_t	Vswprintf_format[32];

static int
vswprintf_func(wchar_t *s, size_t n, wchar_t *format, ...)
{
	int	rval;
	va_list	args;

	va_start(args, format);
	rval = vswprintf(s, n, format, args);
	va_end(args);
	return(rval);
}

int
vswprintf_csf()
{
	int	rval;

	rval = mbstowcs(&Vswprintf_format[0], "%s", 32);

	return(rval == -1 ? 1 : 0);
}

int
vswprintf_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	wchar_t	buf[1];

	errs	= 0;
	fails	= 0;

	(void) vswprintf_func(&buf[0], 0, &Vswprintf_format[0],  "");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
vswprintf_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for vwprintf()					 ******/

#ifdef	vwprintf
#undef	vwprintf
#endif

static wchar_t	Vwprintf_format[32];

static int
vwprintf_func(const wchar_t *format, ... )
{
	int	rval;
	va_list	args;

	va_start(args, format);
	rval = vwprintf(format, args);
	va_end(args);
	return(rval);
}

int
vwprintf_csf()
{
	int	rval;

	rval = mbstowcs(&Vwprintf_format[0], "%s", 32);

	return(0);
}

int
vwprintf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) vwprintf_func(&Vwprintf_format[0], "", NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
vwprintf_cef()
{
	return(0);
}

#endif


