
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_cpw.c	1.12
*
*************************************************************************/


#include "std.h"

#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>

#if	( TEST_XPG > 4 )
#include <sys/uio.h>
#endif

#include <unistd.h>

#if	( TEST_XPG > 4 )
#include <wchar.h>
#include <wctype.h>
#include <wordexp.h>
#endif


#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"



/****** Testset for wait()					 ******/

#ifdef	wait
#undef	wait
#endif

int
wait_csf()
{
	return(0);
}

int
wait_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	status;

	errs	= 0;
	fails	= 0;

	(void) wait(&status);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wait_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for wait3()					 ******/

#ifdef	wait3
#undef	wait3
#endif

int
wait3_csf()
{
	return(0);
}

int
wait3_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	status;

	errs	= 0;
	fails	= 0;

	(void) wait3( &status, 0, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wait3_cef()
{
	return(0);
}

#endif
#endif

#if	( TEST_XPG > 4 )

/****** Testset for waitid()					 ******/

#ifdef	waitid
#undef	waitid
#endif

int
waitid_csf()
{
	return(0);
}

int
waitid_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	siginfo_t	info;

	errs	= 0;
	fails	= 0;

	(void) waitid( P_ALL, (id_t) 0, &info, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
waitid_cef()
{
	return(0);
}

#endif


/****** Testset for waitpid()					 ******/

#ifdef	waitpid
#undef	waitpid
#endif

int
waitpid_csf()
{
	return(0);
}

int
waitpid_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	int	status;

	errs	= 0;
	fails	= 0;

	(void) waitpid(0, &status, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
waitpid_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for wcrtomb()					 ******/

#ifdef	wcrtomb
#undef	wcrtomb
#endif

int
wcrtomb_csf()
{
	return(0);
}

int
wcrtomb_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[MB_LEN_MAX];

	errs	= 0;
	fails	= 0;

	(void) wcrtomb(&buf[0], (wchar_t) 0, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcrtomb_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcscat()					 ******/

#ifdef	wcscat
#undef	wcscat
#endif

static wchar_t	Wcscat_buf1[32];
static wchar_t	Wcscat_buf2[32];

int
wcscat_csf()
{

	int	rval;

	Wcscat_buf1[0] = (wchar_t) 0;

	rval = mbstowcs(&Wcscat_buf2[0], "X", 32);

	return(rval == -1 ? 1 : 0);
}

int
wcscat_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcscat(&Wcscat_buf1[0], &Wcscat_buf2[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcscat_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcschr()					 ******/

#ifdef	wcschr
#undef	wcschr
#endif

static wchar_t	Wcschr_buf[32];

int
wcschr_csf()
{
	int	rval;

	rval = mbstowcs(&Wcschr_buf[0], "X", 32);

	return(rval == -1 ? 1 : 0);
}

int
wcschr_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcschr(&Wcschr_buf[0], (wchar_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcschr_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcscmp()					 ******/

#ifdef	wcscmp
#undef	wcscmp
#endif

static wchar_t	Wcscmp_buf1[32];
static wchar_t	Wcscmp_buf2[32];

int
wcscmp_csf()
{
	int	rval;

	rval = mbstowcs(&Wcscmp_buf1[0], "X", 32);
	if ( rval != -1 )
	{
		rval = mbstowcs(&Wcscmp_buf2[0], "X", 32);
	}
	return(rval == -1 ? 1 : 0);
}

int
wcscmp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcscmp(&Wcscmp_buf1[0], &Wcscmp_buf2[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcscmp_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcscoll()					 ******/

#ifdef	wcscoll
#undef	wcscoll
#endif

static wchar_t	Wcscoll_buf1[32];
static wchar_t	Wcscoll_buf2[32];

int
wcscoll_csf()
{
	int	rval;

	rval = mbstowcs(&Wcscoll_buf1[0], "X", 32);
	if ( rval != -1 )
	{
		rval = mbstowcs(&Wcscoll_buf2[0], "X", 32);
	}
	return(rval == -1 ? 1 : 0);
}

int
wcscoll_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcscoll(&Wcscoll_buf1[0], &Wcscoll_buf2[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcscoll_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcscpy()					 ******/

#ifdef	wcscpy
#undef	wcscpy
#endif

static wchar_t	Wcscpy_buf1[32];
static wchar_t	Wcscpy_buf2[32];

int
wcscpy_csf()
{
	int	rval;

	rval = mbstowcs(&Wcscpy_buf2[0], "X", 32);

	return(rval == -1 ? 1 : 0);
}

int
wcscpy_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcscpy(&Wcscpy_buf1[0], &Wcscpy_buf2[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcscpy_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcscspn()					 ******/

#ifdef	wcscspn
#undef	wcscspn
#endif

static wchar_t	Wcscspn_buf1[32];
static wchar_t	Wcscspn_buf2[32];

int
wcscspn_csf()
{
	int	rval;

	rval = mbstowcs(&Wcscspn_buf1[0], "XYZ", 32);
	if ( rval != -1 )
	{
		rval = mbstowcs(&Wcscspn_buf2[0], "Z", 32);
	}

	return(rval == -1 ? 1 : 0);
}

int
wcscspn_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcscspn(&Wcscspn_buf1[0], &Wcscspn_buf2[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcscspn_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcsftime()					 ******/

#ifdef	wcsftime
#undef	wcsftime
#endif

static wchar_t		Wcsftime_buf[32];
static wchar_t		Wcsftime_format[32];
static struct tm	*Wcsftime_ptm;

int
wcsftime_csf()
{
	int	rval;
	time_t	t;

	rval = mbstowcs(&Wcsftime_format[0], "%A", 32);
	if ( rval != -1 )
	{
		t = time(NULL);
		Wcsftime_ptm = localtime(&t);
	}

	return(rval == -1 ? 1 : 0);
}

int
wcsftime_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

#ifdef	WCSFTIME_BUG
	(void) wcsftime(&Wcsftime_buf[0], 32, (const char *) &Wcsftime_format[0], Wcsftime_ptm);
#else
	(void) wcsftime(&Wcsftime_buf[0], 32, &Wcsftime_format[0], Wcsftime_ptm);
#endif

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcsftime_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcslen()					 ******/

#ifdef	wcslen
#undef	wcslen
#endif

static wchar_t	Wcslen_buf[32];

int
wcslen_csf()
{
	int	rval;

	rval = mbstowcs(&Wcslen_buf[0], "X", 32);

	return(rval == -1 ? 1 : 0);
}

int
wcslen_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcslen(&Wcslen_buf[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcslen_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcsncat()					 ******/

#ifdef	wcsncat
#undef	wcsncat
#endif

static wchar_t	Wcsncat_buf1[32];
static wchar_t	Wcsncat_buf2[32];
static size_t	Wcsncat_len;

int
wcsncat_csf()
{
	int	rval;

	rval = mbstowcs(&Wcsncat_buf1[0], "X", 32);
	if ( rval != -1 )
	{
		Wcsncat_len = mbstowcs(&Wcsncat_buf2[0], "Y", 32);
		if ( Wcsncat_len == -1 )
		{
			rval = -1;
		}
	}

	return(rval == -1 ? 1 : 0);
}

int
wcsncat_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcsncat(&Wcsncat_buf1[0], &Wcsncat_buf2[0], Wcsncat_len);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcsncat_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcsncmp()					 ******/

#ifdef	wcsncmp
#undef	wcsncmp
#endif

static wchar_t	Wcsncmp_buf1[32];
static wchar_t	Wcsncmp_buf2[32];
static int	Wcsncmp_len;

int
wcsncmp_csf()
{
	int	rval;

	rval = mbstowcs(&Wcsncmp_buf1[0], "X", 32);
	if ( rval != -1 )
	{
		Wcsncmp_len = mbstowcs(&Wcsncmp_buf2[0], "Y", 32);
		if ( Wcsncmp_len == -1 )
		{
			rval = -1;
		}
	}

	return(rval == -1 ? 1 : 0);
}

int
wcsncmp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcsncmp(&Wcsncmp_buf1[0], &Wcsncmp_buf2[0], Wcsncmp_len);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcsncmp_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcsncpy()					 ******/

#ifdef	wcsncpy
#undef	wcsncpy
#endif

static wchar_t	Wcsncpy_buf1[32];
static wchar_t	Wcsncpy_buf2[32];
static int	Wcsncpy_len;

int
wcsncpy_csf()
{
	int	rval;

	Wcsncpy_len = mbstowcs(&Wcsncpy_buf2[0], "X", 32);

	return(Wcsncpy_len == -1 ? 1 : 0);
}

int
wcsncpy_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcsncpy(&Wcsncpy_buf1[0], &Wcsncpy_buf2[0], Wcsncpy_len);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcsncpy_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcspbrk()					 ******/

#ifdef	wcspbrk
#undef	wcspbrk
#endif

static wchar_t	Wcspbrk_buf1[32];
static wchar_t	Wcspbrk_buf2[32];

int
wcspbrk_csf()
{
	int	rval;

	rval = mbstowcs(&Wcspbrk_buf1[0], "X", 32);
	if ( rval != -1 )
	{
		rval = mbstowcs(&Wcspbrk_buf2[0], "Y", 32);
	}

	return(rval == -1 ? 1 : 0);
}

int
wcspbrk_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcspbrk(&Wcspbrk_buf1[0], &Wcspbrk_buf2[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcspbrk_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcsrchr()					 ******/

#ifdef	wcsrchr
#undef	wcsrchr
#endif

static wchar_t	Wcsrchr_buf[32];

int
wcsrchr_csf()
{
	int	rval;

	rval = mbstowcs(&Wcsrchr_buf[0], "X", 32);

	return(rval == -1 ? 1 : 0);
}

int
wcsrchr_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcsrchr(&Wcsrchr_buf[0], (wchar_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcsrchr_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcsrtombs()					 ******/

#ifdef	wcsrtombs
#undef	wcsrtombs
#endif

static wchar_t	Wcsrtombs_buf[32];

int
wcsrtombs_csf()
{
	int	rval;

	rval = mbstowcs(&Wcsrtombs_buf[0], "X", 32);

	return(rval == -1 ? 1 : 0);
}

int
wcsrtombs_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	wchar_t	*src;

	errs	= 0;
	fails	= 0;

	src = &Wcsrtombs_buf[0];
	(void) wcsrtombs(NULL, (const wchar_t **) &src, 0, NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcsrtombs_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcsspn()					 ******/

#ifdef	wcsspn
#undef	wcsspn
#endif

static wchar_t	Wcsspn_buf1[32];
static wchar_t	Wcsspn_buf2[32];

int
wcsspn_csf()
{
	int	rval;

	rval = mbstowcs(&Wcsspn_buf1[0], "X", 32);
	if ( rval != -1 )
	{
		rval = mbstowcs(&Wcsspn_buf2[0], "Y", 32);
	}

	return(rval == -1 ? 1 : 0);
}

int
wcsspn_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcsspn(&Wcsspn_buf1[0], &Wcsspn_buf2[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcsspn_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcsstr()					 ******/

#ifdef	wcsstr
#undef	wcsstr
#endif

static wchar_t	Wcsstr_buf1[32];
static wchar_t	Wcsstr_buf2[32];

int
wcsstr_csf()
{
	int	rval;

	rval = mbstowcs(&Wcsstr_buf1[0], "X", 32);
	if ( rval != -1 )
	{
		rval = mbstowcs(&Wcsstr_buf2[0], "Y", 32);
	}

	return(rval == -1 ? 1 : 0);
}

int
wcsstr_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcsstr(&Wcsstr_buf1[0], &Wcsstr_buf2[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcsstr_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcstod()					 ******/

#ifdef	wcstod
#undef	wcstod
#endif

static wchar_t	Wcstod_buf[32];

int
wcstod_csf()
{
	int	rval;

	rval = mbstowcs(&Wcstod_buf[0], "1.0", 32);

	return(rval == -1 ? 1 : 0);
}

int
wcstod_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	wchar_t	*p;

	errs	= 0;
	fails	= 0;

	(void) wcstod(&Wcstod_buf[0], &p);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcstod_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcstok()					 ******/

#ifdef	wcstok
#undef	wcstok
#endif

static wchar_t	Wcstok_buf1[32];
static wchar_t	Wcstok_buf2[32];

int
wcstok_csf()
{
	int	rval;

	rval = mbstowcs(&Wcstok_buf1[0], "xxxXxxxX", 32);
	if ( rval != -1 )
	{
		rval = mbstowcs(&Wcstok_buf2[0], "X", 32);
	}

	return(rval == -1 ? 1 : 0);
}

int
wcstok_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	wchar_t	*p;

	errs	= 0;
	fails	= 0;

	(void) wcstok(&Wcstok_buf1[0], &Wcstok_buf2[0], &p);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcstok_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcstol()					 ******/

#ifdef	wcstol
#undef	wcstol
#endif

static wchar_t	Wcstol_buf[32];

int
wcstol_csf()
{
	int	rval;

	rval = mbstowcs(&Wcstol_buf[0], "1L", 32);

	return(rval == -1 ? 1 : 0);
}

int
wcstol_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	wchar_t	*p;

	errs	= 0;
	fails	= 0;

	(void) wcstol(&Wcstol_buf[0], &p, 10);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcstol_cef()
{
	return(0);
}

#endif


/****** Testset for wcstombs()					 ******/

#ifdef	wcstombs
#undef	wcstombs
#endif

static wchar_t	Wcstombs_buf[32];

int
wcstombs_csf()
{
	int	rval;

	rval = mbstowcs(&Wcstombs_buf[0], "X", 32);

	return(rval == -1 ? 1 : 0);
}

int
wcstombs_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[32];

	errs	= 0;
	fails	= 0;

	(void) wcstombs(&buf[0], &Wcstombs_buf[0], 32);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcstombs_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for wcstoul()					 ******/

#ifdef	wcstoul
#undef	wcstoul
#endif

static wchar_t	Wcstoul_buf[32];

int
wcstoul_csf()
{
	int	rval;

	rval = mbstowcs(&Wcstoul_buf[0], "1L", 32);

	return(rval == -1 ? 1 : 0);
}

int
wcstoul_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	wchar_t	*p;

	errs	= 0;
	fails	= 0;

	(void) wcstoul(&Wcstoul_buf[0], &p, 10);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcstoul_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcswcs()					 ******/

#ifdef	wcswcs
#undef	wcswcs
#endif

static wchar_t	Wcswcs_buf1[32];
static wchar_t	Wcswcs_buf2[32];

int
wcswcs_csf()
{
	int	rval;

	rval = mbstowcs(&Wcswcs_buf1[0], "X", 32);
	if ( rval != -1 )
	{
		rval = mbstowcs(&Wcswcs_buf2[0], "Y", 32);
	}

	return(rval == -1 ? 1 : 0);
}

int
wcswcs_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcswcs(&Wcswcs_buf1[0], &Wcswcs_buf2[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcswcs_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcswidth()					 ******/

#ifdef	wcswidth
#undef	wcswidth
#endif

static wchar_t	Wcswidth_buf[32];

int
wcswidth_csf()
{
	int	rval;

	rval = mbstowcs(&Wcswidth_buf[0], "X", 32);

	return(rval == -1 ? 1 : 0);
}

int
wcswidth_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcswidth(&Wcswidth_buf[0], 32);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcswidth_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcsxfrm()					 ******/

#ifdef	wcsxfrm
#undef	wcsxfrm
#endif

static wchar_t	Wcsxfrm_buf1[32];
static wchar_t	Wcsxfrm_buf2[32];

int
wcsxfrm_csf()
{
	int	rval;

	rval = mbstowcs(&Wcsxfrm_buf2[0], "X", 32);

	return(rval == -1 ? 1 : 0);
}

int
wcsxfrm_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcsxfrm(&Wcsxfrm_buf1[0], &Wcsxfrm_buf2[0], 32);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcsxfrm_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wctob()					 ******/

#ifdef	wctob
#undef	wctob
#endif

int
wctob_csf()
{
	return(0);
}

int
wctob_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wctob((wint_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wctob_cef()
{
	return(0);
}

#endif


/****** Testset for wctomb()					 ******/

#ifdef	wctomb
#undef	wctomb
#endif

int
wctomb_csf()
{
	return(0);
}

int
wctomb_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wctomb(NULL, (wchar_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wctomb_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for wctrans()					 ******/

#ifdef	wctrans
#undef	wctrans
#endif

int
wctrans_csf()
{
	return(0);
}

int
wctrans_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wctrans("tolower");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wctrans_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wctype()					 ******/

#ifdef	wctype
#undef	wctype
#endif

int
wctype_csf()
{
	return(0);
}

int
wctype_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wctype("lower");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wctype_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wcwidth()					 ******/

#ifdef	wcwidth
#undef	wcwidth
#endif

int
wcwidth_csf()
{
	return(0);
}

int
wcwidth_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wcwidth((wchar_t) 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wcwidth_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wmemchr()					 ******/

#ifdef	wmemchr
#undef	wmemchr
#endif

static wchar_t	Wmemchr_buf[32];

int
wmemchr_csf()
{
	int	rval;

	rval = mbstowcs(&Wmemchr_buf[0], "X", 32);

	return(rval == -1 ? 1 : 0);
}

int
wmemchr_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wmemchr(&Wmemchr_buf[0], (wchar_t) 0, 32);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wmemchr_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wmemcmp()					 ******/

#ifdef	wmemcmp
#undef	wmemcmp
#endif

static wchar_t	Wmemcmp_buf1[32];
static wchar_t	Wmemcmp_buf2[32];

int
wmemcmp_csf()
{
	int	rval;

	rval = mbstowcs(&Wmemcmp_buf1[0], "X", 32);
	if ( rval != -1 )
	{
		rval = mbstowcs(&Wmemcmp_buf2[0], "Y", 32);
	}

	return(rval == -1 ? 1 : 0);
}

int
wmemcmp_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wmemcmp( (const wchar_t *) &Wmemcmp_buf1[0],
			(const wchar_t *) &Wmemcmp_buf2[0],
			32 );

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wmemcmp_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wmemcpy()					 ******/

#ifdef	wmemcpy
#undef	wmemcpy
#endif

static wchar_t	Wmemcpy_buf1[32];
static wchar_t	Wmemcpy_buf2[32];

int
wmemcpy_csf()
{
	int	rval;

	rval = mbstowcs(&Wmemcpy_buf2[0], "X", 32);

	return(rval == -1 ? 1 : 0);
}

int
wmemcpy_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wmemcpy(&Wmemcpy_buf1[0], &Wmemcpy_buf2[0], 32);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wmemcpy_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wmemmove()					 ******/

#ifdef	wmemmove
#undef	wmemmove
#endif

static wchar_t	Wmemmove_buf[32];

int
wmemmove_csf()
{
	int	rval;

	rval = mbstowcs(&Wmemmove_buf[0], "1234", 32);

	return(rval == -1 ? 1 : 0);
}

int
wmemmove_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wmemmove(&Wmemmove_buf[0], &Wmemmove_buf[1], 1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wmemmove_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wmemset()					 ******/

#ifdef	wmemset
#undef	wmemset
#endif

int
wmemset_csf()
{
	return(0);
}

int
wmemset_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	wchar_t	wbuf[1];

	errs	= 0;
	fails	= 0;

	(void) wmemset(&wbuf[0], (wchar_t) 0, 1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wmemset_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wordexp()					 ******/

#ifdef	wordexp
#undef	wordexp
#endif

static wordexp_t	Wordexp_words;

int
wordexp_csf()
{
	return(0);
}

int
wordexp_ctf()
{
	int		rval;
	int		errs;
	int		fails;

	errs	= 0;
	fails	= 0;

	(void) wordexp("", &Wordexp_words, 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wordexp_cef()
{
	(void) wordfree(&Wordexp_words);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wprintf()					 ******/

#ifdef	wprintf
#undef	wprintf
#endif

static wchar_t	Wprintf_format[32];

int
wprintf_csf()
{
	int	rval;

	rval = mbstowcs(&Wprintf_format[0], "%s", 32);

	return(rval == -1 ? 1 : 0);
}

int
wprintf_ctf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) wprintf(&Wprintf_format[0], "", NULL);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wprintf_cef()
{
	return(0);
}

#endif


/****** Testset for write()					 ******/

#ifdef	write
#undef	write
#endif

int
write_csf()
{
	return(0);
}

int
write_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) write(-1, &buf[0], 0);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
write_cef()
{
	return(0);
}


#if	( TEST_XPG > 4 )

/****** Testset for writev()					 ******/

#ifdef	writev
#undef	writev
#endif

int
writev_csf()
{
	return(0);
}

int
writev_ctf()
{
	int		rval;
	int		errs;
	int		fails;
	char		buf[1];
	struct iovec	iov;

	errs	= 0;
	fails	= 0;

	iov.iov_base	= (caddr_t) &buf[0];
	iov.iov_len	= 0;

	(void) writev(-1, &iov, 1);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
writev_cef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for wscanf()					 ******/

#ifdef	wscanf
#undef	wscanf
#endif

static wchar_t	Wscanf_format[32];

int
wscanf_csf()
{
	int	rval;

	rval = mbstowcs(&Wscanf_format[0], "", 32);

	return(rval == -1 ? 1 : 0);
}

int
wscanf_ctf()
{
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];

	errs	= 0;
	fails	= 0;

	(void) wscanf(&Wscanf_format[0], &buf[0]);

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
wscanf_cef()
{
	return(0);
}

#endif


