
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_fs.c	1.23
*
*************************************************************************/

#include "std.h"

#include <errno.h>
#include <limits.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <wchar.h>

#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"
#include "vsth/vsth_fs.h"

#include "tet_api.h"

/****** Data structures and functions for PTHR.os/all/General test 2	******/

VSTH_RT	Vsth_fs[] = {
	{	fgetc_fsf,	NULL,	fgetc_fef,	"fgetc"	},
	{	fgets_fsf,	fgets_ftf,	fgets_fef,	"fgets"	},
	{	fprintf_fsf,	fprintf_ftf,	fprintf_fef,	"fprintf"	},
	{	fputc_fsf,	NULL,	fputc_fef,	"fputc"	},
	{	fputs_fsf,	fputs_ftf,	fputs_fef,	"fputs"	},
	{	fread_fsf,	fread_ftf,	fread_fef,	"fread"	},
	{	fscanf_fsf,	fscanf_ftf,	fscanf_fef,	"fscanf"	},
#if	( TEST_XPG > 4 )
#ifdef	NOMSE
	{	fwprintf_fsf,	NULL,	fwprintf_fef,	"fwprintf"	},
#else
	{	fwprintf_fsf,	fwprintf_ftf,	fwprintf_fef,	"fwprintf"	},
#endif
#endif
	{	fwrite_fsf,	fwrite_ftf,	fwrite_fef,	"fwrite"	},
#if	( TEST_XPG > 4 )
#ifdef	NOMSE
	{	fwscanf_fsf,	NULL,	fwscanf_fef,	"fwscanf"	},
#else
	{	fwscanf_fsf,	fwscanf_fsf,	fwscanf_fef,	"fwscanf"	},
#endif
#endif
	{	getc_fsf,	NULL,	getc_fef,	"getc"	},
#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	getw_fsf,	getw_ftf,	getw_fef,	"getw"	},
#endif
#endif
	{	putc_fsf,	NULL,	putc_fef,	"putc"	},
#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)
	{	putw_fsf,	putw_ftf,	putw_fef,	"putw"	},
#endif
#endif
	{	vfprintf_fsf,	vfprintf_ftf,	vfprintf_fef,	"vfprintf"	},
#if	( TEST_XPG > 4 )
#ifdef	NOMSE
	{	vfwprintf_fsf,	NULL,	vfwprintf_fef,	"vfwprintf"	},
#else
	{	vfwprintf_fsf,	vfwprintf_ftf,	vfwprintf_fef,	"vfwprintf"	},
#endif
#endif
	{	NULL,	NULL,	NULL,	NULL	}
};

#define	VSTH_FSCNT	16

/* ARGSUSED */
int
dovfp( FILE *fp, char *fmt, ... )
{
	int	rval;
	va_list	ap;

	va_start(ap, fmt);
	rval = vfprintf(fp, fmt, ap);
	va_end(ap);

	return(rval);
}

#if	( TEST_XPG > 4 )
#ifndef	NOMSE

/* ARGSUSED */
int
dovfwp( FILE *fp, char *fmt, ... )
{
	int	rval;
	wchar_t	wfmt[VSTH_BUFLEN];
	va_list	ap;

	rval = mbstowcs(&wfmt[0], fmt, VSTH_BUFLEN);
	if ( rval != -1 )
	{
		va_start(ap, fmt);
		rval = vfwprintf(fp, wfmt, ap);
		va_end(ap);
	}

	return(rval);
}

#endif
#endif

int
fs_mkf(FILE *fp)
{
	int	c;
	int	cnt;
	int	errs;
	int	limit;
	size_t	val;
	int	rval;
	char	buf[VSTH_BUFLEN + 2];

	errs = 0;

	limit = VSTH_FSCNT * VSTH_SWARM;

	c = 'A';
	for ( cnt = 0; cnt < limit; ++cnt )
	{
		memset(&buf[0], c, VSTH_BUFLEN + 2);

		val = fwrite(&buf[0], sizeof(char), VSTH_BUFLEN, fp);
		if ( val != VSTH_BUFLEN )
		{
			vsth_surprise("fwrite", VSTH_BUFLEN, val, &errs);
			break;
		}

		if ( ++c > 'Z' )
		{
			c = 'A';
		}
	}

	rval = fflush(fp);
	if ( rval != 0 )
	{
		vsth_surprise("fflush", 0, rval, &errs);
	}

	rval = fseek(fp, 0, SEEK_SET);
	if ( rval != 0 )
	{
		vsth_surprise("fseek", 0, rval, &errs);
	}

	rval = 1;
	if ( errs == 0 )
	{
		rval = 0;
	}
	return(rval);
}

int
fs_chkbuf(char *buf)
{
	int	cnt;
	int	rval;
	int	c;

	c	= buf[0];
	rval	= 0;

	if (( 'A' <= c ) && ( c <= 'Z' ))
	{
		for ( cnt = 0; cnt < VSTH_BUFLEN; ++cnt )
		{
			if ( c != (int) buf[cnt] )
			{
				++rval;
				break;
			}
		}
	}

	return(rval);
}

#ifndef	NOMSE

int
fs_mkwf(FILE *fp)
{
	int	c;
	int	cnt;
	int	errs;
	int	limit;
	size_t	val;
	int	rval;
	char	mbs[2];
	wchar_t	wc;
	wchar_t	wfmt[VSTH_BUFLEN];
	wchar_t	buf[VSTH_BUFLEN + 2];

	errs	= 0;
	limit	= VSTH_FSCNT * VSTH_SWARM;

	rval = mbstowcs(&wfmt[0], "%.*ls", VSTH_BUFLEN);
	if ( rval != -1 )
	{

		c	= 'A';
		mbs[1]	= '\0';

		for ( cnt = 0; cnt < limit; ++cnt )
		{
	
			mbs[0] = c;
	
			rval = mbtowc(&wc, &mbs[0], VSTH_MBTOWC_MAXBYTES);
			if ( rval <= 0 )
			{
				++errs;
				tet_printf("mbtowc failed, errno == %d", errno);
				break;
			}
	
			wmemset(&buf[0], wc, VSTH_BUFLEN);
	
			rval = fwprintf(fp, &wfmt[0], VSTH_BUFLEN, &buf[0]);
			if ( rval != VSTH_BUFLEN )
			{
				++errs;
				tet_printf("fwprintf failed, errno == %d", errno);
				break;
			}
	
			if ( ++c > 'Z' )
			{
				c = 'A';
			}
		}

		rval = fflush(fp);
		if ( rval != 0 )
		{
			vsth_surprise("fflush", 0, rval, &errs);
		}
	
		rval = fseek(fp, 0, SEEK_SET);
		if ( rval != 0 )
		{
			vsth_surprise("fseek", 0, rval, &errs);
		}
	}
	else
	{
		++errs;
	}

	rval = 1;
	if ( errs == 0 )
	{
		rval = 0;
	}
	return(rval);
}

int
fs_chkwbuf(wchar_t *buf)
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	limit;
	wchar_t	wc;

	errs	= 0;
	fails	= 0;

	wc	= buf[0];
	for ( cnt = 0; cnt < VSTH_BUFLEN; ++cnt )
	{
		if ( buf[cnt] != wc )
		{
			++fails;
			break;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

#endif	/* ! NOMSE */

#define	VSTH_GETW_VAL	0xa5

int
fs_mkif(FILE *fp)
{
	int	c;
	int	cnt;
	int	errs;
	int	limit;
	int	val;
	int	rval;

	errs = 0;

	limit = VSTH_FSCNT * VSTH_SWARM;

	for ( cnt = 0; cnt < limit; ++cnt )
	{
		for ( val = 0; val < VSTH_BUFLEN; ++val )
		{
			rval = putw(VSTH_GETW_VAL, fp);
			if ( rval != 0 )
			{
				++errs;
			}
		}
	}

	rval = fflush(fp);
	if ( rval != 0 )
	{
		vsth_surprise("fflush", 0, rval, &errs);
	}

	rval = fseek(fp, 0, SEEK_SET);
	if ( rval != 0 )
	{
		vsth_surprise("fseek", 0, rval, &errs);
	}

	rval = 1;
	if ( errs == 0 )
	{
		rval = 0;
	}
	return(rval);
}

int
fs_chkibuf(int *buf)
{
	int	cnt;
	int	rval;
	int	c;

	c	= buf[0];
	rval	= 0;

	for ( cnt = 0; cnt < VSTH_BUFLEN; ++cnt )
	{
		if ( buf[cnt] != VSTH_GETW_VAL )
		{
			++rval;
			break;
		}
	}

	return(rval);
}

/****** Testset for fgetc()					 ******/

#define VSTH_FGETC_FILENAME	"./fgetc_file"

static FILE	*Vsth_fgetc_fp = NULL;
static int	Vsth_fgetc_init = 0;

int
fgetc_fsf()
{
	if ( ! Vsth_fgetc_init )
	{
		(void) remove(VSTH_FGETC_FILENAME);

		Vsth_fgetc_fp = fopen(VSTH_FGETC_FILENAME, "w+");
		Vsth_fgetc_init = 1;
	}
	return(Vsth_fgetc_fp ? 0 : 1);
}

int
fgetc_ftf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* character at a time input, no meaningful test */
	tet_infoline("No meaningful test");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fgetc_fef()
{
	(void) fclose(Vsth_fgetc_fp);
	if ( Vsth_debug == 0 )
	{
		(void) remove(VSTH_FGETC_FILENAME);
	}
	return(0);
}

/****** Testset for fgets()					 ******/

#define VSTH_FGETS_FILENAME	"./fgets_file"

static FILE	*Vsth_fgets_fp = NULL;
static int	Vsth_fgets_init = 0;

int
fgets_fsf()
{
	int	errs;
	int	rval;

	errs = 0;

	if ( ! Vsth_fgets_init )
	{
		(void) remove(VSTH_FGETS_FILENAME);

		Vsth_fgets_fp = fopen(VSTH_FGETS_FILENAME, "w+");
		if ( Vsth_fgets_fp )
		{
			rval = fs_mkf(Vsth_fgets_fp);
			if ( rval )
			{
				++errs;
				tet_infoline("Unable to make test file");
			}
		}
		else
		{
			++errs;
			tet_printf("fopen failed, errno == %d", errno);
		}
		Vsth_fgets_init = 1;
	}

	rval = 1;
	if ( errs == 0 )
	{
		rval = 0;
	}
	return(rval);
}

int
fgets_ftf()
{
	int	rval;
	int	errs;
	int	fails;
	int	cnt;
	char	*val;
	char	buf[VSTH_BUFLEN + 1];

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < VSTH_FSCNT; ++cnt )
	{
		memset(&buf[0], 0, VSTH_BUFLEN + 1);

		val = fgets(&buf[0], VSTH_BUFLEN + 1, Vsth_fgets_fp);
		if ( val == &buf[0] )
		{
			rval = fs_chkbuf(&buf[0]);
			if ( rval )
			{
				++fails;
			}
		}
		else
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fgets_fef()
{
	(void) fclose(Vsth_fgets_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_FGETS_FILENAME);
	}
	return(0);
}

/****** Testset for fprintf()					 ******/

#define VSTH_FPRINTF_FILENAME	"./fprintf_file"

static FILE	*Vsth_fprintf_fp = NULL;
static int	Vsth_fprintf_init = 0;

int
fprintf_fsf()
{
	if ( ! Vsth_fprintf_init )
	{
		(void) remove(VSTH_FPRINTF_FILENAME);

		Vsth_fprintf_fp = fopen(VSTH_FPRINTF_FILENAME, "w+");
		Vsth_fprintf_init = 1;
	}
	return(Vsth_fprintf_fp ? 0 : 1);
}

int
fprintf_ftf()
{
	int	c;
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	c = 'A';
	for ( cnt = 0; cnt < VSTH_FSCNT; ++cnt )
	{
		memset(&buf[0], c, VSTH_BUFLEN);
		fprintf( Vsth_fprintf_fp, "%.*s", VSTH_BUFLEN, &buf[0]);

		if ( ++c > 'Z' )
		{
			c = 'A';
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fprintf_fef()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	limit;
	char	*val;
	char	buf[VSTH_BUFLEN + 1];

	errs	= 0;
	fails	= 0;

	rval = fseek(Vsth_fprintf_fp, 0, SEEK_SET);
	if ( rval == 0 )
	{
		limit = VSTH_FSCNT * VSTH_SWARM;

		for ( cnt = 0; cnt < limit; ++cnt )
		{
			memset(&buf[0], 0, VSTH_BUFLEN + 1);

			val = fgets(&buf[0], VSTH_BUFLEN + 1, Vsth_fprintf_fp);
			if ( val == &buf[0] )
			{
				rval = fs_chkbuf(&buf[0]);
				if ( rval )
				{
					++fails;
					break;
				}
			}
			else
			{
				++fails;
				break;
			}
		}
	}
	else
	{
		++errs;
	}

	(void) fclose(Vsth_fprintf_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_FPRINTF_FILENAME);
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

/****** Testset for fputc()					 ******/

#define VSTH_FPUTC_FILENAME	"./fputc_file"

static FILE	*Vsth_fputc_fp = NULL;
static int	Vsth_fputc_init = 0;

int
fputc_fsf()
{
	if ( ! Vsth_fputc_init )
	{
		(void) remove(VSTH_FPUTC_FILENAME);

		Vsth_fputc_fp = fopen(VSTH_FPUTC_FILENAME, "w+");
		Vsth_fputc_init = 1;
	}
	return(Vsth_fputc_fp ? 0 : 1);
}

int
fputc_ftf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* character at a time input, no meaningful test */
	tet_infoline("No meaningful test");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fputc_fef()
{
	(void) fclose(Vsth_fputc_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_FPUTC_FILENAME);
	}
	return(0);
}

/****** Testset for fputs()					 ******/

#define VSTH_FPUTS_FILENAME	"./fputs_file"

static FILE	*Vsth_fputs_fp = NULL;
static int	Vsth_fputs_init = 0;

int
fputs_fsf()
{
	if ( ! Vsth_fputs_init )
	{
		(void) remove(VSTH_FPUTS_FILENAME);

		Vsth_fputs_fp = fopen(VSTH_FPUTS_FILENAME, "w+");
		Vsth_fputs_init = 1;
	}
	return(Vsth_fputs_fp ? 0 : 1);
}

int
fputs_ftf()
{
	int	c;
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	buf[VSTH_BUFLEN + 1];

	errs	= 0;
	fails	= 0;

	c = 'A';
	for ( cnt = 0; cnt < VSTH_FSCNT; ++cnt )
	{
		memset(&buf[0], c, VSTH_BUFLEN);
		buf[VSTH_BUFLEN] = '\0';

		rval = fputs( &buf[0], Vsth_fputs_fp );
		if ( rval != VSTH_BUFLEN )
		{
			++errs;
		}

		if (++c > 'Z')
		{
			c = 'A';
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fputs_fef()
{
	int	rval;
	int	limit;
	int	errs;
	int	fails;
	int	cnt;
	char	*val;
	char	buf[VSTH_BUFLEN + 1];

	errs	= 0;
	fails	= 0;

	rval = fseek(Vsth_fputs_fp, 0, SEEK_SET);
	if ( rval == 0 )
	{

		limit = VSTH_FSCNT * VSTH_SWARM;

		for ( cnt = 0; cnt < limit; ++cnt )
		{
			memset(&buf[0], 0, VSTH_BUFLEN + 1);
	
			val = fgets(&buf[0], VSTH_BUFLEN + 1, Vsth_fputs_fp);
			if ( val == &buf[0] )
			{
				rval = fs_chkbuf(&buf[0]);
				if ( rval )
				{
					++fails;
				}
			}
			else
			{
				++fails;
			}
		}
	}
	else
	{
		++errs;
	}

	(void) fclose(Vsth_fputs_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_FPUTS_FILENAME);
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

/****** Testset for fread()					 ******/

#define VSTH_FREAD_FILENAME	"./fread_file"

static FILE	*Vsth_fread_fp = NULL;
static int	Vsth_fread_init = 0;

int
fread_fsf()
{
	int	rval;
	int	errs;

	errs = 0;

	if ( ! Vsth_fread_init )
	{
		(void) remove(VSTH_FREAD_FILENAME);

		Vsth_fread_fp = fopen(VSTH_FREAD_FILENAME, "w+");
		if ( Vsth_fread_fp != NULL )
		{
			rval = fs_mkf(Vsth_fread_fp);
			if ( rval != 0 )
			{
				tet_infoline("Unable to make test file");
				++errs;
			}
		}
		else
		{
			tet_printf("fopen failed, errno == %d", errno);
			++errs;
		}

		Vsth_fread_init = 1;
	}

	rval = 1;
	if ( errs == 0 )
	{
		rval = 0;
	}

	return(rval);
}

int
fread_ftf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	size_t	val;
	char	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < VSTH_FSCNT; ++cnt )
	{
		memset(&buf[0], 0, VSTH_BUFLEN);

		val = fread((void *) &buf[0], sizeof(char), VSTH_BUFLEN, Vsth_fread_fp);
		if ( val == VSTH_BUFLEN )
		{
			rval = fs_chkbuf(&buf[0]);
			if ( rval )
			{
				if ( ! fails )
				{
					tet_printf("fread read [%.*s]",
							VSTH_BUFLEN, &buf[0]);
				}
				++fails;
				break;
			}
		}
		else
		{
			if ( ! fails )
			{
				tet_printf("fread returned %d, errno == %d",
						val, errno);
			}
			++fails;
			break;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fread_fef()
{
	(void) fclose(Vsth_fread_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_FREAD_FILENAME);
	}
	return(0);
}

/****** Testset for fscanf()					 ******/

#define VSTH_FSCANF_FILENAME	"./fscanf_file"

static FILE	*Vsth_fscanf_fp = NULL;
static int	Vsth_fscanf_init = 0;

int
fscanf_fsf()
{
	int	errs;
	int	rval;

	errs	= 0;

	if ( ! Vsth_fscanf_init )
	{
		(void) remove(VSTH_FSCANF_FILENAME);

		Vsth_fscanf_fp = fopen(VSTH_FSCANF_FILENAME, "w+");
		if ( Vsth_fscanf_fp != NULL )
		{
			rval = fs_mkf(Vsth_fscanf_fp);
			if ( rval )
			{
				++errs;
				tet_infoline("Unable to make test file");
			}
		}
		else
		{
			++errs;
			tet_printf("fopen failed, errno == %d", errno);
		}

		Vsth_fscanf_init = 1;
	}

	rval = 1;
	if ( errs == 0 )
	{
		rval = 0;
	}
	return(rval);
}

int
fscanf_ftf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	buf[VSTH_BUFLEN + 1];

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < VSTH_FSCNT; ++cnt )
	{
		memset(&buf[0], 0, VSTH_BUFLEN + 1);

		/*	hack warning ... 
			if VSTH_BUFLEN in vsth_fw.h changes, you must change the
			maximum field width specification in the fscanf statement below */

		rval = fscanf(Vsth_fscanf_fp, "%256s", &buf[0]);
		if (( rval == 0 ) || ( rval == EOF ))
		{
			if ( ! errs )
			{
				tet_printf("fscanf returned %d, errno == %d",
						rval, errno);
			}
			++errs;
		}

		rval = fs_chkbuf(&buf[0]);
		if ( rval != 0 )
		{
			if ( ! fails )
			{
				tet_printf("fscanf read [%.*s]", VSTH_BUFLEN, &buf[0]);
			}
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fscanf_fef()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	(void) fclose(Vsth_fscanf_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_FSCANF_FILENAME);
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

#if	( TEST_XPG > 4 )

/****** Testset for fwprintf()					 ******/

#define VSTH_FWPRINTF_FILENAME	"./fwprintf_file"

static FILE	*Vsth_fwprintf_fp = NULL;
static int	Vsth_fwprintf_init = 0;

int
fwprintf_fsf()
{
#ifdef	NOMSE
	return(0);
#else
	if ( ! Vsth_fwprintf_init )
	{
		(void) remove(VSTH_FWPRINTF_FILENAME);

		Vsth_fwprintf_fp = fopen(VSTH_FWPRINTF_FILENAME, "w+");
		Vsth_fwprintf_init = 1;
	}
	return(Vsth_fwprintf_fp ? 0 : 1);
#endif
}

int
fwprintf_ftf()
{
#ifdef	NOMSE
	return(0);
#else
	int	c;
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	mbs[2];
	wchar_t	wc;
	wchar_t	wfmt[VSTH_BUFLEN];
	wchar_t	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	rval = mbstowcs(&wfmt[0], "%.*ls", VSTH_BUFLEN);
	if ( rval != -1 )
	{
		c	= 'A';
		mbs[1]	= '\0';

		for ( cnt = 0; cnt < VSTH_FSCNT; ++cnt )
		{
			mbs[0] = c;
	
			rval = mbtowc(&wc, &mbs[0], VSTH_MBTOWC_MAXBYTES);
			if ( rval <= 0 )
			{
				++errs;
				break;
			}
	
			wmemset(&buf[0], wc, VSTH_BUFLEN);
	
			rval = fwprintf(Vsth_fwprintf_fp, &wfmt[0], VSTH_BUFLEN, &buf[0]);
			if ( rval != VSTH_BUFLEN )
			{
				++errs;
			}
	
			if ( ++c > 'Z' )
			{
				c = 'A';
			}
		}
	}
	else
	{
		++errs;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
#endif
}

int
fwprintf_fef()
{
#ifdef	NOMSE
	return(0);
#else
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	limit;
	wchar_t	buf[VSTH_BUFLEN + 1];
	wchar_t	wfmt[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	/*	hack warning ...
		if VSTH_BUFLEN changes in vsth_fw.h, you must change the maximum
		field width specifier in the format string below */

	rval = mbstowcs(&wfmt[0], "%256ls", VSTH_BUFLEN);
	if ( rval != -1 )
	{
		rval = fseek(Vsth_fwprintf_fp, 0, SEEK_SET);
		if ( rval == 0 )
		{
			limit	= VSTH_FSCNT * VSTH_SWARM;
	
			for ( cnt = 0; cnt < limit; ++cnt )
			{
				wmemset(&buf[0], (wchar_t) 0, VSTH_BUFLEN + 1);
	
				rval = fwscanf(Vsth_fwprintf_fp, &wfmt[0], &buf[0]);
				if ( rval > 0 )
				{
					rval = fs_chkwbuf(&buf[0]);
					if ( rval )
					{
						++fails;
					}
				}
				else
				{
					++fails;
				}
			}
		}
		else
		{
			++errs;
		}
	}
	else
	{
		++errs;
	}

	(void) fclose(Vsth_fwprintf_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_FWPRINTF_FILENAME);
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
#endif
}

#endif

/****** Testset for fwrite()					 ******/

#define VSTH_FWRITE_FILENAME	"./fwrite_file"

static FILE	*Vsth_fwrite_fp = NULL;
static int	Vsth_fwrite_init = 0;

int
fwrite_fsf()
{
	if ( ! Vsth_fwrite_init )
	{
		(void) remove(VSTH_FWRITE_FILENAME);

		Vsth_fwrite_fp = fopen(VSTH_FWRITE_FILENAME, "w+");
		Vsth_fwrite_init = 1;
	}
	return(Vsth_fwrite_fp ? 0 : 1);
}

int
fwrite_ftf()
{
	int	c;
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	c = 'A';
	for ( cnt = 0; cnt < VSTH_FSCNT; ++cnt )
	{
		memset(&buf[0], c, VSTH_BUFLEN);

		rval = fwrite(&buf[0], sizeof(char), VSTH_BUFLEN, Vsth_fwrite_fp) ;
		if ( rval != VSTH_BUFLEN )
		{
			++errs;
			tet_printf("fwrite failed, errno == %d", errno);
		}

		if ( ++c > 'Z' )
		{
			c = 'A';
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fwrite_fef()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	limit;
	char	*val;
	char	buf[VSTH_BUFLEN + 1];

	errs	= 0;
	fails	= 0;

	rval = fseek(Vsth_fwrite_fp, 0, SEEK_SET);
	if ( rval == 0 )
	{
		limit = VSTH_FSCNT * VSTH_SWARM;

		for ( cnt = 0; cnt < limit; ++cnt )
		{
			memset(&buf[0], 0, VSTH_BUFLEN);

			val = fgets(&buf[0], VSTH_BUFLEN + 1, Vsth_fwrite_fp);
			if ( val == &buf[0] )
			{
				rval = fs_chkbuf(&buf[0]);
				if ( rval )
				{
					if ( ! fails )
					{
						tet_printf("read [%.*s]",
								VSTH_BUFLEN, &buf[0]);
					}
					++fails;
				}
			}
			else
			{
				if ( ! fails )
				{
					tet_printf("fgets returned %p, expected %p",
							val, &buf[0]);
				}
				++fails;
			}
		}
	}
	else
	{
		++errs;
	}

	(void) fclose(Vsth_fwrite_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_FWRITE_FILENAME);
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

#if	( TEST_XPG > 4 )

/****** Testset for fwscanf()					 ******/

#define VSTH_FWSCANF_FILENAME	"./fwscanf_file"

static FILE	*Vsth_fwscanf_fp = NULL;
static int	Vsth_fwscanf_init = 0;

int
fwscanf_fsf()
{
#ifdef	NOMSE
	return(0);
#else
	int	rval;
	int	errs;

	errs = 0;

	if ( ! Vsth_fwscanf_init )
	{
		(void) remove(VSTH_FWSCANF_FILENAME);

		Vsth_fwscanf_fp = fopen(VSTH_FWSCANF_FILENAME, "w+");
		if ( Vsth_fwscanf_fp != NULL )
		{
			rval = fs_mkwf(Vsth_fwscanf_fp);
			if ( rval )
			{
				++errs;
				tet_infoline("Unable to make test file");
			}
		}
		else
		{
			++errs;
			tet_printf("fopen failed, errno == %d", errno);
		}

		Vsth_fwscanf_init = 1;
	}

	rval = 1;
	if ( errs == 0 )
	{
		rval = 0;
	}
	return(rval);
#endif
}

int
fwscanf_ftf()
{
#ifdef	NOMSE
	return(0);
#else
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	wchar_t	wfmt[VSTH_BUFLEN];
	wchar_t	buf[VSTH_BUFLEN + 1];

	errs	= 0;
	fails	= 0;

	/* hack alert - if VSTH_BUFLEN changes, change 256 below */

	rval = mbstowcs(&wfmt[0], "%.256ls", VSTH_BUFLEN);
	if ( rval != -1 )
	{
		for ( cnt = 0; cnt < VSTH_FSCNT; ++cnt )
		{
			wmemset(&buf[0], (wchar_t) 0, VSTH_BUFLEN + 1);
	
			rval = fwscanf(Vsth_fwscanf_fp, &wfmt[0], &buf[0]);
			if (( rval != 0 ) && ( rval != EOF ))
			{
				rval = fs_chkwbuf(&buf[0]);
				if ( rval )
				{
					++fails;
				}
			}
			else
			{
				++fails;
			}
		}
	}
	else
	{
		++errs;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
#endif
}

int
fwscanf_fef()
{
#ifdef	NOMSE
	return(0);
#else
	(void) fclose(Vsth_fwscanf_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_FWSCANF_FILENAME);
	}
	return(0);
#endif
}

#endif

/****** Testset for getc()					 ******/

#define VSTH_GETC_FILENAME	"./getc_file"

static FILE	*Vsth_getc_fp = NULL;
static int	Vsth_getc_init = 0;

int
getc_fsf()
{
	if ( ! Vsth_getc_init )
	{
		(void) remove(VSTH_GETC_FILENAME);

		Vsth_getc_fp = fopen(VSTH_GETC_FILENAME, "w+");
		Vsth_getc_init = 1;
	}
	return(Vsth_getc_fp ? 0 : 1);
}

int
getc_ftf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* character at a time input, no meaningful test */
	tet_infoline("No meaningful test");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getc_fef()
{
	(void) fclose(Vsth_getc_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_GETC_FILENAME);
	}
	return(0);
}

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for getw()					 ******/

#define VSTH_GETW_FILENAME	"./getw_file"

static FILE	*Vsth_getw_fp = NULL;
static int	Vsth_getw_init = 0;

int
getw_fsf()
{
	int	rval;
	int	errs;

	errs = 0;

	if ( ! Vsth_getw_init )
	{
		(void) remove(VSTH_GETW_FILENAME);

		Vsth_getw_fp = fopen(VSTH_GETW_FILENAME, "w+");
		if ( Vsth_getw_fp != NULL )
		{
			rval = fs_mkif(Vsth_getw_fp);
			if ( rval != 0 )
			{
				tet_infoline("Unable to make test file");
				++errs;
			}
		}
		else
		{
			tet_printf("fopen failed, errno == %d", errno);
			++errs;
		}
		Vsth_getw_init = 1;
	}

	rval = 1;
	if ( errs == 0 )
	{
		rval = 0;
	}
	return(rval);
}

int
getw_ftf()
{
	int	c;
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	val;
	int	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < VSTH_FSCNT; ++cnt )
	{
		memset(&buf[0], 0, sizeof(int) * VSTH_BUFLEN);

		for ( val = 0; val < VSTH_BUFLEN; ++val )
		{
			buf[val] = getw(Vsth_getw_fp);
		}

		rval = fs_chkibuf(&buf[0]);
		if ( rval != 0 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
getw_fef()
{
	(void) fclose(Vsth_getw_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_GETW_FILENAME);
	}
	return(0);
}

#endif
#endif

/****** Testset for putc()					 ******/

#define VSTH_PUTC_FILENAME	"./putc_file"

static FILE	*Vsth_putc_fp = NULL;
static int	Vsth_putc_init = 0;

int
putc_fsf()
{
	if ( ! Vsth_putc_init )
	{
		(void) remove(VSTH_PUTC_FILENAME);

		Vsth_putc_fp = fopen(VSTH_PUTC_FILENAME, "w+");
		Vsth_putc_init = 1;
	}
	return(Vsth_putc_fp ? 0 : 1);
}

int
putc_ftf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* character at a time input, no meaningful test */
	tet_infoline("No meaningful test");

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
putc_fef()
{
	(void) fclose(Vsth_putc_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_PUTC_FILENAME);
	}
	return(0);
}

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for putw()					 ******/

#define VSTH_PUTW_FILENAME	"./putw_file"

static FILE	*Vsth_putw_fp = NULL;
static int	Vsth_putw_init = 0;

int
putw_fsf()
{
	if ( ! Vsth_putw_init )
	{
		(void) remove(VSTH_PUTW_FILENAME);

		Vsth_putw_fp = fopen(VSTH_PUTW_FILENAME, "w+");
		Vsth_putw_init = 1;
	}
	return(Vsth_putw_fp ? 0 : 1);
}

int
putw_ftf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	val;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < VSTH_FSCNT; ++cnt )
	{
		for ( val = 0; val < VSTH_BUFLEN; ++val )
		{
			rval = putw(VSTH_GETW_VAL, Vsth_putw_fp);
			if ( rval != 0 )
			{
				++errs;
			}
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
putw_fef()
{
	int	cnt;
	int	errs;
	int	fails;
	int	limit;
	int	rval;
	int	val;
	int	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	rval = fseek(Vsth_putw_fp, 0, SEEK_SET);
	if ( rval == 0 )
	{
		limit = VSTH_FSCNT * VSTH_SWARM;

		for ( cnt = 0; cnt < limit; ++cnt )
		{
			memset(&buf[0], 0, sizeof(int) * VSTH_BUFLEN);

			for ( val = 0; val < VSTH_BUFLEN; ++val )
			{
				buf[val] = getw(Vsth_putw_fp);
			}
	
			rval = fs_chkibuf(&buf[0]);
			if ( rval )
			{
				++fails;
			}
		}
	}
	else
	{
		++errs;
	}

	(void) fclose(Vsth_putw_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_PUTW_FILENAME);
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

#endif
#endif

/****** Testset for vfprintf()					 ******/

#define VSTH_VFPRINTF_FILENAME	"./vfprintf_file"

static FILE	*Vsth_vfprintf_fp = NULL;
static int	Vsth_vfprintf_init = 0;

int
vfprintf_fsf()
{
	if ( ! Vsth_vfprintf_init )
	{
		(void) remove(VSTH_VFPRINTF_FILENAME);

		Vsth_vfprintf_fp = fopen(VSTH_VFPRINTF_FILENAME, "w+");
		Vsth_vfprintf_init = 1;
	}
	return(Vsth_vfprintf_fp ? 0 : 1);
}

int
vfprintf_ftf()
{
	int	c;
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	c = 'A';
	for ( cnt = 0; cnt < VSTH_FSCNT; ++cnt )
	{
		memset(&buf[0], c, VSTH_BUFLEN);

		rval = dovfp(Vsth_vfprintf_fp, "%.*s", VSTH_BUFLEN, &buf[0], NULL);
		if ( rval != VSTH_BUFLEN )
		{
			if ( ! fails )
			{
				tet_printf("vfprintf returned %d, errno == %d",
						rval, errno);
			}
			++fails;
		}

		if ( ++c > 'Z' )
		{
			c = 'A';
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
vfprintf_fef()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	limit;
	char	*val;
	char	buf[VSTH_BUFLEN + 1];

	errs	= 0;
	fails	= 0;

	rval = fseek(Vsth_vfprintf_fp, 0, SEEK_SET);
	if ( rval == 0 )
	{
		limit = VSTH_FSCNT * VSTH_SWARM;

		for ( cnt = 0; cnt < limit; ++cnt )
		{
			memset(&buf[0], 0, VSTH_BUFLEN + 1);

			val = fgets(&buf[0], VSTH_BUFLEN + 1, Vsth_vfprintf_fp);
			if ( val == &buf[0] )
			{
				rval = fs_chkbuf(&buf[0]);
				if ( rval )
				{
					++fails;
					break;
				}
			}
			else
			{
				++fails;
				break;
			}
		}
	}
	else
	{
		++errs;
	}

	(void) fclose(Vsth_vfprintf_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_VFPRINTF_FILENAME);
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

#if	( TEST_XPG > 4 )

/****** Testset for vfwprintf()					 ******/

#define VSTH_VFWPRINTF_FILENAME	"./vfwprintf_file"

static FILE	*Vsth_vfwprintf_fp = NULL;
static int	Vsth_vfwprintf_init = 0;

int
vfwprintf_fsf()
{
#ifdef	NOMSE
	return(0);
#else
	if ( ! Vsth_vfwprintf_init )
	{
		(void) remove(VSTH_VFWPRINTF_FILENAME);

		Vsth_vfwprintf_fp = fopen(VSTH_VFWPRINTF_FILENAME, "w+");
		Vsth_vfwprintf_init = 1;
	}
	return(Vsth_vfwprintf_fp ? 0 : 1);
#endif
}

int
vfwprintf_ftf()
{
#ifdef	NOMSE
	return(0);
#else
	int	c;
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	mbs[2];
	wchar_t	wc;
	wchar_t	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	c	= 'A';
	mbs[1]	= '\0';
	for ( cnt = 0; cnt < VSTH_FSCNT; ++cnt )
	{
		mbs[0] = c;

		rval = mbtowc(&wc, &mbs[0], VSTH_MBTOWC_MAXBYTES);
		if ( rval <= 0 )
		{
			++errs;
			break;
		}

		wmemset(&buf[0], wc, VSTH_BUFLEN);

		rval = dovfwp(Vsth_vfwprintf_fp, "%.*ls", VSTH_BUFLEN, &buf[0], NULL);
		if ( rval != VSTH_BUFLEN )
		{
			++errs;
		}

		if ( ++c > 'Z' )
		{
			c = 'A';
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
#endif
}

int
vfwprintf_fef()
{
#ifdef	NOMSE
	return(0);
#else
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	limit;
	wchar_t	wfmt[VSTH_BUFLEN];
	wchar_t	buf[VSTH_BUFLEN + 1];

	errs	= 0;
	fails	= 0;

	/* hack alert - if VSTH_BUFLEN changes, so must 256 */

	/* TSD5TH.00056: format string amended ('.' removed) */
	rval = mbstowcs(&wfmt[0], "%256ls", VSTH_BUFLEN);
	if ( rval != -1 )
	{
		rval = fseek(Vsth_vfwprintf_fp, 0, SEEK_SET);
		if ( rval == 0 )
		{
			limit	= VSTH_FSCNT * VSTH_SWARM;
	
			for ( cnt = 0; cnt < limit; ++cnt )
			{
				wmemset(&buf[0], (wchar_t) 0, VSTH_BUFLEN + 1);
	
				rval = fwscanf(	Vsth_vfwprintf_fp,
						&wfmt[0],
						&buf[0]);
				if ( rval > 0 )
				{
					rval = fs_chkwbuf(&buf[0]);
					if ( rval )
					{
						++fails;
					}
				}
				else
				{
					++fails;
				}
			}
		}
		else
		{
			++errs;
		}
	}
	else
	{
		++errs;
	}

	(void) fclose(Vsth_vfwprintf_fp);
	if ( ! Vsth_debug )
	{
		(void) remove(VSTH_VFWPRINTF_FILENAME);
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
#endif
}

#endif
