
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_rta.c	1.7
*
*
* CHANGE HISTORY
*	12 May 1999 - Incorporate SR#104, to require ifdefs around aio.h
*************************************************************************/

#include "std.h"

#include <unistd.h>
#if	defined(_POSIX_ASYNCHRONOUS_IO)
#include <aio.h>
#endif
#include <assert.h>
#include <errno.h>
#include <math.h>
#include <pthread.h>

#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <time.h>

#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"

#if	( TEST_XPG > 4 )

/****** Testset for a64l()					 ******/

#ifdef	a64l
#undef	a64l
#endif

int
a64l_sf()
{
	return(0);
}

int
a64l_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be re-entrant */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
a64l_ef()
{
	return(0);
}

#endif

/****** Testset for abort()					 ******/

#ifdef	abort
#undef	abort
#endif

int
abort_sf()
{
	return(0);
}

int
abort_tf()
{
	int	rval;
	int	errs;
	int	fails;
	int	status;
	pid_t	pid;
	pid_t	wpid;

	errs	= 0;
	fails	= 0;

	/* not testable - abort aborts all threads */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
abort_ef()
{
	return(0);
}

/****** Testset for abs()					 ******/

#ifdef	abs
#undef	abs
#endif

int
abs_sf()
{
	return(0);
}

int
abs_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = abs(-1);
	if ( rval != 1 )
	{
		++fails;
	}

	rval = abs(1);
	if ( rval != 1 )
	{
		++fails;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
abs_ef()
{
	return(0);
}

/****** Testset for access()					 ******/

#ifdef	access
#undef	access
#endif

static char	*Access_filename = "./nonesuch";

int
access_sf()
{
	(void) remove(Access_filename);
	return(0);
}

int
access_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = access("..", F_OK);
		if ( rval )
		{
			++fails;
		}
	
		rval = access(Access_filename, F_OK);
		if ( rval == 0 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
access_ef()
{
	return(0);
}

/****** Testset for acos()					 ******/

#ifdef	acos
#undef	acos
#endif

#define	VSTH_ACOS_VAL1		-0.2
#define	VSTH_ACOS_VAL2		0.5
int	Vsth_acos_init = 0;
double	Vsth_acos_val1;
double	Vsth_acos_val2;

int
acos_sf()
{
	if ( ! Vsth_acos_init )
	{
		Vsth_acos_val1 = acos(VSTH_ACOS_VAL1);
		Vsth_acos_val2 = acos(VSTH_ACOS_VAL2);
		Vsth_acos_init = 1;
	}
		
	return(0);
}

int
acos_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = acos(VSTH_ACOS_VAL1);
		if ( rval != Vsth_acos_val1 )
		{
			++fails;
		}

		rval = acos(VSTH_ACOS_VAL2);
		if ( rval != Vsth_acos_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
acos_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for acosh()					 ******/

#ifdef	acosh
#undef	acosh
#endif

#define	VSTH_ACOSH_VAL1		1.2
#define	VSTH_ACOSH_VAL2		1.3
int	Vsth_acosh_init = 0;
double	Vsth_acosh_val1;
double	Vsth_acosh_val2;

int
acosh_sf()
{
	if ( ! Vsth_acosh_init )
	{
		Vsth_acosh_val1 = acosh(VSTH_ACOSH_VAL1);
		Vsth_acosh_val2 = acosh(VSTH_ACOSH_VAL2);
		Vsth_acosh_init = 1;
	}
		
	return(0);
}

int
acosh_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = acosh(VSTH_ACOSH_VAL1);
		if ( rval != Vsth_acosh_val1 )
		{
			++fails;
		}

		rval = acosh(VSTH_ACOSH_VAL2);
		if ( rval != Vsth_acosh_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
acosh_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for advance()					 ******/

#ifdef	advance
#undef	advance
#endif

int
advance_sf()
{
	return(0);
}

int
advance_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
advance_ef()
{
	return(0);
}

#endif
#endif

#if	defined(_POSIX_ASYNCHRONOUS_IO)

/****** Testset for aio_cancel()				 ******/

#ifdef	aio_cancel
#undef	aio_cancel
#endif

int
aio_cancel_sf()
{
	return(0);
}

int
aio_cancel_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
aio_cancel_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_ASYNCHRONOUS_IO)

/****** Testset for aio_error()					 ******/

#ifdef	aio_error
#undef	aio_error
#endif

int
aio_error_sf()
{
	return(0);
}

int
aio_error_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
aio_error_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_ASYNCHRONOUS_IO) && defined(_POSIX_SYNCHRONIZED_IO)

/****** Testset for aio_fsync()					 ******/

#ifdef	aio_fsync
#undef	aio_fsync
#endif

int
aio_fsync_sf()
{
	return(0);
}

int
aio_fsync_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
aio_fsync_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_ASYNCHRONOUS_IO)

/****** Testset for aio_read()					 ******/

#ifdef	aio_read
#undef	aio_read
#endif

int
aio_read_sf()
{
	return(0);
}

int
aio_read_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
aio_read_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_ASYNCHRONOUS_IO)

/****** Testset for aio_return()				 ******/

#ifdef	aio_return
#undef	aio_return
#endif

int
aio_return_sf()
{
	return(0);
}

int
aio_return_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
aio_return_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_ASYNCHRONOUS_IO)

/****** Testset for aio_suspend()				 ******/

#ifdef	aio_suspend
#undef	aio_suspend
#endif

int
aio_suspend_sf()
{
	return(0);
}

int
aio_suspend_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
aio_suspend_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_ASYNCHRONOUS_IO)

/****** Testset for aio_write()					 ******/

#ifdef	aio_write
#undef	aio_write
#endif

int
aio_write_sf()
{
	return(0);
}

int
aio_write_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
aio_write_ef()
{
	return(0);
}

#endif

/****** Testset for alarm()					 ******/

#ifdef	alarm
#undef	alarm
#endif

int
alarm_sf()
{
	return(0);
}

int
alarm_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
alarm_ef()
{
	return(0);
}

/****** Testset for asctime()					 ******/

#ifdef	asctime
#undef	asctime
#endif

int
asctime_sf()
{
	return(0);
}

int
asctime_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
asctime_ef()
{
	return(0);
}

/****** Testset for asin()					 ******/

#ifdef	asin
#undef	asin
#endif

#define	VSTH_ASIN_VAL1		-0.2
#define	VSTH_ASIN_VAL2		0.5
int	Vsth_asin_init = 0;
double	Vsth_asin_val1;
double	Vsth_asin_val2;

int
asin_sf()
{
	if ( ! Vsth_asin_init )
	{
		Vsth_asin_val1 = asin(VSTH_ASIN_VAL1);
		Vsth_asin_val2 = asin(VSTH_ASIN_VAL2);
		Vsth_asin_init = 1;
	}
		
	return(0);
}

int
asin_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = asin(VSTH_ASIN_VAL1);
		if ( rval != Vsth_asin_val1 )
		{
			++fails;
		}

		rval = asin(VSTH_ASIN_VAL2);
		if ( rval != Vsth_asin_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
asin_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for asinh()					 ******/

#ifdef	asinh
#undef	asinh
#endif

#define	VSTH_ASINH_VAL1		-0.2
#define	VSTH_ASINH_VAL2		0.5
int	Vsth_asinh_init = 0;
double	Vsth_asinh_val1;
double	Vsth_asinh_val2;

int
asinh_sf()
{
	if ( ! Vsth_asinh_init )
	{
		Vsth_asinh_val1 = asinh(VSTH_ASINH_VAL1);
		Vsth_asinh_val2 = asinh(VSTH_ASINH_VAL2);
		Vsth_asinh_init = 1;
	}
		
	return(0);
}

int
asinh_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = asinh(VSTH_ASINH_VAL1);
		if ( rval != Vsth_asinh_val1 )
		{
			++fails;
		}

		rval = asinh(VSTH_ASINH_VAL2);
		if ( rval != Vsth_asinh_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
asinh_ef()
{
	return(0);
}

#endif

/****** Testset for assert()					 ******/

#ifdef	assert
#undef	assert
#endif

int
assert_sf()
{
	return(0);
}

int
assert_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
assert_ef()
{
	return(0);
}

/****** Testset for atan()					 ******/

#ifdef	atan
#undef	atan
#endif

#define	VSTH_ATAN_VAL1		-0.2
#define	VSTH_ATAN_VAL2		0.5
int	Vsth_atan_init = 0;
double	Vsth_atan_val1;
double	Vsth_atan_val2;

int
atan_sf()
{
	if ( ! Vsth_atan_init )
	{
		Vsth_atan_val1 = atan(VSTH_ATAN_VAL1);
		Vsth_atan_val2 = atan(VSTH_ATAN_VAL2);
		Vsth_atan_init = 1;
	}
		
	return(0);
}

int
atan_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = atan(VSTH_ATAN_VAL1);
		if ( rval != Vsth_atan_val1 )
		{
			++fails;
		}

		rval = atan(VSTH_ATAN_VAL2);
		if ( rval != Vsth_atan_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
atan_ef()
{
	return(0);
}

/****** Testset for atan2()					 ******/

#ifdef	atan2
#undef	atan2
#endif

#define	VSTH_ATAN2_VAL1		-0.2
#define	VSTH_ATAN2_VAL2		0.5
int	Vsth_atan2_init = 0;
double	Vsth_atan2_val1;
double	Vsth_atan2_val2;

int
atan2_sf()
{
	if ( ! Vsth_atan2_init )
	{
		Vsth_atan2_val1 = atan2(VSTH_ATAN2_VAL1, VSTH_ATAN2_VAL2);
		Vsth_atan2_val2 = atan2(VSTH_ATAN2_VAL2, VSTH_ATAN2_VAL1);
		Vsth_atan2_init = 1;
	}
		
	return(0);
}

int
atan2_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = atan2(VSTH_ATAN2_VAL1, VSTH_ATAN2_VAL2);
		if ( rval != Vsth_atan2_val1 )
		{
			++fails;
		}

		rval = atan2(VSTH_ATAN2_VAL2, VSTH_ATAN2_VAL1);
		if ( rval != Vsth_atan2_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
atan2_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for atanh()					 ******/

#ifdef	atanh
#undef	atanh
#endif

#define	VSTH_ATANH_VAL1		-0.2
#define	VSTH_ATANH_VAL2		0.5
int	Vsth_atanh_init = 0;
double	Vsth_atanh_val1;
double	Vsth_atanh_val2;

int
atanh_sf()
{
	if ( ! Vsth_atanh_init )
	{
		Vsth_atanh_val1 = atanh(VSTH_ATANH_VAL1);
		Vsth_atanh_val2 = atanh(VSTH_ATANH_VAL2);
		Vsth_atanh_init = 1;
	}
		
	return(0);
}

int
atanh_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = atanh(VSTH_ATANH_VAL1);
		if ( rval != Vsth_atanh_val1 )
		{
			++fails;
		}

		rval = atanh(VSTH_ATANH_VAL2);
		if ( rval != Vsth_atanh_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
atanh_ef()
{
	return(0);
}

#endif

/****** Testset for atexit()					 ******/

#ifdef	atexit
#undef	atexit
#endif

int
atexit_sf()
{
	return(0);
}

int
atexit_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
atexit_ef()
{
	return(0);
}

/****** Testset for atof()					 ******/

#ifdef	atof
#undef	atof
#endif

#define	VSTH_ATOF_S1	"-1.234"
#define	VSTH_ATOF_S2	"789.012"

int	Vsth_atof_init = 0;
double	Vsth_atof_val1;
double	Vsth_atof_val2;

int
atof_sf()
{
	if ( ! Vsth_atof_init )
	{
		Vsth_atof_val1 = atof(VSTH_ATOF_S1);
		Vsth_atof_val2 = atof(VSTH_ATOF_S2);
		Vsth_atof_init = 1;
	}
	return(0);
}

int
atof_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	double	val;

	errs	= 0;
	fails	= 0;

	for( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		val = atof(VSTH_ATOF_S1);
		if ( val != Vsth_atof_val1 )
		{
			++fails;
		}

		val = atof(VSTH_ATOF_S2);
		if ( val != Vsth_atof_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
atof_ef()
{
	return(0);
}

/****** Testset for atoi()					 ******/

#ifdef	atoi
#undef	atoi
#endif

#define	VSTH_ATOI_S1	"12345"
#define	VSTH_ATOI_S2	"-54321"

int	Vsth_atoi_init = 0;
int	Vsth_atoi_val1;
int	Vsth_atoi_val2;

int
atoi_sf()
{
	if ( ! Vsth_atoi_init )
	{
		Vsth_atoi_val1 = atol(VSTH_ATOI_S1);
		Vsth_atoi_val2 = atol(VSTH_ATOI_S2);
		Vsth_atoi_init = 1;
	}
	return(0);
}

int
atoi_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	val;

	errs	= 0;
	fails	= 0;

	for( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		val = atoi(VSTH_ATOI_S1);
		if ( val != Vsth_atoi_val1 )
		{
			++fails;
		}

		val = atoi(VSTH_ATOI_S2);
		if ( val != Vsth_atoi_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
atoi_ef()
{
	return(0);
}

/****** Testset for atol()					 ******/

#ifdef	atol
#undef	atol
#endif

#define	VSTH_ATOL_S1	"12345"
#define	VSTH_ATOL_S2	"-54321"

int	Vsth_atol_init = 0;
long	Vsth_atol_val1;
long	Vsth_atol_val2;

int
atol_sf()
{
	if ( ! Vsth_atol_init )
	{
		Vsth_atol_val1 = atol(VSTH_ATOL_S1);
		Vsth_atol_val2 = atol(VSTH_ATOL_S2);
		Vsth_atol_init = 1;
	}
	return(0);
}

int
atol_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	long	val;

	errs	= 0;
	fails	= 0;

	for( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		val = atol(VSTH_ATOL_S1);
		if ( val != Vsth_atol_val1 )
		{
			++fails;
		}

		val = atol(VSTH_ATOL_S2);
		if ( val != Vsth_atol_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
atol_ef()
{
	return(0);
}

