
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_rtc.c	1.4
*
*************************************************************************/

#include "std.h"

#include <dirent.h>
#include <fcntl.h>
#include <math.h>

#if	( TEST_XPG > 4 )
#include <nl_types.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>

#if	( TEST_XPG > 4 )
#include <syslog.h>
#endif

#include <termios.h>
#include <time.h>
#include <unistd.h>

#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"

/****** Testset for calloc()					 ******/

#ifdef	calloc
#undef	calloc
#endif

int
calloc_sf()
{
	return(0);
}

int
calloc_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
calloc_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for catclose()					 ******/

#ifdef	catclose
#undef	catclose
#endif

int
catclose_sf()
{
	return(0);
}

int
catclose_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
catclose_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for catgets()					 ******/

#ifdef	catgets
#undef	catgets
#endif

int
catgets_sf()
{
	return(0);
}

int
catgets_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
catgets_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for catopen()					 ******/

#ifdef	catopen
#undef	catopen
#endif

int
catopen_sf()
{
	return(0);
}

int
catopen_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
catopen_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for cbrt()					 ******/

#ifdef	cbrt
#undef	cbrt
#endif

#define	VSTH_CBRT_VAL1		-2.2
#define	VSTH_CBRT_VAL2		5.5
int	Vsth_cbrt_init = 0;
double	Vsth_cbrt_val1;
double	Vsth_cbrt_val2;

int
cbrt_sf()
{
	if ( ! Vsth_cbrt_init )
	{
		Vsth_cbrt_val1 = cbrt(VSTH_CBRT_VAL1);
		Vsth_cbrt_val2 = cbrt(VSTH_CBRT_VAL2);
		Vsth_cbrt_init = 1;
	}
		
	return(0);
}

int
cbrt_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = cbrt(VSTH_CBRT_VAL1);
		if ( rval != Vsth_cbrt_val1 )
		{
			++fails;
		}

		rval = cbrt(VSTH_CBRT_VAL2);
		if ( rval != Vsth_cbrt_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
cbrt_ef()
{
	return(0);
}

#endif

/****** Testset for ceil()					 ******/

#ifdef	ceil
#undef	ceil
#endif

#define	VSTH_CEIL_VAL1		-2.2
#define	VSTH_CEIL_VAL2		5.5
int	Vsth_ceil_init = 0;
double	Vsth_ceil_val1;
double	Vsth_ceil_val2;

int
ceil_sf()
{
	if ( ! Vsth_ceil_init )
	{
		Vsth_ceil_val1 = ceil(VSTH_CEIL_VAL1);
		Vsth_ceil_val2 = ceil(VSTH_CEIL_VAL2);
		Vsth_ceil_init = 1;
	}
		
	return(0);
}

int
ceil_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = ceil(VSTH_CEIL_VAL1);
		if ( rval != Vsth_ceil_val1 )
		{
			++fails;
		}

		rval = ceil(VSTH_CEIL_VAL2);
		if ( rval != Vsth_ceil_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ceil_ef()
{
	return(0);
}

/****** Testset for cfgetispeed()				 ******/

#ifdef	cfgetispeed
#undef	cfgetispeed
#endif

int
cfgetispeed_sf()
{
	return(0);
}

int
cfgetispeed_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
cfgetispeed_ef()
{
	return(0);
}

/****** Testset for cfgetospeed()				 ******/

#ifdef	cfgetospeed
#undef	cfgetospeed
#endif

int
cfgetospeed_sf()
{
	return(0);
}

int
cfgetospeed_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
cfgetospeed_ef()
{
	return(0);
}

/****** Testset for cfsetispeed()				 ******/

#ifdef	cfsetispeed
#undef	cfsetispeed
#endif

int
cfsetispeed_sf()
{
	return(0);
}

int
cfsetispeed_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
cfsetispeed_ef()
{
	return(0);
}

/****** Testset for cfsetospeed()				 ******/

#ifdef	cfsetospeed
#undef	cfsetospeed
#endif

int
cfsetospeed_sf()
{
	return(0);
}

int
cfsetospeed_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
cfsetospeed_ef()
{
	return(0);
}

/****** Testset for chdir()					 ******/

#ifdef	chdir
#undef	chdir
#endif

int
chdir_sf()
{
	return(0);
}

int
chdir_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
chdir_ef()
{
	return(0);
}

/****** Testset for chmod()					 ******/

#ifdef	chmod
#undef	chmod
#endif

int
chmod_sf()
{
	return(0);
}

int
chmod_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
chmod_ef()
{
	return(0);
}

/****** Testset for chown()					 ******/

#ifdef	chown
#undef	chown
#endif

int
chown_sf()
{
	return(0);
}

int
chown_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
chown_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for chroot()					 ******/

#ifdef	chroot
#undef	chroot
#endif

int
chroot_sf()
{
	return(0);
}

int
chroot_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
chroot_ef()
{
	return(0);
}

#endif
#endif

/****** Testset for clearerr()					 ******/

#ifdef	clearerr
#undef	clearerr
#endif

int
clearerr_sf()
{
	return(0);
}

int
clearerr_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
clearerr_ef()
{
	return(0);
}

/****** Testset for clock()					 ******/

#ifdef	clock
#undef	clock
#endif

int
clock_sf()
{
	return(0);
}

int
clock_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
clock_ef()
{
	return(0);
}

#if	defined(_POSIX_TIMERS)

/****** Testset for clock_getres()				 ******/

#ifdef	clock_getres
#undef	clock_getres
#endif

int
clock_getres_sf()
{
	return(0);
}

int
clock_getres_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
clock_getres_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_TIMERS)

/****** Testset for clock_gettime()				 ******/

#ifdef	clock_gettime
#undef	clock_gettime
#endif

int
clock_gettime_sf()
{
	return(0);
}

int
clock_gettime_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
clock_gettime_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_TIMERS)

/****** Testset for clock_settime()				 ******/

#ifdef	clock_settime
#undef	clock_settime
#endif

int
clock_settime_sf()
{
	return(0);
}

int
clock_settime_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
clock_settime_ef()
{
	return(0);
}

#endif

/****** Testset for close()					 ******/

#ifdef	close
#undef	close
#endif

int
close_sf()
{
	return(0);
}

int
close_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
close_ef()
{
	return(0);
}

/****** Testset for closedir()					 ******/

#ifdef	closedir
#undef	closedir
#endif

int
closedir_sf()
{
	return(0);
}

int
closedir_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
closedir_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for closelog()					 ******/

#ifdef	closelog
#undef	closelog
#endif

int
closelog_sf()
{
	return(0);
}

int
closelog_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
closelog_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for compile()					 ******/

#ifdef	compile
#undef	compile
#endif

int
compile_sf()
{
	return(0);
}

int
compile_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
compile_ef()
{
	return(0);
}

#endif
#endif

#if	( TEST_XPG > 4 )

/****** Testset for confstr()					 ******/

#ifdef	confstr
#undef	confstr
#endif

int
confstr_sf()
{
	return(0);
}

int
confstr_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
confstr_ef()
{
	return(0);
}

#endif

/****** Testset for cos()					 ******/

#ifdef	cos
#undef	cos
#endif

#define	VSTH_COS_VAL1		-0.2
#define	VSTH_COS_VAL2		0.5
int	Vsth_cos_init = 0;
double	Vsth_cos_val1;
double	Vsth_cos_val2;

int
cos_sf()
{
	if ( ! Vsth_cos_init )
	{
		Vsth_cos_val1 = cos(VSTH_COS_VAL1);
		Vsth_cos_val2 = cos(VSTH_COS_VAL2);
		Vsth_cos_init = 1;
	}
		
	return(0);
}

int
cos_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = cos(VSTH_COS_VAL1);
		if ( rval != Vsth_cos_val1 )
		{
			++fails;
		}

		rval = cos(VSTH_COS_VAL2);
		if ( rval != Vsth_cos_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
cos_ef()
{
	return(0);
}

/****** Testset for cosh()					 ******/

#ifdef	cosh
#undef	cosh
#endif

#define	VSTH_COSH_VAL1		-0.2
#define	VSTH_COSH_VAL2		0.5
int	Vsth_cosh_init = 0;
double	Vsth_cosh_val1;
double	Vsth_cosh_val2;

int
cosh_sf()
{
	if ( ! Vsth_cosh_init )
	{
		Vsth_cosh_val1 = cosh(VSTH_COSH_VAL1);
		Vsth_cosh_val2 = cosh(VSTH_COSH_VAL2);
		Vsth_cosh_init = 1;
	}
		
	return(0);
}

int
cosh_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = cosh(VSTH_COSH_VAL1);
		if ( rval != Vsth_cosh_val1 )
		{
			++fails;
		}

		rval = cosh(VSTH_COSH_VAL2);
		if ( rval != Vsth_cosh_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
cosh_ef()
{
	return(0);
}

/****** Testset for creat()					 ******/

#ifdef	creat
#undef	creat
#endif

int
creat_sf()
{
	return(0);
}

int
creat_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
creat_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_CRYPT) && (_XOPEN_CRYPT != -1)

/****** Testset for crypt()					 ******/

#ifdef	crypt
#undef	crypt
#endif

int
crypt_sf()
{
	return(0);
}

int
crypt_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
crypt_ef()
{
	return(0);
}

#endif
#endif

/****** Testset for ctermid()					 ******/

#ifdef	ctermid
#undef	ctermid
#endif

int
ctermid_sf()
{
	return(0);
}

int
ctermid_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ctermid_ef()
{
	return(0);
}

/****** Testset for ctime()					 ******/

#ifdef	ctime
#undef	ctime
#endif

int
ctime_sf()
{
	return(0);
}

int
ctime_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ctime_ef()
{
	return(0);
}

#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for ctime_r()					 ******/

#ifdef	ctime_r
#undef	ctime_r
#endif

int
ctime_r_sf()
{
	return(0);
}

int
ctime_r_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* thread safety is tested in PTHR.os/all/ctime_r */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ctime_r_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for cuserid()					 ******/

#ifdef	cuserid
#undef	cuserid
#endif

int
cuserid_sf()
{
	return(0);
}

int
cuserid_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
cuserid_ef()
{
	return(0);
}

#endif
#endif
