
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_rtf.c	1.23
*
*************************************************************************/

#include "std.h"

#include <errno.h>
#include <fcntl.h>

#if	( TEST_XPG > 4 )
#include <fmtmsg.h>
#include <fnmatch.h>
#include <ftw.h>
#endif

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#if	( TEST_XPG > 4 )
#include <strings.h>
#include <stropts.h>
#include <sys/ipc.h>
#endif

#include <sys/stat.h>

#if	( TEST_XPG > 4 )
#include <sys/statvfs.h>
#include <sys/timeb.h>
#endif

#include <sys/types.h>
#include <unistd.h>

#if	( TEST_XPG > 4 )
#include <wchar.h>
#endif

#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"

/****** Testset for fabs()					 ******/

#ifdef	fabs
#undef	fabs
#endif

#define	VSTH_FABS_VAL1		-2.2
#define	VSTH_FABS_VAL2		5.5
int	Vsth_fabs_init = 0;
double	Vsth_fabs_val1;
double	Vsth_fabs_val2;

int
fabs_sf()
{
	if ( ! Vsth_fabs_init )
	{
		Vsth_fabs_val1 = fabs(VSTH_FABS_VAL1);
		Vsth_fabs_val2 = fabs(VSTH_FABS_VAL2);
		Vsth_fabs_init = 1;
	}
		
	return(0);
}

int
fabs_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = fabs(VSTH_FABS_VAL1);
		if ( rval != Vsth_fabs_val1 )
		{
			++errs;
		}

		rval = fabs(VSTH_FABS_VAL2);
		if ( rval != Vsth_fabs_val2 )
		{
			++errs;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fabs_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for fattach()					 ******/

#ifdef	fattach
#undef	fattach
#endif

int
fattach_sf()
{
	return(0);
}

int
fattach_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fattach_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for fchdir()					 ******/

#ifdef	fchdir
#undef	fchdir
#endif

int
fchdir_sf()
{
	return(0);
}

int
fchdir_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fchdir_ef()
{
	return(0);
}

#endif

/****** Testset for fchmod()					 ******/

#ifdef	fchmod
#undef	fchmod
#endif

int
fchmod_sf()
{
	return(0);
}

int
fchmod_tf()
{
	int		cnt;
	int		rval;
	int		errs;
	int		fails;
	int		fd;
	FILE		*fp;
	struct stat	sb;

	errs	= 0;
	fails	= 0;

	vsth_ec();
	fp = tmpfile();
	vsth_xc();

	if ( fp != NULL )
	{
		vsth_ec();
		fd = fileno(fp);
		vsth_xc();

		for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
		{
			rval = fchmod(fd, S_IRWXU | S_IRWXG );
			if ( rval == 0 )
			{
				vsth_ec();
				rval = fstat(fd, &sb);
				vsth_xc();
	
				if ( rval == 0 )
				{
					if ((sb.st_mode & S_IRWXG) == 0)
					{
						++fails;
					}
				}
				else
				{
					++errs;
				}

			}

			rval = fchmod(fd, S_IRWXU);
			if ( rval == 0 )
			{
				vsth_ec();
				rval = fstat(fd, &sb);
				vsth_xc();
	
				if ( rval == 0 )
				{
					if (sb.st_mode & S_IRWXG)
					{
						++fails;
					}
				}
				else
				{
					++errs;
				}

			}
		}

		vsth_ec();
		(void) fclose(fp);
		vsth_xc();
	}
	else
	{
		++errs;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fchmod_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for fchown()					 ******/

#ifdef	fchown
#undef	fchown
#endif

int	Vsth_fchown_init = 0;
uid_t	Vsth_fchown_uid1 = (uid_t) 0;
gid_t	Vsth_fchown_gid1 = (gid_t) 0;
gid_t	Vsth_fchown_gid2 = (gid_t) 0;

int
fchown_sf()
{
	int	errs;
	int	rval;
	char	*p;

	errs = 0;

	if ( ! Vsth_fchown_init )
	{
		p = tet_getvar("VSX_UID0");
		if (( p != NULL) && (*p != '\0'))
		{
			Vsth_fchown_uid1 = (uid_t) atoi(p);
		}
		else
		{
			++errs;
		}

		p = tet_getvar("VSX_GID0");
		if (( p != NULL) && (*p != '\0'))
		{
			Vsth_fchown_gid1 = (gid_t) atoi(p);
		}
		else
		{
			++errs;
		}

		/* some systems may not support supplementary gids */
		p = tet_getvar("VSTH_SUPP_GID");
		if (( p != NULL) && (*p != '\0'))
		{
			Vsth_fchown_gid2 = (gid_t) atoi(p);
		}

		Vsth_fchown_init = 1;
	}

	rval = 1;
	if ( errs == 0 )
	{
		rval = 0;
	}
	return(0);
}

int
fchown_tf()
{
	int		cnt;
	int		rval;
	int		errs;
	int		fails;
	int		fd;
	FILE		*fp;
	struct stat	sb;

	errs	= 0;
	fails	= 0;

	if ( Vsth_fchown_gid2 != (gid_t) 0 )
	{
		vsth_ec();
		fp = tmpfile();
		vsth_xc();
	
		if ( fp != NULL )
		{
			vsth_ec();
			fd = fileno(fp);
			rval = fchmod(fd, S_IRWXU | S_IRWXG | S_IRWXO);
			vsth_xc();
	
			if ( rval != 0 )
			{
				++errs;
			}
	
			for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
			{
				rval = fchown(fd, Vsth_fchown_uid1, Vsth_fchown_gid1);
				if ( rval == 0 )
				{
					vsth_ec();
					rval = fstat(fd, &sb);
					vsth_xc();
		
					if ( rval == 0 )
					{
						if (sb.st_gid != Vsth_fchown_gid1)
						{
							++fails;
						}
					}
					else
					{
						++errs;
					}
				}
				else
				{
					++errs;
				}
	
				rval = fchown(fd, Vsth_fchown_uid1, Vsth_fchown_gid2);
				if ( rval == 0 )
				{
					vsth_ec();
					rval = fstat(fd, &sb);
					vsth_xc();
		
					if ( rval == 0 )
					{
						if (sb.st_gid != Vsth_fchown_gid2)
						{
							++fails;
						}
					}
					else
					{
						++errs;
					}
				}
				else
				{
					++errs;
				}
			}
	
			vsth_ec();
			(void) fclose(fp);
			vsth_xc();
		}
		else
		{
			++errs;
		}
	}
	else
	{
		tet_infoline("fchown is untested. Supplementary gids not supported");
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fchown_ef()
{
	return(0);
}

#endif

/****** Testset for fclose()					 ******/

#ifdef	fclose
#undef	fclose
#endif

int
fclose_sf()
{
	return(0);
}

int
fclose_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	FILE	*fp;
	int	fd;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		vsth_ec();
		fp = tmpfile();
		vsth_xc();

		if ( fp != NULL )
		{
			rval = fclose(fp);
			if ( rval != 0 )
			{
				++fails;
			}
		}
		else
		{
			++errs;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fclose_ef()
{
	return(0);
}

/****** Testset for fcntl()					 ******/

#ifdef	fcntl
#undef	fcntl
#endif

int
fcntl_sf()
{
	return(0);
}

int
fcntl_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	flags;
	int	fd;
	FILE	*fp;

	errs	= 0;
	fails	= 0;

	vsth_ec();
	fp = tmpfile();
	vsth_xc();
	if ( fp )
	{
		vsth_ec();
		fd = fileno(fp);
		vsth_xc();

		for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
		{
			rval = fcntl(fd, F_SETFD, FD_CLOEXEC);
			if ( rval != 0 )
			{
				++fails;
			}

			rval = fcntl(fd, F_GETFD);
			if ((rval & FD_CLOEXEC) == 0)
			{
				++fails;
			}

			rval = fcntl(fd, F_SETFD, 0);
			if ( rval != 0 )
			{
				++fails;
			}

			rval = fcntl(fd, F_GETFD, &flags);
			if (rval & FD_CLOEXEC)
			{
				++fails;
			}
		}

		vsth_ec();
		(void) fclose(fp);
		vsth_xc();
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fcntl_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for fcvt()					 ******/

#ifdef	fcvt
#undef	fcvt
#endif

int
fcvt_sf()
{
	return(0);
}

int
fcvt_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fcvt_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SYNCHRONIZED_IO)

/****** Testset for fdatasync()					 ******/

#ifdef	fdatasync
#undef	fdatasync
#endif

int
fdatasync_sf()
{
	return(0);
}

int
fdatasync_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fdatasync_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for fdetach()					 ******/

#ifdef	fdetach
#undef	fdetach
#endif

int
fdetach_sf()
{
	return(0);
}

int
fdetach_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fdetach_ef()
{
	return(0);
}

#endif

/****** Testset for fdopen()					 ******/

#ifdef	fdopen
#undef	fdopen
#endif

int
fdopen_sf()
{

	return(0);
}

int
fdopen_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	buf[1];
	int	fd1;
	int	fd2;
	off_t	offset;
	char	name1[L_tmpnam + 1];
	char	name2[L_tmpnam + 1];
	char	*p;
	FILE	*fp1;
	FILE	*fp2;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		vsth_ec();

		p = tmpnam( name1 );
		if ( ! p )
		{
			++errs;
			vsth_xc();
			break;
		}

		fd1 = open( name1, O_RDWR | O_CREAT | O_TRUNC, S_IRWXU );
		if ( fd1 == -1 )
		{
			++errs;
			vsth_xc();
			break;
		}

		buf[0] = '1';

		rval = write( fd1, (void *) buf, 1 );
		if ( rval != 1 )
		{
			++errs;
			vsth_xc();
			break;
		}

		offset = lseek( fd1, 0, SEEK_SET );
		if (  offset != 0 )
		{
			++errs;
			vsth_xc();
			break;
		}

		p = tmpnam( name2 );
		if ( ! p )
		{
			++errs;
			vsth_xc();
			break;
		}

		fd2 = open( name2, O_RDWR | O_CREAT | O_TRUNC, S_IRWXU );
		if ( fd2 == -1 )
		{
			++errs;
			vsth_xc();
			break;
		}

		buf[0] = '2';

		rval = write( fd2, (void *) buf, 1 );
		if ( rval != 1 )
		{
			++errs;
			vsth_xc();
			break;
		}

		offset = lseek( fd2, 0, SEEK_SET );
		if (  offset != 0 )
		{
			++errs;
			vsth_xc();
			break;
		}

		vsth_xc();

		fp1 = fdopen(fd1, "r");
		if ( ! fp1 )
		{
			++fails;
			break;
		}

		fp2 = fdopen(fd2, "r");
		if ( ! fp2 )
		{
			++fails;
			break;
		}

		vsth_ec();

		rval = fgetc(fp1);
		if ( rval != '1' )
		{
			++fails;
		}

		rval = fgetc(fp2);
		if ( rval != '2' )
		{
			++fails;
		}

		if ( fp1 )
			(void) fclose(fp1);
		if ( fp2 )
			(void) fclose(fp2);

		(void) remove( name1 );
		(void) remove( name2 );

		vsth_xc();
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fdopen_ef()
{
	return(0);
}

/****** Testset for feof()					 ******/

#ifdef	feof
#undef	feof
#endif

#define	VSTH_FEOF_OFFSET	1

int
feof_sf()
{
	return(0);
}

int
feof_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	FILE	*fp;

	errs	= 0;
	fails	= 0;

	vsth_ec();
	fp = tmpfile();
	vsth_xc();

	if ( fp != NULL )
	{
		vsth_ec();
		rval = fputc('X', fp);
		vsth_xc();

		if ( rval == (int) 'X' )
		{
			for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
			{
				vsth_ec();
				rval = fseek(fp, 0, SEEK_SET);
				(void) fgetc(fp);
				vsth_xc();
	
				if ( rval != 0 )
				{
					++errs;
				}
	
				rval = feof(fp);
				if ( rval )
				{
					++fails;
				}
	
				vsth_ec();
				rval = fseek(fp, VSTH_FEOF_OFFSET, SEEK_SET);
				(void) fgetc(fp);
				vsth_xc();
	
				if ( rval != 0 )
				{
					++errs;
				}
	
				rval = feof(fp);
				if ( rval == 0 )
				{
					++fails;
				}
			}
		}
		else
		{
			++errs;
		}

		(void) fclose(fp);
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
feof_ef()
{
	return(0);
}

/****** Testset for ferror()					 ******/

#ifdef	ferror
#undef	ferror
#endif

int
ferror_sf()
{
	return(0);
}

int
ferror_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	fds[2];
	FILE	*fp;

	errs	= 0;
	fails	= 0;

	vsth_ec();
	rval = pipe(fds);
	vsth_xc();

	if ( rval == 0 )
	{
		vsth_ec();
		rval = fcntl( fds[0], F_SETFL, O_NONBLOCK );
		vsth_xc();

		if ( rval == 0 )
		{

			vsth_ec();
			fp = fdopen( fds[0], "r" );
			vsth_xc();
	
			if ( fp != NULL )
			{
				vsth_ec();
				(void) setbuf(fp, NULL);
				vsth_xc();

				if ( rval == 0 )
				{
					for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
					{
	
						vsth_ec();
						/* read on empty pipe with O_NONBLOCK sets error indicator */
						rval = fgetc(fp);
						vsth_xc();
	
						if (( rval != EOF ) || ( errno != EAGAIN ))
						{
							++errs;
						}
	
						rval = ferror(fp);
						if ( rval == 0 )
						{
							++fails;
						}
	
						vsth_ec();
						clearerr(fp);
						vsth_xc();
	
						rval = ferror(fp);
						if ( rval )
						{
							++fails;
						}
					}
				}
				else
				{
					++errs;
				}
			}
			else
			{
				++errs;
			}
		}
		else
		{
			++errs;
		}

		(void) close(fds[0]);
		(void) close(fds[1]);
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ferror_ef()
{
	return(0);
}

/****** Testset for fflush()					 ******/

#ifdef	fflush
#undef	fflush
#endif

int
fflush_sf()
{
	return(0);
}

int
fflush_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	fd;
	int	c;
	FILE	*fp;

	errs	= 0;
	fails	= 0;
	c	= 'X';

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		vsth_ec();
		fp = tmpfile();
		vsth_xc();

		if ( fp != NULL )
		{
			vsth_ec();
			(void) fputc(c, fp);
			vsth_xc();

			rval = fflush(fp);
			if ( rval != 0 )
			{
				++fails;
			}

			vsth_ec();
			(void) fclose(fp);
			vsth_xc();
		}
		else
		{
			++errs;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fflush_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for ffs()					 ******/

#ifdef	ffs
#undef	ffs
#endif

int
ffs_sf()
{
	return(0);
}

int
ffs_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ffs_ef()
{
	return(0);
}

#endif

/****** Testset for fgetc()					 ******/

#ifdef	fgetc
#undef	fgetc
#endif

int
fgetc_sf()
{
	return(0);
}

int
fgetc_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	FILE	*fp;

	errs	= 0;
	fails	= 0;

	vsth_ec();
	fp = tmpfile();
	vsth_xc();

	if ( fp != NULL )
	{
		vsth_ec();
		rval = fputc('X', fp);
		vsth_xc();

		if ( rval != 'X' )
		{
			++errs;
		}

		for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
		{
			if ( ! errs )
			{
				vsth_ec();
				rval = fseek(fp, 0, SEEK_SET);
				vsth_xc();
	
				if ( rval != 0 )
				{
					++errs;
				}
		
				rval = fgetc(fp);
				if ( rval != 'X' )
				{
					++fails;
				}
		
				rval = fgetc(fp);
				if ( rval != EOF )
				{
					++fails;
				}
			}
		}

		vsth_ec();
		(void) fclose(fp);
		vsth_xc();
	}
	else
	{
		++errs;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fgetc_ef()
{
	return(0);
}

/****** Testset for fgetpos()					 ******/

#ifdef	fgetpos
#undef	fgetpos
#endif

#define	VSTH_FGETPOS_POS1	23
#define	VSTH_FGETPOS_POS2	11

int
fgetpos_sf()
{
	return(0);
}

int
fgetpos_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	FILE	*fp;
	fpos_t	pos;

	errs	= 0;
	fails	= 0;

	vsth_ec();
	fp = tmpfile();
	vsth_xc();

	if ( fp != NULL )
	{
		for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
		{
			vsth_ec();
			rval = lseek(fileno(fp), VSTH_FGETPOS_POS1, SEEK_SET);
			vsth_xc();
	
			if ( rval != VSTH_FGETPOS_POS1 )
			{
				++errs;
			}
	
			rval = fgetpos(fp, &pos);
			if ( rval != 0 )
			{
				++fails;
			}
	
			vsth_ec();
			rval = fsetpos(fp, &pos);
			vsth_xc();
	
			if ( rval != 0 )
			{
				++errs;
			}
	
			vsth_ec();
			rval = ftell(fp);
			vsth_xc();
	
			if ( rval != VSTH_FGETPOS_POS1 )
			{
				++fails;
			}
	
			vsth_ec();
			rval = lseek(fileno(fp), VSTH_FGETPOS_POS2, SEEK_SET);
			vsth_xc();
	
			if ( rval != VSTH_FGETPOS_POS2 )
			{
				++errs;
			}
	
			rval = fgetpos(fp, &pos);
			if ( rval != 0 )
			{
				++fails;
			}
	
			vsth_ec();
			rval = fsetpos(fp, &pos);
			vsth_xc();
	
			if ( rval != 0 )
			{
				++errs;
			}
	
			vsth_ec();
			rval = ftell(fp);
			vsth_xc();
	
			if ( rval != VSTH_FGETPOS_POS2 )
			{
				++fails;
			}
		}

		vsth_ec();
		(void) fclose(fp);
		vsth_xc();
	}
	else
	{
		++errs;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fgetpos_ef()
{
	return(0);
}

/****** Testset for fgets()					 ******/

#ifdef	fgets
#undef	fgets
#endif

#define	VSTH_FGETS_S1	"0981234nadsf98769876KjADf798\n"
#define	VSTH_FGETS_S2	"09()*&ytr45#%$#@ghfx\n"

int
fgets_sf()
{
	return(0);
}

int
fgets_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	buf[VSTH_BUFLEN];
	char	*val;
	FILE	*fp;

	errs	= 0;
	fails	= 0;

	vsth_ec();
	fp = tmpfile();
	vsth_xc();

	if ( fp != NULL )
	{
		vsth_ec();
		rval = fputs(VSTH_FGETS_S1, fp);
		vsth_xc();
		if ( rval == EOF )
		{
			++errs;
		}

		vsth_ec();
		rval = fputs(VSTH_FGETS_S2, fp);
		vsth_xc();

		if ( rval == EOF )
		{
			++errs;
		}

		for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
		{
			vsth_ec();
			rval = fseek(fp, 0, SEEK_SET);
			vsth_xc();

			if ( rval != 0 )
			{
				++errs;
			}

			val = fgets(&buf[0], VSTH_BUFLEN - 1, fp);
			if (val != &buf[0])
			{
				++fails;
			}

			vsth_ec();
			rval =  strcmp(&buf[0], VSTH_FGETS_S1);
			vsth_xc();

			if ( rval )
			{
				++fails;
			}

			val = fgets(&buf[0], VSTH_BUFLEN - 1, fp);
			if (val != &buf[0])
			{
				++fails;
			}

			if (strcmp(&buf[0], VSTH_FGETS_S2))
			{
				++fails;
			}
		}

		vsth_ec();
		(void) fclose(fp);
		vsth_xc();
	}
	else
	{
		++errs;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fgets_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for fgetwc()					 ******/

#ifdef	fgetwc
#undef	fgetwc
#endif

#define	VSTH_FGETWC_FILE	"./fgetwc_file"
wchar_t	Fgetwc_buf[256];

int
fgetwc_sf()
{
	int	cnt;
	int	errs;
	char	mbs[2];
	wint_t	val;
	FILE	*fp;

	errs	= 0;
	mbs[1]	= '\0';

	(void) remove(VSTH_FGETWC_FILE);

	fp = fopen(VSTH_FGETWC_FILE, "w");
	if ( fp != NULL )
	{
		for ( cnt = 0; cnt < 256; ++cnt )
		{
			mbs[0] = cnt;

			(void) mbtowc( &Fgetwc_buf[cnt], &mbs[0], VSTH_MBTOWC_MAXBYTES );
			val = fputwc(Fgetwc_buf[cnt], fp);
			if ( val != (wint_t) Fgetwc_buf[cnt] )
			{
				++errs;
			}
		}
		(void) fclose(fp);
	}
	else
	{
		++errs;
	}

	return(errs);
}

int
fgetwc_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	wchar_t	wc;
	FILE	*fp;

	errs	= 0;
	fails	= 0;

	vsth_ec();
	fp = fopen(VSTH_FGETWC_FILE, "r");
	vsth_xc();

	if ( fp != NULL )
	{
		for ( cnt = 0; cnt < 256; ++cnt )
		{
			wc = (wchar_t) fgetwc(fp);
			if ( wc != Fgetwc_buf[cnt] )
			{
				++fails;
			}
		}

		vsth_ec();
		(void) fclose(fp);
		vsth_xc();
	}
	else
	{
		++errs;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fgetwc_ef()
{
	(void) remove(VSTH_FGETWC_FILE);
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for fgetws()					 ******/

#ifdef	fgetws
#undef	fgetws
#endif

#define	VSTH_FGETWS_S1	"19387*&1348xav29388%$\n"
#define	VSTH_FGETWS_S2	"%j&GfUIY)(%^dR\n"

int
fgetws_sf()
{
	return(0);
}

int
fgetws_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	size_t	val;
	size_t	val1;
	size_t	val2;
	FILE	*fp;
	wchar_t	buf[VSTH_BUFLEN];
	wchar_t	buf1[VSTH_BUFLEN];
	wchar_t	buf2[VSTH_BUFLEN];
	wchar_t	*wval;

	errs	= 0;
	fails	= 0;

	vsth_ec();
	fp = tmpfile();
	vsth_xc();

	if ( fp != NULL )
	{
		vsth_ec();

		memset(&buf1[0], 0, VSTH_BUFLEN);

		val1 = mbstowcs(&buf1[0], VSTH_FGETWS_S1, VSTH_BUFLEN - 1);
		if ( val1 == -1 )
		{
			++errs;
		}

		memset(&buf2[0], 0, VSTH_BUFLEN);

		val2 = mbstowcs(&buf2[0], VSTH_FGETWS_S2, VSTH_BUFLEN - 1);
		if ( val2 == -1 )
		{
			++errs;
		}

		rval = fputws(&buf1[0], fp);
		if ( rval == 0 )
		{
			++errs;
		}

		rval = fputws(&buf2[0], fp);
		if ( rval == 0 )
		{
			++errs;
		}

		vsth_xc();

		if ( errs == 0 )
		{
			for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
			{
				rval = fseek(fp, 0, SEEK_SET);
				if ( rval == 0 )
				{
					vsth_ec();
					memset(&buf[0], 0, VSTH_BUFLEN);
					vsth_xc();

					wval = fgetws( &buf[0], VSTH_BUFLEN - 1, fp);
					if ( wval == &buf[0] )
					{
						vsth_ec();
						rval = memcmp(&buf[0], &buf1[0], VSTH_BUFLEN);
						vsth_xc();

						if ( rval )
						{
							++errs;
						}
					}
					else
					{
						++fails;
					}

					vsth_ec();
					memset(&buf[0], 0, VSTH_BUFLEN);
					vsth_xc();

					wval = fgetws( &buf[0], VSTH_BUFLEN - 1, fp);
					if ( wval == &buf[0] )
					{
						vsth_ec();
						rval = memcmp(&buf[0], &buf2[0], VSTH_BUFLEN);
						vsth_xc();

						if ( rval )
						{
							++errs;
						}
					}
					else
					{
						++fails;
					}

				}
				else
				{
					++errs;
				}
			}
		}

		vsth_ec();
		(void) fclose(fp);
		vsth_xc();
	}
	else
	{
		++errs;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fgetws_ef()
{
	return(0);
}

#endif

/****** Testset for fileno()					 ******/

#ifdef	fileno
#undef	fileno
#endif

int	Vsth_fileno_init = 0;

FILE	*Vsth_fileno_fp1;
FILE	*Vsth_fileno_fp2;
int	Vsth_fileno_fd1;
int	Vsth_fileno_fd2;

int
fileno_sf()
{
	int	errs;

	errs	= 0;

	if ( ! Vsth_fileno_init )
	{
		Vsth_fileno_fp1 = tmpfile();
		Vsth_fileno_fp2 = tmpfile();
		Vsth_fileno_fd1 = fileno(Vsth_fileno_fp1);
		Vsth_fileno_fd2 = fileno(Vsth_fileno_fp2);

		Vsth_fileno_init = 1;
	}
	return(0);
}

int
fileno_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	int	fd;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		fd = fileno(Vsth_fileno_fp1);
		if ( fd != Vsth_fileno_fd1 )
		{
			++fails;
		}

		fd = fileno(Vsth_fileno_fp2);
		if ( fd != Vsth_fileno_fd2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fileno_ef()
{
	(void) fclose(Vsth_fileno_fp1);
	(void) fclose(Vsth_fileno_fp2);
	return(0);
}

#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for flockfile()					 ******/

#ifdef	flockfile
#undef	flockfile
#endif

int
flockfile_sf()
{
	return(0);
}

#define	VSTH_FLOCKFILE_S1	"0123456789\n"
#define	VSTH_FLOCKFILE_S2	"XXXXXXXXXX\n"

int
flockfile_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	buf[VSTH_BUFLEN];
	FILE	*fp;
	char	*p;

	errs	= 0;
	fails	= 0;

	vsth_ec();
	fp = tmpfile();
	vsth_xc();

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		flockfile(fp);
		fputs(VSTH_FLOCKFILE_S1, fp);
		funlockfile(fp);

		flockfile(fp);
		fputs(VSTH_FLOCKFILE_S2, fp);
		funlockfile(fp);
	}

	vsth_ec();

	rval = fseek(fp, 0, SEEK_SET);
	if ( rval == 0 )
	{
		p = fgets( &buf[0], VSTH_BUFLEN - 1, fp );
		if ( p )
		{
			if (strcmp(&buf[0], VSTH_FLOCKFILE_S1))
			{
				++fails;
			}
		}
		else
		{
			++fails;
		}

		p = fgets( &buf[0], VSTH_BUFLEN - 1, fp );
		if ( p )
		{
			if (strcmp(&buf[0], VSTH_FLOCKFILE_S2))
			{
				++fails;
			}
		}
		else
		{
			++fails;
		}
	}
	else
	{
		++errs;
	}

	vsth_xc();

	if ( errs == 0 )
	{
		for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
		{
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
flockfile_ef()
{
	return(0);
}

#endif

/****** Testset for floor()					 ******/

#ifdef	floor
#undef	floor
#endif

#define	VSTH_FLOOR_VAL1		-2.2
#define	VSTH_FLOOR_VAL2		5.5

int	Vsth_floor_init = 0;
double	Vsth_floor_val1;
double	Vsth_floor_val2;

int
floor_sf()
{
	if ( ! Vsth_floor_init )
	{
		Vsth_floor_val1 = floor(VSTH_FLOOR_VAL1);
		Vsth_floor_val2 = floor(VSTH_FLOOR_VAL2);
		Vsth_floor_init = 1;
	}
		
	return(0);
}

int
floor_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = floor(VSTH_FLOOR_VAL1);
		if ( rval != Vsth_floor_val1 )
		{
			++fails;
		}

		rval = floor(VSTH_FLOOR_VAL2);
		if ( rval != Vsth_floor_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
floor_ef()
{
	return(0);
}

/****** Testset for fmod()					 ******/

#ifdef	fmod
#undef	fmod
#endif

#define	VSTH_FMOD_VAL1		-0.2
#define	VSTH_FMOD_VAL2		0.5
int	Vsth_fmod_init = 0;
double	Vsth_fmod_val1;
double	Vsth_fmod_val2;

int
fmod_sf()
{
	if ( ! Vsth_fmod_init )
	{
		Vsth_fmod_val1 = fmod(VSTH_FMOD_VAL1, VSTH_FMOD_VAL2);
		Vsth_fmod_val2 = fmod(VSTH_FMOD_VAL2, VSTH_FMOD_VAL1);
		Vsth_fmod_init = 1;
	}
		
	return(0);
}

int
fmod_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = fmod(VSTH_FMOD_VAL1, VSTH_FMOD_VAL2);
		if ( rval != Vsth_fmod_val1 )
		{
			++fails;
		}

		rval = fmod(VSTH_FMOD_VAL2, VSTH_FMOD_VAL1);
		if ( rval != Vsth_fmod_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fmod_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for fmtmsg()					 ******/

#ifdef	fmtmsg
#undef	fmtmsg
#endif

int
fmtmsg_sf()
{
	return(0);
}

int
fmtmsg_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fmtmsg_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for fnmatch()					 ******/

#ifdef	fnmatch
#undef	fnmatch
#endif

int
fnmatch_sf()
{
	return(0);
}

int
fnmatch_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fnmatch_ef()
{
	return(0);
}

#endif

/****** Testset for fopen()					 ******/

#ifdef	fopen
#undef	fopen
#endif

int
fopen_sf()
{
	return(0);
}

int
fopen_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fopen_ef()
{
	return(0);
}

/****** Testset for fork()					 ******/

#ifdef	fork
#undef	fork
#endif

int
fork_sf()
{
	return(0);
}

int
fork_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fork_ef()
{
	return(0);
}

/****** Testset for fpathconf()					 ******/

#ifdef	fpathconf
#undef	fpathconf
#endif

int
fpathconf_sf()
{
	return(0);
}

int
fpathconf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fpathconf_ef()
{
	return(0);
}

/****** Testset for fprintf()					 ******/

#ifdef	fprintf
#undef	fprintf
#endif

int
fprintf_sf()
{
	return(0);
}

int
fprintf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fprintf_ef()
{
	return(0);
}

/****** Testset for fputc()					 ******/

#ifdef	fputc
#undef	fputc
#endif

int
fputc_sf()
{
	return(0);
}

int
fputc_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fputc_ef()
{
	return(0);
}

/****** Testset for fputs()					 ******/

#ifdef	fputs
#undef	fputs
#endif

int
fputs_sf()
{
	return(0);
}

int
fputs_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fputs_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for fputwc()					 ******/

#ifdef	fputwc
#undef	fputwc
#endif

int
fputwc_sf()
{
	return(0);
}

int
fputwc_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fputwc_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for fputws()					 ******/

#ifdef	fputws
#undef	fputws
#endif

int
fputws_sf()
{
	return(0);
}

int
fputws_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fputws_ef()
{
	return(0);
}

#endif

/****** Testset for fread()					 ******/

#ifdef	fread
#undef	fread
#endif

int
fread_sf()
{
	return(0);
}

int
fread_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fread_ef()
{
	return(0);
}

/****** Testset for free()					 ******/

#ifdef	free
#undef	free
#endif

int
free_sf()
{
	return(0);
}

int
free_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
free_ef()
{
	return(0);
}

/****** Testset for freopen()					 ******/

#ifdef	freopen
#undef	freopen
#endif

int
freopen_sf()
{
	return(0);
}

int
freopen_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
freopen_ef()
{
	return(0);
}

/****** Testset for frexp()					 ******/

#ifdef	frexp
#undef	frexp
#endif

#define	VSTH_FREXP_VAL1		-0.2
#define	VSTH_FREXP_VAL2		0.5
int	Vsth_frexp_init = 0;
double	Vsth_frexp_val1;
int	Vsth_frexp_val1e;
double	Vsth_frexp_val2;
int	Vsth_frexp_val2e;

int
frexp_sf()
{
	if ( ! Vsth_frexp_init )
	{
		Vsth_frexp_val1 = frexp(VSTH_FREXP_VAL1, &Vsth_frexp_val1e);
		Vsth_frexp_val2 = frexp(VSTH_FREXP_VAL2, &Vsth_frexp_val2e);
		Vsth_frexp_init = 1;
	}
		
	return(0);
}

int
frexp_tf()
{
	int	cnt;
	int	val;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = frexp(VSTH_FREXP_VAL1, &val);
		if ( rval != Vsth_frexp_val1 )
		{
			++fails;
		}
		if ( val != Vsth_frexp_val1e )
		{
			++fails;
		}

		rval = frexp(VSTH_FREXP_VAL2, &val);
		if ( rval != Vsth_frexp_val2 )
		{
			++fails;
		}
		if ( val != Vsth_frexp_val2e )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
frexp_ef()
{
	return(0);
}

/****** Testset for fscanf()					 ******/

#ifdef	fscanf
#undef	fscanf
#endif

int
fscanf_sf()
{
	return(0);
}

int
fscanf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fscanf_ef()
{
	return(0);
}

/****** Testset for fseek()					 ******/

#ifdef	fseek
#undef	fseek
#endif

int
fseek_sf()
{
	return(0);
}

int
fseek_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fseek_ef()
{
	return(0);
}

/****** Testset for fsetpos()					 ******/

#ifdef	fsetpos
#undef	fsetpos
#endif

int
fsetpos_sf()
{
	return(0);
}

int
fsetpos_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fsetpos_ef()
{
	return(0);
}

/****** Testset for fstat()					 ******/

#ifdef	fstat
#undef	fstat
#endif

int
fstat_sf()
{
	return(0);
}

int
fstat_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fstat_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for fstatvfs()					 ******/

#ifdef	fstatvfs
#undef	fstatvfs
#endif

int
fstatvfs_sf()
{
	return(0);
}

int
fstatvfs_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fstatvfs_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_FSYNC)

/****** Testset for fsync()					 ******/

#ifdef	fsync
#undef	fsync
#endif

int
fsync_sf()
{
	return(0);
}

int
fsync_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fsync_ef()
{
	return(0);
}

#endif

/****** Testset for ftell()					 ******/

#ifdef	ftell
#undef	ftell
#endif

int
ftell_sf()
{
	return(0);
}

int
ftell_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ftell_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for ftello()					 ******/

#ifdef	ftello
#undef	ftello
#endif

int
ftello_sf()
{
	return(0);
}

int
ftello_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ftello_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for ftime()					 ******/

#ifdef	ftime
#undef	ftime
#endif

int
ftime_sf()
{
	return(0);
}

int
ftime_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ftime_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for ftok()					 ******/

#ifdef	ftok
#undef	ftok
#endif

int
ftok_sf()
{
	return(0);
}

int
ftok_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ftok_ef()
{
	return(0);
}

#endif

/****** Testset for ftruncate()					 ******/

#ifdef	ftruncate
#undef	ftruncate
#endif

int
ftruncate_sf()
{
	return(0);
}

int
ftruncate_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ftruncate_ef()
{
	return(0);
}

#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for ftrylockfile()				 ******/

#ifdef	ftrylockfile
#undef	ftrylockfile
#endif

int
ftrylockfile_sf()
{
	return(0);
}

int
ftrylockfile_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ftrylockfile_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for ftw()					 ******/

#ifdef	ftw
#undef	ftw
#endif

int
ftw_sf()
{
	return(0);
}

int
ftw_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ftw_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for funlockfile()				 ******/

#ifdef	funlockfile
#undef	funlockfile
#endif

int
funlockfile_sf()
{
	return(0);
}

int
funlockfile_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
funlockfile_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for fwide()					 ******/

#ifdef	fwide
#undef	fwide
#endif

int
fwide_sf()
{
	return(0);
}

int
fwide_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fwide_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for fwprintf()					 ******/

#ifdef	fwprintf
#undef	fwprintf
#endif

int
fwprintf_sf()
{
	return(0);
}

int
fwprintf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fwprintf_ef()
{
	return(0);
}

#endif

/****** Testset for fwrite()					 ******/

#ifdef	fwrite
#undef	fwrite
#endif

int
fwrite_sf()
{
	return(0);
}

int
fwrite_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fwrite_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for fwscanf()					 ******/

#ifdef	fwscanf
#undef	fwscanf
#endif

int
fwscanf_sf()
{
	return(0);
}

int
fwscanf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
fwscanf_ef()
{
	return(0);
}

#endif
