
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_rtl.c	1.5
*
*
* CHANGE HISTORY
*	12 May 1999 - Incorporate SR#104, to require ifdefs around aio.h
*************************************************************************/

#include "std.h"

#include <unistd.h>
#if	defined(_POSIX_ASYNCHRONOUS_IO)
#include <aio.h>
#endif
#include <locale.h>
#include <math.h>

#if	( TEST_XPG > 4 )
#include <search.h>
#endif

#include <setjmp.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>

#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"

#if	( TEST_XPG > 4 )

/****** Testset for l64a()					 ******/

#ifdef	l64a
#undef	l64a
#endif

int
l64a_sf()
{
	return(0);
}

int
l64a_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
l64a_ef()
{
	return(0);
}

#endif

/****** Testset for labs()					 ******/

#ifdef	labs
#undef	labs
#endif

int
labs_sf()
{
	return(0);
}

int
labs_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
labs_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for lchown()					 ******/

#ifdef	lchown
#undef	lchown
#endif

int
lchown_sf()
{
	return(0);
}

int
lchown_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lchown_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for lcong48()					 ******/

#ifdef	lcong48
#undef	lcong48
#endif

int
lcong48_sf()
{
	return(0);
}

int
lcong48_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lcong48_ef()
{
	return(0);
}

#endif

/****** Testset for ldexp()					 ******/

#ifdef	ldexp
#undef	ldexp
#endif

#define	VSTH_LDEXP_VAL1		-0.2
#define	VSTH_LDEXP_VAL2		0.5
#define	VSTH_LDEXP_VAL1E	5
#define	VSTH_LDEXP_VAL2E	2
int	Vsth_ldexp_init = 0;
double	Vsth_ldexp_val1;
double	Vsth_ldexp_val2;

int
ldexp_sf()
{
	if ( ! Vsth_ldexp_init )
	{
		Vsth_ldexp_val1 = ldexp(VSTH_LDEXP_VAL1, VSTH_LDEXP_VAL1E);
		Vsth_ldexp_val2 = ldexp(VSTH_LDEXP_VAL2, VSTH_LDEXP_VAL2E);
		Vsth_ldexp_init = 1;
	}
		
	return(0);
}

int
ldexp_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = ldexp(VSTH_LDEXP_VAL1, VSTH_LDEXP_VAL1E);
		if ( rval != Vsth_ldexp_val1 )
		{
			++fails;
		}

		rval = ldexp(VSTH_LDEXP_VAL2, VSTH_LDEXP_VAL2E);
		if ( rval != Vsth_ldexp_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ldexp_ef()
{
	return(0);
}

/****** Testset for ldiv()					 ******/

#ifdef	ldiv
#undef	ldiv
#endif

#define	VSTH_LDIV_N1	1234567L
#define	VSTH_LDIV_D1	89L
#define	VSTH_LDIV_N2	2345678L
#define	VSTH_LDIV_D2	901L

int	Vsth_ldiv_init = 0;
ldiv_t	Vsth_ldiv_val1;
ldiv_t	Vsth_ldiv_val2;

int
ldiv_sf()
{
	if ( ! Vsth_ldiv_init )
	{
		Vsth_ldiv_val1 = ldiv(VSTH_LDIV_N1, VSTH_LDIV_D1);
		Vsth_ldiv_val2 = ldiv(VSTH_LDIV_N2, VSTH_LDIV_D2);
		Vsth_ldiv_init = 1;
	}
	return(0);
}

int
ldiv_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	ldiv_t	val;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		val = ldiv(VSTH_LDIV_N1, VSTH_LDIV_D1);
		if ( val.quot != Vsth_ldiv_val1.quot )
		{
			++fails;
		}

		if ( val.rem != Vsth_ldiv_val1.rem )
		{
			++fails;
		}

		val = ldiv(VSTH_LDIV_N2, VSTH_LDIV_D2);
		if ( val.quot != Vsth_ldiv_val2.quot )
		{
			++fails;
		}

		if ( val.rem != Vsth_ldiv_val2.rem )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ldiv_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for lfind()					 ******/

#ifdef	lfind
#undef	lfind
#endif

int
lfind_sf()
{
	return(0);
}

int
lfind_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lfind_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for lgamma()					 ******/

#ifdef	lgamma
#undef	lgamma
#endif

int
lgamma_sf()
{
	return(0);
}

int
lgamma_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lgamma_ef()
{
	return(0);
}

#endif

/****** Testset for link()					 ******/

#ifdef	link
#undef	link
#endif

int
link_sf()
{
	return(0);
}

int
link_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
link_ef()
{
	return(0);
}

#if	defined(_POSIX_ASYNCHRONOUS_IO)

/****** Testset for lio_listio()				 ******/

#ifdef	lio_listio
#undef	lio_listio
#endif

int
lio_listio_sf()
{
	return(0);
}

int
lio_listio_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lio_listio_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for localconv()					 ******/

#ifdef	localconv
#undef	localconv
#endif

int
localconv_sf()
{
	return(0);
}

int
localconv_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
localconv_ef()
{
	return(0);
}

#endif

/****** Testset for localtime()					 ******/

#ifdef	localtime
#undef	localtime
#endif

int
localtime_sf()
{
	return(0);
}

int
localtime_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
localtime_ef()
{
	return(0);
}

#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for localtime_r()				 ******/

#ifdef	localtime_r
#undef	localtime_r
#endif

int
localtime_r_sf()
{
	return(0);
}

int
localtime_r_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* thread safety is testing in PTHR.os/all/localtime_r */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
localtime_r_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for lockf()					 ******/

#ifdef	lockf
#undef	lockf
#endif

int
lockf_sf()
{
	return(0);
}

int
lockf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lockf_ef()
{
	return(0);
}

#endif

/****** Testset for log()					 ******/

#ifdef	log
#undef	log
#endif

#define	VSTH_LOG_VAL1		0.2
#define	VSTH_LOG_VAL2		0.5

int	Vsth_log_init = 0;
double	Vsth_log_val1;
double	Vsth_log_val2;

int
log_sf()
{
	if ( ! Vsth_log_init )
	{
		Vsth_log_val1 = log(VSTH_LOG_VAL1);
		Vsth_log_val2 = log(VSTH_LOG_VAL2);
		Vsth_log_init = 1;
	}
		
	return(0);
}

int
log_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = log(VSTH_LOG_VAL1);
		if ( rval != Vsth_log_val1 )
		{
			++fails;
		}

		rval = log(VSTH_LOG_VAL2);
		if ( rval != Vsth_log_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
log_ef()
{
	return(0);
}

/****** Testset for log10()					 ******/

#ifdef	log10
#undef	log10
#endif

#define	VSTH_LOG10_VAL1		0.2
#define	VSTH_LOG10_VAL2		0.5
int	Vsth_log10_init = 0;
double	Vsth_log10_val1;
double	Vsth_log10_val2;

int
log10_sf()
{
	if ( ! Vsth_log10_init )
	{
		Vsth_log10_val1 = log10(VSTH_LOG10_VAL1);
		Vsth_log10_val2 = log10(VSTH_LOG10_VAL2);
		Vsth_log10_init = 1;
	}
		
	return(0);
}

int
log10_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = log10(VSTH_LOG10_VAL1);
		if ( rval != Vsth_log10_val1 )
		{
			++fails;
		}

		rval = log10(VSTH_LOG10_VAL2);
		if ( rval != Vsth_log10_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
log10_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for log1p()					 ******/

#ifdef	log1p
#undef	log1p
#endif

#define	VSTH_LOG1P_VAL1		-0.2
#define	VSTH_LOG1P_VAL2		0.5
int	Vsth_log1p_init = 0;
double	Vsth_log1p_val1;
double	Vsth_log1p_val2;

int
log1p_sf()
{
	if ( ! Vsth_log1p_init )
	{
		Vsth_log1p_val1 = log1p(VSTH_LOG1P_VAL1);
		Vsth_log1p_val2 = log1p(VSTH_LOG1P_VAL2);
		Vsth_log1p_init = 1;
	}
		
	return(0);
}

int
log1p_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = log1p(VSTH_LOG1P_VAL1);
		if ( rval != Vsth_log1p_val1 )
		{
			++fails;
		}

		rval = log1p(VSTH_LOG1P_VAL2);
		if ( rval != Vsth_log1p_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
log1p_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for logb()					 ******/

#ifdef	logb
#undef	logb
#endif

#define	VSTH_LOGB_VAL1		-0.2
#define	VSTH_LOGB_VAL2		0.5
int	Vsth_logb_init = 0;
double	Vsth_logb_val1;
double	Vsth_logb_val2;

int
logb_sf()
{
	if ( ! Vsth_logb_init )
	{
		Vsth_logb_val1 = logb(VSTH_LOGB_VAL1);
		Vsth_logb_val2 = logb(VSTH_LOGB_VAL2);
		Vsth_logb_init = 1;
	}
		
	return(0);
}

int
logb_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = logb(VSTH_LOGB_VAL1);
		if ( rval != Vsth_logb_val1 )
		{
			++fails;
		}

		rval = logb(VSTH_LOGB_VAL2);
		if ( rval != Vsth_logb_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
logb_ef()
{
	return(0);
}

#endif

/****** Testset for longjmp()					 ******/

#ifdef	longjmp
#undef	longjmp
#endif

int
longjmp_sf()
{
	return(0);
}

int
longjmp_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
longjmp_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for lrand48()					 ******/

#ifdef	lrand48
#undef	lrand48
#endif

int
lrand48_sf()
{
	return(0);
}

int
lrand48_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lrand48_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for lsearch()					 ******/

#ifdef	lsearch
#undef	lsearch
#endif

int
lsearch_sf()
{
	return(0);
}

int
lsearch_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lsearch_ef()
{
	return(0);
}

#endif

/****** Testset for lseek()					 ******/

#ifdef	lseek
#undef	lseek
#endif

int
lseek_sf()
{
	return(0);
}

int
lseek_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lseek_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for lstat()					 ******/

#ifdef	lstat
#undef	lstat
#endif

int
lstat_sf()
{
	return(0);
}

int
lstat_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
lstat_ef()
{
	return(0);
}

#endif
