
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_rtn.c	1.5
*
*************************************************************************/

#include "std.h"

#if	( TEST_XPG > 4 )
#include <ftw.h>
#include <langinfo.h>
#endif

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>

#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"

#if	defined(_POSIX_TIMERS)

/****** Testset for nanosleep()					 ******/

#ifdef	nanosleep
#undef	nanosleep
#endif

int
nanosleep_sf()
{
	return(0);
}

int
nanosleep_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
nanosleep_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for nextafter()					 ******/

#ifdef	nextafter
#undef	nextafter
#endif

#define	VSTH_NEXTAFTER_VAL1		-0.2
#define	VSTH_NEXTAFTER_VAL2		0.5
int	Vsth_nextafter_init = 0;
double	Vsth_nextafter_val1;
double	Vsth_nextafter_val2;

int
nextafter_sf()
{
	if ( ! Vsth_nextafter_init )
	{
		Vsth_nextafter_val1 = nextafter(VSTH_NEXTAFTER_VAL1, VSTH_NEXTAFTER_VAL2);
		Vsth_nextafter_val2 = nextafter(VSTH_NEXTAFTER_VAL2, VSTH_NEXTAFTER_VAL1);
		Vsth_nextafter_init = 1;
	}
		
	return(0);
}

int
nextafter_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = nextafter(VSTH_NEXTAFTER_VAL1, VSTH_NEXTAFTER_VAL2);
		if ( rval != Vsth_nextafter_val1 )
		{
			++errs;
		}

		rval = nextafter(VSTH_NEXTAFTER_VAL2, VSTH_NEXTAFTER_VAL1);
		if ( rval != Vsth_nextafter_val2 )
		{
			++errs;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
nextafter_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for nftw()					 ******/

#ifdef	nftw
#undef	nftw
#endif

int
nftw_sf()
{
	return(0);
}

int
nftw_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
nftw_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for nice()					 ******/

#ifdef	nice
#undef	nice
#endif

int
nice_sf()
{
	return(0);
}

int
nice_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
nice_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for nl_langinfo()				 ******/

#ifdef	nl_langinfo
#undef	nl_langinfo
#endif

int
nl_langinfo_sf()
{
	return(0);
}

int
nl_langinfo_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
nl_langinfo_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for nrand48()					 ******/

#ifdef	nrand48
#undef	nrand48
#endif

long			Vsth_nrand48_val1;
long			Vsth_nrand48_val2;

int
nrand48_sf()
{
	unsigned short int	xsubi1[3];
	unsigned short int	xsubi2[3];

	xsubi1[0] = 0x10;
	xsubi1[1] = 0x11;
	xsubi1[2] = 0x12;

	xsubi2[0] = 0x01;
	xsubi2[1] = 0x11;
	xsubi2[2] = 0x21;

	Vsth_nrand48_val1 = nrand48(&xsubi1[0]);
	Vsth_nrand48_val2 = nrand48(&xsubi2[0]);

	return(0);
}

int
nrand48_tf()
{
	int			cnt;
	int			rval;
	int			errs;
	int			fails;
	long			val1;
	long			val2;
	unsigned short int	xsubi1[3];
	unsigned short int	xsubi2[3];


	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{

		xsubi1[0] = 0x10;
		xsubi1[1] = 0x11;
		xsubi1[2] = 0x12;

		xsubi2[0] = 0x01;
		xsubi2[1] = 0x11;
		xsubi2[2] = 0x21;

		val1 = nrand48(&xsubi1[0]);
		if ( val1 != Vsth_nrand48_val1 )
		{
			++errs;
		}

		val2 = nrand48(&xsubi2[0]);
		if ( val2 != Vsth_nrand48_val2 )
		{
			++errs;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
nrand48_ef()
{
	return(0);
}

#endif
