
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_rts.c	1.14
*
*
* CHANGE HISTORY
*	12 May 1999 - Incorporate SR#104, to require ifdefs around aio.h
*************************************************************************/

#include "std.h"

#include <dirent.h>
#include <errno.h>
#include <grp.h>
#include <locale.h>
#include <math.h>

#if	( TEST_XPG > 4 )
#include <monetary.h>
#endif

#include <pwd.h>
#include <unistd.h>
#include <sched.h>
#if	defined(_POSIX_SEMAPHORES)
#include <semaphore.h>
#endif
#include <setjmp.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if	( TEST_XPG > 4 )
#include <strings.h>
#endif

#include <sys/mman.h>

#if	( TEST_XPG > 4 )
#include <sys/resource.h>
#include <sys/sem.h>
#include <sys/shm.h>
#endif

#include <sys/stat.h>

#if	( TEST_XPG > 4 )
#include <sys/statvfs.h>
#include <sys/time.h>
#endif

#include <sys/types.h>

#if	( TEST_XPG > 4 )
#include <syslog.h>
#endif

#include <time.h>

#if	( TEST_XPG > 4 )
#include <ucontext.h>
#endif


#if	( TEST_XPG > 4 )
#include <utmpx.h>
#include <wchar.h>
#endif

#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"

#if	( TEST_XPG > 4 )

/****** Testset for sbrk()					 ******/

#ifdef	sbrk
#undef	sbrk
#endif

int
sbrk_sf()
{
	return(0);
}

int
sbrk_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sbrk_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for scalb()					 ******/

#ifdef	scalb
#undef	scalb
#endif

/*
 *  NB Valid scalb() args must be between 1.0 and 10.0
 */
#define	VSTH_SCALB_VAL1		1.0
#define	VSTH_SCALB_VAL2		1.0

int	Vsth_scalb_init = 0;
double	Vsth_scalb_val1;
double	Vsth_scalb_val2;

int
scalb_sf()
{
	if ( ! Vsth_scalb_init )
	{
		Vsth_scalb_val1 = scalb(VSTH_SCALB_VAL1, VSTH_SCALB_VAL2);
		Vsth_scalb_val2 = scalb(VSTH_SCALB_VAL2, VSTH_SCALB_VAL1);
		Vsth_scalb_init = 1;
	}
		
	return(0);
}

int
scalb_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = scalb(VSTH_SCALB_VAL1, VSTH_SCALB_VAL2);
		if ( rval != Vsth_scalb_val1 )
		{
			++errs;
		}

		rval = scalb(VSTH_SCALB_VAL2, VSTH_SCALB_VAL1);
		if ( rval != Vsth_scalb_val2 )
		{
			++errs;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
scalb_ef()
{
	return(0);
}

#endif

/****** Testset for scanf()					 ******/

#ifdef	scanf
#undef	scanf
#endif

int
scanf_sf()
{
	return(0);
}

int
scanf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
scanf_ef()
{
	return(0);
}

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_get_priority_max()			 ******/

#ifdef	sched_get_priority_max
#undef	sched_get_priority_max
#endif

int
sched_get_priority_max_sf()
{
	return(0);
}

int
sched_get_priority_max_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_get_priority_max_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_get_priority_min()			 ******/

#ifdef	sched_get_priority_min
#undef	sched_get_priority_min
#endif

int
sched_get_priority_min_sf()
{
	return(0);
}

int
sched_get_priority_min_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_get_priority_min_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_getparam()				 ******/

#ifdef	sched_getparam
#undef	sched_getparam
#endif

int
sched_getparam_sf()
{
	return(0);
}

int
sched_getparam_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_getparam_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_getscheduler()			 ******/

#ifdef	sched_getscheduler
#undef	sched_getscheduler
#endif

int
sched_getscheduler_sf()
{
	return(0);
}

int
sched_getscheduler_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_getscheduler_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_rr_get_interval()			 ******/

#ifdef	sched_rr_get_interval
#undef	sched_rr_get_interval
#endif

int
sched_rr_get_interval_sf()
{
	return(0);
}

int
sched_rr_get_interval_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_rr_get_interval_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_setparam()				 ******/

#ifdef	sched_setparam
#undef	sched_setparam
#endif

int
sched_setparam_sf()
{
	return(0);
}

int
sched_setparam_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_setparam_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_setscheduler()			 ******/

#ifdef	sched_setscheduler
#undef	sched_setscheduler
#endif

int
sched_setscheduler_sf()
{
	return(0);
}

int
sched_setscheduler_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_setscheduler_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_THREADS) || defined(_POSIX_PRIORITY_SCHEDULING)

/****** Testset for sched_yield()				 ******/

#ifdef	sched_yield
#undef	sched_yield
#endif

int
sched_yield_sf()
{
	return(0);
}

int
sched_yield_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sched_yield_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for seed48()					 ******/

#ifdef	seed48
#undef	seed48
#endif

int
seed48_sf()
{
	return(0);
}

int
seed48_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
seed48_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for seekdir()					 ******/

#ifdef	seekdir
#undef	seekdir
#endif

int
seekdir_sf()
{
	return(0);
}

int
seekdir_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
seekdir_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for select()					 ******/

#ifdef	select
#undef	select
#endif

int
select_sf()
{
	return(0);
}

int
select_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
select_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_close()					 ******/

#ifdef	sem_close
#undef	sem_close
#endif

int
sem_close_sf()
{
	return(0);
}

int
sem_close_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_close_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_destroy()				 ******/

#ifdef	sem_destroy
#undef	sem_destroy
#endif

int
sem_destroy_sf()
{
	return(0);
}

int
sem_destroy_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_destroy_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_getvalue()				 ******/

#ifdef	sem_getvalue
#undef	sem_getvalue
#endif

int
sem_getvalue_sf()
{
	return(0);
}

int
sem_getvalue_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_getvalue_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_init()					 ******/

#ifdef	sem_init
#undef	sem_init
#endif

int
sem_init_sf()
{
	return(0);
}

int
sem_init_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_init_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_open()					 ******/

#ifdef	sem_open
#undef	sem_open
#endif

int
sem_open_sf()
{
	return(0);
}

int
sem_open_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_open_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_post()					 ******/

#ifdef	sem_post
#undef	sem_post
#endif

int
sem_post_sf()
{
	return(0);
}

int
sem_post_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_post_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_trywait()				 ******/

#ifdef	sem_trywait
#undef	sem_trywait
#endif

int
sem_trywait_sf()
{
	return(0);
}

int
sem_trywait_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_trywait_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_unlink()				 ******/

#ifdef	sem_unlink
#undef	sem_unlink
#endif

int
sem_unlink_sf()
{
	return(0);
}

int
sem_unlink_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_unlink_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SEMAPHORES)

/****** Testset for sem_wait()					 ******/

#ifdef	sem_wait
#undef	sem_wait
#endif

int
sem_wait_sf()
{
	return(0);
}

int
sem_wait_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sem_wait_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for semctl()					 ******/

#ifdef	semctl
#undef	semctl
#endif

int
semctl_sf()
{
	return(0);
}

int
semctl_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
semctl_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for semget()					 ******/

#ifdef	semget
#undef	semget
#endif

int
semget_sf()
{
	return(0);
}

int
semget_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
semget_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for semop()					 ******/

#ifdef	semop
#undef	semop
#endif

int
semop_sf()
{
	return(0);
}

int
semop_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
semop_ef()
{
	return(0);
}

#endif

/****** Testset for setbuf()					 ******/

#ifdef	setbuf
#undef	setbuf
#endif

int
setbuf_sf()
{
	return(0);
}

int
setbuf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setbuf_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for setcontext()				 ******/

#ifdef	setcontext
#undef	setcontext
#endif

int
setcontext_sf()
{
	return(0);
}

int
setcontext_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setcontext_ef()
{
	return(0);
}

#endif

/****** Testset for setgid()					 ******/

#ifdef	setgid
#undef	setgid
#endif

int
setgid_sf()
{
	return(0);
}

int
setgid_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setgid_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for setgrent()					 ******/

#ifdef	setgrent
#undef	setgrent
#endif

int
setgrent_sf()
{
	return(0);
}

int
setgrent_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setgrent_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setitimer()					 ******/

#ifdef	setitimer
#undef	setitimer
#endif

int
setitimer_sf()
{
	return(0);
}

int
setitimer_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setitimer_ef()
{
	return(0);
}

#endif

/****** Testset for setjmp()					 ******/

#ifdef	setjmp
#undef	setjmp
#endif

int
setjmp_sf()
{
	return(0);
}

int
setjmp_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setjmp_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_CRYPT) && (_XOPEN_CRYPT != -1)

/****** Testset for setkey()					 ******/

#ifdef	setkey
#undef	setkey
#endif

int
setkey_sf()
{
	return(0);
}

int
setkey_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setkey_ef()
{
	return(0);
}

#endif
#endif

/****** Testset for setlocale()					 ******/

#ifdef	setlocale
#undef	setlocale
#endif

int
setlocale_sf()
{
	return(0);
}

int
setlocale_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setlocale_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for setlogmask()				 ******/

#ifdef	setlogmask
#undef	setlogmask
#endif

int
setlogmask_sf()
{
	return(0);
}

int
setlogmask_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setlogmask_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_JOB_CONTROL)

/****** Testset for setpgid()					 ******/

#ifdef	setpgid
#undef	setpgid
#endif

int
setpgid_sf()
{
	return(0);
}

int
setpgid_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setpgid_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setpgrp()					 ******/

#ifdef	setpgrp
#undef	setpgrp
#endif

int
setpgrp_sf()
{
	return(0);
}

int
setpgrp_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setpgrp_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setpriority()				 ******/

#ifdef	setpriority
#undef	setpriority
#endif

int
setpriority_sf()
{
	return(0);
}

int
setpriority_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setpriority_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setpwent()					 ******/

#ifdef	setpwent
#undef	setpwent
#endif

int
setpwent_sf()
{
	return(0);
}

int
setpwent_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setpwent_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setregid()					 ******/

#ifdef	setregid
#undef	setregid
#endif

int
setregid_sf()
{
	return(0);
}

int
setregid_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setregid_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for setrlimit()					 ******/

#ifdef	setrlimit
#undef	setrlimit
#endif

int
setrlimit_sf()
{
	return(0);
}

int
setrlimit_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setrlimit_ef()
{
	return(0);
}

#endif

/****** Testset for setsid()					 ******/

#ifdef	setsid
#undef	setsid
#endif

int
setsid_sf()
{
	return(0);
}

int
setsid_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setsid_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for setstate()					 ******/

#ifdef	setstate
#undef	setstate
#endif

int
setstate_sf()
{
	return(0);
}

int
setstate_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setstate_ef()
{
	return(0);
}

#endif

/****** Testset for setuid()					 ******/

#ifdef	setuid
#undef	setuid
#endif

int
setuid_sf()
{
	return(0);
}

int
setuid_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setuid_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for setutxent()					 ******/

#ifdef	setutxent
#undef	setutxent
#endif

int
setutxent_sf()
{
	return(0);
}

int
setutxent_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setutxent_ef()
{
	return(0);
}

#endif

/****** Testset for setvbuf()					 ******/

#ifdef	setvbuf
#undef	setvbuf
#endif

int
setvbuf_sf()
{
	return(0);
}

int
setvbuf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
setvbuf_ef()
{
	return(0);
}

#if	defined(_POSIX_SHARED_MEMORY_OBJECTS)

/****** Testset for shm_open()					 ******/

#ifdef	shm_open
#undef	shm_open
#endif

int
shm_open_sf()
{
	return(0);
}

int
shm_open_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
shm_open_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_SHARED_MEMORY_OBJECTS)

/****** Testset for shm_unlink()				 ******/

#ifdef	shm_unlink
#undef	shm_unlink
#endif

int
shm_unlink_sf()
{
	return(0);
}

int
shm_unlink_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
shm_unlink_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for shmat()					 ******/

#ifdef	shmat
#undef	shmat
#endif

int
shmat_sf()
{
	return(0);
}

int
shmat_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
shmat_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for shmctl()					 ******/

#ifdef	shmctl
#undef	shmctl
#endif

int
shmctl_sf()
{
	return(0);
}

int
shmctl_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
shmctl_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for shmdt()					 ******/

#ifdef	shmdt
#undef	shmdt
#endif

int
shmdt_sf()
{
	return(0);
}

int
shmdt_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
shmdt_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for shmget()					 ******/

#ifdef	shmget
#undef	shmget
#endif

int
shmget_sf()
{
	return(0);
}

int
shmget_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
shmget_ef()
{
	return(0);
}

#endif

/****** Testset for sigaction()					 ******/

#ifdef	sigaction
#undef	sigaction
#endif

int
sigaction_sf()
{
	return(0);
}

int
sigaction_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigaction_ef()
{
	return(0);
}

/****** Testset for sigaddset()					 ******/

#ifdef	sigaddset
#undef	sigaddset
#endif

int
sigaddset_sf()
{
	return(0);
}

int
sigaddset_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigaddset_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for sigaltstack()				 ******/

#ifdef	sigaltstack
#undef	sigaltstack
#endif

int
sigaltstack_sf()
{
	return(0);
}

int
sigaltstack_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigaltstack_ef()
{
	return(0);
}

#endif

/****** Testset for sigdelset()					 ******/

#ifdef	sigdelset
#undef	sigdelset
#endif

int
sigdelset_sf()
{
	return(0);
}

int
sigdelset_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigdelset_ef()
{
	return(0);
}

/****** Testset for sigemptyset()				 ******/

#ifdef	sigemptyset
#undef	sigemptyset
#endif

int
sigemptyset_sf()
{
	return(0);
}

int
sigemptyset_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigemptyset_ef()
{
	return(0);
}

/****** Testset for sigfillset()				 ******/

#ifdef	sigfillset
#undef	sigfillset
#endif

int
sigfillset_sf()
{
	return(0);
}

int
sigfillset_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigfillset_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for sighold()					 ******/

#ifdef	sighold
#undef	sighold
#endif

int
sighold_sf()
{
	return(0);
}

int
sighold_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sighold_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for sigignore()					 ******/

#ifdef	sigignore
#undef	sigignore
#endif

int
sigignore_sf()
{
	return(0);
}

int
sigignore_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigignore_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for siginterrupt()				 ******/

#ifdef	siginterrupt
#undef	siginterrupt
#endif

int
siginterrupt_sf()
{
	return(0);
}

int
siginterrupt_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
siginterrupt_ef()
{
	return(0);
}

#endif

/****** Testset for sigismember()				 ******/

#ifdef	sigismember
#undef	sigismember
#endif

int
sigismember_sf()
{
	return(0);
}

int
sigismember_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigismember_ef()
{
	return(0);
}

/****** Testset for siglongjmp()				 ******/

#ifdef	siglongjmp
#undef	siglongjmp
#endif

int
siglongjmp_sf()
{
	return(0);
}

int
siglongjmp_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
siglongjmp_ef()
{
	return(0);
}

/****** Testset for signal()					 ******/

#ifdef	signal
#undef	signal
#endif

int
signal_sf()
{
	return(0);
}

int
signal_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
signal_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for signgam()					 ******/

#ifdef	signgam
#undef	signgam
#endif

int
signgam_sf()
{
	return(0);
}

int
signgam_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
signgam_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for sigpause()					 ******/

#ifdef	sigpause
#undef	sigpause
#endif

int
sigpause_sf()
{
	return(0);
}

int
sigpause_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigpause_ef()
{
	return(0);
}

#endif

/****** Testset for sigpending()				 ******/

#ifdef	sigpending
#undef	sigpending
#endif

int
sigpending_sf()
{
	return(0);
}

int
sigpending_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigpending_ef()
{
	return(0);
}

/****** Testset for sigprocmask()				 ******/

#ifdef	sigprocmask
#undef	sigprocmask
#endif

int
sigprocmask_sf()
{
	return(0);
}

int
sigprocmask_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigprocmask_ef()
{
	return(0);
}

#if	defined(_POSIX_REALTIME_SIGNALS)

/****** Testset for sigqueue()					 ******/

#ifdef	sigqueue
#undef	sigqueue
#endif

int
sigqueue_sf()
{
	return(0);
}

int
sigqueue_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigqueue_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for sigrelse()					 ******/

#ifdef	sigrelse
#undef	sigrelse
#endif

int
sigrelse_sf()
{
	return(0);
}

int
sigrelse_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigrelse_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for sigset()					 ******/

#ifdef	sigset
#undef	sigset
#endif

int
sigset_sf()
{
	return(0);
}

int
sigset_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigset_ef()
{
	return(0);
}

#endif

/****** Testset for sigsetjmp()					 ******/

#ifdef	sigsetjmp
#undef	sigsetjmp
#endif

int
sigsetjmp_sf()
{
	return(0);
}

int
sigsetjmp_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigsetjmp_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for sigstack()					 ******/

#ifdef	sigstack
#undef	sigstack
#endif

int
sigstack_sf()
{
	return(0);
}

int
sigstack_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigstack_ef()
{
	return(0);
}

#endif
#endif

/****** Testset for sigsuspend()				 ******/

#ifdef	sigsuspend
#undef	sigsuspend
#endif

int
sigsuspend_sf()
{
	return(0);
}

int
sigsuspend_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigsuspend_ef()
{
	return(0);
}

/****** Testset for sigwait()					 ******/

#ifdef	sigwait
#undef	sigwait
#endif

int
sigwait_sf()
{
	return(0);
}

int
sigwait_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigwait_ef()
{
	return(0);
}

#ifdef	_POSIX_REALTIME_SIGNALS

/****** Testset for sigtimedwait()					 ******/

#ifdef	sigtimedwait
#undef	sigtimedwait
#endif

int
sigtimedwait_sf()
{
	return(0);
}

int
sigtimedwait_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigtimedwait_ef()
{
	return(0);
}

#endif	/* _POSIX_REALTIME_SIGNALS */

#ifdef	_POSIX_REALTIME_SIGNALS

/****** Testset for sigwaitinfo()				 ******/

#ifdef	sigwaitinfo
#undef	sigwaitinfo
#endif

int
sigwaitinfo_sf()
{
	return(0);
}

int
sigwaitinfo_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sigwaitinfo_ef()
{
	return(0);
}

#endif	/* _POSIX_REALTIME_SIGNALS */

/****** Testset for sin()					 ******/

#ifdef	sin
#undef	sin
#endif

#define	VSTH_SIN_VAL1		-0.2
#define	VSTH_SIN_VAL2		0.5
int	Vsth_sin_init = 0;
double	Vsth_sin_val1;
double	Vsth_sin_val2;

int
sin_sf()
{
	if ( ! Vsth_sin_init )
	{
		Vsth_sin_val1 = sin(VSTH_SIN_VAL1);
		Vsth_sin_val2 = sin(VSTH_SIN_VAL2);
		Vsth_sin_init = 1;
	}
		
	return(0);
}

int
sin_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = sin(VSTH_SIN_VAL1);
		if ( rval != Vsth_sin_val1 )
		{
			++fails;
		}

		rval = sin(VSTH_SIN_VAL2);
		if ( rval != Vsth_sin_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sin_ef()
{
	return(0);
}

/****** Testset for sinh()					 ******/

#ifdef	sinh
#undef	sinh
#endif

#define	VSTH_SINH_VAL1		-0.2
#define	VSTH_SINH_VAL2		0.5
int	Vsth_sinh_init = 0;
double	Vsth_sinh_val1;
double	Vsth_sinh_val2;

int
sinh_sf()
{
	if ( ! Vsth_sinh_init )
	{
		Vsth_sinh_val1 = sinh(VSTH_SINH_VAL1);
		Vsth_sinh_val2 = sinh(VSTH_SINH_VAL2);
		Vsth_sinh_init = 1;
	}
		
	return(0);
}

int
sinh_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = sinh(VSTH_SINH_VAL1);
		if ( rval != Vsth_sinh_val1 )
		{
			++fails;
		}

		rval = sinh(VSTH_SINH_VAL2);
		if ( rval != Vsth_sinh_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sinh_ef()
{
	return(0);
}

/****** Testset for sleep()					 ******/

#ifdef	sleep
#undef	sleep
#endif

int
sleep_sf()
{
	return(0);
}

int
sleep_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sleep_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for snprintf()					 ******/

#ifdef	snprintf
#undef	snprintf
#endif

int
snprintf_sf()
{
	return(0);
}

int
snprintf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
snprintf_ef()
{
	return(0);
}

#endif

/****** Testset for sprintf()					 ******/

#ifdef	sprintf
#undef	sprintf
#endif

int
sprintf_sf()
{
	return(0);
}

int
sprintf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sprintf_ef()
{
	return(0);
}

/****** Testset for sqrt()					 ******/

#ifdef	sqrt
#undef	sqrt
#endif

#define	VSTH_SQRT_VAL1		5.2
#define	VSTH_SQRT_VAL2		2.5
int	Vsth_sqrt_init = 0;
double	Vsth_sqrt_val1;
double	Vsth_sqrt_val2;

int
sqrt_sf()
{
	if ( ! Vsth_sqrt_init )
	{
		Vsth_sqrt_val1 = sqrt(VSTH_SQRT_VAL1);
		Vsth_sqrt_val2 = sqrt(VSTH_SQRT_VAL2);
		Vsth_sqrt_init = 1;
	}
		
	return(0);
}

int
sqrt_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = sqrt(VSTH_SQRT_VAL1);
		if ( rval != Vsth_sqrt_val1 )
		{
			++fails;
		}

		rval = sqrt(VSTH_SQRT_VAL2);
		if ( rval != Vsth_sqrt_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sqrt_ef()
{
	return(0);
}

/****** Testset for srand()					 ******/

#ifdef	srand
#undef	srand
#endif

int
srand_sf()
{
	return(0);
}

int
srand_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
srand_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for srand48()					 ******/

#ifdef	srand48
#undef	srand48
#endif

int
srand48_sf()
{
	return(0);
}

int
srand48_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
srand48_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for srandom()					 ******/

#ifdef	srandom
#undef	srandom
#endif

int
srandom_sf()
{
	return(0);
}

int
srandom_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
srandom_ef()
{
	return(0);
}

#endif

/****** Testset for sscanf()					 ******/

#ifdef	sscanf
#undef	sscanf
#endif

int
sscanf_sf()
{
	return(0);
}

int
sscanf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sscanf_ef()
{
	return(0);
}

/****** Testset for stat()					 ******/

#ifdef	stat
#undef	stat
#endif

int
stat_sf()
{
	return(0);
}

int
stat_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
stat_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for statvfs()					 ******/

#ifdef	statvfs
#undef	statvfs
#endif

int
statvfs_sf()
{
	return(0);
}

int
statvfs_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
statvfs_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for step()					 ******/

#ifdef	step
#undef	step
#endif

int
step_sf()
{
	return(0);
}

int
step_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
step_ef()
{
	return(0);
}

#endif
#endif

#if	( TEST_XPG > 4 )

/****** Testset for strcasecmp()				 ******/

#ifdef	strcasecmp
#undef	strcasecmp
#endif

#define	VSTH_STRCASECMP_S1	"abcdefghijklmnopqrstUvwxuz0123456789"
#define	VSTH_STRCASECMP_S2	"aBcdeFghijKLmNopQRstuvwxuz0123456789"
#define	VSTH_STRCASECMP_S3	"aBcdEfgHijKlmNopQrsTuvWxxZ0123456789"

int
strcasecmp_sf()
{
	return(0);
}

int
strcasecmp_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = strcasecmp(VSTH_STRCASECMP_S1, VSTH_STRCASECMP_S2);
		if ( rval )
		{
			++fails;
		}

		rval = strcasecmp(VSTH_STRCASECMP_S1, VSTH_STRCASECMP_S3);
		if ( rval == 0 )
		{
			++fails;
		}

	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strcasecmp_ef()
{
	return(0);
}

#endif

/****** Testset for strcat()					 ******/

#ifdef	strcat
#undef	strcat
#endif

#define	VSTH_STRCAT_S1	"Q@SEDRFTVGH MI%&Y*HU"
#define	VSTH_STRCAT_S2	"FTVGH MI%&Y*HUQ@SEDR"
#define	VSTH_STRCAT_S3	"Q@SEDR"
#define	VSTH_STRCAT_S4	"FTVGH MI%&Y*HU"

int
strcat_sf()
{
	return(0);
}

int
strcat_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	*p;
	char	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		*buf = '\0';
		p = strcat(buf, VSTH_STRCAT_S3);
		if ( p != buf )
		{
			++fails;
		}

		p = strcat( buf, VSTH_STRCAT_S4);
		if ( p != buf )
		{
			++fails;
		}

		vsth_ec();
		if (strcmp(p, VSTH_STRCAT_S1))
		{
			++fails;
		}
		vsth_xc();

		*buf = '\0';
		p = strcat(buf, VSTH_STRCAT_S4);
		if ( p != buf )
		{
			++fails;
		}

		p = strcat( buf, VSTH_STRCAT_S3);
		if ( p != buf )
		{
			++fails;
		}

		vsth_ec();
		if (strcmp(p, VSTH_STRCAT_S2))
		{
			++fails;
		}
		vsth_xc();
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strcat_ef()
{
	return(0);
}

/****** Testset for strchr()					 ******/

#ifdef	strchr
#undef	strchr
#endif

#define	VSTH_STRCHR_S1	"\001\002\003\004\005\006\007"

int
strchr_sf()
{
	return(0);
}

int
strchr_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	*p;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		p = strchr(VSTH_STRCHR_S1, 2);
		if ((!p) || (*p != '\02'))
		{
			++fails;
		}

		p = strchr(VSTH_STRCHR_S1, 6);
		if ((!p) || (*p != '\06'))
		{
			++fails;
		}

		p = strchr(VSTH_STRCHR_S1, 0);
		if ((!p) || (*p != '\0'))
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strchr_ef()
{
	return(0);
}

/****** Testset for strcmp()					 ******/

#ifdef	strcmp
#undef	strcmp
#endif

#define	VSTH_STRCMP_S1	"\001a^&%$;kj'\001"
#define	VSTH_STRCMP_S2	"\001a^&%$;kj'\001"
#define	VSTH_STRCMP_S3	"\001a^&%$;kj'\002"

int
strcmp_sf()
{
	return(0);
}

int
strcmp_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = strcmp(VSTH_STRCMP_S1, VSTH_STRCMP_S2);
		if ( rval )
		{
			++fails;
		}

		rval = strcmp(VSTH_STRCMP_S1, VSTH_STRCMP_S3);
		if ( rval == 0 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strcmp_ef()
{
	return(0);
}

/****** Testset for strcoll()					 ******/

#ifdef	strcoll
#undef	strcoll
#endif

#define	VSTH_STRCOLL_S1	"&*9030435jhadf8735kadf\\[ad,nzvjlkhadrt"
#define	VSTH_STRCOLL_S2	"&*9030435jhadf8735kadf\\[ad,nzvjlkhadru"
#define	VSTH_STRCOLL_S3	"&*9030435jhadf8735kadf\\[ad,nzvjlkhadrs"

int
strcoll_sf()
{
	return(0);
}

int
strcoll_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = strcoll(VSTH_STRCOLL_S1, VSTH_STRCOLL_S2);
		if ( rval >= 0 )
		{
			++fails;
		}

		rval = strcoll(VSTH_STRCOLL_S1, VSTH_STRCOLL_S3);
		if ( rval <= 0 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strcoll_ef()
{
	return(0);
}

/****** Testset for strcpy()					 ******/

#ifdef	strcpy
#undef	strcpy
#endif

#define	VSTH_STRCPY_S1	")(uHg*Ytf$E!Dsl0ij8uyh(U%Tfvj:L"
#define	VSTH_STRCPY_S2	")(UhG*yTF$e!dSL0IJ8Uyh(U%TFVJ:l"

int
strcpy_sf()
{
	return(0);
}

int
strcpy_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	*p;
	char	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		p = strcpy(buf, VSTH_STRCPY_S1);
		if ( p != buf )
		{
			++fails;
		}

		if ((!p) || (*(p + 10) != 'E'))
		{
			++fails;
		}

		p = strcpy(buf, VSTH_STRCPY_S2);
		if ( p != buf )
		{
			++fails;
		}

		if ((!p) || (*(p + 10) != 'e'))
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strcpy_ef()
{
	return(0);
}

/****** Testset for strcspn()					 ******/

#ifdef	strcspn
#undef	strcspn
#endif

#define	VSTH_STRCSPN_S1	"abcd9"
#define	VSTH_STRCSPN_S2	"1234567890"
#define	VSTH_STRCSPN_S3	"123abcdabcd"
#define	VSTH_STRCSPN_S4	"abcd"

int
strcspn_sf()
{
	return(0);
}

int
strcspn_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = strcspn(VSTH_STRCSPN_S1, VSTH_STRCSPN_S2);
		if ( rval != 4)
		{
			++fails;
		}

		rval = strcspn(VSTH_STRCSPN_S3, VSTH_STRCSPN_S4);
		if ( rval != 3)
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strcspn_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for strdup()					 ******/

#ifdef	strdup
#undef	strdup
#endif

#define	VSTH_STRDUP_S1	"(*&GJHG*&ROJnkuT(*^FkjhGF&65flN:9pu"
#define	VSTH_STRDUP_S2	"(*&gjhg*&rojnKUt(*^FKJHgf&65fln:9Pu"

int
strdup_sf()
{
	return(0);
}

int
strdup_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	*p;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		p = strdup(VSTH_STRDUP_S1);
		if (p)
		{
			vsth_ec();

			if ( strcmp(p, VSTH_STRDUP_S1))
			{
				++fails;
			}

			(void) free(p);

			vsth_xc();
		}
		else
		{
			++fails;
		}

		p = strdup(VSTH_STRDUP_S2);
		if (p)
		{
			vsth_ec();

			if ( strcmp(p, VSTH_STRDUP_S2))
			{
				++fails;
			}

			(void) free(p);

			vsth_xc();
		}
		else
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strdup_ef()
{
	return(0);
}

#endif

/****** Testset for strerror()					 ******/

#ifdef	strerror
#undef	strerror
#endif

int
strerror_sf()
{
	return(0);
}

int
strerror_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strerror_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for strfmon()					 ******/

#ifdef	strfmon
#undef	strfmon
#endif

#define	VSTH_STRFMON_F1	"abc%2n%%%i%=X#2.2i"
#define	VSTH_STRFMON_F2	"abc%2i%%%n%=X8nabc"

int	Vsth_strfmon_init = 0;

int
strfmon_sf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	if ( ! Vsth_strfmon_init )
	{
		memset(&Vsth_buf1[0], 0, VSTH_BUFLEN);
		memset(&Vsth_buf2[0], 0, VSTH_BUFLEN);
	
		rval = strfmon(&Vsth_buf1[0], VSTH_BUFLEN - 1, VSTH_STRFMON_F1, 1.1, 2.2, 3.3);
		if ( rval == -1 )
		{
			++errs;
		}
	
		rval = strfmon(&Vsth_buf2[0], VSTH_BUFLEN - 1, VSTH_STRFMON_F2, 1.1, 2.2, 3.3);
		if ( rval == -1 )
		{
			++errs;
		}

		Vsth_strfmon_init = 1;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strfmon_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = strfmon(&buf[0], VSTH_BUFLEN - 1, VSTH_STRFMON_F1, 1.1, 2.2, 3.3);
		if ( rval != -1 )
		{
			vsth_ec();

			if (strcmp(&buf[0], &Vsth_buf1[0]))
			{
				++fails;
			}

			vsth_xc();
		}
		else
		{
			++fails;
		}

		rval = strfmon(&buf[0], VSTH_BUFLEN - 1, VSTH_STRFMON_F2, 1.1, 2.2, 3.3);
		if ( rval != -1 )
		{
			vsth_ec();

			if (strcmp(&buf[0], &Vsth_buf2[0]))
			{
				++fails;
			}

			vsth_xc();
		}
		else
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strfmon_ef()
{
	return(0);
}

#endif

/****** Testset for strftime()					 ******/

#ifdef	strftime
#undef	strftime
#endif

#define	VSTH_STRFTIME_F1	"%a%b1.4V%X"
#define	VSTH_STRFTIME_F2	"@(#)vsth_rts.c	1.4u%j%c%%"

int		Vsth_strftime_init = 0;
struct tm	*Vsth_strftime_ptm;

int
strftime_sf()
{
	int	rval;
	int	errs;
	int	fails;
	time_t	t;

	errs	= 0;
	fails	= 0;

	if ( ! Vsth_strftime_init )
	{
		memset(&Vsth_buf1[0], 0, VSTH_BUFLEN);
		memset(&Vsth_buf2[0], 0, VSTH_BUFLEN);
	
		t = time(NULL);
		Vsth_strftime_ptm = localtime(&t);
	
		rval = strftime(&Vsth_buf1[0], VSTH_BUFLEN - 1, VSTH_STRFTIME_F1, Vsth_strftime_ptm);
		if ( rval == -1 )
		{
			++errs;
		}
	
		rval = strftime(&Vsth_buf2[0], VSTH_BUFLEN - 1, VSTH_STRFTIME_F2, Vsth_strftime_ptm);
		if ( rval == -1 )
		{
			++errs;
		}

		Vsth_strftime_init = 1;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strftime_tf()
{
	int	cnt;
	int	errs;
	int	fails;
	int	rval;
	char	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = strftime(&buf[0], VSTH_BUFLEN - 1, VSTH_STRFTIME_F1, Vsth_strftime_ptm);
		if ( rval != -1 )
		{
			vsth_ec();

			if (strcmp(&buf[0], &Vsth_buf1[0]))
			{
				++fails;
			}

			vsth_xc();
		}
		else
		{
			++fails;
		}

		rval = strftime(&buf[0], VSTH_BUFLEN - 1, VSTH_STRFTIME_F2, Vsth_strftime_ptm);
		if ( rval != -1 )
		{
			vsth_ec();

			if (strcmp(&buf[0], &Vsth_buf2[0]))
			{
				++fails;
			}

			vsth_xc();
		}
		else
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strftime_ef()
{
	return(0);
}

/****** Testset for strlen()					 ******/

#ifdef	strlen
#undef	strlen
#endif

#define	VSTH_STRLEN_S1	"!@#$%^ERTYU*(dfghj678vbhuio"
#define	VSTH_STRLEN_S2	"!@#$%^ertyu*(DFGHJo"

int	Vsth_strlen_init = 0;
size_t	Vsth_strlen_val1;
size_t	Vsth_strlen_val2;

int
strlen_sf()
{
	if ( ! Vsth_strlen_init )
	{
		Vsth_strlen_val1 = strlen(VSTH_STRLEN_S1);
		Vsth_strlen_val2 = strlen(VSTH_STRLEN_S2);
	}

	return(0);
}

int
strlen_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = strlen(VSTH_STRLEN_S1);
		if ( rval != Vsth_strlen_val1 )
		{
			++fails;
		}

		rval = strlen(VSTH_STRLEN_S2);
		if ( rval != Vsth_strlen_val2 )
		{
			++fails;
		}

	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strlen_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for strncasecmp()				 ******/

#ifdef	strncasecmp
#undef	strncasecmp
#endif

#define	VSTH_STRNCASECMP_S1	"abcdefghijklmnopqrstuvwxuz0123456789"
#define	VSTH_STRNCASECMP_S2	"abCDEfgHijkLMnopQrstuvwXU.0123456789"
#define	VSTH_STRNCASECMP_S3	"abcdefghijklmnopqrstuvwx.z0123456789"

int
strncasecmp_sf()
{
	return(0);
}

int
strncasecmp_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = strncasecmp(VSTH_STRNCASECMP_S1, VSTH_STRNCASECMP_S2, 25);
		if ( rval )
		{
			++fails;
		}

		rval = strncasecmp(VSTH_STRNCASECMP_S1, VSTH_STRNCASECMP_S3, 25);
		if ( rval == 0 )
		{
			++fails;
		}

	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strncasecmp_ef()
{
	return(0);
}

#endif

/****** Testset for strncat()					 ******/

#ifdef	strncat
#undef	strncat
#endif

#define	VSTH_STRNCAT_S1	"Q@SEDRFTVGH MI%&Y*HU"
#define	VSTH_STRNCAT_S2	"FTVGH MI%&Y*HUQ@SEDR"
#define	VSTH_STRNCAT_S3	"Q@SEDR"
#define	VSTH_STRNCAT_S4	"FTVGH MI%&Y*HU"

int
strncat_sf()
{
	return(0);
}

int
strncat_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	*p;
	char	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		*buf = '\0';
		p = strncat(buf, VSTH_STRNCAT_S3, 6);
		if ( p != buf )
		{
			++fails;
		}

		p = strncat( buf, VSTH_STRNCAT_S4, 14);
		if ( p != buf )
		{
			++fails;
		}

		vsth_ec();
		if (strcmp(p, VSTH_STRNCAT_S1))
		{
			++fails;
		}
		vsth_xc();

		*buf = '\0';
		p = strncat(buf, VSTH_STRNCAT_S4, 14);
		if ( p != buf )
		{
			++fails;
		}

		p = strncat( buf, VSTH_STRNCAT_S3, 6);
		if ( p != buf )
		{
			++fails;
		}

		vsth_ec();
		if (strcmp(p, VSTH_STRNCAT_S2))
		{
			++fails;
		}
		vsth_xc();
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strncat_ef()
{
	return(0);
}

/****** Testset for strncmp()					 ******/

#ifdef	strncmp
#undef	strncmp
#endif

#define	VSTH_STRNCMP_S1	"\001a^&%$;kj'\001"
#define	VSTH_STRNCMP_S2	"\001a^&%$;kj'\001"
#define	VSTH_STRNCMP_S3	"\001a^&%$;kj'\002"

int
strncmp_sf()
{
	return(0);
}

int
strncmp_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = strncmp(VSTH_STRNCMP_S1, VSTH_STRNCMP_S2, 11);
		if ( rval )
		{
			++fails;
		}

		rval = strncmp(VSTH_STRNCMP_S1, VSTH_STRNCMP_S3, 11);
		if ( rval == 0 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strncmp_ef()
{
	return(0);
}

/****** Testset for strncpy()					 ******/

#ifdef	strncpy
#undef	strncpy
#endif

#define	VSTH_STRNCPY_S1	")(uHg*Ytf$E!Dsl0ij8uyh(U%Tfvj:L"
#define	VSTH_STRNCPY_S2	")(UhG*yTF$e!dSL0IJ8Uyh(U%TFVJ:l"

int
strncpy_sf()
{
	return(0);
}

int
strncpy_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	*p;
	char	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		p = strncpy(buf, VSTH_STRNCPY_S1, 12);
		if ( p != buf )
		{
			++fails;
		}

		if ((!p) || (*(p + 10) != 'E'))
		{
			++fails;
		}

		p = strncpy(buf, VSTH_STRNCPY_S2, 13);
		if ( p != buf )
		{
			++fails;
		}

		if ((!p) || (*(p + 10) != 'e'))
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strncpy_ef()
{
	return(0);
}

/****** Testset for strpbrk()					 ******/

#ifdef	strpbrk
#undef	strpbrk
#endif

#define	VSTH_STRPBRK_S1	"MNKU&huoi4-0987231;lirnkdskjasodi"
#define	VSTH_STRPBRK_S2	"123"
#define	VSTH_STRPBRK_S3 "xyz"

int
strpbrk_sf()
{
	return(0);
}

int
strpbrk_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	*p;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		p = strpbrk(VSTH_STRPBRK_S1, VSTH_STRPBRK_S2);
		if ((!p) || (*p != '2'))
		{
			++fails;
		}

		p = strpbrk(VSTH_STRPBRK_S1, VSTH_STRPBRK_S3);
		if (p)
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strpbrk_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for strptime()					 ******/

#ifdef	strptime
#undef	strptime
#endif

#define	VSTH_STRPTIME_F1	"%H %M %S"
#define	VSTH_STRPTIME_F2	"%S %M %H"

int		Vsth_strptime_init = 0;
struct tm	Vsth_strptime_tm1;
struct tm	Vsth_strptime_tm2;

int
strptime_sf()
{
	int		rval;
	int		errs;
	int		fails;
	size_t		val;
	char		*p;
	struct tm	*ptm;
	time_t		t;

	errs	= 0;
	fails	= 0;

	if ( ! Vsth_strptime_init )
	{
		t = time(NULL);

		ptm = localtime(&t);
		if ( ptm )
		{
			memset(&Vsth_strptime_tm1, 0, sizeof(struct tm));
			memset(&Vsth_strptime_tm2, 0, sizeof(struct tm));
	
			val = strftime(	&Vsth_buf1[0], VSTH_BUFLEN - 1, VSTH_STRPTIME_F1, ptm);
			if ( val )
			{
				p = strptime(&Vsth_buf1[0], VSTH_STRPTIME_F1, &Vsth_strptime_tm1);
				if ( p == NULL )
				{
					++errs;
				}
			}
			else
			{
				++errs;
			}
		
			val = strftime(&Vsth_buf2[0], VSTH_BUFLEN - 1, VSTH_STRPTIME_F2, ptm);
			if ( val )
			{
				p = strptime(&Vsth_buf2[0], VSTH_STRPTIME_F2, &Vsth_strptime_tm2);
				if ( p == NULL )
				{
					++errs;
				}
			}
			else
			{
				++errs;
			}
		}
		else
		{
			++errs;
		}

		Vsth_strptime_init = 1;
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strptime_tf()
{
	int		cnt;
	int		rval;
	int		errs;
	int		fails;
	char		*p;
	char		buf[VSTH_BUFLEN];
	struct tm	tm;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		memset( &tm, 0, sizeof(struct tm));

		p = strptime(&Vsth_buf1[0], VSTH_STRPTIME_F1, &tm);
		if ( p )
		{
			vsth_ec();
			if (memcmp( &tm, &Vsth_strptime_tm1, sizeof(struct tm)))
			{
				++fails;
			}
			vsth_xc();
		}
		else
		{
			++fails;
		}

		memset( &tm, 0, sizeof(struct tm));

		p = strptime(&Vsth_buf2[0], VSTH_STRPTIME_F2, &tm);
		if ( p )
		{
			vsth_ec();
			if (memcmp( &tm, &Vsth_strptime_tm2, sizeof(struct tm)))
			{
				++fails;
			}
			vsth_xc();
		}
		else
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strptime_ef()
{
	return(0);
}

#endif

/****** Testset for strrchr()					 ******/

#ifdef	strrchr
#undef	strrchr
#endif

#define	VSTH_STRRCHR_S1	"poi.daf9.813xjxvcd9p8yq"

int
strrchr_sf()
{
	return(0);
}

int
strrchr_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	*p;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		p = strrchr( VSTH_STRRCHR_S1, '.');
		if ((p == NULL) || (*++p != '8'))
		{
			++fails;
		}

		p = strrchr( VSTH_STRRCHR_S1, 'x');
		if ((p == NULL) || (*++p != 'v'))
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strrchr_ef()
{
	return(0);
}

/****** Testset for strspn()					 ******/

#ifdef	strspn
#undef	strspn
#endif

#define	VSTH_STRSPN_S1	"122122123789789789"
#define	VSTH_STRSPN_S2	"123"
#define	VSTH_STRSPN_S3	"12"

int
strspn_sf()
{
	return(0);
}

int
strspn_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	size_t	val;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		val = strspn(VSTH_STRSPN_S1, VSTH_STRSPN_S2);
		if ( val != 9 )
		{
			++fails;
		}
		val = strspn(VSTH_STRSPN_S1, VSTH_STRSPN_S3);
		if ( val != 8 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strspn_ef()
{
	return(0);
}

/****** Testset for strstr()					 ******/

#ifdef	strstr
#undef	strstr
#endif

#define	VSTH_STRSTR_S1	"098734kjndv^*%4jhg(*kj231984y"
#define	VSTH_STRSTR_S2	"jndv^*"
#define	VSTH_STRSTR_S3	"%4jhg"

int
strstr_sf()
{
	return(0);
}

int
strstr_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	*p;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		p = strstr(VSTH_STRSTR_S1, VSTH_STRSTR_S2);

		vsth_ec();
		if ((p == NULL) || (strncmp(p, VSTH_STRSTR_S2, strlen(VSTH_STRSTR_S2))))
		{
			++fails;
		}
		vsth_xc();

		p = strstr(VSTH_STRSTR_S1, VSTH_STRSTR_S3);

		vsth_ec();
		if ((p == NULL) || (strncmp(p, VSTH_STRSTR_S3, strlen(VSTH_STRSTR_S3))))
		{
			++fails;
		}
		vsth_xc();
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strstr_ef()
{
	return(0);
}

/****** Testset for strtod()					 ******/

#ifdef	strtod
#undef	strtod
#endif

#define	VSTH_STRTOD_S1	"1.1e3"
#define	VSTH_STRTOD_S2	"2.2e4"

int	Vsth_strtod_init = 0;
double	Vsth_strtod_val1;
double	Vsth_strtod_val2;

int
strtod_sf()
{
	if ( ! Vsth_strtod_init )
	{
		Vsth_strtod_val1 = strtod(VSTH_STRTOD_S1, NULL);
		Vsth_strtod_val2 = strtod(VSTH_STRTOD_S2, NULL);
		Vsth_strtod_init = 1;
	}

	return(0);
}

int
strtod_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	double	val;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		val = strtod(VSTH_STRTOD_S1, NULL);
		if ( val != Vsth_strtod_val1 )
		{
			++fails;
		}

		val = strtod(VSTH_STRTOD_S2, NULL);
		if ( val != Vsth_strtod_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strtod_ef()
{
	return(0);
}

/****** Testset for strtok()					 ******/

#ifdef	strtok
#undef	strtok
#endif

int
strtok_sf()
{
	return(0);
}

int
strtok_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* strtok is not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strtok_ef()
{
	return(0);
}

#if	defined(_POSIX_THREAD_SAFE_FUNCTIONS)

/****** Testset for strtok_r()					 ******/

#ifdef	strtok_r
#undef	strtok_r
#endif

int
strtok_r_sf()
{
	return(0);
}

int
strtok_r_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* tested in PTHR.os/all/strtok_r */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strtok_r_ef()
{
	return(0);
}

#endif

/****** Testset for strtol()					 ******/

#ifdef	strtol
#undef	strtol
#endif

#define	VSTH_STRTOL_S1	"-0x4a8c"
#define	VSTH_STRTOL_S2	"0110010101111100"

int	Vsth_strtol_init = 0;
long	Vsth_strtol_val1;
long	Vsth_strtol_val2;

int
strtol_sf()
{
	if ( ! Vsth_strtol_init )
	{
		Vsth_strtol_val1 = strtol(VSTH_STRTOL_S1, NULL, 0);
		Vsth_strtol_val2 = strtol(VSTH_STRTOL_S2, NULL, 2);
		Vsth_strtol_init = 1;
	}
	return(0);
}

int
strtol_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	long	val;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		val = strtol(VSTH_STRTOL_S1, NULL, 0);
		if ( val != Vsth_strtol_val1 )
		{
			++fails;
		}

		val = strtol(VSTH_STRTOL_S2, NULL, 2);
		if ( val != Vsth_strtol_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strtol_ef()
{
	return(0);
}

/****** Testset for strtoul()					 ******/

#ifdef	strtoul
#undef	strtoul
#endif

#define	VSTH_STRTOUL_S1	"0xa48c"
#define	VSTH_STRTOUL_S2	"0010010101111101"

int		Vsth_strtoul_init = 0;
unsigned long	Vsth_strtoul_val1;
unsigned long	Vsth_strtoul_val2;

int
strtoul_sf()
{
	if ( ! Vsth_strtoul_init )
	{
		Vsth_strtoul_val1 = strtol(VSTH_STRTOUL_S1, NULL, 0);
		Vsth_strtoul_val2 = strtol(VSTH_STRTOUL_S2, NULL, 2);
		Vsth_strtoul_init = 1;
	}
	return(0);
}

int
strtoul_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	long	val;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		val = strtoul(VSTH_STRTOUL_S1, NULL, 0);
		if ( val != Vsth_strtoul_val1 )
		{
			++fails;
		}

		val = strtoul(VSTH_STRTOUL_S2, NULL, 2);
		if ( val != Vsth_strtoul_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strtoul_ef()
{
	return(0);
}

/****** Testset for strxfrm()					 ******/

#ifdef	strxfrm
#undef	strxfrm
#endif

#define	VSTH_STRXFRM_S1	"..................8...."
#define	VSTH_STRXFRM_S2	"....9...."

int	Vsth_strxfrm_init = 0;
int	Vsth_strxfrm_val1;
int	Vsth_strxfrm_val2;

int
strxfrm_sf()
{
	if ( ! Vsth_strxfrm_init )
	{
		memset(&Vsth_buf1[0], 0, VSTH_BUFLEN);
		memset(&Vsth_buf2[0], 0, VSTH_BUFLEN);

		Vsth_strxfrm_val1 = strxfrm(&Vsth_buf1[0], VSTH_STRXFRM_S1, VSTH_BUFLEN - 1);
		Vsth_strxfrm_val2 = strxfrm(&Vsth_buf2[0], VSTH_STRXFRM_S2, VSTH_BUFLEN - 1);
		Vsth_strxfrm_init = 1;
	}
	return(0);
}

int
strxfrm_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	size_t	val;
	char	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	for( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		memset(buf, 0, VSTH_BUFLEN);

		val = strxfrm(&buf[0], VSTH_STRXFRM_S1, VSTH_BUFLEN - 1);
		if ( val != Vsth_strxfrm_val1 )
		{
			++fails;
		}

		vsth_ec();
		if ( strncmp(&buf[0], &Vsth_buf1[0], val))
		{
			++fails;
		}
		vsth_xc();

		memset(buf, 0, VSTH_BUFLEN);

		val = strxfrm(&buf[0], VSTH_STRXFRM_S2, VSTH_BUFLEN - 1);
		if ( val != Vsth_strxfrm_val2 )
		{
			++fails;
		}

		vsth_ec();
		if ( strncmp(&buf[0], &Vsth_buf2[0], val))
		{
			++fails;
		}
		vsth_xc();
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
strxfrm_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for swab()					 ******/

#ifdef	swab
#undef	swab
#endif

#define		VSTH_SWAB_S1	"oiqer09813jna9"
#define		VSTH_SWAB_S2	"0983174kjnads08f7y13"

int		Vsth_swab_init = 0;
int		Vsth_len1;
int		Vsth_len2;

int
swab_sf()
{
	if ( ! Vsth_swab_init )
	{
		memset(&Vsth_buf1[0], 0, VSTH_BUFLEN);
		memset(&Vsth_buf2[0], 0, VSTH_BUFLEN);

		Vsth_len1 = strlen(VSTH_SWAB_S1);
		Vsth_len2 = strlen(VSTH_SWAB_S2);

		swab(VSTH_SWAB_S1, &Vsth_buf1[0], Vsth_len1);
		swab(VSTH_SWAB_S2, &Vsth_buf2[0], Vsth_len2);

		Vsth_swab_init = 1;
	}
	return(0);
}

int
swab_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	buf[VSTH_BUFLEN];

	errs	= 0;
	fails	= 0;

	for( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		memset(&buf[0], 0, VSTH_BUFLEN);
		swab(VSTH_SWAB_S1, &buf[0], Vsth_len1);
		vsth_ec();
		if ( strncmp(&Vsth_buf1[0], &buf[0], Vsth_len1))
		{
			++fails;
		}
		vsth_xc();

		memset(&buf[0], 0, VSTH_BUFLEN);
		swab(VSTH_SWAB_S2, &buf[0], Vsth_len2);
		vsth_ec();
		if ( strncmp(&Vsth_buf2[0], &buf[0], Vsth_len2))
		{
			++fails;
		}
		vsth_xc();
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
swab_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for swapcontext()				 ******/

#ifdef	swapcontext
#undef	swapcontext
#endif

int
swapcontext_sf()
{
	return(0);
}

int
swapcontext_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
swapcontext_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for swprintf()					 ******/

#ifdef	swprintf
#undef	swprintf
#endif

int
swprintf_sf()
{
	return(0);
}

int
swprintf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
swprintf_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for swscanf()					 ******/

#ifdef	swscanf
#undef	swscanf
#endif

int
swscanf_sf()
{
	return(0);
}

int
swscanf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
swscanf_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for symlink()					 ******/

#ifdef	symlink
#undef	symlink
#endif

int
symlink_sf()
{
	return(0);
}

int
symlink_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
symlink_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for sync()					 ******/

#ifdef	sync
#undef	sync
#endif

int
sync_sf()
{
	return(0);
}

int
sync_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sync_ef()
{
	return(0);
}

#endif

/****** Testset for sysconf()					 ******/

#ifdef	sysconf
#undef	sysconf
#endif

int
sysconf_sf()
{
	return(0);
}

int
sysconf_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
sysconf_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for syslog()					 ******/

#ifdef	syslog
#undef	syslog
#endif

int
syslog_sf()
{
	return(0);
}

int
syslog_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
syslog_ef()
{
	return(0);
}

#endif

/****** Testset for system()					 ******/

#ifdef	system
#undef	system
#endif

int
system_sf()
{
	return(0);
}

int
system_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
system_ef()
{
	return(0);
}

