
/*************************************************************************
*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)vsth_rtt.c	1.3
*
*************************************************************************/

#include "std.h"

#include <ctype.h>
#include <dirent.h>
#include <math.h>

#if	( TEST_XPG > 4 )
#include <search.h>
#endif

#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/times.h>
#include <sys/types.h>
#include <termios.h>
#include <time.h>
#include <unistd.h>

#if	( TEST_XPG > 4 )
#include <wctype.h>
#endif

#include "tet_api.h"
#include "vsth/vsth.h"
#include "vsth/vsth_fw.h"

/****** Testset for tan()					 ******/

#ifdef	tan
#undef	tan
#endif

#define	VSTH_TAN_VAL1		-0.2
#define	VSTH_TAN_VAL2		0.5
int	Vsth_tan_init = 0;
double	Vsth_tan_val1;
double	Vsth_tan_val2;

int
tan_sf()
{
	if ( ! Vsth_tan_init )
	{
		Vsth_tan_val1 = tan(VSTH_TAN_VAL1);
		Vsth_tan_val2 = tan(VSTH_TAN_VAL2);
		Vsth_tan_init = 1;
	}
		
	return(0);
}

int
tan_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = tan(VSTH_TAN_VAL1);
		if ( rval != Vsth_tan_val1 )
		{
			++fails;
		}

		rval = tan(VSTH_TAN_VAL2);
		if ( rval != Vsth_tan_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tan_ef()
{
	return(0);
}

/****** Testset for tanh()					 ******/

#ifdef	tanh
#undef	tanh
#endif

#define	VSTH_TANH_VAL1		-0.2
#define	VSTH_TANH_VAL2		0.5
int	Vsth_tanh_init = 0;
double	Vsth_tanh_val1;
double	Vsth_tanh_val2;

int
tanh_sf()
{
	if ( ! Vsth_tanh_init )
	{
		Vsth_tanh_val1 = tanh(VSTH_TANH_VAL1);
		Vsth_tanh_val2 = tanh(VSTH_TANH_VAL2);
		Vsth_tanh_init = 1;
	}
		
	return(0);
}

int
tanh_tf()
{
	int	cnt;
	double	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = tanh(VSTH_TANH_VAL1);
		if ( rval != Vsth_tanh_val1 )
		{
			++fails;
		}

		rval = tanh(VSTH_TANH_VAL2);
		if ( rval != Vsth_tanh_val2 )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tanh_ef()
{
	return(0);
}

/****** Testset for tcdrain()					 ******/

#ifdef	tcdrain
#undef	tcdrain
#endif

int
tcdrain_sf()
{
	return(0);
}

int
tcdrain_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tcdrain_ef()
{
	return(0);
}

/****** Testset for tcflow()					 ******/

#ifdef	tcflow
#undef	tcflow
#endif

int
tcflow_sf()
{
	return(0);
}

int
tcflow_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tcflow_ef()
{
	return(0);
}

/****** Testset for tcflush()					 ******/

#ifdef	tcflush
#undef	tcflush
#endif

int
tcflush_sf()
{
	return(0);
}

int
tcflush_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tcflush_ef()
{
	return(0);
}

/****** Testset for tcgetattr()					 ******/

#ifdef	tcgetattr
#undef	tcgetattr
#endif

int
tcgetattr_sf()
{
	return(0);
}

int
tcgetattr_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tcgetattr_ef()
{
	return(0);
}

#if	defined(_POSIX_JOB_CONTROL)

/****** Testset for tcgetpgrp()					 ******/

#ifdef	tcgetpgrp
#undef	tcgetpgrp
#endif

int
tcgetpgrp_sf()
{
	return(0);
}

int
tcgetpgrp_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tcgetpgrp_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for tcgetsid()					 ******/

#ifdef	tcgetsid
#undef	tcgetsid
#endif

int
tcgetsid_sf()
{
	return(0);
}

int
tcgetsid_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tcgetsid_ef()
{
	return(0);
}

#endif

/****** Testset for tcsendbreak()				 ******/

#ifdef	tcsendbreak
#undef	tcsendbreak
#endif

int
tcsendbreak_sf()
{
	return(0);
}

int
tcsendbreak_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tcsendbreak_ef()
{
	return(0);
}

/****** Testset for tcsetattr()					 ******/

#ifdef	tcsetattr
#undef	tcsetattr
#endif

int
tcsetattr_sf()
{
	return(0);
}

int
tcsetattr_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tcsetattr_ef()
{
	return(0);
}

#if	defined(_POSIX_JOB_CONTROL)

/****** Testset for tcsetpgrp()					 ******/

#ifdef	tcsetpgrp
#undef	tcsetpgrp
#endif

int
tcsetpgrp_sf()
{
	return(0);
}

int
tcsetpgrp_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tcsetpgrp_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for tdelete()					 ******/

#ifdef	tdelete
#undef	tdelete
#endif

int
tdelete_sf()
{
	return(0);
}

int
tdelete_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tdelete_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for telldir()					 ******/

#ifdef	telldir
#undef	telldir
#endif

int
telldir_sf()
{
	return(0);
}

int
telldir_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
telldir_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for tempnam()					 ******/

#ifdef	tempnam
#undef	tempnam
#endif

int
tempnam_sf()
{
	return(0);
}

int
tempnam_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tempnam_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for tfind()					 ******/

#ifdef	tfind
#undef	tfind
#endif

int
tfind_sf()
{
	return(0);
}

int
tfind_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tfind_ef()
{
	return(0);
}

#endif

/****** Testset for time()					 ******/

#ifdef	time
#undef	time
#endif

int
time_sf()
{
	return(0);
}

int
time_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
time_ef()
{
	return(0);
}

#if	defined(_POSIX_TIMERS)

/****** Testset for timer_create()				 ******/

#ifdef	timer_create
#undef	timer_create
#endif

int
timer_create_sf()
{
	return(0);
}

int
timer_create_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
timer_create_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_TIMERS)

/****** Testset for timer_delete()				 ******/

#ifdef	timer_delete
#undef	timer_delete
#endif

int
timer_delete_sf()
{
	return(0);
}

int
timer_delete_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
timer_delete_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_TIMERS)

/****** Testset for timer_getoverrun()				 ******/

#ifdef	timer_getoverrun
#undef	timer_getoverrun
#endif

int
timer_getoverrun_sf()
{
	return(0);
}

int
timer_getoverrun_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
timer_getoverrun_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_TIMERS)

/****** Testset for timer_gettime()				 ******/

#ifdef	timer_gettime
#undef	timer_gettime
#endif

int
timer_gettime_sf()
{
	return(0);
}

int
timer_gettime_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
timer_gettime_ef()
{
	return(0);
}

#endif

#if	defined(_POSIX_TIMERS)

/****** Testset for timer_settime()				 ******/

#ifdef	timer_settime
#undef	timer_settime
#endif

int
timer_settime_sf()
{
	return(0);
}

int
timer_settime_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
timer_settime_ef()
{
	return(0);
}

#endif

/****** Testset for times()					 ******/

#ifdef	times
#undef	times
#endif

int
times_sf()
{
	return(0);
}

int
times_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
times_ef()
{
	return(0);
}

/****** Testset for tmpfile()					 ******/

#ifdef	tmpfile
#undef	tmpfile
#endif

int
tmpfile_sf()
{
	return(0);
}

int
tmpfile_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tmpfile_ef()
{
	return(0);
}

/****** Testset for tmpnam()					 ******/

#ifdef	tmpnam
#undef	tmpnam
#endif

int
tmpnam_sf()
{
	return(0);
}

int
tmpnam_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tmpnam_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for toascii()					 ******/

#ifdef	toascii
#undef	toascii
#endif

int
toascii_sf()
{
	return(0);
}

int
toascii_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = toascii(0x89);
		if ( rval != 0x09 )
		{
			++errs;
		}
	
		rval = toascii(0xff);
		if ( rval != 0x7f )
		{
			++errs;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
toascii_ef()
{
	return(0);
}

#endif

/****** Testset for tolower()					 ******/

#ifdef	tolower
#undef	tolower
#endif

int
tolower_sf()
{
	return(0);
}

int
tolower_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = tolower('A');
		if ( rval != 'a' )
		{
			++fails;
		}

		rval = tolower('Z');
		if ( rval != 'z' )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tolower_ef()
{
	return(0);
}

/****** Testset for toupper()					 ******/

#ifdef	toupper
#undef	toupper
#endif

int
toupper_sf()
{
	return(0);
}

int
toupper_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		rval = toupper('a');
		if ( rval != 'A' )
		{
			++fails;
		}
	
		rval = toupper('z');
		if ( rval != 'Z' )
		{
			++fails;
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
toupper_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )

/****** Testset for towctrans()					 ******/

#ifdef	towctrans
#undef	towctrans
#endif

int
towctrans_sf()
{
	return(0);
}

int
towctrans_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	mbs[2];
	wchar_t	wc1;
	wint_t	wc2;
	wint_t	wc3;

	errs	= 0;
	fails	= 0;
	mbs[1] = '\0';

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		mbs[0] = cnt;

		rval = mbtowc( &wc1, mbs, VSTH_MBTOWC_MAXBYTES );
		if ( rval >= 0 )
		{
			if (iswupper(wc1))
			{
				wc2 = towctrans((wint_t) wc1, wctrans("tolower"));
				vsth_ec();
				wc3 = towupper(wc2);
				vsth_xc();
				if ( wc3 != (wint_t) wc1 )
				{
					++fails;
				}
			}
		}
		else
		{
			tet_infoline("mbtowc failed");
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
towctrans_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for towlower()					 ******/

#ifdef	towlower
#undef	towlower
#endif

int
towlower_sf()
{
	return(0);
}

int
towlower_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	mbs[2];
	wchar_t	wc1;
	wint_t	wc2;
	wint_t	wc3;

	errs	= 0;
	fails	= 0;
	mbs[1] = '\0';

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		mbs[0] = cnt;

		rval = mbtowc( &wc1, mbs, VSTH_MBTOWC_MAXBYTES );
		if ( rval >= 0 )
		{
			if (iswupper(wc1))
			{
				wc2 = towlower((wint_t) wc1);
				vsth_ec();
				wc3 = towupper(wc2);
				vsth_xc();
				if ( wc3 != (wint_t) wc1 )
				{
					++fails;
				}
			}
		}
		else
		{
			tet_infoline("mbtowc failed");
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
towlower_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for towupper()					 ******/

#ifdef	towupper
#undef	towupper
#endif

int
towupper_sf()
{
	return(0);
}

int
towupper_tf()
{
	int	cnt;
	int	rval;
	int	errs;
	int	fails;
	char	mbs[2];
	wchar_t	wc1;
	wint_t	wc2;
	wint_t	wc3;

	errs	= 0;
	fails	= 0;
	mbs[1] = '\0';

	for ( cnt = 0; cnt < Vsth_rt_cnt; ++cnt )
	{
		mbs[0] = cnt;

		rval = mbtowc( &wc1, mbs, VSTH_MBTOWC_MAXBYTES );
		if ( rval >= 0 )
		{
			if (iswlower(wc1))
			{
				wc2 = towupper((wint_t) wc1);
				vsth_ec();
				wc3 = towlower(wc2);
				vsth_xc();
				if ( wc3 != (wint_t) wc1 )
				{
					++fails;
				}
			}
		}
		else
		{
			tet_infoline("mbtowc failed");
		}
	}

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if ((errs == 0) && (fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
towupper_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for truncate()					 ******/

#ifdef	truncate
#undef	truncate
#endif

int
truncate_sf()
{
	return(0);
}

int
truncate_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
truncate_ef()
{
	return(0);
}

#endif

#if	( TEST_XPG > 4 )

/****** Testset for tsearch()					 ******/

#ifdef	tsearch
#undef	tsearch
#endif

int
tsearch_sf()
{
	return(0);
}

int
tsearch_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tsearch_ef()
{
	return(0);
}

#endif

/****** Testset for ttyname()					 ******/

#ifdef	ttyname
#undef	ttyname
#endif

int
ttyname_sf()
{
	return(0);
}

int
ttyname_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	/* not required to be thread safe */

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ttyname_ef()
{
	return(0);
}

/****** Testset for ttyname_r()				 ******/

#ifdef	ttyname_r
#undef	ttyname_r
#endif

int
ttyname_r_sf()
{
	return(0);
}

int
ttyname_r_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ttyname_r_ef()
{
	return(0);
}

#if	( TEST_XPG > 4 )
#if	defined(_XOPEN_LEGACY) && (_XOPEN_LEGACY != -1)

/****** Testset for ttyslot()					 ******/

#ifdef	ttyslot
#undef	ttyslot
#endif

int
ttyslot_sf()
{
	return(0);
}

int
ttyslot_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
ttyslot_ef()
{
	return(0);
}

#endif
#endif

#if	( TEST_XPG > 4 )

/****** Testset for twalk()					 ******/

#ifdef	twalk
#undef	twalk
#endif

int
twalk_sf()
{
	return(0);
}

int
twalk_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
twalk_ef()
{
	return(0);
}

#endif

/****** Testset for tzset()					 ******/

#ifdef	tzset
#undef	tzset
#endif

int
tzset_sf()
{
	return(0);
}

int
tzset_tf()
{
	int	rval;
	int	errs;
	int	fails;

	errs	= 0;
	fails	= 0;

	rval = 1;
	if ( fails )
	{
		rval = 2;
	}
	else if (( errs == 0 ) && ( fails == 0 ))
	{
		rval = 0;
	}
	return(rval);
}

int
tzset_ef()
{
	return(0);
}

