/*
*      SCCS:   @(#)common/vtools/chmog.c	4.2 (03/06/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior 
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		chmog.c - change mode, owner and group of files

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		main()

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	Mon Dec  7 16:04:13 1987
MODIFICATIONS:
		Peter Skan, 26 Sept 1991
			Add explicit privileges to list of setUID root
			programs and call prv_assign() as necessary
			to set them.

		Peter Skan, 1 Oct 1991
			Add programs needing PRV_DEVICE to list of
			privileged programs and check non-setUID programs
			against these entries.

		Peter Skan, 7 Oct 1991
			1. Perform chmod/chown before assigning privileges,
			   otherwise privileges may be lost on some systems.
			2. Correct privilege sets as indicated by trials
			3. Add missing ANSI.os/streamio entries to privileged
			   program list
			4. Add "unlink_t9" to privileged program list

		Peter Skan, 14 Oct 1991
			"T.creat_X" requires PRV_DEVICE
			"T.chown" requires PRV_ACCESS
			"T.chmod" requires PRV_ACCESS
		
		Geoff Clare, 3 Dec 1991
			Add privileged subprograms for new wide char and
			POSIX.2 tests to list.
		
		Geoff Clare, 18 Dec 1991
			Fix problem with use of NULL as integer.

		Peter Skan, 7 Feb 1992
			Fix privileges on ipc tests.
			Remove T.shmget from list: does not require privilege
		
		Geoff Clare, 11 Aug 1992
			Avoid storing modes in int variables.
			Translate octal modes to S_IRUSR, etc.
		
		Geoff Clare, 21 Oct 1992
			Extract proglist[] into separate files for client
			and server.
		
		Geoff Clare, 25 April 1994
			Change open() flag from 0 to O_RDONLY.

		Geoff Clare, 5 March 1997
			Added section name to privilege list.

************************************************************************/

#include <std.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <grp.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <setprv.h>
#include "./chmog.h"
#include <sysdep.h>

#define	IDENTICAL(A,B)	(A.st_dev==B.st_dev && A.st_ino==B.st_ino)
#define ISDIR(A)	(S_ISDIR(A.st_mode))
#define ISREG(A)	(S_ISREG(A.st_mode))

#define BLKSIZE	4096
#define	DELIM	'/'

mode_t	newmode();

char	*ms;
struct	stat	stbuf;
int	uid, gid;
int	mve, cpy;

char	*userlist[]  = { "root", "vsx0", "vsx1", "vsx2", NULL };
char	*grouplist[] = { "root", "vsxg0", "vsxg1", "vsxg2", NULL };

extern struct proglist proglist[];

main(argc,argv)
char **argv;
{
	int i;
	mode_t mode;
	char *p;
	int status = 0;
	char	*srcfile;
	
	mve = 0;
	cpy = 0;

	if (argc < 7) {
		(void) fprintf(stderr, "Usage: chmog mode owner group -[cm] source target ...\n");
		exit(255);
	}

	ms = argv[1];
	(void) newmode((mode_t)0);

	if (strcmp(argv[2], "-") == 0)
		uid = -1;
	else
		uid = checkuser(argv[2]);

	if (strcmp(argv[3], "-") == 0)
		gid = -1;
	else
		gid = checkgrp(argv[3]);

	if (strcmp(argv[4], "-m") == 0)
		mve++;
	else
		cpy++;

	for (i = 6; i < argc; i++) {
		ms = argv[1];
		srcfile = argv[5];
		p = argv[i];
		if (move(srcfile, p) != 0)
		{
			++status;
			continue;
		}
		if (stat(p, &stbuf) < 0) {
			(void) fprintf(stderr, "chmog: can't access file \"%s\"\n", p);
			++status;
			continue;
		}
		mode = newmode(stbuf.st_mode);
		if ((gid >= 0 || uid >= 0) && chown(p,
				(uid_t)(uid >= 0 ? uid : stbuf.st_uid),
				(gid_t)(gid >= 0 ? gid : stbuf.st_gid)) < 0) {
			(void) fprintf(stderr,
			    "chmog: can't change owner/group of \"%s\"\n", p);
			++status;
			continue;
		}
		if (chmod(p, mode) < 0) {
			(void) fprintf(stderr,
				"chmog: can't change mode of \"%s\"\n", p);
			++status;
			continue;
		}
		/*
		 * If program is set-UID root, find it in the list of
		 * privileged programs and assign the required privileges.
		 * Otherwise, check the list in case it needs just PRV_DEVICE
		 */
		if ((mode & S_ISUID) &&
				(uid == 0 || uid == -1 && stbuf.st_uid == 0)) {
			if (checksuprog(p) != 0) {
				++status;
				continue;
			}
		} else if (checkdevprog(p) != 0) {
			++status;
			continue;
		}

	}
	/* 
	 * Attempt to unlink the source if it was a move.
	 */
	 
	if (mve && unlink(srcfile) < 0) {
		(void) fprintf(stderr, "chmog : cannot unlink %s\n", srcfile);
		++status;
	}
	exit(status);

	/* NOTREACHED */
}

checkuser(user)
char *user;
{
	char	**p;
	struct passwd	*pwd, *getpwnam();

	if((pwd=getpwnam(user)) == NULL) {
		(void) fprintf(stderr, "chmog: unknown user id: %s\n", user);
		exit(255);
	}
	for (p = userlist; *p != NULL; p++)
		if (strcmp(*p, user) == 0)
			return pwd->pw_uid;

	(void) fprintf(stderr, "chmog: illegal user id: %s\n", user);
	exit(255);

	/* NOTREACHED */
}

checkgrp(grp)
char *grp;
{
	char	**p;
	struct group	*gr, *getgrnam();

	if((gr=getgrnam(grp)) == NULL) {
		(void) fprintf(stderr, "chmog: unknown group id: %s\n", grp);
		exit(255);
	}
	for (p = grouplist; *p != NULL; p++)
		if (strcmp(*p, grp) == 0)
			return gr->gr_gid;

	(void) fprintf(stderr, "chmog: illegal group id: %s\n", grp);
	exit(255);

	/* NOTREACHED */
}

checksuprog(file)
char *file;
{
	/*
	 * check against list of valid privileged programs
	 * and set the necessary privileges
	 */

	char *prog = NULL, *sect = NULL, *cp;
	struct proglist *p;
	int slashcnt = 0;

	cp = file + strlen(file);
	while (--cp > file) {
		if (cp[-1] == '/') {
			if (++slashcnt == 1)
			    prog = cp;
			else if (slashcnt == 4) {
			    sect = cp;
			    break;
			}
		}
	}

	if (sect == NULL) {
		(void) fprintf(stderr,
			"chmog: no section name in path \"%s\"\n", file);
		return -1;
	}

	for (p = proglist; p->sect != NULL; p++) {
		if (strncmp(p->sect, sect, strlen(p->sect)) == 0 &&
		    sect[strlen(p->sect)] == '/' &&
		    strcmp(p->name, prog) == 0) {
			if (prv_assign(file, p->privileges) != 0) {
				(void) fprintf(stderr,
					"chmog: %s: prv_assign() failed\n",
					prog);
				return -1;
			}
			return 0;
		}
	}
	
	(void) fprintf(stderr, "chmog: illegal mode for file \"%s\"\n", file);
	return -1;
}

checkdevprog(file)
char *file;
{
	/*
	 * check against list of valid programs requiring PRV_DEVICE only
	 * and set the necessary privileges
	 * (Programs needing other privileges in addition to PRV_DEVICE are
	 * handled by checksuprog() above)
	 */

	char *prog = NULL, *sect = NULL, *cp;
	struct proglist *p;
	int slashcnt = 0;

	cp = file + strlen(file);
	while (--cp > file) {
		if (cp[-1] == '/') {
			if (++slashcnt == 1)
			    prog = cp;
			else if (slashcnt == 4) {
			    sect = cp;
			    break;
			}
		}
	}

	if (sect == NULL) {
		(void) fprintf(stderr,
			"chmog: no section name in path \"%s\"\n", file);
		return -1;
	}
	
	for (p = proglist; p->sect != NULL; p++) {
		if (strncmp(p->sect, sect, strlen(p->sect)) == 0 &&
		    sect[strlen(p->sect)] == '/' &&
		    strcmp(p->name, prog) == 0) {
			if (p->privileges[0] == PRV_DEVICE &&
			    p->privileges[1] == 0) {
				if (prv_assign(file, p->privileges) != 0) {
					(void) fprintf(stderr,
	"chmog: %s: prv_assign() failed\n", prog);
					return -1;
				}
				return 0;
			} else {
				(void) fprintf(stderr,
	"chmog: %s: invalid privileges for non-SUID program\n", prog);
				return 0;
			}
		}
	}

	return 0;
}

mode_t
newmode(nm)
mode_t nm;
{
	int m;

	m = mval();
	if (*ms)
	{
		(void) fprintf(stderr, "chmog: mode must be octal\n");
		exit(255);
	}

	/* translate octal integer to mode_t */
	nm = 0;

	if (m & 04000)
		nm |= S_ISUID;
	if (m & 02000)
		nm |= S_ISGID;

	if (m & 0400)
		nm |= S_IRUSR;
	if (m & 0200)
		nm |= S_IWUSR;
	if (m & 0100)
		nm |= S_IXUSR;

	if (m & 040)
		nm |= S_IRGRP;
	if (m & 020)
		nm |= S_IWGRP;
	if (m & 010)
		nm |= S_IXGRP;

	if (m & 04)
		nm |= S_IROTH;
	if (m & 02)
		nm |= S_IWOTH;
	if (m & 01)
		nm |= S_IXOTH;

	return(nm);
}

mval()
{
	int c, i;

	i = 0;
	while ((c = *ms++) >= '0' && c <= '7')
		i = (i << 3) + (c - '0');
	ms--;
	return(i);
}

move(source, target)
char *source, *target;
{
	int last;
	char	*buf = (char *)NULL;
	int from, to, ct;
	char fbuf[BLKSIZE];
	struct stat s1, s2;
	
	/* 
	 * While source or target have trailing 
	 * DELIM (/), remove them (unless only "/")
	 */

	while (((last = strlen(source)) > 1)
	    &&  (source[last-1] == DELIM))
		 source[last-1]= '\0';
	
	while (((last = strlen(target)) > 1)
	    &&  (target[last-1] == DELIM))
		 target[last-1]= '\0';
	
	/*
	 * Make sure source file exists.
	 */
	 
	if (stat(source, &s1) < 0) {
		(void) fprintf(stderr, "chmog: cannot access %s\n", source);
		return(1);
	}
	
	/* 
	 * Make sure source file is not a directory,
	 * we don't move() directories...
	 */
	
	if (ISDIR(s1)) {
		return(0);
	}

	/*
	 * Make sure target file is not a directory,
	 */

	if (stat(target, &s2) == 0 && ISDIR(s2)) {
		(void) fprintf(stderr, "chmog : <%s> directory\n", target);
		return(1);
	}
	
	/*
	 * If stat fails, then the target doesn't exist,
	 * we will create a new target with default file type of regular.
 	 */	
	
	if (stat(target, &s2) >= 0) {
		
		/*
		 * If filenames for the source and target are
		 * the same and the inodes are the same, it is
		 * an error.

		if (IDENTICAL(s1,s2)) {
			(void) fprintf(stderr, "chmog : %s and %s are identical\n", source, target);
			if (buf != NULL)
				free((void *)buf);
			return(1);
		}
		 */
			
	}

	/* 
	 * Attempt to open source for copy.
	 */
	 
	if((from = open(source, O_RDONLY)) < 0) {
		(void) fprintf(stderr, "chmog : cannot open %s\n", source);
		if (buf != NULL)
			free((void *)buf);
		return (1);
	}
	
	/* 
	 * Attempt to create a target.
	 */
	
	if((to = creat (target, 0666)) < 0) {
		(void) fprintf(stderr, "chmog : cannot create %s\n", target);
		if (buf != NULL)
			free((void *)buf);
		(void) close(from);
		return (1);
	}
		
	/*
	 * Block copy from source to target.
	 */
		 
	while((ct = read(from, fbuf, BLKSIZE)) != 0)
		if(ct < 0 || write(to, fbuf, (unsigned)ct) != ct) {
			(void) fprintf(stderr, "chmog : bad copy to %s\n", target);
			/*
			 * If target is a regular file,
			 * unlink the bad file.
			 */
			 
			if (ISREG(s2))
				(void) unlink(target);
			if (buf != NULL)
				free((void *)buf);
			return (1);
		}

	(void) close(from);
	(void) close(to);
	
	if (buf != NULL)
		free((void *)buf);
	return(0);
}
