/*
 *      SCCS:   @(#)common/vtools/cs_arc.c	4.1 (01/21/97)  VSXgen release 1.4
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1991 X/Open Company Limited
 * 
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 * 
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
static	char sccsid[] = "@(#)common/vtools/cs_arc VSXgen release 1.4";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif

/***********************************************************************

NAME:           cs_arc.c - Create and extract files from a C source archive

PROJECT:        VSX (X/OPEN Validation Suite)

AUTHORS:        Mike Rendell, UniSoft Ltd.

************************************************************************/
#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <ts_arc.h>
#include <string.h>
#include <sysdep.h>

#ifndef	BUFSIZ
#define	BUFSIZ	1024
#endif	/* BUFSIZ */

#define	F_CREAT		0
#define	F_EXTRACT	1

extern	char	*get_pname();
extern	void	set_pname();
extern	char	*errname();
extern	char	*arc_errname();

private	int	creat_arc();
private	int	extract_arc();
private	int	append();
private	int	my_read();
private	void	my_lseek();
private	void	addline();
private	void	my_write();
private	void	nomem();
private	void	usage();

main(argc, argv)
	int	argc;
	char	*argv[];
{
	int		(*func)();

	set_pname(argv[0] == (char *) NULL ? "cs_arc" : argv[0]);

	if (argc < 3)
		usage();

	switch (argv[1][0])
	{
	case 'c':
		func = creat_arc;
		break;

	case 'x':
		func = extract_arc;
		break;

	default:
		usage();
	}

	(*func)(argc - 3, argv + 3, argv[2]);

	exit(0);
	/* for lint */
	return 0;
}


private void
usage()
{
	(void) fprintf(stderr, "Usage: %s c L.<arc-name> files\n", get_pname());
	(void) fprintf(stderr, "	or\n");
	(void) fprintf(stderr, "Usage: %s x L.<arc-name>\n", get_pname());
	exit(1);
}

private void
nomem()
{
	(void) fprintf(stderr, "%s: malloc() failed - ran out of memory\n",
		get_pname());
	(void) fprintf(stderr, "exiting\n");
	exit(1);
}

private int
append(sfd, sname, dfd, dname)
	int	sfd;		/* source */
	char	*sname;
	int	dfd;		/* destination */
	char	*dname;
{
	int		nread;
	int		tnread;
	static	char	buf[BUFSIZ];

	tnread = 0;
	while (1)
	{
		nread = my_read(sfd, buf, sizeof(buf), sname);
		if (nread == 0)
			break;
		my_write(dfd, buf, (unsigned) nread, dname);
		tnread += nread;
	}

	return tnread;
}

private	void
addline(arc_ent, name, filesize)
	char	*arc_ent;
	char	*name;
	int	filesize;
{
	char	*s;

	s = arc_ent;				/* copy name in */
	while (*name != NUL)
		*s++ = *name++;

	s = &arc_ent[TS_NAMELEN + TS_SEEKLEN - 1];	/* copy seek value in */
	while (filesize != 0)
	{
		*s-- = (filesize % 10) + '0';
		filesize /= 10;
	}
}

private void
my_lseek(fd, offset, whence)
	int	fd;
	off_t	offset;
	int	whence;
{
	if (lseek(fd, offset, whence) == (off_t) SYSERROR)
	{
		(void) fprintf(stderr, "%s: lseek() failed: %s\n",
			get_pname(), errname(errno));
		(void) fprintf(stderr, "exiting\n");
		exit(2);
	}

	return;
}


private void
my_write(fd, buf, nbytes, fname)
	int		fd;
	char		*buf;
	unsigned	nbytes;
	char		*fname;
{
	int	nwrite;

	nwrite = write(fd, buf, nbytes);
	if (nwrite == SYSERROR)
	{
		(void) fprintf(stderr, "%s: error writing()ing to %s: %s\n",
			get_pname(), fname, errname(errno));
		(void) fprintf(stderr, "exiting\n");
		exit(2);
	}
	else if (nwrite != nbytes)
	{
		(void) fprintf(stderr,
			"%s: could not write whole buffer to %s\n",
			get_pname(), fname);
		(void) fprintf(stderr, "exiting\n");
		exit(2);
	}
}


private int
my_read(fd, buf, nbytes, fname)
	int		fd;
	char		*buf;
	unsigned	nbytes;
	char		*fname;
{
	int	nread;

	nread = read(fd, buf, nbytes);
	if (nread == SYSERROR)
	{
		(void) fprintf(stderr, "%s: error read()ing from %s: %s\n",
			get_pname(), fname, errname(errno));
		(void) fprintf(stderr, "exiting\n");
		exit(2);
	}
	return nread;
}


private int
creat_arc(argc, argv, aname)
	int	argc;
	char	*argv[];
	char	*aname;
{
	int		afd;
	int		fd;
	int		i;
	int		j;
	int		filesize;
	int		nfiles;
	char		headbuf[TS_HEADSIZE - TS_MAGICSIZE];
	char		**arc_dir;

	afd = open(aname, O_WRONLY|O_TRUNC|O_CREAT, S_IRWXU|S_IRWXG);
	if (afd == SYSERROR)
	{
		(void) fprintf(stderr, "%s: %s: %s\n",
			get_pname(), aname, errname(errno));
		exit(2);
	}
	nfiles = 0;
	for (i = 0 ; i < argc ; i++)
		if (access(argv[i], R_OK) == SYSERROR)
		{
			argv[i][0] = NUL;
			(void) fprintf(stderr, "%s: %s: %s\n",
				get_pname(), argv[i], errname(errno));
		}
		else
			nfiles++;
	if (nfiles == 0)
	{
		(void) fprintf(stderr, "%s: could not open any files\n",
			get_pname());
		(void) close(afd);
		exit(2);
	}

	arc_dir = (char **) malloc((size_t) (sizeof(char *) * nfiles));
	if (arc_dir == (char **) NULL)
		nomem();
	for (i = 0 ; i < nfiles ; i++)
	{
		arc_dir[i] = (char *)malloc((size_t)(TS_NAMELEN + TS_SEEKLEN + 2));
		if (arc_dir[i] == (char *) NULL)
			nomem();
		for (j = 0 ; j < TS_NAMELEN + TS_SEEKLEN ; j++)
			arc_dir[i][j] = ' ';
		arc_dir[i][j++] = '\n';
		arc_dir[i][j] = NUL;
	}
	my_lseek(afd, (off_t) (nfiles * (TS_NAMELEN + TS_SEEKLEN + 1)
		+ TS_HEADSIZE), SEEK_SET);
	for (i = 0 ; i < nfiles ; i++, argv++)
	{
		while (*argv != (char *) NULL && **argv == NUL)
			argv++;
		if (*argv == (char *) NULL)
			break;
		if ((fd = open(*argv, O_RDONLY, 0)) == SYSERROR)
		{
			(void) fprintf(stderr, "%s: open(%s) failed - %s\n",
				get_pname(), *argv, errname(errno));
			addline(arc_dir[1], "", 0);
			continue;
		}
		for (j = strlen(*argv) - 1 ; j > 0 && argv[0][j] != '/' ; j--)
			;
		if (argv[0][j] == '/')
			j++;
		filesize = append(fd, *argv, afd, aname);
		addline(arc_dir[i], &(argv[0][j]), filesize);
		(void) close(fd);
	}
	my_lseek(afd, (off_t) 0, SEEK_SET);
	my_write(afd, TS_MAGIC, TS_MAGICSIZE, aname);
	headbuf[0] = nfiles / 100 + '0';
	headbuf[1] = (nfiles % 100) / 10 + '0';
	headbuf[2] = nfiles % 10 + '0';
	headbuf[3] = '\n';
	my_write(afd, headbuf, TS_HEADSIZE - TS_MAGICSIZE, aname);
	for (i = 0 ; i < nfiles ; i++)
		my_write(afd, arc_dir[i], TS_NAMELEN + TS_SEEKLEN + 1, aname);
	(void) close(afd);
	return 0;
}


private int
extract_arc(argc, argv, aname)
	int	argc;
	char	*argv[];
	char	*aname;
{
	char	**files;
	char	*dir;

	dir = argc > 0 ? argv[0] : ".";

	if (arc_extr(aname, dir, &files) == SYSERROR)
		(void) fprintf(stderr, "%s: error extracting files: %s\n",
			get_pname(), arc_errname(aname));
	return 0;
}
