/*
*      SCCS:   @(#)common/wchars(wcc.c)	4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/


/***********************************************************************

NAME:		wcc.c - Wide Character Configuration tool

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	the wcc tool
SYNOPSIS:
	wcc <input file> <output file>
DESCRIPTION:
	wcc reads the input file checking its conformance to the wide 
	character configuration file.
	The wcc tool supports all the functionality described in the wide
	character configuration file template.
	Checks are made to make sure that the format of the entries are valid
	and that every parameter appears once and only once.
	The binary output file is prepared for processing by wcharcfg().


NOTE:

AUTHOR:		David G. Sawyer, UniSoft Ltd.
DATE CREATED:	Fri Dec 06 21:20:06 GMT 1991
MODIFICATIONS:
		Substituted CONVERT_CHECK macro into do_mbchar()
		Put special case in what_value() for empty PROPERTIES line.
		Changed fatal errors in several places into warnings - as a
		result made return value of do_*() called from what_value()
		return int.
		Changed error messages in some of do_*().
		Combined do_mbpairs_x10() and do_mbpairs_x3() into do_mbpairs().
		Restructured convert_field().
		Added checking for all fields where possible = lots of changes.
		Altered synopsis statement.

		David G Sawyer, UniSoft Ltd.
		10th - 16th Januray 1992.

		Geoff Clare, 24 Mar 1992
			Use wctomb() and MB_CUR_MAX in correct locales.

		Geoff Clare, 3-11 June 1992
			Ignore collation and wcsftime()-related parameters
			if ENHANCED I18N not supported.  Include lead
			sequence in size of MBCURMAX[12] parameters.
			Allow all zeros in WIDTHx parameters.

		Geoff Clare, 18 June 1992
			Allow decimal point characters to be single byte.

		Geoff Clare, 29 Sept 1992
			If ENHANCED I18N group not supported, must still
			compile parameters because individual functions
			that use them may be supported.

		Geoff Clare, 24 June 1994
			Null terminate wctomb() output.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <wchar.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <locale.h>
#include <string.h>
#include <errno.h>
#include <limits.h>
#include <wchar_cfg.h>
#include <sysdep.h>

#define COMMENT_CHAR	'#'
#define ASSIGN_CHAR	'='
#define PROPERTY	9999

#define CONVERT_CHECK(_addr, _size1, _size2, _size3)				\
		if (str1 != NULL)				\
			str1 = convert_field(str1, _size1, _size2, _size3);									\
		if ( str1 != NULL)				\
		{						\
			_addr = (unsigned char *) malloc( (size_t) (strlen(str1)+1)); 								\
			if ( _addr == NULL)			\
			{					\
				(void) fprintf( stderr, "malloc failed line %d\n", (__LINE__ - 3));						\
				exit( 1);			\
			}					\
			(void) strcpy((char *) _addr, str1);	\
		}						\
		else						\
			_addr = (unsigned char *)"";		\


#define	WRITE_CFG(_val)						\
		if( write(fileno(fptr), _val ? (char *) _val : "", \
			(unsigned int) (strlen(_val ? (char *) _val : "")+1)) == -1)						\
		{						\
			perror("write");			\
			exit(1);				\
		}						\



char	*what_line;	/* the line being currently parsed */
int	*check;		/* whether all the normal parameters appeared */
int	warning = 0;	/* whether or not any warnings have been given */

/* check that bytes are used consistently */
int	sizesupp[] = {-1, -1, -1, -1};

/* are 2, 3, 4 bytes chars used in the day and month for each locale */
int	mbstringset1[] = {0, 0, 0};
int	mbstringset2[] = {0, 0, 0};
int	stringset = 0;
int	curr_locale = 0;

void	parse_line(), make_cstr(), write_cfg();

int	check_param(), what_value(), do_localename(), do_anumber(), 
	do_mbchar(), do_mbchars(), do_mbpairs(), do_mbstrings(), 
	do_mbtriplets(), do_properties();

char	*convert_field();

struct wchar cfg;
struct wc_prop_names
{
	char 	*is;
	char 	*not;
	int	ischeck;
	int	notcheck;
} *prop_names;

struct {
	char *name;
	int enh_i18n;
} param_list[] = {
	"LOCALE1", 0, "LOCALE2", 0, "ISWCNTRL", 0, "NOTCNTRL", 0,
	"ISWGRAPH", 0, "NOTGRAPH", 0, "ISWLOWER", 0, "NOTLOWER", 0,
	"ISWPUNCT", 0, "NOTPUNCT", 0, "ISWSPACE", 0, "NOTSPACE", 0,
	"ISWUPPER", 0, "NOTUPPER", 0, "TOWUPPER", 0, "TOWLOWER", 0,
	"WIDTH0", 0, "WIDTH1", 0, "WIDTH2", 0, "WIDTH3", 0,
	"DONTCARE", 1, "PRIMARY", 1, "SECONDARY", 1, "PRIMARY2_1", 1,
	"SECONDARY2_1", 1, "EQUAL1_2", 1, "DECPOINT1", 0, "DECPOINT2", 0,
	"MONTH1_1", 1, "MONTH2_1", 1, "MONTH3_1", 1, "MONTH4_1", 1,
	"MONTH5_1", 1, "MONTH6_1", 1, "MONTH7_1", 1, "MONTH8_1", 1,
	"MONTH9_1", 1, "MONTH10_1", 1, "MONTH11_1", 1, "MONTH12_1", 1,
	"ABMON1_1", 1, "ABMON2_1", 1, "ABMON3_1", 1, "ABMON4_1", 1,
	"ABMON5_1", 1, "ABMON6_1", 1, "ABMON7_1", 1, "ABMON8_1", 1,
	"ABMON9_1", 1, "ABMON10_1", 1, "ABMON11_1", 1, "ABMON12_1", 1,
	"DAY1_1", 1, "DAY2_1", 1, "DAY3_1", 1, "DAY4_1", 1,
	"DAY5_1", 1, "DAY6_1", 1, "DAY7_1", 1, "ABDAY1_1", 1,
	"ABDAY2_1", 1, "ABDAY3_1", 1, "ABDAY4_1", 1, "ABDAY5_1", 1,
	"ABDAY6_1", 1, "ABDAY7_1", 1, "AM_STR1", 1, "PM_STR1", 1,
	"MONTH1_2", 1, "MONTH2_2", 1, "MONTH3_2", 1, "MONTH4_2", 1,
	"MONTH5_2", 1, "MONTH6_2", 1, "MONTH7_2", 1, "MONTH8_2", 1,
	"MONTH9_2", 1, "MONTH10_2", 1, "MONTH11_2", 1, "MONTH12_2", 1,
	"ABMON1_2", 1, "ABMON2_2", 1, "ABMON3_2", 1, "ABMON4_2", 1,
	"ABMON5_2", 1, "ABMON6_2", 1, "ABMON7_2", 1, "ABMON8_2", 1,
	"ABMON9_2", 1, "ABMON10_2", 1, "ABMON11_2", 1, "ABMON12_2", 1,
	"DAY1_2", 1, "DAY2_2", 1, "DAY3_2", 1, "DAY4_2", 1,
	"DAY5_2", 1, "DAY6_2", 1, "DAY7_2", 1, "ABDAY1_2", 1,
	"ABDAY2_2", 1, "ABDAY3_2", 1, "ABDAY4_2", 1, "ABDAY5_2", 1,
	"ABDAY6_2", 1, "ABDAY7_2", 1, "AM_STR2", 1, "PM_STR2", 1,
	"MBCURMAX1", 0, "MBCURMAX2", 0, "INVALID_MB", 0, "INVALID_WC", 0,
	"PROPERTIES", 0,
	NULL, 0
};

main( argc, argv)
int	argc;
char	**argv;
{
	FILE 	*fp;
	char	line[512];
	int	ctr, cnt, abortfl = 0;

	if( argc != 3)
	{
		(void) fprintf(stderr,"%s\n%s\n",
			"VSX Wide Character Configuration file parser.",
			"Synopsis: wcc <configuration file> <binary output file>");
		exit(2);
	}

	if((fp = fopen(argv[1], "r")) == (FILE *) NULL)
	{
		perror("Config File");
		(void) fprintf(stderr, "Couldn't open configuration file.\n");
		exit(2);
	}

	/* setup parameter appearance check list */
	for (ctr = 0; param_list[ctr].name != NULL; ctr++)
		;
	if( (check = (int *) malloc((size_t)(sizeof(int) * (ctr+1)))) == NULL)
	{
		perror("malloc");
		exit(2);
	}
	for( cnt = 0; cnt < ctr; cnt++)
	{
		check[cnt] = 0;
	}

	/* initialise properties field */
	cfg.properties = NULL;

	/* parse the config file and build up internal structure */
	while(fgets(line, sizeof(line), fp) != NULL)
	{
		parse_line(line);
	}

	/* check all the parameters appeared in the file */
	for( cnt = 0; cnt < ctr; cnt++)
	{
		if( check[cnt] != 1)
		{
			if( abortfl == 0)
				(void) fprintf(stderr,"The following parameters were missing from the configuration file :\n");
			(void) fprintf(stderr,"%s\n", param_list[cnt].name); 
			abortfl = 1;
		}
	}
	if( cfg.properties != NULL)
	{
		for( ctr = 0; cfg.properties[ctr].name != NULL; ctr++)
		{
			if( prop_names[ctr].ischeck != 1)
			{
				if( abortfl == 0)
					(void) fprintf(stderr,"The following parameters were missing from the configuration file :\n");
				(void) fprintf(stderr,"%s\n", prop_names[ctr].is); 
				abortfl = 1;
			}
			if( prop_names[ctr].notcheck != 1)
			{
				if( abortfl == 0)
					(void) fprintf(stderr,"The following parameters were missing from the configuration file :\n");
				(void) fprintf(stderr,"%s\n", prop_names[ctr].not); 
				abortfl = 1;
			}
		}
	}

	for(cnt = 1; cnt <= 3; cnt++)
	{
		if(sizesupp[cnt] == 1 && mbstringset1[cnt-1] == 0)
		{
			(void) fprintf(stderr, "%d byte character should have appeared in date strings for locale 1\n", cnt+1);
			warning = 1;
		}
		if(sizesupp[cnt] == 1 && mbstringset2[cnt-1] == 0)
		{
			(void) fprintf(stderr, "%d byte character should have appeared in date strings for locale 2\n", cnt+1);
			warning = 1;
		}
	}

	for(cnt = 0; cnt <= 11; cnt++)
	{
		if(! strcmp((char *) cfg.month1[cnt], (char *) cfg.month2[cnt]))
		{
			(void) fprintf(stderr, "The values of the parameters %s and %s must differ\n", param_list[28+cnt].name, param_list[68+cnt].name);
			warning = 1;
		}
		if(! strcmp((char *) cfg.abmon1[cnt], (char *) cfg.abmon2[cnt]))
		{
			(void) fprintf(stderr, "The values of the parameters %s and %s must differ\n", param_list[40+cnt].name, param_list[80+cnt].name);
			warning = 1;
		}
	}
	for(cnt = 0; cnt <= 6; cnt++)
	{
		if(! strcmp((char *) cfg.day1[cnt], (char *) cfg.day2[cnt]))
		{
			(void) fprintf(stderr, "The values of the parameters %s and %s must differ\n", param_list[52+cnt].name, param_list[92+cnt].name);
			warning = 1;
		}
		if(! strcmp((char *) cfg.abday1[cnt], (char *) cfg.abday2[cnt]))
		{
			(void) fprintf(stderr, "The values of the parameters %s and %s must differ\n", param_list[59+cnt].name, param_list[99+cnt].name);
			warning = 1;
		}
	}
	if(!strcmp((char *) cfg.am_str1, (char *) cfg.am_str2))
	{
		(void) fprintf(stderr, "The values of the parameters %s and %s must differ\n", param_list[66].name, param_list[106].name);
		warning = 1;
	}
	if(!strcmp((char *) cfg.pm_str1, (char *) cfg.pm_str2))
	{
		(void) fprintf(stderr, "The values of the parameters %s and %s must differ\n", param_list[67].name, param_list[107].name);
		warning = 1;
	}

	/* did the parameter check fail ? */
	if(abortfl)
		exit(1);

	/* Did any unexpected parameters appear in the file */
	if (warning)
	{
		(void) fprintf(stderr,"Generation of binary file aborted due to unresolved warnings.\n");
		exit(1);
	}

	/* write internal structure to the supplied filename */
	write_cfg(argv[2]);

	exit(EXIT_SUCCESS);
	/* NOTREACHED */
}


void parse_line( line)
char *line;
{
	char	str[512], *param, *value, *tmp_ptr;
	int	ctr, pnum;

	/* Get a copy of the line */
	(void) strcpy(str, line);

	/* remove trailing newline */
	str[strlen(str) - 1] = '\0';

	/* strip leading whitespace */
	for( ctr = 0; isspace((int) str[ctr]); ctr++);

	param = &(str[ctr]);

	/* If not an empty line  or a comment line */
	if( *param != '\0' && *param != COMMENT_CHAR)
	{
		/* <parameter name>=<value> */
		/* look for the assignment character */
		if((tmp_ptr = strchr(param, ASSIGN_CHAR)) != NULL)
		{
			/* value string starts just after assignment char */
			value = tmp_ptr+1;

			/* terminate the parameter name string */
			if(isspace((int) *(tmp_ptr - 1)))
			{
				/* strip off terminating whitespace */
				while(isspace((int) *(--tmp_ptr)));
				++tmp_ptr;
				*tmp_ptr = '\0';
			}
			else
				*tmp_ptr = '\0';

			/* check the parameter name is valid */
			if((pnum = check_param(param)) == -1)
			{
					(void) fprintf(stderr,"Warning : invalid parameter %s\n", param);
					warning = 1;
			}
			/* check the format of and assign the value */
			else if(what_value(pnum, value, param) < 0)
			{
				if( pnum != -2) /* warning not given already */
				{
					(void) fprintf(stderr,"Unable to determine parameter value in line: %s", line);
					warning = 1;
				}
			}
		}
		else
		{
			(void) fprintf(stderr,"Missing assigment char in line: %s", line);
			warning = 1;
		}
	}
}


int check_param(param)
char *param;
{
	int ctr;

	for (ctr = 0; param_list[ctr].name != NULL; ctr++)
	{
		if( ! strcmp(param, param_list[ctr].name))
			return( ctr);
	}

	/* check to see if is one of the properties parameters */
	if(cfg.properties != NULL)
	{
		for( ctr = 0; cfg.properties[ctr].name != NULL; ctr++)
		{
			if( !strncmp(param, prop_names[ctr].is, strlen(prop_names[ctr].is)) || !strncmp(param, prop_names[ctr].not, strlen(prop_names[ctr].not)))
				return(PROPERTY);
		}
	}

	/* failure */
	return( -1);
}


int what_value(pnum, str, param)
int	pnum;
char	*str;
char 	*param;
{
	int	ctr, ok = 0;

	/* check that the value is not empty */
	while(isspace(*str))
		++str;
	if( *str == '\0')
	{
		/* special case for PROPERTIES which is allowed to be NULL */
		if (pnum == 112) /* PROPERTIES */
		{
			check[pnum] = 1;
			return( 0);
		}

		if( pnum == PROPERTY)
			(void) fprintf(stderr,"Parameter was unset for either IS<property> or NOT<proerty>\n");
		else
			(void) fprintf(stderr,"Parameter %s was unset\n", param_list[pnum].name);
		warning = 1;
		return( 0);
	}

	if (pnum != PROPERTY)
	{
		what_line = param_list[pnum].name;

		/* check to see if parameter has already been set */
		if( check[pnum] != 0)
		{
			(void) fprintf(stderr, "Warning : parameter %s has already been set.\n", what_line);
			warning = 1;
			return(-2);
		}
	}

	if (pnum >= 28 && pnum <= 67)
		stringset = 1;
	else
		stringset = 2;

	switch(pnum)
	{
	case 0:
		ok = do_localename(&(cfg.locale1),str);
		break;
	case 1:
		ok = do_localename(&(cfg.locale2),str);
		break;

	case 2:
		ok = do_mbchars(cfg.iswcntrl,str);
		break;
	case 3:
		ok = do_mbchars(cfg.notcntrl,str);
		break;
	case 4:
		ok = do_mbchars(cfg.iswgraph,str);
		break;
	case 5:
		ok = do_mbchars(cfg.notgraph,str);
		break;
	case 6:
		ok = do_mbchars(cfg.iswlower,str);
		break;
	case 7:
		ok = do_mbchars(cfg.notlower,str);
		break;
	case 8:
		ok = do_mbchars(cfg.iswpunct,str);
		break;
	case 9:
		ok = do_mbchars(cfg.notpunct,str);
		break;
	case 10:
		ok = do_mbchars(cfg.iswspace,str);
		break;
	case 11:
		ok = do_mbchars(cfg.notspace,str);
		break;
	case 12:
		ok = do_mbchars(cfg.iswupper,str);
		break;
	case 13:
		ok = do_mbchars(cfg.notupper,str);
		break;

	case 14:
		ok = do_mbpairs(cfg.towupper,str,3);
		break;
	case 15:
		ok = do_mbpairs(cfg.towlower,str,3);
		break;

	case 16:
		ok = do_mbchars(cfg.width0,str);
		break;
	case 17:
		ok = do_mbchars(cfg.width1,str);
		break;
	case 18:
		ok = do_mbchars(cfg.width2,str);
		break;
	case 19:
		ok = do_mbchars(cfg.width3,str);
		break;
	case 20:
		ok = do_mbchars(cfg.dontcare,str);
		break;

	case 21:
		ok = do_mbpairs(cfg.primary,str,10);
		break;
	case 22:
		ok = do_mbpairs(cfg.secondary,str,10);
		break;

	case 23:
		ok = do_mbtriplets(cfg.primary2_1,str);
		break;
	case 24:
		ok = do_mbtriplets(cfg.secondary2_1,str);
		break;
	case 25:
		ok = do_mbtriplets(cfg.equal1_2,str);
		break;

	case 26:
		ok = do_mbchar(&(cfg.decpoint1),str, -1);
		break;
	case 27:
		ok = do_mbchar(&(cfg.decpoint2),str, -1);
		break;

	case 28:
		ok = do_mbstrings(&(cfg.month1[0]),str);
		break;
	case 29:
		ok = do_mbstrings(&(cfg.month1[1]),str);
		break;
	case 30:
		ok = do_mbstrings(&(cfg.month1[2]),str);
		break;
	case 31:
		ok = do_mbstrings(&(cfg.month1[3]),str);
		break;
	case 32:
		ok = do_mbstrings(&(cfg.month1[4]),str);
		break;
	case 33:
		ok = do_mbstrings(&(cfg.month1[5]),str);
		break;
	case 34:
		ok = do_mbstrings(&(cfg.month1[6]),str);
		break;
	case 35:
		ok = do_mbstrings(&(cfg.month1[7]),str);
		break;
	case 36:
		ok = do_mbstrings(&(cfg.month1[8]),str);
		break;
	case 37:
		ok = do_mbstrings(&(cfg.month1[9]),str);
		break;
	case 38:
		ok = do_mbstrings(&(cfg.month1[10]),str);
		break;
	case 39:
		ok = do_mbstrings(&(cfg.month1[11]),str);
		break;
	case 40:
		ok = do_mbstrings(&(cfg.abmon1[0]),str);
		break;
	case 41:
		ok = do_mbstrings(&(cfg.abmon1[1]),str);
		break;
	case 42:
		ok = do_mbstrings(&(cfg.abmon1[2]),str);
		break;
	case 43:
		ok = do_mbstrings(&(cfg.abmon1[3]),str);
		break;
	case 44:
		ok = do_mbstrings(&(cfg.abmon1[4]),str);
		break;
	case 45:
		ok = do_mbstrings(&(cfg.abmon1[5]),str);
		break;
	case 46:
		ok = do_mbstrings(&(cfg.abmon1[6]),str);
		break;
	case 47:
		ok = do_mbstrings(&(cfg.abmon1[7]),str);
		break;
	case 48:
		ok = do_mbstrings(&(cfg.abmon1[8]),str);
		break;
	case 49:
		ok = do_mbstrings(&(cfg.abmon1[9]),str);
		break;
	case 50:
		ok = do_mbstrings(&(cfg.abmon1[10]),str);
		break;
	case 51:
		ok = do_mbstrings(&(cfg.abmon1[11]),str);
		break;
	case 52:
		ok = do_mbstrings(&(cfg.day1[0]),str);
		break;
	case 53:
		ok = do_mbstrings(&(cfg.day1[1]),str);
		break;
	case 54:
		ok = do_mbstrings(&(cfg.day1[2]),str);
		break;
	case 55:
		ok = do_mbstrings(&(cfg.day1[3]),str);
		break;
	case 56:
		ok = do_mbstrings(&(cfg.day1[4]),str);
		break;
	case 57:
		ok = do_mbstrings(&(cfg.day1[5]),str);
		break;
	case 58:
		ok = do_mbstrings(&(cfg.day1[6]),str);
		break;
	case 59:
		ok = do_mbstrings(&(cfg.abday1[0]),str);
		break;
	case 60:
		ok = do_mbstrings(&(cfg.abday1[1]),str);
		break;
	case 61:
		ok = do_mbstrings(&(cfg.abday1[2]),str);
		break;
	case 62:
		ok = do_mbstrings(&(cfg.abday1[3]),str);
		break;
	case 63:
		ok = do_mbstrings(&(cfg.abday1[4]),str);
		break;
	case 64:
		ok = do_mbstrings(&(cfg.abday1[5]),str);
		break;
	case 65:
		ok = do_mbstrings(&(cfg.abday1[6]),str);
		break;
	case 66:
		ok = do_mbstrings(&(cfg.am_str1),str);
		break;
	case 67:
		ok = do_mbstrings(&(cfg.pm_str1),str);
		break;

	case 68:
		ok = do_mbstrings(&(cfg.month2[0]),str);
		break;
	case 69:
		ok = do_mbstrings(&(cfg.month2[1]),str);
		break;
	case 70:
		ok = do_mbstrings(&(cfg.month2[2]),str);
		break;
	case 71:
		ok = do_mbstrings(&(cfg.month2[3]),str);
		break;
	case 72:
		ok = do_mbstrings(&(cfg.month2[4]),str);
		break;
	case 73:
		ok = do_mbstrings(&(cfg.month2[5]),str);
		break;
	case 74:
		ok = do_mbstrings(&(cfg.month2[6]),str);
		break;
	case 75:
		ok = do_mbstrings(&(cfg.month2[7]),str);
		break;
	case 76:
		ok = do_mbstrings(&(cfg.month2[8]),str);
		break;
	case 77:
		ok = do_mbstrings(&(cfg.month2[9]),str);
		break;
	case 78:
		ok = do_mbstrings(&(cfg.month2[10]),str);
		break;
	case 79:
		ok = do_mbstrings(&(cfg.month2[11]),str);
		break;
	case 80:
		ok = do_mbstrings(&(cfg.abmon2[0]),str);
		break;
	case 81:
		ok = do_mbstrings(&(cfg.abmon2[1]),str);
		break;
	case 82:
		ok = do_mbstrings(&(cfg.abmon2[2]),str);
		break;
	case 83:
		ok = do_mbstrings(&(cfg.abmon2[3]),str);
		break;
	case 84:
		ok = do_mbstrings(&(cfg.abmon2[4]),str);
		break;
	case 85:
		ok = do_mbstrings(&(cfg.abmon2[5]),str);
		break;
	case 86:
		ok = do_mbstrings(&(cfg.abmon2[6]),str);
		break;
	case 87:
		ok = do_mbstrings(&(cfg.abmon2[7]),str);
		break;
	case 88:
		ok = do_mbstrings(&(cfg.abmon2[8]),str);
		break;
	case 89:
		ok = do_mbstrings(&(cfg.abmon2[9]),str);
		break;
	case 90:
		ok = do_mbstrings(&(cfg.abmon2[10]),str);
		break;
	case 91:
		ok = do_mbstrings(&(cfg.abmon2[11]),str);
		break;
	case 92:
		ok = do_mbstrings(&(cfg.day2[0]),str);
		break;
	case 93:
		ok = do_mbstrings(&(cfg.day2[1]),str);
		break;
	case 94:
		ok = do_mbstrings(&(cfg.day2[2]),str);
		break;
	case 95:
		ok = do_mbstrings(&(cfg.day2[3]),str);
		break;
	case 96:
		ok = do_mbstrings(&(cfg.day2[4]),str);
		break;
	case 97:
		ok = do_mbstrings(&(cfg.day2[5]),str);
		break;
	case 98:
		ok = do_mbstrings(&(cfg.day2[6]),str);
		break;
	case 99:
		ok = do_mbstrings(&(cfg.abday2[0]),str);
		break;
	case 100:
		ok = do_mbstrings(&(cfg.abday2[1]),str);
		break;
	case 101:
		ok = do_mbstrings(&(cfg.abday2[2]),str);
		break;
	case 102:
		ok = do_mbstrings(&(cfg.abday2[3]),str);
		break;
	case 103:
		ok = do_mbstrings(&(cfg.abday2[4]),str);
		break;
	case 104:
		ok = do_mbstrings(&(cfg.abday2[5]),str);
		break;
	case 105:
		ok = do_mbstrings(&(cfg.abday2[6]),str);
		break;
	case 106:
		ok = do_mbstrings(&(cfg.am_str2),str);
		break;
	case 107:
		ok = do_mbstrings(&(cfg.pm_str2),str);
		break;

	case 108:
		curr_locale = 1;
		ok = do_mbchar(&(cfg.mbcurmax1),str, -2);
		curr_locale = 0;
		break;
	case 109:
		curr_locale = 2;
		ok = do_mbchar(&(cfg.mbcurmax2),str, -2);
		curr_locale = 0;
		break;

	case 110:
		ok = do_mbchar(&(cfg.invalid_mb),str, -3);
		break;

	case 111:
		ok = do_anumber(&(cfg.invalid_wc),str);
		break;

	case 112:
		ok = do_properties(str);
		break;

	case PROPERTY:
		for( ctr = 0; cfg.properties[ctr].name != NULL; ctr++)
		{
			if( !strncmp(param, prop_names[ctr].is, strlen(prop_names[ctr].is)))
			{
				what_line = prop_names[ctr].is;
				if( prop_names[ctr].ischeck != 0)
				{
					(void) fprintf(stderr, "Warning : parameter %s has already been set.\n", what_line);
					warning = 1;
					return(-2);
				}
				ok = do_mbchars(cfg.properties[ctr].is,str);
				prop_names[ctr].ischeck = 1;
				break;
			}
			else if( !strncmp(param, prop_names[ctr].not, strlen(prop_names[ctr].not)))
			{
				what_line = prop_names[ctr].not;
				if( prop_names[ctr].notcheck != 0)
				{
					(void) fprintf(stderr, "Warning : parameter %s has already been set.\n", what_line);
					warning = 1;
					return(-2);
				}
				ok = do_mbchars(cfg.properties[ctr].not,str);
				prop_names[ctr].notcheck = 1;
				break;
			}
		}
		return(0);
				
	default:
		return( -1);
		/* NOTREACHED */
		break;
	}

	if (ok == 0)
		check[pnum] = 1;

	return( 0);
}


int do_localename( addr, str)
unsigned char **addr;
char *str;
{
	char	*tok;

	tok = str;

	/* remove any trailing white-space */
	while(*tok != '\0')
	{
		if(isspace(*tok))
		{
			*tok = '\0';
			break;
		}
		++tok;
	}

	*addr = (unsigned char *) malloc((size_t) (strlen(str) + 1));
	if( *addr == NULL)
	{
		(void) fprintf( stderr, "malloc in do_localename() failed.\n");
		perror("malloc");
		exit ( 1);
	}
	(void) strcpy( (char *) *addr, str);	

	return(0);
}


int do_mbchars( addr, str)
unsigned char *addr[3];
char *str;
{
	char *str1, *ptr, *sep;

	str1 = str;
	ptr = str;

	/* search for seperator */
	while( *ptr != ',')
	{
		++ptr;
		if(*ptr == '\0')
		{
			(void) fprintf(stderr,"badly formated value for parameter : %s\n", what_line);
			warning = 1;
			return(1);
		}
	}
	sep = ptr;
	*sep = '\0';

	/* strip trailing whitespace */
	if( isspace((int) *(--ptr)))
	{
		while( isspace((int) *(--ptr)));
		*(++ptr) = '\0';
	}

	CONVERT_CHECK(addr[0], 2, 0, 0);

	/* strip preceeding white-space */ 
	ptr = sep;
	while( isspace((int) *(++ptr)));

	str1 = ptr;

	/* search for seperator */
	while( *ptr != ',')
	{
		++ptr;
		if(*ptr == '\0')
		{
			(void) fprintf(stderr,"badly formated value for parameter : %s\n", what_line);
			warning = 1;
			return(1);
		}
	}
	sep = ptr;
	*sep = '\0';

	/* strip trailing whitespace */
	if( isspace((int) *(--ptr)))
	{
		while( isspace((int) *(--ptr)));
		*(++ptr) = '\0';
	}

	CONVERT_CHECK(addr[1], 3, 0, 0);

	/* strip preceeding white-space */ 
	ptr = sep;
	while( isspace((int) *(++ptr)));

	str1 = ptr;

	/* search for seperator */
	while( *ptr != '\0')
		++ptr;
	sep = ptr;

	/* strip trailing whitespace */
	if( isspace((int) *(--ptr)))
	{
		while( isspace((int) *(--ptr)));
		*(++ptr) = '\0';
	}

	CONVERT_CHECK(addr[2], 4, 0, 0);

	return(0);
}


int do_mbchar( addr, str1, flag)
unsigned char **addr;
char *str1;
int flag;
{
	char *ptr;

	/* remove trailing whitespace */
	ptr = &str1[(strlen(str1)-1)];
	if( isspace((int) *ptr))
	{
		--ptr;
		while(isspace(*ptr))
			--ptr;
		++ptr;
		*ptr = '\0';
	}

	if (flag == -3 && (str1[0] != '"' || str1[strlen(str1)-1] != '"'))
	{
		(void) fprintf(stderr, "parameter %s should contain a string in double quotes\n", what_line);
		warning = 1;
		*addr = (unsigned char *) "";
	}
	else
		CONVERT_CHECK( *addr, flag, 0, 0);

	return(0);
}


int do_mbpairs( addr, str, asize)
unsigned char *addr[][2];
char *str;
int asize;
{
	char *str1, *ptr, *sep = NULL; /* the equation keeps lint happy */
	int pair = 0, size1, size2; 

	for( pair = 0; pair < asize; pair++)
	{
		if( pair != 0)
		{
			/* strip preceeding white-space */ 
			ptr = sep;
			while( isspace((int) *(++ptr)));

			str1 = ptr;
		}
		else
			str1 = str;

		ptr = str1;

		/* search for seperator */
		while( *ptr != ',')
		{
			++ptr;
			if(*ptr == '\0')
			{
				(void) fprintf(stderr,"badly formated value for parameter : %s\n", what_line);
				warning = 1;
				return(1);
			}
		}
		sep = ptr;
		*sep = '\0';

		/* strip trailing whitespace */
		if( isspace((int) *(--ptr)))
		{
			while( isspace((int) *(--ptr)));
			*(++ptr) = '\0';
		}

		switch(pair)
		{
			case 0:
				if(asize == 3)
				{
					size1 = 2;
					size2 = 2;
				}
				else
				{
					size1 = 1;
					size2 = 1;
				}
				break;
			case 1:
				if(asize == 3)
				{
					size1 = 3;
					size2 = 3;
				}
				else
				{
					size1 = 1;
					size2 = 2;
				}
				break;
			case 2:
				if(asize == 3)
				{
					size1 = 4;
					size2 = 4;
				}
				else
				{
					size1 = 1;
					size2 = 3;
				}
				break;
			case 3: 
				size1 = 1;
				size2 = 4;
				break;
			case 4:
				size1 = 2;
				size2 = 2;
				break;
			case 5:
				size1 = 2;
				size2 = 3;
				break;
			case 6: 
				size1 = 2;
				size2 = 4;
				break;
			case 7:
				size1 = 3;
				size2 = 3;
				break;
			case 8: 
				size1 = 3;
				size2 = 4;
				break;
			default:
				size1 = 4;
				size2 = 4;
				break;
		}

		CONVERT_CHECK(addr[pair][0], size1, size2, 0);

		/* strip preceeding white-space */ 
		ptr = sep;
		while( isspace((int) *(++ptr)));

		str1 = ptr;

		/* search for seperator */
		if( pair == (asize - 1))
		{
			while( *ptr != '\0')
				++ptr;
		}
		else
		{
			while( *ptr != ';')
			{
				++ptr;
				if(*ptr == '\0')
				{
					(void) fprintf(stderr,"badly formated value for parameter : %s\n", what_line);
					warning = 1;
					return(1);
				}
			}
			sep = ptr;
			*sep = '\0';
		}

		/* strip trailing whitespace */
		if( isspace((int) *(--ptr)))
		{
			while( isspace((int) *(--ptr)));
			*(++ptr) = '\0';
		}

		CONVERT_CHECK(addr[pair][1], size2, size1, 0);
	}

	return(0);
}


int do_mbtriplets( addr, str)
unsigned char *addr[6][3];
char *str;
{
	char *str1, *ptr, *sep = NULL; /*assignment keeps lint happy */
	int triplet, size1, size2, size3; 

	for( triplet = 0; triplet < 6; triplet++)
	{
		if( triplet != 0)
		{
			/* strip preceeding white-space */ 
			ptr = sep;
			while( isspace((int) *(++ptr)));

			str1 = ptr;
		}
		else
			str1 = str;

		/* str1 points to the beginning of the first entry */
		ptr = str1;

		/* search for seperator */
		while( *ptr != ',')
		{
			++ptr;
			if(*ptr == '\0')
			{
				(void) fprintf(stderr,"badly formated value for parameter : %s\n", what_line);
				warning = 1;
				return(1);
			}
		}
		sep = ptr;
		*sep = '\0';

		/* strip trailing whitespace of two-byte entry */
		if( isspace((int) *(--ptr)))
		{
			while( isspace((int) *(--ptr)));
			*(++ptr) = '\0';
		}

		switch (triplet)
		{
			case 0:
				size1 = 1;
				size2 = 1;
				size3 = 2;
				break;
			case 1:
				size1 = 2;
				size2 = 3;
				size3 = 1;
				break;
			case 2:
				size1 = 2;
				size2 = 1;
				size3 = 3;
				break;
			case 3:
				size1 = 4;
				size2 = 2;
				size3 = 3;
				break;
			case 4:
				size1 = 1;
				size2 = 3;
				size3 = 3;
				break;
			default:
				size1 = 3;
				size2 = 4;
				size3 = 4;
				break;
		}
		CONVERT_CHECK(addr[triplet][0], size1, size2, size3);

		/* strip preceeding white-space of three-byte entry */ 
		ptr = sep;
		while( isspace((int) *(++ptr)));

		/* str1 points to beginning of second element */
		str1 = ptr;

		/* search for seperator */
		while( *ptr != ',')
		{
			++ptr;
			if(*ptr == '\0')
			{
				(void) fprintf(stderr,"badly formated value for parameter : %s\n", what_line);
				warning = 1;
				exit(1);
			}
		}
		sep = ptr;
		*sep = '\0';

		/* strip trailing whitespace of three-byte entry */
		if( isspace((int) *(--ptr)))
		{
			while( isspace((int) *(--ptr)));
			*(++ptr) = '\0';
		}

		CONVERT_CHECK(addr[triplet][1], size2, size1, size3);

		/* strip preceeding white-space */ 
		ptr = sep;
		while( isspace((int) *(++ptr)));

		/* str1 points to beginning of third element */
		str1 = ptr;

		/* search for seperator */
		if( triplet == 5)
		{
			while( *ptr != '\0')
				++ptr;
		}
		else
		{
			while( *ptr != ';')
			{
				++ptr;
				if(*ptr == '\0')
				{
					(void) fprintf(stderr,"badly formated value for parameter : %s\n", what_line);
					warning = 1;
					return(1);
				}
			}
			sep = ptr;
			*sep = '\0';
		}

		/* strip trailing whitespace */
		if( isspace((int) *(--ptr)))
		{
			while( isspace((int) *(--ptr)));
			*(++ptr) = '\0';
		}

		CONVERT_CHECK(addr[triplet][2], size3, size1, size2);
	}

	return(0);
}


int do_anumber( addr, str)
wchar_t *addr;
char *str;
{
	wchar_t i;

	errno = 0;

	i = strtol(str, (char **)NULL, 0);
	if( i == 0 && errno != 0)
	{
		(void) fprintf(stderr,"Parameter %s, expected an integer got %s\n",
							what_line, str);
		warning = 1;
		return( 1);
	}
	else
		*addr = i;

	return(0);
}


int do_mbstrings( addr, str)
unsigned char **addr;
char *str;
{
	char *str1, *buf, *ptr;
	int ctr;

	/* don't need all this room but not to worry */
	buf = (char *) malloc((size_t) (strlen(str) + 1));
	if( buf == NULL)
	{
		(void) fprintf(stderr,"malloc failed in do_mbstrings\n");
		exit(1);
	}
	
	buf[0] = '\0';

	for( ctr = 0; ;ctr++)
	{
		if( ctr == 0)
			str1 = strtok(str,",\0");
		else
			str1 = strtok((char *) NULL,",\0");

		if( str1 != NULL)
		{
			/* strip off preceeding white-space */
			if(isspace((int) *str1))
				while( isspace((int) *(++str1)));

			/* strip off trailing white-space */
			ptr = &str1[strlen(str1)-1];
			if(isspace((int) *ptr))
			{
				while( isspace((int) *(--ptr)));
				*(++ptr) = '\0';
			}
		}

		if( str1 == NULL)
			break;

		str1 = convert_field(str1, 0, 0, 0);

		if( buf[0] == '\0')
			(void) strcpy(buf, str1);
		else
			(void) strcat(buf, str1);
	}

	*addr = (unsigned char *) malloc((size_t) (strlen(buf) + 1));
	if( *addr == NULL)
	{
		(void) fprintf( stderr, "malloc in do_mbstrings() failed.\n");
		perror("malloc");
		exit ( 1);
	}
	(void) strcpy((char *) *addr, buf);

	free((void *) buf);

	return(0);
}


int do_properties(str)
char *str;
{
	char *str1, *ptr;
	int ctr;

	/* initialise prop_names so that realloc works correctly */
	prop_names = NULL;

	for( ctr = 0; ;ctr++)
	{
		if( ctr == 0)
			str1 = strtok(str,",\0");
		else
			str1 = strtok((char *) NULL,",\0");

		if( str1 != NULL)
		{
			/* strip off preceeding white-space */
			if(isspace((int) *str1))
				while( isspace((int) *(++str1)));

			/* strip off trailing white-space */
			ptr = &str1[strlen(str1)-1];
			if(isspace((int) *ptr))
			{
				while( isspace((int) *(--ptr)));
				*(++ptr) = '\0';
			}
		}

		if( str1 == NULL)
		{
			/* 
		   	 * Final property should be empty with a NULL name 
			 * as this is a loop limiter in the tests.
			 */
			cfg.properties = (struct wc_prop *) realloc( (void *) cfg.properties, (size_t) (sizeof(struct wc_prop) * (ctr+1)));
			if( cfg.properties == NULL)
			{
				(void) fprintf( stderr, "realloc in do_properties() failed.\n");
				perror("realloc");
				exit ( 1);
			}
			cfg.properties[ctr].name = (unsigned char *) NULL;

			/* break out of loop */
			break;
		}

		/* allocate memory for properties structure */
		cfg.properties = (struct wc_prop *) realloc( (void *) cfg.properties, (size_t) (sizeof(struct wc_prop) * (ctr+1)));
		if( cfg.properties == NULL)
		{
			(void) fprintf( stderr, "realloc of cfg.propperties in do_properties() failed.\n");
			perror("realloc");
			exit ( 1);
		}

		/* create space for, and assign property name */
		cfg.properties[ctr].name = (unsigned char *) malloc((size_t) (strlen(str1) + 1));
		if( cfg.properties[ctr].name == NULL)
		{
			(void) fprintf( stderr, "malloc in do_properties() failed.\n");
			perror("malloc");
			exit ( 1);
		}
		(void) strcpy((char *) cfg.properties[ctr].name, str1);

		/* create parameters that support the properties parameter */
		prop_names = (struct wc_prop_names *) realloc((void *) prop_names, (size_t) (sizeof(struct wc_prop_names) * (ctr+1)));

		if( prop_names == NULL)
		{
			(void) fprintf( stderr, "realloc of prop_names in do_properties() failed.\n");
			perror("realloc");
			exit ( 1);
		}

		prop_names[ctr].ischeck = 0;
		prop_names[ctr].notcheck = 0;

		/* construct IS */
		prop_names[ctr].is = (char *) malloc((size_t) (strlen((char *) cfg.properties[ctr].name) + 4));
		if( prop_names[ctr].is == NULL)
		{
			(void) fprintf( stderr, "malloc of prop_names[ctr].is in do_properties() failed.\n");
			perror("malloc");
			exit ( 1);
		}
		(void) strcpy(prop_names[ctr].is, "ISW");
		(void) strcat(prop_names[ctr].is, (char *) cfg.properties[ctr].name);
		for( ptr = &(prop_names[ctr].is[3]); *ptr != '\0'; ptr++)
			*ptr = (char) toupper( (int) *ptr);

		/* construct NOT */
		prop_names[ctr].not = (char *) malloc((size_t) (strlen((char *) cfg.properties[ctr].name) + 4));
		if( prop_names[ctr].not == NULL)
		{
			(void) fprintf( stderr, "malloc of prop_names[ctr].not in do_properties() failed.\n");
			perror("malloc");
			exit ( 1);
		}
		(void) strcpy(prop_names[ctr].not, "NOT");
		(void) strcat(prop_names[ctr].not,(char *) cfg.properties[ctr].name);
		for( ptr = &(prop_names[ctr].not[3]); *ptr != '\0'; ptr++)
			*ptr = (char) toupper( (int) *ptr);
	}

	return(0);
}


char *convert_field(str, size1, size2, size3 )
char *str;
int size1, size2, size3;
{
	static 	char buf[256];	/* need static buffers to use make_cstr() */
	static	char buf2[256];
	char 	*str1, *str2;
	int	wcnum, mbsize = -1, return_null = 0;
	int	leadsize = 0;

	buf[0] = '\0';
	buf2[0] = '\0';

	if (isdigit(*str))
	{
		wcnum = (int) strtol(str, &str1, 0);

		if( wcnum == 0 && str1 != str)
		{
			/* No wide character with specified attribute exists */
			mbsize = 0;
		}
		else if( wcnum == 0 && str1 == str)
		{
			perror("strtol");
			(void) fprintf(stderr,"strtol() failed for parameter %s : entry %s\n", what_line, str);
			warning = 1;
		}
		else
		{
			if (setlocale(LC_CTYPE, (char *)cfg.locale1) == NULL)
			{
				(void) fprintf(stderr, "setlocale(LC_CTYPE) failed for LOCALE1 (%s)", cfg.locale1);
				exit(1);
			}
			if ( (mbsize = wctomb( buf, (wchar_t) wcnum)) == -1)
			{
				(void) fprintf(stderr,"wctomb() failed for parameter %s : entry %s\n", what_line, str);
				warning = 1;
			}
			else
				buf[mbsize] = '\0';

			(void) setlocale(LC_ALL, "C");
		}
	}
	else if( *str == '"' && *(str + 1) == '"')
	{
		/* No wide char with the specified attribute exists */
		mbsize = 0;
	}
	else if( *str == '"') /* A string ie. something beginning with a " */
	{
		/* remove quotes and concatanate lead sequence if necessary */
		while ( (str1 = strchr(&str[1], '"')) != NULL)
		{
			if( *(str1-1) == '\\')
				continue;
			else
				break;
		}
		if(str1 == NULL)
		{
			(void) fprintf(stderr,"No closing quote for C literal string for parameter : %s\nLine : %s\n", what_line, str);
			warning = 1;
		}
		else if( *(str1+1) == '"')
		{
			/* lead sequence */
			*(str1) = '\0';

			/* copy the lead sequence into buf */
			(void) strcpy( buf, &str[1]);
			make_cstr(buf);

			/* find closing quote */
			str2 = strrchr(&str1[2],'"');
			if( str2 == NULL)
			{
				(void) fprintf(stderr,"No closing quote for C literal string for parameter : %s\nLine : %s\n", what_line, str);
				warning = 1;
			}
			else
			{
				*str2 = '\0';

				(void) strcpy(buf2, (str1+2));
				make_cstr(buf2);

				leadsize = strlen(buf);
				mbsize = strlen(buf2);

				/* copy onto the end of the lead sequence */
				(void) strcat( buf, buf2);
			}
		}
		else /* closing quote */
		{
			*str1 = '\0';

			(void) strcpy(buf, &str[1]);
			make_cstr(buf);

			mbsize = strlen(buf);
		}
	}
	else
	{
		(void) fprintf(stderr,"unrecognised value format for parameter %s\n", what_line);
		warning = 1;
	}

	if ( mbsize < 0) /* default or if an error occurs in wctomb() */
	{
		/* failure in generating a valid size for the mbchar */
		return_null = 1;
	}
	else if ( size1 > 0) /* check size */
	{
		if( mbsize != size1 && mbsize != 0)
		{
			(void) fprintf(stderr,"property %s : entry [%s] should be %d bytes\n", what_line, str, size1);
			warning = 1;
			return_null = 1;
		}
		else if (what_line != param_list[16].name &&  /* no checking */
			 what_line != param_list[17].name &&  /* for WIDTHx  */
			 what_line != param_list[18].name &&  /* parameters  */
			 what_line != param_list[19].name)
		{
			/* check use of entries is consistent */
			if (mbsize == 0)
			{
				if (sizesupp[size1 - 1] == -1) /* not set */
					sizesupp[size1 - 1] = 0;
				else if( sizesupp[size1 - 1] != 0)
				{
					/* special check for pairs & triplets */
					if((size2 != 0 && sizesupp[size2 - 1] == 0) || (size3 != 0 && sizesupp[size3 - 1] == 0))
					{
						/* OK */
					}
					else
					{
						(void) fprintf(stderr, "parameter %s : expected %d byte entry to be set.\n", what_line, size1);
						warning = 1;
						return_null = 1;
					}
				}
			}
			else
			{
				if (sizesupp[size1 - 1] == -1) /* not set */
					sizesupp[size1 - 1] = 1;
				else if( sizesupp[size1 - 1] != 1)
				{
					(void) fprintf(stderr, "parameter %s : %d byte entries previously set to NULL.\n", what_line, size1);
					warning = 1;
					return_null = 1;
					sizesupp[size1 - 1] = 1;
				}
			}
		}
	}
	else if (size1 == -1)
	{
		/* special case for decimal point parameters */
#ifdef MB_DECIMAL_POINT
		if (mbsize <= 1 && (sizesupp[1] == 1 || sizesupp[2] == 1 || sizesupp[3] == 1))
		{
			(void) fprintf(stderr,"parameter %s : value must be a multibyte of more than one byte\n", what_line);
			warning = 1;
			return_null = 1;
		}
#else
		if (mbsize < 1)
		{
			(void) fprintf(stderr,"parameter %s : value must not be an empty string\n", what_line);
			warning = 1;
			return_null = 1;
		}
#endif
	}
	else if (size1 == -2)
	{
		/* special case for mb_cur_max */
		if (setlocale(LC_CTYPE, (char *)
			(curr_locale == 2 ? cfg.locale2 : cfg.locale1)) == NULL)
		{
			(void) fprintf(stderr, "setlocale(LC_CTYPE) failed for LOCALE%d (%s)",
				curr_locale, (curr_locale == 2 ? cfg.locale2 : cfg.locale1));
			exit(1);
		}
		if( leadsize + mbsize != MB_CUR_MAX)
		{
			(void) fprintf(stderr,"parameter %s : value should be of size %d bytes%s\n",
				what_line, MB_CUR_MAX,
				leadsize == 0 ? "" : " (including lead sequence)");
			warning = 1;
			return_null = 1;
		}
		(void) setlocale(LC_ALL, "C");
	}
	else if (size1 == 0) /* mbstrings */
	{
		if (mbsize > 1 && mbsize <= 4)
		{
			if (stringset == 1)
				mbstringset1[mbsize - 2] = 1;
			else
				mbstringset2[mbsize - 2] = 1;
			if (sizesupp[mbsize - 1] == 0)
			{
				(void) fprintf(stderr, "parameter %s : %d byte entries previously set to NULL.\n", what_line, mbsize);
				warning = 1;
				return_null = 1;
				sizesupp[size1 - 1] = 1;
			}
		}
	}

	if (return_null)
		return((char *) NULL);
	else
		return(buf);
}


void make_cstr( str)
char	*str;
{
	char	*new, *p, *orig;
	int	i;
	char	num[4];

	orig = str;

	if((new = (char *) malloc((size_t) (strlen(str) + 1))) == NULL)
	{
		(void) fprintf(stderr, "malloc failed in make_cstr()\n");
		exit(1);
	}

	p = new;
	while (*str != '\0')
	{
		if (*str != '\\')
		{
			*p++ = *str++;
			continue;
		}

		str++;

		/* any number of hex digits */
		if (*str == 'x' || *str == 'X')
		{
			++str;
			*p++ = strtol(str, (char **)0, 16);
			/* don't trust strtol() to get end point right */
			while (isxdigit(*str))
				str++;
			continue;
		}

		/* up to 3 octal digits */
		if (isdigit(*str) && *str != '8' && *str != '9')
		{
			(void) strncpy(num, str, (size_t)3);
			num[3] = '\0';
			*p++ = strtol(num, (char **)0, 8);
			/* don't trust strtol() to get end point right */
			for (i = 0; i < 3 && isdigit(*str) &&
					*str != '8' && *str != '9'; i++)
				str++;
			continue;
		}

		switch(*str)
		{
		case '\0' :	/* fallthrough */
		case '\\' :	*p++ = '\\'; break;
		case 'b'  :	*p++ = '\b'; break;
		case 'f'  :	*p++ = '\f'; break;
		case 'n'  :	*p++ = '\n'; break;
		case 'r'  :	*p++ = '\r'; break;
		case 't'  :	*p++ = '\t'; break;
		case 'v'  :	*p++ = '\v'; break;
		default :
			*p++ = '\\';
			*p++ = *str;
		}

		if (*str != '\0')
			str++;
	}

	*p = '\0';

	(void) strcpy(orig, new);

	free((void *) new);
}


void write_cfg(file_name)
char	*file_name;
{
	FILE 	*fptr;
	char 	numstr[128];
	int	ctr;

	if((fptr = fopen(file_name, "w")) == (FILE *) NULL)
	{
		(void) fprintf(stderr, "Error opening file %s in write_cfg()\n", file_name);
		perror("fopen");
		exit(1);
	}
	WRITE_CFG(cfg.locale1);
	WRITE_CFG(cfg.locale2);
	WRITE_CFG(cfg.iswcntrl[0]);
	WRITE_CFG(cfg.iswcntrl[1]);
	WRITE_CFG(cfg.iswcntrl[2]);
	WRITE_CFG(cfg.notcntrl[0]);
	WRITE_CFG(cfg.notcntrl[1]);
	WRITE_CFG(cfg.notcntrl[2]);
	WRITE_CFG(cfg.iswgraph[0]);
	WRITE_CFG(cfg.iswgraph[1]);
	WRITE_CFG(cfg.iswgraph[2]);
	WRITE_CFG(cfg.notgraph[0]);
	WRITE_CFG(cfg.notgraph[1]);
	WRITE_CFG(cfg.notgraph[2]);
	WRITE_CFG(cfg.iswlower[0]);
	WRITE_CFG(cfg.iswlower[1]);
	WRITE_CFG(cfg.iswlower[2]);
	WRITE_CFG(cfg.notlower[0]);
	WRITE_CFG(cfg.notlower[1]);
	WRITE_CFG(cfg.notlower[2]);
	WRITE_CFG(cfg.iswpunct[0]);
	WRITE_CFG(cfg.iswpunct[1]);
	WRITE_CFG(cfg.iswpunct[2]);
	WRITE_CFG(cfg.notpunct[0]);
	WRITE_CFG(cfg.notpunct[1]);
	WRITE_CFG(cfg.notpunct[2]);
	WRITE_CFG(cfg.iswspace[0]);
	WRITE_CFG(cfg.iswspace[1]);
	WRITE_CFG(cfg.iswspace[2]);
	WRITE_CFG(cfg.notspace[0]);
	WRITE_CFG(cfg.notspace[1]);
	WRITE_CFG(cfg.notspace[2]);
	WRITE_CFG(cfg.iswupper[0]);
	WRITE_CFG(cfg.iswupper[1]);
	WRITE_CFG(cfg.iswupper[2]);
	WRITE_CFG(cfg.notupper[0]);
	WRITE_CFG(cfg.notupper[1]);
	WRITE_CFG(cfg.notupper[2]);
	WRITE_CFG(cfg.towupper[0][0]);
	WRITE_CFG(cfg.towupper[0][1]);
	WRITE_CFG(cfg.towupper[1][0]);
	WRITE_CFG(cfg.towupper[1][1]);
	WRITE_CFG(cfg.towupper[2][0]);
	WRITE_CFG(cfg.towupper[2][1]);
	WRITE_CFG(cfg.towlower[0][0]);
	WRITE_CFG(cfg.towlower[0][1]);
	WRITE_CFG(cfg.towlower[1][0]);
	WRITE_CFG(cfg.towlower[1][1]);
	WRITE_CFG(cfg.towlower[2][0]);
	WRITE_CFG(cfg.towlower[2][1]);
	WRITE_CFG(cfg.width0[0]);
	WRITE_CFG(cfg.width0[1]);
	WRITE_CFG(cfg.width0[2]);
	WRITE_CFG(cfg.width1[0]);
	WRITE_CFG(cfg.width1[1]);
	WRITE_CFG(cfg.width1[2]);
	WRITE_CFG(cfg.width2[0]);
	WRITE_CFG(cfg.width2[1]);
	WRITE_CFG(cfg.width2[2]);
	WRITE_CFG(cfg.width3[0]);
	WRITE_CFG(cfg.width3[1]);
	WRITE_CFG(cfg.width3[2]);
	WRITE_CFG(cfg.dontcare[0]);
	WRITE_CFG(cfg.dontcare[1]);
	WRITE_CFG(cfg.dontcare[2]);
	WRITE_CFG(cfg.primary[0][0]);
	WRITE_CFG(cfg.primary[0][1]);
	WRITE_CFG(cfg.primary[1][0]);
	WRITE_CFG(cfg.primary[1][1]);
	WRITE_CFG(cfg.primary[2][0]);
	WRITE_CFG(cfg.primary[2][1]);
	WRITE_CFG(cfg.primary[3][0]);
	WRITE_CFG(cfg.primary[3][1]);
	WRITE_CFG(cfg.primary[4][0]);
	WRITE_CFG(cfg.primary[4][1]);
	WRITE_CFG(cfg.primary[5][0]);
	WRITE_CFG(cfg.primary[5][1]);
	WRITE_CFG(cfg.primary[6][0]);
	WRITE_CFG(cfg.primary[6][1]);
	WRITE_CFG(cfg.primary[7][0]);
	WRITE_CFG(cfg.primary[7][1]);
	WRITE_CFG(cfg.primary[8][0]);
	WRITE_CFG(cfg.primary[8][1]);
	WRITE_CFG(cfg.primary[9][0]);
	WRITE_CFG(cfg.primary[9][1]);
	WRITE_CFG(cfg.secondary[0][0]);
	WRITE_CFG(cfg.secondary[0][1]);
	WRITE_CFG(cfg.secondary[1][0]);
	WRITE_CFG(cfg.secondary[1][1]);
	WRITE_CFG(cfg.secondary[2][0]);
	WRITE_CFG(cfg.secondary[2][1]);
	WRITE_CFG(cfg.secondary[3][0]);
	WRITE_CFG(cfg.secondary[3][1]);
	WRITE_CFG(cfg.secondary[4][0]);
	WRITE_CFG(cfg.secondary[4][1]);
	WRITE_CFG(cfg.secondary[5][0]);
	WRITE_CFG(cfg.secondary[5][1]);
	WRITE_CFG(cfg.secondary[6][0]);
	WRITE_CFG(cfg.secondary[6][1]);
	WRITE_CFG(cfg.secondary[7][0]);
	WRITE_CFG(cfg.secondary[7][1]);
	WRITE_CFG(cfg.secondary[8][0]);
	WRITE_CFG(cfg.secondary[8][1]);
	WRITE_CFG(cfg.secondary[9][0]);
	WRITE_CFG(cfg.secondary[9][1]);
	WRITE_CFG(cfg.primary2_1[0][0]);
	WRITE_CFG(cfg.primary2_1[0][1]);
	WRITE_CFG(cfg.primary2_1[0][2]);
	WRITE_CFG(cfg.primary2_1[1][0]);
	WRITE_CFG(cfg.primary2_1[1][1]);
	WRITE_CFG(cfg.primary2_1[1][2]);
	WRITE_CFG(cfg.primary2_1[2][0]);
	WRITE_CFG(cfg.primary2_1[2][1]);
	WRITE_CFG(cfg.primary2_1[2][2]);
	WRITE_CFG(cfg.primary2_1[3][0]);
	WRITE_CFG(cfg.primary2_1[3][1]);
	WRITE_CFG(cfg.primary2_1[3][2]);
	WRITE_CFG(cfg.primary2_1[4][0]);
	WRITE_CFG(cfg.primary2_1[4][1]);
	WRITE_CFG(cfg.primary2_1[4][2]);
	WRITE_CFG(cfg.primary2_1[5][0]);
	WRITE_CFG(cfg.primary2_1[5][1]);
	WRITE_CFG(cfg.primary2_1[5][2]);
	WRITE_CFG(cfg.secondary2_1[0][0]);
	WRITE_CFG(cfg.secondary2_1[0][1]);
	WRITE_CFG(cfg.secondary2_1[0][2]);
	WRITE_CFG(cfg.secondary2_1[1][0]);
	WRITE_CFG(cfg.secondary2_1[1][1]);
	WRITE_CFG(cfg.secondary2_1[1][2]);
	WRITE_CFG(cfg.secondary2_1[2][0]);
	WRITE_CFG(cfg.secondary2_1[2][1]);
	WRITE_CFG(cfg.secondary2_1[2][2]);
	WRITE_CFG(cfg.secondary2_1[3][0]);
	WRITE_CFG(cfg.secondary2_1[3][1]);
	WRITE_CFG(cfg.secondary2_1[3][2]);
	WRITE_CFG(cfg.secondary2_1[4][0]);
	WRITE_CFG(cfg.secondary2_1[4][1]);
	WRITE_CFG(cfg.secondary2_1[4][2]);
	WRITE_CFG(cfg.secondary2_1[5][0]);
	WRITE_CFG(cfg.secondary2_1[5][1]);
	WRITE_CFG(cfg.secondary2_1[5][2]);
	WRITE_CFG(cfg.equal1_2[0][0]);
	WRITE_CFG(cfg.equal1_2[0][1]);
	WRITE_CFG(cfg.equal1_2[0][2]);
	WRITE_CFG(cfg.equal1_2[1][0]);
	WRITE_CFG(cfg.equal1_2[1][1]);
	WRITE_CFG(cfg.equal1_2[1][2]);
	WRITE_CFG(cfg.equal1_2[2][0]);
	WRITE_CFG(cfg.equal1_2[2][1]);
	WRITE_CFG(cfg.equal1_2[2][2]);
	WRITE_CFG(cfg.equal1_2[3][0]);
	WRITE_CFG(cfg.equal1_2[3][1]);
	WRITE_CFG(cfg.equal1_2[3][2]);
	WRITE_CFG(cfg.equal1_2[4][0]);
	WRITE_CFG(cfg.equal1_2[4][1]);
	WRITE_CFG(cfg.equal1_2[4][2]);
	WRITE_CFG(cfg.equal1_2[5][0]);
	WRITE_CFG(cfg.equal1_2[5][1]);
	WRITE_CFG(cfg.equal1_2[5][2]);
	WRITE_CFG(cfg.decpoint1);
	WRITE_CFG(cfg.decpoint2);
	WRITE_CFG(cfg.month1[0]);
	WRITE_CFG(cfg.month1[1]);
	WRITE_CFG(cfg.month1[2]);
	WRITE_CFG(cfg.month1[3]);
	WRITE_CFG(cfg.month1[4]);
	WRITE_CFG(cfg.month1[5]);
	WRITE_CFG(cfg.month1[6]);
	WRITE_CFG(cfg.month1[7]);
	WRITE_CFG(cfg.month1[8]);
	WRITE_CFG(cfg.month1[9]);
	WRITE_CFG(cfg.month1[10]);
	WRITE_CFG(cfg.month1[11]);
	WRITE_CFG(cfg.abmon1[0]);
	WRITE_CFG(cfg.abmon1[1]);
	WRITE_CFG(cfg.abmon1[2]);
	WRITE_CFG(cfg.abmon1[3]);
	WRITE_CFG(cfg.abmon1[4]);
	WRITE_CFG(cfg.abmon1[5]);
	WRITE_CFG(cfg.abmon1[6]);
	WRITE_CFG(cfg.abmon1[7]);
	WRITE_CFG(cfg.abmon1[8]);
	WRITE_CFG(cfg.abmon1[9]);
	WRITE_CFG(cfg.abmon1[10]);
	WRITE_CFG(cfg.abmon1[11]);
	WRITE_CFG(cfg.day1[0]);
	WRITE_CFG(cfg.day1[1]);
	WRITE_CFG(cfg.day1[2]);
	WRITE_CFG(cfg.day1[3]);
	WRITE_CFG(cfg.day1[4]);
	WRITE_CFG(cfg.day1[5]);
	WRITE_CFG(cfg.day1[6]);
	WRITE_CFG(cfg.abday1[0]);
	WRITE_CFG(cfg.abday1[1]);
	WRITE_CFG(cfg.abday1[2]);
	WRITE_CFG(cfg.abday1[3]);
	WRITE_CFG(cfg.abday1[4]);
	WRITE_CFG(cfg.abday1[5]);
	WRITE_CFG(cfg.abday1[6]);
	WRITE_CFG(cfg.am_str1);
	WRITE_CFG(cfg.pm_str1);
	WRITE_CFG(cfg.month2[0]);
	WRITE_CFG(cfg.month2[1]);
	WRITE_CFG(cfg.month2[2]);
	WRITE_CFG(cfg.month2[3]);
	WRITE_CFG(cfg.month2[4]);
	WRITE_CFG(cfg.month2[5]);
	WRITE_CFG(cfg.month2[6]);
	WRITE_CFG(cfg.month2[7]);
	WRITE_CFG(cfg.month2[8]);
	WRITE_CFG(cfg.month2[9]);
	WRITE_CFG(cfg.month2[10]);
	WRITE_CFG(cfg.month2[11]);
	WRITE_CFG(cfg.abmon2[0]);
	WRITE_CFG(cfg.abmon2[1]);
	WRITE_CFG(cfg.abmon2[2]);
	WRITE_CFG(cfg.abmon2[3]);
	WRITE_CFG(cfg.abmon2[4]);
	WRITE_CFG(cfg.abmon2[5]);
	WRITE_CFG(cfg.abmon2[6]);
	WRITE_CFG(cfg.abmon2[7]);
	WRITE_CFG(cfg.abmon2[8]);
	WRITE_CFG(cfg.abmon2[9]);
	WRITE_CFG(cfg.abmon2[10]);
	WRITE_CFG(cfg.abmon2[11]);
	WRITE_CFG(cfg.day2[0]);
	WRITE_CFG(cfg.day2[1]);
	WRITE_CFG(cfg.day2[2]);
	WRITE_CFG(cfg.day2[3]);
	WRITE_CFG(cfg.day2[4]);
	WRITE_CFG(cfg.day2[5]);
	WRITE_CFG(cfg.day2[6]);
	WRITE_CFG(cfg.abday2[0]);
	WRITE_CFG(cfg.abday2[1]);
	WRITE_CFG(cfg.abday2[2]);
	WRITE_CFG(cfg.abday2[3]);
	WRITE_CFG(cfg.abday2[4]);
	WRITE_CFG(cfg.abday2[5]);
	WRITE_CFG(cfg.abday2[6]);
	WRITE_CFG(cfg.am_str2);
	WRITE_CFG(cfg.pm_str2);
	WRITE_CFG(cfg.mbcurmax1);
	WRITE_CFG(cfg.mbcurmax2);
	WRITE_CFG(cfg.invalid_mb);
	/* the number will be read in as a string at the other end */
	(void) sprintf(numstr, "%d", cfg.invalid_wc);
	WRITE_CFG(numstr);
	if( cfg.properties != NULL)
	{
		for( ctr = 0; cfg.properties[ctr].name != NULL; ctr++)
		{
			WRITE_CFG(cfg.properties[ctr].name);
			WRITE_CFG(cfg.properties[ctr].is[0]);
			WRITE_CFG(cfg.properties[ctr].is[1]);
			WRITE_CFG(cfg.properties[ctr].is[2]);
			WRITE_CFG(cfg.properties[ctr].not[0]);
			WRITE_CFG(cfg.properties[ctr].not[1]);
			WRITE_CFG(cfg.properties[ctr].not[2]);
		}
	}

	(void) fclose(fptr);
}
