: use /bin/sh
#
#	SCCS:	@(#)install/scripts/inccreat.sh	4.1 (01/21/97)  VSXgen release 1.4
#
#	This shell script makes a local copy of the include files on the 
#	system and adds the necessary additional definitions to these include
#	files. The additional definitions are found in $CURDIR/vsxconfig.h
#
#	If the input list contains a line of the form "header.h xpg2" then
#	XPG2_DEFINES is used instead of DEFINES on the cc command line
#	for that header.

trap "" 1 2 3

# Configurable parameters:
#	TGTDIR	- target SYSINC directory
#	CURDIR	- Name of the directory in which vsxconfig.h is to be found
#	(other meanings as per SRC/vsxparams)
#
INCDIRS=${INCDIRS:-/usr/include}
TEST_MODE=${TEST_MODE:-POSIX90}
TEST_XNFS=${TEST_XNFS:-n}
SUBSETS=${SUBSETS:-base}
VSXDIR=${VSXDIR:-$HOME/SRC}
CURDIR=${CURDIR:-$VSXDIR}
TGTDIR=$VSXDIR/SYSINC
CC=${CC:-/bin/cc}
DEFINES=${DEFINES:-"-D_XOPEN_SOURCE"}

INCRPT=$VSXDIR/install/incrpt.data

# Local variables
#	missrep		- report of missing include files and definitions.
#	missdef		- flag missing defines in a file
#	list
#
Efile=/tmp/E$$.tmp
missrep=$CURDIR/vsxconfig.h
missdef=""
errfound=""

# characters delimiting C identifiers
Bsep="[ 	,*(]"		# at the beginning of the name
Esep="[ 	,;()[{]"	# at the end of the name

case $TEST_MODE in
XPG3) listfiles="" ;;
*) listfiles="$VSXDIR/install/Inc.list"	;; # POSIX.1-1990 symbols
esac
for subset in $SUBSETS
do
	list="$VSXDIR/subsets/$subset/Inc.$TEST_MODE"
	case $TEST_XNFS in
	[yY])
		if test -r "$VSXDIR/subsets/$subset/Incnfs.$TEST_MODE"
		then
			list="$VSXDIR/subsets/$subset/Incnfs.$TEST_MODE"
		fi
		;;
	esac
	listfiles="$listfiles $list"
done

list=/tmp/Inc$$
$VSXDIR/install/scripts/incmerge.sh $listfiles > $list

cfile=""
sfile=""
last=""
# Allow for broken shells which leave the leading tab in "$i"
sed 's/^	//' $list | (
while test -z "$last"
do
	if read i
	then :
	else
		last="1"
		i=".h"
	fi
	case "$i" in
	# (( Bracket matcher(s)
	*.h*)
		if test -n "$sfile" -a -n "$missdef"
		then
			if grep -s "\"<$sfile>\"" "$missrep" > /dev/null
			then
				echo "Definitions for file <$sfile> are missing from $missrep"
				errfound="1"
			else
				# change 'sys/name.h' into 'sys\/name.h'
				# (can't rely on sed '\?pattern?')
				spatt=`sed 's?/?\\\\/?g' <<!
$sfile
!
				`
				sed -n "/^\/\*.*<$spatt>/,/End of Definitions.* $spatt/p" "$missrep" >> "$TGTDIR/$sfile"
				if test $? -ne 0
				then
					echo "*** Failed to append definitions to $sfile"
				fi
			fi
			missdef=""
		fi
		if test -n "$sfile"
		then
			echo "*** Completed $sfile"
		fi

		if test -n "$last"
		then
			break
		fi

		defs="$DEFINES"
		case $i in
		# (
		*xpg2)
			defs="$XPG2_DEFINES"
			set -- $i
			i=$1
			;;
		esac

		cfile=""
		for dir in $INCDIRS
		do
			if test -f "$dir/$i"
			then
				cfile="$dir/$i"
				break
			fi
		done
		sfile=$i;
		echo "*** Starting $i"
		if test -z "$cfile"
		then
			cat < /dev/null > ${TGTDIR}/${sfile}
			chmod 0644 ${TGTDIR}/${sfile}
			hdrfound=""
			missdef="1"
		else
			cp "$cfile" ${TGTDIR}/${sfile}
			chmod 0644 ${TGTDIR}/${sfile}
			if test -f $INCRPT/$i.c
			then
				hdrfound="1"
				$CC -I$VSXDIR/SYSINC $COPTS $defs -E $INCRPT/$i.c > "$Efile"
			else
				echo "Unknown header <$i> not checked"
				errfound="1"
				hdrfound=""
			fi
		fi
		;;
	*)
		if test -n "$hdrfound"
		then
			x=`sed -n "/$Bsep$i$Esep/{
					p
					q
				   }
				   /^$i$Esep/{
					p
					q
				   }
				   /$Bsep$i\$/{
					p
					q
				   }
				   /^$i\$/{
					p
					q
				   }" "$Efile"`
			if test -z "$x"
			then
				missdef="1"
			fi
		fi
		;;
	esac
done
)
if test ! -f ${TGTDIR}/regexp.h
then
	cat < /dev/null > ${TGTDIR}/regexp.h
	chmod 0644 ${TGTDIR}/regexp.h
fi	
rm -f "$Efile" "$list"
if test -n "$errfound"
then
	exit 1
fi
exit 0
