: use /bin/sh
#
#	SCCS:	@(#)install/scripts/incrpt.sh	4.1 (01/21/97)  VSXgen release 1.4
#
#	This shell script looks through the include directories on the system
#    and checks that the include files required by VSX are on the system and
#    contain the necessary defines, structure definitions etc.
#	-l	generate a list of include files and defines/variables
#		which are used from each include file.
#
#	If the input list contains a line of the form "header.h xpg2" then
#	XPG2_DEFINES is used instead of DEFINES on the cc command line
#	for that header.

trap "" 1 2 3

# Configurable parameters:
#	TGTDIR	- directory in which to place vsxconfig.h
#	(other meanings as per SRC/vsxparams)
#
INCDIRS=${INCDIRS:-/usr/include}
TEST_MODE=${TEST_MODE:-POSIX90}
TEST_XNFS=${TEST_XNFS:-n}
SUBSETS=${SUBSETS:-base}
VSXDIR=${VSXDIR:-$HOME/SRC}
TGTDIR=${TGTDIR:-$VSXDIR}
CC=${CC:-/bin/cc}
DEFINES=${DEFINES:-"-D_XOPEN_SOURCE"}

INCRPT=$VSXDIR/install/incrpt.data
CHKRPT=$VSXDIR/install/check.data

# Local variables
#	missrep		- report of missing include files and definitions.
#	missdef		- flag missing defines in a file
#	tmpfile		- list of missing defines in a file
#	list
#	dolist
#	docheck
#
ccfile=/tmp/CC$$.c
Efile=/tmp/E$$.tmp
tmpfile=/tmp/T$$.tmp
missrep=$TGTDIR/vsxconfig.h
missdef=""
dolist=""
docheck=""
repmade=""

# characters delimiting C identifiers
Bsep="[ 	,*(]"		# at the beginning of the name
Esep="[ 	,;()[{]"	# at the end of the name

while test "$#" != "0"
do
	case $1 in
	# (((( Bracket matcher(s)
	-c)
		docheck=1
		;;
	-l)
		dolist=1
		;;
	-x)
		# debugging ...
		set -x
		;;
	*)
		echo Unknown option "'$1'"
		echo Usage: $0 [ -l -c ]
		exit 1
		;;
	esac
	shift
done

if test -z "$docheck$dolist"
then
	docheck="1"
fi

case $TEST_MODE in
XPG3) listfiles="" ;;
*) listfiles="$VSXDIR/install/Inc.list"	;; # POSIX.1-1990 symbols
esac
for subset in $SUBSETS
do
	list="$VSXDIR/subsets/$subset/Inc.$TEST_MODE"
	case $TEST_XNFS in
	[yY])
		if test -r "$VSXDIR/subsets/$subset/Incnfs.$TEST_MODE"
		then
			list="$VSXDIR/subsets/$subset/Incnfs.$TEST_MODE"
		fi
		;;
	esac
	listfiles="$listfiles $list"
done

list=/tmp/Inc$$
$VSXDIR/install/scripts/incmerge.sh $listfiles > $list

if test -n "$dolist"
then
	echo "Elements of include files which are used:"
	cat < $list
fi
if test -n "$docheck"
then
	cfile=""
	sfile=""
	last=""
	> $tmpfile
	echo "
/****** This file contains definitions for those elements which    ******/
/****** 'incrpt' found to be missing from the system include files ******/
" > $missrep
	# Allow for broken shells which leave the leading tab in "$i"
	sed 's/^	//' $list | (
	while test -z "$last"
	do
		if read i
		then :
		else
			last="1"
			i=".h"
		fi
		case "$i" in
		# (( Bracket matcher(s)
		*.h*)
			if test -n "$sfile" -a -n "$missdef"
			then
				repmade="1"
				if test -n "$hdrfound"
				then
					echo "/****** <$sfile> - Missing elements ******/" >> $missrep
				fi
				echo "/****** Start of Definitions for file $sfile ******/" >> $missrep
				cat "$tmpfile" | while read line
				do
					def=`sed -n "/$Bsep$line$Esep/{
							p
							q
						   }
						   /^$line$Esep/{
							p
							q
						   }
						   /$Bsep$line\$/{
							p
							q
						   }
						   /^$line\$/{
							p
							q
						   }" $CHKRPT/${sfile}.d`
					if test -z "$def"
					then def="symbol matching /$line/"
					fi
					echo "$def" >> $missrep
					if test -n "$hdrfound"
					then
						echo "Missing: $def"
					fi
				done
				echo "/****** End of Definitions for file $sfile ******/" >> $missrep
				echo "" >> $missrep
				missdef=""
				cat < /dev/null > $tmpfile
			fi
			if test -n "$sfile"
			then
				echo "*** Completed $sfile"
			fi

			if test -n "$last"
			then
				break
			fi

			defs="$DEFINES"
			case $i in
			# (
			*xpg2)
				defs="$XPG2_DEFINES"
				set -- $i
				i=$1
				;;
			esac

			cfile=""
			for dir in $INCDIRS
			do
				if test -f "$dir/$i"
				then
					cfile="$dir/$i"
					break
				fi
			done
			echo "*** Starting $i"
			sfile=$i
			if test -z "$cfile"
			then
				repmade="1"
				echo "/****** <$i> - Missing include file ******/" >> $missrep
				echo "Missing #include file: $i"
				hdrfound=""
				missdef="1"
			elif test ! -f $INCRPT/$i.c 
			then
				repmade="1"
				echo "Unknown header <$i> not checked"
				hdrfound=""
			else
				hdrfound="1"
				$CC $COPTS $defs -E $INCRPT/$i.c > "$Efile"
			fi
			;;
		*)
			if test -n "$hdrfound"
			then
				x=`sed -n "/$Bsep$i$Esep/{
						p
						q
					   }
					   /^$i$Esep/{
						p
						q
					   }
					   /$Bsep$i\$/{
						p
						q
					   }
					   /^$i\$/{
						p
						q
					   }" "$Efile"`
				if test -z "$x"
				then
					missdef="1"
					echo "$i" >> $tmpfile
				fi
			else
				echo "$i" >> $tmpfile
			fi
			;;
		esac
	done
	)
	echo "/****** End of Definitions ******/" >> $missrep
fi
rm -f "$ccfile" "$Efile" "$tmpfile" "$list"
if test -n "$repmade"
then
	exit 1
fi
exit 0
