: use Bourne Shell
#########################################################################
#									#
#	SCCS:	@(#)install/scripts/install.sh	4.3 (03/03/98)  VSXgen release 1.4#
#									#
#		      VSX Installation Script				#
#		Build Basic Software Utilities for VSX			#
#									#
#		Inputs - User Designated Parameter File			#
#		       - User Designated Include File			#
#		Outputs - Installation Report				#
#########################################################################

trap "" 1 2 3

tfile=/tmp/ch.ss.tmp		# miscellaneous temporary files
cfile=/tmp/ch.ss.c
cdirfile=/tmp/chd.ss.c
tdir=/tmp/ch.dr
ifile=/tmp/ch.ss.i

#  Remove temporary files and exit if a hangup or interrupt occurs
rm_tmp1="rm -f /tmp/ans a.out $tfile $cfile"
rm_tmp2="rm -f $cdirfile $ifile"
trap "$rm_tmp1; $rm_tmp2; exit 1" 1 2

echo "
**********************************************************
**		    VSXgen release 1.4			**
**		X/OPEN Verification Suite		**
**		  Installation Procedure		**
**********************************************************
"

VSXDIR="$HOME/SRC"
if test ! -d "$VSXDIR/install"
then
	echo "*** VSX must be installed in your HOME directory ($HOME)"
	exit 1
fi
cd "$VSXDIR/install"

CURDIR=$VSXDIR
. $CURDIR/vsxparams
export VSXDIR TEST_MODE TEST_XNFS SUBSETS INCDIRS CC COPTS THR_COPTS
export DEFINES LDFLAGS SYSLIBS CURDIR PATH TET_EXECUTE
echo "*** The VSX source directory is rooted at $VSXDIR"

#  Work out next journal file number

JNLDIR="$HOME/results"
if test ! -d "$JNLDIR"
then
	echo "*** journal directory $JNLDIR does not exist"
	exit 1
fi

num=`ls -r "$JNLDIR" | \
	awk '/^install\.[0-9]*$/ {
		n = index($0, ".")
		num = substr($0, n+1, 99) + 1
		printf "%4.4d\n", num
		exit
	    }
	    END { if (num == 0) print "0001" }'`

if test -z "$num" -o 0 -eq "$num"
then
	echo "*** error searching journal directory: $JNLDIR"
	exit 1
fi

JNLFILE="$JNLDIR/install.$num"
echo "*** Install journal file is: $JNLFILE"

# all output from here to end is tee'd into journal file
(

#########################################################################
#									#
# Update the file std.h with the system dependencies			#
#									#
#########################################################################
echo "
Updating the file std.h with the system dependencies"

DEF_XPG=_XOPEN_SOURCE	# caters for new X/Open modes without change
case "$TEST_MODE" in
POSIX*|FIPS*) DEF_XPG=0 ;;
XPG3) DEF_XPG=3 ;;
XPG4|UNIX95) DEF_XPG=4 ;;
esac

DEF_UNIX=0
case "$TEST_MODE" in
UNIX*) DEF_UNIX=1 ;;
esac

DEF_FIPS=0
case "$TEST_MODE" in
FIPS*) DEF_FIPS=1 ;;
esac

DEF_XNFS=0
DEF_SPEED=$SPEED
case "$TEST_XNFS" in
[Yy])	DEF_XNFS=1
	DEF_SPEED="(4 * $SPEED)"
	;;
esac

DEF_SOCKET=0
case "$RPC_TYPE" in
[Ss]*) DEF_SOCKET=1 ;;
esac

DEF_VPRINTF=1
case "$VPRINTF_SUPP" in
n|N) DEF_VPRINTF=0 ;;
esac

DEF_LONG_DOUBLE=0
case "$LONG_DOUBLE_SUPP" in
y|Y) DEF_LONG_DOUBLE=1 ;;
esac

sed -e "/define[	 ]*TEST_XPG/s/TEST_XPG.*$/TEST_XPG	$DEF_XPG/" \
    -e "/define[	 ]*TEST_UNIX/s/TEST_UNIX.*$/TEST_UNIX	$DEF_UNIX/" \
    -e "/define[	 ]*TEST_FIPS/s/TEST_FIPS.*$/TEST_FIPS	$DEF_FIPS/" \
    -e "/define[	 ]*TEST_XNFS/s/TEST_XNFS.*$/TEST_XNFS	$DEF_XNFS/" \
    -e "/SPEEDFACTOR/s/SPEEDFACTOR.*$/SPEEDFACTOR	$DEF_SPEED/" \
    -e "/XNFS_SOCKET_RPCLIB/s/RPCLIB.*$/RPCLIB	$DEF_SOCKET/" \
    -e "/ANSI_VPRINTF_FUNC/s/_FUNC.*$/_FUNC	$DEF_VPRINTF/" \
    -e "/LONG_DOUBLE_SUPPORTED/s/ORTED.*$/ORTED	$DEF_LONG_DOUBLE/" \
	../INC/std.h.orig > ../INC/std.h

#########################################################################
#									#
# Create the include files for use by VSX				#
#									#
#########################################################################
echo "
Creating the system include files used by VSX in $VSXDIR/SYSINC"
$VSXDIR/install/scripts/inccreat.sh

# copy sample RPC headers into SYSINC/rpc if using sample RPC library
case $TEST_XNFS in
[yY])
	rm -f $VSXDIR/SYSINC/rpc/*
	case $RPCLIB in
	*SRC/LIB/xnfs_socket.a)
		cp $VSXDIR/common/xnfs_socket/rpc/* $VSXDIR/SYSINC/rpc
		;;
	esac
	;;
esac

#########################################################################
#									#
# Check whether the directory search routines work			#
#									#
#########################################################################

cat > $cdirfile << 'E_O_F'
#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
main()
{
	DIR	*d;
	struct	dirent	*dp;
	char	*i;
	int	c=0;

	d = opendir(".");
	while ((dp = readdir(d)) != 0)
	{
		i = (char *) dp->d_name;
		c++;
	}
	closedir(d);
	printf("%d\n", c);
}
E_O_F

$CC -I../INC -I../SYSINC $DEFINES $COPTS "$cdirfile" $SYSLIBS > $tfile 2>&1

if test "$?" != "0"
then
	echo "
*** Could not compile test program for directory routines:

$CC -I../INC -I../SYSINC $DEFINES $COPTS $cdirfile $SYSLIBS"
	cat $tfile
	echo "
Check SRC/SYSINC/dirent.h before re-running install.sh"
	exit 1
fi

mkdir $tdir
touch $tdir/a $tdir/asjd $tdir/jsdhjas
touch $tdir/kajshdjas $tdir/JH
dir=`pwd`
export dir
(cd $tdir ; $dir/a.out > /tmp/ans 2>&1)
rm -rf $tdir
x=`cat /tmp/ans`
if test "$x" != 7 -a "$x" != 5
then
	echo "
*** Directory routines don't work!
Check SRC/SYSINC/dirent.h before re-running install.sh"
	exit 1
fi

#########################################################################
#									#
# Check whether variable argument lists work				#
#									#
#########################################################################

cat << 'E_O_F' >$cfile
#include <std.h>
#include <stdlib.h>
#if TEST_ANSI
#include <stdarg.h>
static int sub(char *, ...);
#else
#include <varargs.h>
static int sub();
#endif

static char	arg1[] = "abc";
static int	arg2 = 12345;
static double	arg3 = 123.456;
static char	arg4[] = "xyz";

main(c, v)
int c;
char *v[];
{
	exit(sub(arg1, arg2, arg3, arg4));
}

static int
#if TEST_ANSI
sub(char *sarg1, ...)
#else
sub(sarg1, va_alist)
char *sarg1;
va_dcl
#endif
{
	va_list ap;

	if (sarg1 != arg1)
		return 1;
	
#if TEST_ANSI
	va_start(ap, sarg1);
#else
	va_start(ap);
#endif

	if (va_arg(ap, int) != arg2)
		return 2;
	
	if (va_arg(ap, double) != arg3)
		return 3;
	
	if (va_arg(ap, char *) != arg4)
		return 4;
	
	va_end(ap);

	return 0;
}
E_O_F

$CC -I../INC -I../SYSINC $DEFINES $COPTS "$cfile" $SYSLIBS > $tfile 2>&1

if test "$?" != "0" || grep arg "$tfile" > /dev/null 2>&1
then
	echo "
*** Could not compile test program for variable argument lists

$CC -I../INC -I../SYSINC $DEFINES $COPTS $cfile $SYSLIBS"
	cat $tfile
	echo "
Check <stdarg.h>/<varargs.h> before re-running install.sh"
	exit 1
fi

./a.out
x=$?
if test "$x" -ne 0
then
	echo "
*** Variable argument lists don't work!  (test program exit code: $x)
Check <stdarg.h>/<varargs.h> before re-running install.sh"
	exit 1
fi

#########################################################################
#									#
# Set up the testroot 							#
#									#
#########################################################################
echo "
Creating files and directories under $TET_EXECUTE"
(
	dir=$TET_EXECUTE/tset
	if test ! -d "$dir"
	then mkdir "$dir" || exit 1
	fi

	cd $HOME/tset
	umask 0
	find . -type d -print | (
		# Do this the long way instead of using cpio -pd, since
		# the latter would preserve permissions from the source
		# tree, but testroot directories must be 777.
		cd "$TET_EXECUTE"/tset || exit 1
		while read dir
		do
			if test ! -d "$dir"
			then mkdir "$dir"
			fi
		done
	)

	dir=$TET_EXECUTE/BIN
	if test ! -d "$dir"
	then mkdir "$dir" && cp $HOME/TESTROOT/BIN/* "$dir" 2>/dev/null
	fi
	chmod u+w $dir/*.sh 2>/dev/null

	# $TET_EXECUTE/INC and contents are created by drivers/Makefile
)

#########################################################################
#
# Create the TET/DTET config files
#
#########################################################################

case $TEST_XNFS in
[yY])
	echo "
Creating tetbuild.cfg, tetexec.cfg, tetclean.cfg, tetdist.cfg and systems"
	;;
*)
	echo "
Creating tetbuild.cfg, tetexec.cfg and tetclean.cfg"
	;;
esac

if test -f $HOME/tetbuild.cfg
then
	echo >&2 "*** existing tetbuild.cfg moved to oldbuild.cfg"
	rm -f $HOME/oldbuild.cfg
	mv $HOME/tetbuild.cfg $HOME/oldbuild.cfg &&
	$VSXDIR/install/scripts/editcfg.sh $HOME/oldbuild.cfg \
		< $VSXDIR/install/tetbuild.cfg | \
		$VSXDIR/install/scripts/editcfg.sh $VSXDIR/vsxparams \
		> $HOME/tetbuild.cfg
else
	$VSXDIR/install/scripts/editcfg.sh $VSXDIR/vsxparams \
		< $VSXDIR/install/tetbuild.cfg > $HOME/tetbuild.cfg
fi

cat $VSXDIR/vsxparams > $tfile
packagelist=""
for subset in $SUBSETS
do
	CONFIG_PACKAGE=""
	. $VSXDIR/subsets/$subset/config_info
	case " $packagelist " in
	*" $CONFIG_PACKAGE "*) ;;
	*) packagelist="$packagelist $CONFIG_PACKAGE" ;;
	esac
done
echo "TEST_PACKAGES=\"$packagelist\"" >> $tfile
echo "VSX_CC=\"$CC\"" >> $tfile
echo "VSX_CFLAGS=\"$COPTS\"" >> $tfile
echo "VSX_LIBS=\"$MLIB $SYSLIBS $LDFLAGS\"" >> $tfile
case $TEST_XNFS in
[yY])
	templates=$VSXDIR/install/nfsexec.top
	for subset in $SUBSETS
	do
		templates="$templates $VSXDIR/subsets/$subset/nfs_params"
	done
	templates="$templates $VSXDIR/install/nfsexec.bot"
	;;
*)
	echo "VSX_NOSPC_DEV=\"$NOSPC_DEV\"" >> $tfile
	templates=$VSXDIR/install/tetexec.top
	for subset in $SUBSETS
	do
		templates="$templates $VSXDIR/subsets/$subset/exec_params"
	done
	templates="$templates $VSXDIR/install/tetexec.bot"
	;;
esac
if test -f $TET_EXECUTE/tetexec.cfg
then
	echo >&2 "*** existing tetexec.cfg moved to oldexec.cfg"
	rm -f $TET_EXECUTE/oldexec.cfg
	mv $TET_EXECUTE/tetexec.cfg $TET_EXECUTE/oldexec.cfg &&
	cat $templates | \
		$VSXDIR/install/scripts/editcfg.sh $TET_EXECUTE/oldexec.cfg | \
		$VSXDIR/install/scripts/editcfg.sh $tfile | \
		sed "s?{TET_EXECUTE}?$TET_EXECUTE?g" > $TET_EXECUTE/tetexec.cfg
else
	cat $templates | \
		$VSXDIR/install/scripts/editcfg.sh $tfile | \
		sed "s?{TET_EXECUTE}?$TET_EXECUTE?g" > $TET_EXECUTE/tetexec.cfg
fi

if test -f $HOME/tetclean.cfg
then
	echo >&2 "*** existing tetclean.cfg moved to oldclean.cfg"
	rm -f $HOME/oldclean.cfg
	mv $HOME/tetclean.cfg $HOME/oldclean.cfg &&
	$VSXDIR/install/scripts/editcfg.sh $HOME/oldclean.cfg \
		< $VSXDIR/install/tetclean.cfg | \
		$VSXDIR/install/scripts/editcfg.sh $VSXDIR/vsxparams \
		> $HOME/tetclean.cfg
else
	$VSXDIR/install/scripts/editcfg.sh $VSXDIR/vsxparams \
		< $VSXDIR/install/tetclean.cfg > $HOME/tetclean.cfg
fi

case $TEST_XNFS in
[yY])
	if test -f $HOME/tetdist.cfg
	then
		echo >&2 "*** existing tetdist.cfg moved to olddist.cfg"
		rm -f $HOME/olddist.cfg
		mv $HOME/tetdist.cfg $HOME/olddist.cfg
	fi
	echo "TET_REM001_TET_ROOT=\"$SERVER_TET_ROOT\"" > $tfile
	echo "TET_REM001_TET_TSROOT=\"$SERVER_TET_ROOT/vsx4\"" >> $tfile
	echo "TET_REM001_TET_EXECUTE=\"$SERVER_TET_ROOT/vsx4/TESTROOT\"" >> $tfile
	$VSXDIR/install/scripts/editcfg.sh $tfile \
		< $VSXDIR/install/tetdist.cfg > $HOME/tetdist.cfg

	echo "0 $XNFS_CLIENT
1 $XNFS_SERVER" > $tfile
	if test -f $HOME/../systems
	then
		if cmp -s $HOME/../systems $tfile
		then
			rm -f $tfile
		else
			echo >&2 "*** existing systems file moved to systems.old"
			rm -f $HOME/../systems.old
			mv $HOME/../systems $HOME/../systems.old &&
			mv $tfile $HOME/../systems
		fi
	else
		mv $tfile $HOME/../systems
	fi
	;;
esac

#########################################################################
#									#
# Create scenario files including the sections for the selected subsets	#
#									#
#########################################################################
echo "
Creating the scenario files scen.bld and scen.exec"

NFS=
case $TEST_XNFS in
[Yy]) NFS=nfs ;;
esac

if test -f $HOME/scen.bld
then
	echo "*** existing scen.bld moved to oldscen.bld"
	rm -f $HOME/oldscen.bld
	mv $HOME/scen.bld $HOME/oldscen.bld
fi
echo all > $HOME/scen.bld
for subset in $SUBSETS
do
	if test -f $VSXDIR/subsets/$subset/${TEST_MODE}.${NFS}bld
	then sed 1d $VSXDIR/subsets/$subset/${TEST_MODE}.${NFS}bld
	else sed 1d $VSXDIR/subsets/$subset/scen.${NFS}bld
	fi
done >> $HOME/scen.bld

if test -f $HOME/scen.exec
then
	echo "*** existing scen.exec moved to oldscen.exec"
	rm -f $HOME/oldscen.exec
	mv $HOME/scen.exec $HOME/oldscen.exec
fi
echo all > $HOME/scen.exec
for subset in $SUBSETS
do
	if test -f $VSXDIR/subsets/$subset/${TEST_MODE}.${NFS}exec
	then sed 1d $VSXDIR/subsets/$subset/${TEST_MODE}.${NFS}exec
	else sed 1d $VSXDIR/subsets/$subset/scen.${NFS}exec
	fi
done >> $HOME/scen.exec

#########################################################################
#									#
# Install userintf.c, rpcincs.h and testvars.h				#
#									#
#########################################################################
echo "
Installing userintf.c and other files in the relevant directories"

cp $VSXDIR/userintf.c $VSXDIR/common/vport
cp $VSXDIR/rpcincs.h $VSXDIR/INC
rm -f $VSXDIR/common/drivers/testvars.h
echo '/* THIS FILE IS AUTO-GENERATED: DO NOT EDIT */
"VSX_CHRDEV_FILE",
"VSX_BLKDEV_FILE",' > $VSXDIR/common/drivers/testvars.h

#########################################################################
#									#
# Configure the chmog privilege list 					#
#									#
#########################################################################
echo "
Configuring privilege list in $VSXDIR/common/vtools/chmog_c.c"

(
	cat $VSXDIR/common/vtools/chmog_c.top
	for subset in $SUBSETS
	do
		cat $VSXDIR/subsets/$subset/chmog_privs
	done
	echo "{ NULL }"
	echo "};"
) > $VSXDIR/common/vtools/chmog_c.c

#########################################################################
#									#
# Update the makefiles for the common software				#
#									#
#########################################################################
echo "
Updating the Makefiles for the common software elements"

# Decide which TET directories to use - prefer tet3 over dtet2/posix_c
case $TEST_XNFS in
y|Y)
	if test -d $VSXDIR/../../inc/tet3 -a -d $VSXDIR/../../lib/tet3
	then tetdir=tet3
	elif test -d $VSXDIR/../../inc/dtet2 -a -d $VSXDIR/../../lib/dtet2
	then tetdir=dtet2
	else
		echo "*** could not find tet3 or dtet2 inc/lib directories"
		exit 1
	fi

	XNFSLIB=$VSXDIR/LIB/xnfslib.a
	;;
*)
	if test -d $VSXDIR/../../inc/tet3 -a -d $VSXDIR/../../lib/tet3
	then tetdir=tet3
	elif test -d $VSXDIR/../../inc/posix_c -a -d $VSXDIR/../../lib/posix_c
	then tetdir=posix_c
	else
		echo "*** could not find tet3 or posix_c inc/lib directories"
		exit 1
	fi

	XNFSLIB=
        ;;
esac

TETINC=$VSXDIR/../../inc/$tetdir
APILIB=$VSXDIR/../../lib/$tetdir/libapi.a
TCM=$VSXDIR/../../lib/$tetdir/tcm.o
TCMCHILD=$VSXDIR/../../lib/$tetdir/tcmchild.o
THRAPILIB=$VSXDIR/../../lib/$tetdir/libthrapi.a
THRTCM=$VSXDIR/../../lib/$tetdir/thrtcm.o
THRTCMCHILD=$VSXDIR/../../lib/$tetdir/thrtcmchild.o

export TETINC APILIB TCM TCMCHILD THRAPILIB THRTCM THRTCMCHILD XNFSLIB

chmake_cmd='find $VSXDIR/common -name Makefile.org -print |
	$VSXDIR/install/scripts/chmake.sh
	"CC=$CC" "COPTS=$COPTS" "THR_COPTS=$THR_COPTS" "DEFINES=$DEFINES"
	"CFPURE=$CFPURE" "LDFLAGS=$LDFLAGS" "SYSLIBS=$SYSLIBS"
	"AR=$AR" "RANLIB=$RANLIB" "LORDER=$LORDER" "TSORT=$TSORT"
	"CHOWN=$CHOWN" "CHGRP=$CHGRP" "CHMOD=$CHMOD" "TET_EXECUTE=$TET_EXECUTE"
	"TETINC=$TETINC" "APILIB=$APILIB" "TCM=$TCM" "TCMCHILD=$TCMCHILD"
	"THRAPILIB=$THRAPILIB" "THRTCM=$THRTCM" "THRTCMCHILD=$THRTCMCHILD"
	"XNFSLIB=$XNFSLIB"'

for subset in $SUBSETS
do
	for var in `sed -n 's/^\([^#][^=]*\)=.*/\1/p' $VSXDIR/subsets/$subset/config_params`
	do
		eval "chmake_cmd=\"\$chmake_cmd\"' \"$var=\$$var\"'"
		eval export "$var"
	done
done

eval $chmake_cmd

#########################################################################
#									#
# Execute subset-specific install scripts				#
#									#
#########################################################################

for subset in $SUBSETS
do
	$VSXDIR/subsets/$subset/install_sub.sh
done

#########################################################################
#									#
# Make the common software elements					#
#									#
#########################################################################

libs="vport vlib genlib tet_startup"
thrlibs=""
utils="vprog vtools"
for subset in $SUBSETS
do
	INSTALL_LIBS=""
	INSTALL_THRLIBS=""
	INSTALL_UTILS=""
	. $VSXDIR/subsets/$subset/install_info
	for i in $INSTALL_LIBS
	do
		case " $libs " in
		*" $i "*) ;; # already in list
		*) libs="$libs $i" ;;
		esac
	done
	for i in $INSTALL_THRLIBS
	do
		case " $thrlibs " in
		*" $i "*) ;; # already in list
		*) thrlibs="$thrlibs $i" ;;
		esac
	done
	for i in $INSTALL_UTILS
	do
		case " $utils " in
		*" $i "*) ;; # already in list
		*) utils="$utils $i" ;;
		esac
	done
done

case $TEST_XNFS in
[Yy])
	target=INSTALL_xnfs
	case $RPCLIB in
	*SRC/LIB/xnfs_socket.a)
		libs="$libs xnfs_socket"
		;;
	esac
	;;
*)
	target=INSTALL
	;;
esac

for dir in $libs $thrlibs ${thrlibs:+thrstartup} $utils
do
    (
	echo "
Executing make in SRC/common/$dir"
	cd $VSXDIR/common/$dir || exit 1
	if make $target >make.$target 2>&1
	then	echo "$dir make succeeded"
	else	echo "$dir make failed:"
		cat make.$target
	fi
    )
done

# end of output tee'd to journal file

) 2>&1 | tee "$JNLFILE"

# clean up: remove temporary files
$rm_tmp1; $rm_tmp2

echo "
**********************************************************
**		    VSXgen release 1.4			**
**		X/OPEN Verification Suite		**
**		  Installation Procedure		**
**********************************************************
"
