:
# SCCS:	@(#)SRC/subsets/dynl/dynl_mkso.sh	Rel 5.1.1 (04/07/97)
#
# This script is used from DYNL.os Makefiles to create shared objects.
# It should be modified by the user as appropriate for the test system.
#
# SYNOPSIS:
#
#   dynl_mkso.sh -o outfile [-u] [ -d depobj ] file.c ...
#
# OPTIONS:
#
#   -o outfile	Specifies the shared object to be created.
#   -u		Indicates there will be unresolved symbols.  Only used
#		if DYNL_NODEFS_SUPPORTED is 'y'.
#   -d depobj	Specifies a dependency object (i.e. the created shared
#		object will contain references to symbols in depobj).
#		Only used if DYNL_DEPOBJ_SUPPORTED is 'y'.
#
# The remaining arguments are the sources for the shared object.
#
# Dependency objects are specified using full pathnames.  On some
# systems it may be necessary to use additional compiler or linker
# options to ensure that they will be referenced using the full pathname.
#
# The values of CC, CFLAGS, LDFLAGS and SYSLIBS from the Makefile are
# passed down in the environment.

# Add compiler flags needed when making shared objects

# these values for gcc
CFLAGS="$CFLAGS -fPIC"
LDFLAGS="$LDFLAGS -shared"

# Process options

outfile=""
depfiles=""
while [ $# -gt 0 ]
do
	case $1 in
	-u)
		# add an option to allow unresolved references if necessary
		# (it is often the default when making shared objects)

		#LDFLAGS="$LDFLAGS -z nodefs"
		;;
	-o)
		# next argument is the shared object to be created
		outfile="$2"
		shift
		;;
	-d)
		# next argument is a dependency object
		depfiles="$depfiles $2"
		shift
		;;
	-*)
		echo >&2 "$0: unknown option $1"
		exit 2
		;;
	*)
		# end of options
		break
		;;
	esac

	shift
done
if test -z "$outfile"
then
	echo >&2 "$0: no output file specified"
	exit 2
fi

# The remaining arguments are source files

cfiles="$*"

set -x  # so the command will be shown in build failure reports

# Alter the following command as necessary (e.g. if dependency
# objects need to be specified differently)

exec $CC $CFLAGS -o $outfile $LDFLAGS $cfiles $depfiles $SYSLIBS
