#!/bin/sh

#
# install script which manages configuration and installation
# of test sets
#
# Christopher Yeoh <cyeoh@linuxcare.com>
#
# This is $Revision: 1.11 $
#
# $Log: setup_testsets.sh,v $
# Revision 1.11  2001/10/01 08:16:49  ajosey
# function name is just name(), remove leading function
#
# Revision 1.10  2001/09/17 02:22:57  cyeoh
# Set default test mode to be UNIX98
#
# Revision 1.9  2001/09/06 06:26:05  cyeoh
# Put back extra header file dir checking minus the creation
# of the Linux parameter file
#
# Revision 1.8  2001/09/06 06:18:25  cyeoh
# Removes test for extra include directories.
# Adds check for test framework suceeding
#
# Revision 1.7  2001/07/18 08:52:23  ajosey
# remove some backslash c in the printf scripts
#
# Revision 1.6  2001/05/31 08:27:38  cyeoh
# Merge for lsb-os test suite features
#
# Revision 1.5  2001/01/24 08:56:16  cyeoh
# Adds info about re-running tests
#
# Revision 1.4  2001/01/23 05:34:28  cyeoh
# Adds lock area creation/cleaning
#
#

######################################################################
# Configuration 

INSTDIR=${TET_ROOT}/test_sets
PATH=$PATH:$INSTDIR/BIN
TOOLDIR=${TET_ROOT}/LSB.tools
export PATH TOOLDIR
VSXDIR="$HOME/SRC"
CURDIR=$VSXDIR

TESTSET_SCRIPT_DIR=${INSTDIR}/scripts

# Maybe need?
# DEFAULT=linux

######################################################################
# Useful functions

# Run testset specific config scripts
run_testset_config_script()
{
  # Test argument count
  if test $# -lt 1 
  then
    echo "$func_name: Missing argument"; return 1 # unresolved
  fi

  for i in scripts/*; do
    if [ -x $i/$1.sh ]; 
      then $i/$1.sh $2;
      if [ $? -ne 0 ]; then
        echo "Aborting installation"
	exit 1
      fi
    fi;
  done;
}

#####################################################################
# Common setup & checks
cat - <<!EOF
**********************************************************
**		   Verification Suite			**
**	Configuration Procedure for Linux		**
**********************************************************
!EOF

# 
cd $INSTDIR

# Check VSX exists in the correct place
if test ! -d "$VSXDIR/install"
then
        printf "*** VSX must be installed in your HOME directory ($HOME)\n"
        exit 1
fi

# Locate extra header file include needed
printf "Locating header file #include directories...\n"
printf "Calling: gcc -print-file-name=include/stddef.h\n"
STDDEF=`gcc -print-file-name="include/stddef.h"`
if [  $? -eq 0 ]
then 
    INCDIR=`dirname $STDDEF`
    printf "INCDIRS=\"$INCDIR /usr/include\"\n"
    printf "INCDIRS=\"$INCDIR /usr/include\"\n" >>SRC/vsxparams
else
    printf "Defaulting to /usr/include\n"
fi

# Set the default to be UNIX98 mode
echo "TEST_MODE=\"UNIX98\"" >> SRC/vsxparams

#----------------------------------------------------------------------
# Cleanup
# Create/Clean lock areas
if test ! -d "scripts/locks"
then
    mkdir scripts/locks
else
    rm -f scripts/locks/*
fi

#---------------------------------------------------------------------
# Pre Configure testset scripts
run_testset_config_script pre-config

######################################################################
# Run config script
printf "Run config.sh ..? [y] "
read cmd
if [ "$cmd" = "Y" -o "$cmd" = "y" -o "$cmd" = "" ] ; then
	config.sh
fi
printf "\n----------------------------------------------------------\n"

# Common cleanup needed
# Fixup NSIG to _NSIG which common to all Linux's (and CLK_TCK)
sed -e 's/NSIG	(-1)/NSIG	(_NSIG)/' -e 's/#define	CLK_TCK	(-1)//' -e '/symbol matching/d' SRC/vsxconfig.h >SRC/vsxconfig.h.new
mv SRC/vsxconfig.h.new SRC/vsxconfig.h

# Update userint.c file
echo "Updating userintf.c to support features needed by all of the"
echo "test suites supported."
cp $TOOLDIR/userintf.c SRC/userintf.c

#---------------------------------------------------------------------
# Post Configure testset scripts
run_testset_config_script post-config

######################################################################
# Build the test framework
cd $INSTDIR

printf "\n\n----------------------------------------------------------\n"
printf "Building the test framework ... enter the root password\n"
su root -c make
if [ $? -ne 0 ]; then
  echo "Test framework compilation failed. Aborting"
  exit 1
fi
printf "\n----------------------------------------------------------\n"

# Load current vsxgen parameters
. $CURDIR/vsxparams

export VSXDIR TEST_MODE TEST_XNFS SUBSETS INCDIRS CC COPTS THR_COPTS
export DEFINES LDFLAGS SYSLIBS CURDIR PATH TET_EXECUTE NOSPC_DEV
export VSX_OPER VSX_ORG VSX_SYS

#---------------------------------------------------------------------
# Post build tetset scripts
run_testset_config_script post-build

######################################################################
# Parameterisation
printf "Install tetexec.cfg ..? [y] "
read cmd
if [ "$cmd" = "Y" -o "$cmd" = "y" -o "$cmd" = "" ] ; then

  echo "Parameterizing the $TET_EXECUTE/tetexec.cfg file"

  # .newparams file is used for updating the tetexec.cfg file
  cp /dev/null .newparams

  echo "VSX_OPER=\"$VSX_OPER\"" >>.newparams
  echo "VSX_ORG=\"$VSX_ORG\"" >>.newparams
  echo "VSX_SYS=\"$VSX_SYS\"" >>.newparams
  echo "TEST_MODE=\"$TEST_MODE\"" >>.newparams
  echo "VSX_CC=\"$CC\"" >> .newparams
  echo "VSX_CFLAGS=\"$COPTS\"" >> .newparams
  echo "VSX_LIBS=\"$MLIB $SYSLIBS $LDFLAGS\"" >> .newparams

#---------------------------------------------------------------------
# Test set specific parameterisations
  run_testset_config_script parameterisations .newparams

######################################################################
# Finish parameterisation
  cp  $TET_EXECUTE/tetexec.cfg $TET_EXECUTE/tetexec.cfg.old
  cat $TET_EXECUTE/tetexec.cfg.old|$VSXDIR/install/scripts/editcfg.sh .newparams > $TET_EXECUTE/tetexec.cfg
  rm -f .newparams

fi

######################################################################
# Build & Execute testsets
printf "Build and Execute testsets ..? [y]"
read cmd
if [ "$cmd" = "Y" -o "$cmd" = "y" -o "$cmd" = "" ] ; then
  sh run_testsets.sh
fi

echo 
echo "In order to re run the test suites at a later date run the"
echo "rerun_tests program in vsx0's home directory as the vsx0 user"
echo
