/*
*      SCCS:  @(#)  ANSI.os/charhandle/isgraph/isgraph.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/charhandle/isgraph - T.isgraph Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		isgraph.c          

PROJECT:	VSX (X/OPEN Validation Suite)


DESCRIPTION:
	Testset for the isgraph() subroutine.      

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd/Phyfos Photi, Hoskyns Group plc
DATE CREATED:	04/08/88                           

MODIFICATIONS:	Peter Lee	Tue Jul 03 14:45:35 BST 1990
		Added resilience code.

		Geoff Clare, 30 March 1995
			Add setup function which calls pslclass().

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<nlstest.h>
#include	<nlsclass.h>
#include	<report.h>
#include	<ctype.h>
#include	<sys/types.h>
#include	<string.h>
#include	<locale.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef isgraph
#endif


#define 	NO_TESTS	2		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

int	no_tests = NO_TESTS;

private int	testfail;
private char	*testcomment;
private char	*currlang = "";
private int	currcat;

private void	setup();
private void	test1();
private void	test2();
private void	ch_t1();
private void	checkval();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private void
setup()
{
	DBUG_ENTER("setup");

	if (pslclass() != 0)
		xx_all(DELETION, "pslclass() failed in setup");

	DBUG_VOID_RETURN;
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	expval;
	int	rval;
	int	i;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t1");

	testfail = 0;
	testcomment = "isgraph returns non-zero for visible characters";

	if (setlocale(LC_ALL,INFO1LANG) == NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO1LANG);
		in_rpt("Check instalation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	currlang = INFO1LANG;
	currcat  = LC_ALL;

	for (i = 0; i < 256; i++)
	{
		rval = isgraph(i);
		expval = lang1_table[i][0]&DEFGRAPH;
		globok = 0;
		checkval("isgraph", i, rval, expval);
		PATH_FUNC_TRACE;
	}

	if (setlocale(LC_ALL,INFO3LANG) == NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO3LANG);
		in_rpt("Check instalation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	currlang = INFO3LANG;

	for (i = 0; i < 256; i++)
	{
		rval = isgraph(i);
		expval = lang3_table[i][0]&DEFGRAPH;
		globok = 0;
		checkval("isgraph", i, rval, expval);
		PATH_FUNC_TRACE;
	}

	if (setlocale(LC_CTYPE,INFO1LANG) == NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO1LANG);
		in_rpt("Check instalation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO1LANG;
	currcat  = LC_CTYPE;

	for (i = 0; i < 256; i++)
	{
		rval = isgraph(i);
		expval = lang1_table[i][0]&DEFGRAPH;
		globok = 0;
		checkval("isgraph", i, rval, expval);
		PATH_FUNC_TRACE;
	}

	if (setlocale(LC_CTYPE,INFO3LANG) == NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO3LANG);
		in_rpt("Check instalation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO3LANG;

	for (i = 0; i < 256; i++)
	{
		rval = isgraph(i);
		expval = lang3_table[i][0]&DEFGRAPH;
		globok = 0;
		checkval("isgraph", i, rval, expval);
		PATH_FUNC_TRACE;
	}
	if (!testfail)
		PATH_XS_RPT(1028);	

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	int	expval;
	int	rval;
	int	i;
	int	pathok = 0;
	
	DBUG_ENTER("test2");

	testfail = 0;
	testcomment = "isgraph returns non-zero for visible characters";

	currlang = "";

	for (i = 0; i < 256; i++)
	{
		rval = isgraph(i);
		expval = asc_table[i][0]&DEFGRAPH;
		globok = 0;
		checkval("isgraph", i, rval, expval);
		PATH_FUNC_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(256);

	DBUG_VOID_RETURN;
}


private void
checkval(name, val, rval, expval)
char	*name;
int	val, rval, expval;
{

	int pathok = 0;

	DBUG_ENTER("checkval");

	globok = 0;

	if ((rval && expval) || (!rval && !expval))
	{
		PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_VOID_RETURN;
	}

	if (!testfail++)
		xx_rpt(FAILURE);
	in_rpt("%s(0x%x) failed", name, val);
	in_rpt(testcomment);
	if (strlen(currlang) > 0)
	{	in_rpt("Current language is %s", currlang);
		if (currcat == LC_ALL)
			in_rpt("Current category is %s","LC_ALL");
		else
			if (currcat == LC_CTYPE)
				in_rpt("Current category is %s","LC_CTYPE");
	}
	else
		in_rpt("No current language has been set");
	in_rpt("Expected return value: %d", expval);
	in_rpt("Actual   return value: %d", rval);

	DBUG_VOID_RETURN;
}
