/*
*      SCCS:  @(#)  ANSI.os/charhandle/toupper/tolower.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/charhandle/toupper - T.toupper Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.toupper

PROJECT:	VSX (X/OPEN Validation Suite)


DESCRIPTION:
	Testset for the toupper() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Mon Nov 10 15:04:07 WET 1986

MODIFIED:	Linda Taylor, Hoskyns.	August 1988.
		nl_init() replaced by setlocale()

		Peter Lee, Unisoft Ltd., Wed Jul 04 12:45:21 BST 1990
		Added resilience code.

		Geoff Clare, 30 March 1995
			Add setup function which calls pslclass().

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<nlstest.h>
#include	<nlsclass.h>
#include	<report.h>
#include	<ctype.h>
#include	<locale.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef tolower
#endif


#define 	NO_TESTS	3		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

int	no_tests = NO_TESTS;

private int	testfail, thisfail;
private char	*currlang = "";

private void	setup();
private void	test1();
private void	test2();
private void	test3();
private void	ch_t1();
private void	ch_t2();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private void
setup()
{
	DBUG_ENTER("setup");

	if (pslclass() != 0)
		xx_all(DELETION, "pslclass() failed in setup");

	DBUG_VOID_RETURN;
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	rval;
	int	i;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t1");

	testfail = 0;

	if (setlocale (LC_CTYPE, INFO1LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO1LANG);
		in_rpt("Check instalation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO1LANG;
	thisfail = 0;

	for (i = 0; i < 256; i++)
	{
		if (!(lang1_table[i][0]&DEFUPPER))
		{
			PATH_TRACE;
			continue;
		}

		if ((rval = tolower(i)) != lang1_table[i][1])
		{
			if (!testfail++)
				xx_rpt(FAILURE);
			if (!thisfail++)
				in_rpt("Current language is %s", currlang);
			in_rpt("tolower(0x%x) failed", i);
			in_rpt("Returned value 0x%x instead of 0x%x",
				rval, lang1_table[i][1]);
		}
		else
			PATH_TRACE;
	}

	if (setlocale (LC_CTYPE, INFO3LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO3LANG);
		in_rpt("Check instalation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO3LANG;
	thisfail = 0;

	for (i = 0; i < 256; i++)
	{
		if (!(lang3_table[i][0]&DEFUPPER))
		{
			PATH_TRACE;
			continue;
		}
		if ((rval = tolower(i)) != lang3_table[i][1])
		{
			if (!testfail++)
				xx_rpt(FAILURE);
			if (!thisfail++)
				in_rpt("Current language is %s", currlang);
			in_rpt("tolower(0x%x) failed", i);
			in_rpt("Returned value 0x%x instead of 0x%x",
				rval, lang3_table[i][1]);
		}
		else
			PATH_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(514);	

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	rval;
	int	i;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t2");

	testfail = 0;

	if (setlocale (LC_CTYPE, INFO1LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO1LANG);
		in_rpt("Check instalation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO1LANG;
	thisfail = 0;

	for (i = 0; i < 256; i++)
	{
		if (lang1_table[i][0]&DEFUPPER)
		{
			PATH_TRACE;
			continue;
		}
		if ((rval = tolower(i)) != i)
		{
			if (!testfail++)
				xx_rpt(FAILURE);
			if (!thisfail++)
				in_rpt("Current language is %s", currlang);
			in_rpt("tolower(0x%x) failed", i);
			in_rpt("Returned value 0x%x instead of 0x%x", rval, i);
		}
		else
			PATH_TRACE;
	}

	if (setlocale (LC_CTYPE, INFO3LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO3LANG);
		in_rpt("Check instalation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	currlang = INFO3LANG;
	thisfail = 0;

	for (i = 0; i < 256; i++)
	{
		if (lang3_table[i][0]&DEFUPPER)
		{
			PATH_TRACE;
			continue;
		}
		if ((rval = tolower(i)) != i)
		{
			if (!testfail++)
				xx_rpt(FAILURE);
			if (!thisfail++)
				in_rpt("Current language is %s", currlang);
			in_rpt("tolower(0x%x) failed", i);
			in_rpt("Returned value 0x%x instead of 0x%x", rval, i);
		}
		else
			PATH_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(514);	

	DBUG_VOID_RETURN;
}

private	void
test3()
{
	int	rval;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	currlang = "";
	thisfail = 0;

	for (i = 0; i < 256; i++)
	{
		if (!(asc_table[i][0]&DEFUPPER))
		{
			PATH_TRACE;
			continue;
		}
		if ((rval = tolower(i)) != asc_table[i][1])
		{
			if (!testfail++)
				xx_rpt(FAILURE);
			if (!thisfail++)
				in_rpt("No current language has been set");
			in_rpt("tolower(0x%x) failed", i);
			in_rpt("Returned value 0x%x instead of 0x%x",
				rval, asc_table[i][1]);
		}
		else	
			PATH_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(256);

	DBUG_VOID_RETURN;
}
