/*
*      SCCS:  @(#)  ANSI.os/diagnostics/assert/assert.c      Rel 4.4.1	   (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/diagnostics/assert - T.assert Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.assert

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
		Testset for the assert() macro.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 11 June 87
		Removed abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Phyfos Photi, Hoskyns Group plc,
		02/09/88
		test 2 added

		Geoff Clare, 3 Aug 1989
		Do not try to catch or ignore SIGABRT

		David Sawyer
		Mon Aug 06 11:53:30 BST 1990
		Added resilience code.

		Stuart Boutell, 30 Apr 1991
			cpsig return code check corrected

		Geoff Clare, 20 June 1994
			Allow for more output in test 1 than will fit in the
			fixed buffer (e.g. if assert() writes a stack trace).

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<assert.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<signal.h>
#include	<wait_child.h>
#include	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef no_functions
#endif


#define NO_TESTS	2		/* number of tests in testset */
#define WAITTIME	(2 * SPEEDFACTOR)

extern	char	*errname();
extern	char	*itoa();
extern	char	*status_child();
extern	void	(*vsignal())();

extern	int	sg_code;

int	no_tests = NO_TESTS;

private char	*tfile = "./assert_t";
private	int	linno;
private	int	testfail;

private void	test1();
private void	test2();
private void	ch_t1();
private void	ch_t2();
private void	pa_t1();
private void	pa_t2();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private void
ch_t1()
{
	int count;

	DBUG_ENTER("ch_t1");
	if (freopen(tfile, "w+", stderr) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not reopen stderr to file %s",
			tfile);
		DBUG_VOID_RETURN;
	}
	count = 0;

	(void) vsignal(SIGABRT, SIG_DFL);

	assert(count);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	static char	fixedbuf[1024];
	struct stat	st2buf;
	int 		fd;
	char		*buf = fixedbuf;
	unsigned int	bufsize = sizeof(fixedbuf);
	int 		ret;
	int		pathok = 0;

	DBUG_ENTER("test1");

	linno = __LINE__;
	/* if either assert or the line above are moved or new lines added */
	/* this value must be changed */
	linno -= 18;

	testfail = 0;

	globok = 0;
	if (cpsig(ch_t1, pa_t1, WAITTIME, 0) == SYSERROR)
		testfail++;
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = open(tfile, O_RDONLY, 0)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not open %s for reading", tfile);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fstat(fd, &st2buf);
	if (st2buf.st_size == 0)
	{
		xx_rpt(FAILURE);
		in_rpt("assert did not print a message to stderr");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (st2buf.st_size > bufsize)
	{
		bufsize = st2buf.st_size;
		if ((buf = (char *)malloc((size_t)bufsize)) == NULL)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: malloc(%u) failed", bufsize);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if ((ret = read(fd, buf, bufsize)) <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from %s", tfile);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	buf[ret] = '\0';
	if (strstr(buf, itoa(linno)) == (char *) NULL ||
		strstr(buf, __FILE__) == (char *) NULL ||
		strstr(buf, "count") == (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("message from assert did not have expected contents");
		in_rpt("expected to contain: %s, %s, %s", itoa(linno), __FILE__, "count");
		in_rpt("observed contents: %s", buf);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(6);
	}

	(void) unlink(tfile);
	(void) unlink("core");

	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	int 	stat_child, sav_errno;
	int	pathok = 0;

	DBUG_ENTER("pa_t1");
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR)
	{
		sav_errno = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", sav_errno);
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_ERR || stat_child == CHLD_LIVE)
	{
		(void) killchild(tet_child, WAITTIME);
		PATH_TRACE;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}



private void
test2()
{
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	globok = 0;
	if (cpsig(ch_t2, pa_t2, WAITTIME, 0) == SYSERROR)
		testfail++;
	else
		PATH_FUNC_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(1);

	(void) unlink(tfile);
	(void) unlink("core");

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int count = 0;

	DBUG_ENTER("ch_t2");

	(void) freopen(tfile, "w", stderr);
	(void) vsignal(SIGABRT, SIG_DFL);

	assert(count);

	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	int 	stat_child, sav_errno;
	int	pathok = 0;

	DBUG_ENTER("pa_t2");

	/* Wait for child and expect it to have aborted */
	if ((stat_child = wait_child(WAITTIME)) == CHLD_ERR)
	{
		sav_errno = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", sav_errno);
	}
	else if (stat_child != CHLD_ABRT || sg_code != SIGABRT)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: child aborted - abort signal %d",
			SIGABRT);
		in_rpt("observed action: %s", status_child(stat_child));
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_ERR || stat_child == CHLD_LIVE)
	{
		(void) killchild(tet_child, WAITTIME);
		PATH_TRACE;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}
