/*
*      SCCS:  @(#)ANSI.os/genuts/abort/abort.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/genuts/abort - T.abort Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.abort

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:	Testset for the abort() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Mon Apr  3 10:13:44 GMT 1989
MODIFICATIONS:
		Geoff Clare, 17 July 1989
			For tests requiring a background process group, if
			setpgid() is not supported, use setsid() instead.
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 16:11:16 BST 1989
			Remove references to SA_NOCLDSTOP in test6
			Wait for child process in test6 (was leaving zombie)
			Upgrade to correspond to other _exit() modifications.
		
		Geoff Clare, 5 Oct 1989
			Test both ways of creating orphaned process group
			with stopped process.
		
		Geoff Clare, 14 Feb 1990
			Add second sleep() in ch_sleep()
		
		Geoff Clare, 7 June 1990
			UNSUPPORTED result if VSX_TERMIOS_TTY=unsup

		Geoff Clare, 22 June 1990
			Add sleep() in ch_stop()

		David Sawyer
		Wed Jul 11 10:53:51 BST 1990
		Added resillience code.

		Stuart Boutell, 6 Nov 1990
			Corrected child problems

		Stuart Boutell, 7 Nov 1990
			Changed vg_sjmp to internal handler.
			Prefixed all abort() calls with
			vsignal(SIGABRT, SIG_DFL);

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called

		Stuart Boutell, 23 Apr 1991
			Added session relinquish test 11

		Stuart Boutell, 30 Apr 1991
			cpsig() return value check bug corrected

		Geoff Clare, 22 Nov 1991
			Changes to support pseudo-terminals

		Kieron Drake, 10 Jun 1992
			To test when SIGABRT blocked, ignored or caught &
			catcher returns. Hence three cases.

		Geoff Clare, 16 June 1992
			Don't test the three new cases in XPG3 mode.

		John-Paul, 22 September 1992
			XNFS enhancements.

		Geoff Clare, 20 Dec 1993
			Don't use DBUG_EXIT() in children of plain fork().
			Use TEST_ABORT() macro for tested abort() calls.

		Geoff Clare, 19 July 1995
			Reduce WAITTIME when testing XNFS.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<sys/wait.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<termios.h>
#include	<fcntl.h>
#include	<vsx_signal.h>
#include	<setjmp.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<errno.h>
#include 	<tet_api.h>
#include	<synch.h>
#include	<xnfs.h>
#include	"abort_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef abort
#endif



/* When testing XNFS, SPEEDFACTOR is increased in <std.h> to allow extra
   time for NFS file accesses.  However, this can cause some tests here
   to take longer than the auto-sync timeout on the server side.
   To avoid this the "normal" value is used by undoing the increase. */
#if TEST_XNFS
#define	WAITTIME		(5 * SPEEDFACTOR/4)
#else
#define	WAITTIME		(5 * SPEEDFACTOR)
#endif

#define	NO_TESTS		12		/* number of tests in testset */
#define	READ			0
#define	WRITE			1
#define	CREATMODE		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	WAIT			1
#define	WAITPID			2
#define	TEST_SUCCESS		1
#define	TEST_FAIL		2
#define	TEST_SIGNAL		3
#define	TEST_SIGACTION		4
#define	TEST_SETPGID		5
#define	TEST_KILL		6
#define	TEST_PPID		7

#define NO			0
#define YES			1
#define	NAP			4
#define	PATH_NOK		04
#define	NOK			02

#if TEST_XNFS && !defined(DTET1)
#define TEST_ABORT()		(tet_logoff(), abort())
#else
#define TEST_ABORT()		abort()
#endif

extern	void		vsx_sigs();
extern	void		(*(vsignal)())();
extern	void		valrpt();
extern	char		*status_child();
extern	char		*errname();
extern	void		setsigs();

private	sigjmp_buf	sjmp;
extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sg_code;	/* defined in wait_child.c */
extern	int		ex_code;	/* defined in wait_child.c */

int	no_tests = NO_TESTS;

#if !TEST_XNFS
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*tty_termios_p = &tty_termios,
		*loop_termios_p = &loop_termios;

#endif /* TEST_XNFS */
private	char		*t2_lock = "abort-t.2lock";
private	char		*t4_file = "abort-t.4file";
private	int		pipe_fd[2];
private synch		*syn;
private	int		fg_fd[2];
private	int		bg_fd[2];
private	int		lock_fd;
private	struct flock 	lock_info;
private	sigset_t	sig_empty;
private	int		nsigs;
private	int		test_fail;
private	int		test_deleted;
private	void		(*sig_arg)();
private	int		sig_block;
private	int		sig_recvd;
private	int		sig_cont;
private	int		job_control;
private	int		filelocking;
#if !TEST_XNFS
private	unsigned char	chr;
private int		reacquiring;
#endif /* TEST_XNFS */
private int		setupok;

private	void		test1();
private	void		ch_t1();
private	void		pa_t1();
private	void		test2();
private	void		t2_init();
private	void		t2_cln();
private	void		ch_t2();
private	void		pa_t2();
private	void		test3();
private	void		ch_t3();
private	void		pa_t3();
private	void		test4();
private	void		ch_t4();
private	void		pgch_t4();
private	void		gch_t4();
private	void		pa_t4();
private	void		test5();
private	void		t5_init();
private	void		t5_cln();
private	void		ch_t5();
private	void		pgch_t5();
private	void		gch_t5();
private	void		pa_t5();
private	void		test6();
private	void		ch_t6();
private	void		pa_t6();
private	void		test7();
private	void		ch_t7();
private	void		pgch_t7();
private	void		ch_t7fg();
private	void		ch_t7bg();
private	void		pa_t7();
private	void		test8();
private	void		ch_t8();
private	void		pgch_t8();
private	void		pa_t8();
private	void		test9();
private	void		ch_t9();
private	void		pgch_t9();
private	void		pa_t9();
private	void		test10();
private	void		ch_t10();
private	void		pa_t10();
private	void		test11();
private	void		ch_t11();
private	void		pa_t11();
private	void		test12();
#if TEST_XNFS
private	void		ch_t12();
private	void		pa_t12();
#endif /* TEST_XNFS */

private	void		prepare_tests();
private	void		clean_tests();
private	void		sig_catch();
private	void		sig_catch_child();
private	void		sig_resume();
private	int		sig_init();
private	void		ch_stop();
private	void		ch_sleep();
private	void		sjmpfunc();

private	int		ctty();
private	void		interrupt();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	NULL, 0
};

#if TEST_XNFS
private int	snames[] = { 1, 0 };
#endif /* TEST_XNFS */

private	void
prepare_tests()
{
	int	i;	/* Counter */
	int	ret;	/* Used for most return values */
	int 	pathok = 0;

	DBUG_ENTER("prepare_tests");

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	if(uids()!=0)
	{
		for(i = 1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		job_control = 0;
		nsigs = VNOJSIG;
	} else {
		job_control = 1;
		nsigs = VNUMSIG;
	}

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);

	if (pathok==2)
		setupok = 0;
	else
		setupok = -1;

	DBUG_VOID_RETURN;
}

private	void
clean_tests()
{
	(void) unlink("core");
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

private void
sig_catch_child(sg_num)
int	sg_num;
{
	int err;
	int loc_sig_recvd;	/* NOTE: avoids global as we don't want to hit that.
				 If we update the global then we can clash with
				 sig_catch (e.g. for SIGINT in test11). As it
				 is we always write loc_sig_recvd to topmost
				 parent which usually reads it into its global
				 sig_recvd.
			 */
	DBUG_ENTER("sig_catch_child");

	loc_sig_recvd = sg_num;

	/* don't use sendsync() as don't want SIGALRM as well */

	if(write(syn[0][WRITE], (char *)&loc_sig_recvd, sizeof(loc_sig_recvd)) != sizeof(loc_sig_recvd)) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to pipe failed with errno %d", err);
		DBUG_EXIT(A_TSETDIR);
	}

	DBUG_PRINTF("sig_catch_child", "loc_sig_recvd %d", loc_sig_recvd);

	DBUG_VOID_RETURN;
}

private void
sig_resume(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_resume");

	sig_cont = sg_num;

	DBUG_PRINTF("sig_resume", "sig_cont %d", sig_cont);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = setupok;

	DBUG_ENTER("test1");

	/*
	 * Test that abort() terminates the process.
	 */

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if(cpsig(ch_t1, pa_t1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if(cpsig(ch_t1, pa_t1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	sig_arg = SIG_IGN;
	if(cpsig(ch_t1, pa_t1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if ((syn = opensync()) == (synch *)NULL) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - opensync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	sig_recvd = 0;
	sig_arg = sig_catch_child;
	if(cpsig(ch_t1, pa_t1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if (closesync(syn) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - closesync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	if(!test_fail)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
        sigset_t set;
        int     sg_num  = sig_array[VSIGABRT];

	DBUG_ENTER("ch_t1");

	(void) vsignal(sg_num, sig_arg);
	if(sig_arg == SIG_DFL && sig_block) {

		/* Call sigprocmask() to block the signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigprocmask() failed");
			DBUG_EXIT(A_TSETDIR);
		}
	}

	TEST_ABORT();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	int	stat_child;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t1");

	/* Wait for child and expect it to have aborted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_TSETDIR) {
			test_deleted++;
			DBUG_VOID_RETURN;
		} else {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("child process continued after calling abort()");
			test_fail++;
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

	/* if we set a catcher then if executed assume it also returned */
	if(sig_arg == sig_catch_child && (
	    (ret=waitsync(syn[0][READ],(char *)&sig_recvd,sizeof(sig_recvd),WAITTIME) != sizeof(sig_recvd))
	    || sig_recvd != SIGABRT)
	  ) {
		err = errno;
		xx_rpt(DELETION);
		if (ret == sizeof(sig_recvd))
			in_rpt("deletion reason - unexpected signal %d caught instead of SIGABRT (%d)", sig_recvd, SIGABRT);
		else if (ret == SYSERROR && err == EINTR) /* timed out */
			in_rpt("deletion reason - SIGABRT catcher not called");
		else {
			in_rpt("deletion reason - waitsync error");
			valrpt(sizeof(sig_recvd), ret, EINTR, err);
		}
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("child process still active %d seconds after calling abort()", WAITTIME);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to send the signal to the process that's just disappeared */
	errno = NOERROR;
	ret = kill(tet_child, SIGKILL);
	err = errno;

	if(ret != SYSERROR || err != ESRCH) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("kill on process that has called abort() did not return expected value");
		valrpt(SYSERROR, ret, ESRCH, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	pathok = setupok;
#if	   TEST_XNFS
	char	*rlock;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test2");

	/*
	 * Test that abort() closes all file descriptors.
	 */

	test_fail = 0;
	test_deleted = 0;
	filelocking = 1;	/* file locking is supported */

#if	   TEST_XNFS
	if ((rlock = tet_getvar("XNFS_REMLOCK_SUPP")) == NULL || *rlock == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter XNFS_REMLOCK_SUPP is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(*rlock == 'N' || *rlock == 'n')
		filelocking = 0;

#else	/* TEST_XNFS */

	PATH_TRACE;

#endif	/* TEST_XNFS */

	globok = 0;
	t2_init();
	PATH_FUNC_TRACE;

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if(cpsig(ch_t2, pa_t2, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	t2_cln();
	
	if(test_deleted)
		DBUG_VOID_RETURN;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 0;
	t2_init();
	PATH_FUNC_TRACE;

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if(cpsig(ch_t2, pa_t2, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	t2_cln();
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	t2_init();
	PATH_FUNC_TRACE;

	globok = 0;
	sig_arg = SIG_IGN;
	if(cpsig(ch_t2, pa_t2, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	t2_cln();
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	t2_init();
	PATH_FUNC_TRACE;

	if ((syn = opensync()) == (synch *)NULL) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - opensync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	sig_recvd = 0;
	sig_arg = sig_catch_child;
	if(cpsig(ch_t2, pa_t2, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	t2_cln();

	if (closesync(syn) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - closesync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	if(!test_fail)
		PATH_XS_RPT(11);

	DBUG_VOID_RETURN;
}

private void
t2_init()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("t2_init");

	if(filelocking)
	{
		/* Create and open a lock file */
		errno = NOERROR;
		if((lock_fd = creat(t2_lock, CREATMODE)) < 0) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: creat() failed - errno %d", err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	/* Open a pipe before forking */
	errno = NOERROR;
	if(pipe(pipe_fd) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
t2_cln()
{
	DBUG_ENTER("t2_cln");

	(void) close(pipe_fd[READ]);
	(void) close(pipe_fd[WRITE]);

	if(filelocking)
	{
		(void) close(lock_fd);
		(void) unlink(t2_lock);
	}

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
        sigset_t set;
        int     sg_num  = sig_array[VSIGABRT];

	DBUG_ENTER("ch_t2");

	if(filelocking)
	{
		/* Set a lock for the parent to detect, then sleep */
		lock_info.l_type = F_WRLCK;
		lock_info.l_whence = SEEK_SET;
		lock_info.l_start = (off_t)0;
		lock_info.l_len = (off_t)1;
		(void)fcntl(lock_fd, F_SETLK, &lock_info);
	}

	(void) sleep(WAITTIME);

	(void)	vsignal(SIGABRT, sig_arg);
	if(sig_arg == SIG_DFL && sig_block) {

		/* Call sigprocmask() to block the signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigprocmask() failed");
			DBUG_EXIT(A_TSETDIR);
		}
	}
	TEST_ABORT();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	char	buffer[1];
	int	stat_child;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t2");

	/* The child locks a file and sleeps for WAITTIME */
	(void) sleep(WAITTIME/2);

	if(filelocking)
	{
		/* Obtain information on lock file */
		lock_info.l_type = F_WRLCK;
		lock_info.l_whence = SEEK_SET;
		lock_info.l_start = (off_t)0;
		lock_info.l_len = (off_t)1;
		errno = NOERROR;
		ret = fcntl(lock_fd, F_GETLK, &lock_info);
		if(ret == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - fcntl() failed errno %d", err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Check file really is locked */
		if(lock_info.l_pid != tet_child || lock_info.l_type != F_WRLCK) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - file descriptor not locked by child process");
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	/* Wait for child and expect it to have aborted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_TSETDIR) {
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* if we set a catcher then if executed assume it also returned */
	if(sig_arg == sig_catch_child && (
	    (ret=waitsync(syn[0][READ],(char *)&sig_recvd,sizeof(sig_recvd),WAITTIME) != sizeof(sig_recvd))
	    || sig_recvd != SIGABRT)
	  ) {
		err = errno;
		xx_rpt(DELETION);
		if (ret == sizeof(sig_recvd))
			in_rpt("deletion reason - unexpected signal %d caught instead of SIGABRT (%d)", sig_recvd, SIGABRT);
		else if (ret == SYSERROR && err == EINTR) /* timed out */
			in_rpt("deletion reason - SIGABRT catcher not called");
		else {
			in_rpt("deletion reason - waitsync error");
			valrpt(sizeof(sig_recvd), ret, EINTR, err);
		}
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("child process still active %d seconds after calling abort()", WAITTIME);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	/* Initialise the SIGPIPE action */
	if(sig_init(sig_array[VSIGPIPE], &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Close down the read side of the pipe */
	errno = NOERROR;
	ret = close(pipe_fd[READ]);
	if(ret == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - close() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(filelocking)
	{
		/* Obtain information on lock file */
		lock_info.l_type = F_WRLCK;
		lock_info.l_whence = SEEK_SET;
		lock_info.l_start = (off_t)0;
		lock_info.l_len = (off_t)1;
		errno = NOERROR;
		ret = fcntl(lock_fd, F_GETLK, &lock_info);
		if(ret == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - fcntl() failed errno %d", err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Check the file is no longer locked by the child */
		if(lock_info.l_type != F_UNLCK) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("file descriptor still locked after abort() called");
			test_fail++;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	/* Try to write to the write side of the pipe */
	sig_recvd = 0;
	errno = NOERROR;
	ret = write(pipe_fd[WRITE], buffer, sizeof(char));
	err = errno;
	if(ret != SYSERROR || err != EPIPE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("file descriptor still open for reading after abort() called");
		in_rpt("write() to pipe in parent process did not return expected value");
		valrpt(SYSERROR, ret, EPIPE, err);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(sig_recvd != SIGPIPE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal SIGPIPE not received when attempting to write to pipe");
		in_rpt("which is not open for reading by any process");
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(12);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int	pathok = setupok;

	DBUG_ENTER("test3");

	/*
	 * Test that abort() is reported to waiting parent process.
	 */

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if(cpsig(ch_t3, pa_t3, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if(cpsig(ch_t3, pa_t3, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;


	globok = 0;
	sig_arg = SIG_IGN;
	if(cpsig(ch_t3, pa_t3, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if ((syn = opensync()) == (synch *)NULL) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - opensync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	sig_recvd = 0;
	sig_arg = sig_catch_child;
	if(cpsig(ch_t3, pa_t3, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if (closesync(syn) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - closesync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	if(!test_fail)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
        sigset_t set;
        int     sg_num  = sig_array[VSIGABRT];

	DBUG_ENTER("ch_t3");

	/* Sleep a while to give parent chance to wait() for child */
	(void) sleep(WAITTIME/2);

	(void) vsignal(SIGABRT, sig_arg);
	if(sig_arg == SIG_DFL && sig_block) {

		/* Call sigprocmask() to block the signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigprocmask() failed");
			DBUG_EXIT(A_TSETDIR);
		}
	}
	TEST_ABORT();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int	stat_child;
	int	err;
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("pa_t3");

	/* Wait for child and expect it to have aborted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_TSETDIR) {
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* if we set a catcher then if executed assume it also returned */
	if(sig_arg == sig_catch_child && (
	    (ret=waitsync(syn[0][READ],(char *)&sig_recvd,sizeof(sig_recvd),WAITTIME) != sizeof(sig_recvd))
	    || sig_recvd != SIGABRT)
	  ) {
		err = errno;
		xx_rpt(DELETION);
		if (ret == sizeof(sig_recvd))
			in_rpt("deletion reason - unexpected signal %d caught instead of SIGABRT (%d)", sig_recvd, SIGABRT);
		else if (ret == SYSERROR && err == EINTR) /* timed out */
			in_rpt("deletion reason - SIGABRT catcher not called");
		else {
			in_rpt("deletion reason - waitsync error");
			valrpt(sizeof(sig_recvd), ret, EINTR, err);
		}
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("parent not notified of child calling abort()");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != SIGABRT) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("status did not show process terminated by SIGABRT");
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	pathok = setupok;

	DBUG_ENTER("test4");

	/*
	 * Test that abort() does not affect child processes.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) unlink(t4_file);

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if(cpsig(ch_t4, pa_t4, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	(void) unlink(t4_file);

	if(test_deleted)
		DBUG_VOID_RETURN;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if(cpsig(ch_t4, pa_t4, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	(void) unlink(t4_file);

	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	sig_arg = SIG_IGN;
	if(cpsig(ch_t4, pa_t4, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	(void) unlink(t4_file);

	if(test_deleted)
		DBUG_VOID_RETURN;

	if ((syn = opensync()) == (synch *)NULL) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - opensync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	sig_recvd = 0;
	sig_arg = sig_catch_child;
	if(cpsig(ch_t4, pa_t4, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	(void) unlink(t4_file);

	if(test_deleted)
		DBUG_VOID_RETURN;

	if (closesync(syn) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - closesync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	if(!test_fail)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	DBUG_ENTER("ch_t4");

	/*
	 * Turn into a grandchild and it's parent.
	 * The parent aborts immediately.
	 */

	(void) cppair(gch_t4, pgch_t4, WAITTIME, 0);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch_t4()
{
        sigset_t set;
        int     sg_num  = sig_array[VSIGABRT];

	DBUG_ENTER("pgch_t4");

	/* Exit straight away to leave our child orphaned */
	(void)	vsignal(SIGABRT, sig_arg);
	if(sig_arg == SIG_DFL && sig_block) {

		/* Call sigprocmask() to block the signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigprocmask() failed");
			DBUG_EXIT(A_TSETDIR);
		}
	}
	TEST_ABORT();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
gch_t4()
{
	DBUG_ENTER("gch_t4");

	/* Sleep, during which time the parent calls abort() */
	(void) sleep(WAITTIME);

	/* Create a test file */
	(void) creat(t4_file, CREATMODE);

	/* This is not the abort() tested - it is the one in pgch_t4() */
	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t4()
{
	int	stat_child;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t4");

	/* Wait for child and expect it to have aborted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME/2)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_TSETDIR) {
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* if we set a catcher then if executed assume it also returned */
	if(sig_arg == sig_catch_child && (
	    (ret=waitsync(syn[0][READ],(char *)&sig_recvd,sizeof(sig_recvd),WAITTIME) != sizeof(sig_recvd))
	    || sig_recvd != SIGABRT)
	  ) {
		err = errno;
		xx_rpt(DELETION);
		if (ret == sizeof(sig_recvd))
			in_rpt("deletion reason - unexpected signal %d caught instead of SIGABRT (%d)", sig_recvd, SIGABRT);
		else if (ret == SYSERROR && err == EINTR) /* timed out */
			in_rpt("deletion reason - SIGABRT catcher not called");
		else {
			in_rpt("deletion reason - waitsync error");
			valrpt(sizeof(sig_recvd), ret, EINTR, err);
		}
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("child process still active %d seconds after calling abort()", WAITTIME);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* 
	 * Sleep for a while and check for recently created test file.
	 * Presence of the test file means the grandchild did not sleep 
	 * as long as it should have done.
	 */
	(void) sleep(WAITTIME/2);
	
	errno = NOERROR;
	ret = open(t4_file, O_RDONLY);
	if(ret != SYSERROR) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sleep() interrupted when parent called abort()");
		(void) close(ret);
		test_fail++;
		DBUG_VOID_RETURN;
	} 
	else {
		PATH_TRACE;
		if(errno != ENOENT) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - 1. open() failed errno %d", err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	/* 
	 * Sleep for a while and check grandchild has been active since
	 * the existence of the test file was checked.
	 * This is proved by the existence of a file created by grandchild.
	 */
	(void) sleep(WAITTIME);

	errno = NOERROR;
	ret = open(t4_file, O_RDONLY);
	if(ret == SYSERROR && errno == ENOENT) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("file not created by process after parent called abort()");
		test_fail++;
		DBUG_VOID_RETURN;
	} 
	else {
		PATH_TRACE;
		if(ret == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - 2. open() failed errno %d", err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}	
	(void) close(ret);
	PATH_FUNC_RPT(8);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	int pathok = setupok;

	DBUG_ENTER("test5");

	/*
	 * Test that abort() causes children to acquire new parent process ID.
	 */

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	t5_init();
	PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if(cpsig(ch_t5, pa_t5, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if(cpsig(ch_t5, pa_t5, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	sig_arg = SIG_IGN;
	if(cpsig(ch_t5, pa_t5, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if ((syn = opensync()) == (synch *)NULL) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - opensync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	sig_recvd = 0;
	sig_arg = sig_catch_child;
	if(cpsig(ch_t5, pa_t5, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if (closesync(syn) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - closesync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	t5_cln();

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
t5_init()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("t5_init");

	/* Open a pipe before forking */
	errno = NOERROR;
	if(pipe(pipe_fd) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
t5_cln()
{
	DBUG_ENTER("t5_cln");

	(void)close(pipe_fd[READ]);
	(void)close(pipe_fd[READ]);

	DBUG_VOID_RETURN;
}
private void
ch_t5()
{
	DBUG_ENTER("ch_t5");

	/*
	 * Turn into a grandchild and it's parent.
	 * The parent aborts after a sleep during which time grandchild obtains 
	 * parent process ID.
	 * The grandchild obtains new parent process ID after it's parent
	 * calls abort(), expecting it to have altered.
	 */

	(void) cppair(gch_t5, pgch_t5, WAITTIME, 0);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch_t5()
{
        sigset_t set;
        int     sg_num  = sig_array[VSIGABRT];

	DBUG_ENTER("pgch_t5");

	/* Exit after a short pause long enough for child to call getppid() */
	(void) sleep(WAITTIME/2);

	(void)	vsignal(SIGABRT, sig_arg);
	if(sig_arg == SIG_DFL && sig_block) {

		/* Call sigprocmask() to block the signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigprocmask() failed");
			DBUG_EXIT(A_TSETDIR);
		}
	}
	TEST_ABORT();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
gch_t5()
{
	pid_t	ppid;
	pid_t	ppid_old;
	int	pathok = setupok;

	DBUG_ENTER("gch_t5");

	ppid_old = getppid();

	/* Sleep, during which time the parent calls abort() */
	(void) sleep(WAITTIME);

	ppid = getppid();

	if(ppid == ppid_old) {
		if (!test_fail)
			xx_rpt(FAILURE);
		in_rpt("parent process ID not altered after parent called abort()");
		test_fail++;
	} else {
		PATH_TRACE;
		PATH_FUNC_RPT(1); /* need to ensure that setupok was valid */
	}

	/* now write the values of globok, test_fail & test_deleted to pipe */
	(void)write(pipe_fd[WRITE], (char *)&globok, sizeof(globok));
	(void)write(pipe_fd[WRITE], (char *)&test_fail, sizeof(test_fail));
	(void)write(pipe_fd[WRITE], (char *)&test_deleted, sizeof(test_deleted));

	/* This is not the exit() tested - it is the one in pgch_t5() */
	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t5()
{
	int	stat_child;
	int	err;
	int	ret;
	int	pathok = 0;
	int	g_ok;
	int	t_fail;
	int	t_del;

	DBUG_ENTER("pa_t5");

	/* Wait for child and expect it to have aborted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != SIGABRT) {
		xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: child aborted - abort signal %d", SIGABRT);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* if we set a catcher then if executed assume it also returned */
	if(sig_arg == sig_catch_child && (
	    (ret=waitsync(syn[0][READ],(char *)&sig_recvd,sizeof(sig_recvd),WAITTIME) != sizeof(sig_recvd))
	    || sig_recvd != SIGABRT)
	  ) {
		err = errno;
		xx_rpt(DELETION);
		if (ret == sizeof(sig_recvd))
			in_rpt("deletion reason - unexpected signal %d caught instead of SIGABRT (%d)", sig_recvd, SIGABRT);
		else if (ret == SYSERROR && err == EINTR) /* timed out */
			in_rpt("deletion reason - SIGABRT catcher not called");
		else {
			in_rpt("deletion reason - waitsync error");
			valrpt(sizeof(sig_recvd), ret, EINTR, err);
		}
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* Sleep further to give grandchild time to report */
	(void) sleep(WAITTIME);

	/* now collect results from grandchild via pipe */
	if(waitsync(pipe_fd[READ], (char *)&g_ok, sizeof(g_ok), WAITTIME) != sizeof(g_ok) ||
	   waitsync(pipe_fd[READ], (char *)&t_fail, sizeof(t_fail), WAITTIME) != sizeof(t_fail) ||
	   waitsync(pipe_fd[READ], (char *)&t_del, sizeof(t_del), WAITTIME) != sizeof(t_del)) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitsync() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	test_fail += t_fail;
	test_deleted += t_del;
	globok = g_ok;
	
	/* check grandchild's updating of globok */
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	int	pathok = setupok;

	DBUG_ENTER("test6");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}

	/*
	 * Test that abort() sends SIGCHLD to parent process.
	 */

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if(cpsig(ch_t6, pa_t6, 3*WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if(cpsig(ch_t6, pa_t6, 3*WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	sig_arg = SIG_IGN;
	if(cpsig(ch_t6, pa_t6, 3*WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if ((syn = opensync()) == (synch *)NULL) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - opensync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	sig_arg = sig_catch_child;
	/* sig_recvd set and tested in pa_t6 */
	if(cpsig(ch_t6, pa_t6, 3*WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if (closesync(syn) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - closesync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	if(!test_fail)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
        sigset_t set;
        int     sg_num  = sig_array[VSIGABRT];

	/*
	 * Child process which aborts.
	 * This should cause parent to receive signal SIGCHLD.
	 */
	DBUG_ENTER("ch_t6");

	(void) sleep(WAITTIME/2);

	(void)	vsignal(SIGABRT, sig_arg);
	if(sig_arg == SIG_DFL && sig_block) {

		/* Call sigprocmask() to block the signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigprocmask() failed");
			DBUG_EXIT(A_TSETDIR);
		}
	}
	TEST_ABORT();

	DBUG_VOID_RETURN;
}

private void
pa_t6()
{
	int	stat_child;
	int	err;
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("pa_t6");

	/* Set signal catching function for SIGCHLD */
	if(sig_init(SIGCHLD, &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	sig_recvd = 0;

	(void) sleep(WAITTIME);

	/* Check we received signal */
	if(sig_recvd != SIGCHLD) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("SIGCHLD signal not generated when child aborted");
		if(sig_recvd)
			in_rpt("instead signal %d (%s) was generated", 
						sig_recvd, sig_name[sig_recvd]);

		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Set default for SIGCHLD so we get no more interruptions */
	if(sig_init(SIGCHLD, &sig_empty, SIG_DFL, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait for child process so we don't create a zombie */
	if((stat_child=wait_child(WAITTIME)) != CHLD_ABRT || sg_code != SIGABRT) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - status did not show child terminated by SIGABRT");
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* if we set a catcher then if executed assume it also returned */
	sig_recvd = 0;
	if(sig_arg == sig_catch_child && (
	    (ret=waitsync(syn[0][READ],(char *)&sig_recvd,sizeof(sig_recvd),WAITTIME) != sizeof(sig_recvd))
	    || sig_recvd != SIGABRT)
	  ) {
		err = errno;
		xx_rpt(DELETION);
		if (ret == sizeof(sig_recvd))
			in_rpt("deletion reason - unexpected signal %d caught instead of SIGABRT (%d)", sig_recvd, SIGABRT);
		else if (ret == SYSERROR && err == EINTR) /* timed out */
			in_rpt("deletion reason - SIGABRT catcher not called");
		else {
			in_rpt("deletion reason - waitsync error");
			valrpt(sizeof(sig_recvd), ret, EINTR, err);
		}
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test7()
{
	DBUG_ENTER("test7");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test7()
{
	char	*cp;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test7");

	/*
	 * Test that abort() sends SIGHUP to foreground processes.
	 */

	cp = tet_getvar("VSX_TERMIOS_TTY");
	if (cp != NULL && strncmp(cp, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	if(pipe(fg_fd) < 0 || pipe(bg_fd) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if(cpsig(ch_t7, pa_t7, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if(cpsig(ch_t7, pa_t7, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	sig_arg = SIG_IGN;
	if(cpsig(ch_t7, pa_t7, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if ((syn = opensync()) == (synch *)NULL) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - opensync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	sig_arg = sig_catch_child;
	sig_recvd = 0;
	if(cpsig(ch_t7, pa_t7, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if (closesync(syn) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - closesync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	(void) close(fg_fd[READ]);
	(void) close(fg_fd[WRITE]);
	(void) close(bg_fd[READ]);
	(void) close(bg_fd[WRITE]);

	if (!test_fail)
		PATH_XS_RPT(8);
	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	DBUG_ENTER("ch_t7");

	/*
	 * Turn into two grandchildren and parent.
	 * The parent aborts after a sleep.
	 * The grandchildren report to the grandparent on what happens.
	 */

	/* Set up a new session */
	if(setsid() == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("setsid() failed in child");
		DBUG_EXIT(OK);
	}

	/* Set up a controlling terminal for this session */
	/* Note: VSX_TERMIOS_TTY=unsup has already been checked for */
	if (tty_prep(&tty_fildes, tty_termios_p) != 0) {
		xx_rpt(DELETION);
		in_rpt("open/initialise failed on VSX_TERMIOS_TTY");
		DBUG_EXIT(OK);
	}

	/* Produce a process in foreground process group */
	switch(fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		ch_t7fg();

		/* NOTREACHED */
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		break;
	}
	tet_setblock();

	/* Produce a process in background process group */
	switch(fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		ch_t7bg();

		/* NOTREACHED */
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		break;
	}
	tet_setblock();

	pgch_t7();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch_t7()
{
        sigset_t set;
        int     sg_num  = sig_array[VSIGABRT];

	DBUG_ENTER("pgch_t7");

	/* Exit after a short pause long enough to call sigaction() */
	(void) sleep(WAITTIME/2);

	(void)	vsignal(SIGABRT, sig_arg);
	if(sig_arg == SIG_DFL && sig_block) {

		/* Call sigprocmask() to block the signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigprocmask() failed");
			DBUG_EXIT(A_TSETDIR);
		}
	}
	TEST_ABORT();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
ch_t7fg()
{
	char	buffer[1];

	DBUG_ENTER("ch_t7fg");

	if(DBUG_SETJMP(sjmp) != 0) {
		buffer[0] = TEST_SIGNAL;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	setsigs(sjmpfunc);

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}

	sig_recvd = 0;

	/* Sleep, during which time the parent calls abort() */
	(void) sleep(WAITTIME);

	/* Expect signal SIGHUP to have been received */
	if(sig_recvd != SIGHUP) {
		buffer[0] = TEST_FAIL;
	} else {
		buffer[0] = TEST_SUCCESS;
	}

	/* Write down pipe to grandparent success code */
	(void) write(fg_fd[WRITE], buffer, sizeof(char));

	DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
ch_t7bg()
{
	char	buffer[1];
	pid_t	ppid;
	pid_t	ppid_old;

	DBUG_ENTER("ch_t7bg");

	if(DBUG_SETJMP(sjmp) != 0) {
		buffer[0] = TEST_SIGNAL;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	setsigs(sjmpfunc);

	/* Obtain parent process ID */
	ppid_old = getppid();

	/* Create a new (background) process group for this process */
	errno = 0;
	if(setpgid((pid_t)0, (pid_t)0) < 0) {
		if (errno != ENOSYS) {
			buffer[0] = TEST_SETPGID;
			(void) write(bg_fd[WRITE], buffer, sizeof(char));
			DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
		}
		(void) setsid();
	}

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}

	sig_recvd = 0;

	/* Sleep, during which time the parent calls abort() */
	(void) sleep(WAITTIME);

	/* Obtain new parent process ID */
	ppid = getppid();

	if(ppid == ppid_old) {
		buffer[0] = TEST_PPID;
	} else {
		/* Expect no SIGHUP signal to have been received */
		if(sig_recvd == SIGHUP)
			buffer[0] = TEST_FAIL;
		else
			buffer[0] = TEST_SUCCESS;
	}

	/* Write down pipe to grandparent success code */
	(void) write(bg_fd[WRITE], buffer, sizeof(char));

	DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t7()
{
	int	stat_child;
	int	ret;
	int	err;
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("pa_t7");

	/* Wait for child and expect it to have aborted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != SIGABRT) {
		xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: child aborted - abort signal %d", SIGABRT);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* if we set a catcher then if executed assume it also returned */
	if(sig_arg == sig_catch_child && (
	    (ret=waitsync(syn[0][READ],(char *)&sig_recvd,sizeof(sig_recvd),WAITTIME) != sizeof(sig_recvd))
	    || sig_recvd != SIGABRT)
	  ) {
		err = errno;
		xx_rpt(DELETION);
		if (ret == sizeof(sig_recvd))
			in_rpt("deletion reason - unexpected signal %d caught instead of SIGABRT (%d)", sig_recvd, SIGABRT);
		else if (ret == SYSERROR && err == EINTR) /* timed out */
			in_rpt("deletion reason - SIGABRT catcher not called");
		else {
			in_rpt("deletion reason - waitsync error");
			valrpt(sizeof(sig_recvd), ret, EINTR, err);
		}
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* Read from pipes to determine success of grandchildren */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != 1) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d",
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_SETPGID:
		xx_rpt(DELETION);
		in_rpt("deletion reason - setpgid() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling abort() did not send signal SIGHUP to foreground process group");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_SETPGID:
		xx_rpt(DELETION);
		in_rpt("deletion reason - setpgid() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_PPID:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("parent process ID not reassigned in background process group");
		test_fail++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling abort() sent signal SIGHUP to background process group");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	PATH_FUNC_RPT(7);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test8()
{
	int	pathok  = setupok;
	int	err;

	DBUG_ENTER("test8");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test that abort() sends SIGHUP and SIGCONT if process is stopped
	 * when a process group is orphaned.
	 */

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	if(pipe(fg_fd) < 0 ||
	   pipe(bg_fd) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if(cpsig(ch_t8, pa_t8, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if(cpsig(ch_t8, pa_t8, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	sig_arg = SIG_IGN;
	if(cpsig(ch_t8, pa_t8, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if ((syn = opensync()) == (synch *)NULL) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - opensync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	sig_arg = sig_catch_child;
	sig_recvd = 0;
	if(cpsig(ch_t8, pa_t8, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if (closesync(syn) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - closesync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	(void) close(fg_fd[READ]);
	(void) close(fg_fd[WRITE]);
	(void) close(bg_fd[READ]);
	(void) close(bg_fd[WRITE]);

	if(!test_fail)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	DBUG_ENTER("ch_t8");

	/*
	 * Do everything that follows in background process group.
	 */

	errno = 0;
	if(setpgid((pid_t)0, (pid_t)0) < 0) {
		if (errno != ENOSYS) {
			vsxerror("setpgid(0, 0) failed - errno %d (%s)",
				errno, errname(errno));
			DBUG_EXIT(A_MISC);
		}
		(void) setsid();
	}

	/*
	 * Creat process group with one stopped process and one
	 * sleeping process.  Then orphan it by exiting this process
	 * (the group leader).
	 * The grandchildren report to the grandparent on what happens.
	 */

	switch(tet_child = fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		ch_stop();
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		break;
	}
	tet_setblock();

	switch(fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		ch_sleep();
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		break;
	}
	tet_setblock();

	pgch_t8();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch_t8()
{
        sigset_t set;
        int     sg_num  = sig_array[VSIGABRT];

	DBUG_ENTER("pgch_t8");

	/* Exit after a short pause long enough to initialise children */
	(void) sleep(WAITTIME/2);

	/* This is the call that is actually tested */
	(void)	vsignal(SIGABRT, sig_arg);
	if(sig_arg == SIG_DFL && sig_block) {

		/* Call sigprocmask() to block the signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigprocmask() failed");
			DBUG_EXIT(A_TSETDIR);
		}
	}
	TEST_ABORT();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t8()
{
	int	stat_child;
	int	ret;
	int	err;
	pid_t	pid;
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("pa_t8");

	/* Wait for child and expect it to have aborted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
	}
	else {
		PATH_TRACE;
		if(stat_child != CHLD_ABRT || sg_code != SIGABRT) {
			xx_rpt(DELETION);
			in_rpt("child process terminated incorrectly");
			in_rpt("expected action: child aborted - abort signal %d", SIGABRT);
			in_rpt("observed action: %s", status_child(stat_child));
			test_deleted++;
		}
		else
			PATH_TRACE;
	}

	/* if we set a catcher then if executed assume it also returned */
	if(sig_arg == sig_catch_child && (
	    (ret=waitsync(syn[0][READ],(char *)&sig_recvd,sizeof(sig_recvd),WAITTIME) != sizeof(sig_recvd))
	    || sig_recvd != SIGABRT)
	  ) {
		err = errno;
		xx_rpt(DELETION);
		if (ret == sizeof(sig_recvd))
			in_rpt("deletion reason - unexpected signal %d caught instead of SIGABRT (%d)", sig_recvd, SIGABRT);
		else if (ret == SYSERROR && err == EINTR) /* timed out */
			in_rpt("deletion reason - SIGABRT catcher not called");
		else {
			in_rpt("deletion reason - waitsync error");
			valrpt(sizeof(sig_recvd), ret, EINTR, err);
		}
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* Read from pipe to determine PID of stopped grandchild */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], (char *)&pid, sizeof(pid_t));
	CLEAR_ALARM
	if(ret != sizeof(pid_t)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(pid_t));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Read from pipes to determine success of grandchildren */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		(void) killchild(pid, WAITTIME);
		test_deleted++;
	}
	else
		PATH_TRACE;

	if (test_deleted)
		DBUG_VOID_RETURN;

	switch(buffer[0]) {
	case TEST_KILL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling abort() failed to send SIGHUP and SIGCONT signals");
		in_rpt("to process in orphaned process group which is stopped");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_KILL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling abort() failed to send SIGHUP and SIGCONT signals");
		in_rpt("to process in orphaned process group which is not stopped");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	PATH_FUNC_RPT(8);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test9");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test that abort() sends SIGHUP and SIGCONT if process is stopped
	 * when a process group is orphaned.
	 */

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	if(pipe(fg_fd) < 0 ||
	   pipe(bg_fd) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if(cpsig(ch_t9, pa_t9, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if(cpsig(ch_t9, pa_t9, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	sig_arg = SIG_IGN;
	if(cpsig(ch_t9, pa_t9, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if ((syn = opensync()) == (synch *)NULL) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - opensync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	sig_arg = sig_catch_child;
	sig_recvd = 0;
	if(cpsig(ch_t9, pa_t9, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if (closesync(syn) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - closesync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	(void) close(fg_fd[READ]);
	(void) close(fg_fd[WRITE]);
	(void) close(bg_fd[READ]);
	(void) close(bg_fd[WRITE]);

	if(!test_fail)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	DBUG_ENTER("ch_t9");

	/*
	 * Create process group with one stopped process and one
	 * sleeping process.  Then orphan it by exiting this process.
	 * The grandchildren report to the grandparent on what happens.
	 */

	switch(tet_child = fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		errno = 0;
		if(setpgid((pid_t)0, (pid_t)0) < 0) {
			if (errno != ENOSYS) {
				vsxerror("setpgid(0, 0) failed - errno %d (%s)",
					errno, errname(errno));
				exit(A_MISC); /* not DBUG_EXIT */
			}
			(void) setsid();
		}

		switch(fork())
		{
		case SYSERROR:
			vsxerror("external error %d", A_FORKFAIL);
			exit(A_FORKFAIL); /* not DBUG_EXIT */

			/* NOTREACHED */
			break;

		case 0:		/* child */
			tet_setcontext();
			ch_stop();
			break;

		default:
			tet_setblock();
			/* parent does nothing yet */
			break;
		}
		tet_setblock();
		ch_sleep();
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		break;
	}
	tet_setblock();

	pgch_t9();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch_t9()
{
        sigset_t set;
        int     sg_num  = sig_array[VSIGABRT];

	DBUG_ENTER("pgch_t9");

	/* Exit after a short pause long enough to initialise children */
	(void) sleep(WAITTIME/2);

	/* This is the call that is actually tested */
	(void)	vsignal(SIGABRT, sig_arg);
	if(sig_arg == SIG_DFL && sig_block) {

		/* Call sigprocmask() to block the signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigprocmask() failed");
			DBUG_EXIT(A_TSETDIR);
		}
	}
	TEST_ABORT();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t9()
{
	int	stat_child;
	int	ret;
	int	err;
	pid_t	pid;
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("pa_t9");

	/* Wait for child and expect it to have aborted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
	}
	else {
		PATH_TRACE;
		if(stat_child != CHLD_ABRT || sg_code != SIGABRT) {
			xx_rpt(DELETION);
			in_rpt("child process terminated incorrectly");
			in_rpt("expected action: child aborted - abort signal %d", SIGABRT);
			in_rpt("observed action: %s", status_child(stat_child));
			test_deleted++;
		}
		else
			PATH_TRACE;
	}

	/* if we set a catcher then if executed assume it also returned */
	if(sig_arg == sig_catch_child && (
	    (ret=waitsync(syn[0][READ],(char *)&sig_recvd,sizeof(sig_recvd),WAITTIME) != sizeof(sig_recvd))
	    || sig_recvd != SIGABRT)
	  ) {
		err = errno;
		xx_rpt(DELETION);
		if (ret == sizeof(sig_recvd))
			in_rpt("deletion reason - unexpected signal %d caught instead of SIGABRT (%d)", sig_recvd, SIGABRT);
		else if (ret == SYSERROR && err == EINTR) /* timed out */
			in_rpt("deletion reason - SIGABRT catcher not called");
		else {
			in_rpt("deletion reason - waitsync error");
			valrpt(sizeof(sig_recvd), ret, EINTR, err);
		}
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* Read from pipe to determine PID of stopped grandchild */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], (char *)&pid, sizeof(pid_t));
	CLEAR_ALARM
	if(ret != sizeof(pid_t)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(pid_t));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Read from pipes to determine success of grandchildren */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		(void) killchild(pid, WAITTIME);
		test_deleted++;
	}
	else
		PATH_TRACE;

	if (test_deleted)
		DBUG_VOID_RETURN;

	switch(buffer[0]) {
	case TEST_KILL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling abort() failed to send SIGHUP and SIGCONT signals");
		in_rpt("to process in orphaned process group which is stopped");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_KILL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling abort() failed to send SIGHUP and SIGCONT signals");
		in_rpt("to process in orphaned process group which is not stopped");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	PATH_FUNC_RPT(8);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	int	pathok = setupok;

	DBUG_ENTER("test10");

	/*
	 * Test that abort() is reported to waiting parent process.
	 */

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if(cpsig(ch_t10, pa_t10, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if(cpsig(ch_t10, pa_t10, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	sig_arg = SIG_IGN;
	if(cpsig(ch_t10, pa_t10, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if ((syn = opensync()) == (synch *)NULL) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - opensync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	sig_recvd = 0;
	sig_arg = sig_catch_child;
	if(cpsig(ch_t10, pa_t10, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if (closesync(syn) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - closesync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	if(!test_fail)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
        sigset_t set;
        int     sg_num  = sig_array[VSIGABRT];

	DBUG_ENTER("ch_t10");


	(void) vsignal(SIGABRT, sig_arg);
	if(sig_arg == SIG_DFL && sig_block) {

		/* Call sigprocmask() to block the signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigprocmask() failed");
			DBUG_EXIT(A_TSETDIR);
		}
	}
	TEST_ABORT();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t10()
{
	int	stat_child;
	int	err;
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("pa_t10");

	/* Sleep a while to give child time to abort() */
	(void) sleep(WAITTIME/2);
	/* Wait for child and expect it to have aborted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_TSETDIR) {
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* if we set a catcher then if executed assume it also returned */
	if(sig_arg == sig_catch_child && (
	    (ret=waitsync(syn[0][READ],(char *)&sig_recvd,sizeof(sig_recvd),WAITTIME) != sizeof(sig_recvd))
	    || sig_recvd != SIGABRT)
	  ) {
		err = errno;
		xx_rpt(DELETION);
		if (ret == sizeof(sig_recvd))
			in_rpt("deletion reason - unexpected signal %d caught instead of SIGABRT (%d)", sig_recvd, SIGABRT);
		else if (ret == SYSERROR && err == EINTR) /* timed out */
			in_rpt("deletion reason - SIGABRT catcher not called");
		else {
			in_rpt("deletion reason - waitsync error");
			valrpt(sizeof(sig_recvd), ret, EINTR, err);
		}
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("parent not notified of child calling abort()");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != SIGABRT) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("status did not show process terminated by SIGABRT");
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

private void
ch_stop()
{
	char	buffer[1];
	pid_t	pid = getpid();

	DBUG_ENTER("ch_stop");

	/* Write down pipe to grandparent process ID */
	(void) write(fg_fd[WRITE], (char *)&pid, sizeof(pid_t));

	if(DBUG_SETJMP(sjmp) != 0) {
		buffer[0] = TEST_SIGNAL;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	setsigs(sjmpfunc);

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}

	/* Initialise the SIGCONT action */
	if(sig_init(sig_array[VSIGCONT], &sig_empty, sig_resume, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}

	sig_recvd = 0;

	/* Stop ourself */
	if(kill(pid, SIGSTOP) < 0) {
		buffer[0] = TEST_KILL;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}

	/* Allow time for second signal to arrive, if necessary */
	if(sig_recvd != SIGHUP || sig_cont != SIGCONT)
		(void) sleep(SPEEDFACTOR);

	/* Expect signal SIGHUP and SIGCONT to have been received */
	if(sig_recvd != SIGHUP || sig_cont != SIGCONT) {
		buffer[0] = TEST_FAIL;
	} else {
		buffer[0] = TEST_SUCCESS;
	}

	/* Write down pipe to grandparent success code */
	(void) write(fg_fd[WRITE], buffer, sizeof(char));

	DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
ch_sleep()
{
	char	buffer[1];

	DBUG_ENTER("ch_sleep");

	if(DBUG_SETJMP(sjmp) != 0) {
		buffer[0] = TEST_SIGNAL;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	setsigs(sjmpfunc);

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}

	/* Initialise the SIGCONT action */
	if(sig_init(sig_array[VSIGCONT], &sig_empty, sig_resume, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}

	sig_recvd = 0;

	/* Sleep, during which time the parent calls abort() */
	(void) sleep(WAITTIME);

	/* Allow time for second signal to arrive, if necessary */
	if(sig_recvd != SIGHUP || sig_cont != SIGCONT)
		(void) sleep(SPEEDFACTOR);

	/* Expect signal SIGHUP and SIGCONT to have been received */
	if(sig_recvd != SIGHUP || sig_cont != SIGCONT) {
		buffer[0] = TEST_FAIL;
	} else {
		buffer[0] = TEST_SUCCESS;
	}

	/* Write down pipe to grandparent success code */
	(void) write(bg_fd[WRITE], buffer, sizeof(char));

	DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
sjmpfunc(sig)
int sig;
{
	DBUG_ENTER("sjmpfunc");
	DBUG_LONGJMP(sjmp, sig+1);
	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test11()
{
	DBUG_ENTER("test11");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test11()
{
	/* When session leader exits, terminal can be reacquired as ctty */
	char	*termios_tty;
	int	pathok = setupok;

	DBUG_ENTER("test11");

	test_fail = 0;
	test_deleted = 0;

	termios_tty = tet_getvar("VSX_TERMIOS_TTY");
	if (termios_tty == NULL || *termios_tty == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_TTY not defined");
		DBUG_VOID_RETURN;
	}
        else if (strncmp(termios_tty, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* Do pair of tests with SIGABRT handling == SIG_DFL, not blocked */
	reacquiring = 0;
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if (cpsig(ch_t11, pa_t11, WAITTIME, 0) == SYSERROR) 
		test_deleted++;
	else
		PATH_FUNC_TRACE;
		
	if (test_deleted)
		DBUG_VOID_RETURN;

	/* Second call "re-acquires" ctty */
	reacquiring = 1;
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if (cpsig(ch_t11, pa_t11, WAITTIME, 0) == SYSERROR) 
		test_deleted++;
	else
		PATH_FUNC_TRACE;
		
	if (test_deleted)
		DBUG_VOID_RETURN;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

/* Do pair of tests with SIGABRT handling == SIG_DFL, blocked */
	reacquiring = 0;
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if (cpsig(ch_t11, pa_t11, WAITTIME, 0) == SYSERROR) 
		test_deleted++;
	else
		PATH_FUNC_TRACE;
		
	if (test_deleted)
		DBUG_VOID_RETURN;

	/* Second call "re-acquires" ctty */
	reacquiring = 1;
	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if (cpsig(ch_t11, pa_t11, WAITTIME, 0) == SYSERROR) 
		test_deleted++;
	else
		PATH_FUNC_TRACE;
		
	if (test_deleted)
		DBUG_VOID_RETURN;

/* Do pair of tests with SIGABRT handling == SIG_IGN */
	reacquiring = 0;
	globok = 0;
	sig_arg = SIG_IGN;
	if (cpsig(ch_t11, pa_t11, WAITTIME, 0) == SYSERROR) 
		test_deleted++;
	else
		PATH_FUNC_TRACE;
		
	if (test_deleted)
		DBUG_VOID_RETURN;

	/* Second call "re-acquires" ctty */
	reacquiring = 1;
	globok = 0;
	sig_arg = SIG_IGN;
	if (cpsig(ch_t11, pa_t11, WAITTIME, 0) == SYSERROR) 
		test_deleted++;
	else
		PATH_FUNC_TRACE;
		
	if (test_deleted)
		DBUG_VOID_RETURN;

/* Do pair of tests with SIGABRT handling == sig_catch_child */
	if ((syn = opensync()) == (synch *)NULL) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - opensync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	reacquiring = 0;
	globok = 0;
	sig_recvd = 0;
	sig_arg = sig_catch_child;
	if (cpsig(ch_t11, pa_t11, WAITTIME, 0) == SYSERROR) 
		test_deleted++;
	else
		PATH_FUNC_TRACE;
		
	if (test_deleted)
		DBUG_VOID_RETURN;

	/* Second call "re-acquires" ctty */
	reacquiring = 1;
	globok = 0;
	sig_recvd = 0;
	sig_arg = sig_catch_child;
	if (cpsig(ch_t11, pa_t11, WAITTIME, 0) == SYSERROR) 
		test_deleted++;
	else
		PATH_FUNC_TRACE;
		
	if (test_deleted)
		DBUG_VOID_RETURN;

	if (closesync(syn) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - closesync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (test_deleted)
		DBUG_VOID_RETURN;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	if (!test_fail)
		PATH_XS_RPT(11); 

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	/* signal failure/unsupported/deleted with exit codes. abort => pass */
	int	pathok = 0;
        sigset_t set;
        int     sg_num  = sig_array[VSIGABRT];

	DBUG_ENTER("ch_t11");

	globok = 0;
	if (ctty() != 0)
	{
		/* unsupported or deleted */
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_FUNC_TRACE;

	interrupt(YES); /* only place failure can happen */

	if (test_fail)
		DBUG_EXIT(NOK);

	if (test_deleted)
		DBUG_EXIT(A_TSETDIR);

	(void) vsignal(SIGABRT, sig_arg);
	if(sig_arg == SIG_DFL && sig_block) {

		/* Call sigprocmask() to block the signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigprocmask() failed");
			DBUG_EXIT(A_TSETDIR);
		}
	}

	/* only get to do the abort if we haven't failed */
	TEST_ABORT();

	/*NOTREACHED*/
	DBUG_VOID_RETURN;
}

private void
pa_t11()
{
	int	stat_child;
	int	err;
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("pa_t11");

	/* Wait for child and expect it to have aborted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("child process still active %d seconds after calling abort()", WAITTIME);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* If we failed/deleted/unsupported in child then signalled by
	   not doing the abort() but exiting with A_TSETDIR (delete/unsupp.)
	   or NOK (failed).
	 */
	if(stat_child == CHLD_EXIT) {
		if (ex_code == NOK)
			test_fail++;
		else
			test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* if we set a catcher then if executed assume it also returned */
	if(sig_arg == sig_catch_child && (
	    (ret=waitsync(syn[0][READ],(char *)&sig_recvd,sizeof(sig_recvd),WAITTIME) != sizeof(sig_recvd))
	    || sig_recvd != SIGABRT)
	  ) {
		err = errno;
		xx_rpt(DELETION);
		if (ret == sizeof(sig_recvd))
			in_rpt("deletion reason - unexpected signal %d caught instead of SIGABRT (%d)", sig_recvd, SIGABRT);
		else if (ret == SYSERROR && err == EINTR) /* timed out */
			in_rpt("deletion reason - SIGABRT catcher not called");
		else {
			in_rpt("deletion reason - waitsync error");
			valrpt(sizeof(sig_recvd), ret, EINTR, err);
		}
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != SIGABRT) {
		xx_rpt(DELETION);
		in_rpt("status did not show process terminated by SIGABRT");
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test12()
{
	DBUG_ENTER("test12");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test12()
{
	int	pathok = setupok;

	DBUG_ENTER("test12");

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Test that buffer on open files and streams is written upon
	 * execution of abort().
	 */

	if (tet_sync(SP_T12_WRITE1_START, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE1_START failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 0;
	if(cpsig(ch_t12, pa_t12, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if (tet_sync(SP_T12_WRITE1_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE1_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

	if (tet_sync(SP_T12_WRITE2_START, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE2_START failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	sig_arg = SIG_DFL;
	sig_block = 1;
	if(cpsig(ch_t12, pa_t12, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if (tet_sync(SP_T12_WRITE2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T12_WRITE3_START, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE3_START failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	sig_arg = SIG_IGN;
	if(cpsig(ch_t12, pa_t12, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if (tet_sync(SP_T12_WRITE3_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE3_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T12_WRITE4_START, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE4_START failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((syn = opensync()) == (synch *)NULL) {
		xx_rpt(DELETION);
		in_rpt("opensync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	sig_recvd = 0;
	sig_arg = sig_catch_child;
	if(cpsig(ch_t12, pa_t12, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if (closesync(syn) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("closesync() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (tet_sync(SP_T12_WRITE4_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE4_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	if(!test_fail)
		PATH_XS_RPT(14);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
        sigset_t set;
        int     sg_num  = sig_array[VSIGABRT];
	int	fd1;
	FILE	*fd2;
	int	err;
	int	ret;

	DBUG_ENTER("ch_t12");

	(void) vsignal(sg_num, sig_arg);
	if(sig_arg == SIG_DFL && sig_block) {

		/* Call sigprocmask() to block the signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("sigprocmask() failed");
			DBUG_EXIT(A_TSETDIR);
		}
	}

	errno = NOERROR;
	fd1 = creat(T12_1_FILE, CREATMODE);
	if(fd1 == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open() of file '%s' failed. errno = %d",T12_1_FILE, err);
		DBUG_EXIT(A_TSETDIR);
	}

	errno = NOERROR;
	fd2 = fopen(T12_2_FILE, "w");
	if(fd2 == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen() of file '%s' failed. errno = %d",T12_2_FILE, err);
		DBUG_EXIT(A_TSETDIR);
	}


	errno = NOERROR;

        ret = write(fd1, ONE_BYTE_STR, ONE_BYTE_LENGTH);

	if(ret != ONE_BYTE_LENGTH)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() failed errno %d", err);
		DBUG_EXIT(A_TSETDIR);
	}

	errno = NOERROR;
	ret = fwrite((void *)ONE_BYTE_STR, (size_t)1, (size_t)ONE_BYTE_LENGTH, fd2);

	if(ret != ONE_BYTE_LENGTH)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fwrite() failed errno %d", err);
		DBUG_EXIT(A_TSETDIR);
	}

	TEST_ABORT();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t12()
{
	int	stat_child;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t12");

	/* Wait for child and expect it to have aborted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_TSETDIR) {
			xx_rpt(DELETION);
			test_deleted++;
			DBUG_VOID_RETURN;
		} else {
			xx_rpt(DELETION);
			in_rpt("child process continued after calling abort()");
			test_deleted++;
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

	/* if we set a catcher then if executed assume it also returned */
	if(sig_arg == sig_catch_child && (
	    (ret=waitsync(syn[0][READ],(char *)&sig_recvd,sizeof(sig_recvd),WAITTIME) != sizeof(sig_recvd))
	    || sig_recvd != SIGABRT)
	  ) {
		err = errno;
		xx_rpt(DELETION);
		if (ret == sizeof(sig_recvd))
			in_rpt("unexpected signal %d caught instead of SIGABRT (%d)", sig_recvd, SIGABRT);
		else if (ret == SYSERROR && err == EINTR) /* timed out */
			in_rpt("SIGABRT catcher not called");
		else {
			in_rpt("waitsync error");
			valrpt(sizeof(sig_recvd), ret, EINTR, err);
		}
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("child process still active %d seconds after calling abort()", WAITTIME);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	
	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
interrupt(toggle)
int toggle;                     /* YES == expecting SIGINT      */
{                               /* NO == not expecting SIGINT   */
			/* send SIGINT to self and children */

	int err, ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("interrupt");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        /* for child setup, if any - allow for this in cppair() timeouts */
        (void) sleep(NAP);

        sig_recvd= 0;

	DBUG_PRINTF("interrupt", "c_cc[VINTR] = 0%o",
		(int)tty_termios_p->c_cc[VINTR]);

        chr = tty_termios_p->c_cc[VINTR];

        SET_TIMEOUT(WAITTIME/2)

        ret = write(loop_fildes, (char *)&chr, 1);
	err = errno;
	if (ret == 1)
		(void) pause();

        CLEAR_ALARM

	if (ret != 1)
	{
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (toggle == YES && sig_recvd != SIGINT)
	{
		if (reacquiring) {
			if (test_fail++ == 0)
				xx_rpt(FAILURE);
		} else {
			xx_rpt(DELETION);
			test_deleted++;
		}
		in_rpt("process did not receive SIGINT when expected");
	}
	else
		PATH_TRACE;

	if (toggle == NO && sig_recvd == SIGINT)
	{
		if (reacquiring) {
			if (test_fail++ == 0)
				xx_rpt(FAILURE);
		} else {
			xx_rpt(DELETION);
			test_deleted++;
		}
		in_rpt("process received SIGINT when not expected");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4); 

	DBUG_VOID_RETURN;
}

private int
ctty()
{
	/* acquire controlling terminal */
	int	pathok = 0;

	DBUG_ENTER("ctty");

	switch (termios_prep(&tty_fildes, tty_termios_p,
			&loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE; 
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_RETURN(-1);
	default:
		xx_rpt(DELETION);
		test_deleted++;

		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY/LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY/LOOP");

		DBUG_RETURN(-1);
	}
	PATH_FUNC_RPT(1); 

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
