/*
*      SCCS:  @(#)  ANSI.os/genuts/atof/atof.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/genuts/atof - T.atof Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.atof

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the atof() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Fri Nov  7 16:52:37 WET 1986
MODIFICATIONS:
		David Sawyer
		Thu Jul 05 17:42:22 BST 1990
		Added resillience code

		Geoff Clare, 11 July 1991
			Use dcomp() instead of !=

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<errno.h>
#include	<string.h>
#include	<nlstest.h>
#include	<dbug.h>
#include	<report.h>
#include	<math.h>
#include	<locale.h>
#include	<nlsinfo.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef atof
#endif


#define 	NO_TESTS	3		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)
#define		NMRS1		3	/* size of array num1[]	*/

extern	char	*errname();
extern	char	*psl_to_asc();
extern	char	*pslanginfo();

int	no_tests = NO_TESTS;

struct	number 	{
	char	*nmstr;		/* number string */
	double	nmval;		/* value of number string */
};

private struct	number	num1[NMRS1] = 	{
						"-23.45e3",	-23.45e3,
						"12.45",	12.45,
						" +0.67E-6",	0.67E-6
					};

struct	invalnm	{
	char	*nmstr;		/* number string */
	int	nmind;		/* index of first inconsistent char */
};

private void	test1();
private void	test2();
private void	test3();
private void	ch_t2();
private void	ch_t3();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private void
test1()
{
	int	nfails = 0;
	double	rtval;
	struct	number	*nmptr;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test1");

	for (i = 0; i < NMRS1; i++)
	{
		nmptr = &num1[i];
		if ( !dcomp(rtval = atof(nmptr->nmstr), nmptr->nmval) )
		{
			if ( nfails++ == 0 )	/* first fail */
				xx_rpt(FAILURE);
			in_rpt("atof returned the value of %s as %f",
					nmptr->nmstr, rtval);
		}
		else
			PATH_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(NMRS1);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int 	nfails = 0;
	char 	convstr[20];
	double 	rtval;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	(void) strcpy(convstr, "3.14");

	if (!dcomp(rtval = atof(convstr), 3.14))
	{
		if (nfails++ == 0)
			xx_rpt(FAILURE);
		in_rpt("Current language is UNDEFINED");
		in_rpt("atof returned the value of %s as %f",
				convstr, rtval);
	}
	else
		PATH_TRACE;

	if ( setlocale(LC_NUMERIC, NORADIXLANG) == NULL )
	{
		if (nfails++ == 0)
			xx_rpt(DELETION);
		in_rpt("Pseudo langauge %s not available", NORADIXLANG);
		in_rpt("Check installation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!dcomp(rtval = atof(convstr), 3.14))
	{
		if (nfails++ == 0)
			xx_rpt(FAILURE);
		in_rpt("Current language is %s", NORADIXLANG);
		in_rpt("atof returned the value of %s as %f",
				convstr, rtval);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int 	nfails = 0;
	char 	*radchar;
	char 	convstr[20];
	double 	rtval;
	int	pathok = 0;


	DBUG_ENTER("ch_t3");

	(void) strcpy(convstr, "3.14");

	if ( setlocale(LC_NUMERIC, RAD1LANG) == NULL )
	{
		if (nfails++ == 0)
			xx_rpt(DELETION);
		in_rpt("Pseudo langauge %s not available", RAD1LANG);
		in_rpt("Check installation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	radchar = pslanginfo(VSX_RADIXCHAR, VALRAD1LANG);

	convstr[1] = radchar[0];

	if (!dcomp(rtval = atof(convstr), 3.14))
	{
		if (nfails++ == 0)
			xx_rpt(FAILURE);
		in_rpt("Current language is %s", RAD1LANG);
		in_rpt("atof returned the value of %s as %f",
				psl_to_asc(convstr,VALRAD1LANG), rtval);
		in_rpt("the radix character was set to %x", radchar[0]);
	}
	else
		PATH_TRACE;

	if ( setlocale(LC_NUMERIC, RAD2LANG) == NULL )
	{
		if (nfails++ == 0)
			xx_rpt(DELETION);
		in_rpt("Pseudo langauge %s not available", RAD2LANG);
		in_rpt("Check installation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	radchar = pslanginfo(VSX_RADIXCHAR, VALRAD2LANG);

	convstr[1] = radchar[0];

	if (!dcomp(rtval = atof(convstr), 3.14))
	{
		if (nfails++ == 0)
			xx_rpt(FAILURE);
		in_rpt("Current language is %s", RAD2LANG);
		in_rpt("atof returned the value of %s as %f",
				psl_to_asc(convstr,VALRAD2LANG), rtval);
		in_rpt("the radix character was set to %x", radchar[0]);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}
