/*
*      SCCS:  @(#)  ANSI.os/genuts/bsearch/bsearch.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/genuts/bsearch - T.bsearch Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.bsearch

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the bsearch() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		David McCardle, UniSoft Ltd.
DATE CREATED:	Thu Oct 23 15:51:26 PDT 1986
MODIFICATIONS:	Andrew Twigger 09 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		David Sawyer
		Fri Jul 06 10:11:28 BST 1990
		Added resillience code

		Stuart Boutell, 5 Nov 1990
			changed dltd for tet_reason

		Geoff Clare, 27 Jan 1992
			Change node_cmp() args to be compatible with qsort()

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<string.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef bsearch
#endif


#define NO_TESTS	2		/* number of tests in testset */
#define TABSIZE		10		/* number of entries in table */

extern	char	*errname();

int	no_tests = NO_TESTS;
struct	node {
	char *string;
	int length;
	};
struct	node table[TABSIZE];
char	names[10][7] =	{ "arthur",
			  "brian",
			  "colin",
			  "dent",
			  "edward",
			  "ford",
			  "graeme",
			  "henry",
			  "ian",
			  "john"
			};

private int	node_cmp();
private void	preparetests();
private void	test1();
private void	test2();

public	void	(*setfunc) () = preparetests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private void
preparetests()
{
	int	i;

	DBUG_ENTER("preparetests");

	for (i = 0; i < 10; i++)
	{
		if ((table[i].string = (char *) malloc(strlen(names[i]) +
					1 + sizeof(int))) == NULL)
		{
			if (tet_reason(1)==NULL)
				(void) tet_delete(1, "failed to malloc() space for table entry");
			if (tet_reason(2)==NULL)
				(void) tet_delete(2, "failed to malloc() space for table entry");
			break;
		}
		if (strcpy(table[i].string, names[i]) != table[i].string)
		{
			if (tet_reason(1)==NULL)
				(void) tet_delete(1, "failed to copy data into table");
			if (tet_reason(2)==NULL)
				(void) tet_delete(2, "failed to copy data into table");
			break;
		}
		table[i].length = strlen(names[i]);
		DBUG_PRINTF("preparetests", "table[%d] :%ld:", i, &table[i]);
		DBUG_PRINTF("preparetests", "table.string[%d] :%s:",
			i, table[i].string);
		DBUG_PRINTF("preparetests", "table.length[%d] :%d:",
			i, table[i].length);
	}

	DBUG_VOID_RETURN;
}


private void
test1()
{
	int	i, err[TABSIZE];
	int	failed1 = 0;
	struct node *rval[TABSIZE], tnode;
	int	pathok = 0;

	DBUG_ENTER("test1");

	for (i = 0; i < TABSIZE; i++)
	{
		tnode.string = names[i];
		tnode.length = strlen(names[i]);
		err[i] = errno = 0;
		if ((rval[i] = (struct node *) bsearch((void *) &tnode,
			(void *) table, (size_t)TABSIZE,
			(size_t)sizeof(struct node), node_cmp)) != &table[i])
		{
			err[i] = errno;
			failed1 |= 1 << i;
			DBUG_PRINTF("test1", "failed :%d:\tfailed1 :%ld:",
				 i, failed1);
		}
		else	
			PATH_TRACE;
	}
	DBUG_PRINTF("test1", "failed1 :%ld:", failed1);

	if (failed1)
	{
		xx_rpt(FAILURE);

		for (i = 0; i < TABSIZE; i++)
		{
			DBUG_PRINTF("test1", "rval[%d] :%ld:", i, rval[i]);
			DBUG_PRINTF("test1", "err[%d] :%d:", i, err[i]);
			if (failed1 & (1 << i))
			{
				in_rpt("bsearch did not return expected value");
				if (rval[i] != &table[i])
				{
					in_rpt("searching for              : %s", names[i]);
					in_rpt("expected return value      : %ld", &table[i]);
					in_rpt("actual   return value      : %ld", rval[i]);
				}
				if (err[i] != NOERROR)
				{
					in_rpt("expected errno  value      : %d (%s)",
						NOERROR, errname(NOERROR));
					in_rpt("actual   errno  value      : %d (%s)",
						err[i], errname(err[i]));
				}
			}
		}

	}
	else
		PATH_XS_RPT(TABSIZE);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	int	err = 0;
	char	marvin[7];
	struct node *rval, tnode;
	int	pathok = 0;

	DBUG_ENTER("test2");

	if (strcpy(marvin, "marvin") != marvin)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s",
			"unable to make non-existent key");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	tnode.string = marvin;
	tnode.length = strlen(marvin);
	errno = 0;
	if ((rval = (struct node *) bsearch((void *) &tnode,
		(void *) table, (size_t)TABSIZE, (size_t)sizeof(struct node),
		node_cmp)) != (struct node *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("bsearch did not return expected value");
		if (rval != (struct node *) NULL)
		{
			in_rpt("searching for              : %s", marvin);
			in_rpt("expected return value      : %ld", NULL);
			in_rpt("actual   return value      : %ld", rval);
		}
		if (err != NOERROR)
		{
			in_rpt("expected errno  value      : %d (%s)",
				NOERROR, errname(NOERROR));
			in_rpt("actual   errno  value      : %d (%s)",
				err, errname(err));
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}


private int
node_cmp(a1, a2)
#if __STDC__
	const void	*a1;
	const void	*a2;
#else
	char	*a1;
	char	*a2;
#endif
{
	char	*node1 = (char *)a1;
	char	*node2 = (char *)a2;

	DBUG_ENTER("node_cmp");
	DBUG_PRINTF("node_cmp", "node1->string :%s:\tnode2->string :%s:",
		((struct node *) node1)->string,
		((struct node *) node2)->string);
	DBUG_RETURN(strcmp(((struct node *) node1)->string,
				 ((struct node *) node2)->string));
}
