/*
*      SCCS:  @(#)  ANSI.os/genuts/calloc/calloc.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/genuts/calloc - T.calloc Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.calloc

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the calloc() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Tue Dec 1 17:08:25 BST 1986
MODIFICATIONS:	Andrew Twigger 12 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 11 July 1989
		Rewrite tests which misuse pointers

		Geoff Clare, 7 Dec 1989
		Do not assign objects larger than size calloc'ed

		Geoff Clare, 21 Feb 1990
		Append NULL to funcs initialiser list

		David Sawyer
		Fri Jul 06 11:25:14 BST 1990
		Added resillience code

		Stuart Boutell, 27 Nov 1990
			Added globok declaration.
			Removed PATH_XS_RPT and used xs_rpt/xx_rpt

		Stuart Boutell, 14 Jan 1991
			Test 3 added
		
		Geoff Clare, 16 July 1991
			Test long double alignment if supported

		Geoff Clare, 11 July 1994
			Use whole numbers in floating point variables, to
			avoid rounding problems.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<report.h>
#include	<unistd.h>
#include	<string.h>
#include	<setjmp.h>
#include	<signal.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef calloc
#endif


#define NO_TESTS	3		/* number of tests in testset */
#define WAITTIME	(2 * SPEEDFACTOR)

int	no_tests = NO_TESTS;

extern	void	xs_rpt();
extern	void	xx_rpt();
#if TEST_ANSI
extern	void	in_rpt(char *, ...);
#else
extern	void	in_rpt();
#endif
extern	char	*signame();
extern	void	(*vsignal())();
extern	int	globok;

private void	test1();
private void	test2();
private void	test3();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private int	test_align();
private	void	sig_catch();

private	sigjmp_buf	jmpbuf;
private	int	sig_found;
private	int	t3_values[][2] = {
	16, 0,
	0, 16,
	0, 0
} ;
public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private void
test1()
{
	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);
}

private void
ch_t1()
{
	int	i;
	char 	*memcal;
	int	pathok = 0;

	if ((memcal = (char *)calloc((size_t)7, (size_t)23)) == (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("calloc failed to allocate a block of 7 items with 23 bytes each");
		return;
	}
	else
		++pathok;

	for (i=0; i <(7*23); i++)
	{
		if (memcal[i] != '\0')
		{
			xx_rpt(FAILURE);
			in_rpt("byte %d of calloced array not set to NULL", i);
			return;
		}
		else
			++pathok;
	}

	if(pathok==(1+7*23))
	{
		xs_rpt();
	}
	else
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: path trace value %d",pathok);
	}
}


private void
test2()
{
	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);
}

private void
ch_t2()
{
	char *mempos[20];
	int	i,j;
	int	fail = 0;
	int	pathok = 0;

	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		if ((mempos[i] = (char *)calloc((size_t)13, (size_t) j)) == (char *) NULL)
		{
			xx_rpt(FAILURE);
			in_rpt("failed to calloc() 13 items of %d bytes of memory", j);
			return;
		}
		else
			++pathok;
	}

	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		globok = 0;
		fail += test_align(j, mempos[i]);
		if (globok==1)
			++pathok;
		globok = 0;
	}
	
	if (fail == 0)
	{
		if(pathok == (20+20))
		{
			xs_rpt();
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: path trace value %d",pathok);
		}
	}
}

private void
test3()
{
	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);
}

private void
ch_t3()
{
	int	nelem;
	int	elsize;
	int	counter;
	void	(*oldsig)();
	char	*ptr;
	int	pathok = 0;

	counter = 0;

	do {
		nelem  = t3_values[counter][0];
		elsize = t3_values[counter][1];

		ptr = (char *)calloc( (size_t)nelem, (size_t)elsize );

		if (ptr == NULL)
		{
			++pathok;
		}
		else
		{
			oldsig = vsignal(SIGSEGV, sig_catch);
			sig_found = 0;
			if (sigsetjmp(jmpbuf, 1) != 0)
			{
				xx_rpt(FAILURE);
				in_rpt("Pointer returned from calloc((size_t)%d,(size_t)%d)",
						nelem, elsize );
				in_rpt("could not be passed successfully to free()");
				in_rpt("free(ptr) generated signal %d (%s)",
					sig_found, signame(sig_found));
				return;
			}
			else
				++pathok;

			free( (void *)ptr );
			(void) vsignal(SIGSEGV, oldsig);
		}

		counter++;
	}
	while( (nelem != 0) && (elsize != 0) );

	if (pathok == counter)
	{
		xs_rpt();
	}
	else
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: path check value %d", pathok);
	}
}

private int
test_align(nbyte, ptr)
int nbyte;
char *ptr;
{
	char	xchar = 'z';
	short	xshort = 12345;
	int	xint = 12345;
	long	xlong = 1234567;
	float	xfloat = 123.0;
	double	xdouble = 1234.0;
#if LONG_DOUBLE_SUPPORTED
	long double	xldouble = 12345.0;
#endif
	struct x { int a; char b; double c; } xstruct;
	struct x *xstrptr;
	char	buf[256];
	int	pathok = 0;

	*buf = '\0';

	globok = 0;

	/* If there are alignment problems, these tests will no doubt
	   cause signals rather than the nice error message at the end */

	if (nbyte * 13 >= sizeof(short))
	{
		*(short *)ptr = xshort;
		if (++*(short *)ptr != xshort+1)
			(void) strcat(buf, "short ");
	}

	if (nbyte * 13 >= sizeof(int))
	{
		*(int *)ptr = xint;
		if (++*(int *)ptr != xint+1)
			(void) strcat(buf, "int ");
	}

	if (nbyte * 13 >= sizeof(long))
	{
		*(long *)ptr = xlong;
		if (++*(long *)ptr != xlong+1)
			(void) strcat(buf, "long ");
	}

	if (nbyte * 13 >= sizeof(float))
	{
		*(float *)ptr = xfloat;
		if (++*(float *)ptr != xfloat+1)
			(void) strcat(buf, "float ");
	}

	if (nbyte * 13 >= sizeof(double))
	{
		*(double *)ptr = xdouble;
		if (++*(double *)ptr != xdouble+1)
			(void) strcat(buf, "double ");
	}

#if LONG_DOUBLE_SUPPORTED
	if (nbyte * 13 >= sizeof(long double))
	{
		*(long double *)ptr = xldouble;
		if (++*(long double *)ptr != xldouble+1)
			(void) strcat(buf, "long double ");
	}
#endif

	if (nbyte * 13 >= sizeof(char *))
	{
		*(char **)ptr = &xchar;
		if (++*(char **)ptr != &xchar+1)
			(void) strcat(buf, "(char *) ");
	}

	if (nbyte * 13 >= sizeof(short *))
	{
		*(short **)ptr = &xshort;
		if (++*(short **)ptr != &xshort+1)
			(void) strcat(buf, "(short *) ");
	}

	if (nbyte * 13 >= sizeof(int *))
	{
		*(int **)ptr = &xint;
		if (++*(int **)ptr != &xint+1)
			(void) strcat(buf, "(int *) ");
	}

	if (nbyte * 13 >= sizeof(long *))
	{
		*(long **)ptr = &xlong;
		if (++*(long **)ptr != &xlong+1)
			(void) strcat(buf, "(long *) ");
	}

	if (nbyte * 13 >= sizeof(float *))
	{
		*(float **)ptr = &xfloat;
		if (++*(float **)ptr != &xfloat+1)
			(void) strcat(buf, "(float *) ");
	}

	if (nbyte * 13 >= sizeof(double *))
	{
		*(double **)ptr = &xdouble;
		if (++*(double **)ptr != &xdouble+1)
			(void) strcat(buf, "(double *) ");
	}

#if LONG_DOUBLE_SUPPORTED
	if (nbyte * 13 >= sizeof(long double *))
	{
		*(long double **)ptr = &xldouble;
		if (++*(long double **)ptr != &xldouble+1)
			(void) strcat(buf, "(long double *) ");
	}
#endif

	if (nbyte * 13 >= sizeof(struct x *))
	{
		*(struct x **)ptr = &xstruct;
		if (++*(struct x **)ptr != &xstruct+1)
			(void) strcat(buf, "(struct x *) ");
	}

	if (nbyte * 13 >= sizeof(struct x **))
	{
		*(struct x ***)ptr = &xstrptr;
		if (++*(struct x ***)ptr != &xstrptr+1)
			(void) strcat(buf, "(struct x **) ");
	}

	if (*buf != '\0')
	{
		xx_rpt(FAILURE);
		in_rpt("pointer returned by calloc() of 13 items of %d bytes was not aligned correctly for the following types:",
			nbyte);
		in_rpt("%s", buf);
		return 1;
	}
	else
	{
		++pathok;
		globok = (pathok==1?1:0);
	}

	return 0;
}

private void
sig_catch(sig_num)
int	sig_num;
{
	sig_found = sig_num;
	siglongjmp(jmpbuf, 1);
}
