/*
*      SCCS:  @(#)  ANSI.os/genuts/free/free.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/genuts/free - T.free Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.free

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the free() subroutine call.

	This testset does not use dbug.h because the dbug routines'
	use of malloc() & free() would interfere.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Tue Dec 1 17:08:25 BST 1986
MODIFICATIONS:	Andrew Twigger 12 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Joseph Nave 16 May 89
		Inserted signal catching

		Geoff Clare, 30 June 1989
		Corrected signal catching

		Geoff Clare, 7 July 1989
		Rewrite tests which misuse pointers

		David Sawyer
		Mon Jul 09 13:06:24 BST 1990
		Added resillience code.

		Stuart Boutell, 27 Nov 1990
			Removed PATH_XS_RPT - added functionally similar
			code.

		Stuart Boutell, 14 Jan 1991
			Improved signal catching

		Geoff Clare, 30 March 1994
			Rewrite test 1.

		Geoff Clare, 19 March 1997
			Free some blocks in test 1.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<string.h>
#include	<errno.h>
#include	<report.h>
#include	<unistd.h>
#include	<signal.h>
#include	<setjmp.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef free
#endif

#ifndef _XOPEN_SOURCE
#undef memset
extern	void	*memset();
#undef memchr
extern	void	*memchr();
#endif

#define NO_TESTS	2		/* number of tests in testset */
#define WAITTIME	(2 * SPEEDFACTOR)

extern	int	cppair();
extern	char	*errname();
extern	char	*signame();
extern	void	(*vsignal())();
extern	void	xs_rpt();
extern	void	xx_rpt();
#if TEST_ANSI
extern	void	in_rpt(char *, ...);
#else
extern	void	in_rpt();
#endif

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	ch_t1();
private void	ch_t2();
private	int	sig_found;
private	sigjmp_buf	jmpbuf;
private void	sig_catch();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private void
test1()
{
	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);
}

private void
ch_t1()
{
	char *memold;
	char *memnew;
	size_t	size = 20;
	int	i, j;
	int	pathok = 0;

	/* malloc a block and free it */
	if ((memold = (char *)malloc(size)) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("failed to malloc() %ld bytes of memory", (long)size);
		return;
	}
	else
		++pathok;
	free((void *)memold);

	for (i=0; i<1000; i++)
	{
		/* malloc a new block and check if it overlaps the freed block */
		if ((memnew = (char *)malloc(size)) == (char *) NULL)
		{
			xx_rpt(DELETION);
			in_rpt("failed to malloc() %ld bytes of memory %d times",
				(long)size, i+1);
			return;
		}
		else
			++pathok;
		
		for (j = 0; j < size; j++)
		{
			if (memnew == memold+j || memnew+size == memold+j)
			{
				if (pathok == i+2)
					xs_rpt();
				else
				{
					xx_rpt(DELETION);
					in_rpt("path tracing error: path counter %d, expected %d",
						pathok, i+2);
				}
				return;
			}
		}

		/* free some of the new blocks, to allow for systems where
		   freed blocks are coalesced, which may delay their reuse */
		if (i%100 < 10)
			free((void *)memnew);
	}

	xx_rpt(FAILURE);
	in_rpt("failed to reuse freed memory in next %d malloc() calls", i);
}

private void
test2()
{
	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);
}

private void
ch_t2()
{
	char *mempos[20];
	char *memnew[20];
	void (*oldsig)();
	int	i, j;
	int	pathok = 0;

	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		if ((mempos[i] = (char *)malloc((size_t)j)) == (char *) NULL)
		{
			xx_rpt(FAILURE);
			in_rpt("failed to malloc() %d bytes of memory", j);
			return;
		}
		else
			++pathok;
		(void) memset((void *)mempos[i], 0, (size_t) j);
	}

	/* set up signal catcher */
	oldsig = vsignal(SIGSEGV, sig_catch);
	sig_found = 0;

	if (sigsetjmp(jmpbuf, 1) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("free(NULL) generated signal %d (%s)", 
			sig_found, signame(sig_found));
		return;
	}
	else
		++pathok;

	free((void *) NULL);

	(void) vsignal(SIGSEGV, oldsig);

	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		if ((memnew[i] = (char *)malloc((size_t)j)) == (char *) NULL)
		{
			xx_rpt(FAILURE);
			in_rpt("failed to malloc() %d bytes after free(NULL)", j);
			return;
		}
		else
			++pathok;
		(void) memset((void *)memnew[i], 1, (size_t) j);
	}

	/* This is not very conclusive, but it's all we can do */
	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		if (memchr((void *)mempos[i], 1, (size_t) j) != NULL)
		{
			xx_rpt(FAILURE);
			in_rpt("free(NULL) freed some previously allocated memory");
			return;
		}
		else
			++pathok;
	}

	if(pathok==(20+1+20+20))
	{
		xs_rpt();
	}
	else
	{
		xx_rpt(DELETION);
		in_rpt("path tracing error: path counter %d, expected %d",
			pathok, 20+1+20+20);
	}
}

private	void 
sig_catch(sig_num)
int	sig_num;
{
	sig_found = sig_num;
	siglongjmp(jmpbuf, 1);
}
