/*
*      SCCS:  @(#)  ANSI.os/genuts/malloc/malloc.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/genuts/malloc - T.malloc Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.malloc

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the malloc() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Tue Dec 1 17:08:25 BST 1986
MODIFICATIONS:	Andrew Twigger 12 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 7 July 1989
			Rewrite code which misuses pointers
		
		Geoff Clare, 7 Dec 1989
		Do not assign objects larger than size malloc'ed

		David Sawyer
		Mon Jul 09 10:53:54 BST 1990
		Added resillience code.

		Stuart Boutell, 27 Nov 1990
			PATH_XS_RPT replaced by code - no debugging
			in malloc tests.

		Stuart Boutell, 14 Jan 1991
			test 3 added.

		Geoff Clare, 16 July 1991
			Test long double alignment if supported

		Geoff Clare, 11 July 1994
			Use whole numbers in floating point variables, to
			avoid rounding problems.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<string.h>
#include	<report.h>
#include	<setjmp.h>
#include	<signal.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef malloc
#endif

#ifndef _XOPEN_SOURCE
#undef memset
extern	void	*memset();
#endif

#define NO_TESTS	3		/* number of tests in testset */
#define WAITTIME	(2 * SPEEDFACTOR)

extern	void	(*vsignal())();
extern	char	*signame();
extern	void	xs_rpt();
extern	void	xx_rpt();
#if TEST_ANSI
extern	void	in_rpt(char *, ...);
#else
extern	void	in_rpt();
#endif
extern	int	globok;

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private int	test_align();
private	void	sig_catch();

private	int	sig_found;
private	sigjmp_buf	jmpbuf;

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};


private void
test1()
{
	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);
}

private void
ch_t1()
{
	char	*mempos[20];
	int	i,j,k;
	int	pathtot;
	int 	pathok = 0;

	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		if ((mempos[i] = (char *)malloc((size_t) j)) == (char *) NULL)
		{
			xx_rpt(FAILURE);
			in_rpt("failed to malloc() %d bytes of memory", j);
			for (k=1; k<no_tests; k++)
				(void) tet_delete(k, "malloc() failed in test1");
			return;
		}
		else
			++pathok;
		(void) memset((void *)mempos[i], i, (size_t) j);
	}

	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		for (k = 0; k < j; k++)
		{
			if (mempos[i][k] != i)
			{
				xx_rpt(FAILURE);
				in_rpt("malloc() returned buffers which overlap");
				return;
			}
			else
				++pathok;
		}
	}
	/* pathtot checks valuue of pathok */
	pathtot = 20;
	j = 1;
	for (i = 0; i < 20; i++)
	{
		j += i;
		for (k = 0; k < j; k++)
			++pathtot;
	}
	
	if(pathtot==pathok)
	{
		xs_rpt();
	}
	else
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: path check value %d",pathok);
	}
}

private void
test2()
{
	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);
	
	return;
}

private void
ch_t2()
{
	char 	*mempos[20];
	int	i,j,fail;
	int 	pathok = 0;

	fail = 0;

	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		if ((mempos[i] = (char *)malloc((size_t) j)) == (char *) NULL)
		{
			xx_rpt(FAILURE);
			in_rpt("failed to malloc() %d bytes of memory", j);
			return;
		}
		else
			++pathok;
		globok = 0;
		fail += test_align(j, mempos[i]);
		if (globok==1)
			++pathok;
		globok = 0;
	}

	if (fail == 0)
	{
		if(pathok==(2*20))
		{
			xs_rpt();
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: path check value %d",pathok);
		}	
	}

}

private void
test3()
{
	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);
}

private void
ch_t3()
{
	void	(*oldsig)();
	char	*ptr;
	int	pathok = 0;

	ptr = (char *)malloc( (size_t)0 );

	if (ptr == NULL)
	{
		++pathok;
	}
	else
	{
		oldsig = vsignal(SIGSEGV, sig_catch);
		sig_found = 0;
		if (sigsetjmp(jmpbuf, 1) != 0)
		{
			xx_rpt(FAILURE);
			in_rpt("Pointer returned from malloc(0)");
			in_rpt("could not be passed successfully to free()");
			in_rpt("free(ptr) generated signal %d (%s)",
				sig_found, signame(sig_found));
			return;
		}
		else
			++pathok;

		free( (void *)ptr );

		(void) vsignal(SIGSEGV, oldsig);
	}

	if (pathok == 1)
	{
		xs_rpt();
	}
	else
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: path check value %d", pathok);
	}
}


private int
test_align(nbyte, ptr)
int nbyte;
char *ptr;
{
	char	xchar = 'z';
	short	xshort = 12345;
	int	xint = 12345;
	long	xlong = 1234567;
	float	xfloat = 123.0;
	double	xdouble = 1234.0;
#if LONG_DOUBLE_SUPPORTED
	long double	xldouble = 12345.0;
#endif
	struct x { int a; char b; double c; } xstruct;
	struct x *xstrptr;
	char	buf[256];
	int	pathok = 0;

	*buf = '\0';
	globok = 0;

	/* If there are alignment problems, these tests will no doubt
	   cause signals rather than the nice error message at the end */

	if (nbyte >= sizeof(short))
	{
		*(short *)ptr = xshort;
		if (++*(short *)ptr != xshort+1)
			(void) strcat(buf, "short ");
	}

	if (nbyte >= sizeof(int))
	{
		*(int *)ptr = xint;
		if (++*(int *)ptr != xint+1)
			(void) strcat(buf, "int ");
	}

	if (nbyte >= sizeof(long))
	{
		*(long *)ptr = xlong;
		if (++*(long *)ptr != xlong+1)
			(void) strcat(buf, "long ");
	}

	if (nbyte >= sizeof(float))
	{
		*(float *)ptr = xfloat;
		if (++*(float *)ptr != xfloat+1)
			(void) strcat(buf, "float ");
	}

	if (nbyte >= sizeof(double))
	{
		*(double *)ptr = xdouble;
		if (++*(double *)ptr != xdouble+1)
			(void) strcat(buf, "double ");
	}

#if LONG_DOUBLE_SUPPORTED
	if (nbyte >= sizeof(long double))
	{
		*(long double *)ptr = xldouble;
		if (++*(long double *)ptr != xldouble+1)
			(void) strcat(buf, "long double ");
	}
#endif

	if (nbyte >= sizeof(char *))
	{
		*(char **)ptr = &xchar;
		if (++*(char **)ptr != &xchar+1)
			(void) strcat(buf, "(char *) ");
	}

	if (nbyte >= sizeof(short *))
	{
		*(short **)ptr = &xshort;
		if (++*(short **)ptr != &xshort+1)
			(void) strcat(buf, "(short *) ");
	}

	if (nbyte >= sizeof(int *))
	{
		*(int **)ptr = &xint;
		if (++*(int **)ptr != &xint+1)
			(void) strcat(buf, "(int *) ");
	}

	if (nbyte >= sizeof(long *))
	{
		*(long **)ptr = &xlong;
		if (++*(long **)ptr != &xlong+1)
			(void) strcat(buf, "(long *) ");
	}

	if (nbyte >= sizeof(float *))
	{
		*(float **)ptr = &xfloat;
		if (++*(float **)ptr != &xfloat+1)
			(void) strcat(buf, "(float *) ");
	}

	if (nbyte >= sizeof(double *))
	{
		*(double **)ptr = &xdouble;
		if (++*(double **)ptr != &xdouble+1)
			(void) strcat(buf, "(double *) ");
	}

#if LONG_DOUBLE_SUPPORTED
	if (nbyte >= sizeof(long double *))
	{
		*(long double **)ptr = &xldouble;
		if (++*(long double **)ptr != &xldouble+1)
			(void) strcat(buf, "(long double *) ");
	}
#endif

	if (nbyte >= sizeof(struct x *))
	{
		*(struct x **)ptr = &xstruct;
		if (++*(struct x **)ptr != &xstruct+1)
			(void) strcat(buf, "(struct x *) ");
	}

	if (nbyte >= sizeof(struct x **))
	{
		*(struct x ***)ptr = &xstrptr;
		if (++*(struct x ***)ptr != &xstrptr+1)
			(void) strcat(buf, "(struct x **) ");
	}

	if (*buf != '\0')
	{
		xx_rpt(FAILURE);
		in_rpt("pointer returned by malloc(%d) was not aligned correctly for the following types:",
			nbyte);
		in_rpt("%s", buf);
		return 1;
	}
	else
	{
		++pathok;
		globok = (pathok==1?1:0);
	}

	return 0;
}

private void
sig_catch(sig_num)
int	sig_num;
{
	sig_found = sig_num;
	siglongjmp(jmpbuf, 1);
}
