/*
*      SCCS:  @(#)  ANSI.os/genuts/realloc/realloc.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/genuts/realloc - T.realloc Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.realloc

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the realloc() subroutine call.

 	This testset does not use dbug.h because the dbug routines'
 	use of malloc() & free() would interfere.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Tue Dec 1 17:08:25 BST 1986
MODIFICATIONS:	Andrew Twigger 12 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Joseph Nave 16 May 89
		Inserted signal catching

		Geoff Clare, 30 June 1989
		Corrected signal catching

		Geoff Clare, 7 July 1989
		Rewrite tests which misuse pointers

		Geoff Clare, 21 Sep 1989
		Don't test that memory freed with realloc(buf, 0)
		is reused by next malloc().

		Geoff Clare, 7 Dec 1989
		Do not assign objects larger than size allocated

		David Sawyer
		Mon Jul 09 15:19:58 BST 1990
		Added resillience code.

		Stuart Boutell, 27 Nov 1990
			Modified resilience to not use debugging.

		Stuart Boutell, 14 Jan 1991
			test 2 inserted,
			test 6 added.

		Geoff Clare, 16 July 1991
			Test long double alignment if supported

		Geoff Clare, 11 July 1994
			Use whole numbers in floating point variables, to
			avoid rounding problems.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<string.h>
#include	<errno.h>
#include	<report.h>
#include	<unistd.h>
#include	<signal.h>
#include	<setjmp.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef realloc
#endif

#ifndef _XOPEN_SOURCE
#undef memset
extern	void	*memset();
#undef memchr
extern	void	*memchr();
#endif

#define NO_TESTS	6		/* number of tests in testset */
#define WAITTIME	(2 * SPEEDFACTOR)

extern	char	*signame();
extern	void	(*vsignal())();
extern	void	xs_rpt();
extern	void	xx_rpt();
#if TEST_ANSI
extern	void	in_rpt(char *, ...);
#else
extern	void	in_rpt();
#endif
extern	int	globok;

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	ch_t6();
private	int	sig_found;
private void	sig_catch();
private	int	test_align();
private	sigjmp_buf	jmpbuf;

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	NULL, 0
};

private void
test1()
{
	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);
}

private void
ch_t1()
{
	char *mempos[20];
	char *memrea[2];
	int	i, j, k;
	int	pathok = 0;

	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		if ((mempos[i] = (char *)malloc((size_t)j)) == (char *) NULL)
		{
			xx_rpt(FAILURE);
			in_rpt("failed to malloc() %d bytes of memory", j);
			return;
		}
		else
			++pathok;
		for (k=0; k<j; k++)
		{
			if (k==0)
				++pathok;
			mempos[i][k] = 'a' + i;
		}
	}

	if ((memrea[0] = (char *)realloc((void *)mempos[9], (size_t)100)) == (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("failed to realloc() 100 bytes");
		return;
	}
	else
		++pathok;
	if ((memrea[1] = (char *)realloc((void *)mempos[11], (size_t)10)) == (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("failed to realloc() 10 bytes");
		return;
	}
	else
		++pathok;

	for (k=0; k<45; k++)
	{
		if (memrea[0][k] != 'a' + 9)
		{
			xx_rpt(FAILURE);
			in_rpt("reallocated (enlarged) block has erroneous contents at byte %d", k);
			in_rpt("expected value: %c", 'a' + 9);
			in_rpt("observed value: %c", memrea[0][k]);
			return;
		}
		else
			++pathok;
	}

	for (k=0; k<10; k++)
	{
		if (memrea[1][k] != 'a' + 11)
		{
			xx_rpt(FAILURE);
			in_rpt("reallocated (enlarged) block has erroneous contents at byte %d", k);
			in_rpt("expected value: %c", 'a' + 11);
			in_rpt("observed value: %c", memrea[1][k]);
			return;
		}
		else
			++pathok;
	}

	if(pathok == (2*20+2+45+10))
	{
		xs_rpt();
	}
	else
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: path check value %d",pathok);
	}
}

private void
test2()
{
	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);
}

private void
ch_t2()
{
	char	*mempos[20];
	char	*rememp[20];
	int	i,j,k,fail;
	int	pathok = 0;

	fail = 0;
	j = 1;
	for( i=0; i<20; i++)
	{
		j += i;
		k = j + i;

		if((mempos[i] = (char *)malloc((size_t) j)) == NULL)
		{
			xx_rpt(DELETION);
			in_rpt("failed to malloc() %d bytes of memory", j);
			return;
		}
		else
			++pathok;

		if((rememp[i] = (char *)realloc((void *)mempos[i],(size_t)k)) == NULL)
		{
			xx_rpt(FAILURE);
			in_rpt("failed to realloc() %d bytes of memory", k);
			return;
		}
		else
			++pathok;

		globok = 0;
		fail += test_align(k, rememp[i]);
		if( globok == 1 )
			++pathok;
	}
	
	if(fail==0)
	{
		if(pathok==(3*20))
		{
			xs_rpt();
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: path check value %d",pathok);
		}
	}
}

private void
test3()
{
	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);
}

private void
ch_t3()
{
	char *mempos[20];
	char *newpos;
	char *memnew;
	int	i, j;
	size_t	size;
	int	pathok = 0;

	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		if ((mempos[i] = (char *)malloc((size_t)j)) == (char *) NULL)
		{
			xx_rpt(FAILURE);
			in_rpt("failed to malloc() %d bytes of memory", j);
			return;
		}
		else
			++pathok;
		if (i == 10)
			size = j;
	}

	(void) memset((void *)mempos[10], 0, size);

	newpos = (char *)realloc((void *)mempos[10], (size_t)(2*size));
	if (newpos == NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("realloc() failed on pointer returned by malloc()");
		return;
	}
	else
		++pathok;

	if (newpos == mempos[10])
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("realloc() did not move the buffer");
		return;
	}
	else
		++pathok;

	/* malloc a new space the same size as the one freed */
	if ((memnew = (char *)malloc(size)) == (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("failed to malloc() %d bytes after realloc()", (int)size);
		return;
	}
	else
		++pathok;

	(void) memset((void *)memnew, 1, size);

#if 0
/* THIS PART OF TEST DELETED */
	/* This may cause a signal if the memory hasn't been reused */
	if (memchr((void *)mempos[10], 1, size) == NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("failed to reuse freed memory for next malloc()");
	}
	else
		++pathok;
#endif

	if(pathok == (20+3))
	{
		xs_rpt();
	}
	else
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: path check value %d",pathok);
	}
}

private void
test4()
{
	(void) cppair(ch_t4, NULLFN, WAITTIME, 0);
}

private void
ch_t4()
{
	char 	*mempos[20];
	void 	(*oldsig)();
	int	i, j, k;
	int	fail = 0;
	int	pathloops = 0;
	int	pathok = 0;

	/* set up signal catcher */
	oldsig = vsignal(SIGSEGV, sig_catch);
	sig_found = 0;

	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		if (sigsetjmp(jmpbuf, 1) != 0)
			break;

		if ((mempos[i] = (char *)realloc((void *) NULL, (size_t)j)) 
			== (char *) NULL)
		{
			xx_rpt(FAILURE);
			in_rpt("realloc(NULL, %d) returned NULL", j);
			return;
		}
		else
			++pathok;
		(void) memset((void *)mempos[i], i, (size_t) j);
	}

	(void) vsignal(SIGSEGV, oldsig);

	if (sig_found)
	{
		xx_rpt(FAILURE);
		in_rpt("realloc(NULL, %d) generated signal %d (%s)", 
			j, sig_found, signame(sig_found));
		return;
	}
	else
		++pathok;	

	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		for (k = 0; k < j; k++)
		{
			if (mempos[i][k] != i)
			{
				xx_rpt(FAILURE);
				in_rpt("realloc(NULL, nbyte) returned buffers which overlap");
				fail++;
				break;
			}
			else
				++pathok;
		}
		globok = 0;
		fail += test_align(j, mempos[i]);
		if (globok==1)
			++pathok;
		globok = 0;
	}
	j = 1;
	for (i=0; i<20; i++) /* check value for pathok in above loops */
	{
		j += i;	
		for (k = 0; k < j; k++)
			++pathloops;	
		++pathloops;
	}

	if (fail == 0)
	{
		if(pathok == (20+1+pathloops))
		{
			xs_rpt();
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: path check value %d",pathok);
		}
	}
}

private void
test5()
{
	(void) cppair(ch_t5, NULLFN, WAITTIME, 0);
}

private void
ch_t5()
{
	char *mempos[20];
	char *memnew;
	int	i, j;
	int	pathok = 0;

	j = 1;
	for (i=0; i<20; i++)
	{
		j += i;
		if ((mempos[i] = (char *)malloc((size_t)j)) == (char *) NULL)
		{
			xx_rpt(FAILURE);
			in_rpt("failed to malloc() %d bytes of memory", j);
			return;
		}
		else
			++pathok;
	}
	--i;

	(void) memset((void *)mempos[i], 0, (size_t) j);

	(void) realloc((void *)mempos[i], (size_t) 0);

	/* malloc a new space the same size as the one freed */
	if ((memnew = (char *)malloc((size_t)j)) == (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("failed to malloc() %d bytes after realloc()", j);
		return;
	}
	else
		++pathok;

	(void) memset((void *)memnew, 1, (size_t) j);

#if 0
/* THIS PART OF TEST DELETED */
	/* This may cause a signal if the memory hasn't been reused */
	if (memchr((void *)mempos[i], 1, (size_t) j) == NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("failed to reuse freed memory for next malloc()");
	}
	else
		++pathok;
#endif

	if(pathok == (20+1))
	{
		xs_rpt();
	}
	else
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: path check value %d",pathok);
	}
}

private	void
test6()
{
	(void)	cppair(ch_t6, NULLFN, WAITTIME, 0);
}

private void
ch_t6()
{
	void	(*oldsig)();
	char	*ptr;
	int	pathok = 0;

	ptr = (char *)realloc( (void *)NULL, (size_t)0 );

	if (ptr == NULL)
	{
		++pathok;
	}
	else
	{
		oldsig = vsignal(SIGSEGV, sig_catch);
		sig_found = 0;
		if (sigsetjmp(jmpbuf, 1) != 0)
		{
			xx_rpt(FAILURE);
			in_rpt("Pointer returned from realloc((void *)NULL,(size_t)0");
			in_rpt("could not be passed successfully to free()");
			in_rpt("free(ptr) generated signal %d (%s)",
				sig_found, signame(sig_found));
			return;
		}
		else
			++pathok;

		free( (void *)ptr );
		(void) vsignal(SIGSEGV, oldsig);
	}

	if (pathok == 1)
	{
		xs_rpt();
	}
	else
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: path check value %d", pathok);
	}
}


private	void 
sig_catch(sig_num)
int	sig_num;
{
	sig_found = sig_num;
	siglongjmp(jmpbuf, 1);
}

private int
test_align(nbyte, ptr)
int nbyte;
char *ptr;
{
	char	xchar = 'z';
	short	xshort = 12345;
	int	xint = 12345;
	long	xlong = 1234567;
	float	xfloat = 123.0;
	double	xdouble = 1234.0;
#if LONG_DOUBLE_SUPPORTED
	long double	xldouble = 12345.0;
#endif
	struct x { int a; char b; double c; } xstruct;
	struct x *xstrptr;
	char	buf[256];

	*buf = '\0';
	globok = 0;

	/* If there are alignment problems, these tests will no doubt
	   cause signals rather than the nice error message at the end */

	if (nbyte >= sizeof(short))
	{
		*(short *)ptr = xshort;
		if (++*(short *)ptr != xshort+1)
			(void) strcat(buf, "short ");
	}

	if (nbyte >= sizeof(int))
	{
		*(int *)ptr = xint;
		if (++*(int *)ptr != xint+1)
			(void) strcat(buf, "int ");
	}

	if (nbyte >= sizeof(long))
	{
		*(long *)ptr = xlong;
		if (++*(long *)ptr != xlong+1)
			(void) strcat(buf, "long ");
	}

	if (nbyte >= sizeof(float))
	{
		*(float *)ptr = xfloat;
		if (++*(float *)ptr != xfloat+1)
			(void) strcat(buf, "float ");
	}

	if (nbyte >= sizeof(double))
	{
		*(double *)ptr = xdouble;
		if (++*(double *)ptr != xdouble+1)
			(void) strcat(buf, "double ");
	}

#if LONG_DOUBLE_SUPPORTED
	if (nbyte >= sizeof(long double))
	{
		*(long double *)ptr = xldouble;
		if (++*(long double *)ptr != xldouble+1)
			(void) strcat(buf, "long double ");
	}
#endif

	if (nbyte >= sizeof(char *))
	{
		*(char **)ptr = &xchar;
		if (++*(char **)ptr != &xchar+1)
			(void) strcat(buf, "(char *) ");
	}

	if (nbyte >= sizeof(short *))
	{
		*(short **)ptr = &xshort;
		if (++*(short **)ptr != &xshort+1)
			(void) strcat(buf, "(short *) ");
	}

	if (nbyte >= sizeof(int *))
	{
		*(int **)ptr = &xint;
		if (++*(int **)ptr != &xint+1)
			(void) strcat(buf, "(int *) ");
	}

	if (nbyte >= sizeof(long *))
	{
		*(long **)ptr = &xlong;
		if (++*(long **)ptr != &xlong+1)
			(void) strcat(buf, "(long *) ");
	}

	if (nbyte >= sizeof(float *))
	{
		*(float **)ptr = &xfloat;
		if (++*(float **)ptr != &xfloat+1)
			(void) strcat(buf, "(float *) ");
	}

	if (nbyte >= sizeof(double *))
	{
		*(double **)ptr = &xdouble;
		if (++*(double **)ptr != &xdouble+1)
			(void) strcat(buf, "(double *) ");
	}

#if LONG_DOUBLE_SUPPORTED
	if (nbyte >= sizeof(long double *))
	{
		*(long double **)ptr = &xldouble;
		if (++*(long double **)ptr != &xldouble+1)
			(void) strcat(buf, "(long double *) ");
	}
#endif

	if (nbyte >= sizeof(struct x *))
	{
		*(struct x **)ptr = &xstruct;
		if (++*(struct x **)ptr != &xstruct+1)
			(void) strcat(buf, "(struct x *) ");
	}

	if (nbyte >= sizeof(struct x **))
	{
		*(struct x ***)ptr = &xstrptr;
		if (++*(struct x ***)ptr != &xstrptr+1)
			(void) strcat(buf, "(struct x **) ");
	}

	if (*buf != '\0')
	{
		xx_rpt(FAILURE);
		in_rpt("pointer returned by realloc(NULL, %d) was not aligned correctly for the following types:",
			nbyte);
		in_rpt("%s", buf);
		return 1;
	}
	else
	{
		globok = 1;
	}

	return 0;
}
