/*
*      SCCS:  @(#)  ANSI.os/genuts/srand/srand.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/genuts/srand - T.srand Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.srand

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the srand() subroutine.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Mon Nov 24 11:38:47 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Linda Taylor, Hoskyns.	21st August, 1988.
		These tests were removed from T.rand
		Phyfos Photi, Hoskyns. 1st September 1988.
		test 3 added, tests 1&2 changed to use child processes

		David Sawyer
		Fri Jul 06 13:08:06 BST 1990
		Added resillience code

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef srand
#endif


#define 	NO_TESTS	3		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

extern	int	cppair();

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	ch_t1();
private void	ch_t2();
private void	gch_t2();

private	int	savdval[100];

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};


private void
test1()
{	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	i;
	int	svdval, rtval;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	srand(10);

	svdval = rand();

	for ( i = 0 ; i < 9 ; i++ )
	{
		if (i==0)
			PATH_TRACE;
		(void) rand();
	}

	srand(10);

	rtval = rand();

	if ( rtval != svdval ) 
	{
		xx_rpt(FAILURE);
		in_rpt("called srand(10) then called rand() ten times");
		in_rpt("last value returned was %d", svdval);
		in_rpt("called srand(10) again, then called rand() ten times");
		in_rpt("last value returned was %d", svdval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}


private void
test2()
{	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME * 2, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	i;

	DBUG_ENTER("ch_t2");

	srand(20);

	for ( i = 0 ; i < 100 ; i++ )
		savdval[i] = rand();

	(void) cppair(gch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
gch_t2()
{
	int	i;
	int	rtval;
	int	nfails = 0;
	int	pathok = 0;

	DBUG_ENTER("gch_t2");

	srand(20);

	for ( i = 0 ; i < 100 ; i++ )
	{
		if ( (rtval = rand()) != savdval[i] )
		{
			if ( nfails++ == 0 )
			{
				xx_rpt(FAILURE);
				in_rpt("called rand() in parent and child processes ");
			}
			in_rpt("call no. %d(starting from 0) produced", i);
			in_rpt("different results: parent %d child %d",
				savdval[i], rtval);
		}
		else
			PATH_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(100);
				
	DBUG_VOID_RETURN;
}


private void
test3()
{
	int	rtval;
	int	nfails = 0;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test3");

	/* NB srand() must not have been called before now
	      i.e. any calls in previous tests must be in children */

	for (i = 0; i < 100; i++)
	{
		if (i==0)
			PATH_TRACE;
		savdval[i] = rand();
	}

	srand(1);

	for ( i = 0 ; i < 100 ; i++ )
	{
		if ( (rtval = rand()) != savdval[i] )
		{
			if ( nfails++ == 0 )
			{
				xx_rpt(FAILURE);
				in_rpt("called rand() after no srand() calls and after srand(1)");
			}
			in_rpt("call no. %d(starting from 0) produced", i);
			in_rpt("different results: no srand() %d srand(1) %d",
				savdval[i], rtval);
		}
		else
			PATH_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(1+100);
				
	DBUG_VOID_RETURN;
}
