/*
*      SCCS:  @(#)  ANSI.os/genuts/strtol_X/strtol_X.c	Rel 4.4.1   (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/genuts/strtol_X - T.strtol_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strtol_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the strtol() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Fri Nov  7 10:59:37 WET 1986
MODIFICATIONS:	Andrew Twigger	09 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 30 June 1989
			Change ERANGE tests to align with ANSI - give
			warning result if unsuccessful.

		David Sawyer
		Mon Jul 09 15:40:09 BST 1990
		Added resillience code.

		Geoff Clare, 30 Jan 1991
			Change ERANGE test from WARNING to FAILURE.

		Geoff Clare, 15 Feb 1991
			Allow for successful return in EINVAL test
			in XPG3 mode (it's a "may error" in the XPG).

		Geoff Clare, 16 July 1991
			Add negative value tests

		Geoff Clare, 3 Oct 1991
			Fix missing in_rpt() argument.

		Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<string.h>
#include	<limits.h>
#include	<dbug.h>
#include	<report.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strtol
#endif


#define 	NO_TESTS	8		/* number of tests in testset */
#define		NMRBASES	36	/* number of different bases strtol() 
								handles	*/
#define		NMRS		8	/* size of array number[]	*/

extern	char	*errname();

int	no_tests = NO_TESTS;

private	char	*num[NMRBASES + 1] = {	(char *) NULL, (char *) NULL,
	"-10011100010000", "-111201101", "-2130100", "-310000", "-114144",
	"-41104", "-23420", "-14641", "-10000", "-7571", "-5954", "-4723",
	"-3904", "-2E6A", "-2710", "-20A4", "-1CFA", "-18D6", "-1500",
	"-11E4", "-KEC", "-IKI", "-H8G", "-G00", "-EKG", "-DJA", "-CL4",
	"-BPO", "-B3A", "-ACI", "-9OG", "-961", "-8M4", "-85P", "-7PS"
};

struct	number 	{
	char	*nmstr;		/* number string */
	int	nmbase;		/* base of number */
	int	nmind;		/* index of first char inconsistent with base */
};

private struct	number	number[NMRS] = 	{
						"  23 45",	9,	4,
						"2345",		4,	2,
						"abCDxyz",	19,	4,
						"230x3",	16,	3,
						"230X3",	16,	3,
						"0x2345",	17,	1,
						"0X2345",	18,	1,
						"12,34",	6,	2
					};

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	NULL, 0
};

private void
test1()
{
	int	nfails = 0;
	long	rtval;
	int	base;
	int	pathok = 0;

	DBUG_ENTER("test1");

	for (base = 2; base <= NMRBASES; base++)
	{
		if ( (rtval = strtol(&num[base][1], (char **) NULL, base))
				!= (long) 10000 )
		{
			if ( nfails++ == 0 )		/* first fail	*/
				xx_rpt(FAILURE);
			in_rpt("strtol(%s) to base %d not correct:",
				&num[base][1], base);
			in_rpt("should have returned 10000 but returned %ld",
			    			rtval);
		}
		else
			PATH_TRACE;

		if ( (rtval = strtol(num[base], (char **) NULL, base))
				!= (long) -10000 )
		{
			if ( nfails++ == 0 )		/* first fail	*/
				xx_rpt(FAILURE);
			in_rpt("strtol(%s) to base %d not correct:", num[base],
			   			base);
			in_rpt("should have returned -10000 but returned %ld",
			    			rtval);
		}
		else
			PATH_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(2*(NMRBASES-1));

	DBUG_VOID_RETURN;
}


private	void
test2()
{
	struct	number	*nmptr;
	char	*cptr;
	int	nfails = 0;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test2");

	for (i = 0; i < NMRS; i++)
	{
		nmptr = &number[i];

		(void) strtol(nmptr->nmstr, &cptr, nmptr->nmbase); 

		if ( cptr != &nmptr->nmstr[nmptr->nmind] )
		{
			if ( nfails++ == 0 )	/* first fail	*/
				xx_rpt(FAILURE);
			in_rpt("strtol(%s) with base %d:", nmptr->nmstr,
					nmptr->nmbase);
			in_rpt("%c returned as the 1st invalid character instead of %c",
					*cptr, nmptr->nmstr[nmptr->nmind]);
		}
		else
			PATH_TRACE;
	}

	if (nfails == 0)
		PATH_XS_RPT(NMRS);
			
	DBUG_VOID_RETURN;
}


private	void
test3()
{
	long	rtval;
	char	*sptr;
	char	*cptr;
	int	pathok = 0;

	DBUG_ENTER("test3");

	sptr = "  ABC";

	if ( (rtval = strtol(sptr, &cptr, 9)) != 0 || cptr != sptr )
	{
		xx_rpt(FAILURE);
		in_rpt("strtol(%s) with base 9:", sptr);
		if (rtval != 0)
			in_rpt("%ld returned instead of 0", rtval);
		if (cptr != sptr)
			in_rpt("first invalid char not set to first in string");
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}


private void
test4()
{
	long	rtval;
	int	pathok = 0;

	DBUG_ENTER("test4");

	if ( (rtval = strtol("023420", (char **) NULL, 0)) != 10000 )
	{
		xx_rpt(FAILURE);
		in_rpt("strtol(023420) with base 0:");
		in_rpt("%ld returned instead of 10000", rtval);
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}


private void
test5()
{
	long	rtval;
	int	nfails = 0;
	int	pathok = 0;

	DBUG_ENTER("test5");

	if ( (rtval = strtol("0x2710", (char **) NULL, 0)) != 10000 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("strtol(0x2710) with base 0:");
		in_rpt("%ld returned instead of 10000", rtval);
	}
	else
		PATH_TRACE;

	if ( (rtval = strtol("0X2710", (char **) NULL, 0)) != 10000 )
	{
		if ( nfails++ == 0 )	/* first fail	*/
			xx_rpt(FAILURE);
		in_rpt("strtol(0X2710) with base 0:");
		in_rpt("%ld returned instead of 10000", rtval);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test6()
{
	long	rtval;
	int	pathok = 0;

	DBUG_ENTER("test6");

	if ( (rtval = strtol("10000", (char **) NULL, 0)) != 10000 )
	{
		xx_rpt(FAILURE);
		in_rpt("strtol(10000) with base 0:");
		in_rpt("%ld returned instead of 10000", rtval);
	}
	else	
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}

private void
test7()
{
	long	rtval;
	int	nfails = 0;
	int	err;
	char	buf[256];
	int	pathok = 0;

	DBUG_ENTER("test7");

	(void) sprintf(buf, "%ld", LONG_MAX);
	(void) strcat(buf, "0");
	errno = 0;
	if ( (rtval = strtol(buf, (char **) NULL, 0)) != LONG_MAX
		|| errno != ERANGE)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("strtol(%s) with base 0:", buf);
		in_rpt("expected: return value %ld (LONG_MAX), errno ERANGE",
			LONG_MAX);
		in_rpt("actual: return value %ld, errno %d(%s) ", rtval, err, 
			errname(err));
		nfails++;
	}
	else
		PATH_TRACE;

	(void) sprintf(buf, "%ld", LONG_MIN);
	(void) strcat(buf, "0");
	errno = 0;
	if ( (rtval = strtol(buf, (char **) NULL, 0)) != LONG_MIN
		|| errno != ERANGE)
	{
		err = errno;
		if (!nfails)
			xx_rpt(FAILURE);
		in_rpt("strtol(%s) with base 0:", buf);
		in_rpt("expected: return value %ld (LONG_MIN), errno ERANGE",
			LONG_MIN);
		in_rpt("actual: return value %ld, errno %d(%s) ", rtval, err, 
			errname(err));
		nfails++;
	}
	else
		PATH_TRACE;

	if (!nfails)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	long	rtval;
	int	nfails = 0;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test8");

	errno = 0;
	if ( ( (rtval = strtol("10", (char **) NULL, 37)) != 0 
		|| errno != EINVAL)
#if !(TEST_XPG > 3)
		&& rtval != 37
#endif
		)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("strtol(10) with base 37:");
		in_rpt("expected: return value 0, errno EINVAL");
#if !(TEST_XPG > 3)
		in_rpt("      or: return value 37");
#endif
		in_rpt("  actual: return value %ld, errno %d(%s) ",
			rtval, err, errname(err));
		nfails++;
	}
	else
		PATH_TRACE;

	if (!nfails)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}
