/*
*      SCCS:  @(#)  ANSI.os/genuts/system/sys_t56/sys_t56.c	Rel 4.4.1 (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.system

PROJECT:	VSX (X/OPEN Validation Suite)


MODULE CONTAINS:
	tet_main()

SYNOPSIS:
	int main(argc, argv)
		int argc;
		char **argv;

DESCRIPTION:
	Exec'd by T.system_X for test56().

	pipe_xx_rpt etc used because newroot() alters the path to the
	results file.

AUTHOR:		Neil Moses
DATE CREATED:	Fri Oct 04 10:44:53 BST 1991
MODIFICATIONS:
		Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/

#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <dbug.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef system
#endif


#if TEST_XPG > 3
public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
#endif

extern	char	**environ;

extern	char	*errname();
extern	char	*testid();
extern	void	valrpt();
extern	void	set_pname();
extern	void	pipe_xs_rpt();
extern	void	pipe_xx_rpt();
extern	void	pipe_path_xs_rpt();
extern	void	pipe_in_rpt();
extern	void	pipe_end_rpt();
extern	void	pipe_devinod_rpt();

#if TEST_XPG > 3
private	void	subtest56();
#endif

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	set_pname(argv[0]);

#if TEST_XPG > 3

	if (argc < 2 || argc > 3)
	{
		pipe_xx_rpt(DELETION);
		pipe_in_rpt("argument count wrong");
		pipe_end_rpt();
		exit(OK);
	}
	if (argc >= 3)
		vg_dbug = argv[2];
	if (argc >= 2)
		vg_ident = argv[1];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	subtest56();

#else
	pipe_xx_rpt(NOTINUSE);
#endif

	pipe_end_rpt();
	exit(OK);
	/* NOTREACHED */
}


#if TEST_XPG > 3
private void
subtest56()
{
	int	pathok = 0;
	int	err, rval;
	char	msg[512];
	char	*cmd = (char *)NULL;

	DBUG_ENTER("subtest56");

	errno = NOERROR;

	/*
	 * Change 'root' directory for command
	 */
	if(newroot(".") != 0)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
 	      	(void) sprintf(msg, "newroot(\".\") failed - errno %d(%s)",
			err, errname(err));
		pipe_in_rpt(msg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Execute system() with null command pointer
	 */
	rval = system(cmd);

	if(rval != 0)
	{
		pipe_xx_rpt(FAILURE);
 	      	pipe_in_rpt("Call to system() with null command pointer and no ");
		pipe_in_rpt("available command interpreter, did not return zero");
		(void) sprintf(msg, "Expected return value 0 actual return value %d", rval);
		pipe_in_rpt(msg);
	}
	else
		pipe_path_xs_rpt(1, pathok);

	DBUG_VOID_RETURN;
}
#endif
