/*
*      SCCS:  @(#)  ANSI.os/jump/longjmp/longjmp.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/jump/longjmp - T.longjmp Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.longjmp

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the longjmp() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Thu Oct 23 16:52:01 BST 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 22 June 1994
			Ensure setjmp() is used in line with constraints.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<setjmp.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef longjmp
#endif


#define 	NO_TESTS	2		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)
#define		FILE1		"./longjmp-t.1"

extern	char	*errname();
extern	int	cppair();

int	no_tests = NO_TESTS;

private	int	jumpok;
private	jmp_buf	env;
private	int	a;
private void	test1();
private void	test2();
private void	ch_t1();
private void	ch_t2();
private void	func1();
private void	func2();
private void	func3();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private	void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t1()
{
	static int	fail1 = 0;
	static int	fail2 = 0;

	DBUG_ENTER("ch_t1");

	jumpok = 0;

	if (setjmp(env) != 0)	/* `second return' */
	{
		fail1 = 0;
		JUMP_TRACE;
	}
	else				/* `first return' */
	{
		if (fail1 == 0)
		{
			fail1 = 1;
			func1();
		}
	}

	if (setjmp(env) != 0)	/* `second return' */
	{
		fail2 = 0;
		JUMP_TRACE;
	}
	else				/* `first return' */
	{
		if (fail2 == 0)
		{
			fail2 = 1;
			func2();
		}
	}

	if (fail1 || fail2)
	{
		xx_rpt(FAILURE);
		if (fail1)
			in_rpt("longjmp called with value 5 but setjmp returned 0");
		if (fail2)
			in_rpt("longjmp called with value 0 but setjmp returned 0");
	}
	else
	{
		if (jumpok == 4)
			xs_rpt();
		else
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: path check value %d", jumpok);
		}
	}

	DBUG_VOID_RETURN;
}


private	void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{

	DBUG_ENTER("ch_t2");

	jumpok = 0;

	a = 3;

	if (setjmp(env) != 0)	/* `second return' */
	{
		if (a != 5)
		{
			xx_rpt(FAILURE);
			in_rpt("global variable `a' set to 5 before longjmp call - after setjmp returned, `a' had value %d", a);
		}
		else
		{
			if (jumpok == 1)
				xs_rpt();
			else
			{
				xx_rpt(DELETION);
				in_rpt("deletion reason: path check value %d", jumpok);
			}
		}
	}
	else				/* `first return' */
	{
		func3();
	}

	DBUG_VOID_RETURN;
}


private	void
func1()
{
	DBUG_ENTER("func1");

	JUMP_TRACE;

	longjmp(env, 5);

	DBUG_VOID_RETURN;
}


private	void
func2()
{
	DBUG_ENTER("func2");

	JUMP_TRACE;

	longjmp(env, 0);

	DBUG_VOID_RETURN;
}


private	void
func3()
{
	DBUG_ENTER("func3");

	JUMP_TRACE;

	a = 5;		/* global a */
	longjmp(env, 1);

	DBUG_VOID_RETURN;
}
