/*
*      SCCS:  @(#)  ANSI.os/locale/setlocale/setlocale.c	Rel 4.4.4	    (07/19/99)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/locale/setlocale - T.setlocale Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.setlocale

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the setlocale() subroutine call

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Phyfos Photi, Hoskyns Group plc
DATE CREATED:	17/08/88                         
MODIFIED:
		Geoff Clare, 2-6 Oct 1989
			Change INFO3LANG to INFO4LANG where used
			incorrectly as the "@ modified" language.
			Don't use psl_to_asc() on CRNCYSTR.
		
		Geoff Clare, 9 Oct 1989
			Incorrect gmtime() argument in checkday()
		
		Geoff Clare, 26 Oct 1989
			Swap args in strcoll() for INFO4LANG
		
		Geoff Clare, 22 Nov 1989
			Only use strcoll() when available
		
		Geoff Clare, 20 Dec 1989
			Add NULL to end of funcs struct.
		
		Geoff Clare, 4 Jan 1990
			Add #include <nl_types.h>

		Keith Holder, 17 July 1990
			Added 5 new tests.
			Added mods to set environment variable
			LC_ALL to NULL.

		J.A. Nave Thu Nov 29 16:25:22 GMT 1990
			Added resilience testing
		
		Geoff Clare, 29 May 1992
			Don't use literal language names.

		Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

		Geoff Clare, 23 March 1995
			Use character codes from <pslcodes.h>, and
			class tables set up by pslclass().

		Geoff Clare, 8 April 1999
			Fix radix character reporting in checkrad().

************************************************************************/


#include	<std.h>
#include	<stdio.h>
#include	<nlsinfo.h>
#include	<nlstest.h>
#include	<nlsclass.h>
#include	<pslcodes.h>
#include	<locale.h>
#ifdef _XOPEN_SOURCE
#include	<nl_types.h>
#include	<langinfo.h>
#endif /*_XOPEN_SOURCE*/
#include	<time.h>
#include	<dbug.h>
#include	<report.h>
#include	<string.h>
#include	<ctype.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef setlocale
#endif


#define NO_TESTS	18
#define WAITTIME	(20 * SPEEDFACTOR)
#define NUMLANG		6
#define NUMCATS		6
#define CATBREAK    	4
#define COLL_EQ		0
#define COLL_NE		1
#define COLL_GT		2
#define COLL_LT		3

extern	char	*psl_to_asc();
extern 	char	*asc_to_psl();
extern	char	*pslanginfo();
extern	char	*newstr();

extern	char    **environ;

int	no_tests = NO_TESTS;

private	int	testfail;
private	char	*testcomment;
private	char	*currlang;
private int 	langnum;
	
private	char tmp[NUMCATS][50];

private int	categories[NUMCATS] = { LC_ALL, LC_COLLATE, LC_CTYPE,
					LC_MONETARY, LC_NUMERIC, LC_TIME
				      };
 	
private char    *catnames[NUMCATS] = { "LC_ALL", "LC_COLLATE", "LC_CTYPE",
        			       "LC_MONETARY", "LC_NUMERIC", "LC_TIME"
				     };
private int	explang[NUMCATS];

private	char 	*langnames[NUMLANG] = { CLANG, NOINFOLANG, INFO1LANG, 
					INFO2LANG, INFO3LANG, INFO4LANG
				      };
private struct classchars {
	int chr;
	int def;
	int (*func)();
	char *funcnam;
} classchars[3];

private void	setup();
private void	choosefunc();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	ch_t6();
private void	ch_t7();
private void	ch_t8();
private void	ch_t9();
private void	ch_t10();
private void	ch_t11();
private void	ch_t12();
private void	ch_t13();
private void	ch_t14();
private void	ch_t15();
private void	ch_t16();
private void	ch_t17();
private void	ch_t18();
private	void	checklang();

public 	void  	(*setfunc)()     = setup;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	NULL, 0
};

private	void
setup()
{
	int i;

	DBUG_ENTER("setup");

	if (pslclass() != 0)
	{
		xx_all(DELETION, "pslclass() failed in setup");
		DBUG_VOID_RETURN;
	}

	/* find a set of characters which distinguish between locales
	   by classes */

	for (i = 0; i < 256; i++)
	{
		if (lang0_table[i][0] != lang1_table[i][0])
		{
			classchars[0].chr = i;
			choosefunc(&classchars[0],
				lang0_table[i][0], lang1_table[i][0]);
			break;
		}
	}

	for (i = 0; i < 256; i++)
	{
		if (lang0_table[i][0] != lang3_table[i][0])
		{
			classchars[1].chr = i;
			choosefunc(&classchars[1],
				lang0_table[i][0], lang3_table[i][0]);
			break;
		}
	}

	for (i = 0; i < 256; i++)
	{
		if (lang1_table[i][0] != lang3_table[i][0])
		{
			classchars[2].chr = i;
			choosefunc(&classchars[2],
				lang1_table[i][0], lang3_table[i][0]);
			break;
		}
	}

	for (i = 0; i < 3; i++)
	{
		DBUG_PRINTF("setup", "classchars[%d] chr = 0x%x, func = %s",
			i, classchars[i].chr, classchars[i].funcnam);
	}

	DBUG_VOID_RETURN;
}

private	void
choosefunc(classchp, defs1, defs2)
struct classchars *classchp;
int defs1;
int defs2;
{
	/* determine which ctype function to use to distringuish between
	   the classes in defs1 and defs2 */

	int xor;

	DBUG_ENTER("choosefunc");

	xor = (defs1 ^ defs2);

	if (xor & DEFUPPER)
	{
		classchp->def = DEFUPPER;
		classchp->func = (isupper); 
		classchp->funcnam = "isupper"; 
	}
	else if (xor & DEFLOWER)
	{
		classchp->def = DEFLOWER;
		classchp->func = (islower); 
		classchp->funcnam = "islower"; 
	}
	else if (xor & DEFPUNCT)
	{
		classchp->def = DEFPUNCT;
		classchp->func = (ispunct); 
		classchp->funcnam = "ispunct"; 
	}
	else if (xor & DEFSPACE)
	{
		classchp->def = DEFSPACE;
		classchp->func = (isspace); 
		classchp->funcnam = "isspace"; 
	}
	else if (xor & DEFALPHA)
	{
		classchp->def = DEFALPHA;
		classchp->func = (isalpha); 
		classchp->funcnam = "isalpha"; 
	}
	else if (xor & DEFGRAPH)
	{
		classchp->def = DEFGRAPH;
		classchp->func = (isgraph); 
		classchp->funcnam = "isgraph"; 
	}
	else
		xx_all(DELETION,
		    "INTERNAL ERROR: choosefunc() reached \"else\"");

	DBUG_VOID_RETURN;
}

private	void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t1()
{
	int	i, j;
	int	thislnum;
	char	*thislang;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	testfail = 0;
	testcomment = "setlocale sets the category of the programs locale";

	for (thislnum = 0; thislnum < NUMLANG; thislnum++)
	{	
		if (thislnum == VALNOINFOLANG)
			continue;

		thislang = langnames[thislnum];
		
		for (i = 1; i < NUMCATS; i++)
		{	
			if (setlocale(LC_ALL, "C") == (char *) NULL)
			{
				if (!testfail)
					xx_rpt(DELETION);
				in_rpt("deletion reason: setlocale(LC_ALL, C) failed");
			}
			else
			{
				PATH_TRACE;
				for (j=0; j<NUMCATS; j++)
					explang[j] = VALCLANG;
			}

			if (setlocale(categories[i], thislang) == (char *) NULL)
			{	
				if (!testfail)
					xx_rpt(FAILURE);
				in_rpt("setlocale(%s,%s) returned NULL pointer",							catnames[i], thislang);
				testfail++;
				continue;
			}
			else
				PATH_TRACE;

			explang[i] = thislnum;
			globok = 0;
			checklang();                     
			PATH_FUNC_TRACE;
		}	
	}

	if (!testfail)
		PATH_XS_RPT((NUMLANG-1)*(NUMCATS-1)*3);

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	i;
	int	thislnum;
	char	*thislang;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	testfail = 0;
	testcomment = "setlocale sets all categories of the programs locale";

	for (thislnum = 0; thislnum < NUMLANG; thislnum++)
	{	
		if (thislnum == VALNOINFOLANG || thislnum == VALINFO4LANG)
			continue;

		thislang = langnames[thislnum];
		
		if (setlocale(LC_ALL, thislang) == (char *) NULL)
		{	
			if (!testfail)
				xx_rpt(FAILURE);
			in_rpt("setlocale(LC_ALL, %s) returned NULL pointer", thislang);
			testfail++;
			continue;
		}
		else
			PATH_TRACE;

		for (i = 0; i < NUMCATS; i++)
			explang[i] = thislnum;

		globok = 0;
		checklang();                   
		PATH_FUNC_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(2*(NUMLANG-2));

	DBUG_VOID_RETURN;
}

private	void
test3()
{
	DBUG_ENTER("test3");

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()

{
	int	i;
	char	*s1,*s2;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	testfail = 0;
	testcomment = "when locale set to NULL, setlocale returns the current locale for the specified category";

	for (i = 1; i < NUMCATS; i++)
	{	
		if (setlocale(categories[i], INFO1LANG) == (char *) NULL)
		{	
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("setlocale(%s, %s) returned NULL pointer",
				catnames[i], INFO1LANG);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

	 	if ((s1 = setlocale(categories[i], (char *) NULL)) ==										 (char *) NULL)
		{	
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("setlocale(%s, NULL) returned NULL pointer", 								catnames[i]);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if ((s2 = newstr(s1)) == NULL)
		{	
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: malloc() failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

	 	if (setlocale(categories[i], INFO2LANG) == (char *) NULL)
		{	
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("setlocale(%s, %s) returned NULL pointer",
				catnames[i], INFO2LANG);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* restores old setting of locale */

	 	if (setlocale(categories[i], s2) == (char *) NULL)
		{	
			if (!testfail)
				xx_rpt(FAILURE);
			in_rpt("setlocale(%s, %s) returned NULL pointer", 								catnames[i], s2);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		explang[i] = VALINFO1LANG;
		globok = 0;
		checklang();                                    
		PATH_FUNC_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(6*(NUMCATS-1));

	DBUG_VOID_RETURN;
}

private	void
test4()
{
	DBUG_ENTER("test4");

	(void) cppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	i, j;
	int	thislnum;
	char	*thislang;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	testfail = 0;
	testcomment = "setlocale(LC_XXX, NULL) uses value of associated environment variable";

#if	(_XOPEN_SOURCE == 0 || TEST_XPG > 3)
	(void) vputenv(&environ, "LC_ALL=");
#endif
	for (thislnum = 0; thislnum < NUMLANG; thislnum++)
	{	
		if (thislnum == VALNOINFOLANG)
			continue;

		thislang = langnames[thislnum];
		
		for (i = 1; i < NUMCATS; i++)
		{
			if (setlocale(LC_ALL, "C") == (char *) NULL)
			{
				if (!testfail)
					xx_rpt(DELETION);
				in_rpt("deletion reason: setlocale(LC_ALL, C) failed");
			}
			else
			{
				PATH_TRACE;
				for (j=0; j<NUMCATS; j++)
					explang[j] = VALCLANG;
			}

			/* sets environment variables to valid locale */
			(void) strcat(strcat(strcpy(tmp[i], catnames[i]), "="),									thislang);
			(void) vputenv(&environ, tmp[i]);

			if (setlocale(categories[i], "") == (char *) NULL)
			{	
				if (!testfail)
					xx_rpt(DELETION);
				in_rpt("setlocale(%s,\"\") returned NULL pointer",								catnames[i]);
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;

			explang[i] = thislnum;
			globok = 0;
			checklang();
			PATH_FUNC_TRACE;
		}	
	}

	if (!testfail)
		PATH_XS_RPT((NUMLANG-1)*(NUMCATS-1)*3);

	DBUG_VOID_RETURN;
}

private	void
test5()
{
	DBUG_ENTER("test5");

	(void) cppair(ch_t5, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	i, j;
	int	thislnum;
	char	*thislang;
	int	pathok = 0;

	DBUG_ENTER("ch_t5");

	testfail = 0;
	testcomment = "setlocale(LC_XXX, NULL) uses value of LANG when associated environment variable is not set";

#if	(_XOPEN_SOURCE == 0 || TEST_XPG > 3)
	(void) vputenv(&environ, "LC_ALL=");
#endif

	for (thislnum = 0; thislnum < NUMLANG; thislnum++)
	{	
		if (thislnum == VALNOINFOLANG || thislnum == VALINFO4LANG)
			continue;

		thislang = langnames[thislnum];
		
		for (i = 1; i < NUMCATS; i++)
		{
			if (setlocale(LC_ALL, "C") == (char *) NULL)
			{
				if (!testfail++)
					xx_rpt(DELETION);
				in_rpt("deletion reason: setlocale(LC_ALL, C) failed");
			}
			else
			{
				PATH_TRACE;
				for (j=0; j<NUMCATS; j++)
					explang[j] = VALCLANG;
			}
			
			/* sets environment variables to empty string */
			(void) strcat(strcpy(tmp[i], catnames[i]), "=");
			(void) vputenv(&environ, tmp[i]);

			(void) strcat(strcpy(tmp[0], "LANG="), thislang);
			(void) vputenv(&environ, tmp[0]);

			if (setlocale(categories[i], "") == (char *) NULL)
			{	
				if (!testfail)
					xx_rpt(DELETION);
				in_rpt("setlocale(%s,\"\") returned NULL pointer",								catnames[i]);
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;

			explang[i] = thislnum;
			globok = 0;
			checklang();                   
			PATH_FUNC_TRACE;
		}	
	}

	if (!testfail)
		PATH_XS_RPT((NUMLANG-2)*(NUMCATS-1)*3);

	DBUG_VOID_RETURN;
}

private	void
test6()
{
	DBUG_ENTER("test6");

	(void) cppair(ch_t6, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t6");

	testfail = 0;
	testcomment = "setlocale(LC_ALL, NULL) uses value of LANG when other environment variables are not set";

#if	(_XOPEN_SOURCE == 0 || TEST_XPG > 3)
	(void) vputenv(&environ, "LC_ALL=");
#endif

       /* sets environment variables LC_COLLATE, LC_CTYPE, LC_MONETARY
        * to valid locale
        */

	for (i = 1; i < CATBREAK; i++)
	{	(void) strcat(strcat(strcpy(tmp[i],catnames[i]),"="),INFO1LANG);
		(void) vputenv(&environ, tmp[i]);
	}

	
	/* sets environment variables LC_NUMERIC, LC_TIME to empty string */

	for (i = CATBREAK; i < NUMCATS; i++)
	{	(void) strcat(strcpy(tmp[i], catnames[i]), "=");
		(void) vputenv(&environ, tmp[i]);
	}

	(void) strcat(strcat(strcpy(tmp[0],"LANG"),"="),INFO2LANG);
	(void) vputenv(&environ, tmp[0]);

	if (setlocale(LC_ALL, "") == (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("setlocale(LC_ALL, \"\") returned NULL pointer");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	explang[0] = VALINFO2LANG;
	for (i = 1; i < NUMCATS; i++)
	{	
		if (i < CATBREAK)
			explang[i]  = VALINFO1LANG;
		else
			explang[i]  = -1;
	}
	globok = 0;
	checklang();                   
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private	void
test7()
{
	DBUG_ENTER("test7");

	(void) cppair(ch_t7, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t7");

	testfail = 0;
	testcomment = "setlocale uses value of associated environment variable when requested locale is an empty string";

#if	(_XOPEN_SOURCE == 0 || TEST_XPG > 3)
	(void) vputenv(&environ, "LC_ALL=");
#endif

       /* sets environment variables LC_COLLATE, LC_CTYPE, LC_MONETARY
        * to valid locale
        */

	for (i = 1; i < CATBREAK; i++)
	{	(void) strcat(strcat(strcpy(tmp[i],catnames[i]),"="),INFO1LANG);
		(void) vputenv(&environ, tmp[i]);
	}

	
	/* sets environment variables LC_NUMERIC, LC_TIME to empty string */

	for (i = CATBREAK; i < NUMCATS; i++)
	{	(void) strcat(strcpy(tmp[i], catnames[i]), "=");
		(void) vputenv(&environ, tmp[i]);
	}

	(void) strcat(strcat(strcpy(tmp[0],"LANG"),"="),INFO2LANG);
	(void) vputenv(&environ, tmp[0]);

	if (setlocale(LC_ALL, "") == (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("setlocale(LC_ALL, \"\") returned NULL pointer");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	explang[0] = VALINFO2LANG;
	for (i = 1; i < NUMCATS; i++)
	{	
		if (i < CATBREAK)
			explang[i]  = -1;
		else
			explang[i]  = VALINFO2LANG;
	}
	globok = 0;
	checklang();                   
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private	void
test8()
{
	DBUG_ENTER("test8");

	(void) cppair(ch_t8, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t8");

	testfail = 0;
	testcomment = "locale string 'C' sets category to the 'C' locale";

	if (setlocale(LC_ALL, "C") == (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(FAILURE);
		in_rpt("setlocale(LC_ALL,\"C\") returned NULL");
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < NUMCATS; i++)
		explang[i] = VALCLANG;
	globok = 0;
	checklang();                 
	PATH_FUNC_TRACE;

	if (setlocale(LC_ALL, INFO1LANG) == (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("setlocale(LC_ALL, %s) returned NULL", INFO1LANG);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < NUMCATS; i++)
		explang[i] = VALINFO1LANG;

	for (i = 1; i < NUMCATS; i++)
	{	
		if (setlocale(categories[i], "C") == (char *) NULL)
		{	
			if (!testfail)
				xx_rpt(FAILURE);
			in_rpt("setlocale(%s,\"C\") returned NULL",catnames[i]);
			testfail++;
			continue;
		}
		else
			PATH_TRACE;

		explang[i] = VALCLANG;
		globok = 0;
		checklang();
		PATH_FUNC_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(2*(NUMCATS-1)+3);

	DBUG_VOID_RETURN;
}

private	void
test9()
{
	DBUG_ENTER("test9");

	(void) cppair(ch_t9, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int 	i;
	char	*badlocale = "BADLOCALE";
	int	pathok = 0;

	DBUG_ENTER("ch_t9");

	testfail = 0;
	testcomment = "if setlocale has not been succesfully called, \"C\" locale is set";

	/* setlocale() not called at all */

	for (i = 1; i < NUMCATS; i++)
		explang[i] = VALCLANG;
	globok = 0;
	checklang();                    
	PATH_FUNC_TRACE;

	/* setlocale() not successfully called */

	if (setlocale(LC_ALL, badlocale) != (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(FAILURE);
		in_rpt("setlocale(LC_ALL, %s) failed to return NULL", badlocale);
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 1; i < NUMCATS; i++)
		explang[i] = VALCLANG;

	globok = 0;
	checklang();                    
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private	void
test10()
{
	DBUG_ENTER("test10");

	(void) cppair(ch_t10, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	int 	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t10");

	testfail = 0;
	testcomment = "setlocale(LC_XXX, NULL) returns NULL if appropriate environment variable is invalid";

#if	(_XOPEN_SOURCE == 0 || TEST_XPG > 3)
	(void) vputenv(&environ, "LC_ALL=");
#endif

	if (setlocale(LC_ALL, INFO1LANG) == (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("setlocale(LC_ALL, %s) returned NULL", INFO1LANG);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<NUMCATS; i++)
		explang[i] = VALINFO1LANG;

	(void) strcat(strcat(strcpy(tmp[0],"LANG"),"="),INFO2LANG);
	(void) vputenv(&environ, tmp[0]);

       /* sets environment variables LC_COLLATE, LC_CTYPE, LC_MONETARY to
	* invalid locale and LC_NUMERIC, LC_TIME to empty string
	*/

	for (i = 1; i < NUMCATS; i++)
	{
		(void) strcat(strcpy(tmp[i],catnames[i]),"=invalidval");
		(void) vputenv(&environ, tmp[i]);
	}

	for (i = 1; i < NUMCATS; i++)
	{	
		if (setlocale(categories[i], "") != (char *) NULL)
		{	
			if (!testfail)
				xx_rpt(FAILURE);
			in_rpt("setlocale(%s, \"\") failed to return NULL", 									catnames[i]);
			testfail++;
			continue;
		}
		else
			PATH_TRACE;

		globok = 0;
		checklang();                     
		PATH_FUNC_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(2*(NUMCATS-1)+1);

	DBUG_VOID_RETURN;
}

private	void
test11()
{
	DBUG_ENTER("test11");

	(void) cppair(ch_t11, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	int 	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t11");

	testfail = 0;
	testcomment = "setlocale(LC_ALL, NULL) returns NULL if any LC_XXX environment variable is invalid";

#if	(_XOPEN_SOURCE == 0 || TEST_XPG > 3)
	(void) vputenv(&environ, "LC_ALL=");
#endif

	if (setlocale(LC_ALL, INFO1LANG) == (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("setlocale(LC_ALL, %s) returned NULL", INFO1LANG);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<NUMCATS; i++)
		explang[i] = VALINFO1LANG;

	(void) strcat(strcat(strcpy(tmp[0],"LANG"),"="),INFO2LANG);
	(void) vputenv(&environ, tmp[0]);

       /* sets environment variables LC_COLLATE, LC_CTYPE, LC_MONETARY to
	* invalid locale and LC_NUMERIC, LC_TIME to empty string
	*/

	for (i = 1; i < NUMCATS; i++)
		if (i < CATBREAK)
		{	(void) strcat(strcpy(tmp[i],catnames[i]),"=invalidval");
			(void) vputenv(&environ, tmp[i]);
		}
		else
		{	(void) strcat(strcpy(tmp[i], catnames[i]), "=");
			(void) vputenv(&environ, tmp[i]);
		}

	if (setlocale(LC_ALL, "") != (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(FAILURE);
		in_rpt("setlocale(LC_ALL, \"\") failed to return NULL");
		testfail++;
	}
	else
	{
		PATH_TRACE;
		globok = 0;
		checklang();                   
		PATH_FUNC_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private	void
test12()
{
	DBUG_ENTER("test12");

	(void) cppair(ch_t12, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	int 	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t12");

	testfail = 0;
	testcomment = "setlocale(LC_ALL, NULL) returns NULL if any LC_XXX environment variable is not set and LANG is invalid";

	if (setlocale(LC_ALL, INFO1LANG) == (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("setlocale(LC_ALL, %s) returned NULL", INFO1LANG);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<NUMCATS; i++)
		explang[i] = VALINFO1LANG;

	(void) vputenv(&environ, "LANG=invalidval");

#if	(_XOPEN_SOURCE == 0 || TEST_XPG > 3)
	(void) vputenv(&environ, "LC_ALL=");
#endif

       /* sets environment variables LC_COLLATE, LC_CTYPE, LC_MONETARY to
	* invalid locale and LC_NUMERIC, LC_TIME to empty string
	*/

	for (i = 1; i < NUMCATS; i++)
		if (i < CATBREAK)
		{	(void) strcat(strcat(strcpy(tmp[i],catnames[i]),"="),INFO2LANG);
			(void) vputenv(&environ, tmp[i]);
		}
		else
		{	(void) strcat(strcpy(tmp[i], catnames[i]), "=");
			(void) vputenv(&environ, tmp[i]);
		}

	if (setlocale(LC_ALL, "") != (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(FAILURE);
		in_rpt("setlocale(LC_ALL, \"\") failed to return NULL");
		testfail++;
	}
	else
	{
		PATH_TRACE;
		globok = 0;
		checklang();                   
		PATH_FUNC_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private	void
test13()
{
	DBUG_ENTER("test13");

	(void) cppair(ch_t13, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t13()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t13");

	testfail = 0;
	testcomment = "if environment variable has a modifier of format '@dict' appended, it is considered a modifier to requested setting of category";

	/* if the environment variable LC_COLLATE, LC_TYPE ... is of the 
	 * format "@dict" then this environment variable is considered as a 
	 * modifier to the current setting of the locale category.
	 */

	if (setlocale(LC_ALL, INFO1LANG) == (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("setlocale(LC_ALL, %s) returned NULL", INFO1LANG);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < NUMCATS; i++)
		explang[i] = VALINFO1LANG;

	for (i = 1; i < NUMCATS; i++)
	{	
		if (setlocale(categories[i], INFO4LANG) == (char *) NULL)
		{	
			if (!testfail)
				xx_rpt(FAILURE);
			in_rpt("setlocale(%s,%s) returned NULL",catnames[i], INFO4LANG);
			testfail++;
			continue;
		}
		else
			PATH_TRACE;

		explang[i] = VALINFO4LANG;
		globok = 0;
		checklang();
		PATH_FUNC_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(2*(NUMCATS-1) +1);

	DBUG_VOID_RETURN;
}


private void
test14()
{
	DBUG_ENTER("test14");

#if	(_XOPEN_SOURCE == 0 || TEST_XPG > 3)
	(void) cppair(ch_t14, NULLFN, WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
	in_rpt("not tested in XPG3 mode");
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t14()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t14");

	testfail = 0;
	testcomment = "setlocale does not change locale with invalid LANG";

       /* 
	* set environment variables LC_COLLATE, LC_CTYPE, LC_MONETARY
	* LC_NUMERIC, LC_TIME  to valid locale
        */


	for (i = 1; i < NUMCATS; i++) {
		(void) strcat(strcat(strcpy(tmp[i], catnames[i]), "="),INFO1LANG);
		(void) vputenv(&environ, tmp[i]);
	}


	if (setlocale(LC_ALL, "") == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("setlocale(LC_ALL, \"\") returned NULL pointer");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	explang[0] = VALINFO1LANG;

	/* unset appropriate environment variables */
	for (i = 1; i < NUMCATS; i++) {
		(void) strcat(strcpy(tmp[i], catnames[i]), "=");
		(void) vputenv(&environ, tmp[i]);
	}

	(void) vputenv(&environ, "LC_ALL=");

	(void) vputenv(&environ, "LANG=XXXX");

	for (i = 1; i < NUMCATS; i++)
	{
		if (setlocale(categories[i], "") != (char *) NULL)
		{
			if (!testfail++)
				xx_rpt(FAILURE);
			in_rpt("setlocale(%s, \"\") did not return NULL pointer",catnames[i]);
		}
		else
			PATH_TRACE;
	}

	for (i = 1; i < NUMCATS; i++)
		explang[i]  = VALINFO1LANG;

	globok = 0;
	checklang();                   
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT((NUMCATS-1)+2);

	DBUG_VOID_RETURN;
}

private void
test15()
{
	DBUG_ENTER("test15");
#if	(_XOPEN_SOURCE == 0 || TEST_XPG > 3)
	(void) cppair(ch_t15, NULLFN, WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
	in_rpt("not tested in XPG3 mode");
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t15()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t15");

	testfail = 0;
	testcomment = "setlocale changes locale to that of env. variable LC_ALL";

	for (i = 0; i < NUMCATS; i++)
		explang[i]  = VALINFO2LANG;

	(void) strcat(strcat(strcpy(tmp[0],"LC_ALL"),"="),INFO2LANG);
	(void) vputenv(&environ, tmp[0]);

	(void) setlocale(LC_ALL, "") ;

	globok = 0;
	checklang();                   
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	DBUG_ENTER("test16");
#if	(_XOPEN_SOURCE == 0 || TEST_XPG > 3)
	(void) cppair(ch_t16, NULLFN, WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
	in_rpt("not tested in XPG3 mode");
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t16()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t16");


	testfail = 0;
	testcomment = "setlocale does not change locale when env. variable LC_ALL is invalid";



	if (setlocale(LC_ALL, INFO1LANG) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("setlocale(LC_ALL, %s) returned NULL pointer", INFO1LANG);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	(void) vputenv(&environ, "LC_ALL=invalidval");

	for (i = 0; i < NUMCATS; i++)
		explang[i]  = VALINFO1LANG;


	for (i = 0; i < NUMCATS; i++)
	{
		if (setlocale(categories[i], "") != (char *) NULL)
		{
			if (!testfail++) 
				xx_rpt(FAILURE);
			in_rpt("setlocale(%s, \"\") did not return NULL pointer",catnames[i]);
			continue;
		}
		else
			PATH_TRACE;

		globok = 0;
		checklang();                   
		PATH_FUNC_TRACE;
	}


	if (!testfail)
		PATH_XS_RPT(2*NUMCATS+1);

	DBUG_VOID_RETURN;
}

private	void
test17()
{
	DBUG_ENTER("test17");

#if	(_XOPEN_SOURCE == 0 || TEST_XPG > 3)
	(void) cppair(ch_t17, NULLFN, WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
	in_rpt("not tested in XPG3 mode");
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t17()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t17");

	testfail = 0;
	testcomment = "locale string 'POSIX' sets category to the 'C' locale";

	if (setlocale(LC_ALL, "POSIX") == (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(FAILURE);
		in_rpt("setlocale(LC_ALL,\"POSIX\") returned NULL");
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < NUMCATS; i++)
		explang[i] = VALCLANG;

	globok = 0;
	checklang();                 
	PATH_FUNC_TRACE;

	if (setlocale(LC_ALL, INFO1LANG) == (char *) NULL)
	{	
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("setlocale(LC_ALL, %s) returned NULL", INFO1LANG);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < NUMCATS; i++)
		explang[i] = VALINFO1LANG;

	for (i = 1; i < NUMCATS; i++)
	{	
		if (setlocale(categories[i], "POSIX") == (char *) NULL)
		{	
			if (!testfail++)
				xx_rpt(FAILURE);
			in_rpt("setlocale(%s,\"POSIX\") returned NULL",catnames[i]);
			continue;
		}
		else
			PATH_TRACE;

		explang[i] = VALCLANG;
		globok = 0;
		checklang();
		PATH_FUNC_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(2*(NUMCATS-1)+3);

	DBUG_VOID_RETURN;
}

private	void
test18()
{
	DBUG_ENTER("test18");

#if	(_XOPEN_SOURCE == 0 || TEST_XPG > 3)
	(void) cppair(ch_t18, NULLFN, WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
	in_rpt("not tested in XPG3 mode");
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t18()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t18");

	testfail = 0;
	testcomment = "No call to setlocale, then the 'C' locale is the locale associated with the process";

	for (i = 0; i < NUMCATS; i++)
		explang[i] = VALCLANG;

	globok = 0;
	checklang();                 
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;

}

private void
checkval(name, val, rval, expval)

char	*name;
int	val, rval, expval;

{
	int	pathok = 0;

	DBUG_ENTER("checkval");

	if (rval && expval)
	{
		PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_VOID_RETURN;
	}

	if (!rval && !expval)
	{
		PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_VOID_RETURN;
	}

	if (!testfail++)
	{
		xx_rpt(FAILURE);
		in_rpt(testcomment);
	}
	else
		PATH_TRACE;

	in_rpt("%s(0x%x) failed", name, val);
	in_rpt("Current language binding is %s", currlang);
	in_rpt("Expected return value: %d", expval);
	in_rpt("Actual   return value: %d", rval);

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

#if defined(_XOPEN_SOURCE) || TEST_ANSI
private void
checkcoll(s1,s2,expcoll)
char *s1,*s2;
int expcoll;
{
	int	rval;
	int	pathok = 0;

	DBUG_ENTER("checkcoll");

	rval = strcoll(s1, s2);
	if (expcoll == COLL_EQ && rval != 0 ||
	    expcoll == COLL_NE && rval == 0 ||
	    expcoll == COLL_GT && rval <= 0 ||
	    expcoll == COLL_LT && rval >= 0)
	{	
		if (!testfail++)
		{
			xx_rpt(FAILURE);
			in_rpt(testcomment);
		}
		in_rpt("strcoll(%s, %s) failed",
			psl_to_asc(s1, langnum),
			psl_to_asc(s2, langnum));
		in_rpt("current language binding is %s", currlang);
		in_rpt("Expected return value %s 0",
			(expcoll == COLL_EQ) ? "=" :
			(expcoll == COLL_NE) ? "!=" :
			(expcoll == COLL_GT) ? ">" : "<");
		in_rpt("Actual   return value : %d", rval);
	}	
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);
	DBUG_VOID_RETURN;
}

#define checkcomp(s1,s2) checkcoll(s1,s2,COLL_LT)

#endif /* _XOPEN_SOURCE || TEST_ANSI */

private void
checkrad(radstr)
char *radstr;
{
	char pbuf[6];
	int	pathok = 0;

	DBUG_ENTER("checkrad");

	(void) sprintf(pbuf, "%4.2f", 12.34);

#ifdef TESTBED
	pbuf[2] = '.';
#endif

	if (pbuf[2] != radstr[0])
	{	
		if (!testfail++)
		{
			xx_rpt(FAILURE);
			in_rpt(testcomment);
		}
		in_rpt("sprintf(buf, \"%%4.2f\", 12.34) failed");
		in_rpt("current language binding is %s", currlang);
		in_rpt("Expected radix character %s", psl_to_asc(radstr, langnum));
		pbuf[3] = '\0';
		in_rpt("Actual   radix character %s", psl_to_asc(&pbuf[2], langnum));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
checkday(abday5str)
char *abday5str;
{
	char pbuf[12];
	struct tm *timptr;
	time_t t = 0;
	int	pathok = 0;

	DBUG_ENTER("checkday");

	timptr = gmtime(&t);

	(void) strftime(pbuf, 12, "%a", timptr);

	if (strcmp(pbuf, abday5str))
	{	
		if (!testfail++)
		{
			xx_rpt(FAILURE);
			in_rpt(testcomment);
		}
		in_rpt("strftime(pbuf, 12, \"%%a\", timptr) failed");
		in_rpt("current language binding is %s", currlang);
		in_rpt("Expected day name %s", psl_to_asc(abday5str, langnum));
		in_rpt("Actual   day name %s", psl_to_asc(pbuf, langnum));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

#ifdef _XOPEN_SOURCE
private void
checkcrcy(currcystr)
char *currcystr;
{
	char *pbuf;
	int	pathok = 0;

	DBUG_ENTER("checkcrcy");

	pbuf = nl_langinfo(CRNCYSTR);

	if (strcmp(pbuf, currcystr))
	{	
		if (!testfail++)
		{
			xx_rpt(FAILURE);
			in_rpt(testcomment);
		}
		in_rpt("nl_langinfo(CRNCYSTR) failed");
		in_rpt("current language binding is %s", currlang);
		in_rpt("Expected currency %s", psl_to_asc(currcystr, 0));
		in_rpt("Actual   currency %s", psl_to_asc(pbuf, 0));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}
#endif /*_XOPEN_SOURCE*/

private void
checklang()
{
	int	catnum, i, chdefs;
	struct classchars *clchp;
	char	*radstr;
	char	*abday5str;
	char	*currcystr;
	int	pathok = 0;
	int	count = 0;
	unsigned char tstr[10];

	DBUG_ENTER("checklang");

	for (catnum = 0; catnum < NUMCATS; catnum++)
	{
		langnum = explang[catnum];
		if (langnum == -1)
			continue;

		currlang = langnames[langnum];

		if (categories[catnum] == LC_CTYPE)
		{
			for (i = 0, clchp = classchars; i < 3; i++, clchp++)
			{
				chdefs = (langnum == VALCLANG) ?
					  lang0_table[clchp->chr][0] :
				       (langnum == VALINFO1LANG ||
				        langnum == VALINFO2LANG) ?
					  lang1_table[clchp->chr][0] :
					  lang3_table[clchp->chr][0];

				globok = 0;
				checkval(clchp->funcnam, clchp->chr,
					(*(clchp->func))(clchp->chr),
					(clchp->def & chdefs) != 0);
				PATH_FUNC_TRACE;
			}
			count += 3;
		}
#if defined(_XOPEN_SOURCE) || TEST_ANSI
		else if (categories[catnum] == LC_COLLATE)
		{
			/* testable with strcoll() if it's available */
			switch(langnum)
			{
			case VALCLANG :

				/* "a" comes before "A" in INFO1,2,3 */
				globok = 0;
				checkcoll("a", "A", COLL_GT);
				PATH_FUNC_TRACE;

				/* eszet == "ss" in INFO4 */
				tstr[0] = PSL34_eszet;
				tstr[1] = '\0';
				globok = 0;
				checkcoll((char *)tstr, "ss", COLL_NE);
				PATH_FUNC_TRACE;
				break;

			case VALINFO1LANG :

				/* "a" comes after "A" in CLANG and INFO4 */
				globok = 0;
				checkcoll("a", "A", COLL_LT);
				PATH_FUNC_TRACE;

				/* "Ab" comes after "ac" in INFO2 and INFO3 */
				globok = 0;
				checkcoll("Ab", "ac", COLL_LT);
				PATH_FUNC_TRACE;
				break;

			case VALINFO2LANG :

				/* "Ab" comes before "ac" in CLANG and INFO1,4 */
				globok = 0;
				checkcoll("Ab", "ac", COLL_GT);
				PATH_FUNC_TRACE;

				/* "Chi" comes before "Ci" in INFO3 */
				globok = 0;
				checkcoll("Chi", "Ci", COLL_GT);
				PATH_FUNC_TRACE;
				break;

			case VALINFO3LANG :

				/* "Ab" comes before "ac" in CLANG and INFO1,4 */
				globok = 0;
				checkcoll("Ab", "ac", COLL_GT);
				PATH_FUNC_TRACE;

				/* "Chi" comes after "Ci" in INFO2 */
				globok = 0;
				checkcoll("Chi", "Ci", COLL_LT);
				PATH_FUNC_TRACE;
				break;

			case VALINFO4LANG :

				/* "a" comes before "A" in INFO1,2,3 */
				globok = 0;
				checkcoll("a", "A", COLL_GT);
				PATH_FUNC_TRACE;

				/* eszet != "ss" in CLANG */
				tstr[0] = PSL34_eszet;
				tstr[1] = '\0';
				globok = 0;
				checkcoll((char *)tstr, "ss", COLL_EQ);
				PATH_FUNC_TRACE;
				break;
			}
			count += 2;
		} 
#endif /* _XOPEN_SOURCE || TEST_ANSI */
		else if (categories[catnum] == LC_NUMERIC)
		{
			radstr = pslanginfo(VSX_RADIXCHAR, langnum);
			globok = 0;
			checkrad(radstr);
			PATH_FUNC_TRACE;
			count++;
		}
		else if (categories[catnum] == LC_TIME)
		{
			abday5str = pslanginfo(VSX_ABDAY_5, langnum);
			globok = 0;
			checkday(abday5str);
			PATH_FUNC_TRACE;
			count++;
		}
#ifdef _XOPEN_SOURCE
		else if (categories[catnum] == LC_MONETARY)
		{
			/* testable using nl_langinfo - X/Open only */
			currcystr = pslanginfo(VSX_CRNCYSTR, langnum);
			globok = 0;
			checkcrcy(currcystr);
			PATH_FUNC_TRACE;
			count++;
		}
#endif /*_XOPEN_SOURCE*/
	}
	
	PATH_FUNC_RPT(count);

	DBUG_VOID_RETURN;
}
