/*
*      SCCS:  @(#)  ANSI.os/maths/asin/asin.c	     Rel 4.4.1	 (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/maths/asin - T.asin Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.asin

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the asin() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()
		Keith Holder 5 April 90
		Added test for asin(NaN)
		Anthony Lee 3 July 90
		Added resillience code

		J.A. Nave	Thu Oct 11 13:17:16 BST 1990
		Modified dev_report() so that errno must be EDOM,
		if running in vsx4 mode

		Stuart Boutell, 25 Apr 1991
			Modified dev_report so that errno may be EDOM

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<limits.h>
#include	<math.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef asin
#endif


#ifndef M_PI_2
#define M_PI_2	1.57079632679489661923
#endif

#define NO_TESTS	4		 /* number of tests in testset */
#define	WAITTIME	(30 * SPEEDFACTOR * hwfactor)

extern	void	mlmachar();
extern	void	mlvalrpt();
extern	void	mlrantest();
extern	void	mlsetglobals();
extern	void	mlreport();
extern	int	mcppair();

extern double low;
extern double high;
extern int dsignif;
extern int hwfactor;
extern int iterrs;

int	no_tests = NO_TESTS;

private	FILE	*fdnull;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	comply_report();
private void	dev_report();

public 	void	(*setfunc) () = mlmachar;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private void
comply_report(val,pathok,pathcheck)
double val;
int pathok;
int pathcheck;
{
	DBUG_ENTER("comply_report");

	if (!mlresults(val))
	{
		PATH_TRACE; 
		PATH_XS_RPT(++pathcheck);
	}
	else
	{
		xx_rpt(FAILURE);
		mlreport(low, high);
	}

	DBUG_VOID_RETURN;
}

private void
dev_report(failtest, expval, obsval, experr, obserr,pathok,pathchk)
int failtest;
double expval, obsval;
int experr, obserr;
int pathok, pathchk;
{
	DBUG_ENTER("dev_report");

	if (failtest < 0)
	{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
		if (obserr != 0 && obserr != experr)
#else
		if (obserr != 0 && obserr != experr)
#endif
			failtest = 1;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (failtest == 0)
	{
		if (obserr != experr)
			failtest = 1;
		else
			PATH_TRACE;
	}
	else
			PATH_TRACE;
			
	if (failtest > 0)
	{
		xx_rpt(FAILURE);
		mlvalrpt(expval, obsval, experr, obserr);
	}
	else
		PATH_XS_RPT(pathchk+2);

	DBUG_VOID_RETURN;
}

double t1f(x) double x;
{
return(x);
}

double t1g(x) double x;
{
return(sin(asin(x)));
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) mcppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int pathok = 0;

	DBUG_ENTER("ch_t1");

	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();
	low = -1.0;
	high = 1.0;
	iterrs = 2000;
	mlrantest(t1f,t1g,0);
	comply_report(5.0,pathok,0);
	DBUG_VOID_RETURN;
}

double t2f(x) 
double x;
{
	double y;

	y = asin(x);

	if (y < -M_PI_2)
		return(-1.0);

	if (y > M_PI_2)
		return(1.0);

	return(0.1);
}

double t2g(x) double x;
{
#ifdef	lint
	x = x;
#endif	/* lint */
	return(0.1);
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) mcppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int pathok = 0;

	DBUG_ENTER("ch_t2");

	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	low = -1.0;
	high = 1.0;
	iterrs = 2000;
	mlrantest(t2f,t2g,0);
	comply_report(1.0,pathok,0);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) mcppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int pathok = 0;
	double dval;
	int failtest = 1;

	DBUG_ENTER("ch_t3");

	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	errno = 0;
	dval=asin(1.1);
	if (isnan(dval))
	{
		PATH_TRACE; 
		failtest = -1;
	}
	else if (dval == 0.0)
	{
		PATH_TRACE; 
		failtest = 0;
	}
	dev_report(failtest, 0.0, dval, EDOM, errno, pathok,1);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	(void) mcppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int pathok = 0;
	double dval;
	double myNaN;
	int failtest = 1;
	int err;

	DBUG_ENTER("ch_t4");

	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	errno = 0;
	dval = sqrt(-1.0);
	if ( !isnan(dval) ) 
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("could not determine NaN");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	myNaN = dval;
	mlsetglobals();
	errno = 0;
	dval = asin(myNaN);
	err = errno;
	if (isnan(dval))
		failtest = -1;
	dev_report(failtest, myNaN, dval, EDOM, err,pathok,1);

	DBUG_VOID_RETURN;
}
