/*
*      SCCS:  @(#)  ANSI.os/maths/atan2/atan2.c	     Rel 4.4.3	 (07/20/98)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/maths/atan2 - T.atan2 Rel 4.4.3 (07/20/98)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.atan2

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the atan2() subroutines.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()
		Keith Holder 5 April 90
		Added atan2() with Nan test
		Anthony Lee 4 July 1990
		Added resillence code.

		J.A. Nave	Fri Oct 12 10:52:06 BST 1990
		Modified dev_report() so that errno must be EDOM,
		if running in vsx4 mode

		Stuart Boutell, 25 Apr 1991
			Modified dev_report so that errno can be EDOM.
		
		Geoff Clare, 23 Sept 1994
			Add non-XPG3 part of test 2.

		Geoff Clare, 6 March 1998
			Rename random() to mlrandom().
		
************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<limits.h>
#include	<math.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef atan2
#endif


#ifndef M_PI
#define M_PI		3.14159265358979323846
#endif

#define NO_TESTS	3		 /* number of tests in testset */
#define	WAITTIME	(30 * SPEEDFACTOR * hwfactor)

extern	void	mlmachar();
extern	void	mlvalrpt();
extern	void	mlrantest();
extern	void	mlsetglobals();
extern	void	mlreport();
extern	int	mcppair();
extern	char	*errname();

extern double mlrandom();
extern double low;
extern double high;
extern double argy;
extern int dsignif;
extern int hwfactor;
extern int iterrs;

int	no_tests = NO_TESTS;

private	FILE	*fdnull;
private int	nofails;

private void	test1();
private void	test2();
private void	test3();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	comply_report();
private void	dev_report();

public	void	(*setfunc) () = mlmachar;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private void
comply_report(val,pathok,pathchk)
double val;
int pathok, pathchk;
{
	DBUG_ENTER("comply_report");

	if (!mlresults(val))
	{
		PATH_TRACE; 
		PATH_XS_RPT(++pathchk);
	}
	else
	{
		xx_rpt(FAILURE);
		mlreport(low, high);
	}

	DBUG_VOID_RETURN;
}

private void
dev_report(failtest, expval, obsval, experr, obserr)
int failtest;
double expval, obsval;
int experr, obserr;
{
	int pathok = 0; 
	DBUG_ENTER("dev_report");

	globok = 0;
	
	if (failtest < 0)
	{
		if (obserr != 0 && obserr != experr)
		{
			PATH_TRACE; 
			failtest = 1;
		}
		else
			PATH_TRACE; 
	}
	else
		PATH_TRACE;
		
	if (failtest == 0)
	{
		if (obserr != experr)
		{
			PATH_TRACE; 
			failtest = 1;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE; 
		
	if (failtest > 0)
	{
		if (!nofails)
			xx_rpt(FAILURE);
		mlvalrpt(expval, obsval, experr, obserr);
		nofails++;
	}
	else
		PATH_FUNC_RPT(2);


	DBUG_VOID_RETURN;
}

double t1f(x)
double x;
{
	double at;

	at = atan(argy/x);
	if (x > 0)
		return (at);
	if (argy < 0)
		return (at - M_PI);
	else
		return (at + M_PI);
}

double t1g(x)
double x;
{
	return(atan2(argy, x));
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) mcppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int pathok = 0;
	int i;

	DBUG_ENTER("ch_t1");

	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();

	for (i=0; i<50; i++)
	{
		high = 5.0;
		low = -5.0;
		iterrs=40;
		argy=(mlrandom()*(high-low)) + low;
		mlrantest(t1f,t1g,0);
		PATH_TRACE; 
	}
	comply_report(5.0,pathok,50);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) mcppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int pathok = 0;
#if TEST_XPG == 3
	double dval;
	int failtest = 1;
#else
	int err;
#endif

	DBUG_ENTER("ch_t2");

	fdnull = freopen("/dev/null", "w", stderr);

	nofails = 0;

#if TEST_XPG == 3

	mlsetglobals();

	errno = 0;
	dval=atan2(0.0, 0.0);
	if (isnan(dval))
	{	
		PATH_TRACE; 
		failtest = -1;
	}
	else if (dval == 0.0)
	{
		PATH_TRACE; 
		failtest = 0;
	}

	globok = 0; 
	dev_report(failtest, 0.0, dval, EDOM, errno);
	PATH_FUNC_TRACE; 
#else
	PATH_TRACE; 

	errno = 0;
	(void) atan2(0.0, 0.0);
	if (errno != 0 && errno != EDOM)
	{
		err = errno;
		nofails++;
		xx_rpt(FAILURE);
		in_rpt("atan2(0,0) set errno to %d (%s), expected not set or set to %d (EDOM)",
			err, errname(err), EDOM);
	}
	else
		PATH_TRACE; 
#endif

	if ( nofails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;

}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) mcppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	double dval;
	double myNaN;
	int failtest = 1;
	int err;
	int pathok = 0;

	DBUG_ENTER("ch_t3");

	fdnull = freopen("/dev/null", "w", stderr);
	nofails = 0;

	mlsetglobals();
	errno = 0;
	dval = sqrt(-1.0);
	if ( !isnan(dval) )
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("could not determine NaN");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	myNaN = dval;
	mlsetglobals();
	errno = 0;
	dval = atan2(myNaN,1.0);
	err = errno;
	if (isnan(dval))
		failtest = -1;
	globok = 0; 
	dev_report(failtest, myNaN, dval, EDOM, err);
	PATH_FUNC_TRACE; 

	mlsetglobals();
	failtest = 1;
	errno = 0;
	dval = atan2(1.0,myNaN);
	err = errno;
	if (isnan(dval))
		failtest = -1;
	globok = 0; 
	dev_report(failtest, myNaN, dval, EDOM, err);
	PATH_FUNC_TRACE; 

	mlsetglobals();
	failtest = 1;
	errno = 0;
	dval = atan2(myNaN,myNaN);
	err = errno;
	if (isnan(dval))
		failtest = -1;
	globok = 0; 
	dev_report(failtest, myNaN, dval, EDOM, err);
	PATH_FUNC_TRACE; 

	if ( nofails == 0 )
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;

}
