/*
*      SCCS:  @(#)  ANSI.os/maths/exp/exp.c	    Rel 4.4.3	(07/20/98)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/maths/exp - T.exp Rel 4.4.3 (07/20/98)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.exp

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the exp() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 7 Dec 1989
			Avoid using log(mindble) as it may underflow

		Keith Holder 6 April 90
		Added check in dev_report() so
		that it does not do a comparison of NaN against itself.
		Added test for exp(NaN)

		Peter Lee, Unisoft Ltd., Thu Jul 05 16:21:46 BST 1990
		Added resilience code.

		J.A. Nave	Fri Oct 12 12:18:14 BST 1990
		Modified dev_report() so that errno must be EDOM,
		if running in vsx4 mode

		Stuart Boutell, 25 Apr 1991
			Modified dev_report so that errno may be EDOM

		Geoff Clare, 22 Apr 1992
			ERANGE on overflow mandatory in POSIX & XPG4 modes.

		Geoff Clare, 6 March 1998
			Rename random() to mlrandom().

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<limits.h>
#include	<math.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef exp
#endif


#define NO_TESTS	4		 /* number of tests in testset */
#define	WAITTIME	(40 * SPEEDFACTOR * hwfactor)

extern	void	mlmachar();
extern	void	mlvalrpt();
extern	void	mlrantest();
extern	void	mlreport();
extern	void	mlsetglobals();
extern	int	mlresults();
extern	int	mcppair();

extern double mlrandom();
extern double maxdble;
extern double mindble;
extern double low;
extern double high;
extern double argy;
extern int hwfactor;
extern int iterrs;

int	no_tests = NO_TESTS;

private	FILE	*fdnull;
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	comply_report();
private void	dev_report();

public	void	(*setfunc) () = mlmachar;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private void
comply_report(val, pathok, pathchk)
double val;
int pathok, pathchk;
{
	DBUG_ENTER("comply_report");

	if (!mlresults(val))
	{
		PATH_TRACE;
		PATH_XS_RPT(pathchk + 1);
	}
	else
	{
		xx_rpt(FAILURE);
		mlreport(low, high);
	}

	DBUG_VOID_RETURN;
}


private void
dev_report(expval, obsval, experr, obserr, pathok, pathchk)
double expval, obsval;
int experr, obserr;
int pathok, pathchk;
{
	DBUG_ENTER("dev_report");

	if ( isnan(expval) && isnan(obsval) )
	{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
		if ( obserr != experr && obserr != 0 )
#else
		if ( obserr != experr && obserr != 0 )
#endif
		{
			xx_rpt(FAILURE);
			mlvalrpt(expval, obsval, experr, obserr);
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(pathchk + 1);
		}
	}
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	else if (experr == ERANGE && expval != 0.0 &&
		(expval != obsval || obserr != experr))
	{
		xx_rpt(FAILURE);
		mlvalrpt(expval, obsval, experr, obserr);
	}
#endif
	else if (expval != obsval || (obserr != experr && obserr != 0))
	{
		xx_rpt(FAILURE);
		mlvalrpt(expval, obsval, experr, obserr);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(pathchk + 1);
	}

	DBUG_VOID_RETURN;
}

double t1f(x) double x;
{
return(exp(x)*exp(argy));
}

double t1g(x) double x;
{
return(exp(x+argy));
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) mcppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int 	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();

	for (i=0; i<50; i++)
	{
		high = log(maxdble)/2.0;
		low = log(sqrt(mindble));	/* NOT log(mindble)/2.0 */
		iterrs=40;
		argy=(mlrandom()*(high-low)) + low;
		mlrantest(t1f,t1g,0);
		PATH_TRACE; 
 	}

	comply_report(5.0, pathok, 50);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) mcppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	double dval;

	DBUG_ENTER("ch_t2"); 

	fdnull = freopen("/dev/null", "w", stderr);

	errno = 0;
	dval=exp(log(maxdble)*2.0);
	dev_report(HUGE_VAL, dval, ERANGE, errno, 0, 0);

	DBUG_VOID_RETURN; 
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) mcppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	double dval;

	DBUG_ENTER("ch_t3"); 

	fdnull = freopen("/dev/null", "w", stderr);

	errno = 0;
	dval=exp(log(sqrt(mindble))*4.0);	/* NOT log(mindble)*2.0 */
	dev_report(0.0, dval, ERANGE, errno, 0, 0);

	DBUG_VOID_RETURN; 
}


private void
test4()
{
	DBUG_ENTER("test4");

	(void) mcppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	double dval;
	double myNaN;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");
	
	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	errno = 0;
	dval = sqrt((double)-1.0);
	if ( !isnan(dval) ) {
		xx_rpt(UNSUPPORTED);
		in_rpt("could not determine NaN");
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

	myNaN = dval;
	mlsetglobals();
	errno = 0;
	dval = exp(myNaN);
	dev_report(myNaN, dval, EDOM, errno, pathok, 1);

	DBUG_VOID_RETURN; 
}
