/*
*      SCCS:  @(#)  ANSI.os/maths/floor/floor.c	    Rel 4.4.1	(06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/maths/floor - T.floor Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.floor

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the floor() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		keith Holder 6 April 90
		Added deviance test for floor(NaN)

		Geoff Clare, 27 June 1990
		Change calculation of expected values to be more portable

		Anthony Lee 5 July 1990
		Added resillience code.

		J.A. Nave	Fri Oct 12 12:41:51 BST 1990
		Modified dev_report() so that errno must be EDOM,
		if running in vsx4 mode

		Stuart Boutell, 25 Apr 1991
			Modified dev_report so that errno may be EDOM

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<limits.h>
#include	<math.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef floor
#endif


#define NO_TESTS	2		/* number of tests in testset */
#define	WAITTIME	(10 * SPEEDFACTOR * hwfactor)

extern	void	mlmachar();
extern	void	mlrantest();
extern	void	mlreport();
extern	void	mlvalrpt();
extern	void	mlsetglobals();
extern	int	mlresults();

extern double maxdble;
extern int dsignif;
extern int hwfactor;
extern int expbase;
extern double low;
extern double high;
extern int iterrs;

int	no_tests = NO_TESTS;

private double	maxpowtwo;
private int	testwarn;
private	FILE	*fdnull;
private	void	func_setup();
private void	test1();
private void	test2();
private void	ch_t1();
private void	ch_t2();
private void	comply_report();

public	void	(*setfunc) () = func_setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};


private	void
func_setup()
{
	int i;

	DBUG_ENTER("setup");
	mlmachar();
	maxpowtwo=1.0;
	DBUG_PRINTF("setup", "dsignif=%d", dsignif);
	for (i=1; i<dsignif; i++) 
	{
		maxpowtwo = (expbase*maxpowtwo);
		DBUG_PRINTF("setup", "i=%d, maxpowtwo=%g", i, maxpowtwo);
	}
	DBUG_VOID_RETURN;
}

private void
comply_report(val,pathok,pathchk)
double val;
int pathok, pathchk;
{
	DBUG_ENTER("comply_report");

	if (!mlresults(val))
	{
		PATH_TRACE; 
		PATH_XS_RPT(++pathchk);
	}
	else
	{
		if (testwarn)
			xx_rpt(WARNING);
		else
			xx_rpt(FAILURE);
		mlreport(low, high);
	}

	DBUG_VOID_RETURN;
}

private void
dev_report(failtest, expval, obsval, experr, obserr,pathok,pathchk)
int failtest;
double expval, obsval;
int experr, obserr;
int pathok, pathchk;

{
	DBUG_ENTER("dev_report");

	if (failtest < 0)
	{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
		if (obserr != 0 && obserr != experr)
#else
		if (obserr != 0 && obserr != experr)
#endif
		{
			failtest = 1;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE; 

	if (failtest == 0)
	{
		if (obserr != experr)
		{
			failtest = 1;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE; 

	if (failtest > 0)
	{
		xx_rpt(FAILURE);
		mlvalrpt(expval, obsval, experr, obserr);
	}
	else
		PATH_XS_RPT(pathchk+2); 

	DBUG_VOID_RETURN;
}


double t1f(x) double x;
{
	double frac, powtwo;

	if ((frac = x) < 0.0)
		frac = -frac;

	if (frac > expbase * maxpowtwo)
		return x;

	for (powtwo = maxpowtwo; powtwo >= expbase; powtwo /= expbase)
	{
		while (frac > powtwo)
			frac -= powtwo;
	}
	while (frac >= 1.0)
		frac -= 1.0;

	if (frac == 0.0)
		return x;

	if (x < 0.0)
		return (x + frac - 1.0);
	else
		return (x - frac);
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) mcppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int pathok = 0; 

	DBUG_ENTER("ch_t1");

	mlsetglobals();
	testwarn = 0;

	high = maxdble / expbase;
	low = -high;
	iterrs = 2000;
	mlrantest(t1f,floor,1);
	comply_report(1.0,pathok,0);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) mcppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	double dval;
	double myNaN;
	int failtest = 1;
	int err;
	int pathok = 0; 

	DBUG_ENTER("ch_t2");
	
	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	errno = 0;
	dval = sqrt((double)-1.0);
	if ( !isnan(dval) ) {
		xx_rpt(UNSUPPORTED);
		in_rpt("could not determine NaN");
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE; 

	myNaN = dval;
	mlsetglobals();
	errno = 0;
	dval = floor(myNaN);
	err = errno;
	if (isnan(dval))
		failtest = -1;
	dev_report(failtest, myNaN, dval, EDOM, err,pathok,1);

	DBUG_VOID_RETURN; 
}
