/*
*      SCCS:  @(#)  ANSI.os/maths/fmod/fmod.c	    Rel 4.4.1	(06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/maths/fmod - T.fmod Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.fmod

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fmod() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Keith Holder 6 April 1990
		Added deviance test for fmod
		with either argument set to NaN.

		Anthony Lee 5 July 1990
		Added resillience code.

		J.A. Nave	Fri Oct 12 12:51:46 BST 1990
		Modified dev_report() so that errno must be EDOM,
		if running in vsx4 mode

		Stuart Boutell, 25 Apr 1991
			Modified dev_report so that errno may be EDOM

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<limits.h>
#include	<math.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fmod
#endif


#define NO_TESTS	4		/* number of tests in testset */
#define	WAITTIME	(10 * SPEEDFACTOR * hwfactor)

extern	void	mlmachar();
extern	void	mlrantest();
extern	void	mlreport();
extern	void	mlsetglobals();
extern	int	mlresults();
extern	void	mlvalrpt();

extern double randl();
extern double maxdble;
extern double epspos;
extern int dsignif;
extern int hwfactor;
extern int expbase;
extern double low;
extern double high;
extern double argy;
extern int iterrs;

int	no_tests = NO_TESTS;

private int	testwarn;
private int	nofails;
private FILE	*fdnull;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	comply_report();
private void	dev_report();

public	void	(*setfunc) () = mlmachar;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private void
comply_report(val,pathok,pathchk)
double val;
int pathok, pathchk; 
{
	DBUG_ENTER("comply_report");

	if (!mlresults(val))
	{
		PATH_TRACE; 
		PATH_XS_RPT(++pathchk);
	}
	else
	{
		if (testwarn)
			xx_rpt(WARNING);
		else
			xx_rpt(FAILURE);
		mlreport(low, high);
	}

	DBUG_VOID_RETURN;
}

private void
dev_report(failtest, expval, obsval, experr, obserr)
int failtest;
double expval, obsval;
int experr, obserr;
{
	int pathok = 0; 
	DBUG_ENTER("dev_report");
	globok = 0;

	if (failtest < 0)
	{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
		if (obserr != 0 && obserr != experr)
#else
		if (obserr != 0 && obserr != experr)
#endif
		{
			failtest = 1;
		}
		else
			PATH_TRACE; 
	}
	else
		PATH_TRACE; 

	if (failtest == 0 && failtest != 1)
	{
		if (obserr != experr)
		{
			failtest = 1;
		}
		else
			PATH_TRACE; 
	}
	else
		PATH_TRACE;
		
	if (failtest > 0)
	{
		if ( !nofails )
			xx_rpt(FAILURE);
		mlvalrpt(expval, obsval, experr, obserr);
		nofails++;
	}
	else
		PATH_FUNC_RPT(2); 

	DBUG_VOID_RETURN;
}

double t1f(x) double x;
{
	double absx, absy;
	double dval;

	DBUG_ENTER("t1f");
	if (argy == 0.0)
		DBUG_RETURN(0.0);
	if ((absx = x) < 0.0)
		absx = -absx;
	if ((absy = argy) < 0.0)
		absy = -absy;
	if (absx < absy)
		DBUG_RETURN(x);
	dval = absy;
	DBUG_PRINTF("t1f", "absx=%e, absy=%e", absx, absy);
	for (;;)
	{
		if ((dval * 4294967296.0) < absx)
		{
			dval = dval * 4294967296.0;
			continue;
		}
		if ((dval * 65536.0) < absx)
		{
			dval = dval * 65536.0;
			continue;
		}
		if ((dval * 256.0) < absx)
		{
			dval = dval * 256.0;
			continue;
		}
		if ((dval * 16.0) < absx)
		{
			dval = dval * 16.0;
			continue;
		}
		if ((dval * 4.0) < absx)
		{
			dval = dval * 4.0;
			continue;
		}
		if ((dval * 2.0) < absx)
		{
			dval = dval * 2.0;
			continue;
		}
		absx = absx - dval;
		dval = absy;
		if (absx < absy)
			break;
	}
	DBUG_PRINTF("t1f", "absx=%e, fmod=%e", absx, fmod(x,argy));
	if (x < 0.0)
		DBUG_RETURN(-absx);
	DBUG_RETURN(absx);
}

double t1g(x)
double x;
{
	return(fmod(x,argy));
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) mcppair(ch_t1, NULLFN, 1000 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{

	int pathok = 0;
	int i;
	double absargy;

	DBUG_ENTER("ch_t1");
	mlsetglobals();
	testwarn = 1;

	for (i=0; i<50; i++)
	{
		high = maxdble/4294967296.0;
		low = -high;
		argy=randl(log(high));
		iterrs=40;
		if (argy < 0.0)
			absargy = -argy;
		else
			absargy = argy;
		if (absargy < 1.0)
			high = (maxdble/expbase) * absargy;
		else
			high = maxdble/4294967296.0;
		low = -high;
		mlrantest(t1f,t1g,1);
		PATH_TRACE; 
	}
	comply_report(1.0,pathok,50);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) mcppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int pathok = 0; 
	double dval;

	DBUG_ENTER("ch_t2");
	errno = 0;

	if ((dval = fmod(1.0, 0.0)) == 0.0 || isnan(dval) || errno == EDOM)
	{
		PATH_TRACE; 
		PATH_XS_RPT(1);
	}
	else
	{
		xx_rpt(FAILURE);
		in_rpt("fmod with y=0.0 did not return correct value");
		in_rpt("expected value: 0.0 or NaN, observed value: %g", dval);
		in_rpt("expected errno: %d, observed errno: %d", EDOM, errno);
	}
	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) mcppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int pathok = 0; 
	double dval;

	DBUG_ENTER("ch_t3");
	if ((dval = fmod(4.0, epspos)) == 0.0 || dval == 4.0)
	{
		PATH_TRACE; 
		PATH_XS_RPT(1);
	}
	else
	{
		xx_rpt(FAILURE);
		in_rpt("fmod with large x/y did not return correct value");
		in_rpt("expected value: 0.0 or x(4.0), observed value: %g", dval);
	}
	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	(void) mcppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int pathok = 0; 
	double dval;
	double myNaN;
	int failtest = 1;
	int err;

	DBUG_ENTER("ch_t4");
	
	fdnull = freopen("/dev/null", "w", stderr);
	nofails = 0;

	mlsetglobals();
	errno = 0;
	dval = sqrt((double)-1.0);
	if ( !isnan(dval) ) {
		xx_rpt(UNSUPPORTED);
		in_rpt("could not determine NaN");
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE; 

	myNaN = dval;
	mlsetglobals();
	errno = 0;
	dval = fmod(myNaN,(double)1.0);
	err = errno;
	if (isnan(dval))
		failtest = -1;
	globok = 0;
	dev_report(failtest, myNaN, dval, EDOM, err);
	PATH_FUNC_TRACE; 

	mlsetglobals();
	failtest = 1;
	errno = 0;
	dval = fmod((double)1.0,myNaN);
	err = errno;
	if (isnan(dval))
		failtest = -1;
	globok = 0;
	dev_report(failtest, myNaN, dval, EDOM, err);
	PATH_FUNC_TRACE; 
	
	mlsetglobals();
	errno = 0;
	failtest = 1;
	dval = fmod(myNaN,myNaN);
	err = errno;
	if (isnan(dval))
		failtest = -1;
	globok = 0;
	dev_report(failtest, myNaN, dval, EDOM, err);
	PATH_FUNC_TRACE; 
	
	if ( nofails == 0 )
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN; 
}
