/*
*      SCCS:  @(#)  ANSI.os/maths/frexp/frexp.c	    Rel 4.4.3	(07/20/98)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/maths/frexp - T.frexp Rel 4.4.3 (07/20/98)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.frexp

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the frexp() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Keith Holder 6 April 90
		Added deviance test for frexp(NaN,...)

		Anthony Lee 5 July 1990
		Added resillience code.

		J.A. Nave	Fri Oct 12 13:16:45 BST 1990
		Modified dev_report() so that errno must be EDOM,
		if running in vsx4 mode

		Stuart Boutell, 25 Apr 1991
			Modified dev_report so that errno may be EDOM

		Geoff Clare, 6 March 1998
			Delete declaration of (unused) random() function.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<limits.h>
#include	<math.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef frexp
#endif


#define NO_TESTS	4		 /* number of tests in testset */
#define	WAITTIME	(40 * SPEEDFACTOR * hwfactor)

extern	void	mlmachar();
extern	void	mlvalrpt();
extern	void	mlrantest();
extern	void	mlreport();
extern	void	mlsetglobals();
extern	int	mlresults();
extern	int	mcppair();

extern double maxdble;
extern double mindble;
extern int expbase;
extern int hwfactor;
extern double low;
extern double high;
extern double argy;
extern int iterrs;
extern int exctype;

int	no_tests = NO_TESTS;

private	FILE	*fdnull;
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	comply_report();
private void	dev_report();

public	void	(*setfunc) () = mlmachar;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private void
comply_report(val,pathok,pathchk)
double val;
int pathok, pathchk;
{
	DBUG_ENTER("comply_report");

	if (!mlresults(val))
	{
		PATH_TRACE; 
		PATH_XS_RPT(++pathchk);
	}
	else
	{
		xx_rpt(FAILURE);
		mlreport(low, high);
	}

	DBUG_VOID_RETURN;
}

private void
dev_report(expval, obsval, experr, obserr,pathok,pathchk)
double expval, obsval;
int experr, obserr;
int pathok, pathchk;
{
	DBUG_ENTER("dev_report");

	if ( isnan(expval) && isnan(obsval) )
	{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
		if ( obserr != experr && obserr != 0 )
#else
		if ( obserr != experr && obserr != 0 )
#endif
		{
			xx_rpt(FAILURE);
			mlvalrpt(expval, obsval, experr, obserr);
		}
		else
		{
			PATH_TRACE; 
			PATH_XS_RPT(++pathchk);
		}
	}
	else if (expval != obsval || (obserr != experr && obserr != 0))
	{
		xx_rpt(FAILURE);
		mlvalrpt(expval, obsval, experr, obserr);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(++pathchk);
	}

	DBUG_VOID_RETURN;
}
double t1f(x) double x;
{
	double y;
	double dval;
	int ival;

	y = frexp(x,&ival);
	dval = ival;
	if (dval < 0.0)
		return(y / pow(2.0, -dval));
	else
		return(y * pow(2.0, dval));
}

double t1g(x) double x;
{
return(x);
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) mcppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int pathok = 0; 
	DBUG_ENTER("ch_t1");
	
	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();

	high = 0.5;
	low = -0.5;
	iterrs=1000;
	mlrantest(t1f,t1g,1);
	high = maxdble/expbase;
	low = -maxdble/expbase;
	iterrs=1000;
	mlrantest(t1f,t1g,1);

	comply_report(5.0,pathok,0);

	DBUG_VOID_RETURN; 
}

double t2f(x) double x;
{
	double y, absy;
	int ival;

	y = frexp(x, &ival);

	if (y < 0.0)
		absy = -y;
	else
		absy = y;	

	if (absy >= 1.0)
		return(1.0);

	if (absy < 0.5)
		return(-1.0);

	return(0.1);
}

double t2g(x) double x;
{
#ifdef	lint
	x = x;
#endif	/* lint */
	return(0.1);
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) mcppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int pathok = 0; 
	DBUG_ENTER("ch_t2");
	
	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();

	high = 0.5;
	low = -0.5;
	iterrs=1000;
	mlrantest(t2f,t2g,1);

	high = maxdble/expbase;
	low = -maxdble/expbase;
	iterrs=1000;
	mlrantest(t2f,t2g,1);

	comply_report(1.0,pathok,0);

	DBUG_VOID_RETURN; 
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) mcppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	double	rtval;
	int	e;
	int pathok = 0; 

	DBUG_ENTER("ch_t3");

	if ( (rtval = frexp( (double) 0, &e)) != 0 || e != 0 )
	{
		xx_rpt(FAILURE);
		in_rpt("frexp(0) should have returned 0 and set exp to 0");
		in_rpt("%f was returned and exp was set to %d", rtval, e);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	(void) mcppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	double dval;
	double myNaN;
	int e;
	int pathok = 0; 

	DBUG_ENTER("ch_t4");
	
	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	errno = 0;
	dval = sqrt((double)-1.0);
	if ( !isnan(dval) )
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("could not determine NaN");
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;
		
	myNaN = dval;
	mlsetglobals();
	errno = 0;
	dval = frexp(myNaN,&e);
	dev_report(myNaN, dval, EDOM, errno,pathok,1);

	DBUG_VOID_RETURN; 
}
