/*
*      SCCS:  @(#)  ANSI.os/maths/ldexp/ldexp.c	    Rel 4.4.3	(07/20/98)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/maths/ldexp - T.ldexp Rel 4.4.3 (07/20/98)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.ldexp

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the ldexp() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 22 Nov 1989
			Changed yval in test1 to prevent floating exception
		
		Geoff Clare, 17 Jan 1990
			Changed x range in test1 to prevent possible underflow

		Keith Holder 6 April 1990
			Added test for ldexp(Nan,...)
			Modified dev_report, so that we don't
			compare NaN == NaN.

		Geoff Clare, 9 Apr 1990
			Don't expect ldexp() to return denormalized numbers.
			Remove dependency on pow().

		Anthony Lee, 5 July 1990
			Added resilllience code.

		J.A. Nave	Fri Oct 12 13:34:42 BST 1990
		Modified dev_report() so that errno must be EDOM,
		if running in vsx4 mode

		Stuart Boutell, 25 Apr 1991
			Modified dev_report so that errno may be EDOM

		Stuart Boutell, 25 Apr 1991
			Fixed dev_report path_trace problem

		Geoff Clare, 9 June 1995
			Allow for denormalized numbers in test 4.

		Geoff Clare, 6 March 1998
			Delete declaration of (unused) random() function.
		
************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<limits.h>
#include	<math.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ldexp
#endif


#define NO_TESTS	5		 /* number of tests in testset */
#define	WAITTIME	(40 * SPEEDFACTOR * hwfactor)

extern	void	mlmachar();
extern	void	mlvalrpt();
extern	void	mlrantest();
extern	void	mlreport();
extern	void	mlsetglobals();
extern	int	mlresults();
extern	int	mcppair();

extern double maxdble;
extern double mindble;
extern double low;
extern double high;
extern double argy;
extern int hwfactor;
extern int iterrs;

int	no_tests = NO_TESTS;

private	FILE	*fdnull;
private double	ypow2;
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	comply_report();
private void	dev_report();

private	int	nofails;

public	void	(*setfunc) () = mlmachar;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	NULL, 0
};

private void
comply_report(val,pathok,pathchk)
double val;
int pathok, pathchk; 
{
	DBUG_ENTER("comply_report");

	if (!mlresults(val))
	{
		PATH_XS_RPT(pathchk);
	}
	else
	{
		xx_rpt(FAILURE);
		mlreport(low, high);
	}

	DBUG_VOID_RETURN;
}


private void
dev_report(expval, obsval, experr, obserr)
double expval, obsval;
int experr, obserr;
{
	int	pathok = 0;
	DBUG_ENTER("dev_report");

	if ( isnan(expval) && isnan(obsval) )
	{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
		if ( obserr != experr && obserr != 0 )
#else
		if ( obserr != experr && obserr != 0 )
#endif
		{
			if (!nofails)
				xx_rpt(FAILURE);
			mlvalrpt(expval, obsval, experr, obserr);
			nofails++;
		}
		else
		{
			PATH_TRACE; 
		}
	}
	else
	{
		if (expval != obsval || (obserr != experr && obserr != 0))
		{
			if (!nofails)
				xx_rpt(FAILURE);
			mlvalrpt(expval, obsval, experr, obserr);
			nofails++;
		}
		else
			PATH_TRACE; 
	}
	
	PATH_FUNC_RPT(1);
		
	DBUG_VOID_RETURN;
}

double t1f(x)
double x;
{
	int ival;

	if (argy > 0.0)
		ival = (int) (argy + 0.5);
	else
		ival = (int) (argy - 0.5);
	return(ldexp(x,ival));
}

double t1g(x)
double x;
{
	return (x * ypow2);
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) mcppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int pathok = 0;
	int first_iter = 0;
	double minval;

	DBUG_ENTER("ch_t1");

	/* minimum test value (avoiding denormalized numbers) */
	minval = sqrt(mindble);

	DBUG_PRINTF("ch_t1", "minval = %g", minval);

	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();

	ypow2 = 1.0;	/* ypow2 == 2**argy */
	argy = 0.0;

	while (ypow2 < maxdble/1024.0)
	{
		ypow2 *= 1024.0;
		argy += 10.0;
		high = maxdble/ypow2;
		low = -maxdble/ypow2;
		iterrs=10;
		mlrantest(t1f,t1g,1);
		if (first_iter == 0)
		{
			PATH_TRACE;
		}
		first_iter = 1;
	}

	ypow2 = 1.0;
	argy = 0.0;
	first_iter = 0;

	while (ypow2 > minval*1024.0)
	{
		ypow2 /= 1024.0;
		argy -= 10.0;
		low = minval/ypow2;
		high = maxdble;
		iterrs=10;
		globok = 0;
		mlrantest(t1f,t1g,1);
		if (first_iter == 0)
		{
			PATH_TRACE;
		}
		first_iter = 1;
	}

	ypow2 = 1.0;
	argy = 0.0;
	first_iter = 0;

	while (ypow2 > minval*1024.0)
	{
		ypow2 /= 1024.0;
		argy -= 10.0;
		low = -maxdble;
		high = -minval/ypow2;
		iterrs=10;
		globok = 0;
		mlrantest(t1f,t1g,1);
		if (first_iter == 0)
		{
			PATH_TRACE;
		}
		first_iter = 1;
	}

	comply_report(5.0,pathok,3);

	DBUG_VOID_RETURN;
}

double t2f(x) double x;
{
	double y;
	int ival;

	y = frexp(x, &ival);

	return(ldexp(y, ival));
}

double t2g(x) double x;
{
	return(x);
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) mcppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int pathok = 0; 
	DBUG_ENTER("ch_t2"); 
	
	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();

	high = 0.5;
	low = -0.5;
	iterrs=1000;
	mlrantest(t2f,t2g,1);

	high = maxdble/2.0;
	low = -maxdble/2.0;
	iterrs=1000;
	mlrantest(t2f,t2g,1);

	comply_report(1.0,pathok,0);

	DBUG_VOID_RETURN; 
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) mcppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int pathok = 0;
	double	rtval;

	DBUG_ENTER("ch_t3");

	mlmachar();
	nofails = 0;
	errno = NOERROR;
	rtval = ldexp(maxdble, 1);
	globok = 0;
	dev_report(HUGE_VAL, rtval, ERANGE, errno);
	PATH_FUNC_TRACE;
		
	errno = NOERROR;
	rtval = ldexp(-maxdble, 1);
	globok=0;
	dev_report(-HUGE_VAL, rtval, ERANGE, errno);
	PATH_FUNC_TRACE;

	if ( nofails == 0 )
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}


private void
test4()
{
	DBUG_ENTER("test4");

	(void) mcppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

static double
st(x)
double x;
{
	return(x);
}

static double
mlfabs(x)
double x;
{
	if (x < 0.0)
		return (-x);
	else
		return(x);
}

private void
ch_t4()
{
	double	x, y, rtval;
	double	zero = 0;
	int pathok = 0;
	
	DBUG_ENTER("ch_t4");

	nofails = 0;

	/* Find value of x such that x/2.0 underflows */
	y = mindble;
	while(1)
	{
		x = y;
		y /= 2.0;
		if( ( st(y) == zero) || (mlfabs(y) >= x) )
			break;
	}

	DBUG_PRINTF("test4", "mindble = %.20g, x = %.20g", mindble, x);

	errno = NOERROR;
	rtval = ldexp(x, -1);
	globok = 0;
	dev_report(0.0, rtval, ERANGE, errno);
	PATH_FUNC_TRACE;

	if ( nofails == 0 )
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}


private void
test5()
{
	DBUG_ENTER("test5");

	(void) mcppair(ch_t5, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	double dval;
	double myNaN;
	int pathok = 0; 

	DBUG_ENTER("ch_t5"); 

	nofails = 0;
	
	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	errno = 0;
	dval = sqrt((double)-1.0);
	if ( !isnan(dval) ) {
		xx_rpt(UNSUPPORTED);
		in_rpt("could not determine NaN");
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;
		
	myNaN = dval;
	mlsetglobals();
	errno = 0;
	dval = ldexp(myNaN,1);
	globok = 0;
	dev_report(myNaN, dval, EDOM, errno);
	PATH_FUNC_TRACE;

	if ( nofails == 0 )
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN; 
}
