/*
*      SCCS:  @(#)  ANSI.os/maths/log/log.c	    Rel 4.4.1	(06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/maths/log - T.log Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.log

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the log() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Keith Holder 6 April 1990
		Added deviance test for log(NaN)

		Anthony Lee 6 July 1990
		Added resillience code.

		J.A. Nave	Fri Oct 12 13:48:30 BST 1990
		Modified dev_report() so that errno must be EDOM,
		if running in vsx4 mode

		Stuart Boutell, 25 Apr 1991
			Modified dev_report so that errno may be EDOM
		
		Geoff Clare, 24 Apr 1992
			Align deviance tests with latest XSH4.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<limits.h>
#include	<math.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef log
#endif


#define NO_TESTS	5		 /* number of tests in testset */
#define	WAITTIME	(40 * SPEEDFACTOR * hwfactor)

extern	void	mlmachar();
extern	void	mlvalrpt();
extern	void	mlrantest();
extern	void	mlreport();
extern	void	mlsetglobals();
extern	int	mlresults();
extern	int	mcppair();

extern double randl();
extern double maxdble;
extern double mindble;
extern double low;
extern double high;
extern double argy;
extern int hwfactor;
extern int iterrs;

int	no_tests = NO_TESTS;

private	FILE	*fdnull;
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	comply_report();
private void	dev_report();

public	void	(*setfunc) () = mlmachar;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	NULL, 0
};

private void
comply_report(val,pathok,pathchk)
double val;
int pathok, pathchk;
{
	DBUG_ENTER("comply_report");

	if (!mlresults(val))
	{
		PATH_XS_RPT(pathchk);
	}
	else
	{
		xx_rpt(FAILURE);
		mlreport(low, high);
	}

	DBUG_VOID_RETURN;
}

private void
dev_report(failtest, expval, obsval, experr, obserr,pathok,pathchk)
int failtest;
double expval, obsval;
int experr, obserr;
int pathok, pathchk;
{
	DBUG_ENTER("dev_report");

	if (failtest == -1)
	{
		if (obserr != experr && obserr != 0)
			failtest = 1;
		else
		{
			PATH_TRACE;
		}
	}
	else
		PATH_TRACE;
	
	if (failtest == 0)
	{
		if (obserr != experr)
			failtest = 1;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE; 
			
	if (failtest > 0)
	{
		xx_rpt(FAILURE);
		mlvalrpt(expval, obsval, experr, obserr);
	}
	else
		PATH_XS_RPT(pathchk+2); 

	DBUG_VOID_RETURN;
}

double t1f(x) double x;
{
return(log(x)+log(argy));
}

double t1g(x) double x;
{
return(log(x*argy));
}


private void
test1()
{
	DBUG_ENTER("test1");

	(void) mcppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int  i;
	int pathok = 0; 

	DBUG_ENTER("ch_t1"); 
	
	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();
	for (i=0; i<50; i++)
	{
		high = sqrt(maxdble);
		low = sqrt(mindble);
		if ((argy=randl(log(high))) < low)
		{
			i--;
			continue;
		}
		iterrs=40;
		mlrantest(t1f,t1g,1);
		PATH_TRACE;
	}
	comply_report(5.0,pathok,50);

	DBUG_VOID_RETURN; 
}

double t2f(x) double x;
{
return(exp(log(x)));
}

double t2g(x) double x;
{
return x;
}


private void
test2()
{
	DBUG_ENTER("test2");

	(void) mcppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int pathok = 0; 
	DBUG_ENTER("ch_t2");
	
	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();
	high = sqrt(maxdble);
	low = sqrt(mindble);
	iterrs=2000;
	mlrantest(t2f,t2g,1);
	comply_report(5.0,pathok,0);

	DBUG_VOID_RETURN; 
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) mcppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	double dval;
	int failtest = 1;
	int pathok = 0; 

	DBUG_ENTER("ch_t3"); 

	fdnull = freopen("/dev/null", "w", stderr);

	errno = 0;
	dval=log(-1.0);

	if (isnan(dval))
	{
		PATH_TRACE; 
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
		failtest = 0;
#else
		failtest = -1;
#endif
	}
	else if (dval == -HUGE_VAL)
	{
		PATH_TRACE; 
		failtest = 0;
	}

	dev_report(failtest, -HUGE_VAL, dval, EDOM, errno,pathok,1);

	DBUG_VOID_RETURN; 
}

private void
test4()
{
	DBUG_ENTER("test4");

	(void) mcppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	double dval;
	int failtest = 1;
	int pathok = 0; 

	DBUG_ENTER("ch_t4"); 

	fdnull = freopen("/dev/null", "w", stderr);

	errno = 0;
	dval=log(0.0);

#if !defined(_XOPEN_SOURCE)
	if (isnan(dval) || dval == -HUGE_VAL)
	{
		PATH_TRACE; 
		failtest = -1;
	}
#else
#if TEST_XPG > 3
	if (dval == -HUGE_VAL)
	{
		PATH_TRACE; 
		failtest = -1;
	}
#else
	if (isnan(dval))
	{
		if (errno == EDOM || errno == ERANGE || errno == 0)
		{
			PATH_TRACE; 
			failtest = -2;
		}
	}
	else if (dval == -HUGE_VAL)
	{
		if (errno == EDOM || errno == ERANGE)
		{
			PATH_TRACE; 
			failtest = -2;
		}
	}
#endif
#endif

	dev_report(failtest, -HUGE_VAL, dval, ERANGE, errno,pathok,1);

	DBUG_VOID_RETURN; 
}


private void
test5()
{
	DBUG_ENTER("test5");

	(void) mcppair(ch_t5, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	double dval;
	double myNaN;
	int failtest = 1;
	int err;
	int pathok = 0; 

	DBUG_ENTER("ch_t5"); 

	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	errno = 0;
	dval = sqrt((double)-1.0);
	if ( !isnan(dval) ) {
		xx_rpt(UNSUPPORTED);
		in_rpt("could not determine NaN");
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE; 

	myNaN = dval;
	mlsetglobals();
	errno = 0;
	dval = log(myNaN);
	err = errno;
	if (isnan(dval))
		failtest = -1;

	dev_report(failtest, myNaN, dval, EDOM, err,pathok,1);

	DBUG_VOID_RETURN; 
}
