/*
*      SCCS:  @(#)  ANSI.os/maths/modf/modf.c	    Rel 4.4.3	(07/20/98)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/maths/modf - T.modf Rel 4.4.3 (07/20/98)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.modf

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the modf() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Keith Holder 6 April 1990
		Added deviance test for modf(NaN,..)

		Anthony Lee 6 July 1990
		Added resillience code.

		J.A. Nave	Fri Oct 12 15:52:22 BST 1990
		Modified dev_report() so that errno must be EDOM,
		if running in vsx4 mode

		Stuart Boutell, 25 Apr 1991
			Modified dev_report so that errno may be EDOM

		Geoff Clare, 6 March 1998
			Delete declaration of (unused) random() function.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<limits.h>
#include	<math.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef modf
#endif


#define NO_TESTS	3		 /* number of tests in testset */
#define	WAITTIME	(40 * SPEEDFACTOR * hwfactor)

extern	void	mlmachar();
extern	void	mlvalrpt();
extern	void	mlrantest();
extern	void	mlreport();
extern	void	mlsetglobals();
extern	int	mlresults();
extern	int	mcppair();

extern double maxdble;
extern double mindble;
extern double low;
extern double high;
extern double argy;
extern int hwfactor;
extern int iterrs;

int	no_tests = NO_TESTS;

private	FILE	*fdnull;
private void	test1();
private void	test2();
private void	test3();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	comply_report();
private void	dev_report();

public	void	(*setfunc) () = mlmachar;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private void
comply_report(val)
double val;
{
	int pathok = 0;
	
	DBUG_ENTER("comply_report");

	if (!mlresults(val))
	{
		PATH_TRACE; 
		PATH_XS_RPT(1);
	}
	else
	{
		xx_rpt(FAILURE);
		mlreport(low, high);
	}

	DBUG_VOID_RETURN;
}

private void
dev_report(expval, obsval, experr, obserr,pathok,pathchk)
double expval, obsval;
int experr, obserr;
int pathok, pathchk; 
{
	DBUG_ENTER("dev_report");

	if ( isnan(expval) && isnan(obsval) )
	{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
		if ( obserr != experr && obserr != 0 )
#else
		if ( obserr != experr && obserr != 0 )
#endif
		{
			xx_rpt(FAILURE);
			mlvalrpt(expval, obsval, experr, obserr);
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(++pathchk); 
		}
	}
	else if (expval != obsval || (obserr != experr && obserr != 0))
	{
		xx_rpt(FAILURE);
		mlvalrpt(expval, obsval, experr, obserr);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(++pathchk); 
	}

	DBUG_VOID_RETURN;
}

double t1f(x)
double x;
{
	double ival;

	return(modf(x,&ival));
}

double t1g(x)
double x;
{
	if (x > 0.0)
		return(x - floor(x));
	else
		return(x - ceil(x));
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) mcppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	DBUG_ENTER("ch_t1");
	
	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();

	high = 1000.0;
	low = -1000.0;
	iterrs=1000;
	mlrantest(t1f,t1g,0);

	high = maxdble/2.0;
	low = -maxdble/2.0;
	iterrs=1000;
	mlrantest(t1f,t1g,1);

	comply_report(5.0);

	DBUG_VOID_RETURN; 
}

double t2f(x)
double x;
{
	double ival;

	(void) modf(x, &ival);
	return(ival);
}

double t2g(x)
double x;
{
	if (x > 0.0)
		return(floor(x));
	else
		return(ceil(x));
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) mcppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	DBUG_ENTER("ch_t2");
	
	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();

	high = 1000.0;
	low = -1000.0;
	iterrs=1000;
	mlrantest(t2f,t2g,0);

	high = maxdble/2.0;
	low = -maxdble/2.0;
	iterrs=1000;
	mlrantest(t2f,t2g,1);

	comply_report(1.0);

	DBUG_VOID_RETURN; 
}


private void
test3()
{
	DBUG_ENTER("test3");

	(void) mcppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	double dval;
	double myNaN;
	double ip;
	int pathok = 0;
	
	DBUG_ENTER("ch_t3"); 

	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	errno = 0;
	dval = sqrt((double)-1.0);
	if ( !isnan(dval) )
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("could not determine NaN");
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE; 

	myNaN = dval;
	mlsetglobals();
	errno = 0;
	dval = modf(myNaN,&ip);
	dev_report(myNaN, dval, EDOM, errno,pathok,1);

	DBUG_VOID_RETURN; 
}
