/*
*      SCCS:  @(#)  ANSI.os/maths/pow/pow.c	    Rel 4.4.3	(07/20/98)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/maths/pow - T.pow Rel 4.4.3 (07/20/98)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.pow

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the pow() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 22 Nov 1989
			Changed X and Y ranges in test1
		
		Geoff Clare, 22 Jan 1990
			Changed log(...) to 2.0*log(sqrt(...)) to prevent
			underflow.

		Geoff Clare, 22 Jan 1990
			Changed X range in fractional powers test (the limits
			were calculated to prevent the pow() result from
			overflowing, but the limits themselves could overflow).

		Anthony Lee,6 July 1990
			Added resillience code.

		J.A. Nave	Fri Oct 12 16:05:09 BST 1990
		Modified dev_report() so that errno must be EDOM,
		if running in vsx4 mode

		Stuart Boutell, 25 Apr 1991
			Modified dev_report so that errno may be EDOM

		Geoff Clare, 5 Mar 1992
			Relax accuracy requirement by doubling minimum loss.
			Bias the y-range distribution heavily away from
			extreme values in test 1.

		Neil Moses, 1 May 1992
			Alignment changes to tests 4, 5, 6, 7, 8.

		Geoff Clare, 14 May 1992
			Treat EDOM errors as ERANGE errors.

		Neil Moses, 12 June 1992
			Change to alignment change in test 4.
			Added dev2_report() to handle 2 errnos.
		
		Geoff Clare, 31 July 1992
			Fix dev2_report() usage in test 4.
		
		Geoff Clare, 6 Aug 1992
			Give warning only if pow(0, x) returns +/- inf.
		
		Geoff Clare, 9 Dec 1994
			Return value check in test 5 now XPG3 mode only.

		Geoff Clare, 6 March 1998
			Delete declaration of (unused) random() function.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<limits.h>
#include	<math.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef pow
#endif


#define NO_TESTS	8		 /* number of tests in testset */
#define	WAITTIME	(40 * SPEEDFACTOR * hwfactor)

extern	char	*errname();
extern	void	mlmachar();
extern	void	mlvalrpt();
extern	void	mlrantest();
extern	void	mlreport();
extern	void	mlsetglobals();
extern	int	mlresults();
extern	int	mcppair();

extern double randl();
extern double maxdble;
extern double mindble;
extern int expbase;
extern double low;
extern double high;
extern double argy;
extern int hwfactor;
extern int iterrs;

int	no_tests = NO_TESTS;

private	FILE	*fdnull;
private int	nofails;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	ch_t6();
private void	ch_t7();
private void	ch_t8();
private void	comply_report();
private void	dev_report();
#if TEST_XPG != 3
private void	dev2_report();
#endif

public	void	(*setfunc) () = mlmachar;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	NULL, 0
};

private void
comply_report(val,pathok,pathchk)
double val;
int pathok, pathchk; 
{
	DBUG_ENTER("comply_report");

	if (!mlresults(val))
	{
		PATH_TRACE; 
		PATH_XS_RPT(++pathchk);
	}
	else
	{
		xx_rpt(FAILURE);
		mlreport(low, high);
	}

	DBUG_VOID_RETURN;
}

private void
dev_report(failtest, expval, obsval, experr, obserr)
int failtest;
double expval, obsval;
int experr, obserr;
{
	int pathok = 0; 
	DBUG_ENTER("dev_report");
	globok = 0;

	if (failtest < 0)
	{
		if (obserr != 0 && obserr != experr)
			failtest = 1;
		else
			PATH_TRACE; 
	}
	else
		PATH_TRACE; 

	if (failtest == 0)
	{
		if (obserr != experr)
			failtest = 1;
		else
			PATH_TRACE; 
	}
	else
		if (failtest != 1)
			PATH_TRACE; 

	if (failtest > 0)
	{
		if ( nofails == 0 )
			xx_rpt(FAILURE);
		mlvalrpt(expval, obsval, experr, obserr);
		nofails++;
	}
	else
		PATH_FUNC_RPT(2); 

	DBUG_VOID_RETURN;
}

private double
t1f(x)
double x;
{
        double ret;

        ret = pow(x,argy/2.0);

	/*
	 * Some implementations give EDOM errors, which strictly speaking
	 * they shouldn't for the values used, but since this is pow()
	 * we'll be lenient and make mlrantest() treat them as ERANGE errors.  
	 */
        if (errno == EDOM)
                errno = ERANGE;

        return ret*ret;
}

private double
t1g(x)
double x;
{
        double ret;

        ret = pow(x,argy);
        if (errno == EDOM)
                errno = ERANGE;

        return ret;
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) mcppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int i;
	int failed = 0;
	int pathok = 0; 

	DBUG_ENTER("ch_t1"); 

	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();
	for (i=0; i<50; i++)
	{
		high = maxdble/(2.0*expbase);
		argy=exp(exp(randl(log(log(log(high))))));
		iterrs = 40;
		high=exp(log(maxdble/(2.0*expbase))/argy);
		low=exp(2.0*log(sqrt(mindble*2.0*expbase))/argy);
		mlrantest(t1f,t1g,0);
		PATH_TRACE; 
	}
	if (!mlresults(20.0))
	{
		PATH_TRACE;
	}
	else
	{
		xx_rpt(FAILURE);
		mlreport(low, high);
		failed++;
	}

	mlsetglobals();
	for (i=0; i<50; i++)
	{
		high=maxdble/(2.0*expbase);
		low=mindble*(2.0*expbase);
		argy=1.0/exp(exp(randl(log(log(log(high))))));
		iterrs = 40;
		mlrantest(t1f,t1g,0);
		PATH_TRACE; 
	}
	if (!mlresults(20.0))
	{
		PATH_TRACE;
	}
	else
	{
		if (!failed)
			xx_rpt(FAILURE);
		mlreport(low, high);
		failed++;
	}

	if (!failed)
	{
		PATH_XS_RPT(102);
	}

	DBUG_VOID_RETURN; 
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) mcppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int i;
	int pathok = 0; 

	DBUG_ENTER("ch_t2"); 

	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();
	for (i=0; i<50; i++)
	{
		high = log(maxdble);
		low = 1.0;
		argy=randl(log(high))+low;
		argy = ceil(argy/2.0) * 2.0;
		iterrs = 40;
		high=exp(log(maxdble/2.0)/argy);
		low = -high;
		mlrantest(t1f,t1g,1);
		PATH_TRACE; 
	}
	comply_report(20.0,pathok,50);

	DBUG_VOID_RETURN; 
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) mcppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	double dval;
	int failflag = 0;
	int pathok = 0; 

	DBUG_ENTER("ch_t3"); 

	fdnull = freopen("/dev/null", "w", stderr);

	errno = 0;
	if (((dval=pow(0.0,0.0)) != 1.0) 
		|| errno != 0)
	{
		if (dval == 0.0)
			xx_rpt(WARNING);
		else
			xx_rpt(FAILURE);
		in_rpt("pow(0.0, 0.0) gave");
		mlvalrpt(1.0, dval, 0, errno);
		failflag++;
	}
	else
		PATH_TRACE; 

	if (!failflag)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN; 
}

private void
test4()
{
	DBUG_ENTER("test4");

	(void) mcppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	double dval;
	int pathok = 0; 

	DBUG_ENTER("ch_t4"); 

	fdnull = freopen("/dev/null", "w", stderr);

	nofails = 0;

#if TEST_XPG == 3
	errno = 0;
	if ( (dval=pow(0.0,-1.0)) == -HUGE_VAL
		&&
	     (errno == EDOM || errno == 0) )
	{
		PATH_TRACE; 
	}
	else
	{
		nofails++;
		if (dval == 0.0 && (errno == EDOM || errno == 0))
			xx_rpt(WARNING);
		else
			xx_rpt(FAILURE);
		in_rpt("pow(0.0, -1.0) gave");
		mlvalrpt(-HUGE_VAL, dval, EDOM, errno);
	}
#else
	errno = 0;
	if ( (dval=pow(0.0,-1.0)) == -HUGE_VAL 
	&& (errno == EDOM || errno == ERANGE || errno == 0))
	{
		PATH_TRACE; 
	}
	else if ( dval/2.0 == dval
	&& (errno == EDOM || errno == ERANGE || errno == 0))
	{
		/* NCEG allows return of +/- inf, so give warning for
		   systems which align with this future direction */

		nofails++;
		xx_rpt(WARNING);
		in_rpt("pow(0.0, -1.0) gave");
		mlvalrpt(-HUGE_VAL, dval, 0, 0);
	}
	else
	{
		int failtest;

		if(!nofails++)
			xx_rpt(FAILURE);
		if (dval != -HUGE_VAL)
			failtest = 1;
		else
			failtest = -1;
		in_rpt("pow(0.0, -1.0) gave");
		dev2_report(failtest, -HUGE_VAL, dval, EDOM, ERANGE, errno);
	}

#endif

	if(!nofails)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN; 
}

private void
test5()
{
	DBUG_ENTER("test5");

	(void) mcppair(ch_t5, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	double dval;
	int failflag = 0;
	int pathok = 0;

	DBUG_ENTER("ch_t5"); 

	fdnull = freopen("/dev/null", "w", stderr);

#if TEST_XPG == 3
	errno = 0;
	dval = pow(-1.0,0.1);
	if (isnan(dval))
	{
	     	if(errno == EDOM || errno == 0)
		{
			PATH_TRACE;
		}
		else
		{
			failflag = 1;
		}
	}
	else
	{
		if (dval != 0.0 || errno != EDOM)
			failflag = 1;
		else
		{
			PATH_TRACE;
		}
	}
#else
	errno = 0;
	dval = 0.0;
	(void) pow(-1.0,0.1); /* ISO C and XPG4 do not specify return value */
	if(errno == EDOM)
	{
		PATH_TRACE;
	}
	else
	{
		failflag = 1;
	}
#endif

	if (failflag)
	{
		xx_rpt(FAILURE);
		in_rpt("pow(-1.0, 0.1) gave");
		mlvalrpt(0.0, dval, EDOM, errno);
	}
	else
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN; 
}

private void
test6()
{
	DBUG_ENTER("test6");

	(void) mcppair(ch_t6, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	double dval;
	int failtest = 1;
	int pathok = 0;

	DBUG_ENTER("ch_t6"); 

	fdnull = freopen("/dev/null", "w", stderr);

	nofails = 0;
	errno = 0;
#if TEST_XPG == 3
	dval=pow(sqrt(maxdble),3.0);
	if (isnan(dval))
	{
		PATH_TRACE; 
		failtest = -1;
	}
	else 	if (dval == HUGE_VAL)
	{
		PATH_TRACE; 
		failtest = 0;
	}

	globok = 0; 
	dev_report(failtest, HUGE_VAL, dval, ERANGE, errno);
	PATH_FUNC_TRACE; 
#else
	if((dval=pow(sqrt(maxdble),3.0)) == HUGE_VAL || dval == -HUGE_VAL)
	{
		PATH_TRACE; 
		failtest = 0;
	}

	globok = 0; 
	dev_report(failtest, HUGE_VAL, dval, ERANGE, errno);
	PATH_FUNC_TRACE; 
#endif
		
	if ( nofails == 0 )
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN; 
}

private void
test7()
{
	DBUG_ENTER("test7");

	(void) mcppair(ch_t7, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	double dval;
	int failtest = 1;
	int pathok = 0;

	DBUG_ENTER("ch_t7"); 

	fdnull = freopen("/dev/null", "w", stderr);

	nofails = 0;
	errno = 0;
#if TEST_XPG == 3
	dval=pow(sqrt(mindble),3.0);
	if (isnan(dval))
	{
		PATH_TRACE; 
		failtest = -1;
	}
	else if (dval == 0.0)
	{
		PATH_TRACE; 
		failtest = 0;
	}

	globok = 0; 
	dev_report(failtest, 0.0, dval, ERANGE, errno);
	PATH_FUNC_TRACE; 
#else
	if((dval=pow(sqrt(mindble),3.0)) == 0.0)
	{
		PATH_TRACE; 
		failtest = -1;
	}

	globok = 0; 
	dev_report(failtest, 0.0, dval, ERANGE, errno);
	PATH_FUNC_TRACE; 
#endif
		
	if ( nofails == 0 )
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN; 
}

private void
test8()
{
	DBUG_ENTER("test8");

	(void) mcppair(ch_t8, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	double dval;
	double myNaN;
	int failtest = 1;
	int err;
	int pathok = 0;

	DBUG_ENTER("ch_t8"); 

	fdnull = freopen("/dev/null", "w", stderr);
	nofails = 0;

	mlsetglobals();
	errno = 0;
	dval = sqrt((double)-1.0);
	if ( !isnan(dval) )
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("could not determine NaN");
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE; 

	myNaN = dval;
	mlsetglobals();
	errno = 0;
	dval = pow(myNaN,(double)1.0);
	err = errno;
	if (isnan(dval))
		failtest = -1;
	globok = 0; 
	dev_report(failtest, myNaN, dval, EDOM, err);
	PATH_FUNC_TRACE; 
		
	failtest = 0;
	mlsetglobals();
	errno = 0;
	dval = pow((double)1.0,myNaN);
	err = errno;
	if (isnan(dval))
		failtest = -1;
	globok = 0; 
	dev_report(failtest, myNaN, dval, EDOM, err);
	PATH_FUNC_TRACE; 
		
	failtest = 0;
	mlsetglobals();
	errno = 0;
	dval = pow(myNaN,myNaN);
	err = errno;
	if (isnan(dval))
		failtest = -1;
	globok = 0; 
	dev_report(failtest, myNaN, dval, EDOM, err);
	PATH_FUNC_TRACE; 
		
	failtest = 0;
	mlsetglobals();
	errno = 0;
	dval = pow(myNaN,0.0);
	err = errno;
#if TEST_XPG != 3
	if (isnan(dval) || dval == 1.0)
		failtest = -1;

	globok = 0; 
	dev_report(failtest, myNaN, dval, EDOM, err);
	PATH_FUNC_TRACE; 
#else
	if (isnan(dval))
		failtest = -1;

	globok = 0; 
	dev_report(failtest, myNaN, dval, EDOM, err);
	PATH_FUNC_TRACE; 
#endif
		
	if ( nofails == 0 )
		PATH_XS_RPT(5); 

	DBUG_VOID_RETURN; 
}

#if TEST_XPG != 3
private void
dev2_report(failtest, exrval, orval, exerrno1, exerrno2, oerrno)

	int	failtest;
	double	exrval;		/* expected rtn value */
	double	orval;		/* observed rtn value */
	int	exerrno1;	/* expected errno 1 */
	int	exerrno2;	/* expected errno 2 */
	int	oerrno;		/* observed errno */
{
	char	repval[(4*sizeof(double))+1];
	char	*dblchar;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("dev2_report");

	if (failtest == -1 && oerrno != 0 
	&& oerrno != exerrno1 && oerrno != exerrno2)
		failtest = 1;
	else
		PATH_TRACE;

	if (failtest == 0 && oerrno != exerrno1 && oerrno != exerrno2)
		failtest = 1;
	else
		PATH_TRACE;

	if (failtest > 0)
	{
		xx_rpt(FAILURE);

		if (exrval != orval)
		{
			in_rpt("RETURN VALUES: expected: %g, observed: %g",
				exrval, orval);
			dblchar = (void *) &exrval;
			for (i=0; i<sizeof(double); i++)
			{
				repval[4*i] = '\\';
				repval[(4*i)+1] = '0'+((dblchar[i]&0300)/0100);
				repval[(4*i)+2] = '0'+((dblchar[i]&0070)/0010);
				repval[(4*i)+3] = '0'+(dblchar[i]&0007);
			}
			repval[4*i] = '\0';
			in_rpt("    Bit Representation: expected value: %s", repval);
			dblchar = (void *) &orval;
			for (i=0; i<sizeof(double); i++)
			{
				repval[4*i] = '\\';
				repval[(4*i)+1] = '0'+((dblchar[i]&0300)/0100);
				repval[(4*i)+2] = '0'+((dblchar[i]&0070)/0010);
				repval[(4*i)+3] = '0'+(dblchar[i]&0007);
			}
			repval[4*i] = '\0';
			in_rpt("    Bit Representation: observed value: %s", repval);
		}

		if(oerrno != exerrno1 && oerrno != exerrno2)
		{
			in_rpt(
		"ERRNO VALUES: expected: %d (%s) or %d (%s), observed: %d (%s)",
				exerrno1, errname(exerrno1), 
				exerrno2, errname(exerrno2), 
				oerrno, errname(oerrno));
		}
	}
	else
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
#endif
