/*
*      SCCS:  @(#)  ANSI.os/maths/tan/tan.c 	     Rel 4.4.1	 (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/maths/tan - T.tan Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.tan

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the tan() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Keith Holder 6 April 1990
		Added deviance test for tan(NaN)

		Anthony Lee 9 July 1990
		Added resillence code.

		J.A. Nave	Fri Oct 12 16:34:45 BST 1990
		Modified dev_report() so that errno must be EDOM,
		if running in vsx4 mode

		Stuart Boutell, 25 Apr 1991
			Modified dev_report so that errno may be EDOM

		Geoff Clare, 16 Sept 1991
			Fix division by zero problem in tests 2 and 3.
			Fix maxpowtwo calculation.

		Geoff Clare, 20 Aug 1992
			Execute partial loss test in XPG3 mode only.

		Geoff Clare, 4 March 1994
			Increased mlresults() arg to 15.0 in test 2.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<limits.h>
#include	<math.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef tan
#endif


#ifndef M_PI
#define M_PI		3.14159265358979323846
#endif

#define NO_TESTS	4		 /* number of tests in testset */
#define	WAITTIME	(30 * SPEEDFACTOR * hwfactor)

extern	void	mlmachar();
extern	void	mlvalrpt();
extern	void	mlrantest();
extern	void	mlreport();
extern	void	mlsetglobals();
extern	int	mlresults();
extern	int	mcppair();

extern double low;
extern double high;
extern double quaderr;
extern double errmax;
extern int dsignif;
extern int expbase;
extern int hwfactor;
extern int iterrs;

int	no_tests = NO_TESTS;

private double maxpowtwo;
private	FILE	*fdnull;

private void	func_setup();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	ch_t1();
private void	ch_t2();
#if defined(_XOPEN_SOURCE) && !(TEST_XPG > 3)
private void	ch_t3();
#endif
private void	ch_t4();
private void	dev_report();

public	void	(*setfunc) () = func_setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};


private void
dev_report(failtest, expval, obsval, experr, obserr,pathok,pathchk)
int failtest;
double expval, obsval;
int experr, obserr;
int pathok, pathchk; 
{
	DBUG_ENTER("dev_report");

	if (failtest < 0)
	{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
		if (obserr != 0 && obserr != experr)
#else
		if (obserr != 0 && obserr != experr)
#endif
			failtest = 1;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE; 
	
	if (failtest == 0)
	{
		if (obserr != experr)
			failtest = 1;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE; 

	if (failtest > 0)
	{
		xx_rpt(FAILURE);
		mlvalrpt(expval, obsval, experr, obserr);
	}
	else
		PATH_XS_RPT(pathchk+2); 

	DBUG_VOID_RETURN;
}

private	void
func_setup()
{
	int	i;

	mlmachar();
	maxpowtwo = 0.5;
	for (i=0; i<dsignif; i++)
		maxpowtwo = expbase * maxpowtwo;
	return;
}

double t1f(x) double x;
{
	double y;

	y = tan(x/2.0);
	return((2.0*y)/(1.0 - y*y));
}


private void
test1()
{
	DBUG_ENTER("test1");

	(void) mcppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int failed=0;
	int pathok = 0;

	DBUG_ENTER("ch_t1"); 

	fdnull = freopen("/dev/null", "w", stderr);
	mlsetglobals();
	high = 1.55;
	low = -high;
	iterrs = 2000;
	mlrantest(t1f,tan,0);
	if (mlresults(5.0))
	{
		xx_rpt(FAILURE);
		mlreport(low, high);
		failed++;
	}
	else
		PATH_TRACE;
		
	mlsetglobals();
	high = 100.0;
	low = -100.0;
	iterrs = 2000;
	mlrantest(t1f,tan,0);
	if (mlresults(5.0))
	{
		if (!failed)
			xx_rpt(FAILURE);
		mlreport(low,high);
		failed++;
	}
	else
		PATH_TRACE;
		
	if (!failed)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN; 
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) mcppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	double dval,tval;
	int failtest = 1;
	int err;
	int pathok = 0;

	DBUG_ENTER("ch_t2"); 

	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	errno = 0;
	dval=tan(4.0*M_PI*maxpowtwo);
	err = errno;
	tval=t1f(4.0*M_PI*maxpowtwo);
	if (tval == 0)
		errmax = 0;
	else
		errmax=(dval-tval)/tval;
	quaderr= errmax*errmax;
	if (!mlresults(15.0))
	{
		PATH_TRACE; 
		PATH_XS_RPT(1);
	}
	else
	{
		if (isnan(dval))
		{
			PATH_TRACE; 
			failtest = -1;
		}
		else if (dval == 0.0)
		{
			PATH_TRACE; 
			failtest = 0;
		}
		dev_report(failtest, 0.0, dval, ERANGE, err,pathok,1);
	}

	DBUG_VOID_RETURN; 
}

private void
test3()
{
	DBUG_ENTER("test3");

#if defined(_XOPEN_SOURCE) && !(TEST_XPG > 3)
	(void) mcppair(ch_t3, NULLFN, WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}

#if defined(_XOPEN_SOURCE) && !(TEST_XPG > 3)
private void
ch_t3()
{
	double dval,tval;
	int failtest = 0;
	int err;
	int pathok = 0;

	DBUG_ENTER("ch_t3"); 

	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	errno = 0;
	dval = tan(4.0*M_PI*sqrt(maxpowtwo));
	err = errno;
	tval=t1f(4.0*M_PI*sqrt(maxpowtwo));
	if (tval == 0)
		errmax = 0;
	else
		errmax=(dval-tval)/tval;
	quaderr= errmax*errmax;
	if (!mlresults(5.0))
	{
		PATH_TRACE; 
		PATH_XS_RPT(1);
	}
	else
	{
		if (isnan(dval))
			failtest = -1;
		dev_report(failtest, dval, dval, ERANGE, err,pathok,0);
	}

	DBUG_VOID_RETURN; 
}
#endif

private void
test4()
{
	DBUG_ENTER("test4");

	(void) mcppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	double dval;
	double myNaN;
	int failtest = 1;
	int err;
	int pathok = 0;

	DBUG_ENTER("ch_t4"); 

	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	errno = 0;
	dval = sqrt((double)-1.0);
	if ( !isnan(dval) )
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("could not determine NaN");
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;
		
	myNaN = dval;
	mlsetglobals();
	errno = 0;
	dval = tan(myNaN);
	err = errno;
	if (isnan(dval))
		failtest = -1;
	dev_report(failtest, myNaN, dval, EDOM, err,pathok,1);

	DBUG_VOID_RETURN; 
}
