/*
*      SCCS:  @(#)  ANSI.os/maths/tanh/tanh.c	     Rel 4.4.1	 (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/maths/tanh - T.tanh Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.tanh

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the tanh() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Wed Nov 26 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Keith Holder 6 April 1990
		Added deviance test for tanh(NaN)

		Anthony Lee 8 July 1990
		Added resillience code.

		J.A. Nave	Fri Oct 12 16:40:41 BST 1990
		Modified dev_report() so that errno must be EDOM,
		if running in vsx4 mode

		Stuart Boutell, 25 Apr 1991
			Modified dev_report so that errno may be EDOM

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<limits.h>
#include	<math.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef tanh
#endif


#define NO_TESTS	2		/* number of tests in testset */
#define	WAITTIME	(30 * SPEEDFACTOR * hwfactor)

extern	void	mlmachar();
extern	void	mlrantest();
extern	void	mlreport();
extern	void	mlsetglobals();
extern	void	mlvalrpt();
extern	int	mlresults();

extern double maxdble;
extern double low;
extern double high;
extern int hwfactor;
extern int iterrs;

int	no_tests = NO_TESTS;
private	FILE	*fdnull;

private void	test1();
private void	test2();
private void	ch_t1();
private void	ch_t2();
private	void	comply_report();
private	void	dev_report();

public	void	(*setfunc) () = mlmachar;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private void
comply_report(val)
double val;
{
	int pathok = 0;

	DBUG_ENTER("comply_report");

	if (!mlresults(val))
	{
		PATH_TRACE; 
		PATH_XS_RPT(1);
	}
	else
	{
		xx_rpt(FAILURE);
		mlreport(low, high);
	}

	DBUG_VOID_RETURN;
}

private void
dev_report(expval, obsval, experr, obserr,pathok,pathchk)
double expval, obsval;
int experr, obserr;
int pathok, pathchk; 
{
	DBUG_ENTER("dev_report");

	if ( isnan(expval) && isnan(obsval) )
	{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
		if ( obserr != experr && obserr != 0 )
#else
		if ( obserr != experr && obserr != 0 )
#endif
		{
			xx_rpt(FAILURE);
			mlvalrpt(expval, obsval, experr, obserr);
		}
		else
		{
			PATH_TRACE; 
			PATH_XS_RPT(++pathchk);
		}
	}
	else if (expval != obsval || (obserr != experr && obserr != 0))
	{
		xx_rpt(FAILURE);
		mlvalrpt(expval, obsval, experr, obserr);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(++pathchk);
	}

	DBUG_VOID_RETURN;
}

double t1f(x) double x;
{
	double y;

	y = tanh(x/2.0);
	return((2.0*y)/(1.0 + y*y));
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) mcppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{

	DBUG_ENTER("ch_t1");
	mlsetglobals();
	high = log(maxdble);
	low = -log(maxdble);
	iterrs=2000;
	mlrantest(t1f,tanh,1);
	comply_report(5.0);
	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) mcppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	double dval;
	double myNaN;
	int pathok = 0; 

	DBUG_ENTER("ch_t2");
	
	fdnull = freopen("/dev/null", "w", stderr);

	mlsetglobals();
	errno = 0;
	dval = sqrt((double)-1.0);
	if ( !isnan(dval) )
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("could not determine NaN");
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;
		
	myNaN = dval;
	mlsetglobals();
	errno = 0;
	dval = tanh(myNaN);
	dev_report(myNaN, dval, EDOM, errno,pathok,1);

	DBUG_VOID_RETURN; 
}
