/*
*      SCCS:  @(#)  ANSI.os/signal/signal_X/signal_X.c	Rel 4.4.3     (07/20/98) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/signal/signal_X - T.signal_X Rel 4.4.3 (07/20/98)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.signal_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the signal() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Mon Aug 18 13:44:18 PDT 1986
MODIFICATIONS:	D McCardle Wed Jun 24 10:38:03 WET 1987
			Added vg_sjmp, setjmp(), tn_rpt().
			Removed abandon(), testid().
			Modified xx_rpt(), xs_rpt().
		
		Geoff Clare, 30 June 1989: rewrite EINVAL test.

		Geoff Clare, 28 Sep 1989: add sleep(1) in pa_t4_7()

		Dave Cater, 25 April 1990: Correct test4_7 which could give
		false passes. Also add resilience.

		Stuart Boutell, 6 Nov 1990
			Replaced tet_child references with chld
			Replaced vg_sjmp with internal handler

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called

		Stuart Boutell, 15 Jan 1991
			Inserted test eight.

		Stuart Boutell, 5 Mar 1991
			Removed redundant process sync. call & #define

		Stuart Boutell, 12 Mar 1991
			Modified tests 6,7.

		Geoff Clare, 22 June 1994
			Ensure setjmp() and sigsetjmp() are used in line
			with constraints.

		Geoff Clare, 21 Dec 1995
			Make t7sig_recvd and t8sig_recvd volatile.

		Geoff Clare, 5 June 1997
			Changes for UNIX98 mode.

		Geoff Clare, 9 Feb 1998
			Fix incorrect use of sigcppair() return value.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<sys/types.h>
#include	<sys/wait.h>
#include	<unistd.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<setjmp.h>
#include	<time.h>
#include	<timeout.h>
#include	<wait_child.h>
#include	<string.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef signal
#endif

#if !__STDC__
#define volatile
#endif

#define	NO_TESTS	9		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	EXTERR		"external error"
#define	NOK	002
#define	E_PATH	256
#define	KILLWAIT	(2 * SPEEDFACTOR)
#define	CORE	"core"

#define	T7_RETURN	0
#define	T7_ABORT	1
#define	T7_EXIT		2
#define	T7_LONGJMP	3

private	char *t7test_str[] =
	{	"return",
		"abort()",
		"exit()",
		"longjmp()"	};
		

extern	void	delay();
extern	void	vsx_sigs();
extern	char	*newstr();
extern	char	*itoa();
extern	char	*errname();
extern	char	*signame();
extern	void	setsigs();
extern	char	*sig_child();
extern	int	ex_code;
extern	int	sg_code;
extern	int	wait_child();
extern	int	alrm_flag;

private sigjmp_buf	sjmp;
private	jmp_buf	t7_env;
private	void	sjmpfunc();


int	no_tests = NO_TESTS;

private	int	gotsig;
private	char	*flag4_6[VNUMSIG];
private	int	fail4_6;
private	int	cursig;
private pid_t	chld;
private int	testfail;
private int	setupok;

private	int		t7sig_num;
private	volatile int	t7sig_recvd;
private	int		t7sub_test;
private	int		t8sig_num;
private	volatile int	t8sig_recvd;

private	void	(*tb_signal())();
private int	sigcppair();

private void	prepare_tests();
private void	cln_tests();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4_6();
private	void	test7();
private	void	test8();
private	void	test9();
private	void	ch_t1();
private	void	ch_t2();
private	void	ch_t3();
private	void	ch_t4_6();
private	void	pa_t4_6();
private	void	ch_t7();
private	void	pa_t7();
private	void	ch_t8();
private	void	pa_t8();
private	void	chk_t9();
private	void	onsig();
private	void	test_trap();
private	void	test_sigs();
private	int	waitsig();
private	void	t7sig_catch();
private	void	t8sig_catch();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4_6, 4,
	 test4_6, 5,
	 test4_6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	NULL, 0
};

private	void
prepare_tests()
{
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	globok = 0;

	vsx_sigs();
	
	if (globok == 1)
		setupok = 0;
	else
		setupok = -1;

	PATH_TRACE;	/* just get one trace we were here */

	DBUG_VOID_RETURN;
}

/* ARGSUSED */
private	void
onsig(snum)
int snum;
{
	int	pathok = 0;

	DBUG_ENTER("onsig");

	PATH_TRACE;	/* just get one trace we were here */

	globok = 1;

	DBUG_VOID_RETURN;
}


private int
waitsig(sig, timeout)
	int	*sig;
	int	timeout;
{
	/*
	 * Global path check variable globok is not set in this 
	 * routine but in the signal catching function (if any).
	 */
	time_t	tm_end;
	int	leave = 0;
	int	pathok = 0;

	DBUG_ENTER("waitsig");
	(void) time(&tm_end);
	tm_end += (long) timeout;
	while (!leave)
	{
		if (pathok <= 0)	/* just get one trace we were here */
			PATH_TRACE;

		if (time((time_t *) NULL) > tm_end)
			leave = -1;
		else if (*sig != VNULLSIG)
			leave = 1;
	}
	DBUG_RETURN(leave > 0 ? *sig : SYSERROR);
}

private void
test1()
{
	int	i;
	static	char	*flag[VNUMSIG] = {NULL};
	int	fail = 0;
	int	rval;
	int	pathok = setupok;

	DBUG_ENTER("test1");
	
	testfail = 0;
	for (i = 1 ; i <= VNOJSIG ; i++)
	{
		if (sig_array[i] == VNOSIG)
		{
			fail++;
			flag[i] = "signal not defined";
			continue;
		}
		if (i == VSIGKILL || sig_type[sig_array[i]] != TYPE_ABRT)
		{
			PATH_TRACE;
			continue;
		}
		cursig = i;

		globok = 0;

		rval = sigcppair(ch_t1, NULLFN, WAITTIME, 1, 0);

		if (rval == SYSERROR)
		{
			fail++;
			flag[i] = NULL; /* already reported */
		}
		else if (rval != sig_array[i])
		{
			fail++;
			flag[i] = "wrong signal killed child";
		}
		else
		{
			flag[i] = (char *) NULL;
			if (globok == 1)
				PATH_TRACE;
		}
	}

	if (fail)
	{
		if (!testfail)
			xx_rpt(FAILURE);
		for (i = 1 ; i <= VNOJSIG ; i++)
			if (flag[i] != (char *) NULL)
				in_rpt("%s: %s", sig_name[i], flag[i]);
	}
	else
	{
		if (!testfail)
			PATH_XS_RPT(VNOJSIG);
	}

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	pid_t pid;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	DBUG_PRINTF("test1 child", "sig_array[%d] :%d:", cursig, sig_array[cursig]);
	PATH_TRACE;	/* just get one trace we were here */

	pid = getpid();
	(void) tb_signal(sig_array[cursig], SIG_DFL);
	(void) kill(pid, sig_array[cursig]);
	delay(WAITTIME/2);
	exit(A_KILLFAIL);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	i;
	static	char	*flag[VNUMSIG] = {NULL};
	int	fail = 0;
	int	rval;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	testfail = 0;
	for (i = 1 ; i <= VNOJSIG ; i++)
	{
		if (i == VSIGKILL || i == VSIGCHLD)
			continue;
		if (sig_array[i] == VNOSIG)
		{
			fail++;
			flag[i] = "signal not defined";
			continue;
		}
		cursig = i;

		globok = 0;

		rval = sigcppair(ch_t2, NULLFN, 2+WAITTIME, 0, E_PATH);

		if (rval == SYSERROR)
		{
			fail++;
			flag[i] = NULL; /* already reported */
		}
		else if (rval == OK)
		{
			flag[i] = (char *) NULL;
			if (globok == 1)
				PATH_TRACE;
		}
		else if (rval != E_PATH)
		{
			fail++;
			flag[i] = "wrong exit code from child";
		}
	}

	if (fail)
	{
		if (!testfail)
			xx_rpt(FAILURE);
		for (i = 1 ; i <= VNOJSIG ; i++)
			if (flag[i] != (char *) NULL)
				in_rpt("%s: %s", sig_name[i], flag[i]);
	}
	else
	{
		if (!testfail)
			PATH_XS_RPT(VNOJSIG - 2);
	}

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	pid_t	pid;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");
	PATH_TRACE;	/* just get one trace we were here */

	pid = getpid();
	(void) tb_signal(sig_array[cursig], SIG_IGN);
	/*
	*	gotsig should stay equal to VNULLSIG
	*   since there is no signal catcher, so waitsig()
	*   will (should?) timeout.
	*/
	gotsig = VNULLSIG;
	(void) kill(pid, sig_array[cursig]);
	(void) waitsig(&gotsig, 2+SPEEDFACTOR);
	exit(OK);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	testfail = 0;
	(void) sigcppair(ch_t3, NULLFN, WAITTIME, 0 ,0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int	i;
	static	char	*flag[VNUMSIG] = {NULL};
	char	sbuf[64];
	int	fail = 0;
	void	(*rsig)();
	int	pathok = setupok;

	DBUG_ENTER("ch_t3");

	for (i = 1 ; i <= VNOJSIG ; i++)
	{
		if (i == VSIGKILL || i == VSIGCHLD)
			continue;
		if (sig_array[i] == VNOSIG)
		{
			fail++;
			flag[i] = "signal not defined";
			continue;
		}
		(void) tb_signal(sig_array[i], SIG_IGN);
		rsig = tb_signal(sig_array[i], SIG_DFL);
		if (rsig == SIG_ERR)
		{
			fail++;
			(void) strcat(strcpy(sbuf,
				"signal failed - errno "), itoa(errno));
			if ((flag[i] = newstr(sbuf)) ==
							(char *) NULL)
				exit(A_NOMEM);
			continue;
		}
		else if (rsig != SIG_IGN)
		{
			fail++;
			flag[i] = "previous value not returned by signal";
			continue;
		}
		else
			PATH_TRACE;
		rsig = tb_signal(sig_array[i], onsig);
		if (rsig == SIG_ERR)
		{
			fail++;
			(void) strcat(strcpy(sbuf,
				"signal failed - errno "), itoa(errno));
			if ((flag[i] = newstr(sbuf)) ==
							(char *) NULL)
				exit(A_NOMEM);
			continue;
		}
		else if (rsig != SIG_DFL)
		{
			fail++;
			flag[i] = "previous value not returned by signal";
			continue;
		}
		else
			PATH_TRACE;

		rsig = tb_signal(sig_array[i], SIG_IGN);
		if (rsig == SIG_ERR)
		{
			fail++;
			(void) strcat(strcpy(sbuf,
				"signal failed - errno "), itoa(errno));
			if ((flag[i] = newstr(sbuf)) ==
							(char *) NULL)
				exit(A_NOMEM);
			continue;
		}
		else if (rsig != onsig)
		{
			fail++;
			flag[i] = "previous value not returned by signal";
			continue;
		}
		else
			PATH_TRACE;

		flag[i] = (char *) NULL;
	}
	if (fail)
	{
		xx_rpt(FAILURE);
		for (i = 1 ; i <= VNOJSIG ; i++)
			if (flag[i] != (char *) NULL)
				in_rpt("%s: %s", sig_name[i], flag[i]);
	}
	else
		PATH_XS_RPT(3*(VNOJSIG - 2));

	DBUG_VOID_RETURN;
}

private void
test4_6()
{
	int	i;
	int	rval;
	int	pathok = setupok;

	DBUG_ENTER("test4_6");

	testfail = 0;

	fail4_6 = 0;
	for (i = 0 ; i <= VNOJSIG ; i++)
	{
		flag4_6[i] = (char *) NULL;
	}
	for (i = 1 ; i <= VNOJSIG ; i++)
	{
		cursig = i;
		if (i == VSIGKILL || i == VSIGCHLD)
			continue;
		if (sig_array[i] == VNOSIG)
		{
			fail4_6++;
			flag4_6[i] = "signal not defined";
			continue;
		}
#ifdef VSIGTRAP
		if ((i == VSIGILL || i == VSIGTRAP) && tet_thistest == 6)
#else
		if (i == VSIGILL && tet_thistest == 6)
#endif
		{
			PATH_TRACE;
			continue;
		}
	
		globok = 0;

		rval = sigcppair(ch_t4_6, pa_t4_6, WAITTIME, 0, E_PATH);

		if (rval == SYSERROR)
		{
			fail4_6++;
			flag4_6[i] = NULL; /* already reported */
		}
		else if (rval == OK)
		{
			flag4_6[i] = (char *) NULL;
			if (globok == 1)
				PATH_TRACE;
		}
		else if (rval != E_PATH)
		{
			fail4_6++;
			flag4_6[i] = "wrong exit code from child";
		}
	}
	
	if (fail4_6)
	{
		if (!testfail)
			xx_rpt(FAILURE);
		for (i = 1 ; i <= VNOJSIG ; i++)
			if (flag4_6[i] != (char *) NULL)
				in_rpt("%s: %s", sig_name[i],
					flag4_6[i]);
	}
	else
		if (!testfail)
			PATH_XS_RPT(VNOJSIG - 2);
	
	DBUG_VOID_RETURN;
}


private void
ch_t4_6()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t4_6");
	PATH_TRACE;	/* just get one trace we were here */
	
	if (cursig == VSIGILL)
		(void) tb_signal(sig_array[cursig], test_trap);
	else
		(void) tb_signal(sig_array[cursig], test_sigs);

	globok = 0;
	gotsig = VNULLSIG;
	(void) waitsig(&gotsig, WAITTIME);

	if (tet_thistest == 4 && gotsig != 1)
	{
		if (!testfail)
			xx_rpt(FAILURE);
		in_rpt("signal handler was not entered");
		DBUG_EXIT(NOK);
	}

	/* For each test ensure all checks done */
	if (globok == 1)
		DBUG_EXIT(OK);
	else
		DBUG_EXIT(E_PATH);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t4_6()
{
	int	pathok = 0;

	DBUG_ENTER("pa_t4_6");
	PATH_TRACE;	/* just get one trace we were here */

	(void) sleep(1);	/* allow child to initialise */
	(void) kill(chld, sig_array[cursig]);

	globok = 1;	/* needed to keep sigcppair() happy	*/

	DBUG_VOID_RETURN;
}


private	void
test_trap(sig)
{
	int	pathok = 0;

	globok = 0;

	if (sig != sig_array[VSIGILL])
	{
		if (!testfail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caught by wrong function", sig_name[cursig]);
		exit(NOK);
	}
	else
		PATH_TRACE;

	switch(tet_thistest) {
	case 4:
		gotsig = 1;
		PATH_TRACE;
		break;
	case 5:
		gotsig = sig;
		if (sig != sig_array[cursig])
		{
			if (!testfail)
				xx_rpt(FAILURE);
			in_rpt("received signal %s instead of signal %s", 
						signame(sig), sig_name[cursig]);
			exit(NOK);
		}
		else
			PATH_TRACE;
		break;
	case 6:
		/* This should not happen */
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("SIGILL incorrectly raised within test 6");
		exit(NOK);
		break;
	default:
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Unexpected call to test_trap from test %d",
			tet_thistest);
		exit(NOK);
		break;
	}

	if (pathok == 2)
		globok = 1;
	else
		globok = 0;
	return;
}

private	void
test_sigs(sig)
{
	int	pathok = 0;

	globok = 0;

	if (sig == sig_array[VSIGILL])
	{
		if (!testfail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caught by wrong function", sig_name[cursig]);
		exit(NOK);
	}
	else
		PATH_TRACE;

	switch(tet_thistest) {
	case 4:
		gotsig = 1;
		PATH_TRACE;
		break;
	case 5:
		gotsig = sig;
		if (sig != sig_array[cursig])
		{
			if (!testfail)
				xx_rpt(FAILURE);
			in_rpt("received signal %s instead of signal %s", 
						signame(sig), sig_name[cursig]);
			exit(NOK);
		}
		else
			PATH_TRACE;
		break;
	case 6:
		gotsig = sig;
		if (tb_signal(sig_array[cursig], test_trap) != SIG_DFL)
		{
			sigset_t	tmpset;
			if(sigemptyset(&tmpset) == SYSERROR)
			{
				int	err = errno;
				if (!testfail)
					xx_rpt(DELETION);
				in_rpt("sigemptyset() returned errno %d (%s)",
					err, errname(err));
				exit(NOK);
			}
			else
			if(sigprocmask(SIG_SETMASK, (sigset_t *)NULL, &tmpset)
				== SYSERROR)
			{
				int	err = errno;
				if (!testfail)
					xx_rpt(DELETION);
				in_rpt("sigprocmask() returned errno %d (%s)",
					err, errname(err));
				exit(NOK);
			}
			else
			if( !sigismember(&tmpset, sig_array[cursig] ) )
			{
				if (!testfail)
					xx_rpt(FAILURE);
				in_rpt("In handler, %s not set to SIG_DFL and not blocked",
					sig_name[cursig]);
				exit(NOK);
			}
			else
				PATH_TRACE;
		}
		else
			PATH_TRACE;
		break;
	default:
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Unexpected call to test_sigs from test %d",
			tet_thistest);
		exit(NOK);
		break;
	}

	if (pathok == 2)
		globok = 1;
	else
		globok = 0;
	return;
}

private void
test7()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	for(i = 1; i <= VNOJSIG ; i++)
	{
		if( i == VSIGKILL || i == VSIGCHLD )
		{
			PATH_TRACE;
			PATH_TRACE;
			PATH_TRACE;
			PATH_TRACE;
			PATH_TRACE;
			continue;
		}

		DBUG_PRINTF("test7","Testing signal %s",sig_name[i]);

		t7sig_num = sig_array[i];

		if(t7sig_num == VNOSIG)
		{
			if(!testfail)
				xx_rpt(FAILURE);
			in_rpt("signal %s not defined",sig_name[i]);
			testfail++;
		}
		else
			PATH_TRACE;

		for(t7sub_test = T7_RETURN;
			t7sub_test <= T7_LONGJMP;
			t7sub_test++ )
		{
			int	t_res;
			globok = 0; 
#ifdef	TESTBED2
			t7sub_test = T7_LONGJMP+2;
#endif
			t_res = cpsig(ch_t7, pa_t7, WAITTIME*2, NOK);
			switch(t_res)
			{
				case SYSERROR:
				case NOK:
					testfail++;
					break;
				default:
					PATH_FUNC_TRACE;
					break;
			}
		}
	}


	if(!testfail)
		PATH_XS_RPT(VNOJSIG*5); 

	DBUG_VOID_RETURN;
}

private	void
ch_t7()
{
	DBUG_ENTER("ch_t7");

	(void) tb_signal(t7sig_num, t7sig_catch);

	switch(t7sub_test)
	{
		case T7_RETURN:
#ifdef	TESTBED1
			(void) tb_signal(t7sig_num, SIG_IGN);
#endif
			while (!t7sig_recvd)
				;
			DBUG_EXIT(OK);
			/* NOTREACHED */
			break;
		case T7_ABORT:
		case T7_EXIT:
#ifdef	TESTBED1
			(void) tb_signal(t7sig_num, SIG_IGN);
			t7sig_recvd = t7sig_num;
#endif
			while (!t7sig_recvd)
				;
			if (!testfail++)
				xx_rpt(FAILURE);
			in_rpt("%s : program resumed after signal handler called %s()",
				signame(t7sig_recvd),
				t7test_str[t7sub_test] );
			DBUG_EXIT(NOK);
			/* NOTREACHED */
			break;
		case T7_LONGJMP:
			if (setjmp(t7_env) == 0)
			{
#ifdef	TESTBED1
				(void) tb_signal(t7sig_num, SIG_IGN);
				t7sig_recvd = 1;
#endif
				while (!t7sig_recvd)
					;
				if (!testfail++)
					xx_rpt(FAILURE);
				in_rpt("%s : program resumed after signal handler had called %s",
					signame(t7sig_recvd),
					t7test_str[t7sub_test] );
				DBUG_EXIT(NOK);
			}
			else
			{
				DBUG_EXIT(OK);
			}
			/* NOTREACHED */
			break;
		default:
			if(!testfail++)
				xx_rpt(DELETION);
			in_rpt("%s : unknown subtest mode %d",
					signame(t7sig_num),t7sub_test);
			DBUG_EXIT(NOK);
			/* NOTREACHED */
			break;
		}
	/* NOTREACHED */
}

private void
pa_t7()
{
	int	ret;
	int	pathok = 0;
	DBUG_ENTER("pa_t7");

	(void)	sleep(1);
	(void)	kill(tet_child, t7sig_num);

	switch(ret = wait_child(WAITTIME))
	{
		case	CHLD_ERR:
			if (!testfail++)
				xx_rpt(DELETION);
			in_rpt("waitpid() call errored");
			DBUG_VOID_RETURN;
			/* NOTREACHED */
			break;
		case	CHLD_STOP:
			if (!testfail++)
				xx_rpt(FAILURE);
			in_rpt("Child process is stopped erroneously");
			DBUG_VOID_RETURN;
			/* NOTREACHED */
			break;
		case	CHLD_LIVE:
			if (!testfail++)
				xx_rpt(FAILURE);
			in_rpt("%s : Child process did not receive signal or signal handler not entered when testing %s",
				signame(t7sig_num),
				t7test_str[t7sub_test]);
			if(killchild(tet_child, WAITTIME) != SUCCEED)
				in_rpt("Could not terminate child process with SIGKILL");
			DBUG_VOID_RETURN;
			/* NOTREACHED */
			break;
		case	CHLD_ABRT:
			if(t7sub_test != T7_ABORT)
			{
				if (!testfail++)
					xx_rpt(FAILURE);
				in_rpt("Child process terminated by signal %d (%s) - none expected",
					sg_code, signame(sg_code));
				DBUG_VOID_RETURN;
			}
			else
			{
				if(sg_code != SIGABRT)
				{
					if (!testfail++)
						xx_rpt(FAILURE);
					in_rpt("Child process terminated by signal %d (%s), not %d (SIGABRT)",
						sg_code, signame(sg_code), SIGABRT);
					DBUG_VOID_RETURN;
				}
				else
					PATH_TRACE;
			}
			break;
		case	CHLD_EXIT:
			if (ex_code == NOK)
			{
				testfail++;
			}
			else
				PATH_TRACE;
			break;
		default:
			if (!testfail++)
				xx_rpt(DELETION);
			in_rpt("deletion reason - wait_child returned %d  (Unexpected result)", ret);
			DBUG_VOID_RETURN;
	}
	PATH_FUNC_RPT(1);			
	DBUG_VOID_RETURN;
}

private void
t7sig_catch(snum)
int snum;
{
	t7sig_recvd = snum;
	if(t7sig_recvd != t7sig_num)
	{
		if (!testfail)
			xx_rpt(DELETION);

		in_rpt("signal handler received wrong signal; expected %d (%s), received %d (%s)",
				t8sig_num, signame(t7sig_num),
				snum, signame(snum));
		in_rpt("- subtest %d",t7sub_test);
		exit(NOK);
	}

	switch(t7sub_test)
	{
		case T7_RETURN:
			return;
			/* NOTREACHED */
			break;
		case T7_ABORT:
			(void) tb_signal(SIGABRT, SIG_DFL);
			abort();
			/* NOTREACHED */
			break;
		case T7_EXIT:
			exit(OK);
			/* NOTREACHED */
			break;
		case T7_LONGJMP:
			longjmp(t7_env, t7sig_recvd+1);
			/* NOTREACHED */
			break;
		default:
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("unknown subtest mode %d in t7sig_catch",
				t7sub_test);
			exit(NOK);
	}
	/* NOTREACHED */		
}

private void
test8()
{
	int	t_res;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

	for(i = 1; i <= VNOJSIG ; i++)
	{
		if( i == VSIGFPE || i == VSIGKILL || i == VSIGCHLD)
		{
			PATH_TRACE;
			PATH_TRACE;
			continue;
		}

		DBUG_PRINTF("test8","Testing signal %s",sig_name[i]);

		t8sig_num = sig_array[i];

		if(t8sig_num == VNOSIG)
		{
			if(!testfail)
				xx_rpt(FAILURE);
			in_rpt("signal %s not defined",sig_name[i]);
			testfail++;
		}
		else
			PATH_TRACE;

		globok = 0; 
		t_res = cppair(ch_t8, pa_t8, WAITTIME, NOK);
		switch(t_res)
		{
			case SYSERROR:
				testfail++;
				break;
			case NOK:
				testfail++;
				PATH_FUNC_TRACE;
				break;
			default:
				PATH_FUNC_TRACE;
				break;
		}

	}


	if(!testfail)
		PATH_XS_RPT(2*(i-1)); 

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	static	int	i, j;
	int	ch_fail;

	DBUG_ENTER("ch_t8");

	/* Set the required action if we can */
	(void) tb_signal(t8sig_num, t8sig_catch);

	t8sig_recvd = 0;
	ch_fail = 0;

	i = 0;
	j = 0;

	while(!t8sig_recvd)
		i++, j++;

#ifdef T8TEST
	i++;
	t8sig_recvd++;
#endif

	if(i != j)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("Program did not resume execution at the point it was interrupted.");
		ch_fail++; 
	}

	/* Exit code is signal number we received if any */
	if (t8sig_recvd != t8sig_num)
	{
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason - signal %s not caught", signame(t8sig_num));
		in_rpt("observed signal: %s", signame(t8sig_recvd));
		ch_fail++; 
	}

	DBUG_EXIT(ch_fail?NOK:OK);
}

private void
pa_t8()
{
	int	pathok = 0;
	DBUG_ENTER("pa_t8");

	(void)	sleep(1);
	(void)	kill(tet_child, t8sig_num);
	PATH_TRACE;

	PATH_FUNC_RPT(1); 
	DBUG_VOID_RETURN;
}

private void
t8sig_catch(snum)
int snum;
{
	t8sig_recvd = snum;
	return;
}

private void
test9()
{
	int	pathok = setupok;

	DBUG_ENTER("test9");

	testfail = 0;

	globok = 0;
	chk_t9(sig_array[VSIGINVAL], SIG_DFL, "SIG_DFL");
	if (globok == 1)
		PATH_TRACE;

	globok = 0;
	chk_t9(sig_array[VSIGINVAL], SIG_IGN, "SIG_IGN");
	if (globok == 1)
		PATH_TRACE;

	globok = 0;
	chk_t9(sig_array[VSIGINVAL], onsig, "onsig");
	if (globok == 1)
		PATH_TRACE;

	globok = 0;
	chk_t9(SIGKILL, SIG_IGN, "SIG_IGN");
	if (globok == 1)
		PATH_TRACE;

	globok = 0;
	chk_t9(SIGKILL, onsig, "onsig");
	if (globok == 1)
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);
	
	DBUG_VOID_RETURN;
}

private void
chk_t9(sig, func, strfunc)
int sig;
void (*func)();
char *strfunc;
{
	void	(*rval)();
	int	err;
	int	pathok = 0;

	DBUG_ENTER("chk_t9");

	globok = 0;
	errno = 0;
	rval = tb_signal(sig, func);
	err = errno;
	if (rval != SIG_ERR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("signal(%d, %s) did not return SIG_ERR",
			sig, strfunc);
	}
	else if (err != EINVAL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("signal(%d, %s) set errno to %d (%s), expected EINVAL",
			sig, strfunc, err, errname(err));
	}
	else
		PATH_TRACE;

	if (pathok == 1)
		globok = 1;
	else
		globok = 0;

	DBUG_VOID_RETURN;
}

private int signum;

private int
sigcppair(childproc, parentproc, waittime, exitreason, exitvals)
void (*childproc) ();
void (*parentproc) ();
int	waittime;
int	exitreason;
int	exitvals;
{
	int	rtval;
	int	err;
	int	status;
	int	savchild;
	int	pathok = 0;

	DBUG_ENTER("sigcppair");

	globok = 0;

	(void) fflush(stdout);
	(void) fflush(stderr);

	savchild = chld;

	switch (chld = fork())
	{
	
	case SYSERROR:
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fork child process");
		chld = savchild;
		testfail++;
		DBUG_PRINTF("return", "sigcppair = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);

	case 0:
		tet_setcontext();
		if (DBUG_SETJMP(sjmp) != 0)
		{
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: signal %d received", signum);
			DBUG_EXIT(NOK);
		}
		setsigs(sjmpfunc);
		(*childproc) ();
		DBUG_EXIT(OK);

	default:
		tet_setblock();
		if (parentproc != NULL)
		{	
			globok = 0;
			(*parentproc) ();
			if (globok == 1)
				PATH_TRACE;
		}
		else
			PATH_TRACE;

		SET_TIMEOUT(waittime)
		rtval = waitpid(chld, &status, 0);
		CLEAR_ALARM

		if (rtval == SYSERROR)
		{
			err = errno;
			if (!testfail)
				xx_rpt(DELETION);
			testfail++;
			if (alrm_flag > 0)
				if (exitreason != 2)
					in_rpt("deletion reason: child process timed out");
			else
				in_rpt("deletion reason: waitpid() failed - errno %d (%s)",
					err, errname(err));
			(void) killchild(chld, KILLWAIT);
			chld = savchild;
			DBUG_PRINTF("return", "sigcppair = %d", SYSERROR);
			DBUG_RETURN(SYSERROR);
		}

		if (WIFEXITED(status))
		{
			status = WEXITSTATUS(status);
			if (exitreason != 1 && (status == NOK || status == OK ||
				(status & exitvals)) != 0)
			{
				chld = savchild;
				globok = (pathok == 1) ? 1 : 0;
				DBUG_PRINTF("return", "sigcppair = %d", status);
				DBUG_RETURN(status);
			}
			else
			{
				if (!testfail)
					xx_rpt(DELETION);
				testfail++;
				in_rpt("deletion reason: child process gave unexpected exit code %d",
					status);
			}
			chld = savchild;
			DBUG_PRINTF("return", "sigcppair = %d", SYSERROR);
			DBUG_RETURN(SYSERROR);
		}

		if (WIFSIGNALED(status))
		{
			status = WTERMSIG(status);
			if (exitreason == 1)
			{
				chld = savchild;
				globok = (pathok == 1) ? 1 : 0;
				DBUG_PRINTF("return", "sigcppair = %d", status);
				DBUG_RETURN(status);
			}
			if (!testfail)
				xx_rpt(DELETION);
			testfail++;
			in_rpt("deletion reason: child process was terminated by signal %d (%s)",
				status, signame(status));
		}

		if (WIFSTOPPED(status))
		{
			status = WSTOPSIG(status);
			if (!testfail)
				xx_rpt(DELETION);
			testfail++;
			in_rpt("deletion reason: child process was stopped by signal %d (%s)",
				status, signame(status));
			(void) killchild(chld, KILLWAIT);
		}
		else
		{
			if (!testfail)
				xx_rpt(DELETION);
			testfail++;
			in_rpt("deletion reason: child process returned bad wait status (%#x)",
				(unsigned int)status);
		}

		chld = savchild;
		DBUG_PRINTF("return", "sigcppair = %d", SYSERROR);
		DBUG_RETURN(SYSERROR);
	}
}

private	void
(*tb_signal(sig, func))()
int	sig;
void	(*func)();
{
	void	(*retval)();

	retval = signal(sig, func);
#ifdef	TESTBED
	errno++;
	return SIG_ERR;
#else
	return retval;
#endif
}

private	void
cln_tests()
{
	(void) unlink(CORE);
}

private void
sjmpfunc(sig)
int sig;
{
	signum = sig;
}
