/*
*      SCCS:  @(#)  ANSI.os/streamio/fclose/fclose.c	Rel 4.4.4     (07/19/99) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fclose - T.fclose Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fclose

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fclose() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 4 Nov 1988
		Add tests for errno values

		Geoff Clare, 30 May 1989
		Add EIO test

		Steve Ratcliffe, 03 Apr 1990
		Add ENOSPC test.

		Geoff Clare, 13 June 1990
		Do EAGAIN and EPIPE tests on pipe and FIFO

		Anthony Lee, 12 July 1990
		Added resillience code.

		Stuart Boutell, 5 Nov 1990
		vg_env replaced by environ. <ch_t16>
		Removed reason() references

		Geoff Clare, 19 June 1991
		Execute test 4 in XPG3 mode only.

		Neil Moses, 27 May 1992
		Added new tests for POSIX assertions.

		John-Paul, 24 September 1992
		XNFS enhancements.

		Geoff Clare, 7 Jan 1999
		Pass NULL ferr arg to do_ferr(), so it won't call ferror().

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<sys/stat.h>
#include	<sys/wait.h>
#include	<fcntl.h>
#include	<time.h>
#include	<signal.h>
#include	<timeout.h>
#include	<synch.h>
#include	<chktimes.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"fclose_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fclose
#endif


#define	NO_TESTS	21		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	SYNCSIZE	4		/* number of bytes sent by sendsync */
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define NOK		2
#define E_PATH_CHECK	4
#define	MESSAGE	"abcdefghijklmnopqrstuvwxyz"

extern	void	valrpt();
extern	char	*errname();
extern	char	*execpath();
extern	void	do_feio();
extern	void	(*vsignal())();
#if TEST_XNFS
extern  char    *newstr();
#endif

#if TEST_XNFS
extern  int     vguid1;
extern  int     vggid1;
#endif

int	no_tests = NO_TESTS;

private FILE	*t3_fp;
private	struct {
		int	ret;
		int	err;
	} res;
private	synch	*p_syn;
private	synch	*c_syn;

private	char	*t_string =	"test string";
private	char	*t4_string =	"abcdefghijklmnop";
private	char	*t4_numstring =	"1234567890";
private	char	*t5_string =	"abcdefghijk";
private	char	*t6_string =	"abcdefghijk";
private	char	*t1_file =	"./fclose-t.1";
private	char	*t2_file =	"./fclose-t.2";
private	char	*t3_file =	"./fclose-t.3";
private	char	*t4_file =	"./fclose-t.4";
private	char	*t5_file =	"./fclose-t.5";
private	char	*t6_file =	"./fclose-t.6";
#if !TEST_XNFS
private	char	*t7_file =	"./fclose-t.7";
#endif /* TEST_XNFS */
private	char	*t8_file =	"./fclose-t.8";
private	char	*t9_file =	"./fclose-t.9";
private char	*t18_dir =  	"./fclose-t.18";
private char	*testcomment;
private int	testfail;

private int	test_deleted = 0;
private int	test_fail = 0;

private char	*prog18  =	"fclose_t18";

private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	test6();
private void	test7();
private void	test8();
private void	test9();
private	void	test10_16();
private	void	test17();
private	void	test18();
private	void	test19();
private	void	test20();
private	void	test21();
private	void	ch_t3();
private	void	ch_t5();
private void	ch_t6();

#if TEST_XNFS
private void	ch_t21();
#endif /* TEST_XNFS */

private	void	pa_t3();
private	void	pa_t6();
private	int	t10_16func();

#if !TEST_XNFS
private	int	t17io();
private	void	t17rpt();
#endif /* TEST_XNFS */

private void	ch_t18();
private	void	cln_tests();

private	void	tb_fclose();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10_16, 10,
	 test10_16, 11,
	 test10_16, 12,
	 test10_16, 13,
	 test10_16, 14,
	 test10_16, 15,
	 test10_16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	NULL, 0
};

#if TEST_XNFS
private int	snames[] = { 1, 0 };
#endif


private	void
cln_tests()
{
	(void) unlink("t1_file");
	(void) unlink("t2_file");
	(void) unlink("t3_file");
	(void) unlink("t5_file");
#if TEST_XNFS
	(void) unlink("t7_file");
#endif /* TEST_XNFS */
	(void) unlink("t8_file");
	(void) unlink("t9_file");

	(void) unlink("core");	/* Could be produced by abort() */
}

private void
test1()
{
	FILE		*fp;
	int		err;
	int		fail = 0;
	long		size;
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test1");
	errno = NOERROR;
	if ((fp = fopen(t1_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
			t1_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) fputs(t_string, fp);
	if (stat(t1_file, &statb) == SYSERROR)
	{
		fail = 1;
		err = errno;
	}
	else if (statb.st_size != 0)
	{
		fail = 2;
		size = statb.st_size;
	}
	else
		PATH_TRACE;
		
	(void) fclose(fp);
	if (stat(t1_file, &statb) == SYSERROR)
	{
		fail = 3;
		err = errno;
	}
	else if (statb.st_size != (long) strlen(t_string))
	{
		fail = 4;
		size = statb.st_size;
	}
	else
		PATH_TRACE;
		
	if (fail)
	{
		if (fail < 3)
			xx_rpt(DELETION);
		else
			xx_rpt(FAILURE);
		switch (fail)
		{
		case 1:
			if (err == ENOENT)
				in_rpt("fopen(%s, \"w\") did not create file",
					t1_file);
			else
				in_rpt("stat(%s) after fopen(%s, \"w\") failed - errno %d(%s)",
					t1_file, t1_file, err, errname(err));
			break;

		case 2:
			in_rpt("size of %s after fopen(%s, \"w\"), fputs(\"%s\") is not 0, was %ld",
				t1_file, t1_file, t_string, size);
			break;

		case 3:
			in_rpt("stat(%s) after fclose() failed - errno %d(%s)",
				t1_file, err, errname(err));
			break;

		case 4:
			in_rpt("size of %s after fopen(%s, \"w\"), fputs(\"%s\"), fclose() was %ld instead of %d",
				t1_file, t1_file, t_string,
				size, (int)strlen(t_string));
		}
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(4);
	}

	(void) unlink(t1_file);
	DBUG_VOID_RETURN;
}


private void
test2()
{
	FILE		*fp;
	int		fd;
	int		err;
	int		rval;
	struct stat	statb;
	int 		pathok = 0;
	
	DBUG_ENTER("test2");
	errno = NOERROR;
	if ((fp = fopen(t2_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
			t2_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	fd = fileno(fp);
	(void) fclose(fp);
	errno = NOERROR;
	if ((rval = fstat(fd, &statb)) != SYSERROR || errno != EBADF)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fstat() on file descriptor associated with stream after fclose() did not return expected values");
		valrpt(SYSERROR, rval, EBADF, err);
		(void) tet_delete(3, "depends on success of test 2");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}
	
	(void) unlink(t2_file);
	DBUG_VOID_RETURN;
}

private void
test3()
{
#if TEST_XNFS
	char 	*val;
#endif
	int	err;
	int 	pathok = 0; 

	DBUG_ENTER("test3");

#if TEST_XNFS

	val = tet_getvar("XNFS_REMLOCK_SUPP");

	if( (val == NULL) || (*val == '\0') )
		DBUG_VOID_RETURN;

	if( (*val == 'n') || (*val == 'N') )
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("Parameter XNFS_REMLOCK_SUPP is set to 'N'");
		DBUG_VOID_RETURN;
	}
#endif /* TEST_XNFS */

	errno = NOERROR;
	if ((t3_fp = fopen(t3_file, "w+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w+\") failed - errno %d(%s)",
			t3_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) fputs(t_string, t3_fp);
	if ((c_syn = opensync()) == (synch *) NULL
		|| (p_syn = opensync()) == (synch *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: opensync() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;		

	(void) cppair(ch_t3, pa_t3, WAITTIME, 0);

	(void) closesync(p_syn);
	(void) closesync(c_syn);
	(void) fclose(t3_fp);
	(void) unlink(t3_file);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	struct flock	lck;
	int	fd;
	int 	pathok = 0; 

	DBUG_ENTER("ch_t3");
	
	res.ret = SUCCEED;
	lck.l_type = F_WRLCK;
	lck.l_whence = SEEK_SET;
	lck.l_start = 0L;
	lck.l_len = 0L;
	lck.l_pid = 0;
	fd = fileno(t3_fp);
	if (fcntl(fd, F_SETLK, &lck) == SYSERROR)
	{
		res.ret = FAIL;
		res.err = errno;
	}
	else if (fclose(t3_fp) == EOF)
	{
		res.ret = EOF;
		res.err = errno;
	}
	else
		PATH_TRACE;

	if (res.ret == SUCCEED && pathok != 1)
		res.ret = E_PATH_CHECK;
		
	errno = NOERROR;
	if (sendsync(c_syn[0][1], (char *) &res, sizeof(res), WAITTIME)
							!= sizeof(res))
	{
		vsxerror("sendsync() failed - errno %d(%s)",
			errno, errname(errno));
		DBUG_EXIT(A_SENDSYNC);
	}
	else if (waitsync(p_syn[0][0], (char *) &res, sizeof(res),
					WAITTIME) != sizeof(res))
	{
		vsxerror("waitsync() failed - errno %d(%s)",
			errno, errname(errno));
		DBUG_EXIT(A_WAITSYNC);
	}
	
	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int	fd;
	int	err;
	struct flock	lck;
	int 	pathok = 0; 

	DBUG_ENTER("pa_t3");

	errno = NOERROR;
	fd = fileno(t3_fp);
	lck.l_type = F_RDLCK;
	lck.l_whence = SEEK_SET;
	lck.l_start = 0L;
	lck.l_len = 0L;
	lck.l_pid = 0;
	if (waitsync(c_syn[0][0], (char *) &res, sizeof(res),
					WAITTIME) != sizeof(res))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitsync() failed - errno %d(%s)",
			errno, errname(errno));
      		DBUG_VOID_RETURN;
	}
	if (res.ret == E_PATH_CHECK)
		res.ret = SUCCEED;
	else
		PATH_TRACE; 
	if (res.ret != SUCCEED)
	{
		xx_rpt(DELETION);
		if (res.ret == EOF)
			in_rpt("deletion reason: fclose() failed - errno %d(%s)",
				res.err, errname(res.err));
		else
			in_rpt("deletion reason: fcntl(F_SETLK) failed - errno %d(%s)",
				res.err, errname(res.err));
	}
	else if (fcntl(fd, F_SETLK, &lck) == SYSERROR)
	{
		if (errno == EAGAIN || errno == EACCES)
		{
			xx_rpt(FAILURE);
			in_rpt("file that was write locked was still locked after fclose()");
		}
		else
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: could not read-lock file: errno %d(%s)",
				err, errname(err));
		}
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(2);
	}
	
	
	if (sendsync(p_syn[0][1], (char *) &res, sizeof(res), WAITTIME)
						!= sizeof(res))
	{
		DBUG_VOID_RETURN;
	}

	globok = 1; /* must be set for cppair */

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	fd;
	int	fd2;
	FILE	*fp;
	off_t	offval;
	int 	pathok = 0; 

	DBUG_ENTER("test4");

#if defined(_XOPEN_SOURCE) && !(TEST_XPG > 3)

	if ((fd = open(t4_file, O_RDWR|O_CREAT, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to open file %s", t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fd2 = dup(fd)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to dup() file descriptor");
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fdopen(fd2, "w")) == (FILE *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fdopen file %s", t4_file);
		(void) close(fd);
		(void) close(fd2);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, t4_string, (unsigned)strlen(t4_string)) != strlen(t4_string))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to write %d bytes to %s",
			(int)strlen(t4_string), t4_file);
		(void) close(fd);
		(void) close(fd2);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (lseek(fd, (off_t) 0, SEEK_SET) == (off_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek failed on file %s", t4_file);
		(void) close(fd);
		(void) close(fd2);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (fwrite((void *) t4_numstring, strlen(t4_numstring), 1, fp) != 1)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fwrite %d bytes to %s",
			(int)strlen(t4_numstring), t4_file);
		(void) close(fd);
		(void) close(fd2);
(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (fclose(fp) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fclose file %s", t4_file);
		(void) close(fd);
		(void) close(fd2);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((offval = lseek(fd, (off_t) 0, SEEK_CUR)) != (off_t) 10)
	{
		xx_rpt(FAILURE);
		in_rpt("File pointer not correctly set after fclose()");
		in_rpt("Expected: %ld", (long) 10);
		in_rpt("Observed: %ld", (long) offval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(8);
	}

	(void) close(fd);
	(void) close(fd2);
	(void) unlink(t4_file);

#else
	xx_rpt(NOTINUSE);
	in_rpt("no test in POSIX and XPG4 modes");
#endif

	DBUG_VOID_RETURN;
}

private void
test5()
{
	int		err;
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test5");
	errno = NOERROR;

	err = cppair(ch_t5, NULLFN, WAITTIME, NOK | E_PATH_CHECK);

	DBUG_PRINTF("test5", "exit status of tet_child = %d", err);

	
	if (err == E_PATH_CHECK)
		err = OK;
	else
		PATH_TRACE;
		
	if (err == NOK)
	{
		/* test was deleted in child */
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t5_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(%s) in parent process failed - errno %d(%s)",
			t5_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if (statb.st_size != strlen(t5_string))
	{
		xx_rpt(FAILURE);
		in_rpt("fclose() was not automatically performed when exit was called");
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(3);
	}

	(void) unlink(t5_file);
	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	FILE		*fp;
	int		err;
	long		size;
	struct stat	statb;
	int 		pathok = 0;

	DBUG_ENTER("ch_t5");
	if ((fp = fopen(t5_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") in child process failed - errno %d(%s)",
			t5_file, err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
		
	(void) fputs(t5_string, fp);

	if (stat(t5_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		if (err == ENOENT)
			in_rpt("fopen(%s, \"w\") did not create file", t5_file);
		else
			in_rpt("stat(%s) after fopen(%s, \"w\") in child process failed - errno %d(%s)",
				t5_file, t5_file, err, errname(err));
		DBUG_EXIT(NOK);
	}
	else if (statb.st_size != 0)
	{
		size = statb.st_size;
		xx_rpt(DELETION);
		in_rpt("size of %s after fopen(%s, \"w\"), fputs(\"%s\") was not 0, was %ld",
			t5_file, t5_file, t5_string, size);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	DBUG_EXIT( pathok == 2 ? OK : E_PATH_CHECK);
}

private void
test6()
{
	int 	pathok = 0;
	int	err;
	
	DBUG_ENTER("test6");
	
	errno = NOERROR;
	if ((c_syn = opensync()) == (synch *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: opensync() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;		

	errno = NOERROR;
	(void) cpsig(ch_t6, pa_t6, WAITTIME, 0);

	(void) closesync(c_syn);
	(void) unlink(t6_file);
	DBUG_VOID_RETURN;
}

private void
pa_t6()
{
	int		ret, err, status;
	struct stat	statb;
	int 		pathok = 0;

	DBUG_ENTER("pa_t6");
	
	errno = NOERROR;
	if (waitsync(c_syn[0][0], (char *) &res, sizeof(res),
					WAITTIME) != sizeof(res))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitsync() failed - errno %d(%s)",
			errno, errname(errno));
      		DBUG_VOID_RETURN;
	}
	else
	{
		if (res.ret == E_PATH_CHECK)
			res.ret = OK;
		else
			PATH_TRACE;
	}

			
	SET_TIMEOUT(WAITTIME)
	ret = waitpid(tet_child, &status, 0);
	
	err = errno;

	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: child process timed out");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (ret == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("waitpid(%ld, ...) failed - errno %d (%s)",
			(long)tet_child, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (WIFEXITED(status) && WEXITSTATUS(status) == 1)
	{
		DBUG_VOID_RETURN;	/* deleted in child */
	}
	else
		PATH_TRACE;
			
	if (!(WIFSIGNALED(status) && WTERMSIG(status) == SIGABRT))
	{
		xx_rpt(DELETION);
		in_rpt("child was not terminated by SIGABRT (exit status 0x%x)",
			status);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t6_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(%s) in parent process failed - errno %d(%s)",
			t6_file, err, errname(err));
	}
	else if (statb.st_size != strlen(t6_string))
	{
		xx_rpt(FAILURE);
		in_rpt("fclose() was not automatically performed when abort() was called");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(6);    
	}

	globok = 1;
	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	FILE		*fp;
	int		err;
	long		size;
	struct stat	statb;
	int 		pathok = 0; 
	
	DBUG_ENTER("ch_t6");

	if ((fp = fopen(t6_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") in child process failed - errno %d(%s)",
			t6_file, err, errname(err));
		DBUG_EXIT(1);
	}
	else
		PATH_TRACE;
		
	(void) fputs(t6_string, fp);

	if (stat(t6_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		if (err == ENOENT)
			in_rpt("fopen(%s, \"w\") did not create file", t6_file);
		else
			in_rpt("stat(%s) after fopen(%s, \"w\") in child process failed - errno %d(%s)",
				t6_file, t6_file, err, errname(err));
		DBUG_EXIT(1);
	}
	else if (statb.st_size != 0)
	{
		size = statb.st_size;
		xx_rpt(DELETION);
		in_rpt("size of %s after fopen(%s, \"w\"), fputs(\"%s\") was not 0, was %ld",
			t6_file, t6_file,t6_string, size);
		DBUG_EXIT(1);
	}
	else
		PATH_TRACE;
		
	res.ret = (pathok == 2) ? OK : E_PATH_CHECK;
	if (sendsync(c_syn[0][1], (char *) &res, sizeof(res), WAITTIME)
							!= sizeof(res))
	{
		vsxerror("sendsync() failed - errno %d(%s)",
			errno, errname(errno));
		DBUG_EXIT(A_SENDSYNC);
	}
	
	(void) vsignal(SIGABRT, SIG_DFL);
	abort();

	DBUG_EXIT(0);
}

#if TEST_XNFS

private void
test7()
{
	DBUG_ENTER("test7");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test7()
{
	char	buf[sizeof(MESSAGE)];
	int	rval;
	int	fd_rd,	fd_wr;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;
	if ((rval = mkfifo(t7_file , MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : mkfifo(t7_file) failed");
		valrpt(NOERROR,rval,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd_rd = open(t7_file , O_RDONLY|O_NONBLOCK)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : open(t7_file, O_RDONLY|O_NONBLOCK) failed");
		valrpt(NOERROR,fd_rd,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd_wr = open(t7_file , O_WRONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : open(t7_file, O_WRONLY) failed");
		valrpt(NOERROR,fd_wr,NOERROR,err);
		(void) close(fd_rd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = write(fd_wr , MESSAGE , (unsigned)strlen(MESSAGE))) !=
		strlen(MESSAGE))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() failed");
		valrpt((int)strlen(MESSAGE),rval,NOERROR,err);
		(void) close(fd_rd);
		(void) close(fd_wr);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	tb_fclose(fd_rd, "r");
	PATH_FUNC_TRACE;
	globok = 0;
	tb_fclose(fd_wr, "w");
	PATH_FUNC_TRACE;

	if(testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd_rd = open(t7_file , O_RDONLY|O_NONBLOCK)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : open(t7_file, O_RDONLY|O_NONBLOCK) failed");
		valrpt(NOERROR,fd_rd,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd_wr = open(t7_file , O_WRONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : open(t7_file, O_WRONLY) failed");
		valrpt(NOERROR,fd_wr,NOERROR,err);
		(void) close(fd_rd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(5*WAITTIME)

	errno = NOERROR;
	rval = read(fd_rd , buf , (unsigned)strlen(MESSAGE));

	CLEAR_ALARM

	if (rval != SYSERROR || errno != EAGAIN)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("close failed to discard data in FIFO");
		in_rpt("read on reopened FIFO failed");
		if (rval > 0)
			in_rpt("data read \"%.*s\" ",rval,buf);
		valrpt(SYSERROR,rval,EAGAIN,err);
	}
	else
		PATH_TRACE;

	(void) close(fd_rd);
	(void) close(fd_wr);

	if(!testfail)
		PATH_XS_RPT(10);

	(void)  unlink(t7_file);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test8()
{
	int		fd;
	int		err;
	time_t		curtime;
	struct	stat	statb;
	char		buf[1];
	int		pathok = 0;

	DBUG_ENTER("test8");

	if ((fd = open(t8_file, O_RDWR|O_CREAT, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT) failed - errno %d(%s)",
			t8_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, "AB", 2) != 2)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() failed - errno %d(%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (lseek(fd, (off_t) 0, SEEK_SET) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek to start of %s failed with - error %d(%s)",
			t8_file, err, errname(err));
		(void) close(fd);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(2);

	if (read(fd, buf, 1) != 1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : read() failed - errno %d(%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, "C", 1) != 1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() failed - errno %d(%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testcomment = "close on a file descriptor updates all time-related feilds still marked";
	
#if TEST_XNFS
	if (xnfs_rtime_c(&curtime, SP_T8_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else
	curtime = time( (time_t *) 0);
	PATH_TRACE;
#endif

	globok = 0;
	tb_fclose(fd, "w");
	PATH_FUNC_TRACE;

	(void) sleep(2);

	if (stat(t8_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)", t8_file, err, errname(err));
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statb.st_atime < curtime && statb.st_atime > curtime +2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_atime not updated by fclose(), got %ld but expected between %ld and %ld", (long)statb.st_atime, (long)curtime, (long)curtime +2);
	}
	else
		PATH_TRACE;

	if (statb.st_ctime < curtime && statb.st_ctime > curtime +2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_ctime not updated by fclose(), got %ld but expected between %ld and %ld", (long)statb.st_ctime, (long)curtime, (long)curtime +2);
	}
	else
		PATH_TRACE;

	if (statb.st_mtime < curtime && statb.st_mtime > curtime +2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_mtime not updated by fclose(), got %ld but expected between %ld and %ld", (long)statb.st_mtime, (long)curtime, (long)curtime +2);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(11);

	(void) unlink(t8_file);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	FILE	*fp;
	char	*tres;
	int	err;
	int 	pathok = 0;
	
	DBUG_ENTER("test9");

	if ((fp = fopen(t9_file, "w")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t9_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE; 

	globok = 0;
	savtimes(t9_file);
	PATH_FUNC_TRACE;
	
        if (putc('A', fp) == EOF)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("putc('A', fp) failed on file %s, errno %d(%s)",
			t9_file, err, errname(err));
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fclose(fp) == EOF)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fclose() failed on file %s, errno %d(%s)",
			t9_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((tres = chktimes(t9_file, NEWMTIME | NEWCTIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(tres);
	}
	else
	{
		PATH_FUNC_TRACE;
		
		PATH_XS_RPT(5);
	}

	(void) unlink(t9_file);
	DBUG_VOID_RETURN;
}

/* Lowest test number for which following function is used */
#define TBASE	10

private	void
test10_16()
{
	int experr, err, ret, gotsig;
	static int errnums[] = { EAGAIN, -EAGAIN, EBADF, EINTR, EFBIG,
			      /* test10   test11    test12 test13 test14 */
				 EPIPE, -EPIPE };
			      /* test15 test16 */
	int pathok = 0; 

	DBUG_ENTER("test10_16");

#if TEST_XNFS

	if( tet_thistest != 14 )
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}

#endif /* TEST_XNFS */

	experr = errnums[tet_thistest - TBASE];

	globok = 0; 
	ret = do_ferr(experr, 'f', -1, t10_16func, (int *)NULL, &gotsig);
	DBUG_PRINTF("test10_16","globok is = %d",globok);
	PATH_FUNC_TRACE; 
	err = errno;
	if (experr < 0)
		experr = -experr;

	if (tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (err != experr || ret != EOF || (experr == EPIPE && !gotsig))
	{
		xx_rpt(FAILURE);
		if (experr == EPIPE && !gotsig)
			in_rpt("SIGPIPE was not received");
		if (ret != EOF)
			in_rpt("fclose() returned %d when testing for errno %d(%s), expected EOF",
				ret, experr, errname(experr));
		else if (err != experr)
			in_rpt("fclose() set errno incorrectly: expected %d(%s), actual %d(%s)",
				experr, errname(experr), err, errname(err));
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

/* ARGSUSED */

private int
t10_16func(fp, nbyte)
FILE *fp;
int nbyte;
{
	int ret;
	int pathok = 0; 

	DBUG_ENTER("t10_16func");

	globok = 0; 
	ret = fclose(fp);
	PATH_TRACE;
	PATH_FUNC_RPT(1); 

	DBUG_RETURN(ret);
}

#if TEST_XNFS

private void
test17()
{
	DBUG_ENTER("test17");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test17()
{
	DBUG_ENTER("test17");

	do_feio("f", -1, t17io, t17rpt);

	DBUG_VOID_RETURN;
}

private int
t17io(fp)
FILE *fp;
{
	int ret;
	int pathok = 0; 

	DBUG_ENTER("t17io");

	ret = fclose(fp);
	if (ret == EOF)
		PATH_TRACE;
	PATH_FUNC_RPT(1); 

	DBUG_RETURN(ret);
}

private void
t17rpt(ret, err)
int ret, err;
{
	int 	pathok = 0;
	
	DBUG_ENTER("t17rpt");

	PATH_FUNC_TRACE; 

	if (ret != EOF || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("fclose() from orphaned background process group did not give correct results");
		if (ret != EOF)
			in_rpt("expected return: EOF, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(2);
	}
	
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test18()
{

	DBUG_ENTER("test18");

	(void) cppair(ch_t18, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t18()
{
	int 	err;
	char	*args[6];
	extern	char	*vg_ident;
	extern	char	*vg_dbug;
	extern	char	**environ;

	DBUG_ENTER("ch_t18");

	args[0] = prog18;
	args[1] = t18_dir;
	args[2] = "fclose";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog18), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

private void
test19()
{
	DBUG_ENTER("test19");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else

private void
test19()
{

	int	err;
	int	ret;
	FILE	*fd;
	int	pathok = 0;

	DBUG_ENTER("test19");

	test_deleted = 0;

	if (tet_sync(SP_T19_WRITE_START, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE_START failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	fd = fopen(T19_FILE, "w");
	if(fd == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen() of file '%s' failed. errno = %d",T19_FILE, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Write to file */

	errno = NOERROR;
	ret = fwrite((void *)ONE_BYTE_STR, (size_t)1, (size_t)ONE_BYTE_LENGTH, fd);
	if(ret != ONE_BYTE_LENGTH)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fwrite() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * fclose should flush character to remote file. Server side checks.
	 */

	fclose(fd);

	if (tet_sync(SP_T19_WRITE_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(4);
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test20()
{
	DBUG_ENTER("test20");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test20()
{
	int	err;
	int	ret;
	FILE	*fd;
	int	pathok = 0;

	DBUG_ENTER("test20");

	if (tet_sync(SP_T20_CREATE_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREATE_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = fopen(T20_FILE, "w")) == NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen(\"%s\", \"w\") failed - errno %d(%s)",
				T20_FILE, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Write to file */

	errno = NOERROR;
	ret = fwrite((void *)ONE_BYTE_STR, (size_t)1, (size_t)ONE_BYTE_LENGTH, fd);
	if(ret != ONE_BYTE_LENGTH)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fwrite() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T20_CHMOD_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CHMOD_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	ret = fclose(fd);
	if( (ret != EOF) || (errno != EACCES) )
	{
		err = errno;
		xx_rpt(FAILURE);
		if (ret != EOF)
			in_rpt("fclose() returned %d when testing for errno %d(EACCES), expected EOF", ret, EACCES);
		else if (err != EACCES)
			in_rpt("fclose() set errno incorrectly: expected %d(EACCES), actual %d(%s)", EACCES, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T20_FCLOSE_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to FCLOSE_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	PATH_XS_RPT(6)

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test21()
{
	DBUG_ENTER("test21");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else

private void
test21()
{
	int	err;
	int	ret;
	FILE	*fd;
	int	pathok = 0;

	DBUG_ENTER("test21");

	test_deleted = 0;
	test_fail = 0;

	if (tet_sync(SP_T21_WRITE_START, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE_START failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	fd = fopen(T21_FILE, "w");
	if(fd == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen() of file '%s' failed. errno = %d",T21_FILE, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Write to file */

	errno = NOERROR;
	ret = fwrite((void *)ONE_BYTE_STR, (size_t)1, (size_t)ONE_BYTE_LENGTH, fd);
	if(ret != ONE_BYTE_LENGTH)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fwrite() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	if (tet_sync(SP_T21_UNLINK_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to UNLINK_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	ret = fclose(fd);
	if( (ret != EOF) || (errno != ESTALE) )
	{
		xx_rpt(FAILURE);
		if (ret != EOF)
			in_rpt("fclose() returned %d when testing for errno %d(ESTALE), expected EOF", ret, ESTALE);
		else if (err != ESTALE)
			in_rpt("fclose() set errno incorrectly: expected %d(ESTALE), actual %d(%s)", ESTALE, err, errname(err));
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
tb_fclose(tbfd, mode)
int tbfd;
char *mode;
{
	FILE	*fp;
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("tb_fclose");

	if((fp = fdopen(tbfd, mode)) == (FILE *)NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fdopen(%d, \"%s\") failed - errno %d (%s)", 
			tbfd, mode, err, errname(err));
		(void) close(tbfd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(fclose(fp) == EOF)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fclose() failed - errno %d (%s)", 
			err, errname(err));
		(void) close(tbfd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}
