/*
*      SCCS:  @(#)  ANSI.os/streamio/feof/feof.c	Rel 4.4.1     (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/feof - T.feof Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.feof

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the feof() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	18 Nov 1988	(split out from T.ferror)
MODIFICATIONS:	
		Peter Lee, Unisoft Ltd.,
		Thu Jul 12 12:20:31 BST 1990
		Added resilience code.

		John-Paul, 1 October 1992
		XNFS enhancements
		
************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<unistd.h>
#include 	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef feof
#endif


#define	NO_TESTS	2		/* number of tests in testset */
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();

int	no_tests = NO_TESTS;

private	char	*t_string =	"test string";
private	char	*t_file =	"./feof-t";

private int	setupok;

private	void	comp_prep();
private	void	comp_fin();
private	void	test1();
private	void	test2();

public	void	(*setfunc) () = comp_prep;
public	void	(*clnfunc) () = comp_fin;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private	void
comp_prep()
{
	int		fd;
	int		bad = 0;
	static char	msg[150];
	int	pathok = 0;

	DBUG_ENTER("comp_prep");
	if ((fd = creat(t_file, MODEANY)) == SYSERROR)
	{
		bad = 1;
		(void) sprintf(msg, "creat(%s, %#o) failed in preperations - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
	}
	else if (write(fd, t_string, (unsigned) strlen(t_string)) != strlen(t_string))
	{
		bad = 1;
		(void) sprintf(msg, "write() failed in preperations");
	}
	else
		PATH_TRACE;

	if (fd != SYSERROR)
	{
		(void) close(fd);
		PATH_TRACE;
	}

	if (bad)
		for (fd = 0 ; fd < NO_TESTS ; fd++)
			(void) tet_delete(fd, msg);
	else
		PATH_TRACE;

	if (pathok == 3)
		setupok = 0;
	else
		setupok = -1;

	DBUG_VOID_RETURN;
}

private void
comp_fin()
{
	DBUG_ENTER("comp_fin");
	(void) unlink(t_file);
	DBUG_VOID_RETURN;
}


private	void
test1()
{
	FILE		*fp;
	int		err;
	int		i;
	char		buf[20];
	int		pathok = setupok;

	DBUG_ENTER("test1");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0 ; fgets(buf, (int) sizeof(buf), fp) != (char *) NULL
		&& i < 15 ; i++)
			;
	if (feof(fp) == 0)
	{
		xx_rpt(FAILURE);
		in_rpt("feof() returned zero when EOF had been reached");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private void
test2()
{
	FILE		*fp;
	int		err;
	int		pathok = setupok;

	DBUG_ENTER("test2");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) getc(fp);
	if (feof(fp) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("feof() returned non-zero when EOF had not been reached");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}
