/*
*      SCCS:  @(#)  ANSI.os/streamio/ferror/ferror.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/ferror - T.ferror Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.ferror

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the ferror() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Modified xx_rpt(), xs_rpt()

		Linda Taylor, Hoskyns.	19th August, 1988.
		Tests removed to T.fileno

		Geoff Clare, 18 Nov 1988
		Split out feof and clearerr tests.

		Geoff Clare, 17 Nov 1989
		Use setbuf() instead of setvbuf()

		Geoff Clare, 5 June 1990
		Rewrite test 1.
	
		Peter Lee, Unisoft Ltd.,
		Thu Jul 12 15:03:35 BST 1990
		Added resilience code.

		Andrew Twigger, 5 April 1991
		Removed close() in test 1.

		Geoff Clare, 18 Sep 1991
		Change to input stream and reinstate close() in test 1.

		John-Paul, 5 August 1992
		XNFS enhancements.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ferror
#endif


#define	NO_TESTS	2		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();

int	no_tests = NO_TESTS;

private	char	*t_file =	"./ferror-t";

private	void	cln_tests();
private	void	test1();
private	void	test2();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private	void
cln_tests()
{
	(void) unlink(t_file);
}

private	void
test1()
{
	FILE *fp;
	int err;
	int pathok = 0;

	DBUG_ENTER("test1");

	if ((fp = fopen(t_file, "w+")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(\"%s\", \"w+\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (close(fileno(fp)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("close() failed - errno %d(%s)", err, errname(err));
		(void) unlink(t_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (getc(fp) != EOF)
	{
		xx_rpt(DELETION);
		in_rpt("getc() did not return EOF");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ferror(fp) == 0)
	{
		xx_rpt(FAILURE);
		in_rpt("ferror() returned 0, expected non-zero");
	}
	else
		PATH_XS_RPT(3);

	(void) fclose(fp);
	(void) unlink(t_file);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	FILE		*fp;
	int		err;
	int		fail = 0;
	int	pathok = 0;

	DBUG_ENTER("test2");
	errno = NOERROR;
	if ((fp = fopen(t_file, "a")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"a\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ferror(fp) != 0)
		fail = 1;
	else
		PATH_TRACE;
	(void) putc('a', fp);
	if (ferror(fp) != 0)
		fail |= 2;
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("ferror() returned a non zero value after a file was fopen()ed");
		if (fail & 2)
			in_rpt("ferror() returned a non zero value after a putc() was done on a stream opened for writing");
	}
	else
		PATH_XS_RPT(3);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}
