/*
 *      SCCS:  @(#)  ANSI.os/streamio/fflush/enospc.c	Rel 4.4.4     (07/19/99) 
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1991 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#ifndef NO_MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fflush - enospc Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif
#endif  /* lint	*/

/***********************************************************************

NAME:	enospc.c

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()
		enospc()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char *argv[];

DESCRIPTION:
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:
AUTHOR:		Steve Ratcliffe, UniSoft Ltd.
DATE CREATED:	02 Apr 1990
MODIFICATIONS:

		David Sawyer
		Mon Jul 16 09:54:32 BST 1990
		Added resillience code.

		Stuart Boutell, 5 Nov 1990
			vg_env eliminated

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file
		
		Geoff Clare, 12 July 1994
			Fix path tracing problem.
		
		Geoff Clare, 3 Dec 1998
			Obtain write() ENOSPC error before trying flush.

************************************************************************/

#include <dbug.h>
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <report.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef fflush
#undef fclose
#undef fseek
#endif


#ifndef NO_MAIN
public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

#endif

extern	char	**environ;

extern	char	*itoa();
extern	char	*errname();
extern	void	set_pname();
extern	void	valrpt();

private int 	testfail;

void 	enospc();

extern int 	setupnospc();
extern void	tidynospc();

private int 	interface;

#define I_FFLUSH	0
#define I_FCLOSE	1
#define I_FSEEK 	2

#define MODEANY	(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define MAXTRYS 200	/* Number of times we try to fill up system */

#ifndef NO_MAIN

public	int
tet_main(argc, argv)
int	argc;
char	*argv[];
{
char	*mountdir;

	set_pname(argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		return(OK);
	}

	if (argc >= 5)
		vg_dbug = argv[4];
	if (argc >= 4)
		vg_ident = argv[3];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if (strcmp(argv[2], "fflush") == 0)
		interface = I_FFLUSH;
	else if (strcmp(argv[2], "fclose") == 0)
		interface = I_FCLOSE;
	else if (strcmp(argv[2], "fseek") == 0)
		interface = I_FSEEK;
	else {
		xx_rpt(DELETION);
		in_rpt("Bad interface given");
		return(OK);
	}

	mountdir = argv[1];

	if (setupnospc(mountdir)) {
		testfail++;
		return(OK);
	}

	enospc(mountdir);

	/* Tidy up and umount file system */
	tidynospc(mountdir, 3);

	return(OK);
}
#endif /* !NO_MAIN */

/*
 * enospc() tries to run out of space.
 */
void
enospc(mountdir)
char	*mountdir;
{
	char	*name;
	int 	ret, err;
	int 	i;
	char 	*outstr;
	FILE	*fp;
	int	pathok = 0;

	DBUG_ENTER("enospc");

	name = (char*)malloc(strlen(mountdir)+sizeof("/fill.tmp0000"));
	if (name == NULL) {
		testfail++;
		xx_rpt(DELETION);
		in_rpt("Out of memory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Build up  temp file name and open it
	 */
	(void) strcat(strcpy(name, mountdir), "/fill.tmp1");

	fp = fopen(name, "a+");

	if (fp == NULL) {
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: Open of %s failed - errno %s", name, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	outstr = (char*)malloc(BUFSIZ);
	if (outstr == NULL) {
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: Out of memory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	for (i = 0; i < BUFSIZ; i++)
		outstr[i] = i % 256;

	for (i = 0; i < MAXTRYS; i++)
	{
		errno = 0;
		if (write(fileno(fp), (void*)outstr, BUFSIZ) < 0)
		{
			if (errno == ENOSPC)
			{
				PATH_TRACE;
				break;
			}
			else
			{
				err = errno;
				testfail++;
				xx_rpt(DELETION);
				in_rpt("write() failed - errno %d (%s)",
					err, errname(err));
				DBUG_VOID_RETURN;
			}
		}
	}

	for (i = 0; i < BUFSIZ; i++)
	{
		errno = 0;
		if (write(fileno(fp), (void*)outstr, 1) < 0)
		{
			if (errno == ENOSPC)
			{
				PATH_TRACE;
				break;
			}
			else
			{
				err = errno;
				testfail++;
				xx_rpt(DELETION);
				in_rpt("write() failed - errno %d (%s)",
					err, errname(err));
				DBUG_VOID_RETURN;
			}
		}
	}

	if (i == BUFSIZ)
	{
		testfail++;
		xx_rpt(DELETION);
		in_rpt("write() did not give ENOSPC error (wrote %ld bytes)",
			(MAXTRYS+1L)*BUFSIZ);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fseek(fp, 0L, SEEK_END) != 0)
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: initial fseek(fp, 0, SEEK_END) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* place a character in the buffer */
	if (fputc('a', fp) == EOF)
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("fputc() failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	switch (interface)
	{
	case I_FFLUSH:
		if ((ret = fflush(fp)) != EOF || errno != ENOSPC)
		{
			err = errno;
			testfail++;
			xx_rpt(FAILURE);
			in_rpt("fflush() did not return expected values:");
			valrpt(EOF, ret, ENOSPC, err);
		}
		else
			PATH_TRACE;
		break;
	case I_FCLOSE:
		if ((ret = fclose(fp)) != EOF || errno != ENOSPC)
		{
			err = errno;
			testfail++;
			xx_rpt(FAILURE);
			in_rpt("fclose() did not return expected values:");
			valrpt(EOF, ret, ENOSPC, err);
		}
		else
			PATH_TRACE;
		fp = (FILE*)0;
		break;
	case I_FSEEK:
		if ((ret = fseek(fp, 0L, SEEK_SET)) == 0 || errno != ENOSPC)
		{
			err = errno;
			testfail++;
			xx_rpt(FAILURE);
			if (ret == 0)
				in_rpt("fseek() returned 0, expected non-zero");
			else
				in_rpt("fseek() set errno to %d (%s), expected %d (ENOSPC)",
					err, errname(err), ENOSPC);
		}
		else
			PATH_TRACE;
		break;
	default:
		testfail++;
		xx_rpt(DELETION);
		in_rpt("Bad interface");
		break;
	}

	/*
	 * This may fail with ENOSPC as well.
	 */
	if (fp)
		(void) fclose(fp);

	if (!testfail)
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}
