/*
*      SCCS:  @(#)  ANSI.os/streamio/fflush/fflush.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fflush - T.fflush Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fflush

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fflush() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Phyfos Photi, Hoskyns Group plc
		26/08/88
		test 2 replaced

		Geoff Clare, 4 Nov 1988
		Add tests for errno values

		Geoff Clare, 1 June 1989
		Add EIO test.

		Steve Ratcliffe, 30 Mar 1990
		Add ENOSPC test.

		Geoff Clare, 14 June 1990
		Do EAGAIN and EPIPE tests on pipe and FIFO

		Anthony Lee, 13th July 1990
		Added resilience code.

		Stuart Boutell, 29 October 1990
		New test 4 inserted, tests 4->12 renumbered

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ, reason to tet_reason,
				dltd to tet_reason.
		
		Geoff Clare, 14 May 1992
			Do test 2 in XPG3 mode only.

		John-Paul, 9 October 1992
			XNFS enhancements.

		Geoff Clare, 21 Dec 1994
			Give WARNING only if ferror() returns 0 after error.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<synch.h>
#include	<chktimes.h>
#include	<xnfs.h>
#include	"fflush_h.h"
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fflush
#endif


#define	NO_TESTS	15		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

extern	void	valrpt();
extern	char	*errname();
extern	char	*execpath();
extern	void	do_feio();

int	no_tests = NO_TESTS;

private	char	*t_string =	"test string";
private	char	*t1_file =	"./fflush-t.1";
#if TEST_XPG == 3
private char	*t2_string =	"abcdefghijklmnop";
private char	*t2_numstring =	"1234567890";
private	char	*t2_file =	"./fflush-t.2";
#endif
private	char	*t3_file =	"./fflush-t.3";
private char	*t13_dir = 	"./fflush-d.13";
#if TEST_XNFS
private	char	*t14_file =	T14_FILE;
private	char	*t15_file =	T15_FILE;
#endif /* TEST_XNFS */

private char	*prog13 =	"fflush_t13";
#if TEST_XNFS
private char	*prog14 =	"fflush_t14";
#endif /* TEST_XNFS */

#if TEST_XNFS
extern	char	*itoa();
#endif /* TEST_XNFS */

private	void	test1();
private	void	test2();
private	void	test3();
public	void	test4();
private	void	test5_11();
private	int	t5_11func();
private	void	test12();
#if !TEST_XNFS
private	int	t12io();
private	void	t12rpt();
#endif /* TEST_XNFS */
private	void	test13();
private void	ch_t13();
private	void	test14();
#if TEST_XNFS
private	void	ch_t14();
#endif /* TEST_XNFS */
private	void	test15();
#if TEST_XNFS
private	int	t15_func();
private	int	t15_chk_func();
#endif /* TEST_XNFS */

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5_11, 5,
	 test5_11, 6,
	 test5_11, 7,
	 test5_11, 8,
	 test5_11, 9,
	 test5_11, 10,
	 test5_11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	NULL, 0
};


private void
test1()
{
	FILE		*fp;
	int		err;
	int		fail = 0;
	long		size;
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test1");
	errno = NOERROR;
	if ((fp = fopen(t1_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
			t1_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) fputs(t_string, fp);
	if (stat(t1_file, &statb) == SYSERROR)
	{
		fail = 1;
		err = errno;
	}
	else if (statb.st_size != 0)
	{
		fail = 2;
		size = statb.st_size;
	}
	else
		PATH_TRACE;
		
	(void) fflush(fp);
	if (stat(t1_file, &statb) == SYSERROR)
	{
		fail = 3;
		err = errno;
	}
	else if (statb.st_size != (long) strlen(t_string))
	{
		fail = 4;
		size = statb.st_size;
	}
	else
		PATH_TRACE;
		
	if (fail)
	{
		if (fail < 3)
			xx_rpt(DELETION);
		else
			xx_rpt(FAILURE);
		switch (fail)
		{
		case 1:
			if (err == ENOENT)
				in_rpt("deletion reason: fopen(%s, \"w\") did not creat file", t1_file);
			else
				in_rpt("deletion reason: stat(%s) after fopen(%s, \"w\") failed - errno %d(%s)",
					t1_file, t1_file, err, errname(err));
			break;

		case 2:
			in_rpt("deletion reason: size of %s after fopen(%s, \"w\"), fputs(\"%s\") is not 0, was %ld",
				t1_file, t1_file, t_string, size);
			break;

		case 3:
			in_rpt("stat(%s) after fflush() failed - errno %d(%s)",
				t1_file, err, errname(err));
			break;

		case 4:
			in_rpt("size of %s after fopen(%s, \"w\"), fputs(\"%s\"), fflush() was %ld instead of %d",
				t1_file, t1_file, t_string,
				size, (int)strlen(t_string));
		}
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(4);
	}

	(void) unlink(t1_file);
	DBUG_VOID_RETURN;
}

private void
test2()
{
#if TEST_XPG == 3
	int	fd;
	FILE	*fp;
	off_t	offval;
	int 	pathok = 0;
	
	DBUG_ENTER("test2");

	if ((fd = open(t2_file, O_RDWR|O_CREAT, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to open file %s", t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fdopen(fd, "w")) == (FILE *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fdopen file %s", t2_file);
		(void) close(fd);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, t2_string, (unsigned)strlen(t2_string)) != strlen(t2_string))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to write %d bytes to %s", strlen(t2_string), t2_file);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (lseek(fd, (off_t) 0, SEEK_SET) == (off_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek failed on file %s", t2_file);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (fwrite((void *) t2_numstring, strlen(t2_numstring), 1, fp) != 1)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fwrite %d bytes to %s", strlen(t2_numstring), t2_file);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (fflush(fp) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fflush file %s", t2_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((offval = lseek(fd, (off_t) 0, SEEK_CUR)) != (off_t) 10)
	{
		xx_rpt(FAILURE);
		in_rpt("File pointer not correctly set after fflush()");
		in_rpt("Expected: %ld", (long) 10);
		in_rpt("Observed: %ld", (long) offval);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(7);
	}

	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t2_file);

	DBUG_VOID_RETURN;
#else
	DBUG_ENTER("test2");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
#endif
}

private void
test3()
{
	FILE	*fp;
	char	*res;
	int	err;
	int 	pathok = 0;
	
	DBUG_ENTER("test3");

	if ((fp = fopen(t3_file, "w")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t3_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	savtimes(t3_file);

        if (putc('A', fp) == EOF)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("putc('A', fp) failed on file %s, errno %d(%s)",
			t3_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (fflush(fp) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fflush() failed on file %s, errno %d(%s)",
			t3_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((res = chktimes(t3_file, NEWMTIME | NEWCTIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(res);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(4);
	}

	(void) fclose(fp);
	(void) unlink(t3_file);
	DBUG_VOID_RETURN;
}

/* Lowest test number for which following function is used */
#define TBASE	5

private	void
test5_11()
{
	int experr, err, ret, ferr_ret, gotsig;
	static int errnums[] = { EAGAIN, -EAGAIN, EBADF, EINTR, EFBIG,
			      /* test5   test6    test7  test8  test9 */
				 EPIPE, -EPIPE };
			      /* test10 test11 */
	int 	pathok = 0; 

	DBUG_ENTER("test5_11");

#if TEST_XNFS
	if(tet_thistest != 9)
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}
#endif /* TEST_XNFS */

	experr = errnums[tet_thistest - TBASE];

	globok = 0; 
	ret = do_ferr(experr, 'f', -1, t5_11func, &ferr_ret, &gotsig);
	PATH_FUNC_TRACE; 
	err = errno;
	if (experr < 0)
		experr = -experr;

	if (tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (err != experr || ret != EOF || (experr == EPIPE && !gotsig))
	{
		xx_rpt(FAILURE);
		if (experr == EPIPE && !gotsig)
			in_rpt("SIGPIPE was not received");
		if (ret != EOF)
			in_rpt("fflush() returned %d when testing for errno %d(%s), expected EOF",
				ret, experr, errname(experr));
		else if (err != experr)
			in_rpt("fflush() set errno incorrectly: expected %d(%s), actual %d(%s)",
				experr, errname(experr), err, errname(err));
	}
	else if (!ferr_ret)
	{
		xx_rpt(WARNING);
		in_rpt("fflush() failed to set error indicator when testing for errno %d(%s)",
			experr, errname(experr));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

/* ARGSUSED */

private int
t5_11func(fp, nbyte)
FILE *fp;
int nbyte;
{
	DBUG_ENTER("t5_11func");
	globok = 1;
	DBUG_RETURN(fflush(fp));
}

#if TEST_XNFS

private void
test12()
{
	DBUG_ENTER("test12");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test12()
{
	DBUG_ENTER("test12");

	do_feio("f", -1, t12io, t12rpt);

	DBUG_VOID_RETURN;
}

private int
t12io(fp)
FILE *fp;
{
	int ret;
	int pathok = 0;
	
	DBUG_ENTER("t12io");

	ret = fflush(fp);
	PATH_TRACE;
	PATH_FUNC_RPT(1); 

	DBUG_RETURN(ret);
}

private void
t12rpt(ret, err)
int ret, err;
{
	int pathok = 0;
	
	DBUG_ENTER("t12rpt");

	if (ret != EOF || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("fflush() from orphaned background process group did not give correct results");
		if (ret != EOF)
			in_rpt("expected return: EOF, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test13()
{

	DBUG_ENTER("test13");

	(void) cppair(ch_t13, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t13()
{
	int 	err;
	char	*args[6];

	DBUG_ENTER("ch_t13");

	args[0] = prog13;
	args[1] = t13_dir;
	args[2] = "fflush";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog13), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}


#if !TEST_XNFS

private void
test14()
{
	DBUG_ENTER("test14");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test14()
{
	DBUG_ENTER("test14");

	(void) cppair(ch_t14, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t14()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t14");

	args[0] = prog14;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t14_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog14), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog14), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test15()
{
	DBUG_ENTER("test15");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test15()
{
	DBUG_ENTER("test15");

	t_estale_flush_c(t15_file, 0, t15_func, t15_chk_func, SP_T15_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t15_func(fp)
FILE	*fp;
{
	size_t	nbytes;

	DBUG_ENTER("t15_func");

	nbytes = strlen(t_string);

	DBUG_RETURN(fwrite((void *)t_string, (size_t)1, nbytes, fp)
			!= nbytes ? -1 : 0);
}

private int
t15_chk_func(fp, ebytes)
FILE	*fp;
int	ebytes;
{
	int	rval;
	int	i;
	int	err;
	size_t	nbytes = strlen(t_string);
	int	pathok = 0;

	DBUG_ENTER("t15_chk_func");
	
	for(i = 0; i < ebytes; i++ ) {
		(void) fwrite((void *)t_string, (size_t)1, nbytes, fp);
		errno = NOERROR;
		rval = fflush(fp);
		err = errno;
		if(rval != 0)
			break;
	}

	if(rval != EOF || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(rval != EOF)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value (%d) EOF", EOF);
			in_rpt("Actual   return value %d", rval);
		}
		else
			PATH_TRACE;

		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(2);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
