/*
*      SCCS:  @(#)  ANSI.os/streamio/fflush/fflush4.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fflush4 - T.fflush Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fflush

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fflush() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		J.A. Nave, UniSoft Ltd.
DATE CREATED:	Thu Sep 20 18:22:38 BST 1990
MODIFICATIONS:	
		Stuart Boutell, UniSoft Ltd.
		Integrated into VSX testsuite

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fflush
#endif


#define	NUMFILES	3	/*	number of files used	*/

extern	char	*errname();

private	char	*t_string =	"test string";

struct	fflush_s
{
	char	*fname;
	FILE	*fp;
	int	size;
};

private	struct	fflush_s	t1_fbuff[NUMFILES] =	
{ { "./fflush-t.1", (FILE *)NULL, 0 },
  { "./fflush-t.2", (FILE *)NULL, 0 },
  { "./fflush-t.3", (FILE *)NULL, 0 }
};

public	void
test4()
{
	int		err;
	int		i;
	int		testfail = 0;
	struct stat	statb;
	int		pathok = 0;

	DBUG_ENTER("test4");

#if	!defined(_XOPEN_SOURCE) || TEST_XPG > 3
	for (i=0; i<NUMFILES; i++)
	{
		errno = NOERROR;
		if ((t1_fbuff[i].fp = fopen(t1_fbuff[i].fname, "w")) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
				t1_fbuff[i].fname, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

	}

	errno = NOERROR;
	if (fclose(t1_fbuff[1].fp) == EOF)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: close() on middle file pointer failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<NUMFILES; i+=2)
	{
		(void) fputs(t_string, t1_fbuff[i].fp);
		t1_fbuff[i].size = (int) strlen(t_string);
	}

	for (i=0; i<NUMFILES; i++)
	{

		if (stat(t1_fbuff[i].fname, &statb) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			if (err == ENOENT)
				in_rpt("deletion reason: fopen(%s, \"w\") did not creat file", t1_fbuff[i].fname);
			else
				in_rpt("deletion reason: stat(%s) after fopen(%s, \"w\") failed - errno %d(%s)",
					t1_fbuff[i].fname, t1_fbuff[i].fname, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (statb.st_size != 0)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: size of %s after fopen(%s, \"w\"), fputs(\"%s\") was not 0, was %ld",
				t1_fbuff[i].fname, t1_fbuff[i].fname, t_string,
				(long)statb.st_size);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	errno = 0;

	if (fflush((FILE *) NULL) == EOF)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fflush() failed - errno %d(%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<NUMFILES; i++)
	{

		if (stat(t1_fbuff[i].fname, &statb) == SYSERROR)
		{
			err = errno;
			if (!testfail++)
				xx_rpt(DELETION);
			in_rpt("stat(%s) after fflush() failed - errno %d(%s)",
				t1_fbuff[i].fname, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (statb.st_size != (long) t1_fbuff[i].size)
		{
			if (!testfail++)
				xx_rpt(FAILURE);
			in_rpt("size of %s after fopen(%s, \"w\"), fputs(\"%s\"), fflush() was %ld instead of %d",
				t1_fbuff[i].fname, t1_fbuff[i].fname, t_string,
				(long)statb.st_size, t1_fbuff[i].size);
		}
		else
			PATH_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(5*NUMFILES+2);

	for (i=0; i<NUMFILES; i++)
		(void) unlink(t1_fbuff[i].fname);

#else
	xx_rpt(NOTINUSE);
#endif
	DBUG_VOID_RETURN;
}
