/*
*      SCCS:  @(#)  ANSI.os/streamio/fgets/fgets.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fgets - T.fgets Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fgets

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fgets() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 9 Nov 1988
		Add tests for errno values.

		Geoff Clare, 30 May 1989
		Add EIO tests.

		Geoff Clare, 13 June 1990
		Do EAGAIN test on pipe and FIFO.

		Peter Lee, Unisoft Ltd.,
		Thu Jul 12 15:25:04 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed reason to tet_reason, dltd to tet_reason.

		Neil Moses, 10 June 1992
			New test 12.  On ROF time stamps are not changed.

                John-Paul, 23 July 1992
                	mode variables changed from 'int' to 'mode_t'

		John-Paul, 16 October 1992
			XNFS enhancements

		Geoff Clare, 28 Oct 1993
			Use new file in XNFS st_atime test to avoid
			reading cached data.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<chktimes.h>
#include	<xnfs.h>
#include 	<tet_api.h>
#include	"fgets_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fgets
#endif


#define	NO_TESTS	14		/* number of tests in testset */
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	WAITTIME	(SPEEDFACTOR*5)
#define	T1_NUM	10
#define	T2_NUM	20
#define	T3_OFF	20
#define	T3_NUM	10
#define	T4_NUM	5
#define	T5_NUM	10
#define	T6_NUM	10


extern	char	*errname();
extern	char	*execpath();
extern	void	do_feio();
extern	char	*itoa();
extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

public	int	no_tests = NO_TESTS;

#if TEST_XNFS
private	int	snames[] = {1, 0};
#endif /* TEST_XNFS */

private char	*t_string =	"abcdefghijklmnop\nqrstuvwxyz";
private	char	*t1_string =	"abcdefghi";
private	char	*t2_string =	"abcdefghijklmnop\n";
private	char	*t3_string =	"tuvwxyz";
private	char	*t_file =	"./fgets-t";
private	char	*prog_su =	"fgets_su";
#if TEST_XNFS
private char	*t5_file =	T5_FILE;
#else
#define		t5_file		t_file
#endif
private char	*t12_dir =	"fgets-d.12";
private char	*t12_file =	"fgets-d.12/fgets-t.12";
#if TEST_XNFS
private char	*t13_file =	T13_FILE;
private char	*t14_file =	T14_FILE;
#endif /* TEST_XNFS */
#if !TEST_XNFS
private	char	*sigstat;
#endif /* TEST_XNFS */
private	char	t_buf[30];
private int	setupok;

private	void	comp_prep();
private	void	comp_fin();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	test6_9();
private	void	test10();
private	void	test11();
private	void	test12();
private	void	test13();
private	void	test14();
private	void	ch_t12();
#if !TEST_XNFS
private	int	t6_9func();
private	int	t10_11io();
private	void	t10rpt();
private	void	t11rpt();
#endif /* TEST_XNFS */
#if TEST_XNFS
private	void	ch_t13();
private int	t14_func();
private int	t14_chk_func();
#endif /* TEST_XNFS */

public	void	(*setfunc) () = comp_prep;
public	void	(*clnfunc) () = comp_fin;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6_9, 6,
	 test6_9, 7,
	 test6_9, 8,
	 test6_9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	NULL, 0
};

private void
comp_prep()
{
	int		fd;
	int		i;
	int		bad = 0;
	static char	msg[150];
	int		pathok = 0;

	DBUG_ENTER("comp_prep");

	if ((fd = creat(t_file, MODEANY)) == SYSERROR)
	{
		bad = 1;
		(void) sprintf(msg, "creat(%s, %#o) failed in preperations - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
	}
	else
	{
		PATH_TRACE;
		if (write(fd, t_string, (unsigned) strlen(t_string)) != strlen(t_string))
		{
			bad = 1;
			(void) sprintf(msg, "write() failed in preperations - errno %d(%s)",
				errno, errname(errno));
		}
		else
			PATH_TRACE;
		(void) close(fd);
	}
	if (bad)
		for (i = 1 ; i <= NO_TESTS ; i++)
			(void) tet_delete(i, msg);
	if (pathok == 2)
		setupok = 0;
	else
		setupok = -1;
		
	DBUG_VOID_RETURN;
}

private	void
comp_fin()
{
	(void) unlink(t_file);
}


private void
test1()
{
	FILE		*fp;
	char		*rval;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test1");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t_buf[0] = NUL;
	rval = fgets(t_buf, T1_NUM, fp);
	t_buf[sizeof(t_buf) - 1] = NUL;
	if (rval != t_buf || strcmp(t_buf, t1_string))
	{
		xx_rpt(FAILURE);
		if (strcmp(t_buf, t1_string))
		{
			in_rpt("fgets() did not expected string when buffer length reached before newline");
			in_rpt("expected >%s<, got >%s<",
				t1_string, t_buf);
		}
		if (rval != t_buf)
			in_rpt("fgets() did not return it's argument");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private void
test2()
{
	FILE		*fp;
	char		*rval;
	int		err;
	int		pathok = setupok;

	DBUG_ENTER("test2");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	t_buf[0] = NUL;
	rval = fgets(t_buf, T2_NUM, fp);
	t_buf[sizeof(t_buf) - 1] = NUL;
	if (rval != t_buf || strcmp(t_buf, t2_string))
	{
		xx_rpt(FAILURE);
		if (strcmp(t_buf, t2_string))
		{
			in_rpt("fgets() did not expected string when newline reached before buffer length");
			in_rpt("expected >%s<, got >%s<",
				t2_string, t_buf);
		}
		if (rval != t_buf)
			in_rpt("fgets() did not return it's argument");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private void
test3()
{
	FILE		*fp;
	char		*rval;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test3");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fseek(fp, (long) T3_OFF, SEEK_SET);
	t_buf[0] = NUL;
	rval = fgets(t_buf, T3_NUM, fp);
	t_buf[sizeof(t_buf) - 1] = NUL;
	if (rval != t_buf || strcmp(t_buf, t3_string))
	{
		xx_rpt(FAILURE);
		if (strcmp(t_buf, t3_string))
		{
			in_rpt("fgets() did not expected string when end of file reached before newline");
			in_rpt("expected >%s<, got >%s<",
				t3_string, t_buf);
		}
		if (rval != t_buf)
			in_rpt("fgets() did not return it's argument");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private void
test4()
{
	FILE		*fp;
	char		*rval;
	int		err;
	int		pathok = setupok;

	DBUG_ENTER("test4");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fseek(fp, 0L, SEEK_END);
	t_buf[0] = NUL;
	rval = fgets(t_buf, T4_NUM, fp);
	t_buf[sizeof(t_buf) - 1] = NUL;
	if (rval != (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("when at end of file, fgets() did not return NULL");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private void
test5()
{
	FILE	*fp;
	char	*res;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test5");

#if TEST_XNFS
	
	if (tet_sync(SP_T5_CREAT_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREAT_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */
		PATH_TRACE;
#endif /* TEST_XNFS */

	errno = NOERROR;
	if ((fp = fopen(t5_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t5_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(t5_file);
	PATH_FUNC_TRACE;

        if (fgets(t_buf, T5_NUM, fp) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fgets() failed on file %s, errno %d(%s)",
			t5_file, err, errname(err));
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ((res = chktimes(t5_file, NEWATIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(res);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(5);
	}

	(void) fclose(fp);

#if TEST_XNFS
	(void) tet_sync(SP_T5_UNLINK_START, snames, SYNC_TIMEOUT);
#endif /* TEST_XNFS */
	
	DBUG_VOID_RETURN;
}

/* Lowest test number for which following function is used */
#define TBASE	6

#if TEST_XNFS

private	void
test6_9()
{
	DBUG_ENTER("test6_9");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test6_9()
{
	int	experr, err, ret, ferr_ret;
	static int errnums[] = { EAGAIN, -EAGAIN, EBADF, EINTR, };
			      /* test6   test7    test8  test9 */
	int	pathok = setupok;

	DBUG_ENTER("test6_9");

	experr = errnums[tet_thistest - TBASE];

	globok = 0;
	ret = do_ferr(experr, 'r', -1, t6_9func, &ferr_ret, &err);
	PATH_FUNC_TRACE;

	err = errno;
	if (experr < 0)
		experr = -experr;

	if (tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!ferr_ret || err != experr || !ret)
	{
		xx_rpt(FAILURE);
		if (!ferr_ret)
			in_rpt("fgets() failed to set error indicator when testing for errno %d(%s)",
				experr, errname(experr));
		if (!ret)
			in_rpt("fgets() did not return NULL when testing for errno %d(%s)",
				experr, errname(experr));
		if (err != experr)
			in_rpt("fgets() set errno incorrectly: expected %d(%s), actual %d(%s)",
				experr, errname(experr), err, errname(err));
	}
	else
	{
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

/* ARGSUSED */

private int
t6_9func(fp, nbyte)
FILE *fp;
int nbyte;
{
	int ret;
	int pathok = setupok;
	
	DBUG_ENTER("t6_9func");

	ret = (fgets(t_buf, T6_NUM, fp) == NULL);

	PATH_TRACE;

	if (ret)
		globok = 1;
	else
		globok = 0;

	DBUG_RETURN(ret);
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private	void
test10()
{
	DBUG_ENTER("test10");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test10()
{
	DBUG_ENTER("test10");

	sigstat = "ignored";
	do_feio("ri", -1, t10_11io, t10rpt);

	sigstat = "blocked";
	do_feio("rb", -1, t10_11io, t10rpt);

	DBUG_VOID_RETURN;
}

private void
t10rpt(ret, err)
int ret, err;
{
	int	pathok = setupok;

	DBUG_ENTER("t10rpt");

	PATH_FUNC_TRACE;

	if (!ret || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("fgets() from background with SIGTTIN %s did not give correct results",
				sigstat);
		if (!ret)
			in_rpt("expected return: NULL, actual: non-NULL");
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
	{
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private	void
test11()
{
	DBUG_ENTER("test11");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test11()
{
	DBUG_ENTER("test11");

	do_feio("ro", -1, t10_11io, t11rpt);

	DBUG_VOID_RETURN;
}

private void
t11rpt(ret, err)
int ret, err;
{
	int	pathok = setupok;

	DBUG_ENTER("t11rpt");

	PATH_FUNC_TRACE;

	if (!ret || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("fgets() from orphaned background process did not give correct results");
		if (!ret)
			in_rpt("expected return: NULL, actual: non-NULL");
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
	{
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private int
t10_11io(fp)
FILE *fp;
{
	int ret;
	int pathok = setupok;

	DBUG_ENTER("t10_11io");

	ret = (fgets(t_buf, 2, fp) == NULL);

	PATH_TRACE;
	if (ret)
		globok = 1;
	else
		globok = 0;

	DBUG_RETURN(ret);
}

#endif /* TEST_XNFS */

private void
test12()
{
	DBUG_ENTER("test12");

	(void) cppair(ch_t12, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t12_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	char	*rofs;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t12");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_ROFS is not defined");
		DBUG_VOID_RETURN;
	}

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = t12_dir;
	args[3] = t12_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog_su, err, errname(err));

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test13()
{
	DBUG_ENTER("test13");

	(void) tet_fork(ch_t13, NULLFN, WAITTIME, OK);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t13()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t13");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t13_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test14()
{
	DBUG_ENTER("test14");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test14()
{
	DBUG_ENTER("test14");

	t_estale_rd_c(t14_file, 0, t14_func, t14_chk_func, SP_T14_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t14_func(fp)
FILE	*fp;
{
	DBUG_ENTER("t14_func");

	DBUG_RETURN(fgets(t_buf, 2, fp) == (char *)NULL ? -1 : 0);
}

private int
t14_chk_func(fp)
FILE	*fp;
{
	char	*rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("t14_chk_func");

	errno = NOERROR;

	while((rval = fgets(t_buf, 2, fp)) != (char *)NULL)
		;

	err = errno;

	if(rval != (char *)NULL || ferror(fp) == 0 || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(rval != (char *)NULL)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value NULL");
			in_rpt("Actual   return value \"%s\"", rval);
		}
		else
			PATH_TRACE;
		
		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;
		
		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
