/*
 *      SCCS:  @(#)  ANSI.os/streamio/fopen/enospc.c	Rel 4.4.4     (07/19/99) 
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1991 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#ifndef NO_MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fopen - enospc Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif
#endif  /* lint	*/

/***********************************************************************

NAME:	enospc.c

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()
		enospc()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char *argv[];

DESCRIPTION:
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
	One of:
		FOPEN_TEST FREOPEN_TEST

	Depending on which test this is to be compiled for.

	NO_MAIN for when this file is to be linked directly into the main prog.

SIDE EFFECTS:
RELATED DOCUMENTS:
	A similar file is also used in the directories
		POSIX.os/files/open
		XPG2.os/files/mknod
SOURCE:
AUTHOR:		Steve Ratcliffe, UniSoft Ltd.
DATE CREATED:	28 Feb 1990
MODIFICATIONS:
		Andrew Dingwall Mon Jun 25 11:08:13 BST 1990
		Only use fd when appropriate, depedning on which *_TEST
		is defined.

		David Sawyer
		Mon Jul 16 10:03:29 BST 1990
		Added resillience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file
		
		Geoff Clare, 8 May 1991
			Removed unused code.
			Do tests in a subdirectory to avoid extending
			root directory of VSX_NOSPC_DEV.

		Neil Moses, 29 May 1992
			Check that file is not created on ENOSPC, for fopen()

		Neil Moses, 1 June 1992
			Check that file is not created on ENOSPC, for freopen()

		Dave Cater, 13th November 1992
			XNFS enhancements

		Geoff Clare, 25 Oct 1994
			Avoid freopen() of closed stream.

		Geoff Clare, 2 Jul 1999
			Break from loop when fopen/freopen returns NULL.

************************************************************************/

#include <dbug.h>
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <report.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <tet_api.h>
#include <xnfs.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef fopen
#undef freopen
#endif


#define	WAITTIME	(3 * SPEEDFACTOR)

#ifndef NO_MAIN
public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

#endif

extern	char	*itoa();
extern	char	*errname();
extern	void	set_pname();
extern	void	valrpt();

private int 	testfail;

void	enospc();

extern int 	setupnospc();
extern void	tidynospc();

#define MAXTRYS 2000	/* Number of times we try to fill up system */

#ifndef NO_MAIN

public	int
tet_main(argc, argv)
int	argc;
char	*argv[];
{

	set_pname(argv[0]);

	if (argc < 2 || argc > 4)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}

	if (argc >= 4)
		vg_dbug = argv[3];
	if (argc >= 3)
		vg_ident = argv[2];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	enospc(argv[1]);

	return(OK);
}
#endif /* !NO_MAIN */

/*
 * enospc() tries to run out of space.
 */
void
enospc(base)
char	*base;
{
	char	*name;
	int 	err;
	int 	i;
	FILE	*fp;
	int	pathok = 0;

	DBUG_ENTER("enospc");

	if (setupnospc(base))
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	name = (char*)malloc(strlen(base)+sizeof("/fill.dir/fill.tmp0000"));
	if (name == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("Out of memory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	err = 0;
	for (i = 4; i < MAXTRYS && !testfail && !err; i++)
	{
		if (i==4)
			PATH_TRACE;
		/*
		 * Build up  temp file name and open it
		 */
		(void) strcat(strcpy(name, base), "/fill.dir/fill.tmp");
		(void) strcat(name, itoa(i));

		DBUG_PRINTF("enospc", "create %s", name);
#ifdef FOPEN_TEST
		fp = fopen(name, "w");
#endif
#ifdef FREOPEN_TEST
		fp = freopen(name, "w", stdin);
#endif

		if (fp == NULL)
		{

			err = errno;

			switch (err) {
			case ENOSPC:
				if(eaccess(name, F_OK) != SYSERROR)
				{
					if(!testfail++)
						xx_rpt(FAILURE);
#ifdef FOPEN_TEST
					in_rpt("fopen(%s,\"w\")", name);
#endif
#ifdef FREOPEN_TEST
					in_rpt("freopen(%s,\"w\",stdin)",name);
#endif
					in_rpt("created file on error ENOSPC");
				}
				else
					PATH_TRACE;
				break;
			default:
				testfail++;
				xx_rpt(FAILURE);
				in_rpt("open failed errno %s (%d)", errname(err), err);
				valrpt(0, 0, ENOSPC, err);
				break;
			}
			break;
		}

		if (fp != stdin)
			(void) fclose(fp);

		/*
		 * Now check to see if the file actualy seems to be there ie
		 * that the system didn't just run out of space without saying so
		 */
		if (fp != NULL && access(name, F_OK) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("File was not found to be present");
		}
	}

	if (!testfail)
	{
		if (i >= MAXTRYS)
		{
			xx_rpt(FAILURE);
			in_rpt("ENOSPC error didn't occur in %d trys", MAXTRYS);
		}
		else
		{
			PATH_XS_RPT(4);
		}
	}

	tidynospc(base, i);
	free((void*)name);

	DBUG_VOID_RETURN;

}

