/*
*      SCCS:  @(#)  ANSI.os/streamio/fopen/fopen.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fopen - T.fopen Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.fopen

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fopen() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 1 July 87
		Removed abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Phyfos Photi, Hoskyns Group plc 22/08/88
			extracted freopen() and fdopen() testsets

		Geoff Clare, 15 Nov 1988
			Add tests for ENAMETOOLONG, EINTR, EINVAL
			Remove test for EFAULT

		Geoff Clare, 26 May 1989
			Split out ENXIO, ETXTBSY tests into fopen_X

		Geoff Clare, 3 July 1989
			Rewrite OPEN_MAX test.  Add FOPEN_MAX test.

		Geoff Clare, 22 Nov 1989
			Move EINVAL test to fopen_X

		Steve Ratcliffe, 15 Feb 1990
			Add ENFILE test.

		Steve Ratcliffe, 14 Mar 1990
			Add ENOSPC test.

		Geoff Clare, 4 Apr 1990
			New interface with pathtoolng() and nametoolng().
		
		Geoff Clare, 25 May 1990
			Allow for unlimited OPEN_MAX.

		Anthony Lee, 17 July 1990
			Added resillience code.

		Stuart Boutell, 30 October 1990
			Added VSX4 only tests 9-14

		Stuart Boutell, 2 November 1990
			changed execl to tet_exec

		Stuart Boutell, 5 Nov 1990
			changed vg_env to environ
			removed reason() declaration

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		J.A. Nave	Thu Jan 31 12:05:28 GMT 1991
			Added new test to check error and EOF
		indicators are clear after a sucessful fopen()

		Geoff Clare, 7 Mar 1991
			Fix code for checking O_RDONLY etc. in flags
		
		M Young, 22 Nov `91
			made test2 a sub-program and added FIPS code.

		Stuart Boutell, 22 May 1992
			Added pathname and portable filename tests.

		Neil Moses, 27 May 1992
			Deviance tests 20, 21, 23, 24, 28, 29, 30,
			check file is not truncated/created.

		Neil Moses, 8 June 1992
			Increased timeout in test 23.

		Neil Moses, 11 June 1992.
			Tests 4, 5, 7, 8 - check time stamps.

		Neil Moses, 11 June 1992.
			New tests 33, 34 and 35

		John-Paul, 16 July 1992
			Fixed path trace problem in test 28

                John-Paul, 23 July 1992
	                mode variables changed from 'int' to 'mode_t'

		John-Paul, 20 October 1992
			XNFS enhancements.

		Geoff Clare, 8 Jan 1993
			Change xnfs_anon_uid from int to uid_t.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<limits.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<timeout.h>
#include	<string.h>
#include	<chktimes.h>
#include	<time.h>
#include	<pcts_max.h>
#include	<pathres.h>
#include	<exitcodes.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"fopen_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fopen
#endif



#define BUFFER_ELEMENTS	32
#define VSX_ROFS		"VSX_ROFS"
#define NO_TESTS	44		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)
#define SLEEPTIME	13	/* Number of secs slept in chktimes(), test23 */

#define	T_NBYTES	10
#define	T34_NFILES	3

#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	DEFMODE		(mode_t)(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)
#define	MODEMASK	(mode_t)(S_ISUID|S_ISGID|S_IRWXU|S_IRWXG|S_IRWXO)
#define	T2_UMASK	(mode_t)(S_IWUSR|S_IRGRP|S_IXOTH)
#define	T40_UMASK	(mode_t)(S_IWUSR|S_IRGRP|S_IXOTH)
#define	MODE18		(mode_t)(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)
#define	MODE19		(mode_t)(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)
#define	MODE20_1	(mode_t)(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)
#define	MODE20_2	(mode_t)(S_IWUSR|S_IXUSR|S_IWGRP|S_IXGRP|S_IWOTH|S_IXOTH)
#define	NFOP_MODES	6
#define WRSIZE		10              /* size of write() test buffer */

extern	char	*errname();
extern	char	*signame();
extern	char	*itoa();
extern	char	*uidtoa();
extern	char	*newstr();
extern	char	**pathtoolng();
#if TEST_XNFS
extern	char	*xnfs_nametoolng();
#else /* TEST_XNFS */
extern	char	**nametoolng();
#endif /* TEST_XNFS */
extern  char	*execpath();

extern	void	clntoolng();
extern	void	chktms();
extern	void	alrm();

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;
extern	int 	vguid1;
extern	int 	vggid1;

int	no_tests = NO_TESTS;

private	char	*fop_modes[NFOP_MODES] = { "r", "w", "a", "r+", "w+", "a+" };

#if !TEST_XNFS
private char	*prog2 =		"fopen_t2";
#else /* TEST_XNFS */
private char	*prog19 =		"fopen_su";
private char	*prog20 =		"fopen_su";
#endif /* TEST_XNFS */
private char	*prog23 =		"fopen_t23";
private char	*prog29 =		"fopen_t29";
private char	*prog30 =		"fopen_t30";
private char	*prog33 =		"fopen_t33";

#if TEST_XNFS
private char	*prog36 =		"fopen_t36";
private char	*prog37 =		"fopen_t37";
private char	*prog38 =		"fopen_t38";
private char	*prog39 =		"fopen_t39";
#endif

private char	*t1_file =		"./fopen-t.1";

#if !TEST_XNFS
private char	*t2_file =		"./fopen-t.2";
#endif /* TEST_XNFS */

private char	*t3_file =		"./fopen-t.3";
private char	*t4_file =		"./fopen-t.4";
private char	*t4_file2 =		"./fopen-t.4.2";
private char	*t5_file =		"./fopen-t.5";
private char	*t5_file2 =		"./fopen-t.5.2";
private char	*t6_file =		"./fopen-t.6";
private char	*t7_file =		"./fopen-t.7";
private char	*t7_file2 =		"./fopen-t.7.2";
private char	*t8_file =		"./fopen-t.8";
private char	*t8_file2 =		"./fopen-t.8.2";
private char	*t15_file =		"./fopen-t.15";
private char	*t16_file =		"./fopen-t.16";
private char	*t16a_file =		"./fopen-t.16/fopen-t.16a";
private char	*t17_file =		"./fopen-t.17";
private char	*t18_file =		"./fopen-t.18";
private char	*t18a_file =		"./fopen-t.18/fopen-t.18.1";
private char	*t19_dir =		"./fopen-d.19";
private char	*t19_1_file =		"./fopen-d.19/fopen-t.19.1";
#if TEST_XNFS
private char	*t19_2_file =		T19_2_FILE;
#endif /* TEST_XNFS */
private char	*t20_dir =		"./fopen-d.20";
private char	*t20_1_file =		"./fopen-d.20/fopen-t.20.1";
#if TEST_XNFS
private char	*t20_2_file =		T20_2_FILE;
#endif /* TEST_XNFS */
private char	*t21_file =		"./fopen-t.21";
private char	*t22_dir =		"./fopen-d.22";
private char	*t23_dir =		"./fopen-d.23";
private char	*t23_file =		"./fopen-d.23/fopen-t.23";
private char	*t24_file =		"./fopen-t.24";
private char	*t25_file =		"./fopen-t.25";
#if !TEST_XNFS
private char	*t27_file =		"./fopen-t.27";
#endif /* TEST_XNFS */
private char	*t28_file =		"./fopen-t.28";
private char	*t29_file =		"./fopen-t.29";
private char	*t30_dir =		"./fopen-d.30";
private char	*t33_file =		"./fopen-t.33";
private char	*t34_file =		"./fopen-t.34";
private char	*t35_file =		"./fopen-t.35";
#if TEST_XNFS
private char    *t36_file =		"./fopen-t.36";
private char    *t37_file =		"./fopen-t.37";
private char    *t38_mnt =		T38_MNT;
private char	*t40_file =		"./fopen-t.40";
#endif /* TEST_XNFS */

private char	*msg;
public	int	testfail;
private char	wbuf[WRSIZE];
private int	open_max;


private void    prepare_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
public	void	test9();
public	void	test10();
public	void	test11();
public	void	test12();
public	void	test13();
public	void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();
private void	test30();
private	void	test31();
private	void	test32();
private	void	test33();
private	void	test34();
private	void	test35();
private	void	test36_37();
private	void	test38();
private	void	test39();
private	void	test40();
private	void	test41();
private	void	test42();
private	void	test43();
private	void	test44();
private	void	ch_t2();

#if TEST_XNFS
private	void	ch_t19();
private	void	ch_t20();
#endif

private	void	ch_t23();
private	void	ch_t24();
#if !TEST_XNFS
private	void	ch_t27();
#endif
private	void	ch_t28();
private	void	ch_t29();
private	void	ch_t30();
private	void	ch_t33();

#if TEST_XNFS
private	void	ch_t36_37();
private	void	ch_t38();
private	void	ch_t39();
private	void	ch_t40();
private	void	ch_t41();
#endif /* TEST_XNFS */

private	FILE	*tb31_32fopen();
#if TEST_XNFS
private	FILE	*tbfopen();
#endif

private	int	t31_32fail;

#if TEST_XNFS
private int	snames[] = { 1, 0 };
#endif

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	test3, 3,
	test4, 4,
	test5, 5,
	test6, 6,
	test7, 7,
	test8, 8,
	test9, 9,
	test10, 10,
	test11, 11,
	test12, 12,
	test13, 13,
	test14, 14,
	test15, 15,
	test16, 16,
	test17, 17,
	test18, 18,
	test19, 19,
	test20, 20,
	test21, 21,
	test22, 22,
	test23, 23,
	test24, 24,
	test25, 25,
	test26, 26,
	test27, 27,
	test28, 28,
	test29, 29,
	test30, 30,
	test31, 31,
	test32, 32,
	test33, 33,
	test34, 34,
	test35, 35,
	test36_37, 36,
	test36_37, 37,
	test38, 38,
	test39, 39,
	test40, 40,
	test41, 41,
	test42, 42,
	test43, 43,
	test44, 44,
	NULL, 0
};

private	void
prepare_tests()
{
	int i;

	DBUG_ENTER("prepare_tests");

	if(uids()!=0)
	{
		for(i=1; i <= no_tests; i++)
		{
			(void) tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	for (i = 0 ; i < WRSIZE ; i++)
		wbuf[i] = i % 100;
	/*
	 *	this so that setuid children can write to files
	 */
	(void) umask((mode_t)0);

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0)
	{
		if (errno == 0)
			open_max = PCTS_OPEN_MAX+1;
		else
		{
			for (i = 1; i <= NO_TESTS; i++)
				(void) tet_delete(i,
					"sysconf(_SC_OPEN_MAX) failed during prep");
		}
	}

#if TEST_XNFS
	if (xnfs_uids() < 0)
		(void) tet_delete(49, "xnfs_uids() failed");
#endif

	DBUG_VOID_RETURN;
}


private void
test1()
{
	int		fd1;
	int		fd2;
	int		err;
	FILE		*fp;
	struct stat	statb1;
	struct stat	statb2;
	char		*delmsg = "depends on success of test 1";
	int 		pathok = 0;
	
	DBUG_ENTER("test1");

	(void) umask((mode_t)0);

	if ((fd1 = creat(t1_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t1_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd1);
		
	if (stat(t1_file, &statb1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(%s) failed - errno %d(%s)",
			t1_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fopen(t1_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"r\") on readable file failed- errno %d(%s)",
			err, errname(err));
		for (err = 2 ; err <= 8 ; err++)
			(void) tet_delete(err, delmsg);
		(void) tet_delete(15, delmsg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	fd2 = fileno(fp);
	if (fstat(fd2, &statb2) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fstat() on file descriptor of stream returned by fopen() failed - errno %d(%s)",
			err, errname(err));
	}
	else if (statb1.st_ino != statb2.st_ino
		|| statb1.st_dev != statb2.st_dev)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen() did not seem to open file indicated by path");
		in_rpt("dev/inode of file = 0x%lx/%ld",
			(long)statb1.st_dev, (long)statb1.st_ino);
		in_rpt("dev/inode of file discriptor of stream = 0x%lx/%ld",
			(long)statb2.st_dev, (long)statb2.st_ino);
		for (err = 2 ; err <= 8 ; err++)
			(void) tet_delete(err, delmsg);
		(void) tet_delete(15, delmsg);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(4);
	}

	(void) fclose(fp);
	(void) unlink(t1_file);
	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test2()
{
        DBUG_ENTER("test2");
        xx_rpt(NOTINUSE);
        DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test2()
{
	DBUG_ENTER("test2");

	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t2");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		exit(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog2;
	args[1] = t2_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog2), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog2), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private void
test3()
{
	int		fd;
	FILE		*fp;
	int		err;
	int		flags;
	int 		pathok = 0; 

	DBUG_ENTER("test3");
	if ((fd = creat(t3_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t3_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	if ((fp = fopen(t3_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"r\") failed to open file - errno %d(%s)",
			t3_file, err, errname(err));
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDONLY)
	{
		xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"r\") did not open file for read-only");
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(4);
	}

	(void) fclose(fp);
	(void) unlink(t3_file);
	DBUG_VOID_RETURN;
}


private void
test4()
{
	int		fd;
	FILE		*fp;
	time_t		t0;
	int		err;
	int		fail = 0;
	int		flags;
	char		buf[T_NBYTES];
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test4");

	testfail = 0;

	if ((fd = creat(t4_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t4_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	globok = 0;
	savtimes(t4_file);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((fp = fopen(t4_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w\") failed to open file - errno %d(%s)",
			t4_file, err, errname(err));
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_WRONLY)
		fail = 1;
	else
		PATH_TRACE;
		
	if (stat(t4_file, &statb) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t4_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (statb.st_size != 0L)
		fail |= 2;
	else
		PATH_TRACE;
		
	globok = 0;
	msg = chktimes(t4_file,NEWCTIME|NEWMTIME);
	PATH_FUNC_TRACE;

	if(msg != NULL)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w\") : %s", t4_file,msg);
	}
	else
		PATH_TRACE;

	(void) fclose(fp);
	(void) unlink(t4_file);

	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T4_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = fopen(t4_file2, "w")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w\") failed to open file that did not exist - errno %d(%s)",
			t4_file2, err, errname(err));
		(void) unlink(t4_file2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t4_file2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_WRONLY)
		fail |= 4;
	else
		PATH_TRACE;
		
	if (stat(t4_file2, &statb) == SYSERROR)
		fail |= 8;
	else if (statb.st_size != 0L)
		fail |= 16;
	else
		PATH_TRACE;
		
	globok = 0;
	chktms(t4_file2, t0);
	PATH_FUNC_TRACE;

	globok = 0;
	msg = chktimes(".",NEWCTIME|NEWMTIME);
	PATH_FUNC_TRACE;

	if(msg != NULL)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w\") - parent directory: %s", t4_file2,msg);
	}
	else
		PATH_TRACE;

	if (fail)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("fopen(%s, \"w\") did not open file for write-only",
				t4_file2);
		if (fail & 2)
			in_rpt("fopen(%s, \"w\") did not truncate file",
				t4_file2);
		if (fail & 4)
			in_rpt("fopen(%s, \"w\") did not open file for write-only when file did not exist",
				t4_file2);
		if (fail & 8)
			in_rpt("fopen(%s, \"w\") did not create file",
				t4_file2);
		else if (fail & 16)
			in_rpt("fopen(%s, \"w\") did not create file with zero size",
				t4_file2);
	}

	if(!testfail)
		PATH_XS_RPT(19);

	(void) fclose(fp);
	(void) unlink(t4_file);
	(void) unlink(t4_file2);
	DBUG_VOID_RETURN;
}


private void
test5()
{
	int		fd;
	FILE		*fp;
	time_t		t0;
	int		err;
	int		fail = 0;
	int		flags;
	long		size;
	char		buf[T_NBYTES];
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test5");

	testfail = 0;

	if ((fd = creat(t5_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t5_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	if ((fp = fopen(t5_file, "a")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a\") failed to open file - errno %d(%s)",
			t5_file, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_WRONLY)
		fail = 1;
	else
		PATH_TRACE;

	if (!(flags & O_APPEND))
		fail = 2;
	else
		PATH_TRACE;

	if (stat(t5_file, &statb) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((size = statb.st_size) != (long) T_NBYTES)
		fail |= 4;
	else
		PATH_TRACE;

	(void) fclose(fp);
	(void) unlink(t5_file);

	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T5_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = fopen(t5_file2, "a")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a\") failed to open file that did not exist - errno %d(%s)",
			t5_file2, err, errname(err));
		(void) unlink(t5_file2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t5_file2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_WRONLY)
		fail |= 8;
	else
		PATH_TRACE;

	if (!(flags & O_APPEND))
		fail |= 16;
	else
		PATH_TRACE;

	if (stat(t5_file2, &statb) == SYSERROR)
		fail |= 32;
	else if (statb.st_size != 0L)
		fail |= 64;
	else
		PATH_TRACE;

	globok = 0;
	chktms(t5_file2, t0);
	PATH_FUNC_TRACE;

	globok = 0;
	msg = chktimes(".",NEWCTIME|NEWMTIME);
	PATH_FUNC_TRACE;

	if(msg != NULL)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a\") - parent directory: %s", t5_file2,msg);
	}
	else
		PATH_TRACE;

	if (fail)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("fopen(%s, \"a\") did not open file for write-only",
				t5_file2);
		if (fail & 2)
			in_rpt("fopen(%s, \"a\") did not open file for append",
				t5_file2);
		if (fail & 4)
			in_rpt("fopen(%s, \"a\") changed size of file - expected %d, found %ld",
				t5_file2, T_NBYTES, size);
		if (fail & 8)
			in_rpt("fopen(%s, \"a\") when file did not exist, did not open file for write-only",
				t5_file2);
		if (fail & 16)
			in_rpt("fopen(%s, \"a\") when file did not exist, did not open file for append",
				t5_file2);
		if (fail & 32)
			in_rpt("fopen(%s, \"a\") when file did not exist, did not creat file",
				t5_file2);
		else if (fail & 64)
			in_rpt("fopen(%s, \"a\") when file did not exist, did not creat file with zero length",
				t5_file2);
	}

	if(!testfail)
		PATH_XS_RPT(18);

	(void) fclose(fp);
	(void) unlink(t5_file);
	(void) unlink(t5_file2);
	DBUG_VOID_RETURN;
}

private void
test6()
{
	int		fd;
	FILE		*fp;
	int		err;
	int		flags;
	char		buf[T_NBYTES];
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test6");
	if ((fd = creat(t6_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t6_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	if ((fp = fopen(t6_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"r+\") failed to open file - errno %d(%s)",
			t6_file, err, errname(err));
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t6_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR || statb.st_size != T_NBYTES)
	{
		xx_rpt(FAILURE);
		if ((flags & O_ACCMODE) != O_RDWR)
			in_rpt("fopen(%s, \"r+\") did not open file for read-write");
		if (statb.st_size != T_NBYTES)
			in_rpt("fopen(%s, \"r+\") changed size of file, expected size %d, found %ld",
				T_NBYTES, statb.st_size);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(6);
	}

	(void) fclose(fp);
	(void) unlink(t6_file);
	DBUG_VOID_RETURN;
}


private void
test7()
{
	int		fd;
	time_t		t0;
	FILE		*fp;
	int		err;
	int		fail = 0;
	int		flags;
	char		buf[T_NBYTES];
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test7");

	testfail = 0;

	if ((fd = creat(t7_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t7_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) unlink(t7_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	globok = 0;
	savtimes(t7_file);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((fp = fopen(t7_file, "w+")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+\") failed to open file - errno %d(%s)",
			t7_file, err, errname(err));
		(void) unlink(t7_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t7_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR)
		fail = 1;
	else
		PATH_TRACE;
		
	if (stat(t7_file, &statb) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t7_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t7_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (statb.st_size != 0L)
		fail |= 2;
	else
		PATH_TRACE;
		
	globok = 0;
	msg = chktimes(t7_file,NEWCTIME|NEWMTIME);
	PATH_FUNC_TRACE;

	if(msg != NULL)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+\") : %s", t7_file,msg);
	}
	else
		PATH_TRACE;

	(void) fclose(fp);
	(void) unlink(t7_file);

	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T7_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = fopen(t7_file2, "w+")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+\") failed to open file that did not exist - errno %d(%s)",
			t7_file2, err, errname(err));
		(void) unlink(t7_file2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t7_file2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR)
		fail |= 4;
	else
		PATH_TRACE;
		
	if (stat(t7_file2, &statb) == SYSERROR)
		fail |= 8;
	else if (statb.st_size != 0L)
		fail |= 16;
	else
		PATH_TRACE;
		
	globok = 0;
	chktms(t7_file2, t0);
	PATH_FUNC_TRACE;

	globok = 0;
	msg = chktimes(".",NEWCTIME|NEWMTIME);
	PATH_FUNC_TRACE;

	if(msg != NULL)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+\") - parent directory: %s", t7_file2,msg);
	}
	else
		PATH_TRACE;

	if (fail)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("fopen(%s, \"w+\") did not open file for read-write",
				t7_file2);
		if (fail & 2)
			in_rpt("fopen(%s, \"w+\") did not truncate file",
				t7_file2);
		if (fail & 4)
			in_rpt("fopen(%s, \"w+\") did not open file for read-write when file did not exist",
				t7_file2);
		if (fail & 8)
			in_rpt("fopen(%s, \"w+\") did not create file",
				t7_file2);
		else if (fail & 16)
			in_rpt("fopen(%s, \"w+\") did not create file with zero size",
				t7_file2);
	}

	if(!testfail)
		PATH_XS_RPT(19);

	(void) fclose(fp);
	(void) unlink(t7_file);
	(void) unlink(t7_file2);
	DBUG_VOID_RETURN;
}


private void
test8()
{
	int		fd;
	FILE		*fp;
	time_t		t0;
	int		err;
	int		fail = 0;
	int		flags;
	long		size;
	char		buf[T_NBYTES];
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test8");

	testfail = 0;

	if ((fd = creat(t8_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t8_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) close(fd);
	if ((fp = fopen(t8_file, "a+")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+\") failed to open file - errno %d(%s)",
			t8_file, err, errname(err));
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR)
		fail = 1;
	else
		PATH_TRACE;
		
	if (!(flags & O_APPEND))
		fail = 2;
	else
		PATH_TRACE;
		
	if (stat(t8_file, &statb) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t8_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((size = statb.st_size) != (long) T_NBYTES)
		fail |= 4;
	else
		PATH_TRACE;
		
	(void) fclose(fp);
	(void) unlink(t8_file);

	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T8_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */


	if ((fp = fopen(t8_file2, "a+")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+\") failed to open file that did not exist - errno %d(%s)",
			t8_file2, err, errname(err));
		(void) unlink(t8_file2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t8_file2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR)
		fail |= 8;
	else
		PATH_TRACE;
		
	if (!(flags & O_APPEND))
		fail |= 16;
	else
		PATH_TRACE;
		
	if (stat(t8_file2, &statb) == SYSERROR)
		fail |= 32;
	else if (statb.st_size != 0L)
		fail |= 64;
	else
		PATH_TRACE;

	globok = 0;
	chktms(t8_file2, t0);
	PATH_FUNC_TRACE;

	globok = 0;
	msg = chktimes(".",NEWCTIME|NEWMTIME);
	PATH_FUNC_TRACE;

	if(msg != NULL)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+\") - parent directory: %s", t8_file2,msg);
	}
	else
		PATH_TRACE;

	if (fail)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("fopen(%s, \"a+\") did not open file for read-write",
				t8_file2);
		if (fail & 2)
			in_rpt("fopen(%s, \"a+\") did not open file for append",
				t8_file2);
		if (fail & 4)
			in_rpt("fopen(%s, \"a+\") changed size of file - expected %d, found %ld",
				t8_file2, T_NBYTES, size);
		if (fail & 8)
			in_rpt("fopen(%s, \"a+\") when file did not exist, did not open file for read-write",
				t8_file2);
		if (fail & 16)
			in_rpt("fopen(%s, \"a+\") when file did not exist, did not open file for append",
				t8_file2);
		if (fail & 32)
			in_rpt("fopen(%s, \"a+\") when file did not exist, did not creat file",
				t8_file2);
		else if (fail & 64)
			in_rpt("fopen(%s, \"a+\") when file did not exist, did not creat file with zero length",
				t8_file2);
	}

	if(!testfail)
		PATH_XS_RPT(18);

	(void) fclose(fp);
	(void) unlink(t8_file);
	(void) unlink(t8_file2);
	DBUG_VOID_RETURN;
}


private void
test15()
{
	int		fd1;
	int		err;
	FILE		*fp;
	int 		pathok = 0;
	
	DBUG_ENTER("test15");

	testfail = 0;

	if ((fd1 = creat(t15_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t1_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd1);

	if ((fp = fopen(t15_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"r\") on readable file failed- errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ferror(fp) != 0)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen() has not cleared the error indicator for the stream");
	}
	else
		PATH_TRACE; 

	if (feof(fp) != 0)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen() has not cleared the end-of-file indicator for the stream");
	}
	else
		PATH_TRACE; 

	if (!testfail)
		PATH_XS_RPT(4);

	(void) fclose(fp);
	(void) unlink(t15_file);
	DBUG_VOID_RETURN;
}


private	void
test16()
{
	int		fd;
	int		err1;
	int		fail = 0;
	FILE		*fp;
	int 		pathok = 0; 

	DBUG_ENTER("test16");
	if ((fd = creat(t16_file, MODEANY)) == SYSERROR)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d(%s)",
			t16_file, MODEANY, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	errno = NOERROR;
	if ((fp = fopen(t16a_file, "r")) != (FILE *) NULL)
	{
		fail |= 1;
		(void) fclose(fp);
	}
	else if (errno != ENOTDIR)
	{
		err1 = errno;
		fail |= 2;
	}
	else
		PATH_TRACE;
		
	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("fopen(%s) when %s was a file, did not return NULL",
				t16a_file, t16_file);
		if (fail & 2)
		{
			in_rpt("fopen(%s) when %s was a file, did not set errno correctly",
				t16a_file, t16_file);
			in_rpt("expected %d (ENOTDIR), received %d (%s)",
				ENOTDIR, err1, errname(err1));
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	(void) unlink(t16_file);
	DBUG_VOID_RETURN;
}


private	void
test17()
{
	int		err1;
	int		fail = 0;
	FILE		*tfp;
	int 		pathok = 0; 

	DBUG_ENTER("test17");
	if (unlink(t17_file) == SYSERROR && errno != ENOENT)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("could not unlink %s - errno %d(%s)",
			t17_file, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		

	errno = NOERROR;
	if ((tfp = fopen(t17_file, "r")) != (FILE *) NULL)
	{
		fail |= 1;
		(void) fclose(tfp);
		(void) unlink(t17_file);
	}
	else if (errno != ENOENT)
	{
		err1 = errno;
		fail |= 2;
	}
	else
		PATH_TRACE;
		

	errno = NOERROR;
	if ((tfp = fopen(t17_file, "r+")) != (FILE *) NULL)
	{
		fail |= 4;
		(void) fclose(tfp);
		(void) unlink(t17_file);
	}
	else if (errno != ENOENT)
	{
		err1 = errno;
		fail |= 8;
	}
	else
		PATH_TRACE;
		
	if (fail)
	{
		xx_rpt(FAILURE);
		DBUG_PRINTF("test17", "fail - %d", fail);
		if (fail & 1)
			in_rpt("fopen(s, \"r\") on file that did not exist, did not return NULL");
		if (fail & 2)
		{
			in_rpt("fopen(s, \"r\") on file that did not exist, did not set errno correctly");
			in_rpt("expected %d (ENOENT), received %d (%s)",
				ENOENT, err1, errname(err1));
		}
		if (fail & 4)
			in_rpt("fopen(s, \"r+\") on file that did not exist, did not return NULL");
		if (fail & 8)
		{
			in_rpt("fopen(s, \"r+\") on file that did not exist, did not set errno correctly");
			in_rpt("expected %d (ENOENT), received %d (%s)",
				ENOENT, err1, errname(err1));
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}
	DBUG_VOID_RETURN;
}

private void
test18()
{
	int		err1;
	int		err2;
	int		fail = 0;
	FILE		*tfp;
	int 		pathok = 0; 

	DBUG_ENTER("test18");
	if (unlink(t18_file) == SYSERROR && errno != ENOENT)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("could not unlink %s - errno %d(%s)",
			t18_file, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	errno = NOERROR;
	if ((tfp = fopen(t18a_file, "r")) != (FILE *) NULL)
	{
		fail |= 1;
		(void) fclose(tfp);
	}
	else if (errno != ENOENT)
	{
		err2 = errno;
		fail |= 2;
	}
	else
		PATH_TRACE;
		
	if (fail)
	{
		xx_rpt(FAILURE);
		DBUG_PRINTF("test18", "fail - %d", fail);
		if (fail & 1)
			in_rpt("fopen(%s) when %s did not exist, did not return NULL",
				t18a_file, t18_file);
		if (fail & 2)
		{
			in_rpt("fopen(%s) when %s did not exist, did not set errno correctly",
				t18a_file, t18_file);
			in_rpt("expected %d (ENOENT), received %d (%s)",
				ENOENT, err2, errname(err2));
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}
	(void) unlink(t18a_file);
	DBUG_VOID_RETURN;
}

private void
test19()
{
	FILE		*fp;
	int		err2;
	int		fail = 0;
	int		pathcheck = 0;
#if TEST_XNFS
	int	rc;
#endif
	int	 	pathok = 0; 

	DBUG_ENTER("test19");

	testfail = 0;

	if (crdir(t19_dir, (mode_t)MODE18, (int) getuid(), (int) getgid())
								== SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t19_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	errno = NOERROR;
	if ((fp = fopen(t19_1_file, "w")) != (FILE *) NULL)
	{
		fail |= 1;
		(void) fclose(fp);
	}
	else if (errno != EACCES)
	{
		fail |= 2;
		err2 = errno;
	}
	else
		PATH_TRACE;
		
	(void) unlink(t19_1_file);
	if (fail)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("fopen(%s, \"w\") in a non-searchable directory did not return NULL",
				t19_1_file);
		else if (fail & 2)
		{
			in_rpt("fopen(%s, \"w\") in a non-searchable directory did not set errno correctly",
				t19_1_file);
			in_rpt("expected %d (EACCES), received %d (%s)",
				EACCES, err2, errname(err2));
		}
	}
	else
	{
		PATH_TRACE;
	}
	(void) rmdir(t19_dir);

#if TEST_XNFS

	if (tet_sync(SP_T19_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t19, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T19_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pathcheck = 6;

#else /* TEST_XNFS */

	pathcheck = 3;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
ch_t19()
{
	int err;
	char *args[5];

	DBUG_ENTER("ch_t19");

	args[0] = prog19;
	args[1] = t19_2_file;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog19), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog19, err, errname(err));

	DBUG_EXIT(NOK);
}

#endif /* TEST_XNFS */

private void
test20()
{
	FILE		*fp;
	int		err1;
	int		fail = 0;
	int		pathcheck = 0;
	int	 	pathok = 0; 
#if TEST_XNFS
	int	rc;
#endif

	DBUG_ENTER("test20");

	testfail = 0;

	if (crdir(t20_dir, (mode_t)MODE19, (int) getuid(), (int) getgid())
								== SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t20_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	errno = NOERROR;
	if ((fp = fopen(t20_1_file, "w")) != (FILE *) NULL)
	{
		fail |= 1;
		(void) fclose(fp);
	}
	else if (errno != EACCES)
	{
		fail |= 2;
		err1 = errno;
	}
	else
		PATH_TRACE;
		
	if(eaccess(t20_1_file, F_OK) != SYSERROR)
		fail |= 4;
	else
		PATH_TRACE;

	(void) unlink(t20_1_file);

	if (fail)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("fopen(%s, \"w\") in a non-writable directory did not return NULL",
				t20_1_file);
		else if (fail & 2)
		{
			in_rpt("fopen(%s, \"w\") in a non-writable directory did not set errno correctly",
				t20_1_file);
			in_rpt("expected %d (EACCES), received %d (%s)",
				EACCES, err1, errname(err1));
		}
		else if (fail & 4)
			in_rpt("fopen(%s, \"w\") in a non-writable directory on error EACCES created file", t20_1_file);
	}
	else
		PATH_TRACE;

	(void) rmdir(t20_dir);

#if TEST_XNFS

	if (tet_sync(SP_T20_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t20, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T20_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
	}
	else
		PATH_TRACE;

	pathcheck = 7;

#else /* TEST_XNFS */

	pathcheck = 4;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
ch_t20()
{
	int err;
	char *args[5];

	DBUG_ENTER("ch_t20");

	args[0] = prog20;
	args[1] = t20_2_file;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog20), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog20, err, errname(err));

	DBUG_EXIT(NOK);
}

#endif /* TEST_XNFS */

private void
test21()
{
	FILE	*fp;
	struct	stat buf;
	int	fd;
	int	i;
	int	fail = 0;
	int	err1;
	int 	pathok = 0; 

	DBUG_ENTER("test21");
	if ((fd = creat(t21_file, MODE20_1)) == SYSERROR)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			t21_file, MODE20_1, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(write(fd, "1", 1) == SYSERROR)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);
		
	for (i = 1; i < NFOP_MODES; i++)
	{
		errno = NOERROR;
		if ((fp = fopen(t21_file, fop_modes[i])) != (FILE *) NULL)
		{
			if (!fail)
				xx_rpt(FAILURE);
			in_rpt("fopen(%s, %s) in a non-writable file did not return NULL",
				t21_file, fop_modes[i]);
			fail++;
			(void) fclose(fp);
		}
		else if (errno != EACCES)
		{
			err1 = errno;
			if (!fail)
				xx_rpt(FAILURE);
			in_rpt("fopen(%s, %s) in a non-writable file did not set errno correctly",
				t21_file, fop_modes[i]);
			in_rpt("expected %d (EACCES), received %d (%s)",
				EACCES, err1, errname(err1));
			fail++;
		}
		else
			PATH_TRACE; 

		if(stat(t21_file, &buf) < 0)
		{
			err1 = errno;
			if (!fail)
				xx_rpt(DELETION);
			in_rpt("stat(%s, buf) failed - errno %d (%s)",
				t21_file, err1, errname(err1));
			DBUG_VOID_RETURN;
		}
		else if(buf.st_size == 0)
		{
			if (!fail)
				xx_rpt(FAILURE);
			in_rpt("fopen(%s, %s) in a non-writable file truncated file",
				t21_file, fop_modes[i]);
			fail++;
			(void) fclose(fp);
		}
		else
			PATH_TRACE;
	}
	(void) unlink(t21_file);

	if ((fd = creat(t21_file, MODE20_2)) == SYSERROR)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			t21_file, MODE20_2, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if(write(fd, "1", 1) == SYSERROR)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	for (i = 0; i < NFOP_MODES; i += 3)
	{
		errno = NOERROR;
		if ((fp = fopen(t21_file, fop_modes[i])) != (FILE *) NULL)
		{
			if (!fail)
				xx_rpt(FAILURE);
			in_rpt("fopen(%s, %s) in a non-readable file did not return NULL",
				t21_file, fop_modes[i]);
			fail++;
			(void) fclose(fp);
		}
		else if (errno != EACCES)
		{
			err1 = errno;
			if (!fail)
				xx_rpt(FAILURE);
			in_rpt("fopen(%s, %s) in a non-readable file did not set errno correctly",
				t21_file, fop_modes[i]);
			in_rpt("expected %d (EACCES), received %d (%s)",
				EACCES, err1, errname(err1));
			fail++;
		}
		else
			PATH_TRACE; 

		if(stat(t21_file, &buf) < 0)
		{
			err1 = errno;
			if (!fail)
				xx_rpt(DELETION);
			in_rpt("stat(%s, buf) failed - errno %d (%s)",
				t21_file, err1, errname(err1));
			DBUG_VOID_RETURN;
		}
		else if(buf.st_size == 0)
		{
			if (!fail)
				xx_rpt(FAILURE);
			in_rpt("fopen(%s, %s) in a non-writable file truncated file",
				t21_file, fop_modes[i]);
			fail++;
			(void) fclose(fp);
		}
		else
			PATH_TRACE;
	}

	if (!fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(5+((NFOP_MODES - 1)*2)+(((NFOP_MODES + 2) / 3)*2));
	}
	(void) unlink(t21_file);
	DBUG_VOID_RETURN;
}


private void
test22()
{
	FILE		*fp;
	int		errs[NFOP_MODES * 2];
	int		flags[NFOP_MODES];
	int		fail = 0;
	int		i;
	int	 	pathok = 0; 

	DBUG_ENTER("test22");
	if (crdir(t22_dir, (mode_t)MODEANY, (int) getuid(), (int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t22_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	for (i = 1 ; i < NFOP_MODES ; i++)
	{
		errno = NOERROR;
		flags[i] = 0;
		DBUG_PRINTF("test22", "mode=%s", fop_modes[i]);
		if ((fp = fopen(t22_dir, fop_modes[i])) != (FILE *) NULL)
		{
			flags[i] = 1;
			(void) fclose(fp);
			fail = 1;
		}
		else if (errno != EISDIR)
		{
			flags[i] = 2;
			errs[i] = errno;
			fail = 1;
		}
		else
			PATH_TRACE; 
		DBUG_PRINTF("test22", "point 1");
	}
	if (fail)
	{
		xx_rpt(FAILURE);
		for (i = 1 ; i < NFOP_MODES ; i++)
		{
			DBUG_PRINTF("test22", "point 2");
			if (flags[i] & 1)
				in_rpt("fopen(%s) on a directory did not return NULL",
					fop_modes[i]);
			else if (flags[i] & 2)
			{
				in_rpt("fopen(%s) on a directory did not set errno correctly",
					fop_modes[i]);
				in_rpt("expected %d (EISDIR), received %d (%s)",
					EISDIR, errs[i], errname(errs[i]));
			}
			DBUG_PRINTF("test22", "point 3");
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2 + (NFOP_MODES - 1));
	}
	(void) rmdir(t22_dir);
	DBUG_VOID_RETURN;
}


private void
test23()
{
	DBUG_ENTER("test23");

	(void) cppair(ch_t23, NULLFN, 2 * WAITTIME + SLEEPTIME, 0);

	(void) rmdir(t23_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t23()
{
	char		*args[7];
	char		*rofs;

	DBUG_ENTER("ch_t23");

	if ((rofs = tet_getvar(VSX_ROFS)) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s not defined", VSX_ROFS);
		DBUG_VOID_RETURN;
	}

	args[0] = prog23 ;
	args[1] = rofs ;
	args[2] = t23_dir ;
	args[3] = t23_file ;
	args[4] = vg_ident ;
	args[5] = vg_dbug ;
	args[6] =  (char *) NULL;
	(void) tet_exec(execpath(args[0]), args, environ);
	vsxerror("test23: tet_exec(%s, ...) failed - errno %d(%s)",
		execpath(prog23), errno, errname(errno));
	er_rpt("test 23: tet_exec() failed");

	DBUG_VOID_RETURN;
}


private void
test24()
{
	int		fd;
	int		err;
	int	 	pathok = 0; 

	DBUG_ENTER("test24");
	if ((fd = creat(t24_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not creat() %s - errno %d (%s)", t24_file, err,
			errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if(write(fd, "1", 1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err, errname(err));
		DBUG_VOID_RETURN;
	}

	(void) close(fd);

	(void) cppair(ch_t24, NULLFN, WAITTIME, 0);

	(void) unlink(t24_file);

	DBUG_VOID_RETURN;
}

private	void
ch_t24()
{
	long	open_max, count, loop_end;
	struct	stat buf;
	int	err;
	FILE	*fp;
	int	pathok = 0;
	int	fail = 0;
	int 	first_iter = 1;

	DBUG_ENTER("ch_t24");

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0)
	{
		if (errno == 0)
		{
			PATH_TRACE;
			open_max = PCTS_OPEN_MAX+1;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: sysconf(_SC_OPEN_MAX) failed");
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

	loop_end = open_max;
	if (open_max > PCTS_OPEN_MAX)
		loop_end = PCTS_OPEN_MAX-1;

	first_iter = 1;
	for (count = n_open_fds(); count < loop_end; count++)
	{
		if (open(t24_file, O_RDWR) < 0)
		{
			err = errno;
			xx_rpt(DELETION);
			if (open_max <= PCTS_OPEN_MAX)
				in_rpt("could not open OPEN_MAX file descriptors");
			else
				in_rpt("could not open PCTS_OPEN_MAX-1 file descriptors");
			in_rpt("open(\"%s\", O_RDWR) failed - errno %d (%s)",
				t24_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else if (first_iter == 1)
		{
			PATH_TRACE;
			first_iter = 0;
		}
	}

	errno = NOERROR;

	/* Check that file is not truncated */
	fp = fopen(t24_file, "w");

	if (!fail && open_max <= PCTS_OPEN_MAX)
	{
		/* expect EMFILE error */
		if (fp != NULL || errno != EMFILE)
		{
			err = errno;
			if(!fail++)
				xx_rpt(FAILURE);
			if (fp != NULL)
				in_rpt("fopen() succeeded when OPEN_MAX file descritors were already open");
			else
			{
				in_rpt("fopen(\"%s\", \"w\") did not set errno correctly",
					t24_file);
				in_rpt("expected %d (EMFILE), received %d (%s)",
					EMFILE, err, errname(err));
			}
		}
		else
		{
			PATH_TRACE;

			if(stat(t24_file, &buf) < 0)
			{
				err = errno;
				if (!fail)
					xx_rpt(DELETION);
				in_rpt("stat(%s, buf) failed - errno %d (%s)",
					t21_file, err, errname(err));
				DBUG_VOID_RETURN;
			}
			else if(buf.st_size == 0)
			{
				if (!fail++)
					xx_rpt(FAILURE);
				in_rpt("fopen(\"%s\", \"w\") on error EMFILE truncated file", t24_file);
			}
			else
				PATH_TRACE;
		}
	}
	else
	{
		/* expect success */
		
		if (fp == NULL)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("fopen(\"%s\", \"w\") failed when PCTS_OPEN_MAX-1 file descriptors were already open",
				t24_file);
			in_rpt("errno was set to %d (%s)", err, errname(err));
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(3);
		}
	}

	/* Check that file is not created */
	(void) unlink(t24_file);
	fp = fopen(t24_file, "w");

	if (!fail && open_max <= PCTS_OPEN_MAX)
	{
		/* expect EMFILE error */
		if (fp != NULL || errno != EMFILE)
		{
			err = errno;
			if(!fail++)
				xx_rpt(FAILURE);
			if (fp != NULL)
				in_rpt("fopen() succeeded when OPEN_MAX file descritors were already open");
			else
			{
				in_rpt("fopen(\"%s\", \"w\") did not set errno correctly",
					t24_file);
				in_rpt("expected %d (EMFILE), received %d (%s)",
					EMFILE, err, errname(err));
			}
		}
		else
		{
			PATH_TRACE;

			if(eaccess(t24_file, F_OK) != SYSERROR)
			{
				if (!fail++)
					xx_rpt(FAILURE);
				in_rpt("fopen(\"%s\", \"w\") on error EMFILE created file", t24_file);
			}
			else
				PATH_TRACE;

			if(!fail)
				PATH_XS_RPT(6);
		}
	}

	DBUG_VOID_RETURN;
}


private void
test25()
{
	char		**longname;
	FILE		*fp;
	int		err1, err2;
	int		fail = 0;
	int	 	pathok = 0; 

	DBUG_ENTER("test25");

	globok = 0;
	longname = pathtoolng(t25_file);
	PATH_FUNC_TRACE;
	
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;

	}

	errno = NOERROR;
	if ((fp = fopen(longname[0], "w")) == (FILE *) NULL)
	{
		fail |= 1;
		err1 = errno;
	}
	else
		PATH_TRACE;
		
	(void) unlink(t25_file);

	if (longname[1] != NULL)
	{
		errno = NOERROR;
		if ((fp = fopen(longname[1], "w")) != (FILE *) NULL)
		{
			fail |= 2;
			(void) fclose(fp);
		}
		else if (errno != ENAMETOOLONG)
		{
			fail |= 4;
			err2 = errno;
		}
		else
			PATH_TRACE; 
	}
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
		{
			in_rpt("fopen(<%d char path>, \"w\") failed - errno %d (%s)",
				strlen(longname[0]), err1, errname(err1));
		}
		if (fail & 2)
		{
			in_rpt("fopen(<%d char path>, \"w\") did not return NULL",
				strlen(longname[1]));
		}
		if (fail & 4)
		{
			in_rpt("fopen(<%d char path>, \"w\") did not set errno correctly",
				strlen(longname[1]));
			in_rpt("expected %d (ENAMETOOLONG), received %d (%s)",
				ENAMETOOLONG, err2, errname(err2));
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}
	clntoolng();
	(void) unlink(t25_file);
	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test26()
{
	DBUG_ENTER("test26");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test26()
{
	char		**longname;
	FILE		*fp;
	int		err1, err2;
	int		fail = 0;
	int 		pathok = 0; 

	DBUG_ENTER("test26");


	globok = 0;
	if ((longname = nametoolng()) == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((fp = fopen(longname[0], "w")) == (FILE *) NULL)
	{
		fail |= 1;
		err1 = errno;
	}
	else
		PATH_TRACE;
		
	if (longname[1] != NULL)
	{
		errno = NOERROR;
		if ((fp = fopen(longname[1], "w")) != (FILE *) NULL)
		{
			fail |= 2;
			(void) fclose(fp);
		}
		else if (errno != ENAMETOOLONG)
		{
			fail |= 4;
			err2 = errno;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;
		

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
		{
			in_rpt("fopen(<%d char filename>, \"w\") failed - errno %d (%s)",
				strlen(longname[0]), err1, errname(err1));
		}
		if (fail & 2)
		{
			in_rpt("fopen(<%d char filename>, \"w\") did not return NULL",
				strlen(longname[1]));
		}
		if (fail & 4)
		{
			in_rpt("fopen(<%d char filename>, \"w\") did not set errno correctly",
				strlen(longname[1]));
			in_rpt("expected %d (ENAMETOOLONG), received %d (%s)",
				ENAMETOOLONG, err2, errname(err2));
		}
	}
	else
	{
		PATH_XS_RPT(3);
	}

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test27()
{
	DBUG_ENTER("test27");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */


private void
test27()
{
	int	err;
	int	pathok = 0; 

	DBUG_ENTER("test27");
	if (mkfifo(t27_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(\"%s\", 0%o) failed - errno %d (%s)", t27_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) cppair(ch_t27, NULLFN, 2*WAITTIME, 0);

	(void) unlink(t27_file);

	DBUG_VOID_RETURN;
}

private void
ch_t27()
{

	FILE *fp;
	int fail = 0;
	int err;
	int pathok = 0;

	DBUG_ENTER("ch_t27");

	SET_TIMEOUT(WAITTIME)
	errno = NOERROR;
	fp = fopen(t27_file, "r");
	err = errno;
	CLEAR_ALARM

	if (fp != NULL)
	{
		fail = 1;
		(void) fclose(fp);
	}
	else if (err != EINTR)
		fail = 2;
	else
		PATH_TRACE; 

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("fopen(FIFO, \"r\") did not return NULL when interrupted");
		if (fail & 2)
		{
			in_rpt("fopen(FIFO, \"r\") did not set errno correctly when interrupted");
			in_rpt("expected %d (EINTR), received %d (%s)",
				EINTR, err, errname(err));
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test28()
{
	int		fd;
	int		err;
	int 		pathok = 0; 

	DBUG_ENTER("test28");
	if ((fd = creat(t28_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not creat() %s - errno %d (%s)", t28_file, err,
			errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	(void) cppair(ch_t28, NULLFN, 2 * WAITTIME, 0);
	(void) unlink(t28_file);


	DBUG_VOID_RETURN;
}

private	void
ch_t28()
{
	FILE	*fp;
	long	open_max, count, loop_end;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t28");

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0)
	{
		if (errno == 0)
		{
			PATH_TRACE;
			open_max = PCTS_OPEN_MAX+1;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: sysconf(_SC_OPEN_MAX) failed");
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;
		
	loop_end = FOPEN_MAX - open_max + n_open_fds();
	if (loop_end < 0)
		loop_end = 0;

	if ((fp = fopen(t28_file, "w")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("initial fopen(\"%s\", \"w\") call failed - errno %d (%s)",
			t28_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* make sure we won't run out of fd's instead of streams */

	for (count = 1; count <= loop_end; count++)
	{
		DBUG_PRINTF("ch_t28", "fdopen() call, count = %ld", count);
		if (fdopen(fileno(fp), "w") == NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fdopen(fileno(fp), \"w\") failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE; 
	}

	/* No way of knowing how many streams were originally open.  All we
	   can test is that we can't open FOPEN_MAX-1 _more_ streams (there
	   were at least two originally: stdout and stderr) */

	for ( ; count < FOPEN_MAX; count++)
	{
		DBUG_PRINTF("ch_t28", "fopen() call, count = %ld", count);
		errno = NOERROR;
		if (fopen(t28_file, "w") == NULL)
		{
			if (errno == EMFILE)
			{
				PATH_XS_RPT(2+loop_end); 
				DBUG_VOID_RETURN;
			}
			else
			{
				err = errno;
				xx_rpt(FAILURE);
				in_rpt("fopen(\"%s\", \"w\") did not set errno correctly",
					t28_file);
				in_rpt("expected %d (EMFILE), received %d (%s)",
					EMFILE, err, errname(err));
				DBUG_VOID_RETURN;
			}
		}
	}

	/* FOPEN_MAX is really a "minimum guaranteed" number of streams.
	   So exceeding it is not a failure, but give a WARNING as this
	   is useful info. */

	xx_rpt(WARNING);
	in_rpt("fopen() succeeded when FOPEN_MAX streams were already open");

	DBUG_VOID_RETURN;
}

private void
test29()
{
	int 	fd;
	int 	err;
	int	pathok = 0; 

	DBUG_ENTER("test29");

	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}

	(void) umask((mode_t) 0);

	if ((fd = creat(t29_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("could not creat() %s - errno %d (%s)", t29_file, err,
			errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) close(fd);

	(void) cppair(ch_t29, NULLFN, 8 * WAITTIME, 0);

	(void) unlink(t29_file);

	DBUG_VOID_RETURN;
}

private void
ch_t29()
{
	char	*suid, *sgid;
	int 	err;
	char	*args[8];
	int 	pathok = 0; 

	DBUG_ENTER("ch_t29");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		exit(A_NOMEM);
	else
		PATH_TRACE;
		
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);
	else
		PATH_TRACE;
		
	args[0] = prog29;
	args[1] = t29_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = "f";
	args[5] = vg_ident;
	args[6] = vg_dbug;
	args[7] = NULL;

	(void) tet_exec(execpath(prog29), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %d (%s)", err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test30()
{

	DBUG_ENTER("test30");

	(void) cppair(ch_t30, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t30()
{
	int 	err;
	char	*args[5];

	DBUG_ENTER("ch_t30");

	args[0] = prog30;
	args[1] = t30_dir;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;

	(void) tet_exec(execpath(prog30), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}

private FILE *
tb31_32fopen(tbpath, tboflag)
char *tbpath;
char *tboflag;
{
	char	*reppath = tbpath;
	FILE 	*rval;
	int	err;
	char	pathbuf[30];
	int	pathok = 0;

	DBUG_LENTER("tb31_32fopen", 2);

	errno = NOERROR;
	rval = fopen(tbpath, tboflag);

#ifdef TESTBED
	rval = NULL ;
	errno++;
#endif
	err = errno;

	if(rval == NULL) {
		if (t31_32fail++ == 0)
			xx_rpt(FAILURE);

		/* Don't print PATH_MAX length path name */
		if (strlen(tbpath) > 99)
		{
			(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
			reppath = pathbuf;
		}

		in_rpt("fopen(\"%s\", %s) returned NULL. Expected file pointer",
			reppath, tboflag);
		in_rpt("errno was %d (%s)", err, errname(err));
		DBUG_RETURN(rval);
	} else {
		PATH_TRACE;
	}


	PATH_FUNC_RPT(1);

	DBUG_RETURN(rval);
}

private void
test31()
{
	/* pathname resolution */

	char 	*p1, *p2;
	int 	npairs, i;

	FILE	 *fd1, *fd2;
	struct stat stbuf1, stbuf2;
	int	pathok = 0;

	DBUG_ENTER("test31");

	t31_32fail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_CREAT);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_CREAT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test31", "testing \"%s\" == \"%s\"", p1, p2);

		globok = 0;
		fd1 = tb31_32fopen(p1, "r");
		PATH_FUNC_TRACE;

		globok = 0;
		fd2 = tb31_32fopen(p2, "r");
		PATH_FUNC_TRACE;

		if ( (fd1 == NULL) || (fd2 == NULL) )
		{
			(void) fclose(fd1);
			(void) fclose(fd2);
			continue;
		}

		if (fstat(fileno(fd1), &stbuf1) == SYSERROR ||
		    fstat(fileno(fd2), &stbuf2) == SYSERROR)
		{
			if (t31_32fail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: fstat() failed");
		}
		else if (stbuf1.st_dev != stbuf2.st_dev ||
		         stbuf1.st_ino != stbuf2.st_ino)
		{
			if (t31_32fail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("fopen() did not resolve \"%s\" and \"%s\" to the same file",
				p1, p2);
				
		}
		else
			PATH_TRACE;

#ifdef TESTBED2
#undef fd1
#endif

		(void) fclose(fd1);
		(void) fclose(fd2);
	}

	if (t31_32fail == 0)
		PATH_XS_RPT(1+(3*npairs));
	
	cln_pathres();

	DBUG_VOID_RETURN;
}


#ifdef TESTBED3
#undef	BUFFER_ELEMENTS
#define	BUFFER_ELEMENTS 1
#endif



private void
test32()
{
	/* portable filenames */

	char 	**p;
	FILE	*fd;
	int	i, err, nbuf;
	struct	{
		struct	stat s;
		char	*p;
		} buf[ BUFFER_ELEMENTS ]; 
	int	pathok = 0;

	DBUG_ENTER("test32");

	t31_32fail = 0;

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{	
		if (i++ == 0)
			PATH_TRACE;
		if (eaccess(*p, F_OK) == 0 && remove(*p) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("could not remove \"%s\" - errno %d (%s)",
				*p, err, errname(err));
			DBUG_VOID_RETURN;
		}
	}

#ifdef TESTBED2
	(void) close(crfile(portfnam[0], (mode_t)MODEANY, (int)getuid(), (int)getgid()));
#endif

	nbuf = -1;
	for (p = portfnam; *p != NULL; p++)
	{
		globok = 0;
		fd = tb31_32fopen(*p, "w");
		if (fd != NULL) {
			nbuf++;
			if (nbuf > BUFFER_ELEMENTS-1) {
				if (t31_32fail++ == 0)
					xx_rpt(DELETION);
				in_rpt("portfnam too big, or buf too small");
				DBUG_VOID_RETURN;
			}

			if(fstat(fileno(fd), &(buf[nbuf].s) ) == SYSERROR) {
				if (t31_32fail++ == 0)
					xx_rpt(DELETION);
				in_rpt("deletion reason: fstat() failed");
				nbuf--; /* Failed! Not counted */
			} else {
				buf[nbuf].p = *p;
				DBUG_PRINTF("fstat", "%s %d %d", buf[nbuf].p,
						buf[nbuf].s.st_dev,
						buf[nbuf].s.st_ino);

				if (nbuf == 0)
					PATH_FUNC_TRACE;
			}
		
			(void) fclose(fd);
		}
	}

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		(void) unlink(*p);
	}

	for (i=0; i<nbuf; i++) {
		int j;
		for (j=i+1; j<nbuf; j++) {
			if (buf[i].s.st_dev == buf[j].s.st_dev &&
			    buf[i].s.st_ino == buf[j].s.st_ino)
			{
				if (t31_32fail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("fopen() resolved \"%s\" and \"%s\" to the same file",
					buf[i].p, buf[j].p);
					
			}
		}
	}

	if (t31_32fail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
test33()
{
	/* fopen() on existing file does not change mode, ownership,
	   or st_atime. */

	int		fd;
	int		err;

	DBUG_ENTER("test33");
	if ((fd = open(t33_file, O_WRONLY|O_CREAT, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_CREAT, %#o) failed - errno %d (%s)",
			t33_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	(void) close(fd);

	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t33, NULLFN, WAITTIME, 0);

	(void) unlink(t33_file);

	DBUG_VOID_RETURN;
}

private void
ch_t33()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t33");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		exit(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog33;
	args[1] = t33_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog33), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog33), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test34()
{
	/* fopen returns the lowest available file descriptor */

	int 	i, j, err;
	FILE	*fp[T34_NFILES+1];
	int	fd[T34_NFILES+1];
	int	pathok = 0;

	DBUG_ENTER("test34");

	testfail = 0;

	for (i = 0 ; i < T34_NFILES && testfail == 0 ; i++)
	{
		if (i==0)
			PATH_TRACE;
		if((fp[i] = fopen(t34_file, "w")) == (FILE *)NULL) {
			xx_rpt(DELETION);
			in_rpt("fopen() returned NULL");
			for (j = 0; j < i; j++)
				(void) fclose(fp[j]);
			(void) unlink(t34_file);
			DBUG_VOID_RETURN;
		}
		if((fd[i] = fileno(fp[i])) == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fileno() failed - errno %d(%s)",
				err, errname(err));
			for (j = 0; j < i; j++)
				(void) fclose(fp[j]);
			(void) unlink(t34_file);
			DBUG_VOID_RETURN;
		}
	}

	for (i = 1; i < T34_NFILES && testfail == 0; i++)
	{
		if (i==1)
			PATH_TRACE;
		if (fd[i-1] >= fd[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("fopen() did not return lowest numbered file descriptor");
		}
	}

	if (testfail == 0)
	{
		if (fclose(fp[0]) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("fclose() failed - errno %d (%s)",
				err, errname(err));
		}
		else {
			if((fp[T34_NFILES] = fopen(t34_file, "w")) == (FILE *)NULL) {
				xx_rpt(DELETION);
				in_rpt("fopen() returned NULL");
				for (i = 1; i < T34_NFILES; i++)
					(void) fclose(fp[i]);
				(void) unlink(t34_file);
				DBUG_VOID_RETURN;
			}
			if((fd[T34_NFILES] = fileno(fp[T34_NFILES])) == SYSERROR) {
				err = errno;
				xx_rpt(DELETION);
				in_rpt("deletion reason: fileno() failed - errno %d(%s)",
					err, errname(err));
				for (i = 1; i < T34_NFILES+1; i++)
					(void) fclose(fp[i]);
				(void) unlink(t34_file);
				DBUG_VOID_RETURN;
			}
	
			if(fd[T34_NFILES] != fd[0])
			{
				xx_rpt(FAILURE);
				in_rpt("fopen() did not re-use closed file descriptor");
			}
			else
			{
				PATH_TRACE;
				PATH_XS_RPT(3);
			}

			(void) fclose(fp[T34_NFILES]);
		}
	}

	for (i = 1; i < T34_NFILES; i++)
		(void) fclose(fp[i]);
	(void) unlink(t34_file);

	DBUG_VOID_RETURN;
}

private void
test35()
{
	/* opened files are set to remain open across exec calls */

	FILE	*fp;
	int 	fd, err, flags;
	int	pathok = 0;

	DBUG_ENTER("test35");

	if((fp = fopen(t35_file, "w")) == (FILE *)NULL) {
		xx_rpt(DELETION);
		in_rpt("fopen() returned NULL");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if((fd = fileno(fp)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fileno() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ((flags = fcntl(fd, F_GETFD, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl(F_GETFD) failed - errno %d (%s)",
			err, errname(err));
	}
	else if ((flags & FD_CLOEXEC) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("FD_CLOEXEC was set for opened file");
	}
	else
	{
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	(void) unlink(t35_file);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
test36_37()
{
	/* 
	   Test replaces part of test2 when testing XNFS

	   test36: server does not perform user ID mapping
	   test37: server performs user ID mapping
	*/

	char	*file;

	DBUG_ENTER("test36_37");

	switch (tet_thistest) {
	case 36:
		file = t36_file;
		break;
	case 37:
		file = t37_file;
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected tet_thistest = %d",
			tet_thistest);
		DBUG_VOID_RETURN;
	}

	if (eaccess(file, F_OK) == 0 && unlink(file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", file);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t36_37, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t36_37()
{
	char	*file, *prog, *fuid, *suid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t36_37");

	switch (tet_thistest) {
	case 36:
		if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
			exit(A_NOMEM);
		fuid = suid;
		file = t36_file;
		prog = prog36;
		break;
	case 37:
		suid = "0";
		if ((fuid = newstr(uidtoa(xnfs_anon_uid))) == (char *) NULL)
			exit(A_NOMEM);
		file = t37_file;
		prog = prog37;
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected tet_thistest = %d",
			tet_thistest);
		DBUG_VOID_RETURN;
	}

	args[0] = prog;
	args[1] = file;
	args[2] = suid;
	args[3] = fuid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog, err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test36_37()
{
	DBUG_ENTER("test36_37");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test38()
{
	DBUG_ENTER("test38");

	(void) cppair(ch_t38, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t38_mnt);

	DBUG_VOID_RETURN;
}

private void
ch_t38()
{
	char	*fsys, *sgid;
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t38");

	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	if ((fsys = tet_getvar("VSX_MOUNT_DEV")) == NULL || *fsys == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: VSX_MOUNT_DEV is not defined");
		DBUG_VOID_RETURN;
	}

	if (crdir(t38_mnt, MODEANY, (int) getuid(), (int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t38_mnt);
		DBUG_VOID_RETURN;
	}

	args[0] = prog38;
	args[1] = sgid;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog38), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog38, err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test38()
{
	DBUG_ENTER("test38");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test39()
{
	DBUG_ENTER("test39");

	(void) cppair(ch_t39, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t39()
{
	char	*sgid;
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t39");

	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	args[0] = prog39;
	args[1] = sgid;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog39), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog39, err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test39()
{
	DBUG_ENTER("test39");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test40()
{
	DBUG_ENTER("test40");

	if (eaccess(t40_file, F_OK) == 0 && unlink(t40_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t40_file);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t40, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

#define T40_INSTANCES	2

private void
ch_t40()
{
	FILE		*fp;
	int		fail = 0;
	mode_t		mode1;
	mode_t		mode2;
	struct stat	statb;
	int		i, flag[T40_INSTANCES];
	int		err;
	int 		pathok = 0;

	DBUG_ENTER("ch_t40");

	(void) unlink(t40_file);

	if (stat(".", &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\".\") failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for ( i = 0; i < T40_INSTANCES; i++ )
		flag[i] = 0;
		
	if ((fp = fopen(t40_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
			t40_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t40_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(%s) failed - errno %d(%s)",
			t40_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((mode1 = (statb.st_mode & MODEMASK)) != DEFMODE)
		flag[0] |= 1;

	if ( flag[0] != 0 )
		fail = 1;
	else
		PATH_TRACE;
		
	(void) fclose(fp);
	(void) unlink(t40_file);
	(void) umask((mode_t)T40_UMASK);

	if ((fp = fopen(t40_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
			t40_file, err, errname(err));
		(void) umask((mode_t)0);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) umask((mode_t)0);
	if (stat(t40_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(%s) failed - errno %d(%s)",
			t40_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fclose(fp);
		
	if ((mode2 = (statb.st_mode & MODEMASK)) != (DEFMODE & ~T40_UMASK))
		flag[1] |= 1;

	if ( flag[1] != 0 )
		fail |= 2;
	else
		PATH_TRACE;
		
	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
		{
			if (flag[0] & 020)
				in_rpt("fopen() did not creat file with mode %#o when umask was set to 0, found %#lo",
				DEFMODE, (long) mode1);
		}
		else if (fail & 2)
		{
			if (flag[1] & 020)
			{
				in_rpt("fopen() did not creat file with proper mask when umask was set to %#o", T40_UMASK);
				in_rpt("expected %#o, found %#lo", 
				DEFMODE & ~T40_UMASK, (long) mode2);
			}
		}
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(8);
	}

	(void) unlink(t40_file);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test40()
{
	DBUG_ENTER("test40");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test41()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test41");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbfopen(longname, "w", SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test41()
{
	DBUG_ENTER("test41");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test42()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test42");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbfopen(longname, "w", SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test42()
{
	DBUG_ENTER("test42");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test43()
{
	char	*longname;
	FILE	*fd;
	int	pathok = 0;

	DBUG_ENTER("test43");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	fd = tbfopen(longname, "w", SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd != NULL)
		(void) fclose(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test43()
{
	DBUG_ENTER("test43");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test44()
{
	char	*longname;
	FILE	*fd;
	int	pathok = 0;

	DBUG_ENTER("test44");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	fd = tbfopen(longname, "w", SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd != NULL)
		(void) fclose(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test44()
{
	DBUG_ENTER("test44");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private FILE *
tbfopen(tbpath, tbmode, expret, experrno)
char *tbpath;
char *tbmode;
int expret;
int experrno;
{
	char	*reppath = tbpath;
	int	err;
	FILE	*fd;
	char	pathbuf[30];
	int	pathok = 0;

	DBUG_LENTER("tbfopen", 2);

	errno = NOERROR;
	fd = fopen(tbpath, tbmode);
	err = errno;

	if (expret == SUCCEED)
	{
		PATH_TRACE;
		if (fd != NULL)
		{
			PATH_FUNC_RPT(1);
			DBUG_RETURN(fd);
		}
	}
	else
	{
		PATH_TRACE;
		if (fd == NULL && experrno == err)
		{
			if(eaccess(tbpath, F_OK) == SYSERROR)
			{
				PATH_FUNC_RPT(1);
				DBUG_RETURN(fd);
			}
			else
			{
				in_rpt("on error file was created");
			}
		}
	}

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);

	if( (expret != SUCCEED) && (fd != NULL) )
	{
		(void) fclose(fd);
		in_rpt("fopen(%s, \"w\") did not return NULL", reppath);
	}
	else if( (expret == SUCCEED) && (fd == NULL ) )
	{
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)", reppath, err, errname(err));
		
	}
	else if(experrno != err ) 
	{
		in_rpt("errno set incorrectly - expected %d (%s), received %d (%s)",
			ENAMETOOLONG, errname(experrno), err, errname(err));
	}

	DBUG_RETURN(fd);
}

#endif /* TEST_XNFS */
