/*
*      SCCS:  @(#)  ANSI.os/streamio/fopen/fopen1.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.fopen
PROJECT:	VSX

DESCRIPTION:
	Testset for the fopen() system call XPG4 extensions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		K. Holder
DATE CREATED:	Tue Sep 11 11:13:26 BST 1990
MODIFICATIONS:	
	Stuart Boutell, 30 Oct 1990
		Integrated into VSX4

		Neil Moses, 11 June 1992.
			Tests 10, 11, 13, 14 - check time stamps against
			current time.
		
		Geoff Clare, 16 June 1992
			Fix XPG3 mode build problems.

		Dave Cater, 27 October 1992
			Make testfail extern - storage defined in fopen.c

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<time.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<chktimes.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"fopen_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fopen
#endif



#define WAITTIME	(5 * SPEEDFACTOR)

#define	T_NBYTES	10

#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

public	void	test9();
public	void	test10();
public	void	test11();
public	void	test12();
public	void	test13();
public	void	test14();

extern	int	testfail;
extern	char	*errname();

#if	!defined(_XOPEN_SOURCE) || TEST_XPG > 3

extern	void	savtimes();
extern	char 	*chktimes();
extern	void	fsavtimes();

private char	*t9_file =		"./fopen-t.9";
private char	*t10_file =		"./fopen-t.10";
private char	*t11_file =		"./fopen-t.11";
private char	*t12_file =		"./fopen-t.12";
private char	*t13_file =		"./fopen-t.13";
private char	*t14_file =		"./fopen-t.14";

public	void	chktms();

private	char	*msg,*msg1;


public	void
test9()
{
	int		fd;
	FILE		*fp;
	int		err;
	int		flags;
	int 		pathok = 0; 

	DBUG_ENTER("test9");

	testfail = 0;

	if ((fd = creat(t9_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t9_file, (long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	if ((fp = fopen(t9_file, "rb")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"rb\") failed to open file - errno %d(%s)",
			t9_file, err, errname(err));
		(void) unlink(t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDONLY)
	{
		xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"rb\") did not open file for read-only");
	}
	else
		PATH_XS_RPT(3);

	(void) fclose(fp);
	(void) unlink(t9_file);
	DBUG_VOID_RETURN;
}


public	void
test10()
{
	int		fd;
        time_t  	t0;
	FILE		*fp;
	int		err;
	int		flags;
	char		buf[T_NBYTES];
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test10");

	testfail = 0;

	if ((fd = creat(t10_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t10_file, (long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	savtimes(t10_file);

	errno = NOERROR;
	if ((fp = fopen(t10_file, "wb")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb\") failed to open file - errno %d(%s)",
			t10_file, err, errname(err));
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_WRONLY) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb\") did not open file for write-only",
				t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t10_file, &statb) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t10_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (statb.st_size != 0L){
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb\") did not truncate file", t10_file);
		(void) fclose(fp);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ( (msg = chktimes(t10_file,NEWCTIME|NEWMTIME)) != NULL ) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s,\"wb\") %s", t10_file,msg);
		(void) fclose(fp);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) fclose(fp);
	(void) unlink(t10_file);

	savtimes(".");

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T10_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = fopen(t10_file, "wb")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb\") failed to open file that did not exist - errno %d(%s)",
			t10_file, err, errname(err));
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) !=  O_WRONLY ) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb\") did not open file for write-only when file did not exist",
				t10_file);
		(void) fclose(fp);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t10_file, &statb) == SYSERROR) {
		err = errno;
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb\") did not create file",
				t10_file);
		(void) fclose(fp);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (statb.st_size != 0L) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb\") did not create file with zero size",
				t10_file);
		(void) fclose(fp);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ( (msg1 = chktimes(".",NEWCTIME|NEWMTIME)) != NULL ) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s,\"wb\") - parent directory: %s", t10_file,msg1);
		(void) fclose(fp);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	chktms(t10_file, t0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(16);

	(void) fclose(fp);
	(void) unlink(t10_file);
	DBUG_VOID_RETURN;
}


public	void
test11()
{
	int		fd;
	FILE		*fp;
        time_t  	t0;
	int		err;
	int		flags;
	long		size;
	char		buf[T_NBYTES];
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test11");

	testfail = 0;

	if ((fd = creat(t11_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t11_file, (long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	if ((fp = fopen(t11_file, "ab")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab\") failed to open file - errno %d(%s)",
			t11_file, err, errname(err));
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_WRONLY ) 
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab\") did not open file for write-only",
				t11_file);
		(void) fclose(fp);
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!(flags & O_APPEND)) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab\") did not open file for append",
				t11_file);
		(void) fclose(fp);
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(t11_file, &statb) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t11_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((size = statb.st_size) != (long) T_NBYTES)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab\") changed size of file - expected %d, found %ld",
			t11_file, T_NBYTES, size);
		(void) fclose(fp);
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fclose(fp);
	(void) unlink(t11_file);

	savtimes(".");

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T11_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = fopen(t11_file, "ab")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab\") failed to open file that did not exist - errno %d(%s)",
			t11_file, err, errname(err));
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_WRONLY)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab\") when file did not exist, did not open file for write-only",
			t11_file);
		(void) fclose(fp);
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!(flags & O_APPEND))
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab\") when file did not exist, did not open file for append",
			t11_file);
		(void) fclose(fp);
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(t11_file, &statb) == SYSERROR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab\") when file did not exist, did not creat file",
			t11_file);
		(void) fclose(fp);
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statb.st_size != 0L)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab\") when file did not exist, did not creat file with zero length",
				t11_file);
		(void) fclose(fp);
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( (msg = chktimes(".",NEWCTIME|NEWMTIME)) != NULL ) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab\") - parent directory: %s",
			t11_file,msg);
		(void) fclose(fp);
		(void) unlink(t11_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	chktms(t11_file, t0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(17);

	(void) fclose(fp);
	(void) unlink(t11_file);
	DBUG_VOID_RETURN;
}


public	void
test12()
{
	int		fd;
	FILE		*fp;
	int		err;
	int		flags;
	char		buf[T_NBYTES];
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test12");

	testfail = 0;

	if ((fd = creat(t12_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t12_file, (long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) unlink(t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	if ((fp = fopen(t12_file, "rb+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"rb+\") failed to open file - errno %d(%s)",
			t12_file, err, errname(err));
		(void) unlink(t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t12_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t12_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ( ((flags & O_ACCMODE) != O_RDWR) || statb.st_size != T_NBYTES)
	{
		xx_rpt(FAILURE);
		if ((flags & O_ACCMODE) != O_RDWR)
			in_rpt("fopen(%s, \"rb+\") did not open file for read/write",
				t12_file);
		if (statb.st_size != T_NBYTES)
			in_rpt("fopen(%s, \"rb+\") changed size of file, expected size %d, found %ld",
				t12_file,T_NBYTES, statb.st_size);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE; 

	(void) fclose(fp);
		
	if ((fp = fopen(t12_file, "r+b")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"r+b\") failed to open file - errno %d(%s)",
			t12_file, err, errname(err));
		(void) unlink(t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t12_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t12_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (((flags & O_ACCMODE) != O_RDWR) || statb.st_size != T_NBYTES)
	{
		xx_rpt(FAILURE);
		if ((flags & O_ACCMODE) != O_RDWR)
			in_rpt("fopen(%s, \"r+b\") did not open file for read-write",t12_file);
		if (statb.st_size != T_NBYTES)
			in_rpt("fopen(%s, \"r+b\") changed size of file, expected size %d, found %ld",
				t12_file,T_NBYTES, statb.st_size);
	}
	else
		PATH_XS_RPT(9);

	(void) fclose(fp);
	(void) unlink(t12_file);
	DBUG_VOID_RETURN;
}


public	void
test13()
{
	int		fd;
	FILE		*fp;
        time_t  	t0;
	int		err;
	int		flags;
	char		buf[T_NBYTES];
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test13");

	testfail = 0;

	if ((fd = creat(t13_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t13_file, (long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	savtimes(t13_file);

	errno = NOERROR;
	if ((fp = fopen(t13_file, "wb+")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb+\") failed to open file - errno %d(%s)",
			t13_file, err, errname(err));
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb+\") did not open file for read/write",
			t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t13_file, &statb) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t13_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (statb.st_size != 0L)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb+\") did not truncate file", t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ( (msg = chktimes(t13_file,NEWCTIME|NEWMTIME)) != NULL ) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb+\") %s", t13_file,msg);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) fclose(fp);
	(void) unlink(t13_file);

	savtimes(".");

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T13_1_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = fopen(t13_file, "wb+")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb+\") failed to open file that did not exist - errno %d(%s)",
			t13_file, err, errname(err));
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb+\") did not open file for read/write when file did not exist",
			t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t13_file, &statb) == SYSERROR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb+\") did not create file", t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statb.st_size != 0L)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb+\") did not create file with zero size",
			t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ( (msg1 = chktimes(".",NEWCTIME|NEWMTIME)) != NULL ) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"wb+\") - parent directory: %s",t13_file,msg1);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	chktms(t13_file, t0);
	PATH_FUNC_TRACE;

	(void) fclose(fp);
	(void) unlink(t13_file);

	if ((fd = creat(t13_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t13_file, (long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	savtimes(t13_file);

	errno = NOERROR;
	if ((fp = fopen(t13_file, "w+b")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+b\") failed to open file - errno %d(%s)",
			t13_file, err, errname(err));
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+b\") did not open file for read/write",
			t13_file);
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t13_file, &statb) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t13_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (statb.st_size != 0L)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+b\") did not truncate file", t13_file);
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ( (msg = chktimes(t13_file,NEWCTIME|NEWMTIME)) != NULL ) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+b\") %s", t13_file,msg);
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) fclose(fp);
	(void) unlink(t13_file);

	savtimes(".");

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T13_2_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = fopen(t13_file, "w+b")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+b\") failed to open file that did not exist - errno %d(%s)",
			t13_file, err, errname(err));
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+b\") did not open file for read/write when file did not exist",
			t13_file);
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t13_file, &statb) == SYSERROR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+b\") did not create file", t13_file);
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statb.st_size != 0L)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+b\") did not create file with zero size",
			t13_file);
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ( (msg1 = chktimes(".",NEWCTIME|NEWMTIME)) != NULL ) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"w+b\") - parent directory: %s",t13_file,msg1);
		(void) fclose(fp);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	chktms(t13_file, t0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(32);

	(void) fclose(fp);
	(void) unlink(t13_file);
	DBUG_VOID_RETURN;
}


public	void
test14()
{
	int		fd;
        time_t  	t0;
	FILE		*fp;
	int		err;
	int		flags;
	long		size;
	char		buf[T_NBYTES];
	struct stat	statb;
	int 		pathok = 0; 

	DBUG_ENTER("test14");

	testfail = 0;

	if ((fd = creat(t14_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t14_file, (long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) close(fd);
	if ((fp = fopen(t14_file, "ab+")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab+\") failed to open file - errno %d(%s)",
			t14_file, err, errname(err));
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab+\") did not open file for read/write",
			t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (!(flags & O_APPEND))
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab+\") did not open file for append", t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t14_file, &statb) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t14_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((size = statb.st_size) != (long) T_NBYTES)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab+\") changed size of file - expected %d, found %ld",
			t14_file, T_NBYTES, size);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) fclose(fp);
	(void) unlink(t14_file);

	savtimes(".");

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T14_1_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = fopen(t14_file, "ab+")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab+\") failed to open file that did not exist - errno %d(%s)",
			t14_file, err, errname(err));
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab+\") when file did not exist, did not open file for read/write",
			t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (!(flags & O_APPEND))
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab+\") when file did not exist, did not open file for append",
			t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t14_file, &statb) == SYSERROR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab+\") when file did not exist, did not creat file",
			t14_file);
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;
	if (statb.st_size != 0L)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab+\") when file did not exist, did not creat file with zero length",
			t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ( (msg = chktimes(".",NEWCTIME|NEWMTIME)) != NULL ) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"ab+\") - parent directory: %s",t14_file,msg);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	chktms(t14_file, t0);
	PATH_FUNC_TRACE;

	(void) fclose(fp);
	(void) unlink(t14_file);

	if ((fd = creat(t14_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t14_file, (long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) close(fd);
	if ((fp = fopen(t14_file, "a+b")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+b\") failed to open file - errno %d(%s)",
			t14_file, err, errname(err));
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+b\") did not open file for read/write",
			t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (!(flags & O_APPEND))
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+b\") did not open file for append", t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t14_file, &statb) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t14_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((size = statb.st_size) != (long) T_NBYTES)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+b\") changed size of file - expected %d, found %ld",
			t14_file, T_NBYTES, size);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) fclose(fp);
	(void) unlink(t14_file);

	savtimes(".");

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T14_2_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = fopen(t14_file, "a+b")) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+b\") failed to open file that did not exist - errno %d(%s)",
			t14_file, err, errname(err));
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+b\") when file did not exist, did not open file for read/write",
			t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (!(flags & O_APPEND))
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+b\") when file did not exist, did not open file for append",
			t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t14_file, &statb) == SYSERROR)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+b\") when file did not exist, did not creat file",
			t14_file);
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;
	if (statb.st_size != 0L)
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+b\") when file did not exist, did not creat file with zero length",
			t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ( (msg = chktimes(".",NEWCTIME|NEWMTIME)) != NULL ) {
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"a+b\") - parent directory: %s",t14_file,msg);
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	globok = 0;
	chktms(t14_file, t0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(34);

	(void) fclose(fp);
	(void) unlink(t14_file);
	DBUG_VOID_RETURN;
}

#else /* TEST_XPG is not greater than 3 */

public	void
test9()
{
	DBUG_ENTER("test9");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

public	void
test10()
{
	DBUG_ENTER("test10");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

public	void
test11()
{
	DBUG_ENTER("test11");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

public	void
test12()
{
	DBUG_ENTER("test12");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

public	void
test13()
{
	DBUG_ENTER("test13");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

public	void
test14()
{
	DBUG_ENTER("test14");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif


public void
chktms(file, t0)
char	*file;
time_t	t0;
{
	struct	stat stbuf;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("chktms");

	if (stat(file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat() failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf.st_atime < t0)
	{
		if(testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_atime field on %s was not marked for update", file);
		in_rpt("st_atime was %ld; should not be less than %ld", 
						stbuf.st_atime, t0);
	}
	else
		PATH_TRACE;

	if (stbuf.st_ctime < t0)
	{
		if(testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field on %s was not marked for update", file);
		in_rpt("st_ctime was %ld; should not be less than %ld", 
						stbuf.st_ctime, t0);
	}
	else
		PATH_TRACE;

	if (stbuf.st_mtime < t0)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field on %s was not marked for update", file);
		in_rpt("st_mtime was %ld; should not be less than %ld", 
						stbuf.st_mtime, t0);
	}
	else
		PATH_TRACE;
		
	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}
