/*
*      SCCS:  @(#)  ANSI.os/streamio/fopen/fopen_t33.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fopen - fopen_t33 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		fopen_t33 - used by T.fopen in test33()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.fopen in test33().  This should be setuid/setgid
	  to VSX_UID1/VSX_GID1.
	  usage:
		fopen_t33 file uid gid vg_ident vg_dbug

	  where "file" is the file to be fopen()ed.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	20 Mar 1989
MODIFICATIONS:

		Neil Moses, 11 June 1992
			Copied from creat(7).

************************************************************************/

#include <dbug.h>
#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <report.h>
#include <errno.h>
#include <xnfs.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef fopen
#endif


public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	*errname();
extern	void	set_pname();

private	void	subtest33();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	uid_t uid;
	gid_t gid;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if ((uid = atoi(argv[2])) != geteuid() ||
		(gid = atoi(argv[3])) != getegid())
	{
		xx_rpt(DELETION);
		if (uid != getuid())
			in_rpt("deletion reason: uid is not equal to VSX_UID1");
		else
			in_rpt("deletion reason: gid is not equal to VSX_GID1");
	}
	else
	{
		(void) setuid(uid);
		(void) setgid(gid);
		subtest33(argv[1]);
	}

	return(OK);
}


private void
subtest33(file)
char	*file;
{
	int		err;
	FILE		*fp;
	int		fail = 0;
	struct stat	stbuf1, stbuf2;
	int		pathok = 0;

	DBUG_ENTER("subtest33");

	if (stat(file, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed on %s - errno %d (%s)",
			file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) sleep(3);

	if ((fp = fopen(file, "w")) == (FILE *)NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen(\"%s\", \"w\") failed - errno %d(%s)",
				file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fclose(fp);

	if (stat(file, &stbuf2) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed on %s - errno %d (%s)",
			file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf1.st_uid != stbuf2.st_uid)
		fail |= 02;
	else
		PATH_TRACE;

	if (stbuf1.st_gid != stbuf2.st_gid)
		fail |= 04;
	else
		PATH_TRACE;

	if (stbuf1.st_mode != stbuf2.st_mode)
		fail |= 010;
	else
		PATH_TRACE;

	if (stbuf1.st_atime != stbuf2.st_atime)
		fail |= 020;
	else
		PATH_TRACE;

	if (fail != 0)
	{
		xx_rpt(FAILURE);
		if (fail & 02)
			in_rpt("fopen(\"%s\", \"w\") changed uid of existing file",
				file);
		if (fail & 04)
			in_rpt("fopen(\"%s\", \"w\") changed gid of existing file",
				file);
		if (fail & 010)
			in_rpt("fopen(\"%s\", \"w\") changed mode of existing file",
				file);
		if (fail & 020)
			in_rpt("fopen(\"%s\", \"w\") changed st_atime of existing file",
				file);
	}
	else
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}
