/*
 *      SCCS: @(#)  ANSI.os/streamio/fopen/fopen_t36_7.c Rel 4.4.1 (%G)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#define MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fopen/fopen_t36_7 Rel 4.4.1 (%G)";
#ifdef	MAIN
char	*copyright[] = {
		"(C) Copyright 1992 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		fopen_t36_7.c - used by T.fopen in test36() and test37()

PROJECT:	XNFS (X/OPEN NFS tests)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int tet_main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	Exec'd by T.fopen in test36() and test37().
	This source file generates several binaries.
	fopen_t36 should be setuid to VSX_UID1.
	fopen_t37 should be setuid to root.
	usage:
		fopen_tNN file myuid expuid vg_ident vg_dbug

	where:
		myuid is my effective uid
		expuid is the uid that we expect for the created file
		file is the file to be created with fopen().

AUTHOR:		John-Paul, UniSoft Ltd.

DATE CREATED:	12 October 1992

MODIFICATIONS:
		Geoff Clare, 8 Jan 1993
			Change atoi() to atouid().

************************************************************************/

#include <std.h>
#include <dbug.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include <xnfs.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef fopen
#endif


public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	uid_t	atouid();
extern	char	*errname();
extern	void	set_pname();

private void	subtest36_37();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	uid_t	myuid;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug + 2);
	}

	if ((myuid = atouid(argv[2])) != geteuid())
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: expected effective uid %ld, found %ld",
			(long)myuid, (long)geteuid());
	}
	else
	{
		subtest36_37(argv[1], atouid(argv[3]));
	}

	return(OK);
}


private void
subtest36_37(file, uid)
char	*file;
uid_t	uid;
{
	int		err;
	FILE		*fd;
	struct stat	stbuf;
	int		pathok = 0;

	DBUG_ENTER("subtest36_37");

	if ((fd = fopen(file, "w")) == NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen(\"%s\", \"w\") failed - errno %d (%s)", file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fclose(fd);

	if (stat(file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed on file %s  - errno %d (%s)", file, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		if (stbuf.st_uid != uid)
		{
			xx_rpt(FAILURE);
			in_rpt("fopen(\"%s\", \"w\") did not set uid correctly", file );
			in_rpt("expected uid %ld, found uid %ld",
				(long)uid, (long)stbuf.st_uid);
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(3);
		}
	}

	(void) unlink(file);

	DBUG_VOID_RETURN;
}

