/*
 *      SCCS:  @(#) ANSI.os/streamio/fopen/fopen_t38.c Rel 4.4.1 (06/09/97)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
static	char sccsid[] = "@(#)ANSI.os/streamio/fopen/fopen_t38.c Rel 4.4.1 (06/09/97)";
#endif	/* lint */

/***********************************************************************

NAME:		fopen_t38.c

PROJECT:	XNFS (X/OPEN NFS tests)

DESCRIPTION: 	setup routine for test38() subprogram

AUTHOR:		John-Paul, UniSoft Ltd.
DATE CREATED:	19 October 1992

MODIFICATIONS:
		Geoff Clare, 20 Oct 1993
			Delete subdirectory from T38_FILE.
			Use tet_fork() now that subtest does a setuid().

************************************************************************/

#include <std.h>
#include <dbug.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <report.h>
#include <tet_api.h>
#include <setprv.h>
#include <xnfs.h>
#include "fopen_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef fopen
#endif


#define WAITTIME	(5 * SPEEDFACTOR)

private char	*t38_mnt = 	T38_MNT;
private char	*t38_file = 	T38_FILE;
private gid_t	mygid;
private void	chfunc();

extern 	void	subtest3839();


public void
setup3839(gid)
gid_t gid;
{
	int pathok = 0;
	char *fsys;

	DBUG_ENTER("setup3839 for test 38");

	mygid = gid;

	if ((fsys = tet_getvar("VSX_MOUNT_DEV")) == NULL || *fsys == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: VSX_MOUNT_DEV not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (xnfs_mount(fsys, t38_mnt, XNFS_MOUNT_GRPID | XNFS_MOUNT_INTR) < 0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);
	(void) tet_fork(chfunc, NULLFN, 2*WAITTIME, 0);

	(void) xnfs_unmount(fsys, t38_mnt);

	DBUG_VOID_RETURN;
}

private void
chfunc()
{
	int pathok = 0;

	DBUG_ENTER("chfunc");

	PATH_FUNC_TRACE;
	subtest3839(t38_mnt, t38_file, mygid, (gid_t) -1, pathok, 1);

	DBUG_VOID_RETURN;
}
