/*
*      SCCS:  @(#)  ANSI.os/streamio/fopen_X/fopen_X.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fopen_X - T.fopen_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.fopen_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fopen() system call, X/Open extensions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	26 May 1989
MODIFICATIONS:	
		Geoff Clare, 22 Nov 1989
			Add EINVAL test

		Anthony Lee, 16 July 1990
			Added resillience code.

		Peter Skan, 30 Sept 1991
			set the PRV_DEVICE privilege
		
		Geoff Clare, 21 July 1992
			Change pchild from int to pid_t.

		Geoff Clare, 9 June 1995
			Allow VSX_NXIO_BLKDEV="unsup".

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<limits.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<timeout.h>
#include	<string.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fopen
#endif



#define VSX_NXIO_BLKDEV		"VSX_NXIO_BLKDEV"
#define VSX_NXIO_CHRDEV		"VSX_NXIO_CHRDEV"
#define NO_TESTS	3		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)


#define	NFOP_MODES	6

extern	char	*errname();


int	no_tests = NO_TESTS;

private pid_t	pchild = SYSERROR;
private	char	*fop_modes[NFOP_MODES] = { "r", "w", "a", "r+", "w+", "a+" };
private char	*t1_file =		"./fopen_X-t.1";
private char	*t2_file =		"./fopen_X-t.2";
private char	*t2a_file =		"./fopen_X-t.2a";
private char	*t3_file =		"./fopen_X-t.3";


private void	test1();
private void	test2();
private void	test3();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private void
test1()
{
	char		*chr;
	char		*blk;
	FILE		*fp;
	int		err1;
	int		err2;
	int		fail = 0;
	int 		pathok = 0; 

	DBUG_ENTER("test1");

	if ((chr = tet_getvar(VSX_NXIO_CHRDEV)) == (char *) NULL
		|| (blk = tet_getvar(VSX_NXIO_BLKDEV)) == (char *) NULL
		|| *chr == '\0' || *blk == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("%s and/or %s parameter not defined",
			VSX_NXIO_CHRDEV, VSX_NXIO_BLKDEV);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (setprv(PRV_DEVICE) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_DEVICE) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	errno = NOERROR;
	if ((fp = fopen(chr, "r")) != (FILE *) NULL)
	{
		fail |= 1;
		(void) fclose(fp);
	}
	else if (errno != ENXIO)
	{
		fail |= 2;
		err1 = errno;
	}
	else
		PATH_TRACE;
		
	if (strncmp(blk, "unsup", (size_t)5) != 0)
	{
		errno = NOERROR;
		if ((fp = fopen(blk, "r")) != (FILE *) NULL)
		{
			fail |= 4;
			(void) fclose(fp);
		}
		else if (errno != ENXIO)
		{
			fail |= 8;
			err2 = errno;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;
		
	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("fopen(%s, \"r\") did not return NULL", chr);
		else if (fail & 2)
		{
			in_rpt("fopen(%s, \"r\") did not return expected errno",
				chr);
			in_rpt("expected errno ENXIO, received %d(%s)",
				err1, errname(err1));
		}
		if (fail & 4)
			in_rpt("fopen(%s, \"r\") did not return NULL", blk);
		else if (fail & 8)
		{
			in_rpt("fopen(%s, \"r\") did not return expected errno",
				blk);
			in_rpt("expected errno ENXIO, received %d(%s)",
				err2, errname(err2));
		}
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(5);
	}

	(void) unlink(t1_file);
	DBUG_VOID_RETURN;
}

private void
test2()
{
	FILE		*fp;
	int		errs[NFOP_MODES * 2];
	int		flags[NFOP_MODES];
	int		fail = 0;
	int		i;
	int	 	pathok = 0; 

	DBUG_ENTER("test2");

	for (i = 1 ; i < NFOP_MODES ; i++)
	{
		(void) unlink(t2_file);
		if (pure_run(t2_file, WAITTIME, &pchild) != SUCCEED)
		{
			(void) unlink(t2_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE; 
			
		flags[i] = 0;
		errno = NOERROR;
		if ((fp = fopen(t2_file, fop_modes[i])) != (FILE *) NULL)
		{
			flags[i] = 1;
			(void) fclose(fp);
			fail = 1;
		}
		else if (errno != ETXTBSY)
		{
			flags[i] = 2;
			errs[i] = errno;
			fail = 1;
		}
		else
			PATH_TRACE;
			
		if (killchild(pchild, WAITTIME) != SUCCEED)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: failed to kill child process");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
			
		pchild = SYSERROR;
	}
	if (fail)
	{
		xx_rpt(WARNING);
		for (i = 1 ; i < NFOP_MODES ; i++)
		{
			if (flags[i] & 1)
				in_rpt("fopen(%s) on a busy text file did not return NULL",
					fop_modes[i]);
			else if (flags[i] & 2)
			{
				in_rpt("fopen(%s) on a busy text file did not return expected errno",
					fop_modes[i]);
				in_rpt("expected errno ETXTBSY, found %d(%s)",
					errs[i], errname(errs[i]));
			}
		}
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT( ( ((NFOP_MODES-1) * 3) + 1));
	}

	(void) unlink(t2_file);
	(void) unlink(t2a_file);
	DBUG_VOID_RETURN;
}

private void
test3()
{
	char *invaltype = "z+";
	FILE *fp;
	int err;
	int pathok = 0;

	DBUG_ENTER("test3");

	errno = NOERROR;
	if ((fp = fopen(t3_file, invaltype)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("fopen(%s, \"%s\") did not return NULL",
			t3_file, invaltype);
		(void) fclose(fp);
	}
	else if (errno != EINVAL)
	{
		err = errno;
		xx_rpt(WARNING);
		in_rpt("fopen(%s, \"%s\") did not set errno correctly",
			t3_file, invaltype);
		in_rpt("expected EINVAL, received %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}
