/*
*      SCCS:  @(#)  ANSI.os/streamio/fputs/fputs.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fputs - T.fputs Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fputs

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fputs() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	10 Nov 1988
		(split out from T.puts)
MODIFICATIONS:	
		Geoff Clare, 31 May 1989
		Add EIO test.

		Geoff Clare, 22 Nov 1989
		Expect anything >= 0 on success, not number of chars.

		Steve Ratcliffe, 05 Apr 1990
		Add ENOSPC test.

		Geoff Clare, 14 June 1990
		Do EAGAIN and EPIPE tests on pipe and FIFO

		David Sawyer
		Fri Jul 13 12:07:58 BST 1990
		Added resillience code.

		Stuart Boutell, 5 Nov 1990
			Changed reason to tet_reason, dltd to tet_reason,
			vg_env to environ.

		John-Paul, 12 October 1992
			XNFS enhancements.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<chktimes.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"fputs_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fputs
#endif


#define	NO_TESTS	13		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)

extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;
extern	char	*errname();
extern	char	*itoa();
extern	char	*execpath();
extern	void	do_feio();

int	no_tests = NO_TESTS;

private	char	*t_string =	"test string";
private	char	*t1_file =	"./fputs-t.1";
private	char	*t2_file =	"./fputs-t.2";
private char	*t11_dir =	"./fputs-d.11";
#if TEST_XNFS
private	char	*t_string_xnfs = ONE_CHAR_LINE;
private	char	*t12_file =	T12_FILE;
private	char	*t13_file =	T13_FILE;
#endif /* TEST_XNFS */

private char	*prog11 =	"fputs_t11";
#if TEST_XNFS
private char	*prog_su =	"fputs_su";
#endif /* TEST_XNFS */

private	void	test1();
private	void	test2();
private	void	test3_9();
private	void	test10();
private	void	test11();
private	void	test12();
private	void	test13();
private	int	t3_9func();
#if !TEST_XNFS
private	int	t10io();
private	void	t10rpt();
#endif /* TEST_XNFS */
private void	ch_t11();
#if TEST_XNFS
private void	ch_t12();
private int	t13_func();
private	int	t13_chk_func();
#endif /* TEST_XNFS */

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3_9, 3,
	 test3_9, 4,
	 test3_9, 5,
	 test3_9, 6,
	 test3_9, 7,
	 test3_9, 8,
	 test3_9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	NULL, 0
};

private	void
test1()
{
	FILE		*fp;
	char		buf[32];
	int		fd;
	int		len;
	int		rval;
	int		err;
	int		pathok = 0;

	DBUG_ENTER("test1");
	if ((fp = fopen(t1_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("freopen(%s, \"w\", stdout) failed - errno %d(%s)",
			t1_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	rval = fputs(t_string, fp);
	(void) fclose(fp);
	if ((fd = open(t1_file, O_RDONLY, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY, 0) failed - errno %d(%s)",
			t1_file, err, errname(err));
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((len = read(fd, buf, sizeof(buf) - 1)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("read() failed - errno %d(%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	buf[len] = NUL;
	(void) close(fd);
	if (len != strlen(t_string) || strcmp(t_string, buf) || rval < 0)
	{
		xx_rpt(FAILURE);
		if (rval < 0)
		{
			in_rpt("fputs() did not return correct value:");
			in_rpt("expected >= 0, got %d", rval);
		}
		if (len != strlen(t_string))
		{
			in_rpt("fputs() did not write correct number of bytes to stream");
			in_rpt("expected %d, got %d", strlen(t_string), len);
		}
		else if (strncmp(t_string, buf, (size_t)len))
		{
			in_rpt("fputs() did not write correct string to stream");
			in_rpt("expected \"%s\", actual \"%s\"", t_string, buf);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}

	(void) unlink(t1_file);
	DBUG_VOID_RETURN;
}

private void
test2()
{
	FILE	*fp;
	char	*res;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	if ((fp = fopen(t2_file, "w")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t2_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	globok = 0;
	savtimes(t2_file);
	PATH_FUNC_TRACE;

        if (fputs(t_string, fp) == EOF)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fputs() failed on file %s, errno %d(%s)",
			t2_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fclose(fp) == EOF)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fclose() failed on file %s, errno %d(%s)",
			t2_file, err, errname(err));
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ((res = chktimes(t2_file, NEWMTIME | NEWCTIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(res);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(5);
	}

	(void) unlink(t2_file);
	DBUG_VOID_RETURN;
}

/* Lowest test number for which following function is used */
#define TBASE	3

private	void
test3_9()
{
	int experr, err, ret, ferr_ret, gotsig;
	static int errnums[] = { EAGAIN, -EAGAIN, EBADF, EINTR, EFBIG,
			      /* test3   test4    test5  test6  test7 */
				 EPIPE, -EPIPE };
			      /* test8  test9 */
	int	pathok = 0;

	DBUG_ENTER("test3_9");

#if TEST_XNFS
	if(tet_thistest != 7)
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}
#endif /* TEST_XNFS */

	experr = errnums[tet_thistest - TBASE];

	globok = 0;
	ret = do_ferr(experr, 'w', -1, t3_9func, &ferr_ret, &gotsig);
	PATH_FUNC_TRACE;
	err = errno;
	if (experr < 0)
		experr = -experr;

	if (tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!ferr_ret || err != experr || ret != EOF ||
		(experr == EPIPE && !gotsig))
	{
		xx_rpt(FAILURE);
		if (!ferr_ret)
			in_rpt("fputs() failed to set error indicator when testing for errno %d(%s)",
				experr, errname(experr));
		if (experr == EPIPE && !gotsig)
			in_rpt("SIGPIPE was not received");
		if (ret != EOF)
			in_rpt("fputs() returned %d when testing for errno %d(%s), expected EOF",
				ret, experr, errname(experr));
		else if (err != experr)
			in_rpt("fputs() set errno incorrectly: expected %d(%s), actual %d(%s)",
				experr, errname(experr), err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

/* ARGSUSED */

private int
t3_9func(fp, nbyte)
FILE *fp;
int nbyte;
{
	int	pathok = 0;

	DBUG_ENTER("t3_9func");

	PATH_FUNC_RPT(0);

	DBUG_RETURN(fputs(t_string, fp));
}

#if TEST_XNFS

private void
test10()
{
        DBUG_ENTER("test10");
        xx_rpt(NOTINUSE);
        DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test10()
{
	DBUG_ENTER("test10");

	do_feio("w", -1, t10io, t10rpt);

	DBUG_VOID_RETURN;
}

private int
t10io(fp)
FILE *fp;
{
	int 	ret;
	int	pathok = 0;

	DBUG_ENTER("t10io");

	globok = 0;

	ret = fputs("\n", fp);

	PATH_FUNC_RPT(0);

	DBUG_RETURN(ret);
}


private void
t10rpt(ret, err)
int ret, err;
{
	int	pathok = 0;

	DBUG_ENTER("t10rpt");

	PATH_FUNC_TRACE; /* checks path trace in iofunc() */

	if (ret != EOF || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("fputs() from orphaned background process group did not give correct results");
		if (ret != EOF)
			in_rpt("expected return: EOF, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test11()
{

	DBUG_ENTER("test11");

	(void) cppair(ch_t11, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	int 	err;
	char	*args[6];

	DBUG_ENTER("ch_t11");

	args[0] = prog11;
	args[1] = t11_dir;
	args[2] = "fputs";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog11), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}


#if !TEST_XNFS

private void
test12()
{
	DBUG_ENTER("test12");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test12()
{
	DBUG_ENTER("test12");

	(void) cppair(ch_t12, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t12()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t12");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t12_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test13()
{
	DBUG_ENTER("test13");

	t_estale_wr_c(t13_file, 0, t13_func, t13_chk_func, SP_T13_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t13_func(fp)
FILE	*fp;
{
	DBUG_ENTER("t13_func");

	DBUG_RETURN(fputs(t_string_xnfs, fp) == EOF ? -1 : 0);
}

private int
t13_chk_func(fp, ebytes)
FILE	*fp;
int	ebytes;
{
	int	rval;
	int	err;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("t13_chk_func");

	errno = NOERROR;

	for(i = 0; i < ebytes; i++) {
		rval = fputs(t_string_xnfs, fp);
		if(rval != strlen(t_string_xnfs))
			break;
	}

	err = errno;

	if(rval != EOF || ferror(fp) == 0 || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(rval != EOF)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value (%d) EOF", EOF);
			in_rpt("Actual   return value %d", rval);
		}
		else
			PATH_TRACE;
		
		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;
		
		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
