/*
*      SCCS:  @(#)  ANSI.os/streamio/fread/fread.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fread - T.fread Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fread

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fread() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 4 Nov 1988
		Add tests for errno values
		Delete test with negative size & nitems

		Geoff Clare, 31 May 1989
		Add EIO test.

		Geoff Clare, 14 June 1990
		Do EAGAIN test on pipe and FIFO.

		Peter Lee, Unisoft Ltd.,
		Mon Jul 16 12:37:21 BST 1990
		Added resilience code.

		Stuart Boutell, 1 Nov 1990
			Added J A Nave's new VSX4 test, test7, and
			renumbered tests 7-12 to 8-13
			Added setupok to comp_prep and to tests as nec.

		Stuart Boutell, 5 Nov 1990
			changed reason() and dltd() to tet_reason()
		
		Geoff Clare, 27 Jan 1992
			Fix use of NULL as an integer.

		Neil Moses, 9 June 1992
			New test 14.  On ROF time stamps are not changed.

		John-Paul, 23 July 1992
			mode variables changed from 'int' to 'mode_t'.

		John-Paul, 28 October 1992
			XNFS enhancements.

		Geoff Clare, 28 Oct 1993
			Use new file in XNFS st_atime test to avoid
			reading cached data.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<chktimes.h>
#include 	<tet_api.h>
#include	<xnfs.h>
#include	"fread_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fread
#endif


#define	NO_TESTS	16		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	FERR_BUFSZ	20
#define	T_NUM		5
#define	T1_NUM		3
#define	T2_NUM		7
#define	T3_NUM		3
#define	T4_NUM		3
#define T5_NUM		6


extern	char	*errname();
extern	char	*execpath();
extern	char	*itoa();
extern	void	do_feio();
extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

int	no_tests = NO_TESTS;

#if TEST_XNFS
private	int	snames[] = {1, 0};
#endif /* TEST_XNFS */

private	char	*prog_su =	"fread_su";
private	char	*t_file =	"./fread-t";
#if TEST_XNFS
private char	*t6_file =	T6_FILE;
#else
#define		t6_file		t_file
#endif
private char	*t14_dir =	"fread-d.14";
private char	*t14_file =	"fread-d.14/fread-t.14";
#if TEST_XNFS
private char	*t15_file =	T15_FILE;
private char	*t16_file =	T16_FILE;
#endif /* TEST_XNFS */
#if !TEST_XNFS
private	char	*sigstat;
#endif /* !TEST_XNFS */
private	long	numbers[T_NUM] = { 0, -1, 12312312, 12222, 4343423 };
private	int	setupok = -1;

private	void	comp_prep();
private	void	comp_fin();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	test6();
private	void	test7();
private	void	test8_11();
private	void	test12();
private	void	test13();
private	void	test14();
private	void	test15();
private	void	test16();
private	void	ch_t14();
#if !TEST_XNFS
private	int	t8_11func();
private	int	t12_13io();
private	void	t12rpt();
private	void	t13rpt();
#else /* TEST_XNFS */
private void	ch_t15();
private	int	t16_func();
private	int	t16_chk_func();
#endif /* TEST_XNFS */

public	void	(*setfunc) () = comp_prep;
public	void	(*clnfunc) () = comp_fin;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8_11, 8,
	 test8_11, 9,
	 test8_11, 10,
	 test8_11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	NULL, 0
};

private void
comp_prep()
{
	int		fd;
	int		i;
	int		bad = 0;
	static char	msg[150];
	int		pathok = 0;

	if ((fd = creat(t_file, MODEANY)) == SYSERROR)
	{
		bad = 1;
		(void) sprintf(msg, "creat(%s, %#o) failed in preperations - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
	}
	else
	{
		for (i = 0 ; i < T_NUM ; i++)
			if (write(fd, (char *) &numbers[i], sizeof(long)) !=
							sizeof(long))
			{
				(void) sprintf(msg, "write() failed in preperations - errno %d(%s)",
					errno, errname(errno));
				bad = 1;
				break;
			}
			else
				PATH_TRACE;
		(void) close(fd);
	}
	if (bad)
		for (i = 1 ; i <= NO_TESTS ; i++)
			(void) tet_delete(i, msg);

	setupok = (pathok == T_NUM) ? 0 : -1;
}

private	void
comp_fin()
{
	(void) unlink(t_file);
}


private void
test1()
{
	FILE		*fp;
	int		fail = 0;
	int		i;
	int		err;
	int		rval;
	long		num[T1_NUM];
	int		pathok = setupok;

	DBUG_ENTER("test1");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	for (i = 0 ; i < T1_NUM ; i++)
		num[i] = 0L;
	if ((rval = fread((void *) num, (size_t) sizeof(long),
			(size_t)T1_NUM, fp)) != T1_NUM)
		fail = 1;
	else
		PATH_TRACE;

	for (i = 0 ; i < T1_NUM ; i++)
		if (num[i] != numbers[i])
			fail |= 2;
		else
			PATH_TRACE;
	if (fail)
	{
		xx_rpt(FAILURE);
		in_rpt("fread(buf, sizeof(long), %d, fp) did not have expected results",
			T1_NUM);
		if (fail & 1)
			in_rpt("fread() did not return %d - return value = %d",
				T1_NUM, rval);
		if (fail & 2)
		{
			for (i = 0 ; i < T1_NUM ; i++)
				if (num[i] != numbers[i])
					in_rpt("fread() did not read value correctly, expected %ld, read %ld",
						numbers[i], num[i]);
		}
		(void) tet_delete(3, "depends on success of test 1");
	}
	else
		PATH_XS_RPT(2 + T1_NUM);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private	void
test2()
{
	FILE	*fp;
	int	err;
	int	rval;
	int	fail = 0;
	int	i;
	long	num[T2_NUM];
	int	pathok = setupok;

	DBUG_ENTER("test2");
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = fread((void *) num, (size_t) sizeof(long),
			(size_t)T2_NUM, fp)) != T_NUM)
		fail = 1;
	else
		PATH_TRACE;
	for (i = 0 ; i < T_NUM ; i++)
		if (num[i] != numbers[i])
			fail |= 2;
		else
			PATH_TRACE;
	if (fail)
	{
		xx_rpt(FAILURE);
		in_rpt("fread(buf, sizeof(long), %d, fp) on file containing %d longs, did not have expected results",
			T2_NUM, T_NUM);
		if (fail & 1)
			in_rpt("fread() did not return %d - return value = %d",
				T_NUM, rval);
		if (fail & 2)
		{
			for (i = 0 ; i < T_NUM ; i++)
				if (num[i] != numbers[i])
					in_rpt("fread() did not read value correctly, expected %ld, read %ld",
						numbers[i], num[i]);
		}
		(void) tet_delete(3, "depends on success of test 2");
	}
	else
		PATH_XS_RPT(2 + T_NUM);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private	void
test3()
{
	FILE	*fp;
	int	err;
	int	fd;
	int	rval;
	int	fail = 0;
	int	i;
	long	num[T_NUM];
	int	pathok = setupok;

	DBUG_ENTER("test3");
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fread((void *) num, (size_t) sizeof(long), (size_t)T3_NUM, fp);
	if ((fd = open(t_file, O_RDONLY, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d(%s)",
			t_file, err, errname(err));
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = read(fd, (char *) num, sizeof(long) * T_NUM)) != sizeof(long) * T_NUM)
	{
		if (rval == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("read() failed - errno %d(%s)",
				err, errname(err));
			(void) fclose(fp);
			(void) close(fd);
			DBUG_VOID_RETURN;
		}
		else
			fail = 1;
	}
	else
	{
		PATH_TRACE;
		for (i = 0 ; i < T_NUM ; i++)
			if (num[i] != numbers[i])
				fail = 2;
			else
				PATH_TRACE;
	}
	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail == 1)
			in_rpt("fread() changed file. read() could not read original contents of file");
		else
			in_rpt("fread() changed file. values read() from file changed");
	}
	else
		PATH_XS_RPT(3 + T_NUM);

	(void) fclose(fp);
	(void) close(fd);
	DBUG_VOID_RETURN;
}


private	void
test4()
{
	FILE	*fp;
	int	err;
	long	rval;
	long	num[T4_NUM];
	int	pathok = setupok;

	DBUG_ENTER("test4");
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fread((void *) num, (size_t) sizeof(long), (size_t)T4_NUM, fp);
	if ((rval = ftell(fp)) != (long) (sizeof(long) * T4_NUM))
	{
		xx_rpt(FAILURE);
		in_rpt("fread() did not change position of file pointer");
		in_rpt("ftell() after fread() returned %ld instead of %ld",
			rval, (long) (T4_NUM + sizeof(long)));
	}
	else
		PATH_XS_RPT(1);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private	void
test5()
{
	FILE	*fp;
	int	err;
	long	num[T5_NUM];
	int	pathok = setupok;

	DBUG_ENTER("test5");
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
        (void)	fread((void *) num, (size_t) sizeof(long), (size_t)T5_NUM, fp);


	if (!feof(fp))
	{
		xx_rpt(FAILURE);
		in_rpt("fread(buf, sizeof(long), %d, fp)on file containing %d longs failed to set EOF indicator upon reading end of file", T5_NUM, T_NUM);
	}
	else
		PATH_XS_RPT(1);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test6()
{
	FILE	*fp;
	char	*res;
	int	err;
	char	s[1];
	int	pathok = setupok;

	DBUG_ENTER("test6");

#if TEST_XNFS
	
	if (tet_sync(SP_T6_CREAT_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREAT_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */
		PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = fopen(t6_file, "r")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"r\") failed - errno %d(%s)",
			t6_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	globok = 0;
	savtimes(t6_file);
	PATH_FUNC_TRACE;

        if (fread((void *)s, (size_t)1, (size_t)1, fp) != 1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fread(s, 1, 1, fp) failed on file %s, errno %d(%s)",
			t6_file, err, errname(err));
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ((res = chktimes(t6_file, NEWATIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(res);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(5);
	}

	(void) fclose(fp);

#if TEST_XNFS
	(void) tet_sync(SP_T6_UNLINK_START, snames, SYNC_TIMEOUT);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

#if	!defined(_XOPEN_SOURCE) || TEST_XPG > 3
private void
test7()
{
	FILE		*fp;
	int		fail = 0;
	int		i;
	int		err;
	int		rval[2];
	long		num[T1_NUM];
	int		pathok = setupok;

	DBUG_ENTER("test7");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0 ; i < T1_NUM ; i++)
		num[i] = 0L;
	if ((rval[0] = fread((void *) num, (size_t)0, (size_t)T1_NUM, fp)) != 0)
		fail |= 1;
	else
		PATH_TRACE;

	if ((rval[1] = fread((void *) num, sizeof(long), (size_t)0, fp)) != 0)
		fail |= 2;
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("fread(ptr, 0, nitems, strm) did not return 0 - return value = %d",
				rval[0]);
		if (fail & 2)
			in_rpt("fread(ptr, size, 0, strm) did not return 0 - return value = %d",
				rval[1]);
	}
	else
		PATH_XS_RPT(3);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}
#else
private void
test7()
{
	DBUG_ENTER("test7");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#endif

#if TEST_XNFS

private void
test8_11()
{
	DBUG_ENTER("test8_11");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

/* Lowest test number for which following function is used */
#define TBASE	8

private	void
test8_11()
{
	int experr, err, ret, ferr_ret;
	static int errnums[] = { EAGAIN, -EAGAIN, EBADF, EINTR, };
			      /* test8   test9    test10  test11  */
	int	pathok = setupok;

	DBUG_ENTER("test8_11");

	experr = errnums[tet_thistest - TBASE];

	globok = 0;
	ret = do_ferr(experr, 'r', -1, t8_11func, &ferr_ret, &err);
	PATH_FUNC_TRACE;
	err = errno;
	if (experr < 0)
		experr = -experr;

	if (tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!ferr_ret || err != experr || ret > 0)
	{
		xx_rpt(FAILURE);
		if (!ferr_ret)
			in_rpt("fread() failed to set error indicator when testing for errno %d(%s)", experr, errname(experr));
		if (ret > 0)
			in_rpt("fread() returned %d when testing for errno %d(%s), expected <= 0", ret, experr, errname(experr));
		if (err != experr)
			in_rpt("fread() set errno incorrectly: expected %d(%s), actual %d(%s)", experr, errname(experr), err, errname(err));
	}
	else
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private int
t8_11func(fp, nbyte)
FILE *fp;
int nbyte;
{
	char buf[FERR_BUFSZ];
	int	pathok = 0;

	DBUG_ENTER("t8_11func");

	if (nbyte > FERR_BUFSZ)
	{
		er_rpt("nbyte too great in function called from do_ferr()");
		nbyte = FERR_BUFSZ;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(fread((void *)buf, (size_t)1, (size_t)nbyte, fp));
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test12()
{
	DBUG_ENTER("test12");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test12()
{
	DBUG_ENTER("test12");

	sigstat = "ignored";
	do_feio("ri", -1, t12_13io, t12rpt);

	sigstat = "blocked";
	do_feio("rb", -1, t12_13io, t12rpt);

	DBUG_VOID_RETURN;
}

private void
t12rpt(ret, err)
int ret, err;
{
	int	pathok = 0;

	DBUG_ENTER("t12rpt");

	PATH_FUNC_TRACE;
	
	if (ret > 0 || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("fread() from background with SIGTTIN %s did not give correct results",
				sigstat);
		if (ret > 0)
			in_rpt("expected return: <= 0, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test13()
{
	DBUG_ENTER("test13");

	do_feio("ro", -1, t12_13io, t13rpt);

	DBUG_VOID_RETURN;
}


private void
t13rpt(ret, err)
int ret, err;
{
	int	pathok = 0;

	DBUG_ENTER("t13rpt");

	PATH_FUNC_TRACE;

	if (ret > 0 || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("fread() from orphaned background process did not give correct results");
		if (ret > 0)
			in_rpt("expected return: <= 0, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private int
t12_13io(fp)
FILE *fp;
{
	int ret;
	char buf[2];

	DBUG_ENTER("t12_13io");

	ret = fread((void *)buf, (size_t)1, (size_t)1, fp);

	if (ret == 0)
		globok = 1;
	else
		globok = 0;

	DBUG_RETURN(ret);
}

#endif /* TEST_XNFS */

private void
test14()
{
	DBUG_ENTER("test14");

	(void) cppair(ch_t14, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t14_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t14()
{
	char	*rofs;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t14");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_ROFS is not defined");
		DBUG_VOID_RETURN;
	}

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = t14_dir;
	args[3] = t14_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog_su, err, errname(err));

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

private void
test15()
{
	DBUG_ENTER("test15");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test15()
{
	DBUG_ENTER("test15");

	(void) cppair(ch_t15, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t15()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t15");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t15_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test16()
{
	DBUG_ENTER("test16");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test16()
{
	DBUG_ENTER("test16");

	t_estale_rd_c(t16_file, 0, t16_func, t16_chk_func, SP_T16_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t16_func(fp)
FILE	*fp;
{
	char	buf[2];

	DBUG_ENTER("t16_func");

	DBUG_RETURN(fread((void *)buf, (size_t)1, (size_t)1, fp) == 0 ? -1 : 0);
}

private int
t16_chk_func(fp)
FILE	*fp;
{
	int	rval;
	int	err;
	char 	buf[2];
	int	pathok = 0;

	DBUG_ENTER("t16_chk_func");

	errno = NOERROR;

	while((rval = fread((void *)buf, (size_t)1, (size_t)1, fp)) != 0)
		;

	err = errno;

	if(rval != 0 || ferror(fp) == 0 || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(rval != 0)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value 0");
			in_rpt("Actual   return value %d", rval);
		}
		else
			PATH_TRACE;
		
		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;
		
		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
