/*
*      SCCS:  @(#)  ANSI.os/streamio/fread/fread_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fread - fread_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		fread_su - used by T.fread in privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.fread to perform privileged tests.

	  usage:
		fread_su testnum dir file ident dbug

	  where testnum is the number of the test to be executed, dir
	  and file are parameters to be passed to the test.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		John-Paul, UniSoft Ltd.
DATE CREATED:	28 October 1992
MODIFICATIONS:

************************************************************************/
#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <dbug.h>
#include <errno.h>
#include <sys/stat.h>
#include <report.h>
#include <tet_api.h>
#include <xnfs.h>
#include "fread_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef fread
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
public  int	testfail;

extern	char	**environ;
extern	void	savtimes();
extern	char	*chktimes();
extern	char	*errname();

extern	void	valrpt();
extern	void	set_pname();

private	void	subtest14();
#if TEST_XNFS
private int	t15_chk_func();
#endif /* TEST_XNFS */

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int testnum;
	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testnum = atoi(argv[1]);
	switch(testnum)
	{
	case 14:
		subtest14(argv[2], argv[3]);
		break;
#if TEST_XNFS
	case 15:
		t_eacces_rd_c(argv[3], 0, t15_chk_func, SP_T15_EACCES);
		break;
#endif /* TEST_XNFS */
	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed : %d", testnum);
		return(OK);
	}

	return(OK);
}

private void
subtest14(mdir, file)
char	*mdir, *file;
{
	FILE	*fp;
	size_t 	rval;
	char	*errptr;
	char	buf[2];
	int	err;
	int	pathok = 0;

	DBUG_ENTER("subtest14");

	errno = 0;
	testfail = 0;

	globok = 0;
	if (setuprofs(mdir, file, 'f', MODEANY) != 0)
	{
		DBUG_VOID_RETURN;	/* reported in setuprofs() */
	}
	else
		PATH_FUNC_TRACE;

	if((fp = fopen(file, "r")) == NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fopen(\"%s\", \"r\") failed - errno %d (%s)", 
			file, err, errname(err));
		(void) clnuprofs(mdir, file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;		

	globok = 0;
	savtimes(file);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	rval = fread((void *)buf, (size_t)1, (size_t)1, fp);
#ifdef TESTBED
	rval = 0;
	if (errno)
		errno++;
#endif
	err = errno;

	if (rval != 0 || ferror(fp))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (rval != 0)
			in_rpt("fread() returned %d, expected 0", rval);
		else
		{
			in_rpt("fread() set the error indicator for the stream");
			in_rpt("errno was set to %d (%s)", err, errname(err));
		}
		(void) fclose(fp);
		(void) clnuprofs(mdir, file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	errptr = chktimes(file, 0);
	PATH_FUNC_TRACE;

	if(errptr != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("Call to fread() changed time stamps on %s", file);
		in_rpt("%s", errptr);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) fclose(fp);
	(void) clnuprofs(mdir, file);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private int
t15_chk_func(fp)
FILE	*fp;
{
	int	rval, err;
	char	buf[2];
	int	pathok = 0;

	DBUG_ENTER("t15_chk_func");

	errno = NOERROR;

	while((rval = fread((void *)buf, (size_t)1, (size_t)1, fp)) != 0)
		;

	err = errno;

	if(rval != 0 || ferror(fp) == 0 || err != EACCES)
	{
		xx_rpt(FAILURE);

		if(rval != 0)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value 0");
			in_rpt("Actual   return value %d", rval);
		}
		else
			PATH_TRACE;
		
		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;
		
		if(err != EACCES)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",EACCES,errname(EACCES));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
