/*
 *      SCCS:  ANSI.os/streamio/freopen/freop_su.c Rel 4.4.1	(06/09/97)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#define MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/freopen/freopen_su Rel 4.4.1 (06/09/97)";
#ifdef	MAIN
char	*copyright[] = {
		"(C) Copyright 1992 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		freopen_su.c

PROJECT:	XNFS (X/OPEN NFS tests)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int tet_main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	Common subprogram for XNFS uid mapping tests.
	Exec'd by T.freopen.

	usage:
		progname file vg_ident vg_dbug

AUTHOR:		John-Paul, UniSoft Ltd.

DATE CREATED:	3 November 1992

MODIFICATIONS:

************************************************************************/

#include <std.h>
#include <dbug.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include "freopen_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef freopen
#endif


#define MODEANY		((mode_t)(S_IRWXU|S_IRWXG|S_IRWXO))

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	*errname();
extern	void	set_pname();

private int	subtest21_22();

public	int
tet_main(argc, argv)
int argc;
char **argv;
{
	int rc;

	set_pname(argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(NOK);
	}
	if (argc >= 5)
		vg_dbug = argv[4];
	if (argc >= 4)
		vg_ident = argv[3];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug + 2);
	}

	rc = subtest21_22(argv[1], argv[2]);

	(void) unlink(argv[1]);
	(void) unlink(argv[2]);

	return(rc);
}

private int
subtest21_22(file, a_file)
char *a_file;
char *file;
{
	int err, rc;
	FILE *fd;
	int pathok = 0;

	DBUG_ENTER("subtest21_22");

	fd = fopen(file, "w");
	if(fd == NULL)
	{
                err = errno;
                xx_rpt(DELETION);
                in_rpt("fopen(\"%s\", \"w\") failed - errno %d (%s)", file, err, errname(err));
		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;

        fd = freopen(a_file, "w", fd);
	err = errno;
	rc = 0;
	if (fd != NULL || err != EACCES)
	{
		xx_rpt(FAILURE);

		if (fd != NULL)
			in_rpt("freopen(%s, \"w\") did not return NULL", 
						a_file);

		if (err != EACCES)
			in_rpt("freopen(%s, \"w\") did not return expected errno",
						a_file);
			in_rpt("expected errno EACCES, found %d(%s)",
				err, errname(err));

		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;

	rc = OK;
	if (pathok == 2)
		rc |= PATH_OK;

	DBUG_RETURN(rc);
}
