/*
*      SCCS:  @(#)  ANSI.os/streamio/freopen/freop_t25.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/freopen - freop_t25.c Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		freop_t25 - used by T.freopen in test25()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.freopen to test the creation of files on a read-only
	  file system. Must be setuid to root to do mount/umount of
	  file system.

	  usage:
		freop_t25 rofs dir file ident dbug

	  where rofs is the device name of the file system to be mounted,
	  dir is the directory that it is to be mounted on, file is the
	  name os a file (that doesn't exist) on the file system (with dir
	  in it's path), ident is used as the vg_ident of this test and
	  dbug is an optional argument.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Fri Nov 28 12:22:12 WET 1986
MODIFICATIONS:
		Geoff Clare, 15 Nov 1989
			New setuprofs() argument

		Andrew Dingwall Wed Jun 20 15:57:42 BST 1990
		Renamed as freop_t19 after inserting a new test after test 1

		David Sawyer
		Wed Jul 18 11:12:14 BST 1990
		Added resillience code.

		Stuart Boutell, 30 Oct 1990
			Renamed as freop_t25 after inserting 6 tests

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Neil Moses, 1 June 1992
			Check that file is not truncated/created.

		Neil Moses, 8 June 1992
			Check that time stamps are not changed.

		Geoff Clare, 18 June 1992
			Fixes for coding errors in err.4.2.2.322.

                John-Paul, 27 July 1992
                        mode variables changed from 'int' to 'mode_t'.

                John-Paul, 2 December 1992
                        Don't use crfile() and crdir() because
                        chown() can fail over NFS.

		Geoff Clare, 27 Jan 1993
			Don't test "wb", etc. in XPG3 mode.

************************************************************************/
#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dbug.h>
#include <exitcodes.h>
#include <report.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <setprv.h>
#include <unistd.h>
#include <tet_api.h>
#include <xnfs.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef freopen
#endif


#define VSX_ROFS	"VSX_ROFS"
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	void	(*vsignal())();
extern	void	savtimes();
extern	char	*chktimes();

private	char	*rofs;
private	char	*tidydir;
private	char	*tidyfile;
private	int	tsetuprofs();
private	int	tclnuprofs();
private	void	(*oldhdlr)();

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	*errname();
extern	char	*testid();
extern	void	valrpt();
extern	void	set_pname();

private	void	subtest25();

#if TEST_XPG == 3
#define	NFOP_MODES	5
private	char	*fop_modes[NFOP_MODES] = { "w", "a", "r+", "w+", "a+" };
#else
#define	NFOP_MODES	13
private	char	*fop_modes[NFOP_MODES] = {
	"w", "a", "r+", "w+", "a+",
	"wb", "ab", "r+b", "rb+", "w+b", "wb+", "a+b", "ab+"
};
#endif

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	set_pname(argc < 1 || argv[1] == (char *) NULL ?
		"freop_t25" : (argv[1]));
	if (argc == 6)
	{
		if (argv[5] != (char *) NULL && strncmp("-D", argv[5], (size_t)2) == 0)
		{
			vg_dbug = argv[5];
			DBUG_PUSH(&vg_dbug[2]);
		}
		argc--;
	}
	if (argc != 5)
	{
		vsxerror("argument count wrong, got %d, expected 5", argc);
		return(A_MISC);
	}
	if (argv[4] != (char *) NULL && argv[4][0] != NUL)
		vg_ident = argv[4];

	subtest25(argv[1], argv[2], argv[3]);
	return(OK);
}


private void
subtest25(trofs, mdir, file)
	char	*trofs;
	char	*mdir;
	char	*file;
{
	FILE		*fp;
	int		errs[NFOP_MODES];
	int		flags[NFOP_MODES];
	struct		stat buf;
	char		newfile[128];
	char		*errptr;
	char		mess[NFOP_MODES][100];
	int		fail = 0;
	int		err, i;
	int		pathok = 0;

	DBUG_ENTER("subtest25");
	DBUG_PRINTF("test25", "trofs=%s, dir=%s, file=%s", trofs, mdir, file);

	/* Create path to non-existent file */
	(void) sprintf(newfile, "%s/newfile", mdir);

	globok = 0;
	if (tsetuprofs(mdir, file, 'f', (mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)))
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	for (i = 1 ; i < NFOP_MODES ; i++)
	{
		errno = NOERROR;
		flags[i] = 0;

		/* Save time stamps */
		globok = 0;
		if(strpbrk(fop_modes[i], "awb") != NULL
		&& strchr(fop_modes[i], 'r') == NULL)
			savtimes(mdir);
		else
			savtimes(file);
		PATH_FUNC_TRACE;

		if ((fp = fopen(mdir, "r")) == (FILE *) NULL)
		{
			errs[0] = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
				mdir, errs[0], errname(errs[0]));
			(void) clnuprofs(mdir, (char *) NULL);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		errno = NOERROR;
		if ((fp = freopen(file, fop_modes[i], fp)) != (FILE *) NULL)
		{
			flags[i] |= 1;
			(void) fclose(fp);
			fail = 1;
		}
		else if (errno != EROFS)
		{
			flags[i] |= 2;
			errs[i] = errno;
			fail = 1;
		}
		else
		{
			PATH_TRACE;

			/* Check file is not created */
			if(strpbrk(fop_modes[i], "awb") != NULL
			&& strchr(fop_modes[i], 'r') == NULL)
			{
				if ((fp = fopen(newfile, fop_modes[i])) != (FILE *) NULL)
				{
					flags[i] |= 1;
					(void) fclose(fp);
					fail = 1;
				}
				else if (errno != EROFS)
				{
					flags[i] |= 2;
					errs[i] = errno;
					fail = 1;
				}
				else
					PATH_TRACE; 

				if(eaccess(newfile, F_OK) != SYSERROR)
				{
					flags[i] |= 4;
					fail = 1;
				}
				else
					PATH_TRACE;

				/* Check time stamps have not changed */
				globok = 0;
				errptr = chktimes(mdir, 0);
				PATH_FUNC_TRACE;

				if(errptr != NULL)
				{
					(void) strcpy(mess[i], errptr);
					flags[i] |= 16;
					fail = 1;
				}
				else
					PATH_TRACE;
			}
			else
			{
				globok = 0;
				errptr = chktimes(file, 0);
				PATH_FUNC_TRACE;

				if(errptr != NULL)
				{
					(void) strcpy(mess[i], errptr);
					flags[i] |= 16;
					fail = 1;
				}
				else
				{
					PATH_TRACE;
					PATH_TRACE;
					PATH_TRACE;
				}
			}

			/* Check file is not truncated */
			if(strpbrk(fop_modes[i], "wb") != NULL
			&& strpbrk(fop_modes[i], "ar") == NULL)
			{
				if ((fp = fopen(mdir, "r")) == (FILE *) NULL)
				{
					errs[0] = errno;
					xx_rpt(DELETION);
					in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
						mdir, errs[0], errname(errs[0]));
					(void) clnuprofs(mdir, (char *) NULL);
					DBUG_VOID_RETURN;
				}
				else
					PATH_TRACE;
				errno = NOERROR;
				if ((fp = freopen(file, fop_modes[i], fp)) != (FILE *) NULL)
				{
					flags[i] |= 1;
					(void) fclose(fp);
					fail = 1;
				}
				else if (errno != EROFS)
				{
					flags[i] |= 2;
					errs[i] = errno;
					fail = 1;
				}
				else
				{
					PATH_TRACE; 

					if(stat(file, &buf) < 0)
					{
						err = errno;
						if(!fail)
							xx_rpt(DELETION);
						in_rpt("stat(%s, buf) failed - errno %d (%s)", file, err, errname(err));
						DBUG_VOID_RETURN;
					}
					else if(buf.st_size == 0)
					{
						flags[i] |= 8;
						fail = 1;
					}
					else
						PATH_TRACE;
				}
			}
			else
			{			
				PATH_TRACE;
				PATH_TRACE;
				PATH_TRACE;
			}
		}
	}

	if (fail)
	{
		xx_rpt(FAILURE);
		for (i = 1 ; i < NFOP_MODES ; i++)
		{
			if (flags[i] & 1)
				in_rpt("freopen(%s) on a read-only file system did not return NULL",
					fop_modes[i]);
			else if (flags[i] & 2)
			{
				in_rpt("freopen(%s) on a read-only file system did not return expected errno",
					fop_modes[i]);
				in_rpt("expected errno EROFS, found %d(%s)",
					errs[i], errname(errs[i]));
			}

			if (flags[i] & 4)
				in_rpt("freopen(%s) on error EROFS created file", fop_modes[i]);
			if (flags[i] & 8)
				in_rpt("freopen(%s) on error EROFS truncated file", fop_modes[i]);

			if (flags[i] & 16)
			{
				in_rpt("freopen(%s) on error EROFS changed time stamps", fop_modes[i]);
				in_rpt("%s", mess[i]);
			}
		}
	}
	else
		PATH_TRACE;

	if(!fail)
		PATH_XS_RPT(2 + ((NFOP_MODES - 1) * 10)); 

	(void) tclnuprofs(mdir, file);

	DBUG_VOID_RETURN;
}

private void
sigtidy(sig)
int sig;
{
	(void) tclnuprofs(tidydir, tidyfile);

	if (oldhdlr == SIG_DFL)
		_exit(255);
	else if (oldhdlr != SIG_IGN)
		(*oldhdlr)(sig);
}

private	int
tclnuprofs(dirname, filename)
char *dirname, *filename;
{
	DBUG_ENTER("tclnuprofs");

	if (oldhdlr != NULL)
		(void) vsignal(SIGTERM, SIG_IGN);

	if (rofs == (char *) 0 || dirname == (char *) 0)
		DBUG_RETURN(1);

	if (filename != NULL)
	{
		(void) unmnt(rofs, dirname);
		if (mnt_rw(rofs, dirname) < 0)
		{
			(void) rmdir(dirname);
			DBUG_RETURN(1);
		}
	
		(void) remove(filename);
	}

	(void) unmnt(rofs, dirname);
	(void) rmdir(dirname);

	if (oldhdlr != NULL)
		(void) vsignal(SIGTERM, oldhdlr);

	DBUG_RETURN(0);
}

private	int
tsetuprofs(dirname, filename, filetype, mode)
char *dirname, *filename;
int  filetype;
mode_t mode;
{
	int	fd;
	int	err;
	char	*cp;
	int	pathok = 0;

	DBUG_LENTER("tsetuprofs", 2);

	if ((rofs = tet_getvar(VSX_ROFS)) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter %s is not set", VSX_ROFS);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (strncmp(rofs, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter %s is set to \"unsup\"", VSX_ROFS);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (setprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_MOUNT) failed");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (vxmkdir(dirname) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can't make directory %s", dirname);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (mnt_rw(rofs, dirname) < 0)
	{
		(void) rmdir(dirname);
		xx_rpt(DELETION);
		in_rpt("deletion reason: mnt_rw(%s, %s) failed", rofs, dirname);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	/* Remove any junk files left from aborted ENOSPC tests */

	cp = (char *)malloc((size_t)(strlen(dirname) + sizeof("/fill.tmpX")));
	if (cp != NULL)
	{
		(void) strcat(strcpy(cp, dirname), "/fill.tmp1");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.tmp2");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.tmp3");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.dir");
		(void) rmdir(cp);
		free((void *)cp);
	}

	/* install signal handler to tidy up if timed out */
	tidydir = dirname;
	tidyfile = filename;
	oldhdlr = vsignal(SIGTERM, sigtidy);

	/* create requested file/directory */

	if (filename != NULL)
	{
		if (filetype == 'd')
		{
			if (vxmkdir(dirname) == SYSERROR)
			{
				(void) unmnt(rofs, dirname);
				(void) rmdir(dirname);
				xx_rpt(DELETION);
				in_rpt("deletion reason: can't create directory %s",
					filename);
				DBUG_RETURN(1);
			}
			else
			{
				PATH_TRACE;
				PATH_TRACE;
			}
		}
		else
		{
			if ((fd = creat(filename, mode)) == SYSERROR)
			{
				(void) unmnt(rofs, dirname);
				(void) rmdir(dirname);
				xx_rpt(DELETION);
				in_rpt("deletion reason: can't create file %s",
					filename);
				DBUG_RETURN(1);
			}
			else
				PATH_TRACE;

			if(write(fd, "1", 1) == SYSERROR)
			{
				err = errno;
				(void) unmnt(rofs, dirname);
				(void) rmdir(dirname);
				xx_rpt(DELETION);
				in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err, errname(err));
				DBUG_RETURN(1);
			}
			else
				PATH_TRACE;

			(void) close (fd);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	if (unmnt(rofs, dirname) < 0)
	{
		(void) rmdir(dirname);
		xx_rpt(DELETION);
		in_rpt("deletion reason: unmnt(%s, %s) failed", rofs, dirname);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (mnt_ro(rofs, dirname) < 0)
	{
		(void) rmdir(dirname);
		xx_rpt(DELETION);
		in_rpt("deletion reason: mnt_ro(%s, %s) failed", rofs, dirname);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(9);

	DBUG_RETURN(0);
}
