/*
*      SCCS:  @(#)  ANSI.os/streamio/freopen/freop_t37.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/freopen - freop_t37 Rel 4.4.1 (%G)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		freop_t37 - used by T.freopen in test37()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.freopen in test37().  This should be setuid/setgid
	  to VSX_UID1/VSX_GID1.
	  usage:
		freop_t37 file uid gid vg_ident vg_dbug

	  where "file" is the file to be freopen()ed.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	20 Mar 1989
MODIFICATIONS:

		David Sawyer
		Fri Jul 27 16:13:20 BST 1990
		Added resilience code.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Stuart Boutell, 11 June 1992
			Imported from creat test 7
		
		Geoff Clare, 16 June 1992
			Use fp1 instead of fd for freopen return.

************************************************************************/

#include <dbug.h>
#include <std.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include <xnfs.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef freopen
#endif


public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	*errname();
extern	void	set_pname();

private	void	subtest37();

private	char 	*t37a_file = 	"./freopen-t.37a";

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	uid_t uid;
	gid_t gid;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if ((uid = atoi(argv[2])) != geteuid() ||
		(gid = atoi(argv[3])) != getegid())
	{
		xx_rpt(DELETION);
		if (uid != getuid())
			in_rpt("deletion reason: uid is not equal to VSX_UID1");
		else
			in_rpt("deletion reason: gid is not equal to VSX_GID1");
	}
	else
	{
		(void) setuid(uid);
		(void) setgid(gid);
		subtest37(argv[1]);
	}

	(void) unlink(t37a_file);

	return(OK);
}


private void
subtest37(file)
char	*file;
{
	int		err;
	int		fail = 0;
	struct stat	stbuf1, stbuf2;
	FILE		*fp1;
	int		pathok = 0;

	DBUG_ENTER("subtest37");

	if((fp1 = fopen(t37a_file, "w")) == (FILE *)NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
			t37a_file, err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat(file, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed on %s - errno %d (%s)",
			file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(3);

	if ((fp1 = freopen(file, "w", fp1)) == NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(\"%s\", \"w\") failed - errno %d(%s)",
				file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fclose(fp1);

	if (stat(file, &stbuf2) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed on %s - errno %d (%s)",
			file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf1.st_uid != stbuf2.st_uid)
		fail |= 02;
	else
		PATH_TRACE;

	if (stbuf1.st_gid != stbuf2.st_gid)
		fail |= 04;
	else
		PATH_TRACE;

	if (stbuf1.st_mode != stbuf2.st_mode)
		fail |= 010;
	else
		PATH_TRACE;

	if (stbuf1.st_atime != stbuf2.st_atime)
		fail |= 020;
	else
		PATH_TRACE;

	if (fail != 0)
	{
		xx_rpt(FAILURE);
		if (fail & 02)
			in_rpt("freopen(\"%s\", \"w\") changed uid of existing file",
				file);
		if (fail & 04)
			in_rpt("freopen(\"%s\", \"w\") changed gid of existing file",
				file);
		if (fail & 010)
			in_rpt("freopen(\"%s\", \"w\") changed mode of existing file",
				file);
		if (fail & 020)
			in_rpt("freopen(\"%s\", \"w\") changed st_atime of existing file",
				file);
	}
	else
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}
