/*
 *      SCCS: @(#)  ANSI.os/streamio/freopen/freop_t40_1.c Rel 4.4.1 (%G)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#define MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/freopen/freop_t40_1 Rel 4.4.1 (%G)";
#ifdef	MAIN
char	*copyright[] = {
		"(C) Copyright 1992 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		freop_t40_1.c - used by T.freopen in test40() and test41()

PROJECT:	XNFS (X/OPEN NFS tests)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int tet_main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	Exec'd by T.freopen in test40() and test41().
	This source file generates several binaries.
	freop_t40 should be setuid to VSX_UID1.
	freop_t41 should be setuid to root.
	usage:
		freop_tNN file myuid expuid vg_ident vg_dbug

	where:
		myuid is my effective uid
		expuid is the uid that we expect for the created file
		file is the file to be created with freopen().

AUTHOR:		John-Paul, UniSoft Ltd.

DATE CREATED:	22 October 1992

MODIFICATIONS:
		Geoff Clare, 8 Jan 1993
			Change atoi() to atouid().

************************************************************************/

#include <std.h>
#include <dbug.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <xnfs.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef freopen
#endif


public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	uid_t	atouid();
extern	char	*errname();
extern	void	set_pname();

private void	subtest40_41();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	uid_t	myuid;

	set_pname(argv[0]);

	if (argc < 5 || argc > 7)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 7)
		vg_dbug = argv[6];
	if (argc >= 6)
		vg_ident = argv[5];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug + 2);
	}

	if ((myuid = atouid(argv[3])) != geteuid())
	{
		xx_rpt(DELETION);
		in_rpt("expected effective uid %ld, found %ld",
			(long)myuid, (long)geteuid());
	}
	else
	{
		subtest40_41(argv[1], argv[2], atouid(argv[4]));
	}

	return(OK);
}


private void
subtest40_41(file, a_file, uid)
char	*file;
char	*a_file;
uid_t	uid;
{
	int		err;
	FILE		*fd;
	struct stat	stbuf;
	int		pathok = 0;

	DBUG_ENTER("subtest40_41");

	if ((fd = fopen(file, "w")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(\"%s\", \"w\") failed - errno %d (%s)", file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = freopen(a_file, "w", fd)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("freopen(%s, \"w\") failed - errno %d(%s)", a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	(void) fclose(fd);

	if (stat(a_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed on file %s  - errno %d (%s)", a_file, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		if (stbuf.st_uid != uid)
		{
			xx_rpt(FAILURE);
			in_rpt("freopen(\"%s\", \"w\", <fd>) did not set uid correctly", a_file );
			in_rpt("expected uid %ld, found uid %ld",
				(long)uid, (long)stbuf.st_uid);
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(4);
		}
	}

	(void) unlink(file);
	(void) unlink(a_file);

	DBUG_VOID_RETURN;
}

