/*
*      SCCS:  @(#)  ANSI.os/streamio/freopen/freopen.c	Rel 4.4.4     (07/19/99) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/freopen - T.freopen Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.freopen

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the freopen() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 1 July 87
		Removed abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Phyfos Photi, Hoskyns Group plc 23/08/88
			freopen() testset obtained from fopen() testset	

		Geoff Clare, 16 Nov 1988
			Add tests for ENAMETOOLONG, EINTR, EINVAL
			Remove test for EFAULT

		Geoff Clare, 26 May 1989
			Split out ENXIO, ETXTBSY tests into freopen_X

		Joseph Nave Tue Jun 13 16:09:59 BST 1989
			Introduced two new tests for assertions 05 and 06

		Geoff Clare, 22 Nov 1989
			Move EINVAL test to freopen_X

		Geoff Clare, 7 Dec 1989
			Avoid doing fclose(fp) when fp == NULL

		Steve Ratcliff, 19 Feb 1990
			Add ENFILE test.

		Steve Ratcliffe, 15 Mar 1990
			Add ENOSPC test.

		Geoff Clare, 4 Apr 1990
			New interface with pathtoolng() and nametoolng().

		Andrew Dingwall Wed Jun 20 15:59:02 BST 1990
		Added test of permissions on created file.

		David Sawyer
		Wed Jul 18 11:12:31 BST 1990
		Added resillience code.

		Stuart Boutell, 30 Oct 1990
			Integrated new VSX4 tests (12-17)

		Stuart Boutell, 2 Nov 1990
			Changed execl call to tet_exec

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ,deleted reason() declaration.
			
		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 7 Mar 1991
			Fix code for checking O_RDONLY etc. in flags
		M Young, 22 Nov `91
			Moved test2 into sub-program and added FIPS code.

		Stuart Boutell, 26 May 1992
		Added pathname and portable filename tests.

		Neil Moses, 1 June 1992
			Deviance tests 22, 23, 25, 29, 30,
			check file is not truncated/created.

		Neil Moses, 2 June 1992
			Added new conformance tests 33, 34, 35, 36.

		Neil Moses, 8 June 1992
			Increased timeout to account for sleep's in test 25.

		Stuart Boutell, 11 June 1992
			Added modification time checks to test 5,6,8,9.

		Stuart Boutell, 11 June 1992
			Added new tests 37,38,39.

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.
		
		Geoff Clare, 24 Sept 1992
			In "timestamp update on close" test, pass
			freopen() a different file to be opened.

		John-Paul, 22 October 1992
			XNFS enhancements

		Geoff Clare, 8 Jan 1993
			Change xnfs_anon_uid from int to uid_t.

		Geoff Clare, 25 Oct 1994
			Avoid freopen() of closed stream in several tests.

		Andrew Twigger, 1 May 1995
			Change specifiers of files associated with tests
			12 through 17 to be extern.  This matches ANSI
			Strict Ref/Def model.

		Geoff Clare, 7 Jan 1999
			Test 4 closed the same stream twice (via aliases).

		Geoff Clare, 13 Apr 2000
			Tests 38 and 39 closed the same streams twice
			(via aliases).

************************************************************************/
#include	<std.h>
#include	<limits.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<synch.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<time.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<timeout.h>
#include	<string.h>
#include	<chktimes.h>
#include	<pathres.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"freopen_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef freopen
#endif



#define	BUFFER_ELEMENTS	32
#define VSX_ROFS		"VSX_ROFS"
#define NO_TESTS	49		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)
#define SLEEPTIME	13	/* Number of secs slept in chktimes(), test25 */
#define	T_NBYTES	10

#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	DEFMODE		(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)
#define	MODEMASK	(S_ISUID|S_ISGID|S_IRWXU|S_IRWXG|S_IRWXO)
#define	MODE21		(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)
#define	MODE22		(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)
#define	MODE23_1	(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)
#define	MODE23_2	(S_IWUSR|S_IXUSR|S_IWGRP|S_IXGRP|S_IWOTH|S_IXOTH)
#define T44_UMASK	(S_IWUSR|S_IRGRP|S_IXOTH)
#define	NFOP_MODES	6
#define	MESSAGE		"abcdefghijklmnopqrstuvwxyz"
#define E_PATH_CHECK	4

#define	T38_NFILES	3

extern	char	*errname();
extern	char	*signame();
extern	char	*itoa();
extern	char	*uidtoa();
extern	char	*newstr();
extern	void	alrm();
extern	void	valrpt();
extern	char	**pathtoolng();
extern	char 	*execpath();
#if TEST_XNFS
extern  char	*xnfs_nametoolng();
#else /* TEST_XNFS */
extern  char	**nametoolng();
#endif /* TEST_XNFS */
extern	void	clntoolng();
extern	void	chktms();

extern	int 	vguid1;
extern	int 	vggid1;

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;

int	no_tests = NO_TESTS;

private	char	*t_string =	"test string";
private	char	*fop_modes[NFOP_MODES] = { "r", "w", "a", "r+", "w+", "a+" };
private	char	*t10_string =		"abcdefghijklmnop";
private	char	*t10_numstring =	"1234567890";
#if !TEST_XNFS
private char	*prog2 =		"freop_t2";
#else /* TEST_XNFS */
private char	*prog21 =		"freop_su";
private char	*prog22 =		"freop_su";
#endif /* TEST_XNFS */
private char	*prog25 =		"freop_t25";
private char	*prog29 =		"freop_t29";
private char	*prog30 =		"freop_t30";
private char	*prog37 =		"freop_t37";
#if TEST_XNFS
private char	*prog40 =		"freop_t40";
private char	*prog41 =		"freop_t41";
private char	*prog42 =		"freop_t42";
private char	*prog43 =		"freop_t43";
#endif /* TEST_XNFS */
private char	*t1_file =		"./freopen-t.1";
private char	*t1a_file =		"./freopen-t.1a";
#if !TEST_XNFS
private char	*t2_file =		"./freopen-t.2";
private char	*t2a_file =		"./freopen-t.2a";
#endif /* TEST_XNFS */
private char	*t3_file =		"./freopen-t.3";
private char	*t3a_file =		"./freopen-t.3/freopen-t.3a";
private char	*t4_file =		"./freopen-t.4";
private char	*t4a_file =		"./freopen-t.4a";
private char	*t5_file =		"./freopen-t.5";
private char	*t5a_file =		"./freopen-t.5a";
private char	*t6_file =		"./freopen-t.6";
private char	*t6a_file =		"./freopen-t.6a";
private char	*t7_file =		"./freopen-t.7";
private char	*t7a_file =		"./freopen-t.7a";
private char	*t8_file =		"./freopen-t.8";
private char	*t8a_file =		"./freopen-t.8a";
private char	*t9_file =		"./freopen-t.9";
private char	*t9a_file =		"./freopen-t.9a";
private char	*t10_file =		"./freopen-t.10";
private char	*t10a_file =		"./freopen-t.10a";
private char	*t11_file =		"./freopen-t.11";
private char	*t11a_file =		"./freopen-t.11a";
extern	char	*t12_file;
extern	char	*t12a_file;
extern	char	*t13_file;
extern	char	*t13a_file;
extern	char	*t14_file;
extern	char	*t14a_file;
extern	char	*t15_file;
extern	char	*t15a_file;
extern	char	*t16_file;
extern	char	*t16a_file;
extern	char	*t17_file;
extern	char	*t17a_file;
private char	*t18_file =		"./freopen-t.18";
private char	*t18a_file =		"./freopen-t.18/freopen-t.18a";
private char	*t19_file =		"./freopen-t.19";
private char	*t19a_file =		"./freopen-t.19a";
private char	*t19b_file =		"./freopen-t.19/freopen-t.19.1";
private char	*t20_file =		"./freopen-t.20";
private char	*t20a_file =		"./freopen-t.20a";
private char	*t20b_file =		"./freopen-t.20/freopen-t.20.1";
private char	*t21_dir =		"./freopen-d.21";
private char	*t21_file =		"./freopen-d.21/freopen-t.21.1";
private char	*t21a_file =		"./freopen-t.21a";
#if TEST_XNFS
private char	*t21_2_file =		T21_2_FILE;
private char	*t21a_2_file =		T21A_2_FILE;
#endif /* TEST_XNFS */
private char	*t22_dir =		"./freopen-d.22";
private char	*t22_file =		"./freopen-d.22/freopen-t.22.1";
private char	*t22a_file =		"./freopen-t.22a";
#if TEST_XNFS
private char	*t22_2_file =		T22_2_FILE;
private char	*t22a_2_file =		T22A_2_FILE;
#endif /* TEST_XNFS */
private char	*t23_file =		"./freopen-t.23";
private char	*t23a_file =		"./freopen-t.23a";
private char	*t24_dir =		"./freopen-d.24";
private char	*t25_dir =		"./freopen-d.25";
private char	*t25_file =		"./freopen-d.25/freopen-t.25";
private char	*t26_file =		"./freopen-t.26";
private char	*t26a_file =		"./freopen-t.26a";
#if !TEST_XNFS
private char	*t27a_file =		"./freopen-t.27a";
private char	*t28_file =		"./freopen-t.28";
private char	*t28a_file =		"./freopen-t.28a";
#endif /* TEST_XNFS */
private char	*t29_file =		"./freopen-t.29";
private char	*t30_dir =		"./freopen-d.30";
private	char	*t31a_file = 		"./freopen-t.31a";
private	char	*t31b_file = 		"./freopen-t.31b";
private	char	*t32_file = 		"./freopen-t.32";
private	char	*t33_file = 		"./freopen-t.33";
private	char	*t34_file = 		"./freopen-t.34";
private	FILE	*t34_fp;
#if !TEST_XNFS
private	char	*t35_file = 		"./freopen-t.35";
private	char	*t35a_file = 		"./freopen-t.35a";
#endif /* TEST_XNFS */
private	char	*t36_file = 		"./freopen-t.36";
private	char	*t36a_file = 		"./freopen-t.36a";
private	char	*t37_file = 		"./freopen-t.37";
private char    *t37a_file =		"./freopen-t.37a";
private	char	*t38_file = 		"./freopen-t.38";
private	char	*t39_file = 		"./freopen-t.39";
#if TEST_XNFS
private	char	*t40_file = 		"./freopen-t.40";
private char    *t40a_file =		"./freopen-t.40a";
private	char	*t41_file = 		"./freopen-t.41";
private char    *t41a_file =		"./freopen-t.41a";
private	char	*t42_file = 		"./freopen-t.42";
private char    *t42a_file =		"./freopen-t.42a";
private char	*t42_mnt = 		T42_MNT;
private	char	*t43_file = 		"./freopen-t.43";
private char    *t43a_file =		"./freopen-t.43a";
private	char	*t44_file = 		"./freopen-t.44";
private	char	*t44a_file = 		"./freopen-t.44a";
private	char	*t45_file = 		"./freopen-t.45";
private	char	*t46_file = 		"./freopen-t.46";
private	char	*t47_file = 		"./freopen-t.47";
private	char	*t48_file = 		"./freopen-t.48";
private	char	*t49_file = 		T49_FILE;
private	char	*t49a_file = 		"./freopen-t.49a";
#endif /* TEST_XNFS */

private	struct {
		int	ret;
		int	err;
	} res;
private	synch	*p_syn;
private	synch	*c_syn;

private	char	*testcomment;

private	int	t31_32fail;
public	int	testfail;


private void    comp_fin();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
public	void	test12();
public	void	test13();
public	void	test14();
public	void	test15();
public	void	test16();
public	void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();
private void	test30();
private void	test31();
private void	test32();
private void	test33();
private void	test34();
private void	test35();
private void	test36();
private void	test37();
private void	test38();
private void	test39();
private void	test40_41();
private void	test42();
private void	test43();
private void	test44();
private void	test45();
private void	test46();
private void	test47();
private void	test48();
private void	test49();
#if !TEST_XNFS
private	void	ch_t2();
#else /* TEST_XNFS */
private	void	ch_t21();
private	void	ch_t22();
#endif /* TEST_XNFS */
private	void	ch_t25();
#if !TEST_XNFS
private	void	ch_t28();
#endif /* TEST_XNFS */
private	void	ch_t29();
private	void	ch_t30();
private	void	ch_t34();
private	void	pa_t34();
private	void	ch_t37();
#if TEST_XNFS
private void	ch_t40_41();
private	void	ch_t42();
private	void	ch_t43();
private	void	ch_t44();
#endif /* TEST_XNFS */
private	FILE	*tb31_32freopen();
private	void	tb_freopen();
private	FILE	*tbfreopen();

#if TEST_XNFS
private int	snames[] = { 1, 0 };
#endif

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = comp_fin; 
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	test3, 3,
	test4, 4,
	test5, 5,
	test6, 6,
	test7, 7,
	test8, 8,
	test9, 9,
	test10, 10,
	test11, 11,
	test12, 12,
	test13, 13,
	test14, 14,
	test15, 15,
	test16, 16,
	test17, 17,
	test18, 18,
	test19, 19,
	test20, 20,
	test21, 21,
	test22, 22,
	test23, 23,
	test24, 24,
	test25, 25,
	test26, 26,
	test27, 27,
	test28, 28,
	test29, 29,
	test30, 30,
	test31, 31,
	test32, 32,
	test33, 33,
	test34, 34,
	test35, 35,
	test36, 36,
	test37, 37,
	test38, 38,
	test39, 39,
	test40_41, 40,
	test40_41, 41,
	test42, 42,
	test43, 43,
	test44, 44,
	test45, 45,
	test46, 46,
	test47, 47,
	test48, 48,
	test49, 49,
	NULL, 0
};


private void
comp_fin()
{
	(void) unlink (t1_file); 
	(void) unlink (t1a_file);
	(void) unlink (t3_file);
	(void) unlink (t3a_file);
	(void) unlink (t4_file);
	(void) unlink (t4a_file);
	(void) unlink (t5_file);
	(void) unlink (t5a_file);
	(void) unlink (t6_file);
	(void) unlink (t6a_file);
	(void) unlink (t7_file);
	(void) unlink (t7a_file);
	(void) unlink (t8_file);
	(void) unlink (t8a_file);
	(void) unlink (t9_file);
	(void) unlink (t9a_file);
	(void) unlink (t10_file);
	(void) unlink (t10a_file);
	(void) unlink (t11_file);
	(void) unlink (t11a_file);
	(void) unlink (t12_file); 
	(void) unlink (t12a_file);
	(void) unlink (t13_file); 
	(void) unlink (t13a_file);
	(void) unlink (t14_file);
	(void) unlink (t14a_file);
	(void) unlink (t15_file);
	(void) unlink (t15a_file);
	(void) unlink (t16_file);
	(void) unlink (t16a_file);
	(void) unlink (t17_file);
	(void) unlink (t17a_file);
	(void) unlink (t18_file);
	(void) unlink (t18a_file);
	(void) unlink (t19_file);
	(void) unlink (t19a_file);
	(void) unlink (t19b_file);
	(void) unlink (t20_file);
	(void) unlink (t20a_file);
	(void) unlink (t20b_file);
	(void) unlink (t21_file);
	(void) unlink (t21a_file);
	(void) unlink (t22_file);
	(void) unlink (t22a_file);
	(void) unlink (t23_file);
	(void) unlink (t25_file);
	(void) unlink (t26_file); 
	(void) unlink (t26a_file); 
#if !TEST_XNFS
	(void) unlink (t27a_file); 
	(void) unlink (t28_file); 
	(void) unlink (t28a_file); 
#endif /* TEST_XNFS */
	(void) unlink (t29_file);
	(void) rmdir (t30_dir);
	(void) unlink (t31a_file);
	(void) unlink (t31b_file);
	(void) unlink (t32_file);
	(void) unlink (t33_file);
	(void) unlink (t34_file);
#if !TEST_XNFS
	(void) unlink (t35_file);
	(void) unlink (t35a_file);
#endif /* TEST_XNFS */
	(void) unlink (t36_file);
	(void) unlink (t36a_file);
	(void) unlink (t37_file);
	(void) unlink (t37a_file);
	(void) unlink (t38_file);
#if TEST_XNFS
	(void) unlink (t40_file);
	(void) unlink (t40a_file);
	(void) unlink (t41_file);
	(void) unlink (t41a_file);
	(void) unlink (t42_file);
	(void) unlink (t42a_file);
	(void) unlink (t43_file);
	(void) unlink (t43a_file);
	(void) unlink (t44_file);
	(void) unlink (t44a_file);
	(void) unlink (t46_file);
	(void) unlink (t47_file);
	(void) unlink (t48_file);
#endif /* TEST_XNFS */

}

private void
test1()
{
	int		fd;
	int		err;
	int		fail = 0;
	FILE		*fp;
	struct stat	statb1;
	struct stat	statb2;
	char		*delmsg = "depends on success of test 3";
	int	pathok = 0;

	DBUG_ENTER("test1");
	if ((fp = fopen(t1_file, "a+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)",
			t1_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fp = freopen(t1a_file, "a+", fp)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+\") failed - errno %d(%s)",
			t1a_file, err, errname(err));
		tet_delete(3, delmsg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (stat(t1a_file, &statb1) == SYSERROR)
	{
		err = errno;
		fail = 1;
		(void) unlink(t1a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd = fileno(fp);
	if (fstat(fd, &statb2) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fstat() on file descriptor of stream returned by freopen() failed - errno %d(%s)",
			err, errname(err));
	}
	else if (fail || statb1.st_ino != statb2.st_ino
		|| statb1.st_dev != statb2.st_dev)
	{
		xx_rpt(FAILURE);
		in_rpt("freopen() did not seem to open file indicated by path");
		if (fail)
			in_rpt("stat(%s) on file freopen()'ed failed - errno %d(%s)",
				t1a_file, err, errname(err));
		else
		{
			in_rpt("dev/inode of file = 0x%lx/%ld",
				(long)statb1.st_dev, (long)statb1.st_ino);
			in_rpt("dev/inode of file descriptor of stream = 0x%lx/%ld",
				(long)statb2.st_dev, (long)statb2.st_ino);
		}
		tet_delete(3, delmsg);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}

	(void) fclose(fp);
	(void) unlink(t1_file);
	(void) unlink(t1a_file);
	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test2()
{
	DBUG_ENTER("test2");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else /* TEST_XNFS */

private void
test2()
{
	DBUG_ENTER("test2");

	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	char	*suid, *sgid;
	int	err;
	char	*args[8];

	DBUG_ENTER("ch_t2");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		exit(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog2;
	args[1] = t2_file;
	args[2] = t2a_file;
	args[3] = suid;
	args[4] = sgid;
	args[5] = vg_ident;
	args[6] = vg_dbug;
	args[7] = NULL;
	(void) tet_exec(execpath(prog2), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog2, err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test3()
{
	int		fd;
	int		err;
	FILE		*fp;
	struct stat	statb;
	int		pathok = 0;

	DBUG_ENTER("test3");
	if ((fp = fopen(t3_file, "a+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)",
			t3_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd = fileno(fp);
	(void) unlink(t3_file);
	(void) freopen(t3a_file, "a+", fp);
	if (fstat(fd, &statb) != SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("freopen() did not close previously opened file");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) unlink(t3_file);
	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	flags;
	int	err;
	FILE	*fp;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test4");
	if ((fp = fopen(t4_file, "a+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, %#lo) failed - errno %d(%s)",
			t4_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fd = creat(t4a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t4a_file, (unsigned long)MODEANY, err, errname(err));
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (freopen(t4a_file, "r", fp) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("freopen (%s, \"r\") failed - errno %d(%s)",                                   t4a_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t4_file);
		(void) unlink(t4a_file);
	        DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d,                 (%s)", err, errname(err));
		(void) fclose(fp);
		(void) unlink(t4_file);
		(void) unlink(t4a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags & O_ACCMODE) != O_RDONLY)
	{
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"r\") did not open file for reading only",                 t4a_file);
        }
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	(void) fclose(fp);
	(void) unlink(t4a_file);
	(void) unlink(t4_file);
	DBUG_VOID_RETURN;
}		

private void
test5()
{
	int	fd;
	FILE	*fp1;
	FILE	*fp;
	int	err;
	int	fail = 0;
	int	flags;
	char	buf[T_NBYTES];
	struct	stat	statb;
	time_t 	t0;
	char 	*msgd;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	if (( fp1 = fopen(t5_file, "a+"))  == (FILE*) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, %#lo) failed - errno %d(%s)", t5_file,
			(unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t5a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t5a_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((write(fd, buf, T_NBYTES)) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() did not write whole buffer");
		(void) close(fd);
		(void) fclose(fp1);
		(void) unlink(t5a_file);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	savtimes(t5a_file);
	PATH_FUNC_TRACE;

	if ((fp = freopen(t5a_file, "w", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("freopen(%s, \"w\", fp) failed - errno %d (%s)",
			t5a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t5a_file);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d (%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t5a_file);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags & O_ACCMODE) != O_WRONLY)
		fail |= 1;
	if (stat(t5a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed -  errno %d(%s)", t5a_file, err,                        errname(err));
		(void) fclose(fp1);
		(void) unlink(t5a_file);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statb.st_size != 0L)
		fail |= 2;

	globok = 0;
	if ( (msgd = chktimes(t5a_file,NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w\") (file truncation) - parent directory: %s",t5a_file,msgd);
	}
	else
		PATH_FUNC_TRACE;

	/* don't fclose(fp) */

	if (unlink(t5a_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unlink() on test file failed - errno %d (%s)",
			err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T5_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = freopen(t5a_file, "w", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"w\") failed - errno %d(%s) ", t5a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t5a_file);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t5a_file);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags & O_ACCMODE) != O_WRONLY)
	{
		fail |= 4;
	} else
		PATH_TRACE;

	if (stat(t5a_file, &statb) == SYSERROR)
		fail |= 8;
	else if (statb.st_size != 0L)
		fail |= 16;
	else
		PATH_TRACE;

	if (fail)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("freopen(%s, \"w\") did not open file for write-only",
				t5_file);
		if (fail & 2)
			in_rpt("freopen(%s, \"w\") did not truncate file",
				t5_file);
		if (fail & 4)
			in_rpt("freopen(%s, \"w\") did not open file for write-only when file did not exist",
				t5_file);
		if (fail & 8)
			in_rpt("freopen(%s, \"w\") did not create file",
				t5_file);
		else if (fail & 16)
			in_rpt("freopen(%s, \"w\") did not create file with zero size",
				t5_file);

	}
	else
		PATH_TRACE;

	globok = 0;
	if ( (msgd = chktimes(".",NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w\") (new file) - %s",t5a_file,msgd);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	chktms(t5a_file, t0);
	PATH_FUNC_TRACE;
		
	if (!testfail)
		PATH_XS_RPT(18);

	(void) fclose(fp1);
	(void) unlink(t5_file);
	(void) unlink(t5a_file);
	DBUG_VOID_RETURN;
	
}

private void
test6()
{
	int	fd;
	FILE	*fp, *fp1;
	int	size;
	int	err;
	int	fail = 0;
	int	flags;
	char	buf[T_NBYTES];
	struct	stat	statb;
	time_t	t0;
	char 	*msgd;
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	if (( fp1 = fopen(t6_file, "a+"))  == (FILE*) NULL)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fopen(%s, %#lo) failed - errno %d(%s)", t6_file,
			(unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t6a_file, MODEANY)) == SYSERROR) 
	{
		err = errno;
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
	                t6a_file, (unsigned long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((write(fd, buf, T_NBYTES)) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() did not write whole buffer");
		(void) close(fd);
		(void) fclose(fp1);
		(void) unlink(t6a_file);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if ((fp = freopen(t6a_file, "a", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("freopen(%s, \"w\", fp) failed - errno %d (%s)",
			t6a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t6a_file);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d (%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t6a_file);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_WRONLY)
		fail |= 1;
	if (!(flags & O_APPEND ))
		fail |= 2;

	if (stat(t6a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",                                        t6a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t6a_file);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((size = statb.st_size) != (long) T_NBYTES)
		fail |= 4;

	/* don't fclose(fp) */

	if (unlink(t6a_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unlink() on test file failed - errno %d (%s)",
			err, errname(err));
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T6_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */


	if ((fp = freopen(t6a_file, "a", fp1)) == (FILE *) NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a\") failed to open file that did not exist - errno %d(%s)",
			t6a_file, err, errname(err));
		(void) unlink(t6a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t6a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_WRONLY)
		fail |= 8;
	if (!(flags & O_APPEND))
		fail |= 16;
	if (stat(t6a_file, &statb) == SYSERROR)
		fail |= 32;
	else if (statb.st_size != 0L)
		fail |= 64;
	if (fail)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("freopen(%s, \"a\") did not open file for write-only",
				t6a_file);
		if (fail & 2)
			in_rpt("freopen(%s, \"a\") did not open file for append",
				t6a_file);
		if (fail & 4)
			in_rpt("freopen(%s, \"a\") changed size of file - expected %d, found %d",
				t6a_file, T_NBYTES, size);
		if (fail & 8)
			in_rpt("freopen(%s, \"a\") when file did not exist, did not open file for write-only",
				t6a_file);
		if (fail & 16)
			in_rpt("freopen(%s, \"a\") when file did not exist, did not open file for append",
				t6a_file);
		if (fail & 32)
			in_rpt("freopen(%s, \"a\") when file did not exist, did not creat file",
				t6a_file);
		else if (fail & 64)
			in_rpt("freopen(%s, \"a\") when file did not exist, did not creat file with zero length",
				t6a_file);
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( (msgd = chktimes(".",NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a\") (new file) - parent directory: %s",t6a_file,msgd);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	chktms(t6a_file, t0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(14);

	(void) fclose(fp1);
	(void) unlink(t6_file);
	(void) unlink(t6a_file);
	DBUG_VOID_RETURN;

}

private void
test7()
{
	FILE	*fp;
	FILE	*fp1;
	int	fd;
	int	err;
	int	flags;
	char	buf[T_NBYTES];
	struct	stat	statb;
	int	pathok = 0;

	DBUG_ENTER("test7");
	if ((fp1 = fopen(t7_file, "a+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("deletion reason: fopen(%s, %#lo) failed - errno %d(%s)",
			t7_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t7a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t7a_file, (unsigned long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t7_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() on file descriptor of stream failed to write whole buffer");
		(void) fclose(fp1);
		(void) close(fd);
		(void) unlink(t7_file);
		(void) unlink(t7a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);


	if ((fp = freopen(t7a_file, "r+", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("freopen(%s, \"w\", fp) failed - errno %d (%s)", t7a_file, err,                errname(err));
		(void) fclose(fp1);
		(void) unlink(t7_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fcntl () on file descriptor of stream failed - errno %d (%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t7_file);
		(void) unlink(t7a_file);
		DBUG_VOID_RETURN;
	}	
	else
		PATH_TRACE;
	if (stat(t7a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t7a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t7_file);
		(void) unlink(t7a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_RDWR || statb.st_size != T_NBYTES)
	{
		xx_rpt(FAILURE);
		if ((flags & O_ACCMODE) != O_RDWR)
			in_rpt("freopen(%s, \"r+\", fp) did not open file for reading and writing", t7a_file);
		if (statb.st_size != T_NBYTES)
			in_rpt("freopen(%s, \"r+\", fp) changed size of file, expected size %d, found %ld",
				t7a_file, T_NBYTES, (long)statb.st_size);
	}
	else
		PATH_XS_RPT(6);

	(void) fclose(fp1);
	(void) unlink(t7a_file);
	(void) unlink(t7_file);
	DBUG_VOID_RETURN;
}

private void
test8()
{
	int	fd;
	FILE	*fp, *fp1;
	int	err;
	int	fail = 0;
	int	flags;
	char	buf[T_NBYTES];
	struct	stat	statb;
	time_t	t0;
	char	*msgd;
	int	pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

	if (( fp1 = fopen(t8_file, "a+"))  == (FILE*) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen(%s, %#lo) failed - errno %d(%s)", t8_file,
			(unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t8a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t8a_file, (unsigned long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((write(fd, buf, T_NBYTES)) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() did not write whole buffer");
		(void) close(fd);
		(void) fclose(fp1);
		(void) unlink(t8a_file);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	savtimes(t8a_file);
	PATH_FUNC_TRACE;

	if ((fp = freopen(t8a_file, "w+", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("freopen(%s, \"w+\", fp) failed - errno %d (%s)",
			t8a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d (%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t8a_file);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_RDWR)
		fail |= 1;

	if (stat(t8a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t8_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t8a_file);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (statb.st_size != 0L)
		fail |= 2;

	globok = 0;
	if ( (msgd = chktimes(t8a_file,NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w+\") (file truncation) - %s",t8a_file,msgd);
	}
	else
		PATH_FUNC_TRACE;

	/* don't fclose(fp) */

	if (unlink(t8a_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unlink() on test file failed - errno %d (%s)",
			err, errname(err));
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T8_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */


	if ((fp = freopen(t8a_file, "w+", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w+\") failed to open file that did not exist - errno %d(%s)",
			t8a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t8a_file);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t8_file);
		(void) unlink(t8a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags & O_ACCMODE) != O_RDWR)
		fail |= 4;
	if (stat(t8a_file, &statb) == SYSERROR)
		fail |= 8;
	else if (statb.st_size != 0L)
		fail |= 16;
	if (fail)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("freopen(%s, \"w+\") did not open file for reading and writing ", 
				t8a_file);
		if (fail & 2)
			in_rpt("freopen(%s, \"w+\") did not truncate file",
				t8a_file);
		if (fail & 4)
			in_rpt("freopen(%s, \"w+\") when file did not exist, did not open file for reading and writing",
				t8a_file);
		if (fail & 8)
			in_rpt("freopen (%s, \"w+\") did not create file",                               t8a_file);
		if (fail & 16)
			in_rpt("freopen(%s, \"w+\") when file did not exist, did not creat file with zero length",
				t8a_file);
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( (msgd = chktimes(".",NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w+\") (new file) - %s",t8a_file,msgd);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	chktms(t8a_file, t0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(16);

	(void) fclose(fp1);
	(void) unlink(t8_file);
	(void) unlink(t8a_file);
	DBUG_VOID_RETURN;
}


private void
test9()
{
	int		fd;
	FILE		*fp, *fp1;
	int		err;
	int		fail = 0;
	int		flags;
	long		size;
	char		buf[T_NBYTES];
	struct stat	statb;
	time_t		t0;
	char		*msgd;
	int		pathok = 0;

	DBUG_ENTER("test9");

	if (( fp1 = fopen(t9_file, "a+"))  == (FILE*) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fopen(%s, MODEANY) failed - errno %d(%s)", t9_file,
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t9a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t9a_file, (unsigned long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) fclose(fp1);
		(void) close(fd);
		(void) unlink(t9_file);
		(void) unlink(t9a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	if ((fp = freopen(t9a_file, "a+", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+\") failed to open file - errno %d(%s)",
			t9a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t9a_file);
		(void) unlink(t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t9a_file);
		(void) unlink(t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_RDWR)
		fail |= 1;
	if (!(flags & O_APPEND))
		fail |= 2;
	if (stat(t9a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t9a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t9_file);
		(void) unlink(t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((size = statb.st_size) != (long) T_NBYTES)
		fail |= 4;

	/* don't fclose(fp) */

	if (unlink(t9a_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unlink() on test file failed - errno %d (%s)",
			err, errname(err));
		(void) unlink(t9_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T9_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = freopen(t9a_file, "a+", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+\") failed to open file that did not exist - errno %d(%s)",
			t9a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t9a_file);
		(void) unlink(t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t9_file);
		(void) unlink(t9a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_RDWR)
		fail |= 8;
	if (!(flags & O_APPEND))
		fail |= 16;
	if (stat(t9a_file, &statb) == SYSERROR)
		fail |= 32;
	else if (statb.st_size != 0L)
		fail |= 64;
	if (fail)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("freopen(%s, \"a+\") did not open file for reading and writeing ",
				t9a_file);
		if (fail & 2)
			in_rpt("freopen(%s, \"a+\") did not open file for append",
				t9a_file);
		if (fail & 4)
			in_rpt("freopen(%s, \"a+\") changed size of file - expected %d, found %ld",
				t9a_file, T_NBYTES, size);
		if (fail & 8)
			in_rpt("freopen(%s, \"a+\") when file did not exist, did not open file for reading and writeing ",
				t9a_file);
		if (fail & 16)
			in_rpt("freopen(%s, \"a+\") when file did not exist, did not open file for append",
				t9a_file);
		if (fail & 32)
			in_rpt("freopen(%s, \"a+\") when file did not exist, did not creat file",
				t9a_file);
		else if (fail & 64)
			in_rpt("freopen(%s, \"a+\") when file did not exist, did not creat file with zero length",
				t9a_file);
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( (msgd = chktimes(".",NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+\") (new file) - %s",t9a_file,msgd);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	chktms(t9a_file, t0);
	PATH_FUNC_TRACE;

	if(!testfail)
		PATH_XS_RPT(14);

	(void) fclose(fp1);
	(void) unlink(t9_file);
	(void) unlink(t9a_file);
	DBUG_VOID_RETURN;
}

private void
test10()
{
	int	fd;
	int	fd2;
	FILE	*fp;
	off_t	offval;
	int	pathok = 0;

	DBUG_ENTER("test10");

	if ((fd = open(t10_file, O_RDWR|O_CREAT, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to open file %s", t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd2 = dup(fd)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to dup() file descriptor");
		(void) close(fd);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fp = fdopen(fd2, "w")) == (FILE *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fdopen file %s", t10_file);
		(void) close(fd);
		(void) close(fd2);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, t10_string, (unsigned)strlen(t10_string)) != strlen(t10_string))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to write %d bytes to %s",
			(int)strlen(t10_string), t10_file);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (lseek(fd, (off_t) 0, SEEK_SET) == (off_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek failed on file %s", t10_file);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fwrite((void *) t10_numstring, strlen(t10_numstring), (size_t)1, fp) != 1)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fwrite %d bytes to %s",
			(int)strlen(t10_numstring), t10_file);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (freopen(t10a_file, "w+", fp) == (FILE *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to freopen file %s", t10a_file);
		(void) close(fd);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((offval = lseek(fd, (off_t) 0, SEEK_CUR)) != strlen(t10_numstring))
	{
		xx_rpt(FAILURE);
		in_rpt("File pointer not correctly set after freopen()");
		in_rpt("Expected: %ld", (long) strlen(t10_numstring));
		in_rpt("Observed: %ld", (long) offval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(8);
	}

	(void) close(fd);
	(void) fclose(fp);
	(void) unlink(t10_file);
	(void) unlink(t10a_file);

	DBUG_VOID_RETURN;
}

private void
test11()
{
	FILE	*fp;
	char	*tres;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test11");

	if ((fp = fopen(t11_file, "w")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t11_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	globok = 0;
	savtimes(t11_file);
	PATH_FUNC_TRACE;

        if (putc('A', fp) == EOF)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("putc('A', fp) failed on file %s, errno %d(%s)",
			t11_file, err, errname(err));
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (freopen(t11a_file, "w", fp) == (FILE *)NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("freopen() failed on file %s, errno %d(%s)",
			t11a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ((tres = chktimes(t11_file, NEWMTIME | NEWCTIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(tres);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(5);
	}

	(void) unlink(t11_file);
	(void) unlink(t11a_file);
	DBUG_VOID_RETURN;
}

private	void
test18()
{
	int		fd;
	int		err1;
	int		err2;
	int		fail = 0;
	FILE		*fp;
	int		pathok = 0;

	DBUG_ENTER("test18");
	if ((fd = creat(t18_file, MODEANY)) == SYSERROR)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#lo) failed - errno %d(%s)",
			t18_file, (unsigned long)MODEANY, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	if ((fp = fopen(t18_file, "r")) == (FILE *) NULL)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"r\") failed - errno %d(%s)",
			t18_file, err1, errname(err1));
		(void) close(fd);
		(void) unlink(t18_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	errno = NOERROR;
	if ((fp = freopen(t18a_file, "r", fp)) != (FILE *) NULL
		|| errno != ENOTDIR)
	{
		fail |= 1;
		(void) fclose(fp);
	}
	else if (errno != ENOTDIR)
	{
		err2 = errno;
		fail |= 2;
	}
	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("freopen(%s) when %s was a file, did not return NULL",
				t18a_file, t18_file);
		if (fail & 2)
		{
			in_rpt("freopen(%s) when %s was a file, did not return expected errno",
				t18a_file, t18_file);
			in_rpt("expected errno %d(ENOTDIR), received %d(%s)",
				ENOTDIR, err2, errname(err2));
		}
	}
	else
		PATH_XS_RPT(2);

	(void) unlink(t18_file);
	DBUG_VOID_RETURN;
}

private	void
test19()
{
	int		err1;
	int		err2;
	int		fail = 0;
	FILE		*fp;
	FILE		*tfp;
	int		pathok = 0;

	DBUG_ENTER("test19");
	if (unlink(t19_file) == SYSERROR && errno != ENOENT)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("could not unlink %s - errno %d(%s)",
			t19_file, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fp = fopen(t19a_file, "w")) == (FILE *) NULL)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t19a_file, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((tfp = freopen(t19_file, "r", fp)) != (FILE *) NULL)
	{
		fail |= 1;
		(void) fclose(tfp);
	}
	else if (errno != ENOENT)
	{
		err2 = errno;
		fail |= 2;
	}

	if ((fp = fopen(t19a_file, "w")) == (FILE *) NULL)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t19a_file, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((tfp = freopen(t19_file, "r+", fp)) != (FILE *) NULL)
	{
		fail |= 4;
		(void) fclose(tfp);
	}
	else if (errno != ENOENT)
	{
		err2 = errno;
		fail |= 8;
	}

	if (fail)
	{
		xx_rpt(FAILURE);
		DBUG_PRINTF("test19", "fail - %d", fail);
		if (fail & 1)
			in_rpt("freopen(%s,%s) on file that did not exist, did not return NULL", t19_file, "r");
		if (fail & 2)
		{
			in_rpt("freopen(%s,%s) on file that did not exist, did not return expected errno", t19_file, "r");
			in_rpt("expected errno %d(ENOENT), received %d(%s)",
				ENOENT, err2, errname(err2));
		}
		if (fail & 4)
			in_rpt("freopen(%s,%s) on file that did not exist, did not return NULL", t19_file, "r+");
		if (fail & 8)
		{
			in_rpt("freopen(%s,%s) on file that did not exist, did not return expected errno", t19_file, "r+");
			in_rpt("expected errno %d(ENOENT), received %d(%s)",
				ENOENT, err2, errname(err2));
		}
	}
	else
		PATH_XS_RPT(3);

	(void) unlink(t19a_file);
	DBUG_VOID_RETURN;
}

private	void
test20()
{
	int		err1;
	int		err3;
	int		fail = 0;
	FILE		*fp;
	FILE		*tfp;
	int		pathok = 0;

	DBUG_ENTER("test20");
	if (unlink(t20_file) == SYSERROR && errno != ENOENT)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("could not unlink %s - errno %d(%s)",
			t20_file, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fp = fopen(t20a_file, "w")) == (FILE *) NULL)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t20a_file, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((tfp = freopen(t20b_file, "w", fp)) != (FILE *) NULL)
	{
		fail |= 1;
		(void) fclose(tfp);
	}
	else if (errno != ENOENT)
	{
		err3 = errno;
		fail |= 2;
	}
	if (fail)
	{
		xx_rpt(FAILURE);
		DBUG_PRINTF("test20", "fail - %d", fail);
		if (fail & 1)
			in_rpt("freopen(%s) when %s did not exist, did not return NULL",
				t20b_file, t20_file);
		if (fail & 2)
		{
			in_rpt("freopen(%s) when %s did not exist, did not return expected errno",
				t20b_file, t20_file);
			in_rpt("expected errno %d(ENOENT), received %d(%s)",
				ENOENT, err3, errname(err3));
		}
	}
	else
		PATH_XS_RPT(2);

	(void) unlink(t20a_file);
	DBUG_VOID_RETURN;
}

private void
test21()
{
	FILE		*fp;
	int		err1;
	int		err3;
	int		fail = 0;
	int		pathcheck = 0;
#if TEST_XNFS
	int		rc = 0;
#endif /* TEST_XNFS */
	int		pathok = 0;

	DBUG_ENTER("test21");

	testfail = 0;

	if (crdir(t21_dir, (mode_t)MODE21, (int) getuid(), (int) getgid())
								== SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t21_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if ((fp = fopen(t21a_file, "w")) == (FILE *) NULL)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t21a_file, err1, errname(err1));
		(void) unlink(t21a_file);
		(void) rmdir(t21_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((fp = freopen(t21_file, "w", fp)) != (FILE *) NULL)
	{
		fail |= 1;
		(void) fclose(fp);
	}
	else if (errno != EACCES)
	{
		fail |= 2;
		err3 = errno;
	}
	if (fail)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("freopen(%s, \"w\") in a non-searchable directory did not return NULL",
				t21_file);
		else if (fail & 2)
		{
			in_rpt("freopen(%s, \"w\") in a non-searchable directory did not return expected errno",
				t21_file);
			in_rpt("expected errno EACCES, found %d(%s)",
				err3, errname(err3));
		}
	}
	else
		PATH_TRACE;

	(void) unlink(t21_file);
	(void) unlink(t21a_file);
	(void) rmdir(t21_dir);

#if TEST_XNFS

	if (tet_sync(SP_T21_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t21, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T21_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pathcheck = 6;

#else /* TEST_XNFS */

	pathcheck = 3;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
ch_t21()
{
	int err;
	char *args[6];

	DBUG_ENTER("ch_t21");

	args[0] = prog21;
	args[1] = t21_2_file;
	args[2] = t21a_2_file;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(execpath(prog21), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog21, err, errname(err));
	
	DBUG_EXIT(NOK);
}

#endif /* TEST_XNFS */

private void
test22()
{
	FILE		*fp;
	int		err1;
	int		err2;
	int		fail = 0;
	int		pathcheck = 0;
#if TEST_XNFS
	int		rc = 0;
#endif /* TEST_XNFS */
	int		pathok = 0;

	DBUG_ENTER("test22");
	
	testfail = 0;

	if (crdir(t22_dir, (mode_t)MODE22, (int) getuid(), (int) getgid())
								== SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t22_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fp = fopen(t22a_file, "w")) == (FILE *) NULL)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t22a_file, err1, errname(err1));
		(void) unlink(t22a_file);
		(void) rmdir(t22_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	errno = NOERROR;
	if ((fp = freopen(t22_file, "w", fp)) != (FILE *) NULL)
	{
		fail |= 1;
		(void) fclose(fp);
	}
	else if (errno != EACCES)
	{
		fail |= 2;
		err2 = errno;
	}
	
	if(eaccess(t22_file, F_OK) != SYSERROR)
		fail |= 4;
	else
		PATH_TRACE;

	if (fail)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("freopen(%s, \"w\") in a non-writable directory did not return NULL",
				t22_file);
		else if (fail & 2)
		{
			in_rpt("freopen(%s, \"w\") in a non-writable directory did not return expected errno",
				t22_file);
			in_rpt("expected errno EACCES, found %d(%s)",
				err2, errname(err2));
		}
		else if (fail & 4)
			in_rpt("freopen(%s, \"w\") in a non-writable directory on error EACCES created file", t22_file);
	}
	else
		PATH_TRACE;

	(void) unlink(t22_file);
	(void) unlink(t22a_file);
	(void) rmdir(t22_dir);

#if TEST_XNFS

	if (tet_sync(SP_T22_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t22, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T22_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pathcheck = 7;

#else /* TEST_XNFS */

	pathcheck = 4;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
ch_t22()
{
	int err;
	char *args[6];

	DBUG_ENTER("ch_t22");

	args[0] = prog22;
	args[1] = t22_2_file;
	args[2] = t22a_2_file;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(execpath(prog22), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog22, err, errname(err));
	
	DBUG_EXIT(NOK);
}

#endif /* TEST_XNFS */


private void
test23()
{
	FILE	*fp;
	struct	stat buf;
	int	fd;
	int	i;
	int	fail = 0;
	int	err1;
	int	err2;
	int	pathok = 0;

	DBUG_ENTER("test23");
	if ((fd = creat(t23_file, (mode_t)MODE23_1)) == SYSERROR)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			t23_file, MODE23_1, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(write(fd, "1", 1) == SYSERROR)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	for (i = 1; i < NFOP_MODES; i++)
	{
		if ((fp = fopen(t23a_file, "w+")) == (FILE *) NULL)
		{
			err1 = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fopen(%s, \"w+\") failed - errno %d(%s)",
				t23a_file, err1, errname(err1));
			(void) unlink(t23a_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		errno = NOERROR;
		if ((fp = freopen(t23_file, fop_modes[i], fp)) != (FILE *) NULL)
		{
			if (!fail)
				xx_rpt(FAILURE);
			in_rpt("freopen(%s, %s) in a non-writable file did not return NULL",
				t23_file, fop_modes[i]);
			(void) fclose(fp);
			fail++;
		}
		else if (errno != EACCES)
		{
			err2 = errno;
			if (!fail)
				xx_rpt(FAILURE);
			in_rpt("freopen(%s, %s) in a non-writable file did not return expected errno",
				t23_file, fop_modes[i]);
			in_rpt("expected errno EACCES, found %d(%s)",
				err2, errname(err2));
			fail++;
		}
		else
			PATH_TRACE;

		if(stat(t23_file, &buf) < 0)
		{
			err1 = errno;
			if (!fail)
				xx_rpt(DELETION);
			in_rpt("stat(%s, buf) failed - errno %d (%s)",
				t23_file, err1, errname(err1));
			DBUG_VOID_RETURN;
		}
		else if(buf.st_size == 0)
		{
			if (!fail)
				xx_rpt(FAILURE);
			in_rpt("freopen(%s, %s) in a non-writable file truncated file",
				t23_file, fop_modes[i]);
			fail++;
			(void) fclose(fp);
		}
		else
			PATH_TRACE;
	}

	(void) unlink(t23_file);

	if ((fd = creat(t23_file, (mode_t)MODE23_2)) == SYSERROR)
	{
		err1 = errno;
		if (!fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			t23_file, MODE23_2, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(write(fd, "1", 1) == SYSERROR)
	{
		err1 = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < NFOP_MODES; i += 3)
	{
		if ((fp = fopen(t23a_file, "w+")) == (FILE *) NULL)
		{
			err1 = errno;
			if (!fail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: fopen(%s, \"w+\") failed - errno %d(%s)",
				t23a_file, err1, errname(err1));
			(void) unlink(t23a_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		errno = NOERROR;
		if ((fp = freopen(t23_file, fop_modes[i], fp)) != (FILE *) NULL)
		{
			if (!fail)
				xx_rpt(FAILURE);
			in_rpt("freopen(%s, %s) in a non-readable file did not return NULL",
				t23_file, fop_modes[i]);
			(void) fclose(fp);
			fail++;
		}
		else if (errno != EACCES)
		{
			err2 = errno;
			if (!fail)
				xx_rpt(FAILURE);
			in_rpt("freopen(%s, %s) in a non-readable file did not return expected errno",
				t23_file, fop_modes[i]);
			in_rpt("expected errno EACCES, found %d(%s)",
				err2, errname(err2));
			fail++;
		}
		else
			PATH_TRACE;

		if(stat(t23_file, &buf) < 0)
		{
			err1 = errno;
			if (!fail)
				xx_rpt(DELETION);
			in_rpt("stat(%s, buf) failed - errno %d (%s)",
				t23_file, err1, errname(err1));
			DBUG_VOID_RETURN;
		}
		else if(buf.st_size == 0)
		{
			if (!fail)
				xx_rpt(FAILURE);
			in_rpt("freopen(%s, %s) in a non-writable file truncated file",
				t23_file, fop_modes[i]);
			fail++;
			(void) fclose(fp);
		}
		else
			PATH_TRACE;
	}

	if (!fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(5+((NFOP_MODES - 1)*3)+(((NFOP_MODES + 2) / 3)*3));
	}

	(void) unlink(t23_file);
	(void) unlink(t23a_file);
	DBUG_VOID_RETURN;
}


private void
test24()
{
	FILE		*fp;
	int		errs[NFOP_MODES * 2];
	int		flags[NFOP_MODES];
	int		fail = 0;
	int		i;
	int		pathok = 0;

	DBUG_ENTER("test24");
	if (crdir(t24_dir, (mode_t)MODEANY, (int) getuid(), (int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t24_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	for (i = 1 ; i < NFOP_MODES ; i++)
	{
		errno = NOERROR;
		flags[i] = 0;
		DBUG_PRINTF("test24", "mode=%s", fop_modes[i]);
		if ((fp = fopen(t24_dir, "r")) == (FILE *) NULL)
		{
			errs[0] = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
				t24_dir, errs[0], errname(errs[0]));
			(void) rmdir(t24_dir);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		DBUG_PRINTF("test24", "point 1");
		errno = NOERROR;
		if ((fp = freopen(t24_dir, fop_modes[i], fp)) != (FILE *) NULL)
		{
			flags[i] = 1;
			(void) fclose(fp);
			fail = 1;
		}
		else if (errno != EISDIR)
		{
			flags[i] = 2;
			errs[i * 2] = errno;
			fail = 1;
		}
		DBUG_PRINTF("test24", "point 2");
	}
	if (fail)
	{
		xx_rpt(FAILURE);
		for (i = 1 ; i < NFOP_MODES ; i++)
		{
			DBUG_PRINTF("test24", "point 3");
			if (flags[i] & 1)
				in_rpt("freopen(%s) on a directory did not return NULL",
					fop_modes[i]);
			else if (flags[i] & 2)
			{
				in_rpt("freopen(%s) on a directory did not return expected errno",
					fop_modes[i]);
				in_rpt("expected errno EISDIR, found %d(%s)",
					errs[i*2], errname(errs[i*2]));
			}
			DBUG_PRINTF("test24", "point 4");
		}
	}
	else
		PATH_XS_RPT(1+(NFOP_MODES-1));

	(void) rmdir(t24_dir);
	DBUG_VOID_RETURN;
}


private void
test25()
{

	DBUG_ENTER("test25");

	(void) cppair(ch_t25, NULLFN, 2 * WAITTIME + SLEEPTIME, 0);

	(void) rmdir(t25_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t25()
{
	char		*args[7];
	char		*rofs;

	DBUG_ENTER("ch_t25");

	if ((rofs = tet_getvar(VSX_ROFS)) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s not defined", VSX_ROFS);
		DBUG_VOID_RETURN;
	}

	args[0] = prog25;
	args[1] = rofs;
	args[2] = t25_dir;
	args[3] = t25_file; 
	args[4] = vg_ident; 
	args[5] = vg_dbug; 
	args[6] = (char *) NULL;
	(void)	tet_exec(execpath(args[0]), args, environ);
	vsxerror("test25: tet_exec(%s, ...) failed - errno %d(%s)",
		prog25, errno, errname(errno));
	er_rpt("test 25: tet_exec() failed");

	DBUG_VOID_RETURN;
}

private void
test26()
{
	char		**longname;
	FILE		*fp;
	int		err, err1, err2;
	int		fail = 0;
	int		pathok = 0;

	DBUG_ENTER("test26");

	globok = 0;
	longname = pathtoolng(t26_file);
	PATH_FUNC_TRACE;
	if (longname == NULL)
		DBUG_VOID_RETURN;

	errno = NOERROR;
	if ((fp = fopen(t26a_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
			t26a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((fp = freopen(longname[0], "w", fp)) == (FILE *) NULL)
	{
		fail |= 1;
		err1 = errno;
	}
	else
	{
		PATH_TRACE;
		(void) fclose(fp);
	}
	(void) unlink(t26_file);

	if (longname[1] != NULL)
	{
		errno = NOERROR;
		if ((fp = fopen(t26a_file, "w")) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
				t26a_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		errno = NOERROR;
		if ((fp = freopen(longname[1], "w", fp)) != (FILE *) NULL)
		{
			fail |= 2;
			(void) fclose(fp);
		}
		else if (errno != ENAMETOOLONG)
		{
			fail |= 4;
			err2 = errno;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE; PATH_TRACE; 
	}

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
		{
			in_rpt("freopen(<%d char path>, \"w\", fp) failed - errno %d (%s)",
				(int)strlen(longname[0]), err1, errname(err1));
		}
		if (fail & 2)
		{
			in_rpt("freopen(<%d char path>, \"w\", fp) did not return NULL",
				(int)strlen(longname[1]));
		}
		if (fail & 4)
		{
			in_rpt("freopen(<%d char path>, \"w\", fp) did not set errno correctly",
				(int)strlen(longname[1]));
			in_rpt("expected %d (ENAMETOOLONG), received %d (%s)",
				ENAMETOOLONG, err2, errname(err2));
		}
	}
	else
		PATH_XS_RPT(5);

	clntoolng();
	(void) unlink(t26_file);
	(void) unlink(t26a_file);
	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test27()
{
	DBUG_ENTER("test27");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test27()
{
	char		**longname;
	FILE		*fp;
	int		err, err1, err2;
	int		fail = 0;
	int		pathok = 0;

	DBUG_ENTER("test27");

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
		DBUG_VOID_RETURN;
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((fp = fopen(t27a_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
			t27a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((fp = freopen(longname[0], "w", fp)) == (FILE *) NULL)
	{
		fail |= 1;
		err1 = errno;
	}
	else
	{
		PATH_TRACE;
		(void) fclose(fp);
	}

	if (longname[1] != NULL)
	{
		errno = NOERROR;
		if ((fp = fopen(t27a_file, "w")) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
				t27a_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		errno = NOERROR;
		if ((fp = freopen(longname[1], "w", fp)) != (FILE *) NULL)
		{
			fail |= 2;
			(void) fclose(fp);
		}
		else if (errno != ENAMETOOLONG)
		{
			fail |= 4;
			err2 = errno;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE; PATH_TRACE;
	}

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
		{
			in_rpt("freopen(<%d char filename>, \"w\", fp) failed - errno %d (%s)",
				(int)strlen(longname[0]), err1, errname(err1));
		}
		if (fail & 2)
		{
			in_rpt("freopen(<%d char filename>, \"w\", fp) did not return NULL",
				(int)strlen(longname[1]));
		}
		if (fail & 4)
		{
			in_rpt("freopen(<%d char filename>, \"w\", fp) did not set errno correctly",
				(int)strlen(longname[1]));
			in_rpt("expected %d (ENAMETOOLONG), received %d (%s)",
				ENAMETOOLONG, err2, errname(err2));
		}
	}
	else
		PATH_XS_RPT(5);

	(void) unlink(longname[0]);
	clntoolng();
	(void) unlink(t27a_file);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test28()
{
	DBUG_ENTER("test28");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test28()
{
	int	err;

	DBUG_ENTER("test28");
	if (mkfifo(t28_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(\"%s\", %#lo) failed - errno %d (%s)",
			t28_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t28, NULLFN, 2*WAITTIME, 0);

	(void) unlink(t28_file);
	(void) unlink(t28a_file);

	DBUG_VOID_RETURN;
}

private void
ch_t28()
{

	FILE 	*fp;
	int 	fail = 0;
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t28");

	errno = NOERROR;
	if ((fp = fopen(t28a_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
			t28a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)
	errno = NOERROR;
	fp = freopen(t28_file, "r", fp);
	err = errno;
	CLEAR_ALARM

	if (fp != NULL)
	{
		fail = 1;
		(void) fclose(fp);
	}
	else if (err != EINTR)
		fail = 2;
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("freopen(FIFO, \"r\", fp) did not return NULL when interrupted");
		if (fail & 2)
		{
			in_rpt("freopen(FIFO, \"r\", fp) did not set errno correctly when interrupted");
			in_rpt("expected errno EINTR, received %d(%s)",
				err, errname(err));
		}
	}
	else
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test29()
{
	int 	fd;
	int 	err;

	DBUG_ENTER("test29");

	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}

	(void) umask((mode_t) 0);

	if ((fd = creat(t29_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("could not creat() %s - errno %d (%s)", t29_file, err,
			errname(err));
		DBUG_VOID_RETURN;
	}
	(void) close(fd);

	(void) cppair(ch_t29, NULLFN, 8 * WAITTIME, 0);

	(void) unlink(t29_file);

	DBUG_VOID_RETURN;
}

private void
ch_t29()
{
	char	*suid, *sgid;
	int 	err;
	char	*args[8];

	DBUG_ENTER("ch_t29");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		exit(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	args[0] = prog29;
	args[1] = t29_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = "F";
	args[5] = vg_ident;
	args[6] = vg_dbug;
	args[7] = NULL;

	(void) tet_exec(execpath(prog29), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %d (%s)", err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test30()
{

	DBUG_ENTER("test30");

	(void) cppair(ch_t30, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t30()
{
	int 	err;
	char	*args[5];

	DBUG_ENTER("ch_t30");

	args[0] = prog30;
	args[1] = t30_dir;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;

	(void) tet_exec(execpath(prog30), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}

private FILE *
tb31_32freopen(tbpath, tboflag, tbstream)
char *tbpath;
char *tboflag;
FILE *tbstream;
{
	char	*reppath = tbpath;
	FILE 	*rval;
	int	err;
	char	pathbuf[30];
	int	pathok = 0;

	DBUG_LENTER("tb31_32freopen", 2);

	errno = NOERROR;
	rval = freopen(tbpath, tboflag, tbstream);

#ifdef TESTBED
	rval = NULL ;
	errno++;
#endif
	err = errno;

	if(rval == NULL) {
		if (t31_32fail++ == 0)
			xx_rpt(FAILURE);

		/* Don't print PATH_MAX length path name */
		if (strlen(tbpath) > 99)
		{
			(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
			reppath = pathbuf;
		}

		in_rpt("freopen(\"%s\", %s) returned NULL. Expected file pointer",
			reppath, tboflag);
		in_rpt("errno was %d (%s)", err, errname(err));
		DBUG_RETURN(rval);
	} else
		PATH_TRACE;



	PATH_FUNC_RPT(1);

	DBUG_RETURN(rval);
}

private void
test31()
{
	/* pathname resolution */

	char 	*p1, *p2;
	int 	npairs, i, err;
	FILE	*of1, *of2;
	FILE	*fd1, *fd2;
	struct stat stbuf1, stbuf2;
	int	pathok = 0;

	DBUG_ENTER("test31");

	t31_32fail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_CREAT);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_CREAT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test31", "testing \"%s\" == \"%s\"", p1, p2);

		errno = NOERROR;
		if ((of1 = fopen(t31a_file, "w+")) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fopen(%s, \"a+\") failed - errno %d(%s)",
				t31a_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		errno = NOERROR;
		if ((of2 = fopen(t31b_file, "w+")) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fopen(%s, \"a+\") failed - errno %d(%s)",
				t31b_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		globok = 0;
		fd1 = tb31_32freopen(p1, "r", of1);
		PATH_FUNC_TRACE;

		globok = 0;
		fd2 = tb31_32freopen(p2, "r", of2);
		PATH_FUNC_TRACE;

		if ( (fd1 == NULL) || (fd2 == NULL) )
		{
			(void) fclose(of1);
			(void) fclose(of2);
			(void) unlink(t31a_file);
			(void) unlink(t31b_file);
			continue;
		}

		if (fstat(fileno(fd1), &stbuf1) == SYSERROR ||
		    fstat(fileno(fd2), &stbuf2) == SYSERROR)
		{
			if (t31_32fail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: fstat() failed");
		}
		else if (stbuf1.st_dev != stbuf2.st_dev ||
		         stbuf1.st_ino != stbuf2.st_ino)
		{
			if (t31_32fail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("freopen() did not resolve \"%s\" and \"%s\" to the same file",
				p1, p2);
				
		}
		else
			PATH_TRACE;

#ifdef TESTBED2
#undef fd1
#endif

		(void) fclose(of1); /* This should close the reopened files */
		(void) fclose(of2);

		(void) unlink(t31a_file);
		(void) unlink(t31b_file);
	}

	if (t31_32fail == 0)
		PATH_XS_RPT(1+(5*npairs));
	
	cln_pathres();


	DBUG_VOID_RETURN;
}

private void
test32()
{
	/* portable filenames */

	char 	**p;
	FILE	*fd, *ofd;
	int	i, err, nbuf;
	struct  {
		struct  stat s;
		char    *p;
		} buf[ BUFFER_ELEMENTS ];
	int	pathok = 0;

	DBUG_ENTER("test32");

	t31_32fail = 0;

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{	
		if (i++ == 0)
			PATH_TRACE;
		if (eaccess(*p, F_OK) == 0 && remove(*p) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("could not remove \"%s\" - errno %d (%s)",
				*p, err, errname(err));
			DBUG_VOID_RETURN;
		}
	}

#ifdef TESTBED2
	(void) close(crfile(portfnam[0], (mode_t)MODEANY, (int)getuid(), (int)getgid()));
#endif

	nbuf = -1;
	for (p = portfnam; *p != NULL; p++)
	{
		errno = NOERROR;
		if ((ofd = fopen(t32_file, "w")) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
				t32_file, err, errname(err));
			DBUG_VOID_RETURN;
		}

		globok = 0;
		fd = tb31_32freopen(*p, "w", ofd);
		if (fd != NULL) {
			nbuf++;
			if(nbuf > BUFFER_ELEMENTS-1) {
				if (t31_32fail++ == 0)
					xx_rpt(DELETION);
				in_rpt("portfnam too big, or buf too small");
				DBUG_VOID_RETURN;
			}

			if(fstat(fileno(fd), &(buf[nbuf].s) ) == SYSERROR) {
				if (t31_32fail++ == 0)
					xx_rpt(DELETION);
				in_rpt("deletion reason: fstat() failed");
				nbuf--;
			} else {
				buf[nbuf].p = *p;
				DBUG_PRINTF("fstat", "%s %ld %ld", buf[nbuf].p,
						(long)buf[nbuf].s.st_dev,
						(long)buf[nbuf].s.st_ino);

				if (nbuf == 0)
					PATH_FUNC_TRACE;
			}
		
			(void) fclose(fd); 
		} else {
			(void) fclose(ofd);
		}
	}

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		(void) unlink(*p);
	}

	for (i=0; i<nbuf; i++) {
		int j;
		for (j=i+1; j<nbuf; j++) {
			if (buf[i].s.st_dev == buf[j].s.st_dev &&
			    buf[i].s.st_ino == buf[j].s.st_ino)
			{
				if (t31_32fail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("freopen() resolved \"%s\" and \"%s\" to the same file",
					buf[i].p, buf[j].p);
					
			}
		}
	}

	if (t31_32fail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
test33()
{
	FILE		*fp;
	int		fd1, fd2;
	int		err;
	int		rval;
	struct stat	statb;
	int 		pathok = 0;
	
	DBUG_ENTER("test33");
	errno = NOERROR;

	if((fd2 = open(t33_file, O_CREAT|O_RDONLY, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fp = fopen(t33_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
			t33_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	fd1 = fileno(fp);

	if(close(fd2) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: close() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) freopen(t33_file, "w", fp);
		
	errno = NOERROR;
	if ((rval = fstat(fd1, &statb)) != SYSERROR || errno != EBADF)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fstat() on file descriptor associated with stream after freopen() did not return expected values");
		valrpt(SYSERROR, rval, EBADF, err);
		tet_delete(34, "depends on success of test 33");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}
	
	(void) fclose(fp);
	(void) unlink(t33_file);
	DBUG_VOID_RETURN;
}

private void
test34()
{
#if TEST_XNFS
	char	*val;
#endif
	int		err;
	int 		pathok = 0; 

	DBUG_ENTER("test34");

#if TEST_XNFS
	val = tet_getvar("XNFS_REMLOCK_SUPP");

	if( (val == NULL) || (*val == '\0') )
		DBUG_VOID_RETURN;

	if( (*val == 'n') || (*val == 'N') )
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("Parameter XNFS_REMLOCK_SUPP is set to 'N'");
		DBUG_VOID_RETURN;
	}
#endif /* TEST_XNFS */

	errno = NOERROR;
	if ((t34_fp = fopen(t34_file, "w+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w+\") failed - errno %d(%s)",
			t34_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) fputs(t_string, t34_fp);
	if ((c_syn = opensync()) == (synch *) NULL
		|| (p_syn = opensync()) == (synch *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: opensync() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;		

	(void) cppair(ch_t34, pa_t34, WAITTIME, 0);

	(void) closesync(p_syn);
	(void) closesync(c_syn);
	(void) fclose(t34_fp);
	(void) unlink(t34_file);

	DBUG_VOID_RETURN;
}

private void
ch_t34()
{
	struct flock	lck;
	int	fd;
	int 	pathok = 0; 

	DBUG_ENTER("ch_t34");
	
	res.ret = SUCCEED;
	lck.l_type = F_WRLCK;
	lck.l_whence = SEEK_SET;
	lck.l_start = 0L;
	lck.l_len = 0L;
	lck.l_pid = 0;
	fd = fileno(t34_fp);
	if (fcntl(fd, F_SETLK, &lck) == SYSERROR)
	{
		res.ret = FAIL;
		res.err = errno;
	}
	else if (freopen(t34_file, "w+", t34_fp) == NULL)
	{
		res.ret = EOF;
		res.err = errno;
	}
	else
		PATH_TRACE;

	if (res.ret == SUCCEED && pathok != 1)
		res.ret = E_PATH_CHECK;
		
	errno = NOERROR;
	if (sendsync(c_syn[0][1], (char *) &res, sizeof(res), WAITTIME)
							!= sizeof(res))
	{
		vsxerror("sendsync() failed - errno %d(%s)",
			errno, errname(errno));
		DBUG_TET_EXIT(A_SENDSYNC);
	}
	else if (waitsync(p_syn[0][0], (char *) &res, sizeof(res),
					WAITTIME) != sizeof(res))
	{
		vsxerror("waitsync() failed - errno %d(%s)",
			errno, errname(errno));
		DBUG_TET_EXIT(A_WAITSYNC);
	}
	
	DBUG_VOID_RETURN;
}

private void
pa_t34()
{
	int	fd;
	int	err;
	struct flock	lck;
	int 	pathok = 0; 

	DBUG_ENTER("pa_t34");

	errno = NOERROR;
	fd = fileno(t34_fp);
	lck.l_type = F_RDLCK;
	lck.l_whence = SEEK_SET;
	lck.l_start = 0L;
	lck.l_len = 0L;
	lck.l_pid = 0;
	if (waitsync(c_syn[0][0], (char *) &res, sizeof(res),
					WAITTIME) != sizeof(res))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitsync() failed - errno %d(%s)",
			errno, errname(errno));
      		DBUG_VOID_RETURN;
	}
	if (res.ret == E_PATH_CHECK)
		res.ret = SUCCEED;
	else
		PATH_TRACE; 
	if (res.ret != SUCCEED)
	{
		xx_rpt(DELETION);
		if (res.ret == EOF)
			in_rpt("deletion reason: freopen() failed - errno %d(%s)",
				res.err, errname(res.err));
		else
			in_rpt("deletion reason: fcntl(F_SETLK) failed - errno %d(%s)",
				res.err, errname(res.err));
	}
	else if (fcntl(fd, F_SETLK, &lck) == SYSERROR)
	{
		if (errno == EAGAIN || errno == EACCES)
		{
			xx_rpt(FAILURE);
			in_rpt("file that was write locked was still locked after freopen()");
		}
		else
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: could not read-lock file: errno %d(%s)",
				err, errname(err));
		}
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(2);
	}
	
	
	if (sendsync(p_syn[0][1], (char *) &res, sizeof(res), WAITTIME)
						!= sizeof(res))
	{
		DBUG_VOID_RETURN;
	}

	globok = 1; /* must be set for cppair */

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test35()
{
	DBUG_ENTER("test35");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test35()
{
	char	buf[sizeof(MESSAGE)];
	int	rval;
	int	fd_rd,	fd_wr;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test35");

	testfail = 0;
	if ((rval = mkfifo(t35_file , MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : mkfifo(t35_file) failed");
		valrpt(NOERROR,rval,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd_rd = open(t35_file , O_RDONLY|O_NONBLOCK)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : open(t35_file, O_RDONLY|O_NONBLOCK) failed");
		valrpt(NOERROR,fd_rd,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd_wr = open(t35_file , O_WRONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : open(t35_file, O_WRONLY) failed");
		valrpt(NOERROR,fd_wr,NOERROR,err);
		(void) close(fd_rd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = write(fd_wr , MESSAGE , (unsigned)strlen(MESSAGE))) !=
		strlen(MESSAGE))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() failed");
		valrpt((int)strlen(MESSAGE),rval,NOERROR,err);
		(void) close(fd_rd);
		(void) close(fd_wr);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	tb_freopen(t35a_file, fd_wr, "w");
	PATH_FUNC_TRACE;
	globok = 0;
	tb_freopen(t35a_file, fd_rd, "r");
	PATH_FUNC_TRACE;

	if(testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd_rd = open(t35_file , O_RDONLY|O_NONBLOCK)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : open(t35_file, O_RDONLY|O_NONBLOCK) failed");
		valrpt(NOERROR,fd_rd,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd_wr = open(t35_file , O_WRONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : open(t35_file, O_WRONLY) failed");
		valrpt(NOERROR,fd_wr,NOERROR,err);
		(void) close(fd_rd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(5*WAITTIME)

	errno = NOERROR;
	rval = read(fd_rd , buf , (unsigned)strlen(MESSAGE));

	CLEAR_ALARM

	if (rval != SYSERROR || errno != EAGAIN)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("close failed to discard data in FIFO");
		in_rpt("read on reopened FIFO failed");
		if (rval > 0)
			in_rpt("data read \"%.*s\" ",rval,buf);
		valrpt(SYSERROR,rval,EAGAIN,err);
	}
	else
		PATH_TRACE;

	(void) close(fd_rd);
	(void) close(fd_wr);

	if(!testfail)
		PATH_XS_RPT(10);

	(void)  unlink(t35_file);
	(void)  unlink(t35a_file);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test36()
{
	int		fd;
	int		err;
	time_t		curtime;
	struct	stat	statb;
	char		buf[1];
	int		pathok = 0;

	DBUG_ENTER("test36");

	if ((fd = open(t36_file, O_RDWR|O_CREAT, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT) failed - errno %d(%s)",
			t36_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, "AB", 2) != 2)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() failed - errno %d(%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t36_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (lseek(fd, (off_t) 0, SEEK_SET) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek to start of %s failed with - error %d(%s)",
			t36_file, err, errname(err));
		(void) close(fd);
		(void) unlink(t36_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(2);

	if (read(fd, buf, 1) != 1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : read() failed - errno %d(%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t36_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, "C", 1) != 1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() failed - errno %d(%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t36_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testcomment = "close on a file descriptor updates all time-related feilds still marked";
	
#if TEST_XNFS
	if( (xnfs_rtime_c(&curtime, SP_T36_RTIME_START)) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	curtime = time( (time_t *) 0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	globok = 0;
	tb_freopen(t36a_file, fd, "w");
	PATH_FUNC_TRACE;

	(void) sleep(2);

	if (stat(t36_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)", t36_file, err, errname(err));
		(void) unlink(t36_file);
		(void) unlink(t36a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statb.st_atime < curtime && statb.st_atime > curtime +2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_atime not updated by freopen(), got %ld but expected between %ld and %ld",
			(long)statb.st_atime, (long)curtime, (long)curtime +2);
	}
	else
		PATH_TRACE;

	if (statb.st_ctime < curtime && statb.st_ctime > curtime +2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_ctime not updated by freopen(), got %ld but expected between %ld and %ld",
			(long)statb.st_ctime, (long)curtime, (long)curtime +2);
	}
	else
		PATH_TRACE;

	if (statb.st_mtime < curtime && statb.st_mtime > curtime +2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_mtime not updated by freopen(), got %ld but expected between %ld and %ld",
			(long)statb.st_mtime, (long)curtime, (long)curtime +2);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(11);

	(void) unlink(t36_file);
	(void) unlink(t36a_file);

	DBUG_VOID_RETURN;
}

private void
test37()
{
	/* freopen() on existing file does not change mode, ownership,
	   or st_atime. */

	int		fd;
	int		err;

	DBUG_ENTER("test37");
	if ((fd = open(t37_file, O_WRONLY|O_CREAT, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_CREAT, %#lo) failed - errno %d (%s)",
			t37_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	(void) close(fd);

	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t37, NULLFN, WAITTIME, 0);

	(void) unlink(t37_file);
	(void) unlink(t37a_file);

	DBUG_VOID_RETURN;
}

private void
ch_t37()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t37");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		exit(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog37;
	args[1] = t37_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog37), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog37, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test38()
{
	/* freopen returns the lowest available file descriptor */

	int 	i, j, err;
	FILE	*fp[T38_NFILES+1];
	int	fd[T38_NFILES+1];
	int	pathok = 0 ;

	DBUG_ENTER("test38");

	testfail = 0;

	for (i = 0; i < T38_NFILES+1 ; i++)
	{
		if (i==0)
			PATH_TRACE;

		if (( fp[i] = fopen(t38_file, "w"))  == (FILE*) NULL)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
				t38_file, err, errname(err));
			for (j = 0; j < i; j++)
				(void) fclose(fp[j]);
			(void) unlink(t38_file);
			DBUG_VOID_RETURN;
		}
	}

	for (i = 0 ; i < T38_NFILES && testfail == 0 ; i++)
	{
		if (i==0)
			PATH_TRACE;

		if(freopen(t38_file, "r", fp[i]) == (FILE *)NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("freopen(%s, \"w\") failed - errno %d(%s)",
				t38_file, err, errname(err));
			for (j = 0; j < T38_NFILES+1; j++) 
			{
				if (j != i)
					(void) fclose(fp[j]);
			}
			(void) unlink(t38_file);
			DBUG_VOID_RETURN;
		}
		if((fd[i] = fileno(fp[i])) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fileno() failed - errno %d(%s)",
				err, errname(err));
			for (j = 0; j < T38_NFILES+1; j++) 
				(void) fclose(fp[j]);
			(void) unlink(t38_file);
			DBUG_VOID_RETURN;
		}
	}

	for (i = 1; i < T38_NFILES && testfail == 0; i++)
	{
		if (i==1)
			PATH_TRACE;
		if (fd[i-1] >= fd[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("freopen() did not return lowest numbered file descriptor");
		}
	}

	if (testfail == 0)
	{
		if (fclose(fp[0]) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("fclose() failed - errno %d (%s)",
				err, errname(err));
		}
		else
		{
			if(freopen(t38_file, "w", fp[T38_NFILES]) == (FILE *)NULL)
			{
				xx_rpt(DELETION);
				in_rpt("freopen(%s, \"w\") failed - errno %d(%s)",
					t38_file, err, errname(err));
				for (i = 1; i < T38_NFILES; i++)
					(void) fclose(fp[i]);
				(void) unlink(t38_file);
				DBUG_VOID_RETURN;
			}
			if((fd[T38_NFILES] = fileno(fp[T38_NFILES])) == SYSERROR)
			{
				err = errno;
				xx_rpt(DELETION);
				in_rpt("deletion reason: fileno() failed - errno %d(%s)",
					err, errname(err));
				for (i = 1; i < T38_NFILES+1; i++)
					(void) fclose(fp[i]);
				DBUG_VOID_RETURN;
			}
	
			if(fd[T38_NFILES] != fd[0])
			{
				xx_rpt(FAILURE);
				in_rpt("freopen() did not re-use closed file descriptor");
			}
			else
			{
				PATH_TRACE;
				PATH_XS_RPT(4);
			}
		}
	}

	for (i = 1; i < T38_NFILES+1; i++)
		(void) fclose(fp[i]);

	(void) unlink(t38_file);

	DBUG_VOID_RETURN;
}

private void
test39()
{
	/* opened files are set to remain open across exec calls */

	FILE	*fp;
	int 	fd, err, flags;
	int	pathok = 0;

	DBUG_ENTER("test39");

	if((fp = fopen(t39_file, "w")) == (FILE *)NULL) {
		xx_rpt(DELETION);
		in_rpt("fopen() returned NULL");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(freopen(t39_file, "r", fp) == (FILE *)NULL) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("freopen() returned NULL and set errno to %d", err);
		(void) unlink(t39_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if((fd = fileno(fp)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fileno() failed - errno %d(%s)",
			err, errname(err));
		(void) unlink(t39_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ((flags = fcntl(fd, F_GETFD, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl(F_GETFD) failed - errno %d (%s)",
			err, errname(err));
	}
	else if ((flags & FD_CLOEXEC) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("FD_CLOEXEC was set for opened file");
	}
	else
	{
		PATH_XS_RPT(3);
	}

	(void) fclose(fp);
	(void) unlink(t39_file);

	DBUG_VOID_RETURN;
}

private void
tb_freopen(path, tbfd, mode)
char *path;
int tbfd;
char *mode;
{
	FILE	*fp;
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("tb_freopen");

	if((fp = fdopen(tbfd, mode)) == (FILE *)NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("fdopen(%d, \"%s\") failed - errno %d (%s)", 
			tbfd, mode, err, errname(err));
		(void) close(tbfd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(freopen(path, mode, fp) == NULL)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("freopen() failed - errno %d (%s)", 
			err, errname(err));
		(void) close(tbfd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
test40_41()
{
	/* 
	   Test replaces part of test2 when testing XNFS

	   test40: server does not perform user ID mapping
	   test41: server performs user ID mapping
	*/

	char	*file;
	char	*a_file;

	DBUG_ENTER("test40_41");

	switch (tet_thistest) {
	case 40:
		if(uids()!=0)
		{
			xx_rpt(DELETION);
			in_rpt("'uids' failed.");
			DBUG_VOID_RETURN;
		}
		file = t40_file;
		a_file = t40a_file;
		break;
	case 41:
		if(xnfs_uids()!=0)
		{
			xx_rpt(DELETION);
			in_rpt("'uids' failed.");
			DBUG_VOID_RETURN;
		}
		file = t41_file;
		a_file = t41a_file;
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("unexpected tet_thistest = %d", tet_thistest);
		DBUG_VOID_RETURN;
	}

	if (eaccess(file, F_OK) == 0 && unlink(file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("can not remove %s", file);
		DBUG_VOID_RETURN;
	}

	if (eaccess(a_file, F_OK) == 0 && unlink(a_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("can not remove %s", a_file);
		DBUG_VOID_RETURN;
	}


	(void) cppair(ch_t40_41, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t40_41()
{
	char	*file, *a_file, *prog, *fuid, *suid;
	int	err;
	char	*args[8];

	DBUG_ENTER("ch_t40_41");

	switch (tet_thistest) {
	case 40:
		if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
			exit(A_NOMEM);
		fuid = suid;
		file = t40_file;
		a_file = t40a_file;
		prog = prog40;
		break;
	case 41:
		suid = "0";
		if ((fuid = newstr(uidtoa(xnfs_anon_uid))) == (char *) NULL)
			exit(A_NOMEM);
		file = t41_file;
		a_file = t41a_file;
		prog = prog41;
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("unexpected tet_thistest = %d",
			tet_thistest);
		DBUG_VOID_RETURN;
	}

	args[0] = prog;
	args[1] = file;
	args[2] = a_file;
	args[3] = suid;
	args[4] = fuid;
	args[5] = vg_ident;
	args[6] = vg_dbug;
	args[7] = NULL;
	(void) tet_exec(execpath(prog), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		prog, err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test40_41()
{
	DBUG_ENTER("test40_41");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test42()
{
	DBUG_ENTER("test42");

	(void) cppair(ch_t42, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t42_mnt);

	DBUG_VOID_RETURN;
}

private void
ch_t42()
{
	char	*fsys, *sgid;
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t42");

	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}

	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	if ((fsys = tet_getvar("VSX_MOUNT_DEV")) == NULL || *fsys == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: VSX_MOUNT_DEV is not defined");
		DBUG_VOID_RETURN;
	}

	if (crdir(t42_mnt, MODEANY, (int) getuid(),
			(int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t42_mnt);
		DBUG_VOID_RETURN;
	}

	args[0] = prog42;
	args[1] = sgid;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog42), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog42, err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test42()
{
	DBUG_ENTER("test42");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test43()
{
	DBUG_ENTER("test43");

	(void) cppair(ch_t43, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t43()
{
	char	*sgid;
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t43");

	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	args[0] = prog43;
	args[1] = sgid;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog43), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog43, err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test43()
{
	DBUG_ENTER("test43");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test44()
{
	DBUG_ENTER("test44");

	if (eaccess(t44_file, F_OK) == 0 && unlink(t44_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t44_file);
		DBUG_VOID_RETURN;
	}

	if (eaccess(t44a_file, F_OK) == 0 && unlink(t44a_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t44a_file);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t44, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

#define T44_INSTANCES	2

private void
ch_t44()
{
	FILE		*fp;
	int		fail = 0;
	mode_t		mode1;
	mode_t		mode2;
	struct stat	statb;
	int		i, flag[T44_INSTANCES];
	int		err;
	int 		pathok = 0;

	DBUG_ENTER("ch_t44");

	(void) unlink(t44_file);
	(void) unlink(t44a_file);

	if (stat(".", &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\".\") failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for ( i = 0; i < T44_INSTANCES; i++ )
		flag[i] = 0;
		
	if ((fp = fopen(t44_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)", t44_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = freopen(t44a_file, "w", fp)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"w\", fp) failed - errno %d(%s)", t44a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t44a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(%s) failed - errno %d(%s)",
			t44a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((mode1 = (statb.st_mode & MODEMASK)) != DEFMODE)
		flag[0] |= 1;

	if ( flag[0] != 0 )
		fail = 1;
	else
		PATH_TRACE;
		
	(void) fclose(fp);
	(void) unlink(t44_file);
	(void) unlink(t44a_file);
	(void) umask((mode_t)T44_UMASK);

	if ((fp = fopen(t44_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
			t44_file, err, errname(err));
		(void) umask((mode_t)0);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = freopen(t44a_file, "w", fp)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"w\", fp) failed - errno %d(%s)", t44a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) umask((mode_t)0);
	if (stat(t44a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(%s) failed - errno %d(%s)",
			t44a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fclose(fp);
		
	if ((mode2 = (statb.st_mode & MODEMASK)) != (DEFMODE & ~T44_UMASK))
		flag[1] |= 1;

	if ( flag[1] != 0 )
		fail |= 2;
	else
		PATH_TRACE;
		
	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
		{
			if (flag[0] & 020)
				in_rpt("freopen() did not create file with mode %#o when umask was set to 0, found %#lo",
				DEFMODE, (long) mode1);
		}
		else if (fail & 2)
		{
			if (flag[1] & 020)
			{
				in_rpt("freopen() did not create file with correct mode when umask was set to %#o", T44_UMASK);
				in_rpt("expected %#o, found %#lo", 
				DEFMODE & ~T44_UMASK, (long) mode2);
			}
		}
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(10);
	}

	(void) unlink(t44_file);
	(void) unlink(t44a_file);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test44()
{
	DBUG_ENTER("test44");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test45()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test45");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbfreopen(t45_file, longname, "w", SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	(void) unlink(t45_file);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test45()
{
	DBUG_ENTER("test45");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test46()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test46");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbfreopen(t46_file, longname, "w", SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t46_file);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test46()
{
	DBUG_ENTER("test46");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test47()
{
	char	*longname;
	FILE	*fd;
	int	pathok = 0;

	DBUG_ENTER("test47");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	fd = tbfreopen(t47_file, longname, "w", SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd != NULL)
		(void) fclose(fd);
	(void) unlink(t47_file);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test47()
{
	DBUG_ENTER("test47");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test48()
{
	char	*longname;
	FILE	*fd;
	int	pathok = 0;

	DBUG_ENTER("test48");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	fd = tbfreopen(t48_file, longname, "w", SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd != NULL)
		(void) fclose(fd);
	(void) unlink(t48_file);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test48()
{
	DBUG_ENTER("test48");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test49()
{
	FILE	*fd;
	int	err;
	int	length;
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("test49");

	if (tet_sync(SP_T49_WRITE_START, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE_START failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = fopen(t49_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)",
			t49_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Write to file */

	length = strlen(XNFS_STRING);

	errno = NOERROR;
	ret = fwrite((void *)XNFS_STRING, (size_t)1, (size_t)length, fd);
	if(ret != length)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fwrite() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * freopen should flush character to remote file. Server side checks.
	 */

	if ((fd = freopen(t49a_file, "w", fd)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+\") failed - errno %d(%s)",
			t49a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T49_WRITE_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to WRITE_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	fclose(fd);

	(void) unlink(t49a_file);

	PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test49()
{
	DBUG_ENTER("test49");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private FILE *
tbfreopen(tbpath, a_tbpath, tbmode, expret, experrno)
char *tbpath;
char *a_tbpath;
char *tbmode;
int expret;
int experrno;
{
	char	*reppath = tbpath;
	int	err;
	FILE	*fd;
	char	pathbuf[30];
	int	pathok = 0;

	DBUG_LENTER("tbfopen", 2);

	errno = NOERROR;
	if( (fd = fopen(tbpath, tbmode)) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)", tbpath, err, errname(err));
		DBUG_RETURN(fd);
	}
	else
		PATH_TRACE;
	
	errno = NOERROR;
	fd = freopen(a_tbpath, tbmode, fd);
	err = errno;

	if (expret == SUCCEED)
	{
		PATH_TRACE;
		if (fd != NULL)
		{
			PATH_FUNC_RPT(2);
			DBUG_RETURN(fd);
		}
	}
	else
	{
		PATH_TRACE;
		if (fd == NULL && experrno == err)
		{
			if(eaccess(a_tbpath, F_OK) == SYSERROR)
			{
				PATH_FUNC_RPT(2);
				DBUG_RETURN(fd);
			}
			else
			{
				in_rpt("on error file was created");
			}
		}
	}

	/* Don't print PATH_MAX length path name */
	if (strlen(a_tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(a_tbpath));
		reppath = pathbuf;
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);

	if( (expret != SUCCEED) && (fd != NULL) )
	{
		(void) fclose(fd);
		in_rpt("freopen(%s, \"w\", <fd>) did not return NULL", reppath);
	}
	else if( (expret == SUCCEED) && (fd == NULL ) )
	{
		in_rpt("freopen(%s, \"w\", <fd>) failed - errno %d(%s)", reppath, err, errname(err));
		
	}
	else if(experrno != err ) 
	{
		in_rpt("errno set incorrectly - expected %d (%s), received %d (%s)",
			ENAMETOOLONG, errname(experrno), err, errname(err));
	}

	DBUG_RETURN(fd);
}

#endif /* TEST_XNFS */
