/*
*      SCCS:  @(#)  ANSI.os/streamio/freopen/freopen1.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		freopen1.c

PROJECT:	VSX

DESCRIPTION:
	Tests for the freopen() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Keith Holder, UniSoft Ltd.
DATE CREATED:	Thu Sep 13 11:10:18 BST 1990
MODIFICATIONS:	
	Stuart Boutell, 30 Oct 1990
		Integrated into existing freopen testset

	Stuart Boutell, 11 June 1991
		Added timestamp checks to tests 13,14,16,17.
	
	Geoff Clare, 16 June 1992
		Fixed XPG3 mode build problems.

	Geoff Clare, 25 Oct 1994
		Avoid freopen() of closed stream in several tests.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<time.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<chktimes.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"freopen_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef freopen
#endif



#define WAITTIME	(5 * SPEEDFACTOR)
#define	T_NBYTES	10

#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();
extern	void	savtimes();
extern	char	*chktimes();

extern int testfail;

#if	!defined(_XOPEN_SOURCE) || TEST_XPG > 3
private	char	*msg,*msgd;
#endif

public	char	*t12_file =		"./freopen.12";
public	char	*t12a_file =		"./freopen.12a";
public	char	*t13_file =		"./freopen.13";
public	char	*t13a_file =		"./freopen.13a";
public	char	*t14_file =		"./freopen.14";
public	char	*t14a_file =		"./freopen.14a";
public	char	*t15_file =		"./freopen.15";
public	char	*t15a_file =		"./freopen.15a";
public	char	*t16_file =		"./freopen.16";
public	char	*t16a_file =		"./freopen.16a";
public	char	*t17_file =		"./freopen.17";
public	char	*t17a_file =		"./freopen.17a";


public void	test12();
public void	test13();
public void	test14();
public void	test15();
public void	test16();
public void	test17();
public void	chktms();

#ifdef	TESTBED1
#define	stat(x,y)	stat(".",y)
#endif

#ifdef	TESTBED2
#define	fcntl(x,y,z)	fcntl(0,y,z)
#endif

#ifdef	TESTBED3
#define	savtimes(x)	savtimes("..")
#endif

#if	!defined(_XOPEN_SOURCE) || TEST_XPG > 3

public void
test12()
{
	int	flags;
	int	err;
	FILE	*fp, *fp1;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test12");
	if ((fp1 = fopen(t12_file, "a+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)",
			t12_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fd = creat(t12a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t12a_file, (long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) close(fd);
		(void) unlink(t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (( fp = freopen(t12a_file, "rb", fp1 )) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen (%s, \"rb\") failed - errno %d(%s)",                                   t12a_file, err, errname(err));
	        (void) fclose(fp1);	
		(void) unlink(t12_file);
		(void) unlink(t12a_file);
	        DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d,                 (%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t12_file);
		(void) unlink(t12a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags & O_ACCMODE) != O_RDONLY)
	{
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"rb\") did not open file for reading only",                 t12a_file);
        }
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	(void) fclose(fp1);
	(void) unlink(t12a_file);
	(void) unlink(t12_file);
	DBUG_VOID_RETURN;
}		

public void
test13()
{
	int	fd;
	FILE	*fp1;
	FILE	*fp;
	int	err;
	int	flags;
	char	buf[T_NBYTES];
	struct	stat	statb;
	time_t	t0;
	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

	if (( fp1 = fopen(t13_file, "a+"))  == (FILE*) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)", t13_file,                               err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t13a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",                 t13a_file, (long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t13a_file);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((write(fd, buf, T_NBYTES)) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() did not write whole buffer");
		(void) close(fd);
		(void) fclose(fp1);
		(void) unlink(t13a_file);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	savtimes(t13a_file);

	if ((fp = freopen(t13a_file, "wb", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb\") failed - errno %d (%s)",
			t13a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t13a_file);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d (%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t13a_file);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags & O_ACCMODE) != O_WRONLY)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb\") did not open file for write-only",
				t13a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat(t13a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) testfailed -  errno %d(%s)", t13a_file, err,                        errname(err));
		(void) fclose(fp1);
		(void) unlink(t13a_file);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statb.st_size != 0L)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb\") did not truncate file", t13a_file);
		(void) fclose(fp1);
		(void) unlink(t13a_file);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* don't fclose(fp) */

	if ( (msg = chktimes(t13a_file,NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb\") (file truncation) - %s",t13a_file,msg);
		(void) fclose(fp1);
		(void) unlink(t13a_file);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (unlink(t13a_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unlink() on test file failed - errno %d (%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	savtimes(".");

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T13_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = freopen(t13a_file, "wb", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb\") failed - errno %d(%s) ", t13a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t13a_file);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t13a_file);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags & O_ACCMODE) != O_WRONLY)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb\") did not open file for write-only when file did not exist",
			t13a_file);
		(void) fclose(fp1);
		(void) unlink(t13a_file);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(t13a_file, &statb) == SYSERROR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb\") did not create file", t13a_file);
		(void) fclose(fp1);
		(void) unlink(t13a_file);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statb.st_size != 0L)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb\") did not create file with zero size",
			t13a_file);
		(void) fclose(fp1);
		(void) unlink(t13a_file);
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;


	if ( (msgd = chktimes(".",NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb\") (new file) - parent directory: %s",t13a_file,msgd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	chktms(t13a_file, t0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(18);

	(void) fclose(fp1);
	(void) unlink(t13_file);
	(void) unlink(t13a_file);
	DBUG_VOID_RETURN;
	
}

public void
test14()
{
	int	fd;
	FILE	*fp, *fp1;
	int	size;
	int	err;
	int	flags;
	char	buf[T_NBYTES];
	struct	stat	statb;
	time_t	t0;
	int	pathok = 0;

	DBUG_ENTER("test14");

	testfail = 0;

	if (( fp1 = fopen(t14_file, "a+"))  == (FILE*) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)", t14_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t14a_file, MODEANY)) == SYSERROR) 
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
	                t14a_file, (long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((write(fd, buf, T_NBYTES)) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() did not write whole buffer");
		(void) close(fd);
		(void) fclose(fp1);
		(void) unlink(t14a_file);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if ((fp = freopen(t14a_file, "ab", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab\") failed - errno %d (%s)",
			t14a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t14a_file);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d (%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t14a_file);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ( (flags & O_ACCMODE) != O_WRONLY)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab\") did not open file for write-only",
				t14a_file);
		(void) fclose(fp1);
		(void) unlink(t14a_file);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (!(flags & O_APPEND ))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab\") did not open file for append",
			t14a_file);
		(void) fclose(fp1);
		(void) unlink(t14a_file);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat(t14a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t14a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t14a_file);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((size = statb.st_size) != (long) T_NBYTES)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab\") changed size of file - expected %d, found %ld",
			t14a_file, T_NBYTES, size);
		(void) fclose(fp1);
		(void) unlink(t14a_file);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* don't fclose(fp) */

	if (unlink(t14a_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unlink() on test file failed - errno %d (%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t14_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	savtimes(".");

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T14_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = freopen(t14a_file, "ab", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab\") failed to open file that did not exist - errno %d(%s)",
			t14a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t14_file);
		(void) unlink(t14a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t14_file);
		(void) unlink(t14a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ( (flags & O_ACCMODE) != O_WRONLY)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab\") when file did not exist, did not open file for write-only",
			t14a_file);
		(void) fclose(fp1);
		(void) unlink(t14_file);
		(void) unlink(t14a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (!(flags & O_APPEND))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab\") when file did not exist, did not open file for append",
			t14a_file);
		(void) fclose(fp1);
		(void) unlink(t14_file);
		(void) unlink(t14a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat(t14a_file, &statb) == SYSERROR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab\") when file did not exist, did not creat file",
			t14a_file);
		(void) fclose(fp1);
		(void) unlink(t14_file);
		(void) unlink(t14a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (statb.st_size != 0L)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab\") when file did not exist, did not creat file with zero length",
			t14a_file);
		(void) fclose(fp1);
		(void) unlink(t14_file);
		(void) unlink(t14a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (msgd = chktimes(".",NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab\") (new file) - parent directory: %s",t14a_file,msgd);
		(void) fclose(fp1);
		(void) unlink(t14_file);
		(void) unlink(t14a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	chktms(t14a_file, t0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(19);

	(void) fclose(fp1);
	(void) unlink(t14_file);
	(void) unlink(t14a_file);
	DBUG_VOID_RETURN;

}

public void
test15()
{
	FILE	*fp;
	FILE	*fp1;
	int	fd;
	int	err;
	int	flags;
	char	buf[T_NBYTES];
	struct	stat	statb;
	int	pathok = 0;

	DBUG_ENTER("test15");
	if ((fp1 = fopen(t15_file, "a+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)",                 t15_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t15a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",                t15a_file, (long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t15_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() on file descriptor of stream failed to write whole buffer");
		(void) fclose(fp1);
		(void) close(fd);
		(void) unlink(t15_file);
		(void) unlink(t15a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);


	if ((fp = freopen(t15a_file, "rb+", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"rb+\") failed - errno %d (%s)", t15a_file, err,                errname(err));
		(void) fclose(fp1);
		(void) unlink(t15_file);
		(void) unlink(t15a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl () on file descriptor of stream failed - errno %d (%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t15_file);
		(void) unlink(t15a_file);
		DBUG_VOID_RETURN;
	}	
	else
		PATH_TRACE;
	if (stat(t15a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t15a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t15_file);
		(void) unlink(t15a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (((flags & O_ACCMODE) != O_RDWR) || statb.st_size != T_NBYTES)
	{
		xx_rpt(FAILURE);
		if ((flags & O_ACCMODE) != O_RDWR)
			in_rpt("freopen(%s, \"rb+\") did not open file for reading and writing", t15a_file);
		if (statb.st_size != T_NBYTES)
			in_rpt("freopen(%s, \"rb+\") changed size of file, expected size %d, found %ld",
				t15a_file,T_NBYTES, statb.st_size);
		(void) fclose(fp1);
		(void) unlink(t15_file);
		(void) unlink(t15a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fclose(fp1);
	(void) unlink(t15a_file);
	(void) unlink(t15_file);

	if ((fp1 = fopen(t15_file, "a+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)",
			t15_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t15a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t15a_file, (long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t15_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() on file descriptor of stream failed to write whole buffer");
		(void) fclose(fp1);
		(void) close(fd);
		(void) unlink(t15_file);
		(void) unlink(t15a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);


	if ((fp = freopen(t15a_file, "r+b", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"r+b\") failed - errno %d (%s)", t15a_file, err,                errname(err));
		(void) fclose(fp1);
		(void) unlink(t15_file);
		(void) unlink(t15a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl () on file descriptor of stream failed - errno %d (%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t15_file);
		(void) unlink(t15a_file);
		DBUG_VOID_RETURN;
	}	
	else
		PATH_TRACE;
	if (stat(t15a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t15a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t15_file);
		(void) unlink(t15a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (((flags & O_ACCMODE) != O_RDWR) || statb.st_size != T_NBYTES)
	{
		xx_rpt(FAILURE);
		if ((flags & O_ACCMODE) != O_RDWR)
			in_rpt("freopen(%s, \"r+b\") did not open file for reading and writing", t15a_file);
		if (statb.st_size != T_NBYTES)
			in_rpt("freopen(%s, \"r+b\") changed size of file, expected size %d, found %ld",
				t15a_file,T_NBYTES, statb.st_size);
		DBUG_VOID_RETURN;
	}
	else
		PATH_XS_RPT(13);

	(void) fclose(fp1);
	(void) unlink(t15a_file);
	(void) unlink(t15_file);

	DBUG_VOID_RETURN;
}

public void
test16()
{
	int	fd;
	FILE	*fp, *fp1;
	int	err;
	int	flags;
	char	buf[T_NBYTES];
	struct	stat	statb;
	time_t	t0;
	int	pathok = 0;

	DBUG_ENTER("test16");

	testfail = 0;

	if (( fp1 = fopen(t16_file, "a+"))  == (FILE*) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)", t16_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t16a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t16a_file, (long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((write(fd, buf, T_NBYTES)) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() did not write whole buffer");
		(void) close(fd);
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	savtimes(t16a_file);

	if ((fp = freopen(t16a_file, "wb+", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb+\") failed - errno %d (%s)", t16a_file, err,                errname(err));
		(void) fclose(fp1);
		(void) unlink(t16_file);
		(void) unlink(t16a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d (%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_RDWR) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb+\") did not open file for reading and writing ", 
			t16a_file);
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat(t16a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t16_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (statb.st_size != 0L)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb+\") did not truncate file",
			t16a_file);
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* don't fclose(fp) */

	if ( (msg = chktimes(t16a_file,NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb+\") (file truncation) - %s",t16a_file,msg);
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (unlink(t16a_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unlink() on test file failed - errno %d (%s)",
			err, errname(err));
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	savtimes(".");

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T16_1_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = freopen(t16a_file, "wb+", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb+\") failed to open file that did not exist - errno %d(%s)",
			t16a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t16_file);
		(void) unlink(t16a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb+\") when file did not exist, did not open file for reading and writing",
			t16a_file);
		(void) fclose(fp1);
		(void) unlink(t16_file);
		(void) unlink(t16a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (stat(t16a_file, &statb) == SYSERROR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen (%s, \"wb+\") did not create file",t16a_file);
		(void) fclose(fp1);
		(void) unlink(t16_file);
		(void) unlink(t16a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (statb.st_size != 0L)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb+\") when file did not exist, did not creat file with zero length",
			t16a_file);
		(void) fclose(fp1);
		(void) unlink(t16_file);
		(void) unlink(t16a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (msgd = chktimes(".",NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"wb+\") (new file) - parent directory: %s",t16a_file,msgd);
		(void) fclose(fp1);
		(void) unlink(t16_file);
		(void) unlink(t16a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	chktms(t16a_file, t0);
	PATH_FUNC_TRACE;

	(void) fclose(fp1);
	(void) unlink(t16_file);
	(void) unlink(t16a_file);

	if (( fp1 = fopen(t16_file, "a+"))  == (FILE*) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)", t16_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t16a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t16a_file, (long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((write(fd, buf, T_NBYTES)) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() did not write whole buffer");
		(void) close(fd);
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	savtimes(t16a_file);

	if ((fp = freopen(t16a_file, "w+b", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w+b\") failed - errno %d (%s)", t16a_file, err,                errname(err));
		(void) fclose(fp1);
		(void) unlink(t16_file);
		(void) unlink(t16a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d (%s)", err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_RDWR) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w+b\") did not open file for reading and writing ", 
			t16a_file);
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat(t16a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t16_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (statb.st_size != 0L)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w+b\") did not truncate file",
			t16a_file);
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* don't fclose(fp) */

	if ( (msg = chktimes(t16a_file,NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w+b\") (file truncation) - %s",t16a_file,msg);
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (unlink(t16a_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unlink() on test file failed - errno %d (%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	savtimes(".");

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T16_2_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = freopen(t16a_file, "w+b", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w+b\") failed to open file that did not exist - errno %d(%s)",
			t16a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t16a_file);
		(void) unlink(t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t16_file);
		(void) unlink(t16a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w+b\") when file did not exist, did not open file for reading and writing",
			t16a_file);
		(void) fclose(fp1);
		(void) unlink(t16_file);
		(void) unlink(t16a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (stat(t16a_file, &statb) == SYSERROR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen (%s, \"w+b\") did not create file",t16a_file);
		(void) fclose(fp1);
		(void) unlink(t16_file);
		(void) unlink(t16a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (statb.st_size != 0L)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w+b\") when file did not exist, did not creat file with zero length",
			t16a_file);
		(void) fclose(fp1);
		(void) unlink(t16_file);
		(void) unlink(t16a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (msgd = chktimes(".",NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"w+b\") (new file) - parent directory: %s",t16a_file,msgd);
		(void) fclose(fp1);
		(void) unlink(t16_file);
		(void) unlink(t16a_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	chktms(t16a_file, t0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(36);

	(void) fclose(fp1);
	(void) unlink(t16_file);
	(void) unlink(t16a_file);

	DBUG_VOID_RETURN;
}


public void
test17()
{
	int		fd;
	FILE		*fp, *fp1;
	int		err;
	int		flags;
	long		size;
	char		buf[T_NBYTES];
	struct stat	statb;
	time_t		t0;
	int		pathok = 0;

	DBUG_ENTER("test17");

	testfail = 0;

	if (( fp1 = fopen(t17_file, "a+"))  == (FILE*) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)", t17_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t17a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t17a_file, (long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) fclose(fp1);
		(void) close(fd);
		(void) unlink(t17_file);
		(void) unlink(t17a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	if ((fp = freopen(t17a_file, "ab+", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab+\") failed to open file - errno %d(%s)",
			t17a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17_file);
		(void) unlink(t17a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab+\") did not open file for reading and writing ",
			t17a_file);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (!(flags & O_APPEND))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab+\") did not open file for append",
			t17a_file);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat(t17a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t17a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((size = statb.st_size) != (long) T_NBYTES)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab+\") changed size of file - expected %d, found %ld",
			t17a_file, T_NBYTES, size);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* don't fclose(fp) */

	if (unlink(t17a_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unlink() on test file failed - errno %d (%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	savtimes(".");

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T17_1_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = freopen(t17a_file, "ab+", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab+\") failed to open file that did not exist - errno %d(%s)",
			t17a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17_file);
		(void) unlink(t17a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab+\") when file did not exist, did not open file for reading and writing ",
			t17a_file);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (!(flags & O_APPEND))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab+\") when file did not exist, did not open file for append",
			t17a_file);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (stat(t17a_file, &statb) == SYSERROR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab+\") when file did not exist, did not creat file",
			t17a_file);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (statb.st_size != 0L)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab+\") when file did not exist, did not creat file with zero length",
			t17a_file);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (msgd = chktimes(".",NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"ab+\") (new file) - parent directory: %s",t17a_file,msgd);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	chktms(t17a_file, t0);
	PATH_FUNC_TRACE;

	(void) fclose(fp1);
	(void) unlink(t17_file);
	(void) unlink(t17a_file);

	if (( fp1 = fopen(t17_file, "a+"))  == (FILE*) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)", t17_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (( fd = creat(t17a_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t17a_file, (long)MODEANY, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, buf, T_NBYTES) != T_NBYTES)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() did not write whole buffer");
		(void) close(fd);
		(void) fclose(fp1);
		(void) unlink(t17_file);
		(void) unlink(t17a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	if ((fp = freopen(t17a_file, "a+b", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+b\") failed to open file - errno %d(%s)",
			t17a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17_file);
		(void) unlink(t17a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+b\") did not open file for reading and writing ",
			t17a_file);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (!(flags & O_APPEND))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+b\") did not open file for append",
			t17a_file);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat(t17a_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)",
			t17a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((size = statb.st_size) != (long) T_NBYTES)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+b\") changed size of file - expected %d, found %ld",
			t17a_file, T_NBYTES, size);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* don't fclose(fp) */

	if (unlink(t17a_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unlink() on test file failed - errno %d (%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	savtimes(".");

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T17_2_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fp = freopen(t17a_file, "a+b", fp1)) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+b\") failed to open file that did not exist - errno %d(%s)",
			t17a_file, err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags = fcntl(fileno(fp), F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl() on file descriptor of stream failed - errno %d(%s)",
			err, errname(err));
		(void) fclose(fp1);
		(void) unlink(t17_file);
		(void) unlink(t17a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((flags & O_ACCMODE) != O_RDWR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+b\") when file did not exist, did not open file for reading and writing ",
			t17a_file);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (!(flags & O_APPEND))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+b\") when file did not exist, did not open file for append",
			t17a_file);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (stat(t17a_file, &statb) == SYSERROR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+b\") when file did not exist, did not creat file",
			t17a_file);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (statb.st_size != 0L)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+b\") when file did not exist, did not creat file with zero length",
			t17a_file);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (msgd = chktimes(".",NEWCTIME|NEWMTIME)) != NULL) 
	{
		if (testfail++)
			xx_rpt(FAILURE);
		in_rpt("freopen(%s, \"a+b\") (new file) - parent directory: %s",t17a_file,msgd);
		(void) fclose(fp1);
		(void) unlink(t17a_file);
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	chktms(t17a_file, t0);
	PATH_FUNC_TRACE;

	if(!testfail)
		PATH_XS_RPT(38);

	(void) fclose(fp1);
	(void) unlink(t17_file);
	(void) unlink(t17a_file);
	DBUG_VOID_RETURN;
}
#else

public	void
test12()
{
	DBUG_ENTER("test12");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

public	void
test13()
{
	DBUG_ENTER("test13");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

public	void
test14()
{
	DBUG_ENTER("test14");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

public	void
test15()
{
	DBUG_ENTER("test15");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

public	void
test16()
{
	DBUG_ENTER("test16");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

public	void
test17()
{
	DBUG_ENTER("test17");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif

public void
chktms(file, t0)
char	*file;
time_t	t0;
{
	struct	stat stbuf;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("chktms");

	if (stat(file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat() failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf.st_atime < t0)
	{
		if(testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_atime field on %s was not marked for update", file);
		in_rpt("st_atime was %ld; should not be less than %ld",
						stbuf.st_atime, t0);
	}
	else
		PATH_TRACE;

	if (stbuf.st_ctime < t0)
	{
		if(testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field on %s was not marked for update", file);
		in_rpt("st_ctime was %ld; should not be less than %ld",
						stbuf.st_ctime, t0);
	}
	else
		PATH_TRACE;

	if (stbuf.st_mtime < t0)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field on %s was not marked for update", file);
		in_rpt("st_mtime was %ld; should not be less than %ld",
						stbuf.st_mtime, t0);
	}
	else
		PATH_TRACE;
		
	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}
