/*
*      SCCS:  @(#)  ANSI.os/streamio/fseek/fseek.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fseek - T.fseek Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fseek

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fseek() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		Phyfos Photi, Hoskyns Group plc
		22/08/88
		rewind() and ftell() testsets removed.

		Linda Taylor, Hoskyns Group plc
		06/09/88
		Add test 6, renumber later tests

		Geoff Clare, 7 Nov 1988
		Add tests for errno values

		Geoff Clare, 3 July 1989
		Do "illegal whence" test in child, so it can
		timeout if it infinite-loops.

		Geoff Clare, 29 Aug 1989
		Do not check ferror() after fseek() errors.

		Steve Ratcliffe, 03 Apr 1990
		Add ENOSPC test.

		Peter Lee, Unisoft Ltd.,
		Tue Jul 17 15:43:08 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			changed reason(), dltd()  to tet_reason, and
			changed vg_env to environ
		
		Geoff Clare, 19 May 1992
			Add check to "seek beyond EOF" test that file
			is not extended.  Add check to ESPIPE test that
			file pointer is not changed.
		
		Geoff Clare, 16 June 1992
			Turn off O_NONBLOCK on fd_rd in ESPIPE test.
		
		Geoff Clare, 22 June 1992
			Add timeouts on post-seek getc()'s in ESPIPE test.

		John-Paul, 4 November 1992
			XNFS enhancements.
		
		Geoff Clare, 10 March 1993
			Implement interpretation VWG/032/091192.

		Geoff Clare, 16 Feb 1995
			Remove contiguous-alphabet dependencies.

		Geoff Clare, 13 April 1995
			Implement interpretation VWG/030/170992.

		Geoff Clare, 3 Jan 1996
			Use VSX_INVALID_WHENCE parameter.
		
************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<unistd.h>
#include	<report.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<chktimes.h>
#include	<signal.h>
#include	<timeout.h>
#include	"./fseek.h"
#include	<tet_api.h>
#include	<xnfs.h>
#include	"fseek_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fseek
#endif

#ifndef _XOPEN_SOURCE
#undef popen
extern	FILE	*popen();
#endif

#define	NO_TESTS	17		/* number of tests in testset */
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	T2_NUM		10
#define	T3_NUM		10
#define	T4_NUM		-5
#define	T5_NUM		5
#define	T6_NUM		10
#if !TEST_XNFS
#define	T12_NUM		10
#define	T13_NUM		10
#endif /* TEST_XNFS */
#define	T14_NUM		-10

extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;
extern	char	*errname();
extern	char	*execpath();
extern	char	*itoa();

int	no_tests = NO_TESTS;

private char	*t8_string =	"abcdefghijklmnop";
private char	*t8_numstring =	"1234567890";
private	char	*t_file =	"./fseek-t";
private	char	*t5_file =	"./fseek-t.5";
private	char	*t8_file =	"./fseek-t.8";
private	char	*t9_file =	"./fseek-t.9";
#if !TEST_XNFS
private	char	*t12_file =	"./fseek-t.12";
private	char	*t12_popen =	"../fseek_t12";
#endif /* TEST_XNFS */
private	char	*t15_dir =	"./fseek-d.15";
#if TEST_XNFS
private	char	*t16_file =	"./fseek-t.16";
private	char	*t17_file =	"./fseek-t.17";
#endif /* TEST_XNFS */
private char	*prog15 = "fseek_t15";
#if TEST_XNFS
private char	*prog_su = "fseek_su";
#endif /* TEST_XNFS */
private int	setupok;

private void	prepare_tests();
private	void	cln_tests();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	test6();
private	void	test7();
private	void	test8();
private void	test9();
private	void	test10_11();
private	void	test12();
private	void	test13();
private	void	test14();
private	void	test15();
private	void	test16();
private	void	test17();
private	int	t10_11func();
#if !TEST_XNFS
private	void	ch_t13();
#endif /* TEST_XNFS */
private	void	ch_t15();
#if TEST_XNFS
private	void	ch_t16();
private	int	t17_func();
private	int	t17_chk_func();
#endif /* TEST_XNFS */

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10_11, 10,
	 test10_11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	NULL, 0
};

private void
prepare_tests()
{
	int		fd;
	int		i;
	static char	msg[1024];
	int		pathok = 0;

	DBUG_ENTER("prepare_tests");

	if ((fd = creat(t_file, MODEANY)) == SYSERROR)
		(void) sprintf(msg, "creat(%s, %#o) failed in preperations - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
	else if (write(fd, t_string, (unsigned) strlen(t_string)) != strlen(t_string))
	{
		(void) sprintf(msg, "write() failed in setup - errno %d(%s)",
			errno, errname(errno));
		fd = SYSERROR;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}

	if (fd == SYSERROR)
		for (i = 1 ; i <= NO_TESTS ; i++)
			tet_delete(i, msg);
	else
		PATH_TRACE;
	if (pathok == 2)
		setupok = 0;
	else
		setupok = -1;

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");
	(void) unlink(t_file);
	(void) unlink(t5_file);
	(void) unlink(t8_file);
	(void) unlink(t9_file);
	DBUG_VOID_RETURN;
}


private void
test1()
{
	FILE		*fp;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test1");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
	}
	else if (fseek(fp, 10L, SEEK_SET) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("fseek() on newly opened stream did not succeed");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	if (fp != (FILE *) NULL)
		(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private void
test2()
{
	FILE		*fp;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test2");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) getc(fp);
	(void) getc(fp);
	(void) getc(fp);
	(void) fseek(fp, (long) T2_NUM, SEEK_SET);
	if (getc(fp) != t_string[T2_NUM])
	{
		xx_rpt(FAILURE);
		in_rpt("fseek(stream, offset, SEEK_SET) did not set position correctly");
	}
	else
		PATH_XS_RPT(1);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private void
test3()
{
	FILE		*fp;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test3");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) getc(fp);
	(void) getc(fp);
	(void) getc(fp);
	(void) fseek(fp, (long) T3_NUM, SEEK_CUR);
	if (getc(fp) != t_string[T3_NUM + 3])
	{
		xx_rpt(FAILURE);
		in_rpt("fseek(stream, offset, SEEK_CUR) did not set position correctly");
	}
	else
		PATH_XS_RPT(1);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private void
test4()
{
	FILE		*fp;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test4");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) getc(fp);
	(void) getc(fp);
	(void) getc(fp);
	(void) fseek(fp, (long) T4_NUM, SEEK_END);
	if (getc(fp) != (char) ('z' + T4_NUM + 1))
	{
		xx_rpt(FAILURE);
		in_rpt("fseek(stream, offset, SEEK_END) did not set position correctly");
	}
	else
		PATH_XS_RPT(1);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private void
test5()
{
	FILE		*fp;
	int		err;
	struct stat	statbuf;
	int	pathok = setupok;

	DBUG_ENTER("test5");

	if ((fp = fopen(t5_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t5_file, err, errname(err));
	}
	else if (fseek(fp, (long) T5_NUM, SEEK_END) != 0)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fseek() beyond end of file failed - errno %d(%s)",
			err, errname(err));
	}
	else if (stat(t5_file, &statbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed with errno = %d (%s)", err, errname(err));
	}
	else if (statbuf.st_size != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("fseek() beyond end of file changed the size of the file");
		in_rpt("expected file size of 0 - got %ld", (long)statbuf.st_size);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private void
test6()
{
	FILE		*fp;
	int		err;
	char		buf[50];
	int		pathok = setupok;

	DBUG_ENTER("test6");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fread((void *)buf, (size_t)1,
		(size_t)(strlen(t_string) + T6_NUM), fp);
	if (!feof(fp))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: fread() greater than file size did not set EOF indicator");
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fseek(fp, (long) T6_NUM, SEEK_SET) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("fseek() after EOF condition failed");
	}
	else if (feof(fp)) 
	{
		xx_rpt(FAILURE);
		in_rpt("fseek() did not clear EOF indicator");
	}
	else
		PATH_XS_RPT(2);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test7()
{
	FILE		*fp;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test7");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) getc(fp);
	(void) getc(fp);
	(void) getc(fp);
	(void) ungetc('x', fp);
	(void) fseek(fp, 0L, SEEK_SET);
	if (getc(fp) == 'x')
	{
		xx_rpt(FAILURE);
		in_rpt("fseek() did not undo the effects of ungetc()");
	}
	else
		PATH_XS_RPT(1);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test8()
{
	int	fd;
	FILE	*fp;
	off_t	offval;
	int	pathok = setupok;

	DBUG_ENTER("test8");

	if ((fd = open(t8_file, O_RDWR|O_CREAT, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to open file %s", t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fp = fdopen(fd, "w")) == (FILE *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fdopen file %s", t8_file);
		(void) close(fd);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, t8_string, (unsigned) strlen(t8_string)) != (int) strlen(t8_string))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to write %d bytes to %s",
			(int)strlen(t8_string), t8_file);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (lseek(fd, (off_t) 0, SEEK_SET) == (off_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek failed on file %s", t8_file);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fwrite((void *) t8_numstring, strlen(t8_numstring), (size_t)1, fp) != 1)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fwrite %d bytes to %s",
			(int)strlen(t8_numstring), t8_file);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fflush(fp) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fflush file %s", t8_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fseek(fp, 8L, SEEK_SET) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fseek on file %s", t8_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((offval = lseek(fd, (off_t) 0, SEEK_CUR)) != (off_t) 8)
	{
		xx_rpt(FAILURE);
		in_rpt("File pointer not correctly set after fseek()");
		in_rpt("Expected: %ld", (long) 10);
		in_rpt("Observed: %ld", (long) offval);
	}
	else
		PATH_XS_RPT(7);

	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t8_file);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	FILE	*fp;
	char	*res;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test9");

	if ((fp = fopen(t9_file, "w")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t9_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
        if (putc('A', fp) == EOF)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("putc('A', fp) failed on file %s, errno %d(%s)",
			t9_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(t9_file);
	PATH_FUNC_TRACE;

	if (fseek(fp, 0L, SEEK_SET) != 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fseek(fp, 0L, SEEK_SET) failed on file %s, errno %d(%s)",
			t9_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ((res = chktimes(t9_file, NEWMTIME | NEWCTIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(res);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(5);
	}

	(void) fclose(fp);
	(void) unlink(t9_file);
	DBUG_VOID_RETURN;
}

/* Lowest test number for which following function is used */
#define TBASE	10

private	void
test10_11()
{
	int experr, err, ret, ferr_ret;
	static int errnums[] = { EBADF, EFBIG, };
			      /* test10 test11  */
	int	pathok = setupok;

	DBUG_ENTER("test10_11");

#if TEST_XNFS
	if(tet_thistest == 10)
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}
#endif /* TEST_XNFS */

	experr = errnums[tet_thistest - TBASE];

	ret = do_ferr(experr, 'f', -1, t10_11func, &ferr_ret, &err);
	err = errno;

	if (tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (err != experr || ret == 0)
	{
		xx_rpt(FAILURE);
		if (ret == 0)
			in_rpt("fseek() returned %d when testing for errno %d(%s), expected non-zero",
				ret, experr, errname(experr));
		else if (err != experr)
			in_rpt("fseek() set errno incorrectly: expected %d(%s), actual %d(%s)",
				experr, errname(experr), err, errname(err));
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/* ARGSUSED */

private int
t10_11func(fp, nbyte)
FILE *fp;
int nbyte;
{
	DBUG_ENTER("t10_11func");

	globok = 1; /* must be set for do_ferr() */

	DBUG_RETURN(fseek(fp, 0L, SEEK_SET));
}

#if TEST_XNFS

private void
test12()
{
	DBUG_ENTER("test12");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test12()
{
	FILE	*fp;
	int	err, err1, err2;
	int	fail = 0;
	int	fd_rd, fd_wr;
	int	pathok = setupok;

	DBUG_ENTER("test12");
	errno = NOERROR;
	if (mkfifo(t12_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(%s, %#lo) failed - errno %d(%s)",
			t12_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	SET_TIMEOUT(WAITTIME)
	fd_rd = open(t12_file, O_RDONLY|O_NONBLOCK, 0);
	CLEAR_ALARM
	if (fd_rd == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("timed out opening fifo (with O_NONBLOCK!)");
		else
			in_rpt("deletion reason: open(<fifo>, O_RDONLY|O_NONBLOCK) failed - errno %d(%s)",
				err, errname(err));
		(void) unlink(t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fp = fdopen(fd_rd, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fdopen(fd, \"r\") failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	SET_TIMEOUT(WAITTIME)
	fd_wr = open(t12_file, O_WRONLY|O_NONBLOCK, 0);
	CLEAR_ALARM
	if (fd_wr == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("timed out opening fifo (with O_NONBLOCK!)");
		else
			in_rpt("deletion reason: open(<fifo>, O_WRONLY|O_NONBLOCK) failed - errno %d(%s)",
				err, errname(err));
		(void) unlink(t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (fseek(fp, (long) T12_NUM, SEEK_SET) == 0)
		fail |= 1;
	else if (errno != ESPIPE)
	{
		err1 = errno;
		fail |= 2;
	}
	else
		PATH_TRACE;
	(void) unlink(t12_file);

	if ((fp = popen(t12_popen, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: popen(%s, \"r\") failed - errno %d(%s)",
			t12_popen, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (fseek(fp, (long) T12_NUM, SEEK_SET) == 0)
		fail |= 8;
	else if (errno != ESPIPE)
	{
		err2 = errno;
		fail |= 16;
	}
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);

		if (fail & 1)
			in_rpt("fseek() on stream associated with a FIFO succeeded");
		if (fail & 2)
			in_rpt("fseek() on stream associated with a FIFO gave errno = %d(%s) instead of %d(ESPIPE)", err1, errname(err1), ESPIPE);
		if (fail & 8)
			in_rpt("fseek() on stream returned by popen() succeeded");
		if (fail & 16)
			in_rpt("fseek() on stream returned by popen() gave errno = %d(%s) instead of %d(ESPIPE)", err2, errname(err2), ESPIPE);
	}
	else
		PATH_XS_RPT(7);

	(void) sleep(2); /* avoids Broken Pipe message from shell */
	(void) pclose(fp);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test13()
{
	DBUG_ENTER("test13");

	(void) cppair(ch_t13, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t13()
{
	/* This test is liable to cause a SIGSYS on some systems */
	/* If so, it should be ignored via the VSX_SIG_IGN parameter */

	FILE	*fp;
	char	*cp;
	int	err;
	int	badwhence = -1;
	int	pathok = setupok;

	DBUG_ENTER("ch_t13");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	cp = tet_getvar("VSX_INVALID_WHENCE");
	if (cp != NULL && *cp != '\0')
		badwhence = atoi(cp);

	errno = NOERROR;
	if (fseek(fp, (long) T13_NUM, badwhence) == 0)
	{
		xx_rpt(FAILURE);
		in_rpt("fseek() did not fail when given an illegal whence paramater (%d)", badwhence);
	}
	else if (errno != EINVAL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fseek() gave errno = %d(%s) instead of %d(EINVAL) when given an illegal whence paramater (%d)",
			err, errname(err), EINVAL, badwhence);
	}
	else
		PATH_XS_RPT(1);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test14()
{
	FILE		*fp;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test14");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) getc(fp);
	(void) getc(fp);
	(void) getc(fp);
	errno = NOERROR;
	if (fseek(fp, (long) T14_NUM, SEEK_CUR) == 0)
	{
		xx_rpt(FAILURE);
		in_rpt("fseek() succeeded in attempt to seek to before start of file");
	}
	else if (errno != EINVAL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fseek() gave errno = %d(%s) instead of %d(EINVAL) on attempt to seek to before start of file",
			err, errname(err), EINVAL);
	}
	else
		PATH_XS_RPT(1);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test15()
{

	DBUG_ENTER("test15");

	(void) cppair(ch_t15, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t15()
{
	int 	err;
	char	*args[6];

	DBUG_ENTER("ch_t15");

	args[0] = prog15;
	args[1] = t15_dir;
	args[2] = "fseek";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog15), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

private void
test16()
{
	DBUG_ENTER("test16");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test16()
{
	DBUG_ENTER("test16");

	(void) tet_fork(ch_t16, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t16()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t16");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t16_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test17()
{
	DBUG_ENTER("test17");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test17()
{
	DBUG_ENTER("test17");

	t_estale_flush_c(t17_file, 0, t17_func, t17_chk_func, SP_T17_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t17_func(fp)
FILE	*fp;
{
	size_t	nbytes;

	DBUG_ENTER("t17_func");

	nbytes = strlen(t_string);

	DBUG_RETURN(fwrite((void *)t_string, (size_t)1, nbytes, fp)
			!= nbytes ? -1 : 0);
}

private int
t17_chk_func(fp, ebytes)
FILE	*fp;
int	ebytes;
{
	int	rval;
	int	i;
	int	err;
	size_t	nbytes = strlen(t_string);
	int	pathok = 0;

	DBUG_ENTER("t17_chk_func");
	
	for(i = 0; i < ebytes; i++ ) {
		(void) fseek(fp, (long)0, SEEK_END);
		(void) fwrite((void *)t_string, (size_t)1, nbytes, fp);
		errno = NOERROR;
		rval = fseek(fp, (long)0, SEEK_SET);
		err = errno;
		if(rval != 0)
			break;
	}

	if(rval == 0 || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(rval == 0)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected non-zero return value");
		}
		else
			PATH_TRACE;

		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(2);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
