/*
*      SCCS:  @(#)  ANSI.os/streamio/fseek/fseek_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fseek - fseek_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		fseek_su - used by T.fseek in privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.fseek to perform privileged tests.

	  usage:
		fseek_su testnum dir file ident dbug

	  where testnum is the number of the test to be executed, dir
	  and file are parameters to be passed to the test.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Neil Moses, UniSoft Ltd.
DATE CREATED:	9 June 1992
MODIFICATIONS:
		John-Paul, 2 November 1992
			XNFS enhancements.

************************************************************************/
#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <dbug.h>
#include <errno.h>
#include <sys/stat.h>
#include <report.h>
#include <tet_api.h>
#include <xnfs.h>
#include "./fseek.h"
#include "fseek_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef fseek
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	**environ;
extern	void	savtimes();
extern	char	*chktimes();
extern	char	*errname();

extern	void	valrpt();
extern	void	set_pname();

#if TEST_XNFS
private	void	t16_func();
private	int	t16_chk_func();
#endif /* TEST_XNFS */

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int testnum;
	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testnum = atoi(argv[1]);
	switch(testnum)
	{
#if TEST_XNFS
        case 16:
                t_eacces_flush_c(argv[3], 0, t16_chk_func, SP_T16_EACCES);
                break;
#endif /* TEST_XNFS */

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed : %d", testnum);
		return(0);
	}
	
	return(0);
}

#if TEST_XNFS

private int
t16_chk_func(fp, ebytes)
FILE	*fp;
int	ebytes;
{
	int	err;
	int	i;
	int 	rval;
	size_t	nbytes = strlen(t_string);
	int	pathok = 0;

	DBUG_ENTER("t16_chk_func");

	for(i = 0; i < ebytes; i++ ) {
		(void) fseek(fp, (long)0, SEEK_END);
		(void) fwrite((void *)t_string, (size_t)1, nbytes, fp);
		errno = NOERROR;
		rval = fseek(fp, (long)0, SEEK_SET);
		err = errno;
		if(rval != 0)
			break;
	}

	if(rval == 0 || err != EACCES)
	{
		xx_rpt(FAILURE);

		if(rval == 0)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected non zero return value");
		}
		else
			PATH_TRACE;

		if(err != EACCES)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",EACCES,errname(EACCES));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(2);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
