/*
*      SCCS:  @(#)  ANSI.os/streamio/ftell/ftell.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/ftell - T.ftell Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.ftell

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the ftell() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		Phyfos Photi, Hoskyns Group plc
		22/08/88
		ftell() testset separated from fseek() testset

		Geoff Clare, 16 Nov 1988
		Added tests for errno values

		Geoff Clare, 29 Aug 1989
		Do not check ferror() after ftell() errors.

		Geoff Clare, 21 Sep 1989
		Delete EFBIG test.

		Steve Ratcliffe, 03 Apr 1990
		Add ENOSPC test.

		Peter Lee, Unisoft Ltd.,
		Tue Jul 17 12:28:17 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			changed dltd and reason to tet_reason.

		John-Paul, 2 November 1992
			XNFS enhancements.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<unistd.h>
#include	<signal.h>
#include	<report.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<timeout.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ftell
#endif

#ifndef _XOPEN_SOURCE
#undef popen
extern	FILE	*popen();
#endif

#define	NO_TESTS	4		/* number of tests in testset */
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	T1_NUM		10
#define	WAITTIME	(5 * SPEEDFACTOR)


extern	char	*errname();

int	no_tests = NO_TESTS;

private char	*t_string =	"abcdefghijklmnopqrstuvwxyz";
private	char	*t1_file =	"./ftell-t.1";
private	char	*t2_file =	"./ftell-t.2";
#if !TEST_XNFS
private	char	*t4_file =	"./ftell-t.4";
private	char	*t4_popen =	"../ftell_t4";
#endif /* TEST_XNFS */

private	void	cln_tests();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
#if !TEST_XNFS
private	int	t3func();
#endif /* TEST_XNFS */

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};


private void
cln_tests()
{
	DBUG_ENTER("cln_tests");
	(void) unlink(t1_file);
	(void) unlink(t2_file);
#if !TEST_XNFS
	(void) unlink(t4_file);
#endif /* TEST_XNFS */
	DBUG_VOID_RETURN;
}

private void
test1()
{
	FILE		*fp;
	int		fd;
	int		err;
	long		pos;
	int	pathok = 0;

	DBUG_ENTER("test1");

	if ((fd = creat(t1_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed in preperations - errno %d(%s)",
			t1_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (outstr(fd, t_string) != strlen(t_string))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() failed in setup - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	errno = NOERROR;
	if ((fp = fopen(t1_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t1_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (err = 0 ; err < T1_NUM ; err++)
		(void) getc(fp);
	if ((pos = ftell(fp)) != (long) T1_NUM)
	{
		xx_rpt(FAILURE);
		in_rpt("ftell() did not return correct offset in file:");
		in_rpt("expected offset of %d, ftell() returned %ld",
			T1_NUM, pos);
	}
	else
		PATH_XS_RPT(3);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test2()
{
	FILE	*fp;
	long	offval;
	long	savoff;
	int	pathok = 0;

	DBUG_ENTER("test2");

	if ((fp = fopen(t2_file, "w")) == (FILE *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fopen file %s", t2_file);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fwrite((void *) t_string, strlen(t_string), (size_t)1, fp) != 1)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fwrite %d bytes to %s", strlen(t_string), t2_file);
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((savoff = ftell(fp)) == (long) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: ftell failed on file %s", t2_file);
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fflush(fp) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fflush file %s", t2_file);
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((offval = ftell(fp)) != savoff)
	{
		xx_rpt(FAILURE);
		in_rpt("File pointer not correctly set after ftell()");
		in_rpt("Expected: %ld", (long) savoff);
		in_rpt("Observed: %ld", (long) offval);
	}
	else
		PATH_XS_RPT(4);

	(void) fclose(fp);
	(void) unlink(t2_file);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
test3()
{
	DBUG_ENTER("test3");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test3()
{
	int experr, err, ret, ferr_ret;
	int	pathok = 0;

	DBUG_ENTER("test3");

	experr = EBADF;

	ret = do_ferr(experr, 'r', -1, t3func, &ferr_ret, &err);

	err = errno;

	if (tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (err != experr || ret != -1)
	{
		xx_rpt(FAILURE);
		/* don't check ferr_ret */
		if (ret != -1)
			in_rpt("ftell() returned %d when testing for errno %d(%s), expected -1",
				ret, experr, errname(experr));
		else if (err != experr)
			in_rpt("ftell() set errno incorrectly: expected %d(%s), actual %d(%s)",
				experr, errname(experr), err, errname(err));
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/* ARGSUSED */

private int
t3func(fp, nbyte)
FILE *fp;
int nbyte;
{

	DBUG_ENTER("t3func");

	globok = 1; /* must be set for do_ferr() */

	DBUG_RETURN((int)ftell(fp));
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test4()
{
	DBUG_ENTER("test4");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test4()
{
	FILE		*fp;
	int		err, err1, err2;
	int		fail = 0;
	int		fd;
	int	pathok = 0;

	DBUG_ENTER("test4");
	errno = NOERROR;
	if (mkfifo(t4_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(%s, %#o) failed - errno %d(%s)",
			t4_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)
	fd = open(t4_file, O_RDONLY|O_NONBLOCK, 0);
	CLEAR_ALARM
	if (fd == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("timed out opening fifo (with O_NONBLOCK!)");
		else
			in_rpt("deletion reason: open(<fifo>, O_RDONLY|O_NONBLOCK) failed - errno %d(%s)",
				err, errname(err));
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fp = fdopen(fd, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fdopen(fd, \"r\") failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	errno = NOERROR;

	if (ftell(fp) != -1L)
		fail |= 1;
	else if (errno != ESPIPE)
	{
		err1 = errno;
		fail |= 2;
	}
	else
		PATH_TRACE;

	(void) unlink(t4_file);
	if ((fd = creat(t4_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t4_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);
	if ((fp = popen(t4_popen, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: popen(%s, \"w\") failed - errno %d(%s)",
			t4_popen, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (ftell(fp) != -1L)
		fail |= 4;
	else if (errno != ESPIPE)
	{
		err2 = errno;
		fail |= 8;
	}
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("ftell() on stream associated with a FIFO succeeded");
		if (fail & 2)
			in_rpt("ftell() on stream associated with a FIFO gave errno = %d(%s) instead of %d(ESPIPE)", err1, errname(err1), ESPIPE);
		if (fail & 4)
			in_rpt("ftell() on stream returned by popen() succeeded");
		if (fail & 8)
			in_rpt("ftell() on stream returned by popen() gave errno = %d(%s) instead of %d(ESPIPE)", err2, errname(err2), ESPIPE);
	}
	else
		PATH_XS_RPT(7);

	(void) unlink(t4_file);
	(void) pclose(fp);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */
