/*
*      SCCS:  @(#)  ANSI.os/streamio/fwrite/fwrite.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fwrite - T.fwrite Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fwrite

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fwrite() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 4 Nov 1988
		Add tests for errno values
		Delete test with negative size & nitems

		Geoff Clare, 1 June 1989
		Add EIO test.
		Add test with zero size & nitems.

		Geoff Clare, 23 Aug 1989
		EIO test had wrong expected return value.

		Geoff Clare, 20 Sep 1989
		Allow for partial writes in do_ferr() tests.

		Geoff Clare, 17 Nov 1989
		Use setbuf() instead of setvbuf()

		Steve Ratcliffe, 27 Mar 1990
		Add ENOSPC test.

		Geoff Clare, 6 June 1990
		VSX_ULIMIT_BLKS == -1 means unsupported.

		Geoff Clare, 14 June 1990
		Do EAGAIN and EPIPE tests on pipe and FIFO

		Andrew Dingwall Wed Jun 27 13:40:11 BST 1990
		Added a test of stream error indicator after a partial fwrite

		David Sawyer
		Wed Jul 18 12:06:35 BST 1990
		Added resillience code to all tests except 3.

		Stuart Boutell, 5 Nov 1990
			changed dltd and reason to tet_reason,
			eliminated vg_env.

		Stuart Boutell, 17 Apr 1991
			Added "ab" & "ab+" test modes to test 5

		Stuart Boutell, 26 Apr 1991
			Changed stream error after a partial fwrite test
		to use unbuffered streams, and to fill and then drain
		the pipe before testing, and corrected nitems==-1 bug.

		Geoff Clare, 22 May 1991
			Use setbuf() instead of setvbuf().

		Geoff Clare, 1 July 1991
			partial write test gives unresolved for EAGAIN

		Geoff Clare, 10 Aug 1992
			Use unbuffered stream instead of assuming write
			of > BUFSIZ bytes will cause stream to be flushed.

		John-Paul, 2 November 1992
			XNFS enhancements.

		Geoff Clare, 27 Jan 1993
			Skip pipe part of test 3 when testing XNFS.

		Geoff Clare, 15 June 1993
			Cast fwrite() args correctly in t19_func() and
			t19_chk_func().  Add cast to MODEANY definition.

		Geoff Clare, 3 Sept 1993
			Allow fwrite() to return anything from 0 to the
			number of bytes that can be written, when ulimit
			is reached.

		Geoff Clare, 8 August 1994
			In the pipe part of test 3, allow fwrite() to
			return the number of bytes requested.

		Andrew Twigger, 1 May 1994
			Add PATH_TRACE to test 3

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<synch.h>
#include	<signal.h>
#include	<timeout.h>
#include	<chktimes.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"fwrite_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fwrite
#endif



#define	NO_TESTS	19		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	T_NUM		5
#define	T1_NUM		3
#define T3_BUFSIZ	(BUFSIZ + 8)
#define	T4_NUM		3
#define E_UNTESTED	-1
#define E_OK		0
#define	E_ULIMIT	1
#define	E_FOPEN		2
#define	E_FWRITE	3
#define	E_SIG		4
#define	E_TIMEOUT	5
#define E_CREAT		6
#define E_WRITE		7
#define E_WAITSYNC	8
#define E_PATHCHECK	9
#define E_FERROR	10
#define	SYNCSIZE	4		/* number of bytes sent by sendsync */
#define	ULMT_NBLK	4
#define	BLK_SIZE	512
#define FERR_BUFSZ	20

#define MIN(a, b)	((a) < (b) ? (a) : (b))

extern	char	*vg_ident, *vg_dbug;
extern	char	**environ;

extern	char	*itoa();
extern	char	*errname();
extern	char	*execpath();
extern	void	setsigs();
extern	char	*signame();
extern	void	do_feio();
extern	long	setlimit();

extern	int	alrm_flag;

int	no_tests = NO_TESTS;

struct t2_3_str {
	int	flag;
	int	err;
	int	rval;
};

private	char	*t_string =	"test string";
#if TEST_XNFS
private	char	*t_string_xnfs =	XNFS_STRING;
#endif /* TEST_XNFS */
private	char	*t1_file =	"./fwrite-t.1";
private	char	*t2_file =	"./fwrite-t.2";
private	char	*t3_file =	"./fwrite-t.3";
private	char	*t4_file =	"./fwrite-t.4";
private	char	*t5_file =	"./fwrite-t.5";
private	char	*t6_file =	"./fwrite-t.6";
private	char	*t7a_file =	"./fwrite-t.7a";
private	char	*t7b_file =	"./fwrite-t.7b";
private	char	*t8_file =	"./fwrite-t.8";
private	char	*t17_dir =	"./fwrite-d.17";
#if TEST_XNFS
private	char	*t18_file =	T18_FILE;
private	char	*t19_file =	T19_FILE;
#endif /* TEST_XNFS */

private char	*prog17 = "fwrite_t17";
#if TEST_XNFS
private char	*prog18 = "fwrite_t18";
#endif /* TEST_XNFS */

private struct t2_3_str	t2_res;
private	synch	*t2_syn =	(synch *) NULL;
private	int	t2_sent =	0;
private long	t2_ulimblks;

private struct t2_3_str	t3_res;
private	synch	*t3_syn =	(synch *) NULL;
private long	t3_ulimblks;

private	long	numbers[T_NUM] = { 0, -1, 12312312, 12222, 4343423 };

private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	test6();
private	void	test7();
private	void	test8();
private	void	test9_15();
private	void	test16();
private	void	test17();
private	void	test18();
private	void	test19();
private void	pa_t2();
private void	ch_t2();
private void	pa_t3();
private void	ch_t3();
private	int	t9_15func();
#if !TEST_XNFS
private	int	t16io();
private	void	t16rpt();
#endif /* TEST_XNFS */
#if TEST_XNFS
private int	t19_func();
private int	t19_chk_func();
#endif /* TEST_XNFS */

private void	ch_t17();
#if TEST_XNFS
private void	ch_t18();
#endif /* TEST_XNFS */

#if !TEST_XNFS
private	void	t3_fill_pipe();
#endif

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9_15, 9,
	 test9_15, 10,
	 test9_15, 11,
	 test9_15, 12,
	 test9_15, 13,
	 test9_15, 14,
	 test9_15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	NULL, 0
};

private void
test1()
{
	FILE		*fp;
	int		fd;
	int		fail = 0;
	int		i;
	int		err;
	int		rval;
	int		wval;
	long		num[T1_NUM];
	long		a = 1283823, b = 98123;
	static char	*delmsg = "depends on success of test 1";
	int		pathok = 0;

	DBUG_ENTER("test1");

	if ((fd = creat(t1_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d(%s)",
			t1_file, MODEANY, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (write(fd, (char *) &a, sizeof(long)) != sizeof(long)
		|| write(fd, (char *) &b, sizeof(long)) != sizeof(long))
	{
		xx_rpt(DELETION);
		in_rpt("write() failed to write whole buffer");
		(void) close(fd);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);
	if ((fp = fopen(t1_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t1_file, err, errname(err));
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((wval = fwrite((void *) numbers, (size_t) sizeof(long),
		(size_t) T1_NUM, fp)) != T1_NUM)
			fail |= 1;
	else
		PATH_TRACE;

	(void) fclose(fp);
	if ((fd = open(t1_file, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDONLY) failed - errno %d(%s)",
			t1_file, err, errname(err));
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = read(fd, (char *) num, sizeof(num)))
		!= sizeof(long) * T1_NUM)
			fail |= 2;
	else
	{
		PATH_TRACE;
		for (i = 0 ; i < T1_NUM ; i++)
			if (numbers[i] != num[i])
				fail |= 4;
			else
				PATH_TRACE;
	}

	(void) close(fd);
	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
		{
			in_rpt("fwrite() did not return the number of elements written");
			in_rpt("expected %d elements, fwrite() returned %d",
				T1_NUM, wval);
		}
		if (fail & 2)
		{
			in_rpt("fwrite() did not write the correct number of bytes to file");
			in_rpt("expected %d bytest to be written, found %d",
				T1_NUM * sizeof(long), rval);
		}
		if (fail & 4)
			in_rpt("fwrite() did not write correct value to file");
		(void) tet_delete(4, delmsg);
		(void) tet_delete(5, delmsg);
	}
	else
		PATH_XS_RPT(6 + T1_NUM);

	(void) unlink(t1_file);
	DBUG_VOID_RETURN;
}


private	void
t2_catch(sig)
	int	sig;
{
	struct	t2_3_str	res;

	/* if nothing has yet been sent */
	if (t2_sent == 0)
	{
		res.flag = E_SIG;
		res.rval = sig;
		if (sendsync(t2_syn[0][1], (char *) &res, sizeof(res), WAITTIME)
								!= sizeof(res))
			exit(A_SENDSYNC);
	}
	exit(OK);
}


private	void
test2()
{
	char		*ulimaddr;
	int		pathok = 0;

	DBUG_ENTER("test2");
	ulimaddr = tet_getvar("VSX_ULIMIT_BLKS");
	if (ulimaddr == NULL || *ulimaddr == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: VSX_ULIMIT_BLKS parameter not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t2_ulimblks = atol(ulimaddr);
	if (t2_ulimblks == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_ULIMIT_BLKS parameter is set to -1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (t2_ulimblks < (long) ULMT_NBLK)
		t2_ulimblks = (long) ULMT_NBLK;

	if ((t2_syn = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to open sync channel");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) cppair(ch_t2, pa_t2, WAITTIME, 0);
	PATH_FUNC_TRACE;

	if (t2_res.flag == E_ULIMIT)
	{
		xx_rpt(DELETION);
		in_rpt("setlimit(%ld) failed - errno %d(%s)",
			t2_ulimblks, t2_res.err, errname(t2_res.err));
	}
	else if (t2_res.flag == E_TIMEOUT)
	{
		xx_rpt(DELETION);
		in_rpt("timed out waiting for child process to send data");
	}
	else if (t2_res.flag == E_FOPEN)
	{
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t2_file, t2_res.err, errname(t2_res.err));
	}
	else if (t2_res.flag == E_FWRITE)
	{
		xx_rpt(FAILURE);
		in_rpt("fwrite() failed to write out buffer after writing %d bytes", t2_res.rval);
		in_rpt("ulimit is %ld 512 bytes blocks", t2_ulimblks);
	}
	else if (t2_res.flag == E_SIG)
	{
		xx_rpt(FAILURE);
		in_rpt("fwrite() that exceeded file size limit caused signal %d(%s)",
			t2_res.rval, signame(t2_res.rval));
	}
	else if (t2_res.rval < 0 || t2_res.rval > 12)
	{
		xx_rpt(FAILURE);
		in_rpt("fwrite() failed to return expected values when ulimit was exceeded");
		in_rpt("expected between 0 and %d, got %d", 12, t2_res.rval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	(void) unlink(t2_file);
	(void) closesync(t2_syn);
	DBUG_VOID_RETURN;
}


private	void
ch_t2()
{
	FILE		*fp;
	char		buf[BLK_SIZE];
	long	l;
	
	DBUG_ENTER("ch_t2");

	t2_res.flag = 0;
	setsigs(t2_catch);

	if (setlimit(t2_ulimblks) == (long) SYSERROR)
	{
		t2_res.flag = E_ULIMIT;
		t2_res.err = errno;
	}
	else if ((fp = fopen(t2_file, "w")) == (FILE *) NULL)
	{
		t2_res.flag = E_FOPEN;
		t2_res.err = errno;
	}
	else {
		setbuf(fp, (char *) NULL);
		for (l=0; l<(t2_ulimblks*BLK_SIZE)-12; l++)
		{
			if (fwrite((void *)buf, (size_t)1, (size_t)1, fp) != 1)
			{
				t2_res.rval = l;
				t2_res.flag = E_FWRITE;
				break;
			}
		}
	}

	if (t2_res.flag == 0)
	{
		t2_res.rval = fwrite((void *)buf, (size_t)1, (size_t)20, fp);
	}

	if (sendsync(t2_syn[0][1], (char *) &t2_res, sizeof(t2_res),
					WAITTIME) != sizeof(t2_res))
		DBUG_EXIT(A_SENDSYNC);
	t2_sent = 1;
	if (fp != (FILE *) NULL)
		(void) fclose(fp);

	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	int	pathok = 0;
	
	DBUG_ENTER("pa_t2");

	if (waitsync(t2_syn[0][0], (char *) &t2_res, sizeof(t2_res),
					WAITTIME) != sizeof(t2_res))
	{
		if (alrm_flag > 0)
		{
			PATH_TRACE;
			t2_res.flag = E_TIMEOUT;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: failed to receive data from parent");
		}
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);
	
	DBUG_VOID_RETURN;
}


/*
**	we have two goes at verifying this assertion
**		the first one is tried if maximum file size can be set
**			using setlimit()
**		the second one is tried if pipe buffers have a finite size
**			and we are not testing XNFS
*/

private void
test3()
{
	char	*ulimaddr;
	int	passed = 0, failed = 0;
	int	pathok = 0;
#if !TEST_XNFS
	int	i;
	FILE	*fp;
	size_t	nitems;
	int	pipe_fd[2];
	int	flags;
	long	pipe_buf;
	int	err;
	size_t	bufsiz1, bufsiz2;
	char	*bp;
#endif

	DBUG_ENTER("test3");

	ulimaddr = tet_getvar("VSX_ULIMIT_BLKS");

	if (ulimaddr == NULL || *ulimaddr == '\0')
	{
		failed |= 1;
		xx_rpt(DELETION);
		in_rpt("parameter VSX_ULIMIT_BLKS is not set");
	}
	else if ((t3_ulimblks = atol(ulimaddr)) != -1L)
	{
		DBUG_PRINTF("test3", "testing assertion using limited files size");

		if (t3_ulimblks < (long) ULMT_NBLK)
			t3_ulimblks = (long) ULMT_NBLK;

		if ((t3_syn = opensync()) == (synch *) NULL)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: failed to open sync channel");
			DBUG_VOID_RETURN;
		}

		globok = 0;
		(void) cppair(ch_t3, pa_t3, WAITTIME, 0);
		PATH_FUNC_TRACE;

		switch (t3_res.flag)
		{
		case E_OK:
			PATH_TRACE;
			passed |= 1;
			break;
		case E_PATHCHECK:
			DBUG_PRINTF("test3", "path check failure in child");
			passed |= 1;
			break;
		case E_UNTESTED:
			PATH_TRACE;
			break;
		default:
			failed |= 1;
			break;
		}

		(void) closesync(t3_syn);
	}
	else
	{
		/* above code was optional, so ... */
		PATH_TRACE; PATH_TRACE;
	}

#if TEST_XNFS

	PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
	PATH_TRACE; PATH_TRACE; PATH_TRACE;

#else /* TEST_XNFS */

	DBUG_PRINTF("test3", "testing assertion using non-blocking pipe");

	if (pipe(pipe_fd) == SYSERROR)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("pipe() failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(pipe_fd[1], F_GETFL, 0)) == SYSERROR)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("fcntl(F_GETFL) failed, errno = %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	flags |= O_NONBLOCK;
	if (fcntl(pipe_fd[1], F_SETFL, flags) == SYSERROR)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("fcntl(F_SETFL) failed, errno = %d (%s)",
			err, errname(err));
		(void) close(pipe_fd[0]);
		(void) close(pipe_fd[1]);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if ((pipe_buf = fpathconf(pipe_fd[1], _PC_PIPE_BUF)) <= 0L)
	{
		if (errno)
		{
			err = errno;
			if (!failed)
				xx_rpt(DELETION);
			in_rpt("fpathconf(_PC_PIPE_BUF) failed, errno = %d (%s)",	err, errname(err));
			failed |= 2;
		}
		else
		{
			DBUG_PRINTF("test3",
				"can't test - unlimited pipe buffer");
			for (i = 0; i < 5; i++)
				PATH_TRACE;	/* alternate path (1) */
		}
	}
	else
	{
		PATH_TRACE;	/* alternate path (1) */

		bufsiz1 = (size_t) pipe_buf + 16;
		bufsiz2 = bufsiz1 + 8;

		if ((long) bufsiz2 != pipe_buf + 24L)
			DBUG_PRINTF("test3",
				"can't test - pipe buffer too big");
		else
		{
			
			if ((fp = fdopen(pipe_fd[1], "w")) == NULL)
			{
				err = errno;
				if (!failed)
					xx_rpt(DELETION);
				in_rpt("fdopen() of pipe failed, errno = %d (%s)",
					err, errname(err));
				(void) close(pipe_fd[0]);
				(void) close(pipe_fd[1]);
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;

			setbuf(fp, (char *) NULL);

			errno = 0;
			if ((bp = (char *) malloc(bufsiz2)) == NULL)
			{
				err = errno;
				if (!failed)
					xx_rpt(DELETION);
				in_rpt("malloc() failed, errno = %d (%s)",
					err, errname(err));
				(void) fclose(fp);
				(void) close(pipe_fd[0]);
				(void) close(pipe_fd[1]);
				DBUG_VOID_RETURN;
			}
			else
			{
				globok = 0;
				t3_fill_pipe(pipe_fd, bp, pipe_buf);
				PATH_FUNC_TRACE;
			}

			errno = NOERROR;
			if ((nitems = fwrite((void *) bp, (size_t) 1,
				    (size_t) bufsiz2, fp)) <= 0)
			{
				if (ferror(fp) && errno == EAGAIN)
				{
					if (!failed)
						xx_rpt(DELETION);
					in_rpt("fwrite() on non-blocking pipe failed with errno set to EAGAIN");
					(void) fclose(fp);
					(void) close(pipe_fd[0]);
					(void) close(pipe_fd[1]);
					DBUG_VOID_RETURN;
				}
				else
				{
					err = errno;
					if (!failed)
						xx_rpt(FAILURE);
					in_rpt("fwrite() failed to return expected value after fwrite() could only write a partial buffer to a non-blocking pipe");
					in_rpt("expected (> 0 && < %ld), returned %d", bufsiz2, nitems);
					if(errno!=NOERROR)
					{
						in_rpt("errno = %d (%s)",
							err, errname(errno));
					}
					failed |= 2;
				}
			}
			else if (nitems < (size_t) bufsiz2 && !ferror(fp))
			{
				if (!failed)
					xx_rpt(FAILURE);
				in_rpt("stream error indicator was not set after fwrite() could only write a partial buffer to a non-blocking pipe");
				failed |= 2;
			}
			else if (nitems >= (size_t) bufsiz2)
			{
				/* This can happen on systems which do not
				   split writes, regardless how large */
				PATH_TRACE;
				DBUG_PRINTF("test3",
					"can't test - fwrite() wrote all data");
			}
			else
			{
				PATH_TRACE;	/* alternate path (2) */
				passed |= 2;
			}

			(void) fclose(fp);
		}
	}

	(void) close(pipe_fd[0]);
	(void) close(pipe_fd[1]);

#endif /* TEST_XNFS */

	if (!failed)
	{
		if (!passed)
			xx_rpt(UNTESTED);
		else
			PATH_XS_RPT(9);
	}

	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int pathok = 0;

	DBUG_ENTER("pa_t3");

	if (waitsync(t3_syn[0][0], (char *) &t3_res, sizeof(t3_res), WAITTIME) != sizeof(t3_res))
	{
		t3_res.flag = E_WAITSYNC;
	}
	else
		PATH_TRACE;

	switch (t3_res.flag)
	{
	case E_OK:
	case E_PATHCHECK:
	case E_UNTESTED:
		break;
	case E_WAITSYNC:
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to receive data from child");
		break;
	case E_ULIMIT:
		xx_rpt(DELETION);
		in_rpt("setlimit(%ld) failed - errno = %d (%s)",
			t3_ulimblks, t3_res.err, errname(t3_res.err));
		break;
	case E_CREAT:
		xx_rpt(DELETION);
		in_rpt("creat(%s) failed - errno = %d (%s)", t3_file,
			t3_res.err, errname(t3_res.err));
		break;
	case E_WRITE:
		xx_rpt(DELETION);
		in_rpt("write() failed - errno = %d (%s)",
			t3_res.err, errname(t3_res.err));
		break;
	case E_FOPEN:
		xx_rpt(DELETION);
		in_rpt("fopen(%s) failed, errno = %d (%s)", t3_file,
			t3_res.err, errname(t3_res.err));
		break;
	case E_FWRITE:
		xx_rpt(FAILURE);
		in_rpt("fwrite() failed to return expected value when ulimit was exceeded");
		in_rpt("expected (> 0 && < %d), or 0 with errno EFBIG, returned %d",
			T3_BUFSIZ, t3_res.rval);
		if (t3_res.err)
			in_rpt("errno = %d (%s)",
				t3_res.err, errname(t3_res.err));
		break;
	case E_FERROR:
		xx_rpt(FAILURE);
		in_rpt("stream error indicator was not set after fwrite() could only write a partial buffer to a file");
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("unexpected error return from child (%d)", t3_res.flag);
		break;
	}

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int	fd;
	long	count;
	long	ulimbytes;
	unsigned int	nbytes;
	FILE	*fp;
	size_t	nitems;
	char	buf[T3_BUFSIZ];
	int	i;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	t3_res.flag = E_OK;
	t3_res.rval = 0;
	t3_res.err = 0;

	if (setlimit(t3_ulimblks) == (long) SYSERROR)
	{
		t3_res.flag = E_ULIMIT;
		t3_res.err = errno;
		if (sendsync(t3_syn[0][1], (char *) &t3_res, sizeof t3_res, WAITTIME) != sizeof t3_res)
			DBUG_EXIT(A_SENDSYNC);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((ulimbytes = t3_ulimblks * 512L) >= (long) BUFSIZ)
	{
		DBUG_PRINTF("test3",
			"needed to write initial data to file: ulimit = %ld bytes, BUFSIZ = %d bytes\n",
			ulimbytes, BUFSIZ);

		if ((fd = creat(t3_file, MODEANY)) == SYSERROR)
		{
			t3_res.flag = E_CREAT;
			t3_res.err = errno;
		}
		else
		{
			PATH_TRACE;

			for (count = ulimbytes - 16L; count > 0L; count -= BUFSIZ)
			{
				nbytes = (unsigned) MIN(count, BUFSIZ);
				if (write(fd, buf, nbytes) != (int) nbytes)
				{
					t3_res.flag = E_WRITE;
					t3_res.err = errno;
					break;
				}
			}

			(void) close(fd);
			if (count <= 0L && t3_res.flag == E_OK)
				PATH_TRACE;
		}
	}
	else
	{
		/* above code is optional, so ... */
		for (i = 0; i < 2; i++)
			PATH_TRACE;
	}

	if (t3_res.flag != E_OK) {
		if (sendsync(t3_syn[0][1], (char *) &t3_res, sizeof t3_res, WAITTIME) != sizeof t3_res)
			DBUG_EXIT(A_SENDSYNC);
		DBUG_VOID_RETURN;
	}

	if ((fp = fopen(t3_file, "a")) == NULL)
	{
		t3_res.flag = E_FOPEN;
		t3_res.err = errno;
	}
	else
	{
		PATH_TRACE;

		setbuf(fp, (char *) NULL);

		errno = 0;
		if ((nitems = fwrite((void *) buf, (size_t) 1,
				sizeof buf, fp)) <= 0 || nitems >= sizeof buf)
		{
			if (nitems == 0 && errno == EFBIG)
				t3_res.flag = E_UNTESTED;
			else
				t3_res.flag = E_FWRITE;
			t3_res.rval = (int) nitems;
			t3_res.err = errno;
		}
		else if (!ferror(fp))
			t3_res.flag = E_FERROR;
		else {
			PATH_TRACE;
			t3_res.flag = (pathok == 5) ? E_OK : E_PATHCHECK;
		}

		(void) fclose(fp);
		(void) unlink(t3_file);
	}

	if (sendsync(t3_syn[0][1], (char *) &t3_res, sizeof t3_res, WAITTIME) != sizeof t3_res)
		DBUG_EXIT(A_SENDSYNC);

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS
private void
t3_fill_pipe(fd,ptr,pipe_buf)
int	fd[2];
char	*ptr;
long	pipe_buf;
{
	int	data;
	int	attempt = 0;
	int	pathok = 0;

	DBUG_ENTER("t3_fill_pipe");

	data = pipe_buf/2;
	
/* Write data to pipe until it is full */

	DBUG_PRINTF("t3_fill_pipe","Writing data to pipe");

	SET_TIMEOUT(WAITTIME);
	while((write(fd[1], ptr, (unsigned) data) == data) && (alrm_flag==0))
	{
		if(attempt++ == 0)
			PATH_TRACE;
	}

	CLEAR_ALARM;

	DBUG_PRINTF("t3_fill_pipe","Pipe full after %d writes of %ld bytes", attempt, data);

	if(alrm_flag)
	{
		DBUG_PRINTF("t3_fill_pipe","Pipe filling timed out - the pipe is not necessarily full");
	}

/* Relinquish PIPE_BUF bytes */
	(void) read(fd[0], ptr, (unsigned int) pipe_buf);

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}
#endif /* !TEST_XNFS */

private	void
test4()
{
	FILE	*fp;
	int	err;
	int	i;
	long	num[T_NUM];
	int	pathok = 0;

	DBUG_ENTER("test4");
	for (i = 0 ; i < T_NUM ; i++)
	{
		if (i==0)
			PATH_TRACE;
		num[i] = numbers[i];
	}
	if ((fp = fopen(t4_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t4_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fwrite((void *) num, (size_t) sizeof(long), (size_t) T4_NUM, fp);
	(void) fclose(fp);
	(void) unlink(t4_file);

	for (i = 0 ; i < T_NUM ; i++)
	{
		if (num[i] != numbers[i])
		{
			xx_rpt(FAILURE);
			in_rpt("fwrite() changed contents of buffer");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	PATH_XS_RPT(2+T_NUM);

	DBUG_VOID_RETURN;
}


private	void
test5()
{
	FILE	*fp;
	int	err;
	int	fd;
	int	fail = 0;
	int	nread;
	char	*fa = "first append";
	char	buf[512];
	int	pathok = 0;

	DBUG_ENTER("test5");
	if ((fd = creat(t5_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d(%s)",
			t5_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if (write(fd, t_string, (unsigned) strlen(t_string))
		!= strlen(t_string))
	{
		xx_rpt(DELETION);
		in_rpt("write() did not write whole buffer");
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	if ((fp = fopen(t5_file, "a")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a\") failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	rewind(fp);
	if (fwrite((void *)fa, (size_t) 1, strlen(fa), fp) != strlen(fa))
	{
		xx_rpt(FAILURE);
		in_rpt("fwrite() did not write whole buffer to file opened for \"a\"");
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fclose(fp);
	if ((fd = open(t5_file, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((nread = read(fd, buf, sizeof(buf) - 1)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("read() failed - errno %d(%s)",
			err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	(void) unlink(t5_file);
	buf[nread] = '\0';
	if (nread != strlen(t_string) + strlen(fa)
		|| strncmp(buf, t_string, strlen(t_string))
		|| strcmp(&buf[strlen(t_string)], fa))
			fail |= 1;
	else
		PATH_TRACE;

	if ((fd = creat(t5_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d(%s)",
			t5_file, MODEANY, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, t_string, (unsigned) strlen(t_string))
		!= strlen(t_string))
	{
		xx_rpt(DELETION);
		in_rpt("write() did not write whole buffer");
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	if ((fp = fopen(t5_file, "a+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rewind(fp);
	if (fwrite((void *)fa, (size_t) 1, strlen(fa), fp) != strlen(fa))
	{
		xx_rpt(FAILURE);
		in_rpt("fwrite() did not write whole buffer to file opened for \"a+\"");
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fclose(fp);
	if ((fd = open(t5_file, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((nread = read(fd, buf, sizeof(buf) - 1)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("read() failed - errno %d(%s)",
			err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);
	(void) unlink(t5_file);
	buf[nread] = '\0';
	if (nread != strlen(t_string) + strlen(fa)
		|| strncmp(buf, t_string, strlen(t_string))
		|| strcmp(&buf[strlen(t_string)], fa))
			fail |= 2;
	else
		PATH_TRACE;

	if ((fd = creat(t5_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d(%s)",
			t5_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if (write(fd, t_string, (unsigned) strlen(t_string))
		!= strlen(t_string))
	{
		xx_rpt(DELETION);
		in_rpt("write() did not write whole buffer");
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	if ((fp = fopen(t5_file, "ab")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"ab\") failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	rewind(fp);
	if (fwrite((void *)fa, (size_t) 1, strlen(fa), fp) != strlen(fa))
	{
		xx_rpt(FAILURE);
		in_rpt("fwrite() did not write whole buffer to file opened for \"a\"");
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fclose(fp);
	if ((fd = open(t5_file, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((nread = read(fd, buf, sizeof(buf) - 1)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("read() failed - errno %d(%s)",
			err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	(void) unlink(t5_file);
	buf[nread] = '\0';
	if (nread != strlen(t_string) + strlen(fa)
		|| strncmp(buf, t_string, strlen(t_string))
		|| strcmp(&buf[strlen(t_string)], fa))
			fail |= 4;
	else
		PATH_TRACE;

	if ((fd = creat(t5_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d(%s)",
			t5_file, MODEANY, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, t_string, (unsigned) strlen(t_string))
		!= strlen(t_string))
	{
		xx_rpt(DELETION);
		in_rpt("write() did not write whole buffer");
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	if ((fp = fopen(t5_file, "ab+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"a+\") failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rewind(fp);
	if (fwrite((void *)fa, (size_t) 1, strlen(fa), fp) != strlen(fa))
	{
		xx_rpt(FAILURE);
		in_rpt("fwrite() did not write whole buffer to file opened for \"a+\"");
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fclose(fp);
	if ((fd = open(t5_file, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((nread = read(fd, buf, sizeof(buf) - 1)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("read() failed - errno %d(%s)",
			err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);
	(void) unlink(t5_file);
	buf[nread] = '\0';
	if (nread != strlen(t_string) + strlen(fa)
		|| strncmp(buf, t_string, strlen(t_string))
		|| strcmp(&buf[strlen(t_string)], fa))
			fail |= 8;
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("fwrite() did not write to end of file when file was opened with \"a\"");
		if (fail & 2)
			in_rpt("fwrite() did not write to end of file when file was opened with \"a+\"");
		if (fail & 4)
			in_rpt("fwrite() did not write to end of file when file was opened with \"ab\"");
		if (fail & 8)
			in_rpt("fwrite() did not write to end of file when file was opened with \"ab+\"");
	}
	else
		PATH_XS_RPT(26);

	DBUG_VOID_RETURN;
}


private	void
test6()
{
	FILE	*fp;
	int	err;
	long	rval;
	int	pathok = 0;

	DBUG_ENTER("test6");
	if ((fp = fopen(t6_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t6_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fwrite((void *)t_string, (size_t) 1, strlen(t_string), fp);
	if ((rval = ftell(fp)) != (long) strlen(t_string))
	{
		xx_rpt(FAILURE);
		in_rpt("fwrite() did not change the position in file.");
		in_rpt("ftell() returned %ld, expected %d",
			rval, strlen(t_string));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	(void) unlink(t6_file);
	DBUG_VOID_RETURN;
}

private	void
test7()
{
	FILE	*fp;
	int	fd, rval, err, fail;
	size_t	len1, len2;
	char	buf[50];
	int	pathok = 0;

	DBUG_ENTER("test7");

	fail = 0;

	len1 = strlen(t_string);
	len2 = len1 / 2;
	len1 -= len2;

	if ((fp = fopen(t7a_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t7a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)t_string, (size_t) 1, len1, fp) != len1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fwrite(s, 1, %d, fp) failed on file %s - errno %d(%s)",
			(int)len1, t7a_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t7a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = fwrite((void *)t_string, (size_t) 1, (size_t) 0, fp)) != 0)
	{
		fail++;
		xx_rpt(FAILURE);
		in_rpt("fwrite(s, 1, 0, fp) returned %d, expected 0", rval);
	}
	else
		PATH_TRACE;
	if (fwrite((void *)&t_string[len1], (size_t) 1, len2, fp) != len2)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fwrite(s, 1, %d, fp) failed on file %s - errno %d(%s)",
			(int)len2, t7a_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t7a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fclose(fp);
	if ((fd = open(t7a_file, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d(%s)",
			t7a_file, err, errname(err));
		(void) unlink(t7a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (read(fd, buf, sizeof(buf)) != len1+len2 ||
				strncmp(buf, t_string, len1+len2) != 0)
	{
		if (fail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fwrite(s, 1, 0, fp) altered stream");
	}
	else
		PATH_TRACE;
	(void) close(fd);
	(void) unlink(t7a_file);

	if ((fp = fopen(t7b_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t7b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)t_string, (size_t) 1, len1, fp) != len1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fwrite(s, 1, %d, fp) failed on file %s - errno %d(%s)",
			(int)len1, t7b_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t7b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = fwrite((void *)t_string, (size_t) 0, (size_t) 1, fp)) != 0)
	{
		fail++;
		xx_rpt(FAILURE);
		in_rpt("fwrite(s, 0, 1, fp) returned %d, expected 0", rval);
	}
	else
		PATH_TRACE;
	if (fwrite((void *)&t_string[len1], (size_t) 1, len2, fp) != len2)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fwrite(s, 1, %d, fp) failed on file %s - errno %d(%s)",
			(int)len2, t7b_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t7b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fclose(fp);
	if ((fd = open(t7b_file, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d(%s)",
			t7b_file, err, errname(err));
		(void) unlink(t7b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (read(fd, buf, sizeof(buf)) != len1+len2 ||
				strncmp(buf, t_string, len1+len2) != 0)
	{
		if (fail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fwrite(s, 0, 1, fp) altered stream");
	}
	else
		PATH_TRACE;
	(void) close(fd);
	(void) unlink(t7b_file);

	if (fail == 0)
		PATH_XS_RPT(12);

	DBUG_VOID_RETURN;
}


private void
test8()
{
	FILE	*fp;
	char	*res;
	int	err;
	char	s[1];
	int	pathok = 0;

	DBUG_ENTER("test8");

	if ((fp = fopen(t8_file, "w")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t8_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	globok = 0;
	savtimes(t8_file);
	PATH_FUNC_TRACE;

        if (fwrite((void *)s, (size_t)1, (size_t)1, fp) != 1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fwrite(s, 1, 1, fp) failed on file %s, errno %d(%s)",
			t8_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fclose(fp) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fclose() failed on file %s, errno %d(%s)",
			t8_file, err, errname(err));
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ((res = chktimes(t8_file, NEWMTIME | NEWCTIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(res);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(5);
	}

	(void) unlink(t8_file);
	DBUG_VOID_RETURN;
}

/* Lowest test number for which following function is used */
#define TBASE	9

private	void
test9_15()
{
	int 	experr, err, ret, ferr_ret, sig_ret;
	static int errnums[] = { EAGAIN, -EAGAIN, EBADF, EINTR, EFBIG,
			      /* test9   test10   test11 test12 test13 */
				 EPIPE, -EPIPE };
			      /* test14 test15 */
	int	pathok = 0;

	DBUG_ENTER("test9_15");


#if TEST_XNFS
	if(tet_thistest != 13)
	{ 
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}
#endif /* TEST_XNFS */

	experr = errnums[tet_thistest - TBASE];

	globok = 0;
	ret = do_ferr(experr, 'w', -1, t9_15func, &ferr_ret, &sig_ret);
	PATH_FUNC_TRACE;
	err = errno;
	if (experr < 0)
		experr = -experr;

	if (tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (err != experr || !ret || !ferr_ret || (experr == EPIPE && !sig_ret))
	{
		xx_rpt(FAILURE);
		if (!ferr_ret)
			in_rpt("fwrite() failed to set error indicator when testing for errno %d(%s)",
				experr, errname(experr));
		if (experr == EPIPE && !sig_ret)
			in_rpt("SIGPIPE was not received");
		if (!ret)
			in_rpt("fwrite() returned nitems when testing for errno %d(%s), expected < nitems",
				experr, errname(experr));
		else if (err != experr)
			in_rpt("fwrite() set errno incorrectly: expected %d(%s), actual %d(%s)",
				experr, errname(experr), err, errname(err));
	}
	else
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private int
t9_15func(fp, nbyte)
FILE *fp;
int nbyte;
{
	size_t 	rval;
	char 	buf[FERR_BUFSZ];
	int	pathok = 0;

	DBUG_ENTER("t9_15func");

	if (nbyte > FERR_BUFSZ)
	{
		er_rpt("nbyte too great in function called from do_ferr()");
		nbyte = FERR_BUFSZ;
	}
	else
		PATH_TRACE;

	rval = fwrite((void *)buf, (size_t)1, (size_t)nbyte, fp);

	/*
	 * Grey Area:  If a partial write occurs, do a second fwrite() to
	 * force errno to be set.  This is not a very nice way for fwrite()
	 * to behave (it forces applications to loop on fwrite()), but
	 * has to be allowed for the present, until the issue is resolved.
	 */
	if (rval < nbyte && rval > 0)
	{
		nbyte -= rval;
		rval = fwrite((void *)&buf[rval], (size_t)1, (size_t)nbyte, fp);
		PATH_TRACE;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(rval < nbyte);
}

#if TEST_XNFS

private void
test16()
{
	DBUG_ENTER("test16");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test16()
{
	DBUG_ENTER("test16");

	do_feio("w", -1, t16io, t16rpt);

	DBUG_VOID_RETURN;
}

private int
t16io(fp)
FILE *fp;
{
	int 	ret;
	int	pathok = 0;

	DBUG_ENTER("t16io");

	globok = 0;

	ret = fwrite((void *)"\n", (size_t)1, (size_t)1, fp);

	PATH_FUNC_RPT(0);

	DBUG_RETURN(ret);
}

private void
t16rpt(ret, err)
int ret, err;
{
	int	pathok = 0;

	DBUG_ENTER("t16rpt");

	PATH_FUNC_TRACE; /* checking value of globok from iofunc() */

	if (ret > 0 || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("fwrite() from orphaned background process group did not give correct results");
		if (ret > 0)
			in_rpt("expected return: <= 0, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test17()
{

	DBUG_ENTER("test17");

	(void) cppair(ch_t17, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t17()
{
	int 	err;
	char	*args[6];

	DBUG_ENTER("ch_t17");

	args[0] = prog17;
	args[1] = t17_dir;
	args[2] = "fwrite";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog17), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

private void
test18()
{
	DBUG_ENTER("test18");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test18()
{
	DBUG_ENTER("test18");

	(void) cppair(ch_t18, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t18()
{
	int err;
	char *args[6];

	DBUG_ENTER("ch_t18");

	args[0] = prog18;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t18_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog18), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog18), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test19()
{
	DBUG_ENTER("test19");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test19()
{
	DBUG_ENTER("test19");

	t_estale_wr_c(t19_file, 0, t19_func, t19_chk_func, SP_T19_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t19_func(fp)
FILE	*fp;
{
	int	nbytes;

	DBUG_ENTER("t19_func");

	nbytes = strlen(t_string_xnfs);

	DBUG_RETURN(fwrite((void *)t_string_xnfs, (size_t)1, (size_t)nbytes, fp) != nbytes ? -1 : 0);
}

private int
t19_chk_func(fp, ebytes)
FILE	*fp;
int	ebytes;
{
	int	rval, i, err;
	int	nbytes;
	int	pathok = 0;

	DBUG_ENTER("t19_chk_func");

	errno = NOERROR;

	nbytes = strlen(t_string_xnfs);

	for(i = 0; i < ebytes; i++) {
		rval = fwrite((void *)t_string_xnfs, (size_t)1, (size_t)nbytes, fp);
		if(rval != nbytes)
			break;
        }


	err = errno;

	if(ferror(fp) == 0 || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;

		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(2);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
