/*
*      SCCS:  @(#)  ANSI.os/streamio/fwrite/fwrite_t18.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1992 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fwrite - fwrite_t18 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1992 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		fwrite_t18 - used by T.fwrite in privileged tests

PROJECT:	XNFS (X/OPEN NFS tests)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.fwrite to perform privileged tests.

	  usage:
		fwrite_t18 testnum dir file ident dbug

	  where testnum is the test number to be executed, and
	  file is a parameter to be passed to the test.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		John-Paul, UniSoft Ltd.
DATE CREATED:	2 November 1992
MODIFICATIONS:
		Geoff Clare, 15 June 1993
			Cast fwrite() args correctly in t18_chk_func().
			Include <string.h>.

************************************************************************/
#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <dbug.h>
#include <errno.h>
#include <sys/stat.h>
#include <report.h>
#include <tet_api.h>
#include <xnfs.h>
#include "fwrite_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef fwrite
#endif


public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	char	*t_string_xnfs =	XNFS_STRING;

extern	char	**environ;
extern	char	*errname();

extern	void	valrpt();
extern	void	set_pname();

private	int	t18_chk_func();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	t_eacces_wr_c(argv[3], 0, t18_chk_func, SP_T18_EACCES);

 	return(OK);
}

private int
t18_chk_func(fp, ebytes)
FILE	*fp;
int	ebytes;
{
	int	i, rval, err;
	int	nbytes;
	int	pathok = 0;

	DBUG_ENTER("t18_chk_func");

	nbytes = strlen(t_string_xnfs);

	errno = NOERROR;

	for(i = 0; i < ebytes; i++) {
		rval = fwrite((void *)t_string_xnfs, (size_t)1, (size_t)nbytes, fp);
		if(rval != nbytes)
			break;
        }

	err = errno;

	if(ferror(fp) == 0 || err != EACCES)
	{
		xx_rpt(FAILURE);

		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;

		if(err != EACCES)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",EACCES,errname(EACCES));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(2);

	DBUG_RETURN(0);
}
