/*
*      SCCS:  @(#)  ANSI.os/streamio/getc/getc.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/getc - T.getc Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.getc

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION: 	Testset for the getc()/getchar()/fgetc() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 8 Nov 1988
		Add tests for errno values
		Split out T.getw, implement "interface" method

		Geoff Clare, 30 May 1989
		Add EIO tests

		Geoff Clare, 14 June 1990
		Do EAGAIN test on pipe and FIFO.

		David Sawyer
		Thu Jul 19 11:34:43 BST 1990
		Added resillience code.

		Stuart Boutell, 5 Nov 1990
			changed dltd and reason to tet_reason.

		Stuart Boutell, 7 Nov 1990
			test name extraction code rewritten.

		Stuart Boutell, 23 Nov 1990
			All tests move into child processes to prevent
			accidental closure of report file descriptor.

		Neil Moses, 9 June 1992
			New test 11.  On ROF time stamps are not changed.

                John-Paul, 27 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		John-Paul, 11 October 1992
			XNFS enhancements.

		Geoff Clare, 29 Oct 1993
			Use new file in XNFS st_atime test to avoid
			reading cached data.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<chktimes.h>
#include	<xnfs.h>
#include	"getc_h.h"
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getc
#undef getchar
#undef fgetc
#endif


#define	NO_TESTS	13		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	T4_NUM		3

#define I_GETC		1
#define I_GETCHAR	2
#define I_FGETC		3

#define STDIN		 (interface == I_GETC ? 0 : \
			  interface == I_GETCHAR ? 1 : 0 )
#define GETIT(fp)	 (interface == I_GETC ? getc(fp) : \
			  interface == I_GETCHAR ? getchar() : \
			  getfunc(fgetc, fp))

extern	int	alrm_flag;
extern	char	*vg_ident;

extern	char	*errname();
extern	char	*execpath();
extern	char	*itoa();
extern	char	*pr_char();
extern	void	do_feio();
extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

int	no_tests = NO_TESTS;

#if TEST_XNFS
private	int	snames[] = {1, 0};
#endif /* TEST_XNFS */

private	char	*prog_su =	"getc_su";
private	char	*t_file =	"./getc-t";
#if TEST_XNFS
private char	*t4_file =	T4_FILE;
#else
#define		t4_file		t_file
#endif
private char	*t11_dir =	"getc-d.11";
private char	*t11_file =	"getc-d.11/getc-t.11";
#if TEST_XNFS
private char	*t12_file =	T12_FILE;
private char	*t13_file =	T13_FILE;
#endif /* TEST_XNFS */
private	char	*function;
private	int	interface = 0;
#if !TEST_XNFS
private	char	*sigstat;
#endif /* TEST_XNFS */

private	void	comp_prep();
private	void	comp_fin();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5_8();
private	void	test9();
private	void	test10();
private	void	test11();
private	void	test12();
private	void	test13();
#if !TEST_XNFS
private	int	t5_8func();
private	int	t9_10io();
private	void	t9rpt();
private	void	t10rpt();
#endif /* TEST_XNFS */
private	void	ch_t1();
private	void	ch_t2();
private	void	ch_t3();
private	void	ch_t4();
#if !TEST_XNFS
private	void	ch_t5_8();
private	void	ch_t9();
private	void	ch_t10();
#endif /* TEST_XNFS */
private	void	ch_t11();
#if TEST_XNFS
private	void	ch_t12();
private	int	t13_func();
private	int	t13_chk_func();
#endif /* TEST_XNFS */
private	int	getfunc();

public	void	(*setfunc) () = comp_prep;
public	void	(*clnfunc) () = comp_fin;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5_8, 5,
	 test5_8, 6,
	 test5_8, 7,
	 test5_8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	NULL, 0
};


private int
getfunc(farg, fp)
int	(*farg)();
FILE	*fp;
{
	return (*farg)(fp);
}

private void
comp_prep()
{
	char		*s;
	int		fd;
	int		i;
	int		bad = 0;
	static char	msg[150];

	if((s = strrchr(vg_ident, '/'))==NULL)
		s = vg_ident+2;
	else
		s+=3;

	DBUG_PRINTF("comp_prep","vg_ident: %s s:%s", vg_ident, s);
	if (strcmp(s, "getc") == 0)
	{
		interface = I_GETC;
		function = "getc()";
	}
	else if (strcmp(s, "getchar") == 0)
	{
		interface = I_GETCHAR;
		function = "getchar()";
	}
	else if (strcmp(s, "fgetc") == 0)
	{
		interface = I_FGETC;
		function = "fgetc()";
	}
	else
	{
		for (i=1; i<=NO_TESTS; i++)
			(void) tet_delete(i, "bad ident parameter");
		return;
	}

	if ((fd = creat(t_file, MODEANY)) == SYSERROR)
	{
		bad = 1;
		(void) sprintf(msg, "creat(%s, %#o) failed in preperations - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
	}
	else
	{
		if (write(fd, "abcdefghijklmnopqrstuvwxyz", 26) != 26)
		{
			(void) sprintf(msg, "write() failed in preperations - errno %d(%s)",
				errno, errname(errno));
			bad = 1;
		}
		(void) close(fd);
	}
	if (bad)
		for (i = 1 ; i <= NO_TESTS ; i++)
			(void) tet_delete(i, msg);
}

private	void
comp_fin()
{
	(void) unlink(t_file);
}


private	void
test1()
{
	DBUG_ENTER("test1");
	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);
	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	FILE		*fp;
	int		err;
	int		c1;
	int		c2;
	int		c3;
	int		pathok = 0;

	DBUG_ENTER("ch_t1");

	errno = NOERROR;
	if (freopen(t_file, "r", stdin) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fp = stdin;

	c1 = GETIT(fp);
	c2 = GETIT(fp);
	c3 = GETIT(fp);
	if (c1 != 'a' || c2 != 'b' || c3 != 'c')
	{
		xx_rpt(FAILURE);
		if (c1 != 'a')
		{
			in_rpt("%s did not return first character in file",
				function);
			in_rpt("expected %c, got %s(%d)",
				'a', pr_char((char) c1), c1);
		}
		else if (c2 != 'b')
		{
			in_rpt("second %s did not return second character in file", function);
			in_rpt("expected %c, got %s(%d)",
				'b', pr_char((char) c2), c2);
		}
		else if (c3 != 'c')
		{
			in_rpt("third %s did not return third character in file", function);
			in_rpt("expected %c, got %s(%d)",
				'c', pr_char((char) c3), c3);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private	void
test2()
{
	DBUG_ENTER("test2");
	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);
	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	FILE		*fp;
	int		err;
	int		i;
	long		off0;
	long		diff1;
	int		pathok = 0;

	DBUG_ENTER("ch_t2");
	errno = NOERROR;
	if (freopen(t_file, "r", stdin) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fp = stdin;
	off0 = ftell(fp);
	for (i = 0 ; i < T4_NUM ; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) GETIT(fp);
	}
	diff1 = ftell(fp) - off0;
	if (diff1 != T4_NUM)
	{
		xx_rpt(FAILURE);
		in_rpt("%s did not update file pointer properly", function);
		in_rpt("after %d calls to %s, file pointer moved by %d bytes",
			T4_NUM, function, diff1);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private	void
test3()
{
	DBUG_ENTER("test3");
	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);
	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	FILE		*fp;
	int		err;
	int		val1;
	int		pathok = 0;

	DBUG_ENTER("ch_t3");
	errno = NOERROR;
	if (freopen(t_file, "r", stdin) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fp = stdin;
	(void) fseek(fp, 0L, SEEK_END);
	if ((val1 = GETIT(fp)) != EOF)
	{
		xx_rpt(FAILURE);
		in_rpt("%s did not return expected value at end of file",
			function);
		in_rpt("expected %d(EOF), got %d", EOF, val1);
	}
	else if (!feof(fp))
	{
		xx_rpt(FAILURE);
		in_rpt("%s returned EOF correctly but failed to set the EOF indicator",
			function);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private void
test4()
{
	DBUG_ENTER("test4");
	(void) cppair(ch_t4, NULLFN, WAITTIME, 0);
	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	FILE	*fp;
	char	*res;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

#if TEST_XNFS
	
	if (tet_sync(SP_T4_CREAT_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREAT_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */
		PATH_TRACE;
#endif /* TEST_XNFS */

	if (freopen(t4_file, "r", stdin) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
			t4_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fp = stdin;

	globok = 0;
	savtimes(t4_file);
	PATH_FUNC_TRACE;

        if (GETIT(fp) == EOF)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s failed on file %s, errno %d(%s)",
			function, t4_file, err, errname(err));
#ifdef CLOSE_FCLOSE
		(void) fclose(fp);
#endif
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ((res = chktimes(t4_file, NEWATIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(res);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(5);
	}

	(void) fclose(fp);

#if TEST_XNFS
	(void) tet_sync(SP_T4_UNLINK_START, snames, SYNC_TIMEOUT);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

/* Lowest test number for which following function is used */
#define TBASE	5

#if TEST_XNFS

private	void
test5_8()
{
	DBUG_ENTER("test5_8");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test5_8()
{
	DBUG_ENTER("test5_8");
	(void) cppair(ch_t5_8, NULLFN, WAITTIME*2, 0);
	DBUG_VOID_RETURN;
}

private void
ch_t5_8()
{
	int 	experr, err, ret, ferr_ret, reqfd = -1;
	static 	int errnums[] = { EAGAIN, -EAGAIN, EBADF, EINTR, };
			      /* test5   test6    test7  test8  */
	int	pathok = 0;

	DBUG_ENTER("ch_t5_8");

	/* If going to use stdin, it must be a "virgin" stream */
	if (interface == I_GETCHAR)
	{
		reqfd = STDIN_FILENO;
		if (freopen(t_file, "r", stdin) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
				t_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	experr = errnums[tet_thistest - TBASE];

	globok = 0;
	ret = do_ferr(experr, 'r', reqfd, t5_8func, &ferr_ret, &err);
	PATH_FUNC_TRACE;
	err = errno;
	if (experr < 0)
		experr = -experr;

	if (tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!ferr_ret || err != experr || ret != EOF)
	{
		xx_rpt(FAILURE);
		if (!ferr_ret)
			in_rpt("%s failed to set error indicator when testing for errno %d(%s)", function, experr, errname(experr));
		if (ret != EOF)
			in_rpt("%s returned %d when testing for errno %d(%s), expected EOF", function, ret, experr, errname(experr));
		else if (err != experr)
			in_rpt("%s set errno incorrectly: expected %d(%s), actual %d(%s)", function, experr, errname(experr), err, errname(err));
	}
	else
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private int
t5_8func(fp, nbyte)
FILE *fp;
int nbyte;
{
	int 	ret, i;
	int	pathok = 0;

	DBUG_ENTER("t5_8func");

	for (i = 0; i < nbyte; i++)
	{
		if (i==0)
			PATH_TRACE;
		if ((ret = GETIT(fp)) == EOF || alrm_flag > 0)
			break;
	}
	PATH_FUNC_RPT(1);

	DBUG_RETURN(ret);
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private	void
test9()
{
	DBUG_ENTER("test9");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test9()
{
	DBUG_ENTER("test9");
	(void) cppair(ch_t9, NULLFN, WAITTIME*2, 0);
	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int 	reqfd = -1;
	int 	err;

	DBUG_ENTER("ch_t9");

	if (interface == I_GETCHAR)
	{
		reqfd = STDIN_FILENO;
		if (freopen(t_file, "r", stdin) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
				t_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
	}

	sigstat = "ignored";
	do_feio("ri", reqfd, t9_10io, t9rpt);

	if (interface == I_GETCHAR)
	{
		if (freopen(t_file, "r", stdin) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
				t_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
	}

	sigstat = "blocked";
	do_feio("rb", reqfd, t9_10io, t9rpt);

	DBUG_VOID_RETURN;
}

private void
t9rpt(ret, err)
int ret, err;
{
	int	pathok = 0;

	DBUG_ENTER("t9rpt");

	PATH_FUNC_TRACE; /* chech value of globok from iofunc() */

	if (ret != EOF || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("%s from background with SIGTTIN %s did not give correct results",
			function, sigstat);
		if (ret != EOF)
			in_rpt("expected return: EOF, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private	void
test10()
{
	DBUG_ENTER("test10");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test10()
{
	DBUG_ENTER("test10");
	(void) cppair(ch_t10, NULLFN, WAITTIME*2, 0);
	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	int 	reqfd = -1;
	int 	err;

	DBUG_ENTER("ch_t10");

	if (interface == I_GETCHAR)
	{
		reqfd = STDIN_FILENO;
		if (freopen(t_file, "r", stdin) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
				t_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
	}

	do_feio("ro", reqfd, t9_10io, t10rpt);

	DBUG_VOID_RETURN;
}

private void
t10rpt(ret, err)
int ret, err;
{
	int	pathok = 0;

	DBUG_ENTER("t10rpt");

	PATH_FUNC_TRACE; /* check value of globok from iofunc() */

	if (ret != EOF || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("%s from orphaned background process did not give correct results",
			function);
		if (ret != EOF)
			in_rpt("expected return: EOF, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private int
t9_10io(fp)
FILE *fp;
{
	int 	ret;
	int	pathok = 0;

	DBUG_ENTER("t9_10io");

	ret = GETIT(fp);

	PATH_FUNC_RPT(0);

	DBUG_RETURN(ret);
}

#endif /* TEST_XNFS */

private void
test11()
{
	DBUG_ENTER("test11");

	(void) cppair(ch_t11, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t11_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	char	*rofs;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t11");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_ROFS is not defined");
		DBUG_VOID_RETURN;
	}

	args[0] = prog_su;
	args[1] = itoa(interface);
	args[2] = t11_dir;
	args[3] = t11_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog_su, err, errname(err));

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

private void
test12()
{
	DBUG_ENTER("test12");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test12()
{
	DBUG_ENTER("test12");

	(void) cppair(ch_t12, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t12()
{
	int err;
	char *args[6];

	DBUG_ENTER("ch_t12");

	args[0] = prog_su;
	args[1] = itoa(interface);
	args[2] = t12_file;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test13()
{
	DBUG_ENTER("test13");

	t_estale_rd_c(t13_file, STDIN, t13_func, t13_chk_func, SP_T13_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t13_func(fp)
FILE	*fp;
{
	DBUG_ENTER("t13_func");

	DBUG_RETURN(GETIT(fp) == EOF ? -1 : 0);
}

private int
t13_chk_func(fp)
FILE	*fp;
{
	int	rval, err;
	int	pathok = 0;

	DBUG_ENTER("t13_chk_func");

	errno = NOERROR;

	while((rval = GETIT(fp)) != EOF)
		;

	err = errno;

	if(rval != EOF || ferror(fp) == 0 || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(rval != EOF)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value EOF");
			in_rpt("Actual   return value %d", rval);
		}
		else
			PATH_TRACE;
		
		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;
		
		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
