/*
*      SCCS:  @(#)  ANSI.os/streamio/getc/getc_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/getc - getc_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		getc_su - used by T.getc in privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.getc to perform privileged tests.

	  usage:
		getc_su interface [dir] file ident dbug

	  where interface is the function to be executed eg getc, fgetc, and
	  getchar, and dir and file are parameters to be passed to the test.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Neil Moses, UniSoft Ltd.
DATE CREATED:	9 June 1992
MODIFICATIONS:
		Dave Cater, UniSoft Ltd, October 1992
			XNFS enhancements.

		John-Paul, 3 November 1992
			Addition of call to 't_eacces_stdin_c' for getchar
			case.
		Dave Cater, 4 November 1992
			Changed t_eacces_stdin_c to t_eacces_rd_c.

************************************************************************/
#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <dbug.h>
#include <errno.h>
#include <sys/stat.h>
#include <report.h>
#include <tet_api.h>
#include <xnfs.h>
#include "getc_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef getc
#undef getchar
#undef fgetc
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

#define I_GETC		1
#define I_GETCHAR	2
#define I_FGETC		3

#define STDIN		 (interface == I_GETC ? 0 : \
			  interface == I_GETCHAR ? 1 : 0 )
#define GETIT(fp)	 (interface == I_GETC ? getc(fp) : \
			  interface == I_GETCHAR ? getchar() : \
			  getfunc(fgetc, fp))

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
public  int	testfail;

extern	char	**environ;
extern	void	savtimes();
extern	char	*chktimes();
extern	char	*errname();

extern	void	valrpt();
extern	void	set_pname();

private	void	subtest11();
#if TEST_XNFS
private int	t12_chk_func();
#endif /* TEST_XNFS */

private	int	interface;

private int
getfunc(farg, fp)
int	(*farg)();
FILE	*fp;
{
	return (*farg)(fp);
}

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	interface = atoi(argv[1]);

	switch (tet_thistest) {
	case 11:
		subtest11(argv[2], argv[3]);
		break;
#if TEST_XNFS
	case 12:
		t_eacces_rd_c(argv[2], STDIN, t12_chk_func, SP_T12_EACCES);
		break;
#endif /* TEST_XNFS */
	default:
		xx_rpt(DELETION);
		in_rpt("unexpected tet_thistest = %d",
			tet_thistest);
		break;
	}

	return(OK);
}

private void
subtest11(mdir, file)
char	*mdir, *file;
{
	FILE	*fp;
	char	*func;
	int 	rval;
	char	*errptr;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("subtest11");

	errno = 0;
	testfail = 0;

	globok = 0;
	if (setuprofs(mdir, file, 'f', MODEANY) != 0)
	{
		DBUG_VOID_RETURN;	/* reported in setuprofs() */
	}
	else
		PATH_FUNC_TRACE;

	switch(interface)
	{

	case I_GETCHAR:

		if((fp = freopen(file, "r", stdin)) == NULL)
		{
			err = errno;
			if(!testfail++)
				xx_rpt(DELETION);
			in_rpt("freopen(\"%s\", \"r\", stdin) failed - errno %d (%s)", 
				file, err, errname(err));
			(void) clnuprofs(mdir, file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;		
		break;

	default :

		if((fp = fopen(file, "r")) == NULL)
		{
			err = errno;
			if(!testfail++)
				xx_rpt(DELETION);
			in_rpt("fopen(\"%s\", \"r\") failed - errno %d (%s)", 
				file, err, errname(err));
			(void) clnuprofs(mdir, file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;		
		break;

	}

	globok = 0;
	savtimes(file);
	PATH_FUNC_TRACE;

	errno = NOERROR;

	switch(interface)
	{
	case I_GETC:
		func = "getc";
		rval = getc(fp);
		break;
	case I_FGETC:
		func = "fgetc";
		rval = fgetc(fp);
		break;
	case I_GETCHAR:
		func = "getchar";
		rval = getchar();
		break;
	};

#ifdef TESTBED
	rval = EOF;
	if (errno)
		errno++;
#endif
	err = errno;

	if (rval != EOF || ferror(fp))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (rval != EOF)
			in_rpt("%s() returned %d, expected EOF", func, rval);
		else
		{
			in_rpt("%s() set the error indicator for the stream",
				func);
			in_rpt("errno was set to %d (%s)", err, errname(err));
		}
		(void) fclose(fp);
		(void) clnuprofs(mdir, file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	errptr = chktimes(file, 0);
	PATH_FUNC_TRACE;

	if(errptr != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("Call to %s() changed time stamps on %s", func, file);
		in_rpt("%s", errptr);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) fclose(fp);
	(void) clnuprofs(mdir, file);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private int
t12_chk_func(fp)
FILE	*fp;
{
	int	rval, err;
	int	pathok = 0;

	DBUG_ENTER("t12_chk_func");

	errno = NOERROR;

	while((rval = GETIT(fp)) != EOF)
		;

	err = errno;

	if(rval != EOF || ferror(fp) == 0 || err != EACCES)
	{
		xx_rpt(FAILURE);

		if(rval != EOF)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value EOF");
			in_rpt("Actual   return value %d", rval);
		}
		else
			PATH_TRACE;
		
		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;
		
		if(err != EACCES)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",EACCES,errname(EACCES));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
