/*
*      SCCS:  @(#)  ANSI.os/streamio/gets/gets.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/gets - T.gets Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.gets

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the gets() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 9 Nov 1988
		Add tests for errno values.

		Geoff Clare, 30 May 1989
		Add EIO tests

		Geoff Clare, 14 June 1990
		Do EAGAIN test on pipe and FIFO.

		Peter Lee, Unisoft Ltd.,
		Mon Jul 16 17:10:57 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			changed dltd and reason to tet_reason.

		Stuart Boutell, 1 Dec 1990
			Added code to ensure file descriptor STDIN_FILENO
			is open after tests 6-9.

		Neil Moses, 10 June 1992
			New test 12.  On ROF time stamps are not changed.

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		John-Paul, 2 November 1992
			XNFS enhancements.

		Geoff Clare, 29 Oct 1993
			Use new file in XNFS st_atime test to avoid
			reading cached data.

		Geoff Clare, 23 Sept 1994
			Declare buffer passed to tet_delete() as static.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<chktimes.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"gets_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef gets
#endif


#define	NO_TESTS	14		/* number of tests in testset */
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	WAITTIME	(SPEEDFACTOR*5)
#define	T2_OFF	20
#define	T4_OFF	16


extern	char	*errname();
extern	char	*execpath();
#if !TEST_XNFS
extern	void	do_feio();
#endif /* TEST_XNFS */
extern	char	*itoa();
extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

public	int	no_tests = NO_TESTS;

#if TEST_XNFS
private	int	snames[] = {1, 0};
#endif /* TEST_XNFS */

private char	*t_string =	"abcdefghijklmnop\nqrstuvwxyz";
private	char	*t1_string =	"abcdefghijklmnop";
private	char	*t2_string =	"tuvwxyz";
private	char	*t_file =	"./gets-t";
#if TEST_XNFS
private char	*t5_file =	T5_FILE;
#else
#define		t5_file		t_file
#endif
#if TEST_XNFS
private	char	*t13_file =	T13_FILE;
private	char	*t14_file =	T14_FILE;
#endif /* TEST_XNFS */
private	char	*prog_su =	"gets_su";
private char	*t12_dir =	"gets-d.12";
private char	*t12_file =	"gets-d.12/gets-t.12";
#if !TEST_XNFS
private	struct	stat	ensure_buf; /* buffer for reopen stdin code */
private	char	*sigstat;
#endif /* TEST_XNFS */
private	char	t_buf[30];
private FILE	*t_fp =		(FILE *) NULL;
private int	setupok;	

private	void	comp_prep();
private	void	comp_fin();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	ch_t5();
private	void	test6_9();
private	void	test10();
private	void	test11();
private	void	test12();
private	void	test13();
private	void	test14();
private	void	ch_t12();
#if TEST_XNFS
private	void	ch_t13();
#else /* TEST_XNFS */
private	int	t6_9func();
private	void	t10rpt();
private	void	t11rpt();
private	int	t10_11io();
#endif /* TEST_XNFS */
#if TEST_XNFS
private	int	t14_func();
private	int	t14_chk_func();
#endif /* TEST_XNFS */

public	void	(*setfunc) () = comp_prep;
public	void	(*clnfunc) () = comp_fin;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6_9, 6,
	 test6_9, 7,
	 test6_9, 8,
	 test6_9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	NULL, 0
};

private void
comp_prep()
{
	int		fd;
	int		i;
	int		bad = 0;
	static char	msg[150];
	int	pathok = 0;

	DBUG_ENTER("comp_prep");

	if ((fd = creat(t_file, MODEANY)) == SYSERROR)
	{
		bad = 1;
		(void) sprintf(msg, "creat(%s, %#o) failed in preperations - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
	}
	else
	{
		PATH_TRACE;
		if (write(fd, t_string, (unsigned) strlen(t_string)) != strlen(t_string))
		{
			bad = 1;
			(void) sprintf(msg, "write() failed in preperations - errno %d(%s)",
				errno, errname(errno));
		}
		else if (freopen(t_file, "r", stdin) == (FILE *) NULL)
		{
			bad = 1;
			(void) sprintf(msg, "freopen(%s, \"r\", stdin) failed in preperations - errno %d(%s)",
				t_file, errno, errname(errno));
		}
		else
		{
			PATH_TRACE;
			t_fp = stdin;
		}
		(void) close(fd);
	}
	if (bad)
		for (i = 1 ; i <= NO_TESTS ; i++)
			tet_delete(i, msg);
	if (pathok == 2)
		setupok = 0;
	else
		setupok = -1;

	DBUG_VOID_RETURN;
}

private	void
comp_fin()
{
	(void) unlink(t_file);
}

private	void
test1()
{
	char		*rval;
	int	pathok = setupok;

	DBUG_ENTER("test1");
	errno = NOERROR;
	rewind(stdin);
	t_buf[0] = NUL;
	rval = gets(t_buf);
	t_buf[sizeof(t_buf) - 1] = NUL;
	if (rval != t_buf || strcmp(t1_string, t_buf))
	{
		xx_rpt(FAILURE);
		if (rval != t_buf)
			in_rpt("gets() did not return it's argument");
		if (strncmp(t_string, t_buf, (size_t)16))
		{
			in_rpt("gets() did not return correct string");
			in_rpt("expected >%s<, got >%s<",
				t1_string, t_buf);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}


private void
test2()
{
	char		*rval;
	int	pathok = setupok;

	DBUG_ENTER("test2");
	errno = NOERROR;
	(void) fseek(stdin, (long) T2_OFF, SEEK_SET);
	t_buf[0] = NUL;
	rval = gets(t_buf);
	t_buf[sizeof(t_buf) - 1] = NUL;
	if (rval != t_buf || strcmp(t2_string, t_buf))
	{
		xx_rpt(FAILURE);
		in_rpt("when no newline in stream before EOF:");
		if (rval != t_buf)
			in_rpt("gets() did not return it's argument");
		if (strncmp(t_string, t_buf, (size_t)26))
		{
			in_rpt("gets() did not return correct string when EOF reached before a newline");
			in_rpt("expected >%s<, got >%s<",
				t2_string, t_buf);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}


private void
test3()
{
	char		*rval;
	int	pathok = setupok;

	DBUG_ENTER("test3");
	errno = NOERROR;
	(void) fseek(stdin, 0L, SEEK_END);
	t_buf[0] = 'a';
	t_buf[1] = NUL;
	rval = gets(t_buf);
	t_buf[sizeof(t_buf) - 1] = NUL;
	if (rval != (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("when at end of file, gets() did not return NULL");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}


private void
test4()
{
	char		*rval;
	int	pathok = setupok;

	DBUG_ENTER("test4");
	errno = NOERROR;
	(void) fseek(stdin, (long) T4_OFF, SEEK_SET);
	t_buf[0] = NUL;
	rval = gets(t_buf);
	t_buf[sizeof(t_buf) - 1] = NUL;
	if (rval != t_buf || t_buf[0] != NUL)
	{
		xx_rpt(FAILURE);
		in_rpt("when gets() was called with next character in stream a new line:");
		if (rval != t_buf)
			in_rpt("gets() did not return it's argument");
		if (t_buf[0] != NUL)
			in_rpt("gets() did not return a zero length string");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}


private void
test5()
{
	DBUG_ENTER("test5");

	rewind(stdin);

	(void) cppair(ch_t5, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	char	*res;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t5");

#if TEST_XNFS
	
	if (tet_sync(SP_T5_CREAT_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREAT_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (freopen(t5_file, "r", stdin) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
			t5_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */
		PATH_TRACE;
		PATH_TRACE;
#endif /* TEST_XNFS */

	globok = 0;
	savtimes(t5_file);
	PATH_FUNC_TRACE;

        if (gets(t_buf) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("gets() failed on file %s, errno %d(%s)",
			t5_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ((res = chktimes(t5_file, NEWATIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(res);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(5);
	}

#if TEST_XNFS
	(void) tet_sync(SP_T5_UNLINK_START, snames, SYNC_TIMEOUT);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test6_9()
{
	DBUG_ENTER("test6_9");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

/* Lowest test number for which following function is used */
#define TBASE	6

private	void
test6_9()
{
	int	experr, err, ret, ferr_ret;
	static int errnums[] = { EAGAIN, -EAGAIN, EBADF, EINTR, };
			      /* test6   test7    test8  test9  */
	int	pathok = setupok;

	DBUG_ENTER("test6_9");
	DBUG_PRINTF("test6_9","tet_thistest: %d",tet_thistest);

	/* stdin must be a "virgin" stream */
	if (freopen(t_file, "r", stdin) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	experr = errnums[tet_thistest - TBASE];

	globok =0;
	ret = do_ferr(experr, 'r', STDIN_FILENO, t6_9func, &ferr_ret, &err);
	PATH_FUNC_TRACE;
	err = errno;
	if (experr < 0)
		experr = -experr;

	if (tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!ferr_ret || err != experr || !ret)
	{
		xx_rpt(FAILURE);
		if (!ferr_ret)
			in_rpt("gets() failed to set error indicator when testing for errno %d(%s)",
				experr, errname(experr));
		if (!ret)
			in_rpt("gets() did not return NULL when testing for errno %d(%s)",
				experr, errname(experr));
		else if (err != experr)
			in_rpt("gets() set errno incorrectly: expected %d(%s), actual %d(%s)",
				experr, errname(experr), err, errname(err));
	}
	else
		PATH_XS_RPT(3);

	if((fstat(fileno(stdin),&ensure_buf)==SYSERROR) && (freopen(t_file,"r",stdin) == (FILE *)NULL))
	{
		int i;
		static char rbuf[256];
		(void) sprintf(rbuf,"deletion reason: failed to ensure stream stdin was open after test %d", tet_thistest);
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i,rbuf);
		}
	}
	DBUG_VOID_RETURN;
}

/* ARGSUSED */

private int
t6_9func(fp, nbyte)
FILE *fp;
int nbyte;
{
	int ret;
	int	pathok = 0;

	DBUG_ENTER("t6_9func");

	ret = (gets(t_buf) == NULL);

	PATH_TRACE;
	PATH_FUNC_RPT(1);

	DBUG_RETURN(ret);
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test10()
{
	DBUG_ENTER("test10");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test10()
{
	int err;
	int	pathok = setupok;

	DBUG_ENTER("test10");

	if (freopen(t_file, "r", stdin) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sigstat = "ignored";
	do_feio("ri", STDIN_FILENO, t10_11io, t10rpt);

	if (freopen(t_file, "r", stdin) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sigstat = "blocked";
	do_feio("rb", STDIN_FILENO, t10_11io, t10rpt);

	DBUG_VOID_RETURN;
}

private void
t10rpt(ret, err)
int ret, err;
{
	int	pathok = setupok;

	DBUG_ENTER("t10rpt");

	PATH_FUNC_TRACE;
	if (!ret || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("gets() from background with SIGTTIN %s did not give correct results",
				sigstat);
		if (!ret)
			in_rpt("expected return: NULL, actual: non-NULL");
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test11()
{
	DBUG_ENTER("test11");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test11()
{
	int err;
	int	pathok = setupok;

	DBUG_ENTER("test11");

	if (freopen(t_file, "r", stdin) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	do_feio("ro", STDIN_FILENO, t10_11io, t11rpt);

	DBUG_VOID_RETURN;
}

private void
t11rpt(ret, err)
int ret, err;
{
	int	pathok = setupok;

	DBUG_ENTER("t11rpt");

	PATH_FUNC_TRACE;
	if (!ret || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("gets() from orphaned background process did not give correct results");
		if (!ret)
			in_rpt("expected return: NULL, actual: non-NULL");
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if !TEST_XNFS

/* ARGSUSED */

private int
t10_11io(fp)
FILE *fp;
{
	int ret;

	DBUG_ENTER("t10_11io");

	ret = (gets(t_buf) == NULL);

	if (ret)
		globok = 1;
	else
		globok = 0;

	DBUG_RETURN(ret);
}

#endif /* TEST_XNFS */

private void
test12()
{
	DBUG_ENTER("test12");

	(void) cppair(ch_t12, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t12_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	char	*rofs;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t12");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_ROFS is not defined");
		DBUG_VOID_RETURN;
	}

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = t12_dir;
	args[3] = t12_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog_su, err, errname(err));

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test13()
{
	DBUG_ENTER("test13");

	(void) tet_fork(ch_t13, NULLFN, WAITTIME, OK);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t13()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t13");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t13_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test14()
{
	DBUG_ENTER("test14");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test14()
{
	DBUG_ENTER("test14");

	t_estale_rd_c(t14_file, 1, t14_func, t14_chk_func, SP_T14_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t14_func()
{
	DBUG_ENTER("t14_func");

	DBUG_RETURN(gets(t_buf) == NULL ? -1 : 0);
}

private int
t14_chk_func(fp)
FILE	*fp;
{
	char	*rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("t14_chk_func");

	errno = NOERROR;

	while((rval = gets(t_buf)) != (char *)NULL)
		;

	err = errno;

	if(rval != NULL || ferror(fp) == 0 || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(rval != NULL)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value NULL");
			in_rpt("Actual   return value \"%s\"", rval);
		}
		else
			PATH_TRACE;

		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;
		
		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
