/*
*      SCCS:  @(#)  ANSI.os/misc/perror/perror.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/misc/perror - T.perror Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.perror

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the perror() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Thu Oct 30 17:22:41 GMT 1986
MODIFICATIONS:	Bob Campbell, Aug 1988: adapted for NLS (XPG3)

		Geoff Clare, 20 Nov 1989
			Improved thoroughness

		David Sawyer
		Tue Jul 17 09:57:03 BST 1990
		Added resillience code.

		Stuart Boutell, 31 Oct 1990
			Added Ron Nelson's code for test 3

		Geoff Clare, 6 Mar 1991
			Removed declarations of geteuid() and getegid().

		Geoff Clare, 29 May 1992
			Don't use literal language names.

                John-Paul, 27 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		Geoff Clare, 19 Aug 1992
			Rewrite test 3.

		John-Paul, 2 November 1992
			XNFS enhancements.
		
		Geoff Clare, 5 Feb 1993
			Report errno symbol as well as value in chkuniq().

		Geoff Clare, 16 June 1993
			Remove ENOTBLK from errno list in XPG4 mode.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<string.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<nlstest.h>
#include	<locale.h>
#include	<chktimes.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef perror
#endif


#define 	NO_TESTS	3	/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)
#define		FILE1		"./perror_t.1"
#define		FILE2		"./perror_t.2"
#define		MAXCHARS	256
#define		MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)


extern	char	*errname();
extern  char	*psl_to_asc();
extern  char	*asc_to_psl();
extern	char	*newstr();

int	no_tests = NO_TESTS;

char	*terr_file =	"./perror_t.3";

private struct {
	int num;
	char *name;
	char *msgp;
} errtab[] = {
	NOERROR,	"No Error",	NULL,
	E2BIG,		"E2BIG",	NULL,
	EACCES,		"EACCES",	NULL,
	EAGAIN,		"EAGAIN",	NULL,
	EBADF,		"EBADF",	NULL,
	EBUSY,		"EBUSY",	NULL,
	ECHILD,		"ECHILD",	NULL,
	EDEADLK,	"EDEADLK",	NULL,
	EDOM,		"EDOM",		NULL,
	EEXIST,		"EEXIST",	NULL,
	EFAULT,		"EFAULT",	NULL,
	EFBIG,		"EFBIG",	NULL,
	EINTR,		"EINTR",	NULL,
	EINVAL,		"EINVAL",	NULL,
	EIO,		"EIO",		NULL,
	EISDIR,		"EISDIR",	NULL,
	EMFILE,		"EMFILE",	NULL,
	EMLINK,		"EMLINK",	NULL,
	ENAMETOOLONG,	"ENAMETOOLONG",	NULL,
	ENFILE,		"ENFILE",	NULL,
	ENODEV,		"ENODEV",	NULL,
	ENOENT,		"ENOENT",	NULL,
	ENOEXEC,	"ENOEXEC",	NULL,
	ENOLCK,		"ENOLCK",	NULL,
	ENOMEM,		"ENOMEM",	NULL,
	ENOSPC,		"ENOSPC",	NULL,
	ENOSYS,		"ENOSYS",	NULL,
	ENOTDIR,	"ENOTDIR",	NULL,
	ENOTEMPTY,	"ENOTEMPTY",	NULL,
	ENOTTY,		"ENOTTY",	NULL,
	ENXIO,		"ENXIO",	NULL,
	EPERM,		"EPERM",	NULL,
	EPIPE,		"EPIPE",	NULL,
	ERANGE,		"ERANGE",	NULL,
	EROFS,		"EROFS",	NULL,
	ESPIPE,		"ESPIPE",	NULL,
	ESRCH,		"ESRCH",	NULL,
	EXDEV,		"EXDEV",	NULL,
#ifdef _XOPEN_SOURCE
	EIDRM,		"EIDRM",	NULL,
	ENOMSG,		"ENOMSG",	NULL,
#if TEST_XPG == 3
	ENOTBLK,	"ENOTBLK",	NULL,
#endif
	ETXTBSY,	"ETXTBSY",	NULL,
#endif
};
private int	numerr = sizeof(errtab)/sizeof(errtab[0]);
private int	testfail;

private void	test1();
private void    ch_t1();
private void    chkuniq();
private	void	test2();
private	void	cleanup();
private	void	test3();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cleanup;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private void
cleanup()
{
	(void) unlink(terr_file);
}

private void
test1()
{
	DBUG_ENTER("test1");

#ifdef TESTBED
	errtab[2].num = errtab[3].num;
#endif

	(void)   cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	rval, i;
	static	char	testmsg[] = "test message";
	char	expected[20];
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	testfail = 0;

	(void) close(2);
	if ( (rval = creat(FILE1, (mode_t)(S_IRUSR | S_IWUSR))) != 2 )
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat returned file descriptor %d instead of 2 which was previously closed", rval);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < numerr; i++)
	{
		if (i==0)
			PATH_TRACE;
		errno = errtab[i].num;
		perror(testmsg);
	}

	errno = NOERROR;
	if (close(2) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: close failed with errno %d (%s)",
			errno, errname(errno));

		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sprintf(expected, "%s: ", testmsg);

	globok = 0;
	chkuniq(FILE1, expected, VALCLANG);
	PATH_FUNC_TRACE;
	
	(void) unlink(FILE1);

#if !TEST_XNFS
	if (setlocale(LC_ALL, INFO1LANG) == NULL)
	{
		xx_rpt (DELETION);
		in_rpt ("Pseudo language %s is not available ",INFO1LANG);
		in_rpt ("Check installation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) asc_to_psl(testmsg, VALINFO1LANG);

	(void) close(2);
	if ( (rval = creat(FILE1, (mode_t)(S_IRUSR | S_IWUSR))) != 2 )
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat returned file descriptor %d instead of 2 which was previously closed", rval);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < numerr; i++)
	{
		if (i==0)
			PATH_TRACE;
		errno = errtab[i].num;
		perror(testmsg);
	}

	errno = NOERROR;
	if (close(2) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: close failed with errno %d (%s)",
					errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	(void) sprintf(expected, "%s: ", testmsg);
	
	globok = 0;
	chkuniq(FILE1, expected, VALINFO1LANG);
	PATH_FUNC_TRACE;

#else /* TEST_XNFS */
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif /* TEST_XNFS */

	if (!testfail)
		PATH_XS_RPT(9);

	(void) unlink(FILE1);
	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	rval;
	char	*res;
	int	pathok = 0;

	DBUG_ENTER("test2");

	(void) close(2);
	if ( (rval = creat(FILE2, (mode_t)(S_IRUSR | S_IWUSR))) != 2 )
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat returned file descriptor %d instead of 2 which was previously closed", rval);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) unlink(FILE2);

	globok = 0;
	fsavtimes(fileno(stderr));
	PATH_FUNC_TRACE;

	errno = EINTR;
        perror("Error output for EINTR: ");

	globok = 0;
	if ((res = fchktimes(fileno(stderr), NEWCTIME|NEWMTIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(res);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

private void
chkuniq(file, expstr, langnum)
char *file, *expstr;
int langnum;
{
	FILE 	*fp;
	int 	i, j, lastfail;
	char 	buf[MAXCHARS];
	int	pathok = 0;

	DBUG_ENTER("chkuniq");

	lastfail = testfail;

	errno = NOERROR;
	if ((fp = fopen(file, "r")) == NULL) 
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen failed with errno %d (%s)",
					errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	for (i = 0; i < numerr; i++)
	{
		if (fgets(buf, MAXCHARS, fp) == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: fgets() failed with errno %d (%s)",
					errno, errname(errno));
			(void) fclose(fp);	
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		j = strlen(buf);
		if (buf[j-1] == '\n')
			buf[j-1] = '\0';
#ifdef TESTBED
buf[0]++;
#endif
		if (strncmp(buf, expstr, strlen(expstr)) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("perror() gave incorrect output with errno %d",
				errtab[i].num);
			in_rpt("expected start: \"%s\"",
				psl_to_asc(expstr, langnum));
			in_rpt("actual output:  \"%s\"",
				psl_to_asc(buf, langnum));
		}
		else
			PATH_TRACE;

		if (errtab[i].msgp != NULL)
			free((void *)errtab[i].msgp);
		if ((errtab[i].msgp = newstr(buf)) == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: malloc() failed");
			(void) fclose(fp);	
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		for (j = 0; j < i; j++)
		{
			if (j==0)
				PATH_TRACE;
			if (strcmp(errtab[i].msgp, errtab[j].msgp) == 0)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("perror() with errno %d (%s) gave same output as errno %d (%s)",
					errtab[i].num, errtab[i].name,
					errtab[j].num, errtab[j].name);
				break;
			}
		}
	}

	if (testfail > lastfail)
		in_rpt("language setting for above tests was %s",
			(langnum == VALINFO1LANG) ? INFO1LANG : "C locale");

	PATH_FUNC_RPT(1+3*numerr+numerr-1);

	(void) fclose(fp);	
	DBUG_VOID_RETURN;
}




private void
test3()
{
	char	buf1[MAXCHARS];
	char	buf2[MAXCHARS];
	char	*cp1, *cp2;
	int	len1, len2, fd, err;

	int	pathok = 0;

	DBUG_ENTER("test3");

	(void) close(2);
	if (crfile(terr_file,(mode_t)MODEANY,(int)geteuid(),(int)getegid())!=2)
	{
		xx_rpt(DELETION);
		in_rpt("crfile() did not return 2 after close(2)");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	fd = open(terr_file, O_RDONLY);
	if (fd == -1)
	{
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed", terr_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* first obtain the output from perror(string) */

	errno = ENOENT;
	perror("string");

	len1 = read(fd, buf1, sizeof(buf1));
	err = errno;
	if (len1 <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("read(%d, buf1, sizeof(buf1)) returned %d",fd,len1);
		if (len1 < 0)
			in_rpt("errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
	{
		buf1[len1] = '\0';
		PATH_TRACE;
	}

	/* now obtain the output from perror(NULL) */

	errno = ENOENT;
	perror((char *)NULL);

	len2 = read(fd, buf2, sizeof(buf2));
	err = errno;
	if (len2 <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("read(%d, buf2, sizeof(buf2)) returned %d",fd,len2);
		if (len2 < 0)
			in_rpt("errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
	{
		buf2[len2] = '\0';
		PATH_TRACE;
	}

	(void) close(fd);

	/* check that the message after the "string: " from the output
	   of perror(string) is the same as the end of the output from
	   perror(NULL) */

	cp1 = strchr(buf1, ' ');
	if (cp1 == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("output from perror(string) did not contain a space");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	cp1++;

	cp2 = &buf2[len2] - (&buf1[len1] - cp1); 

	if (strcmp(cp1, cp2) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("perror(NULL) output: %s", buf2);
		in_rpt("expected output to end with: %s", cp1);
		DBUG_VOID_RETURN;
	}
	else
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}
