/*
*      SCCS:  @(#)  ANSI.os/streamio/printf_X/printf_X.c	Rel 4.4.3	    (07/20/98)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/printf_X - T.printf_X Rel 4.4.3 (07/20/98)";
char	*copyright[] =
{
	"(C) Copyright 1991 X/Open Company Limited", "All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.printf_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the printf() subroutine, X/Open extensions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	25 May 1989
MODIFICATIONS:
		Steve Ratcliffe, 30 Jan 1990
			Add long double test.

		Steve Ratcliffe, 22 Feb 1990
			Cope with varargs.h / stdarg.h

		Geoff Clare, 11 Apr 1990
			Replace use of regexp with simpler match() routine.

		David Sawyer
		Wed Jul 18 16:18:31 BST 1990
		Added resillience code.

		Geoff Clare, 20 Aug 1990
			Drain pipe at start of each test (in ch_setup()).

		Neil Moses, 14 Nov 1991
			Changes for VSX 4.2
			Added new wide character conversion types C and S
			New tests 20, 21 and 38

		Neil Moses, 3 Jan 1992
			Save and restore locale in setup function

		John-Paul, 27 October 1992
			XNFS enhancements.

		Geoff Clare, 15 April 1993
			Move pipe draining from child to parent to comply
			with POSIX `handle' rules.

		Geoff Clare, April 1997
			Added tests for snprintf() in UNIX98 mode.

		Geoff Clare, 16 April 1998
			Reinitialise output stream in newlocale().

************************************************************************/

#include	<std.h>
#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<string.h>
#include	<locale.h>
#include	<local.h>
#include 	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fprintf
#undef printf
#undef sprintf
#undef snprintf
#endif



#define 	NO_TESTS	57	/* number of tests in testset */

extern	char	*vg_ident;
extern	char	*psl_to_asc();
extern	char	*errname();

int	no_tests = NO_TESTS;

#if TEST_XNFS
public	char	*tfile_xnfs = "./printf_X-xnfs";
#endif /* TEST_XNFS */
public	int	interface;
public	FILE	*f_rd;
public	FILE	*f_wr;
public	char	*function;
public	char	buf[BUFFERSIZE];

public	int	tretval;
public	int	testfail;
public	int	setupok = -1;
public	char	*testcomment;
#if (TEST_XPG > 3) && !TEST_XNFS && !TEST_XNFS
public	char	mbstr[BUFFERSIZE];
public	char	wexpstr[BUFFERSIZE];
public	char	wvalstr[BUFFERSIZE];
public	char	*locale;
public	wchar_t	*wcstr;
#endif
public	int	pipdes[2];

private void	prepare_tests();
private void	clean_tests();
private void	twrap();

extern void	test1();
#if TEST_XNFS
extern void	notinuse();
#else /* TEST_XNFS */
extern void	test2();
extern void	test3();
extern void	test4();
extern void	test5();
extern void	test6();
extern void	test7();
extern void	test8();
extern void	test9();
extern void	test10();
extern void	test11();
extern void	test12();
extern void	test13();
extern void	test14();
extern void	test15();
extern void	test16();
extern void	test17();
extern void	test18();
extern void	test19();
extern void	test20();
extern void	test21();
extern void	test22();
extern void	test23();
extern void	test24();
extern void	test25();
extern void	test26();
extern void	test27();
extern void	test28();
extern void	test29();
extern void	test30();
extern void	test31();
extern void	test32();
extern void	test33();
extern void	test34();
extern void	test35();
extern void	test36();
extern void	test37();
extern void	test38();
extern void	test39();
extern void	test40();
extern void	test41();
extern void	test42();
extern void	test43();
extern void	test44();
extern void	test45();
extern void	test46();
extern void	test47();
extern void	test48();
extern void	test49();
extern void	test50();
extern void	test51();
extern void	test52();
extern void	test53();
extern void	test54();
extern void	test55();
extern void	test56();
extern void	test57();
#endif /* TEST_XNFS */


public void	checkval();
public void	checkpat();
public void	chk_narg();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;

public	struct tet_testlist tet_testlist[] = {
	twrap, 1,
	twrap, 2,
	twrap, 3,
	twrap, 4,
	twrap, 5,
	twrap, 6,
	twrap, 7,
	twrap, 8,
	twrap, 9,
	twrap, 10,
	twrap, 11,
	twrap, 12,
	twrap, 13,
	twrap, 14,
	twrap, 15,
	twrap, 16,
	twrap, 17,
	twrap, 18,
	twrap, 19,
	twrap, 20,
	twrap, 21,
	twrap, 22,
	twrap, 23,
	twrap, 24,
	twrap, 25,
	twrap, 26,
	twrap, 27,
	twrap, 28,
	twrap, 29,
	twrap, 30,
	twrap, 31,
	twrap, 32,
	twrap, 33,
	twrap, 34,
	twrap, 35,
	twrap, 36,
	twrap, 37,
	twrap, 38,
	twrap, 39,
	twrap, 40,
	twrap, 41,
	twrap, 42,
	twrap, 43,
	twrap, 44,
	twrap, 45,
	twrap, 46,
	twrap, 47,
	twrap, 48,
	twrap, 49,
	twrap, 50,
	twrap, 51,
	twrap, 52,
	twrap, 53,
	twrap, 54,
	twrap, 55,
	twrap, 56,
	twrap, 57,
	NULL, 0
};

#if TEST_XNFS

private	void	(*realfuncs[]) () = {
	test1,    notinuse, notinuse, notinuse, notinuse,
	notinuse, notinuse, notinuse, notinuse, notinuse,
	notinuse, notinuse, notinuse, notinuse, notinuse,
	notinuse, notinuse, notinuse, notinuse, notinuse,
	notinuse, notinuse, notinuse, notinuse, notinuse,
	notinuse, notinuse, notinuse, notinuse, notinuse,
	notinuse, notinuse, notinuse, notinuse, notinuse,
	notinuse, notinuse, notinuse, notinuse, notinuse,
	notinuse, notinuse, notinuse, notinuse, notinuse,
	notinuse, notinuse, notinuse, notinuse, notinuse,
	notinuse, notinuse, notinuse, notinuse, notinuse,
	notinuse, notinuse,
	NULL
};

#else /* TEST_XNFS */

private	void	(*realfuncs[]) () = {
	test1,  test2,  test3,  test4,  test5,
	test6,  test7,  test8,  test9,  test10,
	test11, test12, test13, test14, test15,
	test16, test17, test18, test19, test20,
	test21, test22, test23, test24, test25,
	test26, test27, test28, test29, test30,
	test31, test32, test33, test34, test35,
	test36, test37, test38, test39, test40,
	test41, test42, test43, test44, test45,
	test46, test47, test48, test49, test50,
	test51, test52, test53, test54, test55,
	test56, test57,
	NULL
};

#endif /* TEST_XNFS */

private void
twrap()
{
	DBUG_ENTER("twrap");

	/* Drain the pipe in case of leftovers from last test */
	while (fread((void *)buf, (size_t)1, sizeof(buf), f_rd) > 0)
		;

	/* Call the real test function */
	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private	void
prepare_tests()
{

#if (TEST_XPG > 3) && !TEST_XNFS
	char *ptr;
	char current[50];
	static int wtests[] = {	20,21,27,28,29,31,32,33,34,35,36,37,38,39,40,41,
				42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,0};
#endif
	char	*pname;
	static	char	msg[BUFFERSIZE];
	int	i, err;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	if ( (pname = strrchr(vg_ident, '/')) == (char *) NULL )
		pname = vg_ident+2;
	else
		pname+=3;

	if ( strcmp(pname, "printf_X") == 0 )
	{
		interface = I_PR;
		function = "printf(";
	}
	else if ( strcmp(pname, "fprintf_X") == 0 )
	{
		interface = I_FP;
		function = "fprintf(f_wr,";
	}
#if !TEST_XNFS
	else if ( strcmp(pname, "sprintf_X") == 0 )
	{
		interface = I_SP;
		function = "sprintf(buf,";
	}
#  if TEST_XPG >= 500
	else if ( strcmp(pname, "snprintf_X") == 0 )
	{
		interface = I_SNP;
		function = "snprintf(buf, size,";
	}
#  endif /* TEST_XPG */
#endif /* TEST_XNFS */
	else
	{
		for (i=1; i<=NO_TESTS; i++)
			tet_delete(i, "bad ident parameter");
	}
	
#if TEST_XNFS

	errno = NOERROR;
	pipdes[P_WR] = open(tfile_xnfs, O_RDWR|O_CREAT, (mode_t)(S_IRWXU|S_IRWXG|S_IRWXO));
	if ( pipdes[P_WR] == SYSERROR )
	{
		err = errno;
		(void) sprintf(msg, "open() failed in preperations - errno %d(%s)", err, errname(err));
		for (i=1; i<=NO_TESTS; i++)
			tet_delete(i, msg);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	pipdes[P_RD] = open(tfile_xnfs, O_RDONLY, (mode_t)(S_IRWXU|S_IRWXG|S_IRWXO));
	if ( pipdes[P_RD] == SYSERROR )
	{
		err = errno;
		(void) sprintf(msg, "open() failed in preperations - errno %d(%s)", err, errname(err));
		for (i=1; i<=NO_TESTS; i++)
			tet_delete(i, msg);
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */
	
	PATH_TRACE;
	errno = NOERROR;
	if ( pipe(pipdes) == SYSERROR )
	{
		err = errno;
		(void) sprintf(msg, "pipe() failed in preperations - errno %d(%s)", err, errname(err));
		for (i=1; i<=NO_TESTS; i++)
			tet_delete(i, msg);
	}
	else
		PATH_TRACE;

#endif /* TEST_XNFS */

	if (fcntl(pipdes[P_RD], F_SETFL, O_NONBLOCK) == SYSERROR)
	{
		err = errno;
		(void) sprintf(msg, "fcntl() failed in preperations - errno %d(%s)", err, errname(err));
		for (i=1; i<=NO_TESTS; i++)
			tet_delete(i, msg);
	}
	else
		PATH_TRACE;

	if ((f_rd = fdopen(pipdes[P_RD], "r")) == (FILE *) NULL)
	{
		err = errno;
		(void) sprintf(msg, "fdopen() failed in preperations - errno %d(%s)", err, errname(err));
		for (i=1; i<=NO_TESTS; i++)
			tet_delete(i, msg);
	}
	else
		PATH_TRACE;

#if (TEST_XPG > 3) && !TEST_XNFS
	if(wcharstr(&locale, &wcstr) != 0)
	{
		err = errno;
		(void) sprintf(msg, "wcharstr() failed in preperations - errno %d(%s)", err, errname(err));
		for (i=1; i<=NO_TESTS; i++)
			tet_delete(i, msg);
	}
	else
		PATH_TRACE;

	if((ptr = setlocale(LC_CTYPE, NULL)) == (char *)NULL)
	{
		err = errno;
		(void) sprintf(msg, "setlocale() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;

	(void) strcpy(current, ptr);

	if(setlocale(LC_CTYPE, locale) == (char *)NULL)
	{
		err = errno;
		(void) sprintf(msg, "setlocale() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;

	if(wcstombs(mbstr, wcstr, BUFFERSIZE) == (size_t)-1)
	{
		err = errno;
		(void) sprintf(msg, "wcstombs() failed in preperations - errno %d(%s)", err, errname(err));
		for (i=1; i<=NO_TESTS; i++)
			tet_delete(i, msg);
	}
	else
		PATH_TRACE;

	/* Restore the locale */
	if(setlocale(LC_CTYPE, current) == NULL)
	{
		err = errno;
		(void) sprintf(msg, "setlocale() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	PATH_SETUP_RPT(9);

	DBUG_VOID_RETURN;
}

public void
ch_setup()
{
	DBUG_ENTER("ch_setup");

	if ( interface == I_PR )
	{
		if ( close(1) == SYSERROR )
			DBUG_EXIT(A_CLOSEFAIL);

		if ( dup(pipdes[P_WR]) != 1 )
			DBUG_EXIT(A_DUPFAIL);

		(void) close(pipdes[P_WR]);

		if ( close(pipdes[P_RD]) == SYSERROR )
			DBUG_EXIT(A_CLOSEFAIL);

		/* stdout now goes to pipe */
		f_wr = stdout;
	}
	else
	{
		if ( close(pipdes[P_RD]) == SYSERROR )
			DBUG_EXIT(A_CLOSEFAIL);

		if ((f_wr = fdopen(pipdes[P_WR], "w")) == (FILE *) NULL)
			DBUG_EXIT(E_FDOPEN);
	}

	DBUG_VOID_RETURN;
}

#if (TEST_XPG > 3) && !TEST_XNFS
public void
newlocale(cat, loc)
int 	cat;
char 	*loc;
{
	DBUG_ENTER("newlocale");

	if (setlocale(cat, loc) == (char *) NULL)
	{
		in_rpt("Pseudo Language %s is not available", loc);
		in_rpt("Check installation of pseudo language");
		xx_rpt(DELETION);
		DBUG_EXIT(E_LOCFAIL);
	}

#if TEST_XPG >= 500
	/*
	 * When newlocale() is used in the child after some output has
	 * already been sent to the output stream (f_wr), the output
	 * stream must be reopened in order to ensure conversions are
	 * done using the new locale.
	 */

	if (f_wr == NULL)
	{
		/* Not in child */
		DBUG_VOID_RETURN;
	}

	(void) fflush(f_wr);

	if (f_wr == stdout)
	{
		int tmpfd = dup(1);

		if (tmpfd < 0 ||
		    freopen("/dev/null", "w", stdout) == NULL ||
		    dup2(tmpfd, 1) < 0)
		{
			in_rpt("could not reopen stdout in newlocale() - errno %d (%s)",
				errno, errname(errno));
			xx_rpt(DELETION);
			DBUG_EXIT(E_LOCFAIL);
		}

		(void) close(tmpfd);
	}
	else
	{
		if ((f_wr = fdopen(pipdes[P_WR], "w")) == NULL)
		{
			in_rpt("fdopen() failed in newlocale() - errno %d (%s)",
				errno, errname(errno));
			xx_rpt(DELETION);
			DBUG_EXIT(E_LOCFAIL);
		}
	}
#endif

	DBUG_VOID_RETURN;
}
#endif

public void
checkval(convstr, valstr, exp1str)
char *convstr, *valstr;
char *exp1str;
{
	int	pathok = 0;

	DBUG_ENTER("checkval");

	GETBUF
	else
		PATH_TRACE;
	if (strncmp(buf, "%..xx", (size_t)5) == 0)
	{
		testfail++;
		in_rpt("%s", buf);
		while (fgets(buf, sizeof(buf), f_rd) != (char *) NULL)
			in_rpt("%s", buf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( strcmp(buf, exp1str) == 0 )
	{
		PATH_FUNC_RPT(2);
		DBUG_VOID_RETURN;
	}

	if (!testfail++)
		xx_rpt(FAILURE);

	in_rpt("%s \"%s\", %s) failed", function, convstr, valstr);
	if (testcomment != (char *) NULL)
		in_rpt("%s", testcomment);
	in_rpt("Observed output: \"%s\"", psl_to_asc(buf, 0));
	in_rpt("Expected output: \"%s\"", psl_to_asc(exp1str, 0));

	DBUG_VOID_RETURN;
}

/*
 * Match a pattern to a string, using metacharacters:
 *
 *     .   match any character
 *     *   repeat preceding NORMAL character 0 or more times
 *     []  match any of the characters in the brackets
 *     \   remove special meaning from following metacharacter
 *     ^   ignored at front of pattern
 *
 * Returns non-zero for successful match.
 */

private int
match(pat, str)
char *pat, *str;
{
	int	found;
	char	last;

	if (*pat == '^')
		pat++;
	
	for (last = '\0'; *pat != '\0' && *str != '\0'; last = *pat++)
	{
		if (last != '\\')
		{
			switch (*pat)
			{
			case '\\' :
				continue;

			case '.' :
				str++;
				continue;
			
			case '*' :
				/* treat "X*XXX" as "XXXX*" */
				while (*(pat+1) == last && *str == last)
					pat++, str++;

				while (*str == last)
					str++;
				continue;
			
			case '[' :
				found = 0;
				for (pat++; *pat != ']' && *pat != '\0'; pat++)
				{
					if (*str == *pat)
						found = 1;
				}
				if (found == 0)
					return 0;
				str++;
				continue;
			}
		}

		if (*(pat+1) != '*')
		{
			if (*str != *pat)
				return 0;
			str++;
		}
	}

	return (*str == *pat);
}

#if !TEST_XNFS

public void
checkpat(convstr, valstr, exppat, explen)
char *convstr, *valstr;
char *exppat;
int explen;
{
	int	matchok, len;
	int	pathok = 0;

	DBUG_ENTER("checkpat");

	GETBUF
	else
		PATH_TRACE;
	if (strncmp(buf, "%..xx", (size_t)5) == 0)
	{
		testfail++;
		in_rpt("%s", buf);
		while (fgets(buf, sizeof(buf), f_rd) != (char *) NULL)
			in_rpt("%s", buf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	matchok = match(exppat, buf);
	if (explen != 0)
		len = strlen(buf);
	else
		len = 0;

	if (matchok && len == explen)
	{
		PATH_FUNC_RPT(2);
		DBUG_VOID_RETURN;
	}

	if (!testfail++)
		xx_rpt(FAILURE);

	in_rpt("%s \"%s\", %s) failed", function, convstr, valstr);
	if (testcomment != (char *) NULL)
		in_rpt("%s", testcomment);
	in_rpt("Observed output:  \"%s\"", psl_to_asc(buf, 0));
	if (!matchok)
		in_rpt("Expected pattern: \"%s\"", psl_to_asc(exppat, 0));
	if (len != explen)
		in_rpt("Expected length: %d, Actual length: %d", explen, len);

	DBUG_VOID_RETURN;
}

public void
chk_narg(convstr, valstr, expnum)
char *convstr;
char *valstr;
long expnum;
{
	long 	num;
	int	pathok = 0;

	DBUG_ENTER("chk_narg");

	GETBUF
	else
		PATH_TRACE;
	if ((num = atol(buf)) != expnum)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("%s \"%s\", %s) failed", function, convstr, valstr);
		in_rpt("%s", testcomment);
		in_rpt("%%n returned %ld in argument instead of %ld",
			num, expnum);
	}
	else
		PATH_TRACE;
	
	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
clean_tests()
{
#if TEST_XNFS
	(void) unlink(tfile_xnfs);
#endif /* TEST_XNFS */
}
