/*
 *      SCCS:  @(#)  ANSI.os/streamio/putc/enospc.c	Rel 4.4.1     (06/09/97) 
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1991 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#ifndef NO_MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/putc - enospc Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif
#endif  /* lint	*/

/***********************************************************************

NAME:	enospc.c

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()
		enospc()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char *argv[];

DESCRIPTION:
		Program to cause out of space errors for putc/putchar/fputc
		only.  Because putchar writes to stdout and the reporting
		functions do too then have to write the status of the test
		back to another process.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:
AUTHOR:		Steve Ratcliffe, UniSoft Ltd.
DATE CREATED:	28 Feb 1990
MODIFICATIONS:

		David Sawyer
		Fri Jul 13 17:21:55 BST 1990
		Split up main(), creating eno_rpt().
		Added resillience code to enospc() and eno_rpt().

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file
		
		Geoff Clare, 21 May 1992
			Make vprintf code conditional on ANSI_VPRINTF_FUNC.

		Dave Cater, 13 November 1992
			XNFS enhancements
		
		Geoff Clare, 2 Sept 1993
			Increase cppair() timeout.

************************************************************************/

#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <report.h>
#include <dbug.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#if	TEST_ANSI
#include <stdarg.h>
#else
#include <varargs.h>
#endif
#include <tet_api.h>
#include <xnfs.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef putc
#undef putchar
#undef fputc
#undef printf
#undef fprintf
#undef vprintf
#undef vfprintf
#undef puts
#undef fputs
#undef putw
#undef fwrite
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)

#ifndef NO_MAIN
public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

#endif

extern	char	**environ;

extern	char	*itoa();
extern	char	*errname();
extern	void	set_pname();
extern	void	valrpt();

private int 	testfail;
private int 	pdes[2];
private char	*mountdir;

void 	enospc();
#if ANSI_VPRINTF_FUNC
#if 	TEST_ANSI
int 	dovprintf(FILE *fp, char *fmt, ...);
#else
int 	dovprintf();
#endif
#endif

extern int 	setupnospc();
extern void	tidynospc();

/* The interface we are testing and defines for it */
private int 	interface;

#define	I_PUTCHAR	0
#define I_PUTC		1
#define I_FPUTC		2
#define I_PRINTF	3
#define I_FPRINTF	4
#define I_VPRINTF	5
#define I_VFPRINTF	6
#define I_PUTS		7
#define I_FPUTS		8
#define I_PUTW		9
#define I_FWRITE	10

#define PIPEREAD	0
#define PIPEWRITE	1

/*
 * The enospc process exits with OK if it has already printed out
 * the test result.  If it exits with NEEDPRINT then the parent
 * must read the test result from the pipe.
 */
#define NEEDPRINT 	2
#define PATH_OK         4 

#define	BUFLEN	512

#define MODEANY	(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define MAXTRYS 2000	/* Number of times we try to fill up system */

/*
 * A string used as data to write out
 */
#define STRPAT	"abcdefghijklmnopqrstuvwxyz"
#define PATLEN (sizeof(STRPAT)-1)

#define FGETSERROR()	xx_rpt(DELETION); \
		in_rpt("deletion reason: read from pipe failed");

#ifndef NO_MAIN

public	int
tet_main(argc, argv)
int	argc;
char	*argv[];
{
	set_pname( argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}

	if (strcmp(argv[2], "putchar") == 0)
		interface = I_PUTCHAR;
	else if (strcmp(argv[2], "putc") == 0)
		interface = I_PUTC;
	else if (strcmp(argv[2], "fputc") == 0)
		interface = I_FPUTC;
	else if (strcmp(argv[2], "printf") == 0)
		interface = I_PRINTF;
	else if (strcmp(argv[2], "fprintf") == 0)
		interface = I_FPRINTF;
#if ANSI_VPRINTF_FUNC
	else if (strcmp(argv[2], "vprintf") == 0)
		interface = I_VPRINTF;
	else if (strcmp(argv[2], "vfprintf") == 0)
		interface = I_VFPRINTF;
#endif
	else if (strcmp(argv[2], "puts") == 0)
		interface = I_PUTS;
	else if (strcmp(argv[2], "fputs") == 0)
		interface = I_FPUTS;
	else if (strcmp(argv[2], "putw") == 0)
		interface = I_PUTW;
	else if (strcmp(argv[2], "fwrite") == 0)
		interface = I_FWRITE;
	else {
		xx_rpt(DELETION);
		in_rpt("deletion reason: Bad interface");
		return(OK);
	}

	eno_rpt(argc, argv);

	return(OK);
}


public	int
eno_rpt(argc, argv)
int	argc;
char	*argv[];
{
	int 	ret;
	int 	res;
	char	buf[BUFSIZ];
	FILE	*fpread;
	int	pathok = 0;

	DBUG_ENTER("eno_rpt");

	if (argc >= 5)
		vg_dbug = argv[4];
	if (argc >= 4)
		vg_ident = argv[3];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if (pipe(pdes) == SYSERROR) {
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: pipe failed");
		DBUG_TET_EXIT(OK);
	}
	else
		PATH_TRACE;

	mountdir = argv[1];

	globok = 0;
	if (setupnospc(mountdir)) {
		testfail++;
		DBUG_TET_EXIT(OK);
	}
	else
		PATH_FUNC_TRACE;

	ret = cppair(enospc, NULLFN, (3*WAITTIME), NEEDPRINT | PATH_OK);
	if (ret & PATH_OK)
	{
		PATH_TRACE;
		ret = NEEDPRINT;
	}

	if (fcntl(pdes[PIPEREAD], F_SETFL, O_NONBLOCK) == SYSERROR) {
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl failed");
		DBUG_TET_EXIT(OK);
	}
	else
		PATH_TRACE;
	(void) close(pdes[PIPEWRITE]);

	if ((fpread = fdopen(pdes[PIPEREAD], "r")) == NULL) {
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fdopen failed");
		DBUG_TET_EXIT(OK);
	}
	else
		PATH_TRACE;

	if (ret == NEEDPRINT) {
	int 	n;

		DBUG_PRINTF("eno_rpt", "ret == NEEDPRINT");

		/* Get test result code */
		if (fgets(buf, sizeof(buf), fpread) == NULL) {
			FGETSERROR();
			DBUG_TET_EXIT(OK);
		}
		res = atoi(buf);
		if (res != 0) {

			/* Get an info message */
			if (fgets(buf, sizeof(buf), fpread) == NULL) {
				FGETSERROR();
				DBUG_TET_EXIT(OK);
			}
			n = strlen(buf);
			if (n > 0 && buf[n-1] == '\n')
				buf[n-1] = '\0';

			xx_rpt(res);
			in_rpt("%s", buf);

		} else {
			PATH_TRACE;	
			PATH_XS_RPT(6);
		}
	}

	/* Tidy up and umount file system */
	tidynospc(mountdir, 3);

	DBUG_TET_EXIT(OK);
	/*NOTREACHED*/
}
#endif /* !NO_MAIN */

/*
 * enospc() tries to run out of space.
 */
void
enospc()
{
	char	*name;
	int 	err;
	int 	i;
	int 	j;
	char	*outstr;
	FILE	*fp;
	FILE	*fpwrite;
	int	pathok = 0;

	DBUG_ENTER("enospc");

	if (fcntl(pdes[PIPEWRITE], F_SETFL, O_NONBLOCK) == SYSERROR) {
		testfail++;
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl failed errno %s (%d)", errname(err),err);
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;
	(void) close(pdes[PIPEREAD]);

	if ((fpwrite = fdopen(pdes[PIPEWRITE], "w")) == NULL) {
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fdopen failed");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	name = (char*)malloc(strlen(mountdir)+sizeof("/fill.tmp0000"));
	if (name == NULL) {
		testfail++;
		xx_rpt(DELETION);
		in_rpt("Out of memory");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	/*
	 * Build up  temp file name and open it
	 */
	(void) strcat(strcpy(name, mountdir), "/fill.tmp1");

	fp = freopen(name, "rb+", stdout);

	if (fp == NULL) {
		err = errno;
		testfail++;
		(void) fprintf(fpwrite, "%d\n", DELETION);
		(void) fprintf(fpwrite, "deletion reason: Open of %s failed - errno %s\n", name, errname(err));
		DBUG_EXIT(NEEDPRINT);
	}
	else
		PATH_TRACE;

	if (fseek(fp, 0L, SEEK_END) != 0) {
		testfail++;
		(void) fprintf(fpwrite, "%d\n", DELETION);
		(void) fprintf(fpwrite, "deletion reason: Seek to end of file failed\n");
		DBUG_EXIT(NEEDPRINT);
	}
	else
		PATH_TRACE;

	outstr = (char*)malloc(BUFSIZ+1);
	if (outstr == NULL) {
		testfail++;
		(void) fprintf(fpwrite, "%d\n", DELETION);
		(void) fprintf(fpwrite, "deletion reason: Out of memory\n");
		DBUG_EXIT(NEEDPRINT);
	}
	for (i = 0; i < BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		outstr[i] = STRPAT[i%PATLEN];
	}
	outstr[i] = '\0';

	for (i = 0; i < MAXTRYS && !testfail; i++) {

		if (i==0)
			PATH_TRACE;

		DBUG_PRINTF("enospc", "write block on %s", name);

		switch (interface) {
		case I_PUTC:
			for (j = 0; j < BUFSIZ && !testfail; j++)
				if (putc(outstr[j], fp) == EOF)
					testfail++;
			break;
		case I_PUTCHAR:
			for (j = 0; j < BUFSIZ && !testfail; j++)
				if (putchar(outstr[j]) == EOF)
					testfail++;
			break;
		case I_FPUTC:
			for (j = 0; j < BUFSIZ && !testfail; j++)
				if (fputc(outstr[j], fp) == EOF)
					testfail++;
			break;
		case I_PRINTF:
			if (printf("%s", outstr) < 0)
				testfail++;
			break;
		case I_FPRINTF:
			if (fprintf(fp, "%s", outstr) < 0)
				testfail++;
			break;
#if ANSI_VPRINTF_FUNC
		case I_VPRINTF:
			if (dovprintf(fp, "%s", outstr) < 0)
				testfail++;
			break;
		case I_VFPRINTF:
			if (dovprintf(fp, "%s", outstr) < 0)
				testfail++;
			break;
#endif
		case I_PUTS:
			if (puts(outstr) == EOF)
				testfail++;
			break;
		case I_FPUTS:
			if (fputs(outstr, fp) == EOF)
				testfail++;
			break;
		case I_FWRITE:
			if (fwrite((void*)outstr, BUFSIZ, 1, fp) < (size_t)1)
				testfail++;
			break;
		default:
			testfail++;
			(void) fprintf(fpwrite, "%d\n", DELETION);
			(void) fprintf(fpwrite, "deletion reason: Invalid interface\n");
			break;
		}

		if (testfail)
		{
			err = errno;

			switch (err) {
			case ENOSPC:
				(void) fprintf(fpwrite, "%d\n", 0); /* Success */
				break;
			default:
				testfail++;
				(void) fprintf(fpwrite, "%d\n", FAILURE);
				(void) fprintf(fpwrite, "write failed errno %s (%d)\n"
				  , errname(err)
				  , err
				  );
				break;
			}
		}

	}

	/*
	 * This may fail with ENOSPC as well.
	 */
	if (fp)
		(void) fclose(fp);

	if (i == MAXTRYS && !testfail) {
		testfail++;
		(void) fprintf(fpwrite, "%d\n", UNTESTED);	/* Or FAILURE, WARNING ? */
		(void) fprintf(fpwrite, "ENOSPC error didn't occur in %d trys\n", MAXTRYS);
	}
	else
		PATH_TRACE;

	free((void*)name);

	/* If the path count is ok we return a special exit value */
	if (pathok == 8)
		DBUG_EXIT(NEEDPRINT | PATH_OK);

	DBUG_EXIT(NEEDPRINT);
}

#if ANSI_VPRINTF_FUNC

/*VARARGS*/
#if TEST_ANSI
public int
dovprintf(FILE *fp, char *fmt, ...)
#else
public int
dovprintf(va_alist)
va_dcl
#endif
{
	va_list	args;
#if !TEST_ANSI
	char	*fmt;
	FILE	*fp;
#endif
	int	retval;

	DBUG_ENTER("dovprintf");

#if TEST_ANSI
	va_start(args, fmt);
#else
	va_start(args);
	fp = va_arg(args, FILE *);
	fmt = va_arg(args, char *);
#endif
	switch (interface)
	{
	case I_VPRINTF:
		retval = vprintf(fmt, args);
		break;

	case I_VFPRINTF:
		retval = vfprintf(fp, fmt, args);
		break;
	}
	va_end(args);

	DBUG_RETURN(retval);
}
#endif
