/*
*      SCCS:  @(#)  ANSI.os/streamio/puts/puts.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/puts - T.puts Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.puts

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the puts() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	10 Nov 1988
		(created from copy of ../T.putc/putc.c)
MODIFICATIONS:	
		Geoff Clare, 1 June 1989
		Add EIO tests.

		Geoff Clare, 23 Sep 1989
		Recognise deletion messages from child in do_ferr() tests.

		Geoff Clare, 22 Nov 1989
		Expect anything >= 0 on success, not number of chars.

		Steve Ratcliffe, 23 Mar 1990
		Add ENOSPC test.

		Geoff Clare, 14 June 1990
		Do EAGAIN and EPIPE tests on pipe and FIFO

		David Sawyer
		Mon Jul 16 09:43:23 BST 1990
		Added resillience code.

		Stuart Boutell, 5 Nov 1990
			changed dltd and reason to tet_reason,
			eliminated vg_env.

		John-Paul, 12 October 1992
			XNFS enhancements.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<chktimes.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"puts_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef puts
#endif



#define	NO_TESTS	13		/* number of tests in testset */
#define	WAITTIME	(15 * SPEEDFACTOR)	/* > timeout in do_ferr() */
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define	P_RD		0		/* read file descriptor of pipe */
#define	P_WR		1		/* write file descriptor of pipe */
#define	E_FDOPEN	1
#define	E_FCLOSE	2
#define	E_PUTS		3
#define	T_CHAR		'x'
#define	BUF_SIZ		512

#define I_PUTC		1
#define I_PUTCHAR	2
#define I_FPUTC		3

#define	GETBUF	if (fgets(buf, sizeof(buf), f_rd) == NULL) \
		{ \
			err = errno; \
			xx_rpt(DELETION); \
			in_rpt("deletion reason: fgets() failed to read data from pipe - errno %d(%s)", \
				err, errname(err)); \
			DBUG_VOID_RETURN; \
		}


extern	char	*errname();
extern	char	*execpath();
extern	void	do_feio();
#if TEST_XNFS
extern	char	*itoa();
extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;
#endif /* TEST_XNFS */

int	no_tests = NO_TESTS;

private	char	*t_string =	"test string";
private	char	*t1_string =	"test string\n";
#if TEST_XNFS
private	char	*xnfs_string =	ONE_BYTE_STR;

private	char	*tfile =	"./puts-t";
#endif /* TEST_XNFS */
private	char	*t2_file =	"./puts-t.2";
private	char	*t11_dir =	"./puts-d.11";
private char	*prog11  =	"puts_t11";
#if TEST_XNFS
private	char	*t12_file =	T12_FILE;
private	char	*t13_file =	T13_FILE;
#endif /* TEST_XNFS */
#if TEST_XNFS
private char	*prog_su  =	"puts_su";
#endif /* TEST_XNFS */

private	int	pipdes[2];
private	FILE	*f_rd;
private	FILE	*f_wr;
private	char	buf[BUF_SIZ];

public	int	setupok = -1;


private	void	comp_prep();
private	void	clean_tests();
private	void	test1();
private	void	test2();
private	void	test3_9();
private	void	test10();
private	void	test11();
private	void	test12();
private	void	test13();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3_9();
private void	ch_t11();
#if TEST_XNFS
private void	ch_t12();
#endif /* TEST_XNFS */
private	int	t3_9func();
#if !TEST_XNFS
private	int	t10io();
private	void	t10rpt();
#endif /* TEST_XNFS */
#if TEST_XNFS
private	int	t13_func();
private	int	t13_chk_func();
#endif /* TEST_XNFS */

public	void	(*setfunc) () = comp_prep;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3_9, 3,
	 test3_9, 4,
	 test3_9, 5,
	 test3_9, 6,
	 test3_9, 7,
	 test3_9, 8,
	 test3_9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	NULL, 0
};

private void
comp_prep()
{
#if TEST_XNFS
	static  char    msg[BUFSIZ];
	int		err;
#endif /* TEST_XNFS */
	int		i;
	int		pathok = 0;

	DBUG_ENTER("comp_prep");

#if TEST_XNFS

	errno = NOERROR;
	pipdes[P_WR] = open(tfile, O_RDWR|O_CREAT, (mode_t)(S_IRWXU|S_IRWXG|S_IRWXO));
	if ( pipdes[P_WR] == SYSERROR )
	{
		err = errno;
		(void) sprintf(msg, "open() failed in preparations - errno %d(%s)", err, errname(err));
		for (i=1; i<=NO_TESTS; i++)
			(void) tet_delete(i, msg);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	pipdes[P_RD] = open(tfile, O_RDONLY, (mode_t)(S_IRWXU|S_IRWXG|S_IRWXO));
	if ( pipdes[P_RD] == SYSERROR )
	{
		err = errno;
		(void) sprintf(msg, "open() failed in preparations - errno %d(%s)", err, errname(err));
		for (i=1; i<=NO_TESTS; i++)
			(void) tet_delete(i, msg);
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */

	if ( pipe(pipdes) == SYSERROR )
	{
		for (i=1; i<=NO_TESTS; i++)
			(void) tet_delete(i, "failed to create pipe");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

#endif /* TEST_XNFS */

	if (fcntl(pipdes[P_RD], F_SETFL, O_NONBLOCK) == SYSERROR)
	{
		for (i=1; i<=NO_TESTS; i++)
			(void) tet_delete(i, "failed to set O_NONBLOCK flag on pipe (read)");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((f_rd = fdopen(pipdes[P_RD], "r")) == (FILE *) NULL)
	{
		for (i=1; i<=NO_TESTS; i++)
			(void) tet_delete(i, "failed to open pipe for reading");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(4);
}

private void
ch_setup()
{
	DBUG_ENTER("ch_setup");
	
	if ( close(1) == SYSERROR )
		DBUG_EXIT(A_CLOSEFAIL);

	if ( dup(pipdes[P_WR]) != 1 )
		DBUG_EXIT(A_DUPFAIL);

	(void) close(pipdes[P_WR]);

	if ( close(pipdes[P_RD]) == SYSERROR )
		DBUG_EXIT(A_CLOSEFAIL);

	/* stdout now goes to pipe */
	f_wr = stdout;

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	err;
	int	len;
	int	rval;
	int	fail = 0;
	int	pathok = 0;

	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	GETBUF
	else
		PATH_TRACE;
	len = strlen(buf);

	if (len != strlen(t1_string) || strcmp(t1_string, buf) != 0)
	{
		if (fail++ == 0)
			xx_rpt(FAILURE);
		if (len != strlen(t1_string))
		{
			in_rpt("puts() did not write correct number of bytes to stdout");
			in_rpt("expected %d, got %d", strlen(t1_string), len);
		}
		else if (strcmp(t1_string, buf) != 0)
		{
			in_rpt("puts() did not write correct string to stdout");
			in_rpt("expected \"%s\", actual \"%s\"", t1_string, buf);
		}
	}
	else
		PATH_TRACE;

	GETBUF
	else
		PATH_TRACE;
	rval = atoi(buf);
	if (rval < 0)
	{
		if (fail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("puts() did not return correct value:");
		in_rpt("expected >= 0, got %d", rval);
	}
	else
		PATH_TRACE;
	
	if (fail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	rval;

	DBUG_ENTER("ch_t1");

	ch_setup();

	rval = puts(t_string);

	(void) fprintf(f_wr, "%d\n", rval);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	err = cppair(ch_t2, NULLFN, WAITTIME, 0);

	(void) unlink(t2_file);
	if (err == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	GETBUF
	else
		PATH_TRACE;
	if (strcmp(buf, "xs") == 0)
	{
		PATH_XS_RPT(2);
	}
	else
	{
		xx_rpt(FAILURE);
		in_rpt("%s", buf);
	}

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	char	*s;
	int	dupfd;

	DBUG_ENTER("ch_t2");
			
	ch_setup();

	/* save pipe to parent for result message */
	if ((dupfd = dup(fileno(f_wr))) < 0)
		DBUG_EXIT(E_FDOPEN);

	if (freopen(t2_file, "w", f_wr) == NULL )
		DBUG_EXIT(E_FDOPEN);

	savtimes(t2_file);

	(void) puts(t_string);

	if (fclose(f_wr) < 0)
		DBUG_EXIT(E_FCLOSE);
	
	if ((s = chktimes(t2_file, NEWMTIME | NEWCTIME)) == NULL)
		s = "xs";

	if (outstr(dupfd, s) != strlen(s))
		DBUG_EXIT(E_PUTS);

	DBUG_VOID_RETURN;
}

/* Lowest test number for which following function is used */
#define TBASE	3
private int	experr;

private	void
test3_9()
{
	int err, ret, ferr_ret, gotsig;
	static int errnums[] = { EAGAIN, -EAGAIN, EBADF, EINTR, EFBIG,
			      /* test3   test4    test5  test6  test7 */
				 EPIPE, -EPIPE };
			      /* test8  test9 */
	int	pathok = 0;

	DBUG_ENTER("test3_9");

#if TEST_XNFS
	if(tet_thistest != 7)
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}
#endif /* TEST_XNFS */

	experr = errnums[tet_thistest - TBASE];

	(void) cppair(ch_t3_9, NULLFN, WAITTIME, 0);

	if (experr < 0)
		experr = -experr;

	GETBUF
	else
		PATH_TRACE;
	if (strncmp(buf, "del", (size_t)3) == 0)
	{
		int len;

		GETBUF
		len = strlen(buf);
		if (len > 0 && buf[len-1] == '\n')
			buf[--len] = '\0';
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", buf);
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		ret = atoi(buf);
	}

	GETBUF
	else
		PATH_TRACE;
	err = atoi(buf);
	GETBUF
	else
		PATH_TRACE;
	ferr_ret = atoi(buf);
	GETBUF
	else
		PATH_TRACE;
	gotsig = atoi(buf);

	if (!ferr_ret || err != experr || ret != EOF ||
				(experr == EPIPE && !gotsig))
	{
		xx_rpt(FAILURE);
		if (!ferr_ret)
			in_rpt("puts() failed to set error indicator when testing for errno %d(%s)",
				experr, errname(experr));
		if (experr == EPIPE && !gotsig)
			in_rpt("SIGPIPE was not received");
		if (ret != EOF)
			in_rpt("puts() returned %d when testing for errno %d(%s), expected EOF",
				ret, experr, errname(experr));
		else if (err != experr)
			in_rpt("puts() set errno incorrectly: expected %d(%s), actual %d(%s)",
				experr, errname(experr), err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(6);
	}

	DBUG_VOID_RETURN;
}

private	void
ch_t3_9()
{
	int err, ret, ferr_ret, gotsig = 0;

	DBUG_ENTER("ch_t3_9");

	if ( close(pipdes[P_RD]) == SYSERROR )
		DBUG_EXIT(A_CLOSEFAIL);

	if ((f_wr = fdopen(pipdes[P_WR], "w")) == (FILE *) NULL)
		DBUG_EXIT(E_FDOPEN);

	ret = do_ferr(experr, 'w', STDOUT_FILENO, t3_9func, &ferr_ret, &gotsig);
	err = errno;
	if (experr < 0)
		experr = -experr;

	if (tet_reason(tet_thistest)!=NULL)
		(void) fprintf(f_wr, "del\n%s\n", tet_reason(tet_thistest));
	else
		(void) fprintf(f_wr, "%d\n%d\n%d\n%d\n",
			ret, err, ferr_ret, gotsig);

	DBUG_VOID_RETURN;
}

/* ARGSUSED */

private int
t3_9func(fp, nbyte)
FILE *fp;
int nbyte;
{
	DBUG_ENTER("t3_9func");

	globok = 1; /* must be set for do_ferr */

	DBUG_RETURN(puts(t_string));
}

#if TEST_XNFS

private void
test10()
{
	DBUG_ENTER("test10");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test10()
{
	DBUG_ENTER("test10");

	do_feio("w", STDOUT_FILENO, t10io, t10rpt);

	DBUG_VOID_RETURN;
}

/* ARGSUSED */

private int
t10io(fp)
FILE *fp;
{
	int ret;

	DBUG_ENTER("t10io");

	ret = puts("Z");

	globok = 1; /* must be set for do_feio() */

	DBUG_RETURN(ret);
}

private void
t10rpt(ret, err)
int ret, err;
{
	int	pathok = 0;

	DBUG_ENTER("t10rpt");

	if (ret != EOF || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("puts() from orphaned background process group did not give correct results");
		if (ret != EOF)
			in_rpt("expected return: EOF, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test11()
{

	DBUG_ENTER("test11");

	(void) cppair(ch_t11, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	int 	err;
	char	*args[6];
	extern	char	*vg_ident;
	extern	char	*vg_dbug;
	extern	char	**environ;

	DBUG_ENTER("ch_t11");

	args[0] = prog11;
	args[1] = t11_dir;
	args[2] = "puts";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog11), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

private void
test12()
{
	DBUG_ENTER("test12");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test12()
{
	DBUG_ENTER("test12");

	(void) tet_fork(ch_t12, NULLFN, WAITTIME, OK);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t12()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t12");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t12_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test13()
{
	DBUG_ENTER("test13");

	t_estale_wr_c(t13_file, 1, t13_func, t13_chk_func, SP_T13_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t13_func(fp)
FILE	*fp;
{
	DBUG_ENTER("t13_func");

	DBUG_RETURN(puts(xnfs_string) == EOF ? -1 : 0);
}

private int
t13_chk_func(fp, ebytes)
FILE	*fp;
int	ebytes;
{
	int	rval;
	int	err;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("t13_chk_func");

	errno = NOERROR;

	for(i = 0; i < ebytes; i++)
		if((rval = puts(xnfs_string)) != strlen(xnfs_string) + 1)
			break;

	err = errno;

	if(rval != EOF || ferror(fp) == 0 || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(rval != EOF)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value EOF");
			in_rpt("Actual   return value %d", rval);
		}
		else
			PATH_TRACE;

		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;
		
		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */

private void
clean_tests()
{
#if TEST_XNFS
	(void) unlink(tfile);
#endif /* TEST_XNFS */
}

