/*
*      SCCS:  @(#)  ANSI.os/streamio/remove/remove.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/remove - T.remove Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.remove

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the remove() system interface.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	3 May 1989
MODIFICATIONS:	
		Geoff Clare, 9 June 1989:  Add do_access() test.
		Geoff Clare, 22 Aug 1989:  VSX_REMOVE_DIR_EBUSY={Y,S,P,N}
		
		Geoff Clare, 13 Feb 1990
			In EBUSY tests, after chdir(), open "." to ensure
			the directory is considered "in use" by the process.

		Geoff Clare, 4 Apr 1990
			New interface with pathtoolng() and nametoolng().

		David Sawyer
		Mon Jul 16 12:41:32 BST 1990
		Added resillience code, also added DBUG macros to tbremove().

		Stuart Boutell, 5 Nov 1990
			changed vg_env to environ.

		Geoff Clare, 21 May 1992
			Added pathname resolution and portable filename tests.

		Geoff Clare, 18 June 1992
			Add NO_DOT to init_pathres() argument.

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		John-Paul, 16 November 1992
			XNFS enhancements.

		Geoff Clare, 3 Nov 1995
			Changed child timeout for do_access() test.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<string.h>
#include	<fcntl.h>
#include	<dirent.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<chktimes.h>
#include	<pathres.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"remove_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef remove
#endif


#define NO_TESTS	25
#define	WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define NOK		02
#define PATH_OK         04
#define E_CHDIR		02
#define E_OPEN		04

extern	char	*errname();
#if TEST_XNFS
extern  char    *xnfs_nametoolng();
#else /* TEST_XNFS */
extern  char    **nametoolng();
#endif /* TEST_XNFS */
extern	char	**pathtoolng();
extern	char	*execpath();
extern	char	*itoa();
extern	void	clntoolng();
extern	void	valrpt();

extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

public	int	no_tests = NO_TESTS;

private	char	*prog_su =	"remove_su";

private	char	*t1a_file =	"remove-t.1a";
private	char	*t1b_files[] = {"remove-t.1b", "remove-t.1c", "remove-t.1d",
				"remove-t.1e", NULL };
private	char	*t2_file =	"remove-t.2";
private	char	*t3_file =	"remove-t.3";
private	char	*t4_dir =	"remove-d.4";
private	char	*t4_file =	"remove-d.4/remove-t.4";
private	char	*t5_file =	"remove-t.5";
private	char	*t6a_file =	"remove-t.6a";
private	char	*t6b_file =	"remove-t.6b";
private	char	*t7a_file =	"remove-t.7a";
private	char	*t7b_file =	"remove-t.7b";
private char	*t9_dir =	"remove-d.9";
private char	*t9_file =	"remove-d.9/remove-t.9";
#if TEST_XNFS
private char	*t9_2_file =	T9_2_FILE;
#endif /* TEST_XNFS */
private char	*t10_dir =	"remove-d.10";
private char	*t10_file =	"remove-d.10/remove-t.10";
#if TEST_XNFS
private char	*t10_2_file =	T10_2_FILE;
#endif /* TEST_XNFS */
private	char	*t11a_file =	"remove-t.11a";
private	char	*t11b_file =	"remove-t.11b";
private char	*t12_dir =	"remove-d.12";
private char	*t12_file =	"remove-d.12/remove-t.12";
private	char	*t14_file =	"remove-t.14";
private char	*t15_dir =	"remove-d.15";
private char	*t15_file =	"remove-d.15/remove-t.15";
private	char	*t16_file =	"remove-t.16";
private	char	*t17_dir =	"remove-d.17";
private char	*t18_ndir =	"remove-t.18";
private char	*t18_file =	"remove-t.18/remove-t.18";
private char	*t19_dir =	"remove-d.19";
private char	*t19_file =	"remove-d.19/remove-t.19";

private int	testfail;

private void	cln_tests();
private int	tbremove();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	ch1_t7();
private void	ch2_t7();
private void	ch_t8();
#if TEST_XNFS
private void	ch_t9();
private void	ch_t10();
#endif /* TEST_XNFS */
private void	ch1_t11();
private void	ch2_t11();
private void	ch_t17();
private void	ch_t19();
private void	pa_t7();
private void	pa_t11();

#if TEST_XNFS
private int     snames[] = { 1, 0 };
#endif


public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	NULL, 0
};

private void
cln_tests()
{
	char **p;

	DBUG_ENTER("cln_tests");

	(void) unlink(t1a_file);
	(void) rmdir(t2_file);
	(void) unlink(t3_file);
	(void) unlink(t4_file);
	(void) rmdir(t4_dir);
	(void) rmdir(t5_file);
	(void) unlink(t6a_file);
	(void) unlink(t6b_file);
	(void) rmdir(t7a_file);
	(void) rmdir(t7b_file);
	(void) rmdir(t9_file);
	(void) rmdir(t9_dir);
	(void) chmod(t10_dir, MODEANY);
	(void) rmdir(t10_file);
	(void) rmdir(t10_dir);
	(void) rmdir(t11a_file);
	(void) rmdir(t11b_file);
	(void) unlink(t12_file);
	(void) rmdir(t12_dir);
	(void) rmdir(t14_file);
	(void) unlink(t18_ndir);
	(void) rmdir(t19_dir);
	for (p = portfnam; *p != NULL; p++)
		(void) rmdir(*p);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* remove() on plain file decrements link count by 1 */

	int	fd, err, oldcount, i;
	char	**p;
	struct stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	if ((fd = crfile(t1a_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t1a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	i = 0;
	for (p = t1b_files; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		if (link(t1a_file, *p) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: link(\"%s\", \"%s\") failed - errno %d (%s)",
				t1a_file, *p, err, errname(err));
			DBUG_VOID_RETURN;
		}
	}

	if (stat(t1a_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t1a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	oldcount = stbuf.st_nlink;

	i = 0;
	for (p = t1b_files; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		if (tbremove(*p, SUCCEED, NOERROR) != SUCCEED)
			break;

		if (stat(t1a_file, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				t1a_file, err, errname(err));
			break;
		}
#ifdef TESTBED2
		oldcount--;
#endif
		if (stbuf.st_nlink != oldcount - 1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("remove(\"%s\") did not decrement link count by 1",
				*p);
			in_rpt("old link count = %d, new count = %d",
				oldcount, (int)stbuf.st_nlink);
		}
		oldcount = stbuf.st_nlink;
	}

	(void) unlink(t1a_file);

#if TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else /* TEST_XNFS */

	if (mkfifo(t1a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t1a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	i = 0;
	for (p = t1b_files; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		if (link(t1a_file, *p) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: link(\"%s\", \"%s\") failed - errno %d (%s)",
				t1a_file, *p, err, errname(err));
			DBUG_VOID_RETURN;
		}
	}

	if (stat(t1a_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t1a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	oldcount = stbuf.st_nlink;

	i = 0;
	for (p = t1b_files; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		if (tbremove(*p, SUCCEED, NOERROR) != SUCCEED)
			break;

		if (stat(t1a_file, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				t1a_file, err, errname(err));
			break;
		}
#ifdef TESTBED2
		oldcount--;
#endif
		if (stbuf.st_nlink != oldcount - 1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("remove(\"%s\") did not decrement link count by 1",
				*p);
			in_rpt("old link count = %d, new count = %d",
				oldcount, (int)stbuf.st_nlink);
		}
		oldcount = stbuf.st_nlink;
	}

	(void) unlink(t1a_file);

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(8);
	


	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* remove() removes an empty directory */

	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	if (crdir(t2_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbremove(t2_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);
	
	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* open file is still accessible after remove() */

	int 	fd, err, rval;
#if !TEST_XNFS
	int	fd2;
#endif /* TEST_XNFS */
	static 	char wbuf[] = "testing, testing, 1..2..3..4";
	char 	rbuf[sizeof(wbuf)];
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	if ((fd = crfile(t3_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, wbuf, sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() failed - errno %d (%s)", err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	if ((fd = open(t3_file, O_RDONLY)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY) failed - errno %d (%s)",
			t3_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tbremove(t3_file, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	fd = -fd;
#endif

	if ((rval = read(fd, rbuf, sizeof(wbuf))) != sizeof(wbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on removed file returned %d, expected %d",
			rval, (int)sizeof(wbuf));
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (memcmp((void *)rbuf, (void *)wbuf, sizeof(wbuf)) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from removed file returned incorrect data");
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	fd = -fd;
#endif

	(void) close(fd);


#if TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else /* TEST_XNFS */
	
	if (mkfifo(t3_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd2 = open(t3_file, O_RDONLY|O_NONBLOCK)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) failed - errno %d (%s)",
			t3_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t3_file, O_WRONLY|O_NONBLOCK)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_WRONLY|O_NONBLOCK) failed - errno %d (%s)",
			t3_file, err, errname(err));
		(void) close(fd2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, wbuf, sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() failed - errno %d (%s)", err, errname(err));
		(void) close(fd);
		(void) close(fd2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	if ((fd = open(t3_file, O_RDONLY|O_NONBLOCK)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) failed - errno %d (%s)",
			t3_file, err, errname(err));
		(void) close(fd2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tbremove(t3_file, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) close(fd);
		(void) close(fd2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;


#ifdef TESTBED2
	fd = -fd;
#endif

	if ((rval = read(fd, rbuf, sizeof(wbuf))) != sizeof(wbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on removed file returned %d, expected %d",
			rval, (int)sizeof(wbuf));
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (memcmp((void *)rbuf, (void *)wbuf, sizeof(wbuf)) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from removed file returned incorrect data");
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	fd = -fd;
#endif

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(12);
	
	(void) close(fd);
#if !TEST_XNFS
	(void) close(fd2);
#endif /* TEST_XNFS */
	
	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test4()
{
	DBUG_ENTER("test4");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test4()
{
	/* remove() on open directory: no "." & "..", can't create files */

	struct dirent	*dp;
	DIR		*dir;
	int		fd, err;
	int		pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	if (crdir(t4_dir, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t4_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((dir = opendir(t4_dir)) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("opendir(\"%s\") failed - errno %d (%s)",
			t4_dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifndef TESTBED2
	globok = 0;
	if (tbremove(t4_dir, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) closedir(dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
#endif

	rewinddir(dir);
	
	/* Path tracing not possible in this loop */
	while ((dp = readdir(dir)) != NULL)
	{
		if (strcmp(dp->d_name, ".") == 0 ||
		    strcmp(dp->d_name, "..") == 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("open directory contained entry \"%s\" after remove(\"%s\")",
				dp->d_name, t4_dir);
		}
	}

	if ((fd = crfile(t4_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) != SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("file \"%s\" was created successfully in open directory after remove(\"%s\")",
			t4_file, t4_dir);
		(void) close(fd);
		(void) unlink(t4_file);
	}
	else
		PATH_TRACE;

	if (mkfifo(t4_file, MODEANY) != SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("file \"%s\" was created successfully in open directory after remove(\"%s\")",
			t4_file, t4_dir);
		(void) close(fd);
		(void) unlink(t4_file);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
#ifndef TESTBED2
		PATH_XS_RPT(5);
#else
		PATH_XS_RPT(4);
#endif

	(void) closedir(dir);
	
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test5()
{
	DBUG_ENTER("test5");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test5()
{
	/* remove() on open directory: not removed until close */

	int	fd, err;
	struct stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	if (crdir(t5_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t5_file, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY) failed - errno %d (%s)",
			t5_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tbremove(t5_file, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	(void) close(fd);
#endif

	if (fstat(fd, &stbuf) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() failed on fd open to directory after remove(\"%s\")",
			t5_file);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test6()
{
	/* remove() updates timestamps */

	char	*chkt;
	int	fd, err;
	struct stat stbuf1, stbuf2;
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	if ((fd = crfile(t6a_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t6a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (link(t6a_file, t6b_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
			t6a_file, t6b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(".", &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\".\") failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(t6a_file);
	PATH_FUNC_TRACE;

#ifndef TESTBED2
	globok = 0;
	if (tbremove(t6b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
#endif

	globok = 0;
	chkt = chktimes(t6a_file, NEWCTIME);
	PATH_FUNC_TRACE;
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("remaining link %s", chkt);
	}
	else
		PATH_TRACE;

	if (stat(".", &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf2.st_ctime == stbuf1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	if (stbuf2.st_mtime == stbuf1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	(void) unlink(t6a_file);

#if TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else /* TEST_XNFS */

	if (mkfifo(t6a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t6a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (link(t6a_file, t6b_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
			t6a_file, t6b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(".", &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\".\") failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	globok = 0;
	savtimes(t6a_file);
	PATH_FUNC_TRACE;

#ifndef TESTBED2
	globok = 0;
	if (tbremove(t6b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
#endif

	globok = 0;
	chkt = chktimes(t6a_file, NEWCTIME);
	PATH_FUNC_TRACE;
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("remaining link %s", chkt);
	}
	else
		PATH_TRACE;

	if (stat(".", &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf2.st_ctime == stbuf1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	if (stbuf2.st_mtime == stbuf1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

#endif /* TEST_XNFS */

	if (testfail == 0)
#ifndef TESTBED2
		PATH_XS_RPT(20);
#else
		PATH_XS_RPT(18);
#endif

	(void) unlink(t6a_file);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* remove a busy directory */

	char	*supp;
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	if ((supp = tet_getvar("VSX_REMOVE_DIR_EBUSY")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (*supp == 'Y' || *supp == 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY is set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*supp != 'Y' && *supp != 'y' && *supp != 'S' && *supp != 's')
	{
		ret = cppair(ch1_t7, NULLFN, WAITTIME, NOK | PATH_OK);
		if (ret & NOK)
			testfail++;
		else
		{
			if (ret & PATH_OK)
				PATH_TRACE;
		}
	}
	else
	{
		PATH_TRACE;
	}

	if (*supp != 'Y' && *supp != 'y' && *supp != 'P' && *supp != 'p')
	{
		if (crdir(t7b_file, (mode_t)MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t7b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		ret = cppair(ch2_t7, pa_t7, WAITTIME, E_CHDIR|E_OPEN);
		PATH_FUNC_TRACE;
		if (ret == E_CHDIR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: chdir(\"%s\") failed in child",
				t7b_file);
		}
		else if (ret == E_OPEN)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: open(\".\", O_RDONLY) failed in child");
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
ch1_t7()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch1_t7");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t7a_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog_su, err, errname(err));

	DBUG_EXIT(NOK);
}

private void
pa_t7()
{
	int	pathok = 0;

	DBUG_ENTER("pa_t7");

	(void) sleep(WAITTIME/2);

	globok = 0;
	(void) tbremove(t7b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch2_t7()
{
	DBUG_ENTER("ch2_t7");

#ifdef TESTBED2
	DBUG_EXIT(E_CHDIR);
#endif

	if (chdir(t7b_file) == SYSERROR)
		DBUG_EXIT(E_CHDIR);

	if (open(".", O_RDONLY) < 0)
		DBUG_EXIT(E_OPEN);
	
	(void) sleep(WAITTIME);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* write access to parent, search access to path prefix granted */

	int waittime;

	DBUG_ENTER("test8");

	waittime = WAITTIME * (5 + sysconf(_SC_NGROUPS_MAX)/4);

	(void) cppair(ch_t8, NULLFN, 4 * waittime, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t8");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog_su, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test9()
{
	/* search permission denied for component of path prefix */

	int	err;
	int	pathcheck = 0;
	mode_t	mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
#if TEST_XNFS
	int     rc;
#endif
	int	pathok = 0;


	DBUG_ENTER("test9");

	testfail = 0;

	if (crdir(t9_dir, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t9_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t9_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (chmod(t9_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t9_dir, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tbremove(t9_file, SYSERROR, EACCES) != SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (chmod(t9_dir, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: chmod(\"%s\", %#o) failed - errno %d (%s)",
			t9_dir, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	(void) remove(t9_file);
#endif
	if (eaccess(t9_file, F_OK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to remove() removed file");
	}
	else
		PATH_TRACE;

	(void) unlink(t9_file);
	(void) rmdir(t9_dir);

#if TEST_XNFS

	if (tet_sync(SP_T9_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t9, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T9_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pathcheck = 9;

#else /* TEST_XNFS */

	pathcheck = 6;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
ch_t9()
{
	int err;
	char *args[7];

        DBUG_ENTER("ch_t9");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t9_2_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
	prog_su, err, errname(err));

	DBUG_EXIT(NOK);
}

#endif /* TEST_XNFS */

private void
test10()
{
	/* remove subdir of directory which denies write permission */

	int	err;
	int	pathcheck = 0;
	mode_t	mode = (S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
#if TEST_XNFS
	int     rc;
#endif
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	if (crdir(t10_dir, (mode_t)MODEANY,
			(int)geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t10_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (crdir(t10_file, (mode_t)MODEANY,
			(int)geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (chmod(t10_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t10_dir, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tbremove(t10_file, SYSERROR, EACCES) != SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else	
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	(void) chmod(t10_dir, MODEANY);
	(void) remove(t10_file);
#endif
	if (eaccess(t10_file, F_OK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to remove() removed file");
	}
	else
		PATH_TRACE;

	(void) chmod(t10_dir, MODEANY);
	(void) rmdir(t10_file);
	(void) rmdir(t10_dir);

#if TEST_XNFS

	if (tet_sync(SP_T10_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t10, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T10_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pathcheck = 8;

#else /* TEST_XNFS */

	pathcheck = 5;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
ch_t10()
{
	int err;
	char *args[7];

        DBUG_ENTER("ch_t10");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t10_2_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
	prog_su, err, errname(err));

	DBUG_EXIT(NOK);
}

#endif /* TEST_XNFS */


private void
test11()
{
	/* remove a busy directory */

	char	*supp;
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	if ((supp = tet_getvar("VSX_REMOVE_DIR_EBUSY")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (*supp == 'N' || *supp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*supp == 'Y' || *supp == 'y' || *supp == 'S' || *supp == 's')
	{
		ret = cppair(ch1_t11, NULLFN, WAITTIME, NOK | PATH_OK);
		if (ret & NOK)
			testfail++;
		else
		{
			if (ret & PATH_OK)
				PATH_TRACE;
		}
	}
	else
	{
		PATH_TRACE;
	}

	if (*supp == 'Y' || *supp == 'y' || *supp == 'P' || *supp == 'p')
	{
		if (crdir(t11b_file, (mode_t)MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create directory \"%s\"",
				t11b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		ret = cppair(ch2_t11, pa_t11, WAITTIME, E_CHDIR|E_OPEN);
		PATH_FUNC_TRACE;
		if (ret == E_CHDIR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: chdir(\"%s\") failed in child",
				t11b_file);
		}
		else if (ret == E_OPEN)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: open(\".\", O_RDONLY) failed in child");
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) rmdir(t11a_file);
	(void) rmdir(t11b_file);

	DBUG_VOID_RETURN;
}

private void
ch1_t11()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch1_t11");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t11a_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog_su, err, errname(err));

	DBUG_EXIT(NOK);
}

private void
pa_t11()
{
	int	pathok = 0;

	DBUG_ENTER("pa_t11");

	(void) sleep(WAITTIME/2);

	globok = 0;
	if (tbremove(t11b_file, SYSERROR, EBUSY) != SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (eaccess(t11b_file, F_OK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to remove() removed directory");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch2_t11()
{
	DBUG_ENTER("ch2_t11");

#ifdef TESTBED2
	DBUG_EXIT(E_CHDIR);
#endif

	if (chdir(t11b_file) == SYSERROR)
		DBUG_EXIT(E_CHDIR);

	if (open(".", O_RDONLY) < 0)
		DBUG_EXIT(E_OPEN);
	
	(void) sleep(WAITTIME);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	/* remove a non-empty directory */

	int 	fd;
	int	pathok = 0;

	DBUG_ENTER("test12");

	testfail = 0;

	if (crdir(t12_dir, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t12_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t12_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	/* tbremove() also allows ENOTEMPTY when EEXIST expected */
	globok = 0;
	(void) tbremove(t12_dir, SYSERROR, EEXIST);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(t12_file);
	(void) rmdir(t12_dir);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test13()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname[0], (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		PATH_TRACE;
		globok = 0;
		(void) tbremove(longname[0], SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		(void) close(fd);
	}

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbremove(longname[1], SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test14()
{
	/* path longer than PATH_MAX */

	char	**longname;
	int	pathok = 0;

	DBUG_ENTER("test14");

	testfail = 0;

	if (crdir(t14_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	longname = pathtoolng(t14_file);
	PATH_FUNC_TRACE;
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbremove(longname[0], SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (crdir(t14_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (longname[1] != NULL)
	{
		if ( tbremove(longname[1], SYSERROR, ENAMETOOLONG) == SYSERROR)
		{
			PATH_FUNC_TRACE;
#ifdef TESTBED2
			(void) remove(t14_file);
#endif
			if (eaccess(t14_file, F_OK) != 0)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("unsuccessful call to remove() removed file");
			}
		}
		else
			PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	clntoolng();
	(void) rmdir(t14_file);

	DBUG_VOID_RETURN;
}

private void
test15()
{
	/* component of path prefix does not exist */

	int	pathok = 0;

	DBUG_ENTER("test15");

	testfail = 0;

	if (eaccess(t15_dir, F_OK) == 0 && remove(t15_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t15_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbremove(t15_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	/* directory does not exist */

	int	pathok = 0;

	DBUG_ENTER("test16");

	testfail = 0;

	if (eaccess(t16_file, F_OK) == 0 && remove(t16_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbremove(t16_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test17()
{
	/* path is "" */

	DBUG_ENTER("test17");

	testfail = 0;

	if (crdir(t17_dir, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t17_dir);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t17, NULLFN, WAITTIME, 0);

	(void) rmdir(t17_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t17()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("ch_t17");

	/* Do it in a subdirectory, just in case */

	if (chdir(t17_dir) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chdir(\"%s\") failed - errno %d (%s)",
			t17_dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbremove("", SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test18()
{
	/* component of path is not a directory */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test18");

	testfail = 0;

	if ((fd = crfile(t18_ndir, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t18_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbremove(t18_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t18_ndir);

	DBUG_VOID_RETURN;
}

private void
test19()
{
	/* remove() on read-only file system */

	DBUG_ENTER("test19");

	(void) cppair(ch_t19, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t19_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t19()
{
	char	*rofs;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t19");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_ROFS is not defined");
		DBUG_VOID_RETURN;
	}

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = t19_dir;
	args[3] = t19_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog_su, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test20()
{
	/* pathname resolution */

	char	*p1, *p2;
	int	npairs, i;
	int	pathok = 0;

	DBUG_ENTER("test20");

	testfail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOT|PRES_NODOTDOT);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOT|PRES_NODOTDOT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Check that remove() resolves both names to the file
	   to which crdir() and eaccess() resolve the second name */

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test20", "testing \"%s\" == \"%s\"", p1, p2);
		DBUG_PRINTF("test20", "npairs = %d",npairs);

		if (crdir(p2, (mode_t)MODEANY,
				(int)geteuid(), (int)getegid()) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create directory \"%s\"",
				p2);
			break;
		}
		else
			PATH_TRACE;

		globok = 0;
		if (tbremove(p2, SUCCEED, NOERROR) != SUCCEED)
		{
			PATH_FUNC_TRACE; PATH_TRACE; PATH_TRACE;  	/* Needed to be consistant */
			continue;
		}
		else
		{
			PATH_FUNC_TRACE;
		}
		
		/* eaccess(p2) is done in tbremove() */

#ifndef TESTBED2
		if (crdir(p2, (mode_t)MODEANY,
				(int)geteuid(), (int)getegid()) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create directory \"%s\"",
				p2);
			break;
		}
		else
			PATH_TRACE;
#else
		PATH_TRACE; 				/* Valid path if not TESTBED2 */
#endif

		globok = 0;
		(void) tbremove(p1, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;

		/* eaccess(p1) is done in tbremove(), but we want p2 */

		if (eaccess(p2, F_OK) == 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("remove() did not resolve \"%s\" and \"%s\" to the same file",
				p1, p2);
			(void) remove(p2);
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
	{
		PATH_XS_RPT(1 + ((npairs) *5));
	}
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test21()
{
	/* portable filenames */

	char	**p;
	int	pathok = 0;
	int	loop1 , loop2;

	DBUG_ENTER("test21");

	testfail = 0;

	for (p = portfnam, loop1 = 0; *p != NULL; p++,loop1++)
	{
		if (crdir(*p, (mode_t)MODEANY,
				(int)geteuid(), (int)getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

#ifdef TESTBED2
	(void) remove(portfnam[0]);
#endif

	/* undistinguished names will give ENOENT */

	for (p = portfnam, loop2 = 0; *p != NULL; p++,loop2++)
	{
		globok = 0;
		(void) tbremove(*p, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(loop1+loop2); 

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test22()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test22");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbremove(longname, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test22()
{
	DBUG_ENTER("test22");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test23()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test23");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbremove(longname, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test23()
{
	DBUG_ENTER("test23");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test24()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test24");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;


	if ((fd = crfile(longname, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	fd = tbremove(longname, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(longname);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test24()
{
	DBUG_ENTER("test24");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test25()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test25");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	fd = tbremove(longname, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test25()
{
	DBUG_ENTER("test25");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private int
tbremove(tbpath, expret, experrno)
char *tbpath;
int  expret, experrno;
{
	char	*reppath = tbpath;
	int	rval, obserr;
	char	pathbuf[30];
	int	pathok = 0;

	DBUG_ENTER("tbremove");

	errno = 0;
	rval = remove(tbpath);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	obserr = errno;

	if (expret != SYSERROR && rval == expret)
	{
		if (eaccess(tbpath, F_OK) == 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("successful call to remove() did not remove directory");
		}
		else
			PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_RETURN(rval);
	}
	else if (expret == SYSERROR && rval == expret)
	{
		if (experrno == obserr)
		{
			PATH_FUNC_RPT(0);
			DBUG_RETURN(rval);
		}
		if (experrno == EEXIST && obserr == ENOTEMPTY)
		{
			PATH_FUNC_RPT(0);
			DBUG_RETURN(rval);
		}
	}

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("remove(\"%s\") did not give correct results", reppath);
	valrpt(expret, rval, experrno, obserr);
	
	DBUG_RETURN(rval);
}
