/*
*      SCCS:  @(#)  ANSI.os/streamio/remove/remove_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/remove - remove_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		remove_su - used by T.remove in privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.remove to perform privileged tests.

	  usage:
		remove_su testnum dir file ident dbug

	  where testnum is the number of the test to be executed, dir
	  and file are parameters to be passed to the test.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	11 Apr 1989
MODIFICATIONS:
		Geoff Clare, 9 June 1989:  Add do_access() test.
		Geoff Clare, 15 Nov 1989
			New do_access() argument
			New setuprofs() argument

		David Sawyer
		Tue Jul 17 16:44:43 BST 1990
		Added resillience code.

		Stuart Boutell, 5 Nov 1990
			eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Neil Moses, 8 June 1992
			Check that time stamps are not changed.

		John-Paul, 17 November 1992
			XNFS enhancements.

		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <dbug.h>
#include <errno.h>
#include <sys/stat.h>
#include <report.h>
#include <tet_api.h>
#include <xnfs.h>
#include "remove_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef remove
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	NOK		02
#define	PATH_OK		04

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
public  int	testfail;

extern	char	**environ;

extern	void	cln_access();
extern	void	valrpt();
extern	void	set_pname();
extern	void	savtimes();
extern	char	*chktimes();
extern	char	*errname();

private	void	subtest7();
private	void	subtest8();
#if TEST_XNFS
private	void	subtest9_10();
#endif /* TEST_XNFS */
private	void	subtest11();
private	void	subtest19();
private	int	t8_func();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int testnum;
	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testnum = atoi(argv[1]);
	switch(testnum)
	{
	case 7:
		subtest7(argv[3]);
		break;

	case 8:
		subtest8();
		break;

#if TEST_XNFS
	case 9:
	case 10:
		subtest9_10(argv[3]);
		break;
#endif /* TEST_XNFS */

	case 11:
		subtest11(argv[3]);
		break;

	case 19:
		subtest19(argv[2], argv[3]);
		break;

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed : %d", testnum);
		return(OK);
	}

	return(OK);
}


private void
subtest7(file)
char *file;
{
	/* No EBUSY on mount point */

	int 	ret, err;
	int	pathok = 0;

	DBUG_ENTER("subtest7");

	globok = 0;
	if (mountfs(file, (mode_t)MODEANY) != 0)
		DBUG_TET_EXIT(NOK);
	else
		PATH_FUNC_TRACE;

	errno = 0;
	if ((ret = remove(file)) != SUCCEED)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("remove(\"%s\") did not give correct results", file);
		valrpt(SUCCEED, ret, NOERROR, err);
		(void) umntfs(file);
		DBUG_TET_EXIT(NOK);
	}
	else
		PATH_TRACE;
	(void) umntfs(file);
	
	if (pathok==2)
		DBUG_TET_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}

private void
subtest8()
{
	/* write access to parent, search access to path prefix granted */
	int	pathok = 0;

	DBUG_ENTER("subtest8");

	testfail = 0;

	/* plain file */
	globok = 0;
	if (do_access('f', (mode_t)0, (mode_t)(S_IWUSR|S_IXUSR), t8_func, 4*WAITTIME, SP_T8_DO_ACCESS1) != 0)
		testfail++;
	else
		PATH_FUNC_TRACE;
	
	cln_access();

	/* directory */
	globok = 0;
	if (do_access('d', (mode_t)S_IRWXU, (mode_t)(S_IWUSR|S_IXUSR), t8_func, 4*WAITTIME, SP_T8_DO_ACCESS2) != 0)
		testfail++;
	else
		PATH_FUNC_TRACE;
	
	cln_access();

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private int
t8_func(file, desc)
char *file, **desc;
{
	int 	ret, err;
	int	pathok = 0;

	DBUG_ENTER("t8_func");

	errno = NOERROR;
	if ((ret = remove(file)) != SUCCEED)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("remove(\"%s\") did not give correct results", file);
		valrpt(SUCCEED, ret, NOERROR, err);
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(0);
}

#if TEST_XNFS

private void
subtest9_10(file)
char *file;
{
	int err, ret, rc;
	int pathok = 0;

	DBUG_ENTER("subtest9_10");

	errno = NOERROR;
	ret = remove(file);
	err = errno;

	if (ret != -1 || err != EACCES)
	{
		xx_rpt(FAILURE);

		if (ret != -1)
			in_rpt("remove(%s) did not return -1", file);

		if (err != EACCES)
			in_rpt("remove(%s) did not return expected errno",file);
			in_rpt("expected errno EACCES, found %d(%s)",
				err, errname(err));

		DBUG_TET_EXIT(NOK);
	}
	else
		PATH_TRACE;

	rc = OK;
	if (pathok == 1)
		rc |= PATH_OK;

	DBUG_TET_EXIT(rc);
}

#endif /* TEST_XNFS */

private void
subtest11(file)
char *file;
{
	int	pathok = 0;

	/* EBUSY on mount point */

	int ret, err;

	DBUG_ENTER("subtest11");

	globok = 0;
	if (mountfs(file, (mode_t)MODEANY) != 0)
		DBUG_TET_EXIT(NOK);
	else
		PATH_FUNC_TRACE;

	errno = 0;
	if ((ret = remove(file)) != SYSERROR || errno != EBUSY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("remove(\"%s\") did not give correct results", file);
		valrpt(SYSERROR, ret, EBUSY, err);
		(void) umntfs(file);
		DBUG_TET_EXIT(NOK);
	}
	else if (eaccess(file, F_OK) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("unsuccessful call to remove() removed directory");
		(void) umntfs(file);
		DBUG_TET_EXIT(NOK);
	}
	else
		PATH_TRACE;
	(void) umntfs(file);

	if (pathok==2)
		DBUG_TET_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}

private void
subtest19(mdir, file)
char	*mdir, *file;
{
	/* remove file on read only file system */

	char	*errptr;
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("subtest19");

	testfail = 0;

	globok = 0;
	if (setuprofs(mdir, file, 'f', (mode_t)MODEANY) != 0)
	{
		DBUG_VOID_RETURN;	/* reported in setuprofs() */
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(mdir);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	rval = remove(file);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;

	if (rval != SYSERROR || err != EROFS)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("remove(\"%s\") did not give correct results", file);
		valrpt(SYSERROR, rval, EROFS, err);
	}
	else
		PATH_TRACE;

	if (rval == SYSERROR)
	{
		if (eaccess(file, F_OK) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to remove() removed file");
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (err == EROFS)
	{
		globok = 0;
		errptr = chktimes(mdir, 0);
		PATH_FUNC_TRACE;

		if(errptr != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to remove() changed time stamps on %s", mdir);
			in_rpt("%s", errptr);
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) clnuprofs(mdir, file);

	DBUG_VOID_RETURN;
}
