/*
*      SCCS:  @(#)  ANSI.os/streamio/rewind/rewind.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/rewind - T.rewind Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.rewind

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the rewind() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		Phyfos Photi, Hoskyns Group plc
		22/08/88
		rewind() testset separated from fseek() testset

		Geoff Clare, 7 Nov 1988
		Add chktimes() test

		David Sawyer
		Tue Jul 17 17:03:31 BST 1990
		Added resillience code.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<unistd.h>
#include	<signal.h>
#include	<report.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<chktimes.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef rewind
#endif


#define	NO_TESTS	4		/* number of tests in testset */
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define	WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();

int	no_tests = NO_TESTS;

private char	*t_string =	"abcdefghijklmnopqrstuvwxyz";
private	char	*t_file =	"./rewind-t";
private	char	*t4_file =	"./rewind-t.4";

private void	prepare_tests();
private	void	cln_tests();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private void
prepare_tests()
{
	int		fd;
	int		i;
	static char	msg[150];

	DBUG_ENTER("prepare_tests");

	if ((fd = creat(t_file, MODEANY)) == SYSERROR)
		(void) sprintf(msg, "creat(%s, %#o) failed in preperations - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
	else if (write(fd, t_string, (unsigned) strlen(t_string)) != strlen(t_string))
	{
		(void) sprintf(msg, "write() failed in setup - errno %d(%s)",
			errno, errname(errno));
		fd = SYSERROR;
	}
	else
		(void) close(fd);

	if (fd == SYSERROR)
		for (i = 1 ; i <= NO_TESTS ; i++)
			(void) tet_delete(i, msg);

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");
	(void) unlink(t_file);
	(void) unlink(t4_file);
	DBUG_VOID_RETURN;
}


private void
test1()
{
	FILE	*fp;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test1");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) getc(fp);
	(void) getc(fp);
	(void) getc(fp);
	rewind(fp);
	if (getc(fp) != 'a')
	{
		xx_rpt(FAILURE);
		in_rpt("rewind(stream) did not set position at start of file");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private void
test2()
{
	FILE	*fp;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test2");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) getc(fp);
	(void) getc(fp);
	(void) getc(fp);
	(void) ungetc('z', fp);
	rewind(fp);
	if (getc(fp) == 'z')
	{	xx_rpt(FAILURE);
		in_rpt("rewind() did not undo the effects of ungetc()");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test3()
{
	FILE 	*fp;
	int	err;
	int	fail = 0;
	int	pathok = 0;

	DBUG_ENTER("test3");
	errno = NOERROR;

	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s,\"r\") failed, errno %d(%s)", t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fseek(fp, 0L, SEEK_END);
	(void) getc(fp);

	if (!feof(fp))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: after fseek(fp, 0L, SEEK_END) and call to getc(fp), EOF indicator not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rewind(fp);

	if (feof(fp))
		fail |= 1;

	if (close(fileno(fp)) != 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: close(fileno(fp)) failed - errno %d(%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) getc(fp);
	if(!ferror(fp))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason : error indicator not set after getc() on closed file");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rewind(fp);
	if(ferror(fp))
		fail |=2 ;

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("rewind(fp) failed to clear the EOF indicator");

		if (fail & 2)
			in_rpt("rewind(fp) failed to clear the error indicator");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	DBUG_VOID_RETURN;
}

private void
test4()
{
	FILE	*fp;
	char	*res;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test4");

	if ((fp = fopen(t4_file, "w")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t4_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
        if (putc('A', fp) == EOF)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("putc('A', fp) failed on file %s, errno %d(%s)",
			t4_file, err, errname(err));
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(t4_file);
	PATH_FUNC_TRACE;

	rewind(fp);

	globok = 0;
	if ((res = chktimes(t4_file, NEWMTIME | NEWCTIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(res);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(4);
	}

	(void) fclose(fp);
	(void) unlink(t4_file);
	DBUG_VOID_RETURN;
}
