/*
*      SCCS:  @(#)  ANSI.os/streamio/scanf/scanf.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/scanf - T.scanf Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.scanf

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the scanf() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec	 3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 1 July 87
			Removed	abandon(), testid()
			Modified xx_rpt(), xs_rpt()
		Geoff Clare, Oct 1988
			XPG3 Modifications
		
		Geoff Clare, 26 May 1989
			Split out "%k$" tests into scanf_X
		
		Geoff Clare, 31 May 1989
			Add EIO tests.

		Geoff Clare, 1 Aug 1989
			Remove tests with space in exponent ("e 12").
		
		Geoff Clare, 23 Aug 1989
			Add freopen() of stdin in do_ferr() tests.

		Steve Ratcliffe, 29 Jan 1990
			Add tests for long double type.

		Geoff Clare, 14 June 1990
			Do EAGAIN test on pipe and FIFO.

		Peter Lee, Unisoft Ltd.,
		Thu Jul 12 12:02:28 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed dltd() and reason() to tet_reason().

		Stuart Boutell, 1 Dec 1990
			Removed unneccesary fclose() at end of test44.
			Added code to ensure stdio stream open after
			end of tests 45-48.

		Matt Newman, Wed Nov 13 14:01:40 GMT 1991
			4.2 Wide Char support

		Neil Moses, 13 Dec 1991
			Added wcharstr() to obtain test wide char string.

		Geoff Clare, 2 June 1992
			Change %S and %C testing with field widths.
			
		Neil Moses, 10 June 1992
			New test 56.  On ROF time stamps are not changed.

		Geoff Clare, 16 June 1992
			Change printf() to in_rpt() in pseudo-language
			error messages.
		
		Geoff Clare, 22 June 1992
			In test_prep() ensure that t_data[tet_thistest]
			points to data argument.

                John-Paul, 27 July 1992
                        mode variables changed from 'int' to 'mode_t'.
		
		Geoff Clare, 9 Sept 1992
			Change newline to space in test 44 input string.
		
		Geoff Clare, 22 Oct 1992
			Change 0.0L to 0.0 in test 46.

		John-Paul, 1 November 1992
			XNFS enhancements.

		Geoff Clare, 29 Oct 1993
			Use new file in XNFS st_atime test to avoid
			reading cached data.

		Geoff Clare, 24 June 1994
			Changed format strings in tests 38, 41, 44 and 45
			to avoid using ranges with "%[".

		Geoff Clare, 25 Oct 1994
			Avoid freopen() of closed stream in test_prep().

		Geoff Clare, 30 March 1995
			New encodings for VSX4L1 and VSX4L2.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<locale.h>
#include	<nlsinfo.h>
#include	<nlstest.h>
#if defined(_XOPEN_SOURCE) && TEST_XPG > 3 && !TEST_XNFS
#include	<wchar.h>
#endif
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<chktimes.h>
#include	<sys/stat.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"scanf_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fscanf
#undef scanf
#undef sscanf
#endif


#define NO_TESTS	58		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define BSIZ		64
#define BUFFERSIZE	512
#define NULLTEXT	(char **) 0

#define I_SCANF		0
#define I_FSCANF	1
#define I_SSCANF	2

#define T_NUM		12
#define T13_NUM		54

/* Use " _ " to separate SCANIT arguments */
#define	_ ,

#define	SCANIT(args)  \
	switch(interface)\
	{\
	case I_SCANF:\
		tretval = scanf(testfmt, args);\
		rewind(t_fp);\
		break;\
	case I_FSCANF:\
		tretval = fscanf(t_fp, testfmt, args);\
		rewind(t_fp);\
		break;\
	case I_SSCANF:\
		tretval = sscanf(t_data[tet_thistest], testfmt, args);\
		break;\
	};

extern	int	fcomp();
extern	int	dcomp();
#if	LONG_DOUBLE_SUPPORTED
extern	int	ldcomp();
#endif
extern	char	*errname();
extern	char	*execpath();
extern	char	*newstr();
extern	char	*psl_to_asc();
extern	char	*pslanginfo();
extern	char	*pr_char();
extern	char	*itoa();
extern	void	do_feio();
extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

#if defined(_XOPEN_SOURCE) && TEST_XPG > 3 && !TEST_XNFS
extern	char	*pr_wchar();
extern  char	*pr_wstr();
#endif

extern	char	*vg_ident;

int	no_tests = NO_TESTS;

#if TEST_XNFS
private	int	snames[] = {1, 0};
#endif /* TEST_XNFS */

private char	*t_file =	"./scanf-t";
private	char	*prog_su =	"scanf_su";
#if TEST_XNFS
private char	*t49_file =	T49_FILE;
#else
#define		t49_file	t_file
#endif
private char	*t56_dir =	"scanf-d.56";
private char	*t56_file =	"scanf-d.56/scanf-t.56";
#if TEST_XNFS
private char	*t57_file =	T57_FILE;
private char	*t58_file =	T58_FILE;
#endif /* TEST_XNFS */
private FILE	*t_fp =		(FILE *) NULL;
private int	interface = 0;
private int	setupok = -1;
private int	tretval;
private	char	*currlang = "";
#if !TEST_XNFS
private	char	*sigstat;
#endif /* TEST_XNFS */

private	int	testfail;
private char	resbuf[BUFFERSIZE];

#if defined(_XOPEN_SOURCE) && TEST_XPG > 3 && !TEST_XNFS
private	char	*locale;
private	char	mbstr[BUFFERSIZE];
private	char	wexpstr[BUFFERSIZE];
private	wchar_t	targ[BUFFERSIZE];
private	wchar_t	*wcstr;
private int	wlen;
private int	wflag;
#endif

private char	*testfmt;
#if !TEST_XNFS
private char	*float_2_fmt = "%e%e%e%e%e%e";
private char	*float_3_fmt = "%g%g%g%g%g%g";
/* (The backslashes are to protect from SCCS) */
private char	*float_4_fmt = "%E%E%E%E%E%E";
private char	*float_5_fmt = "%G%G%G%G%G%G";
#endif /* TEST_XNFS */
#if TEST_XPG == 3
private char	*x1_fmt = "%[f-a]";
private char	*x2_fmt = "%[^n-l]";
#endif
#if !TEST_XNFS
private char	*x3_fmt = "%[cdabef-]";
#endif /* TEST_XNFS */

private char	*t_fmt[NO_TESTS + 1] = { "",
				"%d %d %d",
				"%d %f %s",
				"%d %d %d %d %d",
				"%d %d",
				"%d%d",
	/* test 6 */		"%d%d%c%[^1]",
				"%dabcd%defgh%d",
				"%%%d",
				"%d%d",
				"%u",
				"%o",
				"%x%X",
				"%i%i%i%i",
				"%f%f%f%f%f%f",
				"%f%f%f%f%f%f",
				"%f%f%f%f%f%f",
				"%f%f%f%f%f%f",
				"%f%f%f%f%f%f",
				"%f%f%f%f%f%f",
				"%f%f%f%f%f%f",
				"%s",
				"%s",
				"%c",
	/* test 24 */		"%c",
				"%S",
				"%S",
				"%C",
				"%C",
	/* test 29 */		"%p",
				"%s%n%s",
				"%[a-f]",
				"%[fbcad]",
				"%[^ml]",
				"%[^e-i]",
				"%[-fcadb]",
				"%[]cabed]",
				"%[^]ced]",
				"%[def]",
				"%ld%li%lo%lu%lx%lX%lf%le%lE%lg%lG%ln",
				"%hd%hi%ho%hu%hx%hX%hn",
				"%5d%5i%6o%5u%4x%4X%6f%9e%9E%9g%9G%3[cdef]%5s",
	/* test 42 */		"%3c",
				"",
	/* test 44 */		"%*d%*i%*o%*u%*x%*X%*f%*e%*E%*g%*G%*c%*[cdef]%*s%*p%d",
				"%*5d%*5i%*6o%*5u%*4x%*4X%*6f%*9e%*9E%*9g%*9G%*4[cdef]%*11s%d",
				"%Lf%Le%LE%Lg%LG",
				"%e%f%g",
				"%e%f%g",
				"%s",
  };


private char	*t_data[NO_TESTS + 1] = { "",
				" 25 26         27 28 29 30",
				"30 54.1 vsx",
				"30 30 30",
				"12 a",
				"     ",
		/* test 6 */	"10\n\t\f 11\n\t\f 1",
				"10abcd11abcd12",
				"%10",
				"   11   12",
				"  54321",
				"    76543",
				"  56cE 5Ba0",
				"123 0123 0x123 0X123",
				"123 +123 -123 123.123 +123.123 -123.123",
				"123e12 +123e12 -123e12 123E12 +123E12 -123E12",
				"123.123e12 +123.123e12 -123.123e12 123.123E12 +123.123E12 -123.123E12",
				"123e+12 +123e+12 -123e+12 123E+12 +123E+12 -123E+12",
				"123.123e+12 +123.123e+12 -123.123e+12 123.123E+12 +123.123E+12 -123.123E+12",
				"123e-12 +123e-12 -123e-12 123E-12 +123E-12 -123E-12",
				"123.123e-12 +123.123e-12\n-123.123e-12 123.123E-12 +123.123E-12 -123.123E-12",
				"       test-string not-included",
				"  test-string    ",
				"test char",
		/* test 24 */	"   test char",
				"",
				"",
				"",
				"",
		/* test 29 */	"",
				" 2345  abcde",
				"abcdefghij-klmnopq",
				"abcdefghij-klmnopq",
				"abcdefghij-klmnopq",
				"abcdefghij-klmnopq",
				"a-bcdefghij-klmnopq",
				"ab]cd]efghij-klmnopq",
				"ab]cd]efghij-klmnopq",
				"abcdefghi",
				" 1234567890 1234567890 011145401322 1234567890 499602d2 499602d2 123456.123456\n1.23e+22 1.23e+22 1.23e+22 1.23e+22",
				"12345 12345 030071 12345 3039 3039",
				" 12345123450300711234530393039456.1211.23e+0021.23e+0031.23e+0041.23e+00cdeftest-string",
		/* test 42 */	"abcdefghi",
				"",
		/* test 44 */	"",
				"12345123450300711234530393039456.1211.23e+0021.23e+0031.23e+0041.23e+00cdeftest-string99",
				"123.123 1.23e+22 1.23e+22 1.23e+22 1.23e+22",
				"",
				"",
				"dummystring",
  };
#if !TEST_XNFS
private float	floatvals[9][6] ={
					{
					123.0, 123.0, -123.0, 
					123.123, 123.123, -123.123
					},
					{
					123e12, 123e12, -123e12,
					123e12, 123e12, -123e12
					},
					{
					123.123e12, 123.123e12, -123.123e12, 
					123.123e12, 123.123e12, -123.123e12 
					},
					{
					123e12, 123e12, -123e12,
					123e12, 123e12, -123e12 
					},
					{
					123.123e12, 123.123e12, -123.123e12, 
					123.123e12, 123.123e12, -123.123e12 
					},
					{
					123e-12, 123e-12, -123e-12, 
					123e-12, 123e-12, -123e-12 
					},
					{
					123.123e-12, 123.123e-12, -123.123e-12,
					123.123e-12, 123.123e-12, -123.123e-12
					}
				};
#endif /* TEST_XNFS */

#define STDIN		(interface == I_FSCANF ? 0 : \
			interface == I_SCANF ? 1 : 0 )

private void	prepare_tests();
private void	cln_tests();
private int	test_prep();
private void	notinuse();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
#if !TEST_XNFS
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14_20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();
private void	test30();
private void	test31();
private void	test32();
private void	test33();
private void	test34();
private void	test35();
private void	test36();
private void	test37();
private void	test38();
private void	test39();
private void	test40();
private void	test41();
private void	test42();
private void	test43();
private void	test44();
private void	test45();
private void	test46();
private void	test47();
private void	test48();
#endif /* TEST_XNFS */
private void	test49();
#if !TEST_XNFS
private void	test50_53();
private void	test54();
private void	test55();
#endif /* TEST_XNFS */
private void	test56();
#if TEST_XNFS
private void	test57();
private void	test58();
#endif /* TEST_XNFS */

#if !TEST_XNFS
private void	ch_t6();
private void	ch_t25();
private void	ch_t26();
private void	ch_t27();
private void	ch_t28();
private void	ch_t43();
private void	ch_t44();
private void	ch_t45();
private void	ch_t47();
private void	ch_t48();
#endif /* TEST_XNFS */
private void	ch_t56();
#if TEST_XNFS
private void	ch_t57();
#endif /* TEST_XNFS */
#if !TEST_XNFS
private int	t50_53func();
private int	t54_55io();
private void	t54rpt(), t55rpt();
#endif /* TEST_XNFS */
private int	t58_func();
private int	t58_chk_func();
private void	printres();
#if defined(_XOPEN_SOURCE) && TEST_XPG > 3 && !TEST_XNFS
private void	wclocale();
#endif

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;

#if !TEST_XNFS

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14_20, 14,
	 test14_20, 15,
	 test14_20, 16,
	 test14_20, 17,
	 test14_20, 18,
	 test14_20, 19,
	 test14_20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	 test29, 29,
	 test30, 30,
	 test31, 31,
	 test32, 32,
	 test33, 33,
	 test34, 34,
	 test35, 35,
	 test36, 36,
	 test37, 37,
	 test38, 38,
	 test39, 39,
	 test40, 40,
	 test41, 41,
	 test42, 42,
	 test43, 43,
	 test44, 44,
	 test45, 45,
	 test46, 46,
	 test47, 47,
	 test48, 48,
	 test49, 49,
	 test50_53, 50,
	 test50_53, 51,
	 test50_53, 52,
	 test50_53, 53,
	 test54, 54,
	 test55, 55,
	 test56, 56,
	 notinuse, 57,
	 notinuse, 58,
	NULL, 0
};

#else /* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 notinuse, 6,
	 notinuse, 7,
	 notinuse, 8,
	 notinuse, 9,
	 notinuse, 10,
	 notinuse, 11,
	 notinuse, 12,
	 notinuse, 13,
	 notinuse, 14,
	 notinuse, 15,
	 notinuse, 16,
	 notinuse, 17,
	 notinuse, 18,
	 notinuse, 19,
	 notinuse, 20,
	 notinuse, 21,
	 notinuse, 22,
	 notinuse, 23,
	 notinuse, 24,
	 notinuse, 25,
	 notinuse, 26,
	 notinuse, 27,
	 notinuse, 28,
	 notinuse, 29,
	 notinuse, 30,
	 notinuse, 31,
	 notinuse, 32,
	 notinuse, 33,
	 notinuse, 34,
	 notinuse, 35,
	 notinuse, 36,
	 notinuse, 37,
	 notinuse, 38,
	 notinuse, 39,
	 notinuse, 40,
	 notinuse, 41,
	 notinuse, 42,
	 notinuse, 43,
	 notinuse, 44,
	 notinuse, 45,
	 notinuse, 46,
	 notinuse, 47,
	 notinuse, 48,
	 test49, 49,
	 notinuse, 50,
	 notinuse, 51,
	 notinuse, 52,
	 notinuse, 53,
	 notinuse, 54,
	 notinuse, 55,
	 test56, 56,
	 test57, 57,
	 test58, 58,
	NULL, 0
};

#endif /* TEST_XNFS */

private	void
prepare_tests()
{
	int	pathok = 0;
	char	*s;
	int	i;
#if defined(_XOPEN_SOURCE) && TEST_XPG > 3 && !TEST_XNFS
	int	j, err;
	wchar_t	tbuf[BUFFERSIZE];
	static	char	msg[BUFFERSIZE];
	static	int	wtests[] = {6,25,26,27,28,43,44,45,0};
	char *ptr;
	char current[50];
#endif

	DBUG_ENTER("prepare_tests");

	if((s=strrchr(vg_ident,'/'))==NULL)
		s=vg_ident+2;
	else
		s+=3;
	if (strcmp(s, "scanf") == 0)
		interface = I_SCANF;
	else if (strcmp(s, "fscanf") == 0)
		interface = I_FSCANF;
	else if (strcmp(s, "sscanf") == 0)
		interface = I_SSCANF;
	else {
		for (i=1; i<=NO_TESTS; i++)
			(void) tet_delete(i, "bad ident parameter");
	}

#if defined(_XOPEN_SOURCE) && TEST_XPG > 3 && !TEST_XNFS
	errno = NOERROR;
	if(wcharstr(&locale, &wcstr) != 0)
	{
		err = errno;
		(void) sprintf(msg, "wcharstr() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			(void) tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;

	if((ptr = setlocale(LC_CTYPE, NULL)) == (char *)NULL)
	{
		err = errno;
		(void) sprintf(msg, "setlocale() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			(void) tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;

	(void) strcpy(current, ptr);

	if(setlocale(LC_CTYPE, locale) == (char *)NULL)
	{
		err = errno;
		(void) sprintf(msg, "setlocale() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			(void) tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;

	/*
	 * Strip out white space
	 */
	for(i = 0, j = 0; *(wcstr + i) != (wchar_t) '\0'; i++)
	{
		if(i == 0)
			PATH_TRACE;

		if(iswspace((wint_t) *(wcstr + i)))
			continue;

		tbuf[j++] = *(wcstr + i);
	}

	(void) wcsncpy(wcstr, tbuf, (size_t)j);
	if (j == 1)
	{
		wcstr[1] = tbuf[0];
		j++;
	}
	*(wcstr + j) = (wchar_t) '\0';
	wlen = j;

	if(wcstombs(mbstr, wcstr, BUFFERSIZE) == (size_t)-1)
	{
		err = errno;
		(void) sprintf(msg, "wcstombs() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			(void) tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;

	/* Restore the locale */
	if(setlocale(LC_CTYPE, current) == NULL)
	{
		err = errno;
		(void) sprintf(msg, "setlocale() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			(void) tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;

	wflag = 0;
#else /* TEST_XNFS */
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif /* TEST_XNFS */

	PATH_SETUP_RPT(6);

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");
	(void) unlink(t_file);
	DBUG_VOID_RETURN;
}


private int
test_prep(fmt,data)
	char *fmt,*data;
{
	int		err;
	int		fd;
	int		wval;
	int		len;
	int		ok = SUCCEED;
	int		pathok = 0;

	DBUG_ENTER("test_prep");

	if ((fd = creat(t_file, (mode_t)MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
		DBUG_RETURN(FAIL);
	}
	else
		PATH_TRACE;

	wval = write(fd, data, (unsigned) (len = strlen(data)));
	(void) close(fd);
	if (wval != len)
	{
		err = errno;
		xx_rpt(DELETION);
		if (wval == SYSERROR)
			in_rpt("deletion reason: write() failed - errno %d(%s)",
				err, errname(err));
		else
			in_rpt("deletion reason: write() wrote only %d buytes of %d",
				wval, len);
		ok = FAIL;
	}
	else if (interface == I_SCANF)
	{
		if (freopen(t_file, "r", stdin) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: freopen(%s, \"r\", stdin) ailed - errno %d(%s)",
				t_file, err, errname(err));
			ok = FAIL;
		}
		else
			PATH_TRACE;
		t_fp = stdin;
	}
	else if (interface == I_FSCANF)
	{
		if (t_fp != (FILE *) NULL)
			(void) fclose(t_fp);
		if ((t_fp = fopen(t_file, "r")) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fopen(%s, \"r\") ailed - errno %d(%s)",
				t_file, err, errname(err));
			ok = FAIL;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	testfmt = fmt;

	/* ensure data string is correct for sscanf() */
	if (t_data[tet_thistest] != data)
		t_data[tet_thistest] = data;

	PATH_FUNC_RPT(2);
	DBUG_RETURN(ok);
}

private void
notinuse()
{
	DBUG_ENTER("notinuse");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

private void
test1()
{
	int		i1, i2, i3;
	char	*expres = "return value = 3";
	int	pathok = setupok;

	DBUG_ENTER("test1");

	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = i3 = 0;

	testfail = 0;

	SCANIT(&i1 _ &i2 _ &i3);

	if (tretval != 3)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	int		i;
	float		f;
	char		s[BSIZ];
	char	*expres = "30, 54.1, \"vsx\"";
	int	pathok = setupok;

	DBUG_ENTER("test2");

	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i = 0;
	f = 0.0;
	s[0] = NUL;

	testfail = 0;

	SCANIT(&i _ &f _ s);

	s[BSIZ - 1] = NUL;
	if (i != 30 || !fcomp(f, 54.1) || strcmp(s, "vsx"))
	{
		(void) sprintf(resbuf, "%d, %f, \"%s\"", i, f, s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
		for (i = 3 ; i <= NO_TESTS ; i++)
			(void) tet_delete(i, "depends on success of test 2");
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test3()
{
	int		fail = 0;
	int		i1, i2, i3, i4, i5;
	char	*expres = "30, 30, 30, 12, 12";
	int	pathok = setupok;

	DBUG_ENTER("test3");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = i3 = 0;

	i4 = i5 = T_NUM;

	testfail = 0;

	SCANIT(&i1 _ &i2 _ &i3 _ &i4 _ &i5);

	if (i1 != 30 || i2 != 30 || i3 != 30)
		fail++;
	else
		PATH_TRACE;
	if (i4 != T_NUM || i5 != T_NUM)
		fail++;
	else
		PATH_TRACE;

	if (tretval != 3 || fail)
	{
		(void) sprintf(resbuf, "%d, %d, %d, %d, %d", i1, i2, i3, i4, i5);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


private void
test4()
{
	int		fail = 0;
	int		i1, i2;
	char	*exp1res = "unread character = 'a'";
	char	*exp2res = "return value = 1";
	int	pathok = setupok;

	DBUG_ENTER("test4");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = 0;

	testfail = 0;

	switch (interface)
	{
	case  I_SCANF:
		tretval = scanf(testfmt, &i1, &i2);
		if ((tretval = getchar()) != 'a')
			fail = 1;
		else
			PATH_TRACE;
		break;

	case I_FSCANF:
		tretval = fscanf(t_fp, testfmt, &i1, &i2);
		if ((tretval = getc(t_fp)) != 'a')
			fail = 1;
		else
			PATH_TRACE;
		break;

	case I_SSCANF:
		tretval = sscanf(t_data[tet_thistest], testfmt, &i1, &i2);
		if (tretval != 1)
			fail = 1;
		else
			PATH_TRACE;
	}

	if (fail)
	{
		if (interface == I_SSCANF)
		{
			(void) sprintf(resbuf, "return value = %d", tretval);
			printres(t_data[tet_thistest], testfmt, exp2res, resbuf, NULLTEXT);
		}
		else
		{
			(void) sprintf(resbuf, "unread character = '%c'", tretval);
			printres(t_data[tet_thistest], testfmt, exp1res, resbuf, NULLTEXT);
		}
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


private void
test5()
{
	int		i1, i2;
	char	*expres = "return value = EOF";
	int	pathok = setupok;

	DBUG_ENTER("test5");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = 0;

	testfail = 0;

	SCANIT(&i1 _ &i2);

	if (tretval != EOF)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

private void
test6()
{
	DBUG_ENTER("test6");

	(void) cppair(ch_t6, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	int		i1, i2;
	char		c1;
	char		s1[BSIZ];
	char	*scanstr = "10\\n\\t\\f 11\\n\\t\\f 1";
	char	*expres = "10, 11, \\n, \"\\t\\f \"";
	int	pathok = setupok;
#if defined(_XOPEN_SOURCE) && TEST_XPG > 3 
	char	*wfmt = "%d%d%C%[^1]";
	wchar_t	wc;
#endif

	DBUG_ENTER("ch_t6");

	globok = 0;
	if(test_prep(t_fmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = 0; c1 = NUL; s1[0] = NUL; testfail = 0;

	SCANIT(&i1 _ &i2 _ &c1 _ s1);

	s1[BSIZ - 1] = NUL;
	if(i1 != 10 || i2 != 11 || c1 != '\n' || strcmp(s1, "\t\f "))
	{
		(void) sprintf(resbuf, 
			"%d, %d, %s, \"%s\"", i1, i2, pr_char(c1), s1);
		printres(scanstr, testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 

	if(test_prep(wfmt, t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = 0; wc = (wchar_t)NUL; s1[0] = NUL;

	SCANIT(&i1 _ &i2 _ &wc _ s1);

	s1[BSIZ - 1] = NUL;
	if(i1 != 10 || i2 != 11 || wc != (wchar_t)'\n' || strcmp(s1, "\t\f "))
	{
		(void) sprintf(resbuf, "%d, %d, %s, \"%s\"", i1, i2, pr_wchar(wc), s1);
		printres(scanstr, wfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);
#else
	if (!testfail)
		PATH_XS_RPT(2);
#endif

	DBUG_VOID_RETURN;
}

private void
test7()
{
	int		i1, i2, i3;
	char	*expres = "return value = 2";
	int	pathok = setupok;

	DBUG_ENTER("test7");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = i3 = 0;

	testfail = 0;

	SCANIT(&i1 _ &i2 _ &i3);

	if (tretval != 2)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test8()
{
	int		i;
	char	*expres = "10";
	int	pathok = setupok;

	DBUG_ENTER("test8");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	i = 0;

	testfail = 0;

	SCANIT(&i);

	if (i != 10)
	{
		(void) sprintf(resbuf, "%d", i);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test9()
{
	int		i1, i2;
	char	*expres = "11, 12";
	int	pathok = setupok;


	DBUG_ENTER("test9");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	i1 = i2 = 0;
	
	testfail = 0;

	SCANIT(&i1 _ &i2);

	if (i1 != 11 || i2 != 12)
	{
		(void) sprintf(resbuf, "%d, %d", i1, i2);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test10()
{
	unsigned int	u;
	char	*expres = "54321";
	int	pathok = setupok;

	DBUG_ENTER("test10");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	u = 0;

	testfail = 0;

	SCANIT(&u);

	if (u != (unsigned) 54321)
	{
		(void) sprintf(resbuf, "%u", u);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test11()
{
	int		i;
	char	*expres = "32099";
	int	pathok = setupok;

	DBUG_ENTER("test11");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i = 0;

	testfail = 0;

	SCANIT(&i);

	if (i != 32099)
	{
		(void) sprintf(resbuf, "%d", i);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test12()
{
	int	i1, i2;
	char	*expres = "22222, 23456";
	int	pathok = setupok;

	DBUG_ENTER("test12");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = 0;

	testfail = 0;

	SCANIT(&i1 _ &i2);

	if (i1 != 22222 || i2 != 23456)
	{
		(void) sprintf(resbuf, "%d, %d", i1, i2);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test13()
{
	int	i1, i2, i3, i4;
	char	*expres = "123, 83, 291, 291";
	int	pathok = setupok;

	DBUG_ENTER("test13");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = i3 = i4 = 0;

	testfail = 0;

	SCANIT(&i1 _ &i2 _ &i3 _ &i4);

	if (i1 != 123 || i2 != 83 || i3 != 291 || i4 != 291)
	{
		(void) sprintf(resbuf, "%d, %d, %d, %d", i1, i2, i3, i4);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test14_20()
{
	float	f[10]; /* extra space is in case scanf() scribbles past end */
	int	i;
	int	val_no;
	char	expres[90];
	int	pathok = setupok;

	DBUG_ENTER("test14_20");

	val_no = tet_thistest - 14;
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	(void) sprintf(expres, "%e, %e, %e, %e, %e, %e",
		floatvals[val_no][0], floatvals[val_no][1],
		floatvals[val_no][2], floatvals[val_no][3],
		floatvals[val_no][4], floatvals[val_no][5]);

	testfail = 0;

	for (i=0; i<6; i++)
		f[i] = 0.0;
	SCANIT(&f[0] _ &f[1] _ &f[2] _ &f[3] _ &f[4] _ &f[5]);

	for (i = 0; i < 6; i++)
		if (!fcomp(f[i], floatvals[val_no][i]))
			break;

	if (i != 6)
	{
		(void) sprintf(resbuf, "%e, %e, %e, %e, %e, %e", 
			f[0], f[1], f[2], f[3], f[4], f[5]);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;


	testfmt = float_2_fmt;

	for (i=0; i<6; i++)
		f[i] = 0.0;
	SCANIT(&f[0] _ &f[1] _ &f[2] _ &f[3] _ &f[4] _ &f[5]);

	for (i = 0; i < 6; i++)
		if (!fcomp(f[i], floatvals[val_no][i]))
			break;
	if (i != 6)
	{
		(void) sprintf(resbuf, "%e, %e, %e, %e, %e, %e", 
			f[0], f[1], f[2], f[3], f[4], f[5]);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;


	testfmt = float_3_fmt;

	for (i=0; i<6; i++)
		f[i] = 0.0;
	SCANIT(&f[0] _ &f[1] _ &f[2] _ &f[3] _ &f[4] _ &f[5]);

	for (i = 0; i < 6; i++)
		if (!fcomp(f[i], floatvals[val_no][i]))
			break;

	if (i != 6)
	{
		(void) sprintf(resbuf, "%e, %e, %e, %e, %e, %e", 
			f[0], f[1], f[2], f[3], f[4], f[5]);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;


	testfmt = float_4_fmt;

	for (i=0; i<6; i++)
		f[i] = 0.0;
	SCANIT(&f[0] _ &f[1] _ &f[2] _ &f[3] _ &f[4] _ &f[5]);

	for (i = 0; i < 6; i++)
		if (!fcomp(f[i], floatvals[val_no][i]))
			break;
	if (i != 6)
	{
		(void) sprintf(resbuf, "%e, %e, %e, %e, %e, %e", 
			f[0], f[1], f[2], f[3], f[4], f[5]);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;


	testfmt = float_5_fmt;

	for (i=0; i<6; i++)
		f[i] = 0.0;
	SCANIT(&f[0] _ &f[1] _ &f[2] _ &f[3] _ &f[4] _ &f[5]);

	for (i = 0; i < 6; i++)
		if (!fcomp(f[i], floatvals[val_no][i]))
			break;

	if (i != 6)
	{
		(void) sprintf(resbuf, "%e, %e, %e, %e, %e, %e", 
			f[0], f[1], f[2], f[3], f[4], f[5]);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test21()
{
	char		s[BSIZ];
	char	*expres = "\"test-string\"";
	int	pathok = setupok;

	DBUG_ENTER("test21");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	s[0] = NUL;

	testfail = 0;

	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (strcmp(s, "test-string"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test22()
{
	char		s[BSIZ];
	char	*expres = "\"test-string\"";
	int	pathok = setupok;

	DBUG_ENTER("test22");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	s[0] = NUL;

	testfail = 0;

	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (strcmp(s, "test-string"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test23()
{
	char		c;
	char	*expres = "'t'";
	int	pathok = setupok;

	DBUG_ENTER("test23");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	c = NUL;

	testfail = 0;

	SCANIT(&c);

	if (c != 't')
	{
		(void) sprintf(resbuf, "%c(%s)", c, pr_char(c));
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test24()
{
	char		c;
	char	*expres = " ";
	int	pathok = setupok;

	DBUG_ENTER("test24");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	c = NUL;

	testfail = 0;

	SCANIT(&c);

	if (c != ' ')
	{
		(void) sprintf(resbuf, "%c(%s)", c, pr_char(c));
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test25()
{
	DBUG_ENTER("test25");

	(void) cppair(ch_t25, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t25()
{
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 
	int	pathok = setupok;

	DBUG_ENTER("ch_t25");

	globok = 0;
	if (test_prep(t_fmt[tet_thistest], mbstr) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	targ[0] = (wchar_t)NUL;

	testfail = 0;

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	SCANIT(targ);

	if(wcscmp(wcstr, targ))
	{
		(void) strcpy(resbuf, pr_wstr(targ));
		(void) strcpy(wexpstr, pr_wstr(wcstr));
		printres(mbstr, testfmt, wexpstr, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}

private void
test26()
{
	DBUG_ENTER("test26");

	(void) cppair(ch_t26, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t26()
{
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 
	int	pathok = setupok;

	DBUG_ENTER("ch_t26");

	globok = 0;
	if (test_prep(t_fmt[tet_thistest], mbstr) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	targ[0] = (wchar_t)NUL;
	targ[wlen] = (wchar_t)'a';
	testfail = 0;

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	SCANIT(targ);

	if(wcscmp(wcstr, targ) || targ[wlen] != (wchar_t)NUL)
	{
		if(wcscmp(wcstr, targ))
		{
			(void) strcpy(resbuf, pr_wstr(targ));
			(void) strcpy(wexpstr, pr_wstr(wcstr));
			printres(mbstr, testfmt, wexpstr, resbuf, NULLTEXT);
		}

		if(targ[wlen] != (wchar_t)NUL)
		{
			if(!testfail++)
				xx_rpt(FAILURE);
			in_rpt("Expected terminating null wide character");
			in_rpt("Actual terminating character was >%s<", 
				pr_wchar(targ[wlen]));
		}
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}

private void
test27()
{
	DBUG_ENTER("test27");

	(void) cppair(ch_t27, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t27()
{
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 
	wchar_t	wc;
	char	tbuf[50];
	int	i, j, err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t27");

	errno = NOERROR;

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';

		globok = 0;
		if (test_prep(t_fmt[tet_thistest], tbuf) == FAIL)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	
		wc = (wchar_t)NUL;
		testfail = 0;
	
		SCANIT(&wc);
	
		if(wc != *(wcstr + i))
		{
			(void) strcpy(resbuf, pr_wchar(wc));
			(void) strcpy(wexpstr, pr_wchar(*(wcstr + i)));
			printres(tbuf, testfmt, wexpstr, resbuf, NULLTEXT);
		}
		else
			PATH_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(1 + i * 2);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}

private void
test28()
{
	DBUG_ENTER("test28");

	(void) cppair(ch_t28, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t28()
{
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 
	static	wchar_t	wbuf[] = { '\n', '\t', '\r', '\f', ' ', 0 };
	wchar_t	wc;
	char	tbuf[50];
	int	i, j, err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t28");

	errno = NOERROR;

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	for(i = 0; *(wbuf + i) != (wchar_t)NUL; i++)
	{
		if((j = wctomb(tbuf, *(wbuf + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';

		globok = 0;
		if (test_prep(t_fmt[tet_thistest], tbuf) == FAIL)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	
		wc = (wchar_t)NUL;
		testfail = 0;
	
		SCANIT(&wc);
	
		if(wc != *(wbuf + i))
		{
			(void) strcpy(resbuf, pr_wchar(wc));
			(void) strcpy(wexpstr, pr_wchar(*(wbuf + i)));
			printres(tbuf, testfmt, wexpstr, resbuf, NULLTEXT);
		}
		else
			PATH_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(1 + i * 2);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}

private void
test29()
{
	int		i;
	int		*p;
	char	expres[50], testdata[50];
	int	pathok = setupok;

	DBUG_ENTER("test29");

	(void) sprintf(testdata, "%p", (void *)&i);
	t_data[tet_thistest] = testdata;

	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i = 0;

	testfail = 0;

	p = 0;
	SCANIT((void **)&p);

	if (p != &i)
	{
		(void) sprintf(expres, "%p", (void *)&i);
		(void) sprintf(resbuf, "%p", (void *)p);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test30()
{
	char	s1[BSIZ], s2[BSIZ];
	int	i;
	char	*expres = "2345, 5, abcde";
	int	pathok = setupok;

	DBUG_ENTER("test30");

	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	s1[0] = s2[0] = NUL;
	i = 0;

	testfail = 0;

	SCANIT(s1 _ &i _ s2);

	s1[BSIZ - 1] = s2[BSIZ - 1] = NUL;
	if (i != 5 || strcmp(s1, "2345") != 0 || strcmp(s2, "abcde") != 0)
	{
		(void) sprintf(resbuf, "%s, %d, %s", s1, i, s2);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (tretval != 2)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, "return value = 2", resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


private void
test31()
{
#if TEST_XPG == 3
	char		s1[BSIZ];
	char		s2[BSIZ];
	char	*exp1res = "\"abcdef\"";
	char	*exp2res = "\"a\"";
	int	pathok = setupok;

	DBUG_ENTER("test31");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	s1[0] = s2[0] = NUL;

	testfail = 0;

	SCANIT(s1);

	if (strcmp(s1, "abcdef"))
	{
		(void) sprintf(resbuf, "\"%s\"", s1);
		printres(t_data[tet_thistest], testfmt, exp1res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	testfmt = x1_fmt;
	SCANIT(s2);

	s1[BSIZ - 1] = s2[BSIZ - 1] = NUL;
	if (strcmp(s2, "a"))
	{
		(void) sprintf(resbuf, "\"%s\"", s2);
		printres(t_data[tet_thistest], testfmt, exp2res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}


private void
test32()
{
	char		s[BSIZ];
	char	*expres = "\"abcd\"";
	int	pathok = setupok;

	DBUG_ENTER("test32");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	s[0] = NUL;

	testfail = 0;

	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (strcmp(s, "abcd"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test33()
{
	char		s[BSIZ];
	char	*expres = "\"abcdefghij-k\"";
	int	pathok = setupok;

	DBUG_ENTER("test33");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	s[0] = NUL;

	testfail = 0;

	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (strcmp(s, "abcdefghij-k"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test34()
{
#if TEST_XPG == 3
	char		s2[BSIZ];
	char		s1[BSIZ];
	char	*exp1res = "\"abcd\"";
	char	*exp2res = "\"abcdefghij\"";
	int	pathok = setupok;

	DBUG_ENTER("test34");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	s1[0] = s2[0] = NUL;

	testfail = 0;

	SCANIT(s1);

	if (strcmp(s1, "abcd"))
	{
		(void) sprintf(resbuf, "\"%s\"", s1);
		printres(t_data[tet_thistest], testfmt, exp1res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	testfmt = x2_fmt;
	SCANIT(s2);

	s1[BSIZ - 1] = s2[BSIZ - 1] = NUL;
	if (strcmp(s2, "abcdefghij"))
	{
		(void) sprintf(resbuf, "\"%s\"", s2);
		printres(t_data[tet_thistest], testfmt, exp2res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}


private void
test35()
{
	char		s1[BSIZ];
	char		s2[BSIZ];
	char	*exp1res = "\"a-bcd\"";
	char	*exp2res = "\"a-bcdef\"";
	int	pathok = setupok;

	DBUG_ENTER("test35");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	s1[0] = s2[0] = NUL;

	testfail = 0;

	SCANIT(s1);

	if (strcmp(s1, "a-bcd"))
	{
		(void) sprintf(resbuf, "\"%s\"", s1);
		printres(t_data[tet_thistest], testfmt, exp1res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;


	testfmt = x3_fmt;
	SCANIT(s2);

	s1[BSIZ - 1] = s2[BSIZ - 1] = NUL;
	if (strcmp(s2, "a-bcdef"))
	{
		(void) sprintf(resbuf, "\"%s\"", s2);
		printres(t_data[tet_thistest], testfmt, exp2res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


private void
test36()
{
	char		s[BSIZ];
	char	*expres = "\"ab]cd]e\"";
	int	pathok = setupok;

	DBUG_ENTER("test36");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	s[0] = NUL;

	testfail = 0;

	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (strcmp(s, "ab]cd]e"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test37()
{
	char		s[BSIZ];
	char	*expres = "\"ab\"";
	int	pathok = setupok;

	DBUG_ENTER("test37");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	s[0] = NUL;

	testfail = 0;

	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (strcmp(s, "ab"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test38()
{
	char		s[BSIZ];
	char	*expres = "return value = 0";
	int	pathok = setupok;

	DBUG_ENTER("test38");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	s[0] = NUL;

	testfail = 0;

	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (s[0] != NUL || tretval != 0)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test39()
{
	long		l1, l2, l3, l4, l5, n;
	unsigned long	u1;
	double		d1, d2, d3, d4, d5;
	int		expn;
	static char	expres[150] = "1234567890, 1234567890, 1234567890, 1234567890, 1234567890, 1234567890, 123456.123456, 1.23e22, 1.23e22, 1.23e22, 1.23e22, ";
	char		*exp2res = "return value = 11";
	int	pathok = setupok;

	DBUG_ENTER("test39");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	l1 = l2 = l3 = l4 = l5 = 0;
	u1 = 0;
	d1 = d2 = d3 = d4 = d5 = 0.0;
	n = 0;

	testfail = 0;

	DBUG_PRINTF("test35", "address for %%n = %p", (void *)&n);

	SCANIT(&l1 _ &l2 _ &l3 _ &u1 _ &l4 _ &l5 _ &d1 _ &d2 _ &d3 _ &d4 _ &d5 _ &n);

	expn = strlen(t_data[tet_thistest]);
	if (l1 != 1234567890 || l2 != 1234567890 || l3 != 1234567890
		|| u1 != (unsigned long) 1234567890
		|| l4 != 1234567890 || l5 != 1234567890
		|| !dcomp(d1, 123456.123456) || !dcomp(d2, 1.23e22)
		|| !dcomp(d3, 1.23e22) || !dcomp(d4, 1.23e22)
		|| !dcomp(d5, 1.23e22) || n != expn)
	{
		(void) sprintf(resbuf, "%ld, %ld, %ld, %lu, %ld, %ld, %f, %g, %g, %g, %g, %ld", l1, l2, l3, u1, l4, l5, d1, d2, d3, d4, d5, n);
		(void) strcat(expres, itoa(expn));
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (tretval != 11)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, exp2res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test40()
{
	short		h1, h2, h3, h4, h5, n;
	unsigned short	u1;
	int		expn;
	static char	expres[60] = "12345, 12345, 12345, 12345, 12345, 12345, ";
	int	pathok = setupok;

	DBUG_ENTER("test40");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	expn = strlen(t_data[tet_thistest]);
	(void) strcat(expres, itoa(expn));

	h1 = h2 = h3 = h4 = h5 = 0;
	u1 = 0;
	n = 0;

	testfail = 0;

	SCANIT(&h1 _ &h2 _ &h3 _ &u1 _ &h4 _ &h5 _ &n);

	if (h1 != 12345 || h2 != 12345 || h3 != 12345 || u1 != 12345
		|| h4 != 12345 || h5 != 12345 || n != expn)
	{
		(void) sprintf(resbuf, "%d, %d, %d, %u, %d, %d, %d", h1, h2, h3, u1, h4, h5, n);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (tretval != 6)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, "return value = 6", resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test41()
{
	int		i1, i2, i3, i4, i5 ;
	unsigned int	u1;
	float		f1, f2, f3, f4, f5;
	char		s1[BSIZ], s2[BSIZ];
	char	*expres = "12345, 12345, 12345, 12345, 12345, 12345, 456.12, 11.23, 21.23, 31.23, 41.23, \"cde\", \"ftest\"";
	static char *warntext[] = {
	    "Warning:- this implementation does not correctly implement",
	    "the (n) format modifier for floating point numbers with exponent.",
	    NULL
	};
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 
	wchar_t	ws[BUFFERSIZE];
	char	wfmt[20];
#endif
	int	pathok = setupok;

	DBUG_ENTER("test41");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = i3 = i4 = i5 = 0;
	u1 = 0;
	f1 = f2 = f3 = f4 = f5 = 0.0;
	s1[0] = s2[0] = NUL;

	testfail = 0;

	SCANIT(&i1 _ &i2 _ &i3 _ &u1 _ &i4 _ &i5 _ &f1 _ &f2 _ &f3 _ &f4 _ &f5 _ s1 _ s2);

	s1[BSIZ - 1] = s2[BSIZ - 1] = NUL;
	if (i1 != 12345 || i2 != 12345 || i3 != 12345 || u1 != 12345
		|| i4 != 12345 || i5 != 12345
		|| !fcomp(f1, 456.12) || !fcomp(f2, 11.23)
		|| !fcomp(f3, 21.23) || !fcomp(f4, 31.23)
		|| !fcomp(f5, 41.23) || strcmp(s1, "cde")
		|| strcmp(s2, "ftest"))
	{
		(void) sprintf(resbuf, "%d, %d, %d, %u, %d, %d, %f, %f, %f, %f, %f, \"%s\", \"%s\"", i1, i2, i3, u1, i4, i5, f1, f2, f3, f4, f5, s1, s2);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, warntext);
	}
	else
		PATH_TRACE;

#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	(void) sprintf(wfmt, "%%%dS", wlen);

	globok = 0;
	if (test_prep(wfmt, mbstr) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	ws[0] = (wchar_t)NUL;
	testfail = 0;
	SCANIT(ws);
	ws[BUFFERSIZE-1] = (wchar_t)NUL;
	
	if(wcscmp(ws, wcstr))
	{
		(void) strcpy(resbuf, pr_wstr(ws));
		(void) strcpy(wexpstr, pr_wstr(wcstr));
		printres(mbstr, testfmt, wexpstr, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	(void) sprintf(wfmt, "%%%dS", wlen-1);

	globok = 0;
	if (test_prep(wfmt, mbstr) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	ws[0] = (wchar_t)NUL;
	testfail = 0;
	SCANIT(ws);
	ws[BUFFERSIZE-1] = (wchar_t)NUL;
	
	if(wcsncmp(ws, wcstr, (size_t)(wlen-1)))
	{
		wchar_t savch;

		(void) strcpy(resbuf, pr_wstr(ws));
		savch = wcstr[wlen-1];
		wcstr[wlen-1] = '\0';
		(void) strcpy(wexpstr, pr_wstr(wcstr));
		wcstr[wlen-1] = savch;
		printres(mbstr, testfmt, wexpstr, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
test42()
{
	char	s[BSIZ];
	char	*expres = "\"abc\"";
	int	pathok = setupok;

	DBUG_ENTER("test42");
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	s[0] = s[1] = s[2] = NUL;

	testfail = 0;

	SCANIT(s);

	s[3] = NUL;
	if (strcmp(s, "abc"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test43()
{
	DBUG_ENTER("test43");

	(void) cppair(ch_t43, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t43()
{
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 
	wchar_t	ws[BUFFERSIZE];
	char	wfmt[20];
	int	pathok = setupok;

	DBUG_ENTER("ch_t43");

	errno = NOERROR;

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	(void) sprintf(wfmt, "%%%dC", wlen);

	globok = 0;
	if (test_prep(wfmt, mbstr) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	ws[0] = (wchar_t)NUL;
	testfail = 0;
	SCANIT(ws);
	ws[wlen] = (wchar_t)NUL;
	
	if(wcscmp(ws, wcstr))
	{
		(void) strcpy(resbuf, pr_wstr(ws));
		(void) strcpy(wexpstr, pr_wstr(wcstr));
		printres(mbstr, testfmt, wexpstr, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	(void) sprintf(wfmt, "%%%dC", wlen-1);

	globok = 0;
	if (test_prep(wfmt, mbstr) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	ws[0] = (wchar_t)NUL;
	testfail = 0;
	SCANIT(ws);
	ws[wlen-1] = (wchar_t)NUL;
	
	if(wcsncmp(ws, wcstr, (size_t)(wlen-1)))
	{
		wchar_t savch;

		(void) strcpy(resbuf, pr_wstr(ws));
		savch = wcstr[wlen-1];
		wcstr[wlen-1] = '\0';
		(void) strcpy(wexpstr, pr_wstr(wcstr));
		wcstr[wlen-1] = savch;
		printres(mbstr, testfmt, wexpstr, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}

private void
test44()
{
	DBUG_ENTER("test44");

	(void) cppair(ch_t44, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t44()
{
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 
	wchar_t	ws[BUFFERSIZE];
	wchar_t	wc;
	char	tbuf[50];
	char	mbs[BUFFERSIZE];
	char	*wfmt1 = "%*S%d";
	char	*wfmt2 = "%*C%d";
	int	err, j, k;
#endif
	long		x;
	char		s[BSIZ];
	int		i;
	char	*expres = "99";
	static char	testdata[150] = "12345 12345 030071 12345 3039 3039 456.12 1.23e+12 1.23e+12 1.23e+12 1.23e+12acdeftest-string ";
	int	pathok = setupok;

	DBUG_ENTER("ch_t44");

	(void) sprintf(s, "%p 99", (void *)&x);
	(void) strcat(testdata, s);
	t_data[tet_thistest] = testdata;

	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i = 0;

	testfail = 0;

	SCANIT(&i _ &x _ &x _ &x _ &x _ &x _ &x _ &x _ &x _ &x _ &x _ s _ s _ (void **)&x _ &x);

	if (i != 99)
	{
		(void) sprintf(resbuf, "%d", i);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 

	errno = NOERROR;

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	(void) sprintf(mbs, "%s 99", mbstr);

	globok = 0;
	if(test_prep(wfmt1, mbs) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	i = 0;
	ws[0] = (wchar_t)NUL;
	SCANIT(&i _ ws);
	
	if(i != 99)
	{
		(void) sprintf(resbuf, "%d", i);
		printres(mbs, wfmt1, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
	{
		if((k = wctomb(tbuf, *(wcstr + j))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[k] = '\0';
		(void) strcat(tbuf, " 99");

		globok = 0;
		if(test_prep(wfmt2, tbuf) == FAIL)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	
		i = 0;
		wc = (wchar_t)NUL;
		SCANIT(&i _ &wc);
	
		if(i != 99)
		{
			(void) sprintf(resbuf, "%d", i);
			printres(tbuf, wfmt2, expres, resbuf, NULLTEXT);
		}
		else
			PATH_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(5 + j * 2);
#else
	if (!testfail)
		PATH_XS_RPT(2);
#endif

	DBUG_VOID_RETURN;
}

private void
test45()
{
	DBUG_ENTER("test45");

	(void) cppair(ch_t45, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t45()
{
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 
	wchar_t	ws[BUFFERSIZE];
	wchar_t	wc;
	char	tbuf[50];
	char	mbs[BUFFERSIZE];
	char	*wfmt1 = "%*33S%d";
	char	*wfmt2 = "%*1C%d";
	int	err, j, k;
#endif
	long		x;
	char		s[BSIZ];
	int		i;
	char	*expres = "99";
	int	pathok = setupok;

	DBUG_ENTER("ch_t45");

	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i = 0;

	testfail = 0;

	SCANIT(&i _ &x _ &x _ &x _ &x _ &x _ &x _ &x _ &x _ &x _ &x _ s _ s _ &x);

	if (i != 99)
	{
		(void) sprintf(resbuf, "%d", i);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 

	errno = NOERROR;

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	(void) sprintf(mbs, "%s 99", mbstr);

	globok = 0;
	if(test_prep(wfmt1, mbs) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	i = 0;
	ws[0] = (wchar_t)NUL;
	SCANIT(&i _ ws);
	
	if(i != 99)
	{
		(void) sprintf(resbuf, "%d", i);
		printres(mbs, wfmt1, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
	{
		if((k = wctomb(tbuf, *(wcstr + j))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[k] = '\0';
		(void) strcat(tbuf, " 99");

		globok = 0;
		if(test_prep(wfmt2, tbuf) == FAIL)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	
		i = 0;
		wc = (wchar_t)NUL;
		SCANIT(&i _ &wc);
	
		if(i != 99)
		{
			(void) sprintf(resbuf, "%d", i);
			printres(tbuf, wfmt2, expres, resbuf, NULLTEXT);
		}
		else
			PATH_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(5 + j * 2);
#else
	if (!testfail)
		PATH_XS_RPT(2);
#endif

	DBUG_VOID_RETURN;
}

private void
test46()
{
#if	LONG_DOUBLE_SUPPORTED

	long double	d1, d2, d3, d4, d5;
	static char	expres[] = "123.123, 1.23e+22, 1.23e+22, 1.23e+22, 1.23e+22";
	int	pathok = setupok;

	DBUG_ENTER("test46");

	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	d1 = d2 = d3 = d4 = d5 = 0.0;

	testfail = 0;

	SCANIT(&d1 _ &d2 _ &d3 _ &d4 _ &d5);

	if (!ldcomp(d1, 123.123L) || !ldcomp(d2, 1.23e22L)
		|| !ldcomp(d3, 1.23e22L) || !ldcomp(d4, 1.23e22L)
		|| !ldcomp(d5, 1.23e22L))
	{
		(void) sprintf(resbuf, "%.3Lf, %Lg, %Lg, %Lg, %Lg", d1, d2, d3, d4, d5);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;

#else /* LONG_DOUBLE_SUPPORTED */

	DBUG_ENTER("test46");

	xx_rpt(UNSUPPORTED);
	in_rpt("long double type is not supported");

	DBUG_VOID_RETURN;

#endif	/* LONG_DOUBLE_SUPPORTED */

}

private void
test47()
{
	DBUG_ENTER("test47");

	(void) cppair(ch_t47, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t47()
{
	char	testdata[20];
	char	teststr[60];
	char	radch[2];
	char	*s;
	char	*expres = "123456, 123456, 123456";
	float	e, f, g;
	int	pathok = setupok;

	DBUG_ENTER("ch_t47");

	if (setlocale(LC_ALL, RAD2LANG) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("Pseudo language %s not available\n", RAD2LANG);
		in_rpt("Check installation of pseudo language\n");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	s = pslanginfo(VSX_RADIXCHAR, VALRAD2LANG);

	radch[1] = s[0];

	if (setlocale(LC_NUMERIC, RAD1LANG) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("Pseudo language %s not available\n", RAD1LANG);
		in_rpt("Check installation of pseudo language\n");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	s = pslanginfo(VSX_RADIXCHAR, VALRAD1LANG);

	radch[0] = s[0];

	(void) strcpy(testdata, "123");
	testdata[3] = radch[0];
	testdata[4] = '\0';
	(void) strcat(testdata, "456e03 ");
	(void) strcpy(teststr, testdata);
	(void) strcat(teststr, testdata);
	(void) strcat(teststr, testdata);
	t_data[tet_thistest] = teststr;

	currlang = RAD1LANG;
	(void) setlocale(LC_ALL, RAD1LANG);
	s = pslanginfo(VSX_RADIXCHAR, VALRAD1LANG);
		
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	e = f = g = 0.0;
	SCANIT(&f _ &e _ &g);


	if (f != 123456.0 || g != 123456.0 || e != 123456.0)
	{
		(void) sprintf(resbuf, "%8.8g, %8.8g, %8.8g", f, e, g);
		printres(psl_to_asc(t_data[tet_thistest], VALRAD1LANG), testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	(void) strcpy(testdata, "123");
	testdata[3] = radch[1];
	testdata[4] = '\0';
	(void) strcat(testdata, "456e03 ");
	(void) strcpy(teststr, testdata);
	(void) strcat(teststr, testdata);
	(void) strcat(teststr, testdata);
	t_data[tet_thistest] = teststr;

	currlang = RAD2LANG;
	(void) setlocale(LC_NUMERIC, RAD2LANG);
	s = pslanginfo(VSX_RADIXCHAR, VALRAD2LANG);
		
	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;


	e = f = g = 0.0;
	SCANIT(&f _ &e _ &g);

	if (f != 123456.0 || g != 123456.0 || e != 123456.0)
	{
		(void) sprintf(resbuf, "%8.8g, %8.8g, %8.8g", f, e, g);
		printres(psl_to_asc(t_data[tet_thistest], VALRAD2LANG), testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test48()
{
	DBUG_ENTER("test48");

	(void) cppair(ch_t48, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t48()
{
	char	testdata[20];
	char	teststr[60];
	char	*expres = "123456, 123456, 123456";
	float	e, f, g;
	int	pathok = setupok;

	DBUG_ENTER("ch_t48");

	currlang = NORADIXLANG;
	if (setlocale(LC_ALL, NORADIXLANG) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("Pseudo language %s not available\n", NORADIXLANG);
		in_rpt("Check installation of pseudo language\n");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) strcpy(testdata, "123");
	testdata[3] = '.';
	testdata[4] = '\0';
	(void) strcat(testdata, "456e03 ");
	(void) strcpy(teststr, testdata);
	(void) strcat(teststr, testdata);
	(void) strcat(teststr, testdata);
	t_data[tet_thistest] = teststr;
		
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	SCANIT(&f _ &e _ &g);

	if (f != 123456.0 || g != 123456.0 || e != 123456.0)
	{
		(void) sprintf(resbuf, "%8.8g, %8.8g, %8.8g", f, e, g);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test49()
{
	char	s[BSIZ];
	char	*res;
	int	pathok = setupok;

	DBUG_ENTER("test49");

	if (interface == I_SSCANF)
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (test_prep(t_fmt[tet_thistest],t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
#if TEST_XNFS
	
	if (tet_sync(SP_T49_CREAT_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREAT_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (freopen(t49_file, "r", t_fp) == (FILE *) NULL)
	{
		int err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"r\", fp) failed - errno %d(%s)",
			t49_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */
		PATH_TRACE;
		PATH_TRACE;
#endif /* TEST_XNFS */

	testfail = 0;

	globok = 0;
	savtimes(t49_file);
	PATH_FUNC_TRACE;

	SCANIT(s);

	globok = 0;
	if ((res = chktimes(t49_file, NEWATIME)) != NULL)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("%s", res);
	}
	else
		PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(6);

#if TEST_XNFS
	(void) tet_sync(SP_T49_UNLINK_START, snames, SYNC_TIMEOUT);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

/* Lowest test number for which following function is used */
#define TBASE	50

private	void
test50_53()
{
	char *function;
	int experr, err, ret, ferr_ret, reqfd;
	static int errnums[] = { EAGAIN, -EAGAIN, EBADF, EINTR, };
			      /* test50  test51   test52 test53  */
	int	pathok = setupok;

	DBUG_ENTER("test50_53");

	if (interface == I_SSCANF)
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}
	else if (interface == I_SCANF)
	{
		reqfd = STDIN_FILENO;
		/* If going to use stdin, it must be a "virgin" stream */
		if (freopen(t_file, "r", stdin) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
				t_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		function = "scanf()";
	}
	else
	{
		reqfd = -1;
		function = "fscanf()";
		PATH_TRACE;
	}

	experr = errnums[tet_thistest - TBASE];

	globok = 0;
	ret = do_ferr(experr, 'r', reqfd, t50_53func, &ferr_ret, &err);
	PATH_FUNC_TRACE;
	err = errno;
	if (experr < 0)
		experr = -experr;

	if (tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!ferr_ret || err != experr || ret != EOF)
	{
		xx_rpt(FAILURE);
		if (!ferr_ret)
			in_rpt("%s failed to set error indicator when testing for errno %d(%s)",
				function, experr, errname(experr));
		if (ret != EOF)
			in_rpt("%s returned %d when testing for errno %d(%s), expected EOF",
				function, ret, experr, errname(experr));
		else if (err != experr)
			in_rpt("%s set errno incorrectly: expected %d(%s), actual %d(%s)",
				function, experr, errname(experr),
				err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}

	if (interface == I_SCANF)
	{
		struct stat ensure_buf;
		if((fstat(fileno(stdin), &ensure_buf)==SYSERROR) &&
			(freopen(t_file, "r", stdin) == (FILE *) NULL))
		{
			int i;
			(void)sprintf(resbuf,"deletion reason: failed to ensure stream stdin was open after test %d",tet_thistest);
			for(i=1;i<=no_tests;i++)
				tet_delete(i,resbuf);
		}
	}

	DBUG_VOID_RETURN;
}

private int
t50_53func(fp, nbyte)
FILE *fp;
int nbyte;
{
	char s[BSIZ];
	int	pathok = 0;

	DBUG_ENTER("t50_53func");

	globok = 0;

	if (nbyte >= BSIZ)
	{
		(void) tet_delete(tet_thistest, "nbyte too great in t50_53func()");
		DBUG_RETURN(0);
	}

	switch(interface)
	{
	case I_SCANF:
		tretval = scanf("%c", s);
		PATH_TRACE;
		break;
	case I_FSCANF:
		tretval = fscanf(fp, "%c", s);
		PATH_TRACE;
		break;
	}
	PATH_FUNC_RPT(1);

	DBUG_RETURN(tretval);
}

private	void
test54()
{
	int err, reqfd = -1;
	int	pathok = setupok;

	DBUG_ENTER("test54");

	if (interface == I_SSCANF)
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (interface == I_SCANF)
	{
		reqfd = STDIN_FILENO;
		if (freopen(t_file, "r", stdin) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
				t_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	sigstat = "ignored";
	do_feio("ri", reqfd, t54_55io, t54rpt);

	if (interface == I_SCANF)
	{
		if (freopen(t_file, "r", stdin) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
				t_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	sigstat = "blocked";
	do_feio("rb", reqfd, t54_55io, t54rpt);

	DBUG_VOID_RETURN;
}

private void
t54rpt(ret, err)
int ret, err;
{
	int	pathok = setupok;

	DBUG_ENTER("t54rpt");

	PATH_FUNC_TRACE;

	if (ret != EOF || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("%s() from background with SIGTTIN %s did not give correct results",
			(interface == I_SCANF ? "scanf" : "fscanf"), sigstat);
		if (ret != EOF)
			in_rpt("expected return: EOF, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private	void
test55()
{
	int err, reqfd = -1;
	int	pathok = setupok;

	DBUG_ENTER("test55");

	if (interface == I_SSCANF)
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (interface == I_SCANF)
	{
		reqfd = STDIN_FILENO;
		if (freopen(t_file, "r", stdin) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
				t_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	do_feio("ro", reqfd, t54_55io, t55rpt);

	DBUG_VOID_RETURN;
}

private void
t55rpt(ret, err)
int ret, err;
{
	int	pathok = setupok;

	DBUG_ENTER("t55rpt");

	PATH_FUNC_TRACE;

	if (ret != EOF || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("%s() from orphaned background process did not give correct results",
			(interface == I_SCANF ? "scanf" : "fscanf"));
		if (ret != EOF)
			in_rpt("expected return: EOF, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private int
t54_55io(fp)
FILE *fp;
{
	char s[2];

	DBUG_ENTER("t54_55io");

	globok = 0;

	switch(interface)
	{
	case I_SCANF:
		tretval = scanf("%1c", s);
		globok = 1;
		break;
	case I_FSCANF:
		tretval = fscanf(fp, "%1c", s);
		globok = 1;
		break;
	}

	DBUG_RETURN(tretval);
}

#endif /* TEST_XNFS */

private void
test56()
{
	DBUG_ENTER("test56");

	if(interface == I_SSCANF)
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t56, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t56_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t56()
{
	char	*rofs;
	int	err;
	char	*args[8];

	DBUG_ENTER("ch_t56");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_ROFS is not defined");
		DBUG_VOID_RETURN;
	}

	args[0] = prog_su;
	args[1] = newstr(itoa(tet_thistest));
	args[2] = newstr(itoa(interface));
	args[3] = t56_dir;
	args[4] = t56_file;
	args[5] = vg_ident;
	args[6] = vg_dbug;
	args[7] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
printres(data, format, expres, obsres, warntext)
char *data, *format, *expres, *obsres;
char **warntext;
{
	char	*func;

	DBUG_ENTER("printres");

	if (!testfail++)
	{
		if (warntext != NULLTEXT)
		{
			char **s;
			xx_rpt(WARNING);
			for (s = warntext; *s != NULL; s++)
				in_rpt("%s", *s);
		}
		else
			xx_rpt(FAILURE);
	}

	switch(interface)
	{
	case	I_SCANF:
		func = "scanf(";
		break;
	case	I_FSCANF:
		func = "fscanf(t_fp, ";
		break;
	case	I_SSCANF:
		func = "sscanf(data, ";
		break;
	}

	in_rpt("%s\"%s\") failed", func, format);
	if (*currlang != '\0')
		in_rpt("Current language is %s", currlang);
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) && !TEST_XNFS
	else if(wflag && *locale != '\0')
		in_rpt("Current language is %s", locale);

	wflag = 0;
#endif
	in_rpt("Input data: %s", psl_to_asc(data, 0));
	in_rpt("Expected result: %s", expres);
	in_rpt("Observed result: %s", obsres);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test57()
{
	DBUG_ENTER("test57");

	(void) cppair(ch_t57, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t57()
{
	int err;
	char *args[8];

	DBUG_ENTER("ch_t57");

	args[0] = prog_su;
	args[1] = newstr(itoa(tet_thistest));
	args[2] = newstr(itoa(interface));
	args[3] = "";
	args[4] = t57_file;
	args[5] = vg_ident;
	args[6] = vg_dbug;
	args[7] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test58()
{
	DBUG_ENTER("test58");

	t_estale_rd_c(t58_file, STDIN, t58_func, t58_chk_func, SP_T58_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t58_func(fp)
FILE	*fp;
{
	char	s[2];
	int	rval;

	DBUG_ENTER("t58_func");

	globok = 0;

	switch(interface)
	{
	case I_SCANF:
		rval = scanf("%1c", s);
		globok = 1;
		break;
	case I_FSCANF:
		rval = fscanf(fp, "%1c", s);
		globok = 1;
		break;
	}

	DBUG_RETURN(rval == EOF ? -1 : 0);
}

private int
t58_chk_func(fp)
FILE	*fp;
{
	int	rval;
	char	s[2];
	int	err;
	int	pathok = 0;

	DBUG_ENTER("t58_chk_func");

	errno = NOERROR;

	for( ;; )
	{
		switch(interface)
		{
		case I_SCANF:
			rval = scanf("%1c", s);
			break;
		case I_FSCANF:
			rval = fscanf(fp, "%1c", s);
			break;
		}
		if(rval == EOF)
			break;
	}

	err = errno;

	if(rval != EOF || ferror(fp) == 0 || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(rval != EOF)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value EOF");
			in_rpt("Actual   return value %d", rval);
		}
		else
			PATH_TRACE;
		
		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;
		
		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */

#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) && !TEST_XNFS
private void
wclocale()
{
	int	pathok = 0;

	DBUG_ENTER("wclocale");

	/*
	 * Change to wide character locale
	 */
	if (setlocale(LC_CTYPE, locale) == (char *) NULL)
	{
		in_rpt("Pseudo Language %s is not available", locale);
		in_rpt("Check installation of pseudo language");
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	wflag = 1;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}
#endif

