/*
*      SCCS:  @(#)  ANSI.os/streamio/scanf/scanf_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/scanf - scanf_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		scanf_su - used by T.scanf in privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.scanf to perform privileged tests.

	  usage:
		scanf_su interface dir file ident dbug

	  where interface is the function to be executed eg scanf, fscanf,
	  dir and file are parameters to be passed to the test.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Neil Moses, UniSoft Ltd.
DATE CREATED:	10 June 1992
MODIFICATIONS:

		John-Paul, 8 November 1992
			XNFS enhancements.
		
		Geoff Clare, 1 Sept 1993
			Delete second argument from t57_chk_func().

************************************************************************/
#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <dbug.h>
#include <errno.h>
#include <sys/stat.h>
#include <report.h>
#include <tet_api.h>
#include <xnfs.h>
#include "scanf_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef fscanf
#undef scanf
#undef sscanf
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

#define I_SCANF		0
#define I_FSCANF	1

#define STDIN		(interface == I_FSCANF ? 0 : \
			interface == I_SCANF ? 1 : 0 )

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
public  int	testfail;

extern	char	**environ;
extern	void	savtimes();
extern	char	*chktimes();
extern	char	*errname();

extern	void	valrpt();
extern	void	set_pname();

private int	interface =	0;
private int	testnum =	0;

private	void	subtest56();
#if TEST_XNFS
private int	t57_chk_func();
#endif /* TEST_XNFS */

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	set_pname(argv[0]);

	if (argc < 5 || argc > 7)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}

	if (argc >= 7)
		vg_dbug = argv[6];
	if (argc >= 6)
		vg_ident = argv[5];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testnum = atoi(argv[1]);
	interface = atoi(argv[2]);
        switch(testnum)
        {
	case 56:
		subtest56(argv[3], argv[4]);
		break;
#if TEST_XNFS
	case 57:
		t_eacces_rd_c(argv[4], STDIN, t57_chk_func, SP_T57_EACCES);
		break;
#endif /* TEST_XNFS */
	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed : %d", testnum);
		return(OK);
	}

	return(OK);
}

private void
subtest56(mdir, file)
char	*mdir, *file;
{
	FILE	*fp;
	char	*func;
	char 	buf[2];
	int 	rval;
	char	*errptr;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("subtest56");

	errno = 0;
	testfail = 0;

	globok = 0;
	if (setuprofs(mdir, file, 'f', (mode_t)MODEANY) != 0)
	{
		DBUG_VOID_RETURN;	/* reported in setuprofs() */
	}
	else
		PATH_FUNC_TRACE;

	switch(interface)
	{

	case I_SCANF:

		if((fp = freopen(file, "r", stdin)) == NULL)
		{
			err = errno;
			if(!testfail++)
				xx_rpt(DELETION);
			in_rpt("freopen(\"%s\", \"r\", stdin) failed - errno %d (%s)", 
				file, err, errname(err));
			(void) clnuprofs(mdir, file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;		
		break;

	case I_FSCANF:

		if((fp = fopen(file, "r")) == NULL)
		{
			err = errno;
			if(!testfail++)
				xx_rpt(DELETION);
			in_rpt("fopen(\"%s\", \"r\") failed - errno %d (%s)", 
				file, err, errname(err));
			(void) clnuprofs(mdir, file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;		
		break;

	default:
		PATH_TRACE;

	}

	globok = 0;
	savtimes(file);
	PATH_FUNC_TRACE;

	errno = NOERROR;

	switch(interface)
	{
	case I_SCANF:
		func = "scanf";
		rval = scanf("%s", buf);
		break;
	case I_FSCANF:
		func = "fscanf";
		rval = fscanf(fp, "%s", buf);
		break;
	};

#ifdef TESTBED
	rval = EOF;
	if (errno)
		errno++;
#endif
	err = errno;

	if (rval != EOF || ferror(fp))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (rval != EOF)
			in_rpt("%s() returned %d, expected EOF", func, rval);
		else
		{
			in_rpt("%s() set the error indicator for the stream",
				func);
			in_rpt("errno was set to %d (%s)", err, errname(err));
		}
		(void) fclose(fp);
		(void) clnuprofs(mdir, file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	errptr = chktimes(file, 0);
	PATH_FUNC_TRACE;

	if(errptr != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("Call to %s() changed time stamps on %s", func, file);
		in_rpt("%s", errptr);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) fclose(fp);
	(void) clnuprofs(mdir, file);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private int
t57_chk_func(fp)
FILE	*fp;
{
	int	rval = 1;
	char	s[2];
	int	err;
	int	pathok = 0;

	DBUG_ENTER("t57_chk_func");

	errno = NOERROR;

	while(rval > 0)
	{
		switch(interface)
		{
		case I_SCANF:
			rval = scanf("%1c", s);
			break;
		case I_FSCANF:
			rval = fscanf(fp, "%1c", s);
			break;
		}
	}

	err = errno;

	if(rval != EOF || ferror(fp) == 0 || err != EACCES)
	{
		xx_rpt(FAILURE);

		if(rval != EOF)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value EOF ");
			in_rpt("Actual   return value %d", rval);
		}
		else
			PATH_TRACE;
		
		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;
		
		if(err != EACCES)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",EACCES,errname(EACCES));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
