/*
*      SCCS:  @(#)  ANSI.os/streamio/scanf_X/scanf_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/scanf_X - T.scanf_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.scanf_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the scanf() subroutine call, X/Open extensions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	26 May 1989
MODIFICATIONS:	
		Geoff Clare, 1 Aug 1989
			Remove tests with space in exponent ("e 12").

		Peter Lee, Unisoft Ltd.,
		Wed Jul 18 12:44:53 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated reason() declaration

		Matt Newman, Thu Nov 14 14:52:32 GMT 1991
			4.2 Wide Char support

		Geoff Clare, 22 June 1992
			In test_prep() ensure that t_data[tet_thistest]
			points to data argument.

		John-Paul, 1 November 1992
			XNFS enhancements.

		Geoff Clare, 25 Oct 1994
			Avoid freopen() of closed stream in test_prep().

		Geoff Clare, 7 Sept 1995
			Tests 31 & 34 only apply in XPG3 mode.
			Changed format strings in tests 38 & 41 to avoid
			using ranges with "%[".

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<chktimes.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<tet_api.h>
#include	<xnfs.h>
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) && !TEST_XNFS
#include	<wchar.h>
#include	<locale.h>
#include	<nlsinfo.h>
#include	<nlstest.h>
#endif
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fscanf
#undef scanf
#undef sscanf
#endif


#define NO_TESTS	44		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define BSIZ		64
#define BUFFERSIZE	512
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3)
#endif
#define NULLTEXT	(char **) 0

#define I_SCANF		0
#define I_FSCANF	1
#define I_SSCANF	2

#define T_NUM		12
#define T13_NUM		54

/* Use " _ " to separate SCANIT arguments */
#define	_ ,

#define	SCANIT(args)  \
	switch(interface)\
	{\
	case I_SCANF:\
		tretval = scanf(testfmt, args);\
		rewind(t_fp);\
		break;\
	case I_FSCANF:\
		tretval = fscanf(t_fp, testfmt, args);\
		rewind(t_fp);\
		break;\
	case I_SSCANF:\
		tretval = sscanf(t_data[tet_thistest], testfmt, args);\
		break;\
	};

extern size_t  mbstowcs();
extern	int	dcomp();
extern	int	fcomp();
extern	char	*errname();
extern	char	*psl_to_asc();
extern	char	*pr_char();
extern	char	*itoa();

#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3)
extern  char    *pr_wchar();
extern  char    *pr_wstr();
#endif

extern	char	*vg_ident;

int	no_tests = NO_TESTS;

private char	*t_file =	"./scanf_X-t";
private FILE	*t_fp =		(FILE *) NULL;
private int	interface = 0;
private int	setupok = -1;
private int	tretval;

private	int	testfail;
private char	resbuf[BUFFERSIZE];

#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) && !TEST_XNFS
private	char	*locale;
private	char	mbstr[BUFFERSIZE];
private	char	wexpstr[BUFFERSIZE];
private	wchar_t	targ[BUFFERSIZE];
private	wchar_t	*wcstr;
private int	wlen;
private int	wflag;
#endif

private char	*testfmt;
#if !TEST_XNFS
#if TEST_XPG == 3
private char	*x1_fmt = "%1$[f-a]";
private char	*x2_fmt = "%1$[^n-l]";
#endif /* TEST_XPG == 3 */
private char	*x3_fmt = "%1$[cdabef-]";
#endif /* TEST_XNFS */

private char	*t_nlfmt[NO_TESTS + 1] = { "",
  				"%3$d %1$d %2$d",
				"%3$d %1$f %2$s",
				"%5$d %1$d %2$d %3$d %4$d",
				"%2$d %1$d",
				"%2$d%1$d",
				"%4$d%1$d%2$c%3$[^1]",
				"%3$dabcd%1$defgh%2$d",
				"%%%1$d",
				"%2$d%1$d",
		/* 10 */	"%1$u",
				"%1$o",
				"%2$x%1$X",
				"%4$i%1$i%2$i%3$i",
				"%6$g%1$g%2$g%3$g%4$g%5$g",
				"%6$g%1$g%2$g%3$g%4$g%5$g",
				"%6$g%1$g%2$g%3$g%4$g%5$g",
				"%6$g%1$g%2$g%3$g%4$g%5$g",
				"%6$g%1$g%2$g%3$g%4$g%5$g",
				"%6$g%1$g%2$g%3$g%4$g%5$g",
		/* 20 */	"%6$g%1$g%2$g%3$g%4$g%5$g",
				"%1$s",
				"%1$s",
				"%1$c",
		/* 24 */	"%1$c",
				"%1$S",
				"%1$S",
				"%1$C",
				"%1$C",
		/* 29 */	"%1$p",
		/* 30 */	"%3$s%1$n%2$s",
				"%1$[a-f]",
				"%1$[fbcad]",
				"%1$[^ml]",
				"%1$[^e-i]",
				"%1$[-fcadb]",
				"%1$[]cabed]",
				"%1$[^]ced]",
				"%1$[def]",
				"%9$ld%1$li%2$lo%3$lu%4$lx%5$lf%6$le%7$lG%8$ln",
		/* 40 */	"%7$hd%1$hi%2$ho%3$hu%4$hx%5$hX%6$hn",
				"%9$5d%1$6o%2$5u%3$4x%4$6f%5$9E%6$9g%7$3[cdef]%8$5s",
		/* 42 */	"%1$3c",
				"",
		/* 44 */	"%5$Lf%1$Le%2$LE%3$Lg%4$LG",
				};


private char	*t_data[NO_TESTS + 1] = { "",
				" 25 26         27 28 29 30",
				"30 54.1 vsx",
				"30 30 30",
				"12 a",
				"     ",
				"10\n\t\f 11\n\t\f 1",
				"10abcd11abcd12",
				"%10",
				"   11   12",
		/* 10 */	"  54321",
				"    76543",
				"  56cE 5Ba0",
				"123 0123 0x123 0X123",
				"123 +123 -123 123.123 +123.123 -123.123",
				"123e12 +123e12 -123e12 123E12 +123E12 -123E12",
				"123.123e12 +123.123e12 -123.123e12 123.123E12 +123.123E12 -123.123E12",
				"123e+12 +123e+12 -123e+12 123E+12 +123E+12 -123E+12",
				"123.123e+12 +123.123e+12 -123.123e+12 123.123E+12 +123.123E+12 -123.123E+12",
				"123e-12 +123e-12 -123e-12 123E-12 +123E-12 -123E-12",
		/* 20 */	"123.123e-12 +123.123e-12\n-123.123e-12 123.123E-12 +123.123E-12 -123.123E-12",
				"       test-string not-included",
				"  test-string    ",
				"test char",
		/* 24 */	"   test char",
				"",
				"",
				"",
				"",
		/* 29 */	"",
		/* 30 */	" 2345  abcde",
				"abcdefghij-klmnopq",
				"abcdefghij-klmnopq",
				"abcdefghij-klmnopq",
				"abcdefghij-klmnopq",
				"a-bcdefghij-klmnopq",
				"ab]cd]efghij-klmnopq",
				"ab]cd]efghij-klmnopq",
				"abcdefghi",
				" 1234567890 1234567890 011145401322 1234567890 499602d2 123456.123456\n1.23e+22 1.23e+22",
		/* 40 */	"12345 12345 030071 12345 3039 3039",
				" 12345030071123453039456.1211.23e+0021.23e+00cdeftest-string",
		/* 42 */	"abcdefghi",
				"",
		/* 44 */	"123.123 1.23e+22 1.23e+22 1.23e+22 1.23e+22",
			   };
#if !TEST_XNFS
private float	floatvals[9][6] ={
					{
					123.0, 123.0, -123.0, 
					123.123, 123.123, -123.123
					},
					{
					123e12, 123e12, -123e12,
					123e12, 123e12, -123e12
					},
					{
					123.123e12, 123.123e12, -123.123e12, 
					123.123e12, 123.123e12, -123.123e12 
					},
					{
					123e12, 123e12, -123e12,
					123e12, 123e12, -123e12 
					},
					{
					123.123e12, 123.123e12, -123.123e12, 
					123.123e12, 123.123e12, -123.123e12 
					},
					{
					123e-12, 123e-12, -123e-12, 
					123e-12, 123e-12, -123e-12 
					},
					{
					123.123e-12, 123.123e-12, -123.123e-12,
					123.123e-12, 123.123e-12, -123.123e-12
					}
				};
#endif /* TEST_XNFS */

private void	prepare_tests();
private void	cln_tests();
private int	test_prep();
#if TEST_XNFS
private void	notinuse();
#endif /* TEST_XNFS */
private void	test1(), test2(), test3(), test4(), test5();
#if !TEST_XNFS
private void	test6(), test7(), test8(), test9(), test10(), test11(),
		test12(), test13(), test14_20(), test21(), test22(), test23(),
		test24(), test25(), test26(), test27(), test28(),
		test29(), test30(), test31(), test32(), test33(), test34(),
		test35(), test36(), test37(), test38(), test39(), test40(),
		test41(), test42(), test43(), test44();
#endif /* TEST_XNFS */
private void	ch_t6(), ch_t25(), ch_t26(), ch_t27(), ch_t28(), ch_t43();
private void	printres();
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) && !TEST_XNFS
private void	wclocale();
#endif

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;

#if !TEST_XNFS

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14_20, 14,
	 test14_20, 15,
	 test14_20, 16,
	 test14_20, 17,
	 test14_20, 18,
	 test14_20, 19,
	 test14_20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	 test29, 29,
	 test30, 30,
	 test31, 31,
	 test32, 32,
	 test33, 33,
	 test34, 34,
	 test35, 35,
	 test36, 36,
	 test37, 37,
	 test38, 38,
	 test39, 39,
	 test40, 40,
	 test41, 41,
	 test42, 42,
	 test43, 43,
	 test44, 44,
	NULL, 0
};

#else /* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 notinuse, 6,
	 notinuse, 7,
	 notinuse, 8,
	 notinuse, 9,
	 notinuse, 10,
	 notinuse, 11,
	 notinuse, 12,
	 notinuse, 13,
	 notinuse, 14,
	 notinuse, 15,
	 notinuse, 16,
	 notinuse, 17,
	 notinuse, 18,
	 notinuse, 19,
	 notinuse, 20,
	 notinuse, 21,
	 notinuse, 22,
	 notinuse, 23,
	 notinuse, 24,
	 notinuse, 25,
	 notinuse, 26,
	 notinuse, 27,
	 notinuse, 28,
	 notinuse, 29,
	 notinuse, 30,
	 notinuse, 31,
	 notinuse, 32,
	 notinuse, 33,
	 notinuse, 34,
	 notinuse, 35,
	 notinuse, 36,
	 notinuse, 37,
	 notinuse, 38,
	 notinuse, 39,
	 notinuse, 40,
	 notinuse, 41,
	 notinuse, 42,
	 notinuse, 43,
	 notinuse, 44,
	NULL, 0
};

#endif /* TEST_XNFS */

private	void
prepare_tests()
{
	int	pathok = 0;
	char	*s;
	int	i;
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) && !TEST_XNFS
	int	j, err;
	wchar_t	tbuf[BUFFERSIZE];
	static	char	msg[BUFFERSIZE];
	static	int	wtests[] = {6,25,26,27,28,43,0};
	char *ptr;
	char current[50];
#endif

	DBUG_ENTER("prepare_tests");
	if ((s=strrchr(vg_ident,'/')) == NULL)
		s=vg_ident+2;
	else
		s+=3;
	if (strcmp(s, "scanf_X") == 0)
		interface = I_SCANF;
	else if (strcmp(s, "fscanf_X") == 0)
		interface = I_FSCANF;
	else if (strcmp(s, "sscanf_X") == 0)
		interface = I_SSCANF;
	else {
		for (i=1; i<=NO_TESTS; i++)
			tet_delete(i, "bad ident parameter");
	}

#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) && !TEST_XNFS
	errno = NOERROR;
	if(wcharstr(&locale, &wcstr) != 0)
	{
		err = errno;
		(void) sprintf(msg, "wcharstr() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;

	if((ptr = setlocale(LC_CTYPE, NULL)) == (char *)NULL)
	{
		err = errno;
		(void) sprintf(msg, "setlocale() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;

	(void) strcpy(current, ptr);

	if(setlocale(LC_CTYPE, locale) == (char *)NULL)
	{
		err = errno;
		(void) sprintf(msg, "setlocale() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;

	/*
	 * Strip out white space
	 */
	for(i = 0, j = 0; *(wcstr + i) != (wchar_t) '\0'; i++)
	{
		if(i == 0)
			PATH_TRACE;

		if(iswspace((wint_t) *(wcstr + i)))
			continue;

		tbuf[j++] = *(wcstr + i);
	}

	(void) wcsncpy(wcstr, tbuf, (size_t)j);
	*(wcstr + j) = (wchar_t) '\0';
	wlen = j;

	if(wcstombs(mbstr, wcstr, BUFFERSIZE) == (size_t)-1)
	{
		err = errno;
		(void) sprintf(msg, "wcstombs() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;

	/* Restore the locale */
	if(setlocale(LC_CTYPE, current) == NULL)
	{
		err = errno;
		(void) sprintf(msg, "setlocale() failed in preperations - errno %d(%s)", err, errname(err));
		for(i = 0; wtests[i] != 0; i++)
			tet_delete(wtests[i], msg);
	}
	else
		PATH_TRACE;

	wflag = 0;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif

	PATH_SETUP_RPT(6);

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");
	(void) unlink(t_file);
	DBUG_VOID_RETURN;
}


private int
test_prep(fmt,data)
	char *fmt,*data;
{
	int		err;
	int		fd;
	int		wval;
	int		len;
	int		ok = SUCCEED;
	int	pathok = 0;

	DBUG_ENTER("test_prep");

	globok = 0;

	if ((fd = creat(t_file, (mode_t)MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
		DBUG_RETURN(FAIL);
	}
	else
		PATH_TRACE;

	wval = write(fd, data, (unsigned) (len = strlen(data)));
	(void) close(fd);
	if (wval != len)
	{
		err = errno;
		xx_rpt(DELETION);
		if (wval == SYSERROR)
			in_rpt("deletion reason: write() failed - errno %d(%s)",
				err, errname(err));
		else
			in_rpt("deletion reason: write() wrote only %d buytes of %d",
				wval, len);
		ok = FAIL;
	}
	else if (interface == I_SCANF)
	{
		if (freopen(t_file, "r", stdin) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: freopen(%s, \"r\", stdin) ailed - errno %d(%s)",
				t_file, err, errname(err));
			ok = FAIL;
		}
		else
			PATH_TRACE;
		t_fp = stdin;
	}
	else if (interface == I_FSCANF)
	{
		if (t_fp != (FILE *) NULL)
			(void) fclose(t_fp);
		if ((t_fp = fopen(t_file, "r")) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fopen(%s, \"r\") ailed - errno %d(%s)",
				t_file, err, errname(err));
			ok = FAIL;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	testfmt = fmt;

	/* ensure data string is correct for sscanf() */
	if (t_data[tet_thistest] != data)
		t_data[tet_thistest] = data;

	PATH_FUNC_RPT(2);
	
	DBUG_RETURN(ok);
}

#if TEST_XNFS

private void
notinuse()
{
	DBUG_ENTER("notinuse");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test1()
{
	int		i1, i2, i3;
	char	*expres = "return value = 3";
	int	pathok = setupok;

	DBUG_ENTER("test1");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = i3 = 0;

	testfail = 0;

	SCANIT(&i2 _ &i3 _ &i1);

	if (tretval != 3)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	int		i;
	float		f;
	char		s[BSIZ];
	char	*expres = "30, 54.1, \"vsx\"";
	int	pathok = setupok;

	DBUG_ENTER("test2");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i = 0;
	f = 0.0;
	s[0] = NUL;

	testfail = 0;

	SCANIT(&f _ s _ &i);

	s[BSIZ - 1] = NUL;
	if (i != 30 || !fcomp(f, 54.1) || strcmp(s, "vsx"))
	{
		(void) sprintf(resbuf, "%d, %f, \"%s\"", i, f, s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
		for (i = 3 ; i <= NO_TESTS ; i++)
			tet_delete(i, "depends on success of test 2");
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test3()
{
	int		fail = 0;
	int		i1, i2, i3, i4, i5;
	char	*expres = "30, 30, 30, 12, 12";
	int	pathok = setupok;

	DBUG_ENTER("test3");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = i3 = 0;
	i4 = i5 = T_NUM;

	testfail = 0;

	SCANIT(&i2 _ &i3 _ &i4 _ &i5 _ &i1);

	if (i1 != 30 || i2 != 30 || i3 != 30)
		fail++;
	else
		PATH_TRACE;

	if (i4 != T_NUM || i5 != T_NUM)
		fail++;
	else
		PATH_TRACE;

	if (tretval != 3 || fail)
	{
		(void) sprintf(resbuf, "%d, %d, %d, %d, %d", i1, i2, i3, i4, i5);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


private void
test4()
{
	int		fail = 0;
	int		i1, i2;
	char	*exp1res = "unread character = 'a'";
	char	*exp2res = "return value = 1";
	int	pathok = setupok;

	DBUG_ENTER("test4");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = 0;

	testfail = 0;

	switch (interface)
	{
	case  I_SCANF:
		tretval = scanf(testfmt, &i2, &i1);
		if ((tretval = getchar()) != 'a')
			fail = 1;
		else
			PATH_TRACE;
		break;

	case I_FSCANF:
		tretval = fscanf(t_fp, testfmt, &i2, &i1);
		if ((tretval = getc(t_fp)) != 'a')
			fail = 1;
		else
			PATH_TRACE;
		break;

	case I_SSCANF:
		tretval = sscanf(t_data[tet_thistest], testfmt, &i2, &i1);
		if (tretval != 1)
			fail = 1;
		else
			PATH_TRACE;
	}

	if (fail)
	{
		if (interface == I_SSCANF)
		{
			(void) sprintf(resbuf, "return value = %d", tretval);
			printres(t_data[tet_thistest], testfmt, exp2res, resbuf, NULLTEXT);
		}
		else
		{
			(void) sprintf(resbuf, "unread character = '%c'", tretval);
			printres(t_data[tet_thistest], testfmt, exp1res, resbuf, NULLTEXT);
		}
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


private void
test5()
{
	int	i1, i2;
	char	*expres = "return value = EOF";
	int	pathok = setupok;

	DBUG_ENTER("test5");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	i1 = i2 = 0;

	testfail = 0;

	SCANIT(&i2 _ &i1);

	if (tretval != EOF)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

private void
test6()
{
	DBUG_ENTER("test6");

	(void) cppair(ch_t6, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	int		i1, i2;
	char		c1;
	char		s1[BSIZ];
	char	*scanstr = "10\\n\\t\\f 11\\n\\t\\f 1";
	char	*expres = "10, 11, \\n, \"\\t\\f \"";
	int	pathok = setupok;
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3)
	char	*wfmt = "%4$d%1$d%2$C%3$[^1]";
	wchar_t	wc;
#endif

	DBUG_ENTER("ch_t6");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i1 = 0;
	c1 = NUL;
	s1[0] = NUL;

	testfail = 0;

	SCANIT(&i2 _ &c1 _ s1 _ &i1);

	s1[BSIZ - 1] = NUL;
	if (i1 != 10 || i2 != 11 || c1 != '\n' || strcmp(s1, "\t\f "))
	{
		(void) sprintf(resbuf, "%d %d '%s', \"%s\"", i1, i2, pr_char(c1), s1);
		printres(scanstr, testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) 

	if(test_prep(wfmt, t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = 0; wc = (wchar_t)NUL; s1[0] = NUL;

	SCANIT(&i2 _ &wc _ s1 _ &i1);

	s1[BSIZ - 1] = NUL;
	if(i1 != 10 || i2 != 11 || wc != (wchar_t)'\n' || strcmp(s1, "\t\f "))
	{
		(void) sprintf(resbuf, "%d, %d, %s, \"%s\"", i1, i2, pr_wchar(wc), s1);
		printres(scanstr, wfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);
#else
	if (!testfail)
		PATH_XS_RPT(2);
#endif

	DBUG_VOID_RETURN;
}


private void
test7()
{
	int		i1, i2, i3;
	char	*expres = "return value = 2";
	int	pathok = setupok;

	DBUG_ENTER("test7");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i1 = i2 = i3 = 0;

	testfail = 0;

	SCANIT(&i2 _ &i3 _ &i1);

	if (tretval != 2)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test8()
{
	int		i;
	char	*expres = "10";
	int	pathok = setupok;

	DBUG_ENTER("test8");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i = 0;

	testfail = 0;

	SCANIT(&i);

	if (i != 10)
	{
		(void) sprintf(resbuf, "%d", i);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test9()
{
	int		i1, i2;
	char	*expres = "11, 12";
	int	pathok = setupok;

	DBUG_ENTER("test9");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	i1 = i2 = 0;
	SCANIT(&i2 _ &i1);

	if (i1 != 11 || i2 != 12)
	{
		(void) sprintf(resbuf, "%d, %d", i1, i2);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test10()
{
	unsigned int	u;
	char	*expres = "54321";
	int	pathok = setupok;

	DBUG_ENTER("test10");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	u = 0;
	SCANIT(&u);

	if (u != (unsigned) 54321)
	{
		(void) sprintf(resbuf, "%u", u);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test11()
{
	int		i;
	char	*expres = "32099";
	int	pathok = setupok;

	DBUG_ENTER("test11");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	i = 0;
	SCANIT(&i);

	if (i != 32099)
	{
		(void) sprintf(resbuf, "%d", i);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test12()
{
	int	i1, i2;
	char	*expres = "22222, 23456";
	int	pathok = setupok;

	DBUG_ENTER("test12");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	i1 = i2 = 0;
	SCANIT(&i2 _ &i1);

	if (i1 != 22222 || i2 != 23456)
	{
		(void) sprintf(resbuf, "%d, %d", i1, i2);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test13()
{
	int	i1, i2, i3, i4;
	char	*expres = "123, 83, 291, 291";
	int	pathok = setupok;

	DBUG_ENTER("test13");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	i1 = i2 = i3 = i4 = 0;
	SCANIT(&i2 _ &i3 _ &i4 _ &i1);

	if (i1 != 123 || i2 != 83 || i3 != 291 || i4 != 291)
	{
		(void) sprintf(resbuf, "%d, %d, %d, %d", i1, i2, i3, i4);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test14_20()
{
	float	f[6];
	int	i;
	int	val_no;
	char	expres[90];
	int	pathok = setupok;

	DBUG_ENTER("test14_20");

	val_no = tet_thistest - 14;

	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	for (i=0; i<6; i++)
		f[i] = 0.0;
	SCANIT(&f[1] _ &f[2] _ &f[3] _ &f[4] _ &f[5] _ &f[0]);

	for (i = 0; i < 6; i++)
		if (!fcomp(f[i], floatvals[val_no][i]))
			break;

	if (i != 6)
	{
		(void) sprintf(expres, "%e, %e, %e, %e, %e, %e",
			floatvals[val_no][0], floatvals[val_no][1],
			floatvals[val_no][2], floatvals[val_no][3],
			floatvals[val_no][4], floatvals[val_no][5]);

		(void) sprintf(resbuf, "%e, %e, %e, %e, %e, %e", 
			f[0], f[1], f[2], f[3], f[4], f[5]);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test21()
{
	char		s[BSIZ];
	char	*expres = "\"test-string\"";
	int	pathok = setupok;

	DBUG_ENTER("test21");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	s[0] = NUL;
	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (strcmp(s, "test-string"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test22()
{
	char		s[BSIZ];
	char	*expres = "\"test-string\"";
	int	pathok = setupok;

	DBUG_ENTER("test22");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	s[0] = NUL;
	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (strcmp(s, "test-string"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test23()
{
	char		c;
	char	*expres = "'t'";
	int	pathok = setupok;

	DBUG_ENTER("test23");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	c = NUL;
	SCANIT(&c);

	if (c != 't')
	{
		(void) sprintf(resbuf, "%c(%s)", c, pr_char(c));
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test24()
{
	char		c;
	char	*expres = " ";
	int	pathok = setupok;

	DBUG_ENTER("test24");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	c = NUL;
	SCANIT(&c);

	if (c != ' ')
	{
		(void) sprintf(resbuf, "%c(%s)", c, pr_char(c));
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test25()
{
	DBUG_ENTER("test25");

	(void) cppair(ch_t25, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t25()
{
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3)
	int	pathok = setupok;

	DBUG_ENTER("ch_t25");

	globok = 0;
	if (test_prep(t_nlfmt[tet_thistest], mbstr) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	targ[0] = (wchar_t)NUL;

	testfail = 0;

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	SCANIT(targ);

	if(wcscmp(wcstr, targ))
	{
		(void) strcpy(resbuf, pr_wstr(targ));
		(void) strcpy(wexpstr, pr_wstr(wcstr));
		printres(mbstr, testfmt, wexpstr, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}

private void
test26()
{
	DBUG_ENTER("test26");

	(void) cppair(ch_t26, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t26()
{
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3)
	int	pathok = setupok;

	DBUG_ENTER("ch_t26");

	globok = 0;
	if (test_prep(t_nlfmt[tet_thistest], mbstr) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	targ[0] = (wchar_t)NUL;
	targ[wlen] = (wchar_t)'a';
	testfail = 0;

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	SCANIT(targ);

	if(wcscmp(wcstr, targ) || targ[wlen] != (wchar_t)NUL)
	{
		if(wcscmp(wcstr, targ))
		{
			(void) strcpy(resbuf, pr_wstr(targ));
			(void) strcpy(wexpstr, pr_wstr(wcstr));
			printres(mbstr, testfmt, wexpstr, resbuf, NULLTEXT);
		}

		if(targ[wlen] != (wchar_t)NUL)
		{
			if(!testfail++)
				xx_rpt(FAILURE);
			in_rpt("Expected terminating null wide character");
			in_rpt("Actual terminating character was >%s<", 
				pr_wchar(targ[wlen]));
		}
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}

private void
test27()
{
	DBUG_ENTER("test27");

	(void) cppair(ch_t27, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t27()
{
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3)
	wchar_t	wc;
	char	tbuf[50];
	int	i, j, err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t27");

	errno = NOERROR;

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';

		globok = 0;
		if (test_prep(t_nlfmt[tet_thistest], tbuf) == FAIL)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	
		wc = (wchar_t)NUL;
		testfail = 0;
	
		SCANIT(&wc);
	
		if(wc != *(wcstr + i))
		{
			(void) strcpy(resbuf, pr_wchar(wc));
			(void) strcpy(wexpstr, pr_wchar(*(wcstr + i)));
			printres(tbuf, testfmt, wexpstr, resbuf, NULLTEXT);
		}
		else
			PATH_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(1 + i * 2);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}

private void
test28()
{
	DBUG_ENTER("test28");

	(void) cppair(ch_t28, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t28()
{
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3)
	static	wchar_t	wbuf[] = { '\n', '\t', '\r', '\f', ' ', 0 };
	wchar_t	wc;
	char	tbuf[50];
	int	i, j, err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t28");

	errno = NOERROR;

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	for(i = 0; *(wbuf + i) != (wchar_t)NUL; i++)
	{
		if((j = wctomb(tbuf, *(wbuf + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';

		globok = 0;
		if (test_prep(t_nlfmt[tet_thistest], tbuf) == FAIL)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	
		wc = (wchar_t)NUL;
		testfail = 0;
	
		SCANIT(&wc);
	
		if(wc != *(wbuf + i))
		{
			(void) strcpy(resbuf, pr_wchar(wc));
			(void) strcpy(wexpstr, pr_wchar(*(wbuf + i)));
			printres(tbuf, testfmt, wexpstr, resbuf, NULLTEXT);
		}
		else
			PATH_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(1 + i * 2);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}

private void
test29()
{
	int		i;
	int		*p;
	char	expres[50], testdata[50];
	int	pathok = setupok;

	DBUG_ENTER("test29");

	(void) sprintf(testdata, "%p", (void *)&i);
	t_data[tet_thistest] = testdata;

	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	i = 0;

	testfail = 0;

	p = 0;
	SCANIT((void **)&p);

	if (p != &i)
	{
		(void) sprintf(expres, "%p", (void *)&i);
		(void) sprintf(resbuf, "%p", (void *)p);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test30()
{
	char	s1[BSIZ], s2[BSIZ];
	int	i;
	char	*expres = "2345, 5, abcde";
	int	pathok = setupok;

	DBUG_ENTER("test30");

	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	s1[0] = s2[0] = NUL;
	i = 0;
	SCANIT(&i _ s2 _ s1);

	s1[BSIZ - 1] = s2[BSIZ - 1] = NUL;
	if (i != 5 || strcmp(s1, "2345") != 0 || strcmp(s2, "abcde") != 0)
	{
		(void) sprintf(resbuf, "%s, %d, %s", s1, i, s2);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (tretval != 2)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, "return value = 2", resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


private void
test31()
{
#if TEST_XPG == 3
	char		s1[BSIZ];
	char		s2[BSIZ];
	char	*exp1res = "\"abcdef\"";
	char	*exp2res = "\"a\"";
	int	pathok = setupok;

	DBUG_ENTER("test31");

	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	s1[0] = NUL;
	SCANIT(s1);

	if (strcmp(s1, "abcdef"))
	{
		(void) sprintf(resbuf, "\"%s\"", s1);
		printres(t_data[tet_thistest], testfmt, exp1res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	testfmt = x1_fmt;
	s2[0] = NUL;
	SCANIT(s2);

	s1[BSIZ - 1] = s2[BSIZ - 1] = NUL;
	if (strcmp(s2, "a"))
	{
		(void) sprintf(resbuf, "\"%s\"", s2);
		printres(t_data[tet_thistest], testfmt, exp2res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}


private void
test32()
{
	char		s[BSIZ];
	char	*expres = "\"abcd\"";
	int	pathok = setupok;

	DBUG_ENTER("test32");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	s[0] = NUL;
	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (strcmp(s, "abcd"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;
	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test33()
{
	char		s[BSIZ];
	char	*expres = "\"abcdefghij-k\"";
	int	pathok = setupok;

	DBUG_ENTER("test33");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	s[0] = NUL;
	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (strcmp(s, "abcdefghij-k"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;
	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test34()
{
#if TEST_XPG == 3
	char		s2[BSIZ];
	char		s1[BSIZ];
	char	*exp1res = "\"abcd\"";
	char	*exp2res = "\"abcdefghij\"";
	int	pathok = setupok;

	DBUG_ENTER("test34");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	s1[0] = NUL;
	SCANIT(s1);

	if (strcmp(s1, "abcd"))
	{
		(void) sprintf(resbuf, "\"%s\"", s1);
		printres(t_data[tet_thistest], testfmt, exp1res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	testfmt = x2_fmt;
	s2[0] = NUL;
	SCANIT(s2);

	s1[BSIZ - 1] = s2[BSIZ - 1] = NUL;
	if (strcmp(s2, "abcdefghij"))
	{
		(void) sprintf(resbuf, "\"%s\"", s2);
		printres(t_data[tet_thistest], testfmt, exp2res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}


private void
test35()
{
	char		s1[BSIZ];
	char		s2[BSIZ];
	char	*exp1res = "\"a-bcd\"";
	char	*exp2res = "\"a-bcdef\"";
	int	pathok = setupok;

	DBUG_ENTER("test35");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	s1[0] = NUL;
	SCANIT(s1);

	if (strcmp(s1, "a-bcd"))
	{
		(void) sprintf(resbuf, "\"%s\"", s1);
		printres(t_data[tet_thistest], testfmt, exp1res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;


	testfmt = x3_fmt;
	s2[0] = NUL;
	SCANIT(s2);

	s1[BSIZ - 1] = s2[BSIZ - 1] = NUL;
	if (strcmp(s2, "a-bcdef"))
	{
		(void) sprintf(resbuf, "\"%s\"", s2);
		printres(t_data[tet_thistest], testfmt, exp2res, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


private void
test36()
{
	char		s[BSIZ];
	char	*expres = "\"ab]cd]e\"";
	int	pathok = setupok;

	DBUG_ENTER("test36");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	s[0] = NUL;
	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (strcmp(s, "ab]cd]e"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test37()
{
	char		s[BSIZ];
	char	*expres = "\"ab\"";
	int	pathok = setupok;

	DBUG_ENTER("test37");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	s[0] = NUL;
	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (strcmp(s, "ab"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;
	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test38()
{
	char		s[BSIZ];
	char	*expres = "return value = 0";
	int	pathok = setupok;

	DBUG_ENTER("test38");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	s[0] = NUL;
	SCANIT(s);

	s[BSIZ - 1] = NUL;
	if (s[0] != NUL || tretval != 0)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test39()
{
	long		l1, l2, l3, l4, n;
	unsigned long	u1;
	double		d1, d2, d3;
	int		expn;
	static char	expnres[120] = "1234567890, 1234567890, 1234567890, 1234567890, 1234567890, 123456.123456, 1.23e22, 1.23e22, ";
	char		*exp2nres = "return value = 8";
	int	pathok = setupok;

	DBUG_ENTER("test39");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	DBUG_PRINTF("test39", "address for %%n = %p", (void *)&n);

	l1 = l2 = l3 = l4 = 0;
	u1 = 0;
	d1 = d2 = d3 = 0.0;
	n = 0;

	SCANIT(&l2 _ &l3 _ &u1 _ &l4 _ &d1 _ &d2 _ &d3 _ &n _&l1);

	expn = strlen(t_data[tet_thistest]);
	if (l1 != 1234567890 || l2 != 1234567890 || l3 != 1234567890
		|| u1 != (unsigned long) 1234567890
		|| l4 != 1234567890 || !dcomp(d1, 123456.123456)
		|| !dcomp(d2, 1.23e22) || !dcomp(d3, 1.23e22) || n != expn)
	{
		(void) sprintf(resbuf, "%ld, %ld, %ld, %lu, %ld, %f, %g, %g, %ld", l1, l2, l3, u1, l4, d1, d2, d3, n);
		(void) strcat(expnres, itoa(expn));
		printres(t_data[tet_thistest], testfmt, expnres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (tretval != 8)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, exp2nres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test40()
{
	short		h1, h2, h3, h4, h5, n;
	unsigned short	u1;
	int		expn;
	static char	expres[60] = "12345, 12345, 12345, 12345, 12345, 12345, ";
	int	pathok = setupok;

	DBUG_ENTER("test40");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	expn = strlen(t_data[tet_thistest]);
	(void) strcat(expres, itoa(expn));

	testfail = 0;

	h1 = h2 = h3 = h4 = h5 = 0;
	u1 = 0;
	n = 0;

	SCANIT(&h2 _ &h3 _ &u1 _ &h4 _ &h5 _ &n _ &h1);

	if (h1 != 12345 || h2 != 12345 || h3 != 12345 || u1 != 12345
		|| h4 != 12345 || h5 != 12345 || n != expn)
	{
		(void) sprintf(resbuf, "%d, %d, %d, %u, %d, %d, %d", h1, h2, h3, u1, h4, h5, n);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (tretval != 6)
	{
		(void) sprintf(resbuf, "return value = %d", tretval);
		printres(t_data[tet_thistest], testfmt, "return value = 6", resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test41()
{
	int		i1, i2, i3;
	unsigned int	u1;
	float		f1, f2, f3;
	char		s1[BSIZ], s2[BSIZ];
	char	*expnres = "12345, 12345, 12345, 12345, 456.12, 11.23, 21.23, \"cde\", \"ftest\"";
	static char *warntext[] = {
	    "Warning:- this implementation does not correctly implement",
	    "the (n) format modifier for floating point numbers with exponent.",
	    NULL
	};
	int	pathok = setupok;

	DBUG_ENTER("test41");

	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	i1 = i2 = i3 = 0;
	u1 = 0;
	f1 = f2 = f3 = 0.0;
	s1[0] = s2[0] = NUL;

	SCANIT(&i2 _ &u1 _ &i3 _ &f1 _ &f2 _ &f3 _ s1 _ s2 _ &i1);

	s1[BSIZ - 1] = s2[BSIZ - 1] = NUL;
	if (i1 != 12345 || i2 != 12345 || u1 != 12345 || i3 != 12345
		|| !fcomp(f1, 456.12) || !fcomp(f2, 11.23)
		|| !fcomp(f3, 21.23) || strcmp(s1, "cde")
		|| strcmp(s2, "ftest"))
	{
		(void) sprintf(resbuf, "%d, %d, %u, %d, %f, %f, %f, \"%s\", \"%s\"", i1, i2, u1, i3, f1, f2, f3, s1, s2);
		printres(t_data[tet_thistest], testfmt, expnres, resbuf, warntext);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test42()
{
	char	s[BSIZ];
	char	*expres = "\"abc\"";
	int	pathok = setupok;

	DBUG_ENTER("test42");
	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	testfail = 0;

	s[0] = s[1] = s[2] = NUL;
	SCANIT(s);

	s[3] = NUL;
	if (strcmp(s, "abc"))
	{
		(void) sprintf(resbuf, "\"%s\"", s);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test43()
{
	DBUG_ENTER("test43");

	(void) cppair(ch_t43, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t43()
{
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3)
	wchar_t	ws[BUFFERSIZE];
	char	wfmt[20];
	int	pathok = setupok;

	DBUG_ENTER("ch_t43");

	errno = NOERROR;

	globok = 0;
	wclocale();
	PATH_FUNC_TRACE;

	(void) sprintf(wfmt, "%%%dC", wlen);

	globok = 0;
	if (test_prep(wfmt, mbstr) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	ws[0] = (wchar_t)NUL;
	testfail = 0;
	SCANIT(ws);
	ws[wlen] = (wchar_t)NUL;
	
	if(wcscmp(ws, wcstr))
	{
		(void) strcpy(resbuf, pr_wstr(ws));
		(void) strcpy(wexpstr, pr_wstr(wcstr));
		printres(mbstr, testfmt, wexpstr, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}

private void
test44()
{
#if	LONG_DOUBLE_SUPPORTED

	long double	d1, d2, d3, d4, d5;
	static char	expres[] = "123.123, 1.23e+22, 1.23e+22, 1.23e+22, 1.23e+22";
	int	pathok = setupok;

	DBUG_ENTER("test44");

	globok = 0;
	if (test_prep( t_nlfmt[tet_thistest], t_data[tet_thistest]) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	d1 = d2 = d3 = d4 = d5 = 0.0L;

	testfail = 0;

	SCANIT(&d2 _ &d3 _ &d4 _ &d5 _ &d1);

	if (!ldcomp(d1, 123.123L) || !ldcomp(d2, 1.23e22L)
		|| !ldcomp(d3, 1.23e22L) || !ldcomp(d4, 1.23e22L)
		|| !ldcomp(d5, 1.23e22L))
	{
		(void) sprintf(resbuf, "%.3Lf, %Lg, %Lg, %Lg, %Lg", d1, d2, d3, d4, d5);
		printres(t_data[tet_thistest], testfmt, expres, resbuf, NULLTEXT);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;

#else /* LONG_DOUBLE_SUPPORTED */

	DBUG_ENTER("test44");

	xx_rpt(UNSUPPORTED);
	in_rpt("long double type is not supported");

	DBUG_VOID_RETURN;

#endif	/* LONG_DOUBLE_SUPPORTED */

}

#endif /* TEST_XNFS */

private void
printres(data, format, expres, obsres, warntext)
char *data, *format, *expres, *obsres;
char **warntext;
{
	char	*func;

	DBUG_ENTER("printres");

	if (!testfail++)
	{
		if (warntext != NULLTEXT)
		{
			char **s;
			xx_rpt(WARNING);
			for (s = warntext; *s != NULL; s++)
				in_rpt(*s);
		}
		else
			xx_rpt(FAILURE);
	}

	switch(interface)
	{
	case	I_SCANF:
		func = "scanf(";
		break;
	case	I_FSCANF:
		func = "fscanf(t_fp, ";
		break;
	case	I_SSCANF:
		func = "sscanf(data, ";
		break;
	}

	in_rpt("%s\"%s\") failed", func, format);
#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) && !TEST_XNFS
	if(wflag && *locale != '\0')
		in_rpt("Current language is %s", locale);
	wflag = 0;
#endif
	in_rpt("Input data: %s", psl_to_asc(data, 0));
	in_rpt("Expected result: %s", expres);
	in_rpt("Observed result: %s", obsres);

	DBUG_VOID_RETURN;
}

#if defined(_XOPEN_SOURCE) && (TEST_XPG > 3) && !TEST_XNFS
private void
wclocale()
{
	int	pathok = 0;

	DBUG_ENTER("wclocale");

	/*
	 * Change to wide character locale
	 */
	if (setlocale(LC_CTYPE, locale) == (char *) NULL)
	{
		in_rpt("Pseudo Language %s is not available", locale);
		in_rpt("Check installation of pseudo language");
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	wflag = 1;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}
#endif
