/*
*      SCCS:  @(#)  ANSI.os/streamio/setbuf/setbuf.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/setbuf - T.setbuf Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.setbuf

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the setbuf() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Tue Dec 1 17:08:25 BST 1986
MODIFICATIONS:	Andrew Twigger 1 July 87
		Removed abandon(), testid()
		Modified xx_rpt(), xs_rpt()
		Phyfos Photi, Hoskyns Group plc
		30/08/88
		test 5 added

		Geoff Clare, 11 Sep 1989
			Remove fseek() in test1 so 2nd fread() will read
			from the existing buffer
		
		Geoff Clare, 20 Sep 1989
			Allow for implentations which don't use the whole
			buffer for data.

		David Sawyer
		Wed Jul 18 12:40:28 BST 1990
		Added resillience code.

		Stuart Boutell, 2 Apr 1991
			Modified comment in test 1

                John-Paul, 27 July 1992
                        mode variables changed from 'int' to 'mode_t'.
		
		Geoff Clare, 13 Aug 1992
			Changes to allow for buffers larger than BUFSIZ.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<string.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef setbuf
#endif


#define NO_TESTS		5		/* number of tests in testset */
#define MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define T2_NBUFS	20


extern	char	*errname();

int	no_tests = NO_TESTS;

private char	*t1_file =	"./setbuf-t.1";
private char	*t2_file =	"./setbuf-t.2";
private char	*t3_file =	"./setbuf-t.3";
private char	*t4_file =	"./setbuf-t.4";
private char	*t5_file =	"./setbuf-t.5";

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	NULL, 0
};

private void
test1()
{
	FILE		*fp;
	int		fd;
	char		buf[BUFSIZ];
	char		buf1[BUFSIZ];
	char		buf2[BUFSIZ];
	int		i;
	int		err;
	int		pathok = 0;

	DBUG_ENTER("test1");
	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
		buf2[i] = '\0';
	}
	errno = NOERROR;
	if ((fd = crfile(t1_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to create file %s", t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to write to file %s", t1_file);
		(void) close(fd);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fp = fopen(t1_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t1_file, err, errname(err));
		(void) close(fd);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	setbuf(fp, buf2);

	/*
	 * Some implementations don't use the whole buffer
	 * for data - hence do two reads of BUFSIZ/3 instead of BUFSIZ/2
	 * (should be safe to assume > two-thirds is used for data)
	 */

	if (fread((void *)buf, (size_t) sizeof(char),
			(size_t) BUFSIZ/3, fp) != (BUFSIZ/3))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fread (%s) failed - errno %d(%s)",
			t1_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	for (i=0; i<BUFSIZ/3; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf[i] != 'a')
		{
			xx_rpt(FAILURE);
			in_rpt("fread() buffer contained wrong character (%c) at position %d", buf[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t1_file);
			DBUG_VOID_RETURN;
		}
	}
	(void) lseek(fd, (off_t)0, SEEK_SET);
	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'b';
	}

	if (write(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to write to file %s", t1_file);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* (void) fseek(fp, 0L, SEEK_SET); deleted 11/9/89, GWC */
	if (fread((void *)buf, (size_t) sizeof(char),
			(size_t) BUFSIZ/3, fp) != (BUFSIZ/3))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fread (%s) failed - errno %d(%s)",
			t1_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/3; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf[i] != 'a')
		{
			xx_rpt(FAILURE);
			in_rpt("read returned wrong character (%c) at position %d", buf[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t1_file);
			DBUG_VOID_RETURN;
		}
	}

	PATH_XS_RPT(10);

	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t1_file);
	DBUG_VOID_RETURN;
}


private void
test2()
{
	FILE		*fp;
	int		fd;
	char		buf[BUFSIZ];
	char		buf1[BUFSIZ*2];
	char		buf2[BUFSIZ];
	int		i;
	int		err;
	int		pathok = 0;

	DBUG_ENTER("test2");

	for (i=0; i<(BUFSIZ*2); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t2_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to create file %s", t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ*2) != (BUFSIZ*2))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to write to file %s", t2_file);
		(void) close(fd);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ*2); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t2_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t2_file, err, errname(err));
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	setbuf(fp, buf);
	if ((fd = open(t2_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to open file %s", t2_file);
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t) sizeof(char),
			(size_t) BUFSIZ/2, fp) != (BUFSIZ/2))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t2_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to read from file %s", t2_file);
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t2_file);
			DBUG_VOID_RETURN;
		}
	}
	(void) lseek(fd, (off_t)0, SEEK_SET);
	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'c';
	}

	(void) fseek(fp, (long) BUFSIZ/2, SEEK_SET);

	/* stdio buffers can be any size, so write several times
	   BUFSIZ bytes to try and ensure the buffer will be filled */
	for (i = 0; i < T2_NBUFS; i++)
	{
		if (fwrite((void *)buf2, (size_t) sizeof(char),
				(size_t) BUFSIZ, fp) != (BUFSIZ))
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
				t2_file, err, errname(err));
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t2_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	for (i=0; i<(BUFSIZ*2); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	(void) lseek(fd, (off_t)0, SEEK_SET);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to read from file %s", t2_file);
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/2; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t2_file);
			DBUG_VOID_RETURN;
		}
	}
	for (i=BUFSIZ/2; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/2)
			PATH_TRACE;
		if (buf1[i] != 'c')
		{
			xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t2_file);
			DBUG_VOID_RETURN;
		}
	}

	PATH_XS_RPT(15 + T2_NBUFS);

	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t2_file);
	DBUG_VOID_RETURN;
}

private void
test3()
{
	FILE		*fp;
	int		fd;
	char		buf[BUFSIZ];
	char		buf1[BUFSIZ];
	char		buf2[BUFSIZ];
	int		i;
	int		err;
	int		failed=0;
	int		pathok = 0;

	DBUG_ENTER("test3");

	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t3_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to create file %s", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to write to file %s", t3_file);
		(void) close(fd);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t3_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t3_file, err, errname(err));
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	setbuf(fp, buf);

	if ((fd = open(t3_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to open file %s", t3_file);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t) sizeof(char),
			(size_t) BUFSIZ/2, fp) != (BUFSIZ/2))
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t3_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fflush(fp);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: failed to read from file %s", t3_file);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/2; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fflush()", buf1[i], i);
			failed++;
			break;
		}
	}
	for (i=BUFSIZ/2; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/2)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fflush()", buf1[i], i);
			failed++;
			break;
		}
	}
	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t3_file);

	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t3_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: failed to create file %s", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: failed to write to file %s", t3_file);
		(void) close(fd);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t3_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t3_file, err, errname(err));
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	setbuf(fp, buf);

	if ((fd = open(t3_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: failed to open file %s", t3_file);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t) sizeof(char),
			(size_t) BUFSIZ/2, fp) != (BUFSIZ/2))
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t3_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fseek(fp, 0L, SEEK_SET);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: failed to read from file %s", t3_file);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/2; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fflush()", buf1[i], i);
			failed++;
			break;
		}
	}
	for (i=BUFSIZ/2; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/2)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fflush()", buf1[i], i);
			failed++;
			break;
		}
	}
	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t3_file);

	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t3_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: failed to create file %s", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: failed to write to file %s", t3_file);
		(void) close(fd);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t3_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t3_file, err, errname(err));
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	setbuf(fp, buf);

	if ((fd = open(t3_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: failed to open file %s", t3_file);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t) sizeof(char),
			(size_t) BUFSIZ/2, fp) != (BUFSIZ/2))
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t3_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fclose(fp);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: failed to read from file %s", t3_file);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/2; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fflush()", buf1[i], i);
			failed++;
			break;
		}
	}
	for (i=BUFSIZ/2; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/2)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fflush()", buf1[i], i);
			failed++;
			break;
		}
	}
	(void) close(fd);
	(void) unlink(t3_file);

	if (!failed)
		PATH_XS_RPT(33);
	DBUG_VOID_RETURN;
}


private void
test4()
{
	FILE		*fp;
	int		fd;
	char		buf1[BUFSIZ];
	char		buf2[BUFSIZ];
	int		i;
	int		err;
	int		pathok = 0;

	DBUG_ENTER("test4");
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t4_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to create file %s", t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to write to file %s", t4_file);
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t4_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t4_file, err, errname(err));
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	setbuf(fp, (char *) NULL);

	if ((fd = open(t4_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to open file %s", t4_file);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t) sizeof(char),
			(size_t) BUFSIZ/2, fp) != (BUFSIZ/2))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t4_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to read from file %s", t4_file);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/2; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t4_file);
			DBUG_VOID_RETURN;
		}
	}
	for (i=BUFSIZ/2; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/2)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t4_file);
			DBUG_VOID_RETURN;
		}
	}

	PATH_XS_RPT(11);

	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t4_file);
	DBUG_VOID_RETURN;
}

private	void
test5()
{
	char		*tmp = "test\nstring\n";
	int		err;
	FILE		*fp;
	struct	stat	statb;
	int		pathok = 0;

	DBUG_ENTER("test5");

	if ((fp = fopen(t5_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fwrite((void *)tmp, (size_t)1, strlen(tmp), fp) != strlen(tmp))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t5_file, err, errname(err));
	}
	else if (stat(t5_file, &statb))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed", t5_file);
	}
	else if (statb.st_size != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("stream associated with file %s is not fully buffered", t5_file);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	(void) unlink(t5_file);

	DBUG_VOID_RETURN;	
}
