/*
*      SCCS:  @(#)  ANSI.os/streamio/setvbuf_X/setvbuf_X.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/setvbuf_X - T.setvbuf_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.setvbuf_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the setvbuf() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Tue Dec 1 17:08:25 BST 1986
MODIFICATIONS:	Andrew Twigger 1 July 87
		Removed abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 31 Aug 1989
			Spurious fopen(t2_file) in test3 removed
			Change 2 lseek()s to fseek() in test6

		David Sawyer
		Wed Jul 18 13:32:20 BST 1990
		Added resillience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated reason().

		J.A. Nave	Mon Feb  4 17:27:13 GMT 1991
			Added test for line buffering on input.
			Added test for null argument.

		Geoff Clare, 30 Jan 1992
			Rewrite test for line buffering on input.

		Geoff Clare, 6 Mar 1992
			Add fflush() between fwrite() and fread() in test4.

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		Geoff Clare, 13 Aug 1992
			Changes to allow for buffers larger than BUFSIZ.

		Geoff Clare, 28 June 1994
			Check for termios_prep() returning 1.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<termios.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef setvbuf
#endif


#define NO_TESTS	9		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define	EXTRABIT	64
#define	RD		0
#define	WR		1
#define	LINELEN		80
#define T4_NBUFS	20


extern	char	*errname();
extern	char	*pr_char();
extern	char	*pr_nchars();
extern	int	alrm_flag;

int	no_tests = NO_TESTS;

private char	*t1_file =	"./setvbuf-t.1";
private char	*t2_file =	"./setvbuf-t.2";
private char	*t3_file =	"./setvbuf-t.3";
private char	*t4_file =	"./setvbuf-t.4";
private char	*t5_file =	"./setvbuf-t.5";
private char	*t6_file =	"./setvbuf-t.6";
private char	*t7_file =	"./setvbuf-t.7";
private char	*t8_file =	"./setvbuf-t.8";
private char	*t9_file =	"./setvbuf-t.9";
private	int	testfail;

private void	clean_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
#if TEST_XPG > 3
private void	ch_t7();
#endif

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	NULL, 0
};


private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

	(void) unlink(t1_file);
	(void) unlink(t2_file);
	(void) unlink(t3_file);
	(void) unlink(t4_file);
	(void) unlink(t5_file);
	(void) unlink(t6_file);
	(void) unlink(t7_file);
	(void) unlink(t8_file);
	(void) unlink(t9_file);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	FILE	*fp;
	int	fd;
	int	rval;
	char	buf[BUFSIZ];
	char	buf1[BUFSIZ];
	char	buf2[BUFSIZ];
	int	i;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;
	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
		buf2[i] = '\0';
	}
	errno = NOERROR;
	if ((fd = crfile(t1_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t1_file);
		(void) close(fd);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fp = fopen(t1_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t1_file, err, errname(err));
		(void) close(fd);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, buf2, _IOFBF, (BUFSIZ/2)+EXTRABIT)) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fread((void *)buf, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fread (%s) failed - errno %d(%s)",
			t1_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) lseek(fd, (off_t)0, SEEK_SET);
	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'b';
	}

	if (write(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t1_file);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fread((void *)buf, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fread (%s) failed - errno %d(%s)",
			t1_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf[i] != 'a')
		{
			if (!testfail)
				xx_rpt(FAILURE);
			in_rpt("read returned wrong character (%c) at position %d", buf[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t1_file);
			DBUG_VOID_RETURN;
		}
	}

	PATH_XS_RPT(10);

	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t1_file);
	DBUG_VOID_RETURN;
}


private void
test2()
{
	FILE	*fp;
	int	fd;
	int	rval;
	char	buf[BUFSIZ];
	char	buf1[BUFSIZ];
	char	buf2[BUFSIZ];
	int	i;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test2");
	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
		buf2[i] = '\0';
	}
	errno = NOERROR;
	if ((fd = crfile(t2_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t2_file);
		(void) close(fd);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fp = fopen(t2_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t2_file, err, errname(err));
		(void) close(fd);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, buf2, _IONBF, BUFSIZ/2)) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fread((void *)buf, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fread (%s) failed - errno %d(%s)",
			t2_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf[i] != 'a')
		{
			xx_rpt(FAILURE);
			in_rpt("read returned wrong character (%c) at position %d", buf[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t2_file);
			DBUG_VOID_RETURN;
		}
	}

	PATH_XS_RPT(7);

	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t2_file);
	DBUG_VOID_RETURN;
}

private void
test3()
{
	FILE		*fp;
	int		fd;
	int		rval;
	char		buf1[BUFSIZ];
	char		buf2[BUFSIZ];
	int		i;
	int		err;
	int		pathok = 0;

	DBUG_ENTER("test3");
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t3_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t3_file);
		(void) close(fd);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t3_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t3_file, err, errname(err));
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, (char *) NULL, _IOFBF, BUFSIZ/2)) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fd = open(t3_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t3_file);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t3_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t3_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t3_file);
			DBUG_VOID_RETURN;
		}
	}
	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'c';
	}

	if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/2, fp) != (BUFSIZ/2))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t3_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	(void) fseek(fp, 0L, SEEK_SET);
	(void) lseek(fd, (off_t)0, SEEK_SET);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t3_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t3_file);
			DBUG_VOID_RETURN;
		}
	}
	for (i=BUFSIZ/4; i<BUFSIZ/2; i++)
	{
		if (i==BUFSIZ/4)
			PATH_TRACE;
		if (buf1[i] != 'c')
		{
			xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t3_file);
			DBUG_VOID_RETURN;
		}
	}

	PATH_XS_RPT(17);

	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t3_file);
	DBUG_VOID_RETURN;
}

private void
test4()
{
	FILE		*fp;
	int		fd;
	int		rval;
	char		buf[BUFSIZ];
	char		buf1[BUFSIZ];
	char		buf2[BUFSIZ];
	char		buf3[2];
	int		i;
	int		err;
	int		failed=0;
	int		pathok = 0;

	DBUG_ENTER("test4");
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	buf3[0] = '\n';
	errno = NOERROR;
	if ((fd = crfile(t4_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t4_file);
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t4_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t4_file, err, errname(err));
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, buf, _IOLBF, (BUFSIZ/2)+EXTRABIT)) != 0)
	{
		if (!failed)
			xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t4_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t4_file);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t4_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t4_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			failed++;
			break;
		}
	}
	if (fwrite((void *)buf3, (size_t)1, (size_t)1, fp) != 1)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t4_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) lseek(fd, (off_t)0, SEEK_SET);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t4_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			failed++;
			break;
		}
	}
	if (buf1[BUFSIZ/4] != '\n')
	{
		if (!failed)
			xx_rpt(FAILURE);
		in_rpt("buffer buf1 did not contain new line character at position %d", BUFSIZ/4);
		failed++;
	}
	else
		PATH_TRACE;
	for (i=1+(BUFSIZ/4); i<BUFSIZ; i++)
	{
		if (i==(1+BUFSIZ/4))
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			failed++;
			break;
		}
	}
	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t4_file);

	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t4_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t4_file);
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t4_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t4_file, err, errname(err));
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, buf, _IOLBF, (BUFSIZ/2)+EXTRABIT)) != 0)
	{
		if (!failed)
			xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t4_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t4_file);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t4_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t4_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			failed++;
			break;
		}
	}

	(void) fflush(fp);
	if (fread((void *)buf3, (size_t)1, (size_t)1, fp) != 1)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fread (%s) failed - errno %d(%s)",
			t4_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) lseek(fd, (off_t)0, SEEK_SET);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t4_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fread()", buf1[i], i);
			failed++;
			break;
		}
	}
	for (i=BUFSIZ/4; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/4)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fread()", buf1[i], i);
			failed++;
			break;
		}
	}
	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t4_file);

	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t4_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t4_file);
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t4_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t4_file, err, errname(err));
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, buf, _IOLBF, (BUFSIZ/2)+EXTRABIT)) != 0)
	{
		if (!failed)
			xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t4_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t4_file);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t4_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t4_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			failed++;
			break;
		}
	}
	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'c';
	}

	/* stdio buffers can be any size, so write several times
	   BUFSIZ bytes to try and ensure the buffer will be filled */
	for (i = 0; i < 2*T4_NBUFS; i++)
	{
		if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/2, fp)
			!= (BUFSIZ/2))
		{
			err = errno;
			if (!failed)
				xx_rpt(DELETION);
			in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
				t4_file, err, errname(err));
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t4_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	(void) lseek(fd, (off_t)0, SEEK_SET);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t4_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			failed++;
			break;
		}
	}
	for (i=BUFSIZ/4; i<BUFSIZ/2; i++)
	{
		if (i==BUFSIZ/4)
			PATH_TRACE;
		if (buf1[i] != 'c')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			failed++;
			break;
		}
	}
	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t4_file);

	if (!failed)
		PATH_XS_RPT(46 + 2*T4_NBUFS);
	DBUG_VOID_RETURN;
}

private void
test5()
{
	FILE		*fp;
	int		fd;
	int		rval;
	char		buf[BUFSIZ];
	char		buf1[BUFSIZ];
	char		buf2[BUFSIZ];
	int		i;
	int		err;
	int		pathok = 0;

	DBUG_ENTER("test5");
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t5_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t5_file);
		(void) close(fd);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t5_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, buf, _IONBF, BUFSIZ/2)) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t5_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t5_file);
		(void) fclose(fp);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t5_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t5_file);
			DBUG_VOID_RETURN;
		}
	}
	for (i=BUFSIZ/4; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/4)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d", buf1[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t5_file);
			DBUG_VOID_RETURN;
		}
	}

	PATH_XS_RPT(12);

	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t5_file);
	DBUG_VOID_RETURN;
}

private void
test6()
{
	FILE		*fp;
	int		fd;
	int		rval;
	char		buf[BUFSIZ];
	char		buf1[BUFSIZ];
	char		buf2[BUFSIZ];
	int		i;
	int		err;
	int		failed=0;
	int		pathok = 0;

	DBUG_ENTER("test6");
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t6_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t6_file);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t6_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, buf, _IOFBF, (BUFSIZ/2)+EXTRABIT)) != 0)
	{
		if (!failed)
			xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t6_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t6_file);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fflush(fp);
	(void) lseek(fd, (off_t)0, SEEK_SET);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t6_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fflush()", buf1[i], i);
			failed++;
			break;
		}
	}
	for (i=BUFSIZ/4; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/4)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fflush()", buf1[i], i);
			failed++;
			break;
		}
	}
	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t6_file);

	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t6_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t6_file);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t6_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, buf, _IOFBF, (BUFSIZ/2)+EXTRABIT)) != 0)
	{
		if (!failed)
			xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t6_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t6_file);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fseek(fp, (off_t) 0, SEEK_SET);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t6_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fseek()", buf1[i], i);
			failed++;
			break;
		}
	}
	for (i=BUFSIZ/4; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/4)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fseek()", buf1[i], i);
			failed++;
			break;
		}
	}
	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t6_file);

	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t6_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t6_file);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t6_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, buf, _IOFBF, (BUFSIZ/2)+EXTRABIT)) != 0)
	{
		if (!failed)
			xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t6_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t6_file);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fclose(fp);
	(void) lseek(fd, (off_t)0, SEEK_SET);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t6_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fclose()", buf1[i], i);
			failed++;
			break;
		}
	}
	for (i=BUFSIZ/4; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/4)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fclose()", buf1[i], i);
			failed++;
			break;
		}
	}
	(void) close(fd);
	(void) unlink(t6_file);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t6_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t6_file);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t6_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, buf, _IOLBF, (BUFSIZ/2)+EXTRABIT)) != 0)
	{
		if (!failed)
			xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t6_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t6_file);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fflush(fp);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t6_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fflush()", buf1[i], i);
			failed++;
			break;
		}
	}
	for (i=BUFSIZ/4; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/4)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fflush()", buf1[i], i);
			failed++;
			break;
		}
	}
	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t6_file);

	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t6_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t6_file);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t6_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, buf, _IOLBF, (BUFSIZ/2)+EXTRABIT)) != 0)
	{
		if (!failed)
			xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t6_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t6_file);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fseek(fp, (off_t) 0, SEEK_SET);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t6_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fseek()", buf1[i], i);
			failed++;
			break;
		}
	}
	for (i=BUFSIZ/4; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/4)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fseek()", buf1[i], i);
			failed++;
			break;
		}
	}
	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t6_file);

	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
	}
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf2[i] = 'b';
	}
	errno = NOERROR;
	if ((fd = crfile(t6_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != (BUFSIZ))
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t6_file);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	for (i=0; i<(BUFSIZ); i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = '\0';
	}
	if ((fp = fopen(t6_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, buf, _IOLBF, (BUFSIZ/2)+EXTRABIT)) != 0)
	{
		if (!failed)
			xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t6_file, O_RDONLY, MODEANY)) == SYSERROR)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t6_file);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fwrite((void *)buf2, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fwrite (%s) failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fclose(fp);
	if (read(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!failed)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not read from file %s", t6_file);
		(void) fclose(fp);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf1[i] != 'b')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fclose()", buf1[i], i);
			failed++;
			break;
		}
	}
	for (i=BUFSIZ/4; i<BUFSIZ; i++)
	{
		if (i==BUFSIZ/4)
			PATH_TRACE;
		if (buf1[i] != 'a')
		{
			if (!failed)
				xx_rpt(FAILURE);
			in_rpt("buffer buf1 contained wrong character (%c) at position %d after fclose()", buf1[i], i);
			failed++;
			break;
		}
	}
	(void) close(fd);
	(void) unlink(t6_file);

	if (!failed)
		PATH_XS_RPT(72);
	DBUG_VOID_RETURN;
}

private void
test7()
{
	DBUG_ENTER("test7");
#if TEST_XPG > 3
	(void) cppair(ch_t7, NULLFN, 2*WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
#endif
	DBUG_VOID_RETURN;
}

#if TEST_XPG > 3
private void
line_buf(filetype, rd_fd, wr_fd)
char *filetype;
int rd_fd, wr_fd;
{
	FILE *fp;
	int rval, err;
	static char line1[] = "aaaaa\n";
	static char line2[] = "bbbbb\n";
	char buf[BUFSIZ];
	int pathok = 0;

	DBUG_ENTER("line_buf");

	fp = fdopen(rd_fd, "r");
	if (fp == NULL)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("fdopen() failed on %s - errno %d (%s)", filetype,
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = setvbuf(fp, buf, _IOLBF, BUFSIZ)) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("setvbuf on %s returned %d", filetype, rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (write(wr_fd, line1, sizeof(line1)-1) != sizeof(line1)-1 ||
	    write(wr_fd, line2, sizeof(line2)-1) != sizeof(line2)-1)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() failed on %s - errno %d (%s)", filetype,
			err, errname(err));
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	(void) sleep(WAITTIME/2);

	errno = 0;
	if ((rval = getc(fp)) != line1[0])
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (rval == EOF)
			in_rpt("getc() failed on %s - errno %d (%s)",
				filetype, err, errname(err));
		else
			in_rpt("getc() on %s returned '%s', expected '%s'",
				filetype, pr_char(rval), pr_char(line1[0]));
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if ((rval = read(rd_fd, buf, BUFSIZ)) <= 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (rval <= 0)
			in_rpt("read() on %s returned %d", filetype, rval);
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (rval != strlen(line2) ||
		strncmp(buf, line2, (size_t)rval) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on %s returned \"%s\", expected \"%s\"",
			filetype, pr_nchars(buf, rval),
			pr_nchars(line2, sizeof(line2)-1));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	char	*supp;
	int	tty_fildes, loop_fildes, pfd[2], err;
	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
	int pathok = 0;

	DBUG_ENTER("ch_t7");

	testfail = 0;

	supp = tet_getvar("VSX_LINE_BUF_SUPP");
	if (supp == NULL || *supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_LINE_BUF_SUPP is not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*supp == 'N' || *supp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_LINE_BUF_SUPP is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else if (*supp != 'Y' && *supp != 'y' &&
	    *supp != 'T' && *supp != 't' &&
	    *supp != 'P' && *supp != 'p')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_LINE_BUF_SUPP is set to '%s'", supp);
		in_rpt("must be 'Y', 'N', 'T' or 'P'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*supp == 'Y' || *supp == 'y' ||
	    *supp == 'T' || *supp == 't')
	{
		switch (termios_prep(&tty_fildes, tty_termios_p,
				     &loop_fildes, loop_termios_p))
		{
		case 0:
			globok = 0;
			line_buf("terminal", tty_fildes, loop_fildes);
			PATH_FUNC_TRACE;
			break;

		case 1:
			/* GTI is not supported */
			/* (in which case LINE_BUF_SUPP is probably set wrong) */

			if (*supp == 'T' || *supp == 't')
			{
				xx_rpt(UNSUPPORTED);
				in_rpt("parameter VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
				in_rpt("and parameter VSX_LINE_BUF_SUPP is not set to 'Y' or 'P'");
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;
			break;

		default:
			if (testfail++ == 0)
				xx_rpt(DELETION);
			if (alrm_flag > 0)
				in_rpt("open/initialise VSX_TERMIOS_TTY/LOOP timed out");
			else
				in_rpt("could not open/initialise VSX_TERMIOS_TTY/LOOP");

		}
	}
	else
		PATH_TRACE;

	if (*supp == 'Y' || *supp == 'y' ||
	    *supp == 'P' || *supp == 'p')
	{
		if (pipe(pfd) != 0)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("pipe() failed - errno %d (%s)",
				err, errname(err));
		}
		else
		{
			int flags = fcntl(pfd[0], F_GETFL);
			(void) fcntl(pfd[0], F_SETFL, flags|O_NONBLOCK);

			globok = 0;
			line_buf("pipe", pfd[0], pfd[1]);
			PATH_FUNC_TRACE;
			
			(void) close(pfd[0]);
			(void) close(pfd[1]);
		}
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}
#endif


private void
test8()
{
	FILE	*fp;
	int	fd;
	int	rval;
	char	buf[BUFSIZ];
	char	buf1[BUFSIZ];
	char	buf2[BUFSIZ];
	int	i;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;
	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'a';
		buf2[i] = '\0';
	}
	errno = NOERROR;
	if ((fd = crfile(t8_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (write(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t8_file);
		(void) close(fd);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fp = fopen(t8_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t8_file, err, errname(err));
		(void) close(fd);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((rval = setvbuf(fp, (char *)NULL, _IOFBF, (BUFSIZ/2)+EXTRABIT)) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("setvbuf returned %d", rval);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (fread((void *)buf, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fread (%s) failed - errno %d(%s)",
			t8_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) lseek(fd, (off_t)0, SEEK_SET);
	for (i=0; i<BUFSIZ; i++)
	{
		if (i==0)
			PATH_TRACE;
		buf1[i] = 'b';
	}

	if (write(fd, buf1, BUFSIZ) != BUFSIZ)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t8_file);
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fread((void *)buf, (size_t)1, (size_t)BUFSIZ/4, fp) != (BUFSIZ/4))
	{
		err = errno;
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fread (%s) failed - errno %d(%s)",
			t8_file, err, errname(err));
		(void) close(fd);
		(void) fclose(fp);
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<BUFSIZ/4; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (buf[i] != 'a')
		{
			if (!testfail)
				xx_rpt(FAILURE);
			in_rpt("read returned wrong character (%c) at position %d", buf[i], i);
			(void) close(fd);
			(void) fclose(fp);
			(void) unlink(t8_file);
			DBUG_VOID_RETURN;
		}
	}

	PATH_XS_RPT(10);

	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t8_file);
	DBUG_VOID_RETURN;
}

private void
test9()
{
	FILE		*fp;
	int		fd;
	int		err;
	char		buf[BUFSIZ];
	int		pathok = 0;

	DBUG_ENTER("test9");
	errno = NOERROR;
	if ((fd = crfile(t9_file, (mode_t)MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fp = fopen(t9_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t9_file, err, errname(err));
		(void) close(fd);
		(void) unlink(t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (setvbuf(fp, buf, _IOFBF+_IONBF+_IOLBF+1, BUFSIZ/2) == 0)
	{
		xx_rpt(FAILURE);
		in_rpt("setvbuf returned 0 when type illegal");
		in_rpt("expected return value non-zero");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	(void) fclose(fp);
	(void) close(fd);
	(void) unlink(t9_file);
	DBUG_VOID_RETURN;
}

