/*
*      SCCS:  @(#)  ANSI.os/streamio/tmpfile/tmpfile.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/tmpfile - T.tmpfile Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.tmpfile

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the tmpfile() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 1 July 87
		Removed abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 22 Nov 1989
		Remove test that file is deleted after being closed.
		Rewrite EMFILE test based on equivalent test for fopen().

		Steve Ratcliffe, 15 Feb 1990
		Add test for ENFILE.

		Geoff Clare, 29 May 1990
			Allow for unlimited OPEN_MAX

		Peter Lee, Unisoft Ltd.,
		Wed Jul 18 17:17:42 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Stuart Boutell, 18 Apr 1991
			Modified test 1 to check whether tmpfile
		allocates fd's to the streams like open() does.

		Dave Cater, Thu Jun 11 09:02:07 BST 1992
			Added 4 new tests in line with P1003.3 D13.
			Three derived from creat, one from fopen.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<errno.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<string.h>
#include	<dbug.h>
#include	<report.h>
#include	<pcts_max.h>
#include	<exitcodes.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef tmpfile
#endif


#define	NO_TESTS		8		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define	NOK			2
#define	PATH_OK			4
#define	T4_NFILES		3

extern	char	*errname();

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;

extern	int 	vguid1;
extern	int 	vggid1;

extern	char	*newstr();
extern	char	*itoa();

int	no_tests = NO_TESTS;

private	int	testfail;
private	char	*t_string =	"test string";
private char	*t7_file =	"./tmpfile-t.7";
private char    *prog2 =	"../tmpfile_t2";
private char	*prog8 =	"../tmpfile_t8";

private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	test6();
private	void	test7();
private	void	test8();
private	void	ch_t1();
private	void	ch_t2();
private	void	ch_t7();
private	void	ch_t8();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	NULL, 0
};


private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	FILE	*fp;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	fd = open(".", O_RDONLY);
	if (fd < 0)
	{
		xx_rpt(DELETION);
		in_rpt("open(\".\", O_RDONLY) failed - errno %d (%s)",
			errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (close(fd) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("close() failed - errno %d (%s)",
			errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fp = tmpfile()) == (FILE *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("tmpfile() returned NULL");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fileno(fp) != fd)
	{
		xx_rpt(FAILURE);
		in_rpt("tmpfile() did not allocate lowest fd to stream");
		in_rpt("expected %d, observed %d", fd, fileno(fp));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(4);

	(void) fclose(fp);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t2");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		exit(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog2;
	args[1] = suid;
	args[2] = sgid;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(prog2, args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog2, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test3()
{
	FILE		*fp;
	int		p_rval;
	char		*g_rval;
	char		buf[64];
	int	pathok = 0;

	DBUG_ENTER("test3");
	if ((fp = tmpfile()) == (FILE *) NULL) {
		xx_rpt(DELETION);
		in_rpt("tmpfile() returned NULL");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	p_rval = fputs(t_string, fp);
	(void) fflush(fp);
	(void) fseek(fp, 0L, SEEK_SET);
	g_rval = fgets(buf, (int)strlen(t_string) + 1, fp);
	if (p_rval == EOF || g_rval == (char *) NULL
		|| strncmp(buf, t_string, strlen(t_string)))
	{
		xx_rpt(FAILURE);
		if (p_rval == EOF)
			in_rpt("puts() failed on stream returned by tmpfile()");
		if (g_rval == (char *) NULL)
			in_rpt("fgets() failed on stream returned by tmpfile()");
		DBUG_PRINTF("test3", ".%s.%s.", t_string, buf);
		if (g_rval != (char *) NULL
			&& strncmp(buf, t_string, strlen(t_string)))
			in_rpt("fgets() did not return expected value on stream returned by tmpfile()");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* tmpfile returns the lowest available file descriptor */

	int 	i, j, err;
	FILE	*fp[T4_NFILES+1];
	int	fd[T4_NFILES+1];
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	for (i = 0 ; i < T4_NFILES && testfail == 0 ; i++)
	{
		if (i==0)
			PATH_TRACE;
		if((fp[i] = tmpfile()) == (FILE *)NULL) {
			xx_rpt(DELETION);
			in_rpt("tmpfile() returned NULL");
			for (j = 0; j < i; j++)
				(void) fclose(fp[j]);
			DBUG_VOID_RETURN;
		}
		if((fd[i] = fileno(fp[i])) == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fileno() failed - errno %d(%s)",
				err, errname(err));
			for (j = 0; j < i; j++)
				(void) fclose(fp[j]);
			DBUG_VOID_RETURN;
		}
	}

	for (i = 1; i < T4_NFILES && testfail == 0; i++)
	{
		if (i==1)
			PATH_TRACE;
		if (fd[i-1] >= fd[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tmpfile() did not return lowest numbered file descriptor");
		}
	}

	if (testfail == 0)
	{
		if (fclose(fp[0]) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("fclose() failed - errno %d (%s)",
				err, errname(err));
		}
		else {
			if((fp[T4_NFILES] = tmpfile()) == (FILE *)NULL) {
				xx_rpt(DELETION);
				in_rpt("tmpfile() returned NULL");
				for (i = 1; i < T4_NFILES; i++)
					(void) fclose(fp[i]);
				DBUG_VOID_RETURN;
			}
			if((fd[T4_NFILES] = fileno(fp[T4_NFILES])) == SYSERROR) {
				err = errno;
				xx_rpt(DELETION);
				in_rpt("deletion reason: fileno() failed - errno %d(%s)",
					err, errname(err));
				for (i = 1; i < T4_NFILES+1; i++)
					(void) fclose(fp[i]);
				DBUG_VOID_RETURN;
			}
	
			if(fd[T4_NFILES] != fd[0])
			{
				xx_rpt(FAILURE);
				in_rpt("tmpfile() did not re-use closed file descriptor");
			}
			else
			{
				PATH_TRACE;
				PATH_XS_RPT(3);
			}

			(void) fclose(fp[T4_NFILES]);
		}
	}

	for (i = 1; i < T4_NFILES; i++)
		(void) fclose(fp[i]);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* opened files are set to remain open across exec calls */

	FILE	*fp;
	int 	fd, err, flags;
	int	pathok = 0;

	DBUG_ENTER("test5");

	if((fp = tmpfile()) == (FILE *)NULL) {
		xx_rpt(DELETION);
		in_rpt("tmpfile() returned NULL");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if((fd = fileno(fp)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fileno() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ((flags = fcntl(fd, F_GETFD, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl(F_GETFD) failed - errno %d (%s)",
			err, errname(err));
	}
	else if ((flags & FD_CLOEXEC) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("FD_CLOEXEC was set for opened file");
	}
	else
	{
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* tmpfile() alters timestamps on new file */

	FILE	*fp;
	int	fd, err, fail = 0;
	time_t	t0;
	struct stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	(void) time(&t0);

	if((fp = tmpfile()) == (FILE *)NULL) {
		xx_rpt(DELETION);
		in_rpt("tmpfile() returned NULL");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if((fd = fileno(fp)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fileno() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else 
		PATH_TRACE;

	if (fstat(fd, &stbuf) == SYSERROR)
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("fstat() failed - errno %d (%s)", err, errname(err));
	}
	else
	{
		if (stbuf.st_atime < t0)
			fail |= 01;
		else
			PATH_TRACE;

		if (stbuf.st_ctime < t0)
			fail |= 02;
		else
			PATH_TRACE;

		if (stbuf.st_mtime < t0)
			fail |= 04;
		else
			PATH_TRACE;
		
		if (fail != 0)
		{
			testfail++;
			xx_rpt(FAILURE);
			if (fail & 01)
				in_rpt("st_atime field on created file was not marked for update");
			if (fail & 02)
				in_rpt("st_ctime field on created file was not marked for update");
			if (fail & 04)
				in_rpt("st_mtime field on created file was not marked for update");
		}
	}

	if (testfail == 0)
		PATH_XS_RPT(5);

	(void) fclose(fp);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	int		fd;
	int		err;

	DBUG_ENTER("test7");
	if ((fd = creat(t7_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not creat() %s - errno %d (%s)",
			t7_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	(void) close(fd);

	(void) cppair(ch_t7, NULLFN, WAITTIME, 0);

	(void) unlink(t7_file);

	DBUG_VOID_RETURN;
}

private	void
ch_t7()
{
	long	open_max, count, loop_end;
	int	err, first;
	FILE	*fp;
	int	pathok = 0;

	DBUG_ENTER("ch_t7");

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0)
	{
		if (errno == 0)
		{
			PATH_TRACE;
			open_max = PCTS_OPEN_MAX+1;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: sysconf(_SC_OPEN_MAX) failed");
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

	loop_end = open_max;
	if (open_max > PCTS_OPEN_MAX)
		loop_end = PCTS_OPEN_MAX-1;

	for (first = count = n_open_fds(); count < loop_end; count++)
	{
		if (open(t7_file, O_RDWR) < 0)
		{
			err = errno;
			xx_rpt(DELETION);
			if (open_max <= PCTS_OPEN_MAX)
				in_rpt("could not open OPEN_MAX file descriptors");
			else
				in_rpt("could not open PCTS_OPEN_MAX-1 file descriptors");
			in_rpt("open(\"%s\", O_RDWR) failed - errno %d (%s)",
				t7_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else if (count == first)
			PATH_TRACE;
	}

	errno = NOERROR;
	fp = tmpfile();

	if (open_max <= PCTS_OPEN_MAX)
	{
		/* expect EMFILE error */
		if (fp != NULL || errno != EMFILE)
		{
			err = errno;
			xx_rpt(FAILURE);
			if (fp != NULL)
				in_rpt("tmpfile() succeeded when OPEN_MAX file descriptors were already open");
			else
			{
				in_rpt("tmpfile() did not set errno correctly");
				in_rpt("expected %d (EMFILE), received %d (%s)",
					EMFILE, err, errname(err));
			}
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(3);
		}
	}
	else
	{
		/* expect success */
		if (fp == NULL)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("tmpfile() failed when PCTS_OPEN_MAX-1 file descriptors were already open");
			in_rpt("errno was set to %d (%s)", err, errname(err));
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(3);
		}
	}

	DBUG_VOID_RETURN;
}

private void
test8()
{

	DBUG_ENTER("test8");

	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}


	(void) cppair(ch_t8, NULLFN, 8 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	char	*suid, *sgid;
	int 	err;
	char	*args[8];

	DBUG_ENTER("ch_t8");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		exit(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	args[0] = prog8;
	args[1] = "";
	args[2] = suid;
	args[3] = sgid;
	args[4] = "t";
	args[5] = vg_ident;
	args[6] = vg_dbug;
	args[7] = NULL;

	(void) tet_exec(prog8, args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %d (%s)", err, errname(err));

	DBUG_VOID_RETURN;
}
