/*
*      SCCS:  @(#)  ANSI.os/streamio/tmpfile/tmpfile_t2.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/tmpfile - tmpfile_t2 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		tmpfile_t2 - used by T.tmpfile in test2()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.tmpfile in test2().  This should
	  be setuid/setgid to VSX_UID1/VSX_GID1.
	  usage:
		tmpfile_t2 uid gid vg_ident vg_dbug

	  where uid, gid are VSX_UID1 and VSX_GID1.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		M Young, UniSoft Ltd.
DATE CREATED:	Mon Dec 02 12:08:18 GMT 1991
MODIFICATIONS:
		Geoff Clare, 12 June 1992
			Test FIPS requirement in XPG4 mode.

		Geoff Clare, 17 Sept 1992
			Remove group ID testing.  This is untestable as
			the group ID of the parent directory of the file
			cannot be determined.

************************************************************************/
#include <dbug.h>
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <stdio.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef tmpfile
#endif


#define MODEMASK	(S_IRWXU|S_IRWXG|S_IRWXO)
#define	T2_UMASK	(S_IWUSR|S_IRGRP|S_IXOTH)
#define	DEFMODE		(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	*errname();
extern	void	set_pname();

private void	subtest2();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	uid_t	uid;
	gid_t	gid;

	set_pname(argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		exit(OK);
	}
	if (argc >= 5)
		vg_dbug = argv[4];
	if (argc >= 4)
		vg_ident = argv[3];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if ((uid = atoi(argv[1])) != geteuid() ||
		(gid = atoi(argv[2])) != getegid())
	{
		xx_rpt(DELETION);
		if (uid != geteuid())
			in_rpt("deletion reason: uid is not equal to VSX_UID1");
		else
			in_rpt("deletion reason: gid is not equal to VSX_GID1");
	}
	else
	{
		subtest2(uid, gid);
	}

	exit(OK);
	/*NOTREACHED*/
}

#define	T2_INSTANCES	2

private void
subtest2(uid, gid)
uid_t	uid;
gid_t	gid;
{
	FILE		*fp;
	int		file_no;
	int		fail = 0;
	mode_t		mode1;
	mode_t		mode2;
	struct stat	statb;
	int		i, flag[T2_INSTANCES];
	int		err;
	int 		pathok = 0;

	DBUG_ENTER("subtest2");

	for ( i = 0; i < T2_INSTANCES; i++ )
		flag[i] = 0;
		
	if ((fp = tmpfile()) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: tmpfile() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if((file_no = fileno(fp)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fileno() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}

	if(fstat(file_no, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (statb.st_uid != uid)
		flag[0] |= 04;
	if ((mode1 = (statb.st_mode & MODEMASK)) != DEFMODE)
		flag[0] |= 020;

	if ( flag[0] != 0 )
		fail = 1;
	else
		PATH_TRACE;
		
	(void) fclose(fp);
	(void) umask((mode_t)T2_UMASK);

	if ((fp = tmpfile()) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: tmpfile() failed - errno %d(%s)",
			err, errname(err));
		(void) umask((mode_t)0);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) umask((mode_t)0);

	if((file_no = fileno(fp)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fileno() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}

	if(fstat(file_no, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fclose(fp);
		
	if (statb.st_uid != uid)
		flag[1] |= 04;
	if ((mode2 = (statb.st_mode & MODEMASK)) != (DEFMODE & ~T2_UMASK))
		flag[1] |= 020;

	if ( flag[1] != 0 )
		fail |= 2;
	else
		PATH_TRACE;
		
	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
		{
			if (flag[0] & 04)
				in_rpt("tmpfile() did not set user id correctly when umask was 0");
			if (flag[0] & 020)
				in_rpt("Note: There is a known issue with this test, see POSIX interpretation 1003.1-1990 #74");
				in_rpt("tmpfile() did not creat file with mode %#o when umask was set to 0, found %#lo",
				DEFMODE, (long) mode1);
		}
		else if (fail & 2)
		{
			{
				if (flag[1] & 04)
					in_rpt("tmpfile() did not set user id correctly when umask set to %#o", T2_UMASK);
				if (flag[1] & 020)
				{
					in_rpt("tmpfile() did not creat file with proper mask when umask was set to %#o", T2_UMASK);
					in_rpt("expected %#o, found %#lo", 
					DEFMODE & ~T2_UMASK, (long) mode2);
				}
			}
		}
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(7);
	}

	DBUG_VOID_RETURN;
}
