/*
*      SCCS:  @(#)  ANSI.os/streamio/tmpnam/tmpnam.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/tmpnam - T.tmpnam Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.tmpnam

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the tmpnam() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:54:58 WET 1986
MODIFICATIONS:	Andrew Twigger 2 July 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Linda Taylor, Hoskyns.	20th August, 1988.
		Test were removed to T.tempnam

		Geoff Clare, 21 Nov 1989
			Remove references to P_tmpdir

		Peter Lee, Unisoft Ltd.,
		Wed Jul 18 10:52:18 BST 1990
		Added resilience code.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<limits.h>
#include	<fcntl.h>
#include	<string.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef tmpnam
#endif


#define	NO_TESTS	3		/* number of tests in testset */

extern	char	*errname();

int	no_tests = NO_TESTS;

private	void	test1();
private	void	test2();
private	void	test3();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private void
test1()
{
	int		fd, err1, err2, fail = 0;
	char		buf[L_tmpnam];
	char		*rval;
	int	pathok = 0;

	DBUG_ENTER("test1");

	rval = tmpnam(buf);
	if (rval != buf)
		fail |= 01;
	else
		PATH_TRACE;

	fd = open(buf, O_WRONLY|O_CREAT|O_EXCL, (mode_t)0);
	if (fd < 0)
	{
		fail |= 02;
		err1 = errno;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}

	rval = tmpnam((char *)NULL);
	fd = open(rval, O_WRONLY|O_CREAT|O_EXCL, (mode_t)0);
	if (fd < 0)
	{
		fail |= 04;
		err2 = errno;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 01)
			in_rpt("tmpnam(buf) did not return buf");
		if (fail & 02)
			in_rpt("open(O_CREAT|O_EXCL) of file returned by tmpnam(buf) failed - errno %d (%s)",
				err1, errname(err1));
		if (fail & 04)
			in_rpt("open(O_CREAT|O_EXCL) of file returned by tmpnam(NULL) failed - errno %d (%s)",
				err2, errname(err2));
	}
	else
		PATH_XS_RPT(3);

	(void) unlink(buf);
	(void) unlink(rval);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	char		buf1[L_tmpnam];
	char		buf2[L_tmpnam];
	char		buf3[L_tmpnam];
	char		*rval1;
	char		*rval2;
	char		*rval3;
	int	pathok = 0;

	DBUG_ENTER("test2");
	rval1 = tmpnam(buf1);
	rval2 = tmpnam(buf2);
	rval3 = tmpnam(buf3);
	if (strcmp(rval1, rval2) == 0 || strcmp(rval2, rval3) == 0
		|| strcmp(rval1, rval3) == 0)
	{
		xx_rpt(FAILURE);
		in_rpt("successive calls of tmpnam(buf) did not yield different results");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}


private void
test3()
{
	char		*rval1;
	char		*rval2;
	int	pathok = 0;

	DBUG_ENTER("test3");
	rval1 = tmpnam((char *) NULL);
	rval2 = tmpnam((char *) NULL);
	if (rval1 != rval2)
	{
		xx_rpt(FAILURE);
		in_rpt("successive calls of tmpnam(NULL) did not return the same pointer");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}
