/*
*      SCCS:  @(#)  ANSI.os/streamio/ungetc/ungetc.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/ungetc - T.ungetc Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.ungetc

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the ungetc() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 1 July 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Peter Lee, Unisoft Ltd.,
		Wed Jul 18 11:37:06 BST 1990
		Added resilience code.

		Stuart Boutell, 18 Apr 1991
			Modified test 1 to ungetc() multiple characters
		and verify they are getc()'ed in the reverse order of
		pushing.

		Geoff Clare, 16 June 1993
			Test 7 not in use in POSIX mode.

		Geoff Clare, 25 Oct 1994
			Avoid freopen() of closed stream in test 1.

		Geoff Clare, 22 Nov 1994
			In test 3, retry ungetc() calls until one gives EOF.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<unistd.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ungetc
#endif


#define	NO_TESTS	7		/* number of tests in testset */
#define	WAITTIME	(SPEEDFACTOR * 5)
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define	T1_CHAR		'u'
#define	T2_CHAR		'v'
#define	T3_CHAR		'w'
#define	T4_CHAR		'x'
#define	T5_CHAR		'y'
#define	T6_CHAR		'z'

#define T7_CHAR	(int) 'Z'
#define T8_CHAR	(int) 'X'
#define SAMPLE 5
#define T3_TRIES	200

extern	void	setsigs();
extern	char	*signame();
extern	char	*pr_char();
extern	char	*errname();

int	no_tests = NO_TESTS;

private	char	*t_file =	"./ungetc-t";
private	char	*alphabet =	"abcdefghijklmnopqrstuvwxyz";

private int	tb_ungetc();
private	void	comp_prep();
private	void	comp_fin();
private	void	t3_catch();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private void	test5();
private	void	test6();
private	void	test7();
private	void	ch_t3();

public	void	(*setfunc) () = comp_prep;
public	void	(*clnfunc) () = comp_fin;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	NULL, 0
};

private void
comp_prep()
{
	int		fd;
	int		i;
	int		bad = 0;
	static char	msg[1024];

	if ((fd = creat(t_file, MODEANY)) == SYSERROR)
	{
		bad = 1;
		(void) sprintf(msg, "creat(%s, %#o) failed in preperations - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
	}
	else
	{
		if (write(fd, alphabet, 26) != 26)
		{
			(void) sprintf(msg, "write() failed in preperations - errno %d(%s)",
				errno, errname(errno));
			bad = 1;
		}
		(void) close(fd);
	}
	if (bad)
		for (i = 1 ; i <= NO_TESTS ; i++)
			tet_delete(i, msg);
}

private	void
comp_fin()
{
	(void) unlink(t_file);
}


private void
test1()
{
	FILE		*fp;
	int		err;
	int		val1;
	int		val2;
	int		val3;
	int		val5;
	int		val6;
	int		fail = 0;
	int		mfail = 0;
	long		off;
	long		diff;
	char		buf[64];
	int		i;
	int	pathok = 0;

	DBUG_ENTER("test1");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	for (i = 0 ; i < 3 ; i++)
		(void) getc(fp);
	if (ungetc(T1_CHAR, fp) != T1_CHAR)
		fail = 1;
	else
		PATH_TRACE;
	val1 = getc(fp);

	if (ungetc(T2_CHAR, fp) != T2_CHAR)
		fail = 1;
	else
		PATH_TRACE;
	val2 = fgetc(fp);

	if (ungetc(T3_CHAR, fp) != T3_CHAR)
		fail = 1;
	else
		PATH_TRACE;
	(void) fgets(buf, (int) sizeof(buf), fp);
	val3 = buf[0];

	rewind(fp);
	for (i = 0 ; i < 3 ; i++)
		(void) getc(fp);
	off = ftell(fp);
	if (ungetc(T4_CHAR, fp) != T4_CHAR)
		fail = 1;
	else
		PATH_TRACE;
	(void) fread((void *)buf, sizeof(int), (size_t)1, fp);
	diff = ftell(fp) - off;

	(void) fclose(fp);
	if (freopen(t_file, "r", stdin) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fp = stdin;
	for (i = 0 ; i < 3 ; i++)
		(void) getchar();
	if (ungetc(T5_CHAR, fp) != T5_CHAR)
		fail = 1;
	else
		PATH_TRACE;
	val5 = getchar();

	if (ungetc(T6_CHAR, fp) != T6_CHAR)
		fail = 1;
	(void) gets(buf);
	val6 = buf[0];
	/* don't fclose(fp) */

	if(freopen(t_file, "r", stdin) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: freopen(%s, \"r\", stdin) failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fp = stdin;

	for(i=0; i<9; i++)
		(void) getc(fp);

	for(i=5; i<11; i++)
	{
		DBUG_PRINTF("test1","ungetc(%c)",alphabet[i]);
		if(i==5)
			PATH_TRACE;
#ifdef TESTBED
#undef	ungetc
#define	ungetc(a,b)  (a)
#endif
		/* only one character of pushback is guaranteed,
		   so stop if an ungetc() call fails */
		if(ungetc(alphabet[i], fp) == EOF)
			break;
	}	
	if(i==5)
		fail = 1;
	else
		PATH_TRACE;

	for(i--; i>=5; i--)
	{
		int	tmp;
		tmp = getc(fp);
		DBUG_PRINTF("test1","getc() = %c (expecting %c)", tmp, alphabet[i]);

		if(i==5)
			PATH_TRACE;
		if(tmp != alphabet[i])
			mfail = 1;
	}


	if (fail || val1 != T1_CHAR
		|| val2 != T2_CHAR
		|| val3 != T3_CHAR
		|| diff != sizeof(int) - 1
		|| val5 != T5_CHAR
		|| val6 != T6_CHAR
		|| mfail)
	{
		xx_rpt(FAILURE);
		if (fail)
			in_rpt("ungetc() did not return it's character argument");
		if (val1 != T1_CHAR)
			in_rpt("getc() did not return value passed to previous ungetc()");
		if (val2 != T2_CHAR)
			in_rpt("fgetc() did not return value passed to previous ungetc()");
		if (val3 != T3_CHAR)
			in_rpt("first character in buffer returned by fgets() not value passed to previous ungetc()");
		if (diff != sizeof(int) - 1)
		{
			in_rpt("change in offset in stream after unget(), fread() was not as expected");
			in_rpt("expected sizeof(int) - 1 = %d, found %ld",
				sizeof(int) - 1, diff);
		}
		if (val5 != T5_CHAR)
			in_rpt("getchar() did not return value passed to previous ungetc()");
		if (val6 != T6_CHAR)
			in_rpt("first character in buffer returned by gets() not value passed to previous ungetc()");
		if (mfail)
			in_rpt("getc() calls on stream after multiple ungetc() did not return the characters in the reverse order of pushing");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(12);
	}

	/* don't fclose(fp) */
	DBUG_VOID_RETURN;
}


private void
test2()
{
	FILE		*fp;
	int		err;
	int		i;
	int	pathok = 0;

	DBUG_ENTER("test2");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	for (i = 0 ; i < 3 ; i++)
		(void) getc(fp);

	(void) ungetc(T1_CHAR, fp);
	(void) fseek(fp, 5L, SEEK_CUR);
	(void) fseek(fp, -5L, SEEK_CUR);
	if (getc(fp) == T1_CHAR)
	{
		xx_rpt(WARNING);
		in_rpt("Warning:- this functionality is not implemented on this system");
		in_rpt("fseek() after ungetc() did not loose the character pushed into stream");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}


private	void
t3_catch(sig)
	int	sig;
{
	xx_rpt(FAILURE);
	in_rpt("ungetc() on newly opened stream (no previous reads) caused signal %d(%s)",
		sig, signame(sig));
	exit(SUCCEED);
}


private void
test3()
{

	DBUG_ENTER("test3");

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	FILE	*fp;
	int	rval, err, ncalls, i;
	int	goteof = 0;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	setsigs(t3_catch);

	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* push back as many characters as possible */

	for (i = 1; i <= T3_TRIES; i++)
	{
		if (i == 1)
			PATH_TRACE;

		if (ungetc((i % UCHAR_MAX), fp) == EOF)
		{
			PATH_TRACE;
			goteof = 1;
			DBUG_PRINTF("test3", "EOF on call %d", i);
			break;
		}
	}
	if (goteof)
		ncalls = i;
	else
		ncalls = i-1;

	/* check successfully pushed characters can be read */

	while (--i > 0)
	{
		if ((rval = getc(fp)) != (i % UCHAR_MAX))
		{
			xx_rpt(FAILURE);
			in_rpt("After %d calls to ungetc()%s,", ncalls,
			    goteof ? ", the last of which returned EOF" : "");
			in_rpt("could not read pushed back character from ungetc() call %d:", i);
			in_rpt("getc() returned %d, expected %d",
				rval, i % UCHAR_MAX);

			(void) fclose(fp);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	if (goteof)
	{
		PATH_TRACE;
		PATH_XS_RPT(3 + ncalls);
	}
	else
	{
		xx_rpt(UNTESTED);
		in_rpt("ungetc() did not return EOF after %d calls", ncalls);
	}

	(void) fclose(fp);

	DBUG_VOID_RETURN;
}


private void
test4()
{
	FILE		*fp;
	int		err;
	int		i;
	int		val;
	int	pathok = 0;

	DBUG_ENTER("test4");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	for (i = 0 ; i < 3 ; i++)
		(void) getc(fp);
	val = ungetc(EOF, fp);
	i = getc(fp);
	if (val != EOF || i != 'd')
	{
		xx_rpt(FAILURE);
		if (val != EOF)
			in_rpt("ungetc(EOF, stream) did not return EOF - returned %s(%d)",
				pr_char((char) val), val);
		if (i != 'd')
		{
			in_rpt("getc() after ungetc(EOF, stream) did not return expected value");
			in_rpt("expected 'd'(%d), got %s(%d)",
				'd', pr_char((char) val), val);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test5()
{
	FILE	*fp;
	int	err;
	int	i;
	int	fail = 0;
	int	pathok = 0;

	DBUG_ENTER("test5");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0 ; i <3 ; i++ )
		(void) getc(fp);

	(void) ungetc(T6_CHAR, fp);
	if (fclose(fp))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fclose(%s) after ungetc() failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed (fopen after call to fclose) - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0 ; i < 26 ; i++)
		if (getc(fp) != alphabet[i])
			fail = 1;
		else
			PATH_TRACE;

	if(fail)
	{
		xx_rpt(FAILURE);
		in_rpt("call to ungetc() caused file associated with the stream to be changed");
	}
	else
		PATH_XS_RPT(29);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	FILE	*fp;
	int	i;
	int	eoftest;
	int	ungetctest;
	int 	err;
	int	pathok = 0;
	
	DBUG_ENTER("test6");

	if ((fp = fopen(t_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"r+\") failed - errno %d(%s)", t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0 ; i <= 26 ; i++ )
		(void) getc(fp);
        /*
	 * Having read all 26 characters from the file feof() 
	 * ought to indicate end of file.
	 */
	 
	 if((eoftest=feof(fp)) == 0 ) {
		xx_rpt(DELETION);
		in_rpt("feof() did not indicate end-of-file as expected for file %s.", t_file);
		in_rpt("feof() returned result %d",eoftest);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ((ungetctest=tb_ungetc(T7_CHAR, fp)) != T7_CHAR) {
		xx_rpt(FAILURE);
		if(ungetctest == EOF)
			in_rpt("ungetc() returned EOF instead of %d.", T7_CHAR);
		else 
			in_rpt("ungetc() returned %d instead of %d.",ungetctest,T7_CHAR);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	 if((eoftest = feof(fp)) != 0 ) {
		xx_rpt(FAILURE);
		in_rpt("ungetc() did not clear the end-of-file indicator for file %s.", t_file);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	PATH_XS_RPT(4);
	DBUG_VOID_RETURN;
}		


private void
test7()
{
#if TEST_XPG > 3 || TEST_FIPS
	FILE 	*fp;
	int 	err;
	int	i;
	int	getctest;
	int	ungetctest;
	fpos_t 	savedpos;
	fpos_t	*savedposp;
	int 	pathok = 0;

	DBUG_ENTER("test7");
	if ((fp = fopen(t_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"r+\") failed - errno %d(%s)", t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0 ; i < SAMPLE ; i++)   /* The SAMPLE is arbitary */
 		(void) getc(fp);

	savedposp = & savedpos;
	
	if(fgetpos(fp,savedposp) != 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fgetpos() failed - errno %d(%s)", err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	

	if ((ungetctest=tb_ungetc((int) T8_CHAR, fp)) != (int) T8_CHAR ) {
		xx_rpt(FAILURE);
		if(ungetctest == EOF)
			in_rpt("ungetc() returned EOF instead of %d.", T7_CHAR);
		else
			in_rpt("ungetc() returned %d instead of %d.",ungetctest,T7_CHAR);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	
	if(fsetpos(fp,savedposp) != 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fsetpos() failed - errno %d(%s)", err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	
	if( (getctest = getc(fp)) != (int) alphabet[SAMPLE] ) {
		xx_rpt(FAILURE);
		in_rpt("fsetpos failed to erase the result of an ungetc()");
		in_rpt("getc() returned %d instead of %d (character '%c').",getctest,(int) alphabet[SAMPLE],alphabet[SAMPLE]);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	PATH_XS_RPT(5);
	DBUG_VOID_RETURN
#else
	DBUG_ENTER("test7");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
#endif
}


private int
tb_ungetc(c,strm)
int c;
FILE *strm;
{
	int result;

#ifndef TESTBED
	result = ungetc(c,strm);
#endif

#ifdef TESTBED1
	result = 0;
#endif

#ifdef TESTBED2
	result = EOF;
#endif	
	return(result);
}
