/*
*      SCCS:  @(#)  ANSI.os/streamio/vprintf/vprintf1.c	Rel 4.4.3	    (07/20/98)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/vprintf - T.vprintf Rel 4.4.3 (07/20/98)";
char	*copyright[] =
{
	"(C) Copyright 1991 X/Open Company Limited", "All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		vprintf1.c

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the vprintf() subroutine.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	25 May 1989
MODIFICATIONS:
		Geoff Clare, 7 Dec 1989
			Give UNSUPPORTED result instead of DELETION in
			'e100' test when exponents > 99 cannot be represented
		
		Geoff Clare, 7 Feb 1990
			Corrections to val1str reporting strings.

		David Sawyer
		Fri Jul 13 10:22:16 BST 1990
		Added resillience code.

		J.A. Nave	Mon Oct 22 11:44:52 BST 1990
		Added tests for negative values for the precision
		and field width in the argumant list

		Geoff Clare, 22 Feb 1991
			Correct val2str in test32.

		Stuart Boutell, 26 Feb 1991
			Correct so that parts of tests 31-34 are executed
			only when not in XPG3 mode

		Neil Moses, 7 Nov 1991
			Changes for VSX 4.2
			Added new wide character conversion types C and S
			New tests 20 and 21

		Neil Moses, 16 Jan 1992
			Missing '\n' after expected strings in tests 20,21.
			Account for unsupported multibyte characters.

		Geoff Clare, 20 Jan 1992
			Ensure wctomb() called in correct locale.
			Calculate 1e100 instead of assigning constant, to
			prevent compilation error if not representable.

		Neil Moses, 27 Jan 1992
			Changed expected output for %S format with precision
			Changed expected output for negative precision

		Geoff Clare, 21 Apr 1992
			Change args for o, u, x, X conversions to unsigned.
			Remove tests for %#C and %#S.
			Test %#d, %#i, %#c and %#s in XPG3 mode only.

		Geoff Clare, 8 May 1992
			Fix bad %C and %S format strings and argument
			value reporting in test 36.

		Geoff Clare, 21 May 1992
			Make code conditional on ANSI_VPRINTF_FUNC.

		John-Paul, 15 November 1992
			XNFS enhancements.

		Geoff Clare, 22 July 1994
			Implement interpretation VWG/060/110293.

		Geoff Clare, April 1997
		Added tests for vsnprintf() in UNIX98 mode.

************************************************************************/

#include	<std.h>

#if ANSI_VPRINTF_FUNC
#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<string.h>
#include	<ctype.h>
#include	<local.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"vprintf_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef vfprintf
#undef vprintf
#undef vsprintf
#undef vsnprintf
#endif



extern	void	mlmachar();
extern	char	*itoa();
#if defined(_XOPEN_SOURCE) && TEST_XPG > 3 && !TEST_XNFS
extern	char	*pr_wchar();
extern	char	*pr_wstr();
#endif
extern	char	*errname();
extern	char	*psl_to_asc();

extern	int	dexplen;

extern	char	*tfile;
extern	int	interface;
extern	FILE	*f_rd;
extern	FILE	*f_wr;
extern	char	*function;
extern	char	buf[BUFFERSIZE];

extern int	tretval;
extern int	testfail;
extern int	setupok;
extern char	*testcomment;

#if defined(_XOPEN_SOURCE) && TEST_XPG > 3 && !TEST_XNFS
extern char	*locale;
extern wchar_t	*wcstr;
extern char	mbstr[BUFFERSIZE];
extern char	wexpstr[BUFFERSIZE];
extern char	wvalstr[BUFFERSIZE];
extern void	wclocale();
#endif

extern void	ch_setup();
extern void	checkval();
extern void	checkpat();
extern void	chk_narg();

#if !TEST_XNFS
private void	chk_t22();
#endif /* TEST_XNFS */

private void	ch_t1();
#if !TEST_XNFS
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	ch_t6();
private void	ch_t7();
private void	ch_t8();
private void	ch_t9();
private void	ch_t10();
private void	ch_t11();
private void	ch_t12();
private void	ch_t13();
private void	ch_t14();
private void	ch_t15();
private void	ch_t16();
private void	ch_t17();
private void	ch_t18();
private void	ch_t19();
#if defined(_XOPEN_SOURCE) && TEST_XPG > 3
private void	ch_t20();
private void	t20();
private void	ch_t21();
#endif
private void	ch_t22();
private void	ch_t23();
private void	ch_t24();
private void	ch_t25();
private void	ch_t26();
private void	ch_t27();
private void	t27();
private void	ch_t28();
private void	t28();
private void	ch_t29();
private void	t29();
private void	ch_t30();
private void	ch_t31();
private void	t31();
private void	ch_t32();
private void	t32();
private void	ch_t33();
private void	t33();
private void	ch_t34();
private void	t34();
private void	ch_t35();
private void	t35();
private void	ch_t36();
private void	t36();
private void	ch_t37();
private void	t37();
#endif /* TEST_XNFS */

public void
notinuse()
{
	DBUG_ENTER("notinuse");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

public void
test1()
{
	char	*convstr = "12345678901234567890\\n";
	char	*exp1str = "12345678901234567890\n";
	int	pathok = setupok;

	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "the number of characters written is returned";

	globok = 0;
	checkval(convstr, "", exp1str);
	PATH_FUNC_TRACE;

	GETBUF
	else
		PATH_TRACE;
	if ( atoi(buf) != 21 )	
	{
		if ( !testfail++ )
			xx_rpt(FAILURE);
		in_rpt("%s \"%s\") did not return 21", function, convstr);
	}
	else
		PATH_TRACE;

	if ( !testfail )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	char	*string = "12345678901234567890\n";
	char	*cptr;

	DBUG_ENTER("ch_t1");

	ch_setup();
	
	PRINTIT(string);

	/* send return value to parent (without using vprintf !) */
	for ( cptr = itoa(tretval) ; *cptr != NUL ; cptr++ )
		if ( fputc(*cptr, f_wr) == EOF )
			DBUG_EXIT(E_PUTC);

	if ( fputc('\n', f_wr) == EOF )
		DBUG_EXIT(E_PUTC);
	
#if TEST_XNFS
	(void) fflush(f_wr);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

public	void
test2()
{
	char	*conv1str = "%3d%d\\n";
	char	*val1str = "12345, 12345";
	char	*exp1str = "1234512345\n";
	int	pathok = setupok;

	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "field width smaller than field does not cause truncation";

	globok = 0;
	checkval(conv1str, val1str, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	i = 12345;

	DBUG_ENTER("ch_t2");

	ch_setup();

	PRINTIT("%3d%d\n" _ i _ i);

	DBUG_VOID_RETURN;
}

public void
test3()
{
	char	*conv1str = "%d%d\\n";
	char	*val1str  = "12345, -12345";
	char	*exp1str = "12345-12345\n";
	int	pathok = setupok;

	DBUG_ENTER("test3");

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);
	
	testfail = 0;
	testcomment = "(d) format converts integer to decimal character string";

	globok = 0;
	checkval(conv1str, val1str, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int	i = 12345;

	DBUG_ENTER("ch_t3");

	ch_setup();

	PRINTIT("%d%d\n" _ i _ -i);

	DBUG_VOID_RETURN;
}

public void
test4()
{
	char	*conv1str = "%i%i\\n";
	char	*val1str  = "12345, -12345";
	char	*exp1str = "12345-12345\n";
	int	pathok = setupok;

	DBUG_ENTER("test4");

	(void) cppair(ch_t4, NULLFN, WAITTIME, 0);
	
	testfail = 0;
	testcomment = "(i) format converts integer to decimal character string";

	globok = 0;
	checkval(conv1str, val1str, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	i = 12345;

	DBUG_ENTER("ch_t4");

	ch_setup();

	PRINTIT("%i%i\n" _ i _ -i);

	DBUG_VOID_RETURN;
}


public void
test5()
{
	char	*conv1str = "%o\\n";
	char	*val1str  = "12345";
	char	*exp1str = "30071\n";
	int	pathok = setupok;

	DBUG_ENTER("test5");

	(void) cppair(ch_t5, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(o) format converts integer to octal character string";

	globok = 0;
	checkval(conv1str, val1str, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
#if TEST_XPG == 3
	int	i = 12345;
#else
	unsigned int	i = 12345;
#endif

	DBUG_ENTER("ch_t5");

	ch_setup();

	PRINTIT("%o\n" _ i);

	DBUG_VOID_RETURN;
}


public void
test6()
{
	char	*conv1str = "%u\\n";
	char	*val1str  = "12345";
	char	*exp1str = "12345\n";
	int	pathok = setupok;

	DBUG_ENTER("test6");

	(void) cppair(ch_t6, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(u) format converts integer to decimal character string";

	globok = 0;
	checkval(conv1str, val1str, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
#if TEST_XPG == 3
	int	i = 12345;
#else
	unsigned int	i = 12345;
#endif

	DBUG_ENTER("ch_t6");

	ch_setup();

	PRINTIT("%u\n" _ i);

	DBUG_VOID_RETURN;
}


public void
test7()
{
	char	*conv1str = "%x\\n";
	char	*val1str  = "11259";
	char	*exp1str = "2bfb\n";
	int	pathok = setupok;

	DBUG_ENTER("test7");

	(void) cppair(ch_t7, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(x) format converts integer to hex character string";

	globok = 0;
	checkval(conv1str, val1str, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);


	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
#if TEST_XPG == 3
	int	i = 11259;
#else
	unsigned int	i = 11259;
#endif

	DBUG_ENTER("ch_t7");

	ch_setup();

	PRINTIT("%x\n" _ i);

	DBUG_VOID_RETURN;
}


public void
test8()
{
	char	*conv1str = "%X\\n";
	char	*val1str  = "11259";
	char	*exp1str = "2BFB\n";
	int	pathok = setupok;

	DBUG_ENTER("test8");

	(void) cppair(ch_t8, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(X) format converts integer to HEX character string";

	globok = 0;
	checkval(conv1str, val1str, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
#if TEST_XPG == 3
	int	i = 11259;
#else
	unsigned int	i = 11259;
#endif

	DBUG_ENTER("ch_t8");

	ch_setup();

	PRINTIT("%X\n" _ i);

	DBUG_VOID_RETURN;
}


public void
test9()
{
	char	*conv1str = "%f %f\\n";
	char	*val1str  = "-12.3, 321.765432";
	char	*exp1str = "-12.300000 321.765432\n";
	int	pathok = setupok;

	DBUG_ENTER("test9");

	(void) cppair(ch_t9, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(f) format converts float or double to [-]ddd.ddd format";

	globok = 0;
	checkval(conv1str, val1str, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	double 	f = 12.3;
	double	d = 321.7654321;

	DBUG_ENTER("ch_t9");

	ch_setup();

	PRINTIT("%f %f\n" _ -f _ d);

	DBUG_VOID_RETURN;
}


public void
test10()
{
	char	*conv1str = "%e %e\\n";
	char	*val1str  = "-0.0123, 321.7654321";
	char	*exp1pat = "^-1\\.230000e-0*02 3\\.217654e+0*02\n";
	int	pathok = setupok;

	DBUG_ENTER("test10");

	(void) cppair(ch_t10, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(e) format converts double or float to [-]d.ddde+dd format";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	double	f = -0.0123;
	double	d = 321.7654321;

	DBUG_ENTER("ch_t10");

	ch_setup();

	PRINTIT("%e %e\n" _ f _ d);

	DBUG_VOID_RETURN;
}


public void
test11()
{
	char	*conv1str = "%e\\n";
	char	*val1str  = "1e100";
	char	*exp1pat = "^1\\.000000e+0*100\n";
	int	pathok = setupok;

	DBUG_ENTER("test11");
	mlmachar();

	if ( dexplen < 10 )
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("cannot represent 1e+100 on this machine");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) cppair(ch_t11, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(e) format allows for additional exponent digits";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	/* do not initialise using d = 1e100
	   (will not compile if not representable) */
	double	d = 1e25;

	DBUG_ENTER("ch_t11");
			
	ch_setup();

	d *= d;
	d *= d;

	PRINTIT("%e\n" _ d);

	DBUG_VOID_RETURN;
}


public void
test12()
{
	char	*conv1str = "%E %E\\n";
	char	*val1str  = "-0.0123, 321.7654321";
	char	*exp1pat = "^-1\\.230000E-0*02 3\\.217654E+0*02\n";
	int	pathok = setupok;

	DBUG_ENTER("test12");

	(void) cppair(ch_t12, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(E) format converts double or float to [-]d.dddE+dd format";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	double	f = -0.0123;
	double	d = 321.7654321;

	DBUG_ENTER("ch_t12");
			
	ch_setup();

	PRINTIT("%E %E\n" _ f _ d);

	DBUG_VOID_RETURN;
}


public void
test13()
{
	char	*conv1str = "%g %g\\n";
	char	*val1str  = "0.0001555, 12345.1234546";
	char	*exp1str = "0.0001555 12345.1\n";
	int	pathok = setupok;

	DBUG_ENTER("test13");

	(void) cppair(ch_t13, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(g) format converts double or float to (f) format when exponent -4 to precision";

	globok = 0;
	checkval(conv1str, val1str, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t13()
{
	double	d1 = 0.0001555;
	double	d2 = 12345.1234546;

	DBUG_ENTER("ch_t13");
			
	ch_setup();

	PRINTIT("%g %g\n" _ d1 _ d2);

	DBUG_VOID_RETURN;
}


public void
test14()
{
	char	*conv1str = "%g %g\\n";
	char	*val1str  = "0.00001555, 12345678.1234546";
	char	*exp1pat = "^1\\.555e-0*05 1\\.2345[67]e+0*07\n";
	int	pathok = setupok;

	DBUG_ENTER("test14");

	(void) cppair(ch_t14, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(g) format converts double or float to (e) format when exponent < -4 or > precision";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t14()
{
	double	d1 = 0.00001555;
	double	d2 = 12345678.1234546;

	DBUG_ENTER("ch_t14");
			
	ch_setup();

	PRINTIT("%g %g\n" _ d1 _ d2);

	DBUG_VOID_RETURN;
}


public void
test15()
{
	char	*conv1str = "%G %G\\n";
	char	*val1str  = "0.00001555, 12345678.1234546";
	char	*exp1pat = "^1\\.555E-0*05 1\\.2345[67]E+0*07\n";
	int	pathok = setupok;

	DBUG_ENTER("test15");

	(void) cppair(ch_t15, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(G) format converts double or float to (E) format when exponent < -4 or > precision";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}


private void
ch_t15()
{
	double	d1 = 0.00001555;
	double	d2 = 12345678.1234546;

	DBUG_ENTER("ch_t15");
			
	ch_setup();

	PRINTIT("%G %G\n" _ d1 _ d2);

	DBUG_VOID_RETURN;
}


public void
test16()
{
	char	*conv1str = "%g %G\\n";
	char	*val1str  = "9.0500001, 9.0500001";
	char	*exp1str = "9.05 9.05\n";
	int	pathok = setupok;

	DBUG_ENTER("test16");

	(void) cppair(ch_t16, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(g) and (G) formats remove trailing zeros";

	globok = 0;
	checkval(conv1str, val1str, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t16()
{
	double	d = 9.0500001;

	DBUG_ENTER("ch_t16");
			
	ch_setup();

	PRINTIT("%g %G\n" _ d _ d);

	DBUG_VOID_RETURN;
}


public void
test17()
{
	char	*conv1str = "%g %G\\n";
	char	*val1str  = "5.0000001, 5.0000001";
	char	*exp1str = "5 5\n";
	int	pathok = setupok;

	DBUG_ENTER("test17");

	(void) cppair(ch_t17, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(g) and (G) formats remove trailing radix characters";

	globok = 0;
	checkval(conv1str, val1str, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t17()
{
	double	g = 5.0000001;

	DBUG_ENTER("ch_t17");
			
	ch_setup();

	PRINTIT("%g %G\n" _ g _ g);

	DBUG_VOID_RETURN;
}


public void
test18()
{
	char	*conv1str = "%c\\n";
	char	*val1str  = "'a'";
	char	*exp1str = "a\n";
	int	pathok = setupok;

	DBUG_ENTER("test18");

	(void) cppair(ch_t18, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(c) format prints the argument character";

	globok = 0;
	checkval(conv1str, val1str, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t18()
{
	char	c = 'a';

	DBUG_ENTER("ch_t18");
			
	ch_setup();

	PRINTIT("%c\n" _ c);

	DBUG_VOID_RETURN;
}


public void
test19()
{
	char	*conv1str = "%s";
	char	*valstr  = "hello world\\n";
	char	*expstr = "hello world\n";
	int	pathok = setupok;

	DBUG_ENTER("test19");

	(void) cppair(ch_t19, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(s) format prints its string argument";

	globok = 0;
	checkval(conv1str, valstr, expstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t19()
{
	char	*string = "hello world\n";

	DBUG_ENTER("ch_t19");
			
	ch_setup();

	PRINTIT("%s" _ string);

	DBUG_VOID_RETURN;
}


public void
test20()
{
	DBUG_ENTER("test20");

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	(void) cppair(t20, NULLFN, WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
#endif
	DBUG_VOID_RETURN;
}

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3

private void
t20()
{
	char	*convstr = "%C";
	char	tbuf[50];
	int	i, j, err;
	int	pathok = setupok;

	DBUG_ENTER("t20");

	wclocale();

	(void) cppair(ch_t20, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(C) format prints the argument character as a multibyte sequence";

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb(str, 0x%lx) failed : errno %d (%s)", 	
				(long) wcstr[i], err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';
		(void) strcpy(wexpstr, tbuf);
		(void) strcat(wexpstr, "\n");
		globok = 0;
		checkval(convstr, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	globok = 0;
	checkval(convstr, "a", "a\n");
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1 + i);

	DBUG_VOID_RETURN;
}

private void
ch_t20()
{
	wchar_t	wc = 'a';
	int	i;

	DBUG_ENTER("ch_t20");

	ch_setup();

	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
		PRINTIT("%C\n" _ *(wcstr + i));

	PRINTIT("%C\n" _ wc);

	DBUG_VOID_RETURN;
}

#endif

public void
test21()
{
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	char	*convstr = "%S %S";
	int	pathok = setupok;
#endif

	DBUG_ENTER("test21");

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3

	(void) strcpy(wexpstr, mbstr);
	(void) strcat(wexpstr, " abc\n");
	(void) strcpy(wvalstr, pr_wstr(wcstr));
	(void) strcat(wvalstr, " abc\n");

	(void) cppair(ch_t21, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(S) format prints the argument string as a multibyte sequence";

	globok = 0;
	checkval(convstr, wvalstr, wexpstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3

private void
ch_t21()
{
	static	wchar_t	nwcstr[] = {'a','b','c',(wchar_t)0};

	DBUG_ENTER("ch_t21");
			
	ch_setup();

	wclocale();

	PRINTIT("%S %S\n" _ wcstr _ nwcstr);

	DBUG_VOID_RETURN;
}

#endif

public void
test22()
{
	char	*conv1str = "%p\\n";
	char	*valstr = "(void *)&i";
	int	pathok = setupok;

	DBUG_ENTER("test22");

	(void) cppair(ch_t22, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(p) format produces a printable sequence of characters";

	globok = 0;
	chk_t22(conv1str, valstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
chk_t22(convstr, valstr)
char *convstr, *valstr;
{
	char	*s;
	int	pathok = 0;

	DBUG_ENTER("chk_t22");

	GETBUF
	else
		PATH_TRACE;
	if (strncmp(buf, "%..xx", (size_t)5) == 0)
	{
		testfail++;
		in_rpt("%s", buf);
		while (fgets(buf, sizeof(buf), f_rd) != (char *) NULL)
			in_rpt("%s", buf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check output is printable non-null string */
	
	if (buf[0] != '\0' && buf[0] != '\n')
	{
		for (s = buf; *s != '\n' && *s != '\0'; s++)
			if (!isprint(*s))
				break;

		if (*s == '\n')		/* success */
		{
			PATH_TRACE;
			PATH_FUNC_RPT(3);
			DBUG_VOID_RETURN;
		}
	}

	/* more code copied from checkval() */
	if (!testfail++)
		xx_rpt(FAILURE);

	in_rpt("%s \"%s\", %s) failed", function, convstr, valstr);
	if (testcomment != (char *) NULL)
		in_rpt("%s", testcomment);
	in_rpt("Observed output: \"%s\"", psl_to_asc(buf, 0));

	DBUG_VOID_RETURN;
}

private void
ch_t22()
{
	int	i;

	DBUG_ENTER("ch_t22");
			
	ch_setup();

	PRINTIT("%p\n" _ (void *)&i);

	DBUG_VOID_RETURN;
}

public void
test23()
{
	char	*conv1str = "%s%n %s\\n";
	char	*valstr  = "hello, &i, world";
	char	*expstr = "hello world\n";
	long	expnum = sizeof("hello")-1;
	int	pathok = setupok;

	DBUG_ENTER("test23");

	(void) cppair(ch_t23, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(n) format returns, via the argument, a count of the characters written so far";

	globok = 0;
	checkval(conv1str, valstr, expstr);
	PATH_FUNC_TRACE;
	globok = 0;
	chk_narg(conv1str, valstr, expnum);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t23()
{
	char *s1 = "hello", *s2 = "world";
	int n1 = 0;

	DBUG_ENTER("ch_t23");
			
	ch_setup();

	PRINTIT("%s%n %s\n" _ s1 _ &n1 _ s2);
	PRINTIT("%d\n" _ n1);


	DBUG_VOID_RETURN;
}

public void
test24()
{
	char	*convstr = "%%\\n";
	char	*valstr  = "";
	char	*exp1str = "%\n";
	int	pathok = setupok;

	DBUG_ENTER("test24");

	(void) cppair(ch_t24, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "%% in format converts to % in output string";

	globok = 0;
	checkval(convstr, valstr, exp1str);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t24()
{
	DBUG_ENTER("ch_t24");
			
	ch_setup();

	PRINTIT("%%\n");

	DBUG_VOID_RETURN;
}


public void
test25()
{
	char	*conv1str = "%ld %li %lo%ln %lu %lx %lX\\n";
	char	*val1str  = "1234567890, 99999, 0127, &l, 7654321, 0xfedca9, 0xfedca9";
	char	*expstr  = "1234567890 99999 127 7654321 fedca9 FEDCA9\n";
	long	expnum = sizeof("1234567890 99999 127")-1;
	int	pathok = setupok;

	DBUG_ENTER("test25");

	(void) cppair(ch_t25, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(l) format modifier specifies argument is long integer";

	globok = 0;
	checkval(conv1str, val1str, expstr);
	PATH_FUNC_TRACE;
	globok = 0;
	chk_narg(conv1str, val1str, expnum);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t25()
{
	long	d = 1234567890;
	long	i = 99999;
#if TEST_XPG == 3
	long	o = 0127;
	long	u = 7654321;
	long	x = 0xfedca9;
#else
	unsigned long	o = 0127;
	unsigned long	u = 7654321;
	unsigned long	x = 0xfedca9;
#endif
	long	n1 = 0;

	DBUG_ENTER("ch_t25");
			
	ch_setup();

	PRINTIT("%ld %li %lo%ln %lu %lx %lX\n" _ d _ i _ o _ &n1 _ u _ x _ x);
	PRINTIT("%ld\n" _ n1);

	DBUG_VOID_RETURN;
}


public void
test26()
{
	char	*conv1str = "%hd %hi %ho%hn %hu %hx %hX\\n";
	char	*val1str  = "12345, 9999, 0127, &short, 7654, 0xca9, 0xca9";
	char	*expstr  = "12345 9999 127 7654 ca9 CA9\n";
	long	expnum = sizeof("12345 9999 127")-1;
	int	pathok = setupok;

	DBUG_ENTER("test26");

	(void) cppair(ch_t26, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(h) format modifier has no effect except to specify short int pointer with %n";

	globok = 0;
	checkval(conv1str, val1str, expstr);
	PATH_FUNC_TRACE;
	globok = 0;
	chk_narg(conv1str, val1str, expnum);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t26()
{
	int	d = 12345;
	int	i = 9999;
#if TEST_XPG == 3
	int	o = 0127;
	int	u = 7654;
	int	x = 0xca9;
#else
	unsigned int	o = 0127;
	unsigned int	u = 7654;
	unsigned int	x = 0xca9;
#endif
	short	n1 = 0;

	DBUG_ENTER("ch_t26");
			
	ch_setup();

	PRINTIT("%hd %hi %ho%hn %hu %hx %hX\n" _ d _ i _ o _ &n1 _ u _ x _ x);
	PRINTIT("%d\n" _ (int)n1);

	DBUG_VOID_RETURN;
}

public void
test27()
{
	DBUG_ENTER("test27");

	(void) cppair(t27, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t27()
{
	char	*conv1str = "%.d %.hi %.lo %.f %.e %.g %.g %.g %.g %.g %.s %.d %.hi %.lo|\\n";
	char	*val1str  = "12345, 1234, 1234567890, 321.7654321, 321.7654321, -0.01, 0.01, 1e-5, 1.4, -1.4, test-string, 0, 0, 0L";
	char	*exp1pat  = "^12345 1234 11145401322 32[12] 3e+0*02 -0\\.01 0\\.01 1e-0*05 1 -1    |\n";
	int	pathok = setupok;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	char	*conv3str = "%.S  ";
#endif

	DBUG_ENTER("t27");

	(void) cppair(ch_t27, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "a null digit string after the precision is treated as zero";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	(void) strcpy(wexpstr, "  ");
	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t27()
{
	double 	g1 = -0.01;
	double	g2 = 0.01;
	double	g3 = 1e-5;
	double	g4 = 1.4;
	double	g5 = -1.4;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t27");
			
	ch_setup();

	PRINTIT("%.d %.hi %.lo %.f %.e %.g %.g %.g %.g %.g %.s %.d %.hi %.lo|\n" _ i _ h _ l _ d _ d _ g1 _ g2 _ g3 _ g4 _ g5 _ s _ 0 _ 0 _ 0L);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();
	PRINTIT("%.S  " _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test28()
{
	DBUG_ENTER("test28");

	(void) cppair(t28, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t28()
{
	char	*conv1str = "%.10d %.5hi %.15lo %.3f %.4e %.4g %.4g %.4g %.4g %.4g %.7s\\n";
	char	*val1str  = "12345, 1234, 1234567890, 321.7654321, 321.7654321, -0.01, 0.01, 1e-5, 1.54321, -1.54321, test-string";
	char	*exp1pat  = "^0000012345 01234 000011145401322 321\\.765 3\\.217[67]e+0*02 -0\\.01 0\\.01 1e-0*05 1\\.543 -1\\.543 test-st\n";
	int	pathok = setupok;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	err, ret;
	char	*conv3str = "%.7S";
#endif

	DBUG_ENTER("t28");

	(void) cppair(ch_t28, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(.m) format modifier defines the precision";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	if((ret = wcstombs(wexpstr, wcstr, 7)) == (size_t)-1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wcstombs() failed : errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	wexpstr[ret] = '\0';

	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_t28()
{
	double 	g1 = -0.01;
	double	g2 = 0.01;
	double	g3 = 1e-5;
	double	g4 = 1.54321;
	double	g5 = -1.54321;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t28");
			
	ch_setup();

	PRINTIT("%.10d %.5hi %.15lo %.3f %.4e %.4g %.4g %.4g %.4g %.4g %.7s\n" _ i _ h _ l _ d _ d _ g1 _ g2 _ g3 _ g4 _ g5 _ s);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();
	PRINTIT("%.7S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test29()
{
	DBUG_ENTER("test29");

	(void) cppair(t29, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t29()
{
	char	*conv1str = "%8d %5hi %14lo %5x %14f %14e %3g %4c %14s\\n";
	char	*val1str  = "12345, 1234, 1234567890, 0xba9, 321.7654321, 321.7654321, 5.0000001, 'a', test-string";
	char 	*exp1pat = "^   12345  1234    11145401322   ba9     321\\.765432  *3\\.217654e+0*02   5    a    test-string\n";
	int 	exp1len = sizeof("   12345  1234    11145401322   ba9     321.765432   3.217654e+02   5    a    test-string\n") - 1;
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	i, j, k, len, err;
	char	tbuf[50];
	char	*conv2str = "%20C";
	char	conv3str[20];
#endif


	DBUG_ENTER("t29");

	(void) cppair(ch_t29, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(n) format modifier defines the field width";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';
		for(k = 0; j < 20; j++, k++)
			wexpstr[k] = ' ';
		wexpstr[k] = '\0';
		(void) strcat(wexpstr, tbuf);
		(void) strcat(wexpstr, "\n");
		globok = 0;
		checkval(conv2str, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	len = strlen(mbstr) + 5;
	(void) strcpy(wexpstr, "     ");
	(void) strcat(wexpstr, mbstr);
	(void) strcpy(conv3str, "%");
	(void) strcat(conv3str, itoa(len));
	(void) strcat(conv3str, "S");
	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2 + i);
#else
	if (!testfail)
		PATH_XS_RPT(1);
#endif
	DBUG_VOID_RETURN;
}

private void
ch_t29()
{
	double	g = 5.0000001;
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	x = 0xba9;
#else
	unsigned long	l = 1234567890;
	unsigned int	x = 0xba9;
#endif
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	char	format[20];
	int	j, len;
#endif

	DBUG_ENTER("ch_t29");
			
	ch_setup();

	PRINTIT("%8d %5hi %14lo %5x %14f %14e %3g %4c %14s\n" _ i _ h _ l _ x _ d _ d _ g _ c _ s);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%20C\n" _ *(wcstr + j));

	len = strlen(mbstr) + 5;
	(void) strcpy(format, "%");
	(void) strcat(format, itoa(len));
	(void) strcat(format, "S");
	PRINTIT(format _ wcstr);
#endif
	DBUG_VOID_RETURN;
}


public void
test30()
{
	char	*conv1str = "%08d %06hi %014lo %05x %014f %016e %03g\\n";
	char	*val1str  = "-12345, -1234, 1234567890, 0xba9, -321.7654321, -321.7654321, -5.0000001";
	char 	*exp1pat = "^-0012345 -01234 00011145401322 00ba9 -000321\\.765432 -0*3\\.217654e+0*02 -05\n";
	int 	exp1len = sizeof("-0012345 -01234 00011145401322 00ba9 -000321.765432 -0003.217654e+02 -05\n") - 1;
	int	pathok = setupok;


	DBUG_ENTER("test30");

	(void) cppair(ch_t30, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(0n) format modifier pads with zeroes to the field width";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t30()
{
	double	g = -5.0000001;
	double	d = -321.7654321;
	int	i = -12345;
	int	h = -1234;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	x = 0xba9;
#else
	unsigned long	l = 1234567890;
	unsigned int	x = 0xba9;
#endif

	DBUG_ENTER("ch_t30");
			
	ch_setup();

	PRINTIT("%08d %06hi %014lo %05x %014f %016e %03g\n" _ i _ h _ l _ x _ d _ d _ g);

	DBUG_VOID_RETURN;
}


public void
test31()
{
	DBUG_ENTER("test31");

	(void) cppair(t31, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t31()
{
	char	*conv1str = "%8.d %5.hi %14.lo %14.f %14.e %3.g %14.s\\n";
	char	*val1str  = "12345, 1234, 1234567890, 321.7654321, 321.7654321, 5.0000001, test-string";
	char 	*exp1pat  = "^   12345  1234    11145401322            32[12]  *3e+0*02   5               \n";
	int 	exp1len  = sizeof("   12345  1234    11145401322            321          3e+02   5               \n") - 1;
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	char	*conv3str = "%14.S";
	char	*exp3pat = "              ";
#endif


	DBUG_ENTER("t31");

	(void) cppair(ch_t31, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(n.) format modifier sets field width to (n) with default precision";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), exp3pat);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);
	
	DBUG_VOID_RETURN;
}

private void
ch_t31()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t31");
			
	ch_setup();

	PRINTIT("%8.d %5.hi %14.lo %14.f %14.e %3.g %14.s\n" _ i _ h _ l _ d _ d _ g _ s);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	PRINTIT("%14.S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test32()
{
	DBUG_ENTER("test32");

	(void) cppair(t32, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t32()
{
	char	*conv1str = "%8.5d %5.2hi %14.5lo %14.3f %16.8e %3.4g %14.15s\\n";
	char	*val1str  = "12345, 1234, 1234567890, 321.7654321, 321.7654321, 5.0000001, test-string";
	char 	*exp1pat = "^   12345  1234    11145401322        321\\.765  *3\\.21765432e+0*02   5    test-string\n";
	int 	exp1len = sizeof("   12345  1234    11145401322        321.765   3.21765432e+02   5    test-string\n") - 1;
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	j, len, ret, err;
	char	tbuf[50];
	char	*conv3str = "%20.15S";
#endif


	DBUG_ENTER("t32");

	(void) cppair(ch_t32, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(n.m) format modifier sets field length and precision";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	if((ret = wcstombs(tbuf, wcstr, 15)) == (size_t)-1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wcstombs() failed : errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tbuf[ret] = '\0';
	len = strlen(tbuf);

	for(j = 0; len < 20; j++, len++)
		wexpstr[j] = ' ';
	wexpstr[j] = '\0';
	(void) strcat(wexpstr, tbuf);

	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_t32()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t32");
			
	ch_setup();

	PRINTIT("%8.5d %5.2hi %14.5lo %14.3f %16.8e %3.4g %14.15s\n" _ i _ h _ l _ d _ d _ g _ s);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	PRINTIT("%20.15S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test33()
{
	DBUG_ENTER("test33");

	(void) cppair(t33, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t33()
{
	char	*conv1str = "%*d %*hi %*lo %*f %*e %*g %*c %*s\\n";
	char	*val1str  = "8, 12345, 5, 1234, 14, 1234567890, 14, 321.7654321, 14, 321.7654321, 3, 5.0000001, 4, 'a', 14, test-string";
	char 	*exp1pat = "^   12345  1234    11145401322     321\\.765432  *3\\.217654e+0*02   5    a    test-string\n";
	int 	exp1len = sizeof("   12345  1234    11145401322     321.765432   3.217654e+02   5    a    test-string\n") - 1;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	char	*val2str  = "-8, 12345, -5, 1234, -14, 1234567890, -14, 321.7654321, -14, 321.7654321, -3 5.0000001, -4, 'a', -14, test-string";
	char 	*exp2pat = "^12345    1234  11145401322    321\\.765432     3\\.217654e+0*02 * 5   a    test-string   \n";
	int 	exp2len = sizeof("12345    1234  11145401322    321.765432     3.217654e+02   5   a    test-string   \n") - 1;
#endif

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	i, j, k, err;
	char	tbuf[50];
	char	*conv3str = "%*C";
	char	*conv4str = "%*S";
#endif
	int	pathok = setupok;

 
	DBUG_ENTER("t33");

	(void) cppair(ch_t33, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(*) format modifier takes the field width from the parameter list";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if !defined(_XOPEN_SOURCE)

	globok = 0;
	checkpat(conv1str, val2str, exp2pat, exp2len);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);
	DBUG_VOID_RETURN;
#endif

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3

	globok = 0;
	checkpat(conv1str, val2str, exp2pat, exp2len);
	PATH_FUNC_TRACE;

	wclocale();

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}

		tbuf[j] = '\0';
		for(k = 0; j < 20; j++, k++)
			wexpstr[k] = ' ';
		wexpstr[k] = '\0';
		(void) strcat(wexpstr, tbuf);
		(void) strcat(wexpstr, "\n");
		globok = 0;
		checkval(conv3str, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	(void) strcpy(wexpstr, "     ");
	(void) strcat(wexpstr, mbstr);
	globok = 0;
	checkval(conv4str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3 + i);
#else
	if (!testfail)
		PATH_XS_RPT(1);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t33()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	j, len;
#endif

	DBUG_ENTER("ch_t33");
			
	ch_setup();

	PRINTIT("%*d %*hi %*lo %*f %*e %*g %*c %*s\n" _ 8 _ i _ 5 _ h _ 14 _ l _ 14 _ d _ 14 _ d _ 3 _ g _ 4 _ c _ 14 _ s);

#if !defined(_XOPEN_SOURCE)

	PRINTIT("%*d %*hi %*lo %*f %*e %*g %*c %*s\n" _ -8 _ i _ -5 _ h _ -14 _ l _ -14 _ d _ -14 _ d _ -3 _ g _ -4 _ c _ -14 _ s);

#endif

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3

	PRINTIT("%*d %*hi %*lo %*f %*e %*g %*c %*s\n" _ -8 _ i _ -5 _ h _ -14 _ l _ -14 _ d _ -14 _ d _ -3 _ g _ -4 _ c _ -14 _ s);

	wclocale();
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%*C\n" _ 20 _ *(wcstr + j));

	len = strlen(mbstr) + 5;
	PRINTIT("%*S" _ len _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test34()
{
	DBUG_ENTER("test34");

	(void) cppair(t34, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t34()
{
	char	*conv1str = "%.*d %.*hi %.*lo %.*f %.*e %.*g %.*g %.*g %.*g %.*g %.*s\\n";
	char	*val1str  = "10, 12345, 5, 1234, 15, 1234567890, 3, 321.7654321, 4, 321.7654321, 2, -0.01, 2, 0.01, 4, 1e-5, 4, 1.54321, 4, -1.54321, 7, test-string";
	char 	*exp1pat  = "^0000012345 01234 000011145401322 321\\.765 3\\.217[67]e+0*02 -0\\.01 0\\.01 1e-0*05 1\\.543 -1\\.543 test-st\n";

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	char	*val2str  = "-10, 12345, -5, 1234, -15, 1234567890, -3, 321.7654321, -4, 321.7654321, -2, -0.01, -2, 0.01, -4, 1e-5, -4, 1.54321, -4, -1.54321, -7, test-string";
	char	*exp2pat = "12345 1234 11145401322 321.765432 3.217654e+02 -0.01 0.01 1e-05 1.54321 -1.54321 test-string\n";
#endif

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	err, ret;
	char	*conv4str = "%.*S";
#endif
	int	pathok = setupok;


	DBUG_ENTER("t34");

	(void) cppair(ch_t34, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(.*) format modifier species the precision from the parameter list";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

	globok = 0;
	checkpat(conv1str, val2str, exp2pat, 0);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3

	wclocale();

	if((ret = wcstombs(wexpstr, wcstr, 7)) == (size_t)-1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wcstombs() failed : errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	wexpstr[ret] = '\0';

	globok = 0;
	checkval(conv4str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t34()
{
	double	g1 = -0.01;
	double	g2 = 0.01;
	double	g3 = 1e-5;
	double	g4 = 1.54321;
	double	g5 = -1.54321;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t34");
			
	ch_setup();

	PRINTIT("%.*d %.*hi %.*lo %.*f %.*e %.*g %.*g %.*g %.*g %.*g %.*s\n" _ 10 _ i _ 5 _ h _ 15 _ l _ 3 _ d _ 4 _ d _ 2 _ g1 _ 2 _ g2 _ 4 _ g3 _ 4 _ g4 _ 4 _ g5 _ 7 _ s);

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

	PRINTIT("%.*d %.*hi %.*lo %.*f %.*e %.*g %.*g %.*g %.*g %.*g %.*s\n" _ -10 _ i _ -5 _ h _ -15 _ l _ -3 _ d _ -4 _ d _ -2 _ g1 _ -2 _ g2 _ -4 _ g3 _ -4 _ g4 _ -4 _ g5 _ -7 _ s);

#endif

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3

	wclocale();

	PRINTIT("%.*S" _ 7 _ wcstr);

#endif

	DBUG_VOID_RETURN;
}


public void
test35()
{
	DBUG_ENTER("test35");

	(void) cppair(t35, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t35()
{
	char	*conv1str = "%*.d %*.hi %*.lo %*.f %*.e %*.g %*.s\\n";
	char	*val1str  = "8, 12345, 5, 1234, 14, 1234567890, 14, 321.7654321, 14, 321.7654321, 3, 5.0000001, 14, test-string";
	char 	*exp1pat  = "^   12345  1234    11145401322            32[12]  *3e+0*02   5               \n";
	int 	exp1len  = sizeof("   12345  1234    11145401322            321          3e+02   5               \n") - 1;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	char	*val2str  = "-8, 12345, -5, 1234, -14, 1234567890, -14, 321.7654321, -14, 321.7654321, -3, 5.0000001, -14, test-string";
	char 	*exp2pat  = "^12345    1234  11145401322    322            3e+0*02 * 5                 \n";
	int 	exp2len  = sizeof("12345    1234  11145401322    322            3e+02          5                 \n") - 1;
#endif

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	char	*conv3str = "%*.S";
#endif
	int	pathok = setupok;
	
	DBUG_ENTER("t35");

	(void) cppair(ch_t35, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(*.) format modifier takes field width from the parameter list using 0 precision";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

	globok = 0;
	checkpat(conv1str, val2str, exp2pat, exp2len);
	PATH_FUNC_TRACE;

#else
	PATH_TRACE;
#endif

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3

	wclocale();

	(void) strcpy(wexpstr, "       ");
	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_t35()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t35");
			
	ch_setup();

	PRINTIT("%*.d %*.hi %*.lo %*.f %*.e %*.g %*.s\n" _ 8 _ i _ 5 _ h _ 14 _ l _ 14 _ d _ 14 _ d _ 3 _ g _ 14 _ s);

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

	PRINTIT("%*.d %*.hi %*.lo %*.f %*.e %*.g %*.s\n" _ -8 _ i _ -5 _ h _ -14 _ l _ -14 _ d _ -14 _ d _ -3 _ g _ -14 _ s);

#endif

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3

	wclocale();

	PRINTIT("%*.S" _ 7 _ wcstr);

#endif

	DBUG_VOID_RETURN;
}


public void
test36()
{
	DBUG_ENTER("test36");

	(void) cppair(t36, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t36()
{
	char	*conv1str = "%*.*d %*.*hi %*.*lo %*.*f %*.*e %*.*g %*.*s\\n";
	char	*val1str  = "8, 5, 12345, 5, 2, 1234, 14, 5, 1234567890, 14, 3, 321.7654321, 16, 8, 321.7654321, 3, 4, 5.0000001, 14, 15, test-string";
	char 	*exp1pat  = "^   12345  1234    11145401322        321\\.765  *3\\.21765432e+0*02   5    test-string\n";
	int 	exp1len  = sizeof("   12345  1234    11145401322        321.765   3.21765432e+02   5    test-string\n") - 1;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	char	*val2str  = "-8, -5, 12345, -5, -2, 1234, -14, -5, 1234567890, -14, -3, 321.7654321, -16, -8, 321.7654321, -3, -4, 5.0000001, -14, -15, test-string";
	char	*exp2pat = "12345    1234  11145401322    321.765432     3.217654e+02     5   test-string   \n";
	int	exp2len = sizeof("12345    1234  11145401322    321.765432     3.217654e+02     5   test-string   \n") - 1;
#endif

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	j, len, ret, err;
	char	tbuf[50];
	char	*conv3str = "%*.*S";
#endif

	int	pathok = setupok;

	DBUG_ENTER("t36");

	(void) cppair(ch_t36, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(*.*) format modifier takes the field width and precision from the parameter list";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

	globok = 0;
	checkpat(conv1str, val2str, exp2pat, exp2len);
	PATH_FUNC_TRACE;

#else
	PATH_TRACE;
#endif

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3

	wclocale();

	if((ret = wcstombs(tbuf, wcstr, 15)) == (size_t)-1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wcstombs() failed : errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tbuf[ret] = '\0';
	len = strlen(tbuf);

	for(j = 0; len < 20; j++, len++)
		wexpstr[j] = ' ';
	wexpstr[j] = '\0';
	(void) strcat(wexpstr, tbuf);

	(void) strcpy(wvalstr, pr_wstr(wcstr));
	(void) strcat(wvalstr, ", 20, 15");

	globok = 0;
	checkval(conv3str, wvalstr, wexpstr);
	PATH_FUNC_TRACE;

#else
	PATH_TRACE;
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t36()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t36");
			
	ch_setup();

	PRINTIT("%*.*d %*.*hi %*.*lo %*.*f %*.*e %*.*g %*.*s\n" _ 8 _ 5 _ i _ 5 _ 2 _ h _ 14 _ 5 _ l _ 14 _ 3 _ d _ 16 _ 8 _ d _ 3 _ 4 _ g _ 14 _ 15 _ s);

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

	PRINTIT("%*.*d %*.*hi %*.*lo %*.*f %*.*e %*.*g %*.*s\n" _ -8 _ -5 _ i _ -5 _ -2 _ h _ -14 _ -5 _ l _ -14 _ -3 _ d _ -16 _ -8 _ d _ -3 _ -4 _ g _ -14 _ -15 _ s);

#endif

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3

	wclocale();
	PRINTIT("%*.*S" _ 20 _ 15 _ wcstr);

#endif

	DBUG_VOID_RETURN;
}


public void
test37()
{
	DBUG_ENTER("test37");

	(void) cppair(t37, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t37()
{
	char	*conv1str = "%#f %#e %#o %#lo %#x %#lX %#g\\n";
	char	*val1str  = "321.7654321, 321.7654321, 12345, 1234567890, 12345, 1234567890, 5.0000001";
	char 	*exp1pat = "^321\\.765432 3\\.217654e+0*02 030071 011145401322 0x3039 0X499602D2 5\\.00000\n";
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%#d %#c %#s %#hi\\n";
	char	*val2str  = "12345, 'a', test-string, 1234";
	char 	*exp2str = "12345 a test-string 1234\n";
#endif

	DBUG_ENTER("t37");

	(void) cppair(ch_t37, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(#) format modifier converts input to alternate format";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t37()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t37");
			
	ch_setup();

	PRINTIT("%#f %#e %#o %#lo %#x %#lX %#g\n" _ d _ d _ u _ l _ u _ l _ g);

#if TEST_XPG == 3

	PRINTIT("%#d %#c %#s %#hi\n" _ i _ c _ s _ h);
#endif

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#else /* ANSI_VPRINTF_FUNC */
static int dummy;
#endif

