/*
*      SCCS:  @(#)  ANSI.os/streamio/vprintf/vprintf2.c	Rel 4.4.4	    (07/19/99)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/vprintf - T.vprintf Rel 4.4.4 (07/19/99)";
char	*copyright[] =
{
	"(C) Copyright 1991 X/Open Company Limited", "All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		vprintf2.c

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the vprintf() subroutine.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	25 May 1989
MODIFICATIONS:
		Geoff Clare, 31 May 1989
		Add EIO test

		Geoff Clare, 23 Sep 1989
		Recognise deletion messages from child in do_ferr() tests.

		Steve Ratcliffe, 25 Jan 1990
		Add long double test.

		Steve Ratcliffe, 22 Mar 1990
		Add ENOSPC test.

		Geoff Clare, 14 June 1990
		Do EAGAIN and EPIPE tests on pipe and FIFO

		David Sawyer
		Fri Jul 13 10:22:43 BST 1990
		Added resillience code.

		J.A. Nave	Mon Oct 22 11:44:52 BST 1990
		Added tests for negative values for the precision
		and field width in the format string

		Stuart Boutell, 5 Nov 1990
			changed dltd and reason to tet_reason,
			eliminated vg_env.

		Neil Moses, 7 Nov 1991
			Changes for VSX 4.2
			Added new wide character conversion types C and S
			Removed obsolete tests 60 and 61

		Neil Moses, 16 Jan 1992
			Account for unsupported multibyte characters.

		Geoff Clare, 20 Jan 1992
			Ensure wctomb() called in correct locale.

		Neil Moses, 27 Jan 1992
			Changed expected output for %S format with precision
			Changed expected output for negative precision

		Geoff Clare, 21 Apr 1992
			Change args for o, u, x, X conversions to unsigned.
			Remove tests for %#C and %#S.
			Test %#d, %#i, %#c and %#s in XPG3 mode only.

		Geoff Clare, 21 May 1992
			Make code conditional on ANSI_VPRINTF_FUNC.

		Geoff Clare, 15 June 1992
			Make exp1pat in test 57 an array instead of a
			pointer, so it can be written to.

		John-Paul, 15 November 1992
			XNFS enhancements.

		Geoff Clare, 8 April 1993
			Correct arguments to t70_func and t70_chk_func.

		Geoff Clare, 22 July 1994
			Implement interpretation VWG/060/110293.

		Geoff Clare, April 1997
		Added tests for vsnprintf() in UNIX98 mode.

		Geoff Clare, 23 June 1998
		Added buffer overflow test for vsnprintf().

		Geoff Clare, 2 Dec 1998
		Align vsnprintf() buffer overflow test with bwg98-006.

************************************************************************/

#include	<std.h>

#if ANSI_VPRINTF_FUNC
#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<nlstest.h>
#include	<locale.h>
#include	<nlsinfo.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<chktimes.h>
#include	<string.h>
#include	<local.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"vprintf_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef vfprintf
#undef vprintf
#undef vsprintf
#undef vsnprintf
#endif


extern	char	**environ;

extern	char	*itoa();
#if defined(_XOPEN_SOURCE) && TEST_XPG > 3 && !TEST_XNFS
extern	char	*pr_wstr();
extern	char	*pr_wchar();
#endif
extern	char	*errname();
extern	char	*execpath();
extern	char	*pslanginfo();
extern	void	do_feio();

extern	int	langnum;
extern	char	*currlang;
extern	char	*tfile;
#if TEST_XNFS
private	char	*t69_file = T69_FILE;
private	char	*t70_file = T70_FILE;
private	char	*t_string_xnfs =	XNFS_STRING;
#endif /* TEST_XNFS */
extern	int	interface;
extern	FILE	*f_rd;
extern	FILE	*f_wr;
extern	char	*function;
extern	char	buf[BUFFERSIZE];

extern	char	*vg_ident;
extern	char	*vg_dbug;

extern	int	tretval;
extern	int	testfail;
extern	int	setupok;
extern	char	*testcomment;
extern	int	pipdes[2];

#if defined(_XOPEN_SOURCE) && TEST_XPG > 3 && !TEST_XNFS
extern char	*locale;
extern wchar_t	*wcstr;
extern char	mbstr[BUFFERSIZE];
extern char	wexpstr[BUFFERSIZE];
extern char	wvalstr[BUFFERSIZE];
extern void	wclocale();
#endif

extern void	ch_setup();
extern void	checkval();
extern void	checkpat();
extern void	chk_narg();

#if !TEST_XNFS

private void	ch_t38();
private void	t38();
private void	ch_t39();
private void	t39();
private void	ch_t40();
private void	t40();
private void	ch_t41();
private void	t41();
private void	ch_t42();
private void	t42();
private void	ch_t43();
private void	t43();
private void	ch_t44();
private void	t44();
private void	ch_t45();
private void	t45();
private void	ch_t46();
private void	t46();
private void	ch_t47();
private void	t47();
private void	ch_t48();
private void	t48();
private void	ch_t49();
private void	t49();
private void	ch_t50();
private void	t50();
private void	ch_t51();
private void	t51();
private void	ch_t52();
private void	t52();
private void	ch_t53();
private void	t53();
private void	ch_t54();
private void	t54();
private void	ch_t55();
private void	t55();
#if	LONG_DOUBLE_SUPPORTED
private void	ch_t56();
#endif
private void	ch_t57();
private void	ch_t58();
#endif /* TEST_XNFS */
private void	ch_t59();
private void	ch_t60_66();
private void	ch_t68();
private void	ch_t69();
private int	t60_66func();
#if !TEST_XNFS
private int	t67io();
private void	t67rpt();
#else /* TEST_XNFS */
private int	t70_func();
private int	t70_chk_func();
#endif /* TEST_XNFS */

private char	*prog68 =	"vprintf_t68";

private char	*t68_dir =	"./vprintf-t.68";

#if TEST_XNFS
private char	*prog_su =	"vprintf_su";
#endif /* TEST_XNFS */

#if !TEST_XNFS

public void
test38()
{
	DBUG_ENTER("test38");

	(void) cppair(t38, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t38()
{
	char	*conv1str = "% u % lo % c % s % d % d % f % e % g % hi % hi\\n";
	char	*val1str  = "12345, 1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 1234, -1234";
	char	*exp1pat = "^12345 11145401322 a test-string -12345  12345  321\\.765432  3\\.217654e+0*02  5  1234 -1234\n";
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	i, j, err;
	char	tbuf[50];
	char	*conv2str = "% C";
	char	*conv3str = "% S";
#endif

	DBUG_ENTER("t38");
	
	(void) cppair(ch_t38, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "( ) modifier replaces a leading positive sign with a space";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j++] = '\n';
		tbuf[j] = '\0';
		globok = 0;
		checkval(conv2str, pr_wchar(*(wcstr + i)), tbuf);
		PATH_FUNC_TRACE;
	}

	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), mbstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2 + i);
#else
	if (!testfail)
		PATH_XS_RPT(1);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t38()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	j;
#endif

	DBUG_ENTER("ch_t38");
			
	ch_setup();

	PRINTIT("% u % lo % c % s % d % d % f % e % g % hi % hi\n" _ u _ l _ c _ s _ -i _ i _ d _ d _ g _ h _ -h);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("% C\n" _ *(wcstr + j));

	PRINTIT("% S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test39()
{
	DBUG_ENTER("test39");

	(void) cppair(t39, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t39()
{
	char	*conv1str = "%+u %+lo %+c %+s %+d %+d %+f %+e %+g %+hi %+hi\\n";
	char	*val1str  = "12345, 1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 1234, -1234";
	char	*exp1pat = "^12345 11145401322 a test-string -12345 +12345 +321\\.765432 +3\\.217654e+0*02 +5 +1234 -1234\n";
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	i, j, err;
	char	tbuf[50];
	char	*conv2str = "%+C";
	char	*conv3str = "%+S";
#endif

	DBUG_ENTER("t39");

	(void) cppair(ch_t39, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(+) format modifier causes a sign to be output for a signed conversion";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j++] = '\n';
		tbuf[j] = '\0';
		globok = 0;
		checkval(conv2str, pr_wchar(*(wcstr + i)), tbuf);
		PATH_FUNC_TRACE;
	}

	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), mbstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2 + i);
#else
	if (!testfail)
		PATH_XS_RPT(1);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t39()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	j;
#endif

	DBUG_ENTER("ch_t39");
			
	ch_setup();

	PRINTIT("%+u %+lo %+c %+s %+d %+d %+f %+e %+g %+hi %+hi\n" _ u _ l _ c _ s _ -i _ i _ d _ d _ g _ h _ -h);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%+C\n" _ *(wcstr + j));

	PRINTIT("%+S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test40()
{
	DBUG_ENTER("test40");

	(void) cppair(t40, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t40()
{
	char	*conv1str = "%-8u %-14lo %-4c %-14s %-8d %-8d %-14f %-14e %-3g %-6hi %-6hi\\n";
	char	*val1str  = "12345, 1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 1234, -1234";
	char	*exp1pat = "^12345    11145401322    a    test-string    -12345   12345    321\\.765432     3\\.217654e+0*02 * 5   1234   -1234 \n";
	int	exp1len = sizeof("12345    11145401322    a    test-string    -12345   12345    321.765432     3.217654e+02   5   1234   -1234 \n") - 1;
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	i, j, err, len;
	char	tbuf[50];
	char	*conv2str = "%-20C";
	char	conv3str[20];
#endif

	DBUG_ENTER("t40");

	(void) cppair(ch_t40, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(-n) format modifier causes left justification within the field width n";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';
		(void) strcpy(wexpstr, tbuf);
		for(; j < 20; j++)
			wexpstr[j] = ' ';
		wexpstr[j++] = '\n';
		wexpstr[j] = '\0';
		globok = 0;
		checkval(conv2str, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	len = strlen(mbstr) + 5;
	(void) strcpy(conv3str, "%-");
	(void) strcat(conv3str, itoa(len));
	(void) strcat(conv3str, "S");
	(void) strcpy(wexpstr, mbstr);
	(void) strcat(wexpstr, "     ");
	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2 + i);
#else
	if (!testfail)
		PATH_XS_RPT(1);
#endif
	DBUG_VOID_RETURN;
}

private void
ch_t40()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	char	format[20];
	int	j, len;
#endif

	DBUG_ENTER("ch_t40");
			
	ch_setup();

	PRINTIT("%-8u %-14lo %-4c %-14s %-8d %-8d %-14f %-14e %-3g %-6hi %-6hi\n" _ u _ l _ c _ s _ -i _ i _ d _ d _ g _ h _ -h);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%-20C\n" _ *(wcstr + j));

	len = strlen(mbstr) + 5;
	(void) strcpy(format, "%-");
	(void) strcat(format, itoa(len));
	(void) strcat(format, "S");
	PRINTIT(format _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test41()
{
	DBUG_ENTER("test41");

	(void) cppair(t41, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t41()
{
	char	*conv1str = "%+-8u %+-14lo %+-4c %+-14s %+-8d %+-8d %+-14f %+-14e %+-3g %+-6hi %+-6hi\\n";
	char	*val1str  = "12345, 1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 1234, -1234";
	char	*exp1pat = "^12345    11145401322    a    test-string    -12345   +12345   +321\\.765432    +3\\.217654e+0*02 * +5  +1234  -1234 \n";
	int	exp1len = sizeof("12345    11145401322    a    test-string    -12345   +12345   +321.765432    +3.217654e+02  +5  +1234  -1234 \n") - 1;
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	i, j, err, len;
	char	tbuf[50];
	char	*conv2str = "%+-20C";
	char	conv3str[20];
#endif

	DBUG_ENTER("t41");

	(void) cppair(ch_t41, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(+-n) format modifier causes left justification and sign inclusion";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';
		(void) strcpy(wexpstr, tbuf);
		for(; j < 20; j++)
			wexpstr[j] = ' ';
		wexpstr[j++] = '\n';
		wexpstr[j] = '\0';
		globok = 0;
		checkval(conv2str, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	len = strlen(mbstr) + 5;
	(void) strcpy(conv3str, "%-");
	(void) strcat(conv3str, itoa(len));
	(void) strcat(conv3str, "S");
	(void) strcpy(wexpstr, mbstr);
	(void) strcat(wexpstr, "     ");
	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2 + i);
#else
	if (!testfail)
		PATH_XS_RPT(1);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t41()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	char	format[20];
	int	j, len;
#endif

	DBUG_ENTER("ch_t41");
			
	ch_setup();

	PRINTIT("%+-8u %+-14lo %+-4c %+-14s %+-8d %+-8d %+-14f %+-14e %+-3g %+-6hi %+-6hi\n" _ u _ l _ c _ s _ -i _ i _ d _ d _ g _ h _ -h);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%+-20C\n" _ *(wcstr + j));

	len = strlen(mbstr) + 5;
	(void) strcpy(format, "%+-");
	(void) strcat(format, itoa(len));
	(void) strcat(format, "S");
	PRINTIT(format _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test42()
{
	DBUG_ENTER("test42");

	(void) cppair(t42, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t42()
{
	char	*conv1str = "% -8u % -14lo % -4c % -14s % -8d % -8d % -14f % -14e % -3g % -6hi % -6hi\\n";
	char	*val1str  = "12345, 1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 1234, -1234";
	char	*exp1pat = "^12345    11145401322    a    test-string    -12345    12345    321\\.765432     3\\.217654e+0*02 * 5   1234  -1234 \n";
	int	exp1len = sizeof("12345    11145401322    a    test-string    -12345    12345    321.765432     3.217654e+02   5   1234  -1234 \n") - 1;
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	i, j, err, len;
	char	tbuf[50];
	char	*conv2str = "% -20C";
	char	conv3str[20];
#endif

	DBUG_ENTER("t42");

	(void) cppair(ch_t42, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "( -n) format modifier causes left justification with positive signs replaced by space";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}

		tbuf[j] = '\0';
		(void) strcpy(wexpstr, tbuf);
		for(; j < 20; j++)
			wexpstr[j] = ' ';
		wexpstr[j++] = '\n';
		wexpstr[j] = '\0';
		globok = 0;
		checkval(conv2str, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	len = strlen(mbstr) + 5;
	(void) strcpy(conv3str, "% -");
	(void) strcat(conv3str, itoa(len));
	(void) strcat(conv3str, "S");
	(void) strcpy(wexpstr, mbstr);
	(void) strcat(wexpstr, "     ");
	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2 + i);
#else
	if (!testfail)
		PATH_XS_RPT(1);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t42()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	char	format[20];
	int	j, len;
#endif

	DBUG_ENTER("ch_t42");
			
	ch_setup();

	PRINTIT("% -8u % -14lo % -4c % -14s % -8d % -8d % -14f % -14e % -3g % -6hi % -6hi\n" _ u _ l _ c _ s _ -i _ i _ d _ d _ g _ h _ -h);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("% -20C\n" _ *(wcstr + j));

	len = strlen(mbstr) + 5;
	(void) strcpy(format, "% -");
	(void) strcat(format, itoa(len));
	(void) strcat(format, "S");
	PRINTIT(format _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test43()
{
	DBUG_ENTER("test43");

	(void) cppair(t43, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t43()
{
	char	*conv1str = "%+ u %+ lo %+ c %+ s %+ d %+ d %+ f %+ e %+ g %+ hi %+ hi\\n";
	char	*val1str  = "12345, 1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 1234, -1234";
	char	*exp1pat = "^12345 11145401322 a test-string -12345 +12345 +321\\.765432 +3\\.217654e+0*02 +5 +1234 -1234\n";
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	i, j, err;
	char	tbuf[50];
	char	*conv2str = "%+ C";
	char	*conv3str = "%+ S";
#endif

	DBUG_ENTER("t43");

	(void) cppair(ch_t43, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(+ ) format modifier acts the same as the (+) format modifier";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';
		(void) strcpy(wexpstr, tbuf);
		(void) strcat(wexpstr, "\n");
		globok = 0;
		checkval(conv2str, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), mbstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2 + i);
#else
	if (!testfail)
		PATH_XS_RPT(1);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t43()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	j;
#endif

	DBUG_ENTER("ch_t43");
			
	ch_setup();

	PRINTIT("%+ u %+ lo %+ c %+ s %+ d %+ d %+ f %+ e %+ g %+ hi %+ hi\n" _ u _ l _ c _ s _ -i _ i _ d _ d _ g _ h _ -h);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%+ C\n" _ *(wcstr + j));

	PRINTIT("%+ S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test44()
{
	DBUG_ENTER("test44");

	(void) cppair(t44, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t44()
{
	char	*conv1str = "%- +u %- +lo %- +c %- +s %- +d %- +d %- +f %- +e %- +g %- +hi %- +hi\\n";
	char	*val1str  = "12345, 1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 1234, -1234";
	char	*exp1pat  = "^12345 11145401322 a test-string -12345 +12345 +321\\.765432 +3\\.217654e+0*02 +5 +1234 -1234\n";
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	i, j, err;
	char	tbuf[50];
	char	*conv2str = "%- +C";
	char	*conv3str = "%- +S";
#endif

	DBUG_ENTER("t44");

	(void) cppair(ch_t44, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(- +) format modifier acts the same as the (-+) format modifier";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';
		(void) strcpy(wexpstr, tbuf);
		(void) strcat(wexpstr, "\n");
		globok = 0;
		checkval(conv2str, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), mbstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2 + i);
#else
	if (!testfail)
		PATH_XS_RPT(1);
#endif
	
	DBUG_VOID_RETURN;
}

private void
ch_t44()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	j;
#endif

	DBUG_ENTER("ch_t44");
			
	ch_setup();

	PRINTIT("%- +u %- +lo %- +c %- +s %- +d %- +d %- +f %- +e %- +g %- +hi %- +hi\n" _ u _ l _ c _ s _ -i _ i _ d _ d _ g _ h _ -h);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%- +C\n" _ *(wcstr + j));

	PRINTIT("%- +S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test45()
{
	DBUG_ENTER("test45");

	(void) cppair(t45, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t45()
{
	char	*conv1str = "%+8.*d %+6.*hi %+14.*lo %+14.*f %+16.*e %+3.*g %+14.*s\\n";
	char	*val1str  = "5, 12345, 2, 1234, 5, 1234567890, 3, 321.7654321, 8, 321.87654321, 4, 5.0000001, 15, test-string";
	char	*exp1pat  = "^  +12345  +1234    11145401322       +321\\.765  *+3\\.21765432e+0*02  +5    test-string\n";
	int	exp1len  = sizeof("  +12345  +1234    11145401322       +321.765  +3.21765432e+02  +5    test-string\n") - 1;
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	j, err, len, ret;
	char	tbuf[50];
	char	*conv3str = "%+20.*S";
#endif

	DBUG_ENTER("t45");

	(void) cppair(ch_t45, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(+n.*) format modifier combination converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	if((ret = wcstombs(tbuf, wcstr, 15)) == (size_t)-1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wcstombs() failed : errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tbuf[ret] = '\0';
	len = strlen(tbuf);

	for(j = 0; len < 20; j++, len++)
		wexpstr[j] = ' ';
	wexpstr[j] = '\0';
	(void) strcat(wexpstr, tbuf);

	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

#else
	PATH_TRACE;
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_t45()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t45");
			
	ch_setup();

	PRINTIT("%+8.*d %+6.*hi %+14.*lo %+14.*f %+16.*e %+3.*g %+14.*s\n" _ 5 _ i _ 2 _ h _ 5 _ l _ 3 _ d _ 8 _ d _ 4 _ g _ 15 _ s);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	PRINTIT("%+20.*S" _ 15 _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test46()
{
	DBUG_ENTER("test46");

	(void) cppair(t46, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t46()
{
	char	*conv1str = "%-8.5d %-5.2hi %-14.5lo %-14.3f %-16.8e %-3.4g %-14.15s\\n";
	char	*val1str  = "12345, 1234, 1234567890, 321.7654321, 321.87654321, 5.0000001, test-string";
	char	*exp1pat = "^12345    1234  11145401322    321\\.765        3\\.21765432e+0*02 * 5   test-string   \n";
	int	exp1len = sizeof("12345    1234  11145401322    321.765        3.21765432e+02   5   test-string   \n") - 1;
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	int	j, err, len, ret;
	char	tbuf[50];
	char	*conv3str = "%-20.15S";
#endif

	DBUG_ENTER("t46");

	(void) cppair(ch_t46, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(-n.m) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	if((ret = wcstombs(tbuf, wcstr, 15)) == (size_t)-1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wcstombs() failed : errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tbuf[ret] = '\0';
	len = strlen(tbuf);

	(void) strcpy(wexpstr, tbuf);
	for(j = len; j < 20; j++)
		wexpstr[j] = ' ';
	wexpstr[j] = '\0';

	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

#else
	PATH_TRACE;
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_t46()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t46");
			
	ch_setup();

	PRINTIT("%-8.5d %-5.2hi %-14.5lo %-14.3f %-16.8e %-3.4g %-14.15s\n" _ i _ h _ l _ d _ d _ g _ s);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	PRINTIT("%-20.15S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test47()
{
	DBUG_ENTER("test47");

	(void) cppair(t47, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t47()
{
	char	*conv1str = "%+-8.d %+-6.hi %+-14.lo %+-14.f %+-14.e %+-3.g %+-14.s\\n";
	char	*val1str  = "12345, 1234, 1234567890, 321.7654321, 321.87654321, 5.0000001, test-string";
	char	*exp1pat  = "^+12345   +1234  11145401322    +32[12]           +3e+0*02 * +5                \n";
	int	exp1len  = sizeof("+12345   +1234  11145401322    +321           +3e+02         +5                \n") - 1;
	int	pathok = setupok;
#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	char	*conv3str = "%+-20.S";
#endif

	DBUG_ENTER("t47");

	(void) cppair(ch_t47, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(+-n.) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	(void) strcpy(wexpstr, "                    ");
	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t47()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t47");
			
	ch_setup();

	PRINTIT("%+-8.d %+-6.hi %+-14.lo %+-14.f %+-14.e %+-3.g %+-14.s\n" _ i _ h _ l _ d _ d _ g _ s);

#if defined (_XOPEN_SOURCE) && TEST_XPG > 3
	wclocale();

	PRINTIT("%+-20.S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test48()
{
	DBUG_ENTER("test48");

	(void) cppair(t48, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t48()
{
	char	*conv1str = "%#.lo %#.f %#.e %#.g\\n";
	char	*val1str  = "1234567890, 321.7654321, 321.87654321, 5.0000001";
	char	*exp1pat  = "^011145401322 32[12]\\. 3\\.e+0*02 5\\.\n";
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%#.d %#.hi %#.s\\n";
	char	*val2str  = "12345, 1234, test-string";
	char	*exp2str  = "12345 1234 \n";
#endif

	DBUG_ENTER("t48");

	(void) cppair(ch_t48, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(#.) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t48()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t48");
			
	ch_setup();

	PRINTIT("%#.lo %#.f %#.e %#.g\n" _ l _ d _ d _ g);

#if TEST_XPG == 3
	PRINTIT("%#.d %#.hi %#.s\n" _ i _ h _ s);
#endif

	DBUG_VOID_RETURN;
}


public void
test49()
{
	DBUG_ENTER("test49");

	(void) cppair(t49, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t49()
{
	char	*conv1str = "%#*.lo %#*.f %#*.e %#*.g\\n";
	char	*val1str  = "14, 1234567890, 14, 321.7654321, 14, 321.87654321, 3, 5.0000001";
	char	*exp1pat  = "^  011145401322           32[12]\\.  *3\\.e+0*02  5\\.\n";
	int	exp1len  = sizeof("  011145401322           321.         3.e+02  5.\n") - 1;
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%#*.d %#*.hi %#*.s\\n";
	char	*val2str  = "8, 12345, 5, 1234, 14, test-string";
	char	*exp2str  = "   12345  1234               \n";
#endif
	
	DBUG_ENTER("t49");

	(void) cppair(ch_t49, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(#*.) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t49()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t49");
			
	ch_setup();

	PRINTIT("%#*.lo %#*.f %#*.e %#*.g\n" _ 14 _ l _ 14 _ d _ 14 _ d _ 3 _ g);

#if TEST_XPG == 3
	PRINTIT("%#*.d %#*.hi %#*.s\n" _ 8 _ i _ 5 _ h _ 14 _ s);
#endif

	DBUG_VOID_RETURN;
}


public void
test50()
{
	DBUG_ENTER("test50");

	(void) cppair(t50, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t50()
{
	char	*conv1str = "%#.15lo %#.3f %#.4e %#.4g\\n";
	char	*val1str  = "1234567890, 321.7654321, 321.87654321, 5.0000001";
	char	*exp1pat  = "^000011145401322 321\\.765 3\\.217[67]e+0*02 5\\.000\n";
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%#.10d %#.5hi %#.7s\\n";
	char	*val2str  = "12345, 1234, test-string";
	char	*exp2str  = "0000012345 01234 test-st\n";
#endif

	DBUG_ENTER("t50");

	(void) cppair(ch_t50, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(#.m) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t50()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t50");
			
	ch_setup();

	PRINTIT("%#.15lo %#.3f %#.4e %#.4g\n" _ l _ d _ d _ g);

#if TEST_XPG == 3
	PRINTIT("%#.10d %#.5hi %#.7s\n" _ i _ h _ s);
#endif

	DBUG_VOID_RETURN;
}


public void
test51()
{
	DBUG_ENTER("test51");

	(void) cppair(t51, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t51()
{
	char	*conv1str = "%# +*lo %# +*f %# +*e %# +*g\\n";
	char	*val1str  = "13, 1234567890, 13, 321.7654321, 16, 321.87654321, 10, 5.0000001";
	char	*exp1pat  = "^ 011145401322   +321\\.765432  *+3\\.217654e+0*02   +5\\.00000\n";
	int	exp1len  = sizeof(" 011145401322   +321.765432    +3.217654e+02   +5.00000\n") - 1;
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%# +*d %# +*hi %# +*c %# +*s\\n";
	char	*val2str  = "8, 12345, 6, 1234, 2, 'a', 12, test-string";
	char	*exp2str  = "  +12345  +1234  a  test-string\n";
#endif


	DBUG_ENTER("t51");

	(void) cppair(ch_t51, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(# +*) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t51()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t51");
			
	ch_setup();

	PRINTIT("%# +*lo %# +*f %# +*e %# +*g\n" _ 13 _ l _ 13 _ d _ 16 _ d _ 10 _ g);

#if TEST_XPG == 3
	PRINTIT("%# +*d %# +*hi %# +*c %# +*s\n" _ 8 _ i _ 6 _ h _ 2 _ c _ 12 _ s);
#endif

	DBUG_VOID_RETURN;
}


public void
test52()
{
	DBUG_ENTER("test52");

	(void) cppair(t52, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t52()
{
	char	*conv1str = "%#+-.*lo %#+-.*f %#+-.*e %#+-.*g\\n";
	char	*val1str  = "15, 1234567890, 3, 321.7654321, 4, 321.7654321, 4, 5.0000001";
	char	*exp1pat  = "^000011145401322 +321\\.765 +3\\.217[67]e+0*02 +5\\.000\n";
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%#+-.*d %#+-.*hi %#+-.*s\\n";
	char	*val2str  = "10, 12345, 6, 1234, 7, test-string";
	char	*exp2str  = "+0000012345 +001234 test-st\n";
#endif

	DBUG_ENTER("t52");

	(void) cppair(ch_t52, NULLFN, WAITTIME, 0);
	
	testfail = 0;
	testcomment = "(#+-.*) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t52()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t52");
			
	ch_setup();

	PRINTIT("%#+-.*lo %#+-.*f %#+-.*e %#+-.*g\n" _ 15 _ l _ 3 _ d _ 4 _ d _ 4 _ g);

#if TEST_XPG == 3
	PRINTIT("%#+-.*d %#+-.*hi %#+-.*s\n" _ 10 _ i _ 6 _ h _ 7 _ s);
#endif

	DBUG_VOID_RETURN;
}


public void
test53()
{
	DBUG_ENTER("test53");

	(void) cppair(t53, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t53()
{
	char	*conv1str = "%# *.*lo %# *.*f %# *.*e %# *.*g\\n";
	char	*val1str  = "14, 5, 1234567890, 7, 3, 321.7654321, 16, 8, 321.7654321, 4, 2, 5.0000001";
	char	*exp1pat  = "^  011145401322  321\\.765  *3\\.21765432e+0*02  5\\.0\n";
	int	exp1len  = sizeof("  011145401322  321.765   3.21765432e+02  5.0\n") - 1;
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%# *.*d %# *.*hi %# *.*s\\n";
	char	*val2str  = "8, 5, 12345, 5, 2, 1234, 14, 15, test-string";
	char	*exp2str  = "   12345  1234    test-string\n";
#endif

	DBUG_ENTER("t53");

	(void) cppair(ch_t53, NULLFN, WAITTIME, 0);


	testfail = 0;
	testcomment = "(# *.*) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t53()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t53");
			
	ch_setup();

	PRINTIT("%# *.*lo %# *.*f %# *.*e %# *.*g\n" _ 14 _ 5 _ l _ 7 _ 3 _ d _ 16 _ 8 _ d _ 4 _ 2 _ g);

#if TEST_XPG == 3
	PRINTIT("%# *.*d %# *.*hi %# *.*s\n" _ 8 _ 5 _ i _ 5 _ 2 _ h _ 14 _ 15 _ s);
#endif

	DBUG_VOID_RETURN;

}


public void
test54()
{
	DBUG_ENTER("test54");

	(void) cppair(t54, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t54()
{
	char	*conv1str = "%#- +14lo %#- +14f %#- +16e %#- +3g\\n";
	char	*val1str  = "1234567890, 321.7654321, 321.87654321, 5.0000001";
	char	*exp1pat = "^011145401322   +321\\.765432    +3\\.217654e+0*02 * +5\\.00000\n";
	int	exp1len = sizeof("011145401322   +321.765432    +3.217654e+02    +5.00000\n") - 1;
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%#- +8d %#- +6hi %#- +4c %#- +14s\\n";
	char	*val2str  = "12345, 1234, 'a', test-string";
	char	*exp2str = "+12345   +1234  a    test-string   \n";
#endif


	DBUG_ENTER("t54");

	(void) cppair(ch_t54, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(#- +n) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t54()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t54");
			
	ch_setup();

	PRINTIT("%#- +14lo %#- +14f %#- +16e %#- +3g\n" _ l _ d _ d _ g);

#if TEST_XPG == 3
	PRINTIT("%#- +8d %#- +6hi %#- +4c %#- +14s\n" _ i _ h _ c _ s);
#endif

	DBUG_VOID_RETURN;
}


public void
test55()
{
	DBUG_ENTER("test55");

	(void) cppair(t55, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t55()
{
	char	*conv1str = "%#-*.5lo %#-*.3f %#-*.8e %#-*.4g\\n";
	char	*val1str  = "14, 1234567890, 14, 321.7654321, 16, 321.7654321, 3, 5.0000001";
	char	*exp1pat  = "^011145401322   321\\.765        3\\.21765432e+0*02 * 5\\.000\n";
	int	exp1len  = sizeof("011145401322   321.765        3.21765432e+02   5.000\n") - 1;
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%#-*.5d %#-*.2hi %#-*.10s\\n";
	char	*val2str  = "8, 12345, 5, 1234, 12, test-string";
	char	*exp2str  = "12345    1234  test-strin  \n";
#endif


	DBUG_ENTER("t55");

	(void) cppair(ch_t55, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(#-*.m) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t55()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t55");
			
	ch_setup();

	PRINTIT("%#-*.5lo %#-*.3f %#-*.8e %#-*.4g\n" _ 14 _ l _ 14 _ d _ 16 _ d _ 3 _ g);

#if TEST_XPG == 3
	PRINTIT("%#-*.5d %#-*.2hi %#-*.10s\n" _ 8 _ i _ 5 _ h _ 12 _ s);
#endif
	DBUG_VOID_RETURN;
}

public void
test56()
{
#if	LONG_DOUBLE_SUPPORTED

	char	*conv1str = "%Lf %Le %Lg %LE %LG\\n";
	char	*val1str  = "-12.3, -0.0123, 0.0001555, -0.0123, 0.0001555";
	char	*exp1pat  = "-12.300000 -1\\.230000e-0*02 0.0001555 -1\\.230000E-0*02 0.0001555\n";
	char	*conv2str = "%Lf %Le %Lg %LE %LG\\n";
	char	*val2str  = "321.765432, 321.765432, 12345.1234546, 321.765432, 12345678.1234546";
	char	*exp2pat  = "321.765432 3\\.217654e+0*02 12345.1 3\\.217654E+0*02 1\\.2345[67]E+0*07\n";
	int	pathok = setupok;

	DBUG_ENTER("test56");

	(void) cppair(ch_t56, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(L) format modifier specifies argument is long double";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;
	globok = 0;
	checkpat(conv2str, val2str, exp2pat, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

#else /* LONG_DOUBLE_SUPPORTED */

	DBUG_ENTER("test56");

	xx_rpt(UNSUPPORTED);
	in_rpt("Long double is not supported");

#endif /* LONG_DOUBLE_SUPPORTED */

	DBUG_VOID_RETURN;
}

#if	LONG_DOUBLE_SUPPORTED
private void
ch_t56()
{
	long double	d1 = -12.3;
	long double	d2 = -0.0123;
	long double	d3 =  0.0001555;
	long double	d4 = 321.765432;
	long double	d5 = 12345.1234546;
	long double	d6 = 12345678.1234546;

	DBUG_ENTER("test56");

	ch_setup();

	PRINTIT("%Lf %Le %Lg %LE %LG\n" _ d1 _ d2 _ d3 _ d2 _ d3);
	PRINTIT("%Lf %Le %Lg %LE %LG\n" _ d4 _ d4 _ d5 _ d4 _ d6);

	DBUG_VOID_RETURN;
}
#endif	/* LONG_DOUBLE_SUPPORTED */

public void
test57()
{
	char	*convstr = "%e\\n";
	char	*valstr  = "12.35";
	static char	exp1pat[] = "^1\\.235000e+0*01\n";
	char	rad1ch, rad2ch;
	int	rval;
	int	pathok = setupok;
	
	DBUG_ENTER("test57");

	testfail = 0;

	rval = cppair(ch_t57, NULLFN, WAITTIME, E_FAIL);

	if (rval == E_FAIL)
	{
		xx_rpt(DELETION);
		in_rpt("%s", fgets(buf, sizeof(buf), f_rd));
		in_rpt("%s", fgets(buf, sizeof(buf), f_rd));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	GETBUF
	else
		PATH_TRACE;
	rad1ch = buf[0];
	rad2ch = buf[1];

	langnum = VALRAD1LANG;
	currlang = RAD1LANG;
	testcomment = "when setlocale() has been called the radix character for this language is used";

	exp1pat[3] = rad1ch;
	globok = 0;
	checkpat(convstr, valstr, exp1pat, 0);
	PATH_FUNC_TRACE;

	langnum = VALRAD2LANG;
	currlang = RAD2LANG;
	testcomment = "when setlocale() has been called the radix character for this language is used";
	exp1pat[3] = rad2ch;
	globok = 0;
	checkpat(convstr, valstr, exp1pat, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

	langnum = 0;
	currlang = "";

	DBUG_VOID_RETURN;
}

private void
ch_t57()
{
	double	d = 12.35;
	char	*s;
	char	radch[3];

	DBUG_ENTER("ch_t57");

	radch[2] = '\0';

	ch_setup();

	if (setlocale(LC_NUMERIC, RAD2LANG) == (char *)NULL)
	{
		PRINTIT("Pseudo language %s not available\n" _ RAD2LANG);
		PRINTIT("Check installation of pseudo language\n");
		DBUG_EXIT(E_FAIL);
	}

	s = pslanginfo(VSX_RADIXCHAR, VALRAD2LANG);

	radch[1] = s[0];

	if (setlocale(LC_NUMERIC, RAD1LANG) == (char *)NULL)
	{
		PRINTIT("Pseudo language %s not available\n" _ RAD1LANG);
		PRINTIT("Check installation of pseudo language\n");
		DBUG_EXIT(E_FAIL);
	}

	s = pslanginfo(VSX_RADIXCHAR, VALRAD1LANG);

	radch[0] = s[0];

	PRINTIT("%s\n" _ radch);

	PRINTIT("%e\n" _ d);

	(void) setlocale(LC_ALL, RAD2LANG);

	PRINTIT("%e\n" _ d);

	DBUG_VOID_RETURN;
}

public void
test58()
{
	char	*convstr = "%e\\n";
	char	*valstr  = "12.35";
	char	*exp1pat = "^1\\.235000e+0*01\n";
	int	rval;
	int	pathok = setupok;
	
	
	DBUG_ENTER("test58");

	rval = cppair(ch_t58, NULLFN, WAITTIME, E_FAIL);

	testfail = 0;
	testcomment = "when setlocale() has not been called '.' is used as the radix";
	globok = 0;
	checkpat(convstr, valstr, exp1pat, 0);
	PATH_FUNC_TRACE;

	if (rval == E_FAIL)
	{
		xx_rpt(DELETION);
		in_rpt("%s", fgets(buf, sizeof(buf), f_rd));
		in_rpt("%s", fgets(buf, sizeof(buf), f_rd));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "when no radix character is specified for a language '.' is used";
	langnum = VALNORADIXLANG;
	currlang = NORADIXLANG;

	globok = 0;
	checkpat(convstr, valstr, exp1pat, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	langnum = 0;
	currlang = "";

	DBUG_VOID_RETURN;
}

private void
ch_t58()
{
	double	d = 12.35;

	DBUG_ENTER("ch_t58");

	ch_setup();

	PRINTIT("%e\n" _ d);

	if (setlocale(LC_ALL, NORADIXLANG) == (char *)NULL)
	{
		PRINTIT("Pseudo language %s not available\n" _ NORADIXLANG);
		PRINTIT("Check installation of pseudo language\n");
		DBUG_EXIT(E_FAIL);
	}

	PRINTIT("%e\n" _ d);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
test59()
{
	/* Timestamp test for v[f]printf() */
	/* size = 0 test for vsnprintf() */

	int	pathok = setupok;

	DBUG_ENTER("test59");

	if ( interface == I_SP )
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}
#if TEST_XPG >= 500
	else if (interface == I_SNP)
	{
		int ret;

		ret = dosnprintf(buf, (size_t)0, "x");
		if (ret >= 1)
		{
			xx_rpt(FAILURE);
			in_rpt("vsnprintf(buf, 0, ...) returned %d, expected < 1",
				ret);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		PATH_XS_RPT(1);
		DBUG_VOID_RETURN;
	}
#endif
	else
		PATH_TRACE;

	testfail = 0;

	globok = 0;
	if (cppair(ch_t59, NULLFN, WAITTIME, 0) != OK)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "the st_mtime and st_ctime fields of the output file are marked for update";

	GETBUF
	else
		PATH_TRACE;
	if (strcmp(buf, "xp") == 0) /* check path tracing in child */
	{
		PATH_TRACE;
		(void) strcpy(buf, "xs");
	}

	if (strcmp(buf, "xs") == 0)
	{
		PATH_XS_RPT(4);
	}
	else
	{
		xx_rpt(FAILURE);
		in_rpt("%s", testcomment);
		in_rpt("%s", buf);
	}

	DBUG_VOID_RETURN;
}

private void
ch_t59()
{
	char	*s;
	int	dupfd;
	int	pathok = 0;

	DBUG_ENTER("ch_t59");
			
	ch_setup();

	/* save pipe to parent for result message */
	if ((dupfd = dup(fileno(f_wr))) < 0)
		DBUG_EXIT(E_FDOPEN);

	if (freopen(tfile, "w", f_wr) == NULL )
		DBUG_EXIT(E_FDOPEN);

	globok = 0;
	savtimes(tfile);
	PATH_FUNC_TRACE;

	PRINTIT("abcdef\n");

	if (fclose(f_wr) < 0)
		DBUG_EXIT(E_FCLOSE);
	
	globok = 0;
	if ((s = chktimes(tfile, NEWMTIME | NEWCTIME)) == NULL)
	{
		PATH_FUNC_TRACE;
		if (pathok==2)
			s = "xp";
		else
			s = "xs";
	}

	if (outstr(dupfd, s) != strlen(s))
		DBUG_EXIT(E_PUTS);

	DBUG_VOID_RETURN;
}

/* Lowest test number for which following function is used */
#define TBASE	60
private int	experr;

public	void
test60_66()
{
	char *func;
	int err, ret, ferr_ret, gotsig;
	static int errnums[] = { EAGAIN, -EAGAIN, EBADF, EINTR, EFBIG,
			      /* test60  test61   test62 test63 test64 */
				 EPIPE, -EPIPE };
			      /* test65 test66 */
	int	pathok = setupok;

	DBUG_ENTER("test60_66");

	if (interface == I_SP
#if TEST_XPG >= 500
	    || interface == I_SNP
#endif
	   )
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}
	else 
	{
		PATH_TRACE;
		if (interface == I_FP)
			func = "vfprintf()";
		else
			func = "vprintf()";
	}

	experr = errnums[tet_thistest - TBASE];

	(void) cppair(ch_t60_66, NULLFN, 4 * WAITTIME, 0);
	if (experr < 0)
		experr = -experr;

	GETBUF
	else
		PATH_TRACE;
	if (strncmp(buf, "del", (size_t)3) == 0)
	{
		int len;

		GETBUF
		len = strlen(buf);
		if (len > 0 && buf[len-1] == '\n')
			buf[--len] = '\0';
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", buf);
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		ret = atoi(buf);
	}

	GETBUF
	else
		PATH_TRACE;
	err = atoi(buf);
	GETBUF
	else
		PATH_TRACE;
	ferr_ret = atoi(buf);
	GETBUF
	else
		PATH_TRACE;
	gotsig = atoi(buf);

	if (!ferr_ret || err != experr || ret >= 0 ||
				(experr == EPIPE && !gotsig))
	{
		xx_rpt(FAILURE);
		if (!ferr_ret)
			in_rpt("%s failed to set error indicator when testing for errno %d(%s)",
				func, experr, errname(experr));
		if (experr == EPIPE && !gotsig)
			in_rpt("SIGPIPE was not received");
		if (ret >= 0)
			in_rpt("%s returned %d when testing for errno %d(%s), expected < 0",
				func, ret, experr, errname(experr));
		else if (err != experr)
			in_rpt("%s set errno incorrectly: expected %d(%s), actual %d(%s)",
				func, experr, errname(experr),
				err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(7);
	}

	DBUG_VOID_RETURN;
}

private	void
ch_t60_66()
{
	int err, ret, ferr_ret, gotsig = 0, reqfd = -1;

	DBUG_ENTER("ch_t60_66");

	if (interface == I_PR)
		reqfd = STDOUT_FILENO;

	ret = do_ferr(experr, 'w', reqfd, t60_66func, &ferr_ret, &gotsig);
	err = errno;
	if (experr < 0)
		experr = -experr;

	if ( close(pipdes[P_RD]) == SYSERROR )
		DBUG_EXIT(A_CLOSEFAIL);

	if ((f_wr = fdopen(pipdes[P_WR], "w")) == (FILE *) NULL)
		DBUG_EXIT(E_FDOPEN);

	if (tet_reason(tet_thistest)!=NULL)
	{
		(void) fputs("del\n", f_wr);
		(void) fputs(tet_reason(tet_thistest), f_wr);
		(void) putc('\n', f_wr);
	}
	else
	{
		(void) fputs(itoa(ret), f_wr);
		(void) putc('\n', f_wr);
		(void) fputs(itoa(err), f_wr);
		(void) putc('\n', f_wr);
		(void) fputs(itoa(ferr_ret), f_wr);
		(void) putc('\n', f_wr);
		(void) fputs(itoa(gotsig), f_wr);
		(void) putc('\n', f_wr);
	}

	DBUG_VOID_RETURN;
}

private int
t60_66func(fp, nbyte)
FILE *fp;
int nbyte;
{
	DBUG_ENTER("t60_66func");

	f_wr = fp;

	PRINTIT("%*s\n" _ nbyte _ "");

	globok = 1; /* must be set for do_ferr() */

	DBUG_RETURN(tretval);
}

#if !TEST_XNFS

public	void
test67()
{
	int reqfd = -1;

	DBUG_ENTER("test67");

	if (interface == I_SP
#if TEST_XPG >= 500
	    || interface == I_SNP
#endif
	   )
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}

	if (interface == I_PR)
		reqfd = STDOUT_FILENO;

	do_feio("w", reqfd, t67io, t67rpt);

	DBUG_VOID_RETURN;
}

private int
t67io(fp)
FILE *fp;
{
	int	pathok = 0;

	DBUG_ENTER("t67io");

	globok = 0;

	f_wr = fp;

	PRINTIT("\n");

	PATH_FUNC_RPT(0);

	DBUG_RETURN(tretval);
}

private void
t67rpt(ret, err)
int ret, err;
{
	int	pathok = setupok;

	DBUG_ENTER("t67rpt");

	PATH_FUNC_TRACE; /* check path tracing in iofunc() */

	if (ret >= 0 || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("%s() from orphaned background process group did not give correct results",
			(interface == I_FP ? "vfprintf" : "vprintf"));
		if (ret >= 0)
			in_rpt("expected return: < 0, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
test68()
{
	int	pathok = setupok;

	DBUG_ENTER("test68");

	testfail = 0;

	if (interface == I_SP)
	{
		xx_rpt(NOTINUSE);
	}
	else
#if TEST_XPG >= 500
	if (interface == I_SNP)
	{
		/* buffer overflow test for vsnprintf() */

		int ret;

		buf[0] = '\0';
		ret = dosnprintf(buf, (size_t)3, "%s", "abc");

		if (ret != 3)
		{
			xx_rpt(FAILURE);
			in_rpt("vsnprintf(buf, 3, \"%%s\", {\"abc\"}) returned %d, expected 3",
				ret);
			testfail++;
		}
		else
			PATH_TRACE;

		if (ret > 0 && memcmp(buf, "ab", (size_t)3) != 0)
		{
			if (testfail++ != 0)
				xx_rpt(FAILURE);
			in_rpt("vsnprintf(buf, 3, \"%%s\", {\"abc\"}) placed \"%.3s\" in buf, expected \"ab\"",
				buf);
		}
		else
			PATH_TRACE;

		buf[0] = '\0';
		ret = dosnprintf(buf, (size_t)5, "1234567890");

		if (ret != 10)
		{
			if (testfail++ != 0)
				xx_rpt(FAILURE);
			in_rpt("vsnprintf(buf, 5, \"1234567890\", {}) returned %d, expected 10",
				ret);
		}
		else
			PATH_TRACE;

		if (ret > 0 && memcmp(buf, "1234", (size_t)5) != 0)
		{
			if (testfail++ != 0)
				xx_rpt(FAILURE);
			in_rpt("vsnprintf(buf, 5, \"1234567890\", {}) placed \"%.5s\" in buf, expected \"1234\"",
				buf);
		}
		else
			PATH_TRACE;

		if (testfail == 0)
			PATH_XS_RPT(4);
	}
	else
#endif
	{
		/* ENOSPC test for file I/O interfaces */

		(void) cppair(ch_t68, NULLFN, 4 * WAITTIME, 0);
	}

	DBUG_VOID_RETURN;
}

private void
ch_t68()
{
	int 	err;
	char	*args[6];

	DBUG_ENTER("ch_t68");

	args[0] = prog68;
	args[1] = t68_dir;
	args[2] = (interface==I_PR)? "vprintf": "vfprintf";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog68), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

public void
test69()
{
	DBUG_ENTER("test69");

	(void) cppair(ch_t69, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t69()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t69");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t69_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

public void
test70()
{
	DBUG_ENTER("test70");

	t_estale_wr_c(t70_file, STDOUT_XNFS, t70_func, t70_chk_func, SP_T70_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t70_func(fp)
FILE	*fp;
{
	DBUG_ENTER("t70_func");

	f_wr = fp;

	PRINTIT("%s" _ t_string_xnfs);
		
	DBUG_RETURN( (tretval == EOF) ? -1 : 0);
}

private int
t70_chk_func(fp, ebytes)
FILE	*fp;
int	ebytes;
{
	int	err;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("t70_chk_func");

	f_wr = fp;

	errno = NOERROR;

	for(i = 0; i < ebytes; i++)
	{
		PRINTIT("%s" _ t_string_xnfs);
		
		if(tretval == EOF)
			break;
	}
	err = errno;

	if(tretval != EOF || ferror(fp) == 0 || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(tretval != EOF)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value %d (EOF)", EOF);
			in_rpt("Actual   return value %d", tretval);
		}
		else
			PATH_TRACE;
		
		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;
		
		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */
		
#else /* ANSI_VPRINTF_FUNC */
static int dummy;
#endif

