/*
*      SCCS:  @(#)  ANSI.os/streamio/vprintf/vprintf_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/vprintf - vprintf_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		vprintf_su - used by T.vprintf in privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.vprintf to perform privileged tests.

	  usage:
		vprintf_su testnum interface file ident dbug

	  where testnum is the number of the test to be executed,
	  interface is I_PUTC, I_PUTCHAR or I_FPUTC,
	  and file is a parameter to be passed to the test.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		John-Paul, UniSoft Ltd.
DATE CREATED:	9 June 1992
MODIFICATIONS:
		Geoff Clare, 8 April 1993
			Correct arguments to t69_chk_func.

************************************************************************/
#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <dbug.h>
#include <errno.h>
#include <sys/stat.h>
#include <report.h>
#include <tet_api.h>
#include <xnfs.h>
#include "local.h"
#include "vprintf_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef vfprintf
#undef vprintf
#undef vsprintf
#undef vsnprintf
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	**environ;
extern	void	savtimes();
extern	char	*chktimes();
extern	char	*errname();

extern	void	valrpt();
extern	void	set_pname();

#if TEST_XNFS
private	char	*t_string_xnfs = 	XNFS_STRING;
private	FILE	*f_wr;
private	int	tretval;
#endif /* TEST_XNFS */

private	int	interface;
#if TEST_XNFS
private	void	t69_func();
private	int	t69_chk_func();
#endif /* TEST_XNFS */

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int testnum;
	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testnum = atoi(argv[1]);
	interface = atoi(argv[2]);
	switch(testnum)
	{
#if TEST_XNFS
        case 69:
                t_eacces_wr_c(argv[3], STDOUT_XNFS, t69_chk_func, SP_T69_EACCES);
                break;
#endif /* TEST_XNFS */

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed : %d", testnum);
		return(OK);
	}

	return(OK);
}

#if TEST_XNFS

private int
t69_chk_func(fp, ebytes)
FILE	*fp;
int	ebytes;
{
	int	err;
	int 	i;
	int	pathok = 0;

	DBUG_ENTER("t69_chk_func");

	f_wr = fp;

	errno = NOERROR;

	for(i = 0; i < ebytes; i++)
	{
		PRINTIT("%s" _ t_string_xnfs);
		
		if(tretval == EOF)
			break;
	}
	err = errno;

	if(ferror(fp) == 0 || tretval != EOF || err != EACCES)
	{
		xx_rpt(FAILURE);

		if(tretval != EOF)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value EOF");
			in_rpt("Actual   return value %d", tretval);
		}
		else
			PATH_TRACE;

		if(ferror(fp) == 0)
			in_rpt("The stream error indicator was not set");
		else
			PATH_TRACE;
		
		if(err != EACCES)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",EACCES,errname(EACCES));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

#if TEST_ANSI
#include <stdarg.h>
#else
#include <varargs.h>
#endif

/*VARARGS1*/
public int
#if TEST_ANSI
dovprintf(char *fmt, ...)
#else
dovprintf(fmt, va_alist)
char *fmt;
va_dcl
#endif
{
	va_list	args;
	FILE	*stream;
	int	retval;

	DBUG_ENTER("dovprintf");

#if TEST_ANSI
	va_start(args, fmt);
#else
	va_start(args);
#endif
	switch (interface)
	{
	case I_PR:
		retval = vprintf(fmt, args);
		break;

	case I_FP:
		stream = f_wr;
		retval = vfprintf(stream, fmt, args);
		break;
	}
	va_end(args);

	DBUG_RETURN(retval);
}

#endif /* TEST_XNFS */
