/*
*      SCCS:  @(#)  ANSI.os/streamio/vprintf_X/vprintf2_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/vprintf_X - T.vprintf_X Rel 4.4.1 (06/09/97)";
char	*copyright[] =
{
	"(C) Copyright 1991 X/Open Company Limited", "All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		vprintf2_X.c

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the vprintf() subroutine, X/Open extensions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	25 May 1989
MODIFICATIONS:
	Steve Ratcliffe, 30 Jan 1990
		Add long double test.

		David Sawyer
		Wed Jul 18 16:18:17 BST 1990
		Added resillience code.

		Neil Moses, 14 Nov 1991
			Changes for VSX 4.2
			Added new wide character conversion types C and S
			New test 38

		Neil Moses, 16 Jan 1992
			Account for unsupported multibyte characters.

		Neil Moses, 27 Jan 1992
			Ensure wctomb() called in correct locale.
			Changed expected output for %S format with precision
			Changed expected output for negative precision
		
		Geoff Clare, 7 Feb 1992
			Fix problem with test 38 declarations in XPG3 mode.

		Geoff Clare, 21 Apr 1992
			Change args for o, u, x, X conversions to unsigned.
			Remove tests for %#C and %#S.
			Test %#d, %#i, %#c and %#s in XPG3 mode only.

		Geoff Clare, 8 May 1992
			Change test 38 to test i, d, u, f, g, G only.

		Geoff Clare, 29 May 1992
			Don't use literal language names.

		Geoff Clare, 25 July 1994
			Implement interpretation VWG/060/110293.

		Geoff Clare, April 1997
		Added tests for vsnprintf() in UNIX98 mode.

************************************************************************/

#include	<std.h>
#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<chktimes.h>
#include	<string.h>
#include	<locale.h>
#include	<local.h>
#include	<nlstest.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef vfprintf
#undef vprintf
#undef vsprintf
#undef vsnprintf
#endif


#if !TEST_XNFS

extern	int	interface;
extern	FILE	*f_rd;
extern	FILE	*f_wr;
extern	char	*function;
extern	char	buf[BUFFERSIZE];

extern	int	tretval;
extern	int	testfail;
extern	int	setupok;
extern	char	*testcomment;
extern	int	pipdes[2];

#if (TEST_XPG > 3)
extern char	*locale;
extern wchar_t	*wcstr;
extern char	mbstr[BUFFERSIZE];
extern char	wexpstr[BUFFERSIZE];
extern char	wvalstr[BUFFERSIZE];
extern void	newlocale();
#endif

extern void	ch_setup();
extern void	checkval();
extern char	*errname();
extern char	*itoa();
extern char	*pr_wchar();
extern char	*pr_wstr();
extern void	checkpat();
extern void	chk_narg();

#if (TEST_XPG > 3)
private void	t38();
private void	ch_t38();
#endif
private void	t39();
private void	ch_t39();
private void	t40();
private void	ch_t40();
private void	t41();
private void	ch_t41();
private void	t42();
private void	ch_t42();
private void	t43();
private void	ch_t43();
private void	t44();
private void	ch_t44();
private void	t45();
private void	ch_t45();
private void	t46();
private void	ch_t46();
private void	t47();
private void	ch_t47();
private void	t48();
private void	ch_t48();
private void	t49();
private void	ch_t49();
private void	t50();
private void	ch_t50();
private void	t51();
private void	ch_t51();
private void	t52();
private void	ch_t52();
private void	t53();
private void	ch_t53();
private void	t54();
private void	ch_t54();
private void	t55();
private void	ch_t55();
private void	t56();
private void	ch_t56();
#if LONG_DOUBLE_SUPPORTED
private void	ch_t57();
#endif

public void
test38()
{
	DBUG_ENTER("test38");

#if (TEST_XPG > 3)
	(void) cppair(t38, NULLFN, WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}

#if (TEST_XPG > 3)

private void
t38()
{
	char	*conv2str = "%9$'u %1$'hi %2$'hi %3$'G %4$'d %5$'d %6$'f %7$'e %8$'g\\n";
	char	*val2str  = "1234, -1234, 55.0000001, -12345, 12345, 321.7654321, 321.7654321, 55.0000001, 12345";
	char	*exp2pat = "^12,34,5 1,23,4 -1,23,4 5,5 -12,34,5 12,34,5 32,1I765432 3I217654e+0*02 5,5\n";
	int	pathok = setupok;

	DBUG_ENTER("t38");

	(void) cppair(ch_t38, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(') format modifier causes non-monetary grouping";

	globok = 0;
	checkpat(conv2str, val2str, exp2pat, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t38()
{
	double	g = 55.0000001;
	unsigned int	u = 12345;
	double	d = 321.7654321;
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t38");

	ch_setup();

	newlocale(LC_NUMERIC, INFO4LANG);

	PRINTIT("%9$'u %1$'hi %2$'hi %3$'G %4$'d %5$'d %6$'f %7$'e %8$'g\n" _ h _ -h _ g _ -i _ i _ d _ d _ g _ u);

	DBUG_VOID_RETURN;
}
#endif

public void
test39()
{
	DBUG_ENTER("test39");

	(void) cppair(t39, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t39()
{
	char	*conv2str = "%9$ u %1$ lo %2$ c %3$ s %4$ d %5$ d %6$ f %7$ e %8$ g\\n";
	char	*conv3str = "%1$ hi %2$ hi\\n";
	char	*val2str  = "1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 12345";
	char	*val3str  = "1234, -1234";
	char	*exp2pat = "^12345 11145401322 a test-string -12345  12345  321\\.765432  3\\.217654e+0*02  5\n";
	char	*exp3str = " 1234 -1234\n";
	int	pathok = setupok;
#if (TEST_XPG > 3)
	int	i, j, err;
	char	tbuf[50];
	char	*conv4str = "%1$ C";
	char	*conv5str = "%1$ S";
#endif

	DBUG_ENTER("t39");
	
	(void) cppair(ch_t39, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "( ) modifier replaces a leading positive sign with a space";

	globok = 0;
	checkpat(conv2str, val2str, exp2pat, 0);
	PATH_FUNC_TRACE;
	globok = 0;
	checkval(conv3str, val3str, exp3str);
	PATH_FUNC_TRACE;

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j++] = '\n';
		tbuf[j] = '\0';
		globok = 0;
		checkval(conv4str, pr_wchar(*(wcstr + i)), tbuf);
		PATH_FUNC_TRACE;
	}

	globok = 0;
	checkval(conv5str, pr_wstr(wcstr), mbstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3 + i);
#else
	if (!testfail)
		PATH_XS_RPT(2);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t39()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if (TEST_XPG > 3)
	int	j;
#endif

	DBUG_ENTER("ch_t39");
			
	ch_setup();

	PRINTIT("%9$ u %1$ lo %2$ c %3$ s %4$ d %5$ d %6$ f %7$ e %8$ g\n" _ l _ c _ s _ -i _ i _ d _ d _ g _ u);

	PRINTIT("%1$ hi %2$ hi\n" _ h _ -h);

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%1$ C\n" _ *(wcstr + j));

	PRINTIT("%1$ S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}

public void
test40()
{
	DBUG_ENTER("test40");

	(void) cppair(t40, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
t40()
{
	char	*conv2str = "%9$+u %1$+lo %2$+c %3$+s %4$+d %5$+d %6$+f %7$+e %8$+g\\n";
	char	*conv3str = "%1$+hi %2$+hi\\n";
	char	*val2str  = "1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 12345";
	char	*val3str  = "1234, -1234";
	char	*exp2pat = "^12345 11145401322 a test-string -12345 +12345 +321\\.765432 +3\\.217654e+0*02 +5\n";
	char	*exp3str = "+1234 -1234\n";
	int	pathok = setupok;
#if (TEST_XPG > 3)
	int	i, j, err;
	char	tbuf[50];
	char	*conv4str = "%1$+C";
	char	*conv5str = "%1$+S";
#endif

	DBUG_ENTER("t40");

	(void) cppair(ch_t40, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(+) format modifier causes a sign to be output for a signed conversion";

	globok = 0;
	checkpat(conv2str, val2str, exp2pat, 0);
	PATH_FUNC_TRACE;
	globok = 0;
	checkval(conv3str, val3str, exp3str);
	PATH_FUNC_TRACE;

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j++] = '\n';
		tbuf[j] = '\0';
		globok = 0;
		checkval(conv4str, pr_wchar(*(wcstr + i)), tbuf);
		PATH_FUNC_TRACE;
	}

	globok = 0;
	checkval(conv5str, pr_wstr(wcstr), mbstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3 + i);
#else
	if (!testfail)
		PATH_XS_RPT(2);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t40()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if (TEST_XPG > 3)
	int	j;
#endif

	DBUG_ENTER("ch_t40");
			
	ch_setup();

	PRINTIT("%9$+u %1$+lo %2$+c %3$+s %4$+d %5$+d %6$+f %7$+e %8$+g\n" _ l _ c _ s _ -i _ i _ d _ d _ g _ u);

	PRINTIT("%1$+hi %2$+hi\n" _ h _ -h);

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%1$+C\n" _ *(wcstr + j));

	PRINTIT("%1$+S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}

public void
test41()
{
	DBUG_ENTER("test41");

	(void) cppair(t41, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
t41()
{
	char	*conv2str = "%9$-8u %1$-14lo %2$-4c %3$-14s %4$-8d %5$-8d %6$-14f %7$-14e %8$-3g\\n";
	char	*conv3str = "%1$-6hi %2$-6hi\\n";
	char	*val2str  = "1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 12345";
	char	*val3str  = "1234, -1234";
	char	*exp2pat = "^12345    11145401322    a    test-string    -12345   12345    321\\.765432     3\\.217654e+0*02 * 5  \n";
	int	exp2len = sizeof("12345    11145401322    a    test-string    -12345   12345    321.765432     3.217654e+02   5  \n") - 1;
	char	*exp3str = "1234   -1234 \n";
	int	pathok = setupok;
#if (TEST_XPG > 3)
	int	i, j, err, len;
	char	tbuf[50];
	char	*conv4str = "%1$-20C";
	char	conv5str[20];
#endif

	DBUG_ENTER("t41");

	(void) cppair(ch_t41, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(-n) format modifier causes left justification within the field width n";

	globok = 0;
	checkpat(conv2str, val2str, exp2pat, exp2len);
	PATH_FUNC_TRACE;
	globok = 0;
	checkval(conv3str, val3str, exp3str);
	PATH_FUNC_TRACE;

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';
		(void) strcpy(wexpstr, tbuf);
		for(; j < 20; j++)
			wexpstr[j] = ' ';
		wexpstr[j++] = '\n';
		wexpstr[j] = '\0';
		globok = 0;
		checkval(conv4str, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	len = strlen(mbstr) + 5;
	(void) strcpy(conv5str, "%1$-");
	(void) strcat(conv5str, itoa(len));
	(void) strcat(conv5str, "S");
	(void) strcpy(wexpstr, mbstr);
	(void) strcat(wexpstr, "     ");
	globok = 0;
	checkval(conv3str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3 + i);
#else
	if (!testfail)
		PATH_XS_RPT(2);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t41()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if (TEST_XPG > 3)
	char	format[20];
	int	j, len;
#endif

	DBUG_ENTER("ch_t41");
			
	ch_setup();

	PRINTIT("%9$-8u %1$-14lo %2$-4c %3$-14s %4$-8d %5$-8d %6$-14f %7$-14e %8$-3g\n" _ l _ c _ s _ -i _ i _ d _ d _ g _ u);

	PRINTIT("%1$-6hi %2$-6hi\n" _ h _ -h);

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%1$-20C\n" _ *(wcstr + j));

	len = strlen(mbstr) + 5;
	(void) strcpy(format, "%1$-");
	(void) strcat(format, itoa(len));
	(void) strcat(format, "S");
	PRINTIT(format _ wcstr);
#endif

	DBUG_VOID_RETURN;
}

public void
test42()
{
	DBUG_ENTER("test42");

	(void) cppair(t42, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
t42()
{
	char	*conv2str = "%9$+-8u %1$+-14lo %2$+-4c %3$+-14s %4$+-8d %5$+-8d %6$+-14f %7$+-14e %8$+-3g\\n";
	char	*conv3str = "%1$+-6hi %2$+-6hi\\n";
	char	*val2str  = "1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 12345";
	char	*val3str  = "1234, -1234";
	char	*exp2pat = "^12345    11145401322    a    test-string    -12345   +12345   +321\\.765432    +3\\.217654e+0*02 * +5 \n";
	int	exp2len = sizeof("12345    11145401322    a    test-string    -12345   +12345   +321.765432    +3.217654e+02  +5 \n") - 1;
	char	*exp3str = "+1234  -1234 \n";
	int	pathok = setupok;
#if (TEST_XPG > 3)
	int	i, j, err, len;
	char	tbuf[50];
	char	*conv4str = "%1$+-20C";
	char	conv5str[20];
#endif

	DBUG_ENTER("t42");

	(void) cppair(ch_t42, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(+-n) format modifier causes left justification and sign inclusion";

	globok = 0;
	checkpat(conv2str, val2str, exp2pat, exp2len);
	PATH_FUNC_TRACE;
	globok = 0;
	checkval(conv3str, val3str, exp3str);
	PATH_FUNC_TRACE;

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';
		(void) strcpy(wexpstr, tbuf);
		for(; j < 20; j++)
			wexpstr[j] = ' ';
		wexpstr[j++] = '\n';
		wexpstr[j] = '\0';
		globok = 0;
		checkval(conv4str, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	len = strlen(mbstr) + 5;
	(void) strcpy(conv5str, "%-");
	(void) strcat(conv5str, itoa(len));
	(void) strcat(conv5str, "S");
	(void) strcpy(wexpstr, mbstr);
	(void) strcat(wexpstr, "     ");
	globok = 0;
	checkval(conv5str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3 + i);
#else
	if (!testfail)
		PATH_XS_RPT(2);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t42()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if (TEST_XPG > 3)
	char	format[20];
	int	j, len;
#endif

	DBUG_ENTER("ch_t42");
			
	ch_setup();

	PRINTIT("%9$+-8u %1$+-14lo %2$+-4c %3$+-14s %4$+-8d %5$+-8d %6$+-14f %7$+-14e %8$+-3g\n" _ l _ c _ s _ -i _ i _ d _ d _ g _ u);

	PRINTIT("%1$+-6hi %2$+-6hi\n" _ h _ -h);

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%1$+-20C\n" _ *(wcstr + j));

	len = strlen(mbstr) + 5;
	(void) strcpy(format, "%1$+-");
	(void) strcat(format, itoa(len));
	(void) strcat(format, "S");
	PRINTIT(format _ wcstr);
#endif

	DBUG_VOID_RETURN;
}

public void
test43()
{
	DBUG_ENTER("test43");

	(void) cppair(t43, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
t43()
{
	char	*conv2str = "%9$ -8u %1$ -14lo %2$ -4c %3$ -14s %4$ -8d %5$ -8d %6$ -14f %7$ -14e %8$ -3g\\n";
	char	*conv3str = "%1$ -6hi %2$ -6hi\\n";
	char	*val2str  = "1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 12345";
	char	*val3str  = "1234, -1234";
	char	*exp2pat = "^12345    11145401322    a    test-string    -12345    12345    321\\.765432     3\\.217654e+0*02 * 5 \n";
	int	exp2len = sizeof("12345    11145401322    a    test-string    -12345    12345    321.765432     3.217654e+02   5 \n") - 1;
	char	*exp3str = " 1234  -1234 \n";
	int	pathok = setupok;
#if (TEST_XPG > 3)
	int	i, j, err, len;
	char	tbuf[50];
	char	*conv4str = "%1$ -20C";
	char	conv5str[20];
#endif

	DBUG_ENTER("t43");

	(void) cppair(ch_t43, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "( -n) format modifier causes left justification with positive signs replaced by space";

	globok = 0;
	checkpat(conv2str, val2str, exp2pat, exp2len);
	PATH_FUNC_TRACE;
	globok = 0;
	checkval(conv3str, val3str, exp3str);
	PATH_FUNC_TRACE;

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}

		tbuf[j] = '\0';
		(void) strcpy(wexpstr, tbuf);
		for(; j < 20; j++)
			wexpstr[j] = ' ';
		wexpstr[j++] = '\n';
		wexpstr[j] = '\0';
		globok = 0;
		checkval(conv4str, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	len = strlen(mbstr) + 5;
	(void) strcpy(conv5str, "% -");
	(void) strcat(conv5str, itoa(len));
	(void) strcat(conv5str, "S");
	(void) strcpy(wexpstr, mbstr);
	(void) strcat(wexpstr, "     ");
	globok = 0;
	checkval(conv5str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3 + i);
#else
	if (!testfail)
		PATH_XS_RPT(2);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t43()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if (TEST_XPG > 3)
	char	format[20];
	int	j, len;
#endif

	DBUG_ENTER("ch_t43");
			
	ch_setup();

	PRINTIT("%9$ -8u %1$ -14lo %2$ -4c %3$ -14s %4$ -8d %5$ -8d %6$ -14f %7$ -14e %8$ -3g\n" _ l _ c _ s _ -i _ i _ d _ d _ g _ u);

	PRINTIT("%1$ -6hi %2$ -6hi\n" _ h _ -h);

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%1$ -20C\n" _ *(wcstr + j));

	len = strlen(mbstr) + 5;
	(void) strcpy(format, "%1$ -");
	(void) strcat(format, itoa(len));
	(void) strcat(format, "S");
	PRINTIT(format _ wcstr);
#endif

	DBUG_VOID_RETURN;
}

public void
test44()
{
	DBUG_ENTER("test44");

	(void) cppair(t44, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
t44()
{
	char	*conv2str = "%9$+ u %1$+ lo %2$+ c %3$+ s %4$+ d %5$+ d %6$+ f %7$+ e %8$+ g\\n";
	char	*conv3str = "%1$+ hi %2$+ hi\\n";
	char	*val2str  = "1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 12345";
	char	*val3str  = "1234, -1234";
	char	*exp2pat = "^12345 11145401322 a test-string -12345 +12345 +321\\.765432 +3\\.217654e+0*02 +5\n";
	char	*exp3str = "+1234 -1234\n";
	int	pathok = setupok;
#if (TEST_XPG > 3)
	int	i, j, err;
	char	tbuf[50];
	char	*conv4str = "%1$+ C";
	char	*conv5str = "%1$+ S";
#endif

	DBUG_ENTER("t44");

	(void) cppair(ch_t44, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(+ ) format modifier acts the same as the (+) format modifier";

	globok = 0;
	checkpat(conv2str, val2str, exp2pat, 0);
	PATH_FUNC_TRACE;
	globok = 0;
	checkval(conv3str, val3str, exp3str);
	PATH_FUNC_TRACE;

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';
		(void) strcpy(wexpstr, tbuf);
		(void) strcat(wexpstr, "\n");
		globok = 0;
		checkval(conv4str, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	globok = 0;
	checkval(conv5str, pr_wstr(wcstr), mbstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3 + i);
#else
	if (!testfail)
		PATH_XS_RPT(2);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t44()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if (TEST_XPG > 3)
	int	j;
#endif

	DBUG_ENTER("ch_t44");
			
	ch_setup();

	PRINTIT("%9$+ u %1$+ lo %2$+ c %3$+ s %4$+ d %5$+ d %6$+ f %7$+ e %8$+ g\n" _ l _ c _ s _ -i _ i _ d _ d _ g _ u);

	PRINTIT("%1$+ hi %2$+ hi\n" _ h _ -h);

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%1$+ C\n" _ *(wcstr + j));

	PRINTIT("%1$+ S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}

public void
test45()
{
	DBUG_ENTER("test45");

	(void) cppair(t45, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
t45()
{
	char	*conv2str = "%9$- +u %1$- +lo %2$- +c %3$- +s %4$- +d %5$- +d %6$- +f %7$- +e %8$- +g\\n";
	char	*conv3str = "%1$- +hi %2$- +hi\\n";
	char	*val2str  = "1234567890, 'a', test-string, -12345, 12345, 321.7654321, 321.7654321, 5.0000001, 12345";
	char	*val3str  = "1234, -1234";
	char	*exp2pat  = "^12345 11145401322 a test-string -12345 +12345 +321\\.765432 +3\\.217654e+0*02 +5\n";
	char	*exp3str  = "+1234 -1234\n";
	int	pathok = setupok;
#if (TEST_XPG > 3)
	int	i, j, err;
	char	tbuf[50];
	char	*conv4str = "%1$- +C";
	char	*conv5str = "%1$- +S";
#endif

	DBUG_ENTER("t45");

	(void) cppair(ch_t45, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(- +) format modifier acts the same as the (-+) format modifier";

	globok = 0;
	checkpat(conv2str, val2str, exp2pat, 0);
	PATH_FUNC_TRACE;
	globok = 0;
	checkval(conv3str, val3str, exp3str);
	PATH_FUNC_TRACE;

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((j = wctomb(tbuf, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb() failed : errno %d (%s)", 	
				err, errname(err));
			DBUG_VOID_RETURN;
		}
			
		tbuf[j] = '\0';
		(void) strcpy(wexpstr, tbuf);
		(void) strcat(wexpstr, "\n");
		globok = 0;
		checkval(conv4str, pr_wchar(*(wcstr + i)), wexpstr);
		PATH_FUNC_TRACE;
	}

	globok = 0;
	checkval(conv5str, pr_wstr(wcstr), mbstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3 + i);
#else
	if (!testfail)
		PATH_XS_RPT(2);
#endif
	
	DBUG_VOID_RETURN;
}

private void
ch_t45()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
	int	u = 12345;
#else
	unsigned long	l = 1234567890;
	unsigned int	u = 12345;
#endif
	double	d = 321.7654321;
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#if (TEST_XPG > 3)
	int	j;
#endif

	DBUG_ENTER("ch_t45");
			
	ch_setup();

	PRINTIT("%9$- +u %1$- +lo %2$- +c %3$- +s %4$- +d %5$- +d %6$- +f %7$- +e %8$- +g\n" _ l _ c _ s _ -i _ i _ d _ d _ g _ u);

	PRINTIT("%1$- +hi %2$- +hi\n" _ h _ -h);

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);
	for(j = 0; *(wcstr + j) != (wchar_t)0; j++)
		PRINTIT("%1$- +C\n" _ *(wcstr + j));

	PRINTIT("%1$- +S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}

public void
test46()
{
	DBUG_ENTER("test46");

	(void) cppair(t46, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
t46()
{
	char	*conv2str = "%1$+8.*5$d %2$+6.*6$hi %3$+14.*7$lo %4$+14.*8$f\\n";
	char	*conv3str = "%1$+16.*4$e %2$+3.*5$g %3$+14.*6$s\\n";
	char	*val2str  = "12345, 1234, 1234567890, 321.7654321, 5, 2, 5, 3";
	char	*val3str  = "321.7654321, 5.0000001, test-string, 8, 4, 15";
	char	*exp2str  = "  +12345  +1234    11145401322       +321.765\n";
	char	*exp3pat  = "^ *+3\\.21765432e+0*02  +5    test-string\n";
	int	exp3len  = sizeof(" +3.21765432e+02  +5    test-string\n") - 1;
	int	pathok = setupok;
#if (TEST_XPG > 3)
	int	j, err, len, ret;
	char	tbuf[50];
	char	*conv5str = "%2$+20.*1$S";
#endif


	DBUG_ENTER("t46");

	(void) cppair(ch_t46, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(+n.*) format modifier combination converts as specified";

	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
	globok = 0;
	checkpat(conv3str, val3str, exp3pat, exp3len);
	PATH_FUNC_TRACE;

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	if((ret = wcstombs(tbuf, wcstr, 15)) == (size_t)-1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wcstombs() failed : errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tbuf[ret] = '\0';
	len = strlen(tbuf);

	for(j = 0; len < 20; j++, len++)
		wexpstr[j] = ' ';
	wexpstr[j] = '\0';
	(void) strcat(wexpstr, tbuf);

	globok = 0;
	checkval(conv5str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

#else
	PATH_TRACE;
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t46()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t46");
			
	ch_setup();

	PRINTIT("%1$+8.*5$d %2$+6.*6$hi %3$+14.*7$lo %4$+14.*8$f\n" _ i _ h _ l _ d _ 5 _ 2 _ 5 _ 3) ;

	PRINTIT("%1$+16.*4$e %2$+3.*5$g %3$+14.*6$s\n" _ d _ g _ s _ 8 _ 4 _ 15);

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	PRINTIT("%2$+20.*1$S" _ 15 _ wcstr);
#endif

	DBUG_VOID_RETURN;
}

public void
test47()
{
	DBUG_ENTER("test47");

	(void) cppair(t47, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
t47()
{
	char	*conv2str = "%6$-8.5d %7$-5.2hi %1$-14.5lo %2$-14.3f %3$-16.8e %4$-3.4g %5$-14.15s\\n";
	char	*val2str  = "1234567890, 321.7654321, 321.87654321, 5.0000001, test-string, 12345, 1234";
	char	*exp1pat = "^12345    1234  11145401322    321\\.765        3\\.21765432e+0*02 * 5   test-string   \n";
	int	exp1len = sizeof("12345    1234  11145401322    321.765        3.21765432e+02   5   test-string   \n") - 1;
	int	pathok = setupok;
#if (TEST_XPG > 3)
	int	j, err, len, ret;
	char	tbuf[50];
	char	*conv5str = "%1$-20.15S";
#endif

	DBUG_ENTER("t47");

	(void) cppair(ch_t47, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(-n.m) format modifier converts as specified";

	globok = 0;
	checkpat(conv2str, val2str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	if((ret = wcstombs(tbuf, wcstr, 15)) == (size_t)-1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wcstombs() failed : errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tbuf[ret] = '\0';
	len = strlen(tbuf);

	(void) strcpy(wexpstr, tbuf);
	for(j = len; j < 20; j++)
		wexpstr[j] = ' ';
	wexpstr[j] = '\0';

	globok = 0;
	checkval(conv5str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;

#else
	PATH_TRACE;
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_t47()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t47");
			
	ch_setup();

	PRINTIT("%6$-8.5d %7$-5.2hi %1$-14.5lo %2$-14.3f %3$-16.8e %4$-3.4g %5$-14.15s\n" _ l _ d _ d _ g _ s _ i _ h);

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	PRINTIT("%1$-20.15S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}

public void
test48()
{
	DBUG_ENTER("test48");

	(void) cppair(t48, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
t48()
{
	char	*conv2str = "%6$+-8.d %7$+-6.hi %1$+-14.lo %2$+-14.f %3$+-14.e %4$+-3.g %5$+-14.s\\n";
	char	*val2str  = "1234567890, 321.7654321, 321.87654321, 5.0000001, test-string, 12345, 1234";
	char	*exp1pat  = "^+12345   +1234  11145401322    +32[12]           +3e+0*02 * +5                \n";
	int	exp1len  = sizeof("+12345   +1234  11145401322    +321           +3e+02         +5                \n") - 1;
	int	pathok = setupok;
#if (TEST_XPG > 3)
	char	*conv4str = "%1$+-20.S";
#endif

	DBUG_ENTER("t48");

	(void) cppair(ch_t48, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(+-n.) format modifier converts as specified";

	globok = 0;
	checkpat(conv2str, val2str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	(void) strcpy(wexpstr, "                    ");
	globok = 0;
	checkval(conv4str, pr_wstr(wcstr), wexpstr);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t48()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;

	DBUG_ENTER("ch_t48");
			
	ch_setup();

	PRINTIT("%6$+-8.d %7$+-6.hi %1$+-14.lo %2$+-14.f %3$+-14.e %4$+-3.g %5$+-14.s\n" _ l _ d _ d _ g _ s _ i _ h);

#if (TEST_XPG > 3)
	newlocale(LC_CTYPE, locale);

	PRINTIT("%1$+-20.S" _ wcstr);
#endif

	DBUG_VOID_RETURN;
}


public void
test49()
{
	DBUG_ENTER("test49");

	(void) cppair(t49, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t49()
{
	char	*conv1str = "%4$#.lo %1$#.f %2$#.e %3$#.g\\n";
	char	*val1str  = "321.7654321, 321.87654321, 5.0000001, 1234567890";
	char	*exp1pat  = "^011145401322 32[12]\\. 3\\.e+0*02 5\\.\n";
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%3$#.d %1$#.hi %2$#.s\\n";
	char	*val2str  = "1234, test-string, 12345";
	char	*exp2str  = "12345 1234 \n";
#endif

	DBUG_ENTER("t49");

	(void) cppair(ch_t49, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(#.) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t49()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t49");
			
	ch_setup();

	PRINTIT("%4$#.lo %1$#.f %2$#.e %3$#.g\n" _ d _ d _ g _ l);

#if TEST_XPG == 3
	PRINTIT("%3$#.d %1$#.hi %2$#.s\n" _ h _ s _ i);
#endif

	DBUG_VOID_RETURN;
}


public void
test50()
{
	DBUG_ENTER("test50");

	(void) cppair(t50, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t50()
{
	char	*conv1str = "%1$#*5$.lo %2$#*6$.f %3$#*7$.e %4$#*8$.g\\n";
	char	*val1str  = "1234567890, 321.7654321, 321.87654321, 5.0000001, 14, 14, 14, 3";
	char	*exp1pat  = "^  011145401322           32[12]\\.  *3\\.e+0*02  5\\.\n";
	int	exp1len  = sizeof("  011145401322           321.         3.e+02  5.\n") - 1;
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%1$#*4$.d %2$#*5$.hi %3$#*6$.s\\n";
	char	*val2str  = "12345, 1234, test-string, 8, 5, 14";
	char	*exp2str  = "   12345  1234               \n";
#endif
	
	DBUG_ENTER("t50");

	(void) cppair(ch_t50, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(#*.) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t50()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t50");
			
	ch_setup();

	PRINTIT("%1$#*5$.lo %2$#*6$.f %3$#*7$.e %4$#*8$.g\n" _ l _ d _ d _ g _ 14 _ 14 _ 14 _ 3);

#if TEST_XPG == 3
	PRINTIT("%1$#*4$.d %2$#*5$.hi %3$#*6$.s\n" _ i _ h _ s _ 8 _ 5 _ 14);
#endif

	DBUG_VOID_RETURN;
}


public void
test51()
{
	DBUG_ENTER("test51");

	(void) cppair(t51, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t51()
{
	char	*conv1str = "%4$#.15lo %1$#.3f %2$#.4e %3$#.4g\\n";
	char	*val1str  = "321.7654321, 321.87654321, 5.0000001, 1234567890";
	char	*exp1pat  = "^000011145401322 321\\.765 3\\.217[67]e+0*02 5\\.000\n";
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%3$#.10d %1$#.5hi %2$#.7s\\n";
	char	*val2str  = "1234, test-string, 12345";
	char	*exp2str  = "0000012345 01234 test-st\n";
#endif

	DBUG_ENTER("t51");

	(void) cppair(ch_t51, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(#.m) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t51()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t51");
			
	ch_setup();

	PRINTIT("%4$#.15lo %1$#.3f %2$#.4e %3$#.4g\n" _ d _ d _ g _ l);

#if TEST_XPG == 3
	PRINTIT("%3$#.10d %1$#.5hi %2$#.7s\n" _ h _ s _ i);
#endif

	DBUG_VOID_RETURN;
}


public void
test52()
{
	DBUG_ENTER("test52");

	(void) cppair(t52, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t52()
{
	char	*conv1str = "%1$# +*5$lo %2$# +*6$f %3$# +*7$e %4$# +*8$g\\n";
	char	*val1str  = "1234567890, 321.7654321, 321.87654321, 5.0000001, 13, 13, 16, 10";
	char	*exp1pat  = "^ 011145401322   +321\\.765432  *+3\\.217654e+0*02   +5\\.00000\n";
	int	exp1len  = sizeof(" 011145401322   +321.765432    +3.217654e+02   +5.00000\n") - 1;
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%1$# +*5$d %2$# +*6$hi %3$# +*7$c %4$# +*8$s\\n";
	char	*val2str  = "12345, 1234, 'a', test-string, 8, 6, 2, 12";
	char	*exp2str  = "  +12345  +1234  a  test-string\n";
#endif


	DBUG_ENTER("t52");

	(void) cppair(ch_t52, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(# +*) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t52()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t52");
			
	ch_setup();

	PRINTIT("%1$# +*5$lo %2$# +*6$f %3$# +*7$e %4$# +*8$g\n" _ l _ d _ d _ g _ 13 _ 13 _ 16 _ 10);

#if TEST_XPG == 3
	PRINTIT("%1$# +*5$d %2$# +*6$hi %3$# +*7$c %4$# +*8$s\n" _ i _ h _ c _ s _ 8 _ 6 _ 2 _ 12);
#endif

	DBUG_VOID_RETURN;
}


public void
test53()
{
	DBUG_ENTER("test53");

	(void) cppair(t53, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t53()
{
	char	*conv1str = "%1$#+-.*5$lo %2$#+-.*6$f %3$#+-.*7$e %4$#+-.*8$g\\n";
	char	*val1str  = "1234567890, 321.7654321, 321.7654321, 5.0000001, 15, 3, 4, 4";
	char	*exp1pat  = "^000011145401322 +321\\.765 +3\\.217[67]e+0*02 +5\\.000\n";
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%1$#+-.*4$d %2$#+-.*5$hi %3$#+-.*6$s\\n";
	char	*val2str  = "12345, 1234, test-string, 10, 6, 7";
	char	*exp2str  = "+0000012345 +001234 test-st\n";
#endif

	DBUG_ENTER("t53");

	(void) cppair(ch_t53, NULLFN, WAITTIME, 0);
	
	testfail = 0;
	testcomment = "(#+-.*) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t53()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t53");
			
	ch_setup();

	PRINTIT("%1$#+-.*5$lo %2$#+-.*6$f %3$#+-.*7$e %4$#+-.*8$g\n" _ l _ d _ d _ g _ 15 _ 3 _ 4 _ 4);

#if TEST_XPG == 3
	PRINTIT("%1$#+-.*4$d %2$#+-.*5$hi %3$#+-.*6$s\n" _ i _ h _ s _ 10 _ 6 _ 7);
#endif

	DBUG_VOID_RETURN;
}


public void
test54()
{
	DBUG_ENTER("test54");

	(void) cppair(t54, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t54()
{
	char	*conv1str = "%1$# *3$.*5$lo %2$# *4$.*6$f\\n";
	char	*val1str  = "1234567890, 321.7654321, 14, 7, 5, 3";
	char	*exp1pat  = "^  011145401322  321\\.765\n";
	int	exp1len  = sizeof("  011145401322  321.765\n") - 1;
	char	*conv2str = "%1$# *3$.*5$e %2$# *4$.*6$g\\n";
	char	*val2str  = "321.7654321, 5.0000001, 16, 4, 8, 2";
	char	*exp2pat  = "^ *3\\.21765432e+0*02  5\\.0\n";
	int	exp2len  = sizeof("  3.21765432e+02  5.0\n") - 1;
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv3str = "%1$# *3$.*5$d %2$# *4$.*6$hi\\n";
	char	*val3str  = "12345, 1234, 8, 5, 5, 2";
	char	*exp3str  = "   12345  1234\n";
	char	*conv4str = "%1$# *2$.*3$s\\n";
	char	*val4str  = "test-string, 14, 15";
	char	*exp4str  = "   test-string\n";
#endif

	DBUG_ENTER("t54");

	(void) cppair(ch_t54, NULLFN, WAITTIME, 0);


	testfail = 0;
	testcomment = "(# *.*) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

	globok = 0;
	checkpat(conv2str, val2str, exp2pat, exp2len);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv3str, val3str, exp3str);
	PATH_FUNC_TRACE;

	globok = 0;
	checkval(conv4str, val4str, exp4str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t54()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t54");
			
	ch_setup();

	PRINTIT("%1$# *3$.*5$lo %2$# *4$.*6$f\n" _ l _ d _ 14 _ 7 _ 5 _ 3);
	PRINTIT("%1$# *3$.*5$e %2$# *4$.*6$g\n" _ d _ g _ 16 _ 4 _ 8 _ 2);

#if TEST_XPG == 3
	PRINTIT("%1$# *3$.*5$d %2$# *4$.*6$hi\n" _ i _ h _ 8 _ 5 _ 5 _ 2);
	PRINTIT("%1$# *2$.*3$s\n" _ s _ 14 _ 15);
#endif

	DBUG_VOID_RETURN;

}


public void
test55()
{
	DBUG_ENTER("test55");

	(void) cppair(t55, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t55()
{
	char	*conv1str = "%4$#- +14lo %1$#- +14f %2$#- +16e %3$#- +3g\\n";
	char	*val1str  = "321.7654321, 321.87654321, 5.0000001, 1234567890";
	char	*exp1pat = "^011145401322   +321\\.765432    +3\\.217654e+0*02 * +5\\.00000\n";
	int	exp1len = sizeof("011145401322   +321.765432    +3.217654e+02    +5.00000\n") - 1;
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%4$#- +8d %1$#- +6hi %2$#- +4c %3$#- +14s\\n";
	char	*val2str  = "1234, 'a', test-string, 12345";
	char	*exp2str = "+12345   +1234  a    test-string   \n";
#endif


	DBUG_ENTER("t55");

	(void) cppair(ch_t55, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(#- +n) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t55()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	c = 'a';
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t55");
			
	ch_setup();

	PRINTIT("%4$#- +14lo %1$#- +14f %2$#- +16e %3$#- +3g\n" _ d _ d _ g _ l);

#if TEST_XPG == 3
	PRINTIT("%4$#- +8d %1$#- +6hi %2$#- +4c %3$#- +14s\n" _ h _ c _ s _ i);
#endif

	DBUG_VOID_RETURN;
}


public void
test56()
{
	DBUG_ENTER("test56");

	(void) cppair(t56, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
t56()
{
	char	*conv1str = "%1$#-*5$.5lo %2$#-*6$.3f %3$#-*7$.8e %4$#-*8$.4g\\n";
	char	*val1str  = "1234567890, 321.7654321, 321.7654321, 5.0000001, 14, 14, 16, 3";
	char	*exp1pat  = "^011145401322   321\\.765        3\\.21765432e+0*02 * 5\\.000\n";
	int	exp1len  = sizeof("011145401322   321.765        3.21765432e+02   5.000\n") - 1;
	int	pathok = setupok;
#if TEST_XPG == 3
	char	*conv2str = "%1$#-*4$.5d %2$#-*5$.2hi %3$#-*6$.10s\\n";
	char	*val2str  = "12345, 1234, test-string, 8, 5, 12";
	char	*exp2str  = "12345    1234  test-strin  \n";
#endif


	DBUG_ENTER("t56");

	(void) cppair(ch_t56, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(#-*.m) format modifier converts as specified";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, exp1len);
	PATH_FUNC_TRACE;

#if TEST_XPG == 3
	globok = 0;
	checkval(conv2str, val2str, exp2str);
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t56()
{
	double	g = 5.0000001;
#if TEST_XPG == 3
	long	l = 1234567890;
#else
	unsigned long	l = 1234567890;
#endif
	double	d = 321.7654321;
#if TEST_XPG == 3
	char	*s = "test-string";
	int	i = 12345;
	int	h = 1234;
#endif

	DBUG_ENTER("ch_t56");
			
	ch_setup();

	PRINTIT("%1$#-*5$.5lo %2$#-*6$.3f %3$#-*7$.8e %4$#-*8$.4g\n" _ l _ d _ d _ g _ 14 _ 14 _ 16 _ 3);

#if TEST_XPG == 3
	PRINTIT("%1$#-*4$.5d %2$#-*5$.2hi %3$#-*6$.10s\n" _ i _ h _ s _ 8 _ 5 _ 12);
#endif
	DBUG_VOID_RETURN;
}


public void
test57()
{
#if	LONG_DOUBLE_SUPPORTED

	char	*conv1str = "%5$Lf %1$Le %2$Lg %3$LE %4$LG\\n";
	char	*val1str  = "-0.0123, 0.0001555, -0.0123, 0.0001555, -12.3";
	char	*exp1pat  = "-12.300000 -1\\.230000e-0*02 0.0001555 -1\\.230000E-0*02 0.0001555\n";
	char	*conv2str = "%5$Lf %1$Le %2$Lg %3$LE %4$LG\\n";
	char	*val2str  = "321.765432, 12345.1234546, 321.765432, 12345678.1234546, 321.765432";
	char	*exp2pat  = "321.765432 3\\.217654e+0*02 12345.1 3\\.217654E+0*02 1\\.2345[67]E+0*07\n";
	int	pathok = setupok;

	DBUG_ENTER("test57");

	(void) cppair(ch_t57, NULLFN, WAITTIME, 0);

	testfail = 0;
	testcomment = "(L) format modifier specifies argument is long double";

	globok = 0;
	checkpat(conv1str, val1str, exp1pat, 0);
	PATH_FUNC_TRACE;
	globok = 0;
	checkpat(conv2str, val2str, exp2pat, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;

#else /* LONG_DOUBLE_SUPPORTED */

	DBUG_ENTER("test57");

	xx_rpt(UNSUPPORTED);
	in_rpt("Long double is not supported");

	DBUG_VOID_RETURN;

#endif /* LONG_DOUBLE_SUPPORTED */

}

#if	LONG_DOUBLE_SUPPORTED
private void
ch_t57()
{
	long double	d1 = -12.3;
	long double	d2 = -0.0123;
	long double	d3 =  0.0001555;
	long double	d4 = 321.765432;
	long double	d5 = 12345.1234546;
	long double	d6 = 12345678.1234546;

	DBUG_ENTER("test57");

	ch_setup();

	PRINTIT("%5$Lf %1$Le %2$Lg %3$LE %4$LG\n" _ d2 _ d3 _ d2 _ d3 _ d1);
	PRINTIT("%5$Lf %1$Le %2$Lg %3$LE %4$LG\n" _ d4 _ d5 _ d4 _ d6 _ d4);

	DBUG_VOID_RETURN;
}
#endif	/* LONG_DOUBLE_SUPPORTED */

#endif /* TEST_XNFS */
