/*
*      SCCS:  @(#)  ANSI.os/string/memcmp_X/memcmp_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/string/memcmp_X - T.memcmp_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.memcmp_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the memcmp() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		David McCardle, UniSoft Ltd.
DATE CREATED:	Thu Oct 23 11:42:47 PDT 1986
MODIFICATIONS:	Andrew Twigger 09 JUne 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		David Sawyer 03 July 1990
		Added Resillience code

		Stuart Boutell, 3 May 1991
			Improved testing by using '\0' character in data.

		Geoff Clare, 16 Feb 1995
			Remove contiguous-alphabet dependencies.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<string.h>
#include	<sys/types.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef memcmp
#endif


#define NO_TESTS	3		/* number of tests in testset */

int	no_tests = NO_TESTS;

private char	s1[20];
private char	s2[20];

private void	preparetests();
private void	test1();
private void	test2();
private void	test3();

public	void	(*setfunc) () = preparetests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

extern char *pr_nchars();

private void
preparetests()
{
	int	i;
	char	alpha;

	DBUG_ENTER("preparetests");
	for (i = 0, alpha = 'a'; i < 17; i++, alpha++)
		s1[i] = s2[i] = alpha;
	s1[12]--;
	for (i = 17; i < 19; i++)
		s1[i] = s2[i] = '\0';

	s1[5] = s2[5] = '\0';
	DBUG_PRINTF("preparetests", "s1 :%s:\ts2 :%s:", pr_nchars(s1,20),
							pr_nchars(s2,20));
	DBUG_VOID_RETURN;
}


private void
test1()
{
	int	rval, num = 15;
	int	pathok = 0;

	DBUG_ENTER("test1");
	DBUG_PRINTF("test1", "s1 :%s:\ts2 :%s:", pr_nchars(s1,15),
						 pr_nchars(s2,15));

	if ((rval = memcmp((void *)s1, (void *)s2, (size_t)num)) >= 0)
	{
		xx_rpt(FAILURE);
		in_rpt("memcmp did not return expected value");
		in_rpt("value of first  argument : %s", pr_nchars(s1,15));
		in_rpt("value of second argument : %s", pr_nchars(s2,15));
		in_rpt("value of third  argument : %d", num);
		if (rval >= 0)
		{
			in_rpt("expected return value    : < 0");
			in_rpt("actual   return value    : %d", rval);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}


private void
test2()
{
	int	rval, num = 10;
	int pathok = 0;

	DBUG_ENTER("test2");
	DBUG_PRINTF("test2", "s1 :%s:\ts2 :%s:", pr_nchars(s1,10),
						 pr_nchars(s2,10));

	if ((rval = memcmp((void *)s1, (void *)s2, (size_t)num)) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("memcmp did not return expected value");
		in_rpt("value of first  argument : %s", pr_nchars(s1,10));
		in_rpt("value of second argument : %s", pr_nchars(s2,10));
		in_rpt("value of third  argument : %d", num);
		if (rval != 0)
		{
			in_rpt("expected return value    : 0");
			in_rpt("actual   return value    : %d", rval);
		}
	}
	else {
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}


private void
test3()
{
	int	rval, num = 15;
	int	pathok = 0;

	DBUG_ENTER("test3");
	DBUG_PRINTF("test3", "s1 :%s:\ts2 :%s:", pr_nchars(s1,15),
						 pr_nchars(s2,15));

	if ((rval = memcmp((void *)s2, (void *)s1, (size_t)num)) <= 0)
	{
		xx_rpt(FAILURE);
		in_rpt("memcmp did not return expected value");
		in_rpt("value of first  argument : %s", pr_nchars(s2,15));
		in_rpt("value of second argument : %s", pr_nchars(s1,15));
		in_rpt("value of third  argument : %d", num);
		if (rval <= 0)
		{
			in_rpt("expected return value    : > 0");
			in_rpt("actual   return value    : %d", rval);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}
