/*
*      SCCS:  @(#)  ANSI.os/string/strcat/strcat.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/string/strcat - T.strcat Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strcat

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the strcat() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		David McCardle, UniSoft Ltd.
DATE CREATED:	Mon Oct 20 13:56:57 PDT 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		David Sawyer
		Tue Jul 03 13:21:26 BST 1990
		Added Resillience code

		Neil Moses, 10 Dec 1991
		Changed %ld to 0x%lx, and cast pointer arg to long

		Geoff Clare, 22 Nov 1994
		Added declaration of checkval() and changed to return void

		Geoff Clare, 22 Aug 1995
			Removed ASCII dependencies.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<string.h>
#include	<limits.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strcat
#endif


#define NO_TESTS	1		/* number of tests in testset */

extern	char	*psl_to_asc();

int	no_tests = NO_TESTS;

private char	s1[50];
private char	s2[20];
private int	testfail;

private void	preparetests();
private void	test1();
private void	checkval();

public	void	(*setfunc) () = preparetests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private void
preparetests()
{
	int	i;

	DBUG_ENTER("preparetests");

	for (i = 0; i < 12; i++)
	{
		s1[i] = (char)(SCHAR_MAX - 12 + i);
		s2[i] = (char)(SCHAR_MIN + i);
	}

	DBUG_VOID_RETURN;
}


private void
test1()
{
	int	i, j;
	char	*rval;
	char	expstr[27];
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	for (i = 0; s1[i] != 0; i++)
		expstr[i] = s1[i];
	for (j = 0; s2[j] != 0; i++, j++)
		expstr[i] = s2[j];
	expstr[i] = '\0';

	rval = strcat(s1, s2);

	globok = 0;
	checkval(s1, s2, -1, rval, expstr, i);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);
	
	DBUG_VOID_RETURN;
}

private void
checkval(str1, str2, num, rval, expstr, len)
char *str1, *str2;
int  num;
char *rval, *expstr;
int  len;
{
	int	i;
	int	failed = 0;
	int	pathok = 0;

	DBUG_ENTER("checkval");

	globok = 0;
	
	if (str1 != rval)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("strcat did not return expected value");
		in_rpt("pointer & value of first  argument : 0x%lx\t%s",
			(long) str1, str1);
		in_rpt("pointer & value of second argument : 0x%lx\t%s",
			(long) str2, str2);
		if (num >= 0)
			in_rpt("number of characters       : %d", num);
		in_rpt("expected return value      : 0x%lx", (long) str1);
		in_rpt("actual   return value      : 0x%lx", (long) rval);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	for (i=0; i < (len+1); i++)
	{
		if (i==0)
			PATH_TRACE;
		if (str1[i] != expstr[i])
			failed++;
	}

	if (failed)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("strcat did did not copy data correctly");
		in_rpt("expected result : %s", psl_to_asc(expstr, 0));
		in_rpt("actual   result : %s", psl_to_asc(str1, 0));
	}
	else 
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}
