/*
*      SCCS:  @(#)  ANSI.os/string/strchr/strchr.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/string/strchr - T.strchr Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strchr

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the strchr() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		David McCardle, UniSoft Ltd.
DATE CREATED:	Tue Oct 21 11:28:32 PDT 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		David Sawyer
		Tue Jul 03 15:02:09 BST 1990
		Added Resillience code

		Neil Moses, 10 Dec 1991
		Changed %ld to 0x%lx, and cast pointer arg to long

		Geoff Clare, 22 Aug 1995
			Removed ASCII dependencies.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<string.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strchr
#endif


#define NO_TESTS	3		/* number of tests in testset */

extern	char	*psl_to_asc();

int	no_tests = NO_TESTS;

private char	s[20];
private int	testfail;

private void	preparetests();
private void	test1();
private void	test2();
private void	test3();
private void	checkval();

public	void	(*setfunc) () = preparetests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private void
preparetests()
{
	unsigned char *p;
	int	i;

	DBUG_ENTER("preparetests");

	p = (unsigned char *)s;

	for (i = 0; i < 19; i++)
	{
		if (i == 3 || i == 8 || i == 15)
			p[i] = ('a'&0177)|0200;
		else if ( i == 7 || i == 12 || i == 18)
			p[i] = ('x'&0177);
		else
			p[i] = '0';
	}
	p[19] = NUL;

	DBUG_VOID_RETURN;
}


private void
test1()
{
	char	*rval;
	int	tval;
	int 	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;
	tval = ('a'&0177)|0200;
	rval = strchr(s, tval);
	globok = 0;
	checkval(s, tval, rval, &s[3]);
	PATH_FUNC_TRACE;

	tval = ('x'&0177);
	rval = strchr(s, tval);
	checkval(s, tval, rval, &s[7]);
	if (globok)
		PATH_TRACE;
	
	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	char	*rval;
	int	tval;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;
	tval = ('x'&0177)|0200;
	rval = strchr(s, tval);
	globok = 0;
	checkval(s, tval, rval, (char *) NULL);
	PATH_FUNC_TRACE;

	tval = ('a'&0177);
	rval = strchr(s, tval);
	globok = 0;
	checkval(s, tval, rval, (char *) NULL);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test3()
{
	char	*rval;
	int	tval;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;
	tval = '\0';
	rval = strchr(s, tval);
	globok = 0;
	checkval(s, tval, rval, &s[19]);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
checkval(str1, chr1, rval, expret)
char *str1;
int chr1;
char *rval, *expret;
{
	unsigned char	str2[2];
	int 	pathok = 0;

	DBUG_ENTER("checkval");

	globok = 0;

	str2[0] = chr1;
	str2[1] = '\0';
	if (rval != expret)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("strchr did not return expected value");
		in_rpt("pointer & value of first  argument : 0x%lx\t%s", 
			(long) str1, psl_to_asc(str1, 0));
		in_rpt("value   of second argument : %s", psl_to_asc((char *)str2, 0));
		in_rpt("expected return value      : 0x%lx", (long) expret);
		in_rpt("actual   return value      : 0x%lx", (long) rval);
	}
	else 
	{
		PATH_TRACE;
		globok = 1;
	}

	DBUG_VOID_RETURN;
}
