/*
*      SCCS:  @(#)  ANSI.os/string/strcoll_X/strcoll_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/string/strcoll_X - T.strcoll_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strcoll_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the strcoll() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		David McCardle, UniSoft Ltd.
DATE CREATED:	Tue Oct 21 13:54:03 PDT 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		Linda Taylor, Hoskyns.	August, 1988. 

		David Sawyer
		Tue Jul 03 17:39:53 BST 1990
		Added resillience code

		Geoff Clare, 29 May 1992
			Don't use literal language names.

		Geoff Clare, 11 Aug 1995
			Remove test of behaviour with VSX4L0.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<nlstest.h>
#include	<string.h>
#include	<locale.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strcoll
#endif


#define NO_TESTS	2		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)

extern	char	*hexstr();
extern	char	*asc_to_psl();
extern	char	*psl_to_asc();

int	no_tests = NO_TESTS;

private int	testfail;
private char	*testcomment;
private char	*currlang;
private	int	nllang = 0;

private void	test1();
private void	test2();
private void	ch_t1();
private void	ch_t2();
private	void	checkval();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	retval;
	char	buf1[50], buf2[50];
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	testfail = 0;
 	if (setlocale(LC_COLLATE, INFO1LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Cannot set locale for LC_COLLATE to %s", INFO1LANG);
		in_rpt("Check installation of locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO1LANG;
	nllang = VALINFO1LANG;

	testcomment = "1-2 character mappings are undertaken";
	(void) strcpy(buf1, "hasse");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strcpy(buf2, "ha&e");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, 0, retval);
	PATH_FUNC_TRACE;

	testcomment = "accented comparisons are undertaken";
	(void) strcpy(buf1, "ta\"che");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strcpy(buf2, "ta^che");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	testcomment = "primary order overrides accented comparison";
	(void) strcpy(buf1, "ta^che");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strcpy(buf2, "ta\"chet");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	testcomment = "accented comparison against non-accented word";
	(void) strcpy(buf1, "te`nse");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strcpy(buf2, "tense");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

	(void) strcpy(buf1, "arc~ed");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strcpy(buf2, "arced");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

	testcomment = "2-1 mappings are undertaken";
	(void) strcpy(buf1, "cache");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strcpy(buf2, "cacet");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

	testcomment = "1-1 mappings are undertaken";
	(void) strcpy(buf1, "barn");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strcpy(buf2, "Barn");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	(void) strcpy(buf1, "Barn");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strcpy(buf2, "bern");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	testcomment = "dont care characters are ignored";
	(void) strcpy(buf1, "barn");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strcpy(buf2, "b-arn");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, 0, retval);
	PATH_FUNC_TRACE;

	if( setlocale(LC_COLLATE, INFO2LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Cannot set locale for LC_COLLATE to %s", INFO2LANG);
		in_rpt("Check installation of locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO2LANG;
	nllang = VALINFO2LANG;

	testcomment = "the collating sequence for the current language is used";
	(void) strcpy(buf1, "Barn");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strcpy(buf2, "clock");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

 	if (setlocale(LC_ALL, INFO2LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Cannot set locale for LC_ALL to %s", INFO2LANG);
		in_rpt("Check installation of locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO2LANG;
	nllang = VALINFO2LANG;

	testcomment = "1-2 character mappings are undertaken";
	(void) strcpy(buf1, "fa&e");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strcpy(buf2, "fasse");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, 0, retval);
	PATH_FUNC_TRACE;

	testcomment = "accented comparisons are undertaken";
	(void) strcpy(buf1, "tre^at");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strcpy(buf2, "tre'at");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

	testcomment = "primary order overrides accented comparison";
	(void) strcpy(buf1, "Fre^d");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strcpy(buf2, "Fre'ddie");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	testcomment = "The collating sequence uses a primary order";
	(void) strcpy(buf1, "Advent");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strcpy(buf2, "zone");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

	testcomment = "accented comparison against non-accented word";
	(void) strcpy(buf1, "pa\"ssed");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strcpy(buf2, "passed");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	(void) strcpy(buf1, "ge^ts");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strcpy(buf2, "gets");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

	testcomment = "2-1 mappings are undertaken";
	(void) strcpy(buf1, "Cused");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strcpy(buf2, "Chase");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	testcomment = "dont care characters are ignored";
	(void) strcpy(buf1, "b-arn");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strcpy(buf2, "barn");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, 0, retval);
	PATH_FUNC_TRACE;

	if (setlocale(LC_ALL, INFO1LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Cannot set locale for LC_ALL to %s", INFO1LANG);
		in_rpt("Check installation of locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO1LANG;
	nllang = VALINFO1LANG;

	testcomment = "the collating sequence for the current language is used";
	(void) strcpy(buf1, "Hot");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strcpy(buf2, "hotter");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(23);

	DBUG_VOID_RETURN;
} 

private	void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);	

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	retval;
	char	buf1[50], buf2[50];
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	currlang = "UNDEFINED";
	nllang = 0;

	testfail = 0;
	testcomment = "strcoll acts like strcmp when setlocale has not successfully been called";
	(void) strcpy(buf1, "cache");
	(void) strcpy(buf2, "caclet");
	retval = strcoll(buf1, buf2);
	globok = 0;
	checkval("strcoll", buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
checkval(name, str1, str2, expval, retval)
char *name, *str1, *str2;
int  expval, retval;
{
	int	failed = 0;
	char	*expstr;
	int	pathok = 0;

	DBUG_ENTER("checkval");

	globok = 0;

	if (expval < 0)
	{
		expstr = "< 0";
		if (retval >= 0)
			failed++;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (expval > 0)
	{
		expstr = "> 0";
		if (retval <= 0)
			failed++;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (expval == 0)
	{
		expstr= "0";
		if (retval != 0)
			failed++;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (failed)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt(testcomment);
		in_rpt("Current language setting is %s", currlang);
		in_rpt("%s did not return expected value", name);
		in_rpt("value of first  argument : %s (%s)", hexstr((unsigned char *)(str1)), psl_to_asc(str1,nllang));
		in_rpt("value of second argument : %s (%s)", hexstr((unsigned char *)(str2)), psl_to_asc(str2,nllang));
		in_rpt("expected return value    : %s", expstr);
		in_rpt("actual   return value    : %d", retval);
		testfail++;
	}
	else
	{
		PATH_FUNC_RPT(3);
	}

	DBUG_VOID_RETURN;
}
