/*
*      SCCS:  @(#)  ANSI.os/misc/strerror_X/strerror_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/misc/strerror_X - T.strerror_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strerror_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the strerror() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Bob Campbell, Hoskyns
DATE CREATED:	Wed Aug 10, 1988
MODIFICATIONS:
		Geoff Clare, 20 Nov 1989
			Improved thoroughness

		David Sawyer
		Wed Jul 04 17:13:28 BST 1990
		Added resillience code

		Geoff Clare, 29 May 1992
			Don't use literal language names.

		Geoff Clare, 16 June 1993
			Remove ENOTBLK from errno list in XPG4 mode.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<string.h>
#include	<errno.h>
#include	<locale.h>
#include	<dbug.h>
#include	<report.h>
#include	<nlstest.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strerror
#endif


#define 	NO_TESTS	1	/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*newstr();

int	no_tests = NO_TESTS;

private struct {
	int num;
	char *msgp;
} errtab[] = {
	NOERROR,	NULL,
	E2BIG,		NULL,
	EACCES,		NULL,
	EAGAIN,		NULL,
	EBADF,		NULL,
	EBUSY,		NULL,
	ECHILD,		NULL,
	EDEADLK,	NULL,
	EDOM,		NULL,
	EEXIST,		NULL,
	EFAULT,		NULL,
	EFBIG,		NULL,
	EINTR,		NULL,
	EINVAL,		NULL,
	EIO,		NULL,
	EISDIR,		NULL,
	EMFILE,		NULL,
	EMLINK,		NULL,
	ENAMETOOLONG,	NULL,
	ENFILE,		NULL,
	ENODEV,		NULL,
	ENOENT,		NULL,
	ENOEXEC,	NULL,
	ENOLCK,		NULL,
	ENOMEM,		NULL,
	ENOSPC,		NULL,
	ENOSYS,		NULL,
	ENOTDIR,	NULL,
	ENOTEMPTY,	NULL,
	ENOTTY,		NULL,
	ENXIO,		NULL,
	EPERM,		NULL,
	EPIPE,		NULL,
	ERANGE,		NULL,
	EROFS,		NULL,
	ESPIPE,		NULL,
	ESRCH,		NULL,
	EXDEV,		NULL,
#ifdef _XOPEN_SOURCE
	EIDRM,		NULL,
	ENOMSG,		NULL,
#if TEST_XPG == 3
	ENOTBLK,	NULL,
#endif
	ETXTBSY,	NULL,
#endif
};
private int	numerr = sizeof(errtab)/sizeof(errtab[0]);
private char	*currlang = "";
private int	testfail;

private void	test1();
private void    ch_t1();
private void    chkuniq();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private void
test1()
{

	DBUG_ENTER("test1");

	(void)   cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int 	pathok = 0;

	DBUG_ENTER("ch_t1");

	testfail = 0;

	currlang = "C locale";

	globok = 0;
	chkuniq();
	PATH_FUNC_TRACE;

	if (setlocale(LC_ALL, INFO1LANG) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setlocale(LC_ALL, %s) failed", INFO1LANG);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO1LANG;
	globok = 0;
	chkuniq();
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
chkuniq()
{
	char 	*ret;
	int 	i, j, lastfail;
	int	lptot;
	int	pathok = 0;

	DBUG_ENTER("chkuniq");

	globok = 0;
	lastfail = testfail;

#ifdef TESTBED
	errtab[2].num = errtab[3].num;
#endif

	for (i = 0; i < numerr; i++)
	{
		if ((ret = strerror(errtab[i].num)) == NULL || *ret == '\0')
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("strerror(%d) returned %s", errtab[i].num,
				(ret == NULL) ? "NULL" : "empty string");
			continue;
		}
		else
			PATH_TRACE;

		if (errtab[i].msgp != NULL)
			free((void *)errtab[i].msgp);

		if ((errtab[i].msgp = newstr(ret)) == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: malloc() failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		for (j = 0; j < i; j++)
		{
			if (strcmp(errtab[i].msgp, errtab[j].msgp) == 0)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("strerror(%d) returned same string as strerror(%d)",
					errtab[i].num, errtab[j].num);
				break;
			}
			else
				PATH_TRACE;
		}
	}

	if (testfail > lastfail)
		in_rpt("language setting for above tests was %s", currlang);
	else
	{
		lptot = numerr * 2;	/* # of traces in i loop */
		for (j = 0; j < numerr; j++)
			lptot += j;	/* (j * 1) == # of traces in j loop */
		PATH_FUNC_RPT(lptot);
	}

	DBUG_VOID_RETURN;
}
