/*
*      SCCS:  @(#)  ANSI.os/string/strncmp/strncmp.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/string/strncmp - T.strncmp Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strncmp

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the strncmp() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		David McCardle, UniSoft Ltd.
DATE CREATED:	Tue Oct 21 13:54:03 PDT 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		David Sawyer
		Wed Jul 04 13:36:00 BST 1990
		Added resillience code

		Geoff Clare, 23 Aug 1995
			Removed ASCII dependencies.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<string.h>
#include	<limits.h>
#include	<sys/types.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strncmp
#endif


#define NO_TESTS	3		/* number of tests in testset */

extern	char	*psl_to_asc();

int	no_tests = NO_TESTS;

private char	s1[20];
private char	s2[20];
private char	s3[20];
private int	testfail;

private void	preparetests();
private void	test1();
private void	test2();
private void	test3();
private	void	checkval();

public	void	(*setfunc) () = preparetests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private void
preparetests()
{
	int	i;

	DBUG_ENTER("preparetests");

	for (i = 0; i < 7; i++)
		s1[i] = s2[i] = s3[i] = (char)(SCHAR_MIN + i);
	for ( ; i < 14; i++)
		s1[i] = s2[i] = s3[i] = (char)(SCHAR_MAX - 14 + i);

	s2[13] = (char) -1; /* use -1 instead of UCHAR_MAX since
			       (unsigned char) -1 is OK, but
			       (signed char) UCHAR_MAX is undefined */

	for ( ; i < 17; i++)
	{
		s1[i] = '9';
		s2[i] = '0';
		s3[i] = '9';
	}
	s1[i] = s2[i] = s3[i] = NUL;

	DBUG_VOID_RETURN;
}


private void
test1()
{
	int	rval, num = 14;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;
	rval = strncmp(s1, s2, (size_t)num);

	globok = 0;
	checkval(s1, s2, num, -1, rval);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	int	rval, num = 13, num2 = 25;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;
	rval = strncmp(s1, s2, (size_t)num);

	globok = 0;
	checkval(s1, s2, num, 0, rval);
	PATH_FUNC_TRACE;

	rval = strncmp(s1, s3, (size_t)num2);

	globok = 0;
	checkval(s1, s3, num2, 0, rval);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test3()
{
	int	rval, num = 14;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;
	rval = strncmp(s2, s1, (size_t)num);

	globok = 0;
	checkval(s2, s1, num, 1, rval);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
checkval(str1, str2, num, expval, retval)
char *str1, *str2;
int  num, expval, retval;
{
	int	failed = 0;
	char	*expstr;
	int	pathok = 0;

	DBUG_ENTER("checkval");

	globok = 0;

	if (expval < 0)
	{
		expstr = "< 0";
		if (retval >= 0)
			failed++;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (expval > 0)
	{
		expstr = "> 0";
		if (retval <= 0)
			failed++;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (expval == 0)
	{
		expstr= "0";
		if (retval != 0)
			failed++;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (failed)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("strncmp did not return expected value");
		in_rpt("value of first  argument : %s", psl_to_asc(str1, 0));
		in_rpt("value of second argument : %s", psl_to_asc(str2, 0));
		if (num >= 0)
			in_rpt("value of third argument : %d", num);
		in_rpt("expected return value    : %s", expstr);
		in_rpt("actual   return value    : %d", retval);
	}
	else
		PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}
