/*
*      SCCS:  @(#)  ANSI.os/string/strncpy/strncpy.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/string/strncpy - T.strncpy Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strncpy

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the strncpy() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		David McCardle, UniSoft Ltd.
DATE CREATED:	Tue Oct 21 16:22:28 PDT 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		David Sawyer
		Wed Jul 04 15:31:59 BST 1990
		Added Resillience

		Neil Moses, 10 Dec 1991
		Changed %ld to 0x%lx, and cast pointer arg to long

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<string.h>
#include	<sys/types.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strncpy
#endif


#define NO_TESTS	2		/* number of tests in testset */
#define SPACE		' '		/* space character */

extern	char	*psl_to_asc();

int	no_tests = NO_TESTS;

private char	s1[50];
private char	s2[30];
private int	testfail;

private void	preparetests();
private void	test1();
private void	test2();
private void	checkval();

public	void	(*setfunc) () = preparetests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private void
preparetests()
{
	int	i;
	char	alpha;

	DBUG_ENTER("preparetests");
	for (i = 0; i < 49; i++)
		s1[i] = SPACE;
	s1[49] = NUL;
	for (i = 0, alpha = 0200; i < 26; i++, alpha++)
		s2[i] = alpha;
	for (i = 26; i < 30; i++)
		s2[i] = NUL;
	DBUG_VOID_RETURN;
}


private void
test1()
{
	int	i;
	char	*rval;
	int	num = 15;
	char	expstr[50];
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;
	(void) preparetests();

	for (i=0; i<50; i++)
	{
		if (i==0)
			PATH_TRACE;
		expstr[i] = s1[i];
	}
	for (i=0; i<num; i++)
	{
		if (i==0)
			PATH_TRACE;
		expstr[i] = s2[i];
	}

	rval = strncpy(s1, s2, (size_t)num);

	globok = 0;
	checkval(s1, s2, num, rval, expstr, 50);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	int	i;
	char	*rval;
	int	num = 50;
	char	expstr[50];
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;
	(void) preparetests();

	for (i=0; i<50; i++)
	{
		if (i==0)
			PATH_TRACE;
		expstr[i] = s2[i];
		if (s2[i] == '\0')
			break;
	}
	for ( ; i<50; i++)
	{
		if (pathok==1)
			PATH_TRACE;
		expstr[i] = '\0';
	}

	rval = strncpy(s1, s2, (size_t)num);

	globok = 0;
	checkval(s1, s2, num, rval, expstr, 50);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
checkval(str1, str2, num, rval, expstr, len)
char *str1, *str2;
int  num;
char *rval, *expstr;
int  len;
{
	int	i;
	int	failed = 0;
	int	pathok = 0;

	DBUG_ENTER("checkval");

	globok = 0;
	
	if (str1 != rval)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("strncpy did not return expected value");
		in_rpt("pointer & value of first  argument : 0x%lx\t%s",
			(long) str1, psl_to_asc(str1, 0));
		in_rpt("pointer & value of second argument : 0x%lx\t%s",
			(long) str2, psl_to_asc(str2, 0));
		if (num >= 0)
			in_rpt("number of characters       : %d", num);
		in_rpt("expected return value      : 0x%lx", (long) str1);
		in_rpt("actual   return value      : 0x%lx", (long) rval);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i < len; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (str1[i] != expstr[i])
			failed++;
	}

	if (failed)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("strncpy did did not copy data correctly");
		in_rpt("expected result : %s", psl_to_asc(expstr, 0));
		in_rpt("actual   result : %s", psl_to_asc(str1, 0));
	}
	else 
		PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}
