/*
*      SCCS:  @(#)  ANSI.os/string/strstr/strstr.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/string/strstr - T.strstr Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strstr

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the strstr() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		David McCardle, UniSoft Ltd.
DATE CREATED:	Wed Oct 22 09:07:10 PDT 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	testid()
		Modified xx_rpt, xs_rpt()

		David Sawyer
		Thu Jul 05 09:49:39 BST 1990
		Added resillience code

		Stuart Boutell, 31 Oct 1990
			Added test 3 code by Alan Madsen

		Neil Moses, 10 Dec 1991
		Changed %ld to 0x%lx, and cast pointer arg to long

		Geoff Clare, 24 Aug 1995
			Removed ASCII dependencies.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<string.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strstr
#endif


#define NO_TESTS	3		/* number of tests in testset */

extern	char	*psl_to_asc();

int	no_tests = NO_TESTS;

private char	s1[20] = "abcdeabcdeabcdeefg";
private char	s2[20];
private int	testfail;

private void	preparetests();
private void	test1();
private void	test2();
private	void	test3();
private void	checkval();

public	void	(*setfunc) () = preparetests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 1, /* test2 depends on test1 */
	 test3, 3,
	NULL, 0
};

private void
preparetests()
{
	unsigned char *us2 = (unsigned char *)s2;

	DBUG_ENTER("preparetests");

	us2[0] = 'e'^0200;
	us2[1] = 'a';
	us2[2] = 'b'^0200;
	us2[3] = '\0';

	DBUG_VOID_RETURN;
}


private void
test1()
{
	char	*rval;
	unsigned char *us1 = (unsigned char *)s1;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;
	rval = strstr(s1, "cde");
	globok = 0;
	checkval(s1, "cde", rval, &s1[2]);
	PATH_FUNC_TRACE;

	rval = strstr(s1, "e");
	globok = 0;
	checkval(s1, "e", rval, &s1[4]);
	PATH_FUNC_TRACE;

	us1[4] ^= 0200;
	rval = strstr(s1, "eab");
	globok = 0;
	checkval(s1, "eab", rval, &s1[9]);	
	PATH_FUNC_TRACE;

	us1[6] ^= 0200;
	rval = strstr(s1, s2);
	globok = 0;
	checkval(s1, s2, rval, &s1[4]);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	char	*rval;
	unsigned char *us1 = (unsigned char *)s1;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;
	s1[8] = '\0';
	rval = strstr(s1, "eab");
	globok = 0;
	checkval(s1, "eab", rval, (char *) NULL);
	PATH_FUNC_TRACE;

	us1[4] ^= 0200;
	rval = strstr(s1, s2);
	globok = 0;
	checkval(s1, s2, rval, (char *) NULL);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	char	*rval;
	char	*nullstr = "";
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	rval = strstr(nullstr, "");
	globok = 0;
	checkval(nullstr, "", rval, nullstr);
	PATH_FUNC_TRACE;

	rval = strstr(s1, "");
	globok = 0;
	checkval(s1, "", rval, s1);
	PATH_FUNC_TRACE;

	rval = strstr((char*) s2, "");
	globok = 0;
	checkval(s2, "", rval, s2);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
checkval(str1, str2, retval, expval)
char *str1, *str2;
char *retval, *expval;
{
	int	pathok = 0;

	DBUG_ENTER("checkval");

	if ( retval != expval)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("strstr did not return expected value");
		in_rpt("pointer & value of first  argument : 0x%lx\t%s", 
			(long) str1, psl_to_asc(str1, 0));
		in_rpt("value   of second argument : %s", psl_to_asc(str2, 0));
		in_rpt("expected return value      : 0x%lx", (long) expval);
		in_rpt("actual   return value      : 0x%lx", (long) retval);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);
	DBUG_VOID_RETURN;
}
