/*
*      SCCS:  @(#)  ANSI.os/string/strtok/strtok.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/string/strtok - T.strtok Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strtok

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the strtok() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		David McCardle, UniSoft Ltd.
DATE CREATED:	Wed Oct 22 11:31:27 PDT 1986
MODIFICATIONS:	Andrew Twigger 10 June 87
		Removed	testid()
		Modified xx_rpt() xs_rpt()

		David Sawyer
		Thu Jul 05 11:38:19 BST 1990
		Added resillience code

		Neil Moses, 10 Dec 1991
		Changed %ld to 0x%lx, and cast pointer arg to long

		Geoff Clare, 23 June 1992
		Changed v from pointer to array in tests 1 and 2.

		Geoff Clare, 24 Aug 1995
			Removed ASCII dependencies.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<string.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strtok
#endif


#define NO_TESTS	4		/* number of tests in testset */

extern	char	*psl_to_asc();

int	no_tests = NO_TESTS;

private int	testfail;

private char	*copyto();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	checkval();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private char *
copyto(s1, s2)
char *s1, *s2;
{
	char *s;

	DBUG_ENTER("copyto");

	s = s1;
	while (*s++ = *s2++)
		;
	s = NUL;
	DBUG_RETURN(s1);
}


private void
test1()
{
	char	s[30], t[30], *rval;
	static char	v[] = "0123456789";
	unsigned char *us = (unsigned char *)s;
	unsigned char *ut = (unsigned char *)t;
	unsigned char *uv = (unsigned char *)v;
	int	pathok = 0;

	DBUG_ENTER("test1");

	(void) copyto(s, "1albert6joe3sam3fred4");
	(void) copyto(t, "1albert6joe3sam3fred4");

	testfail = 0;

	rval = strtok(s, v);
	globok = 0;
	checkval(s, s, t, v, rval, &s[1], 1, 6);
	PATH_FUNC_TRACE;

	(void) copyto(s, "1albert6joe3sam3fred4");
	us[7] ^= 0200;
	ut[7] ^= 0200;
	rval = strtok(s, v);
	globok = 0;
	checkval(s, s, t, v, rval, &s[1], 1, 10);
	PATH_FUNC_TRACE;

	(void) copyto(s, "1albert6joe3sam3fred4");
	us[7] ^= 0200;
	ut[7] ^= 0200;
	uv[6] ^= 0200;
	rval = strtok(s, v);
	globok = 0;
	checkval(s, s, t, v, rval, &s[1], 1, 6);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	char	s[30], t[30], *rval;
	static char	v[] = "0123456789";
	unsigned char *us = (unsigned char *)s;
	unsigned char *ut = (unsigned char *)t;
	unsigned char *uv = (unsigned char *)v;
	int	pathok = 0;

	DBUG_ENTER("test2");

	(void) copyto(s, "1albert6joe39812sam3fred4");
	(void) copyto(t, "1albert6joe39812sam3fred4");

	testfail = 0;

	rval = strtok(s, v);
	globok = 0;
	checkval(s, s, t, v, rval, &s[1], 1, 6);
	PATH_FUNC_TRACE;

	rval = strtok((char *) NULL, v);
	globok = 0;
	checkval((char *) NULL, s, t, v, rval, &s[8], 8, 3);
	PATH_FUNC_TRACE;

	rval = strtok((char *) NULL, v);
	globok = 0;
	checkval((char *) NULL, s, t, v, rval, &s[16], 16, 3);
	PATH_FUNC_TRACE;

	(void) copyto(s, "1albert6joe39812sam3fred4");

	us[11] ^= 0200;
	ut[11] ^= 0200;
	rval = strtok(s, v);
	checkval(s, s, t, v, rval, &s[1], 1, 6);
	if (globok)
		PATH_TRACE;

	rval = strtok((char *) NULL, v);
	globok = 0;
	checkval((char *) NULL, s, t, v, rval, &s[8], 8, 4);
	PATH_FUNC_TRACE;

	rval = strtok((char *) NULL, v);
	globok = 0;
	checkval((char *) NULL, s, t, v, rval, &s[16], 16, 3);
	PATH_FUNC_TRACE;

	(void) copyto(s, "1albert6joe39812sam3fred4");

	us[11] ^= 0200;
	ut[11] ^= 0200;
	uv[3] ^= 0200;
	rval = strtok(s, v);
	globok = 0;
	checkval(s, s, t, v, rval, &s[1], 1, 6);
	PATH_FUNC_TRACE;

	rval = strtok((char *) NULL, v);
	globok = 0;
	checkval((char *) NULL, s, t, v, rval, &s[8], 8, 3);
	PATH_FUNC_TRACE;

	rval = strtok((char *) NULL, v);
	globok = 0;
	checkval((char *) NULL, s, t, v, rval, &s[16], 16, 8);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}


private void
test3()
{
	char	s[10], t[10], *rval;
	char	*v;
	int	pathok = 0;

	DBUG_ENTER("test3");

	(void) copyto(s, "albert6");
	(void) copyto(t, "albert6");
	v = "0123456789";

	testfail = 0;

	rval = strtok(s, v);
	globok = 0;
	checkval(s, s, t, v, rval, &s[0], 0, 6);
	PATH_FUNC_TRACE;

	rval = strtok((char *) NULL, v);
	globok = 0;
	checkval((char *) NULL, s, t, v, rval, (char *) NULL, 0, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test4()
{
	char	s[30], t[30], *rval;
	char	*v;
	int	pathok = 0;

	DBUG_ENTER("test4");

	(void) copyto(s, "albert6jo:;:e3sam,3fred4");
	(void) copyto(t, "albert6jo:;:e3sam,3fred4");
	v = "0123456789";

	testfail = 0;

	rval = strtok(s, v);
	globok = 0;
	checkval(s, s, t, v, rval, &s[0], 0, 6);
	PATH_FUNC_TRACE;

	v = ":;";
	rval = strtok((char *) NULL, v);
	globok = 0;
	checkval((char *) NULL, s, t, v, rval, &s[7], 7, 2);
	PATH_FUNC_TRACE;

	v = ".,";
	rval = strtok((char *) NULL, v);
	globok = 0;
	checkval((char *) NULL, s, t, v, rval, &s[10], 10, 7);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
checkval(str1, str1_ptr, str1_val, str2, retval, expval, tokstart, toksize)
char *str1, *str2;
char *str1_ptr, *str1_val;
char *retval, *expval;
int tokstart, toksize;
{
	int	failed = 0;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("checkval");

	globok = 0;

	for (i = 0; i < toksize; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (str1_ptr[tokstart+i] != str1_val[tokstart+i])
			failed++;
	}
	if (!toksize)
		PATH_TRACE;

	if (toksize && str1_ptr[tokstart+toksize] != '\0')
			failed++;
	else
		PATH_TRACE;


	if (failed || retval != expval)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("strtok did not return correct value");
		in_rpt("pointer of first argument         : 0x%lx",(long) str1);
		in_rpt("pointer & value of search string  : 0x%lx\t%s", 
			(long) str1_ptr, psl_to_asc(str1_val, 0));
		in_rpt("value of second argument          : %s", psl_to_asc(str2, 0));
		in_rpt("expected return value      : 0x%lx\t%s", 
			(long) expval, psl_to_asc(expval, 0));
		in_rpt("actual   return value      : 0x%lx\t%s", 
			(long) retval, psl_to_asc(retval, 0));
	}
	else
	{
		PATH_TRACE;
		PATH_FUNC_RPT(3);
	}
	
	DBUG_VOID_RETURN;
}

