/*
*      SCCS:  @(#)  ANSI.os/string/strxfrm_X/strxfrm_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/string/strxfrm_X - T.strxfrm_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strxfrm_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the strxfrm() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Linda Taylor, Hoskyns.
DATE CREATED:	28th July, 1988.
MODIFICATIONS:
		Geoff Clare, 22 Aug 1989
			Change "n too small" test to 1 less than
			required length instead of 19 less (which
			may underflow).
		
		Geoff Clare, 29 Sep 1989
			Message in checkval() was misleading
		
		Geoff Clare, 5 Dec 1989
			Compare transformed strings in test2 instead
			of expecting no transformation.

		David Sawyer
		Thu Jul 05 12:22:20 BST 1990
		Added resillience code

		Geoff Clare, 29 May 1992
			Don't use literal language names.

		Geoff Clare, 18 March 1994
			Greatly increased size of general strxfrm() output
			buffers.

		Geoff Clare, 12 April 1995
			Fix reporting problem in test 2.

		Geoff Clare, 11 Aug 1995
			Remove test of behaviour with VSX4L0.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<nlstest.h>
#include	<sys/types.h>
#include	<string.h>
#include	<locale.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strxfrm
#endif


#define NO_TESTS	4		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define BSIZE		256

extern	char	*hexstr();
extern	char	*asc_to_psl();
extern	char	*psl_to_asc();

int	no_tests = NO_TESTS;

private int	testfail;
private char	*testcomment;
private char	*currlang;
private	int	nllang = 0;
private	char	buf1[2*BSIZE];
private char	buf2[2*BSIZE];
private char	buf3[2*BSIZE];
private char	store1[2*BSIZE];
private char	store2[2*BSIZE];

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private	void	checkval();
private void	checkres();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	retval;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	testfail = 0;
 	if (setlocale(LC_COLLATE, INFO1LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Cannot set locale for LC_COLLATE to %s", INFO1LANG);
		in_rpt("Check installation of locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	currlang = INFO1LANG;
	nllang = VALINFO1LANG;

	testcomment = "1-2 character mappings are undertaken";
	(void) strcpy(buf1, "hasse");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "ha&e");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, 0, retval);
	PATH_FUNC_TRACE;

	testcomment = "accented comparisons are undertaken";
	(void) strcpy(buf1, "ta\"che");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "ta^che");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	testcomment = "primary order overrides accented comparison";
	(void) strcpy(buf1, "ta^che");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "ta\"chet");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	testcomment = "accented comparison against non-accented word";
	(void) strcpy(buf1, "te`nse");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "tense");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

	(void) strcpy(buf1, "arc~ed");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "arced");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

	testcomment = "2-1 mappings are undertaken";
	(void) strcpy(buf1, "cache");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "cacet");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

	testcomment = "1-1 mappings are undertaken";
	(void) strcpy(buf1, "barn");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "Barn");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	(void) strcpy(buf1, "Barn");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "bern");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	testcomment = "dont care characters are ignored";
	(void) strcpy(buf1, "barn");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "b-arn");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, 0, retval);
	PATH_FUNC_TRACE;

	if (setlocale(LC_COLLATE, INFO2LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Cannot set locale for LC_COLLATE to %s", INFO2LANG);
		in_rpt("Check installation of locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO2LANG;
	nllang = VALINFO2LANG;

	testcomment = "the collating sequence for the current language is used";
	(void) strcpy(buf1, "Barn");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "clock");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

 	if (setlocale(LC_ALL, INFO2LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Cannot set locale for LC_ALL to %s", INFO2LANG);
		in_rpt("Check installation of locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO2LANG;
	nllang = VALINFO2LANG;

	testcomment = "1-2 character mappings are undertaken";
	(void) strcpy(buf1, "fa&e");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "fasse");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, 0, retval);
	PATH_FUNC_TRACE;

	testcomment = "accented comparisons are undertaken";
	(void) strcpy(buf1, "tre^at");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "tre'at");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

	testcomment = "primary order overrides accented comparison";
	(void) strcpy(buf1, "Fre^d");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "Fre'ddie");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	testcomment = "The collating sequence uses a primary order";
	(void) strcpy(buf1, "Advent");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "zone");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

	testcomment = "accented comparison against non-accented word";
	(void) strcpy(buf1, "pa\"ssed");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "passed");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	(void) strcpy(buf1, "ge^ts");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "gets");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, 1, retval);
	PATH_FUNC_TRACE;

	testcomment = "2-1 mappings are undertaken";
	(void) strcpy(buf1, "Cused");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "Chase");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	testcomment = "dont care characters are ignored";
	(void) strcpy(buf1, "b-arn");
	(void) asc_to_psl(buf1, VALINFO2LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "barn");
	(void) asc_to_psl(buf2, VALINFO2LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, 0, retval);
	PATH_FUNC_TRACE;

	if (setlocale(LC_ALL, INFO1LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Cannot set locale for LC_ALL to %s", INFO1LANG);
		in_rpt("Check installation of locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO1LANG;
	nllang = VALINFO1LANG;

	testcomment = "the collating sequence for the current language is used";
	(void) strcpy(buf1, "Hot");
	(void) asc_to_psl(buf1, VALINFO1LANG);
	(void) strxfrm(store1, buf1, (size_t)BSIZE);
	(void) strcpy(buf2, "hotter");
	(void) asc_to_psl(buf2, VALINFO1LANG);
	(void) strxfrm(store2, buf2, (size_t)BSIZE);
	retval = strcmp(store1, store2);
	globok = 0;
	checkval(buf1, buf2, -1, retval);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(23);

	DBUG_VOID_RETURN;
} 

private	void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);	

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	retval;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	testfail = 0;

	currlang = "UNDEFINED";
	nllang = 0;

	(void) strxfrm(buf1, "cache", (size_t)BSIZE);
	(void) strxfrm(buf2, "caclet", (size_t)BSIZE);
	retval = strcmp(buf1, buf2);
	globok = 0;
	checkval("cache", "caclet", -1, retval);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int	retval, expect;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	testfail = 0;

 	if( setlocale(LC_COLLATE, INFO1LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Cannot set locale for LC_COLLATE to %s", INFO1LANG);
		in_rpt("Check installation of locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO1LANG;
	nllang = VALINFO1LANG;

	testcomment = "The returned string contains at the most the same number of characters as the 3rd argument, including the terminating NULL";
	(void) strcpy(buf2, "rotte");	
	(void) asc_to_psl(buf2, VALINFO1LANG);
	(void) strxfrm(buf1, buf2, (size_t)BSIZE);
	expect = strlen(buf1);
	retval = (int) strxfrm(buf3, buf2, (size_t) expect + 1);
	globok = 0;
	checkres(buf2, expect, retval);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
	
private void
test4()
{
	DBUG_ENTER("test4");

	(void) cppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	retval, expect;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	testfail = 0;
 	if ( setlocale(LC_COLLATE, INFO1LANG) == NULL )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Cannot set locale for LC_COLLATE to %s", INFO1LANG);
		in_rpt("Check installation of locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO1LANG;
	nllang = VALINFO1LANG;

	testcomment = "If the returned string exceeds the length set in the 3rd argument, the required length is returned";
	(void) strcpy(buf2, "bern");	
	(void) asc_to_psl(buf2, VALINFO1LANG);
	(void) strxfrm(buf1, buf2, (size_t)BSIZE);
	expect = strlen(buf1);
	retval = (int) strxfrm(buf3, buf2, (size_t)(expect-1));
	globok = 0;
	checkres(buf2, expect, retval);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
checkval(str1, str2, expval, rtval)
char *str1, *str2;
int  expval, rtval;
{
	int	failed = 0;
	char	*expstr;
	int	pathok = 0;

	DBUG_ENTER("checkval");

	globok = 0;

	if (expval < 0)
	{
		expstr = "< 0";
		if (rtval >= 0)
			failed++;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (expval > 0)
	{
		expstr = "> 0";
		if (rtval <= 0)
			failed++;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (expval == 0)
	{
		expstr= "0";
		if (rtval != 0)
			failed++;
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (failed)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt(testcomment);
		in_rpt("Current language setting is %s", currlang);
		in_rpt("strxfrm did not return expected value");
		in_rpt("first word to be transformed :%s (%s)",
			hexstr((unsigned char*)(str1)),
			psl_to_asc(str1, nllang));
		in_rpt("second word to be transformed : %s (%s)",
			hexstr((unsigned char*)(str2)),
			psl_to_asc(str2, nllang));
		in_rpt("strcmp() on transformed strings: expected %s, actual %d",
			expstr, rtval);
		testfail++;
	}
	else
		PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
checkres(str, expval, rtval)
char *str;
int  expval, rtval;
{
	int	failed = 0;
	int	pathok = 0;

	DBUG_ENTER("checkres");

	globok = 0;

	if (expval != rtval)
		failed++;
	else
		PATH_TRACE;

	if (failed)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt(testcomment);
		in_rpt("Current language setting is %s", currlang);
		in_rpt("strxfrm did not return expected value");
		in_rpt("value of string to be transformed: %s (%s)",
			hexstr((unsigned char*)(str)), psl_to_asc(str,nllang));
		in_rpt("expected return value : %d", expval);
		in_rpt("actual   return value    : %d", rtval);
		testfail++;
	}
	else
		PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}
