/*
*      SCCS:  @(#)  ANSI.os/time/ctime/ctime.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/time/ctime - T.ctime Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.ctime

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the ctime() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	20 Dec 1988
		(from copy of new localtime() testset)

MODIFIED:	
		Geoff Clare, 13 Sep 1989
			Don't check tzname[1] when TZ has no dst zone name

		Geoff Clare, 9 Nov 1989
			More thorough testing of allowed zone names
		
		Geoff Clare, 23 Nov 1989
			Split out daylight and timezone tests to ctime_X

		Dave Cater, 18 Dec 1989
			Only check alphabetic characters in allowed zone names

		Geoff Clare, 23 Jan 1990
			Use [0, UCHAR_MAX] instead of [CHAR_MIN, CHAR_MAX]
			to avoid passing invalid values to isalpha()

		Geoff Clare, 2 May 1990
			Use TZNAME_MAX in long timezone name test.
			Return to testing all allowed characters in zone names

		Peter Lee, Unisoft Ltd., Fri Jul 06 09:37:33 BST 1990
			Added resilience code.

		Geoff Clare, 17 Sep 1991
			Check testcomment != NULL before printing

		Geoff Clare, 14 Jan 1992
			Use new tztest() routine for tzname[] testing.

		Geoff Clare, 22 May 1992
			Added tests 12 and 13.

		Geoff Clare, 27 May 1992
			Repeat tests with Julian day numbers in TZ for
			leap and non-leap years.  Merge tests 7 and 9.

		Geoff Clare, 16 June 1993
			Rename clock variable to clockval.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<ctype.h>
#include	<errno.h>
#include	<time.h>
#include	<limits.h>
#include	<dbug.h>
#include	<report.h>
#include	<pcts_max.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ctime
#endif


#define 	NO_TESTS	13		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

extern	int	cppair();
extern	char	*psl_to_asc();
extern	char	*errname();
extern	char	**environ;
extern	void	tztest();

int	no_tests = NO_TESTS;

private void	test_pa();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();

private	int	set_tz();
private	void	checkval();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test_pa, 1,
	 test_pa, 2,
	 test_pa, 3,
	 test_pa, 4,
	 test_pa, 5,
	 test_pa, 6,
	 test_pa, 7,
	 test_pa, 8,
	 test_pa, 9,
	 test_pa, 10,
	 test_pa, 11,
	 test_pa, 12,
	 test_pa, 13,
	NULL, 0
};

private	void	(*chfuncs[]) () = { test1, test2, test3, test4, test5,
				    test6, test7, test8, test9, test10,
				    test11, test12, test13,
				  };

private int		testfail;
private char		*testcomment;

private	void
test_pa()
{
	DBUG_ENTER("test_pa");

	/* Common parent function for all tests.  Tests must take
	   place in child processes because they all set TZ */

	testfail = 0;
	(void) cppair(chfuncs[tet_thistest - 1], NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
test1()
{
	int	pathok = 0;

	DBUG_ENTER("test1");

	if (set_tz("LOC-3") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can take the form of timezone and offset (hour only)";

	globok = 0;
	checkval(567869055L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	int	pathok = 0;

	DBUG_ENTER("test2");

	if (set_tz("LOC5:35") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can take the form of timezone and offset (hour and minutes only)";


	globok = 0;
	checkval(586185855L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private	void
test3()
{
	int	pathok = 0;

	DBUG_ENTER("test3");

	if (set_tz("GMT-2:0:50") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can take the form of timezone and offset (hour, minutes and seconds)";

	globok = 0;
	checkval(586185875L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private	void
test4()
{
	int	pathok = 0;

	DBUG_ENTER("test4");

	if (set_tz("GMT0BST,J81,J300") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - a";


	globok = 0;
	checkval(574999199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - b";


	globok = 0;
	checkval(574999221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - c";


	globok = 0;
	checkval(593917199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - d";


	globok = 0;
	checkval(593917221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - e";


	globok = 0;
	checkval(606535199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - f";


	globok = 0;
	checkval(606535221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - g";


	globok = 0;
	checkval(625453199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - h";


	globok = 0;
	checkval(625453221L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}

private	void
test5()
{
	int	pathok = 0;

	DBUG_ENTER("test5");

	if (set_tz("GMT0BST-2,J81,J300") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - a";


	globok = 0;
	checkval(574999199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - b";


	globok = 0;
	checkval(574999221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - c";


	globok = 0;
	checkval(593913599L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - d";


	globok = 0;
	checkval(593913621L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - e";


	globok = 0;
	checkval(606535199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - f";


	globok = 0;
	checkval(606535221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - g";


	globok = 0;
	checkval(625449599L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - h";


	globok = 0;
	checkval(625453221L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}

private	void
test6()
{
	int	pathok = 0;

	DBUG_ENTER("test6");

	if (set_tz("TZA0TZB,J81/3,J300/1:29:20") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - a";


	globok = 0;
	checkval(575002799L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - b";


	globok = 0;
	checkval(575002821L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - c";


	globok = 0;
	checkval(593915359L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - d";


	globok = 0;
	checkval(593915381L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - e";


	globok = 0;
	checkval(606538799L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - f";


	globok = 0;
	checkval(606538821L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - g";


	globok = 0;
	checkval(625451359L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - h";


	globok = 0;
	checkval(625451381L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}

private	void
test7()
{
	int	pathok = 0;

	DBUG_ENTER("test7");

	if (set_tz("LOC0DST,81,300") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - a";


	globok = 0;
	checkval(574999199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - b";


	globok = 0;
	checkval(574999221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - c";


	globok = 0;
	checkval(593917199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - d";


	globok = 0;
	checkval(593917221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - e";


	globok = 0;
	checkval(606621599L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - f";


	globok = 0;
	checkval(606621621L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - g";


	globok = 0;
	checkval(625539599L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - h";


	globok = 0;
	checkval(625539621L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}

private	void
test8()
{
	int	pathok = 0;

	DBUG_ENTER("test8");

	if (set_tz("LOC0DST,81/3,300/1:29:20") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - a";


	globok = 0;
	checkval(575002799L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - b";


	globok = 0;
	checkval(575002821L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - c";


	globok = 0;
	checkval(593915359L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - d";


	globok = 0;
	checkval(593915381L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - e";


	globok = 0;
	checkval(606625199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - f";


	globok = 0;
	checkval(606625221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - g";


	globok = 0;
	checkval(625537759L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - h";


	globok = 0;
	checkval(625537781L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}

private	void
test9()
{
	DBUG_ENTER("test9");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

private	void
test10()
{
	int	pathok = 0;

	DBUG_ENTER("test10");

	testcomment = "TZ can state daylight saving start and end dates (given as month, week in month and day in week) - a";
	if (set_tz("LOC0DST,M3.2.0,M10.5.0") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	globok = 0;
	checkval(574221599L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as month, week in month and day in week) - b";

	
	globok = 0;
	checkval(574221621L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as month, week in month and day in week) - c";


	globok = 0;
	checkval(594176399L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as month, week in month and day in week) - d";


	globok = 0;
	checkval(594176421L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private char *
t11_func(isdst)
int isdst;
{
	time_t	clockval = 586185855L;

	(void) ctime(&clockval);

	return tzname[isdst];
}

private void
test11()
{
	DBUG_ENTER("test11");

	tztest("ctime()", t11_func);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	int	pathok = 0;

	DBUG_ENTER("test12");

	if (set_tz("CUT0") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "with TZ=CUT0 ctime() returns co-ordinated universal time - a";
	globok = 0;
	checkval(0L);  
	PATH_FUNC_TRACE;

	testcomment = "with TZ=CUT0 ctime() returns co-ordinated universal time - b";
	globok = 0;
	checkval(530454574L);  
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
test13()
{
	time_t	clockval;
	int	pathok = 0;

	DBUG_ENTER("test13");

	if (set_tz("CUT0") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) ctime(&clockval);

	if (set_tz("ABC1") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "when TZ changes, ctime() uses the new value";

	globok = 0;
	checkval(530454574L);  
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private int
set_tz(tzval)
char *tzval;
{
	char	*tz;
	static char	buf[2 * PCTS_TZNAME_MAX + 5];
	int	pathok = 0;

	DBUG_ENTER("set_tz");

	(void) strcpy(buf, "TZ=");
	(void) strcat(buf, tzval);
	(void) vputenv(&environ, buf);
	tz = getenv("TZ");
	if (tz == NULL || strcmp(tz, tzval) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not put \"TZ=%s\" in environment",
			psl_to_asc(tzval, 0));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	DBUG_RETURN(0);
}

private void
checkval(tval)
long tval;
{
	char	*observed, *expected;
	time_t	clockval;
	int	lastfail = testfail;
	int	pathok = 0;

	DBUG_ENTER("checkval");

	globok = 0;
	clockval = tval;
	observed = ctime(&clockval);

	if (observed == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);	
		if (testcomment != NULL)
			in_rpt("%s", testcomment);
		in_rpt("ctime(&clock) returned a NULL pointer");
		globok = 0;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	expected = asctime(localtime(&clockval));
	if (expected == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);	
		in_rpt("deletion reason: asctime() returned a NULL pointer");
		globok = 0;
		DBUG_VOID_RETURN;
	}
	else	
		PATH_TRACE;

#ifdef TESTBED
	observed = "bad time string\n";
#endif

	if (strcmp(observed, expected) != 0)
	{
		if (testfail++ == 0)
		{
			globok = 0;
			xx_rpt(FAILURE);
		}	
		if (testcomment != NULL)
			in_rpt("%s", testcomment);
		in_rpt("ctime() returned: \"%s\"", psl_to_asc(observed, 0));
		in_rpt("expected string:  \"%s\"", psl_to_asc(expected, 0));
	}
	else
		PATH_TRACE;

	if (testfail > lastfail)
		in_rpt("values used: TZ=%s; clock = %ld", getenv("TZ"), tval);

	PATH_FUNC_RPT(3);
	
	DBUG_VOID_RETURN;
}
