/*
*      SCCS:  @(#)  ANSI.os/time/ctime_X/ctime_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/time/ctime_X - T.ctime_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.ctime_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the ctime() system call - X/Open extensions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	23 Nov 1989
		(split out from ctime)

MODIFIED:	Peter Lee, Unisoft Ltd.
		Mon Jul 09 11:02:17 BST 1990
		Added resilience code.

		Geoff Clare, 16 June 1993
			Rename clock variable to clockval.

************************************************************************/
#include	<std.h>
#include	<string.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<stdlib.h>
#include	<time.h>
#include	<limits.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ctime
#endif


#define 	NO_TESTS	2		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

extern	int	cppair();
extern	char	*psl_to_asc();
extern	char	**environ;

int	no_tests = NO_TESTS;

private void	test_pa();
private void	test1();
private void	test2();

private	int	set_tz();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test_pa, 1,
	 test_pa, 2,
	NULL, 0
};

private	void	(*chfuncs[]) () = { test1, test2 };

private int		testfail;

private	void
test_pa()
{
	DBUG_ENTER("test_pa");

	/* Common parent function for all tests.  Tests must take
	   place in child processes because they all set TZ */

	testfail = 0;
	(void) cppair(chfuncs[tet_thistest - 1], NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = 0;

	time_t clockval = 586185855L;
	
	DBUG_ENTER("test1");

	if (set_tz("TZA-9") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) ctime(&clockval);
	if (timezone != -9*60*60L)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("with TZ=TZA-9 ctime() did not set timezone correctly");
		in_rpt("value of timezone was %ld, expected %ld",
			timezone, -9*60*60L);
	}
	else
		PATH_TRACE;

	if (set_tz("JKL3:10PNM4:40") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) ctime(&clockval);
	if ( timezone != 11400 ) 
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("with TZ=JKL3:10PNM4:40 ctime() did not set timezone correctly");
		in_rpt("value of timezone was %ld, expected 11400", timezone);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	pathok = 0;

	time_t clockval = 586185855L;
	
	DBUG_ENTER("test2");

	if (set_tz("ABC-4:30:20") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) ctime (&clockval);
	if ( daylight != 0 )
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("with TZ=ABC-4:30:20 ctime() did not set daylight correctly");
		in_rpt("value of daylight was %d, expected zero", daylight);
	}
	else
		PATH_TRACE;

	if (set_tz("JKL3:10PNM4:40") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) ctime (&clockval);
	if ( daylight == 0 )
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("with TZ=JKL3:10PNM4:40 ctime() did not set daylight correctly");
		in_rpt("value of daylight was %d, expected non-zero", daylight);
	}
	else
		PATH_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private int
set_tz(tzval)
char *tzval;
{
	char	*tz;
	static char	buf[256];
	int	pathok = 0;

	DBUG_ENTER("set_tz");

	(void) strcpy(buf, "TZ=");
	(void) strncat(buf, tzval, (size_t)250);
	(void) vputenv(&environ, buf);
	tz = getenv("TZ");
	if (tz == NULL || strcmp(tz, tzval) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not put \"TZ=%s\" in environment",
			psl_to_asc(tzval, 0));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	DBUG_RETURN(0);
}
