/*
*      SCCS:  @(#)  ANSI.os/time/gmtime/gmtime.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/time/gmtime - T.gmtime Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.gmtime

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the gmtime() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Mon Nov 10 16:03:05 WET 1986

MODIFIED:	Linda Taylor, Hoskyns.  	August, 1988.
			Split out from ctime testset.
		Geoff Clare, 20 Dec 1988
			Allowance for leap-seconds.
			Test 2 added.
		Peter Lee, Unisoft Ltd.
			Mon Jul 09 11:58:17 BST 1990
			Added resilience code.

		Geoff Clare, 16 June 1993
			Rename clock variable to clockval.

************************************************************************/
#include	<std.h>
#include	<string.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<ctype.h>
#include	<time.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef gmtime
#endif


#define 	NO_TESTS	2		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

#define		MAX_LEAP_SECS	10	/* allowance for leap seconds since
					   the epoch.  Actual tm_sec values
					   are allowed to be MAX_LEAP_SECS
					   less than the expected value */

extern	int	cppair();
extern	char	**environ;

int	no_tests = NO_TESTS;

private int		testfail;
private char		*testcomment;
private	struct tm	expected;
private	int		commented;

private void	test1();
private void	ch_t1();
private void	test2();
private void	ch_t2();

private	int	set_tz();
private	void	checkval();
private	void	chkfield();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};


private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t1()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	if (set_tz("CUT") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tzset();

	testfail = 0;

	testcomment = "gmtime returns a structure containing CUT";

	expected.tm_sec  = 34;
	expected.tm_min  = 29;
	expected.tm_hour = 12;
	expected.tm_mday = 23;
	expected.tm_mon  = 9;
	expected.tm_year = 86;
	expected.tm_wday = 4;
	expected.tm_yday = 295;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(530454574L);  
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);
		
	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t2()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	if (set_tz("TZA-3") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tzset();

	testfail = 0;

	testcomment = "gmtime does not take account of the TZ variable";

	expected.tm_sec  = 34;
	expected.tm_min  = 29;
	expected.tm_hour = 12;
	expected.tm_mday = 23;
	expected.tm_mon  = 9;
	expected.tm_year = 86;
	expected.tm_wday = 4;
	expected.tm_yday = 295;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(530454574L);  
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);
		
	DBUG_VOID_RETURN;
}

private int
set_tz(tzval)
char *tzval;
{
	char	*tz;
	static char	buf[54];
	int	pathok = 0;

	DBUG_ENTER("set_tz");

	(void) strcpy(buf, "TZ=");
	(void) strncat(buf, tzval, (size_t)50);
	(void) vputenv(&environ, buf);
	tz = getenv("TZ");
	if (tz == NULL || strcmp(tz, tzval) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not put TZ=%s in environment", tzval);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	DBUG_RETURN(0);
}

private void
checkval(tval)
long tval;
{
	struct tm	*observed;
	time_t		clockval;
	int		lastfail = testfail;
	int		pathok = 0;

	DBUG_ENTER("checkval");

	commented = 0;
	globok = 0;

	clockval = tval;
	observed = gmtime(&clockval);

	if (observed == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);	
		if (commented++ == 0)
			in_rpt("%s", testcomment);
		in_rpt("gmtime(&clock) returned a NULL pointer");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* This is to catch bad expected values during development */
	if (expected.tm_sec <= MAX_LEAP_SECS)
		vsxerror("expected tm_sec must be > %d", MAX_LEAP_SECS);

	if ( observed->tm_sec < expected.tm_sec - MAX_LEAP_SECS ||
	     observed->tm_sec > expected.tm_sec )
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);	
		if (commented++ == 0)
			in_rpt("%s", testcomment);
		in_rpt("gmtime set tm_sec to %d instead of between %d and %d",
			observed->tm_sec, expected.tm_sec - MAX_LEAP_SECS,
			expected.tm_sec);
	}
	else
		PATH_TRACE;

	globok = 0;
	chkfield("tm_min",  observed->tm_min,  expected.tm_min);
	PATH_FUNC_TRACE;
	globok = 0;
	chkfield("tm_hour", observed->tm_hour, expected.tm_hour);
	PATH_FUNC_TRACE;
	globok = 0;
	chkfield("tm_mday", observed->tm_mday, expected.tm_mday);
	PATH_FUNC_TRACE;
	globok = 0;
	chkfield("tm_mon",  observed->tm_mon,  expected.tm_mon);
	PATH_FUNC_TRACE;
	globok = 0;
	chkfield("tm_year", observed->tm_year, expected.tm_year);
	PATH_FUNC_TRACE;
	globok = 0;
	chkfield("tm_wday", observed->tm_wday, expected.tm_wday);
	PATH_FUNC_TRACE;
	globok = 0;
	chkfield("tm_yday", observed->tm_yday, expected.tm_yday);
	PATH_FUNC_TRACE;

	if ( observed->tm_isdst != 0 && expected.tm_isdst == 0 )
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);	
		if (commented++ == 0)
			in_rpt("%s", testcomment);
		in_rpt("gmtime set tm_isdst to %d instead of %d",
			observed->tm_isdst, expected.tm_isdst);
	}
	else if ( observed->tm_isdst == 0 && expected.tm_isdst != 0 )
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);	
		if (commented++ == 0)
			in_rpt("%s", testcomment);
		in_rpt("gmtime set tm_isdst to %d instead of %d",
			observed->tm_isdst, expected.tm_isdst);
	}
	else
		PATH_TRACE;

	if (testfail > lastfail)
		in_rpt("values used: TZ=%s; clock = %ld", getenv("TZ"), tval);

	PATH_FUNC_RPT(10);

	DBUG_VOID_RETURN;
}

private void
chkfield(name, obs, exp)
char *name;
int obs, exp;
{
	globok = 0;

	if ( obs != exp )
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);	
		if (commented++ == 0)
			in_rpt("%s", testcomment);
		in_rpt("gmtime set %s to %d instead of %d", name, obs, exp);
	}
	else
		globok = 1;
}
