/*
*      SCCS:  @(#)  ANSI.os/time/mktime/mktime.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/time/mktime - T.mktime Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.mktime

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the mktime() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	21 Dec 1988

MODIFIED:
		Geoff Clare, 13 Sep 1989
			Don't check tzname[1] when TZ has no dst zone name
			Allow two answers when daylight & standard overlap
			and tm_isdst == -1
		
		Geoff Clare, 27 Sep 1989
			Allow two answers, as above, in test16
		
		Geoff Clare, 1 Nov 1989
			Check structure members set by mktime() in checkval().
			Do individual test cases for each structure
			element in "out of range" test.

		Geoff Clare, 9 Nov 1989
			More thorough testing of allowed zone names

		Geoff Clare, 23 Nov 1989
			Split out daylight and timezone tests to mktime_X

		Dave Cater, 18 Dec 1989
			Only check alphabetic characters in allowed zone names

		Geoff Clare, 23 Jan 1990
			Use [0, UCHAR_MAX] instead of [CHAR_MIN, CHAR_MAX]
			to avoid passing invalid values to isalpha()

		Geoff Clare, 2 May 1990
			Use TZNAME_MAX in long timezone name test.
			Return to testing all allowed characters in zone names

		Peter Lee, Unisoft Ltd.,
			Tue Jul 10 11:15:55 BST 1990
			Added resilience code.

		Geoff Clare, 5 Mar 1991
			Copy tm structure before passing to mktime() in
			timezone name test.
			Cast argument to isdigit() to ensure correct range.

		Stuart Boutell, 18 Apr 1991
			Upgraded test 3,4,5 to use tm_isdst -1,0,1 modes.

		Geoff Clare, 18 July 1991
			Add unrepresentable times test.

		Geoff Clare, 17 Sep 1991
			Check testcomment != NULL before printing

		Geoff Clare, 14 Jan 1992
			Use new tztest() routine for tzname[] testing.

		Geoff Clare, 5 Feb 1992
			Set time structure in test 18.

		Geoff Clare, 22 May 1992
			Do tzname[] testing with maximum values (to ensure
			that if mktime() can fail, tzname[] is still set).
			Added test 20.

		Geoff Clare, 27 May 1992
			Repeat tests with Julian day numbers in TZ and
			(tm_isdst == -1) for leap and non-leap years.
			Merge tests 13 and 15.

		Geoff Clare, 19 June 1992
			In new leap/non-leap tests, allow ambiguous times 
			to be treated as either standard or daylight time.

		Geoff Clare, 14 Sept 1992
			Ignore SIGFPE when determining whether
			unrepresentable times exist.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<signal.h>
#include	<string.h>
#include	<ctype.h>
#include	<errno.h>
#include	<time.h>
#include	<limits.h>
#include	<dbug.h>
#include	<report.h>
#include	<pcts_max.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef mktime
#endif


#define 	NO_TESTS	20		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

#define		MAX_LEAP_SECS	10	/* allowance for leap seconds since
					   the epoch.  Actual mktime() values
					   are allowed to be MAX_LEAP_SECS
					   more than the expected value */

extern	int	cppair();
extern	char	*psl_to_asc();
extern	char	*errname();
extern	char	**environ;
extern	void	tztest();
extern	void	(*vsignal())();

int	no_tests = NO_TESTS;

private void	test_pa();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();

private	int	set_tz();
private	void	checkval();
private	void	chkrange();
private	void	chkfield();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test_pa, 1,
	 test_pa, 2,
	 test_pa, 3,
	 test_pa, 4,
	 test_pa, 5,
	 test_pa, 6,
	 test_pa, 7,
	 test_pa, 8,
	 test_pa, 9,
	 test_pa, 10,
	 test_pa, 11,
	 test_pa, 12,
	 test_pa, 13,
	 test_pa, 14,
	 test_pa, 15,
	 test_pa, 16,
	 test_pa, 17,
	 test_pa, 18,
	 test_pa, 19,
	 test_pa, 20,
	NULL, 0
};

private	void	(*chfuncs[]) () = { test1, test2, test3, test4, test5,
				    test6, test7, test8, test9, test10,
				    test11, test12, test13, test14, test15,
				    test16, test17, test18, test19, test20,
				  };

private long		alt_val = 0;
private int		testfail;
private int		commented;
private char		*testcomment = "";
private struct tm	expected;
private struct tm	observed;

private	void
test_pa()
{
	DBUG_ENTER("test_pa");

	/* Common parent function for all tests.  Tests must take
	   place in child processes because they all set TZ */

	testfail = 0;
	(void) cppair(chfuncs[tet_thistest - 1], NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = 0;

	time_t rval;

	DBUG_ENTER("test1");

	if (set_tz("CUT0") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "with TZ=CUT0 mktime() returns co-ordinated universal time - a";

	expected.tm_sec  = 0;
	expected.tm_min  = 0;
	expected.tm_hour = 0;
	expected.tm_mday = 1;
	expected.tm_mon  = 0;
	expected.tm_year = 70;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	rval = mktime(&expected);
	if (rval != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("%s", testcomment);
		in_rpt("mktime() returned incorrect value");
		in_rpt("actual value %ld; expected 0", (long)rval);
		in_rpt("values used: TZ=%s; tm_isdst = %d",
			getenv("TZ"), expected.tm_isdst);
		in_rpt("sec = %d min = %d hour = %d mday = %d mon = %d year = %d",
			expected.tm_sec,  expected.tm_min, expected.tm_hour,
			expected.tm_mday, expected.tm_mon, expected.tm_year);
	}
	else
		PATH_TRACE;

	testcomment = "with TZ=CUT0 mktime() returns co-ordinated universal time - b";
	expected.tm_sec  = 34;
	expected.tm_min  = 29;
	expected.tm_hour = 12;
	expected.tm_mday = 23;
	expected.tm_mon  = 9;
	expected.tm_year = 86;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(530454574L);  
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	pathok = 0;

	DBUG_ENTER("test2");

	if (set_tz("CUT0") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "mktime() sets the structure members to the values for the time returned - a";

	/* minimum S,M,H,D,M,(Y) - 00:00:00 1st Jan 1970 */
	expected.tm_sec  = 0;
	expected.tm_min  = 0;
	expected.tm_hour = 0;
	expected.tm_mday = 1;
	expected.tm_mon  = 0;
	expected.tm_year = 70;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	observed = expected;
	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	testcomment = "mktime() sets the structure members to the values for the time returned - b";

	/* minimum tm_wday - Sun 4th Jan 1970 */
	expected.tm_mday = 4;

	observed = expected;
	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	testcomment = "mktime() sets the structure members to the values for the time returned - c";

	/* maximum (S),M,H,D,M,wday,yday - 23:59:59 Sat 31st Dec 1988 */
	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 23;
	expected.tm_mday = 31;
	expected.tm_mon  = 11;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	observed = expected;
	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);
	
	DBUG_VOID_RETURN;
}

private	void
test3()
{
	int	pathok = 0;

	DBUG_ENTER("test3");

	if (set_tz("LOC-3") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can take the form of timezone and offset (hour only)";

	expected.tm_sec  = 15;
	expected.tm_min  = 24;
	expected.tm_hour = 16;
	expected.tm_mday = 30;
	expected.tm_mon  = 11;
	expected.tm_year = 87;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(567869055L);
	PATH_FUNC_TRACE;

	expected.tm_sec  = 15;
	expected.tm_min  = 24;
	expected.tm_hour = 16;
	expected.tm_mday = 30;
	expected.tm_mon  = 11;
	expected.tm_year = 87;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 1;

	globok = 0;
	checkval(567869055L); /* NB Although tm_isdst is 1, we are not
				 expecting this to modify the time, because
				 there is no DST component of the TZ. */
	PATH_FUNC_TRACE;

	expected.tm_sec  = 15;
	expected.tm_min  = 24;
	expected.tm_hour = 16;
	expected.tm_mday = 30;
	expected.tm_mon  = 11;
	expected.tm_year = 87;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(567869055L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private	void
test4()
{
	int	pathok = 0;

	DBUG_ENTER("test4");

	if (set_tz("LOC5:35") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can take the form of timezone and offset (hour and minutes only)";

	expected.tm_sec  = 15;
	expected.tm_min  = 49;
	expected.tm_hour = 7;
	expected.tm_mday = 29;
	expected.tm_mon  = 6;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(586185855L);
	PATH_FUNC_TRACE;

	expected.tm_sec  = 15;
	expected.tm_min  = 49;
	expected.tm_hour = 7;
	expected.tm_mday = 29;
	expected.tm_mon  = 6;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 1;

	globok = 0;
	checkval(586185855L);
	PATH_FUNC_TRACE;

	expected.tm_sec  = 15;
	expected.tm_min  = 49;
	expected.tm_hour = 7;
	expected.tm_mday = 29;
	expected.tm_mon  = 6;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(586185855L);
	PATH_FUNC_TRACE;

	if (set_tz("LOC+5:35") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	expected.tm_sec  = 15;
	expected.tm_min  = 49;
	expected.tm_hour = 7;
	expected.tm_mday = 29;
	expected.tm_mon  = 6;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(586185855L);
	PATH_FUNC_TRACE;

	expected.tm_sec  = 15;
	expected.tm_min  = 49;
	expected.tm_hour = 7;
	expected.tm_mday = 29;
	expected.tm_mon  = 6;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 1;

	globok = 0;
	checkval(586185855L);
	PATH_FUNC_TRACE;

	expected.tm_sec  = 15;
	expected.tm_min  = 49;
	expected.tm_hour = 7;
	expected.tm_mday = 29;
	expected.tm_mon  = 6;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(586185855L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

private	void
test5()
{
	int	pathok = 0;

	DBUG_ENTER("test5");

	if (set_tz("GMT-2:0:50") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can take the form of timezone and offset (hour, minutes and seconds)";

	expected.tm_sec  = 25;
	expected.tm_min  = 25;
	expected.tm_hour = 15;
	expected.tm_mday = 29;
	expected.tm_mon  = 6;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(586185875L);
	PATH_FUNC_TRACE;

	expected.tm_sec  = 25;
	expected.tm_min  = 25;
	expected.tm_hour = 15;
	expected.tm_mday = 29;
	expected.tm_mon  = 6;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 1;

	globok = 0;
	checkval(586185875L);
	PATH_FUNC_TRACE;

	expected.tm_sec  = 25;
	expected.tm_min  = 25;
	expected.tm_hour = 15;
	expected.tm_mday = 29;
	expected.tm_mon  = 6;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(586185875L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private	void
test6()
{
	int	pathok = 0;

	DBUG_ENTER("test6");

	if (set_tz("GMT0BST,J81,J300") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - a";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 1;
	expected.tm_mday = 22;
	expected.tm_mon  = 2;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(574999199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - b";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 2;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(574999221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - c";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 0;
	expected.tm_mday = 27;
	expected.tm_mon  = 9;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(593917199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - d";

	expected.tm_sec = 21;
	expected.tm_min = 0;
	expected.tm_hour = 1;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(593917221L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private	void
test7()
{
	int	pathok = 0;

	DBUG_ENTER("test7");

	if (set_tz("GMT0BST,J81,J300") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - a";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 2;
	expected.tm_mday = 22;
	expected.tm_mon  = 2;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 1;

	globok = 0;
	checkval(574999199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - b";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 3;
	expected.tm_isdst = 1;

	globok = 0;
	checkval(574999221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - c";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 1;
	expected.tm_mday = 27;
	expected.tm_mon  = 9;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 1;

	globok = 0;
	checkval(593917199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - d";

	expected.tm_sec = 21;
	expected.tm_min = 0;
	expected.tm_hour = 2;
	expected.tm_isdst = 1;

	globok = 0;
	checkval(593917221L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private	void
test8()
{
	int	pathok = 0;

	DBUG_ENTER("test8");

	if (set_tz("GMT0BST,J81,J300") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - a";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 1;
	expected.tm_mday = 22;
	expected.tm_mon  = 2;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(574999199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - b";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 3;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(574999221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - c";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 0;
	expected.tm_mday = 27;
	expected.tm_mon  = 9;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(593913599L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - d";

	expected.tm_sec = 21;
	expected.tm_min = 0;
	expected.tm_hour = 1;
	expected.tm_isdst = -1;

	alt_val = -3600;
	globok = 0;
	checkval(593917221L);
	alt_val = 0;
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - e";

	expected.tm_sec = 21;
	expected.tm_min = 0;
	expected.tm_hour = 2;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(593920821L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - f";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 1;
	expected.tm_mday = 22;
	expected.tm_mon  = 2;
	expected.tm_year = 89;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(606535199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - g";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 3;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(606535221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - h";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 0;
	expected.tm_mday = 27;
	expected.tm_mon  = 9;
	expected.tm_year = 89;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(625449599L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - i";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 1;
	expected.tm_isdst = -1;

	alt_val = -3600;
	globok = 0;
	checkval(625453221L);
	alt_val = 0;
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - j";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 2;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(625456821L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(11);

	DBUG_VOID_RETURN;
}

private	void
test9()
{
	int	pathok = 0;

	DBUG_ENTER("test9");

	if (set_tz("GMT0BST-2,J81,J300") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - a";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 1;
	expected.tm_mday = 22;
	expected.tm_mon  = 2;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(574999199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - b";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 2;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(574999221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - c";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 23;
	expected.tm_mday = 26;
	expected.tm_mon  = 9;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(593913599L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - d";

	expected.tm_sec = 21;
	expected.tm_min = 0;
	expected.tm_hour = 0;
	expected.tm_mday = 27;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(593913621L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private	void
test10()
{
	int	pathok = 0;

	DBUG_ENTER("test10");

	if (set_tz("GMT0BST-2,J81,J300") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - a";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 3;
	expected.tm_mday = 22;
	expected.tm_mon  = 2;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 1;

	globok = 0;
	checkval(574999199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - b";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 4;
	expected.tm_isdst = 1;

	globok = 0;
	checkval(574999221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - c";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 1;
	expected.tm_mday = 27;
	expected.tm_mon  = 9;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 1;

	globok = 0;
	checkval(593913599L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - d";

	expected.tm_sec = 21;
	expected.tm_min = 0;
	expected.tm_hour = 2;
	expected.tm_isdst = 1;

	globok = 0;
	checkval(593913621L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private	void
test11()
{
	int	pathok = 0;

	DBUG_ENTER("test11");

	if (set_tz("GMT0BST-2,J81,J300") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - a";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 1;
	expected.tm_mday = 22;
	expected.tm_mon  = 2;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(574999199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - b";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 4;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(574999221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - c";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 23;
	expected.tm_mday = 26;
	expected.tm_mon  = 9;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(593906399L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - d";

	expected.tm_sec = 21;
	expected.tm_min = 0;
	expected.tm_hour = 0;
	expected.tm_mday = 27;
	expected.tm_isdst = -1;

	alt_val = -7200;
	globok = 0;
	checkval(593913621L);
	alt_val = 0;
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving offset start and end dates (given as Julian days) - e";

	expected.tm_sec = 21;
	expected.tm_min = 0;
	expected.tm_hour = 2;
	expected.tm_mday = 27;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(593920821L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - f";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 1;
	expected.tm_mday = 22;
	expected.tm_mon  = 2;
	expected.tm_year = 89;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(606535199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - g";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 4;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(606535221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - h";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 23;
	expected.tm_mday = 26;
	expected.tm_mon  = 9;
	expected.tm_year = 89;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(625442399L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - i";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 0;
	expected.tm_mday = 27;
	expected.tm_isdst = -1;

	alt_val = -7200;
	globok = 0;
	checkval(625449621L);
	alt_val = 0;
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) - j";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 2;
	expected.tm_mday = 27;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(625456821L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(11);

	DBUG_VOID_RETURN;
}

private	void
test12()
{
	int	pathok = 0;

	DBUG_ENTER("test12");

	if (set_tz("TZA0TZB,J81/3,J300/1:29:20") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - a";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 2;
	expected.tm_mday = 22;
	expected.tm_mon  = 2;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(575002799L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - b";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 4;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(575002821L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - c";

	expected.tm_sec  = 19;
	expected.tm_min  = 29;
	expected.tm_hour = 0;
	expected.tm_mday = 27;
	expected.tm_mon  = 9;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(593911759L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - d";

	expected.tm_sec = 41;
	expected.tm_min = 29;
	expected.tm_hour = 0;
	expected.tm_isdst = -1;

	alt_val = -3600;
	globok = 0;
	checkval(593915381L);
	alt_val = 0;
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - e";

	expected.tm_sec = 41;
	expected.tm_min = 29;
	expected.tm_hour = 1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(593918981L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - f";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 2;
	expected.tm_mday = 22;
	expected.tm_mon  = 2;
	expected.tm_year = 89;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(606538799L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - g";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 4;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(606538821L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - h";

	expected.tm_sec  = 19;
	expected.tm_min  = 29;
	expected.tm_hour = 0;
	expected.tm_mday = 27;
	expected.tm_mon  = 9;
	expected.tm_year = 89;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(625447759L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - i";

	expected.tm_sec  = 41;
	expected.tm_min  = 29;
	expected.tm_hour = 0;
	expected.tm_isdst = -1;

	alt_val = -3600;
	globok = 0;
	checkval(625451381L);
	alt_val = 0;
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as Julian days) and times - j";

	expected.tm_sec  = 41;
	expected.tm_min  = 29;
	expected.tm_hour = 1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(625454981L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(11);

	DBUG_VOID_RETURN;
}

private	void
test13()
{
	int	pathok = 0;

	DBUG_ENTER("test13");

	if (set_tz("LOC0DST,81,300") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - a";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 1;
	expected.tm_mday = 22;
	expected.tm_mon  = 2;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(574999199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - b";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 3;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(574999221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - c";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 0;
	expected.tm_mday = 27;
	expected.tm_mon  = 9;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(593913599L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - d";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 1;
	expected.tm_isdst = -1;

	alt_val = -3600;
	globok = 0;
	checkval(593917221L);
	alt_val = 0;
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - e";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 2;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(593920821L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - f";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 1;
	expected.tm_mday = 23;
	expected.tm_mon  = 2;
	expected.tm_year = 89;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(606621599L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - g";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 3;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(606621621L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - h";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 0;
	expected.tm_mday = 28;
	expected.tm_mon  = 9;
	expected.tm_year = 89;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(625535999L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - i";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 1;
	expected.tm_isdst = -1;

	alt_val = -3600;
	globok = 0;
	checkval(625539621L);
	alt_val = 0;
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) - j";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 2;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(625543221L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(11);

	DBUG_VOID_RETURN;
}

private	void
test14()
{
	int	pathok = 0;

	DBUG_ENTER("test14");

	if (set_tz("LOC0DST,81/3,300/1:29:20") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - a";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 2;
	expected.tm_mday = 22;
	expected.tm_mon  = 2;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(575002799L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - b";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 4;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(575002821L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - c";

	expected.tm_sec  = 19;
	expected.tm_min  = 29;
	expected.tm_hour = 0;
	expected.tm_mday = 27;
	expected.tm_mon  = 9;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(593911759L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - d";

	expected.tm_sec  = 41;
	expected.tm_min  = 29;
	expected.tm_hour = 0;
	expected.tm_isdst = -1;

	alt_val = -3600;
	globok = 0;
	checkval(593915381L);
	alt_val = 0;
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - e";

	expected.tm_sec  = 41;
	expected.tm_min  = 29;
	expected.tm_hour = 1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(593918981L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - f";

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 2;
	expected.tm_mday = 23;
	expected.tm_mon  = 2;
	expected.tm_year = 89;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(606625199L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - g";

	expected.tm_sec  = 21;
	expected.tm_min  = 0;
	expected.tm_hour = 4;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(606625221L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - h";

	expected.tm_sec  = 19;
	expected.tm_min  = 29;
	expected.tm_hour = 0;
	expected.tm_mday = 28;
	expected.tm_mon  = 9;
	expected.tm_year = 89;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(625534159L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - i";

	expected.tm_sec  = 41;
	expected.tm_min  = 29;
	expected.tm_hour = 0;
	expected.tm_isdst = -1;

	alt_val = -3600;
	globok = 0;
	checkval(625537781L);
	alt_val = 0;
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as zero-based Julian days) and times - j";

	expected.tm_sec  = 41;
	expected.tm_min  = 29;
	expected.tm_hour = 1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(625541381L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(11);

	DBUG_VOID_RETURN;
}

private	void
test15()
{
	DBUG_ENTER("test15");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

private	void
test16()
{
	int	pathok = 0;

	DBUG_ENTER("test16");

	testcomment = "TZ can state daylight saving start and end dates (given as month, week in month and day in week) - a";
	if (set_tz("LOC0DST,M3.2.0,M10.5.0") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	expected.tm_sec = 59;
	expected.tm_min = 59;
	expected.tm_hour = 1;
	expected.tm_mday = 13;
	expected.tm_mon = 2;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(574221599L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as month, week in month and day in week) - b";

	expected.tm_sec = 21;
	expected.tm_min = 0;
	expected.tm_hour = 3;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(574221621L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as month, week in month and day in week) - c";

	expected.tm_sec = 59;
	expected.tm_min = 59;
	expected.tm_hour = 0;
	expected.tm_mday = 30;
	expected.tm_mon = 9;
	expected.tm_year = 88;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(594172799L);
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as month, week in month and day in week) - d";

	expected.tm_sec = 21;
	expected.tm_min = 0;
	expected.tm_hour = 1;
	expected.tm_isdst = -1;

	alt_val = -3600;
	globok = 0;
	checkval(594176421L);
	alt_val = 0;
	PATH_FUNC_TRACE;

	testcomment = "TZ can state daylight saving start and end dates (given as month, week in month and day in week) - e";

	expected.tm_sec = 21;
	expected.tm_min = 0;
	expected.tm_hour = 2;
	expected.tm_isdst = -1;

	globok = 0;
	checkval(594180021L);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test17()
{
	int	pathok = 0;

	DBUG_ENTER("test17");

	if (set_tz("CUT0") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "mktime() corrects values of tm_sec less than 0";

	observed.tm_sec  = -1;
	observed.tm_min  = 0;
	observed.tm_hour = 0;
	observed.tm_mday = 1;
	observed.tm_mon  = 0;
	observed.tm_year = 71;
	observed.tm_wday = -1;
	observed.tm_yday = -1;
	observed.tm_isdst = 0;

	expected.tm_sec  = 59;
	expected.tm_min  = 59;
	expected.tm_hour = 23;
	expected.tm_mday = 31;
	expected.tm_mon  = 11;
	expected.tm_year = 70;

	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	testcomment = "mktime() corrects values of tm_sec greater than 61";

	observed.tm_sec  = 62;
	observed.tm_min  = 59;
	observed.tm_hour = 23;
	observed.tm_mday = 31;
	observed.tm_mon  = 11;
	observed.tm_year = 86;
	observed.tm_wday = -1;
	observed.tm_yday = -1;
	observed.tm_isdst = 0;

	expected.tm_sec  = 2;
	expected.tm_min  = 0;
	expected.tm_hour = 0;
	expected.tm_mday = 1;
	expected.tm_mon  = 0;
	expected.tm_year = 87;

	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	testcomment = "mktime() corrects values of tm_min less than 0";

	observed.tm_sec  = 0;
	observed.tm_min  = -1;
	observed.tm_hour = 0;
	observed.tm_mday = 1;
	observed.tm_mon  = 0;
	observed.tm_year = 71;
	observed.tm_wday = -1;
	observed.tm_yday = -1;
	observed.tm_isdst = 0;

	expected.tm_sec  = 0;
	expected.tm_min  = 59;
	expected.tm_hour = 23;
	expected.tm_mday = 31;
	expected.tm_mon  = 11;
	expected.tm_year = 70;

	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	testcomment = "mktime() corrects values of tm_min greater than 59";

	observed.tm_sec  = 0;
	observed.tm_min  = 60;
	observed.tm_hour = 23;
	observed.tm_mday = 31;
	observed.tm_mon  = 11;
	observed.tm_year = 86;
	observed.tm_wday = -1;
	observed.tm_yday = -1;
	observed.tm_isdst = 0;

	expected.tm_sec  = 0;
	expected.tm_min  = 0;
	expected.tm_hour = 0;
	expected.tm_mday = 1;
	expected.tm_mon  = 0;
	expected.tm_year = 87;

	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	testcomment = "mktime() corrects values of tm_hour less than 0";

	observed.tm_sec  = 0;
	observed.tm_min  = 0;
	observed.tm_hour = -1;
	observed.tm_mday = 1;
	observed.tm_mon  = 0;
	observed.tm_year = 71;
	observed.tm_wday = -1;
	observed.tm_yday = -1;
	observed.tm_isdst = 0;

	expected.tm_sec  = 0;
	expected.tm_min  = 0;
	expected.tm_hour = 23;
	expected.tm_mday = 31;
	expected.tm_mon  = 11;
	expected.tm_year = 70;

	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	testcomment = "mktime() corrects values of tm_hour greater than 23";

	observed.tm_sec  = 0;
	observed.tm_min  = 0;
	observed.tm_hour = 24;
	observed.tm_mday = 31;
	observed.tm_mon  = 11;
	observed.tm_year = 86;
	observed.tm_wday = -1;
	observed.tm_yday = -1;
	observed.tm_isdst = 0;

	expected.tm_sec  = 0;
	expected.tm_min  = 0;
	expected.tm_hour = 0;
	expected.tm_mday = 1;
	expected.tm_mon  = 0;
	expected.tm_year = 87;

	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	testcomment = "mktime() corrects values of tm_mday less than 1";

	observed.tm_sec  = 0;
	observed.tm_min  = 0;
	observed.tm_hour = 0;
	observed.tm_mday = 0;
	observed.tm_mon  = 0;
	observed.tm_year = 71;
	observed.tm_wday = -1;
	observed.tm_yday = -1;
	observed.tm_isdst = 0;

	expected.tm_sec  = 0;
	expected.tm_min  = 0;
	expected.tm_hour = 0;
	expected.tm_mday = 31;
	expected.tm_mon  = 11;
	expected.tm_year = 70;

	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	testcomment = "mktime() corrects values of tm_mday greater than 31";

	observed.tm_sec  = 0;
	observed.tm_min  = 0;
	observed.tm_hour = 0;
	observed.tm_mday = 32;
	observed.tm_mon  = 11;
	observed.tm_year = 86;
	observed.tm_wday = -1;
	observed.tm_yday = -1;
	observed.tm_isdst = 0;

	expected.tm_sec  = 0;
	expected.tm_min  = 0;
	expected.tm_hour = 0;
	expected.tm_mday = 1;
	expected.tm_mon  = 0;
	expected.tm_year = 87;

	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	testcomment = "mktime() corrects values of tm_mon less than 0";

	observed.tm_sec  = 0;
	observed.tm_min  = 0;
	observed.tm_hour = 0;
	observed.tm_mday = 1;
	observed.tm_mon  = -1;
	observed.tm_year = 71;
	observed.tm_wday = -1;
	observed.tm_yday = -1;
	observed.tm_isdst = 0;

	expected.tm_sec  = 0;
	expected.tm_min  = 0;
	expected.tm_hour = 0;
	expected.tm_mday = 1;
	expected.tm_mon  = 11;
	expected.tm_year = 70;

	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	testcomment = "mktime() corrects values of tm_mon greater than 11";

	observed.tm_sec  = 0;
	observed.tm_min  = 0;
	observed.tm_hour = 0;
	observed.tm_mday = 1;
	observed.tm_mon  = 12;
	observed.tm_year = 86;
	observed.tm_wday = -1;
	observed.tm_yday = -1;
	observed.tm_isdst = 0;

	expected.tm_sec  = 0;
	expected.tm_min  = 0;
	expected.tm_hour = 0;
	expected.tm_mday = 1;
	expected.tm_mon  = 0;
	expected.tm_year = 87;

	globok = 0;
	chkrange();
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(11);
	
	DBUG_VOID_RETURN;
}

private char *
t18_func(isdst)
int isdst;
{
	observed = expected;
	(void) mktime(&observed);

	return tzname[isdst];
}

private void
test18()
{
	DBUG_ENTER("test18");

	expected.tm_sec  = 60;
	expected.tm_min  = 59;
	expected.tm_hour = 23;
	expected.tm_mday = 31;
	expected.tm_mon  = 11;
	expected.tm_year = INT_MAX;
	expected.tm_isdst = 0;

	tztest("mktime()", t18_func);

	DBUG_VOID_RETURN;
}

private void
test19()
{
	int	pathok = 0;

	time_t rval;
	double dval;
	void	(*oldsig)();

	DBUG_ENTER("test19");

	if (set_tz("CUT0") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* determine whether unrepresentable times exist */
	/* (if they do, this will cause overflow, so ignore SIGFPE) */

	dval = (INT_MAX - 69) * 3600.0 * 24.0 * 365.2425;

	oldsig = vsignal(SIGFPE, SIG_IGN);
	rval = INT_MAX - 69;
	rval *= (time_t)3600 * 24;
	rval = 365 * rval + (rval/10000 + 1) * 2425;
	(void) vsignal(SIGFPE, oldsig);

	if (rval < dval)
	{
		DBUG_PRINTF("test19", "unrepresentable times exist");

		expected.tm_sec  = 60;
		expected.tm_min  = 59;
		expected.tm_hour = 23;
		expected.tm_mday = 31;
		expected.tm_mon  = 11;
		expected.tm_year = INT_MAX;
		expected.tm_wday = -1;
		expected.tm_yday = -1;
		expected.tm_isdst = 0;

		observed = expected;
		rval = mktime(&observed);
		if (rval != (time_t)-1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("mktime() returned incorrect value");
			in_rpt("actual value %ld; expected -1", (long)rval);
			in_rpt("values used: TZ=%s; tm_isdst = %d",
				getenv("TZ"), expected.tm_isdst);
			in_rpt("sec = %d min = %d hour = %d mday = %d mon = %d year = %d (INT_MAX)",
				expected.tm_sec,  expected.tm_min, expected.tm_hour,
				expected.tm_mday, expected.tm_mon, expected.tm_year);
		}
		else
			PATH_TRACE;
	}
	else
	{
		DBUG_PRINTF("test19", "all times are representable");

		expected.tm_sec  = 0;
		expected.tm_min  = 0;
		expected.tm_hour = 0;
		expected.tm_mday = 1;
		expected.tm_mon  = 0;
		expected.tm_year = INT_MAX;
		expected.tm_wday = -1;
		expected.tm_yday = -1;
		expected.tm_isdst = 0;

		observed = expected;
		rval = mktime(&observed);
		if (rval == (time_t)-1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("mktime() returned incorrect value");
			in_rpt("actual value -1; expected != -1");
			in_rpt("values used: TZ=%s; tm_isdst = %d",
				getenv("TZ"), expected.tm_isdst);
			in_rpt("sec = %d min = %d hour = %d mday = %d mon = %d year = %d (INT_MAX)",
				expected.tm_sec,  expected.tm_min, expected.tm_hour,
				expected.tm_mday, expected.tm_mon, expected.tm_year);
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);
	
	DBUG_VOID_RETURN;
}

private void
test20()
{
	int	pathok = 0;

	DBUG_ENTER("test20");

	if (set_tz("CUT0") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	expected.tm_sec  = 34;
	expected.tm_min  = 29;
	expected.tm_hour = 11;
	expected.tm_mday = 23;
	expected.tm_mon  = 9;
	expected.tm_year = 86;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	(void) mktime(&expected);

	if (set_tz("ABC1") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "when TZ changes, mktime() uses the new value";

	expected.tm_sec  = 34;
	expected.tm_min  = 29;
	expected.tm_hour = 11;
	expected.tm_mday = 23;
	expected.tm_mon  = 9;
	expected.tm_year = 86;
	expected.tm_wday = -1;
	expected.tm_yday = -1;
	expected.tm_isdst = 0;

	globok = 0;
	checkval(530454574L);  
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private int
set_tz(tzval)
char *tzval;
{
	char	*tz;
	static char	buf[2 * PCTS_TZNAME_MAX + 5];
	int	pathok = 0;

	DBUG_ENTER("set_tz");

	(void) strcpy(buf, "TZ=");
	(void) strcat(buf, tzval);
	(void) vputenv(&environ, buf);
	tz = getenv("TZ");
	if (tz == NULL || strcmp(tz, tzval) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not put \"TZ=%s\" in environment",
			psl_to_asc(tzval, 0));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	DBUG_RETURN(0);
}

private void
checkval(tval)
long tval;
{
	time_t	rval;
	int	lastfail = testfail;
	struct tm *loctm;
	int	pathok = 0;

	DBUG_ENTER("checkval");

	commented = 0;

	observed = expected;
	rval = mktime(&observed);
	if ((rval < tval || rval > tval + MAX_LEAP_SECS) &&
		!(alt_val != 0 && rval >= tval + alt_val &&
				  rval <= tval + alt_val + MAX_LEAP_SECS))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (commented++ == 0 && testcomment != NULL)
			in_rpt("%s", testcomment);
		in_rpt("mktime() returned incorrect value");
		in_rpt("actual value %ld; expected between %ld and %ld",
			(long)rval, tval, tval + MAX_LEAP_SECS);
		if (alt_val != 0)
			in_rpt("OR between %ld and %ld", tval + alt_val,
				tval + alt_val + MAX_LEAP_SECS);
	}
	else
	{
		PATH_TRACE;
		/* check structure has been set correctly for time returned */

		loctm = localtime(&rval);
		globok = 0;
		chkfield("tm_sec",   observed.tm_sec,   loctm->tm_sec);
		PATH_FUNC_TRACE;
		globok = 0;
		chkfield("tm_min",   observed.tm_min,   loctm->tm_min);
		PATH_FUNC_TRACE;
		globok = 0;
		chkfield("tm_hour",  observed.tm_hour,  loctm->tm_hour);
		PATH_FUNC_TRACE;
		globok = 0;
		chkfield("tm_mday",  observed.tm_mday,  loctm->tm_mday);
		PATH_FUNC_TRACE;
		globok = 0;
		chkfield("tm_mon",   observed.tm_mon,   loctm->tm_mon);
		PATH_FUNC_TRACE;
		globok = 0;
		chkfield("tm_year",  observed.tm_year,  loctm->tm_year);
		PATH_FUNC_TRACE;
		globok = 0;
		chkfield("tm_wday",  observed.tm_wday,  loctm->tm_wday);
		PATH_FUNC_TRACE;
		globok = 0;
		chkfield("tm_yday",  observed.tm_yday,  loctm->tm_yday);
		PATH_FUNC_TRACE;
		globok = 0;
		chkfield("tm_isdst", observed.tm_isdst, loctm->tm_isdst);
		PATH_FUNC_TRACE;
	}

	if (testfail > lastfail)
	{
	    in_rpt("values used: TZ=%s; tm_isdst = %d",
		psl_to_asc(getenv("TZ"), 0), expected.tm_isdst);
	    in_rpt("sec = %d min = %d hour = %d mday = %d mon = %d year = %d",
		expected.tm_sec,  expected.tm_min, expected.tm_hour,
		expected.tm_mday, expected.tm_mon, expected.tm_year);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(11);

	DBUG_VOID_RETURN;
}

private void
chkrange()
{
	int	lastfail = testfail;
	int	pathok = 0;

	DBUG_ENTER("chkrange");

	commented = 0;

	(void) mktime(&observed);

	globok = 0;
	chkfield("tm_sec",  observed.tm_sec,  expected.tm_sec);
	PATH_FUNC_TRACE;
	globok = 0;
	chkfield("tm_min",  observed.tm_min,  expected.tm_min);
	PATH_FUNC_TRACE;
	globok = 0;
	chkfield("tm_hour", observed.tm_hour, expected.tm_hour);
	PATH_FUNC_TRACE;
	globok = 0;
	chkfield("tm_mday", observed.tm_mday, expected.tm_mday);
	PATH_FUNC_TRACE;
	globok = 0;
	chkfield("tm_mon",  observed.tm_mon,  expected.tm_mon);
	PATH_FUNC_TRACE;
	globok = 0;
	chkfield("tm_year", observed.tm_year, expected.tm_year);
	PATH_FUNC_TRACE;

	if (observed.tm_wday < 0 || observed.tm_wday > 6)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);	
		if (commented++ == 0 && testcomment != NULL)
			in_rpt("%s", testcomment);
		in_rpt("mktime() set tm_wday to %d; expected range 0 to 6",
			observed.tm_wday);
	}
	else
		PATH_TRACE;

	if (observed.tm_yday < 0 || observed.tm_yday > 365)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);	
		if (commented++ == 0 && testcomment != NULL)
			in_rpt("%s", testcomment);
		in_rpt("mktime() set tm_yday to %d; expected range 0 to 365",
			observed.tm_yday);
	}
	else
		PATH_TRACE;

	if (testfail > lastfail)
	{
	    in_rpt("values used: TZ=%s; tm_isdst = %d",
		psl_to_asc(getenv("TZ"), 0), expected.tm_isdst);
	    in_rpt("sec = %d min = %d hour = %d mday = %d mon = %d year = %d",
		expected.tm_sec,  expected.tm_min, expected.tm_hour,
		expected.tm_mday, expected.tm_mon, expected.tm_year);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(9);

	DBUG_VOID_RETURN;
}

private void
chkfield(name, obs, exp)
char *name;
int obs, exp;
{
	globok = 0;

	if ( obs != exp )
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);	
		if (commented++ == 0 && testcomment != NULL)
			in_rpt("%s", testcomment);
		in_rpt("mktime() set %s to %d instead of %d", name, obs, exp);
	}
	else
		globok = 1;
}
