/*
*      SCCS:  @(#)  ANSI.os/time/strftime/strftime.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/time/strftime - T.strftime Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strftime

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the strftime() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Bob Campbell, Hoskyns
DATE CREATED:	August 1988

MODIFIED:	Geoff Clare, 20 Dec 1988
			Set struct tm explicitly instead of using
			gmtime() so leap-seconds don't cause problems

		Geoff Clare, 9 Nov 1989
			More thorough testing of allowed time zone names

		Dave Cater, 18 Dec 1989
			Only check alphabetic characters in allowed zone names

		Geoff Clare, 23 Jan 1990
			Use [0, UCHAR_MAX] instead of [CHAR_MIN, CHAR_MAX]
			to avoid passing invalid values to isalpha()

		Geoff Clare, 2 May 1990
			Use TZNAME_MAX in long timezone name test.
			Return to testing all allowed characters in zone names

		Peter Lee, Unisoft Ltd.,
			Tue Jul 10 13:35:44 BST 1990
			Added resilience code.
		
		Geoff Clare, 4 Oct 1990
			Set currlang in ch_t3().

		Geoff Clare, 20 June 1991
			Split out X/Open extensions into strftime_X.

		Geoff Clare, 14 Jan 1992
			Use new tztest() routine for %Z testing.

		Geoff Clare, 22 May 1992
			Added tests 9 and 10.

		Geoff Clare, 29 May 1992
			Don't use literal language names.

		Geoff Clare, 30 March 1995
			New encodings for VSX4L1 and VSX4L2.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<ctype.h>
#include	<errno.h>
#include	<time.h>
#include	<limits.h>
#include	<nlsinfo.h>
#include	<nlstest.h>
#include        <locale.h>
#include	<dbug.h>
#include	<report.h>
#include	<pcts_max.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strftime
#endif


#define 	NO_TESTS	 10		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)
#define		NUMDAYS	         7
#define		NUMMONTHS	12

extern	int	cppair();
extern	char	*psl_to_asc();
extern	char	*errname();
extern	char	*pslanginfo();
extern  char    **environ;
extern	void	tztest();

int	no_tests = NO_TESTS;

private int	testfail;
private char	*testcomment;
private char	*currlang;
private int     langnum;
private	struct tm	tmbuf;

private	int	dayitems[NUMDAYS] = {	
					VSX_DAY_1, VSX_DAY_2, VSX_DAY_3, VSX_DAY_4,
					VSX_DAY_5, VSX_DAY_6, VSX_DAY_7,
				};			

private	int	abdayitems[NUMDAYS] = {	
					VSX_ABDAY_1, VSX_ABDAY_2, VSX_ABDAY_3, VSX_ABDAY_4,
					VSX_ABDAY_5, VSX_ABDAY_6, VSX_ABDAY_7,
				};

private	int	monitems[NUMMONTHS] = {	
					VSX_MON_1, VSX_MON_2, VSX_MON_3, VSX_MON_4,
					VSX_MON_5, VSX_MON_6, VSX_MON_7, VSX_MON_8,
					VSX_MON_9, VSX_MON_10, VSX_MON_11, VSX_MON_12,
				};

private	int	abmonitems[NUMMONTHS] = {	
					VSX_ABMON_1, VSX_ABMON_2, VSX_ABMON_3, VSX_ABMON_4,
					VSX_ABMON_5, VSX_ABMON_6, VSX_ABMON_7, VSX_ABMON_8,
					VSX_ABMON_9, VSX_ABMON_10, VSX_ABMON_11, VSX_ABMON_12,

				};


private char	*daystr[7] = {
	"0 ", "1 ", "2 ", "3 ", "4 ", "5 ", "6 "
};

private char	*monstr[12] = {
	"01 ", "02 ", "03 ", "04 ", "05 ", "06 ",
	"07 ", "08 ", "09 ", "10 ", "11 ", "12 "
};

private long    dmon[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };    
private void	test1();
private	void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	ch_t9();
private void	ch_t10();

private	int	set_tz();
private void	chkstrftime();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	NULL, 0
};


private void
test1()
{
	DBUG_ENTER("test1");

	(void)  cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{

	char	*expstr;
	char	*format;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 23;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 4;
	tmbuf.tm_yday = 295;
	tmbuf.tm_isdst = 0;

	testfail = 0;
	currlang = "";
	langnum = 0;

	expstr = "%";
	format = "%%";
	testcomment = "the format characters %% are converted to %";
	globok = 0;
	chkstrftime(format, expstr);
	PATH_FUNC_TRACE;

	expstr = "23:10:86";
	format = "%d:%m:%y";
	testcomment = "the day and year are printed according to the format given";
	globok = 0;
	chkstrftime(format, expstr);
	PATH_FUNC_TRACE;

	expstr = "1986";
	format = "%Y";
	testcomment = "the year is printed according to the %Y format";
	globok = 0;
	chkstrftime(format, expstr);
	PATH_FUNC_TRACE;

	expstr = "12/29/34";
	format = "%H/%M/%S";
	testcomment = "the hour, min and sec are printed according to the format given";
	globok = 0;
	chkstrftime(format, expstr);
	PATH_FUNC_TRACE;
	
	expstr = "12";   /* test I with 12 o'clock time */
	format = "%I";
	testcomment = "the hour is printed in 12 hour format according to the %Iformat letter";
	globok = 0;
	chkstrftime(format, expstr);
	PATH_FUNC_TRACE;


	tmbuf.tm_hour = 13;
	expstr = "01";   /* test I with 01 O'clock time */
	format = "%I";
	testcomment = "the hour is printed in 12 hour format according to the %I format letter";
	globok = 0;
	chkstrftime(format, expstr);
	PATH_FUNC_TRACE;


	expstr = "296 4";
	format = "%j %w";
	testcomment = "the year day and week day are printed for formats %j,%w";
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	expstr = "42";
	format = "%U";
	testcomment = "the week no. for the year with first day Sun is  printed according to the format letter %U";
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;


	/* incr time by 4 days to a Monday */
	tmbuf.tm_mday = 27;
	tmbuf.tm_wday = 1;
	tmbuf.tm_yday = 299;

	expstr = "43";
	format = "%W";
	testcomment = "the week no. for the year with first day Mon is  printed according to the format letter %W";
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	testcomment = "format letter %Z returns the appropriate time zone name";
	expstr = "CUT";
	format = "%Z";
	if (set_tz("CUT0") == 0)
	{
		PATH_TRACE;
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
	}

	if(!testfail)
		PATH_XS_RPT(11);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	i;
	char	expstr[30];
	char	ampm[20];
	char	*format;
	char    *day[7];
	char	*abday[7];
	char    *month[12];
	char	*abmon[12];
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	tmbuf.tm_sec  = 35;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 19;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 0;
	tmbuf.tm_yday = 291;
	tmbuf.tm_isdst = 0;

	testfail = 0;

	if (setlocale(LC_ALL, INFO1LANG) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO1LANG);
		in_rpt("Check installation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	currlang = INFO1LANG;
	langnum =  VALINFO1LANG;

	testcomment = "the abbreviated day names for the language are used";
	format = "%w %a";
	for (i=0; i<NUMDAYS; i++)
	{
		abday[i] = pslanginfo(abdayitems[i], langnum);
		(void) strcpy(expstr, daystr[i]);
		(void) strcat(expstr, abday[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mday++;
		tmbuf.tm_wday++;
		tmbuf.tm_yday++;
	}

	tmbuf.tm_mday = 19;
	tmbuf.tm_wday = 0;
	tmbuf.tm_yday = 291;

	testcomment = "the full day names for the language are used";
	format = "%w %A";
	for (i=0; i<NUMDAYS; i++)
	{
		day[i] = pslanginfo(dayitems[i], langnum);
		(void) strcpy(expstr, daystr[i]);
		(void) strcat(expstr, day[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mday++;
		tmbuf.tm_wday++;
		tmbuf.tm_yday++;
	}
	
	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 17;
	tmbuf.tm_mon  = 0;
	tmbuf.tm_year = 87;
	tmbuf.tm_wday = 6;
	tmbuf.tm_yday = 16;
	tmbuf.tm_isdst = 0;

	testcomment = "the abbreviated month names for the language are used    (format letter %b)";

	format = "%m %b";
	for (i=0; i<NUMMONTHS; i++)
	{
		abmon[i] = pslanginfo(abmonitems[i], langnum);
		(void) strcpy(expstr, monstr[i]);
		(void) strcat(expstr, abmon[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mon++;
		tmbuf.tm_wday += dmon[i];
		tmbuf.tm_wday %= NUMDAYS;
		tmbuf.tm_yday += dmon[i];
	}
	

	tmbuf.tm_mon  = 0;
	tmbuf.tm_wday = 6;
	tmbuf.tm_yday = 16;

	testcomment = "the full month names for the language are used ";
	format = "%m %B";
	for (i=0; i<NUMMONTHS; i++)
	{
		month[i] = pslanginfo(monitems[i], langnum);
		(void) strcpy(expstr, monstr[i]);
		(void) strcat(expstr, month[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mon++;
		tmbuf.tm_wday += dmon[i];
		tmbuf.tm_wday %= NUMDAYS;
		tmbuf.tm_yday += dmon[i];
	}

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 23;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 4;
	tmbuf.tm_yday = 295;
	tmbuf.tm_isdst = 0;

	testcomment = "the D_T_FMT for the language is used";

	format = "%c";
	/* sets expstr to "12.29.34 NM 10/23/86" */
	(void) strcat(strcat(strcpy(expstr,
		"12.29.34 "), pslanginfo(VSX_PM_STR, langnum)), " 10/23/86");
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	testcomment = "the D_FMT for the language is used";
	format = "%x";
	(void) strcpy(expstr, "10/23/86");
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	testcomment = "the T_FMT for the language is used";
	format = "%X";
	/* sets expstr to "12.29.34 NM" */
	(void) strcat(strcpy(expstr, "12.29.34 "), pslanginfo(VSX_PM_STR, langnum));
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	testcomment = "the AM/PM string for the language is being used";
	format = "%H:%M:%S%p";
	(void) strcpy(ampm, "12:29:34"); 
	(void) strcat(ampm, pslanginfo(VSX_PM_STR, langnum));
	globok = 0;
	chkstrftime (format, &ampm[0]);
	PATH_FUNC_TRACE;

	/* End of tests info1lang set with LC_ALL */



	if (setlocale(LC_TIME, INFO2LANG) == (char *) NULL  )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available",INFO2LANG);
		in_rpt("Check installation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	currlang = INFO2LANG;
	langnum  = VALINFO2LANG;

	tmbuf.tm_sec  = 35;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 19;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 0;
	tmbuf.tm_yday = 291;
	tmbuf.tm_isdst = 0;

	testcomment = "the abbreviated  day  names for the language are used";

	format = "%w %a";
	for (i=0; i<NUMDAYS; i++)
	{
		abday[i] = pslanginfo(abdayitems[i], langnum);
		(void) strcpy(expstr, daystr[i]);
		(void) strcat(expstr, abday[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mday++;
		tmbuf.tm_wday++;
		tmbuf.tm_yday++;
	}

	tmbuf.tm_mday = 19;
	tmbuf.tm_wday = 0;
	tmbuf.tm_yday = 291;

	testcomment = "the full day names for the language are used";

	format = "%w %A";
	for (i=0; i<NUMDAYS; i++)
	{
		day[i] = pslanginfo(dayitems[i], langnum);
		(void) strcpy(expstr, daystr[i]);
		(void) strcat(expstr, day[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mday++;
		tmbuf.tm_wday++;
		tmbuf.tm_yday++;
	}
	
	testcomment = "the abbreviated month names for the language are used    (format letter %b)";

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 17;
	tmbuf.tm_mon  = 0;
	tmbuf.tm_year = 87;
	tmbuf.tm_wday = 6;
	tmbuf.tm_yday = 16;
	tmbuf.tm_isdst = 0;

	format = "%m %b";
	for (i=0; i<NUMMONTHS; i++)
	{
		abmon[i] = pslanginfo(abmonitems[i], langnum);
		(void) strcpy(expstr, monstr[i]);
		(void) strcat(expstr, abmon[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mon++;
		tmbuf.tm_wday += dmon[i];
		tmbuf.tm_wday %= NUMDAYS;
		tmbuf.tm_yday += dmon[i];
	}
	

	tmbuf.tm_mon  = 0;
	tmbuf.tm_wday = 6;
	tmbuf.tm_yday = 16;

	testcomment = "the full month names for the language are used ";

	format = "%m %B";
	for (i=0; i<NUMMONTHS; i++)
	{
		month[i] = pslanginfo(monitems[i], langnum);
		(void) strcpy(expstr, monstr[i]);
		(void) strcat(expstr, month[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mon++;
		tmbuf.tm_wday += dmon[i];
		tmbuf.tm_wday %= NUMDAYS;
		tmbuf.tm_yday += dmon[i];
	}

	testcomment = "the D_T_FMT for the language is used";

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 23;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 4;
	tmbuf.tm_yday = 295;
	tmbuf.tm_isdst = 0;

	format = "%c";
	(void) strcpy(expstr, "23 10 1986 12:29:34");
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	testcomment = "the D_FMT for the language is used";
	format = "%x";
	(void) strcpy(expstr, "23 10 1986");
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	testcomment = "the T_FMT for the language is used";
	format = "%X";
	(void) strcpy(expstr, "12:29:34");
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	tmbuf.tm_hour = 0;

	testcomment = "the AM/PM string for the language is being used";
	format = "%H:%M:%S%p";
	(void) strcpy(ampm, "00:29:34"); 
	(void) strcat(ampm, pslanginfo(VSX_AM_STR, langnum));
	globok = 0;
	chkstrftime (format, &ampm[0]);
	PATH_FUNC_TRACE;


	if (!testfail)
		PATH_XS_RPT(1 + 2 * NUMDAYS + 2 * NUMMONTHS +
			5 + 2 * NUMDAYS + 2 * NUMMONTHS + 4);

	DBUG_VOID_RETURN;
}

private void 
test3()
{
	DBUG_ENTER("test3");

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int	i;
	char	expstr[30];
	char	*ampmptr;
	char	*format;
	char    *day[7];
	char	*abday[7];
	char	*month[12];	
	char	*abmon[12];
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	testfail = 0;
	currlang = "";
	langnum  = 0;

	tmbuf.tm_sec  = 35;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 19;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 0;
	tmbuf.tm_yday = 291;
	tmbuf.tm_isdst = 0;

	testcomment = "the abbreviated day names for the language are used";

	format = "%w %a";
	for (i=0; i<NUMDAYS; i++)
	{
		abday[i] = pslanginfo(abdayitems[i], langnum);
		(void) strcpy(expstr, daystr[i]);
		(void) strcat(expstr, abday[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mday++;
		tmbuf.tm_wday++;
		tmbuf.tm_yday++;
	}

	tmbuf.tm_mday = 19;
	tmbuf.tm_wday = 0;
	tmbuf.tm_yday = 291;

	testcomment = "the full day names for the language are used";

	format = "%w %A";
	for (i=0; i<NUMDAYS; i++)
	{
		day[i] = pslanginfo(dayitems[i], langnum);
		(void) strcpy(expstr, daystr[i]);
		(void) strcat(expstr, day[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mday++;
		tmbuf.tm_wday++;
		tmbuf.tm_yday++;
	}
	
	testcomment = "the abbreviated month names for the language are used    (format letter %b)";

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 17;
	tmbuf.tm_mon  = 0;
	tmbuf.tm_year = 87;
	tmbuf.tm_wday = 6;
	tmbuf.tm_yday = 16;
	tmbuf.tm_isdst = 0;

	format = "%m %b";
	for (i=0; i<NUMMONTHS; i++)
	{
		abmon[i] = pslanginfo(abmonitems[i], langnum);
		(void) strcpy(expstr, monstr[i]);
		(void) strcat(expstr, abmon[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mon++;
		tmbuf.tm_wday += dmon[i];
		tmbuf.tm_wday %= NUMDAYS;
		tmbuf.tm_yday += dmon[i];
	}

	tmbuf.tm_mon  = 0;
	tmbuf.tm_wday = 6;
	tmbuf.tm_yday = 16;

	testcomment = "the full month names for the language are used ";

	format = "%m %B";
	for (i=0; i<NUMMONTHS; i++)
	{
		month[i] = pslanginfo(monitems[i], langnum);
		(void) strcpy(expstr, monstr[i]);
		(void) strcat(expstr, month[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mon++;
		tmbuf.tm_wday += dmon[i];
		tmbuf.tm_wday %= NUMDAYS;
		tmbuf.tm_yday += dmon[i];
	}

	testcomment = "the D_T_FMT for the language is used";

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 23;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 4;
	tmbuf.tm_yday = 295;
	tmbuf.tm_isdst = 0;

	format = "%c";
	(void) strcpy(expstr, "Thu Oct 23 12:29:34 1986");
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	testcomment = "the D_FMT for the language is used";
	format = "%x";
	(void) strcpy(expstr, "10/23/86");
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	testcomment = "the T_FMT for the language is used";
	format = "%X";
	(void) strcpy(expstr, "12:29:34");
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	testcomment = "the AM/PM string for the language is being used";
	format = "%H:%M:%S%p";
	(void) strcpy(expstr, "12:29:34"); 
	ampmptr = pslanginfo(VSX_PM_STR, langnum);
	(void) strcat(expstr, ampmptr);
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	tmbuf.tm_hour = 0;

	testcomment = "the AM/PM string for the language is being used";

	format = "%H:%M:%S%p";
	(void) strcpy(expstr, "00:29:34"); 
	ampmptr = pslanginfo(VSX_AM_STR, langnum);
	(void) strcat(expstr, ampmptr);
	globok = 0;
	chkstrftime (format, expstr);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2 * NUMDAYS + 2 * NUMMONTHS + 5);
	
	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	(void)  cppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	char	*expstr;
	char	*format;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 23;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 4;
	tmbuf.tm_yday = 295;
	tmbuf.tm_isdst = 0;

	testfail = 0;
	currlang = "";
	langnum = 0;

	if (set_tz("std0") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "with TZ=std0 %Z converts to std when tm_isdst is set to 0";
	expstr = "std";
	format = "%Z";

	globok = 0;
	chkstrftime(format, expstr);
	PATH_FUNC_TRACE;

	if(!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	DBUG_ENTER("test5");

	(void)  cppair(ch_t5, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private char *
t5_func(isdst)
int isdst;
{
	static char str[PCTS_TZNAME_MAX+10];

	tmbuf.tm_isdst = isdst;

	(void) strftime(str, sizeof(str), "%Z", &tmbuf);

	return str;
}

private void
ch_t5()
{
	DBUG_ENTER("ch_t5");

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 23;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 4;
	tmbuf.tm_yday = 295;

	tztest("strftime(%Z)", t5_func);

	DBUG_VOID_RETURN;
}


private void
test6()
{
	char	str[50];
	int	pathok = 0;

	DBUG_ENTER("test6");

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 23;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 4;
	tmbuf.tm_yday = 295;
	tmbuf.tm_isdst = 0;

	if (strftime(str, (size_t)8, "1234567890", &tmbuf) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("strftime(str, 8, \"1234567890\", &tmbuf) placed %ld chars into str",
			(long)strlen(str)+1);
		in_rpt("expected strftime() to return zero");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test7()
{
	size_t	rval;
	char	str[50];
	int	pathok = 0;

	DBUG_ENTER("test7");

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 23;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 4;
	tmbuf.tm_yday = 295;
	tmbuf.tm_isdst = 0;

	rval = strftime(str, (size_t)50, "%m/%d/%y %H:%M:%S", &tmbuf);

	if (rval != strlen(str))
	{
		xx_rpt(FAILURE);
		in_rpt("strftime() did not return correct value");
		in_rpt("actual return value: %ld", (long)rval);
		in_rpt("expected value (length of formatted string): %ld",
			(long)strlen(str));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test8()
{
	size_t	rval;
	char	str[50];
	int	pathok = 0;

	DBUG_ENTER("test8");

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 23;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 4;
	tmbuf.tm_yday = 295;
	tmbuf.tm_isdst = 0;

	rval = strftime(str, (size_t)10, "1234567890", &tmbuf);

	if (rval != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("strftime(str, 10, \"1234567890\", &tmbuf) did not return correct value");
		in_rpt("actual value: %ld, expected 0", (long)rval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test9()
{
	DBUG_ENTER("test9");

	(void)  cppair(ch_t9, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private char *
t9_func(isdst)
int isdst;
{
	static char str[10];

	(void) strftime(str, sizeof(str), "abc", &tmbuf);

	return tzname[isdst];
}

private void
ch_t9()
{
	DBUG_ENTER("ch_t9");

	tztest("strftime()", t9_func);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	DBUG_ENTER("test10");

	(void)  cppair(ch_t10, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t10()
{
	char	*expstr;
	char	*format;
	char	str[10];
	int	pathok = 0;

	DBUG_ENTER("ch_t10");

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 23;
	tmbuf.tm_mon  = 9;
	tmbuf.tm_year = 86;
	tmbuf.tm_wday = 4;
	tmbuf.tm_yday = 295;
	tmbuf.tm_isdst = 0;

	if (set_tz("CUT0") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) strftime(str, sizeof(str), "%Z", &tmbuf);

	if (set_tz("ABC1") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "when TZ changes, strftime() uses the new value";

	expstr = "ABC";
	format = "%Z";
	globok = 0;
	chkstrftime(format, expstr);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private int
set_tz(tzval)
char *tzval;
{
	char	*tz;
	static char	buf[2 * PCTS_TZNAME_MAX + 5];
	int	pathok = 0;

	DBUG_ENTER("set_tz");

	(void) strcpy(buf, "TZ=");
	(void) strcat(buf, tzval);
	(void) vputenv(&environ, buf);
	tz = getenv("TZ");
	if (tz == NULL || strcmp(tz, tzval) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not put \"TZ=%s\" in environment",
			psl_to_asc(tzval, 0));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	DBUG_RETURN(0);
}


private void
chkstrftime(formstr, expstr)
char	*formstr, *expstr;
{
	char	str[50];
	size_t	size = 50;

	DBUG_ENTER("chkstrftime");

	globok = 0;

	(void) strftime(str, size, formstr, &tmbuf);

	if (strcmp(str, expstr))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("strftime(str, size, %s, &tmbuf) returned incorrectly formatted string ", formstr);
		in_rpt("%s", testcomment);
		if (*currlang != '\0')
			in_rpt("Current language is %s", currlang);

		in_rpt("Expected string: %s", psl_to_asc(expstr, langnum));
		in_rpt("Actual   string: %s", psl_to_asc(str,langnum));
	}
	else
		globok = 1;

	DBUG_VOID_RETURN;
}
