/*
*      SCCS:  @(#)  ANSI.os/time/strftime_X/strftime_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/time/strftime_X - T.strftime_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strftime_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the X/Open extensions to the strftime() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	20 June 1991 (split out from strftime testset)

MODIFIED:
		Matt Newman, Tue Oct 15 16:46:00 BST 1991
			XPG4 Intergration

		Neil Moses, 27 Apr 1992
			Removed field precision tests.

		Geoff Clare, 29 May 1992
			Don't use literal language names.

		Geoff Clare, 30 March 1995
			New encodings for VSX4L1 and VSX4L2.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<ctype.h>
#include	<errno.h>
#include	<time.h>
#include	<limits.h>
#include	<nlsinfo.h>
#include	<nlstest.h>
#include        <locale.h>
#include	<dbug.h>
#include	<report.h>
#include	<pcts_max.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strftime
#endif


#define 	NO_TESTS	 4		/* number of tests in testset */

#define		WAITTIME	(5 * SPEEDFACTOR)
#define		NUMMONTHS	12
#define		NUMDAYS	         7

extern	int	cppair();
extern	char	*psl_to_asc();
extern	char	*errname();
extern	char	*pslanginfo();
extern  char    **environ;

int	no_tests = NO_TESTS;

private int	testfail;
private char	*currlang;
private int     langnum;
private	struct tm	tmbuf;

private	int	abmonitems[NUMMONTHS] = {	
	VSX_ABMON_1, VSX_ABMON_2, VSX_ABMON_3, VSX_ABMON_4,
	VSX_ABMON_5, VSX_ABMON_6, VSX_ABMON_7, VSX_ABMON_8,
	VSX_ABMON_9, VSX_ABMON_10, VSX_ABMON_11, VSX_ABMON_12,
};


private char	*monstr[12] = {
	"01 ", "02 ", "03 ", "04 ", "05 ", "06 ",
	"07 ", "08 ", "09 ", "10 ", "11 ", "12 "
};

private long    dmon[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

private void	test1(), test3(), test4(), ch_t1(), ch_t3(), ch_t4();
private void	test2();
#if TEST_XPG > 3
private void	ch_t2();
#endif

private void	chkstrftime();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

#define TEST(name,label,child)	\
private void			\
name()				\
{				\
	DBUG_ENTER(label);	\
	(void) cppair(child, NULLFN, WAITTIME, 0);	\
	DBUG_VOID_RETURN;	\
}

TEST(test1,"test1",ch_t1)
#if TEST_XPG > 3
TEST(test2,"test2",ch_t2)
#else
private void test2() { xx_rpt(NOTINUSE); }
#endif /* TEST_XPG > 3 */
TEST(test3,"test3",ch_t3)
TEST(test4,"test4",ch_t4)

#define SETUP_TIME(tp)	\
	(tp)->tm_sec  = 34;	(tp)->tm_min  = 29;	(tp)->tm_hour = 12;\
	(tp)->tm_mday = 23;	(tp)->tm_mon  = 9;	(tp)->tm_year = 86;\
	(tp)->tm_wday = 4;	(tp)->tm_yday = 295;	(tp)->tm_isdst = 0
#define minitest( fstr, estr)	\
	expstr = estr;	format = fstr;	\
	globok = 0;	\
	chkstrftime(format, expstr);	\
	PATH_FUNC_TRACE;

private void
ch_t1()
{

	char	*expstr;
	char	*format;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	SETUP_TIME(&tmbuf);	/* 12:29:34 23/10/86 */

	testfail = 0;
	currlang = "";
	langnum = 0;

	/* the format characters %t, %n are converted to tab, new-line */
	minitest(	"%t%n", "\t\n");

	/* the date is printed as mm/dd/yy for format %D */
	minitest(	"%D%n", "10/23/86\n");

	/* test r with 01 O'clock time */
	/* time is printed in AM/PM notation with the letter %r */
	tmbuf.tm_hour = 13;
	minitest(	"%r%n", "01:29:34 PM\n");

	/* test r with 11 O'clock time */
	/* time is printed in AM/PM notation with the letter %r */
	tmbuf.tm_hour = 11;
	minitest(	"%r%n", "11:29:34 AM\n");

	/* test r with12 O'clock time */
	/* time is printed in AM/PM notation with the letter %r */
	tmbuf.tm_hour = 12;
	minitest(	"%r%n", "12:29:34 PM\n");

	/* the time is printed as hh:mm:ss for format %T */
	minitest(	"%T%n", "12:29:34\n");

	if(!testfail)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

#if TEST_XPG > 3
private void
ch_t2()
{
	char	*expstr;
	char	*format;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	SETUP_TIME(&tmbuf);	/* 12:29:34 23/10/86 */

	testfail = 0;
	currlang = "";
	langnum = 0;

	minitest(	"%R", "12:29");

	if(!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}
#endif /* TEST_XPG > 3 */

private void
ch_t3()
{
	int	i;
	char	expstr[30];
	char	*format;
	char	*abmon[12];
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	testfail = 0;

	if (setlocale(LC_ALL, INFO1LANG) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO1LANG);
		in_rpt("Check installation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	currlang = INFO1LANG;
	langnum =  VALINFO1LANG;


	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 17;
	tmbuf.tm_mon  = 0;
	tmbuf.tm_year = 87;
	tmbuf.tm_wday = 6;
	tmbuf.tm_yday = 16;
	tmbuf.tm_isdst = 0;

	
	/* The abbreviated  month names for the language are used   (format letter %h) */
	format = "%m %h";
	for (i=0; i<NUMMONTHS; i++)
	{
		abmon[i] = pslanginfo(abmonitems[i], langnum);
		(void) strcpy(expstr, monstr[i]);
		(void) strcat(expstr, abmon[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mon++;
		tmbuf.tm_wday += dmon[i];
		tmbuf.tm_wday %= NUMDAYS;
		tmbuf.tm_yday += dmon[i];
	}


	/* End of tests info1lang set with LC_ALL */



	if (setlocale(LC_TIME, INFO2LANG) == (char *) NULL  )
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available",INFO2LANG);
		in_rpt("Check installation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	currlang = INFO2LANG;
	langnum  = VALINFO2LANG;

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 17;
	tmbuf.tm_mon  = 0;
	tmbuf.tm_year = 87;
	tmbuf.tm_wday = 6;
	tmbuf.tm_yday = 16;
	tmbuf.tm_isdst = 0;

	
	/* the abbreviated  month names for the language are used   (format letter %h) */

	format = "%m %h";
	for (i=0; i<NUMMONTHS; i++)
	{
		abmon[i] = pslanginfo(abmonitems[i], langnum);
		(void) strcpy(expstr, monstr[i]);
		(void) strcat(expstr, abmon[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mon++;
		tmbuf.tm_wday += dmon[i];
		tmbuf.tm_wday %= NUMDAYS;
		tmbuf.tm_yday += dmon[i];
	}

	if (!testfail)
		PATH_XS_RPT(2 * (1 + NUMMONTHS));

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	i;
	char	expstr[30];
	char	*format;
	char	*abmon[12];
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	testfail = 0;
	currlang = "";
	langnum  = 0;

	tmbuf.tm_sec  = 34;
	tmbuf.tm_min  = 29;
	tmbuf.tm_hour = 12;
	tmbuf.tm_mday = 17;
	tmbuf.tm_mon  = 0;
	tmbuf.tm_year = 87;
	tmbuf.tm_wday = 6;
	tmbuf.tm_yday = 16;
	tmbuf.tm_isdst = 0;

	/* the abbreviated  month names for the language are used   (format letter %h) */
	
	format = "%m %h";
	for (i=0; i<NUMMONTHS; i++)
	{
		abmon[i] = pslanginfo(abmonitems[i], langnum);
		(void) strcpy(expstr, monstr[i]);
		(void) strcat(expstr, abmon[i]);
		globok = 0;
		chkstrftime(format, expstr);
		PATH_FUNC_TRACE;
		tmbuf.tm_mon++;
		tmbuf.tm_wday += dmon[i];
		tmbuf.tm_wday %= NUMDAYS;
		tmbuf.tm_yday += dmon[i];
	}

	if (!testfail)
		PATH_XS_RPT(NUMMONTHS);
	
	DBUG_VOID_RETURN;
}


private void
chkstrftime(formstr, expstr)
char	*formstr, *expstr;
{
	char	str[50];
	size_t	size = 50;

	DBUG_ENTER("chkstrftime");

	globok = 0;

	(void) strftime(str, size, formstr, &tmbuf);

	if (strcmp(str, expstr))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("strftime(str, size, %s, &tmbuf) returned incorrectly formatted string ", formstr);
		if (*currlang != '\0')
			in_rpt("Current language is %s", currlang);

		in_rpt("Expected string: %s", psl_to_asc(expstr, langnum));
		in_rpt("Actual   string: %s", psl_to_asc(str,langnum));
	}
	else
		globok = 1;

	DBUG_VOID_RETURN;
}
