/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/aio/aio_cancel/aio_cancel.c
*
* Modifications:
* $Log: aio_cancel.c,v $
* Revision 1.2  2001/10/02 13:25:12  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:13  cyeoh
* Adds files for lsb-os test suite
*
*
*/


#include	<vsrt/vsrt.h>
#include	<aio.h>
#include	<limits.h>
#include	<termios.h>
#include	<setjmp.h>
#include	<setprv.h>

#define 	NO_TESTS	8	/* number of tests in testset */

#ifdef UNDEF_MACROS
#undef aio_cancel
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();

int	no_tests = NO_TESTS;

private	int	tb_aio_cancel();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private	int	dotest1read();
private	int	dotest1write();
private	int	dotest2read();
private	int	dotest2write();
private	int	dotest4read();
private	int	dotest4write();
private	int	dotest5read();
private	int	dotest5write();

int	supports_aio_cancel;
int	supports_aio_read;
int	supports_aio_write;
int	supports_aio_fsync;
int	supports_aio_error;
int	supports_aio_return;

/*determine if a type of file is to be tried. set in startup*/
/*pipe support is determined dynamically*/
int	vsrt_aio_test_with_tty = 0;
int	vsrt_aio_test_with_file = 0;
int	vsrt_aio_test_with_pty = 0;

/*each piece of a test increments on of these. must be zeroed at test start*/
int	vsrt_aio_test_pass;
int	vsrt_aio_test_unsup;
int	vsrt_aio_test_fail;

/*instances of things to test on from tetexec, loaded in startup*/
char	vsrt_aio_tty1[128];
char	vsrt_aio_tty2[128];
char	vsrt_aio_ptym[128];
char	vsrt_aio_testfile[4096];

void atstart(void);
void atstart(void)
{
	char	*str;

	vsrt_check_support("AIO_CANCEL", &supports_aio_cancel);
	vsrt_check_support("AIO_ERROR", &supports_aio_error);
	vsrt_check_support("AIO_RETURN", &supports_aio_return);
	vsrt_check_support("AIO_READ", &supports_aio_read);
	vsrt_check_support("AIO_WRITE", &supports_aio_write);
	vsrt_check_support("AIO_FSYNC", &supports_aio_fsync);

	/*get config info and determine what sorts of files to test with*/
	str = tet_getvar("VSRT_FILE_ASYNC_IO");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_file = 1;
		strcpy(vsrt_aio_testfile, str);
	}

	str = tet_getvar("VSRT_TERMIOS_TTY");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_tty = 1;
		strcpy(vsrt_aio_tty1, str);
		str = tet_getvar("VSRT_TERMIOS_LOOP");
		if ((str != NULL) && (strlen(str) != 0))
			strcpy(vsrt_aio_tty2, str);
		else
			vsrt_aio_test_with_tty = 0;
	}

	str = tet_getvar("VSRT_MASTER_PTY");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_pty = 1;
		strcpy(vsrt_aio_ptym, str);
	}
}

void atend(void);
void atend(void)
{
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = atend;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	NULL, 0
};

int	aio_max = 0;
long	aio_maxl = 0;

int	got_sig = 0;

private	void	handler5();
private	void	handler5(signo)
int	signo;
{
	got_sig++;
}

private void rt_handler7();
private void rt_handler7(signo, info, context)
int	signo;
siginfo_t	*info;
void	*context;
{
	got_sig++;
}

unsigned char	got_requests[50];

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the aio_cancel() function as described in
System Interfaces and Headers, Issue 5, and there are
one or more asynchronous I/O operations which the
implementation defines as cancelable:
	A call to aio_cancel() when aiocbp is not NULL shall
	cancel the the corresponding asynchronous I/O request
	against fildes.
*/

private void test1()
{
	int	terrors;

	DBUG_ENTER("test1");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_cancel) {
		in_rpt("aio_cancel not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 50)
		aio_max = 50;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t1", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest1read);
	vsrt_aio_test_allbutfile_write(dotest1write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest1read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	int	cancelno;

	DBUG_ENTER("dotest1read");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*2)+1)) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1read","kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1read","ensure they are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	if (aio_max < 2)
		cancelno = 0;
	else
		cancelno = aio_max-2;

	retval2 = tb_aio_cancel(readfd, &paiocbs[cancelno]);
	/*may not be cancelled, this is OK (UNSUPPORTED)*/
	if (retval2 == AIO_NOTCANCELED) {
		in_rpt("aio_cancel returned AIO_NOTCANCELED for aio_read request");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(1);
	}
	/*can't be all done however*/
	if (retval2 == AIO_ALLDONE) {
		in_rpt("aio_cancel returned AIO_ALLDONE for aio_read request");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 == -1) {
		vsrt_sysfail("aio_cancel for aio_read request");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 != AIO_CANCELED) {
		in_rpt("aio_cancel returned %d for aio_read request, expected AIO_CANCELED", retval2);
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1read","wait for the cancelled one to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if (aio_error(&paiocbs[cancelno]) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("cancelled aio_read request still in progress");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1read","check its error and return status");
	retval2 = aio_error(&paiocbs[cancelno]);
	if (retval2 == -1) {
		err = errno;
		in_rpt("aio_error failed for canceled aio_read request, errno = %d (%s)", err, errname(err));
		errors++;
	} else
		if (retval2 != ECANCELED) {
			in_rpt("Error status for canceled aio_read request is %d (%s), expected ECANCELED", retval2, errname(retval2));
			errors++;
		}
	retval = aio_return(&paiocbs[cancelno]);
	if (retval != -1) {
		err = errno;
		in_rpt("aio_return returned %d for canceled aio_read request, expected -1", retval);
		errors++;
	}

	PATH_TRACE;

	DBUG_PRINTF("t1read","write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "%3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;

	DBUG_PRINTF("t1read","wait for the others to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if ((i != cancelno) && (aio_error(&paiocbs[i]) == EINPROGRESS)) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if ((i != cancelno) && (aio_error(&paiocbs[i]) == EINPROGRESS))
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1read","check the results of the others");
	for (i = aio_max-1; i >= 0; i--) {
		if (i != cancelno) {
			retval2 = aio_error(&paiocbs[i]);
			if (retval2 == -1) {
				err = errno;
				in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			if (retval2 != 0) {
				in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
				errors++;
				continue;
			}
			retval = aio_return(&paiocbs[i]);
			if (retval == -1) {
				err = errno;
				in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest1write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;
	int	cancelno;

	DBUG_ENTER("dotest1write");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if (write(writefd, bbuf, sizeof(bbuf)) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	DBUG_PRINTF("t1write", "kick off some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_write(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_write failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	sleep(2);
	DBUG_PRINTF("t1write", "ensure they are not complete (should block)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1write", "cancel one");
	if (aio_max < 2)
		cancelno = 0;
	else
		cancelno = aio_max -1;

	retval2 = tb_aio_cancel(writefd, &paiocbs[cancelno]);
	/*might not be cancelled, this is OK (UNSUPPORTED)*/
	if (retval2 == AIO_NOTCANCELED) {
		in_rpt("aio_cancel returned AIO_NOTCANCELED for aio_write request");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(1);
	}
	if (retval2 == AIO_ALLDONE) {
		in_rpt("aio_cancel returned AIO_ALLDONE for aio_write request");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 == -1) {
		vsrt_sysfail("aio_cancel for aio_write request");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 != AIO_CANCELED) {
		in_rpt("aio_cancel returned %d for aio_write request, expected AIO_CANCELED", retval2);
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1write", "wait for the cancelled one to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if (aio_error(&paiocbs[cancelno]) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("cancelled aio_write request still in progress");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1write", "check its error and return status");
	retval2 = aio_error(&paiocbs[cancelno]);
	if (retval2 == -1) {
		err = errno;
		in_rpt("aio_error failed for canceled aio_write request, errno = %d (%s)", err, errname(err));
		errors++;
	} else
		if (retval2 != ECANCELED) {
			in_rpt("Error status for canceled aio_write request is %d (%s), expected ECANCELED", retval2, errname(retval2));
			errors++;
		}
	retval = aio_return(&paiocbs[cancelno]);
	if (retval != -1) {
		err = errno;
		in_rpt("aio_return returned %d for canceled aio_write request, expected -1", retval);
		errors++;
	}

	PATH_TRACE;

	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl #3");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #4");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1write", "read all the data");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
		sleep(4);
	}
	while (read(readfd, bbuf, sizeof(bbuf)) > 0)
		;

	PATH_TRACE;

	DBUG_PRINTF("t1write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if ((i != cancelno) && (aio_error(&paiocbs[i]) == EINPROGRESS)) {
				while (read(readfd, bbuf, sizeof(bbuf)) > 0)
					;
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if ((i != cancelno) && (aio_error(&paiocbs[i]) == EINPROGRESS))
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1write", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		if (i != cancelno) {
			retval2 = aio_error(&paiocbs[i]);
			if (retval2 == -1) {
				err = errno;
				in_rpt("aio_error failed for write #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			if (retval2 != 0) {
				in_rpt("Error status for write %d is %d (%s)", i, retval2, errname(retval2));
				errors++;
				continue;
			}
			retval = aio_return(&paiocbs[i]);
			if (retval == -1) {
				err = errno;
				in_rpt("aio_return failed for write #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the aio_cancel() function as described in
System Interfaces and Headers, Issue 5, and there are
one or more asynchronous I/O operations which the
implementation defines as cancelable:
	A call to aio_cancel() when aiocbp is NULL shall can-
	cel all outstanding cancellable asynchronous I/O
	requests against fildes.
*/

private void test2()
{
	int	terrors;

	DBUG_ENTER("test2");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_cancel) {
		in_rpt("aio_cancel not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 50)
		aio_max = 50;
	else
		aio_max = (int)aio_maxl;

	DBUG_PRINTF("t2", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest2read);
	vsrt_aio_test_allbutfile_write(dotest2write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest2read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;

	DBUG_ENTER("dotest2read");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*2)+1)) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2read", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	sleep(2);

	PATH_TRACE;

	DBUG_PRINTF("t2read", "ensure they are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	retval2 = tb_aio_cancel(readfd, NULL);
	/*they might not all have been cancelled, this is OK (UNSUPPORTED)*/
	if (retval2 == AIO_NOTCANCELED) {
		in_rpt("aio_cancel returned AIO_NOTCANCELED for aio_read requests");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(1);
	}
	/*they can't be all done however*/
	if (retval2 == AIO_ALLDONE) {
		in_rpt("aio_cancel returned AIO_ALLDONE for aio_read requests");
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	if (retval2 == -1) {
		vsrt_sysfail("aio_cancel for aio_read requests");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 != AIO_CANCELED) {
		in_rpt("aio_cancel returned %d for aio_read requests, expected AIO_CANCELED", retval2);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2read", "wait for the them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2read", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != ECANCELED) {
			in_rpt("Error status for read %d is %d (%s), expected ECANCELED", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval != -1) {
			in_rpt("aio_return failed %d for read #%d, expected -1", retval, i);
			errors++;
			continue;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest2write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;

	DBUG_ENTER("dotest2write");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if (write(writefd, bbuf, sizeof(bbuf)) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	DBUG_PRINTF("t2write", "kick off some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_write(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_write failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	PATH_TRACE;

	sleep(2);

	DBUG_PRINTF("t2write", "ensure they are not complete (should block)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	retval2 = tb_aio_cancel(writefd, NULL);
	/*they might not all have been cancelled, this is OK (UNSUPPORTED)*/
	if (retval2 == AIO_NOTCANCELED) {
		in_rpt("aio_cancel returned AIO_NOTCANCELED for aio_write requests");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(1);
	}
	/*they can't be all done however*/
	if (retval2 == AIO_ALLDONE) {
		in_rpt("aio_cancel returned AIO_ALLDONE for aio_write requests");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 == -1) {
		vsrt_sysfail("aio_cancel for aio_write requests");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 != AIO_CANCELED) {
		in_rpt("aio_cancel returned %d for aio_write requests, expected AIO_CANCELED", retval2);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2write", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != ECANCELED) {
			in_rpt("Error status for write %d is %d (%s), expected ECANCELED", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval != -1) {
			err = errno;
			in_rpt("aio_return returned %d for write #%d, expected -1", retval, i);
			errors++;
			continue;
		}
	}

	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
	}
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the aio_cancel() function as described in
System Interfaces and Headers, Issue 5, and there are
one or more asynchronous I/O operations which the
implementation defines as cancelable:
	A call to aio_cancel() shall return AIO_CANCELED if
	all the requested operation(s) were canceled.
*/

private void test3()
{
	int	terrors;

	DBUG_ENTER("test3");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_cancel) {
		in_rpt("aio_cancel not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 50)
		aio_max = 50;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t3", "using %d requests", aio_max);

	/*just rerun tests from 1 and 2 since this is covered there*/
	vsrt_aio_test_allbutfile(dotest1read);
	vsrt_aio_test_allbutfile(dotest2read);
	vsrt_aio_test_allbutfile_write(dotest1write);
	vsrt_aio_test_allbutfile_write(dotest2write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the aio_cancel() function as described in
System Interfaces and Headers, Issue 5, and there are
one or more asynchronous I/O operations which the
implementation defines as cancelable:
	A call to aio_cancel() shall return AIO_ALLDONE if
	all the requested operation(s) have already completed.
*/

private void test4()
{
	int	terrors;

	DBUG_ENTER("test4");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_cancel) {
		in_rpt("aio_cancel not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 50)
		aio_max = 50;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t4", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest4read);
	vsrt_aio_test_allbutfile_write(dotest4write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest4read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	int	cancelno;

	DBUG_ENTER("dotest4read");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*2)+1)) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t4read", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(2);

	PATH_TRACE;

	DBUG_PRINTF("t4read", "ensure they are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t4read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "%3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;

	DBUG_PRINTF("t4read", "wait for the requests to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	PATH_TRACE;

	if (j == VSRT_AIO_ITERATIONS) {
		PATH_TRACE;
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (aio_max == 1)
		cancelno = 0;
	else
		cancelno = 1;
	
	retval2 = tb_aio_cancel(readfd, &paiocbs[cancelno]);
	if (retval2 == AIO_CANCELED) {
		in_rpt("aio_cancel returned AIO_CANCELED for completed aio_read request");
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	if (retval2 == AIO_NOTCANCELED) {
		in_rpt("aio_cancel returned AIO_NOTCANCELED for completed aio_read request");
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	if (retval2 == -1) {
		vsrt_sysfail("aio_cancel for completed aio_read request");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	if (retval2 != AIO_ALLDONE) {
		in_rpt("aio_cancel returned %d for completed aio_read request, expected AIO_ALLDONE", retval2);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	retval2 = tb_aio_cancel(readfd, NULL);
	if (retval2 == AIO_CANCELED) {
		in_rpt("aio_cancel returned AIO_CANCELED for completed aio_read request");
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	if (retval2 == AIO_NOTCANCELED) {
		in_rpt("aio_cancel returned AIO_NOTCANCELED for completed aio_read request");
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	if (retval2 == -1) {
		vsrt_sysfail("aio_cancel for completed aio_read request");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	if (retval2 != AIO_ALLDONE) {
		in_rpt("aio_cancel returned %d for completed aio_read request, expected AIO_ALLDONE", retval2);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t4read", "check the results of the requests");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
	}

	PATH_TRACE;
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest4write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;
	int	cancelno;

	DBUG_ENTER("dotest4write");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t4write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if (write(writefd, bbuf, sizeof(bbuf)) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	DBUG_PRINTF("t4write", "kick off some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_write(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_write failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	PATH_TRACE;

	sleep(2);

	DBUG_PRINTF("t4write", "ensure they are not complete (should block)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl #3");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #4");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t4write", "read all the data");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
		sleep(4);
	}
	while (read(readfd, bbuf, sizeof(bbuf)) > 0)
		;

	PATH_TRACE;

	DBUG_PRINTF("t4write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				while (read(readfd, bbuf, sizeof(bbuf)) > 0)
					;
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t4write", "cancel one");
	if (aio_max == 1)
		cancelno = 0;
	else
		cancelno = 1;

	retval2 = tb_aio_cancel(writefd, &paiocbs[cancelno]);
	if (retval2 == AIO_NOTCANCELED) {
		in_rpt("aio_cancel returned AIO_NOTCANCELED for aio_write request");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 == AIO_CANCELED) {
		in_rpt("aio_cancel returned AIO_CANCELED for aio_write request");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 == -1) {
		vsrt_sysfail("aio_cancel for aio_write request");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 != AIO_ALLDONE) {
		in_rpt("aio_cancel returned %d for completed aio_read request, expected AIO_ALLDONE", retval2);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t4write", "cancel all");
	retval2 = tb_aio_cancel(writefd, NULL);
	if (retval2 == AIO_NOTCANCELED) {
		in_rpt("aio_cancel returned AIO_NOTCANCELED for aio_write request");
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	if (retval2 == AIO_CANCELED) {
		in_rpt("aio_cancel returned AIO_CANCELED for aio_write request");
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	if (retval2 == -1) {
		vsrt_sysfail("aio_cancel for aio_write request");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 != AIO_ALLDONE) {
		in_rpt("aio_cancel returned %d for completed aio_read request, expected AIO_ALLDONE", retval2);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t4write", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for write %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
	}

	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
	}
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the aio_cancel() function as described in
System Interfaces and Headers, Issue 5, and there are
one or more asynchronous I/O operations which the
implementation defines as cancelable:
	A call to aio_cancel() shall cause normal asynchro-
	nous notification to occur for operations that are
	successfully canceled.
*/

private void test5()
{
	int	terrors;

	DBUG_ENTER("test5");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_cancel) {
		in_rpt("aio_cancel not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 50)
		aio_max = 50;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t5", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest5read);
	vsrt_aio_test_allbutfile_write(dotest5write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest5read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	int	err;
	ssize_t	retval;
	siginfo_t	info;
	struct timespec	timeout;
	int	retval2;
	sigset_t	set;

	DBUG_ENTER("dotest5read");

	errors = 0;
	got_sig = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);

	/*block the signal to cause queuing*/
	sigemptyset(&set);
	if (sigaddset(&set, SIGRTMIN) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}
	if (sigprocmask(SIG_BLOCK, &set, NULL) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t5read", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*sizeof(tbuf);
		paiocbs[i].aio_nbytes = sizeof(tbuf);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;

	DBUG_PRINTF("t5read", "cancel them");
	retval2 = tb_aio_cancel(readfd, NULL);
	/*they might not all have been cancelled, this is OK (UNSUPPORTED)*/
	if (retval2 == AIO_NOTCANCELED) {
		in_rpt("aio_cancel returned AIO_NOTCANCELED for aio_read requests");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(1);
	}
	/*they can't be all done however*/
	if (retval2 == AIO_ALLDONE) {
		in_rpt("aio_cancel returned AIO_ALLDONE for aio_read requests");
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	if (retval2 == -1) {
		vsrt_sysfail("aio_cancel for aio_read requests");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 != AIO_CANCELED) {
		in_rpt("aio_cancel returned %d for aio_read requests, expected AIO_CANCELED", retval2);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t5read", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t5read", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != ECANCELED) {
			in_rpt("Error status for read %d is %d (%s), expected ECANCELED", i, retval2, errname(retval2));
			errors++;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval != -1) {
			err = errno;
			in_rpt("aio_return returned %d for read #%d, expected -1", retval, i);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (got_sig != 0) {
		in_rpt("SIGRTMIN is blocked but %d signals were delivered", got_sig);
		errors++;
	}

	PATH_TRACE;

	DBUG_PRINTF("t5read", "retrieve the signals");
	memset(got_requests, 0xff, 50);
	timeout.tv_sec = 1;
	timeout.tv_nsec = 0;
	for (i = 0; i < aio_max; i++) {
		if (sigtimedwait(&set, &info, &timeout) == -1) {
			err = errno;
			in_rpt("sigtimedwait failed on signal #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
		else {
			/*save these as we may be getting the requests back
			out of order*/
			if (info.si_value.sival_int < aio_max)
				got_requests[info.si_value.sival_int] = info.si_value.sival_int;
			else {
				in_rpt("si_value for signal %d = %d, maximum value in requests is %d", i, info.si_value.sival_int, aio_max);
				errors++;
			}

			if (info.si_signo != SIGRTMIN) {
				in_rpt("Expected si_signo for signal %d = %d, is %d", i, SIGRTMIN, info.si_signo);
				errors++;
			}
			if (info.si_code != SI_ASYNCIO) {
				in_rpt("Expected si_code for signal %d = %d, is %d", i, SI_ASYNCIO, info.si_code);
				errors++;
			}
		}
	}
	for (i = 0; i < aio_max; i++) {
		if (got_requests[i] == 0xff) {
			in_rpt("No signal received for request %d", i);
			errors++;
			continue;
		}
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest5write(readfd, writefd)
int	readfd;
int	writefd;
{
	struct	timespec	timeout;
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;
	sigset_t	set;
	siginfo_t	info;

	DBUG_ENTER("dotest5write");

	errors = 0;

	got_sig = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);

	/*block the signal to cause queuing*/
	sigemptyset(&set);
	if (sigaddset(&set, SIGRTMIN) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}
	if (sigprocmask(SIG_BLOCK, &set, NULL) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t5write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if (write(writefd, bbuf, sizeof(bbuf)) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	DBUG_PRINTF("t5write", "kick off some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_write(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_write failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	PATH_TRACE;

	sleep(2);

	DBUG_PRINTF("t5write", "ensure they are not complete (should block)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t5write", "cancel them");
	retval2 = tb_aio_cancel(writefd, NULL);
	/*they might not all have been cancelled, this is OK (UNSUPPORTED)*/
	if (retval2 == AIO_NOTCANCELED) {
		in_rpt("aio_cancel returned AIO_NOTCANCELED for aio_write requests");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(1);
	}
	/*they can't be all done however*/
	if (retval2 == AIO_ALLDONE) {
		in_rpt("aio_cancel returned AIO_ALLDONE for aio_write requests");
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	if (retval2 == -1) {
		vsrt_sysfail("aio_cancel for aio_write requests");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 != AIO_CANCELED) {
		in_rpt("aio_cancel returned %d for aio_write requests, expected AIO_CANCELED", retval2);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t5write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t5write", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != ECANCELED) {
			in_rpt("Error status for write %d is %d (%s), expected ECANCELED", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval != -1) {
			err = errno;
			in_rpt("aio_return returned %d for write #%d, expected -1", retval, i);
			errors++;
		}
	}

	if (got_sig != 0) {
		in_rpt("SIGRTMIN is blocked but %d signals were delivered", got_sig);
		errors++;
	}

	PATH_TRACE;

	DBUG_PRINTF("t5write", "retrieve the signals");
	timeout.tv_sec = 1;
	timeout.tv_nsec = 0;
	memset(got_requests, 0xff, 50);
	for (i = 0; i < aio_max; i++) {
		if (sigtimedwait(&set, &info, &timeout) == -1) {
			err = errno;
			in_rpt("sigtimedwait failed on signal #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
		else {
			/*save these as we may be getting the requests back
			out of order*/
			if (info.si_value.sival_int < aio_max)
				got_requests[info.si_value.sival_int] = info.si_value.sival_int;
			else {
				in_rpt("si_value for signal %d = %d, maximum value in requests is %d", i, info.si_value.sival_int, aio_max);
				errors++;
			}
			if (info.si_signo != SIGRTMIN) {
				in_rpt("Expected si_signo for signal %d = %d, is %d", i, SIGRTMIN, info.si_signo);
				errors++;
			}
			if (info.si_code != SI_ASYNCIO) {
				in_rpt("Expected si_code for signal %d = %d, is %d", i, SI_ASYNCIO, info.si_code);
				errors++;
			}
		}
	}

	for (i = 0; i < aio_max; i++) {
		if (got_requests[i] == 0xff) {
			in_rpt("No signal received for request %d", i);
			errors++;
			continue;
		}
	}

	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
	}
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}


/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the aio_cancel() function as described in
System Interfaces and Headers, Issue 5, and there are
one or more asynchronous I/O operations which the
implementation defines as cancelable:
	After a call to aio_cancel() the associated error
	status for operations that are successfully canceled
	shall be set to ECANCELED and return status set to -1.
*/

private void test6()
{
	int	terrors;

	DBUG_ENTER("test6");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_cancel) {
		in_rpt("aio_cancel not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 50)
		aio_max = 50;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t6", "using %d requests", aio_max);

	/*just rerun tests from 1 and 2 since this is covered there*/
	vsrt_aio_test_allbutfile(dotest1read);
	vsrt_aio_test_allbutfile(dotest2read);
	vsrt_aio_test_allbutfile_write(dotest1write);
	vsrt_aio_test_allbutfile_write(dotest2write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the aio_cancel() function as described in
System Interfaces and Headers, Issue 5, and there are
one or more asynchronous I/O operations which the
implementation defines as cancelable:
	A call to aio_cancel() shall return -1 and set errno
	to EBADF if fildes is not a valid file descriptor.
*/

private void test7()
{
	int	fd;
	pid_t	pid;
	int	status, errors;

	DBUG_ENTER("test7");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_cancel) {
		in_rpt("aio_cancel not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	errors = 0;

	VSRT_FORK(pid);
		VSRT_SIGNAL(SIGUSR1, handler5);

		(void)unlink("vsrt_aio");
		if ((fd = open("vsrt_aio", O_RDWR|O_CREAT, VSRT_PROT_ALL)) == -1) {
			vsrt_sysfail("open");
			exit(1);
		}

		if (close(fd) == -1) {
			vsrt_sysfail("close");
			(void)unlink("vsrt_aio");
			exit(1);
		}
	
		(void)unlink("vsrt_aio");
		
		if (tb_aio_cancel(fd+500, NULL) != -1) {
			in_rpt("aio_cancel succeeded");
			xx_rpt(FAILURE);
			exit(1);
		}
	
		if (errno != EBADF) {
			vsrt_sysfail("aio_cancel");
			xx_rpt(FAILURE);
			exit(1);
		}
	
	
	VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	if (errors == 0)
		xs_rpt();
	
	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is not defined and the imple-
mentation does not support the aio_cancel() function as
described in System Interfaces and Headers, Issue 5:
	A call to aio_cancel() shall return -1 and set errno
	to ENOSYS.
*/

private void test8()
{

#ifndef _POSIX_ASYNCHRONOUS_IO
	struct aiocb	myaiocb;
#endif

	DBUG_ENTER("test8");

#ifdef _POSIX_ASYNCHRONOUS_IO
	in_rpt("_POSIX_ASYNCHRONOUS_IO defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN
#else
	if (supports_aio_cancel) {
		in_rpt("aio_cancel supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	if (tb_aio_cancel(0, &myaiocb) != -1) {
		in_rpt("aio_cancel succeeded");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (errno != ENOSYS) {
		vsrt_sysfail("aio_cancel");
		DBUG_VOID_RETURN
	}

	xs_rpt();

	DBUG_VOID_RETURN
#endif
}

private	int tb_aio_cancel(fildes, aiocbp)
int	fildes;
struct aiocb	*aiocbp;
{
	int 	retval;

	DBUG_ENTER("tb_aio_cancel");
	DBUG_PRINTF("tb_aio_cancel", "fildes = %d, aiocbp = %#lx",
		fildes, (long) aiocbp);

	if (aiocbp != (struct aiocb *) 0) {
		DBUG_PRINTF("tb_aio_cancel", "  fildes = %d",
			aiocbp->aio_fildes);
		DBUG_PRINTF("tb_aio_cancel", "  offset = %ld",
			(long) aiocbp->aio_offset);
		DBUG_PRINTF("tb_aio_cancel", "  buf = %#lx",
			(long) aiocbp->aio_buf);
		DBUG_PRINTF("tb_aio_cancel", "  nbytes = %ld",
			(long) aiocbp->aio_nbytes);
		DBUG_PRINTF("tb_aio_cancel", "  reqprio = %d",
			aiocbp->aio_reqprio);
		DBUG_PRINTF("tb_aio_cancel", "  lio_opcode = %d",
			aiocbp->aio_lio_opcode);
		DBUG_PRINTF("tb_aio_cancel", "  sigevent.sigev_notify = %d",
			aiocbp->aio_sigevent.sigev_notify);
		DBUG_PRINTF("tb_aio_cancel", "  sigevent.sigev_signo = %d (%s)",
			aiocbp->aio_sigevent.sigev_signo,
			signame(aiocbp->aio_sigevent.sigev_signo));
	}

	retval = aio_cancel(fildes, aiocbp);

#ifdef	TESTBED
	errno++;
	DBUG_RETURN(-1);
#else
	DBUG_RETURN(retval);
#endif
}
