/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/aio/aio_error/aio_error.c
*
* Modifications:
* $Log: aio_error.c,v $
* Revision 1.2  2001/10/02 13:25:12  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:13  cyeoh
* Adds files for lsb-os test suite
*
*
*/

#include	<vsrt/vsrt.h>
#include	<aio.h>
#include	<termios.h>
#include	<limits.h>
#include	<setjmp.h>
#include	<setprv.h>

#define 	NO_TESTS	4	/*number of tests in testset */

#ifdef UNDEF_MACROS
#undef aio_error
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();

int	no_tests = NO_TESTS;

private	int	tb_aio_error();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private int	dotest1read();
private int	dotest1write();
private int	dotest2read();
private int	dotest2write();
private int	dotest3read();
private int	dotest3write();

int	supports_aio_read;
int	supports_aio_write;
int	supports_aio_error;
int	supports_aio_return;

/*determine if a type of file is to be tried. set in startup*/
/*pipe support is determined dynamically*/
int	vsrt_aio_test_with_tty = 0;
int	vsrt_aio_test_with_file = 0;
int	vsrt_aio_test_with_pty = 0;

/*each piece of a test increments on of these. must be zeroed at test start*/
int	vsrt_aio_test_pass;
int	vsrt_aio_test_unsup;
int	vsrt_aio_test_fail;

/*instances of things to test on from tetexec, loaded in startup*/
char	vsrt_aio_tty1[128];
char	vsrt_aio_tty2[128];
char	vsrt_aio_ptym[128];
char	vsrt_aio_testfile[4096];

void atstart(void);
void atstart(void)
{
	char	*str;

	vsrt_check_support("AIO_READ", &supports_aio_read);
	vsrt_check_support("AIO_WRITE", &supports_aio_write);
	vsrt_check_support("AIO_RETURN", &supports_aio_return);
	vsrt_check_support("AIO_ERROR", &supports_aio_error);

	/*get config info and determine what sorts of files to test with*/
	str = tet_getvar("VSRT_FILE_ASYNC_IO");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_file = 1;
		strcpy(vsrt_aio_testfile, str);
	}

	str = tet_getvar("VSRT_TERMIOS_TTY");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_tty = 1;
		strcpy(vsrt_aio_tty1, str);
		str = tet_getvar("VSRT_TERMIOS_LOOP");
		if ((str != NULL) && (strlen(str) != 0))
			strcpy(vsrt_aio_tty2, str);
		else
			vsrt_aio_test_with_tty = 0;
	}

	str = tet_getvar("VSRT_MASTER_PTY");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_pty = 1;
		strcpy(vsrt_aio_ptym, str);
	}
}

void atend(void);
void atend(void)
{
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = atend;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

int	aio_max = 0;
long	aio_maxl = 0;

int	got_sig = 0;

private	void	handler5();
private	void	handler5(signo)
int	signo;
{
	got_sig++;
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the aio_error() function as described in
System Interfaces and Headers, Issue 5, and there are
one or more asynchronous I/O operations which the
implementation defines as cancelable:
	A call to aio_error() when the asynchronous I/O
	operation associated with the aiocb structure refer-
	enced by aiocbp completed unsuccessfully shall return
	the errno value that would be set by the correspond-
	ing read(), write(), or fsync() operation.
*/

private void test1()
{
	int	terrors;

	DBUG_ENTER("test1");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 10)
		aio_max = 10;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t1", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest1read);
	vsrt_aio_test_allbutfile_write(dotest1write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest1read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	int	origflags;

	/*check O_NONBLOCK operation*/
	DBUG_ENTER("dotest1read");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	DBUG_PRINTF("t1read", "keep reads from blocking");
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		DBUG_RETURN(-1);
	}

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*2)+1)) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1read", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1read", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (tb_aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (tb_aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1read", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = tb_aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		} else
			if (retval2 != EAGAIN) {
				in_rpt("Error status for read %d is %d (%s), expected EAGAIN", i, retval2, errname(retval2));
				errors++;
			}
		retval = aio_return(&paiocbs[i]);
		if (retval != -1) {
			err = errno;
			in_rpt("aio_return returned %d for read #%d, expected -1", retval);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest1write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;

	/*this code is used to check O-NONBLOCK operation*/
	DBUG_ENTER("dotest1write");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	DBUG_PRINTF("t1write", "try and fill up the data object by writing a lot");
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		DBUG_RETURN(-1);
	}

	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if (write(writefd, bbuf, sizeof(bbuf)) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1write", "kick off some aio writes with O_NONBLOCK set on fd");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_write(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_write failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	PATH_TRACE;

	DBUG_PRINTF("t1write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (tb_aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (tb_aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1write", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = tb_aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error returned %d for write #%d, errno = %d (%s)", retval2, i, err, errname(err));
			errors++;
		} else
			if (retval2 != EAGAIN) {
				in_rpt("Error status for write %d is %d (%s), expected EAGAIN", i, retval2, errname(retval2));
				errors++;
			}
		retval = aio_return(&paiocbs[i]);
		if (retval != -1) {
			err = errno;
			in_rpt("aio_return returned %d for write #%d, expected -1", retval, i);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the aio_error() function as described in
System Interfaces and Headers, Issue 5:
	A call to aio_error() when the asynchronous I/O
	operation associated with the aiocb structure refer-
	enced by aiocbp completed successfully shall return 0.
*/

private void test2()
{
	int	terrors;

	DBUG_ENTER("test2");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 10)
		aio_max = 10;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t2", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest2read);
	vsrt_aio_test_all_write(dotest2write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest2read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	int	retval2;
	int	errors;

	DBUG_ENTER("dotest2read");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*2)+1)) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	DBUG_PRINTF("t2read", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t2read", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (tb_aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (tb_aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t2read", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest2write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	int	retval2;
	int	errors;

	DBUG_ENTER("dotest2write");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2write", "kick off some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_write(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_write failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	PATH_TRACE;

	DBUG_PRINTF("t2write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (tb_aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (tb_aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2write", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = tb_aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error returned %d for write #%d, errno = %d (%s)", retval2, i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for write %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}


/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the aio_error() function as described in
System Interfaces and Headers, Issue 5:
	A call to aio_error() when the asynchronous I/O
	operation associated with the aiocb structure refer-
	enced by aiocbp has not yet completed shall return
	EINPROGRESS.
*/

private void test3()
{
	int	terrors;

	DBUG_ENTER("test3");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 10)
		aio_max = 10;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t3", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest3read);
	vsrt_aio_test_allbutfile_write(dotest3write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest3read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i;
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	int	errors;

	DBUG_ENTER("dotest3read");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*2)+1)) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t3read", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(2);

	PATH_TRACE;
	DBUG_PRINTF("t3read", "ensure they are in progress");
	for (i = 0; i < aio_max; i++) {
		if (tb_aio_error(&paiocbs[i]) != EINPROGRESS) {
			in_rpt("request %d is not in progress", i);
			errors++;
		}
	}

	(void)aio_cancel(readfd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest3write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	int	errors, origflags;
	ssize_t	retval;

	DBUG_ENTER("dotest3write");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t3write", "try and fill up the data object by writing a lot");
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	memset(bbuf, 'A', sizeof(bbuf));
	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		retval = write(writefd, bbuf, sizeof(bbuf));
		if (retval == -1) {
			if (errno == EAGAIN)
				break;
			else {
				vsrt_sysfail("write");
				free(plist);
				free(bufs);
				free(paiocbs);
				DBUG_RETURN(-1);
			}
		}
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(1);	
	}

	PATH_TRACE;
	DBUG_PRINTF("t3write", "turn O_NONBLOCK off");
	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t3write", "kick off some aio writes which should block");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_write(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_write failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	PATH_TRACE;

	sleep(2);

	DBUG_PRINTF("t3write", "ensure they are in progress");
	for (i = 0; i < aio_max; i++) {
		if (tb_aio_error(&paiocbs[i]) != EINPROGRESS) {
			in_rpt("request %d is not in progress", i);
			errors++;
		}
	}

	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
	}

	(void)aio_cancel(writefd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is not defined and the imple-
mentation does not support the aio_error() function as
described in System Interfaces and Headers, Issue 5:
	A call to aio_error() shall return -1 and set errno
	to ENOSYS.
*/

private void test4()
{
#ifndef _POSIX_ASYNCHRONOUS_IO
	struct aiocb	myaiocb;
#endif

	DBUG_ENTER("test4");

#ifdef _POSIX_ASYNCHRONOUS_IO
	in_rpt("_POSIX_ASYNCHRONOUS_IO defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN
#else
	if (supports_aio_error) {
		in_rpt("aio_error supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	if (tb_aio_error(&myaiocb) != -1) {
		in_rpt("aio_error succeeded");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (errno != ENOSYS) {
		vsrt_sysfail("aio_error");
		DBUG_VOID_RETURN
	}

	xs_rpt();

	DBUG_VOID_RETURN
#endif
}

private	int tb_aio_error(aiocbp)
struct aiocb	*aiocbp;
{
	int 	retval;

	retval = aio_error(aiocbp);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
