/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/aio/aio_fsync/aio_fsync.c
*
* Modifications:
* $Log: aio_fsync.c,v $
* Revision 1.2  2001/10/02 13:25:12  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:13  cyeoh
* Adds files for lsb-os test suite
*
*
*/

#include	<vsrt/vsrt.h>
#include	<aio.h>
#include	<limits.h>
#include	<setjmp.h>
#include	<setprv.h>

#define 	NO_TESTS	13	/* number of tests in testset */

#ifdef UNDEF_MACROS
#undef aio_fsync
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();

int	no_tests = NO_TESTS;

private	int	tb_aio_fsync();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private int	dotest1read();
private int	dotest1readfile();
private int	dotest2read();
private int	dotest2readfile();
private int	dotest3read();
private int	dotest3readfile();
private int	dotest4read();
private int	dotest4readfile();
private int	dotest5read();
private int	dotest5readfile();

int	supports_aio_fsync;
int	supports_aio_read;
int	supports_aio_write;
int	supports_aio_error;
int	supports_aio_return;

/*determine if a type of file is to be tried. set in startup*/
/*pipe support is determined dynamically*/
int	vsrt_aio_test_with_tty = 0;
int	vsrt_aio_test_with_file = 0;
int	vsrt_aio_test_with_pty = 0;

/*each piece of a test increments on of these.  must be zeroed at test start*/
int	vsrt_aio_test_pass;
int	vsrt_aio_test_unsup;
int	vsrt_aio_test_fail;

/*instances of things to test on from tetexec, loaded in startup*/
char	vsrt_aio_tty1[128];
char	vsrt_aio_tty2[128];
char	vsrt_aio_ptym[128];
char	vsrt_aio_testfile[4096];

void atstart(void)
{
	char *str;

	vsrt_check_support("AIO_FSYNC", &supports_aio_fsync);
	vsrt_check_support("AIO_READ", &supports_aio_read);
	vsrt_check_support("AIO_WRITE", &supports_aio_write);
	vsrt_check_support("AIO_ERROR", &supports_aio_error);
	vsrt_check_support("AIO_RETURN", &supports_aio_return);

	/*get config info and determine what sorts of files to test with*/
	str = tet_getvar("VSRT_FILE_ASYNC_IO");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_file = 1;
		strcpy(vsrt_aio_testfile, str);
	}

	str = tet_getvar("VSRT_TERMIOS_TTY");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_tty = 1;
		strcpy(vsrt_aio_tty1, str);
		str = tet_getvar("VSRT_TERMIOS_LOOP");
		if ((str != NULL) && (strlen(str) != 0))
			strcpy(vsrt_aio_tty2, str);
		else
			vsrt_aio_test_with_tty = 0;
	}

	str = tet_getvar("VSRT_MASTER_PTY");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_pty = 1;
		strcpy(vsrt_aio_ptym, str);
	}
}

void atend(void)
{
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = atend;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	NULL, 0
};

int	aio_max = 0;
long	aio_maxl = 0;

int	got_sig = 0;
int	got_sig_min, got_sig_max = 0;

private	void	handler5(signo)
int	signo;
{
	got_sig++;
}

private void rt_handler7(signo, info, context)
int	signo;
siginfo_t	*info;
void	*context;
{
	got_sig++;
	if (signo == SIGRTMIN) {
		got_sig_min++;
	}
	if (signo == SIGRTMAX) {
		got_sig_max = got_sig_min+1;
	}
}

private void sleeploop(unsigned int secs)
{
	/* Sleep for the specified number of seconds by calling sleep()
	   in a loop until it returns 0 as the unslept time.
	   This is used where one or more signals are expected to
	   arrive during the sleep time. */

	while (secs != 0)
		secs = sleep(secs);
}

int vsrt_rt_signal_mask(int sig, void (*handler)(int, siginfo_t *, void *), sigset_t set)
{
	int     rval;
	struct sigaction Sigaction;

	Sigaction.sa_mask = set;
	Sigaction.sa_sigaction = handler;
	Sigaction.sa_flags = SA_SIGINFO;
	rval = sigaction(sig,&Sigaction,(struct sigaction *)NULL);
	if (rval == -1) {
		in_rpt("vsrt_rt_signal_mask: sigaction failed for signal %d, errno = %s", sig, errname(errno));
		xx_rpt(FAILURE);
		return -1;
	}
	return 0;
}


unsigned char	got_requests[50];


/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_SYNCHRONOUS_IO are
defined  or the implementation supports the aio_fsync()
function as described in System Interfaces and Headers,
Issue 5:
	A successful call to aio_fsync() shall queue the aiocb structure
	referenced  by  aiocbp  for  the  file  indicated  by
	aiocbp->aio_fildes,  setting  its  error  status   to
	EINPROGRESS and return 0.
*/

private void test1()
{
	int	terrors;

	DBUG_ENTER("test1");

#if !defined(_POSIX_ASYNCHRONOUS_IO) || !defined(_POSIX_SYNCHRONOUS_IO)
	if (!supports_aio_fsync) {
		in_rpt("aio_fsync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 10)
		aio_max = 10;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t1", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest1read);
	vsrt_aio_test_file(dotest1readfile);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest1read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	syncaiocb;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	int	err;
	ssize_t	retval;
	int	retval2;
	unsigned char got_data[100];
	int	c, rval;
	sigset_t	set;

	DBUG_ENTER("dotest1read");

	errors = 0;

	if (fpathconf(readfd, _PC_SYNC_IO) == -1) {
		in_rpt("Synchronous I/O not supported for this file");
		DBUG_RETURN(1);
	}

	got_sig = got_sig_min = got_sig_max = 0;

	rval = sigemptyset(&(set));
	if (rval == -1) {
		in_rpt("sigemptyset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	rval = sigaddset(&set, SIGRTMAX);
	if (rval == -1) {
		in_rpt("sigaddset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	if (vsrt_rt_signal_mask(SIGRTMIN, rt_handler7, set) == -1)
		DBUG_RETURN(-1);

	rval = sigemptyset(&(set));
	if (rval == -1) {
		in_rpt("sigemptyset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	rval = sigaddset(&set, SIGRTMIN);
	if (rval == -1) {
		in_rpt("sigemptyset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	if (vsrt_rt_signal_mask(SIGRTMAX, rt_handler7, set) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1read", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*13];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t1read", "ensure they are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1read", "do an aio_fsync");
	syncaiocb.aio_fildes = readfd;
	syncaiocb.aio_buf = NULL;
	syncaiocb.aio_offset = 0;
	syncaiocb.aio_nbytes = 0;
	syncaiocb.aio_sigevent.sigev_notify = SIGEV_SIGNAL;
	syncaiocb.aio_sigevent.sigev_signo = SIGRTMAX;
	syncaiocb.aio_sigevent.sigev_value.sival_int = 100;
	if (tb_aio_fsync(O_SYNC, &syncaiocb) == -1) {
		vsrt_sysfail("aio_fsync");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(2);

	PATH_TRACE;
	DBUG_PRINTF("t1read", "ensure the aio_fsync is in progress");
	if ((retval2 = aio_error(&syncaiocb)) != EINPROGRESS) {
		in_rpt("aio_fsync request is not in progress, its error status is %d (%s)", retval2, errname(retval2));
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t1read", "wait for reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1read", "ensure the fsync is done too");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if ((retval2 = aio_error(&syncaiocb)) == EINPROGRESS)
			sleep(2);
		else {
			if (retval2 != 0) {
				in_rpt("aio_fsync error status is %s", errname(retval2));
				(void)aio_cancel(readfd, NULL);
				free(plist);
				free(paiocbs);
				free(bufs);
				DBUG_RETURN(-1);
			}
			break;
		}
	}
	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("aio_fsync request is still in progress");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	sleeploop(4);

	PATH_TRACE;
	DBUG_PRINTF("t1read", "check the results");
	memset(got_data, 0xff, 100);
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (retval != strlen(tbuf)) {
			in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
			errors++;
			continue;
		}
		if (sscanf((char *)paiocbs[i].aio_buf, "message %3d\n", &c) != 1) {
		
			in_rpt("Could not match returned data for request %d, \"message xxx \\n\", received \"%s\"", i, paiocbs[i].aio_buf);
			errors++;
		}
		else {
			if ((c < 0) || (c >= aio_max)) {
				in_rpt("Message number returned in request %d is %d, expected between 0 and %d", i, c, aio_max-1);
				errors++;
			} else
				got_data[c] = 0;
		}
	}
	for (i = aio_max-1; i >= 0; i--) {
		if (got_data[i] == 0xff) {
			in_rpt("Message number %d not read by any request", i);
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t1read", "check notification");
	if (got_sig_min != aio_max) {
		in_rpt("Expected %d signals from aio_reads, received %d", aio_max, got_sig_min);
		errors++;
	}

	if (got_sig_max == 0) {
		in_rpt("Signal from aio_fsync operation not received");
		errors++;
	}

	if (got_sig_max != got_sig_min+1) {
		in_rpt("Expected signal from aio_fsync operation to be received after those from aio_reads, got_sig_min = %d, got_sig_max = %d", got_sig_min, got_sig_max);
		errors++;
	}

	PATH_TRACE;
	(void)aio_cancel(readfd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}
private int dotest1readfile(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	syncaiocb;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	int	err;
	int	retval2;
	int	rval;
	sigset_t	set;

	DBUG_ENTER("dotest1readfile");

	errors = 0;

	if (fpathconf(readfd, _PC_SYNC_IO) == -1) {
		in_rpt("Synchronous I/O not supported for this file");
		DBUG_RETURN(1);
	}

	got_sig = got_sig_min = got_sig_max = 0;

	rval = sigemptyset(&(set));
	if (rval == -1) {
		in_rpt("sigemptyset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	rval = sigaddset(&set, SIGRTMAX);
	if (rval == -1) {
		in_rpt("sigaddset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	if (vsrt_rt_signal_mask(SIGRTMIN, rt_handler7, set) == -1)
		DBUG_RETURN(-1);

	rval = sigemptyset(&(set));
	if (rval == -1) {
		in_rpt("sigemptyset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	rval = sigaddset(&set, SIGRTMIN);
	if (rval == -1) {
		in_rpt("sigemptyset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	if (vsrt_rt_signal_mask(SIGRTMAX, rt_handler7, set) == -1)
		DBUG_RETURN(-1);


	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1readf", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*13];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t1readf", "do an aio_fsync");
	syncaiocb.aio_fildes = readfd;
	syncaiocb.aio_buf = NULL;
	syncaiocb.aio_offset = 0;
	syncaiocb.aio_nbytes = 0;
	syncaiocb.aio_sigevent.sigev_notify = SIGEV_SIGNAL;
	syncaiocb.aio_sigevent.sigev_signo = SIGRTMAX;
	syncaiocb.aio_sigevent.sigev_value.sival_int = 100;
	if (tb_aio_fsync(O_SYNC, &syncaiocb) == -1) {
		vsrt_sysfail("aio_fsync");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}


	PATH_TRACE;
	DBUG_PRINTF("t1readf", "wait for reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1readf", "ensure the fsync is done too");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if ((retval2 = aio_error(&syncaiocb)) == EINPROGRESS)
			sleep(2);
		else {
			if (retval2 != 0) {
				in_rpt("aio_fsync error status is %s", errname(retval2));
				(void)aio_cancel(readfd, NULL);
				free(plist);
				free(paiocbs);
				free(bufs);
				DBUG_RETURN(-1);
			}
			break;
		}
	}
	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("aio_fsync request is still in progress");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	sleeploop(4);

	PATH_TRACE;
	DBUG_PRINTF("t1readf", "check notification");
	if (got_sig_min != aio_max) {
		in_rpt("Expected %d signals from aio_reads, received %d", aio_max, got_sig_min);
		errors++;
	}

	if (got_sig_max == 0) {
		in_rpt("Signal from aio_fsync operation not received");
		errors++;
	}

	if (got_sig_max != got_sig_min+1) {
		in_rpt("Expected signal from aio_fsync operation to be received after those from aio_reads");
		errors++;
	}

	(void)aio_cancel(readfd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_SYNCHRONOUS_IO are
defined  or the implementation supports the aio_fsync()
function as described in System Interfaces and Headers,
Issue 5:
	When data for all asynchronous I/O operations for the
	file  indicated by aiocbp->aio_fildes queued prior to
	a  call  to   aio_fsync()   has   been   successfully
	transferred the error status for the the aiocb struc-
	ture referenced by aiocbp shall be  set  to  indicate
	the success or failure of the operation, which can be
	retrieved with aio_error() and aio_return().
*/

private void test2()
{
	int	terrors;

	DBUG_ENTER("test2");

#if !defined(_POSIX_ASYNCHRONOUS_IO) || !defined(_POSIX_SYNCHRONOUS_IO)
	if (!supports_aio_fsync) {
		in_rpt("aio_fsync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 10)
		aio_max = 10;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t2", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest2read);
	vsrt_aio_test_file(dotest2readfile);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest2read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	syncaiocb;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	int	err;
	ssize_t	retval;
	int	retval2;

	DBUG_ENTER("dotest2read");

	errors = 0;

	if (fpathconf(readfd, _PC_SYNC_IO) == -1) {
		in_rpt("Synchronous I/O not supported for this file");
		DBUG_RETURN(1);
	}

	got_sig = got_sig_min = got_sig_max = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);
	if (vsrt_rt_signal(SIGRTMAX, rt_handler7) == -1)
		DBUG_RETURN(-1);


	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2read", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*13];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t2read", "ensure they are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t2read", "do an aio_fsync");
	syncaiocb.aio_fildes = readfd;
	syncaiocb.aio_buf = NULL;
	syncaiocb.aio_offset = 0;
	syncaiocb.aio_nbytes = 0;
	syncaiocb.aio_sigevent.sigev_notify = SIGEV_SIGNAL;
	syncaiocb.aio_sigevent.sigev_signo = SIGRTMAX;
	syncaiocb.aio_sigevent.sigev_value.sival_int = 100;
	if (tb_aio_fsync(O_SYNC, &syncaiocb) == -1) {
		vsrt_sysfail("aio_fsync");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(2);

	PATH_TRACE;
	DBUG_PRINTF("t2read", "ensure the aio_fsync is in progress");
	if ((retval2 = aio_error(&syncaiocb)) != EINPROGRESS) {
		in_rpt("aio_fsync request is not in progress, its error status is %d (%s)", retval2, errname(retval2));
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t2read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t2read", "wait for reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t2read", "ensure the fsync is done too");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if ((retval2 = aio_error(&syncaiocb)) == EINPROGRESS)
			sleep(2);
		else
			break;
	}
	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("aio_fsync request is still in progress");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(2);
	if (retval2 != 0) {
		in_rpt("aio_error indicated aio_fsync request error status is %d (%s)", retval2, errname(retval2));
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if ((retval = aio_return(&syncaiocb)) != 0) {
		in_rpt("aio_return returned %d for aio_fsync request", retval);
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	(void)aio_cancel(readfd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest2readfile(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	syncaiocb;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	int	err;
	int	retval2;
	ssize_t	retval;

	DBUG_ENTER("dotest2readfile");

	errors = 0;

	if (fpathconf(readfd, _PC_SYNC_IO) == -1) {
		in_rpt("Synchronous I/O not supported for this file");
		DBUG_RETURN(1);
	}

	got_sig = got_sig_min = got_sig_max = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);
	if (vsrt_rt_signal(SIGRTMAX, rt_handler7) == -1)
		DBUG_RETURN(-1);


	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2readf", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*13];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t2readf", "do an aio_fsync");
	syncaiocb.aio_fildes = readfd;
	syncaiocb.aio_buf = NULL;
	syncaiocb.aio_offset = 0;
	syncaiocb.aio_nbytes = 0;
	syncaiocb.aio_sigevent.sigev_notify = SIGEV_SIGNAL;
	syncaiocb.aio_sigevent.sigev_signo = SIGRTMAX;
	syncaiocb.aio_sigevent.sigev_value.sival_int = 100;
	if (tb_aio_fsync(O_SYNC, &syncaiocb) == -1) {
		vsrt_sysfail("aio_fsync");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}


	PATH_TRACE;
	DBUG_PRINTF("t2readf", "wait for reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t2readf", "ensure the fsync is done too");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if ((retval2 = aio_error(&syncaiocb)) == EINPROGRESS)
			sleep(2);
		else
			break;
	}
	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("aio_fsync request is still in progress");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(2);

	if (retval2 != 0) {
		in_rpt("aio_error indicated aio_fsync request error status is %d (%s)", retval2, errname(retval2));
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if ((retval = aio_return(&syncaiocb)) != 0) {
		in_rpt("aio_return returned %d for aio_fsync request", retval);
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	(void)aio_cancel(readfd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_SYNCHRONOUS_IO are
defined  or the implementation supports the aio_fsync()
function as described in System Interfaces and Headers,
Issue 5:
	On   a   call    to    aio_fsync()    when    aiocbp-
	>aio_sigev.sigev_notify is SIGEV_NONE no asynchronous
	notification shall be delivered  when  the  operation
	completes.
*/

private void test3()
{
	int	terrors;

	DBUG_ENTER("test3");

#if !defined(_POSIX_ASYNCHRONOUS_IO) || !defined(_POSIX_SYNCHRONOUS_IO)
	if (!supports_aio_fsync) {
		in_rpt("aio_fsync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 10)
		aio_max = 10;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t3", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest3read);
	vsrt_aio_test_file(dotest3readfile);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest3read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	syncaiocb;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	int	err;
	int	retval2;

	DBUG_ENTER("dotest3read");

	errors = 0;

	if (fpathconf(readfd, _PC_SYNC_IO) == -1) {
		in_rpt("Synchronous I/O not supported for this file");
		DBUG_RETURN(1);
	}

	got_sig = got_sig_min = got_sig_max = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);
	if (vsrt_rt_signal(SIGRTMAX, rt_handler7) == -1)
		DBUG_RETURN(-1);


	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t3read", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*13];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t3read", "ensure they are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t3read", "do an aio_fsync");
	syncaiocb.aio_fildes = readfd;
	syncaiocb.aio_buf = NULL;
	syncaiocb.aio_offset = 0;
	syncaiocb.aio_nbytes = 0;
	syncaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	syncaiocb.aio_sigevent.sigev_signo = SIGRTMAX;
	syncaiocb.aio_sigevent.sigev_value.sival_int = 100;
	if (tb_aio_fsync(O_SYNC, &syncaiocb) == -1) {
		vsrt_sysfail("aio_fsync");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(2);

	PATH_TRACE;
	DBUG_PRINTF("t3read", "ensure the aio_fsync is in progress");
	if ((retval2 = aio_error(&syncaiocb)) != EINPROGRESS) {
		in_rpt("aio_fsync request is not in progress, its error status is %d (%s)", retval2, errname(retval2));
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t3read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t3read", "wait for reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t3read", "ensure the fsync is done too");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if ((retval2 = aio_error(&syncaiocb)) == EINPROGRESS)
			sleep(2);
		else {
			if (retval2 != 0) {
				in_rpt("aio_fsync error status is %s", errname(retval2));
				(void)aio_cancel(readfd, NULL);
				free(plist);
				free(paiocbs);
				free(bufs);
				DBUG_RETURN(-1);
			}
			break;
		}
	}
	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("aio_fsync request is still in progress");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	sleeploop(2);

	PATH_TRACE;
	DBUG_PRINTF("t3read", "check notification");
	if (got_sig != 0) {
		in_rpt("Expected no signals, received %d", got_sig);
		errors++;
	}

	(void)aio_cancel(readfd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}
private int dotest3readfile(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	syncaiocb;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	int	err;
	int	retval2;

	DBUG_ENTER("dotest3readfile");

	errors = 0;

	if (fpathconf(readfd, _PC_SYNC_IO) == -1) {
		in_rpt("Synchronous I/O not supported for this file");
		DBUG_RETURN(1);
	}

	got_sig = got_sig_min = got_sig_max = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);
	if (vsrt_rt_signal(SIGRTMAX, rt_handler7) == -1)
		DBUG_RETURN(-1);


	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t3readf", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*13];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t3readf", "do an aio_fsync");
	syncaiocb.aio_fildes = readfd;
	syncaiocb.aio_buf = NULL;
	syncaiocb.aio_offset = 0;
	syncaiocb.aio_nbytes = 0;
	syncaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	syncaiocb.aio_sigevent.sigev_signo = SIGRTMAX;
	syncaiocb.aio_sigevent.sigev_value.sival_int = 100;
	if (tb_aio_fsync(O_SYNC, &syncaiocb) == -1) {
		vsrt_sysfail("aio_fsync");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}


	PATH_TRACE;
	DBUG_PRINTF("t3readf", "wait for reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t3readf", "ensure the fsync is done too");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if ((retval2 = aio_error(&syncaiocb)) == EINPROGRESS)
			sleep(2);
		else {
			if (retval2 != 0) {
				in_rpt("aio_fsync error status is %s", errname(retval2));
				(void)aio_cancel(readfd, NULL);
				free(plist);
				free(paiocbs);
				free(bufs);
				DBUG_RETURN(-1);
			}
			break;
		}
	}
	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("aio_fsync request is still in progress");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	sleeploop(2);

	PATH_TRACE;
	DBUG_PRINTF("t3readf", "check notification");
	if (got_sig != 0) {
		in_rpt("Expected no signals, received %d", got_sig);
		errors++;
	}

	(void)aio_cancel(readfd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_SYNCHRONOUS_IO are
defined  or the implementation supports the aio_fsync()
function as described in System Interfaces and Headers,
Issue 5:
	On   a   call    to    aio_fsync()    when    aiocbp-
	>aio_sigev.sigev_notify  is  SIGEV_SIGNAL  the signal
	specified in aiocbp->aio_sigev.sigev_signo  shall  be
	generated  for  the  process  when the operation com-
	pletes.
*/

private void test4()
{
	int	terrors;

	DBUG_ENTER("test4");

#if !defined(_POSIX_ASYNCHRONOUS_IO) || !defined(_POSIX_SYNCHRONOUS_IO)
	if (!supports_aio_fsync) {
		in_rpt("aio_fsync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 10)
		aio_max = 10;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t4", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest4read);
	vsrt_aio_test_file(dotest4readfile);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest4read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	syncaiocb;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	int	err;
	ssize_t	retval;
	int	retval2;
	unsigned char got_data[100];
	int	c, rval;
	sigset_t	set;

	DBUG_ENTER("dotest4read");

	errors = 0;

	if (fpathconf(readfd, _PC_SYNC_IO) == -1) {
		in_rpt("Synchronous I/O not supported for this file");
		DBUG_RETURN(1);
	}

	got_sig = got_sig_min = got_sig_max = 0;

	rval = sigemptyset(&(set));
	if (rval == -1) {
		in_rpt("sigemptyset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	rval = sigaddset(&set, SIGRTMAX);
	if (rval == -1) {
		in_rpt("sigaddset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	if (vsrt_rt_signal_mask(SIGRTMIN, rt_handler7, set) == -1)
		DBUG_RETURN(-1);

	rval = sigemptyset(&(set));
	if (rval == -1) {
		in_rpt("sigemptyset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	rval = sigaddset(&set, SIGRTMIN);
	if (rval == -1) {
		in_rpt("sigemptyset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	if (vsrt_rt_signal_mask(SIGRTMAX, rt_handler7, set) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t4read", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*13];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t4read", "ensure they are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t4read", "do an aio_fsync");
	syncaiocb.aio_fildes = readfd;
	syncaiocb.aio_buf = NULL;
	syncaiocb.aio_offset = 0;
	syncaiocb.aio_nbytes = 0;
	syncaiocb.aio_sigevent.sigev_notify = SIGEV_SIGNAL;
	syncaiocb.aio_sigevent.sigev_signo = SIGRTMAX;
	syncaiocb.aio_sigevent.sigev_value.sival_int = 100;
	if (tb_aio_fsync(O_SYNC, &syncaiocb) == -1) {
		vsrt_sysfail("aio_fsync");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(2);

	PATH_TRACE;
	DBUG_PRINTF("t4read", "ensure the aio_fsync is in progress");
	if ((retval2 = aio_error(&syncaiocb)) != EINPROGRESS) {
		in_rpt("aio_fsync request is not in progress, its error status is %d (%s)", retval2, errname(retval2));
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t4read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t4read", "wait for reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t4read", "ensure the fsync is done too");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if ((retval2 = aio_error(&syncaiocb)) == EINPROGRESS)
			sleep(2);
		else {
			if (retval2 != 0) {
				in_rpt("aio_fsync error status is %s", errname(retval2));
				(void)aio_cancel(readfd, NULL);
				free(plist);
				free(paiocbs);
				free(bufs);
				DBUG_RETURN(-1);
			}
			break;
		}
	}
	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("aio_fsync request is still in progress");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	sleeploop(4);

	PATH_TRACE;
	DBUG_PRINTF("t4read", "check the results");
	memset(got_data, 0xff, 100);
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (retval != strlen(tbuf)) {
			in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
			errors++;
			continue;
		}
		if (sscanf((char *)paiocbs[i].aio_buf, "message %3d\n", &c) != 1) {
		
			in_rpt("Could not match returned data for request %d, \"message xxx \\n\", received \"%s\"", i, paiocbs[i].aio_buf);
			errors++;
		}
		else {
			if ((c < 0) || (c >= aio_max)) {
				in_rpt("Message number returned in request %d is %d, expected between 0 and %d", i, c, aio_max-1);
				errors++;
			} else
				got_data[c] = 0;
		}
	}
	for (i = aio_max-1; i >= 0; i--) {
		if (got_data[i] == 0xff) {
			in_rpt("Message number %d not read by any request", i);
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t4read", "check notification");
	if (got_sig_min != aio_max) {
		in_rpt("Expected %d signals from aio_reads, received %d", aio_max, got_sig_min);
		errors++;
	}

	if (got_sig_max == 0) {
		in_rpt("Signal from aio_fsync operation not received");
		errors++;
	}

	if (got_sig_max != got_sig_min+1) {
		in_rpt("Expected signal from aio_fsync operation to be received after those from aio_reads, got_sig_min = %d, got_sig_max = %d", got_sig_min, got_sig_max);
		errors++;
	}

	(void)aio_cancel(readfd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest4readfile(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	syncaiocb;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	int	err;
	int	retval2;
	int	rval;
	sigset_t	set;

	DBUG_ENTER("dotest4readfile");

	errors = 0;

	if (fpathconf(readfd, _PC_SYNC_IO) == -1) {
		in_rpt("Synchronous I/O not supported for this file");
		DBUG_RETURN(1);
	}

	got_sig = got_sig_min = got_sig_max = 0;

	rval = sigemptyset(&(set));
	if (rval == -1) {
		in_rpt("sigemptyset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	rval = sigaddset(&set, SIGRTMAX);
	if (rval == -1) {
		in_rpt("sigaddset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	if (vsrt_rt_signal_mask(SIGRTMIN, rt_handler7, set) == -1)
		DBUG_RETURN(-1);

	rval = sigemptyset(&(set));
	if (rval == -1) {
		in_rpt("sigemptyset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	rval = sigaddset(&set, SIGRTMIN);
	if (rval == -1) {
		in_rpt("sigemptyset failed, errno = %s", errname(errno));
		DBUG_RETURN(-1);
	}
	if (vsrt_rt_signal_mask(SIGRTMAX, rt_handler7, set) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t4readf", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*13];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t4readf", "do an aio_fsync");
	syncaiocb.aio_fildes = readfd;
	syncaiocb.aio_buf = NULL;
	syncaiocb.aio_offset = 0;
	syncaiocb.aio_nbytes = 0;
	syncaiocb.aio_sigevent.sigev_notify = SIGEV_SIGNAL;
	syncaiocb.aio_sigevent.sigev_signo = SIGRTMAX;
	syncaiocb.aio_sigevent.sigev_value.sival_int = 100;
	if (tb_aio_fsync(O_SYNC, &syncaiocb) == -1) {
		vsrt_sysfail("aio_fsync");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}


	PATH_TRACE;
	DBUG_PRINTF("t4readf", "wait for reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t4readf", "ensure the fsync is done too");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if ((retval2 = aio_error(&syncaiocb)) == EINPROGRESS)
			sleep(2);
		else {
			if (retval2 != 0) {
				in_rpt("aio_fsync error status is %s", errname(retval2));
				(void)aio_cancel(readfd, NULL);
				free(plist);
				free(paiocbs);
				free(bufs);
				DBUG_RETURN(-1);
			}
			break;
		}
	}
	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("aio_fsync request is still in progress");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	sleeploop(4);

	PATH_TRACE;
	DBUG_PRINTF("t4readf", "check notification");
	if (got_sig_min != aio_max) {
		in_rpt("Expected %d signals from aio_reads, received %d", aio_max, got_sig_min);
		errors++;
	}

	if (got_sig_max == 0) {
		in_rpt("Signal from aio_fsync operation not received");
		errors++;
	}

	if (got_sig_max != got_sig_min+1) {
		in_rpt("Expected signal from aio_fsync operation to be received after those from aio_reads, got_sig_min = %d, got_sig_max = %d", got_sig_min, got_sig_max);
		errors++;
	}

	(void)aio_cancel(readfd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If  _POSIX_ASYNCHRONOUS_IO,  _POSIX_SYNCHRONOUS_IO  and
_POSIX_REALTIME_SIGNALS  are defined or the implementa-
tion supports the aio_fsync() function as described  in
System Interfaces and Headers, Issue 5:
	On   a   call    to    aio_fsync()    when    aiocbp-
	>aio_sigev.sigev_notify   is   SIGEV_SIGNAL  and  the
	SA_SIGINFO flag is set for the  signal  specified  in
	aiocbp->aio_sigev.sigev_signo,  the  signal  shall be
	queued for the process when it is generated  and  the
	value   specified   in  aiocbp->aio_sigev.sigev_value
	shall be in the si_value component of  the  generated signal.
*/

private void test5()
{
	int	terrors;

	DBUG_ENTER("test5");

#if !defined(_POSIX_ASYNCHRONOUS_IO) || !defined(_POSIX_SYNCHRONOUS_IO)
	if (!supports_aio_fsync) {
		in_rpt("aio_fsync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 10)
		aio_max = 10;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t5", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest5read);
	vsrt_aio_test_file(dotest5readfile);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest5read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	syncaiocb;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	int	err;
	ssize_t	retval;
	siginfo_t	info;
	struct timespec	timeout;
	int	retval2;
	sigset_t	set;
	unsigned char got_data[100];
	int	c;

	DBUG_ENTER("dotest5read");

	errors = 0;

	if (fpathconf(readfd, _PC_SYNC_IO) == -1) {
		in_rpt("Synchronous I/O not supported for this file");
		DBUG_RETURN(1);
	}

	got_sig = got_sig_min = got_sig_max = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);
	if (vsrt_rt_signal(SIGRTMAX, rt_handler7) == -1)
		DBUG_RETURN(-1);
	DBUG_PRINTF("t5read", "block the signals to cause queuing");
	sigemptyset(&set);
	if (sigaddset(&set, SIGRTMIN) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}
	if (sigaddset(&set, SIGRTMAX) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}
	if (sigprocmask(SIG_BLOCK, &set, NULL) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}


	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t5read", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*13];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t5read", "ensure reads are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t5read", "do an aio_fsync");
	syncaiocb.aio_fildes = readfd;
	syncaiocb.aio_buf = NULL;
	syncaiocb.aio_offset = 0;
	syncaiocb.aio_nbytes = 0;
	syncaiocb.aio_sigevent.sigev_notify = SIGEV_SIGNAL;
	syncaiocb.aio_sigevent.sigev_signo = SIGRTMAX;
	syncaiocb.aio_sigevent.sigev_value.sival_int = 100;
	if (tb_aio_fsync(O_SYNC, &syncaiocb) == -1) {
		vsrt_sysfail("aio_fsync");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(2);

	PATH_TRACE;
	DBUG_PRINTF("t5read", "ensure the fsync is in progress");
	if ((retval2 = aio_error(&syncaiocb)) != EINPROGRESS) {
		in_rpt("aio_fsync request is not in progress, its error status is %d (%s)", retval2, errname(retval2));
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t5read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t5read", "wait for reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t5read", "ensure the fsync is done too");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if ((retval2 = aio_error(&syncaiocb)) == EINPROGRESS)
			sleep(2);
		else {
			if (retval2 != 0) {
				in_rpt("aio_fsync error status is %s", errname(retval2));
				(void)aio_cancel(readfd, NULL);
				free(plist);
				free(paiocbs);
				free(bufs);
				DBUG_RETURN(-1);
			}
			break;
		}
	}
	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("aio_fsync request is still in progress");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	sleeploop(2);

	PATH_TRACE;
	DBUG_PRINTF("t5read", "check the results");
	memset(got_data, 0xff, 100);
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (retval != strlen(tbuf)) {
			in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
			errors++;
			continue;
		}
		if (sscanf((char *)paiocbs[i].aio_buf, "message %3d\n", &c) != 1) {
		
			in_rpt("Could not match returned data for request %d, \"message xxx \\n\", received \"%s\"", i, paiocbs[i].aio_buf);
			errors++;
		}
		else {
			if ((c < 0) || (c >= aio_max)) {
				in_rpt("Message number returned in request %d is %d, expected between 0 and %d", i, c, aio_max-1);
				errors++;
			} else
				got_data[c] = 0;
		}
	}
	for (i = aio_max-1; i >= 0; i--) {
		if (got_data[i] == 0xff) {
			in_rpt("Message number %d not read by any request", i);
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t5read", "check notification");
	if (got_sig != 0) {
		in_rpt("SIGRTMIN and SIGRTMAX blocked but %d signals were delivered", got_sig);
		errors++;
	}

	timeout.tv_sec = 1;
	timeout.tv_nsec = 0;
	memset(got_requests, 0xff, 50);
	for (i = 0; i < aio_max; i++) {
		if (sigtimedwait(&set, &info, &timeout) == -1) {
			err = errno;
			in_rpt("sigtimedwait failed on signal #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
		else {
			/*save these as we may be getting the requests back
			out of order*/
			if (info.si_value.sival_int < aio_max)
				got_requests[info.si_value.sival_int] = info.si_value.sival_int;
			else {
				in_rpt("si_value for signal %d = %d, maximum value in requests is %d", i, info.si_value.sival_int, aio_max);
				errors++;
			}
			if (info.si_signo != SIGRTMIN) {
				in_rpt("Expected si_signo for signal %d = %d, is %d", i, SIGRTMIN, info.si_signo);
				errors++;
			}
			if (info.si_code != SI_ASYNCIO) {
				in_rpt("Expected si_code for signal %d = %d, is %d", i, SI_ASYNCIO, info.si_code);
				errors++;
			}
		}
	}
	for (i = 0; i < aio_max; i++) {
		if (got_requests[i] == 0xff) {
			in_rpt("No signal received for request %d", i);
			errors++;
			continue;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t5read", "now the aio_fsync one");
	if (sigtimedwait(&set, &info, &timeout) == -1) {
		err = errno;
		in_rpt("sigtimedwait failed on signal from aio_fsync, errno = %d (%s)", err, errname(err));
		errors++;
	}
	else {
		if (info.si_value.sival_int != 100) {
			in_rpt("Expected si_value for signal from aio_fsync = %d, is %d", 100, info.si_value.sival_int);
			errors++;
		}
		if (info.si_signo != SIGRTMAX) {
			in_rpt("Expected si_signo for signal from aio_fsync = %d, is %d", SIGRTMAX, info.si_signo);
			errors++;
		}
		if (info.si_code != SI_ASYNCIO) {
			in_rpt("Expected si_code for signal from aio_fsync = %d, is %d", SI_ASYNCIO, info.si_code);
			errors++;
		}
	}

	(void)aio_cancel(readfd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest5readfile(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	syncaiocb;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	int	err;
	siginfo_t	info;
	struct timespec	timeout;
	int	retval2;
	sigset_t	set;

	DBUG_ENTER("dotest5readfile");

	errors = 0;

	if (fpathconf(readfd, _PC_SYNC_IO) == -1) {
		in_rpt("Synchronous I/O not supported for this file");
		DBUG_RETURN(1);
	}

	got_sig = got_sig_min = got_sig_max = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);
	if (vsrt_rt_signal(SIGRTMAX, rt_handler7) == -1)
		DBUG_RETURN(-1);
	DBUG_PRINTF("t5readf", "block the signals to cause queuing");
	sigemptyset(&set);
	if (sigaddset(&set, SIGRTMIN) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}
	if (sigaddset(&set, SIGRTMAX) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}
	if (sigprocmask(SIG_BLOCK, &set, NULL) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}


	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t5readf", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*13];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read %d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t5readf", "do an aio_fsync");
	syncaiocb.aio_fildes = readfd;
	syncaiocb.aio_buf = NULL;
	syncaiocb.aio_offset = 0;
	syncaiocb.aio_nbytes = 0;
	syncaiocb.aio_sigevent.sigev_notify = SIGEV_SIGNAL;
	syncaiocb.aio_sigevent.sigev_signo = SIGRTMAX;
	syncaiocb.aio_sigevent.sigev_value.sival_int = 100;
	if (tb_aio_fsync(O_SYNC, &syncaiocb) == -1) {
		vsrt_sysfail("aio_fsync");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}


	PATH_TRACE;
	DBUG_PRINTF("t5readf", "wait for reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t5readf", "ensure the fsync is done too");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if ((retval2 = aio_error(&syncaiocb)) == EINPROGRESS)
			sleep(2);
		else {
			if (retval2 != 0) {
				in_rpt("aio_fsync error status is %s", errname(retval2));
				(void)aio_cancel(readfd, NULL);
				free(plist);
				free(paiocbs);
				free(bufs);
				DBUG_RETURN(-1);
			}
			break;
		}
	}
	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("aio_fsync request is still in progress");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleeploop(2);

	PATH_TRACE;
	DBUG_PRINTF("t5readf", "check notification");
	if (got_sig != 0) {
		in_rpt("SIGRTMIN and SIGRTMAX blocked but %d signals were delivered", got_sig);
		errors++;
	}

	timeout.tv_sec = 1;
	timeout.tv_nsec = 0;
	memset(got_requests, 0xff, 50);
	for (i = 0; i < aio_max; i++) {
		if (sigtimedwait(&set, &info, &timeout) == -1) {
			err = errno;
			in_rpt("sigtimedwait failed on signal #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
		else {
			/*save these as we may be getting the requests back
			out of order*/
			if (info.si_value.sival_int < aio_max)
				got_requests[info.si_value.sival_int] = info.si_value.sival_int;
			else {
				in_rpt("si_value for signal %d = %d, maximum value in requests is %d", i, info.si_value.sival_int, aio_max);
				errors++;
			}
			if (info.si_signo != SIGRTMIN) {
				in_rpt("Expected si_signo for signal %d = %d, is %d", i, SIGRTMIN, info.si_signo);
				errors++;
			}
			if (info.si_code != SI_ASYNCIO) {
				in_rpt("Expected si_code for signal %d = %d, is %d", i, SI_ASYNCIO, info.si_code);
				errors++;
			}
		}
	}
	for (i = 0; i < aio_max; i++) {
		if (got_requests[i] == 0xff) {
			in_rpt("No signal received for request %d", i);
			errors++;
			continue;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t5readf", "now the aio_fsync one");
	if (sigtimedwait(&set, &info, &timeout) == -1) {
		err = errno;
		in_rpt("sigtimedwait failed on signal from aio_fsync, errno = %d (%s)", err, errname(err));
		errors++;
	}
	else {
		if (info.si_value.sival_int != 100) {
			in_rpt("Expected si_value for signal from aio_fsync = %d, is %d", 100, info.si_value.sival_int);
			errors++;
		}
		if (info.si_signo != SIGRTMAX) {
			in_rpt("Expected si_signo for signal from aio_fsync = %d, is %d", SIGRTMAX, info.si_signo);
			errors++;
		}
		if (info.si_code != SI_ASYNCIO) {
			in_rpt("Expected si_code for signal from aio_fsync = %d, is %d", SI_ASYNCIO, info.si_code);
			errors++;
		}
	}

	(void)aio_cancel(readfd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If  _POSIX_ASYNCHRONOUS_IO,  _POSIX_SYNCHRONOUS_IO  and
_POSIX_REALTIME_SIGNALS  are defined or the implementa-
tion supports the aio_fsync() function as described  in
System Interfaces and Headers, Issue 5:
	On   a   call    to    aio_fsync()    when    aiocbp-
	>aio_sigev.sigev_notify  is SIGEV_THREAD the function
	in aiocbp->aio_sigev.sigev_notify_function  shall  be
	called      with     the     value     of     aiocbp-
	>aio_sigev.sigev_value as a parameter when the opera-
	tion  completes,  executed in an environment as if it
	were the start_routine for  a  newly  created  thread
	with   thread   attributes   specified   by   aiocbp-
	>aio_sigev.sigev_notify_attributes.
*/

private void test6()
{
	DBUG_ENTER("test6");

#if !defined(_POSIX_ASYNCHRONOUS_IO) || !defined(_POSIX_SYNCHRONOUS_IO)
	if (!supports_aio_fsync) {
		in_rpt("aio_fsync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

/*
If  _POSIX_ASYNCHRONOUS_IO,  _POSIX_SYNCHRONOUS_IO  and
_POSIX_REALTIME_SIGNALS  are defined or the implementa-
tion supports the aio_fsync() function as described  in
System Interfaces and Headers, Issue 5:
	On   a   call    to    aio_fsync()    when    aiocbp-
	>aio_sigev.sigev_notify  is  SIGEV_THREAD and aiocbp-
	>aio_sigev.sigev_notify_attributes is NULL the  func-
	tion in aiocbp->aio_sigev.sigev_notify_function shall
	be    called    with    the    value    of    aiocbp-
	>aio_sigev.sigev_value as a parameter when the opera-
	tion completes, executed in an environment as  if  it
	were  the  start_routine  for  a newly created thread
	with    the    detachstate    attribute    set     to
	PTHREAD_CREATE_DETACHED.
*/

private void test7()
{
	DBUG_ENTER("test7");

#if !defined(_POSIX_ASYNCHRONOUS_IO) || !defined(_POSIX_SYNCHRONOUS_IO)
	if (!supports_aio_fsync) {
		in_rpt("aio_fsync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_SYNCHRONOUS_IO are
defined  or the implementation supports the aio_fsync()
function as described in System Interfaces and Headers,
Issue 5:
	When aiocbp is NULL a call to aio_fsync()  shall  not
	cause  a  signal to be generated on completion of the
	operation.
*/

private void test8()
{
	int	terrors;

	DBUG_ENTER("test8");

#if !defined(_POSIX_ASYNCHRONOUS_IO) || !defined(_POSIX_SYNCHRONOUS_IO)
	if (!supports_aio_fsync) {
		in_rpt("aio_fsync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	/*do an aio_fsync*/
	if (tb_aio_fsync(O_SYNC, NULL) == -1) {
		vsrt_sysfail("aio_fsync");
		DBUG_VOID_RETURN
	}

	xs_rpt();
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_SYNCHRONOUS_IO are
defined  or the implementation supports the aio_fsync()
function as described in System Interfaces and Headers,
Issue 5:
	On a call to aio_fsync() all  members  of  the  aiocb
	structure  referenced by aiocbp other than aio_fildes
	and aio_sigevent shall be ignored.
*/

private void test9()
{
	int	terrors;

	DBUG_ENTER("test9");

#if !defined(_POSIX_ASYNCHRONOUS_IO) || !defined(_POSIX_SYNCHRONOUS_IO)
	if (!supports_aio_fsync) {
		in_rpt("aio_fsync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 10)
		aio_max = 10;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t9", "using %d requests", aio_max);

	/*we'll just run this code again as the assert is tested there*/
	vsrt_aio_test_allbutfile(dotest1read);
	vsrt_aio_test_file(dotest1readfile);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_SYNCHRONOUS_IO are
defined  or the implementation supports the aio_fsync()
function as described in System Interfaces and Headers,
Issue 5:
	A call to aio_fsync() shall return -1 and  set  errno
	to  EBADF when aiocbp->aio_fildes is not a valid file
	descriptor open for writing.
*/

private void test10()
{
	int	fd;
	struct aiocb	myaiocb;
	char	buf[32];
	pid_t	pid;
	int	status, errors;

	DBUG_ENTER("test10");

#if !defined(_POSIX_ASYNCHRONOUS_IO) || !defined(_POSIX_SYNCHRONOUS_IO)
	if (!supports_aio_fsync) {
		in_rpt("aio_fsync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	errors = 0;

	VSRT_FORK(pid);
		if (vsrt_signal(SIGUSR1, handler5) == -1)
			exit(1);

		(void)unlink("vsrt_aio");
		if ((fd = open("vsrt_aio", O_RDWR|O_CREAT, VSRT_PROT_ALL)) == -1) {
			vsrt_sysfail("open");
			exit(1);
		}

		if (close(fd) == -1) {
			vsrt_sysfail("close");
			(void)unlink("vsrt_aio");
			exit(1);
		}
	
		(void)unlink("vsrt_aio");
		
		myaiocb.aio_fildes = fd+500;
		myaiocb.aio_buf = buf;
		myaiocb.aio_offset = 0;
		myaiocb.aio_nbytes = 2;
		myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
		myaiocb.aio_sigevent.sigev_signo = SIGUSR1;
		myaiocb.aio_sigevent.sigev_value.sival_int = 0;
		myaiocb.aio_reqprio= 0;

		if (tb_aio_fsync(O_SYNC, &myaiocb) != -1) {
			in_rpt("aio_fsync succeeded");
			xx_rpt(FAILURE);
			exit(1);
		}
	
		if (errno != EBADF) {
			vsrt_sysfail("aio_fsync");
			xx_rpt(FAILURE);
			exit(1);
		}
	
	
	VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	if (errors == 0)
		xs_rpt();
	
	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_SYNCHRONOUS_IO are
defined  or the implementation supports the aio_fsync()
function as described in System Interfaces and Headers,
Issue  5  and there is a file type for which the imple-
mentation does not support synchronized I/O:
	A call to aio_fsync() shall return -1 and  set  errno
	to  EINVAL  when  the implementation does not support
	synchronized I/O for this file.
*/
		
private void test11()
{
	struct aiocb	myaiocb;
	char	buf[32];
	pid_t	pid;
	int	status, errors;
	int	fd;
	char	*str;

	DBUG_ENTER("test11");

#if !defined(_POSIX_ASYNCHRONOUS_IO) || !defined(_POSIX_SYNCHRONOUS_IO)
	if (!supports_aio_fsync) {
		in_rpt("aio_fsync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	errors = 0;

	str = tet_getvar("VSRT_FILE_NO_SYNC_IO");
	if ((str == NULL) || (strlen(str) == 0)) {
		in_rpt("VSRT_FILE_NO_SYNC_IO not defined in tetexec.cfg");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	VSRT_FORK(pid);
		if (vsrt_signal(SIGUSR1, handler5) == -1)
			exit(1);

		if ((fd = open(str, O_RDWR)) == -1) {
			vsrt_sysfail("open");
			exit(1);
		}

		myaiocb.aio_fildes = fd;
		myaiocb.aio_buf = buf;
		myaiocb.aio_offset = 0;
		myaiocb.aio_nbytes = 2;
		myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
		myaiocb.aio_sigevent.sigev_signo = SIGUSR1;
		myaiocb.aio_sigevent.sigev_value.sival_int = 0;
		myaiocb.aio_reqprio= 0;

		if (tb_aio_fsync(O_SYNC, &myaiocb) != -1) {
			in_rpt("aio_fsync succeeded");
			xx_rpt(FAILURE);
			exit(1);
		}
	
		if (errno != EINVAL) {
			vsrt_sysfail("aio_fsync");
			xx_rpt(FAILURE);
			exit(1);
		}
	
	
	VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	if (errors == 0)
		xs_rpt();
	
	DBUG_VOID_RETURN
}


/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_SYNCHRONOUS_IO are
defined  or the implementation supports the aio_fsync()
function as described in System Interfaces and Headers,
Issue 5:
	A call to aio_fsync() shall return -1 and  set  errno
	to  EINVAL when the value of op is neither O_DSYNC or
	O_SYNC.
*/

private void test12()
{
	int	fd;
	struct aiocb	myaiocb;
	char *str;
	pid_t	pid;
	int	status, errors;

	DBUG_ENTER("test12");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_fsync) {
		in_rpt("aio_fsync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	errors = 0;

	str = tet_getvar("VSRT_FILE_SYNC_IO");
	if ((str == NULL) || (strlen(str) == 0)) {
		in_rpt("VSRT_FILE_SYNC_IO not defined in tetexec.cfg");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	VSRT_FORK(pid);
		VSRT_SIGNAL(SIGUSR1, handler5);

		if ((fd = open(str, O_RDWR)) == -1) {
			vsrt_sysfail("open");
			exit(1);
		}
		myaiocb.aio_fildes = fd;
		myaiocb.aio_buf = NULL;
		myaiocb.aio_offset = 0;
		myaiocb.aio_nbytes = 0;
		myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
		myaiocb.aio_sigevent.sigev_signo = SIGUSR1;
		myaiocb.aio_sigevent.sigev_value.sival_int = 0;
		myaiocb.aio_reqprio= 0;
	
		errno = 512;
		if (tb_aio_fsync(O_RSYNC, &myaiocb) != -1) {
			in_rpt("aio_fsync succeeded");
			xx_rpt(FAILURE);
			exit(1);
		}
		if (errno != EINVAL) {
			vsrt_sysfail("aio_fsync");
			exit(1);
		}
	
	VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	if (errors == 0)
		xs_rpt();
	
	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is not defined and the imple-
mentation does not support the aio_fsync() function as
described in System Interfaces and Headers, Issue 5:
	A call to aio_fsync() shall return -1 and set errno
	to ENOSYS.
*/

private void test13()
{
#ifndef _POSIX_ASYNCHRONOUS_IO
	struct aiocb	myaiocb;
#endif

	DBUG_ENTER("test13");

#ifdef _POSIX_ASYNCHRONOUS_IO
	in_rpt("_POSIX_ASYNCHRONOUS_IO defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN
#else
	if (supports_aio_fsync) {
		in_rpt("aio_fsync supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	if (tb_aio_fsync(0, &myaiocb) != -1) {
		in_rpt("aio_fsync succeeded");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (errno != ENOSYS) {
		vsrt_sysfail("aio_fsync");
		DBUG_VOID_RETURN
	}

	xs_rpt();

	DBUG_VOID_RETURN
#endif
}

private	int tb_aio_fsync(op, aiocbp)
int	op;
struct aiocb	*aiocbp;
{
	int 	retval;

	retval = aio_fsync(op, aiocbp);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
