/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/aio/aio_read/aio_read.c
*
* Modifications:
* $Log: aio_read.c,v $
* Revision 1.2  2001/10/02 13:25:12  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:13  cyeoh
* Adds files for lsb-os test suite
*
*/


#include	<vsrt/vsrt.h>
#include	<aio.h>
#include	<limits.h>
#include	<setjmp.h>

#define 	NO_TESTS	15	/* number of tests in testset */

#ifdef UNDEF_MACROS
#undef aio_read
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();

int	no_tests = NO_TESTS;

private	int	tb_aio_read();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();

private	int	dotest1();
private	int	dotest1block();
private	int	dotest1nonblock();
private	int	dotest1noorder();
private	int	dotest4();
private	int	dotest5();
private	int	dotest6();
private	int	dotest7();
private	int	dotest10();
private	int	dotest12();
private	int	dotest13();
private	int	dotest14();

int	supports_aio_read;
int	supports_aio_error;
int	supports_aio_return;
int	supports_aio_suspend;

/*determine if a type of file is to be tried. set in startup*/
/*pipe support is determined dynamically*/
int	vsrt_aio_test_with_tty = 0;
int	vsrt_aio_test_with_file = 0;
int	vsrt_aio_test_with_pty = 0;

/*each piece of a test increments on of these.  must be zeroed at test start*/
int	vsrt_aio_test_pass;
int	vsrt_aio_test_unsup;
int	vsrt_aio_test_fail;

/*instances of things to test on from tetexec, loaded in startup*/
char	vsrt_aio_tty1[128];
char	vsrt_aio_tty2[128];
char	vsrt_aio_ptym[128];
char	vsrt_aio_testfile[4096];

void atstart(void);
void atstart(void)
{
	char *str;

	vsrt_check_support("AIO_READ", &supports_aio_read);
	vsrt_check_support("AIO_ERROR", &supports_aio_error);
	vsrt_check_support("AIO_RETURN", &supports_aio_return);
	vsrt_check_support("AIO_SUSPEND", &supports_aio_suspend);

	/*get config info and determine what sorts of files to test with*/
	str = tet_getvar("VSRT_FILE_ASYNC_IO");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_file = 1;
		strcpy(vsrt_aio_testfile, str);
	}

	str = tet_getvar("VSRT_TERMIOS_TTY");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_tty = 1;
		strcpy(vsrt_aio_tty1, str);
		str = tet_getvar("VSRT_TERMIOS_LOOP");
		if ((str != NULL) && (strlen(str) != 0))
			strcpy(vsrt_aio_tty2, str);
		else
			vsrt_aio_test_with_tty = 0;
	}

	str = tet_getvar("VSRT_MASTER_PTY");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_pty = 1;
		strcpy(vsrt_aio_ptym, str);
	}
}

void atend(void);
void atend(void)
{
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = atend;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	NULL, 0
};

int	aio_max = 0;
long	aio_maxl = 0;

int	got_sig = 0;

private	void	handler5();
private	void	handler5(signo)
int	signo;
{
	got_sig++;
}

private	void	handler6();
private	void	handler6(signo)
int	signo;
{
	got_sig++;
}

private void rt_handler7();
private void rt_handler7(signo, info, context)
int	signo;
siginfo_t	*info;
void	*context;
{
	got_sig++;
}

unsigned char	got_requests[50];


/*
If _POSIX_ASYNCHRONOUS_IO is defined  or the implementation supports
the aio_read() function as described in System Interfaces and Headers,
Issue 5:
	A successful call to  aio_read()  shall  initiate  or
	queue  a  request on behalf of the calling process to
	read aiocbp->aio_nbytes from the file associated with
	aiocbp->aio_fildes  into  the  buffer  pointed  to by
	aiocbp->aio_buf and return 0.
*/

private void test1()
{
	int	terrors;

	DBUG_ENTER("test1");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 50)
		aio_max = 50;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t1", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest1block);
	vsrt_aio_test_allbutfile(dotest1nonblock);
	vsrt_aio_test_file(dotest1);
	vsrt_aio_test_allbutfile(dotest1noorder);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest1(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;

	/*this code is used for files as it relies on seeking to
	find particular data at a particular offset*/
	DBUG_ENTER("dotest1");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	DBUG_PRINTF("t1", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*strlen(tbuf)];
		paiocbs[i].aio_offset = i*strlen(tbuf);
		paiocbs[i].aio_nbytes = strlen(tbuf);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (tb_aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t1", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval != strlen(tbuf)) {
			in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
			errors++;
			continue;
		}
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (strncmp((char *)paiocbs[i].aio_buf, tbuf, strlen(tbuf)) != 0) {
			bufs[strlen(tbuf)*(i+1)] = 0;
			in_rpt("For request %d, expected returned data \"%s\", received \"%s\"", i, tbuf, paiocbs[i].aio_buf);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest1noorder(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	unsigned char got_data[100];
	int	c;
	int	origflags;
	int	aio_max1;

	/*this code is used for things other than files as it doesn't
	rely on seeking to find particular data at a particular offset*/
	DBUG_ENTER("dotest1norder");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

 	DBUG_PRINTF("t1no", "write some data");
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		DBUG_RETURN(-1);
	}
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			if (errno == EAGAIN)
				break;
			else {
				vsrt_sysfail("write");
				DBUG_RETURN(-1);
			}
		}
	}

	aio_max1 = i;

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max1*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max1*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max1*(sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max1*(sizeof(tbuf)+1));

	PATH_TRACE;

	DBUG_PRINTF("t1no", "kick off some aio reads");
	for (i = 0; i < aio_max1; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*(sizeof(tbuf)+1)];
		paiocbs[i].aio_offset = i*strlen(tbuf);
		paiocbs[i].aio_nbytes = strlen(tbuf);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (tb_aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t1no", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max1; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max1)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max1; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	memset(got_data, 0xff, 100);
	DBUG_PRINTF("t1no", "check the results");
	for (i = aio_max1-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval != strlen(tbuf)) {
			in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
			errors++;
			continue;
		}
		if (sscanf((char *)paiocbs[i].aio_buf, "message %3d\n", &c) != 1) {
			in_rpt("Could not match returned data for request %d, \"message xxx\\n\", received \"%s\"", i, i, paiocbs[i].aio_buf);
			errors++;
		}
		else {
			if ((c < 0) || (c >= aio_max1)) {
				in_rpt("Message number returned in request %d is %d, expected between 0 and %d", i, c, aio_max1-1);
				errors++;
			} else {
				if (got_data[c] != 0xff) {
					in_rpt("Request %d returned message %d but it was previously returned by request %d", i, c, got_data[c]);
					errors++;
				}
				got_data[c] = i;
			}
		}
	}
	for (i = aio_max1-1; i >= 0; i--) {
		if (got_data[i] == 0xff) {
			in_rpt("Message number %d not read by any request", i);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest1block(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;

/*this code is used to check asynchrony a bit*/
	DBUG_ENTER("dotest1block");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*2)+1)) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1bl", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (tb_aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(2);

	PATH_TRACE;
	DBUG_PRINTF("t1bl", "ensure they are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1bl", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "%3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;

	DBUG_PRINTF("t1bl", "wait for reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1bl", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest1nonblock(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	int	origflags;

	/*check O_NONBLOCK operation*/
	DBUG_ENTER("dotest1nonblock");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*2)+1)) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	DBUG_PRINTF("t1nb", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (tb_aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1nb", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1nb", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != EAGAIN) {
			in_rpt("Error status for read %d is %d (%s), expected EAGAIN", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval != -1) {
			err = errno;
			in_rpt("aio_return returned %d for read #%d", retval);
			errors++;
			continue;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_PRIORITIZED_IO are
defined  or the implementation supports the aio_read()
function as described in System Interfaces and Headers,
Issue 5:
	The request generated by a call to aio_read() from  a
	process  whose  scheduling  policy  is  SCHED_FIFO or
	SCHED_RR shall be  queued  in  priority  order  at  a
	priority equal to the scheduling priority of the pro-
	cess minus aiocbp->aio_reqprio for values of  aiocbp-
	>aio_reqprio  which  lower the operation priority, in
	the range 0 to {AIO_PRIO_DELTA_MAX}.
*/

private void test2()
{
	DBUG_ENTER("test2");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

#ifndef _POSIX_PRIORITIZED_IO
	in_rpt("_POSIX_PRIORITIZED_IO not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN
#else

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
#endif
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_PRIORITIZED_IO are
defined  or the implementation supports the aio_read()
function as described in System Interfaces and Headers,
Issue 5:
	Requests  generated  by  calls  to  aio_read()   from
	processes  whose  scheduling  policy is SCHED_FIFO or
	SCHED_RR with the same priority to a  character  spe-
	cial file shall be processed by the underlying device
	in FIFO order.
*/

private void test3()
{
	DBUG_ENTER("test3");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

#ifndef _POSIX_PRIORITIZED_IO
	in_rpt("_POSIX_PRIORITIZED_IO not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN
#else

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
#endif
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion  supports  the aio_read() function as described in
System Interfaces and Headers, Issue 5:
	Unless aiocbp->aio_fildes is associated with a device
	which  is  incapable of seeking the request generated
	by a call to aio_read() shall take place at the abso-
	lute   position   in   the   file  given  by  aiocbp-
	>aio_offset.
*/

private void test4()
{
	int	terrors;

	DBUG_ENTER("test4");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 50)
		aio_max = 50;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t4", "using %d requests", aio_max);

	vsrt_aio_test_file(dotest4);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest4(readfd, writefd)
int	readfd;
int	writefd;
{
	int	errors;
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;

	DBUG_ENTER("dotest4");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	DBUG_PRINTF("t4", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t4", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*strlen(tbuf)];
		paiocbs[i].aio_offset = i*strlen(tbuf);
		paiocbs[i].aio_nbytes = strlen(tbuf);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (tb_aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t4", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t4", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval != strlen(tbuf)) {
			in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
			errors++;
			continue;
		}
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (strncmp((char *)paiocbs[i].aio_buf, tbuf, strlen(tbuf)) != 0) {
			bufs[strlen(tbuf)*(i+1)] = 0;
			in_rpt("For request %d, expected returned data \"%s\", received \"%s\"", i, tbuf, paiocbs[i].aio_buf);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion  supports  the aio_read() function as described in
System Interfaces and Headers, Issue 5:
	On   a    call    to    aio_read()    when    aiocbp-
	>aio_sigevent.sigev_notify is SIGEV_NONE no asynchro-
	nous notification shall be delivered when the  opera-
	tion completes.
*/

private void test5()
{
	int	terrors;

	DBUG_ENTER("test5");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 50)
		aio_max = 50;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t5", "using %d requests", aio_max);

	vsrt_aio_test_all(dotest5);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest5(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	int	errors;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	origflags;
	int	aio_max1;

	DBUG_ENTER("dotest5");

	got_sig = 0;
	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

 	DBUG_PRINTF("t5", "write some data");
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		DBUG_RETURN(-1);
	}
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			if (errno == EAGAIN)
				break;
			else {
				vsrt_sysfail("write");
				DBUG_RETURN(-1);
			}
		}
	}

	aio_max1 = i;

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max1*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max1*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max1*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t5", "kick off some aio reads");
	for (i = 0; i < aio_max1; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*strlen(tbuf)];
		paiocbs[i].aio_offset = i*strlen(tbuf);
		paiocbs[i].aio_nbytes = strlen(tbuf);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (tb_aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t5", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max1; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max1)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max1; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t5", "check the results");
	for (i = aio_max1-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (got_sig != 0) {
		in_rpt("Expected no signals, received %d", got_sig);
		errors++;
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion  supports  the aio_read() function as described in
System Interfaces and Headers, Issue 5:
	On   a    call    to    aio_read()    when    aiocbp-
	>aio_sigevent.sigev_notify is SIGEV_SIGNAL the signal
	specified in  aiocbp->aio_sigevent.sigev_signo  shall
	be  generated for the process when the operation com-
	pletes.
*/

private void test6()
{
	int	terrors;

	DBUG_ENTER("test6");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 25)
		aio_max = 25;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t6", "using %d requests", aio_max);

	vsrt_aio_test_all(dotest6);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest6(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	int	errors;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	origflags;
	int	aio_max1;
	int	retval2;

	DBUG_ENTER("dotest6");

	got_sig = 0;
	errors = 0;

	if (vsrt_signal(SIGUSR1, handler6) == -1)
		DBUG_RETURN(-1);

 	DBUG_PRINTF("t6", "write some data");
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		DBUG_RETURN(-1);
	}
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			if (errno == EAGAIN)
				break;
			else {
				vsrt_sysfail("write");
				DBUG_RETURN(-1);
			}
		}
	}

	aio_max1 = i;

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max1*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max1*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max1*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t6", "kick off some aio reads");
	for (i = 0; i < aio_max1; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*strlen(tbuf)];
		paiocbs[i].aio_offset = i*strlen(tbuf);
		paiocbs[i].aio_nbytes = strlen(tbuf);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (tb_aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t6", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max1; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max1)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max1; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t6", "check the results");
	for (i = aio_max1-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (got_sig == 0) {
		in_rpt("Expected signal delivery, no signals were received");
		errors++;
	}
	if (got_sig > aio_max1) {
		in_rpt("Expected at most %d signals, received %d", aio_max, got_sig);
		errors++;
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO  and  _POSIX_REALTIME_SIGNALS
are   defined   or   the  implementation  supports  the
aio_read() function as described in  System  Interfaces
and Headers, Issue 5:
	On   a    call    to    aio_read()    when    aiocbp-
	>aio_sigevent.sigev_notify  is  SIGEV_SIGNAL  and the
	SA_SIGINFO flag is set for the  signal  specified  in
	aiocbp->aio_sigevent.sigev_signo, the signal shall be
	queued for the process when it is generated  and  the
	value  specified  in aiocbp->aio_sigevent.sigev_value
	shall be in the si_value component of  the  generated
	signal.
*/

private void test7()
{
	int	terrors;

	DBUG_ENTER("test7");

#if !defined(_POSIX_ASYNCHRONOUS_IO) || !defined(_POSIX_REALTIME_SIGNALS)
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 10)
		aio_max = 10;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t7", "using %d requests", aio_max);

	vsrt_aio_test_all(dotest7);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest7(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	int	err;
	ssize_t	retval;
	siginfo_t	info;
	struct timespec	timeout;
	int	retval2;
	sigset_t	set;

	DBUG_ENTER("dotest7");

	errors = 0;
	got_sig = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);

	DBUG_PRINTF("t7", "block the signal to cause queuing");
	sigemptyset(&set);
	if (sigaddset(&set, SIGRTMIN) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}
	if (sigprocmask(SIG_BLOCK, &set, NULL) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t7", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t7", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*strlen(tbuf)];
		paiocbs[i].aio_offset = i*strlen(tbuf);
		paiocbs[i].aio_nbytes = strlen(tbuf);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (tb_aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t7", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t7", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (got_sig != 0) {
		in_rpt("SIGRTMIN is blocked by %d signals were delivered", got_sig);
		errors++;
	}

	timeout.tv_sec = 1;
	timeout.tv_nsec = 0;
	memset(got_requests, 0xff, 50);
	for (i = 0; i < aio_max; i++) {
		if (sigtimedwait(&set, &info, &timeout) == -1) {
			err = errno;
			in_rpt("sigtimedwait failed on signal #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
		else {
			/*save these as we may be getting the requests back
			out of order*/
			if (info.si_value.sival_int < aio_max)
				got_requests[info.si_value.sival_int] = info.si_value.sival_int;
			else {
				in_rpt("si_value for signal %d = %d, maximum value in requests is %d", i, info.si_value.sival_int, aio_max);
				errors++;
			}
			if (info.si_signo != SIGRTMIN) {
				in_rpt("Expected si_signo for signal %d = %d, is %d", i, SIGRTMIN, info.si_signo);
				errors++;
			}
			if (info.si_code != SI_ASYNCIO) {
				in_rpt("Expected si_code for signal %d = %d, is %d", i, SI_ASYNCIO, info.si_code);
				errors++;
			}
		}
	}

	for (i = 0; i < aio_max; i++) {
		if (got_requests[i] == 0xff) {
			in_rpt("No signal received for request %d", i);
			errors++;
			continue;
		}
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO  and  _POSIX_REALTIME_SIGNALS
are   defined   or   the  implementation  supports  the
aio_read() function as described in  System  Interfaces
and Headers, Issue 5:
	On   a    call    to    aio_read()    when    aiocbp-
	>aio_sigevent.sigev_notify  is SIGEV_THREAD the func-
	tion  in   aiocbp->aio_sigevent.sigev_notify_function
	shall   be   called   with   the   value  of  aiocbp-
	>aio_sigevent.sigev_value as  a  parameter  when  the
	operation completes, executed in an environment as if
	it were the start_routine for a newly created  thread
	with   thread   attributes   specified   by   aiocbp-
	>aio_sigevent.sigev_notify_attributes.
*/

private void test8()
{
	DBUG_ENTER("test8");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO  and  _POSIX_REALTIME_SIGNALS
are   defined   or   the  implementation  supports  the
aio_read() function as described in  System  Interfaces
and Headers, Issue 5:
	On   a    call    to    aio_read()    when    aiocbp-
	>aio_sigevent.sigev_notify    is   SIGEV_THREAD   and
	aiocbp->aio_sigevent.sigev_notify_attributes is NULL,
	the           function           in           aiocbp-
	>aio_sigevent.sigev_notify_function shall  be  called
	with the value of aiocbp->aio_sigevent.sigev_value as
	a parameter when the operation completes, executed in
	an  environment as if it were the start_routine for a
	newly created thread with the  detachstate  attribute
	set to PTHREAD_CREATE_DETACHED.
*/

private void test9()
{
	DBUG_ENTER("test9");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion  supports  the aio_read() function as described in
System Interfaces and Headers, Issue 5:
	The aiocbp argument to aio_read() shall be usable  as
	an  argument to aio_error() and aio_return() in order
	to determine the error  and  return  status,  respec-
	tively, of the operation while it is proceeding.
*/

private void test10()
{
	int	terrors;

	DBUG_ENTER("test10");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 50)
		aio_max = 50;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t10", "using %d requests", aio_max);

	vsrt_aio_test_all(dotest10);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest10(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	int	errors;
	ssize_t	retval;
	int	retval2;
	int	origflags;
	int	aio_max1;

	DBUG_ENTER("dotest10");

	got_sig = 0;
	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

 	DBUG_PRINTF("t10", "write some data");
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		DBUG_RETURN(-1);
	}
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			if (errno == EAGAIN)
				break;
			else {
				vsrt_sysfail("write");
				DBUG_RETURN(-1);
			}
		}
	}

	aio_max1 = i;

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max1*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max1*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max1*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t10", "kick off some aio reads");
	for (i = 0; i < aio_max1; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*strlen(tbuf)];
		paiocbs[i].aio_offset = i*strlen(tbuf);
		paiocbs[i].aio_nbytes = strlen(tbuf);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (tb_aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t10", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max1; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max1)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max1; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("Request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t10", "check the results");
	for (i = aio_max1-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion  supports  the aio_read() function as described in
System Interfaces and Headers, Issue 5:
	When aiocbp->aio_fildes is not a valid file  descrip-
	tor open for reading, either:
	the aio_read() function shall  return  -1  and  set
	errno to EBADF
		or
	the return status  of  the  asynchronous  operation
	shall  be  set  to -1 and its error status shall be
	set to EBADF.
*/

private void test11()
{
	int	fd;
	struct aiocb	myaiocb;
	char	buf[32];
	pid_t	pid;
	int	status, errors, retval, j, err;
	ssize_t	retval2;

	DBUG_ENTER("test11");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	errors = 0;

	VSRT_FORK(pid);
		if (vsrt_signal(SIGUSR1, handler5) == -1)
			exit(1);

		(void)unlink("vsrt_aio");
		if ((fd = open("vsrt_aio", O_RDWR|O_CREAT, VSRT_PROT_ALL)) == -1) {
			vsrt_sysfail("open");
			exit(1);
		}

		if (close(fd) == -1) {
			vsrt_sysfail("close");
			(void)unlink("vsrt_aio");
			exit(1);
		}
	
		(void)unlink("vsrt_aio");
		
		myaiocb.aio_fildes = fd+500;
		myaiocb.aio_buf = buf;
		myaiocb.aio_offset = 0;
		myaiocb.aio_nbytes = 2;
		myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
		myaiocb.aio_sigevent.sigev_signo = SIGUSR1;
		myaiocb.aio_sigevent.sigev_value.sival_int = 0;
		myaiocb.aio_reqprio= 0;

		if (tb_aio_read(&myaiocb) == -1) {
			if (errno != EBADF) {
				vsrt_sysfail("aio_read");
				xx_rpt(FAILURE);
				exit(1);
			}
		} else {
			/*wait for it to complete*/
			for (j = 0; j < 5; j++) {
				if (aio_error(&myaiocb) == EINPROGRESS)
					sleep(2);
				else
					break;
			}
		
			if (j == 5) {
				in_rpt("request did not fail immediately and has not completed");
				xx_rpt(FAILURE);
				exit(1);
			}
			retval = aio_error(&myaiocb);
			if (retval == -1) {
				vsrt_sysfail("aio_error");
				exit(1);
			}
			if (retval != EBADF) {
				in_rpt("request did not fail immediately and aio_error returned %d (%s), expected EBADF", retval, errname(retval));
				xx_rpt(FAILURE);
				errors++;
			}
			retval2 = aio_return(&myaiocb);
			if (retval2 != -1) {
				in_rpt("request did not fail immediately and aio_return returned %d, expected -1", retval2);
				xx_rpt(FAILURE);
				errors++;
			}
			exit(errors);
		}
	
	VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	if (errors == 0)
		xs_rpt();
	
	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion  supports  the aio_read() function as described in
System Interfaces and Headers, Issue 5:
	When the offset implied by  aiocbp->aio_offset  would
	be invalid, either:
	the aio_read() function shall  return  -1  and  set
	errno to EINVAL
		or
	the return status  of  the  asynchronous  operation
	shall  be  set  to -1 and its error status shall be
	set to EINVAL.
*/

private void test12()
{
	DBUG_ENTER("test12");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	vsrt_aio_test_file(dotest12);

	if (vsrt_aio_test_fail != 0) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest12(readfd, writefd)
int	readfd;
int	writefd;
{
	int	pathok = 0;
	char	buf[2];
	struct aiocb	myaiocb;
	int	errors;
	int	i;
	int	retval;
	ssize_t	retval2;

	DBUG_ENTER("dotest12");

	errors = 0;

	myaiocb.aio_fildes = readfd;
	myaiocb.aio_buf = buf;
	/*illegal offset*/
	myaiocb.aio_offset = -1;
	myaiocb.aio_nbytes = 1;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGUSR1;
	myaiocb.aio_sigevent.sigev_value.sival_int = 0;
	myaiocb.aio_reqprio= 0;
	PATH_TRACE;

	/*see if it fails straight away*/
	if (tb_aio_read(&myaiocb) == -1) {
		if (errno == EINVAL)
			DBUG_RETURN(0);
		vsrt_sysfail("aio_read");
		DBUG_RETURN(-1);
	}
	PATH_TRACE;
	/*otherwise, wait for it to complete*/
	for (i = 0; i < 5; i++) {
		if (aio_error(&myaiocb) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (i == 5) {
		in_rpt("Request did not fail immediately and has not completed");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	PATH_TRACE;

	retval = aio_error(&myaiocb);
	if (retval == -1) {
		vsrt_sysfail("aio_error");
		DBUG_RETURN(-1);
	}

	if (retval != EINVAL) {
		in_rpt("For offset -1, request did not fail immediately and aio_error returned %d (%s), expected EINVAL", retval, errname(retval));
		xx_rpt(FAILURE);
		errors++;
	}

	retval2 = aio_return(&myaiocb);
	if (retval2 != -1) {
		in_rpt("For offset -1, request did not fail immediately and aio_return returned %d, expected -1", retval2);
		xx_rpt(FAILURE);
		errors++;
	}

	PATH_TRACE;
		
	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}


/*
If _POSIX_ASYNCHRONOUS_IO is
defined  or the implementation supports the aio_read()
function as described in System Interfaces and Headers,
Issue 5:
	When  aiocbp->aio_reqprio  is  not  a  valid   value,
	either:
	the aio_read() function shall  return  -1  and  set
	errno to EINVAL
		or
	the return status  of  the  asynchronous  operation
	shall  be  set  to -1 and its error status shall be
	set to EINVAL.
*/

private void test13()
{
	DBUG_ENTER("test13");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	vsrt_aio_test_all(dotest13);

	if (vsrt_aio_test_fail != 0) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest13(readfd, writefd)
int	readfd;
int	writefd;
{
	int	pathok = 0;
	char	buf[2];
	struct aiocb	myaiocb;
	int	errors;
	int	i;
	int	retval;
	ssize_t	retval2;
	long	dmax;

	DBUG_ENTER("dotest13");

	errors = 0;

	myaiocb.aio_fildes = readfd;
	myaiocb.aio_buf = buf;
	myaiocb.aio_offset = 0;
	myaiocb.aio_nbytes = 2;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGUSR1;
	myaiocb.aio_sigevent.sigev_value.sival_int = 0;
	/*illegal priority*/
	myaiocb.aio_reqprio= -1;

	/*see if it fails straight away*/
	if (tb_aio_read(&myaiocb) == -1) {
		if (errno == EINVAL)
			DBUG_RETURN(0);
		vsrt_sysfail("aio_read");
		DBUG_RETURN(-1);
	}
	PATH_TRACE;
	
	/*otherwise, wait for it to complete*/
	for (i = 0; i < 5; i++) {
		if (aio_error(&myaiocb) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (i == 5) {
		in_rpt("Request did not fail immediately and has not completed");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	retval = aio_error(&myaiocb);
	if (retval == -1) {
		vsrt_sysfail("aio_error");
		errors++;
	} else

		if (retval != EINVAL) {
			in_rpt("For priority -1, request did not fail immediately and aio_error returned %d (%s), expected EINVAL", retval, errname(retval));
			xx_rpt(FAILURE);
			errors++;
		}

	retval2 = aio_return(&myaiocb);
	if (retval2 != -1) {
		in_rpt("For priority -1, request did not fail immediately and aio_return returned %d, expected -1", retval);
		xx_rpt(FAILURE);
		errors++;
	}

	PATH_TRACE;

	if ((dmax = sysconf(_SC_AIO_PRIO_DELTA_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_PRIO_DELTA_MAX)");
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	/*can't test over limit as it would overflow aio_reqprio*/
	if (dmax == INT_MAX) {
		if (errors == 0)
			DBUG_RETURN(0);
		else
			DBUG_RETURN(-1);
	}

	PATH_TRACE;

	myaiocb.aio_fildes = readfd;
	myaiocb.aio_buf = buf;
	myaiocb.aio_offset = 0;
	myaiocb.aio_nbytes = 2;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGUSR1;
	myaiocb.aio_sigevent.sigev_value.sival_int = 0;
	/*illegal priority*/
	myaiocb.aio_reqprio= dmax+1;

	/*see if it fails straight away*/
	if (tb_aio_read(&myaiocb) == -1) {
		if (errno == EINVAL)
			DBUG_RETURN(0);
		vsrt_sysfail("aio_read");
		DBUG_RETURN(-1);
	}
	PATH_TRACE;
	
	/*otherwise, wait for it to complete*/
	for (i = 0; i < 5; i++) {
		if (aio_error(&myaiocb) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (i == 5) {
		in_rpt("Request did not fail immediately and has not completed");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	retval = aio_error(&myaiocb);
	if (retval == -1) {
		vsrt_sysfail("aio_error");
		errors++;
	} else

		if (retval != EINVAL) {
			in_rpt("For priority > AIO_PRIO_DELTA_MAX, request did not fail immediately and aio_error returned %d (%s), expected EINVAL", retval, errname(retval));
			xx_rpt(FAILURE);
			errors++;
		}

	retval2 = aio_return(&myaiocb);
	if (retval2 != -1) {
		in_rpt("For priority > AIO_PRIO_DELTA_MAX, request did not fail immediately and aio_return returned %d, expected -1", retval);
		xx_rpt(FAILURE);
		errors++;
	}

	PATH_TRACE;
		
	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

size_t	aio_nbytes_invalid;

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion  supports  the aio_read() function as described in
System Interfaces and Headers, Issue 5:
	When aiocbp->aio_nbytes is not a valid value, either:
	the aio_read() function shall  return  -1  and  set
	errno to EINVAL
		or
	the return status  of  the  asynchronous  operation
	shall  be  set  to -1 and its error status shall be
	set to EINVAL.
*/

private void test14()
{

	char	*str;

	DBUG_ENTER("test14");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	/*user needs to supply an invalid value, if there is one*/
	str = tet_getvar("VSRT_INVALID_AIO_NBYTES_READ");
	if ((str != NULL) && (strlen(str) != 0))
		aio_nbytes_invalid = (size_t)atol(str);
	else {
		in_rpt("No VSRT_INVALID_AIO_NBYTES_READ value defined in tetexec.cfg");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	vsrt_aio_test_all(dotest14);

	if (vsrt_aio_test_fail != 0) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest14(readfd, writefd)
int	readfd;
int	writefd;
{
	int	pathok = 0;
	char	buf[2];
	struct aiocb	myaiocb;
	int	errors;
	int	i;
	int	retval;
	ssize_t	retval2;

	DBUG_ENTER("dotest14");

	errors = 0;

	myaiocb.aio_fildes = readfd;
	myaiocb.aio_buf = buf;
	myaiocb.aio_offset = 0;
	/*illegal size ?*/
	/*not clear what would be invalid here so we try this and
	yield unsupported if it does not fail*/
	myaiocb.aio_nbytes = aio_nbytes_invalid;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGUSR1;
	myaiocb.aio_sigevent.sigev_value.sival_int = 0;
	myaiocb.aio_reqprio= 0;

	/*see if it fails straight away*/
	if (tb_aio_read(&myaiocb) == -1) {
		if (errno == EINVAL) {
			DBUG_RETURN(0);
		}
		vsrt_sysfail("aio_read");
		DBUG_RETURN(-1);
	}
	PATH_TRACE;
	
	/*otherwise, wait for it to complete*/
	for (i = 0; i < 5; i++) {
		if (aio_error(&myaiocb) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (i == 5) {
		in_rpt("Request did not fail immediately and has not completed");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	retval = aio_error(&myaiocb);
	if (retval == -1) {
		vsrt_sysfail("aio_error");
		errors++;
	} else

		if (retval != EINVAL) {
			in_rpt("For nbytes %ld, request did not fail immediately and aio_error returned %d (%s), expected EINVAL", (long)aio_nbytes_invalid, retval, errname(retval));
			xx_rpt(FAILURE);
			errors++;
		}

	retval2 = aio_return(&myaiocb);
	if (retval2 != -1) {
		in_rpt("For nbytes %ld, request did not fail immediately and aio_return returned %d, expected -1", (long)aio_nbytes_invalid, retval);
		xx_rpt(FAILURE);
		errors++;
	}

	PATH_TRACE;

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is not defined and the imple-
mentation does not support the aio_read() function as
described in System Interfaces and Headers, Issue 5:
	A call to aio_read() shall return -1 and set errno
	to ENOSYS.
*/

private void test15()
{
#ifndef _POSIX_ASYNCHRONOUS_IO
	struct aiocb	myaiocb;
#endif

	DBUG_ENTER("test15");

#ifdef _POSIX_ASYNCHRONOUS_IO
	in_rpt("_POSIX_ASYNCHRONOUS_IO defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN
#else
	if (supports_aio_read) {
		in_rpt("aio_read supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	if (tb_aio_read(&myaiocb) != -1) {
		in_rpt("aio_read succeeded");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (errno != ENOSYS) {
		vsrt_sysfail("aio_read");
		DBUG_VOID_RETURN
	}

	xs_rpt();

	DBUG_VOID_RETURN
#endif
}

private	int tb_aio_read(aiocbp)
struct aiocb	*aiocbp;
{
	int 	retval;

	retval = aio_read(aiocbp);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
