/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/aio/aio_suspend/aio_suspend.c
*
* Modifications:
* $Log: aio_suspend.c,v $
* Revision 1.2  2001/10/02 13:25:13  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:13  cyeoh
* Adds files for lsb-os test suite
*
*
*/


#include	<vsrt/vsrt.h>
#include	<aio.h>
#include	<limits.h>
#include	<setprv.h>

#define 	NO_TESTS	6	/* number of tests in testset */

#define		test_file	"vsrt_aio_suspend"

#ifdef UNDEF_MACROS
#undef aio_suspend
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();

int	no_tests = NO_TESTS;

private	int	tb_aio_suspend();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();

private	int	dotest1();
private	int	dotest2();
private	int	dotest3();
private	int	dotest4();
private	int	dotest5();

int	supports_aio_suspend;
int	supports_aio_read;
int	supports_aio_error;
int	supports_aio_return;

/*determine if a type of file is to be tried. set in startup*/
/*pipe support is determined dynamically*/
int	vsrt_aio_test_with_tty = 0;
int	vsrt_aio_test_with_file = 0;
int	vsrt_aio_test_with_pty = 0;

/*each piece of a test increments on of these.  must be zeroed at test start*/
int	vsrt_aio_test_pass;
int	vsrt_aio_test_unsup;
int	vsrt_aio_test_fail;

/*instances of things to test on from tetexec, loaded in startup*/
char	vsrt_aio_tty1[128];
char	vsrt_aio_tty2[128];
char	vsrt_aio_ptym[128];
char	vsrt_aio_testfile[4096];

void atstart(void);
void atstart(void)
{
	char *str;

	vsrt_check_support("AIO_READ", &supports_aio_read);
	vsrt_check_support("AIO_ERROR", &supports_aio_error);
	vsrt_check_support("AIO_RETURN", &supports_aio_return);
	vsrt_check_support("AIO_SUSPEND", &supports_aio_suspend);

	/*get config info and determine what sorts of files to test with*/
	str = tet_getvar("VSRT_FILE_ASYNC_IO");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_file = 1;
		strcpy(vsrt_aio_testfile, str);
	}

	str = tet_getvar("VSRT_TERMIOS_TTY");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_tty = 1;
		strcpy(vsrt_aio_tty1, str);
		str = tet_getvar("VSRT_TERMIOS_LOOP");
		if ((str != NULL) && (strlen(str) != 0))
			strcpy(vsrt_aio_tty2, str);
		else
			vsrt_aio_test_with_tty = 0;
	}

	str = tet_getvar("VSRT_MASTER_PTY");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_pty = 1;
		strcpy(vsrt_aio_ptym, str);
	}
}

void atend(void);
void atend(void)
{
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = atend;


public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	NULL, 0
};

int	aio_max = 0;
long	aio_maxl = 0;

int	got_sig = 0;

private	void	handler5();
private	void	handler5(signo)
int	signo;
{
	got_sig++;
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion  supports  the aio_suspend() function as described
in System Interfaces and Headers, Issue 5:
	A call to aio_suspend() when timeout  is  NULL  shall
	suspend  the calling thread until at least one of the
	nent asynchronous I/O operations referenced  by  list
	has  completed, unless it is interrupted by a signal,
	and then return 0.
*/

private void test1()
{
	int	terrors;

	DBUG_ENTER("test1");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_suspend) {
		in_rpt("aio_suspend not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 100)
		aio_max = 100;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t1", "using %d requests", aio_max);
	

	vsrt_aio_test_allbutfile(dotest1);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest1(readfd, writefd)
int	readfd;
int	writefd;
{

/*this test needs to be expanded to test in a multi-threaded process*/
	int	i;
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct aiocb	*plist[100];
	char	*bufs;
	int	err;
	int	retval2;
	int	errors;
	pid_t	pid2;
	int	status;
	time_t	start_time, end_time;

	DBUG_ENTER("dotest1");

	errors = 0;	
	got_sig = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((bufs = malloc((aio_max*2)+1)) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1", "ensure they are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	/*this should wake us up in a bit*/
	VSRT_FORK(pid2);
		sleep(5);
		DBUG_PRINTF("t1", "awaken the parent");
		if (write(writefd, "1\n", 2) == -1) {
			vsrt_sysfail("write");
			exit(1);
		}
	VSRT_EFORK;
		
	PATH_TRACE;
	DBUG_PRINTF("t1", "suspend until awoken");
	VSRT_SET_ALARM_WITH_ERROR(10)
	(void)time(&start_time);
	if (tb_aio_suspend(plist, aio_max, NULL) != 0) {
		vsrt_sysfail("aio_suspend");
		(void)aio_cancel(readfd, NULL);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	(void)time(&end_time);

	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t1", "check the child's status");
	status = vsrt_wait_for(pid2, 10);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1", "check we were suspended");
	if ((end_time - start_time) < 3) {
		in_rpt("Expected aio_suspend to suspend for at least 3 seconds, but got %d", end_time - start_time);

		(void)aio_cancel(readfd, NULL);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1", "ensure some really completed");
	for (i = 0; i < aio_max; i++) {
		if (aio_error(&paiocbs[i]) != EINPROGRESS)
			break;
	}

	if (i == aio_max) {
		in_rpt("all requests are still in progress after aio_suspend");
		errors++;
	}

	(void)aio_cancel(readfd, NULL);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion  supports  the aio_suspend() function as described
in System Interfaces and Headers, Issue 5:
	A call to aio_suspend()  when  timeout  is  not  NULL
	shall  return  -1 and set errno to EAGAIN if the time
	interval specified in timeout passes before  any  I/O
	operations  referenced by list complete and it is not
	interrupted by a signal during this interval.
*/

private void test2()
{
	int	terrors;

	DBUG_ENTER("test2");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_suspend) {
		in_rpt("aio_suspend not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 100)
		aio_max = 100;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t2", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest2);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest2(readfd, writefd)
int	readfd;
int	writefd;
{

	int	i;
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct aiocb	*plist[100];
	char	*bufs;
	int	errors;
	time_t	start_time, end_time;
	struct	timespec	timeout;
	int	err, retval2;

	DBUG_ENTER("dotest2");

	errors = 0;	
	got_sig = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((bufs = malloc((aio_max*2)+1)) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2", "ensure they are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t2", "suspend until awoken by the timeout");
	timeout.tv_sec = 5;
	timeout.tv_nsec = 0;
	VSRT_SET_ALARM_WITH_ERROR(10)
	(void)time(&start_time);
	if (tb_aio_suspend(plist, aio_max, &timeout) == 0) {
		in_rpt("aio_suspend succeeded");
		(void)aio_cancel(readfd, NULL);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	(void)time(&end_time);

	if (errno != EAGAIN) {
		vsrt_sysfail("aio_suspend");
		(void)aio_cancel(readfd, NULL);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t2", "check we were suspended");
	if ((end_time - start_time) < 3) {
		in_rpt("Expected aio_suspend to suspend for at least 3 seconds, but got %d", end_time - start_time);
		(void)aio_cancel(readfd, NULL);
		errors++;
	}

	(void)aio_cancel(readfd, NULL);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion  supports  the aio_suspend() function as described
in System Interfaces and Headers, Issue 5:
	A call to aio_suspend() shall ignore NULL entries in list.
*/

private void test3()
{
	int	terrors;

	DBUG_ENTER("test3");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_suspend) {
		in_rpt("aio_suspend not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 2)
		aio_max = 2;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t2", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest3);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest3(readfd, writefd)
int	readfd;
int	writefd;
{

/*this test needs to be expanded to test in a multi-threaded process*/
	int	i;
	int	pathok = 0;
	struct	aiocb	paiocbs[2];
	struct aiocb	*plist[4];
	char	bufs[8];
	int	err;
	int	retval2;
	int	errors;
	pid_t	pid2;
	int	status;
	time_t	start_time, end_time;

	DBUG_ENTER("dotest3");

	errors = 0;	
	got_sig = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;

	DBUG_PRINTF("t3", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	plist[0] = NULL;
	plist[1] = &paiocbs[1];
	plist[2] = NULL;
	plist[3] = &paiocbs[2];

	PATH_TRACE;
	DBUG_PRINTF("t3", "ensure they are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	/*this should wake us up in a bit*/
	VSRT_FORK(pid2);
		sleep(5);
		DBUG_PRINTF("t3", "awaken the parent");
		if (write(writefd, "3\n4\n", 4) == -1) {
			vsrt_sysfail("write");
			exit(1);
		}
	VSRT_EFORK;
	PATH_TRACE;
		
	DBUG_PRINTF("t3", "suspend until awoken");
	VSRT_SET_ALARM_WITH_ERROR(10)
	(void)time(&start_time);
	if (tb_aio_suspend(plist, aio_max, NULL) != 0) {
		vsrt_sysfail("aio_suspend");
		(void)aio_cancel(readfd, NULL);
		DBUG_RETURN(-1);
	}
	(void)time(&end_time);

	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t3", "check the child's status");
	status = vsrt_wait_for(pid2, 10);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t3", "check we were suspended");
	if ((end_time - start_time) < 3) {
		in_rpt("Expected aio_suspend to suspend for at least 3 seconds, but got %d", end_time - start_time);

		(void)aio_cancel(readfd, NULL);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t3", "ensure a real one completed");
	for (i = 0; i < aio_max; i++) {
		if (aio_error(&paiocbs[i]) != EINPROGRESS)
			break;
	}

	if (i == aio_max) {
		in_rpt("all requests are still in progress after aio_suspend");
		errors++;
	}

	(void)aio_cancel(readfd, NULL);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion  supports  the aio_suspend() function as described
in System Interfaces and Headers, Issue 5:
	A call to aio_suspend() when any of  the  I/O  opera-
	tions  reference by list have already completed shall
	not suspend the calling thread.
*/

private void test4()
{
	int	terrors;

	DBUG_ENTER("test4");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_suspend) {
		in_rpt("aio_suspend not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 50)
		aio_max = 50;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t4", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest4);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest4(readfd, writefd)
int	readfd;
int	writefd;
{

	int	i;
	int	pathok = 0;
	char	*bufs;
	int	err;
	int	retval2;
	int	errors;
	time_t	start_time, end_time;
	int	nundone;
	struct	aiocb	*paiocbs;
	struct aiocb	*plist[100];

	DBUG_ENTER("dotest4");

	errors = 0;	
	got_sig = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((bufs = malloc((aio_max*3)+1)) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	memset(bufs, 0, (aio_max*3)+1);

	DBUG_PRINTF("t4", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*3];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		plist[i] = &paiocbs[i];
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t4", "complete one of them");
	if (write(writefd, "5\n", 2) == -1) {
		vsrt_sysfail("write");
		(void)aio_cancel(readfd, NULL);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(3);
	PATH_TRACE;
	DBUG_PRINTF("t4", "ensure one is complete");
	nundone = 0;
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) == EINPROGRESS)
			nundone++;
	}
	if (nundone != aio_max-1) {
		in_rpt("Expected %d requests in progress, found %d", aio_max-1, nundone);
		xx_rpt(FAILURE);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t4", "aio_suspend");
	/*should not suspend*/
	VSRT_SET_ALARM_WITH_ERROR(10)
	(void)time(&start_time);
	if (tb_aio_suspend(plist, aio_max, NULL) != 0) {
		vsrt_sysfail("aio_suspend");
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	(void)time(&end_time);

	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t4", "check we were not suspended");
	if ((end_time - start_time) > 1) {
		in_rpt("Expected aio_suspend to not suspend, but got suspended for %d seconds", end_time - start_time);

		xx_rpt(FAILURE);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	free(paiocbs);
	free(bufs);

	DBUG_RETURN(0);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion  supports  the aio_suspend() function as described
in System Interfaces and Headers, Issue 5:
	A call to aio_suspend() shall return -1 and set errno
	to EINTR if a signal interrupted the function.
*/

private void test5()
{
	int	terrors;

	DBUG_ENTER("test5");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_suspend) {
		in_rpt("aio_suspend not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_maxl = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_maxl = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_maxl > 2)
		aio_max = 2;
	else
		aio_max = (int)aio_maxl;
	DBUG_PRINTF("t5", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest5);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest5(readfd, writefd)
int	readfd;
int	writefd;
{

	int	i;
	int	pathok = 0;
	struct	aiocb	paiocbs[2];
	struct aiocb	*plist[2];
	char	*bufs;
	int	err;
	int	retval2;
	int	errors;
	time_t	start_time, end_time;

	DBUG_ENTER("dotest5");

	errors = 0;	
	got_sig = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;

	DBUG_PRINTF("t5", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		if (aio_read(&paiocbs[i]) == -1) {
			err = errno;
			in_rpt("aio_read failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		DBUG_RETURN(-1);
	}

	plist[0] = &paiocbs[1];
	plist[1] = &paiocbs[0];

	sleep(2);
	PATH_TRACE;
	DBUG_PRINTF("t5", "ensure they are not complete (as no data is available yet)");
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t5", "suspend and interrupt with alarm");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&start_time);
	if (tb_aio_suspend(plist, aio_max, NULL) == 0) {
		in_rpt("aio_suspend succeeded");
		(void)aio_cancel(readfd, NULL);
		DBUG_RETURN(-1);
	}

	if (errno != EINTR) {
		vsrt_sysfail("aio_suspend");
		(void)aio_cancel(readfd, NULL);
		DBUG_RETURN(-1);
	}

	(void)time(&end_time);

	VSRT_EXPECT_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t5", "check we were suspended");
	if ((end_time - start_time) < 3) {
		in_rpt("Expected aio_suspend to suspend for at least 3 seconds, but got %d", end_time - start_time);
		(void)aio_cancel(readfd, NULL);
		DBUG_RETURN(-1);
	}

	(void)aio_cancel(readfd, NULL);
	DBUG_RETURN(0);
}

/*
If _POSIX_ASYNCHRONOUS_IO is not defined and the imple-
mentation does not support the aio_suspend() function as
described in System Interfaces and Headers, Issue 5:
	A call to aio_suspend() shall return -1 and set errno
	to ENOSYS.
*/

private void test6()
{
#ifndef _POSIX_ASYNCHRONOUS_IO
	struct aiocb	myaiocb;
#endif

	DBUG_ENTER("test6");

#ifdef _POSIX_ASYNCHRONOUS_IO
	in_rpt("_POSIX_ASYNCHRONOUS_IO defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN
#else
	if (supports_aio_suspend) {
		in_rpt("aio_suspend supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	if (tb_aio_suspend(NULL, 0, NULL) != -1) {
		in_rpt("aio_suspend succeeded");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (errno != ENOSYS) {
		vsrt_sysfail("aio_suspend");
		DBUG_VOID_RETURN
	}

	xs_rpt();

	DBUG_VOID_RETURN
#endif
}

private	int tb_aio_suspend(list, nent, timeout)
struct aiocb * list[];
int	nent;
struct timespec *timeout;
{
	int 	retval;

	retval = aio_suspend((const struct aiocb **)list, nent, timeout);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
