/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/aio/aio_write_X/aio_write_X.c
*
* Modifications:
* $Log: aio_write_X.c,v $
* Revision 1.2  2001/10/02 13:25:13  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:13  cyeoh
* Adds files for lsb-os test suite
*
*
*/

#include	<vsrt/vsrt.h>
#include	<aio.h>
#include	<setprv.h>

#define 	NO_TESTS	3	/* number of tests in testset */

#define		test_file	"vsrt_aio_write"

#ifdef UNDEF_MACROS
#undef aio_write
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();

int	no_tests = NO_TESTS;

int	errors;

private	int	tb_aio_write();
private void	test1();
private void	test2();
private void	test3();

int	supports_aio_write;
int	supports_aio_read;
int	supports_aio_error;
int	supports_aio_return;

void atstart(void);
void atstart(void)
{
	vsrt_check_support("AIO_READ", &supports_aio_read);
	vsrt_check_support("AIO_WRITE", &supports_aio_write);
	vsrt_check_support("AIO_ERROR", &supports_aio_error);
	vsrt_check_support("AIO_RETURN", &supports_aio_return);
}

void atend(void);
void atend(void)
{
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = atend;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

/*
If  the  implementation  allows  creation  of  a   file
description  with an offset maximum less than the larg-
est size it supports for an off_t:
	For regular files no data transfer will  occur  as  a
	result  of a call to aio_write() past the offset max-
	imum established in the open file description  asoci-
	ated with aiocbp->aio_fildes.
*/

private void test1()
{
	DBUG_ENTER("test1");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	errors = 0;
	
	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

/*
If  the  implementation  allows  creation  of  a   file
description  with an offset maximum less than the larg-
est size it supports for an off_t:
	When the file associated with aiocbp->aio_fildes is a
	regular  file,  aiocbp->aio_nbytes is greater than 0,
	and aiocbp->aio_offset is at the offset maximum esta-
	blished  in  the open file description asociated with
	aiocbp->aio_fildes, either:
	the aio_write() function shall return  -1  and  set
	errno to EFBIG
		or
	the return status  of  the  asynchronous  operation
	shall  be  set  to -1 and its error status shall be
	set to EFBIG.
*/

private void test2()
{
	DBUG_ENTER("test2");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	errors = 0;
	
	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

/*
If  the  implementation  allows  creation  of  a   file
description  with an offset maximum less than the larg-
est size it supports for an off_t:
	When the file associated with aiocbp->aio_fildes is a
	regular  file,  aiocbp->aio_nbytes is greater than 0,
	and aiocbp->aio_offset is beyond the  offset  maximum
	established  in  the  open file description asociated
	with aiocbp->aio_fildes, either:
	the aio_write() function shall return  -1  and  set
	errno to EFBIG
		or
	the return status  of  the  asynchronous  operation
	shall  be  set  to -1 and its error status shall be
	set to EFBIG.
*/

private void test3()
{
	DBUG_ENTER("test3");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_write) {
		in_rpt("aio_write not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_read) {
		in_rpt("aio_read not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	errors = 0;
	
	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

private	int tb_aio_write(aiocbp)
struct aiocb	*aiocbp;
{
	int 	retval;

	retval = aio_write(aiocbp);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
