/* dv @(#)V9TS/tset/LSB.os/devclass/grantpt/grantpt.c	1.4 93/05/24 */
/*
*	SCCS:  @(#) LSB.os/devclass/grantpt/grantpt.c	Ver. 1.8	Rel. 1.3	(92/08/01)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#) LSB.os/devclass/grantpt/grantpt.c	Ver. 1.8	Rel. 1.3	(92/08/01)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.grantpt

PROJECT:	LSB (Generice ABI Validation Suite)

DESCRIPTION:
	Testset for the grantpt() library call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		William P. Malloy, UniSoft Corp.
DATE CREATED:	18 Oct 1990
MODIFICATIONS:	
		Neil Moses, UniSoft Ltd
		7 May 1993
		GABI 2.1 modifications

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<stdio.h>
#include	<grp.h>
/* TMP_MAX is defined twice, once in stdio.h and again in limits.h */
#ifdef TMP_MAX
#undef TMP_MAX
#endif
#include	<limits.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<pathres.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef grantpt
#endif

#define NO_TESTS	4
#define WAITTIME	(5 * SPEEDFACTOR)
#define PTY_MODE	(mode_t)(S_IRUSR|S_IWUSR|S_IWGRP)
#define PERM_MASK       (mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define GNAME       	"tty"	/* Reserved group name */

extern	char	*errname();
extern	void	valrpt();


public	int	no_tests = NO_TESTS;

private int	ABI_vers = 3;
private int	setupok = -1;
private int	testfail;
private char	*t3_file = "./grantpt-t3";

char	*ptsname();  		/* this should be in a system include file */
int	unlockpt();
int	grantpt();

private void	prepare_tests();

private void	test1();
private void	test2();
private void	test3();
private void	test4();

private long	tbgrantpt();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};



private void
delete_all_tests(msg)
char *msg;
{
	int i;

	for (i = 0; i <= NO_TESTS; i++) {
		(void) tet_delete(i, msg);
	}
}

private void
prepare_tests()
{
	int 	fail, err;
	struct 	stat buf;
	int 	pathok = 0;
	char	*abi_vers;
	static 	char delete_msg[128];

	DBUG_ENTER("prepare_tests");

	fail = stat("/dev/ptmx", &buf);
	if (fail == SYSERROR)
	{
		err = errno;
		(void) sprintf(delete_msg,
			"stat(\"/dev/ptmx\", &buf) failed; - errno %d (%s)",
							err, errname(err));
		delete_all_tests(delete_msg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (buf.st_mode & S_IFCHR)
	{
		PATH_TRACE;
	}
	else
	{
		(void) sprintf(delete_msg,
			"/dev/ptmx mode 0x%x should be character device",
							(int) buf.st_mode);
		delete_all_tests(delete_msg);
		DBUG_VOID_RETURN;
	}

	if (pathok == 2)
		setupok = 0;
	DBUG_VOID_RETURN;
}


/* things to remember (i.e. MAGIC) you open the master side of the PTY first
 * which is a clone device.  It is set up on system configuration (via the
 * /stand/system call as to how many open()'s it allows.  So if there are 32
 * PTY's on the system (a standard number) 32 clone opens will be allowed
 * of the master device.  The open() call is then supposed to return a file
 * descriptor who's `rdev' stat entrys minor number is the actual number of
 * which pseudo tty is open.  So for instance the file descriptor for the
 * open may well be 3 (as 0,1, and 2 as already in use) but be FOR the zero-th
 * pseudo terminal device "/dev/pts/0" which you have to find out by doing a
 * fstat(3, &buf) and then "minor(buf.st_rdev) to find out.
 *
 * The slave devices are "/dev/pts/#" which in turn are linked to /dev/pts#
 * these have to opened and be ioctl(PUSH)-ed seperately.
 */


private void
test1()
{
	int	p, fail, err;
	char	*pts;
	struct stat buf;
	uid_t	euid;
	int pathok = setupok;

	DBUG_ENTER("test1");

	testfail = 0;
	p = open("/dev/ptmx", O_RDWR);
	if (p == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(\"/dev/ptmx\", O_RDWR) failed");
		in_rpt("errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tbgrantpt(p, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pts = ptsname(p);
	if (pts == NULL)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("ptsname(%d) failed", p);
		in_rpt("errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	fail = stat(pts, &buf);
	if (fail == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat(\"%s\", &buf) failed", pts);
		in_rpt("errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	euid = getuid();
	err = errno;
	if (err != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("geteuid() failed;  errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (buf.st_uid != euid)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("grantpt() set wrong uid expected %ld, got %ld",
			(long)euid, (long)buf.st_uid);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((buf.st_mode & PERM_MASK) != PTY_MODE)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("incorrect mode on %s: expected 0%lo got 0%lo",
			    ptsname(p), (long)PTY_MODE, (long)buf.st_mode);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) unlockpt(p);
	(void) close(p);

	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	p, err;
	int pathok = setupok;

	DBUG_ENTER("test2");

	testfail = 0;
	p = open("/dev/ptmx", O_RDWR);
	if (p == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(\"/dev/ptmx\", O_RDWR) failed");
		in_rpt("errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) unlockpt(p);
	(void) close(p);

	if (tbgrantpt(p, SYSERROR, EBADF) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int	p, err;
	int pathok = setupok;

	DBUG_ENTER("test3");

	testfail = 0;
	(void) unlink(t3_file);
	p = open(t3_file, O_RDWR|O_CREAT);
	if (p == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed", t3_file);
		in_rpt("errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tbgrantpt(p, SYSERROR, EINVAL) != SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(p);
	(void) unlink(t3_file);

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	p, err;
	int	pathok = setupok;

	DBUG_ENTER("test4");

	in_rpt("No safe way to deny access to the slave device");
	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN;
}

private long
tbgrantpt(tbp, expret, experrno)
int	tbp;
int	expret;
int	experrno;
{
	int	rval;
	int	err;

	DBUG_ENTER("tbgrantpt");

	errno = NOERROR;
	rval = grantpt(tbp);
#ifdef TESTBED
	rval = -1;
	errno++;
#endif
	err = errno;


	if (expret == SUCCEED && rval == expret)
	{
		DBUG_RETURN(rval);
	}
	else if (expret == SYSERROR && rval == expret)
	{
		if (err == experrno) {
			DBUG_RETURN(rval);
		}
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("grantpt(%d) did not give correct results", tbp);
	valrpt(expret, rval, experrno, err);

	DBUG_RETURN(rval);
}
