/* dv @(#)V9TS/tset/LSB.os/devclass/ptsname_L/ptsname_L.c	1.1 93/04/20 */
/*
*	SCCS:  @(#) LSB.os/devclass/ptsname_L/ptsname_L.c	Ver. 1.7	Rel. 1.3	(92/08/01)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#) LSB.os/devclass/ptsname_L/ptsname_L.c	Ver. 1.7	Rel. 1.3	(92/08/01)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.ptsname_L

PROJECT:	Generic ABI test suite

DESCRIPTION:
	Testset for the ptsname() library call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		William P. Malloy, UniSoft Corp.
DATE CREATED:	22 Oct 1990

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<ctype.h>
#include	<stdio.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ptsname
#endif

#define NO_TESTS	2
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();

public	int	no_tests = NO_TESTS;

private int	setupok = -1;
private int	testfail;
extern	char	*ptsname();
extern	int	grantpt();
extern	int	unlockpt();

private void	prepare_tests();

private void	test1();
private void	test2();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private char delete_msg[128];

extern void tet_delete(int, char*);

private void
delete_all_tests(msg)
char *msg;
{
	int i;

	for (i = 0; i <= NO_TESTS; i++) {
		(void) tet_delete(i, msg);
	}
}

private void
prepare_tests()
{
	int fail;
	struct stat buf;
	int pathok = 0;

	DBUG_ENTER("prepare_tests");

	fail = stat("/dev/ptmx", &buf);
	if (fail == SYSERROR) {
		(void) sprintf(delete_msg,
			"stat(\"/dev/ptmx\", &buf) failed; - errno %d (%s)",
							errno, errname(errno));
		delete_all_tests(delete_msg);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ((buf.st_mode & S_IFCHR) == 0) {
		(void) sprintf(delete_msg,
			"/dev/ptmx mode 0x%x should be character device",
							(int) buf.st_mode);
		delete_all_tests(delete_msg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (pathok == 2)
		setupok = 0;
	DBUG_VOID_RETURN;
}

/* things to remember (i.e. MAGIC) you open the master side of the PTY first
 * which is a clone device.  It is set up on system configuration (via the
 * /stand/system call as to how many open()'s it allows.  So if there are 32
 * PTY's on the system (a standard number) 32 clone opens will be allowed
 * of the master device.  The open() call is then supposed to return a file
 * descriptor who's `rdev' stat entrys minor number is the actual number of
 * which pseudo tty is open.  So for instance the file descriptor for the
 * open may well be 3 (as 0,1, and 2 as already in use) but be FOR the zero-th
 * pseudo terminal device "/dev/pts/0" which you have to find out by doing a
 * fstat(3, &buf) and then "minor(buf.st_rdev) to find out.
 *
 * The slave devices are "/dev/pts/#" which in turn are linked to /dev/pts#
 * these have to opened and be ioctl(PUSH)-ed seperately.
 */


private void
test1()
{
	int	p, fail, min;
	char *sp, *rs;
	struct stat buf;
	int	pathok = setupok;

	DBUG_ENTER("test1");

	testfail = 0;
	p = open("/dev/ptmx", O_RDWR);
	if (p == SYSERROR) {
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(\"/dev/ptmx\", O_RDWR) failed");
		in_rpt("errno %d (%s)", errno, errname(errno));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	fail = grantpt(p);
	if (fail == SYSERROR) {
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("grantpt() failed: errno %d (%s)", errno, 
			errname(errno));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	rs = ptsname(p);
	if (rs == NULL) {
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("ptsname() failed: errno %d (%s)", errno, 
			errname(errno));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	fail = sscanf(rs, "/dev/pts/%d", &min);
	if (fail != 1) {
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sscanf(%s, \"/dev/pts/%%d\", ...) returned unexpec result", rs);
		in_rpt("expected 1, got %d: errno %d (%s)", fail, errno, 
			errname(errno));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	sp = (char *)(rs + 9);
	while (isdigit(*sp))
		sp++;
	if (*sp != '\0') {
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("return from ptsname() is not NULL terminated");
		in_rpt("character following slave number is 0x%x", (int)*sp);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* 
	 * since SVID doesn't tell us how to look at st_rdev, we can't
	 * compare the minor number of the device with either the minor
	 * number of the opened pty or N in the name.  All we can do is
	 * verify that it's a character special file.
	 */
	if (stat(rs, &buf) == SYSERROR) {
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed: errno %d (%s)", rs, errno, 
			errname(errno));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	
	if ((buf.st_mode & S_IFCHR) == 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("\"%s\" is not a character special device", rs);
	}
	else
		PATH_TRACE;

	(void)unlockpt(p);
	(void)close(p);

	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	p;
	char	*rs;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	testfail = 0;
	p = open("Test_file2", O_RDWR|O_CREAT);
	if (p == SYSERROR) {
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(\"Test_file2\", O_RDWR|O_CREAT) failed");
		in_rpt("errno %d (%s)", errno, errname(errno));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	rs = ptsname(p);
	if (rs != NULL) {
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("ptsname() succeeded when file descriptor addressed a regular file");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void)close(p);
	(void)unlink("Test_file2");

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

