/*
*      SCCS:  @(#)  XOPEN.os/files/dev_tty/dev_tty.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/files/dev_tty - T.dev_tty Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.dev_tty

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the /dev/tty device

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		J.A. Nave, UniSoft Ltd.
DATE CREATED:	Wed Oct 10 11:47:17 BST 1990
MODIFICATIONS:
		Geoff Clare, 15 March 1991
			Revised strategy in test 1.

		Geoff Clare, 12 Nov 1993
			Unsupported result if termios_prep() returns 1.

		Geoff Clare, 15 Sept 1995
			Block SIGINT and use sigsuspend() instead of
			pause() in test 1.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef dev_tty
#endif


#define NO_TESTS	1
#define WAITTIME	(5 * SPEEDFACTOR)
#define STRLEN		(sizeof(string)-1)
#define NAP		3

extern	char	*errname();

int	no_tests = NO_TESTS;

private void	test1();
private void	ch_t1();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private	char	*dev_tty = "/dev/tty";
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private	int	caught_sig = 0;
private int	testfail;
private	char	string[] = "test-string\n";
private	char	buf[256];

private void
sig_catch(sig)
{
	caught_sig = sig;
}


private void
test1()
{
	/* verify that /dev/tty accesses the controlling terminal */

	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	char	*msg = NULL;
	int	ret, err;
	int	tty_alt_fd;
	int	pathok = 0;
	struct sigaction	sig;
	sigset_t		sigset;
	unsigned char		chr;

	DBUG_ENTER("ch_t1");

	testfail = 0;

	switch (termios_prep(&tty_fildes, tty_termios_p,
		&loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE;
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY or VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY or VSX_TERMIOS_LOOP";
	}

	if (msg != NULL)
	{
		xx_rpt(DELETION);
		in_rpt("%s", msg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* send interrupt to self */

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	DBUG_PRINTF("interrupt", "c_cc[VINTR] = 0%o",
		(int)tty_termios_p->c_cc[VINTR]);

        chr = tty_termios_p->c_cc[VINTR];

        SET_TIMEOUT(WAITTIME/2)

	/* can't just write() and pause(): SIGINT may arrive before
	   the pause() on fast systems */

	(void) sigemptyset(&sigset);
	(void) sigprocmask(SIG_SETMASK, (sigset_t *) 0, &sigset);
	(void) sigaddset(&sigset, SIGINT);
	(void) sigprocmask(SIG_SETMASK, &sigset, (sigset_t *) 0);

        ret = write(loop_fildes, (char *)&chr, 1);
	err = errno;

	(void) sigdelset(&sigset, SIGINT);
	if (ret == 1 && alrm_flag == 0)
		(void) sigsuspend(&sigset);

        CLEAR_ALARM

	if (ret != 1)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (caught_sig != SIGINT)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("process did not receive SIGINT when expected");
	}
	else
		PATH_TRACE;


        SET_TIMEOUT(WAITTIME/2)

	tty_alt_fd = open(dev_tty, O_RDWR);
	err = errno;

	CLEAR_ALARM

	if(tty_alt_fd < 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed with errno %d(%s)",
			dev_tty, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* data written to /dev/tty should be available on TERMIOS_LOOP */

        SET_TIMEOUT(WAITTIME/2)

        ret = write(tty_alt_fd, string, STRLEN);
	err = errno;

        CLEAR_ALARM

	if (ret != STRLEN)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to /dev/tty failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

        SET_TIMEOUT(WAITTIME/2)

        ret = read(loop_fildes, buf, sizeof(buf));
	err = errno;

        CLEAR_ALARM

#ifdef TESTBED
	buf[3]++;
#endif

	if (ret != STRLEN)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
	}
	else if (strcmp(string, buf) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on VSX_TERMIOS_LOOP returned incorrect data");
		buf[STRLEN] = '\0';
		DBUG_PRINTF("test1", "read from loop: \"%s\"", buf);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(8); 

	(void) close(tty_alt_fd);
	(void) close(loop_fildes);
	(void) close(tty_fildes);
	DBUG_VOID_RETURN;
}
