/*
*      SCCS:  @(#)  XOPEN.os/files/seekdir/seekdir.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/files/seekdir - T.seekdir Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.seekdir

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the seekdir() subroutine.

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Thu Nov 13 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger	09 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 18 Feb 1991
			Some ino_t's were not cast to long for printing.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<dirent.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef seekdir
#endif


#define	NO_TESTS		1		/* number of tests in testset */
#define	T_NFILES	6
#define	T_NREMOVE	3
#define	T_NFIND		3
#define	T_NINDEX	3
#define	T_NENT		40
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();

int	no_tests = NO_TESTS;


private	char	*t_dir =		"./seekdir-d";
private	char	*t_files[T_NFILES] =	{
					"seekdir-t.a",
					"seekdir-t.b",
					"seekdir-t.c",
					"seekdir-t.d",
					"seekdir-t.e",
					"seekdir-t.f",
					};
private	int	t_remove[T_NREMOVE] =	{ 0, 2, 5 };
private	int	t_index[T_NINDEX] =	{ 3, 2, 1 };
					

private	void	test1();
private	void	t1_cleanup();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private void
test1()
{
	int		i;
	int		nent;
	int		err;
	int		fd;
	char		buf[50];
	struct dirent	*dp;
	DIR		*curdir;
	struct {
		long	tell;
		ino_t	ino;
		char	name[50];
	} t1_res[T_NENT] ;
	int	pathok = 0;

	DBUG_ENTER("test1");
	(void) umask(0);
	if (crdir(t_dir, (mode_t)MODEANY, (int) getuid(), (int) getgid()) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("could not creat directory %s - errno %d(%s)",
			t_dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	for (i = 0 ; i < T_NFILES ; i++)
	{
		(void) strcat(strcat(strcpy(buf,
			t_dir), "/"), t_files[i]);
		if ((fd = creat(buf, MODEANY)) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("creat(%s, %#o) failed - errno %d(%s)",
				buf, MODEANY, err, errname(err));
			t1_cleanup();
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		(void) close(fd);
	}
	for (i = 0 ; i < T_NREMOVE ; i++)
	{
		(void) strcat(strcat(strcpy(buf,
			t_dir), "/"), t_files[t_remove[i]]);
		if (unlink(buf) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("unlink(%s) failed - errno %d(%s)",
				buf, err, errname(err));
			t1_cleanup();
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	if ((curdir = opendir(t_dir)) == (DIR *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("opendir(%s) failed - errno %d(%s)",
			t_dir);;
		t1_cleanup();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	nent = -1;
	do {
		if (nent <= 0)
			PATH_TRACE;
		nent++;
		t1_res[nent].tell = telldir(curdir);
		dp = readdir(curdir);
		if (dp != (struct dirent *) NULL)
		{
			t1_res[nent].ino = dp->d_ino;
			(void) strncpy(t1_res[nent].name, dp->d_name, 50);
		}
		DBUG_PRINTF("test1", "i=%d", nent);
		DBUG_PRINTF("test1", "tell=%ld, ino=%ld, name=%s.",
			t1_res[nent].tell,
			(long)t1_res[nent].ino,
			t1_res[nent].name);
	} while (nent < T_NENT - 1 && dp != (struct dirent *) NULL);
	DBUG_PRINTF("test1", "---");
	if (nent >= T_NENT - 1)
	{
		xx_rpt(DELETION);
		in_rpt("Could not buffer all returns from readdir() (expected %d entries, got more then %d)",
			T_NFIND + 2, T_NENT);
		(void) closedir(curdir);
		t1_cleanup();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	err = 0;
	for (i = 0 ; i < T_NINDEX ; i++)
	{
		if (t_index[i] > nent)
			continue;
		DBUG_PRINTF("test1", "i=%d", t_index[i]);
		DBUG_PRINTF("test1", "tell=%ld, ino=%ld, name=%s.",
			t1_res[t_index[i]].tell,
			(long)t1_res[t_index[i]].ino,
			t1_res[t_index[i]].name);
		(void) seekdir(curdir, t1_res[t_index[i]].tell);
		if ((dp = readdir(curdir)) == (struct dirent *) NULL)
		{
			err = 1;
			break;
		}
		DBUG_PRINTF("test1", "ino=%ld, name=%s.",
			(long)dp->d_ino,
			dp->d_name);
		if (dp->d_ino != t1_res[t_index[i]].ino || strncmp(dp->d_name,
			t1_res[t_index[i]].name, 50) != 0)
		{
			err = 2;
			break;
		}
		if (i == T_NINDEX - 1)
			PATH_TRACE;
	}
	if (nent == 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not read any entries from directory");
	}
	else if (err)
	{
		xx_rpt(FAILURE);
		in_rpt("after the sequence");
		in_rpt("  tell = teldir(curdir)");
		in_rpt("  seekdir(curdir, tell)");
		if (err == 1)
			in_rpt("readdir(curdir) returned NULL instead of directory entry");
		else
		{
			in_rpt("readdir(curdir) returned { %ld, %s }",
				(long)dp->d_ino, dp->d_name);
			in_rpt("expected return value of { %ld, %s }",
				(long)t1_res[t_index[i]].ino,
				t1_res[t_index[i]].name);
		}
	}
	else
	{
		PATH_XS_RPT(5 + T_NFILES + T_NREMOVE);
	}

	t1_cleanup();
	(void) closedir(curdir);
	DBUG_VOID_RETURN;
}

private	void
t1_cleanup()
{
	int	i;
	char	buf[50];

	DBUG_ENTER("t1_cleanup");
	for (i = 0 ; i < T_NFILES ; i++)
	{
		(void) strcat(strcat(strcpy(buf,
			t_dir), "/"), t_files[i]);
		(void) unlink(buf);
	}
	(void) rmdir(t_dir);
	DBUG_VOID_RETURN;
}
