/*
*      SCCS:  @(#)  LSB.os/files/telldir_L/telldir_L.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)LSB.os/files/telldir_L - T.telldir_L Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.telldir_L

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the telldir() subroutine.

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Thu Nov 13 16:40:02 WET 1986
MODIFICATIONS:	Andrew Twigger	09 June 87
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<dirent.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef telldir
#endif


#define	NO_TESTS		1		/* number of tests in testset */
#define	T_NFILES	6
#define	T_NREMOVE	3
#define	T_NFIND		3
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();

int	no_tests = NO_TESTS;


private	char	*t_dir =		"./telldir-d";
private	char	*t_files[T_NFILES] =	{
					"telldir-t.a",
					"telldir-t.b",
					"telldir-t.c",
					"telldir-t.d",
					"telldir-t.e",
					"telldir-t.f",
					};
private	int	t_remove[T_NREMOVE] =	{ 0, 2, 5 };


private	void	test1();
private	void	t1_cleanup();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private void
test1()
{
	int		i;
	int		j;
	int		err;
	int		fd;
	int		testfail;
	char		buf[50];
	DIR		*curdir;
	struct dirent	*dp;
	long		t1_res[T_NFIND];
	long		act_res;
	int	pathok = 0;

	DBUG_ENTER("test1");
	testfail = 0;
	(void) umask(0);
	if (crdir(t_dir, (mode_t)MODEANY, (int) getuid(), (int) getgid()) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("could not creat directory %s - errno %d(%s)",
			t_dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	for (i = 0 ; i < T_NFILES ; i++)
	{
		(void) strcat(strcat(strcpy(buf,
			t_dir), "/"), t_files[i]);
		if ((fd = creat(buf, MODEANY)) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("creat(%s, %#o) failed - errno %d(%s)",
				buf, MODEANY, err, errname(err));
			t1_cleanup();
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		(void) close(fd);
	}
	for (i = 0 ; i < T_NREMOVE ; i++)
	{
		(void) strcat(strcat(strcpy(buf,
			t_dir), "/"), t_files[t_remove[i]]);
		if (unlink(buf) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("unlink(%s) failed - errno %d(%s)",
				buf, err, errname(err));
			t1_cleanup();
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	if ((curdir = opendir(t_dir)) == (DIR *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("opendir(%s) failed - errno %d(%s)",
			t_dir);;
		t1_cleanup();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	for (i = 0 ; i < T_NFIND ; i++)
	{
		if ((dp = readdir(curdir)) == (struct dirent *) NULL)
			break;
		t1_res[i] = telldir(curdir);
		for (j = 0 ; j < i ; j++)
			if (t1_res[j] == t1_res[i])
				break;
		if (j != i)
			break;
		if (i == T_NFIND - 1)
			PATH_TRACE;
	}

	if (i != T_NFIND && dp != (struct dirent *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("telldir() did not return unique values for unique directory entries");
	}
	else
	{
		for ( i = 0 ; i < T_NFIND ; i++)
		{
			seekdir(curdir, t1_res[i]);
			if ((act_res = telldir(curdir)) != t1_res[i])
			{
				testfail++;
				xx_rpt(FAILURE);
				in_rpt("telldir() did not return value set by previous seekdir()");
				in_rpt("Expected value %ld", t1_res[i]);
				in_rpt("Actual value %ld", act_res);
			}
			else
				PATH_TRACE;
		}
	}

	if (!testfail)
		PATH_XS_RPT(2 + T_NFILES + T_NREMOVE + T_NFIND);
	t1_cleanup();
	(void) closedir(curdir);
	DBUG_VOID_RETURN;
}

private	void
t1_cleanup()
{
	int	i;
	char	buf[50];

	DBUG_ENTER("t1_cleanup");
	for (i = 0 ; i < T_NFILES ; i++)
	{
		(void) strcat(strcat(strcpy(buf,
			t_dir), "/"), t_files[i]);
		(void) unlink(buf);
	}
	(void) rmdir(t_dir);
	DBUG_VOID_RETURN;
}
