/* dv (#)SOLTESTS/tset/LSB.os/genuts/closelog_L/closelog_L.c	1.5 99/04/26 */
/*
* (C) Copyright 1998 UniSoft Limited.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*/
#ifndef	lint
#define	MAIN
#ifdef	MAIN
static	char sccsid[] = "(#)SOLTESTS/tset/LSB.os/genuts/closelog_L/closelog_L.c	1.5 99/04/26";
char	*copyright[] = {
		"(C) Copyright 1998 UniSoft Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.closelog_L

PROJECT:	Solaris ABI Test Suite

DESCRIPTION:
	Testset for the closelog() function.

AUTHOR:		Matthew Hails, UniSoft Ltd.
DATE CREATED:	November 1998

************************************************************************/

#include <std.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>
#include <ctype.h>
#include <fcntl.h>
#include <float.h>
#include <langinfo.h>
#include <limits.h>
#include <locale.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <syslog.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>
#include <wchar.h>
#include <wait_child.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>

#ifdef UNDEF_MACROS
#undef closelog
#endif

#define NO_TESTS 2

#define WAITTIME		(5 * SPEEDFACTOR)
#define WAITLOGDSECS		30
#define MAXSTRINGSIZE		512

/* External prototypes */
extern char *errname(int);
extern int n_open_fds(void);

/* Local prototypes */
static pid_t getsyslogdpid(void);
static int setup_syslogd(char *debuglogfile);
static int cleanup_syslogd(void);
static void prepare(void);
static void clean(void);
static int setupsyslog(char *cmd, char *arg);

/* External data declarations */
extern int ex_code;
extern int sg_code;
extern char **_environ;

/* Global static data */
static char setupprog[] = "../setupsyslog";
static char testmsg[] = "This is only a test";
static char logfile[] = "./debug-messages";
static char *logfiles[5] = { "./debug-messages.0",
			     "./debug-messages.1",
			     "./debug-messages.2",
			     "./debug-messages.3",
			     "./debug-messages.4" };

static void test1(void);
static void sub_test1(int type, char **msgptr);
static void test2(void);
static void sub_test2(int type, char **msgptr);

/*
 * Test case startup function.
 */
static void
prepare(void)
{
	int fd;
	int i;

	DBUG_ENTER("prepare");

	for (i = 0; i <= 4; i++)
	{
		/* Create log file */
		unlink(logfiles[i]);

		fd = open(logfiles[i], O_WRONLY|O_CREAT|O_EXCL,
			S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
		if (fd == -1)
		{
			in_rpt("Error creating %s", logfiles[i]);
			xx_all(DELETION, "Can't create log file to use with syslogd");
			DBUG_VOID_RETURN;
		}
	}

	/* Start up syslogd and tweak config file */
	if (setupsyslog("startup", logfile) != 0)
	{
		xx_all(DELETION, "Error starting/configuring syslogd");
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

/*
 * Test case cleanup function.
 */
static void
clean(void)
{
	int i;

	DBUG_ENTER("clean");

	/* Tidy up syslog conf file etc. */
	setupsyslog("cleanup", NULL);

	/* Remove temporary log file */
	for (i = 0; i <= 4; i++)
		unlink(logfiles[i]);

	DBUG_VOID_RETURN;
}

static int
setupsyslog(char *cmd, char *arg)
{
	static char *args[4];

	DBUG_ENTER("setupsyslog");

	tet_child = fork();
	switch (tet_child)
	{
	case -1:
		in_rpt("fork failure in startup");
		DBUG_RETURN(-1);
	case 0:
		tet_setcontext();
		args[0] = setupprog;
		args[1] = cmd;
		args[2] = arg;
		args[3] = NULL;
		tet_exec(args[0], args, _environ);
		in_rpt("Failed to exec %s", setupprog);
		DBUG_EXIT(1);
	}

	switch (wait_child(WAITTIME))
	{
	case CHLD_EXIT:
		if (ex_code == 0)
		{
			/* Give syslogd time to sort itself out */
			sleep(WAITLOGDSECS * 2);
		}
		else
		{
			in_rpt("%s exited with status %d", setupprog, ex_code);
		}
		DBUG_RETURN(ex_code);
	case CHLD_ABRT:
		in_rpt("%s terminated by signal %d", setupprog, sg_code);
		DBUG_RETURN(-1);
	case CHLD_LIVE:
		in_rpt("%s timed out", setupprog);
		break;
	case CHLD_STOP:
		in_rpt("%s stopped", setupprog);
		break;
	default:
		in_rpt("Error in waitpid()");
		break;
	}

	kill(tet_child, SIGKILL);
	DBUG_RETURN(-1);
}

/********************* Beginning of test cases *********************/

static void
test1(void)
{
	int pathok = 0;
	int start_fds, end_fds;

	DBUG_ENTER("test1");

	start_fds = n_open_fds();
	
	openlog("test-syslog-1", LOG_NDELAY, LOG_DEBUG);

	closelog();

	end_fds = n_open_fds();

	if (start_fds != end_fds)
	{
		xx_rpt(FAILURE);
		in_rpt("closelog() did not close all file descriptors opened by openlog()");
		in_rpt("Number of additional open file descriptors = %d", (end_fds - start_fds));
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

static void
test2(void)
{
	int pathok = 0;
	int start_fds, end_fds;

	DBUG_ENTER("test2");

	start_fds = n_open_fds();
	
	syslog(LOG_DEBUG, testmsg);

	closelog();

	end_fds = n_open_fds();

	if (start_fds != end_fds)
	{
		xx_rpt(FAILURE);
		in_rpt("closelog() did not close all file descriptors opened by syslog()");
		in_rpt("Number of additional open file descriptors = %d", (end_fds - start_fds));
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/************************ End of test cases ************************/

int no_tests = NO_TESTS;

void (*setfunc)() = prepare;
void (*clnfunc)() = clean;

struct tet_testlist tet_testlist[] =
{
	{ test1, 1 },
	{ test2, 1 },
	{ NULL, 0 }
};
