/*
*      SCCS:  @(#)  XPG4.os/genuts/fnmatch/fnmatch.c	Rel 4.4.1   (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XPG4.os/genuts/fnmatch - T.fnmatch Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.fnmatch

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fnmatch() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Peter Mulvenney, UniSoft Ltd.
DATE CREATED:	Wed Aug 21 14:59:37 BST 1991
MODIFICATIONS:
		Neil Moses, 7 May 1992.
			Check that interface is supported.
			Added TEST For ENOSYS if interface not supported.

		Neil Moses, 3 June 1992.
			Use sysconf() to check if interface supported.

		Geoff Clare, 22 Dec 1992
			Align with interpretation VWG/025/081892.

		Geoff Clare, 8 Sept 1995
			Allow for "[.]" to match "." with FNM_PERIOD set.

		Geoff Clare, 20 Jan 1997
			Missing commas in test 8 and 10 initializers for
			pattern[] array.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<string.h>
#include	<limits.h>
#include	<dbug.h>
#include	<report.h>
#include	<errno.h>
#include <tet_api.h>
#include	<fnmatch.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fnmatch
#endif


#define 	NO_TESTS	10	/* number of tests in testset */
#define 	NO_PATTS	18	/* number of patterns in array */

extern	void	xx_all();
extern	char	*errname();

int	no_tests = NO_TESTS;

private	int	tsup = 0;
private	int	setupok = -1;
private	int	testfail;

/*
 * String sets structure, for tests 1 and 2. Pattern is the pattern to
 * match against, mstring is a string expected to match, and nmstring
 * is a string not expected to match.
 */
struct strset {
		char *pattern;
		char *mstring;
		char *nmstring;
		};

/*
 * Test data for tests 1 and 2.  Note that each individual
 * backslash has to be escaped to protect it from the C compiler.
 */

private struct strset td[NO_PATTS] = {
/*
 *<-pattern->		<-matching string->		<-non-matching string->
 */
"*file123",		"/usr/subdir/file123",		"/usr/subdir/file12",
"/usr/subdir?file123",	"/usr/subdir/file123",		"usr/subdir/file123",
"/usr/subdir[-./]file123",	"/usr/subdir/file123", "/usr/subdir?file123",
"/usr/subdir/file123*",	"/usr/subdir/file123",		"/usr/subdir\\file123",
"\\*ile123",		"*ile123",			"\\le123",
"\\?ile123",		"?ile123",			"ile123",
"\\\\e123",		"\\e123",			"e1-3",
"*file",		".profile",			".profiles",
"?profile",		".profile",			".Profile",
"[,.+-]profile",	".profile",			"?profile",
".*profile",		".profile",			"profile",
"/usr/*file",		"/usr/.profile",		"/usr/.profiles",
"/usr/?profile",	"/usr/.profile",		"/usr/.Profile",
"/usr/[,.+-]profile",	"/usr/.profile",		"/usr/?profile",
"$HOME",		"$HOME",			"?HOME",
"`hostname`",		"`hostname`",			"hostname",
"~/file123",		"~/file123",			"/usr/file123",
"~vsx1/file123",	"~vsx1/file123",		"/usr/vsx1/file123"
};

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();

private void	setup();
private int	tbfnmatch();
private char	*flagstr();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	NULL, 0
};

private void
setup()
{
	int	pathok = 0;

	DBUG_ENTER("setup");

	errno = 0;
	if(sysconf(_SC_2_C_VERSION) == -1)
	{
		if (errno != 0)
		{
			xx_all(DELETION, "sysconf(_SC_2_C_VERSION) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		tsup = 0;
	}
	else
	{
		tsup = 1;
		PATH_TRACE;
	}
	
	PATH_SETUP_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = setupok;
	int	i;

	DBUG_ENTER("test1");

	testfail = 0;

	for (i = 0; i < NO_PATTS; i++)
	{
		globok = 0;
		(void) tbfnmatch(td[i].pattern, td[i].mstring, 0, SUCCEED);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(NO_PATTS);

	DBUG_VOID_RETURN;
}


private	void
test2()
{
	int	pathok = setupok;
	int	i;

	DBUG_ENTER("test2");

	testfail = 0;

	for (i = 0; i < NO_PATTS; i++)
	{
		globok = 0;
		(void) tbfnmatch(td[i].pattern, td[i].nmstring, 0, FNM_NOMATCH);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(NO_PATTS);
			
	DBUG_VOID_RETURN;
}


private	void
test3()
{
	int	pathok = setupok;
	int	i;

	static char *pattern1[3] = {
			"*/*/*file",
			"?/subdir/?profile",
			"[.,]/subdir/[.,]profile"
			};
	char *string1 = "./subdir/.profile";

	static char *pattern2[3] = {
			"/*/*file",
			"/subdir/?profile",
			"/subdir/[.,]profile"
			};
	char *string2 = "/subdir/.profile";

	DBUG_ENTER("test3");

	testfail = 0;

	/*
	 * First check that a leading period is matched successfully.
	 */
	for (i = 0; i < 3; i++)
	{
		globok = 0;
		(void) tbfnmatch(pattern1[i], string1, FNM_PATHNAME, SUCCEED);
		PATH_FUNC_TRACE;
	}

	/*
	 * Then check that a period immediately following
	 * a slash character is matched successfully.
	 */
	for (i = 0; i < 3; i++)
	{
		globok = 0;
		(void) tbfnmatch(pattern2[i], string2, FNM_PATHNAME, SUCCEED);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}


private void
test4()
{
	int	pathok = setupok;
	char *pattern = "/*/file123";
	char *string = "/usr/subdir/file123";

	DBUG_ENTER("test4");

	testfail = 0;

	globok = 0;
	(void) tbfnmatch(pattern, string, FNM_PATHNAME, FNM_NOMATCH);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}


private void
test5()
{
	int	pathok = setupok;
	/*
	 * Note that each individual backslash has to be escaped
	 * to protect it from the C compiler.
	 */
	char *pattern = "\\\\ab\\?\\*.\\.";
	char *string = "\\ab?*..";

	DBUG_ENTER("test5");

	testfail = 0;

	globok = 0;
	(void) tbfnmatch(pattern, string, 0, SUCCEED);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}


private void
test6()
{
	int	pathok = setupok;
	/*
	 * Note that each individual backslash has to be escaped
	 * to protect it from the C compiler.
	 */
	char *pattern = "\\\\ab\\?\\*.\\.";
	char *string = pattern;

	DBUG_ENTER("test6");

	testfail = 0;

	globok = 0;
	(void) tbfnmatch(pattern, string, FNM_NOESCAPE, SUCCEED);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	int	pathok = setupok;
	int	i;

	static char *pattern1[3] = {
			".*.profile",
			".?.profile",
			".[/.].profile"
			};
	static char *pattern2[3] = {
			"./*profile",
			"./?profile",
			"./[/.]profile"
			};
	char *string = "./.profile";

	DBUG_ENTER("test7");

	testfail = 0;

	/*
	 * First check that a slash character is matched by special characters
	 */
	for (i = 0; i < 3; i++)
	{
		globok = 0;
		(void) tbfnmatch(pattern1[i], string, FNM_PERIOD, SUCCEED);
		PATH_FUNC_TRACE;
	}

	/*
	 * Then check that a period immediately following
	 * a slash character is matched successfully.
	 */
	for (i = 0; i < 3; i++)
	{
		globok = 0;
		(void) tbfnmatch(pattern2[i], string, FNM_PERIOD, SUCCEED);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	int	pathok = setupok;
	int	i;

	static char *pattern[4] = { "*profile",
			     "?profile",
			     "[!a]profile",
			     "[[:punct:]]profile"
			     };
	char *string = ".profile";

	DBUG_ENTER("test8");

	testfail = 0;

	for (i = 0; i < 4; i++)
	{
		globok = 0;
		(void) tbfnmatch(pattern[i], string, FNM_PERIOD, FNM_NOMATCH);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	int	pathok = setupok;
	char *pattern = ".*/.profile";
	char *string = "./.profile";

	DBUG_ENTER("test9");

	testfail = 0;

	globok = 0;
	(void) tbfnmatch(pattern, string, (FNM_PERIOD | FNM_PATHNAME), SUCCEED);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	int	pathok = setupok;
	int i;

	static char *pattern[8] = {	"./*file",
				"./?profile",
				"./[!a]profile",
			        "./[[:punct:]]profile",
				"*/.profile",
				"?/.profile",
				"[!a]/.profile",
			        "[[:punct:]]/.profile"
				};
	char *string = "./.profile";

	DBUG_ENTER("test10");

	testfail = 0;

	for (i = 0; i < 8; i++)
	{
		globok = 0;
		(void) tbfnmatch(pattern[i], string,
				(FNM_PERIOD | FNM_PATHNAME), FNM_NOMATCH);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

private int
tbfnmatch(pattern, string, flags, expret)
char *pattern;
char *string;
int flags;
int expret;
{
	int pathok = 0;
	int ret, err;
	static int firstsup = -1;

	DBUG_ENTER("tbfnmatch");

	errno = 0;
	ret = fnmatch(pattern, string, flags);
	err = errno;

	/* If the POSIX2 CLB group is not supported, fnmatch() can
	   still be supported on its own, but calls must consistently
	   indicate support or non-support. */

	if (ret != 0 && ret != FNM_NOMATCH && err == ENOSYS)
	{
		if (tsup)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("fnmatch(\"%s\", \"%s\", %s) not correct:",
				pattern, string, flagstr(flags));
			in_rpt("fnmatch() gave ENOSYS when POSIX2 CLB group is supported");
			DBUG_RETURN(ret);
		}
		else if (firstsup == -1)
		{
			firstsup = 0;
			PATH_TRACE;
		}
		else if (firstsup == 1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("fnmatch(\"%s\", \"%s\", %s) not correct:",
				pattern, string, flagstr(flags));
			in_rpt("fnmatch() gave ENOSYS but a previous call did not");
			DBUG_RETURN(ret);
		}
		else
			PATH_TRACE;

		PATH_FUNC_RPT(1);
		DBUG_RETURN(ret);
	}
	else
	{
		if (firstsup == -1)
		{
			firstsup = 1;
			PATH_TRACE;
		}
		else if (firstsup == 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("fnmatch(\"%s\", \"%s\", %s) not correct:",
				pattern, string, flagstr(flags));
			in_rpt("fnmatch() did not give ENOSYS but a previous call did");
		}
		else
			PATH_TRACE;
	}

	/* Now check the return value/errno as per normal */

	if (ret != expret)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fnmatch(\"%s\", \"%s\", %s) not correct:",
			pattern, string, flagstr(flags));
		in_rpt("return value %d%s, expected %d%s",
			ret, (ret == FNM_NOMATCH ? " (FNM_NOMATCH)" : ""),
			expret, (expret == FNM_NOMATCH ? " (FNM_NOMATCH)" : ""));
		if (ret != 0 && ret != FNM_NOMATCH)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);
	DBUG_RETURN(ret);
}

private char *
flagstr(flags)
int flags;
{
	static char buf[128];

	if (flags == 0)
		return "0";
	
	*buf = '\0';

	if (flags & FNM_NOESCAPE)
		(void) strcat(buf, " | FNM_NOESCAPE");

	if (flags & FNM_PATHNAME)
		(void) strcat(buf, " | FNM_PATHNAME");

	if (flags & FNM_PERIOD)
		(void) strcat(buf, " | FNM_PERIOD");
	
	return &buf[3];
}
