/*
*      SCCS:  @(#)  XOPEN.os/genuts/ftw/06/09/97	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/genuts/ftw - T.ftw Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.ftw

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the ftw() subroutine.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Mon Nov 24 11:38:47 WET 1986
MODIFICATIONS:	Andrew Twigger	10 June 87
			Removed	abandon(), testid()
			Modified xx_rpt() xs_rpt()

		Geoff Clare, 18 Nov 1988
			Add tests for errno values

		Dave Cater Tue May 16 14:11:48 BST 1989
			test13 was not testing a path prefix component 

		Geoff Clare, 28 July 1989
			Test with ndirs -1 should allow EINVAL

		Geoff Clare, 21 Sep 1989
			Remove EACCES test.
		
		Geoff Clare, 6 Apr 1990
			New interface to nametoolng() and pathtoolng()

		J.A.Nave	22/05/90
		Added resilience testing
		
		Geoff Clare, 19 Oct 1991
			Remove multiple globok definition.

                John-Paul, 29 July 1992
			'mode_t' casting added to mode parameter in calls to
			crfile() and crdir().

		Neil Moses, 16 Oct 1992
			XNFS enhancements

		Geoff Clare, 20 Nov 1992
			Fix path tracing error in test 7.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<stdlib.h>
#include	<string.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<ftw.h>
#include <tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ftw
#endif


#define		NO_TESTS	17	/* number of tests in testset */
#define		COMP_TESTS	8	/* number of compliance tests */
#define		WAITTIME	(5*SPEEDFACTOR)
#define		NMR_FILES	7
#define		FTWF		01	/* bits in flag set by f3() */
#define		FTWD		02
#define		FTWDNR		04
#define		FTWNS	       010

#define		MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();
extern	char	**pathtoolng();
#if TEST_XNFS
extern   char   *xnfs_nametoolng();
#else
extern	char	**nametoolng();
#endif
extern	void	clntoolng();

int	no_tests = NO_TESTS;
private	int	jumpok = 0;
private	int	setupok = 0;

struct	ft
{
	char	*name;
	ino_t	inode;
	dev_t	device;
	int	flag;
};

private struct	ft	ft[NMR_FILES] =
{
	"./ftw_d.2/ftw_t.4",	0,	0,	TRUE,
	"./ftw_d.2",		0,	0,	TRUE,
	"./ftw_d.1/ftw_t.3",	0,	0,	TRUE,
	"./ftw_d.1/ftw_t.2",	0,	0,	TRUE,
	"./ftw_d.1",		0,	0,	TRUE,
	"./ftw_t.1",		0,	0,	TRUE,
	".",			0,	0,	TRUE
};

private char	*t9_dir =	"./ftw_d.9";
private char	*t11_dir =	"./nonexistant";
private char	*t13_file =	"./ftw_t.13";
private char	*t13a_file =	"./ftw_t.13/ftw_t.13";

private void	prepare_tests();
private void	cln_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private int	f1();
private int	f2();
private int	f3();
private int	f4();
private int	f5();
private int	f6();
private int	f7();
private int	f8();
private int	f9();
private int	f10();
private int	f11();
private int	f12();
private int	f13();
#if TEST_XNFS
private int	f14();
private int	f15();
private int	f16();
private int	f17();
#endif /* TEST_XNFS */

private int	errflag;
private int	flag;
private int	ftopen;

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	NULL, 0
};


private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) unlink("./ftw_d.2/ftw_t.4");
	(void) rmdir("./ftw_d.2");
	(void) unlink("./ftw_d.1/ftw_t.3");
	(void) unlink("./ftw_d.1/ftw_t.2");
	(void) rmdir("./ftw_d.1");
	(void) unlink("./ftw_t.1");

	(void) rmdir(t9_dir);
	(void) unlink(t13_file);

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	int	fd;
	int	i;
	struct	stat	statb;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	if ((fd = creat("./ftw_t.1", MODEANY)) == SYSERROR)
	{
		DBUG_PRINTF("creat", "ftw_t.1");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}

	if ( crdir("./ftw_d.1", MODEANY, (int) getuid(), (int) getgid())
		== SYSERROR )
	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "failed to create directory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( (fd = creat("./ftw_d.1/ftw_t.2", MODEANY)) == SYSERROR )
	{
		DBUG_PRINTF("creat", "ftw_t.2");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}

	if ( (fd = creat("./ftw_d.1/ftw_t.3", MODEANY)) == SYSERROR )
	{
		DBUG_PRINTF("creat", "ftw_t.3");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}

	if ( crdir("./ftw_d.2", MODEANY, (int) getuid(), (int) getgid() )
		== SYSERROR )
	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "failed to create directory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( (fd = creat("./ftw_d.2/ftw_t.4", MODEANY)) == SYSERROR )
	{
		DBUG_PRINTF("creat", "ftw_t.4");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}


	for ( i = 0 ; i < NMR_FILES ; i++ )
	{
		if (stat(ft[i].name, &statb) == SYSERROR)
		{
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				(void) tet_delete(i, "stat call failed");
			DBUG_VOID_RETURN;
		}
		else
		{
			PATH_TRACE;
			ft[i].inode = statb.st_ino;
			ft[i].device = statb.st_dev;
		}
	}

	setupok = (pathok == NMR_FILES+6) ? 0 : -1;

	DBUG_VOID_RETURN;
}


private void
test1()
{
	int	nfails = 0;
	int	rtval;
	int	i;
	int	pathok = setupok;

	DBUG_ENTER("test1");

	jumpok = 0;
	if ( (rtval = ftw(".", f1, 3)) != 0 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 21)
		PATH_TRACE;

	for ( i = 0 ; i < NMR_FILES ; i++ )
	{
		if ( ft[i].flag == TRUE )
		{
			if ( nfails++ == 0 )
				xx_rpt(FAILURE);
			in_rpt("function set by ftw not called for %s",
					ft[i].name);
		}
		else
			PATH_TRACE;

		ft[i].flag = TRUE;	/* reset for later tests	*/
	}

	if ( nfails == 0 )
		PATH_XS_RPT(NMR_FILES+2);

	DBUG_VOID_RETURN;
}


/*ARGSUSED*/
private int
f1(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	int	i;

	DBUG_ENTER("f1");

	for ( i = 0 ; i < NMR_FILES ; i++ )
	{
		if ( statptr == (struct stat *) NULL )
			continue;

		if ( statptr->st_dev == ft[i].device
		    && statptr->st_ino == ft[i].inode )
		{
			ft[i].flag = FALSE;
			DBUG_RETURN(0);
		}
		else
			JUMP_TRACE;
	}

	DBUG_RETURN(0);
}


private void
test2()
{
	int	nfails = 0;
	int	rtval;
	int	i;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	errflag = FALSE;
	jumpok = 0;
	if ( (rtval = ftw(".", f2, 3)) != 0 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 7)
		PATH_TRACE;

	if ( errflag )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("function set by ftw called with 3rd argument");
		in_rpt("other than FTW_D or FTW_F");
	}
	else
		PATH_TRACE;

	for ( i = 0 ; i < NMR_FILES ; i++ )
	{
		if ( ft[i].flag == TRUE )
		{
			if ( nfails++ == 0 )
				xx_rpt(FAILURE);
			in_rpt("function called by ftw set flag for file");
			in_rpt("if its parent directory was already visited.");
			in_rpt("flag was not set for %s", ft[i].name);
		}
		else
			PATH_TRACE;
		ft[i].flag = TRUE;	/* reset for later tests	*/
	}

	if ( nfails == 0 )
		PATH_XS_RPT(NMR_FILES+3);

	DBUG_VOID_RETURN;
}


/*ARGSUSED*/
private int
f2(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	int	i;

	DBUG_ENTER("f2");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	for ( i = 0 ; i < NMR_FILES ; i++ )
	{
		if ( statptr == (struct stat *) NULL )
			continue;

		if ( statptr->st_dev == ft[i].device
		    && statptr->st_ino == ft[i].inode )
		{
			switch (type)
			{

			case	FTW_D:	/*	directory	*/
				ft[i].flag = FALSE;
				break;

			case	FTW_F:	/*	file		*/

				/*
				 *	if parent directory's flag is FALSE
				 *	set this file's flag to FALSE
				 */

				if ( i == 0 && ft[1].flag == FALSE )
					ft[i].flag = FALSE;

				if ( ( i == 2 || i == 3 )
				     && ft[4].flag == FALSE )
					ft[i].flag = FALSE;

				if ( i == 5 && ft[6].flag == FALSE )
					ft[i].flag = FALSE;
				break;

			default :
				errflag = TRUE;
				break;

			}
			DBUG_RETURN(0);
		}
	}
	DBUG_RETURN(0);
}


private void
test3()
{
	int	i;
	int	rtval;
	int	nfails = 0;
	int	pathok = setupok;

	DBUG_ENTER("test3");

	if ( chmod("./ftw_d.1",
	    (mode_t)(S_IWUSR|S_IXUSR|S_IWGRP|S_IXGRP|S_IWOTH|S_IXOTH))
	    == SYSERROR )
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: chmod failed with errno %d (%s)",
				errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( chmod("./ftw_d.2",
	    (mode_t)(S_IWUSR|S_IWGRP|S_IWOTH|S_IRUSR|S_IRGRP|S_IROTH))
	    == SYSERROR )
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: chmod failed with errno %d (%s)",
				errno, errname(errno));

		if ( chmod("./ftw_d.1", MODEANY) == SYSERROR )
			for ( i = 4 ; i <= COMP_TESTS ; i++ )
				(void) tet_delete(i, "chmod call failed");

		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	flag = 0;
	jumpok = 0;
	if ( (rtval = ftw(".", f3, 3)) != 0 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 5)
		PATH_TRACE;

	if ( (flag & FTWF) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_F and");
		in_rpt("correct first and second arguments when traversing");
		in_rpt("file tree");
	}
	else
		PATH_TRACE;

	if ( (flag & FTWD) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_D and");
		in_rpt("correct first and second arguments when traversing");
		in_rpt("file tree");
	}
	else
		PATH_TRACE;

	if ( (flag & FTWDNR) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_DNR and");
		in_rpt("correct first and second arguments when traversing");
		in_rpt("file tree");
	}
	else
		PATH_TRACE;

	if ( (flag & FTWNS) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_NS and");
		in_rpt("correct first argument when traversing");
		in_rpt("file tree");
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(8);

	if ( chmod("./ftw_d.1", MODEANY) == SYSERROR )
		for ( i = 4 ; i <= COMP_TESTS ; i++ )
			(void) tet_delete(i, "chmod call failed");

	if ( chmod("./ftw_d.2", MODEANY) == SYSERROR )
		for ( i = 4 ; i <= COMP_TESTS ; i++ )
			(void) tet_delete(i, "chmod call failed");

	DBUG_VOID_RETURN;


}


/*ARGSUSED*/
private int
f3(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	struct	stat	statb;

	DBUG_ENTER("f3");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	switch (type)
	{

	case	FTW_F:
		if ( stat(path, &statb) != SYSERROR
		    && statb.st_dev == statptr->st_dev
		    && statb.st_ino == statptr->st_ino
		    && statptr->st_dev == ft[5].device
		    && statptr->st_ino == ft[5].inode )
			flag |= FTWF;
		break;

	case	FTW_D:
		if ( stat(path, &statb) != SYSERROR
		    && statb.st_dev == statptr->st_dev
		    && statb.st_ino == statptr->st_ino
		    && statptr->st_dev == ft[6].device
		    && statptr->st_ino == ft[6].inode )
			flag |= FTWD;
		break;

	case	FTW_DNR:
		if ( stat(path, &statb) != SYSERROR
		    && statb.st_dev == statptr->st_dev
		    && statb.st_ino == statptr->st_ino
		    && statptr->st_dev == ft[4].device
		    && statptr->st_ino == ft[4].inode )
			flag |= FTWDNR;
		break;

	case	FTW_NS:
		if ( strcmp(&path[ strlen(path) - 7 ], "ftw_t.4") == 0 )
			flag |= FTWNS;
		break;
	}

	DBUG_RETURN(0);

}


private void
test4()
{
	int	i;
	int	rtval;
	int	nfails = 0;
	int	pathok = setupok;

	DBUG_ENTER("test4");

	if ( chmod("./ftw_d.1",
	    (mode_t)(S_IWUSR|S_IXUSR|S_IWGRP|S_IXGRP|S_IWOTH|S_IXOTH))
	    == SYSERROR )
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: chmod failed with errno %d (%s)",
				errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errflag = FALSE;
	flag = 0;
	jumpok = 0;
	if ( (rtval = ftw("./ftw_d.1", f4, 2)) != 0 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(./ftw_d.1, ..., 2) returned %d instead of 0",
					rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 1)
		PATH_TRACE;

	if ( errflag )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw on file tree with read-only root");
		in_rpt("called function with 3rd argument other");
		in_rpt("than FTW_DIR");
	}
	else
		PATH_TRACE;

	if ( (flag & FTWDNR) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_DNR");
		in_rpt("although the root of the file tree given was");
		in_rpt("read-only");
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(5);

	if ( chmod("./ftw_d.1", MODEANY) == SYSERROR )
		for ( i = 5 ; i <= COMP_TESTS ; i++ )
			(void) tet_delete(i, "chmod call failed");

	DBUG_VOID_RETURN;

}


/*ARGSUSED*/
private int
f4(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	DBUG_ENTER("f4");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	if ( type == FTW_DNR )
		flag |= FTWDNR;
	else
		errflag = TRUE;

	DBUG_RETURN(0);

}


private void
test5()
{
	int	rtval;
	int	nfails = 0;
	int	nopen;
	int	pathok = setupok;

	DBUG_ENTER("test5");

	nopen = n_open_fds();

	jumpok = 0;
	if ( (rtval = ftw(".", f5, 3)) != 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0",
					rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 7)
		PATH_TRACE;

	if ( ftopen - nopen > 2 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw opened %d file descriptors instead of 2 to",
				ftopen - nopen);
		in_rpt("traverse two levels of the file tree");
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;

}


/*ARGSUSED*/
private int
f5(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{

	DBUG_ENTER("f5");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	if ( statptr->st_dev == ft[3].device && statptr->st_ino == ft[3].inode )
		ftopen = n_open_fds();
	DBUG_RETURN(0);

}


private void
test6()
{
	int	rtval;
	int	nfails = 0;
	int	nopen;
	int	pathok = setupok;

	DBUG_ENTER("test6");

	nopen = n_open_fds();

	jumpok = 0;
	if ( (rtval = ftw(".", f6, 1)) != 0 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 1) returned %d instead of 0",
					rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 7)
		PATH_TRACE;

	if ( ftopen - nopen > 1 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 1) opened %d file descriptors",
				ftopen - nopen);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;

}


/*ARGSUSED*/
private int
f6(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{

	DBUG_ENTER("f6");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	if ( statptr->st_dev == ft[3].device && statptr->st_ino == ft[3].inode )
		ftopen = n_open_fds();

	DBUG_RETURN(0);

}


private void
test7()
{
	int	rtval, err;
	int	nfails = 0;
	int	nopen;
	int	pathok = setupok;

	DBUG_ENTER("test7");

	nopen = n_open_fds();
	jumpok = 0;
	if ( (rtval = ftw(".", f7, -1)) != 0 )
	{
		if (rtval != SYSERROR || errno != EINVAL)
		{
			err = errno;
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("ftw(\".\", ..., -1) returned %d with errno %d (%s)",
				rtval, err, errname(err));
			in_rpt("expected return 0, or -1 with errno %d (EINVAL)",
				EINVAL);
		}
		else
			PATH_TRACE;
	}
	else if (jumpok == 7)
		PATH_TRACE;

	ftopen = n_open_fds();

	if ( ftopen - nopen > 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., -1) left %d file descriptors open",
				ftopen - nopen);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;

}


/*ARGSUSED*/
private int
f7(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{

	DBUG_ENTER("f7");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	if ( statptr->st_dev == ft[3].device && statptr->st_ino == ft[3].inode )
		ftopen = n_open_fds();

	DBUG_RETURN(0);

}


private void
test8()
{
	int	rtval;
	int	nfails = 0;
	int	pathok = setupok;

	DBUG_ENTER("test8");

	errflag = FALSE;
	flag = FALSE;
	jumpok = 0;
	if ( (rtval = ftw("./ftw_d.1", f8, 2)) != 99 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(./ftw_d.1, ..., 2) returned %d instead of 99",
					rtval);
		in_rpt("which was the return value of the function set by ftw");
	}
	else
		PATH_TRACE;

	if (jumpok == 1)
		PATH_TRACE;

	if ( errflag )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("function called by ftw more than once although first");
		in_rpt("call returned nonzero value of 99");
	}
	else
		PATH_TRACE;

	if ( flag != TRUE )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("function set by ftw not called for root of file tree");
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;

}


/*ARGSUSED*/
private int
f8(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	DBUG_ENTER("f8");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	if ( statptr->st_dev == ft[4].device && statptr->st_ino == ft[4].inode )
		flag = TRUE;
	else
		errflag = TRUE;

	DBUG_RETURN(99);
}

private void
test9()
{
	int	rtval, err;
	int	fail = 0;
	char	**longname;
	int	pathok = setupok;

	DBUG_ENTER("test9");

	globok = 0;
	if (crdir(t9_dir, (mode_t)MODEANY, (int) getuid(), (int) getgid())
		== SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t9_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	longname = pathtoolng(t9_dir);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else if (globok == 1)
		PATH_TRACE;

	rtval = ftw(longname[0], f9, 2);
	if (rtval == -1)
	{
		fail++;
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("ftw(<%d char path>, ..., 2) failed - errno %d (%s)",
				strlen(longname[0]), err, errname(err));
	}
	else
		PATH_TRACE;

	if (longname[1] != NULL)
	{
	    rtval = ftw(longname[1], f9, 2);
	    if (rtval != -1 || errno != ENAMETOOLONG)
	    {
		fail++;
		err = errno;
		xx_rpt(FAILURE);
		if (rtval != -1)
		{
			in_rpt("ftw(<%d char path>, ..., 2) returned %d instead of -1",
				strlen(longname[1]), rtval);
		}
		else if (errno != ENAMETOOLONG)
		{
			in_rpt("ftw(<%d char path>, ..., 2) set errno to %d (%s) instead of ENAMETOOLONG",
				strlen(longname[1]), err, errname(err));
		}
	    }
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (fail == 0)
		PATH_XS_RPT(4);

	clntoolng();
	(void) rmdir(t9_dir);

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f9(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	DBUG_ENTER("f9");

	DBUG_RETURN(1);

}

#if TEST_XNFS

private void
test10()
{
	DBUG_ENTER("test10");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test10()
{
	int	rtval, err;
	int	fail = 0;
	char	**longname;
	int	pathok = setupok;

	DBUG_ENTER("test10");

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else if (globok == 1)
		PATH_TRACE;

	if (crdir(longname[0], MODEANY, (int) getuid(), (int) getgid())
		== SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s",
			longname[0]);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rtval = ftw(longname[0], f10, 2);
	if (rtval == -1)
	{
		fail++;
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("ftw(<%d char filename>, ..., 2) failed - errno %d (%s)",
				strlen(longname[0]), err, errname(err));
	}
	else
		PATH_TRACE;

	if (longname[1] != NULL)
	{
		rtval = ftw(longname[1], f10, 2);
		if (rtval != -1 || errno != ENAMETOOLONG)
		{
			fail++;
			err = errno;
			xx_rpt(FAILURE);
			if (rtval != -1)
			{
				in_rpt("ftw(<%d char filename>, ..., 2) returned %d instead of -1",
					strlen(longname[1]), rtval);
			}
			else if (errno != ENAMETOOLONG)
			{
				in_rpt("ftw(<%d char filename>, ..., 2) set errno to %d (%s) instead of ENAMETOOLONG",
					strlen(longname[1]), err, errname(err));
			}
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (fail == 0)
		PATH_XS_RPT(4);

	(void) rmdir(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f10(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	DBUG_ENTER("f10");

	DBUG_RETURN(1);
}

#endif /* TEST_XNFS */

private void
test11()
{
	int	rtval, err;
	int	pathok = setupok;

	DBUG_ENTER("test11");

	if ( (rtval = ftw(t11_dir, f11, 2)) != -1 )
	{
		xx_rpt(FAILURE);
		in_rpt("ftw(%s, ..., 2) returned %d instead of -1", t11_dir,
					rtval);
	}
	else if (errno != ENOENT)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("ftw(%s, ..., 2) set errno to %d (%s) instead of ENOENT",
			t11_dir, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f11(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	DBUG_ENTER("f11");

	DBUG_RETURN(1);

}

private void
test12()
{
	int	rtval, err;
	int	pathok = setupok;

	DBUG_ENTER("test12");

	if ( (rtval = ftw("", f12, 2)) != -1 )
	{
		xx_rpt(FAILURE);
		in_rpt("ftw(\"\", ..., 2) returned %d instead of -1", rtval);
	}
	else if (errno != ENOENT)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("ftw(\"\", ..., 2) set errno to %d (%s) instead of ENOENT",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f12(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	DBUG_ENTER("f12");

	DBUG_RETURN(1);

}

private void
test13()
{
	int	rtval, err;
	int	pathok = setupok;

	DBUG_ENTER("test13");

	if (crfile(t13_file, MODEANY, (int) getuid(), (int) getgid())
		== SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( (rtval = ftw(t13a_file, f13, 2)) != -1 )
	{
		xx_rpt(FAILURE);
		in_rpt("ftw(%s, ..., 2) returned %d instead of -1", t13a_file,
					rtval);
		in_rpt("where %s was a plain file", t13_file);
	}
	else if (errno != ENOTDIR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("ftw(%s, ..., 2) set errno to %d (%s) instead of ENOTDIR",
			t13a_file, err, errname(err));
		in_rpt("where %s was a plain file", t13_file);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) unlink(t13_file);

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f13(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	DBUG_ENTER("f13");

	DBUG_RETURN(1);

}

#if TEST_XNFS

private void
test14()
{
	char	*longname;
	int	err, rval;
	int	pathok = 0;

	DBUG_ENTER("test14");


	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	errno = NOERROR;
	rval = ftw(longname, f14, 2);
	err = errno;

	if(rval != -1 || err != ENAMETOOLONG)
	{
		xx_rpt(FAILURE);

		if (rval != -1)
		{
			in_rpt("ftw(<%d char filename>, ..., 2) returned %d instead of -1",
				strlen(longname), rval);
		}
		else if (err != ENAMETOOLONG)
		{
			in_rpt("ftw(<%d char filename>, ..., 2) set errno to %d (%s) instead of ENAMETOOLONG",
				strlen(longname), err, errname(err));
		}
	}
	else
		PATH_XS_RPT(1);

	clntoolng();

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f14(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	DBUG_ENTER("f14");

	DBUG_RETURN(1);

}

#else /* TEST_XNFS */

private void
test14()
{
	DBUG_ENTER("test14");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test15()
{
	char	*longname;
	int	err, rval;
	int	pathok = 0;

	DBUG_ENTER("test15");


	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	errno = NOERROR;
	rval = ftw(longname, f15, 2);
	err = errno;

	if(rval != -1 || err != ENAMETOOLONG)
	{
		xx_rpt(FAILURE);

		if (rval != -1)
		{
			in_rpt("ftw(<%d char filename>, ..., 2) returned %d instead of -1",
				strlen(longname), rval);
		}
		else if (err != ENAMETOOLONG)
		{
			in_rpt("ftw(<%d char filename>, ..., 2) set errno to %d (%s) instead of ENAMETOOLONG",
				strlen(longname), err, errname(err));
		}
	}
	else
		PATH_XS_RPT(1);

	clntoolng();

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f15(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	DBUG_ENTER("f15");

	DBUG_RETURN(1);

}

#else /* TEST_XNFS */

private void
test15()
{
	DBUG_ENTER("test15");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test16()
{
	char	*longname;
	int	err, rval;
	int	pathok = 0;

	DBUG_ENTER("test16");


	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	errno = NOERROR;
	rval = ftw(longname, f16, 1);
	err = errno;

	if(rval == -1 && err == ENAMETOOLONG)
	{
		xx_rpt(FAILURE);
		in_rpt("ftw() failed with errno ENAMETOOLONG");
	}
	else
		PATH_XS_RPT(1);

	clntoolng();

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f16(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	DBUG_ENTER("f16");

	DBUG_RETURN(1);

}

#else /* TEST_XNFS */

private void
test16()
{
	DBUG_ENTER("test16");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test17()
{
	char	*longname;
	int	err, rval;
	int	pathok = 0;

	DBUG_ENTER("test17");


	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	errno = NOERROR;
	rval = ftw(longname, f17, 1);
	err = errno;

	if(rval == -1 && err == ENAMETOOLONG)
	{
		xx_rpt(FAILURE);
		in_rpt("ftw() failed with errno ENAMETOOLONG");
	}
	else
		PATH_XS_RPT(1);

	clntoolng();

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f17(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	DBUG_ENTER("f17");

	DBUG_RETURN(1);

}

#else /* TEST_XNFS */

private void
test17()
{
	DBUG_ENTER("test17");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

