/*
*      SCCS:  @(#)  LSB.os/genuts/ftw_L/ftw_L.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)LSB.os/genuts/ftw_L - T.ftw_L Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.ftw_L

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the ftw() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Tue Feb 20 11:02:51 GMT 2001
MODIFICATIONS:	

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/ipc.h>
#include	<sys/stat.h>
#include	<pathres.h>
#include	<errno.h>
#include	<ftw.h>
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ftw
#endif

#define MODEANY         (mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define		NMR_FILES	7
#define		FTWF		01	/* bits in flag set by func4() */
#define		FTWD		02
#define		FTWDNR		04
#define		FTWNS	       010
#define		FTWSL	       020


#define	NO_TESTS	8	/* number of tests in testset */

extern	char	*errname();
extern	char	**pathtoolng();
extern	int	cr_sym_loop();
extern	int	n_open_fds();

int	no_tests = NO_TESTS;


private	int	jumpok = 0;
private	int	setupok = 0;
private int	open_files = 0;
private int	errflag = 0;
private int	flag = 0;

struct	ft
{
	char	*name;
	ino_t	inode;
	dev_t	device;
	int	depth;
	int	flag;
};

private struct	ft	ft[NMR_FILES] =
{
	"./ftw_d.2/ftw_t.4",	0,	0,	3,	TRUE,
	"./ftw_d.2",		0,	0,	2,	TRUE,
	"./ftw_d.1/ftw_t.3",	0,	0,	3,	TRUE,
	"./ftw_d.1/ftw_t.2",	0,	0,	3,	TRUE,
	"./ftw_d.1",		0,	0,	2,	TRUE,
	"./ftw_t.1",		0,	0,	2,	TRUE,
	".",			0,	0,	1,	TRUE
};

private char	*t4_file = "ftw-t.4";
private char	*t6_dir  =      "ftw-t.6";
private char	*t6_file  =     "ftw-t.6/ftw-t.6";
private char	*t7_dir  =      "ftw-t.7";
private char	*t7_file  =     "ftw-t.7/ftw-t.7";
private char	*t8_file1  =    "ftw-t.8a";
private char	*t8_file2  =    "ftw-t.8b";
private	void	prepare_tests();
private	void	cln_tests();
private	void	test1();
private	int	func1();
private	void	test2();
private	void	test3();
private int	func3();
private	void	test4();
private int	func4();
private	void	test5();
private int	func5();
private	void	test6();
private	void	test7();
private	void	test8();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 1,
	 test3, 1,
	 test4, 1,
	 test5, 1,
	 test6, 1,
	 test7, 1,
	 test8, 1,
	NULL, 0
};

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) unlink("./ftw_d.2/ftw_t.4");
	(void) rmdir("./ftw_d.2");
	(void) unlink("./ftw_d.1/ftw_t.3");
	(void) unlink("./ftw_d.1/ftw_t.2");
	(void) rmdir("./ftw_d.1");
	(void) unlink("./ftw_t.1");

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	int	fd;
	int	i;
	struct	stat	statb;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	if ((fd = creat("./ftw_t.1", MODEANY)) == SYSERROR)
	{
		DBUG_PRINTF("creat", "ftw_t.1");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}

	if ( crdir("./ftw_d.1", MODEANY, (int) getuid(), (int) getgid())
		== SYSERROR )
	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "failed to create directory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( (fd = creat("./ftw_d.1/ftw_t.2", MODEANY)) == SYSERROR )
	{
		DBUG_PRINTF("creat", "ftw_t.2");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}

	if ( (fd = creat("./ftw_d.1/ftw_t.3", MODEANY)) == SYSERROR )
	{
		DBUG_PRINTF("creat", "ftw_t.3");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}

	if ( crdir("./ftw_d.2", MODEANY, (int) getuid(), (int) getgid() )
		== SYSERROR )
	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "failed to create directory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( (fd = creat("./ftw_d.2/ftw_t.4", MODEANY)) == SYSERROR )
	{
		DBUG_PRINTF("creat", "ftw_t.4");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}


	for ( i = 0 ; i < NMR_FILES ; i++ )
	{
		if (stat(ft[i].name, &statb) == SYSERROR)
		{
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				(void) tet_delete(i, "stat call failed");
			DBUG_VOID_RETURN;
		}
		else
		{
			PATH_TRACE;
			ft[i].inode = statb.st_ino;
			ft[i].device = statb.st_dev;
		}
	}

	setupok = (pathok == NMR_FILES+6) ? 0 : -1;

	DBUG_VOID_RETURN;
}

private void
test1()
{
        /* pathname resolution */

        char *p1, *p2;
	int err1, err2;
        int npairs, i;
	int testfail;
        struct stat stbuf1, stbuf2;
        int     pathok = 0;
        int  ret1, ret2;

        DBUG_ENTER("test1");

        testfail = 0;

        npairs = init_pathres(PRES_DIRS|PRES_CREAT|PRES_NOROOT|PRES_NOTRAIL);
        if (npairs <= 0)
	{
 		xx_rpt(DELETION);
                in_rpt("init_pathres(PRES_DIRS|PRES_CREA|PRES_NOROOT|PRES_NOTRAILT) failed");
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

        for (i = 0; i < npairs; i++)
        {
                p1 = pathresnam[2*i];
                p2 = pathresnam[2*i+1];

                DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		errno = 0;
                ret1 = ftw(p1, func1, 3);
		err1 = errno;
                PATH_TRACE;

		errno = 0;
                ret2 = ftw(p2, func1, 3);
		err2 = errno;
                PATH_TRACE;

                if (ret1 == -1 || ret2 == -1)
                {
			testfail++;
			xx_rpt(FAILURE);
			if (ret1 == -1)
			{
				in_rpt("ftw(%s, function, 3) failed with errno %d(%s)", p1, err1, errname(err1));
			}
			if (ret2 == -1)
			{
				in_rpt("ftw(%s, function, 3) failed with errno %d(%s)", p2, err2, errname(err2));
			}
                }
		PATH_TRACE;
	}

 	if (testfail == 0)
        {
                PATH_XS_RPT(1 + (npairs * 3));
        }

        cln_pathres();

        DBUG_VOID_RETURN;
}

private int
func1(path, statptr, type)
char	 *path;
struct stat *statptr;
int	type;
{
	DBUG_ENTER("func1");

	DBUG_RETURN(0);
}
	
private void
test2()
{
        /* portable filenames */

        char    **p;
	int	id;
	int	err1;
        int     nfile, i, j, fd;
	int	testfail;
        int     pathok = 0;
        int     iter = 0;

        DBUG_ENTER("test2");

        testfail = 0;
        iter = 1;                       /* PATH_TRACE 1st iteration only */

	for (nfile = 0, p = portfnam; *p != NULL; p++, nfile++)
        {
                if ((fd = crdir(*p, MODEANY,
                                (int)geteuid(), (int)getegid())) == SYSERROR)
                {
                        xx_rpt(DELETION);
                        in_rpt("could not create file \"%s\"", *p);
                        DBUG_VOID_RETURN;
                }
                else if (iter)
                {
                        PATH_TRACE;
                        iter = 0;               /* Only trace the forst iteratio
n */
                }
                (void) close(fd);
        }

	for (i = 0, p = portfnam; i < nfile; p++, i++)
        {
                if (ftw(*p, func1, 3) == -1)
                {
			testfail++;
			err1 = errno;
			xx_rpt(FAILURE);
			in_rpt("ftw(%s, func1, 3) failed with errno %d(%s)", *p, err1, errname(err1));
                }
                PATH_TRACE;
        }

        if (testfail == 0)
        {
                PATH_XS_RPT(nfile + 1);
        }

        for (p = portfnam; *p != NULL; p++)
                (void) rmdir(*p);

        DBUG_VOID_RETURN;
}

private void
test3()
{
	int	nfails = 0;
	int	rtval;
	int	i;
	int	pathok = setupok;

	DBUG_ENTER("test3");

	jumpok = 0;
	open_files = n_open_fds();

	if ( (rtval = ftw(".", func3, 3)) != 0 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 7)
		PATH_TRACE;

	for ( i = 0 ; i < NMR_FILES ; i++ )
	{
		if ( ft[i].flag == TRUE )
		{
			if ( nfails++ == 0 )
				xx_rpt(FAILURE);
			in_rpt("function set by ftw not called for %s",
					ft[i].name);
		}
		else
			PATH_TRACE;

		ft[i].flag = TRUE;	/* reset for later tests	*/
	}

	if ( nfails == 0 )
		PATH_XS_RPT(NMR_FILES+2);

	DBUG_VOID_RETURN;
}


/*ARGSUSED*/
private int
func3(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	int	i;
	int	current_opens;

	DBUG_ENTER("func3");

	current_opens = n_open_fds();
	JUMP_TRACE;

	for ( i = 0 ; i < NMR_FILES ; i++ )
	{
		if ( statptr == (struct stat *) NULL )
			continue;

		if ( statptr->st_dev == ft[i].device
		    && statptr->st_ino == ft[i].inode
		    && ft[i].depth >= (current_opens - open_files))
		{
			ft[i].flag = FALSE;
			DBUG_RETURN(0);
		}
	}

	DBUG_RETURN(0);
}

private void
test4()
{
	int	i, err;
	int	rtval;
	int	nfails = 0;
	int	pathok = setupok;

	DBUG_ENTER("test4");

	if ( symlink("./ftw_t.1", "./ftw_s.1")
	    == SYSERROR )
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: symlink failed with errno %d (%s)",
				err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	flag = 0;
	jumpok = 0;
	if ( (rtval = ftw(".", func4, 3)) != 0 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 8)
		PATH_TRACE;

	if ( (flag & FTWSL) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_SL and");
		in_rpt("correct first and second arguments when traversing");
		in_rpt("file tree");
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(4);

	(void) unlink("./ftw_s.1");

	DBUG_VOID_RETURN;


}


/*ARGSUSED*/
private int
func4(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	struct	stat	statb;

	DBUG_ENTER("func4");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	switch (type)
	{

	case	FTW_SL:
		if ( stat(path, &statb) != SYSERROR
		    && statptr->st_dev == ft[5].device
		    && statptr->st_ino == ft[5].inode )
			flag |= FTWSL;
		break;

	default:
		break;
	}

	DBUG_RETURN(0);

}

private void
test5()
{
	int	i, err;
	int	rtval;
	int	nfails = 0;
	int	pathok = setupok;

	DBUG_ENTER("test5");

	if ( symlink("./ftw_z.1", "./ftw_s.1")
	    == SYSERROR )
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: symlink failed with errno %d (%s)",
				err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	flag = 0;
	jumpok = 0;
	if ( (rtval = ftw(".", func5, 3)) != 0 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 8)
		PATH_TRACE;

	if ( (flag & (FTWSL|FTWNS)) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_SL or FTW_NS and");
		in_rpt("correct first and second arguments when traversing");
		in_rpt("file tree");
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(4);

	(void) unlink("./ftw_s.1");

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
func5(path, statptr, type)
	char		*path;
	struct	stat	*statptr;
	int		type;
{
	struct	stat	statb;

	DBUG_ENTER("func5");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	switch (type)
	{

	case	FTW_SL:
		if ( stat(path, &statb) == SYSERROR )
			flag |= FTWSL;
		break;

	case	FTW_NS:
		if ( stat(path, &statb) == SYSERROR )
			flag |= FTWNS;
		break;

	default:
		break;
	}

	DBUG_RETURN(0);

}
private void
test6()
{
 	/* search permission denied */

	int	id;
	int	res;
	int	testfail;
        int     err;
        mode_t  mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
        int     pathok = 0;

        DBUG_ENTER("test6");

        testfail = 0;

        if (crdir(t6_dir, MODEANY,
                        (int)geteuid(), (int)getegid()) == SYSERROR)
        {
                xx_rpt(DELETION);
                in_rpt("could not create directory \"%s\"", t6_dir);
                DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t6_file, MODEANY,
                        (int) geteuid(), (int) getegid()) == SYSERROR)
        {
                xx_rpt(DELETION);
                in_rpt("could not create file \"%s\"", t6_file);
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

        if (chmod(t6_dir, mode) == SYSERROR)
        {
                err = errno;
                xx_rpt(DELETION);
                in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
                        t6_dir, (int)mode, err, errname(err));
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

	(void) umask(0);
	res = ftw(t6_file, func1, 3);

	if (res != -1 || errno != EACCES)
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("A call to ftw() with search permission denied did not fail");
		in_rpt("Expected Return: -1, errno EACCES");
		in_rpt("Actual Return: %d, errno %d(%s)", res, err, errname(err));
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

 	(void) chmod(t6_dir, MODEANY);
        (void) rmdir(t6_file);
        (void) rmdir(t6_dir);

	DBUG_VOID_RETURN;
}

private void
test7()
{
 	/* read permission denied */

	int	id;
	int	res;
	int	testfail;
        int     err;
        mode_t  mode = (S_IXUSR|S_IWUSR|S_IXGRP|S_IWGRP|S_IXOTH|S_IWOTH);
        int     pathok = 0;

        DBUG_ENTER("test7");

        testfail = 0;

        if (crdir(t7_dir, MODEANY,
                        (int)geteuid(), (int)getegid()) == SYSERROR)
        {
                xx_rpt(DELETION);
                in_rpt("could not create directory \"%s\"", t7_dir);
                DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t7_file, MODEANY,
                        (int) geteuid(), (int) getegid()) == SYSERROR)
        {
                xx_rpt(DELETION);
                in_rpt("could not create file \"%s\"", t7_file);
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

        if (chmod(t7_file, mode) == SYSERROR)
        {
                err = errno;
                xx_rpt(DELETION);
                in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
                        t7_file, (int)mode, err, errname(err));
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

	(void) umask(0);
	res = ftw(t7_file, func1, 3);

	if (res != -1 || errno != EACCES)
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("A call to ftw() with read permission denied did not fail");
		in_rpt("Expected Return: -1, errno EACCES");
		in_rpt("Actual Return: %d, errno %d(%s)", res, err, errname(err));
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

        (void) rmdir(t7_file);
        (void) rmdir(t7_dir);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	int	id;
	key_t	res;
	int	testfail;
        int     fd, err;
        int     pathok = 0;
	int	globok = 0;

        DBUG_ENTER("test8");

        testfail = 0;

        if (cr_sym_loop(t8_file1, t8_file2) == SYSERROR)
        {
                xx_rpt(DELETION);
                in_rpt("could not create loop of symlinks");
                DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	(void) umask(0);
	res = ftw(t8_file1, func1, 3);

	if (res != -1 || errno != ELOOP)
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("A call to ftw() with looped symbolic link did not fail");
		in_rpt("Expected Return: -1, errno ELOOP");
		in_rpt("Actual Return: %d, errno %d(%s)", res, err, errname(err));
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

        (void) unlink(t8_file1);
        (void) unlink(t8_file2);

	DBUG_VOID_RETURN;
}
