/*
*      SCCS:  @(#)  XPG4.os/genuts/getopt/getopt.c	Rel 4.4.4	    (07/19/99)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XPG4.os/genuts/getopt - T.end Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.getopt

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the getopt() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Neil Moses, UniSoft Ltd.
DATE CREATED:	Wed Sep 25 16:19:08 BST 1991
MODIFICATIONS:	
		Geoff Clare, 19 June 1992
			Change expected return from ':' to '?' in "-:" test.

		Geoff Clare, 22 June 1992
			Change DIAGNOSTIC to NO_OUTPUT in "-:" test.

		Geoff Clare, 14 Aug 1992
			FIP result if diagnostic does not match expected
			string.  Avoid passing null pointer to strcmp().

		Geoff Clare, 6 Nov 1992
			Correct spelling of "diagnostic".

		Geoff Clare, 10 Feb 1995
			Change expected optind in cases where last entry
			in targv[] is an option with a missing argument.

		Geoff Clare, 8 Sept 1995
			Avoid freopen() of closed stream in various tests.

		Geoff Clare, 9 June 1999
			Added some entries to tval[] where there were
			gaps in the coverage of test 1.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/times.h>
#include	<sys/stat.h>
#include 	<locale.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<string.h>
#include	<fcntl.h>
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getopt
#endif


#define 	NO_TESTS	4		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)
#define		MAXARGSZ	10	/* max size of targv array needed */
#define		FILE1		"./getopt_t.1"	
#define		NO_CHECK	(&no_check_dummy)

extern	char	*errname();
extern	int	cppair();

int	no_tests = NO_TESTS;

private	char 	*targv[MAXARGSZ];	/* used to check getopt */
private char	no_check_dummy;
private	int 	targc;
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	ch_t1();
private	void	ch_t3();
private	void	ch_t4();
private	void  	cln_getopt();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cln_getopt;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

#define	NO_OUTPUT 	0
#define	DIAGNOSTIC	1

private	struct	{
	char 	optstr[50];
	char 	*args[20];
	int	optind_in;
	int	optind_out;
	char	*optarg;
	int	ret;
	int	stder;
	int	optopt;
} tval[] = {
	{"", {"--","fred",0}, 1, 2, NO_CHECK, -1, NO_OUTPUT, 0},
	{"", {"-a", "fred",0}, 1, 2, NO_CHECK, '?', DIAGNOSTIC, 'a'},
	{"", {"-a", "fred",0}, 2, 2, NO_CHECK, -1, NO_OUTPUT, 0},
	{":", {"--", "fred",0}, 1, 2, NO_CHECK, -1, NO_OUTPUT, 0},
	{":", {"-a", "fred",0}, 1, 2, NO_CHECK, '?', NO_OUTPUT, 'a'},
	{":", {"-a", "fred",0}, 2, 2, NO_CHECK, -1, NO_OUTPUT, 0},
	{"abcd", {"-a", "-c", "fred",0}, 1, 2, NO_CHECK, 'a', NO_OUTPUT, 0},
	{"abcd", {"-a", "-c", "fred",0}, 2, 3, NO_CHECK, 'c', NO_OUTPUT, 0},
	{"abcd", {"-a", "-c", "fred",0}, 3, 3, NO_CHECK, -1, NO_OUTPUT, 0},
	{"abcd", {"-ac", "fred",0}, 1, 1, NO_CHECK, 'a', NO_OUTPUT, 0},
	{"abcd", {"-ac", "fred",0}, 0, 2, NO_CHECK, 'c', NO_OUTPUT, 0},
	{"abcd", {"-ac", "fred",0}, 2, 2, NO_CHECK, -1, NO_OUTPUT, 0},
	{"abcd", {"-z", "fred",0}, 1, 2, NO_CHECK, '?', DIAGNOSTIC, 'z'},
	{"abcd", {"-a", "--", "-b",0}, 1, 2, NO_CHECK, 'a', NO_OUTPUT, 0},
	{"abcd", {"-a", "--", "-b",0}, 2, 3, NO_CHECK, -1, NO_OUTPUT, 0},
	{":abcd", {"-a", "-c", "fred",0}, 1, 2, NO_CHECK, 'a', NO_OUTPUT, 0},
	{":abcd", {"-a", "-c", "fred",0}, 2, 3, NO_CHECK, 'c', NO_OUTPUT, 0},
	{":abcd", {"-a", "-c", "fred",0}, 3, 3, NO_CHECK, -1, NO_OUTPUT, 0},
	{":abcd", {"-z", "fred",0}, 1, 2, NO_CHECK, '?', NO_OUTPUT, 'z'},
	{"ab:cd:e",{"-a","-d","opt","fred",0}, 1, 2, NO_CHECK, 'a', NO_OUTPUT, 0},
	{"ab:cd:e",{"-a","-d","opt","fred",0},2, 4,"opt", 'd', NO_OUTPUT,0},
	{"ab:cd:e",{"-a","-d","opt",0},2, 4,"opt", 'd', NO_OUTPUT,0},
	{"ab:cd:e",{"-a","-dopt","fred",0}, 2, 3, "opt", 'd', NO_OUTPUT, 0},
	{"ab:cd:e",{"-a","-dopt",0}, 2, 3, "opt", 'd', NO_OUTPUT, 0},
	{"ab:cd:e", {"-a","-d","opt","fred",0}, 4, 4, NO_CHECK, -1, NO_OUTPUT, 0},
	{"ab:cd:e", {"-a","-d",0}, 2, 4, NO_CHECK, '?', DIAGNOSTIC, 'd'},
	{"ab:cd:e", {"-d","-a","fred",0}, 1, 3, "-a", 'd', NO_OUTPUT, 0},
	{"ab:cd:e", {"-d-a","fred",0}, 1, 2, "-a", 'd', NO_OUTPUT, 0},
	{"ab:cd:e",{"-a","opt","-d","opt2",0}, 1, 2, NO_CHECK, 'a', NO_OUTPUT, 0},
	{"ab:cd:e",{"-a","opt","-d","opt2",0}, 2, 2, NO_CHECK, -1, NO_OUTPUT, 0},
	{"ab:cd:e", {"-abd","fred",0}, 1, 1, NO_CHECK, 'a', NO_OUTPUT, 0},
	{"ab:cd:e", {"-abd","fred",0}, 0, 2, "d", 'b', NO_OUTPUT, 0},
	{"ab:cd:e", {"-abd",0}, 1, 1, NO_CHECK, 'a', NO_OUTPUT, 0},
	{"ab:cd:e", {"-abd",0}, 0, 2, "d", 'b', NO_OUTPUT, 0},
	{"ab:cd:e", {"-ab","opt","fred",0}, 1, 1, NO_CHECK, 'a', NO_OUTPUT, 0},
	{"ab:cd:e", {"-ab","opt","fred",0}, 0, 3, "opt", 'b', NO_OUTPUT, 0},
	{"ab:cd:e", {"-ab","opt",0}, 1, 1, NO_CHECK, 'a', NO_OUTPUT, 0},
	{"ab:cd:e", {"-ab","opt",0}, 0, 3, "opt", 'b', NO_OUTPUT, 0},
	{"ab:cd:e", {"-ab",0}, 1, 1, NO_CHECK, 'a', NO_OUTPUT, 0},
	{"ab:cd:e", {"-ab",0}, 0, 3, NO_CHECK, '?', DIAGNOSTIC, 'b'},
	{"ab:cd:e",{"-b","opt","--","fred",0},1, 3, "opt", 'b',NO_OUTPUT,0},
	{"ab:cd:e",{"-b","opt","--","fred",0}, 3, 4, NO_CHECK, -1, NO_OUTPUT, 0},
	{"ab:cd:e",{"-b","--","-c","fred",0}, 1, 3, "--", 'b', NO_OUTPUT,0},
	{"ab:cd:e",{"-b","--","-c","fred",0}, 3, 4, NO_CHECK, 'c', NO_OUTPUT,0},
	{"ab:cd:e",{"-b","","-c","fred",0}, 1, 3, "", 'b', NO_OUTPUT, 0},
	{"ab:cd:e",{"-b","","-c","fred",0}, 3, 4, NO_CHECK, 'c', NO_OUTPUT, 0},
	{"ab:cd:e",{"-b","o1 o2","fred",0},1, 3,"o1 o2", 'b',NO_OUTPUT,0},
	{"ab:cd:e",{"-bo1 o2","fred",0}, 1, 2, "o1 o2", 'b', NO_OUTPUT,0},
	{"ab:cd:e",{"-b o1 o2","fred",0},1, 2, " o1 o2", 'b',NO_OUTPUT,0},
	{":ab:cd:e",{"-a","-d","opt","fred",0},2, 4, "opt", 'd',NO_OUTPUT,0},
	{":ab:cd:e",{"-a","-d","opt",0},2, 4, "opt", 'd',NO_OUTPUT,0},
	{":ab:cd:e",{"-a","-d","opt","fred",0}, 4, 4, NO_CHECK, -1, NO_OUTPUT, 0},
	{":ab:cd:e",{"-a","-d",0}, 2, 4, NULL, ':', NO_OUTPUT, 'd'},
	{":ab:cd:e",{"-ab",0}, 1, 1, NO_CHECK, 'a', NO_OUTPUT, 0},
	{":ab:cd:e",{"-ab",0}, 0, 3, NULL, ':', NO_OUTPUT, 'b'},
	{"ab:", {"-:","-a","fred",0}, 1, 2, NO_CHECK, '?', DIAGNOSTIC, ':'},
	{":ab:",{"-:","-a","fred",0}, 1, 2, NO_CHECK, '?', NO_OUTPUT, ':'}
};

private char argstr[100];
private	char *errfile = "./errfile";
private	int  testfail;

private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	FILE	*fp;
	struct 	stat	buf;
	char	mess[BUFSIZ];
	int	i, j, errmess;
	int	rval, rescode, rpt_optind;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	mess[0] = '\0';
	testfail = 0;

	for(i = 0; i < sizeof(tval)/sizeof(*tval); i++)
	{
		if(i == 0)
			PATH_TRACE;

		/*
	 	 * Re-open stderr
	 	 */
		if((fp = freopen(errfile, "w", stderr)) == NULL)
		{
			in_rpt("Cannot re-open stderr to %s", errfile);
			xx_rpt(DELETION);
			DBUG_VOID_RETURN;
		}

		targv[0] = "pname";

		argstr[0] = '\0';
		for(j = 0; tval[i].args[j] != 0; j++)
		{
			targv[j + 1] = tval[i].args[j];

			(void) strcat(argstr, tval[i].args[j]);
			if (tval[i].args[j+1] != 0)
				(void) strcat(argstr, "] [");
		}

		targv[j + 1] = (char *) NULL;

		targc = j + 1;

		if(tval[i].optind_in != 0)
			optind = tval[i].optind_in;
		rpt_optind = optind;

		optopt = tval[i].optopt;

		rval = getopt(targc, targv, tval[i].optstr);

		/*
	 	 * Check if output written to stderr
		 */
		(void) fflush(fp);
		if(stat(errfile, &buf) != 0)
		{
			in_rpt("Cannot stat() file %s", errfile);
			xx_rpt(DELETION);
			DBUG_VOID_RETURN;
		}

		if(buf.st_size > 0)
			errmess = DIAGNOSTIC;
		else
			errmess = NO_OUTPUT;
		
		rescode = 0;
		if (optind != tval[i].optind_out)
		{
			if (tval[i].optind_out > targc && optind == targc)
				rescode = WARNING;
			else
				rescode = FAILURE;
		}

		if (errmess != tval[i].stder
		|| (tval[i].optarg!=NO_CHECK &&
		     ((tval[i].optarg==NULL && optarg!=NULL) ||
		      (tval[i].optarg!=NULL && (optarg==NULL ||
			 strcmp(optarg,tval[i].optarg)!=0))))
		|| rval != tval[i].ret
		|| (optopt != tval[i].optopt && tval[i].optopt != 0))
		{
			rescode = FAILURE;
		}

		if (rescode != 0)
		{
			xx_rpt(rescode);
			in_rpt("getopt() failed when called with :-");
			in_rpt("        optstring : [%s]", tval[i].optstr);
			in_rpt("        arguments : [%s]", argstr);
			in_rpt("        optind : [%d]", rpt_optind);

			in_rpt("getopt() failed on the following :-");

			if(errmess != tval[i].stder)
			{
				if(tval[i].stder == NO_OUTPUT)
				{
					in_rpt("%s %s", 
				"        Expected no output to be written to",
					"standard error");

					/* don't fclose(fp); */
					if((fp = fopen(errfile, "r")) == NULL)
					{
	     					in_rpt(
					"Unable to open %s to read", errfile);
						xx_rpt(DELETION);
						DBUG_VOID_RETURN;
					}
		
					(void) fgets(mess, BUFSIZ, fp);
					(void) fclose(fp);

					in_rpt(
					"        Diagnostic output : %s", mess);
				}	
				else
					in_rpt("%s %s", 
				"        Expected diagnostic to be written to",
					"standard error");
			}

			if(optind != tval[i].optind_out)
				in_rpt(
				"        optind : expected [%d], actual [%d]",
					tval[i].optind_out, optind);

			if (tval[i].optarg!=NO_CHECK &&
			     ((tval[i].optarg==NULL && optarg!=NULL) ||
			      (tval[i].optarg!=NULL && (optarg==NULL ||
				 strcmp(optarg,tval[i].optarg)!=0))))
				in_rpt(
			"        optarg : expected [%s], actual [%s]",
					tval[i].optarg == 0 ? "(null)" :
						tval[i].optarg,
					optarg == 0 ? "(null)" : optarg);

			if(rval != tval[i].ret)
			{
				if(rval == -1)
					in_rpt(
			  "        return value : expected [%c], actual [-1]",
						tval[i].ret);
				else if(tval[i].ret == -1)
					in_rpt(
			  "        return value : expected [-1], actual [%c]",
						rval);
				else
					in_rpt(
			  "        return value : expected [%c], actual [%c]",
						tval[i].ret, rval);
			}

			if(optopt != tval[i].optopt && tval[i].optopt != 0)
				in_rpt(
				"        optopt : expected [%c], actual [%c]",
				tval[i].optopt, optopt);

			in_rpt(" ");

			testfail++;
		}
	}

	if(!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	pathok = 0;

	DBUG_ENTER("test2");

	if(optind != 1)
	{
		xx_rpt(FAILURE);
		in_rpt(
	"getopt() : Value of optind incorrect before first call to getopt()");
		in_rpt("Expected value : [1]");
		in_rpt("Actual   value : [%d]", optind);
	}
	else
		PATH_XS_RPT(0);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t3()
{
	FILE	*fp;
	char	mess[BUFSIZ];
	char	expstr[BUFSIZ];
	struct 	stat	buf;
	int	rval;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	mess[0] = '\0';
	expstr[0] = '\0';

	/*
	 * Re-open stderr
	 */
	if((fp = freopen(errfile, "w", stderr)) == NULL)
	{
		in_rpt("Cannot re-open stderr to %s", errfile);
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	targv[0] = "pname";
	targv[1] = "-a";
	targv[2] = "fred";
	targv[3] = (char *) NULL;
	targc = 3;

	/*
	 * First show that diagnostics are printed to standard error
	 */
	(void) getopt(targc, targv, "");

	/*
	 * Check if output written to stderr
	 */
	(void) fflush(fp);
	if(stat(errfile, &buf) != 0)
	{
		in_rpt("Cannot stat() file %s", errfile);
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(buf.st_size == 0)
	{
		in_rpt(
		"getopt() failed to print diagnostics to standard error");
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Re-open stderr
	 */
	if((fp = freopen(errfile, "w", stderr)) == NULL)
	{
		in_rpt("Cannot re-open stderr to %s", errfile);
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Now try with opterr set to zero
	 */
	optind = 1;
	opterr = 0;
	rval = getopt(targc, targv, "");

	/*
	 * Check if output written to stderr
	 */
	(void) fflush(fp);
	if(stat(errfile, &buf) != 0)
	{
		in_rpt("Cannot stat() file %s", errfile);
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(buf.st_size > 0)
	{
		xx_rpt(FAILURE);

		/* don't fclose(fp); */
		if((fp = fopen(errfile, "r")) == NULL)
		{
	     		in_rpt("Unable to open %s to read", errfile);
			xx_rpt(DELETION);
			DBUG_VOID_RETURN;
		}
		
		(void) fgets(mess, BUFSIZ, fp);
		(void) fclose(fp);

		in_rpt(
	"getopt() : Unexpected output to stderr when opterr set to zero :-");
		in_rpt("     %s", mess);
	}
	else
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	(void) cppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t4()
{
	FILE	*fp;
	struct 	stat	buf;
	char	mess[BUFSIZ];
	char	expstr[BUFSIZ];
	int	rval;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	/*
	 * Set POSIX locale
	 */
	if (setlocale(LC_ALL, "POSIX") == (char *) NULL)
	{
		in_rpt("setlocale(LC_ALL, \"POSIX\") failed");
		xx_rpt(DELETION);

		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((fp = freopen(errfile, "w", stderr)) == NULL)
	{
		in_rpt("Cannot re-open stderr to %s", errfile);
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	mess[0] = '\0';
	expstr[0] = '\0';
	targv[0] = "pname";
	targv[1] = "-a";
	targv[2] = "fred";
	targv[3] = (char *) NULL;
	targc = 3;

	rval = getopt(targc, targv, "");

	/*
	 * Check if output written to stderr
	 */
	(void) fflush(fp);
	if(stat(errfile, &buf) != 0)
	{
		in_rpt("Cannot stat() file %s", errfile);
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(buf.st_size == 0)
	{
	     	in_rpt(
		"getopt() failed to print diagnostics to standard error");
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Check format of diagnostic
	 */
	/* don't fclose(fp); */
	if((fp = fopen(errfile, "r")) == NULL)
	{
	     	in_rpt("Unable to open %s to read", errfile);
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) fgets(mess, BUFSIZ, fp);
	(void) fclose(fp);

	(void) sprintf(expstr, "%s: illegal option -- %c\n", targv[0], targv[1][1]);

	if(strcmp(mess, expstr) != 0)
	{
		xx_rpt(RFI);
		in_rpt("getopt() : Possibly incorrect diagnostic output to standard error:-");
		in_rpt("Expected diagnostic output :-");
		in_rpt("     %s", expstr);
		in_rpt("Actual   diagnostic output :-");
		in_rpt("     %s", mess);
	}
	else
		PATH_TRACE;

	if((fp = freopen(errfile, "w", stderr)) == NULL)
	{
		in_rpt("Cannot re-open stderr to %s", errfile);
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	optind = 1;
	mess[0] = '\0';
	expstr[0] = '\0';
	targv[0] = "pname";
	targv[1] = "-a";
	targv[2] = (char *) NULL;
	targc = 2;

	rval = getopt(targc, targv, "a:");

	/*
	 * Check if output written to stderr
	 */
	(void) fflush(fp);
	if(stat(errfile, &buf) != 0)
	{
		in_rpt("Cannot stat() file %s", errfile);
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(buf.st_size == 0)
	{
		in_rpt(
		"getopt() failed to print diagnostics to standard error");
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Check format of diagnostic
	 */
	/* don't fclose(fp); */
	if((fp = fopen(errfile, "r")) == NULL)
	{
	     	in_rpt("Unable to open %s to read", errfile);
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) fgets(mess, BUFSIZ, fp);
	(void) fclose(fp);

	(void) sprintf(expstr, "%s: option requires an argument -- %c\n", 
		targv[0], targv[1][1]);

	if(strcmp(mess, expstr) != 0)
	{
		xx_rpt(RFI);
		in_rpt("getopt() : Possibly incorrect diagnostic output to standard error:-");
		in_rpt("Expected diagnostic output :-");
		in_rpt("     %s", expstr);
		in_rpt("Actual   diagnostic output :-");
		in_rpt("     %s", mess);
	}
	else
		PATH_XS_RPT(10);

	DBUG_VOID_RETURN;
}

private void cln_getopt()
{
	(void) unlink(errfile);
}
