/*
*      SCCS:  @(#)  XPG4.os/genuts/glob/glob.c	Rel 4.4.1   (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XPG4.os/genuts/glob - T.glob Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.glob

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the glob() subroutine.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		M Young, UniSoft Ltd.
DATE CREATED:	Wed Sep 04 15:52:35 WET 1991
MODIFICATIONS:
		Geoff Clare, 2 Apr 1992
			Change test 9 so as not to use / within []

		Neil Moses, 7 May 1992.
			Check that interface is supported.
			Added TEST For ENOSYS if interface not supported.
			Removed "[ with no ]" test.

		Geoff Clare, 29 May 1992
			Don't use literal language names.

		Neil Moses, 3 June 1992.
			Use sysconf() to check if interface supported.

		Geoff Clare, 19 June 1992
			Made reporting of gl_pathc more clear.
			Changed "a" to "x" in test_pats in test 3.
			Use pr_nchars() to report pathnames.

		Geoff Clare, 22 Dec 1992
			Align with interpretation VWG/025/081892.

		Geoff Clare, 28 May 1993
			Initialize glob_b to empty_glob in test 20.

		Geoff Clare, 9 Feb 1994
			Allow trailing slash in pathname passed to errfunc.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<stdlib.h>
#include	<string.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<glob.h>
#include	<tet_api.h>
#include	<limits.h>
#include	<string.h>
#include        <locale.h>
#include        <nlstest.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef glob
#undef globfree
#endif


#define		NO_TESTS	31	/* number of tests in testset */
#define		COMP_TESTS	28	/* number of compliance tests */

#define         WAITTIME        (5*SPEEDFACTOR)

#define		MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define		MODENOSRCH	(S_IRUSR|S_IRGRP|S_IROTH)
#define		MODENOREAD	(S_IXUSR|S_IXGRP|S_IXOTH)
#define		NULLFP		((int (*)())0)

#define	N_EXPECTED(x)	(sizeof(x) / sizeof(char *) - 1)

int	no_tests = NO_TESTS;

extern	char	*errname();
extern	char	*pr_nchars();
extern	void	xx_all();

private	int	setupok = -1;
private	int	p_nfails = 0;
private	int	tsup = 0;

private	glob_t	empty_glob;	/* empty struct to clear automatics */
private	char un_matchable[] = "%% un-matchable expression %%";
private	char *test_vecp;

private char dir_001[] = "dir_1";
private char dir_002[] = "dir_1/readable";
private char dir_003[] = "dir_1/Xreadable";
private char dir_004[] = "dir_1/unreadable";
private char dir_005[] = "dir_2";
private char dir_006[] = "dir_2/unreadable";
private char dir_007[] = "dir_2/readable";
private char dir_008[] = "dir_2/Xreadable";
private char dir_009[] = "dir_1/unsearch";

private char file_001[] = "dir_1/a";
private char file_002[] = "dir_1/readable/a";
private char file_003[] = "dir_1/readable/b";
private char file_004[] = "dir_1/unreadable/a";
private char file_005[] = "dir_1/Xreadable/a";
private char file_006[] = "a";
private char file_007[] = "b";
private char file_008[] = "c";
private char file_009[] = "d";
private char file_010[] = "aXb";
private char file_011[] = "aYb";
private char file_012[] = "aZb";
private char file_013[] = "a[b";
private char file_014[] = "a]b";
private char file_015[] = "a-b";
private char file_016[] = "a?b";
private char file_017[] = "a*b";
private char file_018[] = ".hidden";
private char file_019[] = "Xhidden";
private char file_020[] = "nothidden";
private char file_021[] = "dir_1/nothidden";
private char file_022[] = "dir_1/.hidden";
private char file_023[] = "axb";
private char file_024[] = "ayb";
private char file_025[] = "azb";
private char file_026[] = "dir_2/readable/a";
private char file_027[] = "dir_2/Xreadable/a";
private char file_028[] = "dir_2/a?b";
private char file_029[] = "dir_2/a\\b";
private char file_030[] = "dir_2/arb";
private char file_031[] = "dir_2/.rb";
private char file_032[] = "dir_2Xarb";

struct	ft
{
	char	*name;
	mode_t	perm;
};

private struct	ft	glob_dirs[] =
{
	dir_001,	MODEANY,
	dir_002,	MODEANY,
	dir_003,	MODEANY,
	dir_004,	MODENOREAD,
	dir_005,	MODEANY,
	dir_006,	MODENOREAD,
	dir_007,	MODEANY,
	dir_008,	MODEANY,
	dir_009,	MODENOSRCH
};

private struct	ft	glob_files[] =
{
	file_001,	MODEANY,
	file_002,	MODEANY,
	file_003,	MODEANY,
	file_004,	MODEANY,
	file_005,	MODEANY,
	file_006,	MODEANY,
	file_007,	MODEANY,
	file_008,	MODEANY,
	file_009,	MODEANY,
	file_010,	MODEANY,
	file_011,	MODEANY,
	file_012,	MODEANY,
	file_013,	MODEANY,
	file_014,	MODEANY,
	file_015,	MODEANY,
	file_016,	MODEANY,
	file_017,	MODEANY,
	file_018,	MODEANY,
	file_019,	MODEANY,
	file_020,	MODEANY,
	file_021,	MODEANY,
	file_022,	MODEANY,
	file_023,	MODEANY,
	file_024,	MODEANY,
	file_025,	MODEANY,
	file_026,	MODEANY,
	file_027,	MODEANY,
	file_028,	MODEANY,
	file_029,	MODEANY,
	file_030,	MODEANY,
	file_031,	MODEANY,
	file_032,	MODEANY,
};

private void	prepare_tests();
private void	cln_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();
private void	test30();
private void	test31();
private int	f1();
private void	ch_t13();
private void	ch_t15();
private int	f27();
private int	f28();
private int	f30();

private int	errflag;
private int	n_dirs = sizeof(glob_dirs) / sizeof(struct ft);
private int	n_files = sizeof(glob_files) / sizeof(struct ft);

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;

public	struct tet_testlist tet_testlist[] = {
	test1,	 1,
	test2,	 2,
	test3,	 3,
	test4,	 4,
	test5,	 5,
	test6,	 6,
	test7,	 7,
	test8,	 8,
	test9,	 9,
	test10, 10,
	test11,	11,
	test12,	12,
	test13,	13,
	test14,	14,
	test15,	15,
	test16,	16,
	test17,	17,
	test18,	18,
	test19,	19,
	test20, 20,
	test21,	21,
	test22,	22,
	test23,	23,
	test24,	24,
	test25,	25,
	test26,	26,
	test27,	27,
	test28,	28,
	test29,	29,
	test30, 30,
	test31, 31,
	NULL,	 0
};

#define	CHK_OK		0x00		/* list matches expected results */
#define	CHK_NOLIST	0x01		/* list pointer is empty */
#define	CHK_MISSING	0x02		/* an entry is missing */
#define	CHK_UNORDER	0x04		/* list is out of order */
#define	CHK_ORDERED	0x08		/* check for ordered list */
#define	CHK_BADOFF	0x10		/* offs not null */

private char *
flagstr(flags)
int flags;
{
	static char buf[128];

	if (flags == 0)
		return "0";
	
	*buf = '\0';

	if (flags & GLOB_APPEND)
		(void) strcat(buf, " | GLOB_APPEND");

	if (flags & GLOB_DOOFFS)
		(void) strcat(buf, " | GLOB_DOOFFS");

	if (flags & GLOB_ERR)
		(void) strcat(buf, " | GLOB_ERR");

	if (flags & GLOB_MARK)
		(void) strcat(buf, " | GLOB_MARK");

	if (flags & GLOB_NOCHECK)
		(void) strcat(buf, " | GLOB_NOCHECK");

	if (flags & GLOB_NOESCAPE)
		(void) strcat(buf, " | GLOB_NOESCAPE");

	if (flags & GLOB_NOSORT)
		(void) strcat(buf, " | GLOB_NOSORT");

	return &buf[3];
}

private int
chk_sup(pattern, flags, errfunc, rtval, err)
char *pattern;
int flags;
int (*errfunc)();
int rtval;
int err;
{
	int pathok = 0;
	int nfails = 0;
	char *funcstr;
	static int firstsup = -1;

	DBUG_ENTER("chk_sup");

	/* If the POSIX2 CLB group is not supported, glob() can
	   still be supported on its own, but calls must consistently
	   indicate support or non-support. */

	if (errfunc == NULLFP)
		funcstr = "NULL";
	else
		funcstr = "errfunc";

	if (rtval == GLOB_NOSYS && err == ENOSYS)
	{
		if (tsup)
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", %s, %s, pglob) not correct:",
				pattern, flagstr(flags), funcstr);
			in_rpt("glob() gave ENOSYS when POSIX2 CLB group is supported");
		}
		else if (firstsup == -1)
		{
			firstsup = 0;
			PATH_TRACE;
		}
		else if (firstsup == 1)
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", %s, %s, pglob) not correct:",
				pattern, flagstr(flags), funcstr);
			in_rpt("glob() gave ENOSYS but a previous call did not");
		}
		else
			PATH_TRACE;
	}
	else
	{
		if (firstsup == -1)
		{
			firstsup = 1;
			PATH_TRACE;
		}
		else if (firstsup == 0)
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", %s, %s, pglob) not correct:",
				pattern, flagstr(flags), funcstr);
			in_rpt("glob() did not give ENOSYS but a previous call did");
		}
		else
			PATH_TRACE;
	}

	PATH_FUNC_RPT(1);
	DBUG_RETURN(nfails);
}

private	void
rpt_expected(pattern, list, gp, xc)
char	*pattern;
char	**list;
glob_t	*gp;
int	xc;
{
	int c, off;

	c = gp->gl_pathc;

	if ( xc >= 0 )
	{
		in_rpt("glob(\"%s\", ...) did not produce expected list of matched files",
			pattern);
		in_rpt("gl_pathc expected: %d, observed: %d", xc, gp->gl_pathc);
	}

	if ( c < 0 || c > 100 )
		in_rpt("unable to display matched list");
	else
	{
		in_rpt("expected matches:");
		for( off = gp->gl_offs; off > 0; off-- )
			in_rpt("\t<NULL>");
		for( ; *list; list++ )
			in_rpt("\t%s", pr_nchars(*list, (int)strlen(*list)));
		if( !(list = gp->gl_pathv) )
		{
			in_rpt("gl_pathv is <NULL>");
		}
		else
		{
			in_rpt("actual matches:");
			for( off = gp->gl_offs;  off-- > 0; list++ )
			{
				if ( *list )
					in_rpt("\t%s",
					    pr_nchars(*list, (int)strlen(*list)));
				else
					in_rpt("\t<NULL>");
			}
			for( ; *list && c-- > 0; list++ )
				in_rpt("\t%s", pr_nchars(*list, (int)strlen(*list)));
		}
	}
}

private	int
random_chk(list,gp)
char	**list;
glob_t *gp;
{
	char	**matched;
	int	i, ok;

	matched = gp->gl_pathv;

	if( ! matched )
		return(CHK_NOLIST);

	for( i = gp->gl_offs; i-- > 0; matched++ )
	{
		if ( *matched != (char *)NULL )
			return(CHK_BADOFF);
	}

	for( ; *list; list++ )
	{
		ok = 0;
		for( i = gp->gl_pathc, matched = gp->gl_pathv +gp->gl_offs;
					i-- > 0; matched++ )
		{
			if ( ! strcmp(*matched, *list) )
			{
				ok = 1;
				break;
			}
		}
		if( !ok)
			return(CHK_MISSING);
	}
	return(CHK_OK);
}

/*
 * These functions assume that the gl_offs field is set correctly, as
 * the glob_t buffer is always cleared this will be the case.
 */

private	int
ordered_chk(list,gp)
char	**list;
glob_t *gp;
{
	char	**matched, **save_list;

	matched = gp->gl_pathv;

	if( ! matched )
		return(CHK_NOLIST);

	matched += gp->gl_offs;
	save_list = list;

	for( ; *list; list++, matched++ )
	{
		if ( strcmp(*matched, *list) )
		{
			switch( random_chk( save_list, gp) )
			{
				case CHK_OK:
					return(CHK_UNORDER);
				case CHK_BADOFF:
					return(CHK_BADOFF);
				default:
					return(CHK_MISSING);
			}
		}
	}
	return(CHK_OK);
}

private int
return_chk(type, pattern, list, gp)
char	*pattern;
char	**list;
glob_t *gp;
{
	int	rc, nfails;
	int	pathok = 0;

	DBUG_ENTER("return_chk");

	nfails = 0;

	if( type & CHK_ORDERED )
		rc = ordered_chk(list,gp);
	else
		rc = random_chk(list,gp);

	switch ( rc ) {
		case CHK_OK:
			PATH_TRACE;
			break;
		case CHK_NOLIST:
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", ...) allocated no gl_pathv",
				pattern);
			nfails++;
			break;

		case CHK_MISSING:
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", ...) returned vector with expected match missing",
				pattern);
			nfails++;
			break;

		case CHK_BADOFF:
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", ...) returned vector with non-empty offset",
				pattern);
			nfails++;
			break;

		case CHK_UNORDER:
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", ...) returned incorrectly sorted vector",
				pattern);
			nfails++;
			break;

		default:
			xx_rpt(DELETION);
			in_rpt("deletion reason: compare function returned unexpected value - %d", rc);
			DBUG_RETURN(1);
	}

	if( nfails )
		rpt_expected(pattern, list, gp, -1);

	PATH_FUNC_RPT(1);

	DBUG_RETURN(nfails);
}

private int
do_matches(pattern, flags, type, list, gp)
char *pattern;
int flags;
int type;
char **list;
glob_t *gp;
{
	int	nfails = 0;
	int	rtval, err;
	int	n_expected;
	char	**listp;
	int	pathok = 0;

	DBUG_ENTER("do_matches");

	errno = 0;
	rtval = glob(pattern, flags, NULLFP, gp);
	err = errno;

	globok = 0;
	nfails += chk_sup(pattern, flags, NULLFP, rtval, err);
	PATH_FUNC_TRACE;

	if (rtval == GLOB_NOSYS && err == ENOSYS)
	{
		PATH_TRACE;
		PATH_FUNC_RPT(2);
		DBUG_RETURN(nfails);
	}
	else if (rtval != 0)
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("glob(\"%s\", %s, NULL, pglob) returned %d instead of 0",
			pattern, flagstr(flags), rtval);
	}
	else
		PATH_TRACE;
	    
	n_expected = 0;
	for (listp = list; *listp != NULL; listp++)
		n_expected++;

	if( gp->gl_pathc != n_expected )
	{
		nfails++;
		xx_rpt(FAILURE);
		rpt_expected(pattern, list, gp, n_expected);
	}
	else
	{
		globok = 0;
		nfails += return_chk(type, pattern, list, gp);
		PATH_FUNC_TRACE;
	}

	PATH_FUNC_RPT(3);
	DBUG_RETURN(nfails);
}

private	void
prefix_expected(full_list,short_list,space,prefix)
char	**full_list;
char	**short_list;
char	*space;
char	*prefix;
{
	for( ; *short_list; space += _POSIX_PATH_MAX)
	{
		*full_list++ = 
			strcat(strcpy(space, prefix),*short_list++);
	}
	*full_list = (char *)NULL;
}

private void
cln_tests()
{
	int	i;
	struct	ft *ftp;

	DBUG_ENTER("cln_tests");

	for( i = n_dirs, ftp = glob_dirs+n_dirs; i-- > 0; )
	{
		(void) chmod((--ftp)->name, MODEANY);
	}

	for( i = 0, ftp = glob_files; i++ < n_files; ftp++)
	{
		(void) unlink(ftp->name);
	}

	for( i = n_dirs, ftp = glob_dirs+n_dirs; i-- > 0; )
	{
		(void) rmdir((--ftp)->name);
	}

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	int	i;
	struct	ft *ftp;
	int	fd;
	int	save_err;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	errno = 0;
	if(sysconf(_SC_2_C_VERSION) == -1)
	{
		if (errno != 0)
		{
			xx_all(DELETION, "sysconf(_SC_2_C_VERSION) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		tsup = 0;
	}
	else
	{
		tsup = 1;
		PATH_TRACE;
	}
	
	for( i = 0, ftp = glob_dirs; i++ < n_dirs; ftp++)
	{
		if ( crdir(ftp->name, (mode_t)MODEANY,
		    	(int) getuid(), (int) getgid())  == SYSERROR )
		{
			save_err = errno;
			DBUG_PRINTF("crdir", ftp->name);
			DBUG_PRINTF("crdir", "errno = %s", errname(save_err));
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				tet_delete(i, "failed to create directory");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	for( i = 0, ftp = glob_files; i++ < n_files; ftp++)
	{
		if ( (fd = creat(ftp->name, MODEANY))
						== SYSERROR )
		{
			save_err = errno;
			DBUG_PRINTF("creat", ftp->name);
			DBUG_PRINTF("creat", "errno = %s", errname(save_err));
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				tet_delete(i, "creat call failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		(void)close(fd);
	}

	for( i = 0, ftp = glob_dirs; i++ < n_dirs; ftp++)
	{
		if ( chmod(ftp->name, ftp->perm)  == SYSERROR )
		{
			save_err = errno;
			DBUG_PRINTF("chmod", ftp->name);
			DBUG_PRINTF("chmod", "errno = %s", errname(save_err));
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				(void) tet_delete(i, "failed to chmod directory");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	PATH_SETUP_RPT(n_dirs + n_dirs + n_files + 1);

	DBUG_VOID_RETURN;
}

/*
 * When glob() has read permission on each directory of a filename component
 * containing '*', '?' or '[...]', and search permission on each component of
 * a path except the last, then glob completes ok without calling the
 * error function.
 */

private void
test1()
{
	int	nfails = 0;
	int	rtval, err, i;
	int	pathok = setupok;
	glob_t	glob_b;

	static char *patterns[] = { "dir_1/*e", "dir?1/unreadable",
		"dir_1/unsea[Rr]ch", "dir_1/unsearch", "dir_1/unreadable/a",
		NULL};

	DBUG_ENTER("test1");

	for (i = 0; patterns[i] != NULL; i++)
	{
		if (i == 0)
			PATH_TRACE;

		glob_b = empty_glob;
		errflag = FALSE;
		errno = 0;
		rtval = glob(patterns[i], GLOB_ERR, f1, &glob_b);
		err = errno;

		globok = 0;
		nfails += chk_sup(patterns[i], GLOB_ERR, f1, rtval, err);
		PATH_FUNC_TRACE;

		if (rtval != GLOB_NOSYS || err != ENOSYS)
		{
			if ( rtval != 0 )
			{
				nfails++;
				xx_rpt(FAILURE);
				in_rpt("glob(\"%s\", GLOB_ERR, errfunc, pglob) returned %d instead of 0",
					patterns[i], rtval);
			}
			else
				PATH_TRACE;

			if ( errflag == TRUE )
			{
				nfails++;
				xx_rpt(FAILURE);
				in_rpt("glob(\"%s\", GLOB_ERR, errfunc, pglob) called errfunc when no call expected",
					patterns[i]);
			}
			else
				PATH_TRACE;

			globfree(&glob_b);
		}
		else
		{
			PATH_TRACE;
			PATH_TRACE;
		}
	}

	if ( nfails == 0 )
		PATH_XS_RPT(1 + 3*i);

	DBUG_VOID_RETURN;
}


/*ARGSUSED*/
private int
f1(path, err)
	char		*path;
	int		err;
{
	DBUG_ENTER("f1");

	errflag = TRUE;
	DBUG_RETURN(0);
}

/*
 * The glob function creates a list of all accessible pathnames that
 * match pattern.
 */

private void
test2()
{
private char *expect[] = { file_002, file_004, file_005, (char *)NULL};

	int	nfails = 0;
	glob_t	glob_b;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_1/*able/a", 0, CHK_UNORDER, expect,
		&glob_b);
	PATH_FUNC_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(1);

	globfree(&glob_b);

	DBUG_VOID_RETURN;
}

/*
 * When pattern is a single character other than newline, and unquoted ?,
 * * or [ it is a pattern that matches itself.
 */

private void
test3()
{
	int	nfails = 0;
	int	pathok = setupok;
	char 	*test_pats = "x%/.\333";
	char	*expect[2];
	char	pattern[2];
	glob_t	glob_b;

	DBUG_ENTER("test3");

	pattern[1] = '\0';
	expect[0] = pattern;
	expect[1] = (char *)NULL;

	for ( ; *pattern = *test_pats; test_pats++ )
	{
		glob_b = empty_glob;
		globok = 0;
		nfails += do_matches(pattern, GLOB_NOCHECK, CHK_UNORDER, expect,
			&glob_b);
		PATH_FUNC_TRACE;
		globfree(&glob_b);
	}

	if ( nfails == 0 )
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

/*
 * When used in a bracket expression, the characters ?, * and [ lose
 * their special meaning.
 */

private void
test4()
{
private char *expect_a[] = { file_010, file_016, (char *)NULL};
private char *expect_b[] = { file_010, file_017, (char *)NULL};
private char *expect_c[] = { file_010, file_013, (char *)NULL};

	int	nfails = 0;
	glob_t	glob_b;
	int	pathok = setupok;

	DBUG_ENTER("test4");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("a[?X]b", 0, CHK_UNORDER, expect_a, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("a[*X]b", 0, CHK_UNORDER, expect_b, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("a[X[]b", 0, CHK_UNORDER, expect_c, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/*
 * the concatenation of patterns, each of which match a single character
 * matches the concatenation of characters matched by each pattern.
 */

private void
test5()
{
private char *expect[] = { file_010, file_011, file_012, (char *)NULL};

	int	nfails = 0;
	glob_t	glob_b;
	int	pathok = setupok;

	DBUG_ENTER("test5");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("a[X-Z]?", 0, CHK_UNORDER, expect, &glob_b);
	PATH_FUNC_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(1);

	globfree(&glob_b);

	DBUG_VOID_RETURN;
}

/*
 * A slash in a pathname is only matched by and explicit instance of /
 * in a pattern
 */

private void
test6()
{
private char *expect_a[] = { "dir_1*a", (char *)NULL};
private char *expect_b[] = { "dir_1?a", (char *)NULL};
private char *expect_c[] = { "dir_1[ -~]a", (char *)NULL};

	int	nfails = 0;
	glob_t	glob_b;
	int	pathok = setupok;

	DBUG_ENTER("test6");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches(*expect_a, GLOB_NOCHECK, CHK_UNORDER, expect_a,
		&glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches(*expect_b, GLOB_NOCHECK, CHK_UNORDER, expect_b,
		&glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches(*expect_c, GLOB_NOCHECK, CHK_UNORDER, expect_c,
		&glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/*
 * when pattern contains *, any characters except /, leading dot and dot
 * following / are matched.
 */

private void
test7()
{
private char *expect_a[] = { dir_001, dir_005, file_006, file_007,
	file_008, file_009, file_010, file_011, file_012, file_013,
	file_014, file_015, file_016, file_017, file_019, file_020,
	file_023, file_024, file_025, file_032,
	(char *)NULL};

private char *expect_b[] = { file_019, file_020, (char *)NULL};

	int	nfails = 0;
	glob_t	glob_b;
	int	pathok = setupok;

	DBUG_ENTER("test7");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("*", 0, CHK_UNORDER, expect_a, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("*hidden", 0, CHK_UNORDER, expect_b, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

/*
 * when pattern contains ?, any single character except /, leading dot and dot
 * following / is matched.
 */

private void
test8()
{
private char *expect_a[] = { file_006, file_007, file_008, file_009,
	(char *)NULL};

private char *expect_b[] = { file_010, file_011, file_012, file_013,
	file_014, file_015, file_016, file_017,
	file_023, file_024, file_025,
	(char *)NULL};

private char *expect_c[] = { file_032, (char *)NULL};
private char *expect_d[] = { file_019, (char *)NULL};
private char *expect_e[] = { file_030, (char *)NULL};

	int	nfails = 0;
	glob_t	glob_b;
	int	pathok = setupok;

	DBUG_ENTER("test8");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("?", 0, CHK_UNORDER, expect_a, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("a?b", 0, CHK_UNORDER, expect_b, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_2?arb", 0, CHK_UNORDER, expect_c, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("?hidden", 0, CHK_UNORDER, expect_d, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_2/?rb", 0, CHK_UNORDER, expect_e, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	if ( nfails == 0 )
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

/*
 * when pattern contains [s], any one of the characters in the set s
 * except /, leading dot and dot following / are matched.
 */

private void
test9()
{
private char *expect_a[] = { file_011, file_012, file_013, file_014,
	file_023, file_024, (char *)NULL};

private char *expect_b[] = { file_023, file_024, file_025, (char *)NULL};
private char *expect_c[] = { file_032, (char *)NULL};
private char *expect_d[] = { file_019, (char *)NULL};
private char *expect_e[] = { file_030, (char *)NULL};

	int	nfails = 0;
	glob_t	glob_b;
	int	pathok = setupok;

	DBUG_ENTER("test9");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("a[Y-y]b", 0, CHK_UNORDER, expect_a, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("a[a-z]b", 0, CHK_UNORDER, expect_b, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	/* can't put a real "/" in here: unspecified behaviour */
	nfails += do_matches("dir_2[X!-z]arb", 0, CHK_UNORDER, expect_c, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("[.X]hidden", 0, CHK_UNORDER, expect_d, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_2/[.a]rb", 0, CHK_UNORDER, expect_e, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	if ( nfails == 0 )
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	DBUG_ENTER("test10");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

/*
 * when pattern contains [!s], any characters not in the set s
 * except /, leading dot and dot following / are matched.
 */

private void
test11()
{
private char *expect_a[] = { dir_002, dir_004, file_001, file_021, dir_009,
		(char *)NULL};
private char *expect_b[] = { file_020, (char *)NULL};
private char *expect_c[] = { file_032, (char *)NULL};

	int	nfails = 0;
	glob_t	glob_b;
	int	pathok = setupok;

	DBUG_ENTER("test11");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_1/[!A-Z]*", 0, CHK_UNORDER, expect_a, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("[!A-Z]*den", 0, CHK_UNORDER, expect_b, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_2[!a]arb", 0, CHK_UNORDER, expect_c, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/*
 * When a pattern starts with a /, then filename matching starts at the
 * calling processes root directory.
 */

private void
test12()
{
private char *expect[] = { file_006, file_007, file_008, file_009,
		(char *)NULL};

	int	nfails = 0;
	glob_t	glob_b;
	int	pathok = setupok;
	char	root_pat[_POSIX_PATH_MAX];
	char	root_match[_POSIX_PATH_MAX * N_EXPECTED(expect)];
	char	*root_expect[N_EXPECTED(expect) +1];

	DBUG_ENTER("test12");

	if( getcwd(root_pat, _POSIX_PATH_MAX) == NULL ) {
		xx_rpt(DELETION);
		in_rpt("failed to find current directory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) strcat(root_pat,"/");
	prefix_expected(root_expect,expect,root_match,root_pat);
	(void) strcat(root_pat,"?");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches(root_pat, 0, CHK_UNORDER, root_expect, &glob_b);
	PATH_FUNC_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	globfree(&glob_b);

	DBUG_VOID_RETURN;
}

/*
 * When a pattern does not start with a /, then filename matching starts at the
 * calling processes current directory, interpreting ../ as the parent
 * directory of the previous pathname element.
 */

private void
test13()
{
	DBUG_ENTER("test13");

	(void) cppair(ch_t13, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t13()
{
private char *expect[] = { dir_002, dir_003, dir_004, dir_009,
		file_001, file_021,
		(char *)NULL};

	int	nfails = 0;
	glob_t	glob_b;
	int	pathok = 0;
	char	match[_POSIX_PATH_MAX * N_EXPECTED(expect)];
	char	*pr_expect[N_EXPECTED(expect) +1];

	DBUG_ENTER("ch_t13");

	if ( chdir(dir_005)  == SYSERROR )
	{
		DBUG_PRINTF("chdir", dir_005);
		DBUG_PRINTF("chdir", "errno = %s", errname(errno));
		xx_rpt(DELETION);
		in_rpt("failed to chdir directory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	prefix_expected(pr_expect,expect,match,"../");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("../dir_1/*", 0, CHK_UNORDER, pr_expect, &glob_b);
	PATH_FUNC_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	globfree(&glob_b);

	DBUG_VOID_RETURN;
}

/*
 * glob returns zero, with gl_pathc set to the correct count, and gl_pathv
 * pointing to a NULL terminated list of sorted matching pathnames.
 */

private void
test14()
{
private char *expect[] = { dir_003, dir_002, dir_004, (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	glob_t	glob_b;

	DBUG_ENTER("test14");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_1/*able", 0, CHK_ORDERED, expect, &glob_b);
	PATH_FUNC_TRACE;

	if( glob_b.gl_pathc > 0 && glob_b.gl_pathv[glob_b.gl_pathc] != NULL)
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("gl_pathv not correctly NULL terminated");
		in_rpt("gl_pathv[%d] contains 0x%lx",glob_b.gl_pathc,
				(long)glob_b.gl_pathv[glob_b.gl_pathc]);

		globfree(&glob_b);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

/*
 * When GLOB_NOSORT is not set, the matched patterns are sorted according
 * to the setting of the current LC_COLLATE category
 */

private void
test15()
{
private char *expect_a[] = { file_010, file_011, file_012,
		file_023, file_024, file_025,
		(char *)NULL};

	int	nfails = 0;
	glob_t	glob_b;
	int	pathok = setupok;

	DBUG_ENTER("test15");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("a[a-zA-Z]b", 0, CHK_ORDERED, expect_a, &glob_b);
	PATH_FUNC_TRACE;

	p_nfails = nfails;
	PATH_FUNC_RPT(1);
	(void) cppair(ch_t15, NULLFN, WAITTIME, 0);

	globfree(&glob_b);

	DBUG_VOID_RETURN;
}

private void
ch_t15()
{
private char *expect_b[] = { file_023, file_024, file_025,
		file_010, file_011, file_012,
		(char *)NULL};

	int	nfails = p_nfails;
	glob_t	glob_b;
	int	pathok = 0;

	DBUG_ENTER("ch_t15");

	PATH_FUNC_TRACE;

	if (setlocale(LC_COLLATE, INFO3LANG) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("setlocale(LC_COLLATE, %s) failed", INFO3LANG);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("a[a-zA-Z]b", 0, CHK_ORDERED, expect_b, &glob_b);
	PATH_FUNC_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	globfree(&glob_b);

	DBUG_VOID_RETURN;
}

/*
 * When GLOB_NOCHECK is not set in flags, and no pathnames match the pattern
 * then gl_pathc is set to zero.
 */

private void
test16()
{
	int	rtval, err;
	int	nfails = 0;
	int	pathok = setupok;
	char	*pattern;
	glob_t	glob_b;

	DBUG_ENTER("test16");

	pattern = un_matchable;
	glob_b = empty_glob;
	errno = 0;
	rtval = glob(pattern, 0, NULLFP, &glob_b);
	err = errno;

	globok = 0;
	nfails += chk_sup(pattern, 0, NULLFP, rtval, err);
	PATH_FUNC_TRACE;

	if (rtval != GLOB_NOSYS || err != ENOSYS)
	{
		if ( rtval != GLOB_NOMATCH )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", 0, NULL, pglob) returned %d instead of GLOB_NOMATCH",
				pattern, rtval);
		}
		else if( glob_b.gl_pathc != 0 )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("After glob() returned GLOB_NOMATCH, gl_pathc contained %d instead of 0",
				glob_b.gl_pathc);
		}
		else
			PATH_TRACE;

		globfree(&glob_b);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

/*
 * A call to glob allocates the memory pointed to by gl_pathv
 */

private void
test17()
{
private char *expect_a[] = { file_010, file_011, file_012,
		file_023, file_024, file_025,
		(char *)NULL};

private char *expect_b[] = { dir_003, dir_004, dir_002,
		(char *)NULL};

	int	nfails = 0;
	int	n;
	int	pathok = setupok;
	glob_t	glob_ba;
	glob_t	glob_bb;
	char	**bad_val;

	DBUG_ENTER("test17");

	bad_val = &test_vecp;
	glob_ba = empty_glob;
	glob_ba.gl_pathv = bad_val;
	glob_bb = glob_ba;
	globok = 0;
	nfails += do_matches("a[a-zA-Z]b", 0, CHK_UNORDER, expect_a, &glob_ba);
	PATH_FUNC_TRACE;

	if (glob_ba.gl_pathc != 0)
	{
		if( glob_ba.gl_pathv == bad_val )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob() has left gl_pathv unchanged");
		}
		else
			PATH_TRACE;

		globok = 0;
		nfails += do_matches("dir_1/*able", 0, CHK_UNORDER, expect_b, &glob_bb);
		PATH_FUNC_TRACE;

		if( nfails == 0 )
		{
			globok = 0;
			n = return_chk(CHK_UNORDER, "dir_1/*able", expect_a, &glob_ba);
			PATH_FUNC_TRACE;
			nfails += n;
			if( n )
				in_rpt("second glob() changed results of first glob()");
		}

		globfree(&glob_ba);
		globfree(&glob_bb);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

/*
 * globfree frees any space allocated by a previous call to glob.
 * This is a touch test, how do you check the globfree does anything
 * usefull? globfree is used fairly heavily through-out the test anyway,
 * so any real problems should show up.
 */

private void
test18()
{
	int	rtval;
	int	nfails = 0;
	int	pathok = setupok;
	glob_t	glob_b;

	DBUG_ENTER("test18");

	glob_b = empty_glob;
	errno = 0;
	if ( (rtval = glob("dir_1/*", 0, NULLFP, &glob_b)) != 0 )
	{
		if (rtval == GLOB_NOSYS && errno == ENOSYS)
		{
			xx_rpt(UNTESTED);
			in_rpt("glob() is not supported");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("First call to glob() returned %d instead of 0",
				rtval);
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	globfree(&glob_b);
	if (errno != ENOSYS)
	{
		if ( (rtval = glob("un-matchable pattern", 0, NULLFP, &glob_b)) != GLOB_NOMATCH )
		{
			nfails++;
			xx_rpt(DELETION);
			in_rpt("Second call to glob() returned %d instead of GLOB_NOMATCH",
				rtval);
		}
		else
			PATH_TRACE;

		globfree(&glob_b);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

/*
 * When GLOB_APPEND is in flags, the matched patterns are stored after
 * ther current set of paths.
 */

private void
test19()
{
private char *expect_a[] = { file_001, file_021, dir_002, dir_004, dir_009,
		(char *)NULL};

private char *expect_b[] = { file_001, file_021, dir_002, dir_004, dir_009,
		file_006, file_007, file_008, file_009,
		(char *)NULL};

	int	nfails = 0;
	glob_t	glob_b;
	int	pathok = setupok;

	DBUG_ENTER("test19");

	glob_b = empty_glob;

	globok = 0;
	nfails += do_matches("dir_1/[!A-Z]*", 0, CHK_ORDERED, expect_a, &glob_b);
	PATH_FUNC_TRACE;

	globok = 0;
	nfails += do_matches("?", GLOB_APPEND, CHK_ORDERED, expect_b, &glob_b);
	PATH_FUNC_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	globfree(&glob_b);

	DBUG_VOID_RETURN;
}

/*
 * When GLOB_DOOFFS is set in flags, then glob() starts the array of
 * pointers in gl_pathv with the number of NULL pointers specified in
 * gl_offs and follows this with gl_pathc pathname pointers,
 * followed by the terminating NULL pointer.
 */

private void
test20()
{
private char *expect_a[] = { file_001, file_021, dir_002, dir_004, dir_009,
		(char *)NULL};

private char *expect_b[] = { file_001, file_021, dir_002, dir_004, dir_009,
		file_006, file_007, file_008, file_009,
		(char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	glob_t	glob_b;
	int	offset = 3;

	DBUG_ENTER("test20");

	glob_b = empty_glob;

	glob_b.gl_offs = offset;
	globok = 0;
	nfails += do_matches("dir_1/[!A-Z]*", GLOB_DOOFFS, CHK_UNORDER,
		expect_a, &glob_b);
	PATH_FUNC_TRACE;

	globok = 0;
	nfails += do_matches("?", GLOB_DOOFFS|GLOB_APPEND, CHK_ORDERED,
		expect_b, &glob_b);
	PATH_FUNC_TRACE;

	if( glob_b.gl_pathc > 0 &&
		glob_b.gl_pathv[glob_b.gl_pathc + glob_b.gl_offs] != NULL)
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("gl_pathv not correctly NULL terminated");
		in_rpt("gl_pathv[%d] contains 0x%lx",
			glob_b.gl_pathc + glob_b.gl_offs,
			(long)glob_b.gl_pathv[glob_b.gl_pathc + glob_b.gl_offs]);

		globfree(&glob_b);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/*
 * When glob() encounters a directory that cannot be read, and errfunc
 * is null, and GLOB_ERR is not on, the the search continues.
 */

private void
test21()
{
private char *expect[] = { file_002, file_004, file_005, file_026, file_027,
		(char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	glob_t	glob_b;

	DBUG_ENTER("test21");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_?/*able/a", 0, CHK_UNORDER, expect, &glob_b);
	PATH_FUNC_TRACE;

	globfree(&glob_b);

	if ( nfails == 0 )
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/*
 * when GLOB_MARK is set, then matching directories will have a / suffix.
 */

private void
test22()
{
private char *expect[] = { "dir_1/", "dir_2/", file_006, file_007,
	file_008, file_009, file_010, file_011, file_012, file_013,
	file_014, file_015, file_016, file_017, file_019, file_020,
	file_023, file_024, file_025, file_032,
		(char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	glob_t	glob_b;

	DBUG_ENTER("test22");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("*", GLOB_MARK, CHK_UNORDER, expect, &glob_b);
	PATH_FUNC_TRACE;

	globfree(&glob_b);

	if ( nfails == 0 )
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/*
 * when GLOB_NOCHECK is set, and there are no matches, then the pattern
 * will be returned as a single match.
 */

private void
test23()
{
private char *expect[] = { un_matchable, (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	glob_t	glob_b;

	DBUG_ENTER("test23");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches(un_matchable, GLOB_NOCHECK, CHK_UNORDER,
		expect, &glob_b);
	PATH_FUNC_TRACE;

	globfree(&glob_b);

	if ( nfails == 0 )
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/*
 * When GLOB_NOSORT is set, matches are not sorted.
 */

private void
test24()
{
private char *expect[] = { dir_002, dir_004, file_001, file_021, dir_009,
		(char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	glob_t	glob_b;

	DBUG_ENTER("test24");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_1/[!A-Z]*", GLOB_NOSORT, CHK_UNORDER,
		expect, &glob_b);
	PATH_FUNC_TRACE;

	globfree(&glob_b);

	if ( nfails == 0 )
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/*
 * When GLOB_NOESCAPE is not set, then \c matches the same character c.
 */

private void
test25()
{
private char *expect_a[] = { file_028, (char *)NULL};
private char *expect_b[] = { file_030, (char *)NULL};
private char *expect_c[] = { file_029, (char *)NULL};
private char *expect_d[] = { file_013, (char *)NULL};
private char *expect_e[] = { file_017, (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	glob_t	glob_b;

	DBUG_ENTER("test25");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_2/a\\?b", 0, CHK_UNORDER, expect_a, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_2/a\\rb", 0, CHK_UNORDER, expect_b, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_2/a\\\\b", 0, CHK_UNORDER, expect_c, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("a\\[b", 0, CHK_UNORDER, expect_d, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("a\\*b", 0, CHK_UNORDER, expect_e, &glob_b);
	PATH_FUNC_TRACE;
	globfree(&glob_b);

	if ( nfails == 0 )
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

/*
 * When GLOB_NOESCAPE is set, then a \ matches a \ in the filename
 */

private void
test26()
{
private char *expect[] = { file_029,
		(char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	glob_t	glob_b;

	DBUG_ENTER("test26");

	glob_b = empty_glob;
	globok = 0;
	nfails += do_matches("dir_2/a\\b", GLOB_NOESCAPE, CHK_UNORDER,
		expect, &glob_b);
	PATH_FUNC_TRACE;

	globfree(&glob_b);

	if ( nfails == 0 )
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/*
 * when a directory is encountered that cannot be read and errfunc
 * is not NULL, then errfunc is called with two args, the unreadable
 * pathname and the errno produced by opendir etc.
 */

private	int called_f27;

private void
test27()
{
	int	nfails = 0;
	int	rtval, err;
	char	*pattern = "dir_1/*able/*";
	int	pathok = setupok;
	glob_t	glob_b;

	DBUG_ENTER("test27");

	glob_b = empty_glob;
	errno = 0;
	rtval = glob(pattern, 0, f27, &glob_b);
	err = errno;

	globok = 0;
	nfails += chk_sup(pattern, 0, f27, rtval, err);
	PATH_FUNC_TRACE;

	if (rtval != GLOB_NOSYS || err != ENOSYS)
	{
		if( called_f27 != 1 )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", 0, errfunc, pglob) called errfunc incorrect number of times",
				pattern);
			in_rpt("expected number of calls: 1, observed: %d",
				called_f27);
		}
		else
			PATH_TRACE;

		globfree(&glob_b);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private int
f27(path, err)
	char		*path;
	int		err;
{
	int	nfails = 0;
	int	pathok = 0;
	char 	*expect_p1 = "dir_1/unreadable";
	char 	*expect_p2 = "dir_1/unreadable/";

	DBUG_ENTER("f27");

	called_f27++;

	if( strcmp( path, expect_p1) && strcmp( path, expect_p2) )
	{
		errflag = TRUE;
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("glob() called errfunc with in-correct epath arg");
		in_rpt("expected %s or %s, actual %s",
			expect_p1, expect_p2, path);
	}
	else
		PATH_TRACE;

	if( err != EACCES )
	{
		errflag = TRUE;
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("glob() called errfunc with errno arg %d (%s), expected %d (EACCES)",
			err, errname(err), EACCES);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_FUNC_RPT(2);

	DBUG_RETURN(0);
}

/*
 * when a directory is encountered that cannot be read and errfunc
 * is not NULL, and GLOB_ERR is not set in flags, then if errfunc returns
 * zero, further pattern matching will occur.
 */

private	int called_f28;

private void
test28()
{
private char *expect[] = { file_002, file_003, file_005, file_026, file_027,
		(char *)NULL};

	int	nfails = 0;
	int	rtval, err;
	char	*pattern = "dir_?/*able/*";
	int	pathok = setupok;
	glob_t	glob_b;

	DBUG_ENTER("test28");

	glob_b = empty_glob;
	errno = 0;
	rtval = glob(pattern, 0, f28, &glob_b);
	err = errno;

	globok = 0;
	nfails += chk_sup(pattern, 0, f28, rtval, err);
	PATH_FUNC_TRACE;

	if (rtval != GLOB_NOSYS || err != ENOSYS)
	{
		if( called_f28 != 2 )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", 0, errfunc, pglob) called errfunc incorrect number of times",
				pattern);
			in_rpt("expected number of calls: 2, observed: %d",
				called_f28);
		}
		else
			PATH_TRACE;

		if( glob_b.gl_pathc != N_EXPECTED(expect) )
		{
			nfails++;
			xx_rpt(FAILURE);
			rpt_expected(pattern, expect, &glob_b, N_EXPECTED(expect));
		}
		else
		{
			globok = 0;
			nfails += return_chk(CHK_UNORDER, pattern, expect, &glob_b);
			PATH_FUNC_TRACE;
		}

		globfree(&glob_b);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/* ARGSUSED */
private int
f28(path, err)
	char		*path;
	int		err;
{
	DBUG_ENTER("f28");

	called_f28++;

	DBUG_RETURN(0);
}

/*
 * When GLOB_ERR is set in flags and a call to glob encounters a directory
 * that cannot be read, then glob returns GLOB_ABORTED, and sets gl_pathc and
 * gl_pathv to reflect the matches so far.
 */

private void
test29()
{
	int	rtval, err;
	int	nfails = 0;
	int	pathok = setupok;
	char	*pattern = "dir_1/*able/*";
	glob_t	glob_b;

	DBUG_ENTER("test29");

	glob_b = empty_glob;
	errno = 0;
	rtval = glob(pattern, GLOB_ERR, NULLFP, &glob_b);
	err = errno;

	globok = 0;
	nfails += chk_sup(pattern, GLOB_ERR, NULLFP, rtval, err);
	PATH_FUNC_TRACE;

	if (rtval != GLOB_NOSYS || err != ENOSYS)
	{
		if( rtval != GLOB_ABORTED )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", GLOB_ERR, NULL, pglob) returned %d instead of GLOB_ABORTED",
				pattern, rtval);
		}
		else
			PATH_TRACE;

/*
 * We cant test the partial result that will be caused by aborting the
 * search as there is no way of knowing the order that files and directories
 * will be encountered. File creation order is not garuanteed even if it
 * is what most systems do.
 */

		globfree(&glob_b);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

/*
 * When a call to glob encounters a directory that cannot be read,
 * and errfunct returns non-zero, then glob returns GLOB_ABORTED,
 * and sets gl_pathc and gl_pathv to reflect the matches so far.
 */

private	int called_f30;

private void
test30()
{
	int	rtval, err;
	int	nfails = 0;
	char	*pattern = "dir_1/*able/*";
	int	pathok = setupok;
	glob_t	glob_b;

	DBUG_ENTER("test30");

	glob_b = empty_glob;
	errno = 0;
	rtval = glob(pattern, GLOB_ERR, f30, &glob_b);
	err = errno;

	globok = 0;
	nfails += chk_sup(pattern, GLOB_ERR, f30, rtval, err);
	PATH_FUNC_TRACE;

	if (rtval != GLOB_NOSYS || err != ENOSYS)
	{
		if( rtval != GLOB_ABORTED )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", GLOB_ERR, errfunc, pglob) returned %d instead of GLOB_ABORTED",
				pattern, rtval);
		}
		else
			PATH_TRACE;

		if( called_f30 != 1 )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", GLOB_ERR, errfunc, pglob) called errfunc incorrect number of times",
				pattern);
			in_rpt("expected number of calls: 1, observed: %d",
				called_f30);
		}
		else
			PATH_TRACE;

/*
 * We cant test the partial result that will be caused by aborting the
 * search as there is no way of knowing the order that files and directories
 * will be encountered. File creation order is not garuanteed even if it
 * is what most systems do.
 */

		globfree(&glob_b);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	called_f30 = 0;
	glob_b = empty_glob;
	errno = 0;
	rtval = glob(pattern, 0, f30, &glob_b);
	err = errno;

	globok = 0;
	nfails += chk_sup(pattern, 0, f30, rtval, err);
	PATH_FUNC_TRACE;

	if (rtval != GLOB_NOSYS || err != ENOSYS)
	{
		if( rtval != GLOB_ABORTED )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", 0, errfunc, pglob) returned %d instead of GLOB_ABORTED",
				pattern, rtval);
		}
		else
			PATH_TRACE;

		if( called_f30 != 1 )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", 0, errfunc, pglob) called errfunc incorrect number of times",
				pattern);
			in_rpt("expected number of calls: 1, observed: %d",
				called_f30);
		}
		else
			PATH_TRACE;

/*
 * We cant test the partial result that will be caused by aborting the
 * search as there is no way of knowing the order that files and directories
 * will be encountered. File creation order is not garuanteed even if it
 * is what most systems do.
 */

		globfree(&glob_b);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

/* ARGSUSED */
private int
f30(path, err)
	char		*path;
	int		err;
{
	int	nfails = 0;
	int	pathok = 0;

	DBUG_ENTER("f30");

	called_f30++;

	if ( nfails == 0 )
		PATH_XS_RPT(0);

	DBUG_RETURN(1);
}

/*
 * When GLOB_NOCHECK is not set in flags, and there are no matches, then glob
 * returns GLOB_NOMATCH, and sets pathc to 0.
 */

private void
test31()
{
	int	rtval, err;
	int	nfails = 0;
	char	*pattern = un_matchable;
	int	pathok = setupok;
	glob_t	glob_b;

	DBUG_ENTER("test31");

	glob_b = empty_glob;
	errno = 0;
	rtval = glob(pattern, 0, NULLFP, &glob_b);
	err = errno;

	globok = 0;
	nfails += chk_sup(pattern, 0, NULLFP, rtval, err);
	PATH_FUNC_TRACE;

	if (rtval != GLOB_NOSYS || err != ENOSYS)
	{
		if ( rtval != GLOB_NOMATCH )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("glob(\"%s\", 0, NULL, pglob) returned %d instead of GLOB_NOMATCH",
				pattern, rtval);
		}
		else if( glob_b.gl_pathc != 0 )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("After glob() returned GLOB_NOMATCH, gl_pathc contained %d instead of 0",
				glob_b.gl_pathc);
		}
		else
			PATH_TRACE;

		globfree(&glob_b);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
