/* dv @(#)V9TS/tset/GABI.os/genuts/nftw/nftw.c	1.3 95/12/11 */
/*
*	SCCS:  @(#) GABI.os/genuts/nftw/nftw.c	Ver. 1.10	Rel. 1.3	(92/08/01)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#) GABI.os/genuts/nftw/nftw.c	Ver. 1.10	Rel. 1.3	(92/08/01)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.nftw

PROJECT:	Validation Suite

DESCRIPTION:
	Testset for the nftw() subroutine.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:		Derived from ftw() (which is bad enough).

AUTHOR:		Calum Laing, UniSoft Ltd.
DATE CREATED:	13th September, 1990.
MODIFICATIONS:
		December 1990 by Alan:
			eliminated some lint errors, cleaned up error reporting,
			fixed some resiliency (PATH_TRACE), ...

		Andrew Dingwall, UniSoft Ltd., December 2000
		Improved error reporting in test8().

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<stdlib.h>
#include	<string.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<ftw.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef nftw
#endif

#define		NO_TESTS	18	/* number of tests in testset */
#define		COMP_TESTS	13	/* number of compliance tests */
#define		WAITTIME	(5*SPEEDFACTOR)
#define		FTWF		1	/* bits in flag set by f3() */
#define		FTWD		2
#define		FTWDNR		4
#define		FTWNS	        8

#define		FTWDP		16
#define 	FTWSL		32

#define		MODEANY		(int) (S_IRWXU|S_IRWXG|S_IRWXO)
#define		VSX_MOUNT_DEV	"VSX_MOUNT_DEV"

extern	char	*errname();
extern	int	crdir();
extern	int	crfile();
extern	int	cppair();
extern	int	setprv();
extern	int	n_open_fds();
extern	char	**pathtoolng();
extern	char	**nametoolng();
extern	void	clntoolng();

int	no_tests = NO_TESTS;
int	globok;
private	int	jumpok = 0;
private	int	testfail = 0;
private	int	setupok = -1;
private char	t8_cwd[1026];
private int	t8_repcount;

struct	ft
{
	char	*name;
	ino_t	inode;
	dev_t	device;
	int	flag;
};

/*
 * A tree which consists of ordinary files.
 */
 
#define NMR_OFILES 7

private int	ft_level[NMR_OFILES]	= { 2, 1, 2, 2, 1, 1, 0 };
private int	ft_rlevel[NMR_OFILES];
private int	ft_base[NMR_OFILES]	= { 16, 8, 16, 16, 8, 8, 2 };
private int	ft_rbase[NMR_OFILES];
private struct	ft ft[NMR_OFILES] = {
	"./ftwD1/ftw_d.2/ftw_t.4",	0,	0,	TRUE,
	"./ftwD1/ftw_d.2",		0,	0,	TRUE,
	"./ftwD1/ftw_d.1/ftw_t.3",	0,	0,	TRUE,
	"./ftwD1/ftw_d.1/ftw_t.2",	0,	0,	TRUE,
	"./ftwD1/ftw_d.1",		0,	0,	TRUE,
	"./ftwD1/ftw_t.1",		0,	0,	TRUE,
	"./ftwD1",			0,	0,	TRUE
};


/*
 * A tree containing a mounted subtree.
 */
#define NMR_MNTD 3
#define NMR_MFILES 7
#define NMR_NMNTD 4

private int	mounted[NMR_MNTD] = { 2, 3, 4 }; 
private struct	ft mft[NMR_OFILES] = {
	"./ftwD2/ftw_d.3/ftw_t.7",	0,	0,	TRUE,
	"./ftwD2/ftw_d.3",		0,	0,	TRUE,
	"./ftwD2/ftw_mnt/ftw_t.5",	0,	0,	TRUE,
	"./ftwD2/ftw_mnt/ftw_t.6",	0,	0,	TRUE,
	"./ftwD2/ftw_mnt",		0,	0,	TRUE,
	"./ftwD2/ftw_t.8",		0,	0,	TRUE,
	"./ftwD2",			0,	0,	TRUE
};

#define NMR_LFILES 4

/*
 * A tree with a link.
 */

private struct ft lft[NMR_LFILES] = {
	"./ftwD3",				0,	0,	TRUE,
	"./ftwD3/ftw_t.9",			0,	0,	TRUE,
	"./ftwD3/ftw_d.4",			0,	0,	TRUE,
	"./ftwD3/ftw_d.4/ftw_slink",		0,	0,	TRUE
};
					
private char	*t14_dir =	"ftw_d.12";
private char	*t16_dir =	"./nonexistant";
private char	*t18_file =	"./ftwD1/ftw_t.18";
private char	*t18a_file =	"./ftwD1/ftw_t.18/ftw_t.18";

private	char	*mnt_dev = NULL;

private void	prepare_tests();
private void	cln_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	ch_t9();
private void	test10();
private void	ch_t10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();

private int	f1();
private int	f2();
private int	f3();
private int	f3a();
private int	f4();
private int	f5();
private int	f6();
private int	f7();
private int	f8();
private int	f9();
private int	f9a();
private int	f9b();
private int	f10();
private int	f11();
private int	f12();
private int	f13();
private int	f14();
private int	f15();
private int	f16();
private int	f17();
private int	f18();

private int	errflag;
private int	flag;
private int	ftopen;

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 	test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	NULL, 0
};


private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) unlink("./ftwD3/ftw_t.9");
	(void) rmdir("./ftwD3/ftw_d.4/slink");
	(void) rmdir("./ftwD3/ftw_d.4");
	(void) rmdir("./ftwD3");

	(void) unlink("./ftwD2/ftw_t.8");
	(void) unlink("./ftwD2/ftw_d.3/ftw_t.7");
	(void) rmdir("./ftwD2/ftw_d.3");
	(void) unlink("./ftwD2/ftw_mnt/ftw_t.5");
	(void) unlink("./ftwD2/ftw_mnt/ftw_t.6");
	(void) unmnt(mnt_dev, "./ftwD2/ftw_mnt");
	(void) rmdir("./ftwD2/ftw_mnt");
	(void) rmdir("./ftwD2");

	(void) unlink("./ftwD1/ftw_d.2/ftw_t.4");
	(void) rmdir("./ftwD1/ftw_d.2");
	(void) unlink("./ftwD1/ftw_d.1/ftw_t.3");
	(void) unlink("./ftwD1/ftw_d.1/ftw_t.2");
	(void) rmdir("./ftwD1/ftw_d.1");
	(void) unlink("./ftwD1/ftw_t.1");
	(void) rmdir("./ftwD1");
	
	(void) rmdir(t14_dir);
	(void) unlink(t18_file);

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	int	fd;
	int	i;
	struct	stat	statb;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");


/* Ordinary tree. */

	if ( crdir("./ftwD1", MODEANY, (int) getuid(), (int) getgid())  == SYSERROR )	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "failed to create directory ftwD1.");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (fd = creat("./ftwD1/ftw_t.1", MODEANY)) == SYSERROR ) {
		DBUG_PRINTF("creat", "ftw_t.1");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	} else {
		PATH_TRACE;
		(void) close(fd);
	}


	if ( crdir("./ftwD1/ftw_d.1", MODEANY, (int) getuid(), (int) getgid())  == SYSERROR )	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "failed to create directory");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (fd = creat("./ftwD1/ftw_d.1/ftw_t.2", MODEANY))	== SYSERROR )	{
		DBUG_PRINTF("creat", "ftw_t.2");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	} else {
		PATH_TRACE;
		(void) close(fd);
	}

	if ( (fd = creat("./ftwD1/ftw_d.1/ftw_t.3", MODEANY)) == SYSERROR ) {
		DBUG_PRINTF("creat", "ftw_t.3");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	} else {
		PATH_TRACE;
		(void) close(fd);
	}

	if ( crdir("./ftwD1/ftw_d.2", MODEANY, (int) getuid(), (int) getgid() ) == SYSERROR )	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "failed to create directory");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (fd = creat("./ftwD1/ftw_d.2/ftw_t.4", MODEANY))	== SYSERROR ) {
		DBUG_PRINTF("creat", "ftw_t.4");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	} else {
		PATH_TRACE;
		(void) close(fd);
	}

	for ( i = 0 ; i < NMR_OFILES ; i++ ) {
		if (stat(ft[i].name, &statb) == SYSERROR) {
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				(void) tet_delete(i, "stat call failed");
			DBUG_VOID_RETURN;
		} else {
			PATH_TRACE;
			ft[i].inode = statb.st_ino;
			ft[i].device = statb.st_dev;
		}
	}

/* Mount Tree */

	if ( crdir("./ftwD2", MODEANY, (int) getuid(), (int) getgid())  == SYSERROR )	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "failed to create directory ftwD2.");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (fd = creat("./ftwD2/ftw_t.8", MODEANY)) == SYSERROR ) {
		DBUG_PRINTF("creat", "ftw_t.8");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	} else {
		PATH_TRACE;
		(void) close(fd);
	}

        if ((mnt_dev = tet_getvar(VSX_MOUNT_DEV)) == NULL ||
                        *mnt_dev == '\0')
        {
		(void) tet_delete(4, "VSX_MOUNT_DEV is not set.");
		(void) tet_delete(5, "VSX_MOUNT_DEV is not set.");
		pathok += NMR_OFILES + 5;
        } else if( setprv(PRV_MOUNT) != 0) {
		(void) tet_delete(4, "setprv(PRV_MOUNT) failed in setup.");
		(void) tet_delete(5, "setprv(PRV_MOUNT) failed in setup.");
		pathok += NMR_OFILES + 5;
	} else if ((stat(mnt_dev, &statb) == SYSERROR) || 
			(!S_ISBLK(statb.st_mode))) {
		(void) tet_delete(4, "could not stat VSX_MOUNT_DEV.");
		(void) tet_delete(5, "could not stat VSX_MOUNT_DEV.");
		pathok += NMR_OFILES + 5;
	} else if (crdir("./ftwD2/ftw_mnt", MODEANY, (int) getuid(), 
			(int) getgid()) == SYSERROR) {
		(void) tet_delete(4, "could not create ./ftwD2/ftw_mnt.");
		(void) tet_delete(5, "could not create ./ftwD2/ftw_mnt.");
		pathok += NMR_OFILES + 5;
	} else if(mnt_rw(mnt_dev, "./ftwD2/ftw_mnt") < 0) {
		(void) rmdir("./ftwD2/ftw_mnt");
		(void) tet_delete(4, "failed to mount fs at ./ftw_mnt.");
		(void) tet_delete(5, "failed to mount fs at ./ftw_mnt.");
		pathok += NMR_OFILES + 5;
	} else {
		PATH_TRACE;
			
		if ( (fd = creat("./ftwD2/ftw_mnt/ftw_t.5", MODEANY))	== SYSERROR )	{
			DBUG_PRINTF("creat", "ftw_t.5");
			DBUG_PRINTF("creat", "errno = %s", errname(errno));
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				(void) tet_delete(i, "creat call failed");
			DBUG_VOID_RETURN;
		} else {
			PATH_TRACE;
			(void) close(fd);
		}
	
		if ( (fd = creat("./ftwD2/ftw_mnt/ftw_t.6", MODEANY)) == SYSERROR ) {
			DBUG_PRINTF("creat", "ftw_t.6");
			DBUG_PRINTF("creat", "errno = %s", errname(errno));
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				(void) tet_delete(i, "creat call failed");
			DBUG_VOID_RETURN;
		} else {
			PATH_TRACE;
			(void) close(fd);
		}
	
		if ( crdir("./ftwD2/ftw_d.3", MODEANY, (int) getuid(), (int) getgid())  == SYSERROR )	{
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				(void) tet_delete(i, "failed to create directory ftw_mnt/ftwd.3");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	
		if ( (fd = creat("./ftwD2/ftw_d.3/ftw_t.7", MODEANY))	== SYSERROR )	{
			DBUG_PRINTF("creat", "ftw_t.5");
			DBUG_PRINTF("creat", "errno = %s", errname(errno));
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				(void) tet_delete(i, "creat call failed");
			DBUG_VOID_RETURN;
		} else {
			PATH_TRACE;
			(void) close(fd);
		}
	
		for ( i = 0 ; i < NMR_OFILES ; i++ ) {
			if (stat(mft[i].name, &statb) == SYSERROR) {
				for ( i = 1 ; i <= NO_TESTS ; i++ )
					(void) tet_delete(i, "stat call failed");
				DBUG_VOID_RETURN;
			} else {
				PATH_TRACE;
				mft[i].inode = statb.st_ino;
				mft[i].device = statb.st_dev;
			}
		}
	}

/* Link Tree */
	if ( crdir("./ftwD3", MODEANY, (int) getuid(), (int) getgid())  == SYSERROR )	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "failed to create directory ftwD3");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( crdir("./ftwD3/ftw_d.4", MODEANY, (int) getuid(), (int) getgid())  == SYSERROR )	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "failed to create directory ftwD3");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (fd = creat("./ftwD3/ftw_t.9", MODEANY)) == SYSERROR ) {
		DBUG_PRINTF("creat", "ftw_t.9");
		DBUG_PRINTF("creat", "errno = %s", errname(errno));
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			(void) tet_delete(i, "creat call failed");
		DBUG_VOID_RETURN;
	} else {
		PATH_TRACE;
		(void) close(fd);
	}
	

/*
 * Stat() the files in the link tree.
 */

	for ( i = 0 ; i < NMR_LFILES - 1; i++ ) {
		if (stat(lft[i].name, &statb) == SYSERROR) {
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				(void) tet_delete(i, "stat call failed");
			DBUG_VOID_RETURN;
		} else {
			PATH_TRACE;
			lft[i].inode = statb.st_ino;
			lft[i].device = statb.st_dev;
		}
	}

	setupok = (pathok == 3 + 7 + NMR_OFILES + 7 + NMR_OFILES +NMR_LFILES - 1) ? 0 : -1;
	
	DBUG_PRINTF("setup","Setupok = %d pathok = %d",setupok, pathok);
	DBUG_VOID_RETURN;
}


private void
test1()
{
	int	nfails = 0;
	int	rtval;
	int	i;
	int	pathok = setupok;

	DBUG_ENTER("test1");

	jumpok = 0;
	if ( (rtval = nftw("./ftwD1", f1, 3, 0 )) != 0 ){ 
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == NMR_OFILES)
		PATH_TRACE;

	for ( i = 0 ; i < NMR_OFILES ; i++ ) {
		if ( ft[i].flag == TRUE ) {
			if ( nfails++ == 0 ) 
				xx_rpt(FAILURE);
			in_rpt("function set by ftw() not called for %s",ft[i].name);
		} else
			PATH_TRACE;

		ft[i].flag = TRUE;	/* reset for later tests	*/
	}

	if ( nfails == 0 )
		PATH_XS_RPT(NMR_OFILES + 2);

	DBUG_VOID_RETURN;
}

private int
f1(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	int	i;

	DBUG_ENTER("f1");
	DBUG_PRINTF("f1","Path passed is %s ( %ld , %ld )",path,(long) statptr -> st_dev, (long) statptr -> st_ino);

	for ( i = 0 ; i < NMR_OFILES ; i++ ) {
		if ( statptr == (struct stat *) NULL ) {
			continue;
		}

		if ( statptr->st_dev == ft[i].device && statptr->st_ino == ft[i].inode ){
			ft[i].flag = FALSE;
			JUMP_TRACE;
			DBUG_RETURN(0);
		} 
	}
	                      
	DBUG_RETURN(0);       
}	                      
                              

private void
test6()
{
	int	nfails = 0;
	int	rtval;
	int	i;
	int	pathok = setupok;

	DBUG_ENTER("test6");

	errflag = FALSE;
	jumpok = 0;
	if ( (rtval = nftw("./ftwD1", f6, 3,FTW_DEPTH)) != 0 ) {
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	} else
		PATH_TRACE;

	if (jumpok == NMR_OFILES)
		PATH_TRACE;

	if ( errflag )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("function set by ftw called with 3rd argument");
		in_rpt("other than FTW_D or FTW_F");
	}
	else
		PATH_TRACE;

	for ( i = 0 ; i < NMR_OFILES ; i++ )
	{
		if ( ft[i].flag == TRUE )
		{
			if ( nfails++ == 0 )
				xx_rpt(FAILURE);
			in_rpt("function called by ftw set flag for file");
			in_rpt("if its parent directory was already visited.");
			in_rpt("flag was not set for %s", ft[i].name);
		}
		else
			PATH_TRACE;
		ft[i].flag = TRUE;	/* reset for later tests	*/
	}

	if ( nfails == 0 )
		PATH_XS_RPT(NMR_OFILES+3);

	DBUG_VOID_RETURN;
}


private int
f6(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	int	i;

	DBUG_ENTER("f6");
	DBUG_PRINTF("f6","Path passed is %s (%ld,%ld)",path,(long) statptr->st_dev,(long) statptr->st_ino);
	JUMP_TRACE;	/*	to prove that we have been called	*/

	for ( i = 0 ; i < NMR_OFILES ; i++ )	{

		if ( statptr == (struct stat *) NULL )
			continue;

		if ( statptr->st_dev == ft[i].device && statptr->st_ino == ft[i].inode ) {
			switch (i) {

			case 0 : /* ftwD1/ftw_d.2/ftw_t.4 */
				if( ft[1].flag == TRUE )
					ft[0].flag = FALSE;
				break;

			case 1 : /* ftwD1/ftw_d.2 */
			case 4 :
			case 5 :
				if(ft[6].flag == TRUE)
					ft[i].flag = FALSE;
				break;

			case 2 :
			case 3 : /* ftwD1/ftw_d.1/ftw_t.{3,4} */
				if( ft[4].flag == TRUE)
					ft[i].flag = FALSE;
				break;

			case 6 : /* ftwD1/ftwD1 */

				if(ft[1].flag == FALSE && ft[4].flag == FALSE && ft[5].flag == FALSE)
					ft[6].flag = FALSE;

				break;
			default :
				errflag = TRUE;
				break;
			}
			DBUG_RETURN(0);
		}
	}
	DBUG_RETURN(0);
}


private void
test7()
{
	int	nfails = 0;
	int	rtval;
	int	i;
	int	pathok = setupok;

	DBUG_ENTER("test7");

	errflag = FALSE;
	jumpok = 0;
	if ( (rtval = nftw("./ftwD1", f7, 3, 0)) != 0 ) {
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 7)
		PATH_TRACE;

	if ( errflag )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("function set by ftw called with 3rd argument");
		in_rpt("other than FTW_D or FTW_F");
	}
	else
		PATH_TRACE;

	for ( i = 0 ; i < NMR_OFILES ; i++ ) {
		if ( ft[i].flag == TRUE ) {
			if ( nfails++ == 0 )
				xx_rpt(FAILURE);
			in_rpt("function called by ftw set flag for file");
			in_rpt("if its parent directory was already visited.");
			in_rpt("flag was not set for %s", ft[i].name);
		} else
			PATH_TRACE;
		ft[i].flag = TRUE;	/* reset for later tests	*/
	}

	if ( nfails == 0 )
		PATH_XS_RPT(NMR_OFILES+3);

	DBUG_VOID_RETURN;
}


/*ARGSUSED*/
private int
f7(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	int	i;

	DBUG_ENTER("f7");
	DBUG_PRINTF("f7","Path passed is %s (%ld,%ld)",path,(long) statptr->st_dev,(long) statptr->st_ino);
	JUMP_TRACE;	/*	to prove that we have been called	*/


	for ( i = 0 ; i < NMR_OFILES ; i++ )	{

		if ( statptr == (struct stat *) NULL )
			continue;

		if ( statptr->st_dev == ft[i].device && statptr->st_ino == ft[i].inode ) {
			switch (i) {

			case 0 : /* ftwD1/ftw_d.2/ftw_t.4 */
				if( ft[1].flag == FALSE)
					ft[0].flag = FALSE;
				break;

			case 1 :
			case 4 :
			case 5 : /* ftwD1/ftw_d.2 */
				if( ft[6].flag == FALSE)
					ft[i].flag = FALSE;
				break;

			case 2 :
			case 3 : /* ftwD1/ftw_d.1/ftw_t.{3,4} */
				if( ft[4].flag == FALSE)
					ft[i].flag = FALSE;
				break;

			case 6 : /* ./ftwD1 */
				if(ft[1].flag == TRUE && ft[4].flag == TRUE && ft[5].flag == TRUE)
					ft[6].flag = FALSE;

				break;
			default :
				errflag = TRUE;
				break;
			}
			DBUG_RETURN(0);
		}
	}
	DBUG_RETURN(0);
}

private void
test9()
{
	DBUG_ENTER("test9");              
	(void) cppair(ch_t9, NULLFN, WAITTIME,0);
	DBUG_VOID_RETURN;                 
}                                         
	
private void
ch_t9()
{
	int	i;
	int	err;
	int	rtval;
	int	nfails = 0;
	struct stat statb;
	int	pathok = setupok;
	DBUG_ENTER("ch_t9");

	(void) setuid(getuid());
	(void) setgid(getgid());

/*
 * First test depth and offset fields of the FTW struct pointer.
 */
	nfails  = 0;
	jumpok = 0;
	if ( (rtval = nftw("./ftwD1", f9a, 3, 0)) != 0 ) {
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	} else
		PATH_TRACE;

	if(jumpok == NMR_OFILES)
		PATH_TRACE;
	
	for ( i = 0 ; i < NMR_OFILES ; i++ ) {
		if ( ft[i].flag == TRUE ) {
			if ( nfails++ == 0 )
				xx_rpt(FAILURE);
			in_rpt("ftw() function was called with base = %d, level = %d instead of base = %d, level = %d",
					ft_rbase[i],
					ft_rlevel[i],
					ft_base[i],
					ft_level[i]);
		} else
			PATH_TRACE;
		ft[i].flag = TRUE;	/* reset for later tests	*/
	}
	
		
	for ( i = 0 ; i < NMR_OFILES ; i++ ) {
		ft[i].flag = TRUE;	/* reset for later tests	*/
	}
	
/*
 * Search tree with link to verify that FTW_SL and FTW_DP are set properly.
 */
 
	if( symlink("../../ftwD1","./ftwD3/ftw_d.4/ftw_slink") == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("symlink(\"../../ftwD1\",\"./ftwD3/ftw_d.4/ftw_slink\") failed - errno = %d (%s)",err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	
/*
 *  Get <device,inode> pair for the newly created link.
 */

	if (lstat(lft[3].name, &statb) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed for file %s - errno = %d (%s)",lft[3].name,err,errname(err));
		DBUG_VOID_RETURN;
	} else {
		PATH_TRACE;
		lft[3].inode = statb.st_ino;
		lft[3].device = statb.st_dev;
		lft[3].flag  = TRUE;
	}

	flag = 0;
	jumpok = 0;
	if ( (rtval = nftw("./ftwD3", f9b, 3, FTW_PHYS | FTW_DEPTH )) != 0 ) {
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw() returned %d instead of 0", rtval);
	} else
		PATH_TRACE;
	
	if(jumpok == 3)
		PATH_TRACE;
		
	(void) unlink("./ftwD3/ftw_d.4/ftw_slink");

/* No  Read access - for FTW_DNR. */
	if ( chmod("./ftwD1/ftw_d.1", S_IWUSR|S_IXUSR|S_IWGRP|S_IXGRP|S_IWOTH|S_IXOTH) == SYSERROR ) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: chmod failed with errno %d (%s)", errno, errname(errno));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

/* No Execute access - stat() on files in directory will fail. */
	if ( chmod("./ftwD1/ftw_d.2", S_IWUSR|S_IWGRP|S_IWOTH|S_IRUSR|S_IRGRP|S_IROTH) == SYSERROR ) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: chmod failed with errno %d (%s)",
				errno, errname(errno));

		if ( chmod("./ftwD1/ftw_d.1", MODEANY) == SYSERROR )
			for ( i = 4 ; i <= COMP_TESTS ; i++ )
				(void) tet_delete(i, "chmod call failed");

		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	jumpok = 0;
	if ( (rtval = nftw("./ftwD1", f9, 3, 0)) != 0 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 5)
		PATH_TRACE;

	if ( (flag & FTWSL) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_SL and");
		in_rpt("correct first and second arguments when traversing");
		in_rpt("file tree");
	} else
		PATH_TRACE;

	if ( (flag & FTWDP) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_DP and");
		in_rpt("correct first and second arguments when traversing");
		in_rpt("file tree");
	}
	else
		PATH_TRACE;

	if ( (flag & FTWF) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_F and");
		in_rpt("correct first and second arguments when traversing");
		in_rpt("file tree");
	}
	else
		PATH_TRACE;

	if ( (flag & FTWD) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_D and");
		in_rpt("correct first and second arguments when traversing");
		in_rpt("file tree");
	}
	else
		PATH_TRACE;

	if ( (flag & FTWDNR) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_DNR and");
		in_rpt("correct first and second arguments when traversing");
		in_rpt("file tree");
	}
	else
		PATH_TRACE;

	if ( (flag & FTWNS) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_NS and");
		in_rpt("correct first argument when traversing");
		in_rpt("file tree");
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT( 16 + NMR_OFILES);

	if ( chmod("./ftwD1/ftw_d.1", MODEANY) == SYSERROR )
		for ( i = 4 ; i <= COMP_TESTS ; i++ )
			(void) tet_delete(i, "chmod call failed");

	if ( chmod("./ftwD1/ftw_d.2", MODEANY) == SYSERROR )
		for ( i = 4 ; i <= COMP_TESTS ; i++ )
			(void) tet_delete(i, "chmod call failed");

	DBUG_VOID_RETURN;


}


/*ARGSUSED*/
private int
f9(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	struct	stat	statb;

	DBUG_ENTER("f9");
	JUMP_TRACE;

	switch (type) 	{

	case	FTW_F:  /* ftw_t.1 */
		if ( stat(path, &statb) != SYSERROR
		    && statb.st_dev == statptr->st_dev
		    && statb.st_ino == statptr->st_ino
		    && statptr->st_dev == ft[5].device
		    && statptr->st_ino == ft[5].inode )
			flag |= FTWF;
		break;

	case	FTW_D:  /* ftwD1 */
		if ( stat(path, &statb) != SYSERROR
		    && statb.st_dev == statptr->st_dev
		    && statb.st_ino == statptr->st_ino
		    && statptr->st_dev == ft[6].device
		    && statptr->st_ino == ft[6].inode )
			flag |= FTWD;
		break;

	case	FTW_DNR: /* ftw_d.1 */
		if ( stat(path, &statb) != SYSERROR
		    && statb.st_dev == statptr->st_dev
		    && statb.st_ino == statptr->st_ino
		    && statptr->st_dev == ft[4].device
		    && statptr->st_ino == ft[4].inode )
			flag |= FTWDNR;
		break;

	case	FTW_NS:  /* ftw_d.2/ftw_t.4 */
		if ( strncmp(&path[ strlen(path) - 7 ], "ftw_t.4",7) == 0 )
			flag |= FTWNS;
		break;
	}

	DBUG_RETURN(0);

}

private int
f9a(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	int	i;

	DBUG_ENTER("f9a");
	DBUG_PRINTF("f9a","Path passed is %s ( %ld , %ld )",path,(long) statptr -> st_dev, (long) statptr -> st_ino);
	DBUG_PRINTF("f9a","path = %s",path);

	for ( i = 0 ; i < NMR_OFILES ; i++ ) {
		if ( statptr == (struct stat *) NULL ) {
			continue;
		}

		if ( statptr->st_dev == ft[i].device && statptr->st_ino == ft[i].inode ){
			ft_rlevel[i] = info -> level;
			ft_rbase[i] = info -> base;
			if(ft_level[i] == ft_rlevel[i] && ft_base[i] == ft_rbase[i]) {
				DBUG_PRINTF("f9a","FTW.level is %d, FTW.base is %d",ft_level[i], ft_base[i]);
				DBUG_PRINTF("f9a","FTW.level is %d, FTW.base is %d",info->level, info->base);
				ft[i].flag = FALSE;
			}
			JUMP_TRACE;
			DBUG_RETURN(0);
		} 
	}
	                      
	DBUG_RETURN(0);       
}	                      
private int
f9b(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{

	DBUG_ENTER("f9b");
	DBUG_PRINTF("f9b","Path passed is %s ( %ld , %ld )",path,(long) statptr -> st_dev, (long) statptr -> st_ino);
	DBUG_PRINTF("f9b","path = %s",path);

/*
 * Mark the subdirectory as visited.
 */

	if ( statptr->st_dev == lft[2].device && statptr->st_ino == lft[2].inode ){
		lft[2].flag = FALSE;
		JUMP_TRACE;
	}

/*
 * See if we have the  root directory's <inode,device> pair.
 */
	if ( statptr->st_dev == lft[0].device && statptr->st_ino == lft[0].inode ){
		if(type == FTW_DP && lft[2].flag == FALSE) {
			flag |= FTWDP;
			lft[2].flag = TRUE;
		DBUG_PRINTF("f9b","Flag goes wild!");
		}
		JUMP_TRACE;
	}

/*
 * See if we have the symbolic link's <inode,device> pair.
 */

	if ( statptr->st_dev == lft[3].device && statptr->st_ino == lft[3].inode ){
		if(type == FTW_SL) {
			flag |= FTWSL;
		} 
		JUMP_TRACE;	
	}

	DBUG_RETURN(0);       
}	                      

private void
test10()
{
	DBUG_ENTER("test10");
	(void) cppair(ch_t10,NULLFN,WAITTIME,0);
	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	int	i;
	int	rtval;
	int	nfails = 0;
	int	pathok = setupok;
	DBUG_ENTER("ch_t10");
	
	(void) setuid(getuid());
	(void) setgid(getgid());

	if ( chmod("./ftwD1/ftw_d.1", S_IWUSR|S_IXUSR|S_IWGRP|S_IXGRP|S_IWOTH|S_IXOTH) == SYSERROR ) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: chmod failed with errno %d (%s)", errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errflag = FALSE;
	flag = 0;
	jumpok = 0;
	if ( (rtval = nftw("./ftwD1/ftw_d.1", f10, 2,0)) != 0 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(./ftwD1/ftw_d.1, ..., 2) returned %d instead of 0",
					rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 1)
		PATH_TRACE;

	if ( errflag )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw on file tree with read-only root");
		in_rpt("called function with 3rd argument other");
		in_rpt("than FTW_DIR");
	}
	else
		PATH_TRACE;

	if ( (flag & FTWDNR) == 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw did not call function with argument FTW_DNR");
		in_rpt("although the root of the file tree given was");
		in_rpt("read-only");
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(5);

	if ( chmod("./ftwD1/ftw_d.1", MODEANY) == SYSERROR )
		for ( i = 5 ; i <= COMP_TESTS ; i++ )
			(void) tet_delete(i, "chmod call failed");

	DBUG_VOID_RETURN;

}

private int
f10(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	DBUG_ENTER("f10");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	if ( type == FTW_DNR )
		flag |= FTWDNR;
	else
		errflag = TRUE;

	DBUG_RETURN(0);

}


private void
test11()
{
	int	rtval;
	int	nfails = 0;
	int	nopen;
	int	pathok = setupok;

	DBUG_ENTER("test11");

	nopen = n_open_fds();

	jumpok = 0;
	if ( (rtval = nftw("./ftwD1", f11, 3, 0)) != 0 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0",
					rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 7)
		PATH_TRACE;

	if ( ftopen - nopen > 2 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw opened %d file descriptors instead of 2 to",
				ftopen - nopen);
		in_rpt("traverse two levels of the file tree");
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;

}


/*ARGSUSED*/
private int
f11(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{

	DBUG_ENTER("f11");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	if ( statptr->st_dev == ft[3].device && statptr->st_ino == ft[3].inode )
		ftopen = n_open_fds();
	DBUG_RETURN(0);

}


private void
test12()
{
	int	rtval;
	int	nfails = 0;
	int	nopen;
	int	pathok = setupok;

	DBUG_ENTER("test12");

	nopen = n_open_fds();

	jumpok = 0;
	if ( (rtval = nftw("./ftwD1", f12, 1, 0)) != 0 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\"./ftwD1\", ..., 1) returned %d instead of 0",
					rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == 7)
		PATH_TRACE;

	if ( ftopen - nopen > 1 )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 1) opened %d file descriptors",
				ftopen - nopen);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;

}


/*ARGSUSED*/
private int
f12(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{

	DBUG_ENTER("f12");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	if ( statptr->st_dev == ft[3].device && statptr->st_ino == ft[3].inode )
		ftopen = n_open_fds();

	DBUG_RETURN(0);

}

private void
test13()
{
	int	rtval;
	int	nfails = 0;
	int	pathok = setupok;

	DBUG_ENTER("test13");

	errflag = FALSE;
	flag = FALSE;
	jumpok = 0;
	if ( (rtval = nftw("./ftwD1/ftw_d.1", f13, 2,0)) != 99 )
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(./ftwD1/ftw_d.1, ..., 2) returned %d instead of 99",
					rtval);
		in_rpt("which was the return value of the function set by ftw");
	}
	else
		PATH_TRACE;

	if (jumpok == 1)
		PATH_TRACE;

	if ( errflag )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("function called by ftw more than once although first");
		in_rpt("call returned nonzero value of 99");
	}
	else
		PATH_TRACE;

	if ( flag != TRUE )
	{
		if ( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("function set by ftw not called for root of file tree");
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;

}


/*ARGSUSED*/
private int
f13(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	DBUG_ENTER("f13");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	if ( statptr->st_dev == ft[4].device && statptr->st_ino == ft[4].inode )
		flag = TRUE;
	else
		errflag = TRUE;

	DBUG_RETURN(99);
}

private void
test14()
{
	int	rtval, err;
	char	**longname;
	long 	len;
	int	pathok = setupok;
	DBUG_ENTER("test14");

	globok = 0;
	if (crdir(t14_dir, MODEANY, (int) getuid(), (int) getgid()) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t14_dir);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;

	longname = pathtoolng(t14_dir);
	DBUG_PRINTF("t14","nametoolng returned %s",longname[1]);
	len = (long) strlen(longname[1]);
	DBUG_PRINTF("t14","len(longname[1]) = %ld", len);
	DBUG_PRINTF("t14","pathconf(\".\",_PC_PATH_MAX) = %ld",pathconf(longname[1],_PC_PATH_MAX));
	

	if (longname == (char **)0) {
		xx_rpt(DELETION);
		in_rpt("pathtoolng(\"%s\") failed", t14_dir);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	if (longname[1] == (char *)0) {
		xx_rpt(DELETION);
		in_rpt("pathtoolng(\"%s\") failed", t14_dir);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	rtval = nftw(longname[1], f14, 2,0);
	if (rtval != -1 || errno != ENAMETOOLONG) {
		err = errno;
		xx_rpt(FAILURE);	
		in_rpt("nftw() returned %d with errno = %d (%s) instead of -1 with errno = ENAMETOOLONG.",rtval,err,errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
		
	PATH_XS_RPT(4);

	clntoolng();
	(void) rmdir(t14_dir);

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f14(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	DBUG_ENTER("f14");
	DBUG_RETURN(1);
}

private void
test15()
{
	int	slen, rtval, err;
	int	fail = 0;
	char	*longname;
	char	**toolng;
	int	pathok = setupok;
	DBUG_ENTER("test15");

	toolng = nametoolng();

	if (toolng == (char **)0) {
		xx_rpt(DELETION);
		in_rpt("longname() returned NULL.");
		DBUG_VOID_RETURN;
	} else if (*(toolng[1]) == '\0')	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_NO_TRUNC is not defined");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	longname=toolng[1];

	slen = strlen(longname);
	longname[slen-5] = '\0';
	if (crdir(longname, MODEANY, (int) getuid(), (int) getgid()) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s, errno = %s", longname,errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	longname[slen-5] = 'X';
	rtval = nftw(longname, f15, 2, 0);
	if (rtval != -1 || errno != ENAMETOOLONG) {
		fail++;
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("nftw() returned %d with errno = %d (%s) instead of -1 with errno = ENAMETOOLONG.",
				rtval,
				err,
				errname(err));
	} else {
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	longname[slen-5] = '\0';
	(void) rmdir(longname);
	clntoolng();
	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f15(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	DBUG_ENTER("f15");

	DBUG_RETURN(1);

}

private void
test16()
{
	int	rtval, err;
	int	pathok = setupok;

	DBUG_ENTER("test16");

	testfail = 0;
	if ( (rtval = nftw(t16_dir, f16, 2, 0)) != -1 )
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("ftw(%s, ..., 2) returned %d instead of -1", t16_dir,
					rtval);
	}
	else if (errno != ENOENT)
	{
		testfail++;
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("ftw(%s, ..., 2) set errno to %d (%s) instead of ENOENT",
			t16_dir, err, errname(err));
	}
	else
	{
		PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f16(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	DBUG_ENTER("f16");

	DBUG_RETURN(1);

}

private void
test17()
{
	int	rtval, err;
	int	pathok = setupok;

	DBUG_ENTER("test17");

	testfail = 0;
	if ( (rtval = nftw("", f17, 2, 0)) != -1 )
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\"\", ..., 2) returned %d instead of -1", rtval);
	}
	else if (errno != ENOENT)
	{
		testfail++;
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("ftw(\"\", ..., 2) set errno to %d (%s) instead of ENOENT",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f17(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	DBUG_ENTER("f17");

	DBUG_RETURN(1);

}

private void
test18()
{
	int	rtval, err;
	int	pathok = setupok;

	DBUG_ENTER("test18");

	if (crfile(t18_file, MODEANY, (int) getuid(), (int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t18_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( (rtval = nftw(t18a_file, f18, 2, 0)) != -1 )
	{
		xx_rpt(FAILURE);
		in_rpt("ftw(%s, ..., 2) returned %d instead of -1", t18a_file, rtval);
		in_rpt("where %s was a plain file", t18_file);
	} else if (errno != ENOTDIR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("ftw(%s, ..., 2) set errno to %d (%s) instead of ENOTDIR",
			t18a_file, err, errname(err));
		in_rpt("where %s was a plain file", t18_file);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) unlink(t18_file);

	DBUG_VOID_RETURN;
}

/*ARGSUSED*/
private int
f18(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct FTW	*info;
{
	DBUG_ENTER("f18");

	DBUG_RETURN(1);

}

private void
test2()
{
	int err;
	int rtval;
	int nfails = 0;
	int i;
	int pathok = setupok;
	DBUG_ENTER("test2");
	
	if( symlink("../../ftwD1","./ftwD3/ftw_d.4/ftw_slink") == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("symlink(\"../../ftwD1\",\"./ftwD3/ftw_d.4/ftw_slink\") failed - errno = %d (%s)",err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (rtval = nftw("./ftwD3", f2, 3, FTW_PHYS )) != 0 ){   /* Will NOT follow symbolic links */
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	(void) unlink("./ftwD3/ftw_d.4/ftw_slink");
	
	for ( i = 0 ; i < NMR_OFILES ; i++ ) {       /* Verify that none of the files linked to were passed to f2() */
		if ( ft[i].flag != TRUE ) {
			if ( nfails++ == 0 ) 
				xx_rpt(FAILURE);
			in_rpt("nftw() crossed symbolic link to %s",ft[i].name);
		} else
			PATH_TRACE;

		ft[i].flag = TRUE;	/* reset for later tests	*/
	}

	if ( nfails == 0 )
		PATH_XS_RPT(NMR_OFILES+2);

	DBUG_VOID_RETURN;
}

private int
f2(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	int	i;
	DBUG_ENTER("f2");
	DBUG_PRINTF("f2","Path passed is %s (%ld,%ld)",path,(long) statptr->st_dev,(long) statptr->st_ino);

	for ( i = 0 ; i < NMR_OFILES ; i++ ) {
		if ( statptr == (struct stat *) NULL ) {
		DBUG_PRINTF("f2", "statptr was NULL for file %s",path);
			continue;
		}

		if ( statptr->st_dev == ft[i].device && statptr->st_ino == ft[i].inode ){
			ft[i].flag = FALSE;
			JUMP_TRACE;
			DBUG_RETURN(0);
		} 
	}

      	for ( i = 0 ; i < NMR_LFILES ; i++ ) {
		if ( statptr == (struct stat *) NULL ) {
			continue;
		}

		if ( statptr->st_dev == lft[i].device && statptr->st_ino == lft[i].inode ){
			lft[i].flag = FALSE;
			JUMP_TRACE;
			DBUG_RETURN(0);
		} 
	}
	                      
	DBUG_RETURN(0);       
}	                      
                              

private void
test3()
{
	int err;
	int rtval;
	int nfails = 0;
	int i;
	struct stat statb;
	int pathok = setupok;
	DBUG_ENTER("test3");
	
	if( symlink("../../ftwD1","./ftwD3/ftw_d.4/ftw_slink") == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("symlink(\"../../ftwD1\",\"./ftwD3/ftw_d.4/ftw_slink\") failed - errno = %d (%s)",err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (rtval = nftw("./ftwD3", f3, 3, 0 )) != 0 ){    /* traverse tree with links */
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	for ( i = 0 ; i < NMR_OFILES ; i++ ) {                            /* Verify the linked to directory was traversed */
		if ( ft[i].flag == TRUE ) {
			if ( nfails++ == 0 ) 
				xx_rpt(FAILURE);
			in_rpt("nftw() did not cross symbolic link to %s",ft[i].name);
		} else
			PATH_TRACE;
		ft[i].flag = TRUE;	/* reset for later tests	*/
	}

	DBUG_PRINTF("t3", "About to construct a tree with a cycle");
/* 
 * Construct a directory non-tree with a cycle.
 */

	if(unlink("./ftwD3/ftw_d.4/ftw_slink") == SYSERROR) {
		err =errno;
		xx_rpt(DELETION);
		in_rpt("Unlink failed of ftw_slink - errno = %d (%s).", err, errname(err));
		DBUG_VOID_RETURN;
	} else 
		PATH_TRACE;
	
/*
 * Create link in sub-tree back to the directory root.
 */
	if( symlink("../../ftwD3","./ftwD3/ftw_d.4/ftw_slink") == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("symlink(\"../../ftwD3\",\"./ftwD3/ftw_d.4/ftw_slink\") failed - errno = %d (%s)",err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

/*
 *  Get <device,inode> pair for the newly created link.
 */

	if (stat(lft[3].name, &statb) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed for file %s - errno = %d (%s)",lft[3].name,err,errname(err));
		DBUG_VOID_RETURN;
	} else {
		PATH_TRACE;
		lft[3].inode = statb.st_ino;
		lft[3].device = statb.st_dev;
		lft[3].flag  = TRUE;
	}

	if ( (rtval = nftw("./ftwD3", f3a, 3, 0 )) != 0 ){    /* traverse tree with a cyclic link */
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	(void) unlink("./ftwD3/ftw_d.4/ftw_slink");
	

/*
 * Verify that all but the link itself were re-visited.
 */
	for(i=0; i< NMR_LFILES; i++) {
		if ( i !=3 && lft[i].flag == TRUE ) {
			if ( nfails++ == 0 ) 
				xx_rpt(FAILURE);
			in_rpt("nftw() crossed symbolic link to %s",lft[i].name);
		} else
			PATH_TRACE;
		lft[i].flag = TRUE;	/* reset for later tests	*/
	}

	if ( nfails == 0 )
		PATH_XS_RPT(6 + NMR_LFILES + NMR_OFILES);

	DBUG_VOID_RETURN;
}

private int
f3(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	int	i;
	DBUG_ENTER("f3");
	DBUG_PRINTF("f3","Path passed is %s (%ld,%ld)",path,(long) statptr->st_dev,(long) statptr->st_ino);

	for ( i = 0 ; i < NMR_OFILES ; i++ ) {
		if ( statptr == (struct stat *) NULL ) {
		DBUG_PRINTF("f3", "statptr was NULL for file %s",path);
			continue;
		}

		if ( statptr->st_dev == ft[i].device && statptr->st_ino == ft[i].inode ){
			ft[i].flag = FALSE;
			JUMP_TRACE;
			DBUG_RETURN(0);
		} 
	}
	DBUG_RETURN(0);       
}	                      

private int
f3a(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	int	i;
	DBUG_ENTER("f3a");
	DBUG_PRINTF("f3a","Path passed is %s (%ld,%ld)",path,(long) statptr->st_dev,(long) statptr->st_ino);

	for ( i = 0 ; i < NMR_LFILES ; i++ ) {
		if ( statptr == (struct stat *) NULL ) {
		DBUG_PRINTF("f3a", "statptr was NULL for file %s",path);
			continue;
		}

		if ( statptr->st_dev == lft[i].device && statptr->st_ino == lft[i].inode ){
/*
 * Check that the file was not visited before.
 */
			if(lft[i].flag == TRUE)
				lft[i].flag = FALSE;
			JUMP_TRACE;
			DBUG_RETURN(0);
		} 
	}
	DBUG_RETURN(0);       
}	                      


private void
test8()
{
	int err, rtval;
	int pathok = setupok;
	DBUG_ENTER("test8");
	
	jumpok = 0;
	testfail = 0;
	t8_repcount = 0;

	errno = 0;
	if (getcwd(t8_cwd, sizeof t8_cwd) == (char *) 0) {
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("getcwd() failed, errno = %d (%s)",
			errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	DBUG_PRINTF("test8", "starting directory is %s", t8_cwd);

	if ((rtval = nftw("./ftwD1", f8, 3, FTW_CHDIR)) != 0) { 
		err = errno;
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("ftw(..., FTW_CHDIR) returned %d instead of 0",
			rtval);
		in_rpt("errno = %d (%s)", err, errname(err));
	} else
		PATH_TRACE;

	if (t8_repcount != NMR_OFILES) {
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("expected nftw(..., FTW_CHDIR) to call the reporting function %d times, observed %d calls", NMR_OFILES, t8_repcount);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (jumpok == 3 * NMR_OFILES)
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private int
f8(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	int	err, len;
	char	*p;
	char	fullpath[1026];
	char	cwdpath[1026];
	char	parent_dir[1026];
	struct stat pdstat;
	struct stat dotstat;

	DBUG_ENTER("f8");
	t8_repcount++;

	DBUG_PRINTF("f8","Path passed is %s (%ld,%ld)",path,(long) statptr->st_dev,(long) statptr->st_ino);

	p = fullpath;
	switch (*path) {
	default:
		(void) sprintf(p, "%.*s/",
			(int) (sizeof fullpath - 2), t8_cwd);
		len = strlen(p);
		p += len;
		/* fall through */
	case '/':
		(void) sprintf(p, "%.*s",
			(int) (&fullpath[sizeof fullpath] - p) - 1, path);
		break;
	}
	DBUG_PRINTF("f8", "absolute path is %s", fullpath);

	errno = 0;
	if (getcwd(cwdpath, sizeof cwdpath) == (char *) 0) {
		in_rpt("getcwd() failed, errno = %d (%s)",
			errno, errname(errno));
		(void) strcpy(cwdpath, "<unknown>");
	}
	DBUG_PRINTF("f8", "CWD = %s", cwdpath);

	(void) strcpy(parent_dir, fullpath);
	if ((p = strrchr(parent_dir, '/')) == (char *) 0)
		(void) strcpy(parent_dir, ".");
	else
		*p = '\0';
	if (stat(parent_dir, &pdstat) == SYSERROR) {
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed, errno = %d (%s)",
			parent_dir, err, errname(err));
		DBUG_RETURN(0);
	}
	else
		JUMP_TRACE;

	if (stat(".", &dotstat) == SYSERROR) {
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(\".\") failed, errno = %d (%s)",
			err, errname(err));
		DBUG_RETURN(0);
	}
	else
		JUMP_TRACE;

	if (
		(pdstat.st_dev != dotstat.st_dev) ||
		(pdstat.st_ino != dotstat.st_ino)
	) {
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("nftw(..., FTW_CHDIR) did not change directory to %s before reporting on %s", parent_dir, path);
		in_rpt("(current directory is %s)", cwdpath);
	} else
		JUMP_TRACE;

	DBUG_RETURN(0);
}	                      

private void
test4()
{
	int	nfails = 0;
	int	rtval;
	int	i;
	int	pathok = setupok;

	DBUG_ENTER("test4");

	jumpok = 0;
	if ( (rtval = nftw("./ftwD2", f4, 3, FTW_MOUNT )) != 0 ){    /* Will not cross a mount point. */
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == NMR_NMNTD)
		PATH_TRACE;

	for ( i = 0 ; i < NMR_MNTD ; i++ ) {
		if ( mft[ mounted[i] ].flag == FALSE ) {
			if ( nfails++ == 0 ) 
				xx_rpt(FAILURE);
			in_rpt("ftw() crossed mount point for file %s",mft[i].name);
		} else
			PATH_TRACE;
	}

	for(i=0; i < NMR_MNTD; i++)
		mft[i].flag = TRUE;
		
	if ( nfails == 0 )
		PATH_XS_RPT(NMR_MNTD + 2);

	DBUG_VOID_RETURN;
}


/*ARGSUSED*/
private int
f4(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	int	i;

	DBUG_ENTER("f4");
	DBUG_PRINTF("f4","Path passed is %s ( %ld , %ld )",path,(long) statptr -> st_dev, (long) statptr -> st_ino);

	for ( i = 0 ; i < NMR_MFILES ; i++ ) {
		if ( statptr == (struct stat *) NULL ) {
			continue;
		}

		if ( statptr->st_dev == mft[i].device && statptr->st_ino == mft[i].inode ){
			JUMP_TRACE;
			mft[i].flag = FALSE;
			DBUG_RETURN(0);
		} 
	}
	DBUG_RETURN(0);       
}	                      


private void
test5()
{
	int	nfails = 0;
	int	rtval;
	int	i;
	int	pathok = setupok;

	DBUG_ENTER("test5");

	jumpok = 0;
	if ( (rtval = nftw("./ftwD2", f5, 3, 0 )) != 0 ){ 
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("ftw(\".\", ..., 3) returned %d instead of 0", rtval);
	}
	else
		PATH_TRACE;

	if (jumpok == NMR_MFILES)
		PATH_TRACE;

	for ( i = 0 ; i < NMR_OFILES ; i++ ) {
		if ( mft[i].flag == TRUE ) {
			if ( nfails++ == 0 ) 
				xx_rpt(FAILURE);
			in_rpt("function set by ftw() not called for %s",mft[i].name);
		}
		PATH_TRACE;
		mft[i].flag = TRUE;	/* reset for later tests	*/
	}

	if ( nfails == 0 )
		PATH_XS_RPT(NMR_MFILES+2);

	DBUG_VOID_RETURN;
}


private int
f5(path, statptr, type, info)
	char		*path;
	struct	stat	*statptr;
	int		type;
	struct	FTW	*info;
{
	int	i;

	DBUG_ENTER("f5");

	for ( i = 0 ; i < NMR_MFILES ; i++ ) {
		if ( statptr == (struct stat *) NULL ) {
			continue;
		}

		if ( statptr->st_dev == mft[i].device && statptr->st_ino == mft[i].inode ){
			DBUG_PRINTF("f5","Path passed is %s ( %ld , %ld )",path,(long) statptr -> st_dev, (long) statptr -> st_ino);
			JUMP_TRACE;
			mft[i].flag = FALSE;
			DBUG_RETURN(0);
		} 
	}
	                      
	DBUG_RETURN(0);       
}	                      
                              

