/* dv (#)SOLTESTS/tset/LSB.os/genuts/openlog_L/openlog_L.c	1.5 99/04/26 */
/*
* (C) Copyright 1998 UniSoft Limited.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*/
#ifndef	lint
#define	MAIN
#ifdef	MAIN
static	char sccsid[] = "(#)SOLTESTS/tset/LSB.os/genuts/openlog_L/openlog_L.c	1.5 99/04/26";
char	*copyright[] = {
		"(C) Copyright 1998 UniSoft Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.openlog_L

PROJECT:	Solaris ABI Test Suite

DESCRIPTION:
	Testset for the openlog() function.

AUTHOR:		Matthew Hails, UniSoft Ltd.
DATE CREATED:	November 1998

************************************************************************/

#include <std.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>
#include <ctype.h>
#include <fcntl.h>
#include <float.h>
#include <langinfo.h>
#include <limits.h>
#include <locale.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <syslog.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>
#include <wchar.h>
#include <wait_child.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>

#ifdef UNDEF_MACROS
#undef openlog
#endif

#define NO_TESTS 3

#define WAITTIME		(5 * SPEEDFACTOR)
#define WAITLOGDSECS		30
#define MAXSTRINGSIZE		512
#define TIMESTAMPFMT		"%b %e %H:%M"
#define TIMESTAMPBUFLEN		13

#define CALL_SYSLOG		1
#define	CALL_SPRINTF		2
#define CMP_PLAINSTRING		1
#define CMP_USE_SPRINTF	2

#define TOPT_ERRNOVAL(err)	(((err) & 0xff) << 8)
#define TOPT_GETERRNO(topt)	(((topt) >> 8) & 0xff)

/* External prototypes */
char *errname(int);

/* Local prototypes */
static pid_t getsyslogdpid(void);
static int setup_syslogd(char *debuglogfile);
static int cleanup_syslogd(void);
static void prepare(void);
static void clean(void);
static int setupsyslog(char *cmd, char *arg);
static void comntest(unsigned testopts, char *cmparg,
	int (*chkfn)(char *, void *), void *chkarg, char *ident,
	void (*runfn)(int, char **), char *message, char *uselog);
static void gettimestamp(char *tbuf, int buflen);
static int checklong(char *msg, void *chkarg);
static int checkint(char *msg, void *chkarg);
static int checkshort(char *msg, void *chkarg);
static int checktimestamp(char *msg, void *chkarg);

/* External data declarations */
extern int ex_code;
extern int sg_code;
extern char **_environ;

/* Global static data */
static char setupprog[] = "../setupsyslog";
static char testmsg[] = "This is only a test";
static char logfile[] = "./debug-messages";
static char *logfiles[5] = { "./debug-messages.0",
			     "./debug-messages.1",
			     "./debug-messages.2",
			     "./debug-messages.3",
			     "./debug-messages.4" };

static void test1(void);
static void sub_test1(int type, char **msgptr);
static void test2(void);
static void sub_test2(int type, char **msgptr);
static void test3(void);
static void sub_test3(int type, char **msgptr);

/*
 * Test case startup function.
 */
static void
prepare(void)
{
	int fd;
	int i;

	DBUG_ENTER("prepare");

	for (i = 0; i <= 4; i++)
	{
		/* Create log file */
		unlink(logfiles[i]);

		fd = open(logfiles[i], O_WRONLY|O_CREAT|O_EXCL,
			S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
		if (fd == -1)
		{
			in_rpt("Error creating %s", logfiles[i]);
			xx_all(DELETION, "Can't create log file to use with syslogd");
			DBUG_VOID_RETURN;
		}
	}

	/* Start up syslogd and tweak config file */
	if (setupsyslog("startup", logfile) != 0)
	{
		xx_all(DELETION, "Error starting/configuring syslogd");
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

/*
 * Test case cleanup function.
 */
static void
clean(void)
{
	int i;

	DBUG_ENTER("clean");

	/* Tidy up syslog conf file etc. */
	setupsyslog("cleanup", NULL);

	/* Remove temporary log file */
	for (i = 0; i <= 4; i++)
		unlink(logfiles[i]);

	DBUG_VOID_RETURN;
}

static int
setupsyslog(char *cmd, char *arg)
{
	static char *args[4];

	DBUG_ENTER("setupsyslog");

	tet_child = fork();
	switch (tet_child)
	{
	case -1:
		in_rpt("fork failure in startup");
		DBUG_RETURN(-1);
	case 0:
		tet_setcontext();
		args[0] = setupprog;
		args[1] = cmd;
		args[2] = arg;
		args[3] = NULL;
		tet_exec(args[0], args, _environ);
		in_rpt("Failed to exec %s", setupprog);
		DBUG_EXIT(1);
	}

	switch (wait_child(WAITTIME))
	{
	case CHLD_EXIT:
		if (ex_code == 0)
		{
			/* Give syslogd time to sort itself out */
			sleep(WAITLOGDSECS * 2);
		}
		else
		{
			in_rpt("%s exited with status %d", setupprog, ex_code);
		}
		DBUG_RETURN(ex_code);
	case CHLD_ABRT:
		in_rpt("%s terminated by signal %d", setupprog, sg_code);
		DBUG_RETURN(-1);
	case CHLD_LIVE:
		in_rpt("%s timed out", setupprog);
		break;
	case CHLD_STOP:
		in_rpt("%s stopped", setupprog);
		break;
	default:
		in_rpt("Error in waitpid()");
		break;
	}

	kill(tet_child, SIGKILL);
	DBUG_RETURN(-1);
}

/*
 * Common code for testing syslog().
 *
 * cmpform can be CMP_PLAINSTRING or CMP_USE_SPRINTF.
 * cmparg is the string to compare (CMP_PLAINSTRING) or the vsprintf format
 * to use (CMP_USE_SPRINTF).
 *
 * chkfn is a pointer to a function to call to perform additional checks.
 * If NULL it is not used.
 * chkarg is passed to chkfn.
 */
static void
comntest(unsigned testopts, char *cmparg,
	int (*chkfn)(char *, void *), void *chkarg, char *ident,
	void (*runfn)(int, char **), char *expmsg, char *uselog)
{
	int pathok = 0;
	int count;
	int err;
	FILE *fp;
	struct stat statbefore;
	struct stat statafter;
	char gotmsg[MAXSTRINGSIZE];
	char *msgptr;
	int foundit;
	va_list ap;

	DBUG_ENTER("comntest");

	/* Reset logging attributes and priority */

	setlogmask(LOG_MASK(LOG_DEBUG|LOG_WARNING));

	/* Open log file for reading */
	fp = fopen(uselog, "r");
	if (fp == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("Error opening log file (%s) for reading (%s)",
			uselog, errname(err));
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
	}

	/* Seek to the end of the log file */
	if (fseek(fp, 0L, SEEK_END) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fseek() failed, errno = %s", errname(err));
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}

	/* Retrieve modification time using stat() */
	if (stat(uselog, &statbefore) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed, errno = %s", errname(err));
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}

	sleep(1);

	/* Call syslog() to log the message */
	errno = TOPT_GETERRNO(testopts);
	runfn(CALL_SYSLOG, &msgptr);

	/* Loop - waiting for syslogd to log message */
	for (count = 1; count < WAITLOGDSECS; count++)
	{
		if (stat(uselog, &statafter) != 0)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("stat() failed, errno = %s (%d)",
				errname(err), err);
			(void) fclose(fp);
			DBUG_VOID_RETURN;
		}

		if (statafter.st_mtime > statbefore.st_mtime)
			break;

		sleep(1);
	}

	if (statafter.st_mtime > statbefore.st_mtime)
	{
		PATH_TRACE;
	}
	else
	{
		xx_rpt(FAILURE);
		in_rpt("syslog() failed to send message to logging facility");
		in_rpt("  Message = \"%s\"", expmsg);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}

	/* Read latest addition to log file and verify syslog() message was
	 * logged with correct format.
	 */
	foundit = 0;

	while (fgets(gotmsg, sizeof(gotmsg), fp) != NULL)
	{
		if (strstr(gotmsg, expmsg) != NULL)
		{
			foundit = 1;
			break;
		}
	}

	if (ferror(fp))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fgets() failed, errno = %s", errname(err));
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
	}

	if (foundit)
	{
		PATH_TRACE;
	}
	else
	{
		xx_rpt(FAILURE);
		in_rpt("syslog() failed to format argument correctly");
		in_rpt("  Received: \"%s\"", gotmsg);
		in_rpt("  Expected: \"%s\"", expmsg);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}

	/* If the caller provided a function to perform extra checks, call
	 * it here.
	 */
	if (chkfn == NULL)
	{
		PATH_TRACE;
	}
	else if ((*chkfn)(gotmsg, chkarg) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}

	PATH_XS_RPT(7);
	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

static void
gettimestamp(char *tbuf, int buflen)
{
        time_t tod;

	tod = time(NULL);
	strftime(tbuf, buflen, TIMESTAMPFMT, localtime(&tod));
}

static int
checklong(char *msg, void *chkarg)
{
	long expval;
	long gotval;

	expval = (long)strlen(testmsg);
	gotval = *(long *)chkarg;

	if (gotval == expval)
		return 0;

	xx_rpt(FAILURE);
	in_rpt("Correct length not written to long argument (%ld expecting %ld)",
		gotval, expval);
	return -1;
}

static int
checkint(char *msg, void *chkarg)
{
	int expval;
	int gotval;

	expval = (int)strlen(testmsg);
	gotval = *(int *)chkarg;

	if (gotval == expval)
		return 0;

	xx_rpt(FAILURE);
	in_rpt("Correct length not written to int argument (%d expecting %d)",
		gotval, expval);
	return -1;
}

static int
checkshort(char *msg, void *chkarg)
{
	int expval;
	int gotval;

	expval = (int)strlen(testmsg);
	gotval = *(short *)chkarg;

	if (gotval == expval)
		return 0;

	xx_rpt(FAILURE);
	in_rpt("Correct length not written to short argument (%d expecting %d)",
		gotval, expval);
	return -1;
}

/*
 * Check timestamp.
 * Timestamp must match time before or after syslog call, excluding seconds.
 * chkarg is time before call.
 */
static int
checktimestamp(char *msg, void *chkarg)
{
	char buf[TIMESTAMPBUFLEN];

	if (strstr(msg, chkarg) != NULL)
		return 0;

	gettimestamp(buf, sizeof(buf));

	if (strstr(msg, buf) != NULL)
		return 0;

	xx_rpt(FAILURE);
	in_rpt("Failed to find correct timestamp in logged message");
	in_rpt("  Message line: \"%s\"", msg);
	in_rpt("  Expected: \"%s\" or \"%s\"", chkarg, buf);
	return -1;
}


/********************* Beginning of test cases *********************/

static void
test1(void)
{
	int pathok = 0;
	char expmsg[MAXSTRINGSIZE];
	char *msgptr;

	DBUG_ENTER("test1");

	msgptr = expmsg;
	sub_test1(CALL_SPRINTF, &msgptr);
	
	openlog("test-syslog-1", LOG_ODELAY, LOG_DEBUG);

	comntest(0U, testmsg, checktimestamp, "test-syslog-1",
		(char *) NULL, sub_test1, msgptr, logfiles[0]);

	DBUG_VOID_RETURN;
}

static void
sub_test1(int type, char **result)
{

	if (type == CALL_SPRINTF)
	{
		(void) sprintf(*result, testmsg);
	}

	if (type == CALL_SYSLOG)
	{
		syslog(LOG_DEBUG, testmsg);
	}

	return;
}
	
static void
test2(void)
{
	int pathok = 0;
	char expmsg[MAXSTRINGSIZE];
	char *msgptr;

	DBUG_ENTER("test2");

	msgptr = expmsg;
	sub_test2(CALL_SPRINTF, &msgptr);
	
	openlog("test-syslog-2", LOG_PID|LOG_ODELAY, LOG_LOCAL2);

	comntest(0U, testmsg, NULL, NULL,
		(char *) NULL, sub_test2, msgptr, logfiles[2]);

	DBUG_VOID_RETURN;
}

static void
sub_test2(int type, char **result)
{

	if (type == CALL_SPRINTF)
	{
		(void) sprintf(*result, "%d", (int) getpid());
	}

	if (type == CALL_SYSLOG)
	{
		syslog(LOG_WARNING, testmsg);
	}

	return;
}

static void
test3(void)
{
	int pathok = 0;
	char expmsg[MAXSTRINGSIZE];
	char *msgptr;

	DBUG_ENTER("test3");

	msgptr = expmsg;
	sub_test3(CALL_SPRINTF, &msgptr);
	
	openlog("test-syslog-3", LOG_ODELAY, LOG_LOCAL2);

	comntest(0U, testmsg, NULL, NULL,
		"test-syslog-3", sub_test3, msgptr, logfiles[2]);

	DBUG_VOID_RETURN;
}

static void
sub_test3(int type, char **result)
{

	if (type == CALL_SPRINTF)
	{
		(void) sprintf(*result, testmsg);
	}

	if (type == CALL_SYSLOG)
	{
		syslog(LOG_DEBUG, testmsg);
	}

	return;
}
	
/************************ End of test cases ************************/

int no_tests = NO_TESTS;

void (*setfunc)() = prepare;
void (*clnfunc)() = clean;

struct tet_testlist tet_testlist[] =
{
	{ test1, 1 },
	{ test2, 1 },
	{ test3, 1 },
	{ NULL, 0 }
};
