/*
*      SCCS:  @(#)  XOPEN.os/genuts/pclose/pclose.c	Rel 4.4.4     (07/19/99) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/genuts/pclose - T.pclose Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.pclose

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the pclose() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Added	vg_sjmp, setjmp(), tn_rpt()
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Dave Cater Mon May  8 10:08:57 BST 1989
		Replaced HIBYTE() with WIFEXITED(), etc.

		Geoff Clare, 23 Oct 1989
		Don't expect ECHILD on fopen()ed stream
		Add proper ECHILD test

		J.A.Nave	23/05/90
		Added resilience testing

		Geoff Clare, 3 Sept 1990
		Change timeout in test1 to remove race condition.

		Neil Moses, 21 Oct 1991
		Added new tests for VSX 4.2

		Geoff Clare, 25 Sept 1992
		Change rval from short to int in test2().

		Geoff Clare, 2 Dec 1996
		Test 4 only applies to XPG3.

		Geoff Clare, 7 Jan 1999
		In test 1, check time elapsed to ensure pclose() did
		not return early (i.e. when the signal was delivered).
		Removed extra pclose() call at end of test.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<sys/wait.h>
#include	<signal.h>
#include	<unistd.h>
#include	<timeout.h>
#include	<fcntl.h>
#include	<pclose.h>
#include	<string.h>
#include	<time.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef pclose
#endif


#define	NO_TESTS	6		/* number of tests in testset */
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();
extern	int	pipe_open();
extern	void	pipe_close();
extern	void	pipe_parent();
extern	void	pipe_child();

public	int	no_tests = NO_TESTS;

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;

#if TEST_XPG > 3
private	int	tfail;
#endif

private	char	*t3_file =	"./pclose-t.3";
private	char	*t1_pclose =	"../pclose_t1";
private	char	*t2_pclose =	"../pclose_t2";
#if TEST_XPG > 3
private	char	*t6_pclose =	"../pclose_t6";
#endif

private	void	test1();
private	void	test2();
private	void	test3();
#if TEST_XPG > 3
private	void	pa1_t3();
private	void	pa2_t3();
private	void	ch_t3();
#endif
private	void	test4();
private	void	test5();
private	void	test6();
#if TEST_XPG > 3
private	void	ch_t6();
#endif

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	NULL, 0
};


private void
test1()
{
	FILE		*fp;
	time_t		t0, t1;
	int		err;
	int		fd;
	int	pathok = 0;

	DBUG_ENTER("test1");
	errno = NOERROR;
	if ((fd = creat(t1_file, (mode_t)MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t1_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);
	if ((fp = popen(t1_pclose, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: popen(%s, \"r\") failed - errno %d(%s)",
			t1_pclose, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) time(&t0);
	
	SET_TIMEOUT(WAITTIME/2)
	(void) pclose(fp);
	CLEAR_ALARM

	(void) time(&t1);

	(void) unlink(t1_file);

	/* Child exits after WAITTIME seconds, but allow 1 second for
	   rounding */
	if (t1 - t0 < WAITTIME-1)
	{
		xx_rpt(FAILURE);
		in_rpt("pclose() did not wait for command to exit");
		in_rpt("- elapsed time: %ld seconds, expected >= %ld",
			(long)(t1 - t0), WAITTIME-1L);
	}
	else
		PATH_TRACE;

	if (alrm_flag != 0)
	{
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

private void
test2()
{
	FILE		*fp;
	int		err;
	int		rval;
	int	pathok = 0;

	DBUG_ENTER("test2");
	errno = NOERROR;
	if ((fp = popen(t2_pclose, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: popen(%s, \"r\") failed - errno %d(%s)",
			t2_pclose, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rval = pclose(fp);
	if (WIFEXITED(rval) == 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: process did not exit normally");
		if (WIFSIGNALED(rval))
			in_rpt("instead terminated by signal %d", 
								WTERMSIG(rval));
		else if (WIFSTOPPED(rval))
			in_rpt("instead stopped by signal %d", 
								WSTOPSIG(rval));
	} 
	else if (WEXITSTATUS(rval) != T2_EXIT)
	{
		xx_rpt(FAILURE);
		in_rpt("pclose() did not return the exit code of command");
		in_rpt("expected %d, pclose() returned %d",
			T2_EXIT, WEXITSTATUS(rval));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}


private void
test3()
{
#if TEST_XPG > 3
	int	pathok = 0;
#endif

	DBUG_ENTER("test3");

#if TEST_XPG > 3

	tfail = 0;

	globok = 0;
	(void) cpsig(ch_t3, pa1_t3, -1, 0);
	PATH_FUNC_TRACE;

	if(tfail)
	{
		xx_rpt(FAILURE);
 	      	in_rpt("Termination status of child processes affected by call to pclose()");
	}
	else
		PATH_XS_RPT(1);

#else
	xx_rpt(NOTINUSE);
#endif
	DBUG_VOID_RETURN;
}

#if TEST_XPG > 3
private void
pa1_t3()
{
	int	status;
	int	pathok = 0;

	DBUG_ENTER("pa1_t3");

	globok = 0;
	(void) cpsig(ch_t3, pa2_t3, -1, 0);
	PATH_FUNC_TRACE;

	if(waitpid(tet_child, &status, WNOHANG) == -1)
		tfail++;
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	DBUG_ENTER("ch_t3");

	DBUG_VOID_RETURN;
}

private void
pa2_t3()
{
	FILE	*fp;
	int	status;
	int	pathok = 0;

	DBUG_ENTER("pa2_t3");

	errno = NOERROR;

	if((fp = popen("sleep 5", "r")) == (FILE *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason : popen(\"sleep 5\", \"r\") failed - errno %d (%s)", errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(pclose(fp) == -1)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason : pclose(fp) failed - errno %d (%s)", errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(waitpid(tet_child, &status, WNOHANG) == -1)
		tfail++;
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}
#endif


private void
test4()
{
#if TEST_XPG > 3
	DBUG_ENTER("test4");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
#else
	FILE		*fp;
	int		err;
	int		rval;
	int	pathok = 0;

	DBUG_ENTER("test4");
	if ((fp = fopen(t3_file, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"w\") failed - errno %d(%s)",
			t3_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = pclose(fp)) != SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("pclose() on fopen()'d stream returned %d, expected %d",
			rval, SYSERROR);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	(void) unlink(t3_file);
	DBUG_VOID_RETURN;
#endif
}

private void
test5()
{
	FILE		*fp;
	int		err;
	int		rval;
	int	pathok = 0;

	DBUG_ENTER("test5");
	if ((fp = popen(t2_pclose, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: popen(%s, \"r\") failed - errno %d(%s)",
			t2_pclose, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	SET_TIMEOUT(WAITTIME)

	rval = wait((int *)0);
	err = errno;

	CLEAR_ALARM

	if (rval == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("wait() failed - errno %d (%s)", err, errname(err));
		(void) pclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	errno = NOERROR;
	if ((rval = pclose(fp)) != SYSERROR || errno != ECHILD)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("pclose() did not return expected values");
		in_rpt("expected %d, got %d", SYSERROR, rval);
		in_rpt("expected errno ECHILD, got %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

private void
test6()
{
	DBUG_ENTER("test6");

#if TEST_XPG > 3

	if(pipe_open() != 0)
	{
		(void) cppair(ch_t6, NULLFN, WAITTIME * 2, 0);
		pipe_parent();
	}
	pipe_close();

#else
	xx_rpt(NOTINUSE);
#endif
	DBUG_VOID_RETURN;
}

#if TEST_XPG > 3
private void
ch_t6()
{
	char	*args[4];
	int	err;

	DBUG_ENTER("ch_t6");

	pipe_child();

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = t6_pclose;
	args[1] = vg_ident;
	args[2] = vg_dbug;
	args[3] = NULL;
	(void) tet_exec(t6_pclose, args, environ);

	/* If here then tet_exec() failed */
	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		t6_pclose, err, errname(err));

	DBUG_VOID_RETURN;
}
#endif
