/*
*      SCCS:  @(#)  XOPEN.os/genuts/pclose/pclose_t6/pclose_t6.c	Rel 4.4.1 (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.pclose

PROJECT:	VSX (X/OPEN Validation Suite)


MODULE CONTAINS:
	tet_main()

SYNOPSIS:
	int main(argc, argv)
		int argc;
		char **argv;

DESCRIPTION:
	Exec'd by T.pclose for test6().

	pipe_*_rpt() are used because newroot() alters the path to the
	results file.

AUTHOR:		Neil Moses
DATE CREATED:	Fri Oct 04 10:44:53 BST 1991
MODIFICATIONS:

************************************************************************/

#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <dbug.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef pclose
#endif


#if TEST_XPG > 3
public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
#endif

extern	char	**environ;

extern	char	*errname();
extern	char	*testid();
extern	void	valrpt();
extern	void	set_pname();
extern	void	pipe_xx_rpt();
extern	void	pipe_path_xs_rpt();
extern	void	pipe_in_rpt();
extern	void	pipe_end_rpt();
extern	void	pipe_devinod_rpt();

#if TEST_XPG > 3
private	void	subtest6();
#endif

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	set_pname(argv[0]);

#if TEST_XPG > 3

	if (argc < 2 || argc > 3)
	{
		pipe_xx_rpt(DELETION);
		pipe_in_rpt("argument count wrong");
		pipe_end_rpt();
		exit(OK);
	}
	if (argc >= 3)
		vg_dbug = argv[2];
	if (argc >= 2)
		vg_ident = argv[1];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	subtest6();

#else
	pipe_xx_rpt(NOTINUSE);
	pipe_end_rpt();
#endif

	exit(OK);
	/* NOTREACHED */
}


#if TEST_XPG > 3
private void
subtest6()
{
	int	pathok = 0;
	int	bad_exit = 0;
	int	err, rval, status;
	char	msg[512];
	FILE	*fp;

	DBUG_ENTER("subtest6");

	/*
	 * Change 'root' directory for command
	 */
	errno = NOERROR;
	if(newroot(".") != 0)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
 	      	(void) sprintf(msg, "newroot(\".\") failed - errno %d(%s)",
			err, errname(err));
		pipe_in_rpt(msg);
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((fp = popen("exit 0", "r")) == (FILE *)NULL)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void) sprintf(msg,
			"popen(\"exit 0\", \"r\") failed - errno %d (%s)",
			err, errname(err));
		pipe_in_rpt(msg);
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((status = pclose(fp)) == -1)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void) sprintf(msg,
			"pclose(fp) failed - errno %d (%s)",
			err, errname(err));
		pipe_in_rpt(msg);
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(WIFEXITED(status))
		rval = WEXITSTATUS(status);
	else
		bad_exit = 1;

#ifdef TESTBED
rval = 127;
#endif

	if(rval != 127 || bad_exit)
	{
		pipe_xx_rpt(FAILURE);

		if(bad_exit)
		{
			(void) sprintf(msg, 
				"WIFEXITED() was false for pclose() return value: %d",
				status);
			pipe_in_rpt(msg);
		}
		else if(rval != 127)
		{
			pipe_in_rpt("pclose() did not return expected exit status");
			(void) sprintf(msg, "Expected exit status 127 actual exit status %d",rval);
			pipe_in_rpt(msg);
		}
	}
	else
		pipe_path_xs_rpt(3, pathok);

	pipe_end_rpt();

	DBUG_VOID_RETURN;
}
#endif
