/*
*      SCCS:  @(#)  XOPEN.os/genuts/popen/popen.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/genuts/popen - T.popen Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.popen

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the popen() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Added	vg_sjmp, setjmp(), tn_rpt()
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Linda Taylor, Hoskyns plc	September 1988
		Added test 3

		Geoff Clare, 27 Sep 1989
		Downgrade test3 result to warning

		J.A.Nave	23/05/90
		Added resilience testing

		Geoff Clare, 31 May 1990
		Allow for unlimited OPEN_MAX

		Geoff Clare, !4 Feb 1991
		Test 3 only executed in XPG3 mode.

		Neil Moses, 01/10/91
		Added new tests for 4.2

		Geoff Clare, 29 Apr 1992
		Skip XCU4-sh specific cmds if sysconf(_SC_2_VERSION) == -1

		Geoff Clare, 24 June 1992
		Changed test 5 so as not to overwrite string literals.
		
************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/stat.h>
#include	<stdlib.h>
#include	<signal.h>
#include	<fcntl.h>
#include	<popen.h>
#include	<string.h>
#include	<timeout.h>
#include	<sys/wait.h>
#include	<pcts_max.h>
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef popen
#endif


#define	NO_TESTS	9		/* number of tests in testset */
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)


extern	char	*errname();
extern	char	*pr_nchars();
extern	void	(*vsignal())();

public	int	no_tests = NO_TESTS;

private	char	*t1_popen =	"../popen_t1";
private	char	*t2_popen =	"../popen_t2";
#if !(TEST_XPG > 3)
private	char	*t3_popen =	"../non_existant_command";
#endif
#if (TEST_XPG > 3)
private	char	*t4_popen =	"../popen_t4";
private	char	*t4_output =	"../popen_t.4";
private	char	*t7_popen =	"../popen_t7";
#endif
private	char	*t1_resfile =	"../popen_t.1";

private	void	test1();
private	void	test2();
private	void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
#if !(TEST_XPG > 3)
private	void	ch_t3();
#endif
#if (TEST_XPG > 3)
private	void	ch_t5();
#endif
private	void	ch_t6();
#if (TEST_XPG > 3)
private	void	ch_t7();
private	void	ch_t8();
#endif

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	NULL, 0
};

#if (TEST_XPG > 3)

/* "4:" on the front of cmd indicates an XCU4-sh specific command */
private struct {
	char	*cmd;
	char	*expres;
} cmds[] = {
	"NAME=POPEN; echo $NAME", "POPEN\n",
	"NAME=POPEN; echo '$NAME'", "$NAME\n",
	"NAME=POPEN; echo \\$NAME", "$NAME\n",
	"echo \"*\"", "*\n",
	"echo `echo popen`", "popen\n",
	"echo popen #Ignore this", "popen\n",
	"for i in testword\ndo\nif [ \"$i\" = \"testword\" ]\nthen\necho $i\nfi\ndone", "testword\n",
	"NAME=POPEN; unset NAME; echo NONAME$NAME", "NONAME\n",
	"set one two three; echo $1 $2 $3", "one two three\n",
	"set one two three; echo $*", "one two three\n",
	"set one two three; echo $#", "3\n",
	"set one two three; shift; echo $1", "two\n",
	"set one two three; shift; echo $*", "two three\n",
	"set one two three; shift; echo $#", "2\n",
	"NAME=POPEN; (echo $NAME)", "POPEN\n",
	"set one; echo $1", "one\n",
	"4:echo $(expr 1 + 2)", "3\n",
	"echo `expr 1 + 2`", "3\n",
	"echo \"POPEN popen\"", "POPEN popen\n",
	"cat <<!\nPOPEN\n!", "POPEN\n",
	"echo REMOVEPOPEN | grep POPEN | sed -e 's/REMOVE//g'", "POPEN\n",
	"> t5_dummy; ls t?_[dabc]*y; rm -f t5_dummy", "t5_dummy\n",
	"echo POPEN > t5_dummy; cat t5_dummy; rm -f t5_dummy", "POPEN\n",
	"echo POPEN 2> t5_dummy 1>&2; cat t5_dummy; rm -f t5_dummy", "POPEN\n",
	"echo POPEN1 > t5_dummy; echo POPEN2 >> t5_dummy; cat t5_dummy; rm -f t5_dummy", "POPEN1\nPOPEN2\n",
	"ls xxxx 2> t5_dummy; if [ -s t5_dummy -a $? -ne 0 ]\nthen\necho PASS\nfi; rm -f t5_dummy", "PASS\n",
	"echo echo POPEN > t5_dummy; chmod u+x t5_dummy; PATH=.:$PATH; t5_dummy; rm -f t5_dummy", "POPEN\n",
	"trap \"echo TRAPPED\" 15; kill -15 $$", "TRAPPED\n",
	NULL, NULL
};
#endif

private void
test1()
{
	FILE	*fp;
	int	err;
	int	len;
	char	buf[256];
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("test1");
	errno = NOERROR;
	(void) strcat(strcat(strcpy(buf, t1_popen), " >"), t1_resfile);
	if ((fp = popen(buf, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("popen(\"%s\", \"w\") failed - errno %d(%s)",
			buf, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fputs(t_string, fp);

	ret = pclose(fp);

	if (ret)
	{	/*	popen_t1 exits with 2 for a path check failure	*/
		if (WEXITSTATUS(ret) == 2)
			xx_rpt(DELETION);
		else
			xx_rpt(FAILURE);

		/* read in info lines written by popen_t1 and report them */
		if ((fp = fopen(t1_resfile, "r")) == NULL)
		{
			err = errno;
			in_rpt("Error: cannot open %s to read failure info - errno %d(%s)", 
				t1_resfile, err, errname(err));
		}
		else
		{
			while (fgets(buf, 256, fp) != NULL)
			{
				len = strlen(buf);
				if (buf[len-1] == '\n')
					buf[len-1] = '\0';
				in_rpt("%s", buf);
			}
			(void) fclose(fp);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) unlink(t1_resfile);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	FILE		*fp;
	char		*rval;
	int		err;
	char		buf[32];
	int	pathok = 0;

	DBUG_ENTER("test2");
	errno = NOERROR;
	if ((fp = popen(t2_popen, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("popen(%s, \"r\") failed - errno %d(%s)",
			t2_popen, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	SET_TIMEOUT(WAITTIME)
	rval = fgets(buf, sizeof(buf), fp);
	CLEAR_ALARM
	if (rval == NULL)
	{
		xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("fgets() on stream returned by popen() timed out");
		else
			in_rpt("fgets() on stream returned by popen() failed");
	}
	else if (strcmp(t_string, buf))
	{
		xx_rpt(FAILURE);
		in_rpt("fgets() on stream returned by popen() returned wrong string");
		in_rpt("expected >%s<, got %>s<", t_string, buf);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) pclose(fp);
	DBUG_VOID_RETURN;
}

private void
test3()
{
#if !(TEST_XPG > 3)
	int	fail;
	int	testfail = 0;
	int	pathok = 0;
#endif

	DBUG_ENTER("test3");

#if !(TEST_XPG > 3)

	globok = 0;
	fail = cppair(ch_t3, NULLFN, WAITTIME, 017);
	if (fail != 010 && globok == 1)
		PATH_TRACE;

	if (fail)
	{
		if (fail & 02)
		{
			if (!testfail++)
				xx_rpt(WARNING);
			in_rpt("read from stream returned by popen(%s, \"r\") did not give error",
				t3_popen);
		}
		if (fail & 04)
		{
			if (!testfail++)
				xx_rpt(WARNING);
			in_rpt("write to stream returned by popen(%s, \"w\") did not give error",
				t3_popen);
		}
	}
	else
	{
		PATH_XS_RPT(1);
	}

#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}

#if !(TEST_XPG > 3)
private void
ch_t3()
{
	FILE		*fp;
	int		fail;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	fail = 0;

	/* get rid of shell error messages from popen() */
	(void) freopen("/dev/null", "w", stderr);

	/* command t3_popen cannot be executed */
	/* if popen returns NULL the test is successful */
	if ((fp = popen(t3_popen, "r")) != (FILE *) NULL)
	{
		errno = 0;
		(void) getc(fp);
		if (!ferror(fp) && !errno)
			fail |= 02;
	}
	else
		PATH_TRACE;

	(void) pclose(fp);	

	if ((fp = popen(t3_popen, "w")) != (FILE *) NULL)
	{
		(void) vsignal(SIGPIPE, SIG_IGN);
		(void) sleep(2);
		errno = 0;
		(void) putc('a', fp);
		(void) fflush(fp);
		if (!ferror(fp) && !errno)
			fail |= 04;
	}
	else
		PATH_TRACE;

	(void) pclose(fp);	

	if (fail)
	{
		DBUG_EXIT(fail);
	}
	else if (pathok != 2)
		DBUG_EXIT(010);

	DBUG_VOID_RETURN;
}
#endif

private void
test4()
{
#if (TEST_XPG > 3)
	FILE	*tfp, *ofp;
	char	buf[51];
	char	line1[20];
	char	line2[20];
	char	line3[20];
	int	err;
	int	nbytes = 50;
	int	pathok = 0;
#endif

	DBUG_ENTER("test4");

#if (TEST_XPG > 3)

	errno = NOERROR;
	*line1 = *line2 = *line3 = '\0';

	if ((tfp = popen(t4_popen, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
 	      	in_rpt("popen(%s, \"r\") failed - errno %d(%s)", 
			t4_popen, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((ofp = fopen(t4_output, "w")) == (FILE *)NULL)
	{
		err = errno;
		xx_rpt(DELETION);
 	      	in_rpt("fopen(%s, \"w\") failed - errno %d(%s)",
			t4_output, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(fclose(ofp) != 0)
	{
		err = errno;
		xx_rpt(DELETION);
 	      	in_rpt("fclose() failed - errno %d(%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((ofp = fopen(t4_output, "a")) == (FILE *)NULL)
	{
		err = errno;
		xx_rpt(DELETION);
 	      	in_rpt("fopen(%s, \"a\") failed - errno %d(%s)",
			t4_output, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fputs("parent\n", ofp);
	(void) fflush(ofp);
	(void) fgets(buf, 20, tfp);
	(void) fputs("parent\n", ofp);
	(void) fflush(ofp);
	(void) fclose(ofp);

	if(pclose(tfp) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("Child process did not execute successfully");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((ofp = fopen(t4_output, "r")) == (FILE *)NULL)
	{
		err = errno;
		xx_rpt(DELETION);
 	      	in_rpt("fopen(%s, \"r\") failed - errno %d(%s)",
			t4_output, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) fgets(line1, nbytes, ofp);
	(void) fgets(line2, nbytes, ofp);
	(void) fgets(line3, nbytes, ofp);

	(void) fclose(ofp);	
	(void) unlink(t4_output);	

	if(strcmp(line1, "parent\n") != 0
	|| strcmp(line2, "child\n") != 0
	|| strcmp(line3, "parent\n") != 0)
	{
		in_rpt("Expected output from test file was :-");
		in_rpt("line 1 : parent");
		in_rpt("line 2 : child");
		in_rpt("line 3 : parent");
		in_rpt("Actual output from test file was :-");
		in_rpt("line 1 : %s", line1);
		in_rpt("line 2 : %s", line2);
		in_rpt("line 3 : %s", line3);
		xx_rpt(FAILURE);
	}
	else
		PATH_XS_RPT(6);

#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}

private void
test5()
{
	DBUG_ENTER("test5");

#if (TEST_XPG > 3)

	(void) cppair(ch_t5, NULLFN, WAITTIME * 5, 0);

#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}

#if (TEST_XPG > 3)
private void
ch_t5()
{
	FILE	*fp;
	char	actres[512];
	int	ret, err, i;
	int	testfail = 0;
	long	posix2_vers;
	int	sysconf_err;
	int	pathok = 0;

	DBUG_ENTER("ch_t5");

	errno = 0;
	posix2_vers = sysconf(_SC_2_VERSION);
	sysconf_err = errno;

	for(i = 0; cmds[i].cmd != NULL; i++)
	{
		if(i == 0)
			PATH_TRACE;

		/* Check for XCU4-sh specific command */
		if (cmds[i].cmd[0] == '4' &&
		    cmds[i].cmd[1] == ':')
		{
			if (posix2_vers == -1 && sysconf_err == 0)
			{
				PATH_TRACE;
				PATH_TRACE;
				PATH_TRACE;
				PATH_TRACE;
				PATH_TRACE;
				continue;
			}
			else if (posix2_vers == -1 && sysconf_err != 0)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("sysconf(_SC_2_VERSION) failed, errno %d(%s)",
					sysconf_err, errname(sysconf_err));
				continue;
			}
			else
				PATH_TRACE;

			/* skip over the "4:" */
			cmds[i].cmd += 2;
		}
		else
			PATH_TRACE;

		if((fp = popen(cmds[i].cmd, "r")) == (FILE *)NULL)
		{
			err = errno;
			xx_rpt(DELETION);
 	      		in_rpt("popen(\"%s\", \"r\") failed - errno %d(%s)",
				cmds[i].cmd, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		errno = NOERROR;
		if((ret = fread((void *)actres, (size_t)1, sizeof(actres),
			fp)) <= 0 && errno != NOERROR)
		{
			err = errno;
			xx_rpt(DELETION);
 	      		in_rpt("fread() failed - errno %d(%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		actres[ret] = '\0';

		if(strcmp(actres, cmds[i].expres))
		{
			in_rpt("popen(\"%s\") failed", 
				pr_nchars(cmds[i].cmd,
				(int)strlen(cmds[i].cmd)));
			in_rpt("expected output: \"%s\"", 
				pr_nchars(cmds[i].expres,
				(int)strlen(cmds[i].expres)));
			in_rpt("observed output: \"%s\"",
				pr_nchars(actres, ret));
			testfail++;
		}
		else
			PATH_TRACE;

		if(pclose(fp) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
 	      		in_rpt("pclose() failed - errno %d(%s)", 
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	if(testfail)
		xx_rpt(FAILURE);
	else
		PATH_XS_RPT(5 * i + 1);
		
	DBUG_VOID_RETURN;
}
#endif

private void
test6()
{
	DBUG_ENTER("test6");

	(void) cppair(ch_t6, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	FILE	*fp;
	long	open_max;
	long	i;
	char	buf[100];
	int	pathok = 0;

	DBUG_ENTER("ch_t6");

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max == -1 && errno == 0)
		open_max = PCTS_OPEN_MAX * 10L;
	else if (open_max < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sysconf(_SC_OPEN_MAX) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=0; i<open_max; i++)
	{
		if (fopen(t1_popen, "r") == (FILE *) NULL)
			break;
	}
	if (i >= open_max)
	{
		xx_rpt(DELETION);
		in_rpt("could not get fopen() to fail after %ld calls", i);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) strcat(strcat(strcpy(buf, t1_popen), " >"), t1_resfile);
	if ((fp = popen(buf, "w")) != (FILE *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(
	    "popen(\"%s\", \"w\") succeeded when a stream could not be created",
		buf);
		(void) pclose(fp);
	}
	else
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
			
private void
test7()
{
	DBUG_ENTER("test7");

#if (TEST_XPG > 3)
	(void) cppair(ch_t7, NULLFN, WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}

#if (TEST_XPG > 3)
private void
ch_t7()
{
	FILE	*tfp;
	int	fd[2];
	int	i, err, open_max;
	int	pathok = 0;

	DBUG_ENTER("ch_t7");

	/*
	 * Find max no file descriprtors
	 */
	errno = NOERROR;
	if((open_max = sysconf(_SC_OPEN_MAX)) == -1 && errno == 0)
	{
		xx_rpt(UNTESTED);
		in_rpt("sysconf(_SC_OPEN_MAX) returned -1 and errno 0");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(open_max > PCTS_OPEN_MAX)
	{
		xx_rpt(UNTESTED);
		in_rpt("sysconf(_SC_OPEN_MAX) returned > PCTS_OPEN_MAX");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Attempt a successful call to popen()
	 */
	if ((tfp = popen(t7_popen, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
 	      	in_rpt("popen(%s, \"r\") failed - errno %d(%s)",
			t7_popen, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(pclose(tfp) != 0)
	{
		err = errno;
		xx_rpt(DELETION);
 	      	in_rpt("pclose(tfp) failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Use up all file descriptors
	 */
	for(i = 0; i < open_max; i++)
	{
		if(i == 0)
			PATH_TRACE;

		if(pipe(fd) == -1)
			break;
	}

	/*
	 * Attempt an unsuccessful call to popen()
	 */
	if ((tfp = popen(t7_popen, "r")) != (FILE *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("popen() did not return NULL when a pipe could not be created");
	}
	else
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}
#endif

private void
test8()
{
	DBUG_ENTER("test8");

#if (TEST_XPG > 3)
	(void) cppair(ch_t8, NULLFN, WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}

#if (TEST_XPG > 3)
private void
ch_t8()
{
	FILE	*tfp;
	int	i, child_max, err, ret;
	int	pathok = 0;

	DBUG_ENTER("ch_t8");

	/*
	 * Find max no sub processes
	 */
	errno = NOERROR;
	if((child_max = sysconf(_SC_CHILD_MAX)) == -1 && errno == 0)
	{
		xx_rpt(UNTESTED);
		in_rpt("sysconf(_SC_CHILD_MAX) returned -1 and errno 0");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(child_max > PCTS_CHILD_MAX)
	{
		xx_rpt(UNTESTED);
		in_rpt("sysconf(_SC_CHILD_MAX) returned > PCTS_CHILD_MAX");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Attempt a successful call to popen()
	 */
	if ((tfp = popen(t7_popen, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
 	      	in_rpt("popen(%s, \"r\") failed - errno %d(%s)",
			t7_popen, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(pclose(tfp) != 0)
	{
		err = errno;
		xx_rpt(DELETION);
 	      	in_rpt("pclose(tfp) failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Use up all processes
	 */
	for(i = 0; i < child_max; i++)
	{
		if(i == 0)
			PATH_TRACE;

		if((ret = fork()) == 0)
			exit(0);
		else if(ret == -1)
			break;
	}
	
	/*
	 * Attempt an unsuccessful call to popen()
	 */
	if ((tfp = popen(t7_popen, "r")) != (FILE *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("popen() did not return NULL when a subprocess could not be created");
	}
	else
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}
#endif

private void
test9()
{
#if (TEST_XPG > 3)
	FILE	*tfp;
	char	*inv_mode;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test9");

#if (TEST_XPG > 3)

	errno = NOERROR;

	/*
	 * Get invalid mode
	 */
	if((inv_mode = tet_getvar("VSX_INVALID_POPEN_MODE")) == NULL
	|| *inv_mode == '\0')
	{
		xx_rpt(DELETION);
 	      	in_rpt("parameter VSX_INVALID_POPEN_MODE is not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/*
	 * Check if supported
	 */
	if(strncmp(inv_mode, "unsup", 5) == 0)
	{
		xx_rpt(UNSUPPORTED);
 	      	in_rpt("parameter VSX_INVALID_POPEN_MODE is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Attempt a successful call to popen()
	 */
	if ((tfp = popen(t7_popen, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
 	      	in_rpt("popen(%s, \"a\") failed - errno %d(%s)",
			t7_popen, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(pclose(tfp) != 0)
	{
		err = errno;
		xx_rpt(DELETION);
 	      	in_rpt("pclose(tfp) failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Attempt an unsuccessful call to popen()
	 */
	errno = 0;
	tfp = popen(t7_popen, inv_mode);

	if(tfp != (FILE *) NULL || errno != EINVAL)
	{
		err = errno;
		xx_rpt(FAILURE);

		if(tfp != (FILE *)NULL)
		{
			in_rpt("popen(%s, \"%s\") did not return NULL",
				t7_popen, inv_mode);
			(void) pclose(tfp);
		}
		else if(err != EINVAL)
		{
			in_rpt("popen(%s, \"%s\") did not set errno correctly",
				t7_popen, inv_mode);
			in_rpt("Expected errno value %d (EINVAL)", EINVAL);
			in_rpt("Actual   errno value %d (%s)", 
				err, errname(err));
		}
	}
	else
		PATH_XS_RPT(4);

#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}
