/*
*      SCCS:  @(#)  XPG4.os/genuts/wordexp/wordexp.c	Rel 4.4.5   (07/31/00)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XPG4.os/genuts/wordexp - T.wordexp Rel 4.4.5 (07/31/00)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.wordexp

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the wordexp() subroutine.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		M Young, UniSoft Ltd.
DATE CREATED:	Wed Sep 04 15:52:35 WET 1991
MODIFICATIONS:
		Neil Moses, 8 May 1992.
			Check that interface is supported.
			Added TEST For ENOSYS if interface not supported.
		
		Geoff Clare, 2-4 June 1992
			Removed trailing \n from expected output in test 8.
			New test arguments in WRDE_BADCHAR test.
			Removed "[ with no ]" test.
			Updated ": in IFS" test.
			Added declaration of environ.
			Removed test expecting "[/X]" to match "X".

		Neil Moses, 4 June 1992.
			Use sysconf() to check if interface supported.
		
		Geoff Clare, 22 July 1992
			Remove unquoted newlines in test 10.
		
		Geoff Clare, 28 July 1992
			Changed test 11 to expect { "h", "i" } instead
			of { "h", "", "i" }.

		Geoff Clare, 9 March 1993
			Align with interpretation VWG/025/081892.

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)
			Allow # to be treated as normal character in test 12.

		Andrew Twigger, 17 May 1995
			Make env_home a buffer and copy string from
			environment into this buffer.

                Geoff Clare, 21 Feb 2000
			Test for over-zealous `trailing' newline removal
			on command substitutions.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<stdlib.h>
#include	<string.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<wordexp.h>
#include	<tet_api.h>
#include	<limits.h>
#include	<pwd.h>
#include	<string.h>
#include        <nlstest.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef wordexp
#undef wordfree
#endif


#define		NO_TESTS	31	/* number of tests in testset */

#define         WAITTIME        (5*SPEEDFACTOR)
#define		MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define         MODENOSRCH      (S_IRUSR|S_IRGRP|S_IROTH)
#define         MODENOREAD      (S_IXUSR|S_IXGRP|S_IXOTH)

#define	N_EXPECTED(x)	(sizeof(x) / sizeof(char *) - 1)

int	no_tests = NO_TESTS;

extern	char	**environ;
extern	char	*errname();
extern	void	xx_all();

extern	char	*pr_nchars();

private	int	setupok = 0;
private	int	tsup = 0;
private	char	*test_vecp;

private	wordexp_t	empty_wordexp;	/* empty struct to clear automatics */
private	wordexp_t	ref_wordexp_b;	/* struct to check is unchanged */

private char dir_001[] = "dir_1";
private char dir_002[] = "dir_1/readable";
private char dir_003[] = "dir_1/Xreadable";
private char dir_004[] = "dir_1/unreadable";
private char dir_005[] = "dir_2";
private char dir_006[] = "dir_2/unreadable";
private char dir_007[] = "dir_2/readable";
private char dir_008[] = "dir_2/Xreadable";
private char dir_009[] = "dir_1/unsearch";

private char file_001[] = "dir_1/a";
private char file_002[] = "dir_1/readable/a";
private char file_003[] = "dir_1/readable/b";
private char file_004[] = "dir_1/unreadable/a";
private char file_005[] = "dir_1/Xreadable/a";
private char file_006[] = "a";
private char file_007[] = "b";
private char file_008[] = "c";
private char file_009[] = "d";
private char file_010[] = "aXb";
private char file_011[] = "aYb";
private char file_012[] = "aZb";
private char file_013[] = "a[b";
private char file_014[] = "a]b";
private char file_015[] = "a-b";
private char file_016[] = "a?b";
private char file_017[] = "a*b";
private char file_018[] = ".hidden";
private char file_019[] = "Xhidden";
private char file_020[] = "nothidden";
private char file_021[] = "dir_1/nothidden";
private char file_022[] = "dir_1/.hidden";
private char file_023[] = "axb";
private char file_024[] = "ayb";
private char file_025[] = "azb";
private char file_026[] = "dir_2/readable/a";
private char file_027[] = "dir_2/Xreadable/a";
private char file_028[] = "dir_2/a?b";
private char file_029[] = "dir_2/a\\b";
private char file_030[] = "dir_2/arb";
private char file_031[] = "dir_2/.rb";
private char file_032[] = "dir_1Xa";

struct	ft
{
	char	*name;
	mode_t	perm;
};

private struct	ft	wordexp_dirs[] =
{
	dir_001,	MODEANY,
	dir_002,	MODEANY,
	dir_003,	MODEANY,
	dir_004,	MODENOREAD,
	dir_005,	MODEANY,
	dir_006,	MODENOREAD,
	dir_007,	MODEANY,
	dir_008,	MODEANY,
	dir_009,	MODENOSRCH,
};

private struct	ft	wordexp_files[] =
{
	file_001,	MODEANY,
	file_002,	MODEANY,
	file_003,	MODEANY,
	file_004,	MODEANY,
	file_005,	MODEANY,
	file_006,	MODEANY,
	file_007,	MODEANY,
	file_008,	MODEANY,
	file_009,	MODEANY,
	file_010,	MODEANY,
	file_011,	MODEANY,
	file_012,	MODEANY,
	file_013,	MODEANY,
	file_014,	MODEANY,
	file_015,	MODEANY,
	file_016,	MODEANY,
	file_017,	MODEANY,
	file_018,	MODEANY,
	file_019,	MODEANY,
	file_020,	MODEANY,
	file_021,	MODEANY,
	file_022,	MODEANY,
	file_023,	MODEANY,
	file_024,	MODEANY,
	file_025,	MODEANY,
	file_026,	MODEANY,
	file_027,	MODEANY,
	file_028,	MODEANY,
	file_029,	MODEANY,
	file_030,	MODEANY,
	file_031,	MODEANY,
	file_032,	MODEANY,
};

private void	prepare_tests();
private void	cln_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();
private void	test30();
private void	test31();
private void	ch_t11();
private void	ch_t21();
private void	ch_t26();
private void	ch_t27();

private int	n_dirs = sizeof(wordexp_dirs) / sizeof(struct ft);
private int	n_files = sizeof(wordexp_files) / sizeof(struct ft);

private	char	tmp_file[] = "wordexp.tmp";
private	char	env_home[_POSIX_PATH_MAX];
private	char	vsx1_home[_POSIX_PATH_MAX];

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;

public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	test3, 3,
	test4, 4,
	test5, 5,
	test6, 6,
	test7, 7,
	test8, 8,
	test9, 9,
	test10, 10,
	test11, 11,
	test12, 12,
	test13, 13,
	test14, 14,
	test15, 15,
	test16, 16,
	test17, 17,
	test18, 18,
	test19, 19,
	test20, 20,
	test21, 21,
	test22, 22,
	test23, 23,
	test24, 24,
	test25, 25,
	test26, 26,
	test27, 27,
	test28, 28,
	test29, 29,
	test30, 30,
	test31, 31,
	NULL,	 0
};

#define	CHK_OK		0x00		/* list matches expected results */
#define	CHK_NOLIST	0x01		/* list pointer is empty */
#define	CHK_MISSING	0x02		/* an entry is missing */
#define	CHK_UNORDER	0x04		/* list is out of order */
#define	CHK_ORDERED	0x08		/* check for ordered list */
#define	CHK_BADOFF	0x10		/* offs not null */

private char *
flagstr(flags)
int flags;
{
	static char buf[128];

	if (flags == 0)
		return "0";
	
	*buf = '\0';

	if (flags & WRDE_APPEND)
		(void) strcat(buf, " | WRDE_APPEND");

	if (flags & WRDE_DOOFFS)
		(void) strcat(buf, " | WRDE_DOOFFS");

	if (flags & WRDE_NOCMD)
		(void) strcat(buf, " | WRDE_NOCMD");

	if (flags & WRDE_REUSE)
		(void) strcat(buf, " | WRDE_REUSE");

	if (flags & WRDE_SHOWERR)
		(void) strcat(buf, " | WRDE_SHOWERR");

	if (flags & WRDE_UNDEF)
		(void) strcat(buf, " | WRDE_UNDEF");

	return &buf[3];
}

private int
chk_sup(pattern, flags, rtval, err)
char *pattern;
int flags;
int rtval;
int err;
{
	int pathok = 0;
	int nfails = 0;
	static int firstsup = -1;

	DBUG_ENTER("chk_sup");

	/* If the POSIX2 CLB group is not supported, wordexp() can
	   still be supported on its own, but calls must consistently
	   indicate support or non-support. */

	if (rtval == WRDE_NOSYS && err == ENOSYS)
	{
		if (tsup)
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("wordexp(\"%s\", pwordexp, %s) not correct:",
				pr_nchars(pattern, (int)strlen(pattern)),
				flagstr(flags));
			in_rpt("wordexp() gave ENOSYS when POSIX2 CLB group is supported");
		}
		else if (firstsup == -1)
		{
			firstsup = 0;
			PATH_TRACE;
		}
		else if (firstsup == 1)
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("wordexp(\"%s\", pwordexp, %s) not correct:",
				pr_nchars(pattern, (int)strlen(pattern)),
				flagstr(flags));
			in_rpt("wordexp() gave ENOSYS but a previous call did not");
		}
		else
			PATH_TRACE;
	}
	else
	{
		if (firstsup == -1)
		{
			firstsup = 1;
			PATH_TRACE;
		}
		else if (firstsup == 0)
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("wordexp(\"%s\", pwordexp, %s) not correct:",
				pr_nchars(pattern, (int)strlen(pattern)),
				flagstr(flags));
			in_rpt("wordexp() did not give ENOSYS but a previous call did");
		}
		else
			PATH_TRACE;
	}

	PATH_FUNC_RPT(1);
	DBUG_RETURN(nfails);
}

private	void
rpt_expected(pattern,list,gp, xc, altxc)
char	*pattern;
char	**list;
wordexp_t *gp;
int	xc;
int	altxc;
{
	int c, i, off;

	c = gp->we_wordc;

	if ( xc >= 0 )
	{
		in_rpt("wordexp(\"%s\", ...) did not produce expected word list",
			pr_nchars(pattern, (int)strlen(pattern)));
		if (altxc == xc)
			in_rpt("we_wordc expected: %d, observed: %d",
				xc, gp->we_wordc);
		else
			in_rpt("we_wordc expected: %d or %d, observed: %d",
				xc, altxc, gp->we_wordc);
	}

	if ( c < 0 || c > 100 )
		in_rpt("unable to display matched list");
	else
	{
		in_rpt("expected words:");
		for( off = gp->we_offs; off > 0; off-- )
			in_rpt("\t<NULL>");
		for( i = 0; *list; list++, i++ )
		{
			if (i == altxc)
				in_rpt("OR, the above plus:");
			in_rpt("\t%s", pr_nchars(*list, (int)strlen(*list)));
		}
		if( (!(list = gp->we_wordv) ))
		{
			in_rpt("we_wordv is <NULL>");
		}
		else
		{
			in_rpt("actual words:");
			for( off = gp->we_offs; off-- > 0; list++ )
			{
				if ( *list )
					in_rpt("\t%s",
					    pr_nchars(*list, (int)strlen(*list)));
				else
					in_rpt("\t<NULL>");
			}
			for( ; *list && c-- > 0; list++ )
				in_rpt("\t%s", pr_nchars(*list, (int)strlen(*list)));
		}
	}
}

private	int
random_chk(list,gp)
char	**list;
wordexp_t *gp;
{
	char	**matched;
	int	i, ok;

	matched = gp->we_wordv;

	if( ! matched )
		return(CHK_NOLIST);

	for( i = gp->we_offs; i-- > 0; matched++ )
	{
		if ( *matched != (char *)NULL )
			return(CHK_BADOFF);
	}

	for( ; *list; list++ )
	{
		ok = 0;
		for( i = gp->we_wordc, matched = gp->we_wordv +gp->we_offs;
					i-- > 0; matched++ )
		{
			if ( ! strcmp(*matched, *list) )
			{
				ok = 1;
				break;
			}
		}
		if( !ok)
			return(CHK_MISSING);
	}
	return(CHK_OK);
}

/*
 * These functions assume that the we_offs field is set correctly, as
 * the wordexp_t buffer is always cleared this will be the case.
 */

private	int
ordered_chk(list,gp)
char	**list;
wordexp_t *gp;
{
	char	**matched, **save_list;

	matched = gp->we_wordv;

	if( ! matched )
		return(CHK_NOLIST);

	matched += gp->we_offs;
	save_list = list;

	for( ; *list; list++, matched++ )
	{
		if ( strcmp(*matched, *list) )
		{
			switch( random_chk( save_list, gp) )
			{
				case CHK_OK:
					return(CHK_UNORDER);
				case CHK_BADOFF:
					return(CHK_BADOFF);
				default:
					return(CHK_MISSING);
			}
		}
	}
	return(CHK_OK);
}

private int
return_chk(pattern, list, gp)
char	*pattern;
char	**list;
wordexp_t *gp;
{
	int	rc, nfails;
	int	pathok = 0;

	DBUG_ENTER("return_chk");

	nfails = 0;

	rc = ordered_chk(list,gp);

	switch ( rc ) {
		case CHK_OK:
			PATH_TRACE;
			break;
		case CHK_NOLIST:
			xx_rpt(FAILURE);
			in_rpt("wordexp(\"%s\", ...) allocated no we_wordv",
				pr_nchars(pattern, (int)strlen(pattern)));
			nfails++;
			break;

		case CHK_MISSING:
			xx_rpt(FAILURE);
			in_rpt("wordexp(\"%s\", ...) returned vector with expected word missing",
				pr_nchars(pattern, (int)strlen(pattern)));
			nfails++;
			break;

		case CHK_BADOFF:
			xx_rpt(FAILURE);
			in_rpt("wordexp(\"%s\", ...) returned vector with non-empty offset",
				pr_nchars(pattern, (int)strlen(pattern)));
			nfails++;
			break;

		case CHK_UNORDER:
			xx_rpt(FAILURE);
			in_rpt("wordexp(\"%s\", ...) returned incorrectly sorted vector",
				pr_nchars(pattern, (int)strlen(pattern)));
			nfails++;
			break;

		default:
			xx_rpt(DELETION);
			in_rpt("deletion reason: compare function returned unexpected value - %d", rc);
			DBUG_RETURN(1);
	}

	if( nfails )
		rpt_expected(pattern, list, gp, -1, -1);

	PATH_FUNC_RPT(1);

	DBUG_RETURN(nfails);
}

private int
do_expand(pattern, flags, list, wp)
char *pattern;
int flags;
char **list;
wordexp_t *wp;
{
	int	nfails = 0;
	int	rtval, err;
	int	n_expected;
	int	alt_expected;
	char	**listp;
	int	pathok = 0;

	DBUG_ENTER("do_expand");

	errno = 0;
	rtval = wordexp(pattern, wp, flags);
	err = errno;

	globok = 0;
	nfails += chk_sup(pattern, flags, rtval, err);
	PATH_FUNC_TRACE;

	if (rtval == WRDE_NOSYS && err == ENOSYS)
	{
		PATH_TRACE;
		PATH_FUNC_RPT(2);
		DBUG_RETURN(nfails);
	}
	else if (rtval != 0)
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("wordexp(\"%s\", pwordexp, %s) returned %d instead of 0",
			pr_nchars(pattern, (int)strlen(pattern)),
			flagstr(flags), rtval);
	}
	else
		PATH_TRACE;
	    
	n_expected = 0; /* total number of words */
	alt_expected = 0; /* number of words before optional comment */
	for (listp = list; *listp != NULL; listp++)
	{
		/* start of optional comment is marked by "##" */
		if ((*listp)[0] == '#' && (*listp)[1] == '#')
		{
			*listp = "#";
			break;
		}
		n_expected++;
		alt_expected++;
	}
	for ( ; *listp != NULL; listp++)
		n_expected++;

	if( wp->we_wordc != n_expected && wp->we_wordc != alt_expected )
	{
		nfails++;
		xx_rpt(FAILURE);
		rpt_expected(pattern, list, wp, n_expected, alt_expected);
	}
	else
	{
		/* if we_wordc was set to the smaller of the two allowed
		   values, truncate the expected word list accordingly */
		if (wp->we_wordc == alt_expected)
			list[alt_expected] = NULL;

		globok = 0;
		nfails += return_chk(pattern, list, wp);
		PATH_FUNC_TRACE;
	}

	PATH_FUNC_RPT(3);
	DBUG_RETURN(nfails);
}

private int
chk_wordexp(wp)
wordexp_t *wp;
{
	int	nfails;
	int	pathok = 0;

	DBUG_ENTER("chk_wordexp");

	nfails = 0;

	if( wp->we_wordc != ref_wordexp_b.we_wordc)
	{
		if( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("wordexp() incorrectly changed we_wordc member");
	}
	else
		PATH_TRACE;
	if( wp->we_offs != ref_wordexp_b.we_offs)
	{
		if( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("wordexp() incorrectly changed we_offs member");
	}
	else
		PATH_TRACE;
	if( wp->we_wordv != ref_wordexp_b.we_wordv)
	{
		if( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("wordexp() incorrectly changed we_wordv member");
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_FUNC_RPT(3);

	DBUG_RETURN(nfails);
}

private	void
prefix_expected(full_list,short_list,space,prefix)
char	**full_list;
char	**short_list;
char	*space;
char	*prefix;
{
	for( ; *short_list; space += _POSIX_PATH_MAX)
	{
		*full_list++ = 
			strcat(strcpy(space, prefix),*short_list++);
	}
	*full_list = (char *)NULL;
}

private void
cln_tests()
{
	int	i;
	struct	ft *ftp;

	DBUG_ENTER("cln_tests");

	for( i = n_dirs, ftp = wordexp_dirs+n_dirs; i-- > 0; )
	{
		(void) chmod((--ftp)->name, MODEANY);
	}

	for( i = 0, ftp = wordexp_files; i++ < n_files; ftp++)
	{
		(void) unlink(ftp->name);
	}

	for( i = n_dirs, ftp = wordexp_dirs+n_dirs; i-- > 0; )
	{
		(void) rmdir((--ftp)->name);
	}

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	int	i;
	struct	ft *ftp;
	int	fd;
	int	save_err;
	int	pathok = 0;
	struct	passwd *pwp;
	char	**bad_val;
	char	*tmp_home;

	DBUG_ENTER("prepare_tests");

	errno = 0;
	if(sysconf(_SC_2_C_VERSION) == -1)
	{
		if (errno != 0)
		{
			xx_all(DELETION, "sysconf(_SC_2_C_VERSION) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		tsup = 0;
	}
	else
	{
		tsup = 1;
		PATH_TRACE;
	}
	
	bad_val = &test_vecp;

	ref_wordexp_b.we_wordc = 1234;
	ref_wordexp_b.we_offs = 5678;
	ref_wordexp_b.we_wordv = bad_val;

	for( i = 0, ftp = wordexp_dirs; i++ < n_dirs; ftp++)
	{
		if ( crdir(ftp->name, (mode_t)(S_IRWXU|S_IRWXG|S_IRWXO),
					(int) getuid(), (int) getgid())  == SYSERROR )
		{
			save_err = errno;
			DBUG_PRINTF("crdir", ftp->name);
			DBUG_PRINTF("crdir", "errno = %s", errname(save_err));
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				tet_delete(i, "failed to create directory");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	for( i = 0, ftp = wordexp_files; i++ < n_files; ftp++)
	{
		if ( (fd = creat(ftp->name, MODEANY)) == SYSERROR )
		{
			save_err = errno;
			DBUG_PRINTF("creat", ftp->name);
			DBUG_PRINTF("creat", "errno = %s", errname(save_err));
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				tet_delete(i, "creat call failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		(void)close(fd);
	}

	for( i = 0, ftp = wordexp_dirs; i++ < n_dirs; ftp++)
	{
		if ( chmod(ftp->name, ftp->perm)  == SYSERROR )
		{
			save_err = errno;
			DBUG_PRINTF("chmod", ftp->name);
			DBUG_PRINTF("chmod", "errno = %s", errname(save_err));
			for ( i = 1 ; i <= NO_TESTS ; i++ )
				tet_delete(i, "failed to chmod directory");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	if( (tmp_home = getenv("HOME")) == NULL )
	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			tet_delete(i, "failed to find environment variable HOME");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	strcpy(env_home, tmp_home);

	if( (pwp = getpwnam("vsx1")) == NULL )
	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			tet_delete(i, "failed to find vsx1 password struct");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) strcpy( vsx1_home, pwp->pw_dir);
	(void)vputenv(&environ, "IFS= \t\n");
	if( getenv("IFS") == NULL  || strcmp(getenv("IFS"), " \t\n") != 0)
	{
		for ( i = 1 ; i <= NO_TESTS ; i++ )
			tet_delete(i, "failed to set environment variable IFS");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(n_dirs + n_dirs + n_files + 4);

	DBUG_VOID_RETURN;
}

/*
 * wordexp() performs shell word expansions, stores the number of words
 * generated in the we_wordc member, produces a NULL terminated list of
 * words in we_wordv, and returns 0.
 */

private void
test1()
{
private char *expect_a[] = { dir_003, dir_002, dir_004, (char *)NULL};
private char *expect_b[] = { dir_003, dir_002, dir_004,
				"hello",
				dir_008, dir_007, dir_006, (char *)NULL};
private char *expect_c[] = { dir_004, (char *)NULL};
private char *expect_d[] = { dir_004, (char *)NULL};
private char *expect_e[] = { dir_004, (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test1");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("dir_1/*able", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;

	if (wordexp_b.we_wordc > 0 &&
		wordexp_b.we_wordv[wordexp_b.we_wordc] != NULL)
	{
		if( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("we_wordv not correctly NULL terminated");
		in_rpt("we_wordv[%d] contains 0x%lx", wordexp_b.we_wordc,
				(long)wordexp_b.we_wordv[wordexp_b.we_wordc]);
	}
	else
		PATH_TRACE;

	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("dir_1/*able hello dir_2/*able", 0,
		expect_b, &wordexp_b);
	PATH_FUNC_TRACE;

	if (wordexp_b.we_wordc > 0 &&
		wordexp_b.we_wordv[wordexp_b.we_wordc] != NULL)
	{
		if( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("we_wordv not correctly NULL terminated");
		in_rpt("we_wordv[%d] contains 0x%lx", wordexp_b.we_wordc,
				(long)wordexp_b.we_wordv[wordexp_b.we_wordc]);
	}
	else
		PATH_TRACE;

	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("dir?1/unreadable", 0,
		expect_c, &wordexp_b);
	PATH_FUNC_TRACE;

	if (wordexp_b.we_wordc > 0 &&
		wordexp_b.we_wordv[wordexp_b.we_wordc] != NULL)
	{
		if( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("we_wordv not correctly NULL terminated");
		in_rpt("we_wordv[%d] contains 0x%lx", wordexp_b.we_wordc,
				(long)wordexp_b.we_wordv[wordexp_b.we_wordc]);
	}
	else
		PATH_TRACE;

	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("dir_1/un[Rr]eadable", 0,
		expect_d, &wordexp_b);
	PATH_FUNC_TRACE;

	if (wordexp_b.we_wordc > 0 &&
		wordexp_b.we_wordv[wordexp_b.we_wordc] != NULL)
	{
		if( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("we_wordv not correctly NULL terminated");
		in_rpt("we_wordv[%d] contains 0x%lx", wordexp_b.we_wordc,
				(long)wordexp_b.we_wordv[wordexp_b.we_wordc]);
	}
	else
		PATH_TRACE;

	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("dir_1/unreadable", 0, expect_e, &wordexp_b);
	PATH_FUNC_TRACE;

	if (wordexp_b.we_wordc > 0 &&
		wordexp_b.we_wordv[wordexp_b.we_wordc] != NULL)
	{
		if( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("we_wordv not correctly NULL terminated");
		in_rpt("we_wordv[%d] contains 0x%lx", wordexp_b.we_wordc,
				(long)wordexp_b.we_wordv[wordexp_b.we_wordc]);
	}
	else
		PATH_TRACE;

	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(10);

	DBUG_VOID_RETURN;
}

/*
 * an unquoted word containing the text $var or ${var} has the text
 * $var or ${var} replaced with the contents of the environment variable
 * var, or an empty string if var is not defined in the environment.
 */

private void
test2()
{
private char *expect_a[] = { (char *)NULL, (char *)NULL};
private char *expect_b[] = { (char *)NULL, (char *)NULL};
private char *expect_c[] = { (char *)NULL, (char *)NULL};
private char *expect_d[] = { (char *)NULL, (char *)NULL};
private char *expect_e[] = { "====", (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;
	char	tmp_c[_POSIX_PATH_MAX];
	char	tmp_d[_POSIX_PATH_MAX];

	DBUG_ENTER("test2");

	wordexp_b = empty_wordexp;
	(void) sprintf(tmp_c, "==%s==", env_home);
	(void) sprintf(tmp_d, "==%sR==", env_home);

	expect_a[0] = env_home;
	expect_b[0] = env_home;
	expect_c[0] = tmp_c;
	expect_d[0] = tmp_d;

	globok = 0;
	nfails += do_expand("$HOME", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("${HOME}", 0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("==${HOME}==", 0, expect_c, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("==${HOME}R==", 0, expect_d, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("==$HOMER==", 0, expect_e, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

/*
 * an unquoted word ~  or ~/ replaces the text ~
 * with the contents of the environment variable HOME
 */

private void
test3()
{
private char *expect_a[] = { "abc", (char *)NULL, "def", (char *)NULL};
private char *expect_b[] = { "abc", (char *)NULL, "def", (char *)NULL};
private char *expect_c[] = { "abc", "~", "def", (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;
	char	tmp_b[_POSIX_PATH_MAX];

	DBUG_ENTER("test3");

	wordexp_b = empty_wordexp;
	(void) sprintf( tmp_b, "%s/hello", env_home);
	expect_a[1] = env_home;
	expect_b[1] = tmp_b;

	globok = 0;
	nfails += do_expand("abc ~ def", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("abc ~/hello def", 0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("abc \\~ def", 0, expect_c, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/*
 * an unquoted word ~name  or ~name/ replaces the text ~name
 * with the path of names home directory.
 */

private void
test4()
{
private char *expect_a[] = { "abc", (char *)NULL, "def", (char *)NULL};
private char *expect_b[] = { "abc", (char *)NULL, "def", (char *)NULL};
private char *expect_c[] = { "abc", "~vsx1", "def", (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	char	tmp_b[_POSIX_PATH_MAX];
	wordexp_t	wordexp_b;

	DBUG_ENTER("test4");
	if( !strcmp(vsx1_home, env_home) )
	{
		xx_rpt(DELETION);
		in_rpt("vsx0 and vsx1 have same home directory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) sprintf( tmp_b, "%s/hello", vsx1_home);
	expect_a[1] = vsx1_home;
	expect_b[1] = tmp_b;

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("abc ~vsx1 def", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("abc ~vsx1/hello def", 0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("abc \\~vsx1 def", 0, expect_c, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

/*
 * a word containing text quoted in ' results in the quoted text
 * being left unchanged.
 */

private void
test5()
{
private char *expect_a[] = { "a*", "?", "[abc]", "$HOME", "ab cd",
	"~/hello", "~vsx1", "#", "\"a\"",
	(char *)NULL};

private char *expect_b[] = { (char *)NULL, (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	char	tmp_b[_POSIX_PATH_MAX];
	wordexp_t	wordexp_b;

	DBUG_ENTER("test5");
	(void) strcpy(tmp_b, env_home);
	(void) strcat(tmp_b, "$HOME");
	(void) strcat(tmp_b, env_home);
	expect_b[0] = tmp_b;

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand(
		"'a*' '?' '[abc]' '$HOME' a'b c'd '~/hello' '~vsx1' '#' '\"a\"'",
		0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("$HOME'$HOME'$HOME", 0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

/*
 * an unquoted word containing the text $(cmd) has the text $(cmd) replaced
 * with the results of executing the command cmd.
 */

private void
test6()
{
private char *expect_a[] = { "No", "recognised", "arguments", (char *)NULL};
private char *expect_b[] = { "ABabcdefgYZ", (char *)NULL};
private char *expect_c[] = { "abc", "line1", "line2", "xyz", (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test6");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("$(../wordexp.sh)", WRDE_SHOWERR,
		expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("AB$(../wordexp.sh b)YZ", WRDE_SHOWERR,
		expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("abc $(../wordexp.sh c) xyz", WRDE_SHOWERR,
		expect_c, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/*
 * an unquoted word containing the text `cmd` has the text `cmd` replaced
 * with the results of executing the command cmd.
 */

private void
test7()
{
private char *expect_a[] = { "No", "recognised", "arguments", (char *)NULL};
private char *expect_b[] = { "abcdefg", (char *)NULL};
private char *expect_c[] = { "abc", "line1", "line2", "xyz", (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test7");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("`../wordexp.sh`", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("`../wordexp.sh b`", 0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("abc `../wordexp.sh c` xyz", 0, expect_c, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/*
 * when a word contains a pair of matching quote characters ", the quotes are
 * removed and parameter and command substitution are performed but
 * all other characters are left unaltered.
 */

private void
test8()
{
private char *expect_a[] = { (char *)NULL, "line1\nline2", "a*", "?", "[abc]",
				"~/hello", "~vsx1", "#", (char *)NULL};
private char *expect_b[] = { (char *)NULL, (char *)NULL};
private char *expect_c[] = { "a\n\nb", "c\n\nd", (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	char	tmp_b[_POSIX_PATH_MAX];
	wordexp_t	wordexp_b;

	DBUG_ENTER("test8");

	wordexp_b = empty_wordexp;
	expect_a[0] = env_home;
	(void) sprintf(tmp_b, "'%s %s'", env_home, "line1\nline2");
	expect_b[0] = tmp_b;

	globok = 0;
	nfails += do_expand(
		"\"$HOME\" \"$(../wordexp.sh c)\" \"a*\" \"?\" \"[abc]\" \"~/hello\" \"~vsx1\" \"#\"",
		0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("\"'${HOME} `../wordexp.sh c`'\"",
		0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("\"a\n\n$(echo)b\" \"c\n\n`echo`d\"",
		0, expect_c, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/*
 * when a word contains a pair of matching quote characters ', the quotes are
 * removed and the text is not expanded.
 */

private void
test9()
{
private char *expect_a[] = { "$HOME", "$(../wordexp.sh b)", (char *)NULL};
private char *expect_b[] = { "==$HOME `../wordexp.sh c`==", (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test9");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("'$HOME' '$(../wordexp.sh b)'",
		0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("=='$HOME `../wordexp.sh c`'==",
		0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

/*
 * when the value of IFS is exactly ' ', '\t', and '\n' then any sequence
 * of IFS chars serves to delimit a field, and sequences of these chars
 * at the beginning or end of input are ignored.
 */

private void
test10()
{
private char *expect_a[] = { "a", "b", "c", "d", (char *)NULL};
private char *expect_b[] = { "a\fb", "c", (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test10");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand(" \ta\tb c\t\td\t ", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("a\fb c", 0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

/*
 * when IFS is set to ' ', '\t', '\n' and some other characters then
 * sequences of spaces etc. delimit fields as normal but single
 * occurrences of the other chars delimit fields.
 */

private void
test11()
{
	DBUG_ENTER("test11");

	(void) cppair(ch_t11, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
private char *expect[] = { "a:b", "c", "d", "e", "", "", "f", "g",
			   "h", "i", (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("ch_t11");
	wordexp_b = empty_wordexp;

	(void)vputenv(&environ, "IFS=: \t\n");
	if( getenv("IFS") == NULL  || strcmp(getenv("IFS"), ": \t\n") != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("failed to change IFS environment variable");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	nfails += do_expand("a:b `echo ' \tc:d  e:::f \tg \n'` h${IFS}i",
		0, expect, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


/*
 * When an unquoted '#' character appears, it is either treated as
 * a normal character or it and the text following it are ignored.
 */

private void
test12()
{
private char *expect[] = { "a", "b", "c",
	"##", "possible", "comment", (char *)NULL};
	/* note "##" is recognised by the checking functions, which treat
	   it as an optional "#" and treat the words after it as optional */

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test12");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("a b c # possible comment",
		0, expect, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/*
 * When used in a bracket expression, the characters ?, * and [ lose
 * their special meaning.
 */

private void
test13()
{
private char *expect_a[] = { file_016, file_010, (char *)NULL};
private char *expect_b[] = { file_017, file_010, (char *)NULL};
private char *expect_c[] = { file_010, file_013, (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test13");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("a[?X]b", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("a[*X]b", 0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("a[X[]b", 0, expect_c, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/*
 * A slash in a pathname is only matched by and explicit instance of /
 * in a pattern
 */

private void
test14()
{
private char *expect_a[] = { file_032, (char *)NULL};
private char *expect_b[] = { file_032, (char *)NULL};
private char *expect_c[] = { "dir_1[/X]a", (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test14");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("dir_1*a", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("dir_1?a", 0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand(*expect_c, 0, expect_c, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/*
 * when pattern contains *, any characters except /, leading dot and dot
 * following / are matched.
 */

private void
test15()
{
private char *expect_a[] = { file_019, file_006, file_017, file_015, 
	file_016, file_010, file_011, file_012, file_013, file_014, 
	file_023, file_024, file_025, file_007, file_008, file_009, 
	dir_001, file_032, dir_005, file_020, (char *)NULL};

private char *expect_b[] = { file_019, file_020, (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test15");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("*", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("*hidden", 0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

/*
 * when pattern contains ?, any single character except /, leading dot and dot
 * following / is matched.
 */

private void
test16()
{
private char *expect_a[] = { file_006, file_007, file_008, file_009,
	(char *)NULL};

private char *expect_b[] = { file_017, file_015, file_016, file_010,
	file_011, file_012, file_013, file_014, 
	file_023, file_024, file_025,
	(char *)NULL};

private char *expect_c[] = { file_032, (char *)NULL};
private char *expect_d[] = { file_019, (char *)NULL};
private char *expect_e[] = { file_030, (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test16");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("?", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("a?b", 0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("dir_1?a", 0, expect_c, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("?hidden", 0, expect_d, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("dir_2/?rb", 0, expect_e, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

/*
 * when pattern contains [s], any one of the characters in the set s
 * except /, leading dot and dot following / are matched.
 */

private void
test17()
{
private char *expect_a[] = { file_011, file_012, file_013, file_014,
	file_023, file_024, (char *)NULL};

private char *expect_b[] = { file_023, file_024, file_025, (char *)NULL};
private char *expect_c[] = { file_019, (char *)NULL};
private char *expect_d[] = { file_030, (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test17");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("a[Y-y]b", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("a[a-z]b", 0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("[.X]hidden", 0, expect_c, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("dir_2/[.a]rb", 0, expect_d, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test18()
{
	DBUG_ENTER("test18");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

/*
 * when pattern contains [!s], any characters not in the set s
 * except /, leading dot and dot following / are matched.
 */

private void
test19()
{
private char *expect_a[] = { file_001, file_021, dir_002, dir_004, dir_009,
		(char *)NULL};
private char *expect_b[] = { file_020, (char *)NULL};
private char *expect_c[] = { file_032, (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test19");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("dir_1/[!A-Z]*", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("[!A-Z]*den", 0, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	globok = 0;
	nfails += do_expand("dir_1[!a]a", 0, expect_c, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/*
 * When a pattern starts with a /, then filename matching starts at the
 * calling processes root directory.
 */

private void
test20()
{
private char *expect[] = { file_006, file_007, file_008, file_009,
		(char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;
	char	root_pat[_POSIX_PATH_MAX];
	char	root_match[_POSIX_PATH_MAX * N_EXPECTED(expect)];
	char	*root_expect[N_EXPECTED(expect) +1];

	DBUG_ENTER("test20");

	wordexp_b = empty_wordexp;
	if( getcwd(root_pat, _POSIX_PATH_MAX) == NULL ) {
		xx_rpt(DELETION);
		in_rpt("failed to find current directory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) strcat(root_pat,"/");
	prefix_expected(root_expect,expect,root_match,root_pat);

	(void) strcat(root_pat,"?");

	globok = 0;
	nfails += do_expand(root_pat, 0, root_expect, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

/*
 * When a pattern does not start with a /, then filename matching starts at the
 * calling processes current directory, interpreting ../ as the parent
 * directory of the previous pathname element.
 */

private void
test21()
{
	DBUG_ENTER("test21");

	(void) cppair(ch_t21, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t21()
{
private char *expect[] = { dir_003, file_001, file_021, dir_002,
		dir_004, dir_009,
		(char *)NULL};

	int	nfails = 0;
	int	pathok = 0;
	wordexp_t	wordexp_b;
	char	match[_POSIX_PATH_MAX * N_EXPECTED(expect)];
	char	*pr_expect[N_EXPECTED(expect) +1];

	DBUG_ENTER("ch_t21");
	wordexp_b = empty_wordexp;

	if ( chdir(dir_005)  == SYSERROR )
	{
		DBUG_PRINTF("chdir", dir_005);
		DBUG_PRINTF("chdir", "errno = %s", errname(errno));
		xx_rpt(DELETION);
		in_rpt("failed to chdir directory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	prefix_expected(pr_expect,expect,match,"../");

	globok = 0;
	nfails += do_expand("../dir_1/*", 0, pr_expect, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

/*
 * a call to wordexp allocates the memory pointed to by the we_wordv member
 */

private void
test22()
{
private char *expect_a[] = { file_010, file_011, file_012,
		file_023, file_024, file_025,
		(char *)NULL};

private char *expect_b[] = { dir_003, dir_002, dir_004,
		(char *)NULL};

	int	nfails = 0;
	int	n;
	int	pathok = setupok;
	wordexp_t	wordexp_ba;
	wordexp_t	wordexp_bb;
	char	**bad_val;

	DBUG_ENTER("test22");

	bad_val = &test_vecp;
	wordexp_ba = empty_wordexp;
	wordexp_ba.we_wordv = bad_val;
	wordexp_bb = wordexp_ba;
	globok = 0;
	nfails += do_expand("a[a-zA-Z]b", 0, expect_a, &wordexp_ba);
	PATH_FUNC_TRACE;

	if (wordexp_ba.we_wordc != 0)
	{
		if( wordexp_ba.we_wordv == bad_val )
		{
			nfails++;
			xx_rpt(FAILURE);
			in_rpt("wordexp() has left we_wordv unchanged");
		}
		else
			PATH_TRACE;

		globok = 0;
		nfails += do_expand("dir_1/*able", 0, expect_b, &wordexp_bb);
		PATH_FUNC_TRACE;

		if( nfails == 0 )
		{
			globok = 0;
			n = return_chk("dir_1/*able", expect_a, &wordexp_ba);
			PATH_FUNC_TRACE;
			nfails += n;
			if( n )
				in_rpt("second wordexp() changed results of first wordexp()");
		}

		wordfree(&wordexp_ba);
		wordfree(&wordexp_bb);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


/*
 * When WRDE_APPEND is in flags, the matched patterns are stored after
 * the current set of paths.
 */

private void
test23()
{
private char *expect_a[] = { file_001, file_021, dir_002, dir_004,
		dir_009, (char *)NULL};

private char *expect_b[] = { file_001, file_021, dir_002, dir_004,
		dir_009, file_006, file_007, file_008, file_009,
		(char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test23");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("dir_1/[!A-Z]*", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;

	globok = 0;
	nfails += do_expand("?", WRDE_APPEND, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

/*
 * When WRDE_DOOFFS is set in flags.
 */

private void
test24()
{
private char *expect_a[] = { file_001, file_021, dir_002, dir_004,
		dir_009, (char *)NULL};

private char *expect_b[] = { file_001, file_021, dir_002, dir_004,
		dir_009, file_006, file_007, file_008, file_009,
		(char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;
	int	offset = 3;

	DBUG_ENTER("test24");

	wordexp_b = empty_wordexp;
	wordexp_b.we_offs = offset;

	globok = 0;
	nfails += do_expand("dir_1/[!A-Z]*", WRDE_DOOFFS, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;

	globok = 0;
	nfails += do_expand("?", WRDE_DOOFFS|WRDE_APPEND, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;

	if( wordexp_b.we_wordc > 0 &&
		wordexp_b.we_wordv[wordexp_b.we_wordc + wordexp_b.we_offs] != NULL)
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("we_wordv not correctly NULL terminated");
		in_rpt("we_wordv[%d] contains 0x%lx",
			wordexp_b.we_wordc + wordexp_b.we_offs,
			(long)wordexp_b.we_wordv[wordexp_b.we_wordc + wordexp_b.we_offs]);

		wordfree(&wordexp_b);
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


/*
 * a call to wordexp with WRDE_REUSE frees any space used by the wordexp_t
 * structure before allocating a new series of pointers.
 */

private void
test25()
{
private char *expect_a[] = { file_010, file_011, file_012,
		file_023, file_024, file_025,
		(char *)NULL};

private char *expect_b[] = { dir_003, dir_002, dir_004,
		(char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	DBUG_ENTER("test25");

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("a[a-zA-Z]b", 0, expect_a, &wordexp_b);
	PATH_FUNC_TRACE;

	globok = 0;
	nfails += do_expand("dir_1/*able", WRDE_REUSE, expect_b, &wordexp_b);
	PATH_FUNC_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(2);

	wordfree(&wordexp_b);

	DBUG_VOID_RETURN;
}

/*
 * When WRDE_SHOWERR is not set then wordexp redirects stderr to /dev/null
 * during execution of the command.
 */

private void
test26()
{
	DBUG_ENTER("test26");

	(void) cppair(ch_t26, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t26()
{
private char *expect[] = { "stdout", (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	int	tmp_fd;
	struct	stat	sb;
	wordexp_t	wordexp_b;

	DBUG_ENTER("ch_t26");

	if( (tmp_fd = creat(tmp_file,MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to creat tmp file %s",
				tmp_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void)close(2);
	if( dup(tmp_fd) != 2 )
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to dup file onto fd 2" );
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("$(../wordexp.sh e)", 0, expect, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if( fstat(tmp_fd, &sb) == SYSERROR )
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat on %s failed", tmp_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if( sb.st_size > 0 )
	{
		if( nfails++ == 0 )
			xx_rpt(FAILURE);
		in_rpt("wordexp() failed to redirect stderr to /dev/null");
	}
	else
		PATH_TRACE;

	if ( nfails == 0 )
		PATH_XS_RPT(5);

	(void) unlink(tmp_file);

	DBUG_VOID_RETURN;
}

/*
 * When WRDE_SHOWERR is set then wordexp does not redirect stderr to /dev/null
 * during execution of the command.
 */

private void
test27()
{
	DBUG_ENTER("test27");

	(void) cppair(ch_t27, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t27()
{
private char *expect[] = { "stdout", (char *)NULL};

	int	nfails = 0;
	int	pathok = setupok;
	int	tmp_fd;
	struct	stat	sb;
	wordexp_t	wordexp_b;

	DBUG_ENTER("ch_t27");

	if( (tmp_fd = creat(tmp_file,MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to creat tmp file %s",
				tmp_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void)close(2);
	if( dup(tmp_fd) != 2 )
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: failed to dup file onto fd 2" );
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	wordexp_b = empty_wordexp;

	globok = 0;
	nfails += do_expand("$(../wordexp.sh e)", WRDE_SHOWERR,
		expect, &wordexp_b);
	PATH_FUNC_TRACE;
	wordfree(&wordexp_b);

	if (wordexp_b.we_wordc > 0)
	{
		if( fstat(tmp_fd, &sb) == SYSERROR )
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: fstat on %s failed", tmp_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if( sb.st_size == 0 )
		{
			if( nfails++ == 0 )
				xx_rpt(FAILURE);
			in_rpt("wordexp() redirected stderr to /dev/null when WRDE_SHOWERR flag was on");
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(5);

	(void) unlink(tmp_file);

	DBUG_VOID_RETURN;
}

/*
 * When words contains unquoted shell special chars or unquoted {, }, (, or )
 * WDRE_BADCHAR is returned and the wordexp_t struct is unchanged.
 */

private void
test28()
{
	char	**pattern_p;
	int	nfails = 0;
	int	rtval, err;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	static char *patterns[] = { "abc|efg", "abc&efg", "abc;efg",
		"abc<efg", "abc>efg", "{ abcefg }", "(abcefg)", NULL };

	DBUG_ENTER("test28");

	for (pattern_p = patterns; *pattern_p != NULL; pattern_p++)
	{
		wordexp_b = ref_wordexp_b;
		errno = 0;
		rtval = wordexp(*pattern_p, &wordexp_b, 0);
		err = errno;

		globok = 0;
		nfails += chk_sup(*pattern_p, 0, rtval, err);
		PATH_FUNC_TRACE;

		if (rtval != WRDE_NOSYS || err != ENOSYS)
		{
			if ( rtval != WRDE_BADCHAR )
			{
				if( nfails++ == 0 )
					xx_rpt(FAILURE);
				in_rpt("wordexp(\"%s\", ...) returned %d instead of WRDE_BADCHAR",
						*pattern_p, rtval);
			}
			else
				PATH_TRACE;
		}
		else
			PATH_TRACE;

		globok = 0;
		nfails += chk_wordexp(&wordexp_b);
		PATH_FUNC_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(21);

	DBUG_VOID_RETURN;
}

/*
 * When WDRE_UNDEF is in flags and words contains a ref to an undefined shell
 * variable, WDRE_BADVAL is returned and the wordexp_t struct is unchanged.
 */

private void
test29()
{
	char	**pattern_p;
	int	nfails = 0;
	int	rtval, err;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	static char *patterns[] = {
		"abc${UNDEFINED}efg", "abc $UNDEFINED efg", NULL
	};

	DBUG_ENTER("test29");

	for (pattern_p = patterns; *pattern_p != NULL; pattern_p++)
	{
		wordexp_b = ref_wordexp_b;
		errno = 0;
		rtval = wordexp(*pattern_p, &wordexp_b, WRDE_UNDEF);
		err = errno;

		globok = 0;
		nfails += chk_sup(*pattern_p, 0, rtval, err);
		PATH_FUNC_TRACE;

		if (rtval != WRDE_NOSYS || err != ENOSYS)
		{
			if ( rtval != WRDE_BADVAL )
			{
				if( nfails++ == 0 )
					xx_rpt(FAILURE);
				in_rpt("wordexp(\"%s\", pwordexp, WRDE_UNDEF) returned %d instead of WRDE_BADVAL",
						*pattern_p, rtval);
			}
			else
				PATH_TRACE;
		}
		else
			PATH_TRACE;

		globok = 0;
		nfails += chk_wordexp(&wordexp_b);
		PATH_FUNC_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

/*
 * When WDRE_NOCMD is in flags and words contains a string that requires
 * command substitution, WDRE_CMDSUB is returned and the wordexp_t struct
 * is unchanged.
 */

private void
test30()
{
	char	**pattern_p;
	int	nfails = 0;
	int	rtval, err;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	static char *patterns[] = {
		"abc`../wordexp.sh a`efg", "abc $(../wordexp.sh a) efg",
		NULL
	};

	DBUG_ENTER("test30");

	for (pattern_p = patterns; *pattern_p != NULL; pattern_p++)
	{
		wordexp_b = ref_wordexp_b;
		errno = 0;
		rtval = wordexp(*pattern_p, &wordexp_b, WRDE_NOCMD);
		err = errno;

		globok = 0;
		nfails += chk_sup(*pattern_p, 0, rtval, err);
		PATH_FUNC_TRACE;

		if (rtval != WRDE_NOSYS || err != ENOSYS)
		{
			if ( rtval != WRDE_CMDSUB )
			{
				if( nfails++ == 0 )
					xx_rpt(FAILURE);
				in_rpt("wordexp(\"%s\", pwordexp, WRDE_NOCMD) returned %d instead of WRDE_CMDSUB",
						*pattern_p, rtval);
			}
			else
				PATH_TRACE;
		}
		else
			PATH_TRACE;

		globok = 0;
		nfails += chk_wordexp(&wordexp_b);
		PATH_FUNC_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

/*
 * When a shell syntax error occurs during command expansion, WRDE_SYNTAX is
 * returned and the wordexp_t struct is unchanged.
 */

private void
test31()
{
	char	**pattern_p;
	int	nfails = 0;
	int	rtval, err;
	int	pathok = setupok;
	wordexp_t	wordexp_b;

	static char *patterns[] = {
		"`for i in \ndone`", "$(for i in \ndone)", NULL
	};

	DBUG_ENTER("test31");

	for (pattern_p = patterns; *pattern_p != NULL; pattern_p++)
	{
		wordexp_b = ref_wordexp_b;
		errno = 0;
		rtval = wordexp(*pattern_p, &wordexp_b, 0);
		err = errno;

		globok = 0;
		nfails += chk_sup(*pattern_p, 0, rtval, err);
		PATH_FUNC_TRACE;

		if (rtval != WRDE_NOSYS || err != ENOSYS)
		{
			if ( rtval != WRDE_SYNTAX )
			{
				if( nfails++ == 0 )
					xx_rpt(FAILURE);
				in_rpt("wordexp(\"%s\", ...) returned %d instead of WRDE_SYNTAX",
					pr_nchars(*pattern_p,
					(int)strlen(*pattern_p)), rtval);
			}
			else
				PATH_TRACE;
		}
		else
			PATH_TRACE;

		globok = 0;
		nfails += chk_wordexp(&wordexp_b);
		PATH_FUNC_TRACE;
	}

	if ( nfails == 0 )
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}
