/*
*      SCCS:  @(#)  LSB.os/ipc/ftok_L/ftok_L.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)LSB.os/ipc/ftok_L - T.ftok_L Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.ftok_L

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the ftok() subroutine.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Tue Feb 20 11:02:51 GMT 2001
MODIFICATIONS:	

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/ipc.h>
#include	<sys/stat.h>
#include	<pathres.h>
#include	<errno.h>
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ftok
#endif

#define MODEANY         (mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

#define	NO_TESTS	10	/* number of tests in testset */

extern	char	*errname();
extern	char	**pathtoolng();
extern	int	cr_sym_loop();

int	no_tests = NO_TESTS;

private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	test6();
private	void	test7();
private	void	test8();
private	void	test9();
private	void	test10();

private char	*t3_file  =     "ftok-t.3";
private char	*t3_link  =     "ftok-t.3l";
private char	*t4_file  =     "ftok-t.4";
private char	*t5_dir  =      "ftok-t.5";
private char	*t5_file  =     "ftok-t.5/ftok-t.5";
private char	*t6_file1  =    "ftok-t.6a";
private char	*t6_file2  =    "ftok-t.6b";
private char	*t7_file  =     "ftok-t.7";
private char    *t10_ndir =     "ftok-t.10";
private char    *t10_file =     "ftok-t.10/ftok-t.10";

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 1,
	 test3, 1,
	 test4, 1,
	 test5, 1,
	 test6, 1,
	 test7, 1,
	 test8, 1,
	 test9, 1,
	 test10, 1,
	NULL, 0
};

private void
test1()
{
        /* pathname resolution */

        char *p1, *p2;
	int err1, err2;
        int npairs, i;
	int id;
	int testfail;
        struct stat stbuf1, stbuf2;
        int     pathok = 0;
        key_t  ret1, ret2;

        DBUG_ENTER("test1");

        testfail = 0;

        npairs = init_pathres(PRES_DIRS|PRES_CREAT);
        if (npairs <= 0)
	{
 		xx_rpt(DELETION);
                in_rpt("init_pathres(PRES_DIRS|PRES_CREAT) failed");
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

        for (i = 0; i < npairs; i++)
        {
                p1 = pathresnam[2*i];
                p2 = pathresnam[2*i+1];

                DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		id = 0xff;
		errno = 0;
                ret1 = ftok(p1, id);
		err1 = errno;
                PATH_TRACE;

		errno = 0;
                ret2 = ftok(p2, id);
		err2 = errno;
                PATH_TRACE;

                if (ret1 == (key_t) -1 || ret2 == (key_t) -1)
                {
			testfail++;
			xx_rpt(FAILURE);
			if (ret1 == (key_t) -1)
			{
				in_rpt("ftok(%s, 0xff) failed with errno %d(%s)", p1, err1, errname(err1));
			}
			if (ret2 == (key_t) -1)
			{
				in_rpt("ftok(%s, 0xff) failed with errno %d(%s)", p2, err2, errname(err2));
			}
                }
		PATH_TRACE;
	}

 	if (testfail == 0)
        {
                PATH_XS_RPT(1 + (npairs * 3));
        }

        cln_pathres();

        DBUG_VOID_RETURN;
}

private void
test2()
{
        /* portable filenames */

        char    **p;
	int	id;
	int	err1;
        int     nfile, i, j, fd;
	int	testfail;
        int     pathok = 0;
        int     iter = 0;

        DBUG_ENTER("test2");

        testfail = 0;
        iter = 1;                       /* PATH_TRACE 1st iteration only */

	for (nfile = 0, p = portfnam; *p != NULL; p++, nfile++)
        {
                if ((fd = crfile(*p, MODEANY,
                                (int)geteuid(), (int)getegid())) == SYSERROR)
                {
                        xx_rpt(DELETION);
                        in_rpt("could not create file \"%s\"", *p);
                        DBUG_VOID_RETURN;
                }
                else if (iter)
                {
                        PATH_TRACE;
                        iter = 0;               /* Only trace the forst iteratio
n */
                }
                (void) close(fd);
        }

	for (i = 0, p = portfnam; i < nfile; p++, i++)
        {
		id = 0xff;
                if (ftok(*p, id) == (key_t) -1)
                {
			testfail++;
			err1 = errno;
			xx_rpt(FAILURE);
			in_rpt("ftok(%s, 0xff) failed with errno %d(%s)", *p, err1, errname(err1));
                }
                PATH_TRACE;
        }

        if (testfail == 0)
        {
                PATH_XS_RPT(nfile + 1);
        }

        for (p = portfnam; *p != NULL; p++)
                (void) unlink(*p);

        DBUG_VOID_RETURN;
}

private void
test3()
{
	int	id;
	int	fd;
	key_t	ret1, ret2;
	int	err1, err2;
 	mode_t  mode = (mode_t)(S_IRWXU|S_IWGRP|S_IXOTH);
	int	testfail;
	int	pathok = 0;

	DBUG_ENTER("test3");
	testfail = 0;

 	if ((fd = crfile(t3_file, mode,
                        (int)geteuid(), (int)getegid())) == SYSERROR)
        {
                xx_rpt(DELETION);
                in_rpt("could not create file \"%s\"", t3_file);
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

	(void) close(fd);

	if (symlink(t3_file, t3_link) == -1)
	{
		xx_rpt(DELETION);
		in_rpt("could not create symbolic link to \"%s\"", t3_file);
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

	id = 0xff;
	errno = 0;
        ret1 = ftok(t3_file, id);
	err1 = errno;
        PATH_TRACE;

	errno = 0;
        ret2 = ftok(t3_link, id);
	err2 = errno;
        PATH_TRACE;

        if (ret1 == (key_t) -1 || ret2 == (key_t) -1)
        {
		testfail++;
		xx_rpt(FAILURE);
		if (ret1 == (key_t) -1)
		{
			in_rpt("ftok(%s, 0xff) failed with errno %d(%s)", t3_file, err1, errname(err1));
		}
		if (ret2 == (key_t) -1)
		{
			in_rpt("ftok(%s, 0xff) failed with errno %d(%s)", t3_link, err2, errname(err2));
		}
        }
	PATH_TRACE;

	if (ret1 != ret2)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("ftok() did not return same key for two symbolic links");
	}
	PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(6);

	(void) unlink(t3_file);
	(void) unlink(t3_link);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	id;
	int	fd;
	key_t	ret1, ret2;
	int	err1, err2;
 	mode_t  mode = (mode_t)(S_IRWXU|S_IWGRP|S_IXOTH);
	int	testfail;
	int	pathok = 0;

	DBUG_ENTER("test4");
	testfail = 0;

 	if ((fd = crfile(t4_file, mode,
                        (int)geteuid(), (int)getegid())) == SYSERROR)
        {
                xx_rpt(DELETION);
                in_rpt("could not create file \"%s\"", t4_file);
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

	(void) close(fd);

	id = 0xf0;
	errno = 0;
        ret1 = ftok(t4_file, id);
	err1 = errno;
        PATH_TRACE;

	id = 0xf1;
	errno = 0;
        ret2 = ftok(t4_file, id);
	err2 = errno;
        PATH_TRACE;

        if (ret1 == (key_t) -1 || ret2 == (key_t) -1)
        {
		testfail++;
		xx_rpt(FAILURE);
		if (ret1 == (key_t) -1)
		{
			in_rpt("ftok(%s, 0xf0) failed with errno %d(%s)", t4_file, err1, errname(err1));
		}
		if (ret2 == (key_t) -1)
		{
			in_rpt("ftok(%s, 0xf1) failed with errno %d(%s)", t4_file, err2, errname(err2));
		}
        }
	PATH_TRACE;

	if (ret1 == ret2)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("ftok() returned same key for same file with different idents");
	}
	PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(5);

	(void) unlink(t4_file);

	DBUG_VOID_RETURN;
}


private void
test5()
{
 	/* search permission denied */

	int	id;
	key_t	res;
	int	testfail;
        int     fd, err;
        mode_t  mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
        int     pathok = 0;

        DBUG_ENTER("test5");

        testfail = 0;

        if (crdir(t5_dir, MODEANY,
                        (int)geteuid(), (int)getegid()) == SYSERROR)
        {
                xx_rpt(DELETION);
                in_rpt("could not create directory \"%s\"", t5_dir);
                DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t5_file, MODEANY,
                        (int) geteuid(), (int) getegid())) == SYSERROR)
        {
                xx_rpt(DELETION);
                in_rpt("could not create file \"%s\"", t5_file);
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;
        (void) close(fd);

        if (chmod(t5_dir, mode) == SYSERROR)
        {
                err = errno;
                xx_rpt(DELETION);
                in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
                        t5_dir, (int)mode, err, errname(err));
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

	(void) umask(0);
	id = 0xff;
	res = ftok(t5_file, id);

	if (res != (key_t) -1 || errno != EACCES)
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("A call to ftok() with search permission denied did not fail");
		in_rpt("Expected Return: -1, errno EACCES");
		in_rpt("Actual Return: %d, errno %d(%s)", res, err, errname(err));
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

 	(void) chmod(t5_dir, MODEANY);
        (void) unlink(t5_file);
        (void) rmdir(t5_dir);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	int	id;
	key_t	res;
	int	testfail;
        int     fd, err;
        int     pathok = 0;
	int	globok = 0;

        DBUG_ENTER("test6");

        testfail = 0;

        if (cr_sym_loop(t6_file1, t6_file2) == SYSERROR)
        {
                xx_rpt(DELETION);
                in_rpt("could not create loop of symlinks");
                DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	(void) umask(0);
	id = 0xff;
	res = ftok(t6_file1, id);

	if (res != (key_t) -1 || errno != ELOOP)
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("A call to ftok() with looped symbolic link did not fail");
		in_rpt("Expected Return: -1, errno ELOOP");
		in_rpt("Actual Return: %d, errno %d(%s)", res, err, errname(err));
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

        (void) unlink(t6_file1);
        (void) unlink(t6_file2);

	DBUG_VOID_RETURN;
}

private void
test7()
{
 	char    **longname;
        int     fd;
	int	id;
	int	err;
	int	res;
	int	testfail;
	int	pathok = 0;

	DBUG_ENTER("test7");
	testfail = 0;

        if ((fd = creat(t7_file, (mode_t) MODEANY)) == SYSERROR)
        {
                xx_rpt(DELETION);
                in_rpt("could not create file \"%s\"", t7_file);
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;
        (void) close(fd);

 	globok = 0;
        longname = pathtoolng(t7_file);
        if (longname == NULL)
        {
                DBUG_VOID_RETURN;
        }
        else
                PATH_FUNC_TRACE;

	id = 0xff;
        res = ftok(longname[0], id);
	if (res == (key_t) -1)
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("A call to ftok() with maximum sized path name failed");
		in_rpt("Actual Return: %d, errno %d(%s)", res, err, errname(err));
	}
	else
		PATH_TRACE;

        if (longname[1] != NULL)
        {
		id = 0xff;
		res = ftok(longname[1], id);
		if (res != (key_t) -1 || errno != ENAMETOOLONG)
		{
			err = errno;
			testfail++;
			xx_rpt(FAILURE);
			in_rpt("A call to ftok() with path name too long did not fail");
			in_rpt("Expected Return: -1, errno ENAMETOOLONG");
			in_rpt("Actual Return: %d, errno %d(%s)", res, err, errname(err));
		}
		else
			PATH_TRACE;
        }
        else
                PATH_TRACE;

 	if (testfail == 0)
                PATH_XS_RPT(4);

        clntoolng();
        (void) unlink(t7_file);

        DBUG_VOID_RETURN;
}


private void
test8()
{
	int		id;
	int		res;
	int		err;
	int		testfail;
	int	pathok = 0;

	DBUG_ENTER("test8");
	testfail = 0;
	(void) umask(0);
	id = 0xff;
	res = ftok("NotAnExistingFile", id);

	if (res != (key_t) -1 || errno != ENOENT)
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("A call to ftok() with a non-existant file name did not fail");
		in_rpt("Expected Return: -1, errno ENOENT");
		in_rpt("Actual Return: %d, errno %d(%s)", res, err, errname(err));
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	int		id;
	int		res;
	int		err;
	int		testfail;
	int	pathok = 0;

	DBUG_ENTER("test9");
	testfail = 0;
	(void) umask(0);
	id = 0xff;
	res = ftok("", id);

	if (res != (key_t) -1 || errno != ENOENT)
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("A call to ftok() with an empty file name did not fail");
		in_rpt("Expected Return: -1, errno ENOENT");
		in_rpt("Actual Return: %d, errno %d(%s)", res, err, errname(err));
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test10()
{ 
	int     fd;
	int	id;
	int	res;
	int	err;
	int	testfail;
        int     pathok = 0;

        DBUG_ENTER("test10");

        testfail = 0;

        if ((fd = crfile(t10_ndir, MODEANY,
                        (int) geteuid(), (int) getegid())) == SYSERROR)
        {
                xx_rpt(DELETION);
                in_rpt("could not create file \"%s\"", t10_ndir);
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

        (void) close(fd);

	id = 0xff;
	res = ftok(t10_file, id);

	if (res != (key_t) -1 || errno != ENOTDIR)
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("A call to ftok() with a file path element not a directory did not fail");
		in_rpt("Expected Return: -1, errno ENOTDIR");
		in_rpt("Actual Return: %d, errno %d(%s)", res, err, errname(err));
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
