/*
*	SCCS:  @(#) XPG4.os/ipc/msgctl/msgctl.c Rel 4.4.1     (06/09/97)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)  XPG4.os/ipc/msgctl/msgctl.c Rel 4.4.1     (06/09/97)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.msgctl

PROJECT:	XPG4 Validation suite

DESCRIPTION:
	Testset for the msgctl() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Keith Holder
DATE CREATED:	Thu Aug  9 12:24:50 BST 1990
MODIFICATIONS:	

		Matt Newman, 9 Sept 1991
			XPG4 Integration

	Peter Skan, 7 Feb 1992
		Added setprv(PRV_ASSIGN)
		Assign privileges to exec'ed progs using prv_assign()
		Remove privileges with unsetprv() where necessary

	John-Paul, 13 July 1992 
		Test 16: Message queue size decreased before increase to avoid
		wrap-round problem.

	John-Paul, 14 July 1992 
		Tests 15, 16, 17 and 18: Added missing calls to remove message
		queue for the test failures.
	
	Geoff Clare, 28 May 1993
		Initialize msqid_ds structure in ch2_t15() with an IPC_STAT.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<signal.h>
#include	<dbug.h>
#include	<report.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<sys/ipc.h>
#include	<sys/msg.h>
#include	<setprv.h>
#include	<timeout.h>
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef msgctl
#endif


#define 	NO_TESTS	18		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)
#define		MODEANY		((mode_t)(S_IRWXU|S_IRWXG|S_IRWXO))
#define		MAXARG		128
#define		RD_FD		0
#define		WR_FD		1
#define		NOK		(2)
#define		SU_UID		0

int	no_tests = NO_TESTS;

#ifndef ftok
extern	key_t	ftok();
#endif
extern	char	*errname();
extern	char	*newstr();
extern	void	valrpt();
extern	int	uids();
extern	int	setprv();
extern	int	crfile();
extern	int	cppair();
extern	char	*itoa();
extern	char	*ltoa();

extern	int	vguid0,vggid0,vguid1,vggid1,vguid2,vggid2;
extern	char	*vg_ident;
extern	char	*vg_dbug;
private void	comp();
private void	mcreate();
private void	ch_comp();
private	void	test13();
private	void	test14();
private	void	test15();
private	void	ch1_t15();
private	void	ch2_t15();
private	void	test16();
private	void	ch1_t16();
private	void	test17();
private	void	ch1_t17();
private	void	ch2_t17();
private	void	test18();
private	void	ch1_t18();
private	void	ch2_t18();
private	int	prog_mode();
private	int	msg_create();
private	void	setup();
private	void	msgqrm();
private	void	tidy();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = tidy;
public	struct tet_testlist tet_testlist[] = {
	 comp, 1,
	 comp, 2,
	 comp, 3,
	 comp, 4,
	 comp, 5,
	 comp, 6,
	 comp, 7,
	 comp, 8,
	 comp, 9,
	 comp, 10,
	 comp, 11,
	 comp, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	NULL, 0
};

private	char	*p_argv[MAXARG];
private	int	pipe_fd[2];
private	gid_t	*suppgrps;
private	int	ngroups;

private	int	g_msgqid;


/*
 * message access permissions
 */
#define	RW_U		(S_IRUSR|S_IWUSR)
#define	RW_G		(S_IRGRP|S_IWGRP)
#define	RW_O		(S_IROTH|S_IWOTH)
#define	RW_UG		(mode_t)(RW_U|RW_G)
#define	W_UG		(mode_t)(S_IWUSR|S_IWGRP)
#define	RW_UGO		(mode_t)(RW_U|RW_G|RW_O)

/* exec file permissions */
#define	MODEXANY	(S_IRWXU|S_IRWXG|S_IRWXO)
#define	SUX		(MODEXANY|S_ISUID)
#define	SGX		(MODEXANY|S_ISGID)

/*
 * indices into [ug]idlist table.
 */
#define	VSX0	0
#define	VSX1	1
#define	VSX2	2
#define	SU	3	/* super-user/ appropriate privileges */

#define	MSGFLAG		((IPC_CREAT|IPC_EXCL)|RW_U|RW_G)
#define	ID		0x1235
#define	QSZ_INC		64


char	*msgpath	= "./msgctl_t";
char	*msgprog	= "../msgctl_prg";

uid_t	uidlist[4];
gid_t	gidlist[4];

struct	tdat {
	int	ruid;	/* real uid index */
	int	rgid;	/* real gid index */
	int	euid;	/* effective uid index */
	int	egid;	/* effective gid index */
	mode_t	perm;	/* access permission to message queue */
	mode_t	pmode;	/* file permissions on exec'd program */
	int	msg_cmd;/* msgctl() cmd */
};

struct	tdat	tdat[] = {
/*
ruid	rguid	euid_ix	egid_ix	perm	pmode	msg_cmd 
*/
VSX0,	VSX0,	VSX0,	VSX0,	0,	0,	0,		/* not used */
VSX2,	VSX2,	VSX1,	VSX2,	RW_UG,	SUX,	IPC_STAT,	/* test 1 */
VSX2,	VSX2,	VSX0,	VSX2,	RW_UG,	SUX,	IPC_STAT,	/* test 2 */
VSX2,	VSX2,	VSX1,	VSX1,	RW_UG,	SGX,	IPC_STAT,	/* test 3 */
VSX2,	VSX2,	VSX0,	VSX0,	RW_UG,	SGX,	IPC_STAT,	/* test 4 */
VSX2,	VSX2,	VSX2,	VSX2,	RW_UGO,	MODEXANY, IPC_STAT,	/* test 5 */
VSX2,	VSX2,	SU,	VSX2,	RW_UG,	SUX,	IPC_STAT,	/* test 6 */
VSX2,	VSX2,	VSX1,	VSX2,	RW_UG,	SUX,	IPC_SET,	/* test 7 */
VSX2,	VSX2,	VSX0,	VSX2,	RW_UG,	SUX,	IPC_SET,	/* test 8 */
VSX2,	VSX2,	SU,	VSX2,	RW_UG,	SUX,	IPC_SET,	/* test 9 */
VSX2,	VSX2,	VSX1,	VSX2,	RW_UG,	SUX,	IPC_RMID,	/* test 10 */
VSX2,	VSX2,	VSX0,	VSX2,	RW_UG,	SUX,	IPC_RMID,	/* test 11 */
VSX2,	VSX2,	SU,	VSX2,	RW_UG,	SUX,	IPC_RMID,	/* test 12 */
};

private	int	privileges[] = {
	PRV_IPC,
	0
};

private	int
msg_create(ouid,ogid,perm)
uid_t	ouid;
gid_t	ogid;
mode_t	perm;
{
	int msgqid;
	key_t	key;
	int	err;
	int	pathok;
	struct	msqid_ds buf;
	int	msgflag;

	DBUG_ENTER("msg_create");
	DBUG_PRINTF("args","msg_create(ouid = %d, ogid = %d, perm = 0%o",ouid,ogid,perm);

	pathok = 0;
	key = ftok(msgpath,ID);

	if ( key == -1 ) {
		xx_rpt(DELETION);
		in_rpt("ftok failed");
		DBUG_RETURN(SYSERROR);
	} else
		PATH_TRACE;

	msgflag = MSGFLAG;

	if ( (msgqid = msgget(key,msgflag)) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgget failed - errno %d (%s)",err,errname(err));
		DBUG_RETURN(SYSERROR);
	} else
		PATH_TRACE;

	DBUG_PRINTF("msg_create","msgqid = %d",msgqid);
	if ( msgctl(msgqid,IPC_STAT,&buf) == -1 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid,IPC_STAT) failed - errno %d (%s)",
			err,errname(err));
		DBUG_RETURN(SYSERROR);
	} else
		PATH_TRACE;

	buf.msg_perm.uid = ouid;
	buf.msg_perm.gid = ogid;
	buf.msg_perm.mode = perm;

	if ( msgctl(msgqid,IPC_SET,&buf) == -1 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid,IPC_SET) failed - errno %d (%s)",
			err,errname(err));
		DBUG_RETURN(SYSERROR);
	} else
		PATH_TRACE;
	PATH_FUNC_RPT(4);
	DBUG_RETURN(msgqid);
}

private	void
tidy()
{
	(void)unlink(msgpath);
}

private void
msgqrm(msgqid)
int	msgqid;
{
	DBUG_ENTER("msgqrm");
	DBUG_PRINTF("args","msgqrm(msgqid = %d)",msgqid);
	(void)msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0) ;
	DBUG_VOID_RETURN;
}

private void
setup()
{
	int	i;
	char	*msg = NULL;

	DBUG_ENTER("setup");
	(void)uids();
	(void)umask((mode_t)0);

	if ( setprv(PRV_SETID) != 0 ) 
		msg = "setprv(PRV_SETID) failed";
	else if ( setprv(PRV_IPC) != 0 ) 
		msg = "setprv(PRV_IPC) failed";
	else if ( setprv(PRV_ASSIGN) != 0 ) 
		msg = "setprv(PRV_ASSIGN) failed";
	else if ( getuid() != vguid0 )
		msg = "real user id is not VSX_UID0";
	else if ( getgid() != vggid0 )
		msg = "real user gid is not VSX_GID0";

	else
	{
		errno = NOERROR;
		ngroups = sysconf(_SC_NGROUPS_MAX);
		if ( ngroups < 0 ) {
			if ( errno != NOERROR)
				msg = "sysconf(_SC_NGROUPS_MAX) failed";
		}
		else if ( ngroups == 0 )
			/*EMPTY*/
			;
		else if (( suppgrps = (gid_t *) malloc((size_t)(sizeof(gid_t)*
			ngroups))) == NULL)
				msg = "malloc() failed";
		else if ( getgroups(ngroups,suppgrps) != ngroups)
			msg = "getgroups() did not return NGROUPS_MAX";
		else {
			for ( i = 0; i < ngroups; i++) {
				if ( suppgrps[i] == vggid1 ) {
					msg = "group vsxg1 must not be in supplementary groups list";
					break;
				}
				else if ( suppgrps[i] == vggid2 )
				{
					msg = "group vsxg2 must not be in supplementary groups list";
					break;
				}
			}
		}
	}


	if ( crfile(msgpath,MODEANY,(int)getuid(),(int)getgid()) == SYSERROR ) {
		msg = "create failed";
	}

	if ( msg != NULL ) {
		for ( i = 1; i <= no_tests; i++ ) 
			(void)tet_delete(i,msg);
		DBUG_VOID_RETURN;
	}

	uidlist[VSX0] = vguid0;
	uidlist[VSX1] = vguid1;
	uidlist[VSX2] = vguid2;
	uidlist[SU] = geteuid();
	gidlist[VSX0] = vggid0;
	gidlist[VSX1] = vggid1;
	gidlist[VSX2] = vggid2;

	DBUG_VOID_RETURN;
}

/*
 * How this works!
 * create a message queue creator uid vsx0, gid vsxg0
 *		          owner	  uid vsx1, gid vsxg1.
 * chmod/chown  a program that is set[ug]id, to get correct effective [ug]id.
 * run a process where real uid is vsx2, gid is vsxg2.
 * exec set[ug]id program.
 * carry out test.
 */

private	void
comp()
{
	int	err;

	DBUG_ENTER("comp");

	g_msgqid = -1;
	if ( prog_mode(msgprog,uidlist[tdat[tet_thistest].euid],
		gidlist[tdat[tet_thistest].egid],tdat[tet_thistest].pmode)==SYSERROR) {
		DBUG_VOID_RETURN;
	}

	if ( pipe(pipe_fd) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("pipe() failed - errno %d (%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}

	if ( cppair(mcreate,NULLFN,WAITTIME,0) != OK ) {
		DBUG_VOID_RETURN;
	}
	
	SET_TIMEOUT(WAITTIME)

	if ( read(pipe_fd[RD_FD],(char *)&g_msgqid,sizeof(g_msgqid)) != sizeof(g_msgqid)) {
		xx_rpt(DELETION);
		in_rpt("read() on pipe failed - errno %d (%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}

	CLEAR_ALARM

	(void) cppair(ch_comp,NULLFN,WAITTIME,0);
	
	msgqrm(g_msgqid);	/* clean up */
	DBUG_VOID_RETURN;
}

private	void
mcreate()
{
	int	msgqid;
	int	err;
	DBUG_ENTER("mcreate");


	if ( setgid((gid_t)vggid0) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setgid() failed - errno %d (%s)",err,errname(err));
		DBUG_EXIT(NOK);
	}

	if ( setuid((uid_t)vguid0) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setuid() failed - errno %d (%s)",err,errname(err));
		DBUG_EXIT(NOK);
	}

	if ( (msgqid = msg_create((uid_t)vguid1,(gid_t)vggid1,
		tdat[tet_thistest].perm)) == SYSERROR) {
		DBUG_EXIT(NOK);
	}

	if ( write(pipe_fd[WR_FD],(char *)&msgqid,sizeof(msgqid)) != sizeof(msgqid) ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to pipe failed - errno %d (%s)",err,errname(err));
		DBUG_EXIT(NOK);
	}

	DBUG_EXIT(OK);
}

private	void
ch_comp()
{
	extern char **environ;
	int	err;
	DBUG_ENTER("ch_comp");

	if ( setgid(gidlist[tdat[tet_thistest].rgid]) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("setgid(VSX_GID2) failed");
		DBUG_VOID_RETURN;
	}

	if ( setuid(uidlist[tdat[tet_thistest].ruid]) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("setuid(VSX_UID2) failed");
		DBUG_VOID_RETURN;
	}

	DBUG_PRINTF("ch_comp","ruid %ld, rgid %ld",getuid(),getgid());
	DBUG_PRINTF("ch_comp","euid %ld, egid %ld",geteuid(),getegid());
	p_argv[0] = msgprog;
	p_argv[1] = newstr(itoa(tet_thistest));
	p_argv[2] = newstr(ltoa((long)uidlist[tdat[tet_thistest].ruid]));
	p_argv[3] = newstr(ltoa((long)gidlist[tdat[tet_thistest].rgid]));
	p_argv[4] = newstr(ltoa((long)uidlist[tdat[tet_thistest].euid]));
	p_argv[5] = newstr(ltoa((long)gidlist[tdat[tet_thistest].egid]));
	p_argv[6] = newstr(itoa(g_msgqid));
	p_argv[7] = newstr(itoa(tdat[tet_thistest].msg_cmd));
	p_argv[8] = newstr(itoa(vguid0));
	p_argv[9] = newstr(itoa(vggid0));
	p_argv[10] = newstr(itoa(vguid1));
	p_argv[11] = newstr(itoa(vggid1));
	p_argv[12] = newstr(ltoa((long)tdat[tet_thistest].perm));
	p_argv[13] = vg_ident;
	p_argv[14] = vg_dbug;
	p_argv[15] = NULL;

	(void) tet_exec(msgprog,p_argv,environ);
	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec returned unexpectedly - errno %d (%s)",err,errname(err));
	DBUG_VOID_RETURN;

}

private	int
prog_mode(prog,uid,gid,mode)
char	*prog;
uid_t	uid;
gid_t	gid;
mode_t	mode;
{
	int err;

	DBUG_ENTER("prog_mode");
	DBUG_PRINTF("prog_mode","prog =%s, uid = %ld, gid = %ld, mode = 0%lo",prog,uid,gid,mode);

	errno = NOERROR;
	if ( access(prog,F_OK) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("access(%s,\"F_OK\") failed errno %d (%s)",
			prog,err,errname(err));
		DBUG_RETURN(SYSERROR);
	}

	errno = NOERROR;
	if ( chown(prog,uid,gid) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chown() failed - errno %d (%s)",err,errname(err));
		DBUG_RETURN(SYSERROR);
	}

	errno = NOERROR;
	if ( chmod(prog,mode) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod() failed - errno %d (%s)",err,errname(err));
		DBUG_RETURN(SYSERROR);
	}

	if (uid == SU_UID) {
		/* Assign privileges to the file exec'ed */
		if (prv_assign(prog, privileges) != 0) {
			xx_rpt(DELETION);
			in_rpt("prv_assign() failed");
			DBUG_RETURN(SYSERROR);
		}
	}


	DBUG_PRINTF("return","prog_mode = %d",0);
	DBUG_RETURN(0);
}

private	void
test13()
{
	int	err;
	int	msgqid;
	int	rval;
	int	pathok = 0;
	struct	msqid_ds msqid_ds;

	DBUG_ENTER("test13");

	if ( (msgqid = msg_create(getuid(),getgid(),MODEANY)) == SYSERROR ) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unable to remove message queue %d - errno = %d (%s)",
			msgqid,err,errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ((rval=msgctl(msgqid,IPC_STAT,&msqid_ds)) != SYSERROR || errno != EINVAL){
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgctl(IPC_STAT) did not give correct results");
		valrpt(SYSERROR,rval,EINVAL,err);
	} else
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private	void
test14()
{

	int	err;
	int	msgqid;
	int	rval;
	int	pathok = 0;
	int	ipc_cmd;
	struct	msqid_ds msqid_ds;

	DBUG_ENTER("test14");

	if ( (msgqid = msg_create(getuid(),getgid(),MODEANY)) == SYSERROR ) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* setup invalid command with exclusive bitwise or */
	ipc_cmd = IPC_STAT ^ IPC_SET ^ IPC_RMID ;

	errno = NOERROR;
	if ((rval=msgctl(msgqid,ipc_cmd,&msqid_ds)) != SYSERROR || errno != EINVAL){
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgctl(msgqid,%d) (invalid command) did not give correct results",ipc_cmd);
		valrpt(SYSERROR,rval,EINVAL,err);
	} else
		PATH_XS_RPT(1);

	msgqrm(msgqid);
	DBUG_VOID_RETURN;
}


private	void
test15()
{
	int	err;

	DBUG_ENTER("test15");


	g_msgqid = -1;
	if ( pipe(pipe_fd) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("pipe() failed - errno %d (%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}

	if ( cppair(ch1_t15,NULLFN,WAITTIME,0) != OK ) {
		DBUG_VOID_RETURN;
	}
	
	SET_TIMEOUT(WAITTIME)

	if ( read(pipe_fd[RD_FD],(char *)&g_msgqid,sizeof(g_msgqid)) != sizeof(g_msgqid)) {
		xx_rpt(DELETION);
		in_rpt("read() on pipe failed - errno %d (%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}

	CLEAR_ALARM

	(void) cppair(ch2_t15,NULLFN,WAITTIME,0);
	
	msgqrm(g_msgqid);	/* clean up */
	DBUG_VOID_RETURN;
}


private	void
ch1_t15()
{
	int	msgqid;
	int	err;
	DBUG_ENTER("ch1_t15");


	if ( setgid((gid_t)vggid0) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setgid() failed - errno %d (%s)",err,errname(err));
		DBUG_EXIT(NOK);
	}

	if ( setuid((uid_t)vguid0) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setuid() failed - errno %d (%s)",err,errname(err));
		DBUG_EXIT(NOK);
	}

	if ( (msgqid = msg_create((uid_t)vguid1,(gid_t)vggid1,RW_UGO)) == SYSERROR) 
	{
		DBUG_EXIT(NOK);
	}

	if ( write(pipe_fd[WR_FD],(char *)&msgqid,sizeof(msgqid)) != sizeof(msgqid) ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to pipe failed - errno %d (%s)",err,errname(err));
		msgqrm(msgqid);
		DBUG_EXIT(NOK);
	}
}

private	void
ch2_t15()
{
	int	err;
	int	rval;
	struct	msqid_ds msqid_ds;
	int	pathok = 0;

	DBUG_ENTER("ch2_t15");

	if ( msgctl(g_msgqid,IPC_STAT,&msqid_ds) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(IPC_STAT) failed - errno %d (%s)",
			err,errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	msqid_ds.msg_perm.uid = vguid2;
	msqid_ds.msg_perm.gid = vggid2;

	if ( setgid((gid_t)vggid2) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("setgid(VSX_GID2) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( setuid((uid_t)vguid2) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("setuid(VSX_UID2) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( unsetprv(PRV_IPC) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_IPC) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( unsetprv(PRV_SETID) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( unsetprv(PRV_ASSIGN) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ASSIGN) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	errno = NOERROR;
	if ((rval=msgctl(g_msgqid,IPC_SET,&msqid_ds)) != SYSERROR || errno != EPERM){
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgctl(IPC_SET) did not give correct results");
		valrpt(SYSERROR,rval,EPERM,err);
	} else
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}



private	void
test16()
{
	int	err;

	DBUG_ENTER("test16");


	if ( pipe(pipe_fd) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("pipe() failed - errno %d (%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}

	if ( cppair(ch1_t16,NULLFN,WAITTIME,0) != OK ) {
		DBUG_VOID_RETURN;
	}
	
	
	msgqrm(g_msgqid);	/* clean up */
	DBUG_VOID_RETURN;
}


private	void
ch1_t16()
{
	int	msgqid;
	int	err;
	int	rval;
	struct	msqid_ds msqid_ds;
	int	pathok = 0;
	DBUG_ENTER("ch1_t16");


	if ( setgid((gid_t)vggid0) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setgid() failed - errno %d (%s)",err,errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( setuid((uid_t)vguid0) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setuid() failed - errno %d (%s)",err,errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( (msgqid = msg_create((uid_t)vguid1,(gid_t)vggid1,RW_UG)) == SYSERROR) 
	{
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( msgctl(msgqid,IPC_STAT,&msqid_ds) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(IPC_STAT) failed - errno %d (%s)",err,errname(err));
		msgqrm(msgqid);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	msqid_ds.msg_qbytes -= QSZ_INC;
	if ( msgctl(msgqid,IPC_SET,&msqid_ds) == SYSERROR ) {
		xx_rpt(DELETION);
		in_rpt("msgctl(%d,IPC_SET,...) failed errno = %d (%s)",msgqid,err,errname(err));
		msgqrm(msgqid);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( unsetprv(PRV_IPC) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_IPC) failed");
		msgqrm(msgqid);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( unsetprv(PRV_SETID) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		msgqrm(msgqid);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( unsetprv(PRV_ASSIGN) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ASSIGN) failed");
		msgqrm(msgqid);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	msqid_ds.msg_qbytes += QSZ_INC;

	if ( ((rval=msgctl(msgqid,IPC_SET,&msqid_ds)) != SYSERROR) || errno != EPERM) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgctl(IPC_SET) did not give correct results");
		valrpt(SYSERROR,rval,EPERM,err);
		msgqrm(msgqid);
		DBUG_VOID_RETURN;
	} else
		PATH_XS_RPT(8);

	msgqrm(msgqid);
	DBUG_VOID_RETURN;
}

private	void
test17()
{
	int	err;

	DBUG_ENTER("test17");


	g_msgqid = -1;
	if ( pipe(pipe_fd) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("pipe() failed - errno %d (%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}

	if ( cppair(ch1_t17,NULLFN,WAITTIME,0) != OK ) {
		DBUG_VOID_RETURN;
	}
	
	SET_TIMEOUT(WAITTIME)

	if ( read(pipe_fd[RD_FD],(char *)&g_msgqid,sizeof(g_msgqid)) != sizeof(g_msgqid)) {
		xx_rpt(DELETION);
		in_rpt("read() on pipe failed - errno %d (%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}

	CLEAR_ALARM

	(void) cppair(ch2_t17,NULLFN,WAITTIME,0);
	
	msgqrm(g_msgqid);	/* clean up */
	DBUG_VOID_RETURN;
}


private	void
ch1_t17()
{
	int	msgqid;
	int	err;
	DBUG_ENTER("ch1_t17");


	if ( setgid((gid_t)vggid0) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setgid() failed - errno %d (%s)",err,errname(err));
		DBUG_EXIT(NOK);
	}

	if ( setuid((uid_t)vguid0) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setuid() failed - errno %d (%s)",err,errname(err));
		DBUG_EXIT(NOK);
	}

	if ( (msgqid = msg_create((uid_t)vguid1,(gid_t)vggid1,RW_UG)) == SYSERROR) 
	{
		DBUG_EXIT(NOK);
	}

	if ( write(pipe_fd[WR_FD],(char *)&msgqid,sizeof(msgqid)) != sizeof(msgqid) ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to pipe failed - errno %d (%s)",err,errname(err));
		DBUG_EXIT(NOK);
		msgqrm(msgqid);		/* clean up */
	}
}

private	void
ch2_t17()
{
	int	err;
	int	rval;
	int	pathok = 0;

	DBUG_ENTER("ch2_t17");

	if ( setgid((gid_t)vggid2) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("setgid(VSX_GID2) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( setuid((uid_t)vguid2) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("setuid(VSX_UID2) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( unsetprv(PRV_IPC) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_IPC) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( unsetprv(PRV_SETID) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( unsetprv(PRV_ASSIGN) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ASSIGN) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	errno = NOERROR;
	if ((rval=msgctl(g_msgqid,IPC_RMID,(struct msqid_ds *)0)) != SYSERROR || errno != EPERM){
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgctl(IPC_RMID) did not give correct results");
		valrpt(SYSERROR,rval,EPERM,err);
	} else
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}


private	void
test18()
{
	int	err;

	DBUG_ENTER("test18");

	g_msgqid = -1;
	if ( pipe(pipe_fd) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("pipe() failed - errno %d (%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}

	if ( cppair(ch1_t18,NULLFN,WAITTIME,0) != OK ) {
		DBUG_VOID_RETURN;
	}
	
	SET_TIMEOUT(WAITTIME)

	if ( read(pipe_fd[RD_FD],(char *)&g_msgqid,sizeof(g_msgqid)) != sizeof(g_msgqid)) {
		xx_rpt(DELETION);
		in_rpt("read() on pipe failed - errno %d (%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}

	CLEAR_ALARM

	(void) cppair(ch2_t18,NULLFN,WAITTIME,0);
	
	msgqrm(g_msgqid);	/* clean up */
	DBUG_VOID_RETURN;
}


private	void
ch1_t18()
{
	int	msgqid;
	int	err;
	DBUG_ENTER("ch1_t18");


	if ( setgid((gid_t)vggid0) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setgid() failed - errno %d (%s)",err,errname(err));
		DBUG_EXIT(NOK);
	}

	if ( setuid((uid_t)vguid0) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setuid() failed - errno %d (%s)",err,errname(err));
		DBUG_EXIT(NOK);
	}

	if ( unsetprv(PRV_IPC) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_IPC) failed");
		DBUG_EXIT(NOK);
	}

	if ( unsetprv(PRV_SETID) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_EXIT(NOK);
	}

	if ( unsetprv(PRV_ASSIGN) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ASSIGN) failed");
		DBUG_EXIT(NOK);
	}

	if ( (msgqid = msg_create((uid_t)vguid1,(gid_t)vggid1,W_UG)) == SYSERROR) 
	{
		DBUG_EXIT(NOK);
	}

	if ( write(pipe_fd[WR_FD],(char *)&msgqid,sizeof(msgqid)) != sizeof(msgqid) ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to pipe failed - errno %d (%s)",err,errname(err));
		msgqrm(msgqid);
		DBUG_EXIT(NOK);
	}
}

private	void
ch2_t18()
{
	int	err;
	int	rval;
	struct	msqid_ds msqid_ds;
	int	pathok = 0;

	DBUG_ENTER("ch2_t18");

	if ( setgid((gid_t)vggid1) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("setgid(VSX_GID1) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( setuid((uid_t)vguid1) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("setuid(VSX_UID1) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( unsetprv(PRV_IPC) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_IPC) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( unsetprv(PRV_SETID) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( unsetprv(PRV_ASSIGN) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ASSIGN) failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	errno = NOERROR;
	if ((rval=msgctl(g_msgqid,IPC_STAT,&msqid_ds)) != SYSERROR || errno != EACCES){
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgctl(IPC_STAT) did not give correct results");
		valrpt(SYSERROR,rval,EACCES,err);
	} else
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}


