/*
*	SCCS:  @(#)  XPG4.os/ipc/msgctl/msgctl_prg.c Rel 4.4.1     (06/09/97)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)  XPG4.os/ipc/msgctl/msgctl_prg.c Rel 4.4.1     (06/09/97)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		msgctl_prg - used by T.kill in several tests

PROJECT:	XPG4

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.msgctl to test the message queues access.
	  usage:
	msgctl_prg testno ruid rgid euid egid msgqid msg_cmd 
		vsxuid0,vsxgid0,vsxuid1,vsxgid1,msg_perm,vg_ident vg_dbug

	  ruid is the real user id that is to be used in the test,
	  rgid is the real group id that is to be used in the test,
	  euid is the effective user id that is to be used in the test,
	  egid is the effective group id that is to be used in the test,
	  msgqid is the message q identifier to use in the test
	  msg_cmd is the msgctl() command to use in this test.
	  vsx[ug]id are the uid no.s since uids() only works when run as vsx0
	  msg_perm is are the access permissions of the message queue.


METHODS:	This program runs with real uid VSX_UID2 and gid VSX_GID2.
ASSUMES:  msgctl_prg must be set[ug]id to the effective group/user id.
	message queue creator user-id VSX_UID0, group-id VSX_GID0
	              owner  user-id VSX_UID1, group-id VSX_GID1
		      
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Keith Holder
DATE CREATED:	Mon Sep  3 12:57:03 BST 1990
MODIFICATIONS:
		Matt Newman, Mon Sep 09 14:55:33 BST 1991
			XPG4 Intergration
		
		Geoff Clare, 29 Apr 1992
			Fix tet_main arguments.

		John-Paul, 13 July 1992 
			Test 9: Message queue size decreased before increase
			to avoid wrap-round problem.
		
		Geoff Clare, 4 March 1994
			Insert some missing err=errno assignments.
	

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<errno.h>
#include	<setprv.h>
#include	<sys/stat.h>
#include	<sys/ipc.h>
#include	<sys/msg.h>
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef msgctl
#endif



#define	AMODES	(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	MODE_RW_ALL	(mode_t)(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)
#define	INC	64
#define	CUID	((uid_t)vguid0)
#define	CGID	((gid_t)vggid0)
#define	OUID	((uid_t)vguid1)
#define	OGID	((gid_t)vggid1)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_file =	(char *) NULL;
public	char	*vg_dbug =	(char *) NULL;
public	char	**vg_env =	(char **) NULL;


extern	void	set_pname();
extern	char	*errname();
extern	int	atoi();
extern	long	atol();
private	void	subtest();
extern	int	vguid0,vggid0;
extern	int	vguid1,vggid1;
extern	int	vguid2,vggid2;
extern	char	**environ;

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	vg_env = environ;

	set_pname((argc < 1 || argv[0] == (char *) NULL) ?
		"msgctl_prg" : (vg_file = argv[0]));
	switch(argc)
	{
	case 15:
		vg_dbug = argv[14];
		/*FALLTHROUGH*/
	case 14:
		vg_ident = argv[13];
		break;
	default:
		vsxerror("argument count wrong");
		exit(A_MISC);
	}

	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}


	subtest(atoi(argv[1]), (uid_t)atol(argv[2]), (gid_t)atol(argv[3]), 
		(uid_t)atol(argv[4]), (uid_t)atol(argv[5]), atoi(argv[6]), 
		atoi(argv[7]), (uid_t)atol(argv[8]), (gid_t)atol(argv[9]), 
		(uid_t)atol(argv[10]), (gid_t)atol(argv[11]),
		(mode_t)atol(argv[12]));

	exit(OK);

	/* NOTREACHED */

	return(0);
}

private void
subtest(testno, ruid, rgid, euid, egid, msgqid,cmd,vsx0,vsxg0,vsx1,vsxg1,perm)
uid_t	ruid;
gid_t	rgid;
uid_t	euid;
gid_t	egid;
mode_t	perm;
uid_t	vsx0;
uid_t	vsx1;
gid_t	vsxg0;
gid_t	vsxg1;
{
	int	err;
	char	*msg = NULL;
	char	*cmd_str = "";
	struct	msqid_ds msgq_ds;
	uid_t	myuid;
	gid_t	mygid;
	int pathok = 0;

	DBUG_ENTER("subtest");
	DBUG_PRINTF("args","subtest( testno = %d,ruid = %ld,rgid = %ld, euid = %ld, egid = %ld, msgqid = %d, cmd = %d",testno,ruid,rgid,euid,egid,msgqid,cmd);

	DBUG_PRINTF("subtest","uid %ld, gid %ld, euid %ld, egid %ld",getuid(),getgid(),geteuid(),getegid());

	if ( (myuid = getuid()) != (uid_t)ruid )
		msg = "incorrect real user-id for test";
	else if ( (mygid = getgid()) != (gid_t)rgid )
		msg = "incorrect real group-id for test";
	else if ( geteuid() != (uid_t)euid && getegid() != (gid_t)egid )
		msg = "incorrect effective user-id/group-id for test";
	else {
		switch (cmd) {
			case IPC_STAT:
				cmd_str = "IPC_STAT";
				PATH_TRACE;
				break;
			case IPC_SET:
				cmd_str = "IPC_SET";
				PATH_TRACE;
				break;
			case IPC_RMID:
				cmd_str = "IPC_RMID";
				PATH_TRACE;
				break;
			default:
				msg = "unknown msgctl() command";
				break;
		}
	}

	if ( msg != NULL ) {
		xx_rpt(DELETION);
		in_rpt("%s",msg);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( cmd == IPC_SET ) {
		struct	msqid_ds msgq_ds_2;
		long	qsize;

		if ( msgctl(msgqid,IPC_STAT,&msgq_ds) == SYSERROR ) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("msgctl(%d,IPC_STAT,...) failed errno = %d (%s)",
				msgqid, err,errname(err));
			DBUG_VOID_RETURN;
		}  else 
			PATH_TRACE;

		qsize = msgq_ds.msg_qbytes ;

		msgq_ds.msg_perm.uid = myuid;
		msgq_ds.msg_perm.gid = mygid;
		msgq_ds.msg_perm.mode = MODE_RW_ALL;

		qsize -= INC;
		msgq_ds.msg_qbytes = qsize;

		if ( msgctl(msgqid,IPC_SET,&msgq_ds) == SYSERROR ) {
			err = errno;
			if( testno == 9 )
				xx_rpt(DELETION);
			else
				xx_rpt(FAILURE);
			in_rpt("msgctl(%d,%s,...) failed errno = %d (%s)",
				msgqid,cmd_str,err,errname(err));
			DBUG_VOID_RETURN;
		} else
			PATH_TRACE;

		if ( testno == 9 )	/* appropriate privileges */
		{
			qsize += INC;
			msgq_ds.msg_qbytes = qsize;
			if ( msgctl(msgqid,IPC_SET,&msgq_ds) == SYSERROR ) {
				err = errno;
				xx_rpt(FAILURE);
				in_rpt("msgctl(%d,%s,...) failed errno = %d (%s)",msgqid,cmd_str,err,errname(err));
				DBUG_VOID_RETURN;
			} else
				PATH_TRACE;
		}
		else
			PATH_TRACE;

		if ( msgctl(msgqid,IPC_STAT,&msgq_ds_2) == SYSERROR ) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("msgctl(%d,IPC_STAT,...) failed errno = %d (%s)",
				msgqid, err,errname(err));
			DBUG_VOID_RETURN;
		}  else 
			PATH_TRACE;
		if ( msgq_ds_2.msg_perm.uid != myuid )
			msg = "msgctl(IPC_SET) did not set the uid on message queue";
		else if ( msgq_ds_2.msg_perm.gid != mygid )
			msg = "msgctl(IPC_SET) did not set the gid on message queue";
		else if ( (msgq_ds_2.msg_perm.mode & AMODES) != MODE_RW_ALL )
			msg = "msgctl(IPC_SET) did not set the correct mode on message queue";
		else if ( msgq_ds_2.msg_qbytes != qsize )
			msg = "msgctl(IPC_SET) did not set the correct queue size on message queue";
		else
			PATH_TRACE;

		if ( msg != NULL ) {
			xx_rpt(FAILURE);
			in_rpt("%s",msg);
			DBUG_VOID_RETURN;
		} else {
			PATH_XS_RPT(7);
			DBUG_VOID_RETURN;
		}
	} else {
		if ( msgctl(msgqid,cmd,&msgq_ds) == SYSERROR ) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("msgctl(%d,%s,...) failed errno = %d (%s)",msgqid,
				cmd_str,err,errname(err));
			DBUG_VOID_RETURN;
		} else if ( cmd == IPC_STAT)  {
			if ( msgq_ds.msg_perm.cuid != vsx0 ) {
				xx_rpt(FAILURE);
				in_rpt("obtained incorrect message queue creator uid");
				in_rpt("expected %ld, actual %ld",
					vsx0,msgq_ds.msg_perm.cuid);
			}
			else if ( msgq_ds.msg_perm.cgid != vsxg0 ) {
				xx_rpt(FAILURE);
				in_rpt("obtained incorrect message queue creator gid");
				in_rpt("expected %ld, actual %ld",
					vsxg0,msgq_ds.msg_perm.cgid);
			}
			else if ( msgq_ds.msg_perm.uid != vsx1 ) {
				xx_rpt(FAILURE);
				in_rpt("obtained incorrect message queue owner uid");
				in_rpt("expected %ld, actual %ld",
					vsx1,msgq_ds.msg_perm.uid);
			}
			else if ( msgq_ds.msg_perm.gid != vsxg1 ) {
				xx_rpt(FAILURE);
				in_rpt("obtained incorrect message queue owner gid");
				in_rpt("expected %ld, actual %ld",
					vsxg1,msgq_ds.msg_perm.gid);
			}
			else if ( (msgq_ds.msg_perm.mode & AMODES) != perm ) {
				xx_rpt(FAILURE);
				in_rpt("obtained incorrect message queue access permissions");
				in_rpt("expected %#lo, actual %#lo",
					perm,msgq_ds.msg_perm.mode&AMODES);
			}
			else 
				PATH_XS_RPT(2);
			DBUG_VOID_RETURN;
		} else if ( cmd == IPC_RMID ) {
			if ( msgctl(msgqid,IPC_STAT,&msgq_ds) != SYSERROR ) {
				xx_rpt(FAILURE);
				in_rpt("msgctl(IPC_RMID) did not remove message queue");
				DBUG_VOID_RETURN;
			} else
				PATH_XS_RPT(2);
		}
	}

	DBUG_VOID_RETURN;
}
