/*
*	SCCS:  @(#)  XPG4.os/ipc/msgget/msgget.c Rel 4.4.1     (06/09/97)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)  XPG4.os/ipc/msgget/msgget.c Rel 4.4.1     (06/09/97)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.msgget

PROJECT:	XPG4 Validation suite

DESCRIPTION:
	Tests for the msgget() system call.

AUTHOR:		John Onusko
DATE CREATED:	12/17/90
MODIFICATIONS:	
		Matt Newman, Mon Sep 09 15:04:00 BST 1991
			XPG4 Intergration
		
		Geoff Clare, 29 Apr 1992
			Changes for new types of msg_qnum and msg_qbytes.
		
		Geoff Clare, 16 June 1993
			Add declaration of ftok().

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<dbug.h>

#include	<sys/types.h>
#include	<unistd.h>
#include	<time.h>
#include	<errno.h>
#include	<sys/ipc.h>
#include	<sys/msg.h>
#include	<sys/stat.h>

#include	<report.h>
#include	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef msgget
#endif


#define NO_TESTS	7		/* number of tests in testset */
#define	MODEANY		((mode_t)(S_IRWXU|S_IRWXG|S_IRWXO))

#define	ID1	11
#define	ID3	33
#define	ID4	44
#define	ID5	55
#define	ID6	66
#define	ID7	77

/*
 * message access permissions
 */
#define	RW_U		(S_IRUSR|S_IWUSR)	
#define	RW_G		(S_IRGRP|S_IWGRP)
#define	RW_O		(S_IROTH|S_IWOTH)
#define	MSG_READWRITE	(RW_U|RW_G|RW_O)
#define	MSG_READONLY	(S_IRUSR|S_IRGRP|S_IROTH)

public	int	no_tests = NO_TESTS;

#ifndef ftok
extern	key_t	ftok();
#endif
extern	char	*errname();
extern	int	crfile();
extern	void	valrpt();

private	char	*msgpath	= "./msgget.f";
private int	setupok = -1;
private int	test_failed;

private	void	setup();
private	void	cleanup();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	t2and3_func();
private	void	test4();
private	void	test5();
private	void	test6();
private	void	test7();

public 	void	(*setfunc)() = setup;
public	void	(*clnfunc)() = cleanup;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	NULL, 0
};

private void
setup()
{
	static char	errmsg[512];

	int	i;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("setup");

	if (crfile(msgpath, MODEANY, (int) geteuid(), (int) getegid()) 
		== SYSERROR) {

		err = errno;
		(void) sprintf(errmsg, "crfile(\"%s\") failed - errno %d (%s)",
			msgpath, err, errname(err));
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, errmsg);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (pathok == 1)
		setupok = 0;

	DBUG_VOID_RETURN;
}

private	void
cleanup()
{
	DBUG_ENTER("cleanup");

	(void) unlink(msgpath);

	DBUG_VOID_RETURN;
}

private	void
test1()
{
	int 	msgqid;
	key_t	key;
	int	err;
	struct	msqid_ds buf;
	int	pathok = setupok;

	DBUG_ENTER("test1");
	
	if ((key = ftok(msgpath, ID1)) == (key_t) SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(\"%s\", %d) failed", msgpath, ID1);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ((msgqid = msgget(key, (IPC_CREAT|MSG_READWRITE))) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgget(key = %d, msgflg = 0%o) failed - errno %d (%s)",
			(int) key, (IPC_CREAT|MSG_READWRITE), err, 
			errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (msgctl(msgqid, IPC_STAT, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_STAT) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	int	pathok = setupok;

	DBUG_ENTER("test2");
	test_failed = 0;
	
	t2and3_func(IPC_PRIVATE, MSG_READWRITE);
	PATH_TRACE;

	if (!test_failed)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private	void
test3()
{
	key_t	key;
	int	pathok = setupok;

	DBUG_ENTER("test3");
	test_failed = 0;
	
	if ((key = ftok(msgpath, ID3)) == (key_t) SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(\"%s\", %d) failed", msgpath, ID3);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	t2and3_func(key, (IPC_CREAT|MSG_READWRITE));
	PATH_TRACE;

	if (!test_failed)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
t2and3_func(key, msgflg)
key_t key;
int msgflg;
{
	int 	msgqid;
	int	err;
	struct	msqid_ds buf;
	time_t 	time_before, time_after;
	uid_t 	uid;
	gid_t 	gid;
	int	pathok = 0;

	DBUG_ENTER("t2and3_func");
	
	time_before = time((time_t *)0);

	if ((msgqid = msgget(key, msgflg)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgget(key = %d, msgflg = 0%o) failed - errno %d (%s)",
			(int) key, msgflg, err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	time_after = time((time_t *)0);

	if (msgctl(msgqid, IPC_STAT, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_STAT) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	uid = geteuid();

	if (buf.msg_perm.cuid != uid) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_perm.cuid not set to creator's uid");
		in_rpt("Expected: %d, Observed: %d", uid, buf.msg_perm.cuid);
	} else
		PATH_TRACE;

	if (buf.msg_perm.uid != uid) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_perm.uid not set to owner's uid");
		in_rpt("Expected: %d, Observed: %d", uid, buf.msg_perm.uid);
	} else
		PATH_TRACE;

	gid = getegid();
	
	if (buf.msg_perm.cgid != gid) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_perm.cgid not set to creator's gid");
		in_rpt("Expected: %d, Observed: %d", gid, buf.msg_perm.cgid);
	} else
		PATH_TRACE;

	if (buf.msg_perm.gid != gid) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_perm.gid not set to owner's gid");
		in_rpt("Expected: %d, Observed: %d", gid, buf.msg_perm.gid);
	} else
		PATH_TRACE;

	if ((buf.msg_perm.mode & MSG_READWRITE) != MSG_READWRITE) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_perm.mode not set as expected");
		in_rpt("Expected: %d, Observed: %d", MSG_READWRITE, 
			buf.msg_perm.mode);
	} else
		PATH_TRACE;

	if (buf.msg_qnum != 0) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_qnum not set as expected");
		in_rpt("Expected: 0, Observed: %lu", (unsigned long) buf.msg_qnum);
	} else
		PATH_TRACE;

	if (buf.msg_lspid != 0) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_lspid not set as expected");
		in_rpt("Expected: 0, Observed: %ld", (long) buf.msg_lspid);
	} else
		PATH_TRACE;

	if (buf.msg_lrpid != 0) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_lrpid not set as expected");
		in_rpt("Expected: 0, Observed: %ld", (long) buf.msg_lrpid);
	} else
		PATH_TRACE;

	if (buf.msg_stime != 0) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_stime not set as expected");
		in_rpt("Expected: 0, Observed: %ld", (long) buf.msg_stime);
	} else
		PATH_TRACE;

	if (buf.msg_rtime != 0) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_rtime not set as expected");
		in_rpt("Expected: 0, Observed: %ld", (long) buf.msg_rtime);
	} else
		PATH_TRACE;

	if ((buf.msg_ctime < time_before) || (buf.msg_ctime > time_after)) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_ctime not set as expected");
		in_rpt("Expected range: %ld - %ld, Observed: %ld", 
			(long) time_before, (long) time_after,
			(long) buf.msg_ctime);
	} else
		PATH_TRACE;

	if (buf.msg_qbytes == 0) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_qbytes not set as expected");
		in_rpt("Expected: greater than zero bytes, Observed: 0"); 
	} else
		PATH_TRACE;
		
	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(14);

	DBUG_VOID_RETURN;
}

private	void
test4()
{
	int 	msgqid1, msgqid2;
	key_t	key;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test4");
	test_failed = 0;
	
	if ((key = ftok(msgpath, ID4)) == (key_t) SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(\"%s\", %d) failed", msgpath, ID4);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ((msgqid1 = msgget(key, (IPC_CREAT|MSG_READWRITE))) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgget(key = %d, msgflg = 0%o) failed - errno %d (%s)",
			(int) key, (IPC_CREAT|MSG_READWRITE), err, 
			errname(err));
		(void) msgctl(msgqid1, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ((msgqid2 = msgget(key, (IPC_CREAT|MSG_READWRITE))) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgget(key = %d, msgflg = 0%o) failed - errno %d (%s)",
			(int) key, (IPC_CREAT|MSG_READWRITE), err, 
			errname(err));
		(void) msgctl(msgqid1, IPC_RMID, (struct msqid_ds *)0);
		(void) msgctl(msgqid2, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (msgqid2 != msgqid1) {
		xx_rpt(FAILURE);
		in_rpt("msgget(key, IPC_CREAT) failed -");
		in_rpt("did not return message queue id associated with key");
		in_rpt("MESSAGE QUEUE ID: Expected: %d, Observed: %d", msgqid1,
			msgqid2);
		(void) msgctl(msgqid1, IPC_RMID, (struct msqid_ds *)0);
		(void) msgctl(msgqid2, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid1, IPC_RMID, (struct msqid_ds *)0);
	(void) msgctl(msgqid2, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private	void
test5()
{
	int 	msgqid1, msgqid2;
	key_t	key;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test5");
	
	if ((key = ftok(msgpath, ID5)) == (key_t) SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(\"%s\", %d) failed", msgpath, ID1);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ((msgqid1 = msgget(key, (IPC_CREAT|MSG_READONLY))) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgget(key = %d, msgflg = 0%o) failed - errno %d (%s)",
			(int) key, (IPC_CREAT|MSG_READONLY), err, 
			errname(err));
		(void) msgctl(msgqid1, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	msgqid2 = msgget(key, MSG_READWRITE);

	if (msgqid2 != SYSERROR || errno != EACCES) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgget(key, MSG_READWRITE) - succeeded unexpectedly");
		valrpt(SYSERROR, msgqid2, EACCES, err);
		(void) msgctl(msgqid1, IPC_RMID, (struct msqid_ds *)0);
		(void) msgctl(msgqid2, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid1, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(3);
		
	DBUG_VOID_RETURN;
}

private	void
test6()
{
	int 	msgqid;
	key_t	key;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test6");
	
	if ((key = ftok(msgpath, ID5)) == (key_t) SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(\"%s\", %d) failed", msgpath, ID5);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ((msgqid = msgget(key, (IPC_CREAT|MSG_READWRITE))) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgget(key = %d, msgflg = 0%o) failed - errno %d (%s)",
			(int) key, (IPC_CREAT|MSG_READWRITE), err, 
			errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid = %d, IPC_RMID, (struct msqid_ds *)0) failed - errno %d (%s)",
			msgqid, err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	msgqid = msgget(key, MSG_READWRITE);

	if (msgqid != SYSERROR || errno != ENOENT) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgget(key, MSG_READWRITE) - succeeded unexpectedly");
		valrpt(SYSERROR, msgqid, ENOENT, err);
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	PATH_XS_RPT(4);
		
	DBUG_VOID_RETURN;
}
 
private	void
test7()
{
	int 	msgqid1, msgqid2;
	key_t	key;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test7");
	
	if ((key = ftok(msgpath, ID7)) == (key_t) SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(\"%s\", %d) failed", msgpath, ID1);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ((msgqid1 = msgget(key, (IPC_CREAT|MSG_READWRITE))) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgget(key = %d, msgflg = 0%o) failed - errno %d (%s)",
			(int) key, (IPC_CREAT|MSG_READWRITE), err, 
			errname(err));
		(void) msgctl(msgqid1, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	msgqid2 = msgget(key, (IPC_CREAT|IPC_EXCL|MSG_READWRITE));

	if (msgqid2 != SYSERROR || errno != EEXIST) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgget(key, MSG_READWRITE) - succeeded unexpectedly");
		valrpt(SYSERROR, msgqid2, EEXIST, err);
		(void) msgctl(msgqid1, IPC_RMID, (struct msqid_ds *)0);
		(void) msgctl(msgqid2, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid1, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(3);
		
	DBUG_VOID_RETURN;
}
