/*
*	SCCS:  @(#)  XPG4.os/ipc/msgrcv/msgrcv.c Rel 4.4.1     (06/09/97)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)  XPG4.os/ipc/msgrcv/msgrcv.c Rel 4.4.1     (06/09/97)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.msgrcv

PROJECT:	XPG4 Validation suite

DESCRIPTION:
	Additional tests for the msgrcv() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		John Onusko
DATE CREATED:	12/19/90
MODIFICATIONS:	
		Matt Newman, Tue Sep 10 12:12:21 BST 1991
			XPG4 Intergration

		Geoff Clare, 29 Apr 1992
			Changes for new types of msg_qnum and msg_qbytes.

		Geoff Clare, 9 Sept 1992
			Added sleep() in ch_t5().
			Fix path tracing problem with sndmsgs().

		Geoff Clare, 17 Feb 1994
			Removed test for msgsz less than zero.

		Geoff Clare, 4 March 1994
			Changed "msgsnd" to "msgrcv" in some in_rpt() strings.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<dbug.h>

#include	<sys/types.h>
#include	<unistd.h>
#include	<time.h>
#include	<errno.h>
#include	<sys/ipc.h>
#include	<sys/msg.h>
#include	<sys/stat.h>
#include	<string.h>
#include	<wait_child.h>
#include	<signal.h>

#include	<report.h>
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef msgrcv
#endif


#define NO_TESTS	13	/* number of tests in testset */

#define	TEST_TIMEOUT		(10 * SPEEDFACTOR)
#define	CHILD_TIMEOUT		(5 * SPEEDFACTOR)
#define	WAIT_FOR_SUSPEND	(5 * SPEEDFACTOR)
#define	WAIT_FOR_EXIT		(3 * SPEEDFACTOR)
#define	MODEANY			((mode_t)(S_IRWXU|S_IRWXG|S_IRWXO))

#define	TEST_OK			0
#define	TEST_DELETED		1
#define	TEST_PATH_FAIL		2
#define	TEST_DID_NOT_SUSPEND	3
#define	TEST_FAILED		4

#define	ID1	11
#define	ID2	22
#define	ID3	33
#define	ID4	44
#define	ID5	55
#define	ID6	66
#define	ID7	77
#define	ID8	88
#define	ID9	99
#define	ID11	1111
#define	ID12	1212
#define	ID13	1313

#define	INVALID_MSGQID	-1

/*
 * message access permissions
 */
#define	RW_U		(S_IRUSR|S_IWUSR)	
#define	RW_G		(S_IRGRP|S_IWGRP)
#define	RW_O		(S_IROTH|S_IWOTH)
#define	MSG_READWRITE	(RW_U|RW_G|RW_O)
#define	MSG_WRITEONLY	(S_IWUSR|S_IWGRP|S_IWOTH)

public	int	no_tests = NO_TESTS;

extern	int	ex_code;

#ifndef ftok
extern	key_t	ftok();
#endif

extern	char	*errname();
extern	void	valrpt();
extern	int	wait_child();
extern	void	(*vsignal())();

private	char	*msgpath	= "./msgrcv.f";
private	int	msgqid;
private	int	msgtyp;
private	int	id;

#define	NMSGS		4
#define	MBUFFERSZ	256

struct test_msg {
	long	mtype;
	char	mtext[MBUFFERSZ];
};

private	struct test_msg 	rcvmsg;
private	struct test_msg 	clean_rcvmsg;
private	struct test_msg 	*expmsg;

private struct
test_msg msg1 = {
	1,
	"test message 1"
};

private struct
test_msg msg2 = {
	2,
	"test message 2"
};

private struct
test_msg msg3 = {
	3,
	"test message 3, type 3"
};

private struct
test_msg msg4 = {
	3,
	"test message 4, type 3"
};

private struct test_msg * msgs[NMSGS + 1] = { 
	&msg2, &msg1, &msg3, &msg4, NULL 
};

private int	setupok = -1;
private int	test_failed;
private	int	test_deleted;

private	int	cr_msgq();
private	int	sndmsgs();
private	void	setup(), cleanup(), test1(), test2(), test3(), t1thru3_setup(),
		ch_t1thru3(), pa_t1thru3(), test4(), t4_setup(), ch_t4(),
		pa_t4(), test5(), t5_setup(), ch_t5(), pa_t5(), test6(),
		t6_setup(), ch_t6(), pa_6_7_10thru13(), test7(), t7_setup(),
		ch_t7(), test8(), t8_setup(), ch_t8(), pa_t8(), test9(),
		t9_setup(), ch_t9(), pa_t9(), t9_sig_catch(), test10(),
		t10_setup(), ch_t10(), test11(),
		test12(), t12_setup(), ch_t12(), test13(), t13_setup(),
		ch_t13();

public	void	(*setfunc)() = setup;
public	void	(*clnfunc)() = cleanup;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	NULL, 0
};

private void
setup()
{
	static char	errmsg[512];

	int	i;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("setup");

	if (crfile(msgpath, MODEANY, (int) geteuid(), (int) getegid()) 
		== SYSERROR) {

		err = errno;
		(void) sprintf(errmsg, "crfile(\"%s\") failed in setup - errno %d (%s)",
			msgpath, err, errname(err));
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, errmsg);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (pathok == 1)
		setupok = 0;

	DBUG_VOID_RETURN;
}

private	void
cleanup()
{
	DBUG_ENTER("cleanup");

	(void) unlink(msgpath);

	DBUG_VOID_RETURN;
}

private	int
cr_msgq(id,msgflg)
int id;
int msgflg;
{
	key_t	key;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("cr_msgq");
	
	if ((key = ftok(msgpath, id)) == (key_t) SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(\"%s\", %d) failed", msgpath, id);
		DBUG_RETURN(SYSERROR);
	} else
		PATH_TRACE;

	if ((msgqid = msgget(key, (IPC_CREAT|msgflg))) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgget(key = %d, msgflg = 0%o) failed - errno %d (%s)",
			(int) key, (IPC_CREAT|msgflg), err, 
			errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_RETURN(SYSERROR);
	} else
		PATH_TRACE;

	PATH_XS_RPT(2);

	DBUG_RETURN(TEST_OK);
}

private	int
sndmsgs()
{
	struct test_msg	**msg;
	size_t	msgsz;

	int	err;
	int	pathok = 0;

	DBUG_ENTER("sndmsgs");

	for (msg = msgs; *msg; msg++) {

		msgsz = strlen((*msg)->mtext) + 1;

		if (msgsnd(msgqid, *msg, msgsz, 0) == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("msgsnd(msqid = %d, *msgp = 0x%lx, msgsz = %d, NULL) failed - errno %d (%s)",
				msgqid, *msg, msgsz, err, errname(err));
			(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
			DBUG_RETURN(SYSERROR);
		} else
			PATH_TRACE;
	}

	PATH_FUNC_RPT(NMSGS);

	DBUG_RETURN(TEST_OK);
}

private	void
test1()
{
	DBUG_ENTER("test1");

	expmsg = msgs[0];
	msgtyp = 0;
	id = ID1;
	(void) cppair(t1thru3_setup, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	DBUG_ENTER("test2");

	expmsg = msgs[2];
	msgtyp = 3;
	id = ID2;
	(void) cppair(t1thru3_setup, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
test3()
{
	DBUG_ENTER("test3");

	expmsg = msgs[1];
	msgtyp = -1;
	id = ID3;
	(void) cppair(t1thru3_setup, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
t1thru3_setup()
{
	int	pathok = setupok;

	DBUG_ENTER("t1thru3_setup");
	test_failed = 0;
	test_deleted = 0;

	if (cr_msgq(id, MSG_READWRITE) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	if (sndmsgs() == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	if (cpsig(ch_t1thru3, pa_t1thru3, CHILD_TIMEOUT, OK) == FAIL || 
		test_deleted) {
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
ch_t1thru3()
{
	size_t	msgsz;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t1thru3");

	if ((msgsz = msgrcv(msgqid, &rcvmsg, (size_t) MBUFFERSZ, msgtyp, 
		0)) == SYSERROR) {

		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgrcv(msqid, *msgp, msgsz = %d, msgtyp = %d, 0) failed - errno %d (%s)",
			MBUFFERSZ, msgtyp, err, errname(err));
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	if (strcmp(rcvmsg.mtext, expmsg->mtext) != 0) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msgrcv() failed - did not retrieve correct message from the queue");
	} else
		PATH_TRACE;

	if (msgsz != strlen(expmsg->mtext) + 1) {	
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msgrcv() failed - received message size did not match sent message size");
		in_rpt("MESSAGE SIZE: Sent: %d, Received: %d", 
			strlen(expmsg->mtext) + 1, msgsz);
	} else
		PATH_TRACE;

	if (rcvmsg.mtype != expmsg->mtype) {	
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msgrcv() failed - received message type did not match sent message type");
		in_rpt("MESSAGE TYPE: Sent: %d, Received: %d", 
			expmsg->mtype, rcvmsg.mtype);
	} else
		PATH_TRACE;

	if (test_failed) {
		DBUG_EXIT(TEST_FAILED);
	} else if (pathok != 4) {
		in_rpt("pathok != 4 (%d)",pathok);
		DBUG_EXIT(TEST_PATH_FAIL);
	} else {
		DBUG_EXIT(TEST_OK);
	}
}

private void
pa_t1thru3()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_t1thru3");

	if ((stat_child = wait_child(WAIT_FOR_EXIT)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_EXIT, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child == CHLD_EXIT) {
		switch (ex_code) {

		case TEST_FAILED:
		case TEST_DELETED:
			test_deleted++;
			DBUG_VOID_RETURN;

		case TEST_PATH_FAIL:
			break;

		case TEST_OK:
			PATH_TRACE;
			break;

		default:
			xx_rpt(DELETION);
			in_rpt("child exited with unexpected code");
			test_deleted++;
			DBUG_VOID_RETURN;
		}
	} else {
		xx_rpt(DELETION);
		in_rpt("child did not exit as expected");
		(void) kill(tet_child, SIGKILL);
		test_deleted++;
		DBUG_VOID_RETURN;
	} 

	PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private	void
test4()
{
	DBUG_ENTER("test4");

	(void) cppair(t4_setup, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
t4_setup()
{
	int	pathok = setupok;

	DBUG_ENTER("t4_setup");
	test_failed = 0;
	test_deleted = 0;

	if (cr_msgq(ID4, MSG_READWRITE) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (cpsig(ch_t4, pa_t4, CHILD_TIMEOUT, OK) == FAIL || 
		test_deleted) {
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(2);
	
	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	err;

	DBUG_ENTER("ch_t4");

	/* should hang here */
	if (msgrcv(msgqid, &rcvmsg, (size_t) MBUFFERSZ, 0, 0) 
		== SYSERROR) {

		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgrcv(msqid, *msgp, msgsz = %d, 0, 0) failed - errno %d (%s)",
			MBUFFERSZ, err, errname(err));
		DBUG_EXIT(TEST_DELETED);
	} 

	xx_rpt(FAILURE);
	in_rpt("msgrcv() failed - did not suspend when no messages available");
	DBUG_EXIT(TEST_FAILED);
}

private void
pa_t4()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_t4");

	if ((stat_child = wait_child(WAIT_FOR_SUSPEND)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_SUSPEND, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child == CHLD_EXIT) {
		switch (ex_code) {

		case TEST_FAILED:
		case TEST_DELETED:
			test_deleted++;
			DBUG_VOID_RETURN;

		default:
			xx_rpt(DELETION);
			in_rpt("child exited with unexpected code");
			test_deleted++;
			DBUG_VOID_RETURN;
		}
	} else if (stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("child did not suspend as expected");
		(void) kill(tet_child, SIGKILL);
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) kill(tet_child, SIGKILL); 

	PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private	void
test5()
{
	DBUG_ENTER("test5");

	(void) cppair(t5_setup, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
t5_setup()
{
	int	pathok = setupok;

	DBUG_ENTER("t5_setup");
	test_failed = 0;
	test_deleted = 0;

	if (cr_msgq(ID5, MSG_READWRITE) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (cpsig(ch_t5, pa_t5, CHILD_TIMEOUT, OK) == FAIL || 
		test_deleted) {
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(2);
	
	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	struct msqid_ds	buf;

	time_t	time_before, time_after;
	pid_t	rcvpid;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t5");

	time_before = time((time_t *)0);

	if (msgrcv(msgqid, &rcvmsg, (size_t) MBUFFERSZ, 0, 0) 
		== SYSERROR) {

		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgrcv(msqid, *msgp, msgsz = %d, 0, 0) failed - errno %d (%s)",
			MBUFFERSZ, err, errname(err));
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	time_after = time((time_t *)0);

	/* Sleep to allow parent to queue messages */
	(void) sleep(WAIT_FOR_EXIT);

	if (msgctl(msgqid, IPC_STAT, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_STAT) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	if (strcmp(rcvmsg.mtext, msgs[0]->mtext) != 0) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msgrcv() failed - message not read into buffer");
	} else
		PATH_TRACE;

	if (buf.msg_qnum !=  NMSGS - 1) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_qnum not set as expected");
		in_rpt("Expected: %d, Observed: %lu", NMSGS - 1,
			(unsigned long) buf.msg_qnum);
	} else
		PATH_TRACE;
		
	rcvpid = getpid();

	if (buf.msg_lrpid != rcvpid) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_lrpid not set as expected");
		in_rpt("Expected: %ld, Observed: %ld", (long) rcvpid,
			(long) buf.msg_lrpid);
	} else
		PATH_TRACE;

	if ((buf.msg_rtime < time_before) || (buf.msg_rtime > time_after)) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_rtime not set as expected");
		in_rpt("Expected range: %ld - %ld, Observed: %ld",
			(long) time_before, (long) time_after,
			(long) buf.msg_rtime);
	} else
		PATH_TRACE;

	if (test_failed) {
		DBUG_EXIT(TEST_FAILED);
	} else if (pathok != 6) {
		in_rpt("pathok != 6 (%d)",pathok);
		DBUG_EXIT(TEST_PATH_FAIL);
	} else {
		DBUG_EXIT(TEST_OK);
	}
}

private void
pa_t5()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_t5");

	if ((stat_child = wait_child(WAIT_FOR_SUSPEND)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_SUSPEND, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("process did not suspend as expected");
		test_deleted++;
		DBUG_VOID_RETURN;
	} else 
		PATH_TRACE;

	globok = 0;
	if (sndmsgs() == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	if ((stat_child = wait_child(2*WAIT_FOR_EXIT)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_EXIT, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child == CHLD_EXIT) {
		switch (ex_code) {

		case TEST_FAILED:
		case TEST_DELETED:
			test_deleted++;
			DBUG_VOID_RETURN;

		case TEST_PATH_FAIL:
			break;

		case TEST_OK:
			PATH_TRACE;
			break;

		default:
			xx_rpt(DELETION);
			in_rpt("child exited with unexpected code");
			test_deleted++;
			DBUG_VOID_RETURN;
		}
	} else {
		xx_rpt(DELETION);
		in_rpt("child did not exit as expected");
		(void) kill(tet_child, SIGKILL);
		test_deleted++;
		DBUG_VOID_RETURN;
	}

	PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private	void
test6()
{
	DBUG_ENTER("test6");

	(void) cppair(t6_setup, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
t6_setup()
{
	int	pathok = setupok;

	DBUG_ENTER("t6_setup");
	test_failed = 0;
	test_deleted = 0;

	if (cr_msgq(id, MSG_READWRITE) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	if (sndmsgs() == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	if (cpsig(ch_t6, pa_6_7_10thru13, CHILD_TIMEOUT, OK) == FAIL || 
		test_deleted) {
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	size_t	msgsz, msgtruncsz;
	int	i;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t6");

	msgtruncsz = (size_t) (strlen(msg1.mtext) / (size_t) 2);

	if ((msgsz = msgrcv(msgqid, &clean_rcvmsg, msgtruncsz, 0, MSG_NOERROR)) 
		== SYSERROR) {

		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgrcv(msqid, *msgp, msgsz = %d, 0, MSG_NOERROR) failed - errno %d (%s)",
			msgtruncsz, err, errname(err));
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	if (strncmp(clean_rcvmsg.mtext, msg1.mtext, msgtruncsz) != 0) {
		xx_rpt(DELETION);
		in_rpt("msgrcv() failed - did not retrieve correct message from the queue");
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	for (i = msgtruncsz; i < MBUFFERSZ; i++) {
		if (clean_rcvmsg.mtext[i] != '\0') {
			if (!test_failed++)
				xx_rpt(FAILURE);
			in_rpt("msgrcv() failed - received message was not truncated to msgsz");
			break;
		} else
			PATH_TRACE;
	}
			
	if (msgsz != msgtruncsz) {	
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msgrcv() failed - returned message size was not equal to msgsz");
		in_rpt("MESSAGE SIZE: Sent: %d, Received: %d", 
			msgtruncsz, msgsz);
	} else
		PATH_TRACE;

	if (test_failed) {
		DBUG_EXIT(TEST_FAILED);
	} else if (pathok != (3 + (MBUFFERSZ - msgtruncsz))) {
		in_rpt("pathok != %d (%d)",(3 + (MBUFFERSZ - msgtruncsz)),pathok);
		DBUG_EXIT(TEST_PATH_FAIL);
	} else {
		DBUG_EXIT(TEST_OK);
	}
}

private void
pa_6_7_10thru13()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_6_7_10thru13");

	if ((stat_child = wait_child(WAIT_FOR_EXIT)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_EXIT, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child == CHLD_EXIT) {
		switch (ex_code) {

		case TEST_FAILED:
		case TEST_DELETED:
			test_deleted++;
			DBUG_VOID_RETURN;

		case TEST_PATH_FAIL:
			break;

		case TEST_OK:
			PATH_TRACE;
			break;

		default:
			xx_rpt(DELETION);
			in_rpt("child exited with unexpected code");
			test_deleted++;
			DBUG_VOID_RETURN;
		}
	} else {
		xx_rpt(DELETION);
		in_rpt("child did not exit as expected");
		(void) kill(tet_child, SIGKILL);
		test_deleted++;
		DBUG_VOID_RETURN;
	} 

	PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private	void
test7()
{
	DBUG_ENTER("test7");

	(void) cppair(t7_setup, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
t7_setup()
{
	int	pathok = setupok;

	DBUG_ENTER("t7_setup");
	test_failed = 0;
	test_deleted = 0;

	if (cr_msgq(ID7, MSG_READWRITE) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (cpsig(ch_t7, pa_6_7_10thru13, CHILD_TIMEOUT, OK) == FAIL || 
		test_deleted) {
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(2);
	
	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int	ret;
	int	err;

	DBUG_ENTER("ch_t7");

	ret = msgrcv(msgqid, &rcvmsg, (size_t) MBUFFERSZ, 0, IPC_NOWAIT);

	if (ret != SYSERROR || errno != ENOMSG) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgrcv() - did not give correct results");
		valrpt(SYSERROR, ret, ENOMSG, err);
		DBUG_EXIT(TEST_FAILED);
	} 
	
	DBUG_EXIT(TEST_OK);
}

private	void
test8()
{
	DBUG_ENTER("test8");

	(void) cppair(t8_setup, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
t8_setup()
{
	int	pathok = setupok;

	DBUG_ENTER("t8_setup");
	test_failed = 0;
	test_deleted = 0;

	if (cr_msgq(ID8, MSG_READWRITE) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (cpsig(ch_t8, pa_t8, CHILD_TIMEOUT, OK) == FAIL || 
		test_deleted) {
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(2);
	
	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int	ret;
	int	err;

	DBUG_ENTER("ch_t8");

	ret = msgrcv(msgqid, &rcvmsg, (size_t) MBUFFERSZ, 0, 0);

	if (ret != SYSERROR || errno != EIDRM) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgrcv() failed - unexpected results");
		valrpt(SYSERROR, ret, EIDRM, err);
		DBUG_EXIT(TEST_FAILED);
	}
		
	DBUG_EXIT(TEST_OK);
}

private void
pa_t8()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_t8");

	if ((stat_child = wait_child(WAIT_FOR_SUSPEND)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_SUSPEND, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("child process terminated unexpectedly");
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* remove queue from child */
	if (msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_RMID, NULL) failed - errno %d (%s)",
			err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	
	if ((stat_child = wait_child(WAIT_FOR_EXIT)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_SUSPEND, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child == CHLD_EXIT) {
		switch (ex_code) {

		case TEST_FAILED:
		case TEST_DELETED:
			test_deleted++;
			DBUG_VOID_RETURN;

		case TEST_PATH_FAIL:
			break;

		case TEST_OK:
			PATH_TRACE;
			break;

		default:
			xx_rpt(DELETION);
			in_rpt("child exited with unexpected code");
			test_deleted++;
			DBUG_VOID_RETURN;
		}
	} else {
		xx_rpt(DELETION);
		in_rpt("process did not exit as expected");
		test_deleted++;
		DBUG_VOID_RETURN;
	}

	PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private	void
test9()
{
	DBUG_ENTER("test9");

	(void) cppair(t9_setup, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
t9_setup()
{
	int	pathok = setupok;

	DBUG_ENTER("t9_setup");
	test_failed = 0;
	test_deleted = 0;

	if (cr_msgq(ID9, MSG_READWRITE) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (cpsig(ch_t9, pa_t9, CHILD_TIMEOUT, OK) == FAIL || 
		test_deleted) {
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(2);
	
	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t9");

	if (vsignal(SIGUSR1, t9_sig_catch) == SIG_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("vsignal(SIGUSR1, t4_sig_catch) failed - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	ret = msgrcv(msgqid, &rcvmsg, (size_t) MBUFFERSZ, 0, 0);

	if (ret != SYSERROR || errno != EINTR) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgrcv() failed - unexpected results");
		valrpt(SYSERROR, ret, EINTR, err);
		DBUG_EXIT(TEST_FAILED);
	} else
		PATH_TRACE;
		
	if (pathok != 2) {
		in_rpt("pathok != 2 (%d)",pathok);
		DBUG_EXIT(TEST_PATH_FAIL);
	} else {
		DBUG_EXIT(TEST_OK);
	}
}

private void
pa_t9()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_t9");

	if ((stat_child = wait_child(WAIT_FOR_SUSPEND)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_SUSPEND, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("child process terminated unexpectedly");
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* interrupt child */
	if (kill(tet_child, SIGUSR1) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("kill(tet_child, SIGUSR1) failed - errno %d (%s)",
			err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	
	if ((stat_child = wait_child(WAIT_FOR_EXIT)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_SUSPEND, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child == CHLD_EXIT) {
		switch (ex_code) {

		case TEST_FAILED:
		case TEST_DELETED:
			test_deleted++;
			DBUG_VOID_RETURN;

		case TEST_PATH_FAIL:
			break;

		case TEST_OK:
			PATH_TRACE;
			break;

		default:
			xx_rpt(DELETION);
			in_rpt("child exited with unexpected code");
			test_deleted++;
			DBUG_VOID_RETURN;
		}
	} else {
		xx_rpt(DELETION);
		in_rpt("process did not exit as expected");
		test_deleted++;
		DBUG_VOID_RETURN;
	}

	PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
t9_sig_catch() 
{
	DBUG_ENTER("t9_sig_catch");

	DBUG_VOID_RETURN;
}

private	void
test10()
{
	DBUG_ENTER("test10");

	(void) cppair(t10_setup, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
t10_setup()
{
	int	pathok = setupok;

	DBUG_ENTER("t10_setup");
	test_failed = 0;
	test_deleted = 0;

	if (cpsig(ch_t10, pa_6_7_10thru13, CHILD_TIMEOUT, OK) == FAIL || 
		test_deleted) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	PATH_XS_RPT(1);
	
	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	int	ret;
	int	err;

	DBUG_ENTER("ch_t10");

	ret = msgrcv(INVALID_MSGQID, &rcvmsg, (size_t) MBUFFERSZ, 0, 0);

	if (ret != SYSERROR || errno != EINVAL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgrcv() failed - unexpected results");
		valrpt(SYSERROR, ret, EINVAL, err);
		DBUG_EXIT(TEST_FAILED);
	}
		
	DBUG_EXIT(TEST_OK);
}

private	void
test11()
{
	DBUG_ENTER("test11");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

private	void
test12()
{
	DBUG_ENTER("test12");

	(void) cppair(t12_setup, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
t12_setup()
{
	int	pathok = setupok;

	DBUG_ENTER("t12_setup");
	test_failed = 0;
	test_deleted = 0;

	if (cr_msgq(ID12, MSG_WRITEONLY) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	if (sndmsgs() == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	if (cpsig(ch_t12, pa_6_7_10thru13, CHILD_TIMEOUT, OK) == FAIL || 
		test_deleted) {
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	int	ret;
	int	err;

	DBUG_ENTER("ch_t12");

	ret = msgrcv(msgqid, &rcvmsg, (size_t) MBUFFERSZ, 0, 0);

	if (ret != SYSERROR || errno != EACCES) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgrcv() failed - unexpected results");
		valrpt(SYSERROR, ret, EACCES, err);
		DBUG_EXIT(TEST_FAILED);
	}
		
	DBUG_EXIT(TEST_OK);
}

private	void
test13()
{
	DBUG_ENTER("test13");

	(void) cppair(t13_setup, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
t13_setup()
{
	int	pathok = setupok;

	DBUG_ENTER("t13_setup");
	test_failed = 0;
	test_deleted = 0;

	if (cr_msgq(ID13, MSG_READWRITE) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	if (sndmsgs() == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	if (cpsig(ch_t13, pa_6_7_10thru13, CHILD_TIMEOUT, OK) == FAIL || 
		test_deleted) {
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
ch_t13()
{
	int	ret;
	int	err;

	DBUG_ENTER("ch_t13");

	ret = msgrcv(msgqid, &rcvmsg, 1, 0, 0);

	if (ret != SYSERROR || errno != E2BIG) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgrcv() failed - unexpected results");
		valrpt(SYSERROR, ret, E2BIG, err);
		DBUG_EXIT(TEST_FAILED);
	}
		
	DBUG_EXIT(TEST_OK);
}
