/*
*	SCCS:  @(#)  XPG4.os/ipc/msgctl/msgsnd.c Rel 4.4.1     (06/09/97)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)  XPG4.os/ipc/msgctl/msgsnd.c Rel 4.4.1     (06/09/97)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.msgsnd

PROJECT:	XPG4 Validation suite

DESCRIPTION:
	Additional tests for the msgsnd() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		John Onusko
DATE CREATED:	12/19/90
MODIFICATIONS:	
		Matt Newman, Tue Sep 10 13:18:03 BST 1991
			XPG4 Intergration

		Geoff Clare, 29 Apr 1992
			Changes for new types of msg_qnum and msg_qbytes.

		John-Paul, 14 July 1992
			Test 2: Added missing call to remove message queue
			upon failure.
			Tests 2, 4, 5 and 6: Tests now send two messages
			rather than one.

		Geoff Clare, 16 June 1993
			Add declaration of ftok().
		
		Paul Haffenden, 24 Sept 1993
			New version of pa_t2().

		Geoff Clare, 29 March 1994
			Wait for child after sending SIGUSR1 in pa_t2().

************************************************************************/

#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<time.h>
#include	<errno.h>
#include	<sys/ipc.h>
#include	<sys/msg.h>
#include	<sys/stat.h>
#include	<string.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<setprv.h>
#include	<report.h>
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef msgsnd
#endif


#define NO_TESTS	10	/* number of tests in testset */

#define	TEST_TIMEOUT		(10 * SPEEDFACTOR)
#define	WAIT_FOR_SUSPEND	(5 * SPEEDFACTOR)
#define	WAIT_FOR_EXIT		(3 * SPEEDFACTOR)
#define	MODEANY			((mode_t)(S_IRWXU|S_IRWXG|S_IRWXO))

#define	TEST_OK			0
#define	TEST_DELETED		1
#define	TEST_PATH_FAIL		2
#define	TEST_DID_NOT_SUSPEND	3
#define	TEST_FAILED		4

#define	ID1	11
#define	ID2	22
#define	ID3	33
#define	ID4	44
#define	ID5	55
#define	ID6	66
#define	ID8	88
#define	ID9	99
#define	ID10	1010

#define	INVALID_MSGQID	-1
#define	INVALID_SIZE	-1
#define	INVALID_MTYPE	-1

/*
 * message access permissions
 */
#define	RW_U		(S_IRUSR|S_IWUSR)	
#define	RW_G		(S_IRGRP|S_IWGRP)
#define	RW_O		(S_IROTH|S_IWOTH)
#define	MSG_READWRITE	(RW_U|RW_G|RW_O)
#define	MSG_READONLY	(S_IRUSR|S_IRGRP|S_IROTH)

public	int	no_tests = NO_TESTS;

extern	int	ex_code;
extern	int	vguid0, vggid0;

#ifndef ftok
extern	key_t	ftok();
#endif
extern	char	*errname();
extern	void	valrpt();
extern	int	wait_child();
extern	int	uids();
extern	int	setprv();
extern	void	(*vsignal())();

private	char	*msgpath	= "./msgsnd.f";
private	int	msgqid;

#define	MSG_TYPE	1
#define	MSG_BLOCK_TYPE	2
#define MBUFFERSZ	256

struct test_msg {
	long	mtype;
	char	mtext[MBUFFERSZ];
};

private	struct 	
test_msg  msg = {
		MSG_TYPE,
		"this is a test"
};

private	struct 	
test_msg  msg_part1 = {
		MSG_TYPE,
		"this is the first part"
};

private	struct 	
test_msg  msg_part2 = {
		MSG_TYPE,
		"this is the second part"
};

private struct
test_msg  block_msg = {
		MSG_BLOCK_TYPE,
		"this message will block msgtype 1"
};

private	struct 	
test_msg  bad_msg = {
		INVALID_MTYPE,
		"this is a test"
};

private struct test_msg 	rcvmsg;

private	size_t	msgsz;
private size_t  msgsz2;
private	size_t	block_msgsz;
private	pid_t	sendpid;
private int	setupok = -1;
private int	test_failed;
private int	test_deleted;

private	void	setup(), cleanup(), test1(), ch_t1(), test2(), ch_t2(),
		pa_t2(), t2_sig_catch(), test3(), ch_t3(), pa_t3(), test4(),
		ch_t4(), pa_t4(), t4_sig_catch(), test5(), ch_t5(), pa_t5(),
		test6(), ch_t6(), test7(), ch_t7(), test8(), ch_t8(), test9(),
		ch_t9(), test10(), ch_t10();

public	void	(*setfunc)() = setup;
public	void	(*clnfunc)() = cleanup;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	NULL, 0
};

private void
setup()
{
	static char	errmsg[512];

	int	i;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("setup");

	(void) uids();
	PATH_TRACE;

	if (setprv(PRV_SETID)) {
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "setprv(PRV_SETID) failed in setup");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
		
	if (crfile(msgpath, MODEANY, vguid0, vggid0) 
		== SYSERROR) {

		err = errno;
		(void) sprintf(errmsg, "crfile(\"%s\") failed in setup - errno %d (%s)",
			msgpath, err, errname(err));
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, errmsg);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (pathok == 3)
		setupok = 0;

	DBUG_VOID_RETURN;
}

private	void
cleanup()
{
	DBUG_ENTER("cleanup");

	(void) unlink(msgpath);

	DBUG_VOID_RETURN;
}

private	int
cr_msgq(id,msgflg)
int id;
int msgflg;
{
	int	qid;
	key_t	key;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("cr_msgq");
	
	if ((key = ftok(msgpath, id)) == (key_t) SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(\"%s\", %d) failed", msgpath, id);
		DBUG_RETURN(SYSERROR);
	} else
		PATH_TRACE;

	if ((qid = msgget(key, (IPC_CREAT|msgflg))) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgget(key = %d, msgflg = 0%o) failed - errno %d (%s)",
			(int) key, (IPC_CREAT|msgflg), err, 
			errname(err));
		(void) msgctl(qid, IPC_RMID, (struct msqid_ds *)0);
		test_deleted++;
		DBUG_RETURN(SYSERROR);
	} else
		PATH_TRACE;

	PATH_XS_RPT(2);

	DBUG_RETURN(qid);
}

private	void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
ch_t1()
{
	struct msqid_ds buf;

	time_t	time_before, time_after;
	int	ret;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t1");
	test_failed = 0;

	(void) setuid((uid_t)vguid0);

	if ((msgqid = cr_msgq(ID1,MSG_READWRITE)) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (msgctl(msgqid, IPC_STAT, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_STAT) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	msgsz = strlen(msg.mtext) + 1;

	if (buf.msg_qbytes < msgsz) {
		xx_rpt(DELETION);
		in_rpt("insufficent space in message queue");
		in_rpt("Required: %d bytes, Allocated: %lu bytes", msgsz,
			(unsigned long) buf.msg_qbytes);
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	time_before = time((time_t *)0);
	
	ret = msgsnd(msgqid, (void *)&msg, msgsz, 0);

	time_after = time((time_t *)0);

	if (ret == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgsnd(msqid = %d, *msgp = 0x%lx, msgsz = %d, msgflg = NULL) failed -",
			msgqid, (long) &msg, msgsz);
		in_rpt("errno %d (%s)", err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	
	if (ret != 0) {
		xx_rpt(FAILURE);
		in_rpt("msgsnd(msqid = %d, *msgp = 0x%lx, msgsz = %d, msgflg = NULL) failed -",
			msgqid, (long) &msg, msgsz);
		in_rpt("did not return zero");
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (msgctl(msgqid, IPC_STAT, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_STAT) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (buf.msg_qnum != 1) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_qnum not set as expected");
		in_rpt("Expected: 1, Observed: %lu", (unsigned long) buf.msg_qnum);
	} else
		PATH_TRACE;
		
	sendpid = getpid();

	if (buf.msg_lspid != sendpid) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_lspid not set as expected");
		in_rpt("Expected: %ld, Observed: %ld", (long) sendpid,
			(long) buf.msg_lspid);
	} else
		PATH_TRACE;

	if ((buf.msg_stime < time_before) || (buf.msg_stime > time_after)) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_stime not set as expected");
		in_rpt("Expected range: %ld - %ld, Observed: %ld",
			(long) time_before, (long) time_after,
			(long) buf.msg_stime);
	} else
		PATH_TRACE;

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	if (!test_failed)
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	DBUG_ENTER("test2");

	(void) cpsig(ch_t2, pa_t2, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
ch_t2()
{
	struct msqid_ds buf;

	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	(void) setuid((uid_t)vguid0);
	(void) vsignal(SIGUSR1, t2_sig_catch);

	if ((msgqid = cr_msgq(ID2, MSG_READWRITE)) == SYSERROR) {
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	if (msgctl(msgqid, IPC_STAT, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_STAT) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	msgsz = strlen(msg_part1.mtext) + 1;
	msgsz2 = strlen(msg_part2.mtext) + 1;

	buf.msg_qbytes = msgsz + msgsz2 - 1;
	
	if (msgctl(msgqid, IPC_SET, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_SET) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	if (pathok != 3)
		DBUG_EXIT(TEST_PATH_FAIL);

	if (msgsnd(msgqid, (void *)&msg_part1, msgsz, 0) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgsnd(msqid = %d, *msgp = 0x%lx, msgsz = %d, msgflg = NULL) failed -", msgqid, (long) &msg_part1, msgsz);
		in_rpt("errno %d (%s)", err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_DELETED);
	}

	/* should hang here */
	if (msgsnd(msgqid, (void *)&msg_part2, msgsz2, 0) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgsnd(msqid = %d, *msgp = 0x%lx, msgsz2 = %d, msgflg = NULL) failed -",
			msgqid, (long) &msg_part2, msgsz2);
		in_rpt("errno %d (%s)", err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_DELETED);
	} 
		
	/* NOT REACHED */

	DBUG_EXIT(TEST_DID_NOT_SUSPEND);
}

private void
pa_t2()
{
	int	err;
	int	stat_child;
	int	pathok = setupok;

	DBUG_ENTER("pa_t2");

	if ((stat_child = wait_child(WAIT_FOR_SUSPEND)) != CHLD_LIVE)
	{
		if(stat_child == CHLD_ERR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
				WAIT_FOR_SUSPEND, err, errname(err));
			DBUG_VOID_RETURN;
		}
		if (stat_child == CHLD_EXIT)
		{
			switch (ex_code)
			{

				case TEST_DELETED:
				DBUG_VOID_RETURN;

				case TEST_PATH_FAIL:
				break;

				case TEST_DID_NOT_SUSPEND:
				xx_rpt(FAILURE);
				in_rpt("msgsnd() did not suspend process when resources not available");
				(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
				DBUG_VOID_RETURN;

				default:
				xx_rpt(DELETION);
				in_rpt("child exited with unexpected code");
				DBUG_VOID_RETURN;
			}
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("Unexpected return code from wait_child got 0x%x expected 0x%x (CHLD_LIVE)", stat_child, CHLD_LIVE);
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

	/* tell child to clean up and die */
	/* We can only get here if the child is alive */
	(void) kill(tet_child, SIGUSR1);
	(void) wait_child(WAIT_FOR_EXIT);

	PATH_XS_RPT(1);
	DBUG_VOID_RETURN;
}

private void
t2_sig_catch()
{
	DBUG_ENTER("t2_sig_catch");

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	DBUG_EXIT(TEST_OK);
}

private	void
test3()
{
	struct msqid_ds buf;

	time_t	time_before, time_after;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test3");
	test_failed = 0;
	test_deleted = 0;

	if ((msgqid = cr_msgq(ID3, MSG_READWRITE)) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (msgctl(msgqid, IPC_STAT, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_STAT) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	block_msgsz = buf.msg_qbytes = strlen(block_msg.mtext) + 1;
	msgsz = strlen(msg.mtext) + 1;
	
	if (msgctl(msgqid, IPC_SET, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_SET) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (msgsnd(msgqid, (void *)&block_msg, block_msgsz, IPC_NOWAIT) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgsnd(msqid = %d, *msgp = 0x%lx, msgsz = %d, msgflg = IPC_NOWAIT) failed -",
			msgqid, (long) &block_msg, block_msgsz);
		in_rpt("errno %d (%s)", err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
	} else
		PATH_TRACE;
		
	time_before = time((time_t *)0);

	if (cpsig(ch_t3, pa_t3, TEST_TIMEOUT, OK) == FAIL || test_deleted) {
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		(void) kill(sendpid, SIGKILL);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	time_after = time((time_t *)0);

	if (msgctl(msgqid, IPC_STAT, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_STAT) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (buf.msg_qnum != 1) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_qnum not set as expected");
		in_rpt("Expected: 1, Observed: %lu", (unsigned long) buf.msg_qnum);
	} else
		PATH_TRACE;
		
	if (buf.msg_lspid != sendpid) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_lspid not set as expected");
		in_rpt("Expected: %ld, Observed: %ld", (long) sendpid,
			(long) buf.msg_lspid);
	} else
		PATH_TRACE;

	if ((buf.msg_stime < time_before) || (buf.msg_stime > time_after)) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msg_stime not set as expected");
		in_rpt("Expected range: %ld - %ld, Observed: %ld",
			(long) time_before, (long) time_after,
			(long) buf.msg_stime);
	} else
		PATH_TRACE;

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	if (!test_failed)
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}

private	void
ch_t3()
{
	int	err;
	int	ret;

	DBUG_ENTER("ch_t3");

	(void) setuid((uid_t)vguid0);

	/* should hang here */
	ret = msgsnd(msgqid, (void *)&msg, msgsz, 0);

	if (ret == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgsnd(msqid = %d, *msgp = 0x%lx, msgsz = %d, msgflg = NULL) failed -",
			msgqid, (long) &msg, msgsz);
		in_rpt("errno %d (%s)", err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_DELETED);
	}
	
	if (ret != 0) {
		xx_rpt(FAILURE);
		in_rpt("msgsnd(msqid = %d, *msgp = 0x%lx, msgsz = %d, msgflg = NULL) failed -",
			msgqid, (long) &msg, msgsz);
		in_rpt("did not return zero");
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_FAILED);
	} 
	
	DBUG_EXIT(TEST_OK);
}

private void
pa_t3()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_t3");

	sendpid = tet_child;

	if ((stat_child = wait_child(WAIT_FOR_SUSPEND)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_SUSPEND, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child == CHLD_EXIT && ex_code == TEST_DELETED) {
		test_deleted++;
		DBUG_VOID_RETURN;
	} else if (stat_child == CHLD_EXIT && ex_code == TEST_FAILED) {
		/* this should never happen */
		test_deleted++;
		DBUG_VOID_RETURN;
	} else if (stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("child did not suspend as expected");
		test_deleted++;
		DBUG_VOID_RETURN;
	} else 
		/* child suspended as expected */
		PATH_TRACE;

	if ((msgsz = msgrcv(msgqid, (void *)&rcvmsg, (size_t) MBUFFERSZ, 
		MSG_BLOCK_TYPE, IPC_NOWAIT)) == SYSERROR) {
		
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgrcv(msqid, *msgp, msgsz = %d, msgtyp = %d, IPC_NOWAIT) failed - errno %d (%s)",
			MBUFFERSZ, MSG_BLOCK_TYPE, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (msgsz != block_msgsz) {
		xx_rpt(DELETION);
		in_rpt("unable to remove suspension condition");
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	
	if ((stat_child = wait_child(WAIT_FOR_EXIT)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_SUSPEND, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child == CHLD_EXIT && ex_code == TEST_DELETED) {
		test_deleted++;
		DBUG_VOID_RETURN;
	} else if (stat_child == CHLD_EXIT && ex_code == TEST_FAILED) {
		test_failed++;
		DBUG_VOID_RETURN;
	} else if (stat_child != CHLD_EXIT || ex_code != TEST_OK) {
		if (!test_failed++)
			xx_rpt(FAILURE);
		in_rpt("msgsnd() did not complete after suspension condition removed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private	void
test4()
{
	DBUG_ENTER("test4");

	(void) cpsig(ch_t4, pa_t4, TEST_TIMEOUT, OK);

	DBUG_VOID_RETURN;
}

private	void
ch_t4()
{
	struct msqid_ds buf;

	int	err;
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	(void) setuid((uid_t)vguid0);

	if (vsignal(SIGUSR1, t4_sig_catch) == SIG_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("vsignal(SIGUSR1, t4_sig_catch) failed - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	if ((msgqid = cr_msgq(ID4, MSG_READWRITE)) == SYSERROR) {
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	if (msgctl(msgqid, IPC_STAT, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_STAT) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	msgsz = strlen(msg_part1.mtext) + 1;
	msgsz2 = strlen(msg_part2.mtext) + 1;
	buf.msg_qbytes = msgsz + msgsz2 - 1;
	
	if (msgctl(msgqid, IPC_SET, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_SET) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	if (msgsnd(msgqid, (void *)&msg_part1, msgsz, 0) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgsnd(msqid = %d, *msgp = 0x%lx, msgsz = %d, msgflg = NULL) failed -", msgqid, (long) &msg_part1, msgsz);
		in_rpt("errno %d (%s)", err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_DELETED);
	}

	/* should hang here */
	ret = msgsnd(msgqid, (void *)&msg_part2, msgsz2, 0);

	if (ret != SYSERROR || errno != EINTR) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgsnd() failed - unexpected results");
		valrpt(SYSERROR, ret, EINTR, err);
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_FAILED);
	} else
		PATH_TRACE;
		
	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	if (pathok != 5) {
		DBUG_EXIT(TEST_PATH_FAIL);
	} else {
		DBUG_EXIT(TEST_OK);
	}
}

private void
pa_t4()
{
	int	err;
	int	stat_child;
	int	pathok = setupok;

	DBUG_ENTER("pa_t4");

	if ((stat_child = wait_child(WAIT_FOR_SUSPEND)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_SUSPEND, err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("child process terminated unexpectedly");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* tell child to clean up and exit */
	if (kill(tet_child, SIGUSR1) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("kill(tet_child, SIGUSR1) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	
	if ((stat_child = wait_child(WAIT_FOR_EXIT)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_SUSPEND, err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child == CHLD_EXIT) {
		switch (ex_code) {

		case TEST_FAILED:
		case TEST_DELETED:
			DBUG_VOID_RETURN;

		case TEST_PATH_FAIL:
			break;

		case TEST_OK:
			PATH_TRACE;
			break;

		default:
			xx_rpt(DELETION);
			in_rpt("child exited with unexpected code");
			DBUG_VOID_RETURN;
		}
	} else {
		xx_rpt(DELETION);
		in_rpt("process did not exit as expected");
		DBUG_VOID_RETURN;
	}

	PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
t4_sig_catch()
{
	DBUG_ENTER("t4_sig_catch");

	DBUG_VOID_RETURN;
}

private	void
test5()
{
	struct msqid_ds buf;

	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test5");
	test_failed = 0;
	test_deleted = 0;

	if ((msgqid = cr_msgq(ID5, MSG_READWRITE)) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (msgctl(msgqid, IPC_STAT, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_STAT) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	msgsz = strlen(msg_part1.mtext) + 1;
	msgsz2 = strlen(msg_part2.mtext) + 1;
	buf.msg_qbytes = msgsz + msgsz2 - 1;
	
	if (msgctl(msgqid, IPC_SET, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_SET) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (cpsig(ch_t5, pa_t5, TEST_TIMEOUT, OK) == FAIL || test_deleted) {
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else 
		PATH_TRACE;

	PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private	void
ch_t5()
{
	int	err;
	int	ret;

	DBUG_ENTER("ch_t5");

	(void) setuid((uid_t)vguid0);

	if (msgsnd(msgqid, (void *)&msg_part1, msgsz, 0) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgsnd(msqid = %d, *msgp = 0x%lx, msgsz = %d, msgflg = NULL) failed -", msgqid, (long) &msg_part1, msgsz);
		in_rpt("errno %d (%s)", err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_DELETED);
	}

	/* should hang here */
	ret = msgsnd(msgqid, (void *)&msg_part2, msgsz2, 0);

	if (ret != SYSERROR || errno != EIDRM) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgsnd() failed - unexpected results");
		valrpt(SYSERROR, ret, EIDRM, err);
		DBUG_EXIT(TEST_FAILED);
	}
		
	DBUG_EXIT(TEST_OK);
}

private void
pa_t5()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_t5");

	if ((stat_child = wait_child(WAIT_FOR_SUSPEND)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_SUSPEND, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("child process terminated unexpectedly");
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* remove queue from child */
	if (msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_RMID, NULL) failed - errno %d (%s)",
			err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	
	if ((stat_child = wait_child(WAIT_FOR_EXIT)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("wait_child(waittime = %d) failed - errno %d (%s)",
			WAIT_FOR_SUSPEND, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (stat_child == CHLD_EXIT) {
		switch (ex_code) {

		case TEST_FAILED:
		case TEST_DELETED:
			test_deleted++;
			DBUG_VOID_RETURN;

		case TEST_PATH_FAIL:
			break;

		case TEST_OK:
			PATH_TRACE;
			break;

		default:
			xx_rpt(DELETION);
			in_rpt("child exited with unexpected code");
			test_deleted++;
			DBUG_VOID_RETURN;
		}
	} else {
		xx_rpt(DELETION);
		in_rpt("process did not exit as expected");
		test_deleted++;
		DBUG_VOID_RETURN;
	}

	PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	DBUG_ENTER("test6");

	(void) cppair(ch_t6, NULLFN, TEST_TIMEOUT, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t6()
{
	struct msqid_ds buf;

	int	ret;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t6");

	(void) setuid((uid_t)vguid0);

	if ((msgqid = cr_msgq(ID6, MSG_READWRITE)) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (msgctl(msgqid, IPC_STAT, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_STAT) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	msgsz = strlen(msg_part1.mtext) + 1;
	msgsz2 = strlen(msg_part2.mtext) + 1;
	buf.msg_qbytes = msgsz + msgsz2 - 1;
	
	if (msgctl(msgqid, IPC_SET, &buf) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgctl(msgqid, IPC_SET) failed - errno %d (%s)",
			err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (msgsnd(msgqid, (void *)&msg_part1, msgsz, 0) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("msgsnd(msqid = %d, *msgp = 0x%lx, msgsz = %d, msgflg = NULL) failed -", msgqid, (long) &msg_part1, msgsz);
		in_rpt("errno %d (%s)", err, errname(err));
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_EXIT(TEST_DELETED);
	} else
		PATH_TRACE;

	ret = msgsnd(msgqid, (void *)&msg_part2, msgsz2, IPC_NOWAIT);

	if (ret != SYSERROR || errno != EAGAIN) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgsnd() failed - unexpected results");
		valrpt(SYSERROR, ret, EAGAIN, err);
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
		
	PATH_XS_RPT(5);

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	DBUG_ENTER("test7");

	(void) cppair(ch_t7, NULLFN, TEST_TIMEOUT, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t7()
{
	int	ret;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t7");

	(void) setuid((uid_t)vguid0);

	ret = msgsnd(INVALID_MSGQID, (void *)&msg, msgsz, 0);

	if (ret != SYSERROR || errno != EINVAL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgsnd() failed - unexpected results");
		valrpt(SYSERROR, ret, EINVAL, err);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
		
	PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	DBUG_ENTER("test8");

	(void) cppair(ch_t8, NULLFN, TEST_TIMEOUT, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t8()
{
	int	ret;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t8");

	(void) setuid((uid_t)vguid0);

	if ((msgqid = cr_msgq(ID8, MSG_READWRITE)) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	ret = msgsnd(msgqid, (void *)&bad_msg, msgsz, 0);

	if (ret != SYSERROR || errno != EINVAL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgsnd() failed - unexpected results");
		valrpt(SYSERROR, ret, EINVAL, err);
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
		
	PATH_XS_RPT(2);

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	DBUG_ENTER("test9");

	(void) cppair(ch_t9, NULLFN, TEST_TIMEOUT, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t9()
{
	int	ret;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t9");

	(void) setuid((uid_t)vguid0);

	if ((msgqid = cr_msgq(ID9, MSG_READWRITE)) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	ret = msgsnd(msgqid, (void *)&msg, INVALID_SIZE, 0);

	if (ret != SYSERROR || errno != EINVAL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgsnd() failed - unexpected results");
		valrpt(SYSERROR, ret, EINVAL, err);
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
		
	PATH_XS_RPT(2);

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	DBUG_ENTER("test10");

	(void) cppair(ch_t10, NULLFN, TEST_TIMEOUT, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t10()
{
	int	ret;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t10");

	(void) setuid((uid_t)vguid0);

	if ((msgqid = cr_msgq(ID10, MSG_READONLY)) == SYSERROR) {
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	ret = msgsnd(msgqid, (void *)&msg, msgsz, 0);

	if (ret != SYSERROR || errno != EACCES) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msgsnd() failed - unexpected results");
		valrpt(SYSERROR, ret, EACCES, err);
		(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
		
	PATH_XS_RPT(2);

	(void) msgctl(msgqid, IPC_RMID, (struct msqid_ds *)0);

	DBUG_VOID_RETURN;
}
