/*
*	SCCS:  @(#)  XPG4.os/ipc/semctl/semctl.c Rel 4.4.3     (07/20/98)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)  XPG4.os/ipc/semctl/semctl.c Rel 4.4.3     (07/20/98)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.semctl

PROJECT:	XPG4 Validation suite

DESCRIPTION:
	Generic tests for the semctl() system call.

AUTHOR:		Brian DeForest, UniSoft Corp.
DATE CREATED:	Fri Nov 16 11:33:56 PST 1990
MODIFICATIONS:	
		Alan Madsen January, 1991

		Matt Newman, Tue Sep 10 13:52:10 BST 1991
			XPG4 Intergration
		
		Geoff Clare, 17 Jan 1992
			Fix problems with testfail variable in error tests

		Peter Skan, 7 Feb 1992
			Added IPC and ACCESS privileges
			Remove privileges with unsetprv() where necessary
		
		Geoff Clare, 16 June 1993
			Changes for clean lint.
		
		Geoff Clare, 6 March 1998
			Change mbuf.mtype to long.

************************************************************************/
#include        <std.h>
#include        <dbug.h>
#include        <report.h>
#include        <sys/types.h>
#include        <stdio.h>
#include        <stdlib.h>
#include        <unistd.h>
#include        <errno.h>
#include        <string.h>
#include	<signal.h>
#include	<sys/stat.h>
#include	<sys/ipc.h>
#include	<sys/sem.h>
#include	<sys/msg.h>
#include	<sys/wait.h>
#include        <setjmp.h>
#include        <setprv.h>
#include 	<tet_api.h>
#include        <sysdep.h>

#ifdef UNDEF_MACROS
#undef semctl
#endif


#define 	NO_TESTS	32		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

#define		NSEMS_SING	1	/* single sem per set */
#define		NSEMS_MULT	2	/* multiple sems per set */
#define 	NSEMS		NSEMS_SING /* number sems created per set 	*/
#define 	SEMNO		0	/* sem number used in set	*/
#define 	SEMVAL		1	/* sem initial value 		*/

#define 	VERIFY		1
#define 	NOVERIFY	0

#define         FAILFILE        "/tmp/sem_fail"
#define         FAILRM          (void)unlink(FAILFILE)

#define		ERRBUFSZ	256

#define		NOK		02
#define		PATH_NOK	04

/*
 * 	access permissions
 */
#define		RW_U		(S_IRUSR|S_IWUSR)	/* *rw-?????? 0600 */
#define		RW_G		(S_IRGRP|S_IWGRP)	/* *???rw-??? 0060 */
#define		RW_O		(S_IROTH|S_IWOTH)	/* *??????rw- 0006 */
#define		RW_ALL		(RW_U|RW_G|RW_O)	/* *rw-rw-rw- 0666 */
#define		RW_NONE		(0)			/* *--------- 0000 */

/*
 *      keys
 */
#define KEY_BASE	((int)'K' << 8)
#define KEY_RW_ALL	((key_t)(KEY_BASE + 00))
#define KEY_RW_OWNER	((key_t)(KEY_BASE + 01))
#define KEY_RW_NONE	((key_t)(KEY_BASE + 02))
#define KEY_MULT	((key_t)(KEY_BASE + 03))
#define KEY_TEST	((key_t)(KEY_BASE + 04))
#define KEY_MSGID	((key_t)(KEY_BASE + 05))

#define 	SEM_RM(semid,semun)			semctl(semid, 0, IPC_RMID, semun)


#define 	SEMCTL(semid, semno, cmd, cmdstr, arg, verify, fail, err, rpt)  { \
		int ver; \
		ver = verify; \
		err = 0; \
		if ((fail = semctl(semid, semno, cmd, arg)) == SYSERROR) { \
			err = errno; \
               		if (testfail++ == 0 && (tstfailed() == 0)) { \
               			if (ver) { \
                       			xx_rpt(rpt); \
                       			in_rpt("semctl(%d, %d, %s, semun) unexpected return",semid,semno,cmdstr); \
                       			valrpt(NOERROR,fail,NOERROR,err); \
				} \
			} \
		} \
		else \
			PATH_TRACE; \
}

#define		SEMSET(semid, ds, semun, _uid, _gid, _mod, verify, rpt) { \
		ds.sem_perm.uid = _uid; \
		ds.sem_perm.gid = _gid; \
		ds.sem_perm.mode = _mod; \
		semun.buf = &ds; \
		SEMCTL(semid, SEMNO, IPC_SET, "IPC_SET", semun, verify, fail, err, rpt); }


/*	interprocess message passing	*/
#define         MSGOK           "good"
#define         MSGFAIL         "fail"
#define         MSGSZ           4
#define         MSGTYP          1

#define         MSGSND(mbuf,err) { \
				err = 0; \
                                mbuf.mtype = MSGTYP; \
                                if (msgsnd(msgid, (void *)&mbuf, MSGSZ, IPC_NOWAIT) == SYSERROR) { \
                                        err = errno; \
               				if (testfail++ == 0 && (tstfailed() == 0)) { \
                                                xx_rpt(DELETION); \
                                                in_rpt("msgsnd() unexpectedly failed, error: %s",errname(err)); \
                                        } \
                                } }

#define         MSGRCV(mbuf,err) { \
				err = 0; \
                                mbuf.mtype = MSGTYP; \
                                if (msgrcv(msgid, (void *)&mbuf, MSGSZ, 0, IPC_NOWAIT) == SYSERROR) { \
                                        err = errno; \
               				if (testfail++ == 0 && (tstfailed() == 0)) { \
                                                xx_rpt(DELETION); \
                                                in_rpt("msgrcv() unexpectedly failed, error: %s",errname(err)); \
                                        } \
                                } \
                                else \
                                        PATH_TRACE; }

#define		TIMER	5
#define 	SETTIMER      	{ timeout = 0; \
				  (void)alarm(TIMER); \
			 	  (void)vsignal(SIGALRM, semtimer); \
				if (sigsetjmp(jump_buf, 0)==0) \
				{
#define 	CLEARTIMER	} (void)alarm(0); }
#define		TIMEDOUT	(timeout != 0)

private	void	test1(), test2(), test3(), test4(), test5(), test6(), 
		test7(), test8(), test9(), test10(), test11(), test12(),
		test13(), test14(), test15(), test16(), test17(), test18(),
		test19(), test20(), test21(), test22(), test23(), test24(),
		test25(), test26(), test27(), test28(), test29(), test30(),
		test31(), test32();
private	void	ch_t1(), ch_t2(), ch_t3(), ch_t4(), ch_t5(), 
		ch_t6(), ch_t7(), ch_t8(), ch_t9(), ch_t10(), ch_t11(), 
		ch_t12(), ch_t13(), ch_t14(), ch_t15(), ch_t16(), 
		ch_t17(), ch_t18(), ch_t19(), 
		ch_t20(), ch_t21(), ch_t22(), 
		ch_t23(), ch_t24(), ch_t25(), 
		ch_t30(), ch_t31(), ch_t32();

private void	semtimer();
private	void	setup(), cleanup();
private int	tstfailed();
private int	create_semid();
private	void	ch_create_test_semid(), ch_create_rw_all_semid(),
		ch_create_rw_owner_semid(), ch_create_mult_semid(),
		ch_create_rw_none_semid();
private void	sem_access(), sem_get_cnt(), sem_getncnt(), sem_rmid_user(), 
		sem_set_user();
private int	timeout, setupok = 0;
private	int	semid_rw_all, semid_rw_owner, semid_rw_none, semid_mult, 
		semid_test;
private int     msgid;
private uid_t	super_user; 
private	gid_t	deflt_group;

private struct  mbuf {
        long    mtype;
        char    mtext[MSGSZ];
};

private union	semun {
	int	val;
	struct 	semid_ds *buf;
	unsigned short	*array;
} semun;

private	char	errbuf[ERRBUFSZ];

private	sigjmp_buf	jump_buf;

/* 	test harness declarations	*/

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = cleanup;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	 test29, 29,
	 test30, 30,
	 test31, 31,
	 test32, 32,
	NULL, 0
};


public  int     no_tests = NO_TESTS;

extern 	int	vguid0, vguid1, vguid2, vggid0, vggid1, vggid2;
extern	int 	cppair(), crfile(), setprv(), uids();
extern	char* 	errname();
extern	char* 	itoa();
extern	void	valrpt(); 
extern	void	(*vsignal())(); 

private	void
cleanup()
{
	DBUG_ENTER("cleanup");

	/*	remove semaphore sets created in setup()	*/

	(void)SEM_RM(semid_rw_all,semun);

	(void)SEM_RM(semid_rw_owner,semun);

	(void)SEM_RM(semid_rw_none,semun);

	(void)SEM_RM(semid_mult,semun);

	(void)SEM_RM(semid_test,semun);

        (void)msgctl(msgid, IPC_RMID, (struct msqid_ds *)0);

	FAILRM;

	DBUG_VOID_RETURN;
}

#if TESTBED1
#define		semctl(a,b,c,d)		(SYSERROR)
#endif
#if TESTBED2
#define		setuid(a)		(SYSERROR)
#endif
#if TESTBED3
#define		setgid(a)		(SYSERROR)
#endif
private void
setup()
{
	int	pathok = 0;

	DBUG_ENTER("setup");

	if (setprv(PRV_SETID) == -1 ||
	    setprv(PRV_IPC) == -1 ||
	    setprv(PRV_ACCESS) == -1)
	{
		int t;

		(void) sprintf(errbuf,"setup: process does not have appropriate privileges, uid is %d", (int) geteuid());
		for (t=1; t<=NO_TESTS; t++)
			tet_delete(t,errbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	super_user = geteuid();
	deflt_group = getegid();
	(void) uids();

	if (pathok == 1)
		setupok = 0;
	else
		setupok = -1;

	DBUG_VOID_RETURN;
}

private void
semtimer(sig)
int sig;
{
	timeout++;
	siglongjmp(jump_buf,sig);
}

/*      tstfailed:      prevents multiple test failure reports  */
private int
tstfailed()
{
        char *buf = FAILFILE;
        struct stat stbuf;

	DBUG_ENTER("tstfailed");

        if ((stat(buf,&stbuf) == SYSERROR) && (errno == ENOENT))
        {
                if (crfile(buf,(mode_t)0666,(int)getuid(),(int)getgid()) == SYSERROR)
                {
                        xx_rpt(DELETION);
                        in_rpt("Could not create temporary file %s, error: %s",
buf,errname(errno));
                        DBUG_RETURN(1);
                }
                DBUG_RETURN(0);      /* test has not yet failed */
        }
        DBUG_RETURN(1);      /* test has already failed */
}

private int
create_msgqueue()
{
	int	err;
	int	id;
	int	testfail = 0;
	int	pathok = 0;

	DBUG_ENTER("create_msgqueue");

        if ((id = msgget(KEY_MSGID, RW_ALL|IPC_CREAT)) == SYSERROR)
        {
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("msgget(KEY_MSGID, 0%o) failed during setup", RW_ALL|IPC_CREAT);
		valrpt(NOERROR,id,NOERROR,err);
        }
	else
		PATH_TRACE;

	PATH_XS_RPT(1);

	DBUG_RETURN(id);
}

/* 
 *	(re) create the semaphore set from arguments 
 */

private int
create_semid(cr_func, key)
void	(*cr_func)();
key_t	key;
{
	int	err, ret;
	int	semid;
	int	testfail = 0;
	int	pathok = 0;

	DBUG_ENTER("create_semid");

        if ((ret=cppair(cr_func, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
        {
                testfail++;
                DBUG_RETURN(SYSERROR);
        }
	else if (ret == PATH_NOK)
	{
                pathok = -1;
	}
	else
	{
		PATH_TRACE;
	}

	/* Get semaphore id for semaphore set created in child process above
	 * into this process's address space 
	 */
        if ((semid = semget(key, 0, 0)) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("Could not obtain semaphore identifier by key during setup");
		valrpt(NOERROR,semid,NOERROR,err);
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(2);

	DBUG_RETURN(semid);
}

private int
setuser_group(usr,grp)
int	usr,grp;
{
	int	err, testfail = 0, pathok = 0;

	DBUG_ENTER("setuser_group");

	if (setgid((gid_t)grp) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
                in_rpt("Could not set group ID to %d in child process during setup",vggid0);
		valrpt(NOERROR,semid_test,NOERROR,err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	if (setuid((uid_t)usr) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
                in_rpt("Could not set user ID to %d in child process during setup",vguid0);
		valrpt(NOERROR,semid_test,NOERROR,err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	if (usr != super_user) {
		if (unsetprv(PRV_SETID) != 0)
		{
			if (!testfail++)
				xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_SETID) failed");
			DBUG_RETURN(SYSERROR);
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_IPC) != 0)
		{
			if (!testfail++)
				xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_IPC) failed");
			DBUG_RETURN(SYSERROR);
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_ACCESS) != 0)
		{
			if (!testfail++)
				xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_ACCESS) failed");
			DBUG_RETURN(SYSERROR);
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_XS_RPT(5);

	DBUG_RETURN(NOERROR);
}

private	void
ch_create_test_semid()
{
	int	pathok = 0, testfail = 0, err;

	DBUG_ENTER("ch_create_test_semid");

	if (setuser_group(vguid0,vggid0) == SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	/*	recreate sem set	*/
        if ((semid_test = semget(KEY_TEST, NSEMS, RW_ALL|IPC_CREAT)) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
                in_rpt("Could not allocate semaphore in child process during setup");
		valrpt(NOERROR,semid_test,NOERROR,err);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);

	DBUG_VOID_RETURN;
}

private	void
ch_create_rw_all_semid()
{
	int	pathok = 0, testfail = 0, err;

	DBUG_ENTER("ch_create_rw_all_semid");

	if (setuser_group(vguid0,vggid0) == SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	/*	create sem set	*/
        if ((semid_rw_all = semget(KEY_RW_ALL, NSEMS, RW_ALL|IPC_CREAT)) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
                in_rpt("Could not allocate semaphore in child process during setup");
		valrpt(NOERROR,semid_rw_all,NOERROR,err);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);

	DBUG_VOID_RETURN;
}



private	void
ch_create_rw_owner_semid()
{
	int	pathok = 0, testfail = 0, err;

	DBUG_ENTER("ch_create_rw_all_semid");

	if (setuser_group(vguid0,vggid0) == SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	/*	create sem set	*/
	if ((semid_rw_owner = semget(KEY_RW_OWNER, NSEMS, RW_U|IPC_CREAT)) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
                in_rpt("Could not allocate semaphore in child process during setup");
		valrpt(NOERROR,semid_rw_owner,NOERROR,err);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);

	DBUG_VOID_RETURN;
}

private	void
ch_create_rw_none_semid()
{
	int	pathok = 0, testfail = 0, err;

	DBUG_ENTER("ch_create_rw_none_semid");

	if (setuser_group(vguid0,vggid0) == SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	/*	create sem set	*/
	if ((semid_rw_none = semget(KEY_RW_NONE, NSEMS, RW_NONE|IPC_CREAT)) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
                in_rpt("Could not allocate semaphore in child process during setup");
		valrpt(NOERROR,semid_rw_none,NOERROR,err);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);

	DBUG_VOID_RETURN;
}

private	void
ch_create_mult_semid()
{
	int	pathok = 0, testfail = 0, err;

	DBUG_ENTER("ch_create_mult_semid");

	if (setuser_group(vguid0,vggid0) == SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	/*	create sem set	*/
	if ((semid_mult = semget(KEY_MULT, NSEMS_MULT, RW_ALL|IPC_CREAT)) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
                in_rpt("Could not allocate semaphore in child process during setup");
		valrpt(NOERROR,semid_mult,NOERROR,err);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);

	DBUG_VOID_RETURN;
}

/* 
 * Check access on semaphore, always done within a child process so exits
 * are returned.
 */
private void
sem_access(suid, sgid, semid, cmd, cmdstr, arg)
int 	suid;
int 	sgid;
int	semid;
int	cmd;
char	*cmdstr;
int	arg;
{
	int	pathok = 0, testfail = 0, fail, err;

	DBUG_ENTER("sem_access");

	if (setuser_group(suid,sgid) == SYSERROR)
	{
		testfail++;
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (arg)
		semun.val = arg;

	SEMCTL(semid, SEMNO, cmd, cmdstr, semun, VERIFY, fail, err, FAILURE);

	if (testfail)
		DBUG_EXIT(NOK);

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);
	DBUG_VOID_RETURN;
}

private	void
test1()
{
	int	pathok = setupok, testfail = 0, ret, fail, err;
	struct 	semid_ds modes;

	DBUG_ENTER("test1");

	if ((semid_rw_owner = create_semid(ch_create_rw_owner_semid,KEY_RW_OWNER)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/* change owner = vguid1 */
	SEMSET(semid_rw_owner, modes, semun, vguid1, vggid1, RW_U, VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t1, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);
clean_out:
	(void)SEM_RM(semid_rw_owner,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t1()
{
	DBUG_ENTER("ch_t1");

	/*	do semctl() where euid == owner */
	sem_access(vguid1, vggid1, semid_rw_owner, GETNCNT, "GETNCNT", 0);

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	int	pathok = setupok, testfail = 0, ret, fail, err;
	struct 	semid_ds modes;

	DBUG_ENTER("test2");

	if ((semid_rw_owner = create_semid(ch_create_rw_owner_semid,KEY_RW_OWNER)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	SEMSET(semid_rw_owner, modes, semun, vguid1, vggid1, RW_U, VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t2, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);
clean_out:
	(void)SEM_RM(semid_rw_owner,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t2()
{
	DBUG_ENTER("ch_t2");

	/*	do semctl() where euid == creator */
	sem_access(vguid0, vggid2, semid_rw_owner, GETNCNT, "GETNCNT", 0);

	DBUG_VOID_RETURN;
}

private	void
test3()
{
	int	pathok = setupok, testfail = 0, ret, fail, err;
	struct 	semid_ds modes;

	DBUG_ENTER("test3");

	if ((semid_test = create_semid(ch_create_test_semid,KEY_TEST)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/* change group = vggid1 */
	SEMSET(semid_test, modes, semun, vguid0, vggid1, (RW_U|RW_G), VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t3, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);
clean_out:
	(void)SEM_RM(semid_test,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t3()
{
	DBUG_ENTER("ch_t3");

	/*	do semctl() where egid == group ID of owner */
	sem_access(vguid2, vggid1, semid_test, GETNCNT, "GETNCNT", 0);

	DBUG_VOID_RETURN;
}

private	void
test4()
{
	int	pathok = setupok, testfail = 0, ret, fail, err;
	struct 	semid_ds modes;

	DBUG_ENTER("test4");

	if ((semid_test = create_semid(ch_create_test_semid,KEY_TEST)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	SEMSET(semid_test, modes, semun, vguid0, vggid1, (RW_U|RW_G), VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t4, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);
clean_out:
	(void)SEM_RM(semid_test,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t4()
{
	DBUG_ENTER("ch_t4");

	/*	do semctl() where egid == group ID of creator */
	sem_access(vguid2, vggid0, semid_test, GETNCNT, "GETNCNT", 0);

	DBUG_VOID_RETURN;
}

private	void
test5()
{
	int	testfail = 0, ret;
	int	pathok = setupok;

	DBUG_ENTER("test5");

	if ((semid_rw_all = create_semid(ch_create_rw_all_semid,KEY_RW_ALL)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	if ((ret=cppair(ch_t5, NULLFN, WAITTIME, NOK|PATH_NOK)) == NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);
clean_out:
	(void)SEM_RM(semid_rw_all,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t5()
{
	DBUG_ENTER("ch_t5");

	/*	do semctl() where euid == other and egid == other */
	sem_access(vguid2, vggid2, semid_rw_all, GETNCNT, "GETNCNT", 0);

	DBUG_VOID_RETURN;
}

private	void
test6()
{
	int	pathok = setupok, testfail = 0, ret;

	DBUG_ENTER("test6");

	if ((semid_rw_owner = create_semid(ch_create_rw_owner_semid,KEY_RW_OWNER)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	if ((ret=cppair(ch_t6, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);
clean_out:
	(void)SEM_RM(semid_rw_owner,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t6()
{
	DBUG_ENTER("ch_t6");

	/*	do semctl() where euid and egid == 0 (root) 
	 */
	sem_access((int)super_user, (int)deflt_group, semid_rw_owner, GETNCNT, "GETNCNT", SEMVAL);

	DBUG_VOID_RETURN;
}

private	void
test7()
{
	int	pathok = setupok, testfail = 0, ret, fail, err;
	struct 	semid_ds modes;

	DBUG_ENTER("test7");

	if ((semid_test = create_semid(ch_create_test_semid,KEY_TEST)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/* change owner = vguid1 */
	SEMSET(semid_test, modes, semun, vguid1, vggid0, RW_U, VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t7, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);
clean_out:
	(void)SEM_RM(semid_test,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t7()
{
	DBUG_ENTER("ch_t7");

	/*	do semctl() where euid == owner */
	sem_access(vguid1, vggid2, semid_test, SETVAL, "SETVAL", SEMVAL);

	DBUG_VOID_RETURN;
}

private	void
test8()
{
	int	pathok = setupok, testfail = 0, ret, fail, err;
	struct 	semid_ds modes;

	DBUG_ENTER("test8");

	if ((semid_test = create_semid(ch_create_test_semid,KEY_TEST)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	SEMSET(semid_test, modes, semun, vguid1, vggid1, RW_U, VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t8, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);
clean_out:
	(void)SEM_RM(semid_test,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t8()
{
	DBUG_ENTER("ch_t8");

	/*	do semctl() where euid == creator */
	sem_access(vguid0, vggid2, semid_test, SETVAL, "SETVAL", SEMVAL);

	DBUG_VOID_RETURN;
}

private	void
test9()
{
	int	pathok = setupok, testfail = 0, ret, fail, err;
	struct 	semid_ds modes;

	DBUG_ENTER("test9");

	if ((semid_test = create_semid(ch_create_test_semid,KEY_TEST)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/* change group = vggid1 */
	SEMSET(semid_test, modes, semun, vguid0, vggid1, (RW_U|RW_G), VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t9, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);
clean_out:
	(void)SEM_RM(semid_test,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t9()
{
	DBUG_ENTER("ch_t9");

	/*	do semctl() where egid == group ID of owner */
	sem_access(vguid2, vggid1, semid_test, SETVAL, "SETVAL", SEMVAL);

	DBUG_VOID_RETURN;
}

private	void
test10()
{
	int	pathok = setupok, testfail = 0, ret, fail, err;
	struct 	semid_ds modes;

	DBUG_ENTER("test10");

	if ((semid_test = create_semid(ch_create_test_semid,KEY_TEST)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/* change group = vggid1 */
	SEMSET(semid_test, modes, semun, vguid0, vggid1, (RW_U|RW_G), VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t10, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);
clean_out:
	(void)SEM_RM(semid_test,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t10()
{
	DBUG_ENTER("ch_t10");

	/*	do semctl() where euid == group ID of creator */
	sem_access(vguid2, vggid0, semid_test, SETVAL, "SETVAL", SEMVAL);

	DBUG_VOID_RETURN;
}

private	void
test11()
{
	int	pathok = setupok, testfail = 0, ret;

	DBUG_ENTER("test11");

	if ((semid_rw_all = create_semid(ch_create_test_semid,KEY_TEST)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	if ((ret=cppair(ch_t11, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);
clean_out:
	(void)SEM_RM(semid_rw_all,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t11()
{
	DBUG_ENTER("ch_t11");

	/*	do semctl() where euid == other and egid == other */
	sem_access(vguid2, vggid2, semid_rw_all, SETVAL, "SETVAL", SEMVAL);

	DBUG_VOID_RETURN;
}

private	void
test12()
{
	int	pathok = setupok, testfail = 0, fail, err, ret;
	unsigned short	val[NSEMS];

	DBUG_ENTER("test12");

	if ((semid_rw_owner = create_semid(ch_create_rw_owner_semid,KEY_RW_OWNER)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	if ((ret=cppair(ch_t12, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret&PATH_NOK)
	{
		pathok=  -1;
	}
	else
		PATH_TRACE;

	/*	Use GETALL to verify semadj equals 0.  If semval
		changed as a result of child exit(), semadj != 0.
		GETVAL is more appropriate, but GETVAL is tested in test 13.
	*/
	semun.array = val;
	SEMCTL(semid_rw_owner, SEMNO, GETALL, "GETALL", semun, VERIFY, fail, err, DELETION);
	if (testfail)
		goto clean_out;

	if (val[SEMNO] != SEMVAL)
	{
               	if (testfail++ == 0 && (tstfailed() == 0))
       			xx_rpt(FAILURE);
                in_rpt("after semctl(...,SETVAL,...) semadj value != 0, semadj = %d",val[SEMNO]-SEMVAL);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);
clean_out:
	(void)SEM_RM(semid_rw_owner,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t12()
{
	int	pathok = 0, testfail = 0, fail, err;
	unsigned short	val[NSEMS];

	DBUG_ENTER("ch_t12");

	semun.val = SEMVAL;
	SEMCTL(semid_rw_owner, SEMNO, SETVAL, "SETVAL", semun, VERIFY, fail, err, DELETION);
	if (testfail)
		DBUG_EXIT(NOK);

	if (fail != 0)
	{
               	if (testfail++ == 0 && (tstfailed() == 0))
       			xx_rpt(FAILURE);
                in_rpt("semctl(...,SETVAL,...) did not return 0, return value = %d",fail);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	/*	Use GETALL rather than GETVAL to verify semval was set.  
		GETVAL is tested in test 13.
	*/
	semun.array = val;
	SEMCTL(semid_rw_owner, SEMNO, GETALL, "GETALL", semun, VERIFY, fail, err, DELETION);
	if (testfail)
		DBUG_EXIT(NOK);

	if (val[SEMNO] != SEMVAL)
	{
               	if (testfail++ == 0 && (tstfailed() == 0))
       			xx_rpt(FAILURE);
                in_rpt("semctl(...,GETALL,...) expected semval = %d, actual semval = %d, SEMNO = %d, NSEMS = %d",SEMVAL,val[SEMNO],SEMNO,NSEMS);
	}
	else
		PATH_TRACE;

	if (testfail)
		DBUG_EXIT(NOK);

	if (pathok != 4)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);
}

private	void
test13()
{
	int	pathok = setupok, ret, testfail = 0;

	DBUG_ENTER("test13");

	if ((semid_rw_owner = create_semid(ch_create_rw_owner_semid,KEY_RW_OWNER)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	if ((ret=cppair(ch_t13, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);
clean_out:
	(void)SEM_RM(semid_rw_owner,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}
private	void
ch_t13()
{
	int	pathok = setupok, testfail = 0, fail, err;
	unsigned short	sval[NSEMS];

	DBUG_ENTER("ch_t13");

	(void)memset((void *)sval, 0, sizeof(unsigned short)*NSEMS);
	sval[SEMNO] = SEMVAL;

	/*	Use SETALL rather than SETVAL to set semval.
		SETVAL is tested in test 12.
	*/
	semun.array = sval;
	SEMCTL(semid_rw_owner, SEMNO, SETALL, "SETALL", semun, VERIFY, fail, err, DELETION);
	if (testfail)
		DBUG_EXIT(NOK);

	SEMCTL(semid_rw_owner, SEMNO, GETVAL, "GETVAL", semun /* ignored */, VERIFY, fail, err, FAILURE);
	if (testfail)
		DBUG_EXIT(NOK);

	if (fail != SEMVAL)
	{
               	if (testfail++ == 0 && (tstfailed() == 0))
       			xx_rpt(FAILURE);
                in_rpt("semctl(...,GETVAL,...) did not return the expected semval, expected semval = %d, actual semval = %d",SEMVAL,fail); 
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok != 3)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private	void
test14()
{
	int	pathok = setupok, testfail = 0, fail, ret, i, err;
	
	DBUG_ENTER("test14");

	if ((semid_mult = create_semid(ch_create_mult_semid,KEY_MULT)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/*	child sets values via SETALL	
	*/
	if ((ret=cppair(ch_t14, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	/*	now verify semval hasn't changed due to child exiting.
		If semval == SEMVAL, then semadj == 0.
	*/
	for (i=0; i < NSEMS_MULT; i++)
	{
		SEMCTL(semid_mult, i, GETVAL, "GETVAL", semun, VERIFY, fail, err, DELETION);
		if (testfail)
			goto clean_out;

		if (fail != SEMVAL)
		{
               		if (testfail++ == 0 && (tstfailed() == 0))
                       		xx_rpt(FAILURE);
                        in_rpt("after semctl(...,SETALL,...) semadj value != 0, semadj = %d",fail-SEMVAL);
		}
		else 
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2*NSEMS_MULT+2);
clean_out:
	(void) SEM_RM(semid_mult,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t14()
{
	int	pathok = 0, testfail = 0, fail, err, i;
	unsigned short	sval[NSEMS_MULT];

	DBUG_ENTER("ch_t14");

	for (i=0; i < NSEMS_MULT; i++)
		sval[i] = SEMVAL;

	semun.array = sval;
	SEMCTL(semid_mult, SEMNO, SETALL, "SETALL", semun, VERIFY, fail, err, FAILURE);
	if (testfail)
		DBUG_EXIT(NOK);

	if (fail != 0)
	{
               	if (testfail++ == 0 && (tstfailed() == 0))
                        xx_rpt(FAILURE);
                in_rpt("semctl(...,SETALL,...) did not return 0, return value = %d",fail);
	}
	else
		PATH_TRACE;

	/*	Use GETVAL, rather than GETALL, to verify value was set.  
		GETALL is tested in test 15.
		Use NOVERIFY because if the semctl() fails, it's a DELETION, 
		not a FAILURE.
	*/
	for (i=0; i < NSEMS_MULT; i++)
	{
		SEMCTL(semid_mult, i, GETVAL, "GETVAL", semun, VERIFY, fail, err, DELETION);
		if (testfail)
			DBUG_EXIT(NOK);

		if (fail != SEMVAL)
		{
               		if (testfail++ == 0 && (tstfailed() == 0))
                       		xx_rpt(FAILURE);
                	in_rpt("semctl(...,SETALL,...) did not set semval, expected semval = %d, actual semval = %d",SEMVAL,fail);
		}
		else 
			PATH_TRACE;
	}

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok != 2*NSEMS_MULT+2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private	void
test15()
{
	int	pathok = setupok, testfail = 0, ret;

	DBUG_ENTER("test15");
	if ((semid_mult = create_semid(ch_create_mult_semid,KEY_MULT)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;


	if ((ret=cppair(ch_t15, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);
clean_out:
	(void) SEM_RM(semid_mult,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t15()
{
	int	pathok = 0, testfail = 0, fail, err, i;
	unsigned short 	val[NSEMS_MULT];

	DBUG_ENTER("ch_t15");

	/*	Use SETVAL, rather than SETALL, to set semval.  
		SETALL is tested in test 14.
		Use NOVERIFY because if the semctl() fails, it's a DELETION, 
		not a FAILURE.
	*/
	semun.val = SEMVAL;
	for (i=0; i < NSEMS_MULT; i++)
	{
		SEMCTL(semid_mult, i, SETVAL, "SETVAL", semun, VERIFY, fail, err, DELETION);
		if (testfail)
			DBUG_EXIT(NOK);
	}

	semun.array = val;
	SEMCTL(semid_mult, SEMNO, GETALL, "GETALL", semun, VERIFY, fail, err, FAILURE);
	if (testfail)
		DBUG_EXIT(NOK);

	for (i=0; i < NSEMS_MULT; i++)
	{
		if (val[i] != SEMVAL)
		{
               		if (testfail++ == 0 && (tstfailed() == 0))
                       		xx_rpt(FAILURE);
                       	in_rpt("semctl(...,GETALL,...) did not return expected semval, expected semval = %d, actual semval = %d",SEMVAL,val[i]);
		}
		else 
			PATH_TRACE;
	}
	
	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok != 2*NSEMS_MULT+1)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private	void
test16()
{
	int	pathok = setupok, testfail = 0, ret;

	DBUG_ENTER("test16");

	/* Create semaphore in parent process so it is different
	 * from "last" semaphore operation in child process.
	 */
	if ((semid_rw_all = create_semid(ch_create_rw_all_semid,KEY_RW_ALL)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;


	if ((ret=cppair(ch_t16, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);
clean_out:
	(void) SEM_RM(semid_rw_all,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t16()
{
	int	pathok = 0, testfail = 0, fail, err;
	pid_t	pid;

	DBUG_ENTER("ch_t16");

	/* Do "last" semaphore operation here */
	semun.val = SEMVAL;
	SEMCTL(semid_rw_all, SEMNO, SETVAL, "SETVAL", semun, VERIFY, fail, err, DELETION);
	if (testfail)
		DBUG_EXIT(NOK);

	pid = getpid();

	SEMCTL(semid_rw_all, SEMNO, GETPID, "GETPID", semun, VERIFY, fail, err, FAILURE);

	if (fail != (int) pid)
	{
               	if (testfail++ == 0 && (tstfailed() == 0))
                	xx_rpt(FAILURE);
                in_rpt("semctl(...,GETPID,...) did not return expected pid, expected pid = %d, returned pid = %d",(int) pid,fail);
	}
	else 
		PATH_TRACE;

	if (testfail)
		DBUG_EXIT(NOK);

	if (pathok != 3)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private	void
test17()
{
	DBUG_ENTER("test17");

	(void)cppair(ch_t17, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t17()
{
	DBUG_ENTER("ch_t17");
	sem_get_cnt(1);
	DBUG_VOID_RETURN;
}

private	void
sem_get_cnt(ncnt)
int	ncnt;
{
	int	pathok = setupok, testfail = 0, val, fail, err, wstat;
	pid_t	childproc;
	struct  sembuf	ops;
	struct  mbuf msg;
	
	DBUG_ENTER("sem_get_cnt");

	if ((msgid = create_msgqueue()) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	if ((semid_rw_all = create_semid(ch_create_rw_all_semid,KEY_RW_ALL)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	semun.val = SEMVAL;
	SEMCTL(semid_rw_all, SEMNO, SETVAL, "SETVAL", semun, VERIFY, fail, err, DELETION);
	if (testfail)
		goto clean_out;

	/*	get initial semncnt */
	SEMCTL(semid_rw_all, SEMNO, ncnt ? GETNCNT : GETZCNT, ncnt ? "GETNCNT" : "GETZCNT", semun, VERIFY, val, err, FAILURE);
	if (testfail)
		goto clean_out;

	/*	fork another.
		parent:
			attempt to obtain sem via semop(), 
			suspending process, resuming when semval adjusted
		child:
			spin doing GETNCNT/GETZCNT waiting for semncnt/semzcnt
			to increment when parent suspends, adjust semval, 
			then exit
	*/

	/* Entering multiprocess execution, where synchronization for error
	 * reporting is necessary.
	 */
	switch(childproc=fork()) {
		case 0:		/* child */
			/* change context to distinguish output from parent's */
			tet_setcontext();
			sem_getncnt(ncnt,val);
			/* never return to here */
			break;

		case SYSERROR:
			err = errno;
               		if (!testfail++ && (tstfailed() == 0))
                       		xx_rpt(DELETION);
                       	in_rpt("fork() unexpectedly failed");
			valrpt(NOERROR,(int)childproc,NOERROR,err);
			goto clean_out;

		default:
			tet_setblock();
			PATH_TRACE;
			break;	/* parent continues */
	}

    SETTIMER	/* precaution against infinite wait */

	ops.sem_num = SEMNO;
	ops.sem_op = ncnt ? -(SEMVAL+1) : 0;
	ops.sem_flg = 0;

	/*	suspend initially, resume when semval == SEMVAL+1	*/
	if (semop(semid_rw_all, &ops, 1) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0 && (tstfailed() == 0))
			xx_rpt(DELETION);
		in_rpt("semop() where sem_op=%d failed unexpectedly, error: %s",ops.sem_op,errname(err));
	}
	else 
		PATH_TRACE;

    CLEARTIMER

	/*	timeout indicates didn't resume */
	if (TIMEDOUT)
	{
		if (testfail++ == 0 && (tstfailed() == 0))
               		xx_rpt(DELETION);
		in_rpt("semop() where sem_op=%d unexpectedly failed to resume after suspending",ops.sem_op);
	}
	else 
		PATH_TRACE;

	/* End multiprocess execution here (in parent) */
	tet_setblock();
	/*	wait for child if hasn't already exited	*/
	if (waitpid(childproc,&wstat,0) == SYSERROR)
	{
		if (testfail++ == 0 && (tstfailed() == 0))
               		xx_rpt(DELETION);
		in_rpt("waitpid() unexpectedly failed, error: %s",errname(err));
	}
	else 
		PATH_TRACE;

	MSGRCV(msg,err);

	if ((strncmp(msg.mtext, MSGOK, MSGSZ) == 0) && 
			(WEXITSTATUS(wstat) == OK) && (testfail == 0))
		PATH_XS_RPT(9);

clean_out:
	(void) SEM_RM(semid_rw_all,semun);
        (void) msgctl(msgid, IPC_RMID, (struct msqid_ds *)0);
	FAILRM;

	DBUG_VOID_RETURN;
}

private void
sem_getncnt(ncnt,oval)
int	ncnt;
int	oval;
{
	int	pathok=0, testfail = 0, val, err;
	struct	sembuf ops;
	struct  mbuf msg;

	DBUG_ENTER("sem_getncnt");

    SETTIMER	/* precaution against infinite wait */

	val = oval;

	/*	spin waiting for semncnt to increment, indicating
		parent has suspended	
	*/
	while (val == oval)
		SEMCTL(semid_rw_all, SEMNO, ncnt ? GETNCNT : GETZCNT, ncnt ? "GETNCNT" : "GETZCNT", semun, VERIFY, val, err, FAILURE);

    CLEARTIMER

	ops.sem_num = SEMNO;
	ops.sem_op = ncnt ? 1 : -SEMVAL;
	ops.sem_flg = 0;

	/*	increment semval by 1, resuming parent	*/
	if (semop(semid_rw_all, &ops, 1) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0 && (tstfailed() == 0))
                {
			xx_rpt(DELETION);
                }
		in_rpt("semop() where sem_op=%d failed unexpectedly, error: %s",ops.sem_op,errname(err));
	}

	if (val == oval)
	{
		char str[8];
		if (ncnt)
			(void)strcpy(str,"semncnt");
		else
			(void)strcpy(str,"semzcnt");
		if (testfail++ == 0 && (tstfailed() == 0))
                {
               		xx_rpt(FAILURE);
                }
		in_rpt("semctl(...,%s,...) did not return the expected %s, expected %s = %d, actual %s = %d", ncnt?"GETNCNT":"GETZCNT",str,str,oval+1,str,val);
	}

	/*	not guaranteed any synchronization between processes, 
		use message to indicate success or failure, exit() value
		indicates success/failure of msgsnd().
	*/
	if (val==oval || testfail != 0)
		(void)strncpy(msg.mtext,MSGFAIL,MSGSZ+1);
	else
		(void)strncpy(msg.mtext,MSGOK,MSGSZ+1);

	MSGSND(msg,err);

	DBUG_EXIT(err != 0 ? NOK : OK);

	/* End multiprocess execution here (in child) */
	DBUG_VOID_RETURN;
}

private	void
test18()
{
	DBUG_ENTER("test18");

	(void)cppair(ch_t18, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

/*	test18: very similar to 17, except using GETZCNT and semzcnt
*/
private	void
ch_t18()
{
	DBUG_ENTER("ch_t18");

	sem_get_cnt(0);

	DBUG_VOID_RETURN;
}

private	void
test19()
{
	int	pathok = setupok, testfail = 0, ret;

	DBUG_ENTER("test19");

	if ((semid_rw_all = create_semid(ch_create_rw_all_semid,KEY_RW_ALL)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	if ((ret=cppair(ch_t19, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);
clean_out:
	(void) SEM_RM(semid_rw_all,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t19()
{
	int	pathok = 0, testfail = 0, fail, err;
	struct  semid_ds sstat;

	DBUG_ENTER("ch_t19");

	if (setuser_group(vguid2, vggid2) == SYSERROR)
	{
		testfail++;
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	semun.buf = &sstat;
	SEMCTL(semid_rw_all, SEMNO, IPC_STAT, "IPC_STAT", semun, VERIFY, fail, err, FAILURE);
	if (testfail)
		DBUG_EXIT(NOK);

	if (sstat.sem_nsems != NSEMS ||
	    sstat.sem_perm.uid != vguid0 ||
	    sstat.sem_perm.gid != vggid0 ||
	    (sstat.sem_perm.mode & 0777) !=  RW_ALL)
	{
		if (testfail++ == 0 && (tstfailed() == 0))
               		xx_rpt(FAILURE);
                in_rpt("semctl(..., IPC_STAT,...) did not return the expected data:");
		in_rpt("expected nsems = %d, actual nsems = %d, expected uid = %d, actual uid = %ld, expected gid = %d, actual gid = %ld, expected mode = %0lo, actual mode = %0lo",
			NSEMS, sstat.sem_nsems,
			vguid0, (long)sstat.sem_perm.uid,
			vggid0, (long)sstat.sem_perm.gid,
			(unsigned long)RW_ALL,
			(unsigned long)sstat.sem_perm.mode);
	}
	else
		PATH_TRACE;

	if (testfail)
		DBUG_EXIT(NOK);

	if (pathok != 3)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private	void
test20()
{
	int	pathok = setupok, testfail = 0, ret, err, fail;
	struct 	semid_ds modes;

	DBUG_ENTER("test20");

	if ((semid_test = create_semid(ch_create_test_semid,KEY_TEST)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/* Set semaphore to vguid1/vggid1, then set process to vguid1/vggid1 
	 * and try IPC_RMID as owner but not creator 
	 */
	SEMSET(semid_test, modes, semun, vguid1, vggid1, RW_U, VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t20, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
clean_out:
	(void) SEM_RM(semid_test,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t20()
{
	DBUG_ENTER("ch_t20");
	
	sem_rmid_user(vguid1,vggid1);

	DBUG_VOID_RETURN;
}

private	void
sem_rmid_user(suid,sgid)
int	suid,sgid;
{
	int	pathok = 0, testfail = 0, fail, err;

	DBUG_ENTER("sem_rmid_user");

	if (setuser_group(suid, sgid) == SYSERROR)
	{
		testfail++;
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	SEMCTL(semid_test, 0, IPC_RMID, "IPC_RMID", semun, VERIFY, fail, err, FAILURE);

	if (fail != 0)
	{
		if (testfail++ == 0 && (tstfailed() == 0))
			xx_rpt(FAILURE);
		in_rpt("semctl(%d, 0, IPC_RMID, ...) did not return 0, returned %d",
			semid_test, fail); 
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	/* 	now try to access it, expect error */
	SEMCTL(semid_test, SEMNO, GETVAL, "GETVAL", semun, NOVERIFY, fail, err, FAILURE);

	testfail = 0;

	if (fail != SYSERROR || err != EINVAL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("semctl(%d, 0, IPC_RMID, ...) did not remove semaphore set",semid_test); 
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok != 4)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private	void
test21()
{
	int	pathok = setupok, testfail = 0, ret, err, fail;
	struct 	semid_ds modes;

	DBUG_ENTER("test21");

	if ((semid_test = create_semid(ch_create_test_semid,KEY_TEST)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/* Set semaphore to vguid1/vggid1, then set process to vguid0/vggid0 
	 * and try IPC_RMID as creator but not owner. 
	 */
	SEMSET(semid_test, modes, semun, vguid1, vggid1, RW_U, VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t21, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
clean_out:
	(void) SEM_RM(semid_test,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t21()
{
	DBUG_ENTER("ch_t21");

	/* try IPC_RMID as creator */
	sem_rmid_user(vguid0,vggid0);

	DBUG_VOID_RETURN;
}

private	void
test22()
{
	int	pathok = setupok, testfail = 0, ret, err, fail;
	struct 	semid_ds modes;

	DBUG_ENTER("test22");

	if ((semid_test = create_semid(ch_create_test_semid,KEY_TEST)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/* Set semaphore to vguid1/vggid1, then set process to 
	 * super_user/deflt_group and try IPC_RMID "with appropriate privileges"
	 * but not creator or owner.
	 */
	SEMSET(semid_test, modes, semun, vguid1, vggid1, RW_U, VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t22, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
clean_out:
	(void) SEM_RM(semid_test,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t22()
{
	DBUG_ENTER("ch_t22");

	/* do IPC_RMID as super-user (0) */
	sem_rmid_user((int)super_user, (int)deflt_group);

	DBUG_VOID_RETURN;
}

private	void
test23()
{
	int	pathok = setupok, testfail = 0, fail, err, ret;
	struct	semid_ds omodes;

	DBUG_ENTER("test23");

	if ((semid_rw_owner = create_semid(ch_create_rw_owner_semid,KEY_RW_OWNER)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/* Set semaphore to vguid1/vggid1, then set process to vguid1/vggid1 
	 * and try IPC_SET as owner but not creator. 
	 */
	SEMSET(semid_rw_owner, omodes, semun, vguid1, vggid1, RW_U, VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t23, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
clean_out:
	(void) SEM_RM(semid_rw_owner,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private void
ch_t23()
{
	DBUG_ENTER("ch_t23");
	
	sem_set_user(semid_rw_owner, vguid1, vggid1, vguid2, vggid2, (mode_t)RW_ALL);

	DBUG_VOID_RETURN;
}

private	void
sem_set_user(semid, suid, sgid, uid, gid, mod)
int	semid;
int 	suid;
int 	uid;
int	gid;
mode_t 	mod;
{
	int	pathok = setupok, testfail = 0, fail, err;
	struct	semid_ds omodes, modes;

	DBUG_ENTER("sem_set_user");

	if (setuser_group(suid, sgid) == SYSERROR)
	{
		testfail++;
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	/* do IPC_SET */
	SEMSET(semid, omodes, semun, uid, gid, mod, VERIFY, FAILURE);
	if (testfail)
		DBUG_EXIT(NOK);

	semun.buf = &modes;
	SEMCTL(semid, SEMNO, IPC_STAT, "IPC_STAT", semun, VERIFY, fail, err, DELETION);
	if (testfail)
		DBUG_EXIT(NOK);

	if (modes.sem_perm.uid != omodes.sem_perm.uid ||
	    modes.sem_perm.gid != omodes.sem_perm.gid ||
	    (modes.sem_perm.mode & 0777) != (omodes.sem_perm.mode & 0777))
	{
                if (testfail++ == 0 && (tstfailed() == 0))
                        xx_rpt(FAILURE);
		in_rpt("semctl(..., IPC_SET, ...) did not set sem_perm.uid, sem_perm.gid, and/or sem_perm.mode:");
		in_rpt("expected (uid,gid,modes)=(%ld,%ld,0%lo), actual (uid,gid,mode)=(%ld,%ld,0%lo)",
			(long)omodes.sem_perm.uid, (long)omodes.sem_perm.gid,
			(unsigned long)omodes.sem_perm.mode&0777,
			(long)modes.sem_perm.uid, (long)modes.sem_perm.gid,
			(unsigned long)modes.sem_perm.mode&0777);
        }
        else
		PATH_TRACE;

	if (testfail)
		DBUG_EXIT(NOK);

	if (pathok != 4)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private	void
test24()
{
	int	pathok = setupok, testfail = 0, fail, err, ret;
	struct	semid_ds omodes;

	DBUG_ENTER("test24");

	if ((semid_rw_owner = create_semid(ch_create_rw_owner_semid,KEY_RW_OWNER)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/* Set semaphore to vguid1/vggid1, then set process to vguid0/vggid0 
	 * and try IPC_SET as creator but not owner. 
	 */
	SEMSET(semid_rw_owner, omodes, semun, vguid1, vggid1, RW_U, VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t24, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
clean_out:
	(void) SEM_RM(semid_rw_owner,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t24()
{

	DBUG_ENTER("ch_t24");
	/* try IPC_SET as creator (vguid0)	*/
	sem_set_user(semid_rw_owner, vguid0, vggid0, vguid2, vggid2, (mode_t)RW_ALL);

	DBUG_VOID_RETURN;
}

private	void
test25()
{
	int	pathok = setupok, testfail = 0, fail, err, ret;
	struct	semid_ds omodes;

	DBUG_ENTER("test25");

	if ((semid_rw_owner = create_semid(ch_create_rw_owner_semid,KEY_RW_OWNER)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/* Set semaphore to vguid1/vggid1, then set process to 
	 * super_user/deflt_group and try IPC_SET "with appropriate privileges"
	 * but not as owner or creator. 
	 */
	SEMSET(semid_rw_owner, omodes, semun, vguid1, vggid1, RW_U, VERIFY, DELETION);
	if (testfail)
		goto clean_out;

	if ((ret=cppair(ch_t25, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
clean_out:
	(void) SEM_RM(semid_rw_owner,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t25()
{
	DBUG_ENTER("ch_t25");

	/* try IPC_SET as super-user (0)	*/
	sem_set_user(semid_rw_owner, (int)super_user, (int)deflt_group, vguid2, vggid2, (mode_t)RW_ALL);

	DBUG_VOID_RETURN;
}

private	void
test26()
{
	int	pathok = setupok, testfail = 0, sid, fail, err;

	DBUG_ENTER("test26");

	sid = -1;
	/*	just to be sure -1 isn't valid 	*/
	while (sid == semid_rw_all || sid == semid_rw_owner ||
	       sid == semid_rw_none || sid == semid_mult || sid == semid_test)
		sid++;
	
	SEMCTL(sid, SEMNO, GETNCNT, "GETNCNT", semun, NOVERIFY, fail, err, FAILURE);

	testfail = 0;
	if (fail != SYSERROR || err != EINVAL)
	{
		/* NOVERIFY set above so always report error here */
		testfail++;
                xx_rpt(FAILURE);
                in_rpt("semctl() where no semaphores exist for id %d unexpectedly return", sid);
		valrpt(SYSERROR, fail, EINVAL, err);
	}
	else
		PATH_TRACE;

	FAILRM;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private	void
test27()
{

	int	pathok = setupok, testfail = 0, fail, err;
	int	semnumber;

	DBUG_ENTER("test27");

	if ((semid_rw_all = create_semid(ch_create_rw_all_semid,KEY_RW_ALL)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	semnumber  = -1; /* an invalid semaphore number */
	SEMCTL(semid_rw_all, semnumber, GETNCNT, "GETNCNT", semun, NOVERIFY, fail, err, FAILURE);

	testfail = 0;
	if (fail != SYSERROR || err != EINVAL)
	{
		testfail++;
		xx_rpt(FAILURE);
                in_rpt("semctl() where sem_num == -1 unexpectedly return");
		valrpt(SYSERROR, fail, EINVAL, err);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);
clean_out:
	(void) SEM_RM(semid_rw_all,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
test28()
{
	int	pathok = setupok, testfail = 0, fail, err;
	int	semnumber;

	DBUG_ENTER("test28");

	if ((semid_rw_all = create_semid(ch_create_rw_all_semid,KEY_RW_ALL)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	semnumber = NSEMS+1; /* more than the value in the semaphore */
	SEMCTL(semid_rw_all, semnumber, GETNCNT, "GETNCNT", semun, NOVERIFY, fail, err, FAILURE);

	testfail = 0;
	if (fail != SYSERROR || err != EINVAL)
	{
		testfail++;
		xx_rpt(FAILURE);
                in_rpt("semctl() unexpected return, when sem_num argument is greater than actual");
		in_rpt("sem_num = %d, actual nsems = %d",NSEMS+1,NSEMS);
		valrpt(SYSERROR,fail,EINVAL,err); 
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);
clean_out:
	(void) SEM_RM(semid_rw_all,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
test29()
{
	int	pathok = setupok, testfail = 0, cmd, fail, err, ok;

	DBUG_ENTER("test29");

	if ((semid_rw_all = create_semid(ch_create_rw_all_semid,KEY_RW_ALL)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	/* verify invalid cmd */
	cmd = -1;
	do
	{
		switch (cmd) {
			case GETNCNT:
			case GETPID:
			case GETVAL:
			case GETALL:
			case GETZCNT:
			case SETVAL:
			case SETALL:
			case IPC_RMID:
			case IPC_SET:
			case IPC_STAT:	cmd++;
					ok = 0;
					break;

			default:	ok = 1;
					PATH_TRACE;
					break;
		}
	} while (ok==0);

	SEMCTL(semid_rw_all, SEMNO, cmd, itoa(cmd), semun, NOVERIFY, fail, err, FAILURE);

	testfail = 0;
	if (fail != SYSERROR || err != EINVAL)
	{
		testfail++;
		xx_rpt(FAILURE);
                in_rpt("semctl() where cmd is invalid unexpected return");
		valrpt(SYSERROR, fail, EINVAL, err);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
clean_out:
	(void) SEM_RM(semid_rw_all,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
test30()
{
	int	pathok = setupok, testfail = 0, ret;

	DBUG_ENTER("test30");

	if ((semid_rw_none = create_semid(ch_create_rw_none_semid,KEY_RW_NONE)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	if ((ret=cppair(ch_t30, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);
clean_out:
	(void) SEM_RM(semid_rw_none,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t30()
{
	int	pathok = 0, testfail = 0, fail, err;

	DBUG_ENTER("ch_t30");

	/* owner and creator are vguid0/vggid0 */
	if (setuser_group(vguid1, vggid1) == SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	semun.val = SEMVAL;
	SEMCTL(semid_rw_none, SEMNO, SETVAL, "SETVAL", semun, NOVERIFY, fail, err, FAILURE);

	testfail = 0;
	if (fail != SYSERROR || err != EACCES)
	{
		testfail++;
		xx_rpt(FAILURE);
                in_rpt("semctl() where access is denied unexpected return");
		valrpt(SYSERROR, fail, EACCES, err);
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private	void
test31()
{
	int	pathok = setupok, testfail = 0, ret;

	DBUG_ENTER("test31");

	if ((semid_rw_owner = create_semid(ch_create_rw_owner_semid,KEY_RW_OWNER)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	if ((ret=cppair(ch_t31, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);
clean_out:
	(void) SEM_RM(semid_rw_owner,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
ch_t31()
{
	int	pathok = 0, testfail = 0, fail, err;
	struct	semid_ds modes;

	DBUG_ENTER("ch_t31");

	/* owner and creator are vguid0/vggid0 */
	if (setuser_group(vguid1, vggid1) == SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	SEMSET(semid_rw_owner, modes, semun, vguid2, vggid1, RW_U, NOVERIFY, FAILURE);

	testfail = 0;
	if (fail != SYSERROR || err != EPERM)
	{
		testfail++;
		xx_rpt(FAILURE);
                in_rpt("semctl(IPC_SET) unexpected return when calling process does not match owner or creator user id");
		in_rpt("modes = 0%o, owner/creator uid = %d, calling process uid = %ld",
			RW_U, vguid0, (long)geteuid());
		valrpt(SYSERROR, fail, EPERM, err);
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private	void
test32()
{
	int	pathok = setupok, testfail = 0, ret;

	DBUG_ENTER("test32");

	if ((semid_rw_owner = create_semid(ch_create_rw_owner_semid,KEY_RW_OWNER)) == SYSERROR)
	{
		testfail++;
		goto clean_out;
	}
	else
		PATH_TRACE;

	if ((ret=cppair(ch_t32, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret==PATH_NOK)
	{
		pathok= -1;
	}
	else
		PATH_TRACE;


	if (testfail == 0)
		PATH_XS_RPT(2);
clean_out:
	(void) SEM_RM(semid_rw_owner,semun);
	FAILRM;

	DBUG_VOID_RETURN;
}

/*	32:	very similar to 31 except IPC_RMID rather than IPC_SET
*/
private	void
ch_t32()
{
	int	pathok = 0, testfail = 0, fail, err;

	DBUG_ENTER("ch_t32");

	/* owner and creator are vguid0/vggid0 */
	if (setuser_group(vguid1, vggid1) == SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	SEMCTL(semid_rw_owner, SEMNO, IPC_RMID, "IPC_RMID", semun, NOVERIFY, fail, err, FAILURE);

	testfail = 0;
	if (fail != SYSERROR || err != EPERM)
	{
		testfail++;
		xx_rpt(FAILURE);
                in_rpt("semctl(IPC_RMID) unexpected return when calling process does not match owner or creator user id");
		in_rpt("modes = 0%o, owner/creator uid = %d, calling process uid = %ld",
			RW_U, vguid0, (long)geteuid());
		valrpt(SYSERROR, fail, EPERM, err);
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}
