/*
*	SCCS:  @(#)  XPG4.os/ipc/semget/semget.c Rel 4.4.1     (06/09/97)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)  XPG4.os/ipc/semget/semget.c Rel 4.4.1     (06/09/97)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.semget

PROJECT:	XPG4 Validation suite

DESCRIPTION:
	Generic tests for the semget() system call.

AUTHOR:		Brian DeForest, UniSoft Corp.
DATE CREATED:	Thu Nov  8 14:25:01 PST 1990
MODIFICATIONS:	
		Matt Newman, Wed Sep 11 11:02:02 BST 1991
			XPG4 Intergration
		
		Geoff Clare, 12 July 1994
			Change 2nd arg to 0 in semctl() IPC_STAT calls.

************************************************************************/
#include        <std.h>
#include        <dbug.h>
#include        <report.h>
#include        <stdio.h>
#include        <sys/types.h>
#include        <errno.h>
#include        <string.h>
#include        <time.h>
#include        <unistd.h>
#include	<sys/stat.h>
#include	<sys/ipc.h>
#include	<sys/sem.h>
#include 	<tet_api.h>
#include        <sysdep.h>

#ifdef UNDEF_MACROS
#undef semget
#endif


#define 	NO_TESTS	9		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

extern	int	cppair();
extern	char	*errname();

private	void	test1(), test2(), test3(), test4(), test5(), test6(), test7(),
		test8(), test9(), ch_t1(), ch_t2(), ch_t3(), ch_t4(), ch_t5(),
		ch_t6(), ch_t7(), ch_t8(), ch_t9(), setup(), cleanup();

private int	setupok = -1;

/* 	test harness declarations	*/

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = cleanup;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	NULL, 0
};

public  int     no_tests = NO_TESTS;
private int	semid_rw_all, semid_rw_none;

private union semun {
		int val;
		struct semid_ds *buf;
		unsigned short *array;
};

#define NSEMS	1	/* number sems created per set 	*/

/*
 * 	access permissions
 */
#define	RW_U		(S_IRUSR|S_IWUSR)	/* *rw-?????? 0600 */
#define	RW_G		(S_IRGRP|S_IWGRP)	/* *???rw-??? 0060 */
#define	RW_O		(S_IROTH|S_IWOTH)	/* *??????rw- 0006 */
#define	RW_ALL		(RW_U|RW_G|RW_O)	/* *rw-rw-rw- 0666 */
#define	RW_NONE		(0)			/* *--------- 0000 */

/*
 * 	keys
 */
#define KEY_BASE	((int)'K')
#define KEY_RW_ALL	((key_t)(KEY_BASE + 0666))
#define KEY_RW_NONE	((key_t)(KEY_BASE + 0))
#define KEY_JUNK	((key_t)(KEY_BASE + 1))
#define KEY_TEST3	((key_t)(KEY_BASE + 3))

#define SEM_CREAT(key, nsems, mode)	semget(key, nsems, (mode)|IPC_CREAT)
#define SEM_RM(semid,semun)		semctl(semid, 0, IPC_RMID, semun)

private void
setup()
{
	int	err;
	int	pathok = 0;
	static char	errbuf[200];

	DBUG_ENTER("setup");

	/*	create semaphore sets:  
			one accessible to all
			one accessible to noone
		Only using one semaphore per set as multiple semaphores per set
		are currently not tested as per the Generic ABI. 
	*/

	if ((semid_rw_all = SEM_CREAT(KEY_RW_ALL, NSEMS, RW_ALL)) == SYSERROR)
	{
		int *t;
		static int testno[] = {1, 4, 6, 7, 9, 0};

		err = errno;
                (void) sprintf(errbuf,"setup: semget(%d, %d, 0%o) unexpectedly returned error: %d (%s)",KEY_RW_ALL,NSEMS,RW_ALL,err, errname(err));
		for (t=testno; *t != 0; t++)
			tet_delete(*t, errbuf);
	}
	else
		PATH_TRACE;

	if ((semid_rw_none = SEM_CREAT(KEY_RW_NONE, NSEMS, RW_NONE)) == SYSERROR)
	{
		err = errno;
		(void) sprintf(errbuf,"setup: semget(%d, %d, 0%o) unexpectedly returned error: %d (%s)",KEY_RW_NONE,NSEMS,RW_NONE,err, errname(err));
		tet_delete(5, errbuf);
	}
	else
		PATH_TRACE;

	if (pathok == 2)
		setupok = 0;
	else
		setupok = -1;

	DBUG_VOID_RETURN;
}

private	void
cleanup()
{
        union   semun semun;

	DBUG_ENTER("cleanup");

	semun.val = 0;	/* lint */

	/*	remove semaphore sets created in setup()	*/

	(void)SEM_RM(semid_rw_all,semun);

	(void)SEM_RM(semid_rw_none,semun);

	DBUG_VOID_RETURN;
}

private	void
test1()
{
	DBUG_ENTER("test1");

	(void)cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t1()
{
	int	pathok = setupok, testfail = 0, sid, err;

	DBUG_ENTER("ch_t1");

	if ((sid = semget(KEY_RW_ALL, NSEMS, RW_ALL)) == SYSERROR)
	{
		err = errno;
                if (testfail++ == 0)
                {
                        xx_rpt(DELETION);
                        in_rpt("semget(%d, %d, 0%o) unexpectedly returned error: %d (%s)",KEY_RW_ALL,NSEMS,RW_ALL,err, errname(err));
                }
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (sid != semid_rw_all)
	{
                if (testfail++ == 0)
                {
                        xx_rpt(FAILURE);
                        in_rpt("The returned semaphore ID != expected ID, returned ID = %d, expected ID = %d",sid, semid_rw_all);
                }
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	DBUG_ENTER("test2");

	(void)cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t2()
{
	int	pathok = setupok, testfail = 0, sid, err;
	struct semid_ds ds;
	uid_t 	euid;
	gid_t 	egid;
	time_t	tm;
	union semun semun;

	DBUG_ENTER("ch_t2");

	tm = time(0);	/* save current time for later */

	if ((sid = semget(IPC_PRIVATE, NSEMS, RW_ALL)) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
		{
                	xx_rpt(DELETION);
                	in_rpt("semget(IPC_PRIVATE, %d, 0%o) unexpectedly returned error: %d (%s)",NSEMS,RW_ALL,err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	semun.buf = &ds;
	if (semctl(sid, 0, IPC_STAT, semun) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
		{
                	xx_rpt(DELETION);
                	in_rpt("semctl(%d, 0, IPC_STAT, semid_ds *) unexpectedly returned error: %d (%s)",
				sid, err, errname(err));
		}
		goto clean_sem;
	}
	else
		PATH_TRACE;

	euid = geteuid();
	egid = getegid();

	if (ds.sem_perm.cuid != euid)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_perm.cuid != effective uid, sem_perm.cuid = %d, effective uid = %d",(int)ds.sem_perm.cuid, euid);
	}
	else
		PATH_TRACE;

	if (ds.sem_perm.uid != euid)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_perm.uid != effective uid, sem_perm.uid = %d, effective uid = %d",(int)ds.sem_perm.uid, euid);
	}
	else
		PATH_TRACE;

	if (ds.sem_perm.cgid != egid)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_perm.cgid != effective gid, sem_perm.cgid = %d, effective gid = %d",(int)ds.sem_perm.cgid, egid);
	}
	else
		PATH_TRACE;

	if (ds.sem_perm.gid != egid)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_perm.gid != effective gid, sem_perm.gid = %d, effective gid = %d",(int)ds.sem_perm.gid, egid);
	}
	else
		PATH_TRACE;

	if ((ds.sem_perm.mode & 0777) != RW_ALL)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_perm.mode != proper modes, sem_perm.mode & 0777 = 0%o, proper mode = 0%o",ds.sem_perm.mode&0777, RW_ALL);
	}
	else
		PATH_TRACE;

	if (ds.sem_nsems != NSEMS)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_nsems != actual nsems, sem_nsems = %d, actual nsems = %d",ds.sem_nsems, NSEMS);
	}
	else
		PATH_TRACE;

	if (ds.sem_otime != 0)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_otime != 0, sem_otime = %d",(int)ds.sem_otime);
	}
	else
		PATH_TRACE;

	/*	check ctime, plus or minus a few seconds	*/
	if ((ds.sem_ctime < tm-2) || (ds.sem_ctime > tm+2))
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_ctime != time created, sem_ctime = %d, time = %d",(int)ds.sem_ctime, tm);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(10);
	}

clean_sem:
	(void)SEM_RM(sid,semun);	/* remove sem set */

	DBUG_VOID_RETURN;
}

private	void
test3()
{
	DBUG_ENTER("test3");

	(void)cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t3()
{
	int	pathok = setupok, testfail = 0, sid, err;
	struct semid_ds ds;
	uid_t 	euid;
	gid_t 	egid;
	time_t	tm;
	union semun semun;

	DBUG_ENTER("ch_t3");

	tm = time(0);	/* save current time for later */

	if ((sid = SEM_CREAT(KEY_TEST3, NSEMS, RW_ALL)) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
		{
                	xx_rpt(DELETION);
		}
                in_rpt("semget(%d, %d, 0%o | IPC_CREAT) unexpectedly returned error: %d (%s)",KEY_TEST3,NSEMS,RW_ALL,err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	semun.buf = &ds;
	if (semctl(sid, 0, IPC_STAT, semun) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
		{
                	xx_rpt(DELETION);
		}
                in_rpt("semctl(%d, 0, IPC_STAT, semid_ds *) unexpectedly returned error: %d (%s)",
			sid, err, errname(err));
		goto clean_sem;
	}
	else
		PATH_TRACE;

	euid = geteuid();
	egid = getegid();

	if (ds.sem_perm.cuid != euid)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
               	in_rpt("sem_perm.cuid != effective uid, sem_perm.cuid = %d, effective uid = %d",(int)ds.sem_perm.cuid, euid);
	}
	else
		PATH_TRACE;

	if (ds.sem_perm.uid != euid)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_perm.uid != effective uid, sem_perm.uid = %d, effective uid = %d",(int)ds.sem_perm.uid, euid);
	}
	else
		PATH_TRACE;

	if (ds.sem_perm.cgid != egid)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_perm.cgid != effective gid, sem_perm.cgid = %d, effective gid = %d",(int)ds.sem_perm.cgid, egid);
	}
	else
		PATH_TRACE;

	if (ds.sem_perm.gid != egid)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_perm.gid != effective gid, sem_perm.gid = %d, effective gid = %d",(int)ds.sem_perm.gid, egid);
	}
	else
		PATH_TRACE;

	if ((ds.sem_perm.mode & 0777) != RW_ALL)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_perm.mode != proper modes, sem_perm.mode & 0777 = 0%o, proper mode = 0%o",ds.sem_perm.mode&0777, RW_ALL);
	}
	else
		PATH_TRACE;

	if (ds.sem_nsems != NSEMS)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_nsems != actual nsems, sem_nsems = %d, actual nsems = %d",ds.sem_nsems, NSEMS);
	}
	else
		PATH_TRACE;

	if (ds.sem_otime != 0)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_otime != 0, sem_otime = %d",(int)ds.sem_otime);
	}
	else
		PATH_TRACE;

	/*	check ctime, plus or minus a few seconds	*/
	if ((ds.sem_ctime < tm-2) || (ds.sem_ctime > tm+2))
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("sem_ctime != time created, sem_ctime = %d, time = %d",(int)ds.sem_ctime, tm);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(10);
	}

clean_sem:
	(void)SEM_RM(sid,semun);	/* remove sem set */

	DBUG_VOID_RETURN;
}

private	void
test4()
{
	DBUG_ENTER("test4");

	(void)cppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t4()
{
	int	pathok = setupok, testfail = 0, sid, err;

	DBUG_ENTER("ch_t4");

	if ((sid = SEM_CREAT(KEY_RW_ALL, NSEMS, RW_ALL)) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
		{
                	xx_rpt(DELETION);
		}
                in_rpt("semget(%d, %d, 0%o | IPC_CREAT) unexpectedly returned error: %d (%s)",KEY_RW_ALL,NSEMS,RW_ALL,err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (sid != semid_rw_all)
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
                	in_rpt("The returned semaphore ID != expected ID, returned ID = %d, expected ID = %d",sid, semid_rw_all);
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private	void
test5()
{
	DBUG_ENTER("test5");

	(void)cppair(ch_t5, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t5()
{
	int	pathok = setupok, testfail = 0, err;

	DBUG_ENTER("ch_t5");

	if ((semget(KEY_RW_NONE, NSEMS, RW_ALL) != SYSERROR) || ((err = errno) != EACCES))
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("semget(%d, %d, 0%o) where access is denied unexpectedly passed or errno != EACCES, semaphore modes = 0%o, error: %d (%s)",KEY_RW_NONE,NSEMS,RW_ALL,RW_NONE,err, errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private	void
test6()
{
	DBUG_ENTER("test6");

	(void)cppair(ch_t6, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t6()
{
	int	pathok = setupok, testfail = 0, err;

	DBUG_ENTER("ch_t6");

	if ((semget(KEY_RW_ALL, -1, RW_ALL) != SYSERROR) || ((err = errno) != EINVAL))
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("semget(key,-1,mode) unexpectedly passed or errno != EINVAL, error: %d (%s)",err,errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private	void
test7()
{
	DBUG_ENTER("test7");

	(void)cppair(ch_t7, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t7()
{
	int	pathok = setupok, testfail = 0, err;

	DBUG_ENTER("ch_t7");

	if ((semget(KEY_RW_ALL, NSEMS+1, RW_ALL) != SYSERROR) || ((err = errno) != EINVAL))
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("semget(key,%d,mode) where actual nsems < nsems unexpectedly passed or errno != EINVAL, actual nsems=%d, error: %d (%s)",NSEMS,NSEMS+1,err, errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private	void
test8()
{
	DBUG_ENTER("test8");

	(void)cppair(ch_t8, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t8()
{
	int	pathok = setupok, testfail = 0, err;

	DBUG_ENTER("ch_t8");

	if ((semget(KEY_JUNK, NSEMS, RW_ALL) != SYSERROR) || ((err = errno) != ENOENT))
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("semget(key,nsems,mode) where key doesn't exist and IPC_CREAT is not set unexpectedly passed or errno != ENOENT, error: %d (%s)",err,errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private	void
test9()
{
	DBUG_ENTER("test9");

	(void)cppair(ch_t9, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t9()
{
	int	pathok = setupok, testfail = 0, err;

	DBUG_ENTER("ch_t9");

	if ((semget(KEY_RW_ALL, NSEMS, RW_ALL|IPC_CREAT|IPC_EXCL) != SYSERROR) || ((err = errno) != EEXIST))
	{
		if (testfail++ == 0)
		{
                	xx_rpt(FAILURE);
		}
                in_rpt("semget(key,nsems,mode) where key exists and IPC_CREAT|IPC_EXCL are set unexpectedly passed or errno != EEXIST, error: %d (%s)",err,errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

