/*
*	SCCS:  @(#)  XPG4.os/ipc/semop/semop.c Rel 4.4.3     (07/20/98)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)  XPG4.os/ipc/semop/semop.c Rel 4.4.3     (07/20/98)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.semop

PROJECT:	XPG4 Validation suite

DESCRIPTION:
	Generic tests for the semop() system call.

AUTHOR:		Brian DeForest, UniSoft Corp.
DATE CREATED:	Tue Nov 13 16:59:29 PST 1990
MODIFICATIONS:	
		Matt Newman, Tue Sep 17 10:36:13 BST 1991
			XPG4 Intergration

		Calum Laing, Thu Apr 22 13:07:19 BST 1993
			Clean-up in semset(), semval() and semdoop().
			Added tet_setcontext() and tet_setblock() after fork().
			Added  a second SETTIMER to ch_detect_WAKE.
			Fixed cppair() return value check in test 14.

		Geoff Clare, 16 June 1993
			Changes for clean lint.

		Geoff Clare, 6 March 1998
			Change mbuf.mtype to long.

************************************************************************/
#include        <std.h>
#include        <stdlib.h>
#include        <dbug.h>
#include        <stdio.h>
#include        <report.h>
#include        <sys/types.h>
#include        <unistd.h>
#include        <errno.h>
#include        <string.h>
#include	<signal.h>
#include	<sys/stat.h>
#include	<sys/ipc.h>
#include	<sys/sem.h>
#include	<sys/msg.h>
#include        <wait_child.h>
#include        <sys/wait.h>
#include 	<tet_api.h>
#include        <sysdep.h>

#ifdef UNDEF_MACROS
#undef semop
#endif


#define 	NO_TESTS	19		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

#define 	NSEMS		1	/* number sems created per set 	*/
#define 	SEMNO		0	/* sem number used in set	*/
#define 	SEMVAL		5	/* sem initial value 		*/
#define 	SEMADJ		1	/* sem adjust value 		*/
#define 	SEMREM		4	/* sem remainder value 		*/
#define		EXPECT_0	0	/* expect a return of 0		*/
#define		VERIFY		1
#define		NOVERIFY 	0
#define		NOK 		02
#define		PATH_NOK	04

private void	semtimer();
#define		SEMTIMER 	2	/* alarm timeout in seconds	*/
#define		SETTIMER	globtimout = 0; \
				(void)vsignal(SIGALRM, semtimer); \
				(void)alarm(SEMTIMER)
#define		TIMEOUT		(globtimout != 0)

#define		FAILFILE	"/tmp/sem_fail"
#define		FAILRM		(void)unlink(FAILFILE)

#define		MSGOK		"good"
#define		MSGFAIL		"fail"
#define		MSGSZ		5
#define		MSGTYP		1

/*	attempts to reduce amount of redundant code...	*/

#define		MSGSND(mbuf,err) \
				mbuf.mtype = MSGTYP; \
				if (msgsnd(msgid, (void *)&mbuf, MSGSZ, IPC_NOWAIT) == SYSERROR) { \
					err = errno; \
        				if ((testfail++ == 0) && (tstfailed() == 0)) { \
						xx_rpt(DELETION); \
               					in_rpt("msgsnd() unexpectedly failed, error: %d (%s)",err,errname(err)); \
						exit(-1); \
					} \
				} \

#define		MSGRCV(mbuf,err) \
				mbuf.mtype = MSGTYP; \
				err = errno; \
				if (msgrcv(msgid, (void *)&mbuf, MSGSZ, 0, IPC_NOWAIT) == SYSERROR) { \
        				if ((testfail++ == 0) && (tstfailed() == 0)) { \
						xx_rpt(DELETION); \
               					in_rpt("msgrcv() unexpectedly failed, error: %d (%s)",err,errname(err)); \
					} \
				} \
				else \
					PATH_TRACE
/*
 * 	access permissions
 */
#define		RW_U		(S_IRUSR|S_IWUSR)	/* *rw-?????? 0600 */
#define		RW_G		(S_IRGRP|S_IWGRP)	/* *???rw-??? 0060 */
#define		RW_O		(S_IROTH|S_IWOTH)	/* *??????rw- 0006 */
#define		RW_ALL		(RW_U|RW_G|RW_O)	/* *rw-rw-rw- 0666 */
#define		RW_NONE		(0)			/* *--------- 0000 */

#define 	SEM_CREAT(key, nsems, mode)	semget(key, nsems, (mode)|IPC_CREAT)
#define 	SEM_RM(semid,semun)		semctl(semid, 0, IPC_RMID, semun)
#define		SEMBUF_CLR(sb)		(void) memset((void *)sb,0,sizeof(struct sembuf)*NSEMS)

private	void	test1(), test2(), test3(), test4(), test5(), test6(), 
		test7(), test8(), test9(), test10(), test11(), test12(),
		test13(), test14(), test15(), test16(), test17(), test18(),
		test19();
private	void	ch_t1(), ch_t2(), ch_t4(), 
		ch_t5(), ch_t6(), ch_t6_sub1(), ch_t7(), 
		ch_t8(), ch_t9(), ch_t12(),
		ch_t13(), ch_t14(), ch_t14_sub1(), 
		ch_t15();
private	void	detect_EINTR(), ch_detect_EINTR(),
		detect_WAKE(), ch_detect_WAKE();
private	void	setup(), cleanup();
private int	semset(), semval(), semdoop(); 
private int	tstfailed();

private	union 	semun {
		int val;
		struct semid_ds *buf;
		unsigned short *array;

} semun;

private int	setupok = -1;
private int	testfail;
private int	globtimout;
private	struct  sembuf	seminf[NSEMS];	/* stores semop() info */
private	int	semid_rw_all, semid_rw_none;
private	int	msgid;

struct  mbuf {
	long	mtype;
	char	mtext[MSGSZ];
};

/* 	test harness declarations	*/

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = cleanup;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	NULL, 0
};

public  int     no_tests = NO_TESTS;

extern 	char	*errname();
extern 	void	valrpt(), (*vsignal())();


private void
setup()
{
	int	err;
	static char	errbuf[200];
	int	pathok = 0;

	DBUG_ENTER("setup");

	semun.val = SEMVAL;

	/*	create semaphore sets:  
			one accessible to all
			one accessible to noone
		Only using one semaphore per set as multiple semaphores per set
		are currently not tested as per the Generic ABI. 
	*/

	if ((semid_rw_all = SEM_CREAT(IPC_PRIVATE, NSEMS, RW_ALL)) == SYSERROR)
	{
			int *t;
			static int testno[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 
					   12, 13, 14, 15, 17, 18, 0};

			err = errno;
			(void) sprintf(errbuf,"setup: semget(IPC_PRIVATE, %d, 0%o) unexpectedly returned error: %d, (%s)",NSEMS,RW_ALL,err,errname(err));
			for(t=testno; *t != 0; t++)
				tet_delete(*t, errbuf);
	}
	else
		PATH_TRACE;

	if ((semid_rw_none = SEM_CREAT(IPC_PRIVATE, NSEMS, RW_NONE)) == SYSERROR)
	{
			err = errno;
			(void) sprintf(errbuf,"setup: semget(IPC_PRIVATE, %d, 0%o) unexpectedly returned error: %d (%s)",NSEMS,RW_NONE,err,errname(err));
			tet_delete(19, errbuf);
	}
	else
		PATH_TRACE;

	if ((msgid = msgget(IPC_PRIVATE, RW_ALL)) == SYSERROR)
	{
			int *t;
			static int testno[] = {4, 5, 7, 12, 13, 15, 0};

			err = errno;
			(void) sprintf(errbuf,"setup: msgget(IPC_PRIVATE, 0%o) unexpectedly returned error: %d (%s)",RW_ALL,err,errname(err));
			for(t=testno; *t != 0; t++)
				tet_delete(*t, errbuf);
	}
	else
		PATH_TRACE;

	if (pathok!=3)
		setupok = -1;
	else
		setupok = 0;

	DBUG_VOID_RETURN;
}

private	void
cleanup()
{
	DBUG_ENTER("cleanup");

	/*	remove semaphore sets created in setup()	*/

	(void)SEM_RM(semid_rw_all,semun);

	(void)SEM_RM(semid_rw_none,semun);

	(void)msgctl(msgid, IPC_RMID, (struct msqid_ds *)0);

	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

/*	tstfailed:	prevents multiple test failure reports	*/
private int
tstfailed()
{
	char *buf = FAILFILE;
	struct stat stbuf;
	int err;

	if ((stat(buf,&stbuf) == SYSERROR) && (errno == ENOENT))
	{
		if (crfile(buf,(mode_t)0666,(int)getuid(),(int)getgid()) == SYSERROR)
		{
			err = errno;
                        if (!testfail++)
				xx_rpt(DELETION);
                        in_rpt("Could not create temporary file (%s), error: %d (%s)",buf,err,errname(err));
			return(1);
		}
		return(0);	/* test has not yet failed */
	}
	return(1);	/* test has already failed */
}

private int
semset(semid, semno, val)
int 	semid;
int 	semno; 
int	val;
{
	int	pathok = 0, err, getval;

	DBUG_ENTER("semset");
	/*	don't rely on any previous value, set explicitly	*/

	semun.val = val;
	if (semctl(semid, semno, SETVAL, semun) == SYSERROR)
	{
		err = errno;
		if ((testfail++ == 0) && (tstfailed() == 0))
                        xx_rpt(DELETION);
                in_rpt("semctl(%d, %d, SETVAL, %d) unexpectedly failed, error: %d (%s)",semid,semno,val,err,errname(err));
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	if ((getval = semctl(semid, semno, GETVAL, semun /*ignored*/)) == SYSERROR)
	{
		err = errno;
		if ((testfail++ == 0) && (tstfailed() == 0))
                        xx_rpt(DELETION);
                in_rpt("semctl(%d, %d, GETVAL, ...) unexpectedly failed, error: %d (%s)",semid,semno,err,errname(err));
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	if (getval != val)
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
                        xx_rpt(DELETION);
                in_rpt("semctl(%d, %d, GETVAL, ...) unexpectedly returned wrong semval, returned semval = %d, actual semval = %d, error: %d (%s)",semid,semno,getval,val,err,errname(err));
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_RETURN(getval);
}

private int
semval(semid, semno, err)
int 	semid;
int	semno;
int	*err;
{
	int	pathok = setupok, getval;

	DBUG_ENTER("semval");

	*err = 0;

	if ((getval = semctl(semid, semno, GETVAL, semun /*ignored*/)) == SYSERROR)
	{
		*err = errno;
		if ((testfail++ == 0) && (tstfailed() == 0))
                {
                        xx_rpt(DELETION);
                        in_rpt("semctl(%d, %d, GETVAL, ...) unexpectedly failed, error: %d (%s)",
				semid,semno,*err,errname(*err));
                }
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(getval);
}

/*	
	semdoop:	performs semop(), returns SYSERROR on error, errno in *err
			returns new semval on success, or 0 if !verify
*/
private int
semdoop(semid, semno, op, flgs, verify, err)
int 	semid;
int 	semno; 
short	op;
short 	flgs;
int 	verify;
int	*err;
{
	int	pathok = 0, testerr = 0, val, oval;

	DBUG_ENTER("semdoop");

	*err = 0;

	if ((oval = semctl(semid, semno, GETVAL, semun /*ignored*/)) == SYSERROR)
	{
		*err = errno;
		if ((testerr++ == 0) && (tstfailed() == 0))
                {
                        xx_rpt(DELETION);
                        in_rpt("semctl(%d, %d, GETVAL, ...) unexpectedly failed, error: %d (%s)",semid,semno,*err,errname(*err));
                }
	}
	else
		PATH_TRACE;

	SEMBUF_CLR(seminf);	/* clear sembuf array	*/
	seminf[semno].sem_op = op;
	seminf[semno].sem_num = semno;
	seminf[semno].sem_flg = flgs;
	
	if (semop(semid, seminf, NSEMS) == SYSERROR)
	{
		*err = errno;
		if ((testerr++ == 0) && (tstfailed() == 0))
                {
			if (verify)
			{
                        	xx_rpt(FAILURE);
                        	in_rpt("semop() unexpectedly failed where semid = %d, sem_op = %d, errno = %d (%s)",semid,op,*err,errname(*err));
			}
		}
		PATH_FUNC_RPT(1);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	val = 0;

	if (verify && ((val = semctl(semid, semno, GETVAL, semun /*ignored*/)) == SYSERROR))
	{
		*err = errno;
		if ((testerr++ == 0) && (tstfailed() == 0))
                {
                        xx_rpt(DELETION);
                        in_rpt("semctl(%d, %d, GETVAL, ...) unexpectedly failed, error: %d (%s)",semid,semno,*err,errname(*err));
                }
	}
	else
		PATH_TRACE;

	if (verify && (val != oval+op))
	{
		if ((testerr++ == 0) && (tstfailed() == 0))
                {
                        xx_rpt(FAILURE);
                        in_rpt("The semaphore value was not adjusted by %d, original value = %d, new value = %d",op,oval,val);
                }
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	if (testerr != 0)
		DBUG_RETURN(SYSERROR);


	DBUG_RETURN(val);
}

private void	
semtimer()
{
	globtimout++;
}

private	void
test1()
{
	int	pathok = setupok, ret, err, val;

	DBUG_ENTER("test1");

	/*	Since SEM_UNDO is not set, the value should not be
		changed as a result of exit() in ch_t1().
	*/
	testfail = 0;

	if ((ret=cppair(ch_t1, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret&PATH_NOK)
	{
		pathok = -1;
	}
	else
		PATH_TRACE;

	err = 0;
	globok = 0;
	if ((val = semval(semid_rw_all, SEMNO, &err)) != SEMREM)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("Unexpected semaphore value (semadj error), expected %d, got %d",SEMREM,val);
		in_rpt("errno is %d (%s)",err, errname(err));
	}
	else
		PATH_FUNC_TRACE;
	
	if (!testfail)
		PATH_XS_RPT(2);

clean_out:
	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

/*
	ch_t1:	set semval to SEMVAL, adjust by -SEMADJ,  and exit.   
			exit values are flags OK/NOK/PATH_NOK.
*/
private	void
ch_t1()
{
	int 	val, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	globok = 0;
	if ((err=semset(semid_rw_all, SEMNO, SEMVAL)) == SYSERROR)
	{
		testfail++;
		DBUG_EXIT(NOK);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if ((val=semdoop(semid_rw_all, SEMNO, -SEMADJ, 0, VERIFY, &err)) == SYSERROR)
	{
		testfail++;
		DBUG_EXIT(NOK);
	}
	else
		PATH_FUNC_TRACE;

	if (val != SEMREM)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("Unexpected semaphore value, expected %d, got %d",SEMREM,val);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok!=3)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	int	pathok = setupok, ret, err, val;

	DBUG_ENTER("test2");

	/*	Since SEM_UNDO is set, the value should be
		changed to the original SEMVAL as a result of exit() 
		in ch_t2().
	*/

	if ((ret=cppair(ch_t2, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
		goto clean_out;
	}
	else if (ret&PATH_NOK)
	{
		pathok = -1;
	}
	else
		PATH_TRACE;

	err = 0;
	globok = 0;
	if ((val = semval(semid_rw_all, SEMNO, &err)) != SEMVAL)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("Unexpected semaphore value (semadj error), expected %d, got %d",SEMVAL,val);
		in_rpt("errno is %d (%s)",err, errname(err));
	}
	else
		PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

clean_out:
	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

/*
	ch_t2:	set semval to SEMVAL 
			adjust by -SEMADJ with SEM_UNDO set, and exit.   
			exit values are flags OK/NOK/PATH_NOK.
*/

private	void
ch_t2()
{
	int 	val, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	globok = 0;
	if ((err=semset(semid_rw_all, SEMNO, SEMVAL)) == SYSERROR)
	{
		testfail++;
		DBUG_EXIT(NOK);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if ((val=semdoop(semid_rw_all, SEMNO, -SEMADJ, SEM_UNDO, VERIFY, &err)) == SYSERROR)
	{
		testfail++;
		DBUG_EXIT(NOK);
	}
	else
		PATH_FUNC_TRACE;

	if (val != SEMREM)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("Unexpected semaphore value, expected %d, got %d",SEMREM,val);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok!=3)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private	void
test3()
{
	int 	val, err;
	int	pathok = setupok;

	DBUG_ENTER("test3");

	testfail = 0; 
	globok = 0;
	if ((err=semset(semid_rw_all, SEMNO, SEMVAL)) == SYSERROR)
	{
		goto clean_out;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if (((val=semdoop(semid_rw_all, SEMNO, -(SEMVAL+1), IPC_NOWAIT, NOVERIFY, &err)) != SYSERROR) || (err != EAGAIN))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("semop(): unexpected successful return");
		valrpt(SYSERROR,val,EAGAIN,err);
		goto clean_out;
	}
	else
		PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

clean_out:
	FAILRM;

	DBUG_VOID_RETURN;
}

private	void
test4()
{
	DBUG_ENTER("test4");

	testfail = 0; 
	(void)cppair(ch_t4, NULLFN, WAITTIME*2, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t4()
{
	detect_EINTR(FALSE,GETNCNT);
}

private	void
detect_EINTR(detect_eintr, opcnt)
int 	detect_eintr,opcnt;
{
	int	pathok = setupok, val, err, wstat, ret, sem_op;
	struct	mbuf mbuf;

	DBUG_ENTER("detect_EINTR");

	if ((opcnt != GETNCNT) && (opcnt != GETZCNT))
	{
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("Internal test error: BAD opcnt argument to detect_EINTR");
		goto clean_out;
	}
	else
		PATH_TRACE;

	if (opcnt == GETNCNT)
		sem_op = -(SEMVAL+1);
	else /* (opcnt == GETZCNT) */
		sem_op = 0;

	globok = 0;
	if (semset(semid_rw_all, SEMNO, SEMVAL) == SYSERROR)
	{
		goto clean_out;
	}
	else
		PATH_FUNC_TRACE;

	/*	get initial semncnt	*/
	if ((val = semctl(semid_rw_all, SEMNO, opcnt, semun /*ignored*/)) == SYSERROR)
	{
		err = errno;
		if ((testfail++ == 0) && (tstfailed() == 0))
		{
			xx_rpt(DELETION);
                	in_rpt("semctl(%d, %d, %d, ...) unexpectedly failed",semid_rw_all,SEMNO,opcnt);
			valrpt(NOERROR,val,NOERROR,err);
		}
		goto clean_out;
	}
	else
		PATH_TRACE;

	/*	fork():
			child: spin waiting for semncnt to increment
			parent: semop() and suspend, incrementing semncnt
				send SIGALRM to resume and undo semop
	*/

	switch (fork())
	{
		case 0:	
			tet_setcontext();
			ch_detect_EINTR(val,opcnt);	/* child */
			break;		/* never get here */

		case SYSERROR:		/* error */
			err = errno;
			if ((testfail++ == 0) && (tstfailed() == 0))
			{
				xx_rpt(DELETION);
                		in_rpt("fork() unexpectedly failed, error: %d (%s)",err,errname(err));
			}
			DBUG_VOID_RETURN;		/* can't do any more */

		default:		/* parent */
			tet_setblock();
			PATH_TRACE;
			break;		/* fall thru */
	}

	SETTIMER;

	/*	should suspend since IPC_NOWAIT is not set, then wakeup
		due to signal
	*/

	globok = 0;
	if ((ret=semdoop(semid_rw_all, SEMNO, sem_op, 0, NOVERIFY, &err)) != SYSERROR)
	{
		(void)alarm(0);	/* cancel alarm in case didn't go off */

		if (detect_eintr && err != EINTR)
		{
			if (!testfail++)
				xx_rpt(FAILURE);
                	in_rpt("semop() where semid = %d, sem_num = %d, sem_op = %d unexpectedly succeeded",
				semid_rw_all, SEMNO, sem_op);
			valrpt(SYSERROR,ret,EINTR,err);
		}
	}
	else
	{
		(void)alarm(0);	/* cancel alarm in case didn't go off */
		PATH_FUNC_TRACE;
	}

	/*	wait for child if not already done */
	if ((wait(&wstat) == SYSERROR) && ((err = errno) != ECHILD) ) 
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
		{
			xx_rpt(DELETION);
               		in_rpt("wait() unexpectedly failed, error: %d (%s)",err,errname(err));
		}
	}
	else
		PATH_TRACE;

	if (WEXITSTATUS(wstat) != OK)
		testfail++; /* error reported in child process */
	else
		PATH_TRACE;

	MSGRCV(mbuf,err);

	/* if alarm didn't expired, proc didn't suspend */
	if (!TIMEOUT)
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
			xx_rpt(FAILURE);
                in_rpt("semop() where semid = %d, sem_num = %d, sem_op = %d did not suspend the calling process",semid_rw_all, SEMNO, sem_op);
	}
	else
		PATH_TRACE;

	if (strncmp(mbuf.mtext, MSGFAIL, MSGSZ) == 0)
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
			xx_rpt(FAILURE);
                in_rpt("semop() where semid = %d, sem_num = %d, sem_op = %d did not increment semncnt after suspending the calling process",semid_rw_all, SEMNO, sem_op);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(10);

clean_out:
	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

private	void
ch_detect_EINTR(oval,opcnt)
int oval,opcnt;
{
	int	val, err, testerr = 0;
	/* int	pathok = 0; Could find reasonable places for resiliency here */
	struct	mbuf mbuf;

	DBUG_ENTER("ch_detect_EINTR");

	SETTIMER;

	val = oval;

	/*	spin waiting for semncnt to increment	*/
	while (!TIMEOUT && val == oval && val != SYSERROR)
	{
		if ((val = semctl(semid_rw_all, SEMNO, opcnt, semun /*ignored*/)) == SYSERROR)
		{
			err = errno;
			if (!testerr++)
				xx_rpt(DELETION);
                	in_rpt("semctl(%d, %d, %d, ...) unexpectedly failed",semid_rw_all,SEMNO,opcnt);
			valrpt(NOERROR,val,NOERROR,err);
			break;
		}
	}

	(void)alarm(0);	/* cancel alarm */

	/*	
		Since both processes are running concurrently, the exit
		order is unknown so status messages are used rather than exit 	
		values.  Exit value is used as an attempt to indicate error 
		on msgsnd(), which shouldn't fail.
	*/

	if ((val == oval + 1) && (!testerr))
		(void)strncpy(mbuf.mtext, MSGOK, MSGSZ);	/* semncnt ok */
	else
		(void)strncpy(mbuf.mtext, MSGFAIL, MSGSZ);

	MSGSND(mbuf,err);
	
	if (testerr)
		DBUG_EXIT(NOK);

	DBUG_EXIT(OK);
}

private	void
test5()
{
	DBUG_ENTER("test5");

	testfail = 0; 
	(void)cppair(ch_t5, NULLFN, 2*WAITTIME, OK);

	DBUG_VOID_RETURN;
}

private	void
ch_t5()
{
	DBUG_ENTER("ch_t5");

	detect_WAKE(GETNCNT);

	DBUG_VOID_RETURN;
}

/*
	detect_WAKE:  similar to ch_t4 except a signal isn't used
		to wakeup the suspended process.  Instead, the actual
		semaphore is adjusted by ch_detect_WAKE() in order to wakeup
		and grant the semaphore to the previously suspended process.
*/
private	void
detect_WAKE(opcnt)
int	opcnt;
{
	int	pathok = setupok, cnt, val, err, wstat, sem_op, ret;
	struct	mbuf mbuf;

	DBUG_ENTER("detect_WAKE");

	if ((opcnt != GETNCNT) && (opcnt != GETZCNT))
	{
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("Internal test error: BAD opcnt argument to detect_WAKE");
		goto clean_out;
	}
	else
		PATH_TRACE;

	if (opcnt == GETNCNT)
		sem_op = -(SEMVAL+1);
	else /* (opcnt == GETZCNT) */
		sem_op = 0;

	globok = 0;
	if (semset(semid_rw_all, SEMNO, SEMVAL) == SYSERROR)
	{
		goto clean_out;
	}
	else
		PATH_FUNC_TRACE;

	/*	get initial semncnt	*/
	if ((cnt = semctl(semid_rw_all, SEMNO, opcnt, semun /*ignored*/)) == SYSERROR)
	{
		err = errno;
		if ((testfail++ == 0) && (tstfailed() == 0))
			xx_rpt(DELETION);
                in_rpt("semctl(%d, %d, %d, ...) unexpectedly failed", 
			semid_rw_all, SEMNO, opcnt);
		valrpt(NOERROR,cnt,NOERROR,err);
		goto clean_out;
	}
	else
		PATH_TRACE;

	/*	fork():
			child: spin waiting for semncnt to decrement
			       adjust semval to resume parent
			parent: semop() and suspend, incrementing semncnt
				wait for semaphore, get it, resume, 
				decrementing semncnt
	*/

	switch (fork())
	{
		case 0:	
			tet_setcontext();
			ch_detect_WAKE(cnt,opcnt);	/* child */
			DBUG_EXIT(OK);
			break;		/* never get here */

		case SYSERROR:		/* error */
			err = errno;
			if ((testfail++ == 0) && (tstfailed() == 0))
			{
				xx_rpt(DELETION);
                		in_rpt("fork() unexpectedly failed, error: %d (%s)",err,errname(err));
			}
			return;		/* can't do any more */

		default:		/* parent */
			tet_setblock();
			PATH_TRACE;
			break;		/* fall thru */
	}

	/*	should suspend since IPC_NOWAIT is not set, then wakeup
		when we get the semaphore.  Finally, semval should then be 0.
	*/

	globok = 0;
	if ((ret=semdoop(semid_rw_all, SEMNO, sem_op, 0, NOVERIFY, &err)) == SYSERROR)
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
			xx_rpt(FAILURE);
                in_rpt("semop() where semid = %d, sem_num = %d, sem_op = %d did not return 0 after suspending then resuming",semid_rw_all, SEMNO, sem_op);
		valrpt(NOERROR,ret,NOERROR,err);
	}
	else
		PATH_FUNC_TRACE;
		

	/*	wait for child if not already done */
	if ((wait(&wstat) == SYSERROR) && ((err = errno) != ECHILD) ) 
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
		{
			xx_rpt(DELETION);
               		in_rpt("wait() unexpectedly failed, error: %d (%s)",err,errname(err));
		}
	}
	else
		PATH_TRACE;

	if (WEXITSTATUS(wstat) == NOK)
		testfail++; /* error reported in child process */
	else if (WEXITSTATUS(wstat) != PATH_NOK)
		PATH_TRACE;

	MSGRCV(mbuf,err);

	if (strncmp(mbuf.mtext, MSGFAIL, MSGSZ) == 0)
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
		{
			xx_rpt(FAILURE);
                	in_rpt("semop() where semid = %d, sem_num = %d, sem_op = %d did not decrement semncnt after resuming the suspended process",semid_rw_all, SEMNO, sem_op);
		}
	}
	else
		PATH_TRACE;

	globok = 0;
	if ((val = semval(semid_rw_all, SEMNO, &err)) != EXPECT_0)
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
		{
			xx_rpt(FAILURE);
                	in_rpt("semop() where semid = %d, sem_num = %d, sem_op = %d did not change the semval as expected, expected = %d, actual = %d",semid_rw_all, SEMNO, sem_op,EXPECT_0,val);
		}
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(10);

clean_out:
	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

/*
	ch_detect_WAKE:	adjust semval so parent resumes and gets it,
			check semncnt is decremented after parent resumes
*/
private	void
ch_detect_WAKE(oval,opcnt)
int oval,opcnt;
{
	int	ret, val, err, sem_op, resume_op, testerr = 0, pathok = 0;
	struct	mbuf mbuf;

	DBUG_ENTER("ch_detect_WAKE");

	if (opcnt == GETNCNT)
	{
		sem_op = -(SEMVAL+1);
		resume_op = 1;
	}
	else /* (opcnt == GETZCNT) */
	{
		sem_op = 0;
		resume_op = -SEMVAL;
	}

	SETTIMER;

	val = oval;

	/*	spin waiting for semncnt to increment -- parent suspended */
	while (!TIMEOUT && val == oval && val != SYSERROR)
	{
		if ((val = semctl(semid_rw_all, SEMNO, opcnt, semun /*ignored*/)) == SYSERROR)
		{
			err = errno;
			if (tstfailed() == 0)
			{
				xx_rpt(DELETION);
                		in_rpt("semctl(%d, %d, %d, ...) unexpectedly failed, error: %d (%s)",semid_rw_all, SEMNO, opcnt, err,errname(err));
			}
		}
	}

	(void)alarm(0);	/* cancel alarm */

	if (val != oval + 1)
	{
		err = errno;
		if ((testerr++ == 0) && (tstfailed() == 0))
		{
			xx_rpt(DELETION);
                	in_rpt("semop() where semid = %d, sem_num = %d, sem_op = %d did not increment semncnt after suspending the calling process",semid_rw_all, SEMNO, sem_op);
		}
	}
	else
		PATH_TRACE;

	/*	adjust semval, resuming parent 	*/
	globok = 0;
	if ((ret=semdoop(semid_rw_all, SEMNO, resume_op, 0, NOVERIFY, &err)) == SYSERROR)
	{
		if ((testerr++ == 0) && (tstfailed() == 0))
			xx_rpt(FAILURE);
                in_rpt("semop() where semid = %d, sem_num = %d, sem_op = %d returned an error, expecting to resume a process",semid_rw_all, SEMNO, resume_op);
		valrpt(NOERROR,ret,NOERROR,err);
	}
	else
		PATH_FUNC_TRACE;

	SETTIMER;
	/*	spin waiting for semncnt to decrement -- parent resumed	*/
	while (!TIMEOUT && val != oval && val != SYSERROR)
	{
		if ((val = semctl(semid_rw_all, SEMNO, opcnt, semun /*ignored*/)) == SYSERROR)
		{
			err = errno;
			if (!testerr++ && (tstfailed() == 0))
			{
				xx_rpt(DELETION);
                		in_rpt("semctl(%d, %d, %d, ...) unexpectedly failed, error: %d (%s)",semid_rw_all, SEMNO, opcnt, err,errname(err));
			}
		}
	}

	(void)alarm(0);	/* cancel alarm */

	/*	
		Since both processes are running concurrently, the exit
		order is unknown so status messages are used rather than exit 	
		values.  Exit value is used as an attempt to indicate error 
		on msgsnd(), which shouldn't fail.
	*/

	if ((val == oval) && !testerr)
		(void)strncpy(mbuf.mtext, MSGOK, MSGSZ);	/* semncnt ok */
	else
		(void)strncpy(mbuf.mtext, MSGFAIL, MSGSZ);

	MSGSND(mbuf,err);

	if (testerr)
		DBUG_EXIT(NOK);
	
	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);
}

private	void
test6()
{
	int	pathok = setupok, ret, err;

	DBUG_ENTER("test6");

	testfail = 0; 
	if ((ret=cppair(ch_t6, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{
		testfail++;
	} else if (ret & PATH_NOK)
	{
		pathok = -1;
	}
	else
		PATH_TRACE;

	/*	recreate semaphore set */
	if ((semid_rw_all = SEM_CREAT(IPC_PRIVATE, NSEMS, RW_ALL)) == SYSERROR)
	{
		err = errno;
		if ((testfail++ == 0) && (tstfailed() == 0))
                {
                       	xx_rpt(DELETION);
                       	in_rpt("semget(IPC_PRIVATE, %d, 0%o) unexpectedly returned error, error: %d (%s)",NSEMS,RW_ALL,err,errname(err));
                }
	}
	else
		PATH_TRACE;


	if (testfail == 0)
		PATH_XS_RPT(2);

	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

private	void
ch_t6()
{
	int	pathok = 0, ncnt, err, ret, wstat;

	DBUG_ENTER("ch_t6");

	globok = 0;
	if (semset(semid_rw_all, SEMNO, SEMVAL) == SYSERROR)
	{
		testfail++;
		DBUG_EXIT(NOK);
	}
	else
		PATH_FUNC_TRACE;

	/*	get initial semncnt	*/
	if ((ncnt = semctl(semid_rw_all, SEMNO, GETNCNT, semun /*ignored*/)) == SYSERROR)
	{
		err = errno;
		if ((testfail++ == 0) && (tstfailed() == 0))
		{
			xx_rpt(DELETION);
                	in_rpt("semctl(%d, %d, GETNCNT, ...) unexpectedly failed, error: %d (%s)",semid_rw_all, SEMNO, err,errname(err));
		}
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	/*	fork():
			child: 	wait for parent to suspend
			 	remove semaphore set
			parent: semop() and suspend
				resume with EIDRM
	*/

	switch (fork())
	{
		case 0:	
			tet_setcontext();
			ch_t6_sub1(ncnt);	/* child */
			DBUG_EXIT(OK);
			break;		/* never get here */

		case SYSERROR:		/* error */
			err = errno;
			if ((testfail++ == 0) && (tstfailed() == 0))
			{
				xx_rpt(DELETION);
                		in_rpt("fork() unexpectedly failed, error: %d (%s)",err,errname(err));
			}
			DBUG_VOID_RETURN;		/* can't do any more */

		default:		/* parent */
			tet_setblock();
			PATH_TRACE;
			break;		/* fall thru */
	}

	/*	should suspend since IPC_NOWAIT is not set, then wakeup
		due to removal of semid
	*/

	globok = 0;
	if (((ret=semdoop(semid_rw_all, SEMNO, -(SEMVAL+1), 0, NOVERIFY, &err)) 
			!= SYSERROR) && (err != EIDRM))
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
			xx_rpt(FAILURE);
               	in_rpt("semop() where semid = %d, sem_num = %d, sem_op = %d ",
			semid_rw_all, SEMNO, -(SEMVAL+1));
		in_rpt("Unexpected return after semaphore set was removed");
		valrpt(SYSERROR,ret,EIDRM,err);
	}
	else
		PATH_FUNC_TRACE;

	/*	wait for child if not already done */
	if ((wait(&wstat) == SYSERROR) && ((err = errno) != ECHILD) ) 
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
			xx_rpt(DELETION);
               	in_rpt("wait() unexpectedly failed, error: %d (%s)",err,errname(err));
	}
	else
		PATH_TRACE;

	if (WEXITSTATUS(wstat) == NOK)
		testfail++; /* error reported in child process */
	else if (WEXITSTATUS(wstat) != PATH_NOK)
		PATH_TRACE;


	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok != 6)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);
}

private	void
ch_t6_sub1(oval)
int oval;
{
	int	pathok = 0, testerr = 0, val, ret, err;

	DBUG_ENTER("ch_t6_sub1");

	SETTIMER;

	val = oval;

	/*	spin waiting for semncnt to increment	*/
	while (!TIMEOUT && val == oval && val != SYSERROR)
	{
		if ((val = semctl(semid_rw_all, SEMNO, GETNCNT, semun /*ignored*/)) == SYSERROR)
		{
			err = errno;
			if (!testerr++ && tstfailed() == 0)
				xx_rpt(DELETION);
                	in_rpt("semctl(%d, %d, GETNCNT, ...) unexpectedly failed, error: %d (%s)",semid_rw_all, SEMNO, err,errname(err));
		}
	}

	(void)alarm(0);	/* cancel alarm */

	/*	parent suspended, now remove it	*/

	if (val != oval + 1)
	{
		err = errno;
		if ((testerr++ == 0) && (tstfailed() == 0))
                        xx_rpt(DELETION);
                in_rpt("The parent process unexpectedly never suspended from semop() where semid = %d, sem_num = %d, sem_op = %d, error: %d (%s)",semid_rw_all, SEMNO, -(SEMVAL+1),err,errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if ((ret=SEM_RM(semid_rw_all,semun)) == SYSERROR)
	{
		err = errno;
		if ((testerr++ == 0) && (tstfailed() == 0))
                        xx_rpt(DELETION);
                in_rpt("semctl(%d, 0, IPC_RMID, ...) returned unexpectedly",semid_rw_all);
		valrpt(NOERROR,ret,NOERROR,err);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
		
	if (testerr)
		DBUG_EXIT(NOK);

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);
}

private	void
test7()
{
	DBUG_ENTER("test7");

	testfail = 0; 
	(void)cppair(ch_t7, NULLFN, WAITTIME*2, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t7()
{
	DBUG_ENTER("ch_t7");

	/*	since this is so similar to test 4,  use 
		detect_EINTR(TRUE,GETNCNT) and detect EINTR.
	*/
	detect_EINTR(TRUE,GETNCNT);

	DBUG_VOID_RETURN;
}

private	void
test8()
{
	DBUG_ENTER("test8");

	testfail = 0; 

	(void)cppair(ch_t8, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t8()
{
	int	pathok = setupok, val, err;

	DBUG_ENTER("ch_t8");

	globok = 0;
	if (semset(semid_rw_all, SEMNO, SEMVAL) == SYSERROR)
	{
		goto clean_out;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if (semdoop(semid_rw_all, SEMNO, SEMADJ, 0, VERIFY, &err) == SYSERROR)
	{
		goto clean_out;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if ((val = semval(semid_rw_all, SEMNO, &err)) != SEMVAL+SEMADJ)
	{
		if (!testfail++ && tstfailed() == 0)
			xx_rpt(FAILURE);
               	in_rpt("The semaphore value has changed, expected = %d, actual = %d",SEMVAL+SEMADJ,val);
	}
	else
		PATH_FUNC_TRACE;


	if (testfail == 0)
		PATH_XS_RPT(3);

clean_out:

	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

private	void
test9()
{
	int	pathok = setupok, err = 0, ret, val;

	DBUG_ENTER("test9");

	testfail = 0; 
	/*	Since SEM_UNDO is set, the value should be
		changed to the original SEMVAL as a result of exit() 
		in ch_t9().
	*/

	if ((ret=cppair(ch_t9, NULLFN, WAITTIME, NOK|PATH_NOK)) & NOK)
	{ /* expect sem value to be SEMVAL+SEMADJ upon return */
		testfail++;
		goto clean_out;
	}
	else if (ret & PATH_NOK)
	{
		pathok = -1;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ((val=semval(semid_rw_all, SEMNO, &err)) == SYSERROR)
	{
		goto clean_out;
	}
	else
		PATH_FUNC_TRACE;

	if (val != SEMVAL && (tstfailed() == 0))
	{
                if (!testfail++)
			xx_rpt(FAILURE);
                in_rpt("The semadj value has changed, expected %d, actual = %d",-SEMADJ,val-(SEMVAL+SEMADJ));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
clean_out:

	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

/*
	ch_t9:	set semval to SEMVAL 
			adjust by SEMADJ with SEM_UNDO set, and exit.   
			successful exit value is the current semval,
			SEMVAL+SEMADJ.
*/
private	void
ch_t9()
{
	int 	pathok = 0, err = 0, val;

	DBUG_ENTER("ch_t9");

	globok = 0;
	if (semset(semid_rw_all, SEMNO, SEMVAL) == SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if ((val=semdoop(semid_rw_all, SEMNO, SEMADJ, SEM_UNDO, VERIFY, &err)) == SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_FUNC_TRACE;

        if (val != SEMVAL+SEMADJ)
        {
		if ((testfail++ == 0) && (tstfailed() == 0))
                        xx_rpt(FAILURE);
                in_rpt("semop() where sem_op = %d did not increment semval, expected semval = %d, actual semval = %d",SEMADJ,SEMVAL+SEMADJ,val);
		DBUG_EXIT(NOK);
        }
        else
                PATH_TRACE;

	if (pathok != 3)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);
}


private	void
test10()
{
	int	pathok = setupok, err = 0, val;

	DBUG_ENTER("test10");

	testfail = 0; 

	globok = 0;
	if (semset(semid_rw_all, SEMNO, 0) == SYSERROR)
	{
		goto clean_out;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if ((val = semdoop(semid_rw_all, SEMNO, 0, 0, VERIFY, &err)) != EXPECT_0)
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
                	xx_rpt(FAILURE);
                in_rpt("semop() where semval == 0 and semop == 0 didn't return 0, returned = %d",val);
		goto clean_out;
	}
	else
		PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

clean_out:
	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

private	void
test11()
{
	int	pathok = setupok, err = 0, val;

	DBUG_ENTER("test11");

	testfail = 0; 

	globok = 0;
	if (semset(semid_rw_all, SEMNO, SEMVAL) == SYSERROR)
	{
		goto clean_out;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if (((val = semdoop(semid_rw_all, SEMNO, 0, IPC_NOWAIT, NOVERIFY, &err)) != SYSERROR) && (err != EAGAIN))
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
                	xx_rpt(FAILURE);
                in_rpt("semop() where semval == %d and semop == 0 returned unexpectedly", SEMVAL);
		valrpt(SYSERROR,val,EAGAIN,err);
		goto clean_out;
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

clean_out:
	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

private	void
test12()
{
	DBUG_ENTER("test12");

	testfail = 0; 
	(void)cppair(ch_t12, NULLFN, WAITTIME*2, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t12()
{
	/* Tests 4, 7, 12, 15 all use this subroutine to detect/expect EINTR */
	detect_EINTR(FALSE,GETZCNT);
}

private	void
test13()
{
	DBUG_ENTER("test13");

	testfail = 0; 
	(void)cppair(ch_t13, NULLFN, 2*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

/*
	ch_t13: similar to ch_t12 except a signal isn't used
		to wakeup the suspended process.  Instead, the actual
		semaphore is adjusted by ch_detect_WAKE() in order to wakeup
		and grant the semaphore to the previously suspended process.
*/
private	void
ch_t13()
{

	DBUG_ENTER("ch_t13");

	detect_WAKE(GETZCNT);

	DBUG_VOID_RETURN;
}

private	void
test14()
{
	int	pathok = setupok, err;

	DBUG_ENTER("test14");

	testfail = 0; 
	err = cppair(ch_t14, NULLFN, WAITTIME, -1);

	/*	recreate semaphore set */
	if ((semid_rw_all = SEM_CREAT(IPC_PRIVATE, NSEMS, RW_ALL)) == SYSERROR)
	{
		err = errno;
		if ((testfail++ == 0) && (tstfailed() == 0))
                {
                       	xx_rpt(DELETION);
                       	in_rpt("semget(IPC_PRIVATE, %d, 0%o) unexpectedly returned error, error: %d (%s)",NSEMS,RW_ALL,err,errname(err));
                }
	}
	else
		PATH_TRACE;

	FAILRM;		/* remove temporary file */

	if (err == OK && testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private	void
ch_t14()
{
	int	pathok = setupok, zcnt, err;

	globok = 0;
	if (semset(semid_rw_all, SEMNO, SEMVAL) == SYSERROR)
		testfail++;
	else
		PATH_FUNC_TRACE;

	/*	get initial semzcnt	*/
	if ((zcnt = semctl(semid_rw_all, SEMNO, GETZCNT, semun /*ignored*/)) == SYSERROR)
	{
		err = errno;
		if ((testfail++ == 0) && (tstfailed() == 0))
		{
			xx_rpt(DELETION);
                	in_rpt("semctl(%d, %d, GETZCNT, ...) unexpectedly failed, error: %d (%s)",semid_rw_all, SEMNO, err,errname(err));
		}
	}
	else
		PATH_TRACE;

	/*	fork():
			child: 	wait for parent to suspend
			 	remove semaphore set
			parent: semop() and suspend
				resume with EIDRM
	*/

	switch (fork())
	{
		case 0:	
			tet_setcontext();
			ch_t14_sub1(zcnt);	/* child */
			break;		/* never get here */

		case SYSERROR:		/* error */
			err = errno;
			if ((testfail++ == 0) && (tstfailed() == 0))
			{
				xx_rpt(DELETION);
                		in_rpt("fork() unexpectedly failed, error: %d (%s)",err,errname(err));
			}
			return;		/* can't do any more */

		default:		/* parent */
			tet_setblock();
			PATH_TRACE;
			break;		/* fall thru */
	}

	/*	should suspend since IPC_NOWAIT is not set, then wakeup
		due to removal of semid
	*/

	(void)semdoop(semid_rw_all, SEMNO, 0, 0, NOVERIFY, &err);

	if (err != EIDRM)
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
		{
			xx_rpt(FAILURE);
                	in_rpt("semop() where semid = %d, sem_num = %d, sem_op = %d did not fail with errno == EIDRM after semaphore set was removed, error: %d (%s)",semid_rw_all, SEMNO, 0,err,errname(err));
		}
	}
	else
		PATH_TRACE;

	if (testfail != 0 || pathok != 4)
		exit(-1);
	else
		exit(0);

}

private	void
ch_t14_sub1(oval)
int oval;
{
	int	val, err;

	SETTIMER;

	val = oval;

	/*	spin waiting for semzcnt to increment	*/
	while (!TIMEOUT && val == oval && val != SYSERROR)
	{
		if ((val = semctl(semid_rw_all, SEMNO, GETZCNT, semun /*ignored*/)) == SYSERROR)
		{
			err = errno;
			if (tstfailed() == 0)
			{
				xx_rpt(DELETION);
                		in_rpt("semctl(%d, %d, GETZCNT, ...) unexpectedly failed, error: %d (%s)",semid_rw_all, SEMNO, err,errname(err));
			}
		}
	}

	(void)alarm(0);	/* cancel alarm */

	/*	parent suspended, now remove it	*/

	if (val != oval + 1)
	{
		err = errno;
		if ((testfail++ == 0) && (tstfailed() == 0))
                {
			if (tstfailed() == 0)
			{
                        	xx_rpt(DELETION);
                        	in_rpt("The parent process unexpectedly never suspended from semop() where semid = %d, sem_num = %d, sem_op = %d, error: %d (%s)",semid_rw_all, SEMNO, 0,err,errname(err));
                	}
		}
	}

	if ((err = SEM_RM(semid_rw_all,semun)) == SYSERROR)
	{
		err = errno;
		if ((testfail++ == 0) && (tstfailed() == 0))
                {
                        xx_rpt(DELETION);
                        in_rpt("semctl(%d, 0, IPC_RMID, ...) unexpectedly returned error: %d (%s)",semid_rw_all,err,errname(err));
                }
	}
		
	exit(0);
}

private	void
test15()
{
	DBUG_ENTER("test15");

	testfail = 0; 
	(void)cppair(ch_t15, NULLFN, WAITTIME*2, 0);

	DBUG_VOID_RETURN;
}

private	void
ch_t15()
{
	DBUG_ENTER("ch_t15");

	/*	since this is so similar to test 12,  use 
		detect_EINTR(TRUE,GETZCNT) and detect EINTR.
	*/
	detect_EINTR(TRUE,GETZCNT);

	DBUG_VOID_RETURN;
}

private	void
test16()
{
	int	pathok = setupok, sid, err;

	DBUG_ENTER("test16");

	testfail = 0; 
	sid = -1;
	/*	just to be sure -1 isn't valid 	*/
	while (sid == semid_rw_all || sid == semid_rw_none)
		sid++;

	SEMBUF_CLR(seminf);	/* clear sembuf array	*/
	
	if ((semop(sid, seminf, NSEMS) != SYSERROR)  || ((err = errno) != EINVAL))
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
                {
                        xx_rpt(FAILURE);
                        in_rpt("semop() where no semaphores exist for id %d unexpectedly passed or errno != EINVAL, error: %d (%s)",sid,err,errname(err));
                }
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

private	void
test17()
{
	int	pathok = setupok, err;

	DBUG_ENTER("test17");

	testfail = 0; 

	SEMBUF_CLR(seminf);	/* clear sembuf array	*/
	seminf[SEMNO].sem_num = (unsigned short) -1;
	seminf[SEMNO].sem_op = -SEMADJ;
	
	if ((semop(semid_rw_all, seminf, NSEMS) != SYSERROR)  || ((err = errno) != EFBIG))
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
                {
                        xx_rpt(FAILURE);
                        in_rpt("semop() where sem_num == -1 unexpectedly passed or errno != EFBIG, error: %d (%s)",err,errname(err));
                }
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

private	void
test18()
{
	int	pathok = setupok, err;

	DBUG_ENTER("test18");

	testfail = 0; 

	SEMBUF_CLR(seminf);	/* clear sembuf array	*/
	seminf[SEMNO].sem_num = NSEMS+1;
	seminf[SEMNO].sem_op = -SEMADJ;
	
	if ((semop(semid_rw_all, seminf, NSEMS) != SYSERROR)  || ((err = errno) != EFBIG))
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
                {
                        xx_rpt(FAILURE);
                        in_rpt("semop() where sem_num > actual nsems unexpectedly passed or errno != EFBIG, sem_num = %d, actual nsems = %d, error: %d (%s)",NSEMS+1,NSEMS,err,errname(err));
                }
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}

private	void
test19()
{
	int	pathok = setupok, err;

	DBUG_ENTER("test19");

	testfail = 0; 

	SEMBUF_CLR(seminf);	/* clear sembuf array	*/
	seminf[SEMNO].sem_num = SEMNO;
	seminf[SEMNO].sem_op = -SEMADJ;
	
	/* attempt to alter where no permission is granted */
	if ((semop(semid_rw_none, seminf, NSEMS) != SYSERROR)  || ((err = errno) != EACCES))
	{
		if ((testfail++ == 0) && (tstfailed() == 0))
                {
                        xx_rpt(FAILURE);
                        in_rpt("semop() where access is denied unexpectedly passed or errno != EACCESS, modes = 0%o, error: %d (%s)",RW_NONE,err,errname(err));
                }
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	FAILRM;		/* remove temporary file */

	DBUG_VOID_RETURN;
}
