/*
*	SCCS:  @(#)  XPG4.os/ipc/shmat/shmat.c Rel 4.4.1     (06/09/97)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)  XPG4.os/ipc/shmat/shmat.c Rel 4.4.1     (06/09/97)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.shmat

PROJECT:	XPG4 Validation suite

DESCRIPTION:
	Testset for the shmat() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Tony Rems
DATE CREATED:	Thu Oct 15 12:24:50 BST 1990
MODIFICATIONS:	
		Matt Newman, Wed Sep 11 11:26:16 BST 1991
			XPG4 Intergration
		
		Geoff Clare, 5 May 1992
			Change sys/fcntl.h to fcntl.h.

		Neil Moses, 8 May 1992.
			Check that interface is supported.
			Added TEST For ENOSYS if interface not supported.

		Neil Moses, 4 June 1992.
			Use sysconf() to check if interface supported.
		
		Geoff Clare, 9 June 1993
			In test 2: ensure rounded-down address will be
			legal, and use ptrdiff_t instead of int in address
			calculation.
		
		Geoff Clare, 4 March 1994
			Removed <sys/sysmacros.h> header.
			Removed multiple definition of tet_thistest.
		
		Geoff Clare, 11 July 1994
			Test 2 untested if would need to shmget() an
			unreasonably large amount of memory.
		
		Geoff Clare, 9 April 1997
			Fix test 2 problem when (ptrdiff_t)addr is < 0.

************************************************************************/
#include	<std.h>
#include	<stddef.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<string.h>
#include	<sys/types.h>
#include	<sys/ipc.h>
#include 	<sys/shm.h>
#include	<sys/stat.h>
#include	<signal.h>
#include	<fcntl.h>
#include	<limits.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<report.h>
#include	<errno.h>
#include 	<tet_api.h>
#include 	<sysdep.h>

#ifdef UNDEF_MACROS
#undef shmat
#endif


#define		WAITTIME	(5 * SPEEDFACTOR)
#define 	NO_TESTS	8	/* number of tests in testset */
#define		MODEANY		((mode_t)(S_IRWXU|S_IRWXG|S_IRWXO))
#define		MODERDONLY	((mode_t)(S_IRUSR|S_IRGRP|S_IROTH))
#define		MODEMASK	(mode_t)MODEANY
#define		MODEURDONLY	S_IRUSR
#define		RW_U            (S_IRUSR|S_IWUSR)
#define		RW_G            (S_IRGRP|S_IWGRP)
#define		RW_O            (S_IROTH|S_IWOTH)
#define		MODERWANY	((mode_t) (RW_U|RW_G|RW_O))
#define		SEGSIZE		512 	     	/* Use 512 byte shm segments */
#define		NONE		0L
#define		NOK		02
#define		PATH_ERR_EXIT	04
#define		ID1		11		/* id's for use with ftok */
#define		ID2		22	
#define		ID3		33
#define		ID4		44
#define		ID5		55
#define		ID6		66
#define		ID7		77
#define		TOKFILE		"./shmatfile" 	/* file for use with ftok */

#define 	STRING1		"test1 string"	/* strings to write into shm */
#define 	STRING4		"test4 string"	

#define		RD_FD		0
#define		WR_FD		1

int		no_tests = NO_TESTS;
int		pip_fds[2];

#ifndef ftok
extern	key_t	ftok();
#endif
extern 	char	*errname();
extern 	void	xx_all();

extern 	time_t	time();
extern  void    (*vsignal())();
extern  void    valrpt();

private void	stub(), tidy(), setup(), test1(), pa_t1(), ch_t1(), test2(),
		test3(), test4(), pa_t4(), ch_t4(), test5(), test6(), ch_t6(),
		test7(), test8();

public  void    (*realfuncs[]) () = { test1,  test2,  test3,  test4,  test5,
                                      test6,  test7, test8, NULL };

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = tidy;

private void	twrap();

public	struct tet_testlist tet_testlist[] = {
	twrap,	 1,
	twrap,	 2,
	twrap,	 3,
	twrap,	 4,
	twrap,	 5,
	twrap,	 6,
	twrap,	 7,
	test8,	 8,
	NULL,	 0
};

private	int	test_fail;
private	int	tsup = 0;
private	int	test_deleted;
private int	setupok = -1;
private int	g_shmids[2*NO_TESTS];
private key_t	Key1;
private key_t	Key2;
private key_t	Key3;
private key_t	Key4;
private key_t	Key5;
private key_t	Key6;
private key_t	Key7;

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if(!tsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("sysconf(_SC_XOPEN_SHM) returned -1");
	}
	else
		if (tet_thistest >= 1 && tet_thistest <= no_tests)
			(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

#if TESTBED0
#define	shmat(a,b,c)	((void*) SYSERROR)
#endif

private void
setup()	
{
	int	pathok = 0;
	int 	i;

	DBUG_ENTER("setup");

	for (i = 0; i < 2*no_tests; i++)
		g_shmids[i] = -1;

	errno = 0;
	if(sysconf(_SC_XOPEN_SHM) == -1)
	{
		if (errno != 0)
		{
			xx_all(DELETION, "sysconf(_SC_XOPEN_SHM) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		tsup = 0;
	}
	else
	{
		tsup = 1;
		PATH_TRACE;
	}

	if (crfile(TOKFILE, MODEANY, (int)getuid(), (int)getgid()) == SYSERROR) {
		for (i = 1; i <= NO_TESTS; i++) {
			tet_delete(i, "Could not create reference file for keys.");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(2);

	DBUG_VOID_RETURN;
}

		
private	void
tidy()
{
	int i;
	struct shmid_ds cln_ds;

	DBUG_ENTER("tidy");
	(void) unlink(TOKFILE);
	for (i = 0; i < 2*no_tests; i++)
		if (g_shmids[i] != -1)
			(void)shmctl(g_shmids[i], IPC_RMID, &cln_ds);
	(void)close(pip_fds[RD_FD]);
	(void)close(pip_fds[WR_FD]);
	DBUG_VOID_RETURN;
}


private void		/* Test that we can shmat() a file for read/write */
test1()			/* when we have read/write permissions */
{
        struct shmid_ds shmid_ds1;
        int shmid1;
        int pathok = setupok;
        int err = NOERROR;
	int rval;

	DBUG_ENTER("test1");

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	Key1 = ftok(TOKFILE, ID1);
	err = errno;
	if (Key1 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID1, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	


	errno = NOERROR;
	shmid1 = shmget(Key1, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid1;
	if (shmid1 == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmget() with IPC_CREAT set failed with err %d", err);
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	rval = cppair(ch_t1, pa_t1, WAITTIME, NOK|PATH_ERR_EXIT);
	if (rval&NOK) {
		/* failure/deletion was already reported in pa_t1 or ch_t1 */
		test_deleted++;
	}
	else if ((rval&PATH_ERR_EXIT) == 0)
		PATH_TRACE;

	(void)shmctl(shmid1, IPC_RMID, &shmid_ds1);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void	/* This guy maps in the shm segment and writes to it and */
pa_t1()		/* then signals the child when he's done. */
{
	int shmid1;
	int err;
	void *shmidptr1, *origptr;
	int pathok = 0;

	DBUG_ENTER("pa_t1");

	errno = NOERROR;
	shmid1 = shmget(Key1, SEGSIZE, MODERWANY);
	err = errno;
	if (shmid1 == SYSERROR) {
		test_fail++;
		xx_rpt(DELETION);
		in_rpt("shmget() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	shmidptr1 = shmat(shmid1, NULL, NONE);
	err = errno;
	origptr = shmidptr1;
	if ((shmidptr1 == (void*)0) || (shmidptr1 == (void*)SYSERROR)) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmat(shmid1, NULL, 0) of existing shared memory segment failed, returned %d with errno %d (%s)", (int) shmidptr1, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	(void)strcpy((char *)shmidptr1, STRING1);  

	(void)sleep(5);	/* race condition */
	(void)kill (tet_child, SIGUSR1);

	PATH_XS_RPT(2);

	(void)shmdt(origptr);

	DBUG_VOID_RETURN;
}

private void	/* This guy waits for his signal from the parent and then */
ch_t1()		/* reads in the data from the shm segment */
{
	int shmid_ch1;
	void *shmidptr1, *origptr;
	int err;
	int i;
	int pathok = setupok;
	char buf[BUFSIZ];

	DBUG_ENTER("ch_t1");

        (void)vsignal(SIGUSR1, stub);
	(void)pause();

	errno = NOERROR;
	shmid_ch1 = shmget(Key1, SEGSIZE, MODERWANY);
	err = errno;
	if (shmid_ch1 == SYSERROR) {
		xx_rpt(DELETION);
		test_fail++;
		in_rpt("shmget() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	shmidptr1 = shmat(shmid_ch1, NULL, NONE);
	origptr = shmidptr1;
	err = errno;
	if ((shmidptr1 == (void*)0) || (shmidptr1 == (void*)SYSERROR)) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmat(shmid1, NULL, 0) of existing shared memory segment failed, returned %d with errno %d", (int) shmidptr1, err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	i = 0;
	while (((char *)shmidptr1)[i] != '\0' && i<SEGSIZE) {
                buf[i] = ((char *)shmidptr1)[i];
                i++;
        }
        buf[i] = ((char *)shmidptr1)[i]; 

	if (strcmp(buf, STRING1) != NOERROR) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected \"%s\" in shared memory segment, got \"%s\"\n", STRING1, buf);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void)shmdt(origptr);

	if (pathok == 3) {
		DBUG_VOID_RETURN;
	}

	DBUG_EXIT(PATH_ERR_EXIT);
}

private void
stub()
{
	return;
}

private void
test2()
{
	int shmid1;
	struct shmid_ds shmid_ds1;
	int shmid2;
	struct shmid_ds shmid_ds2;
	void *addr, *shmaddr;
	void *addrmod;
	int pathok = setupok;
	int err;
	int rval;
	
	DBUG_ENTER("test2");

	/*
	 * The test strategy involves obtaining a memory segment
	 * of (2 * SHMLBA) bytes.  If SHMLBA is so large that this
	 * would be unreasonable, then give an Untested result.
	 */
	if (SHMLBA > 1024L * 1024L)
	{
		xx_rpt(UNTESTED);
		in_rpt("test would require unreasonable resources, as");
		in_rpt("SHMLBA is greater than 1Mb");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	Key1 = ftok(TOKFILE, ID1);
	err = errno;
	if (Key1 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID1, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	errno = NOERROR;
	shmid1 = shmget(Key1, (2 * SHMLBA), MODERWANY|IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid1;
	if (shmid1 == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmget(key, 2*SHMLBA, mode|IPC_CREAT) failed with err %d", err);
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;


	errno = NOERROR;
	Key2 = ftok(TOKFILE, ID2);
	err = errno;
	if (Key2 == SYSERROR) {
		xx_rpt(DELETION);
		test_fail++;
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID2, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	errno = NOERROR;
	shmid2 = shmget(Key2, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	g_shmids[no_tests+tet_thistest-1] = shmid2;
	if (shmid2 == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmget() with IPC_CREAT set failed with err %d", err);
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	addr = shmat(shmid1, (void *)NULL, 0);
	err = errno;
	if ((int) addr == SYSERROR || addr == NULL) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmat(shmid1, NULL, 0) failed, returned 0x%lx with errno %d (%s)", (long)addr, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = shmdt(addr);
	err = errno;
	if (rval == SYSERROR) {
		xx_rpt(DELETION);
		test_fail++;
		in_rpt("shmdt() returned -1 with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure request address is not already rounded, and will be
	   (after rounding) within previously attached segment */
	if (((ptrdiff_t) addr % SHMLBA) == 0)
		addr = (void *)((char *)addr + SHMLBA/2);
	addr = (void *)((char *)addr + SHMLBA);

	errno = NOERROR;
	shmaddr = shmat(shmid2, addr, SHM_RND);
	err = errno;
	if ((int) shmaddr == SYSERROR || shmaddr == NULL) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmat(shmid2, 0x%lx, SHM_RND) failed, returned 0x%lx with errno %d (%s)",
			(long)addr, (long)shmaddr, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* watch out for negative value after (ptrdiff_t) cast */
	/* (this is assumed to be an oversight in XSH4) */
	if ((ptrdiff_t) addr >= 0)
		addrmod = (void *)((char *)addr - ((ptrdiff_t) addr % SHMLBA));
	else
		addrmod = (void *)((char *)addr - ((unsigned long) addr % SHMLBA));
		/* should be OK as long as SHMLBA is < ULONG_MAX */

	if (shmaddr != addrmod) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmat(shmid2, addr, 0%o | SHM_RND) returned address 0x%lx, expected address 0x%lx with errno %d (%s)",
			MODERWANY, (long) shmaddr, (long) addrmod,
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void)shmdt(shmaddr);
	
	(void)shmctl(shmid1, IPC_RMID, &shmid_ds1);
	(void)shmctl(shmid2, IPC_RMID, &shmid_ds2);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(9);
	
	DBUG_VOID_RETURN;
}


private void	/* We do a shmat() to obtain a valid address then */
test3()		/* try re-attach at the same address and test that */
		/* it really works */
{
	int shmid3;
	struct shmid_ds shmid_ds3;
	void *addr, *shmaddr;
	int err;
	int rval;
	int pathok = setupok;

	DBUG_ENTER("test3");

	test_deleted=0;
	test_fail=0;

	errno = NOERROR;
	Key3 = ftok(TOKFILE, ID3);
	err = errno;
	if (Key3 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID3, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	errno = NOERROR;
	shmid3 = shmget(Key3, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid3;
	if (shmid3 == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmget() with IPC_CREAT set failed with err %d", err);
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	addr = shmat(shmid3, (void *)NULL, MODERWANY);
	err = errno;
	if ((int) addr == SYSERROR || addr == NULL) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmat(shmid3, NULL, 0%o) failed, returned %d with errno %d (%s)", MODERWANY, (int)addr, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = shmdt(addr);
	err = errno;
	if (rval == SYSERROR) {
		xx_rpt(DELETION);
		test_fail++;
		in_rpt("shmdt(shmid3) returned -1 with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	shmaddr = shmat(shmid3, addr, MODERWANY);
	err = errno;
	if ((int) shmaddr == SYSERROR || shmaddr == NULL) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmat(shmid3, addr, 0%o) failed, returned %d with errno %d (%s)", MODERWANY, (int)shmaddr, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((int)shmaddr != (int)addr) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmat(shmid3, addr, 0%o), where SHM_RND is not set", MODERWANY);
		in_rpt("returned address 0x%lx, expected address 0x%lx, errno %d (%s)", (long)shmaddr, (long)addr, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void)shmdt(shmaddr);
	(void)shmctl(shmid3, IPC_RMID, &shmid_ds3);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(6);
	
	DBUG_VOID_RETURN;
}

private void	/* Test that if the calling process has read perms */
test4()		/* it can attach a segment for reading */
{
        struct shmid_ds shmid_ds4;
        int shmid4;
        int pathok = setupok;
        int err = NOERROR;
	int rval;

	DBUG_ENTER("test4");

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	Key4 = ftok(TOKFILE, ID4);
	err = errno;
	if (Key4 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID4, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	shmid4 = shmget(Key4, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid4;
	if (shmid4 == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmget() with IPC_CREAT set failed with err %d", err);
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	rval = cppair(ch_t4, pa_t4, WAITTIME, NOK|PATH_ERR_EXIT);
	if (rval&NOK) {
		/* failure/deletion was already reported in pa_t4 or ch_t4 */
		test_deleted++;
	}
	else if ((rval&PATH_ERR_EXIT) == 0)
		PATH_TRACE;

	(void)shmctl(shmid4, IPC_RMID, &shmid_ds4);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void	/* This guy maps in the shm segment and writes to it and */
pa_t4()		/* then signals the child when he's done. */
{
	int shmid4;
	int err;
	void *shmidptr4, *origptr;
	int pathok = 0;

	DBUG_ENTER("pa_t4");

	errno = NOERROR;
	shmid4 = shmget(Key4, SEGSIZE, MODERWANY);
	err = errno;
	if (shmid4 == SYSERROR) {
		xx_rpt(DELETION);
		test_fail++;
		in_rpt("shmget() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	shmidptr4 = shmat(shmid4, NULL, NONE);
	err = errno;
	origptr = shmidptr4;
	if ((shmidptr4 == (void*)0) || (shmidptr4 == (void*)SYSERROR)) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmat(shmid4, NULL, 0) of existing shared memory segment failed, returned %d with errno %d (%s)", (int)shmidptr4, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	(void)sprintf((char *)shmidptr4, STRING4);  

	(void)sleep(5);	/* race condition */
	(void)kill (tet_child, SIGUSR1);

	PATH_XS_RPT(2);

	(void)shmdt(origptr);

	DBUG_VOID_RETURN;
}

private void	/* This guy waits for his signal from the parent and then */
ch_t4()		/* attaches the segment SHM_RDONLY and tries to read */
{
	int shmid_ch4;
	void *shmidptr4, *origptr;
	int err;
	int i;
	int pathok = 0;
	char buf[BUFSIZ];

	DBUG_ENTER("ch_t4");
        (void)vsignal(SIGUSR1, stub);
	(void)pause();

	errno = NOERROR;
	shmid_ch4 = shmget(Key4, SEGSIZE, MODERDONLY);
	err = errno;
	if (shmid_ch4 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("shmget() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	shmidptr4 = shmat(shmid_ch4, NULL, SHM_RDONLY);
	origptr = shmidptr4;
	err = errno;
	if ((shmidptr4 == (void*)0) || (shmidptr4 == (void*)SYSERROR)) {
		xx_rpt(FAILURE);
		in_rpt("shmat(shmid4, NULL, SHM_RDONLY) of existing shared memory segment failed, returned %d with errno %d (%s)", (int)shmidptr4, err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	i = 0;
	while (((char *)shmidptr4)[i] != '\0' && i<SEGSIZE) {
                buf[i] = ((char *)shmidptr4)[i];
                i++;
        }
        buf[i] = ((char *)shmidptr4)[i]; 

	if (strcmp(buf, STRING4) != NOERROR) {
		xx_rpt(FAILURE);
		in_rpt("expected \"%s\" in shared memory segment, got \"%s\"\n", STRING4, buf);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void) shmdt(origptr);

	if (pathok == 3)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}


private void	/* Attatch w/o SHM_RDONLY set and make sure we can */
test5()		/* read and write from segment */
{
	int shmid5;
	struct shmid_ds shmid_ds5;
	void *shmidptr5;
	int rval;
	int err;
	int i;
	int pathok = setupok;

	DBUG_ENTER("test5");

	test_fail = 0;
	test_deleted = 0;

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	Key5 = ftok(TOKFILE, ID5);
	err = errno;
	if (Key5 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID5, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	errno = NOERROR;	
	shmid5 = shmget(Key5, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid5;
	if (shmid5 == SYSERROR) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmget(Key5, %d, 0%o) returned -1 with errno %d (%s)", SEGSIZE, MODERWANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	shmidptr5 = shmat(shmid5, NULL, 0);
	err = errno;
	if ((shmidptr5 == (void*)0) || (shmidptr5 == (void*)SYSERROR)) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmat(shmid5, NULL, 0) of existing shared memory segment failed, returned %d with errno %d (%s)", (int)shmidptr5, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	errno = NOERROR;
	rval = shmctl(shmid5, IPC_STAT, &shmid_ds5);
	err = errno;
	if (rval == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmctl returned -1 with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((shmid_ds5.shm_perm.mode & MODEMASK) != MODERWANY ) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds5.shm_perm.mode 0%o returned 0%o\n", (MODERWANY & MODEMASK), shmid_ds5.shm_perm.mode);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < SEGSIZE; i++)
		/* signal may occur here if segment is not writeable */
		((char *)shmidptr5)[i] = i & CHAR_MAX; /* mask off character part */

	for (i = 0; i < SEGSIZE; i++)
	{
		/* signal may occur here if segment is not readable */
		if (((char *)shmidptr5)[i] != (char)(i & CHAR_MAX))
		{
			if (!test_fail++)
				xx_rpt(FAILURE); 
			in_rpt("Data in shared memory segment does not match expected contents");
			break;
		}
		else
			PATH_TRACE;
	}

	(void)shmctl(shmid5, IPC_RMID, &shmid_ds5);

	if (!test_deleted && !test_fail)
		PATH_XS_RPT(5 + SEGSIZE);

	DBUG_VOID_RETURN;
}

private void	/* Create shm segment with read-only perms and then have */
test6()		/* have child attempt to attach segment for writing */
{
	int pathok = setupok;
	int shmid6;
	int rval, err;
	struct shmid_ds shmid_ds6;

	DBUG_ENTER("test6");

	test_fail = 0;
	test_deleted = 0;

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	Key6 = ftok(TOKFILE, ID6);
	err = errno;
	if (Key6 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID6, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	errno = NOERROR;
	shmid6 = shmget(Key6, SEGSIZE, MODERDONLY | IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid6;
	if (shmid6 == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmget(Key6, %d, 0%o|IPC_CREAT) returned -1 with errno %d (%s)", SEGSIZE, MODERDONLY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = cppair(ch_t6, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);
	if (rval&NOK) {
		/* failure/deletion was already reported in pa_t6 or ch_t6 */
		test_deleted++;
	}
	else if ((rval&PATH_ERR_EXIT) == 0)
		PATH_TRACE;

	(void)shmctl(shmid6, IPC_RMID, &shmid_ds6);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void	/* Now try to attach read-only segment for writing */
ch_t6()
{
	int ch_shmid;
	int err;
	int pathok = setupok;
	void *shmidptr6;

	DBUG_ENTER("ch_t6");

	pathok = 0;
	errno = NOERROR;
	ch_shmid = shmget(Key6, SEGSIZE, MODERDONLY);
	err = errno;
	if (ch_shmid == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("shmget(Key6, SEGSIZE, 0%o) of existing shared memory segment failed - returned errno %d (%s)\n", MODERDONLY, err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	shmidptr6 = shmat(ch_shmid, NULL, MODERWANY);
	err = errno;
	if ((shmidptr6 != (void*)SYSERROR) || (err != EACCES)) {
		xx_rpt(FAILURE);
		in_rpt("shmat(shmid, NULL, 0%o) of existing shared memory segment returned unexpected value", MODERWANY);
		valrpt(SYSERROR,(int)shmidptr6,EACCES,err);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok == 2) 
		DBUG_VOID_RETURN;
	
	DBUG_EXIT(PATH_ERR_EXIT);
}


private void	/* Create a shared memory segment, remove it and then */
test7()		/* try to attach - this should fail */
{
	int shmid;
	int err;
	int pathok = setupok;
	struct shmid_ds shmid_ds7;
	void *shmidptr7;

	DBUG_ENTER("test7");

	test_fail = 0;
	test_deleted = 0;

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	Key7 = ftok(TOKFILE, ID7);
	err = errno;
	if (Key7 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID7, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	errno = NOERROR;
	shmid = shmget(Key7, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid;
	if (shmid == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmget() with IPC_CREAT set failed with err %d", err);
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	if (shmctl(shmid, IPC_RMID, &shmid_ds7) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmctl(shmid, IPC_RMID, &shmid_ds) failed with errno %d (%s)\n", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	shmidptr7 = shmat(shmid, NULL, MODERWANY);
	err = errno;
	if ((shmidptr7 != (void*)SYSERROR) || (err != EINVAL)) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmat(shmid, NULL, 0%o) of non-existent shared memory segment returned unexpected value", MODERWANY);
		valrpt(SYSERROR,(int)shmidptr7,EINVAL,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(4);
	
	DBUG_VOID_RETURN;
}

private void
test8()
{
	int 	err, shmid, shmflg;
	void	*shmaddr;
	void 	*shmidptr;
	int	pathok = setupok;

	DBUG_ENTER("test8");

	if(tsup)
	{
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	shmidptr = shmat(shmid, (void *)shmaddr, shmflg);
	err = errno;

	if((shmidptr != (void*)SYSERROR) || err != ENOSYS)
	{
		xx_rpt(FAILURE);

		if(shmidptr != (void *)SYSERROR)
		{
			in_rpt("shmat() did not return expected value");
			in_rpt("Expected return value -1");
			in_rpt("Actual   return value %d", (int)shmidptr);
		}

		if(err != ENOSYS)
		{
			in_rpt("shmat() did not set errno correctly");
			in_rpt("Expected %d (ENOSYS)", ENOSYS);
			in_rpt("Actual   %d (%s)", err, errname(err));
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}
