/*
*	SCCS:  @(#)  XPG4.os/ipc/shmctl/shmctl.c Rel 4.4.1     (06/09/97)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)  XPG4.os/ipc/shmctl/shmctl.c Rel 4.4.1     (06/09/97)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.shmctl

PROJECT:	XPG4 Validation suite

DESCRIPTION:
	Testset for the shmctl() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Tony Rems
DATE CREATED:	Thu Nov 5 2:15:42 BST 1990
MODIFICATIONS:	
		Matt Newman, Wed Sep 11 11:50:28 BST 1991
			XPG4 Intergration

		Peter Skan, 7 Feb 1992
			Added IPC and ACCESS privileges
			Remove privileges with unsetprv() where necessary

		Geoff Clare, 29 Apr 1992
			Changes for new type of shm_nattch

		Neil Moses, 8 May 1992.
			Check that interface is supported.
			Added TEST For ENOSYS if interface not supported.

		Neil Moses, 4 June 1992.
			Use sysconf() to check if interface supported.

		Geoff Clare, 14-16 Sept 1992
			Remove unspecified ipc_perm member "key" from
			shm_init() and shm_test().  Change childpid from
			int to pid_t.  Change mode to RWANY in ch2_t7
			so the IPC_STAT is done with read permission.

		Geoff Clare, 4 March 1994
			Arguments to SETID() were swapped in ch_t14.

		Geoff Clare, 22 Sept 1994
			Fix path tracing error in test 17.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<sys/ipc.h>
#include 	<sys/shm.h>
#include	<sys/stat.h>
#include	<signal.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<errno.h>
#include	<tet_api.h>
#include 	<sysdep.h>

#ifdef UNDEF_MACROS
#undef shmctl
#endif


/* Some useful macros */
#define CPPAIR(child, parent, wait_time, exitcodes) { \
	errno = NOERROR; \
	rval =  cppair(child, parent, wait_time, exitcodes); \
	if (rval&NOK) { \
		test_deleted++; \
	} \
	else if ((rval&PATH_ERR_EXIT) == 0) \
		PATH_TRACE; }

#define	KEY(key, id) { \
	errno = NOERROR; \
	key = ftok(TOKFILE, id); \
	err = errno; \
	if (key == SYSERROR) { \
		xx_rpt(DELETION); \
		test_deleted++; \
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, id, err, errname(err)); \
		DBUG_VOID_RETURN; \
	} \
	else \
		PATH_TRACE; }

#define	GSETID(uid, gid)	{ \
	(void) setgid((gid_t)gid); \
	(void) setuid((uid_t)uid); \
	if (geteuid() != (uid_t)uid) { \
                xx_rpt(DELETION); \
		(void)kill(childpid, SIGUSR2);	\
		DBUG_EXIT(NOK); \
        } \
	else \
		PATH_TRACE; \
\
	if (unsetprv(PRV_SETID) != 0) { \
                xx_rpt(DELETION); \
		(void)kill(childpid, SIGUSR2);	\
		DBUG_EXIT(NOK); \
        } \
	else \
		PATH_TRACE; \
\
	if (unsetprv(PRV_IPC) != 0) { \
                xx_rpt(DELETION); \
		(void)kill(childpid, SIGUSR2);	\
		DBUG_EXIT(NOK); \
        } \
	else \
		PATH_TRACE; \
\
	if (unsetprv(PRV_ACCESS) != 0) { \
                xx_rpt(DELETION); \
		(void)kill(childpid, SIGUSR2);	\
		DBUG_EXIT(NOK); \
        } \
	else \
		PATH_TRACE; }


#define	SETID(uid, gid)	{ \
	(void) setgid((gid_t)gid); \
	(void) setuid((uid_t)uid); \
	if (geteuid() != (uid_t)uid) { \
                xx_rpt(DELETION); \
                in_rpt("setuid() did not set effective user ID to %d", uid); \
		DBUG_EXIT(NOK); \
        } \
	else \
		PATH_TRACE; \
\
	if (unsetprv(PRV_SETID) != 0) { \
                xx_rpt(DELETION); \
                in_rpt("unsetprv(PRV_SETID) failed"); \
		DBUG_EXIT(NOK); \
        } \
	else \
		PATH_TRACE; \
\
	if (unsetprv(PRV_IPC) != 0) { \
                xx_rpt(DELETION); \
                in_rpt("unsetprv(PRV_IPC) failed"); \
		DBUG_EXIT(NOK); \
        } \
	else \
		PATH_TRACE; \
\
	if (unsetprv(PRV_ACCESS) != 0) { \
                xx_rpt(DELETION); \
                in_rpt("unsetprv(PRV_ACCESS) failed"); \
		DBUG_EXIT(NOK); \
        } \
	else \
		PATH_TRACE; }


#define	SHMSTAT(shm_id, dat_str) { \
	errno = NOERROR; \
	shm_init(&dat_str); \
	if (shmctl(shm_id, IPC_STAT, &dat_str) == SYSERROR) { \
		err = errno; \
		xx_rpt(FAILURE); \
		test_fail++; \
		in_rpt("shmctl(shmid, IPC_STAT, &shmid_ds) failed with errno %d (%s)", err, errname(err)); \
		DBUG_EXIT(NOK); \
	} \
	else \
		PATH_TRACE; \
	if (shm_test(&dat_str) == SYSERROR) { \
		xx_rpt(FAILURE); \
		test_fail++; \
		in_rpt("shmctl(shmid, IPC_STAT, &shmid_ds) did not set members of ds"); \
		DBUG_EXIT(NOK); \
	} \
	else \
		PATH_TRACE; }

#define	SHMSET(shm_id, dat_str) { \
	errno = NOERROR; \
	if (shmctl(shm_id, IPC_SET, &dat_str) == SYSERROR) { \
		err = errno; \
		xx_rpt(DELETION); \
		test_fail++; \
		in_rpt("shmctl(shmid, IPC_SET, &shmid_ds) failed with errno %d (%s)", err, errname(err)); \
		DBUG_EXIT(NOK); \
	} \
	else \
		PATH_TRACE; }

#define	SHMRM(key, mode) { struct shmid_ds dat_str; \
			(void) shmctl(shmget(key,SEGSIZE,mode), IPC_RMID, &dat_str); }

#define	SHMCREAT(key, flags, shm_id) { \
	errno = NOERROR; \
	shm_id = shmget(key, SEGSIZE, flags|IPC_CREAT); \
	err = errno; \
	if (shm_id == SYSERROR) { \
		xx_rpt(DELETION); \
		in_rpt("shmget() with IPC_CREAT set failed with err %d (%s)", err, errname(err)); \
		DBUG_EXIT(NOK); \
	} \
	else  \
		PATH_TRACE; }

#define	SHMGET(key, flags, shm_id) { \
	errno = NOERROR; \
	shm_id = shmget(key, SEGSIZE, flags); \
	err = errno; \
	if (shm_id == SYSERROR) { \
		xx_rpt(DELETION); \
		in_rpt("shmget() failed with err %d (%s)", err, errname(err)); \
		DBUG_EXIT(NOK); \
	} \
	else  \
		PATH_TRACE; }

#define		WAITTIME	(5 * SPEEDFACTOR)
#define 	NO_TESTS	17	/* number of tests in testset */
#define		MODEANY		((mode_t)(S_IRWXU|S_IRWXG|S_IRWXO))
#define		MODERDALL	((mode_t)(S_IRUSR|S_IRGRP|S_IROTH))
#define		MODEMASK	((mode_t)MODEANY)
#define		MODEURDONLY	((mode_t)S_IRUSR)
#define		RW_U            (S_IRUSR|S_IWUSR)
#define		RW_G            (S_IRGRP|S_IWGRP)
#define		RW_O            (S_IROTH|S_IWOTH)
#define		MODERWANY	((mode_t) (RW_U|RW_G|RW_O))
#define		MODERWUSR	((mode_t) (RW_U))
#define		MODERWGRP	((mode_t) (RW_G))
#define		MODERWOTH	((mode_t) (RW_O))
#define		PATH_ERR_EXIT	0x04
#define		SEGSIZE		512 	     	/* Use 512 byte shm segments */
#define		MODENONE	((mode_t)0)
#define		NONE		0
#define		NOK		02
#define		RD_FD		0
#define		WR_FD		1
#define		ID1		11		/* id's for use with ftok */
#define		ID2		22	
#define		ID3		33
#define		ID4		44
#define		ID5		55
#define		ID6		66
#define		ID7		77
#define		ID8		88
#define		ID9		99
#define		ID10		10
#define		ID11		21
#define		ID12		12
#define		ID13		13
#define		ID14		14
#define		ID15		15
#define		ID16		16

#define		TOKFILE		"./shmctlfile" 	/* file for use with ftok */

#define 	STRING		"test string"	/* string to write into shm */

int		no_tests = NO_TESTS;

extern	char *errname();
extern	void xx_all();

extern	int     vguid1, vggid1, vguid0, vggid0;

#ifndef ftok
extern	key_t	ftok();
#endif
extern	void	*memset();
extern	int	crfile(), cppair(), uids(), setprv();
extern	void	valrpt();
extern	void	(*vsignal())();
extern 	time_t	time();
extern	char	*errname();
private void	stub(), error(), tidy(), setup(), test1(), ch_t1(), test2(),
		ch1_t2(), ch2_t2(), test3(), pa_t3(), ch_t3(), gch_t3(),
		test4(), pa_t4(), ch_t4(), gch_t4(), test5(), pa_t5(), ch_t5(),
		gch_t5(), test6(), ch1_t6(), ch2_t6(), test7(), ch1_t7(),
		ch2_t7(), test8(), ch1_t8(), ch2_t8(), test9(), ch1_t9(),
		ch2_t9(), test10(), ch1_t10(), ch2_t10(), test11(), ch1_t11(),
		ch2_t11(), test12(), ch1_t12(), ch2_t12(), test13(), ch_t13(),
		gch_t13(), pa_t13(), test14(), ch_t14(), test15(), ch_t15(),
		gch_t15(), pa_t15(), test16(), ch_t16(), gch_t16(), pa_t16(),
		shm_init();
private int	shm_test();

private void test17();

public  void    (*realfuncs[]) () = { test1,  test2,  test3,  test4,  test5,
                                      test6,  test7,  test8,  test9,  test10,
                                      test11, test12, test13, test14, test15,
                                      test16, NULL };

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = tidy;

private void	twrap();

public	struct tet_testlist tet_testlist[] = {
	twrap,	 1,
	twrap,	 2,
	twrap,	 3,
	twrap,	 4,
	twrap,	 5,
	twrap,	 6,
	twrap,	 7,
	twrap,	 8,
	twrap,	 9,
	twrap, 10,
	twrap, 11,
	twrap, 12,
	twrap, 13,
	twrap, 14,
	twrap, 15,
	twrap, 16,
	test17, 17,
	NULL,	 0
};

/*public	int	tet_thistest;*/
private	int	setupok = -1;
private	int	tsup = 0;
private pid_t	childpid;

private	int	test_fail;
private	int	test_deleted;
private key_t	Key1, Key2, Key3, Key4, Key5, Key6, Key7, Key8, Key9, Key10,
		Key11, Key12, Key13, Key14, Key15, Key16;

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if(!tsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("sysconf(_SC_XOPEN_SHM) returned -1");
	}
	else
		if (tet_thistest >= 1 && tet_thistest <= no_tests)
			(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
setup()	
{
	int	pathok = 0;
	int 	i;

	DBUG_ENTER("setup");

	errno = 0;
	if(sysconf(_SC_XOPEN_SHM) == -1)
	{
		if (errno != 0)
		{
			xx_all(DELETION, "sysconf(_SC_XOPEN_SHM) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		tsup = 0;
	}
	else
	{
		tsup = 1;
		PATH_TRACE;
	}
	
	if (crfile(TOKFILE, MODEANY, (int)getuid(), (int)getgid()) == SYSERROR) {
		for (i = 1; i <= NO_TESTS; i++) {
			tet_delete(i, "Could not create reference file for keys.");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void)uids();

	if (setprv(PRV_SETID) == SYSERROR ||
	    setprv(PRV_IPC) == SYSERROR ||
	    setprv(PRV_ACCESS) == SYSERROR)
	{
		for (i = 1; i <= NO_TESTS; i++) {
			tet_delete(i, "setprv(PRV_SETID) failed, process does not have appropriate privileges");
		}
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(3);

	DBUG_VOID_RETURN;
}

		
private	void
tidy()
{
	DBUG_ENTER("tidy");
	(void) unlink(TOKFILE);
	DBUG_VOID_RETURN;
}


private void	/* Check that shmctl with IPC_STAT fills in the buf */
test1()		/* when euid is same as uid of owner */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test1");

	test_deleted = 0;

	KEY(Key1, ID1);

	CPPAIR(ch_t1, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_deleted)
		PATH_XS_RPT(2);

	SHMRM(Key1, MODEURDONLY);

	DBUG_VOID_RETURN;

}


private void
ch_t1()
{
	struct shmid_ds shmid_ds1;
	int rval;
	int shmid1;
	int pathok = 0;
	int err = NOERROR;

	DBUG_ENTER("ch_t1");

	SHMCREAT(Key1, MODERWANY, shmid1);

	shmid_ds1.shm_perm.uid = vguid0;
	shmid_ds1.shm_perm.gid = vggid0;
	shmid_ds1.shm_perm.mode = MODEURDONLY;

	SHMSET(shmid1, shmid_ds1);

	SETID(vguid0, vggid0);

	SHMSTAT(shmid1, shmid_ds1);	/* pathok += 2 */

	if (pathok == 8)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);

}


private void	/* Check that shmctl with IPC_STAT fills in the buf */
test2()		/* when euid is same as uid of creator */
{
	int rval;
	int err = NOERROR;
	int pathok = setupok;

	DBUG_ENTER("test2");

	test_deleted = 0;

	KEY(Key2, ID2);

	CPPAIR(ch1_t2, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	CPPAIR(ch2_t2, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_deleted)
		PATH_XS_RPT(3);

	SHMRM(Key2, MODERWUSR);

	DBUG_VOID_RETURN;

}

private void
ch1_t2()
{
	int rval;
	struct shmid_ds shmid_ds2;
	int shmid2;
	int pathok = 0;
	int err = NOERROR;

	DBUG_ENTER("ch1_t2");

	SETID(vguid0, vggid1);

	SHMCREAT(Key2, MODERWANY, shmid2);

	shmid_ds2.shm_perm.uid = (uid_t) vguid1;
	shmid_ds2.shm_perm.gid = (gid_t) vggid1;
	shmid_ds2.shm_perm.mode = MODERWUSR;

	SHMSET(shmid2, shmid_ds2);

	if (pathok != 6)
		DBUG_EXIT(PATH_ERR_EXIT);

	DBUG_VOID_RETURN;
}

private void
ch2_t2()
{
	int rval;
	struct shmid_ds shmid_ds2;
	int shmid2;
	int pathok = 0;
	int err = NOERROR;

	DBUG_ENTER("ch2_t2");

	SETID(vguid0, vggid0);

	SHMGET(Key2, MODERWUSR, shmid2);

	SHMSTAT(shmid2, shmid_ds2);	/* pathok += 2 */

	if (pathok != 7)
		DBUG_EXIT(PATH_ERR_EXIT);

	DBUG_VOID_RETURN;
}

private void	/* Test that shmctl with IPC_STAT set fills in shmid_ds */
test3()		/* when euid of caller != uid of owner but egid matches */
		/* one of gids of owner and perms are set for group */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test3");

	test_fail = 0;
	test_deleted = 0;

	KEY(Key3, ID3);

	CPPAIR(ch_t3, pa_t3, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(2);

	SHMRM(Key3, MODERWGRP);

	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int pathok = 0;	
	int rval;
	
	DBUG_ENTER("pa_t3");

	childpid = tet_child;

	rval = cppair(gch_t3, NULLFN, WAITTIME, NOK);
	if ((rval & NOK) == NOK) 
		test_deleted++;
	else
		PATH_TRACE;
	
	/*if (!test_deleted) */
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

	
private void	
gch_t3()	
{
	int rval;
	int shmid3;
	int err;
	int pathok = 0;

	DBUG_ENTER("gch_t3");

	test_fail = 0;
	test_deleted = 0;

	GSETID(vguid0,vggid0);

	errno = NOERROR; 
	shmid3 = shmget(Key3, SEGSIZE, MODERWGRP|IPC_CREAT);
	err = errno;
	if (shmid3 == SYSERROR) { 
		xx_rpt(DELETION);
		in_rpt("shmget() with IPC_CREAT set failed with err %d (%s)", err, errname(err));
		(void)kill(childpid, SIGUSR2);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void)sleep(5);	/* possible race condition */
	(void) kill (childpid, SIGUSR1);

	PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

/* This guy waits for his signal from the parent and then */
/* then reads in the data from the shm segment */
private void
ch_t3()
{
	int rval;
	int shmid_ch3;
	struct shmid_ds shmid_ds3;
	int err;
	int pathok = 0;

	DBUG_ENTER("ch_t3");

	test_fail = 0;
	test_deleted = 0;

	(void) vsignal(SIGUSR1, stub);
	(void) vsignal(SIGUSR2, error);
	(void) pause();

	SETID(vguid1, vggid0);

	shmid_ch3 = shmget(Key3, SEGSIZE, MODERWGRP);
	err = errno;
	if (shmid_ch3 == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("shmget() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		test_fail++;
		DBUG_EXIT(NOK);
	}
	else 
		PATH_TRACE;

	SHMSTAT(shmid_ch3, shmid_ds3);	/* inc pathok by 2! */

	if (pathok == 7)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}

private void	/* This ain't nothin' but a stub */
stub()	
{
	return;
}

private void
error()
{
	DBUG_ENTER("error");
	DBUG_EXIT(NOK);
}

private void	/* Test that shmctl with IPC_STAT set fills in shmid_ds */
test4()		/* when euid of caller != uid of creator but egid matches */
		/* one of gids of creator and perms are set for group */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test4");

	test_fail = 0;
	test_deleted = 0;

	KEY(Key4, ID4);

	CPPAIR(ch_t4, pa_t4, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_deleted)
		PATH_XS_RPT(2);

	SHMRM(Key4, MODERWGRP);

	DBUG_VOID_RETURN;
}

private void
pa_t4()
{
	int pathok = 0;	
	int rval;
	
	DBUG_ENTER("pa_t4");

	test_deleted = 0;

	childpid = tet_child;

	rval = cppair(gch_t4, NULLFN, WAITTIME, NOK);
	if ((rval & NOK) == NOK) 
		test_deleted++;
	else
		PATH_TRACE;
	
	(void)sleep(5);	/* possible race condition */
	if (kill (childpid, SIGUSR1) < 0)
		in_rpt("pa_t4: kill: %s", errname(errno));
	
	if (!test_deleted)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;


}

private void	
gch_t4()	
{
	int rval;
	struct shmid_ds shmid_ds4;
	int shmid4;
	int err;
	int pathok = 0;

	DBUG_ENTER("gch_t4");

	test_fail = 0;
	test_deleted = 0;

	GSETID(vguid1,vggid0);

	errno = NOERROR; 
	shmid4 = shmget(Key4, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	if (shmid4 == SYSERROR) { 
		xx_rpt(DELETION);
		in_rpt("shmget() with IPC_CREAT set failed with err %d (%s)", err, errname(err));
		(void)kill(childpid, SIGUSR2);
		DBUG_EXIT(NOK);
	} else
		PATH_TRACE;

	shmid_ds4.shm_perm.gid = (gid_t) vggid1;
	shmid_ds4.shm_perm.uid = (uid_t) vguid1;
	shmid_ds4.shm_perm.mode = MODERWGRP;
	SHMSET(shmid4, shmid_ds4);

	if (pathok == 6)
		DBUG_VOID_RETURN;

	DBUG_EXIT(NOK);
}

/* This guy waits for his signal from the parent and then */
/* then reads in the data from the shm segment */
private void
ch_t4()
{
	int rval;
	int shmid4;
	struct shmid_ds shmid_ds4;
	int err;
	int pathok = 0;


	DBUG_ENTER("ch_t4");
	(void) vsignal(SIGUSR1, stub);
	(void) vsignal(SIGUSR2, error);
	(void) pause();

	SETID(vguid0, vggid0);

	errno = NOERROR;
	shmid4 = shmget(Key4, SEGSIZE, MODERWGRP);
	err = errno;
	if (shmid4 == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("shmget() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		test_fail++;
		DBUG_EXIT(NOK);
	} else 
		PATH_TRACE;

	SHMSTAT(shmid4, shmid_ds4);	/* pathok += 2 */

	if (pathok == 7)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}

private void	/* Test that when euid & egid != uid and gid of owner or */
test5()		/* creator but permissions are set for other, caller can */
		/* do a shmctl with IPC_STAT set */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test5");

	test_fail = 0;
	test_deleted = 0;

	KEY(Key5, ID5);

	CPPAIR(ch_t5, pa_t5, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(2);

	SHMRM(Key5, MODERWANY);

	DBUG_VOID_RETURN;
}

private void
pa_t5()
{
	int pathok = 0;	
	int rval;
	
	DBUG_ENTER("pa_t5");

	test_fail = 0;
	test_deleted = 0;

	childpid = tet_child;

	rval = cppair(gch_t5, NULLFN, WAITTIME, NOK);
	if ((rval & NOK) == NOK) 
		test_deleted++;
	else
		PATH_TRACE;

	if (!test_deleted)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

	
private void	
gch_t5()	
{
	int rval;
	int shmid5;
	int err;
	int pathok = 0;

	DBUG_ENTER("gch_t5");

	test_fail = 0;
	test_deleted = 0;

	GSETID(vguid0,vggid0);

	errno = NOERROR; 
	shmid5 = shmget(Key5, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	if (shmid5 == SYSERROR) { 
		xx_rpt(DELETION);
		in_rpt("shmget() with IPC_CREAT set failed");
		valrpt(NOERROR,shmid5,NOERROR,err);
		(void)kill(childpid, SIGUSR2);
		DBUG_EXIT(NOK);
	} else
		PATH_TRACE;

	(void)sleep(5);	/* possible race condition */
	(void) kill (childpid, SIGUSR1);

	DBUG_VOID_RETURN;
}

/* This guy waits for his signal from the parent and then */
/* then reads in the data from the shm segment */

private void
ch_t5()
{
	int rval;
	int shmid_ch5;
	struct shmid_ds shmid_ds5;
	int err;
	int pathok = 0;


	DBUG_ENTER("ch_t5");
	(void) vsignal(SIGUSR1, stub);
	(void) vsignal(SIGUSR2, error);
	(void) pause();

	SETID(vguid0, vggid0);

	shmid_ch5 = shmget(Key5, SEGSIZE, MODERWANY);
	err = errno;
	if (shmid_ch5 == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("shmget() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		test_fail++;
		DBUG_EXIT(NOK);
	} else 
		PATH_TRACE;

	SHMSTAT(shmid_ch5, shmid_ds5);	/* pathok += 2 */

	if (pathok == 7)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}

private void	/* When the calling process has appropriate privileges */
test6()		/* make sure that he can do an IPC_STAT */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test6");

	test_fail = 0;
	test_deleted = 0;

	if (setprv(PRV_IPC) == SYSERROR)
	{
		if (!test_fail++)
			xx_rpt(DELETION);
		in_rpt("setprv(PRV_IPC) returned error, process does not have appropriate privileges");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	KEY(Key6, ID6);

	CPPAIR(ch1_t6, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	CPPAIR(ch2_t6, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(4);

	SHMRM(Key6, MODEURDONLY);

	DBUG_VOID_RETURN;

}

private void	
ch1_t6()	
{
	int rval;
	int shmid6;
	int err = NOERROR;
	int pathok = 0;

	DBUG_ENTER("ch1_t6");

	SETID(vguid0, vggid0); /* I'm the regular guy */

	SHMCREAT(Key6, MODEURDONLY, shmid6);

	if (test_fail || test_deleted)
		DBUG_EXIT(NOK);

	if (pathok == 5)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}

private void	
ch2_t6()	
{
	int rval;
	struct shmid_ds shmid_ds6;
	int shmid6;
	int euid, egid;
	int err = NOERROR;
	time_t timen;
	int pathok = 0;

	DBUG_ENTER("ch2_t6");

	/* superuser has appropriate privileges */

	SHMGET(Key6, MODEURDONLY, shmid6);

	SHMSTAT(shmid6, shmid_ds6);	/* pathok += 2 */

	euid = vguid0; /* see ch1_t6 */
	egid = vggid0;

	if (shmid_ds6.shm_perm.cuid != (uid_t)euid) {
		if (!test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds6.shm_perm.cuid %d returned %d", euid, (int)shmid_ds6.shm_perm.cuid);
	} else
		PATH_TRACE;
	
	if (shmid_ds6.shm_perm.uid != (uid_t)euid) {
		if (!test_fail && !test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds6.shm_perm.uid %d returned %d", euid, (int)shmid_ds6.shm_perm.uid);
	} else
		PATH_TRACE;

	if (shmid_ds6.shm_perm.cgid != (gid_t)egid) {
		if (!test_fail && !test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds6.shm_perm.cgid %d returned %d", egid, (int)shmid_ds6.shm_perm.cgid);
	} else
		PATH_TRACE;
	
	if (shmid_ds6.shm_perm.gid != (gid_t)egid) {
		if (!test_fail && !test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds6.shm_perm.gid %d returned %d", egid, (int)shmid_ds6.shm_perm.gid);
	} else
		PATH_TRACE;

	if ((shmid_ds6.shm_perm.mode & MODEMASK) != MODEURDONLY ) {
		if (!test_fail && !test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds6.shm_perm.mode 0%o returned 0%o", (MODEURDONLY & MODEMASK), shmid_ds6.shm_perm.mode);
	} else
		PATH_TRACE;

     	if (shmid_ds6.shm_lpid != (pid_t)NONE) {
		if (!test_fail && !test_deleted)
                	xx_rpt(FAILURE);
                test_fail++;
                in_rpt("expected shmid_ds6.shm_lpid of 0 returned %d", (int)shmid_ds6.shm_lpid);
        } else
                PATH_TRACE;


	if (shmid_ds6.shm_nattch != NONE) {
		if (!test_fail && !test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds6.shm_nattch of 0 returned %lu",
			(unsigned long) shmid_ds6.shm_nattch);
	} else
		PATH_TRACE;

	if (shmid_ds6.shm_atime != (time_t)NONE) {
		if (!test_fail && !test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds6.shm_atime of 0 returned %d", shmid_ds6.shm_atime);
	} else
		PATH_TRACE;

	if (shmid_ds6.shm_dtime != (time_t)NONE) {
		if (!test_fail && !test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds6.shm_dtime of 0 returned %d", shmid_ds6.shm_dtime);
	} else
		PATH_TRACE;

	timen = time((time_t) NONE);
	if (!(shmid_ds6.shm_ctime > (time_t)NONE && shmid_ds6.shm_ctime <= (time_t)timen)) {
		if (!test_fail && !test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds6.shm_ctime > 0 and < %d, returned %d", timen, shmid_ds6.shm_ctime);
	} else
		PATH_TRACE;

	if (test_fail || test_deleted)
		DBUG_EXIT(NOK);

	if (pathok == 13)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}

private void	/* When euid of caller matches uid of owner, test if */
test7()		/* caller can do shmctl with IPC_SET flag to set values */
		/* of shmid_ds structure */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test7");

	test_fail = 0;
	test_deleted = 0;

	KEY(Key7, ID7);

	CPPAIR(ch1_t7, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	CPPAIR(ch2_t7, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(3);

	SHMRM(Key7, MODERWUSR);

	DBUG_VOID_RETURN;
}


private void
ch1_t7()
{
	int rval;
	struct shmid_ds shmid_ds7;
	int shmid7;
	int err = NOERROR;
	int pathok = 0;

	DBUG_ENTER("ch1_t7");

	SETID(vguid0, vggid0); 

	SHMCREAT(Key7, MODERWUSR, shmid7);

	shmid_ds7.shm_perm.uid = vguid1;
	shmid_ds7.shm_perm.gid = vggid0;
	shmid_ds7.shm_perm.mode = MODERWUSR;

	errno = NOERROR; 
	if (shmctl(shmid7, IPC_SET, &shmid_ds7) == SYSERROR) { 
		err = errno; 
		xx_rpt(DELETION); 
		test_fail++;
		in_rpt("shmctl(shmid, IPC_SET, &shmid_ds) failed with errno %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (test_fail || test_deleted)
		DBUG_EXIT(NOK);

	if (pathok == 6)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);

}

private void
ch2_t7()
{
	int rval;
	struct shmid_ds shmid_ds7;
	struct shmid_ds ds2;
	int shmid7;
	int err = NOERROR;
	int pathok = 0;

	DBUG_ENTER("ch2_t7");

	SETID(vguid1, vggid1);

	SHMGET(Key7, MODERWUSR, shmid7);

	shmid_ds7.shm_perm.uid = (uid_t) vguid0;
	shmid_ds7.shm_perm.gid = (gid_t) vggid0;
	shmid_ds7.shm_perm.mode = MODERWANY;

	errno = NOERROR; 
	if (shmctl(shmid7, IPC_SET, &shmid_ds7) == SYSERROR) { 
		err = errno; 
		xx_rpt(FAILURE); 
		test_fail++;
		in_rpt("shmctl(shmid, IPC_SET, &shmid_ds) failed with errno %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;


	errno = NOERROR;
	if (shmctl(shmid7, IPC_STAT, &ds2) == SYSERROR) {
                err = errno;
                xx_rpt(DELETION);
                test_deleted++;
                in_rpt("shmctl(shmid, IPC_STAT, &shmid_ds) failed with errno %d (%s)", err, errname(err));
                DBUG_EXIT(NOK);
        }
        else 
                PATH_TRACE;

	if (ds2.shm_perm.uid != (uid_t)vguid0) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_SET, shmid_ds7) did not set values for shared memory");
		in_rpt("expected shmid_ds7.shm_perm.uid = %d, got %d", vguid0, ds2.shm_perm.uid); 
	} 
	else
		PATH_TRACE;

	if (ds2.shm_perm.gid != (gid_t) vggid0) {
		if (!test_fail && !test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_SET, shmid_ds7) did not set values for shared memory");
		in_rpt("expected shmid_ds7.shm_perm.gid = %d, got %d", vggid0, ds2.shm_perm.gid); 
	} 
	else
		PATH_TRACE;

	if ((ds2.shm_perm.mode&MODEMASK) != MODERWANY) {
		if (!test_fail && !test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_SET, shmid_ds7) did not set values for shared memory");
		in_rpt("expected shmid_ds7.shm_perm.mode = %d, got %d", MODERWANY, ds2.shm_perm.mode); 
	} 
	else
		PATH_TRACE;

	if (test_fail || test_deleted)
		DBUG_EXIT(NOK);

	if (pathok == 10)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);

}

private void	/* When euid of caller matches uid of creator, test if */
test8()		/* caller can do shmctl with IPC_SET flag to set values */
		/* of shmid_ds structure */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test8");

	test_fail = 0;
	test_deleted = 0;

	KEY(Key8, ID8);

	CPPAIR(ch1_t8, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	CPPAIR(ch2_t8, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(3);

	SHMRM(Key8, MODERWUSR);

	DBUG_VOID_RETURN;
}


private void
ch1_t8()
{
	int rval;
	struct shmid_ds shmid_ds8;
	int shmid8;
	int err = NOERROR;
	int pathok = 0;

	DBUG_ENTER("ch1_t8");

	SETID(vguid0, vggid1); 

	SHMCREAT(Key8, MODERWUSR, shmid8);

	shmid_ds8.shm_perm.uid = vguid1;
	shmid_ds8.shm_perm.gid = vggid1;
	shmid_ds8.shm_perm.mode = MODERWUSR;

	errno = NOERROR; 
	if (shmctl(shmid8, IPC_SET, &shmid_ds8) == SYSERROR) { 
		err = errno; 
		xx_rpt(DELETION); 
		test_fail++;
		in_rpt("shmctl(shmid, IPC_SET, &shmid_ds) failed with errno %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (test_fail || test_deleted)
		DBUG_EXIT(NOK);

	if (pathok == 6)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);

}

private void
ch2_t8()
{
	int rval;
	struct shmid_ds shmid_ds8;
	struct shmid_ds ds2;
	int shmid8;
	int err = NOERROR;
	int pathok = 0;

	DBUG_ENTER("ch2_t8");

	SETID(vguid0, vggid0);

	SHMGET(Key8, MODERWUSR, shmid8);

	shmid_ds8.shm_perm.uid = (uid_t) vguid0;
	shmid_ds8.shm_perm.gid = (gid_t) vggid0;
	shmid_ds8.shm_perm.mode = MODERWUSR|MODERWGRP;

	errno = NOERROR; 
	if (shmctl(shmid8, IPC_SET, &shmid_ds8) == SYSERROR) { 
		err = errno; 
		xx_rpt(FAILURE); 
		test_fail++;
		in_rpt("shmctl(shmid, IPC_SET, &shmid_ds) failed with errno %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;


	errno = NOERROR;
	if (shmctl(shmid8, IPC_STAT, &ds2) == SYSERROR) {
                err = errno;
                xx_rpt(DELETION);
                test_deleted++;
                in_rpt("shmctl(shmid, IPC_STAT, &shmid_ds) failed with errno %d (%s)", err, errname(err));
                DBUG_EXIT(NOK);
        }
        else 
                PATH_TRACE;

	if (ds2.shm_perm.uid != (uid_t)vguid0) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_SET, shmid_ds8) did not set values for shared memory");
		in_rpt("expected shmid_ds8.shm_perm.uid = %d, got %d", vguid0, ds2.shm_perm.uid); 
	} 
	else
		PATH_TRACE;

	if (ds2.shm_perm.gid != (gid_t) vggid0) {
		if (!test_fail && !test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_SET, shmid_ds8) did not set values for shared memory");
		in_rpt("expected shmid_ds8.shm_perm.gid = %d, got %d", vggid0, ds2.shm_perm.gid); 
	} 
	else
		PATH_TRACE;

	if ((ds2.shm_perm.mode&MODEMASK) != (MODERWUSR|MODERWGRP)) {
		if (!test_fail && !test_deleted)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_SET, shmid_ds8) did not set values for shared memory");
		in_rpt("expected shmid_ds8.shm_perm.mode = %d, got %d", MODERWUSR|MODERWGRP, ds2.shm_perm.mode); 
	} 
	else
		PATH_TRACE;

	if (test_fail || test_deleted)
		DBUG_EXIT(NOK);

	if (pathok == 10)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);

}

private void	/* Check that process with "appropriate privileges" can */
test9()		/* perform a shmctl with IPC_SET to set shmid_ds  */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test9");

	test_fail = 0;
	test_deleted = 0;

	if (setprv(PRV_IPC) == SYSERROR)
	{
		if (!test_fail++)
			xx_rpt(DELETION);
		in_rpt("setprv(PRV_IPC) returned error, process does not have appropriate privileges");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	KEY(Key9, ID9);

	CPPAIR(ch1_t9, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	CPPAIR(ch2_t9, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(4);

	SHMRM(Key9, MODEURDONLY);

	DBUG_VOID_RETURN;
}

private void
ch1_t9()
{
	int rval;
	int shmid9;
	int err = NOERROR;
	int pathok = 0;

	DBUG_ENTER("ch1_t9");

	SETID(vguid0, vggid0);

	SHMCREAT(Key9, MODEURDONLY, shmid9);

	if (pathok == 5)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}

private void
ch2_t9()
{
	int rval;
	struct shmid_ds shmid_ds9;
	struct shmid_ds ds2;
	int shmid9;
	int err = NOERROR;
	int pathok = 0;

	DBUG_ENTER("ch2_t9");

	/* super user has appropriate privileges */

	SHMGET(Key9, MODEURDONLY, shmid9);

	SHMSTAT(shmid9, shmid_ds9);	/* pathok += 2 */

	shmid_ds9.shm_perm.uid = vguid1;
	shmid_ds9.shm_perm.gid = vggid1;
	shmid_ds9.shm_perm.mode = MODERWANY;

	errno = NOERROR; 
	if (shmctl(shmid9, IPC_SET, &shmid_ds9) == SYSERROR) { 
		err = errno; 
		xx_rpt(FAILURE); 
		test_fail++;
		in_rpt("shmctl(shmid, IPC_SET, &shmid_ds) failed with errno %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (shmctl(shmid9, IPC_STAT, &ds2) == SYSERROR) {
                err = errno;
                xx_rpt(DELETION);
                test_deleted++;
                in_rpt("shmctl(shmid, IPC_STAT, &shmid_ds) failed");
		valrpt(NOERROR,SYSERROR,NOERROR,err);
                DBUG_EXIT(NOK);
        }
        else 
                PATH_TRACE;

	if (ds2.shm_perm.uid != (uid_t) vguid1) {
		if (!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("shmctl(shmid, IPC_SET, shmid_ds9) did not set values for shared memory segment");
		in_rpt("Expected shmid_ds9.shm_perm.uid = %d, got %d", vguid1, ds2.shm_perm.uid); 
	}
	else
		PATH_TRACE;

	if (ds2.shm_perm.gid != vggid1) {
		if (!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("shmctl(shmid, IPC_SET, shmid_ds9) did not set values for shared memory segment");
		in_rpt("Expected shmid_ds9.shm_perm.gid = %d, got %d", vggid1, ds2.shm_perm.gid); 
	}
	else
		PATH_TRACE;

	if ((ds2.shm_perm.mode&MODEMASK) != MODERWANY) {
		if (!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("shmctl(shmid, IPC_SET, shmid_ds9) did not set values for shared memory segment");
		in_rpt("Expected shmid_ds9.shm_perm.mode = %d, got %d", (MODEMASK&MODERWANY), (ds2.shm_perm.mode&MODEMASK)); 
	}
	else
		PATH_TRACE;

	if (test_fail || test_deleted)
		DBUG_EXIT(NOK);

	if (pathok == 8)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}

private void	/* Check that a when euid of caller matches euid of */
test10()	/* owner that the caller can remove the segment */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test10");

	test_fail = 0;
	test_deleted = 0;

	KEY(Key10, ID10);

	CPPAIR(ch1_t10, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	CPPAIR(ch2_t10, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(3);

	SHMRM(Key10, MODEURDONLY);

	DBUG_VOID_RETURN;
}


private void
ch1_t10()
{
	int rval;
	struct shmid_ds shmid_ds10;
	int shmid10;
	int pathok = 0;
	int err = NOERROR;

	DBUG_ENTER("ch1_t10");

	SETID(vguid0, vggid0);

	SHMCREAT(Key10, MODERWANY, shmid10);

	SHMSTAT(shmid10, shmid_ds10);	/* pathok += 2 */

	shmid_ds10.shm_perm.uid = vguid1;
	shmid_ds10.shm_perm.gid = vggid1;
	shmid_ds10.shm_perm.mode = MODEURDONLY;

	SHMSET(shmid10, shmid_ds10);

	if (pathok == 8)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);

}

private void
ch2_t10()
{
	int rval;
	struct shmid_ds shmid_ds10;
	int shmid10;
	int pathok = 0;
	int err = NOERROR;

	DBUG_ENTER("ch2_t10");

	SETID(vguid1, vggid1); /* not creator, but owner */

	SHMGET(Key10, MODEURDONLY, shmid10);

	errno = NOERROR;
	if (shmctl(shmid10, IPC_RMID, &shmid_ds10) == SYSERROR) {
		err = errno;
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_RMID, &shmid_ds) failed with errno %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok == 6)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);

}

private void	/* Check that a when euid of caller matches euid of */
test11()	/* creator that the caller can remove the segment */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test11");

	test_fail = 0;
	test_deleted = 0;

	KEY(Key11, ID11);

	CPPAIR(ch1_t11, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	CPPAIR(ch2_t11, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(3);

	SHMRM(Key11, MODEURDONLY);

	DBUG_VOID_RETURN;
}


private void
ch1_t11()
{
	int rval;
	struct shmid_ds shmid_ds11;
	int shmid11;
	int pathok = 0;
	int err = NOERROR;

	DBUG_ENTER("ch1_t11");

	SETID(vguid0, vggid0);

	SHMCREAT(Key11, MODERWANY, shmid11);

	SHMSTAT(shmid11, shmid_ds11);	/* pathok += 2 */

	shmid_ds11.shm_perm.uid = vguid1;
	shmid_ds11.shm_perm.gid = vggid1;
	shmid_ds11.shm_perm.mode = MODEURDONLY;

	SHMSET(shmid11, shmid_ds11);

	if (pathok == 8)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);

}

private void
ch2_t11()
{
	int rval;
	struct shmid_ds shmid_ds11;
	int shmid11;
	int pathok = 0;
	int err = NOERROR;

	DBUG_ENTER("ch2_t11");

	SETID(vguid0, vggid0); /* creator, but not owner */

	SHMGET(Key11, MODEURDONLY, shmid11);

	errno = NOERROR;
	if (shmctl(shmid11, IPC_RMID, &shmid_ds11) == SYSERROR) {
		err = errno;
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_RMID, &shmid_ds) failed with errno %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok == 6)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);

}

private void	/* Check that a when euid of caller is superuser */
test12()	/* that the caller can remove the segment */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test12");

	test_fail = 0;
	test_deleted = 0;

	if (setprv(PRV_IPC) == SYSERROR)
	{
		if (!test_fail++)
			xx_rpt(DELETION);
		in_rpt("setprv(PRV_IPC) returned error, process does not have appropriate privileges");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	KEY(Key12, ID12);

	CPPAIR(ch1_t12, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	CPPAIR(ch2_t12, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(4);

	SHMRM(Key12, MODEURDONLY);

	DBUG_VOID_RETURN;
}


private void
ch1_t12()
{
	int rval;
	struct shmid_ds shmid_ds12;
	int shmid12;
	int pathok = 0;
	int err = NOERROR;

	DBUG_ENTER("ch1_t12");

	SETID(vguid0, vggid0);

	SHMCREAT(Key12, MODERWANY, shmid12);

	SHMSTAT(shmid12, shmid_ds12);	/* pathok += 2 */

	shmid_ds12.shm_perm.uid = vguid1;
	shmid_ds12.shm_perm.gid = vggid1;
	shmid_ds12.shm_perm.mode = MODEURDONLY;

	SHMSET(shmid12, shmid_ds12);

	if (pathok == 8)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);

}

private void
ch2_t12()
{
	int rval;
	struct shmid_ds shmid_ds12;
	int shmid12;
	int pathok = 0;
	int err = NOERROR;

	DBUG_ENTER("ch2_t12");

	/* superuser has appropriate privileges for now */

	SHMGET(Key12, MODEURDONLY, shmid12);

	errno = NOERROR;
	if (shmctl(shmid12, IPC_RMID, &shmid_ds12) == SYSERROR) {
		err = errno;
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_RMID, &shmid_ds) failed with errno %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok == 2)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);

}

private void	/* Test that we get an error attempting to do an IPC_STAT */
test13()	/* on a segment we don't have perms for */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test13");

	test_fail = 0;
	test_deleted = 0;

	KEY(Key13, ID13);

	CPPAIR(ch_t13, pa_t13, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(2);

	SHMRM(Key13, MODEURDONLY);

	DBUG_VOID_RETURN;
}

private void
pa_t13()
{
	int rval;
	int pathok = 0;	
	
	DBUG_ENTER("pa_t13");

	childpid = tet_child;

	CPPAIR(gch_t13, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_deleted)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;


}

	
private void	
gch_t13()	
{
	int rval;
	int shmid13;
	int err;
	int pathok = 0;

	DBUG_ENTER("gch_t13");

	GSETID(vguid0,vggid0);

	errno = NOERROR; 
	shmid13 = shmget(Key13, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	if (shmid13 == SYSERROR) { 
		xx_rpt(DELETION);
		in_rpt("shmget() with IPC_CREAT set failed with err %d (%s)", err, errname(err));
		(void)kill(childpid, SIGUSR2);
		DBUG_EXIT(NOK);
	} else
		PATH_TRACE;

	(void)sleep(5);	/* possible race condition */
	(void) kill (childpid, SIGUSR1);

	if (pathok == 5)
		DBUG_VOID_RETURN;

	DBUG_EXIT(NOK);

}

/* This guy waits for his signal from the parent and then */
/* then reads in the data from the shm segment */
private void
ch_t13()
{
	int shmid_ch13;
	struct shmid_ds shmid_ds13;
	int rval;
	int err = NOERROR; 
	int pathok = 0;


	DBUG_ENTER("ch_t13");
	(void) vsignal(SIGUSR1, stub);
	(void) vsignal(SIGUSR2, error);
	(void) pause();

	errno = NOERROR;
	shmid_ch13 = shmget(Key13, SEGSIZE, MODEURDONLY);
	err = errno;
	if (shmid_ch13 == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("shmget() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		test_fail++;
		DBUG_EXIT(NOK);
	}
	else 
		PATH_TRACE;

        errno = NOERROR;
        rval = shmctl(shmid_ch13, IPC_STAT, &shmid_ds13);
        if (rval == SYSERROR)
        {
                err = errno;
                xx_rpt(DELETION);
                test_deleted++;
		in_rpt("shmctl(shmid, IPC_STAT, &shmid_ds) failed");
		valrpt(NOERROR,rval,NOERROR,err);
                DBUG_EXIT(NOK);
        }
        else
                PATH_TRACE;

        shmid_ds13.shm_perm.mode = MODEURDONLY;

        errno = NOERROR;
        rval = shmctl(shmid_ch13, IPC_SET, &shmid_ds13);
        if (rval == SYSERROR)
        {
                err = errno;
                xx_rpt(DELETION);
                test_fail++;
                in_rpt("shmctl(shmid, IPC_SET, &shmid_ds) failed");
                valrpt(NOERROR,rval,NOERROR,err);
                DBUG_EXIT(NOK);
        }
        else
                PATH_TRACE;

	SETID(vguid1, vggid1);

	errno = NOERROR;
	rval = shmctl(shmid_ch13, IPC_STAT, &shmid_ds13);
	if (rval != SYSERROR || errno != EACCES) {
		err = errno;
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_STAT, &shmid_ds) without appropriate permissions returned unexpectedly");
		valrpt(SYSERROR,rval,EACCES,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (pathok == 8)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}


private void	/* Check that we get an error when we call shmctl */
test14()	/* with an invalid shmid and IPC_STAT set */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test14");

	test_fail = 0;
	test_deleted = 0;

	KEY(Key14, ID14);

	CPPAIR(ch_t14, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;

}


private void
ch_t14()
{
	struct shmid_ds shmid_ds14;
	int shmid14;
	int pathok = 0;
	int err = NOERROR;
	int rval;

	DBUG_ENTER("ch_t14");
	
	SETID(vguid0, vggid0);

	SHMCREAT(Key14, MODERWANY, shmid14);

	if (shmctl(shmid14, IPC_RMID, &shmid_ds14) == SYSERROR)
	{
		err = errno;
		if (!test_fail++)
			xx_rpt(DELETION);
		in_rpt("shmctl(IPC_RMID) failed, unable to remove share memory identifier during test setup");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = shmctl(shmid14, IPC_STAT, &shmid_ds14);
	if (rval != SYSERROR || errno != EINVAL) {
		err = errno;
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_STAT, &shmid_ds) of non-existant shmid didn't fail, expected errno %d (%s), got errno %d (%s)", EINVAL, errname(EINVAL), err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok == 7)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}

private void	/* Test that we get an error when we call shmctl with */
test15()	/* IPC_SET set and we don't have perms and our euid  */
		/* does not match the uid of the owner or creator of seg */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test15");

	test_fail = 0;
	test_deleted = 0;

	KEY(Key15, ID15);

	CPPAIR(ch_t15, pa_t15, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(2);

	SHMRM(Key15, MODEURDONLY);

	DBUG_VOID_RETURN;
}

private void
pa_t15()
{
	int rval;
	int pathok = 0;	
	
	DBUG_ENTER("pa_t15");

	childpid = tet_child;

	CPPAIR(gch_t15, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_deleted)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;


}

	
private void	
gch_t15()	
{
	int rval;
	int shmid15;
	int err = NOERROR;
	int pathok = 0;

	DBUG_ENTER("gch_t15");

	GSETID(vguid0,vggid0);

	errno = NOERROR; 
	shmid15 = shmget(Key15, SEGSIZE, MODERDALL|IPC_CREAT);
	err = errno;
	if (shmid15 == SYSERROR) { 
		xx_rpt(DELETION);
		in_rpt("shmget() with IPC_CREAT set failed with err %d (%s)", err, errname(err));
		(void)kill(childpid, SIGUSR2);
		DBUG_EXIT(NOK);
	} else
		PATH_TRACE;

	(void)sleep(5);	/* possible race condition */
	(void) kill (childpid, SIGUSR1);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

/* This guy waits for his signal from the parent and then */
/* then reads in the data from the shm segment */
private void
ch_t15()
{
	int shmid_ch15;
	int rval;
	struct shmid_ds shmid_ds15;
	int err = NOERROR; 
	int pathok = 0;


	DBUG_ENTER("ch_t15");
	(void) vsignal(SIGUSR1, stub);
	(void) vsignal(SIGUSR2, error);
	(void) pause();

	SETID(vguid1, vggid1);

	errno = NOERROR;
	shmid_ch15 = shmget(Key15, SEGSIZE, MODEURDONLY);
	err = errno;
	if (shmid_ch15 == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("shmget() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		test_fail++;
		DBUG_EXIT(NOK);
	}
	else 
		PATH_TRACE;

	(void) memset((void *)&shmid_ds15, 0, sizeof(shmid_ds15));
	errno = NOERROR;
	rval = shmctl(shmid_ch15, IPC_SET, &shmid_ds15);
	err = errno;
	if (rval != SYSERROR || err != EPERM) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_SET, &shmid_ds) without appropriate permissions returned unexpectedly");
		valrpt(SYSERROR,rval,EPERM,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (pathok == 6)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}

private void	/* Test that we get an error when we try to remove */
test16()	/* a segment we don't have perms for */
{
	int rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test16");

	test_fail = 0;
	test_deleted = 0;

	KEY(Key16, ID16);

	CPPAIR(ch_t16, pa_t16, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(2);

	SHMRM(Key16, MODEURDONLY);

	DBUG_VOID_RETURN;
}

private void
pa_t16()
{
	int rval;
	int pathok = 0;	
	
	DBUG_ENTER("pa_t16");

	childpid = tet_child;

	CPPAIR(gch_t16, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);

	if (!test_deleted)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;


}

	
private void	
gch_t16()	
{
	int rval;
	int shmid16;
	int err;
	int pathok = 0;

	DBUG_ENTER("gch_t16");

	GSETID(vguid0,vggid0);

	errno = NOERROR; 
	shmid16 = shmget(Key16, SEGSIZE, MODERDALL|IPC_CREAT);
	err = errno;
	if (shmid16 == SYSERROR) { 
		xx_rpt(DELETION);
		in_rpt("shmget() with IPC_CREAT set failed with err %d (%s)", err, errname(err));
		(void)kill(childpid, SIGUSR2);
		DBUG_EXIT(NOK);
	} else
		PATH_TRACE;
	
	(void)sleep(5);	/* possible race condition */
	(void) kill (childpid, SIGUSR1);

	if (pathok == 5)
		DBUG_VOID_RETURN;

	DBUG_EXIT(NOK);
}

/* This guy waits for his signal from the parent and then */
/* then reads in the data from the shm segment */
private void
ch_t16()
{
	int shmid16;
	int rval;
	struct shmid_ds shmid_ds16;
	int err = NOERROR; 
	int pathok = 0;

	DBUG_ENTER("ch_t16");
	(void) vsignal(SIGUSR1, stub);
	(void) vsignal(SIGUSR2, error);
	(void) pause();

	SETID(vguid1, vggid1);

	errno = NOERROR;
	shmid16 = shmget(Key16, SEGSIZE, MODEURDONLY);
	err = errno;
	if (shmid16 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("shmget() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		test_deleted++;
		DBUG_EXIT(NOK);
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	rval = shmctl(shmid16, IPC_RMID, &shmid_ds16);
	err = errno;
	if (rval != SYSERROR || err != EPERM) {
		err = errno;
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmctl(shmid, IPC_RMID, &shmid_ds) without appropriate permissions returned unexpectedly");
		valrpt(SYSERROR,rval,EPERM,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (pathok == 6)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}



private void 
shm_init (shm_ds)
struct shmid_ds *shm_ds;
{
	shm_ds->shm_segsz = 	0;
	shm_ds->shm_cpid = 	(pid_t) 0;
	shm_ds->shm_lpid = 	(pid_t) 0;
	shm_ds->shm_nattch = 	(unsigned long)0;
	shm_ds->shm_atime = 	(time_t) 0;
	shm_ds->shm_dtime = 	(time_t) 0;
	shm_ds->shm_ctime = 	(time_t) 0;
	shm_ds->shm_perm.cuid =	(uid_t) 0;
	shm_ds->shm_perm.cgid = (gid_t) 0;
	shm_ds->shm_perm.uid = 	(uid_t) 0;
	shm_ds->shm_perm.gid = 	(gid_t) 0;
	shm_ds->shm_perm.mode = (mode_t) 0;
}

private int 
shm_test (shm_ds)
struct shmid_ds *shm_ds;
{
	if (shm_ds->shm_segsz == -1) 
		return (SYSERROR);
	if (shm_ds->shm_cpid == (pid_t) -1)
		return (SYSERROR);
	if (shm_ds->shm_lpid == (pid_t) -1)
		return (SYSERROR);
	if (shm_ds->shm_nattch == (unsigned long) -1)
		return (SYSERROR);
	if (shm_ds->shm_atime == (time_t) -1)
		return (SYSERROR);
	if (shm_ds->shm_dtime == (time_t) -1)
		return (SYSERROR);
	if (shm_ds->shm_ctime == (time_t) -1)
		return (SYSERROR);
	if (shm_ds->shm_perm.cuid == (uid_t) -1)
		return (SYSERROR);
	if (shm_ds->shm_perm.cgid == (gid_t) -1)
		return (SYSERROR);
	if (shm_ds->shm_perm.uid == (uid_t) -1)
		return (SYSERROR);
	if (shm_ds->shm_perm.gid == (gid_t) -1)
		return (SYSERROR);
	if (shm_ds->shm_perm.mode == (mode_t) -1)
		return (SYSERROR);
	
	return(NONE);
}

private void
test17()
{
	int	rval, err, shmid, cmd;
	struct	shmid_ds buf;
	int	pathok = setupok;

	DBUG_ENTER("test17");

	if(tsup)
	{
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	rval = shmctl(shmid, cmd, &buf);
	err = errno;

	if(rval != -1 || err != ENOSYS)
	{
		xx_rpt(FAILURE);

		if(rval != -1)
		{
			in_rpt("shmget(IPC_PRIVATE,1,0) did not return expected value");
			in_rpt("Expected return value -1");
			in_rpt("Actual   return value %d", rval);
		}

		if(err != ENOSYS)
		{
			in_rpt("shmctl(%d,IPC_STAT,bufp) did not set errno correctly");
			in_rpt("Expected errno %d (ENOSYS)", ENOSYS);
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}
