/*
*	SCCS:  @(#)  XPG4.os/ipc/shmdt/shmdt.c Rel 4.4.1     (06/09/97)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)  XPG4.os/ipc/shmdt/shmdt.c Rel 4.4.1     (06/09/97)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.shmdt

PROJECT:	XPG4 Validation suite

DESCRIPTION:
	Testset for the shmdt() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Tony Rems
DATE CREATED:	Thu Nov 19 12:24:50 BST 1990
MODIFICATIONS:	
		Matt Newman, Wed Sep 11 12:18:23 BST 1991
			XPG4 Intergration
		
		Geoff Clare, 21 Apr 1992
			Change caddr_t to char *.

		Neil Moses, 8 May 1992.
			Check that interface is supported.
			Added TEST For ENOSYS if interface not supported.

		Neil Moses, 13 May 1992.
			Repositioned realfuncs[] declaration.

		Neil Moses, 4 June 1992.
			Use sysconf() to check if interface supported.

		Geoff Clare, 16 June 1993
			Add declaration of ftok().

		Geoff Clare, 4 March 1994
			Removed <sys/sysmacros.h> header.

		Geoff Clare, 1 June 1995
			In test 2 add sizeof(long) to address instead of 1.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<stdio.h>
#include	<string.h>
#include	<sys/ipc.h>
#include 	<sys/shm.h>
#include	<signal.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<report.h>
#include	<errno.h>
#include 	<tet_api.h>
#include 	<sysdep.h>

#ifdef UNDEF_MACROS
#undef shmdt
#endif


#define		WAITTIME	(5 * SPEEDFACTOR)
#define 	NO_TESTS	3	/* number of tests in testset */
#define		PATH_ERR_EXIT	0x04
#define		MODEANY		((mode_t)(S_IRWXU|S_IRWXG|S_IRWXO))
#define		MODEURDONLY	S_IRUSR
#define		RW_U            (S_IRUSR|S_IWUSR)
#define		RW_G            (S_IRGRP|S_IWGRP)
#define		RW_O            (S_IROTH|S_IWOTH)
#define		MODERWANY	((mode_t) (RW_U|RW_G|RW_O))
#define		SEGSIZE		512 	     	/* Use 512 byte shm segments */
#define		NOK		02
#define		NONE		(mode_t) 0
#define		ID1		11		/* id's for use with ftok */
#define		ID2		22	
#define		TOKFILE		"./shmdtfile" 	/* file for use with ftok */

#define 	STRING1		"test1 string"	/* strings to write into shm */
#define		RD_FD		0
#define		WR_FD		1

int		no_tests = NO_TESTS;

#ifndef ftok
extern	key_t	ftok();
#endif
extern 	char	*errname();
extern 	void	xx_all();

extern 	void	valrpt();
extern	int	crfile();
extern	char	*errname();
extern  void    (*vsignal())();
private void 	tidy(), setup(), test1(), test2(), test3();

public  void    (*realfuncs[]) () = { test1,  test2, NULL };

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = tidy;

private void	twrap();

public	struct tet_testlist tet_testlist[] = {
	twrap,	 1,
	twrap,	 2,
	test3,	 3,
	NULL,	 0
};

private	int	test_fail;
private	int	tsup = 0;
private	int	test_deleted;
private key_t	Key1, Key2;
private int	g_shmid;
private int	setupok = -1;

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if(!tsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("sysconf(_SC_XOPEN_SHM) returned -1");
	}
	else
		if (tet_thistest >= 1 && tet_thistest <= no_tests)
			(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

#if TESTBED0
#define	shmdt(a)	(SYSERROR)
#endif

private void
setup()	
{
	int	pathok = 0;

	DBUG_ENTER("setup");

	errno = 0;
	if(sysconf(_SC_XOPEN_SHM) == -1)
	{
		if (errno != 0)
		{
			xx_all(DELETION, "sysconf(_SC_XOPEN_SHM) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		tsup = 0;
	}
	else
	{
		tsup = 1;
		PATH_TRACE;
	}

	if (crfile(TOKFILE, MODEANY, (int)getuid(), (int)getgid()) == SYSERROR) {
		(void) tet_delete(1, "Could not create reference file for keys.");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(2);
	
	DBUG_VOID_RETURN;
}

		
private	void
tidy()
{
	struct shmid_ds cln_ds;

	DBUG_ENTER("tidy");
	(void) unlink(TOKFILE);
	(void)shmctl(g_shmid, IPC_RMID, &cln_ds);
	DBUG_VOID_RETURN;
}


private void	/* Test that a call to shmdt detaches specified shm segment */
test1()
{
        struct	shmid_ds shmid_ds1;
        int 	shmid1, rval;
	char	*shmidptr1;
        int 	err = NOERROR;
        int 	pathok = setupok;

	DBUG_ENTER("test1");

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	Key1 = ftok(TOKFILE, ID1);
	err = errno;
	if (Key1 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID1, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	shmid1 = shmget(Key1, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	g_shmid = shmid1;
	if (shmid1 == SYSERROR) {
		if (!test_deleted)
			xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmget() with IPC_CREAT set failed with err %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	shmidptr1 = (char *) shmat(shmid1, NULL, NONE);
	err = errno;
	if (shmidptr1 == NULL || shmidptr1 == (char*)SYSERROR) {
		if (!test_deleted++)
			xx_rpt(DELETION);
		in_rpt("shmat() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if ((rval=shmdt((void *)shmidptr1)) != 0)
	{
		err = errno;
		if (!test_deleted)
			xx_rpt(FAILURE);
		in_rpt("shmdt(0x%lx) of existing shared memory segment failed",
			(long)shmidptr1);
		valrpt(NOERROR,rval,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	(void)shmctl(shmid1, IPC_RMID, &shmid_ds1);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


private void	/* Malloc a segment a segment to obtain an address then */
test2()		/* try to attatch to that address - this should fail */
{
        struct	shmid_ds shmid_ds2;
        int 	shmid2;
	char	*shmidptr2, *bogusptr2;
	int	err, rval;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	bogusptr2 = (char *)malloc(SEGSIZE);
	err = errno;
	if (bogusptr2 == NULL) {	
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("malloc(SEGSIZE) returned -1 on with errno is %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE;

		errno = NOERROR;
		rval = shmdt((void *)bogusptr2);
		err = errno;
		if (rval != SYSERROR || errno != EINVAL) {
			if (!test_fail++)
				xx_rpt(FAILURE);
			in_rpt("shmdt(addr) unexpected return with invalid address 0x%lx in dynamically allocated data area", (long)bogusptr2);
			valrpt(SYSERROR, rval, EINVAL, err);
		}
		else
			PATH_TRACE;
	}

	errno = NOERROR;
	Key2 = ftok(TOKFILE, ID2);
	err = errno;
	if (Key2 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID2, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	shmid2 = shmget(Key2, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	if (shmid2 == SYSERROR) {
		if (!test_deleted)
			xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmget() with IPC_CREAT set failed with err %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	shmidptr2 = (char *) shmat(shmid2, NULL, NONE);
	err = errno;
	if (shmidptr2 == NULL || shmidptr2 == (char*)SYSERROR) {
		if (!test_deleted++)
			xx_rpt(DELETION);
		in_rpt("shmat() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		(void)shmctl(shmid2, IPC_RMID, &shmid_ds2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	/* addr not pointing to beginning of seg.   Adding less than the
	   size of a word might not work on word-based systems. */
	bogusptr2 = shmidptr2 + sizeof(long);

	errno = NOERROR;
	rval = shmdt((void *)bogusptr2);
	err = errno;
	if (rval != SYSERROR || errno != EINVAL) {
		if (!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("shmdt(addr) unexpected return with invalid address 0x%lx not pointing to beginning of shared memory segment", (long)bogusptr2);
		valrpt(SYSERROR, rval, EINVAL, err);
	}
	else
		PATH_TRACE;

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(6);
	
	(void)shmdt((void *)shmidptr2);
	(void)shmctl(shmid2, IPC_RMID, &shmid_ds2);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int 	rval, err;
	void 	*shmidptr;
	int	pathok = setupok;

	DBUG_ENTER("test3");

	if(tsup)
	{
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	rval = shmdt((void *)shmidptr);
	err = errno;

	if(rval != -1 || err != ENOSYS)
	{
		xx_rpt(FAILURE);

		if(rval != -1)
		{
			in_rpt("shmdt() did not return expected value");
			in_rpt("Expected return value -1");
			in_rpt("Actual   return value %d", rval);
		}

		if(err != ENOSYS)
		{
			in_rpt("shmdt() did not set errno correctly");
			in_rpt("Expected %d (ENOSYS)", ENOSYS);
			in_rpt("Actual   %d (%s)", err, errname(err));
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}
