/*
*	SCCS:  @(#)  XPG4.os/ipc/shmget/shmget.c Rel 4.4.1     (06/09/97)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)  XPG4.os/ipc/shmget/shmget.c Rel 4.4.1     (06/09/97)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.shmget

PROJECT:	XPG4 Validation suite

DESCRIPTION:
	Testset for the shmget() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Tony Rems
DATE CREATED:	Thu Oct 15 12:24:50 BST 1990
MODIFICATIONS:	
		Matt Newman, Wed Sep 11 13:26:16 BST 1991
			XPG4 Intergration

		Geoff Clare, 29 Apr 1992
			Changes for new type of shm_nattch

		Neil Moses, 8 May 1992.
			Check that interface is supported.
			Added TEST For ENOSYS if interface not supported.

		Neil Moses, 4 June 1992.
			Use sysconf() to check if interface supported.

		Geoff Clare, 16 June 1993
			Changes for clean lint.

		Geoff Clare, 4 March 1994
			Removed multiple definition of tet_thistest.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<string.h>
#include	<sys/types.h>
#include	<sys/ipc.h>
#include 	<sys/shm.h>
#include	<sys/stat.h>
#include	<signal.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<report.h>
#include	<errno.h>
#include	<tet_api.h>
#include 	<sysdep.h>

#ifdef UNDEF_MACROS
#undef shmget
#endif


#define		WAITTIME	(5 * SPEEDFACTOR)
#define 	NO_TESTS	9	/* number of tests in testset */
#define		MODEANY		((mode_t)(S_IRWXU|S_IRWXG|S_IRWXO))
#define		MODEMASK	(mode_t)MODEANY
#define		MODEURDONLY	S_IRUSR
#define		RW_U            (S_IRUSR|S_IWUSR)
#define		RW_G            (S_IRGRP|S_IWGRP)
#define		RW_O            (S_IROTH|S_IWOTH)
#define		MODERWANY	((mode_t) (RW_U|RW_G|RW_O))
#define		PATH_ERR_EXIT	0x04
#define		SEGSIZE		512 	     	/* Use 512 byte shm segments */
#define		NONE		0L
#define		NOK		02
#define		SUCCESS		02
#define		ID1		11		/* id's for use with ftok */
#define		ID3		33
#define		ID4		44
#define		ID5		55
#define		ID6		66
#define		ID7		77
#define		ID8		88
#define		TOKFILE		"./shmfile" 	/* file for use with ftok */
#define		RD_FD		0		/* rd/write fd names for pipe */
#define		WR_FD		1

#define 	STRING1		"test1 string"	/* string to write into shm */

int		no_tests = NO_TESTS;

#ifndef ftok
extern	key_t	ftok();
#endif
extern	char *errname();
extern	void xx_all();

extern	void	valrpt();
extern	void	(*vsignal())();
extern 	time_t	time();
extern	char	*errname();
private void	stub(), tidy(), setup(), test1(), pa_t1(), ch_t1(), test2(),
		test3(), test4(), ch_t4(), test5(), ch_t5(), test6(), ch_t6(),
		test7(), test8(), ch_t8();

private void test9();

public  void    (*realfuncs[]) () = { test1,  test2,  test3,  test4,  test5,
                                      test6,  test7, test8, NULL };

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = tidy;

private void	twrap();

public	struct tet_testlist tet_testlist[] = {
	twrap,	 1,
	twrap,	 2,
	twrap,	 3,
	twrap,	 4,
	twrap,	 5,
	twrap,	 6,
	twrap,	 7,
	twrap,	 8,
	test9,	 9,
	NULL,	 0
};

private	int	test_fail;
private	int	test_deleted;
private	int	tsup = 0;
private int	setupok = -1;
private int	pip_fds[2];
private key_t	Key1;
private key_t	Key3;
private key_t	Key4;
private key_t	Key5;
private key_t	Key6;
private key_t	Key7;
private key_t	Key8;
private int	g_shmids[NO_TESTS];

#ifdef TESTBED0
#define	shmget(a,b,c)	(SYSERROR)
#endif

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if(!tsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("sysconf(_SC_XOPEN_SHM) returned -1");
	}
	else
		if (tet_thistest >= 1 && tet_thistest <= no_tests)
			(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
setup()	
{
	int	pathok = 0;
	int 	i;

	DBUG_ENTER("setup");

	errno = 0;
	if(sysconf(_SC_XOPEN_SHM) == -1)
	{
		if (errno != 0)
		{
			xx_all(DELETION, "sysconf(_SC_XOPEN_SHM) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		tsup = 0;
	}
	else
	{
		tsup = 1;
		PATH_TRACE;
	}

	if (crfile(TOKFILE, MODEANY, (int)getuid(), (int)getgid()) == SYSERROR) {
		for (i = 1; i <= NO_TESTS; i++) {
			(void) tet_delete(i, "Could not create reference file for keys.");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(2);

	DBUG_VOID_RETURN;
}

		
private	void
tidy()
{
	int i = 0;
	struct shmid_ds cln_ds;

	DBUG_ENTER("tidy");
	(void) unlink(TOKFILE);
	while (i < no_tests)
		(void)shmctl(g_shmids[i++], IPC_RMID, &cln_ds);
	(void)close(pip_fds[RD_FD]);
	(void)close(pip_fds[WR_FD]);
	DBUG_VOID_RETURN;
}


		/* Test that a successful call to shmget returns the shmid
		 * associated with the key */
private void	/* Check that we can get a valid shared memory segment */
test1()		/* read and write to and from it */
{
	struct shmid_ds shmid_ds1;
	int shmid1 = -1, rval;
	int pathok = setupok;
	int err = NOERROR;

	DBUG_ENTER("test1");

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	rval = pipe(pip_fds);
	err = errno;
	if (rval == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("pipe(pip_fds) returned -1 with errno %d (%s)\n", err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	errno = NOERROR;
	Key1 = ftok(TOKFILE, ID1);
	err = errno;
	if (Key1 == SYSERROR) {
		test_deleted++;
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID1, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	


	errno = NOERROR;
	shmid1 = shmget(Key1, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid1;
	if (shmid1 == SYSERROR) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmget(Key1, %d, 0%o | IPC_CREAT) failed with err %d (%s) ", SEGSIZE, MODERWANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	rval = cppair(ch_t1, pa_t1, WAITTIME, NOK|PATH_ERR_EXIT);
	err = errno;
	if (rval&NOK) {
		/* failure/deletion was already reported in pa_t1 or ch_t1 */
		test_deleted++;
	}
	else /*if ((rval&PATH_ERR_EXIT) == 0)*/
		PATH_TRACE;

	(void)shmctl(shmid1, IPC_RMID, &shmid_ds1);
	(void)close(pip_fds[RD_FD]);
	(void)close(pip_fds[WR_FD]);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void	/* This guy maps in the shm segment and writes to it and */
pa_t1()		/* then signals the child when he's done. */
{
	int shmid1;
	int err, rval;
	char *shmidptr1, *origptr;
	int pathok = setupok; 

	DBUG_ENTER("pa_t1");

	test_fail = 0;
	test_deleted = 0;

	(void)close(pip_fds[RD_FD]);
	errno = NOERROR;
	shmid1 = shmget(Key1, SEGSIZE, MODERWANY);
	err = errno;
	if (shmid1 == SYSERROR) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmget(Key1, %d, 0%o) failed with err %d (%s) ", SEGSIZE, MODERWANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	shmidptr1 = (char *) shmat(shmid1, NULL, NONE);
	err = errno;
	origptr = shmidptr1;
	if (shmidptr1 == NULL) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmat() of existing shared memory segment failed with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	(void)sprintf(shmidptr1, STRING1);  
	errno = NOERROR;
	rval = write(pip_fds[WR_FD], (char *)&pathok, sizeof(pathok)); 
	err = errno;
	if (rval == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("write on pipe failed with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) sleep(5);	/* possible race condition */
	(void) kill(tet_child, SIGUSR1);

	PATH_XS_RPT(3);

	(void)shmdt((void *)origptr);
	(void)close(pip_fds[RD_FD]);

	DBUG_VOID_RETURN;
}

private void	/* This guy waits for his signal from the parent and then */
ch_t1()		/* then reads in the data from the shm segment */
{
	int shmid_ch1;
	char *shmidptr1, *origptr;
	int err;
	int i;
	int pathok = setupok;
	char buf[SEGSIZE+1];


	DBUG_ENTER("ch_t1");

	(void)close(pip_fds[WR_FD]);
	(void)vsignal(SIGUSR1, stub); 
	(void)pause();
	(void)read(pip_fds[RD_FD], (char *)&pathok, sizeof(pathok));

	errno = NOERROR;
	shmid_ch1 = shmget(Key1, SEGSIZE, MODERWANY);
	err = errno;
	if (shmid_ch1 == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("shmget(Key1, %d, 0%o) failed with err %d (%s) ", SEGSIZE, MODERWANY, err, errname(err));
		test_fail++;
		DBUG_EXIT(NOK);
	}
	else 
		PATH_TRACE;

	errno = NOERROR;
	shmidptr1 = (char *) shmat(shmid_ch1, NULL, NONE);
	origptr = shmidptr1;
	err = errno;
	if (shmidptr1 == NULL) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmat() of existing shared memory segment failed with errno %d", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	i = 0;
	while (shmidptr1[i] != '\0' && i<SEGSIZE) {
                buf[i] = shmidptr1[i];
                i++;
        }
        buf[i] = shmidptr1[i]; 
	if (strcmp(buf, STRING1) != NOERROR) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected \"%s\" in shared memory segment, got \"%s\"\n", STRING1, buf);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void)shmdt((void *)origptr);

	if (pathok == 3)
		DBUG_VOID_RETURN;

	DBUG_EXIT(PATH_ERR_EXIT);
}

private void	/* This ain't nothin' but a stub */
stub()		/* Note to cander:  yeah, yeah, yeah, I know it's a hack. */
{
	return;
}

private void	/* Create a shm segment by using IPC_PRIVATE for key */
test2()		/* then check all the values from shmctl with stat flag */
{
	int shmid2;
	struct shmid_ds shmid_ds2;
	int rval;
	int err;
	int pathok = setupok;
	int euid, egid;
	time_t timen;

	DBUG_ENTER("test2");

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;	
	shmid2 = shmget(IPC_PRIVATE, SEGSIZE, MODERWANY);
	err = errno;
	g_shmids[tet_thistest-1] = shmid2;
	if (shmid2 == SYSERROR) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmget(IPC_PRIVATE, %d, 0%o) returned -1 with errno %d (%s)", SEGSIZE, MODERWANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = shmctl(shmid2, IPC_STAT, &shmid_ds2);
	err = errno;
	if (rval == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmctl returned -1 with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	euid = (int) geteuid();
	err = errno;
	if (euid == SYSERROR) {
		if (!test_deleted)
			xx_rpt(DELETION);
		in_rpt("geteuid returned -1 with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (shmid_ds2.shm_perm.cuid != (uid_t) euid) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds2.shm_perm.cuid %d returned %d\n", euid, (int) shmid_ds2.shm_perm.cuid);
	}
	else
		PATH_TRACE;
	
	if (shmid_ds2.shm_perm.uid != (uid_t) euid) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds2.shm_perm.uid %d returned %d\n", euid, (int)shmid_ds2.shm_perm.uid);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	egid = (int)getegid();
	err = errno;
	if (egid == SYSERROR) {
		if (!test_deleted++)
			xx_rpt(DELETION);
		in_rpt("getegid returned -1 with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (shmid_ds2.shm_perm.cgid != (gid_t)egid) { 
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds2.shm_perm.cgid %d returned %d\n", egid, (int)shmid_ds2.shm_perm.cgid);
	}
	else
		PATH_TRACE;
	
	if (shmid_ds2.shm_perm.gid != (gid_t)egid) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds2.shm_perm.gid %d returned %d\n", egid, (int)shmid_ds2.shm_perm.gid);
	}
	else
		PATH_TRACE;

	if ((shmid_ds2.shm_perm.mode & MODEMASK) != MODERWANY ) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds2.shm_perm.mode 0%o returned 0%o\n", (MODERWANY & MODEMASK), shmid_ds2.shm_perm.mode);
	}
	else
		PATH_TRACE;

	if (shmid_ds2.shm_lpid != (pid_t) NONE) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds2.shm_lpid of 0 returned %d\n", shmid_ds2.shm_lpid);
	}
	else
		PATH_TRACE;

	if (shmid_ds2.shm_nattch != NONE) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds2.shm_nattch of 0 returned %lu\n",
			(unsigned long) shmid_ds2.shm_nattch);
	}
	else
		PATH_TRACE;

	if (shmid_ds2.shm_atime != (time_t)NONE) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds2.shm_atime of 0 returned %d\n", shmid_ds2.shm_atime);
	}
	else
		PATH_TRACE;

	if (shmid_ds2.shm_dtime != (time_t)NONE) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds2.shm_dtime of 0 returned %d\n", shmid_ds2.shm_dtime);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	timen = time((time_t) NONE);
	err = errno;
	if ((timen - shmid_ds2.shm_ctime) >= WAITTIME)
	{
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected (current time less shmid_ds2.shm_ctime) to be less than %d seconds", WAITTIME);
		in_rpt("creation time = 0x%x, current time = 0x%x, difference in seconds is %d", shmid_ds2.shm_ctime, timen, timen - shmid_ds2.shm_ctime);
	}
	else
		PATH_TRACE;

	(void)shmctl(shmid2, IPC_RMID, &shmid_ds2);

	if (!test_fail)
		PATH_XS_RPT(14);

	DBUG_VOID_RETURN;
}

private void	/* Create a shm segment with IPC_CREAT flag and then check */
test3()		/* return values from shmctl with stat flag */
{
	int shmid3;
	struct shmid_ds shmid_ds3;
	int rval;
	int err;
	int pathok = setupok;
	int euid, egid;
	time_t timen;

	DBUG_ENTER("test3");

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	Key3 = ftok(TOKFILE, ID3);
	err = errno;
	if (Key3 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID3, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	errno = NOERROR;	
	shmid3 = shmget(Key3, SEGSIZE, MODERWANY | IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid3;
	if (shmid3 == SYSERROR) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmget(Key3, %d, 0%o | IPC_CREAT) returned -1 with errno %d (%s)", SEGSIZE, MODERWANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = shmctl(shmid3, IPC_STAT, &shmid_ds3);
	err = errno;
	if (rval == SYSERROR) {
		if (!test_deleted)
			xx_rpt(DELETION);
		in_rpt("shmctl returned -1 with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	euid = (int)geteuid();
	err = errno;
	if (euid == SYSERROR) {
		if (!test_deleted)
			xx_rpt(DELETION);
		in_rpt("geteuid returned -1 with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (shmid_ds3.shm_perm.cuid != (uid_t)euid) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds3.shm_perm.cuid %d returned %d\n", euid, (int)shmid_ds3.shm_perm.cuid);
	}
	else
		PATH_TRACE;
	
	if (shmid_ds3.shm_perm.uid != (uid_t)euid) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds3.shm_perm.uid %d returned %d\n", euid, (int)shmid_ds3.shm_perm.uid);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	egid = (int)getegid();
	err = errno;
	if (egid == SYSERROR) {
		if (!test_deleted)
			xx_rpt(DELETION);
		in_rpt("getegid returned -1 with errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (shmid_ds3.shm_perm.cgid != (gid_t)egid) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds3.shm_perm.cgid %d returned %d\n", egid, (int)shmid_ds3.shm_perm.cgid);
	}
	else
		PATH_TRACE;
	
	if (shmid_ds3.shm_perm.gid != (gid_t)egid) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds3.shm_perm.gid %d returned %d\n", egid, (int)shmid_ds3.shm_perm.gid);
	}
	else
		PATH_TRACE;

	if ((shmid_ds3.shm_perm.mode & MODEMASK) != MODERWANY ) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds3.shm_perm.mode 0%o returned 0%o\n", (MODERWANY & MODEMASK), shmid_ds3.shm_perm.mode);
	}
	else
		PATH_TRACE;

	if (shmid_ds3.shm_lpid != NONE) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds3.shm_lpid of 0 returned %d\n", shmid_ds3.shm_lpid);
	}
	else
		PATH_TRACE;

	if (shmid_ds3.shm_nattch != NONE) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds3.shm_nattch of 0 returned %lu\n",
			(unsigned long) shmid_ds3.shm_nattch);
	}
	else
		PATH_TRACE;

	if (shmid_ds3.shm_atime != NONE) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds3.shm_atime of 0 returned %d\n", shmid_ds3.shm_atime);
	}
	else
		PATH_TRACE;

	if (shmid_ds3.shm_dtime != NONE) {
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected shmid_ds3.shm_dtime of 0 returned %d\n", shmid_ds3.shm_dtime);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	timen = time((time_t) NONE);
	err = errno;
	if ((timen - shmid_ds3.shm_ctime) >= WAITTIME)
	{
		if (!test_fail)
			xx_rpt(FAILURE);
		test_fail++;
		in_rpt("expected (current time less shmid_ds3.shm_ctime) to be less than %d seconds", WAITTIME);
		in_rpt("creation time = 0x%x, current time = 0x%x, difference in seconds is %d", shmid_ds3.shm_ctime, timen, timen - shmid_ds3.shm_ctime);
	}
	else
		PATH_TRACE;

	(void)shmctl(shmid3, IPC_RMID, &shmid_ds3);

	if (!test_deleted && !test_fail)
		PATH_XS_RPT(15);

	DBUG_VOID_RETURN;
}


private void	/* Check that doing shmget with IPC_CREAT flag for the */
test4()		/* same key a 2nd time returns the shmid created 1st time */
{
	int pathok = setupok;
	int shmid4;
	int rval;
	int err;
	int id;
	struct shmid_ds shmid_ds4;

	DBUG_ENTER("test4");

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	Key4 = ftok(TOKFILE, ID4);
	err = errno;
	if (Key4 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID4, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	errno = NOERROR;
	shmid4 = shmget(Key4, SEGSIZE, MODERWANY | IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid4;
	if (shmid4 == SYSERROR) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmget(Key4, %d, 0%o | IPC_CREAT) returned -1 with errno %d (%s)", SEGSIZE, MODERWANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = pipe(pip_fds);
	err = errno;
	if (rval == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("pipe(pip_fds) returned -1 with errno %d (%s)\n", err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	errno = NOERROR;
	rval = cppair(ch_t4, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);
	err = errno;
	if (rval&NOK) {
		/* failure/deletion was already reported in pa_t1 or ch_t1 */
		test_deleted++;
	}
	else if ((rval&PATH_ERR_EXIT) == 0)
		PATH_TRACE;

	errno = NOERROR;
	if ((rval = read(pip_fds[RD_FD], (char *)&id, sizeof(id))) <= 0) {
		err = errno;
		if (!test_deleted)
			xx_rpt(DELETION);
		in_rpt("read(pip_fds[%d], (char *)id, sizeof(id)) returned %d with errno %d (%s)", RD_FD, rval, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( shmid4 != id ) {
		if (!test_deleted)
			xx_rpt(FAILURE);
		in_rpt("expected id returned to be %d received %d", shmid4, id);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void)close(pip_fds[RD_FD]);
	(void)close(pip_fds[WR_FD]);
	(void)shmctl(shmid4, IPC_RMID, &shmid_ds4);

	if (!test_fail && !test_deleted)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void 	/* Child does 2nd IPC_CREAT and pipes shmid back to parent */
ch_t4()
{
	int ch_shmid;
	int pathok = setupok;
	int err;

	DBUG_ENTER("ch_t4");
	errno = NOERROR;
	ch_shmid = shmget(Key4, SEGSIZE, MODERWANY|IPC_CREAT);
	err = errno;
	(void) write(pip_fds[WR_FD], (char *)(&ch_shmid), sizeof(ch_shmid));
	if (ch_shmid == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("shmget(Key4, %d, 0%o | IPC_CREAT) of existing shared memory segment returned -1 with errno %d (%s)\n", SEGSIZE, MODERWANY, err, errname(err));
		test_fail++;
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;


	if (pathok == 1) 
		DBUG_VOID_RETURN;
	
	DBUG_EXIT(PATH_ERR_EXIT);
}
	
private void	/* Check to make sure attempts to get shm segments of larger */
test5()		/* size than they were created with fails */
{
	int pathok = setupok;
	int shmid5;
	int err, rval;
	struct shmid_ds shmid_ds5;

	DBUG_ENTER("test5");

	test_deleted = 0;

	errno = NOERROR;
	Key5 = ftok(TOKFILE, ID5);
	err = errno;
	if (Key5 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID5, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	errno = NOERROR;
	shmid5 = shmget(Key5, SEGSIZE, MODERWANY | IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid5;
	if (shmid5 == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmget(Key5, %d, 0%o|IPC_CREAT) failed during test setup, returned -1 with errno %d (%s)", SEGSIZE, MODERWANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = cppair(ch_t5, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);
	err = errno;
	if (rval&NOK) {
		/* failure/deletion was already reported in pa_t1 or ch_t1 */
		test_deleted++;
	}
	else if ((rval&PATH_ERR_EXIT) == 0)
		PATH_TRACE;

	(void)shmctl(shmid5, IPC_RMID, &shmid_ds5);

	if (!test_deleted)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void	/* Try to get shm segment twice the size of the original */
ch_t5()		/* for the same key */
{
	int ch_shmid;
	int err;
	int pathok = setupok;

	DBUG_ENTER("ch_t5");
	errno = NOERROR;
	ch_shmid = shmget(Key5, SEGSIZE*2, MODERWANY);
	err = errno;
	if (ch_shmid != SYSERROR || err != EINVAL) {
		xx_rpt(FAILURE);
		in_rpt("shmget(Key5, SEGSIZE*2, 0%o) of existing shm segment did not fail as expected for segment size larger than original segment - returned errno %d (%s)\n", MODERWANY, err, errname(err));
		test_fail++;
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok == 1) 
		DBUG_VOID_RETURN;
	
	DBUG_EXIT(PATH_ERR_EXIT);
}

private void	/* Create shm segment with read-only perms and then have */
test6()		/* have child attempt to get segment for writing */
{
	int pathok = setupok;
	int shmid6;
	int err, rval;
	struct shmid_ds shmid_ds6;

	DBUG_ENTER("test6");

	test_deleted = 0;

	errno = NOERROR;
	Key6 = ftok(TOKFILE, ID6);
	err = errno;
	if (Key6 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID6, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	errno = NOERROR;
	shmid6 = shmget(Key6, SEGSIZE, (int)MODEURDONLY | IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid6;
	if (shmid6 == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmget(Key6, %d, 0%o|IPC_CREAT) failed during test setup, returned -1 with errno %d (%s)", SEGSIZE, MODEURDONLY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = cppair(ch_t6, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);
	err = errno;
	if (rval&NOK) {
		/* failure/deletion was already reported in pa_t1 or ch_t1 */
		test_deleted++;
	}
	else if ((rval&PATH_ERR_EXIT) == 0)
		PATH_TRACE;

	(void)shmctl(shmid6, IPC_RMID, &shmid_ds6);

	if (!test_deleted)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void	/* Now try to get read-only segment for writing */
ch_t6()
{
	int ch_shmid;
	int err;
	int pathok = setupok;

	DBUG_ENTER("ch_t6");
	errno = NOERROR;
	ch_shmid = shmget(Key6, SEGSIZE, (int)MODERWANY);
	err = errno;
	if (ch_shmid != SYSERROR || err != EACCES) {
		if (!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("shmget(Key6, %d, 0%o) of existing shm segment without appropriate permissions did not fail as expected", SEGSIZE, MODERWANY);
		valrpt(SYSERROR, ch_shmid, EACCES, err);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;


	if (pathok == 1) 
		DBUG_VOID_RETURN;
	
	DBUG_EXIT(PATH_ERR_EXIT);
}


private void	/* Try to get a shmid that doesn't exist.  We use ftok(3) to */
test7()		/* obtain the key */
{
	int pathok = setupok;
	int shmid7;
	int err;

	DBUG_ENTER("test7");

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	Key7 = ftok(TOKFILE, ID7);
	err = errno;
	if (Key7 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID7, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	errno = NOERROR;
	shmid7 = shmget(Key7, SEGSIZE, MODERWANY);
	err = errno;
	g_shmids[tet_thistest-1] = shmid7;
	if (shmid7 != SYSERROR || err != ENOENT) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("shmget(Key7, %d, 0%o) when no existing shared memory segment associated with key and when IPC_CREAT is not set, did not fail as expected", SEGSIZE, MODERWANY);
		valrpt(SYSERROR,shmid7,ENOENT,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!test_fail && !test_deleted) 
		PATH_XS_RPT(2);
	DBUG_VOID_RETURN;
}

private void	/* Test that getting a previously created shm segment */
test8()		/* with IPC_EXCL flag set fails */
{
	int pathok = setupok;
	int shmid8;
	int err, rval;
	struct shmid_ds shmid_ds8;

	DBUG_ENTER("test8");

	test_deleted = 0;

	errno = NOERROR;
	Key8 = ftok(TOKFILE, ID8);
	err = errno;
	if (Key8 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("ftok(%s,%d) returned -1 with errno %d (%s)", TOKFILE, ID8, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;	

	errno = NOERROR;
	shmid8 = shmget(Key8, SEGSIZE, MODERWANY | IPC_CREAT);
	err = errno;
	g_shmids[tet_thistest-1] = shmid8;
	if (shmid8 == SYSERROR) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("shmget(Key8, %d, 0%o|IPC_CREAT) failed during test setup, returned -1 with errno %d (%s)", SEGSIZE, MODERWANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = cppair(ch_t8, NULLFN, WAITTIME, NOK|PATH_ERR_EXIT);
	err = errno;
	if (rval&NOK) {
		/* failure/deletion was already reported in pa_t1 or ch_t1 */
		test_deleted++;
	}
	else if ((rval&PATH_ERR_EXIT) == 0)
		PATH_TRACE;

	(void)shmctl(shmid8, IPC_RMID, &shmid_ds8);

	if (!test_deleted)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void	/* Now try to create this shm segment exclusively - no dice */
ch_t8()
{
	int ch_shmid;
	int err;
	int pathok = setupok;

	DBUG_ENTER("ch_t8");
	errno = NOERROR;
	ch_shmid = shmget(Key8, SEGSIZE, MODERWANY|IPC_CREAT|IPC_EXCL);
	err = errno;
	if (ch_shmid != SYSERROR || err != EEXIST) {
		xx_rpt(FAILURE);
		in_rpt("shmget(Key8, %d, 0%o|IPC_CREAT|IPC_EXCL) of existing shm segment with IPC_EXCL set did not fail as expected", SEGSIZE, MODERWANY);
		valrpt(SYSERROR, ch_shmid, EEXIST, err);
		test_fail++;
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;


	if (pathok == 1) 
		DBUG_VOID_RETURN;
	
	DBUG_EXIT(PATH_ERR_EXIT);
}

private void
test9()
{
	int	err, rval, shmflg = 0;
	key_t	key = IPC_PRIVATE;
	size_t	size = SEGSIZE;
	int	pathok = setupok;

	DBUG_ENTER("test9");

	if(tsup)
	{
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	rval = shmget(key, size, shmflg);
	err = errno;

	if(rval != -1 || err != ENOSYS)
	{
		xx_rpt(FAILURE);

		if(rval != -1)
		{
			in_rpt("shmget() did not return expected value");
			in_rpt("Expected return value -1");
			in_rpt("Actual   return value %d", rval);
		}

		if(err != ENOSYS)
		{
			in_rpt("shmget() did not set errno correctly");
			in_rpt("Expected errno %d (ENOSYS)", ENOSYS);
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}
