/*
*      SCCS:   @(#)DYNL.os/libdl/dlclose/dlclose.c	Rel 5.1.1 (04/07/97)
*
* (C) Copyright 1997 UniSoft Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* UNIX is a registered trademark and X/Open, the `X Device' and The Open
* Group are trademarks of The Open Group.    
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#) DYNL.os/libdl/dlclose - T.dlclose Rel 5.1.1 (04/07/97)";
static	char	*copyright[] = {
		"(C) Copyright 1997 UniSoft Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.dlclose

PROJECT:	VSX5

DESCRIPTION:	Testset for the dlclose subroutine call.

AUTHOR:		Tony Fisher, UniSoft Ltd.
DATE CREATED:	27 February 1997
MODIFICATIONS:	

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dlfcn.h>
#include	<dbug.h>
#include	<report.h>
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef dlclose
#endif

#define	NO_TESTS	2		/* number of tests in testset */

#define	TEST2_SYM1	"test2_var1"

private	char	*dynlib1 = "../dynlibtest.1.so";
private	char	*dynlib2 = "../dynlibtest.2.1.so";

int	no_tests = NO_TESTS;

private	void	test1();
private	void	test2();


public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	NULL, 0
};

private void 
test1()
{
	void *handle;
	int close_ret = -1;
	int pathok = 0;
	
	DBUG_ENTER("test1");
	
	/* open a dynamic link object */
	if((handle = (void *)dlopen(dynlib1, RTLD_NOW)) == (void *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW) failed - %s", 
			dynlib1, dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* close the dynamic linked object handle */
	/* check for a zero return code */
	close_ret = (int) dlclose(handle);
	if(close_ret != 0)
	{
		xx_rpt(DELETION);
		in_rpt("dlclose() failed for \"%s\". returned <%d> when expecting zero.", dynlib1, close_ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	
	PATH_XS_RPT(2);
	DBUG_VOID_RETURN;
}



private void 
test2()
{
	
#if DYNL_DEPOBJ_SUPPORTED	/* dynamic objects can have dependancies */
	void 	*handle1, *handle2;
	int	pathok = 0;
	int	close_ret = -1;
#endif
	
	DBUG_ENTER("test2");

#if DYNL_DEPOBJ_SUPPORTED	/* dynamic objects can have dependancies */
	/*
	 * Open dynlib2 twice to get two handles to the same shared library.
	 */
	if((handle1 = dlopen(dynlib2, RTLD_NOW)) == (void *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW) failed - %s", 
			dynlib2, dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((handle2 = dlopen(dynlib2, RTLD_NOW)) == (void *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW) failed - %s", 
			dynlib2, dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Verify that dlsym succeeds using both handles.
	 */
	if(dlsym(handle1, TEST2_SYM1) == (void *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlsym(%#x, \"%s\") failed for \"%s\"", 
			handle1, TEST2_SYM1, dynlib2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(dlsym(handle2, TEST2_SYM1) == (void *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlsym(%#x, \"%s\") failed for \"%s\"", 
			handle2, TEST2_SYM1, dynlib2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Close handle 1 and verify that the call
	 * returns zero. Check that a call to
	 * dlsym using handle 2 still succeeds.
	 */
	close_ret = dlclose(handle1);
	if(close_ret != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("dlclose() failed for \"%s\". returned <%d> when expecting zero.", dynlib2, close_ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Verify that using handle 2 dlsym still succeeds.
	 */
	if(dlsym(handle2, TEST2_SYM1) == (void *)NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("dlsym(%#x, \"%s\") failed for \"%s\"", 
			handle2, TEST2_SYM1, dynlib2);
		in_rpt("using the second handle after handle one is closed");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(6);
	}

	(void) dlclose(handle2);
	DBUG_VOID_RETURN;

#else		/* dynamic object dependancies unsupported */

	xx_rpt(UNSUPPORTED);
	in_rpt("Dynamic object dependancies not supported.");
	DBUG_VOID_RETURN;

	
#endif		/* end !DYNL_DEPOBJ_SUPPORTED */
	
}

