/*
*      SCCS:   @(#)DYNL.os/libdl/dlerror/dlerror.c	Rel 5.1.1 (04/07/97)
*
* (C) Copyright 1997 UniSoft Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* UNIX is a registered trademark and X/Open, the `X Device' and The Open
* Group are trademarks of The Open Group.    
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#) DYNL.os/libdl/dlerror - T.dlerror Rel 5.1.1 (04/07/97)";
static	char	*copyright[] = {
		"(C) Copyright 1997 UniSoft Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.dlerror

PROJECT:	VSX5

DESCRIPTION:	Testset for the dlerror subroutine call.

AUTHOR:		Tony Fisher, UniSoft Ltd.
DATE CREATED:	27 February 1997
MODIFICATIONS:	

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<string.h>
#include	<dlfcn.h>
#include	<dbug.h>
#include	<report.h>
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef dlerror
#endif

#define	NO_TESTS	2		/* number of tests in testset */

#define NOTAFILE	"./notafile"
#define NOTASYM		"notasym"

int	no_tests = NO_TESTS;

private	void	test1();
private	void	test2();

private	char	*dynlib = "../dynlib.so";

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	NULL, 0
};

private void 
test1()
{
	char	*str, *endstr;
	void	*handle;
	int 	testfail = 0;
	int 	pathok = 0;
	int	len = 0;

	DBUG_ENTER("test1");

	/* check dlerror() when error occurs during dlclose() */
	
	if((handle = dlopen(dynlib, RTLD_NOW)) == (void *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW) failed - %s", 
			dynlib, dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(dlclose(handle) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("dlclose() failed for \"%s\"", dynlib);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(dlclose(handle) == 0)
	{
		xx_rpt(DELETION);
		in_rpt("dlclose() did not return non-zero");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((str = dlerror()) == (char *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlerror() returned NULL");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(*str == '\0')
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("When an error occured calling dlclose()");
		in_rpt("dlerror() did not return a string");
	}
	else
		PATH_TRACE;

	/* check no trailing new-line */
	len = strlen(str);
	endstr = str + len;
	if(*endstr == '\n')
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("When an error occured calling dlclose()");
		in_rpt("dlerror() appended a new-line to error string");
	}
	else
		PATH_TRACE;
	
	/* check dlerror() when error occurs during dlopen() */	

	(void) unlink(NOTAFILE);
	if(dlopen(NOTAFILE, RTLD_NOW) != (void *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW) did not fail", NOTAFILE);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((str = dlerror()) == (char *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlerror() returned NULL");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(*str == '\0')
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("When an error occured calling dlopen()");
		in_rpt("dlerror() did not return a string");
	}
	else
		PATH_TRACE;

	/* check no trailing new-line */
	len = strlen(str);
	endstr = str + len;
	if(*endstr == '\n')
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("When an error occured calling dlclose()");
		in_rpt("dlerror() appended a new-line to error string");
	}
	else
		PATH_TRACE;
	
	/* check dlerror() when error occurs during dlsym() */	

	if(dlsym((void *)NULL, NOTASYM) != (void *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlsym(NULL, \"%s\") did not fail", NOTASYM);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((str = dlerror()) == (char *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlerror() returned NULL");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(*str == '\0')
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("When an error occured calling dlsym()");
		in_rpt("dlerror() did not return a string");
	}
	else
		PATH_TRACE;

	/* check no trailing new-line */
	len = strlen(str);
	endstr = str + len;
	if(*endstr == '\n')
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("When an error occured calling dlclose()");
		in_rpt("dlerror() appended a new-line to error string");
	}
	else
		PATH_TRACE;

	if(!testfail)
		PATH_XS_RPT(14);

	DBUG_VOID_RETURN;
}

private void 
test2()
{
	void	*handle;
	int 	pathok = 0;

	DBUG_ENTER("test2");

	/* firstly generate an error condition */
	
	if((handle = dlopen(dynlib, RTLD_NOW)) == (void *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW) failed - %s", 
			dynlib, dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(dlclose(handle) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("dlclose() failed for \"%s\"", dynlib);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(dlclose(handle) == 0)
	{
		xx_rpt(DELETION);
		in_rpt("dlclose() did not return non-zero");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check the error */

	if(dlerror() == (char *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("When an error occured calling dlclose()");
		in_rpt("dlerror() returned NULL");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check calling dlerror twice returns NULL */
	
	if(dlerror() != (char *)NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("When no error has occured dlerror() did not return NULL");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	DBUG_VOID_RETURN;
}

