/*
*      SCCS:   @(#)DYNL.os/libdl/dlopen/dlopen.c	Rel 5.1.5 (06/18/98)
*
* (C) Copyright 1997 UniSoft Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* UNIX is a registered trademark and X/Open, the `X Device' and The Open
* Group are trademarks of The Open Group.    
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#) DYNL.os/libdl/dlopen - T.dlopen Rel 5.1.5 (06/18/98)";
static	char	*copyright[] = {
		"(C) Copyright 1997 UniSoft Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.dlopen

PROJECT:	VSX5

DESCRIPTION:	Testset for the dlopen subroutine call.

AUTHOR:		Tony Fisher, UniSoft Ltd.
DATE CREATED:	28 February 1997.
MODIFICATIONS:
	Geoff Clare, 9 Sept 1997
		Change test4_func() declaration to public.

	Geoff Clare, 20 Oct 1997
		Open the "global symbol object" in test 12.
		Compare symbol addresses in test 8, not handles.

	Geoff Clare, 3 March 1998
		Change test12_func() forward declaration to public
		(to match definition).

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<limits.h>
#include	<dbug.h>
#include	<dlfcn.h>
#include	<report.h>
#include 	<tet_api.h>
#include	"defines.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef dlopen
#endif

#define	NO_TESTS	18		/* number of tests in testset 	*/

int	no_tests = NO_TESTS;

extern	char	*errname();
extern	int	cp();
extern	void	resetord();

private	void	test1();
private	void	ch_t1();
private	void	test2();
private	void	ch_t2();
private	void	test3();
private	void	ch_t3();
private	void	test4();
private	void	ch_t4();
public	void	test4_func();
private	void	test5();
private	void	ch_t5();
private	void	test6();
private	void	ch_t6();
private	void	test7();
private	void	ch_t7();
private	void	test8();
private	void	ch_t8();
private	void	test9();
private	void	ch_t9();
private	void	test10();
private	void	ch_t10();
private	void	test11();
private	void	ch_t11();
private	void	test12();
public	void	test12_func();
private	void	ch_t12();
private	void	test13();
private	void	ch_t13();
private	void	test14();
private	void	ch_t14();
private	void	test15();
private	void	ch_t15();
private	void	test16();
private	void	ch_t16();
private	void	test17();
private	void	ch_t17();
private	void	test18();
private	void	ch_t18();


private	char	*libc = "libc.so.1";
private	char	*libtest1 = "../dynlibtest.1.so";
private	char	*lib2dtneeded_1 = "../dynlibtest.2.1.so";
private	char	*libtest3_1 = "../dynlibtest.3.so";
private	char	*libtest3_2 = "/tmp/dynlibtest.3.so";
private	char	*lib3dtneeded_1 = "../dynlibtest.3.1.so";
private	char	*libtest4 = "../dynlibtest.4.so";
private char    *prog5  = "../test5";
private char    *prog6  = "../test6";
private char    *prog7  = "../test7";
private	char	*libtest8 = "../dynlibtest.8.so";
private char    *prog9  = "../test9";
private char    *prog10  = "../test9"; /* 10 uses same programs as 9 */
private	char	*libtest11_1 = "../dynlibtest.11.1.so"; /* used in 11 & 13 */
private	char	*libtest11_2 = "../dynlibtest.11.2.so"; /* used in 11 & 13 */
private	char	*libtest12g = "../dynlibtest.12g.so";
private	char	*libtest14_1 = "../dynlibtest.14.1.so";
private	char	*libtest14_3 = "../dynlibtest.14.3.so";
private char    *libmissing  = "../libmissing.so";
private char    *libnotread  = "../libnotread.so";
private char    *libnotadso  = "../libnotadso.so";
private char	*libtest18 = "../dynlib.18.so";

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	test3, 3,
	test4, 4,
	test5, 5,
	test6, 6,
	test7, 7,
	test8, 8,
	test9, 9,
	test10, 10,
	test11, 11,
	test12, 12,
	test13, 13,
	test14, 14,
	test15, 15,
	test16, 16,
	test17, 17,
	test18, 18,
	NULL, 0
};

private void
test1()
{
	DBUG_ENTER("test1");

	(void) tet_fork(ch_t1, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
}

private void 
ch_t1()
{
	int	pathok = 0;
	int	*addr;
	void 	*handle;

	DBUG_ENTER("ch_t1");

	if((handle = dlopen(libtest1, RTLD_NOW)) == (void *)NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("dlopen(\"%s\", RTLD_NOW) failed", libtest1);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((addr = (int *)dlsym(handle, TEST1_SYM)) == (int *)NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("dlsym(handle, \"%s\") failed", TEST1_SYM);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(*addr != TEST1_VAL)
	{
		xx_rpt(FAILURE);
		in_rpt("Handle returned by dlopen() when used with dlsym()");
		in_rpt("did not return expected value for test symbol");
		in_rpt("Expected value <%d>, actual value <%d>",
		       TEST1_VAL, *addr);
	}
	else
		PATH_XS_RPT(2);

	(void) dlclose(handle);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

#if DYNL_DEPOBJ_SUPPORTED     /* dynamic object dependancies supported */
	
	(void) tet_fork(ch_t2, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
	
#else	      /* dynamic object dependancy unsupported */

	xx_rpt(UNSUPPORTED);
	in_rpt("Dynamic object dependancies not supported.");
	DBUG_VOID_RETURN;

#endif		/* end !DYNL_DEPOBJ_SUPPORTED */
}

private void 
ch_t2()
{
	void	*handle;
	int 	*val1, *val2, *val3;
	int	testfail = 0;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t2");

	/*
	 * Open DT_NEEDED dynamic object
	 */
	if((handle = dlopen(lib2dtneeded_1, RTLD_NOW)) == NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("dlopen() failed to load shared object %s", 
			lib2dtneeded_1);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* get the first symbol */
	if((val1 = (int *)dlsym(handle, TEST2_SYM1)) == NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("dlsym(handle, \"%s\") failed for \"%s\"", 
			TEST2_SYM1, lib2dtneeded_1);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	/* get the second symbol */
	if((val2 = dlsym(handle, TEST2_SYM2)) == NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("dlsym(handle, \"%s\") failed for \"%s\"", 
			TEST2_SYM2, lib2dtneeded_1);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* get the third symbol */
	if((val3 = dlsym(handle, TEST2_SYM3)) == NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("dlsym(handle, \"%s\") failed for \"%s\"", 
			TEST2_SYM3, lib2dtneeded_1);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check they have expected values */
	if(*val1 != TEST2_VAL1 || *val2 != TEST2_VAL2 || *val3 != TEST2_VAL3)
	{
		xx_rpt(FAILURE);
		in_rpt("dlsym() did not return symbols in correct dependancy order");
		in_rpt("Got values %d %d %d when expecting %d %d %d", *val1, *val2, *val3, TEST2_VAL1, TEST2_VAL2, TEST2_VAL3);
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	
	(void) dlclose(handle);

	if(!testfail)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) tet_fork(ch_t3, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
}

private void 
ch_t3()
{
	void	*handle;
	int	testfail = 0;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t3");

	/*
	 * Test for a single slash in pathname
	 */
	if((handle = dlopen(libtest3_1, RTLD_NOW)) == NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("When pathname contains one slash character");
		in_rpt("dlopen() failed to locate shared object %s", 
			libtest3_1);
	}
	else
		PATH_TRACE;

	(void) dlclose(handle);

	/*
	 * Test for two slashes in pathname
	 */
	if(cp(libtest3_1, libtest3_2))
	{
		xx_rpt(DELETION);
		in_rpt("Unable to copy %s to %s",libtest3_1, libtest3_2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((handle = dlopen(libtest3_2, RTLD_NOW)) == NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("When pathname contains two slash characters");
		in_rpt("dlopen() failed to locate shared object %s", 
			libtest3_2);
	}
	else
		PATH_TRACE;

	(void) unlink(libtest3_2);
	(void) dlclose(handle);

#if DYNL_DEPOBJ_SUPPORTED	/* dynamic dependancies supported */

	/*
	 * Test for slash characters in DT_NEEDED
	 */
	if((handle = dlopen(lib3dtneeded_1, RTLD_NOW)) == NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("When DT_NEEDED pathname contains slash characters");
		in_rpt("dlopen() failed to locate shared object %s", 
			lib3dtneeded_1);
	}
	else
		PATH_TRACE;

	(void) dlclose(handle);

	if(!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;

#else		/* dynamic dependancies unsupported */
	
	if(!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;

#endif
}

private void 
test4()
{
	DBUG_ENTER("test4");

		(void) tet_fork(ch_t4, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	void	*handle1;
	void	*handle2;
	void	(*fptr)();
	int	*ptr;
	int	nfails = 0;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	/* open a dynamic object RTLD_GLOBAL */
	if((handle1 = dlopen(libtest4, RTLD_NOW|RTLD_GLOBAL)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW|RTLD_GLOBAL) failed : %s",
			libtest4, dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* open global object */
	if((handle2 = dlopen((char *)0, RTLD_NOW)) == NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("dlopen(0, RTLD_NOW) failed : %s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* test objects in executable image */
	if((fptr = (void (*)()) dlsym(handle2, "test4_func")) == NULL)
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("dlopen(0, RTLD_NOW) did not allow access to symbol \"test4_func\" in the executable image:");
		in_rpt("dlsym(handle2, \"test4_func\") failed : %s", dlerror());
	}
	else if (fptr != test4_func)
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("dlopen(0, RTLD_NOW) did not allow access to symbol \"test4_func\" in the executable image:");
		in_rpt("address returned by dlsym(handle2, \"test4_func\") = 0x%lx", (long)fptr);
		in_rpt("actual address of test4_func = 0x%lx", (long)test4_func);
	}
	else
		PATH_TRACE;

	/* test shared objects loaded with executable image */
	if((fptr = (void (*)()) dlsym(handle2, "resetord")) == 0)
	{
		if (nfails++ == 0)
			xx_rpt(FAILURE);
		in_rpt("dlopen(0, RTLD_NOW) did not allow access to symbol \"resetord\" in object loaded at startup:");
		in_rpt("dlsym(handle2, \"resetord\") failed : %s", dlerror());
	}
	else if (fptr != resetord)
	{
		if (nfails++ == 0)
			xx_rpt(FAILURE);
		in_rpt("dlopen(0, RTLD_NOW) did not allow access to symbol \"resetord\" in object loaded at startup:");
		in_rpt("address returned by dlsym(handle, \"resetord\") = 0x%lx",
			(long)fptr);
		in_rpt("actual address of resetord = 0x%lx", (long)resetord);
	}
	else
		PATH_TRACE;

	/* test objects dlopen'ed with RTLD_GLOBAL set */
	if((ptr = dlsym(handle2, "test4_val")) == 0)
	{
		if (nfails++ == 0)
			xx_rpt(FAILURE);
		in_rpt("dlopen(0, RTLD_NOW) did not allow access to symbol \"test4_val\" in object loaded with RTLD_GLOBAL:");
		in_rpt("dlsym(handle2, \"test4_val\") failed : %s", dlerror());
	}
	else if(*ptr != TEST4_VAL)
	{
		if (nfails++ == 0)
			xx_rpt(FAILURE);
		in_rpt("dlopen(0, RTLD_NOW) did not allow access to symbol \"test4_val\" in object loaded with RTLD_GLOBAL:");
		in_rpt("value at address returned by dlsym() was %d, expected %d",
			*ptr, TEST4_VAL);
	}
	else
		PATH_TRACE;

	if (!nfails)
		PATH_XS_RPT(5);

	(void) dlclose(handle2);
	(void) dlclose(handle1);

	DBUG_VOID_RETURN;
}

public void 
test4_func()
{
	DBUG_ENTER("test4_func");
	DBUG_VOID_RETURN;
}

private void 
test5()
{
	int	pathok = 0;

	DBUG_ENTER("test5");

#if DYNL_DEPOBJ_SUPPORTED	/* dynamic dependant objects supported */
	
	/* fork off another process, which shoyld exec to test5 */
	/* on return check the exit status */
	if(tet_fork(ch_t5, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK) != EXIT_OK)
	{
		xx_rpt(FAILURE);
		in_rpt("dlopen() loaded shared object twice");
	}
	else
		PATH_TRACE;
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;

#else		/* dynamic dependancies unsupported */

	xx_rpt(UNSUPPORTED);
	in_rpt("Dynamic object dependancies not supported.");
	DBUG_VOID_RETURN;

#endif		/* end !DYNL_DEPOBJ_SUPPORTED */
}

private void
ch_t5()
{
	int 	err;
	char	*args[4];
	extern	char	*vg_ident;
	extern	char	*vg_dbug;
	extern	char	**environ;

	DBUG_ENTER("ch_t5");

	args[0] = prog5;
	args[1] = vg_ident;
	args[2] = vg_dbug;
	args[3] = NULL;

	(void) tet_exec(prog5, args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec of %s failed - errno %s (%d)", prog5, errname(err), err);

	DBUG_VOID_RETURN;
}


private void
test6()
{
	DBUG_ENTER("test6");

#if DYNL_DEPOBJ_SUPPORTED	/* dynamic dependant objects supported */

	(void) tet_fork(ch_t6, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
	
#else		/* dynamic dependancies unsupported */

	xx_rpt(UNSUPPORTED);
	in_rpt("Dynamic object dependancies not supported.");
	DBUG_VOID_RETURN;

#endif		/* end !DYNL_DEPOBJ_SUPPORTED */	
}

private void
ch_t6()
{
	int 	err;
	char	*args[4];
	extern	char	*vg_ident;
	extern	char	*vg_dbug;
	extern	char	**environ;

	DBUG_ENTER("ch_t6");

	args[0] = prog6;
	args[1] = vg_ident;
	args[2] = vg_dbug;
	args[3] = NULL;

	(void) tet_exec(prog6, args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec of %s failed - errno %s (%d)", prog6, errname(err), err);

	DBUG_VOID_RETURN;
}

private void 
test7()
{
	int	pathok = 0;

	DBUG_ENTER("test7");

#if DYNL_DEPOBJ_SUPPORTED	/* dynamic dependant objects supported */

	if(tet_fork(ch_t7, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK) != EXIT_OK)
	{
		xx_rpt(FAILURE);
		in_rpt("dlopen() loaded shared object twice");
	}
	else
		PATH_XS_RPT(0);

	DBUG_VOID_RETURN;

#else		/* dynamic dependancies unsupported */

	xx_rpt(UNSUPPORTED);
	in_rpt("Dynamic object dependancies not supported.");
	DBUG_VOID_RETURN;

#endif		/* end !DYNL_DEPOBJ_SUPPORTED */	
}

private void
ch_t7()
{
	int 	err;
	char	*args[4];
	extern	char	*vg_ident;
	extern	char	*vg_dbug;
	extern	char	**environ;

	DBUG_ENTER("ch_t7");

	args[0] = prog7;
	args[1] = vg_ident;
	args[2] = vg_dbug;
	args[3] = NULL;

	(void) tet_exec(prog7, args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec of %s failed - errno %s (%d)", prog7, errname(err), err);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	DBUG_ENTER("test8");

	(void) tet_fork(ch_t8, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
}

private void 
ch_t8()
{
	void	*handle1, *handle2;
	int	err;
	int	*var1, *var2;
	int	pathok = 0;
	char	altpath[_POSIX_PATH_MAX];

	DBUG_ENTER("ch_t8");

	if (getcwd(altpath, sizeof(altpath)) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("getcwd() failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((handle1 = dlopen(libtest8, RTLD_NOW)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen() failed to open shared object %s", libtest8);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) strcat(altpath, "/"); 
	(void) strcat(altpath, libtest8);

	if((handle2 = dlopen(altpath, RTLD_NOW)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen() failed to open shared object %s", altpath);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* get the address of the same object using each handle */
	if((var1 = dlsym(handle1, "test8_var")) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlsym(handle1, \"test8_var\") failed:");
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;		
	}
	else if((var2 = dlsym(handle2, "test8_var")) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlsym(handle2, \"test8_var\") failed:");
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;		
	}
	else
		PATH_TRACE;

	if(var1 != var2)
	{
		xx_rpt(FAILURE);
		in_rpt("Addresses returned by dlsym() differ, indicating second copy of object loaded.");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	(void) dlclose(handle1);
	(void) dlclose(handle2);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	DBUG_ENTER("test9");

	(void) tet_fork(ch_t9, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int 	err;
	char	*args[5];
	extern	char	*vg_ident;
	extern	char	*vg_dbug;
	extern	char	**environ;

	DBUG_ENTER("ch_t9");

	args[0] = prog9;
	args[1] = "9";
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;

	(void) tet_exec(prog9, args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec of %s failed - errno %s (%d)", prog9, errname(err), err);

	DBUG_VOID_RETURN;
}


private void
test10()
{
	DBUG_ENTER("test10");

	(void) tet_fork(ch_t10, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	int 	err;
	char	*args[5];
	extern	char	*vg_ident;
	extern	char	*vg_dbug;
	extern	char	**environ;

	DBUG_ENTER("ch_t10");

	args[0] = prog10;
	args[1] = "10";
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;

	(void) tet_exec(prog10, args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec of %s failed - errno %s (%d)", prog10, errname(err), err);

	DBUG_VOID_RETURN;
}

private void 
test11()
{
	DBUG_ENTER("test11");

	(void) tet_fork(ch_t11, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	void	*handle1;
	void	*handle2;
	int	(*fptr)();
	int	val;
	int	pathok = 0;

	DBUG_ENTER("ch_t11");

	if((handle1 = dlopen(libtest11_1, RTLD_NOW|RTLD_GLOBAL)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW|RTLD_GLOBAL) failed : %s",
			libtest11_1, dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((handle2 = dlopen(libtest11_2, RTLD_NOW)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW) failed : %s",
			libtest11_2, dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((fptr = (int (*)()) dlsym(handle2, "test11_func")) == 0)
	{
		xx_rpt(DELETION);
		in_rpt("dlsym() failed : %s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	val = (*fptr)();
	if(val != TEST11_VAL)
	{
		xx_rpt(FAILURE);
		in_rpt("value returned by test11_func() was %d, expected %d",
			val, TEST11_VAL);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(4);

	(void) dlclose(handle2);
	(void) dlclose(handle1);

	DBUG_VOID_RETURN;
}

private void 
test12()
{
	DBUG_ENTER("test12");

	(void) tet_fork(ch_t12, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	void	*handle1;
	void	*handle2;
	void	(*fptr)();
	int	*ptr;
	int	nfails = 0;
	int	pathok = 0;

	DBUG_ENTER("ch_t12");

	if((handle1 = dlopen(libtest12g, RTLD_NOW|RTLD_GLOBAL)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW|RTLD_GLOBAL) failed : %s",
			libtest12g, dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((handle2 = dlopen((char *)0, RTLD_NOW)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(0, RTLD_NOW) failed : %s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check that can get the function defined in this source file */
	if((fptr = (void (*)()) dlsym(handle2, "test12_func")) == 0)
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("dlsym(handle, \"test12_func\") failed : %s", dlerror());
	}
	else if (fptr != test12_func)
	{
		nfails++;
		xx_rpt(FAILURE);
		in_rpt("dlopen() did not search a.out first:");
		in_rpt("address returned by dlsym(handle, \"test12_func\") = 0x%lx",
			(long)fptr);
		in_rpt("actual address of test12_func in a.out = 0x%lx", (long)test12_func);
	}
	else
		PATH_TRACE;

	/* check that we get the function defined in the */
	/* shared object linked in (static linked) at start up */
	if((fptr = (void (*)()) dlsym(handle2, "resetord")) == 0)
	{
		if (nfails++ == 0)
			xx_rpt(FAILURE);
		in_rpt("dlsym(handle, \"resetord\") failed : %s", dlerror());
	}
	else if (fptr != resetord)
	{
		if (nfails++ == 0)
			xx_rpt(FAILURE);
		in_rpt("dlopen() did not search a.out's needed objects before object loaded with RTLD_GLOBAL:");
		in_rpt("address returned by dlsym(handle, \"resetord\") = 0x%lx",
			(long)fptr);
		in_rpt("actual address of resetord in needed object = 0x%lx",
			(long)resetord);
	}
	else
		PATH_TRACE;
		
	if((ptr = dlsym(handle2, "test12_val")) == 0)
	{
		if (nfails++ == 0)
			xx_rpt(FAILURE);
		in_rpt("dlopen() did not search object loaded with RTLD_GLOBAL:");
		in_rpt("dlsym(handle, \"test12_val\") failed : %s", dlerror());
	}
	else if(*ptr != TEST12_VAL)
	{
		if (nfails++ == 0)
			xx_rpt(FAILURE);
		in_rpt("dlopen() did not search object loaded with RTLD_GLOBAL:");
		in_rpt("value at address returned by dlsym() was %d, expected %d",
			*ptr, TEST12_VAL);
	}
	else
		PATH_TRACE;

	if (!nfails)
		PATH_XS_RPT(5);

	(void) dlclose(handle2);
	(void) dlclose(handle1);

	DBUG_VOID_RETURN;
}

public void 
test12_func()
{
	DBUG_ENTER("test12_func");
	DBUG_VOID_RETURN;
}

private void 
test13()
{
	DBUG_ENTER("test13");

	(void) tet_fork(ch_t13, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t13()
{
	void	*handle;
	int	pathok = 0;

	DBUG_ENTER("ch_t13");

	/* Use same libraries as test 11 */

	if((handle = dlopen(libtest11_1, RTLD_NOW|RTLD_LOCAL)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW|RTLD_LOCAL) failed : %s",
			libtest11_1, dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* libtest11_1 was loaded with RTLD_LOCAL, so this should
	   now fail, although it should have succeeded in test 11 */

	if(dlopen(libtest11_2, RTLD_NOW) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("dlopen(\"%s\", RTLD_NOW) did not return NULL",
			libtest11_2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(2);

	(void) dlclose(handle);

	DBUG_VOID_RETURN;
}


private void 
test14()
{
	DBUG_ENTER("test14");

#if DYNL_DEPOBJ_SUPPORTED     /* dynamic object dependancies supported */

	(void) tet_fork(ch_t14, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;

#else	      /* dynamic object dependancy unsupported */
	
	xx_rpt(UNSUPPORTED);
	in_rpt("Dynamic object dependancies not supported.");
	DBUG_VOID_RETURN;

#endif		/* end !DYNL_DEPOBJ_SUPPORTED */

}

private void
ch_t14()
{
	void	*handle1, *handle2, *handle3;
	int	*val1, *val2, *val3;
	int	pathok = 0;

	DBUG_ENTER("ch_t14");

	/* load an object which others linked (static) to */
	if((handle1 = dlopen(libtest14_3, RTLD_NOW|RTLD_GLOBAL)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW) failed : %s",
			libtest14_3, dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* load the object which depends on previously */
	/* loaded object */
	if((handle2 = dlopen(libtest14_1, RTLD_NOW|RTLD_GLOBAL)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(\"%s\", RTLD_NOW) failed : %s",
			libtest14_1, dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* open the global object */
	/* dlsym using this should use load order to resolve */
	if((handle3 = dlopen((char *)NULL, RTLD_NOW)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("dlopen(NULL, RTLD_NOW) failed : %s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	
	/* ensure have expected value */
	/* i.e. has used LOAD order to resolve */
	/* get the first symbol */
	if((val1 = (int *)dlsym(handle3, TEST14_SYM1)) == NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("dlsym(handle3, \"%s\") failed for \"%s\"", 
			TEST14_SYM1, libtest14_1);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	/* get the second symbol */
	if((val2 = (int *)dlsym(handle3, TEST14_SYM2)) == NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("dlsym(handle3, \"%s\") failed for \"%s\"", 
			TEST14_SYM2, libtest14_1);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* get the third symbol */
	if((val3 = (int *)dlsym(handle3, TEST14_SYM3)) == NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("dlsym(handle3, \"%s\") failed for \"%s\"", 
			TEST14_SYM3, libtest14_1);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check they have expected values */
	if(*val1 != 2 * TEST14_VAL1
	   || *val2 != 2 * TEST14_VAL2
	   || *val3 != 2 * TEST14_VAL3)
	{
		xx_rpt(FAILURE);
		in_rpt("dlsym() did not return symbols in correct load order");
		in_rpt("Got values %d %d %d when expecting %d %d %d", *val1, *val2, *val3, 2 * TEST14_VAL1, 2 * TEST14_VAL2, 2 * TEST14_VAL3);
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
	}

	PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}


private void 
test15()
{
	DBUG_ENTER("test15");
	
	(void) tet_fork(ch_t15, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
}

private void 
ch_t15()
{
	void	*handle;
	char	*errstr;
	int	testfail = 0;
	int	pathok = 0;

	DBUG_ENTER("ch_t15");

	(void) unlink(libmissing);

	/*
	 * dlopen() when DSO does not exist
	 */
	handle = dlopen(libmissing, RTLD_NOW);

	if(handle != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("When the shared object does not exist");
		in_rpt("dlopen() returned non-NULL when expected NULL");
		(void) dlclose(handle);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((errstr = dlerror()) == (char *)NULL)
	{
		xx_rpt(DELETION);	
		in_rpt("dlerror() returned NULL");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(*errstr == '\0')
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("When the shared object does not exist");
		in_rpt("dlerror() did not return a string");
	}
	else
		PATH_TRACE;

	if(!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void 
test16()
{
	DBUG_ENTER("test16");
	
	(void) tet_fork(ch_t16, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
}

private void 
ch_t16()
{
	void	*handle;
	char	*errstr;
	int	err;
	int	testfail;
	int	pathok = 0;
	int	fd;

	DBUG_ENTER("ch_t16");

	errno = NOERROR;
	testfail = 0;

	/*
	 * dlopen() when file cannot be read
	 */
	(void) unlink(libnotread);
	if ((fd = creat(libnotread, (mode_t) S_IWUSR)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(\"%s\", S_IWUSR) failed - errno %d (%s)", 
			libnotread, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	if((handle = dlopen(libnotread, RTLD_NOW)) != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("When the file cannot be read");
		in_rpt("dlopen() failed to return NULL");
		(void) dlclose(handle);
		(void) unlink(libnotread);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((errstr = dlerror()) == (char *)NULL)
	{
		xx_rpt(DELETION);	
		in_rpt("dlerror() returned NULL");
		(void) unlink(libnotread);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(*errstr == '\0')
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("When the file cannot be read");
		in_rpt("dlerror() did not return a string");
	}
	else
		PATH_TRACE;

	(void) unlink(libnotread);

	if(!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


private void 
test17()
{
	DBUG_ENTER("test17");
	
	(void) tet_fork(ch_t17, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

	DBUG_VOID_RETURN;
}

private void 
ch_t17()
{
	void	*handle;
	char	*errstr;
	int	err;
	int	testfail;
	int	pathok = 0;
	int	fd;

	DBUG_ENTER("ch_t17");

	errno = NOERROR;
	testfail = 0;

	/*
	 * dlopen() when file is not a DSO
	 */
	(void) unlink(libnotadso);
	if ((fd = creat(libnotadso, (mode_t) S_IRWXU)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(\"%s\", S_IRWXU) failed - errno %d (%s)", 
			libnotadso, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	handle = dlopen(libnotadso, RTLD_NOW);

	if(handle != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("When the file is not a shared object");
		in_rpt("dlopen() failed to return NULL");
		(void) dlclose(handle);
		(void) unlink(libnotadso);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((errstr = dlerror()) == (char *)NULL)
	{
		xx_rpt(DELETION);	
		in_rpt("dlerror() returned NULL");
		(void) unlink(libnotadso);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(*errstr == '\0')
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("When the file is not a shared object");
		in_rpt("dlerror() did not return a string");
	}
	else
		PATH_TRACE;

	(void) unlink(libnotadso);

	if(!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void 
test18()
{
	DBUG_ENTER("test18");

#if DYNL_NODEFS_SUPPORTED	/* unresovled reference objects */
				/* can be built */
	
	(void) tet_fork(ch_t18, NULLFN, WAITTIME, EXIT_OK|EXIT_NOK);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("Unresolved references in dynamic objects not supported.");

#endif

	DBUG_VOID_RETURN;
}

private void 
ch_t18()
{
	void	*handle;
	char	*errstr;
	int	err;
	int	testfail;
	int	pathok = 0;

	DBUG_ENTER("ch_t18");

	errno = NOERROR;
	testfail = 0;

	/*
	 * dlopen() when unresolved data symbol
	 */
	handle = dlopen(libtest18, RTLD_NOW);

	if(handle != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("When shared object contains an unresolved data symbol");
		in_rpt("dlopen() failed to return NULL");
		(void) dlclose(handle);
	}
	else
		PATH_TRACE;

	if((errstr = dlerror()) == (char *)NULL)
	{
		xx_rpt(DELETION);	
		in_rpt("dlerror() returned NULL");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(*errstr == '\0')
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("When shared object contains an unresolved data symbol");
		in_rpt("dlerror() did not return a string");
	}
	else
		PATH_TRACE;

	if(!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}
