/*
*      SCCS:   @(#)DYNL.os/libdl/dlopen/test5.c	Rel 5.1.4 (11/19/97)
*
* (C) Copyright 1997 UniSoft Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* UNIX is a registered trademark and X/Open, the `X Device' and The Open
* Group are trademarks of The Open Group.    
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#) DYNL.os/libdl/dlopen - test5 Rel 5.1.4 (11/19/97)";
static	char	*copyright[] = {
		"(C) Copyright 1997 UniSoft Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		test5 - used by T.dlopen.

PROJECT:	VSX5

DESCRIPTION:
	  Exec'd by T.dlopen in test5().

		test5 vg_ident vg_dbug

METHODS:	Test5 binary is linked to dynlibtest.5.2.so.
		When exec'ed .so should be loaded. Get address of
		symbol from dynamic object. Load again via
		dynlibteszt.5.1.so with dlopen. Retrieve address
		of variable and check it is same as original address.

AUTHOR:		Tony Fisher, UniSoft Ltd.
DATE CREATED:	28 February 1997
MODIFICATIONS:
	Geoff Clare, 11 Sept 1997
		Change first dlopen() to open dynlibtest.5.2.so
		instead of global handle.

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <stdio.h>
#include <dlfcn.h>
#include <dbug.h>
#include <report.h>
#include "defines.h"
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef dlopen
#endif

public  char    *vg_ident =     "<no ident>";
public  char    *vg_dbug =      (char *) NULL;

private	char	*libtest5_1 = "../dynlibtest.5.1.so";
private	char	*libtest5_2 = "../dynlibtest.5.2.so";

private	void	subtest5();

public	int
tet_main(argc, argv)
int	argc;
char	*argv[];
{

	set_pname(argv[0]);

	if (argc < 1 || argc > 3)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		exit(EXIT_NOK);
	}

	if (argc >= 3)
		vg_dbug = argv[2];
	if (argc >= 2)
		vg_ident = argv[1];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	subtest5();

	exit(EXIT_OK);
	/*NOTREACHED*/
}

private void
subtest5()
{
	void	*handle1, *handle2;
	int	*var1, *var2;
	
	DBUG_ENTER("subtest5");

	/* dlopen libtest5_2 (which contains the test5_var object) */
	if((handle2 = dlopen(libtest5_2, RTLD_NOW)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;		
	}

	/* get the address of the object */
	if((var2 = dlsym(handle2, "test5_var")) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;		
	}

	/* dlopen libtest5_1, which has libtest5_2 as dependency object */
	if((handle1 = dlopen(libtest5_1, RTLD_NOW)) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;
	}

	/* get the address of the (already loaded) object via new handle */
	if((var1 = dlsym(handle1, "test5_var")) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("%s", dlerror());
		DBUG_VOID_RETURN;		
	}

	if(var1 != var2)
	{
		xx_rpt(FAILURE);
		in_rpt("Address of dynamic objects differ, indicating second copy of object loaded.");
		DBUG_VOID_RETURN;
	}

	(void) dlclose(handle1);
	(void) dlclose(handle2);
	
	DBUG_VOID_RETURN;
}	

