/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/mfiles/mmap_P/mmap_P.c
*
* Modifications:
* $Log: mmap_P.c,v $
* Revision 1.2  2001/10/02 13:25:13  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:16  cyeoh
* Adds files for lsb-os test suite
*
*
*/


#include	<vsrt/vsrt.h>
#include	<sys/mman.h>

#define 	NO_TESTS	34	/* number of tests in testset */

#ifdef UNDEF_MACROS
#undef mmap
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();

int	no_tests = NO_TESTS;

private	void	*tb_mmap();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();
private void	test30();
private void	test31();
private void	test32();
private void	test33();
private void	test34();

int	supports_mmap;

/*from vsrtlib/mkmmap.c*/
extern int	vsrt_got_sigsegv;
extern int	vsrt_got_sigbus;
extern int	vsrt_got_sig;
extern unsigned long	vsrt_pgsz;
extern unsigned long	vsrt_ipgsz;
extern int	vsrt_setjmp_called;
extern sigjmp_buf	vsrt_env;

static char     *test_file = "/tmp/vsrt_mmap";

#define	CHILD_1_READY	1
#define	CHILD_2_READY	2
#define	CHILD_2_DONE	3
#define CHILD_1_READY_FOR_3	4
#define CHILD_2_READY_FOR_3	5
#define CHILD_3_READY_FOR_1	6
#define CHILD_3_READY_FOR_2	7

int	errors = 0;

void atstart(void);
void atstart(void)
{
	(void)unlink(test_file);
	vsrt_check_support("MMAP", &supports_mmap);
	VSRT_BEGIN_EVENTS;
}
void atend(void);
void atend(void)
{
	VSRT_END_EVENTS;
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = atend;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	 test29, 29,
	 test30, 30,
	 test31, 31,
	 test32, 32,
	 test33, 33,
	 test34, 34,
	NULL, 0
};

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	A successful call to mmap() when fildes is associated
	with  a  regular  file, the MAP_SHARED flag is set in
	flags, and the PROT_READ flag is set  in  prot  shall
	map  len  bytes  at offset off in the file associated
	with fildes into the address space  of  the  process,
	allow  the  mapped  area  to  be read, and return the
	location in the address space at which the mapping is
	made.
*/

private void test1()
{
	int	fd;
	int	err;
	int	pathok = 0;
	int	i, j;
	int	*addr;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test1");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDWR, (mode_t)VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	if ((addr = (int *)tb_mmap(0, vsrt_pgsz, PROT_READ, MAP_SHARED, fd, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN
	}
	else
		PATH_TRACE;

VSRT_FORK(pid);
	/*Mapped page at that address may be read*/
	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0) {
		for (j = 0; !vsrt_got_sig && j < vsrt_ipgsz; j++) {
			i = addr[j];
			if (i != (j + vsrt_ipgsz)) {
				in_rpt("At offset %d in mapped page expected: %ld  received: %d", j, (j + vsrt_ipgsz), i);
				xx_rpt(FAILURE);
				exit(1);
			}
		}
	}

	if (vsrt_got_sigsegv) {
		in_rpt("SIGSEGV received: offset %d", j*sizeof(addr[0]));
		xx_rpt(FAILURE);
		exit(1);
	}

	if (vsrt_got_sigbus) {
		in_rpt("SIGBUS received: offset %d", j*sizeof(addr[0]));
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)munmap((void *)addr, (size_t)vsrt_pgsz);
	(void)close(fd);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
supports MAP_PRIVATE:
	A successful call to mmap() when fildes is associated
	with  a  regular file, the MAP_PRIVATE flag is set in
	flags, and the PROT_READ flag is set  in  prot  shall
	map  len  bytes  at offset off in the file associated
	with fildes into the address space  of  the  process,
	allow  the  mapped  area  to  be read, and return the
	location in the address space at which the mapping is
	made.
*/

private void test2()
{
	int	fd;
	int	err;
	int	pathok = 0;
	int	i, j;
	int	*addr;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test2");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((addr = (int *)tb_mmap(0, vsrt_pgsz, PROT_READ, MAP_PRIVATE, fd, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
		err = errno;
#ifndef _XOPEN_SOURCE
		if (errno == ENOTSUP)
			xx_rpt(UNSUPPORTED);
		else
#endif
			xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd);
		(void)unlink(test_file);
		DBUG_VOID_RETURN
	}
	else
		PATH_TRACE;

VSRT_FORK(pid);
	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0) {
		for (j = 0; !vsrt_got_sig && j < vsrt_ipgsz; j++) 
		{
			i = addr[j];
			if (i != (j + vsrt_ipgsz)) {
				in_rpt("At offset %d in mapped page expected: %ld  received: %d", j, (j + vsrt_ipgsz), i);
				xx_rpt(FAILURE);
				exit(1);
			}
		}
	}

	if (vsrt_got_sigsegv) {
		in_rpt("SIGSEGV received: offset %d", j*sizeof(addr[0]));
		xx_rpt(FAILURE);
		exit(1);
	}

	if (vsrt_got_sigbus) {
		in_rpt("SIGBUS received: offset %d", j*sizeof(addr[0]));
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)close(fd);
	(void)munmap((void *)addr, (size_t)vsrt_pgsz);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
supports the use of PROT_EXEC:
	A successful call to mmap() when fildes is associated
	with  a  regular  file, the MAP_SHARED flag is set in
	flags, and the PROT_EXEC flag is set  in  prot  shall
	map  len  bytes  at offset off in the file associated
	with fildes into the address space  of  the  process,
	allow  the mapped area to be executed, and return the
	location in the address space at which the mapping is made.
*/

private void test3()
{
	int	err;
	int	pathok = 0;
	int	j;
	int	*addr;
	int	ret;
	char	*(*foo)();
	struct vsrt_exec_resource get;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test3");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	if ((ret = vsrt_create_exec_file(test_file, O_RDONLY, VSRT_PROT_ALL, &get)) != 0)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	errno = 0;
	if ((addr = (int *)tb_mmap(0, 3*vsrt_pgsz, PROT_EXEC, MAP_SHARED, get.fd, 0)) == (int *)MAP_FAILED) {
		if (errno == ENOTSUP) {
			xx_rpt(UNSUPPORTED);
			in_rpt("PROT_EXEC not supported");
			DBUG_VOID_RETURN
		}
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(get.fd);
		(void)unlink(test_file);
		DBUG_VOID_RETURN
	}
	else
		PATH_TRACE;

VSRT_FORK(pid);
	foo = (char *(*)())((long)addr + get.offset);

	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0)
		ret = (int)(*foo)(); 

	if (vsrt_got_sigsegv) {
		in_rpt("SIGSEGV received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (vsrt_got_sigbus) {
		in_rpt("SIGBUS received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (ret != 99) {
		in_rpt("Mapped page could not be executed");
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)close(get.fd);
	(void)munmap((void *)addr, (size_t)3*vsrt_pgsz);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 
/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
supports  the  use of PROT_EXEC and supports the use of
MAP_PRIVATE:
	A successful call to mmap() when fildes is associated
	with  a  regular file, the MAP_PRIVATE flag is set in
	flags, and the PROT_EXEC flag is set  in  prot  shall
	map  len  bytes  at offset off in the file associated
	with fildes into the address space  of  the  process,
	allow  the mapped area to be executed, and return the
	location in the address space at which the mapping is made.
*/

private void test4()
{
	int	err;
	int	pathok = 0;
	int	j;
	int	*addr;
	int	ret;
	char	*(*foo)();
	struct vsrt_exec_resource get;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test4");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	if ((ret = vsrt_create_exec_file(test_file, O_RDONLY, VSRT_PROT_ALL, &get)) != 0)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	errno = 0;
	if ((addr = (int *)tb_mmap(0, 3*vsrt_pgsz, PROT_EXEC, MAP_PRIVATE, get.fd, 0)) == (int *)MAP_FAILED) {
		err = errno;
#ifndef _XOPEN_SOURCE
		if (errno == ENOTSUP)
			xx_rpt(UNSUPPORTED);
		else
#endif
			xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(get.fd);
		(void)unlink(test_file);
		DBUG_VOID_RETURN
	}
	else
		PATH_TRACE;

VSRT_FORK(pid);
	foo = (char *(*)())((long)addr + get.offset);

	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0)
		ret = (int)(*foo)(); 

	if (vsrt_got_sigsegv) {
		in_rpt("SIGSEGV received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (vsrt_got_sigbus) {
		in_rpt("SIGBUS received");
		xx_rpt(FAILURE);
		exit(1);
	}
	if (ret != 99) {
		in_rpt("Mapped page could not be executed");
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)close(get.fd);
	(void)munmap((void *)addr, (size_t)vsrt_pgsz);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	A successful call to mmap() when fildes is associated
	with  a  regular  file, the MAP_SHARED flag is set in
	flags, and the PROT_NONE flag is set  in  prot  shall
	map  len  bytes  at offset off in the file associated
	with fildes into the address space  of  the  process,
	not  allow the mapped area to be accessed, and return
	the location in the address space at which  the  mapping is made.
*/

private void test5()
{
	int	fd;
	int	err;
	int	pathok = 0;
	volatile int	i;
	int	*addr;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test5");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	if ((addr = (int *)tb_mmap(0, vsrt_pgsz, PROT_NONE, MAP_SHARED, fd, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd);
		(void)unlink(test_file);
		DBUG_VOID_RETURN
	}
	else
		PATH_TRACE;

VSRT_FORK(pid);
	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0)
		i = *addr;

	if (!(vsrt_got_sigsegv || vsrt_got_sigbus)) {
		in_rpt("Mapped page could be accessed");
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)close(fd);
	(void)munmap((void *)addr, (size_t)vsrt_pgsz);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
supports MAP_PRIVATE:
	A successful call to mmap() when fildes is associated
	with  a  regular file, the MAP_PRIVATE flag is set in
	flags, and the PROT_NONE flag is set  in  prot  shall
	map  len  bytes  at offset off in the file associated
	with fildes into the address space  of  the  process,
	not  allow the mapped area to be accessed, and return
	the location in the address space at which  the  mapping is made.
*/

private void test6()
{
	int	fd;
	int	err;
	int	pathok = 0;
	volatile int	i;
	int	*addr;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test6");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

VSRT_FORK(pid);
#ifndef _XOPEN_SOURCE
	if ((addr = (int *)tb_mmap(0, vsrt_pgsz, PROT_READ, MAP_PRIVATE, fd, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
		if (errno == ENOTSUP) {
			err = errno;
			xx_rpt(UNSUPPORTED);
			in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
			(void)close(fd);
			(void)unlink(test_file);
			exit(0);
		}
	}
	else
		(void)munmap((void *)addr, (size_t)vsrt_pgsz);
#endif
	if ((addr = (int *)tb_mmap(0, vsrt_pgsz, PROT_NONE, MAP_SHARED, fd, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		exit(1);
	}

	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0)
		i = (char)*addr;

	if (!(vsrt_got_sigsegv || vsrt_got_sigbus)) {
		in_rpt("Mapped page could be accessed");
		xx_rpt(FAILURE);
		exit(1);
	}

	(void)munmap((void *)addr, (size_t)vsrt_pgsz);
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)close(fd);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	A successful call to mmap() when fildes is associated
	with  a  regular  file, the MAP_SHARED flag is set in
	flags, and the PROT_WRITE flag is set in  prot  shall
	map  len  bytes  at offset off in the file associated
	with fildes into the address space  of  the  process,
	allow  the  mapped area to be written such that write
	references change the file, and return  the  location
	in the address space at which the mapping is made.
*/

private void test7()
{
	int	err;
	int	pathok = 0;
	int	*adr1;
	int	fd1, i, j, ibufsz;
	pid_t	pid1, pid2;
	int	buf[512];

	DBUG_ENTER("test7");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	ibufsz = sizeof(buf)/sizeof(buf[0]);

	VSRT_FORK(pid1);

		/*
		 * this child writes 1's from 0 to vsrt_pgsz/2
		 */
		if ((adr1 = (int *)tb_mmap(0, vsrt_pgsz, PROT_WRITE, MAP_SHARED, fd1, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("mmap failed in child 1, errno = %d (%s)", err, errname(err));
			exit(1);
		}
		else
			PATH_TRACE;

		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			for (j = 0; !vsrt_got_sig && j < vsrt_ipgsz/2; j++)
				adr1[j] = 1;

			if (msync((void *)adr1, (size_t)vsrt_pgsz, MS_SYNC) == -1) {
				err = errno;
				xx_rpt(FAILURE);
				in_rpt("msync failed in child 1, errno = %d (%s)", err, errname(err));
				exit(1);
			}

		}

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		VSRT_POST_EVENT(CHILD_1_READY);
	VSRT_EFORK;

	VSRT_FORK(pid2);

		/*
 		* this child writes 2's from vsrt_pgsz/2 to vsrt_pgsz
 		*/

		if ((adr1 = (int *)tb_mmap(0, vsrt_pgsz, PROT_WRITE, MAP_SHARED, fd1, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("mmap failed in child2, errno = %d (%s)", err, errname(err));
			exit(1);
		}

		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			for (j = vsrt_ipgsz/2; !vsrt_got_sig && j < vsrt_ipgsz; j++)
				adr1[j] = 2;

			if (msync((void *)adr1, (size_t)vsrt_pgsz, MS_SYNC) == -1) {
				err = errno;
				xx_rpt(FAILURE);
				in_rpt("msync failed in child 2, errno = %d (%s)", err, errname(err));
				exit(1);
			}

		}

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 2");
			xx_rpt(FAILURE);
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 2");
			xx_rpt(FAILURE);
			exit(1);
		}

		VSRT_POST_EVENT(CHILD_2_READY);
	VSRT_EFORK;

	/*
	 * is it expecting too much to require cache
	 * buffer synchronization with mmap()?  Should
	 * we close and reopen the file?
	 */
	VSRT_WAIT_EVENT(CHILD_1_READY);
	VSRT_WAIT_EVENT(CHILD_2_READY);

	if (lseek(fd1, (off_t)vsrt_pgsz, SEEK_SET) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("lseek failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN
	}
	else
		PATH_TRACE;

	for (i = 0; i < vsrt_ipgsz; i += ibufsz) {

		if (read(fd1, (void*)buf, sizeof(buf)) == -1) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("read failed, errno = %d (%s)", err, errname(err));
			(void)close(fd1);
			(void)unlink(test_file);
			DBUG_VOID_RETURN
		}

		for (j = 0; j < ibufsz; j++) {
			if ((i + j) < vsrt_ipgsz/2) {
				if (buf[j] != 1) {
					in_rpt("Word %d in file was %d, should be 1", i + j, buf[j]);
					xx_rpt(FAILURE);
					errors++;
				}
			}
			else {
				if (buf[j] != 2) {
					in_rpt("Word %d in file was %d, should be 2", i + j, buf[j]);
					xx_rpt(FAILURE);
					errors++;
				}
			}
		}
	}

	VSRT_WAIT_FOR_PID(pid1);
	VSRT_WAIT_FOR_PID(pid2);
	(void)close(fd1);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
supports MAP_PRIVATE:
	A successful call to mmap() when fildes is associated
	with  a  regular file, the MAP_PRIVATE flag is set in
	flags, and the PROT_WRITE flag is set in  prot  shall
	map  len  bytes  at offset off in the file associated
	with fildes into the address space  of  the  process,
	allow  the  mapped area to be written such that write
	references do not change the  file,  and  return  the
	location in the address space at which the mapping is made.
*/

private void test8()
{
	int	err;
	int	pathok = 0;
	int	*adr1;
#ifndef _XOPEN_SOURCE
	int	*addr;
#endif
	int	fd1, i, j, ibufsz;
	pid_t	pid1, pid2;
	int	buf[512];

	DBUG_ENTER("test8");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	ibufsz = sizeof(buf)/sizeof(buf[0]);

#ifndef _XOPEN_SOURCE
	if ((addr = (int *)tb_mmap(0, vsrt_pgsz, PROT_READ, MAP_PRIVATE, fd1, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
		if (errno == ENOTSUP) {
			err = errno;
			xx_rpt(UNSUPPORTED);
			in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
			(void)close(fd1);
			(void)unlink(test_file);
			DBUG_VOID_RETURN 
		}
	}
	else
		(void)munmap((void *)addr, (size_t)vsrt_pgsz);
#endif

	VSRT_FORK(pid1);

		if ((adr1 = (int *)tb_mmap(0,vsrt_pgsz,PROT_WRITE|PROT_READ,MAP_PRIVATE,fd1,0)) == (int *)MAP_FAILED) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("mmap failed in child 1, errno = %d (%s)", err, errname(err));
			exit(1);
		}

		VSRT_WAIT_EVENT(CHILD_2_READY);

		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			for (j = 0; !vsrt_got_sig && j < vsrt_ipgsz/2; j++)
				adr1[j] = 1;
		}

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		VSRT_POST_EVENT(CHILD_1_READY);
		VSRT_WAIT_EVENT(CHILD_2_DONE);
	VSRT_EFORK;

	VSRT_FORK(pid2);

		if ((adr1 = (int *)tb_mmap(0,vsrt_pgsz,PROT_READ|PROT_WRITE,MAP_PRIVATE,fd1,0)) == (int *)MAP_FAILED) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("mmap failed in child 2, errno = %d (%s)", err, errname(err));
			exit(1);
		}

		/*Wait for other child & then mapped page unchanged*/
		VSRT_POST_EVENT(CHILD_2_READY);
		VSRT_WAIT_EVENT(CHILD_1_READY);

		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			for (j = vsrt_ipgsz/2; !vsrt_got_sig && j < vsrt_ipgsz; j++)
				adr1[j] = 2;
		}

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 2");
			xx_rpt(FAILURE);
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 2");
			xx_rpt(FAILURE);
			exit(1);
		}

		VSRT_POST_EVENT(CHILD_2_DONE);
	VSRT_EFORK;

	/*Close & reopen file*/
	(void)close(fd1);

	if ((fd1 = open(test_file,O_RDONLY)) == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	/*File contains unmodified data*/
	for (i = 0; i < vsrt_ipgsz; i += ibufsz) {

		if (read(fd1,(void*)buf,sizeof(buf)) == -1) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("read failed, errno = %d (%s)", err, errname(err));
			(void)close(fd1);
			(void)unlink(test_file);
			DBUG_VOID_RETURN
		}

		for (j = 0; j < ibufsz; j++) {
			if (buf[j] != (i + j)) {
				in_rpt("Word %d in file was %d, should be %d", i + j, buf[j], i + j);
				xx_rpt(FAILURE);
				errors++;
			}
		}
	}

	VSRT_WAIT_FOR_PID(pid1);
	VSRT_WAIT_FOR_PID(pid2);

	(void)close(fd1);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 
/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
and Headers, Issue 5:
	A successful call to mmap() when fildes is associated
	with  a  regular  file, the MAP_SHARED flag is set in
	flags, and the PROT_READ and PROT_WRITE flags are set
	in prot shall map len bytes at offset off in the file
	associated with fildes into the address space of  the
	process,  allow  the mapped area to be read and written,
	and return the location in the address space  at
	which the mapping is made.
*/

private void test9()
{
	int	err;
	int	pathok = 0;
	int	*adr1;
	int	fd, i, j, ibufsz;
	pid_t	pid1, pid2;
	int	buf[512];

	DBUG_ENTER("test9");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif


	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	ibufsz = sizeof(buf)/sizeof(buf[0]);

	VSRT_FORK(pid1);
		/*
		 * this child writes 1's from 0 to vsrt_pgsz/2
		 */
		if ((adr1 = (int *)tb_mmap(0, vsrt_pgsz, PROT_READ|PROT_WRITE, MAP_SHARED, fd, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("mmap failed in child 1, errno = %d (%s)", err, errname(err));
			exit(1);
		}

		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			for (j = 0; !vsrt_got_sig && j < (int)vsrt_ipgsz/2; j++) {
				i = adr1[j];
				if (i != (int)(j + vsrt_ipgsz)) {
					in_rpt("At offset %d in mapped page expected: %ld received: %d", j, (long)(j + vsrt_ipgsz), i);
					xx_rpt(FAILURE);
					errors++;
				}
			}
		}

		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			for (j = 0; !vsrt_got_sig && j < (int)vsrt_ipgsz/2; j++)
				adr1[j] = 1;

			if (msync((void *)adr1, (size_t)vsrt_pgsz, MS_SYNC) == -1) {
				err = errno;
				xx_rpt(FAILURE);
				in_rpt("msync failed in child 1, errno = %d (%s)", err, errname(err));
				errors++;
			}
		}

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGSEGV received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		VSRT_POST_EVENT(CHILD_1_READY);
	VSRT_EFORK;

	VSRT_FORK(pid2);

		/*
 		* this child writes 2's from vsrt_pgsz/2 to vsrt_pgsz
 		*/

		if ((adr1 = (int *)tb_mmap(0, vsrt_pgsz, PROT_READ|PROT_WRITE, MAP_SHARED, fd, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("mmap failed in child 2, errno = %d (%s)", err, errname(err));
			exit(1);
		}
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
		{
			for (j = (int)vsrt_ipgsz/2; !vsrt_got_sig && j < (int)vsrt_ipgsz; j++) {
				i = adr1[j];
				if (i != (int)(j + vsrt_ipgsz)) {
					in_rpt("At offset %d in mapped page, expected: %ld received: %d", j, (long)(j + vsrt_ipgsz), i);
					xx_rpt(FAILURE);
					errors++;
				}
			}
		}

		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			for (j = (int)vsrt_ipgsz/2; !vsrt_got_sig && j < (int)vsrt_ipgsz; j++)
				adr1[j] = 2;

			if (msync((void *)adr1, (size_t)vsrt_pgsz, MS_SYNC) == -1) {
				err = errno;
				xx_rpt(FAILURE);
				in_rpt("msync failed in child 2, errno = %d (%s)", err, errname(err));
				errors++;
			}

		}

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 2");
			xx_rpt(FAILURE);
			errors++;
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGSEGV received in child 2");
			xx_rpt(FAILURE);
			errors++;
		}
		VSRT_POST_EVENT(CHILD_2_READY);
	VSRT_EFORK;

	/*
	 * is it expecting too much to require cache
	 * buffer synchronization with mmap()?  Should
	 * we close and reopen the file?
	 */
	/*Verify that file has been updated*/

	VSRT_WAIT_EVENT(CHILD_1_READY);
	VSRT_WAIT_EVENT(CHILD_2_READY);

	if (lseek(fd, (off_t)vsrt_pgsz, SEEK_SET) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("lseek failed, errno = %d (%s)", err, errname(err));
		(void)close(fd);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	}
	else
		PATH_TRACE;

	for (i = 0; i < (int)vsrt_ipgsz; i += ibufsz) {

		if (read(fd, (void*)buf, sizeof(buf)) == -1) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("read failed, errno = %d (%s)", err, errname(err));
			(void)close(fd);
			(void)unlink(test_file);
			DBUG_VOID_RETURN 
		}
		for (j = 0; j < ibufsz; j++) {
			if ((i + j) < (int)vsrt_ipgsz/2) {
				if (buf[j] != 1) {
					in_rpt("Word %d in file was %d, should be 1", i + j, buf[j]);
					xx_rpt(FAILURE);
					errors++;
				}
			}
			else {
				if (buf[j] != 2) {
					in_rpt("Word %d in file was %d, should be 2", i + j, buf[j]);
					xx_rpt(FAILURE);
					xx_rpt(FAILURE);
					errors++;
				}
			}
		}
	}

	(void)close(fd);
	VSRT_WAIT_FOR_PID(pid1);
	VSRT_WAIT_FOR_PID(pid2);
	(void)unlink(test_file);
	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
supports MAP_PRIVATE:
	A successful call to mmap() when fildes is associated
	with  a  regular file, the MAP_PRIVATE flag is set in
	flags, and the PROT_READ and PROT_WRITE flags are set
	in prot shall map len bytes at offset off in the file
	associated with fildes into the address space of  the
	process,  allow  the mapped area to be read and writ-
	ten, and return the location in the address space  at
	which the mapping is made.
*/

private void test10()
{
	int	err;
	int	pathok = 0;
#ifndef _XOPEN_SOURCE
	int	*addr;
#endif
	int	*adr1;
	int	fd1, i, j, ibufsz;
	pid_t	pid1, pid2;
	int	buf[512];

	DBUG_ENTER("test10");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	ibufsz = sizeof(buf)/sizeof(buf[0]);

#ifndef _XOPEN_SOURCE
	if ((addr = (int *)tb_mmap(0, vsrt_pgsz, PROT_READ, MAP_PRIVATE, fd1, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
		if (errno == ENOTSUP) {
			err = errno;
			xx_rpt(UNSUPPORTED);
			in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
			(void)close(fd1);
			(void)unlink(test_file);
			DBUG_VOID_RETURN 
		}
	}
	else
		(void)munmap((void *)addr, (size_t)vsrt_pgsz);
#endif

	VSRT_FORK(pid1);

		if ((adr1 = (int *)tb_mmap(0, (size_t)vsrt_pgsz,PROT_WRITE|PROT_READ,MAP_PRIVATE,fd1, (off_t)0)) == (int *)MAP_FAILED) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
			exit(1);
		}

		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			for (j = 0; !vsrt_got_sig && j < (int)vsrt_ipgsz/2; j++) {
				i = adr1[j];
				if (i != j ) {
					in_rpt("Child 1: At offset %d in mapped page expected: %ld received: %d", j, (long)j, i);
					xx_rpt(FAILURE);
					exit(1);
				}
			}
		}

		/*Mapped page may be written*/
		VSRT_WAIT_EVENT(CHILD_2_READY);

		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			for (j = 0; !vsrt_got_sig && j < (int)vsrt_ipgsz/2; j++)
				adr1[j] = 1;
		}

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		VSRT_POST_EVENT(CHILD_1_READY);
		VSRT_WAIT_EVENT(CHILD_2_DONE);
	VSRT_EFORK;

	VSRT_FORK(pid2);

		if ((adr1 = (int *)tb_mmap(0, (size_t)vsrt_pgsz,PROT_READ|PROT_WRITE,MAP_PRIVATE,fd1, (off_t)0)) == (int *)MAP_FAILED) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
			exit(1);
		}

		/*Mapped page may be read*/
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			for (j = (int)vsrt_ipgsz/2; !vsrt_got_sig && j < (int)vsrt_ipgsz; j++) {
				i = adr1[j];
				if (i != j ) {

					in_rpt("At offset %d of mapped page expected: %ld  Received: %d", j, (long)(j), i);
					xx_rpt(FAILURE);
					exit(1);
				}
			}
		}


		/*Child2: Pages modified by the other child remain unchanged*/
		VSRT_POST_EVENT(CHILD_2_READY);
		VSRT_WAIT_EVENT(CHILD_1_READY);

		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			for (j = (int)vsrt_ipgsz/2; !vsrt_got_sig && j < (int)vsrt_ipgsz; j++)
				adr1[j] = 2;
		}

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 2");
			xx_rpt(FAILURE);
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 2");
			xx_rpt(FAILURE);
			exit(1);
		}

		VSRT_POST_EVENT(CHILD_2_DONE);
	VSRT_EFORK;

	close(fd1);

	if ((fd1 = open(test_file,O_RDONLY)) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	}
	else
		PATH_TRACE;

	/*File contains unmodified data*/
	for (i = 0; i < (int)vsrt_ipgsz; i += ibufsz) {
		if (read(fd1,(void*)buf,sizeof(buf)) == -1) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("read failed, errno = %d (%s)", err, errname(err));
			(void)close(fd1);
			(void)unlink(test_file);
			DBUG_VOID_RETURN 
		}

		for (j = 0; j < ibufsz; j++) {
			if (buf[j] != (i + j)) {
				in_rpt("Word %d in file was %d, should be %d", i + j, buf[j], i + j);
				xx_rpt(FAILURE);
				errors++;
			}
		}
	}

	(void)close(fd1);
	VSRT_WAIT_FOR_PID(pid1);
	VSRT_WAIT_FOR_PID(pid2);
	unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
supports MAP_FIXED:
	A call to mmap() when fildes  is  associated  with  a
	regular  file  and the MAP_FIXED flag is set in flags
	shall cause the file associated  with  fildes  to  be
	mapped at an address exactly matching addr.
*/

private void test11()
{
	int	err;
	int	pathok = 0;
	int	fd1;
	int	*adr1, *ask_addr;

	DBUG_ENTER("test11");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	/*Find a fixed address for mapping text*/
	if ((ask_addr = (int *)tb_mmap((void *)0, (size_t)(3*vsrt_pgsz), 
	     PROT_NONE, MAP_SHARED, fd1, 0)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	}
	else
		PATH_TRACE;

	if (munmap((void *)ask_addr, (3*vsrt_pgsz)) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("munmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	}

	if ((adr1 = (int *)tb_mmap((void *)ask_addr, (size_t)vsrt_pgsz, 
	    PROT_WRITE|PROT_READ, MAP_FIXED|MAP_SHARED, fd1, 
	    (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
		err = errno;
#ifndef _XOPEN_SOURCE
		if (errno == ENOTSUP)
			xx_rpt(UNSUPPORTED);
		else
#endif
			xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	}
	else
		PATH_TRACE;

	if (adr1 != ask_addr) {
		in_rpt("Call to mmap() with MAP_FIXED didn't return expected address");

		in_rpt("Expected: 0x%p  Received: 0x%p", ask_addr, adr1);
		xx_rpt(FAILURE);
		errors++;
	}

	close(fd1);
	(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	A call to mmap() shall zero-fill any partial page  at
	the end of the file.
*/

private void test12()
{
	int	err;
	int	pathok = 0;
	int	fd1, i;
	int	*buf;
	int	*addr, *check, icheck;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test12");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	
	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((buf = (int *)malloc(vsrt_pgsz)) == NULL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("malloc failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	}
	else
		PATH_TRACE;

	(void)unlink(test_file);

	if ((fd1 = open(test_file, O_RDWR|O_CREAT|O_TRUNC, VSRT_PROT_ALL)) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	for (i = 0; i < vsrt_ipgsz; i++)
		buf[i] = i;

	/*Write a half page of data to file*/
	if (write(fd1, (void*)buf, vsrt_pgsz/2) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("write failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/*Close and reopen the file*/
	(void)close(fd1);
	if ((fd1 = open(test_file, O_RDONLY, VSRT_PROT_ALL)) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("second open failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/*Map a page from file*/
	if ((addr = (int *)tb_mmap(0, vsrt_pgsz, PROT_READ, MAP_SHARED, fd1, 0)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/*Second half of mapped area zero filled*/
VSRT_FORK(pid);
	vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
	check = addr;

	for (i = 0; i < vsrt_ipgsz/2; i++) {
		icheck = *check++;
		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received accessing integer at offset %d of page", i*sizeof(int));
			xx_rpt(FAILURE);
			exit(1);
		} 

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received accessing integer at offset %d of page", i*sizeof(int));
			xx_rpt(FAILURE);
			exit(1);
		}

		if (icheck != i) {
			in_rpt("Integer at offset %d in region = %d, expected =  %d", i*sizeof(int), icheck, i);
			xx_rpt(FAILURE);
			errors++;
		}
	}
	if (errors != 0)
		exit(errors);

	for (i = 0; i < vsrt_ipgsz/2; i++) {
		icheck = *check++;
		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received accessing integer at offset %d of page", i*sizeof(int));
			xx_rpt(FAILURE);
			exit(1);
		}
		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received accessing integer at offset %d of page", i*sizeof(int));
			xx_rpt(FAILURE);
			exit(1);
		}
		if (icheck != 0) {
			in_rpt("Integer at offset %d in region = %d, expected =  %d", (i+(vsrt_ipgsz/2))*sizeof(int), icheck, 0);
			xx_rpt(FAILURE);
			errors++;
		}
	}
	if (errors != 0)
		exit(errors);
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)close(fd1);
	(void)munmap((void *)addr, vsrt_pgsz);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	The system shall not write out modified  portions  of
	the  last  page  of a file mapped by a call to mmap()
	which are beyond the end of the file.
*/

private void test13()
{
	int	err;
	int	pathok = 0;
	int	fd1, i, j;
	int	*buf;
	int	*addr;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test13");

	errors= 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif


	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((buf = (int *)malloc(vsrt_pgsz)) == NULL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("malloc failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	(void)unlink(test_file);

	if ((fd1 = open(test_file, O_RDWR|O_CREAT|O_TRUNC, VSRT_PROT_ALL)) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	for (i = 0; i < vsrt_ipgsz/2; i++)
		buf[i] = i;

	/*Write a half page of data to the file*/
	if (write(fd1, (void*)buf, vsrt_pgsz/2) != vsrt_pgsz/2) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("write failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/*Map in a whole page*/
	if ((addr = (int *)tb_mmap(0,vsrt_pgsz,PROT_READ|PROT_WRITE,MAP_SHARED,fd1,0)) == (int *)MAP_FAILED) {

		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

VSRT_FORK(pid);
	vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

	/*Fill second half of mapped page with -1*/
	for (j = vsrt_ipgsz/2; !vsrt_got_sig && j < vsrt_ipgsz; j++)
		addr[j] = -1;

	if (vsrt_got_sigsegv) {
		in_rpt("SIGSEGV received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (vsrt_got_sigbus) {
		in_rpt("SIGBUS received");
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_VOID_RETURN
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	} else
		errors += WEXITSTATUS(status);

	/*Close & unmap file*/
	(void)close(fd1);

	(void)munmap((void *)addr, vsrt_pgsz);

	/*Open and read file*/
	if ((fd1 = open(test_file, O_RDWR)) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;
	/*Data beyond end of file not written to file*/
	if ((i=read(fd1, (void*)buf, vsrt_pgsz)) > vsrt_pgsz/2) {
		in_rpt("File size is larger, expected %d, got %d", vsrt_pgsz/2, i);
		xx_rpt(FAILURE);
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	for (i = 0; i < vsrt_ipgsz/2; i++) {
		if (buf[i] != i) {
			in_rpt("Expected integer at offset %d = %d, is = %d", i*sizeof(int), i, buf[i]);
			xx_rpt(FAILURE);
			errors++;
		}
	}

	/*Map in a quarter page*/
	if ((addr = (int *)tb_mmap(0,vsrt_pgsz/4,PROT_READ|PROT_WRITE,MAP_SHARED,fd1,0)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

VSRT_FORK(pid);
	vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

	/*Fill in rest of mapped page with -1*/
	for (j = vsrt_ipgsz/4; !vsrt_got_sig && j < vsrt_ipgsz; j++)
		addr[j] = -1;

	if (vsrt_got_sigsegv) {
		in_rpt("SIGSEGV received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (vsrt_got_sigbus) {
		in_rpt("SIGBUS received");
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_VOID_RETURN
	else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	} else
		errors += WEXITSTATUS(status);
	(void)close(fd1);

	(void)munmap((void *)addr, vsrt_pgsz);

	if ((fd1 = open(test_file, O_RDONLY)) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/*Data beyond end of file not written to file*/
	if ((i=read(fd1, (void*)buf, vsrt_pgsz)) > vsrt_pgsz/2) {
		in_rpt("File size is larger, expected %d, got %d", vsrt_pgsz/2, i);
		xx_rpt(FAILURE);
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/* Data in first quarter must be there.  Data between mapped quarter and end of file unspecified */
	for (i = 0; i < vsrt_ipgsz/4; i++) {
		if (buf[i] != i) {
			in_rpt("Expected integer at offset %d = %d, is = %d", i*sizeof(int), i, buf[i]);
			err = errno;
			(void)close(fd1);
			(void)unlink(test_file);
			DBUG_VOID_RETURN 
		}
	}

	(void)close(fd1);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	References to mapped pages beyond the page containing
	the  last  byte  of a file mapped by a call to mmap()
	shall result in the delivery of a SIGBUS signal.
*/

private void test14()
{
	int	err;
	int	pathok = 0;
	int	*addr;
	int	fd1;
	volatile int	i;
	int	*buf;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test14");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((buf = (int *)malloc(vsrt_pgsz)) == NULL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("malloc failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	(void)unlink(test_file);

	if ((fd1 = open(test_file,O_RDWR|O_CREAT|O_TRUNC,VSRT_PROT_ALL)) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	for (i = 0; i < vsrt_ipgsz; i++)
		buf[i] = i;

	if (write(fd1, (void*)buf, vsrt_pgsz/2) != (vsrt_pgsz/2)) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("write failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

		if ((addr = (int *)tb_mmap(0,2*vsrt_pgsz, PROT_READ|PROT_WRITE, MAP_SHARED, fd1, 0)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	VSRT_FORK(pid);
		/*SIGBUS on reference to full page beyond current end*/
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
			i = *(addr+(vsrt_pgsz/sizeof(int)));

		sleep(WAIT_FOR_TIME);

		if (!vsrt_got_sigbus) {
			if (vsrt_got_sigsegv) {
				in_rpt("SIGBUS signal was not received but a SIGSEGV");
				in_rpt("signal was received instead");
				xx_rpt(FAILURE);
				exit(1);
			}
			else {
				in_rpt("SIGBUS signal not received");
				xx_rpt(FAILURE);
				exit(1);
			}
		}

		(void)munmap((void *)addr, vsrt_pgsz*2);
	VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)close(fd1);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	After a successful call to mmap() the mapping created
	shall be retained across a fork.
*/

#define	T15_CHILD1_MSG_1	1

private void test15()
{
	int	err;
	int	pathok = 0;
	int	fd1;
	pid_t	pid;
	int	*buf;
	int	*adr1, *adr2;
	int	status;

	DBUG_ENTER("test15");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((buf = (int *)malloc(vsrt_pgsz)) == NULL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("malloc failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((adr1 = (int *)tb_mmap(0,vsrt_pgsz,PROT_READ|PROT_WRITE,MAP_SHARED,fd1,0)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	vsrt_got_sigsegv = vsrt_got_sigbus = 0;

	VSRT_FORK(pid);

		/*Child: mapped area accessible after fork*/
		*adr1 = T15_CHILD1_MSG_1;

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}
		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		/*child changes reflected in area showing that MAP_SHARED persisted across fork*/
		if (*adr1 != T15_CHILD1_MSG_1) {
			in_rpt("MAP_SHARED didn't persist across fork");
			xx_rpt(FAILURE);
			exit(1);
		}
	VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_VOID_RETURN
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	} else
		errors += WEXITSTATUS(status);

	/*mmap() with MAP_PRIVATE returns location where object is mapped*/
	errno = 0;
	adr2 = (int *)tb_mmap(0, vsrt_pgsz, PROT_READ|PROT_WRITE, MAP_PRIVATE, fd1, (off_t)vsrt_pgsz);
#ifndef _XOPEN_SOURCE
		if ((adr2 != (int *)MAP_FAILED) || ((adr2 == (int *)MAP_FAILED) && (errno != ENOTSUP))) {
			if (adr1 == adr2) {
				in_rpt("Successive mmap's mapped to same address");
				xx_rpt(FAILURE);
				(void)close(fd1);
				(void)unlink(test_file);
				DBUG_VOID_RETURN 
			} else 
				if (adr2 == (int *)MAP_FAILED) {
					xx_rpt(FAILURE);
					in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
					(void)close(fd1);
					(void)unlink(test_file);
					DBUG_VOID_RETURN 
				}
		}
#else
	if (adr2 == (int *)MAP_FAILED) {
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	}

	if (adr1 == adr2) {
		in_rpt("Successive mmap's mapped to same address");
		xx_rpt(FAILURE);
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;
#endif

	VSRT_FORK(pid);
		/*mapped area accessible after fork*/
		*adr2 = T15_CHILD1_MSG_1;
		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 2");
			xx_rpt(FAILURE);
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 2");
			xx_rpt(FAILURE);
			exit(1);
		}

	VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	/*child changes not reflected in area showing that MAP_PRIVATE persisted across fork*/
	if (*adr2 == T15_CHILD1_MSG_1) {
		in_rpt("MAP_PRIVATE didn't persist across fork");
		xx_rpt(FAILURE);
		errors++;
	}

	(void)close(fd1);
	(void)munmap((void *)adr1, vsrt_pgsz);
	(void)munmap((void *)adr2, vsrt_pgsz);
	(void)unlink(test_file);
	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	After a successful call to mmap() a subsequent  close
	operation  shall  not  affect the mapping and changes
	made to the memory image shall be  reflected  in  the
	mapped  portion  of  the  file when the changes would
	have been performed  in  the  absence  of  the  close
	operation.
*/

#define	T16_CHILD1_MSG_1	1

private void test16()
{
	int	fd1;
	int	err;
	int	pathok = 0;
	int	i;
	int 	*adr1;
	int	*buf;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test16");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((buf = (int *)malloc(vsrt_pgsz)) == NULL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("malloc failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	(void)unlink(test_file);

	if ((fd1 = open(test_file,O_RDWR|O_CREAT|O_TRUNC,VSRT_PROT_ALL)) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	for (i = 0; i < vsrt_ipgsz; i++)
		buf[i] = i;

	if (write(fd1,(void*)buf,vsrt_pgsz) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("write failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	if ((adr1 = (int *)tb_mmap(0,vsrt_pgsz,PROT_WRITE,MAP_SHARED,fd1,0)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/*Close file and nap*/
	(void)close(fd1);

	(void)sleep(WAIT_FOR_TIME);

VSRT_FORK(pid);
	/*Changes to file still made even though file closed*/
	vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

	*(adr1) = T16_CHILD1_MSG_1;

	if (vsrt_got_sigsegv) {
		in_rpt("SIGSEGV received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (vsrt_got_sigbus) {
		in_rpt("SIGBUS received");
		xx_rpt(FAILURE);
		exit(1);
	} 
VSRT_KROF2(pid, status);
	if (status == -1) {
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	}
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		errors += WEXITSTATUS(status);

	/*Unmap segment, re-open file and check change applied*/
	(void)munmap((void *)adr1, vsrt_pgsz);

	sleep(WAIT_FOR_TIME);

	if ((fd1 = open(test_file,O_RDONLY)) == -1)  {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("second open failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	if (read(fd1, (void*)buf, vsrt_pgsz) == -1)  {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("read failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/*The changes made to the memory image are reflected in the file*/
	if (buf[0] != T16_CHILD1_MSG_1) {
		in_rpt("Changes not made to file after a close()");
		xx_rpt(FAILURE);
		errors++;
	}

	(void)close(fd1);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	After a call to mmap()  when  MAP_SHARED  is  set  in
	flags  write  references  made  to  the memory region
	shall be visible in all MAP_SHARED  mappings  of  the
	same portion of the file by any process.
*/

#define T17_CHILD1_MSG_1	0xDEADBEEF
#define T17_CHILD2_MSG_1	0xBEEFDEAD
#define T17_CHILD1_MSG_2	0xDEADCAFE
#define T17_CHILD2_MSG_2	0xCAFEDEAD

private void test17()
{
	int	err;
	int	pathok = 0;
	int 	*adr1;
	int	fd1, j;
	pid_t	pid1, pid2;

	DBUG_ENTER("test17");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif


	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1) 
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	VSRT_FORK(pid1);

		/*
		 * this child writes 1's from 0 to vsrt_pgsz/2
		 */
		if ((adr1 = (int *)tb_mmap(0,vsrt_pgsz,PROT_WRITE,MAP_SHARED,fd1,0)) == (int *)MAP_FAILED) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
			exit(1);
		}
		

		/*mapped page at that address may be written*/
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		for (j = 0; !vsrt_got_sig && j < vsrt_ipgsz/2; j++)
			adr1[j] = 1;

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		/*Child1: wait for child2, modify page, tell child2*/
		VSRT_WAIT_EVENT(CHILD_2_READY);

		adr1[vsrt_ipgsz/2] = T17_CHILD1_MSG_1;

		VSRT_POST_EVENT(CHILD_1_READY);

		VSRT_WAIT_EVENT(CHILD_2_READY);

		if (adr1[0] != T17_CHILD2_MSG_1) {
			in_rpt("Child2's change not communicated to child1");
			xx_rpt(FAILURE);
			exit(1);
		}
	VSRT_EFORK;

	VSRT_FORK(pid2);

		/*
		 * this child writes 2's from vsrt_pgsz/2 to vsrt_pgsz
		 */
		if ((adr1 = (int *)tb_mmap(0,vsrt_pgsz,PROT_WRITE,MAP_SHARED,fd1,0)) == (int *)MAP_FAILED) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("malloc failed, errno = %d (%s)", err, errname(err));
			exit(1);
		}

		/*Mapped page at that address may be written*/
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		for (j = vsrt_ipgsz/2; !vsrt_got_sig && j < vsrt_ipgsz; j++)
			adr1[j] = 2;

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 2");
			xx_rpt(FAILURE);
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 2");
			xx_rpt(FAILURE);
			exit(1);
		}

		/*Child2: Inform child1 we initialized & wait for its write*/
		VSRT_POST_EVENT(CHILD_2_READY);

		VSRT_WAIT_EVENT(CHILD_1_READY);

		if (adr1[vsrt_ipgsz/2] != T17_CHILD1_MSG_1) {
			in_rpt("Child1's change not communicated to child2");
			xx_rpt(FAILURE);
			exit(1);
		}

		adr1[0] = T17_CHILD2_MSG_1;

		VSRT_POST_EVENT(CHILD_2_READY);
	VSRT_EFORK;

	(void)close(fd1);
	VSRT_WAIT_FOR_PID(pid1);
	VSRT_WAIT_FOR_PID(pid2);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
supports MAP_PRIVATE:
	After a call to mmap() with MAP_PRIVATE set in  flags
	write  references to the memory region by the calling
	process shall not be visible to any process in  other
	mappings of the same portion of the file.
*/

#define T18_CHILD1_MSG_1	0xDEADBEEF
#define T18_CHILD2_MSG_1	0xBEEFDEAD
#define T18_CHILD1_MSG_2	0xDEADCAFE
#define T18_CHILD2_MSG_2	0xCAFEDEAD
#define T18_CHILD3_MSG_1	0xDEADFACE

private void test18()
{
	int	err;
	int	pathok = 0;
	int 	*addr;
	int	fd1, j;
	pid_t	pid1, pid3;

	DBUG_ENTER("test18");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

#ifndef _XOPEN_SOURCE
	if ((addr = (int *)tb_mmap(0, vsrt_pgsz, PROT_READ, MAP_PRIVATE, fd1, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
		if (errno == ENOTSUP) {
			err = errno;
			xx_rpt(UNSUPPORTED);
			in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
			(void)close(fd1);
			(void)unlink(test_file);
			DBUG_VOID_RETURN 
		}
	}
	else
		(void)munmap((void *)addr, (size_t)vsrt_pgsz);
#endif

	VSRT_FORK(pid1);

		/*
		 * this child writes 1's to [0 .. vsrt_pgsz/2 )
		 * It then tells child3 that it's ready.
		 * after child3 writes some data,
		 * this child verifies that child3's writes are not seen.
		 */

		if ((addr = (int *)tb_mmap(0,vsrt_pgsz,PROT_READ|PROT_WRITE,MAP_SHARED,fd1,0)) == (int *)MAP_FAILED) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
			exit(1);
		}

		/*mapped page at that address may be written*/
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		for (j = 0; !vsrt_got_sig && (j < vsrt_ipgsz/2); j++)
			addr[j] = 1;

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 1");
			xx_rpt(FAILURE);
			exit(1);
		}

		addr[vsrt_ipgsz/2] = T18_CHILD1_MSG_1;

		/*wait for child3 to write segment*/
		VSRT_POST_EVENT(CHILD_1_READY_FOR_3);

		VSRT_WAIT_EVENT(CHILD_3_READY_FOR_1);

		/*changes not visible to child1*/
		if (addr[4] == T18_CHILD3_MSG_1) {
			in_rpt("Child3's change communicated to child1");
			xx_rpt(FAILURE);
			exit(1);
		}

		VSRT_POST_EVENT(CHILD_1_READY_FOR_3);
	VSRT_EFORK;

	/*this child opens MAP_PRIVATE & talks to the other*/
	VSRT_FORK(pid3);

		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

		if ((addr = (int *)tb_mmap(0,vsrt_pgsz,PROT_READ|PROT_WRITE,MAP_PRIVATE,fd1,0)) == (int *)MAP_FAILED) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
			exit(1);
		}

		/*wait for child 1 to get ready*/
		VSRT_WAIT_EVENT(CHILD_1_READY_FOR_3);

		/*Child3: make write references*/
		addr[4] = T18_CHILD3_MSG_1;		/* child3's change */

		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 3");
			xx_rpt(FAILURE);
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 3");
			xx_rpt(FAILURE);
			exit(1);
		}

		VSRT_POST_EVENT(CHILD_3_READY_FOR_1);

		/*Child3: wait for child1 to finish up*/
		VSRT_WAIT_EVENT(CHILD_1_READY_FOR_3);

	VSRT_EFORK;

	(void)close(fd1);
	vsrt_wait_for(pid1,2*MAX_TIME_TO_RUN);
	vsrt_wait_for(pid3, 2*MAX_TIME_TO_RUN);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
supports MAP_PRIVATE:
	A call to mmap() when MAP_PRIVATE is set in flags and
	write  references are made to the memory region shall
	not change the file.
*/

private void test19()
{
	int	err;
	int	pathok = 0;
	char	*addr;
	char	input;
	int	fd, fd1, j;

	DBUG_ENTER("test19");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	/*set the file contents*/
	if ((addr = (char *)tb_mmap(0, vsrt_pgsz, PROT_READ|PROT_WRITE,
	     MAP_SHARED, fd1, 0)) == (char *)MAP_FAILED) {
		vsrt_sysfail("mmap");
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;
	for (j = 0; j < (vsrt_pgsz/sizeof(char)); j++)
		addr[j] = 'B';

	(void)msync((void *)addr, vsrt_pgsz, MS_SYNC);
	(void)munmap((void *)addr, vsrt_pgsz);

	/*change in a private mapping*/
	if ((addr = (char *)tb_mmap(0, vsrt_pgsz, PROT_WRITE,
	     MAP_PRIVATE, fd1, 0)) == (char *)MAP_FAILED) {
		err = errno;
#ifndef _XOPEN_SOURCE
		if (errno == ENOTSUP)
			xx_rpt(UNSUPPORTED);
		else
#endif
			xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;


	for (j = 0; !vsrt_got_sig && (j < vsrt_pgsz/sizeof(char)); j++)
		addr[j] = 'A';

	(void)close(fd1);
	(void)munmap((void *)addr, (size_t)vsrt_pgsz);

	if ((fd = open(test_file, O_RDWR, VSRT_PROT_ALL)) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open failed, errno = %d (%s)", err, errname(err));
		(void)close(fd);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/*Write references do not change the file*/
	for (j = 0; (j < vsrt_pgsz/sizeof(char)); j++) {
		input = 0;
		if (read(fd, &input, sizeof(char)) != sizeof(char)) {
			vsrt_sysfail("read");
			(void)close(fd);
			(void)unlink(test_file);
			DBUG_VOID_RETURN 
		}
		if (input == 'A') {
			in_rpt("Offset %d in object was changed, expected %x, is %x", j, 'B', input);
			xx_rpt(FAILURE);
			errors++;
		}
	}

	close(fd);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	The st_atime field of a file  mapped  by  a  call  to
	mmap()  shall  be marked for update on the first read
	or write reference to a mapped region.
*/

private void test20()
{
	int	err;
	int	pathok = 0;
	int	*addr;
	int	fd1, j;
	struct stat	end_stat;
	time_t		start_time, end_time;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test20");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif


	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	time(&start_time);

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

	if ((addr = (int *)mmap(0, vsrt_pgsz, PROT_READ|PROT_WRITE,
	     MAP_SHARED, fd1, 0)) == (int *)MAP_FAILED) {

		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	}

	sleep(WAIT_FOR_TIME);

VSRT_FORK(pid);
	/*Make write references on the file*/
	for (j = 0; !vsrt_got_sig && (j < vsrt_pgsz/sizeof(int)); j++)
		addr[j] = 99999;
	sleep(WAIT_FOR_TIME);

	if (vsrt_got_sigsegv) {
		in_rpt("SIGSEGV received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (vsrt_got_sigbus) {
		in_rpt("SIGBUS received");
		xx_rpt(FAILURE);
		exit(1);
	} 
VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_VOID_RETURN
	else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	} else
		errors += WEXITSTATUS(status);

	sleep(1);

	if (stat(test_file, &end_stat) < 0) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("stat failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/*Obtain end time*/
	time(&end_time);

	/*st_atime field is marked for update*/
	vsrt_ck_stime(CHK_ATIME, 1, start_time, (struct stat *)NULL, end_time, &end_stat);

	(void)close(fd1);
	(void)munmap((void *)addr, (size_t)vsrt_pgsz);

	(void)unlink(test_file);
	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	The st_ctime field of a file  mapped  by  a  call  to
	mmap()  with  MAP_SHARED  set in flags and PROT_WRITE
	set in prot shall be  marked  for  update  between  a
	write  reference to a mapped region and the next call
	to msync with MS_ASYNC or MS_SYNC.
*/

private void test21()
{
	int	err;
	int	pathok = 0;
	int	*addr;
	int	fd1, j;
	struct stat	end_stat;
	time_t		start_time, end_time;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test21");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	/*Obtain start time*/
	time(&start_time);
	
	sleep(1);

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

	if ((addr = (int *)tb_mmap(0, vsrt_pgsz, PROT_READ|PROT_WRITE,
	     MAP_SHARED, fd1, 0)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	sleep(WAIT_FOR_TIME);
VSRT_FORK(pid);
	/*Make write references on the file*/
	for (j = 0; !vsrt_got_sig && (j < vsrt_pgsz/sizeof(int)); j++)
		addr[j] = 99999;

	if (vsrt_got_sigsegv) {
		in_rpt("SIGSEGV received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (vsrt_got_sigbus) {
		in_rpt("SIGBUS received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (msync((void *)addr, (size_t)vsrt_pgsz, MS_SYNC) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msync failed, errno = %d (%s)", err, errname(err));
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	sleep(WAIT_FOR_TIME);

	if (stat(test_file, &end_stat) < 0) {

		err = errno;
		xx_rpt(FAILURE);
		in_rpt("stat failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/*Obtain end time*/
	time(&end_time);

	sleep(1);

	vsrt_ck_stime(CHK_CTIME, 1, start_time, (struct stat *)NULL, end_time, &end_stat);

	(void)close(fd1);
	(void)munmap((void *)addr, (size_t)vsrt_pgsz);

	(void)unlink(test_file);
	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	The st_mtime field of a file  mapped  by  a  call  to
	mmap()  with  MAP_SHARED  set in flags and PROT_WRITE
	set in prot shall be  marked  for  update  between  a
	write  reference to a mapped region and the next call
	to msync with MS_ASYNC or MS_SYNC.
*/

private void test22()
{
	int	err;
	int	pathok = 0;
	int	*addr;
	int	fd1, j;
	struct stat	end_stat;
	time_t		start_time, end_time;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test22");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif



	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	/*Obtain start time*/
	time(&start_time);

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

	if ((addr = (int *)tb_mmap(0, vsrt_pgsz, PROT_READ|PROT_WRITE,
	     MAP_SHARED, fd1, 0)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	sleep(WAIT_FOR_TIME);
VSRT_FORK(pid);
	/*Make write references on the file*/
	for (j = 0; !vsrt_got_sig && (j < vsrt_pgsz/sizeof(int)); j++)
		addr[j] = 99999;
	sleep(WAIT_FOR_TIME);

	if (vsrt_got_sigsegv) {
		in_rpt("SIGSEGV received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (vsrt_got_sigbus) {
		in_rpt("SIGBUS received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (msync((void *)addr, (size_t)vsrt_pgsz, MS_SYNC) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msync failed, errno = %d (%s)", err, errname(err));
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	sleep(1);

	if (stat(test_file, &end_stat) < 0) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("stat failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/*Obtain end time*/
	time(&end_time);

	vsrt_ck_stime(CHK_MTIME, 1, start_time, (struct stat *)NULL, end_time, &end_stat);

	(void)close(fd1);
	(void)munmap((void *)addr, (size_t)vsrt_pgsz);

	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

static char     *test_file1 = "/tmp/vsrt_mmap.file.1";
static char     *test_file2 = "/tmp/vsrt_mmap.file.2";
static char     *test_file3 = "/tmp/vsrt_mmap.file.3";

static int create_map_file();
static int create_map_file(s, flag, mode, size, data)
char	*s;
int	flag;
mode_t	mode;
int	size;
int	data;
{
	int	fd;
	int	*buf, i;
	int	err;

	unlink(s);
	if ((fd = open(s, (O_RDWR|O_CREAT|O_TRUNC), VSRT_PROT_ALL)) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open failed, errno = %d (%s)", err, errname(err));
		return -1;
	}

	if ((buf = (int *)malloc(sizeof(int) * size)) == NULL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("malloc failed, errno = %d (%s)", err, errname(err));
		return -1;
	}

	for (i = 0; i < size; i++) {
		buf[i] = data;
	}

	if (write(fd, (void*)buf, (sizeof(int)*size)) != (sizeof(int)*size)) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("write failed, errno = %d (%s)", err, errname(err));
		return -1;
	}

	close(fd);
	free(buf);

	if (chmod(s, mode) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("chmod failed, errno = %d (%s)", err, errname(err));
		return -1;
	}

	if ((fd = open(s, flag)) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open failed, errno = %d (%s)", err, errname(err));
		return -1;
	}

	return (fd);
}

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	The mapping established by a  call  to  mmap()  shall
	replace  any  previous mappings for those whole pages
	containing any part of the address space of the  process
	starting at pa and continuing for len bytes.
*/

private void test23()
{
	int	err;
	int	pathok = 0;
	int	fd1, fd2, fd3;
	int	*adr1, *adr2, *adr3;
	int	i, data;
	int	*buf;
	int	status;
	pid_t	pid;

	DBUG_ENTER("test23");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = create_map_file(test_file1, O_RDWR, VSRT_PROT_ALL, (5*vsrt_ipgsz), 11);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;
	fd2 = create_map_file(test_file2, O_RDWR, VSRT_PROT_ALL, (vsrt_ipgsz/2), 22);
	if (fd2 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;
	fd3 = create_map_file(test_file3, O_RDWR, VSRT_PROT_ALL, (2*vsrt_ipgsz), 33);
	if (fd3 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((adr1 = (int *)tb_mmap((void *)0, (size_t)(5*vsrt_pgsz), 
	     PROT_WRITE|PROT_READ, MAP_SHARED, fd1, 0)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

VSRT_FORK(pid);
	/*Verify contents of the mapped area*/
	for (i = 0; i < (5*vsrt_ipgsz); i++) {
		data = adr1[i];
		if (data != 11) {
			in_rpt("Invalid data in the mapped area at %d", i);
			in_rpt("Expected: %d Received: %d", 11, data);
			xx_rpt(FAILURE);
			DBUG_VOID_RETURN
		}
	}

	/*Map file2 such that it overlaps mapping of file1*/
	if ((adr2 = (int *)tb_mmap((void *)adr1, (size_t)(vsrt_pgsz), 
	     PROT_WRITE|PROT_READ, MAP_FIXED|MAP_SHARED, fd2, 0)) == (int *)MAP_FAILED) {
		err = errno;
#ifndef _XOPEN_SOURCE
		if (errno == ENOTSUP)
			xx_rpt(UNSUPPORTED);
		else
#endif
			xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	if (adr2 != adr1) {
		in_rpt("mmap did not map file2 at the desired address");
		in_rpt("Expected: %p  Received: %p", adr1, adr2);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	/*Map file3 such that it overlaps mapping of file1*/
	if ((adr3 = (int *)tb_mmap((void *)(adr1+4*vsrt_ipgsz), 
		(size_t)(2*vsrt_pgsz), PROT_READ, MAP_FIXED|MAP_SHARED, 
		fd3, 0)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	if (adr3 != (adr1+4*vsrt_ipgsz)) {
		in_rpt("mmap did not map file3 at the desired address");
		in_rpt("Expected: %p  Received: %p", (adr1+4*vsrt_ipgsz), adr3);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	/*New mapping replace the contents of previous mapping*/
	for (i = 0; i < (vsrt_ipgsz/2); i++) {
		data = adr2[i];
		if (data != 22) {
			in_rpt("Invalid data in the mapped area at %d", i);
			in_rpt("Expected: %d Received: %d", 22, data);
			xx_rpt(FAILURE);
			DBUG_VOID_RETURN
		}
	}
	for (i = 0; i < (2*vsrt_ipgsz); i++) {
		data = adr3[i];
		if (data != 33) {
			in_rpt("Invalid data in the mapped area at %d", i);
			in_rpt("Expected: %d Received: %d", 33, data);
			xx_rpt(FAILURE);
			DBUG_VOID_RETURN
		}
	}

	/*New mapping zero fills partial page at the end of file*/
	for (i = (vsrt_ipgsz/2); i < vsrt_ipgsz; i++) {
		data = adr2[i];
		if (data != 0) {
			in_rpt("Invalid data in the mapped area at %d", i);
			in_rpt("Expected: %d Received: %d", 0, data);
			xx_rpt(FAILURE);
			DBUG_VOID_RETURN
		}
	}


	/*Write references to the mapped area changes the new file*/
	for (i = 0; i < (vsrt_ipgsz/2); i++)
		adr2[i] = 66;

	if (msync((void *)adr2, (size_t)vsrt_pgsz/2, MS_SYNC) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msync failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	/*read from the file*/
	if (lseek(fd2, (off_t)0, SEEK_SET) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("lseek failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	if ((buf = (int *)malloc(vsrt_pgsz)) == NULL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("malloc failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	if (read(fd2, (void*)buf, (vsrt_pgsz/2)) != (vsrt_pgsz/2)) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("read failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;
	
	for (i = 0; i < (vsrt_ipgsz/2); i++) {
		if (buf[i] != 66) {
			in_rpt("Word %d in file was %d, should be 66", i, buf[i]);
			xx_rpt(FAILURE);
			DBUG_VOID_RETURN
		}
	}
	free(buf);

	/*Write references to the mapped area do not change the previously mapped file*/

	if (lseek(fd1, (off_t)0, SEEK_SET) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("lseek failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	if ((buf = (int *)malloc(vsrt_pgsz*2)) == NULL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("malloc failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;

	if (read(fd1, (void*)buf, (vsrt_pgsz)) != (vsrt_pgsz)) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("read failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	} else
		PATH_TRACE;
	
	for (i = 0; i < (vsrt_ipgsz); i++) {
		if (buf[i] != 11) {
			in_rpt(": Word %d in file was %d, should be 66", i, buf[i]);
			xx_rpt(FAILURE);
			DBUG_VOID_RETURN
		}
	}
	(void)free(buf);
	
	(void)close(fd1);
	(void)close(fd2);
	(void)close(fd3);

	(void)munmap((void *)adr1, (size_t)5*vsrt_pgsz);
	(void)munmap((void *)adr2, (size_t)vsrt_pgsz);
	(void)munmap((void *)adr3, (size_t)2*vsrt_pgsz);
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)unlink(test_file1);
	(void)unlink(test_file2);
	(void)unlink(test_file3);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	A call to mmap() when fildes  is  not  a  valid  file
	descriptor  shall  return MAP_FAILED and set errno to
	EBADF.
*/

private void test24()
{
	int	fd;
	int	err;
	int	pathok = 0;
	void	*result;

	DBUG_ENTER("test24");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	(void)close(fd);

	if ((result = tb_mmap(0, vsrt_pgsz, PROT_READ, MAP_SHARED, fd, 0)) != MAP_FAILED) {
		in_rpt("mmap() did not return MAP_FAILED");
		in_rpt("Expected: %p  Received: %p", MAP_FAILED, result);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (errno != EBADF) {
		err = errno;
		in_rpt("mmap failed but errno = %d (%s)", err, errname(err));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	A call to mmap() when fildes is not open for reading,
	regardless  of the protection specified, shall return
	MAP_FAILED and set errno to EACCES.
*/

private void test25()
{
	int	fd;
	int	err;
	int	pathok = 0;
	void	*result;

	DBUG_ENTER("test25");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_WRONLY, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((result = tb_mmap(0, vsrt_pgsz, PROT_READ, MAP_SHARED, fd, 0)) != MAP_FAILED) {
		in_rpt("mmap() did not return MAP_FAILED");
		in_rpt("Expected: %p  Received: %p", MAP_FAILED, result);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (errno != EACCES) {
		err = errno;
		in_rpt("mmap failed but errno = %d (%s)", err, errname(err));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	(void)close(fd);
	(void)unlink(test_file);
	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
and Headers, Issue 5:
	A call to mmap() when fildes is not open for  writing
	and  PROT_WRITE  is specified for a MAP_SHARED operation
	shall return MAP_FAILED and set errno to EACCES.
*/

private void test26()
{
	int	fd;
	int	err;
	int	pathok = 0;
	void	*result;

	DBUG_ENTER("test26");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDONLY, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((result = tb_mmap(0, vsrt_pgsz, PROT_WRITE, MAP_SHARED, fd, 0)) != MAP_FAILED) {
		in_rpt("mmap() did not return MAP_FAILED");
		in_rpt("Expected: %p  Received: %p", MAP_FAILED, result);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	if (errno != EACCES) {
		err = errno;
		in_rpt("mmap failed but errno = %d (%s)", err, errname(err));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	(void)close(fd);
	(void)unlink(test_file);
	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5:
	A call to mmap() when flags does not  contain  either
	MAP_PRIVATE or MAP_SHARED shall return MAP_FAILED and
	set errno to EINVAL.
*/

private void test27()
{
	int	fd;
	int	err;
	int	pathok = 0;
	void	*result;

	DBUG_ENTER("test27");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)

		DBUG_VOID_RETURN 
	else
		PATH_TRACE;
	if ((result = tb_mmap(0,vsrt_pgsz,PROT_WRITE, 0, fd, 0)) != MAP_FAILED) {
		in_rpt("mmap() did not return MAP_FAILED");
		in_rpt("Expected: %p  Received: %p", MAP_FAILED, result);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	if (errno != EINVAL) {
		err = errno;
		in_rpt("mmap failed but errno = %d (%s)", err, errname(err));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	(void)close(fd);
	(void)unlink(test_file);
	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
provides a file type for which mmap is not supported:
	A call to mmap() when fildes refers to an object  for
	which mmap() is not supported shall return MAP_FAILED
	and set errno to ENODEV.
*/

private void test28()
{
	int	fd;
	int	err;
	int	pathok = 0;
	char	*str;
	void	*result;

	DBUG_ENTER("test28");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	str = tet_getvar("VSRT_FILE_NO_MMAP");
	if ((str == NULL) || (strlen(str) == 0)) {
		in_rpt("No VSRT_FILE_NO_MMAP value specified in tet_exec.cfg");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
	else
		PATH_TRACE;

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;
	if ((fd = open(str, O_RDWR|O_NONBLOCK)) == -1) {
		err = errno;
		in_rpt("open failed for \"%s\", errno = %d (%s)", str, err, errname(err));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	else
		PATH_TRACE;

	if ((result = tb_mmap(0, vsrt_pgsz, PROT_READ, MAP_SHARED, fd, 0)) != MAP_FAILED) {
		in_rpt("mmap() did not return MAP_FAILED, Received: %p", result);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	if (errno != ENODEV) {
		err = errno;
		in_rpt("mmap failed but errno = %d (%s)", err, errname(err));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
supports MAP_FIXED:
	A call to mmap() when MAP_FIXED is not specified and there
	is insufficient room in the address space to effect the mapping
	shall return MAP_FAILED and set errno to ENOMEM.
*/

private void test29()
{
	int	fd;
	int	err;
	int	pathok = 0;
	unsigned long	limit;
	void	*addr, *addr1;
	char	*str;

	DBUG_ENTER("test29");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	str = tet_getvar("VSRT_ADDR_SPACE_PAGES");
	if ((str == NULL) || (strlen(str) == 0)) {
		in_rpt("No VSRT_ADDR_SPACE_PAGES value specified in tet_exec.cfg");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}
	/*map some pages to limit the amount of the address space available*/
	limit = atol(str);
	if ((addr1 = tb_mmap(0, (limit/2)*vsrt_pgsz, PROT_WRITE, MAP_SHARED, fd, 0)) == MAP_FAILED) {
		
		err = errno;
		in_rpt("first mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

	/*now go for one less than the max number of pages (in case would
	overflow when expanded to bytes*/
	if ((addr = tb_mmap(0, limit*vsrt_pgsz, PROT_WRITE, MAP_SHARED, fd, 0)) != MAP_FAILED) {
		in_rpt("mmap() did not return MAP_FAILED");
		in_rpt("Expected: %p  Received: %p", MAP_FAILED, addr);
		xx_rpt(FAILURE);
		(void)munmap((void *)addr, (limit-1)*vsrt_pgsz);
		(void)munmap((void *)addr1, (limit/2)*vsrt_pgsz);
		DBUG_VOID_RETURN 
	}
	if (errno != ENOMEM) {
#ifndef _XOPEN_SOURCE
		if (errno == ENOTSUP)
			xx_rpt(UNSUPPORTED);
		else
#endif
			xx_rpt(FAILURE);
		err = errno;
		in_rpt("mmap failed but errno = %d (%s)", err, errname(err));
		(void)munmap((void *)addr1, (limit/2)*vsrt_pgsz);
		DBUG_VOID_RETURN 
	}

	(void)close(fd);
	(void)unlink(test_file);
	(void)munmap((void *)addr1, (limit/2)*vsrt_pgsz);
	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 


/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
does  not support some of the possibly valid values for
prot:
	A call to mmap() when  the  implementation  does  not
	support the combination of accesses requested in prot
	shall return MAP_FAILED and set errno to ENOTSUP.
*/

struct {
	int     prot;
	char    *prot_string;
} protlist[] = {
	{ 0, "No unsupported access(es)" },
	{ (PROT_EXEC), "PROT_EXEC" },
	{ (PROT_EXEC|PROT_WRITE), "PROT_EXEC|PROT_WRITE" },
	{ (PROT_EXEC|PROT_READ), "PROT_EXEC|PROT_READ" },
};

private void test30()
{
	int	err;
	int	pathok = 0;
	int	prot, unsupported_prot, result;
	void	*ret;
	struct vsrt_exec_resource get;
	char	*str;

	DBUG_ENTER("test30");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	str = tet_getvar("VSRT_MMAP_UNSUPPORTED_PROT");
	if ((str == NULL) || (strlen(str) == 0)) {
		in_rpt("No VSU_MMAP_UNSUPPORTED_PROT value specified in tet_exec.cfg");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}

	unsupported_prot = atoi(str);
	if ((unsupported_prot < 0) || (unsupported_prot > 3)) {
		in_rpt("Configuration parameter VSU_MMAP_UNSUPPORTED_PROT");
		in_rpt("should be a value between 0 and 3");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

	if (unsupported_prot == 0) {
		in_rpt("Implementation has no unsupported access(es)");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
	in_rpt("Implementation does not support use of %s in mmap", protlist[unsupported_prot].prot_string);

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((result = vsrt_create_exec_file(test_file, O_RDONLY, VSRT_PROT_ALL, &get)) != 0)
		DBUG_VOID_RETURN 

	errno = 0;
	if ((ret = tb_mmap(0, (size_t)3*vsrt_pgsz, protlist[unsupported_prot].prot, MAP_SHARED, get.fd, (off_t)0)) != (void *)MAP_FAILED) {
		in_rpt("mmap() did not return MAP_FAILED");
		in_rpt("Expected: %p  Received: %p", MAP_FAILED, result);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	if (errno != ENOTSUP) {
		err = errno;
		in_rpt("mmap failed but errno = %d (%s)", err, errname(err));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	(void)close(get.fd);
	(void)unlink(test_file);
	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES is defined or the implementation
the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
does not support MAP_FIXED:
	A call to mmap() when MAP_FIXED is specified and  the
	implementation  does  not  support this functionality
	shall return MAP_FAILED and set errno to ENOTSUP.
*/

private void test31()
{
#ifndef _XOPEN_SOURCE
	int	pathok = 0;
	int	fd1;
	int	*ask_addr;
	int	*addr;
	int	err;
#endif

	DBUG_ENTER("test31");

	errors = 0;

#ifdef _XOPEN_SOURCE
	/*behavior only valid in POSIX95 mode*/
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN 
#else

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	/*Find an address that can be used as a fixed address for mapping text*/
	if ((ask_addr = (int *)tb_mmap((void *)0, (size_t)(3*vsrt_pgsz), 
	     PROT_NONE, MAP_SHARED, fd1, 0)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	}

	if ((addr = (int *)tb_mmap((void *)ask_addr, (size_t)vsrt_pgsz, 
	    PROT_WRITE|PROT_READ, MAP_FIXED|MAP_SHARED, fd1, 
	    (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
		err = errno;
		if (errno == ENOTSUP)
			xs_rpt();
		else {
			xx_rpt(FAILURE);
			in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
			(void)close(fd1);
			(void)unlink(test_file);
			errors++;
		}
	} else {
		xx_rpt(UNSUPPORTED);
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	}
	
	(void)close(fd1);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}
	DBUG_VOID_RETURN 
#endif
}

/*
If _POSIX_MAPPED_FILES is defined or the implementation
supports  the  mmap()  function  as  defined  in System
Interfaces and Headers, Issue 5 and the  implementation
does not support MAP_PRIVATE:
	A call to mmap() when MAP_PRIVATE  is  specified  and
	the  implementation does not support this functionality
	shall return MAP_FAILED and set errno to ENOTSUP.
*/

private void test32()
{
#ifndef _XOPEN_SOURCE
	int	fd1;
	int	pathok = 0;
	int	*ask_addr;
	int	err;
#endif

	DBUG_ENTER("test32");

	errors = 0;

#ifdef _XOPEN_SOURCE
	/*behavior only valid in POSIX95 mode*/
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN 
#else

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((ask_addr = (int *)tb_mmap((void *)0, (size_t)(3*vsrt_pgsz), 
	     PROT_READ, MAP_PRIVATE, fd1, 0)) == (int *)MAP_FAILED) {
		err = errno;
		if (errno == ENOTSUP)
			xs_rpt();
		else {
			xx_rpt(FAILURE);
			in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
			(void)close(fd1);
			(void)unlink(test_file);
			errors++;
		}
	} else {
		xx_rpt(UNSUPPORTED);
		(void)close(fd1);
		(void)unlink(test_file);
	}

	(void)close(fd1);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}
	DBUG_VOID_RETURN 
#endif
} 

/*
If _POSIX_MAPPED_FILES is not defined and the implemen-
tation  does not support the mmap() function as defined
in System Interfaces and Headers, Issue 5:
	A call to mmap()  shall  return  MAP_FAILED  and  set
	errno to ENOSYS.
*/

private void test33()
{

#ifndef _POSIX_MAPPED_FILES
	int	fd1, errors, err;
	void	*ask_addr;
	int	pathok = 0;
#endif

	DBUG_ENTER("test33");

	errors = 0;

#ifdef _POSIX_MAPPED_FILES
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
#else
	if (supports_mmap) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((ask_addr = tb_mmap((void *)0, (size_t)(3*vsrt_pgsz), 
	     PROT_READ, MAP_PRIVATE, fd1, 0)) != MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap succeeded");
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	}
	if (errno != ENOSYS) {
		xx_rpt(FAILURE);
		in_rpt("mmap failed but errno = %d (%s)", err, errname(err));
		(void)close(fd1);
		(void)unlink(test_file);
		DBUG_VOID_RETURN 
	}

	if (errors == 0) {
		xs_rpt();
	}
	DBUG_VOID_RETURN 
#endif
} 

/*
If _POSIX_SHARED_MEMORY_OBJECTS is defined or the implementa-
tion supports the mmap() function as defined in System
Interfaces and Headers, Issue 5 and the implementation
supports MAP_FIXED:
	A call to mmap() when MAP_FIXED is specified and the range
	[addr,addr+len] exceeds that allowed for the address space of
	a process shall return MAP_FAILED and set errno to ENOMEM.
*/

private void test34()
{
	int	fd;
	int	err;
	int	pathok = 0;
	unsigned long	limit;
	void	*addr, *addr1, *ask_addr;
	char	*str;

	DBUG_ENTER("test34");

	errors = 0;

#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	str = tet_getvar("VSRT_ADDR_SPACE_PAGES");
	if ((str == NULL) || (strlen(str) == 0)) {
		in_rpt("No VSRT_ADDR_SPACE_PAGES value specified in tet_exec.cfg");
		xx_rpt(FAILURE);
		(void)close(fd);
		DBUG_VOID_RETURN 
	}

	limit = atol(str);

	/*map some pages to limit the amount of the address space available*/
	if ((addr1 = tb_mmap(0, (limit/2)*vsrt_pgsz, PROT_WRITE, MAP_SHARED, fd, 0)) == MAP_FAILED) {
		
		err = errno;
		in_rpt("first mmap failed, errno = %d (%s)", err, errname(err));
		(void)close(fd);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

	/*Find a fixed address for mapping text*/
	if ((ask_addr = (int *)tb_mmap((void *)0, (size_t)(3*vsrt_pgsz), PROT_NONE, MAP_SHARED, fd, 0)) == (int *)MAP_FAILED) {
		vsrt_sysfail("mmap");
		(void)close(fd);
		DBUG_VOID_RETURN 
	}
	else
		PATH_TRACE;

	if (munmap((void *)ask_addr, (3*vsrt_pgsz)) == -1) {
		vsrt_sysfail("munmap");
		(void)close(fd);
		DBUG_VOID_RETURN 
	}

	/*now go for one less than the max number of pages (in case would
	overflow when expanded to bytes*/
	limit = atol(str);

	if ((addr = tb_mmap(ask_addr, (limit-1)*vsrt_pgsz, PROT_WRITE, MAP_FIXED|MAP_SHARED, fd, 0)) != MAP_FAILED) {
		in_rpt("second mmap() did not return MAP_FAILED");
		in_rpt("Expected: %p, received: %p", MAP_FAILED, addr);
		(void)close(fd);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}
	if (errno != ENOMEM) {
#ifndef _XOPEN_SOURCE
		if (errno == ENOTSUP) {
			in_rpt("Implementation does not support MAP_FIXED");
			xx_rpt(UNSUPPORTED);
		}
		else
#endif
			xx_rpt(FAILURE);
		err = errno;
		in_rpt("mmap failed but errno = %d (%s)", err, errname(err));
		(void)close(fd);
		DBUG_VOID_RETURN 
	}

	(void)close(fd);
	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

private	void * tb_mmap(addr, len, prot, flags, fildes, off)
void	*addr;
size_t	len;
int	prot;
int	flags;
int	fildes;
off_t	off;
{
	void 	*retval;

	retval = mmap(addr, len, prot, flags, fildes, off);
#ifdef	TESTBED
	errno++;
	return MAP_FAILED;
#else
	return retval;
#endif
}
