/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/mfiles/msync/msync.c
*
* Modifications:
* $Log: msync.c,v $
* Revision 1.2  2001/10/02 13:25:13  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:17  cyeoh
* Adds files for lsb-os test suite
*
*
*/


#include	<vsrt/vsrt.h>
#include	<sys/mman.h>
#include	<setjmp.h>
#include	<setprv.h>

#define 	NO_TESTS	1	/* number of tests in testset */

extern	int	setprv();

#ifdef UNDEF_MACROS
#undef msync
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

int	no_tests = NO_TESTS;

private void	test1();

extern	char	*errname();
private	int	tb_msync();

/*from vsrtlib/mkmmap.c*/
extern int	vsrt_got_sigsegv;
extern int	vsrt_got_sigbus;
extern int	vsrt_got_sig;
extern unsigned long	vsrt_pgsz;
extern unsigned long	vsrt_ipgsz;
extern int	vsrt_setjmp_called;
extern sigjmp_buf	vsrt_env;

static char     *test_file = "/tmp/vsrt_msync";

int	errors = 0;
int	supports_mmap;
int	supports_mlock;
int	supports_msync;

void atstart(void);
void atstart(void)
{
	(void)unlink(test_file);
	vsrt_check_support("MMAP", &supports_mmap);
	vsrt_check_support("MSYNC", &supports_msync);
	vsrt_check_support("MLOCK", &supports_mlock);
}
void atend(void);
void atend(void)
{
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = atend;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

/*
If _POSIX_MAPPED_FILES and  _POSIX_SYNCHRONIZED_IO  are
defined,  or  the  implementation  supports the msync()
function as defined in System Interfaces  and  Headers,
Issue 5:
	A call to msync() when some or all of  the  addresses
	in   the   range   [addr,addr+len]   are  locked  and
	MS_INVALIDATE is specified shall return  -1  and  set
	errno to EBUSY.
*/


private void test1()
{
	int	fd;
	int	err;
	int	pathok = 0;
	void	*addr;

	DBUG_ENTER("test1");

	errors= 0;

#if !defined(_POSIX_MAPPED_FILES) || !defined(_POSIX_SYNCHRONIZED_IO)
	if (!supports_msync) {
		in_rpt("msync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#ifndef _POSIX_MEMLOCK_RANGE
	if (!supports_mlock) {
		in_rpt("mlock not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#if !defined(_POSIX_MAPPED_FILES)
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (setprv(PRV_MEMLOCK) != 0) {
		xx_rpt(FAILURE);
		in_rpt("setprv(PRV_MEMLOCK) failed");
		DBUG_VOID_RETURN 
        }

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN
	PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDWR, (mode_t)VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN
	PATH_TRACE;

	DBUG_PRINTF("t1", "map two pages");
	if ((addr = (int *)mmap(0, vsrt_pgsz*2, PROT_READ, MAP_SHARED, fd, 0)) == (int *)MAP_FAILED) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN
	}
	PATH_TRACE;

	DBUG_PRINTF("t1", "lock the first");
	if (mlock(addr, vsrt_pgsz) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mlock failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN
	}
	PATH_TRACE;

	DBUG_PRINTF("t1", "msync the locked area");
	if (tb_msync(addr, vsrt_pgsz, MS_INVALIDATE) != -1) {
		xx_rpt(FAILURE);
		in_rpt("msync succeeded with all of range locked");
		DBUG_VOID_RETURN
	}
	else {
		if (errno != EBUSY) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("msync failed but errno = %d (%s)", err, errname(err));
			DBUG_VOID_RETURN
		}
	}
	PATH_TRACE;

	DBUG_PRINTF("t1", "msync the entire mapped area");
	if (tb_msync(addr, vsrt_pgsz*2, MS_ASYNC|MS_INVALIDATE) != -1) {
		xx_rpt(FAILURE);
		in_rpt("msync succeeded with first part of range locked");
		DBUG_VOID_RETURN
	}
	else {
		if (errno != EBUSY) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("second msync failed but errno = %d (%s)", err, errname(err));
			DBUG_VOID_RETURN
		}
	}
	PATH_TRACE;

	DBUG_PRINTF("t1", "unlock the page");
	(void)munlock(addr, vsrt_pgsz);

	DBUG_PRINTF("t1", "lock the second page");
	if (mlock((void *)((char *)addr + vsrt_pgsz), vsrt_pgsz) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("mlock failed, errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN
	}
	PATH_TRACE;

	DBUG_PRINTF("t1", "msync the entire mapped area");
	if (tb_msync(addr, vsrt_pgsz*2, MS_SYNC|MS_INVALIDATE) != -1) {
		xx_rpt(FAILURE);
		in_rpt("msync succeeded with second half of range locked");
		DBUG_VOID_RETURN
	}
	else {
		if (errno != EBUSY) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("third msync failed but errno = %d (%s)", err, errname(err));
			DBUG_VOID_RETURN
		}
	}

	(void)munlock(addr, vsrt_pgsz);
	(void)munmap((void *)addr, (size_t)vsrt_pgsz*2);
	(void)close(fd);
	(void)unlink(test_file);

	if (errors == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN 
} 

private	int tb_msync(addr, len, flags)
void	*addr;
size_t	len;
int	flags;
{
	int 	retval;

	retval = msync(addr, len, flags);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
