/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/mfiles/msync_P/msync_P.c
*
* Modifications:
* $Log: msync_P.c,v $
* Revision 1.2  2001/10/02 13:25:13  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:17  cyeoh
* Adds files for lsb-os test suite
*
*
*/

#include	<vsrt/vsrt.h>
#include	<sys/mman.h>
#include	<setjmp.h>
#include	<setprv.h>

#define 	NO_TESTS	9	/* number of tests in testset */

#ifdef UNDEF_MACROS
#undef msync
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();

extern	char	*errname();
private	int	tb_msync();

/*from vsrtlib/mkmmap.c*/
extern int	vsrt_got_sigsegv;
extern int	vsrt_got_sigbus;
extern int	vsrt_got_sig;
extern unsigned long	vsrt_pgsz;
extern unsigned long	vsrt_ipgsz;
extern int	vsrt_setjmp_called;
extern sigjmp_buf	vsrt_env;

static char     *test_file = "/tmp/vsrt_msync";

int	errors = 0;
int	supports_mmap;
int	supports_mlock;
int	supports_msync;

#define MAGIC_STR	"String for the msync test"

void atstart(void);
void atstart(void)
{
	(void)unlink(test_file);
	vsrt_check_support("MMAP", &supports_mmap);
	vsrt_check_support("MSYNC", &supports_msync);
}
void atend(void);
void atend(void)
{
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = atend;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	NULL, 0
};

/*
If _POSIX_MAPPED_FILES and  _POSIX_SYNCHRONIZED_IO  are
defined,  or  the  implementation  supports the msync()
function as defined in System Interfaces  and  Headers,
Issue 5:
	A successful call to msync() when flags contains  the
	flag  MS_ASYNC  shall  write  all  modified copies of
	pages in the range [addr,addr+len] to their permanent
	storage   locations  using  asynchronous  writes  and
	return 0.
*/


private void test1()
{
	int	fd;
	int	s2;
	int	pathok = 0;
	char	*adr;
	struct  stat st_buf1, st_buf2;
	time_t  s_time, e_time;
	int	 *buf;

	DBUG_ENTER("test1");

	errors= 0;

#if !defined(_POSIX_MAPPED_FILES) || !defined(_POSIX_SYNCHRONIZED_IO)
	if (!supports_msync) {
		in_rpt("msync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#if !defined(_POSIX_MAPPED_FILES)
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	/*Allocate memory for buffer*/
	if ((buf = (int *)malloc((size_t)vsrt_pgsz)) == NULL) {
		vsrt_sysfail("malloc");
		DBUG_VOID_RETURN
	}

	fd = vsrt_open_file(test_file, O_RDWR, (mode_t)VSRT_PROT_ALL, 3);
	if (fd == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	adr = (char *)vsrt_map3(fd, PROT_READ, MAP_SHARED, PROT_WRITE, 
			  MAP_SHARED, PROT_READ, MAP_SHARED);
	if (adr == (char *)-1)
		DBUG_VOID_RETURN

	/*Stat file & then modify the shared page*/
	if (time(&s_time) == -1) {
		vsrt_sysfail("time");
		DBUG_VOID_RETURN
	}

	if (fstat(fd, &st_buf1) == -1) {
		vsrt_sysfail("fstat");
		DBUG_VOID_RETURN
	}

	sleep(WAIT_FOR_TIME);

	strcpy((char *)(adr+vsrt_pgsz), MAGIC_STR);

	if ((s2 = tb_msync((void *)(adr+vsrt_pgsz), (size_t)vsrt_pgsz, MS_ASYNC)) != 0) {
		vsrt_sysfail("msync");
		DBUG_VOID_RETURN
	}

	sleep(WAIT_FOR_TIME);

	if (time(&e_time) == -1) {
		vsrt_sysfail("time");
		DBUG_VOID_RETURN
	}

	if (fstat(fd, &st_buf2) == -1) {
		vsrt_sysfail("second fstat");
		DBUG_VOID_RETURN
	}

	vsrt_ck_stime(CHK_MTIME, TRUE, s_time, &st_buf1, e_time, &st_buf2);

	if (lseek(fd, vsrt_pgsz, SEEK_SET) == -1) {
		vsrt_sysfail("lseek");
		DBUG_VOID_RETURN
	}

	if (read(fd, (void*)buf, (size_t)vsrt_pgsz) == -1) {
		vsrt_sysfail("read");
		DBUG_VOID_RETURN
	}

	if (strcmp((char *)buf, MAGIC_STR) != 0) {
		in_rpt("Disk was not updated");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	(void)free(buf);
	(void)munmap((void *)adr, (size_t)(3*vsrt_pgsz));
	(void)close(fd);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 
/*
If _POSIX_MAPPED_FILES and  _POSIX_SYNCHRONIZED_IO  are
defined,  or  the  implementation  supports the msync()
function as defined in System Interfaces  and  Headers,
Issue 5:
	A successful call to msync() when flags contains  the
	flag  MS_SYNC  shall  write  all  modified copies of
	pages in the range [addr,addr+len] to their permanent
	storage   locations  using  synchronous  writes  and
	return 0.
*/


private void test2()
{
	int	fd;
	int	s2;
	int	pathok = 0;
	char	*adr;
	struct  stat st_buf1, st_buf2;
	time_t  s_time, e_time;
	int	 *buf;

	DBUG_ENTER("test2");

	errors= 0;

#if !defined(_POSIX_MAPPED_FILES) || !defined(_POSIX_SYNCHRONIZED_IO)
	if (!supports_msync) {
		in_rpt("msync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#if !defined(_POSIX_MAPPED_FILES)
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	/*Allocate memory for buffer*/
	if ((buf = (int *)malloc((size_t)vsrt_pgsz)) == NULL) {
		vsrt_sysfail("malloc");
		DBUG_VOID_RETURN
	}

	fd = vsrt_open_file(test_file, O_RDWR, (mode_t)VSRT_PROT_ALL, 3);
	if (fd == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	adr = (char *)vsrt_map3(fd, PROT_READ, MAP_SHARED, PROT_WRITE, 
			  MAP_SHARED, PROT_READ, MAP_SHARED);
	if (adr == (char *)-1)
		DBUG_VOID_RETURN

	/*Stat file & then modify the shared page*/
	if (time(&s_time) == -1) {
		vsrt_sysfail("time");
		DBUG_VOID_RETURN
	}

	if (fstat(fd, &st_buf1) == -1) {
		vsrt_sysfail("fstat");
		DBUG_VOID_RETURN
	}

	sleep(WAIT_FOR_TIME);

	strcpy((char *)(adr+vsrt_pgsz), MAGIC_STR);

	if ((s2 = tb_msync((void *)(adr+vsrt_pgsz), (size_t)vsrt_pgsz, MS_ASYNC)) != 0) {
		vsrt_sysfail("msync");
		DBUG_VOID_RETURN
	}

	sleep(WAIT_FOR_TIME);

	if (time(&e_time) == -1) {
		vsrt_sysfail("time");
		DBUG_VOID_RETURN
	}

	if (fstat(fd, &st_buf2) == -1) {
		vsrt_sysfail("second fstat");
		DBUG_VOID_RETURN
	}

	vsrt_ck_stime(CHK_MTIME, TRUE, s_time, &st_buf1, e_time, &st_buf2);

	if (lseek(fd, vsrt_pgsz, SEEK_SET) == -1) {
		vsrt_sysfail("lseek");
		DBUG_VOID_RETURN
	}

	if (read(fd, (void*)buf, (size_t)vsrt_pgsz) == -1) {
		vsrt_sysfail("read");
		DBUG_VOID_RETURN
	}

	if (strcmp((char *)buf, MAGIC_STR) != 0) {
		in_rpt("Disk was not updated");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	(void)free(buf);
	(void)munmap((void *)adr, (size_t)(3*vsrt_pgsz));
	(void)close(fd);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES and  _POSIX_SYNCHRONIZED_IO  are
defined,  or  the  implementation  supports the msync()
function as defined in System Interfaces  and  Headers,
Issue 5:
	A successful call to msync() when flags contains  the
	flag MS_INVALIDATE shall invalidate all cached copies
	of all pages in the range [addr,addr+len] and  return 0.
*/

private void test3()
{
	int	err;
	int	s2;
	int	pathok = 0;
	int	fd, i;
	char	*adr;
	struct {
		int flag;
		char *name;
	} *flagp, flags[] = {
		{MAP_SHARED, "MAP_SHARED"}, 	/* first structure element can't have MAP_FIXED */
		{MAP_PRIVATE, "MAP_PRIVATE"},
		{MAP_SHARED|MAP_FIXED, "MAP_SHARED|MAP_FIXED"},
		{MAP_SHARED|MAP_FIXED, "MAP_PRIVATE|MAP_FIXED"},
		{0, 0} };

	DBUG_ENTER("test3");

	errors= 0;

#if !defined(_POSIX_MAPPED_FILES) || !defined(_POSIX_SYNCHRONIZED_IO)
	if (!supports_msync) {
		in_rpt("msync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#if !defined(_POSIX_MAPPED_FILES)
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDWR, (mode_t)VSRT_PROT_ALL, 3);
	if (fd == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	flagp = flags;
	while ( flagp->flag ) {
                if (flagp->flag & MAP_FIXED) {
			/*Map three pages at a fixed address*/
               		/* Use previous good address as fixed address.*/
			/* Note that flags[0] cannot include MAP_FIXED*/
               		adr = (char *)mmap((void *)adr, (size_t)(3*vsrt_pgsz),(PROT_WRITE|PROT_READ), flagp->flag, fd, 0);
		} else {
	                /*Map three pages*/
                	adr = (char *)mmap((void *)0, (size_t)(3*vsrt_pgsz),(PROT_WRITE|PROT_READ), flagp->flag, fd, 0);
		}
                if (adr == (char *) -1) {
#ifndef _XOPEN_SOURCE
			if (errno == ENOTSUP)
				continue;
#endif
			xx_rpt(FAILURE);
			in_rpt("mmap failed testing %s, errno = %d (%s)", flagp->name, err, errname(err));
			DBUG_VOID_RETURN
                }
                
                /*Read data from mapped page to be sure it's in memory*/
                i = *(adr + vsrt_pgsz);
                
                /*After the file is mapped modify it using the file descriptor*/
                if (lseek(fd, vsrt_pgsz, SEEK_SET) == -1) {
			xx_rpt(FAILURE);
			in_rpt("lseek failed testing %s, errno = %d (%s)", flagp->name, err, errname(err));
			DBUG_VOID_RETURN
                }
                
                if (write(fd, MAGIC_STR, sizeof(MAGIC_STR)) == -1) {
			xx_rpt(FAILURE);
			in_rpt("write failed testing %s, errno = %d (%s)", flagp->name, err, errname(err));
			DBUG_VOID_RETURN
                }
                
                if ((s2 = tb_msync((void *)adr, (size_t)(3*vsrt_pgsz), MS_INVALIDATE)) != 0) {
			in_rpt("msync failed, errno = %d (%s)", err, errname(err));
			xx_rpt(FAILURE);
			DBUG_VOID_RETURN
                }
                
                /*invalidates the cached copies in memory*/
                if (strcmp((char *)(adr+vsrt_pgsz), MAGIC_STR) != 0) {
                	in_rpt("Memory not synchronized with file");
			xx_rpt(FAILURE);
			DBUG_VOID_RETURN
                }

                (void)munmap((void*)adr, (size_t)(3*vsrt_pgsz));
                flagp++;
	}

	(void)close(fd);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES and  _POSIX_SYNCHRONIZED_IO  are
defined,  or  the  implementation  supports the msync()
function as defined in System Interfaces  and  Headers,
Issue 5:
	A call to msync() when the mapped file is written  to
	shall mark the st_ctime field of the file for update.
*/

private void test4()
{
	int	pathok = 0;
	int	*addr;
	int	fd1, j;
	struct stat	end_stat;
	time_t	start_time, end_time;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test4");

	errors= 0;

#if !defined(_POSIX_MAPPED_FILES) || !defined(_POSIX_SYNCHRONIZED_IO)
	if (!supports_msync) {
		in_rpt("msync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#if !defined(_POSIX_MAPPED_FILES)
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, (mode_t)VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	/*Obtain start time*/
	time(&start_time);

	sleep(1);

	vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

	if ((addr = (int *)mmap(0, (size_t)vsrt_pgsz, PROT_READ|PROT_WRITE,
	     MAP_SHARED, fd1, 0)) == (int *)-1) {
		vsrt_sysfail("mmap");
		DBUG_VOID_RETURN
	}

	sleep(2 * WAIT_FOR_TIME);
VSRT_FORK(pid);
	/*Make write references on the file*/
	for (j = 0; !vsrt_got_sig && (j < vsrt_pgsz/sizeof(int)); j++)
		addr[j] = 99999;

	if (vsrt_got_sigsegv) {
		in_rpt("SIGSEGV received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (vsrt_got_sigbus) {
		in_rpt("SIGBUS received");
		xx_rpt(FAILURE);
		exit(1);
	}

	sleep(WAIT_FOR_TIME);

	if (tb_msync((void *)addr, (size_t)vsrt_pgsz, MS_SYNC) == -1) {
		vsrt_sysfail("msync");
		exit(1);
	}

	sleep(1);

	/*Obtain end time*/
	time(&end_time);

	if (stat(test_file, &end_stat) < 0) {
		vsrt_sysfail("stat");
		exit(1);
	}

	vsrt_ck_stime(CHK_CTIME, 1, start_time, (struct stat *)NULL, end_time, &end_stat);
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else 
		errors += WEXITSTATUS(status);

	(void)close(fd1);
	(void)munmap((void *)addr, (size_t)vsrt_pgsz);

	(void)unlink(test_file);
	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MAPPED_FILES and  _POSIX_SYNCHRONIZED_IO  are
defined,  or  the  implementation  supports the msync()
function as defined in System Interfaces  and  Headers,
Issue 5:
	A call to msync() when the mapped file is written  to
	shall mark the st_mtime field of the file for update.
*/

private void test5()
{
	int	pathok = 0;
	int	*addr;
	int	fd1, j;
	struct stat	end_stat;
	time_t	start_time, end_time;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test5");

	errors= 0;

#if !defined(_POSIX_MAPPED_FILES) || !defined(_POSIX_SYNCHRONIZED_IO)
	if (!supports_msync) {
		in_rpt("msync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#if !defined(_POSIX_MAPPED_FILES)
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, (mode_t)VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	/*Obtain start time*/
	time(&start_time);

	sleep(1);

	vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

	if ((addr = (int *)mmap(0, (size_t)vsrt_pgsz, PROT_READ|PROT_WRITE,
	     MAP_SHARED, fd1, 0)) == (int *)-1) {
		vsrt_sysfail("mmap");
		DBUG_VOID_RETURN
	}

	sleep(2 * WAIT_FOR_TIME);
VSRT_FORK(pid);
	/*Make write references on the file*/
	for (j = 0; !vsrt_got_sig && (j < vsrt_pgsz/sizeof(int)); j++)
		addr[j] = 99999;

	if (vsrt_got_sigsegv) {
		in_rpt("SIGSEGV received");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (vsrt_got_sigbus) {
		in_rpt("SIGBUS received");
		xx_rpt(FAILURE);
		exit(1);
	}

	sleep(WAIT_FOR_TIME);

	if (tb_msync((void *)addr, (size_t)vsrt_pgsz, MS_SYNC) == -1) {
		vsrt_sysfail("msync");
		exit(1);
	}

	sleep(1);

	/*Obtain end time*/
	time(&end_time);

	if (stat(test_file, &end_stat) < 0) {
		vsrt_sysfail("stat");
		exit(1);
	}

	vsrt_ck_stime(CHK_MTIME, 1, start_time, (struct stat *)NULL, end_time, &end_stat);
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else 
		errors += WEXITSTATUS(status);

	(void)close(fd1);
	(void)munmap((void *)addr, (size_t)vsrt_pgsz);

	(void)unlink(test_file);
	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
} 

/*This assertion is not in use*/

private void test6()
{
	int	pathok = 0;

	DBUG_ENTER("test6");

		xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN 
}

/*If _POSIX_MAPPED_FILES and  _POSIX_SYNCHRONIZED_IO  are
defined,  or  the  implementation  supports the msync()
function as defined in System Interfaces  and  Headers,
Issue 5:
	A call to msync() when pages not mapped are specified
	shall return -1 and set errno to ENOMEM.
*/

private void test7()
{
	int	fd1;
	int	err;
	int	s2;
	int	pathok = 0;
	void	*addr;

	DBUG_ENTER("test7");

	errors = 0;

#if !defined(_POSIX_MAPPED_FILES) || !defined(_POSIX_SYNCHRONIZED_IO)
	if (!supports_msync) {
		in_rpt("msync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#if !defined(_POSIX_MAPPED_FILES)
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, (mode_t)VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((addr = (int *)mmap(0, (size_t)vsrt_pgsz, PROT_READ|PROT_WRITE,
	     MAP_SHARED, fd1, 0)) == (int *)-1) {
		vsrt_sysfail("mmap");
		DBUG_VOID_RETURN
	}

	if ((s2 = msync((void *)addr, (size_t)vsrt_pgsz*2, MS_SYNC)) != -1) {
		in_rpt("msync() did not return -1, returned %d", s2);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}
	if (errno != ENOMEM) {
		err = errno;
		in_rpt("msync failed but errno = %d (%s)", err, errname(err));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}
	if ((s2 = msync((void *)addr, (size_t)vsrt_pgsz*2, MS_ASYNC)) != -1) {
		in_rpt("msync() did not return -1, returned %d", s2);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}
	if (errno != ENOMEM) {
		err = errno;
		in_rpt("msync failed but errno = %d (%s)", err, errname(err));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

	(void)munmap((void *)addr, (size_t)vsrt_pgsz);
	(void)close(fd1);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
}
/*If _POSIX_MAPPED_FILES and  _POSIX_SYNCHRONIZED_IO  are
defined,  or  the  implementation  supports the msync()
function as defined in System Interfaces  and  Headers,
Issue 5:
	A call to msync() when flags is invalid shall  return
	-1 and set errno to EINVAL.
*/

private void test8()
{
	int	fd1;
	int	err;
	int	s2;
	int	pathok = 0;
	void	*addr;

	DBUG_ENTER("test8");

	errors = 0;

#if !defined(_POSIX_MAPPED_FILES) || !defined(_POSIX_SYNCHRONIZED_IO)
	if (!supports_msync) {
		in_rpt("msync not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#if !defined(_POSIX_MAPPED_FILES)
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	fd1 = vsrt_open_file(test_file, O_RDWR, (mode_t)VSRT_PROT_ALL, 2);
	if (fd1 == -1)
		DBUG_VOID_RETURN 
	else
		PATH_TRACE;

	if ((addr = (int *)mmap(0, (size_t)vsrt_pgsz, PROT_READ|PROT_WRITE,
	     MAP_SHARED, fd1, 0)) == (int *)-1) {
		vsrt_sysfail("mmap");
		DBUG_VOID_RETURN
	}

	if ((s2 = msync((void *)addr, (size_t)vsrt_pgsz, MS_SYNC|MS_ASYNC)) != -1) {
		in_rpt("msync() with flags = MS_SYNC|MS_ASYNC did not return -1, returned %d", s2);
		xx_rpt(FAILURE);
	} else {
		if (errno != EINVAL) {
			err = errno;
			in_rpt("msync failed with flags = MS_SYNC|MS_ASYNC but errno = %d (%s)", err, errname(err));
			xx_rpt(FAILURE);
		} 
	}

	(void)munmap((void *)addr, (size_t)vsrt_pgsz);
	(void)close(fd1);
	(void)unlink(test_file);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
}

/*
If _POSIX_MAPPED_FILES  or  _POSIX_SYNCHRONIZED_IO  are
not defined, or the implementation does not support the
msync() function as defined in  System  Interfaces  and
Headers, Issue 5:
	A call to msync() shall return -1 and  set  errno  to ENOSYS.
*/

private void test9()
{
#if !defined(_POSIX_SYNCHRONIZED_IO) || !defined(_POSIX_MAPPED_FILES)
	int	err;
#endif

	DBUG_ENTER("test9");

	errors = 0;

#if defined(_POSIX_SYNCHRONIZED_IO) && defined(_POSIX_MAPPED_FILES)
		in_rpt("_POSIX_SYNCHRONIZED_IO and _POSIX_MAPPED_FILES defined");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
#else
	if (supports_msync) {
		in_rpt("Implementation supports msync");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}

	if (tb_msync((void *)0, (size_t)vsrt_pgsz, MS_ASYNC) != -1) {
		xx_rpt(FAILURE);
		in_rpt("msync succeeded");
		DBUG_VOID_RETURN 
	}
	if (errno != ENOSYS) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("msync failed but errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN 
	}

	xs_rpt();
	DBUG_VOID_RETURN 
#endif
} 

private	int tb_msync(addr, len, flags)
void	*addr;
size_t	len;
int	flags;
{
	int 	retval;

	retval = msync(addr, len, flags);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
