/*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
* File: tset/LSB.os/mfiles/munmap_P/munmap_P.c
*
* Modifications:
* $Log: munmap_P.c,v $
* Revision 1.2  2001/10/02 09:31:47  ajosey
* remove parts of test using functionality outside the LSB specification
*
* Revision 1.1  2001/05/27 06:59:17  cyeoh
* Adds files for lsb-os test suite
*
*
*/

#include	<vsrt/vsrt.h>
#include	<sys/mman.h>
#include	<setjmp.h>
#include	<setprv.h>

#define 	NO_TESTS	6	/* number of tests in testset */

#ifdef UNDEF_MACROS
#undef munmap
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();

int	no_tests = NO_TESTS;

private	int	tb_munmap();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();

int	supports_munmap;
int	supports_mmap;

/*from vsrtlib/mkmmap.c*/
extern int	vsrt_got_sigsegv;
extern int	vsrt_got_sigbus;
extern int	vsrt_got_sig;
extern unsigned long	vsrt_pgsz;
extern unsigned long	vsrt_ipgsz;
extern int	vsrt_setjmp_called;
extern sigjmp_buf	vsrt_env;

static char	*test_file = "/tmp/vsrt_msync";
char	shbuf[2048];
char	segname[] = "vsrt_shmem2";

void atstart(void);
void atstart(void)
{
	(void)unlink(test_file);
	vsrt_check_support("MUNMAP", &supports_munmap);
	vsrt_check_support("MMAP", &supports_mmap);
}
void atend(void);
void atend(void)
{
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = atend;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	NULL, 0
};

/*
If  _POSIX_MAPPED_FILES 
is defined or the imple-
mentation supports the munmap() function as defined in
System Interfaces and Headers, Issue 5.
	A successful call to munmap() shall remove the map-
	pings of the pages in the range [addr,addr+len] esta-
	blished by a prior call to mmap() from the address
	space of the process and return 0.
*/

private int test1p1();
private int test1p2();
private int dotest1();

private void test1()
{
	int	ret1, ret2;

	DBUG_ENTER("test1");

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	ret1 = test1p1();
	ret2 = test1p2();

	/*any error is a failure*/
	if ((ret1 == -1) || (ret2 == -1)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	/*if both parts cannot be run its unsupported*/
	if ((ret1 == 1) && (ret2 == 1)) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	/*if either part is run successfully its a pass*/
	xs_rpt();

	DBUG_VOID_RETURN
}

/*test for files*/
private int test1p1()
{
	int	fd, ret;

	DBUG_ENTER("test1p1");

#ifndef _POSIX_MAPPED_FILES
	if (!supports_munmap) {
		in_rpt("munmap not supported");
		DBUG_RETURN(1);
	}
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		DBUG_RETURN(1);
	}
#endif

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_RETURN(-1);

	ret = dotest1(fd);

	(void)close(fd);
	(void)unlink(test_file);

	DBUG_RETURN(ret);
}
/*test for shared memory*/
private int test1p2()
{
	int	fd, ret;

	DBUG_ENTER("test1p2");

	in_rpt("shared memory not required");
	DBUG_RETURN(1);
}


private int dotest1(fd)
int	fd;
{
	int	err;
	int	pathok = 0;
	volatile int	i;
	int	*adr1;
	pid_t	pid;
	int 	status;

	DBUG_ENTER("dotest1");

	if ((adr1 = (int *)mmap(0, (size_t)(vsrt_pgsz*2), PROT_READ, MAP_SHARED, fd, 0)) == (int *)MAP_FAILED) {
		err = errno;
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tb_munmap((void *)adr1, (size_t)vsrt_pgsz) != 0) {
		err = errno;
		in_rpt("munmap failed, errno = %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	VSRT_FORK(pid);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
			i = *adr1;
		if (!vsrt_got_sigsegv) {
			if (vsrt_got_sigbus) {
				in_rpt("Accessing unmapped page generated SIGBUS, expected SIGSEGV");
				exit(1);
			} else {
				in_rpt("Accessing unmapped page did not generated SIGSEGV");
				exit(1);
			}
		}
	VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_RETURN(-1);
	else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0) {
			if (WTERMSIG(status) != SIGSEGV) {
				in_rpt("Child process 1 exited due to signal %d, expected SIGSEGV", WTERMSIG(status));
				DBUG_RETURN(-1);
			}
		}
		else {
			in_rpt("Child process exited abnormally with exit status %d, expected exit due to SIGSEGV", WEXITSTATUS(status));
			DBUG_RETURN(-1);
		}
	}

	/*Unreferenced mapping still present*/
	VSRT_FORK(pid);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
			i = adr1[(vsrt_pgsz/sizeof(int))];
		if (vsrt_got_sigsegv) {
			in_rpt("SIGSEGV received in child 2");
			exit(1);
		}

		if (vsrt_got_sigbus) {
			in_rpt("SIGBUS received in child 2");
			exit(1);
		}
	VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_RETURN(-1);
	else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process 2 exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process 2 exited abnormally with exit status %d", WEXITSTATUS(status));
		DBUG_RETURN(-1);
	}

	(void)munmap((void *)&adr1[vsrt_pgsz/sizeof(int)], (size_t)vsrt_pgsz);
	(void)close(fd);
	(void)unlink(test_file);
	DBUG_RETURN(0);
} 


/*
If _POSIX_MAPPED_FILES 
is defined or the imple-
mentation supports the munmap() function as defined in
System Interfaces and Headers, Issue 5.
	A successful call to int munmap(void *addr, size_t
	len) shall round len up to the next multiple of the
	page size.
*/

private int test2p1();
private int test2p2();
private int dotest2();

private void test2()
{
	int	ret1, ret2;

	DBUG_ENTER("test2");

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	ret1 = test2p1();
	ret2 = test2p2();

	/*any error is a failure*/
	if ((ret1 == -1) || (ret2 == -1)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	/*if both parts cannot be run its unsupported*/
	if ((ret1 == 1) && (ret2 == 1)) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	/*if either part is run successfully its a pass*/
	xs_rpt();

	DBUG_VOID_RETURN
}

/*test for files*/
private int test2p1()
{
	int	fd, ret;

	DBUG_ENTER("test2p1");

#ifndef _POSIX_MAPPED_FILES
	if (!supports_munmap) {
		in_rpt("munmap not supported");
		DBUG_RETURN(1);
	}
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		DBUG_RETURN(1);
	}
#endif

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_RETURN(-1);

	ret = dotest2(fd);

	(void)close(fd);
	(void)unlink(test_file);

	DBUG_RETURN(ret);
}

/*test for shared memory*/
private int test2p2()
{
	int	fd, ret;

	DBUG_ENTER("test2p2");

	in_rpt("shared memory not required");
	DBUG_RETURN(1);
}

private int dotest2(fd)
int fd;
{
	int	err;
	int	pathok = 0;
	volatile int	i;
	int	*adr1, len;
	pid_t	pid;
	int 	status;

	DBUG_ENTER("dotest2");

	/*Map two pages*/
	if ((adr1 = (int *)mmap(0, (size_t)(vsrt_pgsz*2), PROT_READ, MAP_SHARED, fd, 0)) == (int *)MAP_FAILED) {
		err = errno;
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	/*Unmap half a page*/
	len = vsrt_pgsz / 2;
	if (tb_munmap((void *)adr1, (size_t)len) != 0) {
		err = errno;
		in_rpt("munmap failed, errno = %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	/*access start of page*/
	VSRT_FORK(pid);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
			i = *adr1;
		if (!vsrt_got_sigsegv) {
			if (vsrt_got_sigbus) {
				in_rpt("Child 1: Accessing unmapped page generated SIGBUS, expected SIGSEGV");
				exit(1);
			} else {
				in_rpt("Child 1: Accessing unmapped page did not generate SIGSEGV");
				exit(1);
			}
		}
	VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_RETURN(-1);
	else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0) {
			if (WTERMSIG(status) != SIGSEGV) {
				in_rpt("Child process 1 exited due to signal %d, expected SIGSEGV", WTERMSIG(status));
				DBUG_RETURN(-1);
			}
		}
		else {
			in_rpt("Child process 1 exited abnormally with exit status %d, expected exit due to SIGSEGV", WEXITSTATUS(status));
			DBUG_RETURN(-1);
		}
	}

	/*Access start of second half of page*/
	VSRT_FORK(pid);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
			i = adr1[vsrt_ipgsz/2+1];
		if (!vsrt_got_sigsegv) {
			if (vsrt_got_sigbus) {
				in_rpt("Child 2: Accessing unmapped page generated SIGBUS, expected SIGSEGV");
				exit(1);
			} else {
				in_rpt("Child 2: The length of the unmap was not rounded to the");
				in_rpt("nearest multiple of page size");
				in_rpt("At offset %d expected no mapping but was mapped", vsrt_pgsz/2+1);
				exit(1);
			}
		}
	VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_RETURN(-1);
	else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0) {
			if (WTERMSIG(status) != SIGSEGV) {
				in_rpt("Child process 2 exited due to signal %d, expected SIGSEGV", WTERMSIG(status));
				DBUG_RETURN(-1);
			}
		}
		else {
			in_rpt("Child process 2 exited abnormally with exit status %d, expected exit due to SIGSEGV", WEXITSTATUS(status));
			DBUG_RETURN(-1);
		}
	}

	/*Access end of second half of page*/
	VSRT_FORK(pid);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
			i = adr1[vsrt_ipgsz-sizeof(int)];
		if (!vsrt_got_sigsegv) {
			if (vsrt_got_sigbus) {
				in_rpt("Child 3: Accessing unmapped page generated SIGBUS, expected SIGSEGV");
				exit(1);
			} else {
				in_rpt("Child 3: The length of the unmap was not rounded to the");
				in_rpt("nearest multiple of page size");
				in_rpt("At offset %d expected no mapping but was mapped", vsrt_pgsz-sizeof(int));
				exit(1);
			}
		}
	VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_RETURN(-1);
	else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0) {
			if (WTERMSIG(status) != SIGSEGV) {
				in_rpt("Child process 3 exited due to signal %d, expected SIGSEGV", WTERMSIG(status));
				DBUG_RETURN(-1);
			}
		}
		else {
			in_rpt("Child process 3 exited abnormally with exit status %d, expected exit due to SIGSEGV", WEXITSTATUS(status));
			DBUG_RETURN(-1);
		}
	}

	/*Access first byte of next page*/
	VSRT_FORK(pid);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
			i = adr1[vsrt_ipgsz];
		if ((vsrt_got_sigsegv) || (vsrt_got_sigbus)) {
			in_rpt("Child 4: The length of the unmap was not rounded to the");
			in_rpt("nearest multiple of page size");
			in_rpt("At offset %d (page size) expected mapping but was unmapped", vsrt_pgsz);
			exit(1);
		}
	VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_RETURN(-1);
	else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0) {
			if (WTERMSIG(status) != SIGSEGV) {
				in_rpt("Child process 4 exited due to signal %d, expected SIGSEGV", WTERMSIG(status));
				DBUG_RETURN(-1);
			}
		}
		else {
			in_rpt("Child process 4 exited abnormally with exit status %d, expected exit due to SIGSEGV", WEXITSTATUS(status));
			DBUG_RETURN(-1);
		}
	}

	/*Unmap the rest of the file*/
	len = vsrt_pgsz;
	if (tb_munmap((void *)&adr1[vsrt_ipgsz], len) != 0) {
		err = errno;
		in_rpt("second munmap failed, errno = %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	/*Map two pages*/
	if ((adr1 = (int *)mmap(0, (size_t)(vsrt_pgsz*2), PROT_READ, MAP_SHARED, fd, 0)) == (int *)MAP_FAILED) {
		err = errno;
		in_rpt("second mmap failed, errno = %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}

	/*Unmap a page and a half*/
	len = vsrt_pgsz + vsrt_pgsz/2;
	if (tb_munmap((void *)adr1, len) != 0) {
		err = errno;
		in_rpt("third munmap failed, errno = %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	/*Access first byte of first page*/
	VSRT_FORK(pid);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
			i = *adr1;
		if (!vsrt_got_sigsegv) {
			if (vsrt_got_sigbus) {
				in_rpt("Child 5: Accessing unmapped page generated SIGBUS, expected SIGSEGV");
				exit(1);
			} else {
				in_rpt("Child 5: The length of the unmap was not rounded to the");
				in_rpt("nearest multiple of page size");
				in_rpt("At offset 0 expected no mapping but was mapped");
				exit(1);
			}
		}
	VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_RETURN(-1);
	else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0) {
			if (WTERMSIG(status) != SIGSEGV) {
				in_rpt("Child process 5 exited due to signal %d, expected SIGSEGV", WTERMSIG(status));
				DBUG_RETURN(-1);
			}
		}
		else {
			in_rpt("Child process 5 exited abnormally with exit status %d, expected exit due to SIGSEGV", WEXITSTATUS(status));
			DBUG_RETURN(-1);
		}
	}

	/*Access start of second half of second page*/
	VSRT_FORK(pid)
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
			i = adr1[vsrt_ipgsz+(vsrt_ipgsz/2)+1];
		if (!vsrt_got_sigsegv) {
			if (vsrt_got_sigbus) {
				in_rpt("Child 6: Accessing unmapped page generated SIGBUS, expected SIGSEGV");
				exit(1);
			} else {
				in_rpt("Child 6: The length of the unmap was not rounded to the");
				in_rpt("nearest multiple of page size");
				in_rpt("At offset %d expected no mapping but was mapped", vsrt_pgsz+(vsrt_pgsz/2)+1);
				exit(1);
			}
		}
	VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_RETURN(-1);
	else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0) {
			if (WTERMSIG(status) != SIGSEGV) {
				in_rpt("Child process 6 exited due to signal %d, expected SIGSEGV", WTERMSIG(status));
				DBUG_RETURN(-1);
			}
		}
		else {
			in_rpt("Child process 6 exited abnormally with exit status %d, expected exit due to SIGSEGV", WEXITSTATUS(status));
			DBUG_RETURN(-1);
		}
	}

	/*Access end of second half of second page*/
	VSRT_FORK(pid);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
			i = adr1[(2 * vsrt_ipgsz) - sizeof(int)];
		if (!vsrt_got_sigsegv) {
			if (vsrt_got_sigbus) {
				in_rpt("Child 7: Accessing unmapped page generated SIGBUS, expected SIGSEGV");
				exit(1);
			} else {
				in_rpt("Child 7: The length of the unmap was not rounded to the");
				in_rpt("nearest multiple of page size");
				in_rpt("At offset %d expected no mapping but was mapped", 2*vsrt_pgsz-sizeof(int));
				exit(1);
			}
		}
	VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_RETURN(-1);
	else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0) {
			if (WTERMSIG(status) != SIGSEGV) {
				in_rpt("Child process 7 exited due to signal %d, expected SIGSEGV", WTERMSIG(status));
				DBUG_RETURN(-1);
			}
		}
		else {
			in_rpt("Child process 7 exited abnormally with exit status %d, expected exit due to SIGSEGV", WEXITSTATUS(status));
			DBUG_RETURN(-1);
		}
	}

	(void)close(fd);
	(void)unlink(test_file);

	DBUG_RETURN(0);
}

/*
If   _POSIX_MAPPED_FILES    
is defined or the imple-
mentation supports the munmap() function as defined  in
System Interfaces and Headers, Issue 5.
	References to unmapped pages after a successful  call
	to  munmap() and before any subsequent mapping of the
	unmapped pages shall result in the delivery of a SIGSEGV
	signal to the process.
*/

private int test3p1();
private int test3p2();
private int dotest3();

private void test3()
{
	int	ret1, ret2;

	DBUG_ENTER("test3");

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	ret1 = test3p1();
	ret2 = test3p2();

	/*any error is a failure*/
	if ((ret1 == -1) || (ret2 == -1)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	/*if both parts cannot be run its unsupported*/
	if ((ret1 == 1) && (ret2 == 1)) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	/*if either part is run successfully its a pass*/
	xs_rpt();

	DBUG_VOID_RETURN
}

/*test for files*/
private int test3p1()
{
	int	fd, ret;

	DBUG_ENTER("test3p1");

#ifndef _POSIX_MAPPED_FILES
	if (!supports_munmap) {
		in_rpt("munmap not supported");
		DBUG_RETURN(1);
	}
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		DBUG_RETURN(1);
	}
#endif

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_RETURN(-1);

	ret = dotest3(fd);

	(void)close(fd);
	(void)unlink(test_file);

	DBUG_RETURN(ret);
}

/*test for shared memory*/
private int test3p2()
{
	int	fd, ret;

	DBUG_ENTER("test3p2");

	in_rpt("shared memory not required");
	DBUG_RETURN(1);
}

private int dotest3(fd)
int fd;
{
	int	err;
	int	pathok = 0;
	volatile int	i;
	pid_t	pid;
	int 	status;
	int	*addr;

	DBUG_ENTER("dotest3");

	if ((addr = (int *)mmap(0, (size_t)(vsrt_pgsz*2), PROT_READ, MAP_SHARED, fd, 0)) == (int *)MAP_FAILED) {
		err = errno;
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	/*SIGSEGV when reference is made after unmap*/
	if (tb_munmap((void *)addr, (size_t)vsrt_pgsz*2) != 0) {
		err = errno;
		in_rpt("munmap failed, errno = %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	VSRT_FORK(pid);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
			i = *addr;
		if (!vsrt_got_sigsegv) {
			if (vsrt_got_sigbus) {
				in_rpt("Accessing unmapped page generated SIGBUS, expected SIGSEGV");
				exit(1);
			} else {
				in_rpt("Accessing unmapped page did not generated SIGSEGV");
				exit(1);
			}
		}
	VSRT_KROF2(pid, status);
	if (status == -1)
		DBUG_RETURN(-1);
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0) {
			if (WTERMSIG(status) != SIGSEGV) {
				in_rpt("Child process 1 exited due to signal %d, expected SIGSEGV", WTERMSIG(status));
				DBUG_RETURN(-1);
			}
		}
		else {
			in_rpt("Child process exited abnormally with exit status %d, expected exit due to SIGSEGV", WEXITSTATUS(status));
			DBUG_RETURN(-1);
		}
	}

	DBUG_RETURN(0);
}


/*
If   _POSIX_MAPPED_FILES  
is defined or the imple-
mentation supports the munmap() function as defined  in
System Interfaces and Headers, Issue 5.
	A call  to  munmap()  when  addresses  in  the  range
	[addr,addr+len]  are  outside the valid range for the
	address space of the calling process shall return  -1
	and set errno to EINVAL.
*/

private int test4p1();
private int test4p2();
private int dotest4();

private void test4()
{
	int	ret1, ret2;

	DBUG_ENTER("test4");

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	ret1 = test4p1();
	ret2 = test4p2();

	/*any error is a failure*/
	if ((ret1 == -1) || (ret2 == -1)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	/*if both parts cannot be run its unsupported*/
	if ((ret1 == 1) && (ret2 == 1)) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	/*if either part is run successfully its a pass*/
	xs_rpt();

	DBUG_VOID_RETURN
}

/*test for files*/
private int test4p1()
{
	int	fd, ret;

	DBUG_ENTER("test4p1");

#ifndef _POSIX_MAPPED_FILES
	if (!supports_munmap) {
		in_rpt("munmap not supported");
		DBUG_RETURN(1);
	}
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		DBUG_RETURN(1);
	}
#endif

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_RETURN(-1);

	ret = dotest4(fd);

	(void)close(fd);
	(void)unlink(test_file);

	DBUG_RETURN(ret);
}

/*test for shared memory*/
private int test4p2()
{
	int	fd, ret;

	DBUG_ENTER("test4p2");

	in_rpt("shared memory not required");
	DBUG_RETURN(1);
}

private int dotest4(fd)
int fd;
{
	int	err;
	int	pathok = 0;
	int	status;
	void	*addr, *addr1;
	char	*str;
	long	limit;
	pid_t	pid1;

	DBUG_ENTER("dotest4");

	str = tet_getvar("VSRT_ADDR_SPACE_PAGES");
	if ((str == NULL) || (strlen(str) == 0)) {
		in_rpt("No VSRT_ADDR_SPACE_PAGES value specified in tet_exec.cfg");
		DBUG_RETURN(-1);
	}

	limit = atol(str);

	/*use up some of the address space*/
	if ((addr1 = mmap(0, (size_t)(vsrt_pgsz*(limit/2)), PROT_READ, MAP_SHARED, fd, 0)) == (int *)MAP_FAILED) {
		err = errno;
		in_rpt("first mmap failed, errno = %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if ((addr = mmap(0, (size_t)(vsrt_pgsz*2), PROT_READ, MAP_SHARED, fd, 0)) == (int *)MAP_FAILED) {
		err = errno;
		in_rpt("second mmap failed, errno = %d (%s)", err, errname(err));
		(void)munmap(addr1, (size_t)(vsrt_pgsz*(limit/2)));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	
VSRT_FORK(pid1);
	/*allow for max size to overflow if we used it all by using limit-1*/
	if (tb_munmap(addr, (size_t)vsrt_pgsz*(limit-1)) != -1) {
		in_rpt("munmap did not fail");
		(void)munmap(addr1, (size_t)(vsrt_pgsz*(limit/2)));
		exit(-1);
	}
	if (errno != EINVAL) {
		err = errno;
		in_rpt("munmap failed but  errno = %d (%s)", err, errname(err));
		(void)munmap(addr1, (size_t)(vsrt_pgsz*(limit/2)));
		exit(-1);
	}
VSRT_KROF2(pid1, status);
	(void)munmap(addr, (size_t)(vsrt_pgsz*2));
	(void)munmap(addr1, (size_t)(vsrt_pgsz*(limit/2)));
	
	if (status != 0) {
		if (status != -1)
			in_rpt("Child process terminated before test could be completed");
		DBUG_RETURN(-1);
	}
	else
		DBUG_RETURN(0);
} 


/*
If   at   least   one   of    _POSIX_MAPPED_FILES    or
_POSIX_SHARED_MEMORY_OBJECTS  is  defined or the imple-
mentation supports the munmap() function as defined  in
System Interfaces and Headers, Issue 5.
	A call  to  munmap()  when  len is 0
	shall return  -1 and set errno to EINVAL.
*/

private int test5p1();
private int test5p2();
private int dotest5();

private void test5()
{
	int	ret1, ret2;

	DBUG_ENTER("test5");

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	ret1 = test5p1();
	ret2 = test5p2();

	/*any error is a failure*/
	if ((ret1 == -1) || (ret2 == -1)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	/*if both parts cannot be run its unsupported*/
	if ((ret1 == 1) && (ret2 == 1)) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	/*if either part is run successfully its a pass*/
	xs_rpt();

	DBUG_VOID_RETURN
}

/*test for files*/
private int test5p1()
{
	int	fd, ret;

	DBUG_ENTER("test5p1");

#ifndef _POSIX_MAPPED_FILES
	if (!supports_munmap) {
		in_rpt("munmap not supported");
		DBUG_RETURN(1);
	}
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		DBUG_RETURN(1);
	}
#endif

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_RETURN(-1);

	ret = dotest5(fd);

	(void)close(fd);
	(void)unlink(test_file);

	DBUG_RETURN(ret);
}

/*test for shared memory*/
private int test5p2()
{
	int	fd, ret;

	DBUG_ENTER("test5p2");

	in_rpt("shared memory not required");
	DBUG_RETURN(1);
}

private int dotest5(fd)
int fd;
{
	int	err;
	int	pathok = 0;
	int	*addr;

	DBUG_ENTER("dotest5");


	if ((addr = (int *)mmap(0, (size_t)(vsrt_pgsz*2), PROT_READ, MAP_SHARED, fd, 0)) == (int *)MAP_FAILED) {
		err = errno;
		in_rpt("mmap failed, errno = %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tb_munmap((void *)addr, 0) != -1) {
		in_rpt("munmap did not fail");
		DBUG_RETURN(-1);
	}
	if (errno != EINVAL) {
		err = errno;
		in_rpt("munmap failed but errno = %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}

	DBUG_RETURN(0);
} 

/*
If _POSIX_MAPPED_FILES is not
defined and the imple-
mentation does not support  the  munmap()  function  as
defined in System Interfaces and Headers, Issue 5.
	A call to munmap() shall return -1 and set  errno  to ENOSYS.
*/

private void test6()
{
#ifndef _POSIX_MAPPED_FILES
	int	err;
#endif

	DBUG_ENTER("test6");

#ifdef _POSIX_MAPPED_FILES
		in_rpt("Implementation supports mapped files");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
#else
#ifdef _POSIX_SHARED_MEMORY_OBJECTS
		in_rpt("Implementation supports shared memory objects");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
#else
	if (supports_munmap) {
		in_rpt("Implementation supports munmap");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}

	if (tb_munmap((void *)0, (size_t)(3*vsrt_pgsz)) != -1) {
		xx_rpt(FAILURE);
		in_rpt("munmap succeeded");
		DBUG_VOID_RETURN 
	}
	if (errno != ENOSYS) {
		err = errno;
		in_rpt("munmap failed but errno = %d (%s)", err, errname(err));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

	xs_rpt();
	DBUG_VOID_RETURN 
#endif
#endif
}

private	int tb_munmap(addr, len)
void	*addr;
size_t	len;
{
	int 	retval;

	retval = munmap(addr, len);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
