/*$Header: /cvsroot/lsb/test_sets/lsb-os/tset/LSB.os/mprotect/mprotect_P/mprotect_P.c,v 1.2 2001/10/02 13:25:13 ajosey Exp $*/
/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/mprotect/mprotect_P/mprotect_P.c
*
* Modifications:
* $Log: mprotect_P.c,v $
* Revision 1.2  2001/10/02 13:25:13  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:17  cyeoh
* Adds files for lsb-os test suite
*
*/

#include	<vsrt/vsrt.h>
#include	<sys/mman.h>
#include	<setjmp.h>

#define 	NO_TESTS	11	/* number of tests in testset */

#ifdef UNDEF_MACROS
#undef mprotect
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();

int	no_tests = NO_TESTS;

private	int	tb_mprotect();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();

int	supports_mprotect = 0;
int	supports_mmap = 0;

#define	test_file	"vsrt_mprotect"

/*from vsrtlib/mkmmap.c*/
extern int	vsrt_got_sigsegv;
extern int	vsrt_got_sigbus;
extern int	vsrt_got_sig;
extern unsigned long	vsrt_pgsz;
extern unsigned long	vsrt_ipgsz;
extern int	vsrt_setjmp_called;
extern sigjmp_buf	vsrt_env;

struct {
	int     prot;
	char    *prot_string;
} protlist[] = {
	{ 0, "No unsupported access(es)" },
	{ (PROT_EXEC), "PROT_EXEC" },
	{ (PROT_EXEC|PROT_WRITE), "PROT_EXEC|PROT_WRITE" },
	{ (PROT_EXEC|PROT_READ), "PROT_EXEC|PROT_READ" },
};


void atstart(void);
void atstart(void)
{
	vsrt_check_support("MPROTECT", &supports_mprotect);
	vsrt_check_support("MMAP", &supports_mmap);
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = NULL;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	NULL, 0
};

/*
If _POSIX_MEMORY_PROTECTION is defined or the implemen-
tation supports the mprotect() function as described in
System Interfaces and Headers, Issue 5:
	A successful call to mprotect()  when  prot  has  the
	PROT_READ  flag  set  shall change the access protec-
	tions for the  range  [addr,addr+len]  to  allow  the
	page(s) to be read and return 0.
*/

private void test1()
{
	int	errors = 0;
	int	fd;
	char	*adr;
	char	c;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test1");

#ifndef _POSIX_MEMORY_PROTECTION
	if (!supports_mprotect) {
		in_rpt("mprotect not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 3);
	if (fd == -1)
		DBUG_VOID_RETURN

	if ((adr = (char *)vsrt_map3(fd, PROT_WRITE, MAP_SHARED, 
	    PROT_WRITE, MAP_SHARED, PROT_WRITE, MAP_SHARED)) == 
	    (char *)-1)
		DBUG_VOID_RETURN

VSRT_FORK(pid);
	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0) {
		*adr = 'x';
		*(adr+vsrt_pgsz) = 'y';
		*(adr+2*vsrt_pgsz-1) = 'z';
	}

	if (tb_mprotect((void *)adr, (size_t)(vsrt_pgsz*3), PROT_READ) != 0) {
		vsrt_sysfail("mprotect");
		exit(1);
	}

	/*Pages may now be read*/
	vsrt_got_sigsegv = vsrt_got_sigbus = 0;

	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0)
		c = *adr;

	if (c != 'x') {
		in_rpt("Invalid value read from the mapped location");
		in_rpt("Expected: %c  Received: %c", 'x', c);
		xx_rpt(FAILURE);
		exit(1);
	}

	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0)
		c = *(adr+vsrt_pgsz);
	if (c != 'y') {
		in_rpt("Invalid value read from the mapped location");
		in_rpt("Expected: %c  Received: %c", 'y', c);
		xx_rpt(FAILURE);
		exit(1);
	}

	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0)
		c = *(adr+2*vsrt_pgsz-1);
	if (c != 'z') {
		in_rpt("Invalid value read from the mapped location");
		in_rpt("Expected: %c  Received: %c", 'z', c);
		xx_rpt(FAILURE);
		exit(1);
	}

	sleep(WAIT_FOR_TIME);

	if (vsrt_got_sigsegv || vsrt_got_sigbus) {
		in_rpt("Page was not readable");
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)munmap((void*)adr,(size_t)(3*vsrt_pgsz));
	(void)close(fd);
	(void)unlink(test_file);

	if (errors == 0)
		xs_rpt();
	DBUG_VOID_RETURN 
} 


/*
If _POSIX_MEMORY_PROTECTION is defined or the implemen-
tation supports the mprotect() function as described in
System Interfaces and Headers, Issue 5:
	A successful call to mprotect()  when  prot  has  the
	PROT_WRITE  flag  set shall change the access protec-
	tions for the  range  [addr,addr+len]  to  allow  the
	page(s) to be written and return 0.
*/

private void test2()
{
	int	errors = 0;
	int	fd;
	char	*adr;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test2");

#ifndef _POSIX_MEMORY_PROTECTION
	if (!supports_mprotect) {
		in_rpt("mprotect not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif


	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 3);
	if (fd == -1)
		DBUG_VOID_RETURN

	if ((adr = (char *)vsrt_map3(fd, PROT_READ, MAP_SHARED, PROT_READ, 
			   MAP_SHARED, PROT_READ, MAP_SHARED)) == (char *)-1)
		DBUG_VOID_RETURN

	if (tb_mprotect((void*)adr, (size_t)vsrt_pgsz*3, PROT_WRITE) != 0) {
		vsrt_sysfail("mprotect");
		DBUG_VOID_RETURN
	}

VSRT_FORK(pid);
	/*Pages may now be written*/
	vsrt_got_sigsegv = vsrt_got_sigbus = 0;

	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0) {
		*adr = 'd';
		*(adr+vsrt_pgsz-1) = 'u';
		*(adr+2*vsrt_pgsz-1) = 'g';
	}

	if (*adr != 'd') {
		in_rpt("Invalid value written to the mapped location");
		in_rpt("Expected: %c  Received: %c", 'd', *adr);
		xx_rpt(FAILURE);
		exit(1);
	}
	if (*(adr+vsrt_pgsz-1) != 'u') {
		in_rpt("Invalid value written to the mapped location");
		in_rpt("Expected: %c  Received: %c", 'u', *(adr+vsrt_pgsz-1));
		xx_rpt(FAILURE);
		exit(1);
	}
	if (*(adr+2*vsrt_pgsz-1) != 'g') {
		in_rpt("Invalid value written to the mapped location");
		in_rpt("Expected: %c  Received: %c", 'g', *(adr+2*vsrt_pgsz-1));
		xx_rpt(FAILURE);
		exit(1);
	}

	sleep(WAIT_FOR_TIME);

	if (vsrt_got_sigsegv || vsrt_got_sigbus) {
		in_rpt("Page was not writeable");
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)munmap((void*)adr, (size_t)(3*vsrt_pgsz));

	(void)close(fd);
	(void)unlink(test_file);

	if (errors == 0)
		xs_rpt();
	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MEMORY_PROTECTION is defined or the implemen-
tation supports the mprotect() function as described in
System Interfaces and Headers, Issue 5:
	A successful call to mprotect()  when  prot  has  the
	PROT_READ  and  PROT_WRITE flags set shall change the
	access protections for the range  [addr,addr+len]  to
	allow  the  page(s) to be read and written and return 0.
*/

private void test3()
{
	int	errors = 0;
	int	*adr1;
	int	fd, i, j, ibufsz;
	int	buf[512];
	pid_t	pid;
	int	status;

	DBUG_ENTER("test3");

#ifndef _POSIX_MEMORY_PROTECTION
	if (!supports_mprotect) {
		in_rpt("mprotect not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN

	ibufsz = sizeof(buf)/sizeof(buf[0]);

	if ((adr1 = (int *)mmap(0, (size_t)vsrt_pgsz, PROT_NONE, MAP_SHARED, fd, (off_t)vsrt_pgsz)) == (int *)MAP_FAILED) {
		vsrt_sysfail("mmap");
		DBUG_VOID_RETURN
	}

	if (tb_mprotect((void*)adr1, (size_t)vsrt_pgsz, PROT_READ|PROT_WRITE) != 0) {
		vsrt_sysfail("mprotect");
		DBUG_VOID_RETURN
	}

VSRT_FORK(pid);
	vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

	i = 0;
	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0) {
		for (j = 0; !vsrt_got_sig && j < vsrt_ipgsz/2; j++) {
			i = adr1[j];
			if (i != (j + vsrt_ipgsz)) {
				in_rpt("ERROR: Mapped page has invalid contents");
	
				in_rpt("At offset %d  Expected: %ld  Received: %d", j, (j + vsrt_ipgsz), i);
				xx_rpt(FAILURE);
				exit(1);
			}
		}
	}

	vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;

	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0) {
		for (j = 0; !vsrt_got_sig && j < vsrt_ipgsz; j++)
			adr1[j] = 2;

		if (msync((void *)adr1, (size_t)vsrt_pgsz, MS_SYNC) == -1) {
			vsrt_sysfail("msync");
			exit(1);
		}

	}

	sleep(WAIT_FOR_TIME);

	for (j = 0; !vsrt_got_sig && j < vsrt_ipgsz; j++) {
		if (adr1[j] != 2) {
			in_rpt("Invalid value written to the mapped location");
			in_rpt("Expected: %d  Received: %d", 2, adr1[i]);
			xx_rpt(FAILURE);
			exit(1);
		}
	}

	if (vsrt_got_sigsegv || vsrt_got_sigbus) {
		in_rpt("Page was not writeable");
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)munmap((void*)adr1, (size_t)(3*vsrt_pgsz));

	close(fd);
	unlink(test_file);

	if (errors == 0)
		xs_rpt();
	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MEMORY_PROTECTION is defined or the implemen-
tation supports the mprotect() function as described in
System Interfaces and Headers, Issue 5 and  the  imple-
mentation supports use of PROT_EXEC:
	A successful call to mprotect()  when  prot  has  the
	PROT_EXEC  flag  set  shall change the access protec-
	tions for the  range  [addr,addr+len]  to  allow  the
	page(s) to be executed and return 0.
*/

private void test4()
{
	int	errors = 0;
	char	*adr;
	char	*(*foo)();
	int	ret;
	struct vsrt_exec_resource get;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test4");

#ifndef _POSIX_MEMORY_PROTECTION
	if (!supports_mprotect) {
		in_rpt("mprotect not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	if ((ret = vsrt_create_exec_file(test_file, O_RDWR, VSRT_PROT_ALL, &get)) != 0)
		DBUG_VOID_RETURN

	/*Map a page that is writeable*/
	if ((adr = (char *)mmap((void*)0, (size_t)3*vsrt_pgsz, PROT_WRITE,MAP_SHARED,get.fd, 0)) == (char *)MAP_FAILED) {
		vsrt_sysfail("mmap");
		DBUG_VOID_RETURN
	}

	(void)close(get.fd);

	if (tb_mprotect((void*)adr, (size_t)(3*vsrt_pgsz), PROT_EXEC) != 0) {
		if (errno == ENOTSUP) {
                	in_rpt("INFO: Implementation does not support use of PROT_EXEC");
                	xx_rpt(TET_UNSUPPORTED);
                	DBUG_VOID_RETURN
        	}
		vsrt_sysfail("mprotect");
		DBUG_VOID_RETURN
	}

VSRT_FORK(pid);
	/*Pages may now be executed*/
	vsrt_got_sigsegv = vsrt_got_sigbus = 0;

	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0) {
		foo = (char *(*)())(adr+get.offset);
		ret = (int)(*foo)();
	}

	sleep(WAIT_FOR_TIME);

	if (vsrt_got_sigsegv || vsrt_got_sigbus) {
		in_rpt("Page was not executable");
		xx_rpt(FAILURE);
		exit(1);
	}

	if (ret != 99) {
		in_rpt("Unexpected function return");
		in_rpt("Expected :99  Received: %d", ret);
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	munmap((void *)adr, (size_t)vsrt_pgsz);

	unlink(test_file);

	if (errors == 0)
		xs_rpt();

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MEMORY_PROTECTION is defined or the implemen-
tation supports the mprotect() function as described in
System Interfaces and Headers, Issue 5:
	A successful call to mprotect()  when  prot  has  the
	PROT_NONE   flag   set   shall   change   the  access
	protections for  the  range  [addr,addr+len]  to  not
	allow the page(s) to be accessed and return 0.
*/

private void test5()
{
	int	errors = 0;
	int	fd;
	char	*adr;
	int	d, c;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test5");

#ifndef _POSIX_MEMORY_PROTECTION
	if (!supports_mprotect) {
		in_rpt("mprotect not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 3);
	if (fd == -1)
		DBUG_VOID_RETURN

	if ((adr = (char *)vsrt_map3(fd, PROT_READ, MAP_SHARED, PROT_READ, MAP_SHARED, PROT_READ, MAP_SHARED)) == (char *)-1)
		DBUG_VOID_RETURN

	if (tb_mprotect((void*)adr, (size_t)(vsrt_pgsz*3), PROT_NONE) != 0) {
		vsrt_sysfail("mprotect");
		DBUG_VOID_RETURN
	}

VSRT_FORK(pid);
	/*Pages may not be read*/
	vsrt_got_sigsegv = vsrt_got_sigbus = 0;

	c = 'X';
	d = 'Y';

	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0) {
		c = *adr;
		d = *(adr+1);
	}

	if ((c == 0) || (d == 1)) {
		in_rpt("The mapped page could be read");
		xx_rpt(FAILURE);
		exit(1);
	}

	sleep(WAIT_FOR_TIME);

	if (!vsrt_got_sigsegv && !vsrt_got_sigbus) {
		in_rpt("Page was still readable");
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

VSRT_FORK(pid);
	/*Pages may not be written*/
	vsrt_got_sigsegv = vsrt_got_sigbus = 0;

	vsrt_setjmp_called = 1;
	if (sigsetjmp(vsrt_env,1) == 0)
		*adr = 'x';

	sleep(WAIT_FOR_TIME);

	if (!vsrt_got_sigsegv && !vsrt_got_sigbus) {
		in_rpt("Page was still writeable");
		xx_rpt(FAILURE);
		exit(1);
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	(void)munmap((void*)adr, (size_t)(3*vsrt_pgsz));
	(void)close(fd);
	(void)unlink(test_file);
	if (errors == 0)
		xs_rpt();
	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MEMORY_PROTECTION is defined or the implemen-
tation supports the mprotect() function as described in
System Interfaces and Headers, Issue 5:
	A successful call to int mprotect(void *addr,  size_t
	len,  int  prot) shall round len to the next multiple
	of the page size.
*/

private void test6()
{
	int	errors = 0;
	int	i, fd;
	char	*adr;
	pid_t	pid;
	int	status;

	DBUG_ENTER("test6");

#ifndef _POSIX_MEMORY_PROTECTION
	if (!supports_mprotect) {
		in_rpt("mprotect not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 3);
	if (fd == -1)
		DBUG_VOID_RETURN

	if ((adr = (char *)vsrt_map3(fd, PROT_READ, MAP_SHARED, PROT_READ, 
			   MAP_SHARED, PROT_READ, MAP_SHARED)) == (char *)-1)
		DBUG_VOID_RETURN

	/*Call mprotect() with len not in multiple of page size*/
	if (tb_mprotect((void*)adr, (size_t)(vsrt_pgsz/2), PROT_WRITE) != 0) {
		vsrt_sysfail("mprotect");
		DBUG_VOID_RETURN
	}

VSRT_FORK(pid);
	vsrt_got_sigsegv = vsrt_got_sigbus = 0;

	for (i = (vsrt_pgsz/2); i < vsrt_pgsz; i++)
		*(adr+i-1) = 'S';

	sleep(WAIT_FOR_TIME);
	if (vsrt_got_sigsegv || vsrt_got_sigbus) {
		in_rpt("Page was not writeable");
		xx_rpt(FAILURE);
		exit(1);
	}

	for (i = (vsrt_pgsz/2); i < vsrt_pgsz; i++) {
		if (*(adr+i-1) != 'S') {
			in_rpt("mprotect() did not round to page size");
			xx_rpt(FAILURE);
			exit(1);
		}
	}
VSRT_KROF2(pid, status);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);


	(void)munmap((void*)adr, (size_t)(3*vsrt_pgsz));
	(void)close(fd);
	(void)unlink(test_file);

	if (errors == 0)
		xs_rpt();

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MEMORY_PROTECTION is defined or the implemen-
tation supports the mprotect() function as described in
System Interfaces and Headers, Issue 5:
	A call to mprotect() when the memory object  was  not
	opened  for  write and PROT_WRITE was specified for a
	MAP_SHARED type mapping shall return -1 and set errno
	to EACCES.
*/

private void test7()
{
	int	err;
	int	errors = 0;
	int	fd;
	char	*adr;

	DBUG_ENTER("test7");

#ifndef _POSIX_MEMORY_PROTECTION
	if (!supports_mprotect) {
		in_rpt("mprotect not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	fd = vsrt_open_file(test_file, O_RDONLY, VSRT_PROT_ALL, 3);
	if (fd == -1)
		DBUG_VOID_RETURN

	if ((adr = (char *)vsrt_map3(fd, PROT_NONE, MAP_SHARED, PROT_NONE, MAP_SHARED, PROT_NONE, MAP_SHARED)) == (char *)-1)
		DBUG_VOID_RETURN

	if (mprotect((void*)adr, (size_t)(vsrt_pgsz*3), PROT_WRITE) != -1) {
		in_rpt("Call to mprotect() succeeded");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (errno != EACCES) {
		err = errno;
		in_rpt("mprotect failed, but errno = %d (%s)", err, errname(err));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	(void)munmap((void*)adr, (size_t)(3*vsrt_pgsz));
	(void)close(fd);
	(void)unlink(test_file);

	if (errors == 0)
		xs_rpt();

	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MEMORY_PROTECTION is defined or the implemen-
tation supports the mprotect() function as described in
System Interfaces and Headers, Issue 5:
	A call to mprotect()  when  addresses  in  the  range
	[addr,addr+len] are outside the range allowed for the
	address space of a process shall return  -1  and  set
	errno to ENOMEM.
*/

private void test8()
{
	int	fd;
	int	errors = 0;
	char		*adr;
	unsigned long	limit;
	char		*ascii_limit;

	DBUG_ENTER("test8");

#ifndef _POSIX_MEMORY_PROTECTION
	if (!supports_mprotect) {
		in_rpt("mprotect not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif


	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 3);
	if (fd == -1)
		DBUG_VOID_RETURN

	if ((adr = (char *)vsrt_map3(fd, PROT_NONE, MAP_SHARED, PROT_NONE, 
			   MAP_SHARED, PROT_NONE, MAP_SHARED)) == (char *)-1)
		DBUG_VOID_RETURN

	if((ascii_limit = tet_getvar("VSRT_ADDR_SPACE_PAGES")) == NULL) {
		in_rpt("VSRT_ADDR_SPACE_PAGES not defined in tet_exec.cfg");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	errno = 0;
	limit = strtoul(ascii_limit, (char**) NULL, 10);
	if (errno == ERANGE) {
		in_rpt("VSU_MAX_PROC_ADDR_SPACE_SIZE out of range");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	if (mprotect((void*)adr, limit*vsrt_pgsz,PROT_READ) != -1) {
		in_rpt("mprotect() succeeded for PROT_READ");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (errno != ENOMEM) {
		vsrt_sysfail("mprotect for PROT_READ");
		DBUG_VOID_RETURN
	}

	if (mprotect((void*)adr, limit*vsrt_pgsz,PROT_NONE) != -1) {
		in_rpt("mprotect() succeeded for PROT_NONE");
		xx_rpt(FAILURE);
		errors++;
	} else if (errno != ENOMEM) {
		vsrt_sysfail("mprotect for PROT_NONE");
		errors++;
	}

	if (mprotect((void*)adr, limit*vsrt_pgsz,PROT_WRITE) != -1) {
		in_rpt("mprotect() succeeded for PROT_WRITE");
		xx_rpt(FAILURE);
		errors++;
	} else if (errno != ENOMEM) {
		vsrt_sysfail("mprotect for PROT_WRITE");
		errors++;
	}

	if (mprotect((void*)adr, limit*vsrt_pgsz, PROT_EXEC) != -1) {
		in_rpt("mprotect() succeeded for PROT_EXEC");
		xx_rpt(FAILURE);
		errors++;
	} else  {
		if (errno == ENOTSUP)
                	in_rpt("INFO: Implementation does not support use of PROT_EXEC");
		else if (errno != ENOMEM) {
			vsrt_sysfail("mprotect for PROT_EXEC");
			errors++;
		}
	}

	(void)munmap((void*)adr, (size_t)(3*vsrt_pgsz));
	(void)close(fd);
	(void)unlink(test_file);

	if (errors == 0)
		xs_rpt();
	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MEMORY_PROTECTION is defined or the implemen-
tation supports the mprotect() function as described in
System Interfaces and Headers, Issue 5:
	A call to mprotect()  when  addresses  in  the  range
	[addr,addr+len]  specify  one  or more pages that are
	not mapped shall return -1 and set errno to ENOMEM.
*/

private void test9()
{
	int	fd;
	int	errors = 0;
	char	*adr;

	DBUG_ENTER("test9");

#ifndef _POSIX_MEMORY_PROTECTION
	if (!supports_mprotect) {
		in_rpt("mprotect not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 3);
	if (fd == -1)
		DBUG_VOID_RETURN

	if ((adr = (char *)vsrt_map3(fd, PROT_NONE, MAP_SHARED,
	     PROT_NONE, MAP_SHARED, PROT_NONE, MAP_SHARED)) == (char *)-1)
		DBUG_VOID_RETURN

	/*Unmap middle page of the three*/
	if (munmap((void*)(adr+vsrt_pgsz), (size_t)vsrt_pgsz) == -1) {
		vsrt_sysfail("munmap");
		DBUG_VOID_RETURN
	}

	if (mprotect((void*)adr, (size_t)(vsrt_pgsz*3),PROT_READ) != -1) {
		in_rpt("mprotect() succeeded for PROT_READ for 3 pages");
		xx_rpt(FAILURE);
		errors++;
	}
	if (errno != ENOMEM) {
		vsrt_sysfail("mprotect for PROT_READ for 3 pages");
		errors++;
	}
	if (mprotect((void*)adr, (size_t)(vsrt_pgsz+1), PROT_READ) != -1) {
		in_rpt("mprotect() succeeded for PROT_READ for 1 page");
		xx_rpt(FAILURE);
		errors++;
	}
	if (errno != ENOMEM) {
		vsrt_sysfail("mprotect for PROT_READ for 1 page");
		errors++;
	}

	if (mprotect((void*)adr, (size_t)(vsrt_pgsz+1), PROT_NONE) != -1) {
		in_rpt("mprotect() succeeded for PROT_NONE for 1 page");
		xx_rpt(FAILURE);
		errors++;
	}
	if (errno != ENOMEM) {
		vsrt_sysfail("mprotect for PROT_NONE for 1 page");
		errors++;
	}
	if (mprotect((void*)adr, (size_t)(vsrt_pgsz*3), PROT_NONE) != -1) {
		in_rpt("mprotect() succeeded for PROT_NONE for 3 pages");
		xx_rpt(FAILURE);
		errors++;
	}
	if (errno != ENOMEM) {
		vsrt_sysfail("mprotect for PROT_NONE for 3 pages");
		errors++;
	}

	if (mprotect((void*)adr, (size_t)(vsrt_pgsz+1), PROT_WRITE) != -1) {
		in_rpt("mprotect() succeeded for PROT_WRITE for 1 page");
		xx_rpt(FAILURE);
		errors++;
	}
	if (errno != ENOMEM) {
		vsrt_sysfail("mprotect for PROT_WRITE for 1 page");
		errors++;
	}
	if (mprotect((void*)adr, (size_t)(vsrt_pgsz*3), PROT_WRITE) != -1) {
		in_rpt("mprotect() succeeded for PROT_WRITE for 3 pages");
		xx_rpt(FAILURE);
		errors++;
	}
	if (errno != ENOMEM) {
		vsrt_sysfail("mprotect for PROT_WRITE for 3 pages");
		errors++;
	}

	if (mprotect((void*)adr, (size_t)(vsrt_pgsz+1), PROT_EXEC) != -1) {
		in_rpt("mprotect() succeeded for PROT_EXEC for 1 page");
		xx_rpt(FAILURE);
		errors++;
	} else {
		if (errno == ENOTSUP)
                	in_rpt("Implementation does not support use of PROT_EXEC");
		else if (errno != ENOMEM) {
			vsrt_sysfail("mprotect for PROT_EXEC for 1 page");
			errors++;
		}
	}
	if (mprotect((void*)adr, (size_t)(vsrt_pgsz*3), PROT_EXEC) != -1) {
		in_rpt("mprotect() succeeded for PROT_EXEC for 3 pages");
		xx_rpt(FAILURE);
		errors++;
	} else {
		if (errno == ENOTSUP)
                	in_rpt("Implementation does not support use of PROT_EXEC");
		else if (errno != ENOMEM) {
			vsrt_sysfail("mprotect for PROT_EXEC for 3 pages");
			errors++;
		}
	}

	(void)munmap((void*)adr, (size_t)(3*vsrt_pgsz));

	(void)close(fd);

	(void)unlink(test_file);
	if (errors == 0)
		xs_rpt();
	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MEMORY_PROTECTION is defined or the implemen-
tation supports the mprotect() function as described in
System Interfaces and Headers, Issue 5 and  the  imple-
mentation  does  not support some of the possibly valid
values for prot:
	A call to mprotect() when the implementation does not
	support the combination of accesses requested in prot
	shall return -1 and set errno to ENOTSUP.
*/

private void test10()
{
	int	errors = 0;
	int	prot, unsupported_prot;
	char	*adr;
	struct vsrt_exec_resource get;
	char	*str;

	DBUG_ENTER("test10");

#ifndef _POSIX_MEMORY_PROTECTION
	if (!supports_mprotect) {
		in_rpt("mprotect not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif
#ifndef _POSIX_MAPPED_FILES
	if (!supports_mmap) {
		in_rpt("mmap not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}
#endif

	str = tet_getvar("VSRT_MMAP_UNSUPPORTED_PROT");
	if (str == NULL) {
		in_rpt("VSRT_MMAP_UNSUPPORTED_PROT not defined in tet_exec.cfg");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	unsupported_prot = atoi(str);
	if ((unsupported_prot < 0) || (unsupported_prot > 3)) {
		in_rpt("VSRT_MMAP_UNSUPPORTED_PROT must be a value between 0 and 3");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}

	if (unsupported_prot == 0) {
		in_rpt("Implementation has no unsupported access(es)");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	if (vsrt_create_exec_file(test_file, O_RDWR, VSRT_PROT_ALL, &get) != 0)
		DBUG_VOID_RETURN

	if ((adr = (char *)mmap((void*)0, (size_t)3*vsrt_pgsz, PROT_NONE,MAP_SHARED,get.fd, 0)) == (char *)MAP_FAILED) {
		vsrt_sysfail("mmap");
		DBUG_VOID_RETURN
	}

	close(get.fd);

	prot = protlist[unsupported_prot].prot;
	errno = 0;
	if (mprotect((void*)adr, (size_t)(3*vsrt_pgsz), prot) != -1) {
		in_rpt("ERROR: mprotect succeeded");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (errno != ENOTSUP) {
		vsrt_sysfail("mprotect");
		DBUG_VOID_RETURN
	}

	unlink(test_file);

	if (errors == 0)
		xs_rpt();
	DBUG_VOID_RETURN 
} 

/*
If _POSIX_MEMORY_PROTECTION  is  not  defined  and  the
implementation does not support the mprotect() function
as described in System Interfaces and Headers, Issue 5:
	A call to mprotect() shall return -1 and set errno to
	ENOSYS.
*/

private void test11()
{
#ifndef _POSIX_MEMORY_PROTECTION
	int	err;
#endif

	DBUG_ENTER("test11");

#ifdef _POSIX_MEMORY_PROTECTION
	in_rpt("_POSIX_MEMORY_PROTECTION is defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN 
#else
	if (supports_mprotect) {
		in_rpt("mprotect is supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN 
	}

	if (tb_mprotect(0, 0, 0) != -1) {
		in_rpt("mprotect() succeeded");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}
	if (errno != ENOSYS) {
		err = errno;
		in_rpt("mprotect failed but errno = %d (%s)", err, errname(err));
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

	xs_rpt();

	DBUG_VOID_RETURN 
#endif
} 



private	int tb_mprotect(addr, len, prot)
void	*addr;
size_t	len;
int	prot;
{
	int	retval;

	retval = mprotect(addr, len, prot);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
