/*
*      SCCS:  @(#)  XOPEN.os/procenv/chroot/chroot.c	Rel 4.4.5     (07/31/00) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/procenv/chroot - T.chroot Rel 4.4.5 (07/31/00)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.chroot

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Thu Aug  7 11:07:26 PDT 1986
MODIFICATIONS:	D McCardle Fri Jun 12 14:47:17 WET 1987
			Added vg_sjmp, setjmp(), tn_rpt().
			Removed abandon(), testid().
			Modified xx_rpt(), xs_rpt().
		
		Geoff Clare, 6 Apr 1990
			New interface to nametoolng() and pathtoolng().
		
		Geoff Clare, 11 Apr 1990
			Change some FAILUREs to DELETIONs

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ,
			Eliminated reason().

		Stuart Boutell, 12 Nov 1990
			uids() return value check added
		
		Stuart Boutell, 27 Nov 1990
			Pipe reporting installed

		Stuart Boutell, 7 Dec 1990
			tet_exec changed to execve in test4; cannot
			perform tet reporting in chroot()ed child

		Peter Skan, 10 Sept 1991
			Add setprv(PRV_NEWROOT) for all tests and remove
			this priviledge for test9

		Geoff Clare, 23 Mar 1992
			Removed globok definition.

		Geoff Clare, 6 June 1992
			Pass dev&ino using pipe instead of command line.

		Neil Moses, 16 Oct 1992
			XNFS enhancements

		Neil Moses, 11 Dec 1992
			Copy chroot_t4 to remote directory before chroot()

		Geoff Clare, 20 Jan 1993
			Create non-directory file in test 5 instead of
			using testset executable.  Fixed pipe reporting
			problem in test 14.

		Geoff Clare, 31 Mar 1993
			Only copy chroot_t4 when testing XNFS.

		Geoff Clare, 10 May 1993
			Give FIP result if execution of chroot_t4 may
			have failed/terminated due to unavailability
			of shared libraries.

		Geoff Clare, 8 Feb 1994
			Only unlink chroot_t4 when testing XNFS.

		Geoff Clare, 7 March 1994
			Only do "path prefix" case in test 14.

		Geoff Clare, May 1998
			Allow ENOSYS if LEGACY feature group not supported.
			Move chroot() call into second child in test 4
			(distributed tet_fork() can't create child
			connections after chroot() on some systems).

		Geoff Clare, 5 Nov 1998
			Create the target directory in ch_t12_13().

		Geoff Clare, 25 Feb 2000
			Move the pipe() call in test 4 to before the chroot().

************************************************************************/
#include	<std.h>
#include	<limits.h>
#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<signal.h>
#include	<sys/wait.h>
#include	<report.h>
#include	<setprv.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<string.h>
#include <tet_api.h>
#include	<xnfs.h>
#include	"chroot_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef chroot
#endif

#define	NO_TESTS	14
#define	WAITTIME	(5 * SPEEDFACTOR)

#define	BUFFERSIZE	512
#define PATH_OK		02

extern	char	*errname();
extern	char	*signame();
extern	char	*itoa();
extern   char   *execpath();
#if TEST_XNFS
extern   char    *xnfs_nametoolng();
#else
extern	char	**nametoolng();
#endif
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	void	xx_all();
extern	int	uids();
extern	int	statdevinod();
extern	int	pipe_open();
extern	void	pipe_close();
extern	void	pipe_child();
extern	void	pipe_parent();
extern	void	pipe_devinod_rpt();
extern	void	pipe_end_rpt();
extern	void	pipe_path_xs_rpt();
extern	void	pipe_xx_rpt();
extern	void	pipe_in_rpt();

extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

int	no_tests = NO_TESTS;
public	char	rbuf[256];

private	int	setupok;
private	int	testfail;
#if TEST_XNFS
private	char	*test4sub =	"chroot_t4";
private	char	*test4prog =	"d.chroot/chroot_t4";
#else
private	char	*test4prog =	"./chroot_t4";
#endif
private	char	*test5_ndir =	"d.chroot/chroot-t.5";
private	char	*test5_file =	"d.chroot/chroot-t.5/xyz";
private	char	*test6_dir =	"./d.chroot/chroot-d.3";
private	char	*dot =		".";
private	char	*dotdot =	"..";
private	char	*slash =	"/";
private char	**longname;
private	ino_t	save_inode;
private	dev_t	save_dev;
private	ino_t	slsh_inode;
private	dev_t	slsh_dev;

#if TEST_XNFS
private	int	snames[] = {1, 0};
private char	*xnfs_longname;
#endif /* TEXT_XNFS */

#if TEST_XPG >= 500
private int	legacy_sup;
#endif

private void	prepare_tests();
private	int	c_chroot();
private	int	d_chroot();
private	int	stat_i_d();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
#if TEST_XNFS
private	void	ch_t4_copy();
#endif
private	void	test5();
private	void	test6();
private	void	test7();
private	void	test8();
private	void	test9();
private	void	test10();
private	void	test11();
private	void	test12();
private	void	test13();
private	void	test14();
private	void	ch_t1();
private	void	ch_t2();
private	void	ch_t3();
private	void	ch_t4();
private	void	pa_t4();
private void	ch2_t4();
private	void	ch_t5();
private	void	ch_t6();
private	void	ch_t7();
#if !TEST_XNFS
private	void	ch_t8();
#endif
private	void	ch_t9();
#if TEST_XNFS
private	void	ch_t10_11();
private	void	ch_t12_13();
private	void	ch_t14();
#endif

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	NULL, 0
};

private void
prepare_tests()
{
	static char sbuf[128];
	int	err;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	if(uids()!=0)
	{
		xx_all(DELETION, "uids() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (setprv(PRV_NEWROOT) != 0)
	{
		xx_all(DELETION, "setprv(PRV_NEWROOT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (chdir("..") == SYSERROR)
	{
		err = errno;
		(void) sprintf(sbuf,
			"could not change directory to \"..\" - errno %d (%s)",
			err, errname(err));
		xx_all(DELETION, sbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statdevinod(dot, &save_inode, &save_dev) == FAIL)
	{
		(void) strcat(strcpy(sbuf, "could not stat "), dot);
		xx_all(DELETION, sbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statdevinod(slash, &slsh_inode, &slsh_dev) == FAIL)
	{
		(void) strcat(strcpy(sbuf, "could not stat "), slash);
		xx_all(DELETION, sbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#if TEST_XPG >= 500
	errno = 0;
	if(sysconf(_SC_XOPEN_LEGACY) == -1)
	{
		if (errno != 0)
		{
			xx_all(DELETION, "sysconf(_SC_XOPEN_LEGACY) failed");
			DBUG_VOID_RETURN;
		}
		legacy_sup = 0;
	}
	else
		legacy_sup = 1;
#endif

	setupok = (pathok == 5) ? 0 : -1;

	DBUG_VOID_RETURN;
}

private	void
test1()
{
	DBUG_ENTER("test1");

	testfail = 0;

	if(pipe_open()!=0)
	{
		(void) cppair(ch_t1, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	ino_t	inode;
	dev_t	dev;
	int	pathok = setupok;

	DBUG_ENTER("ch_t1");

	pipe_child();

	globok = 0;
	if (c_chroot(dot) != 0)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (stat_i_d(slash, &inode, &dev) == FAIL)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (save_inode != inode || save_dev != dev)
	{
		if (testfail++ == 0)
			pipe_xx_rpt(FAILURE);
		(void) sprintf(rbuf,"chroot(\"%s\") failed", dot);
		pipe_in_rpt(rbuf);
		pipe_devinod_rpt(save_inode, save_dev, inode, dev);
	}
	else if (testfail == 0)
	{
		PATH_TRACE;
		pipe_path_xs_rpt(3,pathok);
	}

	pipe_end_rpt();
	DBUG_VOID_RETURN;
}

private	void
test2()
{
	DBUG_ENTER("test2");

	testfail = 0;

	if(pipe_open()!=0)
	{
		(void) cppair(ch_t2, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	ino_t	inode;
	dev_t	dev;
	int	pathok = setupok;

	DBUG_ENTER("ch_t2");

	pipe_child();

	globok = 0;
	if (c_chroot(dot) != 0)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (stat_i_d(dot, &inode, &dev) == FAIL)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (save_inode != inode || save_dev != dev)
	{
		if (testfail++ == 0)
			pipe_xx_rpt(FAILURE);
		(void)sprintf(rbuf,"chroot(\"%s\") failed", dot);
		pipe_in_rpt(rbuf);
		pipe_devinod_rpt(save_inode, save_dev, inode, dev);
	}
	else if (testfail == 0)
	{
		PATH_TRACE;
		pipe_path_xs_rpt(3, pathok);
	}

	pipe_end_rpt();
	DBUG_VOID_RETURN;
}

private	void
test3()
{
	DBUG_ENTER("test3");

	testfail = 0;

	if(pipe_open()!=0)
	{
		(void) cppair(ch_t3, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	ino_t	inode;
	dev_t	dev;
	int	pathok = setupok;

	DBUG_ENTER("ch_t3");

	pipe_child();

	globok = 0;
	if (c_chroot(dot) != 0)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (stat_i_d(dotdot, &inode, &dev) == FAIL)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (save_inode != inode || save_dev != dev)
	{
		if (testfail++ == 0)
			pipe_xx_rpt(FAILURE);
		(void)sprintf(rbuf,"chroot(\"%s\") failed", dot);
		pipe_in_rpt(rbuf);
		pipe_devinod_rpt(save_inode, save_dev, inode, dev);
	}
	else if (testfail == 0)
	{
		PATH_TRACE;
		pipe_path_xs_rpt(3,pathok);
	}

	pipe_end_rpt();
	DBUG_VOID_RETURN;
}

private	void
test4()
{
	DBUG_ENTER("test4");

	testfail = 0;

	if(pipe_open()!=0)
	{
		(void) cppair(ch_t4, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	DBUG_ENTER("ch_t4");

	pipe_child();

#if TEST_XNFS
	if(tet_fork(ch_t4_copy, NULLFN, WAITTIME, OK) != OK)
		DBUG_VOID_RETURN;
#endif

	(void) cpsig(ch2_t4, pa_t4, WAITTIME, 0);

	pipe_end_rpt();	

#if TEST_XNFS
	(void) unlink(test4prog);
#endif

	DBUG_VOID_RETURN;
}

private void
ch2_t4()
{
	ino_t	inode;
	dev_t	dev;
	int	indx;
	int	pfd[2];
	char	*sub_argv[6];		/* this is how many args get passed. */
	int	pathok = setupok;

	DBUG_ENTER("ch2_t4");

	if (pipe(pfd) == SYSERROR)
	{
		(void)sprintf(rbuf,"pipe() failed - errno %d (%s)",
			errno, errname(errno));
		pipe_xx_rpt(DELETION);
		pipe_in_rpt(rbuf);
	}
	else if (c_chroot(dot) != 0)
	{
		DBUG_EXIT(OK);
	}
	else if (stat_i_d(slash, &inode, &dev) == FAIL)
	{
		DBUG_EXIT(OK);
	}
	else if (save_inode != inode || save_dev != dev)
	{
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf,"inode of %s changed after fork", slash);
		pipe_in_rpt(rbuf);
		pipe_devinod_rpt(save_inode, save_dev, inode, dev);
	}
	else if (write(pfd[1], (char *)&dev, sizeof(dev)) != sizeof(dev) ||
		 write(pfd[1], (char *)&inode, sizeof(inode)) != sizeof(inode))
	{
		(void)sprintf(rbuf,"write() to pipe failed - errno %d (%s)",
			errno, errname(errno));
		pipe_xx_rpt(DELETION);
		pipe_in_rpt(rbuf);
	}
	else
	{
		PATH_TRACE;
		indx = 1;
		sub_argv[0] = test4prog;
		if (vg_ident != (char *) NULL)
			sub_argv[indx++] = vg_ident;
		if (vg_dbug != (char *) NULL)
			sub_argv[indx++] = vg_dbug;
		sub_argv[indx++] = itoa(pfd[0]);
		sub_argv[indx] = (char *) NULL;
		(void) execve(test4prog, sub_argv, environ);
		(void) sprintf(rbuf,"execve() of %s failed - errno %d (%s)",
			test4prog, errno, errname(errno));

		if (eaccess(test4prog, X_OK) == 0)
		{
			/* may be due to unavailability of shared libraries */
			pipe_xx_rpt(RFI);
		}
		else
			pipe_xx_rpt(DELETION);
		pipe_in_rpt(rbuf);
	}

	DBUG_EXIT(OK);
}

private void
pa_t4()
{
	int	rtval, err, status;

	DBUG_ENTER("pa_t4");

	SET_TIMEOUT(WAITTIME)

	rtval = waitpid(tet_child, &status, WUNTRACED);
	err = errno; 

	CLEAR_ALARM

	if (rtval == -1)
	{
		pipe_xx_rpt(DELETION);
		if (alrm_flag > 0)
			(void) sprintf(rbuf, "child process timed out");
		else
			(void) sprintf(rbuf, "waitpid() failed - errno %d (%s)",
				err, errname(err));
		pipe_in_rpt(rbuf);
		(void) killchild(tet_child, WAITTIME);
	}
	else if (WIFEXITED(status))
	{
		status = WEXITSTATUS(status);
		if (status != OK)
		{
			pipe_xx_rpt(DELETION);
			(void) sprintf(rbuf,
				"child process gave unexpected exit code %d",
				status);
			pipe_in_rpt(rbuf);
		}
	}
	else if (WIFSIGNALED(status))
	{
		/* may be due to unavailability of shared libraries */
		pipe_xx_rpt(RFI);
		status = WTERMSIG(status);
		(void) sprintf(rbuf,
			"child process was terminated by signal %d (%s).",
			status, signame(status));
		pipe_in_rpt(rbuf);
	}
	else if (WIFSTOPPED(status))
	{
		pipe_xx_rpt(DELETION);
		status = WSTOPSIG(status);
		(void) sprintf(rbuf,
			"child process was stopped by signal %d (%s)",
			status, signame(status));
		pipe_in_rpt(rbuf);
		(void) killchild(tet_child, WAITTIME);
	}
	else
	{
		pipe_xx_rpt(DELETION);
		(void) sprintf(rbuf,
			"child process returned bad wait status (%#x)",
			status);
		pipe_in_rpt(rbuf);
	}

	DBUG_VOID_RETURN;
}

#if TEST_XNFS
private	void
ch_t4_copy()
{
	int	err;
	char	buf[_POSIX_PATH_MAX * 2 + 30];
	char	*from;
	char	*to;

	DBUG_ENTER("ch_t4_copy");

	from = execpath(test4sub);
	to = test4prog;

	if (strlen(from) + strlen(to) > 2 * _POSIX_PATH_MAX)
		DBUG_EXIT(0);

	/* generate the string "cp from to" */
	(void) strcat(strcat(strcat(strcat(strcpy(buf, "cp "), from), " "),
		to), "> /dev/null 2>&1");
	DBUG_PRINTF("file_copy", "%s", buf);

	(void) unlink(to);
	
	if(setuid(getuid()) == SYSERROR)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void) sprintf(rbuf,"setuid(getuid()) failed - errno %d(%s)",
			err, errname(err));
		pipe_in_rpt(rbuf);
		DBUG_EXIT(0);
	}

	if (system(buf) != SUCCEED)
	{
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf, "system(\"%s\") failed", buf);
		pipe_in_rpt(rbuf);
		pipe_end_rpt();
		(void) unlink(to);
		DBUG_EXIT(0);
	}

	DBUG_EXIT(OK);
}
#endif

private	void
test5()
{
	int	fd, err;
	int	pathok = setupok;

	DBUG_ENTER("test5");

	testfail = 0;

	if ((fd = creat(test5_ndir, (mode_t)S_IRWXU)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(\"%s\", %#lo) failed - errno %d (%s)",
			test5_ndir, (unsigned long)S_IRWXU, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	if(pipe_open()!=0)
	{
		PATH_FUNC_RPT(1);
		(void) cppair(ch_t5, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	(void) unlink(test5_ndir);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	ino_t	inode;
	dev_t	dev;
	int	pathok = 0;

	DBUG_ENTER("ch_t5");

	PATH_FUNC_TRACE;

	pipe_child();

	globok = 0;
	if (d_chroot(test5_file, ENOTDIR) != -1)
	{
		pipe_end_rpt();	
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (stat_i_d(slash, &inode, &dev) == FAIL)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (slsh_inode != inode || slsh_dev != dev)
	{
		if (testfail++ == 0)
			pipe_xx_rpt(FAILURE);
		pipe_in_rpt("chroot() returned -1 but changed root:");
		pipe_devinod_rpt(slsh_inode, slsh_dev, inode, dev);
	}
	else if (testfail == 0)
	{
		PATH_TRACE;
		pipe_path_xs_rpt(4, pathok);
	}

	pipe_end_rpt();
	DBUG_VOID_RETURN;
}

private	void
test6()
{
	DBUG_ENTER("test6");

	testfail = 0;

	if(pipe_open() != 0)
	{
		(void) cppair(ch_t6, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	ino_t	inode;
	dev_t	dev;
	struct	stat	statb;
	int	pathok = setupok;

	DBUG_ENTER("ch_t6");

	pipe_child();

	if (stat(test6_dir, &statb) != SYSERROR)
	{
		if (remove(test6_dir) == SYSERROR)
		{
			(void)sprintf(rbuf,"remove(%s) failed - errno %d (%s)",
				test6_dir, errno, errname(errno));
			pipe_xx_rpt(DELETION);
			pipe_in_rpt(rbuf);
			pipe_end_rpt();
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (d_chroot(test6_dir, ENOENT) != -1)
	{
		pipe_end_rpt();	
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (stat_i_d(slash, &inode, &dev) == FAIL)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (slsh_inode != inode || slsh_dev != dev)
	{
		if (testfail++ == 0)
			pipe_xx_rpt(FAILURE);
		pipe_in_rpt("chroot() returned -1 but changed root:");
		pipe_devinod_rpt(slsh_inode, slsh_dev, inode, dev);
	}
	else if (testfail == 0)
	{
		PATH_TRACE;
		pipe_path_xs_rpt(4,pathok);
	}

	pipe_end_rpt();
	DBUG_VOID_RETURN;
}

private	void
test7()
{
	DBUG_ENTER("test7");

	testfail = 0;

	if(pipe_open()!=0)
	{
		(void) cppair(ch_t7, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int	pathok = setupok;

	DBUG_ENTER("ch_t7");

	pipe_child();

	globok = 0;
	longname = pathtoolng(dot);
	if (longname == NULL)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if (c_chroot(longname[0]) == -1)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) d_chroot(longname[1], ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		pipe_path_xs_rpt(3, pathok);

	pipe_end_rpt();
	clntoolng();
	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test8()
{
	DBUG_ENTER("test8");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test8()
{
	int	pathok = setupok;

	DBUG_ENTER("test8");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (crdir(longname[0], (mode_t)S_IRWXU,
		(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", longname[0]);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(pipe_open() != 0)
	{
		PATH_FUNC_RPT(2);
		(void) cppair(ch_t8, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	(void) rmdir(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t8");

	PATH_FUNC_TRACE; /* passed down from parent */

	pipe_child();

	globok = 0;
	if (c_chroot(longname[0]) == -1)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) d_chroot(longname[1], ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		pipe_path_xs_rpt(3, pathok);

	pipe_end_rpt();
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private	void
test9()
{
	DBUG_ENTER("test9");

	testfail = 0;

	if(pipe_open() != 0)
	{
		(void) cppair(ch_t9, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	ino_t	inode;
	dev_t	dev;
	int	pathok = setupok;

	DBUG_ENTER("ch_t9");

	pipe_child();

	if (unsetprv(PRV_NEWROOT) != 0)
	{
		pipe_xx_rpt(DELETION);
		pipe_in_rpt("unsetprv(PRV_NEWROOT) failed");
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (d_chroot(dot, EPERM) != -1)
	{
		pipe_end_rpt();	
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (stat_i_d(slash, &inode, &dev) == FAIL)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (slsh_inode != inode || slsh_dev != dev)
	{
		if (testfail++ == 0)
			pipe_xx_rpt(FAILURE);
		pipe_in_rpt("chroot() returned -1 but changed root:");
		pipe_devinod_rpt(slsh_inode, slsh_dev, inode, dev);
	}
	else if (testfail == 0)
	{
		PATH_TRACE;
		pipe_path_xs_rpt(4, pathok);
	}

	pipe_end_rpt();
	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private	void
test10()
{
	int	pathok = setupok;

	DBUG_ENTER("test10");

	testfail = 0;

	globok = 0;
	xnfs_longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (xnfs_longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(pipe_open()!=0)
	{
		PATH_FUNC_RPT(1);
		(void) cppair(ch_t10_11, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	clntoolng();
	DBUG_VOID_RETURN;
}

private void
ch_t10_11()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t10_11");

	PATH_FUNC_TRACE;

	pipe_child();

	globok = 0;
	if (d_chroot(xnfs_longname, ENAMETOOLONG) != -1)
	{
		pipe_end_rpt();	
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		pipe_path_xs_rpt(2,pathok);

	pipe_end_rpt();
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test10()
{
	DBUG_ENTER("test10");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private	void
test11()
{
	int	pathok = setupok;

	DBUG_ENTER("test11");

	testfail = 0;

	globok = 0;
	xnfs_longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (xnfs_longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(pipe_open()!=0)
	{
		PATH_FUNC_RPT(1);
		(void) cppair(ch_t10_11, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	clntoolng();
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test11()
{
	DBUG_ENTER("test11");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private	void
test12()
{
	int	pathok = setupok;

	DBUG_ENTER("test12");

	testfail = 0;

	globok = 0;
	xnfs_longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (xnfs_longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(pipe_open()!=0)
	{
		PATH_FUNC_RPT(1);
		(void) cppair(ch_t12_13, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	(void) rmdir(xnfs_longname);
	clntoolng();
	DBUG_VOID_RETURN;
}

private void
ch_t12_13()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t12_13");

	PATH_FUNC_TRACE;

	pipe_child();

	if (crdir(xnfs_longname, (mode_t)S_IRWXU,
		(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		pipe_xx_rpt(DELETION);
		(void) sprintf(rbuf, "could not create directory \"%s\"",
			xnfs_longname);
		pipe_in_rpt(rbuf);
		pipe_end_rpt();	
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (c_chroot(xnfs_longname) == -1)
	{
		pipe_end_rpt();	
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		pipe_path_xs_rpt(3,pathok);

	pipe_end_rpt();
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test12()
{
	DBUG_ENTER("test12");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private	void
test13()
{
	int	pathok = setupok;

	DBUG_ENTER("test13");

	testfail = 0;

	globok = 0;
	xnfs_longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (xnfs_longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(pipe_open()!=0)
	{
		PATH_FUNC_RPT(1);
		(void) cppair(ch_t12_13, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	(void) rmdir(xnfs_longname);
	clntoolng();
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test13()
{
	DBUG_ENTER("test13");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test14()
{
	DBUG_ENTER("test14");

	testfail = 0;

	if (tet_sync(SP_T14_CREATE_DIR_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREATE_DIR_END failed");
		DBUG_VOID_RETURN;
	}
	else if(pipe_open()!=0)
	{
		(void) tet_fork(ch_t14, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	(void) tet_sync(SP_T14_RMDIR_END, snames, SYNC_TIMEOUT);

	DBUG_VOID_RETURN;
}

private void
ch_t14()
{
	ino_t	inode;
	dev_t	dev;
	int	pathok = setupok;

	DBUG_ENTER("ch_t14");

	pipe_child();

	globok = 0;
	if (d_chroot(t14_subdir, EACCES) != -1)
	{
		pipe_end_rpt();	
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (stat_i_d(slash, &inode, &dev) == FAIL)
	{
		pipe_end_rpt();
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (slsh_inode != inode || slsh_dev != dev)
	{
		if (testfail++ == 0)
			pipe_xx_rpt(FAILURE);
		pipe_in_rpt("chroot() returned -1 but changed root:");
		pipe_devinod_rpt(slsh_inode, slsh_dev, inode, dev);
	}
	else if (testfail == 0)
	{
		PATH_TRACE;
		pipe_path_xs_rpt(3, pathok);
	}

	pipe_end_rpt();
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test14()
{
	DBUG_ENTER("test14");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private int
c_chroot(path)
char *path;
{
	int	rval, err;
	int	pathok = 0;
	char	reppath[105];

	DBUG_ENTER("c_chroot");

	errno = 0;
	if ((rval = chroot(path)) != 0)
	{
		err = errno;
#if TEST_XPG >= 500
		/* If the LEGACY group is not supported, chroot() can
		   still be supported on its own */
		if (err == ENOSYS)
		{
			if (legacy_sup)
			{
				if (testfail++ == 0)
					pipe_xx_rpt(FAILURE);
				pipe_in_rpt("chroot() set errno to ENOSYS when LEGACY group is supported");
			}
			else
			{
				pipe_xx_rpt(UNSUPPORTED);
				pipe_in_rpt("chroot() set errno to ENOSYS");
			}
			DBUG_RETURN(rval);
		}
#endif

		/* Don't print PATH_MAX length path name */
		if (strlen(path) > 100)
			(void) sprintf(reppath, "<%d char path>",
				(int)strlen(path));
		else
			(void) sprintf(reppath, "\"%s\"", path);

		if (testfail++ == 0)
			pipe_xx_rpt(FAILURE);
		(void)sprintf(rbuf, "chroot(%s) returned %d, expected 0",
			reppath, rval);
		pipe_in_rpt(rbuf);
		(void)sprintf(rbuf, "errno was set to %d (%s)",
			err, errname(err));
		pipe_in_rpt(rbuf);
		DBUG_RETURN(rval);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);
	DBUG_RETURN(rval);
}

private int
d_chroot(path, experr)
char *path;
int experr;
{
	int	rval, err;
	int	pathok = 0;
	char	reppath[105];

	DBUG_ENTER("d_chroot");

	errno = 0;
	rval = chroot(path);
	err = errno;

	if (rval == -1 && err == experr)
	{
		PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_RETURN(rval);
	}
#if TEST_XPG >= 500
	else if (rval == -1 && err == ENOSYS)
	{
		if (legacy_sup)
		{
			if (testfail++ == 0)
				pipe_xx_rpt(FAILURE);
			pipe_in_rpt("chroot() set errno to ENOSYS when LEGACY group is supported");
		}
		else
		{
			pipe_xx_rpt(UNSUPPORTED);
			pipe_in_rpt("chroot() set errno to ENOSYS");
		}
		DBUG_RETURN(0);
	}
#endif

	/* Don't print PATH_MAX length path name */
	if (experr == ENAMETOOLONG && strlen(path) > 100)
		(void) sprintf(reppath, "<%d char path>", (int)strlen(path));
	else
		(void) sprintf(reppath, "\"%s\"", path);

	if (testfail++ == 0)
		pipe_xx_rpt(FAILURE);
	if (rval != -1)
		(void)sprintf(rbuf, "chroot(%s) returned %d, expected -1",
			reppath, rval);
	else
		(void)sprintf(rbuf, "chroot(%s) set errno to %d (%s), expected %d (%s)",
			reppath, err, errname(err), experr, errname(experr));
	pipe_in_rpt(rbuf);

	DBUG_RETURN(rval);
}

private	int
stat_i_d(path, inode, dev)
	char	*path;
	ino_t	*inode;
	dev_t	*dev;
{ 
	int	err;
	struct	stat	statb;

	DBUG_ENTER("stat_i_d");

#ifdef	TESTBED2
#undef	SYSERROR
#define	SYSERROR	0
#endif
	if(stat(path, &statb) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf,"stat(%s) failed",path);
		pipe_in_rpt(rbuf);
		(void)sprintf(rbuf,"errno %d (%s)",err, errname(err));
		pipe_in_rpt(rbuf);
		DBUG_RETURN(FAIL);
	}
	*inode = statb.st_ino;
	*dev   = statb.st_dev;
	DBUG_RETURN(SUCCEED);
}
