/*
*      SCCS:  @(#)  XOPEN.os/procenv/chroot/chroot_s.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1992 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/procenv/chroot/chroot_s Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1992 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		chroot_s.c

PROJECT:	XNFS (X/OPEN NFS tests)

DESCRIPTION:
        Slave part of the chroot() system call test set.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Neil Moses, UniSoft Ltd.
DATE CREATED:	29 September 1992
MODIFICATIONS:
		Geoff Clare, 20 Jan 1993
			Added chdir("..") in setup so paths match client.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<fcntl.h>
#include 	<tet_api.h>
#include	<xnfs.h>
#include	"chroot_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef chroot
#endif


#define	MODEGEN		(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();
extern	void	xx_all();

private	int	setupok = -1;
private	int	*snames = (int *)0;

private	void	setup();
private	void	test10();
private	void	test11();
private	void	test12();
private	void	test13();
private	void	test14();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 t_success_s, 1,
	 t_success_s, 2,
	 t_success_s, 3,
	 t_success_s, 4,
	 t_success_s, 5,
	 t_success_s, 6,
	 t_success_s, 7,
	 t_success_s, 8,
	 t_success_s, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 NULL, 0
};

private void
setup()
{
	int err;
	static char sbuf[128];
	int pathok = 0;

	DBUG_ENTER("setup");

	if (chdir("..") == SYSERROR)
	{
		err = errno;
		(void) sprintf(sbuf,
			"could not change directory to \"..\" - errno %d (%s)",
			err, errname(err));
		xx_all(DELETION, sbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	PATH_SETUP_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	DBUG_ENTER("test10");

	t_nametoolng_s(1, MK_SPNO(0));

	DBUG_VOID_RETURN;
}

private void
test11()
{
	DBUG_ENTER("test11");

	t_nametoolng_s(2, MK_SPNO(0));

	DBUG_VOID_RETURN;
}

private void
test12()
{
	DBUG_ENTER("test12");

	t_nametoolng_s(3, MK_SPNO(0));

	DBUG_VOID_RETURN;
}

private void
test13()
{
	DBUG_ENTER("test13");

	t_nametoolng_s(4, MK_SPNO(0));

	DBUG_VOID_RETURN;
}

private void
test14()
{
	mode_t	mode1 = S_IRUSR | S_IWUSR;
	mode_t	mode2 = MODEGEN;
	int	pathok = setupok;

	DBUG_ENTER("test14");

	if (tet_sync(SP_T14_CREATE_DIR_START, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREATE_DIR_START failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t14_dir, mode1, (int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t14_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t14_subdir, mode2, (int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t14_subdir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T14_CREATE_DIR_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREATE_DIR_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T14_RMDIR_START, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to RMDIR_START failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) rmdir(t14_subdir);
	(void) rmdir(t14_dir);

	if (tet_sync(SP_T14_RMDIR_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to RMDIR_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}
