/*
*      SCCS:  @(#)  XOPEN.os/procenv/chroot/chroot_t4.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/procenv/chroot - chroot_t4 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		chroot_t4 - used by T.chroot in test4()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.chroot to test result of chroot system call.
	  usage:
		chroot_t4 [ vg_ident [ vg_dbug ] ] save_inode save_dev

	  where save_inode is a string containing the octal/decimal/hex
	  value of the inode of / before the exec, likewise for save_dev
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Thu Aug  7 14:02:20 PDT 1986
MODIFICATIONS:	D McCardle Fri Jun 12 15:07:50 WET 1987
			Removed testid().
			Modified xx_rpt(), xs_rpt().
		
		Geoff Clare, 11 Apr 1990
			Change FAILURE to DELETION if stat() fails

		J.A.Nave	09/05/90
		Added resilience testing

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Stuart Boutell, 6 Dec 1990
			Removed all TET calls & interfaces.

		Geoff Clare, 6 June 1992
			Pass dev&ino using pipe instead of command line.

		Geoff Clare, 4 Jan 1993
			Remove DBUG_ENTER and DBUG_EXIT calls from main().

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <dbug.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <report.h>
#include <errno.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef chroot
#endif


public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	void	devinod_rpt();
extern	char	*errname();
extern	void	set_pname();
extern	void	pipe_xs_rpt();
extern	void	pipe_xx_rpt();
extern	void	pipe_path_xs_rpt();
extern	void	pipe_in_rpt();
extern	void	pipe_end_rpt();
extern	void	pipe_devinod_rpt();

private	char	*slash = "/";
public	char	rbuf[256];

private	void	subtest4();
private int	stat_i_d();

int
main(argc, argv)

	int	argc;
	char	**argv;
{
	int	pathok = 0;

	set_pname(argc < 1 || argv[1] == (char *) NULL ?
		"chroot_t4" : (argv[1]));
	if (argc > 2)
	{
		if (argv[1] != (char *) NULL)
			vg_ident = argv[1];
		if (argc > 3)
		{
			if (argv[2] != (char *) NULL)
				vg_dbug = argv[2];
			argv++;
			argc--;
		}
		argv++;
		argc--;
	}
	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}
	if (argc != 2)
	{
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf,"deletion reason: argument count wrong. argc is %d", argc);
		pipe_in_rpt(rbuf);
		pipe_end_rpt();
		exit(OK);
	}
	else if (argv[1] == (char *) NULL)
	{
		pipe_xx_rpt(DELETION);
		pipe_in_rpt("deletion reason: pipe fd string (argv[1]) is null");
		pipe_end_rpt();
		exit(OK);
	}
	else
		PATH_TRACE;

	globok = (pathok == 1) ? 1 : 0;

	subtest4(argv[1]);

	pipe_end_rpt();

	exit(OK);

	/*NOTREACHED*/

	return 0;
}

private void
subtest4(sfd)
char *sfd;
{
	int	fd;
	ino_t	save_inode;
	ino_t	inode;
	dev_t	save_dev;
	dev_t	dev;
	int	pathok = 0;

	DBUG_ENTER("subtest4");

	if (globok == 1)
		PATH_TRACE;

	fd = atoi(sfd);
	if (read(fd, (char *)&save_dev, sizeof(save_dev)) != sizeof(save_dev) ||
	    read(fd, (char *)&save_inode, sizeof(save_inode)) != sizeof(save_inode))
	{
		(void)sprintf(rbuf,"read() failed on pipe - errno %d (%s)",
			errno, errname(errno));
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else if (stat_i_d(slash, &inode, &dev) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else if (inode != save_inode || dev != save_dev)
	{
		pipe_xx_rpt(FAILURE);
		(void)sprintf(rbuf,"%s changed after exec", slash);
		pipe_in_rpt(rbuf);
		pipe_devinod_rpt(save_inode, save_dev, inode, dev);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	pipe_path_xs_rpt(2, pathok);

	DBUG_VOID_RETURN;
}

private	int
stat_i_d(path, inode, dev)
	char	*path;
	ino_t	*inode;
	dev_t	*dev;
{ 
	int	err;
	struct	stat	statb;

	DBUG_ENTER("stat_i_d");

	if(stat(path, &statb) == SYSERROR)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf,"deletion reason: stat(%s) failed",path);
		pipe_in_rpt(rbuf);
		(void)sprintf(rbuf,"errno %d (%s)",err, errname(err));
		pipe_in_rpt(rbuf);
		pipe_end_rpt();
		DBUG_RETURN(FAIL);
	}
	*inode = statb.st_ino;
	*dev   = statb.st_dev;
	DBUG_RETURN(SUCCEED);
}
