/*
*      SCCS:  @(#)  XOPEN.os/procenv/cuserid/cuserid.c	Rel 4.4.3	    (07/20/98)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/procenv/cuserid - T.cuserid Rel 4.4.3 (07/20/98)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.cuserid

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the cuserid() system interface.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	23 Aug 1989
MODIFICATIONS:	
		Dave Cater, 18 Dec 1989
			Issue warning, not failure, if cuserid() uses real id.

		J.A.Nave	25/05/90
		Added resilience testing

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 23 Mar 1992
			Removed globok definition.

		Geoff Clare, 28 July 1992
			Pass test if real user name returned in XPG4 mode.

		Geoff Clare, April 1998
			Allow ENOSYS if LEGACY feature group not supported.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef cuserid
#endif

#define 	NO_TESTS	4
#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();
extern	char	*itoa();
extern	void	xx_all();

extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	int	vguid1;

public	int	no_tests = NO_TESTS;

private	int	setupok;
private char	*prog_su =	"../cuserid_su";
private char	*vsx_ttyuser;
private	int	testfail, testunsup;
private char	buf[256];
#if TEST_XPG >= 500
private int	legacy_sup;
#endif

private	void	prepare_tests();
private	void	tbcuserid();

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private	void	ch_t1();
private	void	ch_t2();
private	void	ch_t3_4();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private	void
prepare_tests()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	if(uids()!=0)
	{
		xx_all(DELETION, "uids() failed");
		DBUG_VOID_RETURN;
	}
	else if (geteuid() != vguid1)
	{
		for (i = 1; i <= 2; i++)
			tet_delete(i, "effective UID != vguid1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (getuid() == geteuid())
	{
		for (i = 1; i <= 2; i++)
			tet_delete(i,
				"effective UID and real UID are not different");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	vsx_ttyuser = tet_getvar("VSX_TTYUSER");
	if (vsx_ttyuser == NULL || *vsx_ttyuser == '\0')
		for (i = 1; i <= 2; i++)
			tet_delete(i, "parameter VSX_TTYUSER is not defined");
	else
		PATH_TRACE;

#if TEST_XPG >= 500
	errno = 0;
	if(sysconf(_SC_XOPEN_LEGACY) == -1)
	{
		if (errno != 0)
		{
			xx_all(DELETION, "sysconf(_SC_XOPEN_LEGACY) failed");
			DBUG_VOID_RETURN;
		}
		legacy_sup = 0;
	}
	else
		legacy_sup = 1;
#endif

	setupok = (pathok == 3) ? 0 : -1;

	DBUG_VOID_RETURN;
}


private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	pathok = setupok;

	DBUG_ENTER("ch_t1");

	testfail = testunsup = 0;

	globok = 0;
	tbcuserid(buf, "vsx1");
	PATH_FUNC_TRACE;
	if (testunsup)
	{
		DBUG_VOID_RETURN;
	}

	(void) setuid(getuid());
	if (geteuid() != getuid())
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: setuid(getuid()) did not set effective UID to real UID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	tbcuserid(buf, vsx_ttyuser);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


private	void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	pathok = setupok;

	DBUG_ENTER("ch_t2");

	testfail = testunsup = 0;

	globok = 0;
	tbcuserid((char *)NULL, "vsx1");
	PATH_FUNC_TRACE;
	if (testunsup)
	{
		DBUG_VOID_RETURN;
	}

	(void) setuid(getuid());
	if (geteuid() != getuid())
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: setuid(getuid()) did not set effective UID to real UID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	tbcuserid((char *)NULL, vsx_ttyuser);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) cppair(ch_t3_4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	(void) cppair(ch_t3_4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3_4()
{
	extern	char	**environ;
	int	err;
	char	*args[5];
	int	pathok = setupok;

	DBUG_ENTER("ch_t3_4");

	PATH_TRACE;	/*	to prove that we have been called	*/

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(prog_su, args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog_su, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
tbcuserid(arg, expect)
char *arg, *expect;
{
	char	*rval, *strarg;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("tbcuserid");

	if (arg == NULL)
		strarg = "NULL";
	else
		strarg = "buf";

	errno = 0;
	rval = cuserid(arg);
	err = errno;

	if (arg != NULL && rval != arg)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (rval == NULL)
			in_rpt("cuserid(buf) returned NULL");
		else
			in_rpt("cuserid(buf) did not return buf");
	}
	else if (rval == NULL || *rval == '\0')
	{
#if TEST_XPG >= 500
		if (err == ENOSYS)
		{
			if (legacy_sup)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("cuserid() set errno to ENOSYS when LEGACY group is supported");
			}
			else
			{
				if (testunsup++ == 0)
					xx_rpt(UNSUPPORTED);
				in_rpt("cuserid() set errno to ENOSYS");
			}
			DBUG_VOID_RETURN;
		}
#endif

		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (rval == NULL)
			in_rpt("cuserid(%s) returned NULL", strarg);
		else
			in_rpt("cuserid(%s) returned an empty string", strarg);
		if (err != 0)
			in_rpt("- errno was set to %d (%s)", err, errname(err));
	}
	else if(strcmp(rval, expect) != 0
#if TEST_XPG >= 4
		&& strcmp(rval, vsx_ttyuser) != 0
#endif
		)
	{
		/* XPG3: Only issue warning if cuserid() uses real user id. */
		if (testfail++ == 0)
		{
			if(strcmp(rval, vsx_ttyuser) == 0)
				xx_rpt(WARNING);
			else
				xx_rpt(FAILURE);
		}
		if (expect != vsx_ttyuser)
			in_rpt("cuserid(%s) returned \"%s\", expected \"%s\" or \"%s\"",
				strarg, rval, expect, vsx_ttyuser);
		else
			in_rpt("cuserid(%s) returned \"%s\", expected \"%s\"",
				strarg, rval, expect);
		in_rpt("current effective UID = %d, real UID = %d",
			(int)geteuid(), (int)getuid());
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);
	DBUG_VOID_RETURN;
}
