/*
*      SCCS:  @(#)  XOPEN.os/procenv/cuserid/cuserid_su.c	Rel 4.4.3     (07/20/98) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/procenv/cuserid - cuserid_su Rel 4.4.3 (07/20/98)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		cuserid_su - used by T.cuserid for privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.cuserid to test cuserid() with unused UID.
	  Must have appropriate privileges to change user ID.

	  usage:
		cuserid_su testnum ident dbug

	  where testnum is the test number being executed.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	23 Aug 1989
MODIFICATIONS:
		Geoff Clare, 3 Jan 1989
			Change tests 3 & 4 to warnings

		J.A.Nave	25/05/90
		Added resilience testing

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Geoff Clare, 9 Jan 1995
			Allow VSX_UNUSED_UID=unsup

		Geoff Clare, April 1998
			Allow ENOSYS if LEGACY feature group not supported.

************************************************************************/
#include <std.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <dbug.h>
#include <report.h>
#include <setprv.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef cuserid
#endif


public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	**environ;

extern	void	set_pname();
extern	char	*errname();

private	void	subtest3_4();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int testnum;
	char buf[256];
	int	pathok = 0;

	set_pname(argv[0]);

	if (argc < 2 || argc > 4)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		exit(OK);
	}
	else
		PATH_TRACE;

	if (argc >= 4)
		vg_dbug = argv[3];
	if (argc >= 3)
		vg_ident = argv[2];

	DBUG_PROCESS(vg_ident);
	if (vg_dbug != NULL)
		DBUG_PUSH(vg_dbug+2);

	globok = (pathok == 1) ? 1 : 0;

	testnum = atoi(argv[1]);
	switch(testnum)
	{
	case 3:
		buf[0] = 'A';
		subtest3_4(buf);		/* Assertion 10 */
		break;

	case 4:
		subtest3_4((char *)NULL);	/* Assertion 11 */
		break;

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed: %d",  testnum);
		exit(OK);
	}

	exit(OK);
	/* NOTREACHED */
}

#if TEST_XPG >= 500
private int
legacy_supp()
{
	DBUG_ENTER("legacy_supp");

	errno = 0;
	if (sysconf(_SC_XOPEN_LEGACY) == -1L)
	{
		if (errno == 0)	/* group is not supported */
			DBUG_RETURN(0);

		in_rpt("sysconf(_SC_XOPEN_LEGACY) failed - errno %d (%s)",
			errno, errname(errno));
		in_rpt("(Remaining test information assumes LEGACY feature group is supported.)");
	}

	DBUG_RETURN(1);
}
#endif

private void
subtest3_4(arg)
char *arg;
{
	/* NULL return/empty buffer, with unused UID */

	char	*cp, *rval;
	uid_t	unused_uid;
	int	err, fail;
	int	pathok = 0;

	DBUG_ENTER("subtest3_4");

	if (globok == 1)
		PATH_TRACE;

	cp = tet_getvar("VSX_UNUSED_UID");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_UNUSED_UID is not set");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(cp, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_UNUSED_UID is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	unused_uid = atol(cp);

	if (setprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) setuid(unused_uid);
	if (geteuid() != unused_uid)
	{
		xx_rpt(DELETION);
		in_rpt("setuid(%ld) did not set effective UID to %ld",
			(long)unused_uid, (long)unused_uid);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	fail = 0;

	errno = 0;
	rval = cuserid(arg);
	err = errno;

#if TEST_XPG >= 500
	if (err == ENOSYS && (rval == NULL || *rval == '\0'))
	{
		if (legacy_supp())
		{
			xx_rpt(FAILURE);
			in_rpt("cuserid() set errno to ENOSYS when LEGACY group is supported");
		}
		else
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("cuserid() set errno to ENOSYS");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else
		PATH_TRACE;
#endif
	if (arg != NULL)
	{
		if (rval != arg || arg[0] != '\0')
		{
			fail++;
			if (rval != arg)
			{
				xx_rpt(FAILURE);
				in_rpt("cuserid(buf) did not return buf");
			}
			if (arg[0] != '\0')
			{
				xx_rpt(WARNING);
				in_rpt("cuserid(buf) did not place a null character in buf[0]");
			}
		}
		else
			PATH_TRACE;

	}
	else
	{
		if (rval != NULL)
		{
			fail++;
			xx_rpt(WARNING);
			in_rpt("cuserid(NULL) did not return NULL");
		}
		else
			PATH_TRACE;
	}

	if (fail > 0)
		in_rpt("current effective user ID is %ld (VSX_UNUSED_UID)",
			(long) unused_uid);
	else
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}
