/*
*      SCCS:  @(#)  XOPEN.os/procenv/nice/nice.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/procenv/nice - T.nice Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.nice

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the nice() system call.

AUTHOR:		Manucher Ghezel-Ayagh, UniSoft Ltd.
DATE CREATED:	Mon Sep 15 10:43:06 BST 1986
MODIFICATIONS:	D McCardle Tue Jun 23 08:57:33 WET 1987
			Removed abandon(), testid().
			Modified xx_rpt(), xs_rpt().

		J.A.Nave	11/05/90
		Added resilience testing

		Stuart Boutell, 2 Nov 1990
			Changed execle to tet_exec
			Changed vg_env to environ.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Peter Skan, 10 Sept 1991
			set/unset PRV_NICE

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<exitcodes.h>
#include	<string.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef nice
#endif


#define	NO_TESTS	9		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)	/* no. of seconds to wait() */

#define	VAL1		10		/* incr values to be used for test3 */
#define	VAL2		-NZERO		/* incr values to be used for test3 */
#define	VAL3		0		/* incr values to be used for test3 */

#define	VSX_NICE	"VSX_NICE"

extern	char	*newstr();
extern	char	*errname();
extern	void	valrpt();
extern	char	*itoa();
extern	int	uids();

extern	char	*vg_ident;
extern	char	**environ;

int	no_tests = NO_TESTS;
private	int	setupok;

private	char	*prog4 =	"../nice_test4";
private int	testrval;

private	int 	tb_nice();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	test6();
private	void	test7();
private	void	test8();
private	void	test9();
private	void	ch_t3();
private	void	ch_t4();
private	void	ch_t5();
private	void	ch_t6();
private	void	ch_t7();
private	void	ch_t8();
private	void	ch_t9();
private void	prepare_tests();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	NULL, 0
};

private	void
prepare_tests()
{
	int j;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	if ((int) geteuid() != 0)
		for (j = 1; j <= no_tests; j++)
			(void) tet_delete(j, "must be super user");
	else
		PATH_TRACE;

	if (setprv(PRV_NICE) != 0)
		for (j = 1; j <= no_tests; j++)
			(void) tet_delete(j, "setprv(PRV_NICE) failed");
	else
		PATH_TRACE;

	if(uids()!=0)
	{
		int i;
		for(i=1; i <= no_tests; i++)
		{
			(void) tet_delete(i, "uids() failed");
		}
	}
	else
		PATH_TRACE;

	setupok = (pathok == 3) ? 0 : -1;

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int		err;
	int		rval;
	int		cur_niceval;
	int		j;
	int		low_nice = 0, high_nice = (2*NZERO)-1;
	int	pathok = setupok;

	DBUG_ENTER("test1");

	errno = NOERROR;
	if (((rval = tb_nice(0)) == SYSERROR) && (errno != NOERROR))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("nice(0) returns %d and errno = %d (%s)", rval, err, errname(err));
		for (j = 2; j <= no_tests; j++)
			(void) tet_delete(j, "depends on success of test 1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (((cur_niceval = (rval + NZERO)) < low_nice) || (cur_niceval > high_nice))
	{
		xx_rpt(FAILURE);
		in_rpt("nice() return value , %d  (+NZERO) is not in range 0-2*[NZERO]-1", rval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}
	
	DBUG_VOID_RETURN;
}

private void
test2()
{
	int		rval;
	int		cur_nice;
	int		j;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	cur_nice = tb_nice(0) + NZERO;
	if (rval = tb_nice(NZERO - cur_nice))
	{
		xx_rpt(FAILURE);
		in_rpt("expected nice() return value: 0");
		in_rpt("observed nice() return value: %d", rval);
		for (j = 3; j <= no_tests; j++)
			(void) tet_delete(j, "depends on success of test 2");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
	
	DBUG_VOID_RETURN;
}

private void
test3()
{
	int		err;

	DBUG_ENTER("test3");

	errno = NOERROR;
	if (((testrval = tb_nice(0)) == SYSERROR) && (errno != NOERROR))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("nice(0) returns %d and errno = %d (%s)", testrval, err, errname(err));
	}
	else
		(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int		rval2;
	int	pathok = setupok;

	DBUG_ENTER("ch_t3");

	if ((rval2 = tb_nice(0)) != testrval)
	{
		xx_rpt(FAILURE);
		in_rpt("expected (parent) nice() return value: %d", testrval);
		in_rpt("observed (child) nice() return value: %d", rval2);
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	err;

	DBUG_ENTER("test4");

	errno = NOERROR;
	if (((testrval = tb_nice(0)) == SYSERROR) && (errno != NOERROR))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("nice(0) returns %d and errno = %d (%s)", testrval, err, errname(err));
	}
	else
		(void) cppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	char	*args[5];
	int	pathok = setupok;
	char	*arg1,	*arg2;

	DBUG_ENTER("ch_t4");

	if ((arg1 = newstr(itoa(testrval))) == (char *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("failed to malloc() memory for arg1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((arg2 = newstr(itoa(pathok))) == (char *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("failed to malloc() memory for arg2");
		DBUG_VOID_RETURN;
	}

	args[0] = prog4;
	args[1] = arg1;
	args[2] = arg2;
	args[3] = vg_ident;
	args[4] = (char *)NULL;
	(void) tet_exec(prog4, args, environ);
	exit(A_EXECFAIL);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	int	err;

	DBUG_ENTER("test5");

	errno = NOERROR;
	if (((testrval = tb_nice(0)) == SYSERROR) && (errno != NOERROR))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("nice(0) returns %d and errno = %d (%s)", testrval, err, errname(err));
	}
	else
		(void) cppair(ch_t5, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	err;
	int	rval;
	int	incr = 10;
	int	pathok = setupok;

	DBUG_ENTER("ch_t5");

	if (unsetprv(PRV_NICE) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: unsetprv(PRV_NICE) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((rval = tb_nice(incr)) != incr)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("expected nice() return value: %d", incr);
		in_rpt("observed nice() return value: %d", rval);
		in_rpt("observed error value = %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private void
test6()
{
	int	err;

	DBUG_ENTER("test6");

	errno = NOERROR;
	if (((testrval = tb_nice(0)) == SYSERROR) && (errno != NOERROR))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("nice(0) returns %d and errno = %d (%s)", testrval, err, errname(err));
	}
	else
		(void) cppair(ch_t6, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	int	err;
	int	rval;
	int	decr = -10;
	int	pathok = setupok;

	DBUG_ENTER("ch_t6");

	errno = NOERROR;
	if ((rval = tb_nice(decr)) != decr)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("expected nice() return value: %d", decr);
		in_rpt("observed nice() return value: %d", rval);
		in_rpt("observed error value = %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test7()
{
	int	err;

	DBUG_ENTER("test7");

	errno = NOERROR;
	if (((testrval = tb_nice(0)) == SYSERROR) && (errno != NOERROR))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("nice(0) returns %d and errno = %d (%s)", testrval, err, errname(err));
	}
	else
		(void) cppair(ch_t7, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int	err;
	int		rval;
	int		incr = NZERO;
	int		nice_max = NZERO-1;
	int	pathok = setupok;

	DBUG_ENTER("ch_t7");

	if (unsetprv(PRV_NICE) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: unsetprv(PRV_NICE) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((rval = tb_nice(incr)) != nice_max)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("expected nice() return value: %d", nice_max);
		in_rpt("observed nice() return value: %d", rval);
		in_rpt("observed error value = %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private void
test8()
{
	int	err;

	DBUG_ENTER("test8");

	errno = NOERROR;
	if (((testrval = tb_nice(0)) == SYSERROR) && (errno != NOERROR))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("nice(0) returns %d and errno = %d (%s)", testrval, err, errname(err));
	}
	else
		(void) cppair(ch_t8, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int	err;
	int	rval;
	int	decr = -(NZERO+1);
	int	nice_min = -NZERO;
	int	pathok = setupok;

	DBUG_ENTER("ch_t8");

	errno = NOERROR;
	if ((rval = tb_nice(decr)) != nice_min)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("expected nice() return value: %d", nice_min);
		in_rpt("observed nice() return value: %d", rval);
		in_rpt("observed error value = %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test9()
{
	int	err;

	DBUG_ENTER("test9");

	errno = NOERROR;
	if (((testrval = tb_nice(0)) == SYSERROR) && (errno != NOERROR))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("nice(0) returns %d and errno = %d (%s)", testrval, err, errname(err));
	}
	else
		(void) cppair(ch_t9, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int	err;
	int	rval;
	int	decr = -1;
	int	pathok = setupok;

	DBUG_ENTER("ch_t9");

	if (unsetprv(PRV_NICE) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: unsetprv(PRV_NICE) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (((rval = tb_nice(decr)) != SYSERROR) || (errno != EPERM && errno != EACCES))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("nice(-1) did not return expected values");
		valrpt(SYSERROR, rval, EPERM, err);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}


private	int
tb_nice(incr)
int	incr;
{
	int	retval;
	int	pathok = 0;

	PATH_TRACE;	/*	to prove that we have been called	*/

	retval = nice(incr);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
