/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/procenv/nice_X/nice_X1.c
*
* Modifications:
* $Log: nice_X1.c,v $
* Revision 1.2  2001/10/02 13:25:13  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:18  cyeoh
* Adds files for lsb-os test suite
*
*
*/

/*Exec-ed by nice_x.c as it requires privilege
to set scheduling priorities*/

#include	<vsrt/vsrt.h>
#include	<pthread.h>
#include	<sched.h>
#include	<setprv.h>

extern	int	setprv();
extern	char	*errname();

private void subtest1();

public  int tet_main(argc, argv)
int     argc;
char    **argv;
{
	subtest1();

	return 0;
}

int	errors = 0;

#define	NPOLS	2
int	policies[NPOLS] = {SCHED_FIFO, SCHED_RR};
char	*policy_names[NPOLS] = {"SCHED_FIFO", "SCHED_RR"};


private void subtest1()
{
	int	err;
	int	retval, maxprio, minprio;
	struct	sched_param	param;
	int	i, j;
#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
	pthread_t	self;
	int		policy;
#endif

	DBUG_ENTER("subtest1");

	if (setprv(PRV_SETRTSCHED) != 0) {
		in_rpt("setprv(PRV_SETRTSCHED) failed");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
       	}
	if (setprv(PRV_GETRTSCHED) != 0) {
		in_rpt("setprv(PRV_GETRTSCHED) failed");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
       	}
	if (setprv(PRV_NICE) != 0) {
		in_rpt("setprv(PRV_NICE) failed");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
       	}
#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
	if (setprv(PRV_SETTHRSCHED) != 0) {
		in_rpt("setprv(PRV_SETTHRSCHED) failed");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
       	}
	if (setprv(PRV_GETTHRSCHED) != 0) {
		in_rpt("setprv(PRV_GETTHRSCHED) failed");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
       	}
#endif

	errors = 0;

	for (j = 0; j < NPOLS; j++) {
		/*get the priority range*/
		maxprio = sched_get_priority_max(policies[j]);
		if (maxprio == -1) {
			vsrt_sysfail("sched_get_priority_max");
			DBUG_VOID_RETURN 
		}
		minprio = sched_get_priority_min(policies[j]);
		if (maxprio == -1) {
			vsrt_sysfail("sched_get_priority_min");
			DBUG_VOID_RETURN 
		}

#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
		self = pthread_self();
#endif

		for (i = minprio; i <= maxprio; i++) {
	
			/*set the priority */
			param.sched_priority = i;
			retval = sched_setscheduler(0, policies[j], &param);
			if (retval == -1) {
				err = errno;
				in_rpt("sched_setscheduler failed for %s with priority %d, errno = %d (%s)", policy_names[j], i, err, errname(err));
				xx_rpt(FAILURE);
				errors++;
				continue;
			}
#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
			param.sched_priority = i;
			retval = pthread_setschedparam(self, policies[j], &param);
			if (retval == -1) {
				err = errno;
				in_rpt("pthread_setschedparam failed for %s with priority %d, policy_names[j], errno = %d (%s)", i, err, errname(err));
				xx_rpt(FAILURE);
				errors++;
				continue;
			}
#endif
			
			/*make nice*/
			/* no error check here, some systems will not allow
			nicing at all with real time scheduling*/
			nice(1);
	
			/*check the priority*/
			retval = sched_getparam(0, &param);
			if (retval == -1) {
				err = errno;
				in_rpt("sched_getscheduler failed for %s with priority %d, errno = %d (%s)", policy_names[j], i, err, errname(err));
				xx_rpt(FAILURE);
				errors++;
				continue;
			}
	
			if (param.sched_priority != i) {
				in_rpt("For %s priority changed after nice(1), expected %d, received %d", policy_names[j], i, param.sched_priority);
				xx_rpt(FAILURE);
				errors++;
			}
	
#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
			retval = pthread_getschedparam(self, &policy, &param);
			if (retval == -1) {
				err = errno;
				in_rpt("pthread_getschedparam failed for %s with priority %d, errno = %d (%s)", policy_names[j], i, err, errname(err));
				xx_rpt(FAILURE);
				errors++;
				continue;
			}
			if (param.sched_priority != i) {
				in_rpt("For %s thread priority changed after nice(-1), expected %d, received %d", policy_names[j], i, param.sched_priority);
				xx_rpt(FAILURE);
				errors++;
			}
#endif

			nice(-1);
	
			/*check the priority*/
			retval = sched_getparam(0, &param);
			if (retval == -1) {
				err = errno;
				in_rpt("sched_getscheduler failed for %s with priority %d, errno = %d (%s)", policy_names[j], i, err, errname(err));
				xx_rpt(FAILURE);
				errors++;
				continue;
			}
	
			if (param.sched_priority != i) {
				in_rpt("For %s priority changed after nice(-1), expected %d, received %d", policy_names[j], i, param.sched_priority);
				xx_rpt(FAILURE);
				errors++;
			}
	
#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
			retval = pthread_getschedparam(self, &policy, &param);
			if (retval == -1) {
				err = errno;
				in_rpt("pthread_getschedparam failed for %s with priority %d, errno = %d (%s)", policy_names[j], i, err, errname(err));
				xx_rpt(FAILURE);
				errors++;
				continue;
			}
			if (param.sched_priority != i) {
				in_rpt("For %s thread priority changed after nice(-1), expected %d, received %d", policy_names[j], i, param.sched_priority);
				xx_rpt(FAILURE);
				errors++;
			}
#endif
		}
	}

	xs_rpt();

	DBUG_VOID_RETURN 
}
