/*
*      SCCS:  @(#)  XOPEN.os/procenv/putenv/putenv1.c	Rel 4.4.4     (07/19/99) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/procenv/putenv - putenv1 Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		putenv1 - used by T.putenv in comply()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.putenv to test putenv() by placing variables in the 
	  environment. The original environment was set up by the tet_exec 
	  call that executed this program.

	  usage:	putenv1 vg_ident vg_dbug

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Fri Oct 31 15:22:10 GMT 1986
MODIFICATIONS:	Andrew Twigger 12 June 87
		Added	vg_sjmp, setjmp(), tn_rpt()
		Removed	testid()
		Modified xx_rpt(), xs_rpt()

		J.A.Nave	11/05/90
		Added resilience testing

		Geoff Clare, 17 Oct 1990
			Use getenv() instead of vgetenv().

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Geoff Clare, 9 June 1999
			Add a check that altering the string changes
			the environment.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<stdlib.h>
#include	<string.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<sys/types.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef putenv
#endif


extern	void	set_pname();
extern	char	*atocp();
extern	char	**environ;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	void	test1();
private	void	test2();

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	int	pathok = 0;
 
	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"putenv1" : (argv[0]));

	if (argc == 4)
	{
		if (argv[3] != (char *) NULL)
			vg_dbug = argv[3];
		argc--;
	}
	if (argc == 3)
	{
		if (argv[2] != (char *) NULL)
			vg_ident = argv[2];
		argc--;
	}
	if (argc != 2)
	{
		vsxerror("argument count wrong");
		exit(A_MISC);
	}
	else
		PATH_TRACE;

	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	globok = (pathok == 1) ? 1 : 0;

	if (argv[1][0] == '1')
	{
		test1();
	}
	else
	{
		test2();
	}

	exit(OK);

	/*NOTREACHED*/
	return 0;
}


private void
test1()
{
	char	*cp;
	int	rval;
	int	pathok = 0;
	static char newstr[] = "ENV0=NEWVAL0";

	DBUG_ENTER("test1");

	if (globok == 1)
		PATH_TRACE;

	if ( (rval = putenv(newstr)) != 0 )
	{
		xx_rpt(DELETION);
		in_rpt("putenv returned %d instead of 0", rval);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( (cp = getenv("ENV0")) == (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("ENV0 no longer in environment after setting it in putenv");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (strcmp(cp, "NEWVAL0") != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("ENV0 previously set to VAL0");
		in_rpt("after putenv(\"ENV0=NEWVAL0\"), ENV0=%s", cp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Now change the string, to check putenv() didn't copy it */
	(void) strcpy(newstr, "ENV0=ALTVAL0");

	if ( (cp = getenv("ENV0")) == (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("ENV0 no longer in environment after altering string");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (strcmp(cp, "ALTVAL0") != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("after altering string, ENV0=%s, expected ALTVAL0", cp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	char	*cp;
	int	rval;
	int	pathok = 0;
	static char newstr[] = "ENV9=VAL9";

	DBUG_ENTER("test2");

	if (globok == 1)
		PATH_TRACE;

	if ( (rval = putenv(newstr)) != 0 )
	{
		xx_rpt(DELETION);
		in_rpt("putenv returned %d instead of 0", rval);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( (cp = getenv("ENV9")) == (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("ENV9 not in environment after setting it in putenv");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (strcmp(cp, "VAL9") != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("after putenv(\"ENV9=VAL9\"), ENV9=%s", cp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Now change the string, to check putenv() didn't copy it */
	(void) strcpy(newstr, "ENV8=VAL8");

	if ( (cp = getenv("ENV8")) == (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("ENV8 not in environment after altering string");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (strcmp(cp, "VAL8") != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("after altering string, ENV8=%s, expected VAL8", cp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(6);
	
	DBUG_VOID_RETURN;
}
