/*
*      SCCS:   @(#)XOPEN.os/procenv/ulimit/ulimit.c	Rel 4.4.1	(06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
static	char sccsid[] = "@(#)XOPEN.os/procenv/ulimit - T.ulimit Rel 4.4.1 (06/09/97)";
#ifdef	MAIN
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.ulimit

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the ulimit() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Manucher Ghezel-Ayagh, UniSoft Ltd.
DATE CREATED:	Tue Sep 16 12:37:24 BST 1986
MODIFICATIONS:	D McCardle Wed Jun 24 14:42:55 WET 1987
			Added vg_sjmp, setjmp(), tn_rpt().
			Removed abandon(), testid().
			Modified xx_rpt(), xs_rpt().

		J.A.Nave	25/05/90
		Added resilience testing

		Geoff Clare, 6 June 1990
		VSX_ULIMIT_BLKS == -1 means unsupported (not allowed here)

		Stuart Boutell, 2 Nov 1990
			execle replaced with tet_exec

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Geoff Clare, 23 Mar 1992
			Removed globok definition.

		Geoff Clare, 5 June 1997
			Changes for UNIX98 mode.

************************************************************************/
#include	<string.h>
#include	<std.h>
#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<ulimit.h>
#include	<signal.h>
#include	<synch.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ulimit
#endif


#define NOK		2		/* child exit value if non fatal error occured */
#define NO_TESTS	6		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)
#define	VSX_ULIMIT_BLKS	"VSX_ULIMIT_BLKS"
#define EXTERR		"external error"
#define GETBRK		3		/* cmd value for the maximum brk size */
#define MASK		0000		/* file mask */
#define MODE		0666		/* file mode */
#define OFLAG		O_RDONLY	/* open flag : read only */
#define BLKSIZE		512		/* block size */

#define READ		0
#define WRITE		1

struct res1 {
	int		err;
	int		call;
	long		rval;
	int		pathok;
} ;
struct result {
	int		rval1, rval2, rval3;
	int		err1, err2, err3;
	int		nblk, ncar;
	int		failed1, failed2, failed3, failed4;
	int		pathok;
} ;

extern	char	*itoa();
extern	char	*errname();
extern	void	valrpt();
extern	synch	*opensync();
extern	void	(*vsignal())();

extern	char	*vg_ident;

int	no_tests = NO_TESTS;
private	int	setupok;

private	int	t2_fd, t2_fd1;
private struct result t2_res;
private long	ulimblks;
private char	buf[BLKSIZE];
private char	*prog4 =	"../ulimit_t4";
private char	*t_file =	"./ulimit-t.1";
private char	*t2_file =	"./ulimit-t.2";
private synch	*pipeline;


private void	prepare_tests();
private void	cleantests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private	void	ch_t1();
private	void	ch_t2();
private	void	ch_t3();
private	void	ch_t4();
private	void	pa_t1();
private	void	pa_t2();
private	void	pa_t3();
private	long	tb_ulimit();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cleantests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	NULL, 0
};

private void
prepare_tests()
{
	char	*ulimaddr;
	int	fd;
	int	i;
	long	j;
	int	pathok = 0;

	DBUG_ENTER("preparetests");

	ulimaddr = tet_getvar(VSX_ULIMIT_BLKS);
	if (ulimaddr == NULL || *ulimaddr == '\0')
	{
		for (i=1; i<= no_tests; i++)
			(void) tet_delete(i, "VSX_ULIMIT_BLKS parameter not set");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		ulimblks = atol(ulimaddr);
		if (ulimblks == -1)
		{
			for (i=1; i<= no_tests; i++)
				(void) tet_delete(i, "VSX_ULIMIT_BLKS parameter must not be set to -1");
			DBUG_VOID_RETURN;
		}
		if (ulimblks < 4L)
			ulimblks = 4L;
	}

	(void) umask (MASK);
	if ((fd = crfile(t_file, (mode_t)MODE, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i, "test preparation: creat failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (j = 0; j <= (ulimblks*2L) ; j++)
	{
		if (j == 0)
			PATH_TRACE;

		if (write(fd, buf, (unsigned) BLKSIZE) != BLKSIZE)
		{
			for (i = 1; i <= no_tests; i++)
				(void) tet_delete(i, "test preparation: write failed");
			break;
		}
	}

	setupok = (pathok == 3) ? 0 : -1;

	(void) close(fd);

	errno = NOERROR;

	DBUG_VOID_RETURN;
}

private void
cleantests()
{
	DBUG_ENTER("cleantests");

	(void) unlink(t_file);

	DBUG_VOID_RETURN;
}


private void
test1()
{
	int		j;

	DBUG_ENTER("test1");

	if ((pipeline = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		for (j = 2; j <= 4; j++)
			(void) tet_delete(j, "depends on success of test 1");
	}
	else
	{
		(void) cppair(ch_t1, pa_t1, WAITTIME, 0);
		(void) closesync(pipeline);
	}

	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	struct res1	results;
	int	j;
	int	pathok = setupok;

	DBUG_ENTER("pa_t1");

	(void) close(pipeline[0][WRITE]);
	if (waitsync(pipeline[0][READ], (char *)&results, sizeof(results), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		for (j = 2; j <= 4; j++)
			(void) tet_delete(j, "depends on success of test 1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (results.pathok == 2)
		PATH_TRACE;

	if (results.call == 1)
	{
		xx_rpt(FAILURE);
		in_rpt("ulimit(%d, %ld) failed with errno = %d (%s)",
			UL_SETFSIZE, ulimblks*2L, results.err, errname(results.err));
		for (j = 2; j <= 4; j++)
			(void) tet_delete(j, "depends on success of test 1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (results.call != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("ulimit(%d) returns %ld and errno = %d (%s)", UL_GETFSIZE, results.rval, results.err, errname(results.err));
		in_rpt("expected filesize : %ld blocks", ulimblks*2L);
		for (j = 2; j <= 4; j++)
			(void) tet_delete(j, "depends on success of test 1");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}
	
	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	struct res1	results;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	(void) close(pipeline[0][READ]);
	results.call = 0;
	results.rval = 0;
	results.err  = 0;
	if ((results.rval = tb_ulimit(UL_SETFSIZE, ulimblks*2L)) == (long) SYSERROR)
	{
		results.err = errno;
		results.call = 1;
	}
	else
	{
		PATH_TRACE;
		errno = NOERROR;
		if ((results.rval = tb_ulimit(UL_GETFSIZE, (long) 0)) != ulimblks*2)
		{
			results.err = errno;
			results.call = 2;
		}
		else
			PATH_TRACE;
	}

	results.pathok = pathok;	/* inform parent of child's pathok */

	if (sendsync(pipeline[0][WRITE], (char *)&results, sizeof(results), WAITTIME) == SYSERROR)
		exit(A_MISC);

	exit(OK);

	/* NOT REACHED */
	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	err;
	struct	stat statbuf;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	errno = NOERROR;
	if ((t2_fd = open(t_file, OFLAG)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s) failed - errno: %d (%s)", t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((t2_fd1 = crfile(t2_file, (mode_t)MODE, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s) failed - errno: %d (%s)", t2_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((pipeline = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		(void) close(t2_fd);
		(void) close(t2_fd1);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (cppair(ch_t2, pa_t2, WAITTIME, 0) == -1)
	{
		/* already reported by cppair() */
		(void) closesync(pipeline);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	(void) closesync(pipeline);

	if (t2_res.pathok == 3)
		PATH_TRACE;

	errno = NOERROR;
	if (fstat(t2_fd1, &statbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat(%s) failed - errno: %d (%s)", t2_file, err, errname(err));
		(void) close(t2_fd);
		(void) close(t2_fd1);
		(void) unlink(t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((long) ulimblks*2*BLKSIZE  != (long) statbuf.st_size)
		t2_res.failed4++;
	else
		PATH_TRACE;

	if (t2_res.failed1 || t2_res.failed2 || t2_res.failed3 || t2_res.failed4)
	{
		xx_rpt(FAILURE);
		in_rpt("maximum file size : %ld blocks", ulimblks*2L);
		if (t2_res.failed1)
			in_rpt("read() failed on %d th block, errno = %d (%s)", t2_res.nblk, t2_res.err1, errname(t2_res.err1));
		if (t2_res.failed2)
		{
			in_rpt("write() failed before reaching the maximum file size");
			in_rpt("write() returned %d on %d th byte, errno = %d (%s)",
					t2_res.rval2, t2_res.ncar, t2_res.err2, errname(t2_res.err2));
		}
		if (t2_res.failed3)
		{
			in_rpt("write() beyond the file size limit did not fail");
			valrpt(SYSERROR, t2_res.rval3, EFBIG, t2_res.err3);
		}
		if (t2_res.failed4)
		{
			in_rpt("actual file size - expected: %ld, observed: %ld",
				BLKSIZE*ulimblks*2L, (long) statbuf.st_size);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(8);
	}

	(void) close(t2_fd);
	(void) close(t2_fd1);
	(void) unlink(t2_file);

	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	int	pathok = 0;

	DBUG_ENTER("pa_t2");

	(void) close(pipeline[0][WRITE]);
	if (waitsync(pipeline[0][READ], (char *)&t2_res, sizeof(t2_res), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = (pathok == 1) ? 1 : 0;

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	struct result	res;
	int	j;
	char	car;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t2");

	res.nblk = res.ncar = 0;
	res.failed1 = res.failed2 = res.failed3 = res.failed4 = 0;
	(void) close(pipeline[0][READ]);
	(void) tb_ulimit(UL_SETFSIZE, ulimblks*2);

	for (j = 0; j <= ulimblks*2L; j++)
	{
		if (j == 0)
			PATH_TRACE;

		DBUG_PRINTF("ch_t2", "j :%d:", j);
		errno = NOERROR;
		if ((res.rval1 = read(t2_fd, buf, BLKSIZE)) == SYSERROR)
		{
			res.err1 = errno;
			res.nblk = j+1;
			res.failed1++;
			break;
		}
	}

#if TEST_UNIX
	(void) vsignal(SIGXFSZ, SIG_IGN);
#endif

	for (j = 0; j < ulimblks*BLKSIZE*2L; j++)
	{
		if (j == 0)
			PATH_TRACE;

		if ((res.rval2 = write(t2_fd1, &car, (unsigned) sizeof (char))) != sizeof (char))
		{
			res.err2 = errno;
			res.ncar = j+1;
			res.failed2++;
			break;
		}
	}

	errno = NOERROR;
	if (!res.failed2)
	{
		if (((res.rval3 = write(t2_fd1, &car, (unsigned) sizeof (char))) != SYSERROR) ||
			(errno != EFBIG))
		{
			res.err3 = errno;
			res.failed3++;
		}
		else
			PATH_TRACE;
	}

	res.pathok = pathok;	/* inform parent of child's pathok */

	if (sendsync(pipeline[0][WRITE], (char *)&res, sizeof(res), WAITTIME) == SYSERROR)
		exit(A_MISC);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	if ((pipeline = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
	}
	else
	{
		(void) cppair(ch_t3, pa_t3, WAITTIME, 0);
		(void) closesync(pipeline);
	}

	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	struct res1 results;
	int	pathok = setupok;

	DBUG_ENTER("pa_t3");

	(void) close(pipeline[0][WRITE]);
	if (waitsync(pipeline[0][READ], (char *)&results, sizeof(results), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (results.pathok == 2)
		PATH_TRACE;

	if (results.call == 1)
	{
		xx_rpt(FAILURE);
		in_rpt("ulimit(%d) returns %ld and errno = %d (%s)", UL_SETFSIZE, results.rval, results.err, errname(results.err));
		in_rpt("expected filesize : %ld blocks", ulimblks);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (results.call == 2)
	{
		xx_rpt(FAILURE);
		in_rpt("ulimit(%d) returns %ld when set to %ld",
			UL_GETFSIZE, results.rval, ulimblks);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	struct res1 results;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	(void) close(pipeline[0][READ]);
	results.call = 0;
	results.rval = 0;
	results.err  = 0;
	if ((results.rval =  tb_ulimit(UL_SETFSIZE, ulimblks)) == (long) SYSERROR)  
	{
		results.err = errno;
		results.call = 1;
	}
	else
	{
		PATH_TRACE;
		errno = NOERROR;
		if ((results.rval = tb_ulimit(UL_GETFSIZE, (long) 0)) != ulimblks)
		{
			results.err = errno;
			results.call = 2;
		}
		else
			PATH_TRACE;
	}

	results.pathok = pathok;	/* inform parent of child's pathok */

	if (sendsync(pipeline[0][WRITE], (char *)&results, sizeof(results), WAITTIME) == SYSERROR)
		exit(A_MISC);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	(void) cppair(ch_t4, NULL, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	extern	char	**environ;
	char	*args[4];
	DBUG_ENTER("ch_t4");

	(void) tb_ulimit(UL_SETFSIZE, tb_ulimit(UL_GETFSIZE, (long) 0) / 2);
	args[0] = prog4;
	args[1] = itoa(setupok);
	args[2] = vg_ident;
	args[3] = (char *)NULL;
	(void) tet_exec(prog4, args, environ);
	exit(A_EXECFAIL);

	/* NOT REACHED */

	DBUG_VOID_RETURN;
}

private void
test5()
{
	int		err;
	long		rval;
	int	pathok = setupok;

	DBUG_ENTER("test5");

	if ((int) geteuid() == 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: should not be super user");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) tb_ulimit(UL_SETFSIZE, tb_ulimit(UL_GETFSIZE, (long) 0) / 2);
	errno = NOERROR;
	if (((rval = tb_ulimit(UL_SETFSIZE, 2 * tb_ulimit(UL_GETFSIZE, (long) 0))) != (long) SYSERROR)
		|| (errno != EPERM))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("attempt by process (effective UID not super-user) to increase its file size limit using ulimit() did not return expected values");
		valrpt(SYSERROR, (int) rval, EPERM, err);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private void
test6()
{
	int	err;
	long	rval;
	int	illval = ((256*UL_SETFSIZE) + (16*UL_GETFSIZE) + 1);
	int	pathok = setupok;

	DBUG_ENTER("test6");

	errno = NOERROR;
	if (((rval = tb_ulimit(illval, (long) 0)) != (long) SYSERROR)
		|| (errno != EINVAL))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("attempt to use illegal command option to ulimit() did not return expected values");
		in_rpt("value of illegal option = %d (UL_SETFSIZE+UL_GETFSIZE+1)", illval);
		valrpt(SYSERROR, (int) rval, EINVAL, err);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private	long
tb_ulimit(cmd, newlimit)
int	cmd;
long 	newlimit;
{
	long	retval;

	retval = ulimit(cmd, newlimit);
#ifdef	TESTBED
	errno++;
	return (long) -1;
#else
	return retval;
#endif
}
