/*
*      SCCS:  @(#)  XOPEN.os/procenv/ulimit/ulimit_t4.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/procenv/ulimit - ulimit_t4 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		ulimit_t4 - used by T.ulimit in test4()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.ulimit to test the super user ability to increase the
	file size limit.

	  usage:
		ulimit_t4 vg_ident 

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Manucher Ghezel_Ayagh, UniSoft Ltd.
DATE CREATED:	Tue Sep 16 17:18:58 BST 1986
MODIFICATIONS:	D McCardle Wed Jun 24 16:19:56 WET 1987
			Removed testid().
			Modified xx_rpt(), xs_rpt().

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Peter Skan, 27 Sept 1991
			Added setprv(PRV_ULIMIT)

************************************************************************/
#include <std.h>
#include <dbug.h>
#include <exitcodes.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <ulimit.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include <setprv.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef ulimit
#endif


public	char	*vg_ident =	"<no ident>";

extern	char	*errname();
extern	void	set_pname();

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	int	err;
	long	plimit, nlimit;
	int	pathok = 0;

	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"ulimit_t4" : (argv[0]));
	if (argc == 3)
	{
		if (argv[2] != (char *) NULL)
			vg_ident = argv[2];
		argc--;
	}
	pathok = atoi(argv[1]);
	if (setprv(PRV_ULIMIT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_ULIMIT) failed");
		exit(OK);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((plimit = ulimit(UL_GETFSIZE, (long) 0)) == (long) SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("ulimit(%d) failed with errno: %d  (%s)", UL_GETFSIZE, err, errname(err));
		exit(OK);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((int) ulimit(UL_SETFSIZE, 2 * plimit) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("super user attempt to double its file size limit failed");
		in_rpt("ulimit(%d) failed with errno = %d (%s)", UL_SETFSIZE, err, errname(err));
		exit(OK);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((nlimit = ulimit(UL_GETFSIZE, (long) 0)) != (2 * plimit))
	{
		xx_rpt(FAILURE);
		in_rpt("ulimit(%d) returns %ld when %ld expected",
				UL_GETFSIZE, nlimit, (2 * plimit));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}

	exit(OK);
	/*NOTREACHED*/
	return 0;
}
