/* dv @(#)V9TS/tset/GABI.os/procprim/exec_A/exec_A.c	1.6 98/08/12 */
/*
*	SCCS:  @(#) GABI.os/procprim/exec_A/exec_A.c	Ver. 1.11	Rel. 1.3	(92/08/01)
*
*
* (C) Copyright 1990,1991 UniSoft Group Ltd.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#) GABI.os/procprim/exec_A/exec_A.c	Ver. 1.11	Rel. 1.3	(92/08/01)";
static	char	*copyright[] = {
		"(C) Copyright 1990,1991 UniSoft Group Ltd.",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.exec_A

PROJECT:	GABI (Generic ABI Validation Suite)	

DESCRIPTION:	Testset for the exec() system call, gABI extensions.

AUTHOR:		Miha Wang, UniSoft Corp.
DATE CREATED:	Tue Oct 23 12:48:26 PDT 1990	
MODIFICATIONS:
		John-Paul, 7 April 1993
			Added in pipe reporting code.

		Andrew Dingwall, UniSoft Ltd., September 1995
		fix to bug 477
		Corrected some path tracing code.

		Andrew Dingwall, UniSoft Ltd., September 1995
		allow one more open file in test2 for the benefit of
		systems whose crt function uses two open file descriptors

		Andrew Twigger, UniSoft Ltd., December 1996
		Decrement limits by 10 rather than 1.  Decrementing by
		one uses valuespace now occupied by RLIM_ constants
		associated with large file system.


************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<errno.h>
#include	<string.h>
#include	<sys/types.h>
#include	<sys/resource.h>
#include	<sys/times.h>
#include	<sys/wait.h>
#include	<unistd.h>
#include	<timeout.h>
#include	<wait_child.h>
#include	<exec.h>
#include	<dbug.h>
#include	<report.h>
#include	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif

#define	NAME()		(i_name[interface])
#define	WAITTIME	(5 * SPEEDFACTOR)
#if 0
#define	NO_TESTS	3		/* number of tests in testset */
#else
#define	NO_TESTS	1		/* number of tests in testset */
#endif


extern	void		vsx_sigs();
extern	void		valrpt();
extern	char		*itoa();
extern	char		*newstr();
extern	char		*status_child();
extern	char		*errname();
extern	int		vputenv();
extern	int		cppair();
extern	int		wait_child();
extern	int		cr_sym_loop();
extern  int     	n_open_fds();
extern  int     	setrlimit();
extern  int     	getrlimit();

extern  int             pipe_open();
extern  void            pipe_close();
extern  void            pipe_parent();
extern  void            pipe_report();
extern  void            pipe_child();

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;
extern	int	sig_array[];
extern	int	ex_code;	/* defined in wait_child.c */
extern  char    *vg_ident;

int	no_tests = NO_TESTS;

private struct	rlimit	rlim_name[RLIM_NLIMITS];		
private char	*p_argv[MAXARG];
private char	**p_envp;
private int	interface;
private char	*i_name[6] = {
				"execl"
				"execle"
				"execlp"
				"execv"
				"execve"
				"execvp"
			    };

private	char	*exec_tsh;
private	int	testfail;
private	int	setupok = -1;


#if 0
private	void		test1();
private	void		ch1_t1();
private	void		ch2_t1();
#endif
private	void		test2();
private	void		ch_t2();
#if 0
private	void		test3();
private	void		ch_t3();
#endif
private	void		before_exec();
private	void		prepare_tests();
private	void		tb_exec();
private	void		tb_exec_ch();
private	void		tb_setrlimit();

#if 0
private	char	*t3_dira = 	"./d-exec_A-t3a";
private	char	*t3_dirb = 	"./d-exec_A-t3b";
#endif

private char	rptbuf[256];

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
#if 0
	 test1, 1,
	 test2, 2,
	 test3, 3,
#else
	test2, 1,
#endif
	NULL, 0
};

private	void
prepare_tests()
{
	int	i;
	char	*s, *p;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	/*
	** Get current process name
	*/
	if((s = strrchr(vg_ident, '/')) != (char *)NULL)
		s++;
	else
		s = vg_ident;

	/* 
	** Which one we are running
	*/ 
	if (strcmp(s, "T.execl_A" ) == 0)
		interface = I_EXECL;
	else if (strcmp(s, "T.execle_A") == 0)
		interface = I_EXECLE;
	else if (strcmp(s, "T.execlp_A") == 0)
		interface = I_EXECLP;
	else if (strcmp(s, "T.execv_A" ) == 0)
		interface = I_EXECV;
	else if (strcmp(s, "T.execve_A") == 0)
		interface = I_EXECVE;
	else if (strcmp(s, "T.execvp_A") == 0)
		interface = I_EXECVP;
	else
	{
		vsxerror("%s: is %s, should be exec{l,le,lp,v,ve,vp}_A",
			"testset id not set correctly", vg_ident);
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i,"testset id not set correctly" );
		DBUG_VOID_RETURN;
	}

	/*
	 *	Setup the environment path 
	 */
	if ((s = tet_getvar("VSX_PATH")) == (char *) NULL)
		s = "::/bin:/usr/bin";
	if ((p = (char *)malloc((size_t) (strlen(s) + strlen(EXEC_PATH) + 1)))
						== (char *) NULL)
	{
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i, "prepare() failed to malloc memory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) strcat(strcpy(p, EXEC_PATH), s);
	if (vputenv(&environ, p) == SYSERROR)
	{
		vsxerror("%s to %s", "could not set environment path ", p);
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i, "could not set environment path ");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((p_envp = (char **) malloc(sizeof(char *))) == (char **) NULL) {
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i, "prepare() failed to malloc memory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	p_envp[0] = (char *) NULL;

	PATH_SETUP_RPT(3);
	DBUG_VOID_RETURN;
}

#if 0
private void
test1()
{
	int	ret;
	int	pathok = setupok;

	/*
	** Assertion 1 when exec() executes the interpreter  
	** file with pathname and optional argument specified 
	** in the first line 
	*/
	DBUG_ENTER("test1");

	testfail = 0;


	if( pipe_open() == 0)
		testfail++;
	else
	{
		PATH_TRACE;
		exec_tsh = EXEC_TSH1;
		if ((ret=cppair(ch1_t1, NULLFN, WAITTIME, NOK|PATH_NOK)) == NOK)
			testfail++;
		else if (ret != PATH_NOK)
			PATH_TRACE;

		pipe_parent();
		pipe_close();
	}
		/*
		** Assertion 1 when exec() executes the interpreter  
		** file with only pathname (no optional argument) 
		** specified in the first line 
		*/

	if( pipe_open() == 0)
		testfail++;
	else
	{
		PATH_TRACE;
		exec_tsh = EXEC_TSH2;

		if ((ret=cppair(ch2_t1, NULLFN, WAITTIME, NOK|PATH_NOK)) == NOK)
			testfail++;
		else if (ret != PATH_NOK)
			PATH_TRACE;

		pipe_parent();
		pipe_close();
	}

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch1_t1()
{

	DBUG_ENTER("ch1_t1");

	pipe_child();

	p_argv[0] = exec_tsh;
	p_argv[1] = P_ARG1;
	p_argv[2] = P_ARG2;
	p_argv[3] = P_ARG3;
	p_argv[4] = P_ARG4;
	p_argv[5] = P_ARG5;
	p_argv[6] = P_ARG6;
	p_argv[7] = P_ARG7;
	p_argv[8] = P_ARG8;
	p_argv[9] = P_ARG9;
	p_argv[10] = P_ARG10;
	p_argv[11] = P_ARG11;
	p_argv[12] = P_ARG12;
	p_argv[13] = P_ARG13;
	p_argv[14] = P_ARG14;
	p_argv[15] = P_ARG15;
	p_argv[16] = (char *)NULL;

	tb_exec(1);

	/*
	** Should never get here if exec succeed.
	*/

	if (!testfail++)
		pipe_xx_rpt(FAILURE);
	pipe_in_rpt("exec failed unexpectedly");

	pipe_end_rpt();

	DBUG_EXIT(NOK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
ch2_t1()
{

	DBUG_ENTER("ch2_t1");

	pipe_child();

	p_argv[0] = exec_tsh;
	p_argv[1] = P_ARG1;
	p_argv[2] = P_ARG2;
	p_argv[3] = P_ARG3;
	p_argv[4] = P_ARG4;
	p_argv[5] = P_ARG5;
	p_argv[6] = P_ARG6;
	p_argv[7] = P_ARG7;
	p_argv[8] = P_ARG8;
	p_argv[9] = P_ARG9;
	p_argv[10] = P_ARG10;
	p_argv[11] = P_ARG11;
	p_argv[12] = P_ARG12;
	p_argv[13] = P_ARG13;
	p_argv[14] = P_ARG14;
	p_argv[15] = P_ARG15;
	p_argv[16] = (char *)NULL;

	tb_exec(1);

	/*
	** Should never get here if exec succeed.
	*/
	if (!testfail++)
		pipe_xx_rpt(FAILURE);
	pipe_in_rpt("exec failed unexpectedly");

	pipe_end_rpt();

	DBUG_EXIT(NOK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}
#endif

private void
test2()
{
	int	ret;
	int	pathok = setupok;

	/*
	** Assertion 2 on a successful call to exec() the system
	** resource limits are inherited by the new process image 
	*/
	DBUG_ENTER("test2");

	testfail = 0;
	exec_tsh = EXEC_TGEN;

	if( pipe_open() == 0)
		testfail++;
	else
	{
		PATH_TRACE;

		if ((ret=cppair(ch_t2, NULLFN, WAITTIME, NOK|PATH_NOK)) == NOK)
			testfail++;
		else if (ret != PATH_NOK)
			PATH_TRACE;

		pipe_parent();
		pipe_close();
	}

	if (testfail ==0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{

	DBUG_ENTER("ch_t2");

	/* Set up resource limits values prior to exec() */

	pipe_child();

	before_exec();

	p_argv[0] = exec_tsh;
	p_argv[1] = newstr(itoa(rlim_name[RLIMIT_CPU].rlim_cur));
	p_argv[2] = newstr(itoa(rlim_name[RLIMIT_CPU].rlim_max));
	p_argv[3] = newstr(itoa(rlim_name[RLIMIT_FSIZE].rlim_cur));
	p_argv[4] = newstr(itoa(rlim_name[RLIMIT_FSIZE].rlim_max));
	p_argv[5] = newstr(itoa(rlim_name[RLIMIT_DATA].rlim_cur));
	p_argv[6] = newstr(itoa(rlim_name[RLIMIT_DATA].rlim_max));
	p_argv[7] = newstr(itoa(rlim_name[RLIMIT_STACK].rlim_cur));
	p_argv[8] = newstr(itoa(rlim_name[RLIMIT_STACK].rlim_max));
	p_argv[9] = newstr(itoa(rlim_name[RLIMIT_CORE].rlim_cur));
	p_argv[10] = newstr(itoa(rlim_name[RLIMIT_CORE].rlim_max));
	p_argv[11] = newstr(itoa(rlim_name[RLIMIT_NOFILE].rlim_cur));
	p_argv[12] = newstr(itoa(rlim_name[RLIMIT_NOFILE].rlim_max));
#if 0
	p_argv[13] = newstr(itoa(rlim_name[RLIMIT_VMEM].rlim_cur));
	p_argv[14] = newstr(itoa(rlim_name[RLIMIT_VMEM].rlim_max));
#else
	p_argv[13] = newstr(itoa(rlim_name[RLIMIT_AS].rlim_cur));
	p_argv[14] = newstr(itoa(rlim_name[RLIMIT_AS].rlim_max));
#endif
	p_argv[15] = P_ARG15;
	p_argv[16] = (char *)NULL;

	tb_exec(2);

	/*
	** Should never get here if exec succeed.
	*/

	if (!testfail++)
		pipe_xx_rpt(FAILURE);
	pipe_in_rpt("exec failed unexpectedly");

	pipe_end_rpt();

	DBUG_EXIT(NOK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

#if 0
private void
test3()
{
	int	pathok = setupok;

	/*
	** Assertion 3 - If the path has too many links  
	** ELOOP in errno and -1 are returned 
	*/
	DBUG_ENTER("test3");

	testfail = 0;

	if( pipe_open() == 0)
		DBUG_VOID_RETURN;

	if( cppair(ch_t3, NULLFN, WAITTIME, 0) == SYSERROR) {
		DBUG_VOID_RETURN;
	}

	pipe_parent();
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	pipe_child();

	if (cr_sym_loop(t3_dira, t3_dirb) != SUCCEED)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"cr_sym_loop(\"%s\", \"%s\") failed - errno %d (%s)", t3_dira, t3_dirb, err, errname(err));
		pipe_in_rpt(rptbuf);

		pipe_end_rpt();

		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	exec_tsh = t3_dira;
	p_argv[0] = t3_dira;

	tb_exec(3);

	(void)unlink(t3_dira);
	(void)unlink(t3_dirb);

	if (testfail == 0)
		PATH_XS_RPT(1);
	
	pipe_end_rpt();

	DBUG_EXIT(OK);
}
#endif

private void
before_exec()
{
	struct	tms	t;
	struct	rlimit	rget;
	rlim_t	set_to;
	char	*pc;
	int	err, num, vub;
	int	hard, soft;
	long	clk_tck;
	int	pathok = 0;

	/*
	** Change the system resource limits
	*/  

	DBUG_ENTER("before_exec");



	/*
	** Set RLIMIT_CPU
	*/

	errno = NOERROR;
	clk_tck = sysconf(_SC_CLK_TCK);
	if (clk_tck == SYSERROR)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"sysconf(_SC_CLK_TCK) failed - errno %d (%s)", err, errname(err));
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;  
	}	
	else
		PATH_TRACE;

	if (times(&t) == SYSERROR)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"times() failed - errno %d (%s)", err, errname(err));
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;  
	}	
	else
		PATH_TRACE;

	hard = ((t.tms_utime + t.tms_stime) / clk_tck) + HARD_TIME;
	soft = ((t.tms_utime + t.tms_stime) / clk_tck) + SOFT_TIME;

	globok = 0;
	tb_setrlimit(RLIMIT_CPU, "RLIMIT_CPU", (rlim_t)hard, (rlim_t)soft);
	PATH_FUNC_TRACE;

	rlim_name[RLIMIT_CPU].rlim_cur = soft;
	rlim_name[RLIMIT_CPU].rlim_max = hard;

	/*
	** Set RLIMIT_FSIZE
	*/
	
	if (((pc = tet_getvar("VSX_ULIMIT_BLKS")) == NULL) || (*pc == '\0')) 
	{
		pipe_xx_rpt(DELETION);
		pipe_in_rpt("VSX_ULIMIT_BLKS unset or NULL");
		DBUG_VOID_RETURN;  
	}
	else
		PATH_TRACE;

	vub = atoi(pc);
	if (vub < 4)
		vub = 4; 
	if (vub > 40)
		vub = 40; 

	hard = vub * BLKSIZE * 2;
	soft = vub * BLKSIZE;

	globok = 0;
	tb_setrlimit(RLIMIT_FSIZE, "RLIMIT_FSIZE", (rlim_t)hard, (rlim_t)soft);
	PATH_FUNC_TRACE;

	rlim_name[RLIMIT_FSIZE].rlim_cur = soft;
	rlim_name[RLIMIT_FSIZE].rlim_max = hard;

	/*
	** Set RLIMIT_DATA
	*/
	
	errno = NOERROR;
	if (getrlimit(RLIMIT_DATA, &rget) == SYSERROR)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"first call to getrlimit(RLIMIT_DATA) failed - errno %d (%s)", err, errname(err));
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;  
	}
	else
		PATH_TRACE;

	if (rget.rlim_cur < SIZE_DATA)
		set_to = rget.rlim_cur - 10;
	else
		set_to = SIZE_DATA;

	globok = 0;
	tb_setrlimit(RLIMIT_DATA, "RLIMIT_DATA", set_to, set_to);
	PATH_FUNC_TRACE;

	rlim_name[RLIMIT_DATA].rlim_cur = set_to;
	rlim_name[RLIMIT_DATA].rlim_max = set_to;

	/*
	** Set RLIMIT_STACK
	*/
	
	err = NOERROR;
	if (getrlimit(RLIMIT_STACK, &rget) == SYSERROR)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"first call to getrlimit(RLIMIT_STACK) failed - errno %d (%s)", err, errname(err));
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;  
	}
	else
		PATH_TRACE;

	set_to = rget.rlim_cur - 10;
	
	globok = 0;
	tb_setrlimit(RLIMIT_STACK, "RLIMIT_STACK", set_to, set_to);
	PATH_FUNC_TRACE;

	rlim_name[RLIMIT_STACK].rlim_cur = set_to;
	rlim_name[RLIMIT_STACK].rlim_max = set_to;

	/*
	** Set RLIMIT_CORE
	*/

	globok = 0;
	tb_setrlimit(RLIMIT_CORE, "RLIMIT_CORE", (rlim_t)SOFT_CORE, (rlim_t)SOFT_CORE);
	PATH_FUNC_TRACE;

	rlim_name[RLIMIT_CORE].rlim_cur = SOFT_CORE;
	rlim_name[RLIMIT_CORE].rlim_max = SOFT_CORE;

	/*
	** Set RLIMIT_NOFILE
	*/

	num = n_open_fds();

	soft = num + 2;	
	hard = num + 3;	
	
	globok = 0;
	tb_setrlimit(RLIMIT_NOFILE, "RLIMIT_NOFILE", (rlim_t)hard, (rlim_t)soft);
	PATH_FUNC_TRACE;

	rlim_name[RLIMIT_NOFILE].rlim_cur = soft;
	rlim_name[RLIMIT_NOFILE].rlim_max = hard;

#if 0
	/*
	** Set RLIMIT_VMEM
	*/

	err = NOERROR;
	if (getrlimit(RLIMIT_VMEM, &rget) == SYSERROR)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"first call to getrlimit(RLIMIT_VMEM) failed - errno %d (%s)", err, errname(err));
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;  
	}
	else
		PATH_TRACE;

	set_to = rget.rlim_cur - 10;
	
	globok = 0;
	tb_setrlimit(RLIMIT_VMEM, "RLIMIT_VMEM", set_to, set_to);
	PATH_FUNC_TRACE;

	rlim_name[RLIMIT_VMEM].rlim_cur = set_to;
	rlim_name[RLIMIT_VMEM].rlim_max = set_to;
#else
	/*
	** Set RLIMIT_AS
	*/

	err = NOERROR;
	if (getrlimit(RLIMIT_AS, &rget) == SYSERROR)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"first call to getrlimit(RLIMIT_AS) failed - errno %d (%s)", err, errname(err));
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;  
	}
	else
		PATH_TRACE;

	set_to = rget.rlim_cur - 10;
	
	globok = 0;
	tb_setrlimit(RLIMIT_AS, "RLIMIT_AS", set_to, set_to);
	PATH_FUNC_TRACE;

	rlim_name[RLIMIT_AS].rlim_cur = set_to;
	rlim_name[RLIMIT_AS].rlim_max = set_to;
#endif

	PATH_FUNC_RPT(13);
	DBUG_VOID_RETURN;
}




private void 
tb_exec(whichtest)
int	whichtest;	
{
	int	ret, err;
	int	pathok = 0;

	DBUG_ENTER("tb_exec_ch");

	err = NOERROR;
	switch (interface)
	{
	case I_EXECL:
		ret = execl(exec_tsh, p_argv[0], p_argv[1], p_argv[2], 
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			p_argv[7], p_argv[8], p_argv[9], p_argv[10], 
			p_argv[11], p_argv[12], p_argv[13], p_argv[14], 
			p_argv[15], (char *) NULL);
		break;
	case I_EXECLE:
		ret = execle(exec_tsh, p_argv[0], p_argv[1], p_argv[2], 
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			p_argv[7], p_argv[8], p_argv[9], p_argv[10], 
			p_argv[11], p_argv[12], p_argv[13], p_argv[14], 
			p_argv[15], (char *) NULL, p_envp);
		break;
	case I_EXECLP:
		ret = execlp(exec_tsh, p_argv[0], p_argv[1], p_argv[2], 
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			p_argv[7], p_argv[8], p_argv[9], p_argv[10], 
			p_argv[11], p_argv[12], p_argv[13], p_argv[14], 
			p_argv[15], (char *) NULL);
		break;
	case I_EXECV:
		ret = execv(exec_tsh, p_argv);
		break;
	case I_EXECVE:
		ret = execve(exec_tsh, p_argv, p_envp);
		break;
	case I_EXECVP:
		ret = execvp(exec_tsh, p_argv);
		break;
	}
	if (whichtest == 1 || whichtest == 2)
	{
		/* for test1 and test1 */ 
		if (ret == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			(void)sprintf(rptbuf,"%s(%s, ...) returned unexpectedly - errno %d (%s) ", NAME(), exec_tsh, err, errname(err));
			in_rpt(rptbuf);
			DBUG_VOID_RETURN;
		}			
		else
			PATH_TRACE;
	}
	else
	{
		/* for test3 */ 
		err = errno;
		if (ret != SYSERROR || err != ELOOP)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			(void)sprintf(rptbuf,"%s(%s, ...) did not give correct results", NAME(), exec_tsh);
			in_rpt(rptbuf);
			valrpt(SYSERROR, ret, ELOOP, err);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	PATH_FUNC_RPT(1);	

	DBUG_VOID_RETURN;
}



private void
tb_setrlimit(resource, rlimstr, expmax, expcur)
int 	resource;
char	*rlimstr;
rlim_t	expmax, expcur;
{
	struct	rlimit	rlp;
	struct	rlimit	rlp2;
	int	err;
	int	pathok = 0;
	
	DBUG_ENTER("tb_setrlimit");

	rlp.rlim_max = expmax;
	rlp.rlim_cur = expcur;

	/*
	** Set the resource with new value 
	*/
	if (setrlimit(resource, &rlp) == SYSERROR)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"setrlimit() failed on %s - errno %d (%s)", rlimstr, err, errname(err));
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	** Verify that the resource get updated from setrlimit()
	*/
	if (getrlimit(resource, &rlp2) == SYSERROR)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"getrlimit() failed on %s - errno %d (%s)", rlimstr, err, errname(err));
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef	TESTBED
	expmax++;
	expcur++;
#endif
	if (rlp2.rlim_max != expmax)
	{
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"getrlimit() check on value st by setrlimit(%s) returned unexpected results:", rlimstr);
		pipe_in_rpt(rptbuf);
		(void)sprintf(rptbuf,"rlim_max: %ld (should be set to %ld)", rlp2.rlim_max, expmax);
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (rlp2.rlim_cur != expcur)
	{
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"getrlimit() check on value st by setrlimit(%s) returned unexpected results:", rlimstr);
		pipe_in_rpt(rptbuf);
		(void)sprintf(rptbuf,"rlim_cur: %ld (should be set to %ld)", rlp2.rlim_cur, expcur);
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4); 
	DBUG_VOID_RETURN;
}
