/*
*      SCCS:  @(#)POSIX.os/procprim/exec_L/exec_L.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/exec_L - T.exec_L Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.exec_L

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the exec() system call, LSB extensions.

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Tue Apr 11 10:26:43 BST 1989
MODIFICATIONS:

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<pcts_max.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<sys/time.h>
#include	<sys/wait.h>
#include	<sys/ipc.h>
#include	<sys/sem.h>
#include	<stdio.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<string.h>
#include	<termios.h>
#include	<exec.h>
#include	<synch.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif


#define	NO_TESTS	8		/* number of tests in testset */
#define	MAXARG		9
#define	NAME()		(i_name[interface])
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	NAP		4
#define	MODEGEN		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	void		vsx_sigs();
extern	void		valrpt();
extern	void		xx_all();
extern	char		*itoa();
extern	char		*newstr();
extern	char		*status_child();
extern	char		*errname();
extern	char		*execpath();
extern	void		pipe_close();
extern	void		pipe_parent();
extern	void		pipe_child();
extern	int		pipe_open();


extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;
extern	int	sig_array[];
extern	int	ex_code;	/* defined in wait_child.c */

int	no_tests = NO_TESTS;

private int	p_argc;
private char	*p_argv[MAXARG];
private char	**p_envp;
private char	*rsn_s = 		"setup() failed to malloc memory";
private int	interface;
private char	*i_name[6] = 		{ "execl",
					  "execle",
					  "execlp",
					  "execv",
					  "execve",
					  "execvp"
					};

private	char	*t_prog;
private	char	*t_path;
private	char	tgen_prog[] =		"../exec_tgn_L";
private	char	tsh_prog[] =		"../exec_tsh_L";
private char	loop_file1[] =		"./loop_filea";
private char	loop_file2[] =		"./loop_fileb";
private	int	test_fail;
private	int	test_deleted;
private	int	niceval = -1;
private	int	nsigs;
private synch	*t8_sync;
public	int	setupok = -1;
private	stack_t	sigstk;
private int	semid;

private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private	int	caught_sig = 0;
private int	testfail;
private	unsigned char chr;


private	void		test1();
private	void		pa1and2();
private	void		test2();
private	void		test3();
private	void		test4();
private void		test5();
private void		test6();
private void		ch_test6();
private	void		test7();
private	void		pa3();
private	void		test8();
private	void		chtest8();
private	void		patest8();
private	void		init_exec();
private	void		ch_exec();
private	void		pipe_ch_exec();
private	void		prepare_tests();
private	void		setup();
private	int		before_exec();
private int		set_timers();
private	int		s_init();
private	int		s_alt_init();
private	int		signal_init();
private int	ctty();
private void	interrupt();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	NULL, 0
};

private	void
prepare_tests()
{
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	PATH_TRACE;


	/* guard against NULL vg_dbug being taken as arg list terminator */
	if (vg_dbug == NULL)
		vg_dbug = "-D";

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		nsigs = VNOJSIG;
	} else {
		nsigs = VNUMSIG;
	}


	setup();

	PATH_SETUP_RPT(2);

	DBUG_VOID_RETURN;
}

private int
signal_init()
{
	int	i;
	sigset_t sig_empty;
	DBUG_ENTER("signal_init");

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) s_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);
	DBUG_RETURN(0);
}

private	void
setup()
{
	int	i;
	char	*s;
	char	*p;
	static	char	delreason[80];

	if((s = strrchr(vg_ident, '/')) != (char *)NULL)
		s+=3;
	else
		s = vg_ident+2;
	if (strcmp(s, "execl_L" ) == 0)
		interface = I_EXECL;
	else if (strcmp(s, "execle_L") == 0)
		interface = I_EXECLE;
	else if (strcmp(s, "execlp_L") == 0)
		interface = I_EXECLP;
	else if (strcmp(s, "execv_L" ) == 0)
		interface = I_EXECV;
	else if (strcmp(s, "execve_L") == 0)
		interface = I_EXECVE;
	else if (strcmp(s, "execvp_L") == 0)
		interface = I_EXECVP;
	else
	{
		(void) strcpy(delreason, "testset id not set correctly");
		vsxerror("%s: is %s, should be exec{l,le,lp,v,ve,vp}_L",
			delreason, vg_ident);
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i, delreason);
		return;
	}
	/*
	 *	Setup the environment path for exec[vl]p
	 */
	if (interface == I_EXECVP || interface == I_EXECLP)
	{
		if ((s = tet_getvar("VSX_PATH")) == (char *) NULL)
			s = "::/bin:/usr/bin";
		if ((p = (char *)malloc((size_t) (strlen(s) + strlen(EXEC_PATH) + 1)))
							== (char *) NULL)
		{
			for (i = 1; i <= no_tests; i++)
				(void) tet_delete(i, rsn_s);
			return;
		}
		(void) strcat(strcpy(p, EXEC_PATH), s);
		if (vputenv(&environ, p) == SYSERROR)
		{
			(void) strcpy(delreason, "could not set environment path");
			vsxerror("%s to %s", delreason, p);
			for (i = 1; i <= no_tests; i++)
				(void) tet_delete(i, delreason);
			return;
		}
	}

	if((p_envp = (char **) malloc(sizeof(char *))) == (char **) NULL) {
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i, rsn_s);
		return;
	}
	p_envp[0] = (char *) NULL;
}

private int
s_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private int
s_alt_init(signum)
int	signum;		/* signal to act for */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to get current signal handling function.
	 * Change current action so that signal is set on alternate stack.
	 * Return the value returned by sigaction()
	 */
	if (sigaction(signum, (struct sigaction *)NULL, &act) == SYSERROR)
		return(-1);

	act.sa_flags   |= SA_ONSTACK;

	return(sigaction(signum, &act, (struct sigaction *) NULL));
}

private void
test1()
{
	DBUG_ENTER("test1");
	signal_init();

	test_fail = 0;
	test_deleted = 0;

	t_prog = tgen_prog;
	t_path = EXEC_PATH;

	if(pipe_open()!=0)
		(void) cpsig(pipe_ch_exec, pa1and2, WAITTIME, 0);
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	signal_init();
	test_fail = 0;
	test_deleted = 0;

	t_prog = tgen_prog;
	t_path = EXEC_PATH;

	if(pipe_open()!=0)
		(void) cpsig(pipe_ch_exec, pa1and2, WAITTIME, 0);
	pipe_close();

	DBUG_VOID_RETURN;
}

private	void
pipe_ch_exec()
{
	DBUG_ENTER("pipe_ch_exec");
	pipe_child();
	ch_exec();
	DBUG_VOID_RETURN;
}
	

private int
call_exec()
{
	int	ret;

	DBUG_ENTER("call_exec");

	switch (interface)
	{
	case I_EXECL:
		ret = execl(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			p_argv[7], (char *) NULL);
		break;
	case I_EXECLE:
		ret = execle(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			p_argv[7], (char *) NULL, p_envp);
		break;
	case I_EXECLP:
		ret = execlp(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			p_argv[7], (char *) NULL);
		break;
	case I_EXECV:
		ret = execv(t_prog, p_argv);
		break;
	case I_EXECVE:
		ret = execve(t_prog, p_argv, p_envp);
		break;
	case I_EXECVP:
		ret = execvp(t_prog, p_argv);
		break;
	}

	DBUG_RETURN(ret);
}

private void
pa1and2()
{
	int	err;
	int	stat_child;
	int	pathok = setupok;

	DBUG_ENTER("pa1and2");

	if (tet_thistest == 6)
	{
		interrupt();
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitpid() failed - errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_EXECFAIL) {
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pipe_parent();

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private int
before_exec()
{
	int	err;
	long	ulmt;

	DBUG_ENTER("before_exec");

	if ((sigstk.ss_sp = malloc(SIGSTKSZ)) == NULL)
		DBUG_RETURN(-1);

	sigstk.ss_size = SIGSTKSZ;
	sigstk.ss_flags = 0;
	if (sigaltstack(&sigstk, (stack_t *) NULL) < 0)
		DBUG_RETURN(-1);

	if (s_alt_init(SIGUSR1) == SYSERROR)
		DBUG_RETURN(-1);

	DBUG_RETURN(0);
}

private int
set_timers()
{
	struct itimerval timer_real, timer_virtual, timer_prof;

	DBUG_ENTER("set_timers");

	timer_real.it_value.tv_sec = 100;
	timer_real.it_value.tv_usec = 10000L;
	timer_real.it_interval.tv_sec = 10;
	timer_real.it_interval.tv_usec = 10000L;
	if (setitimer(ITIMER_REAL, &timer_real, (struct itimerval *) NULL) == SYSERROR)
		DBUG_RETURN(-1);

	timer_virtual.it_value.tv_sec = 200;
	timer_virtual.it_value.tv_usec = 20000L;
	timer_virtual.it_interval.tv_sec = 20;
	timer_virtual.it_interval.tv_usec = 20000L;
	if (setitimer(ITIMER_VIRTUAL, &timer_virtual, (struct itimerval *) NULL) == SYSERROR)
		DBUG_RETURN(-1);

	timer_prof.it_value.tv_sec = 300;
	timer_prof.it_value.tv_usec = 30000L;
	timer_prof.it_interval.tv_sec = 30;
	timer_prof.it_interval.tv_usec = 30000L;
	if (setitimer(ITIMER_PROF, &timer_prof, (struct itimerval *) NULL) == SYSERROR)
		DBUG_RETURN(-1);

	DBUG_RETURN(0);
}

private void
ch_exec()
{
	char	*p1, *p2, *p3;
	int	err;
	struct sembuf	sembuf;

	DBUG_ENTER("ch_exec");

	/* Set up limit values prior to exec() */
	p1 = "";
	p2 = "";
	p3 = "";
	if (tet_thistest == 1 || tet_thistest == 2)
	{
		if (before_exec() == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("Failed to set up alternate signal stack");
			DBUG_EXIT(A_EXECFAIL);
		}
	}

	if (tet_thistest == 3)
	{
		sembuf.sem_num = 0;
		sembuf.sem_op = 3;
		sembuf.sem_flg = SEM_UNDO;
		if (semop(semid, &sembuf, (size_t) 1) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("Failed to set semaphore value for test");
			DBUG_EXIT(A_EXECFAIL);
		}
	}

	if (tet_thistest == 7)
	{
		if (set_timers() == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("Failed to set interval timers");
			DBUG_EXIT(A_EXECFAIL);
		}
		p1 = "10";
		p2 = "20";
		p3 = "30";

	}

	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), "",
		p1, p2, p3, vg_ident, vg_dbug);

	(void) call_exec();
	err = errno;
	xx_rpt(DELETION);
	in_rpt("%s(%s, ...) failed - errno %d (%s)",
		NAME(), t_prog, err, errname(err));
	DBUG_EXIT(A_EXECFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
init_exec(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8)
char	*arg1;
char	*arg2;
char	*arg3;
char	*arg4;
char	*arg5;
char	*arg6;
char	*arg7;
char	*arg8;
{
	int i;

	DBUG_ENTER("init_exec");

	p_argv[0] = arg1;
	p_argv[1] = arg2;
	p_argv[2] = arg3;
	p_argv[3] = arg4;
	p_argv[4] = arg5;
	p_argv[5] = arg6;
	p_argv[6] = arg7;
	p_argv[7] = arg8;
	
	/* Guard against NULL arguments */
	for (i = 0; i < 8; i++)
		if(p_argv[i] == NULL)
			p_argv[i] = "";

	p_argv[8] = (char *) NULL;
	p_argc = 8;

	p_envp[0] = (char *) NULL;
	(void) vputenv(&p_envp, t_path);
	(void) vputenv(&p_envp, P1_TSET);
	(void) vputenv(&p_envp, P1_FILE);
	(void) vputenv(&p_envp, P1_TENV1);
	(void) vputenv(&p_envp, P1_LOWER);
	(void) vputenv(&p_envp, P1_UPPER);
	(void) vputenv(&p_envp, P1_PORT);
	if (interface != I_EXECLE && interface != I_EXECVE)
		environ = p_envp;

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	union semun { int val;
		      struct semid_ds *buf;
		      unsigned short *array;
	} ctlarg;

	int	semval;
	int	pathok = 0;
	int 	err;

	DBUG_ENTER("test5");

	signal_init();
	test_fail = 0;
	test_deleted = 0;

	if ((semid = semget(IPC_PRIVATE, 1, IPC_CREAT | MODEGEN)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("Could not create semaphore");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	ctlarg.val = 4;
	if (semctl(semid, 0, SETVAL, ctlarg) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("First call to semctl failed");
		in_rpt("Returned with error %s(%d)", errname(err), err);
		(void) semctl(semid, 1, IPC_RMID);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t_prog = tgen_prog;
	t_path = EXEC_PATH;

	if(pipe_open()!=0)
		(void) cpsig(pipe_ch_exec, pa1and2, WAITTIME, 0);

	pipe_close();

	if ((semval = semctl(semid, 0, GETVAL)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("Call to semctl failed");
		in_rpt("Returned with error %s(%d)", errname(err), err);
		(void) semctl(semid, 1, IPC_RMID);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (semval != ctlarg.val)
	{
		xx_rpt(FAILURE);
		in_rpt("Call to exec changed semadj value");
		in_rpt("This is reflected in the change in semaphore value");
		in_rpt("Expected value %d, Observed value %d", ctlarg.val, semval);
		(void) semctl(semid, 1, IPC_RMID);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) semctl(semid, 1, IPC_RMID);

	PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	DBUG_ENTER("test6");

	(void) cppair(ch_test6, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_test6()
{
	int	pathok = 0;

	DBUG_ENTER("ch_test6");

	signal_init();
	test_fail = 0;
	test_deleted = 0;

	if (ctty() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	t_prog = tgen_prog;
	t_path = EXEC_PATH;

	if(pipe_open()!=0)
		(void) cpsig(pipe_ch_exec, pa1and2, WAITTIME, 0);
	pipe_close();

	PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	DBUG_ENTER("test7");

	signal_init();
	test_fail = 0;
	test_deleted = 0;

	t_prog = tgen_prog;
	t_path = EXEC_PATH;

	if(pipe_open()!=0)
		(void) cpsig(pipe_ch_exec, pa1and2, WAITTIME, 0);
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
test8()
{
	DBUG_ENTER("test8");
	signal_init();
	if( (t8_sync = opensync())  == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("opensync() failed");
		DBUG_VOID_RETURN;
	}
	(void) cppair(chtest8, patest8, WAITTIME*3, 0);
	(void) closesync(t8_sync);
	(void) unlink(loop_file1);
	(void) unlink(loop_file2);

	DBUG_VOID_RETURN;
}

private void
chtest8()
{
	int	rval;
	int	err;
	int	sendval = 1;
	int	pathok = setupok;

	DBUG_ENTER("chtest8");
	
	globok = 0;
	if (cr_sym_loop(loop_file1, loop_file2) != SUCCEED)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could create symbolic links from and to %s and %s", loop_file1, loop_file2);
		if(sendsync(t8_sync[0][1],(char *)&sendval, sizeof(sendval),
				WAITTIME) != sizeof(sendval))
		{
			in_rpt("sendsync() failed");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	p_argv[0] = vg_ident;
	p_argv[1] = (char *) NULL;
	p_envp[0] = (char *) NULL;
	(void) vputenv(&environ, EXEC_PATH);
	errno = NOERROR;
	t_prog = loop_file1;

	switch (interface)
	{
	case I_EXECL:  rval = execl (t_prog, p_argv[0], (char *) NULL); break;
	case I_EXECLE: rval = execle(t_prog, p_argv[0], (char *) NULL, p_envp); break;
	case I_EXECLP: rval = execlp(t_prog, p_argv[0], (char *) NULL); break;
	case I_EXECV:  rval = execv (t_prog, p_argv); break;
	case I_EXECVE: rval = execve(t_prog, p_argv, p_envp); break;
	case I_EXECVP: rval = execvp(t_prog, p_argv); break;
	}
	err  = errno;

#ifdef TESTBED
	err = ENOSPC;
#endif

	if (rval != SYSERROR || err != ELOOP)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) on a looped symbolic link did not return expected values",
			NAME(), loop_file1);
		valrpt(SYSERROR, rval, ELOOP, err);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	if( sendsync(t8_sync[0][1],(char *)&sendval,
			sizeof(sendval), WAITTIME) !=sizeof(sendval))
	{
		xx_rpt(DELETION);
		in_rpt("sendsync() failed");
	}

	DBUG_VOID_RETURN;
}

private void
patest8()
{
	int	res;
	int	status;
	int	err;
	DBUG_ENTER("patest8");

	errno = NOERROR;

	if ( ( res = waitsync(t8_sync[0][0],(char *)&status, 
				sizeof(status), WAITTIME*2)) != sizeof(status) )
	{
		err = errno;
		DBUG_PRINTF("patest8","res: %d, errno: %d (%s)", res, err,
				errname(err));
		if (alrm_flag)
		{
			xx_rpt(FAILURE);
			in_rpt("%s(%s ,..) call succeeded (waitsync timed out)",
				NAME(), loop_file1);
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("waitsync() failed - errno %d (%s)",
				err, errname(err));
		}
	}
	DBUG_VOID_RETURN;
}

/***************************************
* subroutines
***************************************/

private void
sig_catch(sig)
{
	caught_sig = sig;
#ifdef	TESTBED
	caught_sig++;
#endif
}

private int
ctty()
{
	/* acquire controlling terminal */

	int	pathok = 0;
	DBUG_ENTER("ctty");

	switch (termios_prep(&tty_fildes, tty_termios_p,
			     &loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE; 
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");

		DBUG_RETURN(-1);
	default:
		if (testfail++ == 0)
			xx_rpt(DELETION);

		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY/LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY/LOOP");

		DBUG_RETURN(-1);
	}
	PATH_FUNC_RPT(1); 

	DBUG_RETURN(0);
}

private void
interrupt()
{                               /* NO == not expecting SIGINT   */
	/* send SIGINT to self and children */

	int err, ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("interrupt");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        /* for child setup, if any - allow for this in cppair() timeouts */
        (void) sleep(NAP);

        caught_sig = 0;

	DBUG_PRINTF("interrupt", "c_cc[VINTR] = 0%o",
		(int)tty_termios_p->c_cc[VINTR]);

        chr = tty_termios_p->c_cc[VINTR];

        SET_TIMEOUT(WAITTIME/2)

        ret = write(loop_fildes, (char *)&chr, 1);
	err = errno;
	if (ret == 1)
		(void) pause();

        CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (caught_sig != SIGINT)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("process did not receive SIGINT when expected");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3); 

	DBUG_VOID_RETURN;
}
