/*
*      SCCS:  @(#)  POSIX.os/procprim/exec_L/exec_tgn_L.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/exec_L - exec_tgn_L Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		exec_tgn_L.c

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
SYNOPSIS:
DESCRIPTION:
	Used by T.exec_L in tests 1 and 2 of exec() with X/OPEN extensions.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Fri Apr 28 16:59:06 BST 1989
MODIFICATIONS:
		Geoff Clare, 8 Feb 1990
			Pass expected nice value on command line

		David Sawyer
		Mon Aug 20 13:41:08 BST 1990
		Added resilience code.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file


************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<exitcodes.h>
#include	<report.h>
#include	<signal.h>
#include	<errno.h>
#include	<ctype.h>
#include	<sys/stat.h>
#include	<sys/time.h>
#include	<exec.h>
#include	<sysdep.h>
#include	<timeout.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif

#define	WAITTIME	(5 * SPEEDFACTOR)

extern	void	set_pname();
extern	void	pipe_xx_rpt();
extern	void	pipe_in_rpt();
extern	void	pipe_end_rpt();
extern	void	pipe_path_xs_rpt();

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	int	testnum;
private	int	niceval;
private	int	timer_real;
private	int	timer_virtual;
private	int	timer_prof;
private	char	rbuf[256];
private	int	caught_sig;

private	void	test1();
private	void	test2();
private	void	test5();
private	void	test6();
private	void	test7();
private void	sig_catch();

int	main(argc, argv)
	int	argc;
	char	*argv[];
{
	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"exec_tgen" : (argv[0]));
	if (argc == P1_NARG+2)
	{
		if (argv[P1_NARG+1] != (char *) NULL)
			vg_dbug = argv[P1_NARG+1];
		argc--;
	}
	if (argc == P1_NARG+1)
	{
		if (argv[P1_NARG] != (char *) NULL)
			vg_ident = argv[P1_NARG];
		argc--;
	}
	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if(argc >= 3) {
		testnum = atoi(argv[1]);
		niceval = atoi(argv[2]);
		timer_real = atoi(argv[3]);
		timer_virtual = atoi(argv[4]);
		timer_prof = atoi(argv[5]);
	}
	else {
		vsxerror("internal error - argc is %d - should be at least %d",
								argc, 3);
		exit(A_TSETDIR);
	}

	switch (testnum)
	{
	case 1:
		test1();
		break;
	case 2:
		test2();
		break;
	case 5:
		test5();
		break;
	case 6:
		test6();
		break;
	case 7:
		test7();
		break;
	default:
		vsxerror("internal error - unknown test %d", testnum);
		exit(A_TSETDIR);

		/* NOTREACHED */
		break;
	}

	pipe_end_rpt();
	exit(OK);

	/* NOTREACHED */
	return 0;
}

private void
sig_catch(signum)
int signum;
{
	caught_sig = signum;

	return;
}

private void
test1()
{
	int	err;
	stack_t	*old_stack;
	int	pathok = 0;

	DBUG_ENTER("test1");

	if (sigaltstack((const stack_t *) NULL, old_stack) == SYSERROR) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void) sprintf(rbuf, "deletion reason - sigaltstack() failed - errno %d", err);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((old_stack->ss_flags&SS_DISABLE) == 0) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("alternate signal stack not disabled by exec:");
		(void) sprintf(rbuf, "expected value %d; observed value %d", SS_DISABLE, old_stack->ss_flags);
		pipe_in_rpt(rbuf);
	}
	else
	{
		PATH_TRACE;
		pipe_path_xs_rpt(2, pathok);
	}

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	err;
	struct sigaction oact;
	int	pathok = 0;

	DBUG_ENTER("test2");

	if(sigaction(SIGUSR1, (struct sigaction *) NULL, &oact) == SYSERROR) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void) sprintf(rbuf, "deletion reason - sigaction(SIGUSR1, ...) failed - errno %d", err);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((oact.sa_flags&SA_ONSTACK) != 0) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("SA_ONSTACK flag set incorrectly by exec:");
		(void) sprintf(rbuf, "expected value %d; observed value %d", SA_ONSTACK, oact.sa_flags);
		pipe_in_rpt(rbuf);
	}
	else
	{
		PATH_TRACE;
		pipe_path_xs_rpt(2, pathok);
	}

	DBUG_VOID_RETURN;
}

private void
test5()
{
	int pathok = 0;

	DBUG_ENTER("test5");

	PATH_TRACE;

	pipe_path_xs_rpt(1, pathok);

	DBUG_VOID_RETURN;
}
	
private void
test6()
{
	int	err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test6");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, (struct sigaction *) NULL) == SYSERROR)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void) sprintf(rbuf, "deletion reason - sigaction(SIGINT, ...) failed - errno %s(%d)", errname(err), err);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	SET_TIMEOUT(WAITTIME/2);
	(void) pause();
	
	CLEAR_ALARM;

	if (caught_sig != SIGINT)
	{
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("Exec'ed program did not receive SIGINT on controlling terminal when expected");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pipe_path_xs_rpt(2, pathok);

	DBUG_VOID_RETURN;
}
	
private void
test7()
{
	int	err;
	struct itimerval timer_info;
	int	timers[3];
	int	i;
	int	type[3] = { ITIMER_REAL,
			    ITIMER_VIRTUAL,
			    ITIMER_PROF };
	char	*typename[3];
	int	testfail;
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	timers[0] = timer_real;
	timers[1] = timer_virtual;
	timers[2] = timer_prof;

	typename[0] = "ITIMER_REAL";
	typename[1] = "ITIMER_VIRTUAL";
	typename[2] = "ITIMER_PROF";

	for (i = 0; i < 3; i++)
	{
		if(getitimer(type[i], &timer_info) == SYSERROR) {
			testfail++;
			err = errno;
			pipe_xx_rpt(DELETION);
			(void) sprintf(rbuf, "deletion reason - getitimer(%s, ...) failed - errno %d", typename[i], err);
			pipe_in_rpt(rbuf);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if(timer_info.it_value.tv_sec == (time_t) 0) {
			testfail++;
			pipe_xx_rpt(FAILURE);
			sprintf(rbuf, "For %s: it_value.tv_sec reset incorrectly by exec:", typename[i]);
			pipe_in_rpt(rbuf);
			(void) sprintf(rbuf, "expected value %d; observed value %d", 0, (int) timer_info.it_value.tv_sec);
			pipe_in_rpt(rbuf);
		}
		else
			PATH_TRACE;
	
		if(timer_info.it_interval.tv_sec != timers[i]) {
			testfail++;
			pipe_xx_rpt(FAILURE);
			sprintf(rbuf, "For %s: it_interval.tv_sec reset incorrectly by exec:", typename[i]);
			pipe_in_rpt(rbuf);
			(void) sprintf(rbuf, "expected value %d; observed value %d", timers[i], (int) timer_info.it_interval.tv_sec);
			pipe_in_rpt(rbuf);
		}
		else
			PATH_TRACE;

		if(timer_info.it_interval.tv_usec != (timers[i] * 1000)) {
			testfail++;
			pipe_xx_rpt(FAILURE);
			sprintf(rbuf, "For %s: it_interval.tv_usec reset incorrectly by exec:", typename[i]);
			pipe_in_rpt(rbuf);
			(void) sprintf(rbuf, "expected value %d; observed value %d", (timers[i]*1000), (int) timer_info.it_interval.tv_usec);
			pipe_in_rpt(rbuf);
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		pipe_path_xs_rpt(12, pathok);

	DBUG_VOID_RETURN;
}
