/* dv (#)SOLTESTS/tset/SOL.os/procprim/exec_S/exec_S.m	1.4 99/02/11 */
/*
* (C) Copyright 1998 UniSoft Limited.
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*/
#ifndef	lint
#define	MAIN
#ifdef	MAIN
static	char sccsid[] = "(#)SOLTESTS/tset/SOL.os/procprim/exec_S/exec_S.m	1.4 99/02/11";
char	*copyright[] = {
		"(C) Copyright 1998 UniSoft Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.exec_S

PROJECT:	Solaris ABI Test Suite

DESCRIPTION:
	Supplementary testset for the exec..() family of system calls.

AUTHOR:		Integrated from MSTC by Matthew Hails, UniSoft Ltd.
DATE CREATED:	August 1998

************************************************************************/

#include <std.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <dbug.h>
#include <abi_dbug.h>
#include <report.h>
#include <timeout.h>
#include <tet_api.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif

/* #define NO_TESTS 5 */
#define NO_TESTS 1

static void parent1(void);
static void child1(void);
static void subtest1(void);
static void test1(void);
#if 0
static void parent2(void);
static void child2(void);
static void subtest2(void);
static void test2(void);
static void parent3(void);
static void subtest3(void);
static void test3(void);
static void parent4(void);
static void child4(void);
static void subtest4(void);
static void test4(void);
static void subtest5(void);
static void test5(void);
#endif

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <signal.h>
#if 0
#include <procfs.h>
#include <sys/priocntl.h>
#include <pcsample.h>
#endif
#include <inttypes.h>
#include <sys/time.h>
#include <sys/wait.h>
#if 0
#include <resmgr.h>
#include <resutils.h>
#include <negtest.h>
#endif

#define WAITTIME	(5 * SPEEDFACTOR)
#define NSAMPLES	64L
#define SOME_SHMSIZE	(size_t)(NSAMPLES * sizeof(uintptr_t))

/* External prototypes */
char * errname(int);

/* Local prototypes */
static void handler(int code);
static void wrapper(void (*testfn)(void));
static int callexec(char *path);
#if 0
static void handler(int code);
static int setupsigs(void);
static int waitsig(int code);
static int getpstat(pid_t pid, pstatus_t *pstat);
static int getclassname(pid_t pid, char *clname);
#endif
static void corechildcode(void);
#if 0
static int tracechild(pid_t childpid, int signum);
#endif

/* External data declarations */
extern char **_environ;

/* Global static data */
static char progname[] = "../execprog";
static char *execname;
static int shmid;
static sig_atomic_t gotsig;

/*
 * Wrapper for test functions: repeats testfn for each exec..() function.
 */
static void
wrapper(void (*testfn)(void))
{
	static char *execlist[] =
		{ "execl", "execv", "execle", "execve", "execlp", "execvp" };
	int i;

	DBUG_ENTER("wrapper");

	for (i = 0; i < sizeof(execlist)/sizeof(execlist[0]); i++)
	{
		execname = execlist[i];
		DBUG_PRINTF("Info", "  +++ Testing %s +++", execname);
		errno = 0;
		testfn();
#if 0
		pcsample((uintptr_t *)NULL, 0);
#endif
		res_freeall();
	}

	DBUG_VOID_RETURN;
}

/*
 * Call the exec..() function currently under test.
 */
static int
callexec(char *path)
{
	char *arglist[2];

	arglist[0] = path;
	arglist[1] = NULL;

	if (strcmp(execname, "execl") == 0)
		return execl(path, path, (char *)NULL);
	else if (strcmp(execname, "execv") == 0)
		return execv(path, arglist);
	else if (strcmp(execname, "execle") == 0)
		return execle(path, path, (char *)NULL, _environ);
	else if (strcmp(execname, "execve") == 0)
		return execve(path, arglist, _environ);
	else if (strcmp(execname, "execlp") == 0)
		return execlp(path, path, (char *)NULL);
	else if (strcmp(execname, "execvp") == 0)
		return execvp(path, arglist);

	xx_unresolved("Cosmic ray or programming error in callexec()");
	return -1;
}

/*
 * Signal handler used to catch SIGUSR1 and SIGUSR2.
 */
static void
handler(int code)
{
	if (code == SIGUSR1 || code == SIGUSR2)
		gotsig = code;
}

/*
 * Setup for catching SIGUSR1 and SIGUSR2. Installs signal handler and resets
 * flag.
 */
static int
setupsigs(void)
{
	struct sigaction act;

	DBUG_ENTER("setupsigs");

	gotsig = 0;

	act.sa_handler = handler;
	act.sa_flags = 0;
	sigemptyset(&act.sa_mask);

	if (sigaction(SIGUSR1, &act, NULL) != 0
		|| sigaction(SIGUSR2, &act, NULL) != 0)
	{
		xx_unresolved(
			"Error installing signal handlers for SIGUSR1/2");
		DBUG_RETURN(-1);
	}

	DBUG_RETURN(0);
}

/*
 * Wait until we've caught a particular signal.
 */
static int
waitsig(int code)
{
	int err;

	DBUG_ENTER("waitsig");

	SET_TIMEOUT(WAITTIME / 2);
	while (gotsig != code && alrm_flag == 0)
		pause();
	CLEAR_ALARM;

	if (alrm_flag) {
		xx_rpt(DELETION);
		in_rpt("timed out waiting for signal %s from %s process",
			signame(code), tet_child ? "child" : "parent");
		DBUG_RETURN(-1);
	}

	DBUG_RETURN(0);
}

#if 0
/*
 * Get process status information from /proc file system.
 */
static int
getpstat(pid_t pid, pstatus_t *pstat)
{
	char procstatfile[sizeof("/proc/9223372036854775807/status")];
	int fd;

	DBUG_ENTER("getpstat");

	errno = 0;

	sprintf(procstatfile, "/proc/%ld/status", (long)pid);
	memset(pstat, 0, sizeof(*pstat));

	fd = open(procstatfile, O_RDONLY);
	if (fd == -1)
	{
		xx_unresolved("Error opening \"%s\" (%s)", procstatfile,
			errname(errno));
		DBUG_RETURN(-1);
	}

	if (read(fd, pstat, sizeof(*pstat)) != sizeof(*pstat))
	{
		xx_unresolved("Error reading from \"%s\" (%s)", procstatfile,
			errname(errno));
		close(fd);
		DBUG_RETURN(-1);
	}

	if (close(fd) != 0)
	{
		xx_unresolved("Error closing \"%s\" (%s)", procstatfile,
			errname(errno));
		DBUG_RETURN(-1);
	}

	DBUG_RETURN(0);
}

/*
 * Get scheduling class (name) of a process. Result is written to `clname',
 * which is assumed to be at least PC_CLNMSZ bytes in size.
 */
static int
getclassname(pid_t pid, char *clname)
{
	pcparms_t parms;
	pcinfo_t info;

	DBUG_ENTER("getclassname");

	/* Obtain scheduler class ID of child using priocntl() */
	parms.pc_cid = PC_CLNULL;

	if (priocntl(P_PID, pid, PC_GETPARMS, &parms) == -1)
	{
		xx_unresolved("priocntl(P_PID, %d, PC_GETPARMS,) failed (%s)",
			(int)pid, errname(errno));
		DBUG_RETURN(-1);
	}

	/* Use priocntl() to get name of scheduling class from CID */
	info.pc_cid = parms.pc_cid;

	if (priocntl(0, 0, PC_GETCLINFO, &info) == -1)
	{
		xx_unresolved(
			"priocntl(,,PC_GETCLINFO,) failed (%s) for class ID %d",
			errname(errno), (int)info.pc_cid);
		DBUG_RETURN(-1);
	}

	sprintf(clname, "%.*s", (int)sizeof(info.pc_clname), info.pc_clname);

	DBUG_RETURN(0);
}

#endif

/*
 * Core work for child proccesses. Contains synchronization code using signals:
 * sends parent a SIGUSR1 to indicate is pausing before exec, then waits for
 * SIGUSR1 from parent, then execs the program execprog.
 */
static void
corechildcode(void)
{
	/* Setup signal handler */
	setupsigs();

	/* Send parent SIGUSR1. This indicates we're about to pause before the
	 * exec.
	 */
	kill(getppid(), SIGUSR1);

	/* Wait for SIGUSR1 from parent. This indicates we should go ahead
	 * and exec now.
	 */
	if (waitsig(SIGUSR1) < 0)
		_exit(1);

	/* Call exec function currently under test */
	callexec(progname);

	/* If we get this far, the exec has failed */
	xx_unresolved("Child: %s(\"%s\", ...) failed (%s)", execname, progname,
		errname(errno));
	_exit(1);
}

#if 0

/*
 * Wait for a child process to be stopped, and restart using ptrace().
 */
static int
tracechild(pid_t childpid, int signum)
{
	int status;

	DBUG_ENTER("tracechild");

	if (wait(&status) != childpid)
	{
		xx_unresolved("Wait failed");
		DBUG_RETURN(-1);
	}

	if (!WIFSTOPPED(status))
	{
		xx_unresolved("Child not stopped");
		DBUG_RETURN(-1);
	}

	if (ptrace(7, childpid, 1, signum) != signum)
	{
		xx_unresolved("ptrace() failed in parent (%s)",
			errname(errno));
		DBUG_RETURN(-1);
	}

	DBUG_RETURN(0);
}
#endif


/********************* Beginning of test cases *********************/


/*
 * Parent process code for test 1.
 */
static void
parent1(void)
{
	int pathok = 0;
	void *addr;
	struct shmid_ds buf;

	DBUG_ENTER("parent1");


	/* Wait for SIGUSR1 from child. This indicates child has attached to
	 * shared memory segment.
	 */
	if (waitsig(SIGUSR1) < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
	}

	errno = 0;

	/* Attach to shared memory segment */
	addr = shmat(shmid, (void *)NULL, 0666);
	if (addr == (void*)-1)
	{
		xx_unresolved("Error attaching to shared memory segment (%s)",
			errname(errno));
		DBUG_VOID_RETURN;
	}
	else
	{
		res_register(addr, res_getshmattype());
		PATH_TRACE;
	}

	/* Verify that both parent and child are attached to memory segment */
	if (shmctl(shmid, IPC_STAT, &buf) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		xx_unresolved("Error getting shared memory stats");
		DBUG_VOID_RETURN;
	}

	if (buf.shm_nattch == 2)
	{
		PATH_TRACE;
	}
	else
	{
		xx_unresolved("%d attachments; expecting 2",
			(int)buf.shm_nattch);
		DBUG_VOID_RETURN;
	}

	/* Send child SIGUSR1 to indicate it should go ahead and exec now */
	kill(tet_child, SIGUSR1);

	/* Wait for SIGUSR2 from child. This indicates child is now exec'd
	 * program.
	 */
	if (waitsig(SIGUSR2) < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
	}

	/* Verify that we only have one attachment to shared memory segment,
	 * i.e. the exec has detached the child.
	 */
	if (shmctl(shmid, IPC_STAT, &buf) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		xx_unresolved("Error getting shared memory stats");
		DBUG_VOID_RETURN;
	}

	if (buf.shm_nattch == 1)
	{
		PATH_TRACE;
	}
	else
	{
		xx_fail("%d attachments; expecting 1", (int)buf.shm_nattch);
		DBUG_VOID_RETURN;
	}

	/* Send child SIGUSR2 to indicate it can complete now */
	kill(tet_child, SIGUSR2);

	PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

/*
 * Child process code for test 1.
 */
static void
child1(void)
{
	int pathok = 0;

	DBUG_ENTER("child1");


	errno = 0;

	/* Attach to shared memory segment */
	if (shmat(shmid, (void *)NULL, 0666) == (void *)-1)
	{
		xx_unresolved(
			"Child: error attaching to shared memory segment (%s)",
			errname(errno));
		_exit(1);
	}

	corechildcode();

	DBUG_VOID_RETURN;
}

/*
 * Function used to complete the work for test 1.
 */
static void
subtest1(void)
{
	int pathok = 0;

	DBUG_ENTER("subtest1");


	/* Create shared memory segment */
	shmid = shmget(IPC_PRIVATE, SOME_SHMSIZE, 0666);
	if (shmid >= 0)
	{
		res_register(&shmid, res_getshmsegtype());
	}
	else
	{
		xx_unresolved("Error creating shared memory segment (%s)",
			errname(errno));
		DBUG_VOID_RETURN;
	}

	/* Setup signal handler */
	if (setupsigs() != 0)
	{
		DBUG_VOID_RETURN;
	}

	/* Fork: work continues in child1() and parent1() */
	tet_fork(child1, parent1, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

/*
 * Assertion exec-1 (Compliance)
 *
 *    The shared memory segments attached to the calling process
 *    will not be attached to the new process created by exec().
 *
 * Strategy:
 *    Create shared memory segment.
 *    Setup signal handler.
 *    Fork. In child process:
 *       Attach to shared memory segment.
 *       Setup signal handler.
 *       Send parent SIGUSR1. This indicates a pause before the
 *       exec.
 *       Wait for SIGUSR1 from parent. This will indicate to go
 *       ahead and exec.
 *       Call exec(). In exec'ed program:
 *       Set signal handler for SIGUSR2.
 *       Send SIGUSR2 to parent to indicate we're in child
 *       process.
 *       Wait until parent sends us signal to indicate we can
 *       complete.
 *    In parent:
 *       Wait for SIGUSR1 from child. This indicates child has
 *       attached to shared memory segment.
 *       Attach to shared memory segment.
 *       Verify that both parent and child are attached to memory
 *       segment.
 *       Send child SIGUSR1 to indicate it should go ahead and
 *       exec.
 *       Wait for SIGUSR2 from child. This indicates child is now
 *       exec'd program.
 *       Verify that we only have one attachment to shared memory
 *       segment, i.e. the exec has detached the child.
 *       Send child SIGUSR2 to indicate it can complete now.
 */
static void
test1(void)
{
	int pathok = 0;

	DBUG_ENTER("test1");


	wrapper(subtest1);

	DBUG_VOID_RETURN;
}

#if 0
/*
 * Parent process code for test 2.
 */
static void
parent2(void)
{
	int pathok = 0;
	int testfail = 0;
	void *addr;
	uintptr_t *ip;

	DBUG_ENTER("parent2");


	/* Wait for SIGUSR1 from child. This indicates child has attached to
	 * shared memory segment.
	 */
	waitsig(SIGUSR1);

	errno = 0;

	/* Attach to shared memory segment */
	addr = shmat(shmid, (void *)NULL, 0666);
	if (addr == (void*)-1)
	{
		xx_unresolved("Error attaching to shared memory segment (%s)",
			errname(errno));
		DBUG_VOID_RETURN;
	}
	else
	{
		res_register(addr, res_getshmattype());
		PATH_TRACE;
	}

	/* Zero shared memory */
	memset(addr, 0, SOME_SHMSIZE);

	/* Send child SIGUSR1 to indicate it should go ahead and exec now */
	kill(tet_child, SIGUSR1);

	/* Wait for SIGUSR2 from child. This indicates child is now exec'd
	 * program.
	 */
	waitsig(SIGUSR2);

	/* Verify no more profiling information has been written to the shared
	 * memory segment.
	 */
	for (ip = addr; ip < (uintptr_t *)addr + NSAMPLES; ip++)
	{
		if (*ip == 0)
		{
			PATH_TRACE;
		}
		else
		{
			xx_fail("profile buffer index %lu is non-zero",
				(unsigned long)(ip - (uintptr_t *)addr));
			testfail++;
		}
	}

	/* Send child SIGUSR2 to indicate it can complete now */
	kill(tet_child, SIGUSR2);

	if (testfail == 0)
		PATH_XS_RPT(1 + (int)NSAMPLES);

	DBUG_VOID_RETURN;
}

/*
 * Child process code for test 2.
 */
static void
child2(void)
{
	int pathok = 0;
	void *addr;

	DBUG_ENTER("child2");


	errno = 0;

	/* Attach to shared memory segment */
	addr = shmat(shmid, (void *)NULL, 0666);
	if (addr == (void *)-1)
	{
		xx_unresolved("Error attaching to shared memory segment (%s)",
			errname(errno));
		_exit(1);
	}

	/* Zero shared memory */
	memset(addr, 0, SOME_SHMSIZE);

	/* Start profiling */
	pcsample(addr, NSAMPLES);

	/* Busy wait to allow profiling to start */
	do
	{
		int i;

		for (i = 0 ; i < 100000; i++)
			continue;
	} while (*(uintptr_t *)addr == 0);

	/* Use corechildcode() to do the synchronisation and exec */
	corechildcode();

	DBUG_VOID_RETURN;
}

/*
 * Function used to complete the work for test 2.
 */
static void
subtest2(void)
{
	int pathok = 0;

	DBUG_ENTER("subtest2");


	/* Create shared memory segment */
	shmid = shmget(IPC_PRIVATE, SOME_SHMSIZE, 0666);
	if (shmid >= 0)
	{
		res_register(&shmid, res_getshmsegtype());
	}
	else
	{
		xx_unresolved("Error creating shared memory segment (%s)",
			errname(errno));
		DBUG_VOID_RETURN;
	}

	/* Setup signal handler */
	if (setupsigs() != 0)
	{
		DBUG_VOID_RETURN;
	}

	/* Fork: work continues in child2() and parent2() */
	tet_fork(child2, parent2, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

/*
 * Assertion exec-2 (Compliance)
 *
 *    Profiling is disabled for the new process created by exec().
 *
 * Strategy:
 *    Create shared memory segment.
 *    Setup signal handler.
 *    Fork. In child:
 *       Attach to shared memory segment.
 *       Zero shared memory.
 *       Busy wait to allow profiling to start.
 *       Setup signal handler.
 *       Send parent SIGUSR1. This indicates a pause before the
 *       exec.
 *       Wait for SIGUSR1 from parent. This will indicate to go
 *       ahead and exec.
 *       Call exec(). In exec'ed program:
 *       Set signal handler for SIGUSR2.
 *       Send SIGUSR2 to parent to indicate we're in child
 *       process.
 *       Wait until parent sends us signal to indicate we can
 *       complete.
 *    In parent:
 *       Wait for SIGUSR1 from child. This indicates child has
 *       attached to shared memory segment.
 *       Attach to shared memory segment.
 *       Zero shared memory.
 *       Send child SIGUSR1 to indicate it should go ahead and
 *       exec now.
 *       Wait for SIGUSR2 from child. This indicates child is now
 *       exec'd program.
 *       Verify no more profiling information has been written to
 *       the shared memory segment.
 *       Send child SIGUSR2 to indicate it can complete now.
 */
static void
test2(void)
{
	int pathok = 0;

	DBUG_ENTER("test2");


	wrapper(subtest2);

	DBUG_VOID_RETURN;
}

/*
 * Parent process code for test 3.
 */
static void
parent3(void)
{
	int pathok = 0;
	char classbefore[PC_CLNMSZ];
	char classafter[PC_CLNMSZ];

	DBUG_ENTER("parent3");


	/* Wait for SIGUSR1 from child. This indicates child is pausing before
	 * exec.
	 */
	waitsig(SIGUSR1);

	/* Obtain scheduler class of child using priocntl() */
	if (getclassname(tet_child, classbefore) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		DBUG_VOID_RETURN;
	}

	/* Send child SIGUSR1 to indicate it should go ahead and exec now */
	kill(tet_child, SIGUSR1);

	/* Wait for SIGUSR2 from child. This indicates child is now exec'd
	 * program.
	 */
	waitsig(SIGUSR2);

	/* Obtain scheduler class of child process - now in exec'ed program */
	if (getclassname(tet_child, classafter) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		DBUG_VOID_RETURN;
	}

	/* Verify scheduler class is the same as before */
	if (strcmp(classafter, classbefore) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		xx_fail("Scheduler class not inherited across exec:");
		xx_fail("    Now \"%s\"; was \"%s\"", classafter, classbefore);
		DBUG_VOID_RETURN;
	}

	/* Send child SIGUSR2 to indicate it can complete now */
	kill(tet_child, SIGUSR2);

	PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

/*
 * Function used to complete the work for test 3.
 */
static void
subtest3(void)
{
	int pathok = 0;

	DBUG_ENTER("subtest3");


	/* Setup signal handler */
	if (setupsigs() != 0)
	{
		DBUG_VOID_RETURN;
	}

	/* Fork: work continues in corechildcode() and parent3() */
	tet_fork(corechildcode, parent3, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

/*
 * Assertion exec-3 (Compliance)
 *
 *    A new process created by exec() inherits the scheduler class
 *    from the calling process.
 *
 * Strategy:
 *    Setup signal handler to catch SIGUSR1 and SIGUSR2.
 *    Fork. In child:
 *       Setup signal handler.
 *       Send parent SIGUSR1. This indicates a pause before the
 *       exec.
 *       Wait for SIGUSR1 from parent. This will indicate to go
 *       ahead and exec.
 *       Call exec(). In exec'ed program:
 *       Set signal handler for SIGUSR2.
 *       Send SIGUSR2 to parent to indicate we're in child
 *       process.
 *       Wait until parent sends us signal to indicate we can
 *       complete.
 *    In parent:
 *       Wait for SIGUSR1 from child. This indicates child is
 *       pausing before exec.
 *       Obtain scheduler class of child using priocntl().
 *       Send child SIGUSR1 to indicate it should go ahead and
 *       exec.
 *       Wait for SIGUSR2 from child. This indicates child is now
 *       exec'd program.
 *       Obtain scheduler class of child process - now in exec'ed
 *       program.
 *       Verify that the child's scheduler class is the same as
 *       before the exec.
 *       Send child SIGUSR2 to indicate it can complete now.
 */
static void
test3(void)
{
	int pathok = 0;

	DBUG_ENTER("test3");


	wrapper(subtest3);

	DBUG_VOID_RETURN;
}

/*
 * Parent process code for test 4.
 */
static void
parent4(void)
{
	int pathok = 0;
	pstatus_t pstat;

	DBUG_ENTER("parent4");


	/* ===================================================================
	 * N.B. This code uses the /proc filesystem, which is not currently in
	 * the ABI. If this is not acceptable, the test becomes untestable.
	 * ===================================================================
	 */

	/* Wait for SIGUSR1 from child. This indicates child is pausing before
	 * exec.
	 */
	waitsig(SIGUSR1);

	/* Obtain status of child process from /proc filesystem */
	if (getpstat(tet_child, &pstat) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		xx_unresolved("Error obtaining child process status");
		DBUG_VOID_RETURN;
	}

	/* Verify that the trace flag is set for the child process */
	if (pstat.pr_flags & PR_PTRACE)
	{
		PATH_TRACE;
	}
	else
	{
		xx_unresolved("Trace flag not set in child before exec");
		DBUG_VOID_RETURN;
	}

	/* Restart child with SIGUSR1 using kill() and ptrace() */
	kill(tet_child, SIGUSR1);

	if (tracechild(tet_child, SIGUSR1) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		DBUG_VOID_RETURN;
	}

	/* Wait for child to be stopped as it starts the exec'ed program
	 * and restart it with ptrace().
	 */
	if (tracechild(tet_child, 0) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		DBUG_VOID_RETURN;
	}

	/* Wait for SIGUSR2 from child. This indicates child is now in
	 * exec'ed program.
	 */
	waitsig(SIGUSR2);

	/* Obtain status of child process - now in exec'ed program */
	if (getpstat(tet_child, &pstat) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		xx_unresolved(
			"Error obtaining process status of exec'ed program");
		DBUG_VOID_RETURN;
	}

	/* Verify that the trace flag is still set for the child process */
	if (pstat.pr_flags & PR_PTRACE)
	{
		PATH_TRACE;
	}
	else
	{
		xx_fail("Trace flag not set in child after exec");
		DBUG_VOID_RETURN;
	}

	/* Send child SIGUSR2 and restart with ptrace() to indicate it can
	 * complete now.
	 */
	kill(tet_child, SIGUSR2);

	if (tracechild(tet_child, SIGUSR2) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		DBUG_VOID_RETURN;
	}

	PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

/*
 * Child process code for test 4.
 */
static void
child4(void)
{
	int pathok = 0;

	DBUG_ENTER("child4");


	/* Call ptrace to turn on tracing by parent */
	ptrace(0, 0, 0, 0);

	corechildcode();

	DBUG_VOID_RETURN;
}

/*
 * Function used to complete the work for test 4.
 */
static void
subtest4(void)
{
	int pathok = 0;

	DBUG_ENTER("subtest4");


	/* Setup signal handler */
	if (setupsigs() != 0)
	{
		DBUG_VOID_RETURN;
	}

	/* Fork: work continues in corechildcode() and parent4() */
	tet_fork(child4, parent4, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

/*
 * Assertion exec-4 (Compliance)
 *
 *    A new process created by exec() inherits the trace flag from
 *    the calling process.
 *
 * Strategy:
 *    Set effective user ID to superuser so can use ptrace().
 *    Setup signal handler.
 *    Fork. In child:
 *       Call ptrace() to turn on tracing by parent.
 *       Setup signal handler.
 *       Send parent SIGUSR1. This indicates a pause before the
 *       exec.
 *       Wait for SIGUSR1 from parent. This will indicate to go
 *       ahead and exec.
 *       Call exec(). In exec'ed program:
 *       Set signal handler for SIGUSR2.
 *       Send SIGUSR2 to parent to indicate we're in child
 *       process.
 *       Wait until parent sends us signal to indicate we can
 *       complete.
 *    In parent:
 *       Wait for SIGUSR1 from child. This indicates child is
 *       pausing before exec.
 *       Obtain status of child process from /proc filesystem.
 *       Verify that the trace flag is set for the child process.
 *       Restart child with SIGUSR1 using kill() and ptrace().
 *       Wait for child to be stopped as it starts the exec'ed
 *       program and restart it with ptrace().
 *       Wait for SIGUSR2 from child. This indicates child is now
 *       in exec'ed program.
 *       Obtain status of child process - now in exec'ed program.
 *       Verify that the trace flag is still set for the child
 *       process.
 *       Send child SIGUSR2 and restart with ptrace() to indicate
 *       it can complete now.
 */
static void
test4(void)
{
	int pathok = 0;

	DBUG_ENTER("test4");


	wrapper(subtest4);

	DBUG_VOID_RETURN;
}

/*
 * Function used to complete the work for test 5.
 */
static void
subtest5(void)
{
	int pathok = 0;

	DBUG_ENTER("subtest5");


	negtest((int (*)())callexec, execname, -1, EFAULT, "p", (char *)1);
	DBUG_VOID_RETURN;
}

/*
 * Assertion exec-5 (Deviance)
 *
 *    When an argument is an illegal address, then exec returns -1
 *    and sets errno to EFAULT.
 *
 * Strategy:
 *    Call exec() with path as (char *)1.
 *    Verify exec() returns -1.
 *    Verify errno is set to EFAULT.
 */
static void
test5(void)
{
	int pathok = 0;

	DBUG_ENTER("test5");


	wrapper(subtest5);

	DBUG_VOID_RETURN;
}

#endif

/************************ End of test cases ************************/

int no_tests = NO_TESTS;

void (*setfunc)() = NULL;
void (*clnfunc)() = NULL;

struct tet_testlist tet_testlist[] =
{
	{ test1, 1 },
#if 0
	{ test2, 2 },
	{ test3, 3 },
	{ test4, 4 },
	{ test5, 5 },
#endif
	{ NULL, 0 }
};

