>>DV "99/02/11" "1.4" "02/11/99" "@(#)SOLTESTS/tset/SOL.os/procprim/exec_S/exec_S.m	1.4"

>># ######################################################################
>>#
>># (C) Copyright 1998 UniSoft Limited
>>#
>># All rights reserved.  No part of this source code may be reproduced,
>># stored in a retrieval system, or transmitted, in any form or by any
>># means, electronic, mechanical, photocopying, recording or otherwise,
>># except as in the end-user licence agreement, without the prior
>># permission of the copyright owners.
>>#
>># ######################################################################

>># **********************************************************************

>>NAME		exec_S.m
>>PROJECT	Solaris ABI Test Suite
>>AUTHOR	Integrated from MSTC by Matthew Hails, UniSoft Ltd.
>>DATE CREATED	August 1998
>>DESCRIPTION	Supplementary testset for the exec..() family of system calls.

>># **********************************************************************

>>TITLE exec_S SOL.os/procprim
execute a file

>>SYNOPSIS execl NODECL
int
execl(
	const char *path,
	const char *arg0,
	...,
	const char *argn,
	char * /*NULL*/)

>>SYNOPSIS execv NODECL
int
execv(
	const char *path,
	char *const argv[])

>>SYNOPSIS execle NODECL
int
execle(
	const char *path,
	char *const arg0[],
	...,
	const char *argn,
	char * /*NULL*/,
	char *const envp[])

>>SYNOPSIS execve NODECL
int
execve(
	const char *path,
	char *const argv[],
	char *const envp[])

>>SYNOPSIS execlp NODECL
int
execlp(
	const char *file,
	const char *arg0,
	...,
	const char *argn,
	char * /*NULL*/)

>>SYNOPSIS execvp NODECL
int
execvp(
	const char *file,
	char *const argv[])

>>SET exefile	"execprog"
>>SET cfile	"@{exefile}.c"
>>SET ofile	"@{exefile}.o"

>>FILE @cfile
/*
 * Program used by @name testset as exec'ed program.
 */
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>

static sig_atomic_t gotsig = 0;

static void
handler(int code)
{
	if (code == SIGUSR2)
		gotsig = code;
}

int
main(void)
{
	struct sigaction act;

	/* Set signal handler for SIGUSR2 */
	act.sa_handler = handler;
	act.sa_flags = 0;
	sigemptyset(&act.sa_mask);
	sigaction(SIGUSR2, &act, NULL);

	/* Send SIGUSR2 to parent to indicate we're in child process */
	kill(getppid(), SIGUSR2);

	/* Wait until parent sends us a SIGUSR2 to indicate we can complete */
	while (gotsig != SIGUSR2)
		pause();

	return 0;
}
>>MAKE REL @ofile @cfile
>>MAKE EXE @exefile @ofile

>>EXTERN
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <signal.h>
#include <procfs.h>
#include <sys/priocntl.h>
#include <pcsample.h>
#include <inttypes.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <resmgr.h>
#include <resutils.h>
#include <negtest.h>

#define WAITTIME	(5 * SPEEDFACTOR)
#define NSAMPLES	64L
#define SOME_SHMSIZE	(size_t)(NSAMPLES * sizeof(uintptr_t))

/* External prototypes */
char * errname(int);

/* Local prototypes */
static void handler(int code);
static void wrapper(void (*testfn)(void));
static int callexec(char *path);
static void handler(int code);
static int setupsigs(void);
static void waitsig(int code);
static int getpstat(pid_t pid, pstatus_t *pstat);
static int getclassname(pid_t pid, char *clname);
static void corechildcode(void);
static int tracechild(pid_t childpid, int signum);

/* External data declarations */
extern char **_environ;

/* Global static data */
static char progname[] = "../@exefile";
static char *execname;
static int shmid;
static sig_atomic_t gotsig;

/*
 * Wrapper for test functions: repeats testfn for each exec..() function.
 */
static void
wrapper(void (*testfn)(void))
{
	static char *execlist[] =
		{ "execl", "execv", "execle", "execve", "execlp", "execvp" };
	int i;

	DBUG_ENTER("wrapper");

	for (i = 0; i < sizeof(execlist)/sizeof(execlist[0]); i++)
	{
		execname = execlist[i];
		DBUG_PRINTF("Info", "  +++ Testing %s +++", execname);
		errno = 0;
		testfn();
		pcsample((uintptr_t *)NULL, 0);
		res_freeall();
	}

	DBUG_VOID_RETURN;
}

/*
 * Call the exec..() function currently under test.
 */
static int
callexec(char *path)
{
	char *arglist[2];

	arglist[0] = path;
	arglist[1] = NULL;

	if (strcmp(execname, "execl") == 0)
		return execl(path, path, (char *)NULL);
	else if (strcmp(execname, "execv") == 0)
		return execv(path, arglist);
	else if (strcmp(execname, "execle") == 0)
		return execle(path, path, (char *)NULL, _environ);
	else if (strcmp(execname, "execve") == 0)
		return execve(path, arglist, _environ);
	else if (strcmp(execname, "execlp") == 0)
		return execlp(path, path, (char *)NULL);
	else if (strcmp(execname, "execvp") == 0)
		return execvp(path, arglist);

	xx_unresolved("Cosmic ray or programming error in callexec()");
	return -1;
}

/*
 * Signal handler used to catch SIGUSR1 and SIGUSR2.
 */
static void
handler(int code)
{
	if (code == SIGUSR1 || code == SIGUSR2)
		gotsig = code;
}

/*
 * Setup for catching SIGUSR1 and SIGUSR2. Installs signal handler and resets
 * flag.
 */
static int
setupsigs(void)
{
	struct sigaction act;

	DBUG_ENTER("setupsigs");

	gotsig = 0;

	act.sa_handler = handler;
	act.sa_flags = 0;
	sigemptyset(&act.sa_mask);

	if (sigaction(SIGUSR1, &act, NULL) != 0
		|| sigaction(SIGUSR2, &act, NULL) != 0)
	{
		xx_unresolved(
			"Error installing signal handlers for SIGUSR1/2");
		DBUG_RETURN(-1);
	}

	DBUG_RETURN(0);
}

/*
 * Wait until we've caught a particular signal.
 */
static void
waitsig(int code)
{
	DBUG_ENTER("waitsig");

	while (gotsig != code)
		pause();

	DBUG_VOID_RETURN;
}

/*
 * Get process status information from /proc file system.
 */
static int
getpstat(pid_t pid, pstatus_t *pstat)
{
	char procstatfile[sizeof("/proc/9223372036854775807/status")];
	int fd;

	DBUG_ENTER("getpstat");

	errno = 0;

	sprintf(procstatfile, "/proc/%ld/status", (long)pid);
	memset(pstat, 0, sizeof(*pstat));

	fd = open(procstatfile, O_RDONLY);
	if (fd == -1)
	{
		xx_unresolved("Error opening \"%s\" (%s)", procstatfile,
			errname(errno));
		DBUG_RETURN(-1);
	}

	if (read(fd, pstat, sizeof(*pstat)) != sizeof(*pstat))
	{
		xx_unresolved("Error reading from \"%s\" (%s)", procstatfile,
			errname(errno));
		close(fd);
		DBUG_RETURN(-1);
	}

	if (close(fd) != 0)
	{
		xx_unresolved("Error closing \"%s\" (%s)", procstatfile,
			errname(errno));
		DBUG_RETURN(-1);
	}

	DBUG_RETURN(0);
}

/*
 * Get scheduling class (name) of a process. Result is written to `clname',
 * which is assumed to be at least PC_CLNMSZ bytes in size.
 */
static int
getclassname(pid_t pid, char *clname)
{
	pcparms_t parms;
	pcinfo_t info;

	DBUG_ENTER("getclassname");

	/* Obtain scheduler class ID of child using priocntl() */
	parms.pc_cid = PC_CLNULL;

	if (priocntl(P_PID, pid, PC_GETPARMS, &parms) == -1)
	{
		xx_unresolved("priocntl(P_PID, %d, PC_GETPARMS,) failed (%s)",
			(int)pid, errname(errno));
		DBUG_RETURN(-1);
	}

	/* Use priocntl() to get name of scheduling class from CID */
	info.pc_cid = parms.pc_cid;

	if (priocntl(0, 0, PC_GETCLINFO, &info) == -1)
	{
		xx_unresolved(
			"priocntl(,,PC_GETCLINFO,) failed (%s) for class ID %d",
			errname(errno), (int)info.pc_cid);
		DBUG_RETURN(-1);
	}

	sprintf(clname, "%.*s", (int)sizeof(info.pc_clname), info.pc_clname);

	DBUG_RETURN(0);
}

/*
 * Core work for child proccesses. Contains synchronization code using signals:
 * sends parent a SIGUSR1 to indicate is pausing before exec, then waits for
 * SIGUSR1 from parent, then execs the program @exefile.
 */
static void
corechildcode(void)
{
	/* Setup signal handler */
	setupsigs();

	/* Send parent SIGUSR1. This indicates we're about to pause before the
	 * exec.
	 */
	kill(getppid(), SIGUSR1);

	/* Wait for SIGUSR1 from parent. This indicates we should go ahead
	 * and exec now.
	 */
	waitsig(SIGUSR1);

	/* Call exec function currently under test */
	callexec(progname);

	/* If we get this far, the exec has failed */
	xx_unresolved("Child: %s(\"%s\", ...) failed (%s)", execname, progname,
		errname(errno));
	_exit(1);
}

/*
 * Wait for a child process to be stopped, and restart using ptrace().
 */
static int
tracechild(pid_t childpid, int signum)
{
	int status;

	DBUG_ENTER("tracechild");

	if (wait(&status) != childpid)
	{
		xx_unresolved("Wait failed");
		DBUG_RETURN(-1);
	}

	if (!WIFSTOPPED(status))
	{
		xx_unresolved("Child not stopped");
		DBUG_RETURN(-1);
	}

	if (ptrace(7, childpid, 1, signum) != signum)
	{
		xx_unresolved("ptrace() failed in parent (%s)",
			errname(errno));
		DBUG_RETURN(-1);
	}

	DBUG_RETURN(0);
}

>>ASSERTION 1
The shared memory segments attached to the calling process will not be
attached to the new process created by @name().
>>STRATEGY
.LI
Create shared memory segment.
.LI
Setup signal handler.
.LI
Fork. In child process:
.IN
.LI
Attach to shared memory segment.
.LI
Setup signal handler.
.LI
Send parent SIGUSR1. This indicates a pause before the exec.
.LI
Wait for SIGUSR1 from parent. This will indicate to go ahead and exec.
.LI
Call @name(). In exec'ed program:
.LI
Set signal handler for SIGUSR2.
.LI
Send SIGUSR2 to parent to indicate we're in child process.
.LI
Wait until parent sends us signal to indicate we can complete.
.IE
.LI
In parent:
.IN
.LI
Wait for SIGUSR1 from child. This indicates child has attached to shared
memory segment.
.LI
Attach to shared memory segment.
.LI
Verify that both parent and child are attached to memory segment.
.LI
Send child SIGUSR1 to indicate it should go ahead and exec.
.LI
Wait for SIGUSR2 from child. This indicates child is now exec'd program.
.LI
Verify that we only have one attachment to shared memory segment,
i.e. the exec has detached the child.
.LI
Send child SIGUSR2 to indicate it can complete now.
>>CODE

	wrapper(subtest@number);

>>SUBCODE

	/* Create shared memory segment */
	shmid = shmget(IPC_PRIVATE, SOME_SHMSIZE, 0666);
	if (shmid >= 0)
	{
		res_register(&shmid, res_getshmsegtype());
	}
	else
	{
		xx_unresolved("Error creating shared memory segment (%s)",
			errname(errno));
		DBUG_VOID_RETURN;
	}

	/* Setup signal handler */
	if (setupsigs() != 0)
	{
		DBUG_VOID_RETURN;
	}

	/* Fork: work continues in child@number() and parent@number() */
	tet_fork(child@number, parent@number, WAITTIME, 0);

>>CHILDCODE

	errno = 0;

	/* Attach to shared memory segment */
	if (shmat(shmid, (void *)NULL, 0666) == (void *)-1)
	{
		xx_unresolved(
			"Child: error attaching to shared memory segment (%s)",
			errname(errno));
		_exit(1);
	}

	corechildcode();

>>PARENTCODE
	void *addr;
	struct shmid_ds buf;

	/* Wait for SIGUSR1 from child. This indicates child has attached to
	 * shared memory segment.
	 */
	waitsig(SIGUSR1);

	errno = 0;

	/* Attach to shared memory segment */
	addr = shmat(shmid, (void *)NULL, 0666);
	if (addr == (void*)-1)
	{
		xx_unresolved("Error attaching to shared memory segment (%s)",
			errname(errno));
		DBUG_VOID_RETURN;
	}
	else
	{
		res_register(addr, res_getshmattype());
		PATH_TRACE;
	}

	/* Verify that both parent and child are attached to memory segment */
	if (shmctl(shmid, IPC_STAT, &buf) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		xx_unresolved("Error getting shared memory stats");
		DBUG_VOID_RETURN;
	}

	if (buf.shm_nattch == 2)
	{
		PATH_TRACE;
	}
	else
	{
		xx_unresolved("%d attachments; expecting 2",
			(int)buf.shm_nattch);
		DBUG_VOID_RETURN;
	}

	/* Send child SIGUSR1 to indicate it should go ahead and exec now */
	kill(tet_child, SIGUSR1);

	/* Wait for SIGUSR2 from child. This indicates child is now exec'd
	 * program.
	 */
	waitsig(SIGUSR2);

	/* Verify that we only have one attachment to shared memory segment,
	 * i.e. the exec has detached the child.
	 */
	if (shmctl(shmid, IPC_STAT, &buf) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		xx_unresolved("Error getting shared memory stats");
		DBUG_VOID_RETURN;
	}

	if (buf.shm_nattch == 1)
	{
		PATH_TRACE;
	}
	else
	{
		xx_fail("%d attachments; expecting 1", (int)buf.shm_nattch);
		DBUG_VOID_RETURN;
	}

	/* Send child SIGUSR2 to indicate it can complete now */
	kill(tet_child, SIGUSR2);

	PATH_XS_RPT(5);

>>ASSERTION 2
Profiling is disabled for the new process created by @name().
>>STRATEGY
.LI
Create shared memory segment.
.LI
Setup signal handler.
.LI
Fork. In child:
.IN
.LI
Attach to shared memory segment.
.LI
Zero shared memory.
.LI
Busy wait to allow profiling to start.
.LI
Setup signal handler.
.LI
Send parent SIGUSR1. This indicates a pause before the exec.
.LI
Wait for SIGUSR1 from parent. This will indicate to go ahead and exec.
.LI
Call @name(). In exec'ed program:
.LI
Set signal handler for SIGUSR2.
.LI
Send SIGUSR2 to parent to indicate we're in child process.
.LI
Wait until parent sends us signal to indicate we can complete.
.IE
.LI
In parent:
.IN
.LI
Wait for SIGUSR1 from child. This indicates child has attached to shared
memory segment.
.LI
Attach to shared memory segment.
.LI
Zero shared memory.
.LI
Send child SIGUSR1 to indicate it should go ahead and exec now.
.LI
Wait for SIGUSR2 from child. This indicates child is now exec'd program.
.LI
Verify no more profiling information has been written to the shared
memory segment.
.LI
Send child SIGUSR2 to indicate it can complete now.
>>CODE

	wrapper(subtest@number);

>>SUBCODE

	/* Create shared memory segment */
	shmid = shmget(IPC_PRIVATE, SOME_SHMSIZE, 0666);
	if (shmid >= 0)
	{
		res_register(&shmid, res_getshmsegtype());
	}
	else
	{
		xx_unresolved("Error creating shared memory segment (%s)",
			errname(errno));
		DBUG_VOID_RETURN;
	}

	/* Setup signal handler */
	if (setupsigs() != 0)
	{
		DBUG_VOID_RETURN;
	}

	/* Fork: work continues in child@number() and parent@number() */
	tet_fork(child@number, parent@number, WAITTIME, 0);

>>CHILDCODE
	void *addr;

	errno = 0;

	/* Attach to shared memory segment */
	addr = shmat(shmid, (void *)NULL, 0666);
	if (addr == (void *)-1)
	{
		xx_unresolved("Error attaching to shared memory segment (%s)",
			errname(errno));
		_exit(1);
	}

	/* Zero shared memory */
	memset(addr, 0, SOME_SHMSIZE);

	/* Start profiling */
	pcsample(addr, NSAMPLES);

	/* Busy wait to allow profiling to start */
	do
	{
		int i;

		for (i = 0 ; i < 100000; i++)
			continue;
	} while (*(uintptr_t *)addr == 0);

	/* Use corechildcode() to do the synchronisation and exec */
	corechildcode();

>>PARENTCODE
	int testfail = 0;
	void *addr;
	uintptr_t *ip;

	/* Wait for SIGUSR1 from child. This indicates child has attached to
	 * shared memory segment.
	 */
	waitsig(SIGUSR1);

	errno = 0;

	/* Attach to shared memory segment */
	addr = shmat(shmid, (void *)NULL, 0666);
	if (addr == (void*)-1)
	{
		xx_unresolved("Error attaching to shared memory segment (%s)",
			errname(errno));
		DBUG_VOID_RETURN;
	}
	else
	{
		res_register(addr, res_getshmattype());
		PATH_TRACE;
	}

	/* Zero shared memory */
	memset(addr, 0, SOME_SHMSIZE);

	/* Send child SIGUSR1 to indicate it should go ahead and exec now */
	kill(tet_child, SIGUSR1);

	/* Wait for SIGUSR2 from child. This indicates child is now exec'd
	 * program.
	 */
	waitsig(SIGUSR2);

	/* Verify no more profiling information has been written to the shared
	 * memory segment.
	 */
	for (ip = addr; ip < (uintptr_t *)addr + NSAMPLES; ip++)
	{
		if (*ip == 0)
		{
			PATH_TRACE;
		}
		else
		{
			xx_fail("profile buffer index %lu is non-zero",
				(unsigned long)(ip - (uintptr_t *)addr));
			testfail++;
		}
	}

	/* Send child SIGUSR2 to indicate it can complete now */
	kill(tet_child, SIGUSR2);

	if (testfail == 0)
		PATH_XS_RPT(1 + (int)NSAMPLES);

>>ASSERTION 3
A new process created by @name() inherits the scheduler class from the
calling process.
>>STRATEGY
.LI
Setup signal handler to catch SIGUSR1 and SIGUSR2.
.LI
Fork. In child:
.IN
.LI
Setup signal handler.
.LI
Send parent SIGUSR1. This indicates a pause before the exec.
.LI
Wait for SIGUSR1 from parent. This will indicate to go ahead and exec.
.LI
Call @name(). In exec'ed program:
.LI
Set signal handler for SIGUSR2.
.LI
Send SIGUSR2 to parent to indicate we're in child process.
.LI
Wait until parent sends us signal to indicate we can complete.
.IE
.LI
In parent:
.IN
.LI
Wait for SIGUSR1 from child. This indicates child is pausing before exec.
.LI
Obtain scheduler class of child using priocntl().
.LI
Send child SIGUSR1 to indicate it should go ahead and exec.
.LI
Wait for SIGUSR2 from child. This indicates child is now exec'd program.
.LI
Obtain scheduler class of child process - now in exec'ed program.
.LI
Verify that the child's scheduler class is the same as before the exec.
.LI
Send child SIGUSR2 to indicate it can complete now.
>>CODE

	wrapper(subtest@number);

>>SUBCODE

	/* Setup signal handler */
	if (setupsigs() != 0)
	{
		DBUG_VOID_RETURN;
	}

	/* Fork: work continues in corechildcode() and parent@number() */
	tet_fork(corechildcode, parent@number, WAITTIME, 0);

>>PARENTCODE
	char classbefore[PC_CLNMSZ];
	char classafter[PC_CLNMSZ];

	/* Wait for SIGUSR1 from child. This indicates child is pausing before
	 * exec.
	 */
	waitsig(SIGUSR1);

	/* Obtain scheduler class of child using priocntl() */
	if (getclassname(tet_child, classbefore) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		DBUG_VOID_RETURN;
	}

	/* Send child SIGUSR1 to indicate it should go ahead and exec now */
	kill(tet_child, SIGUSR1);

	/* Wait for SIGUSR2 from child. This indicates child is now exec'd
	 * program.
	 */
	waitsig(SIGUSR2);

	/* Obtain scheduler class of child process - now in exec'ed program */
	if (getclassname(tet_child, classafter) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		DBUG_VOID_RETURN;
	}

	/* Verify scheduler class is the same as before */
	if (strcmp(classafter, classbefore) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		xx_fail("Scheduler class not inherited across exec:");
		xx_fail("    Now \"%s\"; was \"%s\"", classafter, classbefore);
		DBUG_VOID_RETURN;
	}

	/* Send child SIGUSR2 to indicate it can complete now */
	kill(tet_child, SIGUSR2);

	PATH_XS_RPT(3);

>>ASSERTION 4
A new process created by @name() inherits the trace flag from the
calling process.
>>STRATEGY
.LI
Set effective user ID to superuser so can use ptrace().
.LI
Setup signal handler.
.LI
Fork. In child:
.IN
.LI
Call ptrace() to turn on tracing by parent.
.LI
Setup signal handler.
.LI
Send parent SIGUSR1. This indicates a pause before the exec.
.LI
Wait for SIGUSR1 from parent. This will indicate to go ahead and exec.
.LI
Call @name(). In exec'ed program:
.LI
Set signal handler for SIGUSR2.
.LI
Send SIGUSR2 to parent to indicate we're in child process.
.LI
Wait until parent sends us signal to indicate we can complete.
.IE
.LI
In parent:
.IN
.LI
Wait for SIGUSR1 from child. This indicates child is pausing before exec.
.LI
Obtain status of child process from /proc filesystem.
.LI
Verify that the trace flag is set for the child process.
.LI
Restart child with SIGUSR1 using kill() and ptrace().
.LI
Wait for child to be stopped as it starts the exec'ed program
and restart it with ptrace().
.LI
Wait for SIGUSR2 from child. This indicates child is now in exec'ed program.
.LI
Obtain status of child process - now in exec'ed program.
.LI
Verify that the trace flag is still set for the child process.
.LI
Send child SIGUSR2 and restart with ptrace() to indicate it can complete now.
>>CODE

	wrapper(subtest@number);

>>SUBCODE

	/* Setup signal handler */
	if (setupsigs() != 0)
	{
		DBUG_VOID_RETURN;
	}

	/* Fork: work continues in corechildcode() and parent@number() */
	tet_fork(child@number, parent@number, WAITTIME, 0);

>>CHILDCODE

	/* Call ptrace to turn on tracing by parent */
	ptrace(0, 0, 0, 0);

	corechildcode();

>>PARENTCODE
	pstatus_t pstat;

	/* ===================================================================
	 * N.B. This code uses the /proc filesystem, which is not currently in
	 * the ABI. If this is not acceptable, the test becomes untestable.
	 * ===================================================================
	 */

	/* Wait for SIGUSR1 from child. This indicates child is pausing before
	 * exec.
	 */
	waitsig(SIGUSR1);

	/* Obtain status of child process from /proc filesystem */
	if (getpstat(tet_child, &pstat) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		xx_unresolved("Error obtaining child process status");
		DBUG_VOID_RETURN;
	}

	/* Verify that the trace flag is set for the child process */
	if (pstat.pr_flags & PR_PTRACE)
	{
		PATH_TRACE;
	}
	else
	{
		xx_unresolved("Trace flag not set in child before exec");
		DBUG_VOID_RETURN;
	}

	/* Restart child with SIGUSR1 using kill() and ptrace() */
	kill(tet_child, SIGUSR1);

	if (tracechild(tet_child, SIGUSR1) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		DBUG_VOID_RETURN;
	}

	/* Wait for child to be stopped as it starts the exec'ed program
	 * and restart it with ptrace().
	 */
	if (tracechild(tet_child, 0) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		DBUG_VOID_RETURN;
	}

	/* Wait for SIGUSR2 from child. This indicates child is now in
	 * exec'ed program.
	 */
	waitsig(SIGUSR2);

	/* Obtain status of child process - now in exec'ed program */
	if (getpstat(tet_child, &pstat) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		xx_unresolved(
			"Error obtaining process status of exec'ed program");
		DBUG_VOID_RETURN;
	}

	/* Verify that the trace flag is still set for the child process */
	if (pstat.pr_flags & PR_PTRACE)
	{
		PATH_TRACE;
	}
	else
	{
		xx_fail("Trace flag not set in child after exec");
		DBUG_VOID_RETURN;
	}

	/* Send child SIGUSR2 and restart with ptrace() to indicate it can
	 * complete now.
	 */
	kill(tet_child, SIGUSR2);

	if (tracechild(tet_child, SIGUSR2) == 0)
	{
		PATH_TRACE;
	}
	else
	{
		DBUG_VOID_RETURN;
	}

	PATH_XS_RPT(7);

>>ASSERTION 5 -
When an argument is an illegal address,
then @name returns -1 and sets
.M errno
to
.S EFAULT .
>>STRATEGY
.LI
Call @name() with path as (char *)1.
.LI
Verify @name() returns -1.
.LI
Verify errno is set to EFAULT.
>>CODE

	wrapper(subtest@number);

>>SUBCODE

	negtest((int (*)())callexec, execname, -1, EFAULT, "p", (char *)1);
