/*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
* Project: LSB-OS
*
* File: tset/LSB.os/procprim/execlp/execlp.c
*
* Modifications:
* $Log: execlp.c,v $
* Revision 1.2  2001/10/02 12:20:44  ajosey
* update testset so it does not use functionality outside the LSB spec
*
* Revision 1.1  2001/05/27 06:59:19  cyeoh
* Adds files for lsb-os test suite
*
*/

#include	<vsrt/vsrt.h>
#include	<sys/mman.h>
#ifdef _POSIX_MESSAGE_PASSING
#include	<mqueue.h>
#endif
#ifdef _POSIX_SEMAPHORES
#include	<semaphore.h>
#endif

extern	int	cppair();

#define 	NO_TESTS	5	/* number of tests in testset */

#ifdef UNDEF_MACROS
#undef execlp
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();
extern	char	*execpath();

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	ch_t1();
private void	pa_t1();
private void	ch_t3();
#ifdef _POSIX_MESSAGE_PASSING
private void	ch_t5();
private void	pa_t5();
#endif

extern	char	**environ;

#define	test_file	"vsrt_execlp"
#define GO_PARENT      	1
#define GO_CHILD      	2

/*from vsrtlib/mkmmap.c*/
extern int	vsrt_got_sigsegv;
extern int	vsrt_got_sigbus;
extern int	vsrt_got_sig;
extern unsigned long	vsrt_pgsz;
extern unsigned long	vsrt_ipgsz;
extern int	vsrt_setjmp_called;

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	NULL, 0
};

char	shbuf[4096];
char	segname[] = "vsrt_execlp";
char	mqbuf[2048];
char	mqname[] = "vsrt_mqueue";
char	sembuf[2048];
char    semname[] = "vsrt_semph";

char	reply[512];
int	fildes[2];

/*
If _POSIX_SEMAPHORES is defined:
	On a call to execlp() all named  semaphores  that  are
	open  in the calling process shall be closed as if by
	a call to sem_close().
*/

private void test1()
{
	DBUG_ENTER("test1");

#ifndef _POSIX_SEMAPHORES
	in_rpt("_POSIX_SEMAPHORES not supported");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN 
#else

	/*Open communication pipe for exec process*/
	if (pipe(fildes) < 0) {
		vsrt_sysfail("pipe");
		DBUG_VOID_RETURN 
	}

	if (vsrt_get_sem_name(sembuf, semname, sizeof(sembuf)) == -1)
		DBUG_VOID_RETURN
	(void)sem_unlink(sembuf);

	VSRT_BEGIN_EVENTS;

	(void) cppair(ch_t1, pa_t1, 4 * WAITTIME, 0);

	VSRT_END_EVENTS;

	xs_rpt();

	/* clean-up */
	close(fildes[0]);
	close(fildes[1]);
	(void)sem_unlink(sembuf);

	DBUG_VOID_RETURN 
#endif
}

private void ch_t1()
{
	sem_t	*semd;
	int	errors = 0;
	int	unsup = 0;

	DBUG_ENTER("ch_t1");

	/*Connect child's pipe to stdout*/
	close(1);
	if(dup2(fildes[1], 1) < 0) {
		vsrt_sysfail("dup2");
		DBUG_VOID_RETURN
	}

	/* open a semaphore */
	semd = sem_open(sembuf, O_CREAT, S_IRWXU, 1);
	if (semd == SEM_FAILED) {
		if (errno == ENOSYS) {
			xx_rpt(UNSUPPORTED);
			unsup++;
		}
		else {
			in_rpt("sem_open() failed in child for \"%s\", errno = %d (%s)", sembuf, errno, errname(errno));
			xx_rpt(FAILURE);
			errors++;
		}
	}

	/* wait for parent to open,close and unlink the semaphore */
	if (VSRT_POST_EVENT(GO_PARENT) == -1) {
		(void)sem_close(semd);
		DBUG_VOID_RETURN
	}
	if (VSRT_WAIT_EVENT(GO_CHILD) == -1) {
		(void)sem_close(semd);
		DBUG_VOID_RETURN
	}

	if (!unsup) {

		/*Issue execlp*/
		execlp(execpath("execlp_check"), "execlp_check", "semaphores", (char *)0);
		/*should not reach here*/
		vsrt_sysfail("execlp");

	}

	(void)sem_close(semd);

	DBUG_VOID_RETURN
}

private void pa_t1()
{
	int	retval, i, result;
	sem_t	*semd;
	int	errors = 0;

	DBUG_ENTER("pa_t1");

	/* 
	 * wait for child to open the semaphore
	 */
	if (VSRT_WAIT_EVENT(GO_PARENT) == -1) {
		DBUG_VOID_RETURN
	}

	/* open the same semaphore as child's */
	semd = sem_open(sembuf, O_CREAT, S_IRWXU, 1);
	if (semd == SEM_FAILED) {
		if (errno == ENOSYS) {
			xx_rpt(UNSUPPORTED);
			VSRT_POST_EVENT(GO_CHILD);
			DBUG_VOID_RETURN
		}
		in_rpt("sem_open() failed in child for \"%s\", errno = %d (%s)", sembuf, errno, errname(errno));
		xx_rpt(FAILURE);
		errors++;
		VSRT_POST_EVENT(GO_CHILD);
		DBUG_VOID_RETURN
	}

	/* close the semaphore */
	errno = 0;
	if ((retval = sem_close(semd)) == -1) {
		vsrt_sysfail("sem_close");
        	(void)sem_unlink(sembuf);
		errors++;
		VSRT_POST_EVENT(GO_CHILD);
		DBUG_VOID_RETURN 
	}

	/* 
	 * This should succeed but the semaphore should not 
	 * be removed since child has the semaphore open.
	 */
	errno = 0;
	if ((retval = sem_unlink(sembuf)) == -1) {
		vsrt_sysfail("sem_unlink");
        	(void)sem_unlink(sembuf);
		errors++;
		VSRT_POST_EVENT(GO_CHILD);
		DBUG_VOID_RETURN 
	}

	/* ask child to exec a program */
	if (VSRT_POST_EVENT(GO_CHILD) == -1) {
		DBUG_VOID_RETURN 
	}

	/* 
	 * At this point the child process execs another
	 * process that just sleeps for a while.
	 * wait till we hear from the execed program. This
	 * will ensure that execlp() has been invoked.
	 */
	for (i = 0; i < 5; i++) {
		sleep(1);
		if((result = read(fildes[0], reply, sizeof(reply))) >= 0) {
			if ((result != 22) || (strncmp(reply, "EXECED_PROG_IS_RUNNING", 22) != 0)) {
				in_rpt("incorrect data from execed program, expected \"EXEC_PROG_IS_RUNNING\" got \"%s\"", reply);
				xx_rpt(FAILURE);
				DBUG_VOID_RETURN 
			}
			break;
		}
	}
	if (i == 5) {
		in_rpt("Execed process did not send any data");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

	/* 
	 * now since the execed program is running but not
	 * yet exited, test that the semaphore that was 
	 * opened in the child has been closed. This should 
	 * fail with ENOENT error validating that execlp() in child 
	 * closed its reference to the semaphore causing the 
	 * postponed removal of the semaphore to take place.
	 */
	errno = 0;
	if ((retval = sem_unlink(sembuf)) != -1) {
		in_rpt("execlp() in child did not close the semaphore");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}
	else {
		if (errno != ENOENT) {
			in_rpt("sem_unlink() failed with errno = %d (%s)", errno, errname(errno));
			xx_rpt(FAILURE);
			DBUG_VOID_RETURN 
		}
	}

	/* just return. cppair() will wait for the child */
	DBUG_VOID_RETURN
}


/*
If _POSIX_MAPPED_FILES or  _POSIX_SHARED_MEMORY_OBJECTS
is defined:
	On a call to execlp() memory mappings created  in  the
	process shall be unmapped before the address space is
	rebuilt for the new process image.
*/

/*
Set the number of pages to be mapped.  This needs to be large
enough so that the end of the mapped area is sure not to be a valid
address in the exec'd child.  If the value needs to be increased,
it can be defined with a -D in the compiler flags.
*/
#ifndef VSRT_EXEC_MMAP_NPAGES
#define VSRT_EXEC_MMAP_NPAGES	64
#endif

private int test2p1();
private int test2p2();
private int dotest2();

private void test2()
{
	int     ret1, ret2;

	DBUG_ENTER("test2");

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	ret1 = test2p1();
	ret2 = test2p2();

	/*any error is a failure*/
	if ((ret1 == -1) || (ret2 == -1)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	/*if both parts cannot be run its unsupported*/
	if ((ret1 == 1) && (ret2 == 1)) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	/*if either part is run successfully its a pass*/
	xs_rpt();

	DBUG_VOID_RETURN 
} 

/*test for files*/
int test2p1()
{
	int     fd, ret;

	DBUG_ENTER("test2p1");

#ifndef _POSIX_MAPPED_FILES
	in_rpt("_POSIX_MAPPED_FILES not supported");
	xx_rpt(UNSUPPORTED);
	DBUG_RETURN(1);
#else

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL,
		VSRT_EXEC_MMAP_NPAGES);
	if (fd == -1)
		DBUG_RETURN(-1);

	ret = dotest2(fd);

	(void)close(fd);
	(void)unlink(test_file);

	DBUG_RETURN(ret);
#endif
}

int test2p2()
{
       int     fd, ret;

	DBUG_ENTER("test2p2");

/* not in use for LSB testing */
	DBUG_RETURN(1);
}

int dotest2(fd)
int	fd;
{
	int     pathok = 0;
	int     *adr1;
	pid_t   pid;
	int     status;
	int	result;

	DBUG_ENTER("dotest2");

	/*Open communication pipe for exec process*/
	if (pipe(fildes) < 0) {
		vsrt_sysfail("pipe");
		DBUG_RETURN(-1);
	}

	VSRT_FORK(pid)
		/*Connect child's pipe to stdout*/
		close(1);
		if(dup2(fildes[1], 1) < 0) {
			vsrt_sysfail("dup2");
			exit(1);
		}
		/*Map the pages*/
		if ((adr1 = (int *)mmap(0,
		    (size_t)(vsrt_pgsz*VSRT_EXEC_MMAP_NPAGES), PROT_READ,
		    MAP_SHARED, fd, 0)) == (int *)MAP_FAILED) {
			vsrt_sysfail("mmap");
			exit(1);
		}
		else
			PATH_TRACE;

		/*Issue execlp*/
		sprintf(reply, "%lx", (unsigned long)adr1 +
					vsrt_pgsz*(VSRT_EXEC_MMAP_NPAGES-1));
		execlp(execpath("execlp_check"), "execlp_check", "mappings", reply, (char *)0);

		/*should not reach here*/
		vsrt_sysfail("execlp");
		exit(1);
	VSRT_EFORK
	pid = wait(&status);
	if (pid == -1) {
		vsrt_sysfail("fork");
		DBUG_RETURN(-1);
	}

	if(!WIFEXITED(status)) {
		if (WIFSIGNALED(status))
			in_rpt("Child terminated due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child exited abnormally with status = 0x%x", status);
		DBUG_RETURN(-1);
	}

	if (status == 99) {
		in_rpt("Execed process error - could not complete test");
		close(fildes[0]);
        	close(fildes[1]);
		DBUG_RETURN(-1);
	}

	if (WEXITSTATUS(status) != 0) {
		close(fildes[0]);
        	close(fildes[1]);
		DBUG_RETURN(-1);
	}

	/*Read from exec-ed process to see test result*/
	if((result = read(fildes[0], reply, sizeof(reply))) < 0) {
		vsrt_sysfail("read");
		close(fildes[0]);
        	close(fildes[1]);
		DBUG_RETURN(-1);
	}

	if ((result != 4) || (strncmp(reply, "PASS", 4) != 0)) {
		in_rpt("Memory mappings were not removed");
		close(fildes[0]);
        	close(fildes[1]);
		DBUG_RETURN(-1);
	}

	close(fildes[0]);
        close(fildes[1]);

	DBUG_RETURN(0);
}


/*
If _POSIX_PRIORITY_SCHEDULING is defined:
	On a call to execlp(), for the SCHED_FIFO and SCHED_RR
	scheduling policies, the scheduling policy and prior-
	ity settings shall not be changed.
*/

private void test3()
{
	DBUG_ENTER("test3");

#ifndef _POSIX_PRIORITY_SCHEDULING
	in_rpt("_POSIX_PRIORITY_SCHEDULING not defined");
	xx_rpt(UNSUPPORTED);
#else
	/*fork and exec a process with privilege so it can set
	scheduling parameters. It fork and exec another process*/
	(void) cppair(ch_t3, NULLFN, 4 * WAITTIME, 0);

#endif
	DBUG_VOID_RETURN 
} 

private void ch_t3()
{
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t3");

	args[0] = "execlp3";
	args[1] = NULL;
	(void) tet_exec(execpath("execlp3"), args, environ);

	err = errno;
	xx_rpt(FAILURE);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)", "execlp3", err, errname(err));

	DBUG_VOID_RETURN 
}

/*
If _POSIX_TIMERS is defined:
	On a call to execlp(), per-process timers  created  by
	the calling process shall be deleted before replacing
	the current process image with the new process image.
*/

private void test4()
{
	int	pathok = 0;
	timer_t	timerid;
	pid_t	pid;
	int	status;
	int	result;

	DBUG_ENTER("test4");

#ifndef _POSIX_TIMERS
	in_rpt("_POSIX_TIMERS not supported");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN 
#else

	/*Open communication pipe for exec process*/
	if (pipe(fildes) < 0) {
		vsrt_sysfail("pipe");
		DBUG_VOID_RETURN 
	}

	VSRT_FORK(pid)
		/*Connect child's pipe to stdout*/
		close(1);
		if(dup2(fildes[1], 1) < 0) {
			vsrt_sysfail("dup2");
			exit(1);
		}
		/*Open a timer*/
		if (timer_create(CLOCK_REALTIME, NULL, &timerid) == -1) {
			vsrt_sysfail("timer_create");
			exit(1);
		}
		else
			PATH_TRACE;

		/*Issue execlp*/
		sprintf(reply, "%x", timerid);
		execlp(execpath("execlp_check"), "execlp_check", "timers", reply, (char *)0);

		/*should not reach here*/
		vsrt_sysfail("execlp");
		(void)timer_delete(timerid);
		exit(1);
	VSRT_EFORK
	pid = wait(&status);
	if (pid == -1) {
		vsrt_sysfail("fork");
		close(fildes[0]);
		close(fildes[1]);
		DBUG_VOID_RETURN 
	}

	if(!WIFEXITED(status)) {
		if (WIFSIGNALED(status))
			in_rpt("Child terminated due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child exited abnormally with status = 0x%x", status);
		close(fildes[0]);
		close(fildes[1]);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

	if (WEXITSTATUS(status) == 99) {
		in_rpt("Execed process error - could not complete test");
		close(fildes[0]);
		close(fildes[1]);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

	if (status != 0) {
		close(fildes[0]);
		close(fildes[1]);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

	/*Read from exec-ed process to see test result*/
	if((result = read(fildes[0], reply, sizeof(reply))) < 0) {
		vsrt_sysfail("read");
		close(fildes[0]);
		close(fildes[1]);
		DBUG_VOID_RETURN 
	}

	if ((result != 4) || (strncmp(reply, "PASS", 4) != 0)) {
		in_rpt("Timers were not deleted");
		close(fildes[0]);
		close(fildes[1]);
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

	close(fildes[0]);
	close(fildes[1]);

	xs_rpt();

	DBUG_VOID_RETURN 
#endif
} 

/*
If _POSIX_MESSAGE_PASSING is defined:
	On a call to execlp(), all open message queue descrip-
	tors  in  the  calling  process  shall  be closed, as
	described in mq_close().
*/
private void test5()
{
	DBUG_ENTER("test5");

#ifndef _POSIX_MESSAGE_PASSING
	in_rpt("_POSIX_MESSAGE_PASSING not supported");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN 
#else

	/*Open communication pipe for exec process*/
	if (pipe(fildes) < 0) {
		vsrt_sysfail("pipe");
		DBUG_VOID_RETURN 
	}

	if (vsrt_get_mqueue_name(mqbuf, mqname, sizeof(mqbuf)) == -1)
		DBUG_VOID_RETURN
	(void)mq_unlink(mqbuf);

	VSRT_BEGIN_EVENTS;

	(void) cppair(ch_t5, pa_t5, 4 * WAITTIME, 0);

	VSRT_END_EVENTS;

	xs_rpt();

	/* clean-up */
	close(fildes[0]);
	close(fildes[1]);
	(void)mq_unlink(mqbuf);

	DBUG_VOID_RETURN 
#endif
}

#ifdef _POSIX_MESSAGE_PASSING
private void ch_t5()
{
	mqd_t	mqd;
	int	retval, err = 0;
	int	errors = 0;
	struct sigevent enotify;

	DBUG_ENTER("ch_t5");

	/*Connect child's pipe to stdout*/
	close(1);
	if(dup2(fildes[1], 1) < 0) {
		vsrt_sysfail("dup2");
		DBUG_VOID_RETURN
	}

	/* open a message queue descriptor */
	errno = 0;
	mqd = mq_open(mqbuf, O_RDWR|O_CREAT, S_IRWXU, NULL);
	err = errno;
	if (mqd == (mqd_t)-1) {
		xx_rpt(FAILURE);
		in_rpt("mq_open() failed in child for \"%s\", errno = %d (%s)", mqbuf, err, errname(err));
		errors++;
	}
	/* register for notification */
	enotify.sigev_notify = SIGEV_SIGNAL;
	enotify.sigev_signo = SIGRTMIN;

	errno = 0;
	retval = mq_notify(mqd, &enotify);
	err = errno;
	if (retval == -1) {
		xx_rpt(FAILURE);
		in_rpt("mq_notify() failed in child, errno = %d (%s)", err, errname(err));
		errors++;
	}

	if (VSRT_POST_EVENT(GO_PARENT) == -1) {
		(void)mq_close(mqd);
		DBUG_VOID_RETURN
	}

	/*Issue execlp*/
	execlp(execpath("execlp_check"), "execlp_check", "mqueues", (char *)0);

	/*should not reach here*/
	vsrt_sysfail("execlp");

	(void)mq_close(mqd);

	DBUG_VOID_RETURN
}

private void pa_t5()
{
	int	i, result;
	int	err;
	mqd_t	mqd;
	struct sigevent enotify;

	DBUG_ENTER("pa_t5");

	/* open a message queue descriptor */
	errno = 0;
	mqd = mq_open(mqbuf, O_RDWR|O_CREAT, S_IRWXU, NULL);
	err = errno;
	if (mqd == (mqd_t)-1) {
		xx_rpt(FAILURE);
		in_rpt("mq_open() failed in parent for \"%s\", errno = %d (%s)", mqbuf, err, errname(err));
		DBUG_VOID_RETURN
	}

	/* 
	 * wait for child to open the same message queue 
	 * descriptor and register for notification.
	 * At this point the child process execs another
	 * process that just sleeps for a while.
	 */
	if (VSRT_WAIT_EVENT(GO_PARENT) == -1) {
		(void)mq_close(mqd);
		DBUG_VOID_RETURN
	}

	/* 
	 * wait till we hear from the execed program. This
	 * will ensure that execlp() has been invoked.
	 */
	for (i = 0; i < 5; i++) {
		sleep(1);
		if((result = read(fildes[0], reply, sizeof(reply))) >= 0) {
			if ((result != 22) || (strncmp(reply, "EXECED_PROG_IS_RUNNING", 22) != 0)) {
				in_rpt("incorrect data from execed program, expected \"EXEC_PROG_IS_RUNNING\" got \"%s\"", reply);
				xx_rpt(FAILURE);
				(void)mq_close(mqd);
				DBUG_VOID_RETURN 
			}
			break;
		}
	}
	if (i == 5) {
		in_rpt("Execed process did not send any data");
		xx_rpt(FAILURE);
		(void)mq_close(mqd);
		DBUG_VOID_RETURN 
	}

	/* 
	 * now since the execed program is running but not
	 * yet exited, test that the descriptor that was 
	 * opened in the child has been closed. This 
	 * registration should succeed if the descriptor is closed.
	 */
	enotify.sigev_notify = SIGEV_NONE;
	enotify.sigev_signo = SIGRTMIN;
	errno = 0;
	result = mq_notify(mqd, &enotify);
	err = errno;
	if (result == -1) {
		xx_rpt(FAILURE);
		in_rpt("mq_notify() failed in parent, errno = %d (%s)", err, errname(err));
		(void)mq_close(mqd);
		DBUG_VOID_RETURN
	}
	(void)mq_close(mqd);

	/* just return. cppair() will wait for the child */
	DBUG_VOID_RETURN
}
#endif
