/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/procprim/execvp/execvp3.c
*
* Modifications:
* $Log: execvp3.c,v $
* Revision 1.2  2001/10/02 13:25:14  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:19  cyeoh
* Adds files for lsb-os test suite
*
*
*/

/*Exec-ed by execvp.c to test locking as it requires privilege
to set scheduling priority and parameters*/

#include	<vsrt/vsrt.h>
#include	<sched.h>
#include	<sys/mman.h>
#include	<setprv.h>

extern	char	*execpath();
extern	char	*errname();
extern	int	setprv();
extern	char	**environ;

private void subtest3();

public  int tet_main(argc, argv)
int     argc;
char    **argv;
{
	subtest3();

	return 0;
}


#define	NPRIOS	2
int	policy[NPRIOS] = {SCHED_FIFO, SCHED_RR};
char	*policy_name[NPRIOS] = {"SCHED_FIFO", "SCHED_RR"};

int	errors = 0;

char	reply[512];
char	tbuf[32];
int	fildes[2];
int	result;
char	*args[5];

private void subtest3()
{
	int	err;
	int	maxprio, minprio;
	int	i, j;
	struct	sched_param	param;
	pid_t	pid = 0;
	int	status;

	DBUG_ENTER("subtest3");

	if (setprv(PRV_SETRTSCHED) != 0) {
		in_rpt("setprv(PRV_SETRTSCHED) failed");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
       	}
	if (setprv(PRV_GETRTSCHED) != 0) {
		in_rpt("setprv(PRV_GETRTSCHED) failed");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
       	}

	errors = 0;

	for (i = 0; i < NPRIOS; i++) {

		/*get the priority range*/
		maxprio = sched_get_priority_max(policy[i]);
		if (maxprio == -1) {
			err = errno;
			in_rpt("sched_get_priority_max failed for %s, errno = %d (%s)", policy_name[i], err, errname(err));
			xx_rpt(FAILURE);
			errors++;
			continue;
		}
		minprio = sched_get_priority_min(policy[i]);
		if (maxprio == -1) {
			err = errno;
			in_rpt("sched_get_priority_min failed for %s, errno = %d (%s)", policy_name[i], err, errname(err));
			xx_rpt(FAILURE);
			errors++;
			continue;
		}

		/*set the process' priority across the range*/
		for (j = minprio; j <= maxprio; j++) {

			/*Open communication pipe for exec process*/
			if (pipe(fildes) < 0) {
				vsrt_sysfail("pipe");
				DBUG_VOID_RETURN 
			}

			VSRT_FORK(pid)
				/*Connect child's pipe to stdout*/
				close(1);
				if(dup2(fildes[1], 1) < 0) {
					vsrt_sysfail("dup2");
					exit(1);
				}

				param.sched_priority = j;
				if (sched_setscheduler(0, policy[i], &param) == -1) {
					err = errno;
					in_rpt("sched_setscheduler with priority %d failed for %s, errno = %d (%s)", j, policy_name[i], err, errname(err));
					xx_rpt(FAILURE);
					errors++;
					exit(1);
				}

				/*Issue execvp*/
				memset(reply, 0, 16);
				sprintf(reply, "%d", j);
				memset(tbuf, 0, 16);
				sprintf(tbuf, "%d", policy[i]);
				args[0] = "execvp_check";
				args[1] = "scheduling";
				args[2] = tbuf;
				args[3] = reply;
				args[4] = (char *)0;
				execvp(execpath("execvp_check"), args);

				/*should not reach here*/
				err = errno;
				in_rpt("execvp for %s with priority %d failed, errno = %d (%s)", policy_name[i], j, err, errname(err));
				xx_rpt(FAILURE);
				exit(1);
			VSRT_EFORK
			pid = wait(&status);
			if (pid == -1) {
				err = errno;
				in_rpt("fork for %s with priority %d failed, errno = %d (%s)", policy_name[i], j, err, errname(err));
				xx_rpt(FAILURE);
				close(fildes[0]);
				close(fildes[1]);
				continue; 
			}
		
			if(!WIFEXITED(status)) {
				if (WIFSIGNALED(status))
					in_rpt("Child for %s, priority %d terminated due to signal %d", policy_name[i], j, WTERMSIG(status));
				else
					in_rpt("Child for %s, priority %d exited abnormally with status = 0x%x", policy_name[i], j, status);
				close(fildes[0]);
				close(fildes[1]);
				xx_rpt(FAILURE);
				errors++; 
				continue;
			}
		
			if (WEXITSTATUS(status) == 99) {
				in_rpt("Execed process error for %s, priority %d - could not complete test", policy_name[i], j);
				close(fildes[0]);
				close(fildes[1]);
				xx_rpt(FAILURE);
				errors++; 
				continue;
			}
		
			if (status != 0) {
				xx_rpt(FAILURE);
				errors++; 
				continue;
			}
		
			/*Read from exec-ed process to see test result*/
			memset(reply, 0, sizeof(reply));
			if((result = read(fildes[0], reply, sizeof(reply))) < 0) {
				err = errno;
				in_rpt("read for %s with priority %d failed, errno = %d (%s)", policy_name[i], j, err, errname(err));
				xx_rpt(FAILURE);
				errors++; 
			} else
		
				if (result != 4) {
					in_rpt("Scheduling information was not inherited for %s, priority %d", policy_name[i], j);
					in_rpt(reply);
					xx_rpt(FAILURE);
					errors++;
				}
		
			close(fildes[0]);
			close(fildes[1]);
		}
	}

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
}
