/*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
* Project: LSB-OS
*
* File: tset/LSB.os/files/procprim/fork.c
*
* Modifications:
* $Log: fork.c,v $
* Revision 1.4  2001/10/02 10:37:27  ajosey
* remove dependencies outside the LSB specification, from the testcases
*
* Revision 1.3  2001/06/01 03:58:03  cyeoh
* Get test correct way around
*
* Revision 1.2  2001/06/01 03:45:12  cyeoh
* If posix message passing is not implemented on the system
* then the mq_attr struct may not be declared either.
*
* Revision 1.1  2001/05/27 06:59:19  cyeoh
* Adds files for lsb-os test suite
*
*
*/

#include	<time.h>
#include	<vsrt/vsrt.h>
#include	<setjmp.h>
#include	<sys/mman.h>
#ifdef _POSIX_MESSAGE_PASSING
#include	<mqueue.h>
#endif
#ifdef _POSIX_SEMAPHORES
#include	<semaphore.h>
#endif

extern	int	cppair();

#define 	NO_TESTS	10	/* number of tests in testset */

#ifdef UNDEF_MACROS
#undef mlock
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

#define GO_PARENT      	1
#define GO_CHILD	2

#define MAXMSGS			25 /* maximum number of messages in the Q */
#define MAXMSGS_SMALL		2 /* maximum msgs for tests that fill-up Q */
#define MSGSIZE			50 /* maximum message size in the Q */
#define MSGBUF_SIZE		75 /* size of our buffer to receive msgs */
#define DEFAULT_PRIORITY	31 /* priority to use with messages */

extern	char	*errname();
extern	char	*execpath();

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	ch_t2();
private void	ch_t8();

char	shbuf[2048];

char	mqbuf[2048];
char	rcvbuf[MSGBUF_SIZE];
char	segname[] = "vsrt_shmem2";
char	mqname[] = "vsrt_mqueue";

char    sembuf[2048];
char    semname[] = "vsrt_semph";

extern	char	**environ;

/*from vsrtlib/mkmmap.c*/
extern int	vsrt_got_sigsegv;
extern int	vsrt_got_sigbus;
extern int	vsrt_got_sig;
extern unsigned long	vsrt_pgsz;
extern unsigned long	vsrt_ipgsz;
extern int	vsrt_setjmp_called;
extern sigjmp_buf	vsrt_env;

#define	test_file	"vsrt_fork"

void atstart(void);
void atstart(void)
{
	VSRT_BEGIN_EVENTS;
}

void atend(void);
void atend(void)
{
	VSRT_END_EVENTS;
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = atend;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 6,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	NULL, 0
};

#define CHILD_1_READY	1
#define PARENT_READY	2

/*
If _POSIX_SEMAPHORES is defined:
	On a call to fork() all named semaphores that are
	open in the calling process shall also be open in the
	child process.
*/

private void test1(void)
{
#ifdef _POSIX_SEMAPHORES
	sem_t	*semd;
#endif
	pid_t	pid;
	int	status;
        int    	pathok = 0;
	int	errors = 0;

	DBUG_ENTER("test1");

#ifndef _POSIX_SEMAPHORES
	in_rpt("_POSIX_SEMAPHORES not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN 
#else

	errors= 0;

        if (vsrt_get_sem_name(sembuf, semname, sizeof(sembuf)) == -1)
                DBUG_VOID_RETURN
        (void)sem_unlink(sembuf);

	/* open a semaphore */
	semd = sem_open(sembuf, O_CREAT, S_IRWXU, 1);
	if (semd == SEM_FAILED) {
		if (errno == ENOSYS) {
			xx_rpt(UNSUPPORTED);
		}
		else {
		in_rpt("sem_open() failed for \"%s\", errno = %d (%s)", sembuf, errno, errname(errno));
		xx_rpt(FAILURE);
		}
		DBUG_VOID_RETURN
	}
	else
		PATH_TRACE;

	/* lock the semaphore */
	errno = 0;
	if (sem_trywait(semd) == -1) {
		vsrt_sysfail("sem_trywait");
		DBUG_VOID_RETURN
	}
	else
		PATH_TRACE;

        VSRT_FORK(pid)
		/* 
		 * if the semaphore is open in child these
		 * operations should succeed.
		 */
		/* check that the semaphore is locked */
		errno = 0;
		if (sem_trywait(semd) == 0) {
			in_rpt("semaphore locked before fork() found unlocked in child");
			xx_rpt(FAILURE);
			exit(1);
		}
		else
			PATH_TRACE;

		/* unlock the semaphore */
		errno = 0;
		if (sem_post(semd) == -1) {
			vsrt_sysfail("sem_post");
			exit(1);
		}
		else
			PATH_TRACE;

		/* close the semaphore */
		errno = 0;
		if (sem_close(semd) == -1) {
			vsrt_sysfail("sem_close");
			exit(1);
		}
		else
			PATH_TRACE;
	VSRT_KROF2(pid, status);
	if (status == -1) {
		errors++;
	} else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	}

	/* clean up */
	(void)sem_close(semd);
        (void)sem_unlink(sembuf);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
#endif
}

/*
If _POSIX_MEMLOCK is defined:
	On a call to fork() the child process shall not
	inherit any address space memory locks established by
	the parent process via calls to mlockall() or
	mlock().
*/

private void test2()
{
	DBUG_ENTER("test2");

#ifndef _POSIX_MEMLOCK
	in_rpt("_POSIX_MEMLOCK not defined");
	xx_rpt(UNSUPPORTED);
#else
	(void) cppair(ch_t2, NULLFN, 4 * WAITTIME, 0);
#endif

	DBUG_VOID_RETURN 
} 

private void ch_t2()
{
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t2");

	args[0] = "fork2";
	args[1] = NULL;
	(void) tet_exec(execpath("fork2"), args, environ);

	err = errno;
	xx_rpt(FAILURE);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)", "fork2", err, errname(err));

	DBUG_VOID_RETURN 
}

/*
If _POSIX_MAPPED_FILES
is defined:
	On a call to fork() memory mappings created in the
	parent process shall be retained in the child process.
*/

private int test3p1();
private int test3p2();
private int dotest3();

private void test3()
{
	int	ret1, ret2;

	DBUG_ENTER("test3");

#if !defined(_POSIX_MAPPED_FILES) && !defined(_POSIX_SHARED_MEMORY_OBJECTS)
	in_rpt("_POSIX_MAPPED_FILES and _POSIX_SHARED_MEMORY_OBJECTS not defined");
	xx_rpt(UNSUPPORTED);
#else

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	ret1 = test3p1();
	ret2 = test3p2();

	/*any error is a failure*/
	if ((ret1 == -1) || (ret2 == -1)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	/*if both parts cannot be run its unsupported*/
	if ((ret1 == 1) && (ret2 == 1)) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	/*if either part is run successfully its a pass*/
	xs_rpt();

#endif
	DBUG_VOID_RETURN 
} 

/*test for files*/
private int test3p1()
{
	int	fd, ret;

	DBUG_ENTER("test3p1");

#ifndef _POSIX_MAPPED_FILES
	in_rpt("_POSIX_MAPPED_FILES not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_RETURN(1);
#else

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_RETURN(-1);

	ret = dotest3(fd);

	(void)close(fd);
	(void)unlink(test_file);

	DBUG_RETURN(ret);
#endif
}

/*test for shared memory*/
private int test3p2()
{
	int	fd, ret;

	DBUG_ENTER("test3p2");
	DBUG_RETURN(1);
}

private int dotest3(fd)
int	fd;
{
	int	pathok = 0;
	volatile int	i;
	int	*adr1;
	pid_t	pid;
	int 	status;

	DBUG_ENTER("dotest3");

	/*map in parent*/
	if ((adr1 = (int *)mmap(0, (size_t)(vsrt_pgsz), PROT_READ, MAP_SHARED, fd, 0)) == (int *)MAP_FAILED) {
		vsrt_sysfail("mmap");
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	/*access in child*/
	VSRT_FORK(pid);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0)
			i = *adr1;
		if (vsrt_got_sigsegv) {
			in_rpt("Accessing mapped page generated SIGBUS");
			exit(1);
		}
		if (vsrt_got_sigbus) {
			in_rpt("Accessing mapped page generated SIGSEGV");
			exit(1);
		}
	VSRT_KROF2(pid, status);
	if (status == -1) {
		(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
		DBUG_RETURN(-1);
	} else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process 1 exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
		DBUG_RETURN(-1);
	}

	(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
	DBUG_RETURN(0);
} 


/*
If _POSIX_MAPPED_FILES
is defined:
	On a call to fork() MAP_PRIVATE mappings inherited
	from the parent shall also be MAP_PRIVATE mappings in
	the child.
*/

private int test4p1();
private int test4p2();

private void test4()
{
	int	ret1, ret2;

	DBUG_ENTER("test4");

#if !defined(_POSIX_MAPPED_FILES) && !defined(_POSIX_SHARED_MEMORY_OBJECTS)
	in_rpt("_POSIX_MAPPED_FILES and _POSIX_SHARED_MEMORY_OBJECTS not defined");
	xx_rpt(UNSUPPORTED);
#else

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	ret1 = test4p1();
	ret2 = test4p2();

	/*any error is a failure*/
	if ((ret1 == -1) || (ret2 == -1)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	/*if both parts cannot be run its unsupported*/
	if ((ret1 == 1) && (ret2 == 1)) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	/*if either part is run successfully its a pass*/
	xs_rpt();

#endif
	DBUG_VOID_RETURN 
} 

/*test for files*/
private int test4p1()
{
	int	fd, fd1;
	int	pathok = 0;
	char	*adr1;
	pid_t	pid;
	int 	status;
	int	j;
	int	errors;
	char	input;

	DBUG_ENTER("test4p1");

#ifndef _POSIX_MAPPED_FILES
	in_rpt("_POSIX_MAPPED_FILES not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_RETURN(1); 
#else

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_RETURN(-1);

	/*initialize contents*/
	if ((adr1 = (char *)mmap(0, (size_t)(vsrt_pgsz), PROT_WRITE, MAP_SHARED, fd, 0)) == (char *)MAP_FAILED) {
		vsrt_sysfail("mmap");
		(void)close(fd);
		(void)unlink(test_file);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	for (j = 0; j < vsrt_pgsz; j++) 
		adr1[j] = 'B';

	if (msync(adr1, vsrt_pgsz, MS_SYNC) == -1) {
		vsrt_sysfail("msync");
		(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
		(void)close(fd);
		(void)unlink(test_file);
		DBUG_RETURN(-1);
	}

	if (munmap(adr1, vsrt_pgsz) == -1) {
		vsrt_sysfail("munmap");
		(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
		(void)close(fd);
		(void)unlink(test_file);
		DBUG_RETURN(-1);
	}

	/*map private in parent*/
	if ((adr1 = (char *)mmap(0, (size_t)(vsrt_pgsz), PROT_WRITE|PROT_READ, MAP_PRIVATE, fd, 0)) == (char *)MAP_FAILED) {
#ifndef _XOPEN_SOURCE
		if (errno == ENOTSUP) {
			in_rpt("MAP_PRIVATE not supported");
			xx_rpt(UNSUPPORTED);
			(void)close(fd);
			(void)unlink(test_file);
			DBUG_RETURN(1);
		}
#endif
		vsrt_sysfail("mmap");
		(void)close(fd);
		(void)unlink(test_file);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	/*change in child*/
	VSRT_FORK(pid);
		for (j = 0; j < vsrt_pgsz; j++) 
			adr1[j] = 'A';

		if (msync(adr1, vsrt_pgsz, MS_SYNC) == -1) {
			vsrt_sysfail("msync");
			exit(1);
		}

		if ((fd1 = open(test_file, O_RDWR, VSRT_PROT_ALL)) == -1) {
			vsrt_sysfail("open");
			exit(1);
		} else

		errors = 0;
		/*Write references do not change the file*/
		for (j = 0; j < vsrt_pgsz; j++) {
			input = 0;
			if (read(fd, &input, sizeof(char)) != sizeof(char)) {
				vsrt_sysfail("read");
				(void)close(fd1);
				exit(1);
			}
			if (input == 'A') {
				in_rpt("Offset %d in object was changed, expected %x, is %x", j, 'B', input);
				xx_rpt(FAILURE);
				errors++;
			}
		}
		if (errors != 0)
			exit(errors);
	VSRT_KROF2(pid, status);
	if (status == -1) {
		(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
		(void)close(fd);
		(void)unlink(test_file);
		DBUG_RETURN(-1);
	} else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process 1 exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
		(void)close(fd);
		(void)unlink(test_file);
		DBUG_RETURN(-1);
	}

	(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
	(void)close(fd);
	(void)unlink(test_file);

	DBUG_RETURN(0);
#endif
}

/*test for shared memory*/
private int test4p2()
{
	DBUG_ENTER("test4p2");
	DBUG_RETURN(1);
}

/*
If _POSIX_MAPPED_FILES 
is defined:
	Any modifications to MAP_PRIVATE mappings made by the
	parent process prior to calling fork() shall be visi-
	ble to the child process.
*/

private int test5p1();
private int test5p2();
private int dotest5();

private void test5()
{
	int	ret1, ret2;

	DBUG_ENTER("test5");

#if !defined(_POSIX_MAPPED_FILES) && !defined(_POSIX_SHARED_MEMORY_OBJECTS)
	in_rpt("_POSIX_MAPPED_FILES and _POSIX_SHARED_MEMORY_OBJECTS not defined");
	xx_rpt(UNSUPPORTED);
#else

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	ret1 = test5p1();
	ret2 = test5p2();

	/*any error is a failure*/
	if ((ret1 == -1) || (ret2 == -1)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	/*if both parts cannot be run its unsupported*/
	if ((ret1 == 1) && (ret2 == 1)) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	/*if either part is run successfully its a pass*/
	xs_rpt();

#endif
	DBUG_VOID_RETURN 
} 

/*test for files*/
private int test5p1()
{
	int	fd, ret;

	DBUG_ENTER("test5p1");

#ifndef _POSIX_MAPPED_FILES
	in_rpt("_POSIX_MAPPED_FILES not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_RETURN(1);
#else

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_RETURN(-1);

	ret = dotest5(fd);

	(void)close(fd);
	(void)unlink(test_file);

	DBUG_RETURN(ret);
#endif
}

/*test for shared memory*/
private int test5p2()
{
	int	fd, ret;

	DBUG_ENTER("test5p2");
	xx_rpt(UNSUPPORTED);
	DBUG_RETURN(1);
}

private int dotest5(fd)
int	fd;
{
	int	pathok = 0;
	int	j;
	char	*adr1;
	pid_t	pid;
	int 	status;
	int	errors;

	DBUG_ENTER("dotest5");

	/*map in parent*/
	if ((adr1 = (char *)mmap(0, (size_t)(vsrt_pgsz), PROT_READ|PROT_WRITE, MAP_PRIVATE, fd, 0)) == (char *)MAP_FAILED) {
#ifndef _XOPEN_SOURCE
		if (errno == ENOTSUP) {
			in_rpt("MAP_PRIVATE not supported");
			xx_rpt(UNSUPPORTED);
			DBUG_RETURN(1);
		}
#endif
		vsrt_sysfail("mmap");
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	/*modify in parent*/
	for (j = 0; j < vsrt_pgsz; j++)
		adr1[j] = 'A';

	/*access in child*/
	VSRT_FORK(pid);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			errors = 0;
			for (j = 0; j < vsrt_pgsz; j++) {
				if (adr1[j] != 'A') {
					in_rpt("Offset %d in object not correct, expected %x, is %x", j, 'A', adr1[j]);
					errors++;
				}
			}
		}
		if (vsrt_got_sigsegv != 0) {
			in_rpt("Accessing mapped page in child generated SIGBUS at offset %d", j);
			exit(1);
		}
		if (vsrt_got_sigbus != 0) {
			in_rpt("Accessing mapped page in child generated SIGSEGV at offset %d", j);
			exit(1);
		}
		if (errors != 0)
			exit(errors);
	VSRT_KROF2(pid, status);
	if (status == -1) {
		(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
		DBUG_RETURN(-1);
	} else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process 1 exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
		DBUG_RETURN(-1);
	}

	(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
	DBUG_RETURN(0);
} 

/*
If _POSIX_MAPPED_FILES 
is defined:
	Any modifications to MAP_PRIVATE mappings made by the
	parent process after calling fork() shall not be
	visible to the child process.
*/

private int test6p1();
private int test6p2();
private int dotest6();

private void test6()
{
	int	ret1, ret2;

	DBUG_ENTER("test6");

#if !defined(_POSIX_MAPPED_FILES) && !defined(_POSIX_SHARED_MEMORY_OBJECTS)
	in_rpt("_POSIX_MAPPED_FILES and _POSIX_SHARED_MEMORY_OBJECTS not defined");
	xx_rpt(UNSUPPORTED);
#else

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	ret1 = test6p1();
	ret2 = test6p2();

	/*any error is a failure*/
	if ((ret1 == -1) || (ret2 == -1)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	/*if both parts cannot be run its unsupported*/
	if ((ret1 == 1) && (ret2 == 1)) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	/*if either part is run successfully its a pass*/
	xs_rpt();

#endif
	DBUG_VOID_RETURN 
} 

/*test for files*/
private int test6p1()
{
	int	fd, ret;

	DBUG_ENTER("test6p1");

#ifndef _POSIX_MAPPED_FILES
	in_rpt("_POSIX_MAPPED_FILES not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_RETURN(1);
#else
	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_RETURN(-1);

	ret = dotest6(fd);

	(void)close(fd);
	(void)unlink(test_file);

	DBUG_RETURN(ret);
#endif
}

/*test for shared memory*/
private int test6p2()
{
	int	fd, ret;

	DBUG_ENTER("test6p2");

	xx_rpt(UNSUPPORTED);
	DBUG_RETURN(1);
}

private int dotest6(fd)
int	fd;
{
	int	pathok = 0;
	int	j;
	char	*adr1;
	pid_t	pid;
	int 	status;
	int	errors;

	DBUG_ENTER("dotest6");

	/*map in parent*/
	if ((adr1 = (char *)mmap(0, (size_t)(vsrt_pgsz), PROT_READ|PROT_WRITE, MAP_PRIVATE, fd, 0)) == (char *)MAP_FAILED) {
#ifndef _XOPEN_SOURCE
		if (errno == ENOTSUP) {
			in_rpt("MAP_PRIVATE not supported");
			xx_rpt(UNSUPPORTED);
			DBUG_RETURN(1);
		}
#endif
		vsrt_sysfail("mmap");
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	/*modify in parent*/
	for (j = 0; j < vsrt_pgsz; j++)
		adr1[j] = 'A';

	/*access in child*/
	VSRT_FORK(pid);
		VSRT_POST_EVENT(CHILD_1_READY);
		VSRT_WAIT_EVENT(PARENT_READY);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			errors = 0;
			for (j = 0; j < vsrt_pgsz; j++) {
				if (adr1[j] != 'A') {
					in_rpt("Offset %d in object not correct, expected %x, is %x", j, 'A', adr1[j]);
					errors++;
				}
			}
		}
		if (vsrt_got_sigsegv != 0) {
			in_rpt("Accessing mapped page in child generated SIGBUS at offset %d", j);
			exit(1);
		}
		if (vsrt_got_sigbus != 0) {
			in_rpt("Accessing mapped page in child generated SIGSEGV at offset %d", j);
			exit(1);
		}
		if (errors != 0)
			exit(errors);
	VSRT_EFORK;

	VSRT_WAIT_EVENT(CHILD_1_READY);

	/*modify in parent after fork*/
	for (j = 0; j < vsrt_pgsz; j++)
		adr1[j] = 'C';

	VSRT_POST_EVENT(PARENT_READY);

	status = VSRT_WAIT_FOR_PID(pid);
	if (status == -1) {
		(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
		DBUG_RETURN(-1);
	} else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process 1 exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
		DBUG_RETURN(-1);
	}

	(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
	DBUG_RETURN(0);
} 

/*
If _POSIX_MAPPED_FILES or _POSIX_SHARED_MEMORY_OBJECTS
is defined:
	Any modifications to MAP_PRIVATE mappings made by the
	child process after a call to fork() shall be visible
	only to the child process.
*/

private int test7p1();
private int test7p2();
private int dotest7();

private void test7()
{
	int	ret1, ret2;

	DBUG_ENTER("test7");

#if !defined(_POSIX_MAPPED_FILES) && !defined(_POSIX_SHARED_MEMORY_OBJECTS)
	in_rpt("_POSIX_MAPPED_FILES and _POSIX_SHARED_MEMORY_OBJECTS not defined");
	xx_rpt(UNSUPPORTED);
#else

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN

	ret1 = test7p1();
	ret2 = test7p2();

	/*any error is a failure*/
	if ((ret1 == -1) || (ret2 == -1)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	/*if both parts cannot be run its unsupported*/
	if ((ret1 == 1) && (ret2 == 1)) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	/*if either part is run successfully its a pass*/
	xs_rpt();

#endif
	DBUG_VOID_RETURN 
} 

/*test for files*/
private int test7p1()
{
	int	fd, ret;

	DBUG_ENTER("test7p1");

#ifndef _POSIX_MAPPED_FILES
	in_rpt("_POSIX_MAPPED_FILES not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_RETURN(1);
#else

	fd = vsrt_open_file(test_file, O_RDWR, VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_RETURN(-1);

	ret = dotest7(fd);

	(void)close(fd);
	(void)unlink(test_file);

	DBUG_RETURN(ret);
#endif
}

/*test for shared memory*/
private int test7p2()
{
	int	fd, ret;

	DBUG_ENTER("test7p2");

	xx_rpt(UNSUPPORTED);
	DBUG_RETURN(1);
}

private int dotest7(fd)
int	fd;
{
	int	pathok = 0;
	int	j;
	char	*adr1;
	pid_t	pid;
	int 	status;
	int	errors;

	DBUG_ENTER("dotest7");

	/*map in parent*/
	if ((adr1 = (char *)mmap(0, (size_t)(vsrt_pgsz), PROT_READ|PROT_WRITE, MAP_PRIVATE, fd, 0)) == (char *)MAP_FAILED) {
#ifndef _XOPEN_SOURCE
		if (errno == ENOTSUP) {
			in_rpt("MAP_PRIVATE not supported");
			xx_rpt(UNSUPPORTED);
			DBUG_RETURN(1);
		}
#endif
		vsrt_sysfail("mmap");
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	/*modify in parent*/
	for (j = 0; j < vsrt_pgsz; j++)
		adr1[j] = 'A';

	/*change in child*/
	VSRT_FORK(pid);
		VSRT_WAIT_EVENT(PARENT_READY);
		vsrt_got_sigsegv = vsrt_got_sigbus = vsrt_got_sig = 0;
		vsrt_setjmp_called = 1;
		if (sigsetjmp(vsrt_env,1) == 0) {
			for (j = 0; j < vsrt_pgsz; j++)
				adr1[j] = 'C';
		}
		if (vsrt_got_sigsegv != 0) {
			in_rpt("Accessing mapped page in child generated SIGBUS at offset %d", j);
			exit(1);
		}
		if (vsrt_got_sigbus != 0) {
			in_rpt("Accessing mapped page in child generated SIGSEGV at offset %d", j);
			exit(1);
		}
		VSRT_POST_EVENT(CHILD_1_READY);
	VSRT_EFORK;

	VSRT_POST_EVENT(PARENT_READY);

	VSRT_WAIT_EVENT(CHILD_1_READY);

	status = VSRT_WAIT_FOR_PID(pid);
	if (status == -1) {
		(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
		DBUG_RETURN(-1);
	} else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process 1 exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		(void)munmap((void *)adr1, (size_t)vsrt_pgsz);
		DBUG_RETURN(-1);
	}

	/*check for child's changes*/
	errors = 0;
	for (j = 0; j < vsrt_pgsz; j++) {
		if (adr1[j] != 'A') {
			in_rpt("Offset %d in object not correct, expected %x, is %x", j, 'A', adr1[j]);
			errors++;
		}
	}

	(void)munmap((void *)adr1, (size_t)vsrt_pgsz);

	DBUG_RETURN(errors);
} 

/*
If _POSIX_PRIORITY_SCHEDULING is defined:
	On a call to fork(), for the SCHED_FIFO and SCHED_RR
	scheduling policies, the child process shall inherit
	the policy and priority settings of the parent pro-
	cess.
*/

private void test8()
{
	DBUG_ENTER("test8");

#ifndef _POSIX_PRIORITY_SCHEDULING
	in_rpt("_POSIX_PRIORITY_SCHEDULING not defined");
	xx_rpt(UNSUPPORTED);
#else
	(void) cppair(ch_t8, NULLFN, 4 * WAITTIME, 0);

#endif
	DBUG_VOID_RETURN 
} 

private void ch_t8()
{
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t8");

	args[0] = "fork8";
	args[1] = NULL;
	(void) tet_exec(execpath("fork8"), args, environ);

	err = errno;
	xx_rpt(FAILURE);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)", "fork8", err, errname(err));

	DBUG_VOID_RETURN 
}

/*
If _POSIX_TIMERS is defined:
	On a call to fork(), per-process timers created by
	the parent process shall be not be inherited by the
	child process.
*/

private void test9()
{
	int	errors = 0;
	timer_t	timerid;
	struct	itimerspec	value;
	struct sigevent	ev;
	pid_t	pid;
	int 	status;

	DBUG_ENTER("test9");

#ifndef _POSIX_TIMERS
	in_rpt("_POSIX_TIMERS not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN 
#else
	
	ev.sigev_notify = SIGEV_NONE;

	if (timer_create(CLOCK_REALTIME, &ev, &timerid) == -1) {
		vsrt_sysfail("timer_create");
		DBUG_VOID_RETURN 
	}

	VSRT_FORK(pid);
		if (timer_gettime(timerid, &value) != -1) {
			in_rpt("timer_gettime succeeded in child process");
			xx_rpt(FAILURE);
			exit(1); 
		}
		if (errno != EINVAL) {
			vsrt_sysfail("timer_gettime");
			exit(1); 
		}
	VSRT_KROF2(pid, status);
	if (status == -1) {
		errors++;
	} else
	if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process 1 exited due to signal %d", WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d", WEXITSTATUS(status));
		errors++;
	} else
		errors += status;

	if (timer_delete(timerid) == -1) {
		vsrt_sysfail("timer_delete");
		DBUG_VOID_RETURN 
	}

	if (errors == 0)
		xs_rpt();
#endif
	DBUG_VOID_RETURN 
} 


/*
If _POSIX_MESSAGE_PASSING is defined:
	On a call to fork(), the child process shall have its
	own copy of the message queue descriptors of the
	parent process. Each of the message queue descrip-
	tors of the child shall refer to the same open mes-
	sage queue description as the corresponding message
	queue descriptor of the parent.
*/

#define T10_NUM_MSGS		3

char    *t10_pdata[] = {
	"A test message to the Q",
	"A second message to the Q",
	"Yet another message to the Q"
};

private void test10()
{
#ifdef _POSIX_MESSAGE_PASSING
	mqd_t		mqd;
#endif
	pid_t		pid;
	int		i, err;
	unsigned int	mqprio;
#ifdef _POSIX_MESSAGE_PASSING
	struct	mq_attr	attr;
#endif
	ssize_t		msgsize;
        int     	pathok = 0;
	int		errors = 0;

	DBUG_ENTER("test10");

#ifndef _POSIX_MESSAGE_PASSING
	in_rpt("_POSIX_MESSAGE_PASSING not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN 
#else

	errors= 0;

        if (vsrt_get_mqueue_name(mqbuf, mqname, sizeof(mqbuf)) == -1)
                DBUG_VOID_RETURN
        (void)mq_unlink(mqbuf);

	mqprio = DEFAULT_PRIORITY;

	attr.mq_flags = 0;
	attr.mq_maxmsg = MAXMSGS;
	attr.mq_msgsize = MSGSIZE;

        errno = 0;
        mqd = mq_open(mqbuf, O_RDWR|O_CREAT|O_NONBLOCK, S_IRWXU, &attr);
        err = errno;
        if (mqd == (mqd_t)-1) {
                xx_rpt(FAILURE);
                in_rpt("mq_open() failed for \"%s\", errno = %d (%s)", mqbuf, err, errname(err));
                DBUG_VOID_RETURN
        }
        else
                PATH_TRACE;


        VSRT_FORK(pid)	
		/* child */
                if (VSRT_WAIT_EVENT(GO_CHILD) == -1)
                        exit(1);

		/* read a message off the queue */
		errno = 0;
		if ((msgsize = mq_receive(mqd, rcvbuf, MSGBUF_SIZE, NULL)) == -1) {
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("mq_receive() failed in child1, errno = %d (%s)", err, errname(err));
			errors++;
		}
		else {
			PATH_TRACE;
			/*check it*/
			if (msgsize != strlen(t10_pdata[0])) {
				xx_rpt(FAILURE);
				in_rpt("length of message received in child1: expected %d, got %d", strlen(t10_pdata[0]), msgsize);
				errors++;
			}
			rcvbuf[msgsize] = '\0';
			if (strncmp(t10_pdata[0], rcvbuf, strlen(t10_pdata[0])) != 0) {
				xx_rpt(FAILURE);
				in_rpt("message received in child1: expected \"%s\", got \"%s\"", t10_pdata[0], rcvbuf);
				errors++;
			}
			else
				PATH_TRACE;
		}
                if (VSRT_POST_EVENT(GO_PARENT) == -1)
                        exit(1);
        VSRT_EFORK
	/* send multiple messages to the queue */
	for (i = 0; i < T10_NUM_MSGS; i++) {
		DBUG_PRINTF("mq_send()", "Msg: %s size:%d", t10_pdata[i], strlen(t10_pdata[i]));
		errno = 0;
		if (mq_send(mqd, (const char *)t10_pdata[i], strlen(t10_pdata[i]), mqprio) == -1) {
			vsrt_sysfail("mq_send");
			kill(pid, SIGKILL);
			VSRT_WAIT_FOR_PID(pid);
        		(void)mq_close(mqd);
        		(void)mq_unlink(mqbuf);
			DBUG_VOID_RETURN
		}
		else
			PATH_TRACE;
	}
	/* signal child which reads messages sent by the parent. */
	if (VSRT_POST_EVENT(GO_CHILD) == -1) {
		kill(pid, SIGKILL);
		VSRT_WAIT_FOR_PID(pid);
		(void)mq_close(mqd);
		(void)mq_unlink(mqbuf);
		DBUG_VOID_RETURN
	}
	if (VSRT_WAIT_EVENT(GO_PARENT) == -1) {
		kill(pid, SIGKILL);
		VSRT_WAIT_FOR_PID(pid);
		(void)mq_close(mqd);
		(void)mq_unlink(mqbuf);
		DBUG_VOID_RETURN
	}

	/* check that the message was read */
	errno = 0;
	if (mq_getattr(mqd, &attr) == -1) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("parent:mq_getattr() failed for \"%s\", errno = %d (%s)", mqbuf, err, errname(err));
		errors++;
	} else {
		PATH_TRACE;
		if (attr.mq_curmsgs != 2) {
			in_rpt("parent: expected 0 messages, found %ld", attr.mq_curmsgs);
			xx_rpt(FAILURE);
			errors++;
		}
	}
	
	/* wait for children to exit */
	VSRT_WAIT_FOR_PID(pid);

        (void)mq_close(mqd);
        (void)mq_unlink(mqbuf);

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
#endif
} 
