/*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
* Project: LSB-OS
*
* File: tset/LSB.os/procprim/fork/fork2.c
*
* Modifications:
* $Log: fork2.c,v $
* Revision 1.2  2001/10/02 10:37:27  ajosey
* remove dependencies outside the LSB specification, from the testcases
*
* Revision 1.1  2001/05/27 06:59:19  cyeoh
* Adds files for lsb-os test suite
*
*/

/*Exec-ed by fork.c to test locking as it requires privilege
to lock memory*/

#include	<vsrt/vsrt.h>
#include	<sys/mman.h>
#include	<setprv.h>

extern	int	setprv();
extern	char	*errname();

private void subtest2();

int	errors = 0;

/*from vsrtlib/mkmmap.c*/
extern int	vsrt_got_sigsegv;
extern int	vsrt_got_sigbus;
extern int	vsrt_got_sig;
extern unsigned long	vsrt_pgsz;
extern unsigned long	vsrt_ipgsz;
extern int	vsrt_setjmp_called;

#define	test_file	"vsrt_fork"

#define PARENT_READY	1

public  int tet_main(argc, argv)
int     argc;
char    **argv;
{
	if (vsrt_alloc_sem() == -1)
		return 1;
	subtest2();
	VSRT_END_EVENTS;

	return 0;
}

private void subtest2()
{
	int	fd;
	int	pathok = 0;
	void	*addr;
	pid_t	pid;
	int	status;

	DBUG_ENTER("subtest2");

	if (setprv(PRV_MEMLOCK) != 0) {
		xx_rpt(FAILURE);
		in_rpt("setprv(PRV_MEMLOCK) failed");
		DBUG_VOID_RETURN 
        }

	if (vsrt_mmap_setup() == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	fd = vsrt_open_file(test_file, O_RDWR, (mode_t)VSRT_PROT_ALL, 2);
	if (fd == -1)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

	/*map a page*/
	if ((addr = mmap(0, vsrt_pgsz, PROT_READ, MAP_SHARED, fd, 0)) == MAP_FAILED) {
		vsrt_sysfail("mmap");
		DBUG_VOID_RETURN
	}
	else
		PATH_TRACE;

	/*lock it*/
	if (mlock(addr, vsrt_pgsz) == -1) {
		vsrt_sysfail("mlock");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

VSRT_FORK(pid);
	/*wait for the parent to unlock its lock*/
	VSRT_WAIT_EVENT(PARENT_READY);

	/*should succeed as mapping is inherited but lock is not*/
	if (msync(addr, vsrt_pgsz, MS_INVALIDATE) == -1) {
		if (errno == EBUSY) {
			in_rpt("Memory lock set by mlock inherited by child");
			xx_rpt(FAILURE);
		} else
			vsrt_sysfail("msync");
		errors++;
	}
VSRT_EFORK;

	/*clear our lock*/
	(void)munlock(addr, vsrt_pgsz);

	/*tell child to do test*/
	VSRT_POST_EVENT(PARENT_READY);

	/*wait for test to complete*/
	VSRT_WAIT_FOR_PID(pid);

	/*now test with mlockall*/
	if (mlockall(MCL_CURRENT) == -1) {
		vsrt_sysfail("mlockall");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
	}

VSRT_FORK(pid);
	/*wait for the parent to unlock its lock*/
	VSRT_WAIT_EVENT(PARENT_READY);

	/*should succeed as mapping is inherited but lock is not*/
	if (msync(addr, vsrt_pgsz, MS_INVALIDATE) == -1) {
		if (errno == EBUSY) {
			in_rpt("Memory lock set by mlockall inherited by child");
			xx_rpt(FAILURE);
		} else
			vsrt_sysfail("msync");
		errors++;
	}
VSRT_EFORK;

	/*clear our lock*/
	(void)munlock(addr, vsrt_pgsz);

	/*tell child to do test*/
	VSRT_POST_EVENT(PARENT_READY);

	/*wait for test to complete*/
	VSRT_WAIT_FOR_PID(pid);

	(void)close(fd);
	(void)unlink(test_file);

	if (errors == 0)
		xs_rpt();

	DBUG_VOID_RETURN 
}
