/*
* Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
* Project: LSB-OS
*
* File: tset/LSB.os/procprim/fork/fork8.c
*
* Modifications:
* $Log: fork8.c,v $
* Revision 1.2  2001/10/02 10:37:27  ajosey
* remove dependencies outside the LSB specification, from the testcases
*
* Revision 1.1  2001/05/27 06:59:19  cyeoh
* Adds files for lsb-os test suite
*
*
*/

/*Exec-ed by fork.c to test locking as it requires privilege
to set scheduling priority and parameters*/

#include	<vsrt/vsrt.h>
#ifdef _POSIX_PRIORITY_SCHEDULING
#include	<sched.h>
#endif
#include	<sys/mman.h>
#include	<setprv.h>

extern	int	setprv();
extern	char	*errname();

private void subtest8();

public  int tet_main(argc, argv)
int     argc;
char    **argv;
{

#ifdef _POSIX_PRIORITY_SCHEDULING
	subtest8();
#else
	in_rpt("Process Scheduling not supported");
	xx_rpt(NOTINUSE);
#endif
	return 0;
}


#ifdef _POSIX_PRIORITY_SCHEDULING

#define	NPRIOS	3
int	policy[NPRIOS] = {SCHED_FIFO, SCHED_RR, SCHED_OTHER};
char	*policy_name[NPRIOS] = {"SCHED_FIFO", "SCHED_RR", "SCHED_OTHER"};

int	errors = 0;

private void subtest8()
{
	int	err;
	int	retval, maxprio, minprio;
	int	i, j;
	struct	sched_param	param, param2;
	pid_t	pid = 0;
	int	status;

	DBUG_ENTER("subtest8");

	if (setprv(PRV_SETRTSCHED) != 0) {
		in_rpt("setprv(PRV_SETRTSCHED) failed");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
       	}
	if (setprv(PRV_GETRTSCHED) != 0) {
		in_rpt("setprv(PRV_GETRTSCHED) failed");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN 
       	}

	errors = 0;

	for (i = 0; i < NPRIOS; i++) {

		/*get the priority range*/
		maxprio = sched_get_priority_max(policy[i]);
		if (maxprio == -1) {
			err = errno;
			in_rpt("sched_get_priority_max failed for %s, errno = %d (%s)", policy_name[i], err, errname(err));
			xx_rpt(FAILURE);
			errors++;
			continue;
		}
		minprio = sched_get_priority_min(policy[i]);
		if (maxprio == -1) {
			err = errno;
			in_rpt("sched_get_priority_min failed for %s, errno = %d (%s)", policy_name[i], err, errname(err));
			xx_rpt(FAILURE);
			errors++;
			continue;
		}

		/*set the process' priority across the range*/
		for (j = minprio; j <= maxprio; j++) {

			param.sched_priority = j;
			if (sched_setscheduler(0, policy[i], &param) == -1) {
				err = errno;
				in_rpt("sched_setscheduler with priority %d failed for %s, errno = %d (%s)", j, policy_name[i], err, errname(err));
				xx_rpt(FAILURE);
				errors++;
				continue;
			}

			VSRT_FORK(pid);
				/*check the priority*/
				retval = sched_getparam(pid, &param2);
				if (retval == -1) {
					err = errno;
					in_rpt("sched_getparam failed for %s for priority %d, errno = %d (%s)", policy_name[i], j, err, errname(err));
					xx_rpt(FAILURE);
					errors++;
				}
		
				if (param2.sched_priority != j) {
					in_rpt("For %s, set priority to %d, received %d from sched_getparam", policy_name[i], j, param2.sched_priority);
					xx_rpt(FAILURE);
					errors++;
				}
			VSRT_KROF2(pid, status);
			if (status == -1)
				errors++;
			else if (!WIFEXITED(status)) {
				if (WIFSIGNALED(status) != 0)
					in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
				else
					in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
				xx_rpt(FAILURE);
				errors++;
			} else
				errors += WEXITSTATUS(status);
		}
	}

	if (errors == 0) {
		xs_rpt();
	}

	DBUG_VOID_RETURN 
}
#endif /* _POSIX_PRIORITY_SCHEDULING*/
