/*
*      SCCS:  @(#)  GABI4.thr/procprim/fork/fork_uids.c
*
*	UniSoft Ltd., London, England
*
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)GABI4.thr/procprim/fork - fork_uids";
char	*copyright[] = {
		"(C) Copyright 1995 UniSoft Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		fork_uids - used by T.fork in several tests

PROJECT:	SCD

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.fork to user and group ID'S after fork().
	  usage:
		fork_uids savestr vg_ident vg_dbug

	  where savestr indicates whether POSIX_SAVED_IDS to be used.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		John-Paul, Unisoft Ltd.
DATE CREATED:	Fri Apr 19 11:35:01 BST 1996
MODIFICATIONS:	Derived from fork()

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<errno.h>
#include	<vsx_signal.h>
#include	<tet_api.h>
#include	<sysdep.h>

#define	WAITTIME		(5 * SPEEDFACTOR)

extern	char	**environ;

extern	void	set_pname();
extern	char	*status_child();

extern	int	ex_code;	/* defined in wait_child.c */

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	int	test_fail;
private	int	test_deleted;

private	void	sub_uids();
private	void	test_uids();
private	void	test_saveids();
private	void	wait_for_child();

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"fork_uids" : (argv[0]));
	switch(argc)
	{
	case 4:
		vg_dbug = argv[3];
		/* fall through */
	case 3:
		vg_ident = argv[2];
		break;
	default:
		vsxerror("argument count wrong");
		exit(A_MISC);
	}

	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	sub_uids(atoi(argv[1]));

	exit(OK);

	/* NOTREACHED */

	return(0);
}

private void
sub_uids(saveids)
int	saveids;
{
	int	err;
	int	ruid;
	int	euid;
	int	rgid;
	int	egid;
	int	pathok = 0;

	DBUG_ENTER("sub_uids");

	DBUG_PRINTF("sub_uids", "ruid is %d", getuid());
	DBUG_PRINTF("sub_uids", "euid is %d", geteuid());

	/* Obtain and check real and effective uids */
	ruid = getuid();
	euid = geteuid();
	rgid = getgid();
	egid = getegid();

	if(ruid == euid) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - real and effective user ID both %d", 
									ruid);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(rgid == egid) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - real and effective group ID both %d", 
									rgid);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* If saved set-user ID's being tested, check them out */
	if(saveids != SYSERROR) {

		/* Change the effective user ID to the real */
		if(setuid((uid_t)ruid) == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - setuid(ruid) failed errno %d",
								err);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		if(geteuid() != ruid || getuid() != ruid) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - real and effective user ID not both %d", ruid);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Try out the saved set-user ID, euid */
		if(setuid((uid_t)euid) == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - setuid(euid) failed errno %d",
								err);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		if(geteuid() != euid || getuid() != ruid) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - real and effective user ID not %d and %d", ruid, euid);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Change the effective group ID to the real */
		if(setgid((gid_t)rgid) == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - setgid(rgid) failed errno %d",
								err);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		if(getegid() != rgid || getgid() != rgid) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - real and effective group ID not both %d", rgid);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Try out the saved set-group ID, euid */
		if(setgid((gid_t)egid) == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - setgid(egid) failed errno %d",
								err);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		if(getegid() != egid || getgid() != rgid) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - real and effective group ID not %d and %d", rgid, egid);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
	}

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		if(saveids == SYSERROR)
		{
			globok = 0;
			test_uids(ruid, euid, rgid, egid);
			PATH_FUNC_TRACE;
		}
		else
		{
			globok = 0;
			test_saveids(ruid, euid, rgid, egid);
			PATH_FUNC_TRACE;
		}

		if(test_fail)
			DBUG_EXIT(OK);

		PATH_XS_RPT(11);
		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}
	tet_setblock();

	DBUG_VOID_RETURN;
}

private void
test_saveids(ruid, euid, rgid, egid)
int	ruid;
int	euid;
int	rgid;
int	egid;
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test_saveids");

	/* Change the effective user ID to the parent's real */
	if(setuid((uid_t)ruid) == SYSERROR) {
		err = errno;
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("setuid(ruid) failed errno %d", err);
		test_fail++;
	}
	else
		PATH_TRACE;
	if(geteuid() != ruid || getuid() != ruid) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("real and effective user ID not both %d", ruid);
		test_fail++;
	}
	else
		PATH_TRACE;

	/* Try out the parent's saved set-user ID, euid */
	if(setuid((uid_t)euid) == SYSERROR) {
		err = errno;
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("setuid(saved set-user ID) failed errno %d", err);
		test_fail++;
	}
	else
		PATH_TRACE;
	if(geteuid() != euid || getuid() != ruid) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("real and effective user ID not %d and %d", ruid, euid);
		test_fail++;
	}
	else
		PATH_TRACE;

	/* Change the effective group ID to the parent's real */
	if(setgid((gid_t)rgid) == SYSERROR) {
		err = errno;
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("setgid(rgid) failed errno %d", err);
		test_fail++;
	}
	else
		PATH_TRACE;
	if(getegid() != rgid || getgid() != rgid) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("real and effective group ID not both %d", rgid);
		test_fail++;
	}
	else
		PATH_TRACE;

	/* Try out the parent's saved set-group ID, euid */
	if(setgid((gid_t)egid) == SYSERROR) {
		err = errno;
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("setgid(saved set-group ID) failed errno %d", err);
		test_fail++;
	}
	else
		PATH_TRACE;
	if(getegid() != egid || getgid() != rgid) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("real and effective group ID not %d and %d", rgid, egid);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(8);

	DBUG_VOID_RETURN;
}

private void
test_uids(ruid, euid, rgid, egid)
int	ruid;
int	euid;
int	rgid;
int	egid;
{
	int	pathok = 0;

	DBUG_ENTER("test_uids");

	if(getuid() != ruid) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("child real user ID is different from parent real user ID");
		in_rpt("child real user ID %d; parent real user ID %d",
						getuid(), ruid);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(geteuid() != euid) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("child effective user ID is different from parent effective user ID");
		in_rpt("child effective user ID %d; parent effective user ID %d",
						geteuid(), euid);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(getgid() != rgid) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("child real group ID is different from parent real group ID");
		in_rpt("child real group ID %d; parent real group ID %d",
						getgid(), rgid);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(getegid() != egid) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("child effective group ID is different from parent effective group ID");
		in_rpt("child effective group ID %d; parent effective group ID %d",
						getegid(), egid);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
wait_for_child(waittime)
int	waittime;
{
	int	err;
	int	stat_child;

	DBUG_ENTER("wait_for_child");

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(waittime)) == CHLD_ERR) {
		err = errno;
		if(!test_deleted)
			xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		if(!test_deleted)
			xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		if(stat_child == CHLD_LIVE ||
		   stat_child == CHLD_STOP)
			(void) killchild(tet_child, WAITTIME);
		test_deleted++;
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}
