/*
*      SCCS:  @(#)  XPG4.os/streamio/fgetpos/fgetpos.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fgetpos - T.fgetpos Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fgetpos

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fgetpos() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Keith Holder, UniSoft Ltd.
DATE CREATED:	
MODIFICATIONS:	
		Geoff Clare, 29 Jan 1991
			Change EBADF test to use do_ferr().
			Changed expected failure returns from -1 to non-zero

		Geoff Clare, 11 June 1992
			Allow success in error tests (XSH4 says "may fail").

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<unistd.h>
#include	<report.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<chktimes.h>
#include	<signal.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fgetpos
#endif


#define	NO_TESTS	3		/* number of tests in testset */
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define	T1_POS		10
#define	READ	0
#define	WRITE	1

extern	char	*errname();

int	no_tests = NO_TESTS;

private char	*t_string =	"abcdefghijklmnopqrstuvwxyz";
private	char	*t_file =	"./fgetpos-t";
private	char	*t3_file =	"./fgetpos-t3";
private int	setupok=0;

private void	prepare_tests();
private	void	cln_tests();
private	void	test1();
private	void	test2();
private	int	t2func();
private	void	test3();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private int test_fail;


private void
prepare_tests()
{
	int		fd;
	int		i;
	static char	msg[1024];
	int		pathok = 0;

	DBUG_ENTER("prepare_tests");

	if ((fd = creat(t_file, MODEANY)) == SYSERROR)
		(void) sprintf(msg, "creat(%s, %#o) failed in preperations - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
	else if (write(fd, t_string, (unsigned) strlen(t_string)) != strlen(t_string))
	{
		(void) sprintf(msg, "write() failed in setup - errno %d(%s)",
			errno, errname(errno));
		fd = SYSERROR;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}

	if (fd == SYSERROR)
		for (i = 1 ; i <= NO_TESTS ; i++)
			(void) tet_delete(i, msg);
	else
		PATH_TRACE;
	if (pathok == 2)
		setupok = 0;
	else
		setupok = -1;

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");
	(void) unlink(t_file);
	DBUG_VOID_RETURN;
}


private void
test1()
{
	FILE		*fp;
	int		err;
	fpos_t		pos;
	int		i;
	int		pathok = 0;

	DBUG_ENTER("test1");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}

	for (i=0; i < T1_POS; i++) {
		(void)getc(fp);
		PATH_TRACE;
	}

	if ( fgetpos(fp, &pos) != 0 ) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("fgetpos(stream,pos) failed to return zero");
	} else PATH_TRACE;


	if (!test_fail)
		PATH_XS_RPT(1+T1_POS);

	if (fp != (FILE *) NULL)
		(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private	void
test2()
{
	int experr, err, ret, ferr_ret;
	int	pathok = 0;

	DBUG_ENTER("test2");

	experr = EBADF;

	ret = do_ferr(experr, 'r', -1, t2func, &ferr_ret, &err);

	err = errno;

	if (tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ret != 0 && err != experr)
	{
		xx_rpt(FAILURE);
		in_rpt("fgetpos() set errno incorrectly: expected %d(%s), actual %d(%s)",
			experr, errname(experr), err, errname(err));
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/* ARGSUSED */

private int
t2func(fp, nbyte)
FILE *fp;
int nbyte;
{
	fpos_t		pos;

	DBUG_ENTER("t2func");

	globok = 1; /* must be set for do_ferr() */

	DBUG_RETURN(fgetpos(fp, &pos));
}

private void
test3()
{
	FILE		*fp;
	int		err;
	fpos_t		pos;
	int	ret;
	int		pathok = 0;
	int		pipe_fd[2];

	DBUG_ENTER("test3");

	test_fail = 0;
	errno = NOERROR;
	if ( pipe(pipe_fd) < 0 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: pipe() failed - errno %d(%s)",err,errname);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	errno = NOERROR;
	if ((fp = fdopen(pipe_fd[READ], "r")) == (FILE *) NULL) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fdopen(pipe_fd, \"r\") failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	errno = NOERROR;
	ret = fgetpos(fp,&pos) ;
	err = errno;

	if ( ret != 0 && err != ESPIPE ) {
		if (!test_fail) {
			xx_rpt(FAILURE);
		}
		test_fail++;
		in_rpt("Incorrect errno returned, expected %d (ESPIPE), actual %d (%s)",ESPIPE,err,errname(err));
	} else
		PATH_TRACE;

	if (!test_fail)
		PATH_XS_RPT(3);

	(void) fclose(fp);
	(void) unlink(t3_file);
	DBUG_VOID_RETURN;

}
