/*
*      SCCS:  @(#)  XPG4.os/streamio/fsetpos/fsetpos.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)ANSI.os/streamio/fsetpos - T.fsetpos Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fsetpos

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fsetpos() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Keith Holder, UniSoft Ltd.
DATE CREATED:	July 1990
MODIFICATIONS:	
		Stuart Boutell, 29 Nov 1990
			Test 5 changed to a do_ferr call.
		
		Geoff Clare, 29 Jan 1991
			Changed expected failure returns from -1 to non-zero

		Geoff Clare, 11 June 1992
			Allow success in error tests (XSH4 says "may fail").

		Geoff Clare, 16 Feb 1995
			Remove contiguous-alphabet dependencies.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<unistd.h>
#include	<report.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<chktimes.h>
#include	<signal.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fsetpos
#endif


#define	NO_TESTS	6		/* number of tests in testset */
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define	T1_POS		6
#define	T2_POS		10
#define	T2_NUM		8
#define	T4_DATA		("ABCDE")
#define	T4_SIZE		(sizeof(T4_DATA))
#define	T5_NUM		7
#define	READ	0
#define	WRITE	1

extern	char	*errname();

int	no_tests = NO_TESTS;

private char	*t_string =	"abcdefghijklmnopqrstuvwxyz";
private	char	*t_file =	"./fsetpos-t";
private int	setupok=0;

private void	prepare_tests();
private	void	cln_tests();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	int	t5_func();
private	void	test6();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	NULL, 0
};

private int test_fail;
private int test_deleted;


private void
prepare_tests()
{
	int		fd;
	int		i;
	static char	msg[1024];
	int		pathok = 0;

	DBUG_ENTER("prepare_tests");

	if ((fd = creat(t_file, MODEANY)) == SYSERROR)
		(void) sprintf(msg, "creat(%s, %#o) failed in preperations - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
	else if (write(fd, t_string, (unsigned) strlen(t_string)) != strlen(t_string))
	{
		(void) sprintf(msg, "write() failed in setup - errno %d(%s)",
			errno, errname(errno));
		fd = SYSERROR;
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}

	if (fd == SYSERROR)
		for (i = 1 ; i <= NO_TESTS ; i++)
			tet_delete(i, msg);
	else
		PATH_TRACE;
	if (pathok == 2)
		setupok = 0;
	else
		setupok = -1;

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");
	(void) unlink(t_file);
	DBUG_VOID_RETURN;
}


private void
test1()
{
	FILE		*fp;
	int		err;
	fpos_t		pos;
	int		i;
	int		pathok = 0;

	DBUG_ENTER("test1");
	errno = NOERROR;
	test_fail = 0;
	test_deleted = 0;

	if ((fp = fopen(t_file, "r")) == (FILE *) NULL) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	for (i=0; i < T1_POS; i++) {
		(void)getc(fp);
		PATH_TRACE;
	}

	if ( fgetpos(fp, &pos) != 0 ) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("fgetpos(stream,pos) failed to return zero");
		DBUG_VOID_RETURN;
	} else PATH_TRACE;

	/* advance file pointer some more */
	for (i=0; i < T1_POS; i++) {
		(void)getc(fp);
		PATH_TRACE;
	}


	/* check that we are where we think we are */
	if (getc(fp) != t_string[2*T1_POS]) {
		xx_rpt(DELETION);
		test_deleted++;
		in_rpt("file pointer position in file is incorrect");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( fsetpos(fp, &pos) != 0 ) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("fsetpos(stream,pos) failed to return zero");
	} else PATH_TRACE;

	if (getc(fp) != t_string[T1_POS]) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("fsetpos(stream,pos) failed to set file pointer position  in file correctly");
	} else
		PATH_TRACE;

	if (!test_fail)
		PATH_XS_RPT(5+(2*T1_POS));

	if (fp != (FILE *) NULL)
		(void) fclose(fp);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	FILE		*fp;
	int		err;
	char		buf[50];
	int		pathok = setupok;
	fpos_t		pos;

	DBUG_ENTER("test2");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* obtain position at start of file */
	if ( fgetpos(fp,&pos) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("fgetpos failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void) fread((void *)buf, (size_t)1,
		(size_t)(strlen(t_string) + T2_NUM), fp);

	if (!feof(fp))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: fread() greater than file size did not set EOF indicator");
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fsetpos(fp, &pos) != 0 ) {
		xx_rpt(FAILURE);
		in_rpt("fsetpos() after EOF condition failed");
	}
	else if (feof(fp)) 
	{
		xx_rpt(FAILURE);
		in_rpt("fsetpos() did not clear EOF indicator");
	}
	else
		PATH_XS_RPT(3);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test3()
{
	FILE		*fp;
	int		err;
	fpos_t		pos;
	int	pathok = setupok;

	DBUG_ENTER("test3");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) getc(fp);
	(void) getc(fp);
	(void) getc(fp);
	if ( fgetpos(fp,&pos) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("fgetpos failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	(void) ungetc('x', fp);

	if (fsetpos(fp, &pos) != 0 ) {
		xx_rpt(FAILURE);
		in_rpt("fsetpos() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (getc(fp) == 'x')
	{
		xx_rpt(FAILURE);
		in_rpt("fsetpos() did not undo the effects of ungetc()");
	}
	else
		PATH_XS_RPT(3);

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test4()
{
	FILE		*fp;
	int		err;
	fpos_t		pos;
	char		buf[50];
	int		i;
	int		pathok = 0;

	DBUG_ENTER("test4");
	test_fail = 0;

	errno = NOERROR;
	if ((fp = fopen(t_file, "r+")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r+\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) getc(fp);
	(void) getc(fp);
	(void) getc(fp);
	if ( fgetpos(fp,&pos) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("fgetpos failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;
	(void) getc(fp);
	(void) getc(fp);

	if (fsetpos(fp, &pos) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("fsetpos() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	(void)strncpy(buf,T4_DATA,T4_SIZE);
	if ( fwrite((void *)buf, (size_t)T4_SIZE, 1, fp) != 1 ) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("fwrite() after fsetpos() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (fsetpos(fp, &pos) != 0 ) {
		xx_rpt(DELETION);
		in_rpt("fsetpos() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	for (i=0; i < sizeof(buf); i++ )
		buf[i] = 0;

	if ( fread((void *)buf, (size_t)T4_SIZE, 1, fp) != 1 ) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("fread() after fsetpos() failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( strncmp(buf,T4_DATA,T4_SIZE) != 0 ) {
		xx_rpt(FAILURE);
		test_fail++;
		in_rpt("Incorrect data read from file, after fsetpos/fread/fwrite");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (!test_fail)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}
private void
test5()
{
	int	ret;
	int	ferr_ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test5");
	ret = do_ferr(EBADF, 'f', -1, t5_func, &ferr_ret, &err);
	err = errno;

	if(tet_reason(tet_thistest)!=NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s", tet_reason(tet_thistest));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if( ret != 0 && err != EBADF)
	{
		xx_rpt(FAILURE);
		in_rpt("fsetpos() set errno incorrectly: expected %d (EBADF), actual %d (%s)",
			EBADF, err, errname(err));
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}	

/* ARGSUSED */
private	int
t5_func(fp, nbyte)
FILE * fp; int nbyte;
{
	static fpos_t	pos;
	DBUG_ENTER("t5_func");
	
	globok = 1;
	
	DBUG_RETURN(fsetpos(fp,&pos));
}



private void
test6()
{
	FILE		*fp;
	int		err;
	static fpos_t		pos;
	int	ret;
	int		pathok = 0;
	int		pipe_fd[2];

	DBUG_ENTER("test6");

	test_fail = 0;
	errno = NOERROR;
	if ( pipe(pipe_fd) < 0 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: pipe() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	errno = NOERROR;
	if ((fp = fdopen(pipe_fd[READ], "r")) == (FILE *) NULL) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fdopen(pipe_fd, \"r\") failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	errno = NOERROR;
	ret = fsetpos(fp,&pos) ;
	err = errno;
	if ( ret != 0 && err != ESPIPE )
	{
		if (!test_fail) {
			xx_rpt(FAILURE);
		}
		test_fail++;
		in_rpt("Incorrect errno returned, expected %d (ESPIPE), actual %d (%s)",ESPIPE,err,errname(err));
	} else
		PATH_TRACE;


	if (!test_fail)
		PATH_XS_RPT(3);

	(void) fclose(fp);
	DBUG_VOID_RETURN;

}
