/*
*      SCCS:  @(#)  XPG4.os/string/mblen/mblen.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XPG4.os/string/mblen - T.mblen Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.mblen

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the mblen() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		J.A. Nave, UniSoft Ltd.
DATE CREATED:	Fri Nov 30 15:19:08 GMT 1990
MODIFICATIONS:	M Young, UniSoft Ltd
			Added tests for non-null cases,
			renumbered tests to match new assertion list.

		N Moses, 8 Jan 1992.
			Use wcharstr() to obtain test wide character string.

		N Moses, 17 Jan 1992.
			Check that invalid multibyte char is supported.

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<stdlib.h>
#include	<string.h>
#include	<locale.h>
#include	<wchar.h>
#include	<wchar_cfg.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef mblen
#endif


#define NO_TESTS	5		/* number of tests in testset */
#define	COMP_TESTS	4		/* number of compliance tests */

#define MAXSIZE		20		/* max size of multibyte char */

#define	WAITTIME	(5*SPEEDFACTOR)

int	no_tests = NO_TESTS;
private	int	setupok = -1;

private char	*locale;
private wchar_t	*wcstr;
extern char	*errname();
extern char	*pr_wchar();

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();

private void	ch_t1();
private void	ch_t2();
private void	ch_t5();

private void	prepare_tests();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	NULL, 0
};

private void
prepare_tests()
{
	static	char msg[256];
	int	i;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	errno = NOERROR;
	if(wcharstr(&locale, &wcstr) != 0)
	{
		(void) sprintf(msg, 
		    "wcstr() failed in preparations - errno %d(%s)",
			errno, errname(errno));
		for(i = 1; i <= NO_TESTS; i++)
			tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(1);

	DBUG_VOID_RETURN;
}

/*
 * when s points to n or fewer bytes that form a valid multibyte char,
 * then mblen returns then number of bytes in that multibyte character
 */

private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	i, err, expret, actret;
	char	pmb[MAXSIZE];
	int	testfail = 0;
	int	pathok = setupok;

	DBUG_ENTER("ch_t1");

	if (setlocale(LC_CTYPE, locale) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("Wide-character locale %s is not available", locale);
		in_rpt("Check installation of wide-character locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	for(i = 0; *(wcstr + i) != (wchar_t)0; i++)
	{
		if((expret = wctomb(pmb, *(wcstr + i))) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("wctomb(pmb,\"%s\",%d) failed - errno %d (%s)",
				pr_wchar(*(wcstr + i)),
				MAXSIZE, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		pmb[expret] = '\0';

		if((actret = mblen(pmb, (size_t)MAXSIZE)) == -1)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("mblen(\"%s\",%d) failed - errno %d (%s)",
				pmb, MAXSIZE, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if(actret != expret)
		{
			if(!testfail++)
				xx_rpt(FAILURE);
			in_rpt("mblen(\"%s\",%d) failed", pmb, MAXSIZE);
			in_rpt("expected return value %d", expret);
			in_rpt("actual   return value %d", actret);
		}
		else
			PATH_TRACE;
	}

	if(!testfail)
		PATH_XS_RPT((3 * i) + 1);

	DBUG_VOID_RETURN;
}

/*
 * when s is a NULL pointer, then mblen returns 0.
 */

private void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	ret;
	int	pathok = setupok;

	DBUG_ENTER("ch_t2");

	if (setlocale(LC_CTYPE, locale) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("Wide-character locale %s is not available", locale);
		in_rpt("Check installation of wide-character locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((ret = mblen((char *)NULL, (size_t)MAXSIZE)) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("mblen((char *)NULL, %d) failed", MAXSIZE);
		in_rpt("expected return value 0");
		in_rpt("actual   return value %d", ret);
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/*
 * when s points to a null byte, then mblen returns 0.
 */

private void
test3()
{
	int	ret;
	int	pathok = setupok;

	DBUG_ENTER("test3");

	if ((ret = mblen("", (size_t)MAXSIZE)) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("mblen(\"\", %d) failed", MAXSIZE);
		in_rpt("expected return value 0");
		in_rpt("actual   return value %d", ret);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

/*
 * when s is a NULL pointer, and LC_CTYPE of the current locale is set to
 * the 'C' locale, then mblen returns 0.
 */

private void
test4()
{
	int	ret;
	int	pathok = setupok;

	DBUG_ENTER("test4");

	if (setlocale(LC_CTYPE, "C") == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("'C' locale is not available");
		in_rpt("Check installation of 'C' locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((ret = mblen((char *)NULL, (size_t)MAXSIZE)) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("mblen((char *)NULL, %d) failed", MAXSIZE);
		in_rpt("expected return value 0");
		in_rpt("actual   return value %d", ret);
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

/*
 * when s points to a sequence of bytes that do not form a valid multibyte
 * character then mblen returns -1.
 */

private void
test5()
{
	DBUG_ENTER("test5");

	(void) cppair(ch_t5, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	ret;
	struct	wchar w_cfg;
	int	pathok = setupok;

	DBUG_ENTER("ch_t5");

	if (wcharcfg(&w_cfg) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("wcharcfg() failed to obtain multibyte strings");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (setlocale(LC_CTYPE, (char *)w_cfg.locale1) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("Wide-character locale %s is not available", w_cfg.locale1);
		in_rpt("Check installation of wide-character locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(*w_cfg.invalid_mb == '\0')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("Invalid multi-byte character not suported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((ret = mblen((char *)w_cfg.invalid_mb, (size_t)MAXSIZE)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("mblen(%s, %d) failed",(char *)w_cfg.invalid_mb,MAXSIZE);
		in_rpt("expected return value -1");
		in_rpt("actual   return value %d", ret);
	}
	else
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}
