/*
*      SCCS:  @(#)  LSB.os/string/mblen_L/mblen_L.c	Rel 5.1.4	    (11/19/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1997 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)LSB.os/string/mblen_L - T.mblen_L Rel 5.1.4 (11/19/97)";
char	*copyright[] = {
		"(C) Copyright 1997 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.mblen_L

PROJECT:	VSX5 (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the mblen() system call.

AUTHOR:		Tony Fisher, UniSoft Ltd.
DATE CREATED:	April 1997
MODIFICATIONS:

	Geoff Clare, 10 Oct 1997
		Fixed path tracing problem in tests 6 and 7.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<stdlib.h>
#include	<string.h>
#include	<locale.h>
#include	<wchar.h>
#include	<wchar_cfg.h>
#include	<errno.h>
#include	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef mblen
#endif

#define NO_TESTS	1		/* number of tests in testset */

#define MAXSIZE		20		/* max size of multibyte char */

#define	WAITTIME	(5*SPEEDFACTOR)

int	no_tests = NO_TESTS;

private	int	setupok = -1;

private char	*locale;
private wchar_t	*wcstr;
extern char	*errname();
extern char	*pr_wchar();
extern int	wcharcfg();
extern int	wcharstr();

private void	test1();

private void	ch_t1();

private void	prepare_tests();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private void
prepare_tests()
{
	static	char msg[256];
	int	i;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	errno = NOERROR;
	if(wcharstr(&locale, &wcstr) != 0)
	{
		(void) sprintf(msg, 
		    "wcstr() failed in preparations - errno %d(%s)",
			errno, errname(errno));
		for(i = 1; i <= NO_TESTS; i++)
			tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if(setlocale(LC_CTYPE, locale) == (char *) NULL)
        {
		(void) sprintf(msg,
		"setlocale(LC_CTYPE, %s) failed in preparations - errno %d(%s)",		locale, errno, errname(errno));
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test1()
{
	DBUG_ENTER("test1");

	(void) tet_fork(ch_t1, NULLFN, WAITTIME, OK);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	size_t	ret;
	struct	wchar w_cfg;
	int	pathok = setupok;
	int	err;
	
	DBUG_ENTER("ch_t5");

	if (wcharcfg(&w_cfg) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("wcharcfg() failed to obtain multibyte strings");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check we have an invalid multibyte string/character */
	if(*w_cfg.invalid_mb != '\0')
	{
		if (setlocale(LC_CTYPE, (char *)w_cfg.locale1) == (char *) NULL)
		{
			xx_rpt(DELETION);
			in_rpt("Wide-character locale %s is not available",
			       w_cfg.locale1);
			in_rpt("Check installation of wide-character locale");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		
		if(*w_cfg.invalid_mb == '\0')
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("Invalid multi-byte character not suported");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		errno = 0;
		ret = mblen((char *)w_cfg.invalid_mb, (size_t)MAXSIZE);
		err = errno;
		if (ret != (size_t) -1)
		{
			xx_rpt(FAILURE);
			in_rpt("mblen(%s, %d) failed",
			       (char *)w_cfg.invalid_mb,MAXSIZE);
			in_rpt("expected return value -1");
			in_rpt("actual   return value %ld", (long)ret);
		}
		else if (err != EILSEQ)
		{
			xx_rpt(FAILURE);
			in_rpt("mblen(%s, %d) failed to set errno",
			       (char *)w_cfg.invalid_mb,MAXSIZE);
			in_rpt("expected errno %d(%s)",
			       		EILSEQ, errname(EILSEQ));
			in_rpt("actual errno %d(%s)", err, errname(err));
		}
		else
			PATH_XS_RPT(3);
		
		DBUG_VOID_RETURN;
	}
	else
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("cannot get an invalid multibyte character");
		DBUG_VOID_RETURN;
	}
}
