/*
*      SCCS:  @(#)  XPG4.os/string/mbstowcs/mbstowcs.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XPG4.os/string/mbstowcs - T.mbstowcs Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.mbstowcs

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the mbstowcs() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		J.A. Nave, UniSoft Ltd.
DATE CREATED:	Fri Nov 30 15:19:08 GMT 1990
MODIFICATIONS:	
		Matt Newman, Fri Nov 08 13:54:35 GMT 1991
			4.2 Wide Char support

		Neil Moses, 17 Jan 1992
			Check for size of multibyte string from mbytestr().
			Check that invalid multibyte char is supported.

		Neil Moses, 27 Apr 1992
			Set 3rd arg of mbtowc() to 50.
		
		Geoff Clare, 2 June 1992
			Fix path tracing problem in prepare_tests().

		Geoff Clare, 14 Aug 1992
			Align with interpretation VWG/002/060192.

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<errno.h>
#include        <locale.h>
#include        <string.h>
#include        <wchar.h>
#include        <wchar_cfg.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef mbstowcs
#endif


#define NO_TESTS	4		/* number of tests in testset */
#define WCSIZE		512
#define BUFFERSIZE	512
#define MAXBYTES	50

int	no_tests = NO_TESTS;

private int  	setupok = -1;
private size_t 	len;
private char	*locale;
private char	*mbs;
private char	mbstr[WCSIZE];
private wchar_t	ewcstr[WCSIZE];
private wchar_t	twcstr[WCSIZE];

extern	char	*pr_wstr();
extern	char	*pr_nchars();
extern	char	*errname();
extern	void	xx_all();

private void	test1(), test2(), test3(), test4();
private void	prepare_tests();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private void
prepare_tests()
{
	static	char	msg[BUFFERSIZE];
	char	*pmb;
	char	mb[50];
	wchar_t	wc;
        int     i, pt, ret;
	int	bad = 0;
        int     pathok = 0;

        DBUG_ENTER("prepare_tests");

	errno = NOERROR;
	if(mbytestr(&locale, &mbs) != 0)
	{
		(void) sprintf(msg, 
		    "mbytestr() failed in preparations - errno %d(%s)",
			errno, errname(errno));
		xx_all(DELETION, msg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if(setlocale(LC_CTYPE, locale) == (char *) NULL)
        {
		(void) sprintf(msg,
		"setlocale(LC_CTYPE, %s) failed in preparations - errno %d(%s)",		locale, errno, errname(errno));
		bad = 1;
	}
	else
		PATH_TRACE;

	pmb = mbs;
	for(len = 0;; len++)
	{
		if((ret = mbtowc(&wc, pmb, (size_t) MAXBYTES)) == -1)
		{
			(void) sprintf(msg, 
		    	"mbtowc() failed in preparations - errno %d(%s)",
			errno, errname(errno));
			bad = 1;
		}
		else
			PATH_TRACE;

		ewcstr[len] = wc;
		pmb += ret;

		if(ret == 0)
			break;

		if(len == 0)
		{
			(void) strncpy(mb, mbs, (size_t)ret);
			mb[ret] = '\0';
		}	
	}

	pt = len+1;
	(void) strcpy(mbstr, mbs);

	if (len < 1)
	{
		xx_all(DELETION, "Unable to obtain wide character string");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(len < 3)
	{
		PATH_TRACE;
		for(; len < 3; len++)
		{
			ewcstr[len] = ewcstr[0];
			(void) strcat(mbstr, mb);
		}
		ewcstr[len] = (wchar_t)0;
	}
	else
		PATH_TRACE;
		
	if(bad)
		for(i = 1; i <= NO_TESTS; i++)
			tet_delete(i, msg);
	else
		PATH_SETUP_RPT(4 + pt);

        DBUG_VOID_RETURN;
}

private void
test1()
{
	int	testfail = 0;
	size_t	ret;
	int	pathok = setupok;

	DBUG_ENTER("test1");

	/*
	 * Test where len > wide char string
	 */
	ret = mbstowcs(twcstr, mbstr, len + 2);
	if(wcscmp(twcstr, ewcstr) != 0)
	{
		in_rpt("mbstowcs(pwcs,\"%s\",%d) failed", 
			pr_nchars(mbstr, (int) strlen(mbstr)), (int)len);
		in_rpt("expected wide string : %s", pr_wstr(ewcstr));
		in_rpt("actual   wide string : %s", pr_wstr(twcstr));
		testfail++;
	}
	else
		PATH_TRACE;

	if(ret != len)
	{
		in_rpt("mbstowcs(pwcs,\"%s\",%d) failed", 
			pr_nchars(mbstr, (int) strlen(mbstr)), (int)len);
		in_rpt("expected return value : %d", (int)len);
		in_rpt("actual   return value : %d", (int)ret);
		testfail++;
	}
	else
		PATH_TRACE;

	/*
	 * Test where len < wide char string
	 */
	twcstr[len - 2] = (wchar_t)'Z';
	ret = mbstowcs(twcstr, mbstr, len - 2);
	if(wcsncmp(twcstr, ewcstr, len - 2) != 0 
	&& twcstr[len - 2] != (wchar_t)'Z')
	{
		in_rpt("mbstowcs(pwcs,\"%s\",%d) failed", 
			pr_nchars(mbstr, (int) strlen(mbstr)), (int)len - 2);
		in_rpt("expected wide string : %s", pr_wstr(ewcstr));
		in_rpt("actual   wide string : %s", pr_wstr(twcstr));
		testfail++;
	}
	else
		PATH_TRACE;

	if(ret != len - 2)
	{
		in_rpt("mbstowcs(pwcs,\"%s\",%d) failed", 
			pr_nchars(mbstr, (int) strlen(mbstr)), (int)len - 2);
		in_rpt("expected return value : %d", (int)len - 2);
		in_rpt("actual   return value : %d", ret);
		testfail++;
	}
	else
		PATH_TRACE;

	if(testfail)
		xx_rpt(FAILURE);
	else
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	size_t	ret;
	int	testfail = 0;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	ret = mbstowcs((wchar_t *)0, mbstr, len);
	if(ret != len)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("mbstowcs((wchar_t *)0,\"%s\",%d) failed",
			pr_nchars(mbstr, (int) strlen(mbstr)), (int)len);
		in_rpt("expected return value : %d", (int)len);
		in_rpt("actual   return value : %d", (int)ret);
	}
	else
		PATH_TRACE;

	ret = mbstowcs((wchar_t *)0, mbstr, (size_t)(len/2));
	if(ret != len)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("mbstowcs((wchar_t *)0,\"%s\",%d) failed",
			pr_nchars(mbstr, (int) strlen(mbstr)), (int)len/2);
		in_rpt("expected return value : %d", (int)len);
		in_rpt("actual   return value : %d", (int)ret);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test3()
{
	struct  wchar w_cfg;
	char	tmbstr[BUFFERSIZE];
	int	ret;
	int	pathok = setupok;

	DBUG_ENTER("test3");

        if(wcharcfg(&w_cfg) != 0)
        {
		xx_rpt(DELETION);
                in_rpt("wcharcfg() failed - Unable to get invalid character");
                DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

	if(*w_cfg.invalid_mb == '\0')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("Invalid multi-byte character not suported");
		DBUG_VOID_RETURN;
        }
        else
                PATH_TRACE;

	(void) strcpy(tmbstr, mbstr);
	(void) strcat(tmbstr, (char *)w_cfg.invalid_mb);

	ret = mbstowcs(twcstr, tmbstr, (size_t)(len + 1));
	if(ret != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("mbstowcs((wchar_t *)0,\"%s\",%d) failed",
			pr_nchars(tmbstr, (int) strlen(tmbstr)), (int)len + 1);
		in_rpt("expected return value : %d", -1);
		in_rpt("actual   return value : %d", ret);
	}
	else
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	ret;
	int	pathok = setupok;
	int	testfail = 0;

	DBUG_ENTER("test4");

	twcstr[0] = 1;
	if ((ret = mbstowcs(twcstr, "", (size_t)10)) != 0)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("mbstowcs(pwcs, \"\", 10) returned %d, expected 0",
			ret);
	}
	else
		PATH_TRACE;

	if (twcstr[0] != 0)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("mbstowcs(pwcs, \"\", 10) set pwcs[0] to 0%lo, expected 0",
			(long)twcstr[0]);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
