/*
*      SCCS:  @(#)  XPG4.os/string/memmove/memmove.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XPG4.os/string/memmove - T.memmove Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.memmove

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the memmove() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		J.A.Nave, UniSoft Ltd.
DATE CREATED:	Tue Jul 24 16:39:46 BST 1990
MODIFICATIONS:	
		Geoff Clare, 30 Jan 1991
			Test overlap at both ends of source data.
			Cast pointers to long for printing.
		
		Stuart Boutell, 3 May 1991
			Improved testing using \0 in memory.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<string.h>
#include	<sys/types.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef memmove
#endif


#define NO_TESTS	2		/* number of tests in testset */
#define SPACE		' '		/* space character */
#define	NO_ALPHA	26

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

extern	char	*pr_nchars();

private void
test1()
{
	int	i, num = 15;
	char	s1[20], s2[20], t[20], alpha, *rval;
	int	failed1 = 0, failed2 = 0;
	int	pathok = 0;

	DBUG_ENTER("test1");

	for (i = 0, alpha = 'a'; i < 17; ++i, (i == 12) ? (alpha -= 2) : alpha++)
	{
		if (i==0)
			PATH_TRACE;
		if (i!=10)
			s2[i] = t[i] = alpha;
		else
			s2[i] = t[i] = '\0';

		s1[i] = SPACE;
	}
	for (i = 17; i < 19; i++)
	{
		if (i==17)
			PATH_TRACE;
		s2[i] = '\0';
		s1[i] = SPACE;
	}

	s1[19] = '\0';
	if ((rval = (char *)memmove((void *)s1, (void *)s2, (size_t)num)) != s1)
	{
		++failed1;
	}
	else
		PATH_TRACE;
	for (i = 0; i < 15; i++)
		if (s1[i] != t[i])
		{
			++failed2;
			break;
		}
		else
			PATH_TRACE;
	for (i = 15; i < 19; i++)
		if (s1[i] != SPACE)
		{
			++failed2;
			break;
		}
		else
			PATH_TRACE;
	if (s1[19] != '\0')
		++failed2;
	else
		PATH_TRACE;
	DBUG_PRINTF("test1", "s1 :%s:\ts2 :%s:", pr_nchars(s1,20),
						 pr_nchars(s2,20));
	DBUG_PRINTF("test1", "failed1 :%d:\tfailed2 :%d:", failed1, failed2);

#ifdef TESTBED
	failed2++;
	failed1++;
#endif

	if (failed1 || failed2)
	{
		xx_rpt(FAILURE);

		if (failed1)
		{
			in_rpt("memmove did not return expected value");
			in_rpt("pointer & value of first  argument : %p\t|%s|",
				(void *)s1, pr_nchars(s1,20));
			in_rpt("pointer & value of second argument : %p\t|%s|",
				(void *)s2, pr_nchars(s2,20));
			in_rpt("value of third argument            : %d", num);
			if (rval != s1)
			{
				in_rpt("expected return value      : %p",
					(void *)s1);
				in_rpt("actual   return value      : %p",
					(void *)rval);
			}
		}
		if (failed2)
		{
			in_rpt("memmove did did not copy data correctly");
			in_rpt("expected result from memmove() : |%s    \\000|",
							pr_nchars(t, 15));
			in_rpt("actual   result from memmove() : |%s|",
							pr_nchars(s1,20));
		}

	}
	else
		PATH_XS_RPT(23);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	i, num = NO_ALPHA;
	char	s1[3*NO_ALPHA/2 +1], s2[3*NO_ALPHA/2 + 1], alpha, *rval1;
	char	s3[3*NO_ALPHA/2 +1], s4[3*NO_ALPHA/2 + 1], *rval3;
	int	failed1 = 0, failed2 = 0;
	int	failed3 = 0, failed4 = 0;
	int	pathok = 0;

	DBUG_ENTER("test2");

	for (alpha = 'a', i = 0; i < NO_ALPHA; alpha++, i++)
	{
		if (i==0)
			PATH_TRACE;
		s1[i] = s2[i] = alpha;
	}
	s1[NO_ALPHA] = '\0';

	for (alpha = 'a', i = 0; i < NO_ALPHA; alpha++, i++)
	{
		if (i==0)
			PATH_TRACE;
		s2[i+NO_ALPHA/2] = alpha;
	}
	s2[3*NO_ALPHA/2] = '\0';

	if ((rval1 = (char *)memmove((void *)(s1+(NO_ALPHA/2)), (void *)s1,
		(size_t)num)) != s1+NO_ALPHA/2)
	{
		++failed1;
	}
	else
		PATH_TRACE;

	if (strncmp(s1, s2, 3*NO_ALPHA/2))
		++failed2;
	else
		PATH_TRACE;

	for (alpha = 'a', i = 0; i < NO_ALPHA; alpha++, i++)
	{
		if (i==0)
			PATH_TRACE;
		s3[i+NO_ALPHA/2] = s4[i+NO_ALPHA/2] = alpha;
	}
	s3[3*NO_ALPHA/2] = '\0';

	for (alpha = 'a', i = 0; i < NO_ALPHA; alpha++, i++)
	{
		if (i==0)
			PATH_TRACE;
		s4[i] = alpha;
	}
	s4[3*NO_ALPHA/2] = '\0';

	if ((rval3 = (char *)memmove((void *)s3, (void *)(s3+(NO_ALPHA/2)),
		(size_t)num)) != s3)
	{
		++failed3;
	}
	else
		PATH_TRACE;

	if (strncmp(s3, s4, 3*NO_ALPHA/2))
		++failed4;
	else
		PATH_TRACE;


	if (failed1 || failed2 || failed3 || failed4)
	{
		xx_rpt(FAILURE);

		if (failed1)
		{
			in_rpt("memmove did not return expected value");
			in_rpt("pointer & value of first  argument : %p\t|%s|",
				(void *)s1, s1);
			in_rpt("pointer & value of second argument : %p\t|%s|",
				(void *)(s1+NO_ALPHA/2), s1+NO_ALPHA/2);
			in_rpt("value of third argument            : %d", num);
			if (rval1 != s1)
			{
				in_rpt("expected return value      : %p",
					(void *)s1);
				in_rpt("actual   return value      : %p",
					(void *)rval1);
			}
		}
		if (failed2)
		{
			in_rpt("memmove did did not copy data correctly");
			in_rpt("expected result from memmove() : |%s|", s2);
			in_rpt("actual   result from memmove() : |%s|", s1);
		}
		if (failed3)
		{
			in_rpt("memmove did not return expected value");
			in_rpt("pointer & value of first  argument : %p\t|%s|",
				(void *)(s3+NO_ALPHA/2), s3+NO_ALPHA/2);
			in_rpt("pointer & value of second argument : %p\t|%s|",
				(void *)s3, s3);
			in_rpt("value of third argument            : %d", num);
			if (rval3 != s3)
			{
				in_rpt("expected return value      : %p",
					(void *)s3);
				in_rpt("actual   return value      : %p",
					(void *)rval3);
			}
		}
		if (failed4)
		{
			in_rpt("memmove did did not copy data correctly");
			in_rpt("expected result from memmove() : |%s|", s4);
			in_rpt("actual   result from memmove() : |%s|", s3);
		}

	}
	else
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}
