/*
*      SCCS:  @(#)  XPG4.os/string/wcstombs/wcstombs.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XPG4.os/string/wcstombs - T.wcstombs Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.wcstombs

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the wcstombs() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	5 Feb 1991
MODIFICATIONS:	

		Peter Mulvenney 12 Nov 1991.
		Added extra tests using genuine wide characters.
		(Old test 1 is now test 2).

		Neil Moses, 10 Feb 1992.
		Ensure enough wide characters to perform tests.

		Neil Moses, 3 Mar 1992
			Added psl_wc_skip() to test whether a character
			in a non-wide character locale can be used by
			wide character function.

		Neil Moses, 23 Apr 1992
			Removed psl_wc_skip().
		
		Geoff Clare, 6 May 1992
			Corrected expected return when s is null and n
			is less than length of converted string.
		
		Geoff Clare, 14 May 1992
			Undo above change.
		
		Geoff Clare, 14 Aug 1992
			Align with interpretation VWG/002/060192.
		
		Geoff Clare, 17 Dec 1992
			INVALID_WC=0 means all wchar_t values are valid.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<nlsinfo.h>
#include	<nlstest.h>
#include	<stdio.h>
#include	<errno.h>
#include	<locale.h>
#include	<string.h>
#include	<wchar.h>
#include	<wchar_cfg.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef wcstombs
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define MAXSIZE 	6		/* maximum bytes for a single wc */
#define S2SIZE 		14		/* length of s2[] below, exc. null */
#define	SPACE		' '		/* space character used as filler */
#define NO_TESTS	4		/* number of tests in testset */

int	no_tests = NO_TESTS;

extern	char *pr_nchars();
extern	char *pr_wchar();
extern	char *pr_wstr();
extern	char *errname();
extern	void xx_all();

private	struct wchar	cfg;
private	char	s1[BUFSIZ];	/* Array to contain converted mbs */

private	wchar_t	s2[] = { '0', '1', '2', '9', 'a', 'j', 'z', 'A', 'M', 'Z', '.',
			 '$', ' ', '!', '\0'
			};

private	char	s3[] = "0129ajzAMZ.$ !";	/* mbs equivalent of s2 wc's */
private	char	s4[S2SIZE + 1];			/* For copying s3 string */

private int	testfail;
private	char	*locp;
private	char	*mbs, *expmbs;
private wchar_t	ws1[512];
private wchar_t	*wcstr;
private	int	wslen;
private	int	partsiz, fullsiz;
private char	*currlang = "";
private int	currcat;
private int	setupok = -1;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	ch_t1();
private void	ch_t3();
private void	ch_t4();
private void	preparetests();
private void	checkval();

public	void	(*setfunc) () = preparetests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private void
preparetests()
{
	char	*mbsp;
	int	nb;
	wchar_t	*wcp;
	char	current[50];
	char	*ptr;
	int	i, err;
	static 	char	delmsg[80];	/* Used with tet_delete() messages */
	int	pathok = 0;

	DBUG_ENTER("preparetests");

	errno = NOERROR;

	/*
	 * Get wide-character structure to use as known data for tests
	 */
	if (wcharcfg(&cfg))
	{
		(void) strcpy(delmsg, "wcharcfg() failed in preparations");
		tet_delete(1, delmsg);
		tet_delete(3, delmsg);
		tet_delete(4, delmsg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Obtain a string of real wide characters & count how many there are
	 */
	if (wcharstr(&locp, &wcstr) != 0)
	{
		(void) strcpy(delmsg, "wcharstr() failed in preparations");
		tet_delete(1, delmsg);
		tet_delete(3, delmsg);
		tet_delete(4, delmsg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (wslen = 0, wcp = wcstr; *wcp != (wchar_t) '\0'; wcp++)
		wslen++;

	(void) wcscpy(ws1, wcstr);

	if (wslen < 1)
	{
		xx_all(DELETION, "Unable to obtain wide character string");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wslen < 4)
	{
		for(; wslen < 4; wslen++)
			ws1[wslen] = *wcstr;
		ws1[wslen] = (wchar_t)0;
	}

	if (wslen > (BUFSIZ / MAXSIZE))
	{
		(void) strcpy(delmsg, "Wide-character string too long in setup function");
		tet_delete(1, delmsg);
		tet_delete(3, delmsg);
		tet_delete(4, delmsg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((mbs = (char *) malloc((size_t) ((wslen * MAXSIZE) + 1)))
				== (char *) NULL)
	{
		(void) sprintf(delmsg, "malloc(%d) failed in preparations",
			((wslen * MAXSIZE) + 1));
		tet_delete(1, delmsg);
		tet_delete(3, delmsg);
		tet_delete(4, delmsg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((expmbs = (char *) malloc((size_t) ((wslen * MAXSIZE) + 1)))
				== (char *) NULL)
	{
		(void) sprintf(delmsg, "malloc(%d) failed in preparations",
			((wslen * MAXSIZE) + 1));
		tet_delete(1, delmsg);
		tet_delete(3, delmsg);
		tet_delete(4, delmsg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Save current locale */ 
	if((ptr = setlocale(LC_CTYPE, NULL)) == (char *)NULL) {
		err = errno;
		(void) sprintf(delmsg, "setlocale() failed in preperations - errno %d(%s)", err, errname(err));
		xx_all(DELETION, delmsg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	(void) strcpy(current, ptr);

	if(setlocale(LC_CTYPE, locp) == (char *)NULL)
	{
		err = errno;
		(void) sprintf(delmsg, "setlocale() failed in preperations - errno %d(%s)", err, errname(err));
		xx_all(DELETION, delmsg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	* Determine the expected multi-byte string result of converting ws1
	*/
	for (mbsp = mbs, i = 0, wcp = ws1; *wcp != (wchar_t) '\0'; i++, wcp++)
	{
		if (i == 0)
			PATH_TRACE;

		if (i == wslen - 3)		/* Save no. of bytes in a */
			partsiz = (mbsp - mbs);	/* partial version of string */

		if((nb = wctomb(mbsp, *wcp)) == -1)
		{
			(void) sprintf(delmsg,
	"Wide-character string contains invalid char %s in setup function",
				pr_wchar(*wcp));
			tet_delete(1, delmsg);
			tet_delete(3, delmsg);
			tet_delete(4, delmsg);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		mbsp += nb;
	}

	fullsiz = (mbsp - mbs);
	*mbsp = '\0';

	/* Restore the locale */
	if(setlocale(LC_CTYPE, current) == NULL)
	{
		err = errno;
		(void) sprintf(delmsg, "setlocale() failed in preperations - errno %d(%s)", err, errname(err));
		xx_all(DELETION, delmsg);
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(10 + i);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void)  cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	ret;
	int	i;
	int	pathok = setupok;

	DBUG_ENTER("ch_t1");

	/*
	 * Phase 1 - tests using ordinary character values
	 * Put known characters in receiving buffer and in expected mbs
	 * buffer for checking that only expected no. of bytes are stored
	 */
	testfail = 0;

	for (i = 0; i < BUFSIZ; i++)
		s1[i] = SPACE;

	(void) strncpy(s4, s3, (size_t) 6);
	for (i = 6; i < (S2SIZE + 1); i++)
		s4[i] = SPACE;
	/* n < wide-string length */
	ret = wcstombs(s1, s2, (size_t) 6);
	globok = 0;
	checkval(s1, s2, 6, s4, (S2SIZE + 1), ret, 6);
	PATH_FUNC_TRACE;

	(void) strcpy(s4, s3);
	/* n > w-s length */
	ret = wcstombs(s1, s2, (size_t) (2 * S2SIZE));	
	globok = 0;
	checkval(s1, s2, (2 * S2SIZE), s4, (S2SIZE + 1), ret, S2SIZE);
	PATH_FUNC_TRACE;

	/*
	 * Phase 2 - tests using real wide-character values
	 * First set the locale
	 */
	if (setlocale(LC_ALL, locp) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
	in_rpt("Wide-character locale %s is not available", locp);
		in_rpt("Check installation of wide-character locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = locp;
	currcat  = LC_ALL;

	(void) strncpy(expmbs, mbs, (size_t) partsiz);
	for (i = partsiz; i < fullsiz; i++)
		*(expmbs + i) = SPACE;

	ret = wcstombs(s1, ws1, (size_t) partsiz);

	globok = 0;
	checkval(s1, ws1, partsiz, expmbs, partsiz, ret, partsiz);
	PATH_FUNC_TRACE;

	/*
	 * Repeat test using the full string
	 */
	(void) strcpy(expmbs, mbs);
	ret = wcstombs(s1, ws1, (size_t) (fullsiz + 10));

	globok = 0;
	checkval(s1, ws1, (fullsiz + 10), expmbs, (fullsiz + 1), ret, fullsiz);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(5);
}

private void
test2()
{
	int	ret;
	char	s[10];
	wchar_t	wcs[2];
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;
	s[0] = 'A';
	wcs[0] = 0;

#ifdef TESTBED
	wcs[0] = 'A';
	wcs[1] = 0;
#endif

	ret = wcstombs(s, wcs, (size_t) 10);

	globok = 0;
	checkval(s, wcs, 10, s, 0, ret, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void)  cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int	ret;
	int	pathok = setupok;

	DBUG_ENTER("ch_t3");

	testfail = 0;

	/*
	 * Phase 1 - tests using ordinary character values
	 */
	ret = wcstombs((char *) NULL, s2, (size_t) S2SIZE);
	globok = 0;
	checkval((char *)NULL,s2,S2SIZE,(char *)NULL,0,ret,S2SIZE);
	PATH_FUNC_TRACE;

	if (setlocale(LC_ALL, INFO2LANG) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
	in_rpt("Wide-character locale %s is not available", INFO2LANG);
		in_rpt("Check installation of wide-character locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO2LANG;
	currcat  = LC_ALL;

	ret = wcstombs((char *) NULL, s2, (size_t) S2SIZE);
	globok = 0;
	checkval((char *)NULL,s2,S2SIZE,(char *)NULL,0,ret,S2SIZE);
	PATH_FUNC_TRACE;

	if (setlocale(LC_ALL, (char *) cfg.locale1) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
in_rpt("Wide-character locale %s is not available", (char *) cfg.locale1);
		in_rpt("Check installation of wide-character locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = (char *) cfg.locale1;
	currcat  = LC_ALL;

	ret = wcstombs((char *) NULL, s2, (size_t) S2SIZE);

	globok = 0;
	checkval((char *) NULL, s2, S2SIZE, (char *) NULL, 0, ret, S2SIZE);
	PATH_FUNC_TRACE;

	if (setlocale(LC_ALL, (char *) cfg.locale2) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
in_rpt("Wide-character locale %s is not available", (char *) cfg.locale2);
		in_rpt("Check installation of wide-character locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = (char *) cfg.locale2;
	currcat  = LC_ALL;

	ret = wcstombs((char *) NULL, s2, (size_t) S2SIZE);

	globok = 0;
	checkval((char *) NULL, s2, S2SIZE, (char *) NULL, 0, ret, S2SIZE);
	PATH_FUNC_TRACE;

	/*
	 * Phase 2 - tests using real wide-character values
	 * First set the locale
	 */
	if (setlocale(LC_ALL, locp) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
	in_rpt("Wide-character locale %s is not available", locp);
		in_rpt("Check installation of wide-character locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = locp;
	currcat  = LC_ALL;

	ret = wcstombs((char *) NULL, ws1, (size_t) partsiz);

	globok = 0;
	checkval((char *) NULL, ws1, partsiz, (char *) NULL, 0, ret, fullsiz);
	PATH_FUNC_TRACE;

	ret = wcstombs((char *) NULL, ws1, (size_t) fullsiz);

	globok = 0;
	checkval((char *) NULL, ws1, fullsiz, (char *) NULL, 0, ret, fullsiz);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(10);
}

private void
test4()
{
	DBUG_ENTER("test4");

	if (cfg.invalid_wc == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("wide character config parameter INVALID_WC is set to 0");
	}
	else
		(void) cppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	invalpos;
	int	ret;
	int	pathok = setupok;

	DBUG_ENTER("ch_t4");

	testfail = 0;
	invalpos = S2SIZE / 2;
	if (setlocale(LC_ALL, (char *) cfg.locale1) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
in_rpt("Wide-character locale %s is not available", (char *) cfg.locale1);
		in_rpt("Check installation of wide-character locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = (char *) cfg.locale1;
	currcat  = LC_ALL;

	s2[invalpos] = cfg.invalid_wc;

	ret = wcstombs(s1, s2, (size_t) S2SIZE);

	globok = 0;
	checkval(s1, s2, S2SIZE, s3, invalpos, ret, -1);
	PATH_FUNC_TRACE;

	s2[0] = cfg.invalid_wc;

	ret = wcstombs(s1, s2, (size_t) S2SIZE);

	globok = 0;
	checkval(s1, s2, S2SIZE, s3, 0, ret, -1);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

}

private void
checkval(s, wcp, n, expstr, chklen, retval, expval)
char	*s;
wchar_t *wcp;
int	n;
char	*expstr;
int	chklen;
int	retval, expval;
{
	int	failed = 0;
	int	i = 0;
	int	pathok = 0;

	DBUG_ENTER("checkval");

	globok = 0;

	if (retval != expval)
	{
		if (!testfail++)
			xx_rpt(FAILURE);

		in_rpt("wcstombs did not return expected value");
		if (strlen(currlang) > 0)
		{
			in_rpt("Current language is %s", currlang);
			if (currcat == LC_ALL)
				in_rpt("Current category is %s","LC_ALL");
		}
		in_rpt("value of wide-character string : %s", pr_wstr(wcp));
		in_rpt("expected return value      : %d", expval);
		in_rpt("actual   return value      : %d", retval);
	}
	else
	{
		PATH_TRACE;
	}

	if (s != (char *) NULL)
	{
		while (i < chklen)	/* Check appropriate no. of chars */
		{
			if (*(s + i) != *(expstr + i))
				failed++;
			if (*(s + i) == '\0')	/* Or stop after the null */
				break;
			i++;
		}

		if (failed)
		{
			if (!testfail++)
				xx_rpt(FAILURE);
		in_rpt("wcstombs did not produce expected multi-byte string");
			if (strlen(currlang) > 0)
			{
				in_rpt("Current language is %s", currlang);
				if (currcat == LC_ALL)
					in_rpt("Current category is %s",
								"LC_ALL");
			}
			in_rpt("wide-character string : %s", pr_wstr(wcp));
			in_rpt("size argument         : %d", n);
			in_rpt("expected mbs value      : %s",
					pr_nchars(expstr, chklen));
			in_rpt("actual   mbs value      : %s",
					pr_nchars(s, chklen));
		}
		else
		{
			globok = 1;
		}
	}
	else
	{
		globok = 1;
	}

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

