/*
*      SCCS:  @(#)  XPG4.os/string/wctomb/wctomb.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XPG4.os/string/wctomb - T.wctomb Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.wctomb

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the wctomb() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		J.A. Nave, UniSoft Ltd.
DATE CREATED:	Fri Nov 30 15:19:08 GMT 1990
MODIFICATIONS:	
		Peter Mulvenney  11 Nov 1991.
		Added extra tests using genuine wide characters.
		(Old test 1 is now test 3).

		Neil Moses, 17 Jan 1992.
			Check that invalid wide char is supported.

		Neil Moses, 3 Mar 1992
			Ensure normal chars are tested in C locale,
			or INFO1LANG etc

		Neil Moses, 3 Mar 1992
			Added psl_wc_skip() to test whether a character
			in a non-wide character locale can be used by
			wide character function.

		Neil Moses, 23 Apr 1992
			Removed psl_wc_skip().

		Neil Moses, 27 Apr 1992
			Increased size of 3rd arg to mbtowc() to 50.

		Geoff Clare, 17 Dec 1992
			INVALID_WC=0 means all wchar_t values are valid.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<stdlib.h>
#include	<string.h>
#include <tet_api.h>
#include	<nlsinfo.h>
#include	<nlstest.h>
#include	<locale.h>
#include	<wchar.h>
#include	<wchar_cfg.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef wctomb
#endif


extern	char *pr_nchars();

#define	WAITTIME	(5 * SPEEDFACTOR)
#define MAXBYTES 	50	/* maximum bytes for a single wc */
#define NO_TESTS	4		/* number of tests in testset */

int	no_tests = NO_TESTS;

private	struct wchar	cfg;

private	wchar_t	s2[] = { '0', '1', '2', '9', 'a', 'j', 'z', 'A', 'M', 'Z', '.',
			 '$', ' ', '!', '\0'
			};

private	char	s3[] = "0129ajzAMZ.$ !";	/* mbs equivalent of s2 wc's */

private	int	testfail = 0;
private char	*currlang = "";
private int	currcat;
private int     setupok = 0;

private void	preparetests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	ch_t1();
private void	ch_t2();
private void	ch_t4();
private void	checkval();

public	void	(*setfunc) () = preparetests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private void
preparetests()
{
	int	pathok = 0;

	DBUG_ENTER("preparetests");

	/*
	 * Get wide-character structure to use as known data for tests
	 * If this fails, delete tests dependent on this data structure
	 */
	if (wcharcfg(&cfg))
	{
		tet_delete(1, "Wide character data structure is not available");
		tet_delete(2, "Wide character data structure is not available");
		tet_delete(4, "Wide character data structure is not available");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void)  cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t1()
{
	int	ret;
	char	*locp;
	char	*mbs;
	int	nb;
	wchar_t	wc;
	char	s1[MAXBYTES];	/* Array to contain mbs */
	wchar_t	*wcp;
	char	*sp;
	int	otestcnt = 0;	/* No. of tests with ordinary char values */
	int	wtestcnt = 0;	/* No. of tests with real wide characters */
	int	pathok = setupok;

	DBUG_ENTER("ch_t1");

	testfail = 0;
	s1[MAXBYTES - 1] = '\0';
	/*
	 * Get multi-byte string to use for testing & set appropriate locale
	 */
	if (mbytestr(&locp, &mbs) != 0)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Cannot obtain multi-byte string test data");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (setlocale(LC_CTYPE, locp) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Wide-character locale %s is not available", locp);
		in_rpt("Check installation of pseudo-languages");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = locp;
	currcat  = LC_CTYPE;

	while (*mbs != '\0')
	{
		if (wtestcnt == 0)
			PATH_TRACE;

		nb = mbtowc(&wc, mbs, MAXBYTES);
		if (nb < 0)
		{
			if (!testfail)
				xx_rpt(DELETION);
		in_rpt("Cannot convert multi-byte string %s into wide-char",
					pr_nchars(mbs, (size_t) MAXBYTES));
			in_rpt("Check installation of wide-character locale");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		ret = wctomb(s1, wc);
		*(s1 + nb) = '\0';	/* Enforce a null terminator */

		globok = 0;
		checkval(s1, wc, mbs, ret, nb);
		PATH_FUNC_TRACE;

		mbs += nb;
		wtestcnt++;
	}

	/* 
	 * Check that normal chars in s2[] can be used in locale
	 */
	if (setlocale(LC_CTYPE, "C") == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Unable to change to C locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = "C";
	currcat  = LC_CTYPE;

	wcp = s2;
	sp = s3;
	while (*wcp != (wchar_t) '\0')
	{
		if (wcp == s2)
			PATH_TRACE;

		ret = wctomb(s1, *wcp);
		*(s1 + 1) = '\0';	/* Enforce a null terminator */

		globok = 0;
		checkval(s1, *wcp, sp, ret, 1);
		PATH_FUNC_TRACE;

		wcp++;
		sp++;
		otestcnt++;
	}

	if (!testfail)
		PATH_XS_RPT((2 * wtestcnt) + 5 + otestcnt);
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void)  cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	retval;
	int	pathok = setupok;

	DBUG_ENTER("ch_t2");

	testfail = 0;
	if (setlocale(LC_CTYPE, INFO1LANG) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Locale %s is not available", INFO1LANG);
		in_rpt("Check installation of pseudo-languages");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO1LANG;
	currcat  = LC_CTYPE;

	retval = wctomb((char *) NULL, (wchar_t) '\0');
	globok = 0;
	checkval((char *) NULL, (wchar_t) '\0', (char *) NULL, retval, 0);
	PATH_FUNC_TRACE;

	if (setlocale(LC_CTYPE, INFO2LANG) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Locale %s is not available", INFO2LANG);
		in_rpt("Check installation of pseudo-languages");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = INFO2LANG;
	currcat  = LC_CTYPE;

	retval = wctomb((char *) NULL, (wchar_t) '\0');
	globok = 0;
	checkval((char *) NULL, (wchar_t) '\0', (char *) NULL, retval, 0);
	PATH_FUNC_TRACE;

	if (setlocale(LC_CTYPE, (char *) cfg.locale1) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
in_rpt("Wide-character locale %s is not available", (char *) cfg.locale1);
		in_rpt("Check installation of wide-character locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = (char *) cfg.locale1;
	currcat  = LC_CTYPE;

	retval = wctomb((char *) NULL, (wchar_t) '\0');
	globok = 0;
	checkval((char *) NULL, (wchar_t) '\0', (char *) NULL, retval, 0);
	PATH_FUNC_TRACE;

	if (setlocale(LC_CTYPE, (char *) cfg.locale2) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
in_rpt("Wide-character locale %s is not available", (char *) cfg.locale2);
		in_rpt("Check installation of wide-character locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = (char *) cfg.locale2;
	currcat  = LC_CTYPE;

	retval = wctomb((char *) NULL, (wchar_t) '\0');
	globok = 0;
	checkval((char *) NULL, (wchar_t) '\0', (char *) NULL, retval, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int	retval;
	int	pathok = setupok;

	DBUG_ENTER("test3");

	testfail = 0;

	retval = wctomb((char *) NULL, (wchar_t) '\0');
	globok = 0;
	checkval((char *) NULL, (wchar_t) '\0', (char *) NULL, retval, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	if (cfg.invalid_wc == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("wide character config parameter INVALID_WC is set to 0");
	}
	else
		(void) cppair(ch_t4, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	retval;
	char	s1[MAXBYTES];
	int	pathok = setupok;

	DBUG_ENTER("ch_t4");

	testfail = 0;
	s1[MAXBYTES - 1] = '\0';

	if (setlocale(LC_CTYPE, (char *) cfg.locale1) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
in_rpt("Wide-character locale %s is not available", (char *) cfg.locale1);
		in_rpt("Check installation of wide-character locale");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	currlang = (char *) cfg.locale1;
	currcat  = LC_CTYPE;

	retval = wctomb(s1, cfg.invalid_wc);
	globok = 0;
	checkval(s1, cfg.invalid_wc, s1, retval, -1);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
checkval(s, wc, mbs, retval, expval)
char	*s, *mbs;
wchar_t wc;
int	retval, expval;
{
	int	failed = 0;
	int	i = 0;
	int 	pathok = 0;

	DBUG_ENTER("checkval");

	globok = 0;

	if (retval != expval)
	{
		if (!testfail++)
			xx_rpt(FAILURE);

		in_rpt("wctomb did not return expected value");
		if (strlen(currlang) > 0)
		{
			in_rpt("Current language is %s", currlang);
			if (currcat == LC_CTYPE)
				in_rpt("Current category is %s",
							"LC_CTYPE");
		}
		in_rpt("value of argument : 0x%x", wc);
		in_rpt("expected return value      : %d", expval);
		in_rpt("actual   return value      : %d", retval);
	}
	else
	{
		PATH_TRACE;
	}

	if (s != (char *) NULL && expval > 0)
	{
		while (*(s + i) != '\0')
		{
			if (*(s + i) != *(mbs + i))
				failed++;
			i++;
		}

		if (failed)
		{
			if (!testfail++)
				xx_rpt(FAILURE);
		in_rpt("wctomb did not produce expected multi-byte string");
			if (strlen(currlang) > 0)
			{
				in_rpt("Current language is %s", currlang);
				if (currcat == LC_CTYPE)
					in_rpt("Current category is %s",
								"LC_CTYPE");
			}
			in_rpt("value of argument : 0x%x", wc);
			in_rpt("expected mbs value      : %s",
					pr_nchars(mbs, (size_t) MAXBYTES));
			in_rpt("actual   mbs value      : %s",
					pr_nchars(s, (size_t) MAXBYTES));
		}
		else
		{
			globok = 1;
		}
	}
	else
	{
		globok = 1;
	}

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

