/*
*      SCCS:  @(#)  XOPEN.os/time/clock/clock.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XOPEN.os/time/clock - T.clock Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.clock

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the clock() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Thu Oct 23 16:52:01 BST 1986
MODIFICATIONS:	Andrew Twigger 11 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Geoff Clare, 2/1/90
			Call user-supplied routines to accumulate
			user and system CPU time.

		J.A.Nave	01/05/90
			Added resiliance testing

		Geoff Clare, 9 Feb 1995
			Save return from first call to clock(), except
			in XPG3 mode.

		Geoff Clare, 7 Sept 1995
			Test 2 only applies to XPG3 mode.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/times.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<time.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef clock
#endif


#define 	NO_TESTS	2		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)
#define		COUNT_UT	(1000 / SPEEDFACTOR)
#define		COUNT_ST	(10000 / SPEEDFACTOR)
#define		VSX_CLOCK_ERR	"VSX_CLOCK_ERR"
#define		DEFAULT		5	/* default value of VSX_CLOCK_ERR */

extern	void	xx_all();
extern	void	use_time();
extern	int	cppair();
#ifndef clock
extern	clock_t	clock();
#endif


int	no_tests = NO_TESTS;
private	int	setupok;

private	long	tolerance;	
private	long	clock1, clock2;
private long	t1, t2;
private long	clk_tck;

private void	prepare_tests();
private	void	child_prep();
private void	test1();
private void	test2();
private	long	longabs();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private void
prepare_tests()
{
	int	count;
	clock_t	clock0;
	char	*cp;
	struct	tms	tbuf1;
	struct	tms	tbuf2;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	clk_tck = sysconf(_SC_CLK_TCK);

#if TEST_XPG == 3
	clock0 = 0;
	(void) clock();	/* In XPG3, when clock is called, the value returned
			 * is the CPU time used since the first call.
			 * This call is initialisation.
			 */
#else
	clock0 = clock(); /* non-XPG3 : save first value for subtraction */
#endif

	if (times(&tbuf1) == SYSERROR)
	{
		xx_all(DELETION, "times() call failed in setup");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* waste some user time	*/

	for (count = 0; count < COUNT_UT; count++)
	{
		PATH_TRACE;
		use_time();	
	}
		
	/* waste some system time */

	for (count = 0; count < COUNT_ST; count++)
	{
		PATH_TRACE;
		if (sys_call() == SYSERROR)
		{
			xx_all(DELETION, "sys_call() failed in setup");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	clock1 = clock() - clock0;
	if (times(&tbuf2) == SYSERROR)
	{
		xx_all(DELETION, "times() call failed in setup");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t1 = (long) ( tbuf2.tms_utime - tbuf1.tms_utime + tbuf2.tms_stime -
						tbuf1.tms_stime );

#if TEST_XPG == 3
	(void) cppair(child_prep, NULLFN, WAITTIME, 0);

	clock2 = clock() - clock0;
	if (times(&tbuf2) == SYSERROR)
	{
		xx_all(DELETION, "times() call failed in setup");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t2 = (long) ( tbuf2.tms_utime - tbuf1.tms_utime + 
		tbuf2.tms_stime - tbuf1.tms_stime + tbuf2.tms_cutime -
		tbuf1.tms_cutime + tbuf2.tms_cstime - tbuf1.tms_cstime);

#else /* TEST_XPG == 3 */
	PATH_TRACE;
#endif /* TEST_XPG == 3 */

	if ((cp = tet_getvar(VSX_CLOCK_ERR)) == NULL || *cp == '\0')
		tolerance = DEFAULT;
	else
		tolerance = (long) abs(atoi(cp));

	setupok = (pathok == COUNT_UT + 2*COUNT_ST + 3) ? 0 : -1;

	DBUG_VOID_RETURN;
	
}

#if TEST_XPG == 3
private void
child_prep()
{
	int	count;
	int	pathok = 0;

	DBUG_ENTER("child_prep");

	/* waste some user time	*/

	PATH_TRACE;	/*	prove that we have been called	*/

	for (count = 0; count < COUNT_UT; count++)
		use_time();	
	
	/* waste some system time */

	for (count = 0; count < COUNT_ST; count++)
		(void) sys_call();
	
	DBUG_VOID_RETURN;
}
#endif /* TEST_XPG == 3 */
	

private void
test1()
{
	int	pathok = setupok;

	DBUG_ENTER("test1");

	if ( (  longabs( (t1 * 1000000L) / clk_tck - clock1 )  )  >
				( (tolerance * clock1) / 100 ) )
	{
		xx_rpt(FAILURE);
		in_rpt("CPU time used as given by clock() does not correspond to value given by times() within tolerance limits");
		in_rpt("CPU time as given by times is %ld microseconds",
				(t1 * 1000000L) / clk_tck );
		in_rpt("CPU time as given by clock is %ld microseconds",
				clock1 );
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}


private void
test2()
{
#if TEST_XPG == 3
	int	pathok = setupok;

	DBUG_ENTER("test2");

	if ( ( longabs( (t2 * 1000000L) / clk_tck - clock2 ) ) >
				( (tolerance * clock2) / 100 ) )
	{
		xx_rpt(FAILURE);
		in_rpt("CPU time used as given by clock() does not correspond to value given by times() within tolerance limits");
		in_rpt("CPU time as given by times is %ld microseconds",
				(t2 * 1000000L) / clk_tck );
		in_rpt("CPU time as given by clock is %ld microseconds",
				clock2 );
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
#else
	xx_rpt(NOTINUSE);
#endif
}

private long
longabs(x)
long x;
{
	if (x < 0)
		return(-x);
	else	
		return(x);
}

