/*
*      SCCS:  @(#)  XPG4.os/time/difftime/difftime.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XPG4.os/time/difftime - T.difftime Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.difftime

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the difftime() subroutine.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	30 Jan 1991
MODIFICATIONS:	
		Geoff Clare, 16 July 1992
			Fix incorrect return value reporting.

************************************************************************/
#include	<std.h>
#include	<time.h>
#include	<limits.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef difftime
#endif


#define NO_TESTS	1

int	no_tests = NO_TESTS;

private	int	testfail;

private void	checkval();
private void	test1();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};


private void
test1()		
{
	time_t tmax;
	int pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	/* find maximum value of time_t */
	if ((time_t)-1 > 0)
		tmax = (time_t)-1;
	else
	{
		if (sizeof(time_t) == sizeof(char))
			tmax = CHAR_MAX;
		else if (sizeof(time_t) == sizeof(short))
			tmax = SHRT_MAX;
		else if (sizeof(time_t) == sizeof(int))
			tmax = INT_MAX;
		else
			tmax = LONG_MAX;
	}

	globok = 0;
	checkval((time_t)1, (time_t)0, 1.0);
	PATH_FUNC_TRACE;

	globok = 0;
	checkval((time_t)0, (time_t)1, -1.0);
	PATH_FUNC_TRACE;

	globok = 0;
	checkval((time_t)1, (time_t)1, 0.0);
	PATH_FUNC_TRACE;

	globok = 0;
	checkval((time_t)10, (time_t)100, -90.0);
	PATH_FUNC_TRACE;

	globok = 0;
	checkval((time_t)100, (time_t)10, 90.0);
	PATH_FUNC_TRACE;

	globok = 0;
	checkval(tmax, (time_t)0, (double)tmax);
	PATH_FUNC_TRACE;

	globok = 0;
	checkval((time_t)0, tmax, -(double)tmax);
	PATH_FUNC_TRACE;

	globok = 0;
	checkval(tmax, tmax, 0.0);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

private void
checkval(t1, t0, expret)
time_t t1, t0;
double expret;
{
	double ret;
	int pathok = 0;

	DBUG_ENTER("checkval");

	ret = difftime(t1, t0);

	if (!dcomp(ret, expret))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("difftime(%lu, %lu) returned %g, expected %g",
			(unsigned long)t1, (unsigned long)t0,
			ret, expret);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}
