/*
*      SCCS:  @(#)  XPG4.os/time/strptime/strptime.c	Rel 4.4.3	    (07/20/98)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)XPG4.os/time/strptime - T.strptime Rel 4.4.3 (07/20/98)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.strptime

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the strptime() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:			David G. Sawyer, UniSoft Ltd.	
DATE CREATED:		2nd September 1991

MODIFIED:
		Geoff Clare, 29 May 1992
			Don't use literal language names.

		Geoff Clare, 11 June 1992
			Removed declaration of strptime.
			Fixed spelling of "appropriate".
			Added "if (testfail == 0)" before PATH_XS_RPT calls.
			Changed tm_year from 1991 to 91 in several places.
			Check return value in chkstrptime().
			Report expected and observed struct tm values.
			Subtract 1 from constant expected tm_mon values.
			Subtract 1 from expected tm_yday values.
			Don't check struct tm values for members with
			    an expected value of -1.
			Where week number and weekday are supplied, allow
			    either tm_yday or tm_mon&tm_mday to be set.

		Geoff Clare, 28 July 1992
			Changed to use sysconf() instead of _XOPEN_ENH_I18N.

		Geoff Clare, 29 Sept 1992
			Align with interpretation VWG/025/081892.

		Geoff Clare, 6 Oct 1992
			Add 12 to expected hour when PM string used.

		Geoff Clare, 1 Dec 1992
			Move %t testing so it is done in C locale.

		Geoff Clare, 1 Apr 1993
			Change ASCII spaces to psl spaces in tests 3 & 5.
			Use %I instead of %H with %p.

		Geoff Clare, 13 July 1994
			Align with interpretation VWG/067/033094.

		Geoff Clare, 30 March 1995
			New encodings for VSX4L1 and VSX4L2.

		Geoff Clare, 26 Feb 1998
			Align %r test with XSH5 corrigenda.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<ctype.h>
#include	<errno.h>
#include	<time.h>
#include	<limits.h>
#include	<nlsinfo.h>
#include	<nlstest.h>
#include	<locale.h>
#include	<dbug.h>
#include	<report.h>
#include	<pcts_max.h>
#include	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef strptime
#endif



#define		NO_TESTS	8	/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

int	no_tests = NO_TESTS;

extern	void	xx_all();
extern	char	*errname();
extern 	char	*psl_to_asc();
extern 	char	*pslanginfo();
extern 	int	cppair();

private	int	tsup = 0;
private	int	setupok = -1;
private	char	*testcomment;
private	char	*currlang;
private	char	*return_ptr;
private	int		testfail;
private	int		langnum = -1; /* ie not set to a language */
private	struct tm	chkbuf;
private	struct tm	testbuf;


private void	setup();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	ch_t3();
private	void	test4();
private	void	test5();
private	void	ch_t5();
private	void	test6();
private	void	ch_t6();
private	void	test7();
private	void	test8();

private	void	chkstrptime();
private int	chksupp();
private	void	chkweekday();
private	void	init_tms();
private	int	tm_match();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;


public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 NULL,  0
};


#define		NUMDAYS		7
#define		NUMMONTHS	12

private	int	dayitems[NUMDAYS] = 	{	
			VSX_DAY_1, VSX_DAY_2, VSX_DAY_3, VSX_DAY_4,
			VSX_DAY_5, VSX_DAY_6, VSX_DAY_7,
					};			

private	int	abdayitems[NUMDAYS] = 	{	
			VSX_ABDAY_1, VSX_ABDAY_2, VSX_ABDAY_3, VSX_ABDAY_4,
			VSX_ABDAY_5, VSX_ABDAY_6, VSX_ABDAY_7,
					};

private	int	monitems[NUMMONTHS] = 	{	
			VSX_MON_1, VSX_MON_2, VSX_MON_3, VSX_MON_4,
			VSX_MON_5, VSX_MON_6, VSX_MON_7, VSX_MON_8,
			VSX_MON_9, VSX_MON_10, VSX_MON_11, VSX_MON_12,
					};

private	int	abmonitems[NUMMONTHS] = {	
			VSX_ABMON_1, VSX_ABMON_2, VSX_ABMON_3, VSX_ABMON_4,
			VSX_ABMON_5, VSX_ABMON_6, VSX_ABMON_7, VSX_ABMON_8,
			VSX_ABMON_9, VSX_ABMON_10, VSX_ABMON_11, VSX_ABMON_12,
					};



private void
setup()
{
	int	pathok = 0;

	DBUG_ENTER("setup");

	errno = 0;
	if(sysconf(_SC_XOPEN_ENH_I18N) == -1)
	{
		if (errno != 0)
		{
			xx_all(DELETION, "sysconf(_SC_XOPEN_ENH_I18N) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		tsup = 0;
	}
	else
	{
		tsup = 1;
		PATH_TRACE;
	}

	PATH_SETUP_RPT(1);

	DBUG_VOID_RETURN;
}


private void
test1()
{
	char	*format, *str;
	int	pathok = setupok;

	DBUG_ENTER("test1");

	testfail = 0;

	init_tms();
	testcomment = "whitespace in format string matches upto first non-whitespace in test string";
	chkbuf.tm_sec = 56;
	chkbuf.tm_min = 34;
	chkbuf.tm_hour = 12;
	str = "  12   34  56 "; 
	format = " %H %M %S ";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	char	*format, *str;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	testfail = 0;

	init_tms();
	testcomment = "ordinary characters are matched in both strings";
	chkbuf.tm_sec = 56;
	chkbuf.tm_min = 34;
	chkbuf.tm_hour = 12;
	str = "ab 12 !?* 34 fg 56"; 
	format = "ab %H !?* %M fg %S";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}


private void
test3()
{
	DBUG_ENTER("test3");

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t3()
{
	char	*format, *str;
	char	str2[50];
	int	pathok = setupok;

	DBUG_ENTER("ch_t3");

	testfail = 0;

	init_tms();
	testcomment = "day of the month with leading zeroes";
	chkbuf.tm_mday = 1;
	str = "01"; 
	format = "%d";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "day of the month without leading zeroes";
	chkbuf.tm_mday = 31;
	str = "31"; 
	format = "%d";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "date in %m/%d/%y format";
	chkbuf.tm_mon = 1;
	chkbuf.tm_mday = 9;
	chkbuf.tm_year = 91;
	str = "02/09/91"; 
	format = "%D";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "hour with leading zeroes";
	chkbuf.tm_hour = 0;
	str = "00"; 
	format = "%H";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "hour without leading zeroes";
	chkbuf.tm_hour = 23;
	str = "23"; 
	format = "%H";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "hour with leading zeroes";
	chkbuf.tm_hour = 1;
	str = "01"; 
	format = "%I";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "hour without leading zeroes";
	chkbuf.tm_hour = 12;
	str = "12"; 
	format = "%I";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "day of the year with leading zeroes";
	chkbuf.tm_yday = 0;
	str = "001"; 
	format = "%j";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "day of the year without leading zeroes";
	chkbuf.tm_yday = 365;
	str = "366"; 
	format = "%j";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "minute with leading zeroes";
	chkbuf.tm_min = 0;
	str = "00"; 
	format = "%M";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "minute without leading zeroes";
	chkbuf.tm_min = 59;
	str = "59"; 
	format = "%M";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "month of the year with leading zeroes";
	chkbuf.tm_mon = 0;
	str = "01"; 
	format = "%m";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "month of the year without leading zeroes";
	chkbuf.tm_mon = 11;
	str = "12"; 
	format = "%m";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "match any whitespace character sequence";
	str = " \t\r\n\v\f"; 
	format = "%n";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;
	format = "%t";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	/* 
	 * The next sub-test involves the AM/PM string so we set the locale.
	 * We check to see that strptime picks up data from the correct places
	 * in the locale. First we set the locale using LC_ALL and then using
	 * LC_TIME.
	 */

	if (setlocale(LC_ALL, INFO2LANG) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO2LANG);
		in_rpt("Check installation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	langnum = VALINFO2LANG;
	currlang = INFO2LANG;

	init_tms();
	chkbuf.tm_hour = 23;
	chkbuf.tm_min = 40;
	chkbuf.tm_sec = 15;
	
	testfail = 0;
#if TEST_XPG >= 500
	testcomment = "appropriate time as specified in T_FMT_AMPM string";
#else
	testcomment = "time in %I:%M:%S %p format";
#endif
	(void) strcpy(str2,"11:40:15 ");
	(void) strcat(str2, pslanginfo(VSX_PM_STR, langnum));
	format = "%r";
	globok = 0;
	chkstrptime(str2, format);
	PATH_FUNC_TRACE;

	if (setlocale(LC_TIME, INFO1LANG) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO1LANG);
		in_rpt("Check installation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	langnum = VALINFO1LANG;
	currlang = INFO1LANG;

#if TEST_XPG >= 500
	init_tms();
	chkbuf.tm_hour = 11;
	chkbuf.tm_min = 49;
	chkbuf.tm_sec = 45;
	
	testfail = 0;
	testcomment = "appropriate time as specified in T_FMT_AMPM string";
	(void) strcpy(str2,"11.49.45 ");
	(void) strcat(str2, pslanginfo(VSX_AM_STR, langnum));
	format = "%r";
	globok = 0;
	chkstrptime(str2, format);
	PATH_FUNC_TRACE;
#else
	/* skip INFO1LANG test, due to grey area in XPG4 spec */
	PATH_TRACE;
#endif
	
	/* language is no longer important */
	langnum = -1;

	init_tms();
	testcomment = "seconds with leading zeroes";
	chkbuf.tm_sec = 0;
	str = "00"; 
	format = "%S";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "seconds without leading zeroes";
	chkbuf.tm_sec = 61;
	str = "61"; 
	format = "%S";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "time in %H:%M:%S notation";
	chkbuf.tm_hour = 17;
	chkbuf.tm_min = 35;
	chkbuf.tm_sec = 52;
	str = "17:35:52"; 
	format = "%T";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "weekday , two digit year, and week number of the year, with leading zeroes";
	chkbuf.tm_wday = 0;
	chkbuf.tm_yday = 5;
	chkbuf.tm_mday = 6;
	chkbuf.tm_mon = 0;
	/* non-leap year */
	chkbuf.tm_year = 91;
	str = "0091 00 01";
	format = "%y %w %U";
	globok = 0;
	chkweekday(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "weekday , two digit year, and week number of the year, without leading zeroes";
	chkbuf.tm_wday = 4;
	chkbuf.tm_yday = 365;
	chkbuf.tm_mday = 31;
	chkbuf.tm_mon = 11;
	/* leap year */
	chkbuf.tm_year = 92;
	str = "92 4 52";
	format = "%y %w %U";
	globok = 0;
	chkweekday(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "weekday , two digit year, and week number of the year, with leading zeroes";
	chkbuf.tm_wday = 0;
	chkbuf.tm_yday = 12;
	chkbuf.tm_mday = 13;
	chkbuf.tm_mon = 0;
	/* non-leap year */
	chkbuf.tm_year = 91;
	str = "0091 00 01";
	format = "%y %w %W";
	globok = 0;
	chkweekday(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "weekday , two digit year, and week number of the year, without leading zeroes";
	chkbuf.tm_wday = 4;
	chkbuf.tm_yday = 365;
	chkbuf.tm_mday = 31;
	chkbuf.tm_mon = 11;
	/* leap year */
	chkbuf.tm_year = 92;
	str = "92 4 52";
	format = "%y %w %W";
	globok = 0;
	chkweekday(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "last two digits of year with leading zeroes";
	chkbuf.tm_year = 91;
	str = "0091"; 
	format = "%y";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "last two digits of year without leading zeroes";
	chkbuf.tm_year = 91;
	str = "91"; 
	format = "%y";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "%% in format string is mapped to % in test string";
	chkbuf.tm_year = 91;
	str = "% 1991 %";
	format = "%% %Y %%";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(29);

	DBUG_VOID_RETURN;
}


private void
test4()
{
	char	*format, *str;
	int	pathok = setupok;

	DBUG_ENTER("test4");

	testfail = 0;

	init_tms();
	testcomment = "time in %H:%M format";
	chkbuf.tm_hour = 12;
	chkbuf.tm_min = 34;
	str = "12:34"; 
	format = "%R";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}


private void
test5()
{
	DBUG_ENTER("test5");

	(void) cppair(ch_t5, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t5()
{
	char	*format, *str;
	static char *fmt_day[] = { "%a", "%A", NULL};
	static char *fmt_month[] = { "%b", "%B", "%h", NULL};
	char	str2[50];
	int	cnt, i;
	int	pathok = setupok;

	DBUG_ENTER("ch_t5");

	testfail = 0;

	if (setlocale(LC_ALL, INFO1LANG) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO1LANG);
		in_rpt("Check installation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	langnum = VALINFO1LANG;
	currlang = INFO1LANG;

	for( cnt = 0; fmt_day[cnt] != NULL; cnt++)
	{
		testcomment = "The full weekday names are matched.";
		for( i = 0; i < NUMDAYS; i++)
		{
			(void) strcpy( str2, pslanginfo(dayitems[i], langnum));
			init_tms();
			chkbuf.tm_wday = i;
			globok = 0;
			chkstrptime(str2, fmt_day[cnt]);
			PATH_FUNC_TRACE;
		}
		testcomment = "The abbreviated weekday names are matched.";
		for( i = 0; i < NUMDAYS; i++)
		{
			(void) strcpy( str2, pslanginfo(abdayitems[i],langnum));
			init_tms();
			chkbuf.tm_wday = i;
			globok = 0;
			chkstrptime(str2, fmt_day[cnt]);
			PATH_FUNC_TRACE;
		}
	}

	for( cnt = 0; fmt_month[cnt] != NULL; cnt++)
	{
		testcomment = "The full month names are matched.";
		for( i = 0; i < NUMMONTHS; i++)
		{
			(void) strcpy( str2, pslanginfo(monitems[i],langnum));
			init_tms();
			chkbuf.tm_mon = i;
			globok = 0;
			chkstrptime(str2, fmt_month[cnt]);
			PATH_FUNC_TRACE;
		}
		testcomment = "The abbreviated month names are matched.";
		for( i = 0; i < NUMMONTHS; i++)
		{
			(void) strcpy( str2, pslanginfo(abmonitems[i],langnum));
			init_tms();
			chkbuf.tm_mon = i;
			globok = 0;
			chkstrptime(str2, fmt_month[cnt]);
			PATH_FUNC_TRACE;
		}
	}

	init_tms();
	testcomment = "appropriate date and time as specified in D_T_FMT string";
	chkbuf.tm_hour = 18;
	chkbuf.tm_min = 42;
	chkbuf.tm_sec = 21;
	chkbuf.tm_mday = 5;
	chkbuf.tm_mon = 8;
	chkbuf.tm_year = 91;
	(void) strcpy (str2, "6.42.21 ");
	(void) strcat (str2, pslanginfo(VSX_PM_STR, langnum));
	(void) strcat (str2, " 09/05/91");
	format = "%c";
	globok = 0;
	chkstrptime(str2, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "AM string as specified in AM_STR";
	chkbuf.tm_hour = 6;
	(void) strcpy (str2, "6 ");
	(void) strcat (str2, pslanginfo(VSX_AM_STR, langnum));
	format = "%I %p";
	globok = 0;
	chkstrptime(str2, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "appropriate date as specified in D_FMT string";
	chkbuf.tm_mday = 5;
	chkbuf.tm_mon = 8;
	chkbuf.tm_year = 91;
	str = "09/05/91";
	format = "%x";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "appropriate time as specified in T_FMT string";
	chkbuf.tm_hour = 18;
	chkbuf.tm_min = 42;
	chkbuf.tm_sec = 21;
	(void) strcpy (str2, "6.42.21 ");
	(void) strcat (str2, pslanginfo(VSX_PM_STR, langnum));
	format = "%X";
	globok = 0;
	chkstrptime(str2, format);
	PATH_FUNC_TRACE;


	if (setlocale(LC_TIME, INFO2LANG) == (char *) NULL)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("Pseudo Language %s is not available", INFO2LANG);
		in_rpt("Check installation of pseudo language");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	langnum = VALINFO2LANG;
	currlang = INFO2LANG;

	for( cnt = 0; fmt_day[cnt] != NULL; cnt++)
	{
		testcomment = "The full weekday names are matched.";
		for( i = 0; i < NUMDAYS; i++)
		{
			(void) strcpy( str2, pslanginfo(dayitems[i], langnum));
			init_tms();
			chkbuf.tm_wday = i;
			globok = 0;
			chkstrptime(str2, fmt_day[cnt]);
			PATH_FUNC_TRACE;
		}
		testcomment = "The abbreviated weekday names are matched.";
		for( i = 0; i < NUMDAYS; i++)
		{
			(void) strcpy( str2, pslanginfo(abdayitems[i],langnum));
			init_tms();
			chkbuf.tm_wday = i;
			globok = 0;
			chkstrptime(str2, fmt_day[cnt]);
			PATH_FUNC_TRACE;
		}
	}

	for( cnt = 0; fmt_month[cnt] != NULL; cnt++)
	{
		testcomment = "The full month names are matched.";
		for( i = 0; i < NUMMONTHS; i++)
		{
			(void) strcpy( str2, pslanginfo(monitems[i],langnum));
			init_tms();
			chkbuf.tm_mon = i;
			globok = 0;
			chkstrptime(str2, fmt_month[cnt]);
			PATH_FUNC_TRACE;
		}
		testcomment = "The abbreviated month names are matched.";
		for( i = 0; i < NUMMONTHS; i++)
		{
			(void) strcpy( str2, pslanginfo(abmonitems[i],langnum));
			init_tms();
			chkbuf.tm_mon = i;
			globok = 0;
			chkstrptime(str2, fmt_month[cnt]);
			PATH_FUNC_TRACE;
		}
	}

	init_tms();
	testcomment = "appropriate date and time as specified in D_T_FMT string";
	chkbuf.tm_hour = 6;
	chkbuf.tm_min = 42;
	chkbuf.tm_sec = 21;
	chkbuf.tm_mday = 5;
	chkbuf.tm_mon = 8;
	chkbuf.tm_year = 91;
	str = "5 9 1991 6:42:21";
	format = "%c";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "PM string as specified in PM_STR";
	chkbuf.tm_hour = 18;
	(void) strcpy (str2, "6 ");
	(void) strcat (str2, pslanginfo(VSX_PM_STR, langnum));
	format = "%I %p";
	globok = 0;
	chkstrptime(str2, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "appropriate date as specified in D_FMT string";
	chkbuf.tm_mday = 5;
	chkbuf.tm_mon = 8;
	chkbuf.tm_year = 91;
	str = "05 09 1991";
	format = "%x";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "appropriate time as specified in T_FMT string";
	chkbuf.tm_hour = 6;
	chkbuf.tm_min = 42;
	chkbuf.tm_sec = 21;
	str = "6:42:21";
	format = "%X";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(10 + 2 * (2*2*NUMDAYS + 2*3*NUMMONTHS));

	DBUG_VOID_RETURN;
}



private void
test6()
{
	DBUG_ENTER("test6");

	(void) cppair(ch_t6, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t6()
{
	char	*format, *str;
	static char *fmt_day[] = { "%a", "%A", NULL};
	static char *fmt_month[] = { "%b", "%B", "%h", NULL};
	char	str2[50];
	int	cnt, i;
	int	pathok = setupok;

	DBUG_ENTER("ch_t6");

	testfail = 0;

	for( cnt = 0; fmt_day[cnt] != NULL; cnt++)
	{
		testcomment = "The full weekday names are matched.";
		for( i = 0; i < NUMDAYS; i++)
		{
			(void) strcpy( str2, pslanginfo(dayitems[i], 0));
			init_tms();
			chkbuf.tm_wday = i;
			globok = 0;
			chkstrptime(str2, fmt_day[cnt]);
			PATH_FUNC_TRACE;
		}
		testcomment = "The abbreviated weekday names are matched.";
		for( i = 0; i < NUMDAYS; i++)
		{
			(void) strcpy( str2, pslanginfo(abdayitems[i],0));
			init_tms();
			chkbuf.tm_wday = i;
			globok = 0;
			chkstrptime(str2, fmt_day[cnt]);
			PATH_FUNC_TRACE;
		}
	}

	for( cnt = 0; fmt_month[cnt] != NULL; cnt++)
	{
		testcomment = "The full month names are matched.";
		for( i = 0; i < NUMMONTHS; i++)
		{
			(void) strcpy( str2, pslanginfo(monitems[i],0));
			init_tms();
			chkbuf.tm_mon = i;
			globok = 0;
			chkstrptime(str2, fmt_month[cnt]);
			PATH_FUNC_TRACE;
		}
		testcomment = "The abbreviated month names are matched.";
		for( i = 0; i < NUMMONTHS; i++)
		{
			(void) strcpy( str2, pslanginfo(abmonitems[i],0));
			init_tms();
			chkbuf.tm_mon = i;
			globok = 0;
			chkstrptime(str2, fmt_month[cnt]);
			PATH_FUNC_TRACE;
		}
	}

	init_tms();
	testcomment = "appropriate date and time as specified in D_T_FMT string";
	chkbuf.tm_hour = 6;
	chkbuf.tm_min = 42;
	chkbuf.tm_sec = 21;
	chkbuf.tm_mday = 5;
	chkbuf.tm_wday = 4;
	chkbuf.tm_mon = 8;
	chkbuf.tm_year = 91;
	(void) strcpy (str2, pslanginfo(VSX_DAY_5, 0));
	(void) strcat (str2, " ");
	(void) strcat (str2, pslanginfo(VSX_MON_9, 0));
	(void) strcat (str2, " 5 6:42:21 1991");
	format = "%c";
	globok = 0;
	chkstrptime(str2, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "AM string as specified in AM_STR";
	chkbuf.tm_hour = 6;
	(void) strcpy (str2, "6 ");
	(void) strcat (str2, pslanginfo(VSX_AM_STR, 0));
	format = "%I %p";
	globok = 0;
	chkstrptime(str2, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "appropriate date as specified in D_FMT string";
	chkbuf.tm_mday = 5;
	chkbuf.tm_mon = 8;
	chkbuf.tm_year = 91;
	str = "09/05/91";
	format = "%x";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	init_tms();
	testcomment = "appropriate time as specified in T_FMT string";
	chkbuf.tm_hour = 6;
	chkbuf.tm_min = 42;
	chkbuf.tm_sec = 21;
	str = "6:42:21";
	format = "%X";
	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT((2*(2*NUMDAYS))+(3*(2*NUMMONTHS))+4);

	DBUG_VOID_RETURN;
}



private void
test7()
{
	char	*format, *str;
	char	*expstr = " OK.";
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test7");

	testfail = 0;

	init_tms();
	testcomment = "Return value points to character after end of format matching.";
	chkbuf.tm_sec = 56;
	chkbuf.tm_min = 34;
	chkbuf.tm_hour = 12;
	str = "12 34 56 OK."; 
	format = "%H %M %S";

	/* first call just checks for support */

	errno = 0;
	return_ptr = strptime(str, format, &testbuf);
	err = errno;

	globok = 0;
	if (chksupp(return_ptr) == 0)
		PATH_FUNC_TRACE;

	if (return_ptr == NULL && err == ENOSYS)
	{
		if (testfail == 0)
			PATH_XS_RPT(1);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* now do the test proper */

	globok = 0;
	chkstrptime(str, format);
	PATH_FUNC_TRACE;

	if (return_ptr == NULL || strcmp(return_ptr, expstr) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("strptime(\"%s\", \"%s\", &testbuf) did not return expected value",
			str, format);
		in_rpt("expected return: \"%s\"", expstr);
		if (return_ptr == NULL)
			in_rpt("observed return: (null)");
		else
			in_rpt("observed return: \"%s\"", return_ptr);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


private void
test8()
{
	char	*format, *str;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test8");

	testfail = 0;

	str = "12 34 56"; 
	format = "%H %M %S NOT OK";

	errno = 0;
	return_ptr = strptime(str, format, &testbuf);
	err = errno;

	globok = 0;
	if (chksupp(return_ptr) == 0)
		PATH_FUNC_TRACE;

	if (return_ptr == NULL && err == ENOSYS)
	{
		if (testfail == 0)
			PATH_XS_RPT(1);
		DBUG_VOID_RETURN;
	}
	else if( return_ptr != (char *) NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("strptime(\"%s\", \"%s\", &testbuf) did not return NULL",
			str, format);
	}
	else
		PATH_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
chkstrptime(str, fmt)
char *str, *fmt;
{
	int err;
	int pathok = 0;

	DBUG_ENTER("chkstrptime");

	errno = 0;
	return_ptr = strptime(str, fmt, &testbuf);
	err = errno;

#ifdef TESTBED
	return_ptr = NULL;
#endif

	globok = 0;
	if (chksupp(return_ptr) == 0)
		PATH_FUNC_TRACE;

	if (return_ptr == NULL && err == ENOSYS)
	{
		PATH_FUNC_RPT(1);
		DBUG_VOID_RETURN;
	}
	else if (return_ptr == NULL || !tm_match())
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("%s", testcomment);
		if (langnum != -1)
		{
			in_rpt("strptime(\"%s\", \"%s\", &testbuf) failed.",
				psl_to_asc(str, langnum), fmt);
			in_rpt("Current language is %s", currlang);
		}
		else
			in_rpt("strptime(\"%s\", \"%s\", &testbuf) failed.",
				psl_to_asc(str, 0), fmt);

		if (return_ptr == NULL)
			in_rpt("  return value was NULL");
		else
		{
			if (chkbuf.tm_sec != -1 &&
					testbuf.tm_sec != chkbuf.tm_sec)
				in_rpt("  tm_sec = %d, expected %d", 
					testbuf.tm_sec, chkbuf.tm_sec);
			if (chkbuf.tm_min != -1 &&
					testbuf.tm_min != chkbuf.tm_min)
				in_rpt("  tm_min = %d, expected %d", 
					testbuf.tm_min, chkbuf.tm_min);
			if (chkbuf.tm_hour != -1 &&
					testbuf.tm_hour != chkbuf.tm_hour)
				in_rpt("  tm_hour = %d, expected %d", 
					testbuf.tm_hour, chkbuf.tm_hour);
			if (chkbuf.tm_mday != -1 &&
					testbuf.tm_mday != chkbuf.tm_mday)
				in_rpt("  tm_mday = %d, expected %d", 
					testbuf.tm_mday, chkbuf.tm_mday);
			if (chkbuf.tm_mon != -1 &&
					testbuf.tm_mon != chkbuf.tm_mon)
				in_rpt("  tm_mon = %d, expected %d", 
					testbuf.tm_mon, chkbuf.tm_mon);
			if (chkbuf.tm_year != -1 &&
					testbuf.tm_year != chkbuf.tm_year)
				in_rpt("  tm_year = %d, expected %d", 
					testbuf.tm_year, chkbuf.tm_year);
			if (chkbuf.tm_wday != -1 &&
					testbuf.tm_wday != chkbuf.tm_wday)
				in_rpt("  tm_wday = %d, expected %d", 
					testbuf.tm_wday, chkbuf.tm_wday);
			if (chkbuf.tm_yday != -1 &&
					testbuf.tm_yday != chkbuf.tm_yday)
				in_rpt("  tm_yday = %d, expected %d", 
					testbuf.tm_yday, chkbuf.tm_yday);
			if (chkbuf.tm_isdst != -1 &&
					testbuf.tm_isdst != chkbuf.tm_isdst)
				in_rpt("  tm_isdst = %d, expected %d", 
					testbuf.tm_isdst, chkbuf.tm_isdst);
		}
#ifdef TESTBED
		{
		char buf[50];
		strftime(buf, sizeof(buf), fmt, &chkbuf);
		if (strcmp(str, buf) != 0)
			in_rpt("strftime: %s",
			    langnum == -1 ? buf : psl_to_asc(buf, langnum));
		}
#endif
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}


private int
tm_match()
{
	if( (testbuf.tm_sec == chkbuf.tm_sec  || chkbuf.tm_sec  == -1)
	    && (testbuf.tm_min == chkbuf.tm_min || chkbuf.tm_min == -1)
	    && (testbuf.tm_hour == chkbuf.tm_hour  || chkbuf.tm_hour  == -1)
	    && (testbuf.tm_mday == chkbuf.tm_mday || chkbuf.tm_mday == -1)
	    && (testbuf.tm_mon == chkbuf.tm_mon || chkbuf.tm_mon == -1)
	    && (testbuf.tm_year == chkbuf.tm_year || chkbuf.tm_year == -1)
	    && (testbuf.tm_wday == chkbuf.tm_wday || chkbuf.tm_wday == -1)
	    && (testbuf.tm_yday == chkbuf.tm_yday || chkbuf.tm_yday == -1)
	    && (testbuf.tm_isdst == chkbuf.tm_isdst || chkbuf.tm_isdst == -1))
		return(1);
	else
		return(0);
}

private void
chkweekday(str, fmt)
char *str, *fmt;
{
	int err;
	int pathok = 0;

	DBUG_ENTER("chkweekday");

	/* Supplying the week number with year and day of the week gives
	   information which could be represented either by tm_yday or by
	   tm_mday and tm_mon together, so allow either behaviour. */

	errno = 0;
	return_ptr = strptime(str, fmt, &testbuf);
	err = errno;

#ifdef TESTBED
	return_ptr = NULL;
#endif

	globok = 0;
	if (chksupp(return_ptr) == 0)
		PATH_FUNC_TRACE;

	if (return_ptr == NULL && err == ENOSYS)
	{
		PATH_FUNC_RPT(1);
		DBUG_VOID_RETURN;
	}
	else if (return_ptr == NULL || testbuf.tm_wday != chkbuf.tm_wday ||
		!(testbuf.tm_yday == chkbuf.tm_yday ||
		  testbuf.tm_mday == chkbuf.tm_mday &&
		  testbuf.tm_mon == chkbuf.tm_mon))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("%s", testcomment);
		in_rpt("strptime(\"%s\", \"%s\", &testbuf) failed.",
			psl_to_asc(str, 0), fmt);

		if (return_ptr == NULL)
			in_rpt("  return value was NULL");
		else
		{
			if (testbuf.tm_wday != chkbuf.tm_wday)
				in_rpt("  tm_wday = %d, expected %d", 
					testbuf.tm_wday, chkbuf.tm_wday);
			if ( !(testbuf.tm_yday == chkbuf.tm_yday ||
			       testbuf.tm_mday == chkbuf.tm_mday &&
			       testbuf.tm_mon == chkbuf.tm_mon))
			{
				in_rpt("  expected either: tm_yday = %d",
					chkbuf.tm_yday);
				in_rpt("               or: tm_mday = %d and tm_mon = %d",
					chkbuf.tm_mday, chkbuf.tm_mon);
				in_rpt("  observed: tm_yday = %d, tm_mday = %d, tm_mon = %d",
					testbuf.tm_yday, testbuf.tm_mday,
					testbuf.tm_mon);
			}
		}
#ifdef TESTBED
		{
		char buf[50];
		strftime(buf, sizeof(buf), fmt, &chkbuf);
		if (strcmp(str, buf) != 0)
			in_rpt("strftime: %s",
			    langnum == -1 ? buf : psl_to_asc(buf, langnum));
		}
#endif
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}


private int
chksupp(rval)
char *rval;
{
	/* Check on support of strptime().  Return -1 if an error has
	   been reported, else 0. */

	int pathok = 0;
	int err;

	DBUG_ENTER("chksupp");

	/* This function uses the errno value set by strptime(), so
	   there must be no other calls between it and this function */
	err = errno;

	/* If the ENHANCED I18N group is not supported, strptime() can
	   still be supported on its own */
	if (rval == NULL && err == ENOSYS)
	{
		if (tsup)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("strptime() gave ENOSYS when ENHANCED I18N group is supported");
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);
	DBUG_RETURN(0);
}

private void
init_tms()
{
	testbuf.tm_sec = chkbuf.tm_sec = -1;
	testbuf.tm_min = chkbuf.tm_min = -1;
	testbuf.tm_hour = chkbuf.tm_hour = -1;
	testbuf.tm_mday = chkbuf.tm_mday = -1;
	testbuf.tm_mon = chkbuf.tm_mon = -1;
	testbuf.tm_year = chkbuf.tm_year = -1;
	testbuf.tm_wday = chkbuf.tm_wday = -1;
	testbuf.tm_yday = chkbuf.tm_yday = -1;
	testbuf.tm_isdst = chkbuf.tm_isdst = -1;
}
