/*
 * (C) Copyright 2001 The Free Standards Group Inc
 *
 * 2001, Andrew Pham (sh version) 
 * 2001, Christopher Yeoh (Adapted from sh version to use C api)
 *
 *
 */

#ifndef lint
#define MAIN
#ifdef	MAIN
char	*copyright[] = {
		"(C) Copyright 2001 The Free Standards Group Inc"
};
#endif	/* MAIN */
#endif	/* lint */

/* 
 * Name: T.setpgrp
 * Standard: LSB
 *
 *
 *  Assumptions:
 * 	fork(), setpgid(), getpid(), getppid() and getpgrp() are availbe to use. 
 *  
 *  Strategy Statement:
 * 	Write a small c function to test setpgrp(). 
 *  
 *  Expected Results:
 *     	As specified in the spec. 
 */

#include <unistd.h>
#include <sys/types.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include <tet_api.h>


int no_tests = 1;

void test1();



/* TCM Data structures */
void	(*setfunc) () = NULL;
void	(*clnfunc) () = NULL;
struct tet_testlist tet_testlist[] = {
  test1, 1,
  NULL, 0
};

void test1()
{
  pid_t ch_pid;
  unsigned int val, result;
/* Make sure all C API's/Funct's are available; else unresolve: 3 */
  if(getpid() < 0)
  {
    tet_infoline("getpid() returned values less than 0");
    tet_result(TET_UNRESOLVED);
    return;
  }

  if(getpgrp() < 0)
  {
    tet_infoline("getpgrp() returned error");
    tet_printf("  error: %s", strerror(errno));
    tet_result(TET_UNRESOLVED);
    return;
  }
  if(setpgid(0, getpgrp()) < 0)
  {
    tet_infoline("setpgid() returned error");
    tet_printf("  error: %s", strerror(errno));
    tet_result(TET_UNRESOLVED);
    return;
  }

/*
 *Make current pid and current group pid are different.
 */
  if ( getpid() == getpgrp())
  {
	  if (setpgrp() != 0)
	  {
	      tet_infoline("setpgrp failed.");
	      tet_result(TET_FAIL);
	      return;
	  }
/*
 * fork a new process to test
 */
	  ch_pid = fork();
	  if(ch_pid < 0)
	  {
	    tet_infoline("fork() returned error");
	    tet_printf("  error: %s", strerror(errno));
	    tet_result(TET_UNRESOLVED);
	  }
	  else if (ch_pid == 0) {
		if(getpid() == getpgrp())
			setpgid(0, getppid());
		if(getpid() == getpgrp()) {
			tet_infoline("Un-able to make pid and gpid different.");
			exit(3);
			}
		setpgrp();
		if(getpid() != getpgrp()) {
			tet_infoline("setpgrp() failed: pid and gpid not the same.");
			exit(1);
			}
		exit(0);  
	   } 
	  else 
	   {
		wait(&val);
		result = val >> 8;
		if(result == 1)
		      tet_result(TET_FAIL);
		else if (result == 3)
	    	      tet_result(TET_UNRESOLVED);
		else
		      tet_result(TET_PASS);
           }
  }
  else 
  {
	  if (setpgrp() == 0)
	  {
		    if (getpid() == getpgrp()) 
		    {
		      tet_result(TET_PASS);
		    }
		    else
		    {
		      tet_infoline("setpgrp failed to change process group correctly");
		      tet_result(TET_FAIL);
		    }
	  }
	  else
	  {
		    tet_infoline("setpgrp failed");
		    tet_result(TET_FAIL);
	  }
  }
}
