/*
############################################################################
# (C) Copyright 2001 The Free Standards Group  Inc
# ./tset/LSB.usersgroups/usersgroups/commands/root_funcs.sh
# root_funcs.sh : helper functions for paswd-tc.sh
#
# Log: root_funcs.sh,v 
#
# Revision 1.1  2001/07/05 11:22:05  cyeoh
# 	Adds in passwd command tests
#
# Revision 1.2 2001/07/19 15:42:39 Apham
#	Adds more descriptive comments
#
############################################################################
 * Sets up master/slave pseudo ttys. Forks and execs given
 * program and passes standard input of parent to child through
 * pseudo terminal. Attaches slave pty to child's stdin, stdout
 * and stderr before exec'ing.
 *
 * Probably has some problems if the exec'd program outputs
 * too much data after the parent has finished passing data to it.
 * Doesn't do anything with data passed back from child to parent.
 *
 * Chris Yeoh (yeohc@au1.ibm.com), IBM
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <sys/wait.h>
#include <string.h>

#define READBUFSIZE 500

int main(int argc, char *argv[])
/**************************************************************************
* This functions allows interactive testing.
**************************************************************************/
{
  int pt;
  char *pttyname;
  int slave;
  pid_t cpid;

  if (argc<2)
  {
    fprintf(stderr, "Must supply program name to exec\n");
    exit(1);
  }

  /* Setup master/slave pseudo tty's */
  pt = getpt();
  pttyname = ptsname(pt);
  unlockpt(pt);
  slave = open(pttyname, O_RDWR);

  if (slave==-1)
  {
    perror("Couldn't open pty");
    exit(1);
  }

  fflush(NULL);
  cpid = fork();

  if (cpid==-1)
  {
    perror("fork failed");
    exit(1);
  }
  else if (cpid!=0)
  {
    /* Parent */
    char readbuf[READBUFSIZE];
    int rd_count;
    fd_set rds;
    close(slave);


    while (1)
    {
      /* Read from pt so it doesn't block up and cause
         problems on the other end */
      int stdin_exhausted = 0;
      FD_ZERO(&rds);
      if (!stdin_exhausted) FD_SET(0, &rds);
      FD_SET(pt, &rds);

      sleep(1); /* Magic sleep. Ick. */
      if (select(pt+1, &rds, NULL, NULL, NULL)<-1)
      {
	perror("select failed");
      }

      if (FD_ISSET(pt, &rds))
      {
	rd_count = read(pt, readbuf, READBUFSIZE-1);
	if (rd_count<=0)
	{
	  int status;
	  waitpid(cpid, &status, 0);
	  exit(WEXITSTATUS(status));
	}
	/* We just throw away any data from child */
	/* Uncomment the next two lines to get data back
	   from child. Note that data you send gets echoed
	   back and may appear to be out of sync */
/*  	readbuf[rd_count] = 0; */
/*  	fprintf(stderr, "%s\n", readbuf); */
      }
      else if (FD_ISSET(0, &rds))
      {
	/* Read from stdin to get data to pass to child */
	int num_read;
	rd_count = 0;

	while (rd_count<READBUFSIZE)
	{
	  num_read = read(0, readbuf+rd_count, 1);

	  if (num_read==-1)
	  {
	    perror("Read on stdin failed");
	    exit(1);
	  }
	  else if (num_read==0)
	  {
	    /* stdin has closed */
	    stdin_exhausted = 1;
	  }
	  else
	  {
	    rd_count++;
	    if (readbuf[rd_count-1]=='\n') break;
	  }
	}

	{
	  /* Pass data down to child */
	  int count = 0;
	  int wrote;

	  readbuf[rd_count] = 0;
	  while (count<rd_count)
	  {
	    wrote =  write(pt, readbuf+count, rd_count-count);
	    if (wrote<-1)
	    {
	      perror("Write on pt failed");
	      exit(1);
	    }
	    count += wrote;
	  }

	}
      }

    }
  }
  else
  {
    /* Child */
    setsid(); /* Ensure /dev/tty gets set to us */
    close(pt);

    /* Reattach stdin, stdout, stderr */
    dup2(slave,0);
    dup2(slave,1);
    dup2(slave,2);

    execvp(argv[1], argv+1);
    perror("exec failed");
    exit(255);
  }

  /* Shouldn't get here */
  exit(0);
}
