#!/bin/sh
############################################################################
# (C) Copyright 2001 The Free Standards Group  Inc
# ./tset/LSB.usersgroups/usersgroups/commands/passwd-tc.sh
# passwd-tc.sh : test case for passwd command
#
# Log: passwd-tc.sh,v 
#
# Note:  All of the test cases in this file are executed as a 'regular' user.
#
# Revision 1.1  2001/07/05 11:22:05  cyeoh
# 	Adds in passwd command tests
#
# Revision 1.2 2001/07/19 15:42:39 apham
#	Adds more descriptive comments
#
############################################################################

tet_startup="startup"			# startup function
tet_cleanup="cleanup"			# cleanup function
iclist="ic1 ic2 ic3"
ic1="tp1"
ic2="tp2"
ic3="tp3"

tp1()
{
	tpstart "passwd -g : change the password of the named group 14-1"
	tet_infoline "Expected specified group's password to change."

#Calling privileged functions: unique_group() and execute_groupadd().
#Both unique_group() and execute_groupadd() exist in .../SRC/
#	lsblib/usersgroups_funcs.sh
# .../tset/LSB.usersgrous/commands/passwd/root_helper.c gives privileged 
# 	access
	OTHER_GROUP=vsx80
	./root_helper execute_groupadd $OTHER_GROUP
	if [ $? -ne 0 ]; then
		tet_infoline "Unable to add a test group: $OTHER_GROUP"
		tpresult UNRESOLVED
		return
	fi

#Calling another privileged function: execute_passwd_g()
#execute_passwd_g exists in .../tset/LSB.usersgrous/commands/passwd/root_funcs.sh
	./root_helper execute_passwd_g $OTHER_GROUP
	if [ $? -eq 0 ]; then

#Verify that the group password is changed.
		./cpt newgrp $OTHER_GROUP << EOF
testing
groups > /tmp/groups.output1
exit
EOF
		cat /tmp/groups.output1 | grep $OTHER_GROUP > /dev/null 2>&1
		if [ $? -eq 0 ]; then
			tet_infoline "passwd -g: PASS"	
		else
			tet_infoline "group passwd did not change."
			FAIL=Y
		fi
	else
			FAIL=Y
	fi

	./root_helper remove_group $OTHER_GROUP
	rm -f /tmp/groups.output1 > /dev/null 2>&1
	tpresult
}

tp2()
{
    	tpstart "passwd -gr: remove the current password from the named group. 14.1" 
	tet_infoline "Expected specified group's password to change."

#Calling privileged functions: unique_group() and execute_groupadd()
# .../tset/LSB.usersgrous/commands/passwd/root_helper.c gives privileged 
# 	access

	OTHER_GROUP=vsx81
	./root_helper execute_groupadd $OTHER_GROUP
	if [ $? -ne 0 ]; then
		tet_infoline "Unable to add a test group"
		tpresult UNRESOLVED
		return
	fi

#Calling another privileged function: execute_passwd_g()
# .../tset/LSB.usersgrous/commands/passwd/root_helper.c gives privileged 
# 	access
#execute_passwd_g_r exists in .../tset/LSB.usersgrous/commands/passwd/root_funcs.sh

	./root_helper execute_passwd_g_r $OTHER_GROUP
	if [ $? -eq 0 ]; then

#Verify that the group password is changed.
		./cpt newgrp $OTHER_GROUP << EOF
testing
groups > /tmp/groups.output12
exit
EOF
		cat /tmp/groups.output12 | grep $OTHER_GROUP > /dev/null 2>&1
		if [ $? -eq 0 ]; then
			tet_infoline "group passwd did not change."
			FAIL=Y
		else
			tet_infoline "passwd -g -r: PASS"	
		fi
	else
			FAIL=Y
	fi

# .../tset/LSB.usersgrous/commands/passwd/root_helper.c gives privileged 
# 	access

	./root_helper remove_group $OTHER_GROUP
	rm -f /tmp/groups.output12 > /dev/null 2>&1
	tpresult
}

tp3()
{
    	tpstart "passwd -gR: restrict the named group for all users. 14.1" 
	tet_infoline "Expected specified group's password to change."

#Calling privileged functions: unique_group() and execute_groupadd()
# .../tset/LSB.usersgrous/commands/passwd/root_helper.c gives privileged 
# 	access

	OTHER_GROUP=vsx82
	./root_helper execute_groupadd $OTHER_GROUP

	if [ $? -ne 0 ]; then
		tet_infoline "Unable to add a test group: $OTHER_GROUP"
		tpresult UNRESOLVED
		return
	fi

#Calling another privileged function: execute_passwd_g_R()
# .../tset/LSB.usersgrous/commands/passwd/root_helper.c gives privileged 
# 	access.
#execute_passwd_g_R exists in .../tset/LSB.usersgrous/commands/passwd/root_funcs.sh

	./root_helper execute_passwd_g_R $OTHER_GROUP
	if [ $? -eq 0 ]; then

#Verify that the group password is changed.
# .../tset/LSB.usersgrous/commands/passwd/cpt.c allows interactive 
# 	testing with newgrp.

		./cpt newgrp $OTHER_GROUP << EOF
testing
groups > /tmp/groups.output123
exit
EOF
		cat /tmp/groups.output123 | grep $OTHER_GROUP > /dev/null 2>&1
		if [ $? -eq 0 ]; then
			tet_infoline "group passwd did not change."
			FAIL=Y
		else
			tet_infoline "passwd -g -R: PASS"	
		fi
	else
			FAIL=Y
	fi

	./root_helper remove_group $OTHER_GROUP
	rm -f /tmp/groups.output123 > /dev/null 2>&1
	tpresult
}

startup() # startup function
{
     rm -f out.stdout out.stderr out.experr >/dev/null 2>&1
}


cleanup() # clean-up function
{
     rm -f out.stdout out.stderr out.experr >/dev/null 2>&1
}

# source common shell functions

. $TET_EXECUTE/LIB/shfuncs.sh
. $TET_EXECUTE/LIB/usersgroups_funcs.sh



# execute shell test case manager - must be last line
. $TET_ROOT/lib/xpg3sh/tcm.sh
