/*
*      SCCS:  @(#)  POSIX.os/Csupport/fdopen/fdopen.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/Csupport/fdopen - T.fdopen Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.fdopen

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fdopen() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 1 July 87
		Removed abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Phyfos Photi, Hoskyns Group plc
		23/08/88
		fdopen() testset extracted from fopen() testset

		Linda Taylor, Hoskyns	September 1988
		Added tests 3 - 8, inclusive

		Geoff Clare, 16 Nov 1988
		Add tests for EBADF, EINVAL

		Geoff Clare, 23 Aug 1989
		fclose() streams succesfully opened in deviance tests

		Geoff Clare, 26 June 1990
		Delete "mode not allowed by type of stream" test.
		Add feof()/ferror() clear test.

		Anthony Lee, 24 July 1990
		Addded resilience code.

		Stuart Boutell, 31 Oct 1990
		Added Kieth Holders tests 11-16

		Geoff Clare, 15 Feb 1991
		Fix lseek() arguments.

		Geoff Clare, 20 Nov 1992
		fd must have O_APPEND set for "a" mode.

		Geoff Clare, 2 June 1993
		Add O_APPEND to second open() call in test 16.
		
************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<string.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fdopen
#endif



#define NO_TESTS	16		/* number of tests in testset */
#define	T2_NFD		3
#define MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();
extern	int	outstr();

int	no_tests = NO_TESTS;

private char	*t_string =		"abcdefghijklmnopqrstuvwxyz";
private char	*t4_string = 		"ZYXWV";
private char	*t4ret_string = 	"abcdefghijklmnopqrstuvwxyzZYXWV";
private char	*t5_string =		"ABCDEFG";
private char	*t5ret_string = 	"abcdefghijklmnopqrstuvwxyzABCDEFG";
private char	*t6_string =		"IJKLMNOP";
private char	*t6ret_string = 	"abcdefghijklmnopqrstuvwxyzIJKLMNOP";
private char	*t7_string =		"QR";
private char	*t7ret_string = 	"abcdefghijklmnopqrstuvwxyzQR";
private char	*t8_string =		"STU";
private char	*t8ret_string = 	"abcdefghijklmnopqrstuvwxyzSTU";
private char	*t12_string = 		"ZYXWV";
private char	*t12ret_string = 	"abcdefghijklmnopqrstuvwxyzZYXWV";
private char	*t13_string =		"ABCDEFG";
private char	*t13ret_string = 	"abcdefghijklmnopqrstuvwxyzABCDEFG";
private char	*t14_string =		"IJKLMNOP";
private char	*t14ret_string = 	"abcdefghijklmnopqrstuvwxyzIJKLMNOP";
private char	*t15_string =		"QR";
private char	*t15ret_string = 	"abcdefghijklmnopqrstuvwxyzQR";
private char	*t16_string =		"STU";
private char	*t16ret_string = 	"abcdefghijklmnopqrstuvwxyzSTU";
private char	*t1_file =		"./fdopen-t.1";
private char	*t2_file =		"./fdopen-t.2";
private char	*t3_file =		"./fdopen-t.3";
private char	*t4_file =		"./fdopen-t.4";
private char	*t5_file =		"./fdopen-t.5";
private char	*t6_file =		"./fdopen-t.6";
private char	*t7_file =		"./fdopen-t.7";
private char	*t8_file =		"./fdopen-t.8";
private char	*t9_file =		"./fdopen-t.9";
private char	*t10_file =		"./fdopen-t.10";
private char	*t11_file =		"./fdopen-t.11";
private char	*t12_file =		"./fdopen-t.12";
private char	*t13_file =		"./fdopen-t.13";
private char	*t14_file =		"./fdopen-t.14";
private char	*t15_file =		"./fdopen-t.15";
private char	*t16_file =		"./fdopen-t.16";

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private	void	test11();
private	void	test12();
private void	test13();
private	void	test14();
private	void	test15();
private	void	test16();

private void	clean_tests();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	NULL, 0
};

private void
test1()
{
	int		fd;
	int		err;
	FILE		*fp;
	char		*delmsg = "depends on success of test 1";
	int	pathok = 0;
	
	DBUG_ENTER("test1");
	if ((fd = creat(t1_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d(%s)",
			t1_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fdopen(fd, "w")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"w\") on a valid file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		(void) unlink(t1_file);
		(void) tet_delete(2, delmsg);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (fileno(fp) != fd)
	{
		xx_rpt(FAILURE);
		in_rpt("file descriptor of stream returned by fdopen() was not the same as the one passed to it");
		(void) tet_delete(2, delmsg);
	}
	else
	{
		PATH_XS_RPT(2);
	}

	(void) fclose(fp);
	(void) unlink(t1_file);
	DBUG_VOID_RETURN;
}


private void
test2()
{
	int		fd;
	int		err;
	int		i;
	FILE		*fps[T2_NFD];
	int		pathok = 0;
	

	DBUG_ENTER("test2");
	if ((fd = creat(t2_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d(%s)",
			t2_file, MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	for (i = 0 ; i < T2_NFD ; i++)
	{
		if ((fps[i] = fdopen(fd, "w")) == (FILE *) NULL)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("fdopen(fd, \"w\") on a valid file descriptor failed - errno %d(%s)",
				err, errname(err));
			(void) close(fd);
			(void) unlink(t1_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
			
		if (fileno(fps[i]) != fd)
			break;
	}
	if (i != T2_NFD)
	{
		xx_rpt(FAILURE);
		in_rpt("on %d call with same file descriptor, fdopen() did not set file desc. of stream correctly", i + 1);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(2 + T2_NFD);
	}
	
	(void) fclose(fps[1]);
	(void) unlink(t2_file);
	DBUG_VOID_RETURN;
}

private void
test3()
{
	int	fd;
	FILE	*fp;
	int	err;
	int	fail;
	int	rval;
	int	expval;
	char	buf[30];
	int	pathok = 0;
	
	DBUG_ENTER("test3");
	fail = 0;
	
	if ((fd = creat(t3_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s) failed - errno %d(%s)",
			t3_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	if (outstr(fd, t_string) != strlen(t_string))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t3_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) close(fd);

	if ((fd = open (t3_file, O_RDWR, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR) failed - errno %d(%s)",
			t3_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	if ((fp = fdopen(fd, "r")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"r\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((rval = fwrite((void *)t_string, (size_t) 1, strlen(t_string), fp)) > 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"r\") on read/write file descriptor, fwrite() succeeded");
	}
	else
		PATH_TRACE;

	expval = strlen(t_string);	
	if ((rval = fread((void *)buf, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"r\") on read/write file descriptor, fread() failed - errno %d(%s)", err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
	}
	else
		PATH_TRACE;
		
	buf[rval] = '\0';
	if (strcmp(buf, t_string) != 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"r\") on read/write file descriptor, fread() placed incorrect data in buffer");
		in_rpt("expected buffer contents: %s", t_string);
		in_rpt("actual buffer contents: %s", buf);
	}
	else
		PATH_TRACE;
			
	if (!fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(8); 
	}
		
	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	fd;
	int	err;
	int	fail;
	int	rval;
	int	expval;
	FILE	*fp;
	char	buf1[30], buf2[10], buf3[40];
	int	pathok = 0;
	
	DBUG_ENTER("test4");
	fail = 0;
	
	if ((fd = open (t4_file, O_RDWR|O_CREAT|O_TRUNC, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_CREAT|O_TRUNC) failed - errno %d(%s)",
			t4_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		if (!fail++)
			xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t4_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fdopen (fd, "w")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"w\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strcpy(buf2, t4_string);

	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"w\") on read/write file descriptor, fwrite() failed - errno%d(%s)");
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
	}
	else
		PATH_TRACE;
		
	rewind(fp);
	if ((rval = fread((void *)buf2, (size_t) 1, strlen(buf1), fp)) > 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"w\") on read/write file descriptor, fread() succeeded");
	}
	else
		PATH_TRACE;
			
	(void) fclose(fp);

	if ((fd = open(t4_file, O_RDONLY)) == SYSERROR)	
	{
		err = errno;
		if (!fail++)
			xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d(%s)",	
			t4_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	expval = strlen(buf1) + strlen(buf2);	
	if ((rval = read(fd, buf3, (unsigned) expval)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("failed to read expected data from file after write(%s); fdopen(fd, \"w\"); fwrite(%s) - errno%d(%s)",
			 buf1, buf2, err, errname(err));
		in_rpt("expected return value from read(): %d", expval);
		in_rpt("actual return value from read(): %d", rval);
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t4ret_string) != 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("contents of file after write(%s); fdopen(fd, \"w\"); fwrite(%s) were incorrect",
			buf1, buf2);
		in_rpt("expected contents: %s", t4ret_string);
		in_rpt("actual contents: %s", buf3);
	}
	else
		PATH_TRACE;
		
	if (!fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(9); 
	}
	
	(void) close(fd);
	DBUG_VOID_RETURN;
}

private void
test5()
{
	int	fd;
	int	err;
	int	fail;
	int	rval;
	int 	expval;
	FILE	*fp;
	char	buf1[30], buf2[10], buf3[40];
	int	pathok = 0;
	

	DBUG_ENTER("test5");
	fail = 0;
	
	if ((fd = creat(t5_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s) failed - errno %d(%s)",
			t5_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t5_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	if ((fd = open (t5_file, O_RDWR|O_APPEND, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_APPEND) failed - errno %d(%s)",
			 t5_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	if ((fp = fdopen (fd, "a")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"a\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strcpy(buf2, t5_string);

	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"a\") on read/write file, fwrite() failed - errno%d(%s)",
			 err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) fclose(fp);

	if ((fd = open(t5_file, O_RDONLY)) == SYSERROR)	
	{
		err = errno;
		if (!fail++)
			xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d(%s)",
			t5_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	expval = strlen(buf1) + strlen(buf2);
	if ((rval = read(fd, buf3, (unsigned) expval)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("failed to read expected data from file containing \"%s\", after fdopen(fd, \"a\"); fwrite(%s) - errno%d(%s)",
			buf1, buf2, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t5ret_string) != 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("contents of file containing \"%s\", after fdopen(fd, \"a\"); fwrite(%s) were incorrect",
			buf1, buf2);
		in_rpt("expected contents: %s", t5ret_string);
		in_rpt("actual contents: %s", buf3);
	}
	else
		PATH_TRACE;
			
	if (!fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(9);
	}
	(void) close(fd);
	DBUG_VOID_RETURN;
}

private void
test6()
{
	int	fd;
	int	err;
	int	fail;
	int	rval;
	int	expval;
	FILE	*fp;
	char	buf1[30], buf2[10], buf3[40];
	int	pathok = 0;
	
	DBUG_ENTER("test6");
	fail = 0;
	
	if ((fd = open (t6_file, O_RDWR|O_CREAT|O_TRUNC, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_CREAT|O_TRUNC) failed - errno %d(%s)",
			t6_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t6_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fdopen (fd, "r+")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"r+\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strcpy(buf2, t6_string);
	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"r+\") on read/write file, fwrite() failed - errno %d(%s)", err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	rewind(fp);

	expval = strlen(buf1) + strlen(buf2);	
	if ((rval = fread((void *)buf3, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"r+\"), fread() returned incorrect value");
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t6ret_string) != 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after write(\"%s\"); fdopen(fd, \"r+\"); fwrite(\"%s\"), fread() returned incorrect buffer contents",
			buf1, buf2);
		in_rpt("expected contents: %s", t6ret_string);
		in_rpt("actual contents: %s", buf3);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	if (!fail)
	{
		PATH_TRACE; 
		PATH_XS_RPT(7);
	}
	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test7()
{
	int	fd;
	int	err;
	int	fail;
	int	rval;
	int	expval;
	FILE	*fp;
	char	buf1[30], buf2[10], buf3[40];
	int	pathok = 0;
	
	DBUG_ENTER("test7");
	fail = 0;
	
	if ((fd = open (t7_file, O_RDWR|O_CREAT|O_TRUNC, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_TRUNC|O_CREAT) failed - errno %d(%s)",
			t7_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t7_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fdopen (fd, "w+")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"w+\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		

	(void) strcpy(buf2, t7_string);
	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"w+\") on read/write file, fwrite() failed - errno %d(%s)",
			err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	rewind(fp);

	expval = strlen(buf1) + strlen(buf2);	
	if ((rval = fread((void *)buf3, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"w+\"), fread() returned incorrect value");
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t7ret_string))
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after write(\"%s\"); fdopen(fd, \"w+\"); fwrite(\"%s\"), fread() returned incorrect buffer contents",
			buf1, buf2);
		in_rpt("expected contents: %s", t7ret_string);
		in_rpt("actual contents: %s", buf3);
	}
	else
		PATH_TRACE;
			
	if (!fail)
	{

		PATH_TRACE;
		PATH_XS_RPT(7);
	}
	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test8()
{
	int	fd;
	int	err;
	int	fail;
	int	rval;
	int	expval;
	FILE	*fp;
	char	buf1[30], buf2[10], buf3[40];
	int	pathok = 0;
	
	DBUG_ENTER("test8");
	fail = 0;
	
	if ((fd = creat(t8_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s) failed - errno %d(%s)",
			t8_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t8_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) close(fd);

	if ((fd = open (t8_file, O_RDWR|O_APPEND, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_APPEND) failed - errno %d(%s)",
			t8_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	if ((fp = fdopen (fd, "a+")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"a+\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strcpy(buf2, t8_string);
	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"a+\") on read/write file, fwrite() failed - errno %d(%s)",
			err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	rewind(fp);

	expval = strlen(buf1) + strlen(buf2);	
	if ((rval = fread((void *)buf3, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"a+\"), fread() returned incorrect value");
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t8ret_string) != 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("contents of file containing \"%s\", after fdopen(fd, \"a+\"); fwrite(%s) were incorrect",
			buf1, buf2);
		in_rpt("expected contents: %s", t8ret_string);
		in_rpt("actual contents: %s", buf3);
	}
	else
		PATH_TRACE;
			
	if (!fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(8); 
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test9()
{
	int	fd;
	FILE	*fp;
	int	err;
	long	curpos;
	long	fppos;
	int	pathok = 0;
	
	DBUG_ENTER("test9");
	
	if ((fd = creat(t9_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s) failed - errno %d(%s)",
			t9_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	if (outstr(fd, t_string) != strlen(t_string))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t9_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((curpos = (long) lseek(fd, (off_t)0, SEEK_CUR)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("lseek on file %s failed - errno %d(%s)",
			t9_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fdopen(fd, "w")) == (FILE *) NULL )
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"w\") on valid write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fppos = ftell(fp)) != curpos)
	{
		xx_rpt(FAILURE);
		in_rpt("file position indicator of stream differs from that of underlying file descriptor");
		in_rpt("Expected position: %ld", curpos);
		in_rpt("Observed position: %ld", fppos);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(5); 
	}
	(void) fclose(fp);
	(void) close(fd);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	FILE	*fp;
	int	fd, err;
	char	buf[1];
	int	fail = 0;
	int	pathok = 0;

	DBUG_ENTER("test10");
	
	if ((fd = creat(t10_file, MODEANY)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s) failed - errno %d(%s)",
			t10_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fp = fdopen(fd, "w")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fdopen(fd, \"w\") failed - errno %d (%s)",
			err, errname(err));
		(void) close(fd);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (feof(fp))
		fail |= 01;
	else
		PATH_TRACE;

	if (ferror(fp))
		fail |= 02;
	else
		PATH_TRACE;
	
	(void) fclose(fp);

	if ((fd = open(t10_file, O_RDONLY)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d(%s)",
			t10_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) read(fd, buf, 1);	/* create end of file condition */

	if ((fp = fdopen(fd, "r")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fdopen(fd, \"r\") failed - errno %d (%s)",
			err, errname(err));
		(void) close(fd);
		(void) unlink(t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (feof(fp))
		fail |= 04;
	else
		PATH_TRACE;

	if (ferror(fp))
		fail |= 010;
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 01)
			in_rpt("fdopen(fd, \"w\") did not clear EOF indicator");
		if (fail & 02)
			in_rpt("fdopen(fd, \"w\") did not clear error indicator");
		if (fail & 04)
			in_rpt("fdopen(fd, \"r\") did not clear EOF indicator");
		if (fail & 010)
			in_rpt("fdopen(fd, \"r\") did not clear error indicator");
	}
	else
		PATH_XS_RPT(8);

	(void) fclose(fp);
	(void) unlink(t10_file);

	DBUG_VOID_RETURN;
}

#if	!defined(_XOPEN_SOURCE) || TEST_XPG > 3
private void
test11()
{
	int	fd;
	FILE	*fp;
	int	err;
	int	fail;
	int	rval;
	int	expval;
	char	buf[30];
	int	pathok = 0;
	
	DBUG_ENTER("test11");
	fail = 0;
	
	if ((fd = creat(t11_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s) failed - errno %d(%s)",
			t11_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	if (outstr(fd, t_string) != strlen(t_string))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t11_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) close(fd);

	if ((fd = open (t11_file, O_RDWR, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR) failed - errno %d(%s)",
			t11_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	if ((fp = fdopen(fd, "rb")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"rb\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((rval = fwrite((void *)t_string, (size_t) 1, strlen(t_string), fp)) > 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"rb\") on read/write file descriptor, fwrite() succeeded");
	}
	else
		PATH_TRACE;

	expval = strlen(t_string);	
	if ((rval = fread((void *)buf, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"rb\") on read/write file descriptor, fread() failed - errno %d(%s)", err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
	}
	else
		PATH_TRACE;
		
	buf[rval] = '\0';
	if (strcmp(buf, t_string) != 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"rb\") on read/write file descriptor, fread() placed incorrect data in buffer");
		in_rpt("expected buffer contents: %s", t_string);
		in_rpt("actual buffer contents: %s", buf);
	}
	else
		PATH_TRACE;
			
	if (!fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(8); 
	}
		
	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test12()
{
	int	fd;
	int	err;
	int	fail;
	int	rval;
	int	expval;
	FILE	*fp;
	char	buf1[30], buf2[10], buf3[40];
	int	pathok = 0;
	
	DBUG_ENTER("test12");
	fail = 0;
	
	if ((fd = open (t12_file, O_RDWR|O_CREAT|O_TRUNC, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_CREAT|O_TRUNC) failed - errno %d(%s)",
			t12_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		if (!fail++)
			xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t12_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fdopen (fd, "wb")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"wb\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strcpy(buf2, t12_string);

	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"wb\") on read/write file descriptor, fwrite() failed - errno%d(%s)");
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
	}
	else
		PATH_TRACE;
		
	rewind(fp);
	if ((rval = fread((void *)buf2, (size_t) 1, strlen(buf1), fp)) > 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"wb\") on read/write file descriptor, fread() succeeded");
	}
	else
		PATH_TRACE;
			
	(void) fclose(fp);

	if ((fd = open(t12_file, O_RDONLY)) == SYSERROR)	
	{
		err = errno;
		if (!fail++)
			xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d(%s)",	
			t12_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	expval = strlen(buf1) + strlen(buf2);	
	if ((rval = read(fd, buf3, (unsigned) expval)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("failed to read expected data from file after write(%s); fdopen(fd, \"wb\"); fwrite(%s) - errno%d(%s)",
			 buf1, buf2, err, errname(err));
		in_rpt("expected return value from read(): %d", expval);
		in_rpt("actual return value from read(): %d", rval);
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t12ret_string) != 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("contents of file after write(%s); fdopen(fd, \"wb\"); fwrite(%s) were incorrect",
			buf1, buf2);
		in_rpt("expected contents: %s", t12ret_string);
		in_rpt("actual contents: %s", buf3);
	}
	else
		PATH_TRACE;
		
	if (!fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(9); 
	}
	
	(void) close(fd);
	DBUG_VOID_RETURN;
}

private void
test13()
{
	int	fd;
	int	err;
	int	fail;
	int	rval;
	int 	expval;
	FILE	*fp;
	char	buf1[30], buf2[10], buf3[40];
	int	pathok = 0;
	

	DBUG_ENTER("test13");
	fail = 0;
	
	if ((fd = creat(t13_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s) failed - errno %d(%s)",
			t13_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t13_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	if ((fd = open (t13_file, O_RDWR|O_APPEND, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_APPEND) failed - errno %d(%s)",
			 t13_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	if ((fp = fdopen (fd, "ab")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"ab\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strcpy(buf2, t13_string);

	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"ab\") on read/write file, fwrite() failed - errno%d(%s)",
			 err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) fclose(fp);

	if ((fd = open(t13_file, O_RDONLY)) == SYSERROR)	
	{
		err = errno;
		if (!fail++)
			xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d(%s)",
			t13_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	expval = strlen(buf1) + strlen(buf2);
	if ((rval = read(fd, buf3, (unsigned) expval)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("failed to read expected data from file containing \"%s\", after fdopen(fd, \"ab\"); fwrite(%s) - errno%d(%s)",
			buf1, buf2, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t13ret_string) != 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("contents of file containing \"%s\", after fdopen(fd, \"ab\"); fwrite(%s) were incorrect",
			buf1, buf2);
		in_rpt("expected contents: %s", t13ret_string);
		in_rpt("actual contents: %s", buf3);
	}
	else
		PATH_TRACE;
			
	if (!fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(9);
	}
	(void) close(fd);
	DBUG_VOID_RETURN;
}

private void
test14()
{
	int	fd;
	int	err;
	int	fail;
	int	rval;
	int	expval;
	FILE	*fp;
	char	buf1[30], buf2[10], buf3[40];
	int	pathok = 0;
	
	DBUG_ENTER("test14");
	fail = 0;
	
	if ((fd = open (t14_file, O_RDWR|O_CREAT|O_TRUNC, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_CREAT|O_TRUNC) failed - errno %d(%s)",
			t14_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t14_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fdopen (fd, "rb+")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"rb+\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strcpy(buf2, t14_string);
	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"rb+\") on read/write file, fwrite() failed - errno %d(%s)", err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	rewind(fp);

	expval = strlen(buf1) + strlen(buf2);	
	if ((rval = fread((void *)buf3, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"rb+\"), fread() returned incorrect value");
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t14ret_string) != 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after write(\"%s\"); fdopen(fd, \"rb+\"); fwrite(\"%s\"), fread() returned incorrect buffer contents",
			buf1, buf2);
		in_rpt("expected contents: %s", t14ret_string);
		in_rpt("actual contents: %s", buf3);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) fclose(fp);
	(void) close(fd);

	if ((fd = open (t14_file, O_RDWR|O_CREAT|O_TRUNC, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_CREAT|O_TRUNC) failed - errno %d(%s)",
			t14_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t14_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fdopen (fd, "r+b")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"r+b\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strcpy(buf2, t14_string);
	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"r+b\") on read/write file, fwrite() failed - errno %d(%s)", err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	rewind(fp);

	expval = strlen(buf1) + strlen(buf2);	
	if ((rval = fread((void *)buf3, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"r+b\"), fread() returned incorrect value");
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t14ret_string) != 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after write(\"%s\"); fdopen(fd, \"r+b\"); fwrite(\"%s\"), fread() returned incorrect buffer contents",
			buf1, buf2);
		in_rpt("expected contents: %s", t14ret_string);
		in_rpt("actual contents: %s", buf3);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_XS_RPT(11);
			
	(void) fclose(fp);


	DBUG_VOID_RETURN;
}

private void
test15()
{
	int	fd;
	int	err;
	int	fail;
	int	rval;
	int	expval;
	FILE	*fp;
	char	buf1[30], buf2[10], buf3[40];
	int	pathok = 0;
	
	DBUG_ENTER("test15");
	fail = 0;
	
	if ((fd = open (t15_file, O_RDWR|O_CREAT|O_TRUNC, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_TRUNC|O_CREAT) failed - errno %d(%s)",
			t15_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t15_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fdopen (fd, "wb+")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"wb+\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		

	(void) strcpy(buf2, t15_string);
	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"wb+\") on read/write file, fwrite() failed - errno %d(%s)",
			err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	rewind(fp);

	expval = strlen(buf1) + strlen(buf2);	
	if ((rval = fread((void *)buf3, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"wb+\"), fread() returned incorrect value");
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t15ret_string))
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after write(\"%s\"); fdopen(fd, \"wb+\"); fwrite(\"%s\"), fread() returned incorrect buffer contents",
			buf1, buf2);
		in_rpt("expected contents: %s", t15ret_string);
		in_rpt("actual contents: %s", buf3);
	}
	else
		PATH_TRACE;
	(void) fclose(fp);
	(void) close(fd);
			
	if ((fd = open (t15_file, O_RDWR|O_CREAT|O_TRUNC, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_TRUNC|O_CREAT) failed - errno %d(%s)",
			t15_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t15_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fp = fdopen (fd, "w+b")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"w+b\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		

	(void) strcpy(buf2, t15_string);
	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"w+b\") on read/write file, fwrite() failed - errno %d(%s)",
			err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	rewind(fp);

	expval = strlen(buf1) + strlen(buf2);	
	if ((rval = fread((void *)buf3, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"w+b\"), fread() returned incorrect value");
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t15ret_string))
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after write(\"%s\"); fdopen(fd, \"w+b\"); fwrite(\"%s\"), fread() returned incorrect buffer contents",
			buf1, buf2);
		in_rpt("expected contents: %s", t15ret_string);
		in_rpt("actual contents: %s", buf3);
	}
	else
		PATH_XS_RPT(11);

	(void) fclose(fp);
	(void) close(fd);
	DBUG_VOID_RETURN;
}

private void
test16()
{
	int	fd;
	int	err;
	int	fail;
	int	rval;
	int	expval;
	FILE	*fp;
	char	buf1[30], buf2[10], buf3[40];
	int	pathok = 0;
	
	DBUG_ENTER("test16");
	fail = 0;
	
	if ((fd = creat(t16_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s) failed - errno %d(%s)",
			t16_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t16_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) close(fd);

	if ((fd = open (t16_file, O_RDWR|O_APPEND, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_APPEND) failed - errno %d(%s)",
			t16_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	if ((fp = fdopen (fd, "ab+")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"ab+\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strcpy(buf2, t16_string);
	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"ab+\") on read/write file, fwrite() failed - errno %d(%s)",
			err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	rewind(fp);

	expval = strlen(buf1) + strlen(buf2);	
	if ((rval = fread((void *)buf3, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"ab+\"), fread() returned incorrect value");
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t16ret_string) != 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("contents of file containing \"%s\", after fdopen(fd, \"ab+\"); fwrite(%s) were incorrect",
			buf1, buf2);
		in_rpt("expected contents: %s", t16ret_string);
		in_rpt("actual contents: %s", buf3);
	}
	else
		PATH_TRACE;
			
	(void)close(fd);
	(void)fclose(fp);

	if ((fd = creat(t16_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s) failed - errno %d(%s)",
			t16_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	(void) strcpy(buf1, t_string);

	if (write(fd, buf1, (unsigned) strlen(buf1)) != strlen(buf1))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to file %s failed - errno %d(%s)",
			t16_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) close(fd);

	if ((fd = open (t16_file, O_RDWR|O_APPEND, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_APPEND) failed - errno %d(%s)",
			t16_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	if ((fp = fdopen (fd, "a+b")) == (FILE *) NULL )
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("fdopen(fd, \"a+b\") on valid read/write file descriptor failed - errno %d(%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strcpy(buf2, t16_string);
	expval = strlen(buf2);
	if ((rval = fwrite((void *)buf2, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		err = errno;
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"a+b\") on read/write file, fwrite() failed - errno %d(%s)",
			err, errname(err));
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	rewind(fp);

	expval = strlen(buf1) + strlen(buf2);	
	if ((rval = fread((void *)buf3, (size_t) 1, (size_t)expval, fp)) != expval)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("after fdopen(fd, \"a+b\"), fread() returned incorrect value");
		in_rpt("expected return value: %d", expval);
		in_rpt("actual return value: %d", rval);
		(void) fclose(fp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	buf3[rval] = '\0';
	if(strcmp(buf3, t16ret_string) != 0)
	{
		if (!fail++)
			xx_rpt(FAILURE);
		in_rpt("contents of file containing \"%s\", after fdopen(fd, \"a+b\"); fwrite(%s) were incorrect",
			buf1, buf2);
		in_rpt("expected contents: %s", t16ret_string);
		in_rpt("actual contents: %s", buf3);
	}
	else
		PATH_XS_RPT(13); 

	(void) fclose(fp);
	(void) close(fd);
	DBUG_VOID_RETURN;
}
#else
private	void
test11()
{
	DBUG_ENTER("test11");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
private	void
test12()
{
	DBUG_ENTER("test12");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
private	void
test13()
{
	DBUG_ENTER("test13");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
private	void
test14()
{
	DBUG_ENTER("test14");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
private	void
test15()
{
	DBUG_ENTER("test15");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
private	void
test16()
{
	DBUG_ENTER("test16");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#endif

private void
clean_tests()
{
	(void) unlink(t1_file);
	(void) unlink(t2_file);
	(void) unlink(t3_file);
	(void) unlink(t4_file);
	(void) unlink(t5_file);
	(void) unlink(t6_file);
	(void) unlink(t7_file);
	(void) unlink(t8_file);
	(void) unlink(t9_file);
	(void) unlink(t10_file);
	(void) unlink(t11_file);
	(void) unlink(t12_file);
	(void) unlink(t13_file);
	(void) unlink(t14_file);
	(void) unlink(t15_file);
	(void) unlink(t16_file);
}
