/*
*      SCCS:  @(#)  POSIX.os/Csupport/fileno/fileno.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/Csupport/fileno - T.fileno Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fileno

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fileno() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Dec  3 11:57:24 WET 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Modified xx_rpt(), xs_rpt()

		Linda Taylor, Hoskyns.	19th August, 1988.
		Tests were removed from T.ferror

		Anthony Lee, 25 July 1990
		Added resilience code.
		
		Geoff Clare, 19 July 1991
		Added tests for state of stdin/out/err at entry to main()
	
************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<string.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<fcntl.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fileno
#endif


#define	NO_TESTS	5		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	**environ;
extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	*errname();

int	no_tests = NO_TESTS;

private	char	*t_string =	"test string";
private	char	*t_file =	"./fileno-t";
private	char	*t4_file =	"./fileno-t.4";
private	char	*prog4 =	"../fileno_t4";
struct t1_str {
	int	flag;
	int	err;
	int	rval;
};

private	void	comp_prep();
private	void	comp_fin();
private	void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	ch_t4();
private void	test5();

public	void	(*setfunc) () = comp_prep;
public	void	(*clnfunc) () = comp_fin;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	NULL, 0
};

private	void
comp_prep()
{
	int		fd;
	int		bad = 0;
	static char	msg[1024];

	DBUG_ENTER("comp_prep");
	if ((fd = creat(t_file, MODEANY)) == SYSERROR)
	{
		bad = 1;
		(void) sprintf(msg, "creat(%s, %#o) failed in preperations - errno %d(%s)",
			t_file, MODEANY, errno, errname(errno));
	}
	else if (write(fd, t_string, (unsigned) strlen(t_string)) != strlen(t_string))
	{
		bad = 1;
		(void) sprintf(msg, "write() failed in preperations");
	}
	if (fd != SYSERROR)
		(void) close(fd);
	if (bad)
		for (fd = 0 ; fd < NO_TESTS ; fd++)
			(void) tet_delete(fd, msg);
	DBUG_VOID_RETURN;
}

private void
comp_fin()
{
	DBUG_ENTER("comp_fin");

	(void) unlink(t_file);
	(void) unlink(t4_file);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	FILE		*fp;
	int		fd;
	int		err;
	struct stat	statb1;
	struct stat	statb2;
	int		pathok = 0;
	

	DBUG_ENTER("test1");
	errno = NOERROR;
	if ((fp = fopen(t_file, "r")) == (FILE *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fopen(%s, \"r\") failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(t_file, &statb1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(%s) failed - errno %d(%s)",
			t_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	fd = fileno(fp);
	if (fstat(fd, &statb2) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fstat(fileno(fd)) failed - errno %d(%s)",
			err, errname(err));
	}
	else if (statb1.st_ino != statb2.st_ino
		|| statb1.st_dev != statb2.st_dev)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fileno(stream) does not seem to return the correct file descriptor");
		in_rpt("dev/inode of file = 0x%lx/%ld",
			(long)statb1.st_dev, (long)statb1.st_ino);
		in_rpt("dev/inode of file descriptor returned by fileno() = 0x%lx/%ld",
			(long)statb2.st_dev, (long)statb2.st_ino);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(3); 
	}

	(void) fclose(fp);
	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	rval;
	int	testfail = 0;
	int	pathok = 0;
	
	DBUG_ENTER("test2");

	if ((rval = fileno(stdin)) != STDIN_FILENO)
	{
		xx_rpt(FAILURE);
		in_rpt("fileno(stdin) did not match STDIN_FILENO");
		in_rpt("Expected return: %d", STDIN_FILENO);
		in_rpt("Observed return: %d", rval);
		testfail++;
	}
	else
		PATH_TRACE;
		
	if ((rval = fileno(stdout)) != STDOUT_FILENO)
	{
		if (!testfail)
			xx_rpt(FAILURE);
		in_rpt("fileno(stdout) did not match STDOUT_FILENO");
		in_rpt("Expected return: %d", STDOUT_FILENO);
		in_rpt("Observed return: %d", rval);
		testfail++;
	}
	else
		PATH_TRACE;
		
	if ((rval = fileno(stderr)) != STDERR_FILENO)
	{
		if (!testfail)
			xx_rpt(FAILURE);
		in_rpt("fileno(stderr) did not match STDERR_FILENO");
		in_rpt("Expected return: %d", STDERR_FILENO);
		in_rpt("Observed return: %d", rval);
		testfail++;
	}
	else
		PATH_TRACE;
		
	if (!testfail)
	{
		PATH_TRACE; 
		PATH_XS_RPT(4); 
	}

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* stdin is readable, stdout and stderr are writable */
	/* relies on stdin/out/err not being touched by the TCM */

	int	flags;
	int	testfail = 0;
	int	pathok = 0;
	
	DBUG_ENTER("test3");

	if ((flags = fcntl(fileno(stdin), F_GETFL)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("fcntl(fileno(stdin), F_GETFL) failed - errno %d (%s)",
			errno, errname(errno));
	}
	else if ((flags &= O_ACCMODE) != O_RDONLY && flags != O_RDWR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stdin is not readable: access mode 0x%x", flags);
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(stdout), F_GETFL)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("fcntl(fileno(stdout), F_GETFL) failed - errno %d (%s)",
			errno, errname(errno));
	}
	else if ((flags &= O_ACCMODE) != O_WRONLY && flags != O_RDWR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stdout is not writable: access mode 0x%x", flags);
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fileno(stderr), F_GETFL)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("fcntl(fileno(stderr), F_GETFL) failed - errno %d (%s)",
			errno, errname(errno));
	}
	else if ((flags &= O_ACCMODE) != O_WRONLY && flags != O_RDWR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stderr is not writable: access mode 0x%x", flags);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* test position indicators of stdin/out/err */

	DBUG_ENTER("test4");

	(void) cppair(ch_t4, NULLFN, WAITTIME, 0);

	(void) unlink(t4_file);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	fd, err;
	char	*args[5];
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	for (fd = 0; fd <= 2; fd++)
	{
		(void) close(fd);
		if (open(t4_file, O_RDWR|O_CREAT, MODEANY) != fd)
		{
			xx_rpt(DELETION);
			in_rpt("could not reassign fd %d to file", fd);
			DBUG_VOID_RETURN;
		}
		else if (lseek(fd, (off_t)(10*fd + 5), SEEK_SET) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("lseek() failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	args[0] = prog4;
	args[1] = (pathok == 3 ? "Y" : "N");
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(prog4, args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		prog4, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* error and end of file indicators for stdin/out/err are clear */
	/* relies on stdin/out/err not being touched by the TCM */

	int	testfail = 0;
	int	pathok = 0;
	
	DBUG_ENTER("test5");

	if (ferror(stdin))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("ferror(stdin) is not clear");
	}
	else
		PATH_TRACE;

	if (feof(stdin))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("feof(stdin) is not clear");
	}
	else
		PATH_TRACE;

	if (ferror(stdout))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("ferror(stdout) is not clear");
	}
	else
		PATH_TRACE;

	if (feof(stdout))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("feof(stdout) is not clear");
	}
	else
		PATH_TRACE;

	if (ferror(stderr))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("ferror(stderr) is not clear");
	}
	else
		PATH_TRACE;

	if (feof(stderr))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("feof(stderr) is not clear");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}
