/*
*      SCCS:  @(#)  POSIX.os/Csupport/fileno/fileno_t4.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/Csupport/fileno - fileno_t4 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		fileno_t4 - used by T.fileno in test4()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int tet_main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.fileno in test4().
	  usage:
		fileno_t4 parentok vg_ident vg_dbug
	
	  where parentok is "Y" if path tracing was correct in parent.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	19 July 1991
MODIFICATIONS:

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <dbug.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef fileno
#endif


public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	*errname();
extern	void	set_pname();

private void	subtest4();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	set_pname(argv[0]);

	if (argc < 2 || argc > 4)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		exit(OK);
	}
	if (argc >= 4)
		vg_dbug = argv[3];
	if (argc >= 3)
		vg_ident = argv[2];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	subtest4(argv[1]);

	exit(OK);
	/*NOTREACHED*/
}


private void
subtest4(parentok)
char *parentok;
{
	long	ret, expret;
	int 	err;
	int	testfail = 0;
	int	pathok = 0;

	DBUG_ENTER("subtest4");

	if (parentok[0] == 'Y')
		PATH_TRACE;

	if ((expret = lseek(fileno(stdin), (off_t)0, SEEK_CUR)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("lseek(fileno(stdin), 0, SEEK_CUR) failed - errno %d (%s)",
			errno, errname(errno));
	}
	else if ((ret = ftell(stdin)) != expret)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("ftell(stdin) returned %ld, expected %ld",
			ret, expret);
		if (ret == SYSERROR)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;
	
	DBUG_PRINTF("subtest4", "position of stdin: %ld", expret);

	if ((expret = lseek(fileno(stdout), (off_t)0, SEEK_CUR)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("lseek(fileno(stdout), 0, SEEK_CUR) failed - errno %d (%s)",
			errno, errname(errno));
	}
	else if ((ret = ftell(stdout)) != expret)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("ftell(stdout) returned %ld, expected %ld",
			ret, expret);
		if (ret == SYSERROR)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	DBUG_PRINTF("subtest4", "position of stdout: %ld", expret);

	if ((expret = lseek(fileno(stderr), (off_t)0, SEEK_CUR)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("lseek(fileno(stderr), 0, SEEK_CUR) failed - errno %d (%s)",
			errno, errname(errno));
	}
	else if ((ret = ftell(stderr)) != expret)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("ftell(stderr) returned %ld, expected %ld",
			ret, expret);
		if (ret == SYSERROR)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	DBUG_PRINTF("subtest4", "position of stderr: %ld", expret);

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}
