/*
*      SCCS:  @(#)  POSIX.os/Csupport/siglongjmp/siglongjmp.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/Csupport/siglongjmp - T.siglongjmp Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.siglongjmp

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the siglongjmp() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Thu Oct 23 16:52:01 BST 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Dave Cater Mon Sep 18 14:34:05 BST 1989
			Use VNOJSIG not VNUMSIG if no _POSIX_JOB_CONTROL
		Dave Cater Tue Sep 19 11:52:52 BST 1989
			Call sigprocmask() to obtain actual mask set
		J.A.Nave	21/06/90
			Rearranged tests to verify new assertions
			(Major open heart surgry!)

		Anthony Lee, 25 July 1990
			Addded resilience code.
		

		Stuart Boutell, 26 April 1991
			Fixed resilience code problem. Make pathok
		non-automatic so that it can be used before & after
		a siglongjmp.

		Geoff Clare, 22 June 1994
			Ensure sigsetjmp() is used in line with constraints.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<setjmp.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef siglongjmp
#endif


#define 	NO_TESTS	3	/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();
extern	int	cppair();

int	no_tests = NO_TESTS;
private	int	jumpok;
private	int	pathok;		/* Make pathok non-automatic */

private	sigjmp_buf	env;
private	int	testfail;
private int	func1_glob;
private int	func2_glob;
private sigset_t t1_aset;
private sigset_t t1_bset;
private sigset_t t3_aset;
private sigset_t t3_bset;

private void	sig_mask();
private	void	cmp_mask();
private void	test1();
private void	ch_t1();
private void	test2();
private void	ch_t2();
private void	test3();
private void	ch_t3();
private void	func1();
private void	func2();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private	void
test1()
{
	DBUG_ENTER("test1");

	pathok = 0;

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	static int jmpfail = 0;

	DBUG_ENTER("ch_t1");

	jumpok = 0;

	sig_mask(&t1_aset, SIGQUIT);

	if (sigprocmask(SIG_SETMASK, &t1_aset, (sigset_t *) NULL) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (sigprocmask(SIG_SETMASK, (sigset_t *) NULL, &t1_aset) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask() failed to obtain signal mask - error value: %d(%s)", errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	func1_glob = 2;

	if (sigsetjmp(env, 1) != 0)	/* `second return' */
	{
		jmpfail = 0;
		JUMP_TRACE;

		if (sigprocmask(SIG_SETMASK, (sigset_t *) NULL, &t1_bset) != 0)
		{
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		cmp_mask(&t1_aset, &t1_bset);
	}
	else				/* `first return' */
	{
		sig_mask(&t1_bset, SIGINT);

		if (sigprocmask(SIG_SETMASK, &t1_bset, (sigset_t *) NULL) != 0)
		{
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (jmpfail == 0)
		{
			jmpfail = 1;
			func1();
		}
	}

	if (jmpfail)
	{
		if (!testfail)
			xx_rpt(FAILURE);
		in_rpt("siglongjmp() called with value 5");
		in_rpt("sigsetjmp() returned value 0");
		testfail++;
	}
	else
		PATH_TRACE;


	jmpfail = 0;

	if (sigsetjmp(env, 1) != 0)	/* `second return' */
	{
		jmpfail = 0;
		JUMP_TRACE;

		if (sigprocmask(SIG_SETMASK, (sigset_t *) NULL, &t1_bset) != 0)
		{
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		cmp_mask(&t1_aset, &t1_bset);
	}
	else				/* `first return' */
	{
		sig_mask(&t1_bset, SIGINT);
		
		if (sigprocmask(SIG_SETMASK, &t1_bset, (sigset_t *) NULL) != 0)
		{
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (jmpfail == 0)
		{
			jmpfail = 1;
			func2();
		}
	}

	if (jmpfail)
	{
		if (!testfail)
			xx_rpt(FAILURE);
		in_rpt("siglongjmp() called with value 0");
		in_rpt("sigsetjmp() returned value 0");
		testfail++;
	}
	else
		PATH_TRACE;

	if (jumpok == 4)
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(14);

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	DBUG_ENTER("test2");

	pathok = 0;

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	a;			

	DBUG_ENTER("ch_t2");

	jumpok = 0;

	a = 1;

	/* place &a in DBUG_PRINTF to ensure that a is placed on the stack */

	DBUG_PRINTF("ch_t2", "automatic variable address = %p", (void *)&a);

	func1_glob = 2;
	func2_glob = 2;

	if (sigsetjmp(env, 1) != 0)	/* `second return' */
	{
		if (a != 1 || func1_glob != 5)
		{
			if (!testfail)
				xx_rpt(FAILURE);
			in_rpt("Automatic variable before sigsetjmp call, a = 1");
			in_rpt("Automatic variable after siglongjmp call, a = %d", a);
			in_rpt("Global variable before sigsetjmp call, func1_glob = 2");
			in_rpt("Global variable after siglongjmp call, func1_glob = %d", func1_glob);
			testfail++;
		}
		else
			PATH_TRACE;

	}
	else				/* `first return' */
	{
		func1();

	}

	if (sigsetjmp(env, 1) != 0)	/* `second return' */
	{
		if (a != 1 || func2_glob != 3)
		{
			if (!testfail)
				xx_rpt(FAILURE);
			in_rpt("Automatic variable before sigsetjmp call, a = 1");
			in_rpt("Automatic variable after siglongjmp call, a = %d", a);
			in_rpt("Global variable before sigsetjmp call, func2_glob = 2");
			in_rpt("Global variable after siglongjmp call, func2_glob = %d", func2_glob);
			testfail++;
		}
		else
			PATH_TRACE;

	}
	else				/* `first return' */
	{
		func2();

	}

	if (jumpok == 2)
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private	void
test3()
{
	DBUG_ENTER("test3");

	pathok = 0;

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	DBUG_ENTER("ch_t3");

	jumpok = 0;

	sig_mask(&t3_aset, SIGQUIT);

	if (sigprocmask(SIG_SETMASK, &t3_aset, (sigset_t *) NULL) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (sigprocmask(SIG_SETMASK, (sigset_t *) NULL, &t3_aset) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask() failed to obtain signal mask - error value: %d(%s)", errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	if (sigsetjmp(env, 0) != 0)	/* `second return' */
	{
		if (sigprocmask(SIG_SETMASK, (sigset_t *) NULL, &t3_bset) != 0)
		{
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		cmp_mask(&t3_aset, &t3_bset);
	}
	else				/* `first return' */
	{
		sig_mask(&t3_aset, SIGINT);

		if (sigprocmask(SIG_SETMASK, &t3_aset, (sigset_t *) NULL) != 0)
		{
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		func1();
		if(!testfail)
			xx_rpt(FAILURE);
		in_rpt("func1() returned - implies that siglongjmp returned");
		DBUG_VOID_RETURN;

	}

	if (sigsetjmp(env, 0) != 0)	/* `second return' */
	{
		if (sigprocmask(SIG_SETMASK, (sigset_t *) NULL, &t3_bset) != 0)
		{
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		cmp_mask(&t3_aset, &t3_bset);
	}
	else				/* `first return' */
	{
		sig_mask(&t3_aset, SIGQUIT);

		if (sigprocmask(SIG_SETMASK, &t3_aset, (sigset_t *) NULL) != 0)
		{
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		func2();

		if(!testfail)
			xx_rpt(FAILURE);
		in_rpt("func2() returned - implies that siglongjmp returned");
		DBUG_VOID_RETURN;
	}

	if (jumpok == 2)
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(12);

	DBUG_VOID_RETURN;
}


private	void
func1()
{
	DBUG_ENTER("func1");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	func1_glob = 5;
	siglongjmp(env, 5);

	/*NOTREACHED*/
	DBUG_VOID_RETURN;
}

private	void
func2()
{
	DBUG_ENTER("func2");

	JUMP_TRACE;	/*	to prove that we have been called	*/

	func2_glob = 3;
	siglongjmp(env, 0);

	/*NOTREACHED*/
	DBUG_VOID_RETURN;
}

private void
cmp_mask(aset, acmpset)
sigset_t	*aset;		/* signal mask */
sigset_t	*acmpset;	/* signal mask  for comparison */
{
	int	fail = 0;
	int	i;
	int	signum;
	char	*name;
	int	ret;
	int	nsigs;

	DBUG_ENTER("cmp_mask");

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		nsigs = VNOJSIG;
	} else {
		nsigs = VNUMSIG;
	}

	for(i = 1; i < nsigs; i++) {
		if (i == 1)
			PATH_TRACE;

		signum = sig_array[i];
		name = sig_name[i];
		if(sigismember(acmpset, signum)) {
			if(!sigismember(aset, signum)) {
				if(!testfail)
					xx_rpt(FAILURE);
				in_rpt("signal mask includes signal %d %s",
								signum, name);
				testfail++;
				fail++;
			}
		} else {
			if(sigismember(aset, signum)) {
				if(!testfail)
					xx_rpt(FAILURE);
				in_rpt("signal mask omits signal %d %s",
								signum, name);
				testfail++;
				fail++;
			}
		}
	}
	if(fail) {
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private void
sig_mask(set, signum)
int	signum;		/*  a signal to put in mask */
sigset_t *set;		/* signal mask */
{

	DBUG_ENTER("sig_mask");

	/* Return address of signal mask */
	(void) sigemptyset(set);
	(void) sigaddset(set, signum);

	PATH_TRACE;	/*	to prove that we have been called	*/

	DBUG_VOID_RETURN;
}
