/*
*      SCCS:  @(#)  POSIX.os/Csupport/sigsetjmp/sigsetjmp.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/Csupport/sigsetjmp - T.sigsetjmp Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.sigsetjmp

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the sigsetjmp() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Thu Oct 23 16:52:01 BST 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Dave Cater Mon Sep 18 14:34:05 BST 1989
			Use VNOJSIG not VNUMSIG if no _POSIX_JOB_CONTROL
		Dave Cater Tue Sep 19 11:52:52 BST 1989
			Call sigprocmask() to obtain actual mask set

		Anthony Lee, 25 July 1990
			Added resilience code.
		
		Geoff Clare, 29 Aug 1991
			Make pathok variables static, so not restored by jump.

		Geoff Clare, 22 June 1994
			Ensure sigsetjmp() is used in line with constraints.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<setjmp.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef no_functions
#endif


#define 	NO_TESTS	4		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)
#define		T4_VAL		123

extern	char	*errname();
extern	int	cppair();

int	no_tests = NO_TESTS;

private	sigjmp_buf	env;
private	int	testfail;
private int	t1_glob;
private int	jumpok;
private sigset_t t2_aset;
private sigset_t t2_bset;

private void	sig_mask();
private	void	cmp_mask();
private void	test1();
private void	ch_t1();
private void	test2();
private void	ch_t2();
private	void	test3();
private	void	test4();
private void	func1();
private void	func2();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private	void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{

	int	a;
	static int	pathok;
	
	DBUG_ENTER("ch_t1");

	pathok = 0;
	a = 1;
	t1_glob = 2;

	if (sigsetjmp(env, 0) != 0)	/* `second return' */
	{
		if (a != 1 || t1_glob != 5)
		{
			xx_rpt(FAILURE);
			in_rpt("Automatic variable before sigsetjmp call, a = 1");
			in_rpt("Automatic variable after siglongjmp call, a = %d", a);
			in_rpt("Global variable before sigsetjmp call, t1_glob = 2");
			in_rpt("Global variable after siglongjmp call, t1_glob = %d", t1_glob);
		}
		else
		{
			if (jumpok == 1)
				PATH_TRACE;
			PATH_XS_RPT(1); 
		}
	}
	else				/* `first return' */
	{
		DBUG_PRINTF("ch_t1","t1_glob = %d",t1_glob);
		func1();
	}

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{

	int	a;
	static int	pathok;
	
	DBUG_ENTER("ch_t2");

	pathok = 0;
	jumpok = 0;
	
	sig_mask(&t2_aset, VSIGQUIT);
	if (sigprocmask(SIG_SETMASK, &t2_aset, (sigset_t *) NULL) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (sigprocmask(SIG_SETMASK, (sigset_t *) NULL, &t2_aset) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask() failed to obtain signal mask - error value: %d(%s)", errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	a = 1;
	t1_glob = 2;

	if (sigsetjmp(env, 1) != 0)	/* `second return' */
	{
		if (sigprocmask(SIG_SETMASK, (sigset_t *) NULL, &t2_bset) != 0)
		{
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
			
		if (a != 1 || t1_glob != 5)
		{
			if (!testfail)
				xx_rpt(FAILURE);
			in_rpt("Automatic variable before sigsetjmp call, a = 1");
			in_rpt("Automatic variable after siglongjmp call, a = %d", a);
			in_rpt("Global variable before sigsetjmp call, t1_glob = 2");
			in_rpt("Global variable after siglongjmp call, t1_glob = %d", t1_glob);
			testfail++;
		}
		else
			PATH_TRACE;

		globok = 0;
		cmp_mask(&t2_aset, &t2_bset);
		PATH_FUNC_TRACE;
		
		if (!testfail)
		{
			if (jumpok  == 1)
				PATH_TRACE; 
			PATH_XS_RPT(7);
		}
	}
	else				/* `first return' */
	{
		sig_mask(&t2_bset, VSIGINT);
		if (sigprocmask(SIG_SETMASK, &t2_bset, (sigset_t *) NULL) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("sigprocmask() failed to set signal mask - error value: %d(%s)", errno, errname(errno));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		func1();
	}

	DBUG_VOID_RETURN;
}

private void
test3()
{
	static int	pathok;
	
	DBUG_ENTER("test3");

	pathok = 0;

	if (sigsetjmp(env, 0) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("sigsetjmp returned non-zero, expected zero");
	}
	else
	{
		PATH_TRACE; 
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}

private void
test4()
{
	static int	func_called = 0;
	static int	pathok;
	

	DBUG_ENTER("test4");

	pathok = 0;

	if (sigsetjmp(env, 0) != T4_VAL)
	{
		if (func_called)
		{
			xx_rpt(FAILURE);
			in_rpt("sigsetjmp() did not return value passed by siglongjmp()");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		func_called = 1;
		func1();
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private	void
func1()
{

	DBUG_ENTER("func1");

	t1_glob = 5;
	func2();

	DBUG_VOID_RETURN;
}


private	void
func2()
{

	DBUG_ENTER("func2");

	JUMP_TRACE;		/* Just to prove that we have been here */
	siglongjmp(env, T4_VAL);

	DBUG_VOID_RETURN;
}

private void
cmp_mask(aset, acmpset)
sigset_t	*aset;		/* signal mask */
sigset_t	*acmpset;	/* signal mask  for comparison */
{
	int	fail = 0;
	int	i;
	int	signum;
	char	*name;
	int	ret;
	int	nsigs;
	static int	pathok;

	DBUG_ENTER("cmp_mask");

	pathok = 0;
	globok = 0;

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		nsigs = VNOJSIG;
	} else {
		nsigs = VNUMSIG;
	}

	for(i = 1; i < nsigs; i++) {
		signum = sig_array[i];
		name = sig_name[i];
		if(sigismember(acmpset, signum)) {
			if(!sigismember(aset, signum)) {
				if(!testfail)
					xx_rpt(FAILURE);
				in_rpt("signal mask includes signal %d %s",
								signum, name);
				testfail++;
				fail++;
			} else {
				PATH_TRACE; }
		} else {
			if(sigismember(aset, signum)) {
				if(!testfail)
					xx_rpt(FAILURE);
				in_rpt("signal mask omits signal %d %s",
								signum, name);
				testfail++;
				fail++;
			} else {
				PATH_TRACE; }
		}
	}

	if(fail)
	{
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_FUNC_RPT(nsigs-1); 
		DBUG_VOID_RETURN;
	}
	
}

private void
sig_mask(set, signum)
int	signum;		/*  a signal to put in mask */
sigset_t *set;		/* signal mask */
{
	DBUG_ENTER("sig_mask");

	/* Return address of signal mask */
	(void) sigemptyset(set);
	(void) sigaddset(set, signum);

	DBUG_VOID_RETURN;
}
