/*
*      SCCS:  @(#)  POSIX.os/Csupport/tzset_X/tzset_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/Csupport/tzset_X - T.tzset_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.tzset_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the tzset() system call - X/Open extensions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	23 Nov 1989
		(Split out from tzset() testset)

MODIFIED:

		Anthony Lee, 26 July 1990
			Addded resilience code.
		

************************************************************************/
#include	<std.h>
#include	<string.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<stdlib.h>
#include	<time.h>
#include	<limits.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef tzset
#undef timezone
#undef daylight
#endif


#define 	NO_TESTS	2		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

extern	int	cppair();
extern	char	*psl_to_asc();
extern	char	**environ;

int	no_tests = NO_TESTS;

private int	testfail;

private void	test1();
private	void	test2();
private void	ch_t1();
private void	ch_t2();

private int	set_tz();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};


private	void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	testfail = 0;

	globok = 0;
	if (set_tz("TZA-9") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	tzset();
	if (timezone != -9*60*60L)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("with TZ=TZA-9 tzset() did not set timezone correctly");
		in_rpt("value of timezone was %ld, expected %ld",
			timezone, -9*60*60L);
	}
	else
		PATH_TRACE;

	globok = 0;
	if (set_tz("JKL3:10PNM4:4") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	tzset();
	if (timezone != 11400)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("with TZ=JKL3:10PNM4:40 tzset() did not set timezone correctly");
		in_rpt("value of timezone was %ld, expected 11400", timezone);
	}
	else
		PATH_TRACE;
	
	if (testfail == 0)
	{
		PATH_TRACE;
		PATH_XS_RPT(5); 
	}

	DBUG_VOID_RETURN;
}

private	void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	globok = 0;
	if (set_tz("ABC-4:30:20") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	tzset();
	if ( daylight != 0 )
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("with TZ=ABC-4:30:20 tzset() did not set daylight correctly");
		in_rpt("value of daylight was %d, expected zero", daylight);
	}
	else
		PATH_TRACE;

	globok = 0;	
	if (set_tz("JKL3:10PNM4:40") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	tzset();
	if ( daylight == 0 )
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("with TZ=JKL3:10PNM4:40 tzset() did not set daylight correctly");
		in_rpt("value of daylight was %d, expected non-zero", daylight);
	}
	else
		PATH_TRACE;
			
	if (testfail == 0)
	{
		PATH_TRACE; 
		PATH_XS_RPT(5); 
	}

	DBUG_VOID_RETURN;
}

private int
set_tz(tzval)
char *tzval;
{
	char	*tz;
	static char	buf[256];
	int	pathok = 0;
	
	DBUG_ENTER("set_tz");

	(void) strcpy(buf, "TZ=");
	(void) strncat(buf, tzval, (size_t)250);
	(void) vputenv(&environ, buf);
	tz = getenv("TZ");
	if (tz == NULL || strcmp(tz, tzval) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not put \"TZ=%s\" in environment",
			psl_to_asc(tzval, 0));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1); 
		
	DBUG_RETURN(0);
}
