/*
*      SCCS:  @(#)  POSIX.os/dataform/cpio/cpio.c	Rel 4.4.4	    (07/19/99)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/dataform/cpio - T.cpio Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.cpio

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the IEEE Std. 1003.1-1988 "Extended cpio format"

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu May 25 10:45:42 BST 1989
MODIFICATIONS:
		Stuart Boutell, 12 Nov 1990
			uids() return value check added

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		Calum Laing, Mon Nov 23 14:19:19 GMT 1992
			XNFS enhancements

		Geoff Clare, 7 June 1995
			Convert headers to/from ASCII.

		Geoff Clare, 9 April 1999
			to_asc[] was 1 byte too small.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdio.h>
#include	<limits.h>
#include	<string.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/stat.h>
#include	"cpiotest.h"
#include	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


#define NO_TESTS	32

public	int	no_tests = NO_TESTS;

extern	unsigned char	from_asc[128];
extern	unsigned char	to_asc[UCHAR_MAX+1];

extern	char	*c_files;
extern	char	*creg_file;
extern	char	*cemp_file;
extern	char	*clink_file;
extern	char	*clinked_file;
extern	char	*cdir_file;
#if !TEST_XNFS
extern	char	*cfifo_file;
extern	char	*cnew_fifo;
#endif /* TEST_XNFS */
extern	char	*cpio_file;
extern	char	*cnew_file;
extern	char	*cnew_dir;
extern	char	*cpio_type;
extern	int	test_fail;
extern	struct	cpio_head hdr;
extern	char	blk[CBLKSIZ];

extern void	test1();
extern void	test2();
extern void	test3();
extern void	test4();
extern void	test5();
extern void	test6();
extern void	test7();
extern void	test8();
extern void	test9();
extern void	test10();
extern void	test11();
extern void	test12();
extern void	test13();
extern void	test14();
extern void	test15();
extern void	test16();
extern void	test17();
extern void	test18();
extern void	test19();
extern void	test20();
extern void	test21();
extern void	test22();
extern void	test23();
extern void	test24();
extern void	test25();
extern void	test26();
extern void	test27();
extern void	test28();
extern void	test29();
extern void	test30();
extern void	test31();
extern void	test32();

private void	prepare_tests();
private void	cln_tests();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	  test1, 1,
	  test2, 2,
	  test3, 3,
	  test4, 4,
	  test5, 5,
	  test6, 6,
	  test7, 7,
	  test8, 8,
	  test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	 test29, 29,
	 test30, 30,
	 test31, 31,
	 test32, 32,
	 NULL, 0
};

private void
prepare_tests()
{
	int	fd, i;

	DBUG_ENTER("prepare_tests");

	if(uids()!=0)
	{
		int t;
		for(t=1; t <= no_tests; t++)
		{
			tet_delete(t, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	(void) umask((mode_t)0);

	/* Make file with something in it */
	fd = crfile(creg_file, (mode_t)MODEANY, (int)geteuid(), (int)getegid());
	if(fd >= 0) {
		/* Write something to make it not a zero length file */
		(void) write(fd, REGDATA, (unsigned) strlen(REGDATA));
	}
	(void) close(fd);

	/* Make an empty file */
	fd = crfile(cemp_file, (mode_t)MODEANY, (int)geteuid(), (int)getegid());
	(void) close(fd);

	/* Make a file linked to another */
	fd = crfile(clink_file, (mode_t)MODEANY, (int)geteuid(), (int)getegid());
	if(fd >= 0) {
		/* Write something to make it not a zero length file */
		(void) write(fd, LNKDATA, (unsigned) strlen(LNKDATA));
	}
	(void) close(fd);
	(void) link(clink_file, clinked_file);

	/* Make directory, fifo */
	(void) crdir(cdir_file, (mode_t)MODEANY, (int)geteuid(), (int)getegid());

#if !TEST_XNFS
	(void) unlink(cfifo_file);
	(void) mkfifo(cfifo_file, MODEANY);
#endif /* TEST_XNFS */

	/* Initialise native-chars-to-ASCII conversion table */
	for (i = 1; i < 128; i++)
		if (from_asc[i] != 0)
			to_asc[from_asc[i]] = i;

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) unlink(creg_file);
	(void) unlink(cemp_file);
	(void) unlink(clink_file);
	(void) unlink(clinked_file);
	(void) rmdir(cdir_file);
	(void) unlink(cnew_file);
	(void) rmdir(cnew_dir);

#if !TEST_XNFS
	(void) unlink(cfifo_file);
	(void) unlink(cnew_fifo);
#endif /* TEST_XNFS */
	DBUG_VOID_RETURN;
}
