/*
*      SCCS:  @(#)  POSIX.os/dataform/cpio/cpio1.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.cpio

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the IEEE Std. 1003.1-1988 "Extended cpio format".
	This file contains the tests involving creating cpio format archives.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu May 25 13:27:33 BST 1989
MODIFICATIONS:
		Geoff Clare, 17 Nov 1989
			Don't use S_IFMT

		Peter Lee, Unisoft Ltd.,
		Wed Jul 25 14:39:11 BST 1990
		Added resilience code.

		Geoff Clare, 18 Feb 1991
			Changes to remove assumption that cpio_head
			members are contiguous.

		Stuart Boutell, 5 Apr 1991
			Deleted "extended cpio archive format is a series
		of entries, each comprising of a header, name, file" test
		
		Calum Laing, Mon Nov 23 14:19:19 GMT 1992
			XNFS enhancements

		Geoff Clare, 7 June 1995
			Convert headers to/from ASCII.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	"cpiotest.h"
#include 	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


extern	long	aoctol();
extern	char	*print_field();

extern	int	test_fail;

extern	char	*c_files;
extern	char	*creg_file;
extern	char	*cemp_file;
extern	char	*clink_file;
extern	char	*clinked_file;
extern	char	*cdir_file;
#if !TEST_XNFS
extern	char	*cfifo_file;
#endif /* TEST_XNFS */
extern	char	*cpio_file;
extern	struct	cpio_head hdr;
extern	char	headbuf[CHEADSIZE];
extern	char	blk[];

private void
do_chk_oct(str, len, name)
char	*str;	/* Field value to check */
int	len;	/* Field length */
char	*name;	/* Field name */
{
	DBUG_ENTER("do_chk_oct");

	if(chk_oct(str, len) == SYSERROR)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"%s\" field \"%s\" is not a zero-filled octal number",
			name, print_field(len, str));
	}

	DBUG_VOID_RETURN;
}

private int
chk_t2(fd, name)
int	fd;			/* File descriptor of cpio format file */
char	*name;			/* Expected file name in the record */
{
	int	err;
	int	len;
	int	ret;
	int	pathok = 0;

	/*
	 * Check that cpio format record contains header, "c_name" and
	 * "c_filedata".
	 * Note that none of these 3 parts are validated.
	 */

	DBUG_ENTER("chk_t2");

	globok = 0;

	/* Read cpio format header */
	if(read_head(fd) == SYSERROR) 
	{
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
	
	/* Read the name field */
	len = aoctol(hdr.c_namesize, CNAMSIZLEN);
	if(read_name(fd, blk, len) == SYSERROR)
	{
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	/* Read the data */
	len = aoctol(hdr.c_filesize, CFILSIZLEN);
	if((ret = read(fd, blk, (unsigned)len)) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("file %s: read() of cpio format file contents failed - errno %d", name, err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	if(ret < len)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("file %s: unable to read file contents in cpio format archive", name); 
		in_rpt("read() returned %d; expected field length %d", ret, len);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_RETURN(0);
}

public void
test2()
{
	int	cpio_fd;
	int	pathok = 0;

	/*
	 * Test that the cpio format created consists of a series of 
	 * entries, each comprised of a header that describes the file,
	 * the name of the file and the file contents.
	 */

	DBUG_ENTER("test2");

	test_fail = 0;
	globok = 0;

	if(chk_file(creg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chk_file(clink_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chk_file(clinked_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, c_files)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if(chk_t2(cpio_fd, creg_file) == SYSERROR)
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if(chk_t2(cpio_fd, clink_file) == SYSERROR)
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if(chk_t2(cpio_fd, clinked_file) == SYSERROR)
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(7);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test3()
{
	int	cpio_fd;
	int	pathok = 0;

	/*
	 * Test that "c_dev", "c_ino", "c_mode", "c_uid", "c_gid", 
	 * "c_nlink", "c_rdev", "c_mtime", "c_namesize", "c_filesize"
	 * are zero filled octal numbers represented in ASCII.
	 */

	DBUG_ENTER("test3");

	test_fail = 0;

	if(chk_file(creg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the regular file */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the correct file was read */
	if(chk_name(cpio_fd, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	do_chk_oct(hdr.c_dev, CDEVLEN, "c_dev");
	do_chk_oct(hdr.c_ino, CINOLEN, "c_ino");
	do_chk_oct(hdr.c_mode, CMODELEN, "c_mode");
	do_chk_oct(hdr.c_uid, CUGIDLEN, "c_uid");
	do_chk_oct(hdr.c_gid, CUGIDLEN, "c_gid");
	do_chk_oct(hdr.c_nlink, CNLINKLEN, "c_nlink");
	do_chk_oct(hdr.c_rdev, CRDEVLEN, "c_rdev");
	do_chk_oct(hdr.c_mtime, CMTIMELEN, "c_mtime");
	do_chk_oct(hdr.c_namesize, CNAMSIZLEN, "c_namesize");
	do_chk_oct(hdr.c_filesize, CFILSIZLEN, "c_filesize");

	if(!test_fail)
		PATH_XS_RPT(4);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test4()
{
	int	cpio_fd;
	int	pathok = 0;

	/*
	 * Test that "c_magic" contains the magic bytes "070707".
	 */

	DBUG_ENTER("test4");

	test_fail = 0;

	if(chk_file(creg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the regular file */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the correct file was read */
	if(chk_name(cpio_fd, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(strncmp(hdr.c_magic, CMAGIC, (size_t)CMAGLEN))
	{
		xx_rpt(FAILURE);
		in_rpt("incorrect \"c_magic\" field in cpio format archive"); 
		in_rpt("expected value: %s; observed value: %s", CMAGIC, 
					print_field(CMAGLEN, hdr.c_magic));
	}
	else
		PATH_XS_RPT(4);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test5()
{
	int	cpio_fd;
	long	dev, lnk_dev, reg_dev;
	long	ino, lnk_ino, reg_ino;
	int	pathok = 0;

	/*
	 * Test that "c_dev" and "c_ino" pair uniquely identify the files
	 * stored on the archive except if they are links to the same file.
	 */

	DBUG_ENTER("test5");

	test_fail = 0;

	if(chk_file(creg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chk_file(clink_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chk_file(clinked_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, c_files)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read record for the regular file */
	globok = 0;
	if(read_rec(cpio_fd, &hdr, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Get device and inode numbers */
	reg_dev = aoctol(hdr.c_dev, CDEVLEN);
	reg_ino = aoctol(hdr.c_ino, CINOLEN);

	/* Read record for the linked file */
	globok = 0;
	if(read_rec(cpio_fd, &hdr, clink_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Get device and inode numbers */
	lnk_dev = aoctol(hdr.c_dev, CDEVLEN);
	lnk_ino = aoctol(hdr.c_ino, CINOLEN);

	/* Read record for the linked file */
	globok = 0;
	if(read_rec(cpio_fd, &hdr, clinked_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Get device and inode numbers */
	dev = aoctol(hdr.c_dev, CDEVLEN);
	ino = aoctol(hdr.c_ino, CINOLEN);

	if(ino != lnk_ino || dev != lnk_dev)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"c_ino\" and \"c_dev\" values for linked files not identical");
		in_rpt("\"c_ino\" values were %ld and %ld", lnk_ino, ino);
		in_rpt("\"c_dev\" values were %ld and %ld", lnk_dev, dev);
	}
	else
		PATH_TRACE;
	if(ino == reg_ino && dev == reg_dev)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"c_ino\" and \"c_dev\" values for non-linked files were identical");
		in_rpt("\"c_ino\" values were both %ld", ino);
		in_rpt("\"c_dev\" values were both %ld", dev);
	}
	else
		PATH_TRACE;
	if(lnk_ino == reg_ino && lnk_dev == reg_dev)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"c_ino\" and \"c_dev\" values for non-linked files were identical");
		in_rpt("\"c_ino\" values were both %ld", lnk_ino);
		in_rpt("\"c_dev\" values were both %ld", lnk_dev);
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(10);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test6()
{
	int	cpio_fd;
	mode_t	mode;
	int	pathok = 0;

	/*
	 * Test that "c_mode" contains file type and access permissions.
	 */

	DBUG_ENTER("test6");

	test_fail = 0;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the regular file */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the correct file was read */
	if(chk_name(cpio_fd, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	mode = aoctol(hdr.c_mode, CMODELEN);

	if((mode & C_IFMT) != C_ISREG)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"mode\" field did not contain file type information");
		in_rpt("\"mode\" field contained 0%lo; file type 0%o", 
			(unsigned long)(mode & C_IFMT), C_ISREG);
	}
	else
		PATH_TRACE;
	if((mode & C_MODEMASK) != C_MODEANY)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"mode\" field did not contain file access information");
		in_rpt("\"mode\" field contained 0%lo; file access 0%lo", 
			(unsigned long)(mode & C_MODEMASK),
			(unsigned long)C_MODEANY);
	}
	else
		PATH_TRACE;
	if(!test_fail)
		PATH_XS_RPT(5);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test7()
{
	int	err;
	int	cpio_fd;
	uid_t	uid;
	struct	stat stbuf;
	int	pathok = 0;

	/*
	 * Test that "c_uid" contains user ID of the file owner.
	 */

	DBUG_ENTER("test7");

	test_fail = 0;

	if(stat(creg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		if(err == ENOENT)
			in_rpt("file %s is not available for this test", 
								creg_file);
		in_rpt("stat(%s) failed - errno %d", creg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the regular file */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the correct file was read */
	if(chk_name(cpio_fd, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	uid = aoctol(hdr.c_uid, CUGIDLEN);

	if(uid != stbuf.st_uid)
	{
		xx_rpt(FAILURE);
		in_rpt("\"uid\" field was not the user ID of the file owner");
		in_rpt("\"uid\" field was set to %ld; user ID was %ld", 
			(long)uid, (long)stbuf.st_uid);
	}
	else
		PATH_XS_RPT(4);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test8()
{
	int	err;
	int	cpio_fd;
	gid_t	gid;
	struct	stat stbuf;
	int	pathok = 0;

	/*
	 * Test that "c_gid" contains group ID of the file group.
	 */

	DBUG_ENTER("test8");

	test_fail = 0;

	if(stat(creg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		if(err == ENOENT)
			in_rpt("file %s is not available for this test", 
								creg_file);
		in_rpt("stat(%s) failed - errno %d", creg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the regular file */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the correct file was read */
	if(chk_name(cpio_fd, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	gid = aoctol(hdr.c_gid, CUGIDLEN);

	if(gid != stbuf.st_gid)
	{
		xx_rpt(FAILURE);
		in_rpt("\"gid\" field was not the group ID of the file group");
		in_rpt("\"gid\" field was set to %ld; group ID was %ld", 
			(long)gid, (long)stbuf.st_gid);
	}
	else
		PATH_XS_RPT(4);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test9()
{
	int	cpio_fd;
	long	lnk, lnk_lnk, reg_lnk;
	int	pathok = 0;

	/*
	 * Test that "c_nlink" contains the number of links to a file
	 * stored on the archive.
	 */

	DBUG_ENTER("test9");

	test_fail = 0;

	if(chk_file(creg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chk_file(clink_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chk_file(clinked_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, c_files)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read record for the regular file */
	globok = 0;
	if(read_rec(cpio_fd, &hdr, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Get device and inode numbers */
	reg_lnk = aoctol(hdr.c_nlink, CNLINKLEN);

	/* Read record for the linked file */
	globok = 0;
	if(read_rec(cpio_fd, &hdr, clink_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Get device and inode numbers */
	lnk_lnk = aoctol(hdr.c_nlink, CNLINKLEN);

	/* Read record for the linked file */
	globok = 0;
	if(read_rec(cpio_fd, &hdr, clinked_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Get device and inode numbers */
	lnk = aoctol(hdr.c_nlink, CNLINKLEN);

	if(reg_lnk != 1)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"c_nlink\" value for regular file not correct");
		in_rpt("expected value: %d; observed value: %ld", 1, reg_lnk);
	}
	else
		PATH_TRACE;
	if(lnk != 2)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"c_nlink\" value for linked file not correct");
		in_rpt("expected value: %d; observed value: %ld", 2, lnk);
	}
	else
		PATH_TRACE;
	if(lnk_lnk != 2)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"c_nlink\" value for linked file not correct");
		in_rpt("expected value: %d; observed value: %ld", 2, lnk_lnk);
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(10);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test10()
{
	int	err;
	int	cpio_fd;
	time_t	stime, mtime;
	struct	stat stbuf;
	int	pathok = 0;

	/*
	 * Test that "c_mtime" contains the file modification time.
	 */

	DBUG_ENTER("test10");

	test_fail = 0;

	if(stat(creg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		if(err == ENOENT)
			in_rpt("file %s is not available for this test", 
								creg_file);
		in_rpt("stat(%s) failed - errno %d", creg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait a little while to magnify any attempt to reset "mtime" */
	(void) sleep(WAITTIME/2);

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the regular file */
	globok = 0;
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the correct file was read */
	if(chk_name(cpio_fd, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	mtime = aoctol(hdr.c_mtime, CMTIMELEN);

	if((stime = stbuf.st_mtime) != mtime)
	{
		xx_rpt(FAILURE);
		in_rpt("mtime value %ld was not file modification time %ld",
						 mtime, stime);
	}
	else
		PATH_XS_RPT(4);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test11()
{
	int	cpio_fd;
	int	len;
	int	pathok = 0;

	/*
	 * Test that "c_namesize" contains the length of the pathname 
	 * including the NULL terminator.
	 */

	DBUG_ENTER("test11");

	test_fail = 0;

	if(chk_file(creg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the regular file */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the correct file was read */
	if(chk_name(cpio_fd, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = aoctol(hdr.c_namesize, CNAMSIZLEN);

	if(len != (strlen(creg_file) + 1))
	{
		xx_rpt(FAILURE);
		in_rpt("incorrect \"c_namesize\" field in cpio format archive"); 
		in_rpt("expected value: %ld; observed value: %d",
					(long)strlen(creg_file) + 1, len);
	}
	else
		PATH_XS_RPT(4);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test12()
{
	int	cpio_fd;
	int	len;
	int	pathok = 0;

	/*
	 * Test that "c_filesize" contains the length of the data.
	 */

	DBUG_ENTER("test12");

	test_fail = 0;

	if(chk_file(creg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the regular file */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the correct file was read */
	if(chk_name(cpio_fd, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = aoctol(hdr.c_filesize, CFILSIZLEN);

	if(len != strlen(REGDATA))
	{
		xx_rpt(FAILURE);
		in_rpt("incorrect \"c_filesize\" field in cpio format archive"); 
		in_rpt("expected value: %ld; observed value: %d",
					(long)strlen(REGDATA), len);
	}
	else
		PATH_XS_RPT(4);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test13()
{
	int	cpio_fd;
	int	len;
	int	pathok = 0;

	/*
	 * Test that "c_name" follows the header and contains file name.
	 */

	DBUG_ENTER("test13");

	test_fail = 0;

	if(chk_file(creg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the regular file */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to read the file name even if field length is wrong */
	len = strlen(creg_file) + 1;

	if(read_name(cpio_fd, blk, len) == SYSERROR)
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chk_bytes(blk, creg_file, len))
	{
		xx_rpt(FAILURE);
		in_rpt("incorrect \"c_name\" field in cpio format archive"); 
		in_rpt("expected value: %s; observed value: %s",
					creg_file, print_field(len, blk));
	}
	else
		PATH_XS_RPT(4);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test14()
{
	int	cpio_fd;
	int	len;
	int	pathok = 0;

	/*
	 * Test that "c_filedata" follows "c_name"
	 * and contains "c_filesize" bytes of file data.
	 */

	DBUG_ENTER("test14");

	test_fail = 0;

	if(chk_file(creg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the regular file */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = aoctol(hdr.c_namesize, CNAMSIZLEN);

	if(read_name(cpio_fd, blk, len) == SYSERROR)
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = strlen(TRAILER) + 1;

	if(!chk_bytes(blk, TRAILER, len))
	{
		xx_rpt(DELETION);
		in_rpt("cpio format trailer record unexpectedly read");
		in_rpt("file name \"%s\" not archived", creg_file);
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = strlen(creg_file) + 1;

	if(chk_bytes(blk, creg_file, len))
	{
		xx_rpt(DELETION);
		in_rpt("incorrect \"c_name\" field in cpio format archive"); 
		in_rpt("expected value: %s; observed value: %s",
					creg_file, print_field(len, blk));
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = aoctol(hdr.c_filesize, CFILSIZLEN);

	if(read_data(cpio_fd, blk, len) == SYSERROR)
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chk_bytes(blk, REGDATA, len))
	{
		xx_rpt(FAILURE);
		in_rpt("incorrect \"c_filedata\" field in cpio format archive"); 
		in_rpt("expected value: %s", REGDATA);
		in_rpt("observed value: %s", print_field(len, blk));
	}
	else
		PATH_XS_RPT(7);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test15()
{
	int	cpio_fd;
	int	len;
	int	pathok = 0;

	/*
	 * Test that "c_filedata" contains no data
	 * if "c_filesize" is zero.
	 */

	DBUG_ENTER("test15");

	test_fail = 0;

	if(chk_file(cemp_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, cemp_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the empty file */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = aoctol(hdr.c_namesize, CNAMSIZLEN);

	if(read_name(cpio_fd, blk, len) == SYSERROR)
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = strlen(TRAILER) + 1;

	if(!chk_bytes(blk, TRAILER, len))
	{
		xx_rpt(DELETION);
		in_rpt("cpio format trailer record unexpectedly read");
		in_rpt("file name \"%s\" not archived", cemp_file);
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = strlen(cemp_file) + 1;

	if(chk_bytes(blk, cemp_file, len))
	{
		xx_rpt(DELETION);
		in_rpt("incorrect \"c_name\" field in cpio format archive"); 
		in_rpt("expected value: %s; observed value: %s",
					cemp_file, print_field(len, blk));
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = aoctol(hdr.c_filesize, CFILSIZLEN);

	if(len != 0)
	{
		xx_rpt(DELETION);
		in_rpt("incorrect \"c_filesize\" field in cpio format archive");
		in_rpt("expected value: %d; observed value: %d", 0, len);
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Should be able to read next record now. */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(strncmp(hdr.c_magic, CMAGIC, (size_t)CMAGLEN))
	{
		xx_rpt(FAILURE);
		in_rpt("data \"%s ...\" was contained in \"c_filedata\" field",
					print_field(CMAGLEN, hdr.c_magic));
	}
	else
		PATH_XS_RPT(8);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS
public void
test16()
{
	DBUG_ENTER("test16");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#else
public void
test16()
{
	int	cpio_fd;
	int	len;
	int	pathok = 0;

	/*
	 * Test that "c_filesize" is zero for a fifo.
	 */

	DBUG_ENTER("test16");

	test_fail = 0;

	if(chk_file(cfifo_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, cfifo_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the fifo */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the correct file was read */
	if(chk_name(cpio_fd, cfifo_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = aoctol(hdr.c_filesize, CFILSIZLEN);

	if(len != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("\"c_filesize\" field was not zero for a fifo");
		in_rpt("instead, \"c_filesize\" was %d", len);
	}
	else
		PATH_XS_RPT(4);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

public void
test17()
{
	int	cpio_fd;
	int	len;
	int	pathok = 0;

	/*
	 * Test that "c_filesize" is zero for a directory file.
	 */

	DBUG_ENTER("test17");

	test_fail = 0;

	if(chk_file(cdir_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, cdir_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the directory file */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the correct file was read */
	if(chk_name(cpio_fd, cdir_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = aoctol(hdr.c_filesize, CFILSIZLEN);

	if(len != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("\"c_filesize\" field was not zero for a directory file");
		in_rpt("instead, \"c_filesize\" was %d", len);
	}
	else
		PATH_XS_RPT(4);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test18()
{
	int	cpio_fd;
	int	len;
	int	pathok = 0;

	/*
	 * Test that "c_filesize" is zero for the trailer.
	 */

	DBUG_ENTER("test18");

	test_fail = 0;

	if(chk_file(creg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read record for the regular file */
	globok = 0;
	if(read_rec(cpio_fd, &hdr, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the trailer */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = aoctol(hdr.c_namesize, CNAMSIZLEN);

	if(read_name(cpio_fd, blk, len) == SYSERROR)
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = strlen(TRAILER) + 1;

	if(chk_bytes(blk, TRAILER, len))
	{
		xx_rpt(DELETION);
		in_rpt("cpio format trailer record not read as expected");
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = aoctol(hdr.c_filesize, CFILSIZLEN);

	if(len != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("\"c_filesize\" field was not zero for trailer");
		in_rpt("instead, \"c_filesize\" was %d", len);
	}
	else
		PATH_XS_RPT(6);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test19()
{
	int	cpio_fd;
	int	pathok = 0;

	/*
	 * Test that header for next file entry is written directly
	 * after the last byte of the preceding file entry.
	 */

	DBUG_ENTER("test19");

	test_fail = 0;

	if(chk_file(creg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chk_file(clink_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chk_file(clinked_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, c_files)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read record for the regular file */
	globok = 0;
	if(read_rec(cpio_fd, &hdr, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header of the next entry */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(strncmp(hdr.c_magic, CMAGIC, (size_t)CMAGLEN))
	{
		xx_rpt(FAILURE);
		in_rpt("header for next file entry not written directly after preceding entry");
		in_rpt("expected: \"%s ...\"; observed: \"%s ...\"",
				CMAGIC, print_field(CMAGLEN, hdr.c_magic));
	}
	else
		PATH_XS_RPT(6);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
test20()
{
	int	cpio_fd;
	int	len;
	int	pathok = 0;

	/*
	 * Test that header with a filename "TRAILER!!!" appears at the end
	 * of the cpio format archive.
	 */

	DBUG_ENTER("test20");

	test_fail = 0;

	if(chk_file(creg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read record for the regular file */
	globok = 0;
	if(read_rec(cpio_fd, &hdr, creg_file) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Read header for the trailer */
	if(read_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = aoctol(hdr.c_namesize, CNAMSIZLEN);

	if(read_name(cpio_fd, blk, len) == SYSERROR)
	{
		(void) close_cpio(cpio_file, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = strlen(TRAILER) + 1;

	if(chk_bytes(blk, TRAILER, len))
	{
		xx_rpt(FAILURE);
		in_rpt("\"c_name\" field was not \"%s\" for trailer", TRAILER);
		in_rpt("instead, \"c_name\" was \"%s\"", print_field(len, blk));
	}
	else
		PATH_XS_RPT(5);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}
