/*
*      SCCS:  @(#)  POSIX.os/dataform/cpio/cpio2.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.cpio

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the IEEE Std. 1003.1-1988 "Extended cpio format".
	This file contains the tests involving reading cpio format archives.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu May 25 13:27:33 BST 1989
MODIFICATIONS:
		Geoff Clare, 18 Feb 1991
			Changes to remove assumption that cpio_head
			members are contiguous.

		Calum Laing, Mon Nov 23 14:19:19 GMT 1992
			XNFS enhancements

		Geoff Clare, 7 June 1995
			Convert headers to/from ASCII.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	"cpiotest.h"
#include 	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


extern	long	aoctol();
extern	char	*print_field();
extern	char	*file_type();

extern	int	vguid1;
extern	int	test_fail;

extern	char	*creg_file;
extern	char	*cdir_file;
#if !TEST_XNFS
extern	char	*cfifo_file;
extern	char	*cnew_fifo;
#endif /* TEST_XNFS */
extern	char	*cpio_file;
extern	char	*cpio_type;
extern	char	*cnew_file;
extern	char	*cnew_dir;
extern	struct	cpio_head hdr;
extern	char	headbuf[CHEADSIZE];
extern	char	blk[];

private void
chk_t21(fd, headptr, name, type, desc)
int	fd;			/* File descriptor of cpio format file */
struct	cpio_head *headptr;	/* Pointer to buffer for cpio format header */
char	*name;			/* Expected file name in the record */
mode_t	type;			/* file type flag C_ISREG, etc */
char	*desc;			/* Description of file type */
{
	int	len;
	mode_t	mode;
	int	pathok = 0;

	/*
	 * Read a cpio format record from given file and check the given
	 * file has been stored.
	 */

	DBUG_ENTER("chk_t21");

	globok = 0;

	/* Read cpio format header */
	if(read_head(fd) == SYSERROR) 
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check the c_type field has correct file type */
	mode = aoctol(headptr->c_mode, CMODELEN);

	if((mode & C_IFMT) != type)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("for %s, \"mode\" field did not contain correct file type 0%lo",
			desc, (unsigned long)type);
		in_rpt("instead, \"mode\" field contained 0%lo",
			(unsigned long)(mode & C_IFMT));
	}
	else
		PATH_TRACE;

	/* Read and check the name field */
	/* Try to read the file name and see if it's trailer record */
	len = strlen(name) + 1;
	if(len < strlen(TRAILER))
		len = strlen(TRAILER);

	if(read_name(fd, blk, len) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(!strncmp(blk, TRAILER, strlen(TRAILER)))
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("for %s, cpio format trailer record unexpectedly read", desc);
		in_rpt("unable to archive %s \"%s\"", desc, name);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = strlen(name) + 1;

	if(chk_bytes(blk, name, len))
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("for %s, incorrect \"c_name\" field in cpio format archive", desc); 
		in_rpt("expected value: %s; observed value: %s",
					name, print_field(len, blk));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

public void
t21c_dir()
{
	int	cpio_fd;
	int	pathok = 0;

	/*
	 * Test that directory file can be stored in cpio format.
	 */

	DBUG_ENTER("t21c_dir");

	globok = 0;

	if(chk_file(cdir_file) == SYSERROR)
		DBUG_VOID_RETURN;

	if((cpio_fd = open_cpio(cpio_file, cdir_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Check the file was stored in cpio format record */
	globok = 0;
	chk_t21(cpio_fd, &hdr, cdir_file, (mode_t)C_ISDIR, "directory file");
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS
public void
t21c_fifo()
{
	int	cpio_fd;
	int	pathok = 0;

	/*
	 * Test that fifo can be stored in cpio format.
	 */

	DBUG_ENTER("t21c_fifo");

	globok = 0;

	if(chk_file(cfifo_file) == SYSERROR)
		DBUG_VOID_RETURN;

	if((cpio_fd = open_cpio(cpio_file, cfifo_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Check the file was stored in cpio format record */
	globok = 0;
	chk_t21(cpio_fd, &hdr, cfifo_file, (mode_t)C_ISFIFO, "fifo");
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1);
	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

public void
t21c_reg()
{
	int	cpio_fd;
	int	pathok = 0;

	/*
	 * Test that regular file can be stored in cpio format.
	 */

	DBUG_ENTER("t21c_reg");

	globok = 0;

	if(chk_file(creg_file) == SYSERROR)
		DBUG_VOID_RETURN;

	if((cpio_fd = open_cpio(cpio_file, creg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Check the file was stored in cpio format record */
	globok = 0;
	chk_t21(cpio_fd, &hdr, creg_file, (mode_t)C_ISREG, "regular file");
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1);

	(void) close_cpio(cpio_file, cpio_fd);

	DBUG_VOID_RETURN;
}

public void
t21x_dir()
{
	int	cpio_fd;
	int	err;
	int	len;
	struct	stat stbuf;
	char	*s;
	int	pathok = 0;

	/*
	 * Test that directory file can be extracted from the archive.
	 */

	DBUG_ENTER("t21x_dir");

	globok = 0;

	/* Obtain stat structure to use in initialising cpio format header */
	if(stat(cdir_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d", cdir_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Create an empty archive file */
	if((cpio_fd = new_cpio(cpio_type)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Initialise a cpio format header */
	init_head(cnew_dir, &stbuf, &hdr, (mode_t)C_ISDIR);

	if(write_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	len = strlen(cnew_dir) + 1;
	if(write_name(cpio_fd, cnew_dir, len) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(write_data(cpio_fd, "", 0) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	globok = 0;
	if(write_trail(cpio_fd, &stbuf, &hdr) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) rmdir(cnew_dir);
	if(stat(cnew_dir, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", cnew_dir);
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	 
	/* Extract the file */
	globok = 0;
	if(extract_cpio(cpio_type, NORESTORE) == SYSERROR)
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(stat(cnew_dir, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("directory file was not extracted from cpio archive");
		in_rpt("stat(%s) failed - errno %d", cnew_dir, err);
	}
	else if(!(S_ISDIR(stbuf.st_mode)))
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("directory file was not extracted from cpio archive");
		if((s = file_type(&stbuf)) != (char *)NULL)
			in_rpt("instead %s was extracted", s);
	}
	else
		PATH_TRACE;

	(void) rmdir(cnew_dir);
	(void) close_cpio(cpio_type, cpio_fd);

	PATH_FUNC_RPT(9);
	
	DBUG_VOID_RETURN;
}

#if !TEST_XNFS
public void
t21x_fifo()
{
	int	cpio_fd;
	int	err;
	int	len;
	struct	stat stbuf;
	char	*s;
	int	pathok = 0;

	/*
	 * Test that fifo can be extracted from the archive.
	 */

	DBUG_ENTER("t21x_fifo");

	globok = 0;

	/* Obtain stat structure to use in initialising cpio format header */
	if(stat(cfifo_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d", cfifo_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Create an empty archive file */
	if((cpio_fd = new_cpio(cpio_type)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Initialise a cpio format header */
	init_head(cnew_fifo, &stbuf, &hdr, (mode_t)C_ISFIFO);

	if(write_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	len = strlen(cnew_fifo) + 1;
	if(write_name(cpio_fd, cnew_fifo, len) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(write_data(cpio_fd, "", 0) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if(write_trail(cpio_fd, &stbuf, &hdr) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(cnew_fifo);
	if(stat(cnew_fifo, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", cnew_fifo);
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	 
	/* Extract the file */
	globok = 0;
	if(extract_cpio(cpio_type, NORESTORE) == SYSERROR)
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(stat(cnew_fifo, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("fifo was not extracted from cpio archive");
		in_rpt("stat(%s) failed - errno %d", cnew_fifo, err);
	}
	else if(!(S_ISFIFO(stbuf.st_mode)))
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("fifo was not extracted from cpio archive");
		if((s = file_type(&stbuf)) != (char *)NULL)
			in_rpt("instead %s was extracted", s);
	}
	else
		PATH_TRACE;

	(void) unlink(cnew_fifo);
	(void) close_cpio(cpio_type, cpio_fd);

	PATH_FUNC_RPT(9);
	
	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

public void
t21x_reg()
{
	int	cpio_fd;
	int	err;
	int	len;
	struct	stat stbuf;
	char	*s;
	int	pathok = 0;

	/*
	 * Test that regular file can be extracted from the archive.
	 */

	DBUG_ENTER("t21x_reg");

	globok = 0;

	/* Obtain stat structure to use in initialising cpio format header */
	if(stat(creg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d", creg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Create an empty archive file */
	if((cpio_fd = new_cpio(cpio_type)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Initialise a cpio format header */
	init_head(cnew_file, &stbuf, &hdr, (mode_t)C_ISREG);

	if(write_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = strlen(cnew_file) + 1;
	if(write_name(cpio_fd, cnew_file, len) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = strlen(REGDATA);
	if(write_data(cpio_fd, REGDATA, len) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if(write_trail(cpio_fd, &stbuf, &hdr) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(cnew_file);
	if(stat(cnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", cnew_file);
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	 
	/* Extract the file */
	globok = 0;
	if(extract_cpio(cpio_type, NORESTORE) == SYSERROR)
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat(cnew_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("regular file was not extracted from cpio archive");
		in_rpt("stat(%s) failed - errno %d", cnew_file, err);
	}
	else if(!(S_ISREG(stbuf.st_mode)) || stbuf.st_nlink > 1)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("regular file was not extracted from cpio archive");
		if((s = file_type(&stbuf)) != (char *)NULL)
			in_rpt("instead %s was extracted", s);
	}
	else
		PATH_TRACE;

	(void) unlink(cnew_file);
	(void) close_cpio(cpio_type, cpio_fd);
	
	PATH_FUNC_RPT(9);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

public void
test21()
{
	DBUG_ENTER("test21");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else

public void
test21()
{
	int	pathok = 0;

	DBUG_ENTER("test21");

	/*
	 * Test that data from directories, fifos and regular files
	 * can be stored in cpio archive format and extracted from it.
	 */

	test_fail = 0;

	globok = 0;
	t21c_dir();
	PATH_FUNC_TRACE;

	globok = 0;
	t21c_fifo();
	PATH_FUNC_TRACE;

	globok = 0;
	t21c_reg();
	PATH_FUNC_TRACE;

	globok = 0;
	t21x_dir();
	PATH_FUNC_TRACE;

	globok = 0;
	t21x_fifo();
	PATH_FUNC_TRACE;

	globok = 0;
	t21x_reg();
	PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

public void
test29()
{
	int	pathok = 0;

	DBUG_ENTER("test29");

	/*
	 * Test that data from directories and regular files
	 * can be stored in cpio archive format.
	 */

	test_fail = 0;

	globok = 0;
	t21c_dir();
	PATH_FUNC_TRACE;

	globok = 0;
	t21c_reg();
	PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

#else

public void
test29()
{
	DBUG_ENTER("test29");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

public void
test30()
{
	int	pathok = 0;

	DBUG_ENTER("test30");

	/*
	 * Test that data from directories and regular files
	 * can be extracted from cpio archive format.
	 */

	test_fail = 0;

	globok = 0;
	t21x_dir();
	PATH_FUNC_TRACE;

	globok = 0;
	t21x_reg();
	PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

#else

public void
test30()
{
	DBUG_ENTER("test30");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
test22()
{
	int	cpio_fd;
	int	err;
	int	len;
	struct	stat stbuf;
	char	*s;
	int	pathok = 0;

	/*
	 * Test that data from regular file is restored.
	 */

	DBUG_ENTER("test22");

	test_fail = 0;

	/* Obtain stat structure to use in initialising cpio format header */
	if(stat(creg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d", creg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Create an empty archive file */
	if((cpio_fd = new_cpio(cpio_type)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Initialise a cpio format header */
	init_head(cnew_file, &stbuf, &hdr, (mode_t)C_ISREG);

	if(write_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	len = strlen(cnew_file) + 1;
	if(write_name(cpio_fd, cnew_file, len) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	len = strlen(REGDATA);
	if(write_data(cpio_fd, REGDATA, len) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	globok = 0;
	if(write_trail(cpio_fd, &stbuf, &hdr) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(cnew_file);
	if(stat(cnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", cnew_file);
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	 
	/* Extract the file */
	globok = 0;
	if(extract_cpio(cpio_type, NORESTORE) == SYSERROR)
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(stat(cnew_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("regular file was not extracted from cpio archive");
		in_rpt("stat(%s) failed - errno %d", cnew_file, err);
	}
	else if(!(S_ISREG(stbuf.st_mode)) || stbuf.st_nlink > 1)
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("regular file was not extracted from cpio archive");
		if((s = file_type(&stbuf)) != (char *)NULL)
			in_rpt("instead %s was extracted", s);
	}
	else if(chk_data(cnew_file, REGDATA, (int)strlen(REGDATA)) != SYSERROR)
		PATH_XS_RPT(8);
	
	(void) unlink(cnew_file);
	(void) close_cpio(cpio_type, cpio_fd);
	
	DBUG_VOID_RETURN;
}

public void
test23()
{
	int	cpio_fd;
	int	err;
	int	len;
	struct	stat stbuf;
	int	pathok = 0;

	/*
	 * Test that when user does not have appropriate privileges to set 
	 * a particular file mode, the flag is ignored.
	 */

	DBUG_ENTER("test23");

	test_fail = 0;

	/* Obtain stat structure to use in initialising cpio format header */
	if(stat(creg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d", creg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Alter user ID */
	stbuf.st_uid = vguid1;
	
	/* Create an empty archive file */
	if((cpio_fd = new_cpio(cpio_type)) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Initialise a cpio format header */
	init_head(cnew_file, &stbuf, &hdr, (mode_t)(C_ISUID | C_ISREG));

	if(write_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	len = strlen(cnew_file) + 1;
	if(write_name(cpio_fd, cnew_file, len) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	len = strlen(REGDATA);
	if(write_data(cpio_fd, REGDATA, len) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	globok = 0;
	if(write_trail(cpio_fd, &stbuf, &hdr) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(cnew_file);
	if(stat(cnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", cnew_file);
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
		else
		PATH_TRACE;
 
	/* Extract the file */
	globok = 0;
	if(extract_cpio(cpio_type, NORESTORE) == SYSERROR)
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(stat(cnew_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("file %s was not extracted from archive", cnew_file);
		in_rpt("stat(%s) failed - errno %d", cnew_file, err);
	}
	else if(stbuf.st_uid != getuid() && stbuf.st_mode & S_ISUID)
	{
		xx_rpt(FAILURE);
		in_rpt("when user did not have appropriate privilege");
		in_rpt("set-user ID bits of \"mode\" field were not ignored");
	} else
		PATH_XS_RPT(8);
	
	(void) unlink(cnew_file);
	(void) close_cpio(cpio_type, cpio_fd);
	
	DBUG_VOID_RETURN;
}
