/*
*      SCCS:  @(#)  POSIX.os/dataform/cpio/cpio3.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.cpio

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the IEEE Std. 1003.1-1988 "Extended cpio format".
	This file contains the tests in 10.1 "Archive/Interchange File Format"

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Tue Jun  6 15:08:45 BST 1989
MODIFICATIONS:
		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ;
		
		Stuart Boutell, 5 Apr 1991
			Renumbered test 27 to test 1

		Calum Laing,	10th June 1992
			Added pathname resolution test (27)

		Calum Laing,	12th June 1992
			Added portable filename test (28)
		
		Geoff Clare, 18 June 1992
			Added PRES_NODOT to init_pathres() argument.

		Calum Laing, Wed Nov 18 09:44:24 GMT 1992
			Tests 29, 30, 31, 32 XNFS enhancements
		
		Geoff Clare, 24 June 1994
			Increased size of args[] in exec_suproc().

		Geoff Clare, 7 June 1995
			Convert headers to/from ASCII.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	"cpiotest.h"
#include	<pathres.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


extern	char	*errname();
extern	char	*execpath();
extern	char	*pr_char();
extern	char	*newstr();
extern	char	*itoa();
extern	char	*print_field();

extern  char    *cdir_file;
extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;
extern	int	vguid0;
extern	int	vguid1;
extern	int	vggid0;
extern	int	vggid1;
extern	int	test_fail;

extern	char	*creg_file;
extern	char	*cpio_type;
extern	char	*cnew_file;
extern  char    *cnew_dir;
extern	struct	cpio_head hdr;

private	char	*prog_su = "cpio_su";
private	char	*prog_uids = "cpio_uids";
#if TEST_XNFS
private char	*prog_gid = "cpio_gid";
private char	*t31mnt   = "cpio_t31mnt";
#endif /* TEST_XNFS */

private void
exec_suproc(eff_uid, eff_gid, exp_uid, exp_gid)
int	eff_uid;
int	eff_gid;
int	exp_uid;
int	exp_gid;
{
	char	*args[10];
	char	*teststr;
	char	*euid;
	char	*egid;
	char	*expuid;
	char	*expgid;
	char	execsubuf[_POSIX_PATH_MAX];
	int	err;

	DBUG_ENTER("exec_suproc");

	if ((teststr = newstr(itoa(tet_thistest))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((euid = newstr(itoa(eff_uid))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((egid = newstr(itoa(eff_gid))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((expuid = newstr(itoa(exp_uid))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((expgid = newstr(itoa(exp_gid))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}

	(void)strcpy(&execsubuf[0], (char*) execpath(prog_uids));

	args[0] = (char*) execpath(prog_su);
	args[1] = teststr;
	args[2] = euid;
	args[3] = egid;
	args[4] = expuid;
	args[5] = expgid;
	args[6] = vg_ident;
	args[7] = execsubuf;
	args[8] = vg_dbug;
	args[9] = (char *)NULL;
	(void) tet_exec((char*)execpath(prog_su), args, environ);
	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s,..) failed errno %d", execpath(prog_su), err);

	DBUG_VOID_RETURN;
}

private void
test_uids(testfn)
void	(*testfn)();
{
	int	err;
	int	len;
	int	cpio_fd;
	struct	stat stbuf;

	DBUG_ENTER("test_uids");

	test_fail = 0;

	/* Obtain stat structure to use in initialising cpio format header */
	if(stat(creg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d", creg_file, err);
		DBUG_VOID_RETURN;
	}

	/* Create an empty archive file */
	if((cpio_fd = new_cpio(cpio_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a cpio format header */
	init_head(cnew_file, &stbuf, &hdr, (mode_t)C_ISREG);

	if(write_head(cpio_fd) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	len = strlen(cnew_file) + 1;
	if(write_name(cpio_fd, cnew_file, len) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	len = strlen(REGDATA);
	if(write_data(cpio_fd, REGDATA, len) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}
	if(write_trail(cpio_fd, &stbuf, &hdr) == SYSERROR) 
	{
		(void) close_cpio(cpio_type, cpio_fd);
		DBUG_VOID_RETURN;
	}

	/* Do the extraction in a process which is set group ID */
	(void) cppair(testfn, NULLFN, WAITTIME, 0);

	(void) close_cpio(cpio_type, cpio_fd);

	DBUG_VOID_RETURN;
}

private void
ch_t24()
{
	DBUG_ENTER("ch_t24");

	exec_suproc(vguid1, vggid1, vguid1, vggid1);

	DBUG_VOID_RETURN;
}

public void
test24()
{
	DBUG_ENTER("test24");

	/*
	 * Test that the user ID of the extracted file is set to the process's
	 * effective user ID when used by a non-privileged process.
	 */

	test_uids(ch_t24);

	DBUG_VOID_RETURN;
}

private void
ch_t25()
{
	DBUG_ENTER("ch_t25");

	exec_suproc(vguid1, vggid1, vguid1, vggid1);

	DBUG_VOID_RETURN;
}

public void
test25()
{
	DBUG_ENTER("test25");

	/*
	 * Test that the group ID of the extracted file is set to the process's
	 * effective group ID when used by a non-privileged process.
	 */

#if TEST_XNFS
	xx_rpt(NOTINUSE);
#else
	test_uids(ch_t25);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

private void
ch_t26()
{
	DBUG_ENTER("ch_t26");

	exec_suproc(vguid1, vggid1, vguid1, vggid1);

	DBUG_VOID_RETURN;
}

public void
test26()
{
	DBUG_ENTER("test26");

	/*
	 * Test that the file permission of an extracted file is set to the 
	 * file permission value as supplied by the c_mode field in the archive
	 * appropriately modified by the current umask() value
	 * when used by a non-privileged process.
	 */

	test_uids(ch_t26);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	DBUG_ENTER("ch_t1");

	exec_suproc(SU_UID, vggid1, vguid0, vggid0);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS
private void
ch_t1_xnfs()
{
	int	fd;

	DBUG_ENTER("ch_t1_xnfs");

	(void) unlink(creg_file);

        fd = crfile(creg_file, (mode_t)MODEANY, (int)geteuid(), (int)getegid());
        if(fd >= 0) {
                /* Write something to make it not a zero length file */
                (void) write(fd, REGDATA, (unsigned) strlen(REGDATA));
        }
        (void) close(fd);

	test_uids(ch_t1);

	(void) unlink(creg_file);

	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

public void
test1()
{
	DBUG_ENTER("test1");

#if TEST_XNFS
	xnfs_doroot_c(ch_t1_xnfs, WAITTIME, SP_NOSYNC, SP_NOSYNC); 
#else
	test_uids(ch_t1);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

private void
ch_t27()
{
	int	cpio_fd;
	int	len;
	int	err;
	char *p1, *p2;
	int npairs, i;
	struct stat stbuf;
	struct stat stbuf2;
	int	pathok = 0;

	/*
	 * Test that the format utility supports file name resolution.
	 */

	DBUG_ENTER("ch_t27");

	test_fail = 0;

	/* Obtain stat structure to use in initialising cpio format header */
	if(stat(cdir_file, &stbuf2) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d", cdir_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOT|PRES_NODOTDOT);

	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOT|PRES_NODOTDOT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];
	
		DBUG_PRINTF("ch_t27","npairs is %d",npairs);

		/* Create an empty archive file */
		if((cpio_fd = new_cpio(cpio_type)) == SYSERROR)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Initialise a cpio format header. */
		init_head(p1, &stbuf2, &hdr, (mode_t)C_ISDIR);

		if(write_head(cpio_fd) == SYSERROR) 
		{
			(void) close_cpio(cpio_type, cpio_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
			
		len = strlen(p1) + 1;
		if(write_name(cpio_fd, p1, len) == SYSERROR)
		{
			(void) close_cpio(cpio_type, cpio_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

			
		if(write_data(cpio_fd, "", 0) == SYSERROR)
		{
			(void) close_cpio(cpio_type, cpio_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if(write_trail(cpio_fd, &stbuf2, &hdr) == SYSERROR)
		{
			(void) close_cpio(cpio_type, cpio_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Remove the file */
		(void) rmdir(p2);

		/* Check it has really gone */
		if(stat(p2, &stbuf) != SYSERROR || errno != ENOENT) 
		{
			err = errno;
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("rmdir() did not remove directory %s", p2);
			break;
		}
		else
			PATH_TRACE;
			
		/* Extract the file */
		if(extract_cpio(cpio_type, NORESTORE) == SYSERROR)
		{
			(void) close_cpio(cpio_type, cpio_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE; 
		
		/* Check the simple name file has been created */
		if(stat(p2, &stbuf) == SYSERROR)
		{
			err = errno;
			if(err == ENOENT)
			{
				if(!test_fail++)
					xx_rpt(FAILURE);
				in_rpt("when extracting \"%s\" in cpio format", p1);
				in_rpt("file name was not resolved to \"%s\"", p2);
			}
			else
			{
				if(!test_fail++)
					xx_rpt(DELETION);
				in_rpt("stat(%s) failed - errno %d", p2, err);
			}
		}
		else
			PATH_TRACE;

		(void) rmdir(p2);
		(void) close_cpio(cpio_type, cpio_fd);
	}

	if(!test_fail)
	{
		PATH_XS_RPT(2 + (npairs * 8));
	}
	
	(void) unlink(cpio_type);
	cln_pathres();

	DBUG_VOID_RETURN;
}

public void
test27()
{
	DBUG_ENTER("test27");

	(void) cppair(ch_t27, NULLFN, 10 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

public void
test28()
{
	char	**p;
	int	cpio_fd, fd;
	int	i, err;
	int	len;
	int	pathok = 0;
	struct 	stat stbuf;
	char	inbuf[1];

	DBUG_ENTER("test28");

        if(stat(creg_file, &stbuf) == SYSERROR)
        {
                err = errno;
                xx_rpt(DELETION);
                in_rpt("stat(%s) failed - errno %d", creg_file, err);
                DBUG_VOID_RETURN;
        }
	else
		PATH_TRACE;

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		if (eaccess(*p, F_OK) == 0 && unlink(*p) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not unlink \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
	}

	test_fail = 0;

	for (p = portfnam; *p != NULL; p++)
	{

		if((cpio_fd = new_cpio(cpio_type)) == SYSERROR)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* 
		 * Must be done just before the init_head - because the 
		 * next time round loop, write_trail will have overwritten it.
		 */
		stbuf.st_size = (unsigned long)1;

		/* Initialise a cpio format header. */
		init_head(*p, &stbuf, &hdr, (mode_t)C_ISREG);

		if(write_head(cpio_fd) == SYSERROR) 
		{
			(void) close_cpio(cpio_type, cpio_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
			
		len = strlen(*p) + 1;
		if(write_name(cpio_fd, *p, len) == SYSERROR)
		{
			(void) close_cpio(cpio_type, cpio_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* write identifying byte */
		inbuf[0] = 'A' + (p - portfnam);

		if(write_data(cpio_fd, inbuf, 1) == SYSERROR)
		{
			(void) close_cpio(cpio_type, cpio_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if(write_trail(cpio_fd, &stbuf, &hdr) == SYSERROR)
		{
			(void) close_cpio(cpio_type, cpio_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
			
		if(extract_cpio(cpio_type, NORESTORE) == SYSERROR)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE; 

		(void) close_cpio(cpio_type, cpio_fd);
	}

	/* check the contents of the files */

	for (p = portfnam; *p != NULL; p++)
	{
		fd = open(*p, O_RDONLY);
		if (fd < 0)
		{
			err = errno;
			if (!test_fail++)
				xx_rpt(FAILURE);
			in_rpt("open(\"%s\", O_RDONLY) failed - errno %d (%s)",
				*p, err, errname(err));
		}
		else if (read(fd, inbuf, 1) != 1)
		{
			err = errno;
			if (!test_fail++)
				xx_rpt(DELETION);
			in_rpt("read from \"%s\" failed - errno %d (%s)",
				*p, err, errname(err));
		}
		else if (inbuf[0] != 'A' + (p - portfnam))
		{
			err = errno;
			if (!test_fail++)
				xx_rpt(FAILURE);
			in_rpt("file \"%s\" contained '%s', expected '%c'",
				*p, pr_char(inbuf[0]), 'A' + (p - portfnam));
		}
		else
			PATH_TRACE; 

		(void) close(fd);
	}

	if (!test_fail)
		PATH_XS_RPT(1 + 1 + 6 * (p - portfnam) + (p - portfnam));

	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);
	
	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
mount31ch()
{
	char	*args[9];
	int	err;

	DBUG_ENTER("mount31ch");

	args[0] = (char *) execpath(prog_gid);
	args[1] = newstr(itoa(MOUNT_GIDTRUE));
	args[2] = t31mnt;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = (char *)NULL;

	(void) tet_exec((char*)execpath(prog_gid), args, environ);
	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s,..) failed errno %d", execpath(prog_gid), err);

	DBUG_VOID_RETURN;
}

private void
umount31ch()
{
	char	*args[9];
	int	err;

	DBUG_ENTER("umount31ch");

	args[0] = (char *) execpath(prog_gid);
	args[1] = newstr(itoa(UNMOUNT_GIDTRUE));
	args[2] = t31mnt;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = (char *)NULL;

	(void) tet_exec((char*)execpath(prog_gid), args, environ);
	err = errno;
	xx_rpt(DELETION);

	in_rpt("tet_exec(%s,..) failed errno %d", execpath(prog_gid), err);

	DBUG_VOID_RETURN;
}

private void
ch_t31()
{
	DBUG_ENTER("ch_t31");

	exec_suproc(vguid1, vggid1, vguid1, vggid0);

	DBUG_VOID_RETURN;
}

private void
t31ch1()
{
	int	err;
	int	fd;

	DBUG_ENTER("t31ch1");

	if(chdir(t31mnt) == SYSERROR){
                err = errno;
                xx_rpt(DELETION);
                in_rpt("could not chdir(%s) errno %d", t31mnt, err);
                DBUG_VOID_RETURN;
        }

	/* Make file with something in it */
	fd = crfile(creg_file, (mode_t)MODEANY, (int)geteuid(), (int)getegid());
	if(fd >= 0) {
		/* Write something to make it not a zero length file */
		(void) write(fd, REGDATA, (unsigned) strlen(REGDATA));
	}
	(void) close(fd);

	test_uids(ch_t31);

	(void) unlink(creg_file);
}

#endif /* TEST_XNFS */

#if TEST_XNFS

void public
test31()
{
	int	err;

	DBUG_ENTER("test31");

	(void) rmdir(t31mnt);

	/* Create mount point - this is actually on the NFS system already */
	if (vxmkdir(t31mnt) == SYSERROR){
                err = errno;
                xx_rpt(DELETION);
                in_rpt("could not make directory %s errno %d", t31mnt, err);
                DBUG_VOID_RETURN;
        }

	if(tet_fork(mount31ch, NULLFN, WAITTIME, OK|NOK) != OK) {
		(void) rmdir(t31mnt);
		DBUG_VOID_RETURN;
	}
	(void) tet_fork(t31ch1, NULLFN, WAITTIME, OK|NOK);
	(void) tet_fork(umount31ch, NULLFN, WAITTIME, OK|NOK);
	(void) rmdir(t31mnt);
	DBUG_VOID_RETURN;
}

#else

void public
test31()
{
	DBUG_ENTER("test31");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t32()
{
	DBUG_ENTER("ch_t32");

	exec_suproc(vguid1, vggid1, vguid1, vggid1);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void 
test32()
{
	DBUG_ENTER("test32");
#if TEST_XNFS
	test_uids(ch_t32);
#else
	xx_rpt(NOTINUSE);
#endif /* TEST_XNFS */
	DBUG_VOID_RETURN;
}
