/*
*      SCCS:  @(#)  POSIX.os/dataform/cpio/cpio4.c	Rel 4.4.4	    (07/19/99)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.cpio

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the IEEE Std. 1003.1-1988 "Extended cpio format"
	This module contains common functions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu May 25 10:45:42 BST 1989
MODIFICATIONS:
		Geoff Clare, 13 Feb 1990
			Change ltoaoct() first arg from long to unsigned long

		Stuart Boutell, 5 Nov 1990
			TESTROOT -> TET_EXECUTE
			Replaced vg_sjmp reference with internal handler

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called

		Geoff Clare, 18 Feb 1991
			Changes to remove assumption that cpio_head
			members are contiguous.

		Geoff Clare, 23 July 1992
			Change aoctol() to avoid signed overflow.

		Calum Laing, Mon Nov 23 14:19:19 GMT 1992
			XNFS enhancements

		Geoff Clare, 20 Dec 1993
			Don't use DBUG_EXIT() in child of plain fork().

		Geoff Clare, 23 June 1994
			Ensure sigsetjmp() is used in line with constraints.

		Geoff Clare, 7 June 1995
			Convert headers to/from ASCII.

		Geoff Clare, 9 April 1999
			to_asc[] was 1 byte too small.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<limits.h>
#include	<stdio.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<sys/wait.h>
#include	<fcntl.h>
#include	<ctype.h>
#include	<dbug.h>
#include	<report.h>
#include	<timeout.h>
#include	<signal.h>
#include	<setjmp.h>
#include	<exitcodes.h>
#include	"cpiotest.h"
#include 	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


#define	MAXARG		10

extern	void	setsigs();
extern	char	*errname();
extern	char	*signame();

private	sigjmp_buf	sjmp;
private	int		signum;

public	unsigned char	from_asc[128] = {  /* ASCII to native chars */
	0,    0,    0,    0,    0,    0,    0,    0,
	'\b', '\t', '\n', '\v', '\f', '\r', 0,    0,
	0,    0,    0,    0,    0,    0,    0,    0,
	0,    0,    0,    0,    0,    0,    0,    0, 
	' ',  '!',  '"',  '#',  '$',  '%',  '&', '\'',
	'(',  ')',  '*',  '+',  ',',  '-',  '.',  '/',
	'0',  '1',  '2',  '3',  '4',  '5',  '6',  '7',
	'8',  '9',  ':',  ';',  '<',  '=',  '>',  '?',
	'@',  'A',  'B',  'C',  'D',  'E',  'F',  'G',
	'H',  'I',  'J',  'K',  'L',  'M',  'N',  'O',
	'P',  'Q',  'R',  'S',  'T',  'U',  'V',  'W',
	'X',  'Y',  'Z',  '[', '\\',  ']',  '^',  '_',
	'`',  'a',  'b',  'c',  'd',  'e',  'f',  'g',
	'h',  'i',  'j',  'k',  'l',  'm',  'n',  'o',
	'p',  'q',  'r',  's',  't',  'u',  'v',  'w',
	'x',  'y',  'z',  '{',  '|',  '}',  '~',  0,  
};
public	unsigned char	to_asc[UCHAR_MAX+1]; /* native chars to ASCII */

public	char	*c_files =	"cpio-reg cpio-link cpio-linked";
public	char	*creg_file =	"cpio-reg";
public	char	*cemp_file =	"cpio-empty";
public	char	*clink_file =	"cpio-link";
public	char	*clinked_file =	"cpio-linked";
public	char	*cdir_file =	"cpio-dir";
#if !TEST_XNFS
public	char	*cfifo_file =	"cpio-fifo";
public	char	*cnew_fifo =	"cpio-newfifo";
#endif /* TEST_XNFS */
public	char	*cpio_file =	"cpio-file.a";
public	char	*cnew_file =	"cpio-new";
public	char	*cnew_dir =	"cpio-newdir";
public	char	*cpio_type =	"cpio-type.a";
public	int	test_fail;
public	char	blk[CBLKSIZ];
public	char	headbuf[CHEADSIZE];
public	struct	cpio_head hdr = {
	&headbuf[0],
	&headbuf[CMAGLEN],
	&headbuf[CMAGLEN+CDEVLEN],
	&headbuf[CMAGLEN+CDEVLEN+CINOLEN],
	&headbuf[CMAGLEN+CDEVLEN+CINOLEN+CMODELEN],
	&headbuf[CMAGLEN+CDEVLEN+CINOLEN+CMODELEN+CUGIDLEN],
	&headbuf[CMAGLEN+CDEVLEN+CINOLEN+CMODELEN+CUGIDLEN+CUGIDLEN],
	&headbuf[CMAGLEN+CDEVLEN+CINOLEN+CMODELEN+CUGIDLEN+CUGIDLEN+CNLINKLEN],
	&headbuf[CMAGLEN+CDEVLEN+CINOLEN+CMODELEN+CUGIDLEN+CUGIDLEN+CNLINKLEN+CRDEVLEN],
	&headbuf[CMAGLEN+CDEVLEN+CINOLEN+CMODELEN+CUGIDLEN+CUGIDLEN+CNLINKLEN+CRDEVLEN+CMTIMELEN],
	&headbuf[CMAGLEN+CDEVLEN+CINOLEN+CMODELEN+CUGIDLEN+CUGIDLEN+CNLINKLEN+CRDEVLEN+CMTIMELEN+CNAMSIZLEN],
};

public	char	*print_field();
public	char	*file_type();
private	void	sjmpfunc();

private	char	*args[MAXARG];

public char *
file_type(statptr)
struct	stat *statptr;
{
	/*
	 * Return pointer to name of the file type, or (char *) NULL
	 * if it is not one of the expected types.
	 */

	if(S_ISREG(statptr->st_mode)) {
		if(statptr->st_nlink > 1)
			return("linked file");
		else
			return("regular file");
	} else if(S_ISDIR(statptr->st_mode))
		return("directory file");
	else if(S_ISCHR(statptr->st_mode))
		return("character special file");
	else if(S_ISBLK(statptr->st_mode))
		return("block special file");
	else if(S_ISFIFO(statptr->st_mode))
		return("fifo");
	else
		return((char *)NULL);
}

public long
aoctol(str, len)
char	*str;
int	len;
{
	unsigned long	val;
	unsigned long	pow;
	int	i;
	char	digit[2];

	/* 
	 * Calculate the value of the octal value pointed
	 * to by str, which may be preceded or followed by spaces or NULLs.
	 *
	 * len is the total length of the field.
	 */
	val = 0L;
	pow = 1L;
	digit[1] = '\0';

	str = &str[len - 1];
	for(i = 0; i < len; i++, str--)
	{
		if(!isdigit(*str))
			continue;
		digit[0] = *str;
		val += pow * atol(digit);
		pow *= 8L;
	}

	return(val);
}

public void
ltoaoct(total, ptr, len)
unsigned long	total;	/* Calculated checksum value */
char	*ptr;		/* Pointer to location to format data */
int	len;		/* Length of the given field */
{
	char *s = &ptr[len];
	int	i;

	/*
	 * Format the given number as octal number 
	 * preceded by zeros.
	 *
	 * Converse of aoctol().
	 */

	for(i = 0; i < len; i++)
	{
		*--s = '0' + total % 8L;
		total /= 8L;
	}
}

public void
ch_util()
{
	int	err;

	DBUG_ENTER("ch_util");

	(void) execvp(args[0], args);
	err = errno;
	if(!test_fail++)
		xx_rpt(DELETION);
	in_rpt("execvp(%s) failed - errno %d", args[0], err);
	DBUG_LEAVE; exit(A_EXECFAIL); /* not DBUG_EXIT */

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

public int
extract_cpio(name, mode)
char	*name;	/* The name of the cpio format file to extract from. */
int	mode;	/* RESTORE or NORESTORE - restore user and group ID option */
{
	int	ret;
	int	err;
	char	*cpio_name;
	char	*cpio_root;
	struct	stat stbuf;
	char	cpio_cmd[_POSIX_PATH_MAX];
	int	pathok = 0;

	/* 
	 * Extract files from a named cpio format file by calling the 
	 * pre-defined utility.
	 * Return an SYSERROR if test cannot proceed and is deleted, 
	 * otherwise return 0.
	 *
	 * The syntax for the format creating utility is as follows:
	 * XT_CPIO_UTIL is the name of a command file which accepts
	 * a single argument which is the named cpio format file.
	 * XT_ID_CPIO_UTIL is the name of a command file which accepts
	 * a single argument which is the named cpio format file and in
	 * addition has options incorporated to restore user and group ID.
	 * (This requires appropriate privileges).
	 * 
	 * The command files are assumed to be in the TESTROOT/BIN directory.
	 * 
	 * The command file should invoke the format reading utility 
	 * to extract the contents of the named cpio format file.
	 *
	 * In the command file, the options should include any which 
	 * restore file modification times,
	 * and any option to restore directories.
	 */

	DBUG_ENTER("extract_cpio");

	globok = 0;

	if(stat(name, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("cannot read cpio format file %s", name);
		in_rpt("stat() failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	if(mode == RESTORE)
		cpio_name = XT_ID_CPIO_UTIL;
	else
		cpio_name = XT_CPIO_UTIL;
	cpio_root = getenv("TET_EXECUTE");
	if((cpio_root == (char *) NULL) || (*cpio_root == '\0'))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("getenv(\"TET_EXECUTE\") failed");
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	(void) strcpy(cpio_cmd, cpio_root);
	(void) strcat(cpio_cmd, cpio_name);

	args[0] = cpio_cmd;
	args[1] = name;
	args[2] = (char *)NULL;

	ret = fork_util(ch_util, WAITTIME, FORMAT_READ);
	if (ret == PATH_NOK)
		ret = CPIO_OK;
	else
		PATH_TRACE;

	if(ret == SYSERROR || ret == A_EXECFAIL)
	{
		test_fail++;
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
	if(ret == A_MISC)
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("\"%s\"", cpio_cmd);
		in_rpt("was called with incorrect arguments");
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_RETURN(0);
}

public int
create_cpio(name, files)
char	*name;	/* The name of the cpio format file to create. */
char	*files;	/* The files to be placed in the cpio format file. */
{
	int	ret;
	int	err;
	char	*cpio_name;
	char	*cpio_root;
	struct	stat stbuf;
	int	i;
	char	*s;
	char	*str;
	char	cpio_cmd[_POSIX_PATH_MAX];
	int	pathok = 0;

	/* 
	 * Create a named cpio format file by calling the pre-defined utility.
	 * Return an SYSERROR if test cannot proceed and is deleted, 
	 * otherwise return 0.
	 *
	 * The syntax for the format creating utility is as follows:
	 * CR_CPIO_UTIL is the name of a command file which accepts
	 * an argument which is the name of the cpio format file and 
	 * a series of further arguments which are the files to be archived.
	 * 
	 * The command file is assumed to be in the TESTROOT/BIN directory.
	 * 
	 * The command file should invoke the format creating utility 
	 * to archive the files in the named cpio format file.
	 */

	DBUG_ENTER("create_cpio");

	globok = 0;

	(void) unlink(name);

	if(stat(name, &stbuf) != SYSERROR)
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable cpio format file %s already exists", name);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	cpio_name = CR_CPIO_UTIL;
	cpio_root = getenv("TET_EXECUTE");
	if((cpio_root == (char *) NULL) || (*cpio_root == '\0'))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("getenv(\"TET_EXECUTE\") failed");
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	(void) strcpy(cpio_cmd, cpio_root);
	(void) strcat(cpio_cmd, cpio_name);

	if((str = (char *)malloc(strlen(files) + 1)) == (char *)NULL)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("malloc() failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	(void) strcpy(str, files);

	i = 0;
	args[i++] = cpio_cmd;
	args[i++] = name;
	for(s = str; i < MAXARG; i++)
	{
		args[i] = strtok(s, " ");
		s = (char *)NULL;
	}

	ret = fork_util(ch_util, WAITTIME, FORMAT_CREATE);
	if (ret == PATH_NOK)
		ret = CPIO_OK;
	else
		PATH_TRACE;

	if(ret == SYSERROR || ret == A_EXECFAIL)
	{
		test_fail++;
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	if(ret == A_MISC)
	{
		free((void *)str);
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("\"%s\"", cpio_cmd);
		in_rpt("was called with incorrect arguments");
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	if(stat(name, &stbuf) == SYSERROR)
	{
		err = errno;
		free((void *)str);
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("\"%s\"", cpio_cmd);
		in_rpt("did not create cpio format file %s", name);
		in_rpt("stat(%s) failed - errno %d", name, err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(7);

	free((void *)str);
	DBUG_RETURN(0);
}

public int
open_cpio(name, files)
char	*name;	/* The name of the cpio format file to create. */
char	*files;	/* The files to be placed in the cpio format file. */
{
	int	err;
	int	fd;
	int	pathok = 0;

	/* 
	 * Create a named cpio format file by calling create_cpio().
	 * Return an open file descriptor to read it or SYSERROR if test 
	 * cannot proceed and is deleted.
	 */

	DBUG_ENTER("open_cpio");

	globok = 0;
	
	if(create_cpio(name, files) == SYSERROR)
	{
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_FUNC_TRACE;

	if((fd = open(name, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason - failed to open cpio format file");
		in_rpt("open(%s, O_RDONLY) failed - errno %d", name, err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(fd);
}

public int
close_cpio(name, fd)
char	*name;	/* The name of the cpio format file */
int	fd;	/* File descriptor returned by open_cpio() */
{
	/*
	 * Close an open cpio file and unlink the file created.
	 */

	DBUG_ENTER("close_cpio");

	(void) unlink(name);
	
	DBUG_RETURN(close(fd));
}

public int
read_head(fd)
int	fd;			/* File descriptor of cpio format file */
{
	unsigned char *ucp;
	int	err;
	int	bytes;
	int	pathok = 0;

	/*
	 * Read a cpio format header from given file into buffer
	 * and convert from ASCII to native characters.
	 */

	DBUG_ENTER("read_head");

	bytes = read(fd, headbuf, sizeof(headbuf));

	if(bytes == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("read() of cpio format header failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
	
	if(bytes != sizeof(headbuf))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("read() of cpio format header returned %d instead of %d",
					 bytes, sizeof(headbuf));
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	for (ucp = (unsigned char *)headbuf;
	     ucp < (unsigned char *)headbuf + sizeof(headbuf); ucp++)
		*ucp = from_asc[*ucp];

	DBUG_RETURN(0);
}

public int
read_name(fd, buf, len)
int	fd;		/* File descriptor of cpio format file */
char	*buf;		/* Pointer to buffer to read into */
int	len;		/* Length to read */
{
	unsigned char *ucp;
	int	err;
	int	bytes;

	/*
	 * Read a cpio format c_name field from given file into buffer
	 * and convert from ASCII to native characters.
	 */

	DBUG_ENTER("read_name");

	bytes = read(fd, buf, (unsigned)len);

	if(bytes == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("read() of cpio format \"c_name\" field failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	
	if(bytes != len)
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("read() of cpio format \"c_name\" field returned %d instead of %d",
					 bytes, len);
		DBUG_RETURN(SYSERROR);
	}

	for (ucp = (unsigned char *)buf;
	     ucp < (unsigned char *)buf + bytes; ucp++)
		*ucp = from_asc[*ucp];

	DBUG_RETURN(0);
}

public int
read_data(fd, buf, len)
int	fd;		/* File descriptor of cpio format file */
char	*buf;		/* Pointer to buffer to read into */
int	len;		/* Length to read */
{
	int	err;
	int	bytes;

	/*
	 * Read a cpio format field from given file into buffer.
	 */

	DBUG_ENTER("read_data");

	bytes = read(fd, buf, (unsigned)len);

	if(bytes == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("read() of cpio format field failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	
	if(bytes != len)
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("read() of cpio format field returned %d instead of %d",
					 bytes, len);
		DBUG_RETURN(SYSERROR);
	}

	DBUG_RETURN(0);
}

public int
read_rec(fd, headptr, name)
int	fd;		/* File descriptor of cpio format file */
struct	cpio_head *headptr;	/* Pointer to buffer for cpio format header */
char	*name;		/* Expected file name in the record */
{
	int	len;
	int	pathok = 0;

	/* Read cpio record consisting of header, c_name and c_filedata */

	DBUG_ENTER("read_rec");

	globok = 0;

	/* Read header for the regular file */
	if(read_head(fd) == SYSERROR) 
	{
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	len = aoctol(headptr->c_namesize, CNAMSIZLEN);

	if(read_name(fd, blk, len) == SYSERROR)
	{
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	if(!strncmp(blk, TRAILER, strlen(TRAILER)))
	{
		xx_rpt(DELETION);
		in_rpt("cpio format trailer record unexpectedly read");
		in_rpt("file name \"%s\" not archived", name);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	len = strlen(name) + 1;

	if(chk_bytes(blk, name, len))
	{
		xx_rpt(DELETION);
		in_rpt("incorrect \"c_name\" field in cpio format archive"); 
		in_rpt("expected value: %s; observed value: %s",
					name, print_field(len, blk));
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	/* Read without checking the file data */
	len = aoctol(headptr->c_filesize, CFILSIZLEN);

	if(read_data(fd, blk, len) == SYSERROR)
	{
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_RETURN(0);
}

public int
chk_bytes(p1, p2, len)
char	*p1;
char	*p2;
int	len;
{
	/*
	 * Compare bytes. 
	 * Just memcpy() but with arguments cast.
	 */
	return(memcmp((void *)p1, (void *)p2, (size_t)len));
}

public int
chk_data(name, data, len)
char	*name;		/* File to check */
char	*data;		/* Data to compare with */
int	len;		/* Size of data argument buffer */
{
	int	err;
	int	fd;
	int	bytes;

	/*
	 * Check the first block of data in a file contains "data".
	 * "data" must be a NULL terminated string.
	 */

	DBUG_ENTER("chk_data");

	if((fd = open(name, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d", name, err);
		DBUG_RETURN(SYSERROR);
	}

	bytes = read(fd, blk, CBLKSIZ);

	if(bytes == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("read() of regular file failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}

	if(bytes < len)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("read() of regular file returned %d bytes", bytes);
		in_rpt("instead of %d bytes", len);
		DBUG_RETURN(SYSERROR);
	}

	if(chk_bytes(blk, data, len))
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("data in regular file not restored from cpio archive");
		in_rpt("expected data: %s", data);
		in_rpt("observed data: %s", print_field(len, blk));
		DBUG_RETURN(SYSERROR);
	}

	DBUG_RETURN(0);
}

public int
new_cpio(name)
char	*name;	/* The name of the cpio format file to create. */
{
	int	err;
	int	fd;

	/* 
	 * Create an empty cpio format file.
	 * Return an open file descriptor to write it or SYSERROR if test 
	 * cannot proceed and is deleted.
	 */

	DBUG_ENTER("new_cpio");

	if((fd = creat(name, MODEANY)) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason - failed to create empty cpio file");
		in_rpt("creat(%s) failed - errno %d", name, err);
		DBUG_RETURN(SYSERROR);
	}

	DBUG_RETURN(fd);
}

public int
init_head(name, statptr, headptr, type)
char	*name;			/* name of file to archive */
struct	stat *statptr;		/* stat structure of file to archive */
struct	cpio_head *headptr;	/* Pointer to buffer for cpio format header */
mode_t	type;			/* file type flag C_ISREG, etc */
{
	uid_t	euid = geteuid();
	gid_t	egid = getegid();

	/*
	 * Initialise common things needed when writing cpio header to an
	 * empty cpio format file.
	 */

	(void) strncpy(headptr->c_magic, CMAGIC, (size_t)CMAGLEN);

	ltoaoct((unsigned long)statptr->st_dev, headptr->c_dev, CDEVLEN);
	ltoaoct((unsigned long)statptr->st_ino, headptr->c_ino, CINOLEN);

	ltoaoct((unsigned long)(type | C_MODEANY), headptr->c_mode, CMODELEN);

	ltoaoct((unsigned long)euid, headptr->c_uid, CUGIDLEN);
	ltoaoct((unsigned long)egid, headptr->c_gid, CUGIDLEN);

	ltoaoct((unsigned long)statptr->st_nlink, headptr->c_nlink, CNLINKLEN);
	ltoaoct((unsigned long)0, headptr->c_rdev, CRDEVLEN);
	ltoaoct((unsigned long)statptr->st_mtime, headptr->c_mtime, CMTIMELEN);
	ltoaoct((unsigned long)(strlen(name)+1), headptr->c_namesize, CNAMSIZLEN);

	/* Size is zero for anything other than regular files */
	ltoaoct((unsigned long)statptr->st_size, headptr->c_filesize, CFILSIZLEN);
	if(!(S_ISREG(statptr->st_mode)))
		ltoaoct((unsigned long)0, headptr->c_filesize, CFILSIZLEN);
}

public int
write_head(fd)
int	fd;			/* File descriptor of cpio format file */
{
	unsigned char *ucp1, *ucp2;
	char	convbuf[CHEADSIZE];
	int	err;
	int	bytes;
	int	pathok = 0;

	/*
	 * Write a cpio format header from buffer into given file.
	 * Convert to ASCII before writing.
	 */

	DBUG_ENTER("write_head");

	for (ucp1 = (unsigned char *)headbuf, ucp2 = (unsigned char *)convbuf;
	     ucp1 < (unsigned char *)headbuf + sizeof(headbuf); ucp1++, ucp2++)
	{
		if (*ucp1 != 0 && to_asc[*ucp1] == 0)
		{
			/* Should never happen! */
			vsxerror("write_name() cannot convert character 0x%x to ASCII", *ucp1);
#ifdef DEBUG_HEADER
			vsxerror("at offset %ld in following header",
				(long)(ucp1 - (unsigned char *)headbuf));
			write(2, headbuf, sizeof(headbuf));
#endif
		}
		*ucp2 = to_asc[*ucp1];
	}

	bytes = write(fd, convbuf, sizeof(convbuf));

	if(bytes == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("write() of cpio format header failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
	
	if(bytes != sizeof(convbuf))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("write() of cpio format header returned %d instead of %d",
					 bytes, sizeof(convbuf));
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	DBUG_RETURN(0);
}

public int
write_name(fd, buf, len)
int	fd;		/* File descriptor of cpio format file */
char	*buf;		/* Pointer to buffer to write from */
int	len;		/* Length to write */
{
	unsigned char *ucp1, *ucp2;
	int	err;
	int	bytes;
	int	pathok = 0;
	char	*namebuf;

	/*
	 * Write a cpio format c_name field from buffer into given file.
	 * Convert to ASCII before writing.
	 */

	DBUG_ENTER("write_name");

	/* buf[] may point to a string literal, so allocate new buffer
	   to receive ASCII conversion */

	if ((namebuf = (char *)malloc((size_t)len)) == NULL)
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("malloc(%d) failed in write_name()", len);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	for (ucp1 = (unsigned char *)buf, ucp2 = (unsigned char *)namebuf;
	     ucp1 < (unsigned char *)buf + len; ucp1++, ucp2++)
	{
		if (*ucp1 != 0 && to_asc[*ucp1] == 0)
		{
			/* Should never happen! */
			vsxerror("write_name() cannot convert character 0x%x to ASCII", *ucp1);
		}
		*ucp2 = to_asc[*ucp1];
	}

	bytes = write(fd, namebuf, (unsigned)len);
	err = errno;

	free((void *)namebuf);

	if(bytes == SYSERROR)
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("write() of cpio format \"c_name\" field failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
	
	if(bytes != len)
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("write() of cpio format \"c_name\" field returned %d instead of %d",
					 bytes, len);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	DBUG_RETURN(0);
}

public int
write_data(fd, buf, len)
int	fd;		/* File descriptor of cpio format file */
char	*buf;		/* Pointer to buffer to write from */
int	len;		/* Length to write */
{
	int	err;
	int	bytes;
	int	pathok = 0;

	/*
	 * Write a cpio format field from buffer into given file.
	 */

	DBUG_ENTER("write_data");

	bytes = write(fd, buf, (unsigned)len);

	if(bytes == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("write() of cpio format field failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
	
	if(bytes != len)
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("write() of cpio format field returned %d instead of %d",
					 bytes, len);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	DBUG_RETURN(0);
}

public int
write_trail(fd, statptr, headptr)
int	fd;			/* File descriptor of cpio format file */
struct	stat 	*statptr;	/* Pointer to basis stat structure */
struct	cpio_head *headptr;	/* Pointer to buffer for cpio format header */
{
	int	len;
	long	offset;
	int	pathok = 0;

	/*
	 * Write a cpio format trailer record.
	 */

	DBUG_ENTER("write_trail");

	globok = 0;

	statptr->st_size = 0L;
	init_head(TRAILER, statptr, headptr, (mode_t)C_ISREG);

	if(write_head(fd) == SYSERROR)
		DBUG_RETURN(SYSERROR);
	else
		PATH_TRACE;

	len = strlen(TRAILER) + 1;
	if(write_name(fd, TRAILER, len) == SYSERROR)
		DBUG_RETURN(SYSERROR);
	else
		PATH_TRACE;

	/* Fill out to a block in case format reading utility reads blocks */
	offset = lseek(fd, (off_t)0L, SEEK_CUR);
	len = CBLKSIZ - offset % CBLKSIZ;
	if(write_data(fd, blk, len) == SYSERROR)
		DBUG_RETURN(SYSERROR);
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

public int
chk_file(name)
char	*name;
{
	struct	stat stbuf;
	int	err;
	int	pathok = 0;

	/*
	 * Check that the given file exists.
	 */

	DBUG_ENTER("chk_file");

	if(stat(name, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		if(err == ENOENT)
			in_rpt("file %s is not available for this test", name);
		in_rpt("stat(%s) failed - errno %d", name, err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	DBUG_RETURN(0);
}

public int
chk_name(fd, name)
int	fd;		/* File descriptor of cpio format file */
char	*name;		/* Expected file name in the record */
{
	unsigned char *ucp;
	int	err;
	int	ret;
	int	len;
	int	pathok = 0;

	DBUG_ENTER("chk_name");

	/* Try to read the file name and see if it's trailer record */
	len = strlen(name) + 1;
	if(len < strlen(TRAILER))
		len = strlen(TRAILER);

	if((ret = read(fd, blk, (unsigned)len)) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("read() of cpio format \"c_name\" field failed - errno %d",
									err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	for (ucp = (unsigned char *)blk;
	     ucp < (unsigned char *)blk + ret; ucp++)
		*ucp = from_asc[*ucp];

	if(!strncmp(blk, TRAILER, strlen(TRAILER)))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("cpio format trailer record unexpectedly read");
		in_rpt("file name \"%s\" not archived", name);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	len = strlen(name) + 1;

	if(ret < len)
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unable to read \"c_name\" field in cpio format archive"); 
		in_rpt("read() returned %d; expected field length %d", ret, len);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	if(chk_bytes(blk, name, len))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("incorrect \"c_name\" field in cpio format archive"); 
		in_rpt("expected value: %s; observed value: %s",
					name, print_field(len, blk));
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	DBUG_RETURN(0);
}

public int
chk_oct(str, len)
char	*str;	/* Pointer to octal number */
int	len;	/* Maximum length of the field */
{
	int	i;

	/*
	 * Check cpio format field is a leading zero filled octal number.
	 */

	DBUG_ENTER("chk_oct");

	for(i = 0; i < len; str++, i++)
	{
		if(!isdigit(*str))
			DBUG_RETURN(SYSERROR);
		if(*str == '8' || *str == '9')
			DBUG_RETURN(SYSERROR);
	}

	DBUG_RETURN(0);
}

public char *
print_field(len, str)
int	len;	/* Maximum length of the field */
char	*str;	/* Pointer to start of field */
{
	private	char	buf[CBLKSIZ];
	char	*s = buf;
	int	i;

	/*
	 * Return a pointer to as much as is printable of a field. 
	 */

	for(i = 0; i < len && i < CBLKSIZ - 1; str++, i++)
	{
		if(!isprint(*str))
			break;
		*s++ = *str;
	}
	*s = '\0';

	return(buf);
}

/*
 * This is a modified version of the library function cppair().
 */

int fork_util(childproc, waittime, name)
void (*childproc) ();
int	waittime;
char	*name;
{
	int	rtval;
	int	err;
	int	status;
	pid_t	savchild;
	int	pathok = 0;

	DBUG_ENTER("fork_util");

	globok = 0;

	(void) fflush(stdout);
	(void) fflush(stderr);

	savchild = tet_child;

	switch (tet_child = fork())
	{
	
	case SYSERROR:
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fork %s", name);
		tet_child = savchild;
		DBUG_RETURN(SYSERROR);

	case 0:
		tet_setcontext();
		if (DBUG_SETJMP(sjmp) != 0)
		{
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("deletion reason: signal %d received", signum);
			exit(CPIO_NOK); /* not DBUG_EXIT */
		}
		setsigs(sjmpfunc);
		(*childproc) ();
		exit(CPIO_OK); /* not DBUG_EXIT */

	default:
		tet_setblock();
		PATH_TRACE;
		SET_TIMEOUT(waittime)
		rtval = waitpid(tet_child, &status, 0);
		CLEAR_ALARM

		if (rtval == SYSERROR)
		{
			err = errno;
			if(!test_fail++)
				xx_rpt(DELETION);
			if (alrm_flag > 0)
				in_rpt("deletion reason: %s timed out", name);
			else
				in_rpt("deletion reason: waitpid() failed - errno %d (%s)",
					err, errname(err));
			(void) killchild(tet_child, KILLWAIT);
		}
		else if (WIFEXITED(status))
		{
			status = WEXITSTATUS(status);
			if (status == CPIO_OK || status == A_EXECFAIL || status == A_MISC)
			{
				tet_child = savchild;
				if ((pathok != 1) && (status == CPIO_OK))
					status = PATH_NOK;
				DBUG_RETURN(status);
			}
			else
			{
				if(!test_fail++)
					xx_rpt(DELETION);
				in_rpt("deletion reason: %s gave unexpected exit code %d", name,
					status);
			}
		}
		else if (WIFSIGNALED(status))
		{
			status = WTERMSIG(status);
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("deletion reason: %s was terminated by signal %d (%s)", name,
				status, signame(status));
		}
		else if (WIFSTOPPED(status))
		{
			status = WSTOPSIG(status);
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("deletion reason: %s was stopped by signal %d (%s)", name,
				status, signame(status));
			(void) killchild(tet_child, KILLWAIT);
		}
		else
		{
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("deletion reason: %s returned bad wait status (%#x)", name,
				status);
		}

		tet_child = savchild;
		DBUG_RETURN(SYSERROR);
	}
}

private void
sjmpfunc(sig)
int	sig;
{
	DBUG_ENTER("sjmpfunc");
	signum = sig;
	DBUG_LONGJMP(sjmp, 1);
	DBUG_VOID_RETURN;
}

