/*
 *      SCCS:  @(#)  POSIX.os/files/cpio/cpio_gid.c Rel 4.4.1 (06/09/97)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#define MAIN
static	char sccsid[] = "@(#)POSIX.os/files/cpio/cpio_gid.c Rel 4.4.1 (06/09/97)";
#ifdef	MAIN
char	*copyright[] = {
		"(C) Copyright 1992 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		cpio_gid - used by T.cpio to mount/unmount a file system
		with GrpID=True

PROJECT:	XNFS (X/OPEN NFS tests)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int tet_main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	Set-gid subprogram for cpio tests.
	Must be have appropriate privileges to mount
	and unmount a file system.

	usage:
		progname mygid vg_ident vg_dbug

	where mygid is my effective gid

AUTHOR:		Calum Laing, UniSoft Ltd.
DATE CREATED:	24 November 1992

MODIFICATIONS:

************************************************************************/

#include <std.h>
#include <dbug.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include <xnfs.h>
#include "cpiotest.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	void	set_pname();
extern	char	*errname();

public	int
tet_main(argc, argv)
int argc;
char **argv;
{
	int	action;
	char	*fsys;
	char	*t31_mnt;

	set_pname(argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(NOK);
	}

	action = atoi(argv[1]);
	t31_mnt = argv[2];

	if (argc >= 5)
		vg_dbug = argv[4];

	if (argc >= 4)
		vg_ident = argv[3];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug + 2);
	}

        if ((fsys = tet_getvar("VSX_MOUNT_DEV")) == NULL || *fsys == '\0')
        {
                xx_rpt(DELETION);
                in_rpt("deletion reason: VSX_MOUNT_DEV not set");
		return(NOK);
        }

	switch (action)
	{
	case MOUNT_GIDTRUE:
		if (xnfs_mount(fsys, t31_mnt, XNFS_MOUNT_GRPID | XNFS_MOUNT_INTR) < 0)
		{
			xx_rpt(DELETION);
			return(NOK);
		}
		break;
		
	case UNMOUNT_GIDTRUE:
		(void) xnfs_unmount(fsys, t31_mnt);
		(void) rmdir(t31_mnt);
		break;

	default:
		xx_rpt(DELETION);
		in_rpt("bad parameter passed: %d", action);
		break;
	}

	return(OK);
}
