/*
*      SCCS:  @(#)  POSIX.os/dataform/cpio/cpio_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/dataform/cpio - cpio_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
	"(C) Copyright 1991 X/Open Company Limited",
	"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		cpio_su - used by T.cpio in several tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	tet_main()

SYNOPSIS:
	int main(argc, argv)
		int argc;
		char **argv;

DESCRIPTION:
  Exec'd by T.cpio
  usage:
	cpio_su teststr euid egid expuid expgid vg_ident vg_dbug

  where teststr is the number of the test,
  euid is the effective user id that is to be used in the test,
  egid is the effective group id that is to be used in the test,
  expuid is the expected user id of the extracted file,
  expgid is the expected group id of the extracted file.
METHODS:
ASSUMES:  cpio_su must be setuid root.
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, Unisoft Ltd.
DATE CREATED:	Tue Jun  6 15:25:34 BST 1989
MODIFICATIONS:
	Stuart Boutell, 5 Nov 1990
		Changed vg_env to environ.

	Stuart Boutell, 6 Nov 1990
		Eliminated vg_file

	Peter Skan, 11 Sept 1991
		Set the CHOWN and ASSIGN privileges
		Assign privilege to cpio_uids using prv_assign()

	Calum Laing, Mon Nov 23 14:19:19 GMT 1992
		Changed the tet_exec() call to be in a child
		process for the benefit of the DTET when testing XNFS.

	Geoff Clare, 21 March 1994
		Changed size of arv[] from 5 to 6.

	Geoff Clare, 1 Nov 1995
		Allow for systems where super-user ID is not 0.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<errno.h>
#include	"cpiotest.h"
#include 	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


extern	void	set_pname();

extern	char	**environ;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private char	*prog_uids = "cpio_uids"; /* Will be re-assigned to the full execpath */

private char	*arv[6];

private	void	main_ch();
private	void	subtest();

private	int	privileges[] = {
PRV_CHOWN,
PRV_SETID,
0
};


public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{

	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		  "cpio_su" : (argv[0]));

	if (argc == 9)
	{
		if (argv[8] != (char *) NULL)
			vg_dbug = argv[8];
		argc--;
	}

	if (argc == 8)
	{
		if (argv[7] != (char *) NULL)
			prog_uids = argv[7];
		argc--;
	}
	if (argc == 7)
	{
		if (argv[6] != (char *) NULL)
			vg_ident = argv[6];
		argc--;
	}
	if (argc != 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count (%d) wrong", argc);
		return(OK);
	}
	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if (setprv(PRV_CHOWN) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_CHOWN) failed");
		return(OK);
	}

	if (setprv(PRV_ASSIGN) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_ASSIGN) failed");
		return(OK);
	}

	arv[1] = argv[1];
	arv[2] = argv[2];
	arv[3] = argv[3];
	arv[4] = argv[4];
	arv[5] = argv[5];
	
	/*
	 * We cannot call tet_exec() from a tet_exec()'d process 
	 * unless the tet_exec is done from a new process.
	 */

	(void) tet_fork(main_ch, NULLFN, WAITTIME, 1);

	return(OK);
}

private void
main_ch()
{
	subtest(arv[1], arv[2], arv[3], arv[4], arv[5]);
}

private void
subtest(teststr, eustr, egstr, expustr, expgstr)
char	*teststr;
char	*eustr;
char	*egstr;
char	*expustr;
char	*expgstr;
{
	char	*args[9];
	uid_t	euid;
	gid_t	egid;
	int	err;

	DBUG_ENTER("subtest");

	euid = atoi(eustr);
	egid = atoi(egstr);

	if (euid == SU_UID)
		euid = geteuid(); /* might not be the same as SU_UID !*/

	/* Make the owner and mode correct for the file exec'ed */
	if(chown(prog_uids, euid, egid) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chown() failed - errno %d", err);
		DBUG_TET_EXIT(OK);
	}
	if(chmod(prog_uids, (mode_t)SU_MODE) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod() failed - errno %d", err);
		DBUG_TET_EXIT(OK);
	}

	if (euid == geteuid()) {
		/*
		 * Assign privileges to the file exec'ed
		 * Do this after the chown and chmod, since some systems
		 * clear privileges if the file attributes are changed.
		 */
		if (prv_assign(prog_uids, privileges) != 0) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("prv_assign() failed - errno %d", err);
			DBUG_TET_EXIT(OK);
		}
	}

	args[0] = prog_uids;
	args[1] = teststr;
	args[2] = eustr;
	args[3] = egstr;
	args[4] = expustr;
	args[5] = expgstr;
	args[6] = vg_ident;
	args[7] = vg_dbug;
	args[8] = (char *)NULL;
	(void) tet_exec(prog_uids, args, environ);
	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s,..) failed errno %d", prog_uids, errno);

	DBUG_VOID_RETURN;
}
