/*
*      SCCS:  @(#)  POSIX.os/dataform/tar/tar.c	Rel 4.4.4	    (07/19/99)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/dataform/tar - T.tar Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.tar

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the IEEE Std. 1003.1-1988 "Extended tar format"

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu May 18 09:00:23 BST 1989
MODIFICATIONS:
		Geoff Clare, 11 Apr 1990
			New long pathname in linkname error test.
		
		Geoff Clare, 28 June 1990
			Add AREGTYPE extraction test
			Add FIFOTYPE "size ignored" test
		
		Geoff Clare, 4 July 1990
			Test full length name, linkname and prefix fields

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 9 June 1992
			Add portable filename test.

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		Calum Laing, Mon Nov 23 14:19:19 GMT 1992
			XNFS enhancements

		Geoff Clare, 7 June 1995
			Convert headers to/from ASCII.

		Geoff Clare, 9 April 1999
			to_asc[] was 1 byte too small.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<limits.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<tar.h>
#include	<dbug.h>
#include	<report.h>
#include	<pathres.h>
#include	"tartest.h"
#include 	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


#define NO_TESTS	40

public	int	no_tests = NO_TESTS;

extern	unsigned char	from_asc[128];
extern	unsigned char	to_asc[UCHAR_MAX+1];

extern	char	*tprefix;
extern	char	*treg_file;
extern	char	*tlink_file;
extern	char	*tlinked_file;
extern	char	*tdir_file;
#if !TEST_XNFS
extern	char	*tfifo_file;
#endif /* TEST_XNFS */
extern	char	*tnew_file;
extern	char	*tnewlink_file;
extern	char	*tnew_dir;
extern	char	*tnew_fifo;
extern	char	*tlonglink_file;

private void	prepare_tests();
private void	cln_tests();

extern void	test1();
extern void	test2();
extern void	test3();
extern void	test4();
extern void	test5();
extern void	test6();
extern void	test7();
extern void	test8();
extern void	test9();
extern void	test10();
extern void	test11();
extern void	test12();
extern void	test13();
extern void	test14();
extern void	test15();
extern void	test16();
extern void	test17();
extern void	test18();
extern void	test19();
extern void	test20();
extern void	test21();
extern void	test22();
extern void	test23();
extern void	test24();
extern void	test25();
extern void	test26();
extern void	test27();
extern void	test28();
extern void	test29();
extern void	test30();
extern void	test31();
extern void	test32();
extern void	test33();
extern void	test34();
extern void	test35();
extern void	test36();
extern void	test37();
extern void	test38();
extern void	test39();
extern void	test40();

private void	t_pass();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	test3, 3,
	test4, 4,
	test5, 5,
	test6, 6,
	test7, 7,
	test8, 8,
	test9, 9,
	test10, 10,
	test11, 11,
	test12, 12,
	test13, 13,
	test14, 14,
	test15, 15,
	test16, 16,
	test17, 17,
	test18, 18,
	test19, 19,
	test20, 20,
	test21, 21,
	test22, 22,
	test23, 23,
	test24, 24,
	test25, 25,
	test26, 26,
	test27, 27,
	test28, 28,
	test29, 29,
	test30, 30,
	test31, 31,
	test32, 32,
	test33, 33,
	test34, 34,
	test35, 35,
	test36, 36,
	test37, 37,
	test38, 38,
	test39, 39,
	test40, 40,
	NULL, 0
};

private void
prepare_tests()
{
	int	fd, i;

	DBUG_ENTER("prepare_tests");

	if(uids()!=0)
	{
		int t;
		for(t=1; t <= no_tests; t++)
		{
			tet_delete(t, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	(void) umask((mode_t)0);

	fd = crfile(treg_file, (mode_t)MODEANY, (int)geteuid(), (int)getegid());
	if(fd >= 0) {
		/* Write something to make it not a zero length file */
		(void) write(fd, REGDATA, (unsigned) strlen(REGDATA));
	}
	(void) close(fd);
	fd = crfile(tlink_file, (mode_t)MODEANY, (int)geteuid(), (int)getegid());
	if(fd >= 0) {
		/* Write something to make it not a zero length file */
		(void) write(fd, LNKDATA, (unsigned) strlen(LNKDATA));
	}
	(void) close(fd);
	(void) link(tlink_file, tlinked_file);
	(void) crdir(tdir_file, (mode_t)MODEANY, (int)geteuid(), (int)getegid());
#if !TEST_XNFS
	(void) unlink(tfifo_file);
	(void) mkfifo(tfifo_file, MODEANY);
#endif /* TEST_XNFS */

	/* Initialise native-chars-to-ASCII conversion table */
	for (i = 1; i < 128; i++)
		if (from_asc[i] != 0)
			to_asc[from_asc[i]] = i;

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	char **p;
	private	char	pathname[TBLKSIZ];

	DBUG_ENTER("cln_tests");

	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);

	(void) strcpy(pathname, tprefix);
	(void) strcat(pathname, SLASH);
	(void) strcat(pathname, tnew_file);

	(void) unlink(treg_file);
	(void) unlink(tlink_file);
	(void) unlink(tlinked_file);
	(void) rmdir(tdir_file);
#if !TEST_XNFS
	(void) unlink(tfifo_file);
#endif /* TEST_XNFS */
	(void) unlink(tnew_file);
	(void) unlink(tnewlink_file);
	(void) unlink(pathname);
	(void) rmdir(tprefix);
	(void) rmdir(tnew_dir);
	(void) unlink(tnew_fifo);
	(void) unlink(tlonglink_file);

	while (chdir("12345678901234") == 0)
		;
	(void) unlink("1234567890");
	(void) chdir("..");
	while (chdir("12345") != 0 && rmdir("12345678901234") == 0)
		(void) chdir("..");
	while (chdir("12345678901234") == 0)
		;
	(void) unlink("1234");
	(void) chdir("..");
	while (rmdir("12345678901234") == 0)
	{
		(void) rmdir("12345");
		(void) unlink("123456789");
		(void) unlink("1234567890");
		(void) unlink("12345678901");
		(void) chdir("..");
	}

	DBUG_VOID_RETURN;
}
