/*
*      SCCS:  @(#)  POSIX.os/dataform/tar/tar1.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.tar

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the IEEE Std. 1003.1-1988 "Extended tar format"
	This file contains the tests involving creating tar format archives.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu May 18 09:00:23 BST 1989
MODIFICATIONS:
		Geoff Clare, 12 Oct 1989
			Allow extra blocks after trailer in test17
		
		Geoff Clare, 31 Jan 1990
			Change linkname error test from 100 to 101 characters

		Geoff Clare, 11 Apr 1990
			New long pathname in linkname error test.

		Geoff Clare, 4 July 1990
			Test full length name, linkname and prefix fields

		Anthony Lee, 26 July 1990
			Added resilience code.
		
		Geoff Clare, 4 Oct 1990
			Allow "unsup" value for VSX_(BLK|CHR)DEV_FILE

		Stuart Boutell, 2 Nov 1990
			TESTROOT -> TET_EXECUTE + use of getenv

		Geoff Clare, 18 Feb 1991
			Changes to remove assumption that tar_head
			members are contiguous.

		Stuart Boutell, 5 Apr 1991
			Deleted test for ability to create archive file.

		Geoff Clare, 18 July 1991
			Reduce full linkname length to 99 in XPG3 mode.

		Geoff Clare, 21 Apr 1992
			Fix expected path count in test7.

		Geoff Clare, 9 June 1992
			Move full name/prefix/linkname testing from test 7
			(a creation test) to tests 10 & 27 (extraction tests)

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		Calum Laing, Mon Nov 23 14:19:19 GMT 1992
			XNFS enhancements

		Geoff Clare, 25 May 1993
			Reinstate "/" before last pathname component in
			tlong_file (test 12).

		Geoff Clare, 11 March 1994
			Implement interpretation VWG/006/063092.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<limits.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<grp.h>
#include	<pwd.h>
#include	<ctype.h>
#include	<tar.h>
#include	<dbug.h>
#include	<exitcodes.h>
#include	<report.h>
#include	"tartest.h"
#include 	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


#undef memchr
extern void	*memchr();

extern	long	aoctol();
extern	long	calc_sum();
extern	int	open_tar();
extern	int	read_head();
extern	int	chk_head();
extern	int	read_trail();
extern	int	read_next();
extern	int	chk_trail();
extern	int	chk_file();
extern	char	*print_field();
extern	void	ch_util();

extern int	test_fail;
extern int	may_error, got_error;

#ifndef getenv
extern	char	*getenv();
#endif

extern	char	*treg_file;
extern	char	*tlink_file;
extern	char	*tlinked_file;
extern	char	*tlink_both;
extern	char	*tdir_file;
#if !TEST_XNFS
extern	char	*tfifo_file;
#endif /* TEST_XNFS */
extern	char	*tlonglink_file;
extern	char	*tar_file;
extern	struct	tar_head hdr;
extern	struct	tar_trail trl;
extern	char	headbuf[TBLKSIZ];
extern	char	blk[];

#if !TEST_XNFS
private	void	t17_chr();
private	void	t17_blk();
#endif /* TEST_XNFS */

private	void	t17_chk();
private	void	t17_dir();
#if !TEST_XNFS
private	void	t17_fifo();
#endif

public void
test2()
{
	int	tar_fd;
	int	errret;
	int	blks;
	int	ret;
	int	err;
	int	pathok = 0;

	/*
	 * Test that the tar format archive contains fixed size 512 byte blocks.
	 * (TBLKSIZ is defined in tartest.h)
	 */

	DBUG_ENTER("test2");

	test_fail = 0;

	if(chk_file(treg_file) == SYSERROR)
		DBUG_VOID_RETURN;

	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	blks = errret = 0;
	while((ret = read(tar_fd, blk, TBLKSIZ)) > 0) 
	{
		if(ret != TBLKSIZ)
		{
			errret = ret;
			break;
		}
		blks++;
	}

	if(ret == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("read() failed - errno %d", err);
		in_rpt("after reading %d %d byte blocks", blks, TBLKSIZ);
	}
	else if(errret)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d instead of %d", ret, TBLKSIZ);
		in_rpt("after reading %d %d byte blocks", blks, TBLKSIZ);
	}
	else
	{
		PATH_TRACE; 
		xs_rpt();
	}
	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

public void
test3()
{
	int	tar_fd;
	int	ret;
	int	err;
	int	pathok = 0;

		
	/*
	 * Test that for each file archived there is a header block and one 
	 * or more data blocks.
	 */

	DBUG_ENTER("test3");

	test_fail = 0;

	globok = 0; 
	if(chk_file(treg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	ret = read(tar_fd, headbuf, sizeof(headbuf));
	if(ret == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("reading tar header, read() failed - errno %d", err);
		test_fail++;
	}
	else if(ret == 0)
	{
		xx_rpt(FAILURE);
		in_rpt("no header or data blocks created for archived file");
		test_fail++;
	}
	else if(ret != sizeof(headbuf))
	{
		xx_rpt(DELETION);
		in_rpt("reading tar header, read() returned %d", ret);
		in_rpt("instead of %d", sizeof(headbuf));
		test_fail++;
	}
	else if(hdr.tar_name[0] == '\0')
	{
		xx_rpt(FAILURE);
		in_rpt("header block did not contain file name information");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(test_fail)
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}

	ret = read(tar_fd, blk, TBLKSIZ);
	if(ret == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("reading data blocks, read() failed - errno %d", err);
	}
	else if(ret == 0)
	{
		xx_rpt(FAILURE);
		in_rpt("no data blocks created for archived file");
	}
	else if(ret != TBLKSIZ)
	{
		xx_rpt(DELETION);
		in_rpt("reading data blocks, read() returned %d", ret);
		in_rpt("instead of %d", TBLKSIZ);
	}
	else if(strncmp(blk, REGDATA, strlen(REGDATA)))
	{
		xx_rpt(FAILURE);
		in_rpt("data block did not contain original file contents");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	(void) close_tar(tar_file, tar_fd);
	
	DBUG_VOID_RETURN;
}

public void
test4()
{
	int	tar_fd;
	off_t	offset;
	int	err;
	int	pathok = 0;
	
	/*
	 * Test that at the end of the tar format archive are two blocks
	 * filled with binary zeros.
	 */

	DBUG_ENTER("test4");

	test_fail = 0;

	globok = 0; 
	if(chk_file(treg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	if((offset = lseek(tar_fd, (off_t)0, SEEK_END)) == (off_t)SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("lseek(SEEK_END) failed - errno %d", err);
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	offset -= (off_t) sizeof(trl.tar_zeros);
	if(lseek(tar_fd, offset, SEEK_SET) == (off_t)SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("lseek(SEEK_SET) failed - errno %d", err);
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	globok = 0; 
	if(read_trail(tar_fd, &trl) == SYSERROR)
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	if(chk_trail(&trl) == SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("two blocks at end of archive did not contain zeros");
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(5);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

public void
test6()
{
	int	tar_fd;
	int	pathok = 0;
	
	/*
	 * Test that "magic", "uname", and "gname" are NULL terminated
	 * character fields.
	 */

	DBUG_ENTER("test6");

	test_fail = 0;

	globok = 0; 
	if(chk_file(treg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Read header for the regular file */
	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	if(chk_null(hdr.tar_magic, TMAGLEN)) 
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"magic\" field in tar format archive not NULL terminated");
	}
	else
		PATH_TRACE;
		
	if(chk_null(hdr.tar_uname, TUGNAMLEN)) 
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"uname\" field in tar format archive not NULL terminated");
	}
	else
		PATH_TRACE;
		
	if(chk_null(hdr.tar_gname, TUGNAMLEN)) 
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"gname\" field in tar format archive not NULL terminated");
	}
	else
		PATH_TRACE;
		
	if(!test_fail)
	{
		PATH_TRACE; 
		PATH_XS_RPT(6); 
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

public void
test7()
{
	int	tar_fd;
	int	pathok = 0;
	
	/*
	 * Test that "name", "linkname", and "prefix" are NULL terminated
	 * character fields except when all characters are not NULL.
	 */

	DBUG_ENTER("test7");

	test_fail = 0;

	/* First the simple case - small name, empty prefix & linkname */

	globok = 0; 
	if(chk_file(treg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Read header for the regular file */
	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	if(chk_nnull(hdr.tar_name, TNAMLEN)) 
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"name\" field in tar format archive not character field");
	}
	else
		PATH_TRACE;
		
	if(chk_nnull(hdr.tar_linkname, TLNKLEN)) 
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"linkname\" field in tar format archive not character field");
	}
	else
		PATH_TRACE;
		
	if(chk_nnull(hdr.tar_prefix, TPREFLEN)) 
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"prefix\" field in tar format archive not character field");
	}
	else
		PATH_TRACE;
		
	(void) close_tar(tar_file, tar_fd);

	if(!test_fail)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

public void
test8()
{
	int	tar_fd;
	int	pathok = 0;
	
	/*
	 * Test that "version" contains "00".
	 */

	DBUG_ENTER("test8");

	test_fail = 0;

	globok = 0; 
	if(chk_file(treg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Read header for the regular file */
	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if(strncmp(hdr.tar_version, TVERSION, (size_t)TVERSLEN))
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"version\" field \"%s\" in tar format archive is not \"%s\"",
			print_field(TVERSLEN, hdr.tar_version), TVERSION);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3); 
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

private void
do_chk_oct(str, len, name)
char	*str;	/* Field value to check */
int	len;	/* Field length */
char	*name;	/* Field name */
{
	DBUG_ENTER("do_chk_oct");

	if(chk_oct(str, len) == SYSERROR)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"%s\" field \"%s\" is not a zero-filled octal number",
			name, print_field(len, str));
	}

	DBUG_VOID_RETURN;
}

public void
test9()
{
	int	tar_fd;
	int	pathok = 0;
	
	/*
	 * Test that "mode", "uid", "gid", "size", "mtime", "chksum",
	 * "devmajor" and "devminor" are leading zero filled octal numbers 
	 * represented in ASCII terminated by one or more space or NULL
	 * characters.
	 */

	DBUG_ENTER("test9");

	test_fail = 0;

	globok = 0; 
	if(chk_file(treg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Read header for the regular file */
	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	do_chk_oct(hdr.tar_mode, TMODELEN, "mode");
	do_chk_oct(hdr.tar_uid, TUGIDLEN, "uid");
	do_chk_oct(hdr.tar_gid, TUGIDLEN, "gid");
	do_chk_oct(hdr.tar_size, TSIZLEN, "size");
	do_chk_oct(hdr.tar_mtime, TMTIMELEN, "mtime");
	do_chk_oct(hdr.tar_chksum, TSUMLEN, "chksum");
	do_chk_oct(hdr.tar_devmajor, TDEVLEN, "devmajor");
	do_chk_oct(hdr.tar_devminor, TDEVLEN, "devminor");

	if(!test_fail)
	{
		PATH_TRACE; 
		PATH_XS_RPT(3);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

public void
test12()
{
	int	tar_fd, i;
	char	tlong_file[102];
	char	tlong_both[120];
	int	pathok = 0;
	
	/*
	 * Test that a linked file is not archived if the "linkname"
	 * field would be more than 100 (TLNKLEN) characters.
	 * This is tested by creating a link to a file whose name is greater
	 * than TLNKLEN characters in length.
	 */

	DBUG_ENTER("test12");

	test_fail = 0;

	tlong_file[0] = '\0';
	for (i = 0; i < 6; i++)
	{
		(void) strcat(tlong_file, "12345678901234");
		if (crdir(tlong_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory %s", tlong_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		(void) strcat(tlong_file, "/");
	}
	(void) strcat(tlong_file, "12345678901");

	/* length is now (6 * 15) + 11 = 101 */

	(void) strcpy(tlong_both, tlong_file);
	(void) strcat(tlong_both, " ");
	(void) strcat(tlong_both, tlonglink_file);

	tar_fd = crfile(tlong_file,
		(mode_t)MODEANY, (int)geteuid(), (int)getegid());
	(void) close(tar_fd);
	(void) unlink(tlonglink_file);
	(void) link(tlong_file, tlonglink_file);

	globok = 0; 
	if(chk_file(tlong_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0; 
	if(chk_file(tlonglink_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	may_error = 1;	/* utility exit with error is OK */
	if((tar_fd = open_tar(tar_file, tlong_both)) == SYSERROR)
		DBUG_VOID_RETURN;

	if (got_error != 0)
	{
		xx_rpt(RFI);
		in_rpt("create utility exited with code %d", got_error);
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}

	/* Read header for the original file */
	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		

	/* Check tlonglink_file is not archived */
	if(!strncmp(hdr.tar_name, tlonglink_file, strlen(tlonglink_file)))
	{
		xx_rpt(FAILURE);
		if(hdr.tar_typeflag[0] == REGTYPE ||
		   hdr.tar_typeflag[0] == AREGTYPE)
			in_rpt("attempted to store file %s as a regular file (first in archive)", 
							tlonglink_file);
		else
			in_rpt("attempted to store file %s (first in archive)", 
							tlonglink_file);
		in_rpt("when linked to file with pathname of %d characters",
							strlen(tlong_file));
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	/* Should be trailer here - see if it's really a header */
	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Check tlonglink_file is not archived */
	if(!strncmp(hdr.tar_name, tlonglink_file, strlen(tlonglink_file)))
	{
		xx_rpt(FAILURE);
		in_rpt("attempted to store file %s (second in tar archive)",
							tlonglink_file);
		in_rpt("when linked to file with pathname of %d characters",
							strlen(tlong_file));
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(12);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

public void
test14()
{
	int	err;
	int	tar_fd;
	uid_t	uid;
	gid_t	gid;
	struct	stat stbuf;
	int	pathok = 0;
	
	/*
	 * Test that when a file is archived the uid and gid fields 
	 * contain the user ID and group ID respectively.
	 */

	DBUG_ENTER("test14");

	test_fail = 0;

	if(stat(treg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		if(err == ENOENT)
			in_rpt("file %s is not available for this test", 
								treg_file);
		in_rpt("stat(%s) failed - errno %d", treg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Read header for the regular file */
	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Obtain the value of the uid and gid fields */
	uid = aoctol(hdr.tar_uid, TUGIDLEN);
	gid = aoctol(hdr.tar_gid, TUGIDLEN);

	if(uid != stbuf.st_uid)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"uid\" field was not the user ID of the file owner");
		in_rpt("\"uid\" field was set to %d; user ID was %d", 
							uid, stbuf.st_uid);
	}
	else
		PATH_TRACE;
		
	if(gid != stbuf.st_gid)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"gid\" field was not the group ID of the file group");
		in_rpt("\"gid\" field was set to %d; group ID was %d", 
							gid, stbuf.st_gid);
	}
	else
		PATH_TRACE;
		

	if(!test_fail)
	{
		PATH_TRACE; 
		PATH_XS_RPT(5);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

public void
test15()
{
	int	err;
	int	tar_fd;
	long	size;
	struct	stat stbuf;
	int	pathok = 0;
			
	/*
	 * Test that a file archived with "typeflag" '0' has "size" 
	 * equal to the file size.
	 */

	DBUG_ENTER("test15");

	test_fail = 0;

	if(stat(treg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		if(err == ENOENT)
			in_rpt("file %s is not available for this test", 
								treg_file);
		in_rpt("stat(%s) failed - errno %d", treg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if(stbuf.st_size != strlen(REGDATA))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("size of file %s was not as expected", treg_file);
		in_rpt("Expected size %d; Observed size %ld", 
					strlen(REGDATA), stbuf.st_size);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Read header for the regular file */
	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Obtain the value of the size field */
	size = aoctol(hdr.tar_size, TSIZLEN);

	if(size != stbuf.st_size)
	{
		xx_rpt(FAILURE);
		in_rpt("for a regular file, \"size\" was not the file size");
		in_rpt("\"size\" was set to %ld; file size was %ld", 
							size, stbuf.st_size);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(4);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

public void
test16()
{
	int	err;
	int	tar_fd;
	long	size;
	struct	stat stbuf;
	int	pathok = 0;
	
	/*
	 * Test that a file archived with "typeflag" '1' has "size" zero.
	 */

	DBUG_ENTER("test16");

	test_fail = 0;

	if(stat(tlinked_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		if(err == ENOENT)
			in_rpt("file %s is not available for this test", 
								tlinked_file);
		in_rpt("stat(%s) failed - errno %d", tlinked_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if(stbuf.st_size != strlen(LNKDATA))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("size of file %s was not as expected", tlinked_file);
		in_rpt("Expected size %d; Observed size %ld", 
					strlen(LNKDATA), stbuf.st_size);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if((tar_fd = open_tar(tar_file, tlink_both)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Read header for the original file */
	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	/* Read header for the original file */
	globok = 0; 
	if(read_next(tar_fd, &hdr) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Obtain the value of the size field */
	size = aoctol(hdr.tar_size, TSIZLEN);

	if(size != 0L)
	{
		xx_rpt(FAILURE);
		in_rpt("for a linked file, \"size\" was not zero");
		in_rpt("\"size\" was set to %ld", size);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(5);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

public void
test17()
{

	int	pathok = 0;
	/*
	 * For other than regular files, no data blocks are stored.
	 */

	DBUG_ENTER("test17");

#if !TEST_XNFS
	globok = 0;
	t17_chr();
	PATH_FUNC_TRACE;

	globok = 0;
	t17_blk();
	PATH_FUNC_TRACE;

	globok = 0;
	t17_fifo();
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif /* TEST_XNFS */

	globok = 0;
	t17_dir();
	PATH_FUNC_TRACE;

	if(!test_fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	DBUG_VOID_RETURN;
}

private void
t17_chk(fd, filename, skipdirs)
int	fd;			/* File descriptor of tar format file */
char	*filename;		/* Name of type of file being tested */
int	skipdirs;		/* True if extra directories to be skipped */
{
	int	ret;
	int	err;
	int	pathok = 0;
	
	/*
	 * Check that the tar format file contains just header and trailer.
	 */

	DBUG_ENTER("t17_chk");
	
	globok = 0; 
	if(read_head(fd) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	while (skipdirs && hdr.tar_typeflag[0] == DIRTYPE)
	{
		/* skip extra directories on archive (e.g. "/dev") */

		if(read_head(fd) == SYSERROR) 
		{
			DBUG_VOID_RETURN;
		}
	}

	if(hdr.tar_name[0] == '\0')
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("For %s,", filename);
		in_rpt("tar format header containing file name was not created");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	globok = 0; 
	if(read_trail(fd, &trl) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if(chk_trail(&trl) == SYSERROR)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("For %s, non-zero bytes were", filename);
		in_rpt("stored in the tar format trailer");
	}
	else
		PATH_TRACE;
		
	/* Extra blocks are allowed after trailer - just check they
	   are full blocks */

	while((ret = read(fd, blk, TBLKSIZ)) == TBLKSIZ) 
		;

	if(ret == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("For %s, read() from tar format archive", filename);
		in_rpt("returned %d with errno set to %d", ret, err);
	}
	else if(ret > 0 && ret != TBLKSIZ)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("For %s, read() from tar format archive", filename);
		in_rpt("returned %d rather than expected block size %d",
							 ret, TBLKSIZ);
	}
	else
		PATH_TRACE;
		
	PATH_FUNC_RPT(5); 

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS
private void
t17_chr()
{
	int	tar_fd;
	char	*chr_file;
	int	pathok = 0;
	
	/*
	 * Test that a character special file is archived without data blocks.
	 */

	DBUG_ENTER("t17_chr");

	chr_file = tet_getvar("VSX_CHRDEV_FILE");
	if (chr_file == NULL || *chr_file == '\0')
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("parameter VSX_CHRDEV_FILE is not defined");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(chr_file, "unsup", (size_t)5) == 0)
	{
		DBUG_PRINTF("t17_chr", "VSX_CHRDEV_FILE=unsup");
		PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if((tar_fd = open_tar(tar_file, chr_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	globok = 0;
	t17_chk(tar_fd, "character special file", TRUE);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(2); 
	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

private void
t17_blk()
{
	int	tar_fd;
	char	*blk_file;
	int	pathok = 0;
	
	/*
	 * Test that a block special file is archived without data blocks.
	 */

	DBUG_ENTER("t17_blk");

	blk_file = tet_getvar("VSX_BLKDEV_FILE");
	if (blk_file == NULL || *blk_file == '\0')
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("parameter VSX_BLKDEV_FILE is not defined");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(blk_file, "unsup", (size_t)5) == 0)
	{
		DBUG_PRINTF("t17_blk", "VSX_BLKDEV_FILE=unsup");
		PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if((tar_fd = open_tar(tar_file, blk_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	globok = 0;
	t17_chk(tar_fd, "block special file", TRUE);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(2);
	
	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

private void
t17_dir()
{
	int	tar_fd;
	int	pathok = 0;
	
	/*
	 * Test that a directory file is archived without data blocks.
	 */

	DBUG_ENTER("t17_dir");

	globok = 0; 
	if(chk_file(tdir_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if((tar_fd = open_tar(tar_file, tdir_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	globok = 0;
	t17_chk(tar_fd, "directory file", FALSE);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(2); 
	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS
private void
t17_fifo()
{
	int	tar_fd;
	int	pathok = 0;
	
	/*
	 * Test that a fifo is archived without data blocks.
	 */

	DBUG_ENTER("t17_fifo");

	globok = 0; 
	if(chk_file(tfifo_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if((tar_fd = open_tar(tar_file, tfifo_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	globok = 0;
	t17_chk(tar_fd, "fifo", FALSE);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(2); 

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */
	  
public void
test18()
{
	int	err;
	int	tar_fd;
	time_t	stime, mtime;
	struct	stat stbuf;
	int	pathok = 0;
		
	/*
	 * Test that the "mtime" field of the tar format file is correct.
	 */

	DBUG_ENTER("test18");

	test_fail = 0;

	if(stat(treg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		if(err == ENOENT)
			in_rpt("file %s is not available for this test", 
								treg_file);
		in_rpt("stat(%s) failed - errno %d", treg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		

	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	globok = 0; 
	if(read_head(tar_fd) == SYSERROR)
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 		

	/* Obtain the value of the mtime field */
	mtime = aoctol(hdr.tar_mtime, TMTIMELEN);

	if((stime = stbuf.st_mtime) != mtime)
	{
		xx_rpt(FAILURE);
		in_rpt("mtime value %ld was not file modification time %ld",
						 mtime, stime);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(3); 
	}
	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

public void
test19()
{
	int	tar_fd;
	long	total;
	long	chksum;
	int	pathok = 0;
		
	/*
	 * Test that the "chksum" field of the tar format file is correct.
	 */

	DBUG_ENTER("test19");

	test_fail = 0;

	globok = 0; 
	if(chk_file(treg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	globok = 0; 
	if(read_head(tar_fd) == SYSERROR)
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	/* Calculate the expected checksum */
	total = calc_sum();

	/* Obtain the value of the checksum field */
	chksum = aoctol(hdr.tar_chksum, TSUMLEN);

	if(total != chksum)
	{
		xx_rpt(FAILURE);
		in_rpt("Checksum value %ld was not total of byte values %ld",
						 chksum, total);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(3);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

public void
test20()
{
	int	tar_fd;
	int	val;
	int	pathok = 0;
	
	/*
	 * Test that a regular file is archived with "typeflag" '0'.
	 * This is defined as REGTYPE in tar.h
	 */

	DBUG_ENTER("test20");

	test_fail = 0;

	globok = 0; 
	if(chk_file(treg_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 
	globok = 0;
	if(chk_head() == SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("regular file was not archived");
	}
	else if((val = hdr.tar_typeflag[0]) != REGTYPE)
	{
		xx_rpt(FAILURE);
		in_rpt("for a regular file, \"typeflag\" was not \'%c\' (0%o)",
						 REGTYPE, REGTYPE);
		if(isprint(val)) 
			in_rpt("\"typeflag\" was set to \'%c\' (0%o)",val,val);
		else
			in_rpt("\"typeflag\" was set to (0%o)",val);
	}
	else
	{
		PATH_FUNC_TRACE; 
		PATH_XS_RPT(3);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

public void
test21()
{
	int	tar_fd;
	int	val;
	int	pathok = 0;
	
	/*
	 * Test that a linked file is archived with "typeflag" '1'.
	 * This is defined as LNKTYPE in tar.h
	 */

	DBUG_ENTER("test21");

	test_fail = 0;

	globok = 0; 
	if(chk_file(tlink_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0; 
	if(chk_file(tlinked_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if((tar_fd = open_tar(tar_file, tlink_both)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Read header for the original file */
	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Read header for the linked file */
	globok = 0; 
	if(read_next(tar_fd, &hdr) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0; 
	if(chk_head() == SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("linked file was not archived");
	}
	else if((val = hdr.tar_typeflag[0]) != LNKTYPE)
	{
		xx_rpt(FAILURE);
		in_rpt("for a linked file, \"typeflag\" was not \'%c\' (0%o)",
						 LNKTYPE, LNKTYPE);
		if(isprint(val)) 
			in_rpt("\"typeflag\" was set to \'%c\' (0%o)",val,val);
		else
			in_rpt("\"typeflag\" was set to (0%o)",val);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(5);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

public void
test22()
{
	DBUG_ENTER("test22");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else

public void
test22()
{
	int	tar_fd;
	int	val;
	char	*chr_file;
	int	pathok = 0;
	
	/*
	 * Test that a character special file is archived with "typeflag" '3'.
	 * This is defined as CHRTYPE in tar.h
	 */

	DBUG_ENTER("test22");

	test_fail = 0;

	chr_file = tet_getvar("VSX_CHRDEV_FILE");
	if (chr_file == NULL || *chr_file == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_CHRDEV_FILE is not defined");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(chr_file, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_CHRDEV_FILE is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if((tar_fd = open_tar(tar_file, chr_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	while (hdr.tar_typeflag[0] == DIRTYPE)
	{
		/* skip extra directories on archive (e.g. "/dev") */

		if(read_head(tar_fd) == SYSERROR) 
		{
			(void) close_tar(tar_file, tar_fd);
			DBUG_VOID_RETURN;
		}
	}

	globok = 0; 
	if(chk_head() == SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("character special file was not archived");
	}
	else if((val = hdr.tar_typeflag[0]) != CHRTYPE)
	{
		xx_rpt(FAILURE);
		in_rpt("for a character special file, \"typeflag\" was not \'%c\' (0%o)",
						 CHRTYPE, CHRTYPE);
		if(isprint(val)) 
			in_rpt("\"typeflag\" was set to \'%c\' (0%o)",val,val);
		else
			in_rpt("\"typeflag\" was set to (0%o)",val);
	}
	else
	{
		PATH_FUNC_TRACE; 
		PATH_XS_RPT(3);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

public void
test23()
{
	DBUG_ENTER("test23");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else

public void
test23()
{
	int	tar_fd;
	int	val;
	char	*blk_file;
	int	pathok = 0;
	
	/*
	 * Test that a block special file is archived with "typeflag" '4'.
	 * This is defined as BLKTYPE in tar.h
	 */

	DBUG_ENTER("test23");

	test_fail = 0;

	blk_file = tet_getvar("VSX_BLKDEV_FILE");
	if (blk_file == NULL || *blk_file == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_BLKDEV_FILE is not defined");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(blk_file, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_BLKDEV_FILE is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if((tar_fd = open_tar(tar_file, blk_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	while (hdr.tar_typeflag[0] == DIRTYPE)
	{
		/* skip extra directories on archive (e.g. "/dev") */

		if(read_head(tar_fd) == SYSERROR) 
		{
			(void) close_tar(tar_file, tar_fd);
			DBUG_VOID_RETURN;
		}
	}

	globok = 0; 
	if(chk_head() == SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("block special file was not archived");
	}
	else if((val = hdr.tar_typeflag[0]) != BLKTYPE)
	{
		xx_rpt(FAILURE);
		in_rpt("for a block special file, \"typeflag\" was not \'%c\' (0%o)",
						 BLKTYPE, BLKTYPE);
		if(isprint(val)) 
			in_rpt("\"typeflag\" was set to \'%c\' (0%o)",val,val);
		else
			in_rpt("\"typeflag\" was set to (0%o)",val);
	}
	else
	{
		PATH_FUNC_TRACE; 
		PATH_XS_RPT(3);
	}
	
	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
test24()
{
	int	tar_fd;
	int	val;
	int	pathok = 0;
	
	/*
	 * Test that a directory file is archived with "typeflag" '5'.
	 * This is defined as DIRTYPE in tar.h
	 */

	DBUG_ENTER("test24");

	test_fail = 0;

	globok = 0; 
	if(chk_file(tdir_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if((tar_fd = open_tar(tar_file, tdir_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0; 
	if(chk_head() == SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("directory file was not archived");
	}
	else if((val = hdr.tar_typeflag[0]) != DIRTYPE)
	{
		xx_rpt(FAILURE);
		in_rpt("for a directory file, \"typeflag\" was not \'%c\' (0%o)",
						 DIRTYPE, DIRTYPE);
		if(isprint(val)) 
			in_rpt("\"typeflag\" was set to \'%c\' (0%o)",val,val);
		else
			in_rpt("\"typeflag\" was set to (0%o)",val);
	}
	else
	{
		PATH_FUNC_TRACE; 
		PATH_XS_RPT(3);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

public void
test25()
{
	DBUG_ENTER("test25");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else

public void
test25()
{
	int	tar_fd;
	int	val;
	int	pathok = 0;
	
	/*
	 * Test that a fifo is archived with "typeflag" '6'.
	 * This is defined as FIFOTYPE in tar.h
	 */

	DBUG_ENTER("test25");

	test_fail = 0;

	globok = 0; 
	if(chk_file(tfifo_file) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if((tar_fd = open_tar(tar_file, tfifo_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0; 
	if(chk_head() == SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("fifo was not archived");
	}
	else if((val = hdr.tar_typeflag[0]) != FIFOTYPE)
	{
		xx_rpt(FAILURE);
		in_rpt("for a fifo, \"typeflag\" was not \'%c\' (0%o)",
						 FIFOTYPE, FIFOTYPE);
		if(isprint(val)) 
			in_rpt("\"typeflag\" was set to \'%c\' (0%o)",val,val);
		else
			in_rpt("\"typeflag\" was set to (0%o)",val);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(3);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
test30()
{
	int	err;
	int	tar_fd;
	struct	stat stbuf;
	struct	passwd *pwd;
	struct	group *grp;
	char	user[TUGNAMLEN + 1];
	char	group[TUGNAMLEN + 1];
	char	tar_name[TUGNAMLEN + 1];
	int	pathok = 0;
	
	/*
	 * Test that when a file is archived, and the "magic" field 
	 * contains the value TMAGIC, defined in tar.h, then
	 * uname and gname contain the ASCII representation of the 
	 * user ID and group ID respectively.
	 */

	DBUG_ENTER("test30");

	test_fail = 0;

	if(stat(treg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		if(err == ENOENT)
			in_rpt("file %s is not available for this test", 
								treg_file);
		in_rpt("stat(%s) failed - errno %d", treg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if((grp = getgrgid(stbuf.st_gid)) == (struct group *)NULL)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("getgrgid() failed - errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strncpy(group, grp->gr_name, (size_t)TUGNAMLEN);
	group[TUGNAMLEN] = '\0';
	if((pwd = getpwuid(stbuf.st_uid)) == (struct passwd *)NULL)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("getpwuid() failed - errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strncpy(user, pwd->pw_name, (size_t)TUGNAMLEN);
	user[TUGNAMLEN] = '\0';

	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Read header for the regular file */
	globok = 0; 
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Test the "magic" field */
	if(strncmp(TMAGIC, hdr.tar_magic, strlen(TMAGIC)))
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("\"magic\" field of tar format archive is not \"%s\"", 
									TMAGIC);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) strncpy(tar_name, hdr.tar_uname, (size_t)TUGNAMLEN);
	tar_name[TUGNAMLEN] = '\0';
	if(strncmp(user, tar_name, (size_t)TUGNAMLEN))
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"uname\" field was not the user name of the file owner");
		in_rpt("\"uname\" field was set to %s; user name was %s", 
							tar_name, user);
	}
	else
		PATH_TRACE;
	(void) strncpy(tar_name, hdr.tar_gname, (size_t)TUGNAMLEN);
		
	tar_name[TUGNAMLEN] = '\0';
	if(strncmp(group, tar_name, (size_t)TUGNAMLEN))
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("\"gname\" field was not the group name of the file group");
		in_rpt("\"gname\" field was set to %s; group name was %s", 
							tar_name, group);
	}
	else
		PATH_TRACE;
		
	if(!test_fail)
	{
		PATH_TRACE; 
		PATH_XS_RPT(8);
	}

	(void) close_tar(tar_file, tar_fd);

	DBUG_VOID_RETURN;
}
