/*
*      SCCS:  @(#)  POSIX.os/dataform/tar/tar2.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.tar

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the IEEE Std. 1003.1-1988 "Extended tar format"
	This file contains the tests involving reading tar format archives.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu May 18 09:00:23 BST 1989
MODIFICATIONS:	Dave Cater, UniSoft Ltd.
		Wed Sep  6 11:03:10 BST 1989
		Corrected ch_t5() so it really is private.
		Corrected test11() from error report err_hp.030

		Dave Cater, UniSoft Ltd.
		Thu Sep 21 10:15:35 BST 1989
		Add to test11() to ensure link file is linked to a file 
		which does exist in the archive from error report err_hp.070

		Dave Cater, UniSoft Ltd.
		Tue Oct 31 11:51:49 GMT 1989
		Check in test13() that process does not have appropriate 
		privileges.

		Geoff Clare, 15 Nov 1989
		Add new test case in test10

		Geoff Clare, 28 June 1990
		Test several file sizes in REGTYPE extraction test
		Add AREGTYPE extraction test
		Add FIFOTYPE "size ignored" test
		Change "all-null prefix" test to "empty (but not all-null)"

		Anthony Lee, 27 July 1990
		Addded resilience code.
		
		Stuart Boutell, 22 Mar 1991
		Added " = (char *) NULL " to link_file declaration in test 11
		Added (long) cast in ltoaoct() call in test 13

		Stuart Boutell, 5 Apr 1991
			Added new test for empty prefix strings leaving
		pathname as name. (split out from test 10)

		Peter Skan, 27 Sept 1991
			Changed the check for failure of setprv(PRV_CHOWN) to
			a check for success of unsetprv(PRV_CHOWN), in case
			we have a system which actually sets privileges
			when setprv() is called.
		
		Geoff Clare, 6 Feb 1992
			Change to using directories in pathres test.
		
		Geoff Clare, 9 Apr 1992
			Change pathres test to create an archive containing
			the pathname to be resolved.
		
		Geoff Clare, 8 May 1992
			Add PRES_NODOTDOT to init_pathres() call.
		
		Geoff Clare, 9 June 1992
			Move full name/prefix/linkname testing from test 7
			(a creation test) to tests 10 & 27 (extraction tests)
			Add portable filename test.

		Geoff Clare, 19 June 1992
			Add PRES_NODOT to init_pathres() call.

		Calum Laing, Mon Nov 23 14:19:19 GMT 1992
			XNFS enhancements

		Geoff Clare, 18 Nov 1993
			Fix err/errno confusion after stat() in test 5.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<ctype.h>
#include	<tar.h>
#include	<pwd.h>
#include	<dbug.h>
#include	<report.h>
#include	<fcntl.h>
#include	<pathres.h>
#include	<setprv.h>
#include	"tartest.h"
#include 	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
#define	FULL_LNK	100
#else
#define	FULL_LNK	99
#endif

extern	int	close_tar();
extern	int	create_tar();
extern	int	extract_tar();
extern	void	ltoaoct();
extern	long	calc_sum();
extern	char	*errname();
extern	char	*pr_char();

extern	int	vguid1;
extern int	test_fail;

extern	char	*tprefix;
extern	char	*treg_file;
extern	char	*tlink_file;
extern	char	*tlinked_file;
extern	char	*tlink_both;
extern	char	*tlonglink_file;
extern	char	*tdir_file;
#if !TEST_XNFS
extern	char	*tfifo_file;
#endif /* TEST_XNFS */
extern	char	*tar_file;
extern	char	*tnew_file;
extern	char	*tnewlink_file;
extern	char	*tnew_dir;
extern	char	*tnew_fifo;
extern	char	*tar_type;
extern	struct	tar_head hdr;
extern	struct	tar_trail trl;

private	char	pathname[TBLKSIZ];
private	char	linkpath[TBLKSIZ];
private	char	outbuf[TBLKSIZ];
private	char	inbuf[TBLKSIZ];

private void	ch_t5();
private void	regextract();

private char *
file_type(statptr)
struct	stat *statptr;
{
	/*
	 * Return pointer to name of the file type, or (char *) NULL
	 * if it is not one of the expected types.
	 */

	if(S_ISREG(statptr->st_mode)) {
		if(statptr->st_nlink > 1)
			return("linked file");
		else
			return("regular file");
	} else if(S_ISDIR(statptr->st_mode))
		return("directory file");
	else if(S_ISCHR(statptr->st_mode))
		return("character special file");
	else if(S_ISBLK(statptr->st_mode))
		return("block special file");
	else if(S_ISFIFO(statptr->st_mode))
		return("fifo");
	else
		return((char *)NULL);
}

public void
test5()
{
	DBUG_ENTER("test5");

	(void) cppair(ch_t5, NULLFN, 10 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	tar_fd;
	int	err;
	long	total;
	char *p1, *p2;
	char	*s;
	int npairs, i;
	struct stat stbuf;
	int	pathok = 0;
			

	/*
	 * Test that the format utility supports file name resolution.
	 */

	DBUG_ENTER("ch_t5");

	test_fail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOT|PRES_NODOTDOT);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOT|PRES_NODOTDOT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];
		DBUG_PRINTF("ch_t5","npairs is %d",npairs);

		/* Create an archive containing the complex name */

		if((tar_fd = new_tar(tar_file)) == SYSERROR)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Initialise a tar format header */
		init_head(&hdr);

		/* Write it with complex name */
		(void) strncpy(hdr.tar_name, p1, (size_t)TNAMLEN);

		/* Add "typeflag" '5' (DIRTYPE) */
		hdr.tar_typeflag[0] = DIRTYPE;

		total = calc_sum();
		ltoaoct(total, hdr.tar_chksum, TSUMLEN);

		globok = 0; 
		if(write_head(tar_fd) == SYSERROR) 
		{
			(void) close_tar(tar_file, tar_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
			
		init_trail(&trl);

		globok = 0; 	
		if(write_trail(tar_fd, &trl) == SYSERROR) 
		{
			(void) close_tar(tar_file, tar_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
			
		/* Remove the file */
		(void) rmdir(p2);

		/* Check it has really gone */
		if(stat(p2, &stbuf) != SYSERROR || errno != ENOENT)
		{
			err = errno;
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("rmdir() did not remove directory %s", p2);
			in_rpt("errno = %d - %s", err, errname(err));
			break;
		}
		else
			PATH_TRACE;
			
		/* Extract the file */

		globok = 0; 	
		if(extract_tar(tar_file, NORESTORE) == SYSERROR)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE; 
		
		/* Check the simple name file has been created */
		if(stat(p2, &stbuf) == SYSERROR)
		{
			err = errno;
			if(err == ENOENT)
			{
				if(!test_fail++)
					xx_rpt(FAILURE);
				in_rpt("when extracting \"%s\" in tar format",
									p1);
				in_rpt("file name was not resolved to \"%s\"", 
									p2);
			}
			else
			{
				if(!test_fail++)
					xx_rpt(DELETION);
				in_rpt("stat(%s) failed - errno %d", p2, err);
			}
		} else
			if(!(S_ISDIR(stbuf.st_mode)))
			{
				if(!test_fail++)
					xx_rpt(FAILURE);
				in_rpt("directory file %s was not extracted using typeflag \"%c\"", p2, hdr.tar_typeflag[0]);
				if((s = file_type(&stbuf)) != (char *)NULL)
					in_rpt("instead %s was extracted", s);
				(void) unlink(p2);
			} else
				PATH_TRACE;

		(void) rmdir(p2);
		(void) close_tar(tar_file, tar_fd);
	}

	if(!test_fail)
	{
		PATH_XS_RPT(1 + (npairs * 6));
	}
	
	(void) unlink(tar_file);
	cln_pathres();
	DBUG_VOID_RETURN;
}

public void
test10()
{
	int	tar_fd;
	int	i, err;
	long	total;
	struct	stat stbuf;
	char	tlong_file[256];
	int	pathok = 0;
	
	DBUG_ENTER("test10");

	test_fail = 0;

	/*
	 * Test that the "prefix" field , a slash character and the "name" 
	 * field are used to produce a new pathname.
	 */

	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write it with a prefix field and file name */
	(void) strncpy(hdr.tar_prefix, tprefix, (size_t)TPREFLEN);
	(void) strncpy(hdr.tar_name, tnew_file, (size_t)TNAMLEN);

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	init_trail(&trl);
	globok = 0; 
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 
	
	/* 
	 * Ensure the expected file does not exist either where expected 
	 * or in the current directory.
	 */
	(void) strcpy(pathname, tprefix);
	(void) strcat(pathname, SLASH);
	(void) strcat(pathname, tnew_file);

	(void) unlink(pathname);
	(void) unlink(tnew_file);
	if(stat(pathname, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", pathname);
	}
	else
		PATH_TRACE;
		
	if(stat(tnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnew_file);
	}
	else
		PATH_TRACE;
		
	if(test_fail)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	 
	/* Extract the file */
	globok = 0; 
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Trap a common problem - prefix ignored completely */
	if(stat(tnew_file, &stbuf) != SYSERROR)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("file with pathname \"%s\" was unexpectedly extracted",
								 tnew_file);
	}
	else
		PATH_TRACE;

	if(stat(pathname, &stbuf) == SYSERROR)
	{
		err = errno;
		if(err == ENOENT)
		{
			if(!test_fail++)
				xx_rpt(FAILURE);
			in_rpt("expected pathname \"%s\" was not extracted",
							pathname);
		}
		else
		{
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("stat(%s) failed - errno %d", pathname, err);
		}
	}
	else
		PATH_TRACE;
		
	(void) unlink(tnew_file);
	(void) unlink(pathname);
	(void) rmdir(tprefix);
	(void) close_tar(tar_type, tar_fd);

	if (test_fail)
		DBUG_VOID_RETURN;

	/* Now test full fields: name 100, prefix 155 */

	/* name & prefix fields full => path length 256.  Unfortunately this
	   is (just) > _POSIX_PATH_MAX, so we have to test them separately */

	/* test with name field full, using name 100, prefix 149 */

	tlong_file[0] = '\0';
	for (i = 0; i < 16; i++)
		(void) strcat(tlong_file, "12345678901234/");
	(void) strcat(tlong_file, "1234567890");

	/* length of tlong_file is (16 * 15) + 10 = 250 */

	DBUG_PRINTF("test10", "tlong_file: %ld", (long)strlen(tlong_file));

	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write it with a prefix field and file name */
	(void) strncpy(hdr.tar_prefix, tlong_file, (size_t)149);
	hdr.tar_prefix[149] = '\0';
	(void) strncpy(hdr.tar_name, &tlong_file[150], (size_t)100);

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	init_trail(&trl);
	globok = 0; 
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 
	
	/* 
	 * Ensure the expected file does not exist either where expected 
	 * or in the current directory.
	 */

	(void) unlink(tlong_file);
	if(stat(tlong_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", pathname);
	}
	else
		PATH_TRACE;
		
	if(test_fail)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	 
	/* Extract the file */
	globok = 0; 
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	if(stat(tlong_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(err == ENOENT)
		{
			if(!test_fail++)
				xx_rpt(FAILURE);
			in_rpt("expected pathname \"%s\" was not extracted",
							tlong_file);
		}
		else
		{
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("stat(%s) failed - errno %d", tlong_file, err);
		}
	}
	else
		PATH_TRACE;
		
	(void) unlink(tlong_file);
	(void) close_tar(tar_type, tar_fd);

	/* test with prefix field full, using name 94, prefix 155 */

	tlong_file[155] = '/';
	tlong_file[156] = '\0';

	for (i = 0; i < 6; i++)
		(void) strcat(tlong_file, "12345678901234/");
	(void) strcat(tlong_file, "1234");

	/* length of tlong_file is 156 + (6 * 15) + 4 = 250 */

	DBUG_PRINTF("test10", "tlong_file: %ld", (long)strlen(tlong_file));

	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write it with a prefix field and file name */
	(void) strncpy(hdr.tar_prefix, tlong_file, (size_t)155);
	(void) strncpy(hdr.tar_name, &tlong_file[156], (size_t)94);
	hdr.tar_name[94] = '\0';

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	init_trail(&trl);
	globok = 0; 
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 
	
	/* 
	 * Ensure the expected file does not exist either where expected 
	 * or in the current directory.
	 */

	(void) unlink(tlong_file);
	if(stat(tlong_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", pathname);
	}
	else
		PATH_TRACE;
		
	if(test_fail)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	 
	/* Extract the file */
	globok = 0; 
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	if(stat(tlong_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(err == ENOENT)
		{
			if(!test_fail++)
				xx_rpt(FAILURE);
			in_rpt("expected pathname \"%s\" was not extracted",
							tlong_file);
		}
		else
		{
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("stat(%s) failed - errno %d", tlong_file, err);
		}
	}
	else
		PATH_TRACE;
		
	(void) unlink(tlong_file);
	(void) close_tar(tar_type, tar_fd);

	if(!test_fail)
	{
		PATH_XS_RPT(17);
	}
	
	DBUG_VOID_RETURN;
}

public void
test1()
{
	int	tar_fd;
	int	err;
	long	total;
	struct	stat stbuf;

	int	pathok = 0;

	DBUG_ENTER("test1");

	test_fail = 0;

	/*
	 * Test that if the "prefix" field is empty the pathname is
	 * the "name" field
	 */

	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write it with an empty (but not all-null) prefix and a file name */
	(void) strncpy(&hdr.tar_prefix[1], tprefix, (size_t)(TPREFLEN-1));
	(void) strncpy(hdr.tar_name, tnew_file, (size_t)TNAMLEN);

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	init_trail(&trl);
	globok = 0; 
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 
	
	/* 
	 * Ensure the expected file does not exist either where expected 
	 * or in the current directory.
	 */
	(void) strcpy(pathname, tnew_file);
	(void) unlink(pathname);
	if(stat(pathname, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", pathname);
	}
	else
		PATH_TRACE;
		
	if(test_fail)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	 
	/* Extract the file */

	globok = 0; 	
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if(stat(pathname, &stbuf) == SYSERROR)
	{
		err = errno;
		if(err == ENOENT)
		{
			if(!test_fail++)
				xx_rpt(FAILURE);
			in_rpt("expected pathname \"%s\" was not extracted",
							pathname);
		}
		else
		{
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("stat(%s) failed - errno %d", pathname, err);
		}
	}
	else
		PATH_TRACE;
		
	(void) unlink(pathname);
	(void) close_tar(tar_type, tar_fd);

	if(!test_fail)
	{
		PATH_XS_RPT(5);
	}
	
	DBUG_VOID_RETURN;
}

public void
test11()
{
	int	tar_fd;
	int	err;
	long	total;
	struct	stat stbuf;
	ino_t	link_inode;
	ino_t	bad_inode;
	nlink_t	link_count;
	char	*link_file = (char *)NULL;
	int	pathok = 0;
	
	/*
	 * Test that the "linkname" field does not use "prefix"
	 * to produce a pathname.
	 */

	DBUG_ENTER("test11");

	test_fail = 0;

	(void) strcpy(pathname, tprefix);
	(void) strcat(pathname, SLASH);
	(void) strcat(pathname, tnew_file);
	(void) strcpy(linkpath, tprefix);
	(void) strcat(linkpath, SLASH);
	(void) strcat(linkpath, tnewlink_file);

	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write a tar format header (with a prefix) for a test file */
	/* This is the file that should NOT be linked to */
	(void) strncpy(hdr.tar_name, tnew_file, (size_t)TNAMLEN);
	(void) strncpy(hdr.tar_prefix, tprefix, (size_t)TPREFLEN);

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Re-Initialise tar format header */
	init_head(&hdr);

	/* Write a tar format header - to archive the same file! */
	/* Name incorporates prefix - just in case utility works that way */
	/* rather than the correct way of using the prefix field */
	(void) strncpy(hdr.tar_name, pathname, (size_t)TNAMLEN);

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Re-Initialise tar format header */
	init_head(&hdr);

	/* Write a tar format header - to archive the file to link to */
	(void) strncpy(hdr.tar_name, tnew_file, (size_t)TNAMLEN);

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Re-Initialise tar format header */
	init_head(&hdr);

	/* Write a tar format header (with a prefix) for a linked file */
	hdr.tar_typeflag[0] = LNKTYPE;
	(void) strncpy(hdr.tar_linkname, tnew_file, (size_t)TLNKLEN);
	(void) strncpy(hdr.tar_name, tnewlink_file, (size_t)TNAMLEN);
	(void) strncpy(hdr.tar_prefix, tprefix, (size_t)TPREFLEN);

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	init_trail(&trl);

	globok = 0; 
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	/* 
	 * Ensure the test file does not exist
	 */
	(void) unlink(pathname);
	if(stat(pathname, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", pathname);
	}
	else
		PATH_TRACE;
		
	/* 
	 * Ensure the link file does not exist either where expected 
	 * or in the current directory.
	 */
	(void) unlink(linkpath);
	(void) unlink(tnewlink_file);
	if(stat(linkpath, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", linkpath);
	}
	else
		PATH_TRACE;
		
	if(stat(tnewlink_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnewlink_file);
	}
	else
		PATH_TRACE;
		
	if(test_fail)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}

	/* Extract the files */

	globok = 0; 	
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	/* Ensure the linked file was extracted */
	/* Continue even if the prefix was ignored in the extraction */
	if(stat(linkpath, &stbuf) == SYSERROR)
	{
		err = errno;
		if(err == ENOENT)
		{
			if(stat(tnewlink_file, &stbuf) == SYSERROR)
			{
				err = errno;
				if(err == ENOENT)
				{
					if(!test_fail++)
						xx_rpt(DELETION);
					in_rpt("expected file \"%s\" was not extracted",
									tnewlink_file);
				}
				else
				{
					if(!test_fail++)
						xx_rpt(DELETION);
					in_rpt("stat(%s) failed - errno %d", tnewlink_file, err);
				}
			}
			else
			{
				link_file = tnewlink_file;
				PATH_TRACE; 
			}
		}
		else
		{
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("stat(%s) failed - errno %d", linkpath, err);
		}
	}
	else
	{
		link_file = linkpath;
		PATH_TRACE; 
	}

	link_count = stbuf.st_nlink;
	link_inode = stbuf.st_ino;

	if(stat(pathname, &stbuf) == SYSERROR)
	{
		err = errno;
		if(err == ENOENT)
		{
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("expected pathname \"%s\" was not extracted",
							pathname);
		}
		else
		{
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("stat(%s) failed - errno %d", pathname, err);
		}
	}
	else
		PATH_TRACE;
		
	bad_inode = stbuf.st_ino;

	if(test_fail)
	{
		(void) unlink(tnew_file);
		(void) unlink(link_file);
		(void) unlink(pathname);
		(void) rmdir(tprefix);
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}

	if(link_count > 1 && link_inode == bad_inode) 
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("file \"%s\" was linked to file \"%s\"", link_file, 
								pathname);
		in_rpt("\"linkname\" field used \"prefix\" to produce a pathname");
	}

	if(!test_fail)
	{
		PATH_XS_RPT(11);
	}

	(void) unlink(tnew_file);
	(void) unlink(link_file);
	(void) unlink(pathname);
	(void) rmdir(tprefix);
	(void) close_tar(tar_type, tar_fd);

	DBUG_VOID_RETURN;
}

public void
test13()
{
	int	tar_fd;
	int	err;
	long	total;
	struct	stat stbuf;
	int	pathok = 0;

	/*
	 * Test that the "mode" bits are ignored if the user does not
	 * have appropriate privileges.
	 */

	DBUG_ENTER("test13");

	test_fail = 0;

	if (unsetprv(PRV_CHOWN) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: unsetprv(PRV_CHOWN) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write it with just a simple file name */
	(void) strncpy(hdr.tar_name, tnew_file, (size_t)TNAMLEN);

	/* Add set-user ID mode */
	ltoaoct((long)(TSUID|T_MODEANY), hdr.tar_mode, TMODELEN);

	/* Alter file owner */
	ltoaoct((long)vguid1, hdr.tar_uid, TUGIDLEN);

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	init_trail(&trl);

	globok = 0; 
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
			
	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(tnew_file);
	if(stat(tnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnew_file);
	}
	if(test_fail)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			 
	/* Extract the file */

	globok = 0; 
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if(stat(tnew_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d", tnew_file, err);
	}
	else if(stbuf.st_uid != getuid() && stbuf.st_mode & S_ISUID)
	{
		xx_rpt(FAILURE);
		in_rpt("when user did not have appropriate privilege");
		in_rpt("set-user ID bits of \"mode\" field were not ignored");
	}
	else
	{

		PATH_TRACE;
		PATH_XS_RPT(6); 
	}

	(void) unlink(tnew_file);
	(void) close_tar(tar_type, tar_fd);
	
	DBUG_VOID_RETURN;
}

public void
test26()
{
	/*
	 * Test that when reading an archive, a "typeflag" of '0' will
	 * cause a regular file to be generated.
	 */

	int i;
	int	pathok = 0;
	
	DBUG_ENTER("test26");

	test_fail = 0;

	for (i = 0; i < sizeof(outbuf); i++)
		outbuf[i] = i % 256;

	globok = 0;
	regextract(REGTYPE, 0);
	PATH_FUNC_TRACE;

	globok = 0;
	regextract(REGTYPE, 1);
	PATH_FUNC_TRACE;

	globok = 0;
	regextract(REGTYPE, TBLKSIZ);
	PATH_FUNC_TRACE;

	globok = 0;
	regextract(REGTYPE, TBLKSIZ + 1);
	PATH_FUNC_TRACE;

	globok = 0;
	regextract(REGTYPE, 2 * TBLKSIZ);
	PATH_FUNC_TRACE;
	
	if (test_fail == 0)
	{
		PATH_XS_RPT(5);
	}

	DBUG_VOID_RETURN;
}

private void
regextract(typeflag, size)
int typeflag;
int size;
{
	int	tar_fd;
	int	fd;
	int	err;
	int	remain;
	int	nbyte;
	long	total;
	struct	stat stbuf;
	char	*s;
	int	pathok = 0;

	/*
	 * Test that when reading an archive, a "typeflag" of '0' or '\0'
	 * will cause a regular file to be generated.
	 */

	DBUG_ENTER("regextract");

	test_fail = 0;

	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write it with just a simple file name */
	(void) strncpy(hdr.tar_name, tnew_file, (size_t)TNAMLEN);

	/* Add "typeflag" '0' (REGTYPE) or '\0' (AREGTYPE) */
	hdr.tar_typeflag[0] = typeflag;

	/* Add size field */
	ltoaoct((long)size, hdr.tar_size, TSIZLEN);

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	remain = size;
	while (remain > 0)
	{
		if (write(tar_fd, outbuf, TBLKSIZ) != TBLKSIZ)
		{
			if (!test_fail++)
				xx_rpt(DELETION);
			in_rpt("write() failed - errno %d (%s)",
				errno, errname(errno));
			(void) close_tar(tar_type, tar_fd);
			DBUG_VOID_RETURN;
		}
		remain -= TBLKSIZ;
	}

	init_trail(&trl);

	globok = 0; 
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
			
	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(tnew_file);
	if(stat(tnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("could not remove test file \"%s\"", tnew_file);
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	globok = 0; 	 
	/* Extract the file */
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(stat(tnew_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("file %s was not extracted from archive", tnew_file);
		in_rpt("stat(%s) failed - errno %d", tnew_file, err);
	}
	else if(!(S_ISREG(stbuf.st_mode)) || stbuf.st_nlink > 1)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("regular file was not extracted using typeflag \"%c\"",
							hdr.tar_typeflag[0]);
		if((s = file_type(&stbuf)) != (char *)NULL)
			in_rpt("instead %s was extracted", s);
	}
	else if (stbuf.st_size != size)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("file was extracted with wrong size");
		in_rpt("expected %d, actual %ld", size, (long)stbuf.st_size);
	}
	else if (size > 0)
	{
		if ((fd = open(tnew_file, O_RDONLY)) < 0)
		{
			err = errno;
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("open(%s, O_RDONLY) failed - errno %d (%s)",
				tnew_file, err, errname(err));
		}
		else
			PATH_TRACE;
			
		remain = size;
		while (remain > 0)
		{
			if ((nbyte = read(fd, inbuf, TBLKSIZ)) != TBLKSIZ &&
				nbyte != remain)
			{
				if(!test_fail++)
					xx_rpt(DELETION);
				in_rpt("read() returned %d, expected %d", nbyte,
					(remain > TBLKSIZ ? TBLKSIZ : remain));
				break;
			}
			else if (memcmp((void *)inbuf, (void *)outbuf,
				(size_t)nbyte) != 0)
			{
				if(!test_fail++)
					xx_rpt(FAILURE);
				in_rpt("extracted file of %d bytes has incorrect contents",
					size);
				break;
			}
			remain -= nbyte;
		}
		(void) close(fd);
	}
	else
		PATH_TRACE; /* There is a PATH_TRACE in the other path */
		
	(void) close_tar(tar_type, tar_fd);
	(void) unlink(tnew_file);
	PATH_FUNC_RPT(5); 
	
	DBUG_VOID_RETURN;
}

public void
test27()
{
	int	tar_fd;
	int	i, err;
	long	total;
	struct	stat stbuf;
	char	*s;
	char	tlong_file[256];
	int	pathok = 0;
	
	/*
	 * Test that when reading an archive, a typeflag of '1' will
	 * cause a linked file to be generated.
	 */

	DBUG_ENTER("test27");

	test_fail = 0;

	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write the regular file details */
	(void) strncpy(hdr.tar_name, tnew_file, (size_t)TNAMLEN);
	hdr.tar_typeflag[0] = REGTYPE;

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	/* Write a tar format header for a linked file */
	(void) strncpy(hdr.tar_linkname, hdr.tar_name, (size_t)TLNKLEN);
	(void) strncpy(hdr.tar_name, tnewlink_file, (size_t)TNAMLEN);

	/* Add "typeflag" '1' (LNKTYPE) */
	hdr.tar_typeflag[0] = LNKTYPE;

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;		
	init_trail(&trl);

	globok = 0; 	
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* 
	 * Ensure the expected file (and one it is linked to) does not exist.
	 */

	(void) unlink(tnew_file);
	(void) unlink(tnewlink_file);
	if(stat(tnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnew_file);
	}
	else
		PATH_TRACE;
		
	if(stat(tnewlink_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnewlink_file);
	}
	else
		PATH_TRACE;
		
	if(test_fail)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}

	/* Extract the file */

	globok = 0; 	
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(stat(tnewlink_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("file %s was not extracted from archive", tnew_file);
		in_rpt("stat(%s) failed - errno %d", tnewlink_file, err);
	}
	else if(!(S_ISREG(stbuf.st_mode)) || stbuf.st_nlink <= 1)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("linked file was not extracted using typeflag \"%c\"",
							hdr.tar_typeflag[0]);
		if((s = file_type(&stbuf)) != (char *)NULL)
			in_rpt("instead %s was extracted", s);
	}
	else
		PATH_TRACE;
		
	(void) unlink(tnew_file);
	(void) unlink(tnewlink_file);
	(void) close_tar(tar_type, tar_fd);

	if (test_fail)
		DBUG_VOID_RETURN;

	/* Test with linkname field full (100 chars, or 99 in XPG3 mode) */

	tlong_file[0] = '\0';
	for (i = 0; i < 8; i++)
		(void) strcat(tlong_file, "12345678901234/");
	tlong_file[FULL_LNK] = '\0';

#ifdef TESTBED
	tlong_file[FULL_LNK-1] = '\0';
#endif

	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write the regular file details */
	(void) strncpy(hdr.tar_name, tlong_file, (size_t)TNAMLEN);
	hdr.tar_typeflag[0] = REGTYPE;

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	/* Write a tar format header for a linked file */
	(void) strncpy(hdr.tar_linkname, hdr.tar_name, (size_t)TLNKLEN);
	(void) strncpy(hdr.tar_name, tlonglink_file, (size_t)TNAMLEN);

	/* Add "typeflag" '1' (LNKTYPE) */
	hdr.tar_typeflag[0] = LNKTYPE;

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;		
	init_trail(&trl);

	globok = 0; 	
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* 
	 * Ensure the expected file (and one it is linked to) does not exist.
	 */

	(void) unlink(tlong_file);
	(void) unlink(tlonglink_file);
	if(stat(tlong_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tlong_file);
	}
	else
		PATH_TRACE;
		
	if(stat(tlonglink_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tlonglink_file);
	}
	else
		PATH_TRACE;
		
	if(test_fail)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}

	/* Extract the file */

	globok = 0; 	
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(stat(tlonglink_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("file %s was not extracted from archive", tlong_file);
		in_rpt("stat(%s) failed - errno %d", tlonglink_file, err);
	}
	else if(!(S_ISREG(stbuf.st_mode)) || stbuf.st_nlink <= 1)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("linked file was not extracted using typeflag \"%c\"",
							hdr.tar_typeflag[0]);
		if((s = file_type(&stbuf)) != (char *)NULL)
			in_rpt("instead %s was extracted", s);
	}
	else
		PATH_TRACE;
		
	(void) unlink(tlong_file);
	(void) unlink(tlonglink_file);
	(void) close_tar(tar_type, tar_fd);

	if(!test_fail)
		PATH_XS_RPT(14); 

	DBUG_VOID_RETURN;
}

public void
test28()
{
	int	tar_fd;
	int	err;
	long	total;
	struct	stat stbuf;
	char	*s;
	int	pathok = 0;
	
	/*
	 * Test that when reading an archive, a "typeflag" of '5' will
	 * cause a directory file to be generated.
	 */

	DBUG_ENTER("test28");

	test_fail = 0;

	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write it with just a simple file name */
	(void) strncpy(hdr.tar_name, tnew_dir, (size_t)TNAMLEN);

	/* Add "typeflag" '5' (DIRTYPE) */
	hdr.tar_typeflag[0] = DIRTYPE;

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	init_trail(&trl);

	globok = 0; 	
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
			
	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(tnew_dir);
	if(stat(tnew_dir, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnew_dir);
	}
	else
		PATH_TRACE;
		
	if(test_fail)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	 
	/* Extract the file */

	globok = 0; 	
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if(stat(tnew_dir, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("file %s was not extracted from archive", tnew_dir);
		in_rpt("stat(%s) failed - errno %d", tnew_dir, err);
	}
	else if(!(S_ISDIR(stbuf.st_mode)))
	{
		xx_rpt(FAILURE);
		in_rpt("directory file was not extracted using typeflag \"%c\"",
							hdr.tar_typeflag[0]);
		if((s = file_type(&stbuf)) != (char *)NULL)
			in_rpt("instead %s was extracted", s);
		(void) unlink(tnew_dir);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(5);
	}

	(void) rmdir(tnew_dir);
	(void) close_tar(tar_type, tar_fd);
	
	DBUG_VOID_RETURN;
}

#if TEST_XNFS

public void
test29()
{
	DBUG_ENTER("test29");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	

public void
test29()
{
	int	tar_fd;
	int	err;
	long	total;
	struct	stat stbuf;
	char	*s;
	int	pathok = 0;
	
	/*
	 * Test that when reading an archive, a "typeflag" of '6' will
	 * cause a fifo to be generated.
	 */

	DBUG_ENTER("test29");

	test_fail = 0;

	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write it with a fifo type name */
	(void) strncpy(hdr.tar_name, tnew_fifo, (size_t)TNAMLEN);

	/* Add "typeflag" '6' (FIFOTYPE) */
	hdr.tar_typeflag[0] = FIFOTYPE;

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	init_trail(&trl);

	globok = 0; 
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
			
	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(tnew_fifo);
	if(stat(tnew_fifo, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnew_fifo);
	}
	else
		PATH_TRACE;
		
	if(test_fail)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	 
	/* Extract the file */

	globok = 0; 	
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if(stat(tnew_fifo, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("file %s was not extracted from archive", tnew_fifo);
		in_rpt("stat(%s) failed - errno %d", tnew_fifo, err);
	}
	else if(!(S_ISFIFO(stbuf.st_mode)))
	{
		xx_rpt(FAILURE);
		in_rpt("fifo was not extracted using typeflag \"%c\"",
							hdr.tar_typeflag[0]);
		if((s = file_type(&stbuf)) != (char *)NULL)
			in_rpt("instead %s was extracted", s);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	(void) unlink(tnew_fifo);
	(void) close_tar(tar_type, tar_fd);
	
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
test36()
{
	/*
	 * Test that when reading an archive, a "typeflag" of '\0' will
	 * cause a regular file to be generated.
	 */

	int i;
	int	pathok = 0;
	
	DBUG_ENTER("test36");

	test_fail = 0;

	for (i = 0; i < sizeof(outbuf); i++)
		outbuf[i] = i % 256;

	globok = 0;
	regextract(AREGTYPE, 0);
	PATH_FUNC_TRACE;
	
	globok = 0;
	regextract(AREGTYPE, 1);
	PATH_FUNC_TRACE;

	globok = 0;
	regextract(AREGTYPE, TBLKSIZ);
	PATH_FUNC_TRACE;

	globok = 0;
	regextract(AREGTYPE, TBLKSIZ + 1);
	PATH_FUNC_TRACE;

	globok = 0;
	regextract(AREGTYPE, 2 * TBLKSIZ);
	PATH_FUNC_TRACE;
	

	if (test_fail == 0)
	{
		PATH_XS_RPT(5); 
	}

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

public void
test37()
{
	DBUG_ENTER("test37");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	

public void
test37()
{
	int	tar_fd;
	int	err;
	long	total;
	struct	stat stbuf;
	char	*s;
	int	pathok = 0;

	/* size field is ignored for typeflag '6' (FIFOTYPE) */

	DBUG_ENTER("test37");

	test_fail = 0;

	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write it with a fifo type name */
	(void) strncpy(hdr.tar_name, tnew_fifo, (size_t)TNAMLEN);

	/* Add "typeflag" '6' (FIFOTYPE) */
	hdr.tar_typeflag[0] = FIFOTYPE;

	/* Add size field (512) */
	ltoaoct((long)TBLKSIZ, hdr.tar_size, TSIZLEN);

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		

	/* If size field is not ignored, this header will be taken
	   as data blocks for the FIFO */

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write it with just a simple file name */
	(void) strncpy(hdr.tar_name, tnew_file, (size_t)TNAMLEN);

	/* Add "typeflag" '0' (REGTYPE) */
	hdr.tar_typeflag[0] = REGTYPE;

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	init_trail(&trl);

	globok = 0; 	
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
			
	/* 
	 * Ensure the expected files do not exist.
	 */

	(void) unlink(tnew_fifo);
	if(stat(tnew_fifo, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("could not remove test file \"%s\"", tnew_fifo);
	}
	else
		PATH_TRACE;
		
	(void) unlink(tnew_file);
	if(stat(tnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("could not remove test file \"%s\"", tnew_file);
	}
	else
		PATH_TRACE;
		
	if(test_fail)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	 
	/* Extract the files */

	globok = 0; 	
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	if(stat(tnew_fifo, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("file %s was not extracted from archive", tnew_fifo);
		in_rpt("stat(%s) failed - errno %d", tnew_fifo, err);
	}
	else if(!(S_ISFIFO(stbuf.st_mode)))
	{
		xx_rpt(FAILURE);
		in_rpt("fifo was not extracted using typeflag '%c', size %d",
							FIFOTYPE, TBLKSIZ);
		if((s = file_type(&stbuf)) != (char *)NULL)
			in_rpt("instead %s was extracted", s);
	}
	else if(stat(tnew_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("regular file in header block following header with");
		in_rpt("typeflag '%c', size %d was not extracted from archive",
							FIFOTYPE, TBLKSIZ);
		in_rpt("stat(%s) failed - errno %d", tnew_file, err);
	}
	else if(!(S_ISREG(stbuf.st_mode)))
	{
		xx_rpt(FAILURE);
		in_rpt("regular file in header block following header with");
		in_rpt("typeflag '%c', size %d was not extracted from archive",
							FIFOTYPE, TBLKSIZ);
		if((s = file_type(&stbuf)) != (char *)NULL)
			in_rpt("instead %s was extracted", s);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(7);
	}

	(void) unlink(tnew_fifo);
	(void) unlink(tnew_file);
	(void) close_tar(tar_type, tar_fd);
	
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
test38()
{
	char	**p;
	int	tar_fd, fd;
	int	i, err;
	long	total;
	int	pathok = 0;

	DBUG_ENTER("test38");

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		if (eaccess(*p, F_OK) == 0 && unlink(*p) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not unlink \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
	}

	test_fail = 0;

	for (p = portfnam; *p != NULL; p++)
	{
		if((tar_fd = new_tar(tar_file)) == SYSERROR)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Initialise a tar format header */
		init_head(&hdr);

		/* set fields for 1 byte regular file */
		(void) strncpy(hdr.tar_name, *p, (size_t)TNAMLEN);
		ltoaoct(1L, hdr.tar_size, TSIZLEN);

		total = calc_sum();
		ltoaoct(total, hdr.tar_chksum, TSUMLEN);

		globok = 0; 
		if(write_head(tar_fd) == SYSERROR) 
		{
			(void) close_tar(tar_file, tar_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
			
		/* write identifying byte */
		inbuf[0] = 'A' + (p - portfnam);
		if (write(tar_fd, inbuf, TBLKSIZ) != TBLKSIZ)
		{
			if (!test_fail++)
				xx_rpt(DELETION);
			in_rpt("write() failed - errno %d (%s)",
				errno, errname(errno));
			(void) close_tar(tar_file, tar_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		init_trail(&trl);

		globok = 0; 	
		if(write_trail(tar_fd, &trl) == SYSERROR) 
		{
			(void) close_tar(tar_file, tar_fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
			
		/* Extract the file */

		globok = 0; 	
		if(extract_tar(tar_file, NORESTORE) == SYSERROR)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE; 
		
		(void) close_tar(tar_file, tar_fd);
	}

	/* check the contents of the files */

	for (p = portfnam; *p != NULL; p++)
	{
		fd = open(*p, O_RDONLY);
		if (fd < 0)
		{
			err = errno;
			if (!test_fail++)
				xx_rpt(FAILURE);
			in_rpt("open(\"%s\", O_RDONLY) failed - errno %d (%s)",
				*p, err, errname(err));
		}
		else if (read(fd, inbuf, 1) != 1)
		{
			err = errno;
			if (!test_fail++)
				xx_rpt(DELETION);
			in_rpt("read from \"%s\" failed - errno %d (%s)",
				*p, err, errname(err));
		}
		else if (inbuf[0] != 'A' + (p - portfnam))
		{
			err = errno;
			if (!test_fail++)
				xx_rpt(FAILURE);
			in_rpt("file \"%s\" contained '%s', expected '%c'",
				pr_char(inbuf[0]), 'A' + (p - portfnam));
		}
		else
			PATH_TRACE; 

		(void) close(fd);
	}

	if (!test_fail)
		PATH_XS_RPT(1 + 6 * (p - portfnam));

	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);
	
	DBUG_VOID_RETURN;
}
