/*
*      SCCS:  @(#)  POSIX.os/dataform/tar/tar3.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************
  
NAME:		T.tar

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the IEEE Std. 1003.1-1988 "Extended tar format".
	This file contains the tests in 10.1 "Archive/Interchange File Format"

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Fri Jun  9 14:30:43 BST 1989
MODIFICATIONS:

		Anthony Lee, 27 July 1990
		Addded resilience code.
		
		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Calum Laing, Mon Nov 23 14:19:19 GMT 1992
			Tests 34, 35, 39, 40 for TEST_XNFS mode.
		
		Geoff Clare, 11 March 1994
			Increase size of args[] in exec_suproc().

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<grp.h>
#include	<pwd.h>
#include	<exitcodes.h>
#include	<tar.h>
#include	"tartest.h"
#include 	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


extern	char	*execpath();
extern	char	*newstr();
extern	char	*itoa();
extern	char	*print_field();
extern	void	ltoaoct();
extern	long	calc_sum();

extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;
extern	int	vguid0;
extern	int	vguid1;
extern	int	vggid0;
extern	int	vggid1;
extern	int	test_fail;

extern	char	*treg_file;
extern	char	*tar_type;
extern	char	*tar_file;
extern	char	*tnew_file;
extern	struct	tar_head hdr;
extern	struct	tar_trail trl;

private	char	*prog_su = "tar_su";
private char	*prog_uids = "tar_uids";

#if TEST_XNFS
private char	*prog_gid = "tar_gid";
private char	*t39mnt   = "tar_t39mnt";
#endif /* TEST_XNFS */


private void
exec_suproc(eff_uid, eff_gid, exp_uid, exp_gid)
int	eff_uid;
int	eff_gid;
int	exp_uid;
int	exp_gid;
{
	char	*args[10];
	char	*teststr;
	char	*euid;
	char	*egid;
	char	*expuid;
	char	*expgid;
	char	*prog_uidsbuf[_POSIX_PATH_MAX];
	int	err;

	DBUG_ENTER("exec_suproc");

	if ((teststr = newstr(itoa(tet_thistest))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((euid = newstr(itoa(eff_uid))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((egid = newstr(itoa(eff_gid))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((expuid = newstr(itoa(exp_uid))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((expgid = newstr(itoa(exp_gid))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}

	(void)strcpy((char *)prog_uidsbuf, (char*)execpath(prog_uids));

	args[0] = (char*) execpath(prog_su);
	args[1] = teststr;
	args[2] = euid;
	args[3] = egid;
	args[4] = expuid;
	args[5] = expgid;
	args[6] = vg_ident;
	args[7] = (char *)prog_uidsbuf;
	args[8] = vg_dbug;
	args[9] = (char *)NULL;

	(void) tet_exec(execpath(prog_su), args, environ);
	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s,..) failed errno %d", execpath(prog_su), err);

	DBUG_VOID_RETURN;
}

private void
test_uids(testfn)
void	(*testfn)();
{
	int	err;
	int	tar_fd;
	long	total;
	struct	stat stbuf;
	int	pathok = 0;
	
	DBUG_ENTER("test_uids");

	test_fail = 0;

	/* Obtain stat structure to use in initialising tar format header */
	if(stat(treg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d", treg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write it with just a simple file name */
	(void) strncpy(hdr.tar_name, tnew_file, (size_t)TNAMLEN);

	/* Add "typeflag" '0' (REGTYPE) */
	hdr.tar_typeflag[0] = REGTYPE;

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	init_trail(&trl);

	globok = 0; 	
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	/* Do the extraction in a process which is set (user/group) ID */
	(void) cppair(testfn, NULLFN, WAITTIME, 0);

	(void) close_tar(tar_type, tar_fd);

	DBUG_VOID_RETURN;
}

private void
ch_t31()
{
	DBUG_ENTER("ch_t31");

	exec_suproc(vguid1, vggid1, vguid1, vggid1);

	DBUG_VOID_RETURN;
}

public void
test31()
{
	DBUG_ENTER("test31");

	/*
	 * Test that the user ID of the extracted file is set to the process's
	 * effective user ID when used by a non-privileged process.
	 */

	test_uids(ch_t31);

	DBUG_VOID_RETURN;
}

private void
ch_t32()
{
	DBUG_ENTER("ch_t32");

	exec_suproc(vguid1, vggid1, vguid1, vggid1);

	DBUG_VOID_RETURN;
}

public void
test32()
{
	DBUG_ENTER("test32");

	/*
	 * Test that the group ID of the extracted file is set to the process's
	 * effective group ID when used by a non-privileged process.
	 */

	test_uids(ch_t32);

	DBUG_VOID_RETURN;
}

private void
ch_t33()
{
	DBUG_ENTER("ch_t33");

	exec_suproc(vguid1, vggid1, vguid1, vggid1);

	DBUG_VOID_RETURN;
}

public void
test33()
{
	DBUG_ENTER("test33");

	/*
	 * Test that the file permission of an extracted file is set to the 
	 * file permission value as supplied by the c_mode field in the archive
	 * appropriately modified by the current umask() value
	 * when used by a non-privileged process.
	 */

	test_uids(ch_t33);

	DBUG_VOID_RETURN;
}

private void
ch_t34()
{
	DBUG_ENTER("ch_t34");

	exec_suproc(SU_UID, vggid1, vguid0, vggid0);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
ch_t34_xnfs()
{
        int     fd;

        DBUG_ENTER("ch_t34_xnfs");

	/*
	 * Create file in a directory allowing root access.
	 */
		 
	(void) unlink(treg_file);

	fd = crfile(treg_file, (mode_t)MODEANY, (int)geteuid(), (int)getegid());
	if(fd >= 0) {
		/* Write something to make it not a zero length file */
		(void) write(fd, REGDATA, (unsigned) strlen(REGDATA));
	}
	(void) close(fd);

        test_uids(ch_t34);

	(void) unlink(treg_file);

        DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
test34()
{
	DBUG_ENTER("test34");

#if TEST_XNFS
	xnfs_doroot_c(ch_t34_xnfs, WAITTIME, SP_NOSYNC, SP_NOSYNC);
#else
	/*
	 * Test that ownerships and permissions are restored exactly as 
	 * recorded in the archive when used by a privileged process.
	 */

	test_uids(ch_t34);
#endif /* TEST_XNFS */
	DBUG_VOID_RETURN;
}

private void
ch_t35()
{
	DBUG_ENTER("ch_t35");

	exec_suproc(SU_UID, vggid0, vguid1, vggid1);

	DBUG_VOID_RETURN;
}


private void
test35proc()
{
	int	err;
	int	tar_fd;
	long	total;
	struct	stat stbuf;
	struct	group *grp;
	struct	passwd *pwd;
	int	pathok = 0;
	
	/*
	 * Test that a privileged protection preserving version of the
	 * format reading utility will scan password and group files for
	 * symbolic names "uname" and "gname" if "magic" field is TMAGIC.
	 */
	
	DBUG_ENTER("test35proc");

	test_fail = 0;

	/* Obtain stat structure to use in initialising tar format header */
	if(stat(treg_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d", treg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if((tar_fd = open_tar(tar_file, treg_file)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Read header for the regular file */
	if(read_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_file, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) close_tar(tar_file, tar_fd);

	/* Test the "magic" field */

	if(strncmp(TMAGIC, hdr.tar_magic, strlen(TMAGIC)))
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("\"magic\" field of tar format archive is not \"%s\"", 
									TMAGIC);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Create an empty archive file */
	if((tar_fd = new_tar(tar_type)) == SYSERROR)
		DBUG_VOID_RETURN;

	/* Initialise a tar format header */
	init_head(&hdr);

	/* Write it with just a simple file name */
	(void) strncpy(hdr.tar_name, tnew_file, (size_t)TNAMLEN);

	/* Add "typeflag" '0' (REGTYPE) */
	hdr.tar_typeflag[0] = REGTYPE;

	/* Put in the "uname" and "gname" fields for another user ID */
	if((pwd = getpwuid((uid_t)vguid1)) != (struct passwd *)NULL)
		(void) strncpy(hdr.tar_uname, pwd->pw_name, (size_t)TUGNAMLEN);
	hdr.tar_uname[TUGNAMLEN-1] = '\0';
	if((grp = getgrgid((gid_t)vggid1)) != (struct group *)NULL) 
		(void) strncpy(hdr.tar_gname, grp->gr_name, (size_t)TUGNAMLEN);
	hdr.tar_gname[TUGNAMLEN-1] = '\0';

	total = calc_sum();
	ltoaoct(total, hdr.tar_chksum, TSUMLEN);

	globok = 0; 
	if(write_head(tar_fd) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	init_trail(&trl);

	globok = 0; 	
	if(write_trail(tar_fd, &trl) == SYSERROR) 
	{
		(void) close_tar(tar_type, tar_fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
 		

	/* Do the extraction in a process which is set user ID */
	(void) cppair(ch_t35, NULLFN, WAITTIME, 0);

	(void) close_tar(tar_type, tar_fd);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test35xnfs()
{
	int	fd;

	DBUG_ENTER("test35xnfs");

	/*
	 * Create file in a directory allowing root access.
	 */
		 
	(void) unlink(treg_file);

	fd = crfile(treg_file, (mode_t)MODEANY, (int)geteuid(), (int)getegid());
	if(fd >= 0) {
		/* Write something to make it not a zero length file */
		(void) write(fd, REGDATA, (unsigned) strlen(REGDATA));
	}
	(void) close(fd);

        test35proc();

	(void) unlink(treg_file);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
test35()
{
	DBUG_ENTER("test35");
#if TEST_XNFS
	xnfs_doroot_c(test35xnfs, WAITTIME, SP_NOSYNC, SP_NOSYNC);
#else
	test35proc();
#endif /* TEST_XNFS */
	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
mount39ch()
{
	char	*args[9];
	int	err;

	DBUG_ENTER("mount39ch");

	args[0] = (char *) execpath(prog_gid);
	args[1] = newstr(itoa(MOUNT_GIDTRUE));
	args[2] = t39mnt;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = (char *)NULL;

	(void) tet_exec(execpath(prog_gid), args, environ);
	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s,..) failed errno %d", execpath(prog_gid), err);

	DBUG_VOID_RETURN;
}

private void
umount39ch()
{
	char	*args[9];
	int	err;

	DBUG_ENTER("umount39ch");

	args[0] = (char *) execpath(prog_gid);
	args[1] = newstr(itoa(UNMOUNT_GIDTRUE));
	args[2] = t39mnt;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = (char *)NULL;

	(void) tet_exec(execpath(prog_gid), args, environ);
	err = errno;
	xx_rpt(DELETION);

	in_rpt("tet_exec(%s,..) failed errno %d", execpath(prog_gid), err);

	DBUG_VOID_RETURN;
}

private void
ch_t39()
{
	DBUG_ENTER("ch_t39");

	exec_suproc(vguid1, vggid1, vguid1, vggid0);

	DBUG_VOID_RETURN;
}

private void
t39ch1()
{
	int	err;
	int	fd;

	DBUG_ENTER("t39ch1");

	if(chdir(t39mnt) == SYSERROR){
                err = errno;
                xx_rpt(DELETION);
                in_rpt("could not chdir(%s) errno %d", t39mnt, err);
                DBUG_VOID_RETURN;
        }

	(void) unlink(treg_file);

	/* Make file with something in it */
	fd = crfile(treg_file, (mode_t)MODEANY, (int)geteuid(), (int)getegid());
	if(fd >= 0) {
		/* Write something to make it not a zero length file */
		(void) write(fd, REGDATA, (unsigned) strlen(REGDATA));
	}

	(void) close(fd);

	test_uids(ch_t39);

	(void) unlink(treg_file);

}

#endif /* TEST_XNFS */

#if TEST_XNFS

public void
test39()
{
	int	err;

	DBUG_ENTER("test39");

	(void) rmdir(t39mnt);

	/* Create mount point - this is actually on the NFS system already */
	if (vxmkdir(t39mnt) == SYSERROR){
                err = errno;
                xx_rpt(DELETION);
                in_rpt("could not make directory %s errno %d", t39mnt, err);
                DBUG_VOID_RETURN;
        }

	if(tet_fork(mount39ch, NULLFN, WAITTIME, OK|NOK) != OK) {
		(void) rmdir(t39mnt);
		DBUG_VOID_RETURN;
	}
	(void) tet_fork(t39ch1, NULLFN, WAITTIME, OK|NOK);
	(void) tet_fork(umount39ch, NULLFN, WAITTIME, OK|NOK);
	(void) rmdir(t39mnt);
	DBUG_VOID_RETURN;
}

#else

public void
test39()
{
	DBUG_ENTER("test39");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t40()
{
	DBUG_ENTER("ch_t40");

	exec_suproc(vguid1, vggid1, vguid1, vggid1);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
test40()
{
	DBUG_ENTER("test40");

#if TEST_XNFS
	test_uids(ch_t40);
#else
	xx_rpt(NOTINUSE);
#endif /* TEST_XNFS */
	DBUG_VOID_RETURN;
}

