/*
*      SCCS:  @(#)  POSIX.os/dataform/tar/tar4.c	Rel 4.4.4	    (07/19/99)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.tar

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the IEEE Std. 1003.1-1988 "Extended tar format"
	This module contains common functions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu May 18 09:00:23 BST 1989
MODIFICATIONS:
		Geoff Clare, 6 Oct 1989
			Allow non-zero exit code from create utility if
			indicated by global may_error.  Set got_error
			to actual exit code.
		
		Geoff Clare, 31 Jan 1990
			Change tlong_file from 100 to 101 character name.
		
		Geoff Clare, 11 Apr 1990
			New long pathname in linkname error test.

		Anthony Lee, 27 July 1990
			Added resilience code.
		
		Stuart Boutell, 2 Nov 1990
			Improved execvp error reporting

		Stuart Boutell, 7 Nov 1990
			Removed vg_sjmp and implemented internal handler.

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called

		Geoff Clare, 18 Feb 1991
			Changes to remove assumption that tar_head
			members are contiguous.

		Geoff Clare, 20 July 1992
			Change savchild from int to pid_t.

		Geoff Clare, 23 July 1992
			Change aoctol() to avoid signed overflow.

		Calum Laing, Mon Nov 23 14:19:19 GMT 1992
			XNFS enhancements

		Geoff Clare, 20 Dec 1993
			Don't use DBUG_EXIT() in child of plain fork().

		Geoff Clare, 23 June 1994
			Ensure sigsetjmp() is used in line with constraints.

		Geoff Clare, 7 June 1995
			Convert headers to/from ASCII.

		Geoff Clare, 9 April 1999
			to_asc[] was 1 byte too small.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<limits.h>
#include	<stdio.h>
#include	<string.h>
#include	<time.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<ctype.h>
#include	<tar.h>
#include	<pwd.h>
#include	<grp.h>
#include	<dbug.h>
#include	<report.h>
#include	<timeout.h>
#include	<setjmp.h>
#include	<signal.h>
#include	<sys/wait.h>
#include	<exitcodes.h>
#include	"tartest.h"
#include	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif

#ifndef _XOPEN_SOURCE
#undef memset
extern	void	*memset();
#endif

#define	MAXARG		10
#define PATH_NOK	4
#define ASCII_SPACE	32

extern	char	*errname();
extern	char	*signame();
extern	void	setsigs();
#ifndef getenv
extern	char	*getenv();
#endif

private sigjmp_buf	sjmp;
private	int		signum;

public	unsigned char	from_asc[128] = {  /* ASCII to native chars */
	0,    0,    0,    0,    0,    0,    0,    0,
	'\b', '\t', '\n', '\v', '\f', '\r', 0,    0,
	0,    0,    0,    0,    0,    0,    0,    0,
	0,    0,    0,    0,    0,    0,    0,    0, 
	' ',  '!',  '"',  '#',  '$',  '%',  '&', '\'',
	'(',  ')',  '*',  '+',  ',',  '-',  '.',  '/',
	'0',  '1',  '2',  '3',  '4',  '5',  '6',  '7',
	'8',  '9',  ':',  ';',  '<',  '=',  '>',  '?',
	'@',  'A',  'B',  'C',  'D',  'E',  'F',  'G',
	'H',  'I',  'J',  'K',  'L',  'M',  'N',  'O',
	'P',  'Q',  'R',  'S',  'T',  'U',  'V',  'W',
	'X',  'Y',  'Z',  '[', '\\',  ']',  '^',  '_',
	'`',  'a',  'b',  'c',  'd',  'e',  'f',  'g',
	'h',  'i',  'j',  'k',  'l',  'm',  'n',  'o',
	'p',  'q',  'r',  's',  't',  'u',  'v',  'w',
	'x',  'y',  'z',  '{',  '|',  '}',  '~',  0,  
};
public	unsigned char	to_asc[UCHAR_MAX+1]; /* native chars to ASCII */

public	char	*tprefix =	"tar-prefix";
public	char	*treg_file =	"tar-reg";
public	char	*tlink_file =	"tar-link";
public	char	*tlinked_file =	"tar-linked";
public	char	*tlink_both =	"tar-link tar-linked";
public	char	*tdir_file =	"tar-dir";
#if !TEST_XNFS
public	char	*tfifo_file =	"tar-fifo";
#endif /* TEST_XNFS */
public	char	*tar_file =	"tar-file.a";
public	char	*tnew_file =	"tar-new";
public	char	*tnewlink_file= "tar-newlink";
public	char	*tnew_dir =	"tar-newdir";
public	char	*tnew_fifo =	"tar-newfifo";
public	char	*tar_type =	"tar-type.a";
public	char	*tlonglink_file="tar-longlink";
public	int	test_fail;
public	int	may_error = 0;
public	int	got_error = 0;
public	char	blk[TBLKSIZ];
public	char	headbuf[TBLKSIZ];
public	struct	tar_trail trl;
public	struct	tar_head hdr = {
	&headbuf[0],
	&headbuf[TNAMLEN],
	&headbuf[TNAMLEN+TMODELEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN+TSIZLEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN+TSIZLEN+TMTIMELEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN+TSIZLEN+TMTIMELEN+TSUMLEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN+TSIZLEN+TMTIMELEN+TSUMLEN+
		TTYPELEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN+TSIZLEN+TMTIMELEN+TSUMLEN+
		TTYPELEN+TLNKLEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN+TSIZLEN+TMTIMELEN+TSUMLEN+
		TTYPELEN+TLNKLEN+TMAGLEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN+TSIZLEN+TMTIMELEN+TSUMLEN+
		TTYPELEN+TLNKLEN+TMAGLEN+TVERSLEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN+TSIZLEN+TMTIMELEN+TSUMLEN+
		TTYPELEN+TLNKLEN+TMAGLEN+TVERSLEN+TUGNAMLEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN+TSIZLEN+TMTIMELEN+TSUMLEN+
		TTYPELEN+TLNKLEN+TMAGLEN+TVERSLEN+TUGNAMLEN+TUGNAMLEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN+TSIZLEN+TMTIMELEN+TSUMLEN+
		TTYPELEN+TLNKLEN+TMAGLEN+TVERSLEN+TUGNAMLEN+TUGNAMLEN+TDEVLEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN+TSIZLEN+TMTIMELEN+TSUMLEN+
		TTYPELEN+TLNKLEN+TMAGLEN+TVERSLEN+TUGNAMLEN+TUGNAMLEN+TDEVLEN+
		TDEVLEN],
	&headbuf[TNAMLEN+TMODELEN+TUGIDLEN+TUGIDLEN+TSIZLEN+TMTIMELEN+TSUMLEN+
		TTYPELEN+TLNKLEN+TMAGLEN+TVERSLEN+TUGNAMLEN+TUGNAMLEN+TDEVLEN+
		TDEVLEN+TPREFLEN]
};

private	void	sjmpfunc();
private int	fork_util();

private	char	*args[MAXARG];

public long
aoctol(str, len)
char	*str;
int	len;
{
	unsigned long	val;
	unsigned long	pow;
	int	i;
	char	digit[2];

	/* 
	 * Calculate the value of the octal value pointed
	 * to by str, which may be preceded or followed by spaces or NULLs.
	 *
	 * len is the total length of the field.
	 */
	val = 0L;
	pow = 1L;
	digit[1] = '\0';

	str = &str[len - 1];
	for(i = 0; i < len; i++, str--)
	{
		if(!isdigit(*str))
			continue;
		digit[0] = *str;
		val += pow * atol(digit);
		pow *= 8L;
	}

	return(val);
}

public long
calc_sum()
{
	int	i;
	unsigned char	*s;
	int	sum_start;
	int	sum_end;
	long	total;

	/*
	 * Calculate the checksum expected by totalling the byte values
	 * (or the values they will have after conversion to ASCII).
	 */

	sum_start = TNAMLEN + TMODELEN + 2 * TUGIDLEN + TSIZLEN + TMTIMELEN;
	sum_end   = sum_start + TSUMLEN;

	total = 0L;

	for(i = 0, s = (unsigned char *)headbuf; i < sizeof(headbuf); i++, s++)
	{
		if(i >= sum_start && i < sum_end)
		{
			/* Treat the chksum field as spaces */
			total += ASCII_SPACE;
		}
		else
			total += to_asc[*s];
	}

	return(total);
}

public void
ltoaoct(total, ptr, len)
long	total;		/* Calculated checksum value */
char	*ptr;		/* Pointer to location to format data */
int	len;		/* Length of the given field */
{
	char *s = &ptr[len];
	int	i;

	/*
	 * Format the given number as octal number 
	 * preceded by zeros and followed by a space and NULL.
	 *
	 * Converse of aoctol().
	 */

	*--s = '\0';
	*--s = ' ';

	for(i = 0; i < len-2; i++)
	{
		*--s = '0' + total % 8L;
		total /= 8L;
	}
}

public void
ch_util()
{
	int	err;

	DBUG_ENTER("ch_util");

	(void) execvp(args[0], args);
	err = errno;
	if(!test_fail++)
	in_rpt("execvp(%s) failed - errno %d(%s)", args[0], err, errname(err));
		xx_rpt(DELETION);

	DBUG_LEAVE; exit(A_EXECFAIL); /* not DBUG_EXIT */

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

public int
extract_tar(name, mode)
char	*name;	/* The name of the tar format file to extract from. */
int	mode;	/* RESTORE or NORESTORE - restore user and group ID option */
{
	int	err;
	int	ret;
	char	*tar_name;
	char	*tar_root;
	struct	stat stbuf;
	char	tar_cmd[_POSIX_PATH_MAX];
	int	pathok = 0;
	
	/* 
	 * Extract files from a named tar format file by calling the 
	 * pre-defined utility.
	 * Return an SYSERROR if test cannot proceed and is deleted, 
	 * otherwise return 0.
	 *
	 * The syntax for the format creating utility is as follows:
	 * XT_TAR_UTIL is the name of a command file which accepts
	 * a single argument which is the named tar format file.
	 * XT_ID_TAR_UTIL is the name of a command file which accepts
	 * a single argument which is the named tar format file and in
	 * addition has options incorporated to restore user and group ID.
	 * (This requires appropriate privileges).
	 * The command files are assumed to be in the TET_EXECUTE/BIN directory.
	 * 
	 *
	 * The command file should invoke the format reading utility 
	 * to extract the contents of the named tar format file.
	 *
	 * In the command file, the options should include any which 
	 * restore file modification times,
	 * and any to restore directories.
	 */

	DBUG_ENTER("extract_tar");

	if(stat(name, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("cannot read tar format file %s", name);
		in_rpt("stat() failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
		
	if(mode == RESTORE)
		tar_name = XT_ID_TAR_UTIL;
	else
		tar_name = XT_TAR_UTIL;
	tar_root = getenv("TET_EXECUTE");
	if((tar_root == (char *) NULL) || (*tar_root == '\0'))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("getenv(\"TET_EXECUTE\") failed");
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
		
	(void) strcpy(tar_cmd, tar_root);
	(void) strcat(tar_cmd, tar_name);

	args[0] = tar_cmd;
	args[1] = name;
	args[2] = (char *)NULL;

	ret = fork_util(ch_util, WAITTIME, FORMAT_READ);
	
	if(ret == SYSERROR || ret == A_EXECFAIL)
	{
		test_fail++;
		DBUG_RETURN(SYSERROR);
	}
	else if (ret == PATH_NOK)
		ret = TAR_OK;
	else
		PATH_TRACE;
		
	if(ret == A_MISC)
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("\"%s\"", tar_cmd);
		in_rpt("was called with incorrect arguments");
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4); 

	DBUG_RETURN(0);
}

public int
fork_tar(tar_cmd, name, files)
char	*tar_cmd;	/* Name of the tar command */
char	*name;		/* The name of the tar format file to create. */
char	*files;		/* The files to be placed in the tar format file. */
{
	int	ret;
	int	err;
	int	i;
	char	*s;
	char	*str;
	int	pathok = 0;
	
	/*
	 * Fork and exec format creating process to archive the named files.
	 */

	DBUG_ENTER("fork_tar");

	globok = 0;

	if((str = (char *)malloc(strlen(files) + 1)) == (char *)NULL)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("malloc() failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
		
	(void) strcpy(str, files);

	i = 0;
	args[i++] = tar_cmd;
	args[i++] = name;
	for(s = str; i < MAXARG; i++)
	{
		args[i] = strtok(s, " ");
		s = (char *)NULL;
	}

	ret = fork_util(ch_util, WAITTIME, FORMAT_CREATE);

	if(ret == PATH_NOK)
		ret = TAR_OK;
	else
		PATH_TRACE;
		
	if(ret == SYSERROR || ret == A_EXECFAIL)
	{
		free((void *)str);
		test_fail++;
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
		
	if(ret == A_MISC)
	{
		free((void *)str);
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("\"%s\"", tar_cmd);
		in_rpt("was called with incorrect arguments");
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
		
	free((void *)str);
	PATH_FUNC_RPT(4); 
	DBUG_RETURN(0);
}

public int
create_tar(name, files)
char	*name;	/* The name of the tar format file to create. */
char	*files;	/* The files to be placed in the tar format file. */
{
	int	err;
	char	*tar_name;
	char	*tar_root;
	struct	stat stbuf;
	char	tar_cmd[_POSIX_PATH_MAX];
	int	pathok = 0;
	
	/* 
	 * Create a named tar format file by calling the pre-defined utility.
	 * Return an SYSERROR if test cannot proceed and is deleted, 
	 * otherwise return 0.
	 *
	 * The syntax for the format creating utility is as follows:
	 * CR_TAR_UTIL is the name of a command file which accepts
	 * an argument which is the name of the tar format file and 
	 * a series of further arguments which are the files to be archived.
	 * 
	 * The command file is assumed to be in the TET_EXECUTE/BIN directory.
	 *
	 * The command file should invoke the format creating utility 
	 * to archive the files in the named tar format file.
	 */

	DBUG_ENTER("create_tar");

	(void) unlink(name);

	if(stat(name, &stbuf) != SYSERROR)
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable tar format file %s already exists", name);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
		
	tar_name = CR_TAR_UTIL;
	tar_root = getenv("TET_EXECUTE");
	if((tar_root == (char *) NULL) || (*tar_root == '\0'))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("getenv(\"TET_EXECUTE\") failed");
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
		
	(void) strcpy(tar_cmd, tar_root);
	(void) strcat(tar_cmd, tar_name);

	globok = 0; 
	if(fork_tar(tar_cmd, name, files) == SYSERROR)
	{
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_FUNC_TRACE;
	
	if(stat(name, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("\"%s\"", tar_cmd);
		in_rpt("did not create tar format file %s", name);
		in_rpt("stat(%s) failed - errno %d", name, err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4); 
	DBUG_RETURN(0);
}

public int
open_tar(name, files)
char	*name;	/* The name of the tar format file to create. */
char	*files;	/* The files to be placed in the tar format file. */
{
	int	err;
	int	fd;
	int	pathok = 0;
	
	/* 
	 * Create a named tar format file by calling the pre-defined utility.
	 * Return an open file descriptor to read it or SYSERROR if test 
	 * cannot proceed and is deleted.
	 */

	DBUG_ENTER("open_tar");

	if(create_tar(name, files) == SYSERROR)
	{
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_FUNC_TRACE;
		
	if((fd = open(name, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason - failed to open tar format file");
		in_rpt("open(%s, O_RDONLY) failed - errno %d", name, err);
		DBUG_RETURN(SYSERROR);
	}

	DBUG_RETURN(fd);
}

public int
close_tar(name, fd)
char	*name;	/* The name of the tar format file */
int	fd;	/* File descriptor returned by open_tar() */
{
	/*
	 * Close an open tar file and unlink the file created.
	 */

	DBUG_ENTER("close_tar");

	(void) unlink(name);
	
	DBUG_RETURN(close(fd));
}

public int
read_head(fd)
int	fd;			/* File descriptor of tar format file */
{
	unsigned char *ucp;
	int	err;
	int	bytes;
	int	pathok = 0;
	
	/*
	 * Read a tar format header from given file into buffer
	 * and convert from ASCII to native characters.
	 */

	DBUG_ENTER("read_head");

	bytes = read(fd, headbuf, sizeof(headbuf));

	if(bytes == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("read() of tar format header failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
			
	if(bytes != sizeof(headbuf))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("read() of tar format header returned %d instead of %d",
					 bytes, sizeof(headbuf));
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	for (ucp = (unsigned char *)headbuf;
	     ucp < (unsigned char *)headbuf + sizeof(headbuf); ucp++)
		*ucp = from_asc[*ucp];

	PATH_FUNC_RPT(2); 

	DBUG_RETURN(0);
}

public int
chk_head()
{
	char	*str = headbuf;
	int	len = sizeof(headbuf);
	int	i;

	/*
	 * Check a tar format header which should contain something not zero.
	 */

	DBUG_ENTER("chk_head");

	globok = 0;

	for(i = 0; i < len; str++, i++)
	{
		if(*str != '\0')
		{
			globok = 1;
			DBUG_RETURN(0);
		}
	}

	DBUG_RETURN(SYSERROR);
}

public int
new_tar(name)
char	*name;	/* The name of the tar format file to create. */
{
	int	err;
	int	fd;

	/* 
	 * Create an empty tar format file.
	 * Return an open file descriptor to write it or SYSERROR if test 
	 * cannot proceed and is deleted.
	 */

	DBUG_ENTER("new_tar");

	if((fd = creat(name, MODEANY)) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason - failed to create empty tar file");
		in_rpt("creat(%s) failed - errno %d", name, err);
		DBUG_RETURN(SYSERROR);
	}

	DBUG_RETURN(fd);
}

public int
init_head(headptr)
struct	tar_head *headptr;	/* Pointer to buffer for tar format header */
{
	uid_t	euid = geteuid();
	gid_t	egid = getegid();
	struct	passwd *pwd;
	struct	group *grp;
	long	total;

	/*
	 * Initialise common things needed when writing tar header to an
	 * empty tar format file.
	 */

	/*
	 * magic, uname and gname are null terminated character strings.
	 */
	(void) strncpy(headptr->tar_magic, TMAGIC, (size_t)TMAGLEN);
	headptr->tar_magic[TMAGLEN-1] = '\0';
	if((pwd = getpwuid(euid)) != (struct passwd *)NULL)
		(void) strncpy(headptr->tar_uname, pwd->pw_name, (size_t)TUGNAMLEN);
	headptr->tar_uname[TUGNAMLEN-1] = '\0';
	if((grp = getgrgid(egid)) != (struct group *)NULL) 
		(void) strncpy(headptr->tar_gname, grp->gr_name, (size_t)TUGNAMLEN);
	headptr->tar_gname[TUGNAMLEN-1] = '\0';

	/* 
	 * name, linkname and prefix are null terminated character strings
	 * which should be set after this function is called.
	 * Also set typeflag here for a regular file.
	 */
	(void) memset((void *)headptr->tar_name, '\0', (size_t)TNAMLEN);
	(void) memset((void *)headptr->tar_linkname, '\0', (size_t)TLNKLEN);
	(void) memset((void *)headptr->tar_prefix, '\0', (size_t)TPREFLEN);
	headptr->tar_typeflag[0] = REGTYPE;

	/*
	 * Version is a type byte field containing TVERSION (in tar.h)
	 */
	(void) strncpy(headptr->tar_version, TVERSION, (size_t)TVERSLEN);

	/*
	 * mode, uid, gid, size, mtime, chksum, devmajor, and devminor
	 * are leading zero filled octal numbers.
	 */
	ltoaoct((long)T_MODEANY, headptr->tar_mode, TMODELEN);
	ltoaoct((long)euid, headptr->tar_uid, TUGIDLEN);
	ltoaoct((long)egid, headptr->tar_gid, TUGIDLEN);
	ltoaoct((long)0, headptr->tar_size, TSIZLEN);
	ltoaoct((long)time((time_t *)NULL), headptr->tar_mtime, TMTIMELEN);
	ltoaoct((long)0, headptr->tar_devmajor, TDEVLEN);
	ltoaoct((long)0, headptr->tar_devminor, TDEVLEN);

	/*
	 * Work out chksum last - but to be useful the file name and size
	 * need to be added, and the chksum recalculated.
	 */
	total = calc_sum();
	ltoaoct(total, headptr->tar_chksum, TSUMLEN);
}

public int
write_head(fd)
int	fd;			/* File descriptor of tar format file */
{
	unsigned char *ucp1, *ucp2;
	char	convbuf[TBLKSIZ];
	int	err;
	int	bytes;
	int	pathok = 0;
	
	/*
	 * Write a tar format header from buffer into given file.
	 * Convert to ASCII before writing.
	 */

	DBUG_ENTER("write_head");

	for (ucp1 = (unsigned char *)headbuf, ucp2 = (unsigned char *)convbuf;
	     ucp1 < (unsigned char *)headbuf + sizeof(headbuf); ucp1++, ucp2++)
	{
		if (*ucp1 != 0 && to_asc[*ucp1] == 0)
		{
			/* Should never happen! */
			vsxerror("write_name() cannot convert character 0x%x to ASCII", *ucp1);
#ifdef DEBUG_HEADER
			vsxerror("at offset %ld in following header",
				(long)(ucp1 - (unsigned char *)headbuf));
			write(2, headbuf, sizeof(headbuf));
#endif
		}
		*ucp2 = to_asc[*ucp1];
	}

	bytes = write(fd, convbuf, sizeof(convbuf));

	if(bytes == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("write() of tar format header failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
			
	if(bytes != sizeof(convbuf))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("write() of tar format header returned %d instead of %d",
					 bytes, sizeof(convbuf));
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 
	DBUG_RETURN(0);
}

public int
read_trail(fd, trailptr)
int	fd;			/* File descriptor of tar format file */
struct	tar_trail *trailptr;	/* Pointer to buffer for tar format trailer */
{
	int	err;
	int	bytes;
	int	pathok = 0;
	
	/*
	 * Read a tar format trailer.
	 */

	DBUG_ENTER("read_trail");
	globok = 0;

	bytes = read(fd, trailptr->tar_zeros, sizeof(trailptr->tar_zeros));

	if(bytes == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("read() of tar format trailer failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
			
	if(bytes != sizeof(trailptr->tar_zeros))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("read() of tar format trailer returned %d instead of %d",
					 bytes, sizeof(trailptr->tar_zeros));
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);
	
	DBUG_RETURN(0);
}

public int
write_trail(fd, trailptr)
int	fd;			/* File descriptor of tar format file */
struct	tar_trail *trailptr;	/* Pointer to buffer for tar format trailer */
{
	int	err;
	int	bytes;
	int	pathok = 0;
	
	/*
	 * Write a tar format trailer.
	 */

	DBUG_ENTER("write_trail");

	bytes = write(fd, trailptr->tar_zeros, sizeof(trailptr->tar_zeros));

	if(bytes == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("write() of tar format trailer failed - errno %d", err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;
			
	if(bytes != sizeof(trailptr->tar_zeros))
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("write() of tar format trailer returned %d instead of %d",
					 bytes, sizeof(trailptr->tar_zeros));
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 
	DBUG_RETURN(0);
}

public int
read_next(fd, headptr)
int	fd;			/* File descriptor of tar format file */
struct	tar_head *headptr;	/* Pointer to buffer for tar format header */
{
	long	total;
	long	chksum;

	/*
	 * Read the next header in the tar format file.
	 */
	DBUG_ENTER("read_next");

	for( ;; ) 
	{
		/* Read header for the linked file */
		if(read_head(fd) == SYSERROR) 
			DBUG_RETURN(SYSERROR);

		/* Calculate the expected checksum */
		total = calc_sum();

		/* Obtain the value of the checksum field */
		chksum = aoctol(headptr->tar_chksum, TSUMLEN);

		if(total == chksum)
			break;
	}

	DBUG_RETURN(0);
}

public int
chk_trail(trailptr)
struct	tar_trail *trailptr;	/* Pointer to buffer for tar format trailer */
{
	char	*str = trailptr->tar_zeros;
	int	len = sizeof(trailptr->tar_zeros);
	int	i;

	/*
	 * Check a tar format trailer which should contain all zeros.
	 */

	DBUG_ENTER("chk_trail");

	for(i = 0; i < len; str++, i++)
	{
		if(*str != '\0')
			DBUG_RETURN(SYSERROR);
	}

	DBUG_RETURN(0);
}

public int
init_trail(trailptr)
struct	tar_trail *trailptr;	/* Pointer to buffer for tar format trailer */
{
	char	*str = trailptr->tar_zeros;
	int	len = sizeof(trailptr->tar_zeros);
	int	i;

	/*
	 * Ensure a tar format trailer contains all zeros.
	 */

	DBUG_ENTER("set_trail");

	for(i = 0; i < len; str++, i++)
	{
		*str = '\0';
	}

	DBUG_VOID_RETURN;
}

public int
chk_file(name)
char	*name;
{
	struct	stat stbuf;
	int	err;
	int	pathok = 0;
	
	/*
	 * Check that the given file exists.
	 */

	DBUG_ENTER("chk_file");

	if(stat(name, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		if(err == ENOENT)
			in_rpt("file %s is not available for this test", name);
		in_rpt("stat(%s) failed - errno %d", name, err);
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1); 

	DBUG_RETURN(0);
}

public int
chk_null(str, len)
char	*str;	/* Pointer to null terminated field */
int	len;	/* Maximum length of the field */
{
	int	i;

	/*
	 * Check tar format field is a null terminated character string.
	 */

	DBUG_ENTER("chk_null");

	for(i = 0; i < len; str++, i++)
	{
		if(*str == '\0')
			DBUG_RETURN(0);
		if(!isprint(*str))
			DBUG_RETURN(SYSERROR);
	}

	DBUG_RETURN(SYSERROR);
}

public int
chk_nnull(str, len)
char	*str;	/* Pointer to null terminated field */
int	len;	/* Maximum length of the field */
{
	int	i;

	/*
	 * Check tar format field is a null terminated character string
	 * except when all characters including last are non-NULL.
	 */

	DBUG_ENTER("chk_nnull");

	for(i = 0; i < len; str++, i++)
	{
		if(*str == '\0')
			DBUG_RETURN(0);
		if(!isprint(*str))
			DBUG_RETURN(SYSERROR);
	}

	DBUG_RETURN(0);
}

public int
chk_oct(str, len)
char	*str;	/* Pointer to octal number */
int	len;	/* Maximum length of the field */
{
	int	i;

	/*
	 * Check tar format field is a leading
	 * zero filled octal number followed by one or more space or 
	 * NULL characters.
	 */

	DBUG_ENTER("chk_oct");

	for(i = 0; i < len; str++, i++)
	{
		if(!isdigit(*str))
			break;
		if(*str == '8' || *str == '9')
			DBUG_RETURN(SYSERROR);
	}
	if(i <= 0)	/* No numeric part */
		DBUG_RETURN(SYSERROR);
	if(i >= len)	/* No trailing null or space */
		DBUG_RETURN(SYSERROR);

	for( ; i < len; str++, i++)
	{
		if(*str != ' ' && *str != '\0')
			DBUG_RETURN(SYSERROR);
	}

	DBUG_RETURN(0);
}

public char *
print_field(len, str)
int	len;	/* Maximum length of the field */
char	*str;	/* Pointer to start of field */
{
	private	char	buf[TBLKSIZ];
	char	*s = buf;
	int	i;

	/*
	 * Return a pointer to as much as is printable of a field. 
	 */

	for(i = 0; i < len && i < TBLKSIZ - 1; str++, i++)
	{
		if(!isprint(*str))
			break;
		*s++ = *str;
	}
	*s = '\0';

	return(buf);
}

/*
 * This is a modified version of the library function cppair().
 */

private int
fork_util(childproc, waittime, name)
void (*childproc) ();
int	waittime;
char	*name;
{
	int	rtval;
	int	err;
	int	status;
	pid_t	savchild;
	int	may_auto;
	int	pathok = 0;
	
	DBUG_ENTER("fork_util");

	may_auto = may_error;
	may_error = 0;
	got_error = 0;

	(void) fflush(stdout);
	(void) fflush(stderr);

	savchild = tet_child;

	switch (tet_child = fork())
	{
	
	case SYSERROR:
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: failed to fork %s", name);
		tet_child = savchild;
		DBUG_RETURN(SYSERROR);

	case 0:
		tet_setcontext();
		if (DBUG_SETJMP(sjmp) != 0)
		{
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("deletion reason: signal %d received", signum);
			exit(TAR_NOK); /* not DBUG_EXIT */
		}
		setsigs(sjmpfunc);
		(*childproc) ();
		/* Not reached */
		exit(TAR_OK); /* not DBUG_EXIT */

	default:
		tet_setblock();
		PATH_TRACE; 			/* To show we are in parent function */
		SET_TIMEOUT(waittime)
		rtval = waitpid(tet_child, &status, 0);
		CLEAR_ALARM

		if (rtval == SYSERROR)
		{
			err = errno;
			if(!test_fail++)
				xx_rpt(DELETION);
			if (alrm_flag > 0)
				in_rpt("deletion reason: %s timed out", name);
			else
				in_rpt("deletion reason: waitpid() failed - errno %d (%s)",
					err, errname(err));
			(void) killchild(tet_child, KILLWAIT);
		}
		else if (WIFEXITED(status))
		{
			status = WEXITSTATUS(status);
			if (status == TAR_OK || status == A_EXECFAIL || status == A_MISC)
			{
				if (status == TAR_OK && pathok != 1)
					status = PATH_NOK;
				tet_child = savchild;
				DBUG_RETURN(status);
			}
			else if (may_auto)
			{
				got_error = status;
				DBUG_RETURN( (pathok == 1 ? TAR_OK : PATH_NOK) );
			}
			else
			{
				if(!test_fail++)
					xx_rpt(DELETION);
				in_rpt("deletion reason: %s gave unexpected exit code %d", name,
					status);
			}
		}
		else if (WIFSIGNALED(status))
		{
			status = WTERMSIG(status);
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("deletion reason: %s was terminated by signal %d (%s)", name,
				status, signame(status));
		}
		else if (WIFSTOPPED(status))
		{
			status = WSTOPSIG(status);
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("deletion reason: %s was stopped by signal %d (%s)", name,
				status, signame(status));
			(void) killchild(tet_child, KILLWAIT);
		}
		else
		{
			if(!test_fail++)
				xx_rpt(DELETION);
			in_rpt("deletion reason: %s returned bad wait status (%#x)", name,
				status);
		}

		tet_child = savchild;
		DBUG_RETURN(SYSERROR);
	}
}

private void
sjmpfunc(sig)
int sig;
{
	DBUG_ENTER("sjmpfunc");
	signum = sig;
	DBUG_LONGJMP(sjmp, 1);
	DBUG_VOID_RETURN;
}
