/*
*      SCCS:  @(#)  POSIX.os/dataform/tar/tar_uids.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/dataform/tar - tar_uids Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		tar_uids - used by T.tar in several tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by tar_su in several tests.
	  usage:
		tar_uids teststr euid egid expuid expgid vg_ident vg_dbug

	  where teststr is the number of the test,
	  euid is the effective user id that is to be used in the test,
	  egid is the effective group id that is to be used in the test,
	  expuid is the expected user id of the extracted file,
	  expgid is the expected group id of the extracted file.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, Unisoft Ltd.
DATE CREATED:	Tue Jun  6 15:32:34 BST 1989
MODIFICATIONS:
		Geoff Clare, 30 Oct 1989
			Allow group ID of parent dir when non-priv

		Anthony Lee, 27 July 1990
			Added resilience code.
		
		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Peter Skan, 11 Sept 1991
			Added setprv(PRV_SETID)

		Geoff Clare, 15 Nov 1991
			Allow only parent dir group in FIPS mode

		Geoff Clare, 12 June 1992
			Test FIPS requirement in XPG4 mode.

		Calum Laing, Sat Nov 21 16:59:01 GMT 1992
			Added support for tests 39,40 when testing XNFS.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<errno.h>
#include	<setprv.h>
#include	<tar.h>
#include	"tartest.h"
#include	<tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


extern	void	set_pname();

extern	int	test_fail;
extern	char	*treg_file;
extern	char	*tar_type;
extern	char	*tnew_file;
extern	struct	tar_head hdr;
extern	char	blk[];

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	int	testnum;
private	int	euid;
private	int	egid;
private	int	expuid;
private	int	expgid;

private	void	t31();
private	void	t32();
private	void	t33();
private	void	t34();
private	void	t35();
private	void	t39();

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"tar_uids" : (argv[0]));
	if (argc == 8)
	{
		if (argv[7] != (char *) NULL)
			vg_dbug = argv[7];
		argc--;
	}
	if (argc == 7)
	{
		if (argv[6] != (char *) NULL)
			vg_ident = argv[6];
		argc--;
	}
	if (argc != 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count (%d) wrong", argc);
		return(OK);
	}
	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	euid = atoi(argv[2]);
	egid = atoi(argv[3]);
	expuid = atoi(argv[4]);
	expgid = atoi(argv[5]);

	if(geteuid() != euid)
	{
		xx_rpt(DELETION);
		in_rpt("effective UID is not %d", euid);
		return(OK);
	}
	if(getegid() != egid)
	{
		xx_rpt(DELETION);
		in_rpt("effective GID is not %d", egid);
		return(OK);
	}

	switch((testnum = atoi(argv[1])))
	{
		case 31:
			t31();
			break;

		case 40:
		case 32:
			t32();
			break;
		case 33:
			t33();
			break;
		case 34:
			t34();
			break;
		case 35:
			t35();
			break;
		case 39:
			t39();
			break;
		default:
			xx_rpt(DELETION);
			in_rpt("unknown test number %d", testnum);
			break;
	}

	return(OK);
}

private void
t31()
{
	int	err;
	struct	stat stbuf;
	int	pathok = 0;
	
	/*
	 * Test that the user ID of an extracted file is set to the process's
	 * effective user ID when used by a non-privileged process.
	 */

	DBUG_ENTER("t31");

	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(tnew_file);
	if(stat(tnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnew_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			 
	/* Extract the file */
	globok = 0; 	
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	if(stat(tnew_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("regular file was not extracted from tar archive");
		in_rpt("stat(%s) failed - errno %d", tnew_file, err);
	}
	else if(stbuf.st_uid != expuid)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("user ID of file incorrect for non-privileged process");
		in_rpt("expected user ID: %d; observed user ID: %d",
			expuid, (int)stbuf.st_uid);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(3);
	}
	
	(void) unlink(tnew_file);
	
	DBUG_VOID_RETURN;
}

private void
t32()
{
	int	err;
	gid_t	dirgid;
	struct	stat stbuf;
	int	pathok = 0;
	
	/*
	 * Test that the group ID of an extracted file is set to the process's
	 * effective group ID when used by a non-privileged process.
	 */

	DBUG_ENTER("t32");

	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(tnew_file);
	if(stat(tnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnew_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Find group ID of parent directory of new file */
	if(stat(".", &stbuf) == SYSERROR) 
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("stat(\".\") failed - errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	dirgid = stbuf.st_gid;
	 
	/* Extract the file */

	globok = 0; 	
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	if(stat(tnew_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("regular file was not extracted from tar archive");
		in_rpt("stat(%s) failed - errno %d", tnew_file, err);
	}
#if TEST_FIPS || TEST_XPG > 3
	else if(stbuf.st_gid != dirgid)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("group ID of file incorrect for non-privileged process");
		in_rpt("expected group ID: %d; observed group ID: %d",
			(int)dirgid, (int)stbuf.st_gid);
	}
#else
	else if(stbuf.st_gid != expgid && stbuf.st_gid != dirgid)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("group ID of file incorrect for non-privileged process");
		in_rpt("expected group ID: %d or %d; observed group ID: %d",
			expgid, (int)dirgid, (int)stbuf.st_gid);
	}
#endif
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(4);
	}
	
	(void) unlink(tnew_file);
	
	DBUG_VOID_RETURN;
}

private void
t33()
{
	int	err;
	struct	stat stbuf;
	struct	stat orig;
	int	pathok = 0;
	
	/*
	 * Test that the file permission of an extracted file is set to the 
	 * file permission value as supplied by the c_mode field in the archive
	 * appropriately modified by the current umask() value
	 * when used by a non-privileged process.
	 */

	DBUG_ENTER("t33");

	/*
	 * Obtain original file details.
	 */

	if(stat(treg_file, &orig) == SYSERROR) 
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d", treg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(tnew_file);
	if(stat(tnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnew_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Alter the current umask value */
	(void) umask(UMASK);
	 
	/* Extract the file */

	globok = 0; 	
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	if(stat(tnew_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("regular file was not extracted from tar archive");
		in_rpt("stat(%s) failed - errno %d", tnew_file, err);
	}
	else if((orig.st_mode & S_MODEMASK) == (stbuf.st_mode & S_MODEMASK)) 
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("file permissions incorrect for non-privileged process");
		in_rpt("expected value: 0%o; observed value: 0%o", 
					(orig.st_mode & S_MODEMASK & ~UMASK),
					(stbuf.st_mode & S_MODEMASK));
		in_rpt("current umask value was ignored");
	}
	else if((orig.st_mode & S_MODEMASK & ~UMASK) != (stbuf.st_mode & S_MODEMASK)) 
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("file permissions incorrect for non-privileged process");
		in_rpt("expected value: 0%o; observed value: 0%o", 
					(orig.st_mode & S_MODEMASK & ~UMASK),
					(stbuf.st_mode & S_MODEMASK));
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(4);
	}
	
	(void) unlink(tnew_file);
	
	DBUG_VOID_RETURN;
}

private void
t34()
{
	int	err;
	struct	stat stbuf;
	struct	stat orig;
	int	pathok = 0;
	
	/*
	 * Test that ownerships and permissions are restored exactly as 
	 * recorded in the archive when used by a privileged process.
	 */

	DBUG_ENTER("t34");

	if(setprv(PRV_CHOWN) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_CHOWN) failed - process does not have appropriate privileges");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if(setprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_SETID) failed - process does not have appropriate privileges");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Set the real user ID in case format utility uses this */
	if(setuid((uid_t)euid) != 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setuid(%d) failed - errno %d", euid, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/*
	 * Obtain original file details.
	 */

	if(stat(treg_file, &orig) == SYSERROR) 
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d", treg_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(tnew_file);
	if(stat(tnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnew_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Alter the current umask value */
	(void) umask(UMASK);
	 
	/* Extract the file */

	globok = 0; 
	if(extract_tar(tar_type, RESTORE)== SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	if(stat(tnew_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("regular file was not extracted from tar archive");
		in_rpt("stat(%s) failed - errno %d", tnew_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if((orig.st_mode & S_MODEMASK & ~UMASK) == (stbuf.st_mode & S_MODEMASK))
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("file permissions incorrect for privileged process");
		in_rpt("expected value: 0%o; observed value: 0%o", 
					(orig.st_mode & S_MODEMASK),
					(stbuf.st_mode & S_MODEMASK));
		in_rpt("current umask value was not ignored");
	}
	else if(orig.st_mode != stbuf.st_mode) 
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("file permissions incorrect for privileged process");
		in_rpt("expected value: 0%o; observed value: 0%o", 
					orig.st_mode, stbuf.st_mode);
	}
	else
		PATH_TRACE;
		
	if(stbuf.st_uid != expuid)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("user ID of file incorrect for privileged process");
		in_rpt("expected user ID: %d; observed user ID: %d", 
			expuid, (int)stbuf.st_uid);
	}
	else
		PATH_TRACE;
		
	if(stbuf.st_gid != expgid)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("group ID of file incorrect for privileged process");
		in_rpt("expected group ID: %d; observed group ID: %d", 
			expgid, (int)stbuf.st_gid);
	}
	else
		PATH_TRACE;
		
	if(!test_fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(11);
	}
	
	(void) unlink(tnew_file);
	
	DBUG_VOID_RETURN;
}

private void
t35()
{
	int	err;
	struct	stat stbuf;
	int	pathok = 0;
	
	/*
	 * Test that a privileged protection preserving version of the
	 * format reading utility will scan password and group files for
	 * symbolic names "uname" and "gname" if "magic" field is TMAGIC.
	 */

	DBUG_ENTER("t35");

	if(setprv(PRV_CHOWN) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_CHOWN) failed - process does not have appropriate privileges");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if(setprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_SETID) failed - process does not have appropriate privileges");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Set the real user ID in case format utility uses this */
	if(setuid((uid_t)euid) != 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setuid(%d) failed - errno %d", euid, err);
		DBUG_TET_EXIT(OK);
	}
	else
		PATH_TRACE;
		
	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(tnew_file);
	if(stat(tnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnew_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			 
	/* Extract the file */

	globok = 0; 
	if(extract_tar(tar_type, RESTORE)== SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	if(stat(tnew_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("regular file was not extracted from tar archive");
		in_rpt("stat(%s) failed - errno %d", tnew_file, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if(stbuf.st_uid != expuid)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("user ID of file incorrect for privileged process");
		in_rpt("expected user ID: %d; observed user ID: %d",
			expuid, (int)stbuf.st_uid);
	}
	else
		PATH_TRACE;
		
	if(stbuf.st_gid != expgid)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("group ID of file incorrect for privileged process");
		in_rpt("expected group ID: %d; observed group ID: %d",
			expgid, (int)stbuf.st_gid);
	}
	else
		PATH_TRACE;
		
	if(!test_fail)
	{
		PATH_XS_RPT(8); 
	}
	
	(void) unlink(tnew_file);
	
	DBUG_VOID_RETURN;
}

private void
t39()
{
	int	err;
	gid_t	dirgid;
	struct	stat stbuf;
	int	pathok = 0;
	
	/*
	 * Test that the group ID of an extracted file is set that of
	 * the directory in which the file is restored.
	 */

	DBUG_ENTER("t39");

	/* 
	 * Ensure the expected file does not exist.
	 */

	(void) unlink(tnew_file);
	if(stat(tnew_file, &stbuf) != SYSERROR) 
	{
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("unwritable test file \"%s\" already exists", tnew_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Find group ID of parent directory of new file */
	if(stat(".", &stbuf) == SYSERROR) 
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("stat(\".\") failed - errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	dirgid = stbuf.st_gid;
	 
	if(dirgid != expgid)
	{
		xx_rpt(DELETION);
		in_rpt("Parent directory had group ID %ld instead of ID %ld", (long) dirgid, (long) expgid);
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* Extract the file */

	globok = 0; 	
	if(extract_tar(tar_type, NORESTORE) == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	if(stat(tnew_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if(!test_fail++)
			xx_rpt(DELETION);
		in_rpt("regular file was not extracted from tar archive");
		in_rpt("stat(%s) failed - errno %d", tnew_file, err);
	}
	else if(stbuf.st_gid != dirgid)
	{
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("group ID of file incorrect for non-privileged process");
		in_rpt("expected group ID: %d; observed group ID: %d",
			(int)dirgid, (int)stbuf.st_gid);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(5);
	}
	
	(void) unlink(tnew_file);
	
	DBUG_VOID_RETURN;
}

