/*
*      SCCS:  @(#)  POSIX.os/devclass/c_cflag/c_cflag.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/c_cflag - T.c_cflag Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.c_cflag

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the termios control modes

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	24 Feb 1989	
MODIFICATIONS:
		Geoff Clare, 17 July 1989
			Cure sign extension problems in csize() routine.
		
		Geoff Clare, 2 Aug 1989
			Make cflag() args in test9 look consistent (this
			does not actually alter the parity settings used)
		
		Geoff Clare, 6 June 1990
			Move modem control tests to i_modem.
			UNSUPPORTED results.

		Anthony Lee, 06 August 1990
			Added resilience code.
		
		Geoff Clare, 27 Sept 1990
			Add tcgetattr() in csize() and cflag().

		Geoff Clare, 23 May 1991
			Only clear ISTRIP if CSIZE is CS8.

		Geoff Clare, 13 May 1992
			Remove basic "PARENB set" test (covered by
			PARODD set & clear tests).

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef c_cflag
#endif


#define NO_TESTS	11
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		2
#define Strlen		(sizeof(outbuf)-1)
#define FIVE		'\037'
#define SIX		'\077'
#define SEVEN		'\177'
#define EIGHT		'\377'

extern	char	*errname();

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();

private	void	(*realfuncs[]) () = { test1,  test2,  test3,  test4,  test5,
				      test6,  test7,  test8,  test9,  test10,
				      test11, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	  twrap, 2,
	  twrap, 3,
	  twrap, 4,
	  twrap, 5,
	 twrap, 6,
	  twrap, 7,
	  twrap, 8,
	  twrap, 9,
	  twrap, 10,
	 twrap, 11,
	NULL, 0
};


private	int	unsupported = 0;
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	testfail;
private char	CHAR[]  = "\377";
private char	outbuf[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz\n";
private char	inbuf[512];

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	char *msg = NULL;
	int i;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);

		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

        (void) close(tty_fildes);
        (void) close(loop_fildes);

	DBUG_VOID_RETURN;
}

private int
flushem()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("flushem");

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_RETURN(0);
}

private int
ltsetup()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("ltsetup");

        if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

        if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (flushem() != 0)
	{
		DBUG_RETURN(-1);
	}
	else
		PATH_FUNC_TRACE; 

	PATH_FUNC_RPT(3); 
	DBUG_RETURN(0);
}

private void
csize(mask, name, ibits)
tcflag_t mask;
char *name;
int ibits;
{
	int ret, err;
	char bits = ibits;
	int	pathok = 0;

	DBUG_ENTER("csize");

	if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_cflag &= ~CSIZE;
	loop_termios_p->c_cflag |= mask;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			if (testfail++ == 0)
				xx_rpt(UNSUPPORTED);
			in_rpt("%s not supported", name);
		}
		else
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP when setting %s - errno %d (%s)",
				name, err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check if requested size was set - other things may have
	   changed as well in the above tcsetattr which would cause
	   it to succeed without setting unsupported values */
	if (tcgetattr(loop_fildes, loop_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_LOOP after setting %s - errno %d (%s)",
			name, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if ((loop_termios_p->c_cflag & CSIZE) != mask)
	{
		if (testfail++ == 0)
			xx_rpt(UNSUPPORTED);
		in_rpt("%s not supported", name);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (mask == CS8)
		loop_termios_p->c_iflag &= ~ISTRIP;
	loop_termios_p->c_lflag &= ~ICANON;
	loop_termios_p->c_cc[VMIN] = 1;
	loop_termios_p->c_cc[VTIME] = 0;

	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		PATH_FUNC_RPT(3); 		/* A Valid return point */
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (mask == CS8)
		tty_termios_p->c_iflag &= ~ISTRIP;
	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = (cc_t) 1;
	tty_termios_p->c_cc[VTIME] = (cc_t) 0;
	tty_termios_p->c_cflag &= ~CSIZE;
	tty_termios_p->c_cflag |= mask;

	if (ltsetup() != 0)
	{
		PATH_FUNC_TRACE; 
		PATH_FUNC_RPT(5); 		/* A Valid return point */
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	SET_TIMEOUT(WAITTIME)
	ret = write(tty_fildes, CHAR, 1);
	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)
	ret = write(loop_fildes, CHAR, 1);
	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(3*NAP);

	SET_TIMEOUT(WAITTIME)
	ret = read(loop_fildes, inbuf, 1);
	err = errno;
	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_LOOP returned %d, expected 1",
			ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != bits)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("sent character incorrectly - expected %#o, actual %#o",
			(unsigned char)bits, (unsigned char)inbuf[0]);
	}
	else
		PATH_TRACE; 

	SET_TIMEOUT(WAITTIME)
	ret = read(tty_fildes, inbuf, 1);
	err = errno;
	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_TTY returned %d, expected 1",
			ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != bits)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("received character incorrectly - expected %#o, actual %#o",
			(unsigned char)bits, (unsigned char)inbuf[0]);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(9); 

	DBUG_VOID_RETURN;
}

/*
 * cflag() is passed the flag names, c_cflag masks to set and clear
 * for TERMIOS_TTY and TERMIOS_LOOP, and whether the read() should succeed
 */

private void
cflag(tty_name, tty_set, tty_clear, loop_name, loop_set, loop_clear, result)
char *tty_name;
tcflag_t tty_set, tty_clear;
char *loop_name;
tcflag_t loop_set, loop_clear;
int result;
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("cflag");

        if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        loop_termios_p->c_cflag |= loop_set;
        loop_termios_p->c_cflag &= ~loop_clear;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			if (testfail++ == 0)
				xx_rpt(UNSUPPORTED);
			in_rpt("%s not supported on VSX_TERMIOS_LOOP",
				loop_name);
		}
		else
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP when setting %s - errno %d (%s)",
				loop_name, err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check if requested flags were set - other things may have
	   changed as well in the above tcsetattr which would cause
	   it to succeed without setting unsupported values */
	if (tcgetattr(loop_fildes, loop_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_LOOP after setting %s - errno %d (%s)",
			loop_name, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if ((loop_termios_p->c_cflag & (loop_set|loop_clear)) != loop_set)
	{
		if (testfail++ == 0)
			xx_rpt(UNSUPPORTED);
		in_rpt("%s not supported on VSX_TERMIOS_LOOP", loop_name);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		PATH_FUNC_RPT(3); 
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
        tty_termios_p->c_iflag |= INPCK;
        tty_termios_p->c_cflag |= tty_set;
        tty_termios_p->c_cflag &= ~tty_clear;

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			if (testfail++ == 0)
				xx_rpt(UNSUPPORTED);
			in_rpt("%s not supported on VSX_TERMIOS_TTY",
				tty_name);
		}
		else
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY when setting %s - errno %d (%s)",
				tty_name, err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check if requested flags were set - other things may have
	   changed as well in the above tcsetattr which would cause
	   it to succeed without setting unsupported values */
	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_TTY after setting %s - errno %d (%s)",
			tty_name, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if ((tty_termios_p->c_cflag & (tty_set|tty_clear)) != tty_set)
	{
		if (testfail++ == 0)
			xx_rpt(UNSUPPORTED);
		in_rpt("%s not supported on VSX_TERMIOS_TTY", tty_name);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (flushem() != 0)
	{
		PATH_FUNC_TRACE;
		PATH_FUNC_RPT(7); 
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	SET_TIMEOUT(WAITTIME)
	ret = write(loop_fildes, outbuf, Strlen);
	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(4*NAP);

	SET_TIMEOUT(WAITTIME)
	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	CLEAR_ALARM

	if (result == SUCCEED)
	{
                if (ret != Strlen)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("read() returned %d, expected %d", ret, Strlen);
			if (ret < 0)
				in_rpt("errno was set to %d (%s)",
					err, errname(err));
		}
		else if (strncmp(inbuf, outbuf, (size_t)Strlen) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("read string != written string");
		}
		else
			PATH_TRACE;		/* A valid path */
	}
	else	/* result == FAIL */
	{
		/* NB IGNPAR is set on TERMIOS_TTY */
                if (ret == Strlen)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("read() succeeded when expected to fail");
		}
		else
			PATH_TRACE;
	}

	PATH_FUNC_RPT(9); 

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* CS5 */
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	globok = 0;
        csize((tcflag_t)CS5, "CS5", FIVE);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* CS6 */
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	globok = 0;
        csize((tcflag_t)CS6, "CS6", SIX);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* CS7 */
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	globok = 0;
        csize((tcflag_t)CS7, "CS7", SEVEN);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* CS8 */
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	globok = 0;
        csize((tcflag_t)CS8, "CS8", EIGHT);
	PATH_FUNC_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* CSTOPB set (this only checks that is doesn't fail) */
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	globok = 0;
        cflag("CSTOPB", CSTOPB, 0, "CSTOPB", CSTOPB, 0, SUCCEED);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* CSTOPB clear (this only checks that is doesn't fail) */
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	globok = 0;
        cflag("~CSTOPB", 0, CSTOPB, "~CSTOPB", 0, CSTOPB, SUCCEED);
	PATH_FUNC_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* CREAD set */
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	globok = 0;
        cflag("CREAD", CREAD, 0, "defaults", 0, 0, SUCCEED);
	PATH_FUNC_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* CREAD clear */
	int	pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

	globok = 0;
        cflag("~CREAD", 0, CREAD, "defaults", 0, 0, FAIL);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test9()
{
	DBUG_ENTER("test9");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	/* PARODD set */
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	globok = 0;
        cflag("PARENB, PARODD", PARENB|PARODD, 0,
	      "PARENB, PARODD", PARENB|PARODD, 0, SUCCEED);
	PATH_FUNC_TRACE;

	globok = 0;
        cflag("PARENB, PARODD", PARENB|PARODD, 0,
	      "PARENB, ~PARODD", PARENB, PARODD, FAIL);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test11()
{
	/* PARODD clear */
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	globok = 0;
        cflag("PARENB, ~PARODD", PARENB, PARODD,
	      "PARENB, ~PARODD", PARENB, PARODD, SUCCEED);
	PATH_FUNC_TRACE;

	globok = 0;
        cflag("PARENB, ~PARODD", PARENB, PARODD,
	      "PARENB, PARODD", PARENB|PARODD, 0, FAIL);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}
