/*
*      SCCS:  @(#)  POSIX.os/devclass/c_iflag/c_iflag.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/c_iflag - T.c_iflag Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.c_iflag

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the termios input modes

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	16 Feb 1989
MODIFICATIONS:
		Geoff Clare, 2 Aug 1989
			New VSX_TERMIOS_ASYNC parameter
		
		Geoff Clare, 6 Sep 1989
			Missing tcsetattr(loop_fildes) calls in ISTRIP tests
		
		Geoff Clare, 12 Sep 1989
			Write MAX_INPUT bytes in IXOFF test, not MAX_INPUT+1
			PARENB set, INPCK clear: no char received is OK
			PARMRK set: don't check value of 'X' byte
		
		Geoff Clare, 23 Oct 1989
			Use tcsendbreak(fd, 0) in test4 for consistency.
			Check no data is lost in IXOFF test.
		
		Geoff Clare, 18 Dec 1989
			Downgrade IXOFF test to warning as POSIX.1a is
			less strict.  Test will require rewrite after
			POSIX.1a is ratified.
		
		Geoff Clare, 15 Mar 1990
			Change IXOFF test for POSIX.1a
		
		Geoff Clare, 2 May 1990
			Add IGNCR and ICRNL clear test.
			Call tcsendbreak() from separate process when it
			will cause SIGINT, to avoid interrupting itself.

		Geoff Clare, 4 June 1990
			Restore parity on tty_loop in setup.
			Set opposite parity in tests which require it.

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported

		Anthony Lee, 06 August 1990
			Added resilience code.
		
		Geoff Clare, 25 June 1991
			VSX_TERMIOS_ASYNC="N" for unsupported result.

		Geoff Clare, 29 Jan 1992
			Loop on second read() in test 19.

		Geoff Clare, 5 Mar 1992
			Introduced short delay between write() and
			tcsendbreak() in test 1.

		Stuart Boutell, 30 May 1992
			Loop over ISTRIP set and clear for tests 5 & 7.

		Stuart Boutell, 5 June 1992
			Modified strategy in test 19.
			
		Geoff Clare, 8 June 1992
			Allow for systems with no terminal output queue.
			
		Geoff Clare, 17 June 1992
			Increased cppair timeout in test 19.

		Geoff Clare, 29 March 1994
			Improved flushing technique.

		Geoff Clare, 13 Sept 1994
			Further increase to child timeout in test 19.

		Geoff Clare, 3 Jan 1996
			Altered use of sleep() calls in test 19.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <pcts_max.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef c_iflag
#endif

#ifndef _XOPEN_SOURCE
#undef memset
extern	void	*memset();
#undef memchr
extern	void	*memchr();
#endif

#define NO_TESTS	20
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		2
#define Strlen		(sizeof(string)-1)
#define NOK		002
#define E_MALLOC	004
#define E_WRITE		010
#define PATH_NOK	8

extern	char	*errname();
extern	void	delay();

extern	int	data_rate;

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	ch_t3();
private void	pa_t3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	ch_t19();
private void	test20();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();
private	void	istrip_rpt();

public	void	(*realfuncs[]) () = { test1,  test2,  test3,  test4,  test5,
				      test6,  test7,  test8,  test9,  test10,
				      test11, test12, test13, test14, test15,
				      test16, test17, test18, test19, test20,
				      NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	 twrap, 2,
	 twrap, 3,
	 twrap, 4,
	 twrap, 5,
	 twrap, 6,
	 twrap, 7,
	 twrap, 8,
	 twrap, 9,
	 twrap, 10,
	 twrap, 11,
	 twrap, 12,
	 twrap, 13,
	 twrap, 14,
	 twrap, 15,
	 twrap, 16,
	 twrap, 17,
	 twrap, 18,
	 twrap, 19,
	 twrap, 20,
	NULL, 0
};


private	int	unsupported = 0;
private	long	max_input;
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	testfail;
private int	async_unsupp;
private int	parenb_support;
private int	opppar_support;
private char	*opppar_type;
private int	sigint_rcvd;
private char	string[]  = "And they thwam and they thwam all over the dam!";
private char	inbuf[512];
private char	PAR_STR[]  = "\377\0\1";	/* mark parity string	*/
private char	BRK_STR[]  = "\377\0\0";	/* mark break string	*/
private char	Stop       = '\023';		/* ASCII DC3		*/
private char	Start      = '\021';		/* ASCII DC1		*/
private char	NL         = '\n';
private char	CR         = '\r';
private char	EIGHT_BITS = '\377';
private char	SEVEN_BITS = '\177';


private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	char *msg = NULL, *supp;
	int i;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
			      &loop_fildes, loop_termios_p) )
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
		break;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}

	supp = tet_getvar("VSX_TERMIOS_ASYNC");
	if (supp == NULL || *supp == '\0')
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i,
				"parameter VSX_TERMIOS_ASYNC is not set");
		DBUG_VOID_RETURN;
	}
	else
		async_unsupp = (*supp == 'N' || *supp == 'n');

	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 0;
	tty_termios_p->c_cc[VTIME] = 30;

	SET_TIMEOUT(WAITTIME)

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
		msg = "tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY during preparation";

	CLEAR_ALARM

	if (msg == NULL)
	{
		/* see if PARENB supported */
		tty_termios_p->c_cflag |= PARENB;

		SET_TIMEOUT(WAITTIME)

		if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
		{
			if (errno == EINVAL)
				parenb_support = 0;
			else
				msg = "tcsetattr() failed on VSX_TERMIOS_TTY when setting PARENB (errno != EINVAL)";
		}
		else
			parenb_support = 1;

		CLEAR_ALARM
	}

	if (msg == NULL)
	{
		/* see if PARENB and opposite parity supported */
		loop_termios_p->c_cflag |= PARENB;
		if ((tty_termios_p->c_cflag & PARODD) == 
		   (loop_termios_p->c_cflag & PARODD))
			loop_termios_p->c_cflag ^= PARODD;

		SET_TIMEOUT(WAITTIME)

		if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		{
			if (errno == EINVAL)
			{
				opppar_support = 0;
				if ((loop_termios_p->c_cflag & PARODD) == 0)
					opppar_type = "PARODD clear";
				else
					opppar_type = "PARODD set";
			}
			else
				msg = "tcsetattr() failed on VSX_TERMIOS_LOOP when setting PARENB and opposite parity to VSX_TERMIOS_TTY (errno != EINVAL)";
		}
		else
			opppar_support = 1;

		CLEAR_ALARM
	}

	if (msg != NULL)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}

	/* Ensure parity is same on both ports initially */

	if ((loop_termios_p->c_cflag & PARODD) !=
	     (tty_termios_p->c_cflag & PARODD))
		loop_termios_p->c_cflag ^= PARODD;

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

        (void) close(tty_fildes);
        (void) close(loop_fildes);

	DBUG_VOID_RETURN;
}

private int
flushem()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("flushem");

	/* There can be large amounts of data still in output queues,
	   so flush both sides, wait for data in transit to get across,
	   then flush both sides again. */

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4); 

	DBUG_RETURN(0);
}

private int
tsetup()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("tsetup");
 
        if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
        if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (flushem() != 0)
	{
		PATH_FUNC_TRACE; 
		PATH_FUNC_RPT(3); 
		DBUG_RETURN(-1);
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(3); 
	DBUG_RETURN(0);
}

private void
istrip_rpt(istrip)
int istrip;
{
	if (istrip) {
		in_rpt("when ISTRIP flag was set");
	} else {
		in_rpt("when ISTRIP flag was clear");
	}
}

private void
sighandler(sig)
int sig;
{
	/* count number of SIGINTs received */
	if (sig == SIGINT)
		sigint_rcvd++;
}

private void
test1()
{
	int ret, ret2, err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test1");

	/* IGNBRK set: a break condition detected on input is ignored */
	/*	       and does not affect the data read */

	testfail = 0;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_iflag |= (IGNBRK|BRKINT);
	tty_termios_p->c_lflag &= ~NOFLSH;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = sighandler;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* pause until SIGINT or SIGALRM */

	sigint_rcvd = 0;

	SET_TIMEOUT(WAITTIME/2)

	ret = write(loop_fildes, string, Strlen);
	if (ret == Strlen)
	{
		delay(NAP);
		ret2 = tcsendbreak(loop_fildes, 0);
		if (alrm_flag == 0)
			(void) pause();
	}

	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else if (ret2 == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsendbreak(fd, 0) failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else if (sigint_rcvd > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
	    	in_rpt("IGNBRK did not stop break, SIGINT received");
	}
	else
		PATH_TRACE; 

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 
	
	DBUG_VOID_RETURN;
}

private void
test2()
{
	int ret, ret2, ret3;
	char *cp;
	int	buffered;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	/* IGNBRK clear, BRKINT set, non-cntrling tty: break flushes queues */

	loop_termios_p->c_iflag &= ~IGNBRK;	/* these don't affect later */
	loop_termios_p->c_iflag |= BRKINT;	/* uses of loop_fildes      */
	loop_termios_p->c_lflag |= NOFLSH;	/* ensure NOFLSH does not
						   disable flushing on break */

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	if (buffered)
	{
		ret = tcflow(loop_fildes, TCOOFF);
		if (ret != SYSERROR)
			ret2 = write(loop_fildes, string, Strlen);
	}
	if (alrm_flag == 0)
	    	ret3 = write(tty_fildes, string, Strlen);

	CLEAR_ALARM

	if (buffered && ret == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else if (buffered && ret2 != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else if (ret3 != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* just in case ... */
	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	(void) sigaction(SIGINT, &sig, NULLSA);

	SET_TIMEOUT(WAITTIME)

	ret = tcsendbreak(tty_fildes, 0);

	CLEAR_ALARM

	if (ret == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsendbreak(fd, 0) failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	if (buffered)
	{
		ret = tcflow(loop_fildes, TCOON);
		if (ret != SYSERROR)
			ret2 = read(tty_fildes, inbuf, sizeof(inbuf));
	}
	if (alrm_flag == 0)
		ret3 = read(loop_fildes, inbuf, sizeof(inbuf));

	CLEAR_ALARM

	if (buffered && ret == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOON) failed on VSX_TERMIOS_LOOP");
	}
	else if (buffered && ret2 > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
                in_rpt("BRKINT didn't flush output, received %d bytes", ret2);
	}
	else
		PATH_TRACE;

	if (alrm_flag == 0 && ret3 > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
                in_rpt("BRKINT didn't flush input, received %d bytes", ret3);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(8); 

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int ret, ret2, ret3, err;
	char *cp;
	int	buffered;
	struct sigaction sig;
	int	pathok = 0;
	int	retval;

	DBUG_ENTER("test3");

	testfail = 0;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	/* IGNBRK clear, BRKINT set, controlling tty: break flushes
	   queues and generates SIGINT */

	tty_termios_p->c_iflag &= ~IGNBRK;
	tty_termios_p->c_iflag |= BRKINT;
	tty_termios_p->c_lflag |= NOFLSH;	/* ensure NOFLSH does not
						   disable flushing on break */

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	if (buffered)
	{
		ret = tcflow(tty_fildes, TCOOFF);
		if (ret != SYSERROR)
			ret2 = write(tty_fildes, string, Strlen);
	}
	if (alrm_flag == 0)
	    	ret3 = write(loop_fildes, string, Strlen);

	CLEAR_ALARM

	if (buffered && ret == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else if (buffered && ret2 != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed");
		DBUG_VOID_RETURN;
	}
	else if (ret3 != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sighandler;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sigint_rcvd = 0;

	globok = 0; 
	retval= cppair(ch_t3, pa_t3, WAITTIME, NOK | PATH_NOK);
	if (retval == NOK)
	{
		testfail++;
	}
	else if ( retval == PATH_NOK )  /* PATH_TRACE failure */
		retval = OK;
	else
		PATH_FUNC_TRACE; 

	if (sigint_rcvd == 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
                in_rpt("no SIGINT received on break");
	}
	else if (sigint_rcvd > 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
                in_rpt("more than one SIGINT received on break");
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	if (buffered)
	{
		ret = tcflow(tty_fildes, TCOON);
		if (ret != SYSERROR)
			ret2 = read(loop_fildes, inbuf, sizeof(inbuf));
	}
	if (alrm_flag == 0)
		ret3 = read(tty_fildes, inbuf, sizeof(inbuf));

	CLEAR_ALARM

	if (buffered && ret == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOON) failed on VSX_TERMIOS_TTY");
	}
	else if (buffered && ret2 > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
                in_rpt("BRKINT didn't flush output, received %d bytes", ret2);
	}
	else
		PATH_TRACE;

	if (alrm_flag == 0 && ret3 > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
                in_rpt("BRKINT didn't flush input, received %d bytes", ret3);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(9); 

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	(void) sigaction(SIGINT, &sig, NULLSA);

	if (tcsendbreak(loop_fildes, 0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsendbreak(fd, 0) failed on VSX_TERMIOS_LOOP");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 1)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	DBUG_ENTER("pa_t3");

	/* wait for SIGINT */
	(void) sleep(WAITTIME);
	globok = 1;			/* Needed for cppair */

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int ret, err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test4");

	/* IGNBRK, BRKINT and PARMRK clear: break read as '\0' */

	testfail = 0;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_iflag &= ~(IGNBRK | BRKINT | PARMRK);
	tty_termios_p->c_lflag |= NOFLSH;
	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 1;
	tty_termios_p->c_cc[VTIME] = 0;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = sighandler;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sigint_rcvd = 0;

	SET_TIMEOUT(WAITTIME/2)

	ret = tcsendbreak(loop_fildes, 0);

	CLEAR_ALARM

	if (ret == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsendbreak(fd, 0) failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else if (sigint_rcvd > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
	    	in_rpt("BRKINT clear did not stop break, SIGINT received");
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != '\0')
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %#o, expected 0", (unsigned char)inbuf[0]);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 
	
	DBUG_VOID_RETURN;
}

private void
test5()
{
	int ret, err;
	int istrip;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test5");

	/* IGNBRK & BRKINT clear, PARMRK set: break read as '\377' '\0' '\0' */

	testfail = 0;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for( istrip=0; istrip<=1; istrip++) {
		int	pretestfail = testfail;
		 /* For ISTRIP clear and set */
		if (tty_init(tty_fildes, tty_termios_p) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("initialisation failed on VSX_TERMIOS_TTY");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		tty_termios_p->c_iflag &= ~(IGNBRK | BRKINT);
		if (istrip) {
			tty_termios_p->c_iflag |= ISTRIP;  /* ISTRIP set */
		} else {
			tty_termios_p->c_iflag &= ~ISTRIP; /* ISTRIP clear */
		}
		tty_termios_p->c_iflag |= PARMRK;
		tty_termios_p->c_lflag |= NOFLSH;
		tty_termios_p->c_lflag &= ~ICANON;
		tty_termios_p->c_cc[VMIN] = 3;
		tty_termios_p->c_cc[VTIME] = 0;

		globok = 0; 
		if (tsetup() != 0)
		{
			istrip_rpt(istrip);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
			
		sig.sa_handler = sighandler;
		sig.sa_flags = 0;
		(void) sigemptyset(&sig.sa_mask);
		if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
				err, errname(err));
			istrip_rpt(istrip);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		sigint_rcvd = 0;

		SET_TIMEOUT(WAITTIME/2)

		ret = tcsendbreak(loop_fildes, 0);

		CLEAR_ALARM

		if (ret == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("tcsendbreak(fd, 0) failed on VSX_TERMIOS_LOOP");
			istrip_rpt(istrip);
			DBUG_VOID_RETURN;
		}
		else if (sigint_rcvd > 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("BRKINT clear did not stop break, SIGINT received");
		}
		else
			PATH_TRACE;

		(void) sleep(NAP);

		SET_TIMEOUT(WAITTIME)

		ret = read(tty_fildes, inbuf, sizeof(inbuf));
		err = errno;

		CLEAR_ALARM

		if (ret != 3)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("read() returned %d, expected 3", ret);
			if (ret < 0)
				in_rpt("errno was set to %d (%s)", err, errname(err));
		}
		else if (memcmp((void *)inbuf, (void *)BRK_STR, (size_t)3) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("read %#o %#o %#o, expected %#o %#o %#o",
				(unsigned char)inbuf[0], (unsigned char)inbuf[1],
				(unsigned char)inbuf[2], (unsigned char)BRK_STR[0],
				(unsigned char)BRK_STR[1], (unsigned char)BRK_STR[2]);
		}
		else
			PATH_TRACE;

		if(testfail != pretestfail)
			istrip_rpt(istrip);
	}

	if (testfail == 0)
		PATH_XS_RPT(11); 
	
	DBUG_VOID_RETURN;
}

private void
test6()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test6");

        /* IGNPAR set: a byte with a parity error is ignored */

	testfail = 0;

	if (parenb_support == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("PARENB is not supported by VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (opppar_support == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("(PARENB set, %s) is not supported on VSX_TERMIOS_LOOP",
			opppar_type);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_iflag |= IGNPAR;
	tty_termios_p->c_iflag |= INPCK;
	tty_termios_p->c_cflag |= PARENB;
	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 0;
	tty_termios_p->c_cc[VTIME] = 10;

	if ((tty_termios_p->c_cflag & PARODD) == 
	   (loop_termios_p->c_cflag & PARODD))
		loop_termios_p->c_cflag ^= PARODD;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, &EIGHT_BITS, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 0", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 
	
	DBUG_VOID_RETURN;
}

private void
test7()
{
	int istrip;
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test7");

	/* PARMRK set, IGNPAR clear: a byte with a parity error is
	   read as '\377' '\0' 'X' */

	testfail = 0;

	if (parenb_support == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("PARENB is not supported by VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (opppar_support == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("(PARENB set, %s) is not supported on VSX_TERMIOS_LOOP",
			opppar_type);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for( istrip=0; istrip<=1; istrip++) {
		int	pretestfail = testfail;
		 /* For ISTRIP clear and set */
		if (tty_init(tty_fildes, tty_termios_p) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("initialisation failed on VSX_TERMIOS_TTY");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		tty_termios_p->c_cflag |= PARENB;
		tty_termios_p->c_iflag |= INPCK;
		tty_termios_p->c_iflag &= ~IGNPAR;
		if (istrip) {
			tty_termios_p->c_iflag |= ISTRIP;  /* ISTRIP set */
		} else {
			tty_termios_p->c_iflag &= ~ISTRIP; /* ISTRIP clear */
		}
		tty_termios_p->c_iflag |= PARMRK;
		tty_termios_p->c_lflag &= ~ICANON;
		tty_termios_p->c_cc[VMIN] = 3;
		tty_termios_p->c_cc[VTIME] = 0;

		if ((tty_termios_p->c_cflag & PARODD) == 
		   (loop_termios_p->c_cflag & PARODD))
			loop_termios_p->c_cflag ^= PARODD;

		globok = 0; 
		if (tsetup() != 0)
		{
			istrip_rpt(istrip);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
			
		SET_TIMEOUT(WAITTIME/2)

		ret = write(loop_fildes, &PAR_STR[2], 1);

		CLEAR_ALARM

		if (ret != 1)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
			istrip_rpt(istrip);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		(void) sleep(NAP);

		SET_TIMEOUT(WAITTIME)

		ret = read(tty_fildes, inbuf, sizeof(inbuf));
		err = errno;

		CLEAR_ALARM

		/* The 'X' byte must be read, but can be any value */
		if (ret != 3)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("read() returned %d, expected 3", ret);
			if (ret < 0)
				in_rpt("errno was set to %d (%s)", err, errname(err));
		}
		else if (memcmp((void *)inbuf, (void *)PAR_STR, (size_t)2) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("read %#o %#o, expected %#o %#o",
				(unsigned char)inbuf[0], (unsigned char)inbuf[1],
				(unsigned char)PAR_STR[0], (unsigned char)PAR_STR[1]);
		}
		else
			PATH_TRACE;

		if(testfail != pretestfail)
			istrip_rpt(istrip);
	}


	if (testfail == 0)
		PATH_XS_RPT(10); 
	
	DBUG_VOID_RETURN;
}

private void
test8()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test8");

	/* PARMRK and IGNPAR clear: a byte with a parity error is
	   read as '\0' */

	if (parenb_support == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("PARENB is not supported by VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (opppar_support == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("(PARENB set, %s) is not supported on VSX_TERMIOS_LOOP",
			opppar_type);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_iflag |= INPCK;
	tty_termios_p->c_iflag &= ~IGNPAR;
	tty_termios_p->c_iflag &= ~PARMRK;
	tty_termios_p->c_cflag |= PARENB;
	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 1;
	tty_termios_p->c_cc[VTIME] = 0;

	if ((tty_termios_p->c_cflag & PARODD) == 
	   (loop_termios_p->c_cflag & PARODD))
		loop_termios_p->c_cflag ^= PARODD;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME/2)

	ret = write(loop_fildes, &EIGHT_BITS, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != '\0')
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %#o, expected 0", (unsigned char)inbuf[0]);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 
	
	DBUG_VOID_RETURN;
}


private void
test9()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test9");

        /* PARENB and INPCK set: input parity checking is enabled */

	if (parenb_support == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("PARENB is not supported by VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (opppar_support == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("(PARENB set, %s) is not supported on VSX_TERMIOS_LOOP",
			opppar_type);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        tty_termios_p->c_cflag |= PARENB;
        tty_termios_p->c_lflag &= ~ICANON;
        tty_termios_p->c_cc[VMIN] = 0;
        tty_termios_p->c_cc[VTIME] = 10;
        tty_termios_p->c_iflag &= ~ISTRIP;
        tty_termios_p->c_iflag |= INPCK;
        tty_termios_p->c_iflag &= ~IGNPAR;

	if ((tty_termios_p->c_cflag & PARODD) == 
	   (loop_termios_p->c_cflag & PARODD))
		loop_termios_p->c_cflag ^= PARODD;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, &EIGHT_BITS, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != '\0')
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %#o, expected 0", (unsigned char)inbuf[0]);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 
	
	DBUG_VOID_RETURN;
}

private void
test10()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test10");

        /* PARENB set, INPCK clear: input parity checking is disabled */

	if (parenb_support == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("PARENB is not supported by VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (opppar_support == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("(PARENB set, %s) is not supported on VSX_TERMIOS_LOOP",
			opppar_type);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        tty_termios_p->c_cflag |= PARENB;
        tty_termios_p->c_lflag &= ~ICANON;
        tty_termios_p->c_cc[VMIN] = 0;
        tty_termios_p->c_cc[VTIME] = 10;
        tty_termios_p->c_iflag &= ~ISTRIP;
        tty_termios_p->c_iflag &= ~INPCK;
        tty_termios_p->c_iflag &= ~IGNPAR;

	if ((tty_termios_p->c_cflag & PARODD) == 
	   (loop_termios_p->c_cflag & PARODD))
		loop_termios_p->c_cflag ^= PARODD;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, &SEVEN_BITS, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 0 && ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 0 or 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (ret == 1 && inbuf[0] != SEVEN_BITS)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %#o, expected %#o",
			(unsigned char)inbuf[0], (unsigned char)SEVEN_BITS);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 
	
	DBUG_VOID_RETURN;
}

private void
test11()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test11");

	/* PARMRK set, IGNPAR and ISTRIP clear: '\377' read as '\377' '\377' */

	if (parenb_support == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("PARENB is not supported by VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	/* Change parity of TERMIOS_LOOP back to same as TERMIOS_TTY */

        if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_cflag |= PARENB;
	if ((loop_termios_p->c_cflag & PARODD) !=
	     (tty_termios_p->c_cflag & PARODD))
		loop_termios_p->c_cflag ^= PARODD;  /* makes them the same */

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* See if CS8 is supported */

	if ((loop_termios_p->c_cflag & CSIZE) != CS8)
	{
		loop_termios_p->c_cflag &= ~CSIZE;
		loop_termios_p->c_cflag |= CS8;

		if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		{
			err = errno;
			if (err == EINVAL)
			{
				xx_rpt(UNSUPPORTED);
				in_rpt("CS8 not supported");
			}
			else
			{
				xx_rpt(DELETION);
				in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP when setting CS8 - errno %d (%s)",
					err, errname(err));
			}
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_iflag |= INPCK;
	tty_termios_p->c_iflag &= ~(IGNPAR|ISTRIP);
	tty_termios_p->c_iflag |= PARMRK;
	tty_termios_p->c_cflag &= ~CSIZE;
	tty_termios_p->c_cflag |= CS8;
	tty_termios_p->c_cflag |= PARENB;
	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 1;
	tty_termios_p->c_cc[VTIME] = 0;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	SET_TIMEOUT(WAITTIME/2)

	ret = write(loop_fildes, &EIGHT_BITS, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 2", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != EIGHT_BITS || inbuf[1] != EIGHT_BITS)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %#o %#o, expected %#o %#o",
			(unsigned char)inbuf[0], (unsigned char)inbuf[1],
			(unsigned char)EIGHT_BITS, (unsigned char)EIGHT_BITS);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(8); 
	
	DBUG_VOID_RETURN;
}

private void
test12()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test12");

	/* CS8 set, ISTRIP set: input characters are stripped to 7 bits */

	testfail = 0;

        if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* See if CS8 is supported */

	if ((loop_termios_p->c_cflag & CSIZE) != CS8)
	{
		loop_termios_p->c_cflag &= ~CSIZE;
		loop_termios_p->c_cflag |= CS8;

		if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		{
			err = errno;
			if (err == EINVAL)
			{
				xx_rpt(UNSUPPORTED);
				in_rpt("CS8 not supported");
			}
			else
			{
				xx_rpt(DELETION);
				in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP when setting CS8 - errno %d (%s)",
					err, errname(err));
			}
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        tty_termios_p->c_lflag &= ~ICANON;
        tty_termios_p->c_cc[VMIN] = 0;
        tty_termios_p->c_cc[VTIME] = 10;
        tty_termios_p->c_iflag |= ISTRIP;
        tty_termios_p->c_cflag &= ~CSIZE;
        tty_termios_p->c_cflag |= CS8;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, &EIGHT_BITS, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != SEVEN_BITS)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %#o, expected %#o",
			(unsigned char)inbuf[0], (unsigned char)SEVEN_BITS);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7); 
	
	DBUG_VOID_RETURN;
}

private void
test13()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test13");

	/* CS8 set, ISTRIP clear: all 8 bits of input characters are returned */

	testfail = 0;

        if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* See if CS8 is supported */

	if ((loop_termios_p->c_cflag & CSIZE) != CS8)
	{
		loop_termios_p->c_cflag &= ~CSIZE;
		loop_termios_p->c_cflag |= CS8;

		if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		{
			err = errno;
			if (err == EINVAL)
			{
				xx_rpt(UNSUPPORTED);
				in_rpt("CS8 not supported");
			}
			else
			{
				xx_rpt(DELETION);
				in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP when setting CS8 - errno %d (%s)",
					err, errname(err));
			}
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        tty_termios_p->c_lflag &= ~ICANON;
        tty_termios_p->c_cc[VMIN] = 0;
        tty_termios_p->c_cc[VTIME] = 10;
        tty_termios_p->c_iflag &= ~ISTRIP;
        tty_termios_p->c_cflag &= ~CSIZE;
        tty_termios_p->c_cflag |= CS8;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, &EIGHT_BITS, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != EIGHT_BITS)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %#o, expected %#o",
			(unsigned char)inbuf[0], (unsigned char)EIGHT_BITS);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7); 
	
	DBUG_VOID_RETURN;
}

private void
test14()
{
	int ret, err;
	int	pathok = 0;

        /* INLCR set: a received NL character is translated into a CR */

	DBUG_ENTER("test14");

	testfail = 0;

        if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 1;
	tty_termios_p->c_cc[VTIME] = 0;
	tty_termios_p->c_iflag |= INLCR;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, &NL, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != CR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %#o, expected %#o (CR)",
			(unsigned char)inbuf[0], CR);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 
	
	DBUG_VOID_RETURN;
}

private void
test15()
{
	int ret, err;
	int	pathok = 0;

        /* INLCR clear: a received NL character is not translated into a CR */

	DBUG_ENTER("test15");

	testfail = 0;

        if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 1;
	tty_termios_p->c_cc[VTIME] = 0;
	tty_termios_p->c_iflag &= ~INLCR;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, &NL, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != NL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %#o, expected %#o (NL)",
			(unsigned char)inbuf[0], NL);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 
	
	DBUG_VOID_RETURN;
}

private void
test16()
{
	int ret, err;
	int	pathok = 0;

        /* IGNCR set: a received CR character is discarded */

	DBUG_ENTER("test16");

	testfail = 0;

        if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 0;
	tty_termios_p->c_cc[VTIME] = 10;
	tty_termios_p->c_iflag |= IGNCR;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, &CR, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 0", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 
	
	DBUG_VOID_RETURN;
}

private void
test17()
{
	int ret, err;
	int	pathok = 0;

        /* ICRNL set: a received CR character is translated into a NL */

	DBUG_ENTER("test17");

	testfail = 0;

        if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 1;
	tty_termios_p->c_cc[VTIME] = 0;
	tty_termios_p->c_iflag |= ICRNL;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, &CR, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != NL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %#o, expected %#o (NL)",
			(unsigned char)inbuf[0], NL);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 
	
	DBUG_VOID_RETURN;
}

private void
test18()
{
	int ret, err;
	int	pathok = 0;

        /* IXON clear: START and STOP are placed in the input queue */

	DBUG_ENTER("test18");

	testfail = 0;

        if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 3;
	tty_termios_p->c_cc[VTIME] = 0;
	tty_termios_p->c_iflag &= ~IXON;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, &Start, 1);
	if (ret == 1)
		ret += write(loop_fildes, &Stop, 1);
	if (ret == 2)
		ret += write(loop_fildes, &Start, 1);

	CLEAR_ALARM

	if (ret != 3)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 3)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 3", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != Start || inbuf[1] != Stop || inbuf[2] != Start)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %#o %#o %#o, expected %#o %#o %#o",
			(unsigned char)inbuf[0], (unsigned char)inbuf[1],
			(unsigned char)inbuf[2], Start, Stop, Start);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 
	
	DBUG_VOID_RETURN;
}

private void
test19()
{
        /* IXOFF set: system transmits START and STOP as necessary
	   to prevent input queue overflow */

	int	pathok = 0;
	DBUG_ENTER("test19");

	testfail = 0;

	errno = 0;
	max_input = fpathconf(tty_fildes, _PC_MAX_INPUT);
	if (max_input < 0)
	{
		if (errno == 0)
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("MAX_INPUT limit not supported");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("fpathconf(_PC_MAX_INPUT) failed on VSX_TERMIOS_TTY");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (max_input > PCTS_MAX_INPUT)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("MAX_INPUT > PCTS_MAX_INPUT");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_lflag &= ~ICANON;
	loop_termios_p->c_iflag |= IXON;
	loop_termios_p->c_cc[VMIN] = 0;
	loop_termios_p->c_cc[VTIME] = 10;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 0;
	tty_termios_p->c_cc[VTIME] = 10;
	tty_termios_p->c_iflag |= IXOFF;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	PATH_FUNC_RPT(6); 			/* Check the path count in the child */

	(void) cppair(ch_t19, NULLFN,
		(int)(4*max_input/data_rate) + (50 * WAITTIME), 0);


	DBUG_VOID_RETURN;
}

private void
ch_t19()
{
	int ret, err;
	unsigned capacity, total, nbytes;
	int 	iter = 1;
	int	pathok = 0;

	DBUG_ENTER("ch_t19");

	PATH_FUNC_TRACE; 

	/*
	 * First find the capacity of the input queue (which may be greater
	 * than MAX_INPUT) by writing with flow control until a write()
	 * blocks, then reading all the input.
	 */

	alrm_flag = 0;
	while (alrm_flag == 0)
	{
		if (iter == 1)
		{
			PATH_TRACE;		/* Trace the first go */
			iter = 0;
		}

		SET_TIMEOUT(WAITTIME/2)

		ret = write(loop_fildes, string, Strlen);
		err = errno;
		
		CLEAR_ALARM

		if (ret < 0)
			break;
	}

	if (alrm_flag == 0)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcflush(loop_fildes, TCOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush(TCOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	capacity = 0;
	while ((ret = read(tty_fildes, inbuf, sizeof(inbuf))) > 0)
		capacity += ret;
	if (ret < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("first read() from VSX_TERMIOS_TTY failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	DBUG_PRINTF("test19", "capacity = %u (MAX_INPUT = %ld)",
		capacity, max_input);

	/*
	 * Now switch off flow control and fill the input queue
	 */
	
	loop_termios_p->c_iflag &= ~IXON;
	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (capacity < max_input)
		capacity = max_input;

	total = 0;
	do {
		nbytes = capacity - total;
		if (nbytes > Strlen)
			nbytes = Strlen;

		errno = 0;
		ret = write(loop_fildes, string, nbytes);

		if (ret != nbytes)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		} else {
			if (total == 0)
				PATH_TRACE;
			total += ret;
		}

	} while (total < capacity);

	(void) sleep((unsigned)(capacity/data_rate)+NAP);

	/* Read STOP character(s) */

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	if (ret < 1)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("after input queue filled to %u bytes, attempt",
			capacity);
		in_rpt("to read() STOP character(s) returned %d, expected >= 1",
			ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if (memchr((void *)inbuf, Stop, (size_t)ret) == 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("system did not transmit STOP when input queue filled to %u bytes",
			capacity);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* read input to cause START to be sent, and check no data was lost */

	total = 0;
	do {
		ret = read(tty_fildes, inbuf, sizeof(inbuf));

	} while (ret > 0 && (total += ret) < capacity);

	if (total != capacity)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("second read() loop on VSX_TERMIOS_TTY gave %u bytes, expected %u",
			total, capacity);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* Read START character */

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	if (ret < 1)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("attempt to read() START character returned %d, expected >= 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (memchr((void *)inbuf, Start, (size_t)ret) == 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("system did not transmit START after input queue emptied");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(10); 

	DBUG_VOID_RETURN;
}

private void
test20()
{
	int ret, err;
	int	pathok = 0;

        /* IGNCR and ICRNL clear: a received CR character is not altered */

	DBUG_ENTER("test20");

	testfail = 0;

        if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 1;
	tty_termios_p->c_cc[VTIME] = 0;
	tty_termios_p->c_iflag &= ~(ICRNL | IGNCR);

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, &CR, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != CR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %#o, expected %#o (CR)",
			(unsigned char)inbuf[0], CR);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 
	
	DBUG_VOID_RETURN;
}
