/*
*      SCCS:  @(#)  POSIX.os/devclass/c_iflag_X/c_iflag_X.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/c_iflag_X - T.c_iflag_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.c_iflag_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the termios input modes - X/Open extensions

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	6 Mar 1989
MODIFICATIONS:
		Geoff Clare, 25 Aug 1989
			Turn on IEXTEN to enable extensions being tested
		
		Geoff Clare, 13 Sep 1989
			Allow for default CS7 on terminals
		
		Geoff Clare, 26 Sep 1989
			Use data_rate for baud-dependent sleep time

		Geoff Clare, 8 Dec 1989
			Split range of all character values into chunks
			which ensure MAX_INPUT will not be exceeded
		
		Geoff Clare, 15 Feb 1990
			Ensure correct range is passed to ctype.h functions

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported

		Anthony Lee, 06 August 1990
			Added resilience code.

		Geoff Clare, 26 Sept 1990
			Do not send characters which are c_cc[] extensions

		Geoff Clare, 22 Nov 1990
			Only test IUCLC on characters 0 to 127.

		Geoff Clare, 23 May 1991
			Only clear ISTRIP if CSIZE is CS8.

		Geoff Clare, 8 June 1992
			Allow for systems with no terminal output queue.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef c_iflag
#endif


#define NO_TESTS	2
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		2
#define Strlen		(sizeof(string)-1)
#define E_WRITE		02

int	no_tests = NO_TESTS;

extern	char	*errname();
extern	int	data_rate;

private void	test1();
private void	test2();
private void	ch_t1();
private void	pa_t1();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();

public	void	(*realfuncs[]) () = { test1,  test2, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	  twrap, 2,
	NULL, 0
};


private	int	unsupported = 0;
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	testfail;
private char	string[]  = "And they thwam and they thwam all over the dam!";
private int	startchr;
private	char	outbuf[128];
private	char	inbuf[300];
private char	Stop       = '\023';		/* ASCII DC3		*/

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}


private void
prepare_tests()
{
	char *msg = NULL;
	int i;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p) )
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}

	tty_termios_p->c_lflag &= ~(ICANON|ISIG);
	tty_termios_p->c_iflag &= ~(IXON|ICRNL);
	if ((tty_termios_p->c_cflag & CSIZE) == CS8)
		tty_termios_p->c_iflag &= ~ISTRIP;
	tty_termios_p->c_cc[VMIN] = 1;
	tty_termios_p->c_cc[VTIME] = 30;

	SET_TIMEOUT(WAITTIME)

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
		msg = "tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY during preparation";

	CLEAR_ALARM

	if (msg != NULL)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
	}

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

        (void) close(tty_fildes);
        (void) close(loop_fildes);

	DBUG_VOID_RETURN;
}

private int
flushem()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("flushem");

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_RETURN(0);
}

private int
tsetup()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("tsetup");
 
        if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_RETURN(-1);
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_RETURN(0);
}

private void
buf_init()
{
	/* Initialise buffer of all character values, starting at startchr,
	   omitting characters which are c_cc[] extensions */

	int i, ch, ncc, strip;
	int pathok = 0;

	DBUG_ENTER("buf_init");

	PATH_TRACE;

	strip = ((tty_termios_p->c_cflag & CSIZE) == CS7);

	for (i = 0, ch = startchr; i < sizeof(outbuf); ++ch)
	{
		if (strip)
			ch &= 0177;
		else
			ch &= 0377;

		for (ncc = 0; ncc < NCCS; ncc++)
		{
			if (ch == (char)tty_termios_p->c_cc[ncc])
			{
				/* only include if it's a "known" character */
				switch (ncc)
				{
				case VEOF :
				case VEOL :
				case VERASE :
				case VINTR :
				case VKILL :
				case VQUIT :
				case VSTART :
				case VSTOP :
				case VSUSP :
                			outbuf[i++] = ch;
					break;
				
				default:
					if (ncc == VMIN || ncc == VTIME)
                				outbuf[i++] = ch;
					break;
				}
				break;
			}
		}

		if (ncc >= NCCS)	/* not found in c_cc[] */
                	outbuf[i++] = ch;
	}

	DBUG_PRINTF("buf_init", "startchr = %#o, ch after loop = %#o",
		(unsigned char)startchr, ch);

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}


private void
test1()
{
        /* IUCLC set: translate upper case to lower case */

	int ret;
	int	pathok = 0;
	int	loop = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	tty_termios_p->c_iflag |= IUCLC;
	tty_termios_p->c_lflag |= IEXTEN;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	for (startchr = 0; startchr <= 127; startchr += sizeof(outbuf))
	{
		loop++;			/* Iteration of this loop */
		globok = 0;
		buf_init();
		PATH_FUNC_TRACE; 

		globok = 0; 		
		ret = cppair(ch_t1, pa_t1, WAITTIME, E_WRITE);

		if (ret == E_WRITE)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE; 
	}
	DBUG_PRINTF("test1","loop counter is %d",loop);
	if (testfail == 0)
		PATH_XS_RPT(1 + (2 * loop)); 

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	DBUG_ENTER("ch_t1");

	if (write(loop_fildes, outbuf, sizeof(outbuf)) != sizeof(outbuf))
		DBUG_EXIT(E_WRITE);

	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	int c, i, ret, err;
	int	pathok = 0;
	int	loop = 0;			/* Expected path count in loop */

	DBUG_ENTER("pa_t1");
            
	(void) sleep((unsigned)(sizeof(outbuf)/data_rate + NAP));

	SET_TIMEOUT(WAITTIME/2)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
            
	CLEAR_ALARM

#ifdef TESTBED
	++inbuf['a'];
#endif

	if (ret != sizeof(outbuf))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, sizeof(outbuf));
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	for (i = 0; i < ret && i < sizeof(outbuf); ++i)
	{
		c = (unsigned char) outbuf[i];

		if (!isupper(c) && inbuf[i] != outbuf[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("character %#o was not transmitted unchanged, received %#o",
				(unsigned char)outbuf[i],
				(unsigned char)inbuf[i]);
		}
		else if (isupper(c) && inbuf[i] != (char) tolower(c))
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("character %#o was transmitted as %#o, expected %#o",
				(unsigned char)outbuf[i],
				(unsigned char)inbuf[i],
				(unsigned char)tolower(c));
		}
		else
		{
			PATH_TRACE;
			loop++;
		}
	}
	DBUG_PRINTF("pa_t1","loop counter is %d",loop);
	PATH_FUNC_RPT(1 + loop); 

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* IXON and IXANY set - STOP suspends, any character restarts output */

	char	*cp;
	int	buffered;
	int	ret, err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	tty_termios_p->c_iflag |= (IXON|IXANY);
	tty_termios_p->c_lflag |= IEXTEN;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* send STOP and check no output */

	SET_TIMEOUT(WAITTIME)

        ret = write(loop_fildes, &Stop, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME/2)

        ret = write(tty_fildes, string, Strlen);

	CLEAR_ALARM

	if (buffered)
	{
		if (ret != Strlen)
		{
			xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_TTY failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		if (alrm_flag == 0)
		{
			xx_rpt(DELETION);
			in_rpt("STOP character did not suspend output");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

        (void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, Strlen);
	err = errno;

	CLEAR_ALARM

	if (ret != 0)
	{
		xx_rpt(DELETION);
		in_rpt("read() returned %d, expected 0", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* send a character and check output now goes out */

	SET_TIMEOUT(WAITTIME)

        ret = write(loop_fildes, "Z", 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	if (!buffered)
	{
		SET_TIMEOUT(WAITTIME/2)

		ret = write(tty_fildes, string, Strlen);

		CLEAR_ALARM

		if (ret != Strlen)
		{
			xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_TTY failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		(void) sleep(NAP);
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, Strlen);
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(inbuf, string, (size_t)Strlen) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read string != written string");
	}
	else
		PATH_XS_RPT(7); 

	DBUG_VOID_RETURN;
}
