/*
*      SCCS:  @(#)  POSIX.os/devclass/c_lflag/c_lflag.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/c_lflag - T.c_lflag Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.c_lflag

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the termios local modes

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	28 Feb 1989
MODIFICATIONS:
		Geoff Clare, 15 Sep 1989
			Write c_cc[]'s as 1 char, not sizeof(cc_t)

		Geoff Clare, 26 Sep 1989
			Use data_rate for baud-dependent sleep time

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported
		Anthony Lee, 06 August 1990
			Added resilience code.

		Geoff Clare, 24 Jan 1992
			Loop on read() in noflsh().

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef c_lflag
#endif

#ifndef _XOPEN_SOURCE
#undef strtol
extern	long	strtol();
#endif

#define NO_TESTS	14
#define WAITTIME	(5 * SPEEDFACTOR)
#define READ_TIME	10
#define Strlen		(sizeof(outbuf)-1)
#define SubStrlen	7

int	no_tests = NO_TESTS;

extern	char	*errname();
extern	char	*psl_to_asc();

extern	int	data_rate;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();

public	void	(*realfuncs[]) () = { test1,  test2,  test3,  test4,  test5,
				      test6,  test7,  test8,  test9,  test10,
				      test11, test12, test13, test14, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	 twrap, 2,
	 twrap, 3,
	 twrap, 4,
	 twrap, 5,
	 twrap, 6,
	 twrap, 7,
	 twrap, 8,
	 twrap, 9,
	 twrap, 10,
	 twrap, 11,
	 twrap, 12,
	 twrap, 13,
	 twrap, 14,
	NULL, 0
};


private	int	unsupported = 0;
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	testfail;
private int	caught_sig;
private char	outbuf[] =  "Yes Sir, That's my baby.  No Sir, I don't mean maybe";
private char	inbuf[512];
private char	NL         = '\n';
private	unsigned char	chr;
private	unsigned int	NAP;

private void
sig_catch(sig)
int sig;
{
	caught_sig = sig;
}

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	char *msg = NULL;
	int i;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
		break;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
	}

	NAP = Strlen/data_rate + 2;

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

        (void) close(tty_fildes);
        (void) close(loop_fildes);

	DBUG_VOID_RETURN;
}

private int
flushem()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("flushem");

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 
	DBUG_RETURN(0);
}

private int
tsetup()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("tsetup");
 
        if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (flushem() != 0)
	{
		PATH_FUNC_TRACE;
		PATH_FUNC_RPT(2); 
		DBUG_RETURN(-1);
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(2); 
	DBUG_RETURN(0);
}

private char *
interp(str)
char *str;
{
	/* Copy string and interpret '\b', '\n', '\ddd', etc. */

	char *new, *p;
	int i;
	char num[4];

	DBUG_ENTER("interp");

	new = (char *)malloc(strlen(str)+(size_t)1);
	if (new == NULL)
		DBUG_RETURN(new);

	p = new;
	while (*str != '\0')
	{
		if (*str != '\\')
		{
			*p++ = *str++;
			continue;
		}

		str++;

		/* up to 3 octal digits */
		if (isdigit(*str) && *str != '8' && *str != '9')
		{
			(void) strncpy(num, str, (size_t)3);
			num[3] = '\0';
			*p++ = strtol(num, (char **)0, 8);
			/* don't trust strtol() to get end point right */
			for (i = 0; i < 3 && isdigit(*str) &&
					*str != '8' && *str != '9'; i++)
				str++;
			continue;
		}

		switch(*str)
		{
		case '\0' :	/* fallthrough */
		case '\\' :	*p++ = '\\'; break;
		case 'b'  :	*p++ = '\b'; break;
		case 'f'  :	*p++ = '\f'; break;
		case 'n'  :	*p++ = '\n'; break;
		case 'r'  :	*p++ = '\r'; break;
		case 't'  :	*p++ = '\t'; break;
		case 'v'  :	*p++ = '\v'; break;
		default :
			*p++ = '\\';
			*p++ = *str;
		}

		if (*str != '\0')
			str++;
	}

	*p = '\0';

	DBUG_RETURN(new);
}

private void
isig(name, special_char)
char *name;
cc_t special_char;
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("isig");

	chr = special_char;

	SET_TIMEOUT(WAITTIME/2)

        ret = write(loop_fildes, (char *)&chr, 1);
	if (alrm_flag == 0)
        	(void) pause();

	CLEAR_ALARM

	if (ret == 1)
	{
		SET_TIMEOUT(WAITTIME/2)
		ret += write(loop_fildes, &NL, 1);
		CLEAR_ALARM
		PATH_TRACE; 

	}
	else
		PATH_TRACE;

	if (ret != 2)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	CLEAR_ALARM

	if (ret != 2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_TTY returned %d, expected %d",
			ret, 2);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if ((unsigned char)inbuf[0] != chr)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("%s character was not in input", name);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
noflsh(name, special_char)
char *name;
cc_t special_char;
{
	int ret, err, total;
	int	pathok = 0;

	DBUG_ENTER("noflsh");

	chr = special_char;

	SET_TIMEOUT(WAITTIME/2)

        ret = write(loop_fildes, outbuf, Strlen);
	if (ret == Strlen)
        	ret += write(loop_fildes, (char *)&chr, 1);

        if (alrm_flag == 0)
        	(void) pause();

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* check input queue for bytes */

	SET_TIMEOUT(WAITTIME/2)

	total = 0;
	while ((ret = read(tty_fildes, inbuf, sizeof(inbuf))) > 0)
		total += ret;
	err = errno;

	CLEAR_ALARM

	if (total < Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("NOFLSH failed - input queue was flushed by %s", name);
		if (ret < 0)
			in_rpt("read() set errno to %d (%s)",
				err, errname(err));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 
	DBUG_VOID_RETURN;
}


private void
test1()
{
	/* ECHO set: input characters are echoed back [1: ICANON set] */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	tty_termios_p->c_lflag |= (ICANON|ECHO);

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* write input */

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, outbuf, Strlen);
	if (ret == Strlen)
		ret += write(loop_fildes, &NL, 1);

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* read input */

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read() from VSX_TERMIOS_TTY returned %d, expected %d",
			ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read string != input string");
	}
	else
		PATH_TRACE;

	/* check input was echoed */

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_LOOP returned %d, expected %d",
			ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("echoed string != input string");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4); 
	
	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* ECHO set: input characters are echoed back [2: ICANON clear] */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	tty_termios_p->c_lflag |= ECHO;
	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 0;
	tty_termios_p->c_cc[VTIME] = READ_TIME;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* write input */

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, outbuf, Strlen);

	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* read input */

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read() from VSX_TERMIOS_TTY returned %d, expected %d",
			ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read string != input string");
	}
	else
		PATH_TRACE;

	/* check input was echoed */

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_LOOP returned %d, expected %d",
			ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("echoed string != input string");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4); 
	
	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* ECHO clear: input characters are not echoed back [1: ICANON set] */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag &= ~ECHO;
	tty_termios_p->c_lflag |= ICANON;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* write input */

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, outbuf, Strlen);
	if (ret == Strlen)
		ret += write(loop_fildes, &NL, 1);

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* read input */

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read() from VSX_TERMIOS_TTY returned %d, expected %d",
			ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read string != input string");
	}
	else
		PATH_TRACE;

	/* check input was not echoed */

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_LOOP returned %d, expected 0",
			ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 
	
	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* ECHO clear: input characters are not echoed back [2: ICANON clear] */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	tty_termios_p->c_lflag &= ~ECHO;
	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 0;
	tty_termios_p->c_cc[VTIME] = READ_TIME;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* write input */

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, outbuf, Strlen);

	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* read input */

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read() from VSX_TERMIOS_TTY returned %d, expected %d",
			ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read string != input string");
	}
	else
		PATH_TRACE;

	/* check input was not echoed */

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_LOOP returned %d, expected 0",
			ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4); 
	
	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* ICANON, ECHO and ECHOE set: ERASE is echoed as PCTS_ECHOE */

	char *pcts_echoe;
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	pcts_echoe = tet_getvar("PCTS_ECHOE");
	if (pcts_echoe == NULL || *pcts_echoe == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter PCTS_ECHOE not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Interpret '\b' etc. in string */
	pcts_echoe = interp(pcts_echoe);
	if (pcts_echoe == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("could not allocate space to copy PCTS_ECHOE");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag |= (ICANON|ECHO|ECHOE);

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* write input */

	chr = tty_termios_p->c_cc[VERASE];

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, outbuf, Strlen);
	if (ret == Strlen)
                ret += write(loop_fildes, (char *)&chr, 1);
	if (ret == Strlen + 1)
		ret += write(loop_fildes, &NL, 1);

	CLEAR_ALARM

	if (ret != Strlen + 2)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* read input */

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen)	/* string - deleted_char + newline */
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read() from VSX_TERMIOS_TTY returned %d, expected %d",
			ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)(Strlen-1)) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read string != input string");
	}
	else
		PATH_TRACE;

	/* check input + PCTS_ECHOE was echoed */

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen + strlen(pcts_echoe) + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_LOOP returned %d, expected %d",
			ret, Strlen + strlen(pcts_echoe) + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("echoed string != input string");
	}
	else if (strncmp(pcts_echoe, &inbuf[Strlen], strlen(pcts_echoe)) != 0)
	{
		inbuf[ret-1] = '\0'; /* clobber echoed newline */
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("echoed erase sequence != PCTS_ECHOE");
		in_rpt("expected: \"%s\"", psl_to_asc(pcts_echoe, 0));
		in_rpt("observed: \"%s\"", psl_to_asc(&inbuf[Strlen], 0));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7); 
	
	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* ICANON, ECHO and ECHOK set: KILL is echoed as PCTS_ECHOK */

	char *pcts_echok;
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	pcts_echok = tet_getvar("PCTS_ECHOK");
	if (pcts_echok == NULL || *pcts_echok == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter PCTS_ECHOK not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Interpret '\b' etc. in string */
	pcts_echok = interp(pcts_echok);
	if (pcts_echok == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("could not allocate space to copy PCTS_ECHOK");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag |= (ICANON|ECHO|ECHOK);

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	/* write input */

        chr = tty_termios_p->c_cc[VKILL];

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, outbuf, SubStrlen);
	if (ret == SubStrlen)
                ret += write(loop_fildes, (char *)&chr, 1);
	if (ret == SubStrlen + 1)
		ret += write(loop_fildes, &NL, 1);

	CLEAR_ALARM

	if (ret != SubStrlen + 2)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* read input */

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read() from VSX_TERMIOS_TTY returned %d, expected 1",
			ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	/* check input + PCTS_ECHOK was echoed */

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != SubStrlen + strlen(pcts_echok) + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_LOOP returned %d, expected %d",
			ret, SubStrlen + strlen(pcts_echok) + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)SubStrlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("echoed string != input string");
	}
	else if (strncmp(pcts_echok, &inbuf[SubStrlen], strlen(pcts_echok)) != 0)
	{
		inbuf[ret-1] = '\0'; /* clobber echoed newline */
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("echoed kill sequence != PCTS_ECHOK");
		in_rpt("expected: \"%s\"", psl_to_asc(pcts_echok, 0));
		in_rpt("observed: \"%s\"", psl_to_asc(&inbuf[SubStrlen], 0));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7); 
	
	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* ICANON & ECHONL set: NL characters are echoed [1: ECHO set] */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag |= (ICANON|ECHONL|ECHO);

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* write input */

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, outbuf, Strlen);
	if (ret == Strlen)
		ret += write(loop_fildes, &NL, 1);

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* read input */

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read() from VSX_TERMIOS_TTY returned %d, expected %d",
			ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[Strlen] != NL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("NL character was not in input");
	}
	else
		PATH_TRACE;

	/* check NL was echoed */

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_LOOP returned %d, expected %d",
			ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[Strlen] != NL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("NL was not echoed");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 
	
	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* ICANON & ECHONL set: NL characters are echoed [2: ECHO clear] */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag |= (ICANON|ECHONL);
	tty_termios_p->c_lflag &= ~ECHO;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* write input */

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, outbuf, Strlen);
	if (ret == Strlen)
		ret += write(loop_fildes, &NL, 1);

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* read input */

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read() from VSX_TERMIOS_TTY returned %d, expected %d",
			ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[Strlen] != NL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("NL character was not in input");
	}
	else
		PATH_TRACE;

	/* check NL was echoed */

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_LOOP returned %d, expected 1",
			ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (inbuf[0] != NL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("NL was not echoed");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 
	
	DBUG_VOID_RETURN;
}

private void
test9()
{
	/* ISIG clear: INTR characters are placed on the input queue */

	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test9");

	testfail = 0;

	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag &= ~(ISIG|NOFLSH);

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	globok = 0;
        isig("INTR", tty_termios_p->c_cc[VINTR]);
	PATH_FUNC_TRACE;

	if (caught_sig == SIGINT)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGINT was received");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 

	DBUG_VOID_RETURN;
}

private void
test10()
{
	/* ISIG clear: QUIT characters are placed on the input queue */

	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag &= ~(ISIG|NOFLSH);

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGQUIT, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGQUIT");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	globok = 0;
        isig("QUIT", tty_termios_p->c_cc[VQUIT]);
	PATH_FUNC_TRACE;

	if (caught_sig == SIGQUIT)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGQUIT was received");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 

	DBUG_VOID_RETURN;
}

private void
test11()
{
	/* ISIG clear: SUSP characters are placed on the input queue */

	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("Job control not supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag &= ~(ISIG|NOFLSH);

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTSTP, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGTSTP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	globok = 0;
        isig("SUSP", tty_termios_p->c_cc[VSUSP]);
	PATH_FUNC_TRACE;

	if (caught_sig == SIGTSTP)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTSTP was received");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
test12()
{
	/* ISIG and NOFLSH set: INTR character does not cause flushing */

	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test12");

	testfail = 0;

	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag |= (ISIG|NOFLSH);
	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VTIME] = READ_TIME;
	tty_termios_p->c_cc[VMIN] = 0;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	globok = 0;
        noflsh("INTR", tty_termios_p->c_cc[VINTR]);
	PATH_FUNC_TRACE;

	if (caught_sig != SIGINT)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("SIGINT was not received");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 

	DBUG_VOID_RETURN;
}

private void
test13()
{
	/* ISIG and NOFLSH set: QUIT character does not cause flushing */

	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag |= (ISIG|NOFLSH);
	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VTIME] = READ_TIME;
	tty_termios_p->c_cc[VMIN] = 0;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGQUIT, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGQUIT");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	globok = 0;
        noflsh("QUIT", tty_termios_p->c_cc[VQUIT]);
	PATH_FUNC_TRACE;

	if (caught_sig != SIGQUIT)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("SIGQUIT was not received");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 

	DBUG_VOID_RETURN;
}

private void
test14()
{
	/* ISIG and NOFLSH set: SUSP character does not cause flushing */

	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test14");

	testfail = 0;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("Job control not supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag |= (ISIG|NOFLSH);
	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VTIME] = READ_TIME;
	tty_termios_p->c_cc[VMIN] = 0;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTSTP, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGTSTP");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	globok = 0;
        noflsh("SUSP", tty_termios_p->c_cc[VSUSP]);
	PATH_FUNC_TRACE;

	if (caught_sig != SIGTSTP)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("SIGTSTP was not received");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}
