/*
*      SCCS:  @(#)  POSIX.os/devclass/c_oflag/c_oflag.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/c_oflag - T.c_oflag Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.c_oflag

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the termios output modes - POSIX only

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	1 Mar 1989
MODIFICATIONS:
		Geoff Clare, 13 Sep 1989
			Allow for default CS7 on terminals

		Geoff Clare, 26 Sep 1989
			Use data_rate for baud-dependent sleep time

		Geoff Clare, 8 Dec 1989
			Split range of all character values into chunks
			which ensure MAX_INPUT will not be exceeded

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported
			
		Anthony Lee, 07 August 1990
			Added resilience code.
			
************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>
#include <errno.h>
#include <termios.h>
#include <signal.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef c_oflag
#endif


#define NO_TESTS	1
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		3
#define E_WRITE		02

int	no_tests = NO_TESTS;

extern	char	*errname();
extern	int	data_rate;

private void	test1();
private void	pa_t1();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();

public	void	(*realfuncs[]) () = { test1, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	NULL, 0
};


private	int	unsupported = 0;
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	testfail;
private int	startchr;
private	char	outbuf[128];
private	char	inbuf[300];

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	char *msg = NULL;
	int i;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
		break;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
	}

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

        (void) close(tty_fildes);
        (void) close(loop_fildes);

	DBUG_VOID_RETURN;
}

private int
flushem()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("flushem");

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_RETURN(0);
}

private int
tsetup(flags)
tcflag_t flags;
{
	/* Set ONLY requested output modes on TERMIOS_TTY */

	int err;
	int	pathok = 0;

	DBUG_ENTER("tsetup");

	tty_termios_p->c_oflag &= ~OPOST;
	tty_termios_p->c_oflag |= flags;

        if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_RETURN(-1);
	}
	else
		PATH_FUNC_TRACE;
		
	PATH_FUNC_RPT(2); 
	DBUG_RETURN(0);
}

private void
buf_init()
{
	/* Initialise buffer of all character values, starting at startchr */

	int i;
	int	pathok = 0;
	int	iter = 1;

	DBUG_LENTER("buf_init",2);

	for (i = 0; i < sizeof(outbuf); ++i)
	{
		if (iter)
		{
			PATH_TRACE;
			iter = 0;		/* Only trace the first go */
		}
                outbuf[i] = startchr + i;
	}
	iter = 1;
	if ((tty_termios_p->c_cflag & CSIZE) == CS7)
		for (i = 0; i < sizeof(outbuf); ++i)
		{
			if (iter)
			{
				PATH_TRACE;
				iter = 0;		/* Only trace the first go */
			}
			outbuf[i] &= 0177;
		}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 
	DBUG_VOID_RETURN;
}

private void
wr_chars()
{
	/* Write buffer previously initialised by buf_init() */

	DBUG_ENTER("wr_chars");

	if (write(tty_fildes, outbuf, sizeof(outbuf)) != sizeof(outbuf))
		DBUG_EXIT(E_WRITE);

	DBUG_VOID_RETURN;
}

private void
test1()
{
        /* OPOST clear: characters are output without change */

	int ret;
	int	pathok = 0;
	int	loop = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	globok = 0; 
	if (tsetup((tcflag_t)0) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	for (startchr = CHAR_MIN; startchr <= CHAR_MAX;
		startchr += sizeof(outbuf))
	{
		loop++;		/* Iterations of thsi loop */
		globok = 0;
		buf_init();
		PATH_FUNC_TRACE;

		globok = 0; 
		ret = cppair(wr_chars, pa_t1, WAITTIME, E_WRITE);
		if (ret == E_WRITE)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_TTY failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE; 
	}

	if (testfail == 0)
		PATH_XS_RPT(1 + (loop * 2)); 

	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	int i, ret, err;
	int	pathok = 0;
	int	loop = 0;

	DBUG_ENTER("pa_t1");
            
	(void) sleep((unsigned)(sizeof(outbuf)/data_rate + NAP));

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;
            
	CLEAR_ALARM

#ifdef TESTBED
	--inbuf[0];
	++inbuf[127];
#endif

	if (ret != sizeof(outbuf))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, sizeof(outbuf));
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	for (i = 0; i < ret && i < sizeof(outbuf); ++i)
		if (inbuf[i] != outbuf[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("character %#o was not transmitted unchanged, received %#o",
				(unsigned char)outbuf[i],
				(unsigned char)inbuf[i]);
		}
		else
		{
			PATH_TRACE;
			loop++;		/* Expected path count for this loop */
		}

	PATH_FUNC_RPT(1 + loop); 

	DBUG_VOID_RETURN;
}
