/*
*      SCCS:  @(#)  POSIX.os/devclass/c_oflag_X/c_oflag_X.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/c_oflag_X - T.c_oflag_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.c_oflag_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the termios output modes - X/Open extensions

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	6 Mar 1989
MODIFICATIONS:
		Geoff Clare, 13 Sep 1989
			Allow for default CS7 on terminals

		Geoff Clare, 26 Sep 1989
			Use data_rate for baud-dependent sleep time

		Geoff Clare, 8 Dec 1989
			Split range of all character values into chunks
			which ensure MAX_INPUT will not be exceeded

		Geoff Clare, 15 Feb 1990
			Ensure correct range is passed to ctype.h functions

		Geoff Clare, 26 Feb 1990
			Use memchr() to determine if '\n' is sent

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported

		Anthony Lee, 07 August 1990
			Added resilience code.

		Geoff Clare, 26 Sept 1990
			Do not send characters which are c_cc[] extensions

		Geoff Clare, 22 Nov 1990
			Only test OLCUC on characters 0 to 127.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include <errno.h>
#include <termios.h>
#include <signal.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef c_oflag
#endif


#define NO_TESTS	6
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		3
#define E_WRITE		02
#define Strlen		(sizeof(string)-1)

int	no_tests = NO_TESTS;

extern	char	*psl_to_asc();
extern	char	*errname();
extern	int	data_rate;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	pa_t1();
private void	pa_t2();
private void	pa_t3();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();

public	void	(*realfuncs[]) () = { test1, test2, test3, test4,
				      test5, test6, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	 twrap, 2,
	 twrap, 3,
	 twrap, 4,
	 twrap, 5,
	 twrap, 6,
	NULL, 0
};


private	int	unsupported = 0;
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	testfail;
private int	startchr;
private	char	outbuf[128];
private	char	inbuf[300];

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	char *msg = NULL;
	int i;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
	}

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

        (void) close(tty_fildes);
        (void) close(loop_fildes);

	DBUG_VOID_RETURN;
}

private int
flushem()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("flushem");

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 
	DBUG_RETURN(0);
}

private int
tsetup(flags)
tcflag_t flags;
{
	/* Set ONLY requested output modes on TERMIOS_TTY */

	int err;
	int	pathok = 0;

	DBUG_ENTER("tsetup");

	tty_termios_p->c_oflag &= ~(OPOST|OLCUC|ONLCR|OCRNL|ONOCR|ONLRET|OFILL);
	tty_termios_p->c_oflag &= ~(OFDEL|NLDLY|CRDLY|TABDLY|BSDLY|VTDLY|FFDLY);
	tty_termios_p->c_oflag |= flags;

        if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (flushem() != 0)
	{
		PATH_FUNC_TRACE; 
		DBUG_RETURN(-1);
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(2); 
		

	DBUG_RETURN(0);
}

private void
buf_init()
{
	/* Initialise buffer of all character values, starting at startchr,
	   omitting characters which are c_cc[] extensions */

	int i, ch, ncc, strip;
	int pathok = 0;

	DBUG_ENTER("buf_init");

	PATH_TRACE;

	strip = ((tty_termios_p->c_cflag & CSIZE) == CS7);

	for (i = 0, ch = startchr; i < sizeof(outbuf); ++ch)
	{
		if (strip)
			ch &= 0177;
		else
			ch &= 0377;

		for (ncc = 0; ncc < NCCS; ncc++)
		{
			if (ch == (char)tty_termios_p->c_cc[ncc])
			{
				/* only include if it's a "known" character */
				switch (ncc)
				{
				case VEOF :
				case VEOL :
				case VERASE :
				case VINTR :
				case VKILL :
				case VQUIT :
				case VSTART :
				case VSTOP :
				case VSUSP :
                			outbuf[i++] = ch;
					break;
				
				default:
					if (ncc == VMIN || ncc == VTIME)
                				outbuf[i++] = ch;
					break;
				}
				break;
			}
		}

		if (ncc >= NCCS)	/* not found in c_cc[] */
                	outbuf[i++] = ch;
	}

	DBUG_PRINTF("buf_init", "startchr = %#o, ch after loop = %#o",
		(unsigned char)startchr, ch);

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
wr_chars()
{
	/* Write buffer previously initialised by buf_init() */

	DBUG_ENTER("wr_chars");

	if (write(tty_fildes, outbuf, sizeof(outbuf)) != sizeof(outbuf))
		DBUG_EXIT(E_WRITE);

	DBUG_VOID_RETURN;
}

private void
test1()
{
        /* OLCUC set: translate lower case to upper case */

	int ret;
	int	pathok = 0;
	int 	loop = 0;
	
	DBUG_ENTER("test1");

	testfail = 0;

	globok = 0; 
	if (tsetup((tcflag_t)(OPOST|OLCUC)) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	for (startchr = 0; startchr <= 127; startchr += sizeof(outbuf))
	{
		loop++;
		globok = 0;
		buf_init();
		PATH_FUNC_TRACE;

		globok = 0;
		ret = cppair(wr_chars, pa_t1, WAITTIME, E_WRITE);
		if (ret == E_WRITE)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_TTY failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(1 + (loop * 2)); 

	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	int c, i, ret, err;
	int	pathok = 0;
	int	loop = 0;

	DBUG_ENTER("pa_t1");
            
	(void) sleep((unsigned)(sizeof(outbuf)/data_rate + NAP));

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;
            
	CLEAR_ALARM

#ifdef TESTBED
	++inbuf['A'];
#endif

	if (ret != sizeof(outbuf))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, sizeof(outbuf));
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	for (i = 0; i < ret && i < sizeof(outbuf); ++i)
	{
		loop++;
		c = (unsigned char) outbuf[i];

		if (!islower(c) && inbuf[i] != outbuf[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("character %#o was not transmitted unchanged, received %#o",
				(unsigned char)outbuf[i],
				(unsigned char)inbuf[i]);
		}
		else if (islower(c) && inbuf[i] != (char) toupper(c))
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("character %#o was transmitted as %#o, expected %#o",
				(unsigned char)outbuf[i],
				(unsigned char)inbuf[i],
				(unsigned char)toupper(c));
		}
		else
			PATH_TRACE;
	}

	PATH_FUNC_RPT(1 + loop); 
	DBUG_VOID_RETURN;
}

private void
test2()
{
        /* ONLCR set: translate NL into CR-NL */

	int ret;
	int	pathok = 0;
	int	loop = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	globok = 0; 
	if (tsetup((tcflag_t)(OPOST|ONLCR)) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	for (startchr = CHAR_MIN; startchr <= CHAR_MAX;
		startchr += sizeof(outbuf))
	{
		loop++;
		globok = 0;
		buf_init();
		PATH_FUNC_TRACE;

		globok = 0;
		ret = cppair(wr_chars, pa_t2, WAITTIME, E_WRITE);
		if (ret == E_WRITE)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_TTY failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(1 + (loop* 2)); 

	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	int i, j, ret, err, expret;
	int	pathok = 0;
	int	loop = 0;

	DBUG_ENTER("pa_t2");
            
	(void) sleep((unsigned)(sizeof(outbuf)/data_rate + NAP));

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;
            
	CLEAR_ALARM

#ifdef TESTBED
	++inbuf['\n'];
#endif

	expret = sizeof(outbuf);
	if (memchr((void *)outbuf, '\n', sizeof(outbuf)) != NULL)
		expret++;

	if (ret != expret)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, expret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	for (i = j = 0; i < sizeof(outbuf) && j < ret; ++i, ++j)
	{
		loop++;
		if (outbuf[i] == '\n')
		{
			if (inbuf[j] != '\r' || inbuf[j+1] != '\n')
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("NL character was not transmitted as CR-NL, received %#o-%#o",
					(unsigned char)inbuf[j],
					(unsigned char)inbuf[j+1]);
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;
			++j;
		}
		else if (inbuf[j] != outbuf[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("character %#o was not transmitted unchanged, received %#o",
				(unsigned char)outbuf[i],
				(unsigned char)inbuf[j]);
		}
		else
			PATH_TRACE;
	}
	PATH_FUNC_RPT(1 + loop); 

	DBUG_VOID_RETURN;
}

private void
test3()
{
        /* OCRNL set: translate CR into NL */

	int ret;
	int	pathok = 0;
	int 	loop = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	globok = 0; 
	if (tsetup((tcflag_t)(OPOST|OCRNL)) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	for (startchr = CHAR_MIN; startchr <= CHAR_MAX;
		startchr += sizeof(outbuf))
	{
		loop++;
		globok = 0;
		buf_init();
		PATH_FUNC_TRACE;

		globok = 0;
		ret = cppair(wr_chars, pa_t3, WAITTIME, E_WRITE);
		if (ret == E_WRITE)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_TTY failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(1 + (2 * loop)); 

	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int i, ret, err;
	int	pathok = 0;
	int 	loop = 0;

	DBUG_ENTER("pa_t3");
            
	(void) sleep((unsigned)(sizeof(outbuf)/data_rate + NAP));

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;
            
	CLEAR_ALARM

#ifdef TESTBED
	++inbuf['\r'];
#endif

	if (ret != sizeof(outbuf))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, sizeof(outbuf));
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	for (i = 0; i < sizeof(outbuf) && i < ret; ++i)
		if (outbuf[i] == '\r' && inbuf[i] != '\n')
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("CR character was not transmitted as NL, received %#o",
				(unsigned char)inbuf[i]);
		}
		else if (outbuf[i] != '\r' && inbuf[i] != outbuf[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("character %#o was not transmitted unchanged, received %#o",
				(unsigned char)outbuf[i],
				(unsigned char)inbuf[i]);
		}
		else
		{
			loop++;
			PATH_TRACE;
		}
	PATH_FUNC_RPT(1 + loop); 

	DBUG_VOID_RETURN;
}

private void
test4()
{
        /* ONOCR set: no CR output at column 0 (test CR-CR -> CR) */

	int ret, err;
	static char string[] = "test-\r\r-ing";
	static char expstr[] = "test-\r-ing";
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	globok = 0; 
	if (tsetup((tcflag_t)(OPOST|ONOCR)) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = write(tty_fildes, string, Strlen);

	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;
            
	CLEAR_ALARM

#ifdef TESTBED
	inbuf[0] = 'X';
#endif

	if (ret != Strlen-1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen-1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	if (ret >= 0 && strncmp(inbuf, expstr, (size_t)ret) != 0)
	{
		inbuf[ret] = '\0';
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("string \"%s\" transmitted as \"%s\", expected \"%s\"",
			psl_to_asc(string, 0), psl_to_asc(inbuf, 0),
			psl_to_asc(expstr, 0));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
test5()
{
        /* ONLRET set: assume NL does a CR (test with ONOCR: NL-CR -> NL) */

	int ret, err;
	static char string[] = "test-\n\r-ing";
	static char expstr[] = "test-\n-ing";
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	globok = 0; 
	if (tsetup((tcflag_t)(OPOST|ONOCR|ONLRET)) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = write(tty_fildes, string, Strlen);

	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;
            
	CLEAR_ALARM

#ifdef TESTBED
	inbuf[0] = 'X';
#endif

	if (ret != Strlen-1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen-1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	if (ret >= 0 && strncmp(inbuf, expstr, (size_t)ret) != 0)
	{
		inbuf[ret] = '\0';
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("string \"%s\" transmitted as \"%s\", expected \"%s\"",
			psl_to_asc(string, 0), psl_to_asc(inbuf, 0),
			psl_to_asc(expstr, 0));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
test6()
{
        /* TAB3 set: tabs expand to spaces */

	int i, j, cnt, ret, err, fail = 0;
	static char string[] = "abcdefgh\ti\t\tj";
	static char expstr[] = "abcdefgh1i2j";	/* digits indicate min spaces */
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	globok = 0; 
	if (tsetup((tcflag_t)(OPOST|TAB3)) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	SET_TIMEOUT(WAITTIME)

	ret = write(tty_fildes, string, Strlen);

	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	err = errno;
            
	CLEAR_ALARM

#ifdef TESTBED
	inbuf[8] = '\t';
#endif

	if (ret < Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected >= %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	for (i = j = 0; i < sizeof(expstr)-1 && j < ret; i++)
	{
		if (i == 0)
			PATH_TRACE; 
		if (!isdigit(expstr[i]))
		{
			if (inbuf[j] != expstr[i])
				fail++;
			j++;
		}
		else
		{
			for (cnt = 0; j < ret && inbuf[j] == ' '; j++)
				cnt++;
			
			if (cnt < atoi(&expstr[i]))
				fail++;
		}
	}

	if (fail != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		inbuf[ret] = '\0';
		in_rpt("string \"%s\" was not correctly tab-expanded, received \"%s\"",
			psl_to_asc(string, 0),
			psl_to_asc(inbuf, 0));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 

	DBUG_VOID_RETURN;
}
