/*
*      SCCS:  @(#)  POSIX.os/devclass/cfgetispee/cfgetispee.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/cfgetispee - T.cfgetispee Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.cfgetispee

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the cfgetispeed() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	9 Jan 1989
MODIFICATIONS:

		Anthony Lee, 07 August 1990
		Added resilience code.

		Geoff Clare, 13 Apr 1992
			Add zero value to baud table (not XPG3 mode).
		
************************************************************************/

#include	<std.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<termios.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef cfgetispeed
#endif


#define NO_TESTS	1

int	no_tests = NO_TESTS;

private void	test1();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

/* c_cflag baud rate array for speed tests */
private struct baud_table
{
  	char name[8];		/* name string			*/
	speed_t mask;		/* baud rate mask		*/
	int value;		/* integer baud rate value	NOT USED */
	int supported;		/* settable by cfset[io]speed	NOT USED */
	int failed;		/* passed write/read test    	NOT USED */
} baud[] =
{
	"B0",		B0,	0,	0,	0,
	"B50",		B50,	50,	0,	0,
	"B75",		B75,	75,	0,	0,
	"B110",		B110,	110,	0,	0,
	"B134",		B134,	134,	0,	0,
	"B150",		B150,	150,	0,	0,
	"B200",		B200,	200,	0,	0,
	"B300",		B300,	300,	0,	0,
	"B600",		B600,	600,	0,	0,
	"B1200",	B1200,	1200,	0,	0,
	"B1800",	B1800,	1800,	0,	0,
	"B2400",	B2400,	2400,	0,	0,
	"B4800",	B4800,	4800,	0,	0,
	"B9600",	B9600,	9600,	0,	0,
	"B19200",	B19200,	19200,	0,	0,
	"B38400",	B38400,	38400,	0,	0,
#if TEST_XPG != 3
	"0",		(speed_t)0, 0,	0,	0,
#endif
};

#define SPEEDS (sizeof(baud)/sizeof(struct baud_table))

private	struct termios	termios_speed;	/* cfset[io]speed	*/
private	speed_t		ispeed;		/* cfgetispeed() return	*/
private int		testfail;


private void
test1()
{
	int i;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	for (i = 0; i < SPEEDS; ++i)
	{
		(void) cfsetispeed(&termios_speed, baud[i].mask);
		ispeed = cfgetispeed(&termios_speed);

		if (ispeed != baud[i].mask)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("cfgetispeed() returned incorrect value for baud rate %s",
					baud[i].name);
		}
		else
			PATH_TRACE;
	}
			
	if (testfail == 0)
		PATH_XS_RPT(SPEEDS); 

	DBUG_VOID_RETURN;
}
