/*
*      SCCS:  @(#)  POSIX.os/devclass/cfsetospee/cfsetospee.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/cfsetospee - T.cfsetospee Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.cfsetospee

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the cfsetospeed() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	9 Jan 1989
MODIFICATIONS:
		Geoff Clare, 2 Aug 1989
			EINVAL from tcsetattr() means speed is not supported
		
		Geoff Clare, 25 Aug 1989
			UNSUPPORTED result if modem control not available
			Clear CLOCAL on TERMIOS_LOOP in B0 test

		Geoff Clare, 12 Sep 1989
			Increase leeway on write() timeout
		
		Geoff Clare, 20 Sep 1989
			Remove bogus write & read in modem disconnect test

		Geoff Clare, 12 Mar 1990
			Use XON/XOFF flow control to avoid data overrun

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported
			
		Anthony Lee, 07 August 1990
			Added resilience code.

		Geoff Clare, 19 Nov 1990
			Allow for processing time in addition to transmission
			time in baud rate testing.

		Geoff Clare, 25 June 1991
			VSX_MODEM_CONTROL="N" for unsupported result
		
		Geoff Clare, 3 Oct 1991
			Allow more processing time in baud rate testing.

		Stuart Boutell, 21 May 1992
		Removed dependency on GTI in test one.
		
		Geoff Clare, 21 June 1993
			Add E_NOTSUPP to cppair() valid exit codes list
			in test 3.
		
		Geoff Clare, 13 April 1995
			Implement interp VWG/012.

		Geoff Clare, 4 Jan 1996
			Allow for tcgetattr() setting input rate to 0 when
			input rate is the same as output rate.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef cfsetospeed
#endif


#define NO_TESTS	3
#define WAITTIME	(5 * SPEEDFACTOR)

#define Strlen		(unsigned int)(sizeof(outbuf)-1)
#define NAP		2
#define TEN_SECONDS	10	/* number of seconds to write		*/

#define	E_WRFAIL	002
#define	E_RDFAIL	004
#define	E_SAFAIL	010
#define	E_TCSFAIL	020
#define	E_NOTSUPP	040
#define PATH_NOK	0100
extern	char	*errname();


int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	ch_t2();
private void	ch_t3();
private void	pa_t2();
private void	pa_t3();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();

public	void	(*realfuncs[]) () = { test1, test2, test3, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 twrap, 2,
	 twrap, 3,
	NULL, 0
};


/* c_cflag baud rate array for speed tests */
private struct baud_table
{
  	char name[8];		/* name string			*/
	speed_t mask;		/* baud rate mask		*/
	int value;		/* integer baud rate value	*/
	int supported;		/* settable by cfset[io]speed	*/
	int failed;		/* passed write/read test    	*/
} *baudp,  baud[] =
{
	"B0",		B0,	0,	0,	0,
	"B50",		B50,	50,	0,	0,
	"B75",		B75,	75,	0,	0,
	"B110",		B110,	110,	0,	0,
	"B134",		B134,	134,	0,	0,
	"B150",		B150,	150,	0,	0,
	"B200",		B200,	200,	0,	0,
	"B300",		B300,	300,	0,	0,
	"B600",		B600,	600,	0,	0,
	"B1200",	B1200,	1200,	0,	0,
	"B1800",	B1800,	1800,	0,	0,
	"B2400",	B2400,	2400,	0,	0,
	"B4800",	B4800,	4800,	0,	0,
	"B9600",	B9600,	9600,	0,	0,
	"B19200",	B19200,	19200,	0,	0,
	"B38400",	B38400,	38400,	0,	0
};

#define SPEEDS (sizeof(baud)/sizeof(struct baud_table))
#define COMBITS 11

private char inbuf[512];		/* I/O buffers	*/
private char outbuf[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZa\n";

private	int	unsupported = 0;
private	int tty_fildes, loop_fildes;	/* terminal file descriptors */
private	struct termios tty_termios;	/* terminal attribute structure	*/
private	struct termios loop_termios;	/* terminal attribute structure	*/
private	struct termios *tty_termios_p = &tty_termios;
private	struct termios *loop_termios_p = &loop_termios;
private	struct termios termios_speed;	/* cfset[io]speed		*/

private	int baud_error = 0;		/* baud rate test error count	*/
private	int baud_chars = 0;		/* baud rate test char count	*/

private	int	caught_sig = 0;
private	int	testfail;

private void
sig_catch(sig)
{
	caught_sig = sig;
}

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	int i;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
	default:
		{
			char *msg;

			if (alrm_flag > 0)
				msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
			else
				msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
			for (i = 2; i <= NO_TESTS; i++)
				tet_delete(i, msg);
			DBUG_VOID_RETURN;
		}
	}

	tty_termios_p->c_iflag |= IXOFF;
	loop_termios_p->c_iflag |= IXON;

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	if (!unsupported) {
		(void) tty_init(tty_fildes, tty_termios_p);
		(void) loop_init(loop_fildes, loop_termios_p);

		(void) tcsetattr(tty_fildes, TCSANOW, tty_termios_p);
		(void) tcsetattr(loop_fildes, TCSANOW, loop_termios_p);

		(void) close(tty_fildes);
		(void) close(loop_fildes);
	}
}


private void
test1()
{
	int i;
	speed_t ospeed;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	for (i = 0; i < SPEEDS; ++i)
	{
		(void) cfsetospeed(&termios_speed, baud[i].mask);
		ospeed = cfgetospeed(&termios_speed);

		if (ospeed != baud[i].mask)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("cfsetospeed() failed for baud rate %d",
					baud[i].value);
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(SPEEDS); 
	
	DBUG_VOID_RETURN;
}

private void
test2()
{
	speed_t ispeed;
	int i, rval, err;
	int	pathok = 0;
	int	retval;

	DBUG_ENTER("test2");

	testfail = 0;

	for (i = 1, baudp = &baud[1]; i < SPEEDS; ++i, ++baudp)
	{
		DBUG_PRINTF("test2", "starting speed %s", baudp->name);

		if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR ||
		    tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: tcflush() failed during setup - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		baud_chars = baudp->value/COMBITS * TEN_SECONDS;

		/* check if baud rate is supported */
		baudp->supported = 1;

		if (cfsetispeed(tty_termios_p, baudp->mask) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("cfsetispeed(..., %s) failed - errno %d (%s)",
				baudp->name, err, errname(err));
			continue;
		}
		else
			PATH_TRACE;

		if ((rval = cfsetospeed(tty_termios_p, baudp->mask)) != 0)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("cfsetospeed(..., %s) returned %d, expected 0",
				baudp->name, rval);
			in_rpt("errno was set to %d (%s)", err, errname(err));
			continue;
		}
		else
			PATH_TRACE;

		if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
		{
			if (errno == EINVAL)
			{
				PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
				PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
				PATH_TRACE; PATH_TRACE; PATH_TRACE; /* Not failed !*/
				baudp->supported = 0;
				DBUG_PRINTF("test2",
				    "tcsetattr() returned EINVAL for %s %s.",
				    baudp->name, "(TERMIOS_TTY)");
				continue;
			}
			else
			{
				err = errno;
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
					err, errname(err));
				in_rpt("after setting input & output speeds to %s",
					baudp->name);
				DBUG_VOID_RETURN;
			}
		}
		else
			PATH_TRACE;

		if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("tcgetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		ispeed = cfgetispeed(tty_termios_p);
		if (ispeed != baudp->mask && ispeed != 0)
		{
			PATH_TRACE; 	/* Not failed */
			baudp->supported = 0;
			DBUG_PRINTF("test2", "input not supported at %s %s.",
					baudp->name, "(TERMIOS_TTY)");
		}
		else
			PATH_TRACE;

		if (cfgetospeed (tty_termios_p) != baudp->mask)
		{
			PATH_TRACE; 	/* Not failed */
			baudp->supported = 0;
			DBUG_PRINTF("test2", "output not supported at %s %s.",
					baudp->name, "(TERMIOS_TTY)");
		}
		else
			PATH_TRACE;

		if (cfsetispeed(loop_termios_p, baudp->mask) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("cfsetispeed(..., %s) failed - errno %d (%s)",
				baudp->name, err, errname(err));
			continue;
		}
		else
			PATH_TRACE;

		if ((rval = cfsetospeed(loop_termios_p, baudp->mask)) != 0)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("cfsetospeed(..., %s) returned %d, expected 0",
				baudp->name, rval);
			in_rpt("errno was set to %d (%s)", err, errname(err));
			continue;
		}
		else
			PATH_TRACE;

		if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		{
			if (errno == EINVAL)
			{
				PATH_TRACE; PATH_TRACE; PATH_TRACE;
				PATH_TRACE; PATH_TRACE;  /* Not failed */
				baudp->supported = 0;
				DBUG_PRINTF("test2",
				    "tcsetattr() returned EINVAL for %s %s.",
				    baudp->name, "(TERMIOS_LOOP)");
				continue;
			}
			else
			{
				err = errno;
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
					err, errname(err));
				in_rpt("after setting input & output speeds to %s",
					baudp->name);
				DBUG_VOID_RETURN;
			}
		}
		else
			PATH_TRACE;

		if (tcgetattr(loop_fildes, loop_termios_p) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("tcgetattr() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		ispeed = cfgetispeed(loop_termios_p);
		if (ispeed != baudp->mask && ispeed != 0)
		{
			PATH_TRACE; 	/* Not failed */
			baudp->supported = 0;
			DBUG_PRINTF("test2", "input not supported at %s %s.",
					baudp->name, "(TERMIOS_LOOP)");
		}
		else
			PATH_TRACE;

		if (cfgetospeed(loop_termios_p) != baudp->mask)
		{
			PATH_TRACE; 	/* Not failed */
			baudp->supported = 0;
			DBUG_PRINTF("test2", "output not supported at %s %s.",
					baudp->name, "(TERMIOS_LOOP)");
		}
		else
			PATH_TRACE;

		if (!baudp->supported)
		{
			PATH_TRACE;  /* To compensate for the one below */
			continue;
		}
		retval = cppair(ch_t2, pa_t2, WAITTIME, E_WRFAIL | PATH_NOK);
		if (retval == E_WRFAIL)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: write() to VSX_TERMIOS_LOOP failed");
			DBUG_VOID_RETURN;
		}
		else if (retval == PATH_NOK)
			retval = OK;
		else
			PATH_FUNC_TRACE; 		/* Child was ok, now check cppair */
	}

	/* Check at least 1 speed is supported */

	for (i = 1; i < SPEEDS; ++i)
	{
		if (i == 1)
			PATH_TRACE; 
		if (baud[i].supported)
			break;
	}
	if (i == SPEEDS)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("none of specified speeds are supported");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT( ((SPEEDS-1) * 14) + 2 ); 
		
	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int j, write_n;
	int	pathok = 0;
	int	loop = 0;

	DBUG_ENTER("ch_t2");

	(void) sleep(NAP);

	/* In addition to transmission time, allow SPEEDFACTOR seconds
	   per 10Kbytes for processing time */
	SET_TIMEOUT(2 * TEN_SECONDS + SPEEDFACTOR * baud_chars/10240)

	for (j = baud_chars; j > 0; j -= write_n)
	{
		write_n = write(loop_fildes, outbuf, Strlen);
		if (write_n != Strlen)
		{
			DBUG_EXIT(E_WRFAIL);
		}
		else
		{
			loop++;
			PATH_TRACE;
		}
	}

	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		DBUG_EXIT(E_WRFAIL);
	}
	else
		PATH_TRACE;

	if (pathok != (loop + 1))
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	int j, read_n;
	int	pathok = 0;
	int	loop = 0;

	DBUG_ENTER("pa_t2");

	baudp->failed = 0;
	baud_error = 0;

	SET_TIMEOUT(2 * (NAP + TEN_SECONDS) + SPEEDFACTOR * baud_chars/10240)

	for (j = baud_chars; j > 0; j -= read_n)
	{
		read_n = read(tty_fildes, inbuf, sizeof(inbuf));
		if (read_n < 0)
			break;
		else
		{
			PATH_TRACE;
			loop++;
		}
	}

	CLEAR_ALARM

	if (alrm_flag > 0 || read_n < 0)
	{
		baudp->failed = 1;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read error.  %s test failed", baudp->name);
	}
	else
	{
		for (j = 0; j < Strlen; ++j)
			if(inbuf[j] != outbuf[j])
			{
				++baud_error;
				PATH_TRACE;
				loop++;
			}
	}

	if (baud_error > 0)
	{
		baudp->failed = 1;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read errors received (%d).  %s test failed",
			baud_error, baudp->name);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(loop + 1); 

	DBUG_VOID_RETURN;
}

private void
test3()
{
	char *modem;
	int ret;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	modem = tet_getvar("VSX_MODEM_CONTROL");
	if (modem == NULL || *modem == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_MODEM_CONTROL is not set");
		DBUG_VOID_RETURN;
	}
	else if (*modem == 'N' || *modem == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	ret = cppair(ch_t3, pa_t3, (3 * WAITTIME),
		E_RDFAIL|E_SAFAIL|E_TCSFAIL|E_NOTSUPP|PATH_NOK);

	switch (ret)
	{
	case E_RDFAIL :
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read() failed in child");
		break;
	case E_SAFAIL :
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGHUP, ...) failed in child");
		break;
	case E_TCSFAIL :
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr() failed in child (errno != EINVAL)");
		break;
	case E_NOTSUPP :
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("c_cflag combination (~CLOCAL, HUPCL) or (~CLOCAL, B0) not supported");
		break;
	}

	if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE; 		/* Child was ok, check cppair */

	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int read_n;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	SET_TIMEOUT(WAITTIME)
	read_n = read(tty_fildes, inbuf, (unsigned) sizeof(inbuf));
	CLEAR_ALARM

	if (read_n < 0 || alrm_flag > 0)
	{
		DBUG_EXIT(E_RDFAIL);
	}
	else
		PATH_TRACE;

	/*
	 * Catch SIGHUP, in case the child gets it as well (only the
	 * controlling process is required to get it)
	 */
	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGHUP, &sig, NULLSA) == SYSERROR)
	{
		DBUG_EXIT(E_SAFAIL);
	}
	else
		PATH_TRACE;

	tty_termios_p->c_cflag &= ~CLOCAL;
	tty_termios_p->c_cflag |= HUPCL;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		if (errno == EINVAL)
			DBUG_EXIT(E_NOTSUPP);
		else
			DBUG_EXIT(E_TCSFAIL);
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	(void) cfsetospeed(loop_termios_p, B0);
	loop_termios_p->c_cflag &= ~CLOCAL;
	if (tcsetattr(loop_fildes, TCSAFLUSH, loop_termios_p) == SYSERROR)
	{
		if (errno == EINVAL)
			DBUG_EXIT(E_NOTSUPP);
		else
			DBUG_EXIT(E_TCSFAIL);
	}
	else
		PATH_TRACE;

	CLEAR_ALARM

	if (pathok != 4)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int write_n;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("pa_t3");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGHUP, &sig, NULLSA) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGHUP, ...) failed in parent");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)
	write_n = write (loop_fildes, outbuf, Strlen);
	CLEAR_ALARM

	if (write_n < 0 || alrm_flag > 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() failed in parent");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(WAITTIME);		/* pause for signal */

	if (caught_sig != SIGHUP)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGHUP not received by controlling process");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3); 

	DBUG_VOID_RETURN;
}
