/*
*      SCCS:  @(#)  POSIX.os/devclass/i_access/i_access.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/i_access - T.i_access Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.i_access

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the terminal interface, part 4

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	1 Feb 1989
MODIFICATIONS:
		Geoff Clare, 25 Aug 1989
		Change orphaned process tests so session leader stays alive

		Geoff Clare, 11 Sep 1989
		Missing xs_rpt() calls in orphaned process group tests

		Geoff Clare, 13 June 1990
		Test TOSTOP clear from orphaned and non-orphaned pgrp

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported

		Anthony Lee, 07 August 1990
			Added resilience code.

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


#define NO_TESTS	9
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		3
#define MSG_LEN (sizeof(outbuf)-1)
#define NOK		002
#define E_SAFAIL	004
#define E_GOTSIG	010
#define PATH_NOK	0100

extern	void	setsigs();
extern	char	*signame();
extern	char	*errname();


int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	ch1_t6();
private void	ch2_t6();
private void	ch_t7();
private void	ch_t8();
private void	ch_t9();
private void	gch_t2();
private void	gch_t3();
private void	gch_t7();
private void	gch_t8();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();

public	void	(*realfuncs[]) () = { test1, test2, test3, test4, test5,
				      test6, test7, test8, test9, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	 twrap, 2,
	 twrap, 3,
	 twrap, 4,
	 twrap, 5,
	 twrap, 6,
	 twrap, 7,
	 twrap, 8,
	 twrap, 9,
	NULL, 0
};


private	int	unsupported = 0;
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private	int	caught_sig = 0;
private int	testfail;
private int	notinuse = 0;
private int	rd_ret, rd_err;
private int	wr_ret, wr_err;
private char	*termios_loop;
private char	outbuf[]= "Hutsut ralston on the rillarah\n";
private char	inbuf[512];


private void
sig_catch(sig)
int sig;
{
	caught_sig = sig;
}

private void
sig_orphan(sig)
int sig;
{
	/* Signal handler for orphaned children */

	xx_rpt(DELETION);
	in_rpt("deletion reason: caught unexpected signal %d (%s)",
		sig, signame(sig));
	exit(0);
}

private void
do_read()
{
	DBUG_ENTER("do_read");

	rd_ret = read(tty_fildes, inbuf, sizeof(inbuf));
	rd_err = errno;
	globok = 1; 		/* Needed for cppair */

	DBUG_VOID_RETURN;
}

private void
do_write()
{
	DBUG_ENTER("do_write");

	wr_ret = write(tty_fildes, outbuf, MSG_LEN);
	wr_err = errno;
	globok = 1;			/* Needed by cppair */

	DBUG_VOID_RETURN;
}

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	char *msg = NULL;
	int i;

	DBUG_ENTER("prepare_tests");

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		notinuse = 1;
		DBUG_VOID_RETURN;
	}

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
		break;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}

	tty_termios_p->c_lflag |= TOSTOP;

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY in preparation");
		DBUG_VOID_RETURN;
	}

	termios_loop = tet_getvar("VSX_TERMIOS_LOOP");
	/* no need to check: termios_prep succeeded */

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

        (void) close(tty_fildes);
        (void) close(loop_fildes);

	DBUG_VOID_RETURN;
}

private void
test1()
{
        /* Read from a background process not ignoring or blocking SIGTTIN */

	DBUG_ENTER("test1");

	if (notinuse)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	testfail = 0;

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTIN, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGTTIN - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	caught_sig = 0;
	(void) read(tty_fildes, inbuf, sizeof(inbuf));
	
	if (caught_sig != SIGTTIN)
	{
		xx_rpt(FAILURE);
		in_rpt("SIGTTIN not received");
	}
	else
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test2()
{
        /* Read from a background process, ignoring SIGTTIN */

	DBUG_ENTER("test2");

	if (notinuse)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	testfail = 0;

	(void) cppair(ch_t2, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTIN, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTIN, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	ret = cppair(gch_t2, do_read, WAITTIME, E_SAFAIL|E_GOTSIG|PATH_NOK);

	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTIN, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTIN received by child of calling process");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE; 

	if (rd_ret != -1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on controlling terminal returned %d, expected -1", rd_ret);
		DBUG_VOID_RETURN;
	}
	else if (rd_err != EIO)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on controlling terminal set errno to %d (%s), expected EIO",
			rd_err, errname(rd_err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
gch_t2()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("gch_t2");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTIN, &sig, NULLSA) == SYSERROR)
	{
		DBUG_EXIT(E_SAFAIL);
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTIN)
	{
		DBUG_EXIT(E_GOTSIG);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
test3()
{
        /* Read from a background process blocking SIGTTIN */

	DBUG_ENTER("test3");

	if (notinuse)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	testfail = 0;

	(void) cppair(ch_t3, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int ret;
	sigset_t set;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTIN, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTIN, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* block SIGTTIN */

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTTIN);
	if (sigprocmask(SIG_BLOCK, &set, (sigset_t *)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask(SIG_BLOCK, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	ret = cppair(gch_t3, do_read, WAITTIME, E_SAFAIL|E_GOTSIG|PATH_NOK);

	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTIN, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTIN received by child of calling process");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE; 

	if (rd_ret != -1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on controlling terminal returned %d, expected -1", rd_ret);
	}
	else if (rd_err != EIO)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on controlling terminal set errno to %d (%s), expected EIO",
			rd_err, errname(rd_err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 

	DBUG_VOID_RETURN;
}

private void
gch_t3()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("gch_t3");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTIN, &sig, NULLSA) == SYSERROR)
	{
		DBUG_EXIT(E_SAFAIL);
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTIN)
	{
		DBUG_EXIT(E_GOTSIG);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
test4()
{
        /* Read from an orphaned background process */

	DBUG_ENTER("test4");

	if (notinuse)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	testfail = 0;

	(void) cppair(ch_t4, NULLFN, 2 * WAITTIME, 0);

	(void) sleep(WAITTIME);	/* allow orphan to report */

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

        /* Create orphan */

	switch (fork())
	{
	case SYSERROR:
		xx_rpt(DELETION);
		in_rpt("fork() failed");
		break;

	case 0:
		tet_setcontext();
		/* child (orphan) reads from control tty */

		setsigs(sig_orphan);

		if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("setpgid(0, 0) failed");
			exit(0);
		}
		else
			PATH_TRACE;

		sig.sa_handler = sig_catch;
		sig.sa_flags = 0;
		(void) sigemptyset(&sig.sa_mask);
		if (sigaction(SIGTTIN, &sig, NULLSA) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("sigaction(SIGTTIN, ...) failed");
			exit(0);
		}
		else
			PATH_TRACE;

		(void) sleep(NAP);

		caught_sig = 0;

		SET_TIMEOUT(WAITTIME/2);

		rd_ret = read(tty_fildes, inbuf, sizeof(inbuf));
		rd_err = errno;
	    
		CLEAR_ALARM

		if (rd_ret != -1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("read() on controlling terminal returned %d, expected -1", rd_ret);
		}
		else if (rd_err != EIO)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("read() on controlling terminal set errno to %d (%s), expected EIO",
				rd_err, errname(rd_err));
		}
		else
			PATH_TRACE;

		if (caught_sig == SIGTTIN)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("SIGTTIN was received");
		}
		else
			PATH_TRACE;

		if (testfail == 0)
			PATH_XS_RPT(4); 

		DBUG_EXIT(0);

	default:
		tet_setblock();
		/* parent exits, leaving orphan */
		break;
	}
	tet_setblock();
	DBUG_VOID_RETURN;
}

private void
test5()
{
        /* Write from a background process not ignoring or blocking SIGTTOU */

	int err;

	DBUG_ENTER("test5");

	if (notinuse)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	testfail = 0;

        if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t5, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t5");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGTTOU - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	caught_sig = 0;
	(void) write(tty_fildes, outbuf, MSG_LEN);
	
	if (caught_sig != SIGTTOU)
	{
		xx_rpt(FAILURE);
		in_rpt("SIGTTOU not received");
	}
	else
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test6()
{
        /* Write from a background process with TOSTOP clear */

	int err;
	int	pathok = 0;
	int 	retval;

	DBUG_ENTER("test6");

	if (notinuse)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

        if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
        if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag &= ~TOSTOP;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	retval = cppair(ch1_t6, NULLFN, WAITTIME, NOK|PATH_NOK);
	if (retval == NOK)
		testfail++;
	else if (retval == PATH_NOK)
		retval = OK;
	else
		PATH_TRACE; 

	globok = (pathok == 5 ? 1 : 0);	/* globok is passed to child process */
	(void) cppair(ch2_t6, NULLFN, WAITTIME, 0);

	(void) sleep(WAITTIME);	/* allow orphan to report */

	DBUG_VOID_RETURN;
}

private void
ch1_t6()
{
	int err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch1_t6");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGTTOU - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	caught_sig = 0;
	wr_ret = write(tty_fildes, outbuf, MSG_LEN);
	wr_err = errno;
	
	if (wr_ret != MSG_LEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("write() returned %d, expected %d", wr_ret, MSG_LEN);
		in_rpt("errno was set to %d (%s)", wr_err, errname(wr_err));
	}
	else
		PATH_TRACE;

	if (caught_sig == SIGTTOU)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU was received");
	}
	else
		PATH_TRACE;

	if (testfail != 0)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	if (pathok != 5)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
ch2_t6()
{
	struct sigaction sig;
	int err;
	int	pathok = 0;

	DBUG_ENTER("ch2_t6");

        /* Create orphan */

	PATH_FUNC_TRACE; 	/* pathtracing from parent */

	switch (fork())
	{
	case SYSERROR:
		xx_rpt(DELETION);
		in_rpt("fork() failed");
		break;

	case 0:
		tet_setcontext();
		/* child (orphan) reads from control tty */

		setsigs(sig_orphan);

		if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("setpgid(0, 0) failed");
			exit(0);
		}
		else
			PATH_TRACE;

		sig.sa_handler = sig_catch;
		sig.sa_flags = 0;
		(void) sigemptyset(&sig.sa_mask);
		if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("sigaction() failed when setting catching function for SIGTTOU - errno %d (%s)",
				err, errname(err));
			exit(0);
		}
		else
			PATH_TRACE;

		caught_sig = 0;
		wr_ret = write(tty_fildes, outbuf, MSG_LEN);
		wr_err = errno;
		
		if (wr_ret != MSG_LEN)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("write() returned %d, expected %d", wr_ret, MSG_LEN);
			in_rpt("errno was set to %d (%s)", wr_err, errname(wr_err));
		}
		else
			PATH_TRACE;

		if (caught_sig == SIGTTOU)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("SIGTTOU was received");
		}
		else
			PATH_TRACE;

		if (testfail == 0)
			PATH_XS_RPT(5); 

		exit(0);

	default:
		tet_setblock();
		/* parent exits, leaving orphan */
		break;
	}
	tet_setblock();

	DBUG_VOID_RETURN;
}

private void
test7()
{
        /* Write from a background process, ignoring SIGTTOU */

	int err;
	int	pathok = 0;

	DBUG_ENTER("test7");

	if (notinuse)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

        if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
        if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_lflag |= TOSTOP;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = pathok == 4 ? 1 : 0;
	(void) cppair(ch_t7, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t7");

	PATH_FUNC_TRACE;
	
	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	ret = cppair(gch_t7, do_write, WAITTIME, E_SAFAIL|E_GOTSIG|PATH_NOK);

	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE; 
	
	if (wr_ret != MSG_LEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("write() returned %d, expected %d", wr_ret, MSG_LEN);
		in_rpt("errno was set to %d (%s)", wr_err, errname(wr_err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 

	DBUG_VOID_RETURN;
}

private void
gch_t7()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("gch_t7");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		DBUG_EXIT(E_SAFAIL);
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTOU)
	{
		DBUG_EXIT(E_GOTSIG);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
test8()
{
        /* Write from a background process, blocking SIGTTOU */

	int err;
	int	pathok = 0;

	DBUG_ENTER("test8");

	if (notinuse)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

        if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
        if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	globok = pathok == 3 ? 1 : 0;
	(void) cppair(ch_t8, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int ret;
	sigset_t set;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t8");

	PATH_FUNC_TRACE;
	
	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* block SIGTTOU */

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTTOU);
	if (sigprocmask(SIG_BLOCK, &set, (sigset_t *)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask(SIG_BLOCK, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	ret = cppair(gch_t8, do_write, WAITTIME, E_SAFAIL|E_GOTSIG|PATH_NOK);

	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE; 
	
	if (wr_ret != MSG_LEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("write() returned %d, expected %d", wr_ret, MSG_LEN);
		in_rpt("errno was set to %d (%s)", wr_err, errname(wr_err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
gch_t8()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("gch_t8");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		DBUG_EXIT(E_SAFAIL);
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTOU)
	{
		DBUG_EXIT(E_GOTSIG);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
test9()
{
        /* Write from an orphaned background process */

	int err;
	int	pathok = 0;

	DBUG_ENTER("test9");

	if (notinuse)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

        if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
        if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = (pathok == 3) ? 1 : 0;

	(void) cppair(ch_t9, NULLFN, 2 * WAITTIME, 0);

	(void) sleep(WAITTIME);	/* allow orphan to report */

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t9");

        /* Create orphan */
	PATH_FUNC_TRACE; 

	switch (fork())
	{
	case SYSERROR:
		xx_rpt(DELETION);
		in_rpt("fork() failed");
		break;

	case 0:
		tet_setcontext();
		/* child (orphan) writes to control tty */

		setsigs(sig_orphan);

		if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("setpgid(0, 0) failed");
			exit(0);
		}
		else
			PATH_TRACE;

		sig.sa_handler = sig_catch;
		sig.sa_flags = 0;
		(void) sigemptyset(&sig.sa_mask);
		if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("sigaction(SIGTTOU, ...) failed");
			exit(0);
		}
		else
			PATH_TRACE;

		(void) sleep(NAP);

		caught_sig = 0;

		SET_TIMEOUT(WAITTIME/2);

		wr_ret = write(tty_fildes, outbuf, MSG_LEN);
		wr_err = errno;
	    
		CLEAR_ALARM

		if (wr_ret != -1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("write() on controlling terminal returned %d, expected -1", wr_ret);
		}
		else if (wr_err != EIO)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("write() on controlling terminal set errno to %d (%s), expected EIO",
				rd_err, errname(rd_err));
		}
		else
			PATH_TRACE;

		if (caught_sig == SIGTTOU)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("SIGTTOU was received");
		}
		else
			PATH_TRACE;

		if (testfail == 0)
			PATH_XS_RPT(5); 

		exit(0);

	default:
		tet_setblock();
		/* parent exits, leaving orphan */
		break;
	}
	tet_setblock();

	DBUG_VOID_RETURN;
}
