/*
*      SCCS:  @(#)  POSIX.os/devclass/i_canon/i_canon.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/i_canon - T.i_canon Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.i_canon

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the terminal interface, part 6

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	3 Feb 1989
MODIFICATIONS:
		Geoff Clare, 23 Aug 1989
			Race condition on reading nothing/writing partial line

		Geoff Clare, 15 Sep 1989
			Write c_cc[]'s as 1 char, not sizeof(cc_t)

		Geoff Clare, 26 Sep 1989
			Use data_rate for baud-dependent sleep time
		
		Geoff Clare, 29 May 1990
			Allow for unlimited MAX_CANON
			
		Anthony Lee, 08 August 1990
			Added resilience code.

		Geoff Clare, 6 Mar 1992
			Allow for first read() to return some data in test 1.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>
#include <pcts_max.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


#define NO_TESTS	6
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		2
#define E_WRFAIL	02
#define Strlen		(sizeof(String)-1)
#define PATH_NOK	04

int	no_tests = NO_TESTS;

extern	char	*errname();

extern	int	data_rate;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	pa_t1();
private void	pa_t2();
private void	pa_t3();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();

public	void	(*realfuncs[]) () = { test1, test2, test3, test4,
				      test5, test6, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	 twrap, 2,
	 twrap, 3,
	 twrap, 4,
	 twrap, 5,
	 twrap, 6,
	NULL, 0
};


private	int	unsupported = 0;
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	testfail;
private char	String[] = "With a capital T that rhymes with P that stands for pool!";
private unsigned max_canon;
private char	*inbuf;
private char	*outbuf;
private char	NL = '\n';
private	unsigned char chr;

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}


private void
prepare_tests()
{
	char *msg = NULL;
	long ltmp;
	int i;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}

	tty_termios_p->c_lflag |= ICANON;

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY in preparation");
		DBUG_VOID_RETURN;
	}

	/*
	 * Find the "MAX_CANON" value for testing:
	 * lesser of PCTS_MAX_CANON and the real MAX_CANON
	 */

	errno = 0;
	if ((ltmp = fpathconf(tty_fildes, _PC_MAX_CANON)) < 0)
	{
		if (errno == 0)
			max_canon = PCTS_MAX_CANON;
		else
		{
			for (i = 1; i <= NO_TESTS; i++)
				(void) tet_delete(i, "fpathconf(fd, _PC_MAX_CANON) failed in preparation");
			DBUG_VOID_RETURN;
		}
	}
	else
	{
		max_canon = ltmp;
		if (PCTS_MAX_CANON < max_canon)
			max_canon = PCTS_MAX_CANON;
	}

	DBUG_PRINTF("prep", "max_canon = %u", max_canon);

	inbuf = (char *)malloc((size_t)max_canon);
	outbuf = (char *)malloc((size_t)max_canon);

	if (inbuf == NULL || outbuf == NULL)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "could not malloc() space for buffers");
	}

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

        (void) close(tty_fildes);
        (void) close(loop_fildes);

	DBUG_VOID_RETURN;
}

private int
flushem()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("flushem");

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_RETURN(0);
}

private void
test1()
{
	/* Line processing */

	int ret;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	globok = 0; 
	ret = cppair(ch_t1, pa_t1, WAITTIME, E_WRFAIL|PATH_NOK);

	if (ret == E_WRFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE; 

	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int ret;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

        /* Block until NL */

	SET_TIMEOUT(WAITTIME/2)

	ret = write(loop_fildes, String, Strlen);

	CLEAR_ALARM

	if (ret != Strlen)
	{
		DBUG_EXIT(E_WRFAIL);
	}
	else
		PATH_TRACE; 
		
	(void) sleep(2*NAP);

	SET_TIMEOUT(WAITTIME/2)

	ret = write(loop_fildes, &NL, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		DBUG_EXIT(E_WRFAIL);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	int ret, total, err;
	int	pathok = 0;

	DBUG_ENTER("pa_t1");

	SET_TIMEOUT(NAP)

	total = read(tty_fildes, inbuf, max_canon);

	CLEAR_ALARM

	if (alrm_flag == 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned before NL received");
	}
	else
		PATH_TRACE;

	if (total < 0)
		total = 0;

	SET_TIMEOUT(WAITTIME/2)

	ret = read(tty_fildes, &inbuf[total], (unsigned)(max_canon-total));
	err = errno;

	CLEAR_ALARM

	if (ret > 0)
		total += ret;

	if (total != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read %d bytes, expected %d", total, Strlen+1);
		if (ret < 0)
			in_rpt("read() set errno to %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (strncmp(inbuf, String, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("NL test failed: read string != written string");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3); 

	DBUG_VOID_RETURN;
}


private void
test2()
{
	/* Line processing */

	int ret;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	globok = 0; 
	ret = cppair(ch_t2, pa_t2, WAITTIME, E_WRFAIL|PATH_NOK);

	if (ret == E_WRFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;
		
	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int ret;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

        /* Line is delimited by EOL */

	(void) sleep(NAP);

	chr = tty_termios_p->c_cc[VEOL];

	SET_TIMEOUT(WAITTIME/2)

	ret = write(loop_fildes, String, Strlen);
	if (ret == Strlen)
		ret += write(loop_fildes, (char *)&chr, 1);

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		DBUG_EXIT(E_WRFAIL);
	}
	else
		PATH_TRACE; 
		
	if (pathok != 1)
		DBUG_EXIT(PATH_NOK);
	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("pa_t2");

	SET_TIMEOUT(WAITTIME/2)

	ret = read(tty_fildes, inbuf, max_canon);
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d",
			ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (strncmp(inbuf, String, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("EOL test failed: read string != written string");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* Line processing */

	int ret;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0; 
	ret = cppair(ch_t3, pa_t3, WAITTIME, E_WRFAIL|PATH_NOK);

	if (ret == E_WRFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;
		
	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int ret;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

        /* Line is delimited by EOF */

	(void) sleep(NAP);

	chr = tty_termios_p->c_cc[VEOF];

	SET_TIMEOUT(WAITTIME/2)

	ret = write(loop_fildes, String, Strlen);
	if (ret == Strlen)
		ret += write(loop_fildes, (char *)&chr, 1);

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		DBUG_EXIT(E_WRFAIL);
	}
	else
		PATH_TRACE;

	if (pathok != 1)
		DBUG_EXIT(PATH_NOK);
	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("pa_t3");

	SET_TIMEOUT(WAITTIME/2)

	ret = read(tty_fildes, inbuf, max_canon);
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (strncmp(inbuf, String, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("EOF test failed: read string != written string");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test4()
{
        /* A read from a terminal shall return at most one line */
        /* even if more characters are requested */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* write more than a line */
	SET_TIMEOUT(WAITTIME/2)

	ret = write(loop_fildes, String, Strlen);
	if (ret == Strlen)
	{
		ret += write(loop_fildes, &NL, 1);
		if (ret == Strlen + 1)
			ret += write(loop_fildes, String, Strlen);
	}

	CLEAR_ALARM

	if (ret != (2 * Strlen) + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME/2)

	ret = read(tty_fildes, inbuf, max_canon);
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen+1)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen+1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(inbuf, String, (size_t)Strlen) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read string != written string");
	}
	else
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test5()
{
        /* Requesting fewer bytes than available does not lose data */

	int ret, err, ret2, err2;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME/2)

	ret = write(loop_fildes, String, Strlen);
	if (ret == Strlen)
	{
		ret += write(loop_fildes, &NL, 1);
		if (ret == Strlen + 1)
			ret += write(loop_fildes, String, Strlen);
	}

	CLEAR_ALARM

	if (ret != (2 * Strlen) + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME/2)

	ret = read(tty_fildes, inbuf, Strlen/2);
	err = errno;
	if (ret == Strlen/2)
	{
		ret2 = read(tty_fildes, &inbuf[Strlen/2], max_canon);
		err2 = errno;
	}

	CLEAR_ALARM

	if (ret != Strlen/2)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen/2);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (ret2 != Strlen + 1 - Strlen/2)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d",
			ret2, Strlen + 1 - Strlen/2);
		if (ret2 < 0)
			in_rpt("errno was set to %d (%s)", err2, errname(err2));
	}
	else if (strncmp(inbuf, String, (size_t)Strlen) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read string != written string");
	}
	else
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test6()
{
        /* Input queue is able to contain at least max_canon bytes */

	int i, ret, err;
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	for (i = 0; i < max_canon; ++i)
	{
		if (i == 0)
			PATH_TRACE; 
		outbuf[i] = 'A' + i%26;
	}
	outbuf[max_canon-1] = NL;

	SET_TIMEOUT(2 * WAITTIME)

	ret = write(loop_fildes, outbuf, max_canon);

	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		xx_rpt(FAILURE);
		in_rpt("write() of MAX_CANON (%u) bytes to VSX_TERMIOS_LOOP timed out",
			max_canon);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ret != max_canon)
	{
		xx_rpt(FAILURE);
		in_rpt("write() of MAX_CANON (%u) bytes to VSX_TERMIOS_LOOP failed",
			max_canon);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep((unsigned)(max_canon/data_rate + NAP));

	SET_TIMEOUT(2 * WAITTIME)

	ret = read(tty_fildes, inbuf, max_canon);
	err = errno;

	CLEAR_ALARM

	if (ret != max_canon)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, max_canon);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(inbuf, outbuf, (size_t)max_canon) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read string != written string");
	}
	else
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}
