/*
*      SCCS:  @(#)  POSIX.os/devclass/i_close/i_close.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/i_close - T.i_close Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.i_close

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the terminal interface, part 11

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	16 Feb 1989
MODIFICATIONS:
		Geoff Clare, 25 Aug 1989
			UNSUPPORTED result if modem control not available
		
		Geoff Clare, 31 Oct 1989
			Use LOOP side for close() instead of TTY

		J.A.Nave	28/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported

		Anthony Lee, 08 August 1990
			Added resilience code.	

		Geoff Clare, 25 June 1991
			VSX_MODEM_CONTROL="N" for unsupported result
		
		Geoff Clare, 16 Sep 1991
			Add sleep(NAP) in HUPCL test.
		
		Geoff Clare, 20 Nov 1991
			Changes to support pseudo-terminals.

		Geoff Clare, 8 June 1992
			Allow for systems with no terminal output queue.

		Geoff Clare, 13 April 1995
			Test 2 untested for clone-type pseudoterminals.
			Align test 1 with interp VWG/005.

		Geoff Clare, 8 Sept 1995
			Clear HUPCL in test 2.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <string.h>
#include <dbug.h>
#include <report.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


#define NO_TESTS	3
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		2
#define MSG_LEN		(sizeof(outbuf)-1)

extern	char	*errname();
extern	char	*newstr();
extern	void	(*vsignal())();


int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};


private char	outbuf[] = "This is standard output.\n";
private	char	inbuf[512];
private char	NL[] = "\n";
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	testfail;

private int
flushem()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("flushem");

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_RETURN(0);
}

private void
test1()
{
	DBUG_ENTER("test1");

	/* last process to close terminal causes output queue to be sent */

	testfail = 0;

	(void) cppair(ch_t1, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	char	*cp;
	int	buffered;
	int	ret, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	switch (termios_prep(&tty_fildes, tty_termios_p,
			     &loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE;
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	default:
		xx_rpt(DELETION);
		in_rpt("open/initialise of VSX_TERMIOS_TTY/LOOP failed");
		DBUG_VOID_RETURN;
	}

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (buffered && tcflow(loop_fildes, TCOOFF) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(loop_fildes, outbuf, MSG_LEN) != MSG_LEN)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (buffered)
	{
		/* close() is not guaranteed to turn output back
		   on, so we must do it */
		(void) tcflow(loop_fildes, TCOON);
		/* the close() call must follow immediately after this */
	}
	if (close(loop_fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("close() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	if (ret != MSG_LEN)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, MSG_LEN);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	/* last process to close terminal causes input queue to be discarded */

	testfail = 0;

	(void) cppair(ch_t2, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	char	*loopname, *savename;
	int	ret, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	(void) sleep(NAP);
	switch (termios_prep(&tty_fildes, tty_termios_p,
			     &loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE; 
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	default:
		xx_rpt(DELETION);
		in_rpt("open/initialise of VSX_TERMIOS_TTY/LOOP failed");
		DBUG_VOID_RETURN;
	}

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (loop_termios_p->c_cflag & HUPCL)
	{
		/* some pseudoterminal implementations cannot reopen the
		   slave after closing unless HUPCL is clear, so try to
		   clear it, but don't check for success as it might not
		   be supported */

		loop_termios_p->c_cflag &= ~HUPCL;
		(void) tcsetattr(loop_fildes, TCSANOW, loop_termios_p);
	}
		
	if (write(tty_fildes, outbuf, MSG_LEN) != MSG_LEN)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	if (close(loop_fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("close() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* If TERMIOS_LOOP is a clone-type pseudoterminal, loop_prep()
	   will open a new terminal instead of reopening the original
	   one, so we cannot check the input buffer was discarded.  We
	   can detect a clone pty by checking whether the terminal name
	   is overwritten. */

	loopname = tet_getvar("VSX_TERMIOS_LOOP");
	savename = newstr(loopname);

	(void) sleep(NAP);
	switch (loop_prep(&loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE; 
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("2nd open/initialise of VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}

	if (strcmp(loopname, savename) != 0)
	{
		xx_rpt(UNTESTED);
		in_rpt("test cannot be performed on clone-type pseudoterminals");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(tty_fildes, NL, 1) != 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	ret = read(loop_fildes, inbuf, sizeof(inbuf));
	if (ret != 1)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_XS_RPT(7); 

	DBUG_VOID_RETURN;
}

private void
test3()
{
	char *modem;

	DBUG_ENTER("test3");

	/* HUPCL set: last process to close terminal causes disconnect */

	testfail = 0;

	modem = tet_getvar("VSX_MODEM_CONTROL");
	if (modem == NULL || *modem == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_MODEM_CONTROL is not set");
		DBUG_VOID_RETURN;
	}
	else if (*modem == 'N' || *modem == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
	{
		(void) vsignal(SIGHUP, SIG_IGN);
		(void) cppair(ch_t3, NULLFN, 2 * WAITTIME, 0);
	}

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	(void) sleep(NAP);
	switch (termios_prep(&tty_fildes, tty_termios_p,
			     &loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE; 
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	default:
		xx_rpt(DELETION);
		in_rpt("open/initialise of VSX_TERMIOS_TTY/LOOP failed");
		DBUG_VOID_RETURN;
	}

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	loop_termios_p->c_cflag |= HUPCL;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("HUPCL not supported on VSX_TERMIOS_LOOP");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
				err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(loop_fildes, outbuf, MSG_LEN) != MSG_LEN)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	if (ret != MSG_LEN)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("initial read() returned %d, expected %d", ret, MSG_LEN);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_cflag &= ~CLOCAL;

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (close(loop_fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("close() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* disconnect LOOP => read on TTY will give EOF */

	SET_TIMEOUT(WAITTIME/2)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 0)
	{
		xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("read() on VSX_TERMIOS_TTY timed out");
		else
		{
			in_rpt("read() returned %d, expected 0", ret);
			if (ret < 0)
				in_rpt("errno was set to %d (%s)", err, errname(err));
		}
	}
	else
		PATH_XS_RPT(7); 

	DBUG_VOID_RETURN;
}
