/*
*      SCCS:  @(#)  POSIX.os/devclass/i_ctty/i_ctty.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/i_ctty - T.i_ctty Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.i_ctty

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the terminal interface, section 3

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	1 Feb 1989
MODIFICATIONS:
		Geoff Clare, 13 Sep 1989
			Allow for openctl() to return success even though
			the opened tty has not become the controlling tty

		Geoff Clare, 15 Sep 1989
			Write c_cc[]'s as 1 char, not sizeof(cc_t)
		
		Geoff Clare, 17 Oct 1989
			Don't allow openctl() failure to indicate tty
			could not become controlling tty.
		
		Geoff Clare, 31 Oct 1989
			Downgrade test 8 to warning as test will be
			changing when 1003.1a is ratified.
		
		Geoff Clare, 15 Mar 1990
			Delete test for relinquishing of ctty on last
			close (old test 8).

		J.A.Nave	28/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported

		Anthony Lee, 08 August 1990
			Added resilience code.

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called
		
		Stuart Boutell, 1 May 1991
			Added test that controlling terminals are not
			relinquished on last close when other processes
			have open fds on that controlling terminal.
		
		Geoff Clare, 17 June 1991
			Keep terminal open in "session leader exit
			relinquishes" test.
		
		Geoff Clare, 26 June 1991
			VSX_TCGETPGRP_SUPPORTED="N" for unsupported result.
		
		Geoff Clare, 20 Nov 1991
			Add sleep at start of test 8.
			Changes to support pseudo-terminals.

		Stuart Boutell, 10 June 1992
			Added test that access to the terminal by surviving
		processes in that session may be treated as if modem
		disconnect had occurred.
		
		Geoff Clare, 17 Nov 1993
			Execute test 8 in a child so that pseudo-terminal
			loopback will be cleaned up properly.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


#define NO_TESTS	10
#define WAITTIME	(5 * SPEEDFACTOR)
#define	YES		1
#define	NO		0
#define	CHILDREN	10
#define NAP		4
#define NOK		002
#define PATH_NOK	04
extern	void	setsigs();
extern	char	*errname();
extern	char	*signame();
extern	void	(*vsignal())();


int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	ch_t6();
private void	ch_t7();
private void	ch_t8();
private void	ch_t9();
private void	ch_t10();
private void	gch_t2();
private void	gch_t7();
private void	gch_t8();
private	void	gch_t9();
private	void	gpa_t9();
private	void	gch_t10();
private	void	gpa_t10();
private void	prepare_tests();

private int	ctty();
private void	children();
private void	interrupt();
private void	wait_up();
private void	twrap();

public	void	(*realfuncs[]) () = { test1, test2, test3, test4, test5,
				      test6, test7, test8, test9, test10,
				      NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	twrap, 1,
	twrap, 2,
	twrap, 3,
	twrap, 4,
	twrap, 5,
	twrap, 6,
	twrap, 7,
	twrap, 8,
	twrap, 9,
	twrap, 10,
	NULL, 0
};

private	int	unsupported = 0;
private	char	*termios_tty;
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private	int	caught_sig = 0;
private int	testfail;
private	unsigned char chr;
private	int	warntest = -1;

private void
sig_catch(sig)
{
	caught_sig = sig;
#ifdef	TESTBED
	caught_sig++;
#endif
}

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	char *msg = NULL;
	int i;

	DBUG_ENTER("prepare_tests");

	termios_tty = tet_getvar("VSX_TERMIOS_TTY");
	if (termios_tty == NULL || *termios_tty == '\0')
		msg = "parameter VSX_TERMIOS_TTY not defined";
        else if (strncmp(termios_tty, "unsup", (size_t)5) == 0)
		unsupported = 1;

	if (msg != NULL)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
	}

	DBUG_VOID_RETURN;
}


private void
test1()
{
	/* All session's procs share ctty */

	DBUG_ENTER("test1");

	testfail = 0;

	(void) cppair(ch_t1, NULLFN, 2 * NAP + WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	pathok = 0;
	DBUG_ENTER("ch_t1");

	/* acquire controlling terminal */
	if (ctty() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* fork children, pause for signal */
	globok = 0;
	children(CHILDREN);
	PATH_FUNC_TRACE;

	/* send interrupt to self and children */
	globok = 0;
	interrupt(YES);
	PATH_FUNC_TRACE;

	/* check for interrupted children */
	globok = 0;
	wait_up(CHILDREN);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* New session cannot acquire another's controlling terminal */

	DBUG_ENTER("test2");

	testfail = 0;

	(void) cppair(ch_t2, NULLFN, 2 * NAP + 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	if (ctty() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	interrupt(YES);
	PATH_FUNC_TRACE;

	(void) sleep(NAP);	/* NAP was TIMEOUT - GWC*/

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting SIGINT to be ignored - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = pathok == 3 ? 1 : 0;

	(void) cppair(gch_t2, NULLFN, NAP + WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
gch_t2()
{
	int	pathok = 0;
	DBUG_ENTER("gch_t2");

	PATH_FUNC_TRACE;
	
	if (setsid() == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setsid() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME/2)

	tty_fildes = openctl(termios_tty, O_RDWR|O_NONBLOCK);

	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		xx_rpt(DELETION);
		in_rpt("openctl(VSX_TERMIOS_TTY, O_RDWR|O_NONBLOCK) timed out");
		DBUG_VOID_RETURN;
	}
	else if (tty_fildes < 0)
	{
		/* If tty cannot become controlling terminal, openctl() must
		   open it anyway, as non-controlling terminal */

		xx_rpt(DELETION);
		in_rpt("openctl(VSX_TERMIOS_TTY, O_RDWR|O_NONBLOCK) failed - errno %d (%s)",
			errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tty_init(tty_fildes, tty_termios_p) != 0 ||
	    tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not initialise VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	interrupt(NO);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* Non-session leader cannot acquire controlling terminal */

	DBUG_ENTER("test3");

	testfail = 0;

	(void) cppair(ch_t3, NULLFN, 2*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int	pathok = 0;
	DBUG_ENTER("ch_t3");

	switch (terms_prep(O_RDWR, &tty_fildes, tty_termios_p,
			     O_RDWR, &loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE; 
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");

		DBUG_VOID_RETURN;

	default:
		xx_rpt(DELETION);

		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY/LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY/LOOP");

		DBUG_VOID_RETURN;
	}

	globok = 0;
	interrupt(NO);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* O_NOCTTY prevents controlling terminal */

	DBUG_ENTER("test4");

	testfail = 0;

	(void) cppair(ch_t4, NULLFN, 2 * NAP + WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	if (setsid() == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setsid() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch (terms_prep(O_RDWR|O_NOCTTY, &tty_fildes, tty_termios_p,
			     O_RDWR, &loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE; 
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");

		DBUG_VOID_RETURN;

	default:
		xx_rpt(DELETION);

		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY/LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY/LOOP");

		DBUG_VOID_RETURN;
	}

	globok = 0;
	interrupt(NO);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/*
	 * foreground process group of controlling terminal
	 * is set to process group of session leader
	 */

	char *supp;

	DBUG_ENTER("test5");

	testfail = 0;

	supp = tet_getvar("VSX_TCGETPGRP_SUPPORTED");
	if (supp == NULL || *supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TCGETPGRP_SUPPORTED not set");
		DBUG_VOID_RETURN;
	}

	if (*supp == 'N' || *supp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TCGETPGRP_SUPPORTED is set to 'N'");
	}
	else
		(void) cppair(ch_t5, NULLFN, NAP + WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	pathok = 0;
	DBUG_ENTER("ch_t5");

	if (ctty() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	interrupt(YES);
	PATH_FUNC_TRACE;

	if (getpgrp() != tcgetpgrp(tty_fildes))
	{
		xx_rpt(FAILURE);
		in_rpt("Controlling terminal's process group != session group");
	}
	else
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* Controlling terminal is inherited by child process */

	DBUG_ENTER("test6");

	testfail = 0;

	(void) cppair(ch_t6, NULLFN, NAP + WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	int	pathok = 0;
	DBUG_ENTER("ch_t6");

	if (ctty() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	children(1);
	PATH_FUNC_TRACE;

	globok = 0;
	interrupt(YES);
	PATH_FUNC_TRACE;

	globok = 0;
	wait_up(1);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/*
	 * Process loses controlling terminal after changing
	 * process group using setsid()
	 */

	DBUG_ENTER("test7");

	testfail = 0;

	(void) cppair(ch_t7, NULLFN, 3 * NAP + 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t7");

	if (ctty() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	interrupt(YES);
	PATH_FUNC_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting SIGINT to be ignored - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = pathok == 3 ? 1 : 0;
	(void) cppair(gch_t7, NULLFN, 2 * NAP + WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
gch_t7()
{
	int	pathok = 0;
	DBUG_ENTER("gch_t7");

	PATH_FUNC_TRACE;

	globok = 0;
	interrupt(YES);
	PATH_FUNC_TRACE;

	(void) setsid();

	globok = 0;
	interrupt(NO);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* When session leader exits, terminal can be reacquired as ctty */

	DBUG_ENTER("test8");

	testfail = 0;

	(void) cppair(ch_t8, NULLFN, 3 * (NAP + WAITTIME), 0);

	DBUG_VOID_RETURN;
}


private void
ch_t8()
{
	int	pathok = 0;
	int	ret;

	DBUG_ENTER("ch_t8");

	testfail = 0;

	/* keep the terminal open to prove it is the session leader exit
	   which relinquishes the ctty, not the last close */
	
	switch (terms_prep(O_RDWR|O_NOCTTY, &tty_fildes, tty_termios_p,
			     O_RDWR, &loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE; 
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");

		DBUG_VOID_RETURN;

	default:
		xx_rpt(DELETION);

		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY/LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY/LOOP");

		DBUG_VOID_RETURN;
	}

	ret = cppair(gch_t8, NULLFN, NAP + WAITTIME, NOK|PATH_NOK);
	if (ret  == NOK)
	{
		(void) close(tty_fildes);
		(void) close(loop_fildes);
		DBUG_VOID_RETURN;
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_TRACE;
		
	/* Second call "re-acquires" ctty */
	ret = cppair(gch_t8, NULLFN, NAP + WAITTIME, NOK|PATH_NOK);
	if (ret == NOK)
	{
		(void) close(tty_fildes);
		(void) close(loop_fildes);
		DBUG_VOID_RETURN;
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_TRACE;
		
	PATH_XS_RPT(3); 

	(void) close(tty_fildes);
	(void) close(loop_fildes);
	DBUG_VOID_RETURN;
}

private void
gch_t8()
{
	int	pathok = 0;
	DBUG_ENTER("gch_t8");

	globok = 0; 
	if (ctty() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	interrupt(YES);
	PATH_FUNC_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);
	DBUG_VOID_RETURN;
}

private void
test9()
{
	/* When a process closes all file descriptors associated with its
	  controlling terminal and at least one other process has an open
	  file descriptor which references the same controlling terminal,
	  then the first process does not relinquish its controlling term. */

	DBUG_ENTER("test9");

	testfail = 0;

	(void) cppair(ch_t9, NULLFN, (NAP + WAITTIME)*2, NOK|PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int	ret;
	int	pathok = 0;
	DBUG_ENTER("ch_t9");

	globok = 0; 
	if (ctty() != 0)	/* Acquire controlling terminal */
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	
				/* Child keeps open fds, parent closes them,
				  child writes interrupt character, and
				  parent should receive the SIGINT */
	globok = 0;
	ret = cppair(gch_t9, gpa_t9, NAP + WAITTIME, NOK|PATH_NOK);
	if (ret  == NOK)
	{
		DBUG_VOID_RETURN;
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;



	if(!testfail)
	{
		PATH_XS_RPT(2);
	}
	DBUG_VOID_RETURN;
}


private void
gch_t9()
{
	int	pathok = 0;
	DBUG_ENTER("gch_t9");

	(void) sleep(NAP);

	globok = 0;
	interrupt(YES);
	PATH_FUNC_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok != 1)
		DBUG_EXIT(PATH_NOK);
	DBUG_VOID_RETURN;
}

private void
gpa_t9()
{
	struct sigaction sig;
	int	pathok = 0;
	DBUG_ENTER("gpa_t9");

	(void) close(tty_fildes);
	(void) close(loop_fildes);
	
	caught_sig = 0;
	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		int err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	(void) pause();

	CLEAR_ALARM

	if(alrm_flag > 0)
	{
		if(!testfail)
			xx_rpt(FAILURE);
		in_rpt("Parent process did not receive SIGINT when child wrote");
		in_rpt("interrupt character to controlling terminal");
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	if(caught_sig != SIGINT)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("process did not receive SIGINT when expected");
		in_rpt("signal received: %d (%s)", SIGINT, signame(caught_sig));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
test10()
{
	/* When controlling process exits, access may be denied. */

	DBUG_ENTER("test10");

	(void) cppair(ch_t10, NULLFN, 2*NAP + WAITTIME, 0);

	(void) sleep(WAITTIME); /* allow time for reporting */

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	DBUG_ENTER("ch_t10");

	(void) setsid();

	globok = 0; 
	if (ctty() != 0)
	{
		DBUG_VOID_RETURN;
	} 

	/* globok is tested in gch_t10 */

	(void) cppair(gch_t10, gpa_t10, NAP + WAITTIME, 0);

}

private void
gpa_t10()
{

	DBUG_ENTER("gpa_t10");

	(void) sleep(NAP); /* give child a chance to ignore SIGHUP */

	DBUG_PRINTF("gpa_t10","grandparent exiting");

	DBUG_EXIT(OK);
}

private void
gch_t10()
{
	int pathok = 0;
	char *buf = "octarine is the colour of magic";
	unsigned int	buflen;
	int	ret, err;

	DBUG_ENTER("gch_t10");

	PATH_FUNC_TRACE; /* this picks up globok from ctty() call  */
	
	DBUG_PRINTF("gch_t10", "child: %d", getpid());

	(void) vsignal(SIGHUP, SIG_IGN);

	(void) sleep(NAP + WAITTIME/2); /* wait for parent to exit */


	buflen = strlen(buf);
	
	DBUG_PRINTF("gch_t10","about to write");

	errno = NOERROR;
	ret = write(tty_fildes, buf, buflen);
	err = errno;

	DBUG_PRINTF("gch_t10","write return value: %d", ret);
	DBUG_PRINTF("gch_t10","errno: %d (%s)", err, errname(err));

	if (ret != buflen) {
		if (ret == SYSERROR) {
			if (err != EIO) {
				xx_rpt(FAILURE);
				in_rpt("write() to controlling terminal failed");
				in_rpt("expecting: %u bytes written, or EIO",
					buflen);
				in_rpt("got error %d (%s)", err, errname(err));
				DBUG_VOID_RETURN;
			} else
				PATH_TRACE;
		} else {
			xx_rpt(FAILURE);
			in_rpt("write() to controlling terminal failed");
			in_rpt("expecting: %u bytes written, or EIO", buflen);
			in_rpt("got return value %d", ret);
			DBUG_VOID_RETURN;
		}
	} else
		PATH_TRACE;

	PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


/**************************************
* subroutines
***************************************/

private int
ctty()
{
	/* acquire controlling terminal */

	int	pathok = 0;
	DBUG_ENTER("ctty");

	switch (termios_prep(&tty_fildes, tty_termios_p,
			     &loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE; 
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");

		DBUG_RETURN(-1);
	default:
		if (testfail++ == 0)
			xx_rpt(DELETION);

		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY/LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY/LOOP");

		DBUG_RETURN(-1);
	}
	PATH_FUNC_RPT(1); 

	DBUG_RETURN(0);
}

private void
children(number)
int number;
{
	/* fork children, pause for signal */

	int i, err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("children");

	sig.sa_handler = SIG_DFL;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting default action for SIGINT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        for (i = 0; i < number; ++i)
	{
		switch (fork())
		{
		case -1:
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("fork() failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;

		case 0:	/* child */
			tet_setcontext();
			setsigs(SIG_DFL);

			DBUG_PRINTF("children","This is child %d",i);

			(void) sleep(WAITTIME);	/* pause for signal */
			
			DBUG_EXIT(0);

		default:
			tet_setblock();
			PATH_TRACE; 
		}
		tet_setblock();
	}
	PATH_FUNC_RPT(1 + number); 
	DBUG_VOID_RETURN;
}

private void
interrupt(toggle)
int toggle;                     /* YES == expecting SIGINT      */
{                               /* NO == not expecting SIGINT   */
	/* send SIGINT to self and children */

	int err, ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("interrupt");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        /* for child setup, if any - allow for this in cppair() timeouts */
        (void) sleep(NAP);

        caught_sig = 0;

	DBUG_PRINTF("interrupt", "c_cc[VINTR] = 0%o",
		(int)tty_termios_p->c_cc[VINTR]);

        chr = tty_termios_p->c_cc[VINTR];

        SET_TIMEOUT(WAITTIME/2)

        ret = write(loop_fildes, (char *)&chr, 1);
	err = errno;
	if (ret == 1)
		(void) pause();

        CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (toggle == YES && caught_sig != SIGINT)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("process did not receive SIGINT when expected");
	}
	else
		PATH_TRACE;

	if (toggle == NO && caught_sig == SIGINT)
	{
		if (testfail++ == 0)
		{
			if (warntest == tet_thistest)
				xx_rpt(WARNING);
			else
				xx_rpt(FAILURE);
		}
		in_rpt("process received SIGINT when not expected");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
wait_up(number)
int number;
{
	/* check child exit status: should be sigint */

	int i, err, wait_stat;
	pid_t pid;
	int	pathok = 0;

	DBUG_ENTER("wait_up");

	/* allow for this in cppair() timeouts */
        (void) sleep(NAP);

	for (i = 0; i < number; i++)
	{
        	pid = wait(&wait_stat);
        	if (pid <= 0)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("wait() failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (WIFSIGNALED(wait_stat) && WTERMSIG(wait_stat) == SIGINT)
			continue;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
                in_rpt("child %ld did not terminate with SIGINT (exit status 0x%x)",
                        (long)pid, wait_stat);
        }
	PATH_FUNC_RPT(number); 
	DBUG_VOID_RETURN;
}
