/*
*      SCCS:  @(#)  POSIX.os/devclass/i_inproc/i_inproc.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/i_inproc - T.i_inproc Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.i_inproc

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the terminal interface, part 5

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	1 Feb 1989
MODIFICATIONS:
		Geoff Clare, 13 June 1990
			Repeat tests with O_NONBLOCK set in canonical and
			non-canonical modes.

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported

		Anthony Lee, 08 August 1990
			Added resilience code.
			
		Geoff Clare, 8 June 1992
			Change MIN to 1 in non-canonical part of test4.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


#define NO_TESTS	4
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		3
#define	E_WRFAIL	02
#define Strlen		(sizeof(outbuf)-1)
#define PATH_NOK	04

extern	char	*errname();
extern	cc_t	PEOF, PEOL;

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	ch_t1();
private void	pa_t1();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();

public	void	(*realfuncs[]) () = { test1, test2, test3, test4, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	 twrap, 2,
	 twrap, 3,
	 twrap, 4,
	NULL, 0
};

private	int	unsupported = 0;
private char	inbuf[512];
private char	outbuf[]     = "Quothe the raven: Nevermore!\n";
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	testfail;

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}


private void
prepare_tests()
{
	char *msg = NULL;
	int i;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
		break;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

        (void) close(tty_fildes);
        (void) close(loop_fildes);

	DBUG_VOID_RETURN;
}

private int
flushem()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("flushem");

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	PATH_FUNC_RPT(2); 
	DBUG_RETURN(0);
}

private int
tsetup()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("tsetup");
 
        if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (flushem() != 0)
	{
		PATH_FUNC_TRACE; 
		DBUG_RETURN(-1);
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(2); 		

	DBUG_RETURN(0);
}

private void
test1()
{
	/* O_NONBLOCK clear */

	int ret;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* block until alarm */
	SET_TIMEOUT(NAP)

	(void) read(tty_fildes, inbuf, sizeof(inbuf));

	CLEAR_ALARM

	if (alrm_flag == 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read() did not block (until signal received)");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0; 
	ret = cppair(ch_t1, pa_t1, WAITTIME, E_WRFAIL|PATH_NOK);

	if (ret == E_WRFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;
		
	if (testfail == 0)
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	pathok = 0;
	DBUG_ENTER("ch_t1");

	(void) sleep(NAP);

	if (write(loop_fildes, outbuf, Strlen) != Strlen)
	{
		DBUG_EXIT(E_WRFAIL);
	}
	else
		PATH_TRACE;

	if (pathok != 1)
		DBUG_EXIT(PATH_NOK);
	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	int ret;
	int	pathok = 0;

	DBUG_ENTER("pa_t1");

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
    
	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() did not block (until data available)");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* O_NONBLOCK set, enough data */

	int ret, err, flags;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	flags = fcntl (tty_fildes, F_GETFL);
	if (fcntl(tty_fildes, F_SETFL, flags | O_NONBLOCK) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl(F_SETFL) failed when setting O_NONBLOCK on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* canonical mode */

	tty_termios_p->c_lflag |= ICANON;
	tty_termios_p->c_cc[VEOF] = PEOF;
	tty_termios_p->c_cc[VEOL] = PEOL;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, outbuf, Strlen);
	err = errno;
    
	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, Strlen);
	err = errno;
	
	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in canonical mode");
		in_rpt("read() returned %d, expected %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in canonical mode");
		in_rpt("read() gave incorrect buffer contents");
	}
	else
		PATH_TRACE;

	/* non-canonical mode */

	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 0;
	tty_termios_p->c_cc[VTIME] = 0;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, outbuf, Strlen);
	err = errno;
    
	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, Strlen);
	err = errno;
	
	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in non-canonical mode");
		in_rpt("read() returned %d, expected %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in non-canonical mode");
		in_rpt("read() gave incorrect buffer contents");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7); 

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* O_NONBLOCK set, not enough data */

	int ret, err, flags;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	flags = fcntl (tty_fildes, F_GETFL);
	if ((flags & O_NONBLOCK) == 0 &&
		fcntl(tty_fildes, F_SETFL, flags | O_NONBLOCK) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl(F_SETFL) failed when setting O_NONBLOCK on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* canonical mode */

	tty_termios_p->c_lflag |= ICANON;
	tty_termios_p->c_cc[VEOF] = PEOF;
	tty_termios_p->c_cc[VEOL] = PEOL;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, outbuf, Strlen);
	err = errno;
    
	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in canonical mode");
		in_rpt("read() returned %d, expected %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in canonical mode");
		in_rpt("read() gave incorrect buffer contents");
	}
	else
		PATH_TRACE;

	/* non-canonical mode */

	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 0;
	tty_termios_p->c_cc[VTIME] = 0;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, outbuf, Strlen);
	err = errno;
    
	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (ret != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in non-canonical mode");
		in_rpt("read() returned %d, expected %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(outbuf, inbuf, (size_t)Strlen) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in non-canonical mode");
		in_rpt("read() gave incorrect buffer contents");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7); 

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* O_NONBLOCK set, no data */

	int ret, err, flags;
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	flags = fcntl (tty_fildes, F_GETFL);
	if ((flags & O_NONBLOCK) == 0 &&
		fcntl(tty_fildes, F_SETFL, flags | O_NONBLOCK) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl(F_SETFL) failed when setting O_NONBLOCK on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* canonical mode */

	tty_termios_p->c_lflag |= ICANON;
	tty_termios_p->c_cc[VEOF] = PEOF;
	tty_termios_p->c_cc[VEOL] = PEOL;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in canonical mode");
		in_rpt("read() timed out");
	}
	else if (ret != -1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in canonical mode");
		in_rpt("read() returned %d, expected -1", ret);
	}
	else if (err != EAGAIN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in canonical mode");
		in_rpt("read() set errno to %d (%s), expected EAGAIN",
			err, errname(err));
	}
	else
		PATH_TRACE;

	/* non-canonical mode */

	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 1;
	tty_termios_p->c_cc[VTIME] = 0;

	globok = 0; 
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in non-canonical mode");
		in_rpt("read() timed out");
	}
	else if (ret != -1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in non-canonical mode");
		in_rpt("read() returned %d, expected -1", ret);
	}
	else if (err != EAGAIN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("in non-canonical mode");
		in_rpt("read() set errno to %d (%s), expected EAGAIN",
			err, errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 

	DBUG_VOID_RETURN;
}
