/*
*      SCCS:  @(#)  POSIX.os/devclass/i_modem/i_modem.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/i_modem - T.i_modem Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.i_modem

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the terminal interface, part 10

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	15 Feb 1989
MODIFICATIONS:
		Geoff Clare, 25 Aug 1989
			UNSUPPORTED result if modem control not available.
			Allow time for modem lines to establish after
			clearing CLOCAL.
		
		Geoff Clare, 27 Sep 1989
			Open without O_NONBLOCK to allow modem lines
			to establish, instead of tty_prep()/loop_prep().

		Geoff Clare, 26 Oct 1989
			Change sleep(NAP) at end of do_tty() to sleep(2*NAP)
			to ensure loop side is closed before child continues.
		
		Geoff Clare, 6 June 1990
			Added modem control tests moved from c_cflag
		
		Geoff Clare, 13 June 1990
			Add read() test case when SIGHUP caught

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported

		Anthony Lee, 08 August 1990
			Added resilience code.

		Geoff Clare, 12 Mar 1991
			Add "creat() waits for connection" test.

		Geoff Clare, 25 June 1991
			VSX_MODEM_CONTROL="N" for unsupported result.

		Geoff Clare, 12 Mar 1992
			Set HUPCL before closing loop side in ch_t6.

		Geoff Clare, 28 May 1992
			Add two tests to cover missing combinations of
			open(O_NONBLOCK)/open(~O_NONBLOCK)/creat() and
			CLOCAL/~CLOCAL.

		Geoff Clare, 8 June 1992
			Increase child time-outs in tests 6 and 9.

		Geoff Clare, 23 Nov 1992
			Make sure HUPCL is set before closing loop
			side in ch_t9().

		Geoff Clare, 20 July 1994
			Add some sleep(NAP) calls where loop side is
			opened after tty side is closed.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <fcntl.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


#define NO_TESTS	11
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		4
#define NOK		0002
#define E_LOOPPREP	0004
#define E_TCSFAIL	0010
#define E_NOTSUPP	0020
#define E_WRITE		0040
#define E_CLOSE		0100
#define E_PARAM		0200
#define BUFLEN		(sizeof(outbuf)-1)
#define PATH_NOK	03

extern	char	*errname();
extern	void	(*vsignal())();


int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	ch_t1();
private void	ch_t2a();
private void	ch_t2b();
private void	ch_t2c();
private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	ch_t6();
private void	ch_t7();
private void	ch_t8();
private void	ch_t9();
private void	ch_t10();
private void	ch_t11();
private void	gch_t69();
private void	pa_t6();
private void	pa_t9();
private void	prepare_tests();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	NULL, 0
};


private char	outbuf[] = "SIGHUP, TWO, THREE, FOUR!\n";
private	char	inbuf[512];
private char	*termios_tty;
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	caught_sig;
private int	testfail;
private int	pa_fail;
private int	modem_unsup;
private int	unsupported;


private void
sig_catch(sig)
int sig;
{
	caught_sig = sig;
}

private void
prepare_tests()
{
	char *modem;
	int i;

	DBUG_ENTER("prepare_tests");

	termios_tty = tet_getvar("VSX_TERMIOS_TTY");
	if (termios_tty == NULL || *termios_tty == '\0')
	{
		for (i = 1; i <= no_tests; i++)
			tet_delete(i,
				"parameter VSX_TERMIOS_TTY is not set");
		DBUG_VOID_RETURN;
	}

	if (termios_tty != NULL &&
		strncmp(termios_tty, "unsup", (size_t)5) == 0)
	{
		unsupported = 1;
		DBUG_VOID_RETURN;
	}

	modem = tet_getvar("VSX_MODEM_CONTROL");
	if (modem == NULL || *modem == '\0')
	{
		for (i = 1; i <= no_tests; i++)
			tet_delete(i,
				"parameter VSX_MODEM_CONTROL is not set");
		DBUG_VOID_RETURN;
	}
	modem_unsup = (*modem == 'N' || *modem == 'n');

	DBUG_VOID_RETURN;
}

private void
do_tty()
{
	int		ret, err;
	tcflag_t	old_cflag;
	int	pathok = 0;

	DBUG_ENTER("do_tty");

	if (setsid() == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("setsid() failed in child");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	tty_fildes = openctl(termios_tty, O_RDWR);
	err = errno;

	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("openctl(O_RDWR) on VSX_TERMIOS_TTY timed out");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (tty_fildes < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("openctl(O_RDWR) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	old_cflag = tty_termios_p->c_cflag;
	if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	tty_termios_p->c_cflag = old_cflag;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	if (ret != BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("initial read on VSX_TERMIOS_TTY returned %d, expected %d",
			ret, BUFLEN);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	tty_termios_p->c_cflag |= HUPCL;
	tty_termios_p->c_cflag &= ~CLOCAL;

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			if (testfail++ == 0)
				xx_rpt(UNSUPPORTED);
			in_rpt("(~CLOCAL, HUPCL) not supported on VSX_TERMIOS_TTY");
		}
		else
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
		}
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void) sleep(2*NAP);

	PATH_FUNC_RPT(8); 

	DBUG_VOID_RETURN;
}

private void
do_loop()
{
	char		*termios_loop;
	tcflag_t	old_cflag;
	int	pathok = 0;

	DBUG_ENTER("do_loop");

	/* parent sleeps and closes */

	pa_fail = 0;

	termios_loop = tet_getvar("VSX_TERMIOS_LOOP");
	if (termios_loop == NULL || *termios_loop == '\0')
	{
		pa_fail = E_PARAM;
		PATH_FUNC_RPT(0); 		/* Can exit here */
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	loop_fildes = open(termios_loop, O_RDWR);

	CLEAR_ALARM

	if (loop_fildes < 0 ||
		tcgetattr(loop_fildes, loop_termios_p) == SYSERROR)
	{
		pa_fail = E_LOOPPREP;
		PATH_FUNC_RPT(1); 		/* Can exit here */
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	old_cflag = loop_termios_p->c_cflag;
	if (loop_init(loop_fildes, loop_termios_p) != 0)
	{
		pa_fail = E_LOOPPREP;
		PATH_FUNC_RPT(2); 		/* Can exit here */
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_cflag = old_cflag;
	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		pa_fail = E_TCSFAIL;
		PATH_FUNC_RPT(3); 		/* Can exit here */
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_cflag |= HUPCL;
	loop_termios_p->c_cflag &= ~CLOCAL;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		if (errno == EINVAL)
			pa_fail = E_NOTSUPP;
		else
			pa_fail = E_TCSFAIL;
		PATH_FUNC_RPT(4); 		/* Can exit here */
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	if (write(loop_fildes, outbuf, BUFLEN) != BUFLEN)
	{
		pa_fail = E_WRITE;
		PATH_FUNC_RPT(5); 		/* Can exit here */
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	DBUG_PRINTF("do_loop", "before close");

	if (close(loop_fildes) == SYSERROR)
	{
		pa_fail = E_CLOSE;
		PATH_FUNC_RPT(6); 		/* Can exit here */
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(7); 
	
	DBUG_VOID_RETURN;
}

private void
pa_report()
{
	int	pathok = 0;
	DBUG_ENTER("pa_report");

	if (pa_fail != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		switch (pa_fail)
		{
		case E_PARAM:
			in_rpt("parameter VSX_TERMIOS_TTY not set");
			break;

		case E_LOOPPREP:
			in_rpt("open/initialise failed on VSX_TERMIOS_LOOP");
			break;

		case E_TCSFAIL:
			in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP");
			break;

		case E_NOTSUPP:
			in_rpt("(~CLOCAL, HUPCL) not supported on VSX_TERMIOS_LOOP");
			break;

		case E_WRITE:
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
			break;

		case E_CLOSE:
			in_rpt("close() failed on VSX_TERMIOS_LOOP");
			break;
		}
	}
	else
		PATH_TRACE;
	PATH_FUNC_RPT(1); 

	DBUG_VOID_RETURN;
}

private int
ch_prep()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_prep");

	switch (termios_prep(&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE; 
		break;
	case 1:
		if (testfail++ == 0)
			xx_rpt(UNSUPPORTED);
		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP");
		DBUG_RETURN(-1);
		break;
	default:
		if (testfail++ == 0)
			xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP");
		DBUG_RETURN(-1);
	}

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGHUP, &sig, NULLSA) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed to set catching function for SIGHUP");
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_RETURN(0);
}


private void
test1()
{
	int	pathok = 0;
	int	ret;
	DBUG_ENTER("test1");

	testfail = 0;

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (modem_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        ret = cppair(ch_t1, do_loop, 2 * WAITTIME, NOK|PATH_NOK);
        if (ret == NOK)
	{
		testfail++;	/* reported in child */
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;

	globok = 0;
	pa_report();
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4); 
	
	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	/* SIGHUP on disconnect */

	int err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGHUP, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGHUP - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	globok = 0;
	do_tty();
	PATH_FUNC_TRACE;

	if (caught_sig != SIGHUP)
		(void) sleep(WAITTIME);	/* pause for signal */

	if (caught_sig != SIGHUP)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGHUP not received");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 3)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	pathok = 0;
	int	ret;
	DBUG_ENTER("test2");

	testfail = 0;

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (modem_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP); /* wait for modem lines to drop after close */

        ret = cppair(ch_t2a, do_loop, 2 * WAITTIME, NOK|PATH_NOK);
        if (ret == NOK)
	{
		testfail++;	/* reported in child */
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;

	globok = 0;
	pa_report();
	PATH_FUNC_TRACE;

	(void) sleep(NAP); /* wait for modem lines to drop after close */

        ret = cppair(ch_t2b, do_loop, 2 * WAITTIME, NOK|PATH_NOK);
        if (ret == NOK)
	{
		testfail++;	/* reported in child */
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;

	globok = 0;
	pa_report();
	PATH_FUNC_TRACE;

	(void) sleep(NAP); /* wait for modem lines to drop after close */

        ret = cppair(ch_t2c, do_loop, 2 * WAITTIME, NOK|PATH_NOK);
        if (ret == NOK)
	{
		testfail++;	/* reported in child */
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;

	globok = 0;
	pa_report();
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(8); 
	
	DBUG_VOID_RETURN;
}

private void
ch_t2a()
{
	/* SIGHUP blocked */

	int ret, err;
	sigset_t set;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t2a");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGHUP, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGHUP - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGHUP);
	if (sigprocmask(SIG_BLOCK, &set, (sigset_t *)0) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigprocmask(SIG_BLOCK, ...) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	globok = 0;
	do_tty();
	PATH_FUNC_TRACE;

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	if (ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 0", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 4)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t2b()
{
	/* SIGHUP caught */

	int ret, err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t2b");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGHUP, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting SIGHUP to be caught - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	globok = 0;
	do_tty();
	PATH_FUNC_TRACE;

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	if (ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 0", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 3)
		DBUG_EXIT(PATH_NOK);
	DBUG_VOID_RETURN;
}

private void
ch_t2c()
{
	/* SIGHUP ignored */

	int ret, err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t2c");

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGHUP, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting SIGHUP to be ignored - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	globok = 0;
	do_tty();
	PATH_FUNC_TRACE;

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	if (ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 0", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 3)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int	pathok = 0;
	int	ret;
	DBUG_ENTER("test3");

	testfail = 0;

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (modem_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP); /* wait for modem lines to drop after close */

        ret = cppair(ch_t3, do_loop, WAITTIME, NOK|PATH_NOK);
        if (ret == NOK)
	{
		testfail++;	/* reported in child */
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;

	globok = 0;
	pa_report();
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4); 
	
	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	/* Write after disconnect */

	int i, ret, err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGHUP, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting SIGHUP to be ignored - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	globok = 0;
	do_tty();
	PATH_FUNC_TRACE;

	for (i = 0; i < 2; i++)
	{
		ret = write(tty_fildes, outbuf, BUFLEN);
		err = errno;
		if (ret != -1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("write() returned %d, expected -1", ret);
			DBUG_EXIT(NOK);
		}
		else if (err != EIO)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("write() set errno to %d (%s), expected EIO",
				err, errname(err));
			DBUG_EXIT(NOK);
		}
		else
			PATH_TRACE;
	}

	if (close(tty_fildes) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("close() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	errno = 0;
	ret = write(tty_fildes, outbuf, sizeof(outbuf));
	err = errno;

	if (ret == -1 && err == EIO)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("write() still gave errno EIO after device was closed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 6)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* HUPCL clear */
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (modem_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = pathok == 2 ? 1 : 0;

	(void) cppair(ch_t4, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	PATH_FUNC_TRACE; 	/* Pathcheck the parent */

	globok = 0; 
	if (ch_prep() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	tty_termios_p->c_cflag &= ~(HUPCL|CLOCAL);
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("~HUPCL, ~CLOCAL not supported on VSX_TERMIOS_TTY");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	SET_TIMEOUT(NAP)

	(void) close(tty_fildes);
	if (alrm_flag == 0)
		(void) pause();

	CLEAR_ALARM

	if (caught_sig == SIGHUP)
	{
		xx_rpt(FAILURE);
		in_rpt("~HUPCL failed - SIGHUP received");
	}
	else
		PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* CLOCAL set */
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (modem_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = pathok == 2 ? 1 : 0;

	(void) cppair(ch_t5, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("ch_t5");

	PATH_FUNC_TRACE;
	
	globok = 0; 
	if (ch_prep() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	tty_termios_p->c_cflag |= CLOCAL;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("CLOCAL not supported on VSX_TERMIOS_TTY");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_cflag &= ~CLOCAL;
	loop_termios_p->c_cflag |= HUPCL;
	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			xx_rpt(DELETION);
			in_rpt("(~CLOCAL, HUPCL) not supported on VSX_TERMIOS_LOOP");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
				err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	SET_TIMEOUT(NAP)

	(void) close(loop_fildes);
	if (alrm_flag == 0)
		(void) pause();

	CLEAR_ALARM

	if (caught_sig == SIGHUP)
	{
		xx_rpt(FAILURE);
		in_rpt("CLOCAL failed - SIGHUP received");
	}
	else
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* CLOCAL clear, open without O_NONBLOCK */
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (modem_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = pathok == 2 ? 1 : 0;

	(void) cppair(ch_t6, NULLFN, 3*WAITTIME + 2*NAP, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t6");

	PATH_FUNC_TRACE;
	
	globok = 0; 
	if (ch_prep() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	tty_termios_p->c_cflag &= ~CLOCAL;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("~CLOCAL not supported on VSX_TERMIOS_TTY");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_cflag &= ~CLOCAL;
	loop_termios_p->c_cflag |= HUPCL;
	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Don't close(tty_fildes) - need to maintain settings */
	if (close(loop_fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("close() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(WAITTIME/2);	/* wait for SIGHUP */
	(void) sleep(NAP);	/* let modem lines do their thing */

	(void) vsignal(SIGHUP, SIG_IGN); /* so it doesn't hit wait() */
	globok = 0;
        ret = cppair(gch_t69, pa_t6, 3*WAITTIME/2 + NAP, E_LOOPPREP|PATH_NOK);

	if (ret == E_LOOPPREP)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open/initialise failed on VSX_TERMIOS_LOOP");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;
		
	if (testfail == 0)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
pa_t6()
{
	unsigned int unused_time;
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("pa_t6");

	SET_TIMEOUT(WAITTIME)

	ret = open(termios_tty, O_RDWR);
	err = errno;
	unused_time = alarm(0);

	CLEAR_ALARM

	DBUG_PRINTF("pa_t6", "unused_time = %u", unused_time);

	if (ret < 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("open() on VSX_TERMIOS_TTY timed out");
		else
			in_rpt("open() failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* (time taken to open) < (time before modem connect) - leeway */
	if ((WAITTIME - unused_time) < WAITTIME/2 - 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
	    	in_rpt("open() did not wait for modem connection");
	}
	else
		PATH_TRACE;

	(void) close(ret);

	PATH_FUNC_RPT(2);  

	DBUG_VOID_RETURN;
}

private void
gch_t69()	/* shared by tests 6 and 9 */
{
	int	pathok = 0;
	DBUG_ENTER("gch_t69");

	(void) sleep(WAITTIME/2);

	DBUG_PRINTF("gch_t69", "before loop_prep()");

	/*
	 *	exit if GTI is unsupported, this should be caught by ch_prep()
	 *	so we should not get as  far as this
	 */
	if (loop_prep(&loop_fildes, loop_termios_p) != 0)
	{
		DBUG_EXIT(E_LOOPPREP);
	}
	else
		PATH_TRACE;

	if (pathok != 1)
		DBUG_EXIT(PATH_NOK);
	
	(void) sleep(WAITTIME/2 + NAP);
	
	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* CLOCAL clear, open with O_NONBLOCK */
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (modem_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = pathok == 2 ? 1 : 0;
        (void) cppair(ch_t7, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t7");

	PATH_FUNC_TRACE; 

	globok = 0; 
	if (ch_prep() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	tty_termios_p->c_cflag &= ~CLOCAL;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("~CLOCAL not supported on VSX_TERMIOS_TTY");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_cflag &= ~CLOCAL;
	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Don't close(tty_fildes) - need to maintain settings */
	if (close(loop_fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("close() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(WAITTIME/2);	/* wait for SIGHUP */
	(void) sleep(NAP);	/* let modem lines do their thing */

	SET_TIMEOUT(WAITTIME)

	ret = open(termios_tty, O_RDWR|O_NONBLOCK);
	err = errno;

	CLEAR_ALARM

	if (ret < 0)
	{
		xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("open(O_NONBLOCK) on VSX_TERMIOS_TTY timed out");
		else
			in_rpt("open(O_NONBLOCK) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
	}
	else
	{
		PATH_XS_RPT(5); 
		(void) close(ret);
	}

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* CLOCAL set, open without O_NONBLOCK */
	int	pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (modem_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = pathok == 2 ? 1 : 0;

        (void) cppair(ch_t8, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t8");

	globok = 0; 
	if (ch_prep() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	tty_termios_p->c_cflag |= CLOCAL;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("CLOCAL not supported on VSX_TERMIOS_TTY");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_cflag &= ~CLOCAL;
	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Don't close(tty_fildes) - need to maintain settings */
	if (close(loop_fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("close() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(WAITTIME/2);	/* wait for SIGHUP */
	(void) sleep(NAP);	/* let modem lines do their thing */

	SET_TIMEOUT(WAITTIME)

	ret = open(termios_tty, O_RDWR);
	err = errno;

	CLEAR_ALARM

	if (ret < 0)
	{
		xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("open() on VSX_TERMIOS_TTY timed out");
		else
			in_rpt("open() failed on VSX_TERMIOS_TTY - errno %d (%s)", err, errname(err));
	}
	else
	{
		PATH_XS_RPT(4); 
		(void) close(ret);
	}

	DBUG_VOID_RETURN;
}

private void
test9()
{
	/* CLOCAL clear, creat() waits for connection */
	int	pathok = 0;

	DBUG_ENTER("test9");

	testfail = 0;

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (modem_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = pathok == 2 ? 1 : 0;

	(void) cppair(ch_t9, NULLFN, 3*WAITTIME + 2*NAP, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t9");

	PATH_FUNC_TRACE;
	
	globok = 0; 
	if (ch_prep() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	tty_termios_p->c_cflag &= ~CLOCAL;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("~CLOCAL not supported on VSX_TERMIOS_TTY");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_cflag &= ~CLOCAL;
	loop_termios_p->c_cflag |= HUPCL;
	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Don't close(tty_fildes) - need to maintain settings */
	if (close(loop_fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("close() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(WAITTIME/2);	/* wait for SIGHUP */
	(void) sleep(NAP);	/* let modem lines do their thing */

	(void) vsignal(SIGHUP, SIG_IGN); /* so it doesn't hit wait() */
	globok = 0;
        ret = cppair(gch_t69, pa_t9, 3*WAITTIME/2 + NAP, E_LOOPPREP|PATH_NOK);

	if (ret == E_LOOPPREP)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open/initialise failed on VSX_TERMIOS_LOOP");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;
		
	if (testfail == 0)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
pa_t9()
{
	unsigned int unused_time;
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("pa_t9");

	SET_TIMEOUT(WAITTIME)

	ret = creat(termios_tty, (mode_t)0);
	err = errno;
	unused_time = alarm(0);

	CLEAR_ALARM

	DBUG_PRINTF("pa_t9", "unused_time = %u", unused_time);

	if (ret < 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("creat() on VSX_TERMIOS_TTY timed out");
		else
			in_rpt("creat() failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* (time taken to open) < (time before modem connect) - leeway */
	if ((WAITTIME - unused_time) < WAITTIME/2 - 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
	    	in_rpt("creat() did not wait for modem connection");
	}
	else
		PATH_TRACE;

	(void) close(ret);

	PATH_FUNC_RPT(2);  

	DBUG_VOID_RETURN;
}

private void
test10()
{
	/* CLOCAL set, open with O_NONBLOCK */
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (modem_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = pathok == 2 ? 1 : 0;

        (void) cppair(ch_t10, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t10");

	globok = 0; 
	if (ch_prep() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	tty_termios_p->c_cflag |= CLOCAL;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("CLOCAL not supported on VSX_TERMIOS_TTY");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_cflag &= ~CLOCAL;
	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Don't close(tty_fildes) - need to maintain settings */
	if (close(loop_fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("close() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(WAITTIME/2);	/* wait for SIGHUP */
	(void) sleep(NAP);	/* let modem lines do their thing */

	SET_TIMEOUT(WAITTIME)

	ret = open(termios_tty, O_RDWR|O_NONBLOCK);
	err = errno;

	CLEAR_ALARM

	if (ret < 0)
	{
		xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("open() on VSX_TERMIOS_TTY timed out");
		else
			in_rpt("open() failed on VSX_TERMIOS_TTY - errno %d (%s)", err, errname(err));
	}
	else
	{
		PATH_XS_RPT(4); 
		(void) close(ret);
	}

	DBUG_VOID_RETURN;
}

private void
test11()
{
	/* CLOCAL set, creat() does not wait */
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (modem_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_MODEM_CONTROL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = pathok == 2 ? 1 : 0;

        (void) cppair(ch_t11, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t11");

	globok = 0; 
	if (ch_prep() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	tty_termios_p->c_cflag |= CLOCAL;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (err == EINVAL)
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("CLOCAL not supported on VSX_TERMIOS_TTY");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_cflag &= ~CLOCAL;
	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Don't close(tty_fildes) - need to maintain settings */
	if (close(loop_fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("close() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(WAITTIME/2);	/* wait for SIGHUP */
	(void) sleep(NAP);	/* let modem lines do their thing */

	SET_TIMEOUT(WAITTIME)

	ret = creat(termios_tty, (mode_t)0);
	err = errno;

	CLEAR_ALARM

	if (ret < 0)
	{
		xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("creat() on VSX_TERMIOS_TTY timed out");
		else
			in_rpt("creat() failed on VSX_TERMIOS_TTY - errno %d (%s)", err, errname(err));
	}
	else
	{
		PATH_XS_RPT(4); 
		(void) close(ret);
	}

	DBUG_VOID_RETURN;
}
