/*
*      SCCS:  @(#)  POSIX.os/devclass/i_noncanon/i_noncanon.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/i_noncanon - T.i_noncanon Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.i_noncanon

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the terminal interface, part 7

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	6 Feb 1989
MODIFICATIONS:
		Geoff Clare, 15 Sep 1989
			Write c_cc[]'s as 1 char, not sizeof(cc_t)
		
		Geoff Clare, 26 Sep 1989
			Inter-byte timer test did not allow for
			child's sleep(NAP) before writing.
		
		Geoff Clare, 27 Oct 1989
			More leeway in test3 and test10.

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported
		
		Geoff Clare, 2 July 1990
			Add tests for TIME > 0 with data already in queue

		Anthony Lee, 08 August 1990
			Added resilience code.	

		Geoff Clare, 20 Dec 1991
			Changed VMIN to Strlen+1 in test2.

		Geoff Clare, 6 April 1992
			Change tests 14 and 15 to warnings in XPG3 mode.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif


#define NO_TESTS	15
#define WAITTIME	(5 * SPEEDFACTOR)
#define TEN_SECONDS	10
#define NAP		2
#define READ_MIN	10
#define READ_TIME	4*10
#define E_WRFAIL	02
#define Strlen		(sizeof(string)-1)
#define PATH_NOK	04

int	no_tests = NO_TESTS;

extern	char	*errname();

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	ch_t3();
private void	ch_t4();
private void	ch_t7();
private void	ch_t9();
private void	pa_t3();
private void	pa_t4();
private void	pa_t7();
private void	pa_t9();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();

public	void	(*realfuncs[]) () = { test1,  test2,  test3,  test4,  test5,
				      test6,  test7,  test8,  test9,  test10,
				      test11, test12, test13, test14,
				      test15, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	 twrap, 2,
	 twrap, 3,
	 twrap, 4,
	 twrap, 5,
	 twrap, 6,
	 twrap, 7,
	 twrap, 8,
	 twrap, 9,
	 twrap, 10,
	 twrap, 11,
	 twrap, 12,
	 twrap, 13,
	 twrap, 14,
	 twrap, 15,
	NULL, 0
};


private	int	unsupported = 0;
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	testfail;
private char	string[] = "Yes Sir, that's my baby. No Sir, I don't mean maybe.";
private char	inbuf[512];
private	unsigned char chr;

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}


private void
prepare_tests()
{
	char *msg = NULL;
	int i;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

        (void) close(tty_fildes);
        (void) close(loop_fildes);

	DBUG_VOID_RETURN;
}

private int
tprep()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("tprep");

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3); 
	DBUG_RETURN(0);
}

private void
test1()
{
	/*  chars not assembled into lines */
	int	pathok = 0;

	int ret, err;

	DBUG_ENTER("test1");

	testfail = 0;

	tty_termios_p->c_lflag &= ~(ICANON | ECHO);
	tty_termios_p->c_cc[VMIN] = Strlen;
	tty_termios_p->c_cc[VTIME] = 10;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	SET_TIMEOUT(TEN_SECONDS)

	ret = write(loop_fildes, string, Strlen);
	err = errno;
	
	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(TEN_SECONDS)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/*  no erase/kill processing */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	tty_termios_p->c_lflag &= ~(ICANON | ECHO);
	tty_termios_p->c_cc[VMIN] = Strlen+1;
	tty_termios_p->c_cc[VTIME] = 10;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
        /* erase processing */

	chr = tty_termios_p->c_cc[VERASE];

	SET_TIMEOUT(TEN_SECONDS)

	ret = write(loop_fildes, string, Strlen);
	if (ret == Strlen)
		ret += write(loop_fildes, (char *)&chr, 1);
	err = errno;
	    
	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(TEN_SECONDS)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d",
			ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if ((unsigned char)inbuf[Strlen] != chr)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
                in_rpt("ERASE character was not read as data");
	}
	else
		PATH_TRACE;

        /* kill processing */

	chr = tty_termios_p->c_cc[VKILL];

	SET_TIMEOUT(TEN_SECONDS)

	ret = write(loop_fildes, string, Strlen);
	if (ret == Strlen)
		ret += write(loop_fildes, (char *)&chr, 1);
	err = errno;
	    
	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

        inbuf[Strlen] = '\0';

	SET_TIMEOUT(TEN_SECONDS)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d",
			ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if ((unsigned char)inbuf[Strlen] != chr)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
                in_rpt("KILL character was not read as data");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* MIN > 0, TIME > 0, timer activated after 1st byte */

	int ret;
	int	pathok = 0;
	DBUG_ENTER("test3");

	testfail = 0;

	tty_termios_p->c_lflag &= ~(ICANON | ECHO);
	tty_termios_p->c_cc[VMIN] = READ_MIN;
	tty_termios_p->c_cc[VTIME] = (3 * READ_TIME);

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0; 
	ret = cppair(ch_t3, pa_t3, WAITTIME, E_WRFAIL|PATH_NOK);

	if (ret == E_WRFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;
		
	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int	pathok = 0;
	DBUG_ENTER("ch_t3");

	(void) sleep(NAP);

	if (write(loop_fildes, string, READ_MIN-1) != READ_MIN-1)
	{
		DBUG_EXIT(E_WRFAIL);
	}
	else
		PATH_TRACE;

	if (pathok != 1)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int ret, err;
	int used_time;
	int	pathok = 0;

	DBUG_ENTER("pa_t3");

	SET_TIMEOUT((3 * TEN_SECONDS) + NAP)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	used_time = (3 * TEN_SECONDS) - alarm(0);

	CLEAR_ALARM

	if (ret != READ_MIN-1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, READ_MIN-1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	/* allow for a bit of processing time on the upper bound */
	if (used_time > (3 * READ_TIME)/10 + 3)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt ("timer expired too late - expected %d secs, actual %d",
			(3 * READ_TIME)/10, used_time);

	}
	/* tighter margin on the lower bound */
	else if (used_time < (3 * READ_TIME)/10 - 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt ("timer expired too soon - expected %d secs, actual %d",
			(3 * READ_TIME)/10, used_time);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* MIN > 0, TIME > 0, timer reset after each byte */

	int ret;
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	tty_termios_p->c_lflag &= ~(ICANON | ECHO);
	tty_termios_p->c_cc[VMIN] = READ_MIN;
	tty_termios_p->c_cc[VTIME] = READ_TIME;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0; 
	ret = cppair(ch_t4, pa_t4, WAITTIME, E_WRFAIL|PATH_NOK);

	if (ret == E_WRFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;
		
	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int ret;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	/*
	 * write bytes to reset timer:
	 * 	total time > READ_TIME
	 * 	total bytes < READ_MIN
	 */

	ret = write(loop_fildes, string, 1);
	(void) sleep(NAP);
	ret += write(loop_fildes, string, 1);
	(void) sleep(NAP);
	ret += write(loop_fildes, string, 1);
	(void) sleep(NAP);
	ret += write(loop_fildes, string, 1);
	
	if (ret != 4)
	{
		DBUG_EXIT(E_WRFAIL);
	}
	else
		PATH_TRACE;
		
	if (pathok != 1)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
pa_t4()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("pa_t4");

	(void) sleep(NAP);

        /* read 4 chars and then READ_TIME out */

	SET_TIMEOUT(3*NAP + TEN_SECONDS)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 4)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 4", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* MIN > 0, TIME > 0, MIN bytes satisfy read */

	int ret, err;
	int	pathok = 0;
	
	DBUG_ENTER("test5");

	testfail = 0;

	tty_termios_p->c_lflag &= ~(ICANON | ECHO);
	tty_termios_p->c_cc[VMIN] = READ_MIN;
	tty_termios_p->c_cc[VTIME] = 2 * READ_TIME;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(TEN_SECONDS)

	ret = write(loop_fildes, string, READ_MIN);
	err = errno;
	
	CLEAR_ALARM

	if (ret != READ_MIN)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(READ_TIME/10)	/* N.B. TIME is (2 * READ_TIME) */

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read() did not return after READ_MIN bytes received");
	}
	else if (ret != READ_MIN)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, READ_MIN);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* MIN > 0, TIME > 0, when timer expires bytes
	   received to that point are returned */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	tty_termios_p->c_lflag &= ~(ICANON | ECHO);
	tty_termios_p->c_cc[VMIN] = READ_MIN;
	tty_termios_p->c_cc[VTIME] = READ_TIME;

	globok = 0;
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
			
	SET_TIMEOUT(TEN_SECONDS)

	ret = write(loop_fildes, string, READ_MIN-1);
	err = errno;
	
	CLEAR_ALARM

	if (ret != READ_MIN-1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(TEN_SECONDS)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (ret != READ_MIN-1)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, READ_MIN-1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* MIN > 0, TIME > 0, read blocks until 1st byte */

	int ret;
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	tty_termios_p->c_lflag &= ~(ICANON | ECHO);
	tty_termios_p->c_cc[VMIN] = READ_MIN;
	tty_termios_p->c_cc[VTIME] = READ_TIME;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* zero chars written, read hangs until signal */

	SET_TIMEOUT(TEN_SECONDS)

	(void) read(tty_fildes, inbuf, sizeof(inbuf));
	
	CLEAR_ALARM

	if (alrm_flag == 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() did not block when no data available");
	}
	else
		PATH_TRACE;

	globok = 0; 
	ret = cppair(ch_t7, pa_t7, WAITTIME, E_WRFAIL|PATH_NOK);

	if (ret == E_WRFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;
		
	if (testfail == 0)
		PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int	pathok = 0;
	DBUG_ENTER("ch_t7");

	/* wait longer than timer length before writing */

	(void) sleep(2*READ_TIME/10);

	if (write(loop_fildes, string, READ_MIN) != READ_MIN)
	{
		DBUG_EXIT(E_WRFAIL);
	}
	else
		PATH_TRACE;

	if (pathok != 1)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
pa_t7()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("pa_t7");

	SET_TIMEOUT(2*READ_TIME/10 + TEN_SECONDS)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != READ_MIN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, READ_MIN);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* MIN > 0, TIME = 0 */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

        tty_termios_p->c_lflag &= ~ICANON;
        tty_termios_p->c_cc[VMIN] = READ_MIN;
        tty_termios_p->c_cc[VTIME] = 0;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Characters received < MIN, read() ended by SIGALRM */

	SET_TIMEOUT(TEN_SECONDS)

	ret = write(loop_fildes, string, READ_MIN-1);
	err = errno;
	
	CLEAR_ALARM

	if (ret != READ_MIN-1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(TEN_SECONDS)

	(void) read(tty_fildes, inbuf, sizeof(inbuf));
	
	CLEAR_ALARM

	if (alrm_flag == 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() did not block when less than MIN bytes available");
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Characters received == MIN */

	SET_TIMEOUT(TEN_SECONDS)

	ret = write(loop_fildes, string, READ_MIN);
	err = errno;
	
	CLEAR_ALARM

	if (ret != READ_MIN)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(TEN_SECONDS)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (ret != READ_MIN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, READ_MIN);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
test9()
{
	/* MIN == 0, TIME > 0, data arrives before timer expires */

	int ret;
	int	pathok = 0;

	DBUG_ENTER("test9");

	testfail = 0;

        tty_termios_p->c_lflag &= ~ICANON;
        tty_termios_p->c_cc[VMIN] = 0;
        tty_termios_p->c_cc[VTIME] = READ_TIME;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	globok = 0; 
	ret = cppair(ch_t9, pa_t9, WAITTIME, E_WRFAIL|PATH_NOK);

	if (ret == E_WRFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed");
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_FUNC_TRACE;
		
	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int	pathok = 0;
	DBUG_ENTER("ch_t9");

	(void) sleep(NAP);

	if (write(loop_fildes, string, 1) != 1)
	{
		DBUG_EXIT(E_WRFAIL);
	}
	else
		PATH_TRACE;

	if (pathok != 1)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
pa_t9()
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("pa_t9");

	SET_TIMEOUT(TEN_SECONDS)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1); 
	DBUG_VOID_RETURN;
}

private void
test10()
{
	/* MIN == 0, TIME > 0, zero bytes returned when timer expires */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

        tty_termios_p->c_lflag &= ~ICANON;
        tty_termios_p->c_cc[VMIN] = 0;
        tty_termios_p->c_cc[VTIME] = (3 * READ_TIME);

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* 2 seconds leeway isn't quite enough */
	SET_TIMEOUT((3 * READ_TIME)/10 + 3)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() did not return after TIME*0.1 seconds");
	}
	else if (ret != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 0", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test11()
{
	/* MIN == 0, TIME == 0, enough data */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

        tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 0;
	tty_termios_p->c_cc[VTIME] = 0;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(TEN_SECONDS)

	ret = write(loop_fildes, string, Strlen);
	err = errno;
	
	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(TEN_SECONDS)

	ret = read(tty_fildes, inbuf, Strlen);
	err = errno;
	
	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test12()
{
	/* MIN == 0, TIME == 0, not enough data */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test12");

	testfail = 0;

        tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 0;
	tty_termios_p->c_cc[VTIME] = 0;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	SET_TIMEOUT(TEN_SECONDS)

	ret = write(loop_fildes, string, Strlen);
	err = errno;
	
	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(TEN_SECONDS)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test13()
{
	/* MIN == 0, TIME == 0, no data */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

        tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 0;
	tty_termios_p->c_cc[VTIME] = 0;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	SET_TIMEOUT(TEN_SECONDS)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (ret != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 0", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test14()
{
	/* MIN > 0, TIME > 0, data already in queue */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test14");

	testfail = 0;

	tty_termios_p->c_lflag &= ~(ICANON | ECHO);
	tty_termios_p->c_cc[VMIN] = READ_MIN;
	tty_termios_p->c_cc[VTIME] = 2 * READ_TIME;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(TEN_SECONDS)

	ret = write(loop_fildes, string, READ_MIN-1);
	err = errno;
	
	CLEAR_ALARM

	if (ret != READ_MIN-1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(2*READ_TIME/10 + WAITTIME/2);

	SET_TIMEOUT(TEN_SECONDS)

	ret = write(loop_fildes, string, 1);
	err = errno;
	
	CLEAR_ALARM

	if (ret != 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(READ_TIME/10)	/* N.B. TIME is (2 * READ_TIME) */

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (alrm_flag > 0)
	{
#if TEST_XPG == 3
		xx_rpt(WARNING);
#else
		xx_rpt(FAILURE);
#endif
		in_rpt("read() did not return after READ_MIN bytes received");
	}
	else if (ret != READ_MIN)
	{
#if TEST_XPG == 3
		xx_rpt(WARNING);
#else
		xx_rpt(FAILURE);
#endif
		in_rpt("read() returned %d, expected %d", ret, READ_MIN);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
test15()
{
	/* MIN == 0, TIME > 0, data already in queue */

	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("test15");

	testfail = 0;

        tty_termios_p->c_lflag &= ~ICANON;
        tty_termios_p->c_cc[VMIN] = 0;
        tty_termios_p->c_cc[VTIME] = READ_TIME;

	globok = 0; 
	if (tprep() < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		
	SET_TIMEOUT(TEN_SECONDS)

	ret = write(loop_fildes, string, 1);
	err = errno;
	
	CLEAR_ALARM

	if (ret != 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(2*READ_TIME/10 + WAITTIME/2);

	SET_TIMEOUT(TEN_SECONDS)

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != 1)
	{
		if (testfail++ == 0)
#if TEST_XPG == 3
			xx_rpt(WARNING);
#else
			xx_rpt(FAILURE);
#endif
		in_rpt("read() returned %d, expected 1", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);
	DBUG_VOID_RETURN;
}
