/*
*      SCCS:  @(#)  POSIX.os/devclass/i_spchars/i_spchars.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/i_spchars - T.i_spchars Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.i_spchars

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the terminal interface, part 9

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	14 Feb 1989
MODIFICATIONS:
		Geoff Clare, 13 Sep 1989
			Write cc_t's by assigning to char first
			New _POSIX_VDISABLE strategy
		
		Geoff Clare, 19 Mar 1990
			Do "ISIG set - INTR/QUIT/SUSP character discarded"
			tests with NOFLSH set.

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported
		
		Geoff Clare, 29 June 1990
			Add IEXTEN clear test

		David Sawyer
		Thu Aug 09 11:10:14 BST 1990
		Added resilience code.

		Stuart Boutell, 7 Nov 1990
			Added sig_func & setjmp code.

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called

		Geoff Clare, 23 May 1991
			Only clear ISTRIP if CSIZE is CS8.

		Stuart Boutell, 17 Apr 1991
			Added tests for pathconf(_PC_VDISABLE) as copies
			of tests for fpathconf(_PC_VDISABLE).
		
		Geoff Clare, 20 June 1991
			Change ISIG set, NOFLSH clear tests to give
			WARNING if queues not discarded.

		Geoff Clare, 26 June 1991
			VSX_START_STOP_CHNG="N" for unsupported result.

		Geoff Clare, 5 Aug 1991
			Add testwarn variable to prevent path trace error
			on warning.

		Geoff Clare, 16 Sep 1991
			Check posix_vdisable/posix_pdisable flag in
			START/STOP disable tests.

		Geoff Clare, 9 June 1992
			Allow for systems with no terminal output queue.

		Geoff Clare, 23 June 1994
			Ensure sigsetjmp() is used in line with constraints.

		Geoff Clare, 7 Sept 1995
			Check VSX_TERMIOS_BUFFERED in test 17.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <sys/wait.h>
#include <termios.h>
#include <string.h>
#include <timeout.h>
#include <tet_api.h>
#include <setjmp.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef UNDEF_MACROS
#endif

#ifndef _XOPEN_SOURCE
#undef memchr
extern	void	*memchr();
#endif

#define NO_TESTS	24
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		2
#define CHILDREN	4
#define STOP		'\023'	/* ASCII DC3 */
#define START		'\021'	/* ASCII DC1 */
#define Strlen		(sizeof(string)-1)
#define E_SAFAIL	002
#define E_NOSIG		004
#define YES		1
#define NO		0

extern	void	setsigs();
extern	char	*errname();
extern	char	*signame();
private	sigjmp_buf	sjmp;
private int		jmpsignum;

int	no_tests = NO_TESTS;

private	void	sig_func();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private	void	test22();
private	void	test23();
private	void	test24();
private void	prepare_tests();
private void	clean_tests();

private int	chk_sig();
private int	chk_erase();
private int	chk_kill();
private int	chk_eof();
private int	chk_eol();
private int	chk_xon();
private int	do_buf();
private void	twrap();

public	void	(*realfuncs[]) () = { test1,  test2,  test3,  test4,  test5,
				      test6,  test7,  test8,  test9,  test10,
				      test11, test12, test13, test14, test15,
				      test16, test17, test18, test19, test20,
				      test21, test22, test23, test24, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	 twrap, 2,
	 twrap, 3,
	 twrap, 4,
	 twrap, 5,
	 twrap, 6,
	 twrap, 7,
	 twrap, 8,
	 twrap, 9,
	 twrap, 10,
	 twrap, 11,
	 twrap, 12,
	 twrap, 13,
	 twrap, 14,
	 twrap, 15,
	 twrap, 16,
	 twrap, 17,
	 twrap, 18,
	 twrap, 18, /* test19 depends on 18 */
	 twrap, 18, /* test20 depends on 18 */
	 twrap, 18, /* test21 depends on 18 */
	 twrap, 22,
	 twrap, 22, /* test23 depends on 22 */
	 twrap, 22, /* test24 depends on 22 */
	NULL, 0
};


private	int	unsupported = 0;
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	testfail;
private int	testwarn;
private int	caught_sig = 0;
private char	string[] = "yes indeed";
private char	out_buf[256];
private char	in_buf[512];
private char	*testcase = "";
private int	numchars = 0;
private char	NL[] = "\n";
private cc_t	old_cc;
private unsigned char	new_cc = 'A';
private unsigned char	chr;
private long		posix_vdisable; /* from fpathconf() */
private long		posix_pdisable; /* from pathconf() */

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	char *msg = NULL;
	int i;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}

	tty_termios_p->c_iflag |= (IXON);
	tty_termios_p->c_lflag |= (ISIG|ICANON|TOSTOP);
	tty_termios_p->c_lflag &= ~NOFLSH;

	loop_termios_p->c_cc[VMIN] = 0;
	loop_termios_p->c_cc[VTIME] = 1;

	SET_TIMEOUT(WAITTIME)

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
		msg = "tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY during preparation";
	else if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		msg = "tcsetattr(TCSANOW) failed on VSX_TERMIOS_LOOP during preparation";

	CLEAR_ALARM

	if (msg != NULL)
	{
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, msg);
	}
	else
	{
		char	*termios_tty;

		posix_vdisable = fpathconf(tty_fildes, _PC_VDISABLE);
		termios_tty = tet_getvar("VSX_TERMIOS_TTY");
		/* no need to check: termios_prep() succeeded */
		posix_pdisable = pathconf(termios_tty , _PC_VDISABLE);
	}

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

        (void) close(tty_fildes);
        (void) close(loop_fildes);
	(void) unlink("core");

	DBUG_VOID_RETURN;
}

private int
flushem()
{
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("flushem");

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush(TCIOFLUSH) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(0);
}

private int
tsetup()
{
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("tsetup");
 
        if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcsetattr() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_RETURN(-1);
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(0);
}

private void
sig_catch(sig)
{
	caught_sig = sig;
}

private void
test1()
{
	/* ISIG set - INTR char sends SIGINT to dist. process group,
	   		input and output queues are discarded */
	
	char	*cp;
	int	buffered;
	int i, ret, ret2, err, wait_stat;
	pid_t pid;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;
	testwarn = 0;

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = SIG_DFL;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting default action for SIGINT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < CHILDREN; ++i)
	{
		switch (fork())
		{
		case -1:
			err = errno;
			xx_rpt(DELETION);
			in_rpt("fork() failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		
		case 0:
			tet_setcontext();
			setsigs(SIG_DFL);

			(void) sleep(WAITTIME);	/* pause for signal */

			exit(0);
                }
		tet_setblock();
		PATH_TRACE;
	}

        (void) sleep(NAP);

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;

		/* create an output queue */

		if (buffered)
		{
			SET_TIMEOUT(WAITTIME)

			ret = tcflow(tty_fildes, TCOOFF);
			if (ret != SYSERROR)
				ret2 = write(tty_fildes, string, Strlen);
			err = errno;

			CLEAR_ALARM

			if (ret == SYSERROR)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY - errno %d (%s)",
					err, errname(err));
			}
			else if (ret2 != Strlen)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("write() to VSX_TERMIOS_TTY failed after tcflow(TCOOFF)");
			}
			else
				PATH_TRACE;
		}
		else
			PATH_TRACE;

		(void) sleep(NAP);

		caught_sig = 0;

		/* write an input queue and the interrupt char */

		chr = tty_termios_p->c_cc[VINTR];

		SET_TIMEOUT(WAITTIME)

		ret = write(loop_fildes, string, Strlen);
		if (alrm_flag == 0)
			ret += write(loop_fildes, NL, 1);
		if (alrm_flag == 0)
			ret += write(loop_fildes, (char *)&chr, 1);

		if (alrm_flag == 0)
			(void) pause();

		CLEAR_ALARM

		if (ret != Strlen + 2)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
		}
		else if (caught_sig != SIGINT)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("SIGINT was not received by parent");
		}
		else
			PATH_TRACE;

	}

	/* data in queues should have been discarded */
	/* POSIX is not clear on this, so give WARNING only */

	SET_TIMEOUT(WAITTIME)

	ret = tcflow(tty_fildes, TCOON);
	if (ret != SYSERROR)
		ret2 = read(loop_fildes, in_buf, sizeof(in_buf));
	err = errno;
	
	CLEAR_ALARM

	if (ret == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOON) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
	}
	else if (ret2 > 0)
	{
		/* don't increment testfail: later FAILURE must supersede */
		if (testfail == 0)
			xx_rpt(WARNING);
		in_rpt("output queue was not discarded");
		testwarn++;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, in_buf, sizeof(in_buf));
	
	CLEAR_ALARM

	if (ret > 0)
	{
		/* don't increment testfail: later FAILURE must supersede */
		if (testfail == 0)
			xx_rpt(WARNING);
		in_rpt("input queue was not discarded");
		testwarn++;
	}
	else
		PATH_TRACE;

	/* check children received SIGINT */

	for (i = 0; i < CHILDREN; ++i)
	{
		if ((pid = wait(&wait_stat)) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("wait() failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		if (WIFSIGNALED(wait_stat) && WTERMSIG(wait_stat) == SIGINT)
		{
			PATH_TRACE;
			continue;
		}
		if (testfail++ == 0)
			xx_rpt(FAILURE);
                in_rpt("child %ld did not terminate with SIGINT (exit status 0x%x)",
                        (long)pid, wait_stat);
	}

	if (testfail == 0 && testwarn == 0)
		PATH_XS_RPT(8+(3*CHILDREN));

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* ISIG set - INTR char discarded */

	int 	ret, err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	tty_termios_p->c_lflag |= NOFLSH;

	globok = 0;
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting SIGINT to be ignored - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        chr = tty_termios_p->c_cc[VINTR];

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, (char *)&chr, 1);
	if (alrm_flag == 0)
		ret += write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);

	CLEAR_ALARM

	if (ret != Strlen + 2)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        (void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

        ret = read(tty_fildes, in_buf, sizeof(in_buf));
	err = errno;

	CLEAR_ALARM

	DBUG_PRINTF("test2", "read string: \"%.*s\"", ret, in_buf);

	if (ret >= 1 && memchr((void *)in_buf, (int)chr, (size_t)ret) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("INTR character was not discarded");
	}
	else if (ret != Strlen + 1)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(in_buf, string, (size_t)Strlen) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read string != written string");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* ISIG set - QUIT char sends SIGQUIT to dist. process group,
	   		input and output queues are discarded */
	
	char	*cp;
	int	buffered;
	int 	i, ret, ret2, err, wait_stat;
	pid_t 	pid;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;
	testwarn = 0;

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	tty_termios_p->c_lflag &= ~NOFLSH;

	globok = 0;
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = SIG_DFL;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGQUIT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting default action for SIGQUIT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < CHILDREN; ++i)
	{
		switch (fork())
		{
		case -1:
			err = errno;
			xx_rpt(DELETION);
			in_rpt("fork() failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		
		case 0:
			tet_setcontext();
			setsigs(SIG_DFL);

			(void) sleep(WAITTIME);	/* pause for signal */

			exit(0);
                }
		tet_setblock();
		PATH_TRACE;
	}

        (void) sleep(NAP);

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGQUIT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGQUIT - errno %d (%s)",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;

		/* create an output queue */

		if (buffered)
		{
			SET_TIMEOUT(WAITTIME)

			ret = tcflow(tty_fildes, TCOOFF);
			if (ret != SYSERROR)
				ret2 = write(tty_fildes, string, Strlen);
			err = errno;

			CLEAR_ALARM

			if (ret == SYSERROR)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY - errno %d (%s)",
					err, errname(err));
			}
			else if (ret2 != Strlen)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("write() to VSX_TERMIOS_TTY failed after tcflow(TCOOFF)");
			}
			else
				PATH_TRACE;
		}
		else
			PATH_TRACE;

		(void) sleep(NAP);

		caught_sig = 0;

		/* write an input queue and the interrupt char */

		chr = tty_termios_p->c_cc[VQUIT];

		SET_TIMEOUT(WAITTIME)

		ret = write(loop_fildes, string, Strlen);
		if (alrm_flag == 0)
			ret += write(loop_fildes, NL, 1);
		if (alrm_flag == 0)
			ret += write(loop_fildes, (char *)&chr, 1);

		if (alrm_flag == 0)
			(void) pause();

		CLEAR_ALARM

		if (ret != Strlen + 2)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
		}
		else if (caught_sig != SIGQUIT)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("SIGQUIT was not received by parent");
		}
		else
			PATH_TRACE;

	}

	/* data in queues should have been discarded */
	/* POSIX is not clear on this, so give WARNING only */

	SET_TIMEOUT(WAITTIME)

	ret = tcflow(tty_fildes, TCOON);
	if (ret != SYSERROR)
		ret2 = read(loop_fildes, in_buf, sizeof(in_buf));
	err = errno;
	
	CLEAR_ALARM

	if (ret == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOON) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
	}
	else if (ret2 > 0)
	{
		/* don't increment testfail: later FAILURE must supersede */
		if (testfail == 0)
			xx_rpt(WARNING);
		in_rpt("output queue was not discarded");
		testwarn++;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, in_buf, sizeof(in_buf));
	
	CLEAR_ALARM

	if (ret > 0)
	{
		/* don't increment testfail: later FAILURE must supersede */
		if (testfail == 0)
			xx_rpt(WARNING);
		in_rpt("input queue was not discarded");
		testwarn++;
	}
	else
		PATH_TRACE;

	/* check children received SIGQUIT */

	for (i = 0; i < CHILDREN; ++i)
	{
		if ((pid = wait(&wait_stat)) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("wait() failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		if (WIFSIGNALED(wait_stat) && WTERMSIG(wait_stat) == SIGQUIT)
		{
			PATH_TRACE;
			continue;
		}
		if (testfail++ == 0)
			xx_rpt(FAILURE);
                in_rpt("child %ld did not terminate with SIGQUIT (exit status 0x%x)",
                        (long)pid, wait_stat);
	}

	if (testfail == 0 && testwarn == 0)
		PATH_XS_RPT(8+(3*CHILDREN));

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* ISIG set - QUIT char discarded */

	int 	ret, err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	tty_termios_p->c_lflag |= NOFLSH;

	globok = 0;
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGQUIT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting SIGQUIT to be ignored - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        chr = tty_termios_p->c_cc[VQUIT];

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, (char *)&chr, 1);
	if (alrm_flag == 0)
		ret += write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);

	CLEAR_ALARM

	if (ret != Strlen + 2)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        (void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

        ret = read(tty_fildes, in_buf, sizeof(in_buf));
	err = errno;

	CLEAR_ALARM

	if (ret >= 1 && memchr((void *)in_buf, (int)chr, (size_t)ret) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("QUIT character was not discarded");
	}
	else if (ret != Strlen + 1)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(in_buf, string, (size_t)Strlen) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read string != written string");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* ISIG set - SUSP char sends SIGTSTP to dist. process group,
	   		input and output queues are discarded */
	
	char	*cp;
	int	buffered;
	int 	i, ret, ret2, err, wait_stat;
	pid_t 	pid;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test5");

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testwarn = 0;

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	tty_termios_p->c_lflag &= ~NOFLSH;

	globok = 0;
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	for (i = 0; i < CHILDREN; ++i)
	{
		switch (fork())
		{
		case -1:
			err = errno;
			xx_rpt(DELETION);
			in_rpt("fork() failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		
		case 0:
			tet_setcontext();
			setsigs(SIG_DFL);

			sig.sa_handler = sig_catch;
			sig.sa_flags = 0;
			(void) sigemptyset(&sig.sa_mask);
			if (sigaction(SIGTSTP, &sig, NULLSA) == SYSERROR)
				exit(E_SAFAIL);

			(void) sleep(WAITTIME);	/* pause for signal */

			if (caught_sig != SIGTSTP)
				exit(E_NOSIG);

			exit(0);
                }
		tet_setblock();
		PATH_TRACE;
	}

        (void) sleep(NAP);

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTSTP, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGTSTP - errno %d (%s)",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;

		/* create an output queue */

		if (buffered)
		{
			SET_TIMEOUT(WAITTIME)

			ret = tcflow(tty_fildes, TCOOFF);
			if (ret != SYSERROR)
				ret2 = write(tty_fildes, string, Strlen);
			err = errno;

			CLEAR_ALARM

			if (ret == SYSERROR)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY - errno %d (%s)",
					err, errname(err));
			}
			else if (ret2 != Strlen)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("write() to VSX_TERMIOS_TTY failed after tcflow(TCOOFF)");
			}
			else
				PATH_TRACE;
		}
		else
			PATH_TRACE;

		(void) sleep(NAP);

		caught_sig = 0;

		/* write an input queue and the suspend char */

		chr = tty_termios_p->c_cc[VSUSP];

		SET_TIMEOUT(WAITTIME)

		ret = write(loop_fildes, string, Strlen);
		if (alrm_flag == 0)
			ret += write(loop_fildes, NL, 1);
		if (alrm_flag == 0)
			ret += write(loop_fildes, (char *)&chr, 1);

		if (alrm_flag == 0)
			(void) pause();

		CLEAR_ALARM

		if (ret != Strlen + 2)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
		}
		else if (caught_sig != SIGTSTP)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("SIGTSTP was not received by parent");
		}
		else
			PATH_TRACE;

	}

	/* data in queues should have been discarded */
	/* POSIX is not clear on this, so give WARNING only */

	SET_TIMEOUT(WAITTIME)

	ret = tcflow(tty_fildes, TCOON);
	if (ret != SYSERROR)
		ret2 = read(loop_fildes, in_buf, sizeof(in_buf));
	err = errno;
	
	CLEAR_ALARM

	if (ret == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOON) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
	}
	else if (ret2 > 0)
	{
		/* don't increment testfail: later FAILURE must supersede */
		if (testfail == 0)
			xx_rpt(WARNING);
		in_rpt("output queue was not discarded");
		testwarn++;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, in_buf, sizeof(in_buf));
	
	CLEAR_ALARM

	if (ret > 0)
	{
		/* don't increment testfail: later FAILURE must supersede */
		if (testfail == 0)
			xx_rpt(WARNING);
		in_rpt("input queue was not discarded");
		testwarn++;
	}
	else
		PATH_TRACE;

	/* check children received SIGTSTP */

	for (i = 0; i < CHILDREN; ++i)
	{
		if ((pid = wait(&wait_stat)) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("wait() failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		if (WIFEXITED(wait_stat) && WEXITSTATUS(wait_stat) == 0)
		{
			PATH_TRACE;
			continue;
		}
		else if (WIFEXITED(wait_stat) &&
				WEXITSTATUS(wait_stat) == E_SAFAIL)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("sigaction() failed in child when setting catching function for SIGTSTP");
		}
		else if (WIFEXITED(wait_stat) &&
				WEXITSTATUS(wait_stat) == E_NOSIG)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("child %ld did not receive SIGTSTP", (long)pid);
		}
		else
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("child %ld gave bad exit status: 0x%x",
                        	(long)pid, wait_stat);
		}
	}

	if (testfail == 0 && testwarn == 0)
		PATH_XS_RPT(8+(3*CHILDREN));

	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* ISIG set - SUSP char discarded */

	int 	ret, err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test6");

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	tty_termios_p->c_lflag |= NOFLSH;

	globok = 0;
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTSTP, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting SIGTSTP to be ignored - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        chr = tty_termios_p->c_cc[VSUSP];

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, (char *)&chr, 1);
	if (alrm_flag == 0)
		ret += write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);

	CLEAR_ALARM

	if (ret != Strlen + 2)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        (void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

        ret = read(tty_fildes, in_buf, sizeof(in_buf));
	err = errno;

	CLEAR_ALARM

	if (ret >= 1 && memchr((void *)in_buf, (int)chr, (size_t)ret) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("SUSP character was not discarded");
	}
	else if (ret != Strlen + 1)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(in_buf, string, (size_t)Strlen) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read string != written string");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* IXON set - STOP and START chars discarded */

	int 	ret, err;
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

        chr = tty_termios_p->c_cc[VSTOP];
	ret = write(loop_fildes, (char *)&chr, 1);
	if (alrm_flag == 0)
	{
        	chr = tty_termios_p->c_cc[VSTART];
		ret += write(loop_fildes, (char *)&chr, 1);
	}
	if (alrm_flag == 0)
		ret += write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);

	CLEAR_ALARM

	if (ret != Strlen + 3)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        (void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

        ret = read(tty_fildes, in_buf, sizeof(in_buf));
	err = errno;

	CLEAR_ALARM

        chr = tty_termios_p->c_cc[VSTOP];
	if (ret >= 1 && memchr((void *)in_buf, (int)chr, (size_t)ret) != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("STOP character was not discarded");
	}
	else
		PATH_TRACE;

        chr = tty_termios_p->c_cc[VSTART];
	if (ret >= 1 && memchr((void *)in_buf, (int)chr, (size_t)ret) != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("START character was not discarded");
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_VOID_RETURN;

	if (ret != Strlen + 1)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(in_buf, string, (size_t)Strlen) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read string != written string");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* ICANON set - ERASE deletes chars back to NL, EOF or EOL */

	int 	i, ret, ret2, ret3, err, err2, err3;
	int	pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* test one erase char */

	chr = tty_termios_p->c_cc[VERASE];

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, (char *)&chr, 1);
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);

	if (ret == Strlen + 2)
		ret2 = read(tty_fildes, in_buf, sizeof(in_buf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen + 2)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ret2 != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing 1 ERASE character");
		in_rpt("read() returned %d, expected %d", ret2, Strlen);
		if (ret2 < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(in_buf, string, (size_t)(Strlen-1)) != 0 ||
		in_buf[Strlen-1] != '\n')
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing 1 ERASE character");
		in_rpt("read string was incorrect");
	}
	else
		PATH_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* more erase chars than chars after NL */

	chr = tty_termios_p->c_cc[VERASE];

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);
	if (alrm_flag == 0)
		ret += write(loop_fildes, string, Strlen);
	for (i = 0; i < Strlen * 2; ++i)
	{
		if (i==1)
			PATH_TRACE;
		if (alrm_flag > 0)
			break;
		ret += write(loop_fildes, (char *)&chr, 1);
	}
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);
	err = errno;

	if (ret == 4 * Strlen + 2)
	{
		ret2 = read(tty_fildes, in_buf, sizeof(in_buf));
		err2 = errno;
		if (ret2 != SYSERROR)
		{
			ret3 = read(tty_fildes, in_buf, sizeof(in_buf));
			err3 = errno;
		}
	}

	CLEAR_ALARM

	if (ret != 4 * Strlen + 2)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ret2 != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing attempt to ERASE beyond last newline");
		in_rpt("first read() returned %d, expected %d", ret2, Strlen + 1);
		if (ret2 < 0)
			in_rpt("errno was set to %d (%s)", err2, errname(err2));
	}
	else
		PATH_TRACE;

	if (ret2 != SYSERROR && ret3 != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing attempt to ERASE beyond last newline");
		in_rpt("second read() returned %d, expected %d", ret3, 1);
		if (ret3 < 0)
			in_rpt("errno was set to %d (%s)", err3, errname(err3));
	}
	else
		PATH_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* more erase chars than chars after EOF */

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
	{
		chr = tty_termios_p->c_cc[VEOF];
		ret += write(loop_fildes, (char *)&chr, 1);
	}
	if (alrm_flag == 0)
		ret += write(loop_fildes, string, Strlen);
	chr = tty_termios_p->c_cc[VERASE];
	for (i = 0; i < Strlen * 2; ++i)
	{
		if (i==1)
			PATH_TRACE;
		if (alrm_flag > 0)
			break;
		ret += write(loop_fildes, (char *)&chr, 1);
	}
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);
	err = errno;

	if (ret == (2 * Strlen + 1) * 2)
	{
		ret2 = read(tty_fildes, in_buf, sizeof(in_buf));
		err2 = errno;
		if (ret2 != SYSERROR)
		{
			ret3 = read(tty_fildes, in_buf, sizeof(in_buf));
			err3 = errno;
		}
	}

	CLEAR_ALARM

	if (ret != (2 * Strlen + 1) * 2)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ret2 != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing attempt to ERASE beyond EOF");
		in_rpt("first read() returned %d, expected %d", ret2, Strlen);
		if (ret2 < 0)
			in_rpt("errno was set to %d (%s)", err2, errname(err2));
	}
	else
		PATH_TRACE;

	if (ret2 != SYSERROR && ret3 != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing attempt to ERASE beyond EOF");
		in_rpt("second read() returned %d, expected %d", ret3, 1);
		if (ret3 < 0)
			in_rpt("errno was set to %d (%s)", err3, errname(err3));
	}
	else
		PATH_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* more erase chars than chars after EOL */

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
	{
		chr = tty_termios_p->c_cc[VEOL];
		ret += write(loop_fildes, (char *)&chr, 1);
	}
	if (alrm_flag == 0)
		ret += write(loop_fildes, string, Strlen);
	chr = tty_termios_p->c_cc[VERASE];
	for (i = 0; i < Strlen * 2; ++i)
	{
		if (i==1)
			PATH_TRACE;
		if (alrm_flag > 0)
			break;
		ret += write(loop_fildes, (char *)&chr, 1);
	}
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);
	err = errno;

	if (ret == (2 * Strlen + 1) * 2)
	{
		ret2 = read(tty_fildes, in_buf, sizeof(in_buf));
		err2 = errno;
		if (ret2 != SYSERROR)
		{
			ret3 = read(tty_fildes, in_buf, sizeof(in_buf));
			err3 = errno;
		}
	}

	CLEAR_ALARM

	if (ret != (2 * Strlen + 1) * 2)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ret2 != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing attempt to ERASE beyond EOL");
		in_rpt("first read() returned %d, expected %d",
			ret2, Strlen + 1);
		if (ret2 < 0)
			in_rpt("errno was set to %d (%s)", err2, errname(err2));
	}
	else
		PATH_TRACE;

	if (ret2 != SYSERROR && ret3 != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing attempt to ERASE beyond EOL");
		in_rpt("second read() returned %d, expected %d", ret3, 1);
		if (ret3 < 0)
			in_rpt("errno was set to %d (%s)", err3, errname(err3));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(18);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	/* ICANON set - KILL deletes all chars back to NL, EOF or EOL */

	int 	ret, ret2, ret3, err2, err3;
	int	pathok = 0;

	DBUG_ENTER("test9");

	testfail = 0;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* KILL deletes chars back to NL */

	chr = tty_termios_p->c_cc[VKILL];

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);
	if (alrm_flag == 0)
		ret += write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, (char *)&chr, 1);
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);

	if (ret == 2 * (Strlen + 1) + 1)
	{
		ret2 = read(tty_fildes, in_buf, sizeof(in_buf));
		err2 = errno;
		if (ret2 != SYSERROR)
		{
			ret3 = read(tty_fildes, in_buf, sizeof(in_buf));
			err3 = errno;
		}
	}

	CLEAR_ALARM

	if (ret != 2 * (Strlen + 1) + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ret2 != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing KILL deletes back to newline");
		in_rpt("first read() returned %d, expected %d", ret2, Strlen + 1);
		if (ret2 < 0)
			in_rpt("errno was set to %d (%s)", err2, errname(err2));
	}
	else
		PATH_TRACE;

	if (ret2 != SYSERROR && ret3 != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing KILL deletes back to newline");
		in_rpt("second read() returned %d, expected %d", ret3, 1);
		if (ret3 < 0)
			in_rpt("errno was set to %d (%s)", err3, errname(err3));
	}
	else
		PATH_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* KILL deletes chars back to EOF */

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
	{
		chr = tty_termios_p->c_cc[VEOF];
		ret += write(loop_fildes, (char *)&chr, 1);
	}
	if (alrm_flag == 0)
		ret += write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
	{
		chr = tty_termios_p->c_cc[VKILL];
		ret += write(loop_fildes, (char *)&chr, 1);
	}
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);

	if (ret == 2 * (Strlen + 1) + 1)
	{
		ret2 = read(tty_fildes, in_buf, sizeof(in_buf));
		err2 = errno;
		if (ret2 != SYSERROR)
		{
			ret3 = read(tty_fildes, in_buf, sizeof(in_buf));
			err3 = errno;
		}
	}

	CLEAR_ALARM

	if (ret != 2 * (Strlen + 1) + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ret2 != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing KILL deletes back to EOF");
		in_rpt("first read() returned %d, expected %d", ret2, Strlen);
		if (ret2 < 0)
			in_rpt("errno was set to %d (%s)", err2, errname(err2));
	}
	else
		PATH_TRACE;

	if (ret2 != SYSERROR && ret3 != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing KILL deletes back to EOF");
		in_rpt("second read() returned %d, expected %d", ret3, 1);
		if (ret3 < 0)
			in_rpt("errno was set to %d (%s)", err3, errname(err3));
	}
	else
		PATH_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* KILL deletes chars back to EOL */

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
	{
		chr = tty_termios_p->c_cc[VEOL];
		ret += write(loop_fildes, (char *)&chr, 1);
	}
	if (alrm_flag == 0)
		ret += write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
	{
		chr = tty_termios_p->c_cc[VKILL];
		ret += write(loop_fildes, (char *)&chr, 1);
	}
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);

	if (ret == 2 * (Strlen + 1) + 1)
	{
		ret2 = read(tty_fildes, in_buf, sizeof(in_buf));
		err2 = errno;
		if (ret2 != SYSERROR)
		{
			ret3 = read(tty_fildes, in_buf, sizeof(in_buf));
			err3 = errno;
		}
	}

	CLEAR_ALARM

	if (ret != 2 * (Strlen + 1) + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ret2 != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing KILL deletes back to EOL");
		in_rpt("first read() returned %d, expected %d", ret2, Strlen + 1);
		if (ret2 < 0)
			in_rpt("errno was set to %d (%s)", err2, errname(err2));
	}
	else
		PATH_TRACE;

	if (ret2 != SYSERROR && ret3 != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("testing KILL deletes back to EOL");
		in_rpt("second read() returned %d, expected %d", ret3, 1);
		if (ret3 < 0)
			in_rpt("errno was set to %d (%s)", err3, errname(err3));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(12);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	/* ICANON set - EOF char causes read to be satisfied, EOF discarded  */

	int 	ret, err;
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	chr = tty_termios_p->c_cc[VEOF];

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, (char *)&chr, 1);

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, in_buf, sizeof(in_buf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(in_buf, string, (size_t)Strlen) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read string != written string");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

private void
test11()
{
	/* ICANON set - EOF char with no input queue */

	int 	ret, err;
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	chr = tty_termios_p->c_cc[VEOF];

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, (char *)&chr, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, in_buf, sizeof(in_buf));
	err = errno;

	CLEAR_ALARM

	if (ret != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 0", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

private void
test12()
{
	/* ICANON set - NL char is line delimiter */

	int 	ret, err;
	int	pathok = 0;

	DBUG_ENTER("test12");

	testfail = 0;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, in_buf, sizeof(in_buf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(in_buf, string, (size_t)Strlen) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read string != written string");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

private void
test13()
{
	/* ICANON set - EOL char is line delimiter */

	int 	ret, err;
	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	chr = tty_termios_p->c_cc[VEOL];

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, (char *)&chr, 1);

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = read(tty_fildes, in_buf, sizeof(in_buf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d",
			ret, Strlen + 1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(in_buf, string, (size_t)Strlen) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read string != written string");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

private void
test14()
{
	/* IXON set - STOP and START chars suspend and restart output */

	char	*cp;
	int	buffered;
	int 	ret, err;
	char 	start_char = START;
	char 	stop_char = STOP;
	int	pathok = 0;

	DBUG_ENTER("test14");

	testfail = 0;

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* send STOP and check no output */

	SET_TIMEOUT(WAITTIME)

        ret = write(loop_fildes, &stop_char, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME/2)

        ret = write(tty_fildes, string, Strlen);

	CLEAR_ALARM

	if (buffered)
	{
		if (ret != Strlen)
		{
			xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_TTY failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		if (alrm_flag == 0)
		{
			xx_rpt(FAILURE);
			in_rpt("write() to VSX_TERMIOS_TTY did not time out");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

        (void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, in_buf, Strlen);
	err = errno;

	CLEAR_ALARM

	if (ret != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected 0", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* send START and check output now goes out */

	SET_TIMEOUT(WAITTIME)

        ret = write(loop_fildes, &start_char, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!buffered)
	{
		SET_TIMEOUT(WAITTIME)

		ret = write(tty_fildes, string, Strlen);

		CLEAR_ALARM

		if (ret != Strlen)
		{
			xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_TTY failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, in_buf, Strlen);
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(FAILURE);
		in_rpt("read() returned %d, expected %d", ret, Strlen);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(in_buf, string, (size_t)Strlen) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("read string != written string");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(8);
	}

	DBUG_VOID_RETURN;
}

private void
test15()
{
	/* INTR, QUIT, ERASE, KILL, EOF and EOL chars can be changed */
	
	int 	ret, ret2, err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test15");

	testfail = 0;
	tty_termios_p->c_lflag &= ~NOFLSH;

	/* The value for INTR can be changed */

	old_cc = tty_termios_p->c_cc[VINTR];
	tty_termios_p->c_cc[VINTR] = new_cc;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VINTR] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		caught_sig = 0;

		SET_TIMEOUT(WAITTIME)

		ret = write(loop_fildes, (char *)&new_cc, 1);
		if (alrm_flag == 0)
			(void) pause();

		CLEAR_ALARM

		if (ret != 1)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
		}
		else if (caught_sig != SIGINT)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("new INTR character failed");
		}
		else
			PATH_TRACE;
	}

	tty_termios_p->c_cc[VINTR] = old_cc;

	/* The value for QUIT can be changed */

	old_cc = tty_termios_p->c_cc[VQUIT];
	tty_termios_p->c_cc[VQUIT] = new_cc;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VQUIT] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGQUIT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGQUIT - errno %d (%s)",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		caught_sig = 0;

		SET_TIMEOUT(WAITTIME)

		ret = write(loop_fildes, (char *)&new_cc, 1);
		if (alrm_flag == 0)
			(void) pause();

		CLEAR_ALARM

		if (ret != 1)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
		}
		else if (caught_sig != SIGQUIT)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("new QUIT character failed");
		}
		else
			PATH_TRACE;
	}

	tty_termios_p->c_cc[VQUIT] = old_cc;

        /* The value for ERASE can be changed */

	old_cc = tty_termios_p->c_cc[VERASE];
	tty_termios_p->c_cc[VERASE] = new_cc;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VERASE] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, (char *)&new_cc, 1);
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);

	if (ret == Strlen + 2)
		ret2 = read(tty_fildes, in_buf, sizeof(in_buf));

	CLEAR_ALARM

	if (ret != Strlen + 2)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
	}
	else if (ret2 != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("new ERASE character failed");
	}
	else
		PATH_TRACE;

	tty_termios_p->c_cc[VERASE] = old_cc;

        /* The value for KILL can be changed */

	old_cc = tty_termios_p->c_cc[VKILL];
	tty_termios_p->c_cc[VKILL] = new_cc;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VKILL] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, (char *)&new_cc, 1);
	if (alrm_flag == 0)
		ret += write(loop_fildes, NL, 1);

	if (ret == Strlen + 2)
		ret2 = read(tty_fildes, in_buf, sizeof(in_buf));

	CLEAR_ALARM

	if (ret != Strlen + 2)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
	}
	else if (ret2 != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("new KILL character failed");
	}
	else
		PATH_TRACE;

	tty_termios_p->c_cc[VKILL] = old_cc;

        /* The value for EOF can be changed */

	old_cc = tty_termios_p->c_cc[VEOF];
	tty_termios_p->c_cc[VEOF] = new_cc;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VEOF] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, (char *)&new_cc, 1);

	if (ret == Strlen + 1)
		ret2 = read(tty_fildes, in_buf, sizeof(in_buf));

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
	}
	else if (ret2 != Strlen)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("new EOF character failed");
	}
	else
		PATH_TRACE;

	tty_termios_p->c_cc[VEOF] = old_cc;

        /* The value for EOL can be changed */

	old_cc = tty_termios_p->c_cc[VEOL];
	tty_termios_p->c_cc[VEOL] = new_cc;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VEOL] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, string, Strlen);
	if (alrm_flag == 0)
		ret += write(loop_fildes, (char *)&new_cc, 1);

	if (ret == Strlen + 1)
		ret2 = read(tty_fildes, in_buf, sizeof(in_buf));

	CLEAR_ALARM

	if (ret != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
	}
	else if (ret2 != Strlen + 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("new EOL character failed");
	}
	else
		PATH_TRACE;

	tty_termios_p->c_cc[VEOL] = old_cc;

	if (testfail == 0)
		PATH_XS_RPT(14);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	/* SUSP char can be changed */
	
	int 	ret, err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test16");

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	/* The value for SUSP can be changed */

	old_cc = tty_termios_p->c_cc[VSUSP];
	tty_termios_p->c_cc[VSUSP] = new_cc;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VSUSP] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTSTP, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGTSTP - errno %d (%s)",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;

		caught_sig = 0;

		SET_TIMEOUT(WAITTIME)

		ret = write(loop_fildes, (char *)&new_cc, 1);
		if (alrm_flag == 0)
			(void) pause();

		CLEAR_ALARM

		if (ret != 1)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
		}
		else if (caught_sig != SIGTSTP)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("new SUSP character failed");
		}
		else
			PATH_TRACE;
	}

	tty_termios_p->c_cc[VSUSP] = old_cc;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test17()
{
	/* STOP and START chars can be changed */

	char 	*supp;
	char	*cp;
	int	buffered;
	int 	ret, err;
	int	pathok = 0;

	DBUG_ENTER("test17");

	supp = tet_getvar("VSX_START_STOP_CHNG");
	if (supp == NULL || *supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_START_STOP_CHNG is not set");
		DBUG_VOID_RETURN;
	}
	else if (*supp == 'N' || *supp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_START_STOP_CHNG is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	testfail = 0;

	old_cc = tty_termios_p->c_cc[VSTOP];
	tty_termios_p->c_cc[VSTOP] = new_cc;
	tty_termios_p->c_iflag |= IXON;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VSTOP] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR ||
	    tty_termios_p->c_cc[VSTOP] != new_cc)
	{
		xx_rpt(DELETION);
		in_rpt("could not change STOP character for VSX_TERMIOS_TTY");
		tty_termios_p->c_cc[VSTOP] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* send STOP and check no output */

	SET_TIMEOUT(WAITTIME)

        ret = write(loop_fildes, (char *)&new_cc, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		tty_termios_p->c_cc[VSTOP] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME/2)

        ret = write(tty_fildes, string, Strlen);

	CLEAR_ALARM

	if (buffered)
	{
		if (ret != Strlen)
		{
			xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_TTY failed");
			tty_termios_p->c_cc[VSTOP] = old_cc;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		if (alrm_flag == 0)
		{
			xx_rpt(FAILURE);
			in_rpt("write() to VSX_TERMIOS_TTY did not time out");
			tty_termios_p->c_cc[VSTOP] = old_cc;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

        (void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, in_buf, Strlen);

	CLEAR_ALARM

	if (ret != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("new STOP character failed");
		tty_termios_p->c_cc[VSTOP] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_cc[VSTOP] = old_cc;

	/* change START, send it and check output now goes out */

	old_cc = tty_termios_p->c_cc[VSTART];
	tty_termios_p->c_cc[VSTART] = new_cc;

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		tty_termios_p->c_cc[VSTART] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR ||
	    tty_termios_p->c_cc[VSTART] != new_cc)
	{
		xx_rpt(DELETION);
		in_rpt("could not change START character for VSX_TERMIOS_TTY");
		tty_termios_p->c_cc[VSTART] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

        ret = write(loop_fildes, (char *)&new_cc, 1);

	CLEAR_ALARM

	if (ret != 1)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		tty_termios_p->c_cc[VSTART] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!buffered)
	{
		SET_TIMEOUT(WAITTIME)

		ret = write(tty_fildes, string, Strlen);

		CLEAR_ALARM

		if (ret != Strlen)
		{
			xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_TTY failed");
			tty_termios_p->c_cc[VSTART] = old_cc;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	ret = read(loop_fildes, in_buf, Strlen);

	CLEAR_ALARM

	if (ret != Strlen || strncmp(in_buf, string, (size_t)Strlen) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("new START character failed");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(12);
	}

	tty_termios_p->c_cc[VSTART] = old_cc;

	DBUG_VOID_RETURN;
}

private void
test18()
{
	/* INTR, QUIT, ERASE, KILL, EOF and EOL chars can be disabled */
	
	int 	c, i;
	int	pathok = 0;

	DBUG_ENTER("test18");

	if (posix_vdisable == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_VDISABLE not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	/*
	 * set things up so out_buf contains all non-special character
	 * values ("numchars" of them), ending with '\n'
	 *
	 * this setup is used by all the remaining tests
	 */

	tty_termios_p->c_iflag |= IXON;
	tty_termios_p->c_iflag &= ~ICRNL;
	if ((tty_termios_p->c_cflag & CSIZE) == CS8)
	{
		tty_termios_p->c_iflag &= ~ISTRIP;
		numchars = 256;	/* all 8-bit characters */
	}
	else
		numchars = 128;	/* all 7-bit characters */

	for (c = 0, i = 1; c < numchars; c++)
	{
		if (c==0)
			PATH_TRACE;
		if (c == '\n' ||
		    c == tty_termios_p->c_cc[VINTR] ||
		    c == tty_termios_p->c_cc[VQUIT] ||
		    c == tty_termios_p->c_cc[VERASE] ||
		    c == tty_termios_p->c_cc[VKILL] ||
		    c == tty_termios_p->c_cc[VEOF] ||
		    c == tty_termios_p->c_cc[VEOL] ||
		   (c == tty_termios_p->c_cc[VSUSP] &&
			    sysconf(_SC_JOB_CONTROL) != -1) ||
		    c == tty_termios_p->c_cc[VSTART] ||
		    c == tty_termios_p->c_cc[VSTOP])
			continue;
		out_buf[i++] = c;
	}
	/* out_buf[numchars-2] is used to hold the "old" value before it
	   is changed to _POSIX_VDISABLE, so reserve it here */
	out_buf[i++] = '\0';
	out_buf[i++] = '\n';
	numchars = i;

#ifndef DBUG_OFF
	for (i = 0; i < numchars; i++)
	{
		extern char *pr_char();
		DBUG_PRINTF("test18", "out_buf[%d] = '%s'",
			i, pr_char(out_buf[i]));
	}
#endif

	testcase = "with INTR set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VINTR];
	tty_termios_p->c_cc[VINTR] = posix_vdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VINTR] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/*
	 * Test all possible input characters -
	 * Those in out_buf are tested in the "NO" case, the special
	 * characters (which aren't in out_buf) are tested individually
	 */
	globok = 0; 
	if (chk_sig(SIGINT, VINTR, NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VINTR] = old_cc;

	testcase = "with QUIT set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VQUIT];
	tty_termios_p->c_cc[VQUIT] = posix_vdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VQUIT] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0; 
	if (chk_sig(SIGQUIT, VQUIT, NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VQUIT] = old_cc;

	testcase = "with ERASE set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VERASE];
	tty_termios_p->c_cc[VERASE] = posix_vdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VERASE] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	globok = 0; 
	if (chk_erase(NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VERASE] = old_cc;

	testcase = "with KILL set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VKILL];
	tty_termios_p->c_cc[VKILL] = posix_vdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VKILL] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	globok = 0; 
	if (chk_kill(NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VKILL] = old_cc;

	testcase = "with EOF set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VEOF];
	tty_termios_p->c_cc[VEOF] = posix_vdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VEOF] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	globok = 0; 
	if (chk_eof(NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VEOF] = old_cc;

	testcase = "with EOL set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VEOL];
	tty_termios_p->c_cc[VEOL] = posix_vdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VEOL] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	globok = 0; 
	if (chk_eol(NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VEOL] = old_cc;

	if (testfail == 0)
		PATH_XS_RPT(62);

	DBUG_VOID_RETURN;
}

private void
test19()
{
	/* SUSP char can be disabled */
	int	pathok = 0;
	
	DBUG_ENTER("test19");

	if (posix_vdisable == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_VDISABLE not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (numchars == 0)
	{
		xx_rpt(DELETION);
		in_rpt("depends on setup in test18");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	testcase = "with SUSP set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VSUSP];
	tty_termios_p->c_cc[VSUSP] = posix_vdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VSUSP] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	globok = 0; 
	if (chk_sig(SIGTSTP, VSUSP, NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VSUSP] = old_cc;

	if (testfail == 0)
		PATH_XS_RPT(13);

	DBUG_VOID_RETURN;
}

private void
test20()
{
	/* STOP and START chars can be disabled */

	char 	*supp;
	int	pathok = 0;

	DBUG_ENTER("test20");

	if (posix_vdisable == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_VDISABLE not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	supp = tet_getvar("VSX_START_STOP_CHNG");
	if (supp == NULL || *supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_START_STOP_CHNG is not set");
		DBUG_VOID_RETURN;
	}
	else if (*supp == 'N' || *supp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_START_STOP_CHNG is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (numchars == 0)
	{
		xx_rpt(DELETION);
		in_rpt("depends on setup in test18");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	testcase = "with STOP set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VSTOP];
	tty_termios_p->c_cc[VSTOP] = posix_vdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VSTOP] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR ||
	    tty_termios_p->c_cc[VSTOP] != posix_vdisable)
	{
		xx_rpt(DELETION);
		in_rpt("could not change STOP character for VSX_TERMIOS_TTY");
		tty_termios_p->c_cc[VSTOP] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (chk_xon(VSTOP, NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VSTOP] = old_cc;

	testcase = "with START set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VSTART];
	tty_termios_p->c_cc[VSTART] = posix_vdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VSTART] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR ||
	    tty_termios_p->c_cc[VSTART] != posix_vdisable)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not change START character for VSX_TERMIOS_TTY");
		tty_termios_p->c_cc[VSTART] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (chk_xon(VSTART, NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VSTART] = old_cc;

	if (testfail == 0)
		PATH_XS_RPT(25);

	DBUG_VOID_RETURN;
}

private void
test21()
{
	/* IEXTEN clear => non-special characters received without interp */
	int	pathok = 0;

	DBUG_ENTER("test21");

	if (numchars == 0)
	{
		xx_rpt(DELETION);
		in_rpt("depends on setup in test18");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	testcase = "with IEXTEN clear";

	globok = 0;
	if (tsetup() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	old_cc = '\0';	/* do_buf() puts this in out_buf[] */

	globok = 0;
	(void) do_buf();
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test22()
{
	/* INTR, QUIT, ERASE, KILL, EOF and EOL chars can be disabled */
	
	int 	c, i;
	int	pathok = 0;

	DBUG_ENTER("test22");

	if (posix_pdisable == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_VDISABLE not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	/*
	 * set things up so out_buf contains all non-special character
	 * values ("numchars" of them), ending with '\n'
	 *
	 * this setup is used by all the remaining tests
	 */

	tty_termios_p->c_iflag |= IXON;
	tty_termios_p->c_iflag &= ~ICRNL;
	if ((tty_termios_p->c_cflag & CSIZE) == CS8)
	{
		tty_termios_p->c_iflag &= ~ISTRIP;
		numchars = 256;	/* all 8-bit characters */
	}
	else
		numchars = 128;	/* all 7-bit characters */

	for (c = 0, i = 1; c < numchars; c++)
	{
		if (c==0)
			PATH_TRACE;
		if (c == '\n' ||
		    c == tty_termios_p->c_cc[VINTR] ||
		    c == tty_termios_p->c_cc[VQUIT] ||
		    c == tty_termios_p->c_cc[VERASE] ||
		    c == tty_termios_p->c_cc[VKILL] ||
		    c == tty_termios_p->c_cc[VEOF] ||
		    c == tty_termios_p->c_cc[VEOL] ||
		   (c == tty_termios_p->c_cc[VSUSP] &&
			    sysconf(_SC_JOB_CONTROL) != -1) ||
		    c == tty_termios_p->c_cc[VSTART] ||
		    c == tty_termios_p->c_cc[VSTOP])
			continue;
		out_buf[i++] = c;
	}
	/* out_buf[numchars-2] is used to hold the "old" value before it
	   is changed to _POSIX_VDISABLE, so reserve it here */
	out_buf[i++] = '\0';
	out_buf[i++] = '\n';
	numchars = i;

#ifndef DBUG_OFF
	for (i = 0; i < numchars; i++)
	{
		extern char *pr_char();
		DBUG_PRINTF("test22", "out_buf[%d] = '%s'",
			i, pr_char(out_buf[i]));
	}
#endif

	testcase = "with INTR set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VINTR];
	tty_termios_p->c_cc[VINTR] = posix_pdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VINTR] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/*
	 * Test all possible input characters -
	 * Those in out_buf are tested in the "NO" case, the special
	 * characters (which aren't in out_buf) are tested individually
	 */
	globok = 0; 
	if (chk_sig(SIGINT, VINTR, NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VINTR] = old_cc;

	testcase = "with QUIT set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VQUIT];
	tty_termios_p->c_cc[VQUIT] = posix_pdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VQUIT] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0; 
	if (chk_sig(SIGQUIT, VQUIT, NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VQUIT] = old_cc;

	testcase = "with ERASE set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VERASE];
	tty_termios_p->c_cc[VERASE] = posix_pdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VERASE] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	globok = 0; 
	if (chk_erase(NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VERASE] = old_cc;

	testcase = "with KILL set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VKILL];
	tty_termios_p->c_cc[VKILL] = posix_pdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VKILL] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	globok = 0; 
	if (chk_kill(NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VKILL] = old_cc;

	testcase = "with EOF set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VEOF];
	tty_termios_p->c_cc[VEOF] = posix_pdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VEOF] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	globok = 0; 
	if (chk_eof(NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VEOF] = old_cc;

	testcase = "with EOL set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VEOL];
	tty_termios_p->c_cc[VEOL] = posix_pdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VEOL] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	globok = 0; 
	if (chk_eol(NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VEOL] = old_cc;

	if (testfail == 0)
		PATH_XS_RPT(62);

	DBUG_VOID_RETURN;
}

private void
test23()
{
	/* SUSP char can be disabled */
	int	pathok = 0;
	
	DBUG_ENTER("test23");

	if (posix_pdisable == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_VDISABLE not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (numchars == 0)
	{
		xx_rpt(DELETION);
		in_rpt("depends on setup in test22");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	testcase = "with SUSP set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VSUSP];
	tty_termios_p->c_cc[VSUSP] = posix_pdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VSUSP] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	globok = 0; 
	if (chk_sig(SIGTSTP, VSUSP, NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VSUSP] = old_cc;

	if (testfail == 0)
		PATH_XS_RPT(13);

	DBUG_VOID_RETURN;
}

private void
test24()
{
	/* STOP and START chars can be disabled */

	char 	*supp;
	int	pathok = 0;

	DBUG_ENTER("test24");

	if (posix_pdisable == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_VDISABLE not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	supp = tet_getvar("VSX_START_STOP_CHNG");
	if (supp == NULL || *supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_START_STOP_CHNG is not set");
		DBUG_VOID_RETURN;
	}
	else if (*supp == 'N' || *supp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_START_STOP_CHNG is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (numchars == 0)
	{
		xx_rpt(DELETION);
		in_rpt("depends on setup in test22");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	testcase = "with STOP set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VSTOP];
	tty_termios_p->c_cc[VSTOP] = posix_pdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VSTOP] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR ||
	    tty_termios_p->c_cc[VSTOP] != posix_pdisable)
	{
		xx_rpt(DELETION);
		in_rpt("could not change STOP character for VSX_TERMIOS_TTY");
		tty_termios_p->c_cc[VSTOP] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (chk_xon(VSTOP, NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTART, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VSTOP] = old_cc;

	testcase = "with START set to _POSIX_VDISABLE";

	old_cc = tty_termios_p->c_cc[VSTART];
	tty_termios_p->c_cc[VSTART] = posix_pdisable;

	globok = 0;
	if (tsetup() != 0)
	{
		tty_termios_p->c_cc[VSTART] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR ||
	    tty_termios_p->c_cc[VSTART] != posix_pdisable)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not change START character for VSX_TERMIOS_TTY");
		tty_termios_p->c_cc[VSTART] = old_cc;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (chk_xon(VSTART, NO) == 0)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) chk_sig(SIGINT, VINTR, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGQUIT, VQUIT, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_erase(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_kill(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eof(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_eol(YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_sig(SIGTSTP, VSUSP, YES);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) chk_xon(VSTOP, YES);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
	}

	tty_termios_p->c_cc[VSTART] = old_cc;

	if (testfail == 0)
		PATH_XS_RPT(25);

	DBUG_VOID_RETURN;
}


private int
do_buf()
{
	int ret, ret2, err;
	int	pathok = 0;

	DBUG_ENTER("do_buf");

	out_buf[numchars-2] = old_cc;

	/* numchars is < _POSIX_MAX_INPUT, so OK to read in one go */

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, out_buf, (unsigned) numchars);
	if (ret == numchars)
		ret2 = read(tty_fildes, in_buf, sizeof(in_buf));
	err = errno;

	CLEAR_ALARM

	if (ret != numchars)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (ret2 != numchars)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("%s, read() from VSX_TERMIOS_TTY returned %d, expected %d",
			testcase, ret2, numchars);
		if (ret2 < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);
	
	DBUG_RETURN(0);
}

private int
chk_sig(signum, vchar, expect)
int signum, vchar, expect;
{
	int ret, err;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("chk_sig");

	if (vchar == VSUSP && sysconf(_SC_JOB_CONTROL) == -1)
	{
		PATH_FUNC_RPT(0); 
		DBUG_RETURN(0);
	}

	if(DBUG_SETJMP(sjmp) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: signal %d (%s) received",
				jmpsignum, signame(jmpsignum));
		DBUG_RETURN(-1);
	}
	setsigs(sig_func);

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(signum, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for %s - errno %d (%s)",
			signame(signum), err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	if (expect)
	{
		/* check sending c_cc[vchar] causes signum */

		SET_TIMEOUT(WAITTIME/2)

		chr = tty_termios_p->c_cc[vchar];
		ret = write(loop_fildes, (char *)&chr, 1);
		if (alrm_flag == 0)
			(void) pause();

		CLEAR_ALARM

		if (ret != 1)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
			DBUG_RETURN(-1);
		}
		else if (caught_sig != signum)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("%s, %s was not received",
				testcase, signame(signum));
			DBUG_RETURN(-1);
		}
	}
	else
	{
		/* Check all possible characters do not cause signum */

		globok = 0;
		ret = do_buf();
		PATH_FUNC_TRACE;

		if (caught_sig == signum)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("%s, %s was received when not expected",
				testcase, signame(signum));
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;

		PATH_FUNC_RPT(3); 
		DBUG_RETURN(ret);
	}

	PATH_FUNC_RPT(1); 
	DBUG_RETURN(0);
}

private int
chk_erase(expect)
int expect;
{
	int ret, ret2;
	int	pathok = 0;

	DBUG_ENTER("chk_erase");

	if (expect)
	{
		/* check ERASE character works */

		chr = tty_termios_p->c_cc[VERASE];

		SET_TIMEOUT(WAITTIME)

		ret = write(loop_fildes, string, Strlen);
		if (alrm_flag == 0)
			ret += write(loop_fildes, (char *)&chr, 1);
		if (alrm_flag == 0)
			ret += write(loop_fildes, NL, 1);

		if (ret == Strlen + 2)
			ret2 = read(tty_fildes, in_buf, sizeof(in_buf));

		CLEAR_ALARM

		if (ret != Strlen + 2)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;

		if (ret2 != Strlen)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("%s, ERASE character did not work", testcase);
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
	}
	else
	{
		/* check all non-special characters are read */

		globok = 0;
		ret = do_buf();
		PATH_FUNC_TRACE;
		PATH_FUNC_RPT(1); 
		DBUG_RETURN(ret);
	}

	PATH_FUNC_RPT(2); 
	DBUG_RETURN(0);
}

private int
chk_kill(expect)
int expect;
{
	int ret, ret2;
	int	pathok = 0;

	DBUG_ENTER("chk_kill");

	if (expect)
	{
		/* check KILL character works */

		chr = tty_termios_p->c_cc[VKILL];

		SET_TIMEOUT(WAITTIME)

		ret = write(loop_fildes, string, Strlen);
		if (alrm_flag == 0)
			ret += write(loop_fildes, (char *)&chr, 1);
		if (alrm_flag == 0)
			ret += write(loop_fildes, NL, 1);

		if (ret == Strlen + 2)
			ret2 = read(tty_fildes, in_buf, sizeof(in_buf));

		CLEAR_ALARM

		if (ret != Strlen + 2)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;

		if (ret2 != 1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("%s, KILL character did not work", testcase);
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
	}
	else
	{
		/* check all non-special characters are read */

		globok = 0;
		ret = do_buf();
		PATH_FUNC_TRACE;
		PATH_FUNC_RPT(1); 
		DBUG_RETURN(ret);
	}
	PATH_FUNC_RPT(2); 
	DBUG_RETURN(0);
}

private int
chk_eof(expect)
int expect;
{
	int ret, ret2, ret3;
	int	pathok = 0;

	DBUG_ENTER("chk_eof");

	if (expect)
	{
		/* check EOF character works */

		chr = tty_termios_p->c_cc[VEOF];

		SET_TIMEOUT(WAITTIME)

		ret = write(loop_fildes, string, Strlen);
		if (alrm_flag == 0)
			ret += write(loop_fildes, (char *)&chr, 1);
		if (alrm_flag == 0)
			ret += write(loop_fildes, string, Strlen);
		if (alrm_flag == 0)
			ret += write(loop_fildes, NL, 1);

		if (ret == 2 * Strlen + 2)
			ret2 = read(tty_fildes, in_buf, sizeof(in_buf));
		if (ret2 == Strlen)
			ret3 = read(tty_fildes, in_buf, sizeof(in_buf));

		CLEAR_ALARM

		if (ret != 2 * Strlen + 2)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;

		if (ret2 != Strlen || ret3 != Strlen + 1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("%s, EOF character did not work", testcase);
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
	}
	else
	{
		/* check all non-special characters are read */

		globok = 0;
		ret = do_buf();
		PATH_FUNC_TRACE;
		PATH_FUNC_RPT(1); 
		DBUG_RETURN(ret);
	}
	PATH_FUNC_RPT(2); 
	DBUG_RETURN(0);
}

private int
chk_eol(expect)
int expect;
{
	int ret, ret2, ret3;
	int	pathok = 0;

	DBUG_ENTER("chk_eol");

	if (expect)
	{
		/* check EOL character works */

		chr = tty_termios_p->c_cc[VEOL];

		SET_TIMEOUT(WAITTIME)

		ret = write(loop_fildes, string, Strlen);
		if (alrm_flag == 0)
			ret += write(loop_fildes, (char *)&chr, 1);
		if (alrm_flag == 0)
			ret += write(loop_fildes, string, Strlen);
		if (alrm_flag == 0)
			ret += write(loop_fildes, NL, 1);

		if (ret == 2 * Strlen + 2)
			ret2 = read(tty_fildes, in_buf, sizeof(in_buf));
		if (ret2 == Strlen + 1)
			ret3 = read(tty_fildes, in_buf, sizeof(in_buf));

		CLEAR_ALARM

		if (ret != 2 * Strlen + 2)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;

		if (ret2 != Strlen + 1 || ret3 != Strlen + 1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("%s, EOL character did not work", testcase);
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
	}
	else
	{
		/* check all non-special characters are read */

		globok = 0;
		ret = do_buf();
		PATH_FUNC_TRACE;
		PATH_FUNC_RPT(1); 
		DBUG_RETURN(ret);
	}
	PATH_FUNC_RPT(2); 
	DBUG_RETURN(0);
}

private int
chk_xon(vchar, expect)
int vchar, expect;
{
	int ret, ret2;
	int	pathok = 0;

	DBUG_ENTER("chk_xon");

	if (expect)
	{
		/* send STOP/START and check we don't read it */

		chr = tty_termios_p->c_cc[vchar];

		SET_TIMEOUT(WAITTIME)

		ret = write(loop_fildes, (char *)&chr, 1);
		if (alrm_flag == 0)
			ret += write(loop_fildes, NL, 1);
		if (ret == 2)
			ret2 = read(tty_fildes, in_buf, sizeof(in_buf));

		CLEAR_ALARM

		if (ret != 2)
		{
			xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;

		if (ret2 > 0 && (unsigned char)in_buf[0] == chr)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("%s, %s character was read",
				testcase, (vchar == VSTOP ? "STOP" : "START"));
			DBUG_RETURN(-1);
		}
		else if (ret2 != 1)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("%s, read() from VSX_TERMIOS_TTY returned %d, expected 1",
				testcase, ret2);
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
	}
	else
	{
		/* check all non-special characters are read */

		globok = 0;
		ret = do_buf();
		PATH_FUNC_TRACE;
		PATH_FUNC_RPT(1); 
		DBUG_RETURN(ret);
	}
	PATH_FUNC_RPT(2); 
	DBUG_RETURN(0);
}

private	void
sig_func(sig)
int 	sig;
{
	DBUG_ENTER("sig_func");
	jmpsignum = sig;
	DBUG_LONGJMP(sjmp, 1);
	DBUG_VOID_RETURN;
}
