/*
*      SCCS:  @(#)  POSIX.os/devclass/tcdrain/tcdrain.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/tcdrain - T.tcdrain Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.tcdrain

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the tcdrain() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	6 Jan 1989
MODIFICATIONS:
		Geoff Clare, 2 Nov 1989
			cppair() timeout adjustments
		
		Geoff Clare, 1 May 1990
			Sleep before flush at start of each test
			in case some data are still in transit.
		
		Geoff Clare, 16 May 1990
			Ignore SIGTTOU in children not expecting it as
			cppair() now sets it to SIG_DFL.

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported

		David Sawyer
		Wed Aug 08 10:04:45 BST 1990
		Added resilience code.
		
		Geoff Clare, 14 Nov 1991
			Changed pa_fail = 2 to pa_fail |= 2 in drainit().

		Geoff Clare, 23 Jan 1992
			Improved flushing technique.
			Use VSX_TERMIOS_SPEED and buffer size based on
			data_rate instead of fixed B1200 and 1024 bytes.

		Geoff Clare, 3 Mar 1992
			Added flushem() call in clean_tests().

		Stuart Boutell, 20 May 1992
			SIGTTOUT test now performed with TOSTOP mode set
		and clear.

		Stuart Boutell, 21 May 1992
			Removed dependency on GTI for EBADF and ENOTTY tests

		Stuart Boutell, 5 June 1992
			Added non-controlling terminal tests.

		Geoff Clare, 8 June 1992
			Allow for systems with no terminal output queue.

		Kieron Drake, 17 june 1992
			Add tprep_xcntl() and set_xcntl() to avoid having
			Stop/Start chars eaten by the driver when
			using tty and loop the "other way round".
			(tests 8, 9, 10 & 11)

		Geoff Clare, 6 Oct 1992
			Add sleep before TCOON in ch_t7() in case
			all output fits in the buffer.

		Geoff Clare, 4 Mar 1993
			Turn on flow control so that if read side does not
			keep up with write side no data will be lost.

		Geoff Clare, 14 July 1994
			Reduce the amount of data that has to fit in
			the output buffer.
			Flush output queue before TCOON in ch_t7().

		Geoff Clare, 10 Feb 1995
			Use do_fnoeio() instead of do_feio().

		Geoff Clare, 20 Sept 1995
			Fix path tracing problems in checkit() and
			in ch_t2() and ch_t9() exit values.

************************************************************************/

#include <std.h>
#include <stdio.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef tcdrain
#endif

#ifndef _XOPEN_SOURCE
#undef memset
extern	void	*memset();
#undef memchr
extern	void	*memchr();
#endif

#define NO_TESTS	12
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP	 	3
#define READ_TIME 	(2 * NAP)
#define MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define NOK		002
#define E_SAFAIL	004
#define E_GOTSIG	010
#define E_WRFAIL	020
#define E_FLOWOFF	040
#define PATH_OK	 	0100

extern	void	xx_all();
extern	char	*errname();
extern	int	data_rate;
extern	void	do_fnoeio();

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	ch_t7();
private void	ch_t9();
private void	ch_t10();
private void	ch_t11();
private void	gch_t3();
private void	gch_t4();
private void	pa_t7();
private	int	t12io();
private	void	t12rpt();
private void	prepare_tests();
private void	clean_tests();
private void	do_tcdrain();
private void	drainit();
private void	chkread();
private void	checkit();
private void	wr_buf();
private	void	twrap();
private int	set_tostop();
private	int	clear_tostop();

public	void	(*realfuncs[]) () = { test1, test2, test3 ,test4,
				      NULL, NULL, test7, test8,
				      test9, test10, test11, test12, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	twrap, 1,
	twrap, 2,
	twrap, 3,
	twrap, 4,
	test5, 5,
	test6, 6,
	twrap, 7,
	twrap, 8,
	twrap, 9,
	twrap, 10,
	twrap, 11,
	twrap, 12,
	NULL, 0
};

private char	*tfile =	"./tcdrain-t";
private	int	tty_fildes, loop_fildes;
private unsigned int	buflen;
private	char	*inbuf;
private	char	*outbuf;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private	int	caught_sig = 0;
private int	no_action = 0;
private int	testfail;
private	int	do_write = TRUE;
private	int	use_tty = TRUE;
private int	pa_fail;
private int	pa_rval;
private int	pa_errno;
private int	gti_supported;

private void
sig_catch(sig)
{
	caught_sig = sig;
}

private void
twrap()
{
	DBUG_ENTER("twrap");

	if (!gti_supported) {
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if(tet_thistest >=1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();
	
	DBUG_VOID_RETURN;
}

	
private void
prepare_tests()
{
	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		gti_supported = TRUE;
		break;
	case 1:
		gti_supported = FALSE;
		DBUG_VOID_RETURN;

	default:
		{
			char *msg;

			if (alrm_flag > 0)
				msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
			else
				msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
			xx_all(DELETION, msg);
			DBUG_VOID_RETURN;
		}
	}

	/* switch on flow control and set tty side to non-canonical input
	   for when the loopback is used the "other way round" */

	tty_termios_p->c_iflag |= (IXON|IXOFF);
	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 1;
	tty_termios_p->c_cc[VTIME] = 0;

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		xx_all(DELETION, "tcsetattr() failed on VSX_TERMIOS_TTY in setup");
		DBUG_VOID_RETURN;
	}

	loop_termios_p->c_iflag |= (IXON|IXOFF);

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		xx_all(DELETION, "tcsetattr() failed on VSX_TERMIOS_LOOP in setup");
		DBUG_VOID_RETURN;
	}

	/* Allocate buffers containing number of bytes which will
	   require READ_TIME seconds to transmit at VSX_TERMIOS_SPEED */
	buflen = READ_TIME * (unsigned)data_rate;
	inbuf = (char *)malloc((size_t)buflen);
	outbuf = (char *)malloc((size_t)buflen);
	if (inbuf == NULL || outbuf == NULL)
	{
		xx_all(DELETION, "malloc() failed in setup");
		DBUG_VOID_RETURN;
	}

	/* Initialise output buffer */
	(void) memset((void *)outbuf, 'A', (size_t)buflen);

	DBUG_VOID_RETURN;
}

private int
flushem()
{
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("flushem");

	/* There can be large amounts of data still in output queues,
	   so flush both sides, wait for data in transit to get across,
	   then flush both sides again. */

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_RETURN(0);
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

	if (gti_supported) {
		(void) flushem();
		(void) close(tty_fildes);
		(void) close(loop_fildes);
	}
	(void) unlink(tfile);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = 0;

	DBUG_ENTER("test1");

	use_tty = TRUE;
	no_action = 0;

	testfail = 0;


	globok = 0;
	if ( ! clear_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Foreground process */

	do_write = TRUE;

	globok = 0;
	do_tcdrain();
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	pathok = 0;
	int	ret;

	DBUG_ENTER("test2");

	use_tty = TRUE;
	no_action = 1;	/* tcdrain action shall not be performed */

	testfail = 0;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( ! clear_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	do_write = TRUE;
	ret = cppair(ch_t2, NULLFN, 3 * WAITTIME, NOK|PATH_OK);

	if (ret == NOK) {
		testfail++;
		in_rpt("when TOSTOP flag was clear");
	}
	else if (ret == PATH_OK)
		PATH_TRACE;

	globok = 0;
	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	do_write = FALSE;
	ret = cppair(ch_t2, NULLFN, 3 * WAITTIME, NOK|PATH_OK);

	if (ret == NOK) {
		testfail++;
		in_rpt("when TOSTOP flag was set");
	}
	else if (ret == PATH_OK)
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");


	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	caught_sig = 0;

	globok = 0;
	do_tcdrain();
	PATH_FUNC_TRACE;
	/* pause() here ? */

#ifdef TEST
caught_sig++;
#endif

	if (caught_sig != SIGTTOU)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU not received");
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok == 4)
		DBUG_EXIT(PATH_OK);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int	pathok = 0;

	DBUG_ENTER("test3");

	use_tty = TRUE;
	no_action = 0;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( ! clear_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(3); /* set globok for child - just to be thorough */

	do_write = TRUE;

	(void) cppair(ch_t3, NULLFN, 3 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int 	ret;
	sigset_t set;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	PATH_FUNC_TRACE; /* checking globok set in parent */

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* block SIGTTOU */

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTTOU);
	if (sigprocmask(SIG_BLOCK, &set, (sigset_t *)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask(SIG_BLOCK, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t3, do_tcdrain, WAITTIME, E_SAFAIL|E_GOTSIG);
	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_FUNC_TRACE;

	/* Check if SIGTTOU is pending?  (shouldn't be) */

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
gch_t3()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t3");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	pathok = 0;

	DBUG_ENTER("test4");

	use_tty = TRUE;
	no_action = 0;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( ! clear_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(3); /* set globok for child - just to be thorough */

	do_write = TRUE;

	(void) cppair(ch_t4, NULLFN, 3 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int 	ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	PATH_FUNC_TRACE; /* checking globok set in parent */

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t4, do_tcdrain, WAITTIME, E_SAFAIL|E_GOTSIG);
	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
gch_t4()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t4");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	if ((rval = tcdrain(-1)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcdrain(-1) returned %d, expected -1", rval);
	}
	else if (errno != EBADF)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcdrain(-1) gave errno %d (%s), expected EBADF",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}


private void
test6()
{
	int 	fd, rval, err;
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	fd = creat(tfile, MODEANY);
	if (fd == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file %s", tfile);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = tcdrain(fd)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcdrain(fd) returned %d, expected -1", rval);
		in_rpt("where fd was open to a plain file");
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcdrain(fd) gave errno %d (%s), expected ENOTTY",
			err, errname(err));
		in_rpt("where fd was open to a plain file");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) close(fd);
	(void) unlink(tfile);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	char	*cp;
	int 	ret;
	int	pathok = 0;

	DBUG_ENTER("test7");

	use_tty = FALSE;

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		xx_rpt(UNTESTED);
		in_rpt("VSX_TERMIOS_BUFFERED is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( ! clear_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	ret = cppair(ch_t7, pa_t7, WAITTIME, E_FLOWOFF);
	if (ret == E_FLOWOFF)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY");
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	DBUG_ENTER("ch_t7");

	if (tcflow(tty_fildes, TCOOFF) == SYSERROR)
		DBUG_EXIT(E_FLOWOFF);

	SET_TIMEOUT(READ_TIME + WAITTIME/2);

	(void) write(tty_fildes, outbuf, buflen);

	CLEAR_ALARM

	if (alrm_flag == 0)
		(void) sleep(3*NAP);

	(void) tcflush(tty_fildes, TCOFLUSH);
	(void) tcflow(tty_fildes, TCOON);

	DBUG_VOID_RETURN;
}

private void
pa_t7()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("pa_t7");

	(void) sleep(NAP);

	SET_TIMEOUT(NAP);

	rval = tcdrain (tty_fildes);
	err = errno;

	CLEAR_ALARM

	if (alrm_flag == 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("Time-out did not occur");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (rval != -1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcdrain() returned %d, expected -1", rval);
	}
	else if (err != EINTR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcdrain() gave errno %d (%s), expected EINTR",
			err, errname(err));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	int	pathok = 0;

	DBUG_ENTER("test8");

	use_tty = FALSE;
	no_action = 0;

	testfail = 0;

	globok = 0;
	if ( ! clear_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Foreground process */

	do_write = TRUE;

	globok = 0;
	do_tcdrain();
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	int	pathok = 0;
	int	ret;

	DBUG_ENTER("test9");

	use_tty = FALSE;
	no_action = 0;

	testfail = 0;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( ! clear_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	do_write = TRUE;
	ret = cppair(ch_t9, NULLFN, 3 * WAITTIME, NOK|PATH_OK);

	if (ret == NOK) {
		testfail++;
		in_rpt("when TOSTOP flag was clear");
	}
	else if (ret == PATH_OK)
		PATH_TRACE;

	globok = 0;
	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	do_write = FALSE;
	ret = cppair(ch_t9, NULLFN, 3 * WAITTIME, NOK|PATH_OK);

	if (ret == NOK) {
		testfail++;
		in_rpt("when TOSTOP flag was set");
	}
	else if (ret == PATH_OK)
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t9");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	caught_sig = 0;

	globok = 0;
	do_tcdrain();
	PATH_FUNC_TRACE;
	/* pause() here ? */

#ifdef TEST
caught_sig++;
#endif

	if (caught_sig == SIGTTOU)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU was received");
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok == 4)
		DBUG_EXIT(PATH_OK);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	int	pathok = 0;

	DBUG_ENTER("test10");

	use_tty = FALSE;
	no_action = 0;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( ! clear_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(3); /* set globok for child - just to be thorough */

	do_write = TRUE;

	(void) cppair(ch_t10, NULLFN, 3 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	int 	ret;
	sigset_t set;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t10");

	PATH_FUNC_TRACE; /* checking globok set in parent */

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* block SIGTTOU */

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTTOU);
	if (sigprocmask(SIG_BLOCK, &set, (sigset_t *)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask(SIG_BLOCK, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t3, do_tcdrain, WAITTIME, E_SAFAIL|E_GOTSIG);
	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_FUNC_TRACE;

	/* Check if SIGTTOU is pending?  (shouldn't be) */

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test11()
{
	int	pathok = 0;

	DBUG_ENTER("test11");

	use_tty = FALSE;
	no_action = 0;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( ! clear_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(3); /* set globok for child - just to be thorough */

	do_write = TRUE;

	(void) cppair(ch_t11, NULLFN, 3 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	int 	ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t11");

	PATH_FUNC_TRACE; /* checking globok set in parent */

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t4, do_tcdrain, WAITTIME, E_SAFAIL|E_GOTSIG);
	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	DBUG_ENTER("test12");

	do_fnoeio("w", -1, t12io, t12rpt);

	DBUG_VOID_RETURN;
}

private int
t12io(fp)
FILE *fp;
{
	globok = 1;
	return tcdrain(fileno(fp));
}

private void
t12rpt(ret, err)
int ret, err;
{
	int pathok = 0;

	DBUG_ENTER("t12rpt");

	if (ret != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("tcdrain() from orphaned background process group did not give correct results");

		in_rpt("expected return: 0, actual: %d", ret);
		if (ret == -1 && err != NOERROR)
			in_rpt("expected no error. actual errno: %d(%s)",
				err, errname(err));
	} else {
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

	
private int
clear_tostop() 
{
	int	pathok = 0;

	DBUG_ENTER("clear_tostop");

	tty_termios_p->c_lflag &= ~(TOSTOP);
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		DBUG_RETURN(0);
	} else
		PATH_TRACE;

	if (!use_tty) {
		loop_termios_p->c_lflag &= ~(TOSTOP);
		if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		{
			DBUG_RETURN(0);
		} else
			PATH_TRACE;
	} else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(1);
}

private int
set_tostop()
{
	int	pathok = 0;

	DBUG_ENTER("set_tostop");

	tty_termios_p->c_lflag |= TOSTOP;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		DBUG_RETURN(0);
	} else
		PATH_TRACE;

	if (!use_tty) {
		loop_termios_p->c_lflag |= TOSTOP;
		if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		{
			DBUG_RETURN(0);
		} else
			PATH_TRACE;
	} else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(1);
}

private void
do_tcdrain()
{
	int	retval;
	int	pathok = 0;

	DBUG_ENTER("do_tcdrain");

	pa_fail = 0;
	retval = cppair(chkread, drainit, 2*WAITTIME, NOK|PATH_OK);
	if (retval == NOK)
		testfail++;	/* already reported by chkread */
	else if(retval == PATH_OK)
		PATH_FUNC_TRACE;

	if (pa_fail & 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcdrain() on VSX_TERMIOS_%s returned %d, expected 0",
			(use_tty?"TTY":"LOOP"), pa_rval);
		in_rpt("errno was set to %d (%s)", pa_errno, errname(pa_errno));
	}
	else
		PATH_TRACE;

	if (pa_fail & 2)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_%s after tcdrain() failed",
			(use_tty?"TTY":"LOOP"));
	}
	else
		PATH_TRACE;


	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
drainit()
{
	char 	X = 'X';
	int	pathok = 0;

	DBUG_ENTER("drainit");

	/* call tcdrain() and write a character */

	(void) sleep(NAP);

	SET_TIMEOUT(READ_TIME + WAITTIME/2);

	errno = NOERROR;

	if ((pa_rval = tcdrain( (use_tty?tty_fildes:loop_fildes) )) != 0
		&& !no_action)
	{
		pa_fail |= 1;
		pa_errno = errno;
	}
	else
		PATH_TRACE;

	CLEAR_ALARM

	if (do_write) {
		SET_TIMEOUT(WAITTIME/2);

		if (write( (use_tty?tty_fildes:loop_fildes), &X, 1) != 1)
			pa_fail |= 2;
		else
			PATH_TRACE;
		
		CLEAR_ALARM
	} else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
chkread()
{
	int 	ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("chkread");

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	(void) sigaction(SIGTTOU, &sig, NULLSA);

	globok = 0;
	ret = cppair(wr_buf, checkit, WAITTIME+READ_TIME+NAP, E_WRFAIL);
	if (ret == E_WRFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_%s failed",
			(use_tty?"TTY":"LOOP"));
	}
	else
		PATH_FUNC_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok == 1)
		DBUG_EXIT(PATH_OK);

	DBUG_VOID_RETURN;
}

private void
wr_buf()
{
	struct sigaction sig;
	int	wret;

	DBUG_ENTER("wr_buf");

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	(void) sigaction(SIGTTOU, &sig, NULLSA);

	/* write buflen bytes */

	wret = write( (use_tty?tty_fildes:loop_fildes), outbuf, buflen);

	if (wret != buflen && !no_action)
		DBUG_EXIT(E_WRFAIL);
	
	DBUG_VOID_RETURN;
}

private void
checkit()
{
	char	*pos;
	int	i, nbytes;
	int	pathok = 0;

	DBUG_ENTER("checkit");

	if (!use_tty) {
		(void) setpgid(0, tcgetpgrp(tty_fildes) );
	}

	/* read up to buflen bytes */

	(void) memset((void *)inbuf, 0, (size_t)buflen);

	SET_TIMEOUT(READ_TIME + WAITTIME);

	for (i = 0; i < buflen; i += nbytes)
	{
		nbytes = read( (use_tty?loop_fildes:tty_fildes),
				inbuf + i, buflen);
		if (nbytes < 0)
			break;
		if (i==0 && nbytes > 0)
			PATH_TRACE;
	}
	
	CLEAR_ALARM

	DBUG_PRINTF("checkit","alrm_flag:%d i:%d nbytes:%d errno:%d", alrm_flag,
		i, nbytes, errno);

	if (alrm_flag > 0 || nbytes < 0)
	{
		if (!no_action)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("read() from VSX_TERMIOS_%s failed",
				(use_tty?"LOOP":"TTY"));
		}
		else
			PATH_TRACE;

		PATH_FUNC_RPT(2);

		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* look for interleaved character */

	pos = (char *)memchr((void *)inbuf, 'X', (size_t)buflen);
	if (pos != NULL && !no_action)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcdrain() failed - interleaved character at %d",
			pos - inbuf);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

