/*
*      SCCS:  @(#)  POSIX.os/devclass/tcflow/tcflow.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/tcflow - T.tcflow Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.tcflow

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the tcflow() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	20 Jan 1989
MODIFICATIONS: J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported

		David Sawyer
		Tue Aug 07 17:03:32 BST 1990
		Added resilience code.

		Stuart Boutell, 19 May 1992
		Changed test six to test for TOSTOP set and clear.

		Stuart Boutell, 21 May 1992
		Removed dependency on GTI for EBADF and ENOTTY tests

		Stuart Boutell, 1 June 1992
		Add non-controlling terminal tests.

		Geoff Clare, 8 June 1992
			Allow for systems with no terminal output queue.

		Kieron Drake, 17 june 1992
			Add tprep_xcntl() and set_xcntl() to avoid having
			Stop/Start chars eaten by the driver when
			using tty and loop the "other way round".
			(tests 8, 9, 10 & 11)

		Geoff Clare, 14 Sept 1992
			Fix code that allows for no terminal output
			queue in TCOON case of test 6.

		Geoff Clare, 21 July 1994
			Improve flushing in tprep().

		Geoff Clare, 10 Feb 1995
			Use do_fnoeio() instead of do_feio().

************************************************************************/

#include <std.h>
#include <stdio.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef tcflow
#endif

#ifndef _XOPEN_SOURCE
#undef memchr
extern	void	*memchr();
#endif

#define NO_TESTS	16
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		2
#define READ_TIME	50
#define MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define NOK		002
#define E_DEL		004
#define E_SAFAIL	010
#define E_GOTSIG	020
#define PATH_OK		040

#define USE_TTY		0x1
#define	USE_LOOP	0x0
#define WR_SUCC		1
#define WR_FAIL		0

#define BUFLEN		(sizeof(outbuf)-1)
#define NUM(a)		(sizeof(a)/sizeof(*a))   /* number of array elements */

extern	char	*errname();
extern	void	do_fnoeio();
extern	void	xx_all();

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	do_test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private	void	test14();
private	void	do_test14();
private	void	test15();
private	void	test16();

private void	ch_t6();
private void	ch_t7();
private void	ch_t8();
private void	gch_t7();
private void	gch_t8();
private	int	t12io();
private	void	t12rpt();
private	void	ch_t14();
private	void	ch_t15();
private	void	ch_t16();
private	void	gch_t16();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();
private int	set_tostop();
private	int	clear_tostop();
private	int	set_xcntl();

public	void	(*realfuncs[]) () = { test1, test2, test3, test4, test5,
				      test6, test7, test8, NULL, test10,
				      NULL, test12, test13, test14,
				      test15, test16, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	twrap, 1,
	twrap, 2,
	twrap, 3,
	twrap, 4,
	twrap, 5,
	twrap, 6,
	twrap, 7,
	twrap, 8,
	test9, 9,
	twrap, 10,
	test11, 11,
	twrap, 12,
	twrap, 13,
	twrap, 14,
	twrap, 15,
	twrap, 16,
	NULL, 0
};

private	int	unsupported = 0;
private char	*tfile =	"./tcflow-t";
private char	outbuf[] = "Still crazy after all these years...\n";
private	char	inbuf[512];
private int	Stop, Start;
private short	act;
private int	actions[] = { TCOOFF, TCOON, TCIOFF, TCION };
private char	*actnames[] = { "TCOOFF", "TCOON", "TCIOFF", "TCION" };
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	input;		/* read() return	*/
private	int	caught_sig = 0;
private int	tc_ret;
private int	testfail;
private	int	buffered;

private void
sig_catch(sig)
{
	caught_sig = sig;
}


private void
do_looptcflow()
{
	int	pathok = 0;

	DBUG_ENTER("do_tcflow");

	tc_ret = tcflow(loop_fildes, actions[act]);

	PATH_FUNC_RPT(0); /* must set globok to keep cppair happy */

	DBUG_VOID_RETURN;
}

private void
do_tcflow()
{
	int	pathok = 0;

	DBUG_ENTER("do_tcflow");

	tc_ret = tcflow(tty_fildes, actions[act]);

	PATH_FUNC_RPT(0); /* must set globok to keep cppair happy */

	DBUG_VOID_RETURN;
}

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	char *msg = NULL;
	char	*cp;
	int i;

	DBUG_ENTER("prepare_tests");

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_all(DELETION, "parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
		buffered = 0;
	else
		buffered = 1;

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
		break;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}

	tty_termios_p->c_lflag |= TOSTOP;
	tty_termios_p->c_iflag |= (IXOFF|IXON);

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, "tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY in preparation");
		DBUG_VOID_RETURN;
	}

	loop_termios_p->c_iflag &= ~(IXOFF|IXON);
	loop_termios_p->c_lflag &= ~ICANON;
	loop_termios_p->c_cc[VMIN] = 1;
	loop_termios_p->c_cc[VTIME] = 0;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, "tcsetattr(TCSANOW) failed on VSX_TERMIOS_LOOP in preparation");
		DBUG_VOID_RETURN;
	}

	Start = tty_termios_p->c_cc[VSTART];
	Stop = tty_termios_p->c_cc[VSTOP];

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

	if ( !unsupported ) {
        	(void) tcflow(tty_fildes, TCOON);
		(void) close(tty_fildes);
		(void) close(loop_fildes);
	}
	(void) unlink(tfile);

	DBUG_VOID_RETURN;
}

private int
tprep(use_tty)
int use_tty;
{
	int 	fail = 0;
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("tprep");

	/* Flush all queues */

	(void) tcflush(tty_fildes, TCIOFLUSH);
	(void) tcflush(loop_fildes, TCIOFLUSH);

	/* Ensure output is turned on */

	SET_TIMEOUT(WAITTIME)

	if (tcflow(tty_fildes, TCOON) == SYSERROR)
	{
		fail = 1;
		err = errno;
	}
	else if (!use_tty) {
			if (tcflow(loop_fildes, TCOON) == SYSERROR)
			{
				fail = 2;
				err = errno;
			}
			else
				PATH_TRACE;
	}
	else
		PATH_TRACE;


	CLEAR_ALARM

	if (fail != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOON) failed on VSX_TERMIOS_%s during preparation - errno %d (%s)",
			(fail == 1?"TTY":"LOOP"), err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	
	/* Wait for any data still in transit to reach input queue */

	(void) sleep(NAP);

	/* Flush all queues again */

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( ! set_tostop(USE_TTY) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_RETURN(-1);
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if ( ! set_tostop(USE_LOOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_LOP");
		DBUG_RETURN(-1);
	} else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(6);

	DBUG_RETURN(0);
}

private int
tread(on_tty, wr_succ)
int on_tty;
int wr_succ;
{
	int 	ret, err;
	int	pathok = 0;

	DBUG_ENTER("tread");

	/* write to output and read it */

	SET_TIMEOUT(WAITTIME/2)

	errno = 0;
	ret = write( (on_tty?tty_fildes:loop_fildes), outbuf, BUFLEN);
	err = errno;

	CLEAR_ALARM

	if (buffered || wr_succ)
	{
		if (ret != BUFLEN)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("write() to VSX_TERMIOS_%s after tcflow() failed - errno %d (%s)",
				(on_tty?"TTY":"LOOP"), err, errname(err));
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
	}
	else
	{
		if (alrm_flag == 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcflow(TCOOFF) did not suspend output");
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
	}

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME/2)

	input = read( (on_tty?loop_fildes:tty_fildes), inbuf, sizeof(inbuf));
	err = errno;
	
	CLEAR_ALARM

	if (alrm_flag > 0)
		input = 0;	/* nothing to read */

	if (input < 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("read() from VSX_TERMIOS_%s failed - errno %d (%s)",
			(on_tty?"LOOP":"TTY"), err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(0);
}

private int
clear_tostop(use_tty) 
int use_tty;
{
	int	pathok = 0;
	struct termios *termios_p = use_tty ? tty_termios_p : loop_termios_p;
	int	fildes = use_tty ? tty_fildes : loop_fildes;

	DBUG_ENTER("clear_tostop");

	termios_p->c_lflag &= ~(TOSTOP);

	if (tcsetattr( fildes, TCSANOW, termios_p ) == SYSERROR)
	{
		DBUG_RETURN(0);
	} else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(1);
}

private int
set_tostop(use_tty)
int use_tty;
{
	int	pathok = 0;
	struct termios *termios_p = use_tty ? tty_termios_p : loop_termios_p;
	int	fildes = use_tty ? tty_fildes : loop_fildes;

	DBUG_ENTER("set_tostop");

	termios_p->c_lflag |= TOSTOP;

	if (tcsetattr( fildes, TCSANOW, termios_p ) == SYSERROR)
	{
		DBUG_RETURN(0);
	} else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(1);
}

private int
tprep_xcntl(use_tty)
int use_tty;
{
	int	pathok = 0;

	DBUG_ENTER("tprep_xcntl");

	globok = 0;
	if ( ! set_xcntl(tty_fildes, tty_termios_p, use_tty) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to %s IXON/IXOFF failed on VSX_TERMIOS_TTY", use_tty ? "set" : "clear");
		DBUG_RETURN(-1);
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if ( ! set_xcntl(loop_fildes, loop_termios_p, !use_tty) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to %s IXON/IXOFF failed on VSX_TERMIOS_LOOP", use_tty ? "set" : "clear");
		DBUG_RETURN(-1);
	} else
		PATH_FUNC_TRACE;
	PATH_FUNC_RPT(2);

	DBUG_RETURN(0);
}

private int
set_xcntl(fildes, termios_p, for_output)
int fildes;
struct termios *termios_p;
int for_output;
{
	int	pathok = 0;

	DBUG_ENTER("set_xcntl");

	if (tcgetattr( fildes, termios_p ) == SYSERROR)
	{
		DBUG_RETURN(0);
	} else
		PATH_TRACE;

	if (for_output)
		termios_p->c_iflag |= IXON|IXOFF;
	else
	{
		termios_p->c_iflag &= ~(IXON|IXOFF);
		termios_p->c_lflag &= ~ICANON;
		termios_p->c_cc[VMIN] = 1;
		termios_p->c_cc[VTIME] = 0;
	}

	if (tcsetattr( fildes, TCSANOW, termios_p ) == SYSERROR)
	{
		DBUG_RETURN(0);
	} else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(1);
}

private void
test1()
{
	int 	fail = 0;
	int	pathok = 0;

    	DBUG_ENTER("test1");

	/* The default on open of a terminal file is that neither */
	/* its input nor its output are suspended. */

	globok = 0;
	if ( ! set_tostop(USE_TTY) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR ||
	    tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcflush() failed during setup");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	if (write(tty_fildes, outbuf, BUFLEN) <= 0)
		fail |= 1;
	else if (write(loop_fildes, outbuf, BUFLEN) <= 0)
		fail |= 2;
	else
		PATH_TRACE;

	CLEAR_ALARM

	if (fail != 0 || alrm_flag > 0)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("write() to VSX_TERMIOS_TTY failed when newly opened");
		if (fail & 2)
			in_rpt("write() to VSX_TERMIOS_LOOP failed when newly opened");
		if (alrm_flag > 0)
			in_rpt("write() to newly opened terminal timed out");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	if (read(tty_fildes, inbuf, sizeof(inbuf)) <= 0)
		fail |= 1;
	else if (read(loop_fildes, inbuf, sizeof(inbuf)) <= 0)
		fail |= 2;
	else
		PATH_TRACE;

	CLEAR_ALARM

	if (fail != 0 || alrm_flag > 0)
	{
		xx_rpt(FAILURE);
		if (fail & 1)
			in_rpt("read() from VSX_TERMIOS_TTY failed when newly opened");
		if (fail & 2)
			in_rpt("read() from VSX_TERMIOS_LOOP failed when newly opened");
		if (alrm_flag > 0)
			in_rpt("read() from newly opened terminal timed out");
	}
	else
	{
		PATH_XS_RPT(5);
	}

	DBUG_VOID_RETURN;
}


private void
test2()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	globok = 0;
	if (tprep(USE_TTY) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	rval = tcflow(tty_fildes, TCOOFF);
	err = errno;

	CLEAR_ALARM

	if (rval != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(TCOOFF) returned %d, expected 0", rval);
		in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tread(USE_TTY, WR_FAIL) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (input > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(TCOOFF) did not stop output");
	}
	else
		PATH_TRACE;

	if (memchr((void *)inbuf, Stop, (size_t)input) != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("STOP character was sent");
	}
	else
		PATH_TRACE;

	if (memchr((void *)inbuf, Start, (size_t)input) != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("START character was sent");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int 	fail, rval, err;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	globok = 0;
	if (tprep(USE_TTY) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Turn off ouput first, then check TCOON turns it on again */

	fail = 0;
	SET_TIMEOUT(WAITTIME)

	if (tcflow(tty_fildes, TCOOFF) == SYSERROR)
	{
		fail = 1;
		err = errno;
	}
	else
		PATH_TRACE;

	CLEAR_ALARM

	if (fail != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY during setup - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	rval = tcflow(tty_fildes, TCOON);
	err = errno;

	CLEAR_ALARM

	if (rval != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(TCOON) returned %d, expected 0", rval);
		in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tread(USE_TTY, WR_SUCC) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (input < BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(TCOON) did not restart output");
	}
	else
		PATH_TRACE;

	if (memchr((void *)inbuf, Stop, (size_t)input) != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("STOP character was sent");
	}
	else
		PATH_TRACE;

	if (memchr((void *)inbuf, Start, (size_t)input) != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("START character was sent");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	globok = 0;
	if (tprep(USE_TTY) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	rval = tcflow(tty_fildes, TCIOFF);
	err = errno;

	CLEAR_ALARM

	if (rval != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(TCIOFF) returned %d, expected 0", rval);
		in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tread(USE_TTY, WR_SUCC) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (input < BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(TCIOFF) stopped output");
	}
	else
		PATH_TRACE;

	if (memchr((void *)inbuf, Stop, (size_t)input) == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("STOP character was not sent");
	}
	else
		PATH_TRACE;

	if (memchr((void *)inbuf, Start, (size_t)input) != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("START character was sent");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	globok = 0;
	if (tprep(USE_TTY) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	rval = tcflow(tty_fildes, TCION);
	err = errno;

	CLEAR_ALARM

	if (rval != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(TCION) returned %d, expected 0", rval);
		in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tread(USE_TTY, WR_SUCC) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (input < BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(TCION) stopped output");
	}
	else
		PATH_TRACE;

	if (memchr((void *)inbuf, Stop, (size_t)input) != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("STOP character was sent");
	}
	else
		PATH_TRACE;

	if (memchr((void *)inbuf, Start, (size_t)input) == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("START character was not sent");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	int pathok = 0;
	int partfail = 0;
	DBUG_ENTER("test6");

	testfail = 0;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok=0;
	do_test6(0); /* TOSTOP set */
	PATH_FUNC_TRACE;

	if(testfail != 0) {
		in_rpt("when TOSTOP bit was set");
	} else 
		PATH_TRACE;

	partfail = testfail ; 
	testfail = 0;

	globok=0;
	do_test6(1); /* TOSTOP clear */
	PATH_FUNC_TRACE;

	if(testfail != 0) {
		in_rpt("when TOSTOP bit was clear");
	} else 
		PATH_TRACE;

	if (testfail == 0 && partfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
do_test6(mode)
int mode;
{
	int 	fail, err;
	int	pathok = 0;

	DBUG_ENTER("do_test6");


	for (act = 0; act < NUM(actions); act++)
	{
		globok = 0;
		if (tprep(USE_TTY) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		if (mode) {
			globok = 0;
			if ( ! clear_tostop(USE_TTY) ) {
				xx_rpt(DELETION);
				in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
				DBUG_VOID_RETURN;
			} else
				PATH_FUNC_TRACE;
		} else
			PATH_TRACE;
		
		fail = 0;
		SET_TIMEOUT(WAITTIME)

		if (tcflow(tty_fildes,
			actions[act] == TCOON ? TCOOFF : TCOON) == SYSERROR)
		{
			fail = 1;
			err = errno;
		}
		else
			PATH_TRACE;

		CLEAR_ALARM

		if (fail != 0)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("tcflow(%s) failed on VSX_TERMIOS_TTY during setup - errno %d (%s)",
				actions[act] == TCOON ? "TCOOFF" : "TCOON",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		switch (cppair(ch_t6, NULLFN, WAITTIME, E_DEL|NOK|PATH_OK))
		{
		case E_DEL:
			DBUG_VOID_RETURN;
		case NOK:
			testfail++;	/* reported in child */
			break;
		case PATH_OK:
			PATH_TRACE;
			break;
		}

		/* When action is TCOON, a TCOOFF has previously been
		   performed, so the write() will fail on systems
		   with no output buffer.  Hence the TCOON below where
		   you might expect to see TCOOFF. */
		globok = 0;
		if (tread(USE_TTY, actions[act] == TCOON ? WR_FAIL : WR_SUCC) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		switch (actions[act])
		{
		case TCOOFF :
			if (input < BUFLEN)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcflow(TCOOFF) action was performed");
			}
			else
				PATH_TRACE;
			break;

		case TCOON :
			if (input > 0)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcflow(TCOON) action was performed");
			}
			else
				PATH_TRACE;
			break;
		default:
			PATH_TRACE;
			break;
		}

		if (memchr((void *)inbuf, Stop, (size_t)input) != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("STOP character was sent by tcflow(%s)",
				actnames[act]);
		}
		else
			PATH_TRACE;

		if (memchr((void *)inbuf, Start, (size_t)input) != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("START character was sent by tcflow(%s)",
				actnames[act]);
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_FUNC_RPT(9*NUM(actions));

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t6");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;
	caught_sig = 0;

	SET_TIMEOUT(WAITTIME/2)

	(void) tcflow(tty_fildes, actions[act]);
	if (alrm_flag == 0)
		(void) pause();

	CLEAR_ALARM

	if (caught_sig != SIGTTOU)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU not received");
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok == 3)
		DBUG_EXIT(PATH_OK);

	DBUG_VOID_RETURN;
}

private void
test7()	
{
	char 	*p;
	int 	fail, err;
	int	pathok = 0;

	DBUG_ENTER("test7");

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	for (act = 0; act < NUM(actions); act++)
	{
		globok = 0;
		if (tprep(USE_TTY) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
		
		if (actions[act] == TCOON)
		{
			fail = 0;
			SET_TIMEOUT(WAITTIME)

			if (tcflow(tty_fildes, TCOOFF) == SYSERROR)
			{
				fail = 1;
				err = errno;
			}
			else
				PATH_TRACE;

			CLEAR_ALARM

			if (fail != 0)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY during setup - errno %d (%s)",
					err, errname(err));
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;
		}
		else
		{
			PATH_TRACE; PATH_TRACE;
		}

		globok = 0;
		switch (cppair(ch_t7, NULLFN, 2 * WAITTIME, E_DEL|NOK|PATH_OK))
		{
		case E_DEL:
			DBUG_VOID_RETURN;
		case NOK:
			testfail++;	/* reported in child */
			break;
		case PATH_OK:
			PATH_TRACE;
			break;
		}

		globok = 0;
		if (tread(USE_TTY, actions[act] == TCOOFF ? WR_FAIL : WR_SUCC) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		switch (actions[act])
		{
		case TCOOFF :
			if (input > 0)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcflow(TCOOFF) action was not performed");
			}
			else
				PATH_TRACE;
			break;

		case TCOON :
			if (input < BUFLEN)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcflow(TCOON) action was not performed");
			}
			else
				PATH_TRACE;
			break;
		default:
			PATH_TRACE;
			break;
		}

		p = (char *)memchr((void *)inbuf, Stop, (size_t)input);
		if (actions[act] != TCIOFF && p != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("STOP character was sent by tcflow(%s)",
				actnames[act]);
		}
		else if (actions[act] == TCIOFF && p == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcflow(TCIOFF) action was not performed");
		}
		else
			PATH_TRACE;

		p = (char *)memchr((void *)inbuf, Start, (size_t)input);
		if (actions[act] != TCION && p != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("START character was sent by tcflow(%s)",
				actnames[act]);
		}
		else if (actions[act] == TCION && p == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcflow(TCION) action was not performed");
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(1+(8*NUM(actions)));

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int 	ret;
	sigset_t set;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t7");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;

	/* block SIGTTOU */

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTTOU);
	if (sigprocmask(SIG_BLOCK, &set, (sigset_t *)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask(SIG_BLOCK, ...) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t7, do_tcflow, WAITTIME, E_SAFAIL|E_GOTSIG);
	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(E_DEL);
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_FUNC_TRACE;

	if (tc_ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(%s) returned %d, expected 0",
			actnames[act], tc_ret);
	}
	else
		PATH_TRACE;

	/* Check if SIGTTOU is pending?  (shouldn't be) */

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok == 5)
		DBUG_EXIT(PATH_OK);

	DBUG_VOID_RETURN;
}

private void
gch_t7()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t7");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	char 	*p;
	int 	fail, err;
	int	pathok = 0;

	DBUG_ENTER("test8");

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	for (act = 0; act < NUM(actions); act++)
	{
		globok = 0;
		if (tprep(USE_TTY) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
		
		if (actions[act] == TCOON)
		{
			fail = 0;
			SET_TIMEOUT(WAITTIME)

			if (tcflow(tty_fildes, TCOOFF) == SYSERROR)
			{
				fail = 1;
				err = errno;
			}
			else
				PATH_TRACE;

			CLEAR_ALARM

			if (fail != 0)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY during setup - errno %d (%s)",
					err, errname(err));
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;
		}
		else
		{
			PATH_TRACE; PATH_TRACE;
		}

		globok = 0;
		switch (cppair(ch_t8, NULLFN, 2 * WAITTIME, E_DEL|NOK|PATH_OK))
		{
		case E_DEL:
			DBUG_VOID_RETURN;
		case NOK:
			testfail++;	/* reported in child */
			break;
		case PATH_OK:
			PATH_TRACE;
			break;
		}

		globok = 0;
		if (tread(USE_TTY, actions[act] == TCOOFF ? WR_FAIL : WR_SUCC) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		switch (actions[act])
		{
		case TCOOFF :
			if (input > 0)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcflow(TCOOFF) action was not performed");
			}
			else
				PATH_TRACE;
			break;

		case TCOON :
			if (input < BUFLEN)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcflow(TCOON) action was not performed");
			}
			else
				PATH_TRACE;
			break;
		default:
			PATH_TRACE;
			break;
		}

		p = (char *)memchr((void *)inbuf, Stop, (size_t)input);
		if (actions[act] != TCIOFF && p != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("STOP character was sent by tcflow(%s)",
				actnames[act]);
		}
		else if (actions[act] == TCIOFF && p == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcflow(TCIOFF) action was not performed");
		}
		else
			PATH_TRACE;

		p = (char *)memchr((void *)inbuf, Start, (size_t)input);
		if (actions[act] != TCION && p != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("START character was sent by tcflow(%s)",
				actnames[act]);
		}
		else if (actions[act] == TCION && p == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcflow(TCION) action was not performed");
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(1+(8*NUM(actions)));

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int 	ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t8");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t8, do_tcflow, WAITTIME, E_SAFAIL|E_GOTSIG);
	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(E_DEL);
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_FUNC_TRACE;

	if (tc_ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(%s) returned %d, expected 0",
			actnames[act], tc_ret);
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok == 4)
		DBUG_EXIT(PATH_OK);

	DBUG_VOID_RETURN;
}


private void
gch_t8()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t8");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test9");

	testfail = 0;

	if ((rval = tcflow(-1, TCION)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcflow(-1, TCION) returned %d, expected -1", rval);
	}
	else if (errno != EBADF)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcflow(-1, TCION) gave errno %d (%s), expected EBADF",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}

private void
test10()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	if ((rval = tcflow(tty_fildes, -1)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcflow(tty_fildes, -1) returned %d, expected -1", rval);
	}
	else if (errno != EINVAL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcflow(tty_fildes, -1) gave errno %d (%s), expected EINVAL",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}

private void
test11()
{
	int 	fd, rval, err;
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	fd = creat(tfile, MODEANY);
	if (fd == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file %s", tfile);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = tcflow(fd, TCION)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcflow(fd, TCION) returned %d, expected -1", rval);
		in_rpt("where fd was open to a plain file");
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcflow(fd, TCION) gave errno %d (%s), expected ENOTTY",
			err, errname(err));
		in_rpt("where fd was open to a plain file");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) close(fd);
	(void) unlink(tfile);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	DBUG_ENTER("test12");

	do_fnoeio("w", -1,  t12io, t12rpt);

	DBUG_VOID_RETURN;
}

private int
t12io(fp)
FILE *fp;
{
	globok = 1;
	return tcflow(fileno(fp), TCOOFF);
}

private
void
t12rpt(ret, err)
int ret, err;
{
	int	pathok = 0;

	DBUG_ENTER("t12rpt");

	if(ret != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("tcflow() from orphaned background process group did not give correct results");
		in_rpt("expected return: 0, actual: %d", ret);
		if (ret == -1 && err != NOERROR)
			in_rpt("expected no error. actual errno: %d(%s)",
				err, errname(err));
	} else {
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test13()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

	globok = 0;
	if (tprep(USE_LOOP) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	rval = tcflow(loop_fildes, TCOOFF);
	err = errno;

	CLEAR_ALARM

	if (rval != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(TCOOFF) returned %d, expected 0", rval);
		in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tread(USE_LOOP, WR_FAIL) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (input > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(TCOOFF) did not stop output");
	}
	else
		PATH_TRACE;

	if (memchr((void *)inbuf, Stop, (size_t)input) != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("STOP character was sent");
	}
	else
		PATH_TRACE;

	if (memchr((void *)inbuf, Start, (size_t)input) != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("START character was sent");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test14()
{
	int pathok = 0;
	int partfail = 0;
	DBUG_ENTER("test14");

	testfail = 0;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok=0;
	do_test14(0); /* TOSTOP set */
	PATH_FUNC_TRACE;

	if(testfail != 0) {
		in_rpt("when TOSTOP bit was set");
	} else 
		PATH_TRACE;

	partfail = testfail ; 
	testfail = 0;

	globok=0;
	do_test14(1); /* TOSTOP clear */
	PATH_FUNC_TRACE;

	if(testfail != 0) {
		in_rpt("when TOSTOP bit was clear");
	} else 
		PATH_TRACE;

	if (testfail == 0 && partfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
do_test14(mode)
int mode;
{
	int 	fail, err;
	int	pathok = 0;

	DBUG_ENTER("do_test14");


	for (act = 0; act < NUM(actions); act++)
	{
		globok = 0;
		if (tprep(USE_LOOP) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		if (mode) {
			globok = 0;
			if ( ! clear_tostop(USE_LOOP) ) {
				xx_rpt(DELETION);
				in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
				DBUG_VOID_RETURN;
			} else
				PATH_FUNC_TRACE;
		} else
			PATH_TRACE;
		
		if (actions[act] == TCOON)
		{
			fail = 0;
			SET_TIMEOUT(WAITTIME)

			if (tcflow(loop_fildes, TCOOFF) == SYSERROR)
			{
				fail = 1;
				err = errno;
			}
			else
				PATH_TRACE;

			CLEAR_ALARM

			if (fail != 0)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY during setup - errno %d (%s)",
					err, errname(err));
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;
		}
		else
		{
			PATH_TRACE; PATH_TRACE;
		}

		globok = 0;
		switch (cppair(ch_t14, NULLFN, WAITTIME, E_DEL|NOK|PATH_OK))
		{
		case E_DEL:
			DBUG_VOID_RETURN;
		case NOK:
			testfail++;	/* reported in child */
			break;
		case PATH_OK:
			PATH_TRACE;
			break;
		}

		globok = 0;
		if (tread(USE_LOOP, actions[act] == TCOOFF ? WR_FAIL : WR_SUCC) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		switch (actions[act])
		{
		case TCOOFF :
			if ( ! (input < BUFLEN) )
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcflow(TCOOFF) action was not performed");
			}
			else
				PATH_TRACE;
			break;

		case TCOON :
			if (! (input > 0) )
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcflow(TCOON) action was not performed");
			}
			else
				PATH_TRACE;
			break;
		default:
			PATH_TRACE;
			break;
		}

		if (memchr((void *)inbuf, Stop, (size_t)input) != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("STOP character was sent by tcflow(%s)",
				actnames[act]);
		}
		else
			PATH_TRACE;

		if (memchr((void *)inbuf, Start, (size_t)input) != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("START character was sent by tcflow(%s)",
				actnames[act]);
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_FUNC_RPT(9*NUM(actions));

	DBUG_VOID_RETURN;
}

private void
ch_t14()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t14");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;
	caught_sig = 0;

	SET_TIMEOUT(WAITTIME/2)

	(void) tcflow(loop_fildes, actions[act]);
	if (alrm_flag == 0)
		(void) pause();

	CLEAR_ALARM

	if (caught_sig == SIGTTOU)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU was received");
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok == 3)
		DBUG_EXIT(PATH_OK);

	DBUG_VOID_RETURN;
}

private void
test15()	
{
	char 	*p;
	int 	fail, err;
	int	pathok = 0;

	DBUG_ENTER("test15");

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	for (act = 0; act < NUM(actions); act++)
	{
		globok = 0;
		if (tprep(USE_LOOP) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
		
		/* clear IXON & IXOFF from read side so Start/Stop show up */
		globok = 0;
		if (tprep_xcntl(USE_LOOP) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
		
		if (actions[act] == TCOON)
		{
			fail = 0;
			SET_TIMEOUT(WAITTIME)

			if (tcflow(loop_fildes, TCOOFF) == SYSERROR)
			{
				fail = 1;
				err = errno;
			}
			else
				PATH_TRACE;

			CLEAR_ALARM

			if (fail != 0)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_LOOP during setup - errno %d (%s)",
					err, errname(err));
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;
		}
		else
		{
			PATH_TRACE; PATH_TRACE;
		}

		globok = 0;
		switch (cppair(ch_t15, NULLFN, 2 * WAITTIME, E_DEL|NOK|PATH_OK))
		{
		case E_DEL:
			DBUG_VOID_RETURN;
		case NOK:
			testfail++;	/* reported in child */
			break;
		case PATH_OK:
			PATH_TRACE;
			break;
		}

		globok = 0;
		if (tread(USE_LOOP, actions[act] == TCOOFF ? WR_FAIL : WR_SUCC) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		switch (actions[act])
		{
		case TCOOFF :
			if (input > 0)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcflow(TCOOFF) action was not performed");
			}
			else
				PATH_TRACE;
			break;

		case TCOON :
			if (input < BUFLEN)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcflow(TCOON) action was not performed");
			}
			else
				PATH_TRACE;
			break;
		default:
			PATH_TRACE;
			break;
		}

		p = (char *)memchr((void *)inbuf, Stop, (size_t)input);
		if (actions[act] != TCIOFF && p != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("STOP character was sent by tcflow(%s)",
				actnames[act]);
		}
		else if (actions[act] == TCIOFF && p == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcflow(TCIOFF) action was not performed");
		}
		else
			PATH_TRACE;

		p = (char *)memchr((void *)inbuf, Start, (size_t)input);
		if (actions[act] != TCION && p != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("START character was sent by tcflow(%s)",
				actnames[act]);
		}
		else if (actions[act] == TCION && p == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcflow(TCION) action was not performed");
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(1+(9*NUM(actions)));

	DBUG_VOID_RETURN;
}

private void
ch_t15()
{
	int 	ret;
	sigset_t set;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t15");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;

	/* block SIGTTOU */

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTTOU);
	if (sigprocmask(SIG_BLOCK, &set, (sigset_t *)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask(SIG_BLOCK, ...) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t7, do_looptcflow, WAITTIME, E_SAFAIL|E_GOTSIG);
	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(E_DEL);
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_FUNC_TRACE;

	if (tc_ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(%s) returned %d, expected 0",
			actnames[act], tc_ret);
	}
	else
		PATH_TRACE;

	/* Check if SIGTTOU is pending?  (shouldn't be) */

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok == 5)
		DBUG_EXIT(PATH_OK);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	char 	*p;
	int 	fail, err;
	int	pathok = 0;

	DBUG_ENTER("test16");

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	for (act = 0; act < NUM(actions); act++)
	{
		globok = 0;
		if (tprep(USE_LOOP) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
		
		/* clear IXON & IXOFF from read side so Start/Stop show up */
		globok = 0;
		if (tprep_xcntl(USE_LOOP) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
		
		if (actions[act] == TCOON)
		{
			fail = 0;
			SET_TIMEOUT(WAITTIME)

			if (tcflow(loop_fildes, TCOOFF) == SYSERROR)
			{
				fail = 1;
				err = errno;
			}
			else
				PATH_TRACE;

			CLEAR_ALARM

			if (fail != 0)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY during setup - errno %d (%s)",
					err, errname(err));
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;
		}
		else
		{
			PATH_TRACE; PATH_TRACE;
		}

		globok = 0;
		switch (cppair(ch_t16, NULLFN, 2 * WAITTIME, E_DEL|NOK|PATH_OK))
		{
		case E_DEL:
			DBUG_VOID_RETURN;
		case NOK:
			testfail++;	/* reported in child */
			break;
		case PATH_OK:
			PATH_TRACE;
			break;
		}

		globok = 0;
		if (tread(USE_LOOP, actions[act] == TCOOFF ? WR_FAIL : WR_SUCC) != 0)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		switch (actions[act])
		{
		case TCOOFF :
			if (input > 0)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcflow(TCOOFF) action was not performed");
			}
			else
				PATH_TRACE;
			break;

		case TCOON :
			if (input < BUFLEN)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcflow(TCOON) action was not performed");
			}
			else
				PATH_TRACE;
			break;
		default:
			PATH_TRACE;
			break;
		}

		p = (char *)memchr((void *)inbuf, Stop, (size_t)input);
		if (actions[act] != TCIOFF && p != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("STOP character was sent by tcflow(%s)",
				actnames[act]);
		}
		else if (actions[act] == TCIOFF && p == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcflow(TCIOFF) action was not performed");
		}
		else
			PATH_TRACE;

		p = (char *)memchr((void *)inbuf, Start, (size_t)input);
		if (actions[act] != TCION && p != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("START character was sent by tcflow(%s)",
				actnames[act]);
		}
		else if (actions[act] == TCION && p == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcflow(TCION) action was not performed");
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(1+(9*NUM(actions)));

	DBUG_VOID_RETURN;
}

private void
ch_t16()
{
	int 	ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t16");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(E_DEL);
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t16, do_looptcflow, WAITTIME, E_SAFAIL|E_GOTSIG);
	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(E_DEL);
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_FUNC_TRACE;

	if (tc_ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcflow(%s) returned %d, expected 0",
			actnames[act], tc_ret);
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok == 4)
		DBUG_EXIT(PATH_OK);

	DBUG_VOID_RETURN;
}


private void
gch_t16()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t16");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}
