/*
*      SCCS:  @(#)  POSIX.os/devclass/tcgetattr/tcgetattr.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/tcgetattr - T.tcgetattr Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.tcgetattr

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the tcgetattr() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	6 Dec 1988
MODIFICATIONS:
		If setpgid() is not supported, use setsid() instead to
		get a background process group.

		David Sawyer
		Tue Aug 07 11:04:48 BST 1990
		Added resilience code.

		Geoff Clare, 18 June 1991
			Add split baud rates test.

		Stuart Boutell, 21 May 1992
		Removed dependency on GTI for EBADF and ENOTTY tests

		Geoff Clare, 13 Jul 1994
			In test 3, tcgetattr() must set input rate to 0
			when split rates not supported (XPG4 mode).
			Allow for this behaviour in checkall().

		Geoff Clare, 19 Jan 1996
			Fix logic problem with testfail variable in test 3.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<dbug.h>
#include	<report.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<limits.h>
#include	<signal.h>
#include	<termios.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef tcgetattr
#endif


#define NO_TESTS	5
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)

extern cc_t PERASE;		/* declared in sv_tools/termios.c	*/
extern cc_t PKILL;
extern cc_t PINTR;
extern cc_t PQUIT;
extern cc_t PSTART;
extern cc_t PSTOP;
extern cc_t PEOF;
extern cc_t PEOL;
extern cc_t PSUSP;
extern cc_t PMIN;
extern cc_t PTIME;
extern	int	alrm_flag;

extern	speed_t baud_mask();
extern	char	*errname();

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	ch_t2();

private void	prepare_tests();
private int	tryset();
private void	clean_tests();
private void	checkall();
private void	chkset();
private void	chkclr();
private void	chk_cc();
private void	twrap();

public	void	(*realfuncs[]) () = { test1, test2, test3, test4, test5, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	 twrap, 2,
	 twrap, 3,
	 test4, 4,
	 test5, 5,
	NULL, 0
};

private	int	unsupported = 0;
private char		*tfile =	"./tcgetattr-t";
private char		*termios_speed;
private int		testfail;
private	int		tty_fildes;
private int		loop_fildes;
private	struct termios	tty_termios;
private struct termios	loop_termios;
private struct termios	*tty_termios_p = &tty_termios;
private struct termios	*loop_termios_p = &loop_termios;
private tcflag_t	supp_cflag = 0;

private struct baud_table
{
  	char *name;
	speed_t mask;
} baud[] =
{
	"B50",		B50,
	"B75",		B75,
	"B110",		B110,
	"B134",		B134,
	"B150",		B150,
	"B200",		B200,
	"B300",		B300,
	"B600",		B600,
	"B1200",	B1200,
	"B1800",	B1800,
	"B2400",	B2400,
	"B4800",	B4800,
	"B9600",	B9600,
	"B19200",	B19200,
	"B38400",	B38400,
	NULL,		B0
};

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	char *msg = NULL;
	int i;
	tcflag_t sav_cflag;

	DBUG_ENTER("prepare_tests");

	termios_speed = tet_getvar("VSX_TERMIOS_SPEED");
	if (termios_speed == NULL || *termios_speed == '\0')
	{
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, "VSX_TERMIOS_SPEED not defined");
		DBUG_VOID_RETURN;
	}

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
		break;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}

	tty_termios_p->c_iflag |= BRKINT|ICRNL|IGNBRK|IGNCR|IGNPAR;
	tty_termios_p->c_iflag |= INLCR|INPCK|ISTRIP|IXOFF|IXON|PARMRK;

	tty_termios_p->c_oflag |= OPOST;
	tty_termios_p->c_lflag |= ECHO|ECHOE|ECHOK|ECHONL;
	tty_termios_p->c_lflag |= ICANON|ISIG|NOFLSH;
	tty_termios_p->c_lflag |= TOSTOP;

	tty_termios_p->c_cc[VEOF] = PEOF;
#if (VEOF != VMIN)
	tty_termios_p->c_cc[VMIN] = PMIN;
#endif
	tty_termios_p->c_cc[VEOL] = PEOL;
#if (VEOL != VTIME)
	tty_termios_p->c_cc[VTIME] = PTIME;
#endif
	tty_termios_p->c_cc[VERASE] = PERASE;
	tty_termios_p->c_cc[VINTR] = PINTR;
	tty_termios_p->c_cc[VKILL] = PKILL;
	tty_termios_p->c_cc[VQUIT] = PQUIT;
	tty_termios_p->c_cc[VSUSP] = PSUSP;

	(void) cfsetispeed(tty_termios_p, baud_mask(termios_speed));
	(void) cfsetospeed(tty_termios_p, baud_mask(termios_speed));

	if (tcsetattr (tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
		msg = "tcsetattr() failed in preparation";

	/* Find out which c_cflag values are supported */

	if (tryset((tcflag_t)CLOCAL) != 0)
		msg = "tcsetattr() failed in preparation when trying to set CLOCAL (errno != EINVAL)";
	else if (tryset((tcflag_t)CREAD) != 0)
		msg = "tcsetattr() failed in preparation when trying to set CREAD (errno != EINVAL)";
	else if (tryset((tcflag_t)CSTOPB) != 0)
		msg = "tcsetattr() failed in preparation when trying to set CSTOPB (errno != EINVAL)";
	else if (tryset((tcflag_t)HUPCL) != 0)
		msg = "tcsetattr() failed in preparation when trying to set HUPCL (errno != EINVAL)";
	else if (tryset((tcflag_t)PARENB) != 0)
		msg = "tcsetattr() failed in preparation when trying to set PARENB (errno != EINVAL)";
	else if (tryset((tcflag_t)PARODD) != 0)
		msg = "tcsetattr() failed in preparation when trying to set PARODD (errno != EINVAL)";
	else
	{
		sav_cflag = tty_termios_p->c_cflag;
		tty_termios_p->c_cflag &= ~CSIZE;
		if (tryset((tcflag_t)CS7) != 0)
			msg = "tcsetattr() failed in preparation when trying to set CS7 (errno != EINVAL)";
		else if ((supp_cflag & CSIZE) != CS7)
			tty_termios_p->c_cflag = sav_cflag;
	}

	if (msg != NULL)
	{
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, msg);
	}

	DBUG_VOID_RETURN;
}

private int
tryset(flag)
tcflag_t flag;
{
	tcflag_t sav_cflag;

	DBUG_ENTER("tryset");

	sav_cflag = tty_termios_p->c_cflag;
	tty_termios_p->c_cflag |= flag;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) != SYSERROR)
		supp_cflag |= flag;
	else if (errno == EINVAL)	/* not supported */
		tty_termios_p->c_cflag = sav_cflag;
	else
		DBUG_RETURN(-1);

	DBUG_RETURN(0);
}
			
private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

	if (!unsupported) {
		(void) close(tty_fildes);
	}
	(void) unlink(tfile);

	DBUG_VOID_RETURN;
}


private void
test1()
{
	DBUG_ENTER("test1");

	testfail = 0;

	checkall (tty_fildes, tty_termios_p);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int err;

	DBUG_ENTER("ch_t2");

	testfail = 0;

	errno = 0;
	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		if (errno != ENOSYS)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: setpgid(0, 0) failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		(void) setsid();
	}

	checkall (tty_fildes, tty_termios_p);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	struct baud_table *baudp;
	speed_t outspeed, rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test3");

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

	testfail = 0;

	/* reset terminal settings */
	(void) tty_init(tty_fildes, tty_termios_p);
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcsetattr() to initial settings failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	outspeed = cfgetospeed(tty_termios_p); /* == VSX_TERMIOS_SPEED */

	for (baudp = baud; baudp->name != NULL; baudp++)
	{
		if (baudp->mask == outspeed)
			continue;
		
		(void) cfsetispeed(tty_termios_p, baudp->mask);
		(void) cfsetospeed(tty_termios_p, outspeed); /* just in case */
		if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
		{
			err = errno;
			if (err != EINVAL)
			{
				xx_rpt(DELETION);
				in_rpt("tcsetattr() to input baud %s failed - errno %d (%s)",
					baudp->name, err, errname(err));
				DBUG_VOID_RETURN;
			}
		}
		else
		{
			PATH_TRACE;
			break;
		}
	}

#ifdef TESTBED1
baudp->name = NULL;
pathok--;
#endif

	if (baudp->name == NULL)
	{
		DBUG_PRINTF("test3", "split rates not supported");

		PATH_TRACE; /* balances the one before "break" above */

		PATH_TRACE; /* balance the ones in the "else" block below */
		PATH_TRACE;
	}
	else
	{
		DBUG_PRINTF("test3", "input speed = %s", baudp->name);

		/* check actual baud rates returned, when unequal */

		if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("first tcgetattr() call failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

#ifdef TESTBED2
(void) cfsetospeed(tty_termios_p, outspeed+1);
#endif
		if ((rval = cfgetospeed(tty_termios_p)) != outspeed)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcgetattr() did not return actual output baud rate when input = %s, output = %s",
				baudp->name, tet_getvar("VSX_TERMIOS_SPEED"));
			in_rpt("expected %ld, observed %ld", (long) outspeed,
				(long) rval);
		}
		else
			PATH_TRACE;
	}

#ifdef TESTBED2
(void) cfsetospeed(tty_termios_p, outspeed);
#endif

	/* set input speed to output speed */

	(void) cfsetispeed(tty_termios_p, (speed_t)0);
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcsetattr() with input speed 0 failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcgetattr() failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
(void) cfsetospeed(tty_termios_p, outspeed+1);
(void) cfsetispeed(tty_termios_p, outspeed+1);
#endif

	if ((rval = cfgetospeed(tty_termios_p)) != outspeed)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcgetattr() did not return actual output baud rate when input = output");
		in_rpt("expected %ld, observed %ld", (long) outspeed,
			(long) rval);
	}
	else
		PATH_TRACE;

	rval = cfgetispeed(tty_termios_p);
	if (baudp->name != NULL)
	{
		/* split rates supported: input must be actual */

		if (rval != outspeed)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcgetattr() did not return actual input baud rate when input = output");
			in_rpt("expected %ld, observed %ld", (long) outspeed,
				(long) rval);
		}
		else
			PATH_TRACE;
	}
	else
	{
#if defined(_XOPEN_SOURCE)
		/* split rates not supported: input must be 0 */

		if (rval != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcgetattr() did not return correct input baud rate when input = output");
			in_rpt("expected 0, observed %ld", (long) rval);
		}
		else
			PATH_TRACE;
#else /* _XOPEN_SOURCE */
		/* split rates not supported: input may be 0 or actual */

		if (rval != outspeed && rval != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcgetattr() did not return correct input baud rate when input = output");
			in_rpt("expected %ld or 0, observed %ld",
				(long) outspeed, (long) rval);
		}
		else
			PATH_TRACE;
#endif /* _XOPEN_SOURCE */
	}

	if (testfail == 0)
		PATH_XS_RPT(8);

#else
	xx_rpt(NOTINUSE);
	in_rpt("no test in XPG3 mode");
#endif

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	if ((rval = tcgetattr (-1, tty_termios_p)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcgetattr(-1, ...) returned %d, expected -1", rval);
	}
	else if (errno != EBADF)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcgetattr(-1, ...) gave errno %d (%s), expected EBADF",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}

private void
test5()
{
	int 	fd, rval, err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	fd = creat(tfile, MODEANY);
	if (fd == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("could not create file %s - errno %d (%s)",
			tfile, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = tcgetattr (fd, tty_termios_p)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcgetattr(fd, ...) returned %d, expected -1", rval);
		in_rpt("where fd was open to a plain file");
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcgetattr(fd, ...) gave errno %d (%s), expected ENOTTY",
			err, errname(err));
		in_rpt("where fd was open to a plain file");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) close(fd);
	(void) unlink(tfile);

	DBUG_VOID_RETURN;
}

private void
checkall(ttyfd, termios_p)
int ttyfd;
struct termios *termios_p;
{
	int 	i, rval, err;
	int	pathok = 0;

	DBUG_ENTER("checkall");

	termios_p->c_iflag = 0;		/* input modes */
	termios_p->c_oflag = 0;		/* output modes */
	termios_p->c_cflag = 0;		/* control modes */
	termios_p->c_lflag = 0;		/* local modes */

	for (i = 0; i < NCCS; ++i)
	{
		if (i==0)
			PATH_TRACE;
		termios_p->c_cc[i] = 0;		/* control chars */
	}

	if ((rval = tcgetattr(ttyfd, termios_p)) != 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcgetattr() returned %d, expected 0", rval);
		in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (cfgetispeed(termios_p) != baud_mask(termios_speed) &&
	    cfgetispeed(termios_p) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt ("cfsetispeed(termios_p, baud_mask(termios_speed)) failed.");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (cfgetospeed(termios_p) != baud_mask(termios_speed))
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt ("cfsetospeed(termios_p, baud_mask(termios_speed)) failed.");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	chkset((int)(termios_p->c_iflag & BRKINT), "c_iflag & BRKINT");
	chkset((int)(termios_p->c_iflag & ICRNL),  "c_iflag & ICRNL");
	chkset((int)(termios_p->c_iflag & IGNBRK), "c_iflag & IGNBRK");
	chkset((int)(termios_p->c_iflag & IGNCR),  "c_iflag & IGNCR");
	chkset((int)(termios_p->c_iflag & IGNPAR), "c_iflag & IGNPAR");
	chkset((int)(termios_p->c_iflag & INLCR),  "c_iflag & INLCR");
	chkset((int)(termios_p->c_iflag & INPCK),  "c_iflag & INPCK");
	chkset((int)(termios_p->c_iflag & ISTRIP), "c_iflag & ISTRIP");
	chkset((int)(termios_p->c_iflag & IXOFF),  "c_iflag & IXOFF");
	chkset((int)(termios_p->c_iflag & IXON),   "c_iflag & IXON");
	chkset((int)(termios_p->c_iflag & PARMRK), "c_iflag & PARMRK");
	chkset((int)(termios_p->c_oflag & OPOST),  "c_oflag & OPOST");

	if (supp_cflag & CLOCAL)
		chkset((int)(termios_p->c_cflag & CLOCAL), "c_cflag & CLOCAL");
	else
		chkclr((int)(termios_p->c_cflag & CLOCAL), "c_cflag & CLOCAL");

	if (supp_cflag & CREAD)
		chkset((int)(termios_p->c_cflag & CREAD),  "c_cflag & CREAD");
	else
		chkclr((int)(termios_p->c_cflag & CREAD),  "c_cflag & CREAD");

	if (supp_cflag & CSTOPB)
		chkset((int)(termios_p->c_cflag & CSTOPB), "c_cflag & CSTOPB");
	else
		chkclr((int)(termios_p->c_cflag & CSTOPB), "c_cflag & CSTOPB");

	if (supp_cflag & HUPCL)
		chkset((int)(termios_p->c_cflag & HUPCL),  "c_cflag & HUPCL");
	else
		chkclr((int)(termios_p->c_cflag & HUPCL),  "c_cflag & HUPCL");

	if (supp_cflag & PARENB)
		chkset((int)(termios_p->c_cflag & PARENB), "c_cflag & PARENB");
	else
		chkclr((int)(termios_p->c_cflag & PARENB), "c_cflag & PARENB");

	if (supp_cflag & PARODD)
		chkset((int)(termios_p->c_cflag & PARODD), "c_cflag & PARODD");
	else
		chkclr((int)(termios_p->c_cflag & PARODD), "c_cflag & PARODD");

	if ((supp_cflag & CSIZE) == CS7)
	{
		if ((termios_p->c_cflag & CSIZE) != CS7)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("(c_cflag & CSIZE) was not set to CS7");
		}
		else
			PATH_TRACE;
	}
	else
	{
		if ((termios_p->c_cflag & CSIZE) == CS7)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("(c_cflag & CSIZE) was set to CS7, although unsupported");
		}
		else
			PATH_TRACE;
	}

	chkset((int)(termios_p->c_lflag & ECHO),   "c_lflag & ECHO");
	chkset((int)(termios_p->c_lflag & ECHOE),  "c_lflag & ECHOE");
	chkset((int)(termios_p->c_lflag & ECHOK),  "c_lflag & ECHOK");
	chkset((int)(termios_p->c_lflag & ECHONL), "c_lflag & ECHONL");
	chkset((int)(termios_p->c_lflag & ICANON), "c_lflag & ICANON");
	chkset((int)(termios_p->c_lflag & ISIG),   "c_lflag & ISIG");
	chkset((int)(termios_p->c_lflag & NOFLSH), "c_lflag & NOFLSH");
	if (sysconf(_SC_JOB_CONTROL) != -1)
		chkset((int)(termios_p->c_lflag & TOSTOP), "c_lflag & TOSTOP");

	chk_cc(termios_p->c_cc[VINTR], PINTR, "VINTR");
	chk_cc(termios_p->c_cc[VQUIT], PQUIT, "VQUIT");
	chk_cc(termios_p->c_cc[VERASE], PERASE, "VERASE");
	chk_cc(termios_p->c_cc[VKILL], PKILL, "VKILL");
	chk_cc(termios_p->c_cc[VEOF], PEOF, "VEOF");
#if (VEOF != VMIN)
	chk_cc(termios_p->c_cc[VMIN], PMIN, "VMIN");
#endif
	chk_cc(termios_p->c_cc[VEOL], PEOL, "VEOL");
#if (VEOL != VTIME)
	chk_cc(termios_p->c_cc[VTIME], PTIME, "VTIME");
#endif

	if (sysconf(_SC_JOB_CONTROL) != -1)
		chk_cc(termios_p->c_cc[VSUSP], PSUSP, "VSUSP");

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
chkset(biton, text)
int biton;
char *text;
{
	if (!biton)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("(%s) was not set", text);
	}
}

private void
chkclr(biton, text)
int biton;
char *text;
{
	if (biton)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("(%s) was set, although unsupported", text);
	}
}

private void
chk_cc(val, exp, name)
cc_t val, exp;
char *name;
{
	if (val != exp)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("c_cc[%s] is incorrect. Expected %#o, received %#o",
			  name, exp, val);
	}
}
