/*
*      SCCS:  @(#)  POSIX.os/files/tcgetpgrp/tcgetpg_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/tcgetpgrp - tcgetpg_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		tcgetpg_su - used by T.tcgetpgrp in privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.tcgetpgrp to perform privileged tests.

	  usage:
		tcgetpg_su pgrp okstr ident dbug

	  where pgrp is a process group number and okstr is the
	  value of globok passed down from the parent.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	29 June 1990
MODIFICATIONS:

		David Sawyer
		Tue Aug 07 10:53:05 BST 1990
		Updated and added resilience code.


		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <dbug.h>
#include <setprv.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef tcgetpgrp
#endif



public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	**environ;

extern	void	set_pname();
extern	char	*errname();

private	void	subtest3();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	set_pname(argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		exit(OK);
	}
	if (argc >= 5)
		vg_dbug = argv[4];
	if (argc >= 4)
		vg_ident = argv[3];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	subtest3(argv[1], argv[2]);

	exit(OK);
	/* NOTREACHED */
}

private void
subtest3(pgrpstr, okstr)
char *pgrpstr;
char *okstr;
{
	/* check that no processes have the pgrp ID passed down from parent */

	int err;
	pid_t pgrp;
	int pathok = 0;

	DBUG_ENTER("subtest3");

	if (atoi(okstr) == 1)
		PATH_TRACE;

	if (setprv(PRV_KILL) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_KILL) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pgrp = atol(pgrpstr);

	if (kill(-pgrp, 0) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("processes exist with pgrp ID returned by tcgetpgrp()");
		in_rpt("kill(-pgrp, 0) succeeded, expected ESRCH");
	}
	else if (errno != ESRCH)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("kill(-pgrp, 0) failed with errno %d (%s), expected %d (ESRCH)",
			err, errname(err), ESRCH);
	}
	else
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
