/*
*      SCCS:  @(#)  POSIX.os/devclass/tcgetpgrp/tcgetpgrp.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/tcgetpgrp - T.tcgetpgrp Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.tcgetpgrp

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the tcgetpgrp() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	13 Jan 1989
MODIFICATIONS:
		Geoff Clare, 2 May 1990
			Add test case for ENOTTY on controlling terminal
			of another session.
		
		Geoff Clare, 29 June 1990
			Add test for session with no foreground pgrp

		David Sawyer
		Tue Aug 07 10:53:52 BST 1990
		Updated and added resilience code.

		Stuart Boutell, 2 Nov 1990
			Changed execl to tet_exec

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Geoff Clare, 25 June 1991
			VSX_TCGETPGRP_SUPPORTED="N" for unsupported result.

		Geoff Clare, 15 May 1992
			Add test case for ENOTTY on non-terminal.

		Stuart Boutell, 21 May 1992
		Removed dependency on GTI for EBADF and ENOTTY tests

************************************************************************/

#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef tcgetpgrp
#endif


#ifdef TESTBED
#undef sysconf
#define sysconf(a) (-1)
#endif

#define NO_TESTS	7
#define WAITTIME	(5 * SPEEDFACTOR)
#define NOK		02
#define PATH_OK		04
#define NAP		3
#define MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();
extern	char	*ltoa();
extern	void	(*vsignal())();

extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

int	no_tests = NO_TESTS;

private char	*tfile = "./tcgetpgrp-t";
private char	*prog_su = "../tcgetpg_su";

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	ch_t2();
private void	ch1_t3();
private void	ch2_t3();
private void	pa_t3();
private void	ch_t6();
private void	ch_t7();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();

public	void	(*realfuncs[]) () = { test1, test2, test3, test4, test5,
					test6, /* not test7, */ NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 twrap, 1,
	 twrap, 2,
	 twrap, 3,
	 test4, 4,
	 twrap, 5,
	 twrap, 6,
	 test7, 7,
	NULL, 0
};

private	int	gti_unsup = 0;
private	struct termios	tty_termios, *tty_termios_p = &tty_termios;
private	struct termios	loop_termios, *loop_termios_p = &loop_termios;
private	int		tty_fildes, loop_fildes;
private	pid_t		ct_pgrp, orig_pgrp;
private	int		not_supported = 0;
private	int		testfail;


private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (gti_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	char *supp;
	char *msg = NULL;
	int i;

	DBUG_ENTER("prepare_tests");

	supp = tet_getvar("VSX_TCGETPGRP_SUPPORTED");
	if (supp == NULL || *supp == '\0')
	{
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i,
				"parameter VSX_TCGETPGRP_SUPPORTED is not set");
		DBUG_VOID_RETURN;
	}
	not_supported = (*supp == 'N' || *supp == 'n');

	if (not_supported && sysconf(_SC_JOB_CONTROL) != -1)
	{
		msg = "When _POSIX_JOB_CONTROL is defined, parameter VSX_TCGETPGRP_SUPPORTED must be Y";
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
				  &loop_fildes, loop_termios_p))
	{
	case 0:
		tty_termios_p->c_lflag &= ~TOSTOP;
		if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p)
							== SYSERROR)
			msg = "tcsetattr(TCSANOW) failed in preparation";
		break;
	case 1:
		gti_unsup = 1;
		DBUG_VOID_RETURN;
		break;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
	}

	if (msg != NULL)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
	}

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	if (!gti_unsup) {
		(void) close(tty_fildes);
		(void) close(loop_fildes);
	}
}

private void
test1()
{
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	if (not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcgetpgrp() not supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	orig_pgrp = getpgrp();

	/* Tcgetpgrp() from foreground process */
	ct_pgrp = tcgetpgrp (tty_fildes);
	if (ct_pgrp == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() from foreground failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
	}
	else if (ct_pgrp != orig_pgrp)
	{
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() returned incorrect value from foreground process");
		in_rpt("expected %ld (current process group), actual %ld",
			(long) orig_pgrp, (long) ct_pgrp);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int err;

	DBUG_ENTER("test2");

	if (not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcgetpgrp() not supported");
		DBUG_VOID_RETURN;
	}

	testfail = 0;

	orig_pgrp = tcgetpgrp(tty_fildes);
	if (orig_pgrp == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcgetpgrp() from foreground failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	/* Tcgetpgrp shall be successful if called from a background process */

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ct_pgrp = tcgetpgrp(tty_fildes);
	if (ct_pgrp != orig_pgrp)
	{
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() returned incorrect value from background process");
		in_rpt("expected %ld (foreground process group), actual %ld",
			(long) orig_pgrp, (long) ct_pgrp);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}
	
	DBUG_VOID_RETURN;
}

#if defined(_XOPEN_SOURCE) && !(TEST_XPG > 3)
private void
test3()
{
	DBUG_ENTER("test3");

	xx_rpt(NOTINUSE);
	in_rpt("no test in XPG3 mode");

	DBUG_VOID_RETURN;
}

#else /* !XPG3 mode */

private void
test3()
{
	int err;
	int pathok = 0;

	DBUG_ENTER("test3");

	if (not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcgetpgrp() not supported");
		DBUG_VOID_RETURN;
	}

	testfail = 0;

	globok = 0;
	(void) cpsig(ch1_t3, pa_t3, WAITTIME, 0);
	PATH_FUNC_TRACE;
	
	if (testfail > 0)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;
	
	errno = 0;
	ct_pgrp = tcgetpgrp(tty_fildes);
	if (ct_pgrp <= 1)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() with no foreground pgrp returned %ld, expected > 1",
			(long)ct_pgrp);
		if (ct_pgrp == SYSERROR)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (ct_pgrp == getpgrp())
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() with no foreground pgrp returned an existing process group ID (== getpgrp())");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	if (testfail == 0)
		(void) cppair(ch2_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch1_t3()
{
	DBUG_ENTER("ch1_t3");

	(void) setpgid((pid_t)0, (pid_t)0);
	(void) pause();

	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int err;
	int pathok = 0;

	DBUG_ENTER("pa_t3");

	(void) sleep(NAP);

	if (tcsetpgrp(tty_fildes, tet_child) == SYSERROR)
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("tcsetpgrp() failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (killchild(tet_child, WAITTIME) != 0)
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("killchild() failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch2_t3()
{
	char	*args[6];
	char *okstr;
	int err;

	DBUG_ENTER("ch2_t3");

	/* pass path tracing from parent to exec call */
	okstr = (globok == 1 ? "1" : "0");

	args[0] = prog_su;
	args[1] = ltoa((long)ct_pgrp);
	args[2] = okstr;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = (char *)0;

	(void) tet_exec(args[0], args, environ);
	
	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		prog_su, err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* !XPG3 mode */

private void
test4()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test4");

	if (not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcgetpgrp() not supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	if ((rval = tcgetpgrp(-1)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp(-1) returned %d, expected -1", rval);
	}
	else if (errno != EBADF)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp(-1) gave errno %d (%s), expected EBADF",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private void
test5()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	if (!not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcgetpgrp() is supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	/* Unsupported */

	if ((rval = tcgetpgrp(tty_fildes)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() returned %d, expected -1", rval);
	}
	else if (errno != ENOSYS)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() gave errno %d (%s), expected ENOSYS",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private void
test6()
{
	DBUG_ENTER("test6");

	if (not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcgetpgrp() not supported");
		DBUG_VOID_RETURN;
	}

	testfail = 0;

	(void) cppair(ch_t6, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t6");

	/* Doesn't have controlling terminal */

	(void) setsid(); /* should sever connection to control terminal */

	if ((rval = tcgetpgrp(tty_fildes)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() returned %d, expected -1", rval);
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() gave errno %d (%s), expected ENOTTY",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}

private void
test7()
{
	int 	rval, err;
	int	retval, fd;
	int	pathok = 0;

	DBUG_ENTER("test7");

	if (not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcgetpgrp() not supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	fd = creat(tfile, MODEANY);
	if (fd == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file %s", tfile);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = tcgetpgrp(fd)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() on a plain file returned %d, expected -1", rval);
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() on a plain file gave errno %d (%s), expected ENOTTY",
			err, errname(err));
	}
	else
		PATH_TRACE;
	
	(void) close(fd);
	(void) unlink(tfile);
	
	if (gti_unsup)
	{
		PATH_XS_RPT(3);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* device is not controlling terminal */

	if ((rval = tcgetpgrp(loop_fildes)) != -1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("on a non-controlling terminal");
		in_rpt("tcgetpgrp() returned %d, expected -1", rval);
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("on a non-controlling terminal");
		in_rpt("tcgetpgrp() gave errno %d (%s), expected ENOTTY",
			err, errname(err));
	}
	else
		PATH_TRACE;

	/* device is the controlling terminal of another session */

	retval = cppair(ch_t7, NULLFN, WAITTIME, NOK | PATH_OK);
	if (retval == NOK)
		testfail++;
	else if (retval & PATH_OK)
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	char	*termios_loop;
	int	rval, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t7");

	termios_loop = tet_getvar("VSX_TERMIOS_LOOP");
	/* no need to check: termios_prep() succeeded */

	(void) setsid();

	/* TERMIOS_LOOP is already open, so don't need O_NONBLOCK */
	loop_fildes = openctl(termios_loop, O_RDWR);
	if (loop_fildes < 0)
	{
		xx_rpt(DELETION);
		in_rpt("openctl() failed on VSX_TERMIOS_LOOP");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if ((rval = tcgetpgrp(tty_fildes)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("on the controlling terminal of another session");
		in_rpt("tcgetpgrp() returned %d, expected -1", rval);
		DBUG_EXIT(NOK);
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("on the controlling terminal of another session");
		in_rpt("tcgetpgrp() gave errno %d (%s), expected ENOTTY",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok == 2)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}
