/*
*      SCCS:  @(#)  POSIX.os/devclass/tcsendbrea/tcsendbrea.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/tcsendbrea - T.tcsendbrea Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.tcsendbrea

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the tcsendbreak() function, as well as the
	break-handling facilities of tcsetattr()

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	12 Jan 1989
MODIFICATIONS:
		Geoff Clare, 2 Aug 1989
			New VSX_TERMIOS_ASYNC parameter
		
		Geoff Clare, 14 Sep 1989
			Set TOSTOP and ISIG in c_lflag instead of c_iflag
		
		Geoff Clare, 5 Oct 1989
			Tidy up test4
			Keep TERMIOS_LOOP open and setup throughout
		
		Geoff Clare, 23 Nov 1989
			Add tcsetattr() ater openctl() in child
		
		Geoff Clare, 3 May 1990
			Check number of signals received in test1

		David Sawyer Tue Aug 07 10:21:48 BST 1990
			Added resilience code.

		Geoff Clare, 7 Nov 1990
			Downgrade test2 to WARNING if no SIGINT received

		Geoff Clare, 25 June 1991
			VSX_TERMIOS_ASYNC="N" for unsupported result.

		Stuart Boutell, 20 May 1992
			SIGTTOUT test now performed with TOSTOP mode set
		and clear.

		Stuart Boutell, 21 May 1992
		Removed dependency on GTI for EBADF and ENOTTY tests

		Stuart Boutell, 5 June 1992
		Added non-controlling terminal tests.

		Geoff Clare, 9-18 June 1993
		Allow EINTR error in test 9.
		Change tcflow() call to tcsendbreak() in test 13.
		Set BRKINT and clear IGNBRK on tty_fildes.

		Geoff Clare, 3 Sept 1993
		Wait for SIGINT in test 9.

		Geoff Clare, 1 Nov 1994
			Add (unused) argument to sigbreak().

		Geoff Clare, 10 Feb 1995
			Use do_fnoeio() instead of do_feio().

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<signal.h>
#include	<termios.h>
#include	<timeout.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef tcsendbreak
#endif



#define NO_TESTS	13
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define NOK		002
#define E_SAFAIL	004
#define E_GOTSIG	010
#define E_NOSIG		020
#define	PATH_OK		040
#define TEN_SEC		10
#define TWENTY_SEC	20
#define TEN_THOUSAND_MILLISECS	10000
#define REPEAT		40		/* number of zero breaks	*/
#define NAP		2		/* sleep() argument		*/
#define FUDGE		2		/* duration fudge factor	*/
#define	USE_CTTY	1		/* use a controlling tty */
#define	USE_NON_CTTY	2		/* use a non-controlling tty */

extern	char	*errname();
extern	void	do_fnoeio();


int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private	void	test9();
private	void	test10();
private	void	test11();
private	void	test12();
private	void	test13();
private void	ch_t1();
private void	ch_t4();
private void	ch_t5();
private void	ch_t6();
private void	ch_t10();
private void	ch_t11();
private void	ch_t12();
private void	gch_t5();
private void	gch_t6();
private void	gch_t11();
private void	gch_t12();
private void	pa_t1();
private void	pa_t2();
private void	pa_t3();
private void	pa_t10();
private void	t4_send();
private void	t10_send();
private	int	t13io();
private	void	t13rpt();

private void	prepare_tests();
private void	clean_tests();
private int	break_prep();
private void	do_tcsend();
private void	sendit();
private void	chkbreak();
private void	sigbreak();
private void	twrap();
private int	set_ttyflags();
private	int	clear_ttyflags();
private int	set_loopflags();
private	int	clear_loopflags();

public	void	(*realfuncs[]) () = { test1, test2, test3, test4, test5,
				      test6, NULL, NULL, test9, test10,
				      test11, test12, test13, NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	twrap, 1,
	twrap, 2,
	twrap, 3,
	twrap, 4,
	twrap, 5,
	twrap, 6,
	test7, 7,
	test8, 8,
	twrap, 9,
	twrap, 10,
	twrap, 11,
	twrap, 12,
	twrap, 13,
	NULL, 0
};

private	int	unsupported = 0;
private char	*tfile =	"./tcsendbrea-t";
private	int	tty_fildes, loop_fildes, tty_fildes_keep, loop_fildes_keep;
private	char	*termios_loop;
private	char	*termios_tty;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private int	async_unsupp;
private int	sigbreak_rcvd;
private	int	caught_sig = 0;
private	int	tcs_ret, tcs_err;
private int	testfail;
private int	pa_fail, pa_rval, pa_errno;
private	int	break_prep_mode;
private time_t	first, last;

private void
sig_catch(sig)
{
	caught_sig = sig;
}

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	int i;
	char *supp;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		tty_fildes_keep = tty_fildes;
		loop_fildes_keep = loop_fildes;
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
		/* NOTREACHED */
		break;
	default:
		{
			char *msg;
			if (alrm_flag > 0)
				msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
			else
				msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
			for (i = 1; i <= 6; i++)
				(void) tet_delete(i, msg);
			DBUG_VOID_RETURN;
		}
	}

	supp = tet_getvar("VSX_TERMIOS_ASYNC");
	if (supp == NULL || *supp == '\0')
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i,
				"parameter VSX_TERMIOS_ASYNC is not set");
		DBUG_VOID_RETURN;
	}
	else
		async_unsupp = (*supp == 'N' || *supp == 'n');

	if (async_unsupp)
		DBUG_VOID_RETURN;

	tty_termios_p->c_lflag |= TOSTOP;

	/* set these to detect breaks */
	tty_termios_p->c_iflag &= ~IGNBRK;
	tty_termios_p->c_iflag |= BRKINT;
	tty_termios_p->c_lflag |= ISIG;

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY in preparation");
		DBUG_VOID_RETURN;
	}

	/* set these to detect breaks */
	loop_termios_p->c_iflag &= ~IGNBRK;
	loop_termios_p->c_iflag |= BRKINT;
	loop_termios_p->c_lflag |= ISIG;

	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "tcsetattr(TCSANOW) failed on VSX_TERMIOS_LOOP in preparation");
		DBUG_VOID_RETURN;
	}

	termios_loop = tet_getvar("VSX_TERMIOS_LOOP");
	termios_tty = tet_getvar("VSX_TERMIOS_TTY");
	/* no need to check: termios_prep() succeeded */

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

	if (!unsupported) {
		(void) close(tty_fildes);
		(void) close(loop_fildes);
	}
	(void) unlink(tfile);

	DBUG_VOID_RETURN;
}


private int
clear_loopflags(flags) 
tcflag_t flags;
{
	int	pathok = 0;

	DBUG_ENTER("clear_loopflags");

	loop_termios_p->c_lflag &= ~(flags);
	if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		DBUG_RETURN(0);
	} else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(1);
}

private int
clear_ttyflags(flags) 
tcflag_t flags;
{
	int	pathok = 0;

	DBUG_ENTER("clear_ttyflags");

	tty_termios_p->c_lflag &= ~(flags);
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		DBUG_RETURN(0);
	} else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(1);
}

private int
set_ttyflags(flags)
tcflag_t flags;
{
	int	pathok = 0;

	DBUG_ENTER("set_ttyflags");

	tty_termios_p->c_lflag |= flags;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		DBUG_RETURN(0);
	} else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(1);
}

private int
set_loopflags(flags)
tcflag_t flags;
{
	int	pathok = 0;

	DBUG_ENTER("set_loopflags");

	loop_termios_p->c_lflag |= flags;
	if (tcsetattr(tty_fildes, TCSANOW, loop_termios_p) == SYSERROR)
	{
		DBUG_RETURN(0);
	} else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(1);
}

private void
test1()
{
	int	retval;
	int	pathok = 0;

	DBUG_ENTER("test1");

	break_prep_mode = USE_CTTY;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( ! set_ttyflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	testfail = 0;
	pa_fail = 0;

	globok = 0;
	retval = cppair(ch_t1, pa_t1, REPEAT * NAP + 3 * WAITTIME, NOK|PATH_OK);
	if (retval == NOK)
	{
		testfail++;
	}
	else if ( retval & PATH_OK)
		PATH_FUNC_TRACE;

	if (pa_fail)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsendbreak(fd, 0) on VSX_TERMIOS_TTY returned %d, expected 0",
			pa_rval);
		in_rpt("errno was set to %d (%s)", pa_errno, errname(pa_errno));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int 	i, err;
	struct 	sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	if (break_prep() != 0)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sigbreak;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);

	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGINT, ...) failed - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	/* Controlling terminal receives SIGINT on break. */
	sigbreak_rcvd = 0;

	(void) sleep(WAITTIME);	/* pause for signal */

	if (sigbreak_rcvd == 0)
	{
		xx_rpt(FAILURE);
		in_rpt("SIGINT not received");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	/*
	 * parent process will send 40 1/4 second breaks
	 * measure time elapsed between first and last
	 */

	sigbreak_rcvd = 0;
	for (i = 0; i < REPEAT; ++i)
	{
		if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("sigaction(SIGINT, ...) failed - errno %d (%s)",
				err, errname(err));
			DBUG_EXIT(NOK);
		}
		else
			PATH_TRACE;

		(void) sleep(NAP);	/* pause for signal */
	}

	if (sigbreak_rcvd < REPEAT)
	{
		xx_rpt(DELETION);
		in_rpt("only received %d SIGINT signals, expected %d",
			sigbreak_rcvd, REPEAT);
		DBUG_EXIT(NOK);
	}
	else if (last-first < TEN_SEC-FUDGE || last-first > TWENTY_SEC+FUDGE)
	{
		xx_rpt(FAILURE);
		in_rpt("BREAK length from %d calls to tcsendbreak(fd, 0) was incorrect", REPEAT);
		in_rpt("expected %d - %d secs, actual %ld secs",
			TEN_SEC, TWENTY_SEC, (long)(last-first));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok == (4+REPEAT) )
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	int 	i;
	int	pathok = 0;

	DBUG_ENTER("pa_t1");

	/* send one break to see if an interrupt is received */

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME)

	if (tcsendbreak(tty_fildes, 0) == SYSERROR)
	{
		++pa_fail;
		pa_errno = errno;
	}
	else
		PATH_TRACE;

	if (alrm_flag == 0)
		(void) tcdrain(tty_fildes);

	CLEAR_ALARM

	if (pa_fail)
		DBUG_VOID_RETURN;

	SET_TIMEOUT(TWENTY_SEC + WAITTIME * 2)

	for (i = 0; i < REPEAT && alrm_flag == 0; ++i)
	{
		if (i==0)
			PATH_TRACE;
		if (tcsendbreak(tty_fildes, 0) == SYSERROR)
		{
			++pa_fail;
			pa_errno = errno;
			break;
		}
		if (alrm_flag == 0)
			(void) tcdrain(tty_fildes);
	}

	CLEAR_ALARM

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int 	ret;
	int	pathok = 0;

	DBUG_ENTER("test2");

	break_prep_mode = USE_CTTY;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( ! set_ttyflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	testfail = 0;
	pa_fail = 0;

	globok = 0;
	ret = cppair(chkbreak, pa_t2, WAITTIME, NOK|E_NOSIG|PATH_OK);
	if (ret == NOK)
		testfail++;	/* reported in chkbreak() */
	else if (ret & E_NOSIG)
	{
		if (testfail++ == 0)
			xx_rpt(WARNING);
		in_rpt("SIGINT not received");
	}
	else if (ret & PATH_OK)
	{
		PATH_FUNC_TRACE;
	}

	if (pa_fail)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsendbreak(fd, 1) on VSX_TERMIOS_TTY returned %d, expected 0",
			pa_rval);
		in_rpt("errno was set to %d (%s)", pa_errno, errname(pa_errno));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	int	pathok = 0;

	DBUG_ENTER("pa_t2");

	/* send one break to see if an interrupt is received */

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME/2)

	if (tcsendbreak(tty_fildes, 1) == SYSERROR)
	{
		++pa_fail;
		pa_errno = errno;
	}
	else
		PATH_TRACE;

	CLEAR_ALARM

	PATH_FUNC_RPT(1); /* must set globok for cppair */

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int 	ret;
	int	pathok = 0;

	DBUG_ENTER("test3");

	break_prep_mode = USE_CTTY;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( ! set_ttyflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	testfail = 0;
	pa_fail = 0;

	globok = 0;
	ret = cppair(chkbreak, pa_t3, WAITTIME, NOK|E_NOSIG|PATH_OK);
	if (ret == NOK)
		testfail++;	/* reported in chkbreak() */
	else if (ret & E_NOSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGINT not received");
	}
	else if (ret & PATH_OK)
		PATH_FUNC_TRACE;

	if (pa_fail)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsendbreak(fd, -1) on VSX_TERMIOS_TTY returned %d, expected 0",
			pa_rval);
		in_rpt("errno was set to %d (%s)", pa_errno, errname(pa_errno));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int	pathok = 0;

	DBUG_ENTER("pa_t3");

	/* send one break to see if an interrupt is received */

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME/2)

	if (tcsendbreak(tty_fildes, -1) == SYSERROR)
	{
		++pa_fail;
		pa_errno = errno;
	}
	else
		PATH_TRACE;

	CLEAR_ALARM

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	ret;
	int	pathok = 0;
	int 	err;

	DBUG_ENTER("test4");

	break_prep_mode = USE_CTTY;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is set to 'N'");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	testfail = 0; 

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	if ( ! set_ttyflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;


	ret = cppair(ch_t4, NULLFN, 2 * WAITTIME, NOK|PATH_OK);
	if (ret & NOK) {
		testfail++;
		in_rpt("when TOSTOP flag was set");
	} else if (ret & PATH_OK) {
		PATH_TRACE;

		globok = 0;
		if ( ! clear_ttyflags(TOSTOP) ) {
			xx_rpt(DELETION);
			in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
			DBUG_VOID_RETURN;
		} else
			PATH_FUNC_TRACE;

		if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("tcflush() failed on VSX_TERMIOS_TTY - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		} else
			PATH_TRACE;

		ret = cppair(ch_t4, NULLFN, 2 * WAITTIME, NOK|PATH_OK);
		if (ret & NOK) {
			testfail++;
			in_rpt("when TOSTOP flag was clear");
		} else if (ret & PATH_OK)
			PATH_TRACE;
	}

	if (!testfail) 
		PATH_XS_RPT(8);
		

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int 	ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	caught_sig = 0;

	globok = 0;
	ret = cppair(chkbreak, t4_send, WAITTIME, NOK|E_NOSIG|PATH_OK);
	if (ret == NOK)
		testfail++;	/* already reported by chkbreak() */

	if (caught_sig != SIGTTOU)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU not received");
	}
	else
		PATH_TRACE;

	if (ret & OK)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsendbreak() action was performed (SIGINT received)");
	}
	else if (ret & PATH_OK)
		PATH_FUNC_TRACE; /* check globok from cppair */

	if (testfail == 0 && pathok == 4)
		DBUG_EXIT(PATH_OK);

	DBUG_EXIT(NOK);
}

private void
t4_send()
{
	int	pathok = 0;

	DBUG_ENTER("t4_send");

	(void) sleep(NAP);

	/* POSIX doesn't specify return value */
	(void) tcsendbreak(tty_fildes, 0);

	if (caught_sig == 0)
		(void) sleep(WAITTIME/2);	/* wait for SIGTTOU */

	PATH_FUNC_RPT(0); /* cppair requires globok to be set */

	DBUG_VOID_RETURN;
}

private void
test5()	
{
	int err;

	DBUG_ENTER("test5");

	break_prep_mode = USE_CTTY;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is set to 'N'");
		DBUG_VOID_RETURN;
	}

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}

	if ( ! set_ttyflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} 

	(void) cppair(ch_t5, NULLFN, 3 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int 	ret;
	sigset_t set;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t5");

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* block SIGTTOU */

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTTOU);
	if (sigprocmask(SIG_BLOCK, &set, (sigset_t *)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask(SIG_BLOCK, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t5, do_tcsend, 2 * WAITTIME, E_SAFAIL|E_GOTSIG);
	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_FUNC_TRACE;

	if (sigbreak_rcvd == 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsendbreak() action was not performed (SIGINT not received)");
	}
	else
		PATH_TRACE;

	/* Check if SIGTTOU is pending?  (shouldn't be) */

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
gch_t5()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t5");

#ifdef TESTBED
	DBUG_EXIT(E_GOTSIG);
#endif

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	int err;

	DBUG_ENTER("test6");

	break_prep_mode = USE_CTTY;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is set to 'N'");
		DBUG_VOID_RETURN;
	}

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}

	if ( ! set_ttyflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t6, NULLFN, 3 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	int 	ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t6");

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t6, do_tcsend, 2 * WAITTIME, E_SAFAIL|E_GOTSIG);
	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_FUNC_TRACE;

	if (sigbreak_rcvd == 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsendbreak() action was not performed (SIGINT not received)");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
gch_t6()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t6");

#ifdef TESTBED
	DBUG_EXIT(E_GOTSIG);
#endif

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test7");

	break_prep_mode = USE_CTTY;

	testfail = 0;

	if ((rval = tcsendbreak(-1, 0)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsendbreak(-1, 0) returned %d, expected -1", rval);
	}
	else if (errno != EBADF)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcsendbreak(-1, 0) gave errno %d (%s), expected EBADF",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test8()
{
	int 	fd, rval, err;
	int	pathok = 0;

	DBUG_ENTER("test8");

	break_prep_mode = USE_CTTY;

	testfail = 0;

	fd = creat(tfile, MODEANY);
	if (fd == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file %s", tfile);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = tcsendbreak(fd, 0)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsendbreak(fd, 0) returned %d, expected -1", rval);
		in_rpt("where fd was open to a plain file");
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcsendbreak(fd, 0) gave errno %d (%s), expected ENOTTY",
			err, errname(err));
		in_rpt("where fd was open to a plain file");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) close(fd);
	(void) unlink(tfile);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	int 	err;
	struct 	sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("test9");

	break_prep_mode = USE_NON_CTTY;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if ( ! set_ttyflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if ( ! set_loopflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	testfail = 0;
	pa_fail = 0;

	if (break_prep() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sigbreak;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);

	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGINT, ...) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* We should receive SIGINT */
	/* NB tcsendbreak() may get hit by the signal and give EINTR */

	sigbreak_rcvd = 0;

	SET_TIMEOUT(WAITTIME)

	errno = 0;
	if (tcsendbreak(loop_fildes, 0) == SYSERROR && errno != EINTR)
	{
		++pa_fail;
		pa_errno = errno;
	}
	else
		PATH_TRACE;

	if (alrm_flag == 0 && sigbreak_rcvd == 0)
		(void) pause();

	CLEAR_ALARM

	if (pa_fail)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsendbreak(fd, 0) returned -1 and errno %d (%s)",
			pa_errno, errname(pa_errno));
		in_rpt("expected return 0, or return -1 and errno %d (EINTR)",
			EINTR);
	}
	else
		PATH_TRACE;

	if (sigbreak_rcvd == 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsendbreak() action was not performed (SIGINT not received)");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	struct	sigaction	sig;
	int	ret;
	int	pathok = 0;
	int 	err;

	DBUG_ENTER("test10");

	break_prep_mode = USE_NON_CTTY;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is set to 'N'");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	sig.sa_handler = sigbreak;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);

	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGINT, ...) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	testfail = 0; 

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	if ( ! set_ttyflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if ( ! set_loopflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	sigbreak_rcvd = 0;
	ret = cppair(ch_t10, pa_t10, 2 * WAITTIME, NOK|PATH_OK);
	if (ret & NOK) {
		testfail++;
		in_rpt("when TOSTOP flag was set");
	} else if (ret & PATH_OK) {
		if (sigbreak_rcvd == 0) {
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcsendbreak() action was not performed (SIGINT not received)");
			in_rpt("when TOSTOP flag was set");
		} else
			PATH_TRACE;
			

		globok = 0;
		if ( ! clear_loopflags(TOSTOP) ) {
			xx_rpt(DELETION);
			in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_LOOP");
			DBUG_VOID_RETURN;
		} else
			PATH_FUNC_TRACE;

		if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("tcflush() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		} else
			PATH_TRACE;

		sigbreak_rcvd = 0;
		ret = cppair(ch_t10, pa_t10, 2 * WAITTIME, NOK|PATH_OK);
		if (ret & NOK) {
			testfail++;
			in_rpt("when TOSTOP flag was clear");
		} else if (ret & PATH_OK) {
			if (sigbreak_rcvd == 0) {
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("tcsendbreak() action was not performed (SIGINT not received)");
				in_rpt("when TOSTOP flag was clear");
			} else
				PATH_TRACE;
		}
	}

	if (!testfail) 
		PATH_XS_RPT(10);
		

	DBUG_VOID_RETURN;
}

private void
pa_t10()
{
	DBUG_ENTER("pa_t10");

	(void) sleep(WAITTIME*2); /* await interrupt */
	
	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t10");

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	caught_sig = 0;

	globok = 0;
	t10_send();
	if (caught_sig == SIGTTOU)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU was received");
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0) {
		if(pathok == 3) {
			DBUG_EXIT(PATH_OK);
		} else {
			xx_rpt(DELETION);
			in_rpt("path tracing error in ch_t10 - pathok: %d", pathok);
		}
	}

	DBUG_EXIT(NOK);
}

private void
t10_send()
{
	int	pathok = 0;

	DBUG_ENTER("t10_send");

	globok = 0;

	(void) sleep(NAP);

	/* POSIX doesn't specify return value */
	(void) tcsendbreak(loop_fildes, 0);

	if (caught_sig == 0)
		(void) sleep(WAITTIME/2);	/* wait for SIGTTOU */

	PATH_FUNC_RPT(0); /* cppair requires globok to be set */

	DBUG_VOID_RETURN;
}

private void
test11()	
{
	int err;
	int ret;
	struct	sigaction	sig;
	int	pathok = 0;


	DBUG_ENTER("test11");

	break_prep_mode = USE_NON_CTTY;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is set to 'N'");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( ! set_ttyflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( ! set_loopflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	sig.sa_handler = sigbreak;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);

	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGINT, ...) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	sigbreak_rcvd = 0;
	ret = cppair(ch_t11, pa_t10, 3 * WAITTIME, NOK|PATH_OK);
	if (ret & PATH_OK) {
		if (sigbreak_rcvd == 0) {
			xx_rpt(FAILURE);
			in_rpt("tcsendbreak() action was not performed (SIGINT not received)");
		} else
			PATH_XS_RPT(6);
	} 

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	int 	ret;
	sigset_t set;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t11");

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* block SIGTTOU */

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTTOU);
	if (sigprocmask(SIG_BLOCK, &set, (sigset_t *)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask(SIG_BLOCK, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t11, do_tcsend, 2 * WAITTIME, E_SAFAIL|E_GOTSIG);
	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0 && pathok == 4)
		DBUG_EXIT(PATH_OK);

	DBUG_EXIT(NOK);
	
}

private void
gch_t11()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t11");

#ifdef TESTBED
	DBUG_EXIT(E_GOTSIG);
#endif

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	struct	sigaction	sig;
	int err;
	int ret;
	int pathok = 0;

	DBUG_ENTER("test12");

	break_prep_mode = USE_NON_CTTY;

	if (async_unsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_TERMIOS_ASYNC is set to 'N'");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( ! set_ttyflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if ( ! set_loopflags(TOSTOP) ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	sig.sa_handler = sigbreak;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);

	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGINT, ...) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	sigbreak_rcvd = 0;
	ret = cppair(ch_t12, pa_t10, 3 * WAITTIME, NOK|PATH_OK);
	if (ret & PATH_OK) 
		PATH_TRACE;
		
	if (sigbreak_rcvd == 0) {
		xx_rpt(FAILURE);
		in_rpt("tcsendbreak() action was not performed (SIGINT not received)");
	} else
		PATH_TRACE;

	if (! (ret & NOK) )
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	int 	ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t12");

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t12, do_tcsend, 2 * WAITTIME, E_SAFAIL|E_GOTSIG);
	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0 && pathok == 4)
		DBUG_EXIT(PATH_OK);

	if (testfail)
		DBUG_EXIT(NOK);

	DBUG_VOID_RETURN;
}

private void
gch_t12()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t12");

#ifdef TESTBED
	DBUG_EXIT(E_GOTSIG);
#endif

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}

private void
test13()
{
	DBUG_ENTER("test13");

	do_fnoeio("w", -1,  t13io, t13rpt);

	DBUG_VOID_RETURN;
}

private int
t13io(fp)
FILE *fp;
{
	globok = 1;
	return tcsendbreak(fileno(fp), 0);
}

private
void
t13rpt(ret, err)
int ret, err;
{
	int	pathok = 0;

	DBUG_ENTER("t13rpt");

	if(ret != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsendbreak() from orphaned background process group did not give correct results");
		in_rpt("expected return: 0, actual: %d", ret);
		if (ret == SYSERROR)
			in_rpt("expected no error. actual errno: %d(%s)",
				err, errname(err));
	} else {
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

/*
 * Catch SIGINT and set first and last receipt times
 */

/* ARGSUSED */
private void
sigbreak(sig)
int sig;
{
	if (sigbreak_rcvd == 0)
	{
		first = time((time_t *)0);
		last = (time_t)0;
	}
	else
	{
		last = time((time_t *)0);
	}

	++sigbreak_rcvd;
}

private int
break_prep()
{
	/* open TERMIOS_LOOP as another control terminal for detecting breaks */

	DBUG_ENTER("break_prep");


	if (break_prep_mode == USE_CTTY) {

		(void) setsid();

		/* TERMIOS_LOOP is already open, so don't need O_NONBLOCK */
		loop_fildes = openctl(termios_loop, O_RDWR);
		if (loop_fildes < 0)
		{
			xx_rpt(DELETION);
			in_rpt("openctl() failed on VSX_TERMIOS_LOOP");
			DBUG_RETURN(-1);
		}
		tty_fildes = tty_fildes_keep;

		if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr(TCSANOW) failed on VSX_TERMIOS_LOOP");
			DBUG_RETURN(-1);
		}

	} else {
		loop_fildes = loop_fildes_keep;
	}

	DBUG_RETURN(0);
}

private void
do_tcsend()
{
	int 	ret;
	int	pathok = 0;

	DBUG_ENTER("do_tcsend");

	globok = 0;
	ret = cppair(chkbreak, sendit, WAITTIME, NOK|E_NOSIG|PATH_OK);
	if (ret == NOK)
		testfail++;	/* already reported by chkbreak() */
	else if (ret & E_NOSIG)
		sigbreak_rcvd = 0;
	else
		sigbreak_rcvd = 1;

	if (ret & PATH_OK)
		PATH_FUNC_TRACE;

	if (tcs_ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsendbreak() on VSX_TERMIOS_%s returned %d, expected 0",
			(break_prep_mode==USE_CTTY?"TTY":"LOOP"), tcs_ret);
		in_rpt("errno was set to %d (%s)", tcs_err, errname(tcs_err));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
sendit()
{
	int	pathok = 0;

	DBUG_ENTER("sendit");

	(void) sleep(NAP);

	if (break_prep_mode == USE_CTTY) {
		PATH_TRACE;
		tcs_ret = tcsendbreak(tty_fildes, 0);
	} else {
		PATH_TRACE;
		tcs_ret = tcsendbreak(loop_fildes, 0);
	}

	tcs_err = errno;

	PATH_FUNC_RPT(1); /* globok must be set to keep cppair happy */

	DBUG_VOID_RETURN;
}

private void
chkbreak()
{
	int 	err;
	struct sigaction sig;
	int	exit_value;
	int	pathok = 0;

	DBUG_ENTER("chkbreak");

	if (break_prep() != 0)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sigbreak;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);

	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGINT, ...) failed - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	/* Controlling terminal receives SIGINT on break. */
	sigbreak_rcvd = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	exit_value = 0;
	if (pathok == 2)
		exit_value = PATH_OK;

	if (sigbreak_rcvd == 0)
		DBUG_EXIT(exit_value | E_NOSIG);

	DBUG_EXIT(OK | exit_value);

	/* NOTREACHED */

	DBUG_VOID_RETURN;

}
