/*
*      SCCS:  @(#)  POSIX.os/devclass/tcsetattr/tcsetattr.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/tcsetattr - T.tcsetattr Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.tcsetattr

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the tcsetattr() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	6 Jan 1989
MODIFICATIONS:
		Geoff Clare, 20 Sep 1989
			Don't do TCSANOW while data is transferring
		
		Geoff Clare, 8 Dec 1989
			Add sleep(NAP) in test2.
		
		Geoff Clare, 3 May 1990
			Add sleep(NAP) in test3.

		J.A.Nave	29/06/90
			Check for XXX_prep() functions returning 1 to
			indicate GTI unsupported

		David Sawyer
		Mon Aug 06 14:59:20 BST 1990
		Added resilience code.

		M Young, 26 Nov `91
			Renumbered test 7-10 to 9-12,
			Added new FIPS tests 7 and 8
		
		Geoff Clare, 16 Apr 1992
			Test FIPS requirements in XPG4 mode also.

		Stuart Boutell, 21 May 1992
			SIGTTOUT test now performed with TOSTOP mode set
		and clear.

		Stuart Boutell, 21 May 1992
		Removed dependency on GTI for EBADF and ENOTTY tests

		Stuart Boutell, 29 May 1992
		Added checks that terminal input is not flushed when
		SIGTTOU is delivered to test 4.

		Stuart Boutell, 30 May 1992
		Added checks to deviance tests that termios structure
		was not modified.

		Geoff Clare, 9 June 1992
			Allow for systems with no terminal output queue.
			Improved flushing technique.

		Stuart Boutell, 9 June 1992
			Added non-controlling terminal tests.

		Geoff Clare, 15 June 1992
			Added tests for unsupported c_cflag values.

		Geoff Clare, 18 June 1992
			Re-align with latest FIPS (remove CS5 and CS6,
			and test for changing START and STOP characters).

		Geoff Clare, 16 July 1992
			Change end of initialisation list for speed_vals[]
			from "0, NULL" to "NULL, 0".

		Geoff Clare, 18 Aug 1992
			Test 19 does not apply in XPG3 mode.

		Geoff Clare, 24 Sept 1992
			Add tcsetattr() after tty_init() in test 18.

		Geoff Clare, 10 Feb 1995
			Use do_fnoeio() instead of do_feio().

************************************************************************/

#include <std.h>
#include <stdio.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef tcsetattr
#endif


#define NO_TESTS	19
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		3
#define MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define E_SAFAIL	02
#define E_GOTSIG	04
#define	PATH_OK		010
#define	NOK		020

#define BUFLEN		(size_t)(sizeof(outbuf)-1)

#undef strtol
extern	long	strtol();

extern	char	*errname();
extern	void	do_fnoeio();

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	ch_t4();
private void	ch_t5();
private void	ch_t6();
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
private void	ch_t12();
#endif
private void	ch_t14();
private void	ch_t15();
private void	ch_t16();
private void	gch_t5();
private void	gch_t6();
private	void	t17rpt();
private	int	t17io();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();
private int	set_tostop();
private	int	clear_tostop();

private struct {
	char *name;
	tcflag_t val;
} cflag_vals[] = {
	"CS5", CS5,
	"CS6", CS6,
	"CS7", CS7,
	"CS8", CS8,
	"CSTOPB", CSTOPB,
	"CREAD", CREAD,
	"PARENB", PARENB,
	"PARODD", PARODD,
	"HUPCL", HUPCL,
	"CLOCAL", CLOCAL,
	NULL, 0
};

private struct
{
	char *name;
	speed_t val;
} speed_vals[] = {
	"B50", B50,
	"B75", B75,
	"B110", B110,
	"B134", B134,
	"B150", B150,
	"B200", B200,
	"B300", B300,
	"B600", B600,
	"B1200", B1200,
	"B1800", B1800,
	"B2400", B2400,
	"B4800", B4800,
	"B9600", B9600,
	"B19200", B19200,
	"B38400", B38400,
	NULL, 0
};

public	void	(*realfuncs[]) () = { test1, test2, test3, test4, test5,
				      test6, test7, test8, NULL, test10,
				      NULL, test12, test13, test14, test15,
				      test16, test17, test18, test19,
				      NULL };
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	twrap, 1,
	twrap, 2,
	twrap, 3,
	twrap, 4,
	twrap, 5,
	twrap, 6,
	twrap, 7,
	twrap, 8,
	test9, 9,
	twrap, 10,
	test11, 11,
	twrap, 12,
	twrap, 13,
	twrap, 14,
	twrap, 15,
	twrap, 16,
	twrap, 17,
	twrap, 18,
	twrap, 19,
	NULL, 0
};

private	int	unsupported = 0;
private char	*tfile =	"./tcsetattr-t";
private	char	outbuf[] =	"How now, brown cow?\n";
private	char	inbuf[512];
private	int	tty_fildes, loop_fildes;
private	struct termios tty_termios, loop_termios,
		*loop_termios_p = &loop_termios,
		*tty_termios_p = &tty_termios;
private	int	caught_sig = 0;
private int	tcs_ret, tcs_err;
private int	testfail;

private int	use_tty;

private void
sig_catch(sig)
{
	caught_sig = sig;
}

private void
do_tcset()
{
	int	pathok = 0;

	DBUG_ENTER("do_tcset");

	tcs_ret = tcsetattr( (use_tty?tty_fildes:loop_fildes),
		TCSANOW, (use_tty?tty_termios_p:loop_termios_p) );
	tcs_err = errno;

	/* this is the parent func is some cppair calls so globok must be set */
	PATH_FUNC_RPT(0);

	DBUG_VOID_RETURN;
}

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (unsupported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
prepare_tests()
{
	int i;

	DBUG_ENTER("prepare_tests");

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
					  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		unsupported = 1;
		DBUG_VOID_RETURN;
	default:
		{
			char *msg;

			if (alrm_flag > 0)
				msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
			else
				msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
			for (i = 1; i <= NO_TESTS; i++) {
				if( i!=9 &&  i!=11)
					(void) tet_delete(i, msg);
			}
			DBUG_VOID_RETURN;
		}
	}

	tty_termios_p->c_lflag |= TOSTOP;
	tty_termios_p->c_lflag &= ~ICANON;
	tty_termios_p->c_cc[VMIN] = 0;
	tty_termios_p->c_cc[VTIME] = WAITTIME;

	if (tcsetattr (tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "tcsetattr(TCSANOW) failed in preparation");
	}
	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

	if (!unsupported) {
		(void) close(tty_fildes);
		(void) close(loop_fildes);
	}
	(void) unlink(tfile);

	DBUG_VOID_RETURN;
}

private int
flushem()
{
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("flushem");

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_RETURN(0);
}


private int
clear_tostop() 
{
	int	pathok = 0;

	DBUG_ENTER("clear_tostop");

	tty_termios_p->c_lflag &= ~(TOSTOP);
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		DBUG_RETURN(0);
	} else
		PATH_TRACE;

	if (!use_tty) {
		loop_termios_p->c_lflag &= ~(TOSTOP);
		if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		{
			DBUG_RETURN(0);
		} else
			PATH_TRACE;
	} else 
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(1);
}

private int
set_tostop()
{
	int	pathok = 0;

	DBUG_ENTER("set_tostop");

	tty_termios_p->c_lflag |= TOSTOP;
	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		DBUG_RETURN(0);
	} else
		PATH_TRACE;

	if (!use_tty) {
		loop_termios_p->c_lflag |= TOSTOP;
		if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		{
			DBUG_RETURN(0);
		} else
			PATH_TRACE;
	} else 
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(1);
}

private void
test1()
{
	/* TCSANOW: change is immediate, no drain or flush */

	char	*cp;
	int	buffered;
	int 	rval, err, nbytes, fail;
	int	pathok = 0;

	DBUG_ENTER("test1");
	use_tty = TRUE;

	testfail = 0;

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* form input and output queues on TERMIOS_TTY */

	SET_TIMEOUT(WAITTIME)

	fail = 0;
	if (buffered && tcflow(tty_fildes, TCOOFF) == SYSERROR)
		fail = 1;
	else if (write(tty_fildes, outbuf, BUFLEN) != BUFLEN)
		fail = 2;
	else if (write(loop_fildes, outbuf, BUFLEN) != BUFLEN)
		fail = 3;
	else
		PATH_TRACE;

	CLEAR_ALARM

	if (fail != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		if (fail == 1)
			in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY");
		else if (fail == 2)
			in_rpt("write() to VSX_TERMIOS_TTY failed");
		else
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
		(void) tcflow(tty_fildes, TCOON);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(4*NAP);

	SET_TIMEOUT(WAITTIME)

	rval = tcsetattr(tty_fildes, TCSANOW, tty_termios_p);
	err = errno;

	CLEAR_ALARM

	if (rval != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("tcsetattr(TCSANOW) on VSX_TERMIOS_TTY timed out");
		else
		{
			in_rpt("tcsetattr(TCSANOW) on VSX_TERMIOS_TTY returned %d, expected 0", rval);
			in_rpt("errno was set to %d (%s)", err, errname(err));
		}
	}
	else
		PATH_TRACE;

	if (tcflow(tty_fildes, TCOON) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOON) failed on VSX_TERMIOS_TTY");
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_VOID_RETURN;

	/* read input */

	SET_TIMEOUT(WAITTIME)
	nbytes = read(tty_fildes, inbuf, sizeof(inbuf));
	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on VSX_TERMIOS_TTY after tcsetattr(TCSANOW) timed out");
	}
	else if (nbytes != BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCSANOW) discarded input queue");
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* read output */

	SET_TIMEOUT(WAITTIME)
	nbytes = read(loop_fildes, inbuf, sizeof(inbuf));
	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on VSX_TERMIOS_LOOP after tcsetattr(TCSANOW) timed out");
	}
	else if (nbytes != BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCSANOW) discarded output queue");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(10);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* TCSADRAIN: output is drained, input is unaffected */

	int 	rval, err, nbytes, fail;
	int	pathok = 0;

	DBUG_ENTER("test2");
	use_tty = TRUE;

	testfail = 0;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	SET_TIMEOUT(WAITTIME)

	/* write to input and output */
	fail = 0;
	if (write(loop_fildes, outbuf, BUFLEN) != BUFLEN)
		fail = 1;
	else if (write(tty_fildes, outbuf, BUFLEN) != BUFLEN)
		fail = 2;
	else
		PATH_TRACE;

	CLEAR_ALARM

	if (fail != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		if (fail == 1)
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
		else
			in_rpt("write() to VSX_TERMIOS_TTY failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* set attributes and wait for output to drain */
	if ((rval = tcsetattr(tty_fildes, TCSADRAIN, tty_termios_p)) != 0)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCSADRAIN) on VSX_TERMIOS_TTY returned %d, expected 0", rval);
		in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* flush output, if any */
	(void) tcflush(tty_fildes, TCOFLUSH);

	(void) sleep(NAP);

	/* check for lost output */
	SET_TIMEOUT(WAITTIME)
	nbytes = read(loop_fildes, inbuf, sizeof(inbuf));
	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() after tcsetattr(TCSADRAIN) timed out");
	}
	else if (nbytes != BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCSADRAIN) did not drain output");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* TCSAFLUSH: output is drained, input is flushed */

	int 	rval, err, nbytes;
	int	pathok = 0;

	DBUG_ENTER("test3");
	use_tty = TRUE;

	testfail = 0;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;


	/* write input */
	SET_TIMEOUT(WAITTIME)
	rval = write(loop_fildes, outbuf, BUFLEN);
	CLEAR_ALARM

	if (rval != BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("write() to VSX_TERMIOS_LOOP timed out");
		else
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* allow input to transfer across */
	(void) sleep(NAP);

	/* write output */
	SET_TIMEOUT(WAITTIME)
	rval = write(tty_fildes, outbuf, BUFLEN);
	CLEAR_ALARM

	if (rval != BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("write() to VSX_TERMIOS_TTY timed out");
		else
			in_rpt("write() to VSX_TERMIOS_TTY failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	rval = tcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);
	err = errno;

	CLEAR_ALARM

	if (rval != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("tcsetattr(TCSAFLUSH) on VSX_TERMIOS_TTY timed out");
		else
		{
			in_rpt("tcsetattr(TCSAFLUSH) on VSX_TERMIOS_TTY returned %d, expected 0", rval);
			in_rpt("errno was set to %d (%s)", err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* flush output, if any */
	(void) tcflush(tty_fildes, TCOFLUSH);

	(void) sleep(NAP);

	/* check for lost output */
	SET_TIMEOUT(WAITTIME)
	nbytes = read(loop_fildes, inbuf, sizeof(inbuf));
	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() after tcsetattr(TCSAFLUSH) timed out");
	}
	else if (nbytes != BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCSAFLUSH) discarded output");
	}
	else
		PATH_TRACE;

	/* read input */

	SET_TIMEOUT(WAITTIME)
	nbytes = read(tty_fildes, inbuf, sizeof(inbuf));
	CLEAR_ALARM

	if (nbytes != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("tcsetattr(TCSAFLUSH) on VSX_TERMIOS_TTY timed out");
		else
			in_rpt("tcsetattr(TCSAFLUSH) did not discard input");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	pathok = 0;
	int	ret, nbytes;
	DBUG_ENTER("test4");
	use_tty = TRUE;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	/* write input */
	SET_TIMEOUT(WAITTIME);
	ret = write(loop_fildes, outbuf, BUFLEN);
	CLEAR_ALARM

	if (ret != BUFLEN)
	{	if (testfail++ == 0)
			xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("write() to VSX_TERMIOS_LOOP timed out");
		else
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* allow input to propgate */
	(void) sleep(NAP);

	testfail = 0;

	ret = cppair(ch_t4, NULLFN, WAITTIME, NOK|PATH_OK);
	if(ret & NOK) {
		testfail++;
		in_rpt("when TOSTOP flag was set");
	}
	if (ret & PATH_OK)
		PATH_TRACE;

	/* read input */
	SET_TIMEOUT(WAITTIME);
	nbytes = read(tty_fildes, inbuf, sizeof(inbuf));
	CLEAR_ALARM

	if (nbytes != BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCAFLUSH) discarded input when interrupted");
		in_rpt("when TOSTOP flag was set");
	} else
		PATH_TRACE;

/* Now repeat with TOSTOP clear */
	globok = 0;
	if ( ! clear_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	/* write input */
	SET_TIMEOUT(WAITTIME);
	ret = write(loop_fildes, outbuf, BUFLEN);
	CLEAR_ALARM


	if (ret != BUFLEN)
	{	if (testfail++ == 0)
			xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("write() to VSX_TERMIOS_LOOP timed out");
			else
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* allow input to propgate */
	(void) sleep(NAP);

	ret = cppair(ch_t4, NULLFN, WAITTIME, NOK|PATH_OK);
	if (ret & NOK) {
		testfail++;
		in_rpt("when TOSTOP flag was clear");
	}
	if (ret & PATH_OK)
		PATH_TRACE;

	/* read input */
	SET_TIMEOUT(WAITTIME);
	nbytes = read(tty_fildes, inbuf, sizeof(inbuf));
	CLEAR_ALARM

	if (nbytes != BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCAFLUSH) discarded input when interrupted");
		in_rpt("when TOSTOP flag was clear");
	} else
		PATH_TRACE;
 
	if (!testfail)
		PATH_XS_RPT(11);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
#ifdef TEST4
	sig.sa_handler = SIG_IGN;
#endif
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	SET_TIMEOUT(WAITTIME/2);

	tty_termios_p->c_oflag |= OPOST;
	(void) tcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);
	/* return value and errno are tested in a separate test */
	if (alrm_flag == 0)
		(void) pause();

	CLEAR_ALARM

	if (caught_sig != SIGTTOU)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU not received");
	}
	else
		PATH_TRACE;

	(void) tcgetattr(tty_fildes, tty_termios_p);

	if (tty_termios_p->c_oflag & OPOST)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr() action was performed");
	}
	else
		PATH_TRACE;

		
	if(testfail || pathok != 4)
		DBUG_EXIT(NOK);

	DBUG_EXIT(PATH_OK);
}

private void
test5()
{
	DBUG_ENTER("test5");
	use_tty = TRUE;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	/* Make sure OPOST is off (may be on if previous test failed) */
	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	}

	if (tty_termios_p->c_oflag & OPOST)
	{
		tty_termios_p->c_oflag &= ~OPOST;
		if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY during setup");
			DBUG_VOID_RETURN;
		}
	}

	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t5, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int 	ret;
	sigset_t set;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t5");

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* block SIGTTOU */

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTTOU);
	if (sigprocmask(SIG_BLOCK, &set, (sigset_t *)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask(SIG_BLOCK, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_oflag |= OPOST;

	globok = 0;
	ret = cppair(gch_t5, do_tcset, WAITTIME, E_SAFAIL|E_GOTSIG);
	PATH_FUNC_TRACE;

	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_TRACE;

	if (tcs_ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCSANOW) on VSX_TERMIOS_TTY returned %d, expected 0", tcs_ret);
		in_rpt("errno was set to %d (%s)", tcs_err, errname(tcs_err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) tcgetattr(tty_fildes, tty_termios_p);

	if (!(tty_termios_p->c_oflag & OPOST))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr() action was not performed");
	}
	else
		PATH_TRACE;

	/* Check if SIGTTOU is pending?  (shouldn't be) */

	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
gch_t5()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t5");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	DBUG_ENTER("test6");
	use_tty = TRUE;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	/* Make sure OPOST is off (will be on if previous test succeeded) */
	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	}
	if (tty_termios_p->c_oflag & OPOST)
	{
		tty_termios_p->c_oflag &= ~OPOST;
		if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY during setup");
			DBUG_VOID_RETURN;
		}
	}

	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t6, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	int 	ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t6");

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_oflag |= OPOST;

	globok = 0;
	ret = cppair(gch_t6, do_tcset, WAITTIME, E_SAFAIL|E_GOTSIG);
	PATH_FUNC_TRACE;

	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_TRACE;

	if (tcs_ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCSANOW) on VSX_TERMIOS_TTY returned %d, expected 0", tcs_ret);
		in_rpt("errno was set to %d (%s)", tcs_err, errname(tcs_err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) tcgetattr(tty_fildes, tty_termios_p);

	if (!(tty_termios_p->c_oflag & OPOST))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr() action was not performed");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
gch_t6()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t6");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}

private void
test7()
{
#if 0 /* TEST_FIPS || TEST_XPG > 3 */
	int	pathok = 0;
#endif /* TEST_FIPS */
	DBUG_ENTER("test7");

#if 0 /* TEST_FIPS || TEST_XPG > 3 */
	use_tty = TRUE;
	testfail = 0;

	globok = 0;
	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tty_termios_p->c_cc[VSTART] = 'A';
	tty_termios_p->c_cc[VSTOP] = 'B';

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCSANOW) returned non zero when VSTART and VSTOP changed");
	}
	else
		PATH_TRACE;

	(void) tcgetattr(tty_fildes, tty_termios_p);

	if ((tty_termios_p->c_cc[VSTART] != 'A') ||
		(tty_termios_p->c_cc[VSTOP] != 'B'))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr() did not change START and STOP characters");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);
#else
	xx_rpt(NOTINUSE);
	/* in_rpt("FIPS or XPG4 mode not selected"); */
#endif /* TEST_FIPS */
	DBUG_VOID_RETURN;
}

private void
test8()
{
#if TEST_FIPS || TEST_XPG > 3
	int	pathok = 0;
static	struct	test_args {
			tcflag_t mask;
			tcflag_t value;
			char *fail_message;
		} test_args[] = {
	CSIZE,	CS8,	"CSIZE of CS8",
	CSIZE,	CS7,	"CSIZE of CS7",
	CSTOPB,	CSTOPB,	"CSTOPB set",
	CSTOPB,	0,	"CSTOPB clear",
	CREAD,	CREAD,	"CREAD set",
	CREAD,	0,	"CREAD clear",
	HUPCL,	HUPCL,	"HUPCL set",
	HUPCL,	0,	"HUPCL clear",
	PARODD,	PARODD,	"PARODD set",
	PARODD,	0,	"PARODD clear",
	PARENB,	PARENB,	"PARENB set",
	PARENB,	0,	"PARENB clear",
		};
	struct	test_args *ta_p;
	int	i, n_args;
#endif /* TEST_FIPS */
	DBUG_ENTER("test8");
#if TEST_FIPS || TEST_XPG > 3
	use_tty = TRUE;
	testfail = 0;
	n_args = sizeof(test_args) / sizeof(struct test_args);

	globok = 0;
	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for( i = 0, ta_p = test_args; i++ < n_args; ta_p++)
	{
		*loop_termios_p = *tty_termios_p;
#ifndef TESTBED
		loop_termios_p->c_cflag &= ~(ta_p->mask);
		loop_termios_p->c_cflag |= ta_p->value;
#endif

		if (tcsetattr(tty_fildes, TCSANOW, loop_termios_p) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt(ta_p->fail_message);
			in_rpt("tcsetattr(TCSANOW) returned non zero when c_cflag had %s",
				ta_p->fail_message);
			continue;
		}
		else
			PATH_TRACE;

		(void) tcgetattr(tty_fildes, loop_termios_p);

		if ( (loop_termios_p->c_cflag & ta_p->mask) != ta_p->value )
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("tcsetattr() did not change setting correctly when c_cflag had %s",
				ta_p->fail_message);
		}
		else
			PATH_TRACE;
	}
	if (testfail == 0)
		PATH_XS_RPT(2 + 2 * n_args);
#else
	xx_rpt(NOTINUSE);
	in_rpt("FIPS or XPG4 mode not selected");
#endif /* TEST_FIPS */
	DBUG_VOID_RETURN;
}

private void
test9()
{
	int 	rval, err;
	static struct termios	t9_termios, *t9_termios_p = &t9_termios;
	int	pathok = 0;

	DBUG_ENTER("test9");
	use_tty = TRUE;

	testfail = 0;

	if (unsupported) {
		tty_termios = t9_termios;
	} else {
		t9_termios = tty_termios;
	}

	if ((rval = tcsetattr(-1, TCSANOW, t9_termios_p)) != -1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		xx_rpt(FAILURE);
		in_rpt("tcsetattr(-1, TCSANOW, ...) returned %d, expected -1", rval);
	}
	else if (errno != EBADF)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(-1, TCSANOW, ...) gave errno %d (%s), expected EBADF",
			err, errname(err));
	}
	else
		PATH_TRACE;
#ifdef TEST_NOMOD
	t9_termios_p->c_oflag ++;
#endif

	if (memcmp( (void *)t9_termios_p, (void *)tty_termios_p,
	    sizeof(struct termios)) != 0) {
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(-1, TCSANOW, ...) altered the termios structure");
	} else
			PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);
	DBUG_VOID_RETURN;
}

private void
test10()
{
	int 	rval, err;
	static struct termios	t10_termios;
	int	pathok = 0;

	DBUG_ENTER("test10");
	use_tty = TRUE;

	testfail = 0;

	/* Make sure OPOST is off (will be on if previous test succeeded) */
	if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	}
	if (tty_termios_p->c_oflag & OPOST)
	{
		tty_termios_p->c_oflag &= ~OPOST;
		if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY during setup");
			DBUG_VOID_RETURN;
		}
	}

	tty_termios_p->c_oflag |= OPOST;

	t10_termios = tty_termios;

	if ((rval = tcsetattr(tty_fildes,TCSANOW+TCSADRAIN+TCSAFLUSH,
		tty_termios_p)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsetattr(-1, TCSANOW+TCSADRAIN+TCSAFLUSH, ...) returned %d, expected -1", rval);
	}
	else if (errno != EINVAL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcsetattr(-1, TCSANOW+TCSADRAIN+TCSAFLUSH, ...) gave errno %d (%s), expected EINVAL",
			err, errname(err));
	}
	else
		PATH_TRACE;
#ifdef TEST_NOMOD
	t10_termios.c_oflag ++;
#endif

	if (memcmp( (void *)&t10_termios, (void *)tty_termios_p,
	    sizeof(struct termios)) != 0) {
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(fd, TCSANOW, ...) altered the termios structure");
	} else
			PATH_TRACE;

	(void) tcgetattr(tty_fildes, tty_termios_p);

	if (tty_termios_p->c_oflag & OPOST)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr() action was performed");
	}
	else
		PATH_TRACE;

	if (testfail==0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test11()
{
	int 	fd, rval, err;
	static	struct termios	t11_termios, *t11_termios_p = &t11_termios;
	int	pathok = 0;

	DBUG_ENTER("test11");
	use_tty = TRUE;

	testfail = 0;

	if (unsupported) {
		tty_termios = t11_termios;
	} else {
		t11_termios = tty_termios;
	}

	fd = creat(tfile, MODEANY);
	if (fd == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file %s", tfile);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = tcsetattr(fd, TCSANOW, tty_termios_p)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsetattr(fd, TCSANOW, ...) returned %d, expected -1", rval);
		in_rpt("where fd was open to a plain file");
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcsetattr(fd, TCSANOW, ...) gave errno %d (%s), expected ENOTTY",
			err, errname(err));
		in_rpt("where fd was open to a plain file");
	}
	else
		PATH_TRACE;

#ifdef TEST_NOMOD
	t11_termios_p->c_oflag ++;
#endif

	if (memcmp( (void *)t11_termios_p, (void *)tty_termios_p,
	    sizeof(struct termios)) != 0) {
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(fd, TCSANOW, ...) altered the termios structure");
	} else
			PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	(void) close(fd);
	(void) unlink(tfile);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	DBUG_ENTER("test12");
	use_tty = TRUE;

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t12, NULLFN, WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
	in_rpt("no test in XPG3 mode");
#endif

	DBUG_VOID_RETURN;
}

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
private void
ch_t12()
{
	struct termios	t12_termios;
	struct sigaction sig;
	int	rval, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t12");

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	caught_sig = 0;

	tty_termios_p->c_oflag |= OPOST;

	t12_termios = tty_termios;

	rval = tcsetattr(tty_fildes, TCSANOW, tty_termios_p);
	err = errno;

	if (caught_sig != SIGTTOU)
	{
		xx_rpt(DELETION);
		in_rpt("SIGTTOU not received");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (rval != -1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(fd, TCSANOW, ...) returned %d, expected -1", rval);
		in_rpt("when SIGTTOU was generated");
	}
	else if (err != EINTR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(fd, TCSANOW, ...) gave errno %d (%s), expected EINTR",
			err, errname(err));
		in_rpt("when SIGTTOU was generated");
	}
	else
		PATH_TRACE;

#ifdef TEST_NOMOD
	t12_termios.c_oflag ++;
#endif

	if (memcmp( (void *)&t12_termios, (void *)tty_termios_p,
	    sizeof(struct termios)) != 0) {
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(fd, TCSANOW, ...) altered the termios structure");
	} else
			PATH_TRACE;

	(void) tcgetattr(tty_fildes, tty_termios_p);

	if (tty_termios_p->c_oflag & OPOST)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr() action was performed");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}
#endif

private void
test13()
{
	/* TCSANOW: change is immediate, no drain or flush */

	char	*cp;
	int	buffered;
	int 	rval, err, nbytes, fail;
	int	pathok = 0;

	DBUG_ENTER("test13");
	use_tty = FALSE;

	testfail = 0;

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	if (tcgetattr(loop_fildes, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* form input and output queues on TERMIOS_LOOP */

	SET_TIMEOUT(WAITTIME)

	fail = 0;
	if (buffered && tcflow(loop_fildes, TCOOFF) == SYSERROR)
		fail = 1;
	else if (write(tty_fildes, outbuf, BUFLEN) != BUFLEN)
		fail = 2;
	else if (write(loop_fildes, outbuf, BUFLEN) != BUFLEN)
		fail = 3;
	else
		PATH_TRACE;

	CLEAR_ALARM

	if (fail != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		if (fail == 1)
			in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_LOOP");
		else if (fail == 2)
			in_rpt("write() to VSX_TERMIOS_TTY failed");
		else
			in_rpt("write() to VSX_TERMIOS_LOOP failed");
		(void) tcflow(loop_fildes, TCOON);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(4*NAP);

	SET_TIMEOUT(WAITTIME)

	rval = tcsetattr(loop_fildes, TCSANOW, loop_termios_p);
	err = errno;

	CLEAR_ALARM

	if (rval != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("tcsetattr(TCSANOW) on VSX_TERMIOS_LOOP timed out");
		else
		{
			in_rpt("tcsetattr(TCSANOW) on VSX_TERMIOS_LOOP returned %d, expected 0", rval);
			in_rpt("errno was set to %d (%s)", err, errname(err));
		}
	}
	else
		PATH_TRACE;

	if (tcflow(loop_fildes, TCOON) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOON) failed on VSX_TERMIOS_LOOP");
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_VOID_RETURN;

	/* read input */

	SET_TIMEOUT(WAITTIME)
	nbytes = read(loop_fildes, inbuf, sizeof(inbuf));
	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on VSX_TERMIOS_LOOP after tcsetattr(TCSANOW) timed out");
	}
	else if (nbytes != BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCSANOW) discarded input queue");
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* read output */

	SET_TIMEOUT(WAITTIME)
	nbytes = read(tty_fildes, inbuf, sizeof(inbuf));
	CLEAR_ALARM

	if (alrm_flag > 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on VSX_TERMIOS_TTY after tcsetattr(TCSANOW) timed out");
	}
	else if (nbytes != BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCSANOW) discarded output queue");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(10);

	DBUG_VOID_RETURN;
}

private void
test14()
{
	int	pathok = 0;
	int	ret, nbytes;
	DBUG_ENTER("test14");
	use_tty = FALSE;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	globok = 0;
	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	/* write input */
	SET_TIMEOUT(WAITTIME);
	ret = write(tty_fildes, outbuf, BUFLEN);
	CLEAR_ALARM

	if (ret != BUFLEN)
	{	if (testfail++ == 0)
			xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("write() to VSX_TERMIOS_TTY timed out");
		else
			in_rpt("write() to VSX_TERMIOS_TTY failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* allow input to propgate */
	(void) sleep(NAP);

	testfail = 0;

	ret = cppair(ch_t14, NULLFN, WAITTIME, NOK|PATH_OK);
	if(ret & NOK) {
		testfail++;
		in_rpt("when TOSTOP flag was set");
	}
	if (ret & PATH_OK)
		PATH_TRACE;

	/* read input */
	SET_TIMEOUT(WAITTIME);
	nbytes = read(loop_fildes, inbuf, sizeof(inbuf));
	CLEAR_ALARM

	if (nbytes == BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCAFLUSH) did not discard input when interrupted");
		in_rpt("when TOSTOP flag was set");
	} else
		PATH_TRACE;

/* Now repeat with TOSTOP clear */
	globok = 0;
	if ( ! clear_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to clear TOSTOP failed on VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	globok = 0;
	if (flushem() != 0)
	{
		DBUG_VOID_RETURN;
	} else
		PATH_FUNC_TRACE;

	/* write input */
	SET_TIMEOUT(WAITTIME);
	ret = write(tty_fildes, outbuf, BUFLEN);
	CLEAR_ALARM


	if (ret != BUFLEN)
	{	if (testfail++ == 0)
			xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("write() to VSX_TERMIOS_TTY timed out");
			else
			in_rpt("write() to VSX_TERMIOS_TTY failed");
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	/* allow input to propgate */
	(void) sleep(NAP);

	ret = cppair(ch_t14, NULLFN, WAITTIME, NOK|PATH_OK);
	if (ret & NOK) {
		testfail++;
		in_rpt("when TOSTOP flag was clear");
	}
	if (ret & PATH_OK)
		PATH_TRACE;

	/* read input */
	SET_TIMEOUT(WAITTIME);
	nbytes = read(loop_fildes, inbuf, sizeof(inbuf));
	CLEAR_ALARM

	if (nbytes == BUFLEN)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCAFLUSH) did not discard input when interrupted");
		in_rpt("when TOSTOP flag was clear");
	} else
		PATH_TRACE;
 
	if (!testfail)
		PATH_XS_RPT(11);

	DBUG_VOID_RETURN;
}

private void
ch_t14()
{
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t14");

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
#ifdef TEST4
	sig.sa_handler = SIG_IGN;
#endif
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	caught_sig = 0;

	SET_TIMEOUT(WAITTIME/2);

	loop_termios_p->c_oflag |= OPOST;
	(void) tcsetattr(loop_fildes, TCSAFLUSH, loop_termios_p);
	/* return value and errno are tested in a separate test */
	if (alrm_flag == 0)
		(void) pause();

	CLEAR_ALARM

	if (caught_sig == SIGTTOU)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU was received");
	}
	else
		PATH_TRACE;

	(void) tcgetattr(loop_fildes, loop_termios_p);

	if (! (loop_termios_p->c_oflag & OPOST) )
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr() action was not performed");
	}
	else
		PATH_TRACE;

		
	if(testfail || pathok != 4)
		DBUG_EXIT(NOK);

	DBUG_EXIT(PATH_OK);
}

private void
test15()
{
	DBUG_ENTER("test15");
	use_tty = FALSE;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	/* Make sure OPOST is off (may be on if previous test failed) */
	if (tcgetattr(loop_fildes, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_LOOP during setup");
		DBUG_VOID_RETURN;
	}

	if (loop_termios_p->c_oflag & OPOST)
	{
		loop_termios_p->c_oflag &= ~OPOST;
		if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr(TCSANOW) failed on VSX_TERMIOS_LOOP during setup");
			DBUG_VOID_RETURN;
		}
	}

	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t15, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t15()
{
	int 	ret;
	sigset_t set;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t15");

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* block SIGTTOU */

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTTOU);
	if (sigprocmask(SIG_BLOCK, &set, (sigset_t *)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask(SIG_BLOCK, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_oflag |= OPOST;

	globok = 0;
	ret = cppair(gch_t5, do_tcset, WAITTIME, E_SAFAIL|E_GOTSIG);
	PATH_FUNC_TRACE;

	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_TRACE;

	if (tcs_ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCSANOW) on VSX_TERMIOS_LOOP returned %d, expected 0", tcs_ret);
		in_rpt("errno was set to %d (%s)", tcs_err, errname(tcs_err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) tcgetattr(loop_fildes, loop_termios_p);

	if (!(loop_termios_p->c_oflag & OPOST))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr() action was not performed");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	DBUG_ENTER("test16");
	use_tty = FALSE;

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	/* Make sure OPOST is off (will be on if previous test succeeded) */
	if (tcgetattr(loop_fildes, loop_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcgetattr() failed on VSX_TERMIOS_LOOP during setup");
		DBUG_VOID_RETURN;
	}
	if (loop_termios_p->c_oflag & OPOST)
	{
		loop_termios_p->c_oflag &= ~OPOST;
		if (tcsetattr(loop_fildes, TCSANOW, loop_termios_p) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("tcsetattr(TCSANOW) failed on VSX_TERMIOS_LOOP during setup");
			DBUG_VOID_RETURN;
		}
	}

	if ( ! set_tostop() ) {
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) to set TOSTOP failed on VSX_TERMIOS_TTY during setup");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t16, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t16()
{
	int 	ret;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t16");

	testfail = 0;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	loop_termios_p->c_oflag |= OPOST;

	globok = 0;
	ret = cppair(gch_t6, do_tcset, WAITTIME, E_SAFAIL|E_GOTSIG);
	PATH_FUNC_TRACE;

	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_TRACE;

	if (tcs_ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr(TCSANOW) on VSX_TERMIOS_LOOP returned %d, expected 0", tcs_ret);
		in_rpt("errno was set to %d (%s)", tcs_err, errname(tcs_err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) tcgetattr(loop_fildes, loop_termios_p);

	if (!(loop_termios_p->c_oflag & OPOST))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetattr() action was not performed");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test17()
{
	DBUG_ENTER("test17");
	use_tty = FALSE;

	do_fnoeio("w", -1,  t17io, t17rpt);

	DBUG_VOID_RETURN;
}

private int
t17io(fp)
FILE *fp;
{
	globok = 1;
	return tcsetattr(fileno(fp), TCSANOW, tty_termios_p);
}

private void
t17rpt(ret, err)
int ret, err;
{
	int	pathok = 0;

	DBUG_ENTER("t17rpt");

	if (ret != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsetattr() from orphaned background process group did not give correct results");
		in_rpt("expected return: 0, acutal: %d", ret);
		if (ret == -1 && err != NOERROR)
			in_rpt("expected no error. actual errno: %d(%s)",
				err, errname(err));
	} else {
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private int
set_cflag(param, pval)
char *param;
char *pval;
{
	int i;
	int clear = 0;
	int speed = 0;
	tcflag_t cflag_val;
	speed_t speed_val;
	int pathok = 0;

	DBUG_ENTER("test18");

	/* String starting with ~ means clear the specified bits */
	if (*pval == '~')
	{
		clear = 1;
		pval++;
	}

	if (*pval == 'B')
	{
		speed = 1;
		if (clear)
		{
			xx_rpt(DELETION);
			in_rpt("Use of ~ with speeds is not allowed in parameter %s", param);
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;

		for (i = 0; speed_vals[i].name != NULL; i++)
		{
			if (strcmp(speed_vals[i].name, pval) == 0)
			{
				speed_val = speed_vals[i].val;
				break;
			}
		}
		if (speed_vals[i].name == NULL)
		{
			xx_rpt(DELETION);
			in_rpt("invalid speed in parameter %s", param);
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
	}
	else if (*pval >= '0' && *pval <= '9')
	{
		PATH_TRACE;
		cflag_val = strtol(pval, (char **)NULL, 0);
		if (cflag_val == 0)
		{
			xx_rpt(DELETION);
			in_rpt("parameter %s has an invalid value", param);
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		for (i = 0; cflag_vals[i].name != NULL; i++)
		{
			if (strcmp(cflag_vals[i].name, pval) == 0)
			{
				cflag_val = cflag_vals[i].val;
				break;
			}
		}
		if (cflag_vals[i].name == NULL)
		{
			xx_rpt(DELETION);
			in_rpt("unrecognised string in parameter %s", param);
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
	}

	if (speed)
	{
		if (cfsetispeed(tty_termios_p, speed_val) == SYSERROR ||
		    cfsetospeed(tty_termios_p, speed_val) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("invalid speed in parameter %s", param);
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
	}
	else if (cflag_val != CSIZE &&
		(cflag_val == CS5 || cflag_val == CS6 ||
		 cflag_val == CS7 || cflag_val == CS8))
	{
		if (clear)
		{
			xx_rpt(DELETION);
			in_rpt("Use of ~CSn is not allowed in parameter %s", param);
			DBUG_RETURN(-1);
		}
		else
			PATH_TRACE;
		tty_termios_p->c_cflag &= ~CSIZE;
		tty_termios_p->c_cflag |= cflag_val;
	}
	else
	{
		PATH_TRACE;
		if (clear)
			tty_termios_p->c_cflag &= ~cflag_val;
		else
			tty_termios_p->c_cflag |= cflag_val;
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

private void
test18()
{
	char *unsupported_cflag;
	int rval, err;
	int pathok = 0;
	static struct termios	t18_termios;

	DBUG_ENTER("test18");

	unsupported_cflag = tet_getvar("VSX_UNSUPPORTED_CFLAG");
	if (unsupported_cflag == NULL || *unsupported_cflag == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_UNSUPPORTED_CFLAG is not set");
		DBUG_VOID_RETURN;
	}
	else if (strcmp(unsupported_cflag, "none") == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_UNSUPPORTED_CFLAG is set to \"none\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcsetattr(tty_fildes, TCSANOW, tty_termios_p) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("tcsetattr(TCSANOW) failed on VSX_TERMIOS_TTY");
		in_rpt("when trying to restore initial settings");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (set_cflag("VSX_UNSUPPORTED_CFLAG", unsupported_cflag) != 0)
	{
		DBUG_VOID_RETURN; /* reported in set_cflag() */
	}
	else
		PATH_FUNC_TRACE;

	t18_termios = tty_termios;

	errno = 0;
	rval = tcsetattr(tty_fildes, TCSANOW, tty_termios_p);
	if (rval != -1 || errno != EINVAL)
	{
		err = errno;
		xx_rpt(FAILURE);
		if (rval != -1)
			in_rpt("tcsetattr() returned %d, expected -1", rval);
		else if (errno != EINVAL)
			in_rpt("tcsetattr() gave errno %d (%s), expected EINVAL",
				err, errname(err));
	}
	else if (memcmp( (void *)&t18_termios, (void *)tty_termios_p,
			sizeof(struct termios)) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsetattr() altered the termios structure");
	}
	else
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test19()
{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	char *supported_cflag;
	char *unsupported_cflag;
	tcflag_t exp_cflag;
	speed_t exp_speed;
	int rval, err;
	int pathok = 0;

	DBUG_ENTER("test19");

	supported_cflag = tet_getvar("VSX_SUPPORTED_CFLAG");
	if (supported_cflag == NULL || *supported_cflag == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_SUPPORTED_CFLAG is not set");
		DBUG_VOID_RETURN;
	}
	else if (strcmp(supported_cflag, "none") == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_SUPPORTED_CFLAG is set to \"none\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	unsupported_cflag = tet_getvar("VSX_UNSUPPORTED_CFLAG");
	if (unsupported_cflag == NULL || *unsupported_cflag == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_UNSUPPORTED_CFLAG is not set");
		DBUG_VOID_RETURN;
	}
	else if (strcmp(unsupported_cflag, "none") == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_UNSUPPORTED_CFLAG is set to \"none\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (tty_init(tty_fildes, tty_termios_p) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("initialisation failed on VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (set_cflag("VSX_SUPPORTED_CFLAG", supported_cflag) != 0)
	{
		DBUG_VOID_RETURN; /* reported in set_cflag() */
	}
	else
		PATH_FUNC_TRACE;

	exp_cflag = tty_termios_p->c_cflag;
	exp_speed = cfgetospeed(tty_termios_p);

	globok = 0;
	if (set_cflag("VSX_UNSUPPORTED_CFLAG", unsupported_cflag) != 0)
	{
		DBUG_VOID_RETURN; /* reported in set_cflag() */
	}
	else
		PATH_FUNC_TRACE;

	errno = 0;
	rval = tcsetattr(tty_fildes, TCSANOW, tty_termios_p);
	if (rval != 0)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcsetattr() returned %d, expected 0", rval);
		in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (tcgetattr(tty_fildes, tty_termios_p) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcgetattr() failed - errno %d (%s)",
			err, errname(err));
	}
	else if (tty_termios_p->c_cflag != exp_cflag)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsetattr() set incorrect c_cflag value:");
		in_rpt("expected 0x%lx, observed 0x%lx",
			(unsigned long)exp_cflag,
			(unsigned long)tty_termios_p->c_cflag);
	}
	else if (cfgetospeed(tty_termios_p) != exp_speed)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsetattr() set incorrect speed:");
		in_rpt("expected %ld, observed %ld", (long)exp_speed,
			(long)cfgetospeed(tty_termios_p));
	}
	else
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;

#else /* XPG3 mode */
	DBUG_ENTER("test19");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
#endif
}
