/*
*      SCCS:  @(#)  POSIX.os/devclass/tcsetpgrp/tcsetpgrp.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/devclass/tcsetpgrp - T.tcsetpgrp Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.tcsetpgrp

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the tcsetpgrp() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	13 Jan 1989
MODIFICATIONS:
		Geoff Clare, 24 Aug 1989
			Add test for successful call from background group
		
		Geoff Clare, 21 Sep 1989
			Change above test to setting background pgid from
				a foreground process.
			Add deviance tests for calls from background.
		
		Geoff Clare, 6 Oct 1989
			Restore to old foreground group at end of tests.

		Geoff Clare, 31 Oct 1989
			Downgrade test 11 to warning as test will be
			changing when 1003.1a is ratified.
		
		Geoff Clare, 15 Mar 1990
			Delete test for ENOTTY when terminal is no longer
			associated with session (old test 11).

		David Sawyer
		Mon Aug 06 14:28:04 BST 1990
		Added resilience code.

		Geoff Clare, 15 Mar 1990
			Check not_supported flag in test9.

		Geoff Clare, 25 June 1991
			VSX_TCGETPGRP_SUPPORTED="N" for unsupported result.
			VSX_TERMIOS_TTY="unsup" means GTI unsupported.

                Peter Skan, 30 Sept 1991
                        set the PRV_DEVICE privilege

		Stuart Boutell, 21 May 1992
		Removed dependency on GTI for EBADF and ENOTTY tests

		Geoff Clare, 22 May 1992
			New ENOTTY test cases.
			Use VSX_INVALID_PGID in EINVAL test.
			In deviance tests, check pgrp is not changed.

************************************************************************/

#include <std.h>
#include <sys/types.h>
#include <dbug.h>
#include <report.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <timeout.h>
#include <tet_api.h>
#include <setprv.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef tcsetpgrp
#endif


#ifdef TESTBED
#undef sysconf
#define sysconf(a) (-1)
#endif

#define NO_TESTS	10
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define NAP		3
#define NOK		0002
#define E_OPEN		0004
#define E_SETSID	0010
#define E_SAFAIL	0020
#define E_GOTSIG	0040
#define E_PATH		0100

extern	char	*errname();
extern	void	(*vsignal())();


int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	ch_t8();
private void	ch_t9();
private void	ch_t10();
private void	pa_t1();
private void	pa_t10();
private void	gch_t3();
private void	gch_t4();
private void	prepare_tests();
private void	clean_tests();
private void	twrap();

public	void	(*realfuncs[]) () = { test1, test2, test3, test4, test5,
				      test6, test7, test8, test9, test10,
				      NULL };

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	twrap, 1,
	twrap, 2,
	twrap, 3,
	twrap, 4,
	test5, 5,
	twrap, 6,
	twrap, 7,
	twrap, 8,
	test9, 9,
	twrap, 10,
	NULL, 0
};


private char	*tfile = "./tcsetpgrp-t";
private	struct termios	tty_termios, *tty_termios_p = &tty_termios;
private	struct termios	loop_termios, *loop_termios_p = &loop_termios;
private	int	tty_fildes, loop_fildes;
private	int	tcs_ret;
private	int	tcs_err;
private	int	caught_sig;
private	int	not_supported = 0;
private	int	gti_unsup = 0;
private	int	testfail;

private void
twrap()
{
	/* wrapper to check if unsupported and call real test function */

	DBUG_ENTER("twrap");

	if (gti_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}

	if (tet_thistest >= 1 && tet_thistest <= no_tests)
		(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}


private void
prepare_tests()
{
	char *supp;
	char *msg = NULL;
	int i;

	DBUG_ENTER("prepare_tests");

	supp = tet_getvar("VSX_TCSETPGRP_SUPPORTED");
	if (supp == NULL || *supp == '\0')
	{
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i,
				"parameter VSX_TCSETPGRP_SUPPORTED is not set");
		DBUG_VOID_RETURN;
	}
	not_supported = (*supp == 'N' || *supp == 'n');

	if (not_supported && sysconf(_SC_JOB_CONTROL) != -1)
	{
		msg = "When _POSIX_JOB_CONTROL is defined, parameter VSX_TCSETPGRP_SUPPORTED must be Y";
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
		DBUG_VOID_RETURN;
	}

	/* Open and initialize terminal file */

	switch (termios_prep (&tty_fildes, tty_termios_p,
				  &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		gti_unsup = 1;
		DBUG_VOID_RETURN;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
	}

	if (msg != NULL)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, msg);
	}

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	if (!gti_unsup) {
		(void) close(tty_fildes);
	}
	(void) unlink(tfile);
}

private void
sig_catch(sig)
int sig;
{
	caught_sig = sig;
}

private void
do_tcset()
{
	int	pathok = 0;

	DBUG_ENTER("do_tcset");

	tcs_ret = tcsetpgrp(tty_fildes, getpid());
	tcs_err = errno;

/* this function is called as the parent of cppair so globok must be set */
	PATH_FUNC_RPT(0); 

	DBUG_VOID_RETURN;
}


private void
test1()
{
	DBUG_ENTER("test1");

	if (not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcsetpgrp() not supported");
		DBUG_VOID_RETURN;
	}

	testfail = 0;

	(void) cppair(ch_t1, pa_t1, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	pid_t	newid;
	int	rval, err;
	int	pathok = 0;

	DBUG_ENTER("pa_t1");

	(void) sleep(WAITTIME/2);	/* wait for child setup */

	/* set child to be foreground process group */

	if ((rval = tcsetpgrp(tty_fildes, tet_child)) != 0)
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() in parent returned %d, expected 0", rval);
		in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if ((newid = tcgetpgrp(tty_fildes)) != tet_child)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() after tcsetpgrp() in parent returned %ld, expected %ld",
			(long) newid, (long)tet_child);
	}
	else
		PATH_TRACE;

	if (testfail > 0)
	{
		(void) killchild(tet_child, WAITTIME);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (kill(tet_child, SIGUSR1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("kill(child, SIGUSR1) failed - errno %d (%s)",
			err, errname(err));
		(void) killchild(tet_child, WAITTIME);
	}
	else
		PATH_TRACE;

	/* cppair expects globok to be set, path tracing info is not passed */
	/* to the child however */

	if (pathok == 3)
		globok = 1;
	else
	{
		globok = 0;
		DBUG_PRINTF("warning","unexpected path trace value");
	}

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	pid_t	pgrp_id, newid;
	int	rval, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	if ((pgrp_id = tcgetpgrp(tty_fildes)) == SYSERROR)
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("tcgetpgrp() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
	}
	else if (setpgid(0, 0) == SYSERROR)
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: setpgid(0, 0) failed - errno %d (%s)",
			err, errname(err));
	}
	else
		PATH_TRACE;

	(void) vsignal(SIGUSR1, sig_catch);

	caught_sig = 0;
	(void) sleep(WAITTIME);

	if (caught_sig != SIGUSR1)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: child did not receive SIGUSR1");
	}
	else
		PATH_TRACE;

	if (testfail > 0)
		DBUG_VOID_RETURN;

	/* this is now the foreground process group */

	/* restore old foreground process group */

	if ((rval = tcsetpgrp(tty_fildes, pgrp_id)) != 0)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() in child returned %d, expected 0", rval);
		in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	newid = tcgetpgrp(tty_fildes);
	if (newid != pgrp_id)
	{
		xx_rpt(FAILURE);
		in_rpt("tcgetpgrp() after tcsetpgrp() in child returned %ld, expected %ld",
			(long) newid, (long) pgrp_id);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	pid_t 	pgrp_id;
	int 	err;
	struct 	sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	testfail = 0;

	if ((pgrp_id = tcgetpgrp(tty_fildes)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcgetpgrp() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	caught_sig = 0;

	SET_TIMEOUT(WAITTIME/2)

	/* POSIX does not specify return value */
	(void) tcsetpgrp(tty_fildes, getpid());
	if (alrm_flag == 0)
		(void) pause();

	CLEAR_ALARM

	if (caught_sig != SIGTTOU)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU not received");
	}
	else
		PATH_TRACE;

	if (tcgetpgrp(tty_fildes) != pgrp_id)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() action was performed");
		(void) tcsetpgrp(tty_fildes, pgrp_id);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t3, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int 	ret, err;
	pid_t 	pgrp_id;
	sigset_t set;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	testfail = 0;

	if ((pgrp_id = tcgetpgrp(tty_fildes)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcgetpgrp() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* block SIGTTOU */

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTTOU);
	if (sigprocmask(SIG_BLOCK, &set, (sigset_t *)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigprocmask(SIG_BLOCK, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t3, do_tcset, WAITTIME, E_SAFAIL|E_GOTSIG);
	PATH_FUNC_TRACE;

	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_TRACE;

	if (tcs_ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() on VSX_TERMIOS_TTY returned %d, expected 0", tcs_ret);
		in_rpt("errno was set to %d (%s)", tcs_err, errname(tcs_err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcgetpgrp(tty_fildes) != getpid())
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() action was not performed");
	}
	else
		PATH_TRACE;
	(void) tcsetpgrp(tty_fildes, pgrp_id);

	/* Check if SIGTTOU is pending?  (shouldn't be) */

	if (testfail == 0)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

private void
gch_t3()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t3");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	if (sysconf(_SC_JOB_CONTROL) == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_JOB_CONTROL not defined");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t4, NULLFN, 2 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int 	ret, err;
	pid_t 	pgrp_id;
	struct sigaction sig;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	testfail = 0;

	if ((pgrp_id = tcgetpgrp(tty_fildes)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcgetpgrp() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig.sa_handler = SIG_IGN;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cppair(gch_t4, do_tcset, WAITTIME, E_SAFAIL|E_GOTSIG);
	PATH_FUNC_TRACE;

	if (ret == E_SAFAIL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction(SIGTTOU, ...) failed");
	}
	else if (ret == E_GOTSIG)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGTTOU received by child of calling process");
	}
	else
		PATH_TRACE;

	if (tcs_ret != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() on VSX_TERMIOS_TTY returned %d, expected 0", tcs_ret);
		in_rpt("errno was set to %d (%s)", tcs_err, errname(tcs_err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tcgetpgrp(tty_fildes) != getpid())
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() action was not performed");
	}
	else
		PATH_TRACE;
	(void) tcsetpgrp(tty_fildes, pgrp_id);

	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
gch_t4()
{
	struct sigaction sig;

	DBUG_ENTER("gch_t4");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGTTOU, &sig, NULLSA) == SYSERROR)
		DBUG_EXIT(E_SAFAIL);

	caught_sig = 0;

	(void) sleep(WAITTIME/2);	/* pause for signal */

	if (caught_sig == SIGTTOU)
		DBUG_EXIT(E_GOTSIG);

	DBUG_VOID_RETURN;
}


private void
test5()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	if (not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcsetpgrp() not supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	if ((rval = tcsetpgrp(-1, getpgrp())) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp(-1, getpgrp()) returned %d, expected -1", rval);
	}
	else if (errno != EBADF)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp(-1, getpgrp()) gave errno %d (%s), expected EBADF",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private void
test6()
{
	char	*cp;
	int 	rval, err;
	pid_t	pgrp_id;
	pid_t	invalid_pgid = -1;
	int	pathok = 0;

	DBUG_ENTER("test6");

	if (not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcsetpgrp() not supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	cp = tet_getvar("VSX_INVALID_PGID");
	if (cp != NULL && *cp != '\0')
		invalid_pgid = atol(cp);

	if ((pgrp_id = tcgetpgrp(tty_fildes)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcgetpgrp() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = tcsetpgrp(tty_fildes, invalid_pgid)) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp(fd, %ld) returned %d, expected -1",
			(long)invalid_pgid, rval);
		DBUG_VOID_RETURN;
	}
	else if (errno != EINVAL)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetpgrp(fd, %ld) gave errno %d (%s), expected EINVAL",
			(long)invalid_pgid, err, errname(err));
	}
	else
		PATH_TRACE;

	if (tcgetpgrp(tty_fildes) != pgrp_id)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() changed the process group ID of the terminal");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("test7");

	if (!not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcsetpgrp() is supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	/* Unsupported */

	if ((rval = tcsetpgrp(tty_fildes, getpgrp())) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() returned %d, expected -1", rval);
	}
	else if (errno != ENOSYS)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() gave errno %d (%s), expected ENOSYS",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private void
test8()
{
	pid_t	pgrp_id;
	int	retval, err;
	int	pathok = 0;

	DBUG_ENTER("test8");

	if (not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcsetpgrp() not supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	if ((pgrp_id = tcgetpgrp(tty_fildes)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcgetpgrp() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((retval = cppair(ch_t8, NULLFN, WAITTIME, NOK|E_PATH)) == NOK)
		testfail++;
	else if (retval != E_PATH)
		PATH_TRACE;

	if (tcgetpgrp(tty_fildes) != pgrp_id)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() changed the process group ID of the terminal");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int 	rval, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t8");

	/* Doesn't have controlling terminal */

	(void) setsid(); /* should sever connection to control terminal */

	if ((rval = tcsetpgrp(tty_fildes, getpgrp())) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() returned %d, expected -1", rval);
		DBUG_EXIT(NOK);
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() gave errno %d (%s), expected ENOTTY",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 1)
		DBUG_EXIT(E_PATH);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	int 	rval, err;
	int	retval, fd;
	pid_t	pgrp_id;
	int	pathok = 0;

	DBUG_ENTER("test9");

	if (not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcsetpgrp() not supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	fd = creat(tfile, MODEANY);
	if (fd == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file %s", tfile);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = tcsetpgrp(fd, getpgrp())) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() on a plain file returned %d, expected -1", rval);
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() on a plain file gave errno %d (%s), expected ENOTTY",
			err, errname(err));
	}
	else
		PATH_TRACE;
	
	(void) close(fd);
	(void) unlink(tfile);
	
	if (gti_unsup)
	{
		PATH_XS_RPT(3);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* device is not controlling terminal */

	if ((rval = tcsetpgrp(loop_fildes, getpgrp())) != -1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("on a non-controlling terminal");
		in_rpt("tcsetpgrp() returned %d, expected -1", rval);
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("on a non-controlling terminal");
		in_rpt("tcsetpgrp() gave errno %d (%s), expected ENOTTY",
			err, errname(err));
	}
	else
		PATH_TRACE;

	/* device is the controlling terminal of another session */

	if ((pgrp_id = tcgetpgrp(tty_fildes)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcgetpgrp() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	retval = cppair(ch_t9, NULLFN, WAITTIME, NOK | E_PATH);
	if (retval == NOK)
		testfail++;
	else if (retval != E_PATH)
		PATH_TRACE;

	if (tcgetpgrp(tty_fildes) != pgrp_id)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() changed the process group ID of the terminal");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	char	*termios_loop;
	int	rval, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t9");

	termios_loop = tet_getvar("VSX_TERMIOS_LOOP");
	/* no need to check: termios_prep() succeeded */

	(void) setsid();

	/* TERMIOS_LOOP is already open, so don't need O_NONBLOCK */
	loop_fildes = openctl(termios_loop, O_RDWR);
	if (loop_fildes < 0)
	{
		xx_rpt(DELETION);
		in_rpt("openctl() failed on VSX_TERMIOS_LOOP");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if ((rval = tcsetpgrp(tty_fildes, getpgrp())) != -1)
	{
		xx_rpt(FAILURE);
		in_rpt("on the controlling terminal of another session");
		in_rpt("tcsetpgrp() returned %d, expected -1", rval);
		DBUG_EXIT(NOK);
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("on the controlling terminal of another session");
		in_rpt("tcsetpgrp() gave errno %d (%s), expected ENOTTY",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(E_PATH);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	int 	ret;
	int	pathok = 0;

	DBUG_ENTER("test10");

	if (not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("tcsetpgrp() not supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	globok = 0;
	ret = cppair(ch_t10, pa_t10, WAITTIME, E_SETSID);
	PATH_FUNC_TRACE;

	if (ret == E_SETSID)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: setsid() failed in child");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
pa_t10()
{
	int 	rval, err;
	pid_t	pgrp_id;
	int	pathok = 0;

	DBUG_ENTER("pa_t10");

	(void) sleep(NAP);

	if ((pgrp_id = tcgetpgrp(tty_fildes)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("tcgetpgrp() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = tcsetpgrp(tty_fildes, tet_child)) != -1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() returned %d, expected -1", rval);
		DBUG_VOID_RETURN;
	}
	else if (errno != EPERM)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() gave errno %d (%s), expected EPERM",
			err, errname(err));
	}
	else
		PATH_TRACE;

	if (tcgetpgrp(tty_fildes) != pgrp_id)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("tcsetpgrp() changed the process group ID of the terminal");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	DBUG_ENTER("ch_t10");

	if (setsid() == SYSERROR)
		DBUG_EXIT(E_SETSID);

	(void) sleep(WAITTIME/2);

	DBUG_VOID_RETURN;
}
