/*
*      SCCS:  @(#)  POSIX.os/files/access/access.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/access - T.access Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.access

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the access() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	5 Apr 1989
MODIFICATIONS:
		Geoff Clare, 12 June 1989:  Add assertion 30 tests.
		Geoff Clare, 3 July 1989:   Delete X_OK priv fail test.

		Geoff Clare, 4 Aug 1989
			Don't use a supp. group for "other" access
			(1003.3 assertions are wrong).

		Geoff Clare, 24 Aug 1989
			Add testing of supplementary groups
			Change EINVAL test to givfe WARNING result

		Geoff Clare, 15 Nov 1989
			New setuprofs() argument
		
		Geoff Clare, 29 Mar 1990
			New interface with pathtoolng() and nametoolng().

		David Sawyer
		Thu Jul 26 10:06:13 BST 1990
		Added resilience code.

		Geoff Clare, 4 Oct 1990
			Change uid and gid args to crdir/crfile in ch_t36().

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Peter Skan, 16 Oct 1991
			Perform every access() call in a separate child
			process, setting or removing privileges first as
			appropriate.  This is necessary since in a system
			with non-monolithic privileges, access() may still
			observe privilege even with non-root real UID.
			Added nonprvsetup() to remove privileges.
			With this arrangement, some tests no longer need
			child processes of their own.
			Note that test26_35() was previously running test 26
			repeatedly due to "tnum" being incremented in child
			process.
			Removed if-else with duplicate body from tbaccess().

		Geoff Clare, 18 Dec 1991
			Skip egid if it appears in supp group list.
			Add euid and egid tests in devnorm[].

		Geoff Clare, 22 Apr 1992
			Change EINVAL test to use parameter and change
			warning result to failure.

		Geoff Clare, 8 May 1992
			Add PRES_NODOTDOT to init_pathres() call.

		Neil Moses, 9 June 1992
			Check that time stamps are not changed.

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		Geoff Clare, 9 Sept 1992
			Allow VSX_INVALID_AMODE="unsup".

		David Sawyer, 19 Oct 1992
			Added TEST_XNFS code.

		David Sawyer, 20 Oct 1992
			Changed all the references to tests and files, so
			that they relate to test numbers rather than deviance
			test N etc...

		Geoff Clare, 2 Feb 1993
			When testing XNFS, expect getgroups() to return
			min of NGROUPS_MAX values for server and client.

		Geoff Clare, 18 March 1994
			When testing XNFS, don't expect getgroups() to
			return more than 16.

		Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<pathres.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"access_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef access
#endif


#define NO_TESTS	78		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)	/* timeout value */
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define PRIV		0
#define NOPRIV		1

extern	char	*errname();
extern	void	savtimes();
extern	char	*chktimes();
extern	char	*itoa();
#if	   TEST_XNFS
extern	char	*xnfs_nametoolng();
#else	/* TEST_XNFS */
extern	char	**nametoolng();
#endif	/* TEST_XNFS */
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	void	valrpt();
extern	void	xx_all();

extern	int	vguid1, vggid1, vguid0, vggid0;

public	int	no_tests = NO_TESTS;

private int	uidlist[3];
private int	gidlist[3];

struct tdat {
	int	uind;		/* file owner = uidlist[uind] 	*/
	int	gind;		/* file group = gidlist[gind] 	*/
	mode_t	fmode;		/* file mode (S_IRWXU etc.)	*/
	int	amode;		/* access mode (F_OK etc.)	*/
};

private struct tdat compnorm[] = {	/* compliance tests - normal real uid */
/*	ui gi fmode			amode */
	0, 0, 0,			F_OK,
	0, 0, S_IRUSR,			R_OK,
	0, 0, S_IWUSR,			W_OK,
	0, 0, S_IXUSR,			X_OK,
	1, 0, S_IRGRP,			R_OK,
	1, 0, S_IWGRP,			W_OK,
	1, 0, S_IXGRP,			X_OK,
	1, 1, S_IROTH,			R_OK,
	1, 1, S_IWOTH,			W_OK,
	1, 1, S_IXOTH,			X_OK,
	0, 0, S_IRUSR|S_IWUSR,		R_OK|W_OK,
	0, 0, S_IRUSR|S_IXUSR,		R_OK|X_OK,
	0, 0, S_IWUSR|S_IXUSR,		W_OK|X_OK,
	0, 0, S_IRWXU,			R_OK|W_OK|X_OK,
	1, 0, S_IRGRP|S_IWGRP,		R_OK|W_OK,
	1, 0, S_IRGRP|S_IXGRP,		R_OK|X_OK,
	1, 0, S_IWGRP|S_IXGRP,		W_OK|X_OK,
	1, 0, S_IRWXG,			R_OK|W_OK|X_OK,
	1, 1, S_IROTH|S_IWOTH,		R_OK|W_OK,
	1, 1, S_IROTH|S_IXOTH,		R_OK|X_OK,
	1, 1, S_IWOTH|S_IXOTH,		W_OK|X_OK,
	1, 1, S_IRWXO,			R_OK|W_OK|X_OK,
};

private struct tdat comppriv[] = {	/* compliance tests - privileged uid */
/*	ui gi fmode			amode */
	0, 0, 0,			F_OK,
	0, 0, 0,			R_OK,
	0, 0, 0,			W_OK,
	0, 0, S_IXUSR,			X_OK,
	0, 0, S_IXGRP,			X_OK,
	0, 0, S_IXOTH,			X_OK,
	0, 0, 0,			R_OK|W_OK,
	0, 0, S_IXUSR,			R_OK|X_OK,
	0, 0, S_IXGRP,			W_OK|X_OK,
	0, 0, S_IXOTH,			R_OK|W_OK|X_OK,
};

private struct tdat devnorm[] = {	/* deviance tests - normal real uid */
/*	ui gi fmode			amode */
	0, 0, 0,			R_OK,
	0, 0, 0,			W_OK,
	0, 0, 0,			X_OK,
	0, 0, 0,			R_OK|W_OK|X_OK,
	0, 0, (MODEANY & ~S_IRUSR),	R_OK,
	0, 0, (MODEANY & ~S_IWUSR),	W_OK,
	0, 0, (MODEANY & ~S_IXUSR),	X_OK,
	0, 0, (MODEANY & ~S_IRWXU),	R_OK|W_OK|X_OK,
	1, 0, (MODEANY & ~S_IRGRP),	R_OK,
	1, 0, (MODEANY & ~S_IWGRP),	W_OK,
	1, 0, (MODEANY & ~S_IXGRP),	X_OK,
	1, 0, (MODEANY & ~S_IRWXG),	R_OK|W_OK|X_OK,
	1, 1, (MODEANY & ~S_IROTH),	R_OK,
	1, 1, (MODEANY & ~S_IWOTH),	W_OK,
	1, 1, (MODEANY & ~S_IXOTH),	X_OK,
	1, 1, (MODEANY & ~S_IRWXO),	R_OK|W_OK|X_OK,
	2, 0, S_IRUSR,			R_OK,
	2, 0, S_IWUSR,			W_OK,
	2, 0, S_IXUSR,			X_OK,
	1, 2, S_IRGRP,			R_OK,
	1, 2, S_IWGRP,			W_OK,
	1, 2, S_IXGRP,			X_OK,
	2, 0, S_IRUSR|S_IWUSR,		R_OK|W_OK,
	2, 0, S_IRUSR|S_IXUSR,		R_OK|X_OK,
	2, 0, S_IWUSR|S_IXUSR,		W_OK|X_OK,
	2, 0, S_IRWXU,			R_OK|W_OK|X_OK,
	1, 2, S_IRGRP|S_IWGRP,		R_OK|W_OK,
	1, 2, S_IRGRP|S_IXGRP,		R_OK|X_OK,
	1, 2, S_IWGRP|S_IXGRP,		W_OK|X_OK,
	1, 2, S_IRWXG,			R_OK|W_OK|X_OK,
};


private int	testfail;
private int	privsupp;
private	gid_t	*suppgrps;
private	int	ngroups;
private char	*t3_file =	"access-t.3";
private char	*t4_25_file =	"access-t.cN";
private char	*t26_35_file1 =	"access-t.26.1";
private char	*t26_35_file2 =	"access-t.26.2";
private char	*t36_dir1 =	"access-d.36.1";
private char	*t36_file1 =	"access-d.36.1/access-t.36.1";
#if	   TEST_XNFS
private char	*t36_file2 =	T36_2_FILE;
#endif	/* TEST_XNFS */
private	char	*t38_file =	"access-t.38";
private char	*t39_dir =	"access-d.39";
private char	*t39_file =	"access-d.39/access-t.39";
private	char	*t40_file =	"access-t.40";
private char	*t42_ndir =	"access-t.42";
private char	*t42_file =	"access-t.42/access-t.42";
private char	*t43_dir =	"access-d.43";
private char	*t43_file =	"access-d.43/access-t.43";
#if	  !TEST_XNFS
private	char	*t44_file =	"access-t.44";
#endif	/* TEST_XNFS */
private char	*t45_74_file =	"access-t.45";

private void	prepare_tests();
private void	cln_tests();

private void	test1();
private void	test2();
private void	subtest2();
private void	test3();
private void	subtest3();
private void	test4_25();
private void	subtest4_25();
#if	   TEST_XNFS
private void	ch_t4_25();
#endif	/* TEST_XNFS */
private void	test26_35();
private void	subtest26_35();
private void	test36();
private void	test37();
private void	test38();
private void	test39();
private void	test40();
private void	test41();
private void	test42();
private void	test43();
private void	test44();
private void	test45_74();
private void	subtest45_74();
#if	   TEST_XNFS
private void	ch_t45_74();
#endif	/* TEST_XNFS */
private void	test75();
private void	test76();
private void	test77();
private void	test78();

#if	   TEST_XNFS
private void	ch_t2();
private void	ch_t3();
private void	ch_t26_35();
#endif	/* TEST_XNFS */
private void	ch_t36();
private void	ch_tbacc();

private void	dosuppgrps();
private void	tbaccess();

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4_25, 4,
	 test4_25, 5,
	 test4_25, 6,
	 test4_25, 7,
	 test4_25, 8,
	 test4_25, 9,
	 test4_25, 10,
	 test4_25, 11,
	 test4_25, 12,
	 test4_25, 13,
	 test4_25, 14,
	 test4_25, 15,
	 test4_25, 16,
	 test4_25, 17,
	 test4_25, 18,
	 test4_25, 19,
	 test4_25, 20,
	 test4_25, 21,
	 test4_25, 22,
	 test4_25, 23,
	 test4_25, 24,
	 test4_25, 25,
	 test26_35, 26,
	 test26_35, 27,
	 test26_35, 28,
	 test26_35, 29,
	 test26_35, 30,
	 test26_35, 31,
	 test26_35, 32,
	 test26_35, 33,
	 test26_35, 34,
	 test26_35, 35,
	 test36, 36,
	 test37, 37,
	 test38, 38,
	 test39, 39,
	 test40, 40,
	 test41, 41,
	 test42, 42,
	 test43, 43,
	 test44, 44,
	 test45_74, 45,
	 test45_74, 46,
	 test45_74, 47,
	 test45_74, 48,
	 test45_74, 49,
	 test45_74, 50,
	 test45_74, 51,
	 test45_74, 52,
	 test45_74, 53,
	 test45_74, 54,
	 test45_74, 55,
	 test45_74, 56,
	 test45_74, 57,
	 test45_74, 58,
	 test45_74, 59,
	 test45_74, 60,
	 test45_74, 61,
	 test45_74, 62,
	 test45_74, 63,
	 test45_74, 64,
	 test45_74, 65,
	 test45_74, 66,
	 test45_74, 67,
	 test45_74, 68,
	 test45_74, 69,
	 test45_74, 70,
	 test45_74, 71,
	 test45_74, 72,
	 test45_74, 73,
	 test45_74, 74,
	 test75, 75,
	 test76, 76,
	 test77, 77,
	 test78, 78,
	NULL, 0
};


private void
prepare_tests()
{
	char	*supp, *msg = NULL;
	int	i;
	int	ngroups_max;
#if TEST_XNFS
	int	nvals;
	char	**strvals;
	extern char **xnfs_getsv_c();
#endif

	DBUG_ENTER("prepare_tests");


	if(uids()!=0)
	{
		xx_all(DELETION, "uids() failed");
		DBUG_VOID_RETURN;
	}

	uidlist[0] = vguid0;
	uidlist[1] = vguid1;
	uidlist[2] = geteuid();

	gidlist[0] = vggid0;
	gidlist[1] = vggid1;
	gidlist[2] = getegid();

	if (setprv(PRV_CHOWN) != 0)
		msg = "setprv(PRV_CHOWN) failed";
	else if (setprv(PRV_SETID) != 0)
		msg = "setprv(PRV_SETID) failed";
	else if (getuid() != vguid0)
		msg = "real user ID is not VSX_UID0";
	else if (getgid() != vggid0)
		msg = "real group ID is not VSX_GID0";
	else if (geteuid() == vguid0 || geteuid() == vguid1)
		msg = "effective user ID must not be VSX_UID0 or VSX_UID1";
	else if (getegid() == vggid0|| getegid() == vggid1)
		msg = "effective group ID must not be VSX_GID0 or VSX_GID1";

	if (msg != NULL)
	{
		xx_all(DELETION, msg);
		DBUG_VOID_RETURN;
	}

	ngroups_max = sysconf(_SC_NGROUPS_MAX);
	if (ngroups_max < 0)
	{
		xx_all(DELETION, "sysconf(_SC_NGROUPS_MAX) failed");
		DBUG_VOID_RETURN;
	}

#if TEST_XNFS
	/* obtain server value for NGROUPS_MAX and set `ngroups' to
	   minimum of client and server values */

	nvals = 0;
	strvals = xnfs_getsv_c(&nvals, SP_T0_GET_NGROUPS);
	if (strvals == NULL || nvals != 1)
	{
		xx_all(DELETION,
		    "xnfs_getsv_c() failed when getting NGROUPS_MAX from server");
		DBUG_VOID_RETURN;
	}

	ngroups = atoi(*strvals);
	if (ngroups < 0)
	{
		xx_all(DELETION, "could not obtain NGROUPS_MAX from server");
		DBUG_VOID_RETURN;
	}

	if (ngroups > ngroups_max)
		ngroups = ngroups_max;

	/* maximum usable by NFS is 16 */
	if (ngroups > 16)
		ngroups = 16;
#else
	ngroups = ngroups_max;
#endif

	if (ngroups_max == 0)
		/* do nothing */;
	else if ((suppgrps = (gid_t *)malloc((size_t)(sizeof(gid_t) *
						ngroups_max))) == NULL)
		msg = "malloc() failed";
	else if (getgroups(ngroups_max, suppgrps) != ngroups)
#if TEST_XNFS
		msg = "getgroups() did not return minimum of client and server NGROUPS_MAX values and 16";
#else
		msg = "getgroups() did not return NGROUPS_MAX";
#endif
	else
	{
		for (i = 0; i < ngroups; i++)
		{
			if (suppgrps[i] == vggid1)
			{
				msg = "group vsxg1 must not be in the supplementary groups list";
				break;
			}
		}
	}

	if (msg != NULL)
	{
		xx_all(DELETION, msg);
		DBUG_VOID_RETURN;
	}

	supp = tet_getvar("VSX_PRIV_ACCESS_SUPP");
	privsupp = (supp != NULL && (*supp == 'Y' || *supp == 'y'));

	(void) umask((mode_t)0);

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	char **p;

	DBUG_ENTER("cln_tests");

	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);
	(void) rmdir(t3_file);
	(void) unlink(t4_25_file);
	(void) unlink(t26_35_file1);
	(void) unlink(t26_35_file2);
	(void) chmod(t36_dir1, MODEANY);
	(void) unlink(t36_file1);
	(void) rmdir(t36_dir1);
	(void) unlink(t38_file);
	(void) unlink(t42_ndir);
#if	  !TEST_XNFS
	(void) unlink(t44_file);
#endif	/* TEST_XNFS */
	(void) unlink(t45_74_file);

	DBUG_VOID_RETURN;
}

private void
prvsetup()
{
	int	pathok = 0;

	DBUG_ENTER("prvsetup");

	if (!privsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_PRIV_ACCESS_SUPP not set to 'Y'");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	/* Set real user ID to the privileged user */

	(void) setgid(getegid());
	(void) setuid(geteuid());
	if (getuid() != geteuid())
	{
		xx_rpt(DELETION);
		in_rpt("setuid(geteuid()) did not set real uid to effective uid");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;
	if (getgid() != getegid())
	{
		xx_rpt(DELETION);
		in_rpt("setgid(getegid()) did not set real gid to effective gid");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	/* Request privileged access permissions */

	if (setprv(PRV_ACCESS) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_ACCESS) failed");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
nonprvsetup()
{
	int	pathok = 0;

	DBUG_ENTER("nonprvsetup");

	/* Remove all privileges */

	if (unsetprv(PRV_ACCESS) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ACCESS) failed");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_CHOWN) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_CHOWN) failed");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_MOUNT) failed");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}


private void
test1()
{
	/* pathname resolution */

	char *p1, *p2;
	int npairs, i, err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOTDOT);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOTDOT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < npairs; i++)
	{
		if (i==0)
			PATH_TRACE;
		p2 = pathresnam[2*i+1];
		if (eaccess(p2, F_OK) == 0 && remove(p2) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("remove(\"%s\") failed - errno %d (%s)",
				p2, err, errname(err));
			DBUG_VOID_RETURN;
		}
	}

	for (i = 0; i < npairs; i++)
	{
		if (i==0)
			PATH_TRACE;
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		if (vxmkdir(p2) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create directory \"%s\"",
				p2);
		}
		else
		{
			tbaccess(p2, F_OK, SUCCEED, NOERROR, NOPRIV);
			tbaccess(p1, F_OK, SUCCEED, NOERROR, NOPRIV);

			if (rmdir(p2) == SYSERROR)
			{
				err = errno;
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("deletion reason: rmdir(\"%s\") failed - errno %d (%s)",
					p2, err, errname(err));
			}
			else
			{
				tbaccess(p2, F_OK, SYSERROR, ENOENT, NOPRIV);
				tbaccess(p1, F_OK, SYSERROR, ENOENT, NOPRIV);
			}
		}
	}

	/* This is all we can do with root aliases */

	globok = 0;
	tbaccess("/", F_OK, SUCCEED, NOERROR, NOPRIV);
	PATH_FUNC_TRACE;

	globok = 0;
	tbaccess("///", F_OK, SUCCEED, NOERROR, NOPRIV);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);
	
	cln_pathres();

	DBUG_VOID_RETURN;
}


private void
test2()
{
	DBUG_ENTER("test2");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t2, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest2();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t2()
{
	DBUG_ENTER("ch_t2");

	subtest2();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


private void
subtest2()
{
	/* portable filenames */

	char	**p;
	int	i, fd;
	int	amode1, amode2;
	static	mode_t modelist[7] = {
		0,
		S_IRUSR,
		S_IWUSR,
		S_IXUSR,
		S_IRUSR|S_IWUSR,
		S_IRUSR|S_IXUSR,
		S_IWUSR|S_IXUSR,
		/* can't use "rwx": amode2=F_OK would not fail */
	};
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	/* Use a different mode for each file */

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		if (i==0)
			PATH_TRACE;
		if (i >= 7)
		{
			xx_rpt(DELETION);
			in_rpt("internal error: too many names in portfnam[]");
			vsxerror("too many names in portfnam[]");
			DBUG_VOID_RETURN;
		}

		if ((fd = crfile(*p, modelist[i], (int)getuid(), (int)getgid())) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create file \"%s\" with uid %d, gid %d, mode %#o",
			    *p, (int)getuid(), (int)getgid(), modelist[i]);
			DBUG_VOID_RETURN;
		}
		(void) close(fd);
	}

#ifdef TESTBED2
	modelist[1] = S_IWUSR;
#endif

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		if (i==0)
			PATH_TRACE;
		amode1 = 0;
		amode2 = R_OK|W_OK|X_OK;
		if (modelist[i] & S_IRUSR)
		{
			amode1 |= R_OK;
			amode2 &= ~R_OK;
		}
		if (modelist[i] & S_IWUSR)
		{
			amode1 |= W_OK;
			amode2 &= ~W_OK;
		}
		if (modelist[i] & S_IXUSR)
		{
			amode1 |= X_OK;
			amode2 &= ~X_OK;
		}
		if (amode1 == 0)
			amode1 = F_OK;

		tbaccess(*p, amode1, SUCCEED, NOERROR, NOPRIV);
		tbaccess(*p, amode2, SYSERROR, EACCES, NOPRIV);
	}

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		(void) unlink(*p);
	}

	if (testfail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}


private void
test3()
{
	DBUG_ENTER("test3");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t3, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest3();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t3()
{
	DBUG_ENTER("ch_t3");

	subtest3();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


private void
subtest3()
{
	int	pathok = 0;

	/* privileged access: X_OK succeeds on directory */

	DBUG_ENTER("subtest3");

	testfail = 0;

	if (crdir(t3_file, (mode_t)0, (int)getuid(), (int)getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	tbaccess(t3_file, X_OK, SUCCEED, NOERROR, PRIV);
	PATH_FUNC_TRACE;

	(void) rmdir(t3_file);

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test4_25()
{
	DBUG_ENTER("test4_25");

#if	   TEST_XNFS

	/*
	 *  These tests need to be done on a file sytem that does not map
	 *  the root id so that the root privilege is not lost when
	 *  chown()ing files etc. (For example: chown() is used in crfile()).
	 */

	xnfs_doroot_c(ch_t4_25, WAITTIME, SP_T4_25_DOROOT_START, 
		SP_T4_25_DOROOT_END);

#else	/* TEST_XNFS */

	subtest4_25();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t4_25()
{
	DBUG_ENTER("ch_t4_25");

	subtest4_25();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
subtest4_25()
{
	/* compliance tests - normal real user id */

	int		fd;
	struct tdat	*td;
	static int	tnum = 0;
	int	pathok = 0;

	DBUG_ENTER("subtest4_25");

	testfail = 0;

	td = &compnorm[tnum++];

	if ((fd = crfile(t4_25_file, td->fmode,
			uidlist[td->uind], gidlist[td->gind])) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\" with uid %d, gid %d, mode %#o",
		    t4_25_file, uidlist[td->uind], gidlist[td->gind], td->fmode);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	tbaccess(t4_25_file, td->amode, SUCCEED, NOERROR, NOPRIV);
	PATH_FUNC_TRACE;

	/* If this is a group access test, do the supplementary groups */
	if (td->uind == 1 && td->gind == 0)
	{
		globok = 0;
		dosuppgrps(t4_25_file, td->amode, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	(void) unlink(t4_25_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t4_25_file, (mode_t)td->fmode) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\" with mode %#o",
		    t4_25_file, td->fmode);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (chown(t4_25_file, (uid_t)uidlist[td->uind], 
			(gid_t)gidlist[td->gind]) == SYSERROR)
	{
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("could not change ownership of fifo \"%s\" to uid %d, gid %d",
		    t4_25_file, uidlist[td->uind], gidlist[td->gind]);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	tbaccess(t4_25_file, td->amode, SUCCEED, NOERROR, NOPRIV);
	PATH_FUNC_TRACE;

	/* If this is a group access test, do the supplementary groups */
	if (td->uind == 1 && td->gind == 0)
	{
		globok = 0;
		dosuppgrps(t4_25_file, td->amode, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	(void) unlink(t4_25_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}


private void
test26_35()
{
	DBUG_ENTER("test26_35");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t26_35, WAITTIME, SP_T26_35_DOROOT_START, 
		SP_T26_35_DOROOT_END);

#else	/* TEST_XNFS */

	subtest26_35();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t26_35()
{
	DBUG_ENTER("ch_t26_35");

	subtest26_35();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
subtest26_35()
{
	/* compliance tests - privileged real user id */

	int		fd;
	struct tdat	*td;
	static int	tnum = 0;
	int	pathok = 0;

	DBUG_ENTER("subtest26_35");

	testfail = 0;

	td = &comppriv[tnum++];

	if ((fd = crfile(t26_35_file1, td->fmode,
			uidlist[td->uind], gidlist[td->gind])) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\" with uid %d, gid %d, mode %#o",
		    t26_35_file1, uidlist[td->uind], gidlist[td->gind], td->fmode);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);


#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t26_35_file2, (mode_t)td->fmode) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\" with mode %#o",
		    t26_35_file2, td->fmode);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (chown(t26_35_file2, (uid_t)uidlist[td->uind], (gid_t)gidlist[td->gind])
			== SYSERROR)
	{
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("could not change ownership of fifo \"%s\" to uid %d, gid %d",
		    t26_35_file2, uidlist[td->uind], gidlist[td->gind]);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#endif	/* TEST_XNFS */

	globok = 0;
	tbaccess(t26_35_file1, td->amode, SUCCEED, NOERROR, PRIV);
	PATH_FUNC_TRACE;

#if	   TEST_XNFS

	PATH_TRACE;

#else	/* TEST_XNFS */

	globok = 0;
	tbaccess(t26_35_file2, td->amode, SUCCEED, NOERROR, PRIV);
	PATH_FUNC_TRACE;

#endif	/* TEST_XNFS */

	(void) unlink(t26_35_file1);

#if	  !TEST_XNFS

	(void) unlink(t26_35_file2);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test36()
{
	int	pathok = 0;
	/* search permission denied */

	DBUG_ENTER("test36");

	testfail = 0;

	if (tet_fork(ch_t36, NULLFN, WAITTIME, 1) == 1)
		PATH_TRACE;

#if	   TEST_XNFS

	if (tet_sync(SP_T36_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	tbaccess(t36_file2, F_OK, SYSERROR, EACCES, PRIV);
	PATH_FUNC_TRACE;

	if (tet_sync(SP_T36_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t36()
{
	int	fd, err;
	mode_t	mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
	int	pathok = 0;
	int	t36_exit = 0;

	DBUG_ENTER("ch_t36");

	(void) setuid((uid_t)vguid1);
	if (geteuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user id to VSX_UID1");
		DBUG_EXIT(t36_exit);
	}
	else
		PATH_TRACE;

	if (crdir(t36_dir1, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t36_dir1);
		DBUG_EXIT(t36_exit);
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t36_file1, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t36_file1);
		DBUG_EXIT(t36_exit);
	}
	else
		PATH_TRACE;

	(void) close(fd);

	if (chmod(t36_dir1, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t36_dir1, (int)mode, err, errname(err));
		DBUG_EXIT(t36_exit);
	}
	else
		PATH_TRACE;

	globok = 0;
	tbaccess(t36_file1, F_OK, SYSERROR, EACCES, NOPRIV);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{
		PATH_FUNC_RPT(5);
		t36_exit = 1;
	}

	(void) chmod(t36_dir1, MODEANY);
	(void) unlink(t36_file1);
	(void) rmdir(t36_dir1);

	DBUG_EXIT(t36_exit);
}

#if	   TEST_XNFS

private void
test37()
{
	DBUG_ENTER("test37");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test37()
{
	/* path components equal and longer than NAME_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test37");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname[0], (mode_t)MODEANY,
			(int)getuid(), (int)getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		tbaccess(longname[0], F_OK, SUCCEED, NOERROR, NOPRIV);
		PATH_FUNC_TRACE;
		(void) close(fd);
	}

	if (longname[1] != NULL)
	{
		globok = 0;
		tbaccess(longname[1], F_OK, SYSERROR, ENAMETOOLONG, NOPRIV);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test38()
{
	/* paths equal and longer than PATH_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test38");

	testfail = 0;

	if ((fd = creat(t38_file, (mode_t) MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t38_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	longname = pathtoolng(t38_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	tbaccess(longname[0], F_OK, SUCCEED, NOERROR, NOPRIV);
	PATH_FUNC_TRACE;

	if (longname[1] != NULL)
	{
		globok = 0;
		tbaccess(longname[1], F_OK, SYSERROR, ENAMETOOLONG, NOPRIV);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	clntoolng();
	(void) unlink(t38_file);

	DBUG_VOID_RETURN;
}

private void
test39()
{
	int	pathok = 0;

	/* component of path prefix does not exist */

	DBUG_ENTER("test39");

	testfail = 0;

	if (eaccess(t39_dir, F_OK) == 0 && remove(t39_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t39_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	tbaccess(t39_file, R_OK, SYSERROR, ENOENT, NOPRIV);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test40()
{
	int	pathok = 0;

	/* file which does not exist */

	DBUG_ENTER("test40");

	testfail = 0;

	if (eaccess(t40_file, F_OK) == 0 && remove(t40_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t40_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	tbaccess(t40_file, R_OK, SYSERROR, ENOENT, NOPRIV);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test41()
{
	int	pathok = 0;

	/* path is "" */

	DBUG_ENTER("test41");

	testfail = 0;

	globok = 0;
	tbaccess("", R_OK, SYSERROR, ENOENT, NOPRIV);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test42()
{
	int	pathok = 0;

	/* component of path is not a directory */

	int	fd;

	DBUG_ENTER("test42");

	testfail = 0;

	if ((fd = creat(t42_ndir, (mode_t)MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t42_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	tbaccess(t42_file, F_OK, SYSERROR, ENOTDIR, NOPRIV);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t42_ndir);

	DBUG_VOID_RETURN;
}

private void
test43()
{
	char	*errptr;
	int	pathok = 0;

	/* write access on read only file system */

	DBUG_ENTER("test43");

	testfail = 0;

	globok = 0;
	if (setuprofs(t43_dir, t43_file, 'f', (mode_t) MODEANY) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(t43_file);
	PATH_FUNC_TRACE;

	globok = 0;
	tbaccess(t43_file, W_OK, SYSERROR, EROFS, NOPRIV);
	PATH_FUNC_TRACE;

	globok = 0;
	errptr = chktimes(t43_file, 0);
	PATH_FUNC_TRACE;

	if(errptr != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful link() changed time stamps on %s",
			t43_file);
		in_rpt("%s", errptr);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	(void) clnuprofs(t43_dir, t43_file);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test44()
{
	DBUG_ENTER("test44");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test44()
{
	/* invalid amode */

	char *cp;
	int fd, invmode = -1;
	int	pathok = 0;

	DBUG_ENTER("test44");

	testfail = 0;

	cp = tet_getvar("VSX_INVALID_AMODE");
	if (cp != NULL && strncmp(cp, "unsup", (size_t)5) == 0) 
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_INVALID_AMODE is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else if (cp != NULL && *cp != '\0')
	{
		invmode = atoi(cp);
		PATH_TRACE;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t44_file, (mode_t)MODEANY,
			(int) getuid(), (int) getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t44_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	tbaccess(t44_file, invmode, SYSERROR, EINVAL, NOPRIV);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(t44_file);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test45_74()
{
	DBUG_ENTER("test45_74");

#if	   TEST_XNFS

	/*
	 *  These tests need to be done on a file sytem that does not map
	 *  the root id so that the root privilege is not lost when
	 *  chown()ing files etc. (For example: chown() is used in crfile()).
	 */

	xnfs_doroot_c(ch_t45_74, WAITTIME, SP_T45_74_DOROOT_START, 
		SP_T45_74_DOROOT_END);

#else	/* TEST_XNFS */

	subtest45_74();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t45_74()
{
	DBUG_ENTER("ch_t45_74");

	subtest45_74();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
subtest45_74()
{
	/* deviance tests - normal real user id */

	int		fd;
	struct tdat	*td;
	static int	tnum = 0;
	int		pathok = 0;

	DBUG_ENTER("subtest45_74");

	testfail = 0;

	td = &devnorm[tnum++];

	if ((fd = crfile(t45_74_file, td->fmode,
			uidlist[td->uind], gidlist[td->gind])) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\" with uid %d, gid %d, mode %#o",
		    t45_74_file, uidlist[td->uind], gidlist[td->gind], td->fmode);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	tbaccess(t45_74_file, td->amode, SYSERROR, EACCES, NOPRIV);
	PATH_FUNC_TRACE;

	/* If this is a group access test, do the supplementary groups */
	if (td->uind == 1 && td->gind == 0)
	{
		globok = 0;
		dosuppgrps(t45_74_file, td->amode, SYSERROR, EACCES);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(t45_74_file);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
test75()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test75");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	tbaccess(longname, F_OK, SYSERROR, ENAMETOOLONG, NOPRIV);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test75()
{
	DBUG_ENTER("test75");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test76()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test76");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	tbaccess(longname, F_OK, SYSERROR, ENAMETOOLONG, NOPRIV);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test76()
{
	DBUG_ENTER("test76");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test77()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test77");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = creat(longname, (mode_t)MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname);
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = 0;
		tbaccess(longname, F_OK, SUCCEED, NOERROR, NOPRIV);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test77()
{
	DBUG_ENTER("test77");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test78()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test78");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = creat(longname, (mode_t)MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname);
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = 0;
		tbaccess(longname, F_OK, SUCCEED, NOERROR, NOPRIV);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test78()
{
	DBUG_ENTER("test78");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private char *
strmode(tbamode)
int tbamode;
{
	size_t len;
	static char buf[30];

	if (tbamode == F_OK)
		return "F_OK";

	buf[0] = '\0';
	if (tbamode & R_OK)
		(void) strcat(buf, "R_OK|");
	if (tbamode & W_OK)
		(void) strcat(buf, "W_OK|");
	if (tbamode & X_OK)
		(void) strcat(buf, "X_OK|");
	tbamode &= ~(R_OK|W_OK|X_OK);
	if (tbamode != 0)
	{
		len = strlen(buf);
		(void) sprintf(&buf[len], "%#o", (unsigned)tbamode);
	}

	len = strlen(buf);
	if (buf[len-1] == '|')
		buf[len-1] = '\0';
	
	return buf;
}

private void
dosuppgrps(file, amode, expret, experr)
char *file;
int amode, expret, experr;
{
	int i;
	int	pathok = 0;

	DBUG_ENTER("dosuppgrps");

	for (i = 0; i < ngroups; i++)
	{
		if (suppgrps[i] == gidlist[2])
		{
			PATH_TRACE;
			PATH_TRACE;
			continue;
		}

		if (chown(file, (uid_t)uidlist[1], suppgrps[i]) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("could not change ownership of \"%s\" to uid %d, gid %d",
			    file, uidlist[1], (int)suppgrps[i]);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		tbaccess(file, amode, expret, experr, NOPRIV);
		PATH_FUNC_TRACE;
	}

	PATH_FUNC_RPT(2*ngroups);

	DBUG_VOID_RETURN;
}

private	char	*tbpath;
private	int	tbamode;
private	int	tbexpret;
private	int	tbexperr;
private int	tbpriv;

private void
tbaccess(path, amode, expret, experrno, priv)
char	*path;
int	amode;
int	expret;
int	experrno;
int	priv;
{
	int	pathok = 0;

	DBUG_ENTER("tbaccess");

	tbpath = path;
	tbamode = amode;
	tbexpret = expret;
	tbexperr = experrno;
	tbpriv = priv;

	if (tet_fork(ch_tbacc, NULLFN, WAITTIME, 1) == 1)
		PATH_TRACE;
	else
		testfail++;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_tbacc()
{
	char	*reppath = tbpath;
	int	rval, err;
	char	pathbuf[30];
	struct stat stbuf;
	int	pathok = 0;

	DBUG_LENTER("ch_tbacc", 2);

	globok = 0;

	if (tbpriv == PRIV)
		prvsetup();
	else
		nonprvsetup();

	PATH_FUNC_TRACE;

	errno = NOERROR;
	rval = access(tbpath, tbamode);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	if (tbexpret == 0)
	{
		if (rval == tbexpret)
		{
			PATH_TRACE;
			PATH_FUNC_RPT(2);
			DBUG_EXIT(globok);
		}
	}
	else
	{
		if (rval == tbexpret && errno == tbexperr)
		{
			PATH_TRACE;
			PATH_FUNC_RPT(2);
			DBUG_EXIT(globok);
		}
	}

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char name>", (int)strlen(tbpath));
		reppath = pathbuf;
	}

	err = errno;
	if (testfail == 0)
	{
		xx_rpt(FAILURE);
	}
	in_rpt("access(\"%s\", %s) did not give correct results",
		reppath, strmode(tbamode));
	if (stat(tbpath, &stbuf) != SYSERROR)
	{
		in_rpt("file mode: %#o, owner: %d, group: %d",
			(int)stbuf.st_mode, (int)stbuf.st_uid,
			(int)stbuf.st_gid);
		in_rpt("process real uid: %d, real gid: %d",
		    (int)getuid(), (int)getgid());
	}
	valrpt(tbexpret, rval, tbexperr, err);

	globok = 0; /* ensures unexpected path is noted */
	
	DBUG_EXIT(globok);
}
