/*
*      SCCS:  @(#)  POSIX.os/files/chdir/chdir.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/chdir - T.chdir Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.chdir

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the chdir() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	18 Apr 1989
MODIFICATIONS:	
		Geoff Clare, 8 June 1989
			Add do_access() test.

		Geoff Clare, 29 Mar 1990
			New interface with pathtoolng() and nametoolng().

		David Sawyer
		Thu Jul 26 12:35:54 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		David Sawyer, 21 Oct 1992
			Added TEST_XNFS code.

		Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

		Geoff Clare, 3 Nov 1995
			Changed child timeout for do_access() test.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<pathres.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"chdir_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef chdir
#endif


#define NO_TESTS	15
#define	WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define NOK		02
#define PATH_OK		04

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;

extern	char	*errname();
#if        TEST_XNFS
extern	char	*xnfs_nametoolng();
#else   /* TEST_XNFS */
extern	char	**nametoolng();
#endif   /* TEST_XNFS */
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	void	valrpt();
extern	char	*execpath();
extern	char	*itoa();

public	int	no_tests = NO_TESTS;

private	char	*prog_su =	"chdir_su";
private	char	*t1_file =	"chdir-t.1";
private	char	*t3_file =	"chdir-d.3";
private char	*t5_dir =	"chdir-d.5";
private char	*t5_file =	"chdir-d.5/chdir-t.5";
private	char	*t7_file =	"chdir-t.7";
private char	*t8_ndir =	"chdir-t.8";
private char	*t8_file =	"chdir-t.8/chdir-t.8";
private char	*t9_dir =	"chdir-d.9";
private char	*t9_file =	"chdir-d.9/chdir-t.9";
private	char	*t10_file =	"chdir-t.10";

private int		testfail;
private char		*testdir;
private dev_t		expdev;
private ino_t		expino;
private	struct stat	stbuf;

private void	cln_tests();
private int	tbchdir();

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();

private void	ch1_t5();
private void	ch2_t5();
#if	   TEST_XNFS
private void	ch3_t5();
#endif	/* TEST_XNFS */
private void	ch_t4();
private void	ch_t6();
private void	ch_t7();
private void	ch_t8();
private void	ch_t9();
private void	ch_t10();
private void	ch_t11();
#if	   TEST_XNFS
private void	ch_t12();
private void	ch_t13();
private void	ch_t14();
private void	ch_t15();
#endif	/* TEST_XNFS */

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */


public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	  test7, 7,
	  test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	NULL, 0
};

private void
cln_tests()
{
	char **p;

	DBUG_ENTER("cln_tests");

	(void) rmdir(t1_file);
	for (p = portfnam; *p != NULL; p++)
		(void) rmdir(*p);
	(void) rmdir(t3_file);
	(void) chmod(t5_dir, MODEANY);
	(void) rmdir(t5_file);
	(void) rmdir(t5_dir);
	(void) rmdir(t7_file);
	(void) unlink(t8_ndir);

	DBUG_VOID_RETURN;
}

private void
do_chdir()
{
	/* call chdir(testdir) and check "." has the right st_dev & st_ino */

	int	err;
	int	pathok = 0;

	DBUG_ENTER("do_chdir");

	globok = 0;
	if (tbchdir(testdir, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_FUNC_TRACE;

	if (stat(".", &stbuf) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (stbuf.st_dev != expdev || stbuf.st_ino != expino)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\".\") after chdir(\"%s\") did not give correct device/inode numbers",
			testdir);
		in_rpt("expected st_dev, st_ino: 0x%lx, %ld; observed: 0x%lx, %ld",
			(long)expdev, (long)expino,
			(long)stbuf.st_dev, (long)stbuf.st_ino);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok ==  3)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* pathname resolution */

	char	*p1, *p2;
	int	npairs, i, err;
	int	retval;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_CREAT);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_CREAT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		if (stat(p2, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				p2, err, errname(err));
			continue;
		}
		else
			PATH_TRACE;

		expdev = stbuf.st_dev;
		expino = stbuf.st_ino;

#ifdef TESTBED2
		expdev++;
		expino++;
#endif

		testdir = p2;
		retval = cppair(do_chdir, NULLFN, WAITTIME, NOK | PATH_OK);
		if (retval & NOK)
			testfail++;
		else if (retval & PATH_OK)
			PATH_TRACE;

		testdir = p1;
		retval = cppair(do_chdir, NULLFN, WAITTIME, NOK | PATH_OK);
		if (retval & NOK)
			testfail++;
		else if (retval & PATH_OK)
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(1+(3*npairs));
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames */

	char	**p;
	int	err;
	int 	i;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		if (crdir(*p, (mode_t)MODEANY,
				(int)geteuid(), (int)getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
	}

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		if (stat(*p, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				*p, err, errname(err));
			continue;
		}
		expdev = stbuf.st_dev;
		expino = stbuf.st_ino;

#ifdef TESTBED2
		expdev++;
		expino++;
#endif

		testdir = *p;
/* PATH_OK is included in this mask as other calls to do_chdir use tracing */
		if (cppair(do_chdir, NULLFN, WAITTIME, NOK | PATH_OK) == NOK)
			testfail++;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	for (p = portfnam; *p != NULL; p++)
		(void) rmdir(*p);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* chdir() makes named directory the current directory */

	int 	err;
	int	retval;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	if (crdir(t3_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (stat(t3_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t3_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	expdev = stbuf.st_dev;
	expino = stbuf.st_ino;

#ifdef TESTBED2
	expdev++;
	expino++;
#endif

	testdir = t3_file;
	retval = cppair(do_chdir, NULLFN, WAITTIME, NOK | PATH_OK);
	if (retval & NOK)
		testfail++;
	else if (retval & PATH_OK)
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) rmdir(t3_file);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* chdir() succeeds if search access granted */

	int waittime;

	DBUG_ENTER("test4");

	waittime = WAITTIME * (5 + sysconf(_SC_NGROUPS_MAX)/4);

	(void) cppair(ch_t4, NULLFN, waittime, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t4");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* search permission denied for ANY component of path NAME */

	int	err;
	mode_t	mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
	int	retval;
	int	pathok = 0;
#if	   TEST_XNFS
	int	rc;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test5");

	testfail = 0;

	if (crdir(t5_dir, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t5_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t5_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (chmod(t5_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t5_dir, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	retval = cppair(ch1_t5, NULLFN, WAITTIME, NOK | PATH_OK);
	if (retval & NOK)
		testfail++;
	else if (retval & PATH_OK)
		PATH_TRACE;

	retval = cppair(ch2_t5, NULLFN, WAITTIME, NOK | PATH_OK);
	if (retval & NOK)
		testfail++;
	else if (retval & PATH_OK)
		PATH_TRACE;

	(void) chmod(t5_dir, MODEANY);
	(void) rmdir(t5_file);
	(void) rmdir(t5_dir);

#if	   TEST_XNFS

	if (tet_sync(SP_T5_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch3_t5, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T5_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */


	if (testfail == 0)
		PATH_XS_RPT(8);


	DBUG_VOID_RETURN;
}

private void
ch1_t5()
{
	int	pathok = 0;

	/* path prefix */

	DBUG_ENTER("ch1_t5");

	globok = 0;
	(void) tbchdir(t5_file, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (testfail > 0)
		DBUG_EXIT(NOK);

	if (pathok == 1)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}

private void
ch2_t5()
{
	int	pathok = 0;

	/* path name itself */

	DBUG_ENTER("ch2_t5");

	globok = 0;
	(void) tbchdir(t5_dir, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (testfail > 0)
		DBUG_EXIT(NOK);

	if (pathok == 1)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}



#if	   TEST_XNFS
private void
ch3_t5()
{
	int err;
	char *args[5];

	DBUG_ENTER("ch3_t5");

	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test6()
{
	DBUG_ENTER("test6");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test6()
{
	/* path component longer than NAME_MAX */

	DBUG_ENTER("test6");

	testfail = 0;

	(void) cppair(ch_t6, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	char	**longname;
	int	pathok = 0;

	DBUG_ENTER("ch_t6");

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else	
		PATH_FUNC_TRACE;

	if (crdir(longname[0], (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		if (tbchdir(longname[0], SUCCEED, NOERROR) != SYSERROR)
		{
			PATH_FUNC_TRACE;
			globok = 0;
			(void) tbchdir("..", SUCCEED, NOERROR);
			PATH_FUNC_TRACE;
		}
		else
		{
			PATH_TRACE; PATH_TRACE;
		}
	}

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbchdir(longname[1], SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) rmdir(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
test7()
{
	/* path longer than PATH_MAX */

	DBUG_ENTER("test7");

	testfail = 0;

	if (crdir(t7_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t7_file);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t7, NULLFN, WAITTIME, 0);

	(void) rmdir(t7_file);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	char	**longname;
	int	pathok = 0;

	DBUG_ENTER("ch_t7");

	globok = 0;
	longname = pathtoolng(t7_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if (tbchdir(longname[0], SUCCEED, NOERROR) != SYSERROR)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) tbchdir("..", SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}
	else
	{
		PATH_TRACE; PATH_TRACE;
	}

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbchdir(longname[1], SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	clntoolng();

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* component of path is not a directory */

	int	fd;

	DBUG_ENTER("test8");

	testfail = 0;

	if ((fd = crfile(t8_ndir, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t8_ndir);
		DBUG_VOID_RETURN;
	}
	(void) close(fd);

	(void) cppair(ch_t8, NULLFN, WAITTIME, 0);

	(void) unlink(t8_ndir);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t8");

	globok = 0;
	(void) tbchdir(t8_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	/* component of path prefix does not exist */

	DBUG_ENTER("test9");

	testfail = 0;

	if (eaccess(t9_dir, F_OK) == 0 && remove(t9_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t9_dir);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t9, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t9");

	globok = 0;
	(void) tbchdir(t9_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	/* directory does not exist */

	DBUG_ENTER("test10");

	testfail = 0;

	if (eaccess(t10_file, F_OK) == 0 && remove(t10_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t10_file);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t10, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t10");

	globok = 0;
	(void) tbchdir(t10_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test11()
{
	/* path is "" */

	DBUG_ENTER("test11");

	testfail = 0;

	(void) cppair(ch_t11, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t11");

	globok = 0;
	(void) tbchdir("", SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
test12()
{
	DBUG_ENTER("test12");

	testfail = 0;

	(void) cppair(ch_t12, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t12()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("ch_t12");

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchdir(longname, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test12()
{
	DBUG_ENTER("test12");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");

	testfail = 0;

	(void) cppair(ch_t13, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t13()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("ch_t13");

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchdir(longname, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test13()
{
	DBUG_ENTER("test13");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test14()
{
	DBUG_ENTER("test14");

	testfail = 0;

	(void) cppair(ch_t14, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t14()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("ch_t14");

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (crdir(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbchdir(longname, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) rmdir(longname);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test14()
{
	DBUG_ENTER("test14");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test15()
{
	DBUG_ENTER("test15");

	testfail = 0;

	(void) cppair(ch_t15, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t15()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("ch_t15");

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (crdir(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbchdir(longname, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) rmdir(longname);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test15()
{
	DBUG_ENTER("test15");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */



private int
tbchdir(tbpath, expret, experrno)
char *tbpath;
int  expret, experrno;
{
	char	*reppath = tbpath;
	int	rval, obserr, statok, err;
	dev_t	savdev;
	ino_t	savino;
	char	pathbuf[30];
	int	pathok = 0;

	DBUG_LENTER("tbchir", 2);

	if (expret == SYSERROR)
	{
		statok = (stat(".", &stbuf) != SYSERROR);
		if (!statok)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\".\") failed - errno %d (%s)",
				err, errname(err));
		}
		else
		{
			PATH_TRACE;
			savdev = stbuf.st_dev;
			savino = stbuf.st_ino;
		}
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	savdev++;
	savino++;
#endif

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
		PATH_TRACE;
	}
	else
		PATH_TRACE;

	errno = 0;
	rval = chdir(tbpath);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	obserr = errno;

	if (expret != SYSERROR && rval == expret)
	{
		PATH_TRACE;
		PATH_FUNC_RPT(3);
		DBUG_RETURN(rval);
	}
	else if (expret == SYSERROR && rval == expret)
	{
		/* check current directory has not changed */
		
		if (statok && stat(".", &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\".\") failed - errno %d (%s)",
				err, errname(err));
		}
		else if (stbuf.st_dev != savdev || stbuf.st_ino != savino)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to chdir(\"%s\") changed current directory",
				reppath);
		}
		else
			PATH_TRACE;

		if (experrno == obserr)
		{
			PATH_FUNC_RPT(3);
			DBUG_RETURN(rval);
		}
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("chdir(\"%s\") did not give correct results", reppath);
	valrpt(expret, rval, experrno, obserr);
	
	DBUG_RETURN(rval);
}
