/*
*      SCCS:  @(#)  POSIX.os/files/chdir/chdir_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/chdir - chdir_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		chdir_su - used by T.chdir for tests that require privileges

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Must have PRV_SETID and PRV_ACCESS privileges.

	  usage:
		chdir_su test_no ident dbug

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	8 June 1989
MODIFICATIONS:	
		Geoff Clare, 15 Nov 1989
			New do_access() argument

		David Sawyer
		Thu Jul 26 12:36:18 BST 1990
		Added resilience code
		Created subtest4 from main to allow path tracing.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		David Sawyer, 21 Oct 1992
			Added TEST_XNFS code.

		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.
		
		Geoff Clare, 3 Nov 1993
			Save current directory on each call to t4_func()
			when testing XNFS.
		
		Geoff Clare, 7 March 1994
			Only test "path prefix" case in test5.

		Geoff Clare, 7 Sept 1994
			Comment out extra token on #endif line.

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <limits.h>
#include <string.h>
#include <errno.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <xnfs.h>
#include "chdir_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef chdir
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	**environ;

extern	void	set_pname();
extern	void	valrpt();
extern	void	cln_access();
extern	char	*errname();

private	int	t4_func();
private	int	subtest4();
private char	savdir[_POSIX_PATH_MAX];


#if	   TEST_XNFS

private int	subtest5();
private char	*t5_2_file = T5_2_FILE;
private char	*t5_2_dir = T5_2_DIR;

#define NOK		02
#define PATH_OK		04

#endif



public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int	testnum;
	int	rc = OK;

	set_pname(argv[0]);

	if (argc < 1 || argc > 4)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 3)
		vg_dbug = argv[3];
	if (argc >= 2)
		vg_ident = argv[2];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testnum = atoi(argv[1]);
	switch(testnum)
	{
	case 4:
		(void) subtest4();
		break;

#if	   TEST_XNFS

	case 5:
		rc = subtest5();
		break;

#endif	/* TEST_XNFS */

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed: %d", testnum);
		return(OK);
	}

	return(rc);
	/* NOTREACHED */
}

private int
subtest4()
{
	int	pathok = 0;

	DBUG_ENTER("subtest4");

	if (getcwd(savdir, _POSIX_PATH_MAX) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("getcwd() failed - errno %d (%s)",
			errno, errname(errno));
		DBUG_TET_EXIT(OK);
	}
	else
		PATH_TRACE;

	globok = 0;
	if (do_access('d', (mode_t) S_IXUSR, (mode_t) S_IXUSR, t4_func, 
				WAITTIME, SP_T4_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(2);
	}
	
	cln_access();

	DBUG_TET_EXIT(OK);
	/* NOTREACHED */
}

private int
t4_func(file, desc)
char *file, **desc;
{
	int ret, err;
	int	pathok = 0;

	DBUG_ENTER("t4_func");

#if TEST_XNFS
	if (getcwd(savdir, _POSIX_PATH_MAX) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("getcwd() failed - errno %d (%s)",
			errno, errname(errno));
		DBUG_TET_EXIT(OK);
	}
	else
		PATH_TRACE;
#else
		PATH_TRACE;
#endif /* TEST_XNFS */

	errno = NOERROR;
	if ((ret = chdir(file)) != SUCCEED)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("chdir(\"%s\") did not give correct results", file);
		valrpt(SUCCEED, ret, NOERROR, err);
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
		ret = -1;
	}
	else
		PATH_TRACE;

	if (chdir(savdir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: chdir(\"%s\") failed - errno %d (%s)",
			savdir, errno, errname(errno));
		ret = -1;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_RETURN(ret);
}



#if	   TEST_XNFS
private int
subtest5()
{
	int err, ret, rc;
	int pathok = 0;

	DBUG_ENTER("subtest5");

	errno = 0;
	ret = chdir(t5_2_file);
	err = errno;

	if (ret != SYSERROR || err != EACCES)
	{
		xx_rpt(FAILURE);
		in_rpt("chdir(\"%s\") did not give correct results", t5_2_file);
		valrpt(SYSERROR, ret, EACCES, err);
		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;

	rc = OK;
	if (pathok == 1)
		rc |= PATH_OK;

	DBUG_RETURN(rc);
}
#endif	/* TEST_XNFS */

