/*
*      SCCS:  @(#)  POSIX.os/files/chmod/chmod.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/chmod - T.chmod Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.chmod

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the chmod() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	11 Apr 1989
MODIFICATIONS:	Joseph Nave, UniSoft Ltd.  Thu Jun  1 10:38:42 BST 1989
		tests using crfile() were extended to test with mkfifo()

		Geoff Clare, 8 June 1989: add do_access() test.

		Geoff Clare, 15 Nov 1989
			New setuprofs() argument
			New do_access() argument
		
		Geoff Clare, 2 Jan 1990
			Make sure FIFO in test5 has effective GID, not
			parent directory GID, by chown() after mkfifo().

		Geoff Clare, 2 Jan 1990
			Delete FIFO testing from test7 as the tested
			behaviour applies to regular files only.

		Geoff Clare, 3 Apr 1990
			New interface with pathtoolng() and nametoolng().

		David Sawyer
		Thu Jul 26 16:52:18 BST 1990
		Added resiliance code.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 27 June 1991
			VSX_SET_ID_MODES_SUPP=N for unsupported result.

		Peter Skan, 6 Sept 1991
			Added unsetprv(PRV_CHOWN) to normsetup()

		Peter Skan, 14 Oct 1991
			Unset PRV_ACCESS, PRV_SETID and PRV_MOUNT as well
			as PRV_CHOWN in normsetup()
			Unset privileges also in ch2_t7().

		Geoff Clare, 21 Nov 1991
			Check S_ISUID and S_ISGID are supported in FIPS mode.

		Geoff Clare, 9/1/92
			Allow for read-only root file systems in test1.

		Geoff Clare, 16 Apr 1992
			Test FIPS requirements in XPG4 mode also.

		Neil Moses, 9 June 1992
			Check that time stamps are not changed.

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		David Sawyer, 22 Oct 1992
			Added TEST_XNFS code.

		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.

		Geoff Clare, 7-17 March 1994
			Skip FIFO code in test13 when testing XNFS.
			Use xnfs_doroot_c() in test1.

		Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<pathres.h>
#include	<chktimes.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"chmod_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef chmod
#endif


#define NO_TESTS	23
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define MODE_MASK	(mode_t)(S_ISUID|S_ISGID|MODEANY)
#define NOK		02
#define PATH_OK		04

extern	char	*errname();
extern	void	savtimes();
extern	char	*chktimes();
#if	   TEST_XNFS
extern	char	*xnfs_nametoolng();
#else	/* TEST_XNFS */
extern	char	**nametoolng();
#endif	/* TEST_XNFS */
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	void	cln_access();
extern	void	valrpt();

extern	int	vguid1, vggid1, vguid0, vggid0;

public	int	no_tests = NO_TESTS;

private mode_t	exp_modes[] = {
	0,
	S_IRUSR,
	S_IWUSR,
	S_IXUSR,
	S_IRGRP,
	S_IWGRP,
	S_IXGRP,
	S_IROTH,
	S_IWOTH,
	S_IXOTH,
	MODEANY,
	(MODEANY & ~S_IRUSR),
	(MODEANY & ~S_IWUSR),
	(MODEANY & ~S_IXUSR),
	(MODEANY & ~S_IRGRP),
	(MODEANY & ~S_IWGRP),
	(MODEANY & ~S_IXGRP),
	(MODEANY & ~S_IROTH),
	(MODEANY & ~S_IWOTH),
	(MODEANY & ~S_IXOTH),
};
private int	no_mode =	sizeof(exp_modes)/sizeof(exp_modes[0]);


private int	testfail;
private int	privsupp;
private int	privok;
private	char	*t3_file =	"chmod-t.3";
private	char	*t4_file =	"chmod-t.4";
private	char	*t5_file =	"chmod-t.5";
private	char	*t6_file =	"chmod-t.6";
private	char	*t7a_file =	"chmod-t.7a";
private	char	*t7b_file =	"chmod-t.7b";
private	char	*t8_file =	"chmod-t.8";
private char	*t10_dir =	"chmod-d.10";
private char	*t10_file =	"chmod-d.10/chmod-t.10";
#if	   TEST_XNFS
private char	*t10_file2 =	T10_2_FILE;
#endif	/* TEST_XNFS */
private	char	*t12_file =	"chmod-t.12";
private char	*t13_ndir =	"chmod-t.13";
private char	*t13_file =	"chmod-t.13/chmod-t.13";
private char	*t14_dir =	"chmod-d.14";
private char	*t14_file =	"chmod-d.14/chmod-t.14";
private	char	*t15_file =	"chmod-t.15";
private	char	*t17_file =	"chmod-t.17";
#if	   TEST_XNFS
private	char	*t17_file2 =	T17_2_FILE;
#endif	/* TEST_XNFS */
private char	*t18_dir =	"chmod-d.18";
private char	*t18_file =	"chmod-d.18/chmod-t.18";
#if	   TEST_XNFS
private	char	*t23_file =	"chmod-t.23";
#endif	/* TEST_XNFS */

private void	prepare_tests();
private void	cln_tests();

private void	test1();
private void	subtest1();
#if	   TEST_XNFS
private void	ch_t1();
#endif	/* TEST_XNFS */
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	subtest4();
#if	   TEST_XNFS
private void	ch_t4();
#endif	/* TEST_XNFS */
private void	test5();
private void	test6();
private void	subtest6();
#if	   TEST_XNFS
private void	ch_t6();
#endif	/* TEST_XNFS */
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();

private void	ch_t3();
private void	ch_t5();
private void	ch1_t7();
private void	ch2_t7();
private int	t9_func();
private void	ch_t10();
private void	sub2_t17();
private void	ch_t17();
#if	   TEST_XNFS
private int	sub1_t17();
private void	ch_t19();
private void	ch_t20();
private void	ch_t21();
private void	ch_t22();
#endif	/* TEST_XNFS */

private int	tbchmod();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	NULL, 0
};


private void
prepare_tests()
{
	char	*supp;
	int	i;

	DBUG_ENTER("prepare_tests");

	if (setprv(PRV_SETID) != 0)
	{
		for (i = 1; i <= no_tests; i++)
			tet_delete(i, "setprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	}

	/* note chmod and chown privileges are not treated separately */
	/* i.e. the use of "CHOWN" here is not a typo */

	supp = tet_getvar("VSX_PRIV_CHOWN_SUPP");
	privsupp = (supp != NULL && (*supp == 'Y' || *supp == 'y'));
	if (privsupp)
		privok = (setprv(PRV_CHOWN) == 0);

	if(uids()!=0)
	{
		int t;
		for(t=1; t <= no_tests; t++)
		{
			tet_delete(t, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	(void) umask((mode_t)0);

#if TEST_XNFS
	if (xnfs_uids() < 0)
		tet_delete(23, "xnfs_uids() failed");
#endif

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	char **p;

	DBUG_ENTER("cln_tests");

	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);
	(void) unlink(t3_file);
	(void) unlink(t4_file);
	(void) unlink(t5_file);
	(void) unlink(t6_file);
	(void) unlink(t7a_file);
	(void) unlink(t7b_file);
	(void) unlink(t8_file);
	(void) chmod(t10_dir, MODEANY);
	(void) unlink(t10_file);
	(void) rmdir(t10_dir);
	(void) unlink(t12_file);
	(void) unlink(t13_ndir);
	(void) unlink(t17_file);

	DBUG_VOID_RETURN;
}

private void
normsetup()
{
	int	pathok = 0;

	DBUG_ENTER("normsetup");

	/* become a user with no chmod privileges */

	(void) setgid((gid_t)vggid1);
	(void) setuid((uid_t)vguid1);
	if (geteuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user ID to VSX_UID1");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (getegid() != vggid1)
	{
		xx_rpt(DELETION);
		in_rpt("setgid() did not set effective group ID to VSX_GID1");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_CHOWN) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv() did not remove CHOWN privilege");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv() did not remove SETID privilege");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_ACCESS) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv() did not remove ACCESS privilege");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv() did not remove MOUNT privilege");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	DBUG_ENTER("test1");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t1, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest1();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t1()
{
	DBUG_ENTER("ch_t1");

	subtest1();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
subtest1()
{
	/* pathname resolution */

	char	*p1, *p2;
	int	npairs, i, err, experr;
	mode_t	newmode;
	struct stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("subtest1");

	testfail = 0;

	/* Only include the root dir. if privileged chmod is supported */
	if (privsupp)
	{
		if (!privok)
		{
			xx_rpt(DELETION);
			in_rpt("setprv(PRV_CHOWN) failed");
			DBUG_VOID_RETURN;
		}
		npairs = init_pathres(PRES_DIRS|PRES_CREAT);
		PATH_TRACE;
	}
	else
	{
		npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_CREAT);
		PATH_TRACE;
	}

	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		if (privsupp)
			in_rpt("init_pathres(PRES_DIRS|PRES_CREAT) failed");
		else
			in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT|PRES_CREAT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* change the mode using 1st name in each pair, then
	   change it back using the 2nd name.  The change should
	   be fairly innocuous as the root directory is included */

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		if (stat(p2, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				p2, err, errname(err));
			continue;
		}
		else
			PATH_TRACE;

		newmode = (stbuf.st_mode & MODE_MASK) ^ S_IWUSR;
		if (strcmp(p1, "/") == 0 || strcmp(p2, "/") == 0)
			experr = EROFS;
		else
			experr = NOERROR;

#ifndef TESTBED2
		globok = 0;
		if (tbchmod(p1, (mode_t) newmode, SUCCEED, experr, (mode_t) newmode) != SUCCEED)
		{
			PATH_TRACE; PATH_TRACE; PATH_TRACE;
			continue;
		}
		else
			PATH_FUNC_TRACE;
#endif

		/* tbchmod() checks p1 has correct mode, but we want p2 */
		if (stat(p2, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				p2, err, errname(err));
		}
		else if ((stbuf.st_mode & MODE_MASK) != newmode)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("chmod() on \"%s\" did not change mode of \"%s\" as expected",
				p1, p2);
		}
		else
			PATH_TRACE;

		newmode ^= S_IWUSR;	/* go back to original value */

		globok = 0;
		(void) tbchmod(p2, (mode_t) newmode, SUCCEED, NOERROR, (mode_t) newmode);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
#ifndef TESTBED2
		PATH_XS_RPT(2+(4*npairs));
#else
		PATH_XS_RPT(2+(3*npairs));
#endif
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames */

	char	**p;
	int	i, fd, err;
	struct stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	/* Use a different mode for each file */

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		if (i==0)
			PATH_TRACE;
		if (i >= no_mode)
		{
			xx_rpt(DELETION);
			in_rpt("internal error: not enough entries in exp_modes[]");
			vsxerror("not enough entries in exp_modes[]");
			DBUG_VOID_RETURN;
		}

		if ((fd = creat(*p, (mode_t)MODEANY)) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create file \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
		(void) close(fd);
	}

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(*p, (mode_t) exp_modes[i],
				SUCCEED, NOERROR, (mode_t) exp_modes[i]);
	}

	if (testfail > 0)
		DBUG_VOID_RETURN;

#ifdef TESTBED2
	exp_modes[1] ^= S_IWUSR;
#endif

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		if (i==0)
			PATH_TRACE;
		if (stat(*p, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				*p, err, errname(err));
		}
		else if ((stbuf.st_mode & MODE_MASK) != exp_modes[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("incorrect mode on file \"%s\": expected %#o, observed %#o",
				*p, (int)exp_modes[i],
				(int)(stbuf.st_mode & MODE_MASK));
		}
		(void) unlink(*p);
	}

#ifdef TESTBED2
	exp_modes[1] ^= S_IWUSR;
#endif

	if (testfail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}


private void
test3()
{
	/* set permission bits when effective uid matches owner of file */

	DBUG_ENTER("test3");

	testfail = 0;

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t3, WAITTIME, SP_T3_DOROOT_START, SP_T3_DOROOT_END);

#else	/* TEST_XNFS */

	(void) cppair(ch_t3, NULLFN, WAITTIME, NOK);

#endif	/* TEST_XNFS */


	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int	fd, i;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t3");

#if	   TEST_XNFS

	PATH_TRACE;

#else	/* TEST_XNFS */

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

#endif	/* TEST_XNFS */

	if ((fd = crfile(t3_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	
	for (i = 0; i < no_mode; i++)
	{
		globok = 0;
		(void) tbchmod(t3_file, (mode_t) exp_modes[i],
				SUCCEED, NOERROR, (mode_t) exp_modes[i]);
		PATH_FUNC_TRACE;
	}

	(void) unlink(t3_file);


#if	   TEST_XNFS

	PATH_TRACE;
	for (i = 0; i < no_mode; i++)
		PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t3_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	
	for (i = 0; i < no_mode; i++)
	{
		globok = 0;
		(void) tbchmod(t3_file, (mode_t) exp_modes[i],
				SUCCEED, NOERROR, (mode_t) exp_modes[i]);
		PATH_FUNC_TRACE;
	}

	(void) unlink(t3_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(3+(2*no_mode));

	DBUG_VOID_RETURN;
}


private void
test4()
{
	DBUG_ENTER("test4");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t4, WAITTIME, SP_T4_DOROOT_START, SP_T4_DOROOT_END);

#else	/* TEST_XNFS */

	subtest4();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t4()
{
	DBUG_ENTER("ch_t4");

	subtest4();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
subtest4()
{
	/* set permission bits when process has appropriate privilege */

	int	fd, i;
	int	pathok = 0;

	DBUG_ENTER("subtest4");

	testfail = 0;

	if (!privsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_PRIV_CHOWN_SUPP not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!privok)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_CHOWN) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t4_file, (mode_t)MODEANY, vguid1, vggid1)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\" with uid %d, gid %d",
			t4_file, vguid1, vggid1);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	
	for (i = 0; i < no_mode; i++)
	{
		globok = 0;
		(void) tbchmod(t4_file, (mode_t) exp_modes[i],
				SUCCEED, NOERROR, (mode_t) exp_modes[i]);
		PATH_FUNC_TRACE;
	}

	(void) unlink(t4_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	for (i = 0; i < no_mode; i++)
		PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t4_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\" ", t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (chown(t4_file, (uid_t)vguid1, (gid_t)vggid1) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not change ownership of fifo \"%s\" to uid %d, gid %d",
			t4_file, vguid1, vggid1);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	for (i = 0; i < no_mode; i++)
	{
		globok = 0;
		(void) tbchmod(t4_file, (mode_t) exp_modes[i],
				SUCCEED, NOERROR, (mode_t) exp_modes[i]);
		PATH_FUNC_TRACE;
	}

	(void) unlink(t4_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(5+(2*no_mode));

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* set S_ISUID/S_ISGID when file owner matches effective uid
	   and file group matches effective gid or a supplementary gid */

	char *supp;

	DBUG_ENTER("test5");

	testfail = 0;

	supp = tet_getvar("VSX_SET_ID_MODES_SUPP");
	if (supp == NULL || *supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_SET_ID_MODES_SUPP is not set");
		DBUG_VOID_RETURN;
	}
	else if (*supp == 'n' || *supp == 'N')
	{
#if TEST_FIPS || TEST_XPG > 3
		xx_rpt(FAILURE);
		in_rpt("In FIPS and XPG4 modes, parameter VSX_SET_ID_MODES_SUPP must be set to 'Y'");
#else
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_SET_ID_MODES_SUPP is set to 'N'");
#endif
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t5, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	fd, err, ngroups, i;
	gid_t	*suppgrps;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t5");

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	/* effective uid and gid with all test modes */

	if ((fd = crfile(t5_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	/* S_ISUID only */
	for (i = 0; i < no_mode; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(t5_file, (mode_t) (S_ISUID|exp_modes[i]),
			SUCCEED, NOERROR, (mode_t) (S_ISUID|exp_modes[i]));
	}

	/* S_ISGID only */
	for (i = 0; i < no_mode; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(t5_file, (mode_t) (S_ISGID|exp_modes[i]),
			SUCCEED, NOERROR, (mode_t) (S_ISGID|exp_modes[i]));
	}

	/* S_ISUID and S_ISGID */
	for (i = 0; i < no_mode; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(t5_file, (mode_t) (S_ISUID|S_ISGID|exp_modes[i]),
			SUCCEED, NOERROR, (mode_t) (S_ISUID|S_ISGID|exp_modes[i]));
	}

	/* all supplementary groups with S_ISGID */

	errno = 0;
	if ((ngroups = sysconf(_SC_NGROUPS_MAX)) < 0 && errno != 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: sysconf(_SC_NGROUPS_MAX) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ngroups > 0)
	{
		suppgrps = (gid_t *)malloc((size_t)(sizeof(gid_t) * ngroups));
		if (suppgrps == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: malloc() failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if ((ngroups = getgroups(ngroups, suppgrps)) < 0)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: getgroups() failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		for (i = 0; i < ngroups; i++)
		{
			if (i==0)
				PATH_TRACE;
			if (chown(t5_file, geteuid(), suppgrps[i]) == SYSERROR)
			{
				err = errno;
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("deletion reason: chown(\"%s\", %d, %d) failed - errno %d (%s)",
					t5_file, (int)geteuid(),
					(int)suppgrps[i], err, errname(err));
				DBUG_VOID_RETURN;
			}

			(void) tbchmod(t5_file, (mode_t) S_ISGID,
					SUCCEED, NOERROR, (mode_t) S_ISGID);

			(void) tbchmod(t5_file, (mode_t) (S_ISUID|S_ISGID),
				SUCCEED, NOERROR, (mode_t) (S_ISUID|S_ISGID));
		}
		if (i==0)
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE; PATH_TRACE; PATH_TRACE;
	}
	(void) unlink(t5_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	/* effective uid and gid with all test modes */

	if (mkfifo(t5_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (chown(t5_file, geteuid(), getegid()) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: chown(\"%s\", %d, %d) failed - errno %d (%s)",
			t5_file, (int)geteuid(),
			(int)getegid(), err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* S_ISUID only */
	for (i = 0; i < no_mode; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(t5_file, (mode_t) (S_ISUID|exp_modes[i]),
			SUCCEED, NOERROR, (mode_t) (S_ISUID|exp_modes[i]));
	}

	/* S_ISGID only */
	for (i = 0; i < no_mode; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(t5_file, (mode_t) (S_ISGID|exp_modes[i]),
			SUCCEED, NOERROR, (mode_t) (S_ISGID|exp_modes[i]));
	}

	/* S_ISUID and S_ISGID */
	for (i = 0; i < no_mode; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(t5_file, (mode_t) (S_ISUID|S_ISGID|exp_modes[i]),
			SUCCEED, NOERROR, (mode_t) (S_ISUID|S_ISGID|exp_modes[i]));
	}

	/* all supplementary groups with S_ISGID */

	errno = 0;
	if ((ngroups = sysconf(_SC_NGROUPS_MAX)) < 0 && errno != 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: sysconf(_SC_NGROUPS_MAX) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ngroups > 0)
	{
		suppgrps = (gid_t *)malloc((size_t)(sizeof(gid_t) * ngroups));
		if (suppgrps == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: malloc() failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if ((ngroups = getgroups(ngroups, suppgrps)) < 0)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: getgroups() failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		for (i = 0; i < ngroups; i++)
		{
			if (i==0)
				PATH_TRACE;
			if (chown(t5_file, geteuid(), suppgrps[i]) == SYSERROR)
			{
				err = errno;
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("deletion reason: chown(\"%s\", %d, %d) failed - errno %d (%s)",
					t5_file, (int)geteuid(),
					(int)suppgrps[i], err, errname(err));
				DBUG_VOID_RETURN;
			}

			(void) tbchmod(t5_file, (mode_t) S_ISGID,
					SUCCEED, NOERROR, (mode_t) S_ISGID);

			(void) tbchmod(t5_file, (mode_t) (S_ISUID|S_ISGID),
				SUCCEED, NOERROR, (mode_t) (S_ISUID|S_ISGID));
		}
		if (i==0)
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE; PATH_TRACE; PATH_TRACE;
	}
	(void) unlink(t5_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(18);

	DBUG_VOID_RETURN;
}


private void
test6()
{
	DBUG_ENTER("test6");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t6, WAITTIME, SP_T6_DOROOT_START, SP_T6_DOROOT_END);

#else	/* TEST_XNFS */

	subtest6();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t6()
{
	DBUG_ENTER("ch_t6");

	subtest6();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
subtest6()
{
	/* set S_ISUID/S_ISGID when process has appropriate privilege */

	char	*supp;
	int	fd, i;
	int	pathok = 0;

	DBUG_ENTER("subtest6");

	testfail = 0;

	supp = tet_getvar("VSX_SET_ID_MODES_SUPP");
	if (supp == NULL || *supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_SET_ID_MODES_SUPP is not set");
		DBUG_VOID_RETURN;
	}
	else if (*supp == 'n' || *supp == 'N')
	{
#if TEST_FIPS || TEST_XPG > 3
		xx_rpt(FAILURE);
		in_rpt("In FIPS and XPG4 modes, parameter VSX_SET_ID_MODES_SUPP must be set to 'Y'");
#else
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_SET_ID_MODES_SUPP is set to 'N'");
#endif
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!privsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_PRIV_CHOWN_SUPP not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!privok)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_CHOWN) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t6_file, (mode_t)MODEANY, vguid1, vggid1)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\" with uid %d, gid %d",
			t6_file, vguid1, vggid1);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	
	/* S_ISUID only */
	for (i = 0; i < no_mode; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(t6_file, (mode_t) (S_ISUID|exp_modes[i]),
			SUCCEED, NOERROR, (mode_t) (S_ISUID|exp_modes[i]));
	}

	/* S_ISGID only */
	for (i = 0; i < no_mode; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(t6_file, (mode_t) (S_ISGID|exp_modes[i]),
				SUCCEED, NOERROR, (mode_t) (S_ISGID|exp_modes[i]));
	}

	/* S_ISUID and S_ISGID */
	for (i = 0; i < no_mode; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(t6_file, (mode_t) (S_ISUID|S_ISGID|exp_modes[i]),
			SUCCEED, NOERROR, (mode_t) (S_ISUID|S_ISGID|exp_modes[i]));
	}

	(void) unlink(t6_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t6_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\" ", t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (chown(t6_file, (uid_t)vguid1, (gid_t)vggid1) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not change ownership of fifo \"%s\" to uid %d, gid %d",
			t6_file, vguid1, vggid1);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* S_ISUID only */
	for (i = 0; i < no_mode; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(t6_file, (mode_t) (S_ISUID|exp_modes[i]),
			SUCCEED, NOERROR, (mode_t) (S_ISUID|exp_modes[i]));
	}

	/* S_ISGID only */
	for (i = 0; i < no_mode; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(t6_file, (mode_t) (S_ISGID|exp_modes[i]),
			SUCCEED, NOERROR, (mode_t) (S_ISGID|exp_modes[i]));
	}

	/* S_ISUID and S_ISGID */
	for (i = 0; i < no_mode; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchmod(t6_file, (mode_t) (S_ISUID|S_ISGID|exp_modes[i]),
			SUCCEED, NOERROR, (mode_t) (S_ISUID|S_ISGID|exp_modes[i]));
	}

	(void) unlink(t6_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(12);

	(void) unlink(t6_file);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* S_ISGID is cleared when no match with file group */
	/* (applies to regular files only) */

	int	ngroups, err, i;
	gid_t	*suppgrps;
	char	*supp;
	int	retval;
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	supp = tet_getvar("VSX_SET_ID_MODES_SUPP");
	if (supp == NULL || *supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_SET_ID_MODES_SUPP is not set");
		DBUG_VOID_RETURN;
	}
	else if (*supp == 'n' || *supp == 'N')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_SET_ID_MODES_SUPP is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check vsxg1 is not in supplementary groups */

	errno = 0;
	if ((ngroups = sysconf(_SC_NGROUPS_MAX)) < 0 && errno != 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sysconf(_SC_NGROUPS_MAX) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ngroups > 0)
	{
		suppgrps = (gid_t *)malloc((size_t)(sizeof(gid_t) * ngroups));
		if (suppgrps == NULL)
		{
			xx_rpt(DELETION);
			in_rpt("malloc() failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if ((ngroups = getgroups(ngroups, suppgrps)) < 0)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("getgroups() failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		for (i = 0; i < ngroups; i++)
		{
			if (i==0)
				PATH_TRACE;
			if (suppgrps[i] == vggid1)
				break;
		}

		if (i < ngroups)
		{
			xx_rpt(DELETION);
			in_rpt("group VSX_GID1 must not be a supplementary group ID");
			er_rpt("installation error: group VSX_GID1 must not be a supplementary group ID");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
	}

	/* Create files with group vsxg1 */
	retval = cppair(ch1_t7, NULLFN, WAITTIME, NOK | PATH_OK);
	if (retval & NOK)
	{
		DBUG_VOID_RETURN;
	}
	else if (retval & PATH_OK)
		PATH_TRACE;

	/* Run test with effective gid vsxg0 */

	PATH_FUNC_RPT(7); /* set globok for path tracing in child */

	(void) cppair(ch2_t7, NULLFN, WAITTIME, NOK);

	(void) unlink(t7a_file);
	(void) unlink(t7b_file);

	DBUG_VOID_RETURN;
}

private void
ch1_t7()
{
	/* Creat files with group vsxg1.  This is done by a child
	   which calls setgid() rather than by using chown() because
	   PRV_CHOWN may not be supported */

	int fd;
	int	pathok = 0;

	DBUG_ENTER("ch1_t7");

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	if ((fd = crfile(t7a_file, (mode_t)(MODEANY|S_ISGID),
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t7a_file);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if ((fd = crfile(t7b_file, (mode_t)(MODEANY|S_ISGID),
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t7b_file);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (pathok == 3)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}

private void
ch2_t7()
{
	mode_t 	mode = (S_IRWXU|S_IRWXG);
	int	pathok = 0;

	DBUG_ENTER("ch2_t7");

	PATH_FUNC_TRACE; /* checking globok set in parent */

	(void) setgid((gid_t)vggid0);
	(void) setuid((uid_t)vguid1);
	if (geteuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user ID to VSX_UID1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (getegid() != vggid0)
	{
		xx_rpt(DELETION);
		in_rpt("setgid() did not set effective group ID to VSX_GID0");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_CHOWN) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv() did not remove CHOWN privilege");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv() did not remove SETID privilege");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_ACCESS) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv() did not remove ACCESS privilege");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv() did not remove MOUNT privilege");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbchmod(t7a_file, (mode_t) mode, SUCCEED, NOERROR, (mode_t) mode);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchmod(t7b_file, (mode_t) (S_ISGID|mode), SUCCEED, NOERROR, (mode_t) mode);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(9);
	
	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* chmod updates timestamps */

	char	*chkt;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

	if ((fd = creat(t8_file, (mode_t)MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	savtimes(t8_file);
	PATH_FUNC_TRACE;

	/* Use the existing mode: st_ctime should still be updated */
	globok = 0;
	if (tbchmod(t8_file, (mode_t) MODEANY, SUCCEED, NOERROR, (mode_t) MODEANY) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
#ifdef TESTBED2
	globok = 0;
	savtimes(t8_file);
	PATH_FUNC_TRACE;
#endif

	globok = 0;
	chkt = chktimes(t8_file, NEWCTIME);
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("%s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	(void) unlink(t8_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#ifdef TESTBED2
	PATH_TRACE;
#endif
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t8_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	savtimes(t8_file);
	PATH_FUNC_TRACE;

	/* Use the existing mode: st_ctime should still be updated */
	if (tbchmod(t8_file, (mode_t) MODEANY, SUCCEED, NOERROR, (mode_t) MODEANY) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
#ifdef TESTBED2
	globok = 0;
	savtimes(t8_file);
	PATH_FUNC_TRACE;
#endif

	globok = 0;
	chkt = chktimes(t8_file, NEWCTIME);
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("%s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	(void) unlink(t8_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
#ifdef TESTBED2
		PATH_XS_RPT(10);
#else
		PATH_XS_RPT(8);
#endif

	DBUG_VOID_RETURN;
}

private void
test9()
{
	int	pathok = 0;

	/* search permission granted */

	DBUG_ENTER("test9");

	/* give all permissions on bottom dir so can create file */
	/* still has search-only on path prefix */
	globok = 0;
	if (do_access('d', (mode_t) S_IRWXU, (mode_t) S_IXUSR, t9_func, WAITTIME, SP_T9_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(1);
	}
	
	cln_access();

	DBUG_VOID_RETURN;
}

private int
t9_func(dir, desc)
char *dir, **desc;
{
	int fd;
	char file[50];
	int	pathok = 0;
	int	err;

	DBUG_ENTER("t9_func");

	(void) strcat(strcpy(file, dir), "/chmod-t.9");

	if ((fd = creat(file, (mode_t)MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\", errno = %d (%s)", file,
			err, errname(err));
		(void) unlink(file);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	testfail = 0;

	globok = 0;
	(void) tbchmod(file, (mode_t) S_IRWXU, SUCCEED, NOERROR, (mode_t) S_IRWXU);
	PATH_FUNC_TRACE;

	if (testfail != 0)
	{
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
	}
	else
		PATH_FUNC_RPT(2);

	(void) unlink(file);

	DBUG_RETURN(testfail);
}


private void
test10()
{
	/* search permission denied */

	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	if (tet_fork(ch_t10, NULLFN, WAITTIME, 1) == 1)
		PATH_TRACE;

#if	   TEST_XNFS

	if (tet_sync(SP_T10_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbchmod(t10_file2, (mode_t) S_IRWXU, SYSERROR, EACCES, (mode_t) MODEANY);
	PATH_FUNC_TRACE;

	if (tet_sync(SP_T10_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if(!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


private void
ch_t10()
{
	int	fd;
	mode_t	mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
	int	pathok = 0;
	int	t10_exit = 0;

	DBUG_ENTER("ch_t10");

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	if (crdir(t10_dir, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t10_dir);
		DBUG_EXIT(t10_exit);;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t10_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t10_file);
		DBUG_EXIT(t10_exit);;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	if (tbchmod(t10_dir, (mode_t) mode, SUCCEED, NOERROR, (mode_t) mode) != SUCCEED)
	{
		DBUG_EXIT(t10_exit);;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchmod(t10_file, (mode_t) S_IRWXU, SYSERROR, EACCES, (mode_t) MODEANY);
	PATH_FUNC_TRACE;

	(void) chmod(t10_dir, MODEANY);
	(void) unlink(t10_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t10_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t10_file);
		DBUG_EXIT(t10_exit);;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	if (tbchmod(t10_dir, (mode_t) mode, SUCCEED, NOERROR, (mode_t) mode) != SUCCEED)
	{
		DBUG_EXIT(t10_exit);;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchmod(t10_file, (mode_t) S_IRWXU, SYSERROR, EACCES, (mode_t) MODEANY);
	PATH_FUNC_TRACE;

	(void) chmod(t10_dir, MODEANY);
	(void) unlink(t10_file);
	(void) rmdir(t10_dir);

#endif	/* TEST_XNFS */

	if (testfail == 0)
	{
		PATH_FUNC_RPT(8);
		t10_exit = 1;
	}

	DBUG_EXIT(t10_exit);;
}

#if	   TEST_XNFS

private void
test11()
{
	DBUG_ENTER("test11");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test11()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname[0], (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbchmod(longname[0], (mode_t) S_IRWXU, SUCCEED, NOERROR, (mode_t) S_IRWXU);
		PATH_FUNC_TRACE;
		(void) close(fd);
	}

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbchmod(longname[1], (mode_t) S_IRWXU, SYSERROR, ENAMETOOLONG, (mode_t) MODEANY);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test12()
{
	/* path longer than PATH_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test12");

	testfail = 0;

	if ((fd = creat(t12_file, (mode_t)MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	longname = pathtoolng(t12_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchmod(longname[0], (mode_t) S_IRWXU, SUCCEED, NOERROR, (mode_t) S_IRWXU);
	PATH_FUNC_TRACE;

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbchmod(longname[1], (mode_t) S_IRWXU, SYSERROR, ENAMETOOLONG, (mode_t) MODEANY);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	clntoolng();
	(void) unlink(t12_file);

	DBUG_VOID_RETURN;
}


private void
test13()
{
	/* component of path is not a directory */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

	if ((fd = creat(t13_ndir, (mode_t)MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t13_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbchmod(t13_file, (mode_t) S_IRWXU, SYSERROR, ENOTDIR, (mode_t) MODEANY);
	PATH_FUNC_TRACE;

	(void) unlink(t13_ndir);

#if TEST_XNFS
	PATH_TRACE;
	PATH_TRACE;
#else
	if (mkfifo(t13_ndir, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t13_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbchmod(t13_file, (mode_t) S_IRWXU, SYSERROR, ENOTDIR, (mode_t) MODEANY);
	PATH_FUNC_TRACE;

	(void) unlink(t13_ndir);
#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test14()
{
	int	pathok = 0;

	/* component of path prefix does not exist */

	DBUG_ENTER("test14");

	testfail = 0;

	if (eaccess(t14_dir, F_OK) == 0 && remove(t14_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t14_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbchmod(t14_file, (mode_t) S_IRWXU, SYSERROR, ENOENT, (mode_t) MODEANY);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test15()
{
	int	pathok = 0;

	/* file which does not exist */

	DBUG_ENTER("test15");

	testfail = 0;

	if (eaccess(t15_file, F_OK) == 0 && remove(t15_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t15_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbchmod(t15_file, (mode_t) S_IRWXU, SYSERROR, ENOENT, (mode_t) MODEANY);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	int	pathok = 0;

	/* path is "" */

	DBUG_ENTER("test16");

	testfail = 0;

	globok = 0;
	(void) tbchmod("", (mode_t) S_IRWXU, SYSERROR, ENOENT, (mode_t) MODEANY);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test17()
{
	/* file owner does not match effective uid, and no privilege */

	int	pathok = 0;

	DBUG_ENTER("test17");

	testfail = 0;

#if	   TEST_XNFS

	if (sub1_t17() != 0)
		testfail++;
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1);
	xnfs_doroot_c(sub2_t17, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_FUNC_RPT(1);
	sub2_t17();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private int
sub1_t17()
{

	int	pathok = 0;

	DBUG_ENTER("sub1_t17");

	PATH_FUNC_TRACE;

	if (tet_sync(SP_T17_CREATE_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREATE_END failed");
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbchmod(t17_file2, (mode_t) S_IRWXU, SYSERROR, EPERM, (mode_t) MODEANY);
	PATH_FUNC_TRACE;

	if (tet_sync(SP_T17_UNLINK_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to UNLINK_END failed");
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	if(!testfail)
	{
		PATH_FUNC_RPT(3);
		DBUG_RETURN(0);
	}

	DBUG_RETURN(-1);
}

#endif	/* TEST_XNFS */

private void
sub2_t17()
{
	int 	fd;
	int	retval;
	int	pathok = 0;

	DBUG_ENTER("sub2_t17");

	PATH_FUNC_TRACE;

	if ((fd = crfile(t17_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t17_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	retval = cppair(ch_t17, NULLFN, WAITTIME, NOK | PATH_OK);
	if (retval & NOK)
	{
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	}
	else if (retval & PATH_OK)
		PATH_TRACE;

	(void) unlink(t17_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t17_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t17_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	retval = cppair(ch_t17, NULLFN, WAITTIME, NOK | PATH_OK);
	if (retval & NOK)
	{
		(void) unlink(t17_file);
		DBUG_VOID_RETURN;
	}
	else if (retval & PATH_OK)
		PATH_TRACE;

	(void) unlink(t17_file);

#endif	/* TEST_XNFS */

	if(!testfail)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
ch_t17()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t17");

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchmod(t17_file, (mode_t) S_IRWXU, SYSERROR, EPERM, (mode_t) MODEANY);
	PATH_FUNC_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok == 2)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}

private void
test18()
{
	char	*errptr;
	int	pathok = 0;

	/* chmod on read only file system */

	DBUG_ENTER("test18");

	testfail = 0;

	globok = 0;
	if (setuprofs(t18_dir, t18_file, 'f', (mode_t) MODEANY) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(t18_file);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchmod(t18_file, (mode_t) S_IRWXU, SYSERROR, EROFS, (mode_t) MODEANY);
	PATH_FUNC_TRACE;

	globok = 0;
	errptr = chktimes(t18_file, 0);
	PATH_FUNC_TRACE;

	if(errptr != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful chmod() changed time stamps on %s",
			t18_file);
		in_rpt("%s", errptr);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	(void) clnuprofs(t18_dir, t18_file);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
test19()
{
	DBUG_ENTER("test19");

	(void) cppair(ch_t19, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}


private void
ch_t19()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("ch_t19");

	testfail = 0;

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

		globok = 0;
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchmod(longname, (mode_t) S_IRWXU, SYSERROR, ENAMETOOLONG, (mode_t) S_IRWXU);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test19()
{
	DBUG_ENTER("test19");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test20()
{
	DBUG_ENTER("test20");

	(void) cppair(ch_t20, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}


private void
ch_t20()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("ch_t20");

	testfail = 0;

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchmod(longname, (mode_t) S_IRWXU, SYSERROR, ENAMETOOLONG, (mode_t) S_IRWXU);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test20()
{
	DBUG_ENTER("test20");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test21()
{
	DBUG_ENTER("test21");

	(void) cppair(ch_t21, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}


private void
ch_t21()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("ch_t21");

	testfail = 0;

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbchmod(longname, (mode_t) S_IRWXU, SUCCEED, NOERROR, (mode_t) S_IRWXU);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(3);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test21()
{
	DBUG_ENTER("test21");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test22()
{
	DBUG_ENTER("test22");

	(void) cppair(ch_t22, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}


private void
ch_t22()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("ch_t22");

	testfail = 0;

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbchmod(longname, (mode_t) S_IRWXU, SUCCEED, NOERROR, (mode_t) S_IRWXU);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(3);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test22()
{
	DBUG_ENTER("test22");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */



#if	   TEST_XNFS

private void
test23()
{
	/* file owner matches effective uid when mapped */

	int 	fd;
	int	pathok = 0;

	DBUG_ENTER("test23");

	testfail = 0;

	if ((fd = creat(t23_file, (mode_t)MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t23_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tbchmod(t23_file, (mode_t) S_IRWXU, SUCCEED, NOERROR, (mode_t) S_IRWXU);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t23_file);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test23()
{
	DBUG_ENTER("test23");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private int
tbchmod(tbpath, tbmode, expret, experrno, expmode)
char	*tbpath;
mode_t	tbmode;
int	expret;
int	experrno;
mode_t	expmode;
{
	char	*reppath = tbpath;
	int	rval, err;
	char	pathbuf[30];
	struct stat stbuf;
	int	pathok = 0;

	DBUG_LENTER("tbchmod",2);

	errno = NOERROR;
	rval = chmod(tbpath, tbmode);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		PATH_TRACE;
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
	}
	else
		PATH_TRACE;

	if (expret == SUCCEED && rval == expret)
	{
#ifdef TESTBED2
#define expmode (mode_t)(-1)
#endif
		if (stat(tbpath, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				reppath, err, errname(err));
		}
		else if ((stbuf.st_mode & MODE_MASK) != expmode)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("successful call to chmod(\"%s\", %#o) did not change mode of file correctly",
				reppath, (int)tbmode);
			in_rpt("expected: %#o, observed: %#o", (int)expmode,
				(int)(stbuf.st_mode & MODE_MASK));
		}
		else
			PATH_TRACE;

		PATH_FUNC_RPT(2);
		DBUG_RETURN(rval);
	}
	else if (expret == SYSERROR && rval == expret)
	{
		if (stat(tbpath, &stbuf) != SYSERROR &&
				(stbuf.st_mode & MODE_MASK) != expmode)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to chmod(\"%s\", %#o) changed mode of file",
				reppath, (int)tbmode);
#ifdef TESTBED2
#undef expmode
#endif
			(void) chmod(tbpath, expmode);
		}
		else
			PATH_TRACE;
		if (err == experrno)
		{
			PATH_FUNC_RPT(2);
			DBUG_RETURN(rval);
		}
	}
	else if (expret == SUCCEED && experrno == EROFS)
	{
		/* Allow for read-only root file systems */
		if (rval == SYSERROR && err == experrno)
		{
			PATH_TRACE;
			PATH_FUNC_RPT(2);
			DBUG_RETURN(rval);
		}
		else
			experrno = NOERROR; /* for valrpt() */
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("chmod(\"%s\", %#o) did not give correct results",
		reppath, (int)tbmode);
	valrpt(expret, rval, experrno, err);

	if (expret == SYSERROR && rval != expret)
		(void) chmod(tbpath, expmode);
	
	DBUG_RETURN(rval);
}
