/*
*      SCCS:  @(#)  POSIX.os/files/chown/chown.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/chown - T.chown Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.chown

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the chown() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	13 Apr 1989
MODIFICATIONS:
		Geoff Clare, 8 June 1989:  Add do_access() test.
		Geoff Clare, 25 Sep 1989:  unlink() before mkfifo().

		Geoff Clare, 15 Nov 1989
			New setuprofs() argument
			New do_access() argument

		Geoff Clare, 2 Jan 1990
			Delete FIFO testing from test6 as the tested
			behaviour applies to regular files only.

		Geoff Clare, 3 Apr 1990
			New interface with pathtoolng() and nametoolng().
		
		Geoff Clare, 20 Apr 1990
			Was possible for free()'d buffer to be used later.

		David Sawyer
		Fri Jul 27 12:40:27 BST 1990
		Added resilience code.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 27 June 1991
			VSX_INVALID_[GU]ID="unsup" for unsupported result.
			VSX_SET_ID_MODES_SUPP="N" for unsupported result.

		Peter Skan, 9 Sept 1991
			Added unsetprv(PRV_CHOWN) to normsetup()

		Peter Skan, 15 Oct 1991
			Unset all other assigned privileges in normsetup()

		Geoff Clare, 9/1/92
			Allow for read-only root file systems in test1.

		Neil Moses, 8 June 1992
			Check that time stamps are not changed.

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		David Sawyer, 26 October 1992
			Added TEST_XNFS code

		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.
		
		Geoff Clare, 29 Jan 1993
			Uncommented invalid group test in test 19.
			Fixed typographic error in test 12.
			Use current file owner and group in test_setid().
		
		Geoff Clare, 4 June 1993
			Alternative strategy if first chown() gives
			EINVAL in test 21.

		Geoff Clare, 27 Oct 1993
			Implement interpretation XNFS/003/931001.

		Geoff Clare, 3 Nov 1993
			Use creat() instead of crfile() in t8_func().
			When testing XNFS, use current file owner and
			group instead of gete[ug]id() in t8_func().

		Geoff Clare, 2 Mar 1994
			Use group ID -1 in XPG modes in tests 8 and 21.

		Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<pathres.h>
#include	<chktimes.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"chown_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef chown
#endif



#ifndef _XOPEN_SOURCE
#undef strtol
extern	long	strtol();
#endif

#define NO_TESTS	25
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define NOK		02
#define PATH_OK		04

extern	char	*errname();
extern	void	savtimes();
extern	char	*chktimes();
#if	   TEST_XNFS
extern	char	*xnfs_nametoolng();
#else	/* TEST_XNFS */
extern	char	**nametoolng();
#endif	/* TEST_XNFS */
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	void	cln_access();
extern	void	valrpt();
#if	   TEST_XNFS
extern	char	**xnfs_getsv_c();
#endif	/* TEST_XNFS */

extern	int	vguid1, vggid1, vguid0, vggid0;

public	int	no_tests = NO_TESTS;


private int	testfail;
private int	chown_restricted;
private int	privsupp;
private int	privok;
private int	ngroups;
private gid_t	*suppgrps;
private	struct stat stbuf;

private	char	*t3_file =	"chown-t.3";
private	char	*t4a_file =	"chown-t.4a";
private	char	*t4b_file =	"chown-t.4b";
private	char	*t5_file =	"chown-t.5";
#if	  !TEST_XNFS
private	char	*t6_file =	"chown-t.6";
#endif	/* TEST_XNFS */
private	char	*t7_file =	"chown-t.7";
private char	*t9_dir =	"chown-d.9";
private char	*t9_file =	"chown-d.9/chown-t.9";
#if	   TEST_XNFS
private char	*t9_file2 =	T9_2_FILE;
#endif	/* TEST_XNFS */
private	char	*t11_file =	"chown-t.11";
private char	*t12_ndir =	"chown-t.12";
private char	*t12_file =	"chown-t.12/chown-t.12";
private char	*t13_dir =	"chown-d.13";
private char	*t13_file =	"chown-d.13/chown-t.13";
private	char	*t14_file =	"chown-t.14";
private	char	*t16a_file =	"chown-t.16a";
private	char	*t16b_file =	"chown-t.16b";
private	char	*t17a_file =	"chown-t.17a";
private	char	*t17b_file =	"chown-t.17b";
#if	   TEST_XNFS
private	char	*t17_file2 =	T17_2_FILE;
#endif	/* TEST_XNFS */
private char	*t18_dir =	"chown-d.18";
private char	*t18_file =	"chown-d.18/chown-t.18";
private	char	*t19_file =	"chown-t.19";
#if	   TEST_XNFS
private	char	*t20_file =	T20_FILE;
private	char	*t21_file =	T21_FILE;
#endif	/* TEST_XNFS */

private void	prepare_tests();
private void	cln_tests();

private void	test1();
private void	subtest1();
private void	test2();
private void	subtest2();
private void	test3();
private void	subtest3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	subtest7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	subtest11();
private void	test12();
private void	subtest12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	subtest18();
private void	test19();
private void	subtest19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();

#if	   TEST_XNFS
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
#endif	/* TEST_XNFS */
private void	ch1_t4();
private void	ch2_t4();
private void	ch_t5();
#if	  !TEST_XNFS
private void	ch_t6();
#endif	/* TEST_XNFS */
#if	   TEST_XNFS
private void	ch_t7();
#endif	/* TEST_XNFS */
private void	ch_t9();
#if	   TEST_XNFS
private void	ch_t11();
private void	ch_t12();
#endif	/* TEST_XNFS */
private void	ch_t16();
private void	ch1_t17();
private void	ch2_t17();
#if	   TEST_XNFS
private void	ch_t18();
private void	ch_t19();
#endif	/* TEST_XNFS */
private void	ch2_t18();
#if	   TEST_XNFS
private void	ch_t20();
private void	ch_t22();
private void	ch_t23();
private void	ch_t24();
private void	ch_t25();
#endif	/* TEST_XNFS */

private int	t8_func();

private int	is_setid_sup();
private void	test_setid();
private int	tbchown();


public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	NULL, 0
};


private void
prepare_tests()
{
	char	*supp;
	int	i, g;

	DBUG_ENTER("prepare_tests");

	if (setprv(PRV_SETID) != 0)
	{
		for (i = 1; i <= no_tests; i++)
			tet_delete(i, "setprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	}

	errno = 0;
	chown_restricted = (pathconf(".", _PC_CHOWN_RESTRICTED) != -1);
	if (!chown_restricted && errno != 0)
	{
		for (i = 1; i <= no_tests; i++)
			tet_delete(i, "pathconf(\".\", _PC_CHOWN_RESTRICTED) failed");
		DBUG_VOID_RETURN;
	}
#ifdef XRESTRICT
	chown_restricted = !chown_restricted;
#endif

	errno = 0;
	if ((ngroups = sysconf(_SC_NGROUPS_MAX)) < 0 && errno != 0)
	{
		for (i = 1; i <= no_tests; i++)
			tet_delete(i, "sysconf(_SC_NGROUPS_MAX) failed");
		DBUG_VOID_RETURN;
	}


	if(uids()!=0)
	{
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	if (ngroups > 0)
	{
		suppgrps = (gid_t *)malloc((size_t)(sizeof(gid_t) * ngroups));
		if (suppgrps == NULL)
		{
			for (i = 1; i <= no_tests; i++)
				tet_delete(i, "malloc() failed");
			DBUG_VOID_RETURN;
		}
		if ((ngroups = getgroups(ngroups, suppgrps)) < 0)
		{
			for (i = 1; i <= no_tests; i++)
				tet_delete(i, "getgroups() failed");
			DBUG_VOID_RETURN;
		}
		for (g = 0; g < ngroups; g++)
		{
			if (suppgrps[g] == vggid1)
			{
				for (i = 1; i <= no_tests; i++)
					tet_delete(i,
    "installation error: group VSX_GID1 must not be a supplementary group ID");
				DBUG_VOID_RETURN;
			}
		}
	}

	supp = tet_getvar("VSX_PRIV_CHOWN_SUPP");
	privsupp = (supp != NULL && (*supp == 'Y' || *supp == 'y'));
	if (privsupp)
		privok = (setprv(PRV_CHOWN) == 0);

	(void) umask((mode_t)0);

#if TEST_XNFS
	if (xnfs_uids() < 0)
		tet_delete(21, "xnfs_uids() failed");
#endif

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	char **p;

	DBUG_ENTER("cln_tests");

	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);
	(void) unlink(t3_file);
	(void) unlink(t4a_file);
	(void) unlink(t4b_file);
	(void) unlink(t5_file);
	(void) unlink(t7_file);
	(void) chmod(t9_dir, MODEANY);
	(void) unlink(t9_file);
	(void) rmdir(t9_dir);
	(void) unlink(t11_file);
	(void) unlink(t12_ndir);
	(void) unlink(t13_file);
	(void) unlink(t14_file);
	(void) unlink(t16a_file);
	(void) unlink(t16b_file);
	(void) unlink(t17a_file);
	(void) unlink(t17b_file);
	(void) unlink(t18_file);
	(void) unlink(t19_file);

	DBUG_VOID_RETURN;
}

private void
normsetup()
{
	int	pathok = 0;

	DBUG_ENTER("normsetup");

	/* become a user with no chown privileges */

	(void) setgid((gid_t)vggid0);
	(void) setuid((uid_t)vguid0);
	if (geteuid() != vguid0)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user ID to VSX_UID0");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (getegid() != vggid0)
	{
		xx_rpt(DELETION);
		in_rpt("setgid() did not set effective group ID to VSX_GID0");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_CHOWN) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_CHOWN) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_MOUNT) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_ACCESS) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ACCESS) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(6);

	DBUG_VOID_RETURN;
}


private void
test1()
{
	DBUG_ENTER("test1");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t1, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest1();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t1()
{
	DBUG_ENTER("ch_t1");

	subtest1();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


private void
subtest1()
{
	/* pathname resolution */

	char	*p1, *p2;
	int	npairs, i, err, experr;
	gid_t	oldgrp;
	struct stat statb;
	int	pathok = 0;

	DBUG_ENTER("subtest1");

	testfail = 0;

	/* Only include the root dir. if privileged chown is supported */
	if (privsupp)
	{
		if (!privok)
		{
			xx_rpt(DELETION);
			in_rpt("setprv(PRV_CHOWN) failed");
			DBUG_VOID_RETURN;
		}
		PATH_TRACE;
		npairs = init_pathres(PRES_DIRS|PRES_CREAT);
	}
	else
	{
		npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_CREAT);
		PATH_TRACE;
	}

	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		if (privsupp)
			in_rpt("init_pathres(PRES_DIRS|PRES_CREAT) failed");
		else
			in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT|PRES_CREAT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* make a change using 1st name in each pair, then change
	   it back using the 2nd name.  Only the group, not the owner,
	   is changed as the root directory is included */

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		if (stat(p2, &statb) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				p2, err, errname(err));
			continue;
		}
		else
			PATH_TRACE;

		oldgrp = statb.st_gid;
		if (strcmp(p1, "/") == 0 || strcmp(p2, "/") == 0)
			experr = EROFS;
		else
			experr = NOERROR;

#ifndef TESTBED2
		globok = 0;
		if (tbchown(p1, statb.st_uid, (gid_t)vggid1,
					SUCCEED, experr) != SUCCEED)
		{
			PATH_TRACE; PATH_TRACE; PATH_TRACE;
			continue;
		}
		else
			PATH_FUNC_TRACE;
#endif

		/* tbchown() checks p1 has changed, but we want p2 */
		if (stat(p2, &statb) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				p2, err, errname(err));
		}
		else if (statb.st_gid != vggid1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("chown() on \"%s\" did not change \"%s\" as expected",
				p1, p2);
		}
		else
			PATH_TRACE;

		globok = 0;
		(void) tbchown(p2, statb.st_uid, oldgrp, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
#ifndef TESTBED2
		PATH_XS_RPT(2+(4*npairs));
#else
		PATH_XS_RPT(2+(3*npairs));
#endif
	
	cln_pathres();

	DBUG_VOID_RETURN;
}


private void
test2()
{
	DBUG_ENTER("test2");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t2, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest2();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t2()
{
	DBUG_ENTER("ch_t2");

	subtest2();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


private void
subtest2()
{
	/* portable filenames */

	char	**p;
	int	i, fd, err;
	gid_t	*grplist;
	int	pathok = 0;

	DBUG_ENTER("subtest2");

	testfail = 0;

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		if (i==0)
			PATH_TRACE;
		if ((fd = crfile(*p, (mode_t)MODEANY,
				(int)geteuid(), (int)getegid())) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create file \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
		(void) close(fd);
	}

	/* Set a different group for each file. */

	if (chown_restricted && !privsupp)
	{
	   	/* can only set groups from the supplementary groups list */

		if (i > ngroups)
		{
			xx_rpt(DELETION);
			in_rpt("not enough supplementary groups");
			DBUG_VOID_RETURN;
		}
		else
		{
			PATH_TRACE; PATH_TRACE;
		}
		grplist = suppgrps;
	}
	else
	{
		/* can set any groups: use VSX_GID0, VSX_GID0 + 1, etc. */

		if (chown_restricted && !privok)
		{
			xx_rpt(DELETION);
			in_rpt("setprv(PRV_CHOWN) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		grplist = (gid_t *)malloc((size_t)(i * sizeof(gid_t)));
		if (grplist == NULL)
		{
			xx_rpt(DELETION);
			in_rpt("malloc() failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		while (--i >= 0)
			grplist[i] = vggid0 + i;
	}

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) tbchown(*p, geteuid(), grplist[i], SUCCEED, NOERROR);
	}

	if (testfail > 0)
	{
		if (grplist != suppgrps)
			free((void *)grplist);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	grplist[1]++;
#endif

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		if (i==0)
			PATH_TRACE;
		if (stat(*p, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				*p, err, errname(err));
		}
		else if (stbuf.st_gid != grplist[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("incorrect group on file \"%s\": expected %d, observed %d",
				*p, (int)grplist[i], (int)stbuf.st_gid);
		}

		(void) unlink(*p);
	}

#ifdef TESTBED2
	grplist[1]--;
#endif

	if (grplist != suppgrps)
	{
		free((void *)grplist);
		PATH_TRACE;
	}
	else
		PATH_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}


private void
test3()
{
	DBUG_ENTER("test3");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t3, WAITTIME, SP_T3_DOROOT_START, SP_T3_DOROOT_END);

#else	/* TEST_XNFS */

	subtest3();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t3()
{
	DBUG_ENTER("ch_t3");

	subtest3();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
subtest3()
{
	/* process has appropriate privilege */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("subtest3");

	testfail = 0;

	if (!privsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_PRIV_CHOWN_SUPP not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!privok)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_CHOWN) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t3_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	
	globok = 0;
	(void) tbchown(t3_file, (uid_t)vguid1, (gid_t)vggid1, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t3_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t3_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	
	globok = 0;
	(void) tbchown(t3_file, (uid_t)vguid1, (gid_t)vggid1, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t3_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	retval;
	int	pathok = 0;
#if	   TEST_XNFS
	int	server_ch_res;
	char	*cp;
#endif	/* TEST_XNFS */


	/* chown is restricted, requested owner is euid and matches file
	   owner, requested group matches egid or a supplementary group */

	DBUG_ENTER("test4");

	testfail = 0;

#if	   TEST_XNFS

	cp = tet_getvar("XNFS_SERVER_CHOWN_RESTRICTED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter XNFS_SERVER_CHOWN_RESTRICTED is not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	server_ch_res = (*cp == 'Y' || *cp == 'y');

	if (!chown_restricted && !server_ch_res)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_CHOWN_RESTRICTED is not defined on either server or client");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;

	if (!chown_restricted)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_CHOWN_RESTRICTED is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#endif	/* TEST_XNFS */

	/* create file initially with a "disallowed" group ID  */
	globok = 0;
	retval = cppair(ch1_t4, NULLFN, WAITTIME, NOK | PATH_OK);
	if (retval & NOK)
	{
		DBUG_VOID_RETURN;
	}
	else if (retval & PATH_OK)
		PATH_TRACE;

	/* now chown() it to each "allowed" group ID */

	PATH_FUNC_RPT(3); /* pass path tracing status to child via globok */

	(void) cppair(ch2_t4, NULLFN, WAITTIME, NOK);

	(void) unlink(t4a_file);
	(void) unlink(t4b_file);

	DBUG_VOID_RETURN;
}

private void
ch1_t4()
{
	/* Create file with group vsxg1.  This is done by a child
	   which calls setgid() rather than by using chown() because
	   PRV_CHOWN may not be supported */

	int 	fd;
	int	pathok = 0;

	DBUG_ENTER("ch1_t4");

	(void) setgid((gid_t)vggid1);
	(void) setuid((uid_t)vguid0);
	if (geteuid() != vguid0)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user ID to VSX_UID0");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (getegid() != vggid1)
	{
		xx_rpt(DELETION);
		in_rpt("setgid() did not set effective group ID to VSX_GID1");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t4a_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t4a_file);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void) close(fd);

#if	   TEST_XNFS

	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t4b_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t4b_file);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void) close(fd);

#endif	/* TEST_XNFS */

	if (pathok == 4)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}

private void
ch2_t4()
{
	int i;
	int	pathok = 0;
	
	DBUG_ENTER("ch2_t4");

	PATH_FUNC_TRACE; /* checks globok value from parent */

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchown(t4a_file, geteuid(), getegid(), SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	for (i = 0; i < ngroups; i++)
	{
		globok = 0;
		(void) tbchown(t4a_file, geteuid(), suppgrps[i],
				SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

#if	   TEST_XNFS

	PATH_TRACE;
	for (i = 0; i < ngroups; i++)
		PATH_TRACE;

#else	/* TEST_XNFS */

	globok = 0;
	(void) tbchown(t4b_file, geteuid(), getegid(), SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	for (i = 0; i < ngroups; i++)
	{
		globok = 0;
		(void) tbchown(t4b_file, geteuid(), suppgrps[i],
				SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(4+(2*ngroups));

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* chown is not restricted, euid matches file owner */
#if	   TEST_XNFS
	int	server_ch_res;
	char	*cp;
#endif	/* TEST_XNFS */


	DBUG_ENTER("test5");

	testfail = 0;


#if	   TEST_XNFS

	cp = tet_getvar("XNFS_SERVER_CHOWN_RESTRICTED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter XNFS_SERVER_CHOWN_RESTRICTED is not set");
		DBUG_VOID_RETURN;
	}

	server_ch_res = (*cp == 'Y' || *cp == 'y');

	if (chown_restricted || server_ch_res)
	{
		xx_rpt(UNSUPPORTED);
		if (chown_restricted && server_ch_res)
			in_rpt("_POSIX_CHOWN_RESTRICTED is defined on both server and client");
		else
		{
			if (chown_restricted)
				in_rpt("_POSIX_CHOWN_RESTRICTED is defined on client");
			else
				in_rpt("_POSIX_CHOWN_RESTRICTED is defined on server");
		}
		DBUG_VOID_RETURN;
	}

#else	/* TEST_XNFS */

	if (chown_restricted)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_CHOWN_RESTRICTED is defined");
		DBUG_VOID_RETURN;
	}

#endif	/* TEST_XNFS */


	(void) cppair(ch_t5, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	fd;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t5");

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	if ((fd = crfile(t5_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	
	globok = 0;
	(void) tbchown(t5_file, (uid_t)vguid1, (gid_t)vggid1, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t5_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t5_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);
	
	globok = 0;
	(void) tbchown(t5_file, (uid_t)vguid1, (gid_t)vggid1, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t5_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test6()
{
	DBUG_ENTER("test6");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test6()
{
	/* no privilege: set-ID bits are cleared */
	/* (applies to regular files only) */

	DBUG_ENTER("test6");

	if(is_setid_sup() == 0)
		DBUG_VOID_RETURN;

	(void) cppair(ch_t6, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}


private void
ch_t6()
{
	int	fd;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t6");

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	if ((fd = crfile(t6_file, (mode_t)(S_ISUID|S_ISGID|MODEANY),
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	test_setid(t6_file);
	PATH_FUNC_TRACE;

	(void) unlink(t6_file);

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test7()
{
	DBUG_ENTER("test7");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t7, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest7();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t7()
{
	DBUG_ENTER("ch_t7");

	subtest7();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
subtest7()
{
	/* chown updates timestamps */

	char	*chkt;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("subtest7");

	testfail = 0;

	if ((fd = crfile(t7_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t7_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	savtimes(t7_file);
	PATH_FUNC_TRACE;

	/* Use the existing owner & group: st_ctime should still be updated */
	if (tbchown(t7_file, geteuid(), getegid(), SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
#ifdef TESTBED2
	globok = 0;
	savtimes(t7_file);
	PATH_FUNC_TRACE;
#endif

	globok = 0;
	chkt = chktimes(t7_file, NEWCTIME);
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("%s", chkt);
		(void) unlink(t7_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	(void) unlink(t7_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#ifdef TESTBED2
	PATH_TRACE;
#endif

	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t7_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t7_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	savtimes(t7_file);
	PATH_FUNC_TRACE;

	/* Use the existing owner & group: st_ctime should still be updated */
	globok = 0;
	if (tbchown(t7_file, geteuid(), getegid(), SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
#ifdef TESTBED2
	globok = 0;
	savtimes(t7_file);
	PATH_FUNC_TRACE;
#endif

	globok = 0;
	chkt = chktimes(t7_file, NEWCTIME);
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("%s", chkt);
	}
	else
		PATH_FUNC_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
#ifdef TESTBED2
		PATH_XS_RPT(10);
#else
		PATH_XS_RPT(8);
#endif

	(void) unlink(t7_file);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	int	pathok = 0;

	/* search permission granted */

	DBUG_ENTER("test8");

	/* give all permissions on bottom dir so can create file */
	/* still has search-only on path prefix */

	globok = 0;
	if (do_access('d', (mode_t) S_IRWXU, (mode_t) S_IXUSR, t8_func, WAITTIME, SP_T8_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(1);
	}
	
	cln_access();

	DBUG_VOID_RETURN;
}

private int
t8_func(dir, desc)
char *dir, **desc;
{
	int 	fd, err;
	char 	file[50];
	uid_t	uid;
	gid_t	gid;
	int	pathok = 0;

	DBUG_ENTER("t8_func");

	(void) strcat(strcpy(file, dir), "/chown-t.8");

	if ((fd = creat(file, (mode_t)MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(\"%s\") failed - errno %d (%s)",
			file, err, errname(err));
		(void) unlink(file);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	testfail = 0;

#if TEST_XNFS

	if (stat(file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			file, err, errname(err));
		(void) unlink(file);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	
	uid = stbuf.st_uid;
#ifdef _XOPEN_SOURCE
	gid = -1;
#else
	gid = stbuf.st_gid; /* may cause EPERM (grey area) */
#endif

	errno = 0;
	if (chown(file, uid, gid) == SYSERROR)
	{
		err = errno;
		if (err != EINVAL || geteuid() != XNFS_MAPPED_UID)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("chown(%s, %lu, %lu) failed - errno %d (%s)",
				file, (unsigned long)uid, (unsigned long)gid,
				err, errname(err));
		}
		else
		{
			/* server's anon uid is not valid on client */
			/* try alternative strategy */

			uid = geteuid();
#ifdef _XOPEN_SOURCE
			gid = -1;
#else
			gid = getegid();
#endif

			errno = 0;
			if (chown(file, uid, gid) == SYSERROR)
			{
				err = errno;
				if (err == EPERM)
				{
					/* server has restricted chown */
					/* so can't test this case */
					PATH_TRACE;
				}
				else
				{
					if (testfail++ == 0)
						xx_rpt(FAILURE);
					in_rpt("chown(%s, %lu, %lu) failed - errno %d (%s)",
						file, (unsigned long)uid,
						(unsigned long)gid,
						err, errname(err));
				}
			}
			else
				PATH_TRACE;
		}
	}
	else
		PATH_TRACE;
#else
	uid = geteuid();
	gid = getegid();
	PATH_TRACE;

	globok = 0;
	(void) tbchown(file, uid, gid, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
#endif

	if (testfail != 0)
	{
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
	}

	PATH_FUNC_RPT(3);

	(void) unlink(file);
	DBUG_RETURN(testfail);
}


private void
test9()
{
	/* search permission denied */

	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	if (tet_fork(ch_t9, NULLFN, WAITTIME, 1) == 1)
		PATH_TRACE;
	else
		++testfail;

#if	   TEST_XNFS

	if (tet_sync(SP_T9_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbchown(t9_file2, geteuid(), getegid(), SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (tet_sync(SP_T9_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if(testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int	fd, err;
	mode_t	mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
	int	pathok = 0;
	int	t9_exit = 0;

	DBUG_ENTER("ch_t9");

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	if (crdir(t9_dir, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t9_dir);
		DBUG_EXIT(t9_exit);
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t9_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t9_file);
		DBUG_EXIT(t9_exit);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (chmod(t9_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t9_file, (int)mode, err, errname(err));
		DBUG_EXIT(t9_exit);
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbchown(t9_file, geteuid(), getegid(), SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) chmod(t9_dir, MODEANY);
	(void) unlink(t9_file);
	(void) rmdir(t9_dir);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (crdir(t9_dir, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t9_dir);
		DBUG_EXIT(t9_exit);
	}
	else
		PATH_TRACE;

	if (mkfifo(t9_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t9_file);
		DBUG_EXIT(t9_exit);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (chmod(t9_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t9_file, (int)mode, err, errname(err));
		DBUG_EXIT(t9_exit);
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbchown(t9_file, geteuid(), getegid(), SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) chmod(t9_dir, MODEANY);
	(void) unlink(t9_file);
	(void) rmdir(t9_dir);

#endif	/* TEST_XNFS */

	if (testfail == 0)
	{
		PATH_FUNC_RPT(9);
		t9_exit = 1;
	}


	DBUG_EXIT(t9_exit);
}

#if	   TEST_XNFS

private void
test10()
{
	DBUG_ENTER("test10");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test10()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname[0], (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbchown(longname[0], geteuid(), getegid(), SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		(void) close(fd);
	}

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbchown(longname[1], geteuid(), getegid(), SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test11()
{
	DBUG_ENTER("test11");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t11, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest11();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t11()
{
	DBUG_ENTER("ch_t11");

	subtest11();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
subtest11()
{
	/* path longer than PATH_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("subtest11");

	testfail = 0;

	if ((fd = crfile(t11_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	longname = pathtoolng(t11_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchown(longname[0], geteuid(), getegid(), SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbchown(longname[1], geteuid(), getegid(), SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	clntoolng();
	(void) unlink(t11_file);

	DBUG_VOID_RETURN;
}



private void
test12()
{
	DBUG_ENTER("test12");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t12, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest12();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t12()
{
	DBUG_ENTER("ch_t12");

	subtest12();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
subtest12()
{
	/* component of path is not a directory */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("subtest12");

	testfail = 0;

	if ((fd = crfile(t12_ndir, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t12_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbchown(t12_file, geteuid(), getegid(), SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	(void) unlink(t12_ndir);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t12_ndir, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t12_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbchown(t12_file, geteuid(), getegid(), SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	(void) unlink(t12_ndir);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test13()
{
	int	pathok = 0;

	/* component of path prefix does not exist */

	DBUG_ENTER("test13");

	testfail = 0;

	if (eaccess(t13_dir, F_OK) == 0 && remove(t13_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t13_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbchown(t13_file, geteuid(), getegid(), SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test14()
{
	int	pathok = 0;

	/* file which does not exist */

	DBUG_ENTER("test14");

	testfail = 0;

	if (eaccess(t14_file, F_OK) == 0 && remove(t14_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbchown(t14_file, geteuid(), getegid(), SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test15()
{
	int	pathok = 0;

	/* path is "" */

	DBUG_ENTER("test15");

	testfail = 0;

	globok = 0;
	(void) tbchown("", geteuid(), getegid(), SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test16()
{
#if	   TEST_XNFS
	int	server_ch_res;
	char	*cp;
#endif	/* TEST_XNFS */

	/* chown is restricted, requested owner is not file owner or
	   requested group is neither egid nor a supplementary group */

	DBUG_ENTER("test16");

	testfail = 0;

#if	   TEST_XNFS

	cp = tet_getvar("XNFS_SERVER_CHOWN_RESTRICTED");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter XNFS_SERVER_CHOWN_RESTRICTED is not set");
		DBUG_VOID_RETURN;
	}

	server_ch_res = (*cp == 'Y' || *cp == 'y');

	if (!chown_restricted && !server_ch_res)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_CHOWN_RESTRICTED is not defined on either server or client");
		DBUG_VOID_RETURN;
	}

#else	/* TEST_XNFS */

	if (!chown_restricted)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_CHOWN_RESTRICTED is not defined");
		DBUG_VOID_RETURN;
	}

#endif	/* TEST_XNFS */

	(void) cppair(ch_t16, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t16()
{
	int	fd;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t16");

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	if ((fd = crfile(t16a_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t16a_file);
		DBUG_VOID_RETURN;
	}
	else
	{
		(void) close(fd);
		globok = 0;
		(void) tbchown(t16a_file, (uid_t)vguid1, getegid(),
				SYSERROR, EPERM);
		PATH_FUNC_TRACE;
	}

	if ((fd = crfile(t16b_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t16b_file);
		DBUG_VOID_RETURN;
	}
	else
	{
		(void) close(fd);
		globok = 0;
		(void) tbchown(t16b_file, geteuid(), (gid_t)vggid1,
				SYSERROR, EPERM);
		PATH_FUNC_TRACE;
	}

	(void) unlink(t16a_file);
	(void) unlink(t16b_file);


#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t16a_file, MODEANY) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create fifo \"%s\"",
			t16a_file);
		DBUG_VOID_RETURN;
	}
	else
	{
		(void) close(fd);
		globok = 0;
		(void) tbchown(t16a_file, (uid_t)vguid1, getegid(),
				SYSERROR, EPERM);
		PATH_FUNC_TRACE;
	}

	if ((fd = crfile(t16b_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t16b_file);
		DBUG_VOID_RETURN;
	}
	else
	{
		(void) close(fd);
		globok = 0;
		(void) tbchown(t16b_file, geteuid(), (gid_t)vggid1,
				SYSERROR, EPERM);
		PATH_FUNC_TRACE;
	}

	(void) unlink(t16a_file);
	(void) unlink(t16b_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test17()
{
	/* file owner does not match effective uid, and no privilege */
	int	pathok = 0;

	DBUG_ENTER("test17");

	testfail = 0;

	if (cppair(ch1_t17, NULLFN, WAITTIME, NOK) == NOK)
		DBUG_VOID_RETURN;

	if (tet_fork(ch2_t17, NULLFN, WAITTIME, 1) == 1)
		PATH_TRACE;

#if	   TEST_XNFS

	if (tet_sync(SP_T17_CREATE_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREATE_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbchown(t17_file2, geteuid(), getegid(), SYSERROR, EPERM);
	PATH_FUNC_TRACE;

	if (tet_sync(SP_T17_UNLINK_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to UNLINK_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if(testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch1_t17()
{
	/* Creat file with owner vsx1.  This is done by a child
	   which calls setuid() rather than by using chown() because
	   PRV_CHOWN may not be supported */

	int fd;

	DBUG_ENTER("ch1_t17");

	(void) setgid((gid_t)vggid0);
	(void) setuid((uid_t)vguid1);
	if (geteuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user ID to VSX_UID1");
		DBUG_EXIT(NOK);
	}
	if (getegid() != vggid0)
	{
		xx_rpt(DELETION);
		in_rpt("setgid() did not set effective group ID to VSX_GID0");
		DBUG_EXIT(NOK);
	}

	if ((fd = crfile(t17a_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t17a_file);
		DBUG_EXIT(NOK);
	}
	(void) close(fd);

	(void) unlink(t17b_file);

#if	  !TEST_XNFS

	if (mkfifo(t17b_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t17b_file);
		DBUG_EXIT(NOK);
	}
	(void) close(fd);

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}

private void
ch2_t17()
{
	int	pathok = 0;
	int	t17_exit = 0;

	DBUG_ENTER("ch2_t17");

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchown(t17a_file, geteuid(), getegid(), SYSERROR, EPERM);
	PATH_FUNC_TRACE;

	(void) unlink(t17a_file);

#if	   TEST_XNFS

	PATH_TRACE;

#else	/* TEST_XNFS */

	globok = 0;
	(void) tbchown(t17b_file, geteuid(), getegid(), SYSERROR, EPERM);
	PATH_FUNC_TRACE;

	(void) unlink(t17b_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
	{
		if(pathok == 3)
			t17_exit = 1;

		PATH_FUNC_RPT(3);
	}

	DBUG_EXIT(t17_exit);
}


private void
test18()
{
	DBUG_ENTER("test18");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t18, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest18();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t18()
{
	DBUG_ENTER("ch_t18");

	subtest18();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


private void
subtest18()
{
	/* chown on read only file system */

	DBUG_ENTER("subtest18");

	testfail = 0;

	if (setuprofs(t18_dir, t18_file, 'f', (mode_t) MODEANY) != 0)
		DBUG_VOID_RETURN;

	(void) cppair(ch2_t18, NULLFN, WAITTIME, NOK);

	(void) clnuprofs(t18_dir, t18_file);

	DBUG_VOID_RETURN;
}

private void
ch2_t18()
{
	char	*errptr;
	int	pathok = 0;

	DBUG_ENTER("ch2_t18");

	(void) setgid((gid_t)vggid1);
	if (getegid() != vggid1)
	{
		xx_rpt(DELETION);
		in_rpt("setgid() did not set effective group ID to VSX_GID1");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(t18_file);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchown(t18_file, geteuid(), getegid(), SYSERROR, EROFS);
	PATH_FUNC_TRACE;

	globok = 0;
	errptr = chktimes(t18_file, 0);
	PATH_FUNC_TRACE;

	if(errptr != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful chown() changed time stamps on %s",
			t18_file);
		in_rpt("%s", errptr);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}


private void
test19()
{
	DBUG_ENTER("test19");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t19, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest19();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t19()
{
	DBUG_ENTER("ch_t19");

	subtest19();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
subtest19()
{
	/* invalid user ID or group ID, if EINVAL is supported */

	char	*inv_uid, *inv_gid;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("subtest19");

	testfail = 0;

	/* If appropriate privilege is not supported and chown is restricted,
	   EINVAL is untestable due to the precedence ambiguity with EPERM */

	if (!privsupp && chown_restricted)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_PRIV_CHOWN_SUPP not set to 'Y' and _POSIX_CHOWN_RESTRICTED is defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (chown_restricted && !privok)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_CHOWN) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Invalid IDs are either both supported or both not supported */

	inv_uid = tet_getvar("VSX_INVALID_UID");
	inv_gid = tet_getvar("VSX_INVALID_GID");
	if ((inv_uid == NULL || *inv_uid == '\0') ||
	    (inv_gid == NULL || *inv_gid == '\0'))
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_INVALID_UID or VSX_INVALID_GID is not set");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(inv_uid, "unsup", (size_t)5) == 0 &&
		 strncmp(inv_gid, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameters VSX_INVALID_UID and VSX_INVALID_GID are set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t19_file, (mode_t)MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t19_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (strncmp(inv_gid, "unsup", (size_t)5) == 0)
	{
		/* If one was supported they both should have been */
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_INVALID_GID is set to \"unsup\", but VSX_INVALID_UID is not");
	}
	else
	{
		globok = 0;
		(void) tbchown(t19_file, geteuid(),
		    (gid_t)strtol(inv_gid, (char **)0, 0), SYSERROR, EINVAL);
		PATH_FUNC_TRACE;
	}

	if (strncmp(inv_uid, "unsup", (size_t)5) == 0)
	{
		/* If one was supported they both should have been */
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_INVALID_UID is set to \"unsup\", but VSX_INVALID_GID is not");
	}
	else
	{
		globok = 0;
		(void) tbchown(t19_file, (uid_t)strtol(inv_uid, (char **)0, 0),
				getegid(), SYSERROR, EINVAL);
		PATH_FUNC_TRACE;
	}

	(void) unlink(t19_file);

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test20()
{
	/* no privilege: set-ID bits are cleared */
	/* (applies to regular files only) */

	DBUG_ENTER("test20");

	if(is_setid_sup() == 0)
		DBUG_VOID_RETURN;

	(void) cppair(ch_t20, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}


private void
ch_t20()
{
	int	fd;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t20");

	testfail = 0;

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	if ((fd = crfile(t20_file, (mode_t)(S_ISUID|S_ISGID|MODEANY),
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t20_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	test_setid(t20_file);
	PATH_FUNC_TRACE;

	(void) unlink(t20_file);

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test20()
{
	DBUG_ENTER("test20");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test21()
{
	/* privileged user is mapped: set-ID bits are cleared */
	/* (applies to regular files only) */

	int	fd, err;
	uid_t	uid;
	gid_t	gid;
	int	pathok = 0;

	DBUG_ENTER("test21");

	testfail = 0;

	if (!privsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_PRIV_CHOWN_SUPP not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!privok)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_CHOWN) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(is_setid_sup() == 0)
	{
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
	}

	/* The file created will be owned by "nobody" */
	if ((fd = creat(t21_file, (mode_t)(S_ISUID|S_ISGID|MODEANY))) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t21_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	/*
	 *  The "root" userid will be mapped to "nobody" when 
	 *  it tries to chown the file
	 */
	
	if (stat(t21_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t21_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	uid = stbuf.st_uid;
#ifdef _XOPEN_SOURCE
	gid = -1;
#else
	gid = stbuf.st_gid; /* may cause EPERM (grey area) */
#endif

	errno = 0;
	if (chown(t21_file, uid, gid) == SYSERROR)
	{
		err = errno;
		if (err != EINVAL)
		{
			xx_rpt(DELETION);
			in_rpt("chown(%s, %lu, %lu) failed - errno %d (%s)",
				t21_file, (unsigned long)uid,
				(unsigned long)gid, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
		{
			/* server's anon uid is not valid on client */
			/* try alternative strategy */

			uid = geteuid();
#ifdef _XOPEN_SOURCE
			gid = -1;
#else
			gid = getegid();
#endif

			errno = 0;
			if (chown(t21_file, uid, gid) == SYSERROR)
			{
				err = errno;
				if (err == EPERM)
				{
					/* server has restricted chown */
					xx_rpt(UNTESTED);
					in_rpt("First strategy gave EINVAL and second gave EPERM");
				}
				else
				{
					xx_rpt(DELETION);
					in_rpt("chown(%s, %lu, %lu) failed - errno %d (%s)",
						t21_file,
						(unsigned long)uid,
						(unsigned long)gid,
						err, errname(err));
				}
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;
		}

	}
	else
		PATH_TRACE;
	
	if (stat(t21_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t21_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	stbuf.st_mode |= S_ISUID|S_ISGID;
#endif

	if ((stbuf.st_mode & S_ISUID) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("chown() did not clear S_ISUID bit");
	}
	else
		PATH_TRACE;

	if ((stbuf.st_mode & S_ISGID) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("chown() did not clear S_ISGID bit");
	}
	else
		PATH_TRACE;


	(void) unlink(t21_file);

	if (testfail == 0)
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test21()
{
	DBUG_ENTER("test21");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */




#if TEST_XNFS

private void
test22()
{
	DBUG_ENTER("test22");

	(void) cppair(ch_t22, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}


private void
ch_t22()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("ch_t22");

	testfail = 0;

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchown(longname, geteuid(), getegid(), SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test22()
{
	DBUG_ENTER("test22");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test23()
{
	DBUG_ENTER("test23");

	(void) cppair(ch_t23, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}


private void
ch_t23()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("ch_t23");

	testfail = 0;

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbchown(longname, geteuid(), getegid(), SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test23()
{
	DBUG_ENTER("test23");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test24()
{
	DBUG_ENTER("test24");

	(void) cppair(ch_t24, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}


private void
ch_t24()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("ch_t24");

	testfail = 0;

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbchown(longname,geteuid(),getegid(),SUCCEED,NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(3);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test24()
{
	DBUG_ENTER("test24");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test25()
{
	DBUG_ENTER("test25");

	(void) cppair(ch_t25, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}


private void
ch_t25()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("ch_t25");

	testfail = 0;

	globok = 0;
	normsetup();
	PATH_FUNC_TRACE;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbchown(longname,geteuid(),getegid(),SUCCEED,NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(3);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test25()
{
	DBUG_ENTER("test25");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */



/*
 *  Is VSX_SET_ID_MODES_SUPP set ?
 *
 *  Returns 1 if set otherwise 0
 */
private int
is_setid_sup()
{
	int	retval = 1;
	char 	*supp;

	DBUG_ENTER("is_setid_sup");

	supp = tet_getvar("VSX_SET_ID_MODES_SUPP");
	if (supp == NULL || *supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_SET_ID_MODES_SUPP is not set");
		retval = 0;
	}
	else if (*supp == 'n' || *supp == 'N')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_SET_ID_MODES_SUPP is set to 'N'");
		retval = 0;
	}

	DBUG_RETURN(retval);
}


/*
 *  chown() the "test_file" and check to see that the SETUID
 *  and SETGID bits are cleared.
 */
private void
test_setid(test_file)
char *test_file;
{
	int	pathok = 0;
	int	err;
	
	DBUG_ENTER("test_setid");

	if (stat(test_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			test_file, err, errname(err));
	}
	else
		PATH_TRACE;

	if (tbchown(test_file, stbuf.st_uid, stbuf.st_gid,
			SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	stbuf.st_mode |= S_ISUID|S_ISGID;
#endif

	if ((stbuf.st_mode & S_ISUID) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("chown() did not clear S_ISUID bit");
	}
	else
		PATH_TRACE;

	if ((stbuf.st_mode & S_ISGID) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("chown() did not clear S_ISGID bit");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}



private int
tbchown(tbpath, tbowner, tbgroup, expret, experrno)
char	*tbpath;
uid_t	tbowner;
gid_t	tbgroup;
int	expret;
int	experrno;
{
	char	*reppath = tbpath;
	int	rval, err, statok;
	uid_t	oldowner;
	gid_t	oldgroup;
	char	pathbuf[30];
	int	pathok = 0;

	DBUG_LENTER("tbchown", 2);

	statok = (stat(tbpath, &stbuf) != SYSERROR);
	if (statok)
	{
		oldowner = stbuf.st_uid;
		oldgroup = stbuf.st_gid;
		PATH_TRACE;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = chown(tbpath, tbowner, tbgroup);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		PATH_TRACE;
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
	}
	else
		PATH_TRACE;

	if (expret == SUCCEED && rval == expret)
	{
#ifdef TESTBED2
#define tbowner (uid_t)(-1)
#define tbgroup (gid_t)(-1)
#endif
		if (stat(tbpath, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				reppath, err, errname(err));
		}
		else if (stbuf.st_uid != tbowner || stbuf.st_gid != tbgroup)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("successful call to chown(\"%s\", %d, %d) did not make correct change",
				reppath, (int)tbowner, (int)tbgroup);
			if (statok)
				in_rpt("original owner: %d, group: %d",
					(int)oldowner, (int)oldgroup);
			if (stbuf.st_uid != tbowner)
				in_rpt("expected owner: %d, observed: %d",
					(int)tbowner, (int)stbuf.st_uid);
			if (stbuf.st_gid != tbgroup)
				in_rpt("expected group: %d, observed: %d",
					(int)tbgroup, (int)stbuf.st_gid);
		}
		else
			PATH_TRACE;

		PATH_FUNC_RPT(3);
		DBUG_RETURN(rval);
	}
	else if (expret == SYSERROR && rval == expret)
	{
#ifdef TESTBED2
#undef tbowner
#undef tbgroup
#define oldowner tbowner
#define oldgroup tbgroup
#endif
		if (statok && stat(tbpath, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				reppath, err, errname(err));
		}
		else if (statok &&
			(stbuf.st_uid != oldowner || stbuf.st_gid != oldgroup))
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to chown(\"%s\", %d, %d) changed ownership of file",
				reppath, (int)tbowner, (int)tbgroup);
			if (stbuf.st_uid != oldowner)
				in_rpt("original owner: %d, observed: %d",
					(int)oldowner, (int)stbuf.st_uid);
			if (stbuf.st_gid != oldgroup)
				in_rpt("original group: %d, observed: %d",
					(int)oldgroup, (int)stbuf.st_gid);
		}
		else
			PATH_TRACE;
#ifdef TESTBED2
#undef oldowner
#undef oldgroup
#endif
		if (err == experrno)
		{
			PATH_FUNC_RPT(3);
			DBUG_RETURN(rval);
		}
	}
	else if (expret == SUCCEED && experrno == EROFS)
	{
		/* Allow for read-only root file systems */
		if (rval == SYSERROR && err == experrno)
		{
			PATH_TRACE;
			PATH_FUNC_RPT(3);
			DBUG_RETURN(rval);
		}
		else
			experrno = NOERROR; /* for valrpt() */
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("chown(\"%s\", %d, %d) did not give correct results",
		reppath, (int)tbowner, (int)tbgroup);
	valrpt(expret, rval, experrno, err);

	DBUG_RETURN(rval);
}
