/*
*      SCCS:  @(#)  POSIX.os/files/chown_X/chown_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/chown_X - T.chown_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.chown_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the chown() system call, X/Open extensions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	13 Apr 1989
MODIFICATIONS:

		David Sawyer, 27 July 1990
			Added resiliance code.

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		David Sawyer, 27 October 1992
			Modifications for use in the XNFS tests.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef chown
#endif


#define NO_TESTS	2
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();
extern	void	valrpt();

public	int	no_tests = NO_TESTS;


private	struct stat stbuf;

private	char	*t1_file =	"chown_X-t.1";
private	char	*t2_file =	"chown_X-t.2";

private void	cln_tests();
private void	test1();
private void	test2();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};


private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) unlink(t1_file);
	(void) unlink(t2_file);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* owner argument is -1 */

	int	fd, rval, err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	if ((fd = crfile(t1_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	
	errno = NOERROR;
	rval = chown(t1_file, (uid_t)-1, getegid());

#ifdef TESTBED
	rval = -1;
	errno++;
#endif

	if (rval != SUCCEED)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("chown(\"%s\", -1, %d) did not give correct results",
			t1_file, (int)getegid());
		valrpt(SUCCEED, rval, NOERROR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
#define geteuid() 0
#endif

	if (stat(t1_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
			t1_file, err, errname(err));
	}
	else if (stbuf.st_uid != geteuid())
	{
		xx_rpt(FAILURE);
		in_rpt("chown(\"%s\", -1, %d) changed file owner to %d",
			t1_file, getegid(), (int)stbuf.st_uid);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

#ifdef TESTBED2
#undef geteuid
#endif

	(void) unlink(t1_file);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* group argument is -1 */

	int	fd, rval, err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	if ((fd = crfile(t2_file, (mode_t)MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	
	errno = NOERROR;
	rval = chown(t2_file, geteuid(), (gid_t)-1);

#ifdef TESTBED
	rval = -1;
	errno++;
#endif

	if (rval != SUCCEED)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("chown(\"%s\", %d, -1) did not give correct results",
			t2_file, (int)geteuid());
		valrpt(SUCCEED, rval, NOERROR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
#define getegid() 0
#endif

	if (stat(t2_file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
			t2_file, err, errname(err));
	}
	else if (stbuf.st_gid != getegid())
	{
		xx_rpt(FAILURE);
		in_rpt("chown(\"%s\", %d, -1) changed file group to %d",
			t2_file, geteuid(), (int)stbuf.st_gid);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

#ifdef TESTBED2
#undef getegid
#endif

	(void) unlink(t2_file);

	DBUG_VOID_RETURN;
}
