/*
*      SCCS:  @(#)  POSIX.os/files/closedir/closedir.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/closedir - T.closedir Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.closedir

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the closedir() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	21 Apr 1989
MODIFICATIONS:	
		Geoff Clare, 29 May 1990
			Allow for unlimited OPEN_MAX

		David Sawyer
		Fri Jul 27 13:10:25 BST 1990
		Added resilience code.

		Geoff Clare, 16 Oct 1990
			Revise strategy in EBADF test (can't do two
			closedir() calls on the same directory stream).

		Geoff Clare, 20 Nov 1990
			Use VSX_CLOSEDIR_EBADF parameter.
			Add a close(0) to EBADF test.

		Geoff Clare, 14 May 1992
			POSIX.3.1 alignment of EBADF test.

		Geoff Clare, 22 Sept 1994
			Remove readdir() call in test 1.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<dirent.h>
#include	<fcntl.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef closedir
#endif


#define NO_TESTS	3

extern	char	*errname();
extern	void	valrpt();

public	int	no_tests = NO_TESTS;

private int	testfail;

private int	tbclosedir();

private void	test1();
private void	test2();
private void	test3();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private void
test1()
{
	DIR *dirp;
	int err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	if ((dirp = opendir(".")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("opendir(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tbclosedir(dirp, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* if DIR uses an fd, closedir() closes it */

	DIR	*dirp;
	int	open_max, fd, err, *fdtab;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0)
	{
		if (errno == 0)
		{
			PATH_TRACE;
			open_max = 256;
		}
		else
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("sysconf(_SC_OPEN_MAX) failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

	fdtab = (int *)malloc((size_t)(open_max * sizeof(int)));
	if (fdtab == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc(%ld) failed", (long)(open_max * sizeof(int)));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (fd = 0; fd < open_max; fd++)
	{
		if (fd==0)
			PATH_TRACE;
		fdtab[fd] = fcntl(fd, F_GETFD);
	}

	if ((dirp = opendir(".")) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("opendir(\".\") failed - errno %d (%s)",
			err, errname(err));
		free((void *)fdtab);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (fd = 0; fd < open_max; fd++)
	{
		if (fd==0)
			PATH_TRACE;
		if (fdtab[fd] == SYSERROR && fcntl(fd, F_GETFD) != SYSERROR)
			break;
	}

	free((void *)fdtab);

	if (fd == open_max)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("DIR is not implemented using a file descriptor");
		(void) closedir(dirp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifndef TESTBED2
	globok = 0;
	if (tbclosedir(dirp, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
#endif

	if (fcntl(fd, F_GETFD) != SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("file descriptor opened by opendir() was not closed by closedir()");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
#ifndef TESTBED2
		PATH_XS_RPT(8);
#else
		PATH_XS_RPT(7);
#endif
	
	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* dirp is not an open directory stream */

	char	*cp;
	DIR	*dirp2;
	int	open_max, i, *fdtab, dir_fd;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	cp = tet_getvar("VSX_CLOSEDIR_EBADF");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_CLOSEDIR_EBADF is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_CLOSEDIR_EBADF is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0)
	{
		if (errno == 0)
		{
			PATH_TRACE;
			open_max = 256;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("sysconf(_SC_OPEN_MAX) returned non-positive value");
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

	fdtab = (int *)malloc((size_t)(open_max * sizeof(int)));
	if (fdtab == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc(%ld) failed", (long)(open_max * sizeof(int)));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < open_max; i++)
		fdtab[i] = fcntl(i, F_GETFD);

	if ((dirp2 = opendir(".")) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("opendir(\".\") failed");
		free((void *)fdtab);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	dir_fd = -1;
	for (i = 0; i < open_max; i++)
	{
		if (fdtab[i] == SYSERROR && fcntl(i, F_GETFD) != SYSERROR)
		{
			dir_fd = i;
			break;
		}
	}

	free((void *)fdtab);
	
	if (dir_fd == -1)
	{
		xx_rpt(UNTESTED);
		in_rpt("DIR is not implemented using a file descriptor");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(dir_fd);

	globok = 0;
	(void) tbclosedir(dirp2, SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private int
tbclosedir(tbdirp, expret, experrno)
DIR	*tbdirp;
int	expret;
int	experrno;
{
	int	rval, err;
	int	pathok = 0;
	
	DBUG_LENTER("tbclosedir", 2);

	errno = 0;
	rval = closedir(tbdirp);
	err = errno;

#ifdef TESTBED
	rval = -1;
	err++;
#endif

	if (expret == SUCCEED && rval == SUCCEED)
	{
		PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_RETURN(rval);
	}
	if (expret == SYSERROR && rval == SYSERROR && experrno == err)
	{
		PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_RETURN(rval);
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("closedir() did not give correct results");
	valrpt(expret, rval, experrno, err);

	DBUG_RETURN(rval);
}
