/*
*      SCCS:  @(#)  POSIX.os/files/creat/creat.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/creat - T.creat Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.creat

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the creat() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	23 Mar 1989
MODIFICATIONS:
		Geoff Clare, 8 June 1989:  Add do_access() tests.

		Steve Ratcliffe, 15 Feb 1990
			Add ENFILE test.

		Steve Ratcliffe, 12 Feb 1990
			Add ENOSPC test.

		Geoff Clare, 5 Apr 1990
			New interface with pathtoolng() and nametoolng().
		
		Geoff Clare, 29 May 1990
			Allow for unlimited OPEN_MAX

		Geoff Clare, 19 June 1990
			Add test case for O_TRUNC on terminal

		David Sawyer
		Fri Jul 27 16:43:20 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Stuart Boutell, 9 Apr 1991
			Correct path tracing fault in test 24

		Neil Moses, 22 May 1992
			Added extra checks to tbcreat() to ensure on
			failure that file is not created/truncated.

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

                David Sawyer, 22 Septmber 1992
                        XNFS enhancements

		Geoff Clare, 8 Jan 1993
			Change xnfs_anon_uid from int to uid_t.

		Geoff Clare, 17 Nov 1993
			Fix path tracing problem in ch2_t11().

		Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

		Geoff Clare, 15 Sept 1995
			Check VSX_TERMIOS_BUFFERED in test 11.

		Geoff Clare, 3 Nov 1995
			Changed child timeout for do_access() tests.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<termios.h>
#include	<time.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<sys/stat.h>
#include	<string.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<timeout.h>
#include	<pcts_max.h>
#include	<pathres.h>
#include	<chktimes.h>
#include 	<tet_api.h>
#include	<xnfs.h>
#include	"creat_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef creat
#endif



#define NO_TESTS	39		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)
#define NOK		02
#define PATH_OK		04
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define WRSIZE		10		/* size of write() test buffer */
#define T4_NFILES	3
#define NAP		3

#define T_NOK		-1
#define T_OK		0
#define T_CREAT		1
#define T_TRUNC		2
#define UMASKVAL	(S_IRUSR|S_IWGRP|S_IXOTH)

extern	char	*errname();
extern	void	valrpt();
extern	char	*itoa();
extern	char	*uidtoa();
extern	char	*newstr();
extern	int	n_open_fds();
extern	char	**pathtoolng();
#if	   TEST_XNFS
extern	char	*xnfs_nametoolng();
#else	/* TEST_XNFS */
extern	char	**nametoolng();
#endif	/* TEST_XNFS */
extern	void	clntoolng();
extern	char	*execpath();

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;
extern	int	vguid1;
extern	int	vggid1;


public	int	no_tests = NO_TESTS;

private int	testfail;
private char	tflag;
private int	open_max;
private char	wbuf[WRSIZE];

private char	*prog7 =	"creat_t7";
#if	  !TEST_XNFS
private char	*prog8 =	"creat_t8";
#endif	/* TEST_XNFS */
private char	*prog10 =	"creat_t10";
#if	  !TEST_XNFS
private char	*prog11 =	"creat_t11";
#endif	/* TEST_XNFS */
private char	*prog29 =	"creat_t29";
private char	*prog30 =	"creat_t30";
#if	   TEST_XNFS
private char	*prog31 =	"creat_t31";
private char	*prog32 =	"creat_t32";
private char	*prog33 =	"creat_t33";
private char	*prog34 =	"creat_t34";
#endif	/* TEST_XNFS */
private char	*prog_su =	"creat_su";

private char	*t3_file =	"./creat-t.3";
private char	*t4_files[T4_NFILES+1] = {
				"./creat-t.4a",
				"./creat-t.4b",
				"./creat-t.4c",
				"./creat-t.4d" };
private char	*t5_file =	"./creat-t.5";
private char	*t6_file =	"./creat-t.6";
private char	*t7_file =	"./creat-t.7";
#if	  !TEST_XNFS
private char	*t8_file =	"./creat-t.8";
private char	*t9_file =	"./creat-t.9";
#endif	/* TEST_XNFS */
private char	*t10_file =	"./creat-t.10";
#if	  !TEST_XNFS
private char	*t11_file =	"./creat-t.11";
#endif	/* TEST_XNFS */
private char	*t12_file =	"./creat-t.12";
private char	*t13_file =	"./creat-t.13";
private char	*t14_file =	"./creat-t.14";
private char	*t17_1_dir =	"./creat-d.17.1";
private char	*t17_1_file =	"./creat-d.17.1/creat-t.17.1";
#if	   TEST_XNFS
private char	*t17_2_file =	T17_2_FILE;
#endif	/* TEST_XNFS */
private char	*t18_1_file =	"./creat-t.18.1";
#if	   TEST_XNFS
private char	*t18_2_file =	T18_2_FILE;
#endif	/* TEST_XNFS */
private char	*t19_1_dir =	"./creat-d.19.1";
private char	*t19_1_file =	"./creat-d.19.1/creat-t.19.1";
#if	   TEST_XNFS
private char	*t19_2_file =	T19_2_FILE;
#endif	/* TEST_XNFS */
#if	  !TEST_XNFS
private char	*t20_file =	"./creat-t.20";
#endif
private char	*t21_dir =	"./creat-d.21";
private char	*t22_file =	"./creat-t.22";
private char	*t23_file =	"./creat-t.23";
private char	*t25_dir =	"./creat-d.25";
private char	*t25_file =	"./creat-d.25/creat-t.25";
private char	*t27_1_file =	"./creat-t.27";
private char	*t27_2_file =	"./creat-t.27/creat-t.27.1";
private char	*t28_dir =	"./creat-d.28";
private char	*t29_file =	"./creat-t.29";
private char	*t30_dir =	"./creat-d.30";
#if	   TEST_XNFS
private char	*t31_file =	"./creat-t.31";
private char	*t32_file =	"./creat-t.32";
private char	*t33_mnt =	T33_MNT;
private char	*t35_file =	"./creat-t.35";
#endif	/* TEST_XNFS */


private void	prepare_tests();
private void	clean_tests();
private int	tbcreat();
private int	tbcr_to();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15_16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	subtest28();
private void	test29();
private void	test30();
private void	test31_32();
private void	test33();
private void	test34();
private void	test35();
private void	test36();
private void	test37();
private void	test38();
private void	test39();

private void	ch_t7();
#if	  !TEST_XNFS
private void	ch_t8();
private void	ch_t9();
#endif	/* TEST_XNFS */
private void	ch_t10();
#if	  !TEST_XNFS
private void	ch1_t11();
#endif	/* TEST_XNFS */
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
private void	ch2_t11();
#endif
private void	ch_t15_16();
#if	   TEST_XNFS
private void	ch_t17();
private void	ch_t18();
private void	ch_t19();
#endif	/* TEST_XNFS */
private void	ch_t22();
#if	   TEST_XNFS
private void	ch_t28();
#endif	/* TEST_XNFS */
private void	ch2_t28();
private void	ch_t29();
private void	ch_t30();
#if	   TEST_XNFS
private void	ch_t31_32();
private void	ch_t33();
private void	ch_t34();
private void	ch_t35();
#endif	/* TEST_XNFS */
#if	  !TEST_XNFS
private void	pa_t9();
#endif	/* TEST_XNFS */

private int	chk_creat();
private int	savparent();
private void	chkparent();

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	  test2, 2,
	  test3, 3,
	  test4, 4,
	  test5, 5,
	 test6, 6,
	  test7, 7,
	  test8, 8,
	  test9, 9,
	  test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15_16, 15,
	 test15_16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	 test29, 29,
	 test30, 30,
	 test31_32, 31,
	 test31_32, 32,
	 test33, 33,
	 test34, 34,
	 test35, 35,
	 test36, 36,
	 test37, 37,
	 test38, 38,
	 test39, 39,
	NULL, 0
};

private	void
prepare_tests()
{
	int i;

	DBUG_ENTER("prepare_tests");

	if(uids()!=0)
	{
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	for (i = 0 ; i < WRSIZE ; i++)
		wbuf[i] = i % 100;
	/*
	 *	this so that setuid children can write to files
	 */
	(void) umask((mode_t)0);

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0)
	{
		if (errno == 0)
			open_max = PCTS_OPEN_MAX+1;
		else
		{
			for (i = 1; i <= no_tests; i++)
				tet_delete(i,
					"sysconf(_SC_OPEN_MAX) failed during prep");
		}
	}

#if TEST_XNFS
	if (xnfs_uids() < 0)
		tet_delete(32, "xnfs_uids() failed");
#endif
	
	DBUG_VOID_RETURN;
}

private	void
clean_tests()
{
	char **p;
	int i;

	DBUG_ENTER("clean_tests");

	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);
	(void) unlink(t3_file);
	for (i = 0; i <= T4_NFILES; i++)
		(void) unlink(t4_files[i]);
	(void) unlink(t5_file);
	(void) unlink(t6_file);
	(void) unlink(t7_file);
#if	  !TEST_XNFS
	(void) unlink(t8_file);
	(void) unlink(t9_file);
#endif	/* TEST_XNFS */
	(void) unlink(t10_file);
#if	  !TEST_XNFS
	(void) unlink(t11_file);
#endif	/* TEST_XNFS */
	(void) unlink(t12_file);
	(void) unlink(t13_file);
	(void) unlink(t14_file);
	(void) rmdir(t17_1_dir);
	(void) unlink(t18_1_file);
	(void) unlink(t19_1_file);
	(void) rmdir(t19_1_dir);
#if	  !TEST_XNFS
	(void) unlink(t20_file);
#endif	/* TEST_XNFS */
	(void) rmdir(t21_dir);
	(void) unlink(t22_file);
	(void) unlink(t23_file);
	(void) unlink(t27_1_file);
	(void) unlink(t27_2_file);
	(void) rmdir(t28_dir);
	(void) unlink(t29_file);
	(void) rmdir(t30_dir);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* pathname resolution */

	char 	*p1, *p2;
	int 	npairs, i, fd1, fd2;
	struct 	stat stbuf1, stbuf2;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	npairs = init_pathres(PRES_FILES);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_FILES) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		globok = 0;
		fd1 = tbcreat(p1, MODEANY, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		fd2 = tbcreat(p2, MODEANY, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;

		if (fd1 < 0 || fd2 < 0)
		{
			(void) close(fd1);
			(void) close(fd2);
			continue;
		}

#ifdef TESTBED2
#define fd1 0
#endif

		if (fstat(fd1, &stbuf1) == SYSERROR ||
		    fstat(fd2, &stbuf2) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: fstat() failed");
		}
		else if (stbuf1.st_dev != stbuf2.st_dev ||
		         stbuf1.st_ino != stbuf2.st_ino)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("creat() did not resolve \"%s\" and \"%s\" to the same file",
				p1, p2);
				
		}
		else
			PATH_TRACE;

#ifdef TESTBED2
#undef fd1
#endif

		(void) close(fd1);
		(void) close(fd2);
	}

	if (testfail == 0)
		PATH_XS_RPT(1+(npairs*3));
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames */

	char	**p;
	int	fd, err;
	mode_t	mode = (S_IRUSR|S_IRGRP|S_IROTH);
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{	
		if (i == 0)
			PATH_TRACE;
		if (eaccess(*p, F_OK) == 0 && remove(*p) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("could not remove \"%s\" - errno %d (%s)",
				*p, err, errname(err));
			DBUG_VOID_RETURN;
		}
	}

#ifdef TESTBED2
	(void) close(crfile(portfnam[0], (mode_t)mode, (int)getuid(), (int)getgid()));
#endif

	/* create files read-only so undistinguished names will fail */

	for (i = 0, p = portfnam; *p != NULL; p++,i++)
	{
		if (i == 0)
			PATH_TRACE;
		fd = tbcreat(*p, mode, SUCCEED, NOERROR);
		(void) close(fd);
	}

	for (i=0,p = portfnam; *p != NULL; p++,i++)
	{
		if (i == 0)
			PATH_TRACE;
		(void) unlink(*p);
	}

	if (testfail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* creat() returns fd used to refer to the opened file */

	int 	fd, err;
	struct 	stat stbuf1, stbuf2;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	globok = 0;
	fd = tbcreat(t3_file, MODEANY, SUCCEED, NOERROR);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	if (fstat(fd, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fstat() failed on fd from creat() - errno %d (%s)",
			err, errname(err));
	}
	else if (stat(t3_file, &stbuf2) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed after file created with creat() - errno %d (%s)",
			err, errname(err));
	}
	else if (stbuf1.st_dev != stbuf2.st_dev ||
	    stbuf1.st_ino != stbuf2.st_ino)
	{
		xx_rpt(FAILURE);
		in_rpt("fd returned by creat() does not refer to correct file");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) close(fd);
	(void) unlink(t3_file);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* creat returns the lowest available file descriptor */

	int 	i, err, fd[T4_NFILES+1];
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	for (i = 0 ; i < T4_NFILES && testfail == 0 ; i++)
	{
		if (i==0)
			PATH_TRACE;
		fd[i] = tbcreat(t4_files[i], MODEANY, SUCCEED, NOERROR);
	}

	for (i = 1; i < T4_NFILES && testfail == 0; i++)
	{
		if (i==1)
			PATH_TRACE;
		if (fd[i-1] >= fd[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("creat() did not return lowest numbered file descriptor");
		}
	}

	if (testfail == 0)
	{
		if (close(fd[0]) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("close() failed - errno %d (%s)",
				err, errname(err));
		}
		else if ((fd[T4_NFILES] = tbcreat(t4_files[T4_NFILES],
				MODEANY, SUCCEED, NOERROR)) != fd[0])
		{
			xx_rpt(FAILURE);
			in_rpt("creat() did not re-use closed file descriptor");
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(3);
		}
	}

	for (i = 1; i <= T4_NFILES; i++)
		(void) close(fd[i]);
	for (i = 0; i <= T4_NFILES; i++)
		(void) unlink(t4_files[i]);

	DBUG_VOID_RETURN;
}


private void
test5()
{
	/* creat sets file pointer to beginning of file */

	int	fd, fd2, err;
	mode_t	mode = (S_IRWXU|S_IRWXG);
	char	cbuf[16];
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	if ((fd = open(t5_file, O_WRONLY|O_CREAT, mode)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_CREAT, %#o) failed - errno %d (%s)",
			t5_file, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, cbuf, sizeof(cbuf)) != sizeof(cbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() failed - errno %d (%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	fd2 = tbcreat(t5_file, MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{
		if (lseek(fd2, (off_t)0, SEEK_CUR) != 0)
		{
			xx_rpt(FAILURE);
			in_rpt("creat() did not set file pointer to 0");
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(4);
		}

		(void) close(fd2);
	}

	(void) close(fd);
	(void) unlink(t5_file);

	DBUG_VOID_RETURN;
}


private void
test6()
{
	/* creat opens files for writing only */

	int	fd;
	int	err;
	char	rbuf[sizeof(wbuf)];
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	globok = 0;
	fd = tbcreat(t6_file, MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail > 0)
	{
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (write(fd, wbuf, sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("write() failed on file opened with creat() - errno %d (%s)",
			err, errname(err));
	}
	else
		PATH_TRACE;

	(void) lseek(fd, (off_t)0, SEEK_SET);
	
	errno = NOERROR;
	if (read(fd, rbuf, sizeof(rbuf)) != SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() succeeded on file opened with creat()");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) close(fd);
	(void) unlink(t6_file);

	DBUG_VOID_RETURN;
}


private void
test7()
{
	/* creat() on existing file does not change mode, ownership,
	   or st_atime. */

	int		fd;
	int		err;

	DBUG_ENTER("test7");
	if ((fd = open(t7_file, O_WRONLY|O_CREAT, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_CREAT, %#lo) failed - errno %d (%s)",
			t7_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	(void) close(fd);

	(void) cppair(ch_t7, NULLFN, WAITTIME, 0);

	(void) unlink(t7_file);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t7");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog7;
	args[1] = t7_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog7), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog7), err, errname(err));

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test8()
{
	DBUG_ENTER("test8");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test8()
{
	/* creat creates file with correct mode, owner, group */

	DBUG_ENTER("test8");

	if (unlink(t8_file) == SYSERROR && errno != ENOENT)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t8_file);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t8, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t8");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog8;
	args[1] = t8_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog8), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog8), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test9()
{
	DBUG_ENTER("test9");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test9()
{
	/* creat on FIFO will block until process opens for reading */

	int 	err = 0;
	int	pathok = 0;

	DBUG_ENTER("test9");

	testfail = 0;

	if (mkfifo(t9_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(%s) failed - errno %d (%s)",
			t9_file, err, errname(err));
		(void) unlink(t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (cppair(ch_t9, pa_t9, WAITTIME, NOK) == NOK)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open() on FIFO with O_RDONLY|O_NONBLOCK failed in child");
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t9_file);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int	fd;

	DBUG_ENTER("ch_t9");

	(void) sleep(WAITTIME/2);

	fd = open(t9_file, O_RDONLY|O_NONBLOCK);
	
	if (fd == SYSERROR)
		DBUG_EXIT(NOK);

	(void) sleep(WAITTIME/2);

	(void) close(fd);

	DBUG_VOID_RETURN;
}

private void
pa_t9()
{
	int 	fd, err;
	unsigned unused_time;
	int	pathok = 0;

	DBUG_ENTER("pa_t9");

	SET_TIMEOUT(WAITTIME)

	fd = creat(t9_file, MODEANY);
	err = errno;
	unused_time = alarm(0);

	CLEAR_ALARM

	if (fd < 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("creat() on FIFO timed out");
		else
			in_rpt("creat() on FIFO failed - errno %d (%s)",
				err, errname(err));
	}
	else if ((WAITTIME - unused_time) < WAITTIME/4)	/* WAITTIME/4 leeway */
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("creat() returned without waiting for FIFO to be opened for reading");
	}
	else
		PATH_TRACE;

	if (fd >= 0)
		(void) close(fd);

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test10()
{
	/* creat() truncates - mode, owner and group unchanged */

	int		fd;
	int		err;
	char		cbuf[16];

	DBUG_ENTER("test10");
	if ((fd = open(t10_file, O_WRONLY|O_CREAT, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_CREAT, %#lo) failed - errno %d (%s)",
			t10_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	if (write(fd, cbuf, sizeof(cbuf)) != sizeof(cbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() failed - errno %d (%s)", err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	(void) close(fd);

	(void) cppair(ch_t10, NULLFN, WAITTIME, 0);

	(void) unlink(t10_file);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t10");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog10;
	args[1] = t10_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog10), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog10), err, errname(err));

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test11()
{
	DBUG_ENTER("test11");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test11()
{
	/* creat() on FIFO: size, mode & owner unchanged */
	/* creat() on terminal: queues are not flushed */

	int	err;
	int	retval;
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	if (mkfifo(t11_file, MODEANY) == SYSERROR)
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(%s) failed - errno %d (%s)",
			t11_file, err, errname(err));
	}
	else 
	{
		PATH_TRACE;
		globok = 0;
		retval = cppair(ch1_t11, NULLFN, 2 * WAITTIME, NOK | PATH_OK);
		if (retval & NOK)
			testfail++;
		else if (retval & PATH_OK)
			PATH_TRACE;
	}

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 0;
	retval = cppair(ch2_t11, NULLFN, 2 * WAITTIME, NOK | PATH_OK);
	if (retval  & NOK)
		testfail++;
	else if (retval & PATH_OK)
		PATH_TRACE;
#endif

	if (testfail == 0)
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
		PATH_XS_RPT(3);
#else
		PATH_XS_RPT(2);
#endif

	(void) unlink(t11_file);

	DBUG_VOID_RETURN;
}

private void
ch1_t11()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch1_t11");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog11;
	args[1] = t11_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog11), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog11), err, errname(err));

	DBUG_EXIT(NOK);
}

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
private void
ch2_t11()
{
	char		*cp;
	int		buffered, ret, err;
	int		tty_fildes, loop_fildes;
	struct termios	tty_termios, loop_termios;
	static char	buf1[] = "data for tty input queue\n";
	static char	buf2[] = "tty output queue data\n";
	unsigned 	len1 = sizeof(buf1)-1;
	unsigned 	len2 = sizeof(buf2)-1;
	char		inbuf[50];
	int		pathok = 0;

	DBUG_ENTER("ch2_t11");

	switch (termios_prep(&tty_fildes, &tty_termios,
				&loop_fildes, &loop_termios))
	{
	case 0:
		PATH_TRACE;
		break;
	
	case 1:
		/* general terminal interface not supported */
		DBUG_PRINTF("ch2_t11", "termios_prep() returned 1");
		DBUG_EXIT(OK | PATH_OK);
	
	default:
		if (testfail++ == 0)
			xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP");
		DBUG_EXIT(NOK);
	}

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_EXIT(NOK);
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if ((ret = write(loop_fildes, buf1, len1)) != len1)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY returned %d, expected %u",
			ret, len1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (buffered && tcflow(tty_fildes, TCOOFF) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (buffered && (ret = write(tty_fildes, buf2, len2)) != len2)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY returned %d, expected %u",
			ret, len2);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		(void) tcflow(tty_fildes, TCOON);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* Input and output queues now both contain data */

	if (creat(tet_getvar("VSX_TERMIOS_TTY"), (mode_t)0) < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("creat() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		(void) tcflow(tty_fildes, TCOON);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	(void) tcflush(tty_fildes, TCIOFLUSH);
#endif

	/* Check creat() did not discard any data in queues */

	if (buffered && tcflow(tty_fildes, TCOON) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOON) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME/2);

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != len1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_TTY returned %d, expected %u",
			ret, len1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(inbuf, buf1, (size_t)len1) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("data read from VSX_TERMIOS_TTY was incorrect");
	}
	else
		PATH_TRACE;

	if (buffered)
	{
		SET_TIMEOUT(WAITTIME/2);

		ret = read(loop_fildes, inbuf, sizeof(inbuf));
		err = errno;

		CLEAR_ALARM

		if (ret != len2)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("read() from VSX_TERMIOS_LOOP returned %d, expected %u",
				ret, len2);
			if (ret < 0)
				in_rpt("errno was set to %d (%s)", err, errname(err));
		}
		else if (strncmp(inbuf, buf2, (size_t)len2) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("data read from VSX_TERMIOS_LOOP was incorrect");
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);

	if (pathok == 11)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}
#endif /* !XPG3 mode */

#endif	/* TEST_XNFS */

private void
test12()
{
	/* opened files are set to remain open across exec calls */

	int 	fd, err, flags;
	int	pathok = 0;

	DBUG_ENTER("test12");

	testfail = 0;

	globok = 0;
	fd = tbcreat(t12_file, MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fd, F_GETFD, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl(F_GETFD) failed - errno %d (%s)",
			err, errname(err));
	}
	else if ((flags & FD_CLOEXEC) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("FD_CLOEXEC was set for opened file");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	(void) close(fd);
	(void) unlink(t12_file);

	DBUG_VOID_RETURN;
}

private void
test13()
{
	/* creat() alters timestamps on new file and parent dir. */

	char	*chkt;
	int	fd, err, fail = 0;
	time_t	t0;
	struct stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T13_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

	globok = 0;
	fd = tbcreat(t13_file, MODEANY, SUCCEED, NOERROR);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	(void) close(fd);

	if (stat(t13_file, &stbuf) == SYSERROR)
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("stat() failed - errno %d (%s)", err, errname(err));
	}
	else
	{
		if (stbuf.st_atime < t0)
			fail |= 01;
		else
			PATH_TRACE;

		if (stbuf.st_ctime < t0)
			fail |= 02;
		else
			PATH_TRACE;

		if (stbuf.st_mtime < t0)
			fail |= 04;
		else
			PATH_TRACE;
		
		if (fail != 0)
		{
			testfail++;
			xx_rpt(FAILURE);
			if (fail & 01)
				in_rpt("st_atime field on created file was not marked for update");
			if (fail & 02)
				in_rpt("st_ctime field on created file was not marked for update");
			if (fail & 04)
				in_rpt("st_mtime field on created file was not marked for update");
		}
	}

	globok = 0;
	chkt = chktimes(".", NEWCTIME|NEWMTIME);
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("parent directory %s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	(void) unlink(t13_file);

	DBUG_VOID_RETURN;
}

private void
test14()
{
	/* creat() alters st_mtime and st_ctime of existing file */

	char	*chkt;
	int	fd, err;
	int	pathok = 0;

	DBUG_ENTER("test14");

	testfail = 0;

	if ((fd = open(t14_file, O_WRONLY|O_CREAT, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_CREAT, %#o) failed - errno %d (%s)",
			t14_file, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	savtimes(t14_file);
	PATH_FUNC_TRACE;

	globok = 0;
	fd = tbcreat(t14_file, MODEANY, SUCCEED, NOERROR);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	(void) close(fd);

#ifdef TESTBED2
	globok = 0;
	savtimes(t14_file);
	PATH_FUNC_TRACE;
#endif

	globok = 0;
	chkt = chktimes(t14_file, NEWCTIME|NEWMTIME);
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("%s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
#ifdef TESTBED2
		PATH_XS_RPT(5);
#else
		PATH_XS_RPT(4);
#endif

	(void) unlink(t14_file);

	DBUG_VOID_RETURN;
}

private void
test15_16()
{
	/* all the do_access() tests */

	int waittime;

	DBUG_ENTER("test15_16");

	waittime = WAITTIME * (5 + sysconf(_SC_NGROUPS_MAX)/4);

	(void) cppair(ch_t15_16, NULLFN, waittime, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t15_16()
{
	int	err;
	char	*args[6];

	DBUG_ENTER("ch_t15_16");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test17()
{
	/* creat file in directory which denies searching */

	int	pathcheck;
	int	pathok = 0;
#if	   TEST_XNFS
	int	rc;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test17");

	testfail = 0;

	if (crdir(t17_1_dir,
		(mode_t)(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH),
		(int) getuid(), (int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not make directory %s", t17_1_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbcreat(t17_1_file, MODEANY, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) unlink(t17_1_file);
	(void) rmdir(t17_1_dir);

#if	   TEST_XNFS

	if (tet_sync(SP_T17_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t17, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T17_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
	}
	else
		PATH_TRACE;


	pathcheck = 5;

#else	/* TEST_XNFS */

	pathcheck = 2;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS
private void
ch_t17()
{
	int err;
	char *args[6];

	DBUG_ENTER("ch_t17");

	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t17_2_file;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

#endif	/* TEST_XNFS */

private void
test18()
{
	/* creat() without write permission */

	int	pathcheck;
	int	fd, err;
	mode_t	mode = (S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	int	pathok = 0;
#if	   TEST_XNFS
	int	rc;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test18");

	testfail = 0;

	if ((fd = open(t18_1_file, O_WRONLY|O_CREAT, mode)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_CREAT, %#o) failed - errno %d (%s)",
			t18_1_file, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(write(fd, "1", 1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	tflag = 't';

	globok = 0;
	(void) tbcreat(t18_1_file, MODEANY, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) unlink(t18_1_file);

#if	   TEST_XNFS

	if (tet_sync(SP_T18_MKFILE2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKFILE2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t18, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T18_RMFILE2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMFILE2_END failed");
	}
	else
		PATH_TRACE;

	pathcheck = 6;

#else	/* TEST_XNFS */

	pathcheck = 3;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t18()
{
	int err;
	char *args[6];

	DBUG_ENTER("ch_t18");

	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t18_2_file;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}
#endif	/* TEST_XNFS */


private void
test19()
{
	/* creat file in directory which denies write permission */

	int	pathcheck;
	int	pathok = 0;
#if	   TEST_XNFS
	int	rc;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test19");

	testfail = 0;

	if (crdir(t19_1_dir,
		(mode_t)(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH),
		(int) getuid(), (int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not make directory %s", t19_1_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	tflag = 'c';

	globok = 0;
	(void) tbcreat(t19_1_file, MODEANY, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) unlink(t19_1_file);
	(void) rmdir(t19_1_dir);


#if	   TEST_XNFS

	if (tet_sync(SP_T19_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t19, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T19_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
	}
	else
		PATH_TRACE;

	pathcheck = 5;

#else	/* TEST_XNFS */

	pathcheck = 2;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t19()
{
	int err;
	char *args[6];

	DBUG_ENTER("ch_t19");

	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t19_2_file;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test20()
{
	DBUG_ENTER("test20");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test20()
{
	/* creat of blocked FIFO is interrupted by signal */

	char	*chkt;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test20");

	testfail = 0;

	if (mkfifo(t20_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("mkfifo() failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (savparent(".") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(t20_file);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbcr_to(t20_file, MODEANY, WAITTIME/2, SYSERROR, EINTR);
	PATH_FUNC_TRACE;

	if (alrm_flag == 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat() on FIFO did not block");
	}
	else
		PATH_TRACE;

	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	chkt = chktimes(t20_file, 0);
	if (chkt != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("file %s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	chkparent(".");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(8);

	(void) unlink(t20_file);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test21()
{
	/* creat() on a directory */

	char	*chkt;
	int	pathok = 0;

	DBUG_ENTER("test21");

	testfail = 0;

	if (crdir(t21_dir, (mode_t) MODEANY, (int) getuid(),
			(int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t21_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (savparent(".") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(t21_dir);
	PATH_FUNC_TRACE;

	tflag = 'x';
	globok = 0;
	(void) tbcreat(t21_dir, MODEANY, SYSERROR, EISDIR);
	PATH_FUNC_TRACE;

	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	chkt = chktimes(t21_dir, 0);
	if (chkt != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("file %s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	chkparent(".");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	(void) rmdir(t21_dir);

	DBUG_VOID_RETURN;
}

private void
test22()
{
	/* OPEN_MAX/PCTS_OPEN_MAX/EMFILE etc. */

	DBUG_ENTER("test22");

	testfail = 0;

	(void) cppair(ch_t22, NULLFN, 4 * WAITTIME, 0);

	(void) unlink(t22_file);

	DBUG_VOID_RETURN;
}

private void
ch_t22()
{
	long	testmax = PCTS_OPEN_MAX;
	int	err, lfd, fd, i, j;
	int	pathok = 0;

	DBUG_ENTER("ch_t22");

	errno = NOERROR;
	lfd = -1;

	if (open_max < PCTS_OPEN_MAX)
		testmax = open_max;

	/* creat lesser of OPEN_MAX and PCTS_OPEN_MAX files successfully */

	tflag = 'x';
	j = 0;
	for (i = n_open_fds(); i < testmax; i++)
	{
		if (j++ == 0)
			PATH_TRACE;
		fd = tbcreat(t22_file, MODEANY, SUCCEED, NOERROR);
		if (testfail > 0)
			break;
		lfd = fd;
	}

	/* Write some data using last successfully returned fd from creat() */
	if(write(lfd, "1", 1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* if OPEN_MAX is testable, next creat gives EMFILE */

	/* Check that file is not truncated */
	if (testfail == 0 && open_max <= PCTS_OPEN_MAX)
	{
		tflag = 't';
		globok = 0;
		(void) tbcreat(t22_file, MODEANY, SYSERROR, EMFILE);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	/* Check that file is not created */
	if (testfail == 0 && open_max <= PCTS_OPEN_MAX)
	{
		tflag = 'c';
		(void) unlink(t22_file);
		globok = 0;
		(void) tbcreat(t22_file, MODEANY, SYSERROR, EMFILE);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	/* no need to close fd's: exit from child does it */

	DBUG_VOID_RETURN;
}

private void
test23()
{
	/* path longer than PATH_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test23");

	testfail = 0;

	globok = 0;
	longname = pathtoolng(t23_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	fd = tbcreat(longname[0], MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (fd >= 0)
		(void) close(fd);

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbcreat(longname[1], MODEANY, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	clntoolng();
	(void) unlink(t23_file);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test24()
{
	DBUG_ENTER("test24");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test24()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test24");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	fd = tbcreat(longname[0], MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (fd >= 0)
		(void) close(fd);

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbcreat(longname[1], MODEANY, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test25()
{
	/* create file in a directory which does not exist */

	int	pathok = 0;

	DBUG_ENTER("test25");

	testfail = 0;

	if (eaccess(t25_dir, F_OK) == 0 && remove(t25_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not remove \"%s\"", t25_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbcr_to(t25_file, MODEANY, WAITTIME, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test26()
{
	/* create "" */

	int	pathok = 0;

	DBUG_ENTER("test26");

	testfail = 0;

	globok = 0;
	(void) tbcreat("", MODEANY, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test27()
{
	/* creat file with component of path prefix not a directory */

	int	fd, err;
	mode_t	mode = (S_IRWXU|S_IRWXG);
	int	pathok = 0;

	DBUG_ENTER("test27");

	testfail = 0;

	if ((fd = open(t27_1_file, O_WRONLY|O_CREAT, mode)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_CREAT, %#o) failed - errno %d (%s)",
			t27_1_file, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbcreat(t27_2_file, MODEANY, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(2);
	}
	else
		(void) unlink(t27_2_file);

	(void) unlink(t27_1_file);

	DBUG_VOID_RETURN;
}



private void
test28()
{
	DBUG_ENTER("test28");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t28, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest28();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t28()
{
	DBUG_ENTER("ch_t28");

	subtest28();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


private void
subtest28()
{
	/* all the EROFS tests */

	DBUG_ENTER("subtest28");

	(void) cppair(ch2_t28, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t28_dir);

	DBUG_VOID_RETURN;
}

private void
ch2_t28()
{
	char	*rofs;
	int	err;
	char	*args[6];

	DBUG_ENTER("ch2_t28");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_ROFS is not defined");
		DBUG_VOID_RETURN;
	}

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t28_dir;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test29()
{
	int 	fd;
	int 	err;

	DBUG_ENTER("test29");

	if ((fd = creat(t29_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("could not creat() %s - errno %d (%s)", t29_file, err,
			errname(err));
		DBUG_VOID_RETURN;
	}
	(void) close(fd);

	(void) cppair(ch_t29, NULLFN, 8 * WAITTIME, 0);

	(void) unlink(t29_file);

	DBUG_VOID_RETURN;
}

private void
ch_t29()
{
	char	*suid, *sgid;
	int 	err;
	char	*args[8];

	DBUG_ENTER("ch_t29");

	testfail = 0;

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	args[0] = prog29;
	args[1] = t29_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = "c";
	args[5] = vg_ident;
	args[6] = vg_dbug;
	args[7] = NULL;

	(void) tet_exec(execpath(prog29), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %d (%s)", err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test30()
{

	DBUG_ENTER("test30");

	(void) cppair(ch_t30, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t30()
{
	int 	err;
	char	*args[5];

	DBUG_ENTER("ch_t30");

	testfail = 0;

	args[0] = prog30;
	args[1] = t30_dir;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;

	(void) tet_exec(execpath(prog30), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
test31_32()
{
	/*
	 *  test31: server does not perform user ID mapping
	 *  test32: server performs user ID mapping
	 */

	char	*file;

	DBUG_ENTER("test31_32");

	switch (tet_thistest) {
	case 31:
		file = t31_file;
		break;
	case 32:
		file = t32_file;
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected tet_thistest = %d",
			tet_thistest);
		DBUG_VOID_RETURN;
	}

	if (eaccess(file, F_OK) == 0 && unlink(file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", file);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t31_32, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t31_32()
{
	char	*file, *prog, *fuid, *suid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t31_32");

	switch (tet_thistest) {
	case 31:
		if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
			DBUG_EXIT(A_NOMEM);
		fuid = suid;
		file = t31_file;
		prog = prog31;
		break;
	case 32:
		suid = "0";
		if ((fuid = newstr(uidtoa(xnfs_anon_uid))) == (char *) NULL)
			DBUG_EXIT(A_NOMEM);
		file = t32_file;
		prog = prog32;
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected tet_thistest = %d",
			tet_thistest);
		DBUG_VOID_RETURN;
	}

	args[0] = prog;
	args[1] = file;
	args[2] = suid;
	args[3] = fuid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog, err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test31_32()
{
	DBUG_ENTER("test31_32");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test33()
{
	/*
	 *  creates file with correct group when GrpID=True
	 *  replaces part of test8 when testing XNFS
	 */

	DBUG_ENTER("test33");

	(void) cppair(ch_t33, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t33_mnt);

	DBUG_VOID_RETURN;
}

private void
ch_t33()
{
	char	*fsys, *sgid;
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t33");

	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	if ((fsys = tet_getvar("VSX_MOUNT_DEV")) == NULL || *fsys == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: VSX_MOUNT_DEV is not defined");
		DBUG_VOID_RETURN;
	}

	if (crdir(t33_mnt, MODEANY, (int) getuid(),
			(int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t33_mnt);
		DBUG_VOID_RETURN;
	}

	args[0] = prog33;
	args[1] = sgid;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog33), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog33, err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test33()
{
	DBUG_ENTER("test33");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test34()
{
	/*
	 *  creates file with correct group when GrpID=False
	 *  replaces part of test8 when testing XNFS
	 */

	DBUG_ENTER("test34");

	(void) cppair(ch_t34, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t34()
{
	char	*sgid;
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t34");

	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	args[0] = prog34;
	args[1] = sgid;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog34), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog34, err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test34()
{
	DBUG_ENTER("test34");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test35()
{
	/*
	 *  creates file with correct modes
	 *  replaces part of test8 when testing XNFS
	 */

	DBUG_ENTER("test35");

	if (eaccess(t35_file, F_OK) == 0 && unlink(t35_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t35_file);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t35, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t35()
{

#define T35_MODEMASK	(S_IRWXU|S_IRWXG|S_IRWXO)
#define T35_UMASKVAL	(S_IRUSR|S_IWGRP|S_IXOTH)

	static mode_t	modes[] = {
		S_IRWXU|S_IRWXG|S_IRWXO,
		S_IXUSR|S_IWGRP|S_IROTH,
		UMASKVAL
	};

#define NMODES	(sizeof modes / sizeof modes[0])

	int err[NMODES], flag[NMODES];
	struct stat statb;
	int fail, fd, i, um;
	int pathok = 0;

	DBUG_ENTER("ch_t35");

	um = T35_UMASKVAL;
	(void) umask((mode_t) um);

	for (i = 0; i < NMODES; i++)
	{
		err[i] = flag[i] = 0;
		if ((fd = creat(t35_file, modes[i])) == SYSERROR)
		{
			err[i] = errno;
			flag[i] |= 01;
			continue;
		}
		else
			PATH_TRACE;

		if (fstat(fd, &statb) == SYSERROR)
		{
			err[i] = errno;
			flag[i] |= 02;
		}
		else
		{
			PATH_TRACE;
			if ((statb.st_mode & T35_MODEMASK) != (~um & modes[i]))
				flag[i] |= 04;
			else
				PATH_TRACE;
		}

		(void) close(fd);
		if (unlink(t35_file) == SYSERROR)
			flag[i] |= 010;
		else
			PATH_TRACE;
	}

	fail = 0;
	for (i = 0; i < NMODES; i++)
	{
		if (i == 0)
			PATH_TRACE;
		if (flag[i])
			fail = 1;
	}

	if (fail)
	{
		xx_rpt(FAILURE);
		for (i = 0; i < NMODES; i++)
		{
			if (flag[i] & 01)
			{
				in_rpt("creat(\"%s\", %#o) failed - errno %d(%s)",
					t35_file, modes[i], err[i],
					errname(err[i]));
				continue;
			}
			if (flag[i] & 02)
				in_rpt("fstat() on file discriptor returned by creat() failed - errno %d(%s)",
					err[i], errname(err[i]));
			if (flag[i] & 04)
				in_rpt("creat(\"%s\", %#o) did not set access bits correctly",
				t35_file, modes[i]);
			if (flag[i] & 010)
				in_rpt("could not remove file %s, created by creat()",
				t35_file);
		}
	}
	else
		PATH_XS_RPT((NMODES * 4) + 1);

	DBUG_VOID_RETURN;

#undef NMODES

}

#else /* TEST_XNFS */

private void
test35()
{
	DBUG_ENTER("test35");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test36()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test36");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbcreat(longname, MODEANY, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test36()
{
	DBUG_ENTER("test36");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test37()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test37");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbcreat(longname, MODEANY, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test37()
{
	DBUG_ENTER("test37");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test38()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test38");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	fd = tbcreat(longname, MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test38()
{
	DBUG_ENTER("test38");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test39()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test39");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	fd = tbcreat(longname, MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test39()
{
	DBUG_ENTER("test39");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private int
tbcreat(tbpath, tbmode, expret, experrno)
char *tbpath;
mode_t tbmode;
int expret, experrno;
{
	char	*reppath = tbpath;
	int	ret, rval, err;
	char	pathbuf[30];
	int	pathok = 0;

	DBUG_LENTER("tbcreat",2);

	errno = NOERROR;
	rval = creat(tbpath, tbmode);

#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;

	if (expret == 0)
	{
		PATH_TRACE;
		if (rval >= 0 && rval < open_max)
		{
			PATH_FUNC_RPT(1);
			DBUG_RETURN(rval);
		}
	}
	else
	{
		PATH_TRACE;

		if (rval == SYSERROR && experrno == err)
		{
			globok = 0;
			ret = chk_creat(tbpath);
			PATH_FUNC_TRACE;

			if(ret == T_OK)
			{
				PATH_FUNC_RPT(2);
				DBUG_RETURN(rval);
			}
		}
	}

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("creat(\"%s\", %#o) did not give correct results",
		reppath, (int)tbmode);
	if (rval < SYSERROR || rval >= open_max)
		in_rpt("invalid return value %d: valid range %d to %d",
			rval, SYSERROR, open_max);

	if (ret == T_CREAT)
		in_rpt("on error file was created");
	else if (ret == T_TRUNC)
		in_rpt("on error file was truncated");

	valrpt(expret, rval, experrno, err);
	if (expret == SYSERROR && rval != SYSERROR)
		(void) close(rval);
	DBUG_RETURN(rval);
}

private int
tbcr_to(tbpath, tbmode, tbwait, expret, experrno)
char *tbpath;
int tbwait;
mode_t tbmode;
int expret, experrno;
{
	char	*reppath = tbpath;
	int	rval, err;
	char	pathbuf[30];
	int	pathok = 0;

	DBUG_ENTER("tbcr_to");

	SET_TIMEOUT(tbwait)

	errno = NOERROR;
	rval = creat(tbpath, tbmode);
	err = errno;

	CLEAR_ALARM

#ifdef TESTBED
	rval = -1;
	if (err)	
		err++;
#endif
	
	if (expret == 0)
	{
		PATH_TRACE;
		if (rval >= 0 && rval < open_max)
		{
			PATH_FUNC_RPT(1);
			DBUG_RETURN(rval);
		}
	}
	else
	{
		PATH_TRACE;
		if (rval == SYSERROR && experrno == err)
		{
			PATH_FUNC_RPT(1);
			DBUG_RETURN(rval);
		}
	}

	/* Don't print PATH_MAX length path name */
	if (experrno == ENAMETOOLONG && strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", strlen(tbpath));
		reppath = pathbuf;
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("creat(\"%s\", %#o) did not give correct results",
		reppath, (int)tbmode);
	if (err == EINTR && alrm_flag > 0)
		in_rpt("creat() call timed out after %d seconds", tbwait);
	if (rval < SYSERROR || rval >= open_max)
		in_rpt("invalid return value %d: valid range %d to %d",
			rval, SYSERROR, open_max);
	valrpt(expret, rval, experrno, err);
	if (expret == SYSERROR && rval != SYSERROR)
		(void) close(rval);

	DBUG_RETURN(rval);
}

private struct stat stpar1, stpar2;

private int
savparent(dir)
char *dir;
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("savparent");

	if (stat(dir, &stpar1) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			dir, err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(0);
}

private void
chkparent(dir)
char *dir;
{
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("chkparent");

	if (stat(dir, &stpar2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stpar2.st_ctime > stpar1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was updated");
	}
	else
		PATH_TRACE;

	if (stpar2.st_mtime > stpar1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was updated");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private int
chk_creat(path)
char	*path;
{
	struct	stat buf;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("chk_creat");

	errno = 0;

	if (tflag == 'c')
	{
		if(eaccess(path, F_OK) != SYSERROR)
			DBUG_RETURN(T_CREAT);
		else
			PATH_TRACE;
	}
	else if (tflag == 't')
	{
		if(stat(path, &buf) < 0)
		{
			err = errno;
			if(!testfail)
				xx_rpt(DELETION);
			in_rpt("stat(%s, buf) failed - errno %d (%s)",
				path, err, errname(err));
			DBUG_RETURN(T_NOK);
		}
		else if(buf.st_size == 0)
			DBUG_RETURN(T_TRUNC);
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(T_OK);
}
