/*
*      SCCS:  @(#)  POSIX.os/files/creat/creat_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/creat - creat_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		creat_su - used by T.creat in privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.creat to permform privileged tests.

	  usage:
		creat_su testnum dir ident dbug

	  where testnum is the number of the test being executed and dir
	  is a directory name to be passed to the test.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Tue Nov  4 12:08:27 WET 1986
MODIFICATIONS:	D McCardle Tue Jun  9 08:27:34 WET 1987
			Modified xx_rpt(), xs_rpt().
		Geoff Clare, 22 Mar 1989
			Update for XPG3
		Geoff Clare, 15 Nov 1989
			New setuprofs() argument
			New do_access() argument

		David Sawyer
		Fri Jul 27 16:05:08 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Neil Moses, 20 May 1992
			Check that on error EROFS creat() 
			does not create file.

		Neil Moses, 28 May 1992
			Check that on error open() does not truncate file.
			Included version of setuprofs() so that test can
			write to file, before making the file-system
			read-only.

		Neil Moses, 9 June 1992
			Check that time stamps are not changed.

                John-Paul, 28 July 1992
                        mode variables changed from 'int' to 'mode_t'.

		David Sawyer, 6 October 1992
			Added TEST_XNFS code

		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <dbug.h>
#include <sys/stat.h>
#include <signal.h>
#include <setprv.h>
#include <report.h>
#include <errno.h>
#include <xnfs.h>
#include <tet_api.h>
#include "creat_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef creat
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define VSX_ROFS	"VSX_ROFS"
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	void	(*vsignal())();
extern	void	savtimes();
extern	char	*chktimes();

private	char	*rofs;
private	char	*tidydir;
private	char	*tidyfile;
private	int	tsetuprofs();
private	int	tclnuprofs();
private	void	(*oldhdlr)();

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	**environ;

extern	char	*errname();
extern	char	*testid();
extern	void	valrpt();
extern	void	set_pname();
extern	void	cln_access();

private int	testfail;

private	void	subtest15();
private	void	subtest16();
private	void	subtest28();
private	int	t15_func();
private	int	t16_func();
private	int	savparent();
private	void	chkparent();
private	void	chkcreate();
private	void	chktrunc();

#if	   TEST_XNFS
private int	subtest17_19();

#define NOK		02
#define PATH_OK		04

#endif

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int testnum;
	int rc = OK;

	set_pname(argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 5)
		vg_dbug = argv[4];
	if (argc >= 4)
		vg_ident = argv[3];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testnum = atoi(argv[1]);
	switch (testnum)
	{
	case 15:
		subtest15();
		break;

	case 16:
		subtest16();
		break;

#if	   TEST_XNFS
	case 17:
	case 18:
	case 19:
		rc = subtest17_19(argv[2]);
		break;
#endif	/* TEST_XNFS */

	case 28:
		subtest28(argv[2]);
		break;

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed: %d", testnum);
		return(OK);
	}

	return(rc);
	/* NOTREACHED */
}

private void
subtest15()
{
	/* write access to file, search access to path prefix */

	int	pathok = 0;

	DBUG_ENTER("subtest15");

	globok = 0;
	if (do_access('f', (mode_t) S_IWUSR, (mode_t) S_IXUSR, t15_func, 
		WAITTIME, SP_T15_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(1);
	}

	cln_access();

	DBUG_VOID_RETURN;
}

private int
t15_func(file, desc)
char *file, **desc;
{
	int 	fd, err;
	int	pathok = 0;

	DBUG_ENTER("t15_func");

	errno = NOERROR;
	if ((fd = creat(file, MODEANY)) < 0)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("creat(\"%s\", %#o) failed - errno %d (%s)",
			file, (int)MODEANY, err, errname(err));
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	(void) close(fd);
	DBUG_RETURN(0);
}

#if	   TEST_XNFS
private int
subtest17_19(file)
char *file;
{
	int err, fd, rc;
	int pathok = 0;

	DBUG_ENTER("subtest17_19");

	fd = creat(file, MODEANY);
	err = errno;

	if (fd != SYSERROR || err != EACCES)
	{
		xx_rpt(FAILURE);
		valrpt(SYSERROR, fd, EACCES, err);
		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;

	rc = OK;
	if (pathok == 1)
		rc |= PATH_OK;

	DBUG_RETURN(rc);
}
#endif	/* TEST_XNFS */

private void
subtest16()
{
	/* write & search access to dir, search access to path prefix */

	int	pathok = 0;

	DBUG_ENTER("subtest16");

	globok = 0;
	if (do_access('d', (mode_t) S_IWUSR|S_IXUSR, (mode_t) S_IXUSR, 
		t16_func, WAITTIME, SP_T16_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(1);
	}

	cln_access();

	DBUG_VOID_RETURN;
}

private int
t16_func(dir, desc)
char *dir, **desc;
{
	int fd, err;
	char 	file[50];
	int	pathok = 0;

	DBUG_ENTER("t16_func");

	(void) strcat(strcpy(file, dir), "/creat-t.16");

	errno = NOERROR;
	if ((fd = creat(file, MODEANY)) < 0)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("creat(\"%s\", %#o) failed - errno %d (%s)",
			file, (int)MODEANY, err, errname(err));
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
		(void) unlink(file);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	(void) close(fd);
	(void) unlink(file);
	DBUG_RETURN(0);
}


private void
subtest28(mdir)
char	*mdir;
{
	int	fd, err;
	char	*errptr;
	char	existfile[50];
	char	newfile[50];
	int	pathok = 0;

	DBUG_ENTER("subtest28");

	testfail = 0;

	(void) strcat(strcpy(existfile, mdir), "/t.exist");
	(void) strcat(strcpy(newfile, mdir), "/t.new");

	globok = 0;
	if (tsetuprofs(mdir, existfile, (int) 'f', (mode_t) MODEANY) != 0)
	{
		DBUG_VOID_RETURN;	/* reported in tsetuprofs() */
	}
	else
		PATH_FUNC_TRACE;

	/* creat() on existing file */

	globok = 0;
	savtimes(existfile);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((fd = creat(existfile, MODEANY)) != SYSERROR || errno != EROFS)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("creat(\"%s\", MODEANY) on read-only file system did not give correct results",
			existfile);
		valrpt(SYSERROR, fd, EROFS, err);
		if (fd >= 0)
			(void) close(fd);
	}
	else
	{
		PATH_TRACE;

		globok = 0;
		errptr = chktimes(existfile, 0);
		PATH_FUNC_TRACE;

		if(errptr != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful creat() changed time stamps on %s",
				existfile);
			in_rpt("%s", errptr);
		}
		else
			PATH_TRACE;

		globok = 0;
		chktrunc(existfile, "MODEANY");
		PATH_FUNC_TRACE;
	}

	/* create file on read-only file system */

	globok = 0;
	if (savparent(mdir) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	(void) sleep(1);

	errno = NOERROR;
	if ((fd = creat(newfile, MODEANY)) != SYSERROR || errno != EROFS)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("creat(\"%s\", %#o) on read-only file system did not give correct results",
			newfile, MODEANY);
		valrpt(SYSERROR, fd, EROFS, err);
		if (fd != SYSERROR)
			(void) close(fd);
	}
	else
	{
		PATH_TRACE;

		globok = 0;
		chkcreate(newfile, "MODEANY");
		PATH_FUNC_TRACE;
	}

	globok = 0;
	chkparent(mdir);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(10);

	(void) unlink(newfile);
	(void) tclnuprofs(mdir, existfile);

	DBUG_VOID_RETURN;
}

/* These are copies from creat.c */

private struct stat stpar1, stpar2;

private int
savparent(dir)
char *dir;
{
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("savparent");

	if (stat(dir, &stpar1) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			dir, err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(0);
}

private void
chkparent(dir)
char *dir;
{
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("chkparent");

	if (stat(dir, &stpar2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stpar2.st_ctime > stpar1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was updated");
	}
	else
		PATH_TRACE;

	if (stpar2.st_mtime > stpar1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was updated");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
chkcreate(path, flagstr)
char	*path;
char	*flagstr;
{
	int	pathok = 0;

	DBUG_ENTER("chkcreate");

	if(eaccess(path, F_OK) != SYSERROR)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("creat(\"%s\",%s) on error EROFS created file",
			path, flagstr);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
chktrunc(path, flagstr)
char	*path;
char	*flagstr;
{
	struct	stat buf;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("chktrunc");

	if(stat(path, &buf) < 0)
	{
		err = errno;
		if(!testfail)
			xx_rpt(DELETION);
		in_rpt("stat(%s, buf) failed - errno %d (%s)",
			path, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if(buf.st_size == 0)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("open(\"%s\",%s) on error EROFS truncated file",
			path, flagstr);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
sigtidy(sig)
int sig;
{
	(void) tclnuprofs(tidydir, tidyfile);

	if (oldhdlr == SIG_DFL)
		_exit(255);
	else if (oldhdlr != SIG_IGN)
		(*oldhdlr)(sig);
}

private	int
tclnuprofs(dirname, filename)
char *dirname, *filename;
{
	DBUG_ENTER("tclnuprofs");

	if (oldhdlr != NULL)
		(void) vsignal(SIGTERM, SIG_IGN);

	if (rofs == (char *) 0 || dirname == (char *) 0)
		DBUG_RETURN(1);

	if (filename != NULL)
	{
		(void) unmnt(rofs, dirname);
		if (mnt_rw(rofs, dirname) < 0)
		{
			(void) rmdir(dirname);
			DBUG_RETURN(1);
		}
	
		(void) remove(filename);
	}

	(void) unmnt(rofs, dirname);
	(void) rmdir(dirname);

	if (oldhdlr != NULL)
		(void) vsignal(SIGTERM, oldhdlr);

	DBUG_RETURN(0);
}


/*
 *  This function differs from the function in tsetlib in that it writes
 *  a byte to the file it creates.
 */
private	int
tsetuprofs(dirname, filename, filetype, mode)
char *dirname, *filename;
int  filetype;
mode_t mode;
{
	int	fd;
	int	err;
	char	*cp;
	int	pathok = 0;

	DBUG_LENTER("tsetuprofs", 2);

	if ((rofs = tet_getvar(VSX_ROFS)) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter %s is not set", VSX_ROFS);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (strncmp(rofs, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter %s is set to \"unsup\"", VSX_ROFS);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (setprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_MOUNT) failed");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (vxmkdir(dirname) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: can't make directory %s - errno %d (%s)", dirname, err, errname(err));
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (mnt_rw(rofs, dirname) < 0)
	{
		(void) rmdir(dirname);
		xx_rpt(DELETION);
		in_rpt("deletion reason: mnt_rw(%s, %s) failed", rofs, dirname);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	/* Remove any junk files left from aborted ENOSPC tests */

	cp = (char *)malloc((size_t)(strlen(dirname) + sizeof("/fill.tmpX")));
	if (cp != NULL)
	{
		(void) strcat(strcpy(cp, dirname), "/fill.tmp1");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.tmp2");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.tmp3");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.dir");
		(void) rmdir(cp);
		free((void *)cp);
	}

	/* install signal handler to tidy up if timed out */
	tidydir = dirname;
	tidyfile = filename;
	oldhdlr = vsignal(SIGTERM, sigtidy);

	/* create requested file/directory */

	if (filename != NULL)
	{
		if (filetype == 'd')
		{
			if (vxmkdir(filename) == SYSERROR)
			{
				err = errno;
				(void) unmnt(rofs, dirname);
				(void) rmdir(dirname);
				xx_rpt(DELETION);
				in_rpt("deletion reason: can't create directory %s - errno %d (%s)", filename, err, errname(err));
				DBUG_RETURN(1);
			}
			else
			{
				PATH_TRACE;
				PATH_TRACE;
			}
		}
		else
		{
			if ((fd = creat(filename, mode)) == SYSERROR)
			{
				err = errno;
				(void) unmnt(rofs, dirname);
				(void) rmdir(dirname);
				xx_rpt(DELETION);
				in_rpt("deletion reason: can't create file %s - errno %d (%s)", filename, err, errname(err));
				DBUG_RETURN(1);
			}
			else
				PATH_TRACE;

			if(write(fd, "1", 1) == SYSERROR)
			{
				err = errno;
				(void) unmnt(rofs, dirname);
				(void) rmdir(dirname);
				xx_rpt(DELETION);
				in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err, errname(err));
				DBUG_RETURN(1);
			}
			else
				PATH_TRACE;

			(void) close (fd);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	if (unmnt(rofs, dirname) < 0)
	{
		(void) rmdir(dirname);
		xx_rpt(DELETION);
		in_rpt("deletion reason: unmnt(%s, %s) failed", rofs, dirname);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (mnt_ro(rofs, dirname) < 0)
	{
		(void) rmdir(dirname);
		xx_rpt(DELETION);
		in_rpt("deletion reason: mnt_ro(%s, %s) failed", rofs, dirname);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(9);

	DBUG_RETURN(0);
}
