/*
*      SCCS:  @(#)  POSIX.os/files/creat/creat_t10.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/creat - creat_t10 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		creat_t10 - used by T.creat in test10()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.creat in test10().  This should be setuid/setgid
	  to VSX_UID1/VSX_GID1.
	  usage:
		creat_t10 file uid gid vg_ident vg_dbug

	  where "file" is the file to be creat()ed.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	20 Mar 1989
MODIFICATIONS:

		David Sawyer
		Fri Jul 27 16:06:44 BST 1990
		Added resilience code.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

************************************************************************/

#include <dbug.h>
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <chktimes.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef creat
#endif


#define	TMODE	(mode_t)(S_IRUSR|S_IWUSR|S_IRGRP|S_IXGRP|S_IWOTH|S_IXOTH)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	*errname();
extern	void	set_pname();

private	void	subtest10();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	uid_t uid;
	gid_t gid;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if ((uid = atoi(argv[2])) != geteuid() ||
		(gid = atoi(argv[3])) != getegid())
	{
		xx_rpt(DELETION);
		if (uid != getuid())
			in_rpt("deletion reason: uid is not equal to VSX_UID1");
		else
			in_rpt("deletion reason: gid is not equal to VSX_GID1");
	}
	else
	{
		(void) setuid(uid);
		(void) setgid(gid);
		subtest10(argv[1]);
	}

	return(OK);
	/*NOTREACHED*/
}


private void
subtest10(file)
char *file;
{
	/* creat() truncates file, mode & ownership unchanged */

	int		fd;
	int		err;
	int		fail = 0;
	struct stat	save_statb;
	struct stat	new_statb;
	int		pathok = 0;

	DBUG_ENTER("subtest10");

	if (stat(file, &save_statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat() failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = creat(file, TMODE)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("creat(\"%s\", %#o) failed - errno %d(%s)",
				file, (int)TMODE, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	if (stat(file, &new_statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat() failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (new_statb.st_size != 0)
		fail = 01;
	else
		PATH_TRACE;
	if (new_statb.st_mode != save_statb.st_mode)
		fail |= 02;
	else
		PATH_TRACE;
	if (new_statb.st_uid != save_statb.st_uid)
		fail |= 04;
	else
		PATH_TRACE;
	if (new_statb.st_gid != save_statb.st_gid)
		fail |= 010;
	else
		PATH_TRACE;
	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 01)
			in_rpt("creat() did not truncate");
		if (fail & 02)
			in_rpt("creat() changed mode");
		if (fail & 04)
			in_rpt("creat() changed owner");
		if (fail & 010)
			in_rpt("creat() changed group");
	}
	else
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}
