/*
 *      SCCS: @(#)  POSIX.os/files/creat/creat_t31_2.c Rel 4.4.1 (%G)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#define MAIN
static	char sccsid[] = "@(#)POSIX.os/files/creat/creat_t31_2 Rel 4.4.1 (%G)";
#ifdef	MAIN
char	*copyright[] = {
		"(C) Copyright 1992 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		creat_t31_2.c - used by T.creat in test31() and test32()

PROJECT:	XNFS (X/OPEN NFS tests)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int tet_main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	Exec'd by T.creat in test31() and test32().
	This source file generates several binaries.
	creat_t31 should be setuid to VSX_UID1.
	creat_t32 should be setuid to root.
	usage:
		creat_tNN file myuid expuid vg_ident vg_dbug

	where:
		myuid is my effective uid
		expuid is the uid that we expect for the created file
		file is the file to be created with creat().

AUTHOR:		David Sawyer, UniSoft Ltd.

DATE CREATED:	13 October 1992

MODIFICATIONS:
		Geoff Clare, 8 Jan 1993
			Change atoi() to atouid().

************************************************************************/

#include <std.h>
#include <dbug.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef creat
#endif


public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	uid_t	atouid();
extern	char	*errname();
extern	void	set_pname();

private void	subtest31_32();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	uid_t	myuid;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug + 2);
	}

	if ((myuid = atouid(argv[2])) != geteuid())
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: expected effective uid %ld, found %ld",
			(long)myuid, (long)geteuid());
	}
	else
	{
		subtest31_32(argv[1], atouid(argv[3]));
	}

	return(OK);
}


private void
subtest31_32(file, uid)
char	*file;
uid_t	uid;
{
	int		err, fd;
	int		mode = S_IRWXU | S_IRWXG | S_IRWXO;
	struct stat	stbuf;
	int		pathok = 0;

	DBUG_ENTER("subtest31_32");

	if ((fd = creat(file, (mode_t) mode)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("creat(\"%s\", %#o) failed - errno %d (%s)",
			file, mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fstat(fd, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat() failed on fd %d returned by creat() - errno %d (%s)",
			fd, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		if (stbuf.st_uid != uid)
		{
			xx_rpt(FAILURE);
			in_rpt("creat(\"%s\", %#o) did not set uid correctly",
				file, mode);
			in_rpt("expected uid %d, found uid %d",
				uid, stbuf.st_uid);
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(3);
		}
	}

	(void) close(fd);
	(void) unlink(file);

	DBUG_VOID_RETURN;
}

