/*
 *      SCCS:  @(#)  POSIX.os/files/creat/creat_t33_4.c Rel 4.4.1 (06/09/97)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#define MAIN
static	char sccsid[] = "@(#)POSIX.os/files/creat/creat_t33_4.c Rel 4.4.1 (06/09/97)";
#ifdef	MAIN
char	*copyright[] = {
		"(C) Copyright 1992 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		creat_t33_4 - used by T.creat in test33() and test34()

PROJECT:	XNFS (X/OPEN NFS tests)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int tet_main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	Common subprogram for test33 and test34.
	Exec'd by T.creat.
	creat_t33 and creat_t34 must be setgid to VSX_GID1.
	creat_t33 must have appropriate privileges to mount and unmount a
	file system.

	The test33() and test34() versions differ by being linked with
	files containing different versions of the setup33_34() function.

	usage:
		progname mygid vg_ident vg_dbug

	where mygid is my effective gid

AUTHOR:		David Sawyer, UniSoft Ltd.
DATE CREATED:	13 October 1992

MODIFICATIONS:
		Geoff Clare, 20 Oct 1993
			Don't create subdirectory, restructure code,
			set UID to vsx0 to avoid group ID mapping.

************************************************************************/

#include <std.h>
#include <dbug.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include "creat_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef creat
#endif


#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	*errname();
extern	void	set_pname();
extern	int	vguid0;

extern	void	setup33_34();

public	int
tet_main(argc, argv)
int argc;
char **argv;
{
	gid_t mygid;

	set_pname(argv[0]);

	if (argc < 2 || argc > 4)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 4)
		vg_dbug = argv[3];
	if (argc >= 3)
		vg_ident = argv[2];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug + 2);
	}

	if ((mygid = atoi(argv[1])) != getegid())
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: expected effective gid %ld, found %ld",
			(long)mygid, (long)getegid());
	}
	else if (uids() != 0)
	{
		xx_rpt(DELETION);
		in_rpt("uids() failed");
	}
	else
	{
		setup33_34(mygid);
	}

	return(OK);
}


/*
**	subtest33_34() - perform the GID test after the test-specific
**		setup has been done
**
**	dir is the parent directory of file
**	file is the name of the file to create
**	mygid is my effective gid
**	altgid is:
**		mygid if the file's gid may be either the dir's gid or mygid
**		or -1 if the file's gid must be taken from the dir's gid
**	pathok is the current pathok value
**	pathcount is the number of PATH_CHECKs that the setup function
**		should have done
*/

public void
subtest33_34(dir, file, mygid, altgid, pathok, pathcount)
char *dir;
char *file;
gid_t mygid, altgid;
int pathok, pathcount;
{
	gid_t dirgid;
	int fd, err;
	struct stat stbuf;

	DBUG_ENTER("subtest33_34");

	(void) setuid((uid_t)vguid0);
	if (geteuid() != vguid0)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective uid to vsx0");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(dir, &stbuf) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(%s) failed", dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((dirgid = stbuf.st_gid) == mygid)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: directory gid %ld may not be the same as my gid",
			(long)mygid);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = creat(file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s) failed, errno = %d (%s)",
			file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fstat(fd, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat(%d) failed on fd returned by creat(), errno = %d (%s)",
			fd, err, errname(err));
		(void) close(fd);
		(void) unlink(file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf.st_gid != dirgid &&
			(altgid == (gid_t) -1 || stbuf.st_gid != altgid))
	{
		xx_rpt(FAILURE);
		if (altgid == (gid_t) -1)
		{
			in_rpt("gid of created file: expected %ld, found %ld",
				(long)dirgid, (long)stbuf.st_gid);
		}
		else
		{
			in_rpt("gid of created file: expected %ld or %ld, found %ld",
				(long)dirgid, (long)altgid, (long)stbuf.st_gid);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(pathcount + 6);
	}

	(void) close(fd);
	(void) unlink(file);

	DBUG_VOID_RETURN;
}
