/*
*      SCCS:  @(#)  POSIX.os/files/creat/creat_t8.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/creat - creat_t8 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		creat_t8 - used by T.creat in test8()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.creat in test8().  This should
	  be setuid/setgid to VSX_UID1/VSX_GID1.
	  usage:
		creat_t8 file uid gid vg_ident vg_dbug

	  where uid, gid are VSX_UID1 and VSX_GID1. file is
	  the file to be created with creat().
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Thu Aug 28 19:46:34 BST 1986
MODIFICATIONS:	D McCardle Tue Jun  9 08:27:34 WET 1987
			Modified xx_rpt(), xs_rpt().
		Geoff Clare, 20 Mar 1989
			Updated for XPG3

		David Sawyer
		Fri Jul 27 16:16:17 BST 1990
		Added resilience code.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Geoff Clare, 15 Nov 1991
			Allow only parent dir group in FIPS mode

		Geoff Clare, 12 June 1992
			Test FIPS requirement in XPG4 mode.

************************************************************************/
#include <dbug.h>
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef creat
#endif


#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEMASK	(S_IRWXU|S_IRWXG|S_IRWXO)
#define NMODES		3		/* number of modes to test */
#define UMASKVAL	(S_IRUSR|S_IWGRP|S_IXOTH)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	*errname();
extern	void	set_pname();


private mode_t	modes[NMODES] = { S_IRWXU|S_IRWXG|S_IRWXO,
				  S_IXUSR|S_IWGRP|S_IROTH,
				  UMASKVAL };

private void	subtest8();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	uid_t	uid;
	gid_t	gid;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong " );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if ((uid = atoi(argv[2])) != geteuid() ||
		(gid = atoi(argv[3])) != getegid())
	{
		xx_rpt(DELETION);
		if (uid != getuid())
			in_rpt("deletion reason: uid is not equal to VSX_UID1");
		else
			in_rpt("deletion reason: gid is not equal to VSX_GID1");
	}
	else
	{
		(void) umask(UMASKVAL);
		subtest8(argv[1], uid, gid);
	}

	return(OK);
	/*NOTREACHED*/
}


private void
subtest8(file, uid, gid)
char	*file;
uid_t	uid;
gid_t	gid;
{
	int		err[NMODES];
	int		fd;
	int		i;
	int		fail = 0;
	int		um;
	int		flag[NMODES];
	gid_t		dirgid;
	struct stat	statb;
	int		pathok = 0;

	DBUG_ENTER("subtest8");

	if (stat(".", &statb) == SYSERROR)
	{
		err[0] = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed on \".\" - errno %d (%s)",
			err[0], errname(err[0]));
		DBUG_VOID_RETURN
	}
	else
		PATH_TRACE;
	dirgid = statb.st_gid;

	um = UMASKVAL;
	for (i = 0 ; i < NMODES ; i++)
	{
		err[i] = flag[i] = 0;
		if ((fd = creat(file, modes[i])) == SYSERROR)
		{
			err[i] = errno;
			flag[i] |= 01;
		}
		else if (fstat(fd, &statb) == SYSERROR)
		{
			err[i] = errno;
			flag[i] |= 02;
		}
		else
		{
			if (statb.st_uid != uid)
				flag[i] |= 04;
#if TEST_FIPS || TEST_XPG > 3
			if (statb.st_gid != dirgid)
				flag[i] |= 010;
#else
			if (statb.st_gid != gid && statb.st_gid != dirgid)
				flag[i] |= 010;
#endif
			if ((statb.st_mode & MODEMASK) != (~um & modes[i]))
				flag[i] |= 020;
		}
		if (fd != SYSERROR)
			(void) close(fd);
		if (unlink(file) == SYSERROR && errno != ENOENT)
			flag[i] |= 040;
		if (flag[i])
			fail++;
		else
			PATH_TRACE;
	}
	if (fail)
	{
		xx_rpt(FAILURE);
		for (i = 0 ; i < NMODES ; i++)
		{
			if (flag[i] & 01)
				in_rpt("creat(\"%s\", %#o) failed - errno %d(%s)",
					file, modes[i], err[i],
					errname(err[i]));
			else if (flag[i] & 02)
				in_rpt("fstat() on file discriptor returned by creat() failed - errno %d(%s)",
					err[i], errname(err[i]));
			if (flag[i] & 04)
				in_rpt("creat(\"%s\", %#o) did not set user id correctly", file, modes[i]);
			if (flag[i] & 010)
				in_rpt("creat(\"%s\", %#o) did not set group id correctly", file, modes[i]);
			if (flag[i] & 020)
				in_rpt("creat(\"%s\", %#o) did not set access bits correctly", file, modes[i]);
			if (flag[i] & 040)
				in_rpt("could not remove file %s, created by creat()", file);
		}
	}
	else
		PATH_XS_RPT(1+NMODES);

	DBUG_VOID_RETURN;
}
