/*
*      SCCS:  @(#)  POSIX.os/files/fpathconf/fpathconf.c	Rel 4.4.4	    (07/19/99)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/fpathconf - T.fpathconf Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.fpathconf

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fpathconf() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	27 Apr 1989
MODIFICATIONS:
		Geoff Clare, 20 July 1989
			Add _PC_CHOWN_RESTRICTED, _PC_NO_TRUNC tests
		
		Geoff Clare, 9 May 1990
			Use VSX_INVALID_PC parameter

		Geoff Clare, 1 June 1990
			Allow -1 return in "value >= _POSIX_XYZ_MAX" tests
			Repeat FIFO tests on a pipe.

		Geoff Clare, 28 June 1990
			Add EINVAL "not associated" tests

		David Sawyer
		Mon Jul 30 11:56:54 BST 1990
		Added resilience code.

		Stuart Boutell, 2 Apr 1991
			Added missing DBUG_VOID_RETURN in chkval()

                John-Paul, 28 July 1992
                        'int' removed from MODEALL in call to crfile().

		David Sawyer, 15 October 1992
		Added TEST_XNFS code

		Geoff Clare, 31 March 1994
			Close file descriptors in t_einval().

		Geoff Clare, 11 June 1999
			In chkval() value must be equal, unless XPG3.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<string.h>
#include	<limits.h>
#include	<fcntl.h>
#include	<sys/stat.h>
#include	<signal.h>
#include	<dbug.h>
#include	<report.h>
#include	<pathres.h>
#include	<timeout.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fpathconf
#endif


#define NO_TESTS	30
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define T_REG		001
#define T_FIFO		002
#define T_DIR		004
#define T_TERM		010

extern	char	*errname();
extern	char	*itoa();
extern	void	valrpt();


public	int	no_tests = NO_TESTS;

private int	testfail;
private int	pipefd[2] = { -1, -1 };
private char	*pipe0 = "pipefd[0]";
private char	*pipe1 = "pipefd[1]";
private char	*tty_name;

private	char	*tfile =	"fpathcf-reg";
private	char	*tfifo =	"fpathcf-fifo";
private char	*t13_file =	"fpathconf-t.13";
private char	*t14_file =	"fpathconf-t.14";
private char	*t15_file =	"fpathconf-t.15";
private char	*t16_file =	"fpathconf-t.16";

private void	prepare_tests();
private void	cln_tests();
#if	  !TEST_XNFS
private void	chkval();
private void	chkeql();
#endif	/* TEST_XNFS */
private void	minval();
private void	t_einval();

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();
private void	test30();

private long	dofpathconf();
private long	tbfpathconf();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	 test29, 29,
	 test30, 30,
	 NULL, 0
};


private void
prepare_tests()
{
	int fd, i;

	DBUG_ENTER("prepare_tests");

	if ((fd = crfile(tfile, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "could not create test file");
	}
	else
		(void) close(fd);
	
	tty_name = tet_getvar("VSX_TTYNAME");
	if (tty_name != NULL && *tty_name == '\0')
		tty_name = NULL;
	
	(void) pipe(pipefd);

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) unlink(tfile);
	(void) unlink(tfifo);
	(void) unlink(t13_file);
	(void) unlink(t14_file);
	(void) unlink(t15_file);
	(void) unlink(t16_file);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_LINK_MAX) >= _POSIX_LINK_MAX */

	DBUG_ENTER("test1");

	testfail = 0;

#ifdef TESTBED2
#undef _POSIX_LINK_MAX
#define	_POSIX_LINK_MAX (pathconf(".", _PC_LINK_MAX)+1)
#endif

	globok = 0;
	minval(tfile, _PC_LINK_MAX, (long)_POSIX_LINK_MAX, "_POSIX_LINK_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	minval(".", _PC_LINK_MAX, (long)_POSIX_LINK_MAX, "_POSIX_LINK_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	minval("/", _PC_LINK_MAX, (long)_POSIX_LINK_MAX, "_POSIX_LINK_MAX");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test2()
{
	DBUG_ENTER("test2");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test2()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_LINK_MAX) >= LINK_MAX */

	DBUG_ENTER("test2");

	testfail = 0;

#ifdef TESTBED2
#undef LINK_MAX
#define	LINK_MAX (pathconf(".", _PC_LINK_MAX)+1)
#endif

#ifdef LINK_MAX

	globok = 0;
	chkval(tfile, _PC_LINK_MAX, (long)LINK_MAX, "LINK_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval(".", _PC_LINK_MAX, (long)LINK_MAX, "LINK_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval("/", _PC_LINK_MAX, (long)LINK_MAX, "LINK_MAX");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("LINK_MAX is not defined");

#endif

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test3()
{
	DBUG_ENTER("test3");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test3()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_MAX_CANON) >= _POSIX_MAX_CANON */

	DBUG_ENTER("test3");

	testfail = 0;

	if (tty_name == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TTYNAME is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
#undef _POSIX_MAX_CANON
#define	_POSIX_MAX_CANON (pathconf(tty_name, _PC_MAX_CANON)+1)
#endif

	globok = 0;
	minval(tty_name, _PC_MAX_CANON,
		(long)_POSIX_MAX_CANON, "_POSIX_MAX_CANON");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test4()
{
	DBUG_ENTER("test4");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test4()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_MAX_CANON) >= MAX_CANON */

	DBUG_ENTER("test4");

	testfail = 0;

#ifdef TESTBED2
#undef MAX_CANON
#define	MAX_CANON (pathconf(tty_name, _PC_MAX_CANON)+1)
#endif

#ifdef MAX_CANON

	if (tty_name == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TTYNAME is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	chkval(tty_name, _PC_MAX_CANON, (long)MAX_CANON, "MAX_CANON");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("MAX_CANON is not defined");

#endif

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test5()
{
	DBUG_ENTER("test5");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test5()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_MAX_INPUT) >= _POSIX_MAX_INPUT */

	DBUG_ENTER("test5");

	testfail = 0;

	if (tty_name == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TTYNAME is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
#undef _POSIX_MAX_INPUT
#define	_POSIX_MAX_INPUT (pathconf(tty_name, _PC_MAX_INPUT)+1)
#endif

	globok = 0;
	minval(tty_name, _PC_MAX_INPUT,
		(long)_POSIX_MAX_INPUT, "_POSIX_MAX_INPUT");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test6()
{
	DBUG_ENTER("test6");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test6()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_MAX_INPUT) >= MAX_INPUT */

	DBUG_ENTER("test6");

	testfail = 0;

#ifdef TESTBED2
#undef MAX_INPUT
#define	MAX_INPUT (pathconf(tty_name, _PC_MAX_INPUT)+1)
#endif

#ifdef MAX_INPUT

	if (tty_name == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TTYNAME is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	chkval(tty_name, _PC_MAX_INPUT, (long)MAX_INPUT, "MAX_INPUT");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("MAX_INPUT is not defined");

#endif

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test7()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_NAME_MAX) >= _POSIX_NAME_MAX */

	DBUG_ENTER("test7");

	testfail = 0;

#ifdef TESTBED2
#undef _POSIX_NAME_MAX
#define	_POSIX_NAME_MAX (pathconf(".", _PC_NAME_MAX)+1)
#endif

	globok = 0;
	minval(".", _PC_NAME_MAX, (long)_POSIX_NAME_MAX, "_POSIX_NAME_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	minval("/", _PC_NAME_MAX, (long)_POSIX_NAME_MAX, "_POSIX_NAME_MAX");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test8()
{
	DBUG_ENTER("test8");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test8()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_NAME_MAX) >= NAME_MAX */

	DBUG_ENTER("test8");

	testfail = 0;

#ifdef TESTBED2
#undef NAME_MAX
#define	NAME_MAX (pathconf(".", _PC_NAME_MAX)+1)
#endif

#ifdef NAME_MAX

	globok = 0;
	chkval(".", _PC_NAME_MAX, (long)NAME_MAX, "NAME_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval("/", _PC_NAME_MAX, (long)NAME_MAX, "NAME_MAX");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("NAME_MAX is not defined");

#endif

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test9()
{
	/* _PC_NAME_MAX on a directory applies to files in the directory */

	/* this is tested for pathconf() in all the NAME_MAX - ENAMETOOLONG
	   tests, so just check fpathconf() gives the same value */

	long 	rval1, rval2;
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("test9");

	testfail = 0;

	errno = NOERROR;
	rval1 = pathconf(".", _PC_NAME_MAX);
	if (rval1 == SYSERROR && errno != NOERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("pathconf(\".\", _PC_NAME_MAX) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	rval2 = dofpathconf(".", _PC_NAME_MAX);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	rval1++;
#endif

	if (rval1 != rval2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, _PC_NAME_MAX) on \".\" did not return same value as pathconf(\".\", _PC_NAME_MAX)");
		in_rpt("expected: %ld, observed: %ld", rval1, rval2);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_PATH_MAX) >= _POSIX_PATH_MAX */

	DBUG_ENTER("test10");

	testfail = 0;

#ifdef TESTBED2
#undef _POSIX_PATH_MAX
#define	_POSIX_PATH_MAX (pathconf(".", _PC_PATH_MAX)+1)
#endif

	globok = 0;
	minval(".", _PC_PATH_MAX, (long)_POSIX_PATH_MAX, "_POSIX_PATH_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	minval("/", _PC_PATH_MAX, (long)_POSIX_PATH_MAX, "_POSIX_PATH_MAX");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test11()
{
	DBUG_ENTER("test11");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test11()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_PATH_MAX) >= PATH_MAX */

	DBUG_ENTER("test11");

	testfail = 0;

#ifdef TESTBED2
#undef PATH_MAX
#define	PATH_MAX (pathconf(".", _PC_PATH_MAX)+1)
#endif

#ifdef PATH_MAX

	globok = 0;
	chkval(".", _PC_PATH_MAX, (long)PATH_MAX, "PATH_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval("/", _PC_PATH_MAX, (long)PATH_MAX, "PATH_MAX");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("PATH_MAX is not defined");

#endif

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test12()
{
	int	pathok = 0;

	/* _PC_PATH_MAX on a directory applies to files in the directory */

	/* this is tested for pathconf() in all the PATH_MAX - ENAMETOOLONG
	   tests, so just check fpathconf() gives the same value */

	long rval1, rval2;
	int err;

	DBUG_ENTER("test12");

	testfail = 0;

	errno = NOERROR;
	rval1 = pathconf(".", _PC_PATH_MAX);
	if (rval1 == SYSERROR && errno != NOERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("pathconf(\".\", _PC_PATH_MAX) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	rval2 = dofpathconf(".", _PC_PATH_MAX);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	rval1++;
#endif

	if (rval1 != rval2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, _PC_PATH_MAX) on \".\" did not return same value as pathconf(\".\", _PC_PATH_MAX)");
		in_rpt("expected: %ld, observed: %ld", rval1, rval2);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test13()
{
	/* fpathconf(fd, _PC_PIPE_BUF) >= _POSIX_PIPE_BUF */

	int 	err;
	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

	(void) remove(t13_file);
	if (mkfifo(t13_file, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("mkfifo(\"%s\", %#o) failed - errno %d (%s)",
			t13_file, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
#undef _POSIX_PIPE_BUF
#define	_POSIX_PIPE_BUF (pathconf(".", _PC_PIPE_BUF)+1)
#endif

	globok = 0;
	minval(pipe0, _PC_PIPE_BUF, (long)_POSIX_PIPE_BUF, "_POSIX_PIPE_BUF");
	PATH_FUNC_TRACE;

	globok = 0;
	minval(pipe1, _PC_PIPE_BUF, (long)_POSIX_PIPE_BUF, "_POSIX_PIPE_BUF");
	PATH_FUNC_TRACE;

	globok = 0;
	minval(t13_file, _PC_PIPE_BUF,
			(long)_POSIX_PIPE_BUF, "_POSIX_PIPE_BUF");
	PATH_FUNC_TRACE;

	globok = 0;
	minval(".", _PC_PIPE_BUF, (long)_POSIX_PIPE_BUF, "_POSIX_PIPE_BUF");
	PATH_FUNC_TRACE;

	globok = 0;
	minval("/", _PC_PIPE_BUF, (long)_POSIX_PIPE_BUF, "_POSIX_PIPE_BUF");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) unlink(t13_file);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test14()
{
	DBUG_ENTER("test14");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test14()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_PIPE_BUF) >= PIPE_BUF */

	int err;

	DBUG_ENTER("test14");

	testfail = 0;

#ifdef TESTBED2
#undef PIPE_BUF
#define	PIPE_BUF (pathconf(".", _PC_PIPE_BUF)+1)
#endif

#ifdef PIPE_BUF

	(void) remove(t14_file);
	if (mkfifo(t14_file, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("mkfifo(\"%s\", %#o) failed - errno %d (%s)",
			t14_file, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	chkval(pipe0, _PC_PIPE_BUF, (long)PIPE_BUF, "PIPE_BUF");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval(pipe1, _PC_PIPE_BUF, (long)PIPE_BUF, "PIPE_BUF");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval(t14_file, _PC_PIPE_BUF, (long)PIPE_BUF, "PIPE_BUF");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval(".", _PC_PIPE_BUF, (long)PIPE_BUF, "PIPE_BUF");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval("/", _PC_PIPE_BUF, (long)PIPE_BUF, "PIPE_BUF");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);
	
	(void) unlink(t14_file);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("PIPE_BUF is not defined");

#endif

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test15()
{
	DBUG_ENTER("test15");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test15()
{
	/* _PC_PIPE_BUF on a FIFO applies to the FIFO.  This is tested for
	   pathconf() in write(), so check fpathconf() gives the same value */

	long 	rval1, rval2;
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("test15");

	testfail = 0;

	(void) remove(t15_file);
	if (mkfifo(t15_file, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("mkfifo(\"%s\", %#o) failed - errno %d (%s)",
			t15_file, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval1 = pathconf(t15_file, _PC_PIPE_BUF);
	if (rval1 == SYSERROR && errno != NOERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("pathconf(\"%s\", _PC_PIPE_BUF) failed - errno %d (%s)",
			t15_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	rval2 = dofpathconf(t15_file, _PC_PIPE_BUF);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	rval1++;
#endif

	if (rval1 != rval2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, _PC_PIPE_BUF) on \"%s\" did not return same value as pathconf(\"%s\", _PC_PIPE_BUF)",
			t15_file, t15_file);
		in_rpt("expected: %ld, observed: %ld", rval1, rval2);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) unlink(t15_file);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test16()
{
	DBUG_ENTER("test16");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test16()
{
	/* _PC_PIPE_BUF on a directory applies to a FIFO in the directory */

	long 	rval1, rval2;
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("test16");

	testfail = 0;

	(void) remove(t16_file);
	if (mkfifo(t16_file, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("mkfifo(\"%s\", %#o) failed - errno %d (%s)",
			t16_file, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	rval1 = dofpathconf(t16_file, _PC_PIPE_BUF);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	rval2 = dofpathconf(".", _PC_PIPE_BUF);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	rval1++;
#endif

	if (rval1 != rval2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, _PC_PIPE_BUF) on \".\" did not return same value as fpathconf(fd, _PC_PIPE_BUF) on \"%s\"",
			t16_file);
		in_rpt("expected: %ld, observed: %ld", rval1, rval2);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) unlink(t16_file);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test17()
{
	DBUG_ENTER("test17");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test17()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_CHOWN_RESTRICTED) == _POSIX_CHOWN_RESTRICTED */

	DBUG_ENTER("test17");

	testfail = 0;

#ifdef TESTBED2
#undef _POSIX_CHOWN_RESTRICTED
#define	_POSIX_CHOWN_RESTRICTED (pathconf(".", _PC_CHOWN_RESTRICTED)+1)
#endif

#ifdef _POSIX_CHOWN_RESTRICTED

	globok = 0;
	chkeql(tfile, _PC_CHOWN_RESTRICTED,
		(long)_POSIX_CHOWN_RESTRICTED, "_POSIX_CHOWN_RESTRICTED");
	PATH_FUNC_TRACE;

	globok = 0;
	chkeql(".", _PC_CHOWN_RESTRICTED,
		(long)_POSIX_CHOWN_RESTRICTED, "_POSIX_CHOWN_RESTRICTED");
	PATH_FUNC_TRACE;

	globok = 0;
	chkeql("/", _PC_CHOWN_RESTRICTED,
		(long)_POSIX_CHOWN_RESTRICTED, "_POSIX_CHOWN_RESTRICTED");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("_POSIX_CHOWN_RESTRICTED is not defined");

#endif

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test18()
{
	DBUG_ENTER("test18");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test18()
{
	int	pathok = 0;

	/* _PC_CHOWN_RESTRICTED on a directory applies to files
	   in the directory */

	/* this is already tested for pathconf() so just check
	   fpathconf() gives the same value */

	long rval1, rval2;
	int err;

	DBUG_ENTER("test18");

	testfail = 0;

	errno = NOERROR;
	rval1 = pathconf(".", _PC_CHOWN_RESTRICTED);
	if (rval1 == SYSERROR && errno != NOERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("pathconf(\".\", _PC_CHOWN_RESTRICTED) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	rval2 = dofpathconf(".", _PC_CHOWN_RESTRICTED);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	rval1++;
#endif

	if (rval1 != rval2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, _PC_CHOWN_RESTRICTED) on \".\" did not return same value as pathconf(\".\", _PC_CHOWN_RESTRICTED)");
		in_rpt("expected: %ld, observed: %ld", rval1, rval2);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test19()
{
	DBUG_ENTER("test19");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test19()
{
	int	pathok = 0;

	/* fpathconf(fd, _PC_NO_TRUNC) == _POSIX_NO_TRUNC */

	DBUG_ENTER("test19");

	testfail = 0;

#ifdef TESTBED2
#undef _POSIX_NO_TRUNC
#define	_POSIX_NO_TRUNC (pathconf(".", _PC_NO_TRUNC)+1)
#endif

#ifdef _POSIX_NO_TRUNC

	globok = 0;
	chkeql(".", _PC_NO_TRUNC, (long)_POSIX_NO_TRUNC, "_POSIX_NO_TRUNC");
	PATH_FUNC_TRACE;

	globok = 0;
	chkeql("/", _PC_NO_TRUNC, (long)_POSIX_NO_TRUNC, "_POSIX_NO_TRUNC");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("_POSIX_NO_TRUNC is not defined");

#endif

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test20()
{
	DBUG_ENTER("test20");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test20()
{
	int	pathok = 0;

	/* _PC_NO_TRUNC on a directory applies to files in the directory */

	/* this is already tested for pathconf() so just check
	   fpathconf() gives the same value */

	long rval1, rval2;
	int err;

	DBUG_ENTER("test20");

	testfail = 0;

	errno = NOERROR;
	rval1 = pathconf(".", _PC_NO_TRUNC);
	if (rval1 == SYSERROR && errno != NOERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("pathconf(\".\", _PC_NO_TRUNC) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	rval2 = dofpathconf(".", _PC_NO_TRUNC);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	rval1++;
#endif

	if (rval1 != rval2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, _PC_NO_TRUNC) on \".\" did not return same value as pathconf(\".\", _PC_NO_TRUNC)");
		in_rpt("expected: %ld, observed: %ld", rval1, rval2);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test21()
{
	DBUG_ENTER("test21");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test21()
{
	/* fpathconf(fd, _PC_VDISABLE) == _POSIX_VDISABLE */

	char 	rval;
	int	pathok = 0;

	DBUG_ENTER("test21");

	testfail = 0;

#ifdef TESTBED2
#undef _POSIX_VDISABLE
#define	_POSIX_VDISABLE (pathconf(tty_name, _PC_VDISABLE)+1)
#endif

#ifdef _POSIX_VDISABLE

	if (tty_name == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TTYNAME is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* can't use chkeql() - casting _POSIX_VDISABLE to (long) will
	   sign extend */

	globok = 0;
	rval = dofpathconf(tty_name, _PC_VDISABLE);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (rval != (char)_POSIX_VDISABLE)
	{
		xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, _PC_VDISABLE) on \"%s\" did not return _POSIX_VDISABLE",
			tty_name);
		in_rpt("value returned: %d, _POSIX_VDISABLE: %d",
			rval, (char)_POSIX_VDISABLE);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("_POSIX_VDISABLE is not defined");

#endif

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test22()
{
	DBUG_ENTER("test22");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test22()
{
	/* invalid name */

	char 	*cp;
	long 	rval;
	int 	invname = -1, fd, err;
	int	pathok = 0;

	DBUG_ENTER("test22");

	testfail = 0;

	if ((fd = open(".", O_RDONLY)) < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(\".\", O_RDONLY) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	cp = tet_getvar("VSX_INVALID_PC");
	if (cp != NULL && *cp != '\0')
	{
		PATH_TRACE;
		invname = atoi(cp);
	}
	else
		PATH_TRACE;

	/* don't use tbfpathconf(): it allows EINVAL or success */
	errno = 0;
	rval = fpathconf(fd, invname);
	if (rval != -1 || errno != EINVAL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, %d) did not give correct results",
			invname);
		in_rpt("expected: return value == -1, errno == %d (EINVAL)",
			EINVAL);
		in_rpt("observed: return value == %ld, errno == %d (%s)",
			rval, err, errname(err));

	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	(void) close(fd);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test23()
{
	DBUG_ENTER("test23");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test23()
{
	/* invalid fd */

	int	pathok = 0;
	int 	fd, err;

	DBUG_ENTER("test23");

	testfail = 0;

	if ((fd = open(".", O_RDONLY)) < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(\".\", O_RDONLY) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (close(fd) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("close() failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	fd = 0;
#endif

	globok = 0;
	(void) tbfpathconf(fd, _PC_LINK_MAX, "", SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test24()
{
	DBUG_ENTER("test24");

	t_einval(_PC_MAX_CANON, T_REG | T_FIFO | T_DIR);

	DBUG_VOID_RETURN;
}

private void
test25()
{
	DBUG_ENTER("test25");

	t_einval(_PC_MAX_INPUT, T_REG | T_FIFO | T_DIR);

	DBUG_VOID_RETURN;
}

private void
test26()
{
	DBUG_ENTER("test26");

	t_einval(_PC_NAME_MAX, T_REG | T_FIFO | T_TERM);

	DBUG_VOID_RETURN;
}

private void
test27()
{
	DBUG_ENTER("test27");

	t_einval(_PC_PATH_MAX, T_REG | T_FIFO | T_TERM);

	DBUG_VOID_RETURN;
}

private void
test28()
{
	DBUG_ENTER("test28");

	t_einval(_PC_PIPE_BUF, T_REG | T_TERM);

	DBUG_VOID_RETURN;
}

private void
test29()
{
	DBUG_ENTER("test29");

	t_einval(_PC_NO_TRUNC, T_REG | T_FIFO | T_TERM);

	DBUG_VOID_RETURN;
}

private void
test30()
{
	DBUG_ENTER("test30");

	t_einval(_PC_VDISABLE, T_REG | T_FIFO | T_DIR);

	DBUG_VOID_RETURN;
}

private char *
strname(name)
int name;
{
	switch (name)
	{
	    case _PC_LINK_MAX:		return "_PC_LINK_MAX";
	    case _PC_MAX_CANON:		return "_PC_MAX_CANON";
	    case _PC_MAX_INPUT:		return "_PC_MAX_INPUT";
	    case _PC_NAME_MAX:		return "_PC_NAME_MAX";
	    case _PC_PATH_MAX:		return "_PC_PATH_MAX";
	    case _PC_PIPE_BUF:		return "_PC_PIPE_BUF";

	    case _PC_CHOWN_RESTRICTED:	return "_PC_CHOWN_RESTRICTED";
	    case _PC_NO_TRUNC:		return "_PC_NO_TRUNC";
	    case _PC_VDISABLE:		return "_PC_VDISABLE";
	}

	return itoa(name);
}


#if	  !TEST_XNFS

private void
chkval(path, name, expval, constname)
char *path;
int name;
long expval;
char *constname;
{
	/* Check return value is equal to constant in <limits.h> */
	/* (or value >= constant for XPG3 mode) */

	long rval;
	int fail;
	int	pathok = 0;

	DBUG_ENTER("chkval");

	fail = testfail;

	globok = 0;
	rval = dofpathconf(path, name);
	PATH_FUNC_TRACE;
	if (testfail > fail)
		DBUG_VOID_RETURN;

#if TEST_XPG == 3
	if (rval < expval)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, %s) on \"%s\" returned a value less than %s",
			strname(name), path, constname);
		in_rpt("value returned: %ld, %s: %ld", rval, constname, expval);
	}
	else
		PATH_TRACE;
#else
	if (rval != expval)
	{
		if (testfail == 0)
			xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, %s) on \"%s\" did not return the value of %s",
			strname(name), path, constname);
		in_rpt("value returned: %ld, %s: %ld", rval, constname, expval);
		if (testfail == 0)
		{
			in_rpt("(%s should not be defined in <limits.h> if the ",
				constname);
			in_rpt("run-time value can vary.)");
		}
		testfail++;
	}
	else
		PATH_TRACE;
#endif

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
minval(path, name, min, minname)
char *path;
int name;
long min;
char *minname;
{
	/* Check return value is >= POSIX minimum, or == -1 */

	long 	rval;
	int 	fail;
	int	pathok = 0;

	DBUG_ENTER("minval");

	fail = testfail;

	globok = 0;
	rval = dofpathconf(path, name);
	if (testfail > fail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (rval < min && rval != -1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, %s) on \"%s\" returned a value less than %s",
			strname(name), path, minname);
		in_rpt("value returned: %ld, %s: %ld", rval, minname, min);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}


#if	  !TEST_XNFS

private void
chkeql(path, name, val, valname)
char *path;
int name;
long val;
char *valname;
{
	long 	rval;
	int 	fail;
	int	pathok = 0;

	DBUG_ENTER("chkeql");

	fail = testfail;

	globok = 0;
	rval = dofpathconf(path, name);
	if (testfail > fail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (rval != val)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, %s) on \"%s\" did not return %s",
			strname(name), path, valname);
		in_rpt("value returned: %ld, %s: %ld", rval, valname, val);
	}
	else
		PATH_TRACE;
	
	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
t_einval(name, types)
int name;
int types;
{
	int fd, err;
	int pathok = 0;

	DBUG_ENTER("t_einval");

	testfail = 0;

	if (types & T_REG)
	{
		if ((fd = open(tfile, O_RDONLY)) < 0)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("open(\"%s\", O_RDONLY) failed - errno %d (%s)",
				tfile, err, errname(err));
		}
		else
		{
			globok = 0;
			(void) tbfpathconf(fd, name, tfile, SYSERROR, EINVAL);
			PATH_FUNC_TRACE;
			(void) close(fd);
		}
	}
	else
		PATH_TRACE;

#if	!TEST_XNFS
	if (types & T_FIFO)
	{
		(void) unlink(tfifo);
		if (mkfifo(tfifo, MODEANY) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create fifo \"%s\"",
				tfifo);
		}
		else if ((fd = open(tfifo, O_RDONLY|O_NONBLOCK)) < 0)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) failed - errno %d (%s)",
				tfifo, err, errname(err));
		}
		else
		{
			globok = 0;
			(void) tbfpathconf(fd, name, tfifo, SYSERROR, EINVAL);
			PATH_FUNC_TRACE;
			(void) close(fd);
		}

		/* repeat for pipe */
		if (pipefd[0] < 0)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: pipe() failed in prepare routine");
		}
		else
		{
			globok = 0;
			(void) tbfpathconf(pipefd[0], name, pipe0, SYSERROR, EINVAL);
			PATH_FUNC_TRACE;
		}
	}
	else
#endif	/* TEST_XNFS */
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	if (types & T_DIR)
	{
		if ((fd = open(".", O_RDONLY)) < 0)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("open(\".\", O_RDONLY) failed - errno %d (%s)",
				err, errname(err));
		}
		else
		{
			globok = 0;
			(void) tbfpathconf(fd, name, ".", SYSERROR, EINVAL);
			PATH_FUNC_TRACE;
			(void) close(fd);
		}
	}
	else
		PATH_TRACE;

#if	!TEST_XNFS
	if (types & T_TERM)
	{
		if (tty_name == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: parameter VSX_TTYNAME is not defined");
		}
		else if ((fd = open(tty_name, O_RDONLY|O_NONBLOCK)) < 0)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) failed - errno %d (%s)",
				tty_name, err, errname(err));
		}
		else
		{
			globok = 0;
			(void) tbfpathconf(fd, name, tty_name, SYSERROR, EINVAL);
			PATH_FUNC_TRACE;
			(void) close(fd);
		}
	}
	else
#endif	/* TEST_XNFS */
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private long
dofpathconf(file, name)
char *file;
int name;
{
	long 	rval;
	int 	fd, err;
	int	pathok = 0;

	DBUG_ENTER("dofpathconf");

	if (file == pipe0 || file == pipe1)
	{
		fd = pipefd[file == pipe0 ? 0 : 1];
		if (fd < 0)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("pipe() failed in prepare routine");
			DBUG_RETURN(SYSERROR);
		}
		else
			PATH_TRACE;
	}
	else
	{
		SET_TIMEOUT(WAITTIME)

		fd = open(file, O_RDONLY|O_NONBLOCK);
		err = errno;

		CLEAR_ALARM

		if (fd < 0)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			if (alrm_flag > 0)
				in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) timed out",
					file);
			else
				in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) failed - errno %d (%s)",
					file, err, errname(err));
			DBUG_RETURN(SYSERROR);
		}
		else
			PATH_TRACE;
	}

	globok = 0;
	rval = tbfpathconf(fd, name, file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (fd != pipefd[0] && fd != pipefd[1])
		(void) close(fd);

	PATH_FUNC_RPT(2);

	DBUG_RETURN(rval);
}

private long
tbfpathconf(tbfd, tbname, path, expret, experrno)
int	tbfd;
int	tbname;
char	*path;
int	expret;
int	experrno;
{
	long	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("tbfpathconf");

	errno = NOERROR;
	rval = fpathconf(tbfd, tbname);
#ifdef TESTBED
	rval = -1;
	errno++;
#endif
	err = errno;

	if (rval == SYSERROR && err == NOERROR)
	{
		PATH_FUNC_RPT(0);
		DBUG_RETURN(rval);
	}
	else
		PATH_TRACE;

	if (expret == SUCCEED)
	{
		if (rval != SYSERROR)
		{
			PATH_FUNC_RPT(1);
			DBUG_RETURN(rval);
		}

		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, %s) on \"%s\" did not give correct results",
			strname(tbname), path);
		in_rpt("expected: return value != -1");
		in_rpt("      OR: return value == -1, errno == 0 (NO ERROR)");
		in_rpt("observed: return value == %ld, errno == %d (%s)",
			rval, err, errname(err));
	}
	else
	{
		if (rval == SYSERROR && err == experrno)
		{
			PATH_TRACE;
			PATH_FUNC_RPT(2);
			DBUG_RETURN(rval);
		}
		if (rval != SYSERROR)
		{
			PATH_TRACE;
			PATH_FUNC_RPT(2);
			DBUG_RETURN(rval);
		}

		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fpathconf(fd, %s) did not give correct results",
			strname(tbname));
		in_rpt("expected: return value == -1, errno == %d (%s)",
			experrno, errname(experrno));
		in_rpt("      OR: return value != -1");
		in_rpt("observed: return value == %ld, errno == %d (%s)",
			rval, err, errname(err));
	}
	DBUG_RETURN(rval);
}
