/*
*      SCCS:  @(#)  POSIX.os/files/fstat/fstat.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/fstat - T.fstat Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.fstat

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fstat() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	25 Apr 1989
MODIFICATIONS:
		Geoff Clare, 21 Nov 1989
			Cannot use d_ino for testing st_ino
			(as not available in POSIX-only mode)

		David Sawyer
		Mon Jul 30 13:20:39 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 4 Mar 1992
			Remove st_size test on pipe.  Added fstat() to
			force timestamp update after pipe() call.

                John-Paul, 28 July 1992
			'int's removed from before MODEALL in calls to crfile()
			and crdir().

		David Sawyer, 15 October 1992
		Added TEST_XNFS code

		Geoff Clare, 2 Nov 1993
			Use file created on server in XNFS st_atime test
			to avoid reading cached data.

		Geoff Clare, 5 Jan 1994
			Obtain t0 before crfile() call in t1a().

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<time.h>
#include	<string.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<sys/stat.h>
#include	<signal.h>
#include	<dbug.h>
#include	<timeout.h>
#include	<report.h>
#include	<chktimes.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"fstat_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fstat
#endif


#define NO_TESTS	4
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	NOK		02

extern	char	*errname();
extern	char	*newstr();
extern	char	*itoa();
extern	void	valrpt();
extern	char	*execpath();

extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;
extern	int	vguid1, vggid1;

public	int	no_tests = NO_TESTS;

private	struct stat	stbuf;
private int	testfail;

private	char	*prog1 =	"fstat_t1";

private	char	*t1a_file =	"fstat-t.1a";
private	char	*t1b_file =	"fstat-t.1b";
private	char	*t1c_file =	"fstat-t.1c";
private	char	*t1d_file =	"fstat-t.1d";
private	char	*t1_link =	"fstat-l.1";
private	char	*t1_dir =	"fstat-d.1";
private	char	*t1_fifo =	"fstat-f.1";
private	char	*t2_file =	T2_FILE;
#if	   TEST_XNFS
private	char	*t4_file =	T4_FILE;
#endif	/* TEST_XNFS */

private mode_t	exp_modes[] = {
	0,
	S_IRUSR,
	S_IWUSR,
	S_IXUSR,
	S_IRGRP,
	S_IWGRP,
	S_IXGRP,
	S_IROTH,
	S_IWOTH,
	S_IXOTH,
	MODEANY,
	(MODEANY & ~S_IRUSR),
	(MODEANY & ~S_IWUSR),
	(MODEANY & ~S_IXUSR),
	(MODEANY & ~S_IRGRP),
	(MODEANY & ~S_IWGRP),
	(MODEANY & ~S_IXGRP),
	(MODEANY & ~S_IROTH),
	(MODEANY & ~S_IWOTH),
	(MODEANY & ~S_IXOTH),
};
private int	no_mode =	sizeof(exp_modes)/sizeof(exp_modes[0]);

private void	prepare_tests();
private void	cln_tests();

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	t1a();
private void	t1b();
private void	t1c();
private void	t1d();
private void	t1e();
#if	  !TEST_XNFS
private void	t1f();
private void	t1g();
#endif	/* TEST_XNFS */
private void	ch_t1c();

private int	dofstat();
private int	tbfstat();

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};


private void
prepare_tests()
{
	DBUG_ENTER("prepare_tests");

	if(uids()!=0)
	{
		int	i;
		for(i=1; i <= no_tests; i++)
		{
			(void) tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	(void) umask((mode_t)0);

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) unlink(t1a_file);
	(void) unlink(t1b_file);
	(void) unlink(t1c_file);
	(void) unlink(t1d_file);
	(void) unlink(t1_link);
	(void) rmdir(t1_dir);
	(void) unlink(t1_fifo);
	(void) unlink(t2_file);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = 0;

	/* fstat() returns info about file associated with specified fd */

	DBUG_ENTER("test1");

	testfail = 0;

	globok = 0;
	t1a();
	PATH_FUNC_TRACE;

	globok = 0;
	t1b();
	PATH_FUNC_TRACE;

	globok = 0;
	t1c();
	PATH_FUNC_TRACE;

	globok = 0;
	t1d();
	PATH_FUNC_TRACE;

	globok = 0;
	t1e();
	PATH_FUNC_TRACE;

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	globok = 0;
	t1f();
	PATH_FUNC_TRACE;

	globok = 0;
	t1g();
	PATH_FUNC_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
t1a()
{
	/* check st_dev, st_ino, S_ISREG(st_mode), st_size,
	   and st_[amc]time for a plain file */

	time_t		t0, t1;
	dev_t		dev1, dev2;
	ino_t		ino1, ino2;
	int		fd, err;
	char		wbuf[20];
	int		pathok = 0;

	DBUG_ENTER("t1a");

	testfail = 0;

#if	   TEST_XNFS

	if (xnfs_rtime_c(&t0, SP_T1_RTIME1_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	(void) time(&t0);
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if ((fd = crfile(t1a_file, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t1a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(2);

	if (write(fd, wbuf, (unsigned)sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not write to file \"%s\" - errno %d (%s)",
			t1a_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

#if	   TEST_XNFS

	if (xnfs_rtime_c(&t1, SP_T1_RTIME2_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	(void) time(&t1);
	PATH_TRACE;

#endif	/* TEST_XNFS */

	(void) sleep(1);

	if (stat(".", &stbuf) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat(\".\", buf) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	dev1 = stbuf.st_dev;
	ino1 = stbuf.st_ino;

	if (stat("..", &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"..\", buf) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	dev2 = stbuf.st_dev;
	ino2 = stbuf.st_ino;

#ifdef TESTBED2
#define t1a_file "/"
#endif

	globok = 0;
	if (dofstat(t1a_file, &stbuf) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
#undef t1a_file
stbuf.st_ino = ino1;
#endif

	if (stbuf.st_dev != dev1 || stbuf.st_dev != dev2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() did not set st_dev correctly");
		in_rpt("observed %s: 0x%lx, \".\": 0x%lx, \"..\": 0x%lx",
			t1a_file, (long)stbuf.st_dev, (long)dev1, (long)dev2);
	}
	else
		PATH_TRACE;

	if (stbuf.st_ino == ino1 || stbuf.st_ino == ino2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() gave same st_ino for \"%s\" and \"%s\"",
			t1a_file, (stbuf.st_ino == ino1 ? "." : ".."));
	}
	else
		PATH_TRACE;

	if (!S_ISREG(stbuf.st_mode))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() set st_mode file type incorrectly: S_ISREG(st_mode) was false");
	}
	else
		PATH_TRACE;

	if (stbuf.st_size != sizeof(wbuf))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() set st_size incorrectly: expected %ld, observed %ld",
			(long)sizeof(wbuf), (long)stbuf.st_size);
	}
	else
		PATH_TRACE;

	if (stbuf.st_atime != t0 && stbuf.st_atime != t0+1 &&
		stbuf.st_atime != t0+2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() set st_atime incorrectly: expected %ld, observed %ld",
			(long)t0, (long)stbuf.st_atime);
	}
	else
		PATH_TRACE;

	if ((stbuf.st_mtime != t1) && (stbuf.st_mtime != t1+1) &&
		(stbuf.st_mtime != t1+2))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() set st_mtime incorrectly: expected %ld, observed %ld",
			(long)t1, (long)stbuf.st_mtime);
	}
	else
		PATH_TRACE;

	if (stbuf.st_ctime != t1 && stbuf.st_ctime != t1 + 1 &&
		stbuf.st_mtime != t1 + 2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() set st_ctime incorrectly: expected %ld, observed %ld",
			(long)t1, (long)stbuf.st_ctime);
	}
	else
		PATH_TRACE;

	(void) unlink(t1a_file);

	PATH_FUNC_RPT(14);

	DBUG_VOID_RETURN;
}

private void
t1b()
{
	/* check st_mode permissions bits */

	int 	fd, err, i;
	int	pathok = 0;
	
	DBUG_ENTER("t1b");

	if ((fd = crfile(t1b_file, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t1b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	for (i = 0; i < no_mode; i++)
	{
		if (chmod(t1b_file, exp_modes[i]) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
				t1b_file, (int)exp_modes[i], err, errname(err));
			(void) close(fd);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		if (tbfstat(fd, &stbuf, SUCCEED, NOERROR) != SUCCEED)
			continue;
		else
			PATH_FUNC_TRACE;

#ifdef TESTBED2
		exp_modes[1] ^= S_IWUSR;
#endif

		if ((stbuf.st_mode & MODEANY) != exp_modes[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("fstat() set st_mode permission bits incorrectly: expected %#o, observed %#o",
			    (int)exp_modes[i], (int)(stbuf.st_mode & MODEANY));
		}
		else
			PATH_TRACE;
	}

	(void) close(fd);
	(void) unlink(t1b_file);

	PATH_FUNC_RPT(1+(no_mode*3));

	DBUG_VOID_RETURN;
}

private void
t1c()
{
	int	pathok = 0;

	/* check st_uid and st_gid */

	DBUG_ENTER("t1c");

	globok = 0;
	if (cppair(ch_t1c, NULLFN, WAITTIME, NOK) == NOK)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (dofstat(t1c_file, &stbuf) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	stbuf.st_uid++;
	stbuf.st_gid++;
#endif

	if (stbuf.st_uid != vguid1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() set st_uid incorrectly: expected %d, observed %d",
			vguid1, (int)stbuf.st_uid);
	}
	else
		PATH_TRACE;

	if (stbuf.st_gid != vggid1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() set st_gid incorrectly: expected %d, observed %d",
			vggid1, (int)stbuf.st_gid);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t1c()
{
	int	err;
	char	*suid, *sgid;
	char	*args[7];

	DBUG_ENTER("ch_t1c");

	suid = newstr(itoa(vguid1));
	sgid = newstr(itoa(vggid1));
	if (suid == NULL || sgid == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: malloc() failed");
		DBUG_EXIT(NOK);
	}

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog1);
	args[1] = t1c_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog1), args, environ);

	err = errno;
	if (testfail++ == 0)
		xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog1), err, errname(err));
	DBUG_EXIT(NOK);
}

private void
t1d()
{
	/* check st_nlink */

	int 	fd, err;
	int	pathok = 0;
	
	DBUG_ENTER("t1d");

	if ((fd = crfile(t1d_file, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t1d_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	(void) unlink("junk");
	(void) link(t1d_file, "junk");
#endif

	globok = 0;
	if (tbfstat(fd, &stbuf, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (stbuf.st_nlink != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() set st_nlink incorrectly: expected 1, observed %ld",
			(long)stbuf.st_nlink);
	}
	else
		PATH_TRACE;

	if (link(t1d_file, t1_link) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
			t1d_file, t1_link, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tbfstat(fd, &stbuf, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (stbuf.st_nlink != 2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() set st_nlink incorrectly: expected 2, observed %ld",
			(long)stbuf.st_nlink);
	}
	else
		PATH_TRACE;
	
	(void) close(fd);
	(void) unlink(t1_link);
	(void) unlink(t1d_file);

	PATH_FUNC_RPT(6);

	DBUG_VOID_RETURN;
}

private void
t1e()
{
	int	pathok = 0;

	/* check S_ISDIR(st_mode) for a directory */

	DBUG_ENTER("t1e");

	if (crdir(t1_dir, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t1_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (dofstat(t1_dir, &stbuf) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	stbuf.st_mode &= ~S_IFDIR;
#endif

	if (!S_ISDIR(stbuf.st_mode))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() on directory \"%s\" set st_mode file type incorrectly: S_ISDIR(st_mode) was false",
			t1_dir);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}


#if	  !TEST_XNFS
private void
t1f()
{
	/* check S_ISFIFO(st_mode) for a FIFO */

	int 	err;
	int	pathok = 0;

	DBUG_ENTER("t1f");

	if (mkfifo(t1_fifo, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("mkfifo(\"%s\", %#o) failed - errno %d (%s)",
			t1_fifo, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (dofstat(t1_fifo, &stbuf) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	stbuf.st_mode &= ~S_IFIFO;
#endif

	if (!S_ISFIFO(stbuf.st_mode))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() on FIFO \"%s\" set st_mode file type incorrectly: S_ISFIFO(st_mode) was false",
			t1_fifo);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}


private void
t1g()
{
	/* check st_[amc]time for a pipe */

	time_t		t0, t1;
	int		fd[2], err;
	char		wbuf[20];
	int		pathok = 0;

	DBUG_ENTER("t1g");

	testfail = 0;

	if (pipe(fd) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("pipe() failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* force the time fields to be updated */
	(void) fstat(fd[0], &stbuf);
	(void) time(&t0);
	(void) sleep(2);

	if (write(fd[1], wbuf, (unsigned)sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not write to pipe - errno %d (%s)",
			err, errname(err));
		(void) close(fd[1]);
		(void) close(fd[0]);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	/* force the time fields to be updated */
	(void) fstat(fd[1], &stbuf);
	(void) time(&t1);

	(void) sleep(1);

	globok = 0;
	if (tbfstat(fd[0], &stbuf, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) close(fd[1]);
		(void) close(fd[0]);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	stbuf.st_size++;
	stbuf.st_atime += 2;
	stbuf.st_mtime += 2;
	stbuf.st_ctime += 2;
#endif

	if (stbuf.st_atime != t0 && stbuf.st_atime != t0 - 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() on pipe set st_atime incorrectly: expected %ld, observed %ld",
			(long)t0, (long)stbuf.st_atime);
	}
	else
		PATH_TRACE;

	if (stbuf.st_mtime != t1 && stbuf.st_mtime != t1 - 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() on pipe set st_mtime incorrectly: expected %ld, observed %ld",
			(long)t1, (long)stbuf.st_mtime);
	}
	else
		PATH_TRACE;

	if (stbuf.st_ctime != t1 && stbuf.st_ctime != t1 - 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() on pipe set st_ctime incorrectly: expected %ld, observed %ld",
			(long)t1, (long)stbuf.st_ctime);
	}
	else
		PATH_TRACE;


	PATH_FUNC_RPT(6);

	(void) close(fd[1]);
	(void) close(fd[0]);

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
test2()
{
	/* fstat() updates timestamps "marked for update" */

	char	*chkt;
	int	fd, err;
	char	wbuf[20];
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

#if TEST_XNFS
	
	if (tet_sync(SP_T2_CREAT_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREAT_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_TRACE;

#else /* TEST_XNFS */

	if ((fd = crfile(t2_file, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t2_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, wbuf, (unsigned)sizeof(wbuf)) <= 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("first write to file \"%s\" failed - errno %d (%s)",
			t2_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

#endif /* TEST_XNFS */

	if ((fd = open(t2_file, O_RDWR)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d (%s)",
			t2_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	fsavtimes(fd);
	PATH_FUNC_TRACE;

	if (read(fd, wbuf, (unsigned)sizeof(wbuf)) <= 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("could not read from file \"%s\" - errno %d (%s)",
			t2_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, wbuf, (unsigned)sizeof(wbuf)) <= 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("second write to file \"%s\" failed - errno %d (%s)",
			t2_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	fsavtimes(fd);
#endif

	/* the fstat() call is in fchktimes() */

	globok = 0;
	chkt = fchktimes(fd, NEWATIME|NEWMTIME|NEWCTIME);
	if (chkt != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("%s", chkt);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(7);
	}

	(void) close(fd);

#if TEST_XNFS
	(void) tet_sync(SP_T2_UNLINK_START, snames, SYNC_TIMEOUT);
#else
	(void) unlink(t2_file);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test3()
{
	DBUG_ENTER("test3");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test3()
{
	/* invalid fd */

	int 	fd[2], err;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	if (pipe(fd) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("pipe() failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd[1]);
	if (close(fd[0]) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("close() failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	fd[0] = 0;
#endif

	globok = 0;
	(void) tbfstat(fd[0], &stbuf, SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

#if	   TEST_XNFS

private void
test4()
{
	int	fd;
	struct	stat buf;
	int	pathok = 0;

	DBUG_ENTER("test4");

	/*
	 *  NOTE: The default file system has AttribCaching=False
	 */

	testfail = 0;

	if ((fd = creat(t4_file,  MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not creat file \"%s\"", t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T4_UNLINK_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to UNLINK_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbfstat(fd, &buf, SYSERROR, ESTALE);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test4()
{
	DBUG_ENTER("test4");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private int
dofstat(file, buf)
char *file;
struct stat *buf;
{
	int 	fd, rval, err;
	int	pathok = 0;

	DBUG_ENTER("dofstat");

	SET_TIMEOUT(WAITTIME)

	fd = open(file, O_RDONLY|O_NONBLOCK);
	err = errno;

	CLEAR_ALARM

	if (fd < 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) timed out",
				file);
		else
			in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) failed - errno %d (%s)",
				file, err, errname(err));
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	globok = 0;
	rval = tbfstat(fd, buf, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(2);

	(void) close(fd);
	DBUG_RETURN(rval);
}

private int
tbfstat(tbfd, tbbuf, expret, experrno)
int	tbfd;
struct stat *tbbuf;
int	expret;
int	experrno;
{
	int	rval, err;
	int	pathok = 0;

	DBUG_ENTER("tbfstat");

	errno = NOERROR;
	rval = fstat(tbfd, tbbuf);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;

	if (expret == SUCCEED && rval == expret)
	{
		PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_RETURN(rval);
	}
	else if (expret == SYSERROR && rval == expret && err == experrno)
	{
		PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_RETURN(rval);
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("fstat(%d, buf) did not give correct results", tbfd);
	valrpt(expret, rval, experrno, err);

	DBUG_RETURN(rval);
}


