/*
*      SCCS:  @(#)  POSIX.os/files/getcwd/getcwd.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/getcwd - T.getcwd Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.getcwd

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the getcwd() interface.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	3 May 1989
MODIFICATIONS:
		Geoff Clare, 25 Apr 1990
			Varying argument types.
			Only do (size < 0) EINVAL test in XPG3 mode.

		David Sawyer
		Mon Jul 30 13:19:11 BST 1990
		Added resilience code.

		Geoff Clare, 4 Mar 1992
		Add chmod() before stat() in tbgetcwd().

		David Sawyer, 17 Oct 1992
		Added TEST_XNFS code

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#define NO_VGETCWD

#include	<std.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<limits.h>
#include	<dbug.h>
#include	<report.h>
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getcwd
#endif


#define 	NO_TESTS	5
#define 	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();

int	no_tests = NO_TESTS;

private void	prepare_tests();
private void	clean_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private	char	*tbgetcwd();

private int		testfail;
private	struct stat 	statdot;
private	char		buf[_POSIX_PATH_MAX + 100];
private char		*top_dir = "getcwd-d";
private char		*cur_dir = "getcwd-d/getcwd-t";

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	NULL, 0
};

private void
prepare_tests()
{
	int fail, err, i;
	static char msgbuf[256];

	DBUG_ENTER("prepare_tests");

	fail = 0;

	errno = 0;
	if (vxmkdir(top_dir) == SYSERROR)
	{
		err = errno;
		fail++;
		(void) sprintf(msgbuf,
			"could not make directory \"%s\" in preparation - errno %d (%s)",
			top_dir, err, errname(err));
	}
	else if (vxmkdir(cur_dir) == SYSERROR)
	{
		err = errno;
		fail++;
		(void) sprintf(msgbuf,
			"could not make directory \"%s\" in preparation - errno %d (%s)",
			cur_dir, err, errname(err));
	}
	else if (chdir(cur_dir) == SYSERROR)
	{
		err = errno;
		fail++;
		(void) sprintf(msgbuf,
			"chdir(\"%s\") failed in preparation - errno %d (%s)",
			cur_dir, err, errname(err));
	}
	else if (stat(".", &statdot) == SYSERROR)
	{
		err = errno;
		fail++;
		(void) sprintf(msgbuf,
			"stat() failed on \".\" in preparation - errno %d (%s)",
			err, errname(err));
	}

	if (fail > 0)
	{
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, msgbuf);
	}

	DBUG_VOID_RETURN;
}

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

	(void) chmod("..", MODEANY);
	if (chdir("../..") != SYSERROR)
	{
		(void) rmdir(cur_dir);
		(void) rmdir(top_dir);
	}

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = 0;

	/* getcwd() puts absolute path of current dir in buf and returns buf */

	DBUG_ENTER("test1");

	testfail = 0;

	globok = 0;
	(void) tbgetcwd(buf, _POSIX_PATH_MAX, NOERROR);
	PATH_FUNC_TRACE;

	/* checking on contents of "buf" is done in tbgetcwd() */

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test2()
{
	DBUG_ENTER("test2");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test2()
{
	/* size <= 0 [ XPG3 mode ] */
	/* size == 0 [ other modes ] */

	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

#if defined(_XOPEN_SOURCE) && !(TEST_XPG > 3)
	globok = 0;
	(void) tbgetcwd(buf, -1, EINVAL);
	PATH_FUNC_TRACE;
#endif
	globok = 0;
	(void) tbgetcwd(buf, 0, EINVAL);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{
#if defined(_XOPEN_SOURCE) && !(TEST_XPG > 3)
		PATH_XS_RPT(2);
#else
		PATH_XS_RPT(1);
#endif
	}

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test3()
{
	/* size < strlen(path) + 1 */

	int 	size;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	globok = 0;
	if (tbgetcwd(buf, _POSIX_PATH_MAX, NOERROR) == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	size = strlen(buf);
	globok = 0;
	(void) tbgetcwd(buf, size, ERANGE);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);
			
	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* read permission denied for component of absolute path */

	int	err;
	mode_t	mode = S_IWUSR|S_IXUSR|S_IWGRP|S_IXGRP|S_IWOTH|S_IXOTH;
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	if (chmod("..", mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"..\", %#o) failed - errno %d (%s)",
			(int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbgetcwd(buf, _POSIX_PATH_MAX, EACCES);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* search permission denied for component of absolute path */

	int	err;
	mode_t	mode = S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	if (chmod("..", mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"..\", %#o) failed - errno %d (%s)",
			(int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbgetcwd(buf, _POSIX_PATH_MAX, EACCES);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private char *
tbgetcwd(tbbuf, tbsize, experrno)
char *tbbuf;
int tbsize, experrno;
{
	char *rval;
	int err;
	struct	stat 	statcwd;
	int	pathok = 0;

	DBUG_ENTER("tbgetcwd");

	errno = NOERROR;
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	rval = getcwd(tbbuf, (size_t)tbsize);
#else
	rval = getcwd(tbbuf, tbsize);
#endif
	err = errno;

#ifdef TESTBED
	if (rval != NULL)
		rval = NULL;
	else
	{
		*tbbuf = '\0';
		rval = tbbuf;
	}
#endif
#ifdef TESTBED2
	err++;
#endif

	if (experrno == NOERROR && rval == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("getcwd(buf, %d) failed - errno %d (%s)",
			tbsize, err, errname(err));
	}
	else if ((experrno == NOERROR || experrno == EACCES) && rval != NULL)
	{
		if (rval != tbbuf)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("getcwd(buf, %d) did not return buf", tbsize);
			in_rpt("return value: 0x%lx, buf: 0x%lx",
				(long)rval, (long)tbbuf);
		}
		else
			PATH_TRACE;

		if (tbbuf[0] != '/')
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("path returned by getcwd() was not an absolute pathname");
		}
		else
			PATH_TRACE;

		if (experrno == EACCES)
			(void) chmod("..", MODEANY);

		if (stat(tbbuf, &statcwd) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("stat() on path returned by getcwd() failed - errno %d (%s)",
				err, errname(err));
		}
		else if (statcwd.st_dev != statdot.st_dev ||
			 statcwd.st_ino != statdot.st_ino)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("path returned by getcwd() does not have the same st_dev and st_ino as \".\"");
			in_rpt("expected st_dev: 0x%lx, observed: 0x%lx",
				(long)statdot.st_dev, (long)statcwd.st_dev);
			in_rpt("expected st_ino: %ld, observed: %ld",
				(long)statdot.st_ino, (long)statcwd.st_ino);
		}
		else
			PATH_TRACE;

		if (testfail > 0)
			in_rpt("path returned in buf: \"%s\"", tbbuf);
	}
	else if (experrno != NOERROR)
	{
		if (rval != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("getcwd(buf, %d) succeeded unexpectedly",
				tbsize);
			in_rpt("returned string: \"%s\"", rval);
		}
		else if (experrno != err)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("getcwd(buf, %d) did not set errno correctly",
				tbsize);
			in_rpt("expected: %d (%s), observed: %d (%s)",
				experrno, errname(experrno), err, errname(err));
		}
		else
		{
			PATH_TRACE; PATH_TRACE; PATH_TRACE;
		}
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(rval);
}
