/*
*      SCCS:  @(#)  POSIX.os/files/link/link.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/link - T.link Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.link

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the link() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	28 Mar 1989
MODIFICATIONS:	
		Geoff Clare, 8 June 1989:  Add do_access() tests.

		Geoff Clare, 15 Nov 1989
			New setuprofs() argument
			New do_access() argument

		Steve Ratcliffe, 13 Mar 1990
			Add ENOSPC test.

		Geoff Clare, 5 Apr 1990
			New interface with pathtoolng() and nametoolng().
		
		Geoff Clare, 2 May 1990
			EPERM when linking directories is not supported
		
		Geoff Clare, 29 May 1990
			Allow for unlimited LINK_MAX

		David Sawyer
		Tue Jul 31 09:37:36 BST 1990
		Added resilience code.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Peter Skan, 10 Sept 1991
			Call unsetprv(PRV_LINKDIR) where necessary

		Peter Skan, 10 Oct 1991
			Unset other privileges as well, in case of
			overlap of privilege sets

		Geoff Clare, 8 May 1992
			Added PRES_NOTRAIL to one init_pathres() call.

		Neil Moses, 9 June 1992
			Check that time stamps are not changed.

		John-Paul, 28 July 1992
			mode paramemter in calls to crfile() and crdir()
			changed from 'int' to 'mode_t' type.

		David Sawyer, 28 October 1992
			Modifications for XNFS tests.

		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.

		Geoff Clare, 3 Nov 1993
			Create file-to-link-to on each call to
			t11_func() when testing XNFS.

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

                Geoff Clare, 9 June 1995
			Use VSX_LINK_ACCESS_REQD in test 10.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<string.h>
#include	<fcntl.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<pcts_max.h>
#include	<setprv.h>
#include	<pathres.h>
#include	<chktimes.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"link_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef link
#endif


#define NO_TESTS	31
#define	WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	NAMELEN		15
#define	NOK		02
#define	PATH_OK		04

extern	char	*errname();
extern	void	savtimes();
extern	char	*chktimes();
#if	   TEST_XNFS
extern	char	*xnfs_nametoolng();
#else	/* TEST_XNFS */
extern	char	**nametoolng();
#endif	/* TEST_XNFS */
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	void	cln_access();
extern	void	valrpt();
#if	   TEST_XNFS
extern	char	startdir[];
#endif	/* TEST_XNFS */

private	int 	tblink();

extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	int		vggid1;
extern	int		vguid1;

public	int	no_tests = NO_TESTS;

private	char	*t1a_file =	"link-t.1a";
private	char	*t1b_file =	"link-t.1b";
private	char	*t2_file =	"link-t.2";
private	char	*t3_dir =	"link-d.3";
private	char	*t4a_file =	"link-t.4a";
private	char	*t4b_file =	"link-t.4b";
private	char	*t5a_file =	"link-t.5a";
private	char	*t5b_files[] = {"link-t.5b", "link-t.5c", "link-t.5d",
				"link-t.5e", NULL };
private	char	*t6a_file =	"link-t.6a";
private	char	*t6b_file =	"link-t.6b";
private	char	*t7a_file =	"link-t.7a";
private	char	*t7b_file =	"link-t.7b";
private	char	*t8a_file =	"link-t.8a";
private	char	*t8b_file =	"link-t.8b";
#if	  !TEST_XNFS
private char	*t9_dir =	"link-d.9";
private char	*t9a_file =	"link-t.9a";
private char	*t9b_file =	"link-d.9/link-t.9b";
#endif	/* TEST_XNFS */
private	char	*t10_file =	"link-t.10";
private	char	*t11_file =	"link-t.11";
private char	*t12_dir =	"link-d.12";
private char	*t12a_file =	"link-d.12/link-t.12a";
private char	*t12b_file =	"link-t.12b";
#if	   TEST_XNFS
private char	*t12_file2a =	T12_FILE2A;
private char	*t12_file2b =	T12_FILE2B;
private char	*t12_file2c =	T12_FILE2C;
private char	*t12_file2d =	T12_FILE2D;
#endif	/* TEST_XNFS */
private char	*t13_dir =	"link-d.13";
private char	*t13b_file =	"link-d.13/link-t.13b";
private char	*t13a_file =	"link-t.13a";
#if	   TEST_XNFS
private char	*t13_file2a =	T13_FILE2A;
private char	*t13_file2b =	T13_FILE2B;
#endif	/* TEST_XNFS */
private	char	*t14a_file =	"link-t.14a";
private	char	*t14b_file =	"link-t.14b";
#if	  TEST_XNFS
private	char	*t15a_file =	T15A_FILE;
private	char	*t15b_file =	T15B_FILE;
#else
private	char	*t15a_file =	"link-t.15a";
private	char	*t15b_file =	"link-t.15b";
private	char	*t16_file =	"link-t.16";
#endif	/* TEST_XNFS */
private	char	*t17a_file =	"link-t.17a";
private	char	*t17b_file =	"link-t.17b";
private char	*t18_dir =	"link-d.18";
private char	*t18a_file =	"link-d.18/link-t.18a";
private char	*t18b_file =	"link-t.18b";
private	char	*t19a_file =	"link-t.19a";
private	char	*t19b_file =	"link-t.19b";
private	char	*t20_file =	"link-t.20";
private char	*t21_ndir =	"link-t.21";
private char	*t21a_file =	"link-t.21/link-t.21a";
private char	*t21b_file =	"link-t.21b";
#if	  !TEST_XNFS
private	char	*t22a_file =	"link-t.22a";
private	char	*t22b_file =	"link-t.22b";
#endif	/* TEST_XNFS */
private char	*t23_dir =	"link-d.23";
private char	*t23a_file =	"link-d.23/link-t.23a";
private char	*t23b_file =	"link-d.23/link-t.23b";
private char	*t24_dir =	"link-d.24";
private char	*t24a_file =	"link-t.24a";
private char	*t24b_file =	"link-d.24/link-t.24b";
private char	*t25_dir =	"link-d.25";
#if	   TEST_XNFS
private char	*t26_file =	T26_FILE;
private char	*t27_file =	T27_FILE;
private char	*t28_file =	T28_FILE;
private char	*t29_file =	T29_FILE;
private char	*t30_dir =	T30_DIR;
private char	*t30_file =	T30_FILE;
private char	*t31_dir =	T31_DIR;
private char	*t31_file =	T31_FILE;
#endif	/* TEST_XNFS */

private int	testfail;
private int	link_dir_supp;
private int	link_dir_err;
private	struct stat stbuf1, stbuf2;
#if	   TEST_XNFS
private	int	localfilesys;
#endif	/* TEST_XNFS */

private void	prepare_tests();
private void	cln_tests();
private void	test1();
private void	subtest1();
private void	test2();
private void	subtest2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	subtest7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	subtest23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();
private void	test30();
private void	test31();

#if	   TEST_XNFS
private void	ch_t1();
private void	ch_t2();
#endif	/* TEST_XNFS */
private void	ch_t6();
#if	   TEST_XNFS
private void	ch_t7();
#endif	/* TEST_XNFS */
private void	ch_t12();
private void	ch_t13();
#if	  !TEST_XNFS
private void	ch_t22();
#endif	/* TEST_XNFS */
#if	   TEST_XNFS
private void	ch_t23();
private void	ch_t24();
#endif	/* TEST_XNFS */
private void	ch_t25();
#if	   TEST_XNFS
private void	ch_t30();
#endif	/* TEST_XNFS */

private int	t10_func();
private int	t11_func();


public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	 test29, 29,
	 test30, 30,
	 test31, 31,
	NULL, 0
};

private	void
prepare_tests()
{
	char	*supp;
	int	i;

	DBUG_ENTER("prepare_tests");

	if (setprv(PRV_SETID) != 0)
	{
		for (i=1; i<=no_tests; i++)
			tet_delete(i, "setprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	}

	if ((supp = tet_getvar("VSX_LINK_DIR_SUPP")) == NULL ||
		*supp == '\0')
	{
		for (i=1; i<=no_tests; i++)
			tet_delete(i, "parameter VSX_LINK_DIR_SUPP not set");
		DBUG_VOID_RETURN;
	}

	link_dir_supp = (*supp == 'Y' || *supp == 'y');
	if (link_dir_supp)
		link_dir_err = (setprv(PRV_LINKDIR) != 0);

	(void) umask((mode_t)0);

	if(uids()!=0)
	{
		for(i=1; i <= no_tests; i++)
			tet_delete(i, "uids() failed");
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	char **p;

	DBUG_ENTER("cln_tests");

	(void) unlink(t1a_file);
	(void) unlink(t1b_file);
	(void) remove(t2_file);
	(void) rmdir(t3_dir);
	(void) unlink(t4a_file);
	(void) unlink(t4b_file);
	(void) unlink(t5a_file);
	for (p = t5b_files; *p != NULL; p++)
		(void) unlink(*p);
	(void) unlink(t6a_file);
	(void) unlink(t6b_file);
	(void) unlink(t7b_file);
	(void) rmdir(t7a_file);
	(void) unlink(t8a_file);
	(void) unlink(t8b_file);
	/* can't do t9 */
	(void) unlink(t12b_file);
	(void) rmdir(t12_dir);
#if	   TEST_XNFS
	(void) unlink(t12_file2b);
	(void) unlink(t12_file2d);
#endif	/* TEST_XNFS */
	(void) unlink(t13a_file);
	(void) rmdir(t13_dir);
#if	   TEST_XNFS
	(void) unlink(t13_file2a);
#endif	/* TEST_XNFS */
	(void) unlink(t14a_file);
	(void) unlink(t14b_file);
	(void) unlink(t15a_file);
#if	  !TEST_XNFS
	(void) unlink(t16_file);
#endif	/* TEST_XNFS */
	(void) unlink(t17a_file);
	(void) unlink(t17b_file);
	(void) unlink(t18b_file);
	(void) unlink(t20_file);
	(void) unlink(t21b_file);
	(void) unlink(t21_ndir);
#if	  !TEST_XNFS
	(void) rmdir(t22a_file);
#endif	/* TEST_XNFS */
	/* can't do t23 */
	(void) unlink(t24a_file);
	(void) rmdir(t24_dir);
	(void) rmdir(t25_dir);
#if	   TEST_XNFS
	(void) unlink(t26_file);
	(void) unlink(t27_file);
	(void) unlink(t28_file);
	(void) unlink(t29_file);
#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


private void
test1()
{
	DBUG_ENTER("test1");

#if	   TEST_XNFS

	/* Need to be able to link to directories, so use file system
	   with root access */

	xnfs_doroot_c(ch_t1, WAITTIME, SP_T1_DOROOT_START, SP_T1_DOROOT_END);

#else	/* TEST_XNFS */

	subtest1();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t1()
{
	DBUG_ENTER("ch_t1");

	subtest1();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
subtest1()
{
	/* pathname resolution for path1 */

	char 	*p1, *p2;
	int 	npairs, i;
	int	pathok = 0;

	DBUG_ENTER("subtest1");

	testfail = 0;

	if (link_dir_supp)
	{
		if (link_dir_err)
		{
			xx_rpt(DELETION);
			in_rpt("setprv(PRV_LINKDIR) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Can't include "/" because it is usually on a different
		   file system */
		npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_CREAT);
		if (npairs <= 0)
		{
			xx_rpt(DELETION);
			in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT|PRES_CREAT) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		npairs = init_pathres(PRES_FILES|PRES_CREAT);
		if (npairs <= 0)
		{
			xx_rpt(DELETION);
			in_rpt("init_pathres(PRES_FILES|PRES_CREAT) failed");
			DBUG_VOID_RETURN;
		}
		else
		{
			PATH_TRACE; PATH_TRACE;
		}
	}

	/* Create separate links using the two aliases and check they refer
	   to the same file */

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		globok = 0;
		if (tblink(p1, t1a_file, SUCCEED, NOERROR) != SUCCEED)
		{
			PATH_TRACE; PATH_TRACE; PATH_TRACE;
			continue;
		}
		else
			PATH_FUNC_TRACE;

		globok = 0;
		if (tblink(p2, t1b_file, SUCCEED, NOERROR) != SUCCEED)
		{
			PATH_TRACE; PATH_TRACE;
			continue;
		}
		else
			PATH_FUNC_TRACE;

#ifdef TESTBED2
#define t1b_file "."
#endif

		if (stat(t1a_file, &stbuf1) == SYSERROR ||
		    stat(t1b_file, &stbuf2) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat() failed");
		}
		else if (stbuf1.st_ino != stbuf2.st_ino)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("link() did not resolve \"%s\" and \"%s\" to the same file",
				p1, p2);
				
		}
		else
			PATH_TRACE;

#ifdef TESTBED2
#undef t1b_file
#endif

	}

	if (testfail == 0)
		PATH_XS_RPT(2+(3*npairs));
	
	(void) unlink(t1a_file);
	(void) unlink(t1b_file);
	
	cln_pathres();

	DBUG_VOID_RETURN;
}



private void
test2()
{
	DBUG_ENTER("test2");

#if	   TEST_XNFS

	/* Need to be able to link to directories, so use file system
	   with root access */

	xnfs_doroot_c(ch_t2, WAITTIME, SP_T2_DOROOT_START, SP_T2_DOROOT_END);

#else	/* TEST_XNFS */

	subtest2();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t2()
{
	DBUG_ENTER("ch_t2");

	subtest2();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
subtest2()
{
	/* pathname resolution for path2 */

	char 	*p1, *p2;
	int 	fd, npairs, i;
	int	pathok = 0;

	DBUG_ENTER("subtest2");

	testfail = 0;

	if (link_dir_supp)
	{
		if (link_dir_err)
		{
			xx_rpt(DELETION);
			in_rpt("setprv(PRV_LINKDIR) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NOTRAIL);
		if (npairs <= 0)
		{
			xx_rpt(DELETION);
			in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (crdir(t2_file, MODEANY,
				(int)geteuid(), (int)getegid()) == SYSERROR)
		{
			testfail++;
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t2_file);
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE; /* keeps path tracing balanced */
		npairs = init_pathres(PRES_FILES);
		if (npairs <= 0)
		{
			xx_rpt(DELETION);
			in_rpt("init_pathres(PRES_FILES) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if ((fd = crfile(t2_file, MODEANY,
				(int)geteuid(), (int)getegid())) == SYSERROR)
		{
			testfail++;
			xx_rpt(DELETION);
			in_rpt("could not create file \"%s\"", t2_file);
		}
		else
			PATH_TRACE;
		(void) close(fd);
	}

	/* Try linking to both names - the 2nd link should give EEXIST */

	if (testfail == 0)
	{
		for (i = 0; i < npairs; i++)
		{
			p1 = pathresnam[2*i];
			p2 = pathresnam[2*i+1];

			DBUG_PRINTF("test2", "testing \"%s\" == \"%s\"", p1, p2);

#ifndef TESTBED2
			globok = 0;
			(void) tblink(t2_file, p2, SUCCEED, NOERROR);
			PATH_FUNC_TRACE;
#endif
			globok = 0;
			(void) tblink(t2_file, p1, SYSERROR, EEXIST);
			PATH_FUNC_TRACE;

			/* Don't leave this to cln_pathres(), it uses
			   remove() - no good on linked dirs */
			(void) unlink(p2);
		}
	}

	(void) remove(t2_file);

	cln_pathres();

	if (testfail == 0)
#ifndef TESTBED2
		PATH_XS_RPT(3+(npairs*2));
#else
		PATH_XS_RPT(3+(npairs*1));
#endif
	
	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* portable filenames */

	char	**p;
	int	fd, err;
	size_t	dlen;
	char	newf[50];
	int 	i;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	if (vxmkdir(t3_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t3_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) strcat(strcpy(newf, t3_dir), "/");
	dlen = strlen(newf);

	i = 0;
	for (p = portfnam; *p != NULL; p++,i++)
	{
		if (i == 0)
			PATH_TRACE;
		(void) strcpy(&newf[dlen], *p);
		if (eaccess(newf, F_OK) == 0 && remove(newf) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: can not remove %s", newf);
			DBUG_VOID_RETURN;
		}
	}

	/* Link each file into a subdirectory.  A duplicate path2 will
	   fail with EEXIST.  A duplicate path1 will end up with a
	   link count > 2 */

	i = 0;
	for (p = portfnam; *p != NULL; p++,i++)
	{
		if (i == 0)
			PATH_TRACE;
		if ((fd = creat(*p, MODEANY)) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create file \"%s\"",
				*p);
			DBUG_VOID_RETURN;
		}
		(void) close(fd);
#ifdef TESTBED2
		if (p == portfnam)
		{
			(void) strcpy(&newf[dlen], portfnam[1]);
			(void) link(portfnam[0], newf);
		}
#endif
		(void) strcpy(&newf[dlen], *p);
		/* The EEXIST is so tblink won't unlink an existing file */
		(void) tblink(*p, newf, SUCCEED, EEXIST);
	}

	i = 0;
	for (p = portfnam; *p != NULL; p++,i++)
	{
		if (i == 0)
			PATH_TRACE;
		if (stat(*p, &stbuf1) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				*p, err, errname(err));
		}
		else if (stbuf1.st_nlink != 2)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("link count on file \"%s\" was %d, expected 2",
				*p, (int)stbuf1.st_nlink);
		}
	}

	if (testfail == 0)
		PATH_XS_RPT(4);

	for (p = portfnam; *p != NULL; p++)
	{
		(void) strcpy(&newf[dlen], *p);
		(void) unlink(*p);
		(void) unlink(newf);
	}
	(void) rmdir(t3_dir);
	
	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* new link accesses same data */

	int 	fd, err, rval;
#if	  !TEST_XNFS
	int 	fd2;
#endif	/* TEST_XNFS */
	static 	char wbuf[] = "Hey Ford, did you know your robot can hum like Pink Floyd?";
	char 	rbuf[sizeof(wbuf)];
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	if ((fd = creat(t4a_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t4a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, wbuf, sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() to file failed - errno %d (%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t4a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	if (tblink(t4a_file, t4b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
#define wbuf "garbage"
#endif

	if ((fd = open(t4b_file, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY) failed - errno %d (%s)",
			t4b_file, err, errname(err));
	}
	else if ((rval = read(fd, rbuf, sizeof(wbuf))) != sizeof(wbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on new link returned %d, expected %d",
			rval, (int)sizeof(wbuf));
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (memcmp((void *)rbuf, (void *)wbuf, (size_t) sizeof(wbuf)) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from new link did not access same data");
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
#undef wbuf
#endif

	(void) close(fd);
	(void) unlink(t4a_file);
	(void) unlink(t4b_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t4a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t4a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd2 = open(t4a_file, O_RDONLY|O_NONBLOCK)) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) failed - errno %d (%s)",
			t4a_file, err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if ((fd = open(t4a_file, O_WRONLY|O_NONBLOCK)) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_WRONLY|O_NONBLOCK) failed - errno %d (%s)",
			t4a_file, err, errname(err));
	}
	else
		PATH_TRACE;

	if (write(fd, wbuf, sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to fifo failed - errno %d (%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t4a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	if (tblink(t4a_file, t4b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
#define wbuf "garbage"
#endif

	if ((fd = open(t4b_file, O_RDONLY|O_NONBLOCK)) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) failed - errno %d (%s)",
			t4b_file, err, errname(err));
	}
	else if ((rval = read(fd, rbuf, sizeof(wbuf))) != sizeof(wbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on new link returned %d, expected %d",
			rval, (int)sizeof(wbuf));
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (memcmp((void *)rbuf, (void *)wbuf, sizeof(wbuf)) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from new link did not access same data");
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
#undef wbuf
#endif

	(void) close(fd);
	(void) close(fd2);
	(void) unlink(t4a_file);
	(void) unlink(t4b_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(10);
	
	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* link() increments link count by 1 */

	int	fd, err, oldcount;
	char	**p;
	int 	i;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	if ((fd = creat(t5a_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t5a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (stat(t5a_file, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t5a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	oldcount = stbuf1.st_nlink;

	i = 0;
	for (p = t5b_files; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		if (tblink(t5a_file, *p, SUCCEED, NOERROR) != SUCCEED)
			break;

		if (stat(t5a_file, &stbuf1) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				t5a_file, err, errname(err));
			break;
		}
#ifdef TESTBED2
		oldcount++;
#endif
		if (stbuf1.st_nlink != oldcount + 1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("link(\"%s\", \"%s\") did not increment link count by 1",
				t5a_file, *p);
			in_rpt("old link count = %d, new count = %d",
				oldcount, (int)stbuf1.st_nlink);
		}
		oldcount = stbuf1.st_nlink;
	}

	(void) unlink(t5a_file);
	i = 0;
	for (p = t5b_files; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		(void) unlink(*p);
	}

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */


	if (mkfifo(t5a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t5a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(t5a_file, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t5a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	oldcount = stbuf1.st_nlink;

	i = 0;
	for (p = t5b_files; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;

		if (tblink(t5a_file, *p, SUCCEED, NOERROR) != SUCCEED)
			break;

		if (stat(t5a_file, &stbuf1) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				t5a_file, err, errname(err));
			break;
		}
#ifdef TESTBED2
		oldcount++;
#endif
		if (stbuf1.st_nlink != oldcount + 1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("link(\"%s\", \"%s\") did not increment link count by 1",
				t5a_file, *p);
			in_rpt("old link count = %d, new count = %d",
				oldcount, (int)stbuf1.st_nlink);
		}
		oldcount = stbuf1.st_nlink;
	}

	(void) unlink(t5a_file);
	i = 0;
	for (p = t5b_files; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		(void) unlink(*p);
	}

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(8);
	
	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* new link has same mode, uid and gid */

	DBUG_ENTER("test6");

	(void) cppair(ch_t6, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t6()
{
	int	fd, err;
	mode_t	mask = (S_IRUSR|S_IWGRP|S_IXOTH);
	int	pathok = 0;

	DBUG_ENTER("ch_t6");

	testfail = 0;

	(void) setuid((uid_t)vguid1);
	if (getuid() != vguid1 || geteuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user id to VSX_UID1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_LINKDIR) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_LINKDIR) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_ACCESS) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ACCESS) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_MOUNT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t6a_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t6a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (stat(t6a_file, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
			t6a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) umask(mask);

	globok = 0;
	if (tblink(t6a_file, t6b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
#define t6b_file "."
#endif
	if (stat(t6b_file, &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
			t6b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
#undef t6b_file
#endif

	if (stbuf1.st_mode != stbuf2.st_mode)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("new link does not have same mode: expected %#o, observed %#o",
			(unsigned int) stbuf1.st_mode,
			(unsigned int) stbuf2.st_mode);
	}
	else
		PATH_TRACE;

	if (stbuf1.st_uid != stbuf2.st_uid)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("new link does not have same uid: expected %d, observed %d",
			(int) stbuf1.st_uid, (int) stbuf2.st_uid);
	}
	else
		PATH_TRACE;

	if (stbuf1.st_gid != stbuf2.st_gid)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("new link does not have same gid: expected %d, observed %d",
			(int) stbuf1.st_gid, (int) stbuf2.st_gid);
	}
	else
		PATH_TRACE;

	(void) unlink(t6a_file);
	(void) unlink(t6b_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t6a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t6a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(t6a_file, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
			t6a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) umask(mask);

	globok = 0;
	if (tblink(t6a_file, t6b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
#define t6b_file "."
#endif
	if (stat(t6b_file, &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
			t6b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
#undef t6b_file
#endif

	if (stbuf1.st_mode != stbuf2.st_mode)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("new link does not have same mode: expected %#o, observed %#o",
			(unsigned int) stbuf1.st_mode,
			(unsigned int) stbuf2.st_mode);
	}
	else
		PATH_TRACE;

	if (stbuf1.st_uid != stbuf2.st_uid)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("new link does not have same uid: expected %d, observed %d",
			(int) stbuf1.st_uid, (int) stbuf2.st_uid);
	}
	else
		PATH_TRACE;

	if (stbuf1.st_gid != stbuf2.st_gid)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("new link does not have same gid: expected %d, observed %d",
			(int) stbuf1.st_gid, (int) stbuf2.st_gid);
	}
	else
		PATH_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(19);

	(void) unlink(t6a_file);
	(void) unlink(t6b_file);

	DBUG_VOID_RETURN;
}



private void
test7()
{
	DBUG_ENTER("test7");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t7, WAITTIME, SP_T7_DOROOT_START, SP_T7_DOROOT_END);

#else	/* TEST_XNFS */

	subtest7();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t7()
{
	DBUG_ENTER("ch_t7");

	subtest7();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
subtest7()
{
	int	pathok = 0;

	/* link to a directory */

	DBUG_ENTER("subtest7");

	testfail = 0;

	if (!link_dir_supp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_LINK_DIR_SUPP not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (link_dir_err)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_LINKDIR) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t7a_file, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t7a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tblink(t7a_file, t7b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) unlink(t7b_file);
	(void) rmdir(t7a_file);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* link() updates timestamps */

	char	*chkt;
	int	fd, err;
	int	pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

	if ((fd = creat(t8a_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t8a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (stat(".", &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\".\") failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(t8a_file);
	PATH_FUNC_TRACE;

#ifndef TESTBED2
	globok = 0;
	if (tblink(t8a_file, t8b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
#endif

	globok = 0;
	chkt = chktimes(t8a_file, NEWCTIME);
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("path1 %s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	if (stat(".", &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf2.st_ctime == stbuf1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	if (stbuf2.st_mtime == stbuf1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	(void) unlink(t8a_file);
	(void) unlink(t8b_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t8a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t8a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(".", &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\".\") failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(t8a_file);
	PATH_FUNC_TRACE;

#ifndef TESTBED2
	globok = 0;
	if (tblink(t8a_file, t8b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
#endif

	globok = 0;
	chkt = chktimes(t8a_file, NEWCTIME);
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("path1 %s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	if (stat(".", &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf2.st_ctime == stbuf1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	if (stbuf2.st_mtime == stbuf1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	(void) unlink(t8a_file);
	(void) unlink(t8b_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(16);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test9()
{
	DBUG_ENTER("test9");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test9()
{
	/* link across file systems */

	char	*supp;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test9");

	testfail = 0;

	if ((supp = tet_getvar("VSX_LINK_FILESYS_SUPP")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_LINK_FILESYS_SUPP not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*supp != 'Y' && *supp != 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_LINK_FILESYS_SUPP not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t9a_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t9a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	if (mountfs(t9_dir, MODEANY) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tblink(t9a_file, t9b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	
	(void) unlink(t9a_file);
	(void) unlink(t9b_file);
	(void) umntfs(t9_dir);

	if (mkfifo(t9a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t9a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (mountfs(t9_dir, MODEANY) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tblink(t9a_file, t9b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	
	(void) unlink(t9a_file);
	(void) unlink(t9b_file);
	(void) umntfs(t9_dir);

	if (testfail == 0)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test10()
{
	/* search permission granted on path prefix of path1 */

	char	*accreqd;
	mode_t	fmode = 0;
	int	pathok = 0;

	DBUG_ENTER("test10");

	accreqd = tet_getvar("VSX_LINK_ACCESS_REQD");
	if (accreqd != NULL && (*accreqd == 'y' || *accreqd == 'Y'))
		fmode = S_IRWXU;

	globok = 0;
	if (do_access('f', fmode, (mode_t) S_IXUSR, t10_func, WAITTIME, SP_T10_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(1);
	}
	
	cln_access();

	DBUG_VOID_RETURN;
}

private int
t10_func(file, desc)
char *file, **desc;
{
	int	pathok = 0;

	DBUG_ENTER("t10_func");

	testfail = 0;
	globok = 0;
	(void) tblink(file, t10_file, SUCCEED, NOERROR);
	if (testfail != 0)
	{
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1);

	(void) unlink(t10_file);
	DBUG_RETURN(testfail);
}

private void
test11()
{
	/* search & write permission granted on parent of path2 */

	int 	fd;
	int	pathok = 0;

	DBUG_ENTER("test11");

	if ((fd = creat(t11_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	if (do_access('d', (mode_t) (S_IWUSR|S_IXUSR), (mode_t) S_IXUSR, 
			t11_func, WAITTIME, SP_T11_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(2);
	}
	
	cln_access();
	(void) unlink(t11_file);

	DBUG_VOID_RETURN;
}

private int
t11_func(dir, desc)
char *dir, **desc;
{
	char 	file[50];
	int	pathok = 0;
#if TEST_XNFS
	int	fd;
#endif

	DBUG_ENTER("t11_func");

	(void) strcat(strcpy(file, dir), "/link-t.11");

	testfail = 0;

#if TEST_XNFS
	(void) unlink(t11_file);
	if ((fd = creat(t11_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t11_file);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	(void) close(fd);
#else
	PATH_TRACE;
#endif

	globok = 0;
	(void) tblink(t11_file, file, SUCCEED, NOERROR);
	if (testfail != 0)
	{
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(2);

	(void) unlink(file);
#if TEST_XNFS
	(void) unlink(t11_file);
#endif
	DBUG_RETURN(testfail);
}

private void
test12()
{
	int	fd;
	int	rc;
	int	pathok = 0;

	DBUG_ENTER("test12");

	testfail = 0;

	rc = tet_fork(ch_t12, NULLFN, WAITTIME, OK | NOK | PATH_OK);

        if ((rc & OK) == 0)
	{
		testfail++;
	}
	else if (rc & PATH_OK)
	{
		PATH_TRACE;
	}

#if	   TEST_XNFS

	if (tet_sync(SP_T12_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = creat(t12_file2b, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t12_file2b);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	/* path2 is inside a directory that denies search permission */
	globok = 0;
	(void) tblink(t12_file2b, t12_file2c, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	/* path1 is inside a directory that denies search permission */
	globok = 0;
	(void) tblink(t12_file2a, t12_file2d, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (tet_sync(SP_T12_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) unlink(t12_file2b);
	(void) unlink(t12_file2d);

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if(testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	int	fd;
	mode_t	mode = (mode_t)(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
	int	pathok = 0;
	int	t12_exit = NOK;

	DBUG_ENTER("ch_t12");

	(void) setuid((uid_t)vguid1);
	if (getuid() != vguid1 || geteuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user id to VSX_UID1");
		DBUG_EXIT(t12_exit);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_LINKDIR) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_LINKDIR) failed");
		DBUG_EXIT(t12_exit);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_ACCESS) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ACCESS) failed");
		DBUG_EXIT(t12_exit);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_MOUNT) failed");
		DBUG_EXIT(t12_exit);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_EXIT(t12_exit);
	}
	else
		PATH_TRACE;

	if (crdir(t12_dir, mode, vguid1, (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory %s", t12_dir);
		DBUG_EXIT(t12_exit);
	}
	else
		PATH_TRACE;

	/* search permission for path1 denied */

	globok = 0;
	(void) tblink(t12a_file, t12b_file, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	/* search permission for path2 denied */

	if ((fd = crfile(t12b_file, MODEANY,
			vguid1, (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t12b_file);
		DBUG_EXIT(t12_exit);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(t12b_file, t12a_file, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{
		t12_exit = OK;
		if(pathok == 9)
			t12_exit |= PATH_OK;
	}

	(void) unlink(t12b_file);
	(void) rmdir(t12_dir);

	DBUG_EXIT(t12_exit);
}

private void
test13()
{
	/* create link in directory which denies write permission */

	int	pathok = 0;
	int	fd;
	int	rc;

	DBUG_ENTER("test13");

	testfail = 0;


	rc = tet_fork(ch_t13, NULLFN, WAITTIME, OK | NOK | PATH_OK);

        if ((rc & OK) == 0)
	{
		testfail++;
	}
	else if (rc & PATH_OK)
	{
		PATH_TRACE;
	}


#if	   TEST_XNFS

	if (tet_sync(SP_T13_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = creat(t13_file2a, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t13_file2a);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tblink(t13_file2a, t13_file2b, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (tet_sync(SP_T13_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) unlink(t13_file2a);

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if(testfail == 0)
		PATH_XS_RPT(5);


	DBUG_VOID_RETURN;
}


private void
ch_t13()
{
	int	fd;
	mode_t	mode = (mode_t)(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	int	pathok = 0;
	int	t13_exit = NOK;

	DBUG_ENTER("ch_t13");

	(void) setuid((uid_t)vguid1);
	if (getuid() != vguid1 || geteuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user id to VSX_UID1");
		DBUG_EXIT(t13_exit);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_LINKDIR) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_LINKDIR) failed");
		DBUG_EXIT(t13_exit);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_ACCESS) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ACCESS) failed");
		DBUG_EXIT(t13_exit);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_MOUNT) failed");
		DBUG_EXIT(t13_exit);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_EXIT(t13_exit);
	}
	else
		PATH_TRACE;

	if (crdir(t13_dir, mode, vguid1, (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory %s", t13_dir);
		DBUG_EXIT(t13_exit);
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t13a_file, MODEANY,
			vguid1, (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t13a_file);
		DBUG_EXIT(t13_exit);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(t13a_file, t13b_file, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{
		t13_exit = OK;
		if(pathok == 8)
			t13_exit |= PATH_OK;
	}

	(void) unlink(t13a_file);
	(void) rmdir(t13_dir);

	DBUG_EXIT(t13_exit);
}

private void
test14()
{
	/* path2 already exists */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test14");

	testfail = 0;

	if ((fd = creat(t14a_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t14a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if ((fd = creat(t14b_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t14b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(t14a_file, t14b_file, SYSERROR, EEXIST);
	PATH_FUNC_TRACE;

	(void) unlink(t14a_file);
	(void) unlink(t14b_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t14a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t14a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (mkfifo(t14b_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t14b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tblink(t14a_file, t14b_file, SYSERROR, EEXIST);
	PATH_FUNC_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) unlink(t14a_file);
	(void) unlink(t14b_file);

	DBUG_VOID_RETURN;
}

private void
test15()
{
	char	*links[PCTS_LINK_MAX], *nambuf;
	long	i, link_max, testmax = PCTS_LINK_MAX;
	int	fd;
	int	pathok = 0;
#if	   TEST_XNFS
	int	nvals;
	char	**strvals;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test15");

	if ((fd = creat(t15a_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t15a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

#if	   TEST_XNFS

	if (tet_sync(SP_T15_CRFILE_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to CRFILE_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Get the value of LINK_MAX from the server */
	nvals = 0;
	strvals = xnfs_getsv_c(&nvals, SP_T15_PLM_TRANSFER);
	if (strvals == NULL || nvals != 1)
	{
		xx_rpt(DELETION);
		in_rpt("xnfs_getsv_c() failed when getting _PC_LINK_MAX from server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	link_max = atol(strvals[0]);

	if(link_max <= 0)
	{
		if (link_max == 0)
		{
			link_max = PCTS_LINK_MAX+1;
			PATH_TRACE;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("pathconf(\"%s\", _PC_LINK_MAX) failed on server", t15a_file);
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;

	errno = 0;
	link_max = pathconf(t15a_file, _PC_LINK_MAX);
	if (link_max < 0)
	{
		if (errno == 0)
		{
			link_max = PCTS_LINK_MAX+1;
			PATH_TRACE;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("pathconf(\"%s\", _PC_LINK_MAX) failed", t15a_file);
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

#endif	/* TEST_XNFS */

	if (link_max < PCTS_LINK_MAX)
	{
		PATH_TRACE;
		testmax = link_max;
	}
	else
		PATH_TRACE;

	nambuf = (char *) malloc((size_t)(testmax * NAMELEN));
	if (nambuf == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: malloc() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create lesser of LINK_MAX and PCTS_LINK_MAX links successfully */
	/* i.e. make (testmax-1) link() calls, as there is 1 link already */

	for (i = 0; i < testmax-1; i++)
	{
		if (i==0)
			PATH_TRACE;
		links[i] = &nambuf[i*NAMELEN];
		(void) sprintf(links[i], "L%ld", i+1);
		if (tblink(t15a_file, links[i], SUCCEED, NOERROR) != SUCCEED)
			break;
	}
	links[i] = NULL;

	/* if LINK_MAX is testable, next link gives EMLINK */

	if (i == testmax-1 && link_max <= PCTS_LINK_MAX)
	{
		globok = 0;
		(void) tblink(t15a_file, t15b_file, SYSERROR, EMLINK);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	(void) unlink(t15a_file);

	for (i = 0; i < testmax-1 && links[i] != NULL; i++)
	{
		if (i==0)
			PATH_TRACE;
		(void) unlink(links[i]);
	}
	free((void *) nambuf);

	if (testfail == 0)
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test16()
{
	DBUG_ENTER("test16");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test16()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test16");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname[0], MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			longname[0]);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(longname[0], t16_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	(void) unlink(t16_file);

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tblink(longname[1], t16_file, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	(void) unlink(longname[0]);

	if ((fd = crfile(t16_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(t16_file, longname[0], SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tblink(t16_file, longname[1], SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	(void) unlink(longname[0]);
	clntoolng();
	(void) unlink(t16_file);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test17()
{
	/* path component longer than PATH_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test17");

	testfail = 0;

	globok = 0;
	longname = pathtoolng(t17a_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = creat(t17a_file, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t17a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(longname[0], t17b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t17b_file);

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tblink(longname[1], t17b_file, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;
	(void) unlink(t17a_file);

	if ((fd = creat(t17b_file, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t17b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(t17b_file, longname[0], SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tblink(t17b_file, longname[1], SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	clntoolng();
	(void) unlink(t17a_file);
	(void) unlink(t17b_file);

	DBUG_VOID_RETURN;
}

private void
test18()
{
	/* component of path prefix does not exist */

	int 	fd;
	int	pathok = 0;

	DBUG_ENTER("test18");

	testfail = 0;

	if (eaccess(t18_dir, F_OK) == 0 && remove(t18_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t18_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tblink(t18a_file, t18b_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if ((fd = creat(t18b_file, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t18b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(t18b_file, t18a_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) unlink(t18b_file);

	DBUG_VOID_RETURN;
}

private void
test19()
{
	/* link to file which does not exist */

	int	pathok = 0;

	DBUG_ENTER("test19");

	testfail = 0;

	if (eaccess(t19a_file, F_OK) == 0 && remove(t19a_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t19a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tblink(t19a_file, t19b_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test20()
{
	/* path1 or path2 is "" */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test20");

	testfail = 0;

	globok = 0;
	(void) tblink("", t20_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if ((fd = creat(t20_file, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t20_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(t20_file, "", SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(t20_file);

	DBUG_VOID_RETURN;
}

private void
test21()
{
	/* component of path is not a directory */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test21");

	testfail = 0;

	if ((fd = creat(t21_ndir, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t21_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(t21a_file, t21b_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	if ((fd = creat(t21b_file, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t21b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(t21b_file, t21a_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	(void) unlink(t21b_file);
	(void) unlink(t21_ndir);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t21_ndir, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t21_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(t21a_file, t21b_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	if ((fd = crfile(t21b_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t21b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tblink(t21b_file, t21a_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(8);

	(void) unlink(t21b_file);
	(void) unlink(t21_ndir);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test22()
{
	DBUG_ENTER("test22");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test22()
{
	/* link to directory without appropriate privilege */
	/* link to directory when not supported */

	DBUG_ENTER("test22");

	testfail = 0;

	(void) cppair(ch_t22, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t22()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t22");

	if (link_dir_supp)
	{
		/* linking is supported: remove privilege */

		(void) setuid((uid_t)vguid1);
		if (getuid() != vguid1 || geteuid() != vguid1)
		{
			xx_rpt(DELETION);
			in_rpt("setuid() did not set effective user id to VSX_UID1");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_LINKDIR) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_LINKDIR) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_ACCESS) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_ACCESS) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_MOUNT) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_MOUNT) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_SETID) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_SETID) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

	}
	else
	{
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
	}

	if (crdir(t22a_file, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t22a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tblink(t22a_file, t22b_file, SYSERROR, EPERM);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	(void) rmdir(t22a_file);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test23()
{
	DBUG_ENTER("test23");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t23, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest23();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t23()
{
	DBUG_ENTER("ch_t23");

	subtest23();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
subtest23()
{
	/* create link on read only file system */

	char	*errptr;
	int	pathok = 0;

	DBUG_ENTER("subtest23");

	testfail = 0;

	globok = 0;
	if (setuprofs(t23_dir, t23a_file, 'f', (mode_t) MODEANY) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(t23_dir);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tblink(t23a_file, t23b_file, SYSERROR, EROFS);
	PATH_FUNC_TRACE;

	globok = 0;
	errptr = chktimes(t23_dir, 0);
	PATH_FUNC_TRACE;

	if(errptr != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful link() changed time stamps on %s",
			t23_dir);
		in_rpt("%s", errptr);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	(void) clnuprofs(t23_dir, t23a_file);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS


private void
test24()
{
	/* link across file systems */

	int	rc;
	int	pathok = 0;

	DBUG_ENTER("test24");

	/* remote file system to remote file system */
	localfilesys = 0;


	rc = tet_fork(ch_t24, NULLFN, WAITTIME, OK | NOK | PATH_OK);

        if ((rc & OK) == 0)
	{
		DBUG_VOID_RETURN;
	}
	else if (rc & PATH_OK)
	{
		PATH_TRACE;
	}

	/* remote file system to local file system */
	localfilesys = 1;


	rc = tet_fork(ch_t24, NULLFN, WAITTIME, OK | NOK | PATH_OK);

        if ((rc & OK) == 0)
	{
		DBUG_VOID_RETURN;
	}
	else if (rc & PATH_OK)
	{
		PATH_TRACE;
	}

	PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}



private void
ch_t24()
{
	int	fd;
	int	pathok = 0;
	int	t24_exit = NOK;

	DBUG_ENTER("ch_t24");

	/* The value of localfilesys is inherited from the parent */
	if(localfilesys)
	{
		if(chdir(startdir) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("unable to change dir to \"%s\"", startdir);
			DBUG_EXIT(t24_exit);
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	testfail = 0;
	if ((fd = creat(t24a_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t24a_file);
		DBUG_EXIT(t24_exit);
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	if (mountfs(t24_dir, MODEANY) != 0)
	{
		(void) unlink(t24a_file);
		DBUG_EXIT(t24_exit);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tblink(t24a_file, t24b_file, SYSERROR, EXDEV);
	PATH_FUNC_TRACE;
	
	if (testfail == 0)
	{
		t24_exit = OK;
		if(pathok == 4)
			t24_exit |= PATH_OK;
	}

	(void) unlink(t24a_file);
	(void) umntfs(t24_dir);

	DBUG_EXIT(t24_exit);
}


#else	/* TEST_XNFS */


private void
test24()
{
	/* link across file systems */

	int	fd;
	char	*supp;
	int	pathok = 0;

	DBUG_ENTER("test24");

	if ((supp = tet_getvar("VSX_LINK_FILESYS_SUPP")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_LINK_FILESYS_SUPP not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*supp == 'Y' || *supp == 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_LINK_FILESYS_SUPP is set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	if ((fd = crfile(t24a_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t24a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	if (mountfs(t24_dir, MODEANY) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tblink(t24a_file, t24b_file, SYSERROR, EXDEV);
	PATH_FUNC_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(5);

	(void) unlink(t24a_file);
	(void) umntfs(t24_dir);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test25()
{

	DBUG_ENTER("test25");

	(void) cppair(ch_t25, NULLFN, 10 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t25()
{
	extern	void	enospc();

	DBUG_ENTER("ch_t25");

	enospc(t25_dir);

	DBUG_VOID_RETURN;
}



#if TEST_XNFS

private void
test26()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test26");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	/* Note: can't create file because name is too long */

	globok = 0;
	(void) tblink(longname, t26_file, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	(void) unlink(t26_file);

	if ((fd = creat(t26_file, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t26_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tblink(t26_file, longname, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	(void) unlink(t26_file);
	(void) unlink(longname);

	if (testfail == 0)
		PATH_XS_RPT(4);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test26()
{
	DBUG_ENTER("test26");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test27()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test27");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Note: can't create file because name is too long */

	globok = 0;
	(void) tblink(longname, t27_file, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	(void) unlink(t27_file);

	if ((fd = crfile(t27_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t27_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tblink(t27_file, longname, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	(void) unlink(t27_file);
	(void) unlink(longname);

	if (testfail == 0)
		PATH_XS_RPT(4);

	clntoolng();


	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test27()
{
	DBUG_ENTER("test27");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test28()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test28");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = creat(longname, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			longname);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tblink(longname, t28_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t28_file);
	(void) unlink(longname);

	if ((fd = creat(t28_file, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t28_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tblink(t28_file, longname, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t28_file);
	(void) unlink(longname);

	if (testfail == 0)
		PATH_XS_RPT(5);

	clntoolng();


	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test28()
{
	DBUG_ENTER("test28");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test29()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test29");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = creat(longname, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			longname);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tblink(longname, t29_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t29_file);
	(void) unlink(longname);

	if ((fd = creat(t29_file, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t29_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tblink(t29_file, longname, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t29_file);
	(void) unlink(longname);

	if (testfail == 0)
		PATH_XS_RPT(5);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test29()
{
	DBUG_ENTER("test29");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if	   TEST_XNFS

private void
test30()
{
	DBUG_ENTER("test30");


	xnfs_doroot_c(ch_t30, WAITTIME, SP_T30_DOROOT_START, SP_T30_DOROOT_END);

	DBUG_VOID_RETURN;
}


private void
ch_t30()
{
	int	pathok = 0;

	/* link to a directory */

	DBUG_ENTER("ch_t30");

	testfail = 0;

	if (link_dir_supp)
	{
		/* remove privileges */

		(void) setuid((uid_t)vguid1);
		if (getuid() != vguid1 || geteuid() != vguid1)
		{
			xx_rpt(DELETION);
			in_rpt("setuid() did not set effective user id to VSX_UID1");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_LINKDIR) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_LINKDIR) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_ACCESS) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_ACCESS) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_MOUNT) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_MOUNT) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_SETID) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_SETID) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		/* links to directories not supported */
		if (link_dir_err)
		{
			xx_rpt(DELETION);
			in_rpt("setprv(PRV_LINKDIR) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	if (crdir(t30_dir, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t30_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tblink(t30_dir, t30_file, SYSERROR, EPERM);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	(void) unlink(t30_dir);
	(void) rmdir(t30_file);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test30()
{
	DBUG_ENTER("test30");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if	   TEST_XNFS

private void
test31()
{
	int	pathok = 0;

	/* link to a directory */

	DBUG_ENTER("test31");

	testfail = 0;

	if (!link_dir_supp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_LINK_DIR_SUPP not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (link_dir_err)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_LINKDIR) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (vxmkdir(t31_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t31_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tblink(t31_dir, t31_file, SYSERROR, EPERM);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) unlink(t31_dir);
	(void) rmdir(t31_file);

	DBUG_VOID_RETURN;
}


#else /* TEST_XNFS */

private void
test31()
{
	DBUG_ENTER("test31");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private int
tblink(tbpath1, tbpath2, expret, experrno)
char *tbpath1, *tbpath2;
int  expret, experrno;
{
	char	*reppath1 = tbpath1;
	char	*reppath2 = tbpath2;
	char	pathbuf1[30], pathbuf2[30];
	int	rval, obserr, err, statok, oldcount;
	dev_t	olddev;
	ino_t	oldino;
	struct 	stat stbuf;
	int	pathok = 0;

	DBUG_LENTER("tblink",2);

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath1) > 100)
	{
		(void) sprintf(pathbuf1, "<%d char path>", (int)strlen(tbpath1));
		reppath1 = pathbuf1;
		PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (strlen(tbpath2) > 100)
	{
		(void) sprintf(pathbuf2, "<%d char path>", (int)strlen(tbpath2));
		reppath2 = pathbuf2;
		PATH_TRACE;
	}
	else
		PATH_TRACE;

	/* remove any existing file, if necessary */
	if (experrno != EEXIST && eaccess(tbpath2, F_OK) == 0)
	{
		if (unlink(tbpath2) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: unlink(\"%s\") failed - errno %d (%s)",
				reppath2, err, errname(err));
			return (SYSERROR);
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (expret == SUCCEED)
		experrno = NOERROR;	/* may be EEXIST to suppress remove */

	if (stat(tbpath1, &stbuf) == SYSERROR)
	{
		if (expret != SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				reppath1, err, errname(err));
		}
		else
			PATH_TRACE;
		statok = FALSE;
	}
	else
	{
		statok = TRUE;
		oldcount = stbuf.st_nlink;
		olddev = stbuf.st_dev;
		oldino = stbuf.st_ino;
		PATH_TRACE;
	}

	errno = 0;
	rval = link(tbpath1, tbpath2);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	obserr = errno;

	if (expret != SYSERROR && rval == expret)
	{
		/* Check links refer to the same file */

		if (stat(tbpath2, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("stat() failed on new link \"%s\" - errno %d (%s)",
				reppath2, err, errname(err));
		}
		else if (stbuf.st_dev != olddev || stbuf.st_ino != oldino)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("new link does not have the same st_dev and st_ino as original file");
		}
		else
			PATH_TRACE;

		PATH_FUNC_RPT(5);
		DBUG_RETURN(rval);
	}
	else if (expret == SYSERROR && rval == expret)
	{
		/* assertion 11 */

		if (statok && stat(tbpath1, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				reppath1, err, errname(err));
		}
		else if (statok && stbuf.st_nlink != oldcount)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to link() changed link count from %d to %d",
				oldcount, (int)stbuf.st_nlink);
		}
		else
			PATH_TRACE;

		if (eaccess(tbpath2, F_OK) != SYSERROR && experrno != EEXIST)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to link() created new link");
		}
		else
			PATH_TRACE;

		if (experrno == obserr)
		{
			PATH_FUNC_RPT(6);
			DBUG_RETURN(rval);
		}
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("link(\"%s\", \"%s\") did not give correct results",
		reppath1, reppath2);
	valrpt(expret, rval, experrno, obserr);
	
	if (expret == SYSERROR && rval != expret)
		(void) unlink(tbpath2);

	DBUG_RETURN(rval);
}
