/*
*      SCCS:  @(#)  POSIX.os/files/mkdir/mkdir.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/mkdir - T.mkdir Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.mkdir

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the mkdir() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	7 Apr 1989
MODIFICATIONS:
		Geoff Clare, 9 June 1989:  Add do_access() test.

		Geoff Clare, 5 Apr 1990
			New interface with pathtoolng() and nametoolng().
		
		Geoff Clare, 29 May 1990
			Allow for unlimited LINK_MAX

		David Sawyer
		Thu Aug 02 09:29:20 BST 1990
		Added resiliance code.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 8 May 1992
			Added PRES_NOTRAIL to init_pathres() call.

		John-Paul, 28 July 1992
			mode parameter of call to crfile() and crdir() changed
			from 'int' to 'mode_t'.

		David Sawyer, 2 November 1992
			Modfications for XNFS tests.

		Geoff Clare, 8 Jan 1993
			Change xnfs_anon_uid from int to uid_t.

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

		Geoff Clare, 3 Nov 1995
			Changed child timeout for do_access() test.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<dirent.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<pcts_max.h>
#include	<pathres.h>
#include	<chktimes.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"mkdir_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef mkdir
#endif



#define NO_TESTS	27
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define UMASKVAL	(mode_t)(S_IRUSR|S_IWGRP|S_IXOTH)
#define NMODES		3
#define	NAMELEN		15
#if	   TEST_XNFS
#define NOK		02
#define PATH_OK		04
#endif	/* TEST_XNFS */

extern	char	*errname();
extern	char	*itoa();
extern	char	*uidtoa();
extern	char	*newstr();
extern	char	**pathtoolng();
#if	   TEST_XNFS
extern	char	*xnfs_nametoolng();
#else	/* TEST_XNFS */
extern	char	**nametoolng();
#endif	/* TEST_XNFS */
extern	void	clntoolng();
extern	char	*execpath();

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;
extern	int	vguid1;
extern	int	vggid1;

public	int	no_tests = NO_TESTS;
public	int	testfail;

private mode_t	modes[NMODES] = { S_IRWXU|S_IRWXG|S_IRWXO,
				  S_IXUSR|S_IWGRP|S_IROTH,
				  UMASKVAL
};

#if	  !TEST_XNFS
private char	*prog4 =	"mkdir_t4";
private char	*prog5 =	"mkdir_t5";
#endif	/* TEST_XNFS */
private char	*prog19 =	"mkdir_t19";
private char	*prog_su =	"mkdir_su";
#if	   TEST_XNFS
private char	*prog_uid =	"mkdir_uid";
private char	*prog_gid =	"mkdir_gid";
#endif	/* TEST_XNFS */
private char	*t3_file =	"mkdir-t.3";
private char	*t4_file =	"mkdir-t.4";
private char	*t5_file =	"mkdir-t.5";
private char	*t6_file =	"mkdir-t.6";
private char	*t7_file =	"mkdir-t.7";
private char	*t9_dir =	"mkdir-d.9";
private char	*t9_file =	"mkdir-d.9/mkdir-t.9";
#if	   TEST_XNFS
private char	*t9_2_file =	T9_2_FILE;
#endif	/* TEST_XNFS */
private char	*t10_dir =	"mkdir-d.10";
private char	*t10_file =	"mkdir-d.10/mkdir-t.10";
#if	   TEST_XNFS
private char	*t10_2_file =	T10_2_FILE;
#endif	/* TEST_XNFS */
private char	*t11_file =	"mkdir-t.11";
private char	*t12_file =	"mkdir-t.12";
private char	*t14_file =	"mkdir-t.14";
private char	*t15_dir =	"mkdir-d.15";
private char	*t15_file =	"mkdir-d.15/mkdir-t.15";
private char	*t17_1_file =	"mkdir-t.17";
private char	*t17_2_file =	"mkdir-t.17/mkdir-t.17.1";
private char	*t18_dir =	"mkdir-d.18";
private char	*t18_file =	"mkdir-d.18/mkdir-t.18";
private char	*t19_dir =	"mkdir-d.19";
#if	   TEST_XNFS
private char	*t20_dir =	"mkdir-d.20";
private char	*t21_dir =	"mkdir-d.21";
private char	*t22_mnt =	T22_MNT;
#endif	/* TEST_XNFS */


private void	prepare_tests();
private void	clean_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	subtest18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();

private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	ch_t8();
#if	   TEST_XNFS
private void	ch_t9();
private void	ch_t10();
private void	ch_t18();
#endif	/* TEST_XNFS */
private void	ch2_t18();
private void	ch_t19();
#if	   TEST_XNFS
private void	ch_t20();
private void	ch_t21();
private void	ch_t22();
private void	ch_t23();
#endif	/* TEST_XNFS */

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	NULL, 0
};


private	void
prepare_tests()
{
	DBUG_ENTER("prepare_tests");

	if(uids()!=0)
	{
		int	i;
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	(void) umask((mode_t)0);

#if	   TEST_XNFS
	if (xnfs_uids() < 0)
		tet_delete(21, "xnfs_uids() failed");
#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}

private	void
clean_tests()
{
	char **p;

	DBUG_ENTER("clean_tests");

	for (p = portfnam; *p != NULL; p++)
		(void) rmdir(*p);
	(void) rmdir(t3_file);
	(void) rmdir(t4_file);
	(void) rmdir(t5_file);
	(void) rmdir(t6_file);
	(void) rmdir(t7_file);
	(void) rmdir(t9_dir);
	(void) rmdir(t10_dir);
	(void) remove(t11_file);
	(void) rmdir(t12_file);
	(void) rmdir(t14_file);
	(void) unlink(t17_1_file);
	(void) rmdir(t18_dir);
	(void) rmdir(t19_dir);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* pathname resolution */

	char 	*p1, *p2;
	int 	npairs, i;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NOTRAIL);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NOTRAIL) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		/* creat using complex pathname */
		globok = 0;
		if (tbmkdir(p1, MODEANY, SUCCEED, NOERROR) != SUCCEED)
		{
			PATH_TRACE; PATH_TRACE;
			continue;
		}
		else
			PATH_FUNC_TRACE;

#ifdef TESTBED2
		(void) rmdir(p2);
#endif

		/* check for EEXIST using normal pathname */
		globok = 0;
		(void) tbmkdir(p2, MODEANY, SYSERROR, EEXIST);
		PATH_FUNC_TRACE;

		(void) rmdir(p2);
	}

	if (testfail == 0)
		PATH_XS_RPT(1+(npairs*2));
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames */

	char	**p;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		if (eaccess(*p, F_OK) == 0 && remove(*p) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not remove \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
	}

#ifdef TESTBED2
	(void) mkdir(portfnam[0], MODEANY);
#endif

	/* undistinguished names will fail with EEXIST */

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		(void) tbmkdir(*p, MODEANY, SUCCEED, EEXIST);
		/* the EEXIST is to stop tbmkdir removing an existing file */
	}

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		(void) rmdir(*p);
	}

	if (testfail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* mkdir creates dir with correct mode */

	DBUG_ENTER("test3");

	testfail = 0;

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t3()
{
	int		i, err;
	struct stat	statb;
	int		pathok = 0;

	DBUG_ENTER("ch_t3");

	(void) umask(UMASKVAL);

#ifdef TESTBED2
	(void) umask((mode_t)0);
#endif

	for (i = 0; i < NMODES; i++)
	{
		globok = 0;
		if (tbmkdir(t3_file, modes[i], SUCCEED, NOERROR) != SUCCEED)
		{
			PATH_TRACE; PATH_TRACE;
			continue;
		}
		else
			PATH_FUNC_TRACE;

		if (stat(t3_file, &statb) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("stat(\"%s\") after mkdir(\"%s\", %#o) failed - errno %d (%s)",
			    t3_file, t3_file, (int)modes[i], err, errname(err));
		}
		else if ((statb.st_mode & MODEANY) != (~UMASKVAL & modes[i]))
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("mkdir(\"%s\", %#o) with umask = %#o set permissions incorrectly",
				t3_file, (int)modes[i], (int)UMASKVAL);
			in_rpt("expected: %#o, observed: %#o",
				(int)(~UMASKVAL & modes[i]),
				(int)(statb.st_mode & MODEANY));
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2*NMODES);
	
	(void) rmdir(t3_file);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test4()
{
	DBUG_ENTER("test4");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */


private void
test4()
{
	/* mkdir creates dir with correct owner */

	DBUG_ENTER("test4");

	(void) cppair(ch_t4, NULLFN, WAITTIME, 0);

	(void) rmdir(t4_file);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	err;
	char	*args[6];

	DBUG_ENTER("ch_t4");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog4);
	args[1] = t4_file;
	args[2] = itoa(vguid1);
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(execpath(prog4), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog4), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test5()
{
	DBUG_ENTER("test5");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */


private void
test5()
{
	/* mkdir creates dir with correct group */

	DBUG_ENTER("test5");

	(void) cppair(ch_t5, NULLFN, WAITTIME, 0);

	(void) rmdir(t5_file);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	err;
	char	*args[6];

	DBUG_ENTER("ch_t5");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog5);
	args[1] = t5_file;
	args[2] = itoa(vggid1);
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(execpath(prog5), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog5), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test6()	
{
	/* new directory is empty */

	struct dirent	*dp;
	DIR		*dir;
	int		err;
	int		i;
	int		pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	globok = 0;
	if (tbmkdir(t6_file, MODEANY, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	(void) mkdir("mkdir-t.6/junkfile", MODEANY);
#endif

	if ((dir = opendir(t6_file)) == NULL)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("opendir(\"%s\") after mkdir(\"%s\", %#o) failed - errno %d (%s)",
			t6_file, t6_file, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	i = 0;
	while ((dp = readdir(dir)) != NULL)
	{
		if (i++ == 0)
			PATH_TRACE;
		if (strcmp(dp->d_name, ".") != 0 &&
		    strcmp(dp->d_name, "..") != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("mkdir(\"%s\", %#o) created directory containing entry: \"%s\"",
				t6_file, (int)MODEANY, dp->d_name);
		}
	}
	(void) closedir(dir);

	if (testfail == 0)
		PATH_XS_RPT(3);

#ifdef TESTBED2
	(void) rmdir("mkdir-t.6/junkfile");
#endif
	
	(void) rmdir(t6_file);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* mkdir() alters timestamps on new dir. and parent dir. */

	char	*chkt;
	int	err, fail = 0;
	time_t	t0;
	struct stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T7_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else /* TEST_XNFS */
	(void) time(&t0);
	PATH_TRACE;
#endif /* TEST_XNFS */

	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

	globok = 0;
	if (tbmkdir(t7_file, MODEANY, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	(void) sleep(1);
	(void) time(&t0);

	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;
#endif
	
	if (stat(t7_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\") after mkdir(\"%s\", %#o) failed - errno %d (%s)",
		    t7_file, t7_file, (int)MODEANY, err, errname(err));
	}
	else
	{
		if (stbuf.st_atime < t0)
			fail |= 01;
		else
			PATH_TRACE;
		if (stbuf.st_ctime < t0)
			fail |= 02;
		else
			PATH_TRACE;
		if (stbuf.st_mtime < t0)
			fail |= 04;
		else
			PATH_TRACE;
		
		if (fail != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			if (fail & 01)
				in_rpt("st_atime field on created directory was not updated");
			if (fail & 02)
				in_rpt("st_ctime field on created directory was not updated");
			if (fail & 04)
				in_rpt("st_mtime field on created directory was not updated");
		}
	}

	globok = 0;
	chkt = chktimes(".", NEWCTIME|NEWMTIME);
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("parent directory %s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
#ifdef TESTBED2
		PATH_XS_RPT(8);
#else
		PATH_XS_RPT(7);
#endif

	(void) rmdir(t7_file);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* write access to parent and search access to path prefix granted */

	int waittime;

	DBUG_ENTER("test8");

	waittime = WAITTIME * (5 + sysconf(_SC_NGROUPS_MAX)/4);

	(void) cppair(ch_t8, NULLFN, waittime, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t8");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test9()
{
	int	pathok = 0;
#if	   TEST_XNFS
	int	rc;
#endif	/* TEST_XNFS */

	/* creat dir. in directory which denies searching */

	DBUG_ENTER("test9");

	testfail = 0;

	if (crdir(t9_dir,
		(mode_t)(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH),
		(int) getuid(), (int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not make directory %s", t9_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbmkdir(t9_file, MODEANY, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) rmdir(t9_dir);

#if	   TEST_XNFS

	if (tet_sync(SP_T9_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t9, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
	{
		testfail++;
	}
	else if (rc & PATH_OK)
	{
		PATH_TRACE;
	}

	if (tet_sync(SP_T9_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS
private void
ch_t9()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t9");

	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t9_2_file;
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

#endif	/* TEST_XNFS */


private void
test10()
{
	int	pathok = 0;
#if	   TEST_XNFS
	int	rc;
#endif	/* TEST_XNFS */

	/* parent directory denies write permission */

	mode_t mode = (mode_t)(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);

	DBUG_ENTER("test10");

	testfail = 0;

	if (crdir(t10_dir, mode, (int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not make directory %s", t10_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbmkdir(t10_file, MODEANY, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) rmdir(t10_dir);

#if	   TEST_XNFS

	if (tet_sync(SP_T10_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t10, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
	{
		testfail++;
	}
	else if (rc & PATH_OK)
	{
		PATH_TRACE;
	}

	if (tet_sync(SP_T10_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS
private void
ch_t10()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t10");

	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t10_2_file;
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

#endif	/* TEST_XNFS */



private void
test11()
{
	/* file already exists */

	int 	fd;
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	/* existing plain file */

	if ((fd = crfile(t11_file, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbmkdir(t11_file, MODEANY, SYSERROR, EEXIST);
	PATH_FUNC_TRACE;

	/* existing directory */

	globok = 0;
	if (tbmkdir(t11_file, MODEANY, SUCCEED, NOERROR) == SUCCEED)
	{
		PATH_FUNC_TRACE;

		globok = 0;
		(void) tbmkdir(t11_file, MODEANY, SYSERROR, EEXIST);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) remove(t11_file);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	/*  LINK_MAX (+1) links to parent dir */

	char	*links[PCTS_LINK_MAX], *nambuf;
	long	i, link_max, testmax = PCTS_LINK_MAX;
	int	err;
	struct stat stbuf;
	int	pathok = 0;
#if	   TEST_XNFS
	int	nvals;
	char	**strvals;
#endif	/* TEST_XNFS */


	DBUG_ENTER("test12");

	testfail = 0;


#if	   TEST_XNFS

	/* Get the value of LINK_MAX from the server */
	nvals = 0;
	strvals = xnfs_getsv_c(&nvals, SP_T12_PLM_TRANSFER);
	if (strvals == NULL || nvals != 1)
	{
		xx_rpt(DELETION);
		in_rpt("xnfs_getsv_c() failed when getting _PC_LINK_MAX from server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	link_max = atol(strvals[0]);

	if(link_max <= 0)
	{
		if (link_max == 0)
		{
			link_max = PCTS_LINK_MAX+1;
			PATH_TRACE;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("pathconf(\".\", _PC_LINK_MAX) failed on server");
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;

	errno = 0;
	link_max = pathconf(".", _PC_LINK_MAX);
	if (link_max < 0)
	{
		if (errno == 0)
		{
			link_max = PCTS_LINK_MAX+1;
			PATH_TRACE;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("pathconf(\".\", _PC_LINK_MAX) failed");
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

#endif	/* TEST_XNFS */

	if (link_max < PCTS_LINK_MAX)
		testmax = link_max;

	nambuf = (char *) malloc((size_t)(testmax * NAMELEN));
	if (nambuf == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: malloc() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(".", &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\".\") failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create lesser of LINK_MAX and PCTS_LINK_MAX links successfully */

	for (i = 0; i < testmax - stbuf.st_nlink; i++)
	{
		if (i==0)
			PATH_TRACE;
		links[i] = &nambuf[i*NAMELEN];
		(void) sprintf(links[i], "L%ld", i);
		if (tbmkdir(links[i], MODEANY, SUCCEED, NOERROR) != SUCCEED)
			break;
	}
	links[i] = NULL;

#ifdef TESTBED2
	(void) rmdir(links[0]);
#endif

	/* if LINK_MAX is testable, next link gives EMLINK */

	if (i == testmax - stbuf.st_nlink && link_max <= PCTS_LINK_MAX)
	{
		if (stat(".", &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("stat(\".\") failed - errno %d (%s)", err, errname(err));
		}
		else if (stbuf.st_nlink != testmax)
		{
			/* EMLINK test does not apply */
			DBUG_PRINTF("test12", "nlink = %d, testmax = %ld",
				(int)stbuf.st_nlink, testmax);
			PATH_TRACE;
		}
		else
		{
			globok = 0;
			(void) tbmkdir(t12_file, MODEANY, SYSERROR, EMLINK);
			PATH_FUNC_TRACE;
		}
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) rmdir(t12_file);
	for (i = 0; i < testmax-1 && links[i] != NULL; i++)
		(void) rmdir(links[i]);
	free((void *)nambuf);

	DBUG_VOID_RETURN;
}



#if	   TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test13()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbmkdir(longname[0], MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbmkdir(longname[1], MODEANY, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) rmdir(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test14()
{
	/* path longer than PATH_MAX */

	char	**longname;
	int	pathok = 0;

	DBUG_ENTER("test14");

	testfail = 0;

	globok = 0;
	longname = pathtoolng(t14_file);

	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbmkdir(longname[0], MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbmkdir(longname[1], MODEANY, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) rmdir(t14_file);
	clntoolng();

	DBUG_VOID_RETURN;
}

private void
test15()
{
	/* component of path prefix does not exist */

	int	pathok = 0;

	DBUG_ENTER("test15");

	testfail = 0;

	if (eaccess(t15_dir, F_OK) == 0 && remove(t15_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not remove \"%s\"", t15_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbmkdir(t15_file, MODEANY, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	/* mkdir "" */

	int	pathok = 0;

	DBUG_ENTER("test16");

	testfail = 0;

	globok = 0;
	(void) tbmkdir("", MODEANY, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test17()
{
	/* component of path prefix not a directory */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test17");

	testfail = 0;

	if ((fd = crfile(t17_1_file, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t17_1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbmkdir(t17_2_file, MODEANY, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t17_1_file);

	DBUG_VOID_RETURN;
}


private void
test18()
{
	DBUG_ENTER("test18");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t18, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest18();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t18()
{
	DBUG_ENTER("ch_t18");

	subtest18();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */



private void
subtest18()
{
	/* mkdir on read-only file system */

	DBUG_ENTER("subtest18");

	(void) cppair(ch2_t18, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t18_dir);

	DBUG_VOID_RETURN;
}

private void
ch2_t18()
{
	char	*rofs;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch2_t18");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_ROFS is not defined");
		DBUG_VOID_RETURN;
	}

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t18_dir;
	args[3] = t18_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test19()
{

	DBUG_ENTER("test19");

	(void) cppair(ch_t19, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t19()
{
	int 	err;
	char	*args[8];

	DBUG_ENTER("ch_t19");

	testfail = 0;

	args[0] = execpath(prog19);
	args[1] = t19_dir;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;

	(void) tet_exec(execpath(prog19), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test20()
{
	DBUG_ENTER("test20");

	xnfs_doroot_c(ch_t20, WAITTIME, SP_NOSYNC, SP_NOSYNC);

	DBUG_VOID_RETURN;
}


private void
ch_t20()
{
	int	err;
	char	*args[6];

	DBUG_ENTER("ch_t20");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_uid);
	args[1] = t20_dir;
	args[2] = "0";			/* root */
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(execpath(prog_uid), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_uid), err, errname(err));


	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test20()
{
	DBUG_ENTER("test20");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test21()
{
	DBUG_ENTER("test21");

	(void) tet_fork(ch_t21, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t21()
{
	int	err;
	char	*args[6];
	char	*xnfs_anon_buf;

	DBUG_ENTER("ch_t21");

	if ((xnfs_anon_buf = newstr(uidtoa(xnfs_anon_uid))) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("Failed to allocate buffer for xnfs_anon_uid");
		DBUG_VOID_RETURN;
	}

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_uid);
	args[1] = t21_dir;
	args[2] = xnfs_anon_buf;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(execpath(prog_uid), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_uid), err, errname(err));


	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test21()
{
	DBUG_ENTER("test21");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if TEST_XNFS

private void
test22()
{
	/*
	 *  creates file with correct group when GrpID=True
	 *  replaces part of test5 when testing XNFS
	 */

	DBUG_ENTER("test22");

	(void) cppair(ch_t22, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t22_mnt);

	DBUG_VOID_RETURN;
}

private void
ch_t22()
{
	char	*fsys, *sgid, *thistest_buf;
	int	err;
	char	*args[6];

	DBUG_ENTER("ch_t22");

	if ((thistest_buf = newstr(itoa(tet_thistest))) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("Failed to allocate buffer for tet_thistest");
		DBUG_VOID_RETURN;
	}

	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("Failed to allocate buffer for vggid1");
		DBUG_VOID_RETURN;
	}

	if ((fsys = tet_getvar("VSX_MOUNT_DEV")) == NULL || *fsys == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: VSX_MOUNT_DEV is not defined");
		DBUG_VOID_RETURN;
	}

	if (crdir(t22_mnt, MODEANY, (int) getuid(),
			(int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t22_mnt);
		DBUG_VOID_RETURN;
	}

	args[0] = execpath(prog_gid);
	args[1] = thistest_buf;
	args[2] = sgid;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(execpath(prog_gid), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_gid), err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test22()
{
	DBUG_ENTER("test22");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test23()
{
	/*
	 *  creates file with correct group when GrpID=False
	 *  replaces part of test5 when testing XNFS
	 */

	DBUG_ENTER("test23");

	(void) cppair(ch_t23, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t23()
{
	char	*sgid, *thistest_buf;
	int	err;
	char	*args[6];

	DBUG_ENTER("ch_t23");

	if ((thistest_buf = newstr(itoa(tet_thistest))) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("Failed to allocate buffer for tet_thistest");
		DBUG_VOID_RETURN;
	}

	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("Failed to allocate buffer for vggid1");
		DBUG_VOID_RETURN;
	}

	args[0] = execpath(prog_gid);
	args[1] = thistest_buf;
	args[2] = sgid;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(execpath(prog_gid), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_gid), err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test23()
{
	DBUG_ENTER("test23");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test24()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test24");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbmkdir(longname, MODEANY, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test24()
{
	DBUG_ENTER("test24");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test25()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test25");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbmkdir(longname, MODEANY, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test25()
{
	DBUG_ENTER("test25");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test26()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test26");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbmkdir(longname, MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test26()
{
	DBUG_ENTER("test26");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test27()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test27");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbmkdir(longname, MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test27()
{
	DBUG_ENTER("test27");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

