/*
 *      SCCS:  @(#)  POSIX.os/files/mkdir/mkdir_gid.c Rel 4.4.1 (06/09/97)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#define MAIN
static	char sccsid[] = "@(#)POSIX.os/files/mkdir/mkdir_gid.c Rel 4.4.1 (06/09/97)";
#ifdef	MAIN
char	*copyright[] = {
		"(C) Copyright 1992 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		mkdir_gid - used by T.mkdir where different egid needed

PROJECT:	XNFS (X/OPEN NFS tests)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int tet_main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	Set-gid subprogram for mkdir tests.
	Must be set-gid to vsx1 and have appropriate privileges to mount
	and unmount a file system.

	usage:
		progname mygid vg_ident vg_dbug

	where mygid is my effective gid

AUTHOR:		David Sawyer, UniSoft Ltd.
DATE CREATED:	4 November 1992

MODIFICATIONS:
		Geoff Clare, 20 Oct 1993
			Don't create subdirectory, restructure code,
			set UID to vsx0 to avoid group ID mapping.

************************************************************************/

#include <std.h>
#include <dbug.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include <xnfs.h>
#include "mkdir_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef mkdir
#endif


#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	void	set_pname();
extern	char	*errname();
extern	int	vguid0;

private char	*t22_mnt = 	T22_MNT;
private char	*t22_file = 	T22_FILE;
private char	*t23_file = 	T23_FILE;
private gid_t	mygid;

private void	subtest22();
private void	subtest23();
private void	subtest22_23();
private void	ch_t22();


public	int
tet_main(argc, argv)
int argc;
char **argv;
{
	gid_t 	mygid;
	int	testnum;

	set_pname(argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 4)
		vg_dbug = argv[4];
	if (argc >= 3)
		vg_ident = argv[3];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug + 2);
	}

	if ((mygid = atoi(argv[2])) != getegid())
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: expected effective gid %ld, found %ld",
			(long)mygid, (long)getegid());
		return(OK);
	}
	else if (uids() != 0)
	{
		xx_rpt(DELETION);
		in_rpt("uids() failed");
		return(OK);
	}

	testnum = atoi(argv[1]);
	switch (testnum)
	{
	case 22:
		subtest22(mygid);
		break;

	case 23:
		subtest23(mygid);
		break;

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed: %d", testnum);
		return(OK);
	}

	return(OK);
}


private void
subtest22(gid)
gid_t gid;
{
	int pathok = 0;
	char *fsys;

	DBUG_ENTER("subtest22");

	mygid = gid;

	if ((fsys = tet_getvar("VSX_MOUNT_DEV")) == NULL || *fsys == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: VSX_MOUNT_DEV not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (xnfs_mount(fsys, t22_mnt, XNFS_MOUNT_GRPID | XNFS_MOUNT_INTR) < 0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);
	(void) tet_fork(ch_t22, NULLFN, 2*WAITTIME, 0);

	(void) xnfs_unmount(fsys, t22_mnt);

	DBUG_VOID_RETURN;
}

private void
ch_t22()
{
	int pathok = 0;

	DBUG_ENTER("ch_t22");

	PATH_FUNC_TRACE;
	subtest22_23(t22_mnt, t22_file, mygid, (gid_t) -1, pathok, 1);

	DBUG_VOID_RETURN;
}

private void
subtest23(mygid)
gid_t mygid;
{
	gid_t altgid;

	DBUG_ENTER("subtest23");

#if TEST_FIPS || TEST_XPG > 3
	altgid = (gid_t) -1;
#else
	altgid = mygid;
#endif

	subtest22_23(".", t23_file, mygid, altgid, 0, 0);

	DBUG_VOID_RETURN;
}



/*
**	subtest22_23() - perform the GID test after the test-specific
**		setup has been done
**
**	dir is the parent directory of file
**	file is the name of the file to create
**	mygid is my effective gid
**	altgid is:
**		mygid if the file's gid may be either the dir's gid or mygid
**		or -1 if the file's gid must be taken from the dir's gid
**	pathok is the current pathok value
**	pathcount is the number of PATH_CHECKs that the setup function
**		should have done
*/

private void
subtest22_23(dir, file, mygid, altgid, pathok, pathcount)
char *dir, *file;
gid_t mygid, altgid;
int pathok, pathcount;
{
	gid_t dirgid;
	int err;
	struct stat stbuf;

	DBUG_ENTER("subtest22_23");

	(void) setuid((uid_t)vguid0);
	if (geteuid() != vguid0)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective uid to vsx0");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Tidy up any existing file */
	(void) rmdir(file);

	if (stat(dir, &stbuf) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(%s) failed", dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((dirgid = stbuf.st_gid) == mygid)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: directory gid %ld must not be the same as my gid", (long)dirgid);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (mkdir(file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkdir(%s) failed, errno = %d (%s)",
			file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(file, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: stat(%s) failed, errno = %d (%s)",
			file, err, errname(err));
		(void) rmdir(file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf.st_gid != dirgid &&
			(altgid == (gid_t) -1 || stbuf.st_gid != altgid))
	{
		xx_rpt(FAILURE);
		if (altgid == (gid_t) -1)
		{
			in_rpt("gid of created file: expected %ld, found %ld",
				(long)dirgid, (long)stbuf.st_gid);
		}
		else
		{
			in_rpt("gid of created file: expected %ld or %ld, found %ld",
				(long)dirgid, (long)altgid, (long)stbuf.st_gid);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(pathcount + 6);
	}

	(void) rmdir(file);

	DBUG_VOID_RETURN;
}
