/*
*      SCCS:  @(#)  POSIX.os/files/mkdir/mkdir_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/mkdir - mkdir_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		mkdir_su - used by T.mkdir in privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.mkdir to perform privileged tests.

	  usage:
		mkdir_su testnum dir file ident dbug

	  where testnum is the number of the test being executed, dir and
	  file parameters to be passed to the test.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	10 Apr 1989
MODIFICATIONS:
		Geoff Clare, 9 June 1989:  Add do_access() test.
		Geoff Clare, 15 Nov 1989
			New setuprofs() argument
			New do_access() argument

		David Sawyer
		Thu Aug 02 09:29:41 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Neil Moses, 8 June 1992
			Check that time stamps are not changed.

		David Sawyer, 5 November 1992
			Modifications for XNFS tests.

		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <dbug.h>
#include <errno.h>
#include <sys/stat.h>
#include <report.h>
#include <tet_api.h>
#include <xnfs.h>
#include "mkdir_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef mkdir
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

#if	   TEST_XNFS
#define NOK		02
#define PATH_OK		04
#endif	/* TEST_XNFS */

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
public int	testfail;

extern	char	**environ;
extern	void	savtimes();
extern	char	*chktimes();
#if	   TEST_XNFS
extern	void	valrpt();
#endif	/* TEST_XNFS */

extern	void	set_pname();
extern	void	cln_access();

private	void	subtest8();
#if	   TEST_XNFS
private	int	subtest9_10();
#endif	/* TEST_XNFS */
private	void	subtest18();
private	int	t8_func();



public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int testnum;
	int exitval = OK;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );

		return(exitval);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testnum = atoi(argv[1]);
	switch (testnum)
	{
	case 8:
		subtest8();
		break;

#if	   TEST_XNFS

	case 9:
	case 10:
		exitval = subtest9_10(argv[2]);
		break;

#endif	/* TEST_XNFS */

	case 18:
		subtest18(argv[2], argv[3]);
		break;

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed: %d", testnum);

		return(exitval);
	}

	return(exitval);
	/* NOTREACHED */
}

private void
subtest8()
{
	/* write & search access to dir, search access to path prefix */

	int	pathok = 0;

	DBUG_ENTER("subtest8");

	globok = 0;
	if (do_access('d', (mode_t) (S_IWUSR|S_IXUSR), (mode_t) S_IXUSR, t8_func, WAITTIME, SP_T8_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(1);
	}

	cln_access();

	DBUG_VOID_RETURN;
}

private int
t8_func(dir, desc)
char *dir, **desc;
{
	char 	file[50];
	int	pathok = 0;

	DBUG_ENTER("t8_func");

	(void) strcat(strcpy(file, dir), "/mkdir-t.8");

	testfail = 0;

	globok = 0;
	(void) tbmkdir(file, MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail != 0)
	{
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	(void) rmdir(file);
	DBUG_RETURN(testfail);
}


#if	   TEST_XNFS
private int
subtest9_10(file)
char *file;
{
	int err, ret, rc;
	int pathok = 0;

	DBUG_ENTER("subtest9_10");

	ret = mkdir(file, MODEANY);
	err = errno;

	if (ret != SYSERROR || err != EACCES)
	{
		xx_rpt(FAILURE);
		valrpt(SYSERROR, ret, EACCES, err);
		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;

	rc = OK;
	if (pathok == 1)
		rc |= PATH_OK;

	DBUG_RETURN(rc);
}
#endif	/* TEST_XNFS */


private void
subtest18(mdir, file)
char	*mdir, *file;
{
	/* mkdir on read-only file system */

	char	*errptr;
	int	pathok = 0;

	DBUG_ENTER("subtest18");

	testfail = 0;

	globok = 0;
	if (setuprofs(mdir, (char *)NULL, 'f', (mode_t) 0) != 0)
	{
		DBUG_VOID_RETURN;	/* reported in setuprofs() */
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(mdir);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbmkdir(file, MODEANY, SYSERROR, EROFS);
	PATH_FUNC_TRACE;

	globok = 0;
	errptr = chktimes(mdir, 0);
	PATH_FUNC_TRACE;

	if(errptr != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful mkdir() changed time stamps on %s",
			mdir);
		in_rpt("%s", errptr);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	(void) clnuprofs(mdir, (char *)NULL);

	DBUG_VOID_RETURN;
}
