/*
*      SCCS:  @(#)  POSIX.os/files/mkdir/mkdir_t4.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/mkdir - mkdir_t4 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		mkdir_t4 - used by T.mkdir in test4()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.mkdir in test4().  This should be setuid to VSX_UID1.
	  usage:
		mkdir_t4 file uid vg_ident vg_dbug

	  where uid is VSX_UID1 and file is the directory to be
	  created with mkdir().
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	7 Apr 1989
MODIFICATIONS:

		David Sawyer
		Thu Aug 02 09:29:54 BST 1990
		Added resilience code.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <dbug.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef mkdir
#endif


#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
public	int	testfail;

extern	char	*errname();
extern	void	set_pname();

private void	subtest4();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	uid_t	uid;

	set_pname(argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 5)
		vg_dbug = argv[4];
	if (argc >= 4)
		vg_ident = argv[3];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if ((uid = atoi(argv[2])) != geteuid())
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: uid is not equal to VSX_UID1");
	}
	else
		subtest4(argv[1], uid);

	return(OK);
	/*NOTREACHED*/
}


private void
subtest4(file, uid)
char	*file;
uid_t	uid;
{
	int 	err;
	struct stat statb;
	int	pathok = 0;

	DBUG_ENTER("subtest4");

	testfail = 0;

	globok = 0;
	if (tbmkdir(file, MODEANY, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	uid++;
#endif

	if (stat(file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("stat(\"%s\") after mkdir(\"%s\", %#o) failed - errno %d (%s)",
		    file, file, (int)MODEANY, err, errname(err));
	}
	else if (statb.st_uid != uid)
	{
		xx_rpt(FAILURE);
		in_rpt("mkdir(\"%s\", %#o) set owner to %d, expected %d",
			file, (int)MODEANY, (int)statb.st_uid, (int)uid);
		in_rpt("process effective uid: %d, real uid: %d",
			(int)geteuid(), (int)getuid());
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}
