/*
*      SCCS:  @(#)  POSIX.os/files/mkdir/mkdir_uid.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1992 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/mkdir - mkdir_t4 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		mkdir_uid - used by T.mkdir where different euid needed

PROJECT:	XNFS (X/OPEN NFS tests)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Set-uid subprogram for mkdir tests.
	  Must be set-uid to XNFS_MAPPED_UID.
	  usage:
		mkdir_uid file uid vg_ident vg_dbug

	  where uid is the user id that the file is expected to be 
	  created with and file is the directory to be created with mkdir().

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		David Sawyer, UniSoft Ltd.
DATE CREATED:	3 November 1992
MODIFICATIONS:
		Geoff Clare, 8 Jan 1993
			Change atoi() to atouid().

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <dbug.h>
#include <report.h>
#include <errno.h>
#include <xnfs.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef mkdir
#endif


#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
public	int	testfail;

extern	uid_t	atouid();
extern	char	*errname();
extern	void	set_pname();

private void	subtest20_21();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	uid_t	uid;

	set_pname(argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return 0;
	}
	if (argc >= 5)
		vg_dbug = argv[4];
	if (argc >= 4)
		vg_ident = argv[3];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if (geteuid() != XNFS_MAPPED_UID)
	{
		xx_rpt(DELETION);
		in_rpt("Effective UID is not %d", XNFS_MAPPED_UID);
		return 0;
	}

	uid = atouid(argv[2]);

	subtest20_21(argv[1], uid);

	/* remove the directory we created */
	(void) rmdir(argv[1]);

	return 0;
}


private void
subtest20_21(file, uid)
char	*file;
uid_t	uid;
{
	int 	err;
	struct stat statb;
	int	pathok = 0;

	DBUG_ENTER("subtest20_21");

	testfail = 0;

	globok = 0;
	if (tbmkdir(file, MODEANY, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	uid++;
#endif

	if (stat(file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("stat(\"%s\") after mkdir(\"%s\", %#o) failed - errno %d (%s)",
		    file, file, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statb.st_uid != uid)
	{
		xx_rpt(FAILURE);
		in_rpt("mkdir(\"%s\", %#o) set owner to %d, expected %d",
			file, (int)MODEANY, (int)statb.st_uid, (int)uid);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}
