/*
*      SCCS:  @(#)  POSIX.os/files/mkfifo/mkfifo.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/mkfifo - T.mkfifo Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.mkfifo

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the mkfifo() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	7 Apr 1989
MODIFICATIONS:
		Geoff Clare, 9 June 1989:  Add do_access() test.

		Steve Ratcliffe, 13 Mar 1990
			Add ENOSPC test.

		Geoff Clare, 5 Apr 1990
			New interface with pathtoolng() and nametoolng().

		David Sawyer
		Wed Aug 01 15:12:04 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		John-Paul, 29 July 1992
			Mode parameter in calls to crfile() and crdir()
			changed from 'int' to 'mode_t' type.

		Geoff Clare, 3 Nov 1995
			Changed child timeout for do_access() test.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<pathres.h>
#include	<chktimes.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef mkfifo
#endif



#define NO_TESTS	17
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define UMASKVAL	(mode_t)(S_IRUSR|S_IWGRP|S_IXOTH)
#define NMODES		3

extern	char	*errname();
extern	char	*itoa();
extern	char	**pathtoolng();
extern	char	**nametoolng();
extern	void	clntoolng();

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;
extern	int	vguid1;
extern	int	vggid1;

public	int	no_tests = NO_TESTS;
public	int	testfail;

private mode_t	modes[NMODES] = { S_IRWXU|S_IRWXG|S_IRWXO,
				  S_IXUSR|S_IWGRP|S_IROTH,
				  UMASKVAL
};

private char	*prog4 =	"../mkfifo_t4";
private char	*prog5 =	"../mkfifo_t5";
private char	*prog17 =	"../mkfifo_t17";
private char	*prog_su =	"../mkfifo_su";
private char	*t3_file =	"mkfifo-t.3";
private char	*t4_file =	"mkfifo-t.4";
private char	*t5_file =	"mkfifo-t.5";
private char	*t6_file =	"mkfifo-t.6";
private char	*t8_dir =	"mkfifo-d.8";
private char	*t8_file =	"mkfifo-d.8/mkfifo-t.8";
private char	*t9_dir =	"mkfifo-d.9";
private char	*t9_file =	"mkfifo-d.9/mkfifo-t.9";
private char	*t10_file =	"mkfifo-t.10";
private char	*t12_file =	"mkfifo-t.12";
private char	*t13_dir =	"mkfifo-d.13";
private char	*t13_file =	"mkfifo-d.13/mkfifo-t.13";
private char	*t15_1_file =	"mkfifo-t.15";
private char	*t15_2_file =	"mkfifo-t.15/mkfifo-t.15.1";
private char	*t16_dir =	"mkfifo-d.16";
private char	*t16_file =	"mkfifo-d.16/mkfifo-t.16";
private char	*t17_dir =	"mkfifo-d.17";


private void	prepare_tests();
private void	clean_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();

private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	ch_t7();
private void	ch_t16();
private void	ch_t17();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	  test9, 9,
	  test10, 10,
	  test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	NULL, 0
};

private	void
prepare_tests()
{
	DBUG_ENTER("prepare_tests");

	if(uids()!=0)
	{
		int	i;
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	(void) umask((mode_t)0);

	DBUG_VOID_RETURN;
}

private	void
clean_tests()
{
	char **p;

	DBUG_ENTER("clean_tests");

	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);
	(void) unlink(t3_file);
	(void) unlink(t4_file);
	(void) unlink(t5_file);
	(void) unlink(t6_file);
	(void) rmdir(t8_dir);
	(void) rmdir(t9_dir);
	(void) unlink(t10_file);
	(void) unlink(t12_file);
	(void) unlink(t15_1_file);
	(void) rmdir(t16_dir);
	(void) rmdir(t17_dir);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* pathname resolution */

	char 	*p1, *p2;
	int 	npairs, i;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	npairs = init_pathres(PRES_FILES);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_FILES) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		/* creat using complex pathname */
		globok = 0;
		if (tbmkfifo(p1, MODEANY, SUCCEED, NOERROR) != SUCCEED)
			continue;
		else
			PATH_FUNC_TRACE;

#ifdef TESTBED2
		(void) unlink(p2);
#endif

		/* check for EEXIST using normal pathname */
		globok = 0;
		(void) tbmkfifo(p2, MODEANY, SYSERROR, EEXIST);
		PATH_FUNC_TRACE;

		(void) unlink(p2);
	}

	if (testfail == 0)
		PATH_XS_RPT(1+(2*npairs));
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames */

	char	**p;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		if (eaccess(*p, F_OK) == 0 && remove(*p) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not remove \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
	}

#ifdef TESTBED2
	(void) mkfifo(portfnam[0], MODEANY);
#endif

	/* undistinguished names will fail with EEXIST */

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		(void) tbmkfifo(*p, MODEANY, SUCCEED, EEXIST);
		/* the EEXIST is to stop tbmkfifo removing an existing file */
	}

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		(void) unlink(*p);
	}

	if (testfail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* mkfifo creates FIFO with correct mode */

	DBUG_ENTER("test3");

	testfail = 0;

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch_t3()
{
	int		i, err;
	struct stat	statb;
	int		pathok = 0;

	DBUG_ENTER("ch_t3");

	(void) umask(UMASKVAL);

#ifdef TESTBED2
	(void) umask((mode_t)0);
#endif

	for (i = 0; i < NMODES; i++)
	{
		globok = 0;
		if (tbmkfifo(t3_file, modes[i], SUCCEED, NOERROR) != SUCCEED)
		{
			continue;
		}
		else
			PATH_FUNC_TRACE;

		if (stat(t3_file, &statb) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("stat(\"%s\") after mkfifo(\"%s\", %#o) failed - errno %d (%s)",
			    t3_file, t3_file, (int)modes[i], err, errname(err));
		}
		else if ((statb.st_mode & MODEANY) != (~UMASKVAL & modes[i]))
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("mkfifo(\"%s\", %#o) with umask = %#o set permissions incorrectly",
				t3_file, (int)modes[i], (int)UMASKVAL);
			in_rpt("expected: %#o, observed: %#o",
				(int)(~UMASKVAL & modes[i]),
				(int)(statb.st_mode & MODEANY));
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2*NMODES);
	
	(void) unlink(t3_file);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* mkfifo creates FIFO with correct owner */

	DBUG_ENTER("test4");

	(void) cppair(ch_t4, NULLFN, WAITTIME, 0);

	(void) unlink(t4_file);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	err;
	char	*args[6];

	DBUG_ENTER("ch_t4");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog4;
	args[1] = t4_file;
	args[2] = itoa(vguid1);
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(prog4, args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog4, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* mkfifo creates FIFO with correct group */

	DBUG_ENTER("test5");

	(void) cppair(ch_t5, NULLFN, WAITTIME, 0);

	(void) unlink(t5_file);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	err;
	char	*args[6];

	DBUG_ENTER("ch_t5");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog5;
	args[1] = t5_file;
	args[2] = itoa(vggid1);
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;
	(void) tet_exec(prog5, args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog5, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* mkfifo() alters timestamps on new file and parent dir. */

	char	*chkt;
	int	err, fail = 0;
	time_t	t0;
	struct stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	(void) time(&t0);
	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

	globok = 0;
	if (tbmkfifo(t6_file, MODEANY, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	(void) sleep(1);
	(void) time(&t0);
	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;
#endif
	
	if (stat(t6_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\") after mkfifo(\"%s\", %#o) failed - errno %d (%s)",
		    t6_file, t6_file, (int)MODEANY, err, errname(err));
	}
	else
	{
		if (stbuf.st_atime < t0)
			fail |= 01;
		else
			PATH_TRACE;
		if (stbuf.st_ctime < t0)
			fail |= 02;
		else
			PATH_TRACE;
		if (stbuf.st_mtime < t0)
			fail |= 04;
		else
			PATH_TRACE;
		
		if (fail != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			if (fail & 01)
				in_rpt("st_atime field on created directory was not updated");
			if (fail & 02)
				in_rpt("st_ctime field on created directory was not updated");
			if (fail & 04)
				in_rpt("st_mtime field on created directory was not updated");
		}
	}

	globok = 0;
	chkt = chktimes(".", NEWCTIME|NEWMTIME);
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("parent directory %s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
#ifdef TESTBED2
		PATH_XS_RPT(7);
#else
		PATH_XS_RPT(6);
#endif

	(void) unlink(t6_file);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* write access on parent, search access on path prefix granted */

	int waittime;

	DBUG_ENTER("test7");

	waittime = WAITTIME * (5 + sysconf(_SC_NGROUPS_MAX)/4);

	(void) cppair(ch_t7, NULLFN, waittime, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t7");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(prog_su, args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog_su, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* make FIFO in directory which denies searching */
	int	pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

	if (crdir(t8_dir,
		(mode_t)(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH),
		(int) getuid(), (int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not make directory %s", t8_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbmkfifo(t8_file, MODEANY, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) rmdir(t8_dir);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	/* parent directory denies write permission */

	mode_t 	mode = (mode_t)(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	int	pathok = 0;

	DBUG_ENTER("test9");

	testfail = 0;

	if (crdir(t9_dir, mode, (int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not make directory %s", t9_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbmkfifo(t9_file, MODEANY, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) rmdir(t9_dir);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	/* file already exists */

	int 	fd;
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	/* existing plain file */

	if ((fd = crfile(t10_file, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbmkfifo(t10_file, MODEANY, SYSERROR, EEXIST);
	PATH_FUNC_TRACE;

	/* existing FIFO */

	globok = 0;
	if (tbmkfifo(t10_file, MODEANY, SUCCEED, NOERROR) == SUCCEED)
	{
		PATH_FUNC_TRACE;
		globok = 0;
		(void) tbmkfifo(t10_file, MODEANY, SYSERROR, EEXIST);
		PATH_FUNC_TRACE;
	}
	else
	{
		PATH_TRACE; PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) unlink(t10_file);

	DBUG_VOID_RETURN;
}

private void
test11()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbmkfifo(longname[0], MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbmkfifo(longname[1], MODEANY, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

private void
test12()
{
	/* path longer than PATH_MAX */

	char	**longname;
	int	pathok = 0;

	DBUG_ENTER("test12");

	testfail = 0;

	globok = 0;
	longname = pathtoolng(t12_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbmkfifo(longname[0], MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbmkfifo(longname[1], MODEANY, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(t12_file);
	clntoolng();

	DBUG_VOID_RETURN;
}

private void
test13()
{
	/* component of path prefix does not exist */

	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

	if (eaccess(t13_dir, F_OK) == 0 && remove(t13_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not remove \"%s\"", t13_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbmkfifo(t13_file, MODEANY, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test14()
{
	/* mkfifo "" */

	int	pathok = 0;

	DBUG_ENTER("test14");

	testfail = 0;

	globok = 0;
	(void) tbmkfifo("", MODEANY, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test15()
{
	/* component of path prefix not a directory */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test15");

	testfail = 0;

	if ((fd = crfile(t15_1_file, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t15_1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbmkfifo(t15_2_file, MODEANY, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t15_1_file);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	/* mkfifo on read-only file system */

	DBUG_ENTER("test16");

	(void) cppair(ch_t16, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t16_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t16()
{
	char	*rofs;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t16");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_ROFS is not defined");
		DBUG_VOID_RETURN;
	}

	if (crdir(t16_dir, MODEANY, (int) geteuid(),
			(int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t16_dir);
		DBUG_VOID_RETURN;
	}

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = t16_dir;
	args[3] = t16_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(prog_su, args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog_su, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test17()
{

	DBUG_ENTER("test17");

	(void) cppair(ch_t17, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t17()
{
	int 	err;
	char	*args[5];

	DBUG_ENTER("ch_t17");

	testfail = 0;

	args[0] = prog17;
	args[1] = t17_dir;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;

	(void) tet_exec(prog17, args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}
