/*
*      SCCS:  @(#)  POSIX.os/files/mkfifo/mkfifo_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/mkfifo - mkfifo_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		mkfifo_su - used by T.mkfifo in privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.mkfifo to perform privileged tests.

	  usage:
		mkfifo_su testnum dir file ident dbug

	  where testnum is the number of the test being executed, dir
	  and file are parameters to be passed to the test.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	10 Apr 1989
MODIFICATIONS:
		Geoff Clare, 9 June 1989:  Add do_access() test.

		Geoff Clare, 15 Nov 1989
			New setuprofs() argument
			New do_access() argument

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Neil Moses, 8 June 1992
			Check that time stamps are not changed.
		
		Geoff Clare, 16 June 1993
			Add dummy syncpoint argument to do_access() call.

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <dbug.h>
#include <errno.h>
#include <sys/stat.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef mkfifo
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
public int	testfail;

extern	char	**environ;
extern	void	savtimes();
extern	char	*chktimes();

extern	void	set_pname();
extern	void	cln_access();

private	void	subtest7();
private	void	subtest16();
private	int	t7_func();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int testnum;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testnum = atoi(argv[1]);
	switch (testnum)
	{
	case 7:
		subtest7();
		break;

	case 16:
		subtest16(argv[2], argv[3]);
		break;

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed to: %d",  testnum);
		return(OK);
	}

	return(OK);
}

private void
subtest7()
{
	/* write & search access to dir, search access to path prefix */

	int	pathok = 0;

	DBUG_ENTER("subtest7");

	globok = 0;
	if (do_access('d', S_IWUSR|S_IXUSR, S_IXUSR, t7_func, WAITTIME, 0) == 0)
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(1);
	}

	cln_access();

	DBUG_VOID_RETURN;
}

private int
t7_func(dir, desc)
char *dir, **desc;
{
	char 	file[50];
	int	pathok = 0;

	DBUG_ENTER("t7_func");

	(void) strcat(strcpy(file, dir), "/mkfifo-t.7");

	testfail = 0;

	globok = 0;
	(void) tbmkfifo(file, MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail != 0)
	{
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	(void) unlink(file);
	DBUG_RETURN(testfail);
}

private void
subtest16(mdir, file)
char	*mdir, *file;
{
	/* mkfifo on read-only file system */

	char	*errptr;
	int	pathok = 0;

	DBUG_ENTER("subtest16");

	testfail = 0;

	globok = 0;
	if (setuprofs(mdir, (char *)NULL, 'f', 0) != 0)
	{
		DBUG_VOID_RETURN;	/* reported in setuprofs() */
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(mdir);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbmkfifo(file, MODEANY, SYSERROR, EROFS);
	PATH_FUNC_TRACE;

	globok = 0;
	errptr = chktimes(mdir, 0);
	PATH_FUNC_TRACE;

	if(errptr != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful mkfifo() changed time stamps on %s",
			mdir);
		in_rpt("%s", errptr);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	(void) clnuprofs(mdir, (char *)NULL);

	DBUG_VOID_RETURN;
}
