/*
*      SCCS:  @(#)  POSIX.os/files/mkfifo/mkfifo_t5.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/mkfifo - mkfifo_t5 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		mkfifo_t5 - used by T.mkfifo in test5()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.mkfifo in test5().  This should be setgid to VSX_GID1.
	  usage:
		mkfifo_t5 file gid vg_ident vg_dbug

	  where gid is VSX_GID1 and file is the directory to be
	  created with mkfifo().
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	7 Apr 1989
MODIFICATIONS:
		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Geoff Clare, 15 Nov 1991
			Allow only parent dir group in FIPS mode

		Geoff Clare, 12 June 1992
			Test FIPS requirement in XPG4 mode.

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <dbug.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef mkfifo
#endif


#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
public	int	testfail;

extern	char	*errname();
extern	void	set_pname();

private void	subtest5();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	gid_t	gid;

	set_pname(argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		exit(OK);
	}
	if (argc >= 5)
		vg_dbug = argv[4];
	if (argc >= 4)
		vg_ident = argv[3];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if ((gid = atoi(argv[2])) != getegid())
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: gid is not equal to VSX_GID1");
	}
	else
		subtest5(argv[1], gid);

	exit(OK);
	/*NOTREACHED*/
}


private void
subtest5(file, gid)
char	*file;
gid_t	gid;
{
	int 	err;
	gid_t 	dirgid;
	struct stat statb;
	int	pathok = 0;

	DBUG_ENTER("subtest5");

	testfail = 0;

	if (stat(".", &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed on \".\" - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN
	}
	else
		PATH_TRACE;
	dirgid = statb.st_gid;

	globok = 0;
	if (tbmkfifo(file, MODEANY, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	gid++;
#endif

	if (stat(file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("stat(\"%s\") after mkfifo(\"%s\", %#o) failed - errno %d (%s)",
		    file, file, (int)MODEANY, err, errname(err));
	}
#if TEST_FIPS || TEST_XPG > 3
	else if (statb.st_gid != dirgid)
	{
		xx_rpt(FAILURE);
		in_rpt("mkfifo(\"%s\", %#o) set group to %d, expected %d",
			file, (int)MODEANY, (int)statb.st_gid, (int)dirgid);
		in_rpt("group of parent dir: %d, process effective gid: %d, real gid: %d",
			(int)dirgid, (int)getegid(), (int)getgid());
	}
#else
	else if (statb.st_gid != gid && statb.st_gid != dirgid)
	{
		xx_rpt(FAILURE);
		in_rpt("mkfifo(\"%s\", %#o) set group to %d, expected %d or %d",
			file, (int)MODEANY, (int)statb.st_gid,
			(int)gid, (int)dirgid);
		in_rpt("group of parent dir: %d, process effective gid: %d, real gid: %d",
			(int)dirgid, (int)getegid(), (int)getgid());
	}
#endif
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}
