/*
*      SCCS:  @(#)  POSIX.os/files/mkfifo/tbmkfifo.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		tbmkfifo.c

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
		tbmkfifo() checking function

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	10 Apr 1989
MODIFICATIONS:	
		Geoff Clare, 5 Apr 1990
			New interface with pathtoolng() and nametoolng().

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<string.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef mkfifo
#endif


extern	char	*errname();
extern	void	valrpt();

extern int	testfail;

public int
tbmkfifo(tbpath, tbmode, expret, experrno)
char *tbpath;
mode_t tbmode;
int expret, experrno;
{
	char	*reppath = tbpath;
	int	rval, err;
	char	pathbuf[30];
	struct stat stbuf;
	int	pathok = 0;

	DBUG_LENTER("tbmkfifo", 2);

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
		PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (experrno != EEXIST &&
		eaccess(tbpath, F_OK) == 0 && remove(tbpath) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("could not remove \"%s\" - errno %d (%s)",
			reppath, err, errname(err));
		return SYSERROR;
	}
	else
		PATH_TRACE;

	if (expret == SUCCEED)
		experrno = NOERROR;	/* may be EEXIST to suppress remove */

	errno = NOERROR;
	rval = mkfifo(tbpath, tbmode);

#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;

	if (expret == SUCCEED && rval == SUCCEED)
	{
#ifdef TESTBED2
#undef S_ISFIFO
#define S_ISFIFO(m) 0
#endif
		if (stat(tbpath, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("stat() failed with errno %d (%s) after successful mkfifo(\"%s\", %#o)",
			    err, errname(err), reppath, (int)tbmode);
		}
		else if (!S_ISFIFO(stbuf.st_mode))
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("S_ISFIFO(st_mode) was false for file created by mkfifo(\"%s\", %#o)",
				reppath, (int)tbmode);
		}
		else
			PATH_TRACE;
		PATH_FUNC_RPT(3);
		DBUG_RETURN(rval);
	}
	else if (expret == SYSERROR && rval == SYSERROR && experrno == err)
	{
		if (experrno != EEXIST && eaccess(tbpath, F_OK) == 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful mkfifo(\"%s\", %#o) created a file",
				reppath, (int)tbmode);
			(void) remove(tbpath);
		}
		else
			PATH_TRACE;
		PATH_FUNC_RPT(3);
		DBUG_RETURN(rval);
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("mkfifo(\"%s\", %#o) did not give correct results",
		reppath, (int)tbmode);
	valrpt(expret, rval, experrno, err);

	if (expret == SYSERROR && rval != SYSERROR)
		(void) rmdir(tbpath);
	DBUG_RETURN(rval);
}
