/*
 *      SCCS:  @(#)  POSIX.os/files/open/enospc.c	Rel 4.4.1     (06/09/97) 
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1991 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#ifndef NO_MAIN
static	char sccsid[] = "@(#)POSIX.os/files/open - enospc Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif
#endif  /* lint	*/

/***********************************************************************

NAME:	enospc.c

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()
		enospc()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char *argv[];

DESCRIPTION:
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
	One of:
		CREAT_TEST LINK_TEST MKDIR_TEST MKFIFO_TEST OPEN_TEST

	Depending on which test this is to be compiled for.

	NO_MAIN for when this file is to be linked directly into the main prog.

SIDE EFFECTS:
RELATED DOCUMENTS:
	A similar file is also used in the directories
		ANSI.os/streamio/fopen
		XPG2.os/files/mknod
SOURCE:
AUTHOR:		Steve Ratcliffe, UniSoft Ltd.
DATE CREATED:	28 Feb 1990
MODIFICATIONS:

		David Sawyer
		Fri Jul 27 16:58:38 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file
		
		Geoff Clare, 8 May 1991
			Remove unused code.
			Do tests in subdirectory to avoid extending
			root directory of VSX_NOSPC_DEV.

		Neil Moses, 22 May 1992
			Check that on error open() and creat() do not
			create a file and link() does not create a link
			nor change the link count of the existing file.
		
		Geoff Clare, 10 Feb 1993
			Change file existence check after successful call
			to depend on return value != -1 instead of errno 0.

************************************************************************/

#include <dbug.h>
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <report.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef open
#undef creat
#undef link
#undef mkdir
#undef mkfifo
#endif


#define	WAITTIME	(3 * SPEEDFACTOR)

#ifndef NO_MAIN
public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

#endif

extern	char	**environ;

extern	char	*itoa();
extern	char	*errname();
extern	void	set_pname();
extern	void	valrpt();

private int 	testfail;

void	enospc();

extern int 	setupnospc();
extern void	tidynospc();

#ifdef LINK_TEST
private	void	chk_link();
private	int	oldcount;
#endif

#define	BUFLEN	512

#define MODEANY	(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define MAXTRYS 2000	/* Number of times we try to fill up system */

#ifndef NO_MAIN

public	int
tet_main(argc, argv)
int	argc;
char	*argv[];
{

	set_pname(argv[0]);

	if (argc < 2 || argc > 4)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");

		return(OK);
	}

	if (argc >= 4)
		vg_dbug = argv[3];
	if (argc >= 3)
		vg_ident = argv[2];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}


	enospc(argv[1]);

	return(OK);
	/*NOTREACHED*/
}
#endif /* !NO_MAIN */

/*
 * enospc() tries to run out of space.
 */
void
enospc(base)
char	*base;
{
	char	*name;
#ifdef LINK_TEST
	struct	stat stbuf;
	char	*filler;
#endif
	int 	err;
	int 	i;
	int 	fd;
	int	pathok = 0;

	DBUG_ENTER("enospc");

	globok = 0;
	if (setupnospc(base))
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	name = (char*)malloc(strlen(base)+sizeof("/fill.dir/fill.tmp0000"));
	if (name == NULL) {
		xx_rpt(DELETION);
		in_rpt("Out of memory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef LINK_TEST
	filler = (char*)malloc(strlen(base)+sizeof("/filler"));
	if (filler == NULL) {
		xx_rpt(DELETION);
		in_rpt("Out of memory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) strcat(strcpy(filler, base), "/filler");
#else
	PATH_TRACE;
#endif

	err = 0;
	for (i = 4; i < MAXTRYS && !testfail && !err; i++) 
	{

		if (i==4)
			PATH_TRACE;

		/*
		 * Build up  temp file name and open it
		 */
		(void) strcat(strcpy(name, base), "/fill.dir/fill.tmp");
		(void) strcat(name, itoa(i));

		DBUG_PRINTF("enospc", "create %s", name);
#ifdef OPEN_TEST
		fd = open(name, O_RDWR|O_CREAT|O_EXCL, MODEANY);
#endif
#ifdef CREAT_TEST
		fd = creat(name, MODEANY);
#endif
#ifdef LINK_TEST
		fd = link(filler, name);
#endif
#ifdef MKDIR_TEST
		fd = mkdir(name, MODEANY);
#endif
#ifdef MKFIFO_TEST
		fd = mkfifo(name, MODEANY);
#endif

		if (fd == SYSERROR)
		{
			err = errno;

			switch (err) {
			case ENOSPC:
#if defined(OPEN_TEST) || defined(CREAT_TEST) || defined(LINK_TEST)
#ifdef OPEN_TEST
				if(eaccess(name, F_OK) != SYSERROR)
				{
					if(!testfail++)
						xx_rpt(FAILURE);
					in_rpt("open(%s, O_RDWR|O_CREAT|O_EXCL, MODEANY)",name);
					in_rpt("created file on error ENOSPC");
				}
				else
					PATH_TRACE;
#endif

#ifdef CREAT_TEST
				if(eaccess(name, F_OK) != SYSERROR)
				{
					if(!testfail++)
						xx_rpt(FAILURE);
					in_rpt("creat(%s, MODEANY)", name);
					in_rpt("created file on error ENOSPC");
				}
				else
					PATH_TRACE;
#endif

#ifdef LINK_TEST
				globok = 0;
				chk_link(filler, name);
				PATH_FUNC_TRACE;
#endif

#if defined(OPEN_TEST) || defined(CREAT_TEST)
				if(!testfail)
					PATH_XS_RPT(5);
#endif
#ifdef LINK_TEST
				if(!testfail)
					PATH_XS_RPT(6 + i - 5);
#endif
#else /* Other than OPEN_TEST || CREAT_TEST || LINK_TEST */
				if(!testfail)
					PATH_XS_RPT(4);
#endif
				break;
#ifdef OPEN_TEST
			case EEXIST:
				/*
				 * This means that the set up was wrong, or files were left
				 * around from other runs.
				 */
				testfail++;
				xx_rpt(DELETION);
				in_rpt("A File already existed");
				break;
#endif
#ifdef LINK_TEST
			case EMLINK:
				/*
				 * Could possibly link to something else if this were
				 * to happen.
				 */
				testfail++;
				xx_rpt(UNTESTED);
				in_rpt("No more links");
				break;
#endif
			default:
				testfail++;
				xx_rpt(FAILURE);
				valrpt(fd, fd, ENOSPC, err);
				break;
			}
		}

#if defined(OPEN_TEST) || defined(CREAT_TEST)
		(void) close(fd);
#endif
		/*
		 * Now check to see if the file actualy seems to be there ie
		 * that the system didn't just run out of space without saying so
		 */
		if (fd != SYSERROR && eaccess(name, F_OK) == SYSERROR) {
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("Successful call did not create file");
		}

#ifdef LINK_TEST
		/* Save link count for ENOSPC test */
		if(stat(filler, &stbuf) == SYSERROR)
		{
			err = errno;
			if(!testfail)
				xx_rpt(DELETION);
			in_rpt("stat(\"%s\") failed - errno %d (%s)", 
				filler, err, errname(err));
		}
		else
			PATH_TRACE;
		oldcount = stbuf.st_nlink;
#endif
	}

	if (i == MAXTRYS && !testfail) {
		xx_rpt(FAILURE);
		in_rpt("ENOSPC error didn't occur in %d trys", MAXTRYS);
	}

	tidynospc(base, i);
	free((void*)name);


	DBUG_VOID_RETURN;

}

#ifdef LINK_TEST
private void
chk_link(path1, path2)
char	*path1;
char	*path2;
{
	struct	stat stbuf;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("chk_link");

	if(stat(path1, &stbuf) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
		path1, err, errname(err));
	}
	else if(stbuf.st_nlink != oldcount)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to link() changed link count from %d to %d", oldcount, (int)stbuf.st_nlink);
	}
	else
		PATH_TRACE;

	if(eaccess(path2, F_OK) != SYSERROR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to link() created new link");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}
#endif
