/*
*      SCCS:  @(#)  POSIX.os/files/open/open.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/open/T.open Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.open

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the open() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	16 Mar 1989 (from old XPG2 code and NIST-PCTS code)
MODIFICATIONS:
		Geoff Clare, 7 June 1989
			Add do_access tests.

		Steve Ratcliffe, 14 Feb 1990
			Added ENFILE test.

		Steve Ratcliffe, 12 Feb 1990
			Added ENOSPC test.

		Geoff Clare, 5 Apr 1990
			New interface with pathtoolng() and nametoolng().

		Geoff Clare, 29 May 1990
			Allow for unlimited OPEN_MAX
		
		Geoff Clare, 19 June 1990
			Add test case for O_TRUNC on terminal

		David Sawyer
		Thu Aug 02 12:14:58 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Stuart Boutell, 14 Mar 1991
			Inserted test24, renumbered tests as nec.

		Neil Moses, 20 May 1992
			Added extra checks to tbopen() to ensure on
			failure that file is not created/truncated.

		Geoff Clare, 26 May 1992
			Remove testing of O_RDONLY|O_TRUNC combinations.

                John-Paul, 29 July 1992
                        Mode parameter in calls to crfile() and crdir()
                        changed from 'int' to 'mode_t' type.

		Andrew Dingwall, 27 August 1992
			XNFS enhancements

		Geoff Clare, 8 Jan 1993
			Change xnfs_anon_uid from int to uid_t.

		Geoff Clare, 17 Nov 1993
			Fix path tracing problem in ch2_t16().

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

		Geoff Clare, 15 Sept 1995
			Check VSX_TERMIOS_BUFFERED in test 16.

		Geoff Clare, 3 Nov 1995
			Changed child timeout for do_access() tests.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<termios.h>
#include	<time.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<sys/stat.h>
#include	<string.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<timeout.h>
#include	<pcts_max.h>
#include	<pathres.h>
#include	<chktimes.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"open_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef open
#endif



#define NO_TESTS	56		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)
#define NOK		02
#define PATH_OK		04
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define WRSIZE		10		/* size of write() test buffer */
#define T4_NFILES	3
#define NAP		3

#define T_NOK		-1
#define T_OK		0
#define T_CREAT		1
#define T_TRUNC		2
#define UMASKVAL	(S_IRUSR|S_IWGRP|S_IXOTH)

extern	char	*errname();
extern	char	*execpath();
extern	void	valrpt();
extern	char	*itoa();
extern	char	*uidtoa();
extern	char	*newstr();
extern	int	n_open_fds();
extern	char	**pathtoolng();
#if TEST_XNFS
extern	char	*xnfs_nametoolng();
#else
extern	char	**nametoolng();
#endif
extern	void	clntoolng();

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;
extern	int	vguid1;
extern	int	vggid1;

public	int	no_tests = NO_TESTS;


private int	testfail;
private int	open_max;
private char	wbuf[WRSIZE];

private char	*prog9 =	"open_t9";

#if !TEST_XNFS
private char	*prog10 =	"open_t10";
#endif /* !TEST_XNFS */

private char	*prog14 =	"open_t14";
private char	*prog15 =	"open_t15";

#if !TEST_XNFS
private char	*prog16 =	"open_t16";
#endif /* !TEST_XNFS */

private char	*prog20_22 =	"open_t20_3";

#if TEST_XNFS
private char	*prog25 =	"open_su";
private char	*prog30 =	"open_su";
private char	*prog31 =	"open_su";
#endif /* TEST_XNFS */

private char	*prog44 =	"open_t44";
private char	*prog45 =	"open_t45";
private char	*prog46 =	"open_t46";

#if TEST_XNFS
private char	*prog48 =	"open_t48";
private char	*prog49 =	"open_t49";
private char	*prog50 =	"open_t50";
private char	*prog51 =	"open_t51";
#endif /* TEST_XNFS */

private char	*t3_file =	"./open-t.3";
private char	*t4_files[T4_NFILES+1] = {
				"./open-t.4a",
				"./open-t.4b",
				"./open-t.4c",
				"./open-t.4d" };
private char	*t5_file =	"./open-t.5";
private char	*t6_file =	"./open-t.6";
private char	*t7_file =	"./open-t.7";
private char	*t8_file =	"./open-t.8";
private char	*t9_file =	"./open-t.9";

#if !TEST_XNFS
private char	*t10_file =	"./open-t.10";
private char	*t11_file =	"./open-t.11";
private char	*t12_file =	"./open-t.12";
private char	*t13_file =	"./open-t.13";
#endif /* !TEST_XNFS */

private char	*t14_file =	"./open-t.14";
private char	*t15_file =	"./open-t.15";

#if !TEST_XNFS
private char	*t16_file =	"./open-t.16";
#endif /* !TEST_XNFS */

private char	*t17_file =	"./open-t.17";
private char	*t18_file =	"./open-t.18";
private char	*t19_file =	"./open-t.19";
private char	*t24_file =	"./open-t.24";
private char	*t25_1_dir =	"./open-d.25.1";
private char	*t25_1_file =	"./open-d.25.1/open-t.25.1";

#if TEST_XNFS
private char	*t25_2_file =	T25_2_FILE;
#endif /* TEST_XNFS */

private char	*t26_file =	"./open-t.26";
private char	*t27_file =	"./open-t.27";
private char	*t28_file =	"./open-t.28";
private char	*t29_file =	"./open-t.29";
private char	*t30_1_dir =	"./open-d.30.1";
private char	*t30_1_file =	"./open-d.30.1/open-t.30.1";

#if TEST_XNFS
private char	*t30_2_file =	T30_2_FILE;
#endif /* TEST_XNFS */

private char	*t31_1_file =	"./open-t.31.1";

#if TEST_XNFS
private char	*t31_2_file =	T31_2_FILE;
#endif /* TEST_XNFS */

private char	*t32_file =	"./open-t.32";

#if !TEST_XNFS
private char	*t33_file =	"./open-t.33";
#endif /* !TEST_XNFS */

private char	*t34_dir =	"./open-d.34";
private char	*t35_dir =	"./open-d.35";
private char	*t36_file =	"./open-t.36";
private char	*t37_file =	"./open-t.37";
private char	*t39_file =	"./open-t.39";
private char	*t40_dir =	"./open-d.40";
private char	*t40_file =	"./open-d.40/open-t.40";
private char	*t42_1_file =	"./open-t.42";
private char	*t42_2_file =	"./open-t.42/open-t.42.1";

#if !TEST_XNFS
private char	*t43_file =	"./open-t.43";
#endif /* !TEST_XNFS */

private char	*t44_dir =	"./open-d.44";
private char	*t45_file =	"./open-t.45";
private char	*t46_dir =	"./open-d.46";

#if TEST_XNFS
private char	*t47_file =	"./open-t.47";
private char	*t48_file =	"./open-t.48";
private char	*t49_file =	"./open-t.49";
private char	*t50_mnt =	T50_MNT;
private char	*t52_file =	"./open-t.52";
#endif /* TEST_XNFS */


private void	prepare_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20_22();
private	void	test23();
private	void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();
private void	test30();
private void	test31();
private void	test32();
private void	test33();
private void	test34();
private void	test35();
private void	test36();
private void	test37();
private void	test38();
private void	test39();
private void	test40();
private void	test41();
private void	test42();
private void	test43();
private void	test44();
private void	test45();
private void	test46();
private void	test47();
private void	test48_49();
private void	test50();
private void	test51();
private void	test52();
private void	test53();
private void	test54();
private void	test55();
private void	test56();

private void	ch_t9();

#if !TEST_XNFS
private void	ch_t10();
private void	ch_t12();
private void	ch_t13();
#endif /* !TEST_XNFS */

private void	ch_t14();
private void	ch_t15();

#if !TEST_XNFS
private void	ch1_t16();
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
private void	ch2_t16();
#endif
#endif /* !TEST_XNFS */

private void	ch_t20_22();

#if TEST_XNFS
private void	ch_t25();
private void	ch_t30();
private void	ch_t31();
#endif /* TEST_XNFS */

private void	ch_t36();
private void	ch_t44();
private void	ch_t45();
private void	ch_t46();

#if TEST_XNFS
private void	ch_t48_49();
private void	ch_t50();
private void	ch_t51();
private void	ch_t52();
#endif /* TEST_XNFS */

#if !TEST_XNFS
private void	pa_t12();
private void	pa_t13();
#endif /* !TEST_XNFS */

private int	tbopen();
private int	tbop_to();
private int	savparent();
private void	chkparent();
private	int	chk_creat_trunc();

#if TEST_XNFS
private int	snames[] = { 1, 0 };
#endif

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	test3, 3,
	test4, 4,
	test5, 5,
	test6, 6,
	test7, 7,
	test8, 8,
	test9, 9,
	test10, 10,
	test11, 11,
	test12, 12,
	test13, 13,
	test14, 14,
	test15, 15,
	test16, 16,
	test17, 17,
	test18, 18,
	test19, 19,
	test20_22, 20,
	test20_22, 21,
	test20_22, 22,
	test23, 23,
	test24, 24,
	test25, 25,
	test26, 26,
	test27, 27,
	test28, 28,
	test29, 29,
	test30, 30,
	test31, 31,
	test32, 32,
	test33, 33,
	test34, 34,
	test35, 35,
	test36, 36,
	test37, 37,
	test38, 38,
	test39, 39,
	test40, 40,
	test41, 41,
	test42, 42,
	test43, 43,
	test44, 44,
	test45, 45,
	test46, 46,
	test47, 47,
	test48_49, 48,
	test48_49, 49,
	test50, 50,
	test51, 51,
	test52, 52,
	test53, 53,
	test54, 54,
	test55, 55,
	test56, 56,
	NULL, 0
};

private	void
prepare_tests()
{
	int i;

	DBUG_ENTER("prepare_tests");

	if(uids()!=0)
	{
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	for (i = 0 ; i < WRSIZE ; i++)
		wbuf[i] = i % 100;
	/*
	 *	this so that setuid children can write to files
	 */
	(void) umask((mode_t)0);

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0)
	{
		if (errno == 0)
			open_max = PCTS_OPEN_MAX+1;
		else
		{
			for (i = 1; i <= NO_TESTS; i++)
				tet_delete(i,
					"sysconf(_SC_OPEN_MAX) failed during prep");
		}
	}

#if TEST_XNFS
	if (xnfs_uids() < 0)
		tet_delete(49, "xnfs_uids() failed");
#endif

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* pathname resolution */

	char 	*p1, *p2;
	int 	npairs, i, fd1, fd2;
	struct stat stbuf1, stbuf2;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_CREAT);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_CREAT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		globok = 0;
		fd1 = tbopen(p1, O_RDONLY, "O_RDONLY", MODEANY,
				SUCCEED, NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		fd2 = tbopen(p2, O_RDONLY, "O_RDONLY", MODEANY,
				SUCCEED, NOERROR);
		PATH_FUNC_TRACE;

		if (fd1 < 0 || fd2 < 0)
		{
			(void) close(fd1);
			(void) close(fd2);
			continue;
		}

#ifdef TESTBED2
#define fd1 0
#endif

		if (fstat(fd1, &stbuf1) == SYSERROR ||
		    fstat(fd2, &stbuf2) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: fstat() failed");
		}
		else if (stbuf1.st_dev != stbuf2.st_dev ||
		         stbuf1.st_ino != stbuf2.st_ino)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("open() did not resolve \"%s\" and \"%s\" to the same file",
				p1, p2);
				
		}
		else
			PATH_TRACE;

#ifdef TESTBED2
#undef fd1
#endif

		(void) close(fd1);
		(void) close(fd2);
	}

	if (testfail == 0)
		PATH_XS_RPT(1+(3*npairs));
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames */

	char 	**p;
	int 	fd, err;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{	
		if (i++ == 0)
			PATH_TRACE;
		if (eaccess(*p, F_OK) == 0 && remove(*p) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("could not remove \"%s\" - errno %d (%s)",
				*p, err, errname(err));
			DBUG_VOID_RETURN;
		}
	}

#ifdef TESTBED2
	(void) close(crfile(portfnam[0], MODEANY, (int)getuid(), (int)getgid()));
#endif

	/* use O_CREAT|O_EXCL so undistinguished names will give EEXIST */

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		fd = tbopen(*p, O_WRONLY|O_CREAT|O_EXCL,
			"O_WRONLY|O_CREAT|O_EXCL", MODEANY, SUCCEED, NOERROR);
		(void) close(fd);
	}

	i = 0;
	for (p = portfnam; *p != NULL; p++)
	{
		if (i++ == 0)
			PATH_TRACE;
		(void) unlink(*p);
	}

	if (testfail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* open() returns fd used to refer to the opened file */

	int 	fd, err;
	struct stat stbuf1, stbuf2;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	globok = 0;
	fd = tbopen(t3_file, O_RDWR|O_CREAT, "O_RDWR|O_CREAT", MODEANY,
			SUCCEED, NOERROR);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	if (fstat(fd, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fstat() failed on fd from open(O_RDWR|O_CREAT) - errno %d (%s)",
			err, errname(err));
	}
	else if (stat(t3_file, &stbuf2) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed after file created with O_RDWR|O_CREAT - errno %d (%s)",
			err, errname(err));
	}
	else if (stbuf1.st_dev != stbuf2.st_dev ||
	    stbuf1.st_ino != stbuf2.st_ino)
	{
		xx_rpt(FAILURE);
		in_rpt("fd returned by open() does not refer to correct file");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) close(fd);
	(void) unlink(t3_file);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* open returns the lowest available file descriptor */

	int 	i, err, fd[T4_NFILES+1];
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	for (i = 0 ; i < T4_NFILES && testfail == 0 ; i++)
	{
		if (i==0)
			PATH_TRACE;
		fd[i] = tbopen(t4_files[i], O_RDONLY|O_CREAT,
				"O_RDONLY|O_CREAT", MODEANY, SUCCEED, NOERROR);
	}

	for (i = 1; i < T4_NFILES && testfail == 0; i++)
	{
		if (i==1)
			PATH_TRACE;
		if (fd[i-1] >= fd[i])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("open() did not return lowest numbered file descriptor");
		}
	}

	if (testfail == 0)
	{
		if (close(fd[0]) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("close() failed - errno %d (%s)",
				err, errname(err));
		}
		else if ((fd[T4_NFILES] = tbopen(t4_files[T4_NFILES],
			O_RDONLY|O_CREAT, "O_RDONLY|O_CREAT", MODEANY,
			SUCCEED, NOERROR)) != fd[0])
		{
			xx_rpt(FAILURE);
			in_rpt("open() did not re-use closed file descriptor");
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(3);
		}
	}

	for (i = 1; i <= T4_NFILES; i++)
		(void) close(fd[i]);
	for (i = 0; i <= T4_NFILES; i++)
		(void) unlink(t4_files[i]);

	DBUG_VOID_RETURN;
}


private void
test5()
{
	/* open sets file pointer to beginning of file */

	int	fd, fd2, err;
	mode_t	mode = (S_IRWXU|S_IRWXG);
	char	cbuf[16];
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	if ((fd = creat(t5_file, mode)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			t5_file, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, cbuf, sizeof(cbuf)) != sizeof(cbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() failed - errno %d (%s)",
			err, errname(err));
		(void) close(fd);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	fd2 = tbopen(t5_file, O_RDWR, "O_RDWR", MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{
		if (lseek(fd2, (off_t)0, SEEK_CUR) != 0)
		{
			xx_rpt(FAILURE);
			in_rpt("open(\"%s\", O_RDWR) did not set file pointer to 0",
				t5_file);
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(4);
		}

		(void) close(fd2);
	}

	(void) close(fd);
	(void) unlink(t5_file);

	DBUG_VOID_RETURN;
}


private void
test6()
{
	/* open a file for reading only */

	int		fd;
	int		err;
	char		rbuf[sizeof(wbuf)];
	int		pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	errno = NOERROR;
	if ((fd = creat(t6_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t6_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, wbuf, sizeof(wbuf)) != sizeof(wbuf))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not write to file %s", t6_file);
		(void) close(fd);
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	fd = tbopen(t6_file, O_RDONLY, "O_RDONLY", MODEANY, SUCCEED, NOERROR);
	if (testfail > 0)
	{
		(void) unlink(t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	errno = NOERROR;
	if (read(fd, rbuf, sizeof(rbuf)) != sizeof(rbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() failed on O_RDONLY file - errno %d (%s)",
			err, errname(err));
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (write(fd, rbuf, sizeof(rbuf)) != SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("write() succeeded on O_RDONLY file");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	(void) close(fd);
	(void) unlink(t6_file);

	DBUG_VOID_RETURN;
}


private void
test7()
{
	/* open a file for writing only */

	int		cfd;
	int		ofd;
	int		err;
	char		rbuf[sizeof(wbuf)];
	int		pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	errno = NOERROR;
	if ((cfd = creat(t7_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t7_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(cfd);

	globok = 0;
	ofd = tbopen(t7_file, O_WRONLY, "O_WRONLY", MODEANY, SUCCEED, NOERROR);
	if (testfail > 0)
	{
		(void) unlink(t7_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	errno = NOERROR;
	if (write(ofd, wbuf, sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("write() on O_WRONLY file failed - errno %d (%s)",
			err, errname(err));
	}
	else
		PATH_TRACE;

	(void) lseek(ofd, (off_t)0, SEEK_SET);
	
	errno = NOERROR;
	if (read(ofd, rbuf, sizeof(rbuf)) != SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() succeeded on O_WRONLY file");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) close(ofd);
	(void) unlink(t7_file);

	DBUG_VOID_RETURN;
}


private void
test8()
{
	/* open a file for reading and writing */

	int		ofd;
	int		cfd;
	int		err;
	char		rbuf[sizeof(wbuf)];
	int		pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

	errno = NOERROR;
	if ((cfd = creat(t8_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t8_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(cfd);

	globok = 0;
	ofd = tbopen(t8_file, O_RDWR, "O_RDWR", MODEANY, SUCCEED, NOERROR);
	if (testfail > 0)
	{
		(void) unlink(t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	errno = NOERROR;
	if (write(ofd, wbuf, sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("write() on O_RDWR file failed - errno %d (%s)",
			err, errname(err));
	}
	else
		PATH_TRACE;

	(void) lseek(ofd, (off_t)0, SEEK_SET);
	
	errno = NOERROR;
	if (read(ofd, rbuf, sizeof(rbuf)) != sizeof(rbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on O_RDWR file failed - errno %d (%s)",
			err, errname(err));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) close(ofd);
	(void) unlink(t8_file);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	/* O_CREAT does not change mode, ownership, data or
	   timestamps if file exists */

	int		fd;
	int		err;
	char		cbuf[16];

	DBUG_ENTER("test9");
	if ((fd = creat(t9_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t9_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	if (write(fd, cbuf, sizeof(cbuf)) != sizeof(cbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() failed - errno %d (%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	(void) close(fd);

	(void) cppair(ch_t9, NULLFN, WAITTIME, 0);

	(void) unlink(t9_file);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t9");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog9;
	args[1] = t9_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog9), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog9, err, errname(err));

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test10()
{
	DBUG_ENTER("test10");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test10()
{
	/* O_CREAT creates file with correct mode, owner, group */

	DBUG_ENTER("test10");

	if (eaccess(t10_file, F_OK) == 0 && unlink(t10_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t10_file);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t10, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t10");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog10;
	args[1] = t10_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog10), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog10, err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test11()
{
	DBUG_ENTER("test11");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test11()
{
	/* open a FIFO with no delay set for reading */

	int 	fd, err;
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	errno = NOERROR;
	if (mkfifo(t11_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(%s) failed - errno %d (%s)",
			t11_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	fd = tbop_to(t11_file, O_RDONLY|O_NONBLOCK, "O_RDONLY|O_NONBLOCK",
		MODEANY, WAITTIME, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd != SYSERROR)
		(void) close(fd);
	(void) unlink(t11_file);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test12()
{
	DBUG_ENTER("test12");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test12()
{
	/* open FIFO for reading only with O_NONBLOCK clear will block
	   until process opens for writing */

	int 	err = 0;
	int	pathok = 0;

	DBUG_ENTER("test12");

	testfail = 0;

	if (mkfifo(t12_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(%s) failed - errno %d (%s)",
			t12_file, err, errname(err));
		(void) unlink(t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (cppair(ch_t12, pa_t12, WAITTIME, NOK) == NOK)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open() on FIFO with O_WRONLY|O_NONBLOCK failed in child");
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t12_file);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	int	fd;

	DBUG_ENTER("ch_t12");

	(void) sleep(WAITTIME/2);

	fd = open(t12_file, O_WRONLY|O_NONBLOCK);
	
	if (fd == SYSERROR)
		DBUG_EXIT(NOK);

	(void) sleep(WAITTIME/2);

	(void) close(fd);

	DBUG_VOID_RETURN;
}

private void
pa_t12()
{
	int 	fd, err;
	unsigned unused_time;
	int	pathok = 0;

	DBUG_ENTER("pa_t12");

	SET_TIMEOUT(WAITTIME)

	fd = open(t12_file, O_RDONLY);
	err = errno;
	unused_time = alarm(0);

	CLEAR_ALARM

	if (fd < 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("open(\"%s\", O_RDONLY) timed out", t12_file);
		else
			in_rpt("open(\"%s\", O_RDONLY) failed - errno %d (%s)",
				t12_file, err, errname(err));
	}
	else if ((WAITTIME - unused_time) < WAITTIME/4)	/* WAITTIME/4 leeway */
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("open(\"%s\", O_RDONLY) returned without waiting for FIFO to be opened for writing",
			t12_file);
	}
	else
		PATH_TRACE;

	if (fd >= 0)
		(void) close(fd);

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test13()
{
	/* open FIFO for writing only with O_NONBLOCK clear will block
	   until process opens for reading */

	int 	err = 0;
	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

	if (mkfifo(t13_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(%s) failed - errno %d (%s)",
			t13_file, err, errname(err));
		(void) unlink(t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (cppair(ch_t13, pa_t13, WAITTIME, NOK) == NOK)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open() on FIFO with O_RDONLY|O_NONBLOCK failed in child");
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t13_file);

	DBUG_VOID_RETURN;
}

private void
ch_t13()
{
	int	fd;

	DBUG_ENTER("ch_t13");

	(void) sleep(WAITTIME/2);

	fd = open(t13_file, O_RDONLY|O_NONBLOCK);
	
	if (fd == SYSERROR)
		DBUG_EXIT(NOK);

	(void) sleep(WAITTIME/2);

	(void) close(fd);

	DBUG_VOID_RETURN;
}

private void
pa_t13()
{
	int 	fd, err;
	unsigned unused_time;
	int	pathok = 0;

	DBUG_ENTER("pa_t13");

	SET_TIMEOUT(WAITTIME)

	fd = open(t13_file, O_WRONLY);
	err = errno;
	unused_time = alarm(0);

	CLEAR_ALARM

	if (fd < 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (alrm_flag > 0)
			in_rpt("open(\"%s\", O_WRONLY) timed out", t13_file);
		else
			in_rpt("open(\"%s\", O_WRONLY) failed - errno %d (%s)",
				t13_file, err, errname(err));
	}
	else if ((WAITTIME - unused_time) < WAITTIME/4)	/* WAITTIME/4 leeway */
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("open(\"%s\", O_WRONLY) returned without waiting for FIFO to be opened for reading",
			t13_file);
	}
	else
		PATH_TRACE;

	if (fd >= 0)
		(void) close(fd);

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private void
test14()
{
	/* O_TRUNC truncates - mode, owner and group unchanged */

	int		fd;
	int		err;
	char		cbuf[16];

	DBUG_ENTER("test14");
	if ((fd = creat(t14_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t14_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	if (write(fd, cbuf, sizeof(cbuf)) != sizeof(cbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() failed - errno %d (%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	(void) close(fd);

	(void) cppair(ch_t14, NULLFN, WAITTIME, 0);

	(void) unlink(t14_file);

	DBUG_VOID_RETURN;
}

private void
ch_t14()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t14");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog14;
	args[1] = t14_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog14), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog14, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test15()
{
	/* O_TRUNC truncates - mode, owner and group unchanged */

	int		fd;
	int		err;
	char		cbuf[16];

	DBUG_ENTER("test15");
	if ((fd = creat(t15_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t15_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	if (write(fd, cbuf, sizeof(cbuf)) != sizeof(cbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() failed - errno %d (%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	(void) close(fd);

	(void) cppair(ch_t15, NULLFN, WAITTIME, 0);

	(void) unlink(t15_file);

	DBUG_VOID_RETURN;
}

private void
ch_t15()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t15");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog15;
	args[1] = t15_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog15), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog15, err, errname(err));

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test16()
{
	DBUG_ENTER("test16");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test16()
{
	/* O_TRUNC on FIFO: size, mode & owner unchanged */
	/* O_TRUNC on terminal: data in queues not flushed */

	int	err;
	int	retval;
	int	pathok = 0;

	DBUG_ENTER("test16");

	testfail = 0;

	if (mkfifo(t16_file, MODEANY) == SYSERROR)
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(%s) failed - errno %d (%s)",
			t16_file, err, errname(err));
	}
	else 
	{
		PATH_TRACE;
		globok = 0;
		retval = cppair(ch1_t16, NULLFN, 2 * WAITTIME, NOK|PATH_OK);
		if (retval & NOK)
			testfail++;
		else if (retval & PATH_OK)
			PATH_TRACE;
	}

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 0;
	retval = cppair(ch2_t16, NULLFN, 2 * WAITTIME, NOK|PATH_OK);
	if (retval & NOK)
		testfail++;
	else if (retval & PATH_OK)
		PATH_TRACE;
#endif

	if (testfail == 0)
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
		PATH_XS_RPT(3);
#else
		PATH_XS_RPT(2);
#endif

	(void) unlink(t16_file);

	DBUG_VOID_RETURN;
}

private void
ch1_t16()
{
	char	*suid, *sgid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch1_t16");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog16;
	args[1] = t16_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog16), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog16, err, errname(err));

	DBUG_EXIT(NOK);
}

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
private void
ch2_t16()
{
	char		*cp;
	int		buffered, ret, err;
	int		tty_fildes, loop_fildes;
	struct termios	tty_termios, loop_termios;
	static char	buf1[] = "data for tty input queue\n";
	static char	buf2[] = "tty output queue data\n";
	unsigned 	len1 = sizeof(buf1)-1;
	unsigned 	len2 = sizeof(buf2)-1;
	char		inbuf[50];
	int		pathok = 0;

	DBUG_ENTER("ch2_t16");

	switch (termios_prep(&tty_fildes, &tty_termios,
				&loop_fildes, &loop_termios))
	{
	case 0:
		PATH_TRACE;
		break;
	
	case 1:
		/* general terminal interface not supported */
		DBUG_PRINTF("ch2_t16", "termios_prep() returned 1");
		DBUG_EXIT(OK | PATH_OK);
	
	default:
		if (testfail++ == 0)
			xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP");
		DBUG_EXIT(NOK);
	}

	cp = tet_getvar("VSX_TERMIOS_BUFFERED");
	if (cp == NULL || *cp == '\0')
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
		DBUG_EXIT(NOK);
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		PATH_TRACE;
		buffered = 0;
	}
	else
	{
		PATH_TRACE;
		buffered = 1;
	}

	if (tcflush(tty_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (tcflush(loop_fildes, TCIOFLUSH) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflush() failed on VSX_TERMIOS_LOOP - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if ((ret = write(loop_fildes, buf1, len1)) != len1)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY returned %d, expected %u",
			ret, len1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (buffered && tcflow(tty_fildes, TCOOFF) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOOFF) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (buffered && (ret = write(tty_fildes, buf2, len2)) != len2)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY returned %d, expected %u",
			ret, len2);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		(void) tcflow(tty_fildes, TCOON);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	/* Input and output queues now both contain data */

	if (open(tet_getvar("VSX_TERMIOS_TTY"),
			O_RDWR|O_NONBLOCK|O_TRUNC) < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(O_RDWR|O_NONBLOCK|O_TRUNC) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		(void) tcflow(tty_fildes, TCOON);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	(void) tcflush(tty_fildes, TCIOFLUSH);
#endif

	/* Check O_TRUNC did not discard any data in queues */

	if (buffered && tcflow(tty_fildes, TCOON) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("tcflow(TCOON) failed on VSX_TERMIOS_TTY - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME/2);

	ret = read(tty_fildes, inbuf, sizeof(inbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != len1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from VSX_TERMIOS_TTY returned %d, expected %u",
			ret, len1);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (strncmp(inbuf, buf1, (size_t)len1) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("data read from VSX_TERMIOS_TTY was incorrect");
	}
	else
		PATH_TRACE;

	if (buffered)
	{
		SET_TIMEOUT(WAITTIME/2);

		ret = read(loop_fildes, inbuf, sizeof(inbuf));
		err = errno;

		CLEAR_ALARM

		if (ret != len2)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("read() from VSX_TERMIOS_LOOP returned %d, expected %u",
				ret, len2);
			if (ret < 0)
				in_rpt("errno was set to %d (%s)", err, errname(err));
		}
		else if (strncmp(inbuf, buf2, (size_t)len2) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("data read from VSX_TERMIOS_LOOP was incorrect");
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail != 0)
		DBUG_EXIT(NOK);
	
	if (pathok == 11)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}
#endif /* !XPG3 mode */

#endif /* TEST_XNFS */


private void
test17()
{
	/* open files are set to remain open across exec calls */

	int 	fd, err, flags;
	int	pathok = 0;

	DBUG_ENTER("test17");

	testfail = 0;

	globok = 0;
	fd = tbopen(t17_file, O_RDWR|O_CREAT, "O_RDWR|O_CREAT", MODEANY,
		SUCCEED, NOERROR);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((flags = fcntl(fd, F_GETFD, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("fcntl(F_GETFD) failed - errno %d (%s)",
			err, errname(err));
	}
	else if ((flags & FD_CLOEXEC) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("FD_CLOEXEC was set for opened file");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void) close(fd);
	(void) unlink(t17_file);

	DBUG_VOID_RETURN;
}

private void
test18()
{
	/* O_CREAT alters timestamps on new file and parent dir. */

	char	*chkt;
	int	fd, err, fail = 0;
	time_t	t0;
	struct stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("test18");

	testfail = 0;

#if TEST_XNFS
	if (xnfs_rtime_c(&t0, SP_T18_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#else
	(void) time(&t0);
	PATH_TRACE;
#endif
	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

	globok = 0;
	fd = tbopen(t18_file, O_RDWR|O_CREAT, "O_RDWR|O_CREAT", MODEANY,
		SUCCEED, NOERROR);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	(void) close(fd);

	if (stat(t18_file, &stbuf) == SYSERROR)
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("stat() failed - errno %d (%s)", err, errname(err));
	}
	else
	{
		if (stbuf.st_atime < t0)
			fail |= 01;
		else
			PATH_TRACE;

		if (stbuf.st_ctime < t0)
			fail |= 02;
		else
			PATH_TRACE;

		if (stbuf.st_mtime < t0)
			fail |= 04;
		else
			PATH_TRACE;
		
		if (fail != 0)
		{
			testfail++;
			xx_rpt(FAILURE);
			if (fail & 01)
				in_rpt("st_atime field on created file was not marked for update");
			if (fail & 02)
				in_rpt("st_ctime field on created file was not marked for update");
			if (fail & 04)
				in_rpt("st_mtime field on created file was not marked for update");
		}
	}

	globok = 0;
	chkt = chktimes(".", NEWCTIME|NEWMTIME);
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("parent directory %s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	(void) unlink(t18_file);

	DBUG_VOID_RETURN;
}

private void
test19()
{
	/* O_TRUNC marks st_ctime and st_mtime for update */
	/* NB assertion doesn't say file has to be non-empty */

	char	*chkt;
	int	fd, err;
	int	pathok = 0;

	DBUG_ENTER("test19");

	testfail = 0;

	if ((fd = creat(t19_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t19_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	savtimes(t19_file);
	PATH_FUNC_TRACE;

	globok = 0;
	fd = tbopen(t19_file, O_RDWR|O_TRUNC, "O_RDWR|O_TRUNC", MODEANY,
		SUCCEED, NOERROR);
	if (testfail > 0)
	{
		(void) unlink(t19_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	(void) close(fd);

	globok = 0;
	chkt = chktimes(t19_file, NEWCTIME|NEWMTIME);
	if (chkt != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("%s", chkt);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(4);
	}

	(void) unlink(t19_file);

	DBUG_VOID_RETURN;
}

private void
test20_22()
{
	/* all the do_access tests */

	int waittime;

	DBUG_ENTER("test20_22");

	waittime = WAITTIME * (5 + sysconf(_SC_NGROUPS_MAX)/4);

	(void) cppair(ch_t20_22, NULLFN, waittime, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t20_22()
{
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t20_22");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog20_22;
	args[1] = itoa(tet_thistest);
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog20_22), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog20_22, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test23()
{
	DBUG_ENTER("test23");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

private void
test24()
{
	int err, flags;
	int loop;
	int fd;
	static struct { int mask; char *name;	} t24_cases[3] =
		{
			{ O_RDONLY, "O_RDONLY" } ,
			{ O_WRONLY, "O_WRONLY" } ,
			{ O_RDWR,   "O_RDWR"   }
		};
	int pathok = 0;

	DBUG_ENTER("test24");

	testfail = 0;
	if ((fd = creat(t24_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t24_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	for (loop=0; loop<3; loop++)
	{
		DBUG_PRINTF("test24","Testing %s", t24_cases[loop].name);
		globok = 0;
		fd = tbopen(t24_file, t24_cases[loop].mask, 
			t24_cases[loop].name, MODEANY, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	(void) fcntl(fd, F_SETFD, FD_CLOEXEC);
#endif

		if ((flags = fcntl(fd, F_GETFD, 0)) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("fcntl(F_GETFD) failed - errno %d (%s)",
				err, errname(err));
		}
		else if ((flags & FD_CLOEXEC) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("FD_CLOEXEC was set on file descriptor after open(%s)",
					t24_cases[loop].name);
		}
		else
			PATH_TRACE;
		(void) close(fd);
	}

	if (testfail == 0)
		PATH_XS_RPT(1+(3*2));

	(void) unlink(t24_file);

	DBUG_VOID_RETURN;
}

private void
test25()
{
	/* open file in directory which denies searching */

	int	pathcheck;
	int	pathok = 0;
#if TEST_XNFS
	int	rc;
#endif

	DBUG_ENTER("test25");

	testfail = 0;

	if (crdir(t25_1_dir,
		(mode_t)(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH),
		(int) getuid(), (int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not make directory %s", t25_1_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbopen(t25_1_file, O_RDWR, "O_RDWR", MODEANY, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) unlink(t25_1_file);
	(void) rmdir(t25_1_dir);

#if TEST_XNFS

	if (tet_sync(SP_T25_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t25, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T25_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
	}
	else
		PATH_TRACE;

	pathcheck = 5;

#else /* TEST_XNFS */

	pathcheck = 2;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
ch_t25()
{
	int err;
	char *args[5];

	DBUG_ENTER("ch_t25");

	args[0] = prog25;
	args[1] = t25_2_file;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog25), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog25, err, errname(err));

	DBUG_EXIT(NOK);
}

#endif /* TEST_XNFS */

private void
test26()
{
	/* O_RDONLY without read permission */

	int	fd, err;
	mode_t	mode = (S_IWUSR|S_IXUSR|S_IWGRP|S_IXGRP|S_IWOTH|S_IXOTH);
	int	pathok = 0;

	DBUG_ENTER("test26");

	testfail = 0;

	if ((fd = creat(t26_file, mode)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			t26_file, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbopen(t26_file, O_RDONLY, "O_RDONLY", MODEANY, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t26_file);

	DBUG_VOID_RETURN;
}

private void
test27()
{
	/* O_WRONLY without write permission */

	int	fd, err;
	mode_t	mode = (S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	int	pathok = 0;

	DBUG_ENTER("test27");

	testfail = 0;

	if ((fd = creat(t27_file, mode)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			t27_file, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbopen(t27_file, O_WRONLY, "O_WRONLY", MODEANY, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t27_file);

	DBUG_VOID_RETURN;
}

private void
test28()
{
	/* O_RDWR without read permission */

	int	fd, err;
	mode_t	mode = (S_IWUSR|S_IXUSR|S_IWGRP|S_IXGRP|S_IWOTH|S_IXOTH);
	int	pathok = 0;

	DBUG_ENTER("test28");

	testfail = 0;

	if ((fd = creat(t28_file, mode)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			t28_file, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbopen(t28_file, O_RDWR, "O_RDWR", MODEANY, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t28_file);

	DBUG_VOID_RETURN;
}

private void
test29()
{
	/* O_RDWR without write permission */

	int	fd, err;
	mode_t	mode = (S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	int	pathok = 0;

	DBUG_ENTER("test29");

	testfail = 0;

	if ((fd = creat(t29_file, mode)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			t29_file, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbopen(t29_file, O_RDWR, "O_RDWR", MODEANY, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t29_file);

	DBUG_VOID_RETURN;
}

private void
test30()
{
	/* creat file in directory which denies write permission */

	int	pathcheck;
	int	pathok = 0;

#if TEST_XNFS
	int	rc;
#endif

	DBUG_ENTER("test30");

	testfail = 0;

	if (crdir(t30_1_dir,
		(mode_t)(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH),
		(int) getuid(), (int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not make directory %s", t30_1_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbopen(t30_1_file, O_RDWR|O_CREAT, "O_RDWR|O_CREAT", MODEANY,
		SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) unlink(t30_1_file);
	(void) rmdir(t30_1_dir);

#if TEST_XNFS

	if (tet_sync(SP_T30_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t30, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T30_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
	}
	else
		PATH_TRACE;

	pathcheck = 5;

#else /* TEST_XNFS */

	pathcheck = 2;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
ch_t30()
{
	int err;
	char *args[5];

	DBUG_ENTER("ch_t30");

	args[0] = prog30;
	args[1] = t30_2_file;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog30), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog30, err, errname(err));

	DBUG_EXIT(NOK);
}

#endif /* TEST_XNFS */

private void
test31()
{
	/* O_TRUNC without write permission */

	int	fd, err, pathcheck;
	mode_t	mode = (S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	int	pathok = 0;
#if TEST_XNFS
	int	rc;
#endif

	DBUG_ENTER("test31");

	testfail = 0;

	if ((fd = creat(t31_1_file, mode)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			t31_1_file, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(write(fd, "1", 1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tbopen(t31_1_file, O_WRONLY|O_TRUNC, "O_WRONLY|O_TRUNC", MODEANY,
		SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbopen(t31_1_file, O_RDWR|O_TRUNC, "O_RDWR|O_TRUNC", MODEANY,
		SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) unlink(t31_1_file);

#if TEST_XNFS

	if (tet_sync(SP_T31_CREAT2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to CREAT2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t31, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
		testfail++;
	else if (rc & PATH_OK)
		PATH_TRACE;

	if (tet_sync(SP_T31_UNLINK2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to UNLINK2_END failed");
	}
	else
		PATH_TRACE;

	pathcheck = 7;

#else /* TEST_XNFS */

	pathcheck = 4;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
ch_t31()
{
	int err;
	char *args[5];

	DBUG_ENTER("ch_t31");

	args[0] = prog31;
	args[1] = t31_2_file;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog31), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog31, err, errname(err));

	DBUG_EXIT(NOK);
}

#endif /* TEST_XNFS */

private void
test32()
{
	/* O_CREAT|O_EXCL on an existing file */

	char	*chkt;
	int	fd, err;
	int	pathok = 0;

	DBUG_ENTER("test32");

	testfail = 0;

	if ((fd = creat(t32_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t32_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	if (savparent(".") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(t32_file);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbopen(t32_file, O_RDWR|O_CREAT|O_EXCL,
		"O_RDWR|O_CREAT|O_EXCL", MODEANY, SYSERROR, EEXIST);
	PATH_FUNC_TRACE;

	if (testfail > 0)
		DBUG_VOID_RETURN;

	globok = 0;
	chkt = chktimes(t32_file, 0);
	if (chkt != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("file %s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	chkparent(".");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) unlink(t32_file);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test33()
{
	DBUG_ENTER("test33");
	xx_rpt(UNTESTED);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test33()
{
	/* open of blocked FIFO is interrupted by signal */

	char	*chkt;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test33");

	testfail = 0;

	if (mkfifo(t33_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("mkfifo() failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (savparent(".") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(t33_file);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbop_to(t33_file, O_WRONLY, "O_WRONLY", MODEANY,
		WAITTIME/2, SYSERROR, EINTR);
	PATH_FUNC_TRACE;

	if (alrm_flag == 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: O_WRONLY on FIFO did not block");
	}
	else
		PATH_TRACE;

	if (testfail > 0)
		DBUG_VOID_RETURN;

	globok = 0;
	chkt = chktimes(t33_file, 0);
	if (chkt != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("file %s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	chkparent(".");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	(void) unlink(t33_file);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private void
test34()
{
	/* O_WRONLY on a directory */

	char	*chkt;
	int	pathok = 0;

	DBUG_ENTER("test34");

	testfail = 0;

	if (crdir(t34_dir, MODEANY, (int) getuid(),
			(int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t34_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (savparent(".") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(t34_dir);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbopen(t34_dir, O_WRONLY, "O_WRONLY", MODEANY, SYSERROR, EISDIR);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	chkt = chktimes(t34_dir, 0);
	if (chkt != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("file %s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	chkparent(".");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) rmdir(t34_dir);

	DBUG_VOID_RETURN;
}

private void
test35()
{
	/* O_RDWR on a directory */

	char	*chkt;
	int	pathok = 0;

	DBUG_ENTER("test35");

	testfail = 0;

	if (crdir(t35_dir, MODEANY, (int) getuid(),
			(int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t35_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (savparent(".") != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(t35_dir);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbopen(t35_dir, O_RDWR, "O_RDWR", MODEANY, SYSERROR, EISDIR);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	chkt = chktimes(t35_dir, 0);
	if (chkt != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("file %s", chkt);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	chkparent(".");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) rmdir(t35_dir);

	DBUG_VOID_RETURN;
}

private void
test36()
{
	/* OPEN_MAX/PCTS_OPEN_MAX/EMFILE etc. */

	int	fd;
	int	err;

	DBUG_ENTER("test36");

	testfail = 0;

	if ((fd = creat(t36_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("could not creat() %s - errno %d (%s)", t36_file, err,
			errname(err));
		DBUG_VOID_RETURN;
	}

	if(write(fd, "1", 1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err, errname(err));
		DBUG_VOID_RETURN;
	}

	(void) close(fd);

	(void) cppair(ch_t36, NULLFN, 4 * WAITTIME, 0);

	(void) unlink(t36_file);

	DBUG_VOID_RETURN;
}

private void
ch_t36()
{
	long	testmax = PCTS_OPEN_MAX;
	int	i;
	int	j;
	int	pathok = 0;

	DBUG_ENTER("ch_t36");

	if (open_max < PCTS_OPEN_MAX)
		testmax = open_max;

	/* open lesser of OPEN_MAX and PCTS_OPEN_MAX files successfully */

	j = 0;
	for (i = n_open_fds(); i < testmax; i++)
	{
		if (j++ == 0)
			PATH_TRACE;
		(void) tbopen(t36_file, O_RDONLY, "O_RDONLY", MODEANY,
				SUCCEED, NOERROR);
		if (testfail > 0)
			break;
	}

	/* Check that if O_TRUNC used, then file not created on EMFILE error */
	if (testfail == 0 && open_max <= PCTS_OPEN_MAX)
	{
		globok = 0;
		(void) tbopen(t36_file, O_RDWR|O_TRUNC, "O_RDWR|O_TRUNC", MODEANY, SYSERROR, EMFILE);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	/* Check that if O_CREAT used, then file not created on EMFILE error */
	if (testfail == 0 && open_max <= PCTS_OPEN_MAX)
	{
		(void) unlink(t36_file);
		globok = 0;
		(void) tbopen(t36_file, O_RDWR|O_CREAT, "O_RDWR|O_CREAT", MODEANY, SYSERROR, EMFILE);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	/* no need to close fd's: exit from child does it */

	DBUG_VOID_RETURN;
}

private void
test37()
{
	/* path longer than PATH_MAX */

	char	**longname;
	int	fd, err;
	int	pathok = 0;

	DBUG_ENTER("test37");

	testfail = 0;

	if ((fd = creat(t37_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t37_file, (unsigned long)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	longname = pathtoolng(t37_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	fd = tbopen(longname[0], O_RDONLY, "O_RDONLY", MODEANY,
				SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (fd >= 0)
		(void) close(fd);

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbopen(longname[1], O_RDONLY, "O_RDONLY", MODEANY,
				SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	clntoolng();
	(void) unlink(t37_file);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test38()
{
	DBUG_ENTER("test38");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test38()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test38");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	fd = tbopen(longname[0], O_WRONLY|O_CREAT, "O_WRONLY|O_CREAT",
			MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (fd >= 0)
		(void) close(fd);

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbopen(longname[1], O_WRONLY|O_CREAT, "O_WRONLY|O_CREAT",
			MODEANY, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private void
test39()
{
	/* O_CREAT not set and file does not exist */

	int	pathok = 0;

	DBUG_ENTER("test39");

	testfail = 0;

	if (eaccess(t39_file, F_OK) == 0 && unlink(t39_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t39_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbopen(t39_file, O_WRONLY, "O_WRONLY", MODEANY, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(2);
	}
	else
		(void) unlink(t39_file);

	DBUG_VOID_RETURN;
}

private void
test40()
{
	/* open file in a directory which does not exist */

	int	pathok = 0;

	DBUG_ENTER("test40");

	testfail = 0;

	if (eaccess(t40_dir, F_OK) == 0 && remove(t40_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t40_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbop_to(t40_file, O_WRONLY, "O_WRONLY", MODEANY,
		WAITTIME, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);


	DBUG_VOID_RETURN;
}

private void
test41()
{
	/* open "" */

	int	pathok = 0;

	DBUG_ENTER("test41");

	testfail = 0;

	globok = 0;
	(void) tbopen("", O_RDONLY, "O_RDONLY", MODEANY, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test42()
{
	/* open file with component of path prefix not a directory */

	int	fd, err;
	mode_t	mode = (S_IRWXU|S_IRWXG);
	int	pathok = 0;

	DBUG_ENTER("test42");

	testfail = 0;

	if ((fd = creat(t42_1_file, mode)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d (%s)",
			t42_1_file, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbopen(t42_2_file, O_RDWR|O_CREAT, "O_RDWR|O_CREAT", MODEANY,
		SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(2);
	}
	else
		(void) unlink(t42_2_file);

	(void) unlink(t42_1_file);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test43()
{
	DBUG_ENTER("test43");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test43()
{
	/* open FIFO for writing with no delay when not open for reading */

	int 	fd, err;
	int	pathok = 0;

	DBUG_ENTER("test43");

	testfail = 0;

	if (mkfifo(t43_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(%s) failed - errno %d (%s)",
			t43_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	fd = tbop_to(t43_file, O_WRONLY|O_NONBLOCK, "O_WRONLY|O_NONBLOCK",
		MODEANY, WAITTIME, SYSERROR, ENXIO);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(t43_file);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private void
test44()
{
	/* all the EROFS tests */

	DBUG_ENTER("test44");

	(void) cppair(ch_t44, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t44_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t44()
{
	char	*rofs;
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t44");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_ROFS is not defined");
		DBUG_VOID_RETURN;
	}

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = prog44;
	args[1] = t44_dir;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog44), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog44, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test45()
{
	int 	fd;
	int 	err;

	DBUG_ENTER("test45");

	if ((fd = creat(t45_file, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("could not creat() %s - errno %d (%s)", t45_file, err,
			errname(err));
		DBUG_VOID_RETURN;
	}

	if(write(fd, "1", 1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err, errname(err));
		DBUG_VOID_RETURN;
	}

	(void) close(fd);

	(void) cppair(ch_t45, NULLFN, 8 * WAITTIME, 0);

	(void) unlink(t45_file);

	DBUG_VOID_RETURN;
}

private void
ch_t45()
{
	char	*suid, *sgid;
	int 	err;
	char	*args[8];

	DBUG_ENTER("ch_t45");

	testfail = 0;

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	args[0] = prog45;
	args[1] = t45_file;
	args[2] = suid;
	args[3] = sgid;
	args[4] = "o";
	args[5] = vg_ident;
	args[6] = vg_dbug;
	args[7] = NULL;

	(void) tet_exec(execpath(prog45), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}

private void
test46()
{

	DBUG_ENTER("test46");

	(void) cppair(ch_t46, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t46()
{
	int 	err;
	char	*args[5];

	DBUG_ENTER("ch_t46");

	testfail = 0;

	args[0] = prog46;
	args[1] = t46_dir;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;

	(void) tet_exec(execpath(prog46), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test47()
{
	/* open returns fd that can be used in subsequent i/o calls -
		replaces part of test10 when testing XNFS */

	int	err, fd, rc;
	int	pathok = 0;

	DBUG_ENTER("test47");

	if (eaccess(t47_file, F_OK) == 0 && unlink(t47_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: cannot remove %s", t47_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	globok = 0;
	fd = tbopen(t47_file, O_RDWR|O_CREAT, "O_RDWR|O_CREAT", MODEANY,
		SUCCEED, NOERROR);
	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((rc = write(fd, wbuf, sizeof wbuf)) != sizeof wbuf)
	{
		if ((err = errno) == EBADF)
		{
			xx_rpt(FAILURE);
			in_rpt("fd %d returned by open() could not be used for i/o", fd);
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: write(%d) failed", fd);
		}
		valrpt(sizeof wbuf, rc, NOERROR, err);
		testfail = 1;
	}
	else
		PATH_TRACE;

	(void) close(fd);
	(void) unlink(t47_file);

	if (testfail == 0)
		PATH_XS_RPT(3);
}

#else /* TEST_XNFS */

private void
test47()
{
	DBUG_ENTER("test47");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test48_49()
{
	/* O_CREAT creates file with correct owner -
		replaces part of test10 when testing XNFS
	   test48: server does not perform user ID mapping
	   test49: server performs user ID mapping
	*/

	char	*file;

	DBUG_ENTER("test48_49");

	switch (tet_thistest) {
	case 48:
		file = t48_file;
		break;
	case 49:
		file = t49_file;
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected tet_thistest = %d",
			tet_thistest);
		DBUG_VOID_RETURN;
	}

	if (eaccess(file, F_OK) == 0 && unlink(file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", file);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t48_49, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t48_49()
{
	char	*file, *prog, *fuid, *suid;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t48_49");

	switch (tet_thistest) {
	case 48:
		if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
			DBUG_EXIT(A_NOMEM);
		fuid = suid;
		file = t48_file;
		prog = prog48;
		break;
	case 49:
		suid = "0";
		if ((fuid = newstr(uidtoa(xnfs_anon_uid))) == (char *) NULL)
			DBUG_EXIT(A_NOMEM);
		file = t49_file;
		prog = prog49;
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected tet_thistest = %d",
			tet_thistest);
		DBUG_VOID_RETURN;
	}

	args[0] = prog;
	args[1] = file;
	args[2] = suid;
	args[3] = fuid;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog, err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test48_49()
{
	DBUG_ENTER("test48_49");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test50()
{
	/* O_CREAT creates file with correct group when GrpID=True
		replaces part of test10 when testing XNFS
	*/

	DBUG_ENTER("test50");

	(void) cppair(ch_t50, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t50_mnt);

	DBUG_VOID_RETURN;
}

private void
ch_t50()
{
	char	*fsys, *sgid;
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t50");

	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	if ((fsys = tet_getvar("VSX_MOUNT_DEV")) == NULL || *fsys == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: VSX_MOUNT_DEV is not defined");
		DBUG_VOID_RETURN;
	}

	if (crdir(t50_mnt, MODEANY, (int) getuid(),
			(int) getgid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not make directory %s", t50_mnt);
		DBUG_VOID_RETURN;
	}

	args[0] = prog50;
	args[1] = sgid;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog50), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog50, err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test50()
{
	DBUG_ENTER("test50");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test51()
{
	/* O_CREAT creates file with correct group when GrpID=False
		replaces part of test10 when testing XNFS
	*/

	DBUG_ENTER("test51");

	(void) cppair(ch_t51, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t51()
{
	char	*sgid;
	int	err;
	char	*args[5];

	DBUG_ENTER("ch_t51");

	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		DBUG_EXIT(A_NOMEM);

	args[0] = prog51;
	args[1] = sgid;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = NULL;
	(void) tet_exec(execpath(prog51), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		prog51, err, errname(err));

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test51()
{
	DBUG_ENTER("test51");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test52()
{
	/* O_CREAT creates file with correct modes
		replaces part of test10 when testing XNFS
	*/

	DBUG_ENTER("test52");

	if (eaccess(t52_file, F_OK) == 0 && unlink(t52_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t52_file);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t52, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t52()
{

#define T52_MODEMASK	(S_IRWXU|S_IRWXG|S_IRWXO)
#define T52_UMASKVAL	(S_IRUSR|S_IWGRP|S_IXOTH)

	static mode_t	modes[] = {
		S_IRWXU|S_IRWXG|S_IRWXO,
		S_IXUSR|S_IWGRP|S_IROTH,
		UMASKVAL
	};

#define NMODES	(sizeof modes / sizeof modes[0])

	int err[NMODES], flag[NMODES];
	struct stat statb;
	int fail, fd, i;
	mode_t	um;
	int pathok = 0;

	DBUG_ENTER("ch_t52");

	um = T52_UMASKVAL;
	(void) umask(um);

	for (i = 0; i < NMODES; i++)
	{
		err[i] = flag[i] = 0;
		if ((fd = open(t52_file, O_RDWR|O_CREAT, modes[i])) == SYSERROR)
		{
			err[i] = errno;
			flag[i] |= 01;
			continue;
		}
		else
			PATH_TRACE;

		if (fstat(fd, &statb) == SYSERROR)
		{
			err[i] = errno;
			flag[i] |= 02;
		}
		else
		{
			PATH_TRACE;
			if ((statb.st_mode & T52_MODEMASK) != (~um & modes[i]))
				flag[i] |= 04;
			else
				PATH_TRACE;
		}

		(void) close(fd);
		if (unlink(t52_file) == SYSERROR)
			flag[i] |= 010;
		else
			PATH_TRACE;
	}

	fail = 0;
	for (i = 0; i < NMODES; i++)
	{
		if (i == 0)
			PATH_TRACE;
		if (flag[i])
			fail = 1;
	}

	if (fail)
	{
		xx_rpt(FAILURE);
		for (i = 0; i < NMODES; i++)
		{
			if (flag[i] & 01)
			{
				in_rpt("open(\"%s\", O_RDRW|O_CREAT, %#o) failed - errno %d(%s)",
					t52_file, modes[i], err[i],
					errname(err[i]));
				continue;
			}
			if (flag[i] & 02)
				in_rpt("fstat() on file discriptor returned by open() failed - errno %d(%s)",
					err[i], errname(err[i]));
			if (flag[i] & 04)
				in_rpt("open(\"%s\", O_RDWR|O_CREAT, %#o) did not set access bits correctly",
				t52_file, modes[i]);
			if (flag[i] & 010)
				in_rpt("could not remove file %s, created by open()",
				t52_file);
		}
	}
	else
		PATH_XS_RPT((NMODES * 4) + 1);

	DBUG_VOID_RETURN;

#undef NMODES

}

#else /* TEST_XNFS */

private void
test52()
{
	DBUG_ENTER("test52");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test53()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test53");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbopen(longname, O_WRONLY|O_CREAT, "O_WRONLY|O_CREAT",
		MODEANY, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test53()
{
	DBUG_ENTER("test53");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */
#if TEST_XNFS

private void
test54()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test54");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbopen(longname, O_WRONLY|O_CREAT, "O_WRONLY|O_CREAT",
		MODEANY, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test54()
{
	DBUG_ENTER("test54");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */
#if TEST_XNFS

private void
test55()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test55");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	fd = tbopen(longname, O_WRONLY|O_CREAT, "O_WRONLY|O_CREAT",
			MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test55()
{
	DBUG_ENTER("test55");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */
#if TEST_XNFS

private void
test56()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test56");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	fd = tbopen(longname, O_WRONLY|O_CREAT, "O_WRONLY|O_CREAT",
			MODEANY, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test56()
{
	DBUG_ENTER("test56");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private int
tbopen(tbpath, tboflag, oflagstr, tbmode, expret, experrno)
char *tbpath;
int tboflag;
char *oflagstr;
mode_t tbmode;
int expret, experrno;
{
	char	*reppath = tbpath;
	int	rval, ret, err;
	char	pathbuf[30];
	int	pathok = 0;

	DBUG_LENTER("tbopen", 2);

	errno = NOERROR;
	if ((tboflag & O_CREAT) != 0)
	{
		PATH_TRACE;
		rval = open(tbpath, tboflag, tbmode);
	}
	else
	{
		PATH_TRACE;
		rval = open(tbpath, tboflag);
	}

#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;

	if (expret == 0)
	{
		PATH_TRACE;
		if (rval >= 0 && rval < open_max)
		{
			PATH_FUNC_RPT(2);
			DBUG_RETURN(rval);
		}
	}
	else
	{
		PATH_TRACE;

		if (rval == SYSERROR && experrno == err)
		{
			globok = 0;
			ret = chk_creat_trunc(tbpath, tboflag);
			PATH_FUNC_TRACE;

			if(ret == T_OK)
			{
				PATH_FUNC_RPT(3);
				DBUG_RETURN(rval);
			}
		}
	}

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	if ((tboflag & O_CREAT) != 0)
		in_rpt("open(\"%s\", %s, %#o) did not give correct results", 
			reppath, oflagstr, (int)tbmode);
	else
		in_rpt("open(\"%s\", %s) did not give correct results",
			reppath, oflagstr);
	if (rval < SYSERROR || rval >= open_max)
		in_rpt("invalid return value %d: valid range %d to %d",
			rval, SYSERROR, open_max);

	if (ret == T_CREAT)
		in_rpt("on error file was created");
	else if (ret == T_TRUNC)
		in_rpt("on error file was truncated");

	valrpt(expret, rval, experrno, err);
	if (expret == SYSERROR && rval != SYSERROR)
		(void) close(rval);

	DBUG_RETURN(rval);
}

private int
tbop_to(tbpath, tboflag, oflagstr, tbmode, tbwait, expret, experrno)
char *tbpath;
int tboflag, tbwait;
char *oflagstr;
mode_t tbmode;
int expret, experrno;
{
	char	*reppath = tbpath;
	int	rval, err;
	char	pathbuf[30];
	int	pathok = 0;

	DBUG_LENTER("tbop_to", 2);

	SET_TIMEOUT(tbwait)

	errno = NOERROR;
	if ((tboflag & O_CREAT) != 0)
	{
		PATH_TRACE;
		rval = open(tbpath, tboflag, tbmode);
	}
	else
	{
		PATH_TRACE;
		rval = open(tbpath, tboflag);
	}
	err = errno;

	CLEAR_ALARM

#ifdef TESTBED
	rval = -1;
	if (err)	
		err++;
#endif
	
	if (expret == 0)
	{
		PATH_TRACE;
		if (rval >= 0 && rval < open_max)
		{
			PATH_FUNC_RPT(2);
			DBUG_RETURN(rval);
		}
	}
	else
	{
		PATH_TRACE;
		if (rval == SYSERROR && experrno == err)
		{
			PATH_FUNC_RPT(2);
			DBUG_RETURN(rval);
		}
	}

	/* Don't print PATH_MAX length path name */
	if (experrno == ENAMETOOLONG && strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	if ((tboflag & O_CREAT) != 0)
		in_rpt("open(\"%s\", %s, %#o) did not give correct results", 
			reppath, oflagstr, (int)tbmode);
	else
		in_rpt("open(\"%s\", %s) did not give correct results",
			reppath, oflagstr);
	if (err == EINTR && alrm_flag > 0)
		in_rpt("open call timed out after %d seconds", tbwait);
	if (rval < SYSERROR || rval >= open_max)
		in_rpt("invalid return value %d: valid range %d to %d",
			rval, SYSERROR, open_max);
	valrpt(expret, rval, experrno, err);
	if (expret == SYSERROR && rval != SYSERROR)
		(void) close(rval);

	DBUG_RETURN(rval);
}

private struct stat stpar1, stpar2;

private int
savparent(dir)
char *dir;
{
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("savparent");

	if (stat(dir, &stpar1) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			dir, err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(0);
}

private void
chkparent(dir)
char *dir;
{
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("chkparent");

	if (stat(dir, &stpar2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stpar2.st_ctime > stpar1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was updated");
	}
	else
		PATH_TRACE;

	if (stpar2.st_mtime > stpar1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was updated");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private int
chk_creat_trunc(path, flag)
char	*path;
int	flag;
{
	struct	stat buf;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("chk_creat_trunc");

	errno = 0;

	if ((flag & O_CREAT) && !(flag & O_EXCL))
	{
		if(eaccess(path, F_OK) != SYSERROR)
			DBUG_RETURN(T_CREAT);
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (flag & O_TRUNC)
	{
		if(stat(path, &buf) < 0)
		{
			err = errno;
			if(!testfail)
				xx_rpt(DELETION);
			in_rpt("stat(%s, buf) failed - errno %d (%s)",
				path, err, errname(err));
			DBUG_RETURN(T_NOK);
		}
		else if(buf.st_size == 0)
			DBUG_RETURN(T_TRUNC);
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(T_OK);
}
