/*
 *      SCCS:  POSIX.os/files/open/open_su.c Rel 4.4.1	(06/09/97)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#define MAIN
static	char sccsid[] = "@(#)POSIX.os/files/open/open_su Rel 4.4.1 (06/09/97)";
#ifdef	MAIN
char	*copyright[] = {
		"(C) Copyright 1992 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		open_su.c

PROJECT:	XNFS (X/OPEN NFS tests)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int tet_main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	Common subprogram for XNFS uid mapping tests.
	Exec'd by T.open.

	usage:
		progname file vg_ident vg_dbug

AUTHOR:		Andrew Dingwall, UniSoft Ltd.

DATE CREATED:	9 September 1992

MODIFICATIONS:

************************************************************************/

#include <std.h>
#include <dbug.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <tet_api.h>
#include "open_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef open
#endif


#define MODEANY		((mode_t)(S_IRWXU|S_IRWXG|S_IRWXO))
#define NOK		02
#define PATH_OK		04

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	void	set_pname();

private int	subtest25_30(),
		subtest31();

public	int
tet_main(argc, argv)
int argc;
char **argv;
{
	int rc;

	set_pname(argv[0]);

	if (argc < 1 || argc > 4)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return(OK);
	}
	if (argc >= 4)
		vg_dbug = argv[3];
	if (argc >= 3)
		vg_ident = argv[2];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug + 2);
	}

	switch (tet_thistest) {
	case 25:
	case 30:
		rc = subtest25_30(argv[1]);
		break;
	case 31:
		rc = subtest31(argv[1]);
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected tet_thistest = %d",
			tet_thistest);
		rc = NOK;
		break;
	}

	(void) unlink(argv[1]);

	return(rc);
}

private int
subtest25_30(file)
char *file;
{
	int err, fd, rc;
	int pathok = 0;

	DBUG_ENTER("subtest25_30");

	fd = open(file, O_RDWR | O_CREAT, MODEANY);
	err = errno;

	if (fd != SYSERROR || err != EACCES)
	{
		xx_rpt(FAILURE);
		valrpt(SYSERROR, fd, EACCES, err);
		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;

	rc = OK;
	if (pathok == 1)
		rc |= PATH_OK;

	DBUG_RETURN(rc);
}

private int
subtest31(file)
char *file;
{
	static int oflag[] = {
		O_RDWR | O_TRUNC,
		O_WRONLY | O_TRUNC
	};

	int Noflag = sizeof oflag / sizeof oflag[0];
	int i, err, fd, rc;
	int pathok = 0;

	DBUG_ENTER("subtest31");

	rc = OK;
	for (i = 0; i < Noflag; i++)
	{
		fd = open(file, oflag[i]);
		err = errno;
		if (fd != SYSERROR || err != EACCES)
		{
			if (rc == OK)
			{
				xx_rpt(FAILURE);
				rc = NOK;
			}
			in_rpt("open(%s, %#o) did not give correct results",
				file, oflag[i]);
			valrpt(SYSERROR, fd, EACCES, err);
		}
		else
			PATH_TRACE;
	}

	if (pathok == Noflag)
		rc |= PATH_OK;

	DBUG_RETURN(rc);
}

