/*
*      SCCS:  @(#)  POSIX.os/files/open/open_t16.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/open - open_t16 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		open_t16 - used by T.open in test16()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.open in test16().  This should be setuid/setgid
	  to VSX_UID1/VSX_GID1.
	  usage:
		open_t16 file uid gid vg_ident vg_dbug

	  where "file" is the file to be open()ed.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	20 Mar 1989
MODIFICATIONS:
		Geoff Clare, 19 June 1990
			Remove xs_rpt() and exit with NOK if not
			successful to cater for two-part test.

		David Sawyer
		Thu Aug 02 12:15:59 BST 1990
		Added resilience code.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Geoff Clare, 1 Dec 1994
			Missing arguments to an in_rpt() call.

************************************************************************/

#include <dbug.h>
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <signal.h>
#include <timeout.h>
#include <chktimes.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef open
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define NOK		02
#define PATH_OK		04

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	*errname();
extern	void	set_pname();

private int	testfail;

private	void	subtest16();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	uid_t uid;
	gid_t gid;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		exit(NOK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if ((uid = atoi(argv[2])) != geteuid() ||
		(gid = atoi(argv[3])) != getegid())
	{
		xx_rpt(DELETION);
		if (uid != getuid())
			in_rpt("deletion reason: uid is not equal to VSX_UID1");
		else
			in_rpt("deletion reason: gid is not equal to VSX_GID1");
		exit(NOK);
	}
	else
	{
		(void) setuid(uid);
		(void) setgid(gid);
		subtest16(argv[1]);
	}

	exit(OK);
	/*NOTREACHED*/
}


private void
subtest16(file)
char *file;
{
	/* O_TRUNC on FIFO: size, mode & owner unchanged */

	int		fd, fd_rd, fd_wr;
	int		err, err2;
	int		fail = 0;
	char		cbuf[16];
	struct stat	save_statb;
	struct stat	new_statb;
	int		pathok = 0;

	DBUG_ENTER("subtest16");

	testfail = 0;

	SET_TIMEOUT(WAITTIME)

	fd_rd = open(file, O_RDONLY|O_NONBLOCK);
	err = errno;
	if (alrm_flag == 0)
		fd_wr = open(file, O_WRONLY|O_NONBLOCK);
	err2 = errno;
	
	CLEAR_ALARM

	if (fd_rd < 0 || fd_wr < 0)
	{
		testfail++;
		xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("open() with O_NONBLOCK on FIFO timed out");
		else if (fd_rd < 0)
			in_rpt("open(O_RDONLY|O_NONBLOCK) failed on FIFO - errno %d (%s)",
				err, errname(err));
		else
			in_rpt("open(O_WRONLY|O_NONBLOCK) failed on FIFO - errno %d (%s)",
				err2, errname(err2));
	}
	else
		PATH_TRACE;

	if (testfail == 0)
	{
		if (write(fd_wr, cbuf, sizeof(cbuf)) != sizeof(cbuf))
		{
			err = errno;
			testfail++;
			xx_rpt(DELETION);
			in_rpt("deletion reason: write() failed - errno %d (%s)",
				err, errname(err));
		}
		else if (fstat(fd_wr, &save_statb) == SYSERROR)
		{
			err = errno;
			testfail++;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fstat() failed - errno %d (%s)",
				err, errname(err));
		}
		else
		{
			fd = open(file, O_WRONLY|O_TRUNC);
			if (fd == SYSERROR)
			{
				err = errno;
				testfail++;
				xx_rpt(FAILURE);
				in_rpt("open(\"%s\", O_WRONLY|O_TRUNC) failed - errno %d (%s)",
					file, err, errname(err));
			}
			else
				PATH_TRACE;
		}
	}

	if (testfail == 0 && fstat(fd, &new_statb) == SYSERROR)
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat() failed - errno %d (%s)",
			err, errname(err));
	}
	else
		PATH_TRACE;

	(void) close(fd);
	(void) close(fd_wr);
	(void) close(fd_rd);

	if (testfail > 0)
		DBUG_EXIT(NOK);

	if (new_statb.st_size != save_statb.st_size)
		fail = 01;
	else
		PATH_TRACE;

	if (new_statb.st_mode != save_statb.st_mode)
		fail |= 02;
	else
		PATH_TRACE;

	if (new_statb.st_uid != save_statb.st_uid)
		fail |= 04;
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 01)
			in_rpt("open() with O_WRONLY|O_TRUNC changed size");
		if (fail & 02)
			in_rpt("open() with O_WRONLY|O_TRUNC changed mode");
		if (fail & 04)
			in_rpt("open() with O_WRONLY|O_TRUNC changed owner");
		DBUG_EXIT(NOK);
	}

	if (pathok == 6)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}
