/*
*      SCCS:  @(#)  POSIX.os/files/open/open_t20_3.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/open - open_t20_3 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		open_t20_3 - used by T.open in test20_22()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.open to perform do_access() tests.
	  Must have PRV_SETID and PRV_ACCESS privileges.

	  usage:
		open_t20_3 testnum ident dbug
	
	 where testnum is the test number (20 - 22)

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	7 June 1989
MODIFICATIONS:	
		Geoff Clare, 15 Nov 1989
			New do_access() argument

		David Sawyer
		Thu Aug 02 12:16:17 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Geoff Clare, 26 May 1992
			Remove testing of O_RDONLY|O_TRUNC combinations.

		Geoff Clare, 6 Jan 1993
			Add sync point argument to do_access() calls.

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <xnfs.h>
#include "open_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef open
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	TBASE		20	/* first test number used */

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	**environ;

extern	void	set_pname();
extern	void	cln_access();
extern	char	*errname();

private	void	subtest20_3();
private	int	func();
private	int	cr_func();

private int	flagval;
private char	*flagstr;

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	set_pname(argv[0]);

	if (argc < 2 || argc > 4)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		return(OK);
	}
	if (argc >= 4)
		vg_dbug = argv[3];
	if (argc >= 3)
		vg_ident = argv[2];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	subtest20_3(atoi(argv[1]));

	return(OK);
	/* NOTREACHED */
}


private void
subtest20_3(testnum)
int testnum;
{
	int	fail;
	int	pathok = 0;

	DBUG_ENTER("subtest20_3");

	fail = 0;

	switch(testnum)
	{
	case TBASE:
		/* read access to file, search access to path prefix */
		flagval = O_RDONLY;
		flagstr = "O_RDONLY";
		globok = 0;
		if (do_access('f', (mode_t)S_IRUSR, (mode_t)S_IXUSR, func,
			WAITTIME, SP_T20_DO_ACCESS) != 0)
			fail++;
		else
			PATH_FUNC_TRACE;
		break;
		
	case TBASE+1:
		/* write access to file, search access to path prefix */
		flagval = O_WRONLY;
		flagstr = "O_WRONLY";
		globok = 0;
		if (do_access('f', (mode_t)S_IWUSR, (mode_t)S_IXUSR, func,
			WAITTIME, SP_T20_DO_ACCESS) != 0)
			fail++;
		else
			PATH_FUNC_TRACE;
		break;

	case TBASE+2:
		/* write & search access to dir, search access to path prefix */
		flagval = O_WRONLY|O_CREAT|O_EXCL;
		flagstr = "O_WRONLY|O_CREAT|O_EXCL";
		globok = 0;
		if (do_access('d', (mode_t)(S_IWUSR|S_IXUSR), (mode_t)S_IXUSR,
			cr_func, WAITTIME, SP_T20_DO_ACCESS) != 0)
			fail++;
		else
			PATH_FUNC_TRACE;
		break;

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed: %d", testnum);
		DBUG_VOID_RETURN;
	}

	cln_access();

	if (fail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private int
func(file, desc)
char *file, **desc;
{
	int 	fd, err;
	int	pathok = 0;

	DBUG_ENTER("func");

	errno = NOERROR;
	if ((fd = open(file, flagval)) < 0)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open(\"%s\", %s) failed - errno %d (%s)",
			file, flagstr, err, errname(err));
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	(void) close(fd);
	DBUG_RETURN(0);
}

private int
cr_func(dir, desc)
char *dir, **desc;
{
	int 	fd, err;
	char 	file[50];
	int	pathok = 0;

	DBUG_ENTER("cr_func");

	(void) strcat(strcpy(file, dir), "/open-t.20");

	errno = NOERROR;
	if ((fd = open(file, flagval, MODEANY)) < 0)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open(\"%s\", %s, %#o) failed - errno %d (%s)",
			file, flagstr, (int)MODEANY, err, errname(err));
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
		(void) unlink(file);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	(void) close(fd);
	(void) unlink(file);
	DBUG_RETURN(0);
}
