/*
*      SCCS:  @(#)  POSIX.os/files/open/open_t44.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/open - open_t40 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		open_t40 - used by T.open in test40()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.open to test open() errors on a read-only file
	  system.  Must have appropriate privileges to do mount/umount of
	  file system.

	  usage:
		open_t40 dir ident dbug

	  where dir is the directory that the VSX_ROFS file system is
	  to be mounted on.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Tue Nov  4 12:08:27 WET 1986
MODIFICATIONS:	D McCardle Tue Jun  9 08:27:34 WET 1987
			Modified xx_rpt(), xs_rpt().
		Geoff Clare, 22 Mar 1989
			Update for XPG3
		Geoff Clare, 15 Nov 1989
			New setuprofs() argument

		David Sawyer
		Thu Aug 02 12:16:30 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Neil Moses, 20 May 1992
			Check that on error open() does not create file.

		Geoff Clare, 26 May 1992
			Remove testing of O_RDONLY|O_TRUNC combinations.

		Neil Moses, 28 May 1992
			Check that on error open() does not truncate file.
			Included version of setuprofs() so that test can
			write to file, before making the file-system
			read-only.

		Neil Moses, 8 June 1992
			Check that time stamps are not changed.

                John-Paul, 29 July 1992
                        Mode parameter in calls to crfile() and crdir()
                        changed from 'int' to 'mode_t' type.

		Geoff Clare, 13 Oct 1992
			Changes for XNFS testing.

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <dbug.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <setprv.h>
#include <report.h>
#include <errno.h>
#include <xnfs.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef open
#endif


#define VSX_ROFS	"VSX_ROFS"
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	void	(*vsignal())();

private	char	*rofs;
private	char	*tidydir;
private	char	*tidyfile;
private	int	tsetuprofs();
private	int	tclnuprofs();
private	void	(*oldhdlr)();

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	**environ;
extern	void	savtimes();
extern	char	*chktimes();

extern	char	*errname();
extern	char	*testid();
extern	void	valrpt();
extern	void	set_pname();

private int	testfail;

private	void	subtest44();
private	int	savparent();
private	void	chkparent();
private	void	chkcreate();
private	void	chktrunc();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	set_pname(argv[0]);

	if (argc < 2 || argc > 4)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		return(OK);
	}
	if (argc >= 4)
		vg_dbug = argv[3];
	if (argc >= 3)
		vg_ident = argv[2];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	subtest44(argv[1]);

	return(OK);
	/* NOTREACHED */
}


private void
subtest44(mdir)
char	*mdir;
{
	int	fd, err;
	char	existfile[50];
	char	new1file[50];
	char	new2file[50];
	char	*errptr;
	int	pathok = 0;

	DBUG_ENTER("subtest44");

	testfail = 0;

	(void) strcat(strcpy(existfile, mdir), "/t.exist");
	(void) strcat(strcpy(new1file, mdir), "/t.new-1");
	(void) strcat(strcpy(new2file, mdir), "/t.new-2");

	globok = 0;
	if (tsetuprofs(mdir, existfile, 'f', (mode_t)MODEANY) != 0)
	{
		DBUG_VOID_RETURN;	/* reported in tsetuprofs() */
	}
	else
		PATH_FUNC_TRACE;

	/* open existing file for writing */

	globok = 0;
	savtimes(existfile);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((fd = open(existfile, O_WRONLY)) != SYSERROR || errno != EROFS)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("open(\"%s\", O_WRONLY) on read-only file system did not give correct results",
			existfile);
		valrpt(SYSERROR, fd, EROFS, err);
		if (fd >= 0)
			(void) close(fd);
	}
	else
	{
		PATH_TRACE;

		globok = 0;
		errptr = chktimes(existfile, 0);
		PATH_FUNC_TRACE;

		if(errptr != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful open() changed time stamps on %s",
				existfile);
			in_rpt("%s", errptr);
		}
		else
			PATH_TRACE;
	}

	globok = 0;
	savtimes(existfile);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((fd = open(existfile, O_RDWR)) != SYSERROR || errno != EROFS)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("open(\"%s\", O_RDWR) on read-only file system did not give correct results",
			existfile);
		valrpt(SYSERROR, fd, EROFS, err);
		if (fd >= 0)
			(void) close(fd);
	}
	else
	{
		PATH_TRACE;

		globok = 0;
		errptr = chktimes(existfile, 0);
		PATH_FUNC_TRACE;

		if(errptr != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful open() changed time stamps on %s",
				existfile);
			in_rpt("%s", errptr);
		}
		else
			PATH_TRACE;
	}

	/* create file on read-only file system */

	globok = 0;
	if (savparent(mdir) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	(void) sleep(1);

	errno = NOERROR;
	if ((fd = open(new1file, O_WRONLY|O_CREAT, MODEANY)) != SYSERROR
				|| errno != EROFS)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("open(\"%s\", O_WRONLY|O_CREAT, %#o) on read-only file system did not give correct results",
			new1file, MODEANY);
		valrpt(SYSERROR, fd, EROFS, err);
		if (fd != SYSERROR)
			(void) close(fd);
	}
	else
	{
		PATH_TRACE;

		globok = 0;
		chkcreate(new1file, "O_WRONLY|O_CREAT");
		PATH_FUNC_TRACE;
	}

	globok = 0;
	chkparent(mdir);
	PATH_FUNC_TRACE;
	(void) unlink(new1file);

	globok = 0;
	if (savparent(mdir) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	(void) sleep(1);

	errno = NOERROR;
	if ((fd = open(new2file, O_RDWR|O_CREAT, MODEANY)) != SYSERROR
				|| errno != EROFS)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("open(\"%s\", O_RDWR|O_CREAT, %#o) on read-only file system did not give correct results",
			new2file, MODEANY);
		valrpt(SYSERROR, fd, EROFS, err);
		if (fd != SYSERROR)
			(void) close(fd);
	}
	else
	{
		PATH_TRACE;

		globok = 0;
		chkcreate(new2file, "O_RDWR|O_CREAT");
		PATH_FUNC_TRACE;
	}

	globok = 0;
	chkparent(mdir);
	PATH_FUNC_TRACE;
	(void) unlink(new2file);

	/* O_TRUNC on read-only file system */

	globok = 0;
	savtimes(existfile);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((fd = open(existfile, O_WRONLY|O_TRUNC)) != SYSERROR
				|| errno != EROFS)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("open(\"%s\", O_WRONLY|O_TRUNC) on read-only file system did not give correct results",
			existfile);
		valrpt(SYSERROR, fd, EROFS, err);
		if (fd != SYSERROR)
			(void) close(fd);
	}
	else
	{
		PATH_TRACE;

		globok = 0;
		chktrunc(existfile, "O_WRONLY|O_TRUNC");
		PATH_FUNC_TRACE;

		globok = 0;
		errptr = chktimes(existfile, 0);
		PATH_FUNC_TRACE;

		if(errptr != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful open() changed time stamps on %s",
				existfile);
			in_rpt("%s", errptr);
		}
		else
			PATH_TRACE;
	}

	globok = 0;
	savtimes(existfile);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((fd = open(existfile, O_RDWR|O_TRUNC)) != SYSERROR
				|| errno != EROFS)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("open(\"%s\", O_RDWR|O_TRUNC) on read-only file system did not give correct results",
			existfile);
		valrpt(SYSERROR, fd, EROFS, err);
		if (fd != SYSERROR)
			(void) close(fd);
	}
	else
	{
		PATH_TRACE;

		globok = 0;
		chktrunc(existfile, "O_RDWR|O_TRUNC");
		PATH_FUNC_TRACE;

		globok = 0;
		errptr = chktimes(existfile, 0);
		PATH_FUNC_TRACE;

		if(errptr != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful open() changed time stamps on %s",
				existfile);
			in_rpt("%s", errptr);
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(27);

	(void) tclnuprofs(mdir, existfile);

	DBUG_VOID_RETURN;
}

/* These are copies from open.c, with messages changed to say "O_CREAT" */

private struct stat stpar1, stpar2;

private int
savparent(dir)
char *dir;
{
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("savparent");

	if (stat(dir, &stpar1) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			dir, err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(0);
}

private void
chkparent(dir)
char *dir;
{
	int 	err;
	int	pathok = 0;

	DBUG_ENTER("chkparent");

	if (stat(dir, &stpar2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stpar2.st_ctime > stpar1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was updated by O_CREAT on read-only file system");
	}
	else
		PATH_TRACE;

	if (stpar2.st_mtime > stpar1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was updated by O_CREAT on read-only file system");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
chkcreate(path, flagstr)
char	*path;
char	*flagstr;
{
	int	pathok = 0;

	DBUG_ENTER("chkcreate");

	if(eaccess(path, F_OK) != SYSERROR)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("open(\"%s\",%s) on error EROFS created file",
			path, flagstr);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
chktrunc(path, flagstr)
char	*path;
char	*flagstr;
{
	struct	stat buf;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("chktrunc");

	if(stat(path, &buf) < 0)
	{
		err = errno;
		if(!testfail)
			xx_rpt(DELETION);
		in_rpt("stat(%s, buf) failed - errno %d (%s)",
			path, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if(buf.st_size == 0)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("open(\"%s\",%s) on error EROFS truncated file",
			path, flagstr);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
sigtidy(sig)
int sig;
{
	(void) tclnuprofs(tidydir, tidyfile);

	if (oldhdlr == SIG_DFL)
		_exit(255);
	else if (oldhdlr != SIG_IGN)
		(*oldhdlr)(sig);
}

private	int
tclnuprofs(dirname, filename)
char *dirname, *filename;
{
	DBUG_ENTER("tclnuprofs");

	if (oldhdlr != NULL)
		(void) vsignal(SIGTERM, SIG_IGN);

	if (rofs == (char *) 0 || dirname == (char *) 0)
		DBUG_RETURN(1);

	if (filename != NULL)
	{
		(void) unmnt(rofs, dirname);
		if (mnt_rw(rofs, dirname) < 0)
		{
			(void) rmdir(dirname);
			DBUG_RETURN(1);
		}
	
		(void) remove(filename);
	}

	(void) unmnt(rofs, dirname);
	(void) rmdir(dirname);

	if (oldhdlr != NULL)
		(void) vsignal(SIGTERM, oldhdlr);

	DBUG_RETURN(0);
}

private	int
tsetuprofs(dirname, filename, filetype, mode)
char *dirname, *filename;
int  filetype;
mode_t mode;
{
	int	fd;
	int	err;
	char	*cp;
	int	pathok = 0;

	DBUG_LENTER("tsetuprofs", 2);

	if ((rofs = tet_getvar(VSX_ROFS)) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter %s is not set", VSX_ROFS);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (strncmp(rofs, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter %s is set to \"unsup\"", VSX_ROFS);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (setprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_MOUNT) failed");
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (vxmkdir(dirname) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can't make directory %s", dirname);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (mnt_rw(rofs, dirname) < 0)
	{
		(void) rmdir(dirname);
		xx_rpt(DELETION);
		in_rpt("deletion reason: mnt_rw(%s, %s) failed", rofs, dirname);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	/* Remove any junk files left from aborted ENOSPC tests */

	cp = (char *)malloc((size_t)(strlen(dirname) + sizeof("/fill.tmpX")));
	if (cp != NULL)
	{
		(void) strcat(strcpy(cp, dirname), "/fill.tmp1");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.tmp2");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.tmp3");
		(void) unlink(cp);
		(void) strcat(strcpy(cp, dirname), "/fill.dir");
		(void) rmdir(cp);
		free((void *)cp);
	}

	/* install signal handler to tidy up if timed out */
	tidydir = dirname;
	tidyfile = filename;
	oldhdlr = vsignal(SIGTERM, sigtidy);

	/* create requested file/directory */

	if (filename != NULL)
	{
		if (filetype == 'd')
		{
			if (vxmkdir(dirname) == SYSERROR)
			{
				(void) unmnt(rofs, dirname);
				(void) rmdir(dirname);
				xx_rpt(DELETION);
				in_rpt("deletion reason: can't create directory %s",
					filename);
				DBUG_RETURN(1);
			}
			else
			{
				PATH_TRACE;
				PATH_TRACE;
			}
		}
		else
		{
			if ((fd = creat(filename, mode)) == SYSERROR)
			{
				(void) unmnt(rofs, dirname);
				(void) rmdir(dirname);
				xx_rpt(DELETION);
				in_rpt("deletion reason: can't create file %s",
					filename);
				DBUG_RETURN(1);
			}
			else
				PATH_TRACE;

			if(write(fd, "1", 1) == SYSERROR)
			{
				err = errno;
				(void) unmnt(rofs, dirname);
				(void) rmdir(dirname);
				xx_rpt(DELETION);
				in_rpt("deletion reason: write(%d,\"1\",1) failed - errno %d (%s)", fd, err, errname(err));
				DBUG_RETURN(1);
			}
			else
				PATH_TRACE;

			(void) close (fd);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	if (unmnt(rofs, dirname) < 0)
	{
		(void) rmdir(dirname);
		xx_rpt(DELETION);
		in_rpt("deletion reason: unmnt(%s, %s) failed", rofs, dirname);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (mnt_ro(rofs, dirname) < 0)
	{
		(void) rmdir(dirname);
		xx_rpt(DELETION);
		in_rpt("deletion reason: mnt_ro(%s, %s) failed", rofs, dirname);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(9);

	DBUG_RETURN(0);
}
