/*
 *      SCCS:  @(#) POSIX.os/files/open/open_t50.c Rel 4.4.1 (06/09/97)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
static	char sccsid[] = "@(#)POSIX.os/files/open/open_t50.c Rel 4.4.1 (06/09/97)";
#endif	/* lint */

/***********************************************************************

NAME:		open_t50.c

PROJECT:	XNFS (X/OPEN NFS tests)

DESCRIPTION: 	setup routine for test50() subprogram

AUTHOR:		Andrew Dingwall, UniSoft Ltd.
DATE CREATED:	7 September 1992

MODIFICATIONS:
		Geoff Clare, 19 Oct 1993
			Delete subdirectory from T50_FILE.
			Use tet_fork() now that subtest does a setuid().

************************************************************************/

#include <std.h>
#include <dbug.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <report.h>
#include <tet_api.h>
#include <setprv.h>
#include <xnfs.h>
#include "open_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef open
#endif


#define WAITTIME	(5 * SPEEDFACTOR)

private char	*t50_mnt = 	T50_MNT;
private char	*t50_file = 	T50_FILE;
private gid_t	mygid;
private void	chfunc();

extern 	void	subtest5051();

public void
setup5051(gid)
gid_t gid;
{
	int pathok = 0;
	char *fsys;

	DBUG_ENTER("setup5051 for test 50");

	mygid = gid;

	if ((fsys = tet_getvar("VSX_MOUNT_DEV")) == NULL || *fsys == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: VSX_MOUNT_DEV not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (xnfs_mount(fsys, t50_mnt, XNFS_MOUNT_GRPID | XNFS_MOUNT_INTR) < 0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);
	(void) tet_fork(chfunc, NULLFN, 2*WAITTIME, 0);

	(void) xnfs_unmount(fsys, t50_mnt);

	DBUG_VOID_RETURN;
}

private void
chfunc()
{
	int pathok = 0;

	DBUG_ENTER("chfunc");

	PATH_FUNC_TRACE;
	subtest5051(t50_mnt, t50_file, mygid, (gid_t) -1, pathok, 1);

	DBUG_VOID_RETURN;
}
