/*
*      SCCS:  @(#)  POSIX.os/files/open/open_t9.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/open - open_t9 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		open_t9 - used by T.open in test9()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.open in test9().  This should be setuid/setgid
	  to VSX_UID1/VSX_GID1.
	  usage:
		open_t9 file uid gid vg_ident vg_dbug

	  where "file" is the file to be open()ed.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	20 Mar 1989
MODIFICATIONS:

		David Sawyer
		Thu Aug 02 12:16:50 BST 1990
		Added resilience code.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

************************************************************************/

#include <dbug.h>
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <report.h>
#include <errno.h>
#include <chktimes.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef open
#endif


#define	TMODE		(mode_t)(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	*errname();
extern	void	set_pname();

private	void	subtest9();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	uid_t uid;
	gid_t gid;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		return(OK);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if ((uid = atoi(argv[2])) != geteuid() ||
		(gid = atoi(argv[3])) != getegid())
	{
		xx_rpt(DELETION);
		if (uid != getuid())
			in_rpt("deletion reason: uid is not equal to VSX_UID1");
		else
			in_rpt("deletion reason: gid is not equal to VSX_GID1");
	}
	else
	{
		(void) setuid(uid);
		(void) setgid(gid);
		subtest9(argv[1]);
	}

	return(OK);
	/*NOTREACHED*/
}


private void
subtest9(file)
char	*file;
{
	char		*chkt;
	int		err;
	int		fd;
	int		fail = 0;
	struct stat	stbuf1, stbuf2;
	int		pathok = 0;

	DBUG_ENTER("subtest9");

	if (stat(file, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed on %s - errno %d (%s)",
			file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(file);
	PATH_FUNC_TRACE;

	if ((fd = open(file, O_RDWR|O_CREAT, TMODE)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open(\"%s\", O_RDWR|O_CREAT, %#o) failed - errno %d(%s)",
				file, TMODE, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (stat(file, &stbuf2) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed on %s - errno %d (%s)",
			file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf1.st_size != stbuf2.st_size)
		fail |= 01;
	else
		PATH_TRACE;

	if (stbuf1.st_uid != stbuf2.st_uid)
		fail |= 02;
	else
		PATH_TRACE;

	if (stbuf1.st_gid != stbuf2.st_gid)
		fail |= 04;
	else
		PATH_TRACE;

	if (stbuf1.st_mode != stbuf2.st_mode)
		fail |= 010;
	else
		PATH_TRACE;

	globok = 0;
	chkt = chktimes(file, 0);
	if (chkt != NULL)
		fail |= 020;
	else
		PATH_FUNC_TRACE;

	if (fail != 0)
	{
		xx_rpt(FAILURE);
		if (fail & 01)
			in_rpt("open(\"%s\", O_RDWR|O_CREAT, %#o) changed size of existing file",
				file, TMODE);
		if (fail & 02)
			in_rpt("open(\"%s\", O_RDWR|O_CREAT, %#o) changed uid of existing file",
				file, TMODE);
		if (fail & 04)
			in_rpt("open(\"%s\", O_RDWR|O_CREAT, %#o) changed gid of existing file",
				file, TMODE);
		if (fail & 010)
			in_rpt("open(\"%s\", O_RDWR|O_CREAT, %#o) changed mode of existing file",
				file, TMODE);
		if (fail & 020)
		{
			in_rpt("open(\"%s\", O_RDWR|O_CREAT, %#o) changed timestamps of existing file:",
				file, TMODE);
			in_rpt("%s", chkt);
		}
	}
	else
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}
