/*
*      SCCS:  @(#)  POSIX.os/files/open_X/open_X.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/open_X - T.open_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.open_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the open() system call, X/Open extensions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	21 Mar 1989
MODIFICATIONS:
		Geoff Clare, 29 May 1990
			Allow for unlimited OPEN_MAX

		Geoff Clare, 20 June 1990
			Give WARNING only in ETXTBSY test if call succeeds

		Peter Lee, Unisoft Ltd.,
		Thu Aug 02 11:46:32 BST 1990
		Added resilience code.

		Peter Skan, 30 Sept 1991
			Set the PRV_DEVICE privilege

		Geoff Clare, 21 July 1992
			Change pchild from int to pid_t.

		Geoff Clare, 22 Nov 1994
			Added declaration of tbop_to().

		Geoff Clare, 9 June 1995
			Allow VSX_NXIO_BLKDEV="unsup".

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<timeout.h>
#include	<string.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef open
#endif



#define NO_TESTS	3		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)


extern	char	*errname();
extern	void	valrpt();


public	int	no_tests = NO_TESTS;

private int	testfail;
private int	open_max;
private int	setupok;

private char	*t1_file =	"./open_X-t.1";
private char	*t3_file =	"./open_X-t.3";

private void	prepare_tests();
private void	test1();
private void	test2();
private void	test3();
private int	tbop_to();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private	void
prepare_tests()
{
	int i;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0 && errno != 0)
	{
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i,
				"sysconf(_SC_OPEN_MAX) failed during prep");
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(1);

	DBUG_VOID_RETURN;
}


private void
test1()
{
	/* open a FIFO with O_WRONLY|O_NONBLOCK when it is
	   already open for reading */

	int rfd, wfd, err;
	int	pathok = setupok;

	DBUG_ENTER("test1");

	testfail = 0;

	if (mkfifo(t1_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(%s) failed - errno %d (%s)",
			t1_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	rfd = tbop_to(t1_file, O_RDONLY|O_NONBLOCK, "O_RDONLY|O_NONBLOCK",
			MODEANY, WAITTIME, SUCCEED, NOERROR);
	if (testfail > 0)
	{
		(void) unlink(t1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	wfd = tbop_to(t1_file, O_WRONLY|O_NONBLOCK, "O_WRONLY|O_NONBLOCK",
			MODEANY, WAITTIME, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) close(rfd);
	if (wfd >= 0)
		(void) close(wfd);
	(void) unlink(t1_file);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	/* open a device which does not exist */

	char *chr, *blk;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	testfail = 0;

	if ((chr = tet_getvar("VSX_NXIO_CHRDEV")) == NULL ||
	    (blk = tet_getvar("VSX_NXIO_BLKDEV")) == NULL ||
	    *chr == '\0' || *blk == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("VSX_NXIO_CHRDEV and/or VSX_NXIO_BLKDEV not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (setprv(PRV_DEVICE) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_DEVICE) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbop_to(chr, O_RDONLY, "O_RDONLY", MODEANY,
			WAITTIME, SYSERROR, ENXIO);
	PATH_FUNC_TRACE;

	if (strncmp(blk, "unsup", (size_t)5) != 0)
	{
		globok = 0;
		(void) tbop_to(blk, O_RDONLY, "O_RDONLY", MODEANY,
				WAITTIME, SYSERROR, ENXIO);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


private void
test3()
{
	/* open executing pure text program with write permission */

	pid_t	pchild;
	int	pathok = setupok;

	DBUG_ENTER("test3");

	testfail = 0;

	globok = 0;
	if (pure_run(t3_file, WAITTIME, &pchild) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbop_to(t3_file, O_WRONLY, "O_WRONLY", MODEANY,
			WAITTIME, SYSERROR, ETXTBSY);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) killchild(pchild, WAITTIME);
	(void) unlink(t3_file);

	DBUG_VOID_RETURN;
}


private int
tbop_to(tbpath, tboflag, oflagstr, tbmode, tbwait, expret, experrno)
char *tbpath;
int tboflag, tbwait;
char *oflagstr;
mode_t tbmode;
int expret, experrno;
{
	int rval;
	int err;

	globok = 0;

	SET_TIMEOUT(tbwait)

	errno = NOERROR;
	if ((tboflag & O_CREAT) != 0)
		rval = open(tbpath, tboflag, tbmode);
	else
		rval = open(tbpath, tboflag);
	err = errno;

	CLEAR_ALARM

#ifdef TESTBED
	rval = -1;
	if (err)	
		err++;
#endif
	
	if (expret == 0)
	{
		if (rval >= 0 && (rval < open_max || open_max == -1))
		{
			globok = 1;
			return(rval);
		}
	}
	else
	{
		if (rval == SYSERROR && experrno == err)
		{
			globok = 1;
			return(rval);
		}
	}

	testfail++;
	if (experrno == ETXTBSY && rval != SYSERROR)
		xx_rpt(WARNING);
	else
		xx_rpt(FAILURE);
	if ((tboflag & O_CREAT) != 0)
		in_rpt("open(\"%s\", %s, %#o) did not give correct results", 
			tbpath, oflagstr, (int)tbmode);
	else
		in_rpt("open(\"%s\", %s) did not give correct results",
			tbpath, oflagstr);
	if (err == EINTR && alrm_flag > 0)
		in_rpt("open call timed out after %d seconds", tbwait);
	if (rval < SYSERROR || (rval >= open_max && open_max != -1))
	{
		in_rpt("invalid return value %d", rval);
		if (open_max != -1)
			in_rpt("valid range %d to %d", SYSERROR, open_max);
	}
	else
		valrpt(expret, rval, experrno, err);

	if (expret == SYSERROR && rval != SYSERROR)
		(void) close(rval);

	return(rval);
}
