/*
*      SCCS:  @(#)  POSIX.os/files/opendir/opendir.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/opendir - T.opendir Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.opendir

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the opendir() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	18 Apr 1989
MODIFICATIONS:	
		Geoff Clare, 8 June 1989: Add do_access() test.

		Geoff Clare, 21 Nov 1989
			Remove all references to d_ino.

		Steve Ratcliffe, 15 Feb 1990
			Add ENFILE test.
		
		Geoff Clare, 16 Mar 1990
			Remove FD_CLOEXEC test.
			Add empty string ENOENT test.

		Geoff Clare, 3 Apr 1990
			New interface with pathtoolng() and nametoolng().
		
		Geoff Clare, 29 May 1990
			Allow for unlimited OPEN_MAX.

		Peter Lee, Unisoft Ltd.,
		Thu Aug 02 09:55:44 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added
		
		Geoff Clare, 15 May 1992
			New VSX_OPENDIR_EMNFILE parameter.

                John-Paul, 29 July 1992
                        Mode parameter in calls to crfile() and crdir()
                        changed from 'int' to 'mode_t' type.

		David Sawyer, 5 November 1992
			Modifiations for XNFS tests.

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

		Geoff Clare, 3 Nov 1995
			Changed child timeout for do_access() test.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<string.h>
#include	<fcntl.h>
#include	<dirent.h>
#include	<limits.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<pathres.h>
#include	<pcts_max.h>
#include	<exitcodes.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"opendir_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef opendir
#endif


#define NO_TESTS	18
#define	WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define NULLDIR		(DIR *)0
#if	   TEST_XNFS
#define NOK		02
#define PATH_OK		04
#endif	/* TEST_XNFS */

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;

extern	char	*newstr();
extern	char	*errname();
#if	   TEST_XNFS
extern	char	*xnfs_nametoolng();
#else	/* TEST_XNFS */
extern	char	**nametoolng();
#endif	/* TEST_XNFS */
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	char	*newstr();
extern	char	*itoa();
extern	char	*execpath();

extern	int 	vguid1;
extern	int	vggid1;

public	int	no_tests = NO_TESTS;

private	char	*prog5 =	"opendir_t5";
private	char	*prog14 =	"opendir_t14";
#if	   TEST_XNFS
private	char	*prog_su =	"opendir_su";
#endif

private	char	*t1_file =	"opendir-t.1";
private	char	*t3_dir =	"opendir-d.3";
private	char	*t3_file =	"opendir-d.3/opendir-t.3";
private	char	*t3_fnam =	"opendir-t.3";
private	char	*t4_dir =	"opendir-d.4";
private	char	*t4a_file =	"opendir-d.4/opendir-t.4a";
private	char	*t4b_file =	"opendir-d.4/opendir-t.4b";
private char	*t7_dir =	"opendir-d.7";
private char	*t7_file =	"opendir-d.7/opendir-t.7";
#if	   TEST_XNFS
private char	*t7_2_file =	T7_2_FILE;
#endif	/* TEST_XNFS */
private char	*t8_file =	"opendir-t.8";
private	char	*t10_file =	"opendir-t.10";
private	char	*t11_file =	"opendir-t.11";
private char	*t12_ndir =	"opendir-t.12";
private char	*t12_file =	"opendir-t.12/opendir-t.12";
private	char	*t13_file =	"opendir-t.13";
private	char	*t14_dir =	"opendir-d.14";

private int	testfail;
private int	DIR_uses_fd = -1;
private	char	fnam[_POSIX_PATH_MAX];

private void	prepare_tests();
private void	cln_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();

private void	ch_t5();
#if	   TEST_XNFS
private void	ch_t7();
#endif	/* TEST_XNFS */
private void	ch_t13();
private void	ch_t14();

private DIR	*tbopendir();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 13, /* depends on test13 */
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 NULL, 0
};


private void
prepare_tests()
{
	DBUG_ENTER("prepare_tests");
	if(uids()!=0)
	{
		int i;
		for(i=1; i <= no_tests; i++)
			tet_delete(i, "uids() failed");
	}

	(void) umask((mode_t)0);
	DBUG_VOID_RETURN;
}


private void
cln_tests()
{
	char **p;

	DBUG_ENTER("cln_tests");

	(void) rmdir(t1_file);
	for (p = portfnam; *p != NULL; p++)
	{
		(void) strcpy(fnam, *p);
		(void) strcat(fnam, "/idfile");
		(void) unlink(fnam);
		(void) rmdir(*p);
	}
	(void) unlink(t3_file);
	(void) rmdir(t3_dir);
	(void) unlink(t4a_file);
	(void) unlink(t4b_file);
	(void) rmdir(t4_dir);
	(void) chmod(t7_dir, MODEANY);
	(void) rmdir(t7_file);
	(void) rmdir(t7_dir);
	(void) rmdir(t8_file);
	(void) rmdir(t10_file);
	(void) unlink(t12_ndir);
	(void) rmdir(t13_file);
	(void) rmdir(t14_dir);

	DBUG_VOID_RETURN;
}


private void
test1()
{
	/* pathname resolution */

	char		*p1, *p2;
	DIR		*dirp1, *dirp2;
	struct dirent	*dp1, *dp2;
	int		npairs, i, fail, fd;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_CREAT);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_CREAT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		if (strcmp(p2, "/") != 0 && strcmp(p1, "/") != 0)
		{
			/* Put an identifying file in directory */

			(void) strcpy(fnam, p2);
			(void) strcat(fnam, "/idfile");
			if ((fd = crfile(fnam, MODEANY,
				(int)geteuid(), (int)getegid())) == SYSERROR)
			{
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("deletion reason: could not create file \"%s\"",
					fnam);
				break;
			}
			else
				PATH_TRACE;
			(void) close(fd);
		}
		else
		{
			fnam[0] = '\0';
			PATH_TRACE;
		}

		globok = 0;
		dirp1 = tbopendir(p1, NOERROR);
		PATH_FUNC_TRACE;
		if (dirp1 == NULL)
		{
			PATH_TRACE;
			PATH_TRACE;
			continue;
		}

		globok = 0;
		dirp2 = tbopendir(p2, NOERROR);
		PATH_FUNC_TRACE;
		if (dirp2 == NULL)
		{
			PATH_TRACE;
			(void) closedir(dirp1);
			continue;
		}

#ifdef TESTBED2
		(void) readdir(dirp2);
#endif

		/* check both open directories "read" the same */

		fail = 0;
		while ((dp1 = readdir(dirp1)) != NULL)
		{
			if ((dp2 = readdir(dirp2)) == NULL)
				fail++;
			else if (strcmp(dp1->d_name, dp2->d_name) != 0)
				fail++;
		}
		if (readdir(dirp2) != NULL)
			fail++;

		if (fail)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("opendir() did not resolve \"%s\" and \"%s\" to the same directory",
				p1, p2);
		}
		else
			PATH_TRACE;

		(void) closedir(dirp1);
		(void) closedir(dirp2);
		if (fnam[0] != '\0')
			(void) unlink(fnam);
	}

	if (testfail == 0)
		PATH_XS_RPT(1+ (4 * npairs));
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames */

	char		**p;
	DIR		*dirp;
	struct dirent	*dp;
	int		fd, i;
	int		loop1,loop2;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	for (i = 0, p = portfnam, loop1 = 0 ; *p != NULL; p++, i++, loop1++)
	{
		if (crdir(*p, MODEANY,
				(int)geteuid(), (int)getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
		else 
			PATH_TRACE;
	}

	for (i = 0, p = portfnam, loop2 = 0 ; *p != NULL; p++, i++, loop2++)
	{
		(void) sprintf(fnam, "%s/id.%d", *p, i);
		if ((fd = crfile(fnam, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create file \"%s\"", fnam);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		(void) close(fd);
	}

#ifdef TESTBED2
	(void) unlink(fnam);
#endif

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		if (i == 0)
			PATH_TRACE;

		dirp = tbopendir(*p, NOERROR);
		if (dirp == NULL)
			continue;

		while ((dp = readdir(dirp)) != NULL)
		{
			if (strncmp(dp->d_name, "id.", (size_t)3) == 0)
				break;
		}
		if (dp == NULL || atoi(&dp->d_name[3]) != i)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("opendir(\"%s\") did not open the correct directory",
				*p);
		}
		(void) closedir(dirp);
	}

	if (testfail == 0)
		PATH_XS_RPT(loop1 + loop2 + 1);

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		(void) sprintf(fnam, "%s/id.%d", *p, i);
		(void) unlink(fnam);
		(void) rmdir(*p);
	}

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* opendir() opens the specified directory */

	DIR		*dirp;
	struct dirent	*dp;
	int		fd;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	if (crdir(t3_dir, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t3_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fd = crfile(t3_file, MODEANY,
		(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) close(fd);

	globok = 0;
	dirp = tbopendir(t3_dir, NOERROR);
	if (dirp == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	t3_fnam = "junk";
#endif

	while ((dp = readdir(dirp)) != NULL)
		if (strcmp(dp->d_name, t3_fnam) == 0)
			break;

	if (dp == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("opendir(\"%s\") did not open the correct directory",
			t3_dir);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) closedir(dirp);
	(void) unlink(t3_file);
	(void) rmdir(t3_dir);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* opendir() positions stream at first directory entry */

	DIR		*dirp;
	struct dirent	*dp;
	int		fd;
	char		*firstnam;
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	if (crdir(t4_dir, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t4_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create two directory entries - in case there are no "." & ".." */
	if ((fd = crfile(t4a_file, MODEANY,
		(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t4a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	if ((fd = crfile(t4b_file, MODEANY,
		(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t4b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		

	globok = 0;
	dirp = tbopendir(t4_dir, NOERROR);
	if (dirp == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	dp = readdir(dirp);
	if (dp == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("readdir() failed on stream returned by opendir(\"%s\")",
			t4_dir);
		(void) closedir(dirp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	firstnam = newstr(dp->d_name);
	if (firstnam == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: malloc() failed");
		(void) closedir(dirp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rewinddir(dirp);

	dp = readdir(dirp);
	if (dp == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: readdir() failed after rewinddir()");
		(void) closedir(dirp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	firstnam[0] = '\0';
#endif

	if (strcmp(dp->d_name, firstnam) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("opendir(\"%s\") did not position stream at first directory entry",
			t4_dir);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(8);

	free((void *) firstnam);
	(void) closedir(dirp);
	(void) unlink(t4a_file);
	(void) unlink(t4b_file);
	(void) rmdir(t4_dir);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* read access on dir/search access on path prefix granted */

	int waittime;

	DBUG_ENTER("test5");

	waittime = WAITTIME * (5 + sysconf(_SC_NGROUPS_MAX)/4);

	(void) cppair(ch_t5, NULLFN, waittime, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	err;
	char	*args[4];

	DBUG_ENTER("ch_t5");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog5);
	args[1] = vg_ident;
	args[2] = vg_dbug;
	args[3] = NULL;
	(void) tet_exec(execpath(prog5), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog5), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* pathname is empty string */
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	globok = 0;
	(void) tbopendir("", ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}


private void
test7()
{
	/* search permission denied for component of path prefix */

	int	err;
	mode_t	mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
	int	pathok = 0;
#if	   TEST_XNFS
	int	rc;
#endif	/* TEST_XNFS */


	DBUG_ENTER("test7");

	testfail = 0;

	if (crdir(t7_dir, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t7_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (crdir(t7_file, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t7_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (chmod(t7_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t7_dir, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbopendir(t7_file, EACCES);
	PATH_FUNC_TRACE;

	(void) chmod(t7_dir, MODEANY);
	(void) rmdir(t7_file);
	(void) rmdir(t7_dir);

#if	   TEST_XNFS

	if (tet_sync(SP_T7_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t7, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
	{
		testfail++;
	}
	else if (rc & PATH_OK)
	{
		PATH_TRACE;
	}

	if (tet_sync(SP_T7_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS
private void
ch_t7()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t7");

	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t7_2_file;
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

#endif	/* TEST_XNFS */


private void
test8()
{
	/* open directory which denies read permission */

	mode_t mode = (mode_t)(S_IWUSR|S_IXUSR|S_IWGRP|S_IXGRP|S_IWOTH|S_IXOTH);
	int	pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

	if (crdir(t8_file, mode, (int)geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbopendir(t8_file, EACCES);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) rmdir(t8_file);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test9()
{
	DBUG_ENTER("test9");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test9()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	DIR	*dirp;
	int	pathok = 0;

	DBUG_ENTER("test9");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (crdir(longname[0], MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", longname[0]);
	}
	else
	{
		PATH_TRACE;
		globok = 0;
		dirp = tbopendir(longname[0], NOERROR);
		PATH_FUNC_TRACE;
		if (dirp != NULL)
			(void) closedir(dirp);
	}

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbopendir(longname[1], ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) rmdir(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test10()
{
	/* path longer than PATH_MAX */

	char	**longname;
	DIR	*dirp;
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	if (crdir(t10_file, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	longname = pathtoolng(t10_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	dirp = tbopendir(longname[0], NOERROR);
	PATH_FUNC_TRACE;
	if (dirp != NULL)
		(void) closedir(dirp);
	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbopendir(longname[1], ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	clntoolng();
	(void) rmdir(t10_file);

	DBUG_VOID_RETURN;
}

private void
test11()
{
	/* directory does not exist */
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	if (eaccess(t11_file, F_OK) == 0 && remove(t11_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbopendir(t11_file, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	/* component of path is not a directory */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test12");

	testfail = 0;

	if ((fd = crfile(t12_ndir, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t12_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbopendir(t12_file, ENOTDIR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t12_ndir);

	DBUG_VOID_RETURN;
}

private void
test13()
{
	/* OPEN_MAX/PCTS_OPEN_MAX/EMFILE etc. */

	char	*cp;
	DIR	*dirp;
	int	open_max, i, *fdtab;
	int	pathok = 0;

	DBUG_ENTER("test13");

	testfail = 0;

	cp = tet_getvar("VSX_OPENDIR_EMNFILE");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_OPENDIR_EMNFILE is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_OPENDIR_EMNFILE is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0)
	{
		if (errno == 0)
		{
			PATH_TRACE;
			open_max = 256;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("sysconf(_SC_OPEN_MAX) returned non-positive value");
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

	fdtab = (int *)malloc((size_t)(open_max * sizeof(int)));
	if (fdtab == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc(%ld) failed", (long)(open_max * sizeof(int)));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < open_max; i++)
		fdtab[i] = fcntl(i, F_GETFD);

	if ((dirp = opendir(".")) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("opendir(\".\") failed");
		free((void *)fdtab);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	DIR_uses_fd = 0;
	for (i = 0; i < open_max; i++)
	{
		if (fdtab[i] == SYSERROR && fcntl(i, F_GETFD) != SYSERROR)
		{
			DIR_uses_fd = 1;
			break;
		}
	}

	free((void *)fdtab);
	(void) closedir(dirp);
	
	if (DIR_uses_fd == 0)
	{
		xx_rpt(UNTESTED);
		in_rpt("DIR is not implemented using a file descriptor");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t13_file, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(6);

	(void) cppair(ch_t13, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t13_file);

	DBUG_VOID_RETURN;
}

private void
ch_t13()
{
	long	open_max, testmax = PCTS_OPEN_MAX;
	int	i;
	int	pathok = 0;
	int 	loop1;

	DBUG_ENTER("ch_t13");

	PATH_FUNC_TRACE;

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0)
	{
		if (errno == 0)
		{
			PATH_TRACE;
			open_max = PCTS_OPEN_MAX+1;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("sysconf(_SC_OPEN_MAX) failed");
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

	if (open_max < PCTS_OPEN_MAX)
		testmax = open_max;

	/* open lesser of OPEN_MAX and PCTS_OPEN_MAX files successfully */

	for (loop1 = i = n_open_fds(); i < testmax; i++)
	{
		globok = 0;
		if (tbopendir(t13_file, NOERROR) == NULL)
		{
			DBUG_VOID_RETURN;
		}
		else if (i == loop1)
			PATH_FUNC_TRACE;
	}

	/* if OPEN_MAX is testable, next open gives EMFILE */

	if (open_max <= PCTS_OPEN_MAX)
	{
		globok = 0;
		(void) tbopendir(t13_file, EMFILE);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	/* no need to close fd's: exit from child does it */

	DBUG_VOID_RETURN;
}

private void
test14()
{
	char *cp;

	DBUG_ENTER("test14");

	cp = tet_getvar("VSX_OPENDIR_EMNFILE");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_OPENDIR_EMNFILE is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_OPENDIR_EMNFILE is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else if (DIR_uses_fd == -1)
	{
		xx_rpt(DELETION);
		in_rpt("could not determine whether DIR is implemented using a file descriptor");
		DBUG_VOID_RETURN;
	}
	else if (DIR_uses_fd == 0)
	{
		xx_rpt(UNTESTED);
		in_rpt("DIR is not implemented using a file descriptor");
		DBUG_VOID_RETURN;
	}
	else if (crdir(t14_dir, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t14_dir);
		DBUG_VOID_RETURN;
	}
	else
		(void) cppair(ch_t14, NULLFN, 8 * WAITTIME, 0);

	(void) rmdir(t14_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t14()
{
	char	*suid, *sgid;
	int 	err;
	char	*args[8];

	DBUG_ENTER("ch_t14");

	testfail = 0;

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		exit(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	args[0] = execpath(prog14);
	args[1] = t14_dir;
	args[2] = suid;
	args[3] = sgid;
	args[4] = "d";
	args[5] = vg_ident;
	args[6] = vg_dbug;
	args[7] = NULL;

	(void) tet_exec(execpath(prog14), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %d (%s)", err, errname(err));

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
test15()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test15");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbopendir(longname, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test15()
{
	DBUG_ENTER("test15");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test16()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test16");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbopendir(longname, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test16()
{
	DBUG_ENTER("test16");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test17()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test17");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (crdir(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid()) == SYSERROR)
	{
		testfail++;
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname);
	}
	else
	{
		globok = 0;
		(void) tbopendir(longname, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) rmdir(longname);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test17()
{
	DBUG_ENTER("test17");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test18()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test18");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (crdir(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid()) == SYSERROR)
	{
		testfail++;
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname);
	}
	else
	{
		globok = 0;
		(void) tbopendir(longname, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) rmdir(longname);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test18()
{
	DBUG_ENTER("test18");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private DIR *
tbopendir(tbpath, experrno)
char *tbpath;
int  experrno;
{
	char	*reppath = tbpath;
	DIR	*rval;
	int	err;
	char	pathbuf[30];
	int	pathok = 0;

	DBUG_LENTER("tbopendir", 2);

	errno = 0;
	rval = opendir(tbpath);
	err = errno;
	globok = 0;

#ifdef TESTBED
	if (rval != NULL)
	{
		(void) closedir(rval);
		rval = NULL;
	}
	else
		rval = opendir(".");
#endif
#ifdef TESTBED2
	err++;
#endif


	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
	}

	if (experrno == NOERROR && rval == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("opendir(\"%s\") failed - errno %d (%s)",
			reppath, err, errname(err));
	}
	else if (experrno != NOERROR)
	{
		if (rval != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("opendir(\"%s\") succeeded unexpectedly",
				reppath);
			(void) closedir(rval);
		}
		else if (experrno != err)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("opendir(\"%s\") did not set errno correctly",
				reppath);
			in_rpt("expected: %d (%s), observed: %d (%s)",
				experrno, errname(experrno), err, errname(err));
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(rval);
}
