/*
*      SCCS:  @(#)  POSIX.os/files/opendir/opendir_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/opendir - opendir_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		opendir_su - used by T.opendir in privileged tests

PROJECT:	XNFS (X/OPEN XNFS tests)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.opendir to perform privileged tests.

	  usage:
		opendir_su testnum file file ident dbug

	  where testnum is the number of the test being executed, file and
	  file parameters to be passed to the test.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		David Sawyer
DATE CREATED:	5 November 1992
MODIFICATIONS:

************************************************************************/
#include 	<std.h>
#include 	<stdlib.h>
#include 	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<dirent.h>
#include 	<dbug.h>
#include	<errno.h>
#include 	<sys/stat.h>
#include 	<report.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	<sysdep.h>

#ifdef UNDEF_MACROS
#undef opendir
#endif



#if	   TEST_XNFS
#define NOK		02
#define PATH_OK		04
#endif	/* TEST_XNFS */

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	**environ;
extern	void	set_pname();

#if	   TEST_XNFS
private	int	subtest7();
#endif	/* TEST_XNFS */



public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int testnum;
	int exitval = OK;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return exitval;
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}


#if	   TEST_XNFS
	if (geteuid() != XNFS_MAPPED_UID)
	{
		xx_rpt(DELETION);
		in_rpt("Effective UID is not %d", XNFS_MAPPED_UID);
		return exitval;
	}
#endif	/* TEST_XNFS */

	testnum = atoi(argv[1]);
	switch (testnum)
	{

#if	   TEST_XNFS

	case 7:
		exitval = subtest7(argv[2]);
		break;

#endif	/* TEST_XNFS */

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed: %d", testnum);
		break;
	}

	return exitval;
}


#if	   TEST_XNFS
private int
subtest7(file)
char *file;
{
	int err, rc;
	DIR *ret;
	int pathok = 0;

	DBUG_ENTER("subtest7");

	ret = opendir(file);
	err = errno;

	if (ret != (DIR *) NULL || err != EACCES)
	{
		xx_rpt(FAILURE);
		if(ret != NULL)
			in_rpt("opendir(): did not return NULL as expected");
		if(err != EACCES)
			in_rpt("opendir(): errno = %d expected EACCES", err);
		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;

	rc = OK;
	if (pathok == 1)
		rc |= PATH_OK;

	DBUG_RETURN(rc);
}
#endif	/* TEST_XNFS */
