/*
*      SCCS:  @(#)  POSIX.os/files/pathconf/pathcf_t19.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/pathconf - pathcf_t19 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		pathcf_t19 - used by T.pathconf in test19()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.pathconf to perform do_access() test.
	  Must have PRV_SETID and PRV_ACCESS privileges.

	  usage:
		pathcf_t19 ident dbug

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	8 June 1989
MODIFICATIONS:	
		Geoff Clare, 15 Nov 1989
			New do_access() argument

		Peter Lee, Unisoft Ltd.,
		Wed Aug 01 13:34:12 BST 1990
		Added resilience code.
		Split main into main and t19_func_call for
			path checking and debugging purposes.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env, Repaired broken t19_func_call.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file
		
		Geoff Clare, 13 Nov 1992
			Changes for XNFS.

		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#if TEST_XNFS
#include <xnfs.h>
#endif /* TEST_XNFS */
#include "pathconf_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef pathconf
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	**environ;

extern	void	set_pname();
extern	void	cln_access();
extern	char	*errname();

private	int	t19_func();
private	void	t19_func_call();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	set_pname(argv[0]);

	if (argc < 1 || argc > 3)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );
		return OK;
	}
	if (argc >= 3)
		vg_dbug = argv[2];
	if (argc >= 2)
		vg_ident = argv[1];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	t19_func_call();

	return OK;
}

private void
t19_func_call()
{
	int	pathok = 0;

	DBUG_ENTER("t19_func_call");

	globok = 0;
	if (do_access('f', (mode_t)0, (mode_t)S_IXUSR, t19_func, WAITTIME, SP_T19_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(1);
	}
	
	cln_access();

	DBUG_TET_EXIT(OK);
	/* NOTREACHED */
}

private int
t19_func(file, desc)
char *file, **desc;
{
	long ret;
	int err;
	int	pathok = 0;

	DBUG_ENTER("t19_func");

	errno = 0;
	if ((ret = pathconf(file, _PC_LINK_MAX)) == -1 && errno != 0)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("pathconf(\"%s\", _PC_LINK_MAX) did not give correct results",
			file);
		in_rpt("expected: return value != -1");
		in_rpt("      OR: return value == -1, errno == 0 (NO ERROR)");
		in_rpt("observed: return value == %ld, errno == %d (%s)",
			ret, err, errname(err));
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(0);
}
