/*
*      SCCS:  @(#)  POSIX.os/files/pathconf/pathconf.c	Rel 4.4.4	    (07/19/99)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/pathconf - T.pathconf Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.pathconf

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the pathconf() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	25 Apr 1989
MODIFICATIONS:	Joseph Nave, UniSoft Ltd.  Thu Jun  1 14:32:07 BST 1989
		tests using crfile() were extended to test with mkfifo()

		Geoff Clare, 9 June 1989
			Add do_access() test.

		Geoff Clare, 20 July 1989
			Add _PC_NAME_MAX, _PC_PATH_MAX, _PC_NO_TRUNC tests

		Geoff Clare, 4 Apr 1990
			New interface with pathtoolng() and nametoolng().
		
		Geoff Clare, 9 May 1990
			Use VSX_INVALID_PC parameter
		
		Geoff Clare, 1 June 1990
			Allow -1 return in "value >= _POSIX_XYZ_MAX" tests
		
		Geoff Clare, 28 June 1990
			Add EINVAL "not associated" tests
		
		Geoff Clare, 5 July 1990
			Allow for unlimited NAME_MAX in "applies to files
			in directory" test.

		Peter Lee, Unisoft Ltd.,
		Wed Aug 01 12:38:31 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ, repaired tbpathfunc
			resilience.

		Stuart Boutell, 22 Mar 1991
			Fixed short malloc problem in test 20

		Stuart Boutell, 2 Apr 1991
			Added missing DBUG_VOID_RETURN to chkval()

		Stuart Boutell, 3 Apr 1991
			Corrected path trace errors in tests 6, 14, and 18.

		Geoff Clare, 8 Nov 1991
			Corrected path trace error in test 20

		Geoff Clare, 28 Apr 1992
			Corrected path trace error in test 16

		Geoff Clare, 12 June 1992
			Added tests 37, 38 and 39.

                John-Paul, 29 July 1992
                        Mode parameter in calls to crfile() and crdir()
                        changed from 'int' to 'mode_t' type.

		David Sawyer, 6 November 1992
			Modifications for XNFS tests.

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

		Geoff Clare, 3 Nov 1995
			Changed child timeout for do_access() test.

		Geoff Clare, 11 June 1999
			In chkval() value must be equal, unless XPG3.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<string.h>
#include	<limits.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<pathres.h>
#include	<pcts_max.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"pathconf_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef pathconf
#endif


#define NO_TESTS	43
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define T_REG		001
#define T_FIFO		002
#define T_DIR		004
#define T_TERM		010
#if	   TEST_XNFS
#define NOK		02
#define PATH_OK		04
#endif	/* TEST_XNFS */

extern	char	*errname();
extern	char	*itoa();
#if	   TEST_XNFS
extern	char	*xnfs_nametoolng();
#else	/* TEST_XNFS */
extern	char	**nametoolng();
#endif	/* TEST_XNFS */
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	void	valrpt();
extern	char	*execpath();

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;

public	int	no_tests = NO_TESTS;

private int	testfail;
#if	  !TEST_XNFS
private char	*tty_name;
#endif	/* TEST_XNFS */
private int	setupok;

private	char	*prog19 =	"pathcf_t19";
#if	   TEST_XNFS
private	char	*prog_su =	"pathcf_su";
#endif	/* TEST_XNFS */
private	char	*tfile =	"pathconf-reg";
#if	  !TEST_XNFS
private	char	*tfifo =	"pathconf-fifo";
private char	*t13_file =	"pathconf-t.13";
private char	*t14_file =	"pathconf-t.14";
private char	*t15_file =	"pathconf-t.15";
#endif	/* TEST_XNFS */
private char	*t21_file =	"pathconf-t.21";
private char	*t23_dir =	"pathconf-d.23";
private char	*t23_file =	"pathconf-d.23/pathconf-t.23";
#if	   TEST_XNFS
private char	*t23_2_file =	T23_2_FILE;
#endif	/* TEST_XNFS */
private	char	*t25_file =	"pathconf-t.25";
private char	*t26_dir =	"pathconf-d.26";
private char	*t26_file =	"pathconf-d.26/pathconf-t.26";
private	char	*t27_file =	"pathconf-t.27";
private char	*t29_ndir =	"pathconf-t.29";
private char	*t29_file =	"pathconf-t.29/pathconf-t.29";

private void	prepare_tests();
private void	cln_tests();
#if	  !TEST_XNFS
private void	chkval();
private void	chkeql();
#endif	/* TEST_XNFS */
private void	minval();
private void	t_einval();

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();
private void	test30();
private void	test31();
private void	test32();
private void	test33();
private void	test34();
private void	test35();
private void	test36();
private void	test37();
private void	test38();
private void	test39();
private void	test40();
private void	test41();
private void	test42();
private void	test43();

private void	ch_t19();
#if	   TEST_XNFS
private void	ch_t23();
#endif	/* TEST_XNFS */

private long	tbpathconf();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	  test2, 2,
	  test3, 3,
	  test4, 4,
	  test5, 5,
	 test6, 6,
	  test7, 7,
	  test8, 8,
	  test9, 9,
	  test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	 test29, 29,
	 test30, 30,
	 test31, 31,
	 test32, 32,
	 test33, 33,
	 test34, 34,
	 test35, 35,
	 test36, 36,
	 test37, 37,
	 test38, 38,
	 test39, 39,
	 test40, 40,
	 test41, 41,
	 test42, 42,
	 test43, 43,
	NULL, 0
};


private void
prepare_tests()
{
	int fd, i;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	if ((fd = crfile(tfile, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, "could not create test file");
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}
	
#if	   TEST_XNFS

	PATH_TRACE;

#else	/* TEST_XNFS */

	(void) unlink(tfifo);
	if (mkfifo(tfifo, MODEANY) == SYSERROR)
	{
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, "could not create test fifo");
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);
	}
	
	tty_name = tet_getvar("VSX_TTYNAME");
	if (tty_name != NULL && *tty_name == '\0')
		tty_name = NULL;

#endif	/* TEST_XNFS */

	PATH_SETUP_RPT(2);

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	char **p;

	DBUG_ENTER("cln_tests");

	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);
	(void) unlink(tfile);
#if	  !TEST_XNFS
	(void) unlink(tfifo);
	(void) unlink(t13_file);
	(void) unlink(t14_file);
	(void) unlink(t15_file);
#endif	/* TEST_XNFS */
	(void) chmod(t23_dir, MODEANY);
	(void) unlink(t23_file);
	(void) rmdir(t23_dir);
	(void) unlink(t25_file);
	(void) unlink(t29_ndir);

	DBUG_VOID_RETURN;
}


private void
test1()
{
	/* pathname resolution */

	char	*p1, *p2;
	long	sav_link_max, sav_name_max, sav_path_max,
		sav_pipe_buf, sav_chown_res, sav_no_trunc;
	long	new_link_max, new_name_max, new_path_max,
		new_pipe_buf, new_chown_res, new_no_trunc;
	int	npairs, i, fail;
	int	pathok = setupok;

	DBUG_ENTER("test1");

	testfail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_CREAT);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_CREAT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check that the same information is produced from both pathnames.
	   Hardly conclusive, but it's all we can do. */

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		fail = testfail;

		globok = 0;
		sav_link_max = tbpathconf(p2, _PC_LINK_MAX, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		if (testfail > fail)
			continue;

		globok = 0;
		sav_name_max = tbpathconf(p2, _PC_NAME_MAX, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		if (testfail > fail)
			continue;
	
		globok = 0;
		sav_path_max = tbpathconf(p2, _PC_PATH_MAX, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		if (testfail > fail)
			continue;
	
#if	   TEST_XNFS
		sav_pipe_buf = 0;
		PATH_TRACE;
#else	/* TEST_XNFS */
		globok = 0;
		sav_pipe_buf = tbpathconf(p2, _PC_PIPE_BUF, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		if (testfail > fail)
			continue;
#endif	/* TEST_XNFS */

		globok = 0;
		sav_chown_res = tbpathconf(p2, _PC_CHOWN_RESTRICTED,
					SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		if (testfail > fail)
			continue;

		globok = 0;
		sav_no_trunc = tbpathconf(p2, _PC_NO_TRUNC, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		if (testfail > fail)
			continue;


		globok = 0;
		new_link_max = tbpathconf(p1, _PC_LINK_MAX, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		if (testfail > fail)
			continue;

		globok = 0;
		new_name_max = tbpathconf(p1, _PC_NAME_MAX, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		if (testfail > fail)
			continue;

		globok = 0;
		new_path_max = tbpathconf(p1, _PC_PATH_MAX, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		if (testfail > fail)
			continue;

#if	   TEST_XNFS
		new_pipe_buf = sav_pipe_buf;
		PATH_TRACE;
#else	/* TEST_XNFS */
		globok = 0;
		new_pipe_buf = tbpathconf(p1, _PC_PIPE_BUF, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		if (testfail > fail)
			continue;
#endif	/* TEST_XNFS */

		globok = 0;
		new_chown_res = tbpathconf(p1, _PC_CHOWN_RESTRICTED,
					SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		if (testfail > fail)
			continue;

		globok = 0;
		new_no_trunc = tbpathconf(p1, _PC_NO_TRUNC, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		if (testfail > fail)
			continue;

#ifdef TESTBED2
		new_link_max++;
#endif

		if (new_link_max != sav_link_max ||
		    new_name_max != sav_name_max ||
		    new_path_max != sav_path_max ||
		    new_pipe_buf != sav_pipe_buf ||
		    new_chown_res != sav_chown_res ||
		    new_no_trunc != sav_no_trunc)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("pathconf() did not resolve \"%s\" and \"%s\" to the same file",
				p1, p2);
				
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(1 + 13 * npairs);
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames */

	char	**p;
	int	fd;
	int	pathok = setupok;
	int	loop1 = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	/* There is no way to test the files are distinguished correctly
	   - just check the calls succeed */

	for (p = portfnam, loop1 = 0; *p != NULL; p++, loop1++)
	{
		if ((fd = crfile(*p, MODEANY,
				(int)geteuid(), (int)getegid())) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create file \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		(void) close(fd);

		globok = 0;
		(void) tbpathconf(*p, _PC_LINK_MAX, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;

		(void) unlink(*p);
	}

	if (testfail == 0)
		PATH_XS_RPT(2 * loop1);
	
	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* pathconf(path, _PC_LINK_MAX) >= _POSIX_LINK_MAX */
	int	pathok = setupok;

	DBUG_ENTER("test3");

	testfail = 0;

#ifdef TESTBED2
#undef _POSIX_LINK_MAX
#define	_POSIX_LINK_MAX (pathconf(".", _PC_LINK_MAX)+1)
#endif

	globok = 0;
	minval(tfile, _PC_LINK_MAX, (long)_POSIX_LINK_MAX, "_POSIX_LINK_MAX");
	PATH_FUNC_TRACE;

#if	   TEST_XNFS
	PATH_TRACE;
#else	/* TEST_XNFS */
	globok = 0;
	minval(tfifo, _PC_LINK_MAX, (long)_POSIX_LINK_MAX, "_POSIX_LINK_MAX");
	PATH_FUNC_TRACE;
#endif	/* TEST_XNFS */
	
	globok = 0;
	minval(".", _PC_LINK_MAX, (long)_POSIX_LINK_MAX, "_POSIX_LINK_MAX");
	PATH_FUNC_TRACE;
	
#if	   TEST_XNFS
	PATH_TRACE;
#else	/* TEST_XNFS */
	globok = 0;
	minval("/", _PC_LINK_MAX, (long)_POSIX_LINK_MAX, "_POSIX_LINK_MAX");
	PATH_FUNC_TRACE;
#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test4()
{
	DBUG_ENTER("test4");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test4()
{
	/* pathconf(path, _PC_LINK_MAX) >= LINK_MAX */
	int	pathok = setupok;

	DBUG_ENTER("test4");

	testfail = 0;

#ifdef TESTBED2
#undef LINK_MAX
#define	LINK_MAX (pathconf(".", _PC_LINK_MAX)+1)
#endif

#ifdef LINK_MAX

	globok = 0;
	chkval(tfile, _PC_LINK_MAX, (long)LINK_MAX, "LINK_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval(tfifo, _PC_LINK_MAX, (long)LINK_MAX, "LINK_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval(".", _PC_LINK_MAX, (long)LINK_MAX, "LINK_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval("/", _PC_LINK_MAX, (long)LINK_MAX, "LINK_MAX");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("LINK_MAX is not defined");

#endif

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test5()
{
	DBUG_ENTER("test5");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test5()
{
	/* pathconf(path, _PC_MAX_CANON) >= _POSIX_MAX_CANON */
	int	pathok = setupok;

	DBUG_ENTER("test5");

	testfail = 0;

	if (tty_name == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TTYNAME is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
#undef _POSIX_MAX_CANON
#define	_POSIX_MAX_CANON (pathconf(tty_name, _PC_MAX_CANON)+1)
#endif

	globok = 0;
	minval(tty_name, _PC_MAX_CANON,
		(long)_POSIX_MAX_CANON, "_POSIX_MAX_CANON");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test6()
{
	DBUG_ENTER("test6");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test6()
{
	/* pathconf(path, _PC_MAX_CANON) >= MAX_CANON */
	int	pathok = setupok;

	DBUG_ENTER("test6");

	testfail = 0;

#ifdef TESTBED2
#undef MAX_CANON
#define	MAX_CANON (pathconf(tty_name, _PC_MAX_CANON)+1)
#endif

#ifdef MAX_CANON

	if (tty_name == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TTYNAME is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	chkval(tty_name, _PC_MAX_CANON, (long)MAX_CANON, "MAX_CANON");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("MAX_CANON is not defined");

#endif

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test7()
{
	DBUG_ENTER("test7");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test7()
{
	/* pathconf(path, _PC_MAX_INPUT) >= _POSIX_MAX_INPUT */
	int	pathok = setupok;

	DBUG_ENTER("test7");

	testfail = 0;

	if (tty_name == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TTYNAME is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
#undef _POSIX_MAX_INPUT
#define	_POSIX_MAX_INPUT (pathconf(tty_name, _PC_MAX_INPUT)+1)
#endif

	globok = 0;
	minval(tty_name, _PC_MAX_INPUT,
		(long)_POSIX_MAX_INPUT, "_POSIX_MAX_INPUT");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test8()
{
	DBUG_ENTER("test8");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test8()
{
	/* pathconf(path, _PC_MAX_INPUT) >= MAX_INPUT */
	int	pathok = setupok;

	DBUG_ENTER("test8");

	testfail = 0;

#ifdef TESTBED2
#undef MAX_INPUT
#define	MAX_INPUT (pathconf(tty_name, _PC_MAX_INPUT)+1)
#endif

#ifdef MAX_INPUT

	if (tty_name == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TTYNAME is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	chkval(tty_name, _PC_MAX_INPUT, (long)MAX_INPUT, "MAX_INPUT");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("MAX_INPUT is not defined");

#endif

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
test9()
{
	/* pathconf(path, _PC_NAME_MAX) >= _POSIX_NAME_MAX */
	int	pathok = setupok;

	DBUG_ENTER("test9");

	testfail = 0;

#ifdef TESTBED2
#undef _POSIX_NAME_MAX
#define	_POSIX_NAME_MAX (pathconf(".", _PC_NAME_MAX)+1)
#endif

	globok = 0;
	minval(".", _PC_NAME_MAX, (long)_POSIX_NAME_MAX, "_POSIX_NAME_MAX");
	PATH_FUNC_TRACE;

#if	   TEST_XNFS
	PATH_TRACE;
#else	/* TEST_XNFS */
	globok = 0;
	minval("/", _PC_NAME_MAX, (long)_POSIX_NAME_MAX, "_POSIX_NAME_MAX");
	PATH_FUNC_TRACE;
#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test10()
{
	DBUG_ENTER("test10");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test10()
{
	/* pathconf(path, _PC_NAME_MAX) >= NAME_MAX */
	int	pathok = setupok;

	DBUG_ENTER("test10");

	testfail = 0;

#ifdef TESTBED2
#undef NAME_MAX
#define	NAME_MAX (pathconf(".", _PC_NAME_MAX)+1)
#endif

#ifdef NAME_MAX

	globok = 0;
	chkval(".", _PC_NAME_MAX, (long)NAME_MAX, "NAME_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval("/", _PC_NAME_MAX, (long)NAME_MAX, "NAME_MAX");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("NAME_MAX is not defined");

#endif

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
test11()
{
	/* pathconf(path, _PC_PATH_MAX) >= _POSIX_PATH_MAX */
	int	pathok = setupok;

	DBUG_ENTER("test11");

	testfail = 0;

#ifdef TESTBED2
#undef _POSIX_PATH_MAX
#define	_POSIX_PATH_MAX (pathconf(".", _PC_PATH_MAX)+1)
#endif

	globok = 0;
	minval(".", _PC_PATH_MAX, (long)_POSIX_PATH_MAX, "_POSIX_PATH_MAX");
	PATH_FUNC_TRACE;

#if	   TEST_XNFS
	PATH_TRACE;
#else	/* TEST_XNFS */
	globok = 0;
	minval("/", _PC_PATH_MAX, (long)_POSIX_PATH_MAX, "_POSIX_PATH_MAX");
	PATH_FUNC_TRACE;
#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test12()
{
	DBUG_ENTER("test12");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test12()
{
	/* pathconf(path, _PC_PATH_MAX) >= PATH_MAX */
	int	pathok = setupok;

	DBUG_ENTER("test12");

	testfail = 0;

#ifdef TESTBED2
#undef PATH_MAX
#define	PATH_MAX (pathconf(".", _PC_PATH_MAX)+1)
#endif

#ifdef PATH_MAX

	globok = 0;
	chkval(".", _PC_PATH_MAX, (long)PATH_MAX, "PATH_MAX");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval("/", _PC_PATH_MAX, (long)PATH_MAX, "PATH_MAX");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("PATH_MAX is not defined");

#endif

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test13()
{
	/* pathconf(path, _PC_PIPE_BUF) >= _POSIX_PIPE_BUF */
	int	pathok = setupok;
	int err;

	DBUG_ENTER("test13");

	testfail = 0;

	(void) remove(t13_file);
	if (mkfifo(t13_file, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("mkfifo(\"%s\", %#o) failed - errno %d (%s)",
			t13_file, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
#undef _POSIX_PIPE_BUF
#define	_POSIX_PIPE_BUF (pathconf(".", _PC_PIPE_BUF)+1)
#endif

	globok = 0;
	minval(t13_file, _PC_PIPE_BUF,
			(long)_POSIX_PIPE_BUF, "_POSIX_PIPE_BUF");
	PATH_FUNC_TRACE;

	globok = 0;
	minval(".", _PC_PIPE_BUF, (long)_POSIX_PIPE_BUF, "_POSIX_PIPE_BUF");
	PATH_FUNC_TRACE;

	globok = 0;
	minval("/", _PC_PIPE_BUF, (long)_POSIX_PIPE_BUF, "_POSIX_PIPE_BUF");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) unlink(t13_file);

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test14()
{
	DBUG_ENTER("test14");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test14()
{
	/* pathconf(path, _PC_PIPE_BUF) >= PIPE_BUF */

	int err;
	int	pathok = setupok;

	DBUG_ENTER("test14");

	testfail = 0;

#ifdef TESTBED2
#undef PIPE_BUF
#define	PIPE_BUF (pathconf(".", _PC_PIPE_BUF)+1)
#endif

#ifdef PIPE_BUF

	(void) remove(t14_file);
	if (mkfifo(t14_file, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("mkfifo(\"%s\", %#o) failed - errno %d (%s)",
			t14_file, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	chkval(t14_file, _PC_PIPE_BUF, (long)PIPE_BUF, "PIPE_BUF");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval(".", _PC_PIPE_BUF, (long)PIPE_BUF, "PIPE_BUF");
	PATH_FUNC_TRACE;

	globok = 0;
	chkval("/", _PC_PIPE_BUF, (long)PIPE_BUF, "PIPE_BUF");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);
	
	(void) unlink(t14_file);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("PIPE_BUF is not defined");

#endif

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */



#if	   TEST_XNFS

private void
test15()
{
	DBUG_ENTER("test15");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test15()
{
	/* _PC_PIPE_BUF on a directory applies to a FIFO in the directory */

	long rval1, rval2;
	int err;
	int	pathok = setupok;

	DBUG_ENTER("test15");

	testfail = 0;

	(void) remove(t15_file);
	if (mkfifo(t15_file, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("mkfifo(\"%s\", %#o) failed - errno %d (%s)",
			t15_file, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	rval1 = tbpathconf(t15_file, _PC_PIPE_BUF, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	if (testfail > 0)
		DBUG_VOID_RETURN;

	globok = 0;
	rval2 = tbpathconf(".", _PC_PIPE_BUF, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	if (testfail > 0)
		DBUG_VOID_RETURN;

#ifdef TESTBED2
	rval1++;
#endif

	if (rval1 != rval2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("pathconf(\".\", _PC_PIPE_BUF) did not return same value as pathconf(\"%s\", _PC_PIPE_BUF)",
			t15_file);
		in_rpt("expected: %ld, observed: %ld", rval1, rval2);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) unlink(t15_file);

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test16()
{
	DBUG_ENTER("test16");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test16()
{
	/* pathconf(path, _PC_CHOWN_RESTRICTED) == _POSIX_CHOWN_RESTRICTED */
	int	pathok = setupok;

	DBUG_ENTER("test16");

	testfail = 0;

#ifdef TESTBED2
#undef _POSIX_CHOWN_RESTRICTED
#define	_POSIX_CHOWN_RESTRICTED (pathconf(".", _PC_CHOWN_RESTRICTED)+1)
#endif

#ifdef _POSIX_CHOWN_RESTRICTED

	globok = 0;
	chkeql(tfile, _PC_CHOWN_RESTRICTED,
		(long)_POSIX_CHOWN_RESTRICTED, "_POSIX_CHOWN_RESTRICTED");
	PATH_FUNC_TRACE;

	globok = 0;
	chkeql(tfifo, _PC_CHOWN_RESTRICTED,
		(long)_POSIX_CHOWN_RESTRICTED, "_POSIX_CHOWN_RESTRICTED");
	PATH_FUNC_TRACE;

	globok = 0;
	chkeql(".", _PC_CHOWN_RESTRICTED,
		(long)_POSIX_CHOWN_RESTRICTED, "_POSIX_CHOWN_RESTRICTED");
	PATH_FUNC_TRACE;

	globok = 0;
	chkeql("/", _PC_CHOWN_RESTRICTED,
		(long)_POSIX_CHOWN_RESTRICTED, "_POSIX_CHOWN_RESTRICTED");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("_POSIX_CHOWN_RESTRICTED is not defined");

#endif

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */




#if	   TEST_XNFS

private void
test17()
{
	DBUG_ENTER("test17");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test17()
{
	/* pathconf(path, _PC_NO_TRUNC) == _POSIX_NO_TRUNC */
	int	pathok = setupok;

	DBUG_ENTER("test17");

	testfail = 0;

#ifdef TESTBED2
#undef _POSIX_NO_TRUNC
#define	_POSIX_NO_TRUNC (pathconf(".", _PC_NO_TRUNC)+1)
#endif

#ifdef _POSIX_NO_TRUNC

	globok = 0;
	chkeql(".", _PC_NO_TRUNC, (long)_POSIX_NO_TRUNC, "_POSIX_NO_TRUNC");
	PATH_FUNC_TRACE;

	globok = 0;
	chkeql("/", _PC_NO_TRUNC, (long)_POSIX_NO_TRUNC, "_POSIX_NO_TRUNC");
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("_POSIX_NO_TRUNC is not defined");

#endif

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test18()
{
	DBUG_ENTER("test18");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test18()
{
	/* pathconf(path, _PC_VDISABLE) == _POSIX_VDISABLE */

	char rval;
	int	pathok = setupok;

	DBUG_ENTER("test18");

	testfail = 0;

#ifdef TESTBED2
#undef _POSIX_VDISABLE
#define	_POSIX_VDISABLE (pathconf(tty_name, _PC_VDISABLE)+1)
#endif

#ifdef _POSIX_VDISABLE

	if (tty_name == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TTYNAME is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* can't use chkeql() - casting _POSIX_VDISABLE to (long) will
	   sign extend */

	globok = 0;
	rval = tbpathconf(tty_name, _PC_VDISABLE, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	if (testfail > 0)
		DBUG_VOID_RETURN;

	if (rval != (char)_POSIX_VDISABLE)
	{
		xx_rpt(FAILURE);
		in_rpt("pathconf(\"%s\", _PC_VDISABLE) did not return _POSIX_VDISABLE",
			tty_name);
		in_rpt("value returned: %d, _POSIX_VDISABLE: %d",
			rval, (char)_POSIX_VDISABLE);
	}
	else
		PATH_XS_RPT(2);

#else

	xx_rpt(UNSUPPORTED);
	in_rpt("_POSIX_VDISABLE is not defined");

#endif

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
test19()
{
	/* search access to path prefix granted */

	int waittime;

	DBUG_ENTER("test19");

	waittime = WAITTIME * (5 + sysconf(_SC_NGROUPS_MAX)/4);

	(void) cppair(ch_t19, NULLFN, waittime, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t19()
{
	int	err;
	char	*args[4];

	DBUG_ENTER("ch_t19");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog19);
	args[1] = vg_ident;
	args[2] = vg_dbug;
	args[3] = NULL;
	(void) tet_exec(execpath(prog19), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog19), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test20()
{
	/* _PC_NAME_MAX on a directory applies to files in the directory */
	/* _PC_NO_TRUNC on a directory applies to files in the directory */

	char	*longname;
	int	fd, err, len, i;
	int	pathok = setupok;

	DBUG_ENTER("test20");

	testfail = 0;

	errno = 0;
	len = pathconf(".", _PC_NAME_MAX);
	if (len == SYSERROR)
	{
		if (errno == 0)
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("pathconf(\".\", _PC_NAME_MAX) returned -1 with errno 0");
		}
		else
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("pathconf(\".\", _PC_NAME_MAX) failed - errno %d (%s)",
				err, errname(err));
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (len > PCTS_NAME_MAX)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("NAME_MAX > PCTS_NAME_MAX");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((longname = (char *)malloc((size_t)(len+2))) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc(%u) failed", len+1);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < len+1; i++)
		longname[i] = '0' + (i+1)%10;
	longname[i] = '\0';

	errno = 0;
	if (pathconf(".", _PC_NO_TRUNC) >= 0)
	{
		/* _POSIX_NO_TRUNC is defined */

		PATH_TRACE;

		errno = 0;
		fd = creat(longname, MODEANY);
		if (fd >= 0 || errno != ENAMETOOLONG)
		{
			err = errno;
			testfail++;
			xx_rpt(FAILURE);
			in_rpt("creat() with file name longer than NAME_MAX did not fail with ENAMETOOLONG");
			valrpt(SYSERROR, fd, ENAMETOOLONG, err);
			(void) unlink(longname);
		}
		else
			PATH_TRACE;
	}
	else if (errno == 0)
	{
		/* _POSIX_NO_TRUNC is not defined */

#if	   TEST_XNFS

		/* this is tested in all the pathname resolution tests */
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_NO_TRUNC is not defined on the client");
		DBUG_VOID_RETURN;

#else	/* TEST_XNFS */

		fd = creat(longname, MODEANY);
		if (fd < 0)
		{
			err = errno;
			testfail++;
			xx_rpt(FAILURE);
			in_rpt("creat() with file name longer than NAME_MAX failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		longname[len] = '\0';
		if (eaccess(longname, F_OK) != 0)
		{
			testfail++;
			xx_rpt(FAILURE);
			in_rpt("creat() with file name longer than NAME_MAX did not create file with truncated name");
		}
		else
			PATH_TRACE;
		(void) unlink(longname);

#endif	/* TEST_XNFS */
	}
	else
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("pathconf(\".\", _PC_NO_TRUNC) failed - errno %d (%s)",
			err, errname(err));
	}

	if (testfail == 0)
		PATH_XS_RPT(5);

	free((void *)longname);

	DBUG_VOID_RETURN;
}

private void
test21()
{
	/* _PC_PATH_MAX on current directory gives the maximum length
	   of relative path names */

	char	**longname;
	int	fd, err;
	int	pathok = setupok;

	DBUG_ENTER("test21");

	testfail = 0;

	/* pathtoolng() contains a pathconf(".", _PC_PATH_MAX) call */
	globok = 0;
	longname = pathtoolng(t21_file);
	PATH_FUNC_TRACE;
	if (longname == NULL)
		DBUG_VOID_RETURN;
	
	if (longname[1] == NULL)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("PATH_MAX > PCTS_PATH_MAX or == -1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	fd = creat(longname[1], MODEANY);
	if (fd >= 0 || errno != ENAMETOOLONG)
	{
		err = errno;
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("creat() with relative path longer than PATH_MAX did not fail with ENAMETOOLONG");
		valrpt(SYSERROR, fd, ENAMETOOLONG, err);
	}
	else
		PATH_TRACE;

	clntoolng();
	(void) unlink(t21_file);

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}



#if	   TEST_XNFS

private void
test22()
{
	DBUG_ENTER("test22");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test22()
{
	/* invalid name */

	char *cp;
	long rval;
	int err;
	int invname = -1;
	int	pathok = setupok;

	DBUG_ENTER("test22");

	testfail = 0;

	cp = tet_getvar("VSX_INVALID_PC");
	if (cp != NULL && *cp != '\0')
		invname = atoi(cp);

	/* don't use tbpathconf(): it allows EINVAL or success */
	errno = 0;
	rval = pathconf(".", invname);
	if (rval != -1 || errno != EINVAL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("pathconf(\".\", %d) did not give correct results",
			invname);
		in_rpt("expected: return value == -1, errno == %d (EINVAL)",
			EINVAL);
		in_rpt("observed: return value == %ld, errno == %d (%s)",
			rval, err, errname(err));

	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */


private void
test23()
{
	/* search permission denied */

	int	fd, err;
	mode_t	mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
	int	pathok = setupok;
#if	   TEST_XNFS
	int	rc;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test23");

	testfail = 0;

	if (crdir(t23_dir, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t23_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((fd = crfile(t23_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t23_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (chmod(t23_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t23_dir, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbpathconf(t23_file, _PC_LINK_MAX, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) chmod(t23_dir, MODEANY);
	(void) unlink(t23_file);

#if	   TEST_XNFS

	if (tet_sync(SP_T23_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t23, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
	{
		testfail++;
	}
	else if (rc & PATH_OK)
	{
		PATH_TRACE;
	}

	if (tet_sync(SP_T23_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */


	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}



#if	   TEST_XNFS
private void
ch_t23()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t23");

	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t23_2_file;
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

#endif	/* TEST_XNFS */



#if	   TEST_XNFS

private void
test24()
{
	DBUG_ENTER("test24");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test24()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test24");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname[0], MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		PATH_TRACE;
		globok = 0;
		(void) tbpathconf(longname[0], _PC_LINK_MAX, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		(void) close(fd);
	}

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbpathconf(longname[1], _PC_LINK_MAX, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
test25()
{
	/* path longer than PATH_MAX */

	char	**longname;
	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test25");

	testfail = 0;

	if ((fd = crfile(t25_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t25_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	longname = pathtoolng(t25_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbpathconf(longname[0], _PC_LINK_MAX, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbpathconf(longname[1], _PC_LINK_MAX, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	clntoolng();
	(void) unlink(t25_file);

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test26()
{
	/* component of path prefix does not exist */
	int	pathok = setupok;

	DBUG_ENTER("test26");

	testfail = 0;

	if (eaccess(t26_dir, F_OK) == 0 && remove(t26_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t26_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbpathconf(t26_file, _PC_LINK_MAX, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test27()
{
	/* file which does not exist */
	int	pathok = setupok;

	DBUG_ENTER("test27");

	testfail = 0;

	if (eaccess(t27_file, F_OK) == 0 && remove(t27_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t27_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbpathconf(t27_file, _PC_LINK_MAX, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test28()
{
	/* path is "" */
	int	pathok = setupok;

	DBUG_ENTER("test28");

	testfail = 0;

	globok = 0;
	(void) tbpathconf("", _PC_LINK_MAX, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}


private void
test29()
{
	/* component of path is not a directory */

	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test29");

	testfail = 0;

	if ((fd = crfile(t29_ndir, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t29_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbpathconf(t29_file, _PC_LINK_MAX, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	(void) unlink(t29_ndir);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t29_ndir, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t29_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbpathconf(t29_file, _PC_LINK_MAX, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

#endif	/* TEST_XNFS */

	(void) unlink(t29_ndir);

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test30()
{
	DBUG_ENTER("test30");

	t_einval(_PC_MAX_CANON, T_REG | T_FIFO | T_DIR);

	DBUG_VOID_RETURN;
}

private void
test31()
{
	DBUG_ENTER("test31");

	t_einval(_PC_MAX_INPUT, T_REG | T_FIFO | T_DIR);

	DBUG_VOID_RETURN;
}

private void
test32()
{
	DBUG_ENTER("test32");

	t_einval(_PC_NAME_MAX, T_REG | T_FIFO | T_TERM);

	DBUG_VOID_RETURN;
}

private void
test33()
{
	DBUG_ENTER("test33");

	t_einval(_PC_PATH_MAX, T_REG | T_FIFO | T_TERM);

	DBUG_VOID_RETURN;
}

private void
test34()
{
	DBUG_ENTER("test34");

	t_einval(_PC_PIPE_BUF, T_REG | T_TERM);

	DBUG_VOID_RETURN;
}

private void
test35()
{
	DBUG_ENTER("test35");

	t_einval(_PC_NO_TRUNC, T_REG | T_FIFO | T_TERM);

	DBUG_VOID_RETURN;
}

private void
test36()
{
	DBUG_ENTER("test36");

	t_einval(_PC_VDISABLE, T_REG | T_FIFO | T_DIR);

	DBUG_VOID_RETURN;
}

private void
test37()
{
	long rval;
	int err;
	int pathok = 0;

	DBUG_ENTER("test37");

#if TEST_FIPS || TEST_XPG > 3
	errno = 0;
	rval = pathconf(".", _PC_CHOWN_RESTRICTED);
	if (rval == -1)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("pathconf(\".\", _PC_CHOWN_RESTRICTED) did not give correct results");
		in_rpt("expected: return value != -1");
		in_rpt("observed: return value == %ld, errno == %d (%s)",
			rval, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}

private void
test38()
{
	long rval;
	int err;
	int pathok = 0;

	DBUG_ENTER("test38");

#if TEST_FIPS || TEST_XPG > 3
	errno = 0;
	rval = pathconf(".", _PC_NO_TRUNC);
	if (rval == -1)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("pathconf(\".\", _PC_NO_TRUNC) did not give correct results");
		in_rpt("expected: return value != -1");
		in_rpt("observed: return value == %ld, errno == %d (%s)",
			rval, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test39()
{
	DBUG_ENTER("test39");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test39()
{
	long rval;
	int err;
	int pathok = setupok;

	DBUG_ENTER("test39");

#if TEST_FIPS || TEST_XPG > 3
	if (tty_name == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TTYNAME is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	rval = pathconf(tty_name, _PC_VDISABLE);
	if (rval == -1)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("pathconf(\".\", _PC_VDISABLE) did not give correct results");
		in_rpt("expected: return value != -1");
		in_rpt("observed: return value == %ld, errno == %d (%s)",
			rval, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}
#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if TEST_XNFS

private void
test40()
{
	char	*longname;
	int	pathok = setupok;

	DBUG_ENTER("test40");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbpathconf(longname, _PC_LINK_MAX, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test40()
{
	DBUG_ENTER("test40");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test41()
{
	char	*longname;
	int	pathok = setupok;

	DBUG_ENTER("test41");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbpathconf(longname, _PC_LINK_MAX, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test41()
{
	DBUG_ENTER("test41");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test42()
{
	char	*longname;
	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test42");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbpathconf(longname, _PC_LINK_MAX, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test42()
{
	DBUG_ENTER("test42");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test43()
{
	char	*longname;
	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test43");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbpathconf(longname, _PC_LINK_MAX, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test43()
{
	DBUG_ENTER("test43");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private char *
strname(name)
int name;
{
	switch (name)
	{
	    case _PC_LINK_MAX:		return "_PC_LINK_MAX";
	    case _PC_MAX_CANON:		return "_PC_MAX_CANON";
	    case _PC_MAX_INPUT:		return "_PC_MAX_INPUT";
	    case _PC_NAME_MAX:		return "_PC_NAME_MAX";
	    case _PC_PATH_MAX:		return "_PC_PATH_MAX";
	    case _PC_PIPE_BUF:		return "_PC_PIPE_BUF";

	    case _PC_CHOWN_RESTRICTED:	return "_PC_CHOWN_RESTRICTED";
	    case _PC_NO_TRUNC:		return "_PC_NO_TRUNC";
	    case _PC_VDISABLE:		return "_PC_VDISABLE";
	}

	return itoa(name);
}

#if	  !TEST_XNFS

private void
chkval(path, name, expval, constname)
char *path;
int name;
long expval;
char *constname;
{
	/* Check return value is equal to constant in <limits.h> */
	/* (or value >= constant for XPG3 mode) */

	long rval;
	int fail;
	int	pathok = setupok;

	DBUG_ENTER("chkval");

	fail = testfail;

	globok = 0;
	rval = tbpathconf(path, name, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	if (testfail > fail)
		DBUG_VOID_RETURN;

#if TEST_XPG == 3
	if (rval < expval)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("pathconf(\"%s\", %s) returned a value less than %s",
			path, strname(name), constname);
		in_rpt("value returned: %ld, %s: %ld", rval, constname, expval);
	}
	else
		PATH_TRACE;
#else
	if (rval != expval)
	{
		if (testfail == 0)
			xx_rpt(FAILURE);
		in_rpt("pathconf(\"%s\", %s) did not return the value of %s",
			path, strname(name), constname);
		in_rpt("value returned: %ld, %s: %ld", rval, constname, expval);
		if (testfail == 0)
		{
			in_rpt("(%s should not be defined in <limits.h> if the ",
				constname);
			in_rpt("run-time value can vary.)");
		}
		testfail++;
	}
	else
		PATH_TRACE;
#endif

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
minval(path, name, min, minname)
char *path;
int name;
long min;
char *minname;
{
	/* Check return value is >= POSIX minimum, or == -1 */

	long rval;
	int fail;
	int	pathok = setupok;

	DBUG_ENTER("minval");

	fail = testfail;

	globok = 0;
	rval = tbpathconf(path, name, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	if (testfail > fail)
		DBUG_VOID_RETURN;

	if (rval < min && rval != -1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("pathconf(\"%s\", %s) returned a value less than %s",
			path, strname(name), minname);
		in_rpt("value returned: %ld, %s: %ld", rval, minname, min);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

#if	  !TEST_XNFS
private void
chkeql(path, name, val, valname)
char *path;
int name;
long val;
char *valname;
{
	long rval;
	int fail;
	int	pathok = setupok;

	DBUG_ENTER("chkeql");

	fail = testfail;

	globok = 0;
	rval = tbpathconf(path, name, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	if (testfail > fail)
		DBUG_VOID_RETURN;

	if (rval != val)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("pathconf(\"%s\", %s) did not return %s",
			path, strname(name), valname);
		in_rpt("value returned: %ld, %s: %ld", rval, valname, val);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
t_einval(name, types)
int name;
int types;
{
	int pathok = setupok;

	DBUG_ENTER("t_einval");

	testfail = 0;

	if (types & T_REG)
	{
		globok = 0;
		(void) tbpathconf(tfile, name, SYSERROR, EINVAL);
		if (globok == 1)
			PATH_TRACE;
	}
	else
		PATH_TRACE;

#if	   TEST_XNFS

	PATH_TRACE;

#else	/* TEST_XNFS */

	if (types & T_FIFO)
	{
		globok = 0;
		(void) tbpathconf(tfifo, name, SYSERROR, EINVAL);
		if (globok == 1)
			PATH_TRACE;
	}
	else
		PATH_TRACE;

#endif	/* TEST_XNFS */

	if (types & T_DIR)
	{
		globok = 0;
		(void) tbpathconf(".", name, SYSERROR, EINVAL);
		if (globok == 1)
			PATH_TRACE;
	}
	else
		PATH_TRACE;

#if	   TEST_XNFS

	PATH_TRACE;

#else	/* TEST_XNFS */

	if (types & T_TERM)
	{
		if (tty_name == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: parameter VSX_TTYNAME is not defined");
		}
		else
		{
			globok = 0;
			(void) tbpathconf(tty_name, name, SYSERROR, EINVAL);
			if (globok == 1)
				PATH_TRACE;
		}
	}
	else
		PATH_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private long
tbpathconf(tbpath, tbname, expret, experrno)
char	*tbpath;
int	tbname;
int	expret;
int	experrno;
{
	char	*reppath = tbpath;
	long	rval;
	int	err;
	char	pathbuf[30];
	int	pathok = 0;
	DBUG_ENTER("tbpathconf");

	errno = NOERROR;
	rval = pathconf(tbpath, tbname);
#ifdef TESTBED
	rval = -1;
	errno++;
#endif
	err = errno;

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
	}

	if (rval == SYSERROR && err == NOERROR)
	{
		PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_RETURN(rval);
	}
	else
		PATH_TRACE;

	if (expret == SUCCEED)
	{
		if (rval != SYSERROR)
		{
			PATH_FUNC_RPT(1);
			DBUG_RETURN(rval);
		}
		else
			PATH_TRACE;

		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("pathconf(\"%s\", %s) did not give correct results",
			reppath, strname(tbname));
		in_rpt("expected: return value != -1");
		in_rpt("      OR: return value == -1, errno == 0 (NO ERROR)");
		in_rpt("observed: return value == %ld, errno == %d (%s)",
			rval, err, errname(err));
		PATH_TRACE;
	}
	else
	{
		if (rval == SYSERROR && err == experrno)
		{
			PATH_FUNC_RPT(1);
			DBUG_RETURN(rval);
		}
		else
			PATH_TRACE;

		if (rval != SYSERROR)
		{
			PATH_FUNC_RPT(2);
			DBUG_RETURN(rval);
		}
		else
			PATH_TRACE;

		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("pathconf(\"%s\", %s) did not give correct results",
			reppath, strname(tbname));
		in_rpt("expected: return value == -1, errno == %d (%s)",
			experrno, errname(experrno));
		in_rpt("      OR: return value != -1");
		in_rpt("observed: return value == %ld, errno == %d (%s)",
			rval, err, errname(err));
	}

	PATH_FUNC_RPT(3);
	DBUG_RETURN(rval);
}
