/*
*      SCCS:  @(#)  POSIX.os/files/readdir/readdir.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/readdir - T.readdir Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.readdir

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the readdir() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	18 Apr 1989
MODIFICATIONS:	
		Geoff Clare, 21 Nov 1989
			Don't test d_ino in POSIX-only mode
		
		Geoff Clare, 16 Mar 1990
			Delete test that "." and ".." both exist if one does
		
		Geoff Clare, 5 July 1990
			Allow for NAME_MAX > PCTS_NAME_MAX

		Peter Lee, Unisoft Ltd.,
		Mon Jul 30 17:20:24 BST 1990
		Added resilience code.

		Geoff Clare, 20 Nov 1990
			Revise strategy in EBADF test (can't use dirp
			after closedir(dirp) - may not be accessible)

		Geoff Clare, 14 May 1992
			POSIX.3.1 alignment of EBADF test.

		John-Paul, 29 July 1992
			Mode parameter of call to crfile() changed from
			'int' to 'mode_t' type.

		David Sawyer, 9 November 1992
			Modifications for XNFS tests

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<string.h>
#include	<dirent.h>
#include	<limits.h>
#include	<signal.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<chktimes.h>
#include	<pcts_max.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"readdir_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef readdir
#endif



#define NO_TESTS	12
#define	WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define ENDOFDIR	-1
#define E_NOSIG		02
#if	   TEST_XNFS
#define NOK		02
#define PATH_OK		04
#endif	/* TEST_XNFS */


extern	char	*errname();
extern	char	*newstr();
extern	void	(*vsignal())();


public	int	no_tests = NO_TESTS;

private DIR	*dirp;

private struct {
	int	readcount;
	ino_t	ino;
	char	*name;
} *entp, entries[_POSIX_NAME_MAX + 4] = {
	0, 0, ".",		/* NB ino not used for "." and ".." */
	0, 0, "..",
	0, 0, "1",
	0, 0, "12",
	0, 0, "123",
	0, 0, "1234",
	0, 0, "12345",
	0, 0, "123456",
	0, 0, "1234567",
	0, 0, "12345678",
	0, 0, "123456789",
	0, 0, "1234567890",
	0, 0, "12345678901",
	0, 0, "123456789012",
	0, 0, "1234567890123",
	0, 0, "12345678901234",	/* _POSIX_NAME_MAX entry */
	0, 0, NULL,		/* space for NAME_MAX entry */
	0, 0, NULL,		/* end marker */
};

private int	testfail;
private int	setupok;

private void	prepare_tests();
private void	cln_tests();
private void	chkcounts();
private struct dirent	*tbreaddir();
#if	   TEST_XNFS
extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;
#endif	/* TEST_XNFS */

#if	   TEST_XNFS
extern	char	*execpath();
extern	char	*itoa();
extern	char	*errname();
#endif	/* TEST_XNFS */

#if	   TEST_XNFS
private	char	*prog_su = "readdir_su";
#endif	/* TEST_XNFS */

#if	   TEST_XNFS
private	char	*t11_dir = T11_DIR;
private	char	*t12_dir = T12_DIR;
#endif	/* TEST_XNFS */

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();

private void	ch_t7();
private void	ch_t8();
private void	pa_t7();
#if	   TEST_XNFS
private void	ch_t11();
#endif	/* TEST_XNFS */


public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	NULL, 0
};

private void
prepare_tests()
{
	long	name_max;
	int	fd, i;
	struct stat stbuf;
	int	pathok = 0;
	int	loop;

	DBUG_ENTER("prepare_tests");

	errno = 0;
	name_max = pathconf(".", _PC_NAME_MAX);
	if (name_max == -1 && errno != 0)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i,
				"pathconf(\".\", _PC_NAME_MAX) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED1
	name_max = 100;
#endif

	if (name_max > _POSIX_NAME_MAX && name_max <= PCTS_NAME_MAX)
	{
		/* add an entry for a name of length NAME_MAX */

		entp = &entries[_POSIX_NAME_MAX + 2];
		entp->readcount = 0;
		entp->name = (char *)malloc((size_t)(name_max+1));
		if (entp->name == NULL)
		{
			for (i = 1; i <= NO_TESTS; i++)
				(void) tet_delete(i,
					"malloc() failed in preparation");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		for (i = 0; i < name_max; i++)
			entp->name[i] = '0' + (i + 1)%10;
		entp->name[i] = '\0';
	}
	else
		PATH_TRACE;

	/* create all the directory entries (except "." and "..") */

	for (entp = &entries[2], loop = 0; entp->name != NULL; entp++, loop++)
	{
		if ((fd = crfile(entp->name, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
		{
			for (i = 1; i <= NO_TESTS; i++)
				(void) tet_delete(i, "could not create directory entry in preparation");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		(void) close(fd);
		if (stat(entp->name, &stbuf) == SYSERROR)
		{
			for (i = 1; i <= NO_TESTS; i++)
				(void) tet_delete(i, "stat() failed in preparation on file just created");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		entp->ino = stbuf.st_ino;
	}

	dirp = opendir(".");
	if (dirp == NULL)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i,
				"opendir(\".\") failed in preparation");
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(3 + 2 * loop);

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) closedir(dirp);

	for (entp = &entries[2]; entp->name != NULL; entp++)
		if (*entp->name != '\0')
			(void) unlink(entp->name);

	DBUG_VOID_RETURN;
}

private void
sigcatch(sig)
int sig;
{
	if (sig == SIGUSR1)
		exit(OK);
}

private void
test1()
{
	/* readdir() returns current directory entry */

	int i;
	int	pathok = setupok;

	DBUG_ENTER("test1");

	testfail = 0;

	globok = 0;
	(void) tbreaddir(dirp, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(1);
	}
	else
	{
		for (i = 2; i <= NO_TESTS; i++)
			(void) tet_delete(i, "depends on success of test 1");
	}
	
	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* readdir() advances to next directory entry */

	struct dirent	*dp;
	char		*savname;
	int		i;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	testfail = 0;

	savname = entp->name;	/* entp set by tbreaddir() in test1 */

	globok = 0;
	dp = tbreaddir(dirp, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{

#ifdef TESTBED2
		(void) strcpy(dp->d_name, savname);
#endif

		/* check it is not the same entry as read in test1 */
		if (strcmp(savname, dp->d_name) == 0)
		{
			testfail++;
			xx_rpt(FAILURE);
			in_rpt("readdir() did not advance the directory stream");
		}
		else
			PATH_TRACE;
	}

#ifndef TESTBED2

	if (testfail == 0)
	{
		PATH_XS_RPT(2);
	}
	else
	{
		for (i = 3; i <= NO_TESTS; i++)
			(void) tet_delete(i, "depends on success of test 2");
	}

#endif

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* readdir() returns NULL at end of stream */

	struct dirent *dp;
	int i;
	int	pathok = setupok;

	DBUG_ENTER("test3");

	testfail = 0;

	/* do enough calls to ensure we hit the end */
	for (i = 0; i < 1000; i++)
	{
		if (i == 0)
			PATH_TRACE;
		if ((dp = tbreaddir(dirp, ENDOFDIR)) == NULL)
			break;
#ifdef TESTBED2
		break;
#endif
	}

	if (dp != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("readdir() did not return NULL after 1000 calls!");
	}
	else
		PATH_TRACE;

#ifndef TESTBED2

	if (testfail == 0)
	{
		PATH_XS_RPT(2);
	}
	else
	{
		for (i = 4; i <= NO_TESTS; i++)
			(void) tet_delete(i, "depends on success of test 3");
	}

#endif

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	pathok = setupok;
	/* file names of length 1 to NAME_MAX are returned in d_name */

	DBUG_ENTER("test4");

	testfail = 0;

#ifdef TESTBED2
	entries[4].readcount++;
#endif

	globok = 0;
	chkcounts(1);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* structure not overwritten by reading another stream */

	DIR		*dirp2;
	struct dirent	*dp1, *dp2;
	char		*savname;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test5");

	testfail = 0;

	rewinddir(dirp);

	globok = 0;
	dp1 = tbreaddir(dirp, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	savname = newstr(dp1->d_name);

	if (savname == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* worst case: both streams refer to the same directory */
	dirp2 = opendir(".");
	if (dirp2 == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("opendir(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* read until we get a different entry */
	errno = NOERROR;
	while (testfail == 0)
	{
		dp2 = tbreaddir(dirp2, NOERROR);
		if (strcmp(savname, dp2->d_name) != 0)
		{
			PATH_TRACE;
			break;
		}
	}
	if (testfail > 0)
	{
		(void) closedir(dirp2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	savname = dp2->d_name;
#endif

	if (strcmp(dp1->d_name, savname) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("structure returned by readdir() was overwritten by readdir() on another directory stream");
		in_rpt("original 1st stream d_name: \"%s\"", savname);
		in_rpt("observed 1st stream d_name: \"%s\"", dp1->d_name);
		in_rpt("observed 2nd stream d_name: \"%s\"", dp2->d_name);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	(void) closedir(dirp2);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* readdir() marks st_atime for update */

	DIR	*dirp2;
	char	*chkt;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test6");

	testfail = 0;

	/* The savtimes() must come before opendir() in case an
	   initial buffer is read in opendir() */

	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;

	dirp2 = opendir(".");
	if (dirp2 == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("opendir(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbreaddir(dirp2, NOERROR);
	PATH_FUNC_TRACE;

#ifdef TESTBED2
	globok = 0;
	savtimes(".");
	PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	if (testfail == 0)
	{
		globok = 0;
		chkt = chktimes(".", NEWATIME);
		PATH_FUNC_TRACE;

		if (chkt != NULL)
		{
			xx_rpt(FAILURE);
			in_rpt("%s", chkt);
		}
		else
			PATH_XS_RPT(5);
	}

	(void) closedir(dirp2);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* after fork(), parent can use readdir() and rewinddir() */

	int i;
	int	retval;
	int	pathok = setupok;

	DBUG_ENTER("test7");

	testfail = 0;

	rewinddir(dirp);
	for (entp = entries; entp->name != NULL; entp++)
		entp->readcount = 0;

	/* read partway before forking */

	for (i = 0; i < 5; i++)
	{
		globok = 0;
		(void) tbreaddir(dirp, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	retval = cppair(ch_t7, pa_t7, WAITTIME, E_NOSIG);
	if (retval == E_NOSIG)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: child did not receive SIGUSR1 from parent");
	}
	else  
		PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	DBUG_ENTER("ch_t7");

	(void) vsignal(SIGUSR1, sigcatch);
	(void) pause();

	DBUG_EXIT(E_NOSIG);
}

private void
pa_t7()
{
	int	pathok = setupok;

	DBUG_ENTER("pa_t7");

	(void) sleep(2); /* allow child to set signal catcher */

	/* read remainder of directory */
	while (testfail == 0)
		if (tbreaddir(dirp, ENDOFDIR) == NULL)
		{
			PATH_TRACE;
			break;
		}
	
	/* now rewind and read through again */
	rewinddir(dirp);
	while (testfail == 0)
		if (tbreaddir(dirp, ENDOFDIR) == NULL)
		{
			PATH_TRACE;
			break;
		}

	/* all entries should have been read twice */
	if (testfail == 0)
		chkcounts(2);
	
	(void) kill(tet_child, SIGUSR1);

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* after fork(), child can use readdir() and rewinddir() */

	int i;
	int	pathok = setupok;

	DBUG_ENTER("test8");

	testfail = 0;

	rewinddir(dirp);
	for (entp = entries; entp->name != NULL; entp++)
		entp->readcount = 0;

	/* read partway before forking */

	for (i = 0; i < 5; i++)
	{
		globok = 0;
		(void) tbreaddir(dirp, NOERROR);
		PATH_FUNC_TRACE;
	}

	PATH_FUNC_RPT(5);

	if (testfail == 0)
		(void) cppair(ch_t8, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int	pathok = setupok;

	DBUG_ENTER("ch_t8");

	PATH_FUNC_TRACE;

	/* read remainder of directory */
	while (testfail == 0)
	{
		if (tbreaddir(dirp, ENDOFDIR) == NULL)
		{
			PATH_TRACE;
			break;
		}
	}
	/* now rewind and read through again */
	rewinddir(dirp);
	while (testfail == 0)
		if (tbreaddir(dirp, ENDOFDIR) == NULL)
		{
			PATH_TRACE;
			break;
		}
	
	/* all entries should have been read twice */
	if (testfail == 0)
	{
		PATH_TRACE;

		globok = 0;
		chkcounts(2);
		PATH_FUNC_TRACE;
	}
	
	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	/* readdir() does not return empty names */

	int err;
	int	pathok = setupok;

	DBUG_ENTER("test9");

	testfail = 0;

	rewinddir(dirp);
	for (entp = entries; entp->name != NULL; entp++)
		entp->readcount = 0;

	if (unlink(entries[2].name) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unlink(\"%s\") failed - errno %d (%s)",
			entries[2].name, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	entries[2].name = "";

	/* tbreaddir() will detect empty name as "unexpected" */

	while (testfail == 0)
		if (tbreaddir(dirp, ENDOFDIR) == NULL)
		{
			PATH_TRACE;
			break;
		}

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test10()
{
	DBUG_ENTER("test10");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test10()
{
	/* dirp is not an open directory stream */

	char	*cp;
	DIR	*dirp2;
	int	open_max, i, *fdtab, dir_fd;
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	cp = tet_getvar("VSX_READDIR_EBADF");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_READDIR_EBADF is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_READDIR_EBADF is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0)
	{
		if (errno == 0)
		{
			PATH_TRACE;
			open_max = 256;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("sysconf(_SC_OPEN_MAX) returned non-positive value");
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

	fdtab = (int *)malloc((size_t)(open_max * sizeof(int)));
	if (fdtab == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc(%ld) failed", (long)(open_max * sizeof(int)));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < open_max; i++)
		fdtab[i] = fcntl(i, F_GETFD);

	if ((dirp2 = opendir(".")) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("opendir(\".\") failed");
		free((void *)fdtab);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	dir_fd = -1;
	for (i = 0; i < open_max; i++)
	{
		if (fdtab[i] == SYSERROR && fcntl(i, F_GETFD) != SYSERROR)
		{
			dir_fd = i;
			break;
		}
	}

	free((void *)fdtab);
	
	if (dir_fd == -1)
	{
		xx_rpt(UNTESTED);
		in_rpt("DIR is not implemented using a file descriptor");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(dir_fd);

	globok = 0;
	(void) tbreaddir(dirp2, EBADF);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) closedir(dirp2);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

#if	   TEST_XNFS

private void
test11()
{
	int	pathok = 0;
	int	rc;

	DBUG_ENTER("test11");

	testfail = 0;

	if (tet_sync(SP_T11_MKDIR_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t11, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
	{
		testfail++;
	}
	else if (rc & PATH_OK)
	{
		PATH_TRACE;
	}

	if (tet_sync(SP_T11_RMDIR_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t11");

	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t11_dir;
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}


#else	/* TEST_XNFS */

private void
test11()
{
	DBUG_ENTER("test11");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test12()
{
	int	testfail = 0;
	int	pathok = 0;
	int 	err;
	DIR 	*dirp;
	struct dirent *ret;


	DBUG_ENTER("test12");

	if (tet_sync(SP_T12_MKDIR_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	dirp = opendir(t12_dir);
	if (dirp == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("opendir(\"%s\") failed - errno %d (%s)",
			t12_dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T12_RMDIR_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	ret = readdir(dirp);
	err = errno;
	if (ret != (struct dirent *) NULL || err != ESTALE)
	{
		xx_rpt(FAILURE);
		if(ret != NULL)
			in_rpt("readdir(): did not return NULL as expected");
		if(err != EACCES)
			in_rpt("readdir(): errno = %d (%s) expected %d (ESTALE)",
				err, errname(err), ESTALE);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test12()
{
	DBUG_ENTER("test12");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
chkcounts(expcount)
int expcount;
{
	int	pathok = setupok;
	int	loop = 0;
	/* check read counts are all expcount */

	DBUG_ENTER("chkcounts");

	globok = 0;

	for (entp = &entries[2]; entp->name != NULL; entp++, loop++)
	{
		if (entp->readcount != expcount)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("directory entry for \"%s\" was read %d times, expected %d",
				entp->name, entp->readcount, expcount);
		}
		else
			PATH_TRACE;
	}

	PATH_FUNC_RPT(loop);

	DBUG_VOID_RETURN;
}

private struct dirent *
tbreaddir(tbdirp, experrno)
DIR	*tbdirp;
int	experrno;
{
	struct dirent	*rval;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("tbreaddir");

	errno = 0;
	rval = readdir(tbdirp);
	err = errno;
	globok = 0;

#ifdef TESTBED
	rval = NULL;
	err++;
#endif

	if (experrno == NOERROR && rval == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (err == NOERROR)
			in_rpt("readdir() returned unexpected end of directory indication");
		else
			in_rpt("readdir() failed - errno %d (%s)", err, errname(err));
	}
	else if (experrno == ENDOFDIR && rval == NULL && err != NOERROR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("readdir() set errno to %d (%s) on reaching end of directory",
			err, errname(err));
	}
	else if (experrno > 0)
	{
		if (rval != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("readdir() succeeded unexpectedly");
		}
		else if (experrno != err)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("readdir() did not set errno correctly");
			in_rpt("expected: %d (%s), observed: %d (%s)",
				experrno, errname(experrno), err, errname(err));
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (rval != NULL)
	{
#ifdef TESTBED3
		*rval->d_name = '\0';
#endif
		PATH_TRACE;
		/* check it is a valid entry */
		for (entp = entries; entp->name != NULL; entp++)
		{
			if (entp == entries)
				PATH_TRACE;
			if (*entp->name != '\0' &&
				strcmp(entp->name, rval->d_name) == 0)
			{
				entp->readcount++;
				break;
			}
		}

#ifdef TESTBED4
		rval->d_ino++;
#endif

		if (entp->name == NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("readdir() returned an unexpected directory entry, d_name = \"%s\"",
				rval->d_name);
		}
#ifdef _XOPEN_SOURCE
		else if (entp >= &entries[2] && entp->ino != rval->d_ino)
		{
			/* check inode number (except "." and "..") */
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("readdir() returned incorrect d_ino for file \"%s\"",
				rval->d_name);
			in_rpt("expected d_ino: %ld, observed: %ld",
				(long) entp->ino, (long)rval->d_ino);
		}
		else
			PATH_TRACE;
#else
		else
			PATH_TRACE;
#endif /* _XOPEN_SOURCE */
	}
	else
	{
		PATH_TRACE; /* to keep the check value constant */
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(4);

	DBUG_RETURN(rval);
}
