/*
*      SCCS:  @(#)  POSIX.os/files/readdir/readdir_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/readdir - readdir_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		readdir_su - used by T.readdir in privileged tests

PROJECT:	XNFS (X/OPEN XNFS tests)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.readdir to perform privileged tests.

	  usage:
		readdir_su testnum file file ident dbug

	  where testnum is the number of the test being executed, file and
	  file parameters to be passed to the test.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		David Sawyer
DATE CREATED:	5 November 1992
MODIFICATIONS:

************************************************************************/
#include 	<std.h>
#include 	<stdlib.h>
#include 	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<dirent.h>
#include 	<dbug.h>
#include	<errno.h>
#include 	<sys/stat.h>
#include 	<report.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"readdir_h.h"
#include 	<sysdep.h>

#ifdef UNDEF_MACROS
#undef readdir
#endif



#if	   TEST_XNFS
#define NOK		02
#define PATH_OK		04
#endif	/* TEST_XNFS */

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	char	**environ;

extern	void	set_pname();

#if	   TEST_XNFS
extern	char	*errname();
#endif	/* TEST_XNFS */

#if	   TEST_XNFS
private	int	subtest11();
#endif	/* TEST_XNFS */

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */


public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int testnum;
	int exitval = OK;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong" );

		return(exitval);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

#if	   TEST_XNFS
	if(geteuid() != (uid_t) XNFS_MAPPED_UID)
	{
		xx_rpt(DELETION);
		in_rpt("%s should be installed with setuid equal to %d", argv[0], (int) XNFS_MAPPED_UID);

		return(exitval);
	}
#endif	/* TEST_XNFS */

	testnum = atoi(argv[1]);
	switch (testnum)
	{

#if	   TEST_XNFS

	case 11:
		exitval = subtest11(argv[2]);
		break;

#endif	/* TEST_XNFS */

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed: %d", testnum);

		return(exitval);
	}

	return(exitval);
	/* NOTREACHED */
}


#if	   TEST_XNFS
private int
subtest11(file)
char *file;
{
	int 	err, rc;
	DIR 	*dirp;
	struct dirent *ret;
	int 	pathok = 0;


	DBUG_ENTER("subtest11");

	/* obtain directory stream while directory is readable */
	dirp = opendir(file);
	if (dirp == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("opendir(\"%s\") failed - errno %d (%s)",
			file, err, errname(err));
		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T11_MODE_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MODE_END failed");
		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;
	
	/* directory is no longer readable */

	(void) rewinddir(dirp);

	errno = 0;
	ret = readdir(dirp);
	err = errno;
	if (ret != (struct dirent *) NULL || err != EACCES)
	{
		xx_rpt(FAILURE);
		if(ret != NULL)
			in_rpt("readdir(): did not return NULL as expected");
		if(err != EACCES)
			in_rpt("readdir(): errno = %d (%s) expected %d (EACCES)",
				err, errname(err), EACCES);
		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;
	
	rc = OK;

	if (pathok == 3)
		rc |= PATH_OK;

	DBUG_RETURN(rc);
}
#endif	/* TEST_XNFS */
