/*
*      SCCS:  @(#)  POSIX.os/files/rename/rename.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/rename - T.rename Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.rename

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the rename() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	28 Mar 1989
MODIFICATIONS:	
		Geoff Clare, 8 June 1989
			Add do_access() tests.
			Merge all privileged tests into one subprogram.

		Geoff Clare, 22 Aug 1989
			VSX_REMOVE_DIR_EBUSY and VSX_RENAME_DIR_EBUSY can
			now take values S (system only) and P (process only)
			as well as Y (both) and N (neither)
		
		Geoff Clare, 13 Feb 1990
			In EBUSY tests, after chdir(), open "." to ensure
			the directory is considered "in use" by the process.

		Geoff Clare, 5 Apr 1990
			New interface with pathtoolng() and nametoolng().
		
		Steve Ratcliffe, 20 Apr 1990
			Test to check that resourses are released to the
			system when an a file is removed as a result of a
			rename().
		
		Geoff Clare, 20 June 1990
			Add EMLINK test

		Anthony Lee, 01 August 1990
			Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutel, 23 Nov 1990
			Resilience code modified. cppair path tracing
			improved.

		Geoff Clare, 8 May 1992
			Add PRES_NODOTDOT to init_pathres() call.

		Geoff Clare, 12 June 1992
			Add PRES_NODOT to init_pathres() call.

                John-Paul, 29 July 1992
                        Mode parameter in calls to crfile() and crdir()
                        changed from 'int' to 'mode_t' type.

		David Sawyer, 10 November 1992
			Modifications for XNFS tests
		
		Geoff Clare, 15 Oct 1993
			Increase child timeout in test 17.

		Geoff Clare, 13 April 1995
			Change sleep(1) to sleep(2) in test 9 for XNFS.

		Geoff Clare, 3 Nov 1995
			Changed child timeout for do_access() tests.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<string.h>
#include	<fcntl.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<pcts_max.h>
#include	<setprv.h>
#include	<pathres.h>
#include	<chktimes.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"rename_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef rename
#endif


#define NO_TESTS	39
#define	WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define E_CHDIR		02
#define E_OPEN		04
#define NOK		02
#define	NAMELEN		25
#define	PATH_NOK	8

extern	char	*itoa();
extern	char	*errname();
extern	char	*execpath();
#if	   TEST_XNFS
extern	char	*xnfs_nametoolng();
#else	/* TEST_XNFS */
extern	char	**nametoolng();
#endif	/* TEST_XNFS */
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	void	valrpt();

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;

public	int	no_tests = NO_TESTS;

private	char	*t1_file =	"rename-t.1";
private	char	*t2_file =	"rename-t.2";
private	char	*t3_dir =	"rename-d.3";
private	char	*t3_file =	"rename-d.3/rename-t.3";
private	char	*t4a_file =	"rename-t.4a";
private	char	*t4b_file =	"rename-t.4b";
private	char	*t5a_file =	"rename-t.5a";
private	char	*t5b_file =	"rename-t.5b";
private	char	*t6a_file =	"rename-t.6a";
private	char	*t6b_file =	"rename-t.6b";
private	char	*t7a_file =	"rename-t.7a";
private	char	*t7b_file =	"rename-t.7b";
private	char	*t8a_file =	"rename-t.8a";
private	char	*t8b_file =	"rename-t.8b";
private	char	*t9a_file =	"rename-t.9a";
private	char	*t9b_file =	"rename-t.9b";
private	char	*t10a_file =	"rename-t.10a";
private	char	*t10b_file =	"rename-t.10b";
private	char	*t11a_file =	"rename-t.11a";
private	char	*t11b_file =	"rename-t.11b";
private	char	*t12a_file =	"rename-t.12a";
private	char	*t12b_file =	"rename-t.12b";
private	char	*t12c_file =	"rename-t.12c";
private	char	*t12d_file =	"rename-t.12d";
private	char	*t13a_file =	"rename-t.13a";
private	char	*t13b_file =	"rename-t.13b";
private	char	*t13c_file =	"rename-t.13c";
private	char	*t13d_file =	"rename-t.13d";
#if	  !TEST_XNFS
private	char	*t14a_file =	"rename-t.14a";
#endif	/* TEST_XNFS */
private	char	*t17_dir =	"rename-d.17";
private char	*t18_dir =	"rename-d.18";
private char	*t18a_file =	"rename-t.18a";
private char	*t18b_file =	"rename-d.18/rename-t.18b";
#if	   TEST_XNFS
private char	*t18_file2a =	T18_FILE2A;
private char	*t18_file2b =	T18_FILE2B;
private char	*t18_file2c =	T18_FILE2C;
private char	*t18_file2d =	T18_FILE2D;
#endif	/* TEST_XNFS */
private char	*t19_dir =	"rename-d.19";
private char	*t19a_file =	"rename-d.19/rename-t.19a";
private char	*t19b_file =	"rename-t.19b";
#if	   TEST_XNFS
private char	*t19_file2a =	T19_FILE2A;
private char	*t19_file2b =	T19_FILE2B;
private char	*t19_file2c =	T19_FILE2C;
private char	*t19_file2d =	T19_FILE2D;
#endif	/* TEST_XNFS */
private	char	*t20a_dir =	"rename-t.20a";
private	char	*t20b_dir =	"rename-t.20b";
#if	   TEST_XNFS
private	char	*t20_dir2a =	T20_DIR2A;
private	char	*t20_dir2d =	T20_DIR2D;
#endif	/* TEST_XNFS */
private	char	*t21a_file =	"rename-t.21a";
private	char	*t21b_file =	"rename-t.21b";
private	char	*t21c_file =	"rename-t.21c";
private	char	*t21d_file =	"rename-t.21d";
private	char	*t22a_file =	"rename-t.22a";
private	char	*t22b_file =	"rename-t.22b";
private	char	*t22c_file =	"rename-t.22c";
private	char	*t22d_file =	"rename-t.22d";
private	char	*t23a_dir =	"rename-d.23a";
private	char	*t23b_dir =	"rename-d.23b";
private	char	*t23_file =	"rename-d.23b/rename-t.23";
private	char	*t24a_dir =	"rename-t.24a";
private	char	*t24b_dir =	"rename-t.24a/rename-t.24b";
private	char	*t25_file =	"rename-t.25";
private	char	*t25_dir =	"rename-d.25";
#if	  !TEST_XNFS
private	char	*t26_file =	"rename-t.26";
#endif	/* TEST_XNFS */
private	char	*t27a_file =	"rename-t.27a";
private	char	*t27b_file =	"rename-t.27b";
private char	*t28_dir =	"rename-d.28";
private char	*t28a_file =	"rename-d.28/rename-t.28a";
private char	*t28b_file =	"rename-t.28b";
private	char	*t29a_file =	"rename-t.29a";
private	char	*t29b_file =	"rename-t.29b";
private	char	*t30_file =	"rename-t.30";
private char	*t31_ndir =	"rename-t.31";
private char	*t31a_file =	"rename-t.31/rename-t.31a";
private char	*t31b_file =	"rename-t.31b";
private	char	*t32_file =	"rename-t.32";
private	char	*t32_dir =	"rename-d.32";
private	char	*t34a_file =	"rename-t.34a";
#if	   TEST_XNFS
private	char	*t35_dir =	T35_DIR;
#else	/* TEST_XNFS */
private	char	*t35_dir =	"rename-d.35";
#endif	/* TEST_XNFS */
private	char	*t35a_file =	"rename-t.35";
private	char	*t35b_file =	"rename-d.35/rename-t.35";
#if	   TEST_XNFS
private char	*t36_file =	T36_FILE;
private char	*t37_file =	T37_FILE;
private char	*t38_file =	T38_FILE;
private char	*t39_file =	T39_FILE;
#endif	/* TEST_XNFS */

private char	*prog_su =	"rename_su";
private char	*prog17 =	"rename_t17";

public int	testfail;
private	struct stat stbuf1, stbuf2;

private void	cln_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();
private void	test30();
private void	test31();
private void	test32();
private void	test33();
private void	test34();
private void	test35();
private void	test36();
private void	test37();
private void	test38();
private void	test39();

private void	ch1_t12();
private void	ch2_t12();
private void	ch1_t13();
private void	ch2_t13();
#if	  !TEST_XNFS
private void	ch_t14();
#endif	/* TEST_XNFS */
private void	ch_t15_16();
#if	   TEST_XNFS
private void	ch_t18();
private void	ch_t19();
private void	ch_t20();
#endif	/* TEST_XNFS */
private void	ch1_t21();
private void	ch2_t21();
private void	ch1_t22();
private void	ch2_t22();
private void	ch_t33();
private void	ch_t34();
private void	ch_t17();

private void	pa_t12();
private void	pa_t13();
private void	pa_t21();
private void	pa_t22();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cln_tests;

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	  test1, 1,
	  test2, 2,
	  test3, 3,
	  test4, 4,
	  test5, 5,
	  test6, 6,
	  test7, 7,
	  test8, 8,
	  test9, 9,
	  test10, 10,
	  test11, 11,
	  test12, 12,
	  test13, 13,
	  test14, 14,
	  test15, 15,
	  test16, 16,
	  test17, 17,
	  test18, 18,
	  test19, 19,
	  test20, 20,
	  test21, 21,
	  test22, 22,
	  test23, 23,
	  test24, 24,
	  test25, 25,
	  test26, 26,
	  test27, 27,
	  test28, 28,
	  test29, 29,
	  test30, 30,
	  test31, 31,
	  test32, 32,
	  test33, 33,
	  test34, 34,
	  test35, 35,
	  test36, 36,
	  test37, 37,
	  test38, 38,
	  test39, 39,
	 NULL, 0
};

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) remove(t1_file);
	(void) remove(t2_file);
	(void) unlink(t3_file);
	(void) rmdir(t3_dir);
	(void) unlink(t4a_file);
	(void) unlink(t4b_file);
	(void) unlink(t5a_file);
	(void) unlink(t5b_file);
	(void) unlink(t6a_file);
	(void) unlink(t6b_file);
	(void) rmdir(t7a_file);
	(void) rmdir(t7b_file);
	(void) unlink(t8a_file);
	(void) unlink(t8b_file);
	(void) unlink(t9a_file);
	(void) unlink(t9b_file);
	(void) rmdir(t10a_file);
	(void) rmdir(t10b_file);
	(void) rmdir(t11a_file);
	(void) rmdir(t11b_file);
	(void) rmdir(t12a_file);
	(void) rmdir(t12b_file);
	(void) rmdir(t12c_file);
	(void) rmdir(t12d_file);
	(void) rmdir(t13a_file);
	(void) rmdir(t13b_file);
	(void) rmdir(t13c_file);
	(void) rmdir(t13d_file);
#if	  !TEST_XNFS
	(void) unlink(t14a_file);
#endif	/* TEST_XNFS */
	(void) unlink(t18a_file);
	(void) rmdir(t18_dir);
	(void) chmod(t19_dir, MODEANY);
	(void) unlink(t19a_file);
	(void) unlink(t19b_file);
	(void) rmdir(t19_dir);
	(void) rmdir(t20a_dir);
	(void) rmdir(t20b_dir);
	(void) rmdir(t21a_file);
	(void) rmdir(t21b_file);
	(void) rmdir(t21c_file);
	(void) rmdir(t21d_file);
	(void) rmdir(t22a_file);
	(void) rmdir(t22b_file);
	(void) rmdir(t22c_file);
	(void) rmdir(t22d_file);
	(void) unlink(t23_file);
	(void) rmdir(t23a_dir);
	(void) rmdir(t23b_dir);
	(void) rmdir(t24b_dir);
	(void) rmdir(t24a_dir);
	(void) unlink(t25_file);
	(void) rmdir(t25_dir);
#if	  !TEST_XNFS
	(void) unlink(t26_file);
#endif	/* TEST_XNFS */
	(void) unlink(t27a_file);
	(void) unlink(t27b_file);
	(void) unlink(t28b_file);
	(void) unlink(t29b_file);
	(void) unlink(t30_file);
	(void) unlink(t31b_file);
	(void) unlink(t31_ndir);
	(void) unlink(t32_file);
	(void) rmdir(t32_dir);
	(void) unlink(t34a_file);
	(void) unlink(t17_dir);
	(void) rmdir(t35a_file);
	(void) rmdir(t35b_file);
	(void) rmdir(t35_dir);
#if	   TEST_XNFS
	(void) unlink(t36_file);
	(void) unlink(t37_file);
	(void) unlink(t38_file);
	(void) unlink(t39_file);
#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* pathname resolution */

	char *p1, *p2;
	int npairs, i, err;
	int	pathok = 0;
	int ret1, ret2;

	DBUG_ENTER("test1");

	testfail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOT|PRES_NODOTDOT);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOT|PRES_NODOTDOT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Rename using first name then rename back using second name */
	/* then repeat the other way round */

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		if (crdir(p2, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", p2);
			continue;
		}
		else
			PATH_TRACE;

		if (stat(p2, &stbuf1) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				p2, err, errname(err));
			continue;
		}
		else
			PATH_TRACE;

		globok = 0;
		ret1 = tbrename(p1, t1_file, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		ret2 = tbrename(t1_file, p2, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		
		if (ret1 != SUCCEED || ret2 != SUCCEED)
		{
			PATH_TRACE; PATH_TRACE; 	/* Must account for the 4 PATH_TRACEs below  */
			PATH_TRACE; PATH_TRACE; 	/* Must account for the PATH_TRACEs below */
			continue;	/* As this is a valid path */
		}

#ifdef TESTBED2
#define p2 "."
#endif
		if (stat(p2, &stbuf2) == SYSERROR ||
			stbuf1.st_dev != stbuf2.st_dev ||
			stbuf1.st_ino != stbuf2.st_ino)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("rename() did not resolve old=\"%s\" and new=\"%s\" to the same file",
				p1, p2);
				
			continue;
		}
		else
			PATH_TRACE;
#ifdef TESTBED2
#undef p2
#endif

		globok = 0;
		ret1 = tbrename(p2, t1_file, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		ret2 = tbrename(t1_file, p1, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;

		if (ret1 != SUCCEED || ret2 != SUCCEED)
		{
			PATH_TRACE; 	/* Must account for the PATH_TRACE below */
			continue;	/* As this is a valid path */
		}

		if (stat(p2, &stbuf2) == SYSERROR ||
			stbuf1.st_dev != stbuf2.st_dev ||
			stbuf1.st_ino != stbuf2.st_ino)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("rename() did not resolve old=\"%s\" and new=\"%s\" to the same file",
				p2, p1);
				
			continue;
		}
		else
			PATH_TRACE;
			
	}

	if (testfail == 0)
		PATH_XS_RPT(1 + (npairs * 8));
	
	(void) remove(t1_file);
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames for "old" */

	char	**p;
	int	fd;
	int	pathok = 0;
	int	loop;
		
	DBUG_ENTER("test2");

	testfail = 0;

	/* Create all the files then rename them one by one.
	   Duplicates will fail with ENOENT */

	for (loop = 0, p = portfnam; *p != NULL; p++, loop++)
	{
		if ((fd = crfile(*p, MODEANY,
				(int)geteuid(), (int)getegid())) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create file \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		(void) close(fd);
	}

#ifdef TESTBED2
	(void) unlink(portfnam[0]);
#endif

	for (p = portfnam; *p != NULL; p++,loop++)
	{
		globok = 0;
		(void) tbrename(*p, t2_file, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		(void) unlink(t2_file);
	}

	if (testfail == 0)
		PATH_XS_RPT(loop); 

	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);
	(void) unlink(t2_file);
	
	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* portable filenames for "new" */

	char	**p;
	int	fd;
	int	pathok = 0;
	int 	loop = 0;
	int 	loop2 = 0;
	
	DBUG_ENTER("test3");

	testfail = 0;

	/* Rename a non-empty directory to each name.  Duplicates
	   will fail with ENOTEMPTY or EEXIST */

	for (p = portfnam; *p != NULL; p++)
	{
#ifdef TESTBED2
#define p portfnam
#endif
		loop++;				/* Iterations of this loop */
		if (crdir(t3_dir, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t3_dir);
			break;
		}
		else
			PATH_TRACE;

		if ((fd = crfile(t3_file, MODEANY,
				(int)geteuid(), (int)getegid())) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create file \"%s\"",
				t3_file);
			break;
		}
		else
			PATH_TRACE;
		(void) close(fd);

		globok = 0;
		(void) tbrename(t3_dir, *p, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		
#ifdef TESTBED2
#undef p
#endif
	}

	for (p = portfnam; *p != NULL; p++)
	{
		(void) rename(*p, t3_dir);
		(void) unlink(t3_file);
		(void) rmdir(t3_dir);
	}

#if	   TEST_XNFS

	for (p = portfnam; *p != NULL; p++)
	{
		PATH_TRACE; 
		PATH_TRACE; 
		PATH_TRACE;
		loop2++;		/* Iteration of this loop */
	}

#else	/* TEST_XNFS */


	for (p = portfnam; *p != NULL; p++)
	{
#ifdef TESTBED2
#define p portfnam
#endif
		loop2++;		/* Iteration of this loop */
		if (crdir(t3_dir, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t3_dir);
			break;
		}
		else
			PATH_TRACE;

		if (mkfifo(t3_file, MODEANY) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create fifo \"%s\"",
				t3_file);
			break;
		}
		else
			PATH_TRACE;

		globok = 0;
		(void) tbrename(t3_dir, *p, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		
#ifdef TESTBED2
#undef p
#endif
	}

	for (p = portfnam; *p != NULL; p++)
	{
		(void) rename(*p, t3_dir);
		(void) unlink(t3_file);
		(void) rmdir(t3_dir);
	}

#endif	/* TEST_XNFS */
	
	if (testfail == 0)
		PATH_XS_RPT((loop * 3) + (loop2 * 3)); 

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* rename() changes file name "old" to "new" */

	int	fd;
	int	pathok = 0;
	
	DBUG_ENTER("test4");

	testfail = 0;

	if ((fd = crfile(t4a_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t4a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	/* tbrename() checks dev & ino are same for "old" and "new" */

	globok = 0;
	(void) tbrename(t4a_file, t4b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t4a_file);
	(void) unlink(t4b_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t4a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t4a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* tbrename() checks dev & ino are same for "old" and "new" */

	globok = 0;
	(void) tbrename(t4a_file, t4b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t4a_file);
	(void) unlink(t4b_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* old and new are links to same file */

	int	fd, rval, err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	if ((fd = crfile(t5a_file, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t5a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	if (link(t5a_file, t5b_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
			t5a_file, t5b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(t5a_file, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t5a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* don't use tbrename(): expects "new" to be removed */

	rval = rename(t5a_file, t5b_file);
	err = errno;
#ifdef TESTBED
	rval = -1;
	err++;
#endif
	if (rval != SUCCEED)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("rename(\"%s\", \"%s\") did not give correct results",
			t5a_file, t5b_file);
		valrpt(SUCCEED, rval, NOERROR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
#ifdef TESTBED2
#define t5a_file "."
#define t5b_file "."
#endif
	if (stat(t5a_file, &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t5a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if (stbuf1.st_dev != stbuf2.st_dev ||
		 stbuf1.st_ino != stbuf2.st_ino)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("rename(\"%s\", \"%s\") changed file \"%s\"",
			t5a_file, t5b_file, t5a_file);
	}
	else
		PATH_TRACE;

	if (stat(t5b_file, &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t5b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if (stbuf1.st_dev != stbuf2.st_dev ||
		 stbuf1.st_ino != stbuf2.st_ino)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("rename(\"%s\", \"%s\") changed file \"%s\"",
			t5a_file, t5b_file, t5b_file);
	}
	else
		PATH_TRACE;
		
#ifdef TESTBED2
#undef t5a_file
#undef t5b_file
#endif

	(void) unlink(t5a_file);
	(void) unlink(t5b_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t5a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t5a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (link(t5a_file, t5b_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
			t5a_file, t5b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(t5a_file, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t5a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* don't use tbrename(): expects "new" to be removed */

	rval = rename(t5a_file, t5b_file);
	err = errno;
#ifdef TESTBED
	rval = -1;
	err++;
#endif
	if (rval != SUCCEED)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("rename(\"%s\", \"%s\") did not give correct results",
			t5a_file, t5b_file);
		valrpt(SUCCEED, rval, NOERROR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
#ifdef TESTBED2
#define t5a_file "."
#define t5b_file "."
#endif
	if (stat(t5a_file, &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t5a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if (stbuf1.st_dev != stbuf2.st_dev ||
		 stbuf1.st_ino != stbuf2.st_ino)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("rename(\"%s\", \"%s\") changed file \"%s\"",
			t5a_file, t5b_file, t5a_file);
	}
	else
		PATH_TRACE;

	if (stat(t5b_file, &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t5b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if (stbuf1.st_dev != stbuf2.st_dev ||
		 stbuf1.st_ino != stbuf2.st_ino)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("rename(\"%s\", \"%s\") changed file \"%s\"",
			t5a_file, t5b_file, t5b_file);
	}
	else
		PATH_TRACE;
		
#ifdef TESTBED2
#undef t5a_file
#undef t5b_file
#endif

	(void) unlink(t5a_file);
	(void) unlink(t5b_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(12); 
	
	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* "new" already exists as a plain file */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;

	if ((fd = crfile(t6a_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t6a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	if ((fd = crfile(t6b_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t6b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbrename(t6a_file, t6b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t6a_file);
	(void) unlink(t6b_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t6a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t6a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (mkfifo(t6b_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t6b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrename(t6a_file, t6b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t6a_file);
	(void) unlink(t6b_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* "new" already exists as a directory */
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	if (crdir(t7a_file, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t7a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		

	if (crdir(t7b_file, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t7b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		

	globok = 0;
	(void) tbrename(t7a_file, t7b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) rmdir(t7a_file);
	(void) rmdir(t7b_file);

	if (testfail == 0)
		PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}


private void
test8()
{
	/* existing "new" file is still accessible if open */

	int fd;
#if !TEST_XNFS
	int fd2;
#endif /* TEST_XNFS */
	int err, rval;
	static char wbuf[] = "That hoopy Ford Prefect - he's a frood\
			      who really knows where his towel is!";
	char rbuf[sizeof(wbuf)];
	int	pathok = 0;
	

	DBUG_ENTER("test8");

	testfail = 0;

	if ((fd = crfile(t8a_file, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t8a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if ((fd = crfile(t8b_file, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t8b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, wbuf, sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() failed - errno %d (%s)", err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	
	if ((fd = open(t8b_file, O_RDONLY)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY) failed - errno %d (%s)",
			t8b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (tbrename(t8a_file, t8b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	fd = -fd;
#endif

	if ((rval = read(fd, rbuf, sizeof(wbuf))) != sizeof(wbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on unlinked file returned %d, expected %d",
			rval, (int)sizeof(wbuf));
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (memcmp((void *)rbuf, (void *)wbuf, sizeof(wbuf)) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from unlinked file returned incorrect data");
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	fd = -fd;
#endif

	(void) close(fd);
	(void) unlink(t8a_file);
	(void) unlink(t8b_file);

#if	   TEST_XNFS

	PATH_TRACE; PATH_TRACE;
	PATH_TRACE; PATH_TRACE;
	PATH_TRACE; PATH_TRACE;
	PATH_TRACE; PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t8a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t8a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (mkfifo(t8b_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t8b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd2 = open(t8b_file, O_RDONLY|O_NONBLOCK)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) failed - errno %d (%s)",
			t8b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fd = open(t8b_file, O_WRONLY|O_NONBLOCK)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_WRONLY|O_NONBLOCK) failed - errno %d (%s)",
			t8b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (write(fd, wbuf, sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() failed - errno %d (%s)", err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if ((fd = open(t8b_file, O_RDONLY|O_NONBLOCK)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) failed - errno %d (%s)",
			t8b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		

	globok = 0; 
	if (tbrename(t8a_file, t8b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		

#ifdef TESTBED2
	fd = -fd;
#endif

	if ((rval = read(fd, rbuf, sizeof(wbuf))) != sizeof(wbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("*** read on fifo ***");
		in_rpt("read() on unlinked file returned %d, expected %d",
			rval, (int)sizeof(wbuf));
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (memcmp((void *)rbuf, (void *)wbuf, sizeof(wbuf)) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from unlinked file returned incorrect data");
	}
	else
		PATH_TRACE;
		

#ifdef TESTBED2
	fd = -fd;
#endif

	(void) close(fd);
	(void) close(fd2);

	(void) unlink(t8a_file);
	(void) unlink(t8b_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(14); 
	
	DBUG_VOID_RETURN;
}


private void
test9()
{
	/* rename() updates parent directory timestamps */

	int	fd, err;
	int	pathok = 0;

	DBUG_ENTER("test9");

	testfail = 0;

	if ((fd = crfile(t9a_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t9a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (stat(".", &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\".\") failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) sleep(2);

#ifndef TESTBED2

	globok = 0; 
	if (tbrename(t9a_file, t9b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
#else
	PATH_TRACE; 		/* Valid path */
#endif

	if (stat(".", &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf2.st_ctime == stbuf1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was not updated");
	}
	else
		PATH_TRACE;
	
	if (stbuf2.st_mtime == stbuf1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was not updated");
	}
	else
		PATH_TRACE;
		
	(void) unlink(t9a_file);
	(void) unlink(t9b_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t9a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t9a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(".", &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\".\") failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) sleep(1);

#ifndef TESTBED2

	globok = 0; 
	if (tbrename(t9a_file, t9b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
#else
	PATH_TRACE; 		/*Valid path */
#endif

	if (stat(".", &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf2.st_ctime == stbuf1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	if (stbuf2.st_mtime == stbuf1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	(void) unlink(t9a_file);
	(void) unlink(t9b_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(12); 

	DBUG_VOID_RETURN;
}

private void
test10()
{
	/* "old" is a non-writable directory */

	char	*reqd;
	mode_t	mode = (mode_t)(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = 0;

	reqd = tet_getvar("VSX_RENAME_DIR_WPERM_REQD");
	if (reqd == NULL || *reqd == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_RENAME_DIR_WPERM_REQD not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*reqd == 'Y' || *reqd == 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_RENAME_DIR_WPERM_REQD is set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (crdir(t10a_file, mode,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t10a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrename(t10a_file, t10b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);
		
	(void) rmdir(t10a_file);
	(void) rmdir(t10b_file);

	DBUG_VOID_RETURN;
}

private void
test11()
{
	/* "new" already exists as a non-writable directory */

	char	*reqd;
	mode_t	mode = (mode_t)(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = 0;

	reqd = tet_getvar("VSX_RENAME_DIR_WPERM_REQD");
	if (reqd == NULL || *reqd == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_RENAME_DIR_WPERM_REQD not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (*reqd == 'Y' || *reqd == 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_RENAME_DIR_WPERM_REQD is set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t11a_file, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t11a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t11b_file, mode,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t11b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrename(t11a_file, t11b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 

	(void) rmdir(t11a_file);
	(void) rmdir(t11b_file);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	/* "old" is a busy directory */

	char	*supp, *supp2;
	int	ret;
	int	pathok = 0;
	int 	retval;

	DBUG_ENTER("test12");

	testfail = 0;

	if ((supp = tet_getvar("VSX_RENAME_DIR_EBUSY")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_RENAME_DIR_EBUSY not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (*supp == 'Y' || *supp == 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_RENAME_DIR_EBUSY is set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*supp != 'Y' && *supp != 'y' && *supp != 'S' && *supp != 's')
	{
	    /* can only do mount point test if linking across file
	       systems is OK (otherwise it would give EXDEV) */

	    supp2 = tet_getvar("VSX_LINK_FILESYS_SUPP");
	    if (supp2 != NULL && (*supp2 == 'Y' || *supp2 == 'y'))
	    {
		if (crdir(t12a_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t12a_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0; 
		retval = cppair(ch1_t12, NULLFN, WAITTIME, NOK|PATH_NOK);
		if (retval == NOK)
			testfail++;
		else if (retval == PATH_NOK)
			retval = OK;
		else
			PATH_TRACE; 		/* Child ok & check cppair*/
	    }
	    else
	    {
		PATH_TRACE; PATH_TRACE; 	/* Valid Alternate paths */
	    }
	}
	else
	{
	    	PATH_TRACE; PATH_TRACE; 	/* Valid Alternate paths */
	}

	if (*supp != 'Y' && *supp != 'y' && *supp != 'P' && *supp != 'p')
	{
		if (crdir(t12c_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t12c_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0; 
		ret = cppair(ch2_t12, pa_t12, WAITTIME, E_CHDIR|E_OPEN|PATH_NOK);
		if (ret == E_CHDIR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: chdir(\"%s\") failed in child",
				t12c_file);
		}
		else if (ret == E_OPEN)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: open(\".\", O_RDONLY) failed in child");
		}
		else if (ret == PATH_NOK)
			ret = OK;
		else
			PATH_FUNC_TRACE; 
	}
	else
	{
		PATH_TRACE; PATH_TRACE;
	}
		
	if (testfail == 0)
		PATH_XS_RPT(6); 

	(void) rmdir(t12a_file);
	(void) rmdir(t12b_file);
	(void) rmdir(t12c_file);
	(void) rmdir(t12d_file);

	DBUG_VOID_RETURN;
}

private void
ch1_t12()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch1_t12");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t12a_file;
	args[3] = t12b_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

private void
pa_t12()
{
	int	pathok = 0;
	
	DBUG_ENTER("pa_t12");

	(void) sleep(WAITTIME/2);

	globok = 0;
	(void) tbrename(t12c_file, t12d_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1); 		/* Needed for cppair */
	
	DBUG_VOID_RETURN;
}

private void
ch2_t12()
{
	int	pathok = 0;
	
	DBUG_ENTER("ch2_t12");

#ifdef TESTBED2
	DBUG_EXIT(E_CHDIR);
#endif

	if (chdir(t12c_file) == SYSERROR)
	{
		DBUG_EXIT(E_CHDIR);
	}
	else
		PATH_TRACE;

	if (open(".", O_RDONLY) < 0)
	{
		DBUG_EXIT(E_OPEN);
	}
	else
		PATH_TRACE;
	
	(void) sleep(WAITTIME);

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);		/* Ok, but path trace wrong */

	DBUG_VOID_RETURN;
}

private void
test13()
{
	/* "new" exists as a busy directory */

	char	*supp;
	int	ret;
	int	pathok = 0;
	int	retval;

	DBUG_ENTER("test13");

	testfail = 0;

	if ((supp = tet_getvar("VSX_REMOVE_DIR_EBUSY")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (*supp == 'Y' || *supp == 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY is set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*supp != 'Y' && *supp != 'y' && *supp != 'S' && *supp != 's')
	{
		if (crdir(t13a_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t13a_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0; 
		retval = cppair(ch1_t13, NULLFN, WAITTIME, NOK|PATH_NOK);
		if (retval == NOK)
			testfail++;
		else if (retval == PATH_NOK)
			retval = OK;
		else
			PATH_TRACE; 		/* Child ok & check cppair */
	}
	else
	{
		PATH_TRACE; PATH_TRACE; 	/* Valid alternate path */
	}

	if (*supp != 'Y' && *supp != 'y' && *supp != 'P' && *supp != 'p')
	{
		if (crdir(t13c_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t13c_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (crdir(t13d_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t13d_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0; 
		ret = cppair(ch2_t13, pa_t13, WAITTIME, E_CHDIR|E_OPEN|PATH_NOK);
		if (ret == E_CHDIR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: chdir(\"%s\") failed in child",
				t13d_file);
		}
		else if (ret == E_OPEN)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: open(\".\", O_RDONLY) failed in child");
		}
		else if (ret == PATH_NOK)
			ret = OK;
		else
			PATH_FUNC_TRACE; 
	}
	else
	{
		PATH_TRACE;PATH_TRACE; PATH_TRACE; 	/* Valid alternate path */
	}

	if (testfail == 0)
		PATH_XS_RPT(7); 

	(void) rmdir(t13a_file);
	(void) rmdir(t13b_file);
	(void) rmdir(t13c_file);
	(void) rmdir(t13d_file);

	DBUG_VOID_RETURN;
}

private void
ch1_t13()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch1_t13");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t13a_file;
	args[3] = t13b_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

private void
pa_t13()
{
	int	pathok = 0;
	
	DBUG_ENTER("pa_t13");

	(void) sleep(WAITTIME/2);

	globok = 0;
	(void) tbrename(t13c_file, t13d_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
ch2_t13()
{
	int	pathok = 0;
	
	DBUG_ENTER("ch2_t13");

#ifdef TESTBED2
	DBUG_EXIT(E_CHDIR);
#endif

	if (chdir(t13d_file) == SYSERROR)
	{
		DBUG_EXIT(E_CHDIR);
	}
	else
		PATH_TRACE;

	if (open(".", O_RDONLY) < 0)
	{
		DBUG_EXIT(E_OPEN);
	}
	else
		PATH_TRACE;
	
	(void) sleep(WAITTIME);
	 if (pathok != 2)
		DBUG_EXIT(PATH_NOK);	/* OK, but path trace wrong */

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test14()
{
	DBUG_ENTER("test14");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test14()
{
	/* rename across file systems */

	char	*supp;
	int	fd;
	int	pathok = 0;
	int	retval;

	DBUG_ENTER("test14");

	if ((supp = tet_getvar("VSX_LINK_FILESYS_SUPP")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_LINK_FILESYS_SUPP not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (*supp != 'Y' && *supp != 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_LINK_FILESYS_SUPP not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t14a_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t14a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	retval = cppair(ch_t14, NULLFN, 4 * WAITTIME, NOK|PATH_NOK);
	if (retval == NOK)
	{
		DBUG_VOID_RETURN;
	}
	else if (retval == PATH_NOK)
		retval = OK;
	else
		PATH_TRACE; 

	(void) unlink(t14a_file);

	if (mkfifo(t14a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t14a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	retval = cppair(ch_t14, NULLFN, 4 * WAITTIME, NOK|PATH_NOK);
	if (retval == NOK)
	{
		DBUG_VOID_RETURN;
	}
	else if (retval == PATH_NOK)
		retval = OK;
	else
		PATH_TRACE; 

	(void) unlink(t14a_file);

	if (testfail == 0)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
ch_t14()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t14");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t14a_file;
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test15()
{
	/* access granted to "old" */

	int waittime;

	DBUG_ENTER("test15");

	waittime = WAITTIME * (5 + sysconf(_SC_NGROUPS_MAX)/4);

	(void) cppair(ch_t15_16, NULLFN, 2 * waittime, 0);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	/* access granted to "new" */

	int waittime;

	DBUG_ENTER("test16");

	waittime = WAITTIME * (5 + sysconf(_SC_NGROUPS_MAX)/4);

	(void) cppair(ch_t15_16, NULLFN, 2 * waittime, NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t15_16()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t15_16");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test17()
{

	DBUG_ENTER("test17");

	(void) cppair(ch_t17, NULLFN, 4 * WAITTIME + 20, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t17()
{
	int 	err;
	char	*args[6];

	DBUG_ENTER("ch_t17");

	args[0] = execpath(prog17);
	args[1] = t17_dir;
	args[2] = "rename";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog17), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}


private void
test18()
{
	/* search permission denied on either path prefix */

	int	fd;
	mode_t	mode = (mode_t)(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
	int	pathok = 0;
#if	   TEST_XNFS
	int	ret;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test18");

	testfail = 0;

	if (crdir(t18_dir, mode,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory %s", t18_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t18a_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t18a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbrename(t18a_file, t18b_file, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbrename(t18b_file, t18a_file, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) unlink(t18a_file);
	(void) rmdir(t18_dir);

#if	   TEST_XNFS

	/*
	 * This part of the test revolves around a directory that denies
	 * search permission for the anonymous user.
	 *
	 * Here's what we are trying to achieve:
	 *
	 * File 2a exists and is in the directory
	 * File 2b does not exist, but path is outside the directory.
	 * Renaming 2a to 2b should give EACCES.
	 * Note - File 2a is created on the server side.
	 *
	 * File 2c exists, but path is outside the directory.
	 * File 2d does not exist and path is in the directory
	 * Renaming 2c to 2d should give EACCES.
	 *
	 */

	if (tet_sync(SP_T18_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t18_file2c, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t18_file2c);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	ret = tet_fork(ch_t18, NULLFN, WAITTIME, OK | NOK | PATH_NOK);

	if ((ret & OK) == 0)
	{
		testfail++;
	}
	else if ((ret & PATH_NOK) == 0)
	{
		PATH_TRACE; 
	}

	if (tet_sync(SP_T18_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) unlink(t18_file2a);
	(void) unlink(t18_file2b);
	(void) unlink(t18_file2c);
	(void) unlink(t18_file2d);

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(8); 

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t18()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t18");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}
#endif	/* TEST_XNFS */


private void
test19()
{
	/* write permission denied in directory containing either file */

	int	fd, err;
	mode_t	mode = (S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	int	pathok = 0;
#if	   TEST_XNFS
	int	ret;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test19");

	testfail = 0;

	if (crdir(t19_dir, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory %s", t19_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t19a_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t19a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (chmod(t19_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t19_dir, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t19b_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t19b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	globok = 0; 
	if (tbrename(t19a_file, t19b_file, SYSERROR, EACCES) != SYSERROR)
	{
		if ((fd = crfile(t19b_file, MODEANY,
				(int) geteuid(), (int) getegid())) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create file \"%s\"",
				t19b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE; 
			(void) close(fd);
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	(void) tbrename(t19b_file, t19a_file, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) chmod(t19_dir, MODEANY);
	(void) unlink(t19a_file);
	(void) unlink(t19b_file);
	(void) rmdir(t19_dir);

#if	   TEST_XNFS

	/*
	 * This part of the test revolves around a directory that denies
	 * write permission for the anonymous user.
	 *
	 * Here's what we are trying to achieve:
	 *
	 * File 2a exists and is in the directory
	 * File 2b does not exist, but path is outside the directory.
	 * Renaming 2a to 2b should give EACCES.
	 * Note - File 2a is created on the server side.
	 *
	 * File 2c exists, but path is outside the directory.
	 * File 2d does not exists and path is in the directory
	 * Renaming 2c to 2d should give EACCES.
	 *
	 */

	if (tet_sync(SP_T19_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t19_file2c, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t19_file2c);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	ret = tet_fork(ch_t19, NULLFN, WAITTIME, OK | NOK | PATH_NOK);

	if ((ret & OK) == 0)
	{
		testfail++;
	}
	else if ((ret & PATH_NOK) == 0)
	{
		PATH_TRACE; 
	}

	if (tet_sync(SP_T19_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) unlink(t19_file2a);
	(void) unlink(t19_file2b);
	(void) unlink(t19_file2c);
	(void) unlink(t19_file2d);

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(10); 

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t19()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t19");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}
#endif	/* TEST_XNFS */


private void
test20()
{
	/* write permission denied for existing directories */

	char	*reqd;
	mode_t	mode = (mode_t)(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	int	pathok = 0;
#if	   TEST_XNFS
	int	ret;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test20");

	testfail = 0;

	reqd = tet_getvar("VSX_RENAME_DIR_WPERM_REQD");
	if (reqd == NULL || *reqd == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_RENAME_DIR_WPERM_REQD not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*reqd != 'Y' && *reqd != 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_RENAME_DIR_WPERM_REQD not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t20a_dir, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t20a_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t20b_dir, mode,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t20b_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	globok = 0; 
	if (tbrename(t20a_dir, t20b_dir, SYSERROR, EACCES) != SYSERROR)
	{
		if (crdir(t20b_dir, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create directory \"%s\"",
				t20b_dir);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE; 
	}
	else
		PATH_FUNC_TRACE; 
		

	globok = 0;
	(void) tbrename(t20b_dir, t20a_dir, SYSERROR, EACCES);
	PATH_FUNC_TRACE;
	
	(void) rmdir(t20a_dir);
	(void) rmdir(t20b_dir);

#if	   TEST_XNFS

	/*
	 * Directory 2a exists and is writeable
	 * Directory 2b exists but denies write permission for 
	 * the anonymous user.
	 * Renaming 2a to 2b should give EACCES.
	 * Note - Directory 2b is created on the server side.
	 *
	 * Directory 2c exists but denies write permission for 
	 * the anonymous user.
	 * Directory 2d exists and is writeable
	 * Renaming 2c to 2d should give EACCES.
	 * Note - Directory 2c is created on the server side.
	 *
	 */


	if (crdir(t20_dir2a, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t20_dir2a);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (crdir(t20_dir2d, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t20_dir2d);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (tet_sync(SP_T20_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ret = tet_fork(ch_t20, NULLFN, WAITTIME, OK | NOK | PATH_NOK);

	if ((ret & OK) == 0)
	{
		testfail++;
	}
	else if ((ret & PATH_NOK) == 0)
	{
		PATH_TRACE; 
	}

	if (tet_sync(SP_T20_RMDIR2_START, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) rmdir(t20_dir2a);
	(void) rmdir(t20_dir2d);

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(11); 

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t20()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t20");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}
#endif	/* TEST_XNFS */


private void
test21()
{
	/* "old" is a busy directory */

	char	*supp, *supp2;
	int	ret;
	int	pathok = 0;
	int	retval;
	
	DBUG_ENTER("test21");

	testfail = 0;

	if ((supp = tet_getvar("VSX_RENAME_DIR_EBUSY")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_RENAME_DIR_EBUSY not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (*supp == 'N' || *supp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_RENAME_DIR_EBUSY is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*supp == 'Y' || *supp == 'y' || *supp == 'S' || *supp == 's')
	{
	    /* can only do mount point test if linking across file
	       systems is OK (otherwise it could equally well give EXDEV) */

	    supp2 = tet_getvar("VSX_LINK_FILESYS_SUPP");
	    if (supp2 != NULL && (*supp2 == 'Y' || *supp2 == 'y'))
	    {
		if (crdir(t21a_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t21a_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0; 
		retval = cppair(ch1_t21, NULLFN, WAITTIME, NOK|PATH_NOK);
		if (retval == NOK)
			testfail++;
		else if (retval == PATH_NOK)
			ret = OK;
		else
			PATH_TRACE; 
	    }
	    else
	    {
	    	PATH_TRACE; PATH_TRACE; /* Valid alternative path */
	    }
	}
	else
	{
	    	PATH_TRACE; PATH_TRACE; /* Valid alternative path */
	}

	if (*supp == 'Y' || *supp == 'y' || *supp == 'P' || *supp == 'p')
	{
		if (crdir(t21c_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t21c_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0; 
		ret = cppair(ch2_t21, pa_t21, WAITTIME, E_CHDIR|E_OPEN|PATH_NOK);
		if (ret == E_CHDIR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: chdir(\"%s\") failed in child",
				t21c_file);
		}
		else if (ret == E_OPEN)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: open(\".\", O_RDONLY) failed in child");
		}
		else if (ret == PATH_NOK)
			ret = OK;
		else
			PATH_FUNC_TRACE; 
	}
	else
	{
		PATH_TRACE; PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(6); 

	(void) rmdir(t21a_file);
	(void) rmdir(t21b_file);
	(void) rmdir(t21c_file);
	(void) rmdir(t21d_file);

	DBUG_VOID_RETURN;
}

private void
ch1_t21()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch1_t21");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t21a_file;
	args[3] = t21b_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

private void
pa_t21()
{
	int	pathok = 0;
	DBUG_ENTER("pa_t21");

	(void) sleep(WAITTIME/2);

	globok = 0;
	(void) tbrename(t21c_file, t21d_file, SYSERROR, EBUSY);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
ch2_t21()
{
	int	pathok = 0;
	DBUG_ENTER("ch2_t21");

#ifdef TESTBED2
	DBUG_EXIT(E_CHDIR);
#endif

	if (chdir(t21c_file) == SYSERROR)
	{
		DBUG_EXIT(E_CHDIR);
	}
	else
		PATH_TRACE;

	if (open(".", O_RDONLY) < 0)
	{
		DBUG_EXIT(E_OPEN);
	}
	else
		PATH_TRACE;
	
	(void) sleep(WAITTIME);

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
test22()
{
	/* "new" exists as a busy directory */

	char	*supp;
	int	ret;
	int	pathok = 0;
	int 	retval;

	DBUG_ENTER("test22");

	testfail = 0;

	if ((supp = tet_getvar("VSX_REMOVE_DIR_EBUSY")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (*supp == 'N' || *supp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*supp == 'Y' || *supp == 'y' || *supp == 'S' || *supp == 's')
	{
		if (crdir(t22a_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t22a_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		retval = cppair(ch1_t22, NULLFN, WAITTIME, NOK|PATH_NOK);
		if (retval == NOK)
			testfail++;
		else if (retval == PATH_NOK)
			ret = OK;
		else
			PATH_TRACE; 
	}
	else
	{
		PATH_TRACE; PATH_TRACE;
	}

	if (*supp == 'Y' || *supp == 'y' || *supp == 'P' || *supp == 'p')
	{
		if (crdir(t22c_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t22c_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
			
		if (crdir(t22d_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t22d_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
			
		globok = 0; 
		ret = cppair(ch2_t22, pa_t22, WAITTIME, E_CHDIR|E_OPEN|PATH_NOK);
		if (ret == E_CHDIR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: chdir(\"%s\") failed in child",
				t22d_file);
		}
		else if (ret == E_OPEN)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: open(\".\", O_RDONLY) failed in child");
		}
		else if (ret == PATH_NOK)
			ret = OK;
		else
			PATH_FUNC_TRACE; 
	}
	else
	{
		PATH_TRACE; PATH_TRACE; PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(7);
		
	(void) rmdir(t22a_file);
	(void) rmdir(t22b_file);
	(void) rmdir(t22c_file);
	(void) rmdir(t22d_file);

	DBUG_VOID_RETURN;
}

private void
ch1_t22()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch1_t22");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t22a_file;
	args[3] = t22b_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

private void
pa_t22()
{
	int	pathok = 0;
	DBUG_ENTER("pa_t22");
	
	(void) sleep(WAITTIME/2);

	globok = 0;
	(void) tbrename(t22c_file, t22d_file, SYSERROR, EBUSY);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
ch2_t22()
{
	int	pathok = 0;
	DBUG_ENTER("ch2_t22");
	

#ifdef TESTBED2
	DBUG_EXIT(E_CHDIR);
#endif

	if (chdir(t22d_file) == SYSERROR)
	{
		DBUG_EXIT(E_CHDIR);
	}
	else
		PATH_TRACE;

	if (open(".", O_RDONLY) < 0)
	{
		DBUG_EXIT(E_OPEN);
	}
	else
		PATH_TRACE;
	
	(void) sleep(WAITTIME);
	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
test23()
{
	/* "new" exists as a non-empty directory */

	int fd;
	int	pathok = 0;

	DBUG_ENTER("test23");

	testfail = 0;

	if (crdir(t23a_dir, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t23a_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t23b_dir, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t23b_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t23_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t23_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbrename(t23a_dir, t23b_dir, SYSERROR, EEXIST);
	PATH_FUNC_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(4); 

	(void) unlink(t23_file);
	(void) rmdir(t23a_dir);
	(void) rmdir(t23b_dir);

	DBUG_VOID_RETURN;
}

private void
test24()
{
	/* "new" is a directory below "old" */
	int	pathok = 0;

	DBUG_ENTER("test24");

	testfail = 0;

	if (crdir(t24a_dir, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t24a_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t24b_dir, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t24b_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrename(t24a_dir, t24b_dir, SYSERROR, EINVAL);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3); 

	(void) rmdir(t24b_dir);
	(void) rmdir(t24a_dir);

	DBUG_VOID_RETURN;
}

private void
test25()
{
	/* "new" is a directory, "old" is not */

	int fd;
	int	pathok = 0;

	DBUG_ENTER("test25");

	testfail = 0;

	if (crdir(t25_dir, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t25_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t25_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t25_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbrename(t25_file, t25_dir, SYSERROR, EISDIR);
	PATH_FUNC_TRACE;

	(void) unlink(t25_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t25_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t25_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrename(t25_file, t25_dir, SYSERROR, EISDIR);
	PATH_FUNC_TRACE;

	(void) unlink(t25_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(5); 

	(void) rmdir(t25_dir);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test26()
{
	DBUG_ENTER("test26");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#else	/* TEST_XNFS */

private void
test26()
{
	/* old or new has path component longer than NAME_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test26");

	testfail = 0;

	globok = 0; 
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	if ((fd = crfile(longname[0], MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			longname[0]);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbrename(longname[0], t26_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	
	(void) unlink(t26_file);

	globok = 0; 
	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbrename(longname[1], t26_file, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;
	(void) unlink(longname[0]);
		
	if ((fd = crfile(t26_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t26_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbrename(t26_file, longname[0], SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (longname[1] != NULL)
	{
		if ((fd = crfile(t26_file, MODEANY,
				(int) geteuid(), (int) getegid())) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create file \"%s\"",
				t26_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		(void) close(fd);

		globok = 0;
		(void) tbrename(t26_file, longname[1], SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
	{
		PATH_TRACE; PATH_TRACE;
	}
		
	if (testfail == 0)
		PATH_XS_RPT(8); 

	(void) unlink(longname[0]);
	clntoolng();
	(void) unlink(t26_file);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test27()
{
	/* old or new path longer than PATH_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test27");

	testfail = 0;
	
	globok = 0; 
	longname = pathtoolng(t27a_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	if ((fd = crfile(t27a_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t27a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbrename(longname[0], t27b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	
	(void) unlink(t27b_file);

	if (longname[1] != NULL)
	{

		globok = 0;
		(void) tbrename(longname[1], t27b_file, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
		
	}
	else
		PATH_TRACE;
	(void) unlink(t27a_file);
		
	if ((fd = crfile(t27b_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t27b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbrename(t27b_file, longname[0], SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (longname[1] != NULL)
	{
		if ((fd = crfile(t27b_file, MODEANY,
				(int) geteuid(), (int) getegid())) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create file \"%s\"",
				t27b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		(void) close(fd);

		globok = 0;
		(void) tbrename(t27b_file, longname[1], SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
		
	}
	else
	{
		PATH_TRACE;PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(8); 

	clntoolng();
	(void) unlink(t27a_file);
	(void) unlink(t27b_file);

	DBUG_VOID_RETURN;
}

private void
test28()
{
	/* component of path prefix for old or new does not exist */

	int fd;
	int	pathok = 0;

	DBUG_ENTER("test28");

	testfail = 0;

	if (eaccess(t28_dir, F_OK) == 0 && remove(t28_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t28_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t28b_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t28b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0; 
	if (tbrename(t28a_file, t28b_file, SYSERROR, ENOENT) != SYSERROR)
	{
		if ((fd = crfile(t28b_file, MODEANY,
				(int) geteuid(), (int) getegid())) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create file \"%s\"",
				t28b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE; 
		(void) close(fd);
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	(void) tbrename(t28b_file, t28a_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(4); 

	(void) unlink(t28b_file);

	DBUG_VOID_RETURN;
}

private void
test29()
{
	/* "old" does not exist */

	int fd;
	int	pathok = 0;

	DBUG_ENTER("test29");

	testfail = 0;

	if (eaccess(t29a_file, F_OK) == 0 && remove(t29a_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove %s", t29a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t29b_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t29b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbrename(t29a_file, t29b_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3); 
	
	(void) unlink(t29b_file);

	DBUG_VOID_RETURN;
}

private void
test30()
{
	/* old or new path is "" */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test30");

	testfail = 0;

	if ((fd = crfile(t30_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t30_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0; 
	if (tbrename("", t30_file, SYSERROR, ENOENT) != SYSERROR)
	{
		if ((fd = crfile(t30_file, MODEANY,
				(int) geteuid(), (int) getegid())) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create file \"%s\"",
				t30_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE; 
		(void) close(fd);
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	(void) tbrename(t30_file, "", SYSERROR, ENOENT);
	PATH_FUNC_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(3); 

	(void) unlink(t30_file);

	DBUG_VOID_RETURN;
}

private void
test31()
{
	/* component of either path prefix is not a directory */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test31");

	testfail = 0;

	if ((fd = crfile(t31_ndir, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t31_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if ((fd = crfile(t31b_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t31b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0; 
	if (tbrename(t31a_file, t31b_file, SYSERROR, ENOTDIR) != SYSERROR)
	{
		if ((fd = crfile(t31b_file, MODEANY,
				(int) geteuid(), (int) getegid())) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create file \"%s\"",
				t31b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE; 
		(void) close(fd);
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	(void) tbrename(t31b_file, t31a_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	(void) unlink(t31b_file);
	(void) unlink(t31_ndir);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t31_ndir, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t31_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (mkfifo(t31b_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t31b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (tbrename(t31a_file, t31b_file, SYSERROR, ENOTDIR) != SYSERROR)
	{
		if (mkfifo(t31b_file, MODEANY) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create fifo \"%s\"",
				t31b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE; 
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	(void) tbrename(t31b_file, t31a_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	(void) unlink(t31b_file);
	(void) unlink(t31_ndir);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(8); 

	DBUG_VOID_RETURN;
}

private void
test32()
{
	/* "old" is a directory, "new" is not */

	int fd;
	int	pathok = 0;

	DBUG_ENTER("test32");

	testfail = 0;

	if (crdir(t32_dir, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t32_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t32_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t32_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbrename(t32_dir, t32_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	(void) unlink(t32_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t32_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t32_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrename(t32_dir, t32_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	(void) unlink(t32_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(5); 

	(void) rmdir(t32_dir);

	DBUG_VOID_RETURN;
}

private void
test33()
{
	/* rename on read-only file system */

	char	*rofs;
	int	pathok = 0;
	int 	retval;

	DBUG_ENTER("test33");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL ||
		*rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_ROFS not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	retval = cppair(ch_t33, NULLFN, 4 * WAITTIME, NOK|PATH_NOK);
	if (retval == PATH_NOK)
		;
	else
		PATH_TRACE;	/* Child ok , check cppair */
		
	if (retval != NOK)
		PATH_XS_RPT(2); 
	
	DBUG_VOID_RETURN;
}

private void
ch_t33()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t33");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test34()
{
	/* rename across file systems */

#if	  !TEST_XNFS
	char	*supp;
#endif	/* TEST_XNFS */
	int	fd;
	int	pathok = 0;
	int	retval;

	DBUG_ENTER("test34");


	testfail = 0;

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if ((supp = tet_getvar("VSX_LINK_FILESYS_SUPP")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_LINK_FILESYS_SUPP not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*supp == 'Y' || *supp == 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_LINK_FILESYS_SUPP is set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#endif	/* TEST_XNFS */

	if ((fd = crfile(t34a_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t34a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	retval = cppair(ch_t34, NULLFN, 4 * WAITTIME, NOK|PATH_NOK);
	if (retval == NOK)
		testfail++;
	else if (retval == PATH_NOK)
		retval = OK;
	else
		PATH_TRACE; 	/* Child ok , check cppair */

	(void) unlink(t34a_file);

	if (testfail == 0)
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
ch_t34()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t34");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t34a_file;
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test35()
{
	/*  LINK_MAX (+1) links to parent dir */

#if defined(_XOPEN_SOURCE) && !(TEST_XPG > 3)
	DBUG_ENTER("test35");
	xx_rpt(NOTINUSE);
	in_rpt("no test in XPG3 mode");
	DBUG_VOID_RETURN;
#else
	char	*links[PCTS_LINK_MAX], *nambuf;
	long	i, link_max, testmax = PCTS_LINK_MAX;
	int	err;
	struct stat stbuf;
	int	pathok = 0;
	int	loop = 0;
#if	   TEST_XNFS
	int	nvals;
	char	**strvals;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test35");

	testfail = 0;

	if (mkdir(t35_dir, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("mkdir(\"%s\", %#o) failed - errno %d (%s)",
			t35_dir, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#if	   TEST_XNFS

	if (tet_sync(SP_T35_CRDIR_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to CRDIR_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Get the value of LINK_MAX from the server */
	nvals = 0;
	strvals = xnfs_getsv_c(&nvals, SP_T35_PLM_TRANSFER);
	if (strvals == NULL || nvals != 1)
	{
		xx_rpt(DELETION);
		in_rpt("xnfs_getsv_c() failed when getting _PC_LINK_MAX from server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	link_max = atol(strvals[0]);

	if(link_max <= 0)
	{
		if (link_max == 0)
		{
			link_max = PCTS_LINK_MAX+1;
			PATH_TRACE;
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("pathconf(\"%s\", _PC_LINK_MAX) failed on server", t35_dir);
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;

	errno = 0;
	link_max = pathconf(t35_dir, _PC_LINK_MAX);
	if (link_max < 0)
	{
		if (errno == 0)
		{
			link_max = PCTS_LINK_MAX+1;
			PATH_TRACE; 
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("pathconf(\"%s\", _PC_LINK_MAX) failed",
				t35_dir);
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

#endif	/* TEST_XNFS */

	if (link_max < PCTS_LINK_MAX)
		testmax = link_max;

	if (mkdir(t35a_file, MODEANY) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("mkdir(\"%s\", %#o) failed - errno %d (%s)",
			t35a_file, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(t35_dir, &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t35_dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	nambuf = (char *) malloc((size_t)(testmax * NAMELEN));
	if (nambuf == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: malloc() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create lesser of LINK_MAX and PCTS_LINK_MAX links with mkdir() */

	for (i = 0; i < testmax - stbuf.st_nlink; i++)
	{
		links[i] = &nambuf[i*NAMELEN];
		(void) sprintf(links[i], "%s/L%ld", t35_dir, i);
		if (mkdir(links[i], MODEANY) == SYSERROR)
		{
			err = errno;
			testfail++;
			xx_rpt(DELETION);
			in_rpt("mkdir(\"%s\", %#o) failed - errno %d (%s)",
				links[i], (int)MODEANY, err, errname(err));
			break;
		}
		else
		{
			PATH_TRACE;
			loop++;
		}
	}
	links[i] = NULL;


#if	   TEST_XNFS

	/* The server side has this sync point to prevent it timing out */

	if (tet_sync(SP_T35_LOOP_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to LOOP_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;

#endif	/* TEST_XNFS */

#ifdef TESTBED2
	(void) rmdir(links[0]);
#endif

	/* if LINK_MAX is testable, next link gives EMLINK */

	if (i == testmax - stbuf.st_nlink)
	{
	    if (link_max <= PCTS_LINK_MAX)
	    {
		if (stat(t35_dir, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("stat(\"%s\") failed - errno %d (%s)",
				t35_dir, err, errname(err));
		}
		else if (stbuf.st_nlink != testmax)
		{
			/* EMLINK test does not apply */
			DBUG_PRINTF("test35", "nlink = %d, testmax = %ld",
				(int)stbuf.st_nlink, testmax);
			globok = 0;
			(void) tbrename(t35a_file, t35b_file, SUCCEED, NOERROR);
			PATH_FUNC_TRACE;
		}
		else
		{
			globok = 0;
			(void) tbrename(t35a_file, t35b_file, SYSERROR, EMLINK);
			PATH_FUNC_TRACE;
		}
	    }
	    else
	    {
		globok = 0;
		(void) tbrename(t35a_file, t35b_file, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	    }
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(9 + loop);
		
	for (i = 0; i < testmax-1 && links[i] != NULL; i++)
		(void) rmdir(links[i]);
	(void) rmdir(t35b_file);
	(void) rmdir(t35_dir);
	(void) rmdir(t35a_file);
	free((void *)nambuf);

	DBUG_VOID_RETURN;

#endif /* !XPG3 mode */
}


#if TEST_XNFS

private void
test36()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test36");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
	/* Note: can't create file because name is too long */

	globok = 0;
	(void) tbrename(longname, t36_file, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	(void) unlink(t36_file);

	if ((fd = crfile(t36_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t36_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tbrename(t36_file, longname, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	(void) unlink(t36_file);
	(void) unlink(longname);

	if (testfail == 0)
		PATH_XS_RPT(4);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test36()
{
	DBUG_ENTER("test36");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test37()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test37");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Note: can't create file because name is too long */

	globok = 0;
	(void) tbrename(longname, t37_file, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	(void) unlink(t37_file);

	if ((fd = crfile(t37_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t37_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tbrename(t37_file, longname, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	(void) unlink(t37_file);
	(void) unlink(longname);

	if (testfail == 0)
		PATH_XS_RPT(4);

	clntoolng();


	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test37()
{
	DBUG_ENTER("test37");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test38()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test38");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = creat(longname, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			longname);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tbrename(longname, t38_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t38_file);
	(void) unlink(longname);

	if ((fd = creat(t38_file, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t38_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tbrename(t38_file, longname, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t38_file);
	(void) unlink(longname);

	if (testfail == 0)
		PATH_XS_RPT(5);

	clntoolng();


	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test38()
{
	DBUG_ENTER("test38");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test39()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test39");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = creat(longname, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			longname);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tbrename(longname, t39_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t39_file);
	(void) unlink(longname);

	if ((fd = creat(t39_file, MODEANY)) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file \"%s\"",
			t39_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = 0;
	(void) tbrename(t39_file, longname, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) unlink(t39_file);
	(void) unlink(longname);

	if (testfail == 0)
		PATH_XS_RPT(5);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test39()
{
	DBUG_ENTER("test39");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */
