/*
*      SCCS:  @(#)  POSIX.os/files/rename/rename_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/rename - rename_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		rename_su - used by T.rename in privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.rename to perform tests requiring appropriate
	  privileges.

	  usage:
		rename_su testnum file1 file2 ident dbug

	  where "testnum" is the test number being executed and "file1"
	  and "file2" are file names used by the subtest.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	8 June 1989
MODIFICATIONS:
		Geoff Clare, 15 Nov 1989
			New setuprofs() argument
			New do_access() argument

		Steve Ratcliffe, 20 Apr 1990
			Renumber.

		Anthony Lee, 01 August 1990
			Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file
			PATH_NOK changed to eight from six.

		Neil Moses, 9 June 1992
			Check that time stamps are not changed.

                John-Paul, 29 July 1992
                        Mode parameter in calls to crfile() changed from 
			'int' to 'mode_t' type.

		David Sawyer, 11 November 1992
			Modifications for XNFS tests

		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.

		Geoff Clare, 3 Nov 1993
			Use creat() instead of crfile() in t16_func().

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <dbug.h>
#include <sys/stat.h>
#include <report.h>
#include <tet_api.h>
#include <xnfs.h>
#include "rename_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef rename
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	NOK	02
#define PATH_NOK 8
public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
public	int	testfail;

extern	char	**environ;
extern	void	savtimes();
extern	char	*chktimes();

extern	void	set_pname();
extern	void	cln_access();

private char	*t14_dir =	"rename-d.14";
private char	*t14_file =	"rename-d.14/rename-t.14";
private char	*t15_file =	"rename-t.15";
private char	*t16_file =	"rename-t.16";
#if	   TEST_XNFS
private char	*t18_file2a =	T18_FILE2A;
private char	*t18_file2b =	T18_FILE2B;
private char	*t18_file2c =	T18_FILE2C;
private char	*t18_file2d =	T18_FILE2D;
private char	*t19_file2a =	T19_FILE2A;
private char	*t19_file2b =	T19_FILE2B;
private char	*t19_file2c =	T19_FILE2C;
private char	*t19_file2d =	T19_FILE2D;
private char	*t20_dir2a =	T20_DIR2A;
private char	*t20_dir2b =	T20_DIR2B;
private char	*t20_dir2c =	T20_DIR2C;
private char	*t20_dir2d =	T20_DIR2D;
#endif	/* TEST_XNFS */
private char	*t33_dir =	"rename-d.33";
private char	*t33a_file =	"rename-d.33/rename-t.33a";
private char	*t33b_file =	"rename-d.33/rename-t.33b";
private char	*t34_dir =	"rename-d.34";
private char	*t34_file =	"rename-d.34/rename-t.34";

private	int	subtest12();
private	int	subtest13();
private	int	subtest14();
private	int	subtest15();
private	int	subtest16();
#if	   TEST_XNFS
private	int	subtest18_20();
#endif	/* TEST_XNFS */
private	int	subtest21();
private	int	subtest22();
private	int	subtest33();
private	int	subtest34();
private	int	t15_func();
private	int	t16_func();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int testnum;
	int exitval = NOK;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		return(exitval);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 5)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testnum = atoi(argv[1]);
	switch (testnum)
	{
	case 12:
		exitval = subtest12(argv[2], argv[3]);
		break;

	case 13:
		exitval = subtest13(argv[2], argv[3]);
		break;

	case 14:
		exitval = subtest14(argv[2]);
		break;

	case 15:
		exitval = subtest15();
		break;

	case 16:
		exitval = subtest16();
		break;

#if	   TEST_XNFS
	case 18:
		exitval = subtest18_20(t18_file2a, t18_file2b, t18_file2c, 
					t18_file2d);
		break;

	case 19:
		exitval = subtest18_20(t19_file2a, t19_file2b, t19_file2c, 
					t19_file2d);
		break;

	case 20:
		exitval = subtest18_20(t20_dir2a, t20_dir2b, t20_dir2c, 
					t20_dir2d);
		break;
#endif	/* TEST_XNFS */

	case 21:
		exitval = subtest21(argv[2], argv[3]);
		break;

	case 22:
		exitval = subtest22(argv[2], argv[3]);
		break;

	case 33:
		exitval = subtest33();
		break;

	case 34:
		exitval = subtest34(argv[2]);
		break;

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed: %d", testnum);
		break;
	}

	return(exitval);
}


private int
subtest12(old, new)
char *old, *new;
{
	/* No EBUSY when "old" is mount point */
	int	pathok = 0;

	DBUG_ENTER("subtest12");

	testfail = 0;

	globok = 0; 
	if (mountfs(old, MODEANY) != 0)
	{
		DBUG_RETURN(NOK);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbrename(old, new, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) umntfs(old);

	if (testfail != 0)
	{
		DBUG_RETURN(NOK);
	}
	else if (pathok != 2)
		DBUG_RETURN(PATH_NOK);	/* OK, but path trace wrong */
		

	DBUG_RETURN(OK);
}

private int
subtest13(old, new)
char *old, *new;
{
	/* No EBUSY when "new" exists as mount point */
	int	pathok = 0;

	DBUG_ENTER("subtest13");

	testfail = 0;

	globok = 0; 
	if (mountfs(new, MODEANY) != 0)
	{
		DBUG_RETURN(NOK);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbrename(old, new, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	(void) umntfs(new);

	if (testfail != 0)
	{
		DBUG_RETURN(NOK);
	}
	else if (pathok != 2)
		DBUG_RETURN(PATH_NOK);	/* OK, but path trace wrong */

	DBUG_RETURN(OK);
}

private int
subtest14(old)
char *old;
{
	/* rename across file systems */
	int	pathok = 0;

	DBUG_ENTER("subtest14");

	testfail = 0;

	globok = 0; 
	if (mountfs(t14_dir, MODEANY) != 0)
	{
		DBUG_RETURN(OK);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbrename(old, t14_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	
	(void) unlink(t14_file);
	(void) umntfs(t14_dir);

	if (testfail != 0)
	{
		DBUG_RETURN(NOK);
	}
	else if (pathok != 2)
		DBUG_RETURN(PATH_NOK);	/* OK, but path trace wrong */


	DBUG_RETURN(OK);
}

private int
subtest15()
{
	/* search & write permission granted on parent of path1 */

	int	pathok = 0;
	DBUG_ENTER("subtest15");

	if (do_access('f', (mode_t) 0, (mode_t) S_IWUSR|S_IXUSR, t15_func, WAITTIME, SP_T15_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE; 
		PATH_XS_RPT(1); 
	}
	
	cln_access();

	DBUG_RETURN(OK);
}

private int
t15_func(file, desc)
char *file, **desc;
{
	int	pathok = 0;
	DBUG_ENTER("t15_func");

	testfail = 0;

	globok = 0;
	(void) tbrename(file, t15_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	
	if (testfail != 0)
	{
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
	}
	PATH_FUNC_RPT(1); 		/* Needed for do_access */
	(void) unlink(t15_file);
	DBUG_RETURN(testfail);
}

private int
subtest16()
{
	/* search & write permission granted on parent of path2 */

	int	pathok = 0;
	DBUG_ENTER("subtest16");
	
	if (do_access('d', (mode_t) S_IWUSR|S_IXUSR, (mode_t) S_IXUSR, t16_func, WAITTIME, SP_T16_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE; 
		PATH_XS_RPT(1); 
	}
	
	cln_access();

	DBUG_RETURN(OK);
}

private int
t16_func(dir, desc)
char *dir, **desc;
{
	int  fd;
	char file[50];
	int	pathok = 0;

	DBUG_ENTER("t16_func");

	if ((fd = creat(t16_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t16_file);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	(void) strcat(strcpy(file, dir), "/rename-t.16");

	testfail = 0;

	globok = 0;
	(void) tbrename(t16_file, file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	
	if (testfail != 0)
	{
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
	}

	PATH_FUNC_RPT(2); 		/* Needed for do_access */	
	(void) unlink(file);
	(void) unlink(t16_file);
	DBUG_RETURN(testfail);
}


#if	   TEST_XNFS
private int
subtest18_20(file2a, file2b, file2c, file2d)
char *file2a;
char *file2b;
char *file2c;
char *file2d;
{
	int	pathok = 0;

	DBUG_ENTER("subtest18_20");

	testfail = 0;

	globok = 0;
	(void) tbrename(file2a, file2b, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbrename(file2c, file2d, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (testfail != 0)
	{
		DBUG_RETURN(NOK);
	}
	else if (pathok != 2)
		DBUG_RETURN(PATH_NOK);	/* OK, but path trace wrong */

	DBUG_RETURN(OK);
}
#endif	/* TEST_XNFS */


private int
subtest21(old, new)
char *old, *new;
{
	/* EBUSY when "old" is mount point */

	int	pathok = 0;
	DBUG_ENTER("subtest21");

	testfail = 0;

	globok = 0; 
	if (mountfs(old, MODEANY) != 0)
	{
		DBUG_RETURN(NOK);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbrename(old, new, SYSERROR, EBUSY);
	PATH_FUNC_TRACE;

	(void) umntfs(old);

	if (testfail != 0)
		DBUG_RETURN(NOK);
	else if (pathok != 2)
		DBUG_RETURN(PATH_NOK);

	DBUG_RETURN(OK);
}

private int
subtest22(old, new)
char *old, *new;
{
	/* EBUSY when "new" exists as mount point */
	int	pathok = 0;

	DBUG_ENTER("subtest22");

	testfail = 0;

	globok = 0; 
	if (mountfs(new, MODEANY) != 0)
	{
		DBUG_RETURN(NOK);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbrename(old, new, SYSERROR, EBUSY);
	PATH_FUNC_TRACE;

	(void) umntfs(new);

	if (testfail != 0)
		DBUG_RETURN(NOK);
	else if (pathok != 2)
		DBUG_RETURN(PATH_NOK);

	DBUG_RETURN(OK);
}

private int
subtest33()
{
	/* rename on read only file system */
	char	*errptr;
	int	pathok = 0;

	DBUG_ENTER("subtest33");

	testfail = 0;

	globok = 0; 
	if (setuprofs(t33_dir, t33a_file, 'f', MODEANY) != 0)
	{
		DBUG_RETURN(OK);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(t33_dir);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbrename(t33a_file, t33b_file, SYSERROR, EROFS);
	PATH_FUNC_TRACE;
	
	globok = 0;
	errptr = chktimes(t33_dir, 0);
	PATH_FUNC_TRACE;

	if(errptr != NULL)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("Unsuccessful rename() changed the time stamps on %s",
			t33_dir);
		in_rpt("%s", errptr);
	}
	else
		PATH_TRACE;

	(void) clnuprofs(t33_dir, t33a_file);

	if (testfail != 0)
		DBUG_RETURN(NOK);
	else if (pathok != 5)
		DBUG_RETURN(PATH_NOK);
		
	DBUG_RETURN(OK);
}

private int
subtest34(old)
char *old;
{
	/* rename across file systems */
	int	pathok = 0;

	int fd;

	DBUG_ENTER("subtest34");

	testfail = 0;

	globok = 0; 
	if (mountfs(t34_dir, MODEANY) != 0)
	{
		DBUG_RETURN(OK);
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = creat(t34_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t34_file);
		DBUG_RETURN(OK);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbrename(old, t34_file, SYSERROR, EXDEV);
	PATH_FUNC_TRACE;
	
	(void) unlink(t34_file);
	(void) umntfs(t34_dir);

	if (testfail != 0)
		DBUG_RETURN(NOK);
	else if (pathok != 3)
		DBUG_RETURN(PATH_NOK);

	DBUG_RETURN(OK);
}
