/*
*      SCCS:  @(#)  POSIX.os/files/rename/tbrename.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		tbrename.c

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
		tbrename() checking function

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	3 Apr 1989
MODIFICATIONS:	
		Geoff Clare, 5 Apr 1990
			New interface with pathtoolng() and nametoolng().

		Geoff Clare, 20 June 1990
			Give WARNING only in ETXTBSY test if call succeeds

		Anthony Lee, 01 August 1990
			Addded resilience code.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<string.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef rename
#endif


extern	char	*errname();
extern	void	valrpt();

extern int	testfail;

public int
tbrename(tbpath1, tbpath2, expret, experrno)
char *tbpath1, *tbpath2;
int  expret, experrno;
{
	char	*reppath1 = tbpath1;
	char	*reppath2 = tbpath2;
	int	rval, obserr, err, statok, statok2;
	dev_t	olddev, olddev2;
	ino_t	oldino, oldino2;
	char	pathbuf1[30], pathbuf2[30];
	struct stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("tbrename");

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath1) > 100)
	{
		(void) sprintf(pathbuf1, "<%d char path>", (int)strlen(tbpath1));
		reppath1 = pathbuf1;
	}
	if (strlen(tbpath2) > 100)
	{
		(void) sprintf(pathbuf2, "<%d char path>", (int)strlen(tbpath2));
		reppath2 = pathbuf2;
	}

	if (stat(tbpath1, &stbuf) == SYSERROR)
	{
		if (expret != SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				reppath1, err, errname(err));
		}
		else
			PATH_TRACE;
		statok = FALSE;
	}
	else
	{
		PATH_TRACE; 
		statok = TRUE;
		olddev = stbuf.st_dev;
		oldino = stbuf.st_ino;
	}

	if (expret == SYSERROR)
	{
		if (stat(tbpath2, &stbuf) == SYSERROR)
			statok2 = FALSE;
		else
		{
			statok2 = TRUE;
			olddev2 = stbuf.st_dev;
			oldino2 = stbuf.st_ino;
		}
	}

	errno = 0;
	rval = rename(tbpath1, tbpath2);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	obserr = errno;

	if (expret != SYSERROR && rval == expret)
	{
		/* "new" now points to same file as "old" did
		   (specifically for assertion 7, but also applicable
		   to all successful calls) */

		if (stat(tbpath2, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("stat() failed on new path \"%s\" - errno %d (%s)",
				reppath2, err, errname(err));
		}
		else if (statok &&
			(stbuf.st_dev != olddev || stbuf.st_ino != oldino))
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("new path does not refer to same file as old path did");
		}
		else
			PATH_TRACE;

		/* "old" path no longer exists */

		if (eaccess(tbpath1, F_OK) != SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("old path still exists after successful call to rename()");
			(void) unlink(tbpath1);
		}
		else
			PATH_TRACE;
			
		PATH_FUNC_RPT(3); 
		DBUG_RETURN(rval);
	}
	else if (expret == SYSERROR && rval == expret)
	{
		/* check files were not changed */

		if (statok && stat(tbpath1, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("after unsuccessful call to rename(), stat() failed on old path \"%s\" - errno %d (%s)",
				reppath1, err, errname(err));
		}
		else if (statok &&
			(stbuf.st_dev != olddev || stbuf.st_ino != oldino))
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to rename() changed file pointed to by old path \"%s\"",
				reppath1);
		}
		else
			PATH_TRACE;

		if (statok2 && stat(tbpath2, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("after unsuccessful call to rename(), stat() failed on new path \"%s\" - errno %d (%s)",
				reppath2, err, errname(err));
		}
		else if (statok2 &&
			(stbuf.st_dev != olddev2 || stbuf.st_ino != oldino2))
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to rename() changed file pointed to by new path \"%s\"",
				reppath2);
		}
		else
			PATH_TRACE;
			
		if (experrno == obserr)
		{
			PATH_FUNC_RPT(3); 
			DBUG_RETURN(rval);
		}

		/* Existing non-empty directory can also give ENOTEMPTY */
		if (experrno == EEXIST && obserr == ENOTEMPTY)
		{
			PATH_FUNC_RPT(3); 
			DBUG_RETURN(rval);
		}
	}

	testfail++;
#ifdef _XOPEN_SOURCE
	if (experrno == ETXTBSY && rval != SYSERROR)
		xx_rpt(WARNING);
	else
		xx_rpt(FAILURE);
#else
	xx_rpt(FAILURE);
#endif
	in_rpt("rename(\"%s\", \"%s\") did not give correct results",
		reppath1, reppath2);
	valrpt(expret, rval, experrno, obserr);
	
	if (expret == SYSERROR && rval != expret)
		(void) rename(tbpath2, tbpath1);

	DBUG_RETURN(rval);
}
