/*
*      SCCS:  @(#)  POSIX.os/files/rewinddir/rewinddir.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/rewinddir - T.rewinddir Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.rewinddir

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the rewinddir() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	18 Apr 1989
MODIFICATIONS:	
		Geoff Clare, 21 Nov 1989
			Remove all references to d_ino

		Peter Lee, 30 July 1990
			Added resilience code.

                John-Paul, 29 July 1992
                        Mode parameter in calls to crfile() and crdir()
                        changed from 'int' to 'mode_t' type.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<string.h>
#include	<dirent.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include 	<tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef rewinddir
#endif


#define NO_TESTS	2
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char	*errname();
extern	char	*newstr();

public	int	no_tests = NO_TESTS;

private int	testfail;
private char	*t1_dir =	"rewinddir-d.1";
private char	*t1a_file =	"rewinddir-d.1/rewinddir-t.1a";
private char	*t1b_file =	"rewinddir-d.1/rewinddir-t.1b";
private char	*t2_dir =	"rewinddir-d.2";
private char	*t2a_file =	"rewinddir-d.2/rewinddir-t.2a";
private char	*t2a_name =	"rewinddir-t.2a";
private char	*t2b_file =	"rewinddir-d.2/rewinddir-t.2b";
private char	*t2b_name =	"rewinddir-t.2b";

private void	cln_tests();
private void	test1();
private void	test2();


public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) unlink(t1a_file);
	(void) unlink(t1b_file);
	(void) rmdir(t1_dir);
	(void) unlink(t2a_file);
	(void) unlink(t2b_file);
	(void) rmdir(t2_dir);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* rewinddir() positions stream at first directory entry */

	DIR		*dirp;
	struct dirent	*dp;
	char		*firstname;
	int		fd, err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	if (crdir(t1_dir, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t1_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create two directory entries - in case there are no "." & ".." */
	if ((fd = crfile(t1a_file, MODEANY,
		(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t1a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	if ((fd = crfile(t1b_file, MODEANY,
		(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t1b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	dirp = opendir(t1_dir);
	if (dirp == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("opendir(\"%s\") failed - errno %d (%s)",
			t1_dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	dp = readdir(dirp);
	if (dp == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("readdir() failed - errno %d (%s)", err, errname(err));
		(void) closedir(dirp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	firstname = newstr(dp->d_name);
	if (firstname == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc() failed");
		(void) closedir(dirp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rewinddir(dirp);

	errno = NOERROR;
	dp = readdir(dirp);
	if (dp == NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("readdir() failed after rewinddir() - errno %d (%s)",
			err, errname(err));
		(void) closedir(dirp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED
	firstname = "junk";
#endif

	if (strcmp(dp->d_name, firstname) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("rewinddir() did not position stream at first directory entry");
		in_rpt("1st entry after opendir(): \"%s\"", firstname);
		in_rpt("1st entry after rewinddir(): \"%s\"", dp->d_name);
	}
	else
		PATH_XS_RPT(7);

	(void) closedir(dirp);
	(void) unlink(t1a_file);
	(void) unlink(t1b_file);
	(void) rmdir(t1_dir);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* rewinddir() causes stream to reflect current status of directory */

	DIR		*dirp;
	struct dirent	*dp;
	int		fd, err, found1, found2;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	if (crdir(t2_dir, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t2_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create two directory entries - in case there are no "." & ".." */
	if ((fd = crfile(t2a_file, MODEANY,
		(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t2a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if ((fd = crfile(t2b_file, MODEANY,
		(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t2b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	dirp = opendir(t2_dir);
	if (dirp == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("opendir(\"%s\") failed - errno %d (%s)",
			t2_dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* read to the end */
	while (readdir(dirp) != NULL)
		;

#ifdef TESTBED
	(void) unlink(t2b_file);
#else
	if (unlink(t2a_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("unlink(\"%s\") failed - errno %d (%s)",
			t2a_file, err, errname(err));
		(void) closedir(dirp);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#endif

	rewinddir(dirp);

	/* check correct entry has gone away */
	found1 = found2 = 0;
	while ((dp = readdir(dirp)) != NULL)
	{
		if (strcmp(dp->d_name, t2a_name) == 0)
			found1++;
		if (strcmp(dp->d_name, t2b_name) == 0)
			found2++;
	}
	if (found1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("directory entry \"%s\" for unlinked file was found after rewinddir()",
			t2a_name);
	}
	else
		PATH_TRACE;

	if (!found2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("directory entry \"%s\" was not found after rewinddir()",
			t2b_name);
	}
	else
		PATH_TRACE;

#ifdef TESTBED
	if (testfail == 0)
		PATH_XS_RPT(6);
#else
	if (testfail == 0)
		PATH_XS_RPT(7);
#endif

	(void) closedir(dirp);
	(void) unlink(t2b_file);
	(void) rmdir(t2_dir);

	DBUG_VOID_RETURN;
}
