/*
*      SCCS:  @(#)  POSIX.os/files/rmdir/rmdir.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/rmdir - T.rmdir Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.rmdir

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the rmdir() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	10 Apr 1989
MODIFICATIONS:	
		Geoff Clare, 8 June 1989:  Add do_access() test.
		Geoff Clare, 22 Aug 1989:  VSX_REMOVE_DIR_EBUSY={Y,S,P,N}
		
		Geoff Clare, 13 Feb 1990
			In EBUSY tests, after chdir(), open "." to ensure
			the directory is considered "in use" by the process.

		Geoff Clare, 4 Apr 1990
			New interface with pathtoolng() and nametoolng().

		Steve Ratcliffe, 23 Apr 1990
			Add test to check for release of resorces after
			a rmdir()

		Anthony Lee, 30 July 1990
			Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Geoff Clare, 8 May 1992
			Add PRES_NODOTDOT to init_pathres() call.

		Geoff Clare, 12 June 1992
			Add PRES_NODOT to init_pathres() call.

                John-Paul, 29 July 1992
                        Mode parameter in calls to crfile() and crdir()
                        changed from 'int' to 'mode_t' type.

		David Sawyer, 13 November 1992
			Modifications for XNFS tests

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

		Geoff Clare, 3 Nov 1995
			Changed child timeout for do_access() test.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<string.h>
#include	<fcntl.h>
#include	<dirent.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<pathres.h>
#include	<chktimes.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"rmdir_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef rmdir
#endif


#define NO_TESTS	24
#define	WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define NOK		02
#define E_CHDIR		02
#define E_OPEN		04
#define PATH_NOK 	 8 

extern	char	*errname();
extern	char	*execpath();
extern	char	*itoa();
#if	   TEST_XNFS
extern	char	*xnfs_nametoolng();
#else	/* TEST_XNFS */
extern	char	**nametoolng();
#endif	/* TEST_XNFS */
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	void	valrpt();

extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

public	int	no_tests = NO_TESTS;

private	char	*prog_su =	"rmdir_su";
private	char	*prog9 =	"rmdir_t9";

private	char	*t1_file =	"rmdir-t.1";
private	char	*t3_file =	"rmdir-t.3";
private	char	*t4_dir =	"rmdir-d.4";
private	char	*t4_file =	"rmdir-d.4/rmdir-t.4";
private	char	*t5_file =	"rmdir-t.5";
private	char	*t6_file =	"rmdir-t.6";
private	char	*t7a_file =	"rmdir-t.7a";
private	char	*t7b_file =	"rmdir-t.7b";
private char	*t10_dir =	"rmdir-d.10";
private char	*t10_file =	"rmdir-d.10/rmdir-t.10";
#if	   TEST_XNFS
private char	*t10_file2 =	T10_FILE2;
#endif	/* TEST_XNFS */
private char	*t11_dir =	"rmdir-d.11";
private char	*t11_file =	"rmdir-d.11/rmdir-t.11";
#if	   TEST_XNFS
private char	*t11_file2 =	T11_FILE2;
#endif	/* TEST_XNFS */
private	char	*t12a_file =	"rmdir-t.12a";
private	char	*t12b_file =	"rmdir-t.12b";
private char	*t13_dir =	"rmdir-d.13";
private char	*t13_file =	"rmdir-d.13/rmdir-t.13";
private	char	*t15_file =	"rmdir-t.15";
private char	*t16_dir =	"rmdir-d.16";
private char	*t16_file =	"rmdir-d.16/rmdir-t.16";
private	char	*t17_file =	"rmdir-t.17";
private	char	*t18_dir =	"rmdir-d.18";
private char	*t19_ndir =	"rmdir-t.19";
private char	*t19_file =	"rmdir-t.19/rmdir-t.19";
private char	*t20_dir =	"rmdir-d.20";
private char	*t20_file =	"rmdir-d.20/rmdir-t.20";
private char	*t9_dir =	"rmdir-d.9";

private int	testfail;

private void	cln_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();

#if	   TEST_XNFS
private void	ch_t5();
private void	pa_t5();
#endif	/* TEST_XNFS */
private void	ch1_t7();
private void	ch2_t7();
private void	ch_t8();
#if	   TEST_XNFS
private void	ch_t10();
private void	ch_t11();
#endif	/* TEST_XNFS */
private void	ch1_t12();
private void	ch2_t12();
private void	ch_t18();
private void	ch_t20();
private void	ch_t9();

private void	pa_t7();
private void	pa_t12();

private	int	tbrmdir();


public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cln_tests;

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	NULL, 0
};


private void
cln_tests()
{
	char **p;

	DBUG_ENTER("cln_tests");

	(void) rmdir(t1_file);
	for (p = portfnam; *p != NULL; p++)
		(void) rmdir(*p);
	(void) rmdir(t3_file);
	(void) unlink(t4_file);
	(void) rmdir(t4_dir);
	(void) rmdir(t5_file);
	(void) rmdir(t6_file);
	(void) rmdir(t7a_file);
	(void) rmdir(t7b_file);
	(void) chmod(t10_dir, MODEANY);
	(void) rmdir(t10_file);
	(void) rmdir(t10_dir);
	(void) chmod(t11_dir, MODEANY);
	(void) rmdir(t11_file);
	(void) rmdir(t11_dir);
	(void) rmdir(t12a_file);
	(void) rmdir(t12b_file);
	(void) unlink(t13_file);
	(void) rmdir(t13_dir);
	(void) rmdir(t15_file);
	(void) rmdir(t18_dir);
	(void) unlink(t19_ndir);
	(void) rmdir(t20_dir);
	(void) rmdir(t9_dir);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* pathname resolution */

	char	*p1, *p2;
	int	npairs, i;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOT|PRES_NODOTDOT);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOT|PRES_NODOTDOT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Check that rmdir() resolves both names to the file
	   to which crdir() and eaccess() resolve the second name */

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);
		DBUG_PRINTF("test1", "npairs = %d",npairs);

		if (crdir(p2, MODEANY,
				(int)geteuid(), (int)getegid()) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create directory \"%s\"",
				p2);
			break;
		}
		else
			PATH_TRACE;

		globok = 0;
		if (tbrmdir(p2, SUCCEED, NOERROR) != SUCCEED)
		{
			PATH_FUNC_TRACE; PATH_TRACE; PATH_TRACE;  	/* Needed to be consistant */
			continue;
		}
		else
		{
			PATH_FUNC_TRACE;
		}
		
		/* eaccess(p2) is done in tbrmdir() */

#ifndef TESTBED2
		if (crdir(p2, MODEANY,
				(int)geteuid(), (int)getegid()) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create directory \"%s\"",
				p2);
			break;
		}
		else
			PATH_TRACE;
#else
		PATH_TRACE; 				/* Valid path if not TESTBED2 */
#endif

		globok = 0;
		(void) tbrmdir(p1, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;

		/* eaccess(p1) is done in tbrmdir(), but we want p2 */

		if (eaccess(p2, F_OK) == 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("rmdir() did not resolve \"%s\" and \"%s\" to the same file",
				p1, p2);
			(void) rmdir(p2);
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
	{
		PATH_XS_RPT(1 + ((npairs) *5));
	}
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames */

	char	**p;
	int	pathok = 0;
	int	loop1 , loop2;

	DBUG_ENTER("test2");

	testfail = 0;

	for (p = portfnam, loop1 = 0; *p != NULL; p++,loop1++)
	{
		if (crdir(*p, MODEANY,
				(int)geteuid(), (int)getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

#ifdef TESTBED2
	(void) rmdir(portfnam[0]);
#endif

	/* undistinguished names will give ENOENT */

	for (p = portfnam, loop2 = 0; *p != NULL; p++,loop2++)
	{
		globok = 0;
		(void) tbrmdir(*p, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(loop1+loop2); 

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* rmdir() removes an empty directory */
	int	pathok = 0;
	
	DBUG_ENTER("test3");

	testfail = 0;

	if (crdir(t3_file, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrmdir(t3_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(2); 
	
	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test4()
{
	DBUG_ENTER("test4");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test4()
{
	/* rmdir() on open directory: no "." & "..", can't create files */

	struct dirent	*dp;
	DIR		*dir;
	int		fd, err;
	int		pathok = 0;
	int 		loop = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	if (crdir(t4_dir, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t4_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((dir = opendir(t4_dir)) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("opendir(\"%s\") failed - errno %d (%s)",
			t4_dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		

#ifndef TESTBED2
	globok = 0; 
	if (tbrmdir(t4_dir, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) closedir(dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 
#else
		PATH_TRACE;		/* Valid path if not TESTBED2 */

#endif

	rewinddir(dir);

	while ((dp = readdir(dir)) != NULL)
	{
		loop++;			/* Expected path count */
		if (strcmp(dp->d_name, ".") == 0 ||
		    strcmp(dp->d_name, "..") == 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("open directory contained entry \"%s\" after rmdir(\"%s\")",
				dp->d_name, t4_dir);
		}
		else
		{
			PATH_TRACE;
		}
	}

	if ((fd = crfile(t4_file, MODEANY,
			(int)geteuid(), (int)getegid())) != SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("file \"%s\" was created successfully in open directory after rmdir(\"%s\")",
			t4_file, t4_dir);
		(void) close(fd);
		(void) unlink(t4_file);
	}
	else
		PATH_TRACE;
		
	if (testfail == 0)
	{
		PATH_XS_RPT(4 + loop); 
	}

	(void) closedir(dir);
	
	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

#if	  !TEST_XNFS

private void
test5()
{
	/* rmdir() on open directory: not removed until close */

	struct stat stbuf;
	int	fd, err;
	int	pathok = 0;
	
	DBUG_ENTER("test5");

	testfail = 0;

	if (crdir(t5_file, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fd = open(t5_file, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY) failed - errno %d (%s)",
			t5_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	(void) close(fd);
#endif

	globok = 0; 
	if (tbrmdir(t5_file, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	if (fstat(fd, &stbuf) == SYSERROR)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fstat() failed on fd open to directory after rmdir(\"%s\")",
			t5_file);
	}
	else
		PATH_TRACE;

	(void) close(fd);

	if (testfail == 0)
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

#else	  /* TEST_XNFS */

private void
test5()
{
	/* rmdir() on open directory: removed even though child has dir open */

	int	ret;
	int	pathok = 0;
	
	DBUG_ENTER("test5");

	testfail = 0;

	if (crdir(t5_file, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	globok = 0;
	ret = cppair(ch_t5, pa_t5, 2*WAITTIME, E_OPEN);
	if ((ret & OK) == 0)
		testfail++;
	else
		PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	fd, err;

	DBUG_ENTER("ch_t5");

	if ((fd = open(t5_file, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY) failed - errno %d (%s)",
			t5_file, err, errname(err));
		DBUG_EXIT(E_OPEN);
	}

	/* sleep with the directory open whilst parent calls rmdir() */
	(void) sleep(WAITTIME);

	(void) close(fd);

	DBUG_VOID_RETURN;
}

private void
pa_t5()
{
	int	fd, err;
	struct stat stbuf;
	int	pathok = 0;
	
	DBUG_ENTER("pa_t5");

	(void) sleep(WAITTIME/2);

	globok = 0; 
	if (tbrmdir(t5_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_FUNC_TRACE; 
	}

	fd = stat(t5_file, &stbuf);
	err = errno;
	if(fd != SYSERROR || err != ENOENT)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\") did not give correct results", t5_file);
		valrpt(SYSERROR, fd, ENOENT, err);
	}
	else
		PATH_TRACE;

	(void) close(fd);

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

#endif	  /* TEST_XNFS */

private void
test6()
{
	/* rmdir() updates timestamps on parent dir. */

	int	err;
	struct stat stbuf1, stbuf2;
	int	pathok = 0;
	
	DBUG_ENTER("test6");

	testfail = 0;

	if (crdir(t6_file, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stat(".", &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\".\") failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) sleep(1);
	
#ifndef TESTBED2
	globok = 0; 
	if (tbrmdir(t6_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
#else
	PATH_TRACE; 			/* Valid path if not TESTBED2 */
#endif

	if (stat(".", &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (stbuf2.st_ctime == stbuf1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was not updated");
	}
	else
		PATH_TRACE;
		
	if (stbuf2.st_mtime == stbuf1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was not updated");
	}
	else
		PATH_TRACE;
		
	if (testfail == 0)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* remove a busy directory */

	char	*supp;
	int	ret;
	int	pathok = 0;
	int 	retval ;
	
	DBUG_ENTER("test7");

	testfail = 0;

	if ((supp = tet_getvar("VSX_REMOVE_DIR_EBUSY")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
			
	if (*supp == 'Y' || *supp == 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY is set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (*supp != 'Y' && *supp != 'y' && *supp != 'S' && *supp != 's')
	{
		retval = cppair(ch1_t7, NULLFN, WAITTIME, NOK | PATH_NOK ); /* PATH_NOK is now an allowable exit value */
		if (retval == NOK)				/* PATH_NOK defined as 8 so as to no interfere with masks with 3 LS bits set */
		{
			testfail++;
		}
		else if (retval == PATH_NOK)			/* Child will return PATH_NOK if pathchecking failed */
		{
			retval = OK;
		}
		else
		{
			PATH_TRACE; 				/* Child pathchecked ok && cppair too */
		}
	}
	else
	{
		PATH_TRACE;				/* Valid not to go through above */
	}
		
	
	if (*supp != 'Y' && *supp != 'y' && *supp != 'P' && *supp != 'p')
	{
		if (crdir(t7b_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t7b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
			
		ret = cppair(ch2_t7, pa_t7, 2*WAITTIME, E_CHDIR|E_OPEN|PATH_NOK);
		if (ret == E_CHDIR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: chdir(\"%s\") failed in child",
				t7b_file);
		}
		else if (ret == E_OPEN)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: open(\".\", O_RDONLY) failed in child");
		}
		else if (ret == PATH_NOK)			/* Child will return PATH_NOK if pathchecking failed */
		{
			ret = OK;
		}
		else
		{
			PATH_FUNC_TRACE; 				/* Child pathchecked ok && cppair too */
		}
	}
	else
	{
		PATH_TRACE; PATH_TRACE;			/* Valid not to go through above */
	}

	if (testfail == 0)
	{
		PATH_XS_RPT(5);
	}

	DBUG_VOID_RETURN;
}

private void
ch1_t7()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch1_t7");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t7a_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

private void
pa_t7()
{

	int	pathok = 0;
	
	DBUG_ENTER("pa_t7");

	(void) sleep(WAITTIME/2);

	globok = 0;
	(void) tbrmdir(t7b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1); 			/* Need to report back cppair */
	
	DBUG_VOID_RETURN;
}

private void
ch2_t7()
{
	int	pathok = 0;

	DBUG_ENTER("ch2_t7");

#ifdef TESTBED2
	DBUG_EXIT(E_CHDIR);
#endif

	if (chdir(t7b_file) == SYSERROR)
	{
		DBUG_EXIT(E_CHDIR);
	}
	else
		PATH_TRACE;
			

	if (open(".", O_RDONLY) < 0)
	{
		DBUG_EXIT(E_OPEN);
	}
	else
		PATH_TRACE;
		
	(void) sleep(WAITTIME);

	if (pathok != 2)			/* All is well except pathchecking failed */
		DBUG_EXIT(PATH_NOK);		
		
	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* write access granted to parent, search access to path prefix */

	int waittime;

	DBUG_ENTER("test8");

	waittime = WAITTIME * (5 + sysconf(_SC_NGROUPS_MAX)/4);

	(void) cppair(ch_t8, NULLFN, waittime, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t8");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}


private void
test9()
{

	DBUG_ENTER("test9");

	(void) cppair(ch_t9, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int 	err;
	char	*args[6];

	DBUG_ENTER("ch_t9");

	args[0] = execpath(prog9);
	args[1] = t9_dir;
	args[2] = "rmdir";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog9), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}


private void
test10()
{
	/* search permission denied for component of path prefix */

	int	err;
	mode_t	mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
	int	pathok = 0;
#if	   TEST_XNFS
	int	ret;
#endif	/* TEST_XNFS */
	
	DBUG_ENTER("test10");

	testfail = 0;

	if (crdir(t10_dir, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t10_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (crdir(t10_file, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (chmod(t10_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t10_dir, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (tbrmdir(t10_file, SYSERROR, EACCES) != SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_FUNC_TRACE;
	}

	if (chmod(t10_dir, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: chmod(\"%s\", %#o) failed - errno %d (%s)",
			t10_dir, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
#ifdef TESTBED2
	(void) rmdir(t10_file);
#endif
	if (eaccess(t10_file, F_OK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to rmdir() removed directory");
	}
	else
		PATH_TRACE;
		
	(void) rmdir(t10_file);
	(void) rmdir(t10_dir);

#if	   TEST_XNFS

	if (tet_sync(SP_T10_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ret = tet_fork(ch_t10, NULLFN, WAITTIME, OK | NOK | PATH_NOK);

	if ((ret & OK) == 0)
	{
		testfail++;
	}
	else if ((ret & PATH_NOK) == 0)
	{
		PATH_TRACE; 
	}

	if (tet_sync(SP_T10_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
	{
		PATH_XS_RPT(9);
	}

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t10()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t10");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t10_file2;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

#endif	/* TEST_XNFS */


private void
test11()
{
	/* remove subdir of directory which denies write permission */

	int	err;
	int	pathok = 0;
	mode_t	mode = (S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
#if	   TEST_XNFS
	int	ret;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test11");

	testfail = 0;

	if (crdir(t11_dir, MODEANY,
			(int)geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t11_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (crdir(t11_file, MODEANY,
			(int)geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (chmod(t11_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t11_dir, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (tbrmdir(t11_file, SYSERROR, EACCES) != SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		

#ifdef TESTBED2
	(void) chmod(t11_dir, MODEANY);
	(void) rmdir(t11_file);
#endif
	if (eaccess(t11_file, F_OK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to rmdir() removed directory");
	}
	else
		PATH_TRACE;

	(void) chmod(t11_dir, MODEANY);
	(void) rmdir(t11_file);
	(void) rmdir(t11_dir);

#if	   TEST_XNFS

	if (tet_sync(SP_T11_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ret = tet_fork(ch_t11, NULLFN, WAITTIME, OK | NOK | PATH_NOK);

	if ((ret & OK) == 0)
	{
		testfail++;
	}
	else if ((ret & PATH_NOK) == 0)
	{
		PATH_TRACE; 
	}

	if (tet_sync(SP_T11_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if (testfail == 0)
	{
		PATH_XS_RPT(8);
	}

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t11()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t11");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t11_file2;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

#endif	/* TEST_XNFS */


private void
test12()
{
	/* remove a busy directory */

	char	*supp;
	int	ret;
	int	pathok = 0;
	int 	retval;
	
	DBUG_ENTER("test12");

	testfail = 0;

	if ((supp = tet_getvar("VSX_REMOVE_DIR_EBUSY")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (*supp == 'N' || *supp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (*supp == 'Y' || *supp == 'y' || *supp == 'S' || *supp == 's')
	{
		globok = 0; 
		retval = cppair(ch1_t12, NULLFN, WAITTIME, NOK|PATH_NOK);
		if (retval == NOK)
			testfail++;
		else if (retval == PATH_NOK)		/* Finished ok but pathtracing failed */
			ret = OK;
		else
			PATH_TRACE; 		/* Child was ok && check if cppair ok */
	}
	else
	{
		PATH_TRACE; 			/* A valid path */
	}

	if (*supp == 'Y' || *supp == 'y' || *supp == 'P' || *supp == 'p')
	{
		if (crdir(t12b_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: could not create directory \"%s\"",
				t12b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0; 
		ret = cppair(ch2_t12, pa_t12, 2*WAITTIME, E_CHDIR|E_OPEN|PATH_NOK);
		if (ret == E_CHDIR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: chdir(\"%s\") failed in child",
				t12b_file);
		}
		else if (ret == E_OPEN)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: open(\".\", O_RDONLY) failed in child");
		}
		else if (ret == PATH_NOK)			/* Child will return PATH_NOK if pathchecking failed */
		{
			ret = OK;
		}
		else
		{
			PATH_FUNC_TRACE;			/* Child pathchecked ok && cppair too */
		}
	}
	else
	{
		PATH_TRACE; PATH_TRACE; 			/* A Valid path */
	}
		

	if (testfail == 0)
	{
		PATH_XS_RPT(5); 
	}

	(void) rmdir(t12a_file);
	(void) rmdir(t12b_file);

	DBUG_VOID_RETURN;
}

private void
ch1_t12()
{
	int	err;
	char	*args[7];

	DBUG_ENTER("ch1_t12");

	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t12a_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

private void
pa_t12()
{
	int	pathok = 0;
	
	DBUG_ENTER("pa_t12");

	(void) sleep(WAITTIME/2);

	globok = 0; 
	if (tbrmdir(t12b_file, SYSERROR, EBUSY) != SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (eaccess(t12b_file, F_OK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to rmdir() removed directory");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 			/* Need to set globok for cppair */

	DBUG_VOID_RETURN;
}

private void
ch2_t12()
{
	int	pathok = 0;
	
	DBUG_ENTER("ch2_t12");

#ifdef TESTBED2
	DBUG_EXIT(E_CHDIR);
#endif

	if (chdir(t12b_file) == SYSERROR)
	{
		DBUG_EXIT(E_CHDIR);
	}
	else
		PATH_TRACE; 

	globok = 0; 
	if (open(".", O_RDONLY) < 0)
	{
		DBUG_EXIT(E_OPEN);
	}
	else
		PATH_TRACE; 
		
	
	(void) sleep(WAITTIME);

	if (pathok != 2)			/* PATH_TRACEing wrong */
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
test13()
{
	/* remove a non-empty directory */

	int fd;
	int	pathok = 0;
	
	DBUG_ENTER("test13");

	testfail = 0;

	if (crdir(t13_dir, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t13_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t13_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
		
	/* tbrmdir() also allows ENOTEMPTY when EEXIST expected */
	globok = 0;
	(void) tbrmdir(t13_dir, SYSERROR, EEXIST);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3); 

	(void) unlink(t13_file);
	(void) rmdir(t13_dir);

	DBUG_VOID_RETURN;
}

#if	   TEST_XNFS

private void
test14()
{
	DBUG_ENTER("test14");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test14()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	int	pathok = 0;
	
	DBUG_ENTER("test14");

	testfail = 0;

	globok = 0; 
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	if (crdir(longname[0], MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", longname[0]);
	}
	else
	{
		globok = 0; 
		(void) tbrmdir(longname[0], SUCCEED, NOERROR);
		PATH_FUNC_TRACE; 
	}

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbrmdir(longname[1], SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3); 

	(void) rmdir(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
test15()
{
	/* path longer than PATH_MAX */

	char	**longname;
	int	pathok = 0;

	DBUG_ENTER("test15");

	testfail = 0;

	if (crdir(t15_file, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t15_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	longname = pathtoolng(t15_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	(void) tbrmdir(longname[0], SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	
	if (crdir(t15_file, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t15_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if (tbrmdir(longname[1], SYSERROR, ENAMETOOLONG) == SYSERROR
			&&	longname[1] != NULL )
	{
#ifdef TESTBED2
		(void) rmdir(t15_file);
#endif
		if (eaccess(t15_file, F_OK) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to rmdir() removed directory");
		}
		else
			PATH_FUNC_TRACE; 
	}
	else if (longname[1] != NULL)
		PATH_FUNC_TRACE;
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5); 

	clntoolng();
	(void) rmdir(t15_file);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	/* component of path prefix does not exist */
	int	pathok = 0;
	
	DBUG_ENTER("test16");

	testfail = 0;

	if (eaccess(t16_dir, F_OK) == 0 && remove(t16_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t16_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrmdir(t16_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test17()
{
	/* directory does not exist */
	int	pathok = 0;

	DBUG_ENTER("test17");

	testfail = 0;

	if (eaccess(t17_file, F_OK) == 0 && remove(t17_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t17_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrmdir(t17_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test18()
{
	/* path is "" */
	int	pathok = 0;
	
	DBUG_ENTER("test18");

	testfail = 0;

	if (crdir(t18_dir, MODEANY,
			(int) geteuid(), (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t18_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);	/* Pass path checking to child */

	(void) cppair(ch_t18, NULLFN, WAITTIME, 0);

	(void) rmdir(t18_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t18()
{
	int err;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t18");

	PATH_FUNC_TRACE;	/* check globok set in parent */

	/* Do it in a subdirectory, just in case */

	if (chdir(t18_dir) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chdir(\"%s\") failed - errno %d (%s)",
			t18_dir, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrmdir("", SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{
		PATH_XS_RPT(3); 
	}

	DBUG_VOID_RETURN;
}

private void
test19()
{
	/* component of path is not a directory */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test19");

	testfail = 0;

	if ((fd = crfile(t19_ndir, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t19_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	(void) close(fd);

	globok = 0;
	(void) tbrmdir(t19_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2); 

	(void) unlink(t19_ndir);

	DBUG_VOID_RETURN;
}

private void
test20()
{
	/* rmdir on read-only file system */

	DBUG_ENTER("test20");

	(void) cppair(ch_t20, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t20_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t20()
{
	char	*rofs;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t20");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_ROFS is not defined");
		DBUG_VOID_RETURN;
	}
		
 	/* can't use execle: null vg_dbug stops environ being passed */
	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t20_dir;
	args[3] = t20_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}



#if TEST_XNFS

private void
test21()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test21");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbrmdir(longname, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test21()
{
	DBUG_ENTER("test21");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test22()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test22");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbrmdir(longname, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test22()
{
	DBUG_ENTER("test22");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test23()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test23");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (crdir(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid()) == SYSERROR)
	{
		testfail++;
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname);
	}
	else
	{
		globok = 0;
		(void) tbrmdir(longname, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) rmdir(longname);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test23()
{
	DBUG_ENTER("test23");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test24()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test24");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (crdir(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid()) == SYSERROR)
	{
		testfail++;
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname);
	}
	else
	{
		globok = 0;
		(void) tbrmdir(longname, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) rmdir(longname);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test24()
{
	DBUG_ENTER("test24");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private int
tbrmdir(tbpath, expret, experrno)
char *tbpath;
int  expret, experrno;
{
	char	*reppath = tbpath;
	int	rval, obserr;
	char	pathbuf[30];
	int	pathok = 0;
	
	DBUG_ENTER("tbrmdir");
	
	errno = 0;
	rval = rmdir(tbpath);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	obserr = errno;

	if (expret != SYSERROR && rval == expret)
	{
		if (eaccess(tbpath, F_OK) == 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("successful call to rmdir() did not remove directory");
		}
		else
		{
			PATH_TRACE;
			PATH_FUNC_RPT(1);
		}
		DBUG_RETURN(rval);
	}
	else if (expret == SYSERROR && rval == expret)
	{
		if (experrno == obserr)
		{
			PATH_TRACE;
			PATH_FUNC_RPT(1);
			DBUG_RETURN(rval);
		}
		if (experrno == EEXIST && obserr == ENOTEMPTY)
		{
			PATH_TRACE;
			PATH_FUNC_RPT(1);
			DBUG_RETURN(rval);
		}

	}

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("rmdir(\"%s\") did not give correct results", reppath);
	valrpt(expret, rval, experrno, obserr);
	
	DBUG_RETURN(rval);
}
