/*
*      SCCS:  @(#)  POSIX.os/files/rmdir/rmdir_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/rmdir - rmdir_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		rmdir_su - used by T.rmdir for privileged tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.rmdir to test rmdir() error on a read-only file
	  system.  Must have appropriate privileges to do mount/umount of
	  file system.

	  usage:
		rmdir_su testnum dir file ident dbug

	  where testnum is the test number being executed, dir and file
	  are parameters to be passed to the test function.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	11 Apr 1989
MODIFICATIONS:
		Geoff Clare, 10 July 1989
			Add tests for EBUSY on mount point

		Geoff Clare, 15 Nov 1989
			New setuprofs() argument
			New do_access() argument
		
		Steve Ratcliffe, 23 Apr 1990
			Renumber.

		Anthony Lee, 31 July 1990
			Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file
			PATH_NOK value changed to 8

		Neil Moses, 8 June 1992
			Check that time stamps are not changed.

		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.

************************************************************************/
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <dbug.h>
#include <errno.h>
#include <sys/stat.h>
#include <report.h>
#include <tet_api.h>
#include <xnfs.h>
#include "rmdir_h.h"
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef rmdir
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define NOK		02
#define PATH_NOK	8

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
public int	testfail;

extern	char	**environ;
extern	void	savtimes();
extern	char	*chktimes();

extern	void	valrpt();
extern	void	cln_access();
extern	void	set_pname();

private	int	subtest7();
private	void	subtest8();
#if	   TEST_XNFS
private	int	subtest10_11();
#endif	/* TEST_XNFS */
private	int	subtest12();
private	void	subtest20();
private	int	t8_func();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
	int testnum;
	int exitval = OK;

	set_pname(argv[0]);

	if (argc < 4 || argc > 6)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		return(exitval);
	}
	if (argc >= 6)
		vg_dbug = argv[5];
	if (argc >= 4)
		vg_ident = argv[4];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testnum = atoi(argv[1]);
	switch(testnum)
	{
	case 7:
		exitval = subtest7(argv[3]);
		break;

	case 8:
		subtest8();
		break;

#if	   TEST_XNFS
	case 10:
	case 11:
		exitval = subtest10_11(argv[3]);
		break;
#endif	/* TEST_XNFS */

	case 12:
		exitval = subtest12(argv[3]);
		break;

	case 20:
		subtest20(argv[2], argv[3]);
		break;

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed: %d", testnum);
		break;
	}

	return(exitval);
}

private int
subtest7(file)
char *file;
{
	/* No EBUSY on mount point */

	int ret, err;
	int	pathok = 0;
	
	DBUG_ENTER("subtest7");

	globok = 0; 
	if (mountfs(file, (mode_t) MODEANY) != 0)
	{
		DBUG_RETURN(NOK);
	}
	else
		PATH_FUNC_TRACE;
		
	errno = 0;
	if ((ret = rmdir(file)) != SUCCEED)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("rmdir(\"%s\") did not give correct results", file);
		valrpt(SUCCEED, ret, NOERROR, err);
		(void) umntfs(file);
		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;
	(void) umntfs(file);

	if (pathok != 2)		/* The number of expected path traces */
		DBUG_RETURN(PATH_NOK);	/* PATH_NOK same value as in rmdir.c */

	DBUG_RETURN(OK);	/* This will make cppair give an exit(OK) */
}

private void
subtest8()
{
	/* write & search access granted to parent */
	int	pathok = 0;

	DBUG_ENTER("subtest8");

	globok = 0;
	if (do_access('d', (mode_t) S_IRWXU, (mode_t) (S_IWUSR|S_IXUSR), t8_func, WAITTIME, SP_T8_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(1);
	}
	
	cln_access();

	DBUG_VOID_RETURN;
}

private int
t8_func(file, desc)
char *file, **desc;
{
	int ret, err;
	int	pathok = 0;
	
	DBUG_ENTER("t8_func");

	errno = NOERROR;
	if ((ret = rmdir(file)) != SUCCEED)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("rmdir(\"%s\") did not give correct results", file);
		valrpt(SUCCEED, ret, NOERROR, err);
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;
		
	PATH_FUNC_RPT(1);			/* Checked by do_access */
	
	DBUG_RETURN(0);
}


#if	   TEST_XNFS
private int
subtest10_11(file)
char *file;
{
	int	ret, err;
	int	pathok = 0;
	
	DBUG_ENTER("subtest10_11");

	errno = 0;
	if ((ret = rmdir(file)) != SYSERROR || errno != EACCES)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("rmdir(\"%s\") did not give correct results", file);
		valrpt(SYSERROR, ret, EACCES, err);
		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 1)			/* PATH_TRACEing wrong */
		DBUG_RETURN(PATH_NOK);

	DBUG_RETURN(OK);
}
#endif	/* TEST_XNFS */


private int
subtest12(file)
char *file;
{
	/* EBUSY on mount point */

	int ret, err;
	int	pathok = 0;
	
	DBUG_ENTER("subtest12");

	globok = 0; 
	if (mountfs(file, (mode_t) MODEANY) != 0)
	{
		DBUG_RETURN(NOK);
	}
	else
		PATH_FUNC_TRACE;
		
	errno = 0;
	if ((ret = rmdir(file)) != SYSERROR || errno != EBUSY)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("rmdir(\"%s\") did not give correct results", file);
		valrpt(SYSERROR, ret, EBUSY, err);
		(void) umntfs(file);
		DBUG_RETURN(NOK);
	}
	else if (eaccess(file, F_OK) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("unsuccessful call to rmdir() removed directory");
		(void) umntfs(file);
		DBUG_RETURN(NOK);
	}
	else
		PATH_TRACE;
		
	(void) umntfs(file);

	if (pathok != 2)			/* PATH_TRACEing wrong */
		DBUG_RETURN(PATH_NOK);

	DBUG_RETURN(OK);
}

private void
subtest20(mdir, file)
char	*mdir, *file;
{
	/* remove directory on read only file system */

	char	*errptr;
	int rval, err;
	int	pathok = 0;
	
	DBUG_ENTER("subtest20");

	testfail = 0;

	globok = 0; 
	if (setuprofs(mdir, file, 'd', (mode_t) MODEANY) != 0)
	{
		DBUG_VOID_RETURN;	/* reported in setuprofs() */
	}
	else
		PATH_FUNC_TRACE;
		
	globok = 0;
	savtimes(mdir);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	rval = rmdir(file);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;

	if (rval != SYSERROR || err != EROFS)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("rmdir(\"%s\") did not give correct results", file);
		valrpt(SYSERROR, rval, EROFS, err);
	}
	else
		PATH_TRACE;

	if (err == EROFS)
	{
		globok = 0;
		errptr = chktimes(mdir, 0);
		PATH_FUNC_TRACE;

		if(errptr != NULL)
		{
			if(!testfail++)
				xx_rpt(FAILURE);
			in_rpt("Unsucessful rename() changed the time stamps on %s", mdir);
			in_rpt("%s", errptr);
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	if (rval == SYSERROR)
	{
		if (eaccess(file, F_OK) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to rmdir() removed directory");
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6); 

	(void) clnuprofs(mdir, file);

	DBUG_VOID_RETURN;
}
