/*
*      SCCS:  @(#)  POSIX.os/files/stat/stat.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/stat - T.stat Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.stat

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the stat() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	25 Apr 1989
MODIFICATIONS:
		Geoff Clare, 21 Nov 1989
			Cannot use d_ino for testing st_ino
			(as not available in POSIX-only mode)

		Geoff Clare, 4 Apr 1990
			New interface with pathtoolng() and nametoolng().

		Anthony Lee, 30 July 1990
			Addded resilience code.

                John-Paul, 29 July 1992
                        Mode parameter in calls to crfile() and crdir()
                        changed from 'int' to 'mode_t' type.

		David Sawyer, 16 November 1992
			Modifications for XNFS tests

		Geoff Clare, 2 Nov 1993
			Use file created on server in XNFS timestamp test
			to avoid reading cached data.

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<time.h>
#include	<string.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<pathres.h>
#include	<chktimes.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"stat_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef stat
#endif


#define NO_TESTS	17
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define NULLST		(struct stat *)0
#if	   TEST_XNFS
#define NOK		02
#define PATH_OK		04
#endif	/* TEST_XNFS */

#if	   TEST_XNFS
extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;
#endif	/* TEST_XNFS */

extern	char	*errname();
extern	char	*execpath();
#if	   TEST_XNFS
extern	char	*itoa();
extern	char	*xnfs_nametoolng();
#else	/* TEST_XNFS */
extern	char	**nametoolng();
#endif	/* TEST_XNFS */
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	void	valrpt();


public	int	no_tests = NO_TESTS;

private int	testfail;

private	char	*prog_su =	"stat_su";
private	char	*t3_file =	"stat-t.3";
private	char	*t4_file =	"stat-t.4";
private char	*t5_dir =	"stat-d.5";
private char	*t5_file =	"stat-d.5/stat-t.5";
private	char	*t6_file =	"stat-t.6";
private char	*t7_dir =	"stat-d.7";
private char	*t7_file =	"stat-d.7/stat-t.7";
#if	   TEST_XNFS
private char	*t7_file2 =	T7_FILE2;
#endif	/* TEST_XNFS */
private	char	*t9_file =	"stat-t.9";
private char	*t10_dir =	"stat-d.10";
private char	*t10_file =	"stat-d.10/stat-t.10";
private	char	*t11_file =	"stat-t.11";
private char	*t13_ndir =	"stat-t.13";
private char	*t13_file =	"stat-t.13/stat-t.13";

private void	prepare_tests();
private void	cln_tests();

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();

#if	   TEST_XNFS
private void	ch_t7();
#endif	/* TEST_XNFS */

private int	tbstat();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	NULL, 0
};


private void
prepare_tests()
{
	DBUG_ENTER("prepare_tests");

	(void) umask((mode_t)0);

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	char **p;

	DBUG_ENTER("cln_tests");

	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);
	(void) unlink(t3_file);
	(void) unlink(t4_file);
	(void) chmod(t5_dir, MODEANY);
	(void) unlink(t5_file);
	(void) rmdir(t5_dir);
	(void) unlink(t6_file);
	(void) chmod(t7_dir, MODEANY);
	(void) unlink(t7_file);
	(void) rmdir(t7_dir);
	(void) unlink(t9_file);
	(void) unlink(t13_ndir);

	DBUG_VOID_RETURN;
}


private void
test1()
{
	/* pathname resolution */

	char *p1, *p2;
	int npairs, i;
	struct stat stbuf1, stbuf2;
	int	pathok = 0;
	int 	ret1, ret2;
	
	DBUG_ENTER("test1");

	testfail = 0;

	npairs = init_pathres(PRES_DIRS|PRES_CREAT);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_DIRS|PRES_CREAT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

#ifdef TESTBED2
#define p2 "."
#endif

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		globok = 0;
		ret1 = tbstat(p2, &stbuf2, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		ret2 = tbstat(p1, &stbuf1, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		
		if (ret1 != SUCCEED || ret2 != SUCCEED)
		    {
		    	PATH_TRACE; 		/* There is a PATH_TRACE in this loop so must have one here also if continue-ing */
			continue;
		    }

#ifdef TESTBED2
#undef p2
#endif

		if (stbuf1.st_dev != stbuf2.st_dev ||
		    stbuf1.st_ino != stbuf2.st_ino)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("stat() did not resolve \"%s\" and \"%s\" to the same file",
				p1, p2);
				
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
	{
		PATH_XS_RPT(1 + (npairs * 3));
	}
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames */

	char	**p;
	dev_t	*devtab;
	ino_t	*inotab;
	int	nfile, i, j, fd;
	struct stat stbuf;
	int	pathok = 0;
	int	iter = 0;
	DBUG_ENTER("test2");

	testfail = 0;
	iter = 1;			/* PATH_TRACE 1st iteration only */
	for (nfile = 0, p = portfnam; *p != NULL; p++, nfile++)
	{
		if ((fd = crfile(*p, MODEANY,
				(int)geteuid(), (int)getegid())) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create file \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
		else if (iter)
		{
			PATH_TRACE;
			iter = 0;		/* Only trace the forst iteration */
		}
		(void) close(fd);
	}

	devtab = (dev_t *)malloc((size_t)(nfile * sizeof(dev_t)));
	inotab = (ino_t *)malloc((size_t)(nfile * sizeof(ino_t)));
	if (devtab == NULL || inotab == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	portfnam[1] = portfnam[0];
#endif

	for (i = 0, p = portfnam; i < nfile; p++, i++)
	{
		globok = 0; 
		if (tbstat(*p, &stbuf, SUCCEED, NOERROR) != SUCCEED)
		{
			inotab[i] = 0;
		}
		else
		{
			devtab[i] = stbuf.st_dev;
			inotab[i] = stbuf.st_ino;
		}
		PATH_FUNC_TRACE; 
	}

	/* test for uniqueness of st_dev, st_ino pairs */
	DBUG_PRINTF("test2","nfile is = %d",nfile);
	iter = 0;				/* The expected path count for this loop */
	for (i = 0; i < nfile; i++)
	{
		if (inotab[i] == 0)
		{
			PATH_TRACE; 	/* Need to have one to balance the one below */
			iter++;		/* Expected loop count */
			continue;
		}

		for (j = i+1; j < nfile; j++)
		{
			if (inotab[i] == inotab[j] &&
			    devtab[i] == devtab[j])
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("stat() gave the same st_dev and st_ino for files \"%s\" and \"%s\"",
					portfnam[i], portfnam[j]);
			}
			else
			{
				PATH_TRACE;
				iter++;
			}
		}
	}

	if (testfail == 0)
	{
		PATH_XS_RPT(2 + nfile + iter); 
	}
	
	free((void *)devtab);
	free((void *)inotab);
	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* stat() returns information about specified file.
	   This is tested more thoroughly in the following testsets:

		st_mode:	chmod, mkfifo, mkdir
		st_uid, st_gid:	chown
		st_nlink:	link
		st_ino:		readdir
	*/

	time_t		t0, t1;
	dev_t		dev1, dev2;
	ino_t		ino1, ino2;
	int		fd, err;
	mode_t		mode = (mode_t)(S_IRWXU|S_IWGRP|S_IXOTH);
	struct stat	stbuf;
	char		wbuf[20];
	int	pathok = 0;
	
	DBUG_ENTER("test3");

	testfail = 0;


#if	   TEST_XNFS

	if (xnfs_rtime_c(&t0, SP_T3_RTIME1_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	(void) time(&t0);
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if ((fd = crfile(t3_file, mode,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		

#if	   TEST_XNFS

	if (xnfs_rtime_c(&t1, SP_T3_RTIME2_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	(void) time(&t1);
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if (write(fd, wbuf, (unsigned)sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("could not write to file \"%s\" - errno %d (%s)",
			t3_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	(void) sleep(1);

	if (stat(".", &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\".\", buf) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	dev1 = stbuf.st_dev;
	ino1 = stbuf.st_ino;

	if (stat("..", &stbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"..\", buf) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	dev2 = stbuf.st_dev;
	ino2 = stbuf.st_ino;

	globok = 0; 
	if (tbstat(t3_file, &stbuf, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

#ifdef TESTBED2
stbuf.st_dev++;
stbuf.st_ino = ino1;
#endif

	if (stbuf.st_dev != dev1 || stbuf.st_dev != dev2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat() did not set st_dev correctly");
		in_rpt("observed %s: 0x%lx, \".\": 0x%lx, \"..\": 0x%lx",
			t3_file, (long)stbuf.st_dev, (long)dev1, (long)dev2);
	}
	else
		PATH_TRACE;
 		
	if (stbuf.st_ino == ino1 || stbuf.st_ino == ino2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat() gave same st_ino for \"%s\" and \"%s\"",
			t3_file, (stbuf.st_ino == ino1 ? "." : ".."));
	}
	else
		PATH_TRACE;
		
	if (!S_ISREG(stbuf.st_mode))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\", buf) set st_mode file type incorrectly: S_ISREG(st_mode) was false",
			t3_file);
	}
	else
		PATH_TRACE;
		
	if ((stbuf.st_mode & MODEANY) != mode)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\", buf) set st_mode permission bits incorrectly: expected %#o, observed %#o",
			t3_file, (int)mode, (int)(stbuf.st_mode & MODEANY));
	}
	else
		PATH_TRACE;
		
	if (stbuf.st_uid != geteuid())
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\", buf) set st_uid incorrectly: expected %d, observed %d",
			t3_file, (int)geteuid(), (int)stbuf.st_uid);
	}
	else
		PATH_TRACE;
		
	if (stbuf.st_gid != getegid())
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\", buf) set st_gid incorrectly: expected %d, observed %d",
			t3_file, (int)getegid(), (int)stbuf.st_gid);
	}
	else
		PATH_TRACE;

	if (stbuf.st_size != sizeof(wbuf))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\", buf) set st_size incorrectly: expected %ld, observed %ld",
			t3_file, (long)sizeof(wbuf), (long)stbuf.st_size);
	}
	else
		PATH_TRACE;
		
	if (stbuf.st_nlink != 1)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\", buf) set st_nlink incorrectly: expected 1, observed %ld",
			t3_file, (long)stbuf.st_nlink);
	}
	else
		PATH_TRACE;
		
	if (stbuf.st_atime != t0 && stbuf.st_atime != t0 + 1 && 
						stbuf.st_atime != t0 + 2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\", buf) set st_atime incorrectly: expected %ld, observed %ld",
			t3_file, (long)t0, (long)stbuf.st_atime);
	}
	else
		PATH_TRACE;
		
	if (stbuf.st_mtime != t1 && stbuf.st_mtime != t1 + 1 && 
						stbuf.st_mtime != t1 + 2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\", buf) set st_mtime incorrectly: expected %ld, observed %ld",
			t3_file, (long)t1, (long)stbuf.st_mtime);
	}
	else
		PATH_TRACE;
		
	if (stbuf.st_ctime != t1 && stbuf.st_ctime != t1 + 1 && 
						stbuf.st_ctime != t1 + 2)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("stat(\"%s\", buf) set st_ctime incorrectly: expected %ld, observed %ld",
			t3_file, (long)t1, (long)stbuf.st_ctime);
	}
	else
		PATH_TRACE;
		
	if (testfail == 0)
	{
		PATH_XS_RPT(18);
	}

	(void) unlink(t3_file);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* stat() does not require permissions on file */

	int	fd;
	int	pathok = 0;
	
	DBUG_ENTER("test4");

	testfail = 0;

	if ((fd = crfile(t4_file, (mode_t)0,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	
	globok = 0; 
	(void) tbstat(t4_file, NULLST, SUCCEED, NOERROR);
	PATH_FUNC_TRACE; 

	if (testfail == 0)
		PATH_XS_RPT(2); 

	(void) unlink(t4_file);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* stat() succeeds when directories in path are searchable */

	int	fd, err;
	mode_t	mode = (S_IXUSR|S_IXGRP|S_IXOTH);
	int	pathok = 0;
	
	DBUG_ENTER("test5");

	testfail = 0;

	if (crdir(t5_dir, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t5_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if ((fd = crfile(t5_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (chmod(t5_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t5_dir, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbstat(t5_file, NULLST, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	

	if (testfail == 0)
	{
		PATH_XS_RPT(4); 
	}

	(void) chmod(t5_dir, MODEANY);
	(void) unlink(t5_file);
	(void) rmdir(t5_dir);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* stat() updates timestamps "marked for update" */

	char	*chkt;
	int	fd, err;
	char	wbuf[20];
	int	pathok = 0;
	
	DBUG_ENTER("test6");

	testfail = 0;

#if TEST_XNFS
	
	if (tet_sync(SP_T6_CREAT_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREAT_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_TRACE;

#else /* TEST_XNFS */

	if ((fd = crfile(t6_file, MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t6_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, wbuf, (unsigned)sizeof(wbuf)) <= 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("first write to file \"%s\" failed - errno %d (%s)",
			t6_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

#endif /* TEST_XNFS */

	if ((fd = open(t6_file, O_RDWR)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d (%s)",
			t6_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(t6_file);
	PATH_FUNC_TRACE;

	if (read(fd, wbuf, (unsigned)sizeof(wbuf)) <= 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("could not read from file \"%s\" - errno %d (%s)",
			t6_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, wbuf, (unsigned)sizeof(wbuf)) <= 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("second write to file \"%s\" failed - errno %d (%s)",
			t6_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	savtimes(t6_file);
#endif

	/* note: no close() before the stat() [in chktimes()] */

	globok = 0;
	chkt = chktimes(t6_file, NEWATIME|NEWMTIME|NEWCTIME);
	if (chkt != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("%s", chkt);
	}
	else
	{
		PATH_FUNC_TRACE; 
		PATH_XS_RPT(7);
	}

	(void) close(fd);

#if TEST_XNFS
	(void) tet_sync(SP_T6_UNLINK_START, snames, SYNC_TIMEOUT);
#else
	(void) unlink(t6_file);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* search permission denied */

	int	fd, err;
	mode_t	mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
	int	pathok = 0;
#if	   TEST_XNFS
	int	rc;
#endif	/* TEST_XNFS */
	
	DBUG_ENTER("test7");

	testfail = 0;

	if (crdir(t7_dir, MODEANY,
			(int)geteuid(), (int)getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t7_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t7_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t7_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (chmod(t7_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t7_dir, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbstat(t7_file, NULLST, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	(void) chmod(t7_dir, MODEANY);
	(void) unlink(t7_file);
	(void) rmdir(t7_dir);

#if	   TEST_XNFS

	if (tet_sync(SP_T7_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rc = cppair(ch_t7, NULLFN, WAITTIME, NOK | PATH_OK);

	if (rc & NOK)
	{
		testfail++;
	}
	else if (rc & PATH_OK)
	{
		PATH_TRACE;
	}

	if (tet_sync(SP_T7_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */


	if (testfail == 0)
		PATH_XS_RPT(7); 

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS
private void
ch_t7()
{
	int err;
	char *args[7];

	DBUG_ENTER("ch_t7");

	args[0] = execpath(prog_su);
	args[1] = itoa(tet_thistest);
	args[2] = t7_file2;
	args[3] = "";
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_EXIT(NOK);
}

#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test8()
{
	DBUG_ENTER("test8");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test8()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;
	
	DBUG_ENTER("test8");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname[0], MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbstat(longname[0], NULLST, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		
		(void) close(fd);
	}

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbstat(longname[1], NULLST, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(3); 

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */

private void
test9()
{
	/* path longer than PATH_MAX */

	char	**longname;
	int	fd;
	int	pathok = 0;
	
	DBUG_ENTER("test9");

	testfail = 0;

	if ((fd = crfile(t9_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t9_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	longname = pathtoolng(t9_file);
	PATH_FUNC_TRACE;
	
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbstat(longname[0], NULLST, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	
	globok = 0; 
	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbstat(longname[1], NULLST, SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(5); 

	clntoolng();
	(void) unlink(t9_file);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	/* component of path prefix does not exist */

	int	pathok = 0;
	
	DBUG_ENTER("test10");

	testfail = 0;

	if (eaccess(t10_dir, F_OK) == 0 && remove(t10_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t10_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbstat(t10_file, NULLST, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test11()
{
	/* file which does not exist */
	int	pathok = 0;
	
	DBUG_ENTER("test11");

	testfail = 0;

	if (eaccess(t11_file, F_OK) == 0 && remove(t11_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbstat(t11_file, NULLST, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test12()
{
	/* path is "" */
	int	pathok = 0;
	
	DBUG_ENTER("test12");

	testfail = 0;

	globok = 0;
	(void) tbstat("", NULLST, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
	{
		PATH_TRACE;
		PATH_XS_RPT(2); 
	}

	DBUG_VOID_RETURN;
}

private void
test13()
{
	/* component of path is not a directory */

	int	fd;
	int	pathok = 0;
	
	DBUG_ENTER("test13");

	testfail = 0;

	if ((fd = crfile(t13_ndir, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t13_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbstat(t13_file, NULLST, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2); 

	(void) unlink(t13_ndir);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
test14()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test14");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbstat(longname, NULLST, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test14()
{
	DBUG_ENTER("test14");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test15()
{
	char	*longname;
	int	pathok = 0;

	DBUG_ENTER("test15");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbstat(longname, NULLST, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test15()
{
	DBUG_ENTER("test15");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test16()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test16");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbstat(longname, NULLST, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test16()
{
	DBUG_ENTER("test16");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test17()
{
	char	*longname;
	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test17");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname, (mode_t)MODEANY,
			(int)getuid(), (int)getgid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbstat(longname, NULLST, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test17()
{
	DBUG_ENTER("test17");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private int
tbstat(tbpath, tbbuf, expret, experrno)
char	*tbpath;
struct stat *tbbuf;
int	expret;
int	experrno;
{
	char	*reppath = tbpath;
	int	rval, err;
	char	pathbuf[30];
	struct stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("tbstat");
	
	if (tbbuf == NULLST)
		tbbuf = &stbuf;

	errno = NOERROR;
	rval = stat(tbpath, tbbuf);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
		PATH_TRACE; 			/* Just really to show tbstat has been called */
	}					/* Both paths of the if are valid paths */
	else
		PATH_TRACE;
		
	if (expret == SUCCEED && rval == expret)
	{
		PATH_FUNC_RPT(1);
		DBUG_RETURN (rval);
	}
	else if (expret == SYSERROR && rval == expret && err == experrno)
	{
		PATH_FUNC_RPT(1); 
		DBUG_RETURN(rval);
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("stat(\"%s\", buf) did not give correct results", reppath);
	valrpt(expret, rval, experrno, err);

	DBUG_RETURN(rval);
}
