/*
*      SCCS:  @(#)  POSIX.os/files/stat_X/stat_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/stat_X - T.stat_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.stat_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the stat() system call, X/Open extensions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	25 Apr 1989
MODIFICATIONS:
		Anthony Lee, 30 July 1990
			Added resilience code.
		
		Geoff Clare, 4 Oct 1990
			Allow "unsup" value for VSX_(BLK|CHR)DEV_FILE

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef stat
#endif


#define NO_TESTS	2

extern	char	*errname();
extern	void	valrpt();


public	int	no_tests = NO_TESTS;

private void	test1();
private void	test2();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};


private void
test1()
{
	/* st_rdev element is device ID for block special files */

	char		*file;
	int		rval, err;
	struct stat	stbuf;
	int	pathok = 0;
	
	DBUG_ENTER("test1");

	file = tet_getvar("VSX_BLKDEV_FILE");
	if (file == NULL || *file == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_BLKDEV_FILE is not defined");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(file, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_BLKDEV_FILE is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	errno = NOERROR;
	if ((rval = stat(file, &stbuf)) != SUCCEED)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("stat(\"%s\", buf) did not give correct results", file);
		valrpt(SUCCEED, rval, NOERROR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (pathok == 2)		/* The number of expected path traces */
	{
		xx_rpt(RFI);
		in_rpt("stat(\"%s\", buf) returned st_rdev value of 0x%lx",
			file, (long)stbuf.st_rdev);
	}
	else
	{
		xx_rpt(DELETION);
		in_rpt("Deletion reason: Pathcheck value %d",pathok);
		DBUG_VOID_RETURN; 
	}
	
	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* st_rdev element is device ID for character special files */

	char		*file;
	int		rval, err;
	struct stat	stbuf;
	int	pathok = 0;
	
	DBUG_ENTER("test2");

	file = tet_getvar("VSX_CHRDEV_FILE");
	if (file == NULL || *file == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_CHRDEV_FILE is not defined");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(file, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_CHRDEV_FILE is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	errno = NOERROR;
	if ((rval = stat(file, &stbuf)) != SUCCEED)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("stat(\"%s\", buf) did not give correct results", file);
		valrpt(SUCCEED, rval, NOERROR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if (pathok == 2)		/* The number of expected path traces */
	{
		xx_rpt(RFI);
		in_rpt("stat(\"%s\", buf) returned st_rdev value of 0x%lx",
			file, (long)stbuf.st_rdev);
	}
	else
	{
		xx_rpt(DELETION);
		in_rpt("Deletion reason: Pathcheck value %d",pathok);
		DBUG_VOID_RETURN; 
	}
	
	DBUG_VOID_RETURN;
}
