/*
*      SCCS:  @(#)  POSIX.os/files/umask/umask.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/umask - T.umask Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.umask

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the umask() interface.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	3 May 1989
MODIFICATIONS:
		Geoff Clare, 21 Feb 1990
			Add clean_tests() function.
		
		Geoff Clare, 20 June 1990
			Add umask(umask()) test.

		Anthony Lee, 30 July 1990
			Added resilience code.
		
		Geoff Clare, 27 May 1992
			Slight change to reporting in chkmode().

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef umask
#endif


#define	NO_TESTS	3
#define	WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define FUNC_OPEN	1
#define FUNC_CREAT	2
#define FUNC_MKDIR	3
#define FUNC_MKFIFO	4

extern	char	*errname();

public	int	no_tests =  NO_TESTS;

private mode_t	mask[] = {
	0,
	S_IRUSR,
	S_IWUSR,
	S_IXUSR,
	S_IRGRP,
	S_IWGRP,
	S_IXGRP,
	S_IROTH,
	S_IWOTH,
	S_IXOTH,
	(MODEANY & ~S_IRUSR),
	(MODEANY & ~S_IWUSR),
	(MODEANY & ~S_IXUSR),
	(MODEANY & ~S_IRWXU),
	(MODEANY & ~S_IRGRP),
	(MODEANY & ~S_IWGRP),
	(MODEANY & ~S_IXGRP),
	(MODEANY & ~S_IRWXG),
	(MODEANY & ~S_IROTH),
	(MODEANY & ~S_IWOTH),
	(MODEANY & ~S_IXOTH),
	(MODEANY & ~S_IRWXO),
	MODEANY,
};
private int	nmasks = sizeof(mask)/sizeof(mask[0]);
private int	testfail;
private char	*t2_file = "umask-t.2";

private void 	test1();
private void 	test2();
private void 	test3();
private void 	clean_tests();
private void 	chkmode();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private void
clean_tests()
{
	DBUG_ENTER("clean_tests");

	(void) remove(t2_file);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* umask() sets the umask and returns the old value */

	int	i;
	mode_t	rval;
	int	pathok = 0;
	
	DBUG_ENTER("test1");

	testfail = 0;

	(void) umask(mask[0]);

#ifdef TESTBED
	(void) umask(mask[1]);
#endif

	for (i = 1; i < nmasks; i++)
		if ((rval = umask(mask[i])) != mask[i-1])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("umask(%#o) did not return mask value set with previous call",
				(int)mask[i]);
			in_rpt("expected: %#o, observed: %#o",
				(int)mask[i-1], (int)rval);
		}
		else
			PATH_TRACE;
			
	if (testfail == 0)
	{
		PATH_XS_RPT(nmasks-1);
	}

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* mask turns off permission bits in mode passed to
	   open(), creat(), mkdir() and mkfifo() */

	int	i;
	mode_t	mode = S_IRUSR|S_IWGRP|S_IXOTH;
	int	pathok = 0;
	
	DBUG_ENTER("test2");

	testfail = 0;

	for (i = 0; i < nmasks; i++)
	{
		(void) umask(mask[i]);

#ifdef TESTBED
		(void) umask(mask[1]);
#endif

		globok = 0;
		chkmode(FUNC_OPEN,   mask[i], MODEANY);
		PATH_FUNC_TRACE;

		globok = 0;
		chkmode(FUNC_OPEN,   mask[i], mode);
		PATH_FUNC_TRACE;

		globok = 0;
		chkmode(FUNC_CREAT,  mask[i], MODEANY);
		PATH_FUNC_TRACE;

		globok = 0;
		chkmode(FUNC_CREAT,  mask[i], mode);
		PATH_FUNC_TRACE;

		globok = 0;
		chkmode(FUNC_MKDIR,  mask[i], MODEANY);
		PATH_FUNC_TRACE;

		globok = 0;
		chkmode(FUNC_MKDIR,  mask[i], mode);
		PATH_FUNC_TRACE;

		globok = 0;
		chkmode(FUNC_MKFIFO, mask[i], MODEANY);
		PATH_FUNC_TRACE;

		globok = 0;
		chkmode(FUNC_MKFIFO, mask[i], mode);
		PATH_FUNC_TRACE;
		
	}

	if (testfail == 0)
	{
		PATH_TRACE; 
		PATH_XS_RPT(1 + (nmasks * 8)); 
	}

	(void) remove(t2_file);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* umask(umask()) does not change the umask */

	int	i;
	mode_t	rval;
	int	pathok = 0;
	
	DBUG_ENTER("test3");

	testfail = 0;

	(void) umask(mask[0]);

#ifdef TESTBED
	(void) umask(mask[1]);
#endif

	for (i = 1; i < nmasks; i++)
	{
		(void) umask(umask(mask[i]));

		if ((rval = umask(mask[i])) != mask[i-1])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("umask(umask(%#o)) altered mask from %#o to %#o",
				(int)mask[i], (int)mask[i-1], (int)rval);
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
	{
	PATH_XS_RPT(nmasks-1);
	}

	DBUG_VOID_RETURN;
}


private char *
strfunc(func, file, mode)
int func;
char *file;
mode_t mode;
{
	static char buf[80];

	buf[0] = '\0';

	switch (func)
	{
	case FUNC_OPEN :
		(void) sprintf(buf, "open(\"%s\", O_WRONLY|O_CREAT, %#o)",
			file, (int)mode);
		break;

	case FUNC_CREAT :
		(void) sprintf(buf, "creat(\"%s\", %#o)", file, (int)mode);
		break;

	case FUNC_MKDIR :
		(void) sprintf(buf, "mkdir(\"%s\", %#o)", file, (int)mode);
		break;

	case FUNC_MKFIFO :
		(void) sprintf(buf, "mkfifo(\"%s\", %#o)", file, (int)mode);
		break;
	}

	return buf;
}

private void
chkmode(func, currmask, mode)
int func;
mode_t currmask, mode;
{
	/* create a file and check its mode */

	int		rval, err;
	struct stat 	statb;
	static mode_t	rptmask;
	static int	first_rpt = 1;
	int	pathok = 0;

	DBUG_ENTER("chkmode");
	
	(void) remove(t2_file);
	if (eaccess(t2_file, F_OK) == 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: could not remove \"%s\"", t2_file);
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;	/* Just to trace this function */
		
	switch (func)
	{
	case FUNC_OPEN :
		rval = open(t2_file, O_WRONLY|O_CREAT, mode);
		break;

	case FUNC_CREAT :
		rval = creat(t2_file, mode);
		break;

	case FUNC_MKDIR :
		rval = mkdir(t2_file, mode);
		break;

	case FUNC_MKFIFO :
		rval = mkfifo(t2_file, mode);
		break;
	}

	if (rval == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: %s failed - errno %d (%s)",
			strfunc(func, t2_file, mode), err, errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

	if (func == FUNC_OPEN || func == FUNC_CREAT)
		(void) close(rval);

	if (stat(t2_file, &statb) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("stat() failed on \"%s\" - errno %d (%s)",
			t2_file, err, errname(err));
	}
	else if ((statb.st_mode & MODEANY) != (mode & ~currmask))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		if (first_rpt || rptmask != currmask)
		{
			in_rpt("umask(%#o) did not set file creation mask correctly",
				(int)currmask);
			rptmask = currmask;
			first_rpt = 0;
		}
		in_rpt("%s created file with permissions %#o, expected %#o",
			strfunc(func, t2_file, mode),
			(int)(statb.st_mode & MODEANY),
			(int)(mode & ~currmask));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);
	
	DBUG_VOID_RETURN; 
}
