/*
 *      SCCS:  @(#)  POSIX.os/files/unlink/enospc.c	Rel 4.4.1     (06/09/97) 
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1991 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#ifndef NO_MAIN
static	char sccsid[] = "@(#)POSIX.os/files/unlink - enospc Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif
#endif  /* lint	*/

/***********************************************************************

NAME:	enospc.c

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()
		enospc()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char *argv[];

DESCRIPTION:
		Test that space freed up by removing files is made available
		to extend new files.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:
AUTHOR:		Steve Ratcliffe, UniSoft Ltd.
DATE CREATED:	28 Feb 1990
MODIFICATIONS:
		Peter Lee, Unisoft Ltd.,
		Wed Aug 01 10:04:02 BST 1990
		Added resilience code.
		Split main into main and enospccall for debugging
			and path checking purposes.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Dave Cater, 17 June 1992
			Altered RENAME case to remove space associated with 
			fill.tmp3 (the last file created) instead of fill.tmp2

		David Sawyer, 12 Nov 1992
			Modifications for XNFS tests
		
		Geoff Clare, 29 July 1993
			Retry the space-reusing call if it gives ENOSPC

************************************************************************/

#include <std.h>
#include <dbug.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <report.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <tet_api.h>
#include <xnfs.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef unlink
#undef rename
#undef rmdir
#endif


#define	WAITTIME	(3 * SPEEDFACTOR)
#define NAP		2
#define MAXTRYS		10

#ifndef NO_MAIN
public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

#endif

extern	char	**environ;

extern	char	*itoa();
extern	char	*errname();
extern	void	set_pname();
extern	void	valrpt();

private int 	testfail;

private int 	interface;
#define	I_UNLINK	0
#define I_RENAME	1
#define	I_RMDIR 	2

void 	enospc();
void	enospccall();

extern int 	setupnospc();
extern void	tidynospc();
extern char	*newstr();

#define MODEANY	(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

#define BUFLEN	512

/*
 * A string used as data to write out
 */
#define STRPAT	"abcdefghijklmnopqrstuvwxyz"
#define PATLEN (sizeof(STRPAT)-1)

#ifndef NO_MAIN

public	int
tet_main(argc, argv)
int	argc;
char	*argv[];
{
char	*mountdir;

	set_pname(argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		return(OK);
	}

	if (argc >= 5)
		vg_dbug = argv[4];
	if (argc >= 4)
		vg_ident = argv[3];

	if (strcmp(argv[2], "unlink") == 0)
		interface = I_UNLINK;
	else if (strcmp(argv[2], "rename") == 0)
		interface = I_RENAME;
	else if (strcmp(argv[2], "rmdir") == 0)
		interface = I_RMDIR;
	else {
		xx_rpt(DELETION);
		in_rpt("Bad interface");
		return(OK);
	}

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	mountdir = argv[1];

	enospccall(mountdir);

	/* NOTREACHED */
}

void
enospccall(mountdir)
char* mountdir;
{
	int	pathok = 0;

	DBUG_ENTER("enospccall");

	globok = 0;
	if (setupnospc(mountdir)) {
		testfail++;
		DBUG_TET_EXIT(OK);
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	enospc(mountdir);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	/* Tidy up and umount file system */
	tidynospc(mountdir, 3);

	DBUG_TET_EXIT(OK);
	/*NOTREACHED*/
}
#endif /* !NO_MAIN */

/*
 * enospc() tries to run out of space.
 */
void
enospc(mountdir)
char	*mountdir;
{
	char	*name;
	char	*name1;
	char	*outstr;
	int 	ret, err;
	int 	i;
	int 	fd;
	int	pathok = 0;

	DBUG_ENTER("enospc");

	globok = 0;

	name = (char*)malloc(strlen(mountdir)+sizeof("/fill.tmp0000"));
	if (name == NULL) {
		testfail++;
		xx_rpt(DELETION);
		in_rpt("Out of memory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Build up  temp file name and open it
	 */
	(void) strcat(strcpy(name, mountdir), "/fill.tmp1");

	fd = open(name, O_RDWR);

	if (fd == SYSERROR) {
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: Open of %s failed - errno %s", name, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (lseek(fd, (off_t)0, SEEK_END) == SYSERROR) {
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: Seek to end of file failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	outstr = (char*) malloc((size_t) BUFLEN);
	if (outstr == NULL) {
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: Out of memory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < BUFLEN; i++)
	{
		if (i==0)
			PATH_TRACE;
		outstr[i] = STRPAT[i%PATLEN];
	}

	DBUG_PRINTF("enospc", "write block on %s", name);

	while (write(fd, outstr, BUFLEN) != SYSERROR)
		;

	if (errno != ENOSPC) {
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("attempt to fill file-system failed");
		valrpt(-1, -1, ENOSPC, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) strcat(strcpy(name, mountdir), "/fill.tmp3");

	switch (interface) {

	case I_UNLINK:
		DBUG_PRINTF("enospc", "unlinking %s", name);
		if (unlink(name) == SYSERROR) {
			err = errno;
			testfail++;
			xx_rpt(DELETION);
			in_rpt("unlink of %s failed", name);
		}
		else
			PATH_TRACE;
		break;
	case I_RENAME:
		/* 
		 * Modified to remove fill.tmp3 (just like in the unlink tests).
		 * The rationale for this is that fill.tmp3 is the last file 
		 * created. So, we are freeing space that was allocated just
		 * prior to filling the file system. This prevents spurious 
		 * failures on some high performance file systems.
		 */
		name1 = newstr(name);
		(void) strcat(strcpy(name, mountdir), "/fill.tmp2");
		DBUG_PRINTF("enospc", "renaming %s to %s", name, name1);
		if (rename(name, name1) == SYSERROR) {
			err = errno;
			testfail++;
			xx_rpt(DELETION);
			in_rpt("rename of %s to %s failed", name, name1);
		}
		else
			PATH_TRACE;
		break;
	case I_RMDIR:
		(void) strcat(strcpy(name, mountdir), "/fill.dir");
		DBUG_PRINTF("enospc", "rmdir %s", name);
		if (rmdir(name) == SYSERROR) {
			err = errno;
			testfail++;
			xx_rpt(DELETION);
			in_rpt("rmdir of %s failed", name);
		}
		else
			PATH_TRACE;
#ifdef TEST_BED
		DBUG_PRINTF("enospc", "In TEST_BED code");
		system("df $VSX_NOSPC_DEV > /dev/null");
#endif
		break;
	default:
		testfail++;
		xx_rpt(DELETION);
		in_rpt("Bad interface");
		break;
	}

	if (!testfail) {

		/*
		 * Now that we have made room for more data this write should
		 * not fail.  Don't test for rmdir() since there is no reason to
		 * suppose that the resources freed up could be used for write
		 */
		if (interface != I_RMDIR) {
			DBUG_PRINTF("enospc", "writing more data");
			for (i = 0; i < MAXTRYS; i++) {
				ret = write(fd, outstr, BUFLEN);
				err = errno;
				if (ret != SYSERROR || err != ENOSPC)
					break;
				/* allow for a delay before space can be reused */
				(void) sleep(NAP);
			}
			if (ret == SYSERROR) {
				if (!testfail++)
					xx_rpt(FAILURE);
				in_rpt("write failed - errno %d (%s)",
					err, errname(err));
			}
			else
				PATH_TRACE;
		}
		else
			PATH_TRACE;

		/*
		 * Now try to create a new file
		 */
		(void) strcat(strcpy(name, mountdir), "/newname");
		DBUG_PRINTF("enospc", "creating new file %s", name);
		if (interface == I_RMDIR) 
		{
			for (i = 0; i < MAXTRYS; i++) {
				ret = mkdir(name, MODEANY);
				err = errno;
				if (ret != SYSERROR || err != ENOSPC)
					break;
				/* allow for a delay before space can be reused */
				(void) sleep(NAP);
			}
			if (ret == SYSERROR) 
			{
				if (!testfail++)
					xx_rpt(FAILURE);
				in_rpt("mkdir of new directory failed - errno %d (%s)",
					err, errname(err));
			}
			else
				PATH_TRACE;
			(void) rmdir(name);
		} 
		else 
		{
			for (i = 0; i < MAXTRYS; i++) {
				ret = creat(name, MODEANY);
				err = errno;
				if (ret != SYSERROR || err != ENOSPC)
					break;
				/* allow for a delay before space can be reused */
				(void) sleep(NAP);
			}
			if (ret == SYSERROR) {
				if (!testfail++)
					xx_rpt(FAILURE);
				in_rpt("creat of new file failed - errno %d (%s)",
					err, errname(err));
			}
			else
				PATH_TRACE;
			(void) close(ret);
			(void) unlink(name);
		}

	}

	PATH_FUNC_RPT(9);

	(void) close(fd);

	free((void*)name);

	DBUG_VOID_RETURN;
}
