/*
*      SCCS:  @(#)  POSIX.os/files/unlink/unlink.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/unlink - T.unlink Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.unlink

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the unlink() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	30 Mar 1989
MODIFICATIONS:	
		Geoff Clare, 9 June 1989:  Add do_access() test.
		Geoff Clare, 22 Aug 1989:  VSX_REMOVE_DIR_EBUSY={Y,S,P,N}

		Geoff Clare, 15 Nov 1989
			New setuprofs() argument
			New do_access() argument

		Geoff Clare, 13 Feb 1990
			In EBUSY tests, after chdir(), open "." to ensure
			the directory is considered "in use" by the process.

		Geoff Clare, 4 Apr 1990
			New interface with pathtoolng() and nametoolng().

		Steve Ratcliffe, 19 Apr 1990
			Add test to check that space is released to the system
			when a file is removed.
		
		Geoff Clare, 30 Apr 1990
			VSX_LINK_DIR_SUPP=U means only unlinking is supported.

		Geoff Clare, 11 May 1990
			New strategies for tests which unlink directories,
			as link() may not be supported.
		
		Geoff Clare, 1 June 1990
			Merge EPERM tests.

		Peter Lee, Unisoft Ltd.,
		Tue Jul 31 17:05:57 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added
		
		Geoff Clare, 14 June 1991
			Check timestamps in EROFS test.

		Peter Skan, 10 Sept 1991
			Call unsetprv(PRV_LINKDIR) where necessary

		Peter Skan, 9 Oct 1991
			Also unset all other assigned privileges
		
		Geoff Clare, 28 Apr 1992
			Don't try and link directories in pathres test
			if only unlinking directories is supported.

		Geoff Clare, 8 May 1992
			Add PRES_NODOT|PRES_NODOTDOT to init_pathres() call.

                John-Paul, 29 July 1992
                        Mode parameter in calls to crfile() and crdir()
                        changed from 'int' to 'mode_t' type.

		David Sawyer, 17 November 1992
			Modifications for XNFS tests

		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.

		Geoff Clare, 15 Oct 1993
			Increase child timeout in test 9.

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<string.h>
#include	<fcntl.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<pathres.h>
#include	<chktimes.h>
#include 	<tet_api.h>
#include 	<xnfs.h>
#include 	"unlink_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef unlink
#endif


#define NO_TESTS	26
#define	WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define NOK		02
#define E_CHDIR		02
#define E_OPEN		04
#define PATH_NOK	8	

extern	char	*errname();
extern	char	*execpath();
#if	   TEST_XNFS
extern	char	*xnfs_nametoolng();
#else	/* TEST_XNFS */
extern	char	**nametoolng();
#endif	/* TEST_XNFS */
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	void	cln_access();
extern	void	valrpt();

extern	int	vggid1;
extern	int	vguid1;

public	int	no_tests = NO_TESTS;

private	char	*t1_file =	"unlink-t.1";
private	char	*t3a_file =	"unlink-t.3a";
private	char	*t3b_files[] = {"unlink-t.3b", "unlink-t.3c", "unlink-t.3d",
				"unlink-t.3e", NULL };
private	char	*t4_file =	"unlink-t.4";
private	char	*t5a_file =	"unlink-t.5a";
private	char	*t5b_file =	"unlink-t.5b";
private	char	*t6a_file =	"unlink-t.6a";
private	char	*t6b_file =	"unlink-t.6b";
private	char	*t7a_file =	"unlink-t.7a";
private	char	*t7b_file =	"unlink-t.7b";
private	char	*t7c_file =	"unlink-t.7c";
private char	*t9_dir =	"unlink-d.9";
private char	*t10_dir =	"unlink-d.10";
private char	*t10_file =	"unlink-d.10/unlink-t.10";
#if	   TEST_XNFS
private char	*t10_file2 =	T10_FILE2;
#endif	/* TEST_XNFS */
private char	*t11_dir =	"unlink-d.11";
private char	*t11_file =	"unlink-d.11/unlink-t.11";
#if	   TEST_XNFS
private char	*t11_file2 =	T11_FILE2;
#endif	/* TEST_XNFS */
private	char	*t12a_file =	"unlink-t.12a";
private	char	*t12b_file =	"unlink-t.12b";
private	char	*t12c_file =	"unlink-t.12c";
private	char	*t14_file =	"unlink-t.14";
private char	*t15_dir =	"unlink-d.15";
private char	*t15_file =	"unlink-d.15/unlink-t.15";
private	char	*t16_file =	"unlink-t.16";
private char	*t18_ndir =	"unlink-t.18";
private char	*t18_file =	"unlink-t.18/unlink-t.18";
#if	  !TEST_XNFS
private	char	*t19a_file =	"unlink-t.19a";
private	char	*t19b_file =	"unlink-t.19b";
#endif	/* TEST_XNFS */
private char	*t20_dir =	"unlink-d.20";
private char	*t20_file =	"unlink-d.20/unlink-t.20";
#if	   TEST_XNFS
private	char	*t25a_file =	T25_FILEA;
private	char	*t25b_file =	T25_FILEB;
private	char	*t26_file =	T26_FILE;
#endif	/* TEST_XNFS */

private char	*prog9 = "unlink_t9";

private int	testfail;
private int	link_dir_supp;
private int	link_dir_err;
private int	unlink_dir_supp;
private	struct stat stbuf1, stbuf2;
private int	setupok;

private void	prepare_tests();
private void	cln_tests();
private void	test1();
private void	subtest1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	subtest5();
private void	test6();
private void	test7();
private void	subtest7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	subtest12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
#if	   TEST_XNFS
private void	subtest25();
#endif	/* TEST_XNFS */

#if	   TEST_XNFS
private void	ch_t1();
private void	ch_t5();
private void	ch1_t7();
#endif	/* TEST_XNFS */
private void	ch2_t7();
private void	ch_t9();
private void	ch_t10();
private void	ch_t11();
private void	ch2_t12();
#if	   TEST_XNFS
private void	ch1_t12();
#endif	/* TEST_XNFS */
private void	ch_t17();
#if	  !TEST_XNFS
private void	ch_t19();
#endif	/* TEST_XNFS */
#if	   TEST_XNFS
private void	ch1_t25();
private void	ch2_t25();
#endif	/* TEST_XNFS */

private void	pa_t7();
private void	pa_t12();

private int	t8_func();

private int	tbunlink();


public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;

#if	   TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif	/* TEST_XNFS */

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	NULL, 0
};

private	void
prepare_tests()
{
	char	*supp;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	if (setprv(PRV_SETID) != 0)
	{
		for (i=1; i<=no_tests; i++)
		{
			tet_delete(i, "setprv(PRV_SETID) failed");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 *  Note: This value applies to both client and server.
	 */
	if ((supp = tet_getvar("VSX_LINK_DIR_SUPP")) == NULL ||
		*supp == '\0')
	{
		for (i=1; i<=no_tests; i++)
		{
			tet_delete(i, "parameter VSX_LINK_DIR_SUPP not set");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	link_dir_supp = (*supp == 'Y' || *supp == 'y');
	unlink_dir_supp = (*supp == 'Y' || *supp == 'y' ||
			 *supp == 'U' || *supp == 'u');

	if (link_dir_supp || unlink_dir_supp)
		link_dir_err = (setprv(PRV_LINKDIR) != 0);

	(void) umask((mode_t)0);

	if(uids()!=0)
	{
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_SETUP_RPT(3);

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	char **p;

	DBUG_ENTER("cln_tests");

	(void) remove(t1_file);
	(void) unlink(t3a_file);
	for (p = t3b_files; *p != NULL; p++)
		(void) unlink(*p);
	(void) unlink(t4_file);
	(void) unlink(t5b_file);
	(void) rmdir(t5a_file);
	(void) unlink(t6a_file);
	(void) unlink(t6b_file);
	(void) unlink(t7b_file);
	(void) unlink(t7c_file);
	(void) rmdir(t7a_file);
	(void) chmod(t10_dir, MODEANY);
	(void) unlink(t10_file);
	(void) rmdir(t10_dir);
	(void) chmod(t11_dir, MODEANY);
	(void) unlink(t11_file);
	(void) rmdir(t11_dir);
	(void) unlink(t12b_file);
	(void) unlink(t12c_file);
	(void) rmdir(t12a_file);
	(void) unlink(t14_file);
	(void) unlink(t18_ndir);
#if	  !TEST_XNFS
	if (link_dir_supp)
	{
		(void) unlink(t19b_file);
		(void) rmdir(t19a_file);
	}
	else
		(void) rmdir(t19b_file);
#endif	/* TEST_XNFS */
	/* can't do t20 */
	(void) rmdir(t9_dir);

	DBUG_VOID_RETURN;
}


private void
test1()
{
	DBUG_ENTER("test1");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t1, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest1();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t1()
{
	DBUG_ENTER("ch_t1");

	subtest1();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
subtest1()
{
	/* pathname resolution */

	char *p1, *p2;
	int fd, err, npairs, i;
	int	pathok = setupok;

	DBUG_ENTER("subtest1");

	testfail = 0;

	if (link_dir_supp)
	{
		if (link_dir_err)
		{
			xx_rpt(DELETION);
			in_rpt("setprv(PRV_LINKDIR) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOT|PRES_NODOTDOT);
		if (npairs <= 0)
		{
			xx_rpt(DELETION);
			in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT|PRES_NODOT|PRES_NODOTDOT) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (crdir(t1_file, MODEANY,
				(int)geteuid(), (int)getegid()) == SYSERROR)
		{
			testfail++;
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t1_file);
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE; /* too keep path count constant */

		npairs = init_pathres(PRES_FILES);
		if (npairs <= 0)
		{
			xx_rpt(DELETION);
			in_rpt("init_pathres(PRES_FILES) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if ((fd = crfile(t1_file, MODEANY,
				(int)geteuid(), (int)getegid())) == SYSERROR)
		{
			testfail++;
			xx_rpt(DELETION);
			in_rpt("could not create file \"%s\"", t1_file);
		}
		else
			PATH_TRACE;
		(void) close(fd);
	}

	/* Check that unlink() resolves both names to the file
	   to which eaccess() resolves the second name */

	if (testfail == 0)
	{
		for (i = 0; i < npairs; i++)
		{
			p1 = pathresnam[2*i];
			p2 = pathresnam[2*i+1];

			DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

			(void) remove(p2);
			if (link(t1_file, p2) == SYSERROR)
			{
				err = errno;
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("deletion reason: link(\"%s\", \"%s\") failed - errno %d (%s)",
					t1_file, p2, err, errname(err));
				continue;
			}
			else
				PATH_TRACE;

			globok = 0;
			if (tbunlink(p2, SUCCEED, NOERROR) != SUCCEED)
			{
				PATH_TRACE;
				continue;
			}
			else
				PATH_FUNC_TRACE;

			if (link(t1_file, p2) == SYSERROR)
			{
				err = errno;
				if (testfail++ == 0)
					xx_rpt(DELETION);
				in_rpt("deletion reason: link(\"%s\", \"%s\") failed - errno %d (%s)",
					t1_file, p2, err, errname(err));
				continue;
			}
			else
				PATH_TRACE;
#ifndef TESTBED2
			globok = 0;
			(void) tbunlink(p1, SUCCEED, NOERROR);
			PATH_FUNC_TRACE;
#else
			PATH_TRACE;
#endif
			if (eaccess(p2, F_OK) == 0)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("unlink() did not resolve \"%s\" and \"%s\" to the same file",
					p1, p2);
				(void) unlink(p2);
			}
			else
				PATH_TRACE;
		}
	}

	if (testfail == 0)
		PATH_XS_RPT(3 + 5 * npairs);
	
	(void) remove(t1_file);
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames */

	char	**p;
	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	testfail = 0;

	for (p = portfnam; *p != NULL; p++)
	{
		if ((fd = creat(*p, MODEANY)) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create file \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
		else if (p == portfnam)
			PATH_TRACE;
		(void) close(fd);
	}

#ifdef TESTBED2
	(void) unlink(portfnam[0]);
#endif

	/* duplicates will give ENOENT */

	for (p = portfnam; *p != NULL; p++)
		(void) tbunlink(*p, SUCCEED, NOERROR);

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* unlink() decrements link count by 1 */

	int	fd, err, oldcount;
	char	**p;
	int	pathok = setupok;
	int	loop1, loop2;
#if	  !TEST_XNFS
	int	loop3, loop4;
#endif	/* TEST_XNFS */
	int	pathcheck;

	DBUG_ENTER("test3");

	testfail = 0;

	if ((fd = creat(t3a_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t3a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	for (p = t3b_files, loop1 = 0; *p != NULL; p++, loop1++)
	{
		if (link(t3a_file, *p) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: link(\"%s\", \"%s\") failed - errno %d (%s)",
				t3a_file, *p, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else	
			PATH_TRACE;
	}

	if (stat(t3a_file, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t3a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	oldcount = stbuf1.st_nlink;

	for (p = t3b_files, loop2 = 0; *p != NULL; p++, loop2++)
	{
		if (tbunlink(*p, SUCCEED, NOERROR) != SUCCEED)
		{
			PATH_TRACE; /* to keep path count constant */
			PATH_TRACE;
			loop2 += 1;
			break;
		}

		if (stat(t3a_file, &stbuf1) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				t3a_file, err, errname(err));
			break;
		}
		else
			PATH_TRACE;
#ifdef TESTBED2
		oldcount--;
#endif
		if (stbuf1.st_nlink != oldcount - 1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unlink(\"%s\") did not decrement link count by 1",
				*p);
			in_rpt("old link count = %d, new count = %d",
				oldcount, (int)stbuf1.st_nlink);
		}
		else
			PATH_TRACE;
		oldcount = stbuf1.st_nlink;
	}

	(void) unlink(t3a_file);

#if	  !TEST_XNFS

	if (mkfifo(t3a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t3a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (p = t3b_files, loop3 = 0; *p != NULL; p++, loop3++)
	{
		if (link(t3a_file, *p) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: link(\"%s\", \"%s\") failed - errno %d (%s)",
				t3a_file, *p, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else 
			PATH_TRACE;
	}

	if (stat(t3a_file, &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\"%s\") failed - errno %d (%s)",
			t3a_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	oldcount = stbuf1.st_nlink;

	for (p = t3b_files, loop4 = 0; *p != NULL; p++, loop4++)
	{
		if (tbunlink(*p, SUCCEED, NOERROR) != SUCCEED)
		{
			PATH_TRACE;
			PATH_TRACE;
			loop4 += 2;
			break;
		}
		if (stat(t3a_file, &stbuf1) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				t3a_file, err, errname(err));
			break;
		}
		else
			PATH_TRACE;
#ifdef TESTBED2
		oldcount--;
#endif
		if (stbuf1.st_nlink != oldcount - 1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unlink(\"%s\") did not decrement link count by 1",
				*p);
			in_rpt("old link count = %d, new count = %d",
				oldcount, (int)stbuf1.st_nlink);
		}
		else
			PATH_TRACE;
		oldcount = stbuf1.st_nlink;
	}

	(void) unlink(t3a_file);

#endif	/* TEST_XNFS */

#if	   TEST_XNFS

	pathcheck = 1 + loop1 + 1 + 2 * loop2;

#else	/* TEST_XNFS */

	pathcheck = 1 + loop1 + 1 + 2 * loop2 + 1 + loop3 + 1 + 2 * loop4;

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(pathcheck);

	DBUG_VOID_RETURN;
}


private void
test4()
{
	/* open file is still accessible after unlink */

	int fd, err, rval;
#if	  !TEST_XNFS
	int fd2;
#endif	/* TEST_XNFS */
	static char wbuf[] = "The first 10 million years were the worst.\
			      The second 10 million, they were the worst too.\
			      After that I went into a bit of a decline.";
	char rbuf[sizeof(wbuf)];
	int	pathok = setupok;

	DBUG_ENTER("test4");

	testfail = 0;

	if ((fd = creat(t4_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, wbuf, sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() failed - errno %d (%s)", err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);
	if ((fd = open(t4_file, O_RDONLY)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY) failed - errno %d (%s)",
			t4_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tbunlink(t4_file, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	fd = -fd;
#endif

	if ((rval = read(fd, rbuf, sizeof(wbuf))) != sizeof(wbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on unlinked file returned %d, expected %d",
			rval, (int)sizeof(wbuf));
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (memcmp((void *)rbuf, (void *)wbuf, (size_t) sizeof(wbuf)) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from unlinked file returned incorrect data");
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	fd = -fd;
#endif

	(void) close(fd);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t4_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t4_file, O_RDONLY|O_NONBLOCK)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) failed - errno %d (%s)",
			t4_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd2 = open(t4_file, O_WRONLY|O_NONBLOCK)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_WRONLY|O_NONBLOCK) failed - errno %d (%s)",
			t4_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd2, wbuf, sizeof(wbuf)) != sizeof(wbuf))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() failed - errno %d (%s)", err, errname(err));
		(void) close(fd2);
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tbunlink(t4_file, SUCCEED, NOERROR) != SUCCEED)
	{
		(void) close(fd2);
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	fd = -fd;
#endif

	if ((rval = read(fd, rbuf, sizeof(wbuf))) != sizeof(wbuf))
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() on unlinked fifo returned %d, expected %d",
			rval, (int)sizeof(wbuf));
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (memcmp((void *)rbuf, (void *)wbuf, sizeof(wbuf)) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() from unlinked fifo returned incorrect data");
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	fd = -fd;
#endif

	(void) close(fd2);
	(void) close(fd);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(11);
	
	DBUG_VOID_RETURN;
}


private void
test5()
{
	DBUG_ENTER("test5");

#if	   TEST_XNFS

	xnfs_doroot_c(ch_t5, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest5();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch_t5()
{
	DBUG_ENTER("ch_t5");

	subtest5();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
subtest5()
{
	/* unlink a directory */

	int err;
	int	pathok = setupok;

	DBUG_ENTER("subtest5");

	testfail = 0;

	if (!unlink_dir_supp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_LINK_DIR_SUPP not set to 'Y' or 'U'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (link_dir_err)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_LINKDIR) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (link_dir_supp)
	{
		if (crdir(t5a_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t5a_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (link(t5a_file, t5b_file) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
				t5a_file, t5b_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		if (crdir(t5b_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t5b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

	}

	globok = 0;
	(void) tbunlink(t5b_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	(void) rmdir(t5a_file);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* unlink() updates timestamps */

	char	*chkt;
	int	fd, err;
	int	pathok = setupok;

	DBUG_ENTER("test6");

	testfail = 0;

	if ((fd = creat(t6a_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t6a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (link(t6a_file, t6b_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
			t6a_file, t6b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(".", &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\".\") failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	globok = 0;
	savtimes(t6a_file);
	PATH_FUNC_TRACE;

#ifndef TESTBED2
	globok = 0;
	if (tbunlink(t6b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	globok = 0;
	chkt = chktimes(t6a_file, NEWCTIME);
	PATH_FUNC_TRACE;

	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("remaining link %s", chkt);
	}
	else
		PATH_TRACE;

	if (stat(".", &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf2.st_ctime == stbuf1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	if (stbuf2.st_mtime == stbuf1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	(void) unlink(t6a_file);

#if	   TEST_XNFS

	PATH_TRACE; PATH_TRACE;
	PATH_TRACE; PATH_TRACE;
	PATH_TRACE; PATH_TRACE;
	PATH_TRACE; PATH_TRACE;
	PATH_TRACE; PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t6a_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t6a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (link(t6a_file, t6b_file) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
			t6a_file, t6b_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stat(".", &stbuf1) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(\".\") failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	globok = 0;
	savtimes(t6a_file);
	PATH_FUNC_TRACE;

#ifndef TESTBED2
	globok = 0;
	if (tbunlink(t6b_file, SUCCEED, NOERROR) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
#else
	PATH_TRACE;
#endif

	globok = 0;
	chkt = chktimes(t6a_file, NEWCTIME);
	PATH_FUNC_TRACE;
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("remaining link %s", chkt);
	}
	else
		PATH_TRACE;

	if (stat(".", &stbuf2) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\".\") failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (stbuf2.st_ctime == stbuf1.st_ctime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_ctime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	if (stbuf2.st_mtime == stbuf1.st_mtime)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("st_mtime field of parent directory was not updated");
	}
	else
		PATH_TRACE;

	(void) unlink(t6a_file);

#endif	/* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(20);

	DBUG_VOID_RETURN;
}


private void
test7()
{
	DBUG_ENTER("test7");

#if	   TEST_XNFS

	xnfs_doroot_c(ch1_t7, 4*WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest7();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch1_t7()
{
	DBUG_ENTER("ch1_t7");

	subtest7();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
subtest7()
{
	/* unlink a busy directory */

	char	*supp;
	int	ret, err;
	int	pathok = setupok;

	DBUG_ENTER("subtest7");

	testfail = 0;

	if (!unlink_dir_supp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_LINK_DIR_SUPP not set to 'Y' or 'U'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((supp = tet_getvar("VSX_REMOVE_DIR_EBUSY")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (*supp == 'Y' || *supp == 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY is set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (link_dir_err)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_LINKDIR) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (link_dir_supp)
	{
		if (crdir(t7a_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t7a_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		(void)unlink(t7b_file);

		if (link(t7a_file, t7b_file) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
				t7a_file, t7b_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		(void)unlink(t7c_file);

		if (link(t7a_file, t7c_file) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
				t7a_file, t7c_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		if (crdir(t7b_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t7b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (crdir(t7c_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t7c_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	/* directory being used by the system (mount point) */

	if (*supp != 'Y' && *supp != 'y' && *supp != 'S' && *supp != 's')
	{
		globok = 0;
		if (mountfs(t7b_file, (mode_t) MODEANY) != 0)
			testfail++;
		else
		{
			PATH_FUNC_TRACE;
			globok = 0;
			(void) tbunlink(t7b_file, SUCCEED, NOERROR);
			PATH_FUNC_TRACE;
			(void) umntfs(t7b_file);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	/* directory being used by another process */

	if (*supp != 'Y' && *supp != 'y' && *supp != 'P' && *supp != 'p')
	{
		ret = cppair(ch2_t7, pa_t7, 2*WAITTIME, E_CHDIR|E_OPEN|PATH_NOK);
		if (ret == PATH_NOK)
			ret = OK;
		else 
			PATH_TRACE;
		if (ret == E_CHDIR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: chdir(\"%s\") failed in child",
				t7c_file);
		}
		else if (ret == E_OPEN)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: open(\".\", O_RDONLY) failed in child");
		}
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(10);

	(void) rmdir(t7a_file);

	DBUG_VOID_RETURN;
}

private void
pa_t7()
{
	int	pathok = setupok;

	DBUG_ENTER("pa_t7");

	(void) sleep(WAITTIME/2);

	globok = 0;
	(void) tbunlink(t7c_file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch2_t7()
{
	int	pathok = setupok;

	DBUG_ENTER("ch2_t7");

#ifdef TESTBED2
	DBUG_EXIT(E_CHDIR);
#endif

	if (chdir(t7c_file) == SYSERROR)
	{
		DBUG_EXIT(E_CHDIR);
	}
	else
		PATH_TRACE;

	if (open(".", O_RDONLY) < 0)
	{
		DBUG_EXIT(E_OPEN);
	}
	else
		PATH_TRACE;
	
	(void) sleep(WAITTIME);

	if (pathok != 2)	/* report path check error */
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* write access to parent, search access to path prefix granted */
	int	pathok = setupok;

	DBUG_ENTER("test8");

	globok = 0;
	if (do_access('f', (mode_t) 0, (mode_t) S_IWUSR|S_IXUSR, t8_func, WAITTIME, SP_T8_DO_ACCESS) == 0)
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
	
	cln_access();

	DBUG_VOID_RETURN;
}

private int
t8_func(file, desc)
char *file, **desc;
{
	int	pathok = setupok;

	DBUG_ENTER("t8_func");

	testfail = 0;
	globok = 0;
	(void) tbunlink(file, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail != 0)
	{
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(testfail);
}

private void
test9()
{

	DBUG_ENTER("test9");

	(void) cppair(ch_t9, NULLFN, 4 * WAITTIME + 20, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int 	err;
	char	*args[6];
	extern	char	*vg_ident;
	extern	char	*vg_dbug;
	extern	char	**environ;

	DBUG_ENTER("ch_t9");

	args[0] = execpath(prog9);
	args[1] = t9_dir;
	args[2] = "unlink";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(prog9), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	/* search permission denied */
	int	ret;
	int	pathok = setupok;

	DBUG_ENTER("test10");

	testfail = 0;

	ret = cppair(ch_t10, NULLFN, WAITTIME, PATH_NOK | NOK);

	if(ret != OK && ret != PATH_NOK)
	{
		/* failure in child */
		DBUG_VOID_RETURN;
	}

	if(ret == OK)
		PATH_TRACE;

#if	   TEST_XNFS

	if (tet_sync(SP_T10_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbunlink(t10_file2, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (tet_sync(SP_T10_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if(testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	int	fd, err;
	mode_t	mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
	int	pathok = setupok;

	DBUG_ENTER("ch_t10");

	(void) setuid((uid_t)vguid1);
	if (getuid() != vguid1 || geteuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user id to VSX_UID1");
		DBUG_EXIT(NOK);;
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_LINKDIR) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_LINKDIR) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_ACCESS) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ACCESS) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_MOUNT) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (crdir(t10_dir, MODEANY, vguid1, (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t10_dir);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t10_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t10_file);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void) close(fd);
	if (chmod(t10_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t10_dir, (int)mode, err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tbunlink(t10_file, SYSERROR, EACCES) != SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_FUNC_TRACE;

	if (chmod(t10_dir, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: chmod(\"%s\", %#o) failed - errno %d (%s)",
			t10_dir, (int)MODEANY, err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	(void) unlink(t10_file);
#endif
	if (eaccess(t10_file, F_OK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to unlink() removed link");
	}
	else
		PATH_TRACE;

	(void) unlink(t10_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t10_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t10_file);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (chmod(t10_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t10_dir, (int)mode, err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tbunlink(t10_file, SYSERROR, EACCES) != SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_FUNC_TRACE;

	if (chmod(t10_dir, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: chmod(\"%s\", %#o) failed - errno %d (%s)",
			t10_dir, (int)MODEANY, err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	(void) unlink(t10_file);
#endif
	if (eaccess(t10_file, F_OK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to unlink() removed link");
	}
	else
		PATH_TRACE;

	(void) unlink(t10_file);
	(void) rmdir(t10_dir);

#endif	/* TEST_XNFS */

	if (testfail == 0)
	{
		if(pathok == 16)
		{
			DBUG_EXIT(OK);;
		}
		else
		{
			DBUG_EXIT(PATH_NOK);;
		}
	}

	DBUG_EXIT(NOK);;
}

private void
test11()
{
	/* remove link in directory which denies write permission */
	int	ret;
	int	pathok = setupok;

	DBUG_ENTER("test11");

	testfail = 0;

	ret = cppair(ch_t11, NULLFN, WAITTIME, PATH_NOK | NOK);

	if(ret != OK && ret != PATH_NOK)
	{
		/* failure in child */
		DBUG_VOID_RETURN;
	}

	if(ret == OK)
		PATH_TRACE;

#if	   TEST_XNFS

	if (tet_sync(SP_T11_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbunlink(t11_file2, SYSERROR, EACCES);
	PATH_FUNC_TRACE;

	if (tet_sync(SP_T11_RMDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to RMDIR2_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else	/* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif	/* TEST_XNFS */

	if(testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	int	fd, err;
	mode_t	mode = (S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	int	pathok = setupok;

	DBUG_ENTER("ch_t11");

	(void) setuid((uid_t)vguid1);
	if (getuid() != vguid1 || geteuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user id to VSX_UID1");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_LINKDIR) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_LINKDIR) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_ACCESS) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ACCESS) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_MOUNT) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (crdir(t11_dir, MODEANY, vguid1, (int) getegid()) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t11_dir);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if ((fd = crfile(t11_file, MODEANY,
			(int) geteuid(), (int) getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t11_file);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (chmod(t11_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t11_dir, (int)mode, err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tbunlink(t11_file, SYSERROR, EACCES) != SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	(void) chmod(t11_dir, MODEANY);
	(void) unlink(t11_file);
#endif
	if (eaccess(t11_file, F_OK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to unlink() removed link");
	}
	else
		PATH_TRACE;

	(void) chmod(t11_dir, MODEANY);
	(void) unlink(t11_file);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t11_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t11_file);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (chmod(t11_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t11_dir, (int)mode, err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	globok = 0;
	if (tbunlink(t11_file, SYSERROR, EACCES) != SYSERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_FUNC_TRACE;

#ifdef TESTBED2
	(void) chmod(t11_dir, MODEANY);
	(void) unlink(t11_file);
#endif
	if (eaccess(t11_file, F_OK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to unlink() removed link");
	}
	else
		PATH_TRACE;

	(void) chmod(t11_dir, MODEANY);
	(void) unlink(t11_file);

#endif	/* TEST_XNFS */

	(void) rmdir(t11_dir);

	if (testfail == 0)
	{
		if(pathok == 14)
		{
			DBUG_EXIT(OK);;
		}
		else
		{
			DBUG_EXIT(PATH_NOK);;
		}
	}

	DBUG_EXIT(NOK);
}


private void
test12()
{
	DBUG_ENTER("test12");

#if	   TEST_XNFS

	xnfs_doroot_c(ch1_t12, 4*WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else	/* TEST_XNFS */

	subtest12();

#endif	/* TEST_XNFS */

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch1_t12()
{
	DBUG_ENTER("ch1_t12");

	subtest12();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
subtest12()
{
	/* unlink a busy directory */

	char	*supp;
	int	ret, err;
	int	pathok = setupok;

	DBUG_ENTER("subtest12");

	testfail = 0;

	if (!unlink_dir_supp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_LINK_DIR_SUPP not set to 'Y' or 'U'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((supp = tet_getvar("VSX_REMOVE_DIR_EBUSY")) == NULL ||
		*supp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (*supp == 'N' || *supp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_REMOVE_DIR_EBUSY is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (link_dir_err)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_LINKDIR) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (link_dir_supp)
	{
		if (crdir(t12a_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t12a_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (link(t12a_file, t12b_file) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
				t12a_file, t12b_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (link(t12a_file, t12c_file) == SYSERROR)
		{
			err = errno;
			if(err == EEXIST)
			{
				PATH_TRACE;
			}
			else
			{
				xx_rpt(DELETION);
				in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
					t12a_file, t12c_file, err, errname(err));
				DBUG_VOID_RETURN;
			}
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		if (crdir(t12b_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t12b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (crdir(t12c_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t12c_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}


	/* directory being used by the system (mount point) */

	if (*supp == 'Y' || *supp == 'y' || *supp == 'S' || *supp == 's')
	{
		globok = 0;
		if (mountfs(t12b_file, (mode_t) MODEANY) != 0)
			testfail++;
		else
		{
			PATH_FUNC_TRACE;
			globok = 0;
			if (tbunlink(t12b_file, SYSERROR, EBUSY) == SYSERROR)
			{
				PATH_FUNC_TRACE;
				if (eaccess(t12b_file, F_OK) != 0)
				{
					if (testfail++ == 0)
						xx_rpt(FAILURE);
					in_rpt("unsuccessful call to unlink() removed link");
				}
				else
					PATH_TRACE;
			}
			(void) umntfs(t12b_file);
		}
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}
	/* directory being used by another process */

	if (*supp == 'Y' || *supp == 'y' || *supp == 'P' || *supp == 'p')
	{
		ret = cppair(ch2_t12, pa_t12, 2*WAITTIME, E_CHDIR|E_OPEN|PATH_NOK);
		if (ret == PATH_NOK)
			ret = OK;
		else
			PATH_TRACE;
		if (ret == E_CHDIR)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: chdir(\"%s\") failed in child",
				t12c_file);
		}
		else if (ret == E_OPEN)
		{
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: open(\".\", O_RDONLY) failed in child");
		}
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(11);

	(void) unlink(t12b_file);
	(void) unlink(t12c_file);
	(void) rmdir(t12a_file);

	DBUG_VOID_RETURN;
}

private void
pa_t12()
{
	int	pathok = setupok;

	DBUG_ENTER("pa_t12");

	(void) sleep(WAITTIME/2);

	globok = 0;
	if (tbunlink(t12c_file, SYSERROR, EBUSY) != SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (eaccess(t12c_file, F_OK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to unlink() removed link");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch2_t12()
{
	int	pathok = setupok;

	DBUG_ENTER("ch2_t12");

#ifdef TESTBED2
	DBUG_EXIT(E_CHDIR);
#endif

	if (chdir(t12c_file) == SYSERROR)
	{
		DBUG_EXIT(E_CHDIR);
	}
	else
		PATH_TRACE;

	if (open(".", O_RDONLY) < 0)
	{
		DBUG_EXIT(E_OPEN);
	}
	else
		PATH_TRACE;
	
	(void) sleep(WAITTIME);

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test13()
{
	int	pathok = setupok;
	/* path component longer than NAME_MAX */

	char	**longname;
	int	fd;

	DBUG_ENTER("test13");

	testfail = 0;

	globok = 0;
	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = crfile(longname[0], MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		PATH_TRACE;
		(void) close(fd);

		globok = 0;
		(void) tbunlink(longname[0], SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (longname[1] != NULL)
	{
		globok = 0;
		(void) tbunlink(longname[1], SYSERROR, ENAMETOOLONG);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}
#endif	/* TEST_XNFS */



private void
test14()
{
	int	pathok = setupok;
	/* path longer than PATH_MAX */

	char	**longname;
	int	fd;

	DBUG_ENTER("test14");

	testfail = 0;

	if ((fd = creat(t14_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	longname = pathtoolng(t14_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbunlink(longname[0], SUCCEED, NOERROR);
	PATH_FUNC_TRACE;

	if ((fd = creat(t14_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t14_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	if (longname[1] != NULL &&
		tbunlink(longname[1], SYSERROR, ENAMETOOLONG) == SYSERROR)
	{
		PATH_FUNC_TRACE;
#ifdef TESTBED2
		(void) unlink(t14_file);
#endif
		if (eaccess(t14_file, F_OK) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to unlink() removed link");
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(6);

	clntoolng();
	(void) unlink(t14_file);

	DBUG_VOID_RETURN;
}

private void
test15()
{
	int	pathok = setupok;
	/* component of path prefix does not exist */

	DBUG_ENTER("test15");

	testfail = 0;

	if (eaccess(t15_dir, F_OK) == 0 && remove(t15_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t15_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbunlink(t15_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	int	pathok = setupok;
	/* unlink file which does not exist */

	DBUG_ENTER("test16");

	testfail = 0;

	if (eaccess(t16_file, F_OK) == 0 && remove(t16_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t16_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbunlink(t16_file, SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test17()
{
	/* path is "" */

	DBUG_ENTER("test17");

	testfail = 0;

	(void) cppair(ch_t17, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t17()
{
	int	pathok = setupok;

	DBUG_ENTER("ch_t17");

	/* Do it without permission to unlink directories, just in case */

	(void) setuid((uid_t)vguid1);
	if (getuid() != vguid1 || geteuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user id to VSX_UID1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_LINKDIR) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_LINKDIR) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_ACCESS) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ACCESS) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_MOUNT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbunlink("", SYSERROR, ENOENT);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test18()
{
	/* component of path is not a directory */

	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test18");

	testfail = 0;

	if ((fd = creat(t18_ndir, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t18_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbunlink(t18_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	(void) unlink(t18_ndir);

#if	   TEST_XNFS

	PATH_TRACE;
	PATH_TRACE;

#else	/* TEST_XNFS */

	if (mkfifo(t18_ndir, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t18_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbunlink(t18_file, SYSERROR, ENOTDIR);
	PATH_FUNC_TRACE;

	(void) unlink(t18_ndir);

#endif	/* TEST_XNFS */
	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
test19()
{
	DBUG_ENTER("test19");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test19()
{
	/* unlink directory without appropriate privilege */
	/* OR unlink directory when not supported */

	int err;
	int	pathok = setupok;

	DBUG_ENTER("test19");

	testfail = 0;

	if (link_dir_supp)
	{
		if (crdir(t19a_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t19a_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		(void) unlink(t19b_file);

		if (link(t19a_file, t19b_file) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
				t19a_file, t19b_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		if (crdir(t19b_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t19b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	PATH_FUNC_RPT(2);	/* pass path check okay to child */

	(void) cppair(ch_t19, NULLFN, WAITTIME, 0);

	if (link_dir_supp)
	{
		(void) unlink(t19b_file);
		(void) rmdir(t19a_file);
	}
	else
		(void) rmdir(t19b_file);

	DBUG_VOID_RETURN;
}

private void
ch_t19()
{
	int	pathok = setupok;

	DBUG_ENTER("ch_t19");

	PATH_FUNC_TRACE;

	if (unlink_dir_supp)
	{
		(void) setuid((uid_t)vguid1);
		if (getuid() != vguid1 || geteuid() != vguid1)
		{
			xx_rpt(DELETION);
			in_rpt("setuid() did not set effective user id to VSX_UID1");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_LINKDIR) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_LINKDIR) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_SETID) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_SETID) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_ACCESS) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_ACCESS) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_MOUNT) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_MOUNT) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	globok = 0;
	if (tbunlink(t19b_file, SYSERROR, EPERM) == SYSERROR)
	{
		PATH_FUNC_TRACE;
		if (eaccess(t19b_file, F_OK) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to unlink() removed link");
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private void
test20()
{
	/* remove link on read only file system */
	char	*chkt;
	int	pathok = setupok;

	DBUG_ENTER("test20");

	testfail = 0;

	globok = 0;
	if (setuprofs(t20_dir, t20_file, 'f', (mode_t) MODEANY) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(t20_dir);
	PATH_FUNC_TRACE;

	globok = 0;
	if (tbunlink(t20_file, SYSERROR, EROFS) == SYSERROR)
	{
		PATH_FUNC_TRACE;
		if (eaccess(t20_file, F_OK) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to unlink() removed link");
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
	{
		globok = 0;
		chkt = chktimes(t20_dir, 0);
		PATH_FUNC_TRACE;
		if (chkt != NULL)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("parent directory %s", chkt);
		}
		else
			PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) clnuprofs(t20_dir, t20_file);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
test21()
{
	char	*longname;
	int	pathok = setupok;

	DBUG_ENTER("test21");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbunlink(longname, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test21()
{
	DBUG_ENTER("test21");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test22()
{
	char	*longname;
	int	pathok = setupok;

	DBUG_ENTER("test22");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbunlink(longname, SYSERROR, ENAMETOOLONG);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test22()
{
	DBUG_ENTER("test22");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if TEST_XNFS

private void
test23()
{
	char	*longname;
	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test23");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = creat(longname, (mode_t)MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbunlink(longname, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test23()
{
	DBUG_ENTER("test23");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test24()
{
	char	*longname;
	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test24");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = creat(longname, (mode_t)MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		globok = 0;
		(void) tbunlink(longname, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	if (fd >= 0)
		(void) close(fd);
	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test24()
{
	DBUG_ENTER("test24");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


#if	   TEST_XNFS

private void
test25()
{
	DBUG_ENTER("test25");

	/* 
	 * Some parts of this test require "appropriate privileges"
	 * which would be lost by the anon mapping
	 *
	 */

	xnfs_doroot_c(ch1_t25, WAITTIME, SP_NOSYNC, SP_NOSYNC);

	DBUG_VOID_RETURN;
}


#if	   TEST_XNFS

private void
ch1_t25()
{
	DBUG_ENTER("ch1_t25");

	subtest25();

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
subtest25()
{
	/* unlink directory without appropriate privilege */
	/* OR unlink directory when not supported */

	int err;
	int	pathok = setupok;

	DBUG_ENTER("subtest25");

	testfail = 0;

	if (link_dir_supp)
	{
		if (crdir(t25a_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t25a_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		(void) unlink(t25b_file);

		if (link(t25a_file, t25b_file) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("link(\"%s\", \"%s\") failed - errno %d (%s)",
				t25a_file, t25b_file, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		if (crdir(t25b_file, MODEANY,
				(int) geteuid(), (int) getegid()) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create directory \"%s\"", t25b_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	PATH_FUNC_RPT(2);	/* pass path check okay to child */

	(void) cppair(ch2_t25, NULLFN, WAITTIME, 0);

	if (link_dir_supp)
	{
		(void) unlink(t25b_file);
		(void) rmdir(t25a_file);
	}
	else
		(void) rmdir(t25b_file);

	DBUG_VOID_RETURN;
}

private void
ch2_t25()
{
	int	pathok = setupok;

	DBUG_ENTER("ch2_t25");

	PATH_FUNC_TRACE;

	if (unlink_dir_supp)
	{
		(void) setuid((uid_t)vguid1);
		if (getuid() != vguid1 || geteuid() != vguid1)
		{
			xx_rpt(DELETION);
			in_rpt("setuid() did not set effective user id to VSX_UID1");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_LINKDIR) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_LINKDIR) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_SETID) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_SETID) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_ACCESS) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_ACCESS) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (unsetprv(PRV_MOUNT) != 0)
		{
			xx_rpt(DELETION);
			in_rpt("unsetprv(PRV_MOUNT) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
		PATH_TRACE;
	}

	globok = 0;
	if (tbunlink(t25b_file, SYSERROR, EPERM) == SYSERROR)
	{
		PATH_FUNC_TRACE;
		if (eaccess(t25b_file, F_OK) != 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to unlink() removed link");
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test25()
{
	DBUG_ENTER("test25");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#endif	/* TEST_XNFS */


#if	   TEST_XNFS

private void
test26()
{
	int	pathok = setupok;

	DBUG_ENTER("test26");

	testfail = 0;

	if (!unlink_dir_supp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_LINK_DIR_SUPP not set to 'Y' or 'U'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (vxmkdir(t26_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t26_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	globok = 0;
	(void) tbunlink(t26_file, SYSERROR, EPERM);
	PATH_FUNC_TRACE;

	if (eaccess(t26_file, F_OK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("unsuccessful call to unlink() removed link");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) rmdir(t26_file);

	DBUG_VOID_RETURN;
}

#else	/* TEST_XNFS */

private void
test26()
{
	DBUG_ENTER("test26");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}


#endif	/* TEST_XNFS */


private int
tbunlink(tbpath, expret, experrno)
char *tbpath;
int  expret, experrno;
{
	char	*reppath = tbpath;
	int	rval, obserr;
	char	pathbuf[30];
	int	pathok = setupok;

	DBUG_ENTER("tbunlink");

	errno = 0;
	rval = unlink(tbpath);
	globok = 0;

#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	obserr = errno;

	if (expret != SYSERROR && rval == expret)
	{
		if (eaccess(tbpath, F_OK) == 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("successful call to unlink() did not remove link");
		}
		else
			PATH_TRACE;
		PATH_FUNC_RPT(1);
		DBUG_RETURN(rval);
	}
	else if (expret == SYSERROR && rval == expret && experrno == obserr)
	{
		PATH_FUNC_RPT(0);
		DBUG_RETURN(rval);
	}
	else
		PATH_TRACE;

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("unlink(\"%s\") did not give correct results", reppath);
	valrpt(expret, rval, experrno, obserr);
	
	PATH_FUNC_RPT(1);
	DBUG_RETURN(rval);
}
