/*
*      SCCS:  @(#)  POSIX.os/files/utime/utime.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/files/utime - T.utime Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.utime

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the utime() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	17 Apr 1989
MODIFICATIONS:
		Geoff Clare, 9 June 1989:  Add do_access() test.

		Geoff Clare, 15 Nov 1989
			New setuprofs() argument
			New do_access() argument
		
		Geoff Clare, 5 Dec 1989
			Call utime() direct in do_access() test, as tbutime()
			checks extra stuff not relevant to the assertion.

		Geoff Clare, 4 Apr 1990
			New interface with pathtoolng() and nametoolng().

		Peter Lee, Unisoft Ltd.,
		Mon Jul 30 10:31:49 BST 1990
		Added resilience code.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Stuart Boutell, 16 Apr 1991
			"other structure members have no effect" code
			added to tests 5 and 7

		Peter Skan, 10 Sept 1991
			Remove PRV_ACCESS in normsetup()

		Peter Skan, 9 Oct 1991
			Remove other assigned privs as well

		Geoff Clare, 9 Jan 1992
			Allow for read-only root file systems in test1.

                John-Paul, 29 July 1992
                        Mode parameter in calls to crfile() and crdir()
                        changed from 'int' to 'mode_t' type.

		Neil Moses, 13 Nov 1992
			XNFS enhancements

		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.

		Geoff Clare, 4 Nov 1993
			Don't use xnfs_doroot_c() in test9.

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

		Geoff Clare, 3 Nov 1995
			Changed child timeout for do_access() test.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<time.h>
#include	<string.h>
#include	<utime.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<pathres.h>
#include	<chktimes.h>
#include <tet_api.h>
#include 	<xnfs.h>
#include 	"utime_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef utime
#endif


#define NO_TESTS	29
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define NOK		02
#define NULLUTB		(struct utimbuf *)0
#define NOCHK		((time_t)-1)

#undef memset
extern	void	*memset();
extern	char	*errname();
#if TEST_XNFS
extern   char    *xnfs_nametoolng();
#else
extern	char	**nametoolng();
#endif
extern	char	**pathtoolng();
extern	void	clntoolng();
extern	void	cln_access();
extern	void	valrpt();

extern	int	vguid1, vggid1, vguid0, vggid0;

public	int	no_tests = NO_TESTS;

#if TEST_XNFS
private	int	snames[] = { 1, 0 };
#endif /* TEXT_XNFS */

private int		testfail;
private int		privsupp;
private int		privok;
private struct utimbuf	utimb;
private int		setupok;

private	char	*t3_file =	"utime-t.3";
private	char	*t4a_file =	"utime-t.4a";
private	char	*t4b_file =	"utime-t.4b";
private	char	*t5_file =	"utime-t.5";
private	char	*t6a_file =	"utime-t.6a";
private	char	*t6b_file =	"utime-t.6b";
private	char	*t7a_file =	"utime-t.7a";
private	char	*t7b_file =	"utime-t.7b";
private	char	*t8_file =	"utime-t.8";
private char	*t10_dir =	"utime-d.10";
private char	*t10_file =	"utime-d.10/utime-t.10";
#if TEST_XNFS
private	char	*t10_file2 =	T10_FILE2;
#endif
private	char	*t11_file =	"utime-t.11";
private	char	*t13_file =	"utime-t.13";
private char	*t14_dir =	"utime-d.14";
private char	*t14_file =	"utime-d.14/utime-t.14";
private	char	*t15_file =	"utime-t.15";
private char	*t17_ndir =	"utime-t.17";
private char	*t17_file =	"utime-t.17/utime-t.17";
private	char	*t18_file =	"utime-t.18";
private char	*t19_dir =	"utime-d.19";
private char	*t19_file =	"utime-d.19/utime-t.19";
#if TEST_XNFS
private	char	*t21_file =	T21_FILE;
private	char	*t27_file =	T27_FILE;
private	char	*t28_file =	"utime-t.28";
private	char	*t29_file =	T29_FILE;
#endif

private void	prepare_tests();
private void	cln_tests();

private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();

private void	ch_t3();
private void	ch_t4();
#if TEST_XNFS
private void	gch_t4();
#endif
private void	ch_t5();
private void	ch_t6();
private void	do_t6();
private void	ch_t7();
private void	do_t7();
private void	ch_t9();
private void	ch_t10();
private void	ch_t11_20();
private void	ch_t18_26();
#if TEST_XNFS
private void	ch_t21();
private void	ch_t27();
#endif
private int	t9_func();

private int	tbutime();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	 test29, 29,
	NULL, 0
};


private void
prepare_tests()
{
	char	*supp;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	if (setprv(PRV_SETID) != 0)
	{
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, "setprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	supp = tet_getvar("VSX_PRIV_ACCESS_SUPP");
	privsupp = (supp != NULL && (*supp == 'Y' || *supp == 'y'));
	if (privsupp)
		privok = (setprv(PRV_ACCESS) == 0);

	if(uids()!=0)
	{
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
	}
	else
		PATH_TRACE;

#if TEST_XNFS
	if (xnfs_uids() < 0)
	{
		tet_delete(20, "xnfs_uids() failed");
		tet_delete(21, "xnfs_uids() failed");
		tet_delete(26, "xnfs_uids() failed");
		tet_delete(27, "xnfs_uids() failed");
		tet_delete(28, "xnfs_uids() failed");
		tet_delete(29, "xnfs_uids() failed");
	}
	else
		PATH_TRACE;
#else
	PATH_TRACE;
#endif

	(void) umask((mode_t)0);

	PATH_SETUP_RPT(3);

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	char **p;

	DBUG_ENTER("cln_tests");

	for (p = portfnam; *p != NULL; p++)
		(void) unlink(*p);
	(void) unlink(t3_file);
	(void) unlink(t4a_file);
	(void) unlink(t4b_file);
	(void) unlink(t5_file);
	(void) unlink(t6a_file);
	(void) unlink(t6b_file);
	(void) unlink(t7a_file);
	(void) unlink(t7b_file);
	(void) unlink(t8_file);
	(void) chmod(t10_dir, MODEANY);
	(void) unlink(t10_file);
	(void) rmdir(t10_dir);
	(void) unlink(t11_file);
	(void) unlink(t13_file);
	(void) unlink(t17_ndir);
	(void) unlink(t18_file);
#if TEST_XNFS
	(void)unlink(t21_file);
	(void)unlink(t27_file);
	(void)unlink(t28_file);
	(void)unlink(t29_file);
#endif

	DBUG_VOID_RETURN;
}

private void
normsetup()
{
	int	pathok = 0;
	DBUG_ENTER("normsetup");

	/* become a user with no appropriate privileges */

	(void) setgid((gid_t)vggid1);
	(void) setuid((uid_t)vguid1);
	if (geteuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("setuid() did not set effective user ID to VSX_UID1");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	if (getegid() != vggid1)
	{
		xx_rpt(DELETION);
		in_rpt("setgid() did not set effective group ID to VSX_GID1");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_ACCESS) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_ACCESS) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_SETID) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_MOUNT) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("unsetprv(PRV_MOUNT) failed");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	DBUG_VOID_RETURN;
}


private void
test1()
{
	/* pathname resolution */

	char	*p1, *p2;
	int	npairs, i, err, experr;
	struct stat stbuf;
	int	pathok = setupok;

	DBUG_ENTER("test1");

	testfail = 0;

	/* Only include the root dir. if privilege is supported */
	if (privsupp)
	{
		if (!privok)
		{
			xx_rpt(DELETION);
			in_rpt("setprv(PRV_ACCESS) failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		npairs = init_pathres(PRES_DIRS|PRES_CREAT);
	}
	else
	{
		PATH_TRACE;
		npairs = init_pathres(PRES_DIRS|PRES_NOROOT|PRES_CREAT);
	}

	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		if (privsupp)
			in_rpt("init_pathres(PRES_DIRS|PRES_CREAT) failed");
		else
			in_rpt("init_pathres(PRES_DIRS|PRES_NOROOT|PRES_CREAT) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test1", "testing \"%s\" == \"%s\"", p1, p2);

		if (stat(p2, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				p2, err, errname(err));
			continue;
		}
		else
			PATH_TRACE;

		utimb.actime = stbuf.st_atime;
		utimb.modtime = stbuf.st_mtime + 1;
		if (strcmp(p1, "/") == 0 || strcmp(p2, "/") == 0)
			experr = EROFS;
		else
			experr = NOERROR;

#ifndef TESTBED2
		if (tbutime(p2, &utimb, SUCCEED, experr,
				NOCHK, utimb.modtime) != SUCCEED)
		{
			PATH_TRACE;
			continue;
		}
#endif

		utimb.actime = stbuf.st_atime;
		utimb.modtime = stbuf.st_mtime + 2;

#ifndef TESTBED2
		if (tbutime(p1, &utimb, SUCCEED, NOERROR,
				NOCHK, utimb.modtime) != SUCCEED)
		{
			PATH_TRACE;
			continue;
		}
#endif

		/* tbutime() checks p1, but we want p2 */

		if (stat(p2, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				p2, err, errname(err));
		}
		else if (stbuf.st_mtime != utimb.modtime)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("utime() on file \"%s\" did not change \"%s\" as expected",
				p1, p2);
		}
		else
			PATH_TRACE;

	}

	if (testfail == 0)
		PATH_XS_RPT(2 + 2 * npairs);
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* portable filenames */

	char	**p;
	time_t	t0;
	int	i, fd, err;
	struct stat stbuf;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	testfail = 0;

	/* Use a different time for each file */

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		if ((fd = creat(*p, MODEANY)) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("could not create file \"%s\"", *p);
			DBUG_VOID_RETURN;
		}
		else if (i == 0)
			PATH_TRACE;
		(void) close(fd);
	}

	(void) sleep(2);
	(void) time(&t0);

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		utimb.actime = t0;
		utimb.modtime = t0 + i;
		(void) tbutime(*p, &utimb, SUCCEED, NOERROR, NOCHK, NOCHK);
		if (i == 0)
			PATH_TRACE;
	}

	if (testfail > 0)
		DBUG_VOID_RETURN;

#ifdef TESTBED2
	t0++;
#endif

	/* undistinguished files will have wrong times */

	for (i = 0, p = portfnam; *p != NULL; p++, i++)
	{
		if (stat(*p, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				*p, err, errname(err));
		}
		else if (stbuf.st_mtime != t0 + i)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("incorrect st_mtime on file \"%s\": expected %ld, observed %ld",
				*p, (long)(t0 + i), (long)stbuf.st_mtime);
		}
		else if (i == 0)
			PATH_TRACE;

		(void) unlink(*p);
	}

	if (testfail == 0)
		PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* times=NULL sets current time when euid matches owner of file */

	DBUG_ENTER("test3");

	testfail = 0;

	(void) cppair(ch_t3, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	time_t	t0;
	int	fd;
	int	pathok = setupok;
	
	DBUG_ENTER("ch_t3");

	normsetup();

	/* Create file with no access permissions */

	if ((fd = crfile(t3_file, (mode_t)0,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);
	
	(void) sleep(2);

#if TEST_XNFS
	
	if (xnfs_rtime_c(&t0, SP_T3_RTIME) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */

	(void) time(&t0);
	PATH_TRACE;

#endif /* TEST_XNFS */

	globok = 0;
	(void) tbutime(t3_file, NULLUTB, SUCCEED, NOERROR, t0, t0);
	PATH_FUNC_TRACE;

	(void) unlink(t3_file);

#if !TEST_XNFS

	if (mkfifo(t3_file, (mode_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t3_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	(void) sleep(2);
	(void) time(&t0);

	globok = 0;
	(void) tbutime(t3_file, NULLUTB, SUCCEED, NOERROR, t0, t0);
	PATH_FUNC_TRACE;

	(void) unlink(t3_file);

#else /* TEST_XNFS */

	PATH_TRACE;
	PATH_TRACE;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* times=NULL sets current time when write permission granted */

	int	fd;
	mode_t	mode = (mode_t)(S_IWUSR|S_IWGRP|S_IWOTH);
	int	pathok = setupok;

	DBUG_ENTER("test4");

	testfail = 0;

	if ((fd = creat(t4a_file, mode)) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t4a_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

#if !TEST_XNFS
	if (mkfifo(t4b_file, mode) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t4b_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */

	PATH_TRACE;

#endif /* TEST_XNFS */

	PATH_FUNC_RPT(2);

	(void) cppair(ch_t4, NULLFN, WAITTIME, NOK);

	(void) unlink(t4a_file);
#if !TEST_XNFS
	(void) unlink(t4b_file);
#endif /* !TEST_XNFS */

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	time_t t0;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	PATH_FUNC_TRACE;

	normsetup();

	(void) sleep(2);

#if TEST_XNFS
	
	if (xnfs_rtime_c(&t0, SP_T4_RTIME) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */

	(void) time(&t0);
	PATH_TRACE;

#endif /* TEST_XNFS */

	globok = 0;
	(void) tbutime(t4a_file, NULLUTB, SUCCEED, NOERROR, t0, t0);
	PATH_FUNC_TRACE;

#if !TEST_XNFS

	(void) sleep(2);
	(void) time(&t0);

	globok = 0;
	(void) tbutime(t4b_file, NULLUTB, SUCCEED, NOERROR, t0, t0);
	PATH_FUNC_TRACE;

#else /* TEST_XNFS */

	PATH_TRACE;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* set specified times when euid matches owner of file */

	DBUG_ENTER("test5");

	testfail = 0;

	(void) cppair(ch_t5, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	time_t	t0;
	int	fd;
	int	pathok = setupok;
	
	DBUG_ENTER("ch_t5");

	normsetup();

	/* Create file with no access permissions */

	if ((fd = crfile(t5_file, (mode_t)0,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	(void) memset((void *)&utimb, 'S', sizeof(utimb)); /* Fil utimb */
	(void) time(&t0);
	utimb.actime = t0 + 10;
	utimb.modtime = 0;

	globok = 0;
	(void) tbutime(t5_file, &utimb,
			SUCCEED, NOERROR, utimb.actime, utimb.modtime);
	PATH_FUNC_TRACE;

	(void) unlink(t5_file);

#if !TEST_XNFS

	if (mkfifo(t5_file, (mode_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) memset((void *)&utimb, 'S', sizeof(utimb)); /* Fil utimb */
	(void) time(&t0);
	utimb.actime = t0 + 10;
	utimb.modtime = 0;

	globok = 0;
	(void) tbutime(t5_file, &utimb,
			SUCCEED, NOERROR, utimb.actime, utimb.modtime);
	PATH_FUNC_TRACE;

	(void) unlink(t5_file);

#else /* TEST_XNFS */
	PATH_TRACE;
	PATH_TRACE;
#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	/* times=NULL sets current time when privileged */

	DBUG_ENTER("test6");

#if TEST_XNFS

	xnfs_doroot_c(do_t6, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else /* TEST_XNFS */

	do_t6();

#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

private void
do_t6()
{
	int	pathok = setupok;
	time_t t0;

	DBUG_ENTER("do_t6");

	testfail = 0;

	if (!privsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_PRIV_ACCESS_SUPP not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!privok)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_ACCESS) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Create files owned by "other" */
	if (cppair(ch_t6, NULLFN, WAITTIME, NOK) == NOK)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(2);

#if TEST_XNFS
	
	if (xnfs_rtime_c(&t0, SP_T6_RTIME) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */

	(void) time(&t0);
	PATH_TRACE;

#endif /* TEST_XNFS */

	globok = 0;
	(void) tbutime(t6a_file, NULLUTB, SUCCEED, NOERROR, t0, t0);
	PATH_FUNC_TRACE;
	(void) unlink(t6a_file);

#if !TEST_XNFS

	(void) sleep(2);
	(void) time(&t0);

	globok = 0;
	(void) tbutime(t6b_file, NULLUTB, SUCCEED, NOERROR, t0, t0);
	PATH_FUNC_TRACE;
	(void) unlink(t6b_file);

#else /* TEST_XNFS */

	PATH_TRACE;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	int	fd;
	
	DBUG_ENTER("ch_t6");

	normsetup();

	/* Create file with no access permissions */

	if ((fd = crfile(t6a_file, (mode_t)0,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t6a_file);
		DBUG_EXIT(NOK);
	}
	(void) close(fd);
	
#if !TEST_XNFS
	if (mkfifo(t6b_file, (mode_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t6b_file);
		DBUG_EXIT(NOK);
	}
#endif

	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* set specified times when privileged */

	DBUG_ENTER("test7");

#if TEST_XNFS

	xnfs_doroot_c(do_t7, WAITTIME, SP_NOSYNC, SP_NOSYNC);

#else /* TEST_XNFS */

	do_t7();

#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

private void
do_t7()
{

	time_t t0;
	int	pathok = setupok;

	DBUG_ENTER("do_t7");

	testfail = 0;

	if (!privsupp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_PRIV_ACCESS_SUPP not set to 'Y'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!privok)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_ACCESS) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Create file owned by "other" */
	if (cppair(ch_t7, NULLFN, WAITTIME, NOK) == NOK)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) memset((void *)&utimb, 'S', sizeof(utimb)); /* Fil utimb */
	(void) time(&t0);
	utimb.actime = 0;
	utimb.modtime = t0 + 10;
	globok = 0;
	(void) tbutime(t7a_file, &utimb,
			SUCCEED, NOERROR, utimb.actime, utimb.modtime);
	PATH_FUNC_TRACE;

	(void) unlink(t7a_file);

#if !TEST_XNFS

	(void) memset((void *)&utimb, 'S', sizeof(utimb)); /* Fil utimb */
	(void) time(&t0);
	utimb.actime = 0;
	utimb.modtime = t0 + 10;
	globok = 0;
	(void) tbutime(t7b_file, &utimb,
			SUCCEED, NOERROR, utimb.actime, utimb.modtime);
	PATH_FUNC_TRACE;

	(void) unlink(t7b_file);

#else /* TEST_XNFS */

	PATH_TRACE;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int	fd;
	
	DBUG_ENTER("ch_t7");

	normsetup();

	/* Create file with no access permissions */

	if ((fd = crfile(t7a_file, (mode_t)0,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t7a_file);
		DBUG_EXIT(NOK);
	}
	(void) close(fd);
	
#if !TEST_XNFS
	if (mkfifo(t7b_file, (mode_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t7b_file);
		DBUG_EXIT(NOK);
	}
#endif /* TEST_XNFS */
	
	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* utime updates timestamps */

	char	*chkt;
	int	fd, err;
	struct stat stbuf;
	int	pathok = setupok;

	DBUG_ENTER("test8");

	testfail = 0;

	if ((fd = creat(t8_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (stat(t8_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
			t8_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(t8_file);
	PATH_FUNC_TRACE;

	/* Use the existing actime/modtime: st_ctime should still be updated */
	utimb.actime = stbuf.st_atime;
	utimb.modtime = stbuf.st_mtime;
	globok = 0;
	if (tbutime(t8_file, &utimb, SUCCEED, NOERROR, NOCHK, NOCHK) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
#ifdef TESTBED2
	savtimes(t8_file);
#endif

	globok = 0;
	chkt = chktimes(t8_file, NEWCTIME);
	PATH_FUNC_TRACE;
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("%s", chkt);
	}
	else
		PATH_TRACE;

	(void) unlink(t8_file);

#if !TEST_XNFS
	if (mkfifo(t8_file, MODEANY) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create fifo \"%s\"", t8_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (stat(t8_file, &stbuf) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
			t8_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(t8_file);
	PATH_FUNC_TRACE;

	/* Use the existing actime/modtime: st_ctime should still be updated */
	utimb.actime = stbuf.st_atime;
	utimb.modtime = stbuf.st_mtime;
	globok = 0;
	if (tbutime(t8_file, &utimb, SUCCEED, NOERROR, NOCHK, NOCHK) != SUCCEED)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
	
#ifdef TESTBED2
	savtimes(t8_file);
#endif

	globok = 0;
	chkt = chktimes(t8_file, NEWCTIME);
	PATH_FUNC_TRACE;
	if (chkt != NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("%s", chkt);
	}
	else
		PATH_TRACE;

	(void) unlink(t8_file);

#else /* TEST_XNFS */
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(12);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	int waittime;

	DBUG_ENTER("test9");

	waittime = WAITTIME * (5 + sysconf(_SC_NGROUPS_MAX)/4);

	(void) cppair(ch_t9, NULLFN, waittime, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int	pathok = setupok;
	/* write access on path, search access on path prefix granted */

	DBUG_ENTER("test9");

	globok = 0;
	if (do_access('f', (mode_t)S_IWUSR, (mode_t)S_IXUSR, t9_func, WAITTIME, SP_T9_DO_ACCESS) == 0)
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(1);
	}
	
	cln_access();

	DBUG_VOID_RETURN;
}

private int
t9_func(file, desc)
char *file, **desc;
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("t9_func");

	globok = 0;
	errno = 0;

	if (utime(file, NULLUTB) != SUCCEED)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("utime(\"%s\", NULL) failed - errno %d (%s)",
			file, err, errname(err));
		for ( ; *desc != NULL; desc++)
			in_rpt("%s", *desc);
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(0);
}

private void
test10()
{
	/* search permission denied */

	int	pathok = setupok;

	DBUG_ENTER("test10");

	testfail = 0;

#if TEST_XNFS

	/* Test with uid mapped */

	if (tet_sync(SP_T10_MKDIR2_END, snames, SYNC_TIMEOUT) < 0)
	{
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to MKDIR2_END failed");
	}
	else
	{
		globok = 0;
		(void) tbutime(t10_file2, NULLUTB, SYSERROR, EACCES, NOCHK, NOCHK);
		PATH_FUNC_TRACE;
	}

	(void) tet_sync(SP_T10_RMDIR2_START, snames, SYNC_TIMEOUT);

#else /* TEST_XNFS */
	PATH_TRACE;
#endif /* TEST_XNFS */

	/* Test with uid not mapped */

	PATH_FUNC_RPT(1);
	(void) cppair(ch_t10, NULLFN, WAITTIME, NOK);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{

	int	fd, err;
	mode_t	mode = (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
	int	pathok = 0;

	DBUG_ENTER("ch_t10");

	PATH_FUNC_TRACE;

	normsetup();

	if (vxmkdir(t10_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create directory \"%s\"", t10_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = creat(t10_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t10_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if (chmod(t10_dir, mode) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("chmod(\"%s\", %#o) failed - errno %d (%s)",
			t10_file, (int)mode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbutime(t10_file, NULLUTB, SYSERROR, EACCES, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(5);

	(void) chmod(t10_dir, MODEANY);
	(void) unlink(t10_file);
	(void) rmdir(t10_dir);

	DBUG_VOID_RETURN;
}

private void
test11()
{
	/* times=NULL, not owner of file, no write access */

	DBUG_ENTER("test11");

#if TEST_XNFS
	xx_rpt(NOTINUSE);
#else /* TEST_XNFS */
	(void) cppair(ch_t11_20, NULLFN, WAITTIME, NOK);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

private void
ch_t11_20()
{
	int	fd;
	mode_t	mode = (mode_t)(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	int	pathok = setupok;

	DBUG_ENTER("ch_t11_20");

	testfail = 0;

	if ((fd = creat(t11_file, mode)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t11_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	normsetup();

	globok = 0;
	(void) tbutime(t11_file, NULLUTB, SYSERROR, EACCES, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t11_file);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS
private void
test12()
{
	DBUG_ENTER("test12");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#else /* TEST_XNFS */
private void
test12()
{
	/* path component longer than NAME_MAX */

	char	**longname;
	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test12");

	testfail = 0;

	longname = nametoolng();
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = crfile(longname[0], MODEANY,
			(int)geteuid(), (int)getegid())) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname[0]);
	}
	else
	{
		PATH_TRACE;
		globok = 0;
		(void) tbutime(longname[0], NULLUTB, SUCCEED,
			NOERROR, NOCHK, NOCHK);
		PATH_FUNC_TRACE;
		(void) close(fd);
	}

	if (longname[1] != NULL)
		(void) tbutime(longname[1], NULLUTB, SYSERROR,
			ENAMETOOLONG, NOCHK, NOCHK);

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

private void
test13()
{
	/* path longer than PATH_MAX */

	char	**longname;
	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test13");

	testfail = 0;

	if ((fd = creat(t13_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	longname = pathtoolng(t13_file);
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbutime(longname[0], NULLUTB, SUCCEED, NOERROR, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (longname[1] != NULL)
		(void) tbutime(longname[1], NULLUTB, SYSERROR,
			ENAMETOOLONG, NOCHK, NOCHK);

	if (testfail == 0)
		PATH_XS_RPT(3);

	clntoolng();
	(void) unlink(t13_file);

	DBUG_VOID_RETURN;
}

private void
test14()
{
	int	pathok = setupok;

	/* component of path prefix does not exist */

	DBUG_ENTER("test14");

	testfail = 0;

	if (eaccess(t14_dir, F_OK) == 0 && remove(t14_dir) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t14_dir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbutime(t14_file, NULLUTB, SYSERROR, ENOENT, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test15()
{
	int	pathok = setupok;
	/* file which does not exist */

	DBUG_ENTER("test15");

	testfail = 0;

	if (eaccess(t15_file, F_OK) == 0 && remove(t15_file) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: can not remove \"%s\"", t15_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbutime(t15_file, NULLUTB, SYSERROR, ENOENT, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	int	pathok = setupok;
	/* path is "" */

	DBUG_ENTER("test16");

	testfail = 0;

	globok = 0;
	(void) tbutime("", NULLUTB, SYSERROR, ENOENT, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test17()
{
	int	pathok = setupok;
	/* component of path is not a directory */

	int	fd;

	DBUG_ENTER("test17");

	testfail = 0;

	if ((fd = creat(t17_ndir, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t17_ndir);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbutime(t17_file, NULLUTB, SYSERROR, ENOTDIR, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t17_ndir);

	DBUG_VOID_RETURN;
}

private void
test18()
{
	/* set specific times when not file owner, and no privilege */

	DBUG_ENTER("test18");

#if TEST_XNFS
	xx_rpt(NOTINUSE);
#else /* TEST_XNFS */
	(void) cppair(ch_t18_26, NULLFN, WAITTIME, NOK);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

private void
ch_t18_26()
{
	int fd;
	time_t t0;
	int	pathok = setupok;

	DBUG_ENTER("ch_t18_26");

	testfail = 0;

	if ((fd = creat(t18_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t18_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	normsetup();

	(void) time(&t0);
	utimb.actime = t0;
	utimb.modtime = t0;

	globok = 0;
	(void) tbutime(t18_file, &utimb, SYSERROR, EPERM, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t18_file);

	DBUG_VOID_RETURN;
}

private void
test19()
{
	int	pathok = setupok;
	/* read only file system */

	DBUG_ENTER("test19");

	testfail = 0;

	if (setuprofs(t19_dir, t19_file, 'f', MODEANY) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbutime(t19_file, NULLUTB, SYSERROR, EROFS, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) clnuprofs(t19_dir, t19_file);

	DBUG_VOID_RETURN;
}

private void
test20()
{
	/* times=NULL, not owner of file, no write access, uid not mapped */

	DBUG_ENTER("test20");

#if !TEST_XNFS
	xx_rpt(NOTINUSE);
#else /* TEST_XNFS */
	(void) cppair(ch_t11_20, NULLFN, WAITTIME, NOK);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

#if TEST_XNFS
private void
test21()
{
	DBUG_ENTER("test21");
	(void) cppair(ch_t21, NULLFN, WAITTIME, NOK);
	(void)unlink(t21_file);
	DBUG_VOID_RETURN;
}

private void
ch_t21()
{
	/* times=NULL, not owner of file, no write access, uid mapped */

	int	pathok = setupok;

	DBUG_ENTER("ch_t21");

	testfail = 0;

	/* Sync with server to create file owned by root */
	if (tet_sync(SP_T21_CR_FILE, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to SP_T21_CR_FILE failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbutime(t21_file, NULLUTB, SYSERROR, EACCES, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
#else /* TEST_XNFS */
private void
test21()
{
	DBUG_ENTER("test21");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

#if TEST_XNFS
private void
test22()
{
	char	*longname;
	int	pathok = setupok;

	DBUG_ENTER("test22");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbutime(longname, NULLUTB, SYSERROR, ENAMETOOLONG, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
#else /* TEST_XNFS */
private void
test22()
{
	DBUG_ENTER("test22");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

#if TEST_XNFS
private void
test23()
{
	char	*longname;
	int	pathok = setupok;

	DBUG_ENTER("test23");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbutime(longname, NULLUTB, SYSERROR, ENAMETOOLONG, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
#else /* TEST_XNFS */
private void
test23()
{
	DBUG_ENTER("test23");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

#if TEST_XNFS
private void
test24()
{
	int	fd;
	char	*longname;
	int	pathok = setupok;

	DBUG_ENTER("test24");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = creat(longname, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbutime(longname, NULLUTB, SUCCEED, NOERROR, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(longname);

	DBUG_VOID_RETURN;
}
#else /* TEST_XNFS */
private void
test24()
{
	DBUG_ENTER("test24");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

#if TEST_XNFS
private void
test25()
{
	int	fd;
	char	*longname;
	int	pathok = setupok;

	DBUG_ENTER("test25");

	testfail = 0;

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if (longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = creat(longname, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", longname);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	globok = 0;
	(void) tbutime(longname, NULLUTB, SUCCEED, NOERROR, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(longname);

	DBUG_VOID_RETURN;
}
#else /* TEST_XNFS */
private void
test25()
{
	DBUG_ENTER("test25");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

private void
test26()
{
	/* set specific times when not file owner, and uid not mapped */

	DBUG_ENTER("test26");

#if !TEST_XNFS
	xx_rpt(NOTINUSE);
#else /* TEST_XNFS */
	(void) cppair(ch_t18_26, NULLFN, WAITTIME, NOK);
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

#if TEST_XNFS
private void
test27()
{
	DBUG_ENTER("test27");
	(void) cppair(ch_t27, NULLFN, WAITTIME, NOK);
	(void)unlink(t27_file);
	DBUG_VOID_RETURN;
}

private void
ch_t27()
{
	time_t	t0;
	int	pathok = setupok;

	DBUG_ENTER("ch_t27");

	testfail = 0;

	/* Sync with server to create file owned by root */
	if (tet_sync(SP_T27_CR_FILE, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to SP_T27_CR_FILE failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) time(&t0);
	utimb.actime = t0;
	utimb.modtime = t0;

	globok = 0;
	(void) tbutime(t27_file, &utimb, SYSERROR, EPERM, NOCHK, NOCHK);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}
#else /* TEST_XNFS */
private void
test27()
{
	DBUG_ENTER("test27");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

#if TEST_XNFS
private void
test28()
{
	int	fd;
	time_t	t0;
	int	pathok = setupok;

	DBUG_ENTER("test28");

	testfail = 0;

	(void) unlink(t28_file);
	if ((fd = creat(t28_file, (mode_t)0)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("could not create file \"%s\"", t28_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	(void) sleep(2);

	if (xnfs_rtime_c(&t0, SP_T28_RTIME) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = 0;
		(void) tbutime(t28_file, NULLUTB, SUCCEED, NOERROR, t0, t0);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t28_file);

	DBUG_VOID_RETURN;
}
#else /* TEST_XNFS */
private void
test28()
{
	DBUG_ENTER("test28");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

#if TEST_XNFS
private void
test29()
{
	time_t	t0;
	int	pathok = setupok;

	DBUG_ENTER("test29");

	testfail = 0;

	/* Sync with server to create file owned by root */
	if (tet_sync(SP_T29_CR_FILE, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sync to SP_T29_CR_FILE failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(2);

	if (xnfs_rtime_c(&t0, SP_T29_RTIME) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = 0;
		(void) tbutime(t29_file, NULLUTB, SUCCEED, NOERROR, t0, t0);
		PATH_FUNC_TRACE;
	}

	if (testfail == 0)
		PATH_XS_RPT(2);

	(void) unlink(t29_file);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */
private void
test29()
{
	DBUG_ENTER("test29");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#endif /* TEST_XNFS */

private char *
strtimes(times)
struct utimbuf *times;
{
	static char buf[50];

	if (times == NULL)
		return "NULL";
	
	(void) sprintf(buf, "{actime=%ld, modtime=%ld}",
		times->actime, times->modtime);
	
	return buf;
}

private int
tbutime(tbpath, tbtimes, expret, experrno, expatime, expmtime)
char	*tbpath;
struct utimbuf *tbtimes;
int	expret;
int	experrno;
time_t	expatime;
time_t	expmtime;
{
	char	*reppath = tbpath;
	time_t	oldatime, oldmtime;
	int	rval, err, statok;
	char	pathbuf[30];
	struct stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("tbutime");

	globok = 0;

	if (expret == SYSERROR)
	{
		statok = (stat(tbpath, &stbuf) != SYSERROR);
		if (statok)
		{
			oldatime = stbuf.st_atime;
			oldmtime = stbuf.st_mtime;
		}
	}

	errno = NOERROR;
	rval = utime(tbpath, tbtimes);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;

	/* Don't print PATH_MAX length path name */
	if (strlen(tbpath) > 100)
	{
		(void) sprintf(pathbuf, "<%d char path>", (int)strlen(tbpath));
		reppath = pathbuf;
	}

	if (expret == SUCCEED && rval == expret)
	{
#ifdef TESTBED3
		expatime -= 3;
		expmtime -= 3;
#endif
		if ((expatime != NOCHK || expmtime != NOCHK) &&
			stat(tbpath, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				reppath, err, errname(err));
			DBUG_RETURN(rval);
		}
		else
			PATH_TRACE;

		if (expatime != NOCHK && (
			tbtimes != NULLUTB && stbuf.st_atime != expatime ||
			tbtimes == NULLUTB && stbuf.st_atime != expatime &&
#if TEST_XNFS
			    stbuf.st_atime != expatime + 2 &&
#endif
			    stbuf.st_atime != expatime + 1
			)
		    )
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("successful call to utime(\"%s\", %s) did not set st_atime correctly",
				reppath, strtimes(tbtimes));
			if (tbtimes == NULLUTB)
			{
				in_rpt("expected %ld-%ld, observed %ld",
#if TEST_XNFS
					(long)expatime, (long)expatime+2,
#else
					(long)expatime, (long)expatime+1,
#endif
					(long)stbuf.st_atime);
			}
			else
			{
				in_rpt("expected %ld, observed %ld",
					(long)expatime, (long)stbuf.st_atime);
			}
		}
		else
			PATH_TRACE;

		if (expmtime != NOCHK && (
			tbtimes != NULLUTB && stbuf.st_mtime != expmtime ||
			tbtimes == NULLUTB && stbuf.st_mtime != expmtime &&
#if TEST_XNFS
			    stbuf.st_mtime != expmtime + 2 &&
#endif
			    stbuf.st_mtime != expmtime + 1
			)
		    )
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("successful call to utime(\"%s\", %s) did not set st_mtime correctly",
				reppath, strtimes(tbtimes));
			if (tbtimes == NULLUTB)
			{
				in_rpt("expected %ld-%ld, observed %ld",
#if TEST_XNFS
					(long)expmtime, (long)expmtime+2,
#else
					(long)expmtime, (long)expmtime+1,
#endif
					(long)stbuf.st_mtime);
			}
			else
			{
				in_rpt("expected %ld, observed %ld",
					(long)expmtime, (long)stbuf.st_mtime);
			}
		}
		else
			PATH_TRACE;

		PATH_FUNC_RPT(3);
		DBUG_RETURN(rval);
	}
	else if (expret == SYSERROR && rval == expret)
	{
#ifdef TESTBED3
		oldatime--;
		oldmtime--;
#endif
		if (statok && stat(tbpath, &stbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat(\"%s\") failed - errno %d (%s)",
				reppath, err, errname(err));
		}
		else if (statok && (stbuf.st_atime != oldatime ||
				    stbuf.st_mtime != oldmtime))
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("unsuccessful call to utime(\"%s\", %s) changed timestamps of file",
				reppath, strtimes(tbtimes));
		}
		else
			PATH_TRACE;
		if (err == experrno)
		{
			PATH_FUNC_RPT(1);
			DBUG_RETURN(rval);
		}
	}
	else if (expret == SUCCEED && experrno == EROFS)
	{
		/* Allow for read-only root file systems */
		if (rval == SYSERROR && err == experrno)
		{
			PATH_TRACE;
			PATH_FUNC_RPT(1);
			DBUG_RETURN(rval);
		}
		else
			experrno = NOERROR; /* for valrpt() */
	}

	if (testfail++ == 0)
		xx_rpt(FAILURE);
	in_rpt("utime(\"%s\", %s) did not give correct results",
		reppath, strtimes(tbtimes));
	valrpt(expret, rval, experrno, err);

	DBUG_RETURN(rval);
}
