/*
*      SCCS:  @(#)  POSIX.os/ioprim/close/close.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/ioprim/close - T.close Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.close

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the close() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Wed Mar  1 17:53:46 GMT 1989
MODIFICATIONS:	J.A.Nave	04/06/90
		Added new test to check that all time-related fields marked
		for update are updated when the last process has closed the
		file.

		Peter Lee, Unisoft Ltd.,
		Mon Aug 06 09:49:04 BST 1990
		Added resilience code.

		Geoff Clare, 23 Mar 1992
			Removed globok definition.

		Neil Moses, 5 Oct 1992
			XNFS enhancements.

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<errno.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<synch.h>
#include	<unistd.h>
#include	<timeout.h>
#include	<time.h>
#include	<stdlib.h>
#include	<string.h>
#include <tet_api.h>
#include	<xnfs.h>
#include	"close_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef close
#endif


#define	NO_TESTS	6		/* number of tests in testset */
#define	EXTERR		"external error"
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	SYNCSIZE	4		/* number of bytes sent by sendsync */
#define	MODEANY		(S_IRWXU|S_IRWXG|S_IRWXO)
#define	MESSAGE	"abcdefghijklmnopqrstuvwxyz"
#define PATH_NOK	8

extern	void	valrpt();
extern	char	*errname();

int	no_tests = NO_TESTS;

#if TEST_XNFS
private	int		snames[] = {1, 0};
#endif /* TEST_XNFS */

private	int	t2_fd1, t2_fd2;
private	synch	*t2_p_syn;
private	synch	*t2_c_syn;

private	char	*t1_file =	"./close-t.1";
private	char	*t2_file =	"./close-t.2";
#if !TEST_XNFS
private	char	*t3_file =	"./close-t.3";
#endif /* TEST_XNFS */
private	char	*t4_file =	"./close-t.4";
#if !TEST_XNFS
private	char	*t5_file =	"./close-t.5";
#endif /* TEST_XNFS */

private int	testfail;
private char	*testcomment;

private	void	test1();
private	void	test2();
private void	ch_t2();
private void	pa_t2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	test6();
private void	tb_close();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	NULL, 0
};

private void
test1()	
{
	int		fd;
	int		err;
	int		rval;
	struct stat	statb;
	int	pathok = 0;

	DBUG_ENTER("test1");
	if ((fd = open(t1_file, O_RDWR|O_CREAT, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT) failed - errno %d(%s)",
			t1_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testcomment = "close on open file descriptor";

	globok = 0;
	tb_close(fd, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if ((rval = fstat(fd, &statb)) != SYSERROR || errno != EBADF)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("operation on a close()'d file descriptor did not return expected values");
		valrpt(SYSERROR, rval, EBADF, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		PATH_XS_RPT(3);

	(void) unlink(t1_file);
	DBUG_VOID_RETURN;
}

private void
test2()
{
#if TEST_XNFS
	char	*rlock;
#endif /* TEST_XNFS */

	int	err;
	int	pathok = 0;

	DBUG_ENTER("test2");

#if TEST_XNFS

	if ((rlock = tet_getvar("XNFS_REMLOCK_SUPP")) == NULL || *rlock == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter XNFS_REMLOCK_SUPP is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(*rlock != 'Y' && *rlock != 'y')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("Remote file locking is not supported");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else

	PATH_TRACE;
	PATH_TRACE;

#endif /* TEST_XNFS */

	if ((t2_fd1 = open(t2_file, O_RDWR|O_CREAT, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT) failed - errno %d(%s)",
			t2_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else if ((t2_fd2 = open(t2_file, O_RDWR, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR) failed - errno %d(%s)",
			t2_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t2_c_syn = opensync()) == (synch *) NULL
		|| (t2_p_syn = opensync()) == (synch *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: opensync() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	(void) cppair(ch_t2, pa_t2, WAITTIME, 0);

	(void) closesync(t2_p_syn);
	(void) closesync(t2_c_syn);
	(void) close(t2_fd2);
	(void) unlink(t2_file);
	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	struct flock	lck;
	struct {
		int	ret;
		int	err;
	} res;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

	res.ret = SUCCEED;
	lck.l_type = F_WRLCK;
	lck.l_whence = SEEK_SET;
	lck.l_start = 0L;
	lck.l_len = 0L;
	lck.l_pid = 0;

	PATH_FUNC_TRACE; /* pick up path check errors in calling function */

	if (fcntl(t2_fd1, F_SETLK, &lck) == SYSERROR)
	{
		res.ret = FAIL;
		res.err = errno;
	}
	else  if (close(t2_fd1) == SYSERROR)
	{
		res.ret = SYSERROR;
		res.err = errno;
	}
	else
		PATH_TRACE;

	errno = NOERROR;

	if ((res.ret == SUCCEED) && (pathok != 2)) /* pass path check error */
		res.ret = PATH_NOK;			/* back to parent */

	if (sendsync(t2_c_syn[0][1], (char *) &res, sizeof(res), WAITTIME)
							!= sizeof(res))
	{
		in_rpt("sendsync() failed - errno %d(%s)",
			errno, errname(errno));
		exit(A_SENDSYNC);
	}
	else if (waitsync(t2_p_syn[0][0], (char *) &res, sizeof(res),
					WAITTIME) != sizeof(res))
	{
		in_rpt("waitsync() failed - errno %d(%s)",
			errno, errname(errno));
		exit(A_WAITSYNC);
	}

	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	int	err;
	struct flock	lck;
	struct {
		int	ret;
		int	err;
	} res;
	int	pathok = 0;

	DBUG_ENTER("pa_t2");

	errno = NOERROR;
	lck.l_type = F_RDLCK;
	lck.l_whence = SEEK_SET;
	lck.l_start = 0L;
	lck.l_len = 0L;
	lck.l_pid = 0;
	if (waitsync(t2_c_syn[0][0], (char *) &res, sizeof(res),
					WAITTIME) != sizeof(res))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitsync() failed");
		in_rpt("waitsync() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;

		if (res.ret == PATH_NOK) 	/* check for path check error */
			res.ret = SUCCEED;	/* in child process */
		else
			PATH_TRACE;

		if (res.ret != SUCCEED)
		{
			xx_rpt(DELETION);
			if (res.ret == SYSERROR)
				in_rpt("deletion reason: close(fd) failed - errno %d(%s)",
					res.err, errname(res.err));
			else
				in_rpt("deletion reason: fcntl(F_WRLCK) failed - errno %d(%s)",
					res.err, errname(res.err));
		}
		else if (fcntl(t2_fd2, F_SETLK, &lck) == SYSERROR)
			if (errno == EAGAIN || errno == EACCES)
			{
				xx_rpt(FAILURE);
				in_rpt("file that was write locked was still locked after close()");
			}
			else
			{
				err = errno;
				xx_rpt(DELETION);
				in_rpt("deletion reason: could not read-lock file: errno %d(%s)",
					err, errname(err));
			}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(3);
		}
	}

	if (sendsync(t2_p_syn[0][1], (char *) &res, sizeof(res), WAITTIME)
							!= sizeof(res))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: sendsync() failed");
		in_rpt("sendsync() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	globok = 1;
	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
test3()
{
	DBUG_ENTER("test3");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test3()
{

	char		buf[sizeof(MESSAGE)];
	int	rval;
	int	fd_rd,	fd_wr;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;
	if ((rval = mkfifo(t3_file , MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : mkfifo(t3_file) failed");
		valrpt(NOERROR,rval,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd_rd = open(t3_file , O_RDONLY|O_NONBLOCK)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : open(t3_file, O_RDONLY|O_NONBLOCK) failed");
		valrpt(NOERROR,fd_rd,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd_wr = open(t3_file , O_WRONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : open(t3_file, O_WRONLY) failed");
		valrpt(NOERROR,fd_wr,NOERROR,err);
		(void) close(fd_rd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rval = write(fd_wr , MESSAGE , (unsigned)strlen(MESSAGE))) !=
		strlen(MESSAGE))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() failed");
		valrpt((int)strlen(MESSAGE),rval,NOERROR,err);
		(void) close(fd_rd);
		(void) close(fd_wr);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	tb_close(fd_rd, NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	tb_close(fd_wr, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if(testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd_rd = open(t3_file , O_RDONLY|O_NONBLOCK)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : open(t3_file, O_RDONLY|O_NONBLOCK) failed");
		valrpt(NOERROR,fd_rd,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd_wr = open(t3_file , O_WRONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : open(t3_file, O_WRONLY) failed");
		valrpt(NOERROR,fd_wr,NOERROR,err);
		(void) close(fd_rd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(5*WAITTIME)

	errno = NOERROR;
	rval = read(fd_rd , buf , (unsigned)strlen(MESSAGE));

	CLEAR_ALARM

	if (rval != SYSERROR || errno != EAGAIN)
	{
		err = errno;
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("close failed to discard data in FIFO");
		in_rpt("read on reopened FIFO failed");
		if (rval > 0)
			in_rpt("data read \"%.*s\" ",rval,buf);
		valrpt(SYSERROR,rval,EAGAIN,err);
	}
	else
		PATH_TRACE;

	globok = 0;
	tb_close(fd_rd, NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	tb_close(fd_wr, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if(!testfail)
		PATH_XS_RPT(12);

	(void)  unlink(t3_file);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test4()
{
	int		fd;
	int		err;
	time_t		curtime;
	struct	stat	statb;
	char		buf[1];
	int		pathok = 0;

	DBUG_ENTER("test4");

	if ((fd = open(t4_file, O_RDWR|O_CREAT, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT) failed - errno %d(%s)",
			t4_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, "AB", 2) != 2)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() failed - errno %d(%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (lseek(fd, (off_t) 0, SEEK_SET) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek to start of %s failed with - error %d(%s)",
			t4_file, err, errname(err));
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(2);

#if TEST_XNFS

	if (xnfs_rtime_c(&curtime, SP_T4_RTIME_START) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("could not obtain current time on server");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */

	(void) time(&curtime);

	PATH_TRACE;

#endif /* TEST_XNFS */

	if (read(fd, buf, 1) != 1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : read() failed - errno %d(%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, "C", 1) != 1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason : write() failed - errno %d(%s)", err, errname(err));
		(void) close(fd);
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testcomment = "close on a file descriptor updates all time-related feilds still marked";
	
	globok = 0;
	tb_close(fd, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	(void) sleep(2);

	if (stat(t4_file, &statb) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat(%s) failed - errno %d(%s)", t4_file, err, errname(err));
		(void) unlink(t4_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statb.st_atime < curtime && statb.st_atime > curtime +2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_atime not updated by close(), got %ld but expected in range %ld - %ld", (long)statb.st_atime, (long)curtime, (long)curtime +2);
	}
	else
		PATH_TRACE;

	if (statb.st_ctime < curtime && statb.st_ctime > curtime +2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_ctime not updated by close(), got %ld but expected in range %ld - %ld", (long)statb.st_ctime, (long)curtime, (long)curtime +2);
	}
	else
		PATH_TRACE;

	if (statb.st_mtime < curtime && statb.st_mtime > curtime +2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("st_mtime not updated by close(), got %ld but expected in range %ld - %ld", (long)statb.st_mtime, (long)curtime, (long)curtime +2);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(11);

	(void) unlink(t4_file);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test5()
{
	DBUG_ENTER("test5");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test5()
{
	int		fd;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test5");
	if ((fd = open(t5_file, O_RDWR|O_CREAT, MODEANY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT) failed - errno %d(%s)",
			t5_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testcomment = "close on previously closed file descriptor";
	
	globok = 0;
	tb_close(fd, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	tb_close(fd, SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);
	(void) unlink(t5_file);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test6()
{
	DBUG_ENTER("test6");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test6()
{
	int	err, fd;
	int	pathok = 0;

	DBUG_ENTER("test6");

	errno = NOERROR;

	if ((fd = open(FILE6, O_RDWR|O_CREAT, 0777)) == SYSERROR) 
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s,O_RDWR|O_CREAT,0777) failed - errno %d (%s)", FILE6, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, T6_DATA, 6) != 6)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() failed to write 6 bytes to file %s - errno %d (%s)", FILE6, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (close(fd) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("close() on file %s failed - errno %d (%s)", 
			FILE6, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T6_CHECK_DATA_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CHECK_DATA_END failed");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}

	(void) close(fd);

	(void) unlink(FILE6);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
tb_close(tbfd, expret, experrno)
int tbfd;
int expret, experrno;
{
	int rval;
	int err;
	struct stat	statb;
	int	pathok = 0;

	pathok = 0;
	rval = close(tbfd);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif /* TESTBED */

	err = errno;
	if (expret == 0)
	{
		if (rval != 0)
		{
			if (!testfail++)
				xx_rpt(FAILURE);
			in_rpt("close(%d) failed", tbfd);
			in_rpt(testcomment);
			in_rpt("close() failed - errno %d(%s)",
				err, errname(err));
			return;
		}
		else
			PATH_TRACE;

		if ((rval = fstat(tbfd, &statb)) != SYSERROR || errno != EBADF)
		{
			if (!testfail++)
				xx_rpt(FAILURE);
			in_rpt("close(%d) failed", tbfd);
			in_rpt(testcomment);
			in_rpt("fstat() on a close()'d file descriptor did not return expected values");
			valrpt(SYSERROR, rval, EBADF, err);
		}
		else
			PATH_TRACE;

		globok = (pathok == 2) ? 1 : 0;
		return;
	}
	else
		PATH_TRACE;

	if (expret == rval && experrno == err)
	{
		globok = (pathok == 1) ? 1 : 0;
		return;
	}

	in_rpt("close(%d) failed", tbfd);
	if (testcomment != (char *) NULL)
		in_rpt(testcomment);
	valrpt(expret, rval, experrno, err);

}

