/*
*      SCCS:  @(#)  POSIX.os/ioprim/dup/dup.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
static	char sccsid[] = "@(#)POSIX.os/ioprim/dup - T.dup Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif  /* lint	*/

/***********************************************************************

NAME:		POSIX.os/ioprim/dup/dup.c
PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Implements the dup test set as defined in the
	dup functional and design specification documents.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.;
DATE CREATED:	Mon Feb 27 15:26:53 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 11 Oct 1989
			Tidy up test2.
		
		Geoff Clare, 29 May 1990
			Allow for unlimited OPEN_MAX

		Peter Lee, Unisoft Ltd.,
		Mon Aug 06 10:37:40 BST 1990
		Added resilience code.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<report.h>
#include	<limits.h>
#include	<exitcodes.h>
#include	<unistd.h>
#include	<pcts_max.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef dup
#endif


#define	NO_TESTS	5		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	READ		0
#define	WRITE		1
#define NOK		02
#define PATH_NOK	8

extern	void	valrpt();
extern	char	*errname();

public	int 	no_tests = NO_TESTS;

private int	testfail;
private char	*testcomment;

private void	prepare_tests();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private void	ch_t2();
private void	ch_t5();
private void	cln_tests();
private	int	tb_dup();

private	char	*t1_path =	"./dup-t.1";
private	char	*t5_path =	"./dup-t.5";

private int 	fd, dfd;
private int	open_max;
private struct	flock	flck;

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	NULL, 0
};

private void
prepare_tests()
{
	int	j;

	DBUG_ENTER("prepare_tests");

	if ((fd = open(t1_path, O_WRONLY|O_CREAT|O_TRUNC|O_APPEND, S_IRWXU)) 
		== SYSERROR)
	{
		for (j = 1; j <= 4; j++)
			(void) tet_delete(j, "open() fail in preparation");
	}

	DBUG_PRINTF("prep_ctests", "fd = %d\n", fd);


	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0 && !(open_max == -1 && errno == 0))
	{
		for (j = 1; j <= NO_TESTS; j++)
			(void) tet_delete(j,
				"sysconf(_SC_OPEN_MAX) failed during prep");
	}

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int i;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test1");
	
	testfail = 0;

	if (close(0) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Unable to close standard input (fd 0), errno follows");
		in_rpt(errname(err));
		for (i = 2; i <= 3; i++)
			(void) tet_delete(i, "close() fail in compliance test setup (1)");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	dfd = tb_dup(fd, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail)
	{
		for (i = 2; i <= 3; i++)
			(void) tet_delete(i, "deletion reason: test 1 failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (dfd != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("dup() returns the lowest available file descriptor");
		in_rpt("lowest available was 0 ( 0 closed for this test)");
		in_rpt("dup() returned %d", dfd);
	}
	else
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	long ret;
	int	ret1, ret2;
	struct stat fd_buf,	dfd_buf;
	int	fildes[2];
	int	fildes_dup;
	int	statw;
	int	err;
	int	pathok = 0;
	int	retval;

	DBUG_ENTER("test2");

	testfail = 0;

	if (fstat(fd, &fd_buf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fstat(dfd, &dfd_buf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* file descriptors have the same file	*/

	if (dfd_buf.st_dev != fd_buf.st_dev)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup() does not have same st_dev");
		in_rpt("expected %d - got %d",fd_buf.st_dev,dfd_buf.st_dev);
	}
	else if (dfd_buf.st_ino != fd_buf.st_ino)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup() does not have same st_ino");
		in_rpt("expected %d - got %d",fd_buf.st_ino,dfd_buf.st_ino);
	}
	else
		PATH_TRACE;

/* file descriptors have the same file offset	*/

	if (write(fd, "0123456789", 10) != 10)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() failed - errno %d(%s)",
			err, errname(err));
	}
	else if ( (ret = lseek(dfd, (off_t)0, SEEK_CUR)) == SYSERROR)
	{
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: lseek() failed - errno %d(%s)",
			err, errname(err));
	}
	else if (ret != 10L)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup() does not share same file pointer");
	}
	else
		PATH_TRACE;

/* file descriptors have the same file access mode and status flags	*/

	ret1 = fcntl(fd, F_GETFL);
	ret2 = fcntl(dfd, F_GETFL);
	if (ret1 != ret2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup() has different access mode and/or status flags");
		in_rpt("expected 0x%x - got 0x%x", ret1, ret2);
	}
	else
		PATH_TRACE;

/* file descriptors have the same file locks	*/

	flck.l_type = F_WRLCK;
	flck.l_whence = SEEK_SET;
	flck.l_start = 1;
	flck.l_len = 2;
	if (fcntl(fd , F_SETLK , &flck) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_SETLK) failed - errno %d (%s)",
			err, errname(err));
	}
	else 
	{
		retval = cppair(ch_t2, NULLFN, WAITTIME, NOK|PATH_NOK);
		if (retval == PATH_NOK)
			retval = OK;
		else
			PATH_TRACE;

		if (retval == NOK)
		{
			testfail++;
		}
		else
			PATH_TRACE;
	}

/* dup() on a pipe: file descriptors have the same file access
   mode and status flags	*/

	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: pipe() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	fildes_dup = tb_dup(fildes[READ], NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	if (fildes_dup < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ret1 = fcntl(fildes[READ], F_GETFL);
	ret2 = fcntl(fildes_dup, F_GETFL);
	if (ret1 != ret2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup() on pipe[%d] has different access mode and/or status flags",
			READ);
		in_rpt("expected 0x%x - got 0x%x", ret1, ret2);
	}
	else
		PATH_TRACE;

	(void) close(fildes_dup);

	statw = fcntl(fildes[WRITE], F_GETFL);
	if (fcntl(fildes[WRITE], F_SETFL, statw|O_NONBLOCK) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_SETFL) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	fildes_dup = tb_dup(fildes[WRITE], NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	if (fildes_dup < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ret1 = fcntl(fildes[WRITE], F_GETFL);
	ret2 = fcntl(fildes_dup, F_GETFL);
	if (ret1 != ret2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup() on pipe[%d] has different access mode and/or status flags",
			WRITE);
		in_rpt("expected 0x%x - got 0x%x", ret1, ret2);
	}
	else
		PATH_TRACE;

	(void) close(fildes_dup);

	if(!testfail)
		PATH_XS_RPT(15);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	struct flock dfd_flck;
	int err;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");

/* file descriptors have the same file locks	*/

	dfd_flck.l_type = F_WRLCK;
	dfd_flck.l_whence = SEEK_SET;
	dfd_flck.l_start = 0;
	dfd_flck.l_len = 9;

	if (fcntl(dfd, F_GETLK, &dfd_flck) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_GETLK) failed - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if ((flck.l_type != dfd_flck.l_type) ||
		 (flck.l_start != dfd_flck.l_start) ||
		 (flck.l_len != dfd_flck.l_len))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup() does not have same locks");
		in_rpt("expected l_type  %d - got %d",
			flck.l_type, dfd_flck.l_type);
		in_rpt("expected l_start %ld - got %ld",
			(long)flck.l_start, (long)dfd_flck.l_start);
		in_rpt("expected l_len   %ld - got %ld",
			(long)flck.l_len, (long)dfd_flck.l_len);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);	/* path check error */

	DBUG_VOID_RETURN;
}


private void
test3()
{
	int execflag;
	int	pathok = 0;

	DBUG_ENTER("test3");

	execflag = fcntl(dfd, F_GETFD, 0);

	DBUG_PRINTF("test3", "execflag = %d, execflag & FD_CLOEXEC = %d\n",
		execflag, (execflag & FD_CLOEXEC));

	if (execflag & FD_CLOEXEC)
	{
		xx_rpt(FAILURE);
		in_rpt("close-on-exec flag for file decriptor returned by dup()");
		in_rpt("is set");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) close(fd);
	(void) close(dfd);
	(void) unlink(t1_path);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;
	testcomment = "dup with an illegal file descriptor";

	globok = 0;
	(void) tb_dup(-1, SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	if (close(fd) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: close() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "dup with a closed file descriptor";

	globok = 0;
	(void) tb_dup(fd, SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test5()
{

	int	err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	if ((fd = creat(t5_path, O_ACCMODE)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	DBUG_PRINTF("test5", "fd = %d\n", fd);

	(void) cppair(ch_t5, NULLFN, WAITTIME, 0);

	(void) unlink(t5_path);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	max_fd;
	int i, *fds;
	int	pathok = 0;
	int	loop1;

	DBUG_ENTER("ch_t5");

	DBUG_PRINTF("ch_t5", "OPEN_MAX = %d\n", open_max);

	testfail = 0;
	if (open_max <= PCTS_OPEN_MAX && open_max != -1)
		testcomment = "dup gives EMFILE error when OPEN_MAX fd's used";
	else
		testcomment = "PCTS_OPEN_MAX files can be opened";

	max_fd = (open_max <= PCTS_OPEN_MAX && open_max != -1) ?
		open_max : PCTS_OPEN_MAX;

	fds = (int *) malloc((size_t)((max_fd+1) * sizeof(int)));
	if (fds == (int *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: malloc() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	for (i = n_open_fds(), loop1 = i ; i < max_fd ; i++)
	{
		globok = 0;
		fds[i] = tb_dup(fd, NOERROR, NOERROR);
		PATH_FUNC_TRACE;

		if (testfail)
		{
			in_rpt("dup failed with %d file descriptors open", i-1);
			in_rpt("OPEN_MAX is set to %d", open_max);
			break;
		}
	}
	loop1 = i - loop1;

	if (!testfail)
	{
		PATH_TRACE;
		if (open_max <= PCTS_OPEN_MAX && open_max != -1)
			fds[i] = tb_dup(fd, SYSERROR, EMFILE);
	}

	if (!testfail)
		PATH_XS_RPT(2 + loop1);

	DBUG_VOID_RETURN;
}

private int
tb_dup(tbfd, expret, experrno)
int tbfd;
int expret, experrno;
{
	int rval;
	int err;
	int	pathok = 0;

	DBUG_LENTER("tb_dup", 2);

	errno = 0;
	rval = dup(tbfd);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif /* TESTBED */

	err = errno;
	globok = 0;

	if (expret == NOERROR)
	{
		if (rval != tbfd && rval >= 0 &&
				(rval < open_max || open_max == -1))
		{
			PATH_FUNC_RPT(0);
			DBUG_RETURN(rval);
		}
	}
	else
	{
		if (rval == expret && err == experrno)
		{
			PATH_FUNC_RPT(0);
			DBUG_RETURN(rval);
		}
	}

	if (!testfail++)
		xx_rpt(FAILURE);

	in_rpt("dup(%d) failed", tbfd);
	if (testcomment != (char *) NULL); 
		in_rpt(testcomment);
	valrpt(expret, rval, experrno, err);
		
	DBUG_RETURN(rval);
}
