/*
*      SCCS:  @(#)  POSIX.os/ioprim/dup2/dup2.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
static	char sccsid[] = "@(#)POSIX.os/ioprim/dup2 - T.dup2 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif  /* lint	*/

/***********************************************************************

NAME:		POSIX.os/ioprim/dup2/dup2.c
PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Implements the dup2 test set as defined in the
	dup2 functional and design specification documents.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.;
DATE CREATED:	Mon Feb 27 15:26:53 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 18 Sep 1989
			Add EBADF case with fildes2 == OPEN_MAX
			Tidy up OPEN_MAX test
		
		Geoff Clare, 11 Oct 1989
			Tidy up test3.  Change a DELETION to FAILURE in
			test2 and test5.
		
		Geoff Clare, 29 May 1990
			Revise EBADF tests.

		Peter Lee, Unisoft Ltd.,
		Mon Aug 06 11:34:25 BST 1990
		Added resilience code.

		Stuart Boutell, 1 Dec 1990
			Added code to ensure fd 0 is open before
			leaving tests 1,3 and 4 else delete
			all outstanding tests.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<report.h>
#include	<limits.h>
#include	<unistd.h>
#include	<pcts_max.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef dup2
#endif


#define	NO_TESTS	7		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODE1		(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP)
#define	READ		0
#define	WRITE		1
#define	NOK		02
#define PATH_NOK	8

extern	void	valrpt();
extern	char	*errname();

public	int 	no_tests = NO_TESTS;

private int	testfail;
private char	*testcomment;
private	struct	stat	ensure_buf; /* fstat buffer used to ensure fd 0 open */

private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	test6();
private	void	test7();
private void	ch_t3();
private void	cln_tests();
private	int	tb_dup2();

private	char	*t1_path =	"./dup2-t.1";
private	char	*t5_path =	"./dup2-t.5";
private int	t3_fd;
private	struct flock flck;

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	NULL, 0
};

private void
test1()
{
	int	err;
	int 	fd, dfd;
	int	pathok = 0;

	DBUG_ENTER("test1");
	
	testfail = 0;

	if ((fd = creat(t1_path, MODE1)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("Unable to create test file, errno and filename follows");
		in_rpt(t1_path);
		in_rpt("creat() failed : %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (close(0) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("Unable to close standard input (fd 0), errno follows");
		in_rpt(errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	dfd = tb_dup2(fd, 0, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (dfd != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("dup2() returns the lowest available file descriptor");
		in_rpt("lowest available was 0 ( 0 closed for this test)");
		in_rpt("dup2() returned %d", dfd);
	}
	else
		PATH_XS_RPT(3);

	/* Reopen file descriptor 0 before exiting */
	(void) close(fd);
	(void) close(dfd);
	if((fstat(0,&ensure_buf)==SYSERROR) && (open(t1_path,O_RDONLY)!=0))
	{
		int	i;
		for(i=1;i<=no_tests;i++)
			tet_delete(i,"Failed to ensure file descriptor 0 remained open after test1");
	}
	(void) unlink(t1_path);
	DBUG_VOID_RETURN;
}

private void
test2()
{

	
	int 	fd, dfd;
	off_t	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	if ((fd = creat(t1_path, MODE1)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("Unable to create test file, errno and filename follows");
		in_rpt(t1_path);
		in_rpt("creat() failed : %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	dfd = tb_dup2(fd, fd, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (dfd != fd)
	{
		xx_rpt(FAILURE);
		in_rpt("file descriptor returned by dup2() is a different");
		in_rpt("from the file descriptor returned by creat()");
		in_rpt("expected %d - got %d",fd,dfd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* seek to the start of the file to check that is is open	*/

	errno = 0;
	ret = lseek(dfd, (off_t)0, SEEK_SET);
	if (ret == SYSERROR && errno == EBADF)
	{
		xx_rpt(FAILURE);
		in_rpt("dup2() has closed the file descriptor %d",dfd);
		DBUG_VOID_RETURN;
	}
	else if (ret == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_XS_RPT(3);

	(void) close(fd);
	(void) close(dfd);
	(void) unlink(t1_path);
	DBUG_VOID_RETURN;
}

#if 0
private void
oldtest3()
{
	int 	fd, dfd;
	int	fildes[2];
	int	fildes_dup;
	int	statw;
	long ret;
	struct stat fd_buf,	dfd_buf;
	struct flock fd_flck,	dfd_flck;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test3");

	if ((fd = creat(t1_path, MODE1)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("Unable to create test file, errno and filename follows");
		in_rpt(t1_path);
		in_rpt("creat() failed : %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* set up lock for later use	*/
	flck.l_type = F_WRLCK;
	flck.l_whence = SEEK_SET;
	flck.l_start = (off_t)0;
	flck.l_len = (off_t)0;
	if (fcntl(fd , F_SETLK , &flck) == SYSERROR)
	{
		in_rpt("fcntl(fd, F_SETLK, &flck) failed during prep");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* set up status flag for later use */
	statw = fcntl(fd, F_GETFL, 0);
	if (fcntl(fd , F_SETFL , statw|O_APPEND) == SYSERROR)
	{
		in_rpt("fcntl(fd, F_SETFL, statw|O_APPEND) failed during prep");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	dfd = tb_dup2(fd, 0, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (fstat(fd, &fd_buf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fstat(dfd, &dfd_buf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* file descriptors have the same file	*/

	if (fd_buf.st_ino != fd_buf.st_ino)
	{
		xx_rpt(FAILURE);
		in_rpt("file descriptor returned by dup2() does not describe");
		in_rpt("the same open file as the descriptor returned by creat()");
		in_rpt("expected %d - got %d",fd_buf.st_ino,dfd_buf.st_ino);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* file descriptors have the same file offset	*/

	if (write(fd, "0123456789", 10) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	
	if ( (ret = lseek(dfd, (off_t)0, SEEK_CUR)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ret != 10L)
	{
		xx_rpt(FAILURE);
		in_rpt("file decriptor returned by dup2() does not share");
		in_rpt("the same file pointer as the file descriptor returned");
		in_rpt("by creat()");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* file descriptors have the same file access modes	*/

	if (fd_buf.st_mode != fd_buf.st_mode)
	{
		xx_rpt(FAILURE);
		in_rpt("file descriptor returned by dup2() has a different");
		in_rpt("access mode to the file descriptor returned by creat()");
		in_rpt("expected %d - got %d",fd_buf.st_mode,dfd_buf.st_mode);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* file descriptors have the same file status flags	*/

	if ( (fcntl(fd, F_GETFL, 0) & O_ACCMODE )!= 
		(fcntl(dfd, F_GETFL, 0) & O_ACCMODE) )
	{
		xx_rpt(FAILURE);
		in_rpt("file descriptor returned by dup2() has a different");
		in_rpt("status flag to the file descriptor returned by creat()");
		in_rpt("expected %d - got %d",
			(fcntl(fd, F_GETFL, 0) & O_ACCMODE),
			(fcntl(dfd, F_GETFL, 0) & O_ACCMODE));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( (fcntl(fd, F_GETFL, 0) & O_APPEND )!= 
		(fcntl(dfd, F_GETFL, 0) & O_APPEND) )
	{
		xx_rpt(FAILURE);
		in_rpt("file descriptor returned by dup2() has a different");
		in_rpt("status flag to the file descriptor returned by creat()");
		in_rpt("expected %d - got %d",
			(fcntl(fd, F_GETFL, 0) & O_APPEND),
			(fcntl(dfd, F_GETFL, 0) & O_APPEND));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* file descriptors have the same file locks	*/

	fd_flck.l_type = dfd_flck.l_type = F_WRLCK;
	fd_flck.l_whence = dfd_flck.l_whence = SEEK_SET;
	fd_flck.l_start = dfd_flck.l_start = (off_t)0;
	fd_flck.l_len = dfd_flck.l_len = (off_t)0;

	(void) fcntl(fd, F_GETFL, &fd_flck);
	(void) fcntl(dfd, F_GETFL, &dfd_flck);

	if ((fd_flck.l_type != dfd_flck.l_type) ||
		(fd_flck.l_start != dfd_flck.l_start) ||
		(fd_flck.l_len != dfd_flck.l_len))
	{
		xx_rpt(FAILURE);
		in_rpt("file descriptor returned by dup2() has a different");
		in_rpt("lock type to the file descriptor returned by creat()");
		in_rpt("expected %d - got %d",fd_flck.l_type,dfd_flck.l_type);
		in_rpt("expected %d - got %d",fd_flck.l_start,dfd_flck.l_start);
		in_rpt("expected %d - got %d",fd_flck.l_len,dfd_flck.l_len);
	}
	else
		PATH_TRACE;

/* file descriptors have the same file status flags on a pipe */
	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: pipe() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	statw = fcntl(fildes[WRITE], F_GETFL, 0);
	(void) fcntl(fildes[WRITE] , F_SETFL , statw|O_NONBLOCK);

	globok = 0;
	fildes_dup = tb_dup2(fildes[WRITE], fildes[WRITE], NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if ( (fcntl(fildes[WRITE], F_GETFL, 0) & O_ACCMODE )!= 
		(fcntl(fildes_dup, F_GETFL, 0) & O_ACCMODE) )
	{
		xx_rpt(FAILURE);
		in_rpt("file descriptor returned by dup2() has a different");
		in_rpt("status flags to the file descriptor returned by creat()");
		in_rpt("expected %d - got %d",
			(fcntl(fildes[WRITE], F_GETFL, 0) & O_ACCMODE),
			(fcntl(fildes_dup, F_GETFL, 0) & O_ACCMODE));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( (fcntl(fildes[WRITE], F_GETFL, 0) & O_NONBLOCK )!= 
		(fcntl(fildes_dup, F_GETFL, 0) & O_NONBLOCK) )
	{
		xx_rpt(FAILURE);
		in_rpt("file descriptor returned by dup2() has a different");
		in_rpt("status flags to the file descriptor returned by creat()");
		in_rpt("expected %d - got %d",
			(fcntl(fildes[WRITE], F_GETFL, 0) & O_NONBLOCK),
			(fcntl(fildes_dup, F_GETFL, 0) & O_NONBLOCK));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	(void) close(fildes_dup);

	if(!testfail)
		PATH_XS_RPT(18);

	(void) close(fd);
	(void) close(dfd);
	(void) unlink(t1_path);
	DBUG_VOID_RETURN;
}
#endif

private void
test3()
{
	int 	fd, dfd;
	long	ret;
	int	ret1, ret2;
	struct stat fd_buf,	dfd_buf;
	int	fildes[2];
	int	fildes_dup;
	int	statw;
	int	err;
	int	pathok = 0;
	int	retval;

	DBUG_ENTER("test3");

	testfail = 0;

	if ((fd = open(t1_path, O_WRONLY|O_CREAT|O_TRUNC|O_APPEND, S_IRWXU)) 
		== SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_CREAT|O_TRUNC|O_APPEND, S_IRWXU) failed - errno %d (%s)",
			t1_path, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	dfd = tb_dup2(fd, 0, NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	if (dfd < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	t3_fd = dfd;

	if (fstat(fd, &fd_buf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fstat(dfd, &dfd_buf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fstat() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* file descriptors have the same file	*/

	if (dfd_buf.st_dev != fd_buf.st_dev)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup2() does not have same st_dev");
		in_rpt("expected %d - got %d",fd_buf.st_dev,dfd_buf.st_dev);
	}
	else if (dfd_buf.st_ino != fd_buf.st_ino)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup2() does not have same st_ino");
		in_rpt("expected %d - got %d",fd_buf.st_ino,dfd_buf.st_ino);
	}
	else
		PATH_TRACE;

/* file descriptors have the same file offset	*/

	if (write(fd, "0123456789", 10) != 10)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() failed - errno %d(%s)",
			err, errname(err));
	}
	else if ( (ret = lseek(dfd, (off_t)0, SEEK_CUR)) == SYSERROR)
	{
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: lseek() failed - errno %d(%s)",
			err, errname(err));
	}
	else if (ret != 10L)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup2() does not share same file pointer");
	}
	else
		PATH_TRACE;

/* file descriptors have the same file access mode and status flags	*/

	ret1 = fcntl(fd, F_GETFL);
	ret2 = fcntl(dfd, F_GETFL);
	if (ret1 != ret2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup2() has different access mode and/or status flags");
		in_rpt("expected 0x%x - got 0x%x", ret1, ret2);
	}
	else
		PATH_TRACE;

/* file descriptors have the same file locks	*/

	flck.l_type = F_WRLCK;
	flck.l_whence = SEEK_SET;
	flck.l_start = 1;
	flck.l_len = 2;
	if (fcntl(fd , F_SETLK , &flck) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_SETLK) failed - errno %d (%s)",
			err, errname(err));
	}
	else 
	{
		retval = cppair(ch_t3, NULLFN, WAITTIME, NOK|PATH_NOK);
		if (retval == PATH_NOK)
			retval = OK;
		else
			PATH_TRACE;

		if (retval == NOK)
		{
			testfail++;
		}
		else
			PATH_TRACE;
	}

/* dup2() on a pipe: file descriptors have the same file access
   mode and status flags	*/

	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: pipe() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	fildes_dup = tb_dup2(fildes[READ], fildes[READ], NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	if (fildes_dup < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ret1 = fcntl(fildes[READ], F_GETFL);
	ret2 = fcntl(fildes_dup, F_GETFL);
	if (ret1 != ret2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup2() on pipe[%d] has different access mode and/or status flags",
			READ);
		in_rpt("expected 0x%x - got 0x%x", ret1, ret2);
	}
	else
		PATH_TRACE;

	(void) close(fildes_dup);

	statw = fcntl(fildes[WRITE], F_GETFL);
	if (fcntl(fildes[WRITE], F_SETFL, statw|O_NONBLOCK) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_SETFL) failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	fildes_dup = tb_dup2(fildes[WRITE], fildes[WRITE], NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	if (fildes_dup < 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ret1 = fcntl(fildes[WRITE], F_GETFL);
	ret2 = fcntl(fildes_dup, F_GETFL);
	if (ret1 != ret2)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup2() on pipe[%d] has different access mode and/or status flags",
			WRITE);
		in_rpt("expected 0x%x - got 0x%x", ret1, ret2);
	}
	else
		PATH_TRACE;

	(void) close(fildes_dup);
	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);

	if(!testfail)
		PATH_XS_RPT(18);

	(void) close(fd);
	(void) close(dfd);
	if((fstat(0,&ensure_buf)==SYSERROR) && (open(t1_path,O_RDONLY)!=0))
	{
		int	i;
		for(i=1;i<=no_tests;i++)
			tet_delete(i,"Failed to ensure fd(0) remained open after test3");
	}
	(void) unlink(t1_path);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	struct flock dfd_flck;
	int err;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

/* file descriptors have the same file locks	*/

	dfd_flck.l_type = F_WRLCK;
	dfd_flck.l_whence = SEEK_SET;
	dfd_flck.l_start = 0;
	dfd_flck.l_len = 9;

	if (fcntl(t3_fd, F_GETLK, &dfd_flck) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_GETLK) failed - errno %d (%s)",
			err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if ((flck.l_type != dfd_flck.l_type) ||
		 (flck.l_start != dfd_flck.l_start) ||
		 (flck.l_len != dfd_flck.l_len))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fd returned by dup2() does not have same locks");
		in_rpt("expected l_type  %d - got %d",
			flck.l_type, dfd_flck.l_type);
		in_rpt("expected l_start %ld - got %ld",
			(long)flck.l_start, (long)dfd_flck.l_start);
		in_rpt("expected l_len   %ld - got %ld",
			(long)flck.l_len, (long)dfd_flck.l_len);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 2)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int 	fd, dfd;
	int execflag;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test4");

	if ((fd = creat(t1_path, MODE1)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("Unable to create test file, errno and filename follows");
		in_rpt(t1_path);
		in_rpt("creat() failed : %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	dfd = tb_dup2(fd, 0, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	execflag = fcntl(dfd, F_GETFD, 0);

	if (execflag & FD_CLOEXEC)
	{
		xx_rpt(FAILURE);
		in_rpt("close-on-exec flag for file decriptor returned by dup2()");
		in_rpt("is set");
	}
	else
		PATH_XS_RPT(2);

	(void) close(fd);
	(void) close(dfd);
	if((fstat(0,&ensure_buf)==SYSERROR) && (open(t1_path,O_RDONLY)!=0))
	{
		int	i;
		for(i=1;i<=no_tests;i++)
			tet_delete(i,"Failed to ensure fd(0) remained open after test4");
	}
	(void) unlink(t1_path);
	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) unlink(t1_path);
	(void) unlink(t5_path);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	int 	fd;
	int	fildes2;
	int	rval;
	char	buf[1];
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;
	if ((fildes2 = creat(t5_path, MODE1)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fildes2);

	if ((fildes2 = open(t5_path, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "dup2 with an illegal file descriptor fildes";

	errno = NOERROR;
	rval = read(fildes2 , buf , 0);
	if (rval == SYSERROR || errno != NOERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: read(fildes2,buf,0) before dup2() failed");
		valrpt(0,rval,NOERROR,err);
		(void) close(fildes2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tb_dup2(-1, fildes2, SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	rval = read(fildes2 , buf , 0);
	if (rval == SYSERROR || errno != NOERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: read(fildes2,buf,0) after dup2() failed");
		valrpt(0,rval,NOERROR,err);
		(void) close(fildes2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "dup2 with a closed file descriptor";

	errno = NOERROR;
	rval = read(fildes2 , buf , 0);
	if (rval == SYSERROR || errno != NOERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: read(fildes2,buf,0) before dup2() failed");
		valrpt(0,rval,NOERROR,err);
		(void) close(fildes2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = creat(t1_path, MODE1)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("Unable to create test file, errno and filename follows");
		in_rpt(t1_path);
		in_rpt("creat() failed : %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (close(fd) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: close() failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tb_dup2(fd, fildes2, SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	rval = read(fildes2 , buf , 0);
	if (rval == SYSERROR || errno != NOERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("deletion reason: read(fildes2,buf,0) after dup2() failed");
		valrpt(0,rval,NOERROR,err);
		(void) close(fildes2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(10);

	(void) close(fildes2);
	(void) unlink(t1_path);
	DBUG_VOID_RETURN;
}

private void
test6()
{
	int 	fd;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;
	testcomment = "dup2 with an illegal file descriptor fildes2";

	if ((fd = creat(t1_path, MODE1)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("Unable to create test file, errno and filename follows");
		in_rpt(t1_path);
		in_rpt("creat() failed : %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tb_dup2(fd, -1, SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	(void) close(fd);
	(void) unlink(t1_path);
	DBUG_VOID_RETURN;
}

private void
test7()
{
	long	open_max;
	int	fd, fd2, err;
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0 && !(open_max == -1 && errno == 0))
	{
		xx_rpt(DELETION);
		in_rpt("sysconf(_SC_OPEN_MAX) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#ifdef TESTBED2
	open_max = -1;
#endif

	DBUG_PRINTF("test7", "open_max = %ld", open_max);

	if ((fd = creat(t1_path, MODE1)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s) failed - errno %d (%s)",
			t1_path, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (open_max != -1)
	{
		testcomment = "with fildes2 == OPEN_MAX-1";
		globok = 0;
		fd2 = tb_dup2(fd, (int)(open_max-1), NOERROR, NOERROR);
		PATH_FUNC_TRACE;
	}
	else
	{
		testcomment = "with fildes2 == PCTS_OPEN_MAX";
		globok = 0;
		fd2 = tb_dup2(fd, PCTS_OPEN_MAX, NOERROR, NOERROR);
		PATH_FUNC_TRACE;
	}
	(void) close(fd2);

	if (open_max != -1 && open_max <= INT_MAX)
	{
		testcomment = "with fildes2 == OPEN_MAX";
		globok = 0;
		(void) tb_dup2(fd, (int)open_max, SYSERROR, EBADF);
		PATH_FUNC_TRACE;

		testcomment = "with fildes2 == INT_MAX";
		globok = 0;
		(void) tb_dup2(fd, (int)INT_MAX, SYSERROR, EBADF);
		PATH_FUNC_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(5);

	(void) close(fd);
	(void) unlink(t1_path);
	DBUG_VOID_RETURN;
}

private int
tb_dup2(tbfd1, tbfd2, expret, experrno)
int tbfd1,	tbfd2;
int expret,	experrno;
{
	int rval;
	int err;
	int	pathok = 0;

	DBUG_LENTER("tb_dup2", 2);

	errno = 0;
	rval = dup2(tbfd1 , tbfd2);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif /* TESTBED */

	err = errno;
	globok = 0;

	if (expret == NOERROR)
	{
		if (rval == tbfd2)
		{
			PATH_FUNC_RPT(0);
			DBUG_RETURN(rval);
		}
	}
	else
	{
		if (rval == expret && err == experrno)
		{
			PATH_FUNC_RPT(0);
			DBUG_RETURN(rval);
		}
	}

	if (!testfail++)
		xx_rpt(FAILURE);

	if (testcomment != (char *) NULL); 
		in_rpt(testcomment);
	in_rpt("dup2(%d , %d) returned incorrect values", tbfd1, tbfd2);
	valrpt(expret, rval, experrno, err);
		
	DBUG_RETURN(rval);
}
