/*
*      SCCS:  @(#)  POSIX.os/ioprim/fcntl/fcntl.c	Rel 4.4.3	    (07/20/98)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/ioprim/fcntl - T.fcntl Rel 4.4.3 (07/20/98)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.fcntl

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fcntl() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Fri Mar 17 16:09:56 GMT 1989
MODIFICATIONS:	Source file split into 4 separate files
		Joseph Nave, UniSoft Ltd.  Tue May 30 13:40:46 BST 1989

		Geoff Clare, 30 Aug - 1 Sep 1989
			Check F_DUPFD gives fd with FD_CLOEXEC clear,
				even if duplicated fd had it set.
			Extra F_SETFL test cases added.
			Add F_GETLK case: lock not overlapping existing ones.
			Bad arguments to tb_fcntl() corrected.
			Improvements to reduce amount of sleeping.
		
		Geoff Clare, 1 Nov 1989
			Fix ld_lck() argument problems.
		
		Geoff Clare, 29 May 1990
			Allow for unlimited OPEN_MAX

		J.A.Nave	08/06/90
			Modified test9 to check the lock type returned
			with a lock specifed by SEEK_SET, SEEK_CUR & SEEK_END
		
		Geoff Clare, 22 June 1990
			Add test that fcntl() cannot alter file access mode.

		Peter Lee, Unisoft Ltd.,
		Mon Aug 06 12:05:00 BST 1990
		Added resilience code.

		Stuart Boutell, 2 Nov 1990
			replaced execl with tet_exec

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Geoff Clare, 23 Mar 1992
			Removed globok definition.

		Geoff Clare, 28 May 1992
			New VSX_FCNTL_EDEADLK parameter.

		Geoff Clare, 25 Sept 1992
			Allow any value != -1 where success expected, with
			all operations except F_DUPFD, F_GETFD and F_GETFL.

		Neil Moses, 7 October 1992
			XNFS enhancements

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

                Geoff Clare, 10 Oct 1997
			Allow for extra (non-X/Open) flags to be
			returned in test 5.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<signal.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<synch.h>
#include	<timeout.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fcntl
#endif


#define EXTERR			"External error"
#define NO_TESTS	45		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEGEN		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

#define T1_NFILES	3
#define T4_NFD		3

#define LOCKBS		40		/* size of buffer */
#define	MODE1		O_RDWR|O_NONBLOCK|O_CREAT|O_APPEND
#define	READ		0
#define	WRITE		1
#define NULLFLK		((struct flock *)NULL)

extern	char	*newstr();
extern	char	*itoa();
extern	void	valrpt();
extern	char	*errname();
extern	char	*execpath();
extern	void	xx_all();

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;

public	int	no_tests = NO_TESTS;

private	char	*fcntl_t03	=	"fcntl_t03";
private	char	*fcntl_t04	=	"fcntl_t04";
private	int		t3_fd;
private	int		t4_fd;
private	int		t9_fd;
private	int		t10_fd;
private	int		t11_fd;
private	synch		*t11_syn_pa_ch;
private	synch		*t11_syn_ch_pa;
private	int		t12_fd;
private	synch		*t12_syn_pa_ch;
private	synch		*t12_syn_ch_pa;

public int	fd_chk_lck;
public int	open_max;
public char	lckbuf[LOCKBS];

private int	all_o_flags;
private int	t4_status[T4_NFD] =	{ O_RDONLY | O_NONBLOCK | O_APPEND,
					  O_RDWR | O_APPEND,
					  O_WRONLY | O_NONBLOCK
					};
private char	*t1_files[T1_NFILES] =	{ "./fcntl-t.1a",
					  "./fcntl-t.1b",
					  "./fcntl-t.1c"
					};

private char	*t2_file =		"./fcntl-t.2";
private char	*t3_file =		"./fcntl-t.3";
private char	*t4_file =		"./fcntl-t.4";
private char	*t5_file =		"./fcntl-t.5";
private char	*t6_file =		"./fcntl-t.6";
private char	*t7_file =		"./fcntl-t.7";
private char	*t8_file =		"./fcntl-t.8";
private char	*t9_file =		"./fcntl-t.9";
private char	*t10_file =		"./fcntl-t.10";
private char	*t11_file =		"./fcntl-t.11";
private char	*t12_file =		"./fcntl-t.12";

private void	prepare_tests();
private void	cln_tests();
private void	twrap();
private void	test1();
private void	test2();
private void	test3();
private void	ch_t3();
private void	test4();
private void	ch_t4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	ch_t9();
private void	test10();
private void	ch_t10();
private void	test11();
private void	ch_t11();
private void	pa_t11();
private void	test12();
private void	ch_t12();
private void	pa_t12();
extern void	test13();
extern void	test14();
extern void	test15();
extern void	test16();
extern void	test17();
extern void	test18();
extern void	test19();
extern void	test20();
extern void	test21();
extern void	test22();
extern void	test23();
extern void	test24();
extern void	test25();
extern void	test26();
extern void	test27();
extern void	test28();
extern void	test29();
extern void	test30();
extern void	test31();
extern void	test32();
extern void	test33();
extern void	test34();
extern void	test35();
extern void	test36();
extern void	test37();
extern void	test38();
extern void	test39();
extern void	test40();
extern void	test41();
extern void	test42();
extern void	test43();
extern void	test44();
extern void	test45();
public char	*pr_slck();
public	int	open_to();
public	int	wrt_to();
public	int	cmp_lcks();
public	void	ld_lck();
public	void	setlk();
public	void	setlkw();
public	void	tstlk();
public	int	fcn_to();

public	int	testfail;
public	char	*testcomment;
public  int	remlock_supp = 1;

private void    (*realfuncs[]) () = { 
	test1,  test2,  test3,  test4,  test5,  test6,  test7,  test8,
	test9,  test10, test11, test12, test13, test14, test15, test16,
	test17, test18, test19, test20, test21, test22, test23, test24,
	test25, test26, test27, test28, test29, test30, test31, test32,
	test33, test34, test35, test36, test37, test38, test39, test40,
	test41, test42, test43, test44, test45,
	NULL 
};

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 twrap, 9,
	 twrap, 10,
	 twrap, 11,
	 twrap, 12,
	 twrap, 13,
	 twrap, 14,
	 twrap, 15,
	 twrap, 16,
	 twrap, 17,
	 twrap, 18,
	 twrap, 19,
	 twrap, 20,
	 twrap, 21,
	 twrap, 22,
	 twrap, 23,
	 twrap, 24,
	 twrap, 25,
	 twrap, 26,
	 twrap, 27,
	 test28, 28,
	 twrap, 29,
	 twrap, 30,
	 twrap, 31,
	 test32, 32,
	 twrap, 33,
	 twrap, 34,
	 twrap, 35,
	 test36, 36,
	 test37, 37,
	 twrap, 38,
	 twrap, 39,
	 twrap, 40,
	 twrap, 41,
	 test42, 42,
	 test43, 43,
	 twrap, 44,
	 twrap, 45,
	NULL, 0
};

private void
prepare_tests()
{
#if TEST_XNFS
	char	*rlock;
#endif
	DBUG_ENTER("prepare_tests");

	/* Mask for all standard open flags */
	all_o_flags = (O_NONBLOCK|O_APPEND|O_CREAT|O_EXCL|O_NOCTTY|O_TRUNC);
	all_o_flags |= (O_RDONLY|O_WRONLY|O_RDWR|O_ACCMODE);
#ifdef O_SYNC
	all_o_flags |= O_SYNC;
#endif
#ifdef O_DSYNC
	all_o_flags |= O_DSYNC;
#endif
#ifdef O_RSYNC
	all_o_flags |= O_RSYNC;
#endif

#if TEST_XNFS
	if ((rlock = tet_getvar("XNFS_REMLOCK_SUPP")) == NULL || *rlock == '\0')
	{
		xx_all(DELETION, "parameter XNFS_REMLOCK_SUPP is not defined");
		DBUG_VOID_RETURN;
	}

	if(*rlock == 'N' || *rlock == 'n')
		remlock_supp = 0;
#endif

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max < 0 && !(open_max == -1 && errno == 0))
	{
		xx_all(DELETION, "sysconf(_SC_OPEN_MAX) failed in prep");
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private void
twrap()
{
	DBUG_ENTER("twrap");

#if TEST_XNFS

	if (!remlock_supp)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("Remote file locking is not supported");
		DBUG_VOID_RETURN;
	}

#endif /* TEST_XNFS */

        if(tet_thistest >= 1 && tet_thistest <= no_tests)
        	(*realfuncs[tet_thistest-1])();

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");
	DBUG_VOID_RETURN;
}


public char *
pr_slck(fl)
	struct flock	*fl;
{
	static char	buf[128];

	(void) sprintf(buf,
		"{l_type=%d, l_whence=%d, l_start=%ld, l_len=%ld, l_pid=%d}",
		fl->l_type, fl->l_whence,
		(long)fl->l_start, (long)fl->l_len, (int)fl->l_pid);

	return buf;
}


private void
test1()
{
	int		fd[T1_NFILES+1];
	int		i;
	int		err;
	int		rval1, rval2;
#if !TEST_XNFS
	int		fildes[2];
	int		fildes_dup;
#endif
	int		statw;
	off_t		offset;
	struct stat	statb1;
	struct stat	statb2;
	static struct flock	flck1,	flck2;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = NOERROR;
	testcomment = "the lowest fd available is returned";

	for (i = 0 ; i < T1_NFILES ; i++)
	{
		if ((fd[i] = open_to(t1_files[i], MODE1, MODEGEN, WAITTIME)) 
			== SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: open(%s, O_RDONLY|O_NONBLOCK|O_CREAT|O_APPEND, %#lo) failed - errno %d(%s)",
				t1_files[i], (unsigned long)MODEGEN, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (remlock_supp && i == 1)
		{
			/* set up lock for later use */

			/* this can't go after the loop, because over NFS the
			   fcntl(F_SETLK) may open some file descriptors */

			flck1.l_type = flck2.l_type = F_WRLCK;
			flck1.l_whence = flck2.l_whence = SEEK_SET;
			flck1.l_start = flck2.l_start = (off_t)0;
			flck1.l_len = flck2.l_len = (off_t)0;
			globok = 0;
			(void) fcn_to(fd[1], F_SETLK, 0, &flck1,
					NOERROR, NOERROR, WAITTIME);
			if (testfail)
			{
				DBUG_VOID_RETURN;
			}
			else
				PATH_FUNC_TRACE;
		}
		else
			PATH_TRACE;
	}

	
	/* set FD_CLOEXEC flag */
	statw = fcntl(fd[1], F_GETFD);
	globok = 0;
	(void) tb_fcntl(fd[1], F_SETFD, statw|FD_CLOEXEC, NULLFLK,
		NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd[0]);
	globok = 0;
	fd[T1_NFILES] = tb_fcntl(fd[1], F_DUPFD, fd[1], NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if(!testfail)
	{

		/* check that the returned fd is the lowest available */
		if (fd[T1_NFILES] != fd[T1_NFILES - 1] + 1
			|| fd[T1_NFILES] <= fd[1])
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("fcntl(F_DUPFD, %d) did not return lowest available fd greater than %d",
				fd[1], fd[1]);
			in_rpt("    returned %d, expected %d",
				fd[T1_NFILES], fd[T1_NFILES-1]+1);
		}
		else
			PATH_TRACE;

		if (fstat(fd[1], &statb1) == SYSERROR
			|| fstat(fd[T1_NFILES], &statb2) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: fstat() failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* check that the same file is refered to */
		testcomment = "the same file is refered to";
		if (statb1.st_dev != statb2.st_dev
			|| statb1.st_ino != statb2.st_ino)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("file descriptor returned by fcntl(F_DUPFD) has different device/inode number");
		}
		else
			PATH_TRACE;

		if (wrt_to(fd[1], lckbuf, sizeof(lckbuf), WAITTIME)
							!= sizeof(lckbuf))
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: write() failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* check that the file pointer is the same */
		if ((offset = lseek(fd[T1_NFILES], (off_t)0, SEEK_CUR))
			!= (long) sizeof(lckbuf))
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("lseek on F_DUPFD'd file descriptor did not return expected value");
			in_rpt("    returned %ld, expected %ld",
				(long)offset, (long)sizeof(lckbuf));
		}
		else
			PATH_TRACE;

		/* check that the file access modes & status flags are the same */
		testcomment = "the file access modes & status flags are the same"; 
		globok = 0;
		rval1 = tb_fcntl(fd[1], F_GETFL, 0, NULLFLK, NOERROR, NOERROR);
		PATH_FUNC_TRACE;
		globok = 0;
		rval2 = tb_fcntl(fd[T1_NFILES], F_GETFL, 0, NULLFLK,
					NOERROR, NOERROR);
		PATH_FUNC_TRACE;

		if (rval1 != rval2)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("fcntl(F_GETFL) returned different values for F_DUPFD'd file descriptor");
			in_rpt("    returned %d, expected %d", rval1, rval2);
		}
		else
			PATH_TRACE;

		/* check the FD_CLOEXEC flag */
		testcomment = "the FD_CLOEXEC flag is clear"; 
		globok = 0;
		rval1 = tb_fcntl(fd[T1_NFILES], F_GETFD, 0, NULLFLK, NOERROR, NOERROR);
		PATH_FUNC_TRACE;

		if (rval1 != SYSERROR && (rval1 & FD_CLOEXEC))
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("FD_CLOEXEC flag was set on file descriptor returned by fcntl(F_DUPFD)");
		}
		else
			PATH_TRACE;

		/* check the locks are the same */
		if (remlock_supp)
		{
			testcomment = "the locks are the same"; 
			globok = 0;
			rval1 = fcn_to(fd[1], F_GETLK, 0, &flck1,
				NOERROR, NOERROR, WAITTIME);
			PATH_FUNC_TRACE;
			globok = 0;
			rval2 = fcn_to(fd[T1_NFILES], F_GETLK, 0, &flck2,
				NOERROR, NOERROR, WAITTIME);
			PATH_FUNC_TRACE;

			if(rval1 != SYSERROR && rval2 != SYSERROR)
			{
				globok = 0;
				(void) cmp_lcks(&flck1 , &flck2);
				PATH_FUNC_TRACE;
			}
		}
		else
		{
			PATH_TRACE;
			PATH_TRACE;
			PATH_TRACE;
		}

#if !TEST_XNFS
		/* check the file status flags for a pipe */
		if (pipe(fildes) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: pipe() failed - errno %d(%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		testcomment = "setting file status flags";
		globok = 0;
		statw = tb_fcntl(fildes[WRITE], F_GETFL, 0, NULLFLK, NOERROR, NOERROR);
		PATH_FUNC_TRACE;
		globok = 0;
		(void) tb_fcntl(fildes[WRITE], F_SETFL, statw|O_NONBLOCK,
			NULLFLK, NOERROR, NOERROR);
		PATH_FUNC_TRACE;

		testcomment = (char *)NULL;
		globok = 0;
		fildes_dup = tb_fcntl(fildes[WRITE], F_DUPFD, fildes[WRITE],
			NULLFLK, NOERROR, NOERROR);
		PATH_FUNC_TRACE;

		if (!testfail)
		{
			testcomment = "the file access modes & status flags are the same"; 
			globok = 0;
			rval1 = tb_fcntl(fildes[WRITE], F_GETFL, 0, NULLFLK, NOERROR, 
				NOERROR);
			PATH_FUNC_TRACE;
			globok = 0;
			rval2 = tb_fcntl(fildes_dup, F_GETFL, 0, NULLFLK, NOERROR,
				NOERROR);
			PATH_FUNC_TRACE;

			if ((rval1 & O_ACCMODE) != (rval2 & O_ACCMODE))
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("fcntl(F_DUPFD) did not duplicate the access mode correctly");
				in_rpt("    returned %d, expected %d",
					(rval2 & O_ACCMODE), (rval1 & O_ACCMODE));
			}
			else
				PATH_TRACE;

			globok = 0;
			rval1 = tb_fcntl(fildes[WRITE], F_GETFL, 0, NULLFLK, NOERROR, 
				NOERROR);
			PATH_FUNC_TRACE;
			globok = 0;
			rval2 = tb_fcntl(fildes_dup, F_GETFL, 0, NULLFLK, NOERROR,
				NOERROR);
			PATH_FUNC_TRACE;
	
			if ((rval1 & O_NONBLOCK) != (rval2 & O_NONBLOCK))
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("fcntl(F_DUPFD) did not duplicate the status flags correctly");
				in_rpt("    returned %d, expected %d",
					(rval2 & O_NONBLOCK),
					(rval1 & O_NONBLOCK));

			}
			else
				PATH_TRACE;
		}
#else /* if TEST_XNFS */
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
		PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
#endif /* TEST_XNFS */
	}

	if(!testfail)
		PATH_XS_RPT(2*T1_NFILES + 26);

	for (i = 1 ; i <= T1_NFILES ; i++)
		if (fd[i] != SYSERROR)
			(void) close(fd[i]);
#if !TEST_XNFS
	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	(void) close(fildes_dup);
#endif
	for (i = 0 ; i < T1_NFILES ; i++)
		(void) unlink(t1_files[i]);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	int		fd;
	int		err;
	int		rval;
	int		fail = 0;
	int	pathok = 0;

	DBUG_ENTER("test2");
	testfail = NOERROR;
	testcomment = (char *)NULL;

	if ((fd = creat(t2_file, MODEGEN)) == SYSERROR)
		fail = 1;
	else
		PATH_TRACE;

	globok = 0;
	rval = tb_fcntl(fd, F_GETFD, 0, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	if (rval & FD_CLOEXEC)
		fail = 2;
	else
		PATH_TRACE;
	(void) close(fd);

	if (fail)
	{
		err = errno;
		xx_rpt(fail == 1 ? DELETION : FAILURE);
		switch (fail)
		{
		case 1:
			in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
				t2_file, (unsigned long)MODEGEN, err, errname(err));
			break;

		case 2:
			in_rpt("close-on-exec flag set on file descriptor returned by creat()");
		}
	}
	else
		if(!testfail)
			PATH_XS_RPT(3);

	if (fd != SYSERROR)
		(void) close(fd);
	(void) unlink(t2_file);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int		err;
	int		rval;
	int		flags;
	int		fail = 0;
	int		pathok = 0;

	DBUG_ENTER("test3");
	testfail = NOERROR;
	testcomment = (char *)NULL;
	if ((t3_fd = creat(t3_file, MODEGEN)) == SYSERROR)
		fail |= 01;
	else
		PATH_TRACE;

	DBUG_PRINTF("test3", "t3_fd = %d", t3_fd);

	flags = fcntl(t3_fd, F_GETFD);
	globok = 0;
	(void) tb_fcntl(t3_fd, F_SETFD, flags|FD_CLOEXEC, NULLFLK,
		NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	rval = tb_fcntl(t3_fd, F_GETFD, 0, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (!(rval & FD_CLOEXEC))
		fail |= 02;
	else
		PATH_TRACE;
	err = errno;
	if (fail)
	{
		xx_rpt(fail & 01 ? DELETION : FAILURE);
		if (fail & 01)
			in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
				t3_file, (unsigned long)MODEGEN, err, errname(err));
		else if (fail & 02)
			in_rpt("fcntl(%d, F_SETFD, FD_CLOEXEC) did not set the close-on-exec flag", t3_fd);
	}
	else
	{
		PATH_FUNC_RPT(4);
		(void) cppair(ch_t3, NULLFN, WAITTIME,0);
	}

	if (t3_fd != SYSERROR)
		(void) close(t3_fd);
	(void) unlink(t3_file);

	DBUG_VOID_RETURN;
}

private void
ch_t3()		/* Assertion 05 */
{
	char	*cfd;
	char 	*args[5];
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	PATH_FUNC_TRACE;	/* check for path check errors in */
					/* calling function */

	if ((cfd = newstr(itoa(t3_fd))) == (char *)NULL)
		DBUG_EXIT(A_NOMEM);

	args[0] = fcntl_t03;
	args[1] = cfd;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = (char *) NULL;
	
	(void) tet_exec(execpath(fcntl_t03), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("tet_exec(\"%s\"...) failed - errno %d(%s)", execpath(fcntl_t03), err, errname(err));
	DBUG_VOID_RETURN;
}

private void
test4()
{
	int		err;
	int		rval;
	int		flags;
	int		fail = 0;
	int	pathok = 0;

	DBUG_ENTER("test4");
	testfail = NOERROR;
	testcomment = (char *)NULL;
	if ((t4_fd = creat(t4_file, MODEGEN)) == SYSERROR)
	{
		fail = 01;
		err = errno;
	}
	else
		PATH_TRACE;

	if (t4_fd != SYSERROR)
	{
		flags = fcntl(t4_fd, F_GETFD);
		globok = 0;
		(void) tb_fcntl(t4_fd, F_SETFD, flags|FD_CLOEXEC, NULLFLK,
			NOERROR, NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		rval = tb_fcntl(t4_fd, F_GETFD, 0, NULLFLK, NOERROR, NOERROR);
		PATH_FUNC_TRACE;

		if (!(rval & FD_CLOEXEC))
			fail = 02;
		else
			PATH_TRACE;

		globok = 0;
		(void) tb_fcntl(t4_fd, F_SETFD, 0, NULLFLK, NOERROR, NOERROR); 
		PATH_FUNC_TRACE;
		globok = 0;
		rval = tb_fcntl(t4_fd, F_GETFD, 0, NULLFLK, NOERROR, NOERROR);
		PATH_FUNC_TRACE;

		if (rval & FD_CLOEXEC)
			fail |= 04;
		else
			PATH_TRACE;
	}

	if (fail)
	{
		xx_rpt(fail <= 03 ? DELETION : FAILURE);
		if (fail & 01)
			in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
				t4_file, (unsigned long)MODEGEN, err, errname(err));
		else if (fail & 02)
			in_rpt("deletion reason: fcntl(%d, F_SETFD, FD_CLOEXEC) did not set the close-on-exec flag", t4_fd);
		if (fail & 040)
			in_rpt("fcntl(%d, F_SETFD, 0) did not clear the close-on-exec flag", t4_fd);
	}
	else
	{
		PATH_FUNC_RPT(7);
		(void) cppair(ch_t4, NULLFN, WAITTIME,0);
	}

	if (t4_fd != SYSERROR)
		(void) close(t4_fd);
	(void) unlink(t4_file);

	DBUG_VOID_RETURN;
}

private void
ch_t4()		/* Assertion 05 */
{
	char	*cfd;
	char	*args[5];
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	PATH_FUNC_TRACE;

	if ((cfd = newstr(itoa(t4_fd))) == (char *)NULL)
		DBUG_EXIT(A_NOMEM);

	args[0] = fcntl_t04;
	args[1] = cfd ;
	args[2] = vg_ident ;
	args[3] = vg_dbug ;
	args[4] = (char *)NULL;

	(void)	tet_exec(execpath(fcntl_t04), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("tet_exec(\"%s\"...) failed - errno %d(%s)", execpath(fcntl_t04), err, errname(err));

	DBUG_VOID_RETURN;
}

private void
test5()
{
	int		i;
	int		fail = 0;
	int		fd[T4_NFD];
	int		flag[T4_NFD];
	int		err[T4_NFD];
	int		rval[T4_NFD];
	int	pathok = 0;

	DBUG_ENTER("test5");
	testfail = NOERROR;
	testcomment = (char *)NULL;
	for (i = 0 ; i < T4_NFD ; i++)
	{
		flag[i] = 0;
		if ((fd[i] = open_to(t5_file, O_CREAT|t4_status[i], MODEGEN,
			WAITTIME)) == SYSERROR)
		{
			err[i] = errno;
			fail |= 1;
			flag[i] = 1;
			(void) unlink(t5_file);
			continue;
		}
		else
			PATH_TRACE;

		globok = 0;
		rval[i] = tb_fcntl(fd[i], F_GETFL, 0, NULLFLK, NOERROR, NOERROR);
		PATH_FUNC_TRACE;

		if (rval[i] == SYSERROR ||
		    (rval[i] & all_o_flags) != t4_status[i])
		{
			fail |= 2;
			flag[i] = 2;
		}
		(void) close(fd[i]);
		(void) unlink(t5_file);
	}

	if (fail)
	{
		xx_rpt(fail == 1 ? DELETION : FAILURE);
		for (i = 0 ; i < T4_NFD ; i++)
			if (flag[i] == 1)
				in_rpt("open(%s, %#o, %#lo) failed - errno %d(%s)",
					t5_file, O_CREAT|t4_status[i],
					(unsigned long)MODEGEN,
					err[i], errname(err[i]));
			else if (flag[i] == 2 && rval[i] != SYSERROR)
			{
				in_rpt("fcntl(fd, F_GETFL, 0) did not return expected flags:");
				in_rpt("returned flags included %#o, file was opened with %#o",
					(rval[i] & all_o_flags), t4_status[i]);
			}
	}
	else
		if(!testfail)
			PATH_XS_RPT(2 * T4_NFD);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	int		fd;
	int		err;
	int		rval;
	int	pathok = 0;

	DBUG_ENTER("test6");
	testfail = NOERROR;
	testcomment = (char *)NULL;
	if ((fd = open_to(t6_file, O_RDWR|O_CREAT, MODEGEN, WAITTIME))
								== SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT) failed - errno %d(%s)",
			t6_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tb_fcntl(fd, F_SETFL, O_NONBLOCK, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	rval = tb_fcntl(fd, F_GETFL, 0, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (rval != (O_NONBLOCK | O_RDWR))
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl(F_SETFL) did not set flags properly, fcntl(F_GETFL) returned %#o, expected %#o", rval, O_NONBLOCK | O_RDWR);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if ((fd = open_to(t6_file, MODE1, MODEGEN, WAITTIME)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT|O_NONBLOCK|O_APPEND) failed - errno %d(%s)",
			t6_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tb_fcntl(fd, F_SETFL, O_NONBLOCK, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	rval = tb_fcntl(fd, F_GETFL, 0, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (rval != (O_NONBLOCK | O_RDWR))
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl(F_SETFL) did not set flags properly, fcntl(F_GETFL) returned %#o, expected %#o", rval, O_NONBLOCK | O_RDWR);
	}
	else
		PATH_TRACE;

	if(!testfail)
		PATH_XS_RPT(8);

	(void) close(fd);
	(void) unlink(t6_file);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	int		fd;
	int		err;
	int		rval;
	int	pathok = 0;

	DBUG_ENTER("test7");
	testfail = NOERROR;
	testcomment = (char *)NULL;
	if ((fd = open_to(t7_file, O_RDWR|O_CREAT, MODEGEN, WAITTIME))
								== SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT) failed - errno %d(%s)",
			t7_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tb_fcntl(fd, F_SETFL, O_APPEND, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	rval = tb_fcntl(fd, F_GETFL, 0, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (rval != (O_APPEND | O_RDWR))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl(F_SETFL) did not set flags properly, fcntl(F_GETFL) returned %#o, expected %#o", rval, O_APPEND | O_RDWR);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if ((fd = open_to(t7_file, MODE1, MODEGEN, WAITTIME)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT|O_NONBLOCK|O_APPEND) failed - errno %d(%s)",
			t7_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tb_fcntl(fd, F_SETFL, O_APPEND, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	rval = tb_fcntl(fd, F_GETFL, 0, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (rval != (O_APPEND | O_RDWR))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl(F_SETFL) did not set flags properly, fcntl(F_GETFL) returned %#o, expected %#o", rval, O_APPEND | O_RDWR);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(8);

	(void) close(fd);
	(void) unlink(t7_file);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	int		fd;
	int		err;
	int		rval;
	int	pathok = 0;

	DBUG_ENTER("test8");
	testfail = NOERROR;
	testcomment = (char *)NULL;
	if ((fd = open_to(t8_file, O_RDWR|O_CREAT, MODEGEN, WAITTIME))
								== SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT) failed - errno %d(%s)",
			t8_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tb_fcntl(fd, F_SETFL, O_APPEND|O_NONBLOCK, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	rval = tb_fcntl(fd, F_GETFL, 0, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (rval != (O_APPEND|O_NONBLOCK | O_RDWR))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl(F_SETFL) did not set flags properly, fcntl(F_GETFL) returned %#o, expected %#o", rval, O_APPEND|O_NONBLOCK | O_RDWR);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if ((fd = open_to(t8_file, MODE1, MODEGEN, WAITTIME)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT|O_NONBLOCK|O_APPEND) failed - errno %d(%s)",
			t8_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tb_fcntl(fd, F_SETFL, 0, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	rval = tb_fcntl(fd, F_GETFL, 0, NULLFLK, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (rval != O_RDWR)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl(F_SETFL) did not set flags properly, fcntl(F_GETFL) returned %#o, expected %#o", rval, O_RDWR);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(8);

	(void) close(fd);
	(void) unlink(t8_file);

	DBUG_VOID_RETURN;
}


private void
test9()
{
	int		err;
	struct flock	flockb;
	int	pathok = 0;

	DBUG_ENTER("test9");

	testfail = NOERROR;
	testcomment = "fcntl(F_GETLK) on write-locked file did not set lock struct fields properly";

	if ((t9_fd = creat(t9_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t9_file, (unsigned long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t9_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&flockb , F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);

	globok = 0;
	(void) fcn_to(t9_fd, F_SETLK, 0, &flockb, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&flockb , F_WRLCK, SEEK_END, -10, 5, (pid_t)0);

	globok = 0;
	(void) fcn_to(t9_fd, F_SETLK, 0, &flockb, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_FUNC_RPT(5);
		(void) cppair(ch_t9 , NULLFN, 5*WAITTIME, 0);
	}

	(void) close(t9_fd);
	(void) unlink(t9_file);
	DBUG_VOID_RETURN;
}

private void
ch_t9()		/* Assertion 11 */
{
	struct flock	flockb[2];
	int	pathok = 0;

	DBUG_ENTER("ch_t9");

	PATH_FUNC_TRACE;

	ld_lck(&flockb[1] , F_RDLCK, SEEK_SET, 0, 37, (pid_t)0);
	globok = 0;
	(void) fcn_to(t9_fd, F_GETLK, 0, &flockb[1], NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (!testfail)
	{
		ld_lck(&flockb[0] , F_WRLCK, SEEK_SET, 10, 15, getppid());
		globok = 0;
		(void) cmp_lcks(&flockb[1] , &flockb[0]);
		PATH_FUNC_TRACE;
	}

	if (lseek(t9_fd, (off_t)20, SEEK_SET) != 20)
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: lseek(fd, 20, SEEK_SET) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&flockb[1] , F_RDLCK, SEEK_CUR, -10, 37, (pid_t)0);
	globok = 0;
	(void) fcn_to(t9_fd, F_GETLK, 0, &flockb[1], NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (!testfail)
	{
		ld_lck(&flockb[0] , F_WRLCK, SEEK_SET, 10, 15, getppid());
		globok = 0;
		(void) cmp_lcks(&flockb[1] , &flockb[0]);
		PATH_FUNC_TRACE;
	}

	ld_lck(&flockb[1] , F_RDLCK, SEEK_END, -30, 37, (pid_t)0);
	globok = 0;
	(void) fcn_to(t9_fd, F_GETLK, 0, &flockb[1], NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (!testfail)
	{
		ld_lck(&flockb[0] , F_WRLCK, SEEK_SET, 10, 15, getppid());
		globok = 0;
		(void) cmp_lcks(&flockb[1] , &flockb[0]);
		PATH_FUNC_TRACE;
	}

	if (!testfail);
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}


private void
test10()
{
	struct flock	flockb[2];
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test10");

	testfail = NOERROR;
	testcomment = "fcntl(F_GETLK) on a newly created file";

	if ((t10_fd = creat(t10_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT, %#lo) failed - errno %d(%s)",
			t10_file, (unsigned long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t10_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t10_fd;

	ld_lck(&flockb[0] , F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t10_fd, F_GETLK, 0, &flockb[0], NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (!testfail)
	{
		ld_lck(&flockb[1] , F_UNLCK, SEEK_SET, 10, 15, (pid_t)0);
		globok = 0;
		(void) cmp_lcks(&flockb[0] , &flockb[1]);
		PATH_FUNC_TRACE;
	}

	testcomment = "fcntl(F_GETLK) where the described lock does not overlap any existing locks";
	if (!testfail)
	{
		ld_lck(&flockb[0], F_WRLCK, SEEK_SET, 5, 5, (pid_t)0);
		ld_lck(&flockb[1], F_WRLCK, SEEK_SET, 15, 5, (pid_t)0);

		globok = 0;
		setlk(&flockb[0], &flockb[1]);
		PATH_FUNC_TRACE;
	}

	if (!testfail)
	{
		PATH_FUNC_RPT(5);
		(void) cppair(ch_t10, NULLFN, WAITTIME, 0);
	}

	(void) close(t10_fd);
	(void) unlink(t10_file);
	DBUG_VOID_RETURN;
}

private	void
ch_t10()
{
	struct flock	flockb[2];
	int	pathok = 0;

	DBUG_ENTER("ch_t10");

	PATH_FUNC_TRACE;

	ld_lck(&flockb[0], F_WRLCK, SEEK_SET, 10, 5, (pid_t)0);
	globok = 0;
	(void) fcn_to(t10_fd, F_GETLK, 0, &flockb[0], NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (!testfail)
	{
		ld_lck(&flockb[1], F_UNLCK, SEEK_SET, 10, 5, (pid_t)0);
		globok = 0;
		(void) cmp_lcks(&flockb[0] , &flockb[1]);
		PATH_FUNC_TRACE;
	}

	if (!testfail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test11()
{

	int		err;
	int	pathok = 0;

	DBUG_ENTER("test11");

	testfail = NOERROR;
	if ((t11_syn_pa_ch = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t11_syn_ch_pa = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t11_fd = creat(t11_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t11_file, (unsigned long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t11_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(t11_fd);
	testcomment = "F_RDLCK not created/detected properly";
	if ((t11_fd = open(t11_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t11_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	fd_chk_lck = t11_fd;
	(void) lseek(t11_fd, (off_t)5, SEEK_SET);

	PATH_FUNC_RPT(5);
	(void) cppair(ch_t11, pa_t11, WAITTIME, 0);

	(void) close(t11_fd);

	(void) unlink(t11_file);
	(void) closesync(t11_syn_pa_ch);
	(void) closesync(t11_syn_ch_pa);
	DBUG_VOID_RETURN;
}

private	void
pa_t11()
{
	int	child_flag;
	struct	flock	lock;
	int	pathok = 0;

	DBUG_ENTER("pa_t11");

	child_flag = 0;

	/* create a new lock */
	ld_lck(&lock, F_RDLCK, SEEK_SET, 5, 5, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t11_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t11_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* create an adjacent lock of the same type */
	ld_lck(&lock, F_RDLCK, SEEK_SET, 10, 5, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t11_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t11_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create an overlapping lock of the same type */
	ld_lck(&lock, F_RDLCK, SEEK_SET, 10, 10, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t11_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t11_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock that wholly contains the old */
	ld_lck(&lock, F_RDLCK, SEEK_SET, 5, 20, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t11_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t11_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock of the opposite type that overlaps */
	ld_lck(&lock, F_WRLCK, SEEK_SET, 20, 10, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t11_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* now use child_flag to pass the current path value to ch_t11 */
		/* by use of the existing pipe */
	child_flag = pathok;
	if (sendsync(t11_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed in path reporting", EXTERR);
		DBUG_VOID_RETURN;
	}
	PATH_FUNC_RPT(19); 
	DBUG_VOID_RETURN;
}


private	void
ch_t11()
{
	struct		flock	test_lock;
	struct		flock	chk_lock;
	int		child_flag;
	int	pathok = 0;

	DBUG_ENTER("ch_t11");

	PATH_FUNC_TRACE;

	/* create a new lock */
	if (waitsync(t11_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 5, 5, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t11_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create an adjacent lock of the same type */
	if (waitsync(t11_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 5, 10, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t11_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create an overlapping lock of the same type */
	if (waitsync(t11_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 5, 15, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t11_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock that wholly contains the old */
	if (waitsync(t11_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 5, 25, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 5, 20, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t11_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock of the opposite type that overlaps */
	if (waitsync(t11_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 5, 25, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 5, 15, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 20, 25, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 20, 10, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;


	/* now check for path check errors in pa_t11 by receiving path */
		/* check value through child_flag */
	if (waitsync(t11_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed in path check reporting", EXTERR);
		DBUG_VOID_RETURN;
	}
	else if (child_flag == 19)
		PATH_TRACE;


	if (!testfail)
		PATH_XS_RPT(22);

	DBUG_VOID_RETURN;
}


private void
test12()
{

	int		err;
	int	pathok = 0;
	DBUG_ENTER("test12");

	testfail = NOERROR;
	if ((t12_syn_pa_ch = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t12_syn_ch_pa = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t12_fd = creat(t12_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t12_file, (unsigned long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t12_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(t12_fd);
	testcomment = "F_RDLCK not created/detected properly";
	if ((t12_fd = open(t12_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t12_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	fd_chk_lck = t12_fd;
	(void) lseek(t12_fd, (off_t)5, SEEK_SET);

	PATH_FUNC_RPT(5);
	(void) cppair(ch_t12, pa_t12, WAITTIME, 0);

	(void) close(t12_fd);

	(void) unlink(t12_file);
	(void) closesync(t12_syn_pa_ch);
	(void) closesync(t12_syn_ch_pa);
	DBUG_VOID_RETURN;
}

private	void
pa_t12()
{
	int	child_flag;
	struct	flock	lock;
	int	pathok = 0;
	DBUG_ENTER("pa_t12");

	child_flag = 0;

	/* create a new lock */
	ld_lck(&lock, F_WRLCK, SEEK_SET, 5, 5, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t12_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t12_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* create an adjacent lock of the same type */
	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 5, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t12_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t12_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create an overlapping lock of the same type */
	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 10, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t12_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t12_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock that wholly contains the old */
	ld_lck(&lock, F_WRLCK, SEEK_SET, 5, 20, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t12_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t12_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock of the opposite type that overlaps */
	ld_lck(&lock, F_RDLCK, SEEK_SET, 20, 10, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t12_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* now use child_flag to pass the current path value to ch_t12 */
		/* by use of the existing pipe */
	child_flag = pathok;
	if (sendsync(t12_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed in path reporting", EXTERR);
		DBUG_VOID_RETURN;
	}
	globok = 1;
	DBUG_VOID_RETURN;
}


private	void
ch_t12()
{
	struct		flock	test_lock;
	struct		flock	chk_lock;
	int		child_flag;
	int	pathok = 0;

	DBUG_ENTER("ch_t12");

	PATH_FUNC_TRACE;

	/* create a new lock */
	if (waitsync(t12_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 5, 5, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t12_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create an adjacent lock of the same type */
	if (waitsync(t12_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 5, 10, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t12_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create an overlapping lock of the same type */
	if (waitsync(t12_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 5, 15, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t12_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock that wholly contains the old */
	if (waitsync(t12_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 5, 25, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 5, 20, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t12_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock of the opposite type that overlaps */
	if (waitsync(t12_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 5, 25, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 5, 15, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 20, 25, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 20, 10, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	/* now check for path check errors in pa_t12 by receiving path */
		/* check value through child_flag */
	if (waitsync(t12_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed in path check reporting", EXTERR);
		DBUG_VOID_RETURN;
	}
	else if (child_flag == 19)
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(22);

	DBUG_VOID_RETURN;
}


public	void
setlk(flck1 , flck2)
struct	flock	*flck1;
struct	flock	*flck2;
{
	int lastfail = testfail;
	int	pathok = 0;

	DBUG_ENTER("setlk");

	globok = 0;
	(void) fcn_to(fd_chk_lck, F_SETLK, 0, flck1, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (testfail == lastfail && flck2 != (struct flock *)NULL)
	{
		globok = 0;
		(void) fcn_to(fd_chk_lck, F_SETLK, 0, flck2, NOERROR, NOERROR, WAITTIME);
		PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

public	void
setlkw(flck1 , flck2)
struct	flock	*flck1;
struct	flock	*flck2;
{
	int lastfail = testfail;

	DBUG_ENTER("setlkw");

	(void) fcn_to(fd_chk_lck, F_SETLKW, 0, flck1, NOERROR, NOERROR, WAITTIME);

	if (testfail == lastfail && flck2 != (struct flock *)NULL)
	{
		(void) fcn_to(fd_chk_lck, F_SETLKW, 0, flck2, NOERROR, NOERROR, WAITTIME);
	}
	DBUG_VOID_RETURN;
}

public	void
tstlk(flck1 , flck2)
struct	flock	*flck1;
struct	flock	*flck2;
{

	int	lastfail = testfail;
	int	pathok = 0;

	DBUG_ENTER("tstlk");

	globok = 0;
	(void) fcn_to(fd_chk_lck, F_GETLK, 0, flck1, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if(testfail > lastfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) cmp_lcks(flck1, flck2);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}


public	int
open_to(path , oflag , prot , wait_time)
char	*path;
int	oflag;
mode_t	prot;
int	wait_time;
{
	int	rval;

	DBUG_ENTER("open_to");

	SET_TIMEOUT(wait_time)

	rval = open(path , oflag , prot);

	CLEAR_ALARM

	DBUG_RETURN(rval);
}

public	int
wrt_to(fildes , buf , nbyte , wait_time)
int	fildes;
char	*buf;
unsigned	nbyte;
int	wait_time;
{
	int	rval;

	DBUG_ENTER("wrt_to");

	SET_TIMEOUT(wait_time)

	rval = write(fildes , buf , nbyte);

	CLEAR_ALARM

	DBUG_RETURN(rval);
}


public	void
ld_lck(flck , type , whence , start , len , pid)
struct	flock	*flck;
int	type;
int	whence;
int	start;
int	len;
pid_t	pid;
{

	DBUG_ENTER("ld_lck");

	flck->l_type = type;
	flck->l_whence = whence;
	flck->l_start = start;
	flck->l_len = len;
	flck->l_pid = pid;

	DBUG_VOID_RETURN;
}

public	int
cmp_lcks(flck1 , flck2)
struct	flock	*flck1;
struct	flock	*flck2;
{

	int	pathok = 0;

	DBUG_ENTER("cmp_lcks");

	if ((flck1->l_type != flck2->l_type) ||
		(flck1->l_whence != flck2->l_whence) ||
		(flck1->l_start != flck2->l_start) ||
		(flck1->l_len != flck2->l_len) ||
		(flck1->l_pid != flck2->l_pid))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		if (testcomment != (char *)NULL)
			in_rpt(testcomment);
	}
	else
		PATH_TRACE;

	if (flck1->l_type != flck2->l_type)
		in_rpt("lock types differ: got %d - expected %d",
			flck1->l_type,flck2->l_type);

	if (flck1->l_whence != flck2->l_whence)
		in_rpt("lock start indicators differ: got %d - expected %d",
			flck1->l_whence,flck2->l_whence);

	if (flck1->l_start != flck2->l_start)
		in_rpt("lock start points differ: got %d - expected %d",
			flck1->l_start,flck2->l_start);

	if (flck1->l_len != flck2->l_len)
		in_rpt("lock lengths differ: got %d - expected %d",
			flck1->l_len,flck2->l_len);

	if (flck1->l_pid != flck2->l_pid)
		in_rpt("lock process ids differ: got %ld - expected %ld",
			(long)flck1->l_pid,(long)flck2->l_pid);

	globok = (pathok == 1) ? 1 : 0;

	DBUG_RETURN((testfail == 0) ? NOERROR : SYSERROR);
}


public	int
fcn_to(fildes, cmd, opt, lck, expret, experrno, wait_time)
int	fildes;
int	cmd;
int	opt;
struct flock *lck;
int	expret;
int	experrno;
int	wait_time;
{

	int lastfail = testfail;
	int ret;

	DBUG_ENTER("fcn_to");

	SET_TIMEOUT(wait_time)

	ret = tb_fcntl(fildes, cmd, opt, lck, expret, experrno);

	CLEAR_ALARM

	if (testfail > lastfail && alrm_flag > 0 && experrno != EINTR)
		in_rpt("fcntl() call timed out");

	DBUG_RETURN(ret);
}

public	int
tb_fcntl(tbfd, cmd, opt, lck, expret, experrno)
int	tbfd;
int	cmd;
int	opt;
struct flock *lck;
int	expret;
int	experrno;
{
	int	rval;
	int	err;
	char	*cmd_str;
	char	*arg_str = "lockptr";
	int	pathok = 0;

	errno = NOERROR;
	switch(cmd)
	{
	case F_GETLK:
	case F_SETLK:
	case F_SETLKW:
		rval = fcntl(tbfd, cmd, lck);
		break;

	default:
		rval = fcntl(tbfd, cmd, opt);
		arg_str = itoa(opt);
		break;
	}

#ifdef	TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif

	PATH_TRACE;

	switch(cmd)
	{
		case	F_DUPFD:
				cmd_str = "F_DUPFD";
				break;
		case	F_GETFD:
				cmd_str = "F_GETFD";
				break;
		case	F_SETFD:
				cmd_str = "F_SETFD";
				break;
		case	F_GETFL:
				cmd_str = "F_GETFL";
				break;
		case	F_SETFL:
				cmd_str = "F_SETFL";
				break;
		case	F_GETLK:
				cmd_str = "F_GETLK";
				break;
		case	F_SETLK:
				cmd_str = "F_SETLK";
				break;
		case	F_SETLKW:
				cmd_str = "F_SETLKW";
				break;
		default:
				cmd_str = "unknown cmd value";
				break;
	}


	globok = 0;
	err = errno;
	if (expret == NOERROR && cmd == F_DUPFD)
	{
	/* duplicated fds must be distinct and 0 <= fd < OPEN_MAX */
		if (rval != tbfd && rval >= 0 &&
				(rval < open_max || open_max == -1))
		{
			globok = 1;
			return(rval);
		}

		if( !testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl(%d, %s, %s) returned %d",
			tbfd, cmd_str, arg_str, rval);
		in_rpt("expected value between 0 and OPEN_MAX-1, and != %d",
			tbfd);
		if (rval == -1)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (expret == NOERROR && (cmd == F_GETFD || cmd == F_GETFL))
	{
	/* file descriptor flags and file status flags + access modes 
	 * must be >= 0
	 */
		if (rval >= 0)
		{
			globok = 1;
			return(rval);
		}

		if( !testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl(%d, %s, %s) returned %d, expected >= 0",
			tbfd, cmd_str, arg_str, rval);
		if (rval == -1)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else if (expret == NOERROR)
	{
	/* allow any value not equal to -1 */
		if (rval != SYSERROR)
		{
			globok = 1;
			return(rval);
		}

		if( !testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl(%d, %s, %s) returned %d, expected != -1",
			tbfd, cmd_str, arg_str, rval);
		in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
	{
	/* Allow EAGAIN when EACCES expected *
         * Allow success when ENOLCK expected */
		if ((rval == expret && err == experrno) || 
		    (experrno == EACCES && rval == SYSERROR && err == EAGAIN) ||
		    (experrno == ENOLCK && rval != SYSERROR))
		{
			globok = 1;
			return(rval);
		}

		if( !testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl(%d, %s, %s) did not return expected values",
			tbfd, cmd_str, arg_str);
		valrpt(expret,rval, experrno, err);
	}

	switch(cmd)
	{
		case	F_GETLK:
		case	F_SETLK:
		case	F_SETLKW:
				in_rpt("lock values: %s",pr_slck(lck));
	}

	if (cmd == F_DUPFD && expret == SYSERROR && rval > 0)
		(void) close(rval);

	return(rval);

}
