/*
*      SCCS:  @(#)  POSIX.os/ioprim/fcntl/fcntl1.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/ioprim/fcntl - T.fcntl Rel 4.4.1 (06/09/97)";
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		fcntl1.c

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fcntl() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Tue May 30 13:42:16 BST 1989
MODIFICATIONS:	
		Geoff Clare, 22 Aug 1989
			SETLKW blocking test should set up an exclusive lock
			for each test case.
		
		Geoff Clare, 30 Aug - 1 Sep 1989
			Bad arguments to tb_fcntl() corrected
			Improvements to reduce amount of sleeping

		Peter Lee, Unisoft Ltd.,
		Tue Aug 07 11:37:27 BST 1990
		Added resilience code.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<synch.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fcntl
#endif


#define EXTERR			"External error"
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEGEN		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

#define LOCKBS		40		/* size of buffer */
#define	READ		0
#define	WRITE		1

extern	char	*errname();

private	int		t13_fd;
private	synch		*t13_syn_pa_ch;
private	synch		*t13_syn_ch_pa;
private	int		t14_fd;
private	int		t15_fd;
private	synch		*t15_syn_pa_ch;
private	synch		*t15_syn_ch_pa;
private	int		t16_fd;
private	synch		*t16_syn_pa_ch;
private	synch		*t16_syn_ch_pa;
private	int		t17_fd;
private	synch		*t17_syn_pa_ch;
private	synch		*t17_syn_ch_pa;
private	int		t18_fd;
private	synch		*t18_syn_pa_ch;
private	synch		*t18_syn_ch_pa;

extern	int	fd_chk_lck;
extern	char	lckbuf[LOCKBS];

private char	*t13_file =		"./fcntl-t.13";
private char	*t14_file =		"./fcntl-t.14";
private char	*t15_file =		"./fcntl-t.15";
private char	*t16_file =		"./fcntl-t.16";
private char	*t17_file =		"./fcntl-t.17";
private char	*t18_file =		"./fcntl-t.18";

public	void	test13();
private void	ch_t13();
private void	pa_t13();
public	void	test14();
private void	ch_t14();
public	void	test15();
private void	ch_t15();
private void	pa_t15();
public	void	test16();
private void	ch_t16();
private void	pa_t16();
public	void	test17();
private	void	ch_t17();
private	void	pa_t17();
public	void	test18();
private	void	ch_t18();
private	void	pa_t18();
extern	char	*pr_slck();
extern	int	open_to();
extern	int	wrt_to();
extern	int	cmp_lcks();
extern	void	ld_lck();
extern	void	setlk();
extern	void	setlkw();
extern	void	tstlk();
extern	int	fcn_to();

extern	int	testfail;
extern	char	*testcomment;


public void
test13()
{

	int		err;
	int	pathok = 0;
	DBUG_ENTER("test13");

	testfail = NOERROR;
	if ((t13_syn_pa_ch = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t13_syn_ch_pa = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t13_fd = creat(t13_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t13_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t13_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t13_fd);
	testcomment = "F_UNLCK not removed properly";
	if ((t13_fd = open(t13_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t13_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	fd_chk_lck = t13_fd;
	(void) lseek(t13_fd, (off_t)5, SEEK_SET);

	PATH_FUNC_RPT(5);
	(void) cppair(ch_t13, pa_t13, WAITTIME, 0);

	(void) close(t13_fd);

	(void) unlink(t13_file);
	(void) closesync(t13_syn_ch_pa);
	(void) closesync(t13_syn_pa_ch);
	DBUG_VOID_RETURN;
}

private	void
pa_t13()
{
	int	child_flag;
	struct	flock	lock1,	lock2;
	int	pathok = 0;
	DBUG_ENTER("pa_t13");

	/* locks wholly within the requested segement are removed */
	child_flag = 0;
	ld_lck(&lock1, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	ld_lck(&lock2, F_UNLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	setlk(&lock1 , &lock2);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t13_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* locks partially within the requested segment are modified */
	if (waitsync(t13_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&lock1, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	ld_lck(&lock2, F_UNLCK, SEEK_SET, 20, 15, (pid_t)0);
	globok = 0;
	setlk(&lock1 , &lock2);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t13_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* locks partially within the requested segment are modified */
	if (waitsync(t13_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&lock1, F_UNLCK, SEEK_SET, 12, 6, (pid_t)0);
	globok = 0;
	setlk(&lock1 , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t13_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* now use child_flag to pass the current path value to ch_t13 */
		/* by use of the existing pipe */
	child_flag = pathok;
	if (sendsync(t13_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed in path reporting", EXTERR);
		DBUG_VOID_RETURN;
	}
	globok = 1; /* Needed by cppair */

	DBUG_VOID_RETURN;
}


private	void
ch_t13()
{
	struct		flock	test_lock;
	struct		flock	chk_lock;
	int		child_flag;
	int	pathok = 0;

	DBUG_ENTER("ch_t13");

	PATH_FUNC_TRACE;

	/* locks wholly within the requested segement are removed */
	if (waitsync(t13_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_WRLCK, SEEK_END, -35, 20, (pid_t)0);
	ld_lck(&chk_lock , F_UNLCK, SEEK_END, -35, 20, (pid_t)0);

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;
	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (sendsync(t13_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* locks partially within the requested segment are modified */
	if (waitsync(t13_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 10, 10, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 10, 10, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (sendsync(t13_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* lock split into two by requested segment being unlocked */
	if (waitsync(t13_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check first lock */
	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 10, 2, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 10, 2, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if(testfail)
		DBUG_VOID_RETURN;
	/* check second lock */
	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 18, 2, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 18, 2, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	/* now check for path check errors in pa_t13 by receiving path */
		/* check value through child_flag */
	if (waitsync(t13_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed in path check reporting", EXTERR);
		DBUG_VOID_RETURN;
	}
	else if (child_flag == 11)
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(16);

	DBUG_VOID_RETURN;
}

public void
test14()
{
	struct	flock	test_lock;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test14");

	testfail = NOERROR;
	if ((t14_fd = creat(t14_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t14_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t14_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(t14_fd);
	testcomment = "fcntl(fd, F_SETLK, arg) does not wait if lock cannot be applied";
	if ((t14_fd = open(t14_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t14_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	fd_chk_lck = t14_fd;
	(void) lseek(t14_fd, (off_t)5, SEEK_SET);
	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t14_fd, F_SETLK, 0, &test_lock, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	(void) cppair(ch_t14, NULLFN, 3*WAITTIME ,0);

	(void) close(t14_fd);

	if (!testfail)
		PATH_XS_RPT(4);

	(void) unlink(t14_file);
	DBUG_VOID_RETURN;
}

private	void
ch_t14()
{
	struct	flock	lock;

	DBUG_ENTER("ch_t14");

	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	(void) fcn_to(t14_fd , F_SETLK, 0, &lock, SYSERROR, EACCES, WAITTIME);

	ld_lck(&lock, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);
	(void) fcn_to(t14_fd , F_SETLK, 0, &lock, SYSERROR, EACCES, WAITTIME);

	
	DBUG_VOID_RETURN;
}

public void
test15()
{
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test15");

	testfail = NOERROR;
	if ((t15_syn_pa_ch = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t15_syn_ch_pa = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t15_fd = creat(t15_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t15_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t15_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(t15_fd);
	testcomment = "F_RDLCK not created/detected properly";
	if ((t15_fd = open(t15_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t15_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t15_fd;
	(void) lseek(t15_fd, (off_t)5, SEEK_SET);

	PATH_FUNC_RPT(5);
	(void) cppair(ch_t15, pa_t15, WAITTIME, 0);

	(void) close(t15_fd);

	(void) unlink(t15_file);
	(void) closesync(t15_syn_pa_ch);
	(void) closesync(t15_syn_ch_pa);
	DBUG_VOID_RETURN;
}

private	void
pa_t15()
{
	int	child_flag;
	struct	flock	lock;
	int	pathok = 0;
	DBUG_ENTER("pa_t15");

	child_flag = 0;

	/* create a new lock */
	ld_lck(&lock, F_RDLCK, SEEK_SET, 5, 5, (pid_t)0);
	setlkw(&lock , (struct flock *)NULL);

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t15_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t15_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* create an adjacent lock of the same type */
	ld_lck(&lock, F_RDLCK, SEEK_SET, 10, 5, (pid_t)0);
	setlkw(&lock , (struct flock *)NULL);

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t15_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t15_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create an overlapping lock of the same type */
	ld_lck(&lock, F_RDLCK, SEEK_SET, 10, 10, (pid_t)0);
	setlkw(&lock , (struct flock *)NULL);

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t15_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t15_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock that wholly contains the old */
	ld_lck(&lock, F_RDLCK, SEEK_SET, 5, 20, (pid_t)0);
	setlkw(&lock , (struct flock *)NULL);

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t15_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (waitsync(t15_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock of the opposite type that overlaps */
	ld_lck(&lock, F_WRLCK, SEEK_SET, 20, 10, (pid_t)0);
	setlkw(&lock , (struct flock *)NULL);

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t15_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* now use child_flag to pass the current path value to ch_t15 */
		/* by use of the existing pipe */
	child_flag = pathok;
	if (sendsync(t15_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed in path reporting", EXTERR);
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}


private	void
ch_t15()
{
	struct		flock	test_lock;
	struct		flock	chk_lock;
	int		child_flag;
	int	pathok = 0;

	DBUG_ENTER("ch_t15");

	PATH_FUNC_TRACE;

	/* create a new lock */
	if (waitsync(t15_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 5, 5, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t15_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create an adjacent lock of the same type */
	if (waitsync(t15_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 5, 10, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t15_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create an overlapping lock of the same type */
	if (waitsync(t15_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 5, 15, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t15_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock that wholly contains the old */
	if (waitsync(t15_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 5, 25, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 5, 20, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t15_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock of the opposite type that overlaps */
	if (waitsync(t15_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 5, 25, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 5, 15, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 20, 25, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 20, 10, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	/* now check for path check errors in pa_t15 by receiving path */
		/* check value through child_flag */
	if (waitsync(t15_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed in path check reporting", EXTERR);
		DBUG_VOID_RETURN;
	}
	else if (child_flag == 14)
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(22);

	DBUG_VOID_RETURN;
}


public void
test16()
{
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test16");

	testfail = NOERROR;
	if ((t16_syn_pa_ch = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t16_syn_ch_pa = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t16_fd = creat(t16_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t16_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (wrt_to(t16_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(t16_fd);
	testcomment = "F_RDLCK not created/detected properly";

	if ((t16_fd = open(t16_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t16_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t16_fd;
	(void) lseek(t16_fd, (off_t)5, SEEK_SET);

	PATH_FUNC_RPT(5);
	(void) cppair(ch_t16, pa_t16, WAITTIME, 0);

	(void) close(t16_fd);

	(void) unlink(t16_file);
	(void) closesync(t16_syn_pa_ch);
	(void) closesync(t16_syn_ch_pa);
	DBUG_VOID_RETURN;
}

private	void
pa_t16()
{
	int	child_flag;
	struct	flock	lock;
	int	pathok = 0;

	DBUG_ENTER("pa_t16");

	child_flag = 0;

	/* create a new lock */
	ld_lck(&lock, F_WRLCK, SEEK_SET, 5, 5, (pid_t)0);
	setlkw(&lock , (struct flock *)NULL);

	if (testfail)
	{
		child_flag++;
	}
	else
		PATH_TRACE;

	if (sendsync(t16_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t16_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* create an adjacent lock of the same type */
	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 5, (pid_t)0);
	setlkw(&lock , (struct flock *)NULL);

	if (testfail)
	{
		child_flag++;
	}
	else
		PATH_TRACE;

	if (sendsync(t16_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t16_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create an overlapping lock of the same type */
	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 10, (pid_t)0);
	setlkw(&lock , (struct flock *)NULL);

	if (testfail)
	{
		child_flag++;
	}
	else
		PATH_TRACE;

	if (sendsync(t16_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t16_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock that wholly contains the old */
	ld_lck(&lock, F_WRLCK, SEEK_SET, 5, 20, (pid_t)0);
	setlkw(&lock , (struct flock *)NULL);

	if (testfail)
	{
		child_flag++;
	}
	else
		PATH_TRACE;

	if (sendsync(t16_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t16_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock of the opposite type that overlaps */
	ld_lck(&lock, F_RDLCK, SEEK_SET, 20, 10, (pid_t)0);
	setlkw(&lock , (struct flock *)NULL);

	if (testfail)
	{
		child_flag++;
	}
	else
		PATH_TRACE;

	if (sendsync(t16_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* now use child_flag to pass the current path value to ch_t16 */
		/* by use of the existing pipe */
	child_flag = pathok;
	if (sendsync(t16_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed in path reporting", EXTERR);
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}


private	void
ch_t16()
{
	struct		flock	test_lock;
	struct		flock	chk_lock;
	int		child_flag;
	int	pathok = 0;

	DBUG_ENTER("ch_t16");

	PATH_FUNC_TRACE;

	/* create a new lock */
	if (waitsync(t16_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 5, 5, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t16_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create an adjacent lock of the same type */
	if (waitsync(t16_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 5, 10, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t16_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create an overlapping lock of the same type */
	if (waitsync(t16_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 5, 15, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t16_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock that wholly contains the old */
	if (waitsync(t16_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 5, 25, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 5, 20, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (sendsync(t16_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* create a new lock of the opposite type that overlaps */
	if (waitsync(t16_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 5, 25, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 5, 15, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 20, 25, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 20, 10, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	/* now check for path check errors in pa_t16 by receiving path */
		/* check value through child_flag */
	if (waitsync(t16_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed in path check reporting", EXTERR);
		DBUG_VOID_RETURN;
	}
	else if (child_flag == 14)
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(22);

	DBUG_VOID_RETURN;
}

public void
test17()
{
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test17");

	testfail = NOERROR;
	if ((t17_syn_pa_ch = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t17_syn_ch_pa = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t17_fd = creat(t17_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t17_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t17_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(t17_fd);
	testcomment = "F_UNLCK not removed properly";
	if ((t17_fd = open(t17_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t17_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t17_fd;
	(void) lseek(t17_fd, (off_t)5, SEEK_SET);

	PATH_FUNC_RPT(6);
	(void) cppair(ch_t17, pa_t17, WAITTIME, 0);

	(void) close(t17_fd);

	(void) unlink(t17_file);
	(void) closesync(t17_syn_ch_pa);
	(void) closesync(t17_syn_pa_ch);
	DBUG_VOID_RETURN;
}

private	void
pa_t17()
{
	int	child_flag;
	struct	flock	lock1,	lock2;
	int	pathok = 0;
	DBUG_ENTER("pa_t17");

	/* locks wholly within the requested segement are removed */
	child_flag = 0;
	ld_lck(&lock1, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	ld_lck(&lock2, F_UNLCK, SEEK_SET, 10, 15, (pid_t)0);
	setlkw(&lock1 , &lock2);

	if (testfail)
	{
		child_flag++;
	}
	else
		PATH_TRACE;

	if (sendsync(t17_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* locks partially within the requested segment are modified */
	if (waitsync(t17_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&lock1, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	ld_lck(&lock2, F_UNLCK, SEEK_SET, 20, 15, (pid_t)0);
	setlkw(&lock1 , &lock2);

	if (testfail)
	{
		child_flag++;
	}
	else
		PATH_TRACE;

	if (sendsync(t17_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}

	else
		PATH_TRACE;

	/* locks partially within the requested segment are modified */
	if (waitsync(t17_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&lock1, F_UNLCK, SEEK_SET, 12, 6, (pid_t)0);
	setlkw(&lock1 , (struct flock *)NULL);

	if (testfail)
	{
		child_flag++;
	}
	else
		PATH_TRACE;

	if (sendsync(t17_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* now use child_flag to pass the current path value to ch_t17 */
		/* by use of the existing pipe */
	child_flag = pathok;
	if (sendsync(t17_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed in path reporting", EXTERR);
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}


private	void
ch_t17()
{
	struct		flock	test_lock;
	struct		flock	chk_lock;
	int		child_flag;
	int	pathok = 0;

	DBUG_ENTER("ch_t17");

	/* locks wholly within the requested segement are removed */
	if (waitsync(t17_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_WRLCK, SEEK_END, -35, 20, (pid_t)0);
	ld_lck(&chk_lock , F_UNLCK, SEEK_END, -35, 20, (pid_t)0);

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (sendsync(t17_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* locks partially within the requested segment are modified */
	if (waitsync(t17_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 10, 10, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 10, 10, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (sendsync(t17_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* lock split into two by requested segment being unlocked */
	if (waitsync(t17_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check first lock */
	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 10, 2, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 10, 2, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if(testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	/* check second lock */
	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 18, 2, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 18, 2, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	/* now check for path check errors in pa_t17 by receiving path */
		/* check value through child_flag */
	if (waitsync(t17_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed in path check reporting", EXTERR);
		DBUG_VOID_RETURN;
	}
	else if (child_flag == 8)
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(15);

	DBUG_VOID_RETURN;
}

public void
test18()
{
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test18");

	testfail = NOERROR;
	if ((t18_syn_pa_ch = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t18_syn_ch_pa = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t18_fd = creat(t18_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t18_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t18_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(t18_fd);
	testcomment = "fcntl( , F_SETLKW, ..) will wait until the request can be fullfilled";
	if ((t18_fd = open(t18_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t18_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t18_fd;
	(void) lseek(t18_fd, (off_t)5, SEEK_SET);

	PATH_FUNC_RPT(5);
	(void) cppair(ch_t18, pa_t18, WAITTIME, 0);

	(void) close(t18_fd);

	(void) unlink(t18_file);
	(void) closesync(t18_syn_ch_pa);
	(void) closesync(t18_syn_pa_ch);
	DBUG_VOID_RETURN;
}

private	void
pa_t18()
{
	int	child_flag;
	struct	flock	lock1;
	int	pathok = 0;
	DBUG_ENTER("pa_t18");

	/* F_SETLKW on an already locked segment will time out */
	if (waitsync(t18_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&lock1, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	(void) fcn_to(t18_fd , F_SETLKW, 0, &lock1, SYSERROR, EINTR, WAITTIME);

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t18_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* once the segment has been unlocked it may then be locked */
	if (waitsync(t18_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&lock1, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	(void) fcn_to(t18_fd , F_SETLKW, 0, &lock1, NOERROR, NOERROR, WAITTIME);
	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t18_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* remove the current lock */
	if (waitsync(t18_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ld_lck(&lock1, F_UNLCK, SEEK_SET, 10, 15, (pid_t)0);

	globok = 0;
	(void) fcn_to(t18_fd, F_SETLK, 0, &lock1, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;
	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t18_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* F_SETLKW on an already locked segment will time out */
	if (waitsync(t18_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&lock1, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);
	(void) fcn_to(t18_fd , F_SETLKW, 0, &lock1, SYSERROR, EINTR, WAITTIME);

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t18_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* once the segment has been unlocked it may then be locked */
	if (waitsync(t18_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&lock1, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);
	(void) fcn_to(t18_fd , F_SETLKW, 0, &lock1, NOERROR, NOERROR, WAITTIME);
	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t18_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* now use child_flag to pass the current path value to ch_t18 */
		/* by use of the existing pipe */
	child_flag = pathok;
	if (sendsync(t18_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed in path reporting", EXTERR);
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}


private	void
ch_t18()
{
	struct		flock	test_lock;
	struct		flock	chk_lock;
	int		child_flag;
	int	pathok = 0;

	DBUG_ENTER("ch_t18");

	PATH_FUNC_TRACE;

	/* F_SETLKW on an already locked segment will time out */
	child_flag = 0;
	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	setlk(&test_lock , (struct flock *)NULL);

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (sendsync(t18_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t18_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), 
		2*WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock, F_UNLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t18_fd, F_SETLK, 0, &test_lock, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (sendsync(t18_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* once the segment has been unlocked it may then be locked */
	if (waitsync(t18_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), 
		2*WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock , F_RDLCK, SEEK_SET, 10, 20, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 10, 15, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* wait for current lock to be removed */
	if (sendsync(t18_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t18_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* F_SETLKW on an already locked segment will time out */
	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	setlk(&test_lock , (struct flock *)NULL);

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (sendsync(t18_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t18_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), 
		2*WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock, F_UNLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t18_fd, F_SETLK, 0, &test_lock, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (sendsync(t18_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* once the segment has been unlocked it may then be locked */
	if (waitsync(t18_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), 
		2*WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		testfail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock , F_WRLCK, SEEK_SET, 10, 20, (pid_t)0);
	ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 10, 15, getppid());

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	/* now check for path check errors in pa_t18 by receiving path */
		/* check value through child_flag */
	if (waitsync(t18_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed in path check reporting", EXTERR);
		DBUG_VOID_RETURN;
	}
	else if (child_flag == 16)
		PATH_TRACE;
	
	if (!testfail)
		PATH_XS_RPT(24);

	DBUG_VOID_RETURN;
}

