/*
*      SCCS:  @(#)  POSIX.os/ioprim/fcntl/fcntl2.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/ioprim/fcntl - T.fcntl Rel 4.4.1 (06/09/97)";
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		fcntl2.c

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fcntl() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Tue May 30 13:42:57 BST 1989
MODIFICATIONS:	
		Geoff Clare, 30 Aug 1989
			Bad arguments to tb_fcntl() corrected
		
		Geoff Clare, 1 Sep 1989
			Improvements to reduce amount of sleeping
			Test that closing removes locks must be
				done with two processes
		
		Geoff Clare, 1 Nov 1989
			Improved strategy for "locks not inherited" test

		J.A.Nave	20/06/90
			New test to check that when a new lock is applied
			it will replace the existing lock. (test27)
		
		Geoff Clare, 22 June 1990
			Add test that fcntl() cannot change file access mode

		Peter Lee, Unisoft Ltd.,
		Mon Aug 06 15:54:59 BST 1990
		Added resilience code.

		Stuart Boutell, 27 Mar 1991
			Upgraded test 23 to test F_RDLCK as well.
		
		Geoff Clare, 20 May 1992
			Add check to test28 that O_CREAT, O_TRUNC, O_EXCL
			and O_NOCTTY are ignored.
		
		Geoff Clare, 8 Sept 1992
			In ctty_sig() check signal has not already
			arrived before calling pause().

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<termios.h>
#include	<signal.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<timeout.h>
#include	<synch.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fcntl
#endif


#define EXTERR			"External error"
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEGEN		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define BADFD		-1		/* a bad file descriptor */
#define NOK		02
#define E_PATH		04

#define LOCKBS		40		/* size of buffer */
#define	READ		0
#define	WRITE		1
#define NULLFLK		((struct flock *)NULL)

#define	T23_RDLCK_MODE  1
#define	T23_WRLCK_MODE  2

extern	char	*errname();

private	int		t19_fd;
private	int		t20_fd;
private	int		t21_fd;
private	int		t22_fd;
private	int		t23_mode;
private	int		t23_fd;
private	int		t24_fd;
private	synch		*t23_syn_pa_ch;
private	synch		*t23_syn_ch_pa;
private	int		t25_fd;
private	int		t26_fd;
private	synch		*t26_syn;
private	int		t27_itter;
private	int		t27_fildes;
private	int		t27_fds[2];
private	int		t29_fd;
private	int		t30_fd;
private	int		t31_fd;
private	int		t35_fd;

extern	int	globok;
extern	int	fd_chk_lck;
extern	char	lckbuf[LOCKBS];

private char	*t19_file =		"./fcntl-t.19";
private char	*t20_file =		"./fcntl-t.20";
private char	*t21_file =		"./fcntl-t.21";
private char	*t22_file =		"./fcntl-t.22";
private char	*t23_file =		"./fcntl-t.23";
private char	*t24_file =		"./fcntl-t.24";
private char	*t25_file =		"./fcntl-t.25";
private char	*t26_file =		"./fcntl-t.26";
private char	*t27_file =		"./fcntl-t.27";
private char	*t28_file =		"./fcntl-t.28";
private char	*t29_file =		"./fcntl-t.29";
private char	*t30_file =		"./fcntl-t.30";
private char	*t31_file =		"./fcntl-t.31";
#if !TEST_XNFS
private char	*t32_file =		"./fcntl-t.32";
#endif
private char	*t33_file =		"./fcntl-t.33";
private char	*t34_file =		"./fcntl-t.34";
private char	*t35_file =		"./fcntl-t.35";

private	void	ch_t19();
private	void	ch_t20();
private	void	ch_t21();
private	void	ch_t22();
private	void	ch_t23();
private	void	pa_t23();
private	void	ch_t24();
private	void	ch_t25();
private	void	ch_t26();
private	void	pa_t26();
private	void	ch_t27();
private	void	pa_t27();
private	void	ch_t28();
private	void	t28sub();
private	void	ch_t29();
private	void	ch_t30();
private	void	ch_t31();
private	void	ch_t35();
private	int	ctty_sig();
private	void	ctty_post();
private	void	sig_establish();
private	int	tty_fd;
#if !TEST_XNFS
private	struct	termios	tty_term,	*tty_term_p = &tty_term;
private	int	loop_fd;
private	struct	termios	loop_term;
private	int	sigint_posted = FALSE;
private	int	gti_supported;
#endif /* TEST_XNFS */

extern	char	*pr_slck();
extern	int	open_to();
extern	int	wrt_to();
extern	int	cmp_lcks();
extern	void	ld_lck();
extern	void	setlk();
extern	void	setlkw();
extern	void	tstlk();
extern	int	fcn_to();

extern	int	testfail;
extern	char	*testcomment;


typedef	struct {	int	lock1;
			int	lock2;
			int	cmd;
		}	t27_req_t;

private	t27_req_t	t27_reqs[] = { 
					{ F_RDLCK, F_WRLCK, F_SETLK } ,
					{ F_WRLCK, F_RDLCK, F_SETLK } ,
					{ F_RDLCK, F_UNLCK, F_SETLK } ,
					{ F_WRLCK, F_UNLCK, F_SETLK } ,
					{ F_RDLCK, F_WRLCK, F_SETLKW } ,
					{ F_WRLCK, F_RDLCK, F_SETLKW } ,
					{ F_RDLCK, F_UNLCK, F_SETLKW } ,
					{ F_WRLCK, F_UNLCK, F_SETLKW } 
				      };

public void
test19()
{
	int		err;
	struct		flock	test_lock;
	int	pathok = 0;

	DBUG_ENTER("test19");

	testfail = NOERROR;
	if ((t19_fd = creat(t19_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t19_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t19_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(t19_fd);
	testcomment = "fcntl( , F_SETLKW, ..) will wait until the request can be fullfilled";
	if ((t19_fd = open(t19_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t19_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t19_fd;
	(void) lseek(t19_fd, (off_t)5, SEEK_SET);

	ld_lck(&test_lock, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	setlk(&test_lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (!testfail)
	{
		PATH_FUNC_RPT(4);
		(void) cppair(ch_t19, NULLFN, 2*WAITTIME, 0);
	}

	(void) close(t19_fd);
	(void) unlink(t19_file);

	DBUG_VOID_RETURN;
}

private	void
ch_t19()
{
	struct	flock	lock1;
	int	pathok = 0;
	DBUG_ENTER("ch_t19");

	PATH_FUNC_TRACE;

	/* lock overlapping before parent lock */
	ld_lck(&lock1, F_RDLCK, SEEK_SET, 5, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t19_fd , F_SETLK, 0, &lock1, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	/* lock overlapping after parent lock */
	ld_lck(&lock1, F_RDLCK, SEEK_SET, 20, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t19_fd , F_SETLK, 0, &lock1, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	/* lock covering parent lock */
	ld_lck(&lock1, F_RDLCK, SEEK_SET, 0, 35, (pid_t)0);
	globok = 0;
	(void) fcn_to(t19_fd , F_SETLK, 0, &lock1, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


public void
test20()
{
	int		err;
	struct	flock	lock1;
	int	pathok = 0;

	DBUG_ENTER("test20");

	testfail = NOERROR;
	if ((t20_fd = creat(t20_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t20_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t20_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(t20_fd);
	testcomment = "shared locks prevent exclusive locks being set on the locked segment";
	if ((t20_fd = open(t20_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t20_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t20_fd;
	(void) lseek(t20_fd, (off_t)5, SEEK_SET);

	ld_lck(&lock1, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	setlk(&lock1 , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (!testfail)
	{
		PATH_FUNC_RPT(4);
		(void) cppair(ch_t20, NULLFN, WAITTIME, 0);
	}

	(void) close(t20_fd);
	(void) unlink(t20_file);

	DBUG_VOID_RETURN;
}

private	void
ch_t20()
{
	struct		flock	test_lock;
	struct		flock	chk_lock;
	int	pathok = 0;

	DBUG_ENTER("ch_t20");

	PATH_FUNC_TRACE;

	/* lock overlapping before parent lock */
	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 5, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t20_fd , F_SETLK, 0, &test_lock, SYSERROR, EACCES, WAITTIME);
	PATH_FUNC_TRACE;

	/* lock overlapping after parent lock */
	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 20, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t20_fd , F_SETLK, 0, &test_lock, SYSERROR, EACCES, WAITTIME);
	PATH_FUNC_TRACE;

	/* lock covering parent lock */
	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 0, 35, (pid_t)0);
	globok = 0;
	(void) fcn_to(t20_fd , F_SETLK, 0, &test_lock, SYSERROR, EACCES, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock, F_RDLCK, SEEK_SET, 10, 15, getppid());
	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

public void
test21()
{
	int		err;
	struct	flock	lock1;
	int	pathok = 0;

	DBUG_ENTER("test21");

	testfail = NOERROR;

	if ((t21_fd = creat(t21_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t21_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t21_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(t21_fd);
	testcomment = "exclusive locks prevent shared and exclusive locks being set on the locked segment";
	if ((t21_fd = open(t21_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t21_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t21_fd;
	(void) lseek(t21_fd, (off_t)5, SEEK_SET);

	ld_lck(&lock1, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	setlk(&lock1 , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (!testfail)
	{
		PATH_FUNC_RPT(4);
		(void) cppair(ch_t21, NULLFN, 2*WAITTIME, 0);
	}

	(void) close(t21_fd);
	(void) unlink(t21_file);

	DBUG_VOID_RETURN;
}

private	void
ch_t21()
{
	struct		flock	test_lock;
	struct		flock	chk_lock;
	int	pathok = 0;

	DBUG_ENTER("ch_t21");

	PATH_FUNC_TRACE;

	/* lock overlapping before parent lock */
	ld_lck(&test_lock, F_RDLCK, SEEK_SET, 5, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t21_fd , F_SETLK, 0, &test_lock, SYSERROR, EACCES, WAITTIME);
	PATH_FUNC_TRACE;

	/* lock overlapping after parent lock */
	ld_lck(&test_lock, F_RDLCK, SEEK_SET, 20, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t21_fd , F_SETLK, 0, &test_lock, SYSERROR, EACCES, WAITTIME);
	PATH_FUNC_TRACE;

	/* lock covering parent lock */
	ld_lck(&test_lock, F_RDLCK, SEEK_SET, 0, 35, (pid_t)0);
	globok = 0;
	(void) fcn_to(t21_fd , F_SETLK, 0, &test_lock, SYSERROR, EACCES, WAITTIME);
	PATH_FUNC_TRACE;

	/* lock overlapping before parent lock */
	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 5, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t21_fd , F_SETLK, 0, &test_lock, SYSERROR, EACCES, WAITTIME);
	PATH_FUNC_TRACE;

	/* lock overlapping after parent lock */
	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 20, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t21_fd , F_SETLK, 0, &test_lock, SYSERROR, EACCES, WAITTIME);
	PATH_FUNC_TRACE;

	/* lock covering parent lock */
	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 0, 35, (pid_t)0);
	globok = 0;
	(void) fcn_to(t21_fd , F_SETLK, 0, &test_lock, SYSERROR, EACCES, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_RDLCK, SEEK_SET, 5, 20, (pid_t)0);
	ld_lck(&chk_lock, F_WRLCK, SEEK_SET, 10, 15, getppid());
	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}

public void
test22()
{
	int		err;
	struct	flock	lock;
	int	pathok = 0;

	DBUG_ENTER("test22");

	testfail = NOERROR;

	if ((t22_fd = creat(t22_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t22_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t22_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t22_fd);

	testcomment = "lock not created/detected properly when beyond end of file";
	if ((t22_fd = open(t22_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t22_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t22_fd;
	(void) lseek(t22_fd, (off_t)5, SEEK_SET);

	ld_lck(&lock, F_WRLCK, SEEK_END, 10, 15, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (!testfail)
	{
		PATH_FUNC_RPT(4);
		(void) cppair(ch_t22, NULLFN, WAITTIME, 0);
	}

	(void) close(t22_fd);
	(void) unlink(t22_file);

	DBUG_VOID_RETURN;
}

private	void
ch_t22()
{
	struct		flock	test_lock;
	struct		flock	chk_lock;
	int	pathok = 0;

	DBUG_ENTER("ch_t22");

	PATH_FUNC_TRACE;

	ld_lck(&test_lock , F_RDLCK, SEEK_SET, sizeof(lckbuf)+15, 10, (pid_t)0);
	ld_lck(&chk_lock , F_WRLCK, SEEK_SET, sizeof(lckbuf)+10, 15, getppid());
	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

public void
test23()
{
	int		rval;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test23");

	testfail = NOERROR;
	if ((t23_syn_pa_ch = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t23_syn_ch_pa = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t23_fd = creat(t23_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t23_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t23_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t23_fd);

	for(t23_mode = T23_RDLCK_MODE; t23_mode <= T23_WRLCK_MODE; t23_mode++)
	{
			if ((t23_fd = open(t23_file, O_RDWR, MODEGEN)) == SYSERROR)
			{
				err = errno;
				xx_rpt(DELETION);
				in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
					t23_file, err, errname(err));
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;
			fd_chk_lck = t23_fd;
			(void) lseek(t23_fd, (off_t)5, SEEK_SET);

			rval = cppair(ch_t23, pa_t23, WAITTIME, NOK|E_PATH);
			if (rval == NOK)
				testfail++;

			if (rval != E_PATH)
				PATH_FUNC_TRACE;
			(void) close(t23_fd);
	}

	if (!testfail)
		PATH_XS_RPT(4+(t23_mode-1)*2);

	(void) unlink(t23_file);
	(void) closesync(t23_syn_pa_ch);
	(void) closesync(t23_syn_ch_pa);

	DBUG_VOID_RETURN;
}

private	void
pa_t23()
{

	int	child_flag;
	struct	flock	lock;
	int	pathok = 0;
	DBUG_ENTER("pa_t23");

	child_flag = 0;
	switch(t23_mode)
	{
		case T23_WRLCK_MODE:
			DBUG_PRINTF("pa_t23","Testing F_WRLCK");
			ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 0, (pid_t)0);
			break;
		case T23_RDLCK_MODE:
			DBUG_PRINTF("pa_t23","Testing F_RDLCK");
			ld_lck(&lock, F_RDLCK, SEEK_SET, 10, 0, (pid_t)0);
			break;
		default:
			xx_rpt(DELETION);
			in_rpt("Bad test mode: %d - expecting T23_WRLCK_MODE (%d), T23_RDLCK_MODE (%d)",
				t23_mode, T23_WRLCK_MODE, T23_RDLCK_MODE );
			DBUG_VOID_RETURN;
	}

	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t23_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t23_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(t23_mode)
	{
		case T23_WRLCK_MODE:
			ld_lck(&lock, F_WRLCK, SEEK_SET, 0, 0, (pid_t)0);
			break;
		case T23_RDLCK_MODE:
			ld_lck(&lock, F_RDLCK, SEEK_SET, 0, 0, (pid_t)0);
			break;
		default:
			xx_rpt(DELETION);
			in_rpt("Bad test mode: %d - expecting T23_WRLCK_MODE (%d), T23_RDLCK_MODE (%d)",
				t23_mode, T23_WRLCK_MODE, T23_RDLCK_MODE);
			DBUG_VOID_RETURN;
	}
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
	{
		child_flag++;
	}
	else
		PATH_TRACE;

	if (sendsync(t23_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
		!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* now use child_flag to pass the current path value to ch_t23 */
		/* by use of the existing pipe */
#ifdef TESTBED1
	pathok--;
#endif
	child_flag = pathok;
	if (sendsync(t23_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed in path reporting", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(10); 

	DBUG_VOID_RETURN;
}


private	void
ch_t23()
{
	struct		flock	test_lock;
	struct		flock	chk_lock;
	int		child_flag;
	int	pathok = 0;

	DBUG_ENTER("ch_t23");

	child_flag = NOERROR;
	testcomment = "lock not created properly when l_len is 0";
	if (waitsync(t23_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	switch(t23_mode)
	{
	case T23_WRLCK_MODE:
		ld_lck(&test_lock , F_RDLCK, SEEK_SET, 20, 5, (pid_t)0);
		ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 10, 0, getppid());
		break;
	case T23_RDLCK_MODE:
		ld_lck(&test_lock , F_WRLCK, SEEK_SET, 20, 5, (pid_t)0);
		ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 10, 0, getppid());
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("Bad test mode: %d - expecting T23_WRLCK_MODE (%d), T23_RDLCK_MODE (%d)",
			t23_mode, T23_WRLCK_MODE, T23_RDLCK_MODE);
		DBUG_EXIT(NOK);
	}

globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t23_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (testfail)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	testcomment = "the whole file was not locked when l_len = l_start = 0";
	if (waitsync(t23_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (child_flag != NOERROR)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	switch(t23_mode)
	{
	case T23_WRLCK_MODE:
		ld_lck(&test_lock , F_RDLCK, SEEK_SET, 20, 5, (pid_t)0);
		ld_lck(&chk_lock , F_WRLCK, SEEK_SET, 0, 0, getppid());
		break;
	case T23_RDLCK_MODE:
		ld_lck(&test_lock , F_WRLCK, SEEK_SET, 20, 5, (pid_t)0);
		ld_lck(&chk_lock , F_RDLCK, SEEK_SET, 0, 0, getppid());
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("Bad test mode: %d - expecting T23_WRLCK_MODE (%d), T23_RDLCK_MODE (%d)",
			t23_mode, T23_WRLCK_MODE, T23_RDLCK_MODE);
		DBUG_EXIT(NOK);
	}

	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	/* now check for path check errors in pa_t23 by receiving path */
		/* check value through child_flag */
	if (waitsync(t23_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), WAITTIME)
							!= sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed in path check reporting", EXTERR);
		DBUG_EXIT(NOK);
	}
	else if (child_flag == 9)
		PATH_TRACE;

#ifdef TESTBED2
	pathok--;
#endif
	if (!testfail)
	{
		if(pathok == 10)
		{
			DBUG_EXIT(OK);
		}
		else
		{
			DBUG_PRINTF("trace",
				"ch_t23 path counter %d, expected %d",
				pathok, 10);
			DBUG_EXIT(E_PATH);
		}
	}

	DBUG_EXIT(NOK);
}

public void
test24()
{
	struct	flock	test_lock;
	int		fd;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test24");

	testfail = NOERROR;
	if ((fd = creat(t24_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t24_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "locks are removed when any fd for the file is closed";
	if ((t24_fd = open(t24_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t24_file, err, errname(err));
		(void) close(fd);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t24_fd;
	(void) lseek(t24_fd, (off_t)5, SEEK_SET);
	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);

	globok = 0;
	(void) fcn_to(t24_fd, F_SETLK, 0, &test_lock, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	(void) close(fd);
	if (!testfail)
	{
		PATH_FUNC_RPT(4);
		(void) cppair(ch_t24, NULLFN, WAITTIME, 0);
	}

	(void) close(t24_fd);
	(void) unlink(t24_file);

	DBUG_VOID_RETURN;
}

private	void
ch_t24()
{
	struct	flock	test_lock,	chk_lock;
	int	pathok = 0;

	DBUG_ENTER("ch_t24");

	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	ld_lck(&chk_lock, F_UNLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

public void
test25()
{
	struct	flock	test_lock,	chk_lock;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test25");

	testfail = NOERROR;
	if ((t25_fd = creat(t25_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t25_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t25_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t25_fd);
	testcomment = "all locks created by a process are removed when it exits";

	if ((t25_fd = open(t25_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t25_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t25_fd;
	(void) lseek(t25_fd, (off_t)5, SEEK_SET);
	if (cppair(ch_t25, NULLFN, WAITTIME, NOK) == NOK)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	ld_lck(&chk_lock, F_UNLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	tstlk(&test_lock , &chk_lock);
	PATH_FUNC_TRACE;

	(void) close(t25_fd);

	if (!testfail)
		PATH_XS_RPT(5);

	(void) unlink(t25_file);
	DBUG_VOID_RETURN;
}

private	void
ch_t25()
{
	struct	flock	lock;
	int	pathok = 0;
	DBUG_ENTER("ch_t25");

	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (testfail)
		DBUG_EXIT(NOK);
	DBUG_VOID_RETURN;

	/* do here */
}

public void
test26()
{
	struct	flock	lock;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test26");

	testfail = NOERROR;

	if ((t26_syn = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t26_fd = creat(t26_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t26_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t26_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(t26_fd);
	testcomment = "locks are not inherited via fork()";
	if ((t26_fd = open(t26_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t26_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t26_fd;
	(void) lseek(t26_fd, (off_t)5, SEEK_SET);

	/* Using a shared instead of exculsive lock means a full scan
	   of the lock table will be done */
	ld_lck(&lock, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (!testfail)
	{
		PATH_FUNC_RPT(5);
		(void) cppair(ch_t26, pa_t26, WAITTIME, 0);
	}

	(void) closesync(t26_syn);
	(void) close(t26_fd);
	(void) unlink(t26_file);

	DBUG_VOID_RETURN;
}

private	void
pa_t26()
{
	int child_flag;
	struct	flock	lock;
	int	pathok = 0;

	DBUG_ENTER("pa_t26");

	/* If shared lock was inherited by child, parent will
	   not be able to overwrite it with an exclusive lock */
	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	setlk(&lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (sendsync(t26_syn[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);
	globok = 1;

	DBUG_VOID_RETURN;
}

private	void
ch_t26()
{
	int child_flag;
	int	pathok = 0;

	DBUG_ENTER("ch_t26");

	PATH_FUNC_TRACE;

	if (waitsync(t26_syn[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

public void
test27()
{
	int	ret = 0;
	int	err = 0;
	int	pathok = 0;

	DBUG_ENTER("test27");

	testfail = NOERROR;
	testcomment = "Lock type has not been completely changed after a change of locks";

	for (t27_itter = 0; t27_itter <	sizeof(t27_reqs)/sizeof(t27_reqs[0]);
		t27_itter++)
	{
		if (pipe(t27_fds))
		{
			err = errno;
			if (!testfail++)
				xx_rpt(DELETION);
			in_rpt("pipe() failed - %d(%s)", err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if ((t27_fildes = open(t27_file, O_RDWR|O_CREAT|O_TRUNC, MODEGEN))
			== SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: open(%s, O_RDWR|O_CREAT|O_TRUNC, %#o) failed - errno %d(%s)",
				t25_file, MODEGEN, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (wrt_to(t27_fildes, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: %s - write(2) failure", EXTERR);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		if ((ret = cppair(ch_t27, pa_t27, 2*WAITTIME, NOK|E_PATH)) != E_PATH)
			PATH_TRACE;

		PATH_FUNC_TRACE;

		if (ret == NOK)
			testfail++;
		else
			PATH_TRACE;

		(void) close(t27_fds[READ]);
		(void) close(t27_fds[WRITE]);
		(void) unlink(t27_file);

	}

	if (!testfail)
		PATH_XS_RPT(6 * t27_itter);

	DBUG_VOID_RETURN;
}

private	void
pa_t27()
{
	struct	flock	lock1, lock2;
	int	ret;
	int	err = 0;
	int	err1 = 0;
	int	pathok = 0;

	DBUG_ENTER("pa_t27");

	(void) close(t27_fds[READ]);

	ld_lck(&lock1, t27_reqs[t27_itter].lock1, SEEK_SET, 0, 0, (pid_t)0);
	ld_lck(&lock2, t27_reqs[t27_itter].lock2, SEEK_SET, 0, 0, (pid_t)0);

	globok = 0;
	(void) fcn_to(t27_fildes, t27_reqs[t27_itter].cmd, 0, &lock1, NOERROR,
		NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	(void) sleep(2);

	/*
	 *	calls to fcntl() must be in as rapid a sucession as possible
	 *	therefore tb_fcntl() is not used to save the over-haed of a
	 *	function call, and the locks have been loaded in advance
	 */

	ret = fcntl(t27_fildes, t27_reqs[t27_itter].cmd, &lock2);
	err = errno;

	/*	unblock the child process	*/

	if (close(t27_fds[WRITE]))
	{
		err1 = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("close() failed - %d(%s)", err1, errname(err1));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ret == -1)
	{
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() failed - %d(%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private	void
ch_t27()
{

	struct	flock	tst_lck, ret_lck;
	int	err = 0;
	char	buf[1];
	int	pathok = 0;

	DBUG_ENTER("ch_t27");

	if (close(t27_fds[WRITE]))
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("close() failed - %d(%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	ld_lck(&tst_lck, F_WRLCK, SEEK_SET, 0, 0, getppid());
	ld_lck(&ret_lck, t27_reqs[t27_itter].lock2, SEEK_SET, 0, 0, getppid());

	/*
	 *	read on an empty pipe will cause the process to block
	 *	O_NONBLOCK is cleared for the fds in the pipe() call
	 *	no process other than the child must have the pipe
	 *	open for writing
	 */

	(void) read(t27_fds[READ], buf, 1);

	if (fcntl(t27_fildes, F_GETLK, &tst_lck) == -1)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("fcntl() failed - %d(%s)", err, errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	(void) cmp_lcks( &tst_lck, &ret_lck );

	if (globok == 1)
		PATH_TRACE;

	if (testfail)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 4)
		DBUG_EXIT(E_PATH);

	DBUG_VOID_RETURN;
}

public void
test28()
{
	/* fcntl() cannot change file access mode, and ignores O_CREAT
	   O_TRUNC, O_EXCL and O_NOCTTY */

	int	fd, err, flags;
	int	pathok = 0;
	struct stat statbuf;

	DBUG_ENTER("test28");

	testfail = 0;

	if ((fd = open(t28_file, O_RDONLY|O_CREAT, MODEGEN)) < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY|O_CREAT, %#o) failed - errno %d(%s)",
			t28_file, MODEGEN, err, errname(err));
	}
	else
	{
		globok = 0;
		t28sub(fd, O_RDONLY, O_WRONLY);
		PATH_FUNC_TRACE;

		globok = 0;
		t28sub(fd, O_RDONLY, O_RDWR);
		PATH_FUNC_TRACE;
#ifdef TESTBED2
		globok = 0;
		t28sub(fd, O_WRONLY, O_RDONLY);
		PATH_FUNC_TRACE;
#else
		PATH_TRACE;
#endif

		(void) close(fd);
	}

	if ((fd = open(t28_file, O_WRONLY|O_CREAT, MODEGEN)) < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_CREAT, %#o) failed - errno %d(%s)",
			t28_file, MODEGEN, err, errname(err));
	}
	else
	{
		globok = 0;
		t28sub(fd, O_WRONLY, O_RDONLY);
		PATH_FUNC_TRACE;

		globok = 0;
		t28sub(fd, O_WRONLY, O_RDWR);
		PATH_FUNC_TRACE;

		(void) close(fd);
	}

	if ((fd = open(t28_file, O_RDWR|O_CREAT, MODEGEN)) < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_CREAT, %#o) failed - errno %d(%s)",
			t28_file, MODEGEN, err, errname(err));
	}
	else
	{
		globok = 0;
		t28sub(fd, O_RDWR, O_RDONLY);
		PATH_FUNC_TRACE;

		globok = 0;
		t28sub(fd, O_RDWR, O_WRONLY);
		PATH_FUNC_TRACE;

		(void) close(fd);
	}

	/* test O_CREAT and O_EXCL are ignored */

	if ((fd = open(t28_file, O_WRONLY|O_CREAT, MODEGEN)) < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_CREAT, %#o) failed - errno %d(%s)",
			t28_file, MODEGEN, err, errname(err));
	}
	else
	{
		flags = fcntl(fd, F_GETFL);
		flags |= (O_CREAT | O_EXCL);
		if (fcntl(fd, F_SETFL, flags) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("fcntl(fd, F_SETFL, oldflags|O_CREAT|O_EXCL) failed - errno %d(%s)",
				err, errname(err));
		}
		else
			PATH_TRACE;
		
		(void) close(fd);
	}

	/* test O_TRUNC is ignored */

	if ((fd = open(t28_file, O_WRONLY|O_CREAT, MODEGEN)) < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_CREAT, %#o) failed - errno %d(%s)",
			t28_file, MODEGEN, err, errname(err));
	}
	else if (write(fd, "abc", 3) != 3)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() failed - errno %d(%s)", err, errname(err));
		(void) close(fd);
	}
	else
	{
		flags = fcntl(fd, F_GETFL);
		flags |= O_TRUNC;
		if (fcntl(fd, F_SETFL, flags) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("fcntl(fd, F_SETFL, oldflags|O_TRUNC) failed - errno %d(%s)",
				err, errname(err));
		}
		else if (fstat(fd, &statbuf) == SYSERROR)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("stat() failed - errno %d(%s)", err, errname(err));
		}
		else if (statbuf.st_size == 0)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("fcntl(fd, F_SETFL, oldflags|O_TRUNC) truncated the file");
		}
		else
			PATH_TRACE;
		
		(void) close(fd);
	}

	/* test O_NOCTTY is ignored */

	PATH_FUNC_RPT(9);
	(void) cppair(ch_t28, NULLFN, 3*WAITTIME, E_PATH);

	/* xs_rpt() is in child */

	(void) unlink(t28_file);

	DBUG_VOID_RETURN;
}

private void
t28sub(fd, oldmode, newmode)
int fd, oldmode, newmode;
{
	int	setfl, getfl, err;
	int	pathok = 0;

	DBUG_ENTER("t28sub");

	setfl = fcntl(fd, F_GETFL);
	setfl &= ~oldmode;
	setfl |= (newmode | O_APPEND | O_NONBLOCK);
	if (fcntl(fd, F_SETFL, setfl) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("fcntl(fd, F_SETFL, %#o) failed - errno %d(%s)",
			setfl, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	getfl = fcntl(fd, F_GETFL);
	if ((getfl & O_ACCMODE) != oldmode)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fcntl(fd, F_SETFL, %#o) changed file access mode",
			setfl);
		in_rpt("old mode: %#o, new mode: %#o", oldmode,
			(getfl & O_ACCMODE));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t28()
{
	int	flags, err;
	int	pathok = 0;

	DBUG_ENTER("ch_t28");

	PATH_FUNC_TRACE;

	errno = 0;
#if !TEST_XNFS
	switch (termios_prep(&tty_fd , &tty_term, &loop_fd , &loop_term))
	{
	case 0:
		gti_supported = 1;
		break;
	case 1:
		gti_supported = 0;
		break;

	default:
		err = errno;
		xx_rpt(DELETION);
		in_rpt("termios_prep() failed - errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
#endif /* TEST_XNFS */

#if !TEST_XNFS
	if (gti_supported)
	{
		globok = 0;
		sig_establish(ctty_post);
		PATH_FUNC_TRACE;

		errno = 0;
		globok = 0;
		if (!ctty_sig(TRUE))
		{
			xx_rpt(DELETION);
			in_rpt("1st SIGINT from controlling terminal not caught");
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	}
	else
#endif /* TEST_XNFS */
	{
		PATH_TRACE;
		if ((tty_fd = open(t28_file, O_WRONLY|O_CREAT, MODEGEN)) < 0)
		{
			err = errno;
			if (testfail++ == 0)
				xx_rpt(DELETION);
			in_rpt("open(%s, O_WRONLY|O_CREAT, %#o) failed - errno %d(%s)",
				t28_file, MODEGEN, err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	flags = fcntl(tty_fd, F_GETFL);
	flags |= O_NOCTTY;
	if (fcntl(tty_fd, F_SETFL, flags) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("fcntl(fd, F_SETFL, oldflags|O_NOCTTY) failed - errno %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#if !TEST_XNFS
	if (gti_supported)
	{
		errno = 0;
		globok = 0;
		if (!ctty_sig(TRUE))
		{
			xx_rpt(FAILURE);
			in_rpt("2nd SIGINT from controlling terminal not caught");
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	}
	else
#endif /* TEST_XNFS */
		PATH_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

#if !TEST_XNFS
private int
ctty_sig(sig_exp)
int	sig_exp;
{

	int	rval;
	unsigned char chr;
	int	pathok = 0;

	DBUG_ENTER("ctty_sig");

	sigint_posted = FALSE;

	chr = tty_term_p->c_cc[VINTR];

	SET_TIMEOUT(WAITTIME);
	errno = 0;
	rval = write(loop_fd , (char *)&chr, 1);

	if (sigint_posted == FALSE)
		(void) pause();

	CLEAR_ALARM;

	if (rval != 1)
	{
		xx_rpt(FAILURE);
		in_rpt("write() failed to write %d bytes instead wrote %d bytes"			,1,rval);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(sig_exp == sigint_posted); 
} 

private	void
ctty_post()
{
	sigint_posted = TRUE;
}

private void
sig_establish(func)
void	(*func) ();
{
	struct	sigaction	sig;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("sig_establish");

	sig.sa_handler = func;
	(void) sigemptyset(&sig.sa_mask);
	sig.sa_flags = 0;

	if (sigaction(SIGINT , &sig , NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sigaction() failed %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN; 
}
#endif /* TEST_XNFS */

public void
test29()
{
	struct	flock	test_lock;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test29");

	testfail = NOERROR;
	if ((t29_fd = creat(t29_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t29_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t29_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t29_fd);

	testcomment = "fcntl(fd, F_SETLK, F_RDLCK) fails when file segment is already exclusively locked";
	if ((t29_fd = open(t29_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t29_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t29_fd;
	(void) lseek(t29_fd, (off_t)5, SEEK_SET);
	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);

	globok = 0;
	(void) fcn_to(t29_fd, F_SETLK, 0, &test_lock, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;
	if (!testfail)
	{
		PATH_FUNC_RPT(4);
		(void) cppair(ch_t29, NULLFN, WAITTIME, 0);
	}

	(void) close(t29_fd);
	(void) unlink(t29_file);

	DBUG_VOID_RETURN;
}

private	void
ch_t29()
{
	struct	flock	lock;
	int	pathok = 0;

	DBUG_ENTER("ch_t29");

	PATH_FUNC_TRACE;

	ld_lck(&lock, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t29_fd , F_SETLK, 0, &lock, SYSERROR, EACCES, WAITTIME);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

public void
test30()
{
	struct	flock	test_lock;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test30");

	testfail = NOERROR;
	if ((t30_fd = creat(t30_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t30_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t30_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t30_fd);

	testcomment = "fcntl(fd, F_SETLK, F_WRLCK) fails when file segment is already exclusively locked";
	if ((t30_fd = open(t30_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t30_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t30_fd;
	(void) lseek(t30_fd, (off_t)5, SEEK_SET);
	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);

	globok = 0;
	(void) fcn_to(t30_fd, F_SETLK, 0, &test_lock, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;
	if (!testfail)
	{
		PATH_FUNC_RPT(4);
		(void) cppair(ch_t30, NULLFN, WAITTIME, 0);
	}

	(void) close(t30_fd);
	(void) unlink(t30_file);

	DBUG_VOID_RETURN;
}

private	void
ch_t30()
{
	struct	flock	lock;
	int	pathok = 0;

	DBUG_ENTER("ch_t30");

	PATH_FUNC_TRACE;

	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t30_fd , F_SETLK, 0, &lock, SYSERROR, EACCES, WAITTIME);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

public void
test31()
{
	struct	flock	test_lock;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test31");

	testfail = NOERROR;
	if ((t31_fd = creat(t31_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t31_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t31_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t31_fd);
	
	testcomment = "fcntl(fd, F_SETLK, F_WRLCK) fails when file segment is already shared locked";
	if ((t31_fd = open(t31_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t31_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	fd_chk_lck = t31_fd;
	(void) lseek(t31_fd, (off_t)5, SEEK_SET);
	ld_lck(&test_lock, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);

	globok = 0;
	(void) fcn_to(t31_fd, F_SETLK, 0, &test_lock, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;
	if (!testfail)
	{
		PATH_FUNC_RPT(4);
		(void) cppair(ch_t31, NULLFN, WAITTIME, 0);
	}

	(void) close(t31_fd);
	(void) unlink(t31_file);

	DBUG_VOID_RETURN;
}

private	void
ch_t31()
{
	struct	flock	lock;
	int	pathok = 0;

	DBUG_ENTER("ch_t31");

	PATH_FUNC_TRACE;

	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t31_fd , F_SETLK, 0, &lock, SYSERROR, EACCES, WAITTIME);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

public void
test32()
{
	DBUG_ENTER("test32");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

public void
test32()
{
	int	t32_fd;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test32");

	testfail = NOERROR;
	testcomment = "fcntl(fd, ...) will fail when fd is invalid";

	globok = 0;
	(void) tb_fcntl(BADFD, F_GETFD, 0, NULLFLK, SYSERROR, EBADF);
	PATH_FUNC_TRACE;
	if ((t32_fd = creat(t32_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t32_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t32_fd);
	globok = 0;
	(void) tb_fcntl(t32_fd, F_GETFD, 0, NULLFLK, SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	(void) unlink(t32_file);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
test33()
{
	int	t33_fd;
	struct	flock	lock;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test33");

	testfail = 0;
	testcomment = "fcntl(fd, F_SETLK[W], arg) will fail if the lock type is F_RDLCK and the file is open write only";

	if ((t33_fd = open_to(t33_file, O_WRONLY|O_CREAT, MODEGEN, WAITTIME))
		== SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_WRONLY|O_CREAT) failed - errno %d(%s)",
			t33_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t33_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&lock, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t33_fd , F_SETLKW, 0, &lock, SYSERROR, EBADF, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&lock, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t33_fd , F_SETLK, 0, &lock, SYSERROR, EBADF, WAITTIME);
	PATH_FUNC_TRACE;

	(void) close(t33_fd);
	if (!testfail)
		PATH_XS_RPT(4);

	(void) unlink(t33_file);
	DBUG_VOID_RETURN;
}

public void
test34()
{
	int	t34_fd;
	struct	flock	lock;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test34");

	testfail = 0;
	testcomment = "fcntl(fd, F_SETLK[W], arg) will fail if the lock type is F_WRLCK and the file is open read only";

	if ((t34_fd = creat(t34_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t34_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t34_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t34_fd);

	if ((t34_fd = open_to(t34_file, O_RDONLY, MODEGEN, WAITTIME))
		== SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open(%s, O_RDONLY) failed - errno %d(%s)",
			t34_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t34_fd , F_SETLKW, 0, &lock, SYSERROR, EBADF, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t34_fd , F_SETLK, 0, &lock, SYSERROR, EBADF, WAITTIME);
	PATH_FUNC_TRACE;

	(void) close(t34_fd);
	if (!testfail)
		PATH_XS_RPT(5);

	(void) unlink(t34_file);
	DBUG_VOID_RETURN;
}

public void
test35()
{
	int		err;
	struct		flock	test_lock;
	int	pathok = 0;

	DBUG_ENTER("test35");

	testfail = NOERROR;
	if ((t35_fd = creat(t35_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t35_file, MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (wrt_to(t35_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t35_fd);

	testcomment = "fcntl( , F_SETLKW, ..) will fail if interupted by a signal";
	if ((t35_fd = open(t35_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t35_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t35_fd;
	(void) lseek(t35_fd, (off_t)5, SEEK_SET);

	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	setlk(&test_lock , (struct flock *)NULL);
	PATH_FUNC_TRACE;

	if (!testfail)
	{
		PATH_FUNC_RPT(4);
		(void) cppair(ch_t35, NULLFN, 2*WAITTIME, 0);
	}

	(void) close(t35_fd);
	(void) unlink(t35_file);

	DBUG_VOID_RETURN;
}

private	void
ch_t35()
{
	struct	flock	lock1;
	int	pathok = 0;
	DBUG_ENTER("ch_t35");

	PATH_FUNC_TRACE;

	ld_lck(&lock1, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	(void) fcn_to(t35_fd , F_SETLKW, 0, &lock1, SYSERROR, EINTR, WAITTIME/2);

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}
