/*
*      SCCS:  @(#)  POSIX.os/ioprim/fcntl/fcntl3.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/ioprim/fcntl - T.fcntl Rel 4.4.1 (06/09/97)";
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		fcntl3.c

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fcntl() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Tue May 30 13:43:36 BST 1989
MODIFICATIONS:	
		Geoff Clare, 23 Aug 1989
			If VSX_UNLOCKABLE_FILE is not set, result
			should be UNSUPPORTED, not DELETION

		Geoff Clare, 30 Aug - 5 Sep 1989
			Bad arguments to tb_fcntl() corrected
			Restructured ENOLCK and EDEADLK tests
		
		Geoff Clare, 18 Sep 1989
			New parameter VSX_FCNTL_MAXLOCK

		Geoff Clare, 2 Oct 1989
			(VSX_FCNTL_MAXLOCK == -1) => no limit => UNTESTED
		
		Geoff Clare, 29 May 1990
			Allow for unlimited OPEN_MAX

		Peter Lee, Unisoft Ltd.,
		Mon Aug 06 16:45:57 BST 1990
		Added resilience code.
		
		Geoff Clare, 27 June 1991
			VSX_UNLOCKABLE_FILE="unsup" for unsupported result

		Geoff Clare, 28 May 1992
			New VSX_FCNTL_EDEADLK parameter.
		
		Geoff Clare, 20 Sept 1995
			Missing PATH_XS_RPT() in test 41.

		Geoff Clare, 3 Jan 1996
			Use VSX_INVALID_WHENCE parameter.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<limits.h>
#include	<signal.h>
#include	<dbug.h>
#include	<report.h>
#include	<synch.h>
#include	<timeout.h>
#include	<pcts_max.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fcntl
#endif


#define EXTERR			"External error"
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEGEN		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define BAD_FTYPE	(F_RDLCK+F_WRLCK+F_UNLCK+1)	/* a bad flock type */
#define NOK		02
#define PATH_NOK	8

#define LOCKBS		40		/* size of buffer */
#define	READ		0
#define	WRITE		1
#define NULLFLK		((struct flock *)NULL)

extern	char	*errname();


private	int		t42_fd;
private	int		t45_fd;
private	synch		*t45_syn_pa_ch;
private	synch		*t45_syn_ch_pa;

extern	int	fd_chk_lck;
extern	int	open_max;
extern	char	lckbuf[LOCKBS];

#if !TEST_XNFS
private char	*t36_file =		"./fcntl-t.36";
private char	*t37_file =		"./fcntl-t.37";
#endif
private char	*t38_file =		"./fcntl-t.38";
private char	*t39_file =		"./fcntl-t.39";
private char	*t40_file =		"./fcntl-t.40";
private char	*t42_file =		"./fcntl-t.42";
private char	*t43_file =		"./fcntl-t.43";
private char	*t44_file =		"./fcntl-t.44";
private char	*t45_file =		"./fcntl-t.45";

private	void	ch_t41();
private	void	ch_t42();
private	void	pa_t45();
private	void	ch_t45();
extern	int	wrt_to();
extern	void	ld_lck();
extern	void	setlk();
extern	void	setlkw();
extern	void	tstlk();
extern	int	fcn_to();

extern	int	testfail;
extern	char	*testcomment;


#if TEST_XNFS

public void
test36()
{
	DBUG_ENTER("test36");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

public void
test36()
{
	int		err;
	int		fd;
	int	pathok = 0;

	DBUG_ENTER("test36");
	testfail = NOERROR;
	testcomment = "when arg is negative, -1 returned and errno set to EINVAL";
	if ((fd = creat(t36_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t36_file, (long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tb_fcntl(fd, F_DUPFD, -1, NULLFLK, SYSERROR, EINVAL);
	PATH_FUNC_TRACE;
	if (!testfail)
		PATH_XS_RPT(2);

	(void) unlink(t36_file);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

public void
test37()
{
	DBUG_ENTER("test37");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

public void
test37()
{
	int		err;
	int		fd, fd2;
	int	pathok = 0;

	DBUG_ENTER("test37");
	testfail = NOERROR;
	if ((fd = creat(t37_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t37_file, (long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (open_max != -1)
	{
		testcomment = "when arg == OPEN_MAX-1, no EINVAL error";
		globok = 0;
		fd2 = tb_fcntl(fd, F_DUPFD, open_max-1, NULLFLK, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		(void) close(fd2);

		testcomment = "when arg >= OPEN_MAX, -1 returned and errno set to EINVAL";

		globok = 0;
		(void) tb_fcntl(fd, F_DUPFD, open_max, NULLFLK, SYSERROR, EINVAL);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) tb_fcntl(fd, F_DUPFD, INT_MAX, NULLFLK, SYSERROR, EINVAL);
		PATH_FUNC_TRACE;
	}
	else
	{
		testcomment = "when arg == PCTS_OPEN_MAX, no EINVAL error";
		PATH_TRACE; PATH_TRACE;

		globok = 0;
		fd2 = tb_fcntl(fd, F_DUPFD, PCTS_OPEN_MAX, NULLFLK, SUCCEED, NOERROR);
		PATH_FUNC_TRACE;
		(void) close(fd2);
	}

	if (!testfail)
		PATH_XS_RPT(4);

	(void) close(fd);
	(void) unlink(t37_file);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
test38()
{
	int	t38_fd;
	struct	flock	test_lock;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test38");

	testfail = NOERROR;
	if ((t38_fd = creat(t38_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t38_file, (long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t38_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t38_fd);

	testcomment = "fcntl() will fail when attempting to set a lock before the start of the file";
	if ((t38_fd = open(t38_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t38_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	fd_chk_lck = t38_fd;
	(void) lseek(t38_fd, (off_t)5, SEEK_SET);

	ld_lck(&test_lock, F_WRLCK, SEEK_SET, -1, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t38_fd, F_GETLK, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_WRLCK, SEEK_SET, -1, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t38_fd, F_SETLKW, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_WRLCK, SEEK_SET, -1, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t38_fd, F_SETLK, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	(void) close(t38_fd);

	if (!testfail)
		PATH_XS_RPT(6);

	(void) unlink(t38_file);
	DBUG_VOID_RETURN;
}

public void
test39()
{
	int	t39_fd;
	struct	flock	test_lock;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("test39");

	testfail = NOERROR;
	if ((t39_fd = creat(t39_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t39_file, (long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t39_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t39_fd);

	testcomment = "fcntl() will fail when attempting to set a lock with an invlaid l_type";
	if ((t39_fd = open(t39_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t39_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) lseek(t39_fd, (off_t)5, SEEK_SET);
	ld_lck(&test_lock, BAD_FTYPE, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t39_fd, F_GETLK, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, BAD_FTYPE, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t39_fd, F_SETLKW, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, BAD_FTYPE, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t39_fd, F_SETLK, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	(void) close(t39_fd);

	if (!testfail)
		PATH_XS_RPT(6);

	(void) unlink(t39_file);
	DBUG_VOID_RETURN;
}

public void
test40()
{
	char		*cp;
	int		t40_fd;
	struct	flock	test_lock;
	int		err;
	int		badwhence = -1;
	int		pathok = 0;

	DBUG_ENTER("test40");

	cp = tet_getvar("VSX_INVALID_WHENCE");
	if (cp != NULL && *cp != '\0')
		badwhence = atoi(cp);

	testfail = NOERROR;
	if ((t40_fd = creat(t40_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t40_file, (long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t40_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(t40_fd);

	testcomment = "fcntl() will fail when attempting to set a lock with an invlaid l_whence";
	if ((t40_fd = open(t40_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t40_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) lseek(t40_fd, (off_t)5, SEEK_SET);
	ld_lck(&test_lock, F_WRLCK, badwhence, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t40_fd, F_GETLK, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_WRLCK, badwhence, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t40_fd, F_SETLKW, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_WRLCK, badwhence, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(t40_fd, F_SETLK, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	(void) close(t40_fd);

	if (!testfail)
		PATH_XS_RPT(6);

	(void) unlink(t40_file);
	DBUG_VOID_RETURN;
}

public void
test41()
{

	DBUG_ENTER("test41");

	testfail = NOERROR;
	testcomment = "fcntl() will fail when attempting to set a lock on a file that does not support locking";

	(void) cppair(ch_t41, NULLFN, 5*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t41()
{
	int	fdr, fdw;
	char	*unlockable;
	struct	flock	test_lock;
	int		err;
	int	pathok = 0;

	DBUG_ENTER("ch_t41");
	
	if ((unlockable = tet_getvar("VSX_UNLOCKABLE_FILE")) ==  NULL ||
	     *unlockable == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_UNLOCKABLE_FILE is not set");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(unlockable, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_UNLOCKABLE_FILE is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fdr = open(unlockable, O_RDONLY|O_NONBLOCK)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDONLY|O_NONBLOCK) failed - errno %d(%s)",
			unlockable, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fdw = open(unlockable, O_WRONLY|O_NONBLOCK)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_WRONLY|O_NONBLOCK) failed - errno %d(%s)",
			unlockable, err, errname(err));
		(void) close(fdr);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&test_lock, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(fdr, F_GETLK, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(fdr, F_SETLKW, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_RDLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(fdr, F_SETLK, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_UNLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(fdr, F_SETLK, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(fdw, F_GETLK, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(fdw, F_SETLKW, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_WRLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(fdw, F_SETLK, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	ld_lck(&test_lock, F_UNLCK, SEEK_SET, 10, 15, (pid_t)0);
	globok = 0;
	(void) fcn_to(fdw, F_SETLK, 0, &test_lock, SYSERROR, EINVAL, WAITTIME);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(11);

	(void) close(fdw);
	(void) close(fdr);
	DBUG_VOID_RETURN;
}

public void
test42()
{
	int		err;

	DBUG_ENTER("test42");
	testfail = NOERROR;
	testcomment = "the lower of OPEN_MAX and PCTS_OPEN_MAX fds can be opened";

	if (open_max <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sysconf(_SC_OPEN_MAX) failed during prep");
		DBUG_VOID_RETURN;
	}

	if ((t42_fd = creat(t42_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t42_file, (long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t42 , NULLFN , WAITTIME , 0);

	(void) close(t42_fd);
	(void) unlink(t42_file);

	DBUG_VOID_RETURN;
}

private void
ch_t42()
{
	int		i;
	int		*fd;
	int		max_fd;
	int		pathok = 0;
	int		loop1;

	DBUG_ENTER("ch_t42");

	max_fd = (open_max <= PCTS_OPEN_MAX) ? open_max : PCTS_OPEN_MAX;

	fd = (int *) malloc((size_t)((max_fd+1) * sizeof(int)));
	if (fd == (int *)NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: malloc() failed");
		DBUG_VOID_RETURN; 
	} 
	else
		PATH_TRACE;

	for (i = n_open_fds(), loop1 = 0 ; i < max_fd ; i++)
	{
		globok = 0;
		fd[i] = tb_fcntl(t42_fd, F_DUPFD, 0, NULLFLK, NOERROR, NOERROR);
		if (testfail)
		{
			DBUG_VOID_RETURN;
		}
		else if (loop1 == 0)
		{
			loop1 = 1;
			PATH_FUNC_TRACE;
		}
	} 

	if (open_max <= PCTS_OPEN_MAX)
	{
		globok = 0;
		(void) tb_fcntl(t42_fd, F_DUPFD, 0, NULLFLK, SYSERROR, EMFILE);
		PATH_FUNC_TRACE;

		if (testfail)
			in_rpt("fcntl(F_DUPFD) allowed more than OPEN_MAX files to be duplicated");
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(2 + loop1);

	DBUG_VOID_RETURN;
}

public void
test43()
{
	int		err;
	int		fd;
	int	pathok = 0;

	DBUG_ENTER("test43");
	testfail = NOERROR;

	testcomment = "when no fds >= arg are available, -1 returned and errno set to EMFILE";
	if (open_max <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sysconf(_SC_OPEN_MAX) failed during prep");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = creat(t43_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t43_file, (long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tb_fcntl(fd, F_DUPFD, (open_max-1), NULLFLK, SUCCEED, NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	(void) tb_fcntl(fd, F_DUPFD, (open_max-1), NULLFLK, SYSERROR, EMFILE);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

	(void) close(open_max-1);
	(void) close(fd);
	(void) unlink(t43_file);

	DBUG_VOID_RETURN;
}


public void
test44()
{
	char	*cp;
	int	maxlock;
	int	start;
	int	t44_fd;
	struct	flock	test_lock;
	int		err;
	int	pathok = 0;
	int	loop1, loop2;

	DBUG_ENTER("test44");

	testfail = NOERROR;

	if ((cp = tet_getvar("VSX_FCNTL_MAXLOCK")) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_FCNTL_MAXLOCK not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	maxlock = atoi(cp);
	if (maxlock == -1)
	{
		xx_rpt(UNTESTED); /* not "unsupported": (B) assertion */
		in_rpt("parameter VSX_FCNTL_MAXLOCK == -1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (maxlock <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_FCNTL_MAXLOCK must be > 0 or -1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "fcntl() will fail when attempting to exceed the maxium number of lock available";
	if ((t44_fd = open(t44_file, O_RDWR|O_CREAT, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR|O_CREAT) failed - errno %d(%s)",
			t44_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	SET_TIMEOUT((1 + maxlock/100)*WAITTIME)

	test_lock.l_type = F_WRLCK;
	for (start=0, loop1 = 0; start<maxlock+1 && alrm_flag == 0; start++)
	{
		if (test_lock.l_type == F_RDLCK)
			ld_lck(&test_lock, F_WRLCK, SEEK_SET, start, 1, (pid_t)0);
		else
			ld_lck(&test_lock, F_RDLCK, SEEK_SET, start, 1, (pid_t)0);

		if (fcn_to(t44_fd, F_SETLK, 0, &test_lock,
				SYSERROR, ENOLCK, WAITTIME) == SYSERROR)
			break;
		else if (loop1 == 0)
		{
			loop1 = 1;
			PATH_FUNC_TRACE;
		}
		if (testfail)
			break;
	}

	CLEAR_ALARM

	if (start == maxlock+1)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl(F_SETLK) did not fail when VSX_FCNTL_MAXLOCK (%d) was exceeded",
			maxlock);
	}
	else
		PATH_TRACE;

	(void) close(t44_fd);
	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t44_fd = open(t44_file, O_RDWR, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %d(%s)",
			t44_file, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT((1 + maxlock/100)*WAITTIME)

	test_lock.l_type = F_WRLCK;
	for (start=0, loop2 = 0; start<maxlock+1 && alrm_flag == 0; start++)
	{
		if (test_lock.l_type == F_RDLCK)
			ld_lck(&test_lock, F_WRLCK, SEEK_SET, start, 1, (pid_t)0);
		else
			ld_lck(&test_lock, F_RDLCK, SEEK_SET, start, 1, (pid_t)0);

		if (fcn_to(t44_fd, F_SETLKW, 0, &test_lock,
				SYSERROR, ENOLCK, WAITTIME) == SYSERROR)
			break;
		else if (loop2 == 0)
		{
			loop2 = 1;
			PATH_FUNC_TRACE;
		}
		if (testfail)
			break;
	}

	CLEAR_ALARM

	if (start == maxlock+1)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("fcntl(F_SETLKW) did not fail when VSX_FCNTL_MAXLOCK (%d) was exceeded",
			maxlock);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(8 + loop1 + loop2);

	(void) close(t44_fd);
	(void) unlink(t44_file);
	DBUG_VOID_RETURN;
}

public void
test45()
{
	char	*cp;
	int	err;
	int	pathok = 0;
	int	retval;

	DBUG_ENTER("test45");

	testfail = NOERROR;
	testcomment = "fcntl() will fail when attempting to create a deadly embrace";
	cp = tet_getvar("VSX_FCNTL_EDEADLK");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_FCNTL_EDEADLK is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_FCNTL_EDEADLK is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t45_syn_pa_ch = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t45_syn_ch_pa = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - opensync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((t45_fd = creat(t45_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d(%s)",
			t45_file, (long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (wrt_to(t45_fd, lckbuf, sizeof(lckbuf), WAITTIME) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - write(2) failure",EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	retval = cppair(ch_t45, pa_t45, WAITTIME, NOK | PATH_NOK);
	if (retval == PATH_NOK)
		retval = OK;
	else
		PATH_TRACE;
	if (retval == NOK)
	{
		testfail++;
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(7);

	(void) close(t45_fd);
	(void) unlink(t45_file);
	(void) closesync(t45_syn_pa_ch);
	(void) closesync(t45_syn_ch_pa);
	DBUG_VOID_RETURN;
}

private	void
pa_t45()
{
	int	child_flag;
	struct	flock	lock;
	int	pathok = 0;

	DBUG_ENTER("pa_t45");

	child_flag = 0;
	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t45_fd, F_SETLK, 0, &lock, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (waitsync(t45_syn_ch_pa[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (testfail)
	{
		child_flag++;
	}
	else
		PATH_TRACE;

	if (sendsync(t45_syn_pa_ch[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ld_lck(&lock, F_WRLCK, SEEK_SET, 20, 10, (pid_t)0);
	(void) fcn_to(t45_fd, F_SETLKW, 0, &lock, NOERROR, NOERROR, 2*WAITTIME);

	PATH_FUNC_RPT(6);

	DBUG_VOID_RETURN;
}


private	void
ch_t45()
{
	int	child_flag;
	struct		flock	lock;
	int	pathok = 0;

	DBUG_ENTER("ch_t45");

	child_flag = 0;
	ld_lck(&lock, F_WRLCK, SEEK_SET, 20, 10, (pid_t)0);
	globok = 0;
	(void) fcn_to(t45_fd, F_SETLK, 0, &lock, NOERROR, NOERROR, WAITTIME);
	PATH_FUNC_TRACE;

	if (testfail)
		child_flag++;
	else
		PATH_TRACE;

	if (sendsync(t45_syn_ch_pa[0][1], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync(t45_syn_pa_ch[0][0], (char *)&child_flag, sizeof(int), 
		WAITTIME) != sizeof(int))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (child_flag)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(WAITTIME/2);

	ld_lck(&lock, F_WRLCK, SEEK_SET, 10, 10, (pid_t)0);
	(void) fcn_to(t45_fd, F_SETLKW, 0, &lock, SYSERROR, EDEADLK, WAITTIME/2);

	if (testfail)
		DBUG_EXIT(NOK);

	if (pathok != 6)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

