/*
*      SCCS:  @(#)  POSIX.os/ioprim/fcntl_X/fcntl_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/ioprim/fcntl - T.fcntl_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.fcntl_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the fcntl() system interface - X/Open extensions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	6 Sep 1989
MODIFICATIONS:	
		Peter Lee, Unisoft Ltd.,
		Tue Aug 07 13:18:22 BST 1990
		Added resilience code.

		Geoff Clare, 3 Jan 1996
		Use VSX_INVALID_FCNTL_CMD parameter.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fcntl
#endif


#define NO_TESTS	1
#define MODEGEN		(S_IRWXU|S_IRWXG|S_IRWXO)

extern	void	valrpt();
extern	char	*errname();

int	no_tests = NO_TESTS;

private char	*t1_file	=	"./fcntl-t.1";

private void	test1();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private void
test1()
{
	/* EINVAL if cmd is invalid */

	char	*cp;
	int	fd, rval, err;
	int	badcmd = -1;
	int	pathok = 0;

	DBUG_ENTER("test1");

	if ((fd = creat(t1_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d(%s)",
			t1_file, (int)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	cp = tet_getvar("VSX_INVALID_FCNTL_CMD");
	if (cp != NULL && *cp != '\0')
		badcmd = atoi(cp);

	errno = 0;
	if ((rval = fcntl(fd, badcmd, 0)) != SYSERROR || errno != EINVAL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("fcntl(fd, %d, 0) did not give correct results",
			badcmd);
		valrpt(SYSERROR, rval, EINVAL, err);
	}
	else
		PATH_XS_RPT(1);

	(void) close(fd);
	(void) unlink(t1_file);

	DBUG_VOID_RETURN;
}
