/*
*      SCCS:  @(#)  POSIX.os/ioprim/lseek/lseek.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/ioprim/lseek - T.lseek Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.lseek

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the lseek() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Thu Mar  2 11:26:58 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 6 Sep 1989
			Use a fixed buffer size instead of PIPE_BUF
			Check failed lseek() calls do not change offset

		Peter Lee, Unisoft Ltd.,
		Tue Aug 07 13:21:43 BST 1990
		Added resilience code.

		Geoff Clare, 8 Mar 1991
			Remove test for reads in a gap

		Neil Moses, 8 Oct 1992
			XNFS enhancements

		Geoff Clare, 3 Jan 1996
			Use VSX_INVALID_WHENCE parameter

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<report.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<string.h>
#include <tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef lseek
#endif


#define	BAD_FD		(-1)			/* bad file descriptor */
#define	NO_TESTS	9			/* no. of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	FILE1		"./lseek_t.1"	/* used in tests 1-3, 6, 7 */
#define	FILE4		"./lseek_t.4"	/* used in test 4 */
#define	FILE5		"./lseek_t.5"	/* used in test 5 */
#define	FILE9		"./lseek_t.9"	/* used in test 9 */
#define	RW_MODE		(mode_t)(S_IRUSR | S_IWUSR)
#define RWX_MODE	(mode_t)(S_IRUSR | S_IWUSR | S_IXUSR)
#define	SAMPLEDATA	"0123456789"		/* written to FILE1 */
#define	SAMPLESIZE	(sizeof(SAMPLEDATA)-1)
#define	K		7
#define	J		3
#define	N		5
#define	M		9
#define BSIZ		512

extern	void	valrpt();
extern	char	*errname();
extern	char	*itoa();

public	int	no_tests = 		NO_TESTS;

private int	testfail;
private char	*testcomment;

private char	*strwhence();
private void 	prepare_tests();
private void 	test1();
private void 	test2();
private void 	test3();
private void 	test4();
private void 	cln_tests();
private void 	test5();
private void 	test6();
private void 	test7();
private void 	test8();
private void 	test9();
private off_t	tblseek();
private	void	chk_file();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	NULL, 0
};


private void
prepare_tests()
{
	int	fd;
	int	i;
	char	*r = (char *) NULL;

	DBUG_ENTER("prepare_tests");

	if ((fd = creat(FILE1, RW_MODE)) == SYSERROR)
		r = "creat call failed";

	if (write(fd, SAMPLEDATA, SAMPLESIZE) != SAMPLESIZE)
		r = "write call failed";

	if (close(fd) == SYSERROR)
		r = "close call failed";

	if (r != (char *) NULL)
		for (i = 1; i <= 4; i++)
			tet_delete(i, r);
		
	DBUG_VOID_RETURN;
}


private void
test1()
{
	int	fd;
	int	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	errno = NOERROR;
	if ((fd = open(FILE1, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open call failed with errno = %d",err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testcomment = "lseek from beginning of file";

	globok = 0;
	rval = tblseek(fd, (off_t) (K - 1), SEEK_SET, (off_t) (K - 1), NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	chk_file(fd, (off_t) (K - 1), SEEK_SET, rval );
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	(void) close(fd);
	DBUG_VOID_RETURN;
}


private void
test2()
{
	int	rval;
	int	fd;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	errno = NOERROR;
	if ((fd = open(FILE1, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open call failed with errno = %d (%s)", err,
			errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testcomment = "lseek forward from current position";

	globok = 0;
	rval = tblseek(fd, (off_t) (K + J - 1), SEEK_CUR, (off_t) (K + J - 1), 
		NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	chk_file(fd, (off_t) (K + J - 1), SEEK_CUR, rval );
	PATH_FUNC_TRACE;
	
	testcomment = "lseek backward from current position";

	/* note that chk_file() reads 1 byte, hence we expect K, not K-1 */
	globok = 0;
	rval = tblseek(fd, (off_t) -J, SEEK_CUR, (off_t) K, NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	chk_file(fd, (off_t) -J, SEEK_CUR, rval );
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(5);

	(void) close(fd);
	DBUG_VOID_RETURN;
}


private void
test3()
{
	int	rval;
	int	fd;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test3");

	errno = NOERROR;
	if ((fd = open(FILE1, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open call failed with errno = %d (%s)", err,
			errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testcomment = "lseek to end of file";

	globok = 0;
	rval = tblseek(fd, (off_t) 0, SEEK_END, (off_t) SAMPLESIZE, NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	chk_file(fd, (off_t) 0, SEEK_END, rval );
	PATH_FUNC_TRACE;

	testcomment = "lseek to one character before end of file";

	globok = 0;
	rval = tblseek(fd, (off_t) -1, SEEK_END, 
		(off_t) (SAMPLESIZE - 1), NOERROR);
	PATH_FUNC_TRACE;
	globok = 0;
	chk_file(fd, (off_t) -1, SEEK_END, rval );
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(5);

	(void) close(fd);
	DBUG_VOID_RETURN;
}


private void
test4()
{
	int	fd;
	struct	stat	statbuf;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test4");

	errno = NOERROR;
	if ((fd = creat(FILE4, RW_MODE)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat() failed with errno = %d (%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	errno = NOERROR;
	if ((fd = open(FILE4, O_RDWR)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open() failed with errno = %d (%s)", err, errname(err));
		(void) unlink(FILE4);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testcomment = "lseek does not change the size of a file";

	errno = NOERROR;
	globok = 0;
	(void) tblseek(fd, (off_t) 1, SEEK_END, (off_t) 1, NOERROR);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if (stat(FILE4, &statbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed with errno = %d (%s)", err, errname(err));
		(void) unlink(FILE4);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statbuf.st_size != (off_t) 0)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("lseek(fd , SEEK_END , 1) has changed the size of the file");
		in_rpt("expected file size of 0 - got %d",statbuf.st_size);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	globok = 0;
	(void) tblseek(fd, (off_t) BSIZ, SEEK_END, (off_t) BSIZ, NOERROR);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if (stat(FILE4, &statbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed with errno = %d (%s)", err, errname(err));
		(void) unlink(FILE4);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (statbuf.st_size != (off_t) 0)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt("lseek(fd , SEEK_END , %d) has changed the size of the file",
			BSIZ);
		in_rpt("expected file size of 0 - got %d",statbuf.st_size);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(8);
	
	(void) close(fd);
	(void) unlink(FILE4);

	DBUG_VOID_RETURN;
}


private void
cln_tests()
{
	DBUG_ENTER("cln_tests");
	(void) unlink(FILE1);
	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test5()
{
	DBUG_ENTER("test5");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test5()
{
	int	fd;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;
	testcomment = "lseek on bad file descriptor";

	globok = 0;
	(void) tblseek(BAD_FD, (off_t) 0, SEEK_SET, (off_t) SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((fd = creat(FILE5, RW_MODE)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat failed with errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (close(fd) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("close failed with errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "lseek on closed file descriptor";

	globok = 0;
	(void) tblseek(fd, (off_t) 0, SEEK_SET, (off_t) SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

	(void) unlink(FILE5);
	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test6()
{
	/* Note: this test may generate SIGSYS on some systems */
	/* If so, it should be ignored via the VSX_SIG_IGN parameter */

	char	*cp;
	int	fd;
	int	err;
	int	badwhence = -1;
	int	pathok = 0;

	DBUG_ENTER("test6");

	errno = NOERROR;
	if ((fd = open(FILE1, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open call failed with errno = %d",err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testcomment = "lseek with an invalid whence";

	cp = tet_getvar("VSX_INVALID_WHENCE");
	if (cp != NULL && *cp != '\0')
		badwhence = atoi(cp);

	globok = 0;
	(void) tblseek(fd, (off_t) J, badwhence, (off_t) SYSERROR, EINVAL);
	PATH_FUNC_TRACE;

	globok = 0;
	chk_file(fd, (off_t) J, badwhence, 0 );
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	(void) close(fd);

	DBUG_VOID_RETURN;
}


private void
test7()
{
	int	fd;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test7");

	errno = NOERROR;
	if ((fd = open(FILE1, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open call failed with errno = %d",err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testcomment = "lseek to position before start of file";

	globok = 0;
	(void) tblseek(fd, (off_t) N, SEEK_SET, (off_t) N, NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tblseek(fd, (off_t) -M, SEEK_CUR, (off_t) SYSERROR, EINVAL);
	PATH_FUNC_TRACE;

	globok = 0;
	chk_file(fd, (off_t) -M, SEEK_CUR, N );
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);
	
	(void) close(fd);

	DBUG_VOID_RETURN;
}


#if TEST_XNFS

private void
test8()
{
	DBUG_ENTER("test8");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test8()
{
	int	fildes[2];
	int	rval, err;
	char	buf[SAMPLESIZE];
	int	pathok = 0;

	DBUG_ENTER("test8");

	errno = NOERROR;
	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("Pipe failed with errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;
	testcomment = "lseek on a pipe";

	if (write(fildes[1], SAMPLEDATA, SAMPLESIZE/2) != SAMPLESIZE/2)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to pipe failed with errno = %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tblseek(fildes[1], (off_t) 1, SEEK_SET, (off_t) SYSERROR, ESPIPE);
	PATH_FUNC_TRACE;

	if (write(fildes[1], &SAMPLEDATA[SAMPLESIZE/2],
	    (unsigned)(SAMPLESIZE - SAMPLESIZE/2)) != SAMPLESIZE - SAMPLESIZE/2)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to pipe failed with errno = %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tblseek(fildes[0], (off_t) 2, SEEK_SET, (off_t) SYSERROR, ESPIPE);
	PATH_FUNC_TRACE;

	errno = 0;
	if ((rval = read(fildes[0], buf, SAMPLESIZE)) != SAMPLESIZE)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("read from pipe failed");
		valrpt(SAMPLESIZE, rval, NOERROR, err);
	}
	else if (strncmp(buf, SAMPLEDATA, (size_t)SAMPLESIZE) != 0)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("read returned wrong data from pipe");
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(6);

	(void) close(fildes[0]);
	(void) close(fildes[1]);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

#if TEST_XNFS

private void
test9()
{
	DBUG_ENTER("test9");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test9()
{
	int	fildes[2];
	int	rval, err;
	char	buf[SAMPLESIZE];
	int	pathok = 0;

	DBUG_ENTER("test9");

	errno = NOERROR;
	if (mkfifo(FILE9, RWX_MODE) == SYSERROR) 
	{ 
		err = errno;
		xx_rpt(DELETION); 
		in_rpt("mkfifo failed with errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((fildes[0] = open(FILE9, O_RDONLY | O_NONBLOCK)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(O_RDONLY|O_NONBLOCK) of FIFO failed with errno = %d (%s)", 
			err, errname(err));
		(void) unlink(FILE9);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((fildes[1] = open(FILE9, O_WRONLY | O_NONBLOCK)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(O_WRONLY|O_NONBLOCK) of FIFO failed with errno = %d (%s)", 
			err, errname(err));
		(void) unlink(FILE9);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testcomment = "lseek on a FIFO";

	if (write(fildes[1], SAMPLEDATA, SAMPLESIZE/2) != SAMPLESIZE/2)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to FIFO failed with errno = %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tblseek(fildes[1], (off_t) 1, SEEK_SET, (off_t) SYSERROR, ESPIPE);
	PATH_FUNC_TRACE;

	if (write(fildes[1], &SAMPLEDATA[SAMPLESIZE/2],
	    (unsigned)(SAMPLESIZE - SAMPLESIZE/2)) != SAMPLESIZE - SAMPLESIZE/2)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write to FIFO failed with errno = %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tblseek(fildes[0], (off_t) 2, SEEK_SET, (off_t) SYSERROR, ESPIPE);
	PATH_FUNC_TRACE;

	errno = 0;
	if ((rval = read(fildes[0], buf, SAMPLESIZE)) != SAMPLESIZE)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("read from FIFO failed");
		valrpt(SAMPLESIZE, rval, NOERROR, err);
	}
	else if (strncmp(buf, SAMPLEDATA, (size_t)SAMPLESIZE) != 0)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("read returned wrong data from FIFO");
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(8);

	(void) close(fildes[0]);
	(void) close(fildes[1]);
	(void) unlink(FILE9);
	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */


private off_t
tblseek(tbfildes, tboffset, tbwhence, expret, experrno)
int tbfildes, tbwhence;
off_t tboffset;
off_t expret;
int experrno;
{
	off_t rval;
	int err;
	int	pathok = 0;

	DBUG_LENTER("tblseek", 2);

	errno = 0;
	rval = lseek(tbfildes, (off_t)tboffset, tbwhence);
#ifdef TESTBED
	rval = -1L;
	if (errno)
		errno++;
#endif

	err = errno;
	globok = 0;

	if (expret != rval || experrno != err)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("lseek(%d, %ld, %s) failed", tbfildes, (long)tboffset,
			strwhence(tbwhence));
		if (testcomment != (char *) NULL)
			in_rpt(testcomment);
		valrpt((int) expret, (int) rval, experrno, err);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(rval);
}

private void
chk_file(tbfildes, tboffset, tbwhence, offset)
int tbfildes, tbwhence;
off_t tboffset;
int offset;
{
	int rdlen, rval;
	char	buf[2];
	int	pathok = 0;

	DBUG_ENTER("chk_file");

	globok = 0;

	if (offset == (off_t) strlen(SAMPLEDATA))
		rdlen = 0;
	else
		rdlen = 1;

	if ((rval = read(tbfildes, buf, 1)) != rdlen)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("lseek(%d, %ld, %s) failed", tbfildes, (long)tboffset,
			strwhence(tbwhence));
		if (testcomment != (char *) NULL)
			in_rpt(testcomment);
		in_rpt("read returned %d bytes instead of %d bytes", 
			rval, rdlen);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (!rdlen)
	{
		PATH_FUNC_RPT(1);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (buf[0] != SAMPLEDATA[(int) offset])
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("lseek(%d, %ld, %s) failed", tbfildes, (long)tboffset,
			strwhence(tbwhence));
		if (testcomment != (char *) NULL)
			in_rpt(testcomment);
		in_rpt("character at current position: Observed %c", buf[0]);
		in_rpt("character at current position: Expected %c", 
			SAMPLEDATA[(int) offset]);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private char
*strwhence(tbwhence)
int tbwhence;
{
	switch(tbwhence)
	{
	case SEEK_SET:
		return("SEEK_SET");
	case SEEK_CUR:
		return("SEEK_CUR");
	case SEEK_END:
		return("SEEK_END");
	default:
		return(itoa(tbwhence));
	}
}
