/*
*      SCCS:  @(#)  POSIX.os/ioprim/pipe/pipe.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/ioprim/pipe - T.pipe Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.pipe

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the pipe() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Tue Feb 28 16:28:35 GMT 1989
MODIFICATIONS:	
		Steve Ratcliffe, 19 Feb 1990
			Add ENFILE test.
			The temp file in test9 never got deleted.
		
		Geoff Clare, 30 May 1990
			Allow for unlimited OPEN_MAX
		
		Geoff Clare, 21 June 1990
			Add FD_CLOEXEC clear test

		Peter Lee, Unisoft Ltd.,
		Tue Aug 07 16:30:42 BST 1990
		Added resilience code.
		Modification of test6 to allow sync pipes to pass
			path checking values.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ,
			Eliminated reason().

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 20 July 1992
			Change savchild from int to pid_t.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<signal.h>
#include	<report.h>
#include	<limits.h>
#include	<exitcodes.h>
#include	<fcntl.h>
#include	<synch.h>
#include	<timeout.h>
#include	<chktimes.h>
#include	<pcts_max.h>
#include	<stdlib.h>
#include	<string.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef pipe
#endif


#define	NO_TESTS	10		/* number of tests in testset */
#define EXTERR		"External error"
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	SYNCSIZE	4		/* number of bytes sent by sendsync */
#define	PART1		"abcdefgh"
#define	PART2		"ijklmnop"
#define	PART3		"qrstuvwx"
#define MESSAGE		"abcdefghijklmnopqrstuvwx"
#define	TOCHILD1	0		/* used in test 3, child sync */
#define	FRCHILD1	1
#define	TOCHILD2	2
#define	FRCHILD2	3
#define	FILE9		"./pipe_t.9"	/* used in test9 */
#define	READ		0
#define	WRITE		1

private	char	*prog10 = "../pipe_t10";

extern	void	valrpt();
extern 	int	n_open_fds();
extern	char	*errname();
extern	char	*itoa();
extern	char	*newstr();

extern	int 	vguid1;
extern	int 	vggid1;
extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

int	no_tests = NO_TESTS;

private	int	fildes[2];	/* used for pipe() call */
private synch	*syn[4];
private int	testfail;
private	int	open_max;

private void 	prepare_tests();
private void 	test1();
private void 	test2();
private void 	test3();
private void 	test4();
private void 	test5();
private void 	test6();
private void	ch1_t6();
private void	ch2_t6();
private void	pa1_t6();
private void	pa2_t6();
private void 	test7();
private void 	test8();
private void 	test9();
private void 	test10();
private void	ch_t9();
private void	ch_t10();

private int 	tb_pipe();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	NULL, 0
};

private void
prepare_tests()	
{
	DBUG_ENTER("prepare_tests");

	if(uids()!=0)
	{
		int i;
		for(i=1; i <= no_tests; i++)
		{
			(void) tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	(void) umask((mode_t)0);

	errno = 0;
	if ((open_max = sysconf(_SC_OPEN_MAX)) == -1 && errno == 0)
		open_max = PCTS_OPEN_MAX+1;

	DBUG_VOID_RETURN;
}

private void
test1()
{
	/* file descriptors from pipe have correct mode */

	int	rdonly;
	int	wronly;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	globok = 0;
	(void) tb_pipe(fildes, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if(testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rdonly = fcntl(fildes[READ], F_GETFL, 0)) == SYSERROR  ||
		(wronly = fcntl(fildes[WRITE], F_GETFL, 0)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(fildes, F_GETFL, 0)) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	rdonly &= O_ACCMODE;
	wronly &= O_ACCMODE;

	if (rdonly == O_WRONLY || wronly == O_RDONLY)
	{
		testfail++;
		xx_rpt(FAILURE);
		if (rdonly == O_WRONLY)
			in_rpt("O_WRONLY flag set on fildes[0]");
		if (wronly == O_RDONLY)
			in_rpt("O_RDONLY flag set on fildes[1]");
	}
	else
		PATH_XS_RPT(3);

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	DBUG_VOID_RETURN;
}

private void
test2()
{
	/* data can be written to fildes[1] */

	int	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	globok = 0;
	(void) tb_pipe(fildes, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if(testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = write(fildes[WRITE], MESSAGE, (unsigned) strlen(MESSAGE));
	if (rval != strlen(MESSAGE) || errno != NOERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("write(fildes[1]...) failed");
		valrpt((int)strlen(MESSAGE),rval,NOERROR,err);
	}
	else
		PATH_XS_RPT(2);

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	DBUG_VOID_RETURN;
}

private void
test3()
{
	/* data can be read from fildes[0] */

	int	rval;
	char	buf[sizeof(MESSAGE)];	
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test3");

	testfail = 0;

	globok = 0;
	(void) tb_pipe(fildes, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if(testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = write(fildes[WRITE], MESSAGE, (unsigned) strlen(MESSAGE));
	if (rval != strlen(MESSAGE) || errno != NOERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write(fildes[1]...) failed");
		valrpt((int)strlen(MESSAGE),rval,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = read(fildes[READ], buf, (unsigned) strlen(MESSAGE));
	if (rval != strlen(MESSAGE) || errno != NOERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("read(fildes[0]...) failed");
		valrpt((int)strlen(MESSAGE),rval,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (strncmp(MESSAGE , buf , (size_t)strlen(MESSAGE)) != NOERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("read(fildes[0]...) has found unexpected data");
		in_rpt("expected to read \"%s\" - but got \"%s\" ",MESSAGE,buf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_XS_RPT(4);

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	DBUG_VOID_RETURN;
}

private void
test4()
{
	/* pipe() returns the 2 lowest file descriptors */

	int	fd;
	int	pathok = 0;

	DBUG_ENTER("test4");

	testfail = 0;

	if (open_max < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sysconf(_SC_OPEN_MAX) failed in prep");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(0);

	/* find the lowest unused file descriptor available */
	for (fd=1 ; fd<open_max ; fd++)
	{
		if (dup2(fd , fd) == SYSERROR)
			break;
	}
	if (fd >= open_max)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: unable to find an unused file descriptor");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tb_pipe(fildes, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if(testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((fildes[READ] != 0 && fildes[READ] != fd) || 
		(fildes[WRITE] != 0 &&fildes[WRITE] != fd) ||
			(fildes[READ] == fildes[WRITE]))
	{
		xx_rpt(FAILURE);
		in_rpt("pipe() has failed to return the two lowest file descriptors");
		in_rpt("expected the values 0 and %d - got %d and %d",
			fd,fildes[READ],fildes[WRITE]);
	}
	else
		PATH_XS_RPT(4);

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	DBUG_VOID_RETURN;
}

private void
test5()
{
	/* file descriptors from pipe have O_NONBLOCK clear */

	int	rd_ndelay;
	int	wr_ndelay;
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

	globok = 0;
	(void) tb_pipe(fildes, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rd_ndelay = fcntl(fildes[READ], F_GETFL, 0)) == SYSERROR  ||
		(wr_ndelay = fcntl(fildes[WRITE], F_GETFL, 0)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(fildes, F_GETFL, 0)) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((rd_ndelay & O_NONBLOCK) || (wr_ndelay & O_NONBLOCK))
	{
		xx_rpt(FAILURE);
		testfail++;
		if (rd_ndelay & O_NONBLOCK)
			in_rpt("O_NONBLOCK flag set on read side of pipe");
		if (wr_ndelay & O_NONBLOCK)
			in_rpt("O_NONBLOCK flag set on write side of pipe");
	}
	else
		PATH_XS_RPT(3);

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	DBUG_VOID_RETURN;
}


private void
test6()
{
	/* data read from a pipe is in FIFO order */

	int	i;
	int	pathok = 0;

	DBUG_ENTER("test6");
	
	testfail = 0;

	globok = 0;
	(void) tb_pipe(fildes, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < 4; i++)
		if ((syn[i] = opensync()) == (synch *) NULL)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: %s - opensync failed", EXTERR);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

	PATH_FUNC_RPT(6);
	(void) cppair(ch1_t6, pa1_t6, WAITTIME, 0);

	for (i = 0; i < 4; i++)
		(void) closesync(syn[i]);
	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);

	DBUG_VOID_RETURN;
}

private void
ch1_t6()
{
	int	rval;
	char	syncbuf[SYNCSIZE];
	int	pathok = 0;

	DBUG_ENTER("ch1_t6");

	PATH_FUNC_TRACE; /* test path count of calling function test6 */

	if (waitsync((*syn[TOCHILD1])[READ], syncbuf, sizeof(syncbuf),
		2*WAITTIME) != sizeof(syncbuf))
		exit(A_WAITSYNC);
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	rval = write(fildes[WRITE], PART2, (unsigned) strlen(PART2));

	CLEAR_ALARM

	if (rval != strlen(PART2))
		exit(A_WRITEFAIL);
	else
		PATH_TRACE;

	/* use this sync pipe to pass path check value to pa2_t6 */
	*syncbuf = (char) pathok;
	if (sendsync((*syn[FRCHILD1])[WRITE], syncbuf,
		sizeof(syncbuf), WAITTIME) != sizeof(syncbuf))
		exit(A_SENDSYNC);

	if (waitsync((*syn[TOCHILD1])[READ], syncbuf, sizeof(syncbuf),
		2*WAITTIME) != sizeof(syncbuf))
		exit(A_WAITSYNC);

	DBUG_VOID_RETURN;
}

private void
pa1_t6()
{
	char	syncbuf[SYNCSIZE];
	pid_t	savchild;

	DBUG_ENTER("pa1_t6");

	/* this function cannot be path checked, as all the action 
		has happened when cppair returns */

	savchild = tet_child;
	(void) cppair(ch2_t6, pa2_t6, WAITTIME, 0);
	tet_child = savchild;

	if (sendsync((*syn[TOCHILD1])[WRITE],syncbuf,
		sizeof(syncbuf), WAITTIME) != sizeof(syncbuf))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	globok = 1;
	
	DBUG_VOID_RETURN;
}

private void
ch2_t6()
{
	int	rval;
	char	syncbuf[SYNCSIZE];
	int	pathok = 0;

	DBUG_ENTER("ch2_t6");

	if (waitsync((*syn[TOCHILD2])[READ], syncbuf, 
		sizeof(syncbuf), WAITTIME) != sizeof(syncbuf))
		exit(A_WAITSYNC);
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	rval = write(fildes[WRITE], PART1, (unsigned) strlen(PART1));

	CLEAR_ALARM

	if (rval != strlen(PART1))
	{
		exit(A_WRITEFAIL);
	}
	else
		PATH_TRACE;

	if (sendsync((*syn[FRCHILD2])[WRITE],syncbuf,
		sizeof(syncbuf), WAITTIME) != sizeof(syncbuf))
		exit(A_SENDSYNC);
	else
		PATH_TRACE;

	if (waitsync((*syn[TOCHILD2])[READ],syncbuf,
		sizeof(syncbuf), WAITTIME) != sizeof(syncbuf))
		exit(A_WAITSYNC);
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	rval = write(fildes[WRITE], PART3, (unsigned) strlen(PART3));

	CLEAR_ALARM

	if (rval != strlen(PART3))
		exit(A_WRITEFAIL);
	else
		PATH_TRACE;

	/* send path count to pa2_t6 */
	*syncbuf = (char) pathok;
	if (sendsync((*syn[FRCHILD2])[WRITE],syncbuf,
		sizeof(syncbuf), WAITTIME) != sizeof(syncbuf))
		exit(A_SENDSYNC);

	DBUG_VOID_RETURN;
}

private void
pa2_t6()
{
	int	rval;
	char	syncbuf[SYNCSIZE];
	char	buf[sizeof(MESSAGE)];	
	int	pathok = 0;

	DBUG_ENTER("pa_t6");

	if (sendsync((*syn[TOCHILD2])[WRITE], syncbuf,
		sizeof(syncbuf), WAITTIME) != sizeof(syncbuf))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (waitsync((*syn[FRCHILD2])[READ],syncbuf,
		sizeof(syncbuf), WAITTIME) != sizeof(syncbuf))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (sendsync((*syn[TOCHILD1])[WRITE],syncbuf,
		sizeof(syncbuf), WAITTIME) != sizeof(syncbuf))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check path count from ch1_t6 */
	if (waitsync((*syn[FRCHILD1])[READ],syncbuf,
		sizeof(syncbuf), WAITTIME) != sizeof(syncbuf))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else if (*syncbuf == (char) 3)
		PATH_TRACE;
	
	if (sendsync((*syn[TOCHILD2])[WRITE],syncbuf,
		sizeof(syncbuf), WAITTIME) != sizeof(syncbuf))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - sendsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* get path count from ch2_t6 */
	if (waitsync((*syn[FRCHILD2])[READ],syncbuf,
		sizeof(syncbuf), WAITTIME) != sizeof(syncbuf))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitsync failed", EXTERR);
		DBUG_VOID_RETURN;
	}
	else if (*syncbuf == (char) 5)
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	rval = read(fildes[READ], buf, (unsigned) strlen(MESSAGE));

	CLEAR_ALARM

	if (rval != strlen(MESSAGE))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - read(2) failure", EXTERR);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	buf[strlen(MESSAGE)] = NUL;

	if (strcmp(buf, MESSAGE) != 0)
	{
		xx_rpt(FAILURE);
		testfail++;
		in_rpt("Messages sent were `%s', `%s' and `%s'",
			PART1, PART2, PART3);
		in_rpt("Message received was %s instead of %s",
			buf, MESSAGE);
	}
	else
		PATH_XS_RPT(7);
	globok = 1;
	
	DBUG_VOID_RETURN;
}

private void
test7()
{
	/* st_ctime, st_atime and st_mtime shall be marked for update */

	char	*mess = NULL;
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	globok = 0;
	(void) tb_pipe(fildes, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if(testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* save the start times	*/
	globok = 0;
	fsavtimes(fildes[READ]);
	PATH_FUNC_TRACE;

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);

	globok = 0;
	(void) tb_pipe(fildes, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if(testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

/* compare current times against saved times */
	globok = 0;
	if ((mess = fchktimes(fildes[READ],NEWATIME|NEWMTIME|NEWCTIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(mess);
		testfail++;
		(void) close(fildes[READ]);
		(void) close(fildes[WRITE]);
		DBUG_VOID_RETURN; 
	}
	else
		PATH_FUNC_TRACE;

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	globok = 0;
	(void) tb_pipe(fildes, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

/* save the start times	*/
	globok = 0;
	fsavtimes(fildes[WRITE]);
	PATH_FUNC_TRACE;

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);

	globok = 0;
	(void) tb_pipe(fildes, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if(testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


/* compare current times against saved times */
	globok = 0;
	if ((mess = fchktimes(fildes[WRITE],NEWATIME|NEWMTIME|NEWCTIME)) != NULL)
	{
		xx_rpt(FAILURE);
		in_rpt(mess);
		testfail++;
	}
	else
		PATH_FUNC_TRACE;

	if(!testfail)
		PATH_XS_RPT(11);

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	/* FD_CLOEXEC clear on both file descriptors */

	int err, flags;
	int pathok = 0;

	DBUG_ENTER("test8");

	testfail = 0;

	globok = 0;
	(void) tb_pipe(fildes, NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail > 0)
		DBUG_VOID_RETURN
	else
		PATH_TRACE;

#ifdef TESTBED2
	(void) fcntl(fildes[READ], F_SETFD, FD_CLOEXEC);
	(void) fcntl(fildes[WRITE], F_SETFD, FD_CLOEXEC);
#endif

	if ((flags = fcntl(fildes[READ], F_GETFD, 0)) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("fcntl(F_GETFD) failed - errno %d (%s)",
			err, errname(err));
	}
	else if ((flags & FD_CLOEXEC) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("FD_CLOEXEC was set on reading side of pipe");
	}
	else
		PATH_TRACE;

	if ((flags = fcntl(fildes[WRITE], F_GETFD, 0)) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("fcntl(F_GETFD) failed - errno %d (%s)",
			err, errname(err));
	}
	else if ((flags & FD_CLOEXEC) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("FD_CLOEXEC was set on writing side of pipe");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	DBUG_ENTER("test9");

	if (open_max < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sysconf(_SC_OPEN_MAX) failed in prep");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t9, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int	i;
	int	max_fd;
	int	no_open_fds;
	int	fd;
	int	(*fds)[2];
	int	rval = 1;
	int filecreated = 0;
	int	pathok = 0;
	int	loop1;

	DBUG_ENTER("ch_t9");

	testfail = 0;

	fds = (int (*)[2]) malloc((size_t)((open_max/2 + 1) * sizeof(*fds)));
	if (fds == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: malloc() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	i = 0;
	no_open_fds = n_open_fds();

	/* ensure that an odd number of fds are available */
	if (no_open_fds % 2 == 0)
	{
		fd = creat(FILE9 , (S_IRUSR|S_IWUSR));
		no_open_fds++;
		filecreated++;
	}

	max_fd = (open_max <= PCTS_OPEN_MAX) ? (open_max - 1) : PCTS_OPEN_MAX;

	for (loop1 = 0 ; no_open_fds < max_fd ; no_open_fds += 2)
	{
		globok = 0;
		(void) tb_pipe(fds[i++], NOERROR, NOERROR);
		if (testfail)
			break;
		else
		{
			loop1++;
			PATH_FUNC_TRACE;
		}
	}

	if (!testfail)
	{
		if (open_max <= PCTS_OPEN_MAX)
			rval = tb_pipe(fds[i], SYSERROR, EMFILE);
		if (rval == 0)
			no_open_fds += 2;
	}
	
 	if (testfail)
	{
		in_rpt("no. file descriptors open (%d) > OPEN_MAX (%d)",
			no_open_fds, open_max);
	}
	else
		PATH_XS_RPT(1 + loop1);

	while ( --i >= 0)
	{
		(void) close(fds[i][READ]);
		(void) close(fds[i][WRITE]);
	}
	if (filecreated)
	{
		(void) close(fd);
		(void) unlink(FILE9);
	}

	DBUG_VOID_RETURN;
}

private void
test10()
{

	DBUG_ENTER("test10");

	(void) cppair(ch_t10, NULLFN, 8 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	char	*suid, *sgid;
	int 	err;
	char	*args[8];

	DBUG_ENTER("ch_t10");

	testfail = 0;

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
		exit(A_NOMEM);
	if ((sgid = newstr(itoa(vggid1))) == (char *) NULL)
		exit(A_NOMEM);

	args[0] = prog10;
	args[1] = "";
	args[2] = suid;
	args[3] = sgid;
	args[4] = "p";
	args[5] = vg_ident;
	args[6] = vg_dbug;
	args[7] = NULL;

	(void) tet_exec(prog10, args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %d (%s)", err, errname(err));

	DBUG_VOID_RETURN;
}

private int 
tb_pipe(tbfildes, expret, experrno)
int tbfildes[2];
int expret, experrno;
{
	int rval;
	int err;
	int	pathok = 0;

	DBUG_LENTER("tbfildes", 2);

	errno = 0;
	rval = pipe(tbfildes);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;
	globok = 0;

	if (expret == 0)
	{
		if (rval == 0)
		{
			PATH_FUNC_RPT(0);
			DBUG_RETURN(rval);
		}
	}
	else
	{
		if (expret == rval && experrno == err)
		{
			PATH_FUNC_RPT(0);
			DBUG_RETURN(rval);
		}
	}

	if (!testfail++)
		xx_rpt(FAILURE);
	in_rpt("pipe(fildes) failed");
	valrpt(expret, rval, experrno, err);

	DBUG_RETURN(rval);
}
