/*
*      SCCS:  @(#)  POSIX.os/ioprim/read/read.c Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/ioprim/read - T.read Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.read

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the read() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Thu Mar  9 16:40:50 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 6 July 1989
			Extensive tidy-up.

		Geoff Clare, 7 Sept 1989
			More tidy-up.
		
		Geoff Clare, 25 Apr 1990
			Varying argument types.
		
		Geoff Clare, 4 June 1990
			Remove erroneous DBUG_VOID_RETURN in setup.
		
		J.A. Nave, 6 June 1990
			Add timestamp checking to test2.

		Geoff Clare, 7 June 1990
			UNSUPPORTED result if termios_prep() returns 1

		Peter Lee, Unisoft Ltd.,
		Wed Aug 08 11:03:51 BST 1990
		Added resilience code.

		Stuart Boutell, 26 Mar 1991
			Renumbered test 15-19 to 16-20,
		Inserted test 15 (st_atime updated on successful zero byte
		read).

		Geoff Clare, 15 May 1992
			In "gap" read test, repeat with fseek().

		Neil Moses, 9 June 1992
			New test 21.  On ROF time stamps are not changed.

                John-Paul, 29 July 1992
                        Mode parameter in calls to crfile() and crdir()
                        changed from 'int' to 'mode_t' type.

		Neil Moses, 8 Oct 1992
			XNFS enhancements

		Geoff Clare, 28 Oct 1993
			Use new files in XNFS st_atime tests to avoid
			reading cached data.

************************************************************************/
#define NO_VREAD

#include	<std.h>
#include	<stdio.h>
#include	<errno.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<signal.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<string.h>
#include	<timeout.h>
#include	<limits.h>
#include	<stdlib.h>
#include	<chktimes.h>
#include 	<termios.h>
#include 	<time.h>
#include	<tet_api.h>
#include	<xnfs.h>
#include	"read_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef read
#endif

#ifndef _XOPEN_SOURCE
#undef memset
extern	void	*memset();
#endif


#define NO_TESTS	23		/* number of tests in testset */
#define	OFF_SET		10
#define WAITTIME	(5 * SPEEDFACTOR)
#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	READ		0		/* fd for reading from a pipe */
#define	WRITE		1		/* fd for writting to a pipe */
#define NAP		3
#define NOK		02
#define EXP_SIZE	(sizeof(exp_data)-1)
#define PATH_NOK	8

extern	char	*errname();
extern	char	*execpath();
extern	void	valrpt();
extern	void	do_feio();
extern	char	*psl_to_asc();
extern	char	*itoa();
extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

int	no_tests = NO_TESTS;

#if TEST_XNFS
private	int	snames[] = {1, 0};
#endif /* TEST_XNFS */

private char	exp_data[] =	"1234567890abcdefghijklmnopqrstuvwxyz";
private	int	caught_sig = 0;
private int	notinuse = 0;
private int	alt_errno = NOERROR;
private int	testfail;
#if !TEST_XNFS
private char	*sigstat;
#endif
private char	*in_buf, *out_buf;
private int	pipe_buf;
#if !TEST_XNFS
private int	fildes[2];
#endif
private int	setupok;

private	char	*prog_su =	"read_su";
private char	*t_file =	"./read-t";
private char	*t1_file =	"./read-t.1";
#if !TEST_XNFS
private char	*t5_file =	"./read-t.5";
private char	*t8_file =	"./read-t.8";
private char	*t10_file =	"./read-t.10";
private char	*t11_file =	"./read-t.11";
#endif
private char	*t12_file =	"./read-t.12";
#if TEST_XNFS
private char	*t13_file =	T13_FILE;
private char	*t15_file =	T15_FILE;
#else
#define		t13_file	t_file
private char	*t13_fifo =	"./read-p.13";
private char	*t14_file =	"./read-t.14";
#define		t15_file	t_file
private char	*t16_file =	"./read-t.16";
#endif
private char	*t17_file =	"./read-t.17";
#if !TEST_XNFS
private char	*t18_file =	"./read-t.18";
#endif
private char	*t21_dir =	"read-d.21";
private char	*t21_file =	"read-d.21/fread-t.21";
#if TEST_XNFS
private	char	*t22_file = 	T22_FILE;
private	char	*t23_file = 	T23_FILE;
#endif

private void	prepare_tests();
private void	cln_tests();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
#if !TEST_XNFS
private void	ch_t7();
#endif
private void	test8();
#if !TEST_XNFS
private void	ch_t8();
private void	pa_t8();
#endif
private void	test9();
private void	test10();
private void	test10();
private void	test11();
#if !TEST_XNFS
private void	ch_t11();
private void	pa_t11();
#endif
private void	test12();
private void	t12_read();
private void	test13();
private void	test14();
private	void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	ch_t18();
private void	pa_t18();
private void	test19();
#if !TEST_XNFS
private void	t19rpt();
private int	t19_20io();
#endif
private void	test20();
#if !TEST_XNFS
private void	t20rpt();
#endif
private void	test21();
private void	ch_t21();
private void	test22();
#if TEST_XNFS
private int	t22_func();
private int	t22_chk_func();
#endif
private void	test23();
#if TEST_XNFS
private void	ch_t23();
#endif

private	int	tbread();
private	int	tbrd_to();

public	void	(*setfunc) () = prepare_tests;

public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	NULL, 0
};

private void
prepare_tests()
{
	int	fd;
	int 	i;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	if ((fd = crfile(t_file, MODEANY, (int) getuid(), (int) getgid())) == SYSERROR)
	{
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i, "creat() failed in preparations");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, exp_data, (unsigned) EXP_SIZE) != EXP_SIZE)
	{
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i, "write failed in preparations");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fd);

	if (sysconf(_SC_JOB_CONTROL) == -1)
		notinuse = 1;

#if !TEST_XNFS

	/* obtain the value of PIPE_BUF from pathconf() */
	if((pipe_buf = pathconf(".", _PC_PIPE_BUF)) == SYSERROR)
	{
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i,
				"pathconf(\".\", _PC_PIPE_BUF) failed in prep");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else
	PATH_TRACE;

	pipe_buf = 1024;

#endif /* TEST_XNFS */

	out_buf = (char *) malloc((size_t) (2*pipe_buf + 1));
	in_buf = (char *) malloc((size_t) (2*pipe_buf + 1));

	if (out_buf == NULL || in_buf == NULL)
	{
		for (i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "malloc() failed in prep");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* NB this will contain nulls, so don't use strcmp() etc. on it */
	/* Start at 1 so as much as possible can be printed with psl_to_asc() */
	for (i = 0; i < 2 * pipe_buf; i++)
		out_buf[i] = 1 + i % (CHAR_MAX - CHAR_MIN + 1);

	PATH_SETUP_RPT(4);

	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	(void) unlink (t_file);
}

private int
getfifo(path, fds, waittime)
char *path;
int fds[2];
unsigned waittime;
{
	/* create and open a FIFO.  N.B. it is left in NONBLOCK mode */

	int err;

	DBUG_ENTER("getfifo");

	(void) remove(path);
	if (mkfifo(path, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(%s, %#o) failed - errno %d (%s)",
			path, (int)MODEANY, err, errname(err));
		DBUG_RETURN(-1);
	}

	SET_TIMEOUT(waittime)
	fds[READ] = open(path, O_RDONLY|O_NONBLOCK);
	CLEAR_ALARM

	if (fds[READ] < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: open(O_RDONLY|O_NONBLOCK) failed on FIFO %s - errno %d (%s)",
			path, err, errname(err));
		DBUG_RETURN(-1);
	}

	SET_TIMEOUT(waittime)
	fds[WRITE] = open(path, O_WRONLY|O_NONBLOCK);
	CLEAR_ALARM

	if (fds[WRITE] < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: open(O_WRONLY|O_NONBLOCK) failed on FIFO %s - errno %d (%s)",
			path, err, errname(err));
		(void) close(fds[READ]);
		DBUG_RETURN(-1);
	}

	DBUG_RETURN(0);
}


private void
sig_catch(sig)
{
	caught_sig = sig;
}

private void
test1()
{
	int		fd;
	int		i;
	int		rval;
	int		err;
	int		pathok = setupok;
	int		loop1;
#if !TEST_XNFS
	int		loop2, loop3;
#endif

	DBUG_ENTER("test1");
/* for a regular file */

	fd = creat(t1_file , (mode_t)S_IRWXU|S_IRWXG|S_IRWXO);
	if (fd < 0)
	{
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d (%s)",
			t1_file, (int)MODEANY, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);
	testfail = 0;

	for (i = 1, loop1 = 0 ; i<_POSIX_PIPE_BUF ; i *= 2, loop1++)
	{

		errno = NOERROR;
		if ((fd = open(t1_file, O_WRONLY|O_TRUNC)) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: could not open file %s", 
				t1_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		errno = NOERROR;
		if ((rval = write(fd , out_buf , (unsigned) i)) != i)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: write() failed");
			valrpt(i,rval,NOERROR,err);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		(void) close(fd);

		errno = NOERROR;
		if ((fd = open(t1_file, O_RDONLY)) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: could not open file %s", 
				t1_file);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		(void) tbread(fd, in_buf, (unsigned) (i), (i), NOERROR, 
			out_buf);
		PATH_FUNC_TRACE;

		(void) close(fd);
	}

	(void) unlink(t1_file);

#if !TEST_XNFS

/* for a FIFO */
	if (getfifo(t1_file, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=1 , loop2 = 0; i<_POSIX_PIPE_BUF ; i *= 2, loop2++)
	{
		errno = NOERROR;
		if ((rval = write(fildes[WRITE] , out_buf , (unsigned) i)) != i)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: write() failed");
			valrpt(i,rval,NOERROR,err);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		(void) tbread(fildes[READ], in_buf, (unsigned) (i), i, 
			NOERROR,out_buf);
		PATH_FUNC_TRACE;

	}

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);
	(void) unlink(t1_file);

/* for a pipe */
	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: pipe() failed - errno: %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	for (i=1, loop3 = 0 ; i<_POSIX_PIPE_BUF ; i *= 2, loop3++)
	{
		errno = NOERROR;
		if ((rval = write(fildes[WRITE] , out_buf , (unsigned) i)) != i)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: write() failed");
			valrpt(i,rval,NOERROR,err);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		(void) tbread(fildes[READ], in_buf, (unsigned) (i), i, 
			NOERROR,out_buf);
		PATH_FUNC_TRACE;
	}

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);

	if (!testfail)
		PATH_XS_RPT(3 + 4 * loop1 + 2 * loop2 + 2 * loop3);

#else

	if (!testfail)
		PATH_XS_RPT(1 + (4 * loop1));

#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int		fd;
	int		rval;
	int		err;
	char		*mess;
	int		pathok = setupok;

	DBUG_ENTER("test2");
	if ((fd = open(t_file, O_RDONLY, 0)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	globok = 0;
	fsavtimes(fd);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbread(fd, in_buf, (unsigned) 0, 0, NOERROR, (char *)NULL);
	PATH_FUNC_TRACE;

	globok = 0;
	if ((mess = fchktimes(fd,0)) != (char *)NULL)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("%s", mess);
	}
	else
		PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((rval = lseek(fd, (off_t)0, SEEK_CUR)) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: lseek() failed");
		valrpt(0,rval,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (rval != 0)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("read() with number of bytes to be read = 0 has changed the file pointer");
		in_rpt("file points to byte %d instead of byte 0",rval);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((rval = lseek(fd, (off_t)OFF_SET, SEEK_CUR)) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: lseek() failed");
		valrpt(OFF_SET,rval,NOERROR,err);
	}
	else
		PATH_TRACE;

	globok = 0;
	fsavtimes(fd);
	PATH_FUNC_TRACE;


	globok = 0;
	(void) tbread(fd, in_buf, (unsigned) 0, 0, NOERROR, (char *)NULL);
	PATH_FUNC_TRACE;

	globok = 0;
	if ((mess = fchktimes(fd,0)) != (char *)NULL)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("%s", mess);
	}
	else
		PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((rval = lseek(fd, (off_t)0, SEEK_CUR)) == SYSERROR)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("deletion reason: lseek() failed");
		valrpt(0,rval,NOERROR,err);
	}
	else
		PATH_TRACE;

	if (rval != OFF_SET)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("read() with number of bytes to be read = 0 has changed the file pointer");
		in_rpt("file points to byte %d instead of byte %d",
			rval,OFF_SET);
	}
	else
		PATH_TRACE;

	if (!testfail)
		PATH_XS_RPT(12);

	(void) close(fd);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int	fd;
	int	offset;
	int	rval;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test3");
	if ((fd = open(t_file, O_RDONLY, 0)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	for (offset=1 ; offset<EXP_SIZE ; offset+=2)
	{
		errno = NOERROR;
		if ((rval = lseek(fd, (off_t)offset, SEEK_SET)) == SYSERROR)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("lseek() failed");
			valrpt(offset,rval,NOERROR,err);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		(void) tbrd_to(fd, in_buf, 1, WAITTIME, 1, NOERROR, 
			&exp_data[offset]);
		PATH_FUNC_TRACE;

	}

	if (!testfail)
		PATH_XS_RPT(1 + offset - 1);

	(void) close(fd);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test4");
	if ((fd = open(t_file, O_RDONLY, 0)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	globok = 0;
	(void) tbread(fd, in_buf, (unsigned) (EXP_SIZE/2), (EXP_SIZE/2), 
		NOERROR, exp_data);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbread(fd, &in_buf[EXP_SIZE/2], (unsigned) (EXP_SIZE/2), 
		(EXP_SIZE/2), NOERROR, &exp_data[EXP_SIZE/2]);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	(void) close(fd);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test5()
{
	DBUG_ENTER("test5");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test5()
{
	int	rval;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test5");

	testfail = 0;

	if (getfifo(t5_file, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	rval = write(fildes[WRITE] , exp_data, EXP_SIZE);

	if (rval != EXP_SIZE)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() failed");
		valrpt(EXP_SIZE,rval,NOERROR,err);
		(void) close(fildes[READ]);
		(void) close(fildes[WRITE]);
		(void) unlink(t5_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbread(fildes[READ], in_buf, (unsigned) (EXP_SIZE/2),
		(EXP_SIZE/2), NOERROR, exp_data);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbread(fildes[READ], in_buf, (unsigned) (EXP_SIZE/2), 
		(EXP_SIZE/2), NOERROR, &exp_data[EXP_SIZE/2]);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	(void) unlink(t5_file);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test6()
{
	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test6");

	testfail = 0;

	errno = NOERROR;
	if ((fd = open(t_file, O_RDONLY)) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrd_to(fd, in_buf, (2*EXP_SIZE), WAITTIME,
		EXP_SIZE, NOERROR, exp_data);
	PATH_FUNC_TRACE;

	(void) close(fd);

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test7()
{
	DBUG_ENTER("test7");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test7()
{
	DBUG_ENTER("test7");

	(void) cppair(ch_t7, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int	ret, err;
	int	tty_fildes, loop_fildes;
	struct termios tty_termios, loop_termios;
	int	pathok = setupok;

	DBUG_ENTER("ch_t7");

	testfail = 0;

	switch (termios_prep(&tty_fildes, &tty_termios,
			&loop_fildes, &loop_termios))
	{
	case 0:
		PATH_TRACE;
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	default:
		xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY/VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}

	SET_TIMEOUT(WAITTIME)

	ret = write(loop_fildes, exp_data, (unsigned) (EXP_SIZE/2));
	err = errno;

	CLEAR_ALARM

	if (ret != (EXP_SIZE/2))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() to VSX_TERMIOS_LOOP failed - errno: %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	alt_errno = EINTR;
	globok = 0;
	(void) tbrd_to(tty_fildes, in_buf, EXP_SIZE, WAITTIME/2, (EXP_SIZE/2), 
		NOERROR, exp_data);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(3);

	(void) close(tty_fildes);
	(void) close(loop_fildes);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

#if TEST_XNFS

private void
test8()
{
	DBUG_ENTER("test8");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test8()
{
	int	err;
	int	pathok = setupok;
	int	retval;

	DBUG_ENTER("test8");

	testfail = 0;

	errno = NOERROR;
	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create pipe, errno: %d (%s)",			 err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	retval = cppair(ch_t8, pa_t8, WAITTIME, NOK | PATH_NOK); 
	if (retval == PATH_NOK)
		retval = OK;
	else
		PATH_TRACE;
	if (retval == NOK)
		testfail++;
	else
		PATH_TRACE;

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);

	if (getfifo(t8_file, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	retval = cppair(ch_t8, pa_t8, WAITTIME, NOK | PATH_NOK); 
	if (retval == PATH_NOK)
		retval = OK;
	else
		PATH_FUNC_TRACE; 
	if (retval == NOK)
		testfail++;
	else
		PATH_TRACE;

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	(void) unlink(t8_file);

	if (!testfail)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	struct	sigaction	sig;
	int	pathok = setupok;

	DBUG_ENTER("ch_t8");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGUSR1 , &sig , NULLSA) == SYSERROR)
		DBUG_EXIT(A_SIGFAIL);
	else
		PATH_TRACE;
	
	(void) sleep(WAITTIME/2);
	globok = 0;
	(void) tbrd_to(fildes[READ], in_buf, EXP_SIZE, (2*WAITTIME), 
		(EXP_SIZE/2), NOERROR, exp_data);
	PATH_FUNC_TRACE;
	
	if (testfail > 0)
	{
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok != 3)
		DBUG_EXIT(PATH_NOK);

	DBUG_VOID_RETURN;
}

private void
pa_t8()
{
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("pa_t8");

	globok = 0;
	errno = NOERROR;
	if (write(fildes[WRITE], exp_data, (unsigned) (EXP_SIZE/2)) != 
		(EXP_SIZE/2))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() failed - errno: %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(WAITTIME);

	(void) kill (tet_child, SIGUSR1);

	PATH_FUNC_RPT(1); 

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test9()
{
	int		fd;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test9");

	errno = NOERROR;
	if ((fd = open(t_file, O_RDONLY, 0)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	errno = NOERROR;
	if ((int) lseek(fd, (off_t) 0, SEEK_END) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek() failed - errno: %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbread(fd, in_buf, (unsigned) EXP_SIZE, 0, NOERROR, (char *) NULL);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((int) lseek(fd, (off_t) EXP_SIZE, SEEK_END) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek() failed - errno: %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbread(fd, in_buf, (unsigned) EXP_SIZE, 0, NOERROR, (char *) NULL);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(5);

	(void) close(fd);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test10()
{
	DBUG_ENTER("test10");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test10()
{
	int flags;
	int	pathok = setupok;

	DBUG_ENTER("test10");

	testfail = 0;

/* for a FIFO */
	if (getfifo(t10_file, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fildes[WRITE]);

	globok = 0;
	(void) tbrd_to(fildes[READ], in_buf, EXP_SIZE, WAITTIME, 0, NOERROR, 
		(char *)NULL);
	PATH_FUNC_TRACE;

	flags = fcntl(fildes[READ] , F_GETFL);
	(void) fcntl(fildes[READ] , F_SETFL , flags & ~O_NONBLOCK);

	globok = 0;
	(void) tbrd_to(fildes[READ], in_buf, EXP_SIZE, WAITTIME, 0, NOERROR, 
		(char *)NULL);
	PATH_FUNC_TRACE;

	(void) close(fildes[READ]);

	(void) unlink (t10_file);

/*for a pipe */
	if (pipe(fildes) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create pipe");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fildes[WRITE]);

	globok = 0;
	(void) tbrd_to(fildes[READ], in_buf, EXP_SIZE, WAITTIME, 0, NOERROR, 
		(char *)NULL);
	PATH_FUNC_TRACE;

	flags = fcntl(fildes[READ] , F_GETFL);
	(void) fcntl(fildes[READ] , F_SETFL , flags | O_NONBLOCK);

	globok = 0;
	(void) tbrd_to(fildes[READ], in_buf, EXP_SIZE, WAITTIME, 0, NOERROR, 
		(char *)NULL);
	PATH_FUNC_TRACE;

	(void) close(fildes[READ]);

	if (!testfail)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

#if TEST_XNFS

private void
test11()
{
	DBUG_ENTER("test11");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test11()
{
	int	flags;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test11");

	testfail = 0;

	errno = NOERROR;
	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create pipe, errno: %d (%s)",			 err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) cppair(ch_t11, pa_t11, WAITTIME, 0);
	PATH_FUNC_TRACE;

	if (getfifo(t11_file, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	flags = fcntl(fildes[READ] , F_GETFL);
	(void) fcntl(fildes[READ] , F_SETFL , flags & ~O_NONBLOCK);

	globok = 0;
	(void) cppair(ch_t11, pa_t11, WAITTIME, 0);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

	(void) unlink(t11_file);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	int	pathok = setupok;
	DBUG_ENTER("ch_t11");

	(void) close(fildes[READ]);

	(void) sleep(WAITTIME/2);

	(void) write(fildes[WRITE] , exp_data , EXP_SIZE);
	PATH_TRACE;

	(void) sleep(WAITTIME/2);

	(void) close(fildes[WRITE]);

	DBUG_VOID_RETURN;
}

private void
pa_t11()
{
	time_t t0, t1;
	int	pathok = setupok;

	DBUG_ENTER("pa_t11");

	(void) close(fildes[WRITE]);

	/* read blocking until data written */
	globok = 0;
	if (tbrd_to(fildes[READ], in_buf, EXP_SIZE, WAITTIME, EXP_SIZE, 
			NOERROR, exp_data) != EXP_SIZE)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* read blocking until writing processes close fds */
	(void) time(&t0);
	globok = 0;
	(void) tbrd_to(fildes[READ], in_buf, EXP_SIZE, WAITTIME, 0, 
			NOERROR, (char *) NULL);
	PATH_FUNC_TRACE;

	(void) time(&t1);
	if (t1 < t0 + WAITTIME/4)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("read() did not block");
	}
	else
		PATH_TRACE;

	(void) close(fildes[READ]);

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test12()
{
	FILE	*fp;
	int	fd;
	int	pathok = setupok;

	DBUG_ENTER("test12");
	
	testfail = 0;

	/* First test using lseek() to set up */

	if ((fd = creat(t12_file , MODEANY)) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d (%s)",
			t12_file, (int)MODEANY, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) write(fd , exp_data , (unsigned) EXP_SIZE);
	(void) lseek(fd, (off_t)EXP_SIZE, SEEK_CUR);
	(void) write(fd , exp_data , (unsigned) EXP_SIZE);
	(void) close(fd);

	globok = 0;
	t12_read();
	PATH_FUNC_TRACE;

	(void) unlink(t12_file);

	/* Repeat using fseek() */

	if ((fp = fopen(t12_file, "w")) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("fopen(%s, \"w\") failed - errno %d (%s)",
			t12_file, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fwrite((void *) exp_data, (size_t) 1, (size_t) EXP_SIZE, fp);
	(void) fseek(fp, (long)EXP_SIZE, SEEK_CUR);
	(void) fwrite((void *) exp_data, (size_t) 1, (size_t) EXP_SIZE, fp);
	(void) fclose(fp);

	globok = 0;
	t12_read();
	PATH_FUNC_TRACE;

	(void) unlink(t12_file);

	if (!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
t12_read()
{
	int fd;
	int pathok = 0;

	DBUG_ENTER("t12_read");

	errno = NOERROR;
	if ((fd = open(t12_file, O_RDONLY)) == SYSERROR)
	{
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t12_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbread(fd, in_buf, (unsigned) (EXP_SIZE), (EXP_SIZE), NOERROR, 
		exp_data);
	PATH_FUNC_TRACE;

	(void) memset((void *)out_buf, '\0', (size_t)EXP_SIZE);
	globok = 0;
	(void) tbread(fd, in_buf, (unsigned) (EXP_SIZE), (EXP_SIZE), NOERROR, 
		out_buf);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbread(fd, in_buf, (unsigned) (EXP_SIZE), (EXP_SIZE), NOERROR, 
		exp_data);
	PATH_FUNC_TRACE;

	(void) close(fd);

	PATH_FUNC_RPT(4);
	DBUG_VOID_RETURN;
}


private void
test13()
{
	int		fd;
	char		*mess = (char *)NULL;
#if !TEST_XNFS
	int		err;
#endif /* TEST_XNFS */
	int	pathok = setupok;

	DBUG_ENTER("test13");

/* for a regular file */
	testfail = 0;

#if TEST_XNFS
	
	if (tet_sync(SP_T13_CREAT_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREAT_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */
		PATH_TRACE;
#endif /* TEST_XNFS */

	errno = NOERROR;
	if ((fd = open(t13_file, O_RDONLY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t13_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(t13_file);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbread(fd, in_buf, (unsigned) (EXP_SIZE), (EXP_SIZE), NOERROR, 
		exp_data);
	PATH_FUNC_TRACE;

	globok = 0;
	if ((mess = chktimes(t13_file , NEWATIME)) != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt(mess);
	}
	else
		PATH_FUNC_TRACE;

	(void) close(fd);

#if TEST_XNFS
	
	(void) tet_sync(SP_T13_UNLINK_START, snames, SYNC_TIMEOUT);

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#else

/* for a FIFO */
	if (getfifo(t13_fifo, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) write(fildes[WRITE], exp_data, (unsigned) EXP_SIZE);

	globok = 0;
	savtimes(t13_fifo);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbread(fildes[READ], in_buf, (unsigned) (EXP_SIZE), (EXP_SIZE), 
		NOERROR, exp_data);
	PATH_FUNC_TRACE;

	globok = 0;
	if ((mess = chktimes(t13_fifo , NEWATIME)) != NULL)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt(mess);
	}
	else
		PATH_FUNC_TRACE;

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);
	(void) unlink(t13_fifo);

/* for a pipe */

	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: pipe() failed - errno: %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) write(fildes[WRITE], exp_data, (unsigned) EXP_SIZE);

	globok = 0;
	fsavtimes(fildes[READ]);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbread(fildes[READ], in_buf, (unsigned) (EXP_SIZE), (EXP_SIZE), 
		NOERROR, exp_data);
	PATH_FUNC_TRACE;

	globok = 0;
	if ((mess = fchktimes(fildes[READ] , NEWATIME)) != NULL)
	{
		if(!testfail++)
			xx_rpt(FAILURE);
		in_rpt(mess);
	}
	else
		PATH_FUNC_TRACE;

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);


#endif /* TEST_XNFS */

	if (!testfail)
		PATH_XS_RPT(13);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test14()
{
	DBUG_ENTER("test14");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test14()
{
	int	flags, err;
	int	pathok = setupok;

	DBUG_ENTER("test14");

	testfail = 0;

	errno = NOERROR;
	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create pipe, errno: %d (%s)",			 err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (write(fildes[WRITE], exp_data, 1) != 1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() failed - errno: %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrd_to(fildes[READ], in_buf, EXP_SIZE, WAITTIME, 
		1, NOERROR, exp_data);
	PATH_FUNC_TRACE;

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);


	if (getfifo(t14_file, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	flags = fcntl(fildes[READ], F_GETFL);
	(void) fcntl(fildes[READ], F_SETFL, flags & ~O_NONBLOCK);

	errno = NOERROR;
	if (write(fildes[WRITE], exp_data, 1) != 1)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() failed - errno: %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrd_to(fildes[READ], in_buf, EXP_SIZE, WAITTIME, 
		1, NOERROR, exp_data);
	PATH_FUNC_TRACE;

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);
	(void) unlink(t14_file);

	if (testfail == 0)
		PATH_XS_RPT(6);
	
	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test15()
{
	int		fd;
	int		rval;
	int		err;
	char		*mess;
	int	pathok = 0;

	DBUG_ENTER("test15");

	testfail = 0;

#if TEST_XNFS
	
	if (tet_sync(SP_T15_CREAT_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CREAT_END failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else /* TEST_XNFS */
		PATH_TRACE;
#endif /* TEST_XNFS */

	if ((fd = open(t15_file, O_RDONLY, 0)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not open file %s", t15_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((rval = lseek(fd, (off_t)0, SEEK_END)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek() failed");
		valrpt(0,rval,NOERROR,err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	globok = 0;
	fsavtimes(fd);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbread(fd, in_buf, (unsigned) 1, 0, NOERROR, (char *)NULL);
	PATH_FUNC_TRACE;

	globok = 0;
	if ((mess = fchktimes(fd,NEWATIME)) != (char *)NULL)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("%s", mess);
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(6);
	}

	(void) close(fd);

#if TEST_XNFS
	(void) tet_sync(SP_T15_UNLINK_START, snames, SYNC_TIMEOUT);
#endif

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test16()
{
	DBUG_ENTER("test16");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test16()
{
	int		flags;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test16");

	testfail = 0;

/* for a FIFO */
	if (getfifo(t16_file, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbrd_to(fildes[READ], in_buf, EXP_SIZE, WAITTIME , SYSERROR, 
		EAGAIN, (char *)NULL);
	PATH_FUNC_TRACE;

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);
	(void) unlink(t16_file);

/* for a pipe */

	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: pipe() failed - errno: %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	flags = fcntl(fildes[READ], F_GETFL);
	(void) fcntl(fildes[READ], F_SETFL, flags | O_NONBLOCK);

	globok = 0;
	(void) tbrd_to(fildes[READ], in_buf, EXP_SIZE, WAITTIME , SYSERROR, 
		EAGAIN, (char *)NULL);
	PATH_FUNC_TRACE;

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);

	if (!testfail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

private void
test17()
{
	int		fd;
	char		car;
	int	pathok = setupok;

	DBUG_ENTER("test17");

	testfail = 0;

	globok = 0;
	(void) tbread(-1, &car, (unsigned) 1, SYSERROR, EBADF, (char *) NULL);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	if ((fd = creat(t17_file, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create file %s", t17_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	globok = 0;
	(void) tbread(fd, &car, (unsigned) 1, SYSERROR, EBADF, (char *) NULL);
	PATH_FUNC_TRACE;

	(void) close(fd);


	globok = 0;
	(void) tbread(fd, &car, (unsigned) 1, SYSERROR, EBADF, (char *) NULL);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(4);

	(void) unlink(t17_file);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test18()
{
	DBUG_ENTER("test18");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test18()
{
	int	err;

	DBUG_ENTER("test18");

	errno = NOERROR;
	if (mkfifo(t18_file, S_IRWXU) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create FIFO %s - errno: %d (%s)", t18_file, err, errname(err));
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t18, pa_t18, 2*WAITTIME, 0);

	(void) unlink(t18_file);
	
	DBUG_VOID_RETURN;
}

private void
ch_t18()
{
	int		fd;
	struct	sigaction	sig;
	int	pathok = setupok;

	DBUG_ENTER("ch_t18");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGUSR1 , &sig , NULLSA) == SYSERROR)
		DBUG_EXIT(A_SIGFAIL);

	SET_TIMEOUT(WAITTIME)
	fd = open(t18_file, O_RDONLY);
	CLEAR_ALARM
	if ((fd == SYSERROR) || (alrm_flag))
		DBUG_EXIT(A_OPENFAIL);
	else
		PATH_TRACE;

	testfail = 0;

	globok = 0;
	(void) tbrd_to(fd, in_buf, EXP_SIZE, WAITTIME, SYSERROR, EINTR, 
						(char *) NULL);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
pa_t18()
{
	int	fd;

	DBUG_ENTER("pa_t18");

	SET_TIMEOUT(WAITTIME)
	fd = open(t18_file, O_WRONLY);
	CLEAR_ALARM
	if ((fd == SYSERROR) || (alrm_flag))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason:  open(FIFO, O_RDONLY) failed");
		(void) kill(tet_child, SIGKILL);
		(void) close (fd);
		(void) unlink (t18_file);
		DBUG_VOID_RETURN;
	}

	(void) sleep(WAITTIME/2);

	(void) kill (tet_child, SIGUSR1);

	(void) sleep(WAITTIME/2);

	(void) close (fd);
	globok = 1;

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test19()
{
	DBUG_ENTER("test19");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */


private	void
test19()
{
	DBUG_ENTER("test19");

	sigstat = "ignored";
	do_feio("ri", -1, t19_20io, t19rpt);

	sigstat = "blocked";
	do_feio("rb", -1, t19_20io, t19rpt);

	DBUG_VOID_RETURN;
}

private void
t19rpt(ret, err)
int ret, err;
{
	int	pathok = setupok;

	DBUG_ENTER("t19rpt");

	PATH_FUNC_TRACE;

	if (ret != SYSERROR || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("read() from background with SIGTTIN %s did not give correct results",
				sigstat);
		valrpt(SYSERROR, ret, EIO, err);
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

#endif	/* TEST_XNFS */

#if TEST_XNFS

private void
test20()
{
	DBUG_ENTER("test20");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test20()
{
	DBUG_ENTER("test20");

	do_feio("ro", -1, t19_20io, t20rpt);

	DBUG_VOID_RETURN;
}

private void
t20rpt(ret, err)
int ret, err;
{
	int	pathok = setupok;

	DBUG_ENTER("t20rpt");

	PATH_FUNC_TRACE;

	if (ret != SYSERROR || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("read() from orphaned background process did not give correct results");
		valrpt(SYSERROR, ret, EIO, err);
	}
	else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private int
t19_20io(fp)
FILE *fp;
{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	globok = 1;
	return read(fileno(fp), (void *)in_buf, (size_t)1);
#else
	globok = 1;
	return read(fileno(fp), in_buf, 1);
#endif
}

#endif	/* TEST_XNFS */

private int
tbrd_to(fd, buf, nbyte, timeout, expret, experrno, expdata)

	int		fd;		/* file descriptor		*/
	char		*buf;		/* buffer for data to be read	*/
	int		nbyte;		/* number of bytes to be read	*/
	unsigned	timeout;	/* number of seconds to timeout */
	int		expret, experrno;
	char		*expdata;
{
	int num;		/* number of bytes read		*/
	int	pathok = setupok;

	DBUG_LENTER("tbrd_to", 2);

	SET_TIMEOUT(timeout)

	globok = 0;
	num = tbread(fd, buf, (unsigned) nbyte, expret, experrno, expdata);
	PATH_FUNC_TRACE;

	CLEAR_ALARM

	if (experrno != EINTR && alt_errno != EINTR && alrm_flag > 0)
	{
		if (!testfail)
			xx_rpt(FAILURE);
	 	in_rpt("read(%d, buf, %d) timed out", fd, nbyte);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_RETURN(num);
}

private int
tbread(tbfildes, tbbuf, tbnbyte, expret, experrno, expdata)
int tbfildes;
char *tbbuf;
unsigned tbnbyte;
int expret, experrno;
char *expdata;
{
	int	rval, err;
	int	failed = 0;
	char	savchar;
	int	pathok = setupok;

	DBUG_LENTER("tbread", 2);

	globok = 0;
	errno = 0;
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	rval = read(tbfildes, (void *)tbbuf, (size_t)tbnbyte);
#else
	rval = read(tbfildes, tbbuf, tbnbyte);
#endif

#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;

	if (expret == SYSERROR)
	{
		if (rval == expret && err == experrno)
		{
			PATH_FUNC_RPT(0);
			DBUG_RETURN(rval);
		}
		else
			failed++;
	}
	else
	{
		/* expected failures are permitted */
		if (rval == SYSERROR && err == alt_errno)
		{
			PATH_FUNC_RPT(0);
			DBUG_RETURN(rval);
		}
		if (expret != rval)
			failed++;
		else
			PATH_TRACE;
	}


	if (failed)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
	 	in_rpt("read(%d, buf, %d) failed", tbfildes, tbnbyte);
		valrpt(expret, rval, experrno, err);
		DBUG_RETURN(rval);
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	tbbuf[0]++;
#endif

	if (expdata != NULL && memcmp((void *)expdata, (void *)tbbuf, (size_t)rval))
	{
		if (!testfail++)
			xx_rpt(FAILURE);
	 	in_rpt("read(%d, buf, %d) returned incorrect data in buf",
			tbfildes, tbnbyte);
		/* for buffers containing nulls this will be incomplete */
		savchar = expdata[rval];
		expdata[rval] = '\0';
		in_rpt("expected: \"%s\"", psl_to_asc(expdata, 0));
		expdata[rval] = savchar;
		tbbuf[rval] = '\0';
		in_rpt("observed: \"%s\"", psl_to_asc(tbbuf, 0));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);
	
	DBUG_RETURN(rval);
}
private void
test21()
{
	DBUG_ENTER("test21");

	(void) cppair(ch_t21, NULLFN, 4 * WAITTIME, 0);

	(void) rmdir(t21_dir);

	DBUG_VOID_RETURN;
}

private void
ch_t21()
{
	char	*rofs;
	int	err;
	char	*args[7];

	DBUG_ENTER("ch_t21");

	if ((rofs = tet_getvar("VSX_ROFS")) == NULL || *rofs == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: parameter VSX_ROFS is not defined");
		DBUG_VOID_RETURN;
	}

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = t21_dir;
	args[3] = t21_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;
	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason: tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

#if ! TEST_XNFS

private void
test22()
{
	DBUG_ENTER("test22");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test22()
{
	DBUG_ENTER("test22");

	t_estale_rd_c(t22_file, 0, t22_func, t22_chk_func, SP_T22_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t22_func(fp)
FILE	*fp;
{
	DBUG_ENTER("t22_func");

	DBUG_RETURN(vread(fileno(fp), in_buf, 1) == -1 ? -1 : 0);
}

private int
t22_chk_func(fp)
FILE	*fp;
{
	int	rval, err;
	int	pathok = 0;

	DBUG_ENTER("t22_chk_func");

	errno = NOERROR;

	while((rval = vread(fileno(fp), in_buf, 1)) != -1)
		;

	err = errno;

	if(rval != -1 || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(rval != -1)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value -1");
			in_rpt("Actual   return value %d", rval);
		}
		else
			PATH_TRACE;
		
		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(2);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test23()
{
	DBUG_ENTER("test23");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test23()
{
	DBUG_ENTER("test23");

	(void) cppair(ch_t23, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t23()
{
	int err;
	char *args[6];

	DBUG_ENTER("ch_t23");

	args[0] = prog_su;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t23_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog_su), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog_su), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */
