/*
 *      SCCS:  @(#)  POSIX.os/ioprim/write/enospc.c	Rel 4.4.1     (06/09/97) 
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1991 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
#ifndef NO_MAIN
static	char sccsid[] = "@(#)POSIX.os/ioprim/write - enospc Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif
#endif  /* lint	*/

/***********************************************************************

NAME:	enospc.c

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()
		enospc()
		shortwrite()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char *argv[];

DESCRIPTION:
		Program to cause out of space errors.
		Also check for write returning a short count on file system
		full errors.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:
AUTHOR:		Steve Ratcliffe, UniSoft Ltd.
DATE CREATED:	28 Feb 1990
MODIFICATIONS:
		Peter Lee, Unisoft Ltd.,
		Thu Aug 09 09:37:39 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file
		
		Geoff Clare, 10 Jan 1992
			Relax shortwrite() test.
		
		Geoff Clare, 16 Dec 1992
			Fix path tracing problem in shortwrite().

		Geoff Clare, 29 July 1993
			Retry the space-reusing call if it gives ENOSPC

		Geoff Clare, 20 June 1994
			Place #include <ulimit.h> within #ifndef MAKE_DEPEND

		Geoff Clare, 29 May 1997
			Don't bother trying to get an ENOSPC first in
			shortwrite(); just do a (much) larger write.

************************************************************************/

#include <dbug.h>
#include <std.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <report.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <tet_api.h>
#include <sysdep.h>

#ifdef UNDEF_MACROS
#undef write
#endif


#define	WAITTIME	(3 * SPEEDFACTOR)

#ifndef NO_MAIN
public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

#endif

extern	char	**environ;

extern	char	*itoa();
extern	char	*errname();
extern	void	set_pname();
extern	void	valrpt();

private int 	testfail;

private void 	enospc();
private void 	shortwrite();

extern int 	setupnospc();
extern void	tidynospc();

#define MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define MAXTRYS 	2000	/* Number of times we try to fill up system */
#define MAXMULT 	16	/* Max multiple of file size to use */
#define NAP		2
#define MAXTRYS2	10	/* Number of retries for partial write */

#define BUFLEN	512

/*
 * A string used as data to write out
 */
#define STRPAT	"abcdefghijklmnopqrstuvwxyz"
#define PATLEN (sizeof(STRPAT)-1)

#ifndef NO_MAIN

public	int
tet_main(argc, argv)
int	argc;
char	*argv[];
{
char	*mountdir;

	set_pname(argv[0]);

	if (argc < 3 || argc > 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		return(OK);
	}

	if (argc >= 5)
		vg_dbug = argv[4];
	if (argc >= 4)
		vg_ident = argv[3];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	mountdir = argv[1];

	if (setupnospc(mountdir)) {
		testfail++;
		return(OK);
	}

	if (strcmp(argv[2], "shortwrite") == 0)
		shortwrite(mountdir);
	else
		enospc(mountdir);

	/* Tidy up and umount file system */
	tidynospc(mountdir, 3);

	return(OK);
	/*NOTREACHED*/
}
#endif /* !NO_MAIN */

/*
 * enospc() tries to run out of space.
 */
private void
enospc(mountdir)
char	*mountdir;
{
	char	*name;
	int 	err;
	int 	i;
	char	*outstr;
	int 	fd;
	int	pathok = 0;
	int	loop1;

	DBUG_ENTER("enospc");

	name = (char*)malloc(strlen(mountdir)+sizeof("/fill.tmp0000"));
	if (name == NULL) 
	{
		testfail++;
		xx_rpt(DELETION);
		in_rpt("Out of memory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Build up  temp file name and open it
	 */
	(void) strcat(strcpy(name, mountdir), "/fill.tmp1");

	fd = open(name, O_RDWR);

	if (fd == SYSERROR) 
	{
		err = errno;
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: Open of %s failed - errno %s", name, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (lseek(fd, (off_t)0, SEEK_END) == SYSERROR) 
	{
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: Seek to end of file failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	outstr = (char*)malloc(BUFLEN);
	if (outstr == NULL) 
	{
		testfail++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: Out of memory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < BUFLEN; i++)
		outstr[i] = STRPAT[i%PATLEN];

	err = 0;

	for (i = 0, loop1 = 0; i < MAXTRYS && !err && !testfail; i++) {

		DBUG_PRINTF("enospc", "write block on %s", name);

		if (write(fd, outstr, BUFLEN) == SYSERROR)
			err = errno;

		if (err)
		{
			switch (err) {
			case ENOSPC:
				PATH_XS_RPT(4 + loop1);
				break;
			default:
				testfail++;
				xx_rpt(FAILURE);
				in_rpt("write failed errno %s (%d)", errname(err), err);
				break;
			}
		}
		else
		{
			loop1++;
			PATH_TRACE;
		}

	}

	(void) close(fd);

	if (i == MAXTRYS && !testfail) {
		testfail++;
		xx_rpt(UNTESTED);
		in_rpt("ENOSPC error didn't occur in %d trys", MAXTRYS);
	}

	free((void*)name);

	DBUG_VOID_RETURN;
}

#ifdef TEST_BED
#ifndef MAKE_DEPEND	/* don't want dependency in Makefile */
#include <ulimit.h>
#endif /* MAKE_DEPEND */
#endif

private void
shortwrite(mountdir)
char	*mountdir;
{
	char	*name;
	struct	stat	sbuf;
	char	*buf;
	off_t	size;
	int 	nw;
	int 	fd;
	int 	err;
	unsigned	wsize;
	int	pathok = 0;
		
	DBUG_ENTER("shortwrite");

	name = (char*)malloc(strlen(mountdir)+sizeof("/fill.tmp0000"));
	(void) strcat(strcpy(name, mountdir), "/fill.tmp1");

	if (stat(name, &sbuf) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("stat() failed on file \"%s\", errno %s (%d)", name,
			errname(err), err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	size = sbuf.st_size;

	wsize = 20*size;

	buf = (char*)malloc((size_t)wsize);
	if (buf == NULL)
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("out of memory");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	fd = open(name, O_RDWR);
	if (fd == SYSERROR) 
	{
		err = errno;
		if (!testfail++)
			xx_rpt(DELETION);
		in_rpt("open(\"%s\", O_RDWR) failed - errno %s (%d)",
			name, errname(err), err);
		free((void*)buf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TEST_BED
	ulimit(UL_SETFSIZE, 5*size/512);
#endif

	errno = 0;
	nw = write(fd, buf, wsize);
	err = errno;

	(void) close(fd);
	free((void*)buf);

	/* It is not clear from POSIX.1 that we can insist on
	   nw >= size here, although that's what might be
	   expected at first sight.
	   Instead allow any value 0 < nw < wsize. */

	if (nw <= 0) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("write() returned %d, expected > 0", nw);
		if (nw < 0 && err != 0)
			in_rpt("errno was set to %s (%d)",
				errname(err), err);
	} 
	else if (nw >= wsize) 
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("write() returned %d, expected < %u", nw, wsize);
	}
	else
	{
		if (!testfail)
			PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

