/*
*      SCCS:  @(#)  POSIX.os/ioprim/write/write.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/ioprim/write - T.write Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		POSIX.os/ioprim/write/write.c
PRODUCT:	T.write
PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Implements the write test set as defined in the
	write functional and design specification documents.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.;
DATE CREATED:	Thu Mar  9 17:23:41 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 4 July 1989
			Extensive tidy-up, pipe/FIFO tests rewritten
		
		Geoff Clare, 13 Oct 1989
			Add EFBIG test.
		
		Geoff Clare, 2 Jan 1990
			Change timeouts on fill_pipe() calls to allow
			for slow machines with large pipe capacity.
		
		Geoff Clare, 27 Feb 1990
			Use termios_prep() instead of tty_prep() in test8.

		Steve Ratcliffe, 28 Mar 1990
			Add ENOSPC test.
		
		Steve Ratcliffe, 11 Apr 1990
			Add test to check that write returns number of bytes
			actually writen when out of space.
		
		Geoff Clare, 25 Apr 1990
			Varying argument types.
		
		Geoff Clare, 6 June 1990
			VSX_ULIMIT_BLKS == -1 means unsupported

		J.A.Nave	07/06/90
			Extended test2 to cover O_APPEND being set in the
			open(), that the timestamps are not updated and
			added resilience for that test.
		
		Geoff Clare, 7 June 1990
			UNSUPPORTED result if termios_prep() returns 1
		
		Geoff Clare, 21 June 1990
			Add "overwrites existing data" test

		Peter Lee, Unisoft Ltd.,
		Wed Aug 08 13:17:44 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Geoff Clare, 21 Nov 1990
			Change some timeouts to depend on size of PIPE_BUF.
			Close pipe in ch_t13() and ch_t14()

		Geoff Clare, 6 Mar 1992
			Allow for systems with no output queue or huge
			output queue in test 8.

		Geoff Clare, 18 May 1992
			Test O_APPEND with O_WRONLY and O_RDWR.
			EAGAIN not allowed in test13 (except XPG3 mode).
			New test9: pipe/FIFO nbyte>PIPE_BUF EAGAIN test.

		Neil Moses, 12 October 1992
			XNFS enhancements

		Geoff Clare, 16 Dec 1992
			Change t7sub() argument from mode_t to int.

		Geoff Clare, 24 Aug 1993
			Add test for partial write at file size limit.

		Geoff Clare, 14 July 1994
			Modify test 12 strategy to ensure the "low water
			mark" is reached on systems that have one.
			Initialize testfail variable in some tests.

		Geoff Clare, 28 Sept 1994
			Changed test 18 to remove assumptions about how
			pipes are implemented.

************************************************************************/
#define NO_VWRITE

#include	<std.h>
#include	<stdio.h>
#include	<errno.h>
#include	<sys/types.h>
#include	<fcntl.h>
#include	<dbug.h>
#include	<sys/stat.h>
#include	<signal.h>
#include	<report.h>
#include	<limits.h>
#include	<exitcodes.h>
#include	<unistd.h>
#include	<synch.h>
#include	<vsx_signal.h>
#include	<stdlib.h>
#include	<timeout.h>
#include	<ctype.h>
#include	<chktimes.h>
#include 	<sys/wait.h>
#include 	<termios.h>
#include <tet_api.h>
#include	<xnfs.h>
#include	"write_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef write
#endif


#define NO_TESTS	28		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)

#define NBYTES		12
#define MBYTES		18
#define LBYTES		6
#define	ULMT_NBLK	16L		/* allow for large tet_xres file */
#define	BLK_SIZE	512
#define	MODEANY		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	MODE3		(mode_t)(S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH)

#define EXIT_ERR	2
#define READ		0
#define WRITE		1
#define	IGNERROR	-2		/* ignore errors from tbwrite() flag */

#define NAP		3
#define MSG_LEN (sizeof(alphabet)-1)
#define HUGE_QUEUE	100000

#undef memcpy
extern void	*memcpy();
extern char	*itoa();

extern	void	setsigs();
extern	void	do_feio();
extern	char	*psl_to_asc();
extern	long	setlimit();
extern	char	*errname();
extern	char	*execpath();
extern	void	valrpt();
extern	void	vsx_sigs();

extern	char	*vg_ident;
extern	char	*vg_dbug;
extern	char	**environ;

private int	fildes[2];
private int	bytes_in_pipe;
private	int	caught_sig = 0;
private int	setupok;

public	int	no_tests = NO_TESTS;

private void	prepare_tests();
private int	tbread();
private int	tbwrite();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	t7sub();
private void	test8();
private void	ch_t8();
private void	test9();
#if !TEST_XNFS
private void	ch_t9();
#endif
private void	test10();
private void	test11();
#if !TEST_XNFS
private void	ch_t11();
#endif
private void	test12();
#if !TEST_XNFS
private void	pa_t12();
private void	ch_t12();
#endif
private void	test13();
#if !TEST_XNFS
private void	ch_t13();
#endif
private void	test14();
#if !TEST_XNFS
private void	ch_t14();
#endif
private void	test15();
private void	test16();
private void	ch_t16();
private void	test17();
private void	test18();
#if !TEST_XNFS
private void	ch_t18();
#endif
private void	test19();
private void	test20();
private void	test21();
private void	pa_t21();
private void	ch_t21();
private void	test22();
#if !TEST_XNFS
private int	t22io();
private void	t22rpt();
#endif
private void	test23();
private void	test24();
private void	ch_t24();
private void	test25();
private void	ch_t25();
private void	test26();
#if TEST_XNFS
private int	t26_func();
private int	t26_chk_func();
#endif
private void	test27();
#if TEST_XNFS
private void	ch_t27();
#endif
private void	test28();
private void	ch_t28();

private char	*t1_path =	"./write-t.1";
private char	*t2_path =	"./write-t.2";
private char	*t3_path =	"./write-t.3";
private char	*t4_path =	"./write-t.4";
private char	*t5_path =	"./write-t.5";
#if !TEST_XNFS
private char	*t6_path =	"./write-t.6";
#endif
private char	*t7_path =	"./write-t.7";
#if !TEST_XNFS
private char	*t9_path =	"./write-t.9";
private char	*t10_path =	"./write-t.10";
private char	*t11_path =	"./write-t.11";
private char	*t12_path =	"./write-t.12";
private char	*t13_path =	"./write-t.13";
#endif
#if !TEST_XNFS
private char	*t14_path =	"./write-t.14";
#endif
private char	*t15_path =	"./write-t.15";
private char	*t16_dir =	"./write-d.16";
private char	*t17_path =	"./write-t.17";
#if !TEST_XNFS
private char	*t18_path =	"./write-t.18";
#endif
private char	*t20_path =	"./write-t.20";
private char	*t24_path =	"./write-t.24";
private char	*t25_dir =	"./write-d.25";
#if TEST_XNFS
private char	*t26_file =	T26_FILE;
private char	*t27_file =	T27_FILE;
#endif
private char	*t28_path =	"./write-t.28";

private char	*progsu = "write_su";
private	char	*prog27 = "write_t27";

#if !TEST_XNFS
private synch	*t21_syn;
#endif
private	int	pipe_buf;

private char	alphabet[] = "abcdefghijklmnopqrstuvwxyz";
private char	*out_buf, *in_buf;

private int	testfail;
public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	NULL, 0
};

private int
vwrite(fd, buf, nbyte)
int fd;
char *buf;
unsigned int nbyte;
{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	return write(fd, (void *)buf, (size_t)nbyte);
#else
	return write(fd, buf, nbyte);
#endif
}

private	void
prepare_tests()
{
	int i;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

#if !TEST_XNFS

	/* obtain the value of PIPE_BUF from pathconf() */
	if((pipe_buf = pathconf(".", _PC_PIPE_BUF)) == SYSERROR)
	{
		for (i = 1; i <= no_tests; i++)
			tet_delete(i,
				"pathconf(\".\", _PC_PIPE_BUF) failed in prep");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#else
	PATH_TRACE;

	pipe_buf = 1024;

#endif /* TEST_XNFS */

	out_buf = (char *) malloc((size_t) (2*pipe_buf + 1));
	in_buf = (char *) malloc((size_t) (2*pipe_buf + 1));

	if (out_buf == NULL || in_buf == NULL)
	{
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, "malloc() failed in prep");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* NB this will contain nulls, so don't use strcmp() etc. on it */
	/* Start at 1 so as much as possible can be printed with psl_to_asc() */
	for (i = 0; i < 2 * pipe_buf; i++)
		out_buf[i] = 1 + i % (CHAR_MAX - CHAR_MIN + 1);

	/* test for SIGKILL and SIGTERM */
	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	PATH_SETUP_RPT(3);

	DBUG_VOID_RETURN;
}


private int
getfifo(path, fds, waittime)
char *path;
int fds[2];
unsigned waittime;
{
	/* create and open a FIFO.  N.B. it is left in NONBLOCK mode */

	int err;
	int	pathok = setupok;

	DBUG_ENTER("getfifo");

	globok = 0;

	(void) remove(path);
	if (mkfifo(path, MODEANY) == SYSERROR)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: mkfifo(%s, %#o) failed - errno %d (%s)",
			path, (int)MODEANY, err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(waittime)
	fds[READ] = open(path, O_RDONLY|O_NONBLOCK);
	CLEAR_ALARM

	if (fds[READ] < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: open(O_RDONLY|O_NONBLOCK) failed on FIFO %s - errno %d (%s)",
			path, err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(waittime)
	fds[WRITE] = open(path, O_WRONLY|O_NONBLOCK);
	CLEAR_ALARM

	if (fds[WRITE] < 0)
	{
		err = errno;
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: open(O_WRONLY|O_NONBLOCK) failed on FIFO %s - errno %d (%s)",
			path, err, errname(err));
		(void) close(fds[READ]);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_RETURN(0);
}

private int
fill_pipe(fds, waittime, nonblock)
int fds[2];
int waittime;
int nonblock;
{
	/* Fill up a pipe/FIFO with data.  If (nonblock & O_NONBLOCK) is
	   TRUE, write fd is left in O_NONBLOCK mode, else BLOCKING mode */

	int ret, statw;

	/* save current file descriptor state, set O_NONBLOCK flag */
	statw = fcntl(fds[WRITE], F_GETFL, 0);
	if ((statw & O_NONBLOCK) == 0)
		(void) fcntl(fds[WRITE], F_SETFL, statw | O_NONBLOCK);
	
	/* keep count of the number of bytes in the pipe */
	bytes_in_pipe = 0;

	SET_TIMEOUT(waittime)

	while (vwrite(fds[WRITE], " ", 1) > 0)
	{
		bytes_in_pipe++;
		if (alrm_flag > 0)
			break;
	}

	/* STREAMS pipes do not have fixed capacity: fill high
	   enough so that reading (PIPE_BUF-1) bytes off will
	   leave the pipe above the "high water" mark */
	
	if (alrm_flag == 0)
	{
		(void) read(fds[READ], in_buf, 1);
		bytes_in_pipe--;
		ret = vwrite(fds[WRITE], out_buf, (unsigned)pipe_buf);
	}

	/* Normal pipes won't have written anything on the last write,
	   and so just require the 1 byte to fill them up again */

	if (alrm_flag == 0 && ret <= 0)
		ret = vwrite(fds[WRITE], " ", 1);

	CLEAR_ALARM

	if (ret > 0)
		bytes_in_pipe += ret;

	if (alrm_flag > 0)
		return -1;

	DBUG_PRINTF("fill_pipe", "last write returned %d", ret);

	/* clear O_NONBLOCK flag if required */
	if ((nonblock & O_NONBLOCK) == 0)
		(void) fcntl(fds[WRITE], F_SETFL, statw & ~O_NONBLOCK);

	return 0;
}

private void
sig_catch(sig)
int sig;
{
	caught_sig = sig;
}

private void
test1()
{
	int		fd;
	int		i;
	int		err;
	int		pathok = setupok;
	int		loop1;
#if !TEST_XNFS
	int		loop2, loop3;
#endif

	DBUG_ENTER("test1");
/* for a regular file */

	fd = creat(t1_path , MODEANY);
	(void) close(fd);
	testfail = 0;

	for (i=1, loop1 = 0 ; i<_POSIX_PIPE_BUF ; i*=2, loop1++)
	{

		errno = NOERROR;
		if ((fildes[WRITE] = open(t1_path, O_WRONLY|O_TRUNC)) 
			== SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: could not open file %s", 
				t1_path);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		errno = NOERROR;
		if ((fildes[READ] = open(t1_path, O_RDONLY)) == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: could not open file %s", 
				t1_path);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		(void) tbwrite(fildes[WRITE], out_buf, (unsigned) i, i, 
			NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) tbread(fildes[READ], in_buf, (unsigned) i, i, 
			NOERROR, out_buf);
		PATH_FUNC_TRACE;

		errno = NOERROR;
		if (close(fildes[WRITE]) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: could not close file %s", 
				t1_path);
			valrpt(NOERROR,SYSERROR,NOERROR,err);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		errno = NOERROR;
		if (close(fildes[READ]) == SYSERROR)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: could not close file %s", 
				t1_path);
			valrpt(NOERROR,SYSERROR,NOERROR,err);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	(void) unlink(t1_path);

#if !TEST_XNFS

/* for a FIFO */
	globok = 0;
	if (getfifo(t1_path, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	for (i=1, loop2 = 0; i<_POSIX_PIPE_BUF ; i*=2, loop2++)
	{
		globok = 0;
		(void) tbwrite(fildes[WRITE], out_buf, (unsigned) i, i, 
			NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) tbread(fildes[READ], in_buf, (unsigned) i, i, 
			NOERROR,out_buf);
		PATH_FUNC_TRACE;
	}

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);
	(void) unlink(t1_path);

/* for a pipe */
	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: pipe() failed - errno: %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i=1, loop3 = 0 ; i<_POSIX_PIPE_BUF ; i*=2, loop3++)
	{
		globok = 0;
		(void) tbwrite(fildes[WRITE], out_buf, (unsigned) i, i, 
			NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) tbread(fildes[READ], in_buf, (unsigned) i, i, 
			NOERROR,out_buf);
		PATH_FUNC_TRACE;
	}
	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);

	if (testfail == 0)
		PATH_XS_RPT(2 + 6 * loop1 + 2 * loop2 + 2 * loop3);
#else

	if (!testfail)
		PATH_XS_RPT(6 * loop1);

#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	fd;
	long	off_set;
	char	*mess;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	testfail = 0;

	if ((fd = creat(t2_path, MODEANY)) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d (%s)",
			t2_path, (int)MODEANY, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	fsavtimes(fd);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbwrite(fd , "A" , 0 , 0 , NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	if ((mess = fchktimes(fd,0)) != (char *)NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt(mess);
	}
	else
		PATH_FUNC_TRACE;

	off_set = lseek(fd , (off_t)0 , SEEK_END);

	if (off_set != 0)
	{
		if(testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("write(fd, buf, 0) wrote %ld bytes to the file", off_set);
	}
	else
		PATH_TRACE;

	(void) close(fd);

	if ((fd = open(t2_path, O_WRONLY|O_APPEND)) < 0)
	{
		if(testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY|O_APPEND) failed - errno %d (%s)",
			t2_path, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbwrite(fd, alphabet, MSG_LEN, MSG_LEN, NOERROR);
	PATH_FUNC_TRACE;

	off_set = lseek(fd , (off_t)0 , SEEK_SET);

	if (off_set != 0)
	{
		if(testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("lseek() returned %ld, expected 0", (long)off_set);
	}
	else
		PATH_TRACE;

	globok = 0;
	fsavtimes(fd);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbwrite(fd , "A" , 0 , 0 , NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	if ((mess = fchktimes(fd,0)) != (char *)NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt(mess);
	}
	else 
		PATH_FUNC_TRACE;

	off_set = lseek(fd , (off_t)0 , SEEK_CUR);

	if (off_set != 0)
	{
		if(testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("write(fd, buf, 0) with O_APPEND set moved the file pointer from 0 to %ld",
			(long)off_set);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(12);

	(void) close(fd);
	(void) unlink(t2_path);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int fd, i;
	char buf1[NBYTES * 2], buf2[NBYTES * 2];
	int	pathok = setupok;

	DBUG_ENTER("test3");

	for (i = 0; i < (NBYTES * 2); i++)
		buf1[i] = buf2[i] = '\0';

	/* first duplicate the expected file contents in a string */
	for (i = 0; i < (NBYTES / 2); i++)
		buf1[i] = alphabet[i];
	for (i = 0; i < NBYTES; i++)
		buf1[i + (NBYTES / 2)] = alphabet[i];

	DBUG_PRINTF("test3", "expected string = %s\n", buf1);

	if ((fd = open(t3_path, O_RDWR | O_CREAT, MODE3)) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR|O_CREAT, %#o) failed - errno %d (%s)",
			t3_path, (int)MODE3, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	globok = 0;
	(void) tbwrite(fd, alphabet, NBYTES, NBYTES, NOERROR);
	PATH_FUNC_TRACE;

	(void) lseek(fd, (off_t) (NBYTES/2), SEEK_SET);

	globok = 0;
	(void) tbwrite(fd, alphabet, NBYTES, NBYTES, NOERROR);
	PATH_FUNC_TRACE;
	
	if (testfail)
	{
		(void) close(fd);
		(void) unlink(t3_path);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* need to read back file contents for contents check */
	(void) lseek(fd, (off_t)0, SEEK_SET);
	(void) read(fd, buf2, ((3 * NBYTES) / 2));

	/* test now */

	if (strcmp(buf1, buf2))
	{
		xx_rpt(FAILURE);
		in_rpt("The contents of the file differ from that expected.");
		in_rpt("Contents should be %s, actual is %s", buf1, buf2);
	}
	else
		PATH_XS_RPT(4);

	(void) close(fd);
	(void) unlink(t3_path);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int fd;
	long where;
	int	pathok = setupok;

	DBUG_ENTER("test4");

	if ((fd = creat(t4_path, MODEANY)) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d (%s)",
			t4_path, (int)MODEANY, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	globok = 0;
	(void) tbwrite(fd, alphabet, NBYTES, NBYTES, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail)
	{
		(void) close(fd);
		(void) unlink(t4_path);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	where = lseek(fd, (off_t)0, SEEK_CUR);

	if (where != NBYTES)
	{
		xx_rpt(FAILURE);
		in_rpt("file pointer was incorrect after write()");
		in_rpt("Position should be %d, actual is %ld", NBYTES, where);
	}
	else
		PATH_XS_RPT(3);

	(void) close(fd);
	(void) unlink(t4_path);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	int	fd;
	long	off_set;
	struct	stat	fbuf;
	int	pathok = setupok;

	DBUG_ENTER("test5");

	testfail = 0;

	if ((fd = creat(t5_path, MODEANY)) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d (%s)",
			t5_path, (int)MODEANY, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbwrite(fd, alphabet, NBYTES, NBYTES, NOERROR);
	PATH_FUNC_TRACE;

	if (testfail)
	{
		(void) close(fd);
		(void) unlink(t5_path);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	off_set = lseek(fd , (off_t)0 , SEEK_CUR);

	(void) fstat(fd , &fbuf);

	if (off_set != fbuf.st_size)
	{
		if(testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("length of file was not equal to file offset after write() past end of file");
		in_rpt("file pointer: %ld, length of file: %ld", off_set,
			(long)fbuf.st_size);
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) close(fd);
	(void) unlink(t5_path);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test6()
{
	DBUG_ENTER("test6");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test6()
{
	int	rval;
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test6");

	testfail = 0;

	errno = NOERROR;
	globok = 0;
	if (getfifo(t6_path, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbwrite(fildes[WRITE], alphabet, (NBYTES/2), (NBYTES/2), 
		NOERROR);
	PATH_FUNC_TRACE;

	errno = NOERROR;
	rval = lseek(fildes[WRITE] , (off_t)0 , SEEK_SET);
	if (rval != SYSERROR || errno != ESPIPE)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek did not fail as expected");
		valrpt(SYSERROR,rval,ESPIPE,err);
		(void) close(fildes[WRITE]);
		(void) close(fildes[READ]);
		(void) unlink(t6_path);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbwrite(fildes[WRITE], &alphabet[NBYTES/2], (NBYTES/2), 
		(NBYTES/2), NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbread(fildes[READ], in_buf, (unsigned) (NBYTES), NBYTES, 
		NOERROR, alphabet);
	PATH_FUNC_TRACE;

	if(testfail == 0)
		PATH_XS_RPT(5);

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);
	(void) unlink(t6_path);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test7()
{
	int	pathok = setupok;

	DBUG_ENTER("test7");

	testfail = 0;

	globok = 0;
	t7sub(O_RDWR, "O_RDWR");
	PATH_FUNC_TRACE;

	globok = 0;
	t7sub(O_WRONLY, "O_WRONLY");
	PATH_FUNC_TRACE;

	if(testfail == 0)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
t7sub(mode, strmode)
int mode;
char *strmode;
{
	int	fd, err;
	int	pathok = 0;

	DBUG_ENTER("test7");

	testfail = 0;

	if ((fd = creat(t7_path, MODEANY)) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d (%s)",
			t7_path, (int)MODEANY, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbwrite(fd, alphabet, NBYTES/2, NBYTES/2, NOERROR);
	PATH_FUNC_TRACE;
	(void) close(fd);

	if (testfail > 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t7_path, mode | O_APPEND)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, %s|O_APPEND) failed - errno %d (%s)",
			t7_path, strmode, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbwrite(fd, &alphabet[NBYTES/2], NBYTES/2, NBYTES/2, NOERROR);
	PATH_FUNC_TRACE;

	(void) close(fd);

	if (testfail > 0)
	{
		(void) unlink(t7_path);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = open(t7_path, O_RDONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d (%s)",
			err, errname(err));
		(void) unlink(t7_path);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbread(fd, in_buf, NBYTES, NBYTES, NOERROR, alphabet);
	PATH_FUNC_TRACE;

	PATH_FUNC_RPT(8);

	(void) unlink(t7_path);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test8()
{
	DBUG_ENTER("test8");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test8()
{
	DBUG_ENTER("test8");

	(void) cppair(ch_t8, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int	rval, err;
	int	tty_fildes, loop_fildes;
	long	total;
	struct termios tty_termios, loop_termios;
	int	pathok = setupok;

	DBUG_ENTER("ch_t8");

	testfail = 0;

	switch (termios_prep(&tty_fildes, &tty_termios,
			 &loop_fildes, &loop_termios))
	{
	case 0:
		PATH_TRACE;
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	default:
		xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("open/initialise of VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP");
		DBUG_VOID_RETURN;
	}

	/* Turn off output, so the queue can fill up */
	(void) tcflow(tty_fildes, TCOOFF);

	/* Write an initial odd number of bytes, so we don't fill
	   up the output queue _exactly_ writing 512 byte buffers  */

	SET_TIMEOUT(WAITTIME)

	rval = vwrite(tty_fildes, out_buf, 13);
	err = errno;

	CLEAR_ALARM

	/* If there is no output queue, first write will block */
	if (alrm_flag > 0)
	{
		if (rval == -1 && err != EINTR)
		{
			xx_rpt(FAILURE);
			in_rpt("write() set errno to %d (%s), expected %d (EINTR)",
				err, errname(err), EINTR);
		}
		else if (rval < -1)
		{
			xx_rpt(FAILURE);
			in_rpt("write() returned %d, expected -1 or number of bytes",
				rval);
		}
		else
			PATH_XS_RPT(1);

		(void) tcflow(tty_fildes, TCOON);

		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (rval != 13)
	{
		xx_rpt(DELETION);
		in_rpt("initial write() to VSX_TERMIOS_TTY failed");
		(void) tcflow(tty_fildes, TCOON);
		(void) close(tty_fildes);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	   
	/* Fill up the terminal output queue - the last write() is the one
	   we're interested in */

	SET_TIMEOUT(2*WAITTIME)

	total = 0;
	errno = NOERROR;
	while ((rval = vwrite(tty_fildes, out_buf, _POSIX_PIPE_BUF)) > 0)
		if (alrm_flag || (total += rval) > HUGE_QUEUE)
			break;
	err = errno;

	CLEAR_ALARM

	if (total > HUGE_QUEUE)
	{
		xx_rpt(UNTESTED);
		in_rpt("write() did not block after writing %ld bytes to queue",
			total);
	}
	else if (alrm_flag == 0)
	{
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_TTY failed before output queue was full");
	}
	else if (rval == -1 && err != EINTR)
	{
		xx_rpt(FAILURE);
		in_rpt("write() set errno to %d (%s), expected %d (EINTR)",
			err, errname(err), EINTR);
	}
	else if (rval < -1)
	{
		xx_rpt(FAILURE);
		in_rpt("write() returned %d, expected -1 or number of bytes",
			rval);
	}
	else
		PATH_XS_RPT(3);

	(void) tcflush(tty_fildes, TCOFLUSH);
	(void) tcflow(tty_fildes, TCOON);
	(void) close(tty_fildes);
	(void) close(loop_fildes);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test9()
{
	DBUG_ENTER("test9");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test9()
{
	DBUG_ENTER("test9");

	(void) cppair(ch_t9, NULLFN, 2*WAITTIME*(1 + pipe_buf/512), 0);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int	pathok = setupok;

	DBUG_ENTER("ch_t9");

	testfail = 0;

	if (pipe(fildes) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("pipe() fail in child process");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fill_pipe(fildes, WAITTIME * pipe_buf/512, O_NONBLOCK) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("write() timed out when attempting to fill pipe");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	globok = 0;
	(void) tbwrite(fildes[WRITE], out_buf, (unsigned) (pipe_buf+1),
		SYSERROR, EAGAIN);
	PATH_FUNC_TRACE;
	
	CLEAR_ALARM

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);

	globok = 0;
	if (getfifo(t9_path, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (fill_pipe(fildes, WAITTIME * pipe_buf/512, O_NONBLOCK) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("write() timed out when attempting to fill FIFO");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	globok = 0;
	(void) tbwrite(fildes[WRITE], out_buf, (unsigned) (pipe_buf+1),
		SYSERROR, EAGAIN);
	PATH_FUNC_TRACE;

	CLEAR_ALARM
	
	if(testfail == 0)
		PATH_XS_RPT(6);

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	(void) unlink(t9_path);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test10()
{
	DBUG_ENTER("test10");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test10()
{
	int		err;
	int	pathok = setupok;

	DBUG_ENTER("test10");

	testfail = 0;

	errno = NOERROR;
	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not create pipe - errno: %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbwrite(fildes[WRITE], alphabet, (NBYTES/2), (NBYTES/2), 
		NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbwrite(fildes[WRITE], &alphabet[NBYTES/2], (NBYTES/2), 
		(NBYTES/2), NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbread(fildes[READ], in_buf, (unsigned) (NBYTES), NBYTES, 
		NOERROR, alphabet);
	PATH_FUNC_TRACE;

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);

	globok = 0;
	if (getfifo(t10_path, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbwrite(fildes[WRITE], alphabet, (NBYTES/2), (NBYTES/2), 
		NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbwrite(fildes[WRITE], &alphabet[NBYTES/2], (NBYTES/2), 
		(NBYTES/2), NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbread(fildes[READ], in_buf, (unsigned) (NBYTES), NBYTES, 
		NOERROR, alphabet);
	PATH_FUNC_TRACE;

	if(testfail == 0)
		PATH_XS_RPT(8);

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);
	(void) unlink(t10_path);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test11()
{
	DBUG_ENTER("test11");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test11()
{

	DBUG_ENTER("test11");

	(void) cppair(ch_t11, NULLFN, WAITTIME * pipe_buf/512, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	int i, statw, lastfail;
	int	pathok = setupok;
	int	loop1, loop2;

	DBUG_ENTER("ch_t11");

	testfail = 0;
	if (pipe(fildes) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("pipe() fail in child process");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* store initial file status flags */
	statw = fcntl(fildes[WRITE], F_GETFL, 0);

	(void) fcntl(fildes[WRITE], F_SETFL, statw | O_NONBLOCK);

	for (i=1, loop1 = 0 ; i < pipe_buf ; i*=2, loop1++)
	{
		globok = 0;
		(void) tbwrite(fildes[WRITE], out_buf, (unsigned) i, i, 
			NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) tbread(fildes[READ], in_buf, (unsigned) i, i, 
			NOERROR,out_buf);
		PATH_FUNC_TRACE;
		
		if (testfail > 0)
			break;
		else
			PATH_TRACE;
	}
	if (i != pipe_buf && testfail == 0)
	{
		i = pipe_buf;
		globok = 0;
		(void) tbwrite(fildes[WRITE], out_buf, (unsigned) i, i, 
			NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) tbread(fildes[READ], in_buf, (unsigned) i, i, 
			NOERROR,out_buf);
		PATH_FUNC_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);

	globok = 0;
	if (getfifo(t11_path , fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	lastfail = testfail;
	for (i=1, loop2 = 0 ; i < pipe_buf ; i*=2, loop2++)
	{
		globok = 0;
		(void) tbwrite(fildes[WRITE], out_buf, (unsigned) i, i, 
			NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) tbread(fildes[READ], in_buf, (unsigned) i, i, 
			NOERROR,out_buf);
		PATH_FUNC_TRACE;

		if (testfail > lastfail)
			break;
		else
			PATH_TRACE;
	}
	if (i != pipe_buf && testfail == lastfail)
	{
		i = pipe_buf;
		globok = 0;
		(void) tbwrite(fildes[WRITE], out_buf, (unsigned) i, i, 
			NOERROR);
		PATH_FUNC_TRACE;

		globok = 0;
		(void) tbread(fildes[READ], in_buf, (unsigned) i, i, 
			NOERROR,out_buf);
		PATH_FUNC_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}


	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);
	(void) unlink(t11_path);

	if(testfail == 0)
		PATH_XS_RPT(6 + 3 * loop1 + 3 * loop2);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test12()
{
	DBUG_ENTER("test12");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test12()
{
	int	pathok = setupok;

	DBUG_ENTER("test12");

	testfail = 0;

	if (pipe(fildes) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("pipe() fail in child process");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fill_pipe(fildes, WAITTIME * pipe_buf/512, ~O_NONBLOCK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() timed out when attempting to fill pipe");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) cppair(ch_t12, pa_t12, WAITTIME, 0);
	PATH_FUNC_TRACE;

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);

	globok = 0;
	if (getfifo(t12_path, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (fill_pipe(fildes, WAITTIME * pipe_buf/512, ~O_NONBLOCK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() timed out when attempting to fill FIFO");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) cppair(ch_t12, pa_t12, WAITTIME, 0);
	PATH_FUNC_TRACE;

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	(void) unlink(t12_path);

	if (testfail == 0)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
pa_t12()
{
	int ret, err;
	int	pathok = setupok;

	DBUG_ENTER("pa_t12");

	/* child sleeps WAITTIME/2 then reads from pipe */

	SET_TIMEOUT(WAITTIME)

	errno = 0;
	ret = vwrite(fildes[WRITE], out_buf, (unsigned)pipe_buf);
	err = errno;
	
	CLEAR_ALARM

	if (ret != pipe_buf)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("write() returned %d, expected %d",
			ret, pipe_buf);
		if (err != 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	int ret;
	unsigned int read_size;

	DBUG_ENTER("ch_t12");

	(void) sleep(WAITTIME/2);

	/* read from pipe until less than pipe_buf bytes remain */
	read_size = pipe_buf;
	do {
		ret = read(fildes[READ], in_buf, read_size);
		bytes_in_pipe -= ret;
		read_size = pipe_buf/2;

	} while (ret > 0 && bytes_in_pipe >= pipe_buf);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test13()
{
	DBUG_ENTER("test13");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test13()
{
	DBUG_ENTER("test13");

	(void) cppair(ch_t13, NULLFN, 2*WAITTIME*(1 + pipe_buf/512), 0);

	DBUG_VOID_RETURN;
}


private void
ch_t13()
{
	int	ret;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t13");

	testfail = 0;

	if (pipe(fildes) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("pipe() fail in child process");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fill_pipe(fildes, WAITTIME * pipe_buf/512, O_NONBLOCK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() timed out when attempting to fill pipe");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) read(fildes[READ], in_buf, (unsigned)pipe_buf);

	SET_TIMEOUT(WAITTIME)

	errno = NOERROR;
	ret = vwrite(fildes[WRITE], out_buf, (unsigned) (pipe_buf + 1));
	err = errno;

	CLEAR_ALARM

#if TEST_XPG == 3
	if (ret <= 0 && (ret != SYSERROR || errno != EAGAIN))
	{
		testfail++;
		xx_rpt(FAILURE);
		if (ret != SYSERROR)
			in_rpt("write() returned %d, expected -1", ret);
		if (err != EAGAIN)
			in_rpt("write() set errno to %d (%s), expected %d (EAGAIN)",
				err, errname(err), EAGAIN);
	}
#else
	if (ret <= 0)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("write() returned %d, expected > 0", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
#endif
	else
		PATH_TRACE;
	
	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);

	globok = 0;
	if (getfifo(t13_path, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (fill_pipe(fildes, WAITTIME * pipe_buf/512, O_NONBLOCK) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(DELETION);
		in_rpt("deletion reason: write() timed out when attempting to fill FIFO");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) read(fildes[READ], in_buf, (unsigned)pipe_buf);

	SET_TIMEOUT(WAITTIME)

	errno = NOERROR;
	ret = vwrite(fildes[WRITE], out_buf, (unsigned) (pipe_buf + 1));
	err = errno;

	CLEAR_ALARM

#if TEST_XPG == 3
	if (ret <= 0 && (ret != SYSERROR || errno != EAGAIN))
	{
		testfail++;
		xx_rpt(FAILURE);
		if (ret != SYSERROR)
			in_rpt("write() returned %d, expected -1", ret);
		if (err != EAGAIN)
			in_rpt("write() set errno to %d (%s), expected %d (EAGAIN)",
				err, errname(err), EAGAIN);
	}
#else
	if (ret <= 0)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("write() returned %d, expected > 0", ret);
		if (ret < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
#endif
	else
		PATH_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(6);

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	(void) unlink(t13_path);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test14()
{
	DBUG_ENTER("test14");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test14()
{
	/* can write at least PIPE_BUF bytes to empty non-blocking pipe/FIFO */

	DBUG_ENTER("test14");

	(void) cppair(ch_t14, NULLFN, (2*WAITTIME), 0);

	(void) unlink(t14_path);

	DBUG_VOID_RETURN;
}


private void
ch_t14()
{
	int	statw;
	int	ret, err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t14");

	testfail = 0;

	if (pipe(fildes) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("pipe() fail in child process");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* store initial file status flag */
	statw = fcntl(fildes[WRITE], F_GETFL, 0);

	(void) fcntl(fildes[WRITE], F_SETFL, statw | O_NONBLOCK);

	globok = 0;
	(void) tbwrite(fildes[WRITE], out_buf, (unsigned) (pipe_buf/2), 
		(pipe_buf/2), NOERROR);
	PATH_FUNC_TRACE;

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ret = read(fildes[READ], in_buf, (unsigned) (pipe_buf/2));

	if (ret != (pipe_buf/2))
	{
		xx_rpt(DELETION);
		in_rpt("could not read data written to pipe");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	errno = 0;
	ret = vwrite(fildes[WRITE], out_buf, (unsigned) (2 * pipe_buf));
	err = errno;

	CLEAR_ALARM

	if (ret < pipe_buf)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("write() to pipe returned %d, expected >= PIPE_BUF (%d)",
			ret, pipe_buf);
		if (err != 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);

	globok = 0;
	if (getfifo(t14_path, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if (tbwrite(fildes[WRITE], out_buf, (unsigned) (pipe_buf/2), 
			(pipe_buf/2), NOERROR) != pipe_buf/2)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	ret = read(fildes[READ], in_buf, (unsigned) (pipe_buf/2));

	if (ret != (pipe_buf/2))
	{
		xx_rpt(DELETION);
		in_rpt("could not read data written to FIFO");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)

	errno = 0;
	ret = vwrite(fildes[WRITE], out_buf, (unsigned) (2 * pipe_buf));
	err = errno;

	CLEAR_ALARM

	if (ret < pipe_buf)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt("write() to FIFO returned %d, expected >= PIPE_BUF (%d)",
			ret, pipe_buf);
		if (err != 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
		PATH_TRACE;

	if(testfail == 0)
		PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test15()
{
	int		fd;
	char		*mess = (char *)NULL;
#if !TEST_XNFS
	int		err;
#endif /* TEST_XNFS */
	int	pathok = setupok;

	DBUG_ENTER("test15");

/* for a regular file */
	testfail = 0;

	errno = NOERROR;
	if ((fd = creat(t15_path , MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed",
			t15_path, (int)MODEANY);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	savtimes(t15_path);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbwrite(fd, alphabet, (unsigned) MSG_LEN, MSG_LEN, NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	if ((mess = chktimes(t15_path , NEWCTIME|NEWMTIME)) != NULL)
	{
		testfail++;
		xx_rpt(FAILURE);
		in_rpt(mess);
	}
	else
		PATH_FUNC_TRACE;

	(void) close(fd);
	(void) unlink(t15_path);

#if !TEST_XNFS

/* for a FIFO */
	if (getfifo(t15_path, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	savtimes(t15_path);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbwrite(fildes[WRITE], alphabet, (unsigned) MSG_LEN, 
		MSG_LEN, NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	if ((mess = chktimes(t15_path , NEWCTIME|NEWMTIME)) != NULL)
	{
		if(testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt(mess);
	}
	else
		PATH_FUNC_TRACE;

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);
	(void) unlink(t15_path);

/* for a pipe */

	if (pipe(fildes) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: pipe() failed - errno: %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	fsavtimes(fildes[WRITE]);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tbwrite(fildes[WRITE], alphabet, (unsigned) MSG_LEN, 
		MSG_LEN, NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	if ((mess = fchktimes(fildes[WRITE] , NEWCTIME|NEWMTIME)) != NULL)
	{
		if(testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt(mess);
	}
	else
		PATH_FUNC_TRACE;

	(void) close(fildes[WRITE]);
	(void) close(fildes[READ]);

#else
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif /* TEST_XNFS */

	if (testfail == 0)
		PATH_XS_RPT(12);

	DBUG_VOID_RETURN;
}

private void
test16()
{

	DBUG_ENTER("test16");

	(void) cppair(ch_t16, NULLFN, 4 * WAITTIME + 20, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t16()
{
	int 	err;
	char	*args[6];

	DBUG_ENTER("ch_t16");

	args[0] = progsu;
	args[1] = t16_dir;
	args[2] = "shortwrite";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(progsu), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("exec failed - errno %s (%d)", errname(err), err);

	DBUG_VOID_RETURN;
}

private void
test17()
{
	/* write() to regular file overwrites existing data */

	int fd, err;
	char expbuf[30];
	int pathok = setupok;
	static char digits[] = "0123456789";

	DBUG_ENTER("test17");

	testfail = 0;

	if ((fd = creat(t17_path, MODEANY)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed - errno %d (%s)",
			t17_path, (int)MODEANY, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbwrite(fd, alphabet, MSG_LEN, MSG_LEN, NOERROR);

	(void) close(fd);

	if (testfail > 0)
		DBUG_VOID_RETURN
	else
		PATH_FUNC_TRACE;

	if ((fd = open(t17_path, O_RDWR)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDWR) failed - errno %d (%s)",
			t17_path, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbread(fd, in_buf, 5, 5, NOERROR, alphabet);
	if (testfail > 0)
	{
		(void) close(fd);
		DBUG_VOID_RETURN
	}
	else 
		PATH_FUNC_TRACE;

	globok = 0;
	(void) tbwrite(fd, digits, 10, 10, NOERROR);

	(void) close(fd);

	if (testfail > 0)
		DBUG_VOID_RETURN
	else 
		PATH_FUNC_TRACE;

	if ((fd = open(t17_path, O_RDONLY)) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY) failed - errno %d (%s)",
			t17_path, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	(void) memcpy((void *)expbuf, (void *)alphabet, (size_t)MSG_LEN);
	(void) memcpy((void *)&expbuf[5], (void *)digits, (size_t)10);

	globok = 0;
	(void) tbread(fd, in_buf, MSG_LEN, MSG_LEN, NOERROR, expbuf);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	(void) close(fd);
	(void) unlink(t17_path);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test18()
{
	DBUG_ENTER("test18");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test18()
{
	DBUG_ENTER("test18");

	(void) cppair(ch_t18, NULLFN, 2*WAITTIME*(1 + pipe_buf/512), 0);

	DBUG_VOID_RETURN;
}

private void
ch_t18()
{
	int	ret, err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t18");

	testfail = 0;

	if (pipe(fildes) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("pipe() fail in child process");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (fill_pipe(fildes, WAITTIME * pipe_buf/512, O_NONBLOCK) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("write() timed out when attempting to fill pipe");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check for EAGAIN with pipe full */

	SET_TIMEOUT(WAITTIME)

	globok = 0;
	(void) tbwrite(fildes[WRITE], out_buf, (unsigned) (pipe_buf), SYSERROR ,
		EAGAIN);
	PATH_FUNC_TRACE;
	
	CLEAR_ALARM

	(void) read(fildes[READ], in_buf, (unsigned)(pipe_buf-1));

	/* fixed-capacity pipes now have room for PIPE_BUF-1 more bytes,
	   others may be able to accept PIPE_BUF bytes */

	SET_TIMEOUT(WAITTIME)

	errno = 0;
	ret = vwrite(fildes[WRITE], out_buf, (unsigned) (pipe_buf));
	err = errno;
	
	CLEAR_ALARM

	if (ret != pipe_buf && !(ret == SYSERROR && err == EAGAIN))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("after reading %d (PIPE_BUF-1) bytes from full pipe,",
			pipe_buf-1);
	 	in_rpt("write(fd, buf, %d) returned %d with errno %d (%s)",
			pipe_buf, ret, err, errname(err));
		in_rpt("expected return value %d, or -1 with EAGAIN",
			pipe_buf);
	}
	else
		PATH_TRACE;

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);

	globok = 0;
	if (getfifo(t18_path, fildes, WAITTIME) != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (fill_pipe(fildes, WAITTIME * pipe_buf/512, O_NONBLOCK) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("write() timed out when attempting to fill FIFO");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check for EAGAIN with FIFO full */

	SET_TIMEOUT(WAITTIME)

	globok = 0;
	(void) tbwrite(fildes[WRITE], out_buf, (unsigned) (pipe_buf), SYSERROR ,
		EAGAIN);
	PATH_FUNC_TRACE;
	
	CLEAR_ALARM

	(void) read(fildes[READ], in_buf, (unsigned)(pipe_buf-1));

	/* fixed-capacity FIFOs now have room for PIPE_BUF-1 more bytes,
	   others may be able to accept PIPE_BUF bytes */

	SET_TIMEOUT(WAITTIME)

	errno = 0;
	ret = vwrite(fildes[WRITE], out_buf, (unsigned) (pipe_buf));
	err = errno;
	
	CLEAR_ALARM

	if (ret != pipe_buf && !(ret == SYSERROR && err == EAGAIN))
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("after reading %d (PIPE_BUF-1) bytes from full FIFO,",
			pipe_buf-1);
	 	in_rpt("write(fd, buf, %d) returned %d with errno %d (%s)",
			pipe_buf, ret, err, errname(err));
		in_rpt("expected return value %d, or -1 with EAGAIN",
			pipe_buf);
	}
	else
		PATH_TRACE;

	if(testfail == 0)
		PATH_XS_RPT(8);

	(void) close(fildes[READ]);
	(void) close(fildes[WRITE]);
	(void) unlink(t18_path);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test19()
{
	DBUG_ENTER("test19");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test19()
{
	int	pathok = setupok;

	DBUG_ENTER("test19");

	testfail = 0;

	globok = 0;
	(void) tbwrite(-1, alphabet, NBYTES, SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test20()
{
	int ret, fd;
	int	pathok = setupok;

	DBUG_ENTER("test20");

	testfail = 0;

	fd = open(t20_path, O_RDONLY | O_CREAT, MODEANY);
	if (fd < 0)
	{
		xx_rpt(DELETION);
		in_rpt("open(%s, O_RDONLY|O_CREAT, %#o) failed - errno %d (%s)",
			t20_path, (int)MODEANY, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = tbwrite(fd, alphabet, NBYTES, SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	DBUG_PRINTF("test20", "File is read only, return = %d\n", ret);

	(void) close(fd);

	globok = 0;
	ret = tbwrite(fd, alphabet, NBYTES, SYSERROR, EBADF);
	PATH_FUNC_TRACE;

	DBUG_PRINTF("test20", "File is closed, return = %d\n", ret);

	if (testfail == 0)
		PATH_XS_RPT(3);

	(void) unlink(t20_path);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test21()
{
	DBUG_ENTER("test21");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test21()
{

	DBUG_ENTER("test21");

	if ((t21_syn = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: opensync() failed");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t21, pa_t21, (2*WAITTIME), 0);

	(void) closesync(t21_syn);

	DBUG_VOID_RETURN;
}


private void
pa_t21()
{
	int dummy;

	DBUG_ENTER("pa_t21");

	(void) close(t21_syn[0][WRITE]);
	(void) waitsync(t21_syn[0][READ], (char *) &dummy, sizeof(dummy),
		(unsigned)(2*WAITTIME*(1 + pipe_buf/512)));

	(void) sleep(WAITTIME/2);

	(void) kill(tet_child, SIGUSR1);
	globok = 1;

	DBUG_VOID_RETURN;
}

private void
ch_t21()
{
	int dummy;
	struct	sigaction	sig;
	int	pathok = setupok;

	DBUG_ENTER("ch_t21");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if(sigaction(SIGUSR1, &sig , NULLSA) == SYSERROR)
	{
		DBUG_EXIT(A_SIGFAIL);
	}
	else
		PATH_TRACE;

	if (pipe(fildes) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("pipe() failure in child process");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	testfail = 0;

	if (fill_pipe(fildes, WAITTIME * pipe_buf/512, ~O_NONBLOCK) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("write() timed out when attempting to fill pipe");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sendsync(t21_syn[0][WRITE], (char *) &dummy, sizeof dummy, WAITTIME);

	globok = 0;
	(void) tbwrite(fildes[WRITE], alphabet, NBYTES, SYSERROR, EINTR);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test22()
{
	DBUG_ENTER("test22");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private	void
test22()
{
	DBUG_ENTER("test22");

	do_feio("w", -1, t22io, t22rpt);

	DBUG_VOID_RETURN;
}

private int
t22io(fp)
FILE *fp;
{
	globok = 1;
	return vwrite(fileno(fp), alphabet, NBYTES);
}

private void
t22rpt(ret, err)
int ret, err;
{
	int	pathok = setupok;

	DBUG_ENTER("t22rpt");

	if (ret != SYSERROR || err != EIO)
	{
		xx_rpt(FAILURE);
		in_rpt("write() from orphaned background process group did not give correct results");
		if (ret != SYSERROR)
			in_rpt("expected return: -1, actual: %d", ret);
		else if (err != EIO)
			in_rpt("expected errno: %d(EIO), actual: %d(%s)",
				EIO, err, errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
test23()
{
	DBUG_ENTER("test23");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test23()
{
	struct sigaction sig;
	int	pathok = setupok;

	DBUG_ENTER("test23");

	testfail = 0;

	if (pipe(fildes) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: pipe() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) close(fildes[READ]);

	caught_sig = 0;
	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if(sigaction(SIGPIPE, &sig , NULLSA) == SYSERROR)
	{
		DBUG_EXIT(A_SIGFAIL);
	}
	else
		PATH_TRACE;

	globok = 0;
	(void) tbwrite(fildes[WRITE], alphabet, NBYTES, SYSERROR, EPIPE);
	PATH_FUNC_TRACE;

	if (caught_sig != SIGPIPE)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("SIGPIPE was not received");
	}
	else
		PATH_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(4);

	(void) close(fildes[WRITE]);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test24()
{
	DBUG_ENTER("test24");

	(void) cppair(ch_t24, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t24()
{
	char	*ulimaddr;
	long	nbyte, ulimblks;
	int	fd;
	char	buf[BLK_SIZE];
	int	pathok = setupok;

	DBUG_ENTER("ch_t24");

	testfail = 0;

	ulimaddr = tet_getvar("VSX_ULIMIT_BLKS");
	if (ulimaddr == NULL || *ulimaddr == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_ULIMIT_BLKS not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ulimblks = atol(ulimaddr);
	if (ulimblks == -1)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_ULIMIT_BLKS is set to -1");
		DBUG_VOID_RETURN;
	}
	else if (ulimblks <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_ULIMIT_BLKS must be -1 or > 0");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ulimblks < ULMT_NBLK)
		ulimblks = ULMT_NBLK - ULMT_NBLK%ulimblks + ulimblks;

	errno = NOERROR;
	if ((fd = creat(t24_path, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed", t24_path, (int)MODEANY);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	DBUG_PRINTF("ch_t24","A successful call to setlimit() - limiting the file block size to %ld will prevent any further output to the dbug file in this process. Directing dbug info to the screen via stderr will allow path tracing to be seen.", ulimblks);
	if (setlimit(ulimblks) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setlimit(%ld) failed - errno %d (%s)",
			ulimblks, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Write out exactly the ulimit */
	nbyte = ulimblks * BLK_SIZE;
	while (nbyte > 0)
	{
		int n;

		errno = NOERROR;
		n = vwrite(fd, buf,
		    (unsigned)(nbyte < BLK_SIZE ? nbyte : BLK_SIZE));
		if (n <= 0)
		{
			xx_rpt(DELETION);
			in_rpt("write() failed before file size limit was reached - errno %d (%s)",
				errno, errname(errno));
			DBUG_VOID_RETURN;
		}
		nbyte -= n;
	}

	globok = 0;
	(void) tbwrite(fd, "A", 1, SYSERROR, EFBIG);
	PATH_FUNC_TRACE;

	(void) close(fd);
	(void) unlink(t24_path);

	if (testfail == 0)
		PATH_XS_RPT(5);
	
	DBUG_VOID_RETURN;
}

private void
test25()
{

	DBUG_ENTER("test25");

	(void) cppair(ch_t25, NULLFN, 4 * WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t25()
{
	int 	err;
	char	*args[6];

	DBUG_ENTER("ch_t25");

	args[0] = progsu;
	args[1] = t25_dir;
	args[2] = "write";
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = NULL;

	(void) tet_exec(execpath(progsu), args, environ);

	err = errno;

	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(progsu), err, errname(err));

	DBUG_VOID_RETURN;
}

private int
tbread(tbfildes, tbbuf, tbnbyte, expret, experrno, expdata)
int tbfildes;
char *tbbuf;
unsigned tbnbyte;
int expret, experrno;
char *expdata;
{
	int	rval, err;
	int	failed = 0;
	int	pathok = setupok;
	char	savchar;

	DBUG_LENTER("tbread", 2);

	errno = 0;
	rval = read(tbfildes, tbbuf, tbnbyte);
	err = errno;
	globok = 0;

	if (expret == SYSERROR)
	{
		if (rval == expret && err == experrno)
		{
			PATH_TRACE;
			globok = 1;
			DBUG_RETURN(rval);
		}
		else
			failed++;
	}
	else
	{
		if (expret != rval)
			failed++;
		else
			PATH_TRACE;
	}

	if (failed)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
	 	in_rpt("read(%d, buf, %d) failed", tbfildes, tbnbyte);
		valrpt(expret, rval, experrno, err);
		DBUG_RETURN(rval);
	}
	else
		PATH_TRACE;

#ifdef TESTBED2
	tbbuf[0]++;
#endif

	if (expdata != (char *)NULL &&
	    memcmp((void *)expdata, (void *)tbbuf, (size_t)rval) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
	 	in_rpt("read(%d, buf, %d) returned incorrect data in buf",
			tbfildes, tbnbyte);
		/* for buffers containing nulls this will be incomplete */
		savchar = expdata[rval];
		expdata[rval] = '\0';
		in_rpt("expected: \"%s\"", psl_to_asc(expdata, 0));
		expdata[rval] = savchar;
		tbbuf[rval] = '\0';
		in_rpt("observed: \"%s\"", psl_to_asc(tbbuf, 0));
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_RETURN(rval);

}

private int
tbwrite(tbfildes, tbbuf, tbnbyte, expret, experrno)
int tbfildes;
char *tbbuf;
unsigned tbnbyte;
int expret, experrno;
{
	int	rval, err;
	int	failed = 0;
	int	pathok = setupok;

	DBUG_LENTER("tbwrite", 2);

	errno = 0;
	rval = vwrite(tbfildes, tbbuf, tbnbyte);
#ifdef TESTBED
	rval = -1;
	if (errno)
		errno++;
#endif
	err = errno;

	if (expret == IGNERROR)
	{
		PATH_TRACE;
		globok = 1;
		DBUG_RETURN(rval);
	}
	else
		PATH_TRACE;

	if (expret == SYSERROR)
	{
		if (rval == expret && err == experrno)
		{
			PATH_TRACE;
			PATH_FUNC_RPT(2);
			DBUG_RETURN(rval);
		}
		else
			failed++;
	}
	else
	{
		if (expret != rval)
			failed++;
		else
			PATH_TRACE;
	}

	if (failed)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
	 	in_rpt("write(%d, buf, %d) did not behave as expected",
			tbfildes, tbnbyte);
		valrpt(expret, rval, experrno, err);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_RETURN(rval);
}

#if ! TEST_XNFS

private void
test26()
{
	DBUG_ENTER("test26");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test26()
{
	DBUG_ENTER("test26");

	t_estale_wr_c(t26_file, 0, t26_func, t26_chk_func, SP_T26_ESTALE);

	DBUG_VOID_RETURN;
}

private int
t26_func(fp)
FILE	*fp;
{
	DBUG_ENTER("t26_func");

	DBUG_RETURN(vwrite(fileno(fp), out_buf, 1) == -1 ? -1 : 0);
}

private int
t26_chk_func(fp, ebytes)
FILE	*fp;
int	ebytes;
{
	char	buf[2];
	int	rval, err;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("t26_chk_func");

	errno = NOERROR;

	for(i = 0; i < ebytes; i++) {
		rval = vwrite(fileno(fp), buf, 1);
		if(rval != 1)
			break;
	}

	err = errno;

	if(rval != -1 || err != ESTALE)
	{
		xx_rpt(FAILURE);

		if(rval != -1)
		{
			in_rpt("Return value was incorrect");
			in_rpt("Expected return value -1");
			in_rpt("Actual   return value %d", rval);
		}
		else
			PATH_TRACE;
		
		if(err != ESTALE)
		{
			in_rpt("Errno not set correctly");
			in_rpt("Expected errno %d (%s)",ESTALE,errname(ESTALE));
			in_rpt("Actual   errno %d (%s)", err, errname(err));
		}
		else
			PATH_TRACE;

		DBUG_RETURN(-1);
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	PATH_FUNC_RPT(2);

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
test27()
{
	DBUG_ENTER("test27");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test27()
{
	DBUG_ENTER("test27");

	(void) cppair(ch_t27, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

private void
ch_t27()
{
	int err;
	char *args[6];

	DBUG_ENTER("ch_t27");

	args[0] = prog27;
	args[1] = itoa(tet_thistest);
	args[2] = "";
	args[3] = t27_file;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = NULL;

	(void) tet_exec(execpath(prog27), args, environ);

	err = errno;
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed - errno %d (%s)",
		execpath(prog27), err, errname(err));

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test28()
{
	DBUG_ENTER("test28");

	(void) cppair(ch_t28, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t28()
{
	char	*ulimaddr;
	long	nbyte, ulimblks;
	int	fd;
	char	buf[BLK_SIZE];
	int	pathok = setupok;

	DBUG_ENTER("ch_t28");

	testfail = 0;

	ulimaddr = tet_getvar("VSX_ULIMIT_BLKS");
	if (ulimaddr == NULL || *ulimaddr == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_ULIMIT_BLKS not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ulimblks = atol(ulimaddr);
	if (ulimblks == -1)
	{
		xx_rpt(UNTESTED);
		in_rpt("parameter VSX_ULIMIT_BLKS is set to -1");
		DBUG_VOID_RETURN;
	}
	else if (ulimblks <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_ULIMIT_BLKS must be -1 or > 0");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ulimblks < ULMT_NBLK)
		ulimblks = ULMT_NBLK - ULMT_NBLK%ulimblks + ulimblks;

	errno = NOERROR;
	if ((fd = creat(t28_path, MODEANY)) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("creat(%s, %#o) failed", t28_path, (int)MODEANY);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	DBUG_PRINTF("ch_t28","A successful call to setlimit() - limiting the file block size to %ld will prevent any further output to the dbug file in this process. Directing dbug info to the screen via stderr will allow path tracing to be seen.", ulimblks);
	if (setlimit(ulimblks) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setlimit(%ld) failed - errno %d (%s)",
			ulimblks, errno, errname(errno));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Write to less than a block from the ulimit */
	nbyte = (ulimblks - 1) * BLK_SIZE + 1;
	while (nbyte > 0)
	{
		int n;

		errno = NOERROR;
		n = vwrite(fd, buf,
		    (unsigned)(nbyte < BLK_SIZE ? nbyte : BLK_SIZE));
		if (n <= 0)
		{
			xx_rpt(DELETION);
			in_rpt("write() failed before file size limit was reached - errno %d (%s)",
				errno, errname(errno));
			DBUG_VOID_RETURN;
		}
		nbyte -= n;
	}

	globok = 0;
	(void) tbwrite(fd, buf, BLK_SIZE, BLK_SIZE-1, NOERROR);
	PATH_FUNC_TRACE;

	(void) close(fd);
	(void) unlink(t28_path);

	if (testfail == 0)
		PATH_XS_RPT(5);
	
	DBUG_VOID_RETURN;
}
