/*
*      SCCS:  @(#)  POSIX.os/procenv/ctermid/ctermid.c	 Rel 4.4.1     (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/ctermid - T.ctermid Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.ctermid

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the ctermid() subroutine call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	29 June 1989
MODIFICATIONS:	
		Geoff Clare, 7 June 1990
			UNTESTED results if termios_prep() returns 1

		J.A.Nave	10/07/90
		Added resilience testing

		Stuart Boutell, 26 Mar 91
			Added testing for value 's' returned by ctermid
			in test 3.
		
		Geoff Clare, 8 May 1991
			Change UNTESTED's to UNSUPPORTED

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<string.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<termios.h>
#include	<signal.h>
#include	<timeout.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ctermid
#endif


#define NO_TESTS	3		/* number of tests in testset */
#define WAITTIME	(5 * SPEEDFACTOR)
#define NAP		3
#define	Strlen		(unsigned)(sizeof(wbuf)-1)

extern	char	*errname();
extern	char	*psl_to_asc();

int	no_tests = NO_TESTS;

private void	test1();
private void	test2();
private void	test3();
private void	setup();
private void	cleanup();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = cleanup;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	NULL, 0
};

private char		*prep_fail = NULL;
private int		setupok = 0;
private int		prep_err = 0;
private int		gti_unsup = 0;
private	int		tty_fd, loop_fd;
private	struct termios	tty_term, loop_term;
private char		wbuf[] = "Alas, poor Yorik!  I knew him, Horatio.\n";
private	char		rbuf[256];

private	void
setup()
{
	int ret;
	int	pathok =  0;

	DBUG_ENTER("setup");

	switch (termios_prep(&tty_fd, &tty_term, &loop_fd, &loop_term))
	{
	case 0:
		break;
	case 1:
		gti_unsup = 1;
		DBUG_VOID_RETURN;
	default:
		(void) tet_delete(1, "termios_prep() failed in setup");
		(void) tet_delete(2, "termios_prep() failed in setup");
		DBUG_VOID_RETURN;
	}

	/* check loopback works */

	SET_TIMEOUT(WAITTIME/2)

	errno = 0;
	ret = write(tty_fd, wbuf, Strlen);
	prep_err = errno;

	CLEAR_ALARM

	if (ret != Strlen)
	{
		prep_fail = "write() to VSX_TERMIOS_TTY failed";
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME/2)

	errno = 0;
	ret = read(loop_fd, rbuf, sizeof(rbuf));
	prep_err = errno;

	CLEAR_ALARM

	if (ret != Strlen)
	{
		prep_fail = "read() from VSX_TERMIOS_LOOP failed";
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	rbuf[ret] = '\0';
	if (strcmp(rbuf, wbuf) != 0)
	{
		prep_fail = "data written to VSX_TERMIOS_TTY was not read correctly from VSX_TERMIOS_LOOP";
		prep_err = -1;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	setupok = (pathok == 3) ? 0 : -1;

	DBUG_VOID_RETURN;
}

private	void
cleanup()
{
	DBUG_ENTER("cleanup");
	(void) close(tty_fd);
	(void) close(loop_fd);
	DBUG_VOID_RETURN;
}

private int
chk_ctty(file)
char *file;
{
	int ret, fd, err;
	int	pathok = 0;

	DBUG_ENTER("chk_ctty");

	if (prep_fail != NULL)
	{
		xx_rpt(DELETION);
		in_rpt("%s", prep_fail);
		if (prep_err > 0)
			in_rpt("errno was set to %d (%s)",
				prep_err, errname(prep_err));
		else if (prep_err == -1)
		{
			in_rpt("expected: \"%s\"", psl_to_asc(wbuf, 0));
			in_rpt("observed: \"%s\"", psl_to_asc(rbuf, 0));
		}

		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	(void) tcflush(tty_fd, TCIOFLUSH);
	(void) tcflush(loop_fd, TCIOFLUSH);

	/* write to device returned by ctermid() and expect to
	   read it on VSX_TERMIOS_LOOP */

	SET_TIMEOUT(WAITTIME/2)

	errno = 0;
	fd = open(file, O_WRONLY|O_NONBLOCK);
	err = errno;

	CLEAR_ALARM

	if (fd < 0)
	{
		xx_rpt(FAILURE);
		in_rpt("open(O_WRONLY|O_NONBLOCK) failed on path returned by ctermid() - errno %d (%s)",
			err, errname(err));
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME/2)

	errno = 0;
	ret = write(fd, wbuf, Strlen);
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(FAILURE);
		in_rpt("write() to path returned by ctermid() failed - errno %d (%s)",
			err, errname(err));
		(void) close(fd);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	(void) sleep(NAP);

	SET_TIMEOUT(WAITTIME/2)

	errno = 0;
	ret = read(loop_fd, rbuf, sizeof(rbuf));
	err = errno;

	CLEAR_ALARM

	if (ret != Strlen)
	{
		xx_rpt(FAILURE);
		in_rpt("data written to path returned by ctermid() could not be read from VSX_TERMIOS_LOOP");
		in_rpt("read() failed with errno %d (%s)", err, errname(err));
		(void) close(fd);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	rbuf[ret] = '\0';
	if (strcmp(rbuf, wbuf) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("data written to path returned by ctermid() was not read correctly from VSX_TERMIOS_LOOP");
		in_rpt("expected: \"%s\"", psl_to_asc(wbuf, 0));
		in_rpt("observed: \"%s\"", psl_to_asc(rbuf, 0));
		(void) close(fd);
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	(void) close(fd);

	globok = (pathok == 5) ? 1 : 0;

	DBUG_RETURN(0);
}

private void
test1()
{
	char	s[2 * L_ctermid];
	int	pathok = setupok;

	DBUG_ENTER("test1");

	if (gti_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( ctermid(s) != s )
	{
		xx_rpt(FAILURE);
		in_rpt("ctermid(valid pointer) did not return that pointer");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (chk_ctty(s) == 0)
	{
		if (globok == 1)
			PATH_TRACE;
		PATH_XS_RPT(3);
	}
	
	DBUG_VOID_RETURN;
}

private void
test2()
{
	char *s;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	if (gti_unsup)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( (s = ctermid((char *) NULL)) == (char *) NULL )
	{
		xx_rpt(FAILURE);
		in_rpt("ctermid(NULL) returned NULL pointer");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (chk_ctty(s) == 0)
	{
		if (globok == 1)
			PATH_TRACE;
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

private void
test3()
{
	size_t	len;
	char	s[2 * L_ctermid];
	char	*retval;
	int	pathok = setupok;

	DBUG_ENTER("test3");

	s[0] = '\0';
	if ( ( retval=ctermid(s) ) == NULL )
	{
		xx_rpt(FAILURE);
		in_rpt("ctermid(s) returned NULL");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((len = strlen(s)) >= L_ctermid)
	{
		xx_rpt(FAILURE);
		in_rpt("ctermid(s) placed string of length %d in s", (int)len);
		in_rpt("expected length < L_ctermid (%d)", (int)L_ctermid);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#ifdef TESTBED
	retval++;
#endif
	if (retval != s)
	{
		xx_rpt(FAILURE);
		in_rpt("ctermid(s) did not return s");
		in_rpt("Expected 0x%lx, observed 0x%lx",
			(long)s , (long)retval);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	PATH_XS_RPT(3);
	DBUG_VOID_RETURN;
}
