/*
*      SCCS:  @(#)  POSIX.os/procenv/getegid/getegid_t1.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/getegid - getegid_t1 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/
/***********************************************************************

NAME:		getegid_t1

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the getegid() system call.

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Thr Jan 12 16:35 GMT 1989
MODIFICATIONS:	J.A.Nave	10/07/90
		Added resilience testing

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env, envp.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

                Geoff Clare, 8 June 1992
			Changed return type of tb_getegid() to gid_t.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/times.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<limits.h>
#include	<pwd.h>
#include	<exitcodes.h>
#include	<unistd.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getegid
#endif


extern	char	*errname();
extern	void	set_pname();
extern	int	uids();

extern	int	vggid1;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	gid_t	tb_getegid();
private	void	subtest1();

public	int
tet_main(argc, argv )		/* Assertion 04 */
	int	argc;
	char	**argv;
{
	int	pathok = 0;
 
	DBUG_ENTER("subtest1(main)");
	set_pname(argc < 1 || argv[1] == (char *) NULL ?
		"getegid_t1" : (argv[1]));
	if (argc == 3)
	{
		if (argv[2] != (char *) NULL && strncmp("-D", argv[2], 2) == 0)
		{
			vg_dbug = argv[2];
			DBUG_PUSH(&vg_dbug[2]);
		}
		argc--;
	}

	if (argc != 2)
	{
		in_rpt("argument count wrong, got %d, expected 2",argc);
		exit(A_MISC);
	}
	else
		PATH_TRACE;

	if (argv[1] != (char *) NULL && argv[1][0] != NUL)
		vg_ident = argv[1];


	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;
	
	globok = (pathok == 2) ? 1 : 0;

	subtest1();

	DBUG_EXIT(OK);

}


private void
subtest1()
{
	gid_t	egid;
	int	pathok = 0;

	DBUG_ENTER("subtest1");

	if (globok == 1)
		PATH_TRACE;

	egid = tb_getegid();
	if ( egid != (gid_t)vggid1)
	{
		xx_rpt(FAILURE);
		in_rpt("Effective GID returned (%ld) not equal to VSX_GID1 (%d)",
			(long)egid, vggid1);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private	gid_t
tb_getegid()
{
	gid_t	retval;

	retval = getegid();
#ifdef	TESTBED
	errno++;
	return 9999;
#else
	return retval;
#endif
}
