/*
*      SCCS:  @(#)  POSIX.os/procenv/getenv/getenv.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/getenv - T.end Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.getenv

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the getenv() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Tue Apr 11 17:55:25 BST 1989
MODIFICATIONS:
		Geoff Clare, 18 Oct 1989
			Check for NULL return from malloc()

		J.A.Nave	10/07/90
		Added resilience testing

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.
			Added (void *) cast in free() calls.
		
		Geoff Clare, 22 Jan 1993
			Allow for VSX_DBUG_FILE being unset.

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<errno.h>
#include	<pathres.h>
#include	<string.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getenv
#endif


#define 	NO_TESTS	4		/* number of tests in testset */
#define		EXTERR		"external error"
#define		WAITTIME	(5 * SPEEDFACTOR)
#define		LENGTH		128	/* used in calculating length of 
								buffer */
#define		NMR_ENV_VARS	11	/* number of environment variables for
								tet_exec */

extern	int	cppair();
extern	char	*itoa();
extern	char	*vg_dbug;
extern	char	*vg_ident;

int	no_tests = NO_TESTS;

private	char	*env[NMR_ENV_VARS];
private	char	*args[10];

private void	test1();
private void	ch_t1();
private	void	test2();
private	void	ch_t2();
private	void	test3();
private	void	ch_t3();
private	void	test4();
private	void	ch_t4();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private void
test1()
{
	char *cp;

	DBUG_ENTER("test1");

	/* set up environment for tet_exec call */

	cp = tet_getvar("VSX_DBUG_FILE");
	if (cp == NULL)
		cp = "";
	env[0] = (char *) malloc( (size_t)(15+strlen(cp)));
	(void) sprintf(env[0],"VSX_DBUG_FILE=%s", cp);
	env[1] = "ENV0=VALUE0";
	env[2] = "ENV1=VALUE1";
	env[3] = "ENV2=VALUE2";
	env[4] = (char *) NULL;
	args[0] = "getenv1";
	args[1] = "1";
	args[2] = vg_ident;
	args[3] = "0";
	args[4] = vg_dbug;
	args[5] = (char *) NULL;

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	free((void *)env[0]);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	DBUG_ENTER("ch_t1");

	(void) tet_exec("../getenv1", args, env);

	exit(A_EXECFAIL);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	char *cp;

	DBUG_ENTER("test2");

	/* set up environment for tet_exec call */

	cp = tet_getvar("VSX_DBUG_FILE");
	if (cp == NULL)
		cp = "";
	env[0] = (char *) malloc( (size_t)(15+strlen(cp)));
	(void) sprintf(env[0],"VSX_DBUG_FILE=%s", cp);
	env[1] = "ENV0=VALUE0";
	env[2] = "ENV1=VALUE1";
	env[3] = "ENV2=VALUE2";
	env[4] = (char *) NULL;
	args[0] = "getenv1";
	args[1] = "2";
	args[2] = vg_ident;
	args[3] = "0";
	args[4] = vg_dbug;
	args[5] = (char *) NULL;

	(void) cppair(ch_t2, NULLFN, WAITTIME, 0);

	free((void *)env[0]);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	DBUG_ENTER("ch_t2");
	
	(void) tet_exec("../getenv1", args, env);

	exit(A_EXECFAIL);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	char *cp;

	DBUG_ENTER("test3");

	/* set up environment for tet_exec call */

	cp = tet_getvar("VSX_DBUG_FILE");
	if (cp == NULL)
		cp = "";
	env[0] = (char *) malloc( (size_t)(15+strlen(cp)));
	(void) sprintf(env[0],"VSX_DBUG_FILE=%s", cp);
	env[1] = "AaZz=AaZz";
	env[2] = "ABCabc=ABCabc";
	env[3] = "xyzXYZ=xyzXYZ";
	env[4] = (char *) NULL;
	args[0] = "getenv1";
	args[1] = "3";
	args[2] = vg_ident;
	args[3] = "0";
	args[4] = vg_dbug;
	args[5] = (char *) NULL;

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	free((void *)env[0]);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	DBUG_ENTER("ch_t3");
	
	(void) tet_exec("../getenv1", args, env);

	exit(A_EXECFAIL);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	i;
	char	**p;
	char	*cp;
	int	pathok = 0;

	DBUG_ENTER("test4");

	/* set up environment for tet_exec call */

	cp = tet_getvar("VSX_DBUG_FILE");
	if (cp == NULL)
		cp = "";
	env[0] = (char *) malloc( (size_t)(15+strlen(cp)));
	(void) sprintf(env[0],"VSX_DBUG_FILE=%s", cp);

	for( i=1 , p=portfnam ; *p != NULL && i < NMR_ENV_VARS - 1; p++ , i++)
	{
		env[i] = (char *) malloc((size_t) (2*strlen(*p)+2));
		if (env[i] == NULL)
		{
			xx_rpt(DELETION);
			in_rpt("malloc() failed");
			DBUG_VOID_RETURN;
		}
		(void) strcpy(env[i] , *p);
		(void) strcat(env[i] , "=");
		(void) strcat(env[i] , *p);
		if (i == 1)
			PATH_TRACE;
	}
	env[i] = (char *) NULL;
	args[0] = "getenv1";
	args[1] = "4";
	args[2] = vg_ident;
	args[3] = itoa(pathok);
	args[4] = vg_dbug;	/* must be last! */
	args[5] = (char *) NULL;

	(void) cppair(ch_t4, NULLFN, WAITTIME, 0);

	free((void *)env[0]);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	DBUG_ENTER("ch_t4");
	
	(void) tet_exec("../getenv1", args, env);

	exit(A_EXECFAIL);

	DBUG_VOID_RETURN;
}
