/*
*      SCCS:  @(#)  POSIX.os/procenv/getenv/getenv1.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/getenv - getenv1 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		getenv1 - used by T.getenv in comply()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.getenv to test getenv() byecking if variables
	  given as arguments to tet_exec() are returned by getenv().

	  usage:	T.getenv vg_ident vg_dbug

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Tue Oct 28 16:04:12 GMT 1986
MODIFICATIONS:	J.A.Nave	10/07/90
		Added resilience testing

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

************************************************************************/
#include	<std.h>
#include	<setjmp.h>
#include	<dbug.h>
#include	<signal.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<string.h>
#include	<pathres.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getenv
#endif


#define	NO_TESTS	4		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	NMR_ENV_VARS	10

extern	void	set_pname();
extern	char	*atocp();

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	void	subtest1();
private	void	subtest2();
private	void	subtest3();
private	void	subtest4();

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	int pathok = 0;

	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"getenv1" : (argv[0]));

	switch (argc)
	{
	case 5:
		vg_dbug = argv[4];
		/* fall through */
	case 4:
		if (argv[3] != (char *) NULL)
			pathok = atoi(argv[3]);
		vg_ident = argv[2];
		break;
	default:
		in_rpt("argument count wrong");
		exit(A_MISC);
	}

	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	switch (atoi(argv[1]))
	{
		case 1:
			subtest1();
			break;
		case 2:
			subtest2();
			break;
		case 3:
			subtest3();
			break;
		case 4:
			subtest4(pathok);
			break;
		default:
			in_rpt("Invalid testcase");
			exit(A_MISC);
			break;
	}

	exit(OK);

	/*NOTREACHED*/
	return 0;
}


private void
subtest1()
{
	char	*cp0, *cp1, *cp2;
	int	fail = 0;
	int	pathok = 0;

	DBUG_ENTER("subtest1");

	if ((cp0 = getenv("ENV0")) == (char  *) NULL)
		fail = 01;
	else if (strcmp(cp0, "VALUE0") != 0)
		fail = 02;
	else
		PATH_TRACE;

	if ((cp1 = getenv("ENV1")) == (char  *) NULL)
		fail |= 04;
	else if (strcmp(cp1, "VALUE1") != 0)
		fail |= 010;
	else
		PATH_TRACE;

	if ((cp2 = getenv("ENV2")) == (char  *) NULL)
		fail |= 020;
	else if (strcmp(cp2, "VALUE2") != 0)
		fail |= 040;
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 01)
			in_rpt("getenv(ENV0) returned NULL pointer");
		else if (fail & 02)
			in_rpt("getenv(ENV0) returned %s - should have been %s",
								cp0, "VALUE0");
		else if (fail & 04)
			in_rpt("getenv(ENV1) returned NULL pointer");
		else if (fail & 010)
			in_rpt("getenv(ENV1) returned %s - should have been %s",
								cp1, "VALUE1");
		else if (fail & 020)
			in_rpt("getenv(ENV2) returned NULL pointer");
		else if (fail & 040)
			in_rpt("getenv(ENV2) returned %s - should have been %s",
								cp2, "VALUE2");
	}
	else
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


private void
subtest2()
{
	char	*cp;
	int	pathok = 0;

	DBUG_ENTER("subtest2");

	if ((cp = getenv("NO_SUCH_VAR")) != (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("getenv on nonexistant variable NO_SUCH_VAR returned 0x%x instead of NULL pointer", cp);
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}

private void
subtest3()
{
	char	*cp0, *cp1, *cp2;
	int	fail = 0;
	int	pathok = 0;

	DBUG_ENTER("subtest3");

	if ((cp0 = getenv("AaZz")) == (char  *) NULL)
		fail = 01;
	else if (strcmp(cp0, "AaZz") != 0)
		fail = 02;
	else
		PATH_TRACE;

	if ((cp1 = getenv("ABCabc")) == (char  *) NULL)
		fail |= 04;
	else if (strcmp(cp1, "ABCabc") != 0)
		fail |= 010;
	else
		PATH_TRACE;

	if ((cp2 = getenv("xyzXYZ")) == (char  *) NULL)
		fail |= 020;
	else if (strcmp(cp2, "xyzXYZ") != 0)
		fail |= 040;
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 01)
			in_rpt("getenv(AaZz) returned NULL pointer");
		else if (fail & 02)
			in_rpt("getenv(AaZz) returned %s - should have been %s",
								cp0, "AaZz");
		else if (fail & 04)
			in_rpt("getenv(ABCabc) returned NULL pointer");
		else if (fail & 010)
			in_rpt("getenv(ABCabc) returned %s - should have been %s",
								cp1, "ABCabc");
		else if (fail & 020)
			in_rpt("getenv(xyzXYZ) returned NULL pointer");
		else if (fail & 040)
			in_rpt("getenv(xyzXYZ) returned %s - should have been %s",
								cp2, "xyzXYZ");
	}
	else
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
subtest4(pathok)
int pathok;	/* expect 1 */
{
	char	*cp0;
	int	fail = 0;
	char	**p;
	int	i = 0;

	DBUG_ENTER("subtest4");

	for(p=portfnam ; *p != NULL ; p++)
	{
		if ((cp0 = getenv(*p)) == (char  *) NULL)
		{
			if(!fail++)
				xx_rpt(FAILURE);
			in_rpt("getenv(%s) returned NULL pointer",*p);
		}
		else if (strcmp(cp0, *p) != 0)
		{
			if(!fail++)
				xx_rpt(FAILURE);
			in_rpt("getenv(%s) returned %s - should have been %s",
				*p, cp0, *p);
		}
		else if (i++ == 0)
			PATH_TRACE;

	}

	if (!fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

