/*
*      SCCS:  @(#)  POSIX.os/procenv/getgid/getgid.c	Rel 4.4.1 (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/getgid - T.getgid Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.getgid

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the getgid() system call.

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Fri Jan 13 11:57:13 GMT 1989
MODIFICATIONS:	
		Dave Cater, UniSoft Ltd.
		Tue Oct 31 12:51:58 GMT 1989
		Altered to use setprv(PRV_SETID) to check appropriate
		privileges.

		J.A.Nave	10/07/90
		Added resilience testing

		Stuart Boutell, 2 Nov 1990
			changed execl for tet_exec.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

************************************************************************/
#include	<std.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<exitcodes.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getgid
#endif


#define	NO_TESTS	1		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)

int	no_tests = NO_TESTS;

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	int	uids();
extern	char	*newstr();
extern	char	*utoa();
extern	char	*itoa();
extern	int	vggid1;
extern	int	vggid2;

private	void	test1();
private	void	ch_t1();
private	char	*getgid_t01	=	"../getgid_t01";

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private void
test1()
{

	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	extern	char	**environ;
	char	*args[7];
	char	*gid1str,	*gid2str,	*pathok_str;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	if(uids()!=0)
	{
		xx_rpt(DELETION);
	}
	else if (setprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_SETID) failed");
	}
	else if(setgid((gid_t) vggid1) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setgid(vsxg1) failed");
	}
	else
	{
		PATH_TRACE;
		if ((gid1str = newstr(utoa((unsigned) vggid1))) == (char *)NULL)
			exit(A_NOMEM);
		if ((gid2str = newstr(utoa((unsigned) vggid2))) == (char *)NULL)
			exit(A_NOMEM);
		if ((pathok_str = newstr(itoa(pathok))) == (char *)NULL)
			exit(A_NOMEM);
		/* note that vg_dbug must be last active parameter */

		args[0] =getgid_t01;
		args[1] =gid1str;
		args[2] =gid2str;
		args[3] =vg_ident;
		args[4] =pathok_str;
		args[5] =vg_dbug;
		args[6] =(char *)NULL;
		(void)	tet_exec(getgid_t01, args, environ);
	}

	DBUG_VOID_RETURN;
}
