/*
*      SCCS:  @(#)  POSIX.os/procenv/getgid_t01/getgid_t01.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/getgid_t01 - getgid_t01 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		getgid_t01

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the getgid() system call.

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Thr Jan 12 16:35 GMT 1989
MODIFICATIONS:	
		J.A.Nave	10/07/90
		Added resilience testing

		David Sawyer
		Wed Aug 08 15:43:13 BST 1990
		Removed resilience code from main and reorganised
		argument checking.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env and envp.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

                Geoff Clare, 8 June 1992
			Changed return type of tb_getgid() to gid_t.

************************************************************************/


#include	<std.h>
#include	<errno.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/times.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<limits.h>
#include	<pwd.h>
#include	<exitcodes.h>
#include	<unistd.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getgid
#endif


extern	char	*errname();
extern	void	set_pname();

extern	int	vggid1;
extern	int	vggid2;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	gid_t	tb_getgid();
private	void	subtest1();

public	int
tet_main(argc, argv)		/* Assertion 03 */

	int	argc;
	char	**argv;
{
	int	pathok = 0;

	set_pname(argc < 1 || argv[1] == (char *) NULL ?
		"getgid_t01" : (argv[1]));

	switch(argc)
	{
	case 6:
		vg_dbug = argv[5];
		/* fall through */
	case 5:
		if (argv[4] != (char *) NULL)
			pathok = atoi(argv[4]);
		vg_ident = argv[3];
		break;
	default:
		in_rpt("argument count wrong");
		exit(A_MISC);
	}

	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	vggid1 = atoi(argv[1]);
	vggid2 = atoi(argv[2]);

	subtest1(pathok);

	exit(OK);

}


private void
subtest1(pathok)
int	pathok;
{
	gid_t	gid;

	DBUG_ENTER("subtest1");
	
	if ( (gid = getegid()) != (gid_t)vggid2)
	{
		xx_rpt(FAILURE);
		in_rpt("Effective GID returned (%ld) not equal to VSX_GID2 (%d)",
			(long)gid, vggid2);
	}
	else if ( (gid = tb_getgid()) != (gid_t)vggid1)
	{
		xx_rpt(FAILURE);
		in_rpt("Real GID returned (%ld) not equal to VSX_GID1 (%d)",
			(long)gid, vggid1);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private	gid_t
tb_getgid()
{
	gid_t	retval;

	retval = getgid();
#ifdef	TESTBED
	errno++;
	return 9999;
#else
	return retval;
#endif
}
