/*
*      SCCS:  @(#)  POSIX.os/procenv/getgroups/getgroups.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/getgroups - T.end Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.getgroups

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the getgroups() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	6 July 1989
MODIFICATIONS:	
		Geoff Clare, 14 Sep 1989
			Array entries from retval to gidsetsize-1
			are allowed to be modified - only check
			gidsetsize and above

		J.A.Nave	10/07/90
		Added resilience testing

		David Sawyer
		Tue Jul 24 15:18:49 BST 1990
		Added resilience code.

		Geoff Clare, 15 Oct 1990
			setgrps() return -1 with ENOSYS means not supported

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 6 Mar 1992
			use actual supplementary groups list instead of
			arbitrary groups.

		Geoff Clare, 23 Nov 1994
			Initialize testfail to zero in test4.
			Fix code in setup() that removes egid from list.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getgroups
#endif


#define 	NO_TESTS	4		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)
#define		NGROUPS_DUMMY	10

extern	char	*errname();

extern	int	vggid0, vggid1, vggid2;

int	no_tests = NO_TESTS;
private	int	setupok;

private	void	setup();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private	int	tbgetgroups();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private	gid_t	*init_grps, *grps;
private	gid_t	dumgrps[NGROUPS_DUMMY+1];

private	int	ngroups_max;
private	int	testfail = 0;


private void
setup()
{
	int	i, j;
	gid_t	egid;
	int	pathok = 0;

	DBUG_ENTER("setup");

	if ((ngroups_max = sysconf(_SC_NGROUPS_MAX)) < 0) 
	{
		for (i=1; i<=no_tests; i++)
			tet_delete(i,
				"sysconf(_SC_NGROUPS_MAX) failed in setup");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((egid=getegid()) == 0)
	{
		/* 0 value is used as a "not-set" flag in group lists */
		for (i=1; i<=no_tests; i++)
			tet_delete(i, "effective group ID must not be 0");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (ngroups_max > 0 && setprv(PRV_SETGRPS) != 0)
	{
		for (i=1; i<=no_tests; i++)
			tet_delete(i, "setprv(PRV_SETGRPS) failed in setup");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	init_grps = (gid_t *) malloc((size_t)((ngroups_max+2)*sizeof(gid_t)));
	grps = (gid_t *) malloc((size_t)((ngroups_max+2)*sizeof(gid_t)));
	if (init_grps == NULL || grps == NULL)
	{
		for (i=1; i<=no_tests; i++)
			tet_delete(i, "malloc() failed in setup");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(uids()!=0)
	{
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}
 	else
		PATH_TRACE;

	if (getgroups(ngroups_max+2, init_grps) != ngroups_max)
	{
		for (i=1; i<=no_tests; i++)
			tet_delete(i,
			    "getgroups() in setup did not return NGROUPS_MAX");
		DBUG_VOID_RETURN;
	}
 	else
		PATH_TRACE;

	for (i=0; i<ngroups_max; i++)
	{
		if (init_grps[i] == 0)
		{
			for (i=1; i<=no_tests; i++)
				tet_delete(i, "supplementary group IDs for user vsx0 must not include 0");
			DBUG_VOID_RETURN;
		}
		else if (init_grps[i] == egid)
		{
			/* setting a group list including the egid just
			   confuses matters, so remove it */
			for (j = i; j+1 < ngroups_max; j++)
				init_grps[j] = init_grps[j+1];
			ngroups_max--;
		}
	}
	init_grps[ngroups_max] = vggid1;
	init_grps[ngroups_max+1] = vggid2;

	PATH_SETUP_RPT(6);

	DBUG_VOID_RETURN;
}


private void
test1()
{
	int	i;
	int	pathok = setupok;

	DBUG_ENTER("test1");

	testfail = 0;

	if (ngroups_max == 0)
	{
		globok = 0;
		(void) tbgetgroups(NGROUPS_DUMMY, dumgrps, 0, NOERROR,
			(gid_t *)0, NGROUPS_DUMMY);

		PATH_FUNC_TRACE;

		if (testfail == 0)
			PATH_XS_RPT(1);

		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 1; i <= ngroups_max; i++)
	{
		errno = 0;
		if (setgrps(i, init_grps) != 0)
		{
			if (errno == ENOSYS)
			{
				xx_rpt(UNSUPPORTED);
				in_rpt("setgrps() returned -1 with errno ENOSYS");
			}
			else
			{
				xx_rpt(DELETION);
				in_rpt("setgrps(%d, ..) failed", i);
			}
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		(void) tbgetgroups(ngroups_max, grps, i, NOERROR, init_grps,
			ngroups_max);
		PATH_FUNC_TRACE;


		if (testfail > 0)
			break;
	}
	
	if (testfail == 0)
		PATH_XS_RPT(2*ngroups_max+1);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	i;
	int	pathok = setupok;

	DBUG_ENTER("test2");

	testfail = 0;

	if (ngroups_max == 0)
	{
		globok = 0;
		(void) tbgetgroups(0, dumgrps, 0, NOERROR, (gid_t *)0,
			NGROUPS_DUMMY);

		PATH_FUNC_TRACE;

		if (testfail == 0)
			PATH_XS_RPT(1);

		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i <= ngroups_max; i++)
	{
		errno = 0;
		if (setgrps(i, init_grps) != 0)
		{
			if (errno == ENOSYS)
			{
				xx_rpt(UNSUPPORTED);
				in_rpt("setgrps() returned -1 with errno ENOSYS");
			}
			else
			{
				xx_rpt(DELETION);
				in_rpt("setgrps(%d, ..) failed", i);
			}
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		(void) tbgetgroups(0, grps, i, NOERROR, (gid_t *)0,
			ngroups_max);

		PATH_FUNC_TRACE;

		if (i > 0 && testfail > 0)
			break;
	}

	if (testfail == 0)
		PATH_XS_RPT(2*ngroups_max+3);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int rval;
	int	pathok = setupok;

	DBUG_ENTER("test3");

	testfail = 0;

	errno = 0;
	if (ngroups_max > 0 && setgrps(ngroups_max, init_grps) != 0)
	{
		if (errno == ENOSYS)
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("setgrps() returned -1 with errno ENOSYS");
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("setgrps(%d, ..) failed", ngroups_max);
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	rval = tbgetgroups(ngroups_max+1, grps, ngroups_max, NOERROR, init_grps,
		ngroups_max+1);
	PATH_FUNC_TRACE;

	if (rval > ngroups_max)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("getgroups(%d, grouplist) returned %d > NGROUPS_MAX (%d)",
			ngroups_max+1, rval, ngroups_max);
	}
	else
		PATH_TRACE;
	
	if (testfail == 0)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test4()
{

	int	i;
	int	pathok = setupok;

	DBUG_ENTER("test4");

	testfail = 0;

	if (ngroups_max == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("NGROUPS_MAX is zero");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i <= ngroups_max; i++)
	{
		if (i == 1)
			continue;

		errno = 0;
		if (setgrps(i, init_grps) != 0)
		{
			if (errno == ENOSYS)
			{
				xx_rpt(UNSUPPORTED);
				in_rpt("setgrps() returned -1 with errno ENOSYS");
			}
			else
			{
				xx_rpt(DELETION);
				in_rpt("setgrps(%d, ..) failed", i);
			}
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		globok = 0;
		(void) tbgetgroups(i-1, grps, SYSERROR, EINVAL, (gid_t *)0,
			ngroups_max);
		PATH_FUNC_TRACE;

		if (i > 0 && testfail > 0)
			break;
	}

	if (testfail == 0)
		PATH_XS_RPT(2*ngroups_max+1);

	DBUG_VOID_RETURN;
}


#define BUFSIZE 1024

private char *
grp_prt(ngrps, glist)
int ngrps;
gid_t glist[];
{
	int	i;
	char	*end;
	static char	buf[BUFSIZE];

	end = buf;
	for (i = 0; i < ngrps; i++)
	{
		(void) sprintf(end, "%3d ", (int)glist[i]);
		end += strlen(end);
		if ((end - buf) > BUFSIZE-20)
		{
			er_rpt("need to increase static buffer size in grp_prt()");
			break;
		}
	}
	*--end = '\0';

	return buf;
}

private	int
tbgetgroups(gidsetsize, grouplist, expret, experr, expgrps, ngrps_chk)
int	gidsetsize;
gid_t	grouplist[];
int	expret;
int	experr;
gid_t	expgrps[];
int	ngrps_chk;
{
	int	rval, err, i, j, hits;
	gid_t	egid;
	int	pathok = 0;

	DBUG_ENTER("tbgetgroups");

	for (i = 0; i <= ngrps_chk; i++)
		grouplist[i] = 0;

	errno = 0;
	rval = getgroups(gidsetsize, grouplist);
	err = errno;

#ifdef	TESTBED
	rval = -1;
	errno++;
#endif

#ifdef TESTBED1
	if (rval > 0)
		grouplist[rval-1]++;
	grouplist[ngrps_chk-1]++;
#endif

#ifdef TESTBED2
	if (rval > 0)
		grouplist[rval++] = getegid();
#endif

	if (ngroups_max == 0 || gidsetsize == 0)
	{
		PATH_TRACE;
		if (rval != expret)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("getgroups(%d, grouplist) returned %d, expected 0",
				gidsetsize, rval);
			if (err != 0)
				in_rpt("errno was set to %d (%s)",
					err, errname(err));
		}
		else
		{
			PATH_TRACE;
			hits = 0;
			for (i = 0; i <= ngrps_chk; i++)
				if (grouplist[i] != 0)
					++hits;
			if (hits != 0)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("getgroups(%d, grouplist) altered %d entries in grouplist array",
					gidsetsize, hits);
			}
			else
				PATH_TRACE;
		}
	}
	else if (expret != SYSERROR)
	{
		if (rval != expret && rval != expret + 1)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("after setting up %d groups with setgrps()",
				expret);
			in_rpt("getgroups(%d, grouplist) returned %d, expected %d or %d",
				gidsetsize, rval, expret, expret+1);
			if (err != 0)
				in_rpt("errno was set to %d (%s)",
					err, errname(err));
		}
		else
		{
			PATH_TRACE;
			egid = getegid();
			hits = 0;
			for (i = 0; i < rval; i++)
			{
				if (grouplist[i] == egid)
					continue;
				for (j = 0; j < expret; j++)
					if (grouplist[i] == expgrps[j])
						++hits;
			}
			if (hits != expret)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("getgroups(%d, grouplist) returned incorrect values in grouplist",
					gidsetsize);
				in_rpt("expected: %s [%d]",
					grp_prt(expret, init_grps), (int)egid);
				in_rpt("observed: %s",
					grp_prt(rval, grouplist));
			}
			else
				PATH_TRACE;

			hits = 0;
			for (i = gidsetsize; i <= ngrps_chk; i++)
				if (grouplist[i] != 0)
					++hits;
			if (hits != 0)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("getgroups(%d, grouplist) altered %d entries at grouplist[%d] or above",
					gidsetsize, hits, gidsetsize);
			}
			else
				PATH_TRACE;
		}
	}
	else
	{
		PATH_TRACE;
		if (rval != SYSERROR || err != experr)
		{
			if (testfail++ == 0)
				xx_rpt(FAILURE);
			in_rpt("getgroups(%d, grouplist) did not give correct results",
				gidsetsize);
			in_rpt("expected: return value %d, errno %d (%s)",
				expret, experr, errname(experr));
			in_rpt("observed: return value %d, errno %d (%s)",
				rval, err, errname(err));
		}
		else
		{
			PATH_TRACE;
			hits = 0;
			for (i = 0; i <= ngrps_chk; i++)
				if (grouplist[i] != 0)
					++hits;
			if (hits != 0)
			{
				if (testfail++ == 0)
					xx_rpt(FAILURE);
				in_rpt("getgroups(%d, grouplist) altered %d entries in grouplist array",
					gidsetsize, hits);
			}
			else
				PATH_TRACE;
		}
	}

	PATH_FUNC_RPT(3);

	DBUG_RETURN(rval);
}
