/*
*      SCCS:  @(#)  POSIX.os/procenv/getlogin/getlogin.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/getlogin - T.getlogin Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.getlogin

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
		Testset for the getlogin() system call.

METHODS:
ASSUMES:
		For this testset to work the controlling terminal must
		still be the same as the original login session from
		which vtest was run.  (There used to be a setsid() in
		vtest which has now been taken out for this reason).

		It may also fail if stdin, stdout and stderr are all
		redirected (e.g. if vtest was run from "nohup" in the
		background).

CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	29 June 1989
MODIFICATIONS:	
		Geoff Clare, 3 Aug 1989
			change to PRV_SETID process in order to test
			with real & eff. UIDs different to login user.

		J.A.Nave	11/07/90
		Added resilience testing

		David Sawyer
		Wed Aug 08 11:15:57 BST 1990
		Updated resilience code.
		
		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 23 Mar 1992
			Removed globok definition.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<sys/types.h>
#include	<stdlib.h>
#include	<unistd.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getlogin
#endif


#define 	NO_TESTS	1		/* number of tests in testset */

extern	char	*errname();
extern	int	vguid1;

int	no_tests = NO_TESTS;

private void	test1();
private void	tbgetlogin();

private int	testfail;

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private void
test1()
{
	char	*user;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	user = tet_getvar("VSX_TTYUSER");
	if (user == (char *) NULL || *user == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_TTYUSER is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Case 1:  real uid != effective */

	if (geteuid() == getuid())
	{
		xx_rpt(DELETION);
		in_rpt("real user ID and effective user ID are equal");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	tbgetlogin(user);
	PATH_FUNC_TRACE;

	/* Case 2:  real uid == effective ( != login user ) */

	if (setprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) setuid((uid_t)vguid1);

	if (geteuid() != vguid1 || getuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setuid(vguid1) did not set real & effective user IDs to vguid1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	tbgetlogin(user);
	PATH_FUNC_TRACE;

	if (testfail == 0)
		PATH_XS_RPT(7);

	DBUG_VOID_RETURN;
}

private void
tbgetlogin(user)
char *user;
{
	char *rval;
	int err;
	int	pathok = 0;

	DBUG_ENTER("tbgetlogin");

	errno = 0;
	rval = getlogin();
	err = errno;

	if (rval == NULL)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("getlogin() returned failed - errno %d (%s)",
			err, errname(err));
		in_rpt("current real user ID: %d, effective user ID: %d",
			(int)getuid(), (int)geteuid());
	}
	else if (strcmp(rval, user) != 0)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("getlogin() returned \"%s\", expected \"%s\"",
			rval, user);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;

}
