/*
*      SCCS:  @(#)  POSIX.os/procenv/getpid/getpid.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/getpid - T.getpid Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		POSIX.os/procenv/getpid.c

DESCRIPTION:
		Implements the getpid test set as defined in the
		getpid functional and design specification documents.

AUTHOR:		Joseph Nave, UniSoft Ltd.;
DATE CREATED:	Fri Jan 13 16:00:42 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 3 July 1989
			Correct sending of signal from parent to child
		
		Geoff Clare, 27 Oct 1989
			pa_t1 was writing sizeof(int), but ch_t1 reads
			sizeof(pid_t)

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<synch.h>
#include	<sys/types.h>
#include	<errno.h>
#include	<signal.h>
#include	<unistd.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getpid
#endif


#define	NO_TESTS		1

#define READ			0	/* pipe file descriptors */
#define WRITE			1

#define SEND_TIME		60	/* time out values */
#define	WAITTIME	(5 * SPEEDFACTOR)

extern	void	(*vsignal())();

public	int	no_tests = NO_TESTS;
int	setupok;

private	void	test1();
private	void	prep_tests();

public	void	(*setfunc) () = prep_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1 , 1,
	NULL, 0
};

private	pid_t	tb_getpid();
private	void	pa_t1();
private void	ch_t1();

private synch	*pipe_1;
private int	caught_sig;

private	void
prep_tests()
{
	int	pathok = 0;

	DBUG_ENTER("prep_tests");

	/* open Parent -> Child pipe */

	if ((pipe_1 = opensync()) == (synch *) NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	DBUG_PRINTF("prep_tests", "pipe_1[READ] = %o, pipe_1[WRITE]= %o",
		pipe_1[0][READ], pipe_1[0][WRITE]);

	setupok = (pathok == 1) ? 0 : -1;

	DBUG_VOID_RETURN;
}

private void
sig_catch(sig)
int sig;
{
	caught_sig = sig;
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1 , pa_t1 , WAITTIME , 0);

	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	int	ret;
	pid_t	forkval = tet_child;
	
	DBUG_ENTER("pa_t1");

	/* Sleep a while to allow the child to initialise */
	(void) sleep(WAITTIME/2);

	/* send a signal to child and check that is is caught by the child */
	/* this is to verify fork() returned the correct value */
	(void) kill(forkval, SIGUSR1); 
	
	/* write to Child */
	ret = sendsync(pipe_1[0][WRITE], (char *) &forkval, sizeof forkval,
		SEND_TIME);

	if (ret == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sendsync() failed in pa_t1()");
	}
	globok = 1;

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	pid_t	gp, forkval;
	int	ret;
	int	pathok = setupok;

	DBUG_ENTER("ch_t1");

	(void) vsignal(SIGUSR1, sig_catch);

	/* pause for signal */
	caught_sig = 0;
	(void) sleep(WAITTIME);

	if (caught_sig != SIGUSR1)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: child did not receive SIGUSR1 from parent");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* read from Parent */
	ret = waitsync(pipe_1[0][READ], (char *) &forkval, sizeof forkval, WAITTIME);
	if (ret == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitsync() failed in ch_t1()");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((gp = tb_getpid()) != forkval)
	{
		xx_rpt(FAILURE);
		in_rpt("getpid() did not return process ID of calling process");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}
	
	DBUG_PRINTF("test1", "getpid() = %d", gp);

	DBUG_VOID_RETURN;
}


private	pid_t
tb_getpid()
{
	int	retval;

	retval = getpid();
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
