/*
*      SCCS:  @(#)  POSIX.os/procenv/getppid/getppid.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/getppid - T.getppid Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		POSIX.os/procenv/getppid.c

DESCRIPTION:
		Implements the getppid test set as defined in the
		getppid functional and design specification documents.

AUTHOR:		Joseph Nave, UniSoft Ltd.;
DATE CREATED:	Fri Jan 13 16:00:42 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 3 July 1989
			Remove unnecessary SIGALRM sent from child to parent

		J.A.Nave	12/07/90
		Added resilience testing

************************************************************************/

#include	<std.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<synch.h>
#include	<sys/types.h>
#include	<signal.h>
#include	<errno.h>
#include	<unistd.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getppid
#endif


#define	NO_TESTS		1

#define READ			0	/* pipe file descriptors */
#define WRITE			1

#define SEND_TIME		60	/* time out values */
#define	WAITTIME	(5 * SPEEDFACTOR)

public	int	no_tests = NO_TESTS;
private	int	setupok;

private	void	prep_tests();
private	void	test1();

public	void	(*setfunc) () = prep_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1 , 1,
	NULL, 0
};

private	pid_t	tb_getppid();
private	void	pa_t1();
private void	ch_t1();

private synch	*pipe_1;

private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1 , pa_t1 , WAITTIME , 0);

	DBUG_VOID_RETURN;
}

private	void
prep_tests()
{
	int	pathok = 0;

	DBUG_ENTER("prep_tests");

	/* open Parent -> Child pipe */

	if ((pipe_1 = opensync()) == (synch *) NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	DBUG_PRINTF("prep_tests", "pipe_1[READ] = %o, pipe_1[WRITE]= %o",
		pipe_1[0][READ], pipe_1[0][WRITE]);

	setupok = (pathok == 1) ? 0 : -1;

	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	pid_t	thispid;
	int	ret;
	
	DBUG_ENTER("pa_t1");
	
	/* write to Child */
	thispid = getpid();
	DBUG_PRINTF("pa_t1", "thispid = %d", thispid);
	ret = sendsync(pipe_1[0][WRITE], (char *) &thispid, sizeof(pid_t), SEND_TIME);

	if (ret == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sendsync() failed in pa_t1()");
		DBUG_VOID_RETURN;
	}
	globok = 1;
	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	ret;
	pid_t	forkval, ppid;
	int	pathok = setupok;

	DBUG_ENTER("ch_t1");

	ret = waitsync(pipe_1[0][READ], (char *) &forkval, sizeof(pid_t), WAITTIME);

	if (ret == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitsync() failed in ch_t1()");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ppid = tb_getppid();

	if (forkval != ppid)
	{
		xx_rpt(FAILURE);
		in_rpt("getppid() did not return the parent process ID of");
		in_rpt("   the calling process");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_PRINTF("ch_t1", "parent process id = %d, getppid_id = %d", forkval, ppid);

	DBUG_VOID_RETURN;
}

private	pid_t
tb_getppid()
{
	pid_t	retval;

	retval = getppid();
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
