/*
*      SCCS:  @(#)  POSIX.os/procenv/isatty/isatty.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/isatty - T.isatty Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.isatty

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the isatty() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Tue Feb 14 16:07:42 GMT 1989
MODIFICATIONS:	J.A.Nave	18/07/90
		Added resilience testing

		Peter Skan, 1 Oct 1991
			set the PRV_DEVICE privilege

		Geoff Clare, 10 March 1993
			Added #include <unistd.h>

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<limits.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<timeout.h>
#include	<string.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef isatty
#endif


#define NO_TESTS	2		/* number of tests in testset */
#define	INVALID_ID	-1		/* default invalid user id */
#define	WAITTIME	(5 * SPEEDFACTOR)

extern	char	*atocp();
extern	void	valrpt();
extern	char	*errname();


int	no_tests = NO_TESTS;

private	int	tb_isatty();
private void	test1();
private void	test2();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};


private void
test1()
{
	int	fd;
	char	*path;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	if ((path = tet_getvar("VSX_TTYNAME")) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("VSX_TTYNAME not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (setprv(PRV_DEVICE) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_DEVICE) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)
	errno = 0;
	fd = open(path, O_RDONLY);
	err = errno;
	CLEAR_ALARM
	if(fd == (int) SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("open(%s, O_RDONLY) has failed %d(%s)",path,err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if(tb_isatty(fd) != 1)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("isatty() did not return 1 : errno %d(%s)",err,errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}

	(void) close(fd);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	errors = 0;
	int	fd;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	errno = 0;
	if((fd = open("/dev/null",O_RDONLY)) == (int) SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(/dev/null,O_RDONLY) failed %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if(tb_isatty(fd) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("isatty() on /dev/null did not return 0");
		errors++;
	}
	else if (errno != ENOTTY)
	{
		err = errno;
		if (!errors)
			xx_rpt(WARNING);
		in_rpt("isatty() on /dev/null did not return ENOTTY in errno - instead got : errno %d(%s)",err,errname(err));
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private	int
tb_isatty(fd)
int	fd;
{
	int	rval;

	errno = 0;
	rval = isatty(fd);
#ifdef	TESTBED
	errno++;
	rval = -1;
#endif

	return(rval);
}
