/*
*      SCCS:  @(#)  POSIX.os/procenv/setgid/setgid.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/setgid - T.setgid Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.setgid

PROJECT:	VSX (X/OPEN Validation Suite)


MODULE CONTAINS:

SYNOPSIS:


DESCRIPTION:
	Testset for the setgid() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Mon Feb  6 11:09:51 GMT 1989
MODIFICATIONS:
		Geoff Clare, 19 June 1990
			Merge EPERM tests

		J.A.Nave	18/07/90
		Added resilience testing

		Stuart Boutell, 2 Nov 1990
			Changed execl for tet_exec

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ,
			Eliminated reason(). 

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 27 June 1991
			VSX_INVALID_GID="unsup" for unsupported result

		Geoff Clare, 4 Mar 1992
			Change SAVED_IDS check in test 4.

		Neil Moses, 19 May 1992
			Check gid and egid are not changed in deviance tests.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<report.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef setgid
#endif


#define	NO_TESTS	6		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)

extern  char    *newstr();
extern	char	*errname();
extern	void	valrpt();
extern	char 	*itoa();
extern	int 	uids();
extern	int	vggid0;
extern	int	vggid1;
extern 	int	vggid2;
extern	int	vguid0;
extern	int	vguid1;
extern	int 	vguid2;

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;

int	no_tests = NO_TESTS;

private char 	*setgid_sg1 = "../setgid_sg1";
private char 	*setgid_t01 = "../setgid_t01";
private char 	*setgid_t03 = "../setgid_t03";
private void	setup();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	test6();
private	void	ch_t1();
private	void	ch_sg1();
private	void	ch_t3();
private	int	tb_setgid();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	NULL, 0
};

private void
setup()
{
	DBUG_ENTER("setup");

	if(uids()!=0)
	{
		int	i;
		for(i=1; i <= no_tests; i++)
		{
			(void) tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}


private void
test1()
{

	DBUG_ENTER("test1");

	(void) cppair(ch_t1 , NULLFN , WAITTIME , 0);

	DBUG_VOID_RETURN;

}

private	void
ch_t1()
{
	char	*args[8];
	char 	*gid1str,	*gid2str,	*uid1str,	*pathok_str;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	gid1str = newstr(itoa(vggid1));
	gid2str = newstr(itoa(vggid2));
	uid1str = newstr(itoa(vguid1));
	if (gid1str == NULL || gid2str == NULL || uid1str == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pathok_str = newstr(itoa(pathok));

	/* note vg_dbug must be last active parameter */
	args[0] = setgid_t01;
	args[1] = gid1str ;
	args[2] = gid2str ;
	args[3] = uid1str ;
	args[4] = vg_ident;
	args[5] = pathok_str;
	args[6] = vg_dbug ;
	args[7] = (char *) NULL;
	(void) tet_exec(setgid_t01,args,environ);

	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed", setgid_t01);
	DBUG_VOID_RETURN;

}

private void
test2()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	errno = 0;
	if (sysconf(_SC_SAVED_IDS) == SYSERROR)
	{
		if (errno != 0)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("sysconf(_SC_SAVED_IDS) failed - errno %d (%s)",
				err, errname(err));
		}
		else
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("_POSIX_SAVED_IDS not implemented");
		}
	}
	else
	{
		PATH_TRACE;
		PATH_FUNC_RPT(1);
		(void) cppair(ch_sg1 , NULLFN , WAITTIME , 0);
	}

	DBUG_VOID_RETURN;
}

private	void
ch_sg1()
{
	char	*args[8];
	char	*tnum;
	char 	*gid0str;	
	char 	*gid1str;
	int	pathok = 0;
	char	*pathok_str;

	DBUG_ENTER("ch_sg1");

	PATH_FUNC_TRACE;

	tnum = newstr(itoa(tet_thistest));
	gid0str = newstr(itoa(vggid0));
	gid1str = newstr(itoa(vggid1));
	if (tnum == NULL || gid0str == NULL || gid1str == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pathok_str = newstr(itoa(pathok));

	/* note that vg_dbug must be the last active parameter */
	args[0] = setgid_sg1;
	args[1] = tnum;
	args[2] = gid0str;
	args[3] = gid1str;
	args[4] = vg_ident; 
	args[5] = pathok_str;
	args[6] = vg_dbug;
	args[7] = (char *) NULL;

	(void) tet_exec(setgid_sg1, args, environ);
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed", setgid_sg1);
	DBUG_VOID_RETURN;
}


private void
test3()
{

	DBUG_ENTER("test3");

	(void) cppair(ch_t3 , NULLFN , WAITTIME , 0);

	DBUG_VOID_RETURN;

}

private	void
ch_t3()
{

	char	*args[7];
	char 	*gid1str,	*gid2str;
	int	pathok = 0;
	char	*pathok_str;

	DBUG_ENTER("ch_t3");

	gid1str = newstr(itoa(vggid1));
	gid2str = newstr(itoa(vggid2));
	if (gid1str == NULL || gid2str == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pathok_str = newstr(itoa(pathok));

	/* note that vg_dbug must be the last active parameter */
	args[0] = setgid_t03;
	args[1] = gid1str;
	args[2] = gid2str;
	args[3] = pathok_str;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = (char *) NULL;

	(void) tet_exec(setgid_t03, args, environ );
	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed", setgid_t03);
	DBUG_VOID_RETURN;
}

private void
test4()
{
	int err;
	int	pathok = 0;

	DBUG_ENTER("test4");

	errno = 0;
	if (sysconf(_SC_SAVED_IDS) == SYSERROR)
	{
		if (errno != 0)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("sysconf(_SC_SAVED_IDS) failed - errno %d (%s)",
				err, errname(err));
		}
		else
		{
			PATH_TRACE;
			PATH_FUNC_RPT(1);
			(void) cppair(ch_sg1 , NULLFN , WAITTIME , 0);
		}
	}
	else
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_SAVED_IDS is implemented");
	}

	DBUG_VOID_RETURN;
}

private void
test5()
{
	char	*cp;
	int	rval;
	gid_t	invalgid;
	gid_t	s_gid, s_egid, c_gid, c_egid;
	int	nfails, err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	nfails = 0;

	if ((cp = tet_getvar("VSX_INVALID_GID")) == NULL || *cp == '\0')
	{
		if(!nfails)
			xx_rpt(DELETION);
		in_rpt("parameter VSX_INVALID_GID is not set");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(cp, "unsup", (size_t)5) == 0)
	{
		if(!nfails)
			xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_INVALID_GID is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	invalgid = atoi(cp);

	s_gid = getgid();
	s_egid = getegid();

	errno = 0;
	if (((rval = tb_setgid( invalgid )) != SYSERROR) ||
		(errno != EINVAL))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setgid(%d) did not fail as expected", (int)invalgid);
		valrpt(SYSERROR, rval, EINVAL, err);
	}
	else
		PATH_TRACE;

	c_gid = getgid();
	c_egid = getegid();

	if(s_gid != c_gid || s_egid != c_egid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		if(s_gid != c_gid)
		{
			in_rpt("setgid(%d) on failure changed the group id", (int)invalgid);
			in_rpt("of the calling process");
		}
		if(s_egid != c_egid)
		{
			in_rpt("setgid(%d) on failure changed the effective group id", (int)invalgid);
			in_rpt("of the calling process");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!nfails)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


private void
test6()
{
	int	rval;
	gid_t	s_gid, s_egid, c_gid, c_egid;
	int	nfails, err;
	int	pathok = 0;

	DBUG_ENTER("test6");

	nfails = 0;

	if (setprv(PRV_SETID) == 0)
	{
		if(!nfails)
			xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_SETID) succeeded");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	if (getegid() == vggid2 || getgid() == vggid2)
	{
		if(!nfails)
			xx_rpt(DELETION);
		in_rpt("deletion reason: real or effective GID is VSX_GID2");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	s_gid = getgid();
	s_egid = getegid();

	errno = 0;
	if (((rval = tb_setgid( (gid_t) vggid2 )) != SYSERROR) ||
		(errno != EPERM))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setgid(VSX_GID2) did not fail as expected");
		valrpt(SYSERROR, rval, EPERM, err);
		DBUG_VOID_RETURN;
	} 
	else
		PATH_TRACE;

	c_gid = getgid();
	c_egid = getegid();

	if(s_gid != c_gid || s_egid != c_egid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		if(s_gid != c_gid)
		{
			in_rpt("setgid(%d) on failure changed the group id", vggid2);
			in_rpt("of the calling process");
		}
		if(s_egid != c_egid)
		{
			in_rpt("setgid(%d) on failure changed the effective group id", vggid2);
			in_rpt("of the calling process");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if (sysconf(_SC_SAVED_IDS) == SYSERROR)
	{
		if (errno != 0)
		{
			err = errno;
			if(!nfails)
				xx_rpt(DELETION);
			in_rpt("sysconf(_SC_SAVED_IDS) failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		PATH_FUNC_RPT(4);

		(void) cppair(ch_sg1, NULLFN, WAITTIME, 0);
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}


private	int
tb_setgid(gid)
gid_t	gid;
{
	int	retval;

	retval = setgid(gid);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
