/*
*      SCCS:  @(#)  POSIX.os/procenv/setgid/setgid_sg1.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/setgid - setgid_sg1 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		setgid_sg1 - used by T.setgid for SGID-vsxg1 tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.setgid in tests requiring an effective gid of vsxg1.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Fri Feb  3 13:26:14 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 18 Oct 1989
			Check for NULL return from calloc()
		
		Geoff Clare, 19 June 1990
			Merge all SGID vsxg1 programs into one

		J.A.Nave	18/07/90
		Added resilience testing

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Geoff Clare, 4 Mar 1992
			Allow for effective group ID to appear in
			supplementary groups list.

		Geoff Clare, 23 Mar 1992
			Removed globok definition.

		Neil Moses, 18 May 1992
			Check gid and egid are not changed in deviance tests.

		Geoff Clare, 24 Sept 1992
			Fixed path tracing problem in same_groups().

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef setgid
#endif



extern	char	**environ;
extern	char	*errname();
extern	void	set_pname();
extern	void	valrpt();

extern	int	vggid0;
extern	int	vggid1;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	void	subtest2();
private	void	subtest4();
private	void	subtest6();
private	int	tb_setgid();
private	gid_t	*mkgroup();
private	int	same_groups();

public	int
tet_main(argc, argv)
int	argc;
char	**argv;
{
 
	int	tnum;
	int	pathok = 0;

	set_pname("setgid_sg1");

	switch(argc)
	{
	case 7:
		vg_dbug = argv[6];
		/* fall through */
	case 6:
		if (argv[5] != (char *) NULL)
			pathok = atoi(argv[5]);
		vg_ident =  argv[4];
		break;
	default:
		in_rpt("argument count wrong");
		exit(OK);
	}

	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	tnum = atoi(argv[1]);
	vggid0 = atoi(argv[2]);
	vggid1 = atoi(argv[3]);
	if (getegid() != vggid1)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: effective GID NOT= VSX_GID1");
		exit(OK);
	}

	switch(tnum)
	{
	case 2:
		subtest2(pathok);
		break;

	case 4:
		subtest4(pathok);
		break;

	case 6:
		/* this test does not seem to be called ? */
		subtest6();
		break;

	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed to setgid_sg1: %d", tnum);
	}

	exit(OK);

	/* NOTREACHED */
}


private void
subtest2(pathok)
int pathok;
{
	gid_t	gid, egid, start_egid, curr_egid;
	gid_t	*groups_start;
	gid_t	*groups_curr;
	int	max_groups;
	int	start_size;
	int	curr_size;
	int	err;

	DBUG_ENTER("subtest2");

	if ((max_groups = sysconf(_SC_NGROUPS_MAX)) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sysconf(_SC_NGROUPS_MAX) failed");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	if (max_groups > 0)
	{
		globok = 0;
		groups_start = mkgroup(max_groups);
		if (groups_start == NULL)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		PATH_FUNC_TRACE;

		errno = 0;
		if ((start_size = getgroups(max_groups , groups_start)) == SYSERROR)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("Getgroups call failed with errno = %d (%s)", 
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		start_egid = getegid();
	}
	else
		PATH_TRACE;
	
	DBUG_PRINTF("subtest2", "Real gid = %d", getgid());
	DBUG_PRINTF("subtest2", "Effective gid = %d", getegid());
	DBUG_PRINTF("subtest2", "vggid0 = %d", vggid0);
	DBUG_PRINTF("subtest2", "vggid1 = %d", vggid1);
	/*
	 * Attempt to set effective group ID to real group ID
	 * by calling setgid(real-group-ID).
	 */
	errno = 0;
	if (tb_setgid( (gid_t) vggid0 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setgid call no1 failed with errno = %d (%s)", 
			err, errname(err));
		in_rpt("when attempting to set the effective group ID to the real group ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check real group ID unaltered */
	gid = getgid();
	if (gid != vggid0)
	{
		xx_rpt(FAILURE);
		in_rpt("GID (%d) not equal to VSX_GID0 (%d)", gid, vggid0);
		in_rpt("when attempting to set the effective group ID to the real group ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check effective group ID changed to "gid" */
	egid = getegid();
	if (egid != vggid0)
	{
		xx_rpt(FAILURE);
		in_rpt("GID (%d) not equal to effective VSX_GID0 (%d)",
			egid, vggid0);
		in_rpt("when attempting to set the effective group ID to the real group ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (max_groups > 0)
	{
		errno = 0;
		globok = 0;
		groups_curr = mkgroup(max_groups);
		if (groups_curr == NULL)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		PATH_FUNC_TRACE;
	
		errno = 0;
		if ((curr_size = getgroups(max_groups , groups_curr)) == SYSERROR)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("Getgroups call failed with errno = %d (%s)", 
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		curr_egid = getegid();
	
		globok = 0;
		if (same_groups(start_size, groups_start, curr_size,
			groups_curr, start_egid, curr_egid) == SYSERROR)
		{
			xx_rpt(FAILURE);
			in_rpt("List of supplementry groups has changed");
			in_rpt("when attempting to set the effective group ID to the real group ID");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		PATH_FUNC_TRACE;

	}
	else
		PATH_TRACE;

	/*
	 * Attempt to set effective group ID to saved set group ID
	 * by calling setgid(saved-set-group-ID).
	 */
	errno = 0;
	if (tb_setgid( (gid_t) vggid1 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setgid call no2 failed with errno = %d (%s)", 
			err, errname(err));
		in_rpt("when attempting to set the effective group ID to the saved set group ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check that real group ID is unchanged */
	gid = getgid();
	if (gid != vggid0)
	{
		xx_rpt(FAILURE);
		in_rpt("GID (%d) not equal to VSX_GID0 (%d)", gid, vggid0);
		in_rpt("when attempting to set the effective group ID to the saved set group ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check that effective group ID is changed to "gid" */
	egid = getegid();
	if (egid != vggid1)
	{
		xx_rpt(FAILURE);
		in_rpt("GID (%d) not equal to effective VSX_GID1 (%d)",
			egid, vggid1);
		in_rpt("when attempting to set the effective group ID to the saved set group ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (max_groups > 0)
	{
		errno = 0;
		if ((curr_size = getgroups(max_groups , groups_curr)) == SYSERROR)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("Getgroups call failed with errno = %d (%s)", 
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		curr_egid = getegid();
	
		globok = 0;
		if (same_groups(start_size, groups_start, curr_size,
			groups_curr, start_egid, curr_egid) == SYSERROR)
		{
			xx_rpt(FAILURE);
			in_rpt("List of supplementry groups has changed");
			in_rpt("when attempting to set the effective group ID to the saved set group ID");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		PATH_FUNC_TRACE;

	}
	else
		PATH_TRACE;

	/*
	 * Call setgid(saved-set-group-ID) and expect no change to real,
	 * effective or saved set group ID - because effective group ID 
	 * is already vggid1.
	 */
	errno = 0;
	if (tb_setgid( (gid_t) vggid1 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setgid call no3 failed with errno = %d (%s)", 
			err, errname(err));
		in_rpt("when attempting to set the effective group ID to the saved set group ID a second time");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	gid = getgid();
	if (gid != vggid0)
	{
		xx_rpt(FAILURE);
		in_rpt("GID (%d) not equal to VSX_GID0 (%d)", gid, vggid0);
		in_rpt("when attempting to set the effective group ID to the saved set group ID a second time");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	egid = getegid();
	if (egid != vggid1)
	{
		xx_rpt(FAILURE);
		in_rpt("GID (%d) not equal to effective VSX_GID1 (%d)",
			egid, vggid1);
		in_rpt("when attempting to set the effective group ID to the saved set group ID a second time");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (max_groups > 0)
	{
		errno = 0;
		if ((curr_size = getgroups(max_groups , groups_curr)) == SYSERROR)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("Getgroups call failed with errno = %d (%s)", 
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		curr_egid = getegid();
	
		globok = 0;
		if (same_groups(start_size, groups_start, curr_size,
			groups_curr, start_egid, curr_egid) == SYSERROR)
		{
			xx_rpt(FAILURE);
			in_rpt("List of supplementry groups has changed");
			in_rpt("when attempting to set the effective group ID to the saved set group ID a second time");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		PATH_FUNC_TRACE;

	}
	else
		PATH_TRACE;

	errno = 0;
	if (tb_setgid( (gid_t) vggid0 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setgid call no4 failed with errno = %d (%s)", 
			err, errname(err));
		in_rpt("when attempting to set the effective group ID to the real group ID a second time");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if (tb_setgid( (gid_t) vggid1 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setgid call no5 failed with errno = %d (%s)", 
			err, errname(err));
		in_rpt("when attempting to set the effective group ID to the saved set group ID a second time");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(max_groups>0 ? 28 : 18);

	DBUG_VOID_RETURN;
}

private void
subtest4(pathok)
int pathok;
{
	gid_t	gid, egid, start_egid, curr_egid;
	gid_t	*groups_start;
	gid_t	*groups_curr;
	int	max_groups;
	int	start_size;
	int	curr_size;
	int	err;

	DBUG_ENTER("subtest4");

	if ((max_groups = sysconf(_SC_NGROUPS_MAX)) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sysconf(_SC_NGROUPS_MAX) failed");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	if (max_groups > 0)
	{
		errno = NOERROR;
		globok = 0;
		groups_start = mkgroup(max_groups);
		if (groups_start == NULL)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		PATH_FUNC_TRACE;
	
		errno = NOERROR;
		if ((start_size = getgroups(max_groups , groups_start)) == SYSERROR)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("Getgroups call failed with errno = %d (%s)", 
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		start_egid = getegid();
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (tb_setgid( (gid_t) vggid0 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setgid call failed with errno = %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	gid = getgid();
	if (gid != vggid0)
	{
		xx_rpt(FAILURE);
		in_rpt("Real group ID has not changed");
		in_rpt("GID (%d) not equal to VSX_GID0 (%d)", gid, vggid0);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	egid = getegid();
	if (egid != vggid0)
	{
		xx_rpt(FAILURE);
		in_rpt("Effective group ID has not changed");
		in_rpt("GID (%d) not equal to effective VSX_GID1 (%d)",
			egid, vggid0);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (max_groups > 0)
	{
		errno = NOERROR;
		globok = 0;
		groups_curr = mkgroup(max_groups);
		if (groups_curr == NULL)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		PATH_FUNC_TRACE;
	
		errno = NOERROR;
		if ((curr_size = getgroups(max_groups , groups_curr)) == SYSERROR)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("Getgroups call failed with errno = %d (%s)", 
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		curr_egid = getegid();
	
		globok = 0;
		if (same_groups(start_size, groups_start, curr_size,
			groups_curr, start_egid, curr_egid) == SYSERROR)
		{
			xx_rpt(FAILURE);
			in_rpt("List of supplementary groups has changed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		PATH_FUNC_TRACE;

	}
	else
		PATH_TRACE;

	PATH_XS_RPT(max_groups>0 ? 14 : 8);

	DBUG_VOID_RETURN;
}

private void
subtest6()
{
	gid_t	egid;
	gid_t	s_gid, s_egid, c_gid, c_egid;
	int	nfails, rval, err;
	int	pathok = 0;

	DBUG_ENTER("subtest6");

	nfails = 0;

	errno = 0;
	if (tb_setgid( (gid_t) vggid0 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setgid(VSX_GID0) failed with errno = %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	egid = getegid();
	if (egid != vggid0)
	{
		xx_rpt(DELETION);
		in_rpt("Effective group ID has not changed");
		in_rpt("GID (%d) not equal to VSX_GID0 (%d)", egid, vggid0);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* If _POSIX_SAVED_IDS was supported the saved id would now
	   be vg_gid1 */
	
	s_gid = getgid();
	s_egid = getegid();

	errno = 0;
	if (((rval = tb_setgid( (gid_t) vggid1 )) != SYSERROR) ||
		(errno != EPERM))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setgid(VSX_GID1) did not fail as expected");
		valrpt(SYSERROR, rval, EPERM, err);
		DBUG_VOID_RETURN;
	} 
	else
		PATH_TRACE;

	c_gid = getgid();
	c_egid = getegid();

	if(s_gid != c_gid || s_egid != c_egid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		if(s_gid != c_gid)
		{
			in_rpt("setgid(%d) on failure changed the user id", vggid1);
			in_rpt("of the calling process");
		}
		if(s_egid != c_egid)
		{
			in_rpt("setgid(%d) on failure changed the effective user id", vggid1);
			in_rpt("of the calling process");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(nfails++)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


private	int
tb_setgid(gid)
gid_t	gid;
{
	int	retval;

	retval = setgid(gid);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}

private	int
same_groups(size1, group1, size2, group2, egid1, egid2)
int	size1;
gid_t	group1[];
int	size2;
gid_t	group2[];
gid_t	egid1;
gid_t	egid2;
{
	int	i, j;
	int	pathok = 0;

	DBUG_ENTER("same_groups");

	if (size1 != size2)
	{
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	for (i=0 ; i<size1 ; i++)
	{
		if (group1[i] == egid1)
		{
			PATH_TRACE;
			continue;	/* don't check effective group ID */
		}

		for (j=0 ; j<size2 ; j++)
		{
			if (j == 0)
				PATH_TRACE;
			if (group2[j] == egid2)
				break;	/* don't check effective group ID */
			if (group1[i] == group2[j])
				break;	/* match found check next group1[i] */
		}
		if ( j>=size2 )	/* no match found so return error condition */
		{
			DBUG_RETURN(SYSERROR);
		}
	}

	PATH_FUNC_RPT(1+size1);

	DBUG_RETURN(0);	
}

private	gid_t *
mkgroup(n)
int	n;
{
	gid_t *ret;
	int err;
	int	pathok = 0;

	ret = (gid_t *) calloc((size_t) n , sizeof(gid_t));
	if (ret == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("calloc() failed with errno = %d (%s)", 
			err, errname(err));
	}
	else
		PATH_TRACE;


	PATH_FUNC_RPT(1);

	return ret;
}
