/*
*      SCCS:  @(#)  POSIX.os/procenv/setgid/setgid_t03.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/setgid - setgid_t03 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		setgid_t03 - used by T.setgid in comply()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.setgid to test setgid() by setting
	  the effective UID to the real UID and back again, and
	  by setting the effective GID to the real GID.

  	  Setuid_ou should be a owned by a different user/group to
	  the testset process and should have its set-user-ID and
	  set-group-ID bits set.

	  Usage:
		


METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Fri Feb  3 13:15:36 GMT 1989
MODIFICATIONS:	J.A.Nave	18/07/90
		Added resilience testing

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env and envp.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<signal.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<setjmp.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef setgid
#endif


#define	NO_TESTS	7		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();
extern	void	set_pname();

extern	int	vggid1;
extern	int	vggid2;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	void	subtest3();
private	int	tb_setgid();

public	int
tet_main(argc, argv)		/* Assertion 12 */

	int	argc;
	char	**argv;
{

	set_pname(argc < 1 || argv[1] == (char *) NULL ?
		"setgid_t03" : (argv[1]));

	switch(argc)
	{
	case 6:
		vg_dbug = argv[5];
		/* fall through */
	case 5:
		vg_ident = argv[4];
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("argument count wrong");
		exit(OK);
	}

	if ( vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	vggid1 = atoi(argv[1]);
	vggid2 = atoi(argv[2]);

	subtest3( atoi(argv[3]));

	exit(OK);

	/* NOTREACHED */
}



private void
subtest3(pathchk)
int	pathchk;
{
	gid_t	gid;
	gid_t	egid;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("subtest3");

	if (pathchk == 1)
		PATH_TRACE;

	if (sysconf(_SC_SAVED_IDS) != SYSERROR)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_SAVED_IDS implemented");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (setprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tb_setgid( (gid_t) vggid1 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setgid call has failed with %d (%s)", err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (getegid() != vggid1)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: effective GID NOT= VSX_GID1");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (tb_setgid( (gid_t) vggid2 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setgid call failed with errno = %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	gid = getgid();
	if (gid != vggid2)
	{
		xx_rpt(FAILURE);
		in_rpt("Real group ID has not changed");
		in_rpt("GID (%d) not equal to VSX_GID2 (%d)", gid, vggid2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	egid = getegid();
	if (egid != vggid2)
	{
		xx_rpt(FAILURE);
		in_rpt("Effective group ID has not changed");
		in_rpt("GID (%d) not equal to effective VSX_GID2 (%d)",
			egid, vggid2);
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(8);
	}

	DBUG_VOID_RETURN;
}

private	int
tb_setgid(gid)
gid_t	gid;
{
	int	retval;

	retval = setgid(gid);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
