/*
*      SCCS:  @(#)  POSIX.os/procenv/setpgid/setpgid.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/setpgid - T.setpgid Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.setpgid

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the setpgid() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Wed Feb 15 14:43:33 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 30 June 1989
			Introduce VSX_SETPGID_SUPPORTED parameter
		
		Geoff Clare, 12 Sep 1989
			Do not expect EINTR from interrupted sleep()
		
		Geoff Clare, 20 Oct 1989
			Casts missing in some setpgid() args.

		Dave Cater, 31 Oct 1989
			Some FAILURE messages said "deletion reason: ....."
		
		Geoff Clare, 27 Feb 1990
			Don't assume sleep() sets a signal catcher for SIGALRM

		David Sawyer
		Wed Jul 25 11:35:50 BST 1990
		Added resilience code.

		Stuart Boutell, 2 Nov 1990
			execl call replaced with tet_exec call

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Geoff Clare, 27 June 1991
			VSX_SETPGID_SUPPORTED="N" for unsupported result.

		Neil Moses, 19 May 1992
			Check pgrp is not changed in deviance tests.
		
		Geoff Clare, 20 July 1992
			Fix many problems with use of pid_t.
		
		Geoff Clare, 3 Sept 1992
			Use atol() instead of atoi() to obtain unsup pgid.

		Geoff Clare, 20 Nov 1992
			Allow VSX_UNSUPPORTED_PGID="unsup".

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<timeout.h>
#include	<unistd.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef setpgid
#endif


#define 	NO_TESTS	13		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

#define		IFNOTSUPP \
	if (not_supported) \
	{ \
		xx_rpt(UNSUPPORTED); \
		in_rpt("parameter VSX_SETPGID_SUPPORTED is set to 'N'"); \
		DBUG_VOID_RETURN; \
	}

extern	char	*errname();
extern	int	cppair();
extern	void	(*vsignal())();
extern	char	*vg_ident;
extern	char	*vg_dbug;

int	no_tests = NO_TESTS;

private	int	tb_setpgid();
private	void	setup();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	pa_t6();
private void	pa_t10();
private void	pa_t12();
private void	pa_t13();
private void	ch_t1();
private void	ch_t2();
private void	ch_t3();
private void	ch_t4();
private void	ch_t5();
private void	ch_t6();
private void	ch_t7();
private void	ch_t8();
private void	ch_t9();
private void	ch_t10();
private void	ch_t11();
private void	ch_t12();
private void	ch_t13();
private void	gch1_t1();
private void	gch2_t1();
private void	gch1_t2();
private void	gch2_t2();
private void	gch1_t4();
private void	gch2_t4();
private void	gch1_t11();
private void	gch2_t11();
private void	gch1_t12();
private void	gch2_t12();

private int	not_supported;
private	pid_t	ch_t1_pid = 0;
private	pid_t	ch_t2_pid = 0;
private	pid_t	ch_t4_pid = 0;
private	pid_t	ch_t11_pid = 0;
private	pid_t	ch_t12_pid = 0;
private	pid_t	ch_t13_pid = 0;
private	char	*setpgid_t6	=	"../setpgid_t6";

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	NULL, 0
};

private	void
setup()
{
	char	*supp, *msg = NULL;
	int	i;

	DBUG_ENTER("setup");

	supp = tet_getvar("VSX_SETPGID_SUPPORTED");
	if (supp == NULL || *supp  == '\0')
	{
		for(i=1 ; i<=no_tests ; i++)
			(void) tet_delete(i,
				"parameter VSX_SETPGID_SUPPORTED is not set");
		DBUG_VOID_RETURN;
	}
	not_supported = (*supp == 'N' || *supp == 'n');

	errno = 0;
	if (sysconf(_SC_JOB_CONTROL) == SYSERROR)
	{
		if (errno != 0)
			msg = "sysconf(_SC_JOB_CONTROL) failed in setup";
	}
	else if (not_supported)
		msg = "when _POSIX_JOB_CONTROL is defined, parameter VSX_SETPGID_SUPPORTED must be set to 'Y'";

	if (msg != NULL)
	{
		for(i=1 ; i<=NO_TESTS ; i++)
			(void) tet_delete(i, msg);
	}
	else
		(void) setsid();

#ifdef TESTBED2
	not_supported = !not_supported;
#endif

	DBUG_VOID_RETURN;
}

private void
test1()
{
	DBUG_ENTER("test1");

	IFNOTSUPP

	(void) cppair(ch_t1, NULLFN, (4*WAITTIME), 0);

	DBUG_VOID_RETURN; 
} 

private void
ch_t1()
{

	DBUG_ENTER("ch_t1");

	(void) cppair(gch1_t1, gch2_t1, (3*WAITTIME), 0);

	DBUG_VOID_RETURN;
}

private void
gch1_t1() /* child */
{

	pid_t	act_gid;
	pid_t	exp_gid;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("gch1_t1");
	
	errno = NOERROR;
	if (tb_setpgid((pid_t)0, (pid_t)0) == SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setpgid(0, 0) failed %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((act_gid = getpgrp()) != (exp_gid = getpid()))
	{
		xx_rpt(FAILURE);
		in_rpt("setpgid(0, 0) has not changed the process group ID to the current pid: expected %ld - got %ld",
			(long)exp_gid, (long)act_gid);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) vsignal(SIGALRM, alrm);
	if (sleep(2*WAITTIME) == 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: child did not recieve signal from parent");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((act_gid = getpgrp()) != (exp_gid = getppid()))
	{
		xx_rpt(FAILURE);
		in_rpt("setpgid() has not changed the process group ID to the correct value: expected %ld - got %ld",
			(long)exp_gid, (long)act_gid);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}
	
	DBUG_VOID_RETURN;
}

private void
gch2_t1() /* parent */
{
	int	err;
	DBUG_ENTER("gch2_t1");
	
	globok = 1; /* cppair expects globok to be set if parent exists */

	ch_t1_pid = tet_child;

	(void) sleep(WAITTIME);

	errno = NOERROR;
	if (tb_setpgid((pid_t)0, (pid_t)0) == SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setpgid(0, 0) failed %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}

	errno = NOERROR;
	if (tb_setpgid((pid_t)ch_t1_pid, getpid()) == SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setpgid() failed %d(%s)", err, errname(err));
		in_rpt("setpgid(pid, pgid) with pid = %ld, pgid = %ld",
			(long)ch_t1_pid, (long)getpid());
		DBUG_VOID_RETURN;
	}

	(void) kill((pid_t)ch_t1_pid, SIGALRM);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	IFNOTSUPP

	(void) cppair(ch_t2, NULLFN, (4*WAITTIME), 0);

	DBUG_VOID_RETURN; 
} 

private void
ch_t2()
{

	DBUG_ENTER("ch_t2");

	(void) cppair(gch1_t2, gch2_t2, (3*WAITTIME), 0);

	DBUG_VOID_RETURN;
}

private void
gch1_t2() /* child */
{

	pid_t	act_gid;
	pid_t	exp_gid;
	int	pathok = 0;

	DBUG_ENTER("gch1_t2");
	
	(void) vsignal(SIGALRM, alrm);
	if (sleep(2*WAITTIME) == 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: child did not recieve signal from parent");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((act_gid = getpgrp()) != (exp_gid = getpid()))
	{
		xx_rpt(FAILURE);
		in_rpt("setpgid() has not changed the process group ID to the correct value: expected %ld - got %ld",
			(long)exp_gid, (long)act_gid);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}
	
	DBUG_VOID_RETURN;
}

private void
gch2_t2() /* parent */
{
	int	err;
	DBUG_ENTER("gch2_t2");

	globok = 1; /* cppair expects globok to be set if parent exists */

	(void) sleep(WAITTIME/2);

	ch_t2_pid = tet_child;
	errno = NOERROR;
	if (tb_setpgid((pid_t)0, (pid_t)0) == SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setpgid(0, 0) failed %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}

	errno = NOERROR;
	if (tb_setpgid((pid_t)ch_t2_pid, (pid_t)ch_t2_pid) == SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setpgid() failed %d(%s)", err, errname(err));
		in_rpt("setpgid(pid, pgid) with pid = %ld, pgid = %ld",
			(long)ch_t2_pid, (long)ch_t2_pid);
		DBUG_VOID_RETURN;
	}

	(void) kill((pid_t)ch_t2_pid, SIGALRM);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	IFNOTSUPP

	(void) cppair(ch_t3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN; 
} 

private void 
ch_t3() 
{

	pid_t	act_gid;
	pid_t	exp_gid;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");
	
	errno = NOERROR;
	if (tb_setpgid((pid_t)0, (pid_t)0) == SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setpgid(0, 0) failed %d(%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((act_gid = getpgrp()) != (exp_gid = getpid()))
	{
		xx_rpt(FAILURE);
		in_rpt("setpgid(0, 0) has not changed the process group ID to the correct value: expected %ld - got %ld",
			(long)exp_gid, (long)act_gid);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (tb_setpgid((pid_t)0, getppid()) == SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setpgid() failed %d(%s)", err, errname(err));
		in_rpt("setpgid(0, pgid) with pgid = %ld", (long)getppid());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((act_gid = getpgrp()) != (exp_gid = getppid()))
	{
		xx_rpt(FAILURE);
		in_rpt("setpgid(pid, pgid) has not changed the process group ID to the correct value: expected %ld - got %ld",
			(long)exp_gid, (long)act_gid);
	}
	else
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}


private void
test4()
{
	DBUG_ENTER("test4");

	IFNOTSUPP

	(void) cppair(ch_t4, NULLFN, (4*WAITTIME), 0);

	DBUG_VOID_RETURN; 
} 

private void
ch_t4()
{

	DBUG_ENTER("ch_t4");

	(void) cppair(gch1_t4, gch2_t4, (3*WAITTIME), 0);

	DBUG_VOID_RETURN;
}

private void
gch1_t4()
{

	pid_t	act_gid;
	pid_t	exp_gid;
	int	pathok = 0;

	DBUG_ENTER("gch1_t4");
	
	(void) vsignal(SIGALRM, alrm);
	if (sleep(2*WAITTIME) == 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: child did not recieve signal from parent");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((act_gid = getpgrp()) != (exp_gid = getpid()))
	{
		xx_rpt(FAILURE);
		in_rpt("setpgid() has not changed the process group ID to the correct value: expected %ld - got %ld",
			(long)exp_gid, (long)act_gid);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}
	
	DBUG_VOID_RETURN;
}

private void
gch2_t4() /* parent */
{
	int	err;
	DBUG_ENTER("gch2_t4");

	globok = 1; /* cppair expects globok to be set if parent exists */

	ch_t4_pid = tet_child;

	(void) sleep(WAITTIME/2);

	errno = NOERROR;
	if (tb_setpgid((pid_t)0, (pid_t)0) == SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setpgid(0, 0) failed %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}

	errno = NOERROR;
	if (tb_setpgid((pid_t)ch_t4_pid, (pid_t)0) == SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setpgid() failed %d(%s)", err, errname(err));
		in_rpt("setpgid(pid, 0) with pid = %ld", (long)ch_t4_pid);
		DBUG_VOID_RETURN;
	}

	(void) kill((pid_t)ch_t4_pid, SIGALRM);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	DBUG_ENTER("test5");

	IFNOTSUPP

	(void) cppair(ch_t5, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN; 
} 

private void 
ch_t5() 
{

	pid_t	act_gid;
	pid_t	exp_gid;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t5");
	
	errno = NOERROR;
	if (tb_setpgid((pid_t)0, (pid_t)0) == SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setpgid(0, 0) failed %d(%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((act_gid = getpgrp()) != (exp_gid = getpid()))
	{
		xx_rpt(FAILURE);
		in_rpt("setpgid(0, 0) has not changed the process group ID to the correct value: expected %ld - got %ld",
			(long)exp_gid, (long)act_gid);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private void
test6()
{
	DBUG_ENTER("test6");

	IFNOTSUPP

	(void) cppair(ch_t6, pa_t6, (3*WAITTIME), 0);

	DBUG_VOID_RETURN; 
} 

private void
ch_t6()
{
	extern	char	**environ;
	char	*args[4];
	int err;

	DBUG_ENTER("ch_t6");
	args[0] = setpgid_t6;
	args[1] = vg_ident;
	args[2] = vg_dbug;
	args[3] = (char *)NULL;

	(void) tet_exec(setpgid_t6, args, environ);
	
	err = errno;
	xx_rpt(DELETION);
	in_rpt("deletion reason : tet_exec(%s) failed - errno %d (%s)",
		setpgid_t6, err, errname(err));

	DBUG_VOID_RETURN;
}

private void
pa_t6()
{

	int	rval;
	int	nfails, err;
	pid_t	s_pgid, c_pgid;
	int	pathok = 0;

	DBUG_ENTER("pa_t6");

	nfails = 0;

	(void) sleep(WAITTIME);

	s_pgid = getpgrp();

	errno = 0;
	rval = tb_setpgid(tet_child, (pid_t)0);
	if ((rval != SYSERROR) || (errno != EACCES))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid(pid, 0) with pid = %ld, did not fail as expected with errno = EACCES",
			(long)tet_child);
		in_rpt("setpgid() returned %d with errno = %d(%s)",
			rval, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	c_pgid = getpgrp();

	if(s_pgid != c_pgid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid() on failure changed the process group");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!nfails)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	DBUG_ENTER("test7");

	IFNOTSUPP

	(void) cppair(ch_t7, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN; 
} 

private void
ch_t7()
{

	int	rval;
	int	nfails, err;
	pid_t	s_pgid, c_pgid;
	int	pathok = 0;

	DBUG_ENTER("ch_t7");

	nfails = 0;

	s_pgid = getpgrp();

	errno = 0;
	rval = tb_setpgid((pid_t)0, (pid_t)-1);
	if ((rval != SYSERROR) || (errno != EINVAL))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid(0, -1) did not fail as expected with errno = EINVAL");
		in_rpt("setpgid() returned %d with errno = %d(%s)",
			rval, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	c_pgid = getpgrp();

	if(s_pgid != c_pgid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid() on failure changed the process group");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!nfails)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	DBUG_ENTER("test8");

	IFNOTSUPP

	(void) cppair(ch_t8, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN; 
} 

private void
ch_t8()
{

	char	*cp;
	int	rval;
	pid_t	pgid;
	int	nfails, err;
	pid_t	s_pgid, c_pgid;
	int	pathok = 0;

	DBUG_ENTER("ch_t8");

	nfails = 0;

	if ((cp = tet_getvar("VSX_UNSUPPORTED_PGID")) == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_UNSUPPORTED_PGID is not set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (strncmp(cp, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_UNSUPPORTED_PGID is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pgid = atol(cp);

	if (pgid < 0)
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_UNSUPPORTED_PGID must not be < 0");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	s_pgid = getpgrp();

	errno = 0;
	rval = tb_setpgid((pid_t)0, pgid);
	if ((rval != SYSERROR) || (errno != EINVAL))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid(0, pgid) with pgid = %ld did not fail as expected with errno = EINVAL",
			(long)pgid);
		in_rpt("setpgid() returned %d with errno = %d(%s)",
			rval, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	c_pgid = getpgrp();

	if(s_pgid != c_pgid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid() on failure changed the process group");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!nfails)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	DBUG_ENTER("test9");

	if (!not_supported)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_SETPGID_SUPPORTED is set to 'Y'");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t9, NULLFN, 10*WAITTIME, 0);

	DBUG_VOID_RETURN; 
} 


private	void
ch_t9()
{
	int	rval;
	int	nfails, err;
	pid_t	s_pgid, c_pgid;
	int	pathok = 0;

	DBUG_ENTER("ch_t9");

	nfails = 0;

	(void) setsid();

	s_pgid = getpgrp();

	errno = 0;
	rval = tb_setpgid((pid_t)0, (pid_t)0);
	if ((rval != SYSERROR) || (errno != ENOSYS))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid(0, 0) did not fail as expected with errno = ENOSYS");
		in_rpt("setpgid() returned %d with errno = %d(%s)",
			rval, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	c_pgid = getpgrp();

	if(s_pgid != c_pgid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid() on failure changed the process group");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!nfails)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	DBUG_ENTER("test10");

	IFNOTSUPP

	(void) cppair(ch_t10, pa_t10, 10*WAITTIME, 0);

	DBUG_VOID_RETURN; 
} 


private void
pa_t10()
{

	int	rval;
	int	nfails, err;
	pid_t	s_pgid, c_pgid;
	int	pathok = 0;

	DBUG_ENTER("pa_t10");

	nfails = 0;

	(void) sleep(WAITTIME);

	s_pgid = getpgrp();

	errno = 0;
	rval = tb_setpgid(tet_child, (pid_t)0);
	if ((rval != SYSERROR) || (errno != EPERM))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid(pid, 0) with pid = %ld, when the process is in a different session, did not fail as expected with errno = EPERM",
			(long)tet_child);
		in_rpt("setpgid() returned %d with errno = %d(%s)",
			rval, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	c_pgid = getpgrp();

	if(s_pgid != c_pgid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid() on failure changed the process group");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!nfails)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private	void
ch_t10()
{

	DBUG_ENTER("ch_t10");
	(void) setsid();
	(void) sleep(2*WAITTIME);
	DBUG_VOID_RETURN;

}

private void
test11()
{
	DBUG_ENTER("test11");

	IFNOTSUPP

	(void) cppair(ch_t11, NULLFN, 5*WAITTIME, 0);

	DBUG_VOID_RETURN; 
} 


private	void
ch_t11()
{

	DBUG_ENTER("ch_t11");

	(void) cppair(gch1_t11, gch2_t11, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private	void
gch1_t11() /* child */
{
	DBUG_ENTER("gch1_t11");

	errno = 0;

	(void) vsignal(SIGALRM, alrm);
	if (sleep(2*WAITTIME) == 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: child did not recieve signal from parent");
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private void
gch2_t11() /* parent */
{
	int 	rval;
	int	nfails, err;
	pid_t	s_pgid, c_pgid;
	int	pathok = 0;

	DBUG_ENTER("gch2_t11");

	nfails = 0;
	
	ch_t11_pid = tet_child;

	(void) sleep(WAITTIME);

	errno = NOERROR;
	if (setsid() == SYSERROR) 
	{
		err = errno;
		if(!nfails)
			xx_rpt(DELETION);
		in_rpt("deletion reason: setsid(0, 0) failed %d(%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	s_pgid = getpgrp();

	errno = NOERROR;
	rval = tb_setpgid((pid_t) ch_t11_pid, (pid_t) 0);

	if ((rval != SYSERROR) || (errno != EPERM))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid(pid, 0) with pid = %ld, when the process is in a different session, did not fail as expected with errno = EPERM",
			(long)ch_t11_pid);
		in_rpt("setpgid() returned %d with errno = %d(%s)",
			rval, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	c_pgid = getpgrp();

	if(s_pgid != c_pgid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid() on failure changed the process group");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!nfails)
		PATH_XS_RPT(3);

	(void) kill((pid_t)ch_t11_pid, SIGALRM);

	DBUG_VOID_RETURN;
}


private void
test12()
{
	DBUG_ENTER("test12");

	IFNOTSUPP

	(void) cppair(ch_t12, pa_t12, 4*WAITTIME, 0);

	DBUG_VOID_RETURN;

}

private void
pa_t12()
{

	DBUG_ENTER("pa_t12");

	ch_t12_pid = tet_child;

	(void) cppair(gch1_t12, gch2_t12, (3*WAITTIME), 0);

	DBUG_VOID_RETURN; 
} 


private void
ch_t12()
{
	DBUG_ENTER("ch_t12"); 

	(void) sleep(3*WAITTIME);

	DBUG_VOID_RETURN;
}

private void
gch1_t12() /* child */
{
	DBUG_ENTER("gch1_t12"); 

	(void) setsid();
	(void) sleep(2*WAITTIME);

	DBUG_VOID_RETURN;
}

private void
gch2_t12() /* parent */
{
	int	rval;
	int	nfails, err;
	pid_t	s_pgid, c_pgid;
	int	pathok = 0;

	DBUG_ENTER("gch2_t12"); 

	nfails = 0;

	(void) sleep(WAITTIME);

	s_pgid = getpgrp();

	errno = NOERROR;
	rval = tb_setpgid(ch_t12_pid, tet_child);
	if ((rval != SYSERROR) || (errno != EPERM))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid(pid, pgid) pid = %ld, pgid = %ld, when the process group ID did not match any in the session, did not fail as expected with errno = EPERM",
			(long)ch_t12_pid, (long)tet_child);
		in_rpt("setpgid() returned %d with errno = %d(%s)",
			rval, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	c_pgid = getpgrp();

	if(s_pgid != c_pgid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
	       	in_rpt("setpgid() on failure changed the process group");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	s_pgid = getpgrp();

	errno = NOERROR;
	rval = tb_setpgid((pid_t)ch_t12_pid, (pid_t)(getpgrp()-1));
	if ((rval != SYSERROR) || (errno != EPERM))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid(pid, pgid) pid = %ld, pgid = %ld, when the process group ID did not match any in the session, did not fail as expected with errno = EPERM",
			(long)ch_t12_pid, (long)getpgrp()-1);
		in_rpt("setpgid() returned %d with errno = %d(%s)",
			rval, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	c_pgid = getpgrp();

	if(s_pgid != c_pgid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
	       	in_rpt("setpgid() on failure changed the process group");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!nfails)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test13()
{
	int	nfails, err;
	pid_t	s_pgid, c_pgid;
	int	rval;
	int	pathok = 0;

	DBUG_ENTER("test13");

	nfails = 0;

	IFNOTSUPP

	globok = 0;
	if (cppair(ch_t13, pa_t13, WAITTIME, 0) != OK)
	{
		DBUG_VOID_RETURN;
	}
	else	
		PATH_FUNC_TRACE;

	s_pgid = getpgrp();

	errno = 0;
	rval = tb_setpgid((pid_t)ch_t13_pid, (pid_t)0);
	if ((rval != SYSERROR) || (errno != ESRCH))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid(pid, pgid) pid = %ld, pgid = 0, did not fail as expected with errno = ESRCH",
			(long)ch_t13_pid);
		in_rpt("setpgid() returned %d with errno = %d(%s)",
			rval, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	c_pgid = getpgrp();

	if(s_pgid != c_pgid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setpgid() on failure changed the process group");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!nfails)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN; 
} 


private void
pa_t13()
{
	int	pathok = 0;

	DBUG_ENTER("pa_t13");

	ch_t13_pid = tet_child;

	PATH_FUNC_RPT(0); /* cppair expects path tracing in parent */

	DBUG_VOID_RETURN; 
} 

private void
ch_t13()
{
	DBUG_ENTER("ch_t13"); 

	DBUG_VOID_RETURN;
}


private	int
tb_setpgid(pid, pgid)
pid_t	pid;
pid_t	pgid;
{
	int	retval;

	retval = setpgid(pid, pgid);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
