/*
*      SCCS:  @(#)  POSIX.os/procenv/setsid/setsid.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/setsid - T.setsid Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.setsid

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the setsid() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Wed Feb 15 14:43:33 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 17 July 1989
			If setpgid() is not supported, second part
			of EPERM test is not required.

		Geoff Clare, 15 Sep 1989
			Write c_cc[]'s as 1 char, not sizeof(cc_t)
		
		Geoff Clare, 7 June 1990
			Align with P1003.3 draft 11

		David Sawyer
		Wed Jul 25 12:33:56 BST 1990
		Added resilience code.
		
		Geoff Clare, 22 Nov 1991
			Changes to support pseudo-terminals.

		Stuart Boutell, 18 May 1992
			Added tests to ensure that setsid deviance
		tests do not relinquish the controlling terminal.

                Geoff Clare, 15 July 1994
			Add #include <stdio.h> (for sprintf)

                Geoff Clare, 16 Oct 1996
			Remove xx_rpt() in setup function.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<fcntl.h>
#include	<termios.h>
#include	<signal.h>
#include	<timeout.h>
#include	<unistd.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef setsid
#endif


#define 	NO_TESTS	6		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)
#define		NOK	02	/* failure values from the children */
#define		PATH_OK 04

extern	void	valrpt();
extern	char	*errname();
extern	int	cppair();

int	no_tests = NO_TESTS;

private	pid_t	tb_setsid();
private	void	setup();
private	void	cleanup();
private void	test1();
private void	ch_t1();
private void	test2();
private void	ch_t2();
private void	test3();
private void	ch_t3();
private void	test4();
private void	ch_t4();
private void	test5();
private void	ch1_t5();
private void	test6();
private	void	pa1_t6();
private	void	pa2_t6();
private void	ch1_t6();
private void	ch2_t6();
private	int	ctty_sig();
private	void	ctty_post();
private	void	sig_establish();
private	int	tty_fd;
private	struct	termios	tty_term,	*tty_term_p = &tty_term;
private	int	loop_fd;
private	struct	termios	loop_term;
private	int	sigint_posted = FALSE;
private	pid_t	ch1_t6_pid;
private	int	gti_supported;

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = cleanup;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	NULL, 0
};

private	void
setup()
{
	int i;
	static char msgbuf[256];
	int	err;

	DBUG_ENTER("setup");

	errno = 0;
	if (tb_setsid() == (pid_t)SYSERROR)
	{
		err = errno;
		(void) sprintf(msgbuf, "setsid() failed in setup - errno %d (%s)",
			err,errname(err));
		for (i = 1; i <= NO_TESTS; i++)
			tet_delete(i, msgbuf);
		DBUG_VOID_RETURN;
	}

	errno = 0;
	switch (terms_prep(O_RDWR, &tty_fd , &tty_term,
			O_RDWR, &loop_fd , &loop_term))
	{
	case 0:
		gti_supported = 1;
		break;
	case 1:
		gti_supported = 0;
		break;

	default:
		tet_delete(3, "terms_prep() failed in setup");
		gti_supported = 0;
		break;
	}

	DBUG_VOID_RETURN;
}

private void
test1()
{

	DBUG_ENTER("test1");
	(void) cppair(ch_t1 , NULLFN , WAITTIME , 0);
	DBUG_VOID_RETURN; 
} 

private void
ch_t1()
{
	pid_t	rval;
	pid_t	pgid;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");
	
	errno = 0;
	if ((rval = tb_setsid()) == SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setsid() failed - errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((pgid = getpgrp()) != rval)
	{
		xx_rpt(FAILURE);
		in_rpt("setsid() did not return new process group ID");
		in_rpt("setsid() returned: %ld, getpgrp() returned: %ld",
			(long)rval, (long)pgid);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private void
test2()
{

	DBUG_ENTER("test2");
	(void) cppair(ch_t2 , NULLFN , WAITTIME , 0);
	DBUG_VOID_RETURN; 
} 

private void
ch_t2()
{
	pid_t	pid;
	pid_t	pgid;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t2");
	
	errno = 0;
	if (tb_setsid() == SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setsid() failed - errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if ((pgid = getpgrp()) != (pid = getpid()))
	{
		xx_rpt(FAILURE);
		in_rpt("After setsid() process is not a process group leader");
		in_rpt("process ID: %ld, process group ID: %ld",
			(long)pid, (long)pgid);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

private void
test3()
{

	int	err;

	DBUG_ENTER("test3");

	if(!gti_supported) {
		xx_rpt(UNTESTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN; 
	}

	sig_establish(SIG_IGN);
	(void) cppair(ch_t3 , NULLFN , 10*WAITTIME , 0);
	DBUG_VOID_RETURN; 
} 

private void
ch_t3()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");
	
	globok = 0;
	sig_establish(ctty_post);
	PATH_FUNC_TRACE;

	errno = 0;
	globok = 0;
	if (!ctty_sig(TRUE))
	{
		xx_rpt(FAILURE);
		in_rpt("1st SIGINT from controlling terminal not caught");
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	errno = 0;
	globok = 0;
	if (!ctty_sig(TRUE))
	{
		xx_rpt(FAILURE);
		in_rpt("2nd SIGINT from controlling terminal not caught");
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	errno = 0;
	if (tb_setsid() == (pid_t)SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setsid() failed %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	errno = 0;
	globok = 0;
	if (ctty_sig(TRUE))
	{
		xx_rpt(FAILURE);
		in_rpt("SIGINT from controlling terminal not caught");
	}
	else
	{
		PATH_FUNC_TRACE;
		PATH_XS_RPT(5);
	}
	DBUG_VOID_RETURN; 
} 

private int
ctty_sig(sig_exp)
int	sig_exp;
{

	int	rval;
	unsigned char chr;
	int	pathok = 0;

	DBUG_ENTER("ctty_sig");
	(void) sleep(WAITTIME);
	sigint_posted = FALSE;

	chr = tty_term_p->c_cc[VINTR];

	SET_TIMEOUT(WAITTIME);
	errno = 0;
	rval = write(loop_fd , (char *)&chr, 1);

	(void) pause();

	CLEAR_ALARM;

	if (rval != 1)
	{
		xx_rpt(FAILURE);
		in_rpt("write() failed to write %d bytes instead wrote %d bytes"			,1,rval);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(sig_exp == sigint_posted); 
} 

private	void
ctty_post()
{
	sigint_posted = TRUE;
}

private void
sig_establish(func)
void	(*func) ();
{
	struct	sigaction	sig;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("sig_establish");

	sig.sa_handler = func;
	(void) sigemptyset(&sig.sa_mask);
	sig.sa_flags = 0;

	if (sigaction(SIGINT , &sig , NULLSA) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sigaction() failed %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN; 
}

private void
test4()
{

	DBUG_ENTER("test4");
	(void) cppair(ch_t4 , NULLFN , WAITTIME , 0);
	DBUG_VOID_RETURN; 
} 

private void
ch_t4()
{
	pid_t	rval;
	pid_t	pid;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");
	
	errno = 0;
	if ((rval = tb_setsid()) == (pid_t)SYSERROR) 
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setsid() failed %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((pid = getpid()) != rval)
	{
		xx_rpt(FAILURE);
		in_rpt("setsid() failed to change the process group ID to the process ID");
		in_rpt("process group ID %ld - process ID %ld",
			(long)pid, (long)rval);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test5()
{
	int	retval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	errno = 0;
	retval = tb_setsid();
	if ((retval != (pid_t)SYSERROR) && (errno != EPERM))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setsid() did not fail as expected");
		in_rpt("setsid() returned %d(%s)",err,errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if(gti_supported) {
		globok = 0;
		sig_establish(ctty_post);
		PATH_FUNC_TRACE;

		globok = 0;
		if (!ctty_sig(TRUE))
		{
			xx_rpt(FAILURE);
			in_rpt("SIGINT from controlling terminal not caught");
			in_rpt("failed setsid() command caused controlling");
			in_rpt("terminal to be relinquished");
			DBUG_EXIT(NOK);	
		}
		else
			PATH_FUNC_TRACE;
	} else {
		PATH_TRACE; PATH_TRACE;
	}

	retval = cppair(ch1_t5 , NULLFN , WAITTIME , NOK | PATH_OK);
	if (retval & NOK)
	{
		DBUG_VOID_RETURN;
	}
	else if (retval & PATH_OK)
		PATH_TRACE;

	PATH_XS_RPT(4);

	DBUG_VOID_RETURN; 
} 

private void
ch1_t5()
{
	pid_t	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch1_t5");
	
	errno = 0;
	if (setpgid(0,0) == SYSERROR)
	{
		if (errno == ENOSYS)
		{
			/* setpgid() not supported */
			DBUG_VOID_RETURN;
		}
		else
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: setpgid() failed %d(%s)",
				err, errname(err));
			DBUG_EXIT(NOK);
		}
	}
	else
		PATH_TRACE;

	errno = 0;
	rval = tb_setsid();
	if ((rval != (pid_t)SYSERROR) && (errno != EPERM))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setsid() did not fail as expected");
		in_rpt("setsid() returned %d(%s)",err,errname(err));
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;

	if (pathok == 2)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}

private void
test6()
{

	int	err;
	DBUG_ENTER("test6");

	errno = 0;
	if (sysconf(_SC_JOB_CONTROL) == SYSERROR)
	{
		if (errno != 0)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("sysconf(_SC_JOB_CONTROL) failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		else
		{
			xx_rpt(UNTESTED);
			in_rpt("cannot test if _POSIX_JOB_CONTROL not implemented");
			DBUG_VOID_RETURN;
		}
	}

	sig_establish(SIG_IGN);
	(void) cppair(ch1_t6, pa1_t6, (7*WAITTIME), 0);

	DBUG_VOID_RETURN; 
} 

private void
ch1_t6()		/* Assertion 07 */
{
	int	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch1_t6");

	errno = NOERROR;
	if ((rval = setpgid(0, 0)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: setpgid(0, child1) has failed");
		valrpt(0, rval, NOERROR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(3*WAITTIME);

	errno = NOERROR;
	if (getpgrp() != getppid())
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: first child not in parent process group");
		in_rpt("process ID of parent %ld",(long)getppid());
		in_rpt("process group of first child %ld",(long)getpgrp());
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if ((rval = tb_setsid()) != SYSERROR || errno != EPERM)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setsid() did not fail as expected with EPERM");
		valrpt(SYSERROR, rval, EPERM, err);
	}
	else
	{
		PATH_TRACE;

		if(gti_supported) {
			globok = 0;
			sig_establish(ctty_post);
			PATH_FUNC_TRACE;

			errno = 0;
			globok = 0;
			if (!ctty_sig(TRUE))
			{
				xx_rpt(FAILURE);
				in_rpt("SIGINT from controlling terminal not caught");
				in_rpt("failed setsid() command caused controlling");
				in_rpt("terminal to be relinquished");
				DBUG_VOID_RETURN;
			}
			else
				PATH_FUNC_TRACE;
		} else {
			PATH_TRACE; PATH_TRACE;
		}

		PATH_XS_RPT(5);
	}

	DBUG_VOID_RETURN; 
} 

private void
pa1_t6()		/* Assertion 07 */
{

	DBUG_ENTER("pa1_t6");

	globok = 1; /* cppair expects globok to be set if parent exists */

	ch1_t6_pid = tet_child;
	(void) cppair(ch2_t6, pa2_t6, (6*WAITTIME), 0);


	DBUG_VOID_RETURN; 
} 

private void
pa2_t6()		/* Assertion 07 */
{
	int	rval;
	int	err;

	DBUG_ENTER("pa2_t6");

	globok = 1; /* cppair expects globok to be set if parent exists */

	(void) sleep(2*WAITTIME);

	if ((rval = setpgid(ch1_t6_pid, getpgrp())) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: setpgid(child1, pid) has failed");
		valrpt(0, rval, NOERROR, err);
		in_rpt("process ID of child no1 %ld",(long)ch1_t6_pid);
		in_rpt("process group of parent %ld",(long)getpgrp());
		DBUG_VOID_RETURN; 
	}

	DBUG_VOID_RETURN; 
} 

private void
ch2_t6()		/* Assertion 07 */

{
	int	rval;
	int	err;

	DBUG_ENTER("ch2_t6");

	(void) sleep(WAITTIME);

	if ((rval = setpgid(0, ch1_t6_pid)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: setpgid(0, child1) has failed");
		valrpt(0, rval, NOERROR, err);
		DBUG_VOID_RETURN; 
	}

	(void) sleep(4*WAITTIME);

	DBUG_VOID_RETURN; 
} 

private	void
cleanup()
{
	DBUG_ENTER("cleanup");
	(void) close(tty_fd);
	(void) close(loop_fd);
	DBUG_VOID_RETURN;
}


private	pid_t
tb_setsid()
{
	pid_t	retval;

	retval = setsid();
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
