/*
*      SCCS:  @(#)  POSIX.os/procenv/setuid/setuid.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/setuid - T.setuid Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.setuid

PROJECT:	VSX (X/OPEN Validation Suite)


MODULE CONTAINS:

SYNOPSIS:


DESCRIPTION:
	Testset for the setuid() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Mon Feb  6 11:09:51 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 18 June 1990
			Merge EPERM tests

		David Sawyer
		Wed Jul 25 13:04:36 BST 1990
		Added resilience code.

		Stuart Boutell, 2 Nov 1990
			changed execl for tet_exec

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 27 June 1991
			VSX_INVALID_UID="unsup" for unsupported result

		Geoff Clare, 4 Mar 1992
			Change SAVED_IDS check in test 4.

		Neil Moses, 18 May 1992
			Check uid and euid are not changed in deviance tests.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef setuid
#endif


#define	NO_TESTS	6		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)

extern  char    *newstr();
extern	char	*errname();
extern	void	valrpt();
extern	char 	*itoa();
extern	int 	uids();
extern	int	vggid0;
extern	int	vggid1;
extern 	int	vggid2;
extern	int	vguid0;
extern	int	vguid1;
extern	int 	vguid2;

extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;

int	no_tests = NO_TESTS;

private char 	*setuid_t01 = "../setuid_t01";
private char 	*setuid_su1 = "../setuid_su1";
private char 	*setuid_t03 = "../setuid_t03";
private void	setup();
private	void	test1();
private	void	test2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	test6();
private	void	ch_t1();
private	void	ch_su1();
private	void	ch_t3();
private	int	tb_setuid();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	NULL, 0
};

private void
setup()
{
	DBUG_ENTER("setup");

	if(uids()!=0)
	{
		int i;
		for(i=1; i <= no_tests; i++)
		{
			(void) tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}


	DBUG_VOID_RETURN;
}

private void
test1()
{

	DBUG_ENTER("test1");

	(void) cppair(ch_t1 , NULLFN , WAITTIME , 0);

	DBUG_VOID_RETURN;

}

private	void
ch_t1()
{
	char	*args[6];
	char 	*uid1str;	
	char 	*uid2str;

	DBUG_ENTER("ch_t1");

	uid1str = newstr(itoa(vguid1));
	uid2str = newstr(itoa(vguid2));
	if (uid1str == NULL || uid2str == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc() failed");
		DBUG_VOID_RETURN;
	}

	args[0] = setuid_t01;
	args[1] = uid1str;
	args[2] = uid2str;
	args[3] = vg_ident;
	args[4] = vg_dbug;
	args[5] = (char *)NULL;
	(void) tet_exec(setuid_t01, args, environ);

	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed", setuid_t01);
	DBUG_VOID_RETURN;
}

private void
test2()
{
	int err;

	DBUG_ENTER("test2");

	errno = 0;
	if (sysconf(_SC_SAVED_IDS) == SYSERROR)
	{
		if (errno != 0)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("sysconf(_SC_SAVED_IDS) failed - errno %d (%s)",
				err, errname(err));
		}
		else
		{
			xx_rpt(UNSUPPORTED);
			in_rpt("_POSIX_SAVED_IDS not implemented");
		}
	}
	else
		(void) cppair(ch_su1 , NULLFN , WAITTIME , 0);

	DBUG_VOID_RETURN;
}

private	void
ch_su1()
{
	char	*args[7];
	char	*tnum;
	char 	*uid0str;	
	char 	*uid1str;

	DBUG_ENTER("ch_su1");

	tnum = newstr(itoa(tet_thistest));
	uid0str = newstr(itoa(vguid0));
	uid1str = newstr(itoa(vguid1));
	if (tnum == NULL || uid0str == NULL || uid1str == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc() failed");
		DBUG_VOID_RETURN;
	}

	args[0] = setuid_su1;
	args[1] = tnum;
	args[2] = uid0str;
	args[3] = uid1str;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = (char *)NULL;
	(void) tet_exec(setuid_su1, args, environ);

	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed", setuid_su1);
	DBUG_VOID_RETURN;
}

private void
test3()
{

	DBUG_ENTER("test3");

	(void) cppair(ch_t3 , NULLFN , WAITTIME , 0);

	DBUG_VOID_RETURN;

}

private	void
ch_t3()
{
	char	*args[5];
	char 	*uid2str;

	DBUG_ENTER("ch_t3");

	if ((uid2str = newstr(itoa(vguid2))) == (char *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("malloc() failed");
		DBUG_VOID_RETURN;
	}

	args[0] = setuid_t03;
	args[1] = uid2str;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = (char *)NULL;
	(void) tet_exec(setuid_t03, args, environ);

	xx_rpt(DELETION);
	in_rpt("tet_exec(%s, ...) failed", setuid_t03);
	DBUG_VOID_RETURN;
}

private void
test4()
{
	int err;

	DBUG_ENTER("test4");

	errno = 0;
	if (sysconf(_SC_SAVED_IDS) == SYSERROR)
	{
		if (errno != 0)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("sysconf(_SC_SAVED_IDS) failed - errno %d (%s)",
				err, errname(err));
		}
		else
			(void) cppair(ch_su1 , NULLFN , WAITTIME , 0);
	}
	else
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_SAVED_IDS is implemented");
	}

	DBUG_VOID_RETURN;

}

private void
test5()
{
	char	*cp;
	int	rval;
	uid_t	invaluid;
	uid_t	s_uid, s_euid, c_uid, c_euid;
	int	nfails, err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	nfails = 0;

	if ((cp = tet_getvar("VSX_INVALID_UID")) == NULL || *cp == '\0')
	{
		if(!nfails)
			xx_rpt(DELETION);
		in_rpt("parameter VSX_INVALID_UID is not set");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(cp, "unsup", (size_t)5) == 0)
	{
		if(!nfails)
			xx_rpt(UNSUPPORTED);
		in_rpt("parameter VSX_INVALID_UID is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	invaluid = atoi(cp);

	s_uid = getuid();
	s_euid = geteuid();

	errno = 0;
	if (((rval = tb_setuid( invaluid )) != SYSERROR) ||
		(errno != EINVAL))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setuid(%d) did not fail as expected", (int)invaluid);
		valrpt(SYSERROR, rval, EINVAL, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	c_uid = getuid();
	c_euid = geteuid();

	if(s_uid != c_uid || s_euid != c_euid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		if(s_uid != c_uid)
		{
			in_rpt("setuid(%d) on failure changed the user id", (int)invaluid);
			in_rpt("of the calling process");
		}
		if(s_euid != c_euid)
		{
			in_rpt("setuid(%d) on failure changed the effective user id", (int)invaluid);
			in_rpt("of the calling process");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!nfails)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	int	rval;
	int	nfails, err;
	uid_t	s_uid, s_euid, c_uid, c_euid;
	int	pathok = 0;

	DBUG_ENTER("test6");

	nfails = 0;

	if (setprv(PRV_SETID) == 0)
	{
		if(!nfails)
			xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_SETID) succeeded");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (geteuid() == vguid2 || getuid() == vguid2)
	{
		if(!nfails)
			xx_rpt(DELETION);
		in_rpt("deletion reason: real or effective UID is VSX_UID2");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	s_uid = getuid();
	s_euid = geteuid();

	errno = 0;
	if (((rval = tb_setuid( (uid_t) vguid2 )) != SYSERROR) ||
		(errno != EPERM))
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("setuid(VSX_UID2) did not fail as expected");
		valrpt(SYSERROR, rval, EPERM, err);
		DBUG_VOID_RETURN;
	} 
	else
		PATH_TRACE;

	c_uid = getuid();
	c_euid = geteuid();

	if(s_uid != c_uid || s_euid != c_euid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		if(s_uid != c_uid)
		{
			in_rpt("setuid(%d) on failure changed the user id", vguid2);
			in_rpt("of the calling process");
		}
		if(s_euid != c_euid)
		{
			in_rpt("setuid(%d) on failure changed the effective user id", vguid2);
			in_rpt("of the calling process");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if (sysconf(_SC_SAVED_IDS) == SYSERROR)
	{
		if (errno != 0)
		{
			err = errno;
			if(!nfails)
				xx_rpt(DELETION);
			in_rpt("sysconf(_SC_SAVED_IDS) failed - errno %d (%s)",
				err, errname(err));
			DBUG_VOID_RETURN;
		}
		(void) cppair(ch_su1, NULLFN, WAITTIME, 0);
	}
	else
		PATH_TRACE;

	if(!nfails)
		PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private	int
tb_setuid(uid)
uid_t	uid;
{
	int	retval;

	retval = setuid(uid);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
