/*
*      SCCS:  @(#)  POSIX.os/procenv/setuid/setuid_su1.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/setuid - setuid_su1 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		setuid_su1 - used by T.setuid for SUID-vsx1 tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.setuid in tests requiring an effective user id of vsx1.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Fri Feb  3 13:26:14 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 18 June 1990
			Merge all SUID vsx1 subprograms into one

		David Sawyer
		Wed Jul 25 13:04:54 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Neil Moses, 18 May 1992
			Check uid and euid are not changed in test6.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef setuid
#endif



extern	char	*errname();
extern	void	set_pname();
extern	void	valrpt();

extern	int	vguid0;
extern	int	vguid1;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	void	subtest2();
private	void	subtest4();
private	void	subtest6();
private	int	tb_setuid();

public	int
tet_main(argc, argv)
	int	argc;
	char	**argv;
{
	int tnum;
 
	set_pname("setuid_su1");
	if (argc == 6)
	{
		if (argv[5][0] == '-' && argv[5][1] == 'D')
		{
			vg_dbug = argv[5];
			DBUG_PUSH(&vg_dbug[2]);
		}
		argc--;
	}
	if (argc != 5)
	{
		xx_rpt(DELETION);
		in_rpt("argument count wrong, got %d, expected 5",argc);
		exit(OK);
	}
	if (argv[4] != NULL && argv[4][0] != NUL)
		vg_ident = argv[4];

	tnum = atoi(argv[1]);
	vguid0 = atoi(argv[2]);
	vguid1 = atoi(argv[3]);

	DBUG_PRINTF("setuid_su1", "tnum = %d", tnum);
	DBUG_PRINTF("setuid_su1", "vguid0 = %d", vguid0);
	DBUG_PRINTF("setuid_su1", "vguid1 = %d", vguid1);

	if (geteuid() != vguid1)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: effective UID NOT= VSX_UID1");
		exit(OK);
	}

	switch (tnum)
	{
	case 2:
		subtest2();
		break;

	case 4:
		subtest4();
		break;

	case 6:
		subtest6();
		break;
	
	default:
		xx_rpt(DELETION);
		in_rpt("bad test number passed to setuid_su1: %d", tnum);
	}

	exit(OK);

	/* NOTREACHED */
}


private void
subtest2()
{
	uid_t	uid;
	uid_t	euid;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("subtest2");

	/*
	 * Attempt to set effective user ID to real ID
	 * by calling setuid(real-user-ID).
	 */
	errno = 0;
	if (tb_setuid( (uid_t) vguid0 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setuid call no.1 failed with errno = %d (%s)", 
			err, errname(err));
		in_rpt("when attempting to set the effective user ID to the real user ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	uid = getuid();
	if (uid != vguid0)
	{
		xx_rpt(FAILURE);
		in_rpt("UID (%d) not equal to VSX_UID0 (%d)", uid, vguid0);
		in_rpt("when attempting to set the effective user ID to the real user ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	euid = geteuid();
	if (euid != vguid0)
	{
		xx_rpt(FAILURE);
		in_rpt("UID (%d) not equal to effective VSX_UID0 (%d)",
			euid, vguid0);
		in_rpt("when attempting to set the effective user ID to the real user ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Attempt to set effective user ID to saved set user ID
	 * by calling setuid(real-user-ID).
	 */
	errno = 0;
	if (tb_setuid( (uid_t) vguid1 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setuid call no.2 failed with errno = %d (%s)", 
			err, errname(err));
		in_rpt("when attempting to set the effective user ID to the saved set user ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	uid = getuid();
	if (uid != vguid0)
	{
		xx_rpt(FAILURE);
		in_rpt("UID (%d) not equal to VSX_UID0 (%d)", uid, vguid0);
		in_rpt("when attempting to set the effective user ID to the saved set user ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	euid = geteuid();
	if (euid != vguid1)
	{
		xx_rpt(FAILURE);
		in_rpt("UID (%d) not equal to effective VSX_UID1 (%d)",
			euid, vguid1);
		in_rpt("when attempting to set the effective user ID to the saved set user ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Call setuid(saved-set-user-ID) and expect no change to real,
	 * effective or saved set user ID - because effective user ID
	 * is already vguid1.
	 */
	errno = 0;
	if (tb_setuid( (uid_t) vguid1 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setuid call no.3 failed with errno = %d (%s)", 
			err, errname(err));
		in_rpt("when attempting to set the effective user ID to the saved set user ID a second time");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	uid = getuid();
	if (uid != vguid0)
	{
		xx_rpt(FAILURE);
		in_rpt("UID (%d) not equal to VSX_UID0 (%d)", uid, vguid0);
		in_rpt("when attempting to set the effective user ID to the saved set user ID a second time");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	euid = geteuid();
	if (euid != vguid1)
	{
		xx_rpt(FAILURE);
		in_rpt("UID (%d) not equal to effective VSX_UID1 (%d)",
			euid, vguid1);
		in_rpt("when attempting to set the effective user ID to the saved set user ID a second time");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if (tb_setuid( (uid_t) vguid0 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setuid call no.4 failed with errno = %d (%s)", 
			err, errname(err));
		in_rpt("when attempting to set the effective user ID to the real user ID a second time");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if (tb_setuid( (uid_t) vguid1 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setuid call no.5 failed with errno = %d (%s)", 
			err, errname(err));
		in_rpt("when attempting to set the effective user ID to the saved set user ID a second time");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(11);

	DBUG_VOID_RETURN;
}

private void
subtest4()
{
	uid_t	uid;
	uid_t	euid;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("subtest4");

	errno = 0;
	if (tb_setuid( (uid_t) vguid0 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setuid call failed with errno = %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	uid = getuid();
	if (uid != vguid0)
	{
		xx_rpt(FAILURE);
		in_rpt("Real user ID has not changed");
		in_rpt("UID (%d) not equal to VSX_UID0 (%d)", uid, vguid0);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	euid = geteuid();
	if (euid != vguid0)
	{
		xx_rpt(FAILURE);
		in_rpt("Effective user ID has not changed");
		in_rpt("UID (%d) not equal to effective VSX_UID0 (%d)",
			euid, vguid0);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
subtest6()
{
	uid_t	euid;
	int	nfails, rval, err;
	uid_t	s_uid, s_euid, c_uid, c_euid;
	int	pathok = 0;

	DBUG_ENTER("subtest6");

	nfails = 0;

	errno = 0;
	if (tb_setuid( (uid_t) vguid0 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("setuid(VSX_UID0) failed with errno = %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	euid = geteuid();
	if (euid != vguid0)
	{
		xx_rpt(DELETION);
		in_rpt("Effective user ID has not changed");
		in_rpt("UID (%d) not equal to VSX_UID0 (%d)", euid, vguid0);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* If _POSIX_SAVED_IDS was supported the saved id would now
	   be vg_uid1 */
	
	s_uid = getuid();
	s_euid = geteuid();

	errno = 0;
	if (((rval = tb_setuid( (uid_t) vguid1 )) != SYSERROR) ||
		(errno != EPERM))
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("setuid(VSX_UID1) did not fail as expected");
		valrpt(SYSERROR, rval, EPERM, err);
		DBUG_VOID_RETURN;
	} 
	else
		PATH_TRACE;

	c_uid = getuid();
	c_euid = geteuid();

	if(s_uid != c_uid || s_euid != c_euid)
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		if(s_uid != c_uid)
		{
			in_rpt("setuid(%d) on failure changed the user id", vguid1);
			in_rpt("of the calling process");
		}
		if(s_euid != c_euid)
		{
			in_rpt("setuid(%d) on failure changed the effective user id", vguid1);
			in_rpt("of the calling process");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(nfails++)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}



private	int
tb_setuid(uid)
uid_t	uid;
{
	int	retval;

	retval = setuid(uid);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}

