/*
*      SCCS:  @(#)  POSIX.os/procenv/setuid/setuid_t01.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/setuid - setuid_t01 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		setuid_t01 - used by T.setuid in comply()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.setuid to test privileged setuid() calls.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Fri Feb  3 16:11:41 GMT 1989
MODIFICATIONS:	

		David Sawyer
		Wed Jul 25 13:05:12 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Peter Skan, 10 Sept 1991
			Remove SETID privilege after first setuid()

		Geoff Clare, 18 Aug 1995
			Changed arg of second setuid() call from vguid1
			to zero. 

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<signal.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<setjmp.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef setuid
#endif


#define	NO_TESTS	7		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();
extern	void	set_pname();

extern	int	vguid2;
extern	int	vguid1;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	void	subtest1();
private	int	tb_setuid();

public	int
tet_main(argc, argv)		/* Assertion 10 */

	int	argc;
	char	**argv;
{
 
	DBUG_ENTER("subtest1(main)");
	set_pname(argc < 1 || argv[1] == (char *) NULL ?
		"setuid_t01" : (argv[1]));
	if (argc == 5)
	{
		if (argv[4] != (char *) NULL && strncmp("-D", argv[4], 2) == 0)
		{
			vg_dbug = argv[4];
			DBUG_PUSH(&vg_dbug[2]);
		}
		argc--;
	}
	if (argc != 4)
	{
		in_rpt("argument count wrong, got %d, expected 4",argc);
		exit(A_MISC);
	}
	if (argv[3] != (char *) NULL && argv[3][0] != NUL)
		vg_ident = argv[3];

	if (sysconf(_SC_SAVED_IDS) == SYSERROR)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_SAVED_IDS not implemented");
		DBUG_EXIT(OK);
	}

	vguid1 = atoi(argv[1]);
	vguid2 = atoi(argv[2]);

	if (setprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_SETID) failed");
		DBUG_EXIT(OK);
	}

	subtest1();

	DBUG_EXIT(OK);

	/* NOTREACHED */
}


private void
subtest1()
{
	uid_t	uid;
	uid_t	euid;
	int	rval, err;
	int	pathok = 0;

	DBUG_ENTER("subtest1");

	errno = NOERROR;
	if (tb_setuid( (uid_t) vguid2 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setuid call no.1 failed with errno = %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	uid = getuid();
	if (uid != vguid2)
	{
		xx_rpt(FAILURE);
		in_rpt("Real user ID has not changed");
		in_rpt("UID (%ld) not equal to VSX_UID2 (%d)",
			(long)uid, vguid2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	euid = geteuid();
	if (euid != vguid2)
	{
		xx_rpt(FAILURE);
		in_rpt("Effective user ID has not changed");
		in_rpt("UID (%ld) not equal to effective VSX_UID2 (%d)",
			(long)euid, vguid2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (unsetprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: unsetprv(PRV_SETID) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check saved set-uid has changed (it was originally 0) */

	errno = NOERROR;
	rval = tb_setuid( (uid_t) 0 );
	if (rval != SYSERROR || errno != EPERM)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setuid call no.2 has NOT failed with EPERM");
		valrpt(SYSERROR, rval, EPERM, err);
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	DBUG_VOID_RETURN;
}


private	int
tb_setuid(uid)
uid_t	uid;
{
	int	retval;

	retval = setuid(uid);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
