/*
*      SCCS:  @(#)  POSIX.os/procenv/setuid/setuid_t03.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/setuid - setuid_t03 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		setuid_t03 - used by T.setuid in comply()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.setuid to test setuid() by setting
	  the effective UID to the real UID and back again.

  	  Setuid_ou should be a owned by a different user/group to
	  the testset process and should have its set-user-ID and
	  set-group-ID bits set.

	  Usage:
		


METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Fri Feb  3 13:15:36 GMT 1989
MODIFICATIONS:	

		David Sawyer
		Wed Jul 25 13:05:26 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<signal.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<setjmp.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef setuid
#endif


#define	NO_TESTS	7		/* number of tests in testset */
#define	WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();
extern	void	set_pname();

extern	int	vguid2;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	void	subtest3();
private	int	tb_setuid();

public	int
tet_main(argc, argv)		/* Assertion 12 */

	int	argc;
	char	**argv;
{
 
	DBUG_ENTER("subtest3(main)");
	set_pname(argc < 1 || argv[1] == (char *) NULL ?
		"setuid_t03" : (argv[1]));
	if (argc == 4)
	{
		if (argv[3] != (char *) NULL && strncmp("-D", argv[3], 2) == 0)
		{
			vg_dbug = argv[3];
			DBUG_PUSH(&vg_dbug[2]);
		}
		argc--;
	}
	if (argc != 3)
	{
		in_rpt("argument count wrong, got %d, expected 3",argc);
		exit(A_MISC);
	}
	if (argv[2] != (char *) NULL && argv[2][0] != NUL)
		vg_ident = argv[2];

	if (sysconf(_SC_SAVED_IDS) != SYSERROR)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("_POSIX_SAVED_IDS implemented");
		DBUG_EXIT(OK);
	}

	vguid2 = atoi(argv[1]);
	
	if (setprv(PRV_SETID) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_SETID) failed");
		DBUG_EXIT(OK);
	}

	subtest3();

	DBUG_EXIT(OK);

	/* NOTREACHED */
}


private void
subtest3()
{
	uid_t	uid;
	uid_t	euid;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("subtest3");

	errno = NOERROR;
	if (tb_setuid( (uid_t) vguid2 ) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("Setuid call failed with errno = %d (%s)", 
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	uid = getuid();
	if (uid != vguid2)
	{
		xx_rpt(FAILURE);
		in_rpt("Real user ID has not changed");
		in_rpt("UID (%d) not equal to VSX_UID2 (%d)", uid, vguid2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	euid = geteuid();
	if (euid != vguid2)
	{
		xx_rpt(FAILURE);
		in_rpt("Effective user ID has not changed");
		in_rpt("UID (%d) not equal to effective VSX_UID2 (%d)",
			euid, vguid2);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private	int
tb_setuid(uid)
uid_t	uid;
{
	int	retval;

	retval = setuid(uid);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
