/*
*      SCCS:  @(#)  POSIX.os/procenv/sysconf/sysconf.c	Rel 4.4.4	    (07/19/99)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/sysconf - T.sysconf Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.sysconf

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the sysconf() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Fri Feb 24 13:09:21 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 9 May 1990
			Use VSX_INVALID_SC parameter
		
		Geoff Clare, 25 May 1990
			Allow -1 & errno not set in some tests
		
		Geoff Clare, 12 June 1990
			Add tests for _SC_STREAM_MAX and _SC_TZNAME_MAX

		David Sawyer
		Wed Jul 25 14:34:37 BST 1990
		Added resilience code.

		Geoff Clare, 14 June 1991
			Fix path tracing problem in EINVAL test.

		Geoff Clare, 21 Nov 1991
			Test {CHILD_MAX} >= 25 in FIPS mode.
			Test {OPEN_MAX} >= 20 in FIPS mode.

                Geoff Clare, 16 Apr 1992
			Test FIPS requirements in XPG4 mode also.

                Geoff Clare, 14 May 1992
			Add tests using VSX_JOB_CONTROL_SUPP and
			VSX_SAVED_IDS_SUPP.

                Geoff Clare, 12 June 1992
			Test job control and saved ids are always
			supported in FIPS and XPG4 modes.

                Geoff Clare, 11 June 1999
			For `run-time invariant values', return must
			equal the value defined in <limits.h>, except
			in XPG3 mode.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<exitcodes.h>
#include	<signal.h>
#include	<limits.h>
#include	<unistd.h>
#include	<time.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sysconf
#endif


#define 	NO_TESTS	13		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

extern	void	valrpt();
extern	char	*errname();


int	no_tests = NO_TESTS;

private	long	tb_sysconf();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	NULL, 0
};


private void
test1()
{
	long	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test1");
	
	errno = 0;
	rval = tb_sysconf(_SC_ARG_MAX);
	err = errno;

#ifdef	ARG_MAX

	if (
#if TEST_XPG == 3
	    rval < ARG_MAX
#else
	    rval != ARG_MAX
#endif
	   )
	{
		xx_rpt(FAILURE);
#if TEST_XPG == 3
		in_rpt("sysconf(_SC_ARG_MAX) did not return a value >= ARG_MAX");
#else
		in_rpt("sysconf(_SC_ARG_MAX) did not return the value of ARG_MAX");
#endif
		in_rpt("value returned: %ld, ARG_MAX: %ld", rval, (long) ARG_MAX);
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
#if TEST_XPG != 3
		if (rval > 0 || err == 0)
		{
			in_rpt("(ARG_MAX should not be defined in <limits.h> if ");
			in_rpt("the run-time value can vary.)");
		}
#endif
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#else	/* ARG_MAX not defined */

	if (rval < _POSIX_ARG_MAX && !(rval == -1 && err == 0))
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_ARG_MAX) failed expected >= %ld or -1, got %ld",
			(long)_POSIX_ARG_MAX,rval);
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#endif	/* ARG_MAX */

	DBUG_VOID_RETURN; 
} 


private void
test2()
{
#ifndef	CHILD_MAX
	long	expect;
#endif
	long	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	errno = 0;
	rval = tb_sysconf(_SC_CHILD_MAX);
	err = errno;

#ifdef	CHILD_MAX

	/* for FIPS, CHILD_MAX >= 25 is tested elsewhere */
	if (
#if TEST_XPG == 3
	    rval < CHILD_MAX
#else
	    rval != CHILD_MAX
#endif
	   )
	{
		xx_rpt(FAILURE);
#if TEST_XPG == 3
		in_rpt("sysconf(_SC_CHILD_MAX) did not return a value >= CHILD_MAX");
#else
		in_rpt("sysconf(_SC_CHILD_MAX) did not return the value of CHILD_MAX");
#endif
		in_rpt("value returned: %ld, CHILD_MAX: %ld", rval, (long) CHILD_MAX);
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
#if TEST_XPG != 3
		if (rval > 0 || err == 0)
		{
			in_rpt("(CHILD_MAX should not be defined in <limits.h> if ");
			in_rpt("the run-time value can vary.)");
		}
#endif
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#else	/* CHILD_MAX not defined */

#if TEST_FIPS || TEST_XPG > 3
	expect = 25;
#else
	expect = _POSIX_CHILD_MAX;
#endif

	if (rval < expect && !(rval == -1 && err == 0))
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_CHILD_MAX) failed expected >= %ld or -1, got %ld",
			expect, rval);
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#endif	/* CHILD_MAX */

	DBUG_VOID_RETURN; 
} 


private void
test3()
{
	long	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test3");

#ifdef	CLK_TCK

	errno = 0;
	if ((rval = tb_sysconf(_SC_CLK_TCK)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_CLK_TCK) failed, errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

	if (rval != CLK_TCK)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_CLK_TCK) failed expected %ld, got %ld",
			(long)CLK_TCK,rval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

#else	/* CLK_TCK not defined */

	xx_rpt(DELETION);
	in_rpt("CLK_TCK not defined");

#endif	/* CLK_TCK */

	DBUG_VOID_RETURN; 
} 


private void
test4()
{
	long	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test4");

	errno = 0;
	if ((rval = tb_sysconf(_SC_NGROUPS_MAX)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_NGROUPS_MAX) failed, errno %d(%s)",
			err,errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

#ifdef	NGROUPS_MAX

	if (rval < NGROUPS_MAX)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_NGROUPS_MAX) failed expected >= %ld, got %ld",
			(long)NGROUPS_MAX,rval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

#else	/* NGROUPS_MAX not defined */

	xx_rpt(DELETION);
	in_rpt("NGROUPS_MAX not defined");

#endif	/* NGROUPS_MAX */

	DBUG_VOID_RETURN; 
} 


private void
test5()
{
#ifndef	OPEN_MAX
	long	expect;
#endif
	long	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	errno = 0;
	rval = tb_sysconf(_SC_OPEN_MAX);
	err = errno;

#ifdef	OPEN_MAX

	/* for FIPS, OPEN_MAX >= 20 is tested elsewhere */
	if (
#if TEST_XPG == 3
	    rval < OPEN_MAX
#else
	    rval != OPEN_MAX
#endif
	   )
	{
		xx_rpt(FAILURE);
#if TEST_XPG == 3
		in_rpt("sysconf(_SC_OPEN_MAX) did not return a value >= OPEN_MAX");
#else
		in_rpt("sysconf(_SC_OPEN_MAX) did not return the value of OPEN_MAX");
#endif
		in_rpt("value returned: %ld, OPEN_MAX: %ld", rval, (long) OPEN_MAX);
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
#if TEST_XPG != 3
		if (rval > 0 || err == 0)
		{
			in_rpt("(OPEN_MAX should not be defined in <limits.h> if ");
			in_rpt("the run-time value can vary.)");
		}
#endif
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#else	/* OPEN_MAX not defined */

#if TEST_FIPS || TEST_XPG > 3
	expect = 20;
#else
	expect = _POSIX_OPEN_MAX;
#endif

	if (rval < expect && !(rval == -1 && err == 0))
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_OPEN_MAX) failed expected >= %ld or -1, got %ld",
			expect, rval);
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#endif	/* OPEN_MAX */

	DBUG_VOID_RETURN; 
} 


private void
test6()
{
	int	rval;
	int	pathok = 0;

	DBUG_ENTER("test6");
	errno = 0;
	rval = tb_sysconf(_SC_JOB_CONTROL);

#if defined(_POSIX_JOB_CONTROL) || TEST_FIPS || TEST_XPG > 3

	if (rval == SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_JOB_CONTROL) failed");
#if defined(_POSIX_JOB_CONTROL)
		in_rpt("when _POSIX_JOB_CONTROL defined");
#endif
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#else	/* _POSIX_JOB_CONTROL not defined */

	if (errno != 0)
	{
		int err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_JOB_CONTROL) changed errno from 0 to %d(%s)",err,errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#endif	/* JOB_CONTROL */

	DBUG_VOID_RETURN; 
} 


private void
test7()
{
	int	rval;
	int	pathok = 0;

	DBUG_ENTER("test7");
	errno = 0;
	rval = tb_sysconf(_SC_SAVED_IDS);

#if defined(_POSIX_SAVED_IDS) || TEST_FIPS || TEST_XPG > 3

	if (rval == SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_SAVED_IDS) failed");
#if defined(_POSIX_SAVED_IDS)
		in_rpt("when _POSIX_SAVED_IDS defined");
#endif
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#else	/* _POSIX_SAVED_IDS not defined */

	if (errno != 0)
	{
		int err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_SAVED_IDS) changed errno from 0 to %d(%s)",
			err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#endif	/* _POSIX_SAVED_IDS */

	DBUG_VOID_RETURN; 
} 


private void
test8()
{
	long	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test8");

	errno = 0;
	rval = tb_sysconf(_SC_VERSION);
	err = errno;

	if (rval != _POSIX_VERSION)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_VERSION) failed expected %ld, got %ld",
			(long)_POSIX_VERSION,rval);
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN; 
} 

private void
test9()
{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	long	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test9");

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

	errno = 0;
	rval = tb_sysconf(_SC_STREAM_MAX);
	err = errno;

#ifdef	STREAM_MAX

	if (rval != STREAM_MAX)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_STREAM_MAX) did not return the value of STREAM_MAX");
		in_rpt("value returned: %ld, STREAM_MAX: %ld", rval, (long) STREAM_MAX);
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		if (rval > 0 || err == 0)
		{
			in_rpt("(STREAM_MAX should not be defined in <limits.h> if ");
			in_rpt("the run-time value can vary.)");
		}
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#else	/* STREAM_MAX not defined */

	if (rval < _POSIX_STREAM_MAX && !(rval == -1 && err == 0))
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_STREAM_MAX) failed expected >= %ld or -1, got %ld",
			(long)_POSIX_STREAM_MAX,rval);
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#endif	/* STREAM_MAX */

#else
	xx_rpt(NOTINUSE);
	in_rpt("no test in XPG3 mode");
#endif

	DBUG_VOID_RETURN; 
} 

private void
test10()
{
#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3
	long	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test10");

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

	errno = 0;
	rval = tb_sysconf(_SC_TZNAME_MAX);
	err = errno;

#ifdef	TZNAME_MAX

	if (rval != TZNAME_MAX)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_TZNAME_MAX) did not return the value of TZNAME_MAX");
		in_rpt("value returned: %ld, TZNAME_MAX: %ld", rval, (long) TZNAME_MAX);
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
		if (rval > 0 || err == 0)
		{
			in_rpt("(TZNAME_MAX should not be defined in <limits.h> if ");
			in_rpt("the run-time value can vary.)");
		}
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#else	/* TZNAME_MAX not defined */

	if (rval < _POSIX_TZNAME_MAX && !(rval == -1 && err == 0))
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_TZNAME_MAX) failed expected >= %ld or -1, got %ld",
			(long)_POSIX_TZNAME_MAX,rval);
		if (rval < 0)
			in_rpt("errno was set to %d (%s)", err, errname(err));
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

#endif	/* TZNAME_MAX */

#else
	xx_rpt(NOTINUSE);
	in_rpt("no test in XPG3 mode");
#endif

	DBUG_VOID_RETURN; 
} 

private void
test11()
{
	char	*cp;
	int	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test11");

	cp = tet_getvar("VSX_JOB_CONTROL_SUPP");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_JOB_CONTROL_SUPP is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'Y' || *cp == 'y')
	{
		errno = 0;
		rval = tb_sysconf(_SC_JOB_CONTROL);
		if (rval == SYSERROR)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("sysconf(_SC_JOB_CONTROL) failed - errno %d (%s)",
				err, errname(err));
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(1);
		}
		DBUG_VOID_RETURN; 
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		errno = 0;
		rval = tb_sysconf(_SC_JOB_CONTROL);
		if (rval != SYSERROR || errno != 0)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("sysconf(_SC_JOB_CONTROL) did not give expected results");
			valrpt(SYSERROR,rval,0,err);
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(1);
		}
		DBUG_VOID_RETURN; 
	}

	xx_rpt(DELETION);
	in_rpt("parameter VSX_JOB_CONTROL_SUPP must be 'Y' or 'N'");

	DBUG_VOID_RETURN;
} 

private void
test12()
{
	char	*cp;
	int	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test12");

	cp = tet_getvar("VSX_SAVED_IDS_SUPP");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_SAVED_IDS_SUPP is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'Y' || *cp == 'y')
	{
		errno = 0;
		rval = tb_sysconf(_SC_SAVED_IDS);
		if (rval == SYSERROR)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("sysconf(_SC_SAVED_IDS) failed - errno %d (%s)",
				err, errname(err));
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(1);
		}
		DBUG_VOID_RETURN; 
	}
	else if (*cp == 'N' || *cp == 'n')
	{
		errno = 0;
		rval = tb_sysconf(_SC_SAVED_IDS);
		if (rval != SYSERROR || errno != 0)
		{
			err = errno;
			xx_rpt(FAILURE);
			in_rpt("sysconf(_SC_SAVED_IDS) did not give expected results");
			valrpt(SYSERROR,rval,0,err);
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(1);
		}
		DBUG_VOID_RETURN; 
	}

	xx_rpt(DELETION);
	in_rpt("parameter VSX_SAVED_IDS_SUPP must be 'Y' or 'N'");

	DBUG_VOID_RETURN;
} 

private void
test13()
{
	char	*cp;
	int	rval;
	int	sc_invalid = -1;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test13");

	cp = tet_getvar("VSX_INVALID_SC");
	if (cp != NULL && *cp != '\0')
		sc_invalid = atoi(cp);

	errno = 0;
	rval = tb_sysconf(sc_invalid);
	if (rval != SYSERROR || errno != EINVAL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(%d) did not fail as expected", sc_invalid);
		valrpt(SYSERROR,rval,EINVAL,err);
		DBUG_VOID_RETURN; 
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}


	DBUG_VOID_RETURN; 
} 



private	long
tb_sysconf(name)
int	name;
{
	long	retval;

	retval = sysconf(name);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
