/*
*      SCCS:  @(#)  POSIX.os/procenv/sysconf_X/sysconf_X.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/sysconf_X - T.sysconf_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.sysconf_X

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the sysconf() system call, X/Open extensions.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Wed Sep 20 17:56:02 BST 1989
MODIFICATIONS:	

		David Sawyer
		Wed Jul 25 14:37:42 BST 1990
		Added resilience code.

		Geoff Clare, 20 June 1991
			test against _XOPEN_VERSION no longer warning
			in XPG4 mode.

		Neil Moses, 24 October 1991
			Added new tests 2 - 18 for VSX 4.2

		Geoff Clare, 16 April 1992
			Removed tests for _SC_XPG_WIDE_CHAR and
			_SC_ISO_WIDE_CHAR

		Geoff Clare, 17 June 1992
			Re-aligned tests for _SC_2_* with latest XSH4.

		Geoff Clare, 29 Sept 1992
			Align with anticipated resolution of
			interpretation VWG/029/160992.

		Geoff Clare, 16 Dec 1992
			Changed tests 12, 13, 14 & 16 to align with
			published XSH4.

		Geoff Clare, 16 Feb 1995
			Added "feature group FIP" test.

		Geoff Clare, 28 May 1997
			Added LEGACY feature group to FIP test.

************************************************************************/
#include	<std.h>
#include	<stdio.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<exitcodes.h>
#include	<signal.h>
#include	<limits.h>
#include	<unistd.h>
#include	<time.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sysconf
#endif


#define 	NO_TESTS	17		/* number of tests in testset */
#define		WAITTIME	(5 * SPEEDFACTOR)

extern	void	valrpt();
extern	char	*errname();

int	no_tests = NO_TESTS;

private	long	tb_sysconf();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	NULL, 0
};


private void
test1()
{

	int	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test1");
	
	errno = 0;
	if ((rval = tb_sysconf(_SC_PASS_MAX)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_PASS_MAX) failed errno %d(%s)",
			err,errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

#ifdef	PASS_MAX

	if (rval < PASS_MAX)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_PASS_MAX) failed expected >= %d - got %d",
			PASS_MAX,rval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

#else	/* PASS_MAX not defined */

	if (rval < 8)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_PASS_MAX) failed expected >= %d - got %d",
			8, rval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

#endif	/* PASS_MAX */

	DBUG_VOID_RETURN; 
} 


private void
test2()
{
	int	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test2");
	errno = 0;
	if ((rval = tb_sysconf(_SC_XOPEN_VERSION)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_XOPEN_VERSION) failed, errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

#ifdef	_XOPEN_VERSION

	if (rval != _XOPEN_VERSION)
	{
#if TEST_XPG > 3
		xx_rpt(FAILURE);
#else
		xx_rpt(WARNING);
#endif
		in_rpt("sysconf(_SC_XOPEN_VERSION) failed expected %d - got %d",
			_XOPEN_VERSION,rval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

#else	/* _XOPEN_VERSION not defined */

	xx_rpt(DELETION);
	in_rpt("_XOPEN_VERSION not defined");

#endif	/* _XOPEN_VERSION */

	DBUG_VOID_RETURN; 
} 

private void
test3()
{
#if TEST_XPG > 3
	int	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test3");

#if TEST_XPG > 3

	errno = 0;
	if ((rval = tb_sysconf(_SC_BC_BASE_MAX)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_BC_BASE_MAX) failed, errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

#ifdef	BC_BASE_MAX

	if (rval < BC_BASE_MAX)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_BC_BASE_MAX) failed expected %d - got %d",
			BC_BASE_MAX,rval);
	}
	else
		PATH_XS_RPT(1);

#else	/* BC_BASE_MAX not defined */

	xx_rpt(DELETION);
	in_rpt("BC_BASE_MAX not defined");

#endif	/* BC_BASE_MAX */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}

private void
test4()
{
#if TEST_XPG > 3
	int	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test4");

#if TEST_XPG > 3

	errno = 0;
	if ((rval = tb_sysconf(_SC_BC_DIM_MAX)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_BC_DIM_MAX) failed, errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

#ifdef	BC_DIM_MAX

	if (rval < BC_DIM_MAX)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_BC_DIM_MAX) failed expected %d - got %d",
			BC_DIM_MAX,rval);
	}
	else
		PATH_XS_RPT(1);

#else	/* BC_DIM_MAX not defined */

	xx_rpt(DELETION);
	in_rpt("BC_DIM_MAX not defined");

#endif	/* BC_DIM_MAX */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}


private void
test5()
{
#if TEST_XPG > 3
	int	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test5");

#if TEST_XPG > 3

	errno = 0;
	if ((rval = tb_sysconf(_SC_BC_SCALE_MAX)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_BC_SCALE_MAX) failed, errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

#ifdef	BC_SCALE_MAX

	if (rval < BC_SCALE_MAX)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_BC_SCALE_MAX) failed expected %d - got %d",
			BC_SCALE_MAX,rval);
	}
	else
		PATH_XS_RPT(1);

#else	/* BC_SCALE_MAX not defined */

	xx_rpt(DELETION);
	in_rpt("BC_SCALE_MAX not defined");

#endif	/* BC_SCALE_MAX */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}


private void
test6()
{
#if TEST_XPG > 3
	int	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test6");

#if TEST_XPG > 3

	errno = 0;
	if ((rval = tb_sysconf(_SC_BC_STRING_MAX)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_BC_STRING_MAX) failed, errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

#ifdef	BC_STRING_MAX

	if (rval < BC_STRING_MAX)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_BC_STRING_MAX) failed expected %d - got %d",
			BC_STRING_MAX,rval);
	}
	else
		PATH_XS_RPT(1);

#else	/* BC_STRING_MAX not defined */

	xx_rpt(DELETION);
	in_rpt("BC_STRING_MAX not defined");

#endif	/* BC_STRING_MAX */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}


private void
test7()
{
#if TEST_XPG > 3
	int	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test7");

#if TEST_XPG > 3

	errno = 0;
	if ((rval = tb_sysconf(_SC_COLL_WEIGHTS_MAX)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_COLL_WEIGHTS_MAX) failed, errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

#ifdef	COLL_WEIGHTS_MAX

	if (rval < COLL_WEIGHTS_MAX)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_COLL_WEIGHTS_MAX) failed expected %d - got %d",
			COLL_WEIGHTS_MAX,rval);
	}
	else
		PATH_XS_RPT(1);

#else	/* COLL_WEIGHTS_MAX not defined */

	xx_rpt(DELETION);
	in_rpt("COLL_WEIGHTS_MAX not defined");

#endif	/* COLL_WEIGHTS_MAX */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}


private void
test8()
{
#if TEST_XPG > 3
	int	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test8");

#if TEST_XPG > 3

	errno = 0;
	if ((rval = tb_sysconf(_SC_EXPR_NEST_MAX)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_EXPR_NEST_MAX) failed, errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

#ifdef	EXPR_NEST_MAX

	if (rval < EXPR_NEST_MAX)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_EXPR_NEST_MAX) failed expected %d - got %d",
			EXPR_NEST_MAX,rval);
	}
	else
		PATH_XS_RPT(1);

#else	/* EXPR_NEST_MAX not defined */

	xx_rpt(DELETION);
	in_rpt("EXPR_NEST_MAX not defined");

#endif	/* EXPR_NEST_MAX */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}


private void
test9()
{
#if TEST_XPG > 3
	int	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test9");

#if TEST_XPG > 3

	errno = 0;
	if ((rval = tb_sysconf(_SC_LINE_MAX)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_LINE_MAX) failed, errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

#ifdef	LINE_MAX

	if (rval < LINE_MAX)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_LINE_MAX) failed expected %d - got %d",
			LINE_MAX,rval);
	}
	else
		PATH_XS_RPT(1);

#else	/* LINE_MAX not defined */

	xx_rpt(DELETION);
	in_rpt("LINE_MAX not defined");

#endif	/* LINE_MAX */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}


private void
test10()
{
#if TEST_XPG > 3
	int	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test10");

#if TEST_XPG > 3

	errno = 0;
	if ((rval = tb_sysconf(_SC_RE_DUP_MAX)) == SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_RE_DUP_MAX) failed, errno %d(%s)",err,errname(err));
		DBUG_VOID_RETURN; 
	}
	else
		PATH_TRACE;

#ifdef	RE_DUP_MAX

	if (rval < RE_DUP_MAX)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_RE_DUP_MAX) failed expected %d - got %d",
			RE_DUP_MAX,rval);
	}
	else
		PATH_XS_RPT(1);

#else	/* RE_DUP_MAX not defined */

	xx_rpt(DELETION);
	in_rpt("RE_DUP_MAX not defined");

#endif	/* RE_DUP_MAX */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}


private void
test11()
{
#if TEST_XPG > 3
	long	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test11");

#if TEST_XPG > 3

	errno = 0;
	rval = tb_sysconf(_SC_2_VERSION);
	err = errno;

#ifdef	_POSIX2_VERSION

#if _POSIX2_VERSION != -1
	if (rval != _POSIX2_VERSION)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_VERSION) failed expected %ld - got %ld",
			(long)_POSIX2_VERSION,rval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#else
	if (rval != -1 || err != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_VERSION) did not return expected values");
		in_rpt("returned %ld, errno %d (%s)", rval, err, errname(err));
		in_rpt("expected -1, errno 0");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#endif

#else	/* _POSIX2_VERSION not defined */

	if (rval != 199209L && (rval != -1 || err != 0))
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_VERSION) did not return expected values");
		in_rpt("returned %ld, errno %d (%s)", rval, err, errname(err));
		in_rpt("expected return value 199209, or -1 and errno 0");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

#endif	/* _POSIX2_VERSION */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}

private void
test12()
{
#if TEST_XPG > 3
	long	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test12");

#if TEST_XPG > 3

	errno = 0;
	rval = tb_sysconf(_SC_2_C_VERSION);
	err = errno;

#ifdef	_POSIX2_C_VERSION

#if _POSIX2_C_VERSION != -1
	if (rval != _POSIX2_C_VERSION)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_C_VERSION) failed expected %ld - got %ld",
			(long)_POSIX2_C_VERSION,rval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#else
	if (rval != -1 || err != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_C_VERSION) did not return expected values");
		in_rpt("returned %ld, errno %d (%s)", rval, err, errname(err));
		in_rpt("expected -1, errno 0");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#endif

#else	/* _POSIX2_C_VERSION not defined */

	if (rval != 199209L && (rval != -1 || err != 0))
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_C_VERSION) did not return expected values");
		in_rpt("returned %ld, errno %d (%s)", rval, err, errname(err));
		in_rpt("expected return value 199209, or -1 and errno 0");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

#endif	/* _POSIX2_C_VERSION */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}



private void
test13()
{
#if TEST_XPG > 3
	long	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test13");

#if TEST_XPG > 3

	errno = 0;
	rval = tb_sysconf(_SC_2_VERSION);
	err = errno;

#if _XOPEN_XCU_VERSION >= 4

#ifdef _POSIX2_VERSION
	if (rval != _POSIX2_VERSION)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_VERSION) failed expected %ld - got %ld",
			(long)_POSIX2_VERSION,rval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#else /* _POSIX2_VERSION */
	if (rval != 199209L && (rval != -1 || err != 0))
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_VERSION) did not return expected values");
		in_rpt("returned %ld, errno %d (%s)", rval, err, errname(err));
		in_rpt("expected return value 199209, or -1 and errno 0");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#endif /* _POSIX2_VERSION */

#else /* _XOPEN_XCU_VERSION */

	xx_rpt(UNSUPPORTED);
	in_rpt("_XOPEN_XCU_VERSION is not >= 4");

#endif /* _XOPEN_XCU_VERSION */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}


private void
test14()
{
#if TEST_XPG > 3
	long	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test14");

#if TEST_XPG > 3

	errno = 0;
	rval = tb_sysconf(_SC_2_C_VERSION);
	err = errno;

#if _XOPEN_XCU_VERSION >= 4

#ifdef _POSIX2_C_VERSION
	if (rval != _POSIX2_C_VERSION)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_C_VERSION) failed expected %ld - got %ld",
			(long)_POSIX2_C_VERSION,rval);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#else /* _POSIX2_C_VERSION */
	if (rval != 199209L && (rval != -1 || err != 0))
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_C_VERSION) did not return expected values");
		in_rpt("returned %ld, errno %d (%s)", rval, err, errname(err));
		in_rpt("expected return value 199209, or -1 and errno 0");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#endif /* _POSIX2_C_VERSION */

#else /* _XOPEN_XCU_VERSION */

	xx_rpt(UNSUPPORTED);
	in_rpt("_XOPEN_XCU_VERSION is not >= 4");

#endif /* _XOPEN_XCU_VERSION */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}

private void
test15()
{
#if TEST_XPG > 3
	long	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test15");

#if TEST_XPG > 3

	errno = 0;
	rval = tb_sysconf(_SC_2_LOCALEDEF);
	err = errno;

#ifdef	_POSIX2_LOCALEDEF

#if _POSIX2_LOCALEDEF != -1
	if (rval == -1)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_LOCALEDEF) did not return expected values");
		in_rpt("returned -1, errno %d (%s)", err, errname(err));
		in_rpt("expected return value != -1");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#else /* _POSIX2_LOCALEDEF != -1 */
#if _XOPEN_XCU_VERSION >= 4
	xx_rpt(DELETION);
	in_rpt("_POSIX2_LOCALEDEF must not be -1 when _XOPEN_XCU_VERSION is >= 4");
#else
	if (rval != -1 || err != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_LOCALEDEF) did not return expected values");
		in_rpt("returned %ld, errno %d (%s)", rval, err, errname(err));
		in_rpt("expected -1, errno 0");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#endif /* _XOPEN_XCU_VERSION */
#endif /* _POSIX2_LOCALEDEF != -1 */

#else	/* _POSIX2_LOCALEDEF not defined */

#if _XOPEN_XCU_VERSION >= 4
	if (rval == -1)
	{
		in_rpt("sysconf(_SC_2_LOCALEDEF) did not return expected values");
		in_rpt("returned -1, errno %d (%s)", err, errname(err));
		in_rpt("expected return value != -1");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#else /* _XOPEN_XCU_VERSION */
	if (rval == -1 && err != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_LOCALEDEF) did not return expected values");
		in_rpt("returned %ld, errno %d (%s)", rval, err, errname(err));
		in_rpt("expected return value != -1, or -1 and errno 0");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#endif /* _XOPEN_XCU_VERSION */

#endif	/* _POSIX2_LOCALEDEF */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}


private void
test16()
{
#if TEST_XPG > 3
	long	rval;
	int	err;
	int	pathok = 0;
#endif

	DBUG_ENTER("test16");

#if TEST_XPG > 3

	errno = 0;
	rval = tb_sysconf(_SC_2_C_BIND);
	err = errno;

#ifdef	_POSIX2_C_BIND

#if _POSIX2_C_BIND != -1
	if (rval == -1)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_C_BIND) did not return expected values");
		in_rpt("returned -1, errno %d (%s)", err, errname(err));
		in_rpt("expected return value != -1");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#else /* _POSIX2_C_BIND != -1 */
#if _XOPEN_XCU_VERSION >= 4
	xx_rpt(DELETION);
	in_rpt("_POSIX2_C_BIND must not be -1 when _XOPEN_XCU_VERSION is >= 4");
#else
	if (rval != -1 || err != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_C_BIND) did not return expected values");
		in_rpt("returned %ld, errno %d (%s)", rval, err, errname(err));
		in_rpt("expected -1, errno 0");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#endif /* _XOPEN_XCU_VERSION */
#endif /* _POSIX2_C_BIND != -1 */

#else	/* _POSIX2_C_BIND not defined */

#if _XOPEN_XCU_VERSION >= 4
	if (rval == -1)
	{
		in_rpt("sysconf(_SC_2_C_BIND) did not return expected values");
		in_rpt("returned -1, errno %d (%s)", err, errname(err));
		in_rpt("expected return value != -1");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#else /* _XOPEN_XCU_VERSION */
	if (rval == -1 && err != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("sysconf(_SC_2_C_BIND) did not return expected values");
		in_rpt("returned %ld, errno %d (%s)", rval, err, errname(err));
		in_rpt("expected return value != -1, or -1 and errno 0");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#endif /* _XOPEN_XCU_VERSION */

#endif	/* _POSIX2_C_BIND */

#else	/* TEST_XPG <= 3 */

	xx_rpt(NOTINUSE);

#endif	/* TEST_XPG	*/

	DBUG_VOID_RETURN; 
}

#if TEST_XPG > 3

private int
subt17(name, namestr, partallow)
int name;
char *namestr;
int partallow;
{
	char	*suppstr;
	long	rval;
	int	err;

	DBUG_ENTER("subt17");

	errno = 0;
	rval = tb_sysconf(name);
	err = errno;

	if (rval == -1 && err != 0)
	{
		in_rpt("sysconf(%s) failed - errno %d (%s)",
			namestr, err, errname(err));
		DBUG_RETURN(1); 
	}

	if (partallow)
	{
		if (rval == -1)
			suppstr = "not (fully) supported";
		else
			suppstr = "fully supported";
	}
	else
	{
		if (rval == -1)
			suppstr = "not supported";
		else
			suppstr = "supported";
	}

	in_rpt("sysconf(%s) returned %ld", namestr, rval);
	in_rpt("   - indicating the feature group is %s", suppstr);

	DBUG_RETURN(0); 
}

#endif	/* TEST_XPG	*/

private void
test17()
{
#if TEST_XPG > 3
	int fail = 0;
#endif

	DBUG_ENTER("test17");

#if TEST_XPG > 3
	fail += subt17(_SC_2_C_VERSION, "_SC_2_C_VERSION", 1);
	fail += subt17(_SC_XOPEN_ENH_I18N, "_SC_XOPEN_ENH_I18N", 1);
	fail += subt17(_SC_XOPEN_CRYPT, "_SC_XOPEN_CRYPT", 0);
	fail += subt17(_SC_XOPEN_SHM, "_SC_XOPEN_SHM", 0);
#  if TEST_XPG >= 500
	fail += subt17(_SC_XOPEN_LEGACY, "_SC_XOPEN_LEGACY", 1);
#  endif

	if (fail != 0)
		xx_rpt(FAILURE);
	else
		xx_rpt(RFI);
#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN; 
}

private	long
tb_sysconf(name)
int	name;
{
	long	retval;

	retval = sysconf(name);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
