/*
*      SCCS:  @(#)  POSIX.os/procenv/time/time.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/time - T.time Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.time

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the time() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Mon Sep 1 1986
MODIFICATIONS:	D McCardle Fri Jun 12 09:02:26 WET 1987
			Removed abandon(), testid().
			Modified xx_rpt(), xs_rpt().

		David Sawyer
		Wed Jul 25 14:49:45 BST 1990
		Added resilience code.

************************************************************************/
#include	<time.h>
#include	<std.h>
#include	<errno.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef time
#endif


#define START85		(15*60*60*24*365L) /* begining of 1985 */
#define NO_TESTS		2		/* number of tests in testset */

extern	char		*errname();
extern	char		*atocp();
extern	void		valrpt();

int	no_tests = NO_TESTS;

private	int		testfail;
private char		*testcomment;
private time_t		tm_start;

private	time_t		tb_time();
private void		test1();
private void		test2();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private void
test1()
{
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;
	testcomment = "time returns the current time";
	tm_start = START85;

	globok = 0;
	tm_start = tb_time((time_t *) 0, (time_t) NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	globok = 0;
	(void) tb_time((time_t *) 0, (time_t) NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test2()
{
	time_t	tm_value;
	int	pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;
	testcomment = "time sets the current time in the location tloc";
	tm_start = START85;

	globok = 0;
	(void) tb_time(&tm_value, (time_t) NOERROR, NOERROR);
	PATH_FUNC_TRACE;

	if (!testfail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private	time_t
tb_time(tbtloc, expret, experrno)
time_t	*tbtloc;
time_t expret;
int experrno;
{
	time_t	rval;
	int	err;
	int	failed = 0;
	int	pathok = 0;

	DBUG_ENTER("tb_time");

	errno = 0;
	rval = time(tbtloc);
#ifdef	TESTBED
	errno++;
	rval = -1;
#endif
	err = errno;

	if (expret != 0L)
	{
		if (expret == rval && experrno == err)
		{
			PATH_FUNC_RPT(0);
			DBUG_RETURN(rval);
		}
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("time(%x) failed", tbtloc);
		if (testcomment != (char *) NULL)
			in_rpt(testcomment);
		valrpt((int) expret, (int) rval, experrno, err);
		DBUG_RETURN(rval);
	}
	else
		PATH_TRACE;
	
	if ((tbtloc != (time_t *) NULL) && (rval != *tbtloc))
		failed++;
	else
		PATH_TRACE;

	if (rval < tm_start)
		failed++;
	else
		PATH_TRACE;

	if (failed)
	{
		if (!testfail++)
			xx_rpt(FAILURE);	
		in_rpt("time(%x) failed", tbtloc);
		if (testcomment != (char *) NULL)
			in_rpt(testcomment);
		if (rval == SYSERROR)
			valrpt((int) expret, (int) rval, experrno, err);
		else
		{
			in_rpt("time gave unacceptable value");
			in_rpt("Acceptable range: from %ld", tm_start);
			in_rpt("Observed value: %ld", rval);
			if (tbtloc != (time_t *) NULL)
				in_rpt("Buffer value: %ld", *tbtloc);
		}
	}
	else
		PATH_FUNC_RPT(3);

	DBUG_RETURN(rval);
}
