/*
*      SCCS:  @(#)  POSIX.os/procenv/times/times.c	Rel 4.4.3     (07/20/98) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/times - T.times Rel 4.4.3 (07/20/98)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.times

PROJECT:	VSX (X/OPEN Validation Suite)


MODULE CONTAINS:

SYNOPSIS:

DESCRIPTION:
	Testset for the times() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Fri Feb 10 15:14:27 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 27 Oct 1989
			Make waste_time() do less processing.
			Tidying up.
		
		Geoff Clare, 11 Dec 1989
			Use sys_call() routine from userintf.c instead
			of assuming stat() will use system CPU time.
		
		Geoff Clare, 2 Jan 1990
			Use use_time() routine to accumulate user CPU time.
		
		Geoff Clare, 24 Apr 1990
			Allow for 1 second sleep() resolution in test1.
		
		Geoff Clare, 28 June 1990
			Align with POSIX.3 draft 11

		David Sawyer
		Wed Jul 25 15:42:57 BST 1990
		Added resilience code.

		Geoff Clare, 15 Feb 1991
			Allow for 1 second sleep() resolution in test7.

		Stuart Boutell, 12 Apr 1991
			Align with POSIX.3.1 draft 11.8 (Modified test 5,6)

		Geoff Clare, 16 Dec 1992
			Allow for rounding in test5.

		Geoff Clare, 17 Sept 1997
			Allow for rounding in test6.

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<sys/types.h>
#include	<sys/wait.h>
#include	<unistd.h>
#include	<signal.h>
#include	<report.h>
#include	<sys/times.h>
#include	<exitcodes.h>
#include	<synch.h>
#include	<string.h>
#include	<setjmp.h>
#include	<time.h> 
#include	<stdlib.h> 
#include	<timeout.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef times
#endif


#define	NO_TESTS	7	/* number of tests in test set */
#define	NOK		2	/* child exit value if non fatal error occured */
#define PATH_OK		4	/* path checking error in child */
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	COUNT_UT	(1000 / SPEEDFACTOR)	/* count for user time wasting */
#define	COUNT_ST	(10000 / SPEEDFACTOR)	/* count for system time wasting */

#define	VSX_CLOCK_ERR	"VSX_CLOCK_ERR"
#define	DEFAULT_ACCURACY	5	/*default accuarcy for tests */
#define	CONTINUE	1	/* packet passed to tell child to continue */
#define	READ	0
#define	WRITE	1
#define	INIT	0
#define	FIN	1
#define	MED	2


extern	int	alrm_flag;
extern	char	*errname();
extern	char	*signame();
extern	void	use_time();


int	no_tests = NO_TESTS;

private	int	fd[2];
private	synch	*syn;
private	int	testfail;
private	long	accuracy;
private	int	t_mode;

private	void	test1();
private	void	test2();
private	void	ch_t2();
private	void	pa_t2();
private	void	test3();
private	void	test4();
private	void	test5();
private	void	ch_t5();
private	void	gch_t5();
private	void	gpa_t5and6();
private	void	test6();
private	void	ch_t6();
private	void	gch_t6();
private	void	test7();
private	void	setup();
private	int	waste_time();
private	clock_t	tb_times();
private	int	wait_sync();
private int	send_sync();


public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	NULL, 0
};


private	void
setup()
{
	char	*cp;
	DBUG_ENTER("setup");

	if ((cp = tet_getvar(VSX_CLOCK_ERR)) == (char *) NULL)
		accuracy = (long) DEFAULT_ACCURACY;
	else
		accuracy = (long) abs(atoi(cp));

	DBUG_VOID_RETURN;
}


private	void
test1()
{
	struct	tms	buf;
	time_t	kron[2],	delta_kron;
	clock_t	clck[2],	delta_clck;
	float	range;
	long	clk_tck;
	int	pathok = 0;

	DBUG_ENTER("test1");

	if ((clk_tck = sysconf(_SC_CLK_TCK)) == (long) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sysconf() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((clck[INIT] = tb_times(&buf)) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((kron[INIT] = time((time_t *) NULL)) == (time_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: time() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(20);

	if ((clck[FIN] = tb_times(&buf)) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((kron[FIN] = time((time_t *) NULL)) == (time_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: time() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	delta_kron = kron[FIN] - kron[INIT];
	delta_clck = clck[FIN] - clck[INIT];
	range = ((float) accuracy/100.0) * ((float)delta_kron * clk_tck) / 2.0;
	if ((float)delta_clck < (float)(delta_kron - 1) * clk_tck - range)
	{
		xx_rpt(FAILURE);
		in_rpt("change in elapsed time measured in CLK_TCKs is outside required accuracy");
		in_rpt("change in elapsed time is %f", (float) delta_clck);
		in_rpt("least acceptable value is %f", 
			(float) (delta_kron - 1) * clk_tck - range);
	}
	else if ((float)delta_clck > (float)(delta_kron + 1) * clk_tck + range)
	{
		xx_rpt(FAILURE);
		in_rpt("change in elapsed time measured in CLK_TCKs is outside required accuracy");
		in_rpt("change in elapsed time is %f", (float) delta_clck);
		in_rpt("greatest acceptable value is %f", 
			(float) (delta_kron + 1) * clk_tck + range);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(6);
	}

	DBUG_VOID_RETURN;
}


private	int
waste_time()
{
	int	count;

	DBUG_ENTER("waste_time");
	for (count = 0; count < COUNT_UT; count++)
	{
		use_time();
	}

	for (count = 0; count < COUNT_ST; count++)
	{
		if (sys_call() == SYSERROR)
		{
			if (!testfail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: sys_call() failed");
			DBUG_RETURN(SYSERROR);
		}
	}

	DBUG_RETURN(OK);
}


private	void
test2()
{

	DBUG_ENTER("test2");

	if ((syn = opensync()) == (synch *) NULL)
	{
		xx_rpt(DELETION);
		in_rpt("opensync() failed");
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t2, pa_t2, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private	void
pa_t2()
{

	struct	tms	buf[2];
	char	mess;
	int	pathok = 0;

	DBUG_ENTER("pa_t2");

	testfail = 0;
	if (tb_times(&buf[INIT]) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;


	DBUG_PRINTF("pa_t2","waiting on child");
	globok = 0;
	if (wait_sync(syn[0][READ], &mess, sizeof(mess), 4*WAITTIME)
		== SYSERROR || mess != CONTINUE)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: wait_sync failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (tb_times(&buf[FIN]) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (buf[FIN].tms_cutime - buf[INIT].tms_cutime != 0)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("value for child user time has changed");
		in_rpt("child user time initially %d",buf[INIT].tms_cutime);
		in_rpt("child user time finially %d",buf[FIN].tms_cutime);
		testfail++;
	}
	else
		PATH_TRACE;

	if (buf[FIN].tms_cstime - buf[INIT].tms_cstime != 0)
	{
		if (!testfail++)
			xx_rpt(FAILURE);
		in_rpt("value for child system time has changed"); 
		in_rpt("child system time initially %d",buf[INIT].tms_cstime);
		in_rpt("child system time finially %d",buf[FIN].tms_cstime);
		testfail++; 
	} 
	else
		PATH_TRACE;

	if (!testfail)
	{
		PATH_XS_RPT(5);
	}

	DBUG_VOID_RETURN;
}


private	void
ch_t2()
{
	char	mess;

	DBUG_ENTER("ch_t2");

	if (waste_time() == SYSERROR)
	{
		DBUG_EXIT(A_STATFAIL);
	}

	DBUG_PRINTF("ch_t2","sending to parent");

	mess = CONTINUE; 
	if (send_sync(syn[0][WRITE], &mess, sizeof(mess), WAITTIME) == SYSERROR)
		DBUG_EXIT(A_SENDSYNC);

	DBUG_VOID_RETURN;
}


private void
test3()
{
	int		count;
	struct	tms	buf;
	clock_t	ptime;
	int	pathok = 0;

	DBUG_ENTER("test3");
	if (tb_times(&buf) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ptime = buf.tms_utime;

	for (count = 0; count < COUNT_UT; count++)
	{
		if (count==0)
			PATH_TRACE;
		use_time();
	}

	if (tb_times(&buf) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (buf.tms_utime <= ptime)
	{
		xx_rpt(FAILURE);
		in_rpt("tms_utime (as returned by times()) does not seem to change");
		in_rpt("observed current time: %ld, observed past time: %ld",
				(long) buf.tms_utime, (long) ptime);
		(void) tet_delete(5, "depends on success of test 3");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}
	
	DBUG_VOID_RETURN;
}

private void
test4()
{
	int		count;
	struct	tms	buf;
	clock_t	ptime;
	int	pathok = 0;

	DBUG_ENTER("test4");
	if (tb_times(&buf) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ptime = buf.tms_stime;

	for (count = 0; count < COUNT_ST; count++)
	{
		if (count==0)
			PATH_TRACE;
		if (sys_call() == SYSERROR)
		{
			xx_rpt(DELETION);
			in_rpt("deletion reason: sys_call() failed");
			DBUG_VOID_RETURN;
		}
	}

	if (tb_times(&buf) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (buf.tms_stime <= ptime)
	{
		xx_rpt(FAILURE);
		in_rpt("observed current time: %ld, observed past time: %ld",
				(long) buf.tms_stime, (long) ptime);
		in_rpt("(after %d sys_call() calls", COUNT_ST);
		(void) tet_delete(6, "depends on success of test 4");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}
	
	DBUG_VOID_RETURN;
}


private void
test5()
{
	int	pathok = 0;

	DBUG_ENTER("test5");

	testfail = 0;

 	t_mode = 0;
	
	while (t_mode < 2)
	{
		clock_t starttime;
		clock_t	child_time;
		struct	tms	buf;
		int 	retval;

		DBUG_PRINTF("test5","pass %d", t_mode);

		if (pipe(fd) == SYSERROR)
		{
			if(!testfail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: pipe() failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (tb_times(&buf) == (clock_t) SYSERROR)
		{
			if(!testfail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: times() returns error");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		starttime = buf.tms_cutime;

		globok = 0;
		retval = cppair(ch_t5, NULLFN, 3*WAITTIME, 0 | PATH_OK | NOK);
		if (retval & NOK)
		{
			DBUG_VOID_RETURN;
		}
		else
		{
			if (retval & PATH_OK)
				PATH_TRACE;
		}

		(void) close(fd[1]);
		if (read(fd[0], (char *) (&child_time), sizeof(clock_t)) != 
			sizeof(clock_t))
		{
			if(!testfail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: read from pipe failed"); 
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (tb_times(&buf) == (clock_t) SYSERROR)
		{
			if(!testfail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: times() returns error");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* The "-1" is to allow for rounding, in case the system maintains
		   the times at higher precision than clock ticks */
		if (buf.tms_cutime - starttime < child_time-1)
		{
			if(!testfail)
				xx_rpt(FAILURE);
			in_rpt("When using waitpid() to wait for child process,");
			in_rpt("observed cutime increase: %ld",
				(long)(buf.tms_cutime - starttime));
			in_rpt("observed child utime: %ld", (long) child_time);
			testfail++;
		}
		else
			PATH_TRACE;

		(void) close(fd[0]);
		t_mode++;
	}

	if(!testfail)
		PATH_XS_RPT(12);

	DBUG_VOID_RETURN;
}
	

private	void
ch_t5()
{
	int		count;
	clock_t	ptime;
	clock_t	child_time;
	struct	tms	buf;
	int	pathok = 0;

	DBUG_ENTER("ch_t5");
	(void) close(fd[0]);
	if (tb_times(&buf) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	
	ptime = buf.tms_utime;
	
	for (count = 0; count < COUNT_UT; count++)
	{
		if (count==0)
			PATH_TRACE;
		use_time();
	}

	globok = 0;
	(void) cpsig(gch_t5, gpa_t5and6, WAITTIME, 0);
	PATH_FUNC_TRACE;

	if (tb_times(&buf) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	
	child_time = buf.tms_utime - ptime + buf.tms_cutime;

	if (write(fd[1], (char *) (&child_time), sizeof(clock_t)) != sizeof(clock_t))
	{
		DBUG_EXIT(A_WRITEFAIL);
	}
	else
		PATH_TRACE;

	(void) close(fd[1]);

	if (pathok == 5)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}

private	void
gch_t5()
{
	int		count;

	DBUG_ENTER("gch_t5");

	for (count = 0; count < COUNT_UT; count++)
		use_time();

	DBUG_VOID_RETURN;
}

private void
gpa_t5and6()
{
	int status;
	int err;
	int ret;
	int pathok = 0;

	DBUG_ENTER("gpa_t5and6");
	SET_TIMEOUT(WAITTIME);

	errno = NOERROR;
	/* Wait test */
	if(t_mode == 0)
	{
		DBUG_PRINTF("gpa_t5and6","wait() test");
		ret = wait(&status);
		PATH_TRACE;
	}
	else
	{
		DBUG_PRINTF("gpa_t5and6","waitpid() test");
		ret = waitpid(tet_child,&status,WUNTRACED);
		PATH_TRACE;
	}
	err = errno; 

	CLEAR_ALARM;

	/* check child wait status shows valid exit code, if not
	   report wait status and give UNRESOLVED result */

#ifdef TESTBED2
	ret = -1;
#endif

	if (ret == -1)
	{
		if (!testfail)
			xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt( "child process timed out" );
		else
			in_rpt("%s() failed - errno %d (%s)",
				(t_mode == 0?"wait":"waitpid"),
				err, errname(err));
		(void) kill(tet_child, SIGKILL);
		DBUG_EXIT(NOK);
	}
	else if (WIFSIGNALED(status))
	{
		status = WTERMSIG(status);
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("child process was terminated by signal %d (%s)",
			status, signame(status));
		DBUG_EXIT(NOK);
	}
	else if (WIFSTOPPED(status))
	{
		status = WSTOPSIG(status);
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("child process was stopped by signal %d (%s)",
			status, signame(status));
		(void) kill(tet_child, SIGKILL);
		DBUG_EXIT(NOK);
	}
	else if (WIFEXITED(status))
	{
		PATH_TRACE;
		PATH_FUNC_RPT(2);
	}
	else
	{
		if (!testfail)
			xx_rpt(DELETION);
		in_rpt("child process returned bad wait status (%#x)", status);
		DBUG_EXIT(NOK);
	}

	DBUG_VOID_RETURN;
}

private void
test6()
{
	int		pathok = 0;

	DBUG_ENTER("test6");

	testfail = 0;
	t_mode = 0;
	while (t_mode < 2)
	{
		clock_t		starttime;
		clock_t		child_time;
		struct	tms	buf;
		int		retval;

		if (pipe(fd) == SYSERROR)
		{
			if(!testfail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: read from pipe failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (tb_times(&buf) == (clock_t) SYSERROR)
		{
			if(!testfail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: times() returns error");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		starttime = buf.tms_cstime;

		retval = cppair(ch_t6, NULLFN, 3*WAITTIME, 0 | PATH_OK | NOK);
		if (retval & NOK)
		{
			DBUG_VOID_RETURN;
		}
		else
		{
			if (retval & PATH_OK)
				PATH_TRACE;
		}

		(void) close(fd[1]);

		if (read(fd[0], (char *) (&child_time), sizeof(clock_t)) != 
			sizeof(clock_t))
		{
			if(!testfail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: read from pipe failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		if (tb_times(&buf) == (clock_t) SYSERROR)
		{
			if(!testfail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: times() returns error");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* The "-1" is to allow for rounding, in case the system maintains
		   the times at higher precision than clock ticks */
		if (buf.tms_cstime - starttime < child_time-1)
		{
			if(!testfail)
				xx_rpt(FAILURE);
			in_rpt("observed cstime increase: %ld",
				(long)(buf.tms_cstime - starttime));
			in_rpt("observed child stime: %ld", (long) child_time);
			testfail++;
		}
		else
			PATH_TRACE;

		(void) close(fd[0]);
		t_mode ++;
	}

	if(!testfail)
		PATH_XS_RPT(12);

	DBUG_VOID_RETURN;
}
	
private	void
ch_t6()
{
	int		count;
	clock_t		ptime;
	clock_t		child_time;
	struct	tms	buf;
	int	pathok = 0;

	DBUG_ENTER("ch_t6");

	(void) close(fd[0]);

	if (tb_times(&buf) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	
	ptime = buf.tms_stime;
	
	for (count = 0; count < COUNT_ST; count++)
	{
		if (count==0)
			PATH_TRACE;
		if (sys_call() == SYSERROR)
			DBUG_EXIT(A_STATFAIL);
	}

	globok = 0;
	(void) cpsig(gch_t6, gpa_t5and6, WAITTIME, 0);
	PATH_FUNC_TRACE;

	if (tb_times(&buf) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_EXIT(NOK);
	}
	else
		PATH_TRACE;
	
	child_time = buf.tms_stime - ptime + buf.tms_cstime;

	if (write(fd[1], (char *) (&child_time), sizeof(clock_t)) != sizeof(clock_t))
	{
		DBUG_EXIT(A_WRITEFAIL);
	}
	else
		PATH_TRACE;

	(void) close(fd[1]);

	if (pathok == 5)
		DBUG_EXIT(OK | PATH_OK);

	DBUG_VOID_RETURN;
}

private	void
gch_t6()
{
	int		count;

	DBUG_ENTER("gch_t6");

	for (count = 0; count < COUNT_ST; count++)
		if (sys_call() == SYSERROR)
			DBUG_EXIT(A_STATFAIL);

	DBUG_VOID_RETURN;
}


private	void
test7()
{
	struct	tms	buf;
	clock_t	clck[3],	delta_clck1,	delta_clck2;
	float	range;
	long	clk_tck;
	int	pathok = 0;

	DBUG_ENTER("test7");

	if ((clk_tck = sysconf(_SC_CLK_TCK)) == (long) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sysconf() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((clck[INIT] = tb_times(&buf)) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(20);

	if ((clck[MED] = tb_times(&buf)) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sleep(20);

	if ((clck[FIN] = tb_times(&buf)) == (clock_t) SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() returns error");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	delta_clck1 = clck[FIN] - clck[INIT];
	delta_clck2 = clck[MED] - clck[INIT];
	range = clk_tck + ((float) accuracy/100.0) * (float)delta_clck1 / 2.0;

	if ((float)delta_clck2 < (float)(delta_clck1 - range)/2.0)
	{
		xx_rpt(FAILURE);
		in_rpt("change in elapsed time is outside required accuracy");
		in_rpt("change in elapsed time is %f", (float) delta_clck2);
		in_rpt("least acceptable value is %f", 
			(float)(delta_clck1 - range)/2.0);
	}
	else if ((float)delta_clck2 > (float)(delta_clck1 + range)/2.0)
	{
		xx_rpt(FAILURE);
		in_rpt("change in elapsed time measured in CLK_TCKs is outside required accuracy");
		in_rpt("change in elapsed time is %f", (float) delta_clck2);
		in_rpt("greatest acceptable value is %f", 
			(float)(delta_clck1 + range)/2.0);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(5);
	}

	DBUG_VOID_RETURN;
}

private	clock_t
tb_times(buffer)
struct	tms	*buffer;
{
	clock_t	retval;

	retval = times(buffer);
#ifdef	TESTBED
	errno++;
	return (clock_t) -1;
#else
	return retval;
#endif
}


private	int
wait_sync(chan , buf , size , delay)
int	chan;
char	*buf;
unsigned	size;
unsigned	delay;
{
	int	rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("wait_sync");

	rval = waitsync(chan, buf, size, delay);
	if (rval == SYSERROR && errno == EINTR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitsync timed out");
		DBUG_RETURN(SYSERROR);
	}
	else if (rval == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("waitsync() on fd %d failed: %d(%s)", chan, err, 
			errname(err));
		in_rpt("deletion reason: waitsync failed");
		DBUG_RETURN(SYSERROR);
	}
	else if (rval != size)
	{
		xx_rpt(DELETION);
		in_rpt("waitsync() didnh not recieve entire packet over pipe, expected %d - got %d", size, rval);
		in_rpt("deletion reason: waitsync failed");
		DBUG_RETURN(SYSERROR);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_RETURN(NOERROR);
}


private	int
send_sync(chan , buf , size , delay)
int	chan;
char	*buf;
unsigned	size;
unsigned	delay;
{
	int	rval;
	int	err;

	DBUG_ENTER("send_sync");

	rval = sendsync(chan, buf, size, delay);
	if (rval == SYSERROR && errno == EINTR)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sendsync timed out");
		DBUG_RETURN(SYSERROR);
	}
	else if (rval == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sendsync() on fd %d failed: %d(%s)", chan, err, 
			errname(err));
		in_rpt("deletion reason: sendsync failed");
		DBUG_RETURN(SYSERROR);
	}
	else if (rval != size)
	{
		xx_rpt(DELETION);
		in_rpt("sendsync() did not send entire packet over pipe, expected %d - sent %d", size, rval);
		in_rpt("deletion reason: sendsync failed");
		DBUG_RETURN(SYSERROR);
	}

	DBUG_RETURN(NOERROR);
}
