/*
*      SCCS:  @(#)  POSIX.os/procenv/ttyname/ttyname.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/ttyname - T.ttyname Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.ttyname

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the ttyname() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Mon Feb 13 18:15:49 GMT 1989
MODIFICATIONS:	

		David Sawyer
		Wed Jul 25 15:49:29 BST 1990
		Added resilience code.

		Peter Skan, 1 Oct 1991
			set the PRV_DEVICE privilege

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<limits.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<string.h>
#include	<unistd.h>
#include	<signal.h>
#include	<timeout.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef ttyname
#endif


#define NO_TESTS	2		/* number of tests in testset */
#define	INVALID_ID	-1		/* default invalid user id */
#define	WAITTIME	(5*SPEEDFACTOR)

extern	char	*atocp();
extern	void	valrpt();
extern	char	*errname();


int	no_tests = NO_TESTS;

private	char	*tb_ttyname();
private void	test1();
private void	test2();

private	char	*t2_file	=	"./ttyname-t.2";

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};


private void
test1()
{
	int	fd;
	char	*path;
	char	*rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	if ((path = tet_getvar("VSX_TTYNAME")) == (char *) NULL ||
		*path=='\0')
	{
		xx_rpt(DELETION);
		in_rpt("VSX_TTYNAME not defined/is null");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;

	if (setprv(PRV_DEVICE) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("setprv(PRV_DEVICE) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WAITTIME)
	fd = open(path, O_RDONLY);
	err = errno;
	CLEAR_ALARM

	if(fd == (int) SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("open terminal %s failed %d(%s)",path, err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if((rval = tb_ttyname(fd)) == (char *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("ttyname has failed %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (strcmp(path , rval) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("ttyname() has returned an incorrect device name - expected %s, got %s",path,rval);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	fd;
	char	*rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test2");

	errno = 0;
	if((fd = open("/dev/null",O_RDONLY)) == (int) SYSERROR)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("open(/dev/null, O_RDONLY)  has failed %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if((rval = tb_ttyname(fd)) != (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("ttyname() did not return (char *) NULL - got %s",rval);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	fd = creat(t2_file, (mode_t)S_IRWXU|S_IRWXG|S_IRWXO);
	(void) close(fd);

	errno = 0;
	if((rval = tb_ttyname(fd)) != (char *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("ttyname() did not return (char *) NULL - got %s",rval);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(3);
	(void) unlink(t2_file);

	DBUG_VOID_RETURN;
}

private	char	*
tb_ttyname(fd)
int	fd;
{
	char	*rval;

	errno = 0;
	rval = ttyname(fd);
#ifdef	TESTBED
	errno++;
	rval = -1;
#endif

	return(rval);
}
