/*
*      SCCS:  @(#)  POSIX.os/procenv/uname/uname.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procenv/uname - T.uname Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.uname

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the uname() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Andrew Twigger, UniSoft Ltd.
DATE CREATED:	Mon Feb 13 15:43:03 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 26 Apr 1990
			Add null-termination test

		David Sawyer
		Wed Jul 25 16:10:48 BST 1990
		Added resilience code.

************************************************************************/
#include	<std.h>
#include	<stdlib.h>
#include	<errno.h>
#include	<limits.h>
#include	<sys/utsname.h>
#include	<dbug.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef uname
#endif


#define NO_TESTS		2		/* number of tests in testset */

extern	void	valrpt();
extern	char	*errname();

int	no_tests = NO_TESTS;

private int		testfail;

private	int		tb_uname();
private	void		checknull();
private void		test1();
private void		test2();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private void
test1()
{
	struct utsname	namestruct;
	int	pathok = 0;

	DBUG_ENTER("test1");

	testfail = 0;

	(void) tb_uname(&namestruct, NOERROR, NOERROR);

	if (testfail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (pathok == 1)
		xx_rpt(RFI);
	else
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: path check value %d", pathok);
	}

	in_rpt("system name - %.*s", sizeof(namestruct.sysname), namestruct.sysname);
	in_rpt("node name   - %.*s", sizeof(namestruct.nodename), namestruct.nodename);
	in_rpt("release     - %.*s", sizeof(namestruct.release), namestruct.release);
	in_rpt("version     - %.*s", sizeof(namestruct.version), namestruct.version);
	in_rpt("machine     - %.*s", sizeof(namestruct.machine), namestruct.machine);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	struct utsname	names;
	int		err;
	int		pathok = 0;

	DBUG_ENTER("test2");

	testfail = 0;

	if (uname(&names) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("uname() failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	checknull(names.sysname,  sizeof(names.sysname),  "sysname");
	PATH_FUNC_TRACE;

	globok = 0;
	checknull(names.nodename, sizeof(names.nodename), "nodename");
	PATH_FUNC_TRACE;

	globok = 0;
	checknull(names.release,  sizeof(names.release),  "release");
	PATH_FUNC_TRACE;

	globok = 0;
	checknull(names.version,  sizeof(names.version),  "version");
	PATH_FUNC_TRACE;

	globok = 0;
	checknull(names.machine,  sizeof(names.machine),  "machine");
	PATH_FUNC_TRACE;

	if (testfail == 0)
			PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}


private void
checknull(member, len, name)
char *member;
size_t len;
char *name;
{
	int i;
	int pathok = 0;

	DBUG_ENTER("checknull");

	for (i = 0; i < len ; i++)
	{
		if (i==0)
			PATH_TRACE;
		if (member[i] == '\0')
			break;
	}

#ifdef TESTBED2
	i = len;
#endif
	if (i >= len)
	{
		if (testfail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("Array not null terminated: utsname.%s", name);
		in_rpt("sizeof utsname.%s: %lu", name, (unsigned long)len);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private	int
tb_uname(name, expret, experrno)
struct	utsname	*name;
int expret, experrno;
{
	int	rval;
	int	err;

	errno = 0;
	rval = uname(name);
#ifdef	TESTBED
	errno++;
	rval = -1;
#endif
	err = errno;

	if (expret == 0)
	{
		if (rval >= 0)
			return(rval);
	}
	else
	{
		if (expret == rval && experrno == err)
			return(rval);
	}

	if (!testfail++)
		xx_rpt(FAILURE);
	in_rpt("uname(%x) failed", name);
	valrpt(expret, rval, experrno, err);

	return(rval);
}
