/*
*      SCCS:  @(#)POSIX.os/procprim/_exit/_exit.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/_exit - T._exit Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T._exit

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the _exit() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Mon Apr  3 10:13:44 GMT 1989
MODIFICATIONS:
		Geoff Clare, 17 July 1989
			For tests requiring a background process group, if
			setpgid() is not supported, use setsid() instead.
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 16:11:16 BST 1989
			Remove references to SA_NOCLDSTOP in test7
			Wait for child process in test7 (was leaving zombie)
			Add exit codes with more than 8 bits set - test3

		Geoff Clare, 5 Oct 1989
			Test both ways of creating orphaned process group
			with stopped process.

		Geoff Clare, 14 Feb 1990
			Add second sleep() in ch_sleep()

		Geoff Clare, 7 June 1990
			UNSUPPORTED result if VSX_TERMIOS_TTY=unsup

		Geoff Clare, 22 June 1990
			Add sleep() in ch_stop()

		Peter Lee, Unisoft Ltd.,
		Thu Aug 09 11:16:41 BST 1990
			Added resilience code.

		Stuart Boutell, 12 Nov 1990
			Added test 11 code.
			Cleaned up tet_child references.
			uids() return value check added

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called

		Stuart Boutell, 20 Mar 1991
			Deleted test 5 for _exit in a process having no
			effect on child processes
			Inserted new test 5 for _exit relinquishing the
			controlling tty.

		Stuart Boutell, 30 Apr 1991
			Changed cpsig() return code check from FAIL to
			SYSERROR

		Geoff Clare, 22 Nov 1991
			Changes to support pseudo-terminals.

		Geoff Clare, 16 June 1993
			Test 11 not in use in POSIX mode.

		Geoff Clare, 20 Dec 1993
			Don't use DBUG_EXIT() in children of plain fork().
			Use TEST_EXIT() macro for tested _exit() calls.

		Geoff Clare, 19 July 1995
			Reduce WAITTIME when testing XNFS.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<sys/wait.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<termios.h>
#include	<fcntl.h>
#include	<vsx_signal.h>
#include	<setjmp.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include 	<xnfs.h>
#include 	"_exit_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef _exit
#endif


/* When testing XNFS, SPEEDFACTOR is increased in <std.h> to allow extra
   time for NFS file accesses.  However, this can cause some tests here
   to take longer than the auto-sync timeout on the server side.
   To avoid this the "normal" value is used by undoing the increase. */
#if TEST_XNFS
#define	WAITTIME		(5 * SPEEDFACTOR/4)
#else
#define	WAITTIME		(5 * SPEEDFACTOR)
#endif

#define NAP			4
#define NOK			02
#define PATH_NOK		04
#define	NO_TESTS		12		/* number of tests in testset */
#define	READ			0
#define	WRITE			1
#define	CREATMODE		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	WAIT			1
#define	WAITPID			2
#define	TEST_SUCCESS		1
#define	TEST_FAIL		2
#define	TEST_SIGNAL		3
#define	TEST_SIGACTION		4
#define	TEST_SETPGID		5
#define	TEST_KILL		6
#define	TEST_PPID		7
#define	BYTEMASK		0xff
#define	TWOBYTEMASK		0xffff
#define	ATEXIT_FAILED		1
#define	ATEXIT_CALLED		2
#define NO			0
#define YES			1

#if TEST_XNFS && !defined(DTET1)
#define TEST_EXIT(x)		(tet_logoff(), _exit(x))
#else
#define TEST_EXIT(x)		_exit(x)
#endif

extern	void		vsx_sigs();
extern	void		valrpt();
extern	char		*status_child();
extern	char		*errname();
extern	void		setsigs();

extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sg_code;	/* defined in wait_child.c */
extern	int		ex_code;	/* defined in wait_child.c */

int	no_tests = NO_TESTS;

#if TEST_XNFS
private	int		snames[] = {1, 0};
#endif /* TEST_XNFS */

private	sigjmp_buf	sjmp;

#if !TEST_XNFS
private	int	tty_fildes;
private	struct termios tty_termios,
		*tty_termios_p = &tty_termios;
private	int	loop_fildes;
private	struct termios loop_termios,
		*loop_termios_p = &loop_termios;
#endif /* TEST_XNFS */
private	char		*t2_lock = "_exit-t.2lock";
private	int		pipe_fd[2];
private	int		fg_fd[2];
private	int		bg_fd[2];
private	int		lock_fd;
private	struct flock 	lock_info;
private	sigset_t	sig_empty;
private	int		nsigs;
private	int		test_case;
private	int		test_fail;
private	int		test_deleted;
private	int		sig_recvd;
private	int		sig_cont;
private	int		job_control;
private	int		glob_exit;
private	int		filelocking;
#if	TEST_XPG > 3 || TEST_FIPS
private	int		t11_pfd[2];
#endif

#if !TEST_XNFS
private	int		testfail;
#endif

private	void		test1();
private	void		ch_t1();
private	void		pa_t1();
private	void		test2();
private	void		t2_init();
private	void		t2_cln();
private	void		ch_t2();
private	void		pa_t2();
private	void		test3();
private	void		ch_t3();
private	void		pa_t3();
private	void		test4();
private	void		ch_t4();
private	void		pa_t4();
private	void		test5();
private	void		ch_t5();
private	void		test6();
private	void		ch_t6();
private	void		pgch_t6();
private	void		gch_t6();
private	void		pa_t6();
private	void		test7();
private	void		ch_t7();
private	void		pa_t7();
private	void		test8();
private	void		ch_t8();
private	void		pgch_t8();
private	void		ch_t8fg();
private	void		ch_t8bg();
private	void		pa_t8();
private	void		test9();
private	void		ch_t9();
private	void		pgch_t9();
private	void		pa_t9();
private	void		test10();
private	void		ch_t10();
private	void		pgch_t10();
private	void		pa_t10();
private	void		test11();
#if	TEST_XPG > 3 || TEST_FIPS
private	void		ch_t11();
private	void		aefunc11();
#endif
private	void		test12();
#if TEST_XNFS
private	void		ch_t12();
private	void		pa_t12();
#endif
private	void		prepare_tests();
private	void		sig_catch();
private	void		sig_resume();
private	int		sig_init();
private	void		ch_stop();
private	void		ch_sleep();
private	void		sjmpfunc();
#if !TEST_XNFS
private	int		ctty();
private	void		interrupt();
#endif

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	NULL, 0
};

private	void
prepare_tests()
{
	int	i;	/* Counter */
	int	ret;	/* Used for most return values */
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;


	if(uids()!=0)
	{
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		job_control = 0;
		nsigs = VNOJSIG;
	} else {
		job_control = 1;
		nsigs = VNUMSIG;
	}

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

private void
sig_resume(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_resume");

	sig_cont = sg_num;

	DBUG_PRINTF("sig_resume", "sig_cont %d", sig_cont);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = 0;

	DBUG_ENTER("test1");

	/*
	 * Test that _exit() terminates the process.
	 */

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(ch_t1, pa_t1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	PATH_FUNC_TRACE;
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	DBUG_ENTER("ch_t1");

	TEST_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	int	stat_child;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t1");

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		xx_rpt(FAILURE);
		in_rpt("child process still active %d seconds after calling _exit()", WAITTIME);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to send the signal to the process that's just disappeared */
	errno = NOERROR;
	ret = kill(tet_child, SIGKILL);
	err = errno;

	if(ret != SYSERROR || err != ESRCH) {
		xx_rpt(FAILURE);
		in_rpt("kill on process that has called _exit() did not return expected value");
		valrpt(SYSERROR, ret, ESRCH, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	pathok = 0;
#if	   TEST_XNFS
	char	*rlock;
#endif	/* TEST_XNFS */

	DBUG_ENTER("test2");

	/*
	 * Test that _exit() closes all file descriptors.
	 */

	test_fail = 0;
	test_deleted = 0;
	filelocking = 1;	/* file locking is supported */

#if	   TEST_XNFS
	if ((rlock = tet_getvar("XNFS_REMLOCK_SUPP")) == NULL || *rlock == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter XNFS_REMLOCK_SUPP is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(*rlock == 'N' || *rlock == 'n')
		filelocking = 0;

#else	/* TEST_XNFS */

	PATH_TRACE;

#endif	/* TEST_XNFS */

	globok = 0;
	t2_init();
	PATH_FUNC_TRACE;

	globok = 0;
	if(cpsig(ch_t2, pa_t2, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	PATH_FUNC_TRACE;

	t2_cln();
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
t2_init()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("t2_init");

	globok = 0;

	if(filelocking)
	{
		/* Create and open a lock file */
		errno = NOERROR;
		if((lock_fd = creat(t2_lock, CREATMODE)) < 0) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: creat() failed - errno %d", err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	/* Open a pipe before forking */
	errno = NOERROR;
	if(pipe(pipe_fd) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
t2_cln()
{
	DBUG_ENTER("t2_cln");

	(void) close(pipe_fd[READ]);
	(void) close(pipe_fd[WRITE]);

	if(filelocking)
	{
		(void) close(lock_fd);
		(void) unlink(t2_lock);
	}

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	DBUG_ENTER("ch_t2");

	if(filelocking)
	{
		/* Set a lock for the parent to detect, then sleep */
		lock_info.l_type = F_WRLCK;
		lock_info.l_whence = SEEK_SET;
		lock_info.l_start = (off_t)0;
		lock_info.l_len = (off_t)1;
		(void)fcntl(lock_fd, F_SETLK, &lock_info);
	}

	(void) sleep(WAITTIME);

	TEST_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	char	buffer[1];
	int	stat_child;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t2");

	globok = 0;

	/* The child locks a file and sleeps for WAITTIME */
	(void) sleep(WAITTIME/2);

	if(filelocking)
	{
		/* Obtain information on lock file */
		lock_info.l_type = F_WRLCK;
		lock_info.l_whence = SEEK_SET;
		lock_info.l_start = (off_t)0;
		lock_info.l_len = (off_t)1;
		errno = NOERROR;
		ret = fcntl(lock_fd, F_GETLK, &lock_info);
		if(ret == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - fcntl() failed errno %d", err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Check file really is locked */
		if(lock_info.l_pid != tet_child || lock_info.l_type != F_WRLCK) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - file descriptor not locked by child process");
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		xx_rpt(FAILURE);
		in_rpt("child process still active %d seconds after calling _exit()", WAITTIME);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Initialise the SIGPIPE action */
	if(sig_init(sig_array[VSIGPIPE], &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Close down the read side of the pipe */
	errno = NOERROR;
	ret = close(pipe_fd[READ]);
	if(ret == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - close() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(filelocking)
	{
		/* Obtain information on lock file */
		lock_info.l_type = F_WRLCK;
		lock_info.l_whence = SEEK_SET;
		lock_info.l_start = (off_t)0;
		lock_info.l_len = (off_t)1;
		errno = NOERROR;
		ret = fcntl(lock_fd, F_GETLK, &lock_info);
		if(ret == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - fcntl() failed errno %d", err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Check the file is no longer locked by the child */
		if(lock_info.l_type != F_UNLCK) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("file descriptor still locked after _exit() called");
			test_fail++;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;
		PATH_TRACE;
	}

	/* Try to write to the write side of the pipe */
	sig_recvd = 0;
	errno = NOERROR;
	ret = write(pipe_fd[WRITE], buffer, sizeof(char));
	err = errno;
	if(ret != SYSERROR || err != EPIPE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("file descriptor still open for reading after _exit() called");
		in_rpt("write() to pipe in parent process did not return expected value");
		valrpt(SYSERROR, ret, EPIPE, err);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(sig_recvd != SIGPIPE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal SIGPIPE not received when attempting to write to pipe");
		in_rpt("which is not open for reading by any process");
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(10);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test3");

	/*
	 * Test that _exit() is reported to waiting parent process.
	 */

	test_fail = 0;
	test_deleted = 0;
	glob_exit = 0;

	for(i = 0; glob_exit < TWOBYTEMASK; i++) {

		if(i == 0)
			glob_exit = 0;
		else if(i == 1) 
			glob_exit = 1;
		else
			glob_exit = (2*glob_exit) | glob_exit;

		globok = 0;
		if(cpsig(ch_t3, pa_t3, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		PATH_FUNC_TRACE;
		
		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	if(!test_fail)
		PATH_XS_RPT(2 * i);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	DBUG_ENTER("ch_t3");

	/* Sleep a while to give parent chance to wait() for _child */
	(void) sleep(WAITTIME/2);

	TEST_EXIT(glob_exit);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int	stat_child;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t3");

	globok = 0;

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != (glob_exit&BYTEMASK)) {
		xx_rpt(FAILURE);
		in_rpt("low order 8 bits of exit status not made available");
		in_rpt("expected exit status %d; observed exit status %d",
						glob_exit&BYTEMASK, ex_code);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	int	pathok = 0;

	DBUG_ENTER("test4");

	/*
	 * Test that _exit() saves information for waitpid() or wait().
	 */

	test_fail = 0;
	test_deleted = 0;

	test_case = WAIT;
	globok = 0;
	if(cpsig(ch_t4, pa_t4, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	PATH_FUNC_TRACE;
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	test_case = WAITPID;
	globok = 0;
	if(cpsig(ch_t4, pa_t4, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	PATH_FUNC_TRACE;
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	DBUG_ENTER("ch_t4");

	/* Call _exit() straight away whilst parent is sleeping */
	TEST_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t4()
{
	int	stat_child;
	int	ret;
	int	err;
	int	status;
	int	pathok = 0;

	DBUG_ENTER("pa_t4");

	globok = 0;

	/* Sleep whilst child calls _exit() */
	(void) sleep(WAITTIME/2);

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	SET_TIMEOUT(WAITTIME)
	switch(test_case) {
	case WAITPID:
		PATH_TRACE;
		ret = waitpid(tet_child, &status, WUNTRACED);
		break;
	case WAIT:
		PATH_TRACE;
		ret = wait(&status);
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("deletion reason - test_case not known");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	CLEAR_ALARM
	err = errno;
	if(ret == SYSERROR) {
		if(err == EINTR && alrm_flag) {
			stat_child = CHLD_LIVE;
		} else {
			stat_child = CHLD_ERR;
		}
	} else if(WIFEXITED(status)) {
		PATH_TRACE;
		stat_child = CHLD_EXIT;
		ex_code = WEXITSTATUS(status);
	} else if(WIFSIGNALED(status)) {
		PATH_TRACE;
		stat_child = CHLD_ABRT;
		sg_code = WTERMSIG(status);
	} else if(WIFSTOPPED(status)) {
		PATH_TRACE;
		stat_child = CHLD_STOP;
	} else {
		/* SHOULD NOT GET HERE - waitpid() WRONGLY IMPLEMENTED */
		stat_child = CHLD_ERR;
	}

	if(stat_child == CHLD_ERR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		xx_rpt(FAILURE);
		in_rpt("timed out calling waitpid() after child called _exit()"); 
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(FAILURE);
		in_rpt("low order 8 bits of status not saved for later use");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test5()
{
	DBUG_ENTER("test5");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test5()
{
	/* When session leader exits, terminal can be reacquired as ctty */
	char	*cp;
	int	pathok = 0;
	int	ret;

	DBUG_ENTER("test5");

	testfail = 0;

	cp = tet_getvar("VSX_TERMIOS_TTY");
	if (cp != NULL && strncmp(cp, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ret = cppair(ch_t5, NULLFN, NAP + WAITTIME, NOK|PATH_NOK);
	if (ret  == NOK)
	{
		DBUG_VOID_RETURN;
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_TRACE;
		
	/* Second call "re-acquires" ctty */
	ret = cppair(ch_t5, NULLFN, NAP + WAITTIME, NOK|PATH_NOK);
	if (ret == NOK)
	{
		DBUG_VOID_RETURN;
	}
	else if (ret == PATH_NOK)
		ret = OK;
	else
		PATH_TRACE;
		
	PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	pathok = 0;
	DBUG_ENTER("ch_t5");

	globok = 0; 
	if (ctty() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	interrupt(YES);
	PATH_FUNC_TRACE;

	TEST_EXIT((pathok==2)?OK:PATH_NOK);
	/* NOTREACHED */
	xx_rpt(DELETION);
	in_rpt("_exit failed");
	DBUG_EXIT(NOK);
}

#endif /* TEST_XNFS */

private void
test6()
{
	DBUG_ENTER("test6");

	/*
	 * Test that _exit() causes children to acquire new parent process ID.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cpsig(ch_t6, pa_t6, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	DBUG_ENTER("ch_t6");

	/*
	 * Turn into a grandchild and it's parent.
	 * The parent exits after a sleep during which time grandchild obtains 
	 * parent process ID.
	 * The grandchild obtains new parent process ID after it's parent
	 * calls _exit(), expecting it to have altered.
	 */

	(void) cppair(gch_t6, pgch_t6, WAITTIME, 0);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch_t6()
{
	DBUG_ENTER("pgch_t6");

	/* Exit after a short pause long enough for child to call getppid() */
	(void) sleep(WAITTIME/2);

	TEST_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
gch_t6()
{
	pid_t	ppid;
	pid_t	ppid_old;
	int	pathok = 0;

	DBUG_ENTER("gch_t6");

	ppid_old = getppid();

	/* Sleep, during which time the parent calls _exit() */
	(void) sleep(WAITTIME);

	ppid = getppid();

	if(ppid == ppid_old) {
		xx_rpt(FAILURE);
		in_rpt("parent process ID not altered after parent called _exit()");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	/* This is not the _exit() tested - it is the one in pgch_t6() */
	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t6()
{
	int	stat_child;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t6");

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Sleep further to give grandchild time to report */
	(void) sleep(WAITTIME);

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	DBUG_ENTER("test7");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}

	/*
	 * Test that _exit() causes SIGCHLD to be sent to parent process.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cpsig(ch_t7, pa_t7, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	/*
	 * Child process which exits.
	 * This should cause parent to receive signal SIGCHLD.
	 */
	DBUG_ENTER("ch_t7");

	(void) sleep(WAITTIME/2);

	TEST_EXIT(OK);

	DBUG_VOID_RETURN;
}

private void
pa_t7()
{
	int	pathok = 0;

	DBUG_ENTER("pa_t7");

	/* Set signal catching function for SIGCHLD */
	if(sig_init(SIGCHLD, &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	sig_recvd = 0;

	(void) sleep(WAITTIME);

	/* Check we received signal */
	if(!test_fail && sig_recvd != SIGCHLD) {
		xx_rpt(FAILURE);
		in_rpt("SIGCHLD signal not generated when child exitted");
		if(sig_recvd)
			in_rpt("instead signal %d (%s) was generated", 
						sig_recvd, sig_name[sig_recvd]);

		test_fail++;
	}
	else
		PATH_TRACE;

	/* Set default for SIGCHLD so we get no more interruptions */
	if(sig_init(SIGCHLD, &sig_empty, SIG_DFL, 0) < 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		test_fail++;
	}
	else
		PATH_TRACE;

	/* Wait for child process so we don't create a zombie */
	(void) wait_child(WAITTIME);

	if(!test_fail)
		PATH_XS_RPT(3);

	globok = 1; /* globok must be set for cppair */

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test8()
{
	DBUG_ENTER("test8");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test8()
{
	char	*cp;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test8");

	/*
	 * Test that _exit() sends SIGHUP to foreground processes.
	 */

	cp = tet_getvar("VSX_TERMIOS_TTY");
	if (cp != NULL && strncmp(cp, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	if(pipe(fg_fd) < 0 ||
	   pipe(bg_fd) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);
	(void) cpsig(ch_t8, pa_t8, WAITTIME, 0);

	(void) close(fg_fd[READ]);
	(void) close(fg_fd[WRITE]);
	(void) close(bg_fd[READ]);
	(void) close(bg_fd[WRITE]);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t8");

	PATH_FUNC_TRACE;

	/*
	 * Turn into two grandchildren and parent.
	 * The parent exits after a sleep.
	 * The grandchildren report to the grandparent on what happens.
	 */

	/* Set up a new session */
	if(setsid() == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("setsid() failed in child");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	/* Set up a controlling terminal for this session */
	/* Note: VSX_TERMIOS_TTY=unsup has already been checked for */
	if(tty_prep(&tty_fildes, tty_termios_p) != 0) {
		xx_rpt(DELETION);
		in_rpt("open/initialise failed on VSX_TERMIOS_TTY");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	/* Produce a process in foreground process group */
	switch(fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		PATH_FUNC_RPT(3);
		ch_t8fg();

		/* NOTREACHED */
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		PATH_TRACE;
		break;
	}
	tet_setblock();
	/* Produce a process in background process group */
	switch(fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		PATH_FUNC_RPT(4);
		ch_t8bg();

		/* NOTREACHED */
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		break;
	}
	tet_setblock();
	pgch_t8();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch_t8()
{
	DBUG_ENTER("pgch_t8");

	/* Exit after a short pause long enough to call sigaction() */
	(void) sleep(WAITTIME/2);

	TEST_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
ch_t8fg()
{
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("ch_t8fg");

	PATH_FUNC_TRACE;

	if(DBUG_SETJMP(sjmp) != 0) {
		buffer[0] = TEST_SIGNAL;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	else
		PATH_TRACE;
	setsigs(sjmpfunc);

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	else
		PATH_TRACE;

	sig_recvd = 0;

	/* Sleep, during which time the parent calls _exit() */
	(void) sleep(WAITTIME);

	/* Expect signal SIGHUP to have been received */
	if(sig_recvd != SIGHUP) {
		buffer[0] = TEST_FAIL;
	} else {
		PATH_TRACE;
		buffer[0] = TEST_SUCCESS;
	}

	/* Write down pipe to grandparent success code */
	(void) write(fg_fd[WRITE], buffer, sizeof(char));

	/* write path count success flag to grandparent */
	PATH_FUNC_RPT(4);
	buffer[0] = (char) globok;
	(void) write(fg_fd[WRITE], buffer, sizeof(char));
	
	DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
ch_t8bg()
{
	char	buffer[1];
	pid_t	ppid;
	pid_t	ppid_old;
	int	pathok = 0;

	DBUG_ENTER("ch_t8bg");

	PATH_FUNC_TRACE;

	if(DBUG_SETJMP(sjmp) != 0) {
		buffer[0] = TEST_SIGNAL;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	else
		PATH_TRACE;
	setsigs(sjmpfunc);

	/* Obtain parent process ID */
	ppid_old = getppid();

	/* Create a new (background) process group for this process */
	errno = 0;
	if(setpgid((pid_t)0, (pid_t)0) < 0) {
		if (errno != ENOSYS) {
			buffer[0] = TEST_SETPGID;
			(void) write(bg_fd[WRITE], buffer, sizeof(char));
			DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
		}
		else
			PATH_TRACE;
		(void) setsid();
	}
	else
		PATH_TRACE;

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	else
		PATH_TRACE;

	sig_recvd = 0;

	/* Sleep, during which time the parent calls _exit() */
	(void) sleep(WAITTIME);

	/* Obtain new parent process ID */
	ppid = getppid();

	if(ppid == ppid_old) {
		buffer[0] = TEST_PPID;
	} else {
		/* Expect no SIGHUP signal to have been received */
		if(sig_recvd == SIGHUP)
			buffer[0] = TEST_FAIL;
		else
		{
			PATH_TRACE;
			buffer[0] = TEST_SUCCESS;
		}
	}

	/* Write down pipe to grandparent success code */
	(void) write(bg_fd[WRITE], buffer, sizeof(char));

	/* write path count success flag to grandparent */
	PATH_FUNC_RPT(5);
	buffer[0] = (char) globok;
	(void) write(bg_fd[WRITE], buffer, sizeof(char));

	DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t8()
{
	int	stat_child;
	int	ret;
	int	err;
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("pa_t8");

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Read from pipes to determine success of grandchildren */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_SETPGID:
		xx_rpt(DELETION);
		in_rpt("deletion reason - setpgid() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling _exit() did not send signal SIGHUP to foreground process group");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	/* read path check success flag from ch_t8fg */
	globok = 0;
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = (int) buffer[0];
		PATH_FUNC_TRACE;
	}


	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_SETPGID:
		xx_rpt(DELETION);
		in_rpt("deletion reason - setpgid() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_PPID:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("parent process ID not reassigned in background process group");
		test_fail++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling _exit() sent signal SIGHUP to background process group");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	/* read path check success flag from ch_t8bg */
	globok = 0;
	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = (int) buffer[0];
		PATH_FUNC_TRACE;
	}

	if(test_fail || test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(9);

	globok = 1; /* globok must be set for cpsig */

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

private void
test9()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test9");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test that _exit() sends SIGHUP and SIGCONT if process is stopped
	 * when a process group is orphaned (by leader exiting).
	 */

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	if(pipe(fg_fd) < 0 ||
	   pipe(bg_fd) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);
	(void) cpsig(ch_t9, pa_t9, WAITTIME, 0);

	(void) close(fg_fd[READ]);
	(void) close(fg_fd[WRITE]);
	(void) close(bg_fd[READ]);
	(void) close(bg_fd[WRITE]);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t9");

	/*
	 * Do everything that follows in background process group.
	 */

	errno = 0;

	PATH_FUNC_TRACE;
	
	if(setpgid((pid_t)0, (pid_t)0) < 0) {
		if (errno != ENOSYS) {
			vsxerror("setpgid(0, 0) failed - errno %d (%s)",
				errno, errname(errno));
			DBUG_EXIT(A_MISC);
		}
		else
			PATH_TRACE;
		(void) setsid();
	}
	else
		PATH_TRACE;

	/*
	 * Creat process group with one stopped process and one
	 * sleeping process.  Then orphan it by exiting this process
	 * (the group leader).
	 * The grandchildren report to the grandparent on what happens.
	 */

	/* Produce two processes, stop one via SIGSTOP */
	switch(tet_child = fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		PATH_FUNC_RPT(2);
		ch_stop();
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		PATH_TRACE;
		break;
	}
	tet_setblock();
	switch(fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		PATH_FUNC_RPT(3);
		ch_sleep();
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		break;
	}
	tet_setblock();

	pgch_t9();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch_t9()
{
	DBUG_ENTER("pgch_t9");

	/* Exit after a short pause long enough to initialise children */
	(void) sleep(WAITTIME/2);

	/* This is the call that is actually tested */
	TEST_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t9()
{
	int	stat_child;
	int	ret;
	int	err;
	pid_t	pid;
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("pa_t9");

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
	}
	else if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
	}
	else
		PATH_TRACE;

	/* Read from pipe to determine PID of stopped grandchild */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], (char *)&pid, sizeof(pid_t));
	CLEAR_ALARM
	if(ret != sizeof(pid_t)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(pid_t));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Read from pipes to determine success of grandchildren */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		(void) killchild(pid, WAITTIME);
		test_deleted++;
	}
	else
		PATH_TRACE;

	if (test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_KILL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling _exit() failed to send SIGHUP and SIGCONT signals");
		in_rpt("to process in orphaned process group which is stopped");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	/* Read from pipes to determine path count success of grandchildren */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = (int) buffer[0];
		PATH_FUNC_TRACE;
	}


	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_KILL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling _exit() failed to send SIGHUP and SIGCONT signals");
		in_rpt("to process in orphaned process group which is not stopped");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	/* read from pipe to determine path count success of grandchild */
	globok = 0;
	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = (int) buffer[0];
		PATH_FUNC_TRACE;
	}

	if(test_fail || test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(10);

	globok = 1; /* globok must be set for cpsig */

	DBUG_VOID_RETURN;
}

private void
test10()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test10");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test that _exit() sends SIGHUP and SIGCONT if process is stopped
	 * when a process group is orphaned (by parent of leader exiting).
	 */

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	if(pipe(fg_fd) < 0 ||
	   pipe(bg_fd) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);
	(void) cpsig(ch_t10, pa_t10, WAITTIME, 0);

	(void) close(fg_fd[READ]);
	(void) close(fg_fd[WRITE]);
	(void) close(bg_fd[READ]);
	(void) close(bg_fd[WRITE]);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t10");

	/*
	 * Creat process group with one stopped process and one
	 * sleeping process.  Then orphan it by exiting this process.
	 * The grandchildren report to the grandparent on what happens.
	 */

	PATH_FUNC_TRACE;

	switch(tet_child = fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		errno = 0;
		if(setpgid((pid_t)0, (pid_t)0) < 0) {
			if (errno != ENOSYS) {
				vsxerror("setpgid(0, 0) failed - errno %d (%s)",
					errno, errname(errno));
				exit(A_MISC); /* not DBUG_EXIT */
			}
			else
				PATH_TRACE;
			(void) setsid();
		}
		else
			PATH_TRACE;

		switch(fork())
		{
		case SYSERROR:
			vsxerror("external error %d", A_FORKFAIL);
			exit(A_FORKFAIL); /* not DBUG_EXIT */

			/* NOTREACHED */
			break;

		case 0:		/* child */
			tet_setcontext();
			PATH_FUNC_RPT(2);
			ch_stop();
			break;

		default:
			tet_setblock();
			/* parent does nothing yet */
			PATH_TRACE;
			break;
		}
		tet_setblock();
		PATH_FUNC_RPT(3);
		ch_sleep();
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		break;
	}

	tet_setblock();
	pgch_t10();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch_t10()
{
	DBUG_ENTER("pgch_t10");

	/* Exit after a short pause long enough to initialise children */
	(void) sleep(WAITTIME/2);

	/* This is the call that is actually tested */
	TEST_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t10()
{
	int	stat_child;
	int	ret;
	int	err;
	pid_t	pid;
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("pa_t10");

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
	}
	else if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
	}
	else
		PATH_TRACE;

	/* Read from pipe to determine PID of stopped grandchild */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], (char *)&pid, sizeof(pid_t));
	CLEAR_ALARM
	if(ret != sizeof(pid_t)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(pid_t));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Read from pipes to determine success of grandchildren */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		(void) killchild(pid, WAITTIME);
		test_deleted++;
	}
	else
		PATH_TRACE;

	if (test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_KILL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling _exit() failed to send SIGHUP and SIGCONT signals");
		in_rpt("to process in orphaned process group which is stopped");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	/* Read from pipes to determine path count success of grandchildren */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = (int) buffer[0];
		PATH_FUNC_TRACE;
	}

	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_KILL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling _exit() failed to send SIGHUP and SIGCONT signals");
		in_rpt("to process in orphaned process group which is not stopped");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	/* read path count success flag from grandchild */
	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = (int) buffer[0];
		PATH_FUNC_TRACE;
	}

	if(test_fail || test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(10);

	globok = 1; /* globok must be set for cpsig */

	DBUG_VOID_RETURN;
}


#if	TEST_XPG > 3 || TEST_FIPS
private void
aefunc11()
{
	int	ret = ATEXIT_CALLED;
	DBUG_ENTER("aefunc11");
	(void)write(t11_pfd[WRITE],(char *)&ret, sizeof(int));
	DBUG_VOID_RETURN;
}

private	void
test11()
{
	int	err;
	int	ret;
	int	value;

	DBUG_ENTER("test11");

	errno = NOERROR;

	if(pipe(t11_pfd) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else

	(void)cppair(ch_t11, NULLFN, WAITTIME, 0);

	(void)close(t11_pfd[WRITE]);

	errno = NOERROR;
	ret = read(t11_pfd[READ], (char *)&value, sizeof(int));
	switch (ret)
	{
		case -1:
			err = errno;
			xx_rpt(DELETION);
			in_rpt("read() from pipe returned error %d (%s)", err, errname(err));
			break;
		case 0:
			xs_rpt();
			break;
		case (sizeof(int)):
			switch(value)
			{
				case ATEXIT_CALLED:
					xx_rpt(FAILURE);
					in_rpt("_exit() called a function registered with atexit()");
					break;
				case ATEXIT_FAILED:
					xx_rpt(DELETION);
					in_rpt("atexit() call returned non-zero result");
					break;
				default:
					xx_rpt(DELETION);
					in_rpt("Unexpected value read from pipe : %d - Expecting ATEXIT_CALLED (%d) or ATEXIT_FAILED (%d)", value, ATEXIT_CALLED, ATEXIT_FAILED);
					break;
			}
			break;

		default:
			xx_rpt(DELETION);
			in_rpt("Read from pipe returned unexpected number of bytes: Expected %d, recieved %d", (sizeof(int)), ret);
			break;
	}

	(void) close(t11_pfd[READ]);

	DBUG_VOID_RETURN;
}
#else
private	void
test11()
{
	DBUG_ENTER("test11");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}
#endif

#if	TEST_XPG > 3 || TEST_FIPS
private void
ch_t11()
{
	int	ret = ATEXIT_FAILED;

	(void)close(t11_pfd[READ]);
	if ( atexit(aefunc11) != 0 )
	{
		(void)write(t11_pfd[WRITE],(char *)&ret, sizeof(int));
	}
	TEST_EXIT(OK);
}
#endif


#if !TEST_XNFS

private void
test12()
{
	DBUG_ENTER("test12");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test12()
{
	int	pathok = 0;

	DBUG_ENTER("test12");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(ch_t12, pa_t12, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	PATH_FUNC_TRACE;
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	int	err, fd;
	int	pathok = 0;
	
	DBUG_ENTER("ch_t12");

	errno = NOERROR;

	if ((fd = open(FILE12, O_RDWR|O_CREAT, 0777)) == SYSERROR) 
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s,O_RDWR|O_CREAT,0777) failed - errno %d (%s)", FILE12, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (write(fd, T12_DATA, 6) != 6)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() failed to write 6 bytes to file %s - errno %d (%s)", FILE12, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(pathok == 2)
		TEST_EXIT(OK);
	else
		TEST_EXIT(SYSERROR);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t12()
{
	int	stat_child;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t12");

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		xx_rpt(FAILURE);
		in_rpt("child process still active %d seconds after calling _exit()", WAITTIME);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T12_CHECK_DATA_START, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CHECK_DATA_START failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T12_CHECK_DATA_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CHECK_DATA_END failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		PATH_FUNC_RPT(4);
	}

	(void) unlink(FILE12);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private void
ch_stop()
{
	char	buffer[1];
	pid_t	pid = getpid();
	int	pathok = 0;

	DBUG_ENTER("ch_stop");

	PATH_FUNC_TRACE;

	/* Write down pipe to grandparent process ID */
	(void) write(fg_fd[WRITE], (char *)&pid, sizeof(pid_t));

	if(DBUG_SETJMP(sjmp) != 0) {
		buffer[0] = TEST_SIGNAL;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	else
		PATH_TRACE;
	setsigs(sjmpfunc);

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	else
		PATH_TRACE;

	/* Initialise the SIGCONT action */
	if(sig_init(sig_array[VSIGCONT], &sig_empty, sig_resume, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	else
		PATH_TRACE;

	sig_recvd = 0;

	/* Stop ourself */
	if(kill(pid, SIGSTOP) < 0) {
		buffer[0] = TEST_KILL;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	else
		PATH_TRACE;

	/* Allow time for second signal to arrive, if necessary */
	if(sig_recvd != SIGHUP || sig_cont != SIGCONT)
		(void) sleep(SPEEDFACTOR);

	/* Expect signal SIGHUP and SIGCONT to have been received */
	if(sig_recvd != SIGHUP || sig_cont != SIGCONT) {
		buffer[0] = TEST_FAIL;
	} else {
		PATH_TRACE;
		buffer[0] = TEST_SUCCESS;
	}

	/* Write down pipe to grandparent success code */
	(void) write(fg_fd[WRITE], buffer, sizeof(char));

	/* write path check success flag to grandparent */
	PATH_FUNC_RPT(6);
	buffer[0] = (char) globok;
	(void) write(fg_fd[WRITE], buffer, sizeof(char));

	DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
ch_sleep()
{
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("ch_sleep");

	PATH_FUNC_TRACE;

	if(DBUG_SETJMP(sjmp) != 0) {
		buffer[0] = TEST_SIGNAL;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	else
		PATH_TRACE;
	setsigs(sjmpfunc);

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	else
		PATH_TRACE;

	/* Initialise the SIGCONT action */
	if(sig_init(sig_array[VSIGCONT], &sig_empty, sig_resume, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */
	}
	else
		PATH_TRACE;

	sig_recvd = 0;

	/* Sleep, during which time the parent calls _exit() */
	(void) sleep(WAITTIME);

	/* Allow time for second signal to arrive, if necessary */
	if(sig_recvd != SIGHUP || sig_cont != SIGCONT)
		(void) sleep(SPEEDFACTOR);

	/* Expect signal SIGHUP and SIGCONT to have been received */
	if(sig_recvd != SIGHUP || sig_cont != SIGCONT) {
		buffer[0] = TEST_FAIL;
	} else {
		PATH_TRACE;
		buffer[0] = TEST_SUCCESS;
	}

	/* Write down pipe to grandparent success code */
	(void) write(bg_fd[WRITE], buffer, sizeof(char));

	/* write path success flag to grandparent */
	PATH_FUNC_RPT(5);
	buffer[0] = (char) globok;
	(void) write(bg_fd[WRITE], buffer, sizeof(char));

	DBUG_LEAVE; exit(OK); /* not DBUG_EXIT */

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private	void
sjmpfunc(sig)
int	sig;
{
	DBUG_ENTER("sjmpfunc");
	DBUG_LONGJMP(sjmp, sig+1);
	DBUG_VOID_RETURN;
}

#if !TEST_XNFS

private int
ctty()
{
	/* acquire controlling terminal */

	int	pathok = 0;
	DBUG_ENTER("ctty");

	switch (termios_prep(&tty_fildes, tty_termios_p,
			&loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE; 
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_RETURN(-1);
	default:
		if (testfail++ == 0)
			xx_rpt(DELETION);

		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY/LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY/LOOP");

		DBUG_RETURN(-1);
	}
	PATH_FUNC_RPT(1); 

	DBUG_RETURN(0);
}

#endif /* TEST_XNFS */

#if !TEST_XNFS

private void
interrupt(toggle)
int toggle;                     /* YES == expecting SIGINT      */
{                               /* NO == not expecting SIGINT   */
	/* send SIGINT to self and children */

	int err, ret;
	struct sigaction sig;
	unsigned char chr;
	int	pathok = 0;

	DBUG_ENTER("interrupt");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (test_fail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        /* for child setup, if any - allow for this in cppair() timeouts */
        (void) sleep(NAP);

        sig_recvd= 0;

	DBUG_PRINTF("interrupt", "c_cc[VINTR] = 0%o",
		(int)tty_termios_p->c_cc[VINTR]);

        chr = tty_termios_p->c_cc[VINTR];

        SET_TIMEOUT(WAITTIME/2)

        ret = write(loop_fildes, (char *)&chr, 1);
	err = errno;
	if (ret == 1)
		(void) pause();

        CLEAR_ALARM

	if (ret != 1)
	{
		if (test_fail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (toggle == YES && sig_recvd != SIGINT)
	{
		if (test_fail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("process did not receive SIGINT when expected");
	}
	else
		PATH_TRACE;

	if (toggle == NO && sig_recvd == SIGINT)
	{
		if (test_fail++ == 0)
		{
			xx_rpt(FAILURE);
		}
		in_rpt("process received SIGINT when not expected");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4); 

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */
