/*
*      SCCS:  @(#)  POSIX.os/procprim/alarm/alarm.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/alarm - T.alarm Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.alarm

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the alarm() system call.

AUTHOR:		Mike Rendell, UniSoft Ltd.
DATE CREATED:	Wed Aug 13 16:26:20 PDT 1986
MODIFICATIONS:	D McCardle Fri Jun 12 11:36:18 WET 1987
			Removed abandon(), testid().
			Modified xx_rpt(), xs_rpt().
		Dave Cater Fri Mar 10 13:08:59 GMT 1989
			upgraded for XPG 3 to comply with POSIX assertions 
		Dave Cater Mon Sep 18 14:34:05 BST 1989
			Use VNOJSIG not VNUMSIG if no _POSIX_JOB_CONTROL

		Peter Lee, Unisoft Ltd.,
		Fri Aug 10 12:18:32 BST 1990
		Added resilience code.

		Geoff Clare, 8 Mar 1991
			Added test 5.

		Stuart Boutell, 30 Apr 1991
			Changed cpsig() return code check from FAIL to
		SYSERROR

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef alarm
#endif


#define	ALRMTIME	(SPEEDFACTOR * 5)	/* alarm-time basic unit */
#define	WAITTIME	(ALRMTIME * 2) + (SPEEDFACTOR * 5)
#define	EXTERR		"external error"
#define VSX_AL_ACCURACY "VSX_AL_ACCURACY"
#define	DEF_ACCURACY	SPEEDFACTOR	/* default accuracy */
#define	NO_TESTS		5		/* number of tests in testset */
#define PATH_NOK	3

extern	void		(*vsignal())();
extern	void		vsx_sigs();
extern	int		sig_array[];
extern	int		ex_code;
extern	int		sg_code;

int	no_tests = NO_TESTS;

private	int		test_fail;
private	int		test_deleted;
private	int		accuracy;
private	int		save_sig;
private	unsigned	alrm_time = ALRMTIME;
private int		setupok;

private	void		test1();
private	void		test2();
private	void		test3();
private	void		test4();
private	void		test5();
private void		ch_p1();
private void		pa_p1();
private void		ch_p2();
private void		pa_p2();
private void		ch_t2();
private void		ch_t3();
private void		ch_t4();
private void		prepare_tests();
private	void		sig_set_up();
private	void		sig_handler();
private	unsigned	tb_alarm();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	NULL, 0
};

private	void
prepare_tests()
{
	char	*sacc;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	sacc = tet_getvar(VSX_AL_ACCURACY);
	if ( (sacc == (char *) NULL) || (*sacc == '\0') )
		accuracy = DEF_ACCURACY;
	else
		accuracy = atoi(sacc);

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	PATH_SETUP_RPT(1);

	DBUG_VOID_RETURN;
}


private void
sig_set_up()
{
	int	i;
	int	ret;
	int	nsigs;

	DBUG_ENTER("sig_set_up");

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		nsigs = VNOJSIG;
	} else {
		nsigs = VNUMSIG;
	}

	for (i=1 ; i < nsigs ; i++)
		if (sig_array[i] != VNOSIG && i != VSIGTERM && i != VSIGKILL)
		    if (vsignal(sig_array[i], SIG_IGN) != SIG_IGN)
			(void) vsignal(sig_array[i], sig_handler);
	DBUG_VOID_RETURN;
}


private void
sig_handler(sig)

	int	sig;
{
	DBUG_ENTER("sig_handler");
	save_sig = sig;
	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = setupok;

	DBUG_ENTER("test1");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(ch_p1, pa_p1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if(cpsig(ch_p2, pa_p2, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch_p1()
{
	unsigned	tm_out = 0;
	time_t		tm_start;
	time_t		tm_now;
	int	pathok = setupok;
	int	loop1 = 0;

	DBUG_ENTER("ch_p1");
	sig_set_up();
	save_sig = VNOSIG;
	(void) time(&tm_start);
	(void) tb_alarm(alrm_time);

	loop1 = 0;
	while (save_sig == VNOSIG)
		if (time(&tm_now) > tm_start + (time_t) (accuracy + alrm_time))
		{
			tm_out++;
			break;
		}
		else
		{
			loop1++;
			PATH_TRACE;
		}
	if (tm_out || save_sig != SIGALRM
		|| tm_now < tm_start + (time_t) (alrm_time - accuracy))
	{
		if(!test_fail)
			xx_rpt(FAILURE);
		if (tm_out)
			in_rpt("alarm signal not recieved within tolerance limits");
		else if (save_sig != SIGALRM)
			in_rpt("signal recieved was not SIGALRM, received signal %d",
				save_sig);
		else
			in_rpt("SIGALRM recieved %d seconds too soon",
				(tm_start + (time_t) alrm_time) - tm_now);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(test_fail)
	{
		DBUG_EXIT(A_MISC);
	}
	else
		PATH_TRACE;

	if (pathok != (2 + loop1))
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_p1()
{
	int	err;
	int	stat_child;
	int	pathok = setupok;

	DBUG_ENTER("pa_p1");

	/* Wait for the child and work out if success or failure */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_MISC)
			test_fail++;
		else if(ex_code == A_TSETDIR)
			test_deleted++;
		else if (ex_code != PATH_NOK)
			PATH_TRACE;
	} else if(stat_child == CHLD_ABRT) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected signal %d received",
								 sg_code);
		test_deleted++;
	} else {
		xx_rpt(DELETION);
		in_rpt("deletion reason: timed out after %d seconds", 
								WAITTIME);
		test_deleted++;
	}

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_p2()
{
	DBUG_ENTER("ch_p2");
	sig_set_up();
	save_sig = VNOSIG;
	(void) vsignal(sig_array[VSIGALRM], SIG_DFL);
	(void) tb_alarm(alrm_time);
	(void) sleep((unsigned)(2 * alrm_time));

	/* Should not get here. Parent checks exit code */
	DBUG_EXIT(A_MISC);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_p2()
{
	int	err;
	int	stat_child;
	int	pathok = setupok;

	DBUG_ENTER("pa_p2");

	/* Wait for the child and work out if success or failure */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_MISC) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("alarm() returned when uncaught signal SIGTERM generated");
		in_rpt("expected process to terminate on delivery of signal");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_TSETDIR) 
		test_deleted++;
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, (int)(alrm_time * 15), 0);

	DBUG_VOID_RETURN;
}


private void
ch_t2()
{
	unsigned	tm_out = 0;
	time_t		tm_start;
	time_t		tm_now;
	int	pathok = setupok;

	DBUG_ENTER("ch_t2");
	sig_set_up();
	save_sig = VNOSIG;
	(void) tb_alarm(alrm_time * 10);
	(void) time(&tm_start);
	(void) tb_alarm(alrm_time);
	while (save_sig != SIGALRM)
		if (time(&tm_now) > tm_start + (time_t) (accuracy + alrm_time))
		{
			tm_out++;
			break;
		}
	if (tm_out || tm_now < tm_start + (time_t) (alrm_time - accuracy))
	{
		xx_rpt(FAILURE);
		in_rpt("when alarm() time was shortened before previous alarm() went off");
		if (tm_out)
			in_rpt("alarm signal was not recieved within tolerance limits");
		else
			in_rpt("SIGALRM was recieved %d seconds too soon",
				(tm_start + (time_t) alrm_time) - tm_now);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) cppair(ch_t3, NULLFN, (int)(alrm_time * 15), 0);

	DBUG_VOID_RETURN;
}


private void
ch_t3()
{
	time_t		tm_start;
	time_t		tm_now;
	int	pathok = setupok;

	DBUG_ENTER("ch_t3");

	sig_set_up();
	save_sig = VNOSIG;
	(void) time(&tm_start);
	(void) tb_alarm(alrm_time);
	(void) tb_alarm(0);
	while (save_sig != SIGALRM &&
		time(&tm_now) < tm_start + (time_t) (accuracy + alrm_time))
			;
	if (save_sig == SIGALRM)
	{
		xx_rpt(FAILURE);
		in_rpt("recieved SIGALRM after alarm(0)");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	(void) cppair(ch_t4, NULLFN, (int)(alrm_time * 15), 0);

	DBUG_VOID_RETURN;
}


private void
ch_t4()
{
	unsigned	sec_left;
	time_t		tm_start;
	time_t		tm_now;
	unsigned	talrm_time;		/* a tenth of the alarm time */
	unsigned	al_time;		/* alarm time rounded up */
	int	pathok = setupok;

	DBUG_ENTER("ch_t4");

	/*
	 * Round up the alarm time to multiple of 10
	 * This way it is also a multiple of 2.
	 * So there should be no rounding problems in either the shift
	 * or the divide operations that follow ......
	 */
	al_time = alrm_time - alrm_time % 10 + 10;
	sig_set_up();
	save_sig = VNOSIG;
	(void) time(&tm_start);
	(void) tb_alarm(al_time);
	talrm_time = al_time / 10;
	if (talrm_time == 0)
		talrm_time = 1;
	while (time(&tm_now) < tm_start + (time_t) (al_time >> 1))
		;
	sec_left = tb_alarm(0);
	if (sec_left < tm_now - tm_start - (time_t) talrm_time
		|| sec_left > tm_now - tm_start + (time_t) talrm_time)
	{
		xx_rpt(FAILURE);
		in_rpt("time left on clock not within tolerance limits");
	}
	else
		PATH_TRACE;

	DBUG_PRINTF("ch_t4", "sec left %u tm_now %ld tm_start %ld talrm_time %u",
				sec_left, tm_now, tm_start, talrm_time);

	/*
	 * For POSIX, check also that alarm() returns 0 now since we 
	 * have just cancelled the alarm.
	 */ 
	sec_left = tb_alarm(0);
	if(sec_left != 0) {
		xx_rpt(FAILURE);
		in_rpt("time left on clock not zero with no outstanding alarm");
	}
	else
	{
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test5()
{
	unsigned int ret;
	int pathok = 0;

	DBUG_ENTER("test5");

#if !defined(_XOPEN_SOURCE) || TEST_XPG > 3

	(void) alarm(1);
	ret = tb_alarm(0);

	if (ret != 1)
	{
		xx_rpt(FAILURE);
		in_rpt("alarm() returned %u, expected 1", ret);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}
#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}

private unsigned
tb_alarm(sec)
unsigned sec;
{
	unsigned retval;

	retval = alarm(sec);

#ifdef TESTBED
	errno++;
	return 9999;
#else
	return retval;
#endif
}
