/*
*      SCCS:  @(#)  POSIX.os/procprim/exec/common.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/exec - common.c Rel 4.4.1 (06/09/97)";
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		common.c - common routines used in exec tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
	main()
SYNOPSIS:
DESCRIPTION:
	Called in many exec tests.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Neil Moses, Unisoft Ltd.
DATE CREATED:	Thu Nov  5 15:17:28 GMT 1992
MODIFICATIONS:
		Dave Cater, 3 December 1992
		Added unlink(to) before system() in case file permission
		would prevent copy from completing over NFS.

		Dave Cater, 10 December 1992
		Added chown() and chmod() to file_copy() if there is 
		privilege to do that. Needed for exec_tstop for example.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<pcts_max.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<setprv.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<sys/times.h>
#include	<sys/wait.h>
#include	<stdio.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<string.h>
#include	<exec.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif


#define	NAME()		(i_name[iwhich])
#define	MAXARG		8
#define	MODEGEN		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	char		*errname();
extern	char		*signame();

extern	int		iwhich;		/* what are we? (execl, execvp,... ) */
extern	char		*p_argv[MAXARG];
extern	char		**p_envp;
extern	int		p_argc;
extern	char		**environ;
extern	int		test_fail;
extern	int		test_deleted;
extern	char		*i_name[];
extern	sigset_t 	sig_empty;
extern	int		sig_recvd;

public  char    	*t_prog;
public  char    	*t_path;
public	char		tclose_file[] =		"./exec_t.close";
public	int		file_close;
public	int		doexec();
public	void		init_exec();
public	int		sig_init();
public	void		sig_catch();
public	char		**save_p();
public	char		**save_p_ptr();
public	void		cmp_p();
public	void		cmp_p_ptr();
public	void		save_args();
public	void		save_envp();
public	void		save_fds();
public	void		save_ids();
public	void		save_sigs();
public	void		save_p_environ();
public	void		cmp_args();
public	void		cmp_envp();
public	void		cmp_fds();
public	void		cmp_ids();
public	void		cmp_sigs();
public	void		cmp_p_environ();
public	void		do_save();
public	void		do_cmp();
public	int		file_copy();

private	int		save_euid;
private	int		save_egid;
private	char		**args_p;
private	char		**args_p_ptr;
private	char		**envp_p;
private	char		**envp_p_ptr;
private	char		**s_environ;
private	char		**s_p_environ;
private	char		**s_p_ptr_environ;

public void
init_exec(arg1, arg2, arg3, arg4, arg5, arg6, arg7)
char	*arg1;
char	*arg2;
char	*arg3;
char	*arg4;
char	*arg5;
char	*arg6;
char	*arg7;
{
	DBUG_ENTER("init_exec");

	p_argv[0] = arg1;
	p_argv[1] = arg2;
	p_argv[2] = arg3;
	p_argv[3] = arg4;
	p_argv[4] = arg5;

	if(arg6 == (char *)NULL)
		p_argv[5] = "";
	else
		p_argv[5] = arg6;
	if(arg7 == (char *)NULL)
		p_argv[6] = "";
	else
		p_argv[6] = arg7;

	p_argv[7] = (char *) NULL;
	p_argc = 7;

	p_envp[0] = (char *) NULL;
	(void) vputenv(&p_envp, t_path);
	(void) vputenv(&p_envp, P1_TSET);
	(void) vputenv(&p_envp, P1_FILE);
	(void) vputenv(&p_envp, P1_TENV1);
	(void) vputenv(&p_envp, P1_LOWER);
	(void) vputenv(&p_envp, P1_UPPER);
	(void) vputenv(&p_envp, P1_PORT);
	if (iwhich != I_EXECLE && iwhich != I_EXECVE)
		environ = p_envp;

	DBUG_VOID_RETURN;
}

public int
doexec()
{
	int	ret, i;

	DBUG_ENTER("doexec");

	DBUG_PRINTF("doexec", "executing %s with arguments:", t_prog);
	for (i = 0; i < 7; i++)
	{
		DBUG_PRINTF("doexec", "arg%d = %s", i,
		    p_argv[i] == NULL ? "NULL" :
		    (strlen(p_argv[i]) > 500 ? "> 500 bytes" : p_argv[i]));
	}

	switch (iwhich)
	{
	case I_EXECL:
		ret = execl(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			(char *) NULL);
		break;
	case I_EXECLE:
		ret = execle(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			(char *) NULL, p_envp);
		break;
	case I_EXECLP:
		ret = execlp(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			(char *) NULL);
		break;
	case I_EXECV:
		ret = execv(t_prog, p_argv);
		break;
	case I_EXECVE:
		ret = execve(t_prog, p_argv, p_envp);
		break;
	case I_EXECVP:
		ret = execvp(t_prog, p_argv);
		break;
	}

	DBUG_RETURN(ret);
}

public int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

public void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

private	void
save_args()
{
	DBUG_ENTER("save_args");

	switch ( iwhich ) {
		case I_EXECV:
		case I_EXECVE:
		case I_EXECVP:
				break;
		default:
			DBUG_VOID_RETURN;
	}

	DBUG_PRINTF("save_args:","iwhich = %d",iwhich);

	args_p = save_p(p_argv);
	args_p_ptr = save_p_ptr(p_argv);

	DBUG_VOID_RETURN;

}

private	void
save_envp()
{
	DBUG_ENTER("save_envp");

	switch ( iwhich ) {
		case I_EXECVE:
		case I_EXECLE:
				break;
		default:
			DBUG_VOID_RETURN;
	}

	DBUG_PRINTF("save_envp:","iwhich = %d",iwhich);

	envp_p = save_p(p_envp);
	envp_p_ptr = save_p_ptr(p_envp);

	DBUG_VOID_RETURN;

}

private void
cmp_args()
{
	DBUG_ENTER("cmp_args");

	switch ( iwhich ) {
		case I_EXECV:
		case I_EXECVE:
		case I_EXECVP:
				break;
		default:
			DBUG_VOID_RETURN;
	}

	DBUG_PRINTF("cmp_args:","iwhich = %d",iwhich);

	if (! args_p || ! args_p_ptr) {
		if ( !test_deleted) {
			xx_rpt(DELETION);
			test_deleted++;
		}
		in_rpt("deletion reason - unexpected NULL argv pointer");
		DBUG_VOID_RETURN;
	}

	cmp_p(p_argv, args_p, "argv");
	cmp_p_ptr(p_argv, args_p_ptr, "argv");

	(void) free((void *)args_p);
	args_p = (char **)0;
	(void) free((void *)args_p_ptr);
	args_p_ptr = (char **)0;

	DBUG_VOID_RETURN;
}

private void
cmp_envp()
{
	DBUG_ENTER("cmp_envp");

	switch ( iwhich ) {
		case I_EXECVE:
		case I_EXECLE:
				break;
		default:
			DBUG_VOID_RETURN;
	}

	DBUG_PRINTF("cmp_envp:","iwhich = %d",iwhich);

	if (! envp_p || ! envp_p_ptr) {
		if (! test_deleted) {
			xx_rpt(DELETION);
			test_deleted++;
		}
		in_rpt("deletion reason - unexpected NULL envp pointer");
		DBUG_VOID_RETURN;
	}

	cmp_p(p_envp, envp_p, "envp");
	cmp_p_ptr(p_envp, envp_p_ptr, "envp");

	(void)free((void *)envp_p);
	envp_p = (char **)0;
	(void)free((void *)envp_p_ptr);
	envp_p_ptr = (char **)0;

	DBUG_VOID_RETURN;
}

private	void
save_fds()
{
	int	err;
	int	flags;

	DBUG_ENTER("save_fds");

	/* Open a file which should still be open after a failed exec */
	if ((file_close = creat(tclose_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			tclose_file, MODEGEN, err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	(void) close(file_close);

	if ((file_close = open(tclose_file, O_RDWR)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open() failed - errno %d (%s)",
					err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	if((flags = fcntl(file_close, F_GETFD, 0)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_GETFD) failed - errno %d (%s)",
					err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	flags |= FD_CLOEXEC;
	if((flags = fcntl(file_close, F_SETFD, flags)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_SETFD) failed - errno %d (%s)",
					err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	
	DBUG_VOID_RETURN;

}

private void
cmp_fds()
{
	struct	stat stbuf;

	DBUG_ENTER("cmp_fds");

        if(fstat(file_close, &stbuf) != 0) {
		if ( !test_fail ) {
			xx_rpt(FAILURE);
			test_fail++;
		}
		in_rpt("File descriptor closed during %s",NAME());
	}

	(void) close(file_close);
	(void) unlink(tclose_file);

	DBUG_VOID_RETURN;
}

private	void
save_ids()
{
	DBUG_ENTER("save_ids");
	
	save_euid = geteuid();
	save_egid = getegid();

	DBUG_VOID_RETURN;
}

private void
cmp_ids()
{
	int	euid, egid;

	DBUG_ENTER("cmp_ids");

	euid = geteuid();
	egid = getegid();

        if(save_euid != euid) {
		if ( !test_fail ) {
			xx_rpt(FAILURE);
			test_fail++;
		}
		in_rpt("Effective user ID changed during %s",NAME());
		in_rpt("Expected user ID: %d", save_euid);
		in_rpt("Observed user ID: %d", euid);
	}
        if(save_egid != egid) {
		if ( !test_fail ) {
			xx_rpt(FAILURE);
			test_fail++;
		}
		in_rpt("Effective group ID changed during %s",NAME());
		in_rpt("Expected group ID: %d", save_egid);
		in_rpt("Observed group ID: %d", egid);
	}

	DBUG_VOID_RETURN;
}

private	void
save_sigs()
{
	DBUG_ENTER("save_sigs");

	if(sig_init(SIGQUIT, &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed - signal SIGQUIT");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	
	DBUG_VOID_RETURN;

}

private void
cmp_sigs()
{
        struct  sigaction act;
	int	sg_num = SIGQUIT;
	char	*sg_name = signame(SIGQUIT);

	DBUG_ENTER("cmp_sigs");

	if(sigaction(sg_num, (struct sigaction *)NULL, &act) < 0) {
		if(!test_deleted) {
			xx_rpt(DELETION);
			test_deleted++;
		}
		in_rpt("deletion reason - sigaction() failed - signal %d (%s)",
					sg_num, sg_name);
		DBUG_VOID_RETURN;
	}

	if(act.sa_handler == SIG_DFL) {
		if ( !test_fail ) {
			xx_rpt(FAILURE);
			test_fail++;
		}
		in_rpt("Signal SIGQUIT was set to default during %s",NAME());
	}

	DBUG_VOID_RETURN;
}

private	void
save_p_environ()
{
	DBUG_ENTER("save_p_environ");

	s_environ = environ;
	s_p_environ = save_p(environ);
	s_p_ptr_environ = save_p_ptr(environ);

	DBUG_VOID_RETURN;
}

private	char **
save_p(eptr)
char	**eptr;
{
	register char **pptr,**tptr;
	size_t nenv;
	register int i = 0;

	DBUG_ENTER("save_p");

	pptr = eptr;
	while ( *pptr ) {	/* calculate total space needed */
		pptr++;
	}
	nenv = pptr - eptr + 1;	/* number of env variables plus NULL */

	if ( (tptr = (char **)malloc(nenv*sizeof(char **))) == NULL ) {
		vsxerror("malloc() failed");
		DBUG_TET_EXIT(A_NOMEM);
	}
	
	i = 0;
	pptr = eptr;
	while ( *pptr ) {
		tptr[i++] = *pptr++;
	}
	tptr[i] = (char *)0;

	DBUG_RETURN(tptr);
}

private	char **
save_p_ptr(eptr)
char	**eptr;
{
	register char **pptr,**tptr,*ptr;
	size_t envsize = 0;
	size_t nenv;
	register int i = 0;

	DBUG_ENTER("save_p_ptr");

	pptr = eptr;
	while ( *pptr ) {	/* calculate total space needed */
		envsize += sizeof(char **);
		envsize += strlen(*pptr) + 1;
		pptr++;
	}
	envsize += sizeof(char **); /* one more for the NULL ptr at the end */
	nenv = pptr - eptr + 1;	/* number of env variables plus NULL */

	if ( (tptr = (char **)malloc(envsize)) == NULL ) {
		vsxerror("malloc() failed");
		DBUG_TET_EXIT(A_NOMEM);
	}
	DBUG_PRINTF("save_p_ptr:","envsize = %d",envsize);
	
	i = 0;
	ptr = (char *)(tptr + nenv); /* string data */
	pptr = eptr;
	while ( *pptr ) {
		(void)strcpy(ptr,*pptr);
	DBUG_PRINTF("save_p_ptr:","ptr = 0x%x",(long)ptr);
	DBUG_PRINTF("save_p_ptr:","env[%d] (%d chars) = %s",i,strlen(ptr),ptr);
		tptr[i++] = ptr;
		ptr += strlen(*pptr) + 1;
		pptr++;
	}
	tptr[i] = (char *)0;

	DBUG_RETURN(tptr);
}

private void
cmp_p_environ()
{
	DBUG_ENTER("cmp_p_environ");

	if (! s_p_environ || !s_p_ptr_environ) {
		if (! test_deleted) {
			xx_rpt(DELETION);
			test_deleted++;
		}
		in_rpt("deletion reason - unexpected NULL env pointer");
		DBUG_VOID_RETURN;
	}

	if(environ != s_environ) {
		if ( !test_fail ) {
			xx_rpt(FAILURE);
			test_fail++;
		}
		in_rpt("The value of environ changed during %s",NAME());
	}

	cmp_p(environ, s_p_environ, "environ");
	cmp_p_ptr(environ, s_p_ptr_environ, "environ");

	(void)free((void *)s_p_environ);
	s_p_environ = (char **)0;
	(void)free((void *)s_p_ptr_environ);
	s_p_ptr_environ = (char **)0;

	DBUG_VOID_RETURN;
}

private void
cmp_p(eptr, pptr, str)
char	**eptr;
char	**pptr;
char	*str;
{
	register char **p1,**p2;
	int failcmp = 0;
	int i = 0;

	DBUG_ENTER("cmp_p");

	p1 = eptr;
	p2 = pptr;

	i = 0;
	failcmp = 0;
	while ( *p1 && *p2  ) {
		DBUG_PRINTF("cmp_p","p1[%d] = %X",i,*p1);
		DBUG_PRINTF("cmp_p","p2[%d] = %X",i,*p2);
		DBUG_PRINTF("cmp_p","p1[%d] (%d chars) = %s",i,strlen(*p1),*p1);
		DBUG_PRINTF("cmp_p","p2[%d] (%d chars) = %s",i,strlen(*p2),*p2);
		i++;
		if ( *p1++ != *p2++ ) {
			failcmp++;
			break;
		}
	}
	if(*p1 != NULL)
		DBUG_PRINTF("cmp_p","p1[%d] (%d chars) = %s",i,strlen(*p1),*p1);
	if(*p2 != NULL)
		DBUG_PRINTF("cmp_p","p2[%d] (%d chars) = %s",i,strlen(*p2),*p2);

	if ( !(*p1 == NULL && *p2 == NULL && failcmp == 0) ) {
		if ( !test_fail ) {
			xx_rpt(FAILURE);
			test_fail++;
		}
		in_rpt("The pointers in %s changed during %s",str, NAME());
	}

	DBUG_VOID_RETURN;
}

private void
cmp_p_ptr(eptr, pptr, str)
char	**eptr;
char	**pptr;
char	*str;
{
	register char **p1,**p2;
	int failcmp = 0;
	int i = 0;

	DBUG_ENTER("cmp_p_ptr");

	p1 = eptr;
	p2 = pptr;

	i = 0;
	failcmp = 0;
	while ( *p1 && *p2  ) {
		DBUG_PRINTF("cmp_p_ptr","p1[%d] (%d chars) = %s",i,strlen(*p1),*p1);
		DBUG_PRINTF("cmp_p_ptr","p2[%d] (%d chars) = %s",i,strlen(*p2),*p2);
		i++;
		if ( strcmp(*p1++,*p2++) ) {
			failcmp++;
			break;
		}
	}

	if ( !(*p1 == NULL && *p2 == NULL && failcmp == 0) ) {
		if ( !test_fail ) {
			xx_rpt(FAILURE);
			test_fail++;
		}
		in_rpt("Strings in %s changed during %s",str, NAME());
	}

	DBUG_VOID_RETURN;
}

public void
do_save()
{
	save_args();
	save_envp();
	save_fds();
	if(test_deleted)
        	(void) unlink(tclose_file);
	save_ids();
	save_sigs();
	save_p_environ();
}

public void
do_cmp()
{
	cmp_args();
	cmp_envp();
	cmp_fds();
	cmp_ids();
	cmp_sigs();
	cmp_p_environ();
}

public	int
file_copy(from, to)
	char	*from;
	char	*to;
{
	struct	stat stbuf;
	int	err;
	char	buf[_POSIX_PATH_MAX * 2 + 30];

	if (strlen(from) + strlen(to) > 2 * _POSIX_PATH_MAX)
	{
		vsxerror("path names too long in file_copy()");
		return(SYSERROR);
	}

	/* generate the string "cp from to" */
	(void) strcat(strcat(strcat(strcpy(buf, "cp "), from), " "), to);
	DBUG_PRINTF("file_copy", "%s", buf);

	(void) unlink(to);
	
	if (stat(from, &stbuf) == SYSERROR)
	{
		err = errno;
		vsxerror("stat failed, errno %d", err);
		return (SYSERROR);
	}
	if (system(buf) != SUCCEED)
	{
		err = errno;
		(void) unlink(to);
		vsxerror("system(\"%s\") failed", buf);
		return (SYSERROR);
	}
	if (setprv(PRV_CHOWN) == SYSERROR)
		return (SUCCEED);
	if (chown(to, stbuf.st_uid, stbuf.st_gid) == SYSERROR)
	{
		err = errno;
		(void) unlink(to);
		vsxerror("chown failed, errno %d", err);
		return (SYSERROR);
	}
	if (chmod(to, stbuf.st_mode) == SYSERROR)
	{
		err = errno;
		(void) unlink(to);
		vsxerror("chmod failed, errno %d", err);
		return (SYSERROR);
	}
	return (SUCCEED);
}

