/*
*      SCCS:  @(#)POSIX.os/procprim/exec/exec.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/exec - T.exec Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.exec

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the exec() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Tue Apr 11 10:26:43 BST 1989
MODIFICATIONS:	
		Joseph Nave, UniSoft Ltd.  Wed May 31 12:41:06 BST 1989
			Source split into two files

		DPJ Cater, UniSoft Ltd.  Tue Jul 11 18:13:54 BST 1989
			Added do_access() test (test 32).

		Geoff Clare, 19 Oct 1989
			Check malloc() return value.

		Dave Cater, 1 Nov 1989
			Close file opened in pa21p2().
			Initialise global variables in tests 22 - 24.
			Add _POSIX_SAVED_IDS check in test29.

		Geoff Clare, 6 Apr 1990
			Delete redundant nametoolng() etc. declarations
		
		Geoff Clare, 30 May 1990
			Allow for unlimited ARG_MAX

		Keith Holder, 22 Jun 1990
			Report UNSUPPORTED, if VSX_SET_ID_MODES_SUPP
			is not set to 'Y' in tests 27 & 28
		
		Geoff Clare, 26 June 1990
			Add test that directory streams are closed across exec

		David Sawyer
		Fri Aug 17 16:41:27 BST 1990
		Added resilience code.
		Tracing may seem a little patchy; the philosophy adopted was
		to path trace where it was expedient to do so.

		Geoff Clare, 12 Oct 1990
			Change calculations in big_exec().
			Guard against NULL vg_dbug.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.
			Changed execve() to tet_exec() in exec_su().

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Stuart Boutell, 14 Nov 1990
			Moved signal initialisation from prepare_tests()
			to signal_init()

		Stuart Boutell, 21 Nov 1990
			Added pipe reporting mechanism; (add pipe_open,
			pipe_close, modify testgen & tests 16,20,22,23,24)
			Added pipe_delete, pipe_ch_exec, pipe_untested,
			pipe_ch_big,pipe_child, pipe_parent.

		Stuart Boutell, 22 Nov 1990
			Moved pipe_ functions to pipeplib.c

		Stuart Boutell, 30 Apr 1991
			changed cpsig() return code check from FAIL to SYSERROR 

		Geoff Clare, 24 Jan 1992
			Make allowance for alignment bytes in big_exec().

		Geoff Clare, 20 July 1992
			Cast pid_t to long for printing.

		Neil Moses, 22 October 1992
			XNFS enhancements

		Dave Cater, 3 December 1992
			Some fixes to XNFS start and cleanup code.

		Neil Moses, 7 December 1992
			Only do locking if file locking supported.

		Dave Cater, 10 December 1992
			Fixes to test21 to read results from pipe.

		Geoff Clare, 10 Feb 1993
			Revised use of XNFS DEFAULT/ROOT/SPEC file systems.
			Changed fdopen()+fputs() to write() in file_setup().

		Geoff Clare, 21 April 1993
			Added tet_sync() at end of prepare_tests().

		Geoff Clare, 4 Jan 1994
			Changed prepare_tests() to use tet_fork() instead
			of fork(), else tet_exec() closes parent connection.
			Inserted tet_logoff() calls before doexec() calls
			that are expected to succeed.

		Geoff Clare, 4 March 1994
			Moved "if (locksupp)" condition in file_setup().

		Geoff Clare, 20 Sept 1995
			Changed child timeout in pa_suproc().

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<pcts_max.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<sys/times.h>
#include	<sys/wait.h>
#include	<stdio.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<string.h>
#include	<pathres.h>
#include	<chktimes.h>
#include	<pcts_max.h>
#include	<exec.h>
#include <tet_api.h>
#include	<xnfs.h>
#include	"exec_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif


#define	UNKNOWN_FILE		"<unknown file>"
#define	MODEGEN		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

#define	MAXARG		8
#define	NAME()		(i_name[iwhich])
#define	WAITTIME		(5 * SPEEDFACTOR)
#define	EXTERR			"external error"
#define	NO_TESTS		50		/* number of tests in testset */
#define	DOTDOT		".."
#define	READ		0
#define	WRITE		1
#define	SLASH		"/"
#define	UMASK		022
#define	ZERORET		0
#define	VSX_SET_ID_MODES_SUPP	"VSX_SET_ID_MODES_SUPP"
#define PATH_OK	02

extern	int		doexec();
extern	void		init_exec();
extern	int		sig_init();
extern	void		sig_catch();
extern	void		vsx_sigs();
extern	void		valrpt();
extern	void		xx_all();
extern	char		*itoa();
extern	char		*ltoa();
extern	char		*newstr();
extern	char		*status_child();
extern	char		*errname();
extern	char		*execpath();
extern	int		file_copy();
#if TEST_XNFS
extern	char		startdir[];
#else /* TEST_XNFS */
private	char		startdir[] = "..";
#endif /* TEST_XNFS */
#ifndef getenv
extern	char		*getenv();
#endif

extern	int	vguid2;
extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;
extern	int	sig_array[];
extern	int	ex_code;	/* defined in wait_child.c */
extern	char	*t_path;
extern	char	*t_prog;
extern	char	tclose_file[];
extern	int	file_close;

public	int	no_tests = NO_TESTS;

public	int	p_argc;
public	char	*p_argv[MAXARG];
public	char	**p_envp;
public	long	arg_max;
public	int	arg_err;
public	int	sig_recvd;
public	sigset_t sig_empty;
public	int	job_control;
public	int	nsigs;
public	int	test_fail;
public	int	test_deleted;
public	int	iwhich;		/* what are we? (execl, execvp,... ) */
public	char	*i_name[6] = 		{ "execl",
					  "execle",
					  "execlp",
					  "execv",
					  "execve",
					  "execvp"
					};

public	char	t1_prog[] =		"../exec_tok";
public	char	tgen_prog[] =		"../exec_tgen";
public	char	tfail_prog[] =		"../exec_tfail";
public	char	su_prog[] =		"exec_su";
public	char	topen_file[] =		"./exec_t.open";
public	void	ch_fildes();
public	void	pa_exec();

private	int	t_portfnum;
private	int	lock_supp = 0;
private char	*rsn_s = 		"setup() failed to malloc memory";
private	char	texec_prog[] =		"exec_tok";
private	char	tnoent_prog[] =		"./exec_tok";
private	char	t6_path[] =		"PATH=..";
private	char	t7_path[] =		"PATH=/tmp:..";
private	char	t8_path[] =		"PATH=/tmp:..";
private	char	t8_failprog[] =		"..exec_tok";
private	char	t9_path[] =		"PATH=/tmp::/bin";
private	char	t10_path[] =		"PATH=:/tmp:/bin";
private	char	t11_path[] =		"PATH=/tmp:/bin:";
private	char	t12_path[] =		"PATH=/tmp:..::/bin";
private	char	t18_prog[] =		"../../exec_tok";
private	char  *	t19_prog[] =		{ "../exec_tp0",
					  "../exec_tp1",
					  "../exec_tp2",
					  "../exec_tp3",
					  "../exec_tp4",
					  "../exec_tp5",
					  "../exec_tp6",
					  "../exec_tp7",
					  "../exec_tp8",
					  "../exec_tp9"
					};

#if TEST_XNFS

private	char	*prog_list[] = PROG_LIST;
private int	snames[] = {1, 0};
private	char 	*mount_root;

private	char	su_xnfs[] =		"xnfs_su";
private	void	prep_child();
public	void	xnfs_prog();

#endif /* TEST_XNFS */

private	int	t_case;
private	int	file_open;
public	int	setupok = -1 ;

private	void		test1();
private	void		pa1();
private	void		test6();
private	void		ch6();
private	void		test7();
private	void		ch7();
private	void		test8();
private	void		ch8();
private	void		pa8();
private	void		test9();
private	void		ch9();
private	void		test10();
private	void		ch10();
private	void		test11();
private	void		ch11();
private	void		test12();
private	void		ch12();
private	void		pa12();
private	void		test15();
private	void		pa15();
private	void		test16();
private	void		test18();
private	void		pa18p1();
private	void		pa18p2();
private	void		test19();
private	void		pa19();
private	void		test20();
private	void		test21();
private	void		pa21p1();
private	void		pa21p2();
private	void		test22();
private	void		ch22();
private	void		test23();
private	void		ch23();
private	void		test24();
private	void		ch24();
private	void		test25_26();
private	void		test27_28();
private	void		test29();
private	void		testgen();
public	void		testuid();
private	void		big_exec();
private	void		exec_suproc();
private	void		pa_noent();
private	void		ch_big();
private	void		ch_noent();
private	void		prepare_tests();
#if TEST_XNFS
private	void		clean_tests();
#endif
private	void		setup();
private	void		file_setup();
public	void		signal_init();

extern	void		test30();
extern	void		test31();
extern	void		test32();
extern	void		test33();
extern	void		test34();
extern	void		test35();
extern	void		test36();
extern	void		test37();
extern	void		test38();
extern	void		test39();
extern	void		test40();
extern	void		test41();
extern	void		test42();
extern	void		test43();

extern	void		test44();
extern	void		test45();
extern	void		test46();
extern	void		test47();
extern	void		test48();
extern	void		test49();
extern	void		test50();

public	void		ch_exec();
public	void		pa_suproc();
public	void		ch_suproc();

extern	int		pipe_open();
extern	void		pipe_close();
extern	void		pipe_parent();
extern  void		pipe_report();
extern	void		pipe_child();

public	void		pipe_ch_exec();
private	void		pipe_ch_big();
private	void		pipe_ch_fildes();
public	void		pipe_ch_suproc();

public	void	(*setfunc) () = prepare_tests;

#if TEST_XNFS
public	void	(*clnfunc) () = clean_tests;
#else
public	void	(*clnfunc) () = NULL;
#endif

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	   testgen, 2,
	 testgen, 3,
	 testgen, 4,
	 testgen, 5,
	 test6, 6,
	   test7, 7,
	   test8, 8,
	 test9, 9,
	   test10, 10,
	  test11, 11,
	  test12, 12,
	 testgen, 13,
	 testgen, 14,
	 test15, 15,
	  test16, 16,
	 testgen, 17,
	 test18, 18,
	  test19, 19,
	  test20, 20,
	 test21, 21,
	  test22, 22,
	  test23, 23,
	  test24, 24,
	 test25_26, 25,
	 test25_26, 26,
	 test27_28, 27,
	 test27_28, 28,
	 test29, 29,
	  test30, 30,
	  test31, 31,
	  test32, 32,
	 test33, 33,
	 test34, 34,
	 test35, 35,
	 test36, 36,
	 test37, 37,
	 test38, 38,
	 test39, 39,
	 test40, 40,
	 test41, 41,
	 test42, 42,
	 test43, 43,
	 test44, 44,
	 test45, 45,
	 test46, 46,
	 test47, 47,
	 test48, 48,
	 test49, 49,
	 test50, 50,
	NULL, 0
};

private	void
prepare_tests()
{
#if TEST_XNFS
	int	i;
	char 	*rlock;
	char	buf[_POSIX_PATH_MAX];
	char	to_file[20];
#endif
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	/* guard against NULL vg_dbug being taken as arg list terminator */
	if (vg_dbug == NULL)
		vg_dbug = "-D";

	if(uids()!=0)
	{
		xx_all(DELETION, "uids() failed in setup");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

#if TEST_XNFS

	if ((rlock = tet_getvar("XNFS_REMLOCK_SUPP")) == NULL || *rlock == '\0')
	{
		(void)tet_delete(21, 
			"parameter XNFS_REMLOCK_SUPP is not defined");
	}
	else
		PATH_TRACE;

	if(*rlock == 'Y' || *rlock == 'y')
		lock_supp = 1;
	else
		lock_supp = 0;

	if ((mount_root = tet_getvar("XNFS_MOUNT_ROOT")) == NULL ||
			*mount_root == '\0')
	{
		xx_all(DELETION, "parameter XNFS_MOUNT_ROOT is not defined");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch (tet_fork(prep_child, NULLFN, 5*WAITTIME,
			XNFS_OK|XNFS_NOK|XNFS_CHDIR_FAIL|XNFS_EXEC_FAIL))
	{
	case -1:
		xx_all(DELETION, "tet_fork() failed in setup");
		DBUG_VOID_RETURN;

	case XNFS_OK:
		PATH_TRACE;
		break;

	case XNFS_NOK:
		xx_all(DELETION,
			"could not copy files to XNFS_MOUNT_ROOT in setup");
		DBUG_VOID_RETURN;

	case XNFS_CHDIR_FAIL:
		xx_all(DELETION,
			"child could not chdir() to XNFS_MOUNT_ROOT in setup");
		DBUG_VOID_RETURN;

	case XNFS_EXEC_FAIL:
		xx_all(DELETION, "tet_exec() failed in setup");
		DBUG_VOID_RETURN;

	default:
		xx_all(DELETION,
			"unexpected return code from tet_fork() in setup");
		DBUG_VOID_RETURN;
	}

	for(i = 0; prog_list[i] != NULL; i++)
	{
		if(i == 0)
			PATH_TRACE;

		(void) strcat(strcat(strcpy(buf, startdir), "/"), prog_list[i]);
		(void) strcat(strcpy(to_file, "../"), prog_list[i]);
		(void) unlink(to_file);
		ret = creat(to_file, MODEGEN);
		if (ret < 0)
		{
			in_rpt("creat(\"%s\", 0x%lx) failed in setup",
				to_file, (unsigned long)MODEGEN);
			xx_all(DELETION, "creat() failed in setup function");
			DBUG_VOID_RETURN;
		}
		(void) close(ret);
		if(file_copy(buf, to_file) == SYSERROR)
		{
			in_rpt("file_copy(\"%s\", \"%s\") failed in setup", buf,
				to_file);
			xx_all(DELETION, "file_copy() failed in setup function");
			DBUG_VOID_RETURN;
		}
	}

#else /* TEST_XNFS */

	lock_supp = 1;

	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;
	PATH_TRACE;

#endif /* TEST_XNFS */

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		job_control = 0;
		nsigs = VNOJSIG;
	} else {
		job_control = 1;
		nsigs = VNUMSIG;
	}

	errno = 0;
	arg_max = sysconf(_SC_ARG_MAX);
	arg_err = errno;

	setup();

#if TEST_XNFS
	/* Sync manually as auto sync could timeout on server */
	(void) tet_sync(SP_SETUP_END, snames, SYNC_TIMEOUT);
#endif

	PATH_SETUP_RPT(6);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
prep_child()
{
	DBUG_ENTER("prep_child");

	if (chdir(mount_root) != 0)
		DBUG_EXIT(XNFS_CHDIR_FAIL);
	xnfs_prog();
	DBUG_EXIT(XNFS_EXEC_FAIL);

	DBUG_VOID_RETURN;
}


public void
xnfs_prog()
{
	int 	err;
	char	*args[7];

	DBUG_ENTER("xnfs_prog");

	args[0] = su_xnfs;
	args[1] = startdir;
	args[2] = newstr(itoa(tet_thistest));
	args[3] = newstr(itoa(iwhich));
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = (char *) NULL;

	(void) tet_exec(execpath(su_xnfs), args, environ);

	err = errno;

	if (tet_thistest != 0)
	{
		xx_rpt(DELETION);
		in_rpt("tet_exec() failed - errno %s (%d)", errname(err), err);
	}

	DBUG_VOID_RETURN;
}

private	void
clean_tests()
{
	int i;

	DBUG_ENTER("clean_tests");

	(void) chdir("..");
	for(i = 0; prog_list[i] != NULL; i++)
		(void) unlink(prog_list[i]);

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

public void
signal_init()
{
	int i;
	DBUG_ENTER("signal_init");
	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}
	(void) sigemptyset(&sig_empty);
	DBUG_VOID_RETURN;
}

private	void
setup()
{
	int	i;
	char	*s;
	char	*p;
	static	char	delreason[80];

/*	This shouldn't be necessary - should've been done in the driver

	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}
*/
	if((s = strrchr(vg_ident, '/')) != (char *)NULL)
		s+=3;
	else
		s = vg_ident+2;
	if (strcmp(s, "execl" ) == 0)
		iwhich = I_EXECL;
	else if (strcmp(s, "execle") == 0)
		iwhich = I_EXECLE;
	else if (strcmp(s, "execlp") == 0)
		iwhich = I_EXECLP;
	else if (strcmp(s, "execv" ) == 0)
		iwhich = I_EXECV;
	else if (strcmp(s, "execve") == 0)
		iwhich = I_EXECVE;
	else if (strcmp(s, "execvp") == 0)
		iwhich = I_EXECVP;
	else
	{
		(void) strcpy(delreason, "testset id not set correctly");
		vsxerror("%s: is %s, should be exec{l,le,lp,v,ve,vp}",
			delreason, vg_ident);
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i, delreason);
		return;
	}
	/*
	 *	Setup the environment path for exec[vl]p
	 */
	if (iwhich == I_EXECVP || iwhich == I_EXECLP)
	{
		if ((s = tet_getvar("VSX_PATH")) == (char *) NULL)
			s = "::/bin:/usr/bin";
		if ((p = (char *)malloc((size_t) (strlen(s) + strlen(EXEC_PATH) + 1)))
							== (char *) NULL)
		{
			for (i = 1; i <= no_tests; i++)
				(void) tet_delete(i, rsn_s);
			return;
		}
		(void) strcat(strcpy(p, EXEC_PATH), s);
		if (vputenv(&environ, p) == SYSERROR)
		{
			(void) strcpy(delreason, "could not set environment path");
			vsxerror("%s to %s", delreason, p);
			for (i = 1; i <= no_tests; i++)
				(void) tet_delete(i, delreason);
			return;
		}
	}

	if((p_envp = (char **) malloc(sizeof(char *))) == (char **) NULL) {
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i, rsn_s);
		return;
	}
	p_envp[0] = (char *) NULL;
}

private void
test1()
{
	int	pathok = setupok;

	DBUG_ENTER("test1");

	test_fail = 0;
	test_deleted = 0;

	t_prog = t1_prog;
	t_path = EXEC_PATH;

	globok = 0;
	if(cpsig(ch_exec, pa1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
testgen()
{
	DBUG_ENTER("testgen");

	/* check setupok value - generate warning only */

	if (setupok != 0)
		DBUG_PRINTF("testgen","unexpected path trace value in prepare tests function");

	/*
	 * The test number "tet_thistest" is passed to tgen_prog, distinguishing 
	 * the various tests.
	 * This function is merely a test driver.
	 */

	test_fail = 0;
	test_deleted = 0;

	t_prog = tgen_prog;
	t_path = EXEC_PATH;

	if(pipe_open())
	{
		(void) cppair(pipe_ch_exec, NULLFN, WAITTIME * 4, 0);
		pipe_parent();
	}
	(void) pipe_close();
	DBUG_VOID_RETURN;
}

private void
pa1()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa1");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_EXECFAIL) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) returned unexpectedly", NAME(), t_prog);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
big_exec()
{
	/*
	 * Try to exec using as much data as possible.
	 */
	private	char	*p = (char *) NULL;
	private	char	**pp;
	int	ptr_max;		/* max space needed for pointers */
	int	i;
	size_t	size2;
	char	*big2;
	char	*str;
	static char numbuf[30];

	DBUG_ENTER("big_exec");

	p_argc = 7;
	p_argv[0] = t_prog;
	p_argv[1] = newstr(itoa(tet_thistest));
	p_argv[2] = newstr(ltoa(arg_max));
	p_argv[3] = numbuf;
	/* arg 4 to be calculated */
	p_argv[5] = vg_ident;
	p_argv[6] = vg_dbug;
	p_argv[7] = (char *) NULL;

	/* calculate space used by above arguments and their pointers */

	ptr_max = 2 * sizeof(char **) +		/* p_argv and pp */
			 9 * sizeof(char *)  + 	/* p_argv[0 to 7] plus *pp */
			 strlen(p_argv[0]) + 1 +
			 strlen(p_argv[1]) + 1 +
			 strlen(p_argv[2]) + 1 +
			 sizeof(numbuf) +
			 strlen(p_argv[5]) + 1 +
			 strlen(p_argv[6]) + 1;

	/* Add allowance for alignment bytes */

	ptr_max += 9 * (sizeof(char *) - 1);

	(void) sprintf(p_argv[3], "%-*d", sizeof(numbuf)-1, ptr_max);

	size2 = arg_max - ptr_max;
	big2 = (char *)malloc(size2);
	if (big2 == NULL)
	{
		vsxerror("malloc() failed");
		DBUG_EXIT(A_NOMEM);
	}
	for(str = big2, i = 0; i < size2; str++, i++) {
		if(i & 01) {
			*str = '1';
		} else {
			*str = '0';
		}
	}
	big2[size2 - 1] = '\0';
	p_argv[4] = big2;
	p_argv[5] = vg_ident;
	p_argv[6] = vg_dbug;
	p_argv[7] = (char *) NULL;
	p_argc = 7;

	pp = &p;
	if (iwhich != I_EXECLE && iwhich != I_EXECVE)
		environ = pp;

	DBUG_VOID_RETURN;
}

private void
test6()
{
	DBUG_ENTER("test6");

	if(iwhich != I_EXECLP && iwhich != I_EXECVP) {
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch6, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch6()
{
	int	pathok = setupok;

	DBUG_ENTER("ch6");

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Check that PATH is not searched when file name includes "/"
	 */
	t_prog = tnoent_prog;
	t_path = t6_path;

	globok = 0;
	if(cpsig(ch_noent, pa_noent, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/*
	 * Check that PATH is searched when file name does not include "/"
	 */
	t_prog = texec_prog;
	t_path = t6_path;

	globok = 0;
	if(cpsig(ch_exec, pa_exec, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

public void
pipe_ch_exec()
{
	DBUG_ENTER("pipe_ch_exec");
	pipe_child();
	ch_exec();
	DBUG_VOID_RETURN;
}

public void
ch_exec()
{
	int		err;

	DBUG_ENTER("ch_exec");

	DBUG_PRINTF("ch_exec", "tet_thistest is %d", tet_thistest);
	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2, P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
#if TEST_XNFS && !defined(DTET1)
	tet_logoff();
#endif
	(void) doexec();
	err = errno;
	vsxerror("%s(%s, ...) failed - errno %d (%s)",
		NAME(), t_prog, err, errname(err));
	DBUG_EXIT(A_EXECFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

public void
pa_exec()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_exec");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_EXECFAIL) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) failed to search PATH \"%s\"", 
						NAME(), t_prog, t_path);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_noent()
{
	int		err;

	DBUG_ENTER("ch_noent");

	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2, P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
	(void) doexec();
	err = errno;
	if(err != ENOENT) {
		vsxerror("%s(%s, ...) failed - errno %d (%s)",
			NAME(), t_prog, err, errname(err));
		DBUG_EXIT(A_TSETDIR);
	} else 
		DBUG_EXIT(A_EXECFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_noent()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_noent");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == OK) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) searched PATH unexpectedly", NAME(), t_prog);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != A_EXECFAIL) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d",
								A_EXECFAIL);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	DBUG_ENTER("test7");

	if(iwhich != I_EXECLP && iwhich != I_EXECVP) {
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch7, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch7()
{
	int	pathok = setupok;

	DBUG_ENTER("ch7");

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Check that PATH contains prefixes separated by colons.
	 */
	t_prog = texec_prog;
	t_path = t7_path;

	globok = 0;
	if(cpsig(ch_exec, pa_exec, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	DBUG_ENTER("test8");

	if(iwhich != I_EXECLP && iwhich != I_EXECVP) {
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch8, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch8()
{
	int	pathok = setupok;

	DBUG_ENTER("ch8");

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Check that PATH is searched from beginning to end.
	 */
	t_prog = texec_prog;
	t_path = t8_path;

	/*
	 * Copy a file that will give failure on execution into current 
	 * directory with a name preceded directly by the path prefix that 
	 * will be used.
	 */
	if(file_copy(tfail_prog, t8_failprog) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: file_copy() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if(cpsig(ch_exec, pa8, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	(void) unlink(t8_failprog);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	(void) unlink(t8_failprog);

	DBUG_VOID_RETURN;
}

private void
pa8()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa8");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_EXECFAIL) {
		xx_rpt(DELETION);
		in_rpt("%s(%s, ...) returned unexpectedly", NAME(), t_prog);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_MISC) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) failed to insert \"/\" after path prefix",
							 NAME(), t_prog);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	DBUG_ENTER("test9");

	if(iwhich != I_EXECLP && iwhich != I_EXECVP) {
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch9, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch9()
{
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch9");

	test_fail = 0;
	test_deleted = 0;

	if(chdir(DOTDOT) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: chdir() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Check that "::" in PATH specifies current working directory.
	 */
	t_prog = texec_prog;
	t_path = t9_path;

	globok = 0;
	if(cpsig(ch_exec, pa_exec, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	DBUG_ENTER("test10");

	if(iwhich != I_EXECLP && iwhich != I_EXECVP) {
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch10, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch10()
{
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch10");

	test_fail = 0;
	test_deleted = 0;

	if(chdir(DOTDOT) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: chdir() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Check that leading ":" in PATH specifies current working directory.
	 */
	t_prog = texec_prog;
	t_path = t10_path;

	globok = 0;
	if(cpsig(ch_exec, pa_exec, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test11()
{
	DBUG_ENTER("test11");

	if(iwhich != I_EXECLP && iwhich != I_EXECVP) {
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch11, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch11()
{
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch11");

	test_fail = 0;
	test_deleted = 0;

	if(chdir(DOTDOT) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: chdir() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Check that trailing ":" in PATH specifies current working directory.
	 */
	t_prog = texec_prog;
	t_path = t11_path;

	globok = 0;
	if(cpsig(ch_exec, pa_exec, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	DBUG_ENTER("test12");

	if(iwhich != I_EXECLP && iwhich != I_EXECVP) {
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch12, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch12()
{
	int	pathok = setupok;

	DBUG_ENTER("ch12");

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Check that PATH is searched from beginning to end.
	 */
	t_prog = texec_prog;
	t_path = t12_path;

	/*
	 * Copy a file that will give failure on execution into current 
	 * directory. This means the test will fail if the DOTDOT entry
	 * in t12_path, which precedes DOT, is not found first.
	 */
	if(file_copy(tfail_prog, t_prog) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: file_copy() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if(cpsig(ch_exec, pa12, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	(void) unlink(t_prog);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	(void) unlink(t_prog);

	DBUG_VOID_RETURN;
}

private void
pa12()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa12");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_EXECFAIL) {
		xx_rpt(DELETION);
		in_rpt("%s(%s, ...) returned unexpectedly", NAME(), t_prog);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_MISC) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) searched PATH backwards", NAME(), t_prog);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test15()
{
	int	pathok = setupok;

	DBUG_ENTER("test15");

	test_fail = 0;
	test_deleted = 0;

	if (arg_max < 0 && !(arg_max == -1 && arg_err == 0))
	{
		xx_rpt(DELETION);
		in_rpt("sysconf(_SC_ARG_MAX) failed in prep");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t_prog = t1_prog;
	t_path = EXEC_PATH;

	globok = 0;
	if(cpsig(ch_big, pa15, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
pa15()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa15");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_EXECFAIL) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) did not accept ARG_MAX (%ld) bytes of data",
						NAME(), t_prog, arg_max);
		in_rpt("instead %s(%s, ...) returned with errno set to E2BIG", 
							NAME(), t_prog);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	DBUG_ENTER("test16");

	/*
	 * The test number "tet_thistest" is passed to test16.
	 * This function is merely a test driver.
	 */

	test_fail = 0;
	test_deleted = 0;

	if (arg_max < 0 && !(arg_max == -1 && arg_err == 0))
	{
		xx_rpt(DELETION);
		in_rpt("sysconf(_SC_ARG_MAX) failed in prep");
		DBUG_VOID_RETURN;
	}

	t_prog = tgen_prog;
	t_path = EXEC_PATH;
	if(pipe_open())
	{
		(void) cppair(pipe_ch_big, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
pipe_ch_big()
{
	DBUG_ENTER("pipe_ch_big");
	pipe_child();
	ch_big();
	DBUG_VOID_RETURN;
}


private void
ch_big()
{
	int		err;

	DBUG_ENTER("ch_big");

	if (arg_max == -1)
		/* OK to modify this in child process */
		arg_max = PCTS_ARG_MAX;

	big_exec();
#if TEST_XNFS && !defined(DTET1)
	tet_logoff();
#endif
	(void) doexec();
	err = errno;
	vsxerror("%s(%s, ...) failed - errno %d (%s)",
		NAME(), t_prog, err, errname(err));
	if(err != E2BIG) {
		vsxerror("%s(%s, ...) failed - errno %d (%s)",
			NAME(), t_prog, err, errname(err));
		DBUG_EXIT(A_TSETDIR);
	} else 
		DBUG_EXIT(A_EXECFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
test18()
{
	/* pathname resolution */

	char 	*p1, *p2;
	int 	npairs, i;
	int	pathok = setupok;

	DBUG_ENTER("test18");

	test_fail = 0;
	test_deleted = 0;

	npairs = init_pathres(PRES_FILES);
	if (npairs <= 0)
	{
		xx_rpt(DELETION);
		in_rpt("init_pathres(PRES_FILES) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0; i < npairs; i++)
	{
		p1 = pathresnam[2*i];
		p2 = pathresnam[2*i+1];

		DBUG_PRINTF("test18", "testing \"%s\" == \"%s\"", p1, p2);

		/* Copy executable file into required place. */
		if(file_copy(t18_prog, p2) == SYSERROR) {
			if(!test_fail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: file_copy() failed");
			test_deleted++;
			break;
		}
		else
			PATH_TRACE;

		/* Ensure it can be executed with simple path name */
		t_prog = p2;
		t_path = EXEC_PATH;

		globok = 0;
		if(cpsig(ch_exec, pa18p1, WAITTIME, 0) == SYSERROR) {
			test_deleted++;
			break;
		}
		else
			PATH_FUNC_TRACE;

		if(test_deleted)
			break;

		/* Check complex path name resolves to same executable file */
		t_prog = p1;
		t_path = EXEC_PATH;

		globok = 0;
		if(cpsig(ch_exec, pa18p2, WAITTIME, 0) == SYSERROR) {
			test_deleted++;
			break;
		}
		else
			PATH_FUNC_TRACE;

		/* Clear out copy so next path test is unhindered */
		(void) unlink(p1);
	}

	if(!test_fail && !test_deleted)
		PATH_XS_RPT(1+(3*npairs));
	
	cln_pathres();

	DBUG_VOID_RETURN;
}

private void
pa18p1()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa18p1");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_EXECFAIL) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("%s(%s, ...) returned unexpectedly", NAME(), t_prog);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
pa18p2()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa18p2");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_EXECFAIL) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) - path name not resolved correctly", NAME(),
									t_prog);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test19()
{
	/* portable filenames */

	char	**p;
	int	i;
	int	fd, err;
	mode_t	mode = MODEGEN;
	int	loop = 0;
	int	pathok = setupok;
	

	DBUG_ENTER("test19");

	test_fail = 0;
	test_deleted = 0;

	i = 0;
	for (p = portfnam; *p != NULL; p++) {	
		if (i++ == 0)
			PATH_TRACE;
		if (eaccess(*p, F_OK) == 0 && remove(*p) == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("could not remove \"%s\" - errno %d (%s)",
				*p, err, errname(err));
			DBUG_VOID_RETURN;
		}
	}

	i = 0;
	for (p = portfnam; *p != NULL; p++) {
		if (i++ == 0)
			PATH_TRACE;
		fd = creat(*p, mode);
		if(fd == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
				*p, (long)mode, err, errname(err));
			DBUG_VOID_RETURN;
		}
		(void) close(fd);
	}

	/* exec each portable file name - check expected exit code */
	for (i = 0, p = portfnam; *p != NULL; i++, p++) {

		/* Copy executable file into required place. */
		if(file_copy(t19_prog[i], *p) == SYSERROR) {
			if(!test_fail)
				xx_rpt(DELETION);
			in_rpt("deletion reason: file_copy() failed");
			test_deleted++;
			break;
		}
		else
			PATH_TRACE;

		/* The exit code should correspond to the loop index */
		t_portfnum = i;
		t_prog = *p;
		t_path = EXEC_PATH;

		globok = 0;
		if(cpsig(ch_exec, pa19, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		else
			PATH_FUNC_TRACE;

		(void) unlink(*p);

		if(test_deleted)
			break;

		loop++;
	}

	if(!test_fail && !test_deleted)
		PATH_XS_RPT(2+(2*loop));
	
	DBUG_VOID_RETURN;
}

private void
pa19()
{
	char	**p;
	char	*s = UNKNOWN_FILE;
	int	i;
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa19");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_EXECFAIL) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("%s(%s, ...) returned unexpectedly", NAME(), t_prog);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	for (i = 0, p = portfnam; *p != NULL; i++, p++) {
		if (i==0)
			PATH_TRACE;
		if(i == ex_code) {	
			s = *p;
			break;
		}
	}
		
	if(stat_child != CHLD_EXIT || ex_code != t_portfnum) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) executed wrong file %s", NAME(), t_prog, s);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test20()
{
	DBUG_ENTER("test20");

	test_fail = 0;
	test_deleted = 0;

	t_prog = tgen_prog;
	t_path = EXEC_PATH;

	if(pipe_open())
	{
		(void) cppair(pipe_ch_fildes, NULLFN, WAITTIME * 2, 0);
		pipe_parent();
	}
	pipe_close();

	(void) unlink(topen_file);
	(void) unlink(tclose_file);

	DBUG_VOID_RETURN;
}

private void
test21()
{
	int	pathok = setupok;

	DBUG_ENTER("test21");

	test_fail = 0;
	test_deleted = 0;

	t_prog = tgen_prog;
	t_path = EXEC_PATH;
	t_case = 1;

	if(!pipe_open())
		DBUG_VOID_RETURN;

	globok = 0;
	if(cpsig(pipe_ch_fildes, pa21p1, 2*WAITTIME, 0) == SYSERROR) 
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	(void) unlink(topen_file);
	(void) unlink(tclose_file);

	if(lock_supp)
	{
		t_case = 2;
		{
		globok = 0;
		if(cpsig(pipe_ch_fildes, pa21p2, 2*WAITTIME, 0) == SYSERROR) 
			test_deleted++;
		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
		}
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	(void) pipe_close();

	(void) unlink(topen_file);
	(void) unlink(tclose_file);

	DBUG_VOID_RETURN;
}

private void
pa21p1()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa21p1");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME/2)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_MISC) {
		test_fail++;
		pipe_parent();
	} else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_TSETDIR) {
		test_deleted++;
		pipe_parent();
	} else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == PATH_OK)
		PATH_TRACE;

	if(stat_child != CHLD_EXIT) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

private void
pa21p2()
{
	int	err;
	int	stat_child;
	int	fail = 0;
	struct	flock	lck;
	int	pathok = 0;

	DBUG_ENTER("pa21p2");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME/2)) == CHLD_ERR) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT) {
		test_fail++;
		pipe_parent();
		DBUG_VOID_RETURN;
	} else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("expected action: process still active");
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* 
	 * Obtain lock information on test file.
	 */
	if ((file_open = open(topen_file, FD_MODE)) == SYSERROR) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: open() failed - errno %d (%s)",
					err, errname(err));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	lck.l_whence = SEEK_SET;
	lck.l_type   = FD_LCKTYPE;
	lck.l_start  = FD_LCKSTART-1;
	lck.l_len    = FD_LCKLEN+1;
	if(fcntl(file_open, F_GETLK, &lck) == SYSERROR) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("fcntl(F_GETLK) failed - errno %d (%s)", 
						err, errname(err));
		test_deleted++;
		(void) close(file_open);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(lck.l_whence != SEEK_SET) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("l_whence not set to %d after fcntl(F_GETLK)", SEEK_SET);
		test_deleted++;
		(void) close(file_open);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(lck.l_type  != FD_LCKTYPE ||
	   lck.l_start != FD_LCKSTART ||
	   lck.l_len   != FD_LCKLEN ||
	   lck.l_pid   != tet_child)
		fail |= 040;
	else
		PATH_TRACE;
	  
	if(fail) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("file locks altered by exec:");
		in_rpt("Expected values: Type: %d Start: %d Length: %d Process ID: %ld",
			FD_LCKTYPE, FD_LCKSTART, FD_LCKLEN, (long)tet_child);
		in_rpt("Observed values: Type: %d Start: %d Length: %d Process ID: %ld",
			lck.l_type, lck.l_start, lck.l_len, (long)lck.l_pid);
		test_fail++;
		(void) close(file_open);
		DBUG_VOID_RETURN;
	}

	(void) close(file_open);

	PATH_FUNC_RPT(7);

	DBUG_VOID_RETURN;
}

private void
test22()
{
	DBUG_ENTER("test22");

	t_prog = tgen_prog;
	t_path = EXEC_PATH;

	if(pipe_open())
	{
		(void) cppair(ch22, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();
	

	DBUG_VOID_RETURN;
}

private void
ch22()
{
	DBUG_ENTER("ch22");

	if(sig_init(SIGQUIT, &sig_empty, SIG_DFL, 0) < 0) {
		pipe_report(DELETION,"deletion reason - sigaction() failed - signal SIGQUIT");
		DBUG_VOID_RETURN;
	}

	(void) cppair(pipe_ch_exec, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
test23()
{
	DBUG_ENTER("test23");

	t_prog = tgen_prog;
	t_path = EXEC_PATH;
	if(pipe_open())
	{
		(void) cppair(ch23, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
ch23()
{
	DBUG_ENTER("ch23");

	if(sig_init(SIGQUIT, &sig_empty, SIG_IGN, 0) < 0) {
		pipe_report(DELETION,"deletion reason - sigaction() failed - signal SIGQUIT");
		DBUG_VOID_RETURN;
	}

	(void) cppair(pipe_ch_exec, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
test24()
{
	DBUG_ENTER("test24");

	t_prog = tgen_prog;
	t_path = EXEC_PATH;

	if(pipe_open())
	{
		(void) cppair(ch24, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
ch24()
{
	DBUG_ENTER("ch24");

	if(sig_init(SIGQUIT, &sig_empty, sig_catch, 0) < 0) {
		pipe_report(DELETION,"deletion reason - sigaction() failed - signal SIGQUIT");
		DBUG_VOID_RETURN;
	}

	(void) cppair(pipe_ch_exec, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

public void
testuid()
{

	DBUG_ENTER("testuid");

	t_prog = su_prog;

	if(pipe_open())
		(void)cpsig(pipe_ch_suproc, pa_suproc, -1, 0 );
	pipe_close();
	
	DBUG_VOID_RETURN;
}

public void
pipe_ch_suproc()
{
	pipe_child();
	ch_suproc();
}

public void
ch_suproc()
{
	DBUG_ENTER("ch_suproc");

	exec_suproc();
	
	DBUG_VOID_RETURN;
}

public void
pa_suproc()
{
	int	err;
	int	stat_child;
	int	waittime = 5*WAITTIME;
	int	pathok = setupok;

	DBUG_ENTER("pa_suproc");

	/* Wait for the sending child */

	if (tet_thistest == 32)	/* do_access() test */
		waittime *= (5 + sysconf(_SC_NGROUPS_MAX)/4);

	errno = NOERROR;
	if((stat_child = wait_child(waittime)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != RPTED) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", RPTED);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		pipe_parent();
		PATH_TRACE;
	}

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
exec_suproc()
{
	int	err;
	char	*teststr;
	char	*intstr;
	char	*args[7];

	DBUG_ENTER("exec_suproc");

	if ((teststr = newstr(itoa(tet_thistest))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((intstr = newstr(itoa(iwhich))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	
	/* Can't use execle() - null vg_dbug stops environ being passed */

	args[0] = t_prog;
	args[1] = teststr;
	args[2] = intstr;
	args[3] = startdir;
	args[4] = vg_ident;
	args[5] = vg_dbug;
	args[6] = (char *) NULL;

	(void) tet_exec(execpath(t_prog), args, environ);
	err = errno;
	vsxerror("tet_exec(%s,..) failed errno %d", execpath(t_prog), err);
	DBUG_EXIT(A_EXECFAIL);

	DBUG_VOID_RETURN;
}

private void
test25_26()
{
	DBUG_ENTER("test25_26");

#if TEST_XNFS
	xnfs_doroot_c(testuid, WAITTIME*2, SP_NOSYNC, SP_NOSYNC);
#else
	testuid();
#endif

	DBUG_VOID_RETURN;
}

private void
test27_28()
{

	char	*setid_supp;

	DBUG_ENTER("test27_28");

#if TEST_XNFS

	if(tet_thistest == 27)
	{
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}

#endif /* TEST_XNFS */

	setid_supp = tet_getvar(VSX_SET_ID_MODES_SUPP);
	DBUG_PRINTF("test27_28","VSX_SET_ID_MODES_SUPP = %s", setid_supp);
	if ( setid_supp ) {
		if ( (setid_supp[0] == 'y' || setid_supp[0] == 'Y' ) ) {
#if TEST_XNFS
			xnfs_doroot_c(testuid, WAITTIME*2, SP_NOSYNC, SP_NOSYNC);
#else
			testuid();
#endif /* TEST_XNFS */
			DBUG_VOID_RETURN;
		}
		if ( (setid_supp[0] == 'n' || setid_supp[0] == 'N' ) ) {
			xx_rpt(UNSUPPORTED);
			DBUG_VOID_RETURN;
		}
		/* else fall thru */
	}

	xx_rpt(DELETION);
	in_rpt("Parameter %s is not set to 'y', 'Y', 'n' or 'N'", 
		VSX_SET_ID_MODES_SUPP);
	DBUG_VOID_RETURN;
}

private void
test29()
{
	int	err;

	DBUG_ENTER("test29");

	errno = NOERROR;
	if(sysconf(_SC_SAVED_IDS) == -1) {
		err = errno;
		if(err != NOERROR) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sysconf() failed errno %d", 
									err);
			DBUG_VOID_RETURN;
		} else {
			xx_rpt(UNSUPPORTED);
			DBUG_VOID_RETURN;
		}
	}

#if TEST_XNFS
	xnfs_doroot_c(testuid, WAITTIME*2, SP_NOSYNC, SP_NOSYNC);
#else
	testuid();
#endif /* TEST_XNFS */
	
	DBUG_VOID_RETURN;
}

private	void
pipe_ch_fildes()
{
	DBUG_ENTER("pipe_ch_fildes");
	pipe_child(); ch_fildes();
	DBUG_VOID_RETURN;
}

public void
ch_fildes()
{
	char	*str0, *str1, *str2, *str3;
	int	err;

	DBUG_ENTER("ch_fildes");

	file_setup();
	if((str0 = newstr(itoa(tet_thistest))) == (char *)NULL ||
	   (str1 = newstr(itoa(file_open))) == (char *)NULL ||
	   (str2 = newstr(itoa(file_close))) == (char *)NULL ||
	   (str3 = newstr(itoa(t_case))) == (char *)NULL) {
		err = errno;
		vsxerror("malloc() failed - errno %d (%s)", err, errname(err));
		DBUG_EXIT(A_NOMEM);
	}
	(void) init_exec(t_prog, str0, str1, str2, str3, vg_ident, vg_dbug);
#if TEST_XNFS && !defined(DTET1)
	tet_logoff();
#endif
	(void) doexec();
	err = errno;
	vsxerror("%s(%s, ...) failed - errno %d (%s)",
		NAME(), t_prog, err, errname(err));
	DBUG_EXIT(A_EXECFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}


private void
file_setup()
{
	int	err;
	int	flags;
	struct	flock	lck;

	DBUG_ENTER("file_setup");

	if ((file_open = creat(topen_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			topen_file, (long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	(void) close(file_open);

	/*
	 * For test20(), test21()
	 */
	/* Open a file which should stay open after exec */
	if ((file_open = open(topen_file, FD_MODE)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open() failed - errno %d (%s)",
					err, errname(err));
		DBUG_VOID_RETURN;
	}
	if((flags = fcntl(file_open, F_GETFD, 0)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_GETFD) failed - errno %d (%s)",
					err, errname(err));
		DBUG_VOID_RETURN;
	}
	flags &= ~FD_CLOEXEC;
	if((flags = fcntl(file_open, F_SETFD, flags)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_SETFD) failed - errno %d (%s)",
					err, errname(err));
		DBUG_VOID_RETURN;
	}

	/*
	 * For test21().
	 */
	/* Set file status flags */
	if((flags = fcntl(file_open, F_GETFL, 0)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_GETFL) failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	flags |= O_APPEND;
	flags |= O_NONBLOCK;
	if((flags = fcntl(file_open, F_SETFL, flags)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_SETFL) failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}

	/* Set file offset */
	if(lseek(file_open, FD_OFF, SEEK_SET) == (off_t)SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: lseek() failed - errno %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}

	if(lock_supp)
	{
		/* Set a file lock */
		lck.l_whence = SEEK_SET;
		lck.l_type   = FD_LCKTYPE;
		lck.l_start  = FD_LCKSTART;
		lck.l_len    = FD_LCKLEN;
		if(fcntl(file_open, F_SETLK, &lck) == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("fcntl(F_SETLK) failed - errno %d (%s)", 
							err, errname(err));
			DBUG_VOID_RETURN;
		}
	}

	/*
	 * For test20().
	 */
	/* Open a file which should be closed after exec */
	if ((file_close = creat(tclose_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			tclose_file, (long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	(void) close(file_close);

	if ((file_close = open(tclose_file, O_RDWR)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: open() failed - errno %d (%s)",
					err, errname(err));
		DBUG_VOID_RETURN;
	}
	if((flags = fcntl(file_close, F_GETFD, 0)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_GETFD) failed - errno %d (%s)",
					err, errname(err));
		DBUG_VOID_RETURN;
	}
	flags |= FD_CLOEXEC;
	if((flags = fcntl(file_close, F_SETFD, flags)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl(F_SETFD) failed - errno %d (%s)",
					err, errname(err));
		DBUG_VOID_RETURN;
	}
#if TEST_XNFS
	/*
	 * For test50()
	 */
	if(write(file_close, T50_DATA, sizeof(T50_DATA)) <= 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: write() failed in file_setup() - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
#endif /* TEST_XNFS */

	DBUG_VOID_RETURN;
}
