/*
*      SCCS:  @(#)POSIX.os/procprim/exec/exec1.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/exec - T.exec Rel 4.4.1 (06/09/97)";
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		exec1.c

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the exec() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Wed May 31 12:39:15 BST 1989
MODIFICATIONS:
		Geoff Clare, 30 June 1989
			Don't do test30 part 2 (stopped process tests)
			when job control not supported.

		Dave Cater, 11 July 1989
			Add test (number 32) using do_access().
			Remove some unused definitions.

		Geoff Clare, 19 Oct 1989
			Check malloc() return values.

		Dave Cater, 1 Nov 1989
			Remove _POSIX_SAVED_IDS check from test31().
			Make error message better in test40().
		
		Geoff Clare, 6 Apr 1990
			New interface to nametoolng() and pathtoolng().

		Steve Ratcliffe, 24 Apr 1990
			Test to check errno when exec'ing a file type that is not
			executable.
			---
			Modified so that message is sent down pipe to parent
			to verify that exec did not happen.

		Geoff Clare, 30 May 1990
			Allow for unlimited ARG_MAX

		Keith Holder, 22 Jun 1990
			Added checks, to see if the argv and envp
			arguments to exec changed after an
			unsuccessful exec.

		Geoff Clare, 26 June 1990
			Add test that directory streams are closed across exec

		David Sawyer
		Mon Aug 20 12:30:13 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated tet_child declaration.

		Stuart Boutell, 14 Nov 1990
			Added signal_init() to test30.

		Stuart Boutell, 21 Nov 1990
			Changed test 33 to use pipe_reporting

		Stuart Boutell, 30 Apr 1991
			Changed cpsig() return code check from FAIL to
			SYSERROR

		Geoff Clare, 27 June 1991
			VSX_NONEXEC_FILE="unsup" for unsupported result

		Dave Cater, 9 June 1992
			Expanded do_save etc to save and compare new things.
			Altered save_env to save_envp.
			Altered cmp_argv to cmp_args.

		Geoff Clare, 16 June 1992
			Only use exec_setid to change ownership of t35_dir
			if it exists.

                John-Paul, 29 July 1992
                        Mode parameter in call to crdir() changed from 'int'
			to 'mode_t' type.

		Neil Moses, 22 October 1992
			XNFS enhancements

		Dave Cater, 10 December 1992
			Corrected test 35/36 to check returns from tet_fork.
		
		Geoff Clare, 10 Feb 1993
			Revised use of XNFS DEFAULT/ROOT/SPEC file systems.

		Geoff Clare, 2 Nov 1993
			Use file created on server in XNFS st_atime test
			to avoid reading cached data.

		Geoff Clare, 7 Jan 1994
			Insert tet_logoff() calls before doexec() calls that
			are expected to succeed.

		Geoff Clare, 7 March 1994
			Minor reporting change in test36.

		Geoff Clare, 20 Sept 1995
			Change child timeout in test 32.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<dirent.h>
#include	<pcts_max.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<sys/times.h>
#include	<sys/wait.h>
#include	<stdio.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<pathres.h>
#include	<chktimes.h>
#include	<string.h>
#include	<exec.h>
#include <tet_api.h>
#include	<xnfs.h>
#include	"exec_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif


#define	MODEGEN		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	MODESETID	(S_IRWXU|S_IRWXG|S_IRWXO)|(S_ISUID|S_ISGID)
#define	T35_MODE	((S_IRWXU|S_IRWXG|S_IRWXO) & ~(S_IXUSR|S_IXGRP|S_IXOTH))
#define	T36_MODE	((S_IRWXU|S_IRWXG|S_IRWXO) & ~(S_IXUSR|S_IXGRP|S_IXOTH))|(S_ISUID|S_ISGID)

#define	MAXARG		8
#define	NAME()		(i_name[iwhich])
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	READ		0
#define	WRITE		1
#define	SLASH		"/"
#define	UMASK		022
#define	ZERORET		0
#define NOK		2
#define E_EXEC		4

#define	VSX_SET_ID_MODES_SUPP	"VSX_SET_ID_MODES_SUPP"
#define VSX_NONEXEC_FILE	"VSX_NONEXEC_FILE"

extern	void		valrpt();
extern	void		clntoolng();
extern	char		*itoa();
extern	char		*ltoa();
extern	char		*newstr();
extern	char		*execpath();
extern	char		*status_child();
extern	char		**pathtoolng();
#if TEST_XNFS
extern  void    	xnfs_prog();
extern  char    	*xnfs_nametoolng();
#else
extern	char		**nametoolng();
#endif
extern	char	*errname();
extern	char	*signame();
extern	void	init_exec();
extern	void	ch_exec();
extern	void	sig_catch();
extern	int	sig_init();
extern	int	doexec();
extern	int	file_copy();
extern	void	pa_suproc();
extern	void	testuid();
extern	void	signal_init();
extern	void	pipe_close();
extern	int	pipe_open();
extern	void	pipe_parent();
extern	void	pipe_child();
extern	void	pipe_report();
extern	void	pipe_ch_suproc();
extern	void	ch_fildes();
extern	void	pa_exec();

extern	void	do_save();
extern	void	do_cmp();

extern	int	vguid1;
extern	int	vggid1;
extern	int	vguid2;
extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;
extern	char	**p_envp;
extern	int	sig_array[];
extern	int	ex_code;	/* defined in wait_child.c */
extern	int	p_argc;
extern	char	*p_argv[MAXARG];
extern	char	*t_prog;
extern	char	su_prog[];
extern	char	*t_path;
extern	long	arg_max;
extern	int	arg_err;
extern	int	iwhich;		/* what are we? (execl, execvp,... ) */
extern	int	sig_recvd;
extern	sigset_t sig_empty;
extern	int	job_control;
extern	int	nsigs;
extern	int	test_fail;
extern	int	test_deleted;
extern	 char	*i_name[];
extern	char	t1_prog[];
extern	char	tgen_prog[];
extern	char	tfail_prog[];
extern	char	tclose_file[];
extern	char	topen_file[];
extern	int	setupok;

#if TEST_XNFS
private	int		snames[] = {1, 0};
#endif /* TEST_XNFS */

private	char	tstop_prog[] =		"../exec_tstop";
#if TEST_XNFS
private	char	t31_prog[] =		"./exec-t.31";
#else
#define		t31_prog		t1_prog
#endif /* TEST_XNFS */
private	char	t35_file[] =		FILE35;
private	char	t35_dir[] =		DIR35;
private	char	t35_prog[] =		"../exec_tp4";
private	char	t36_file[] =		FILE36;
private	char	t36_prog[] =		"../exec_tp4";
private	char	t37_prog[] =		"../exec_tok";
#if !TEST_XNFS
private	char	t38_prog[] =		"../exec_tok";
#endif
private	char	t39_1_file[] =		"./exec-t.39";
private	char	t39_dir[] =		"./exec-d.39";
private	char	t39_2_file[] =		"./exec-d.39/exec-t.39";
private	char	t40_file[] =		"";
private	char	t41_1_file[] =		"./exec-t.41";
private	char	t41_2_file[] =		"./exec-t.41/exec-t.41a";
private	char	t42_file[] =		"./exec-t.42";
#if TEST_XNFS
private	char	t49_prog[] =		"../exec_uids";
private	char	t50_prog[] =		"../exec_tok";
#endif /* TEST_XNFS */
private	char	t_setid[] =		"exec_setid";
private	struct	tres t30ch;
private	struct	tres t30pa;
private	int	res_pipe[2];
private	char	*tsetid_file;
private	int	tsetid_mode;

private	void		do_t30();
private	void		pa30p1();
private	void		ch30p1();
private	void		pa30p2();
private	void		ch30p2();
private	void		pa31();
private	void		ch1t37();
private	void		ch2t37();
private	void		ch1t38();
private	void		ch2t38();
private	void		ch43();
private	void		pa43();
private	void		cht33();
private	void		t35_pt2();
private	void		cht35();
private	void		t36_pt2();
private	void		cht36();
private	void		cht39();
private	void		cht40();
private	void		cht41();
private	void		cht42();
#if TEST_XNFS
private	void		cht44();
private	void		cht45();
private	void		cht46();
private	void		cht47();
private	void		cht49();
private	void		cht50();
#endif
private	void		pa_toobig();
private	void		ch_toobig();
private	void		exec_setid();
private	void		results_exec();
private	void		cmp_grps();
private	void		cmp_mask();
private	void		before_exec();
private	void		vxrmdir();
private	void		toobig_exec();
private	int 		checkxperm();


public void
test30()
{
	DBUG_ENTER("test30");

#if TEST_XNFS
	xnfs_doroot_c(do_t30, 10*WAITTIME, SP_NOSYNC, SP_NOSYNC);
#else
	do_t30();
#endif

	DBUG_VOID_RETURN;
}

private void
do_t30()
{
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("do_t30");
	signal_init();

	test_fail = 0;
	test_deleted = 0;

	if(pipe(res_pipe) != 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t_prog = tgen_prog;

	globok = 0;
	if(cpsig(ch30p1, pa30p1, WAITTIME, 0) == SYSERROR)
		test_deleted++;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	t_prog = tstop_prog;

	if (job_control)
	{
		if(cpsig(ch30p2, pa30p2, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;


	PATH_FUNC_RPT(3); /* set globok to be checked in results_exec */

	results_exec();

	DBUG_VOID_RETURN;
}

private void
results_exec()
{
	int	pathok = 0;

	DBUG_ENTER("results_exec");

	PATH_FUNC_TRACE;

	if(t30ch.pid != t30pa.pid) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), Process ID not inherited", NAME());
		in_rpt("Expected value %ld; Observed value: %ld",
			(long)t30pa.pid, (long)t30ch.pid);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(t30ch.ppid != t30pa.ppid) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), Parent process ID not inherited", NAME());
		in_rpt("Expected value %ld; Observed value: %ld",
			(long)t30pa.ppid, (long)t30ch.ppid);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(t30ch.pgrp != t30pa.pgrp) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), Process group not inherited", NAME());
		in_rpt("Expected value %ld; Observed value: %ld",
			(long)t30pa.pgrp, (long)t30ch.pgrp);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(t30ch.uid != t30pa.uid) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), real user ID not inherited", NAME());
		in_rpt("Expected value %ld; Observed value: %ld",
			(long)t30pa.uid, (long)t30ch.uid);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(t30ch.gid != t30pa.gid) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), real group ID not inherited", NAME());
		in_rpt("Expected value %ld; Observed value: %ld",
			(long)t30pa.gid, (long)t30ch.gid);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(t30ch.ngrps != t30pa.ngrps) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), Number of supplementary groups not inherited", NAME());
		in_rpt("Expected value %d; Observed value: %d", t30pa.ngrps, 
								t30ch.ngrps);
		test_fail++;
	} else {
		PATH_TRACE;
		/* only test as many as were saved */
		if(t30pa.ngrps > MAXGRP)
			t30pa.ngrps = MAXGRP;	
		cmp_grps(t30pa.ngrps, t30pa.grps, t30ch.grps);
	}

	if(t30ch.tm_to_alarm == 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), time to alarm was cleared", NAME());
		test_fail++;
	}
	else
		PATH_TRACE;

	if(t30ch.tm_to_alarm > t30pa.tm_to_alarm ||
	   t30ch.tm_to_alarm < t30pa.tm_to_alarm - SPEEDFACTOR) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), alarm not set correctly", NAME());
		in_rpt("Expected value %u; Observed value %u",
				t30pa.tm_to_alarm, t30ch.tm_to_alarm);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(strncmp(t30pa.cwdir, t30ch.cwdir, (size_t)_POSIX_PATH_MAX)) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), current directory not inherited", NAME());
		in_rpt("Expected value %s", t30pa.cwdir);
		in_rpt("Observed value %s", t30ch.cwdir);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(strncmp(t30pa.rootdir, t30ch.rootdir, (size_t)_POSIX_PATH_MAX)) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), root directory not inherited", NAME());
		in_rpt("Expected value %s", t30pa.rootdir);
		in_rpt("Observed value %s", t30ch.rootdir);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(t30ch.umsk != t30pa.umsk) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), process creation mask not inherited", NAME());
		in_rpt("Expected value 0%lo; Observed value: 0%lo",
			(unsigned long)t30pa.umsk, (unsigned long)t30ch.umsk);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!t30ch.sigpend) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), pending signals cleared", NAME());
		test_fail++;
	}
	else
		PATH_TRACE;
	cmp_mask(&t30pa.sigmask, &t30ch.sigmask);

	if(t30ch.timb.tms_utime < t30pa.timb.tms_utime) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), tms_utime not set correctly", NAME());
		in_rpt("Expected value >= %ld; Observed value %ld",
			(long)t30pa.timb.tms_utime, (long)t30ch.timb.tms_utime);
		test_fail++;
	}
	else
		PATH_TRACE;
	if(t30ch.timb.tms_stime < t30pa.timb.tms_stime) {

		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), tms_stime not set correctly", NAME());
		in_rpt("Expected value >= %ld; Observed value %ld",
			(long)t30pa.timb.tms_stime, (long)t30ch.timb.tms_stime);
		test_fail++;
	}
	else
		PATH_TRACE;
	if(t30ch.timb.tms_cutime != t30pa.timb.tms_cutime) {

		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), tms_cutime not set correctly", NAME());
		in_rpt("Expected value %ld; Observed value %ld",
			(long)t30pa.timb.tms_cutime, (long)t30ch.timb.tms_cutime);
		test_fail++;
	}
	else
		PATH_TRACE;
	if(t30ch.timb.tms_cstime != t30pa.timb.tms_cstime) {

		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("Following %s(...), tms_cstime not set correctly", NAME());
		in_rpt("Expected value %ld; Observed value %ld",
			(long)t30pa.timb.tms_cstime, (long)t30ch.timb.tms_cstime);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(17);

	DBUG_VOID_RETURN;
}

private void
cmp_mask(aset, acmpset)
sigset_t	*aset;		/* parent signal mask */
sigset_t	*acmpset;	/* child signal mask  */
{
	int	i;
	int	signum;
	char	*name;

	DBUG_ENTER("cmp_mask");

	for(i = 1; i < nsigs; i++) {
		signum = sig_array[i];
		name = sig_name[i];
		if(sigismember(acmpset, signum)) {
			if(!sigismember(aset, signum)) {
				if(!test_fail)
					xx_rpt(FAILURE);
				in_rpt("Following %s(...), signal mask includes extra signal %d %s",
							NAME(), signum, name);
				test_fail++;
			}
		} else {
			if(sigismember(aset, signum)) {
				if(!test_fail)
					xx_rpt(FAILURE);
				in_rpt("Following %s(...), signal mask omits signal %d %s",
							NAME(), signum, name);
				test_fail++;
			}
		}
	}

	DBUG_VOID_RETURN;
}

private void
cmp_grps(num, pagrps, chgrps)
int	num;
gid_t	*pagrps;
gid_t	*chgrps;
{
	gid_t	*p_grptr;
	gid_t	*c_grptr;
	int	i;
	int	j;
	int	gotgroup;

	/* Check each parent group is in the child's group list. */
	p_grptr = pagrps;
	for(i = 0; i < num; i++) {
		DBUG_PRINTF("cmp_grps", "parent group %d", *p_grptr);
		gotgroup = 0;
		c_grptr = chgrps;
		for(j = 0; j < num; j++) {
			if(*c_grptr++ == *p_grptr)
				gotgroup++;
		}
		if(!gotgroup) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("Following %s(...), process does not have supplementary group %ld",
				NAME(), (long)*p_grptr);
			test_fail++;
		}
		p_grptr++;
	}

	/* Check each child group is in the parent's group list. */
	c_grptr = chgrps;
	for(i = 0; i < num; i++) {
		DBUG_PRINTF("cmp_grps", "child group %d", *c_grptr);
		gotgroup = 0;
		p_grptr = pagrps;
		for(j = 0; j < num; j++) {
			if(*p_grptr++ == *c_grptr)
				gotgroup++;
		}
		if(!gotgroup) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("Following %s(...), process has extra supplementary group %ld",
				NAME(), (long)*c_grptr);
			test_fail++;
		}
		c_grptr++;
	}
}

private void
before_exec()
{
	int	i;
	int	err;
	int	pa_ngrps;
	gid_t	*pa_grps;
	int	max_groups;
	sigset_t set;
	int	ret;
	pid_t	myself;
	int	pathok = 0;

	DBUG_ENTER("before_exec");

	t30pa.pid  = getpid();
	t30pa.ppid = getppid();
	t30pa.pgrp = getpgrp();
	t30pa.uid  = getuid();
	t30pa.gid  = getgid();

	/* supplementary groups */
	if((max_groups = sysconf(_SC_NGROUPS_MAX)) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sysconf(_SC_NGROUPS_MAX) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pa_grps = (gid_t *)calloc((size_t)max_groups, sizeof(gid_t));
	if (pa_grps == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason - calloc() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((pa_ngrps = getgroups(max_groups, pa_grps)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - getgroups() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t30pa.ngrps = pa_ngrps;
	for(i = 0; i < max_groups && i < MAXGRP; i++)
		t30pa.grps[i] = pa_grps[i];

	/* current working directory */
	if(getcwd(t30pa.cwdir, sizeof(t30pa.cwdir)) == (char *)NULL) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - getcwd() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chdir(SLASH) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - chdir() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* root directory */
	if(getcwd(t30pa.rootdir, sizeof(t30pa.rootdir)) == (char *)NULL) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - getcwd() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chdir(t30pa.cwdir) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - chdir() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* file mode creation mask */
	(void) umask((mode_t)UMASK);
	if((t30pa.umsk = umask((mode_t)UMASK)) != (mode_t)UMASK) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - umask() did not return old mask");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* pending signals */
	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Call sigprocmask() to block signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGHUP);
	(void) sigaddset(&set, SIGTTIN);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the signal to this process */
	sig_recvd = 0;
	myself = getpid();
	ret = kill(myself, SIGHUP);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(sig_recvd) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() did not block signal");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Get the process signal mask */
	(void) sigemptyset(&t30pa.sigmask);
	if(sigprocmask(SIG_SETMASK, (sigset_t *)NULL, &t30pa.sigmask) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* DO THIS NEXT TO LAST - setting of process times */
	if(times(&t30pa.timb) == (clock_t)SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - times() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* DO THIS LAST - setting of alarm time */
	t30pa.tm_to_alarm = (unsigned) (10 * WAITTIME);	
	(void) alarm(t30pa.tm_to_alarm);

	PATH_FUNC_RPT(14);

	DBUG_VOID_RETURN;
}

private void
ch30p1()
{
	char	*teststr;
	char	*pipestr;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch30p1");

	/* Initialise the environment and program arguments for the exec() */
	teststr  = newstr(itoa(tet_thistest));
	pipestr  = newstr(itoa(res_pipe[WRITE]));
	if(teststr == (char *)NULL ||
	   pipestr == (char *)NULL) {
		vsxerror("external error - Out of memory");
		DBUG_EXIT(A_NOMEM);
	}
	else
		PATH_TRACE;

	(void) init_exec(t_prog, teststr, pipestr, "", "", vg_ident, vg_dbug);

	/* Obtain results before exec() and write to the parent */
	globok = 0;
	before_exec();
	PATH_FUNC_TRACE;

	t30pa.pathok = (pathok == 2 ? 1 : 0);

	errno = NOERROR;
	ret = write(res_pipe[WRITE], (char *)&t30pa, sizeof(t30pa));
	err = errno;
	if(ret != sizeof(t30pa)) {
		if(ret == SYSERROR) {
			vsxerror("write failed, errno %d", err);
			test_deleted++;
		} else {
			vsxerror("write returned wrong value");
			in_rpt("Expected value %d; Observed value %d", 
							sizeof(t30pa), ret);
			test_deleted++;
		}
	}
	if(test_deleted)
		DBUG_EXIT(A_WRITEFAIL);

	/* Now at last the exec() call */
#if TEST_XNFS && !defined(DTET1)
	tet_logoff();
#endif
	(void) doexec();
	err = errno;
	vsxerror("%s(%s, ...) failed - errno %d (%s)",
		NAME(), t_prog, err, errname(err));
	DBUG_EXIT(A_EXECFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa30p1()
{
	int	err;
	int	ret;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa30p1");

	/* Read results from after fork() but before exec() */
	SET_TIMEOUT(WAITTIME/2)
	errno = NOERROR;
	ret = read(res_pipe[READ], (char *)&t30pa, sizeof(t30pa));
	err = errno;
	CLEAR_ALARM
	if(ret == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - read from pipe failed, errno %d",
									err);
		test_deleted++;
	} else if(ret != sizeof(t30pa)) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - read from pipe returned wrong value");
		in_rpt("Expected value %d; Observed value %d", sizeof(t30pa),
								ret);
		test_deleted++;
	}
	else
		PATH_TRACE;

	if (t30pa.pathok == 1)
		PATH_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Read results from after exec() */
	SET_TIMEOUT(2*WAITTIME)
	errno = NOERROR;
	ret = read(res_pipe[READ], (char *)&t30ch, sizeof(t30ch));
	err = errno;
	CLEAR_ALARM
	if(ret == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - read from pipe failed, errno %d",
									err);
		test_deleted++;
	} else if(ret != sizeof(t30ch)) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - read from pipe returned wrong value");
		in_rpt("Expected value %d; Observed value %d", sizeof(t30ch),
								ret);
		test_deleted++;
	}
	else
		PATH_TRACE; 

	if (t30ch.pathok == 1)
		PATH_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitpid() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code != OK)
		test_deleted++;
	else
		PATH_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(stat_child != CHLD_EXIT) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(9);

	DBUG_VOID_RETURN;
}


private void
ch30p2()
{
	char	*suid;
	int	err;

	DBUG_ENTER("ch30p2");

	/* Initialise the environment and program arguments for the exec() */
	if ((suid = newstr(itoa(vguid2))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	(void) init_exec(t_prog, suid, "", "", "", vg_ident, vg_dbug);

#if TEST_XNFS && !defined(DTET1)
	tet_logoff();
#endif
	(void) doexec();
	err = errno;
	vsxerror("%s(%s, ...) failed - errno %d (%s)",
		NAME(), t_prog, err, errname(err));
	DBUG_EXIT(A_EXECFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa30p2()
{
	int	stat_child;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa30p2");

	/* Wait for child and expect it to be stopped */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if(stat_child != CHLD_STOP) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - child process not stopped");
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to send the SIGKILL signal - expect error */
	errno = NOERROR;
	ret = kill(tet_child, SIGKILL);
	err = errno;

	if(ret != SYSERROR || err != EPERM) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill(pid, SIGKILL) returned un-expected value");
		valrpt(SYSERROR, ret, EPERM, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to send the SIGCONT signal - expect success */
	errno = NOERROR;
	ret = kill(tet_child, SIGCONT);
	err = errno;

	if(ret == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("kill(pid, SIGCONT) failed");
		valrpt(ZERORET, ret, NOERROR, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if(stat_child != CHLD_EXIT || ex_code != SIGCONT) {
		xx_rpt(FAILURE);
		in_rpt("kill(pid, SIGCONT) did not cause delivery of SIGCONT signal");
		in_rpt("expected action: process exitted - exit code %d", SIGCONT);
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(6);

	DBUG_VOID_RETURN;
}

public void
test31()
{
	char	*str;
	int	pathok = setupok;

	DBUG_ENTER("test31");

	test_fail = 0;
	test_deleted = 0;

	t_prog = t31_prog;
	t_path = EXEC_PATH;

	globok = 0;
	savtimes(t31_prog);
	PATH_FUNC_TRACE;

	globok = 0;
	if(cpsig(ch_exec, pa31, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if((str = chktimes(t31_prog, NEWATIME)) != (char *)NULL) {
		xx_rpt(FAILURE);
		in_rpt("After %s(%s, ...) completed successfully", 
						NAME(), t31_prog);
		in_rpt("%s", str);
	}
	else {
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	DBUG_VOID_RETURN;
}

private void
pa31()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa31");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitpid() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

public void
test32()
{
	DBUG_ENTER("test32");

	t_prog = su_prog;
	if(pipe_open())
		(void) cpsig(pipe_ch_suproc, pa_suproc, -1, 0);
	pipe_close();
	
	DBUG_VOID_RETURN;
}

public void
test33()
{
	/* if opendir() opens an fd, it is closed across exec */

	DBUG_ENTER("test33");

	t_prog = tgen_prog;

	if(pipe_open())
	{
		(void) cppair(cht33, NULLFN, WAITTIME, 0);
		pipe_parent();
	}
	pipe_close();
	
	DBUG_VOID_RETURN;
}

private void
cht33()
{
	char	rptbuf[256];
	DIR	*dirp;
	int	open_max, fd, *fdtab;
	char	*str0, *str1;
	int	err;

	DBUG_ENTER("cht33");
	
	pipe_child();

	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if (open_max <= 0)
	{
		if (errno == 0)
			open_max = 256;
		else
		{
			pipe_report(DELETION,"sysconf(_SC_OPEN_MAX) failed");
			DBUG_VOID_RETURN;
		}
	}

	fdtab = (int *)malloc((size_t)(open_max * sizeof(int)));
	if (fdtab == NULL)
	{
		(void)sprintf(rptbuf,"malloc(%ld) failed", (long)(open_max * sizeof(int)));
		pipe_report(DELETION,rptbuf);
		DBUG_VOID_RETURN;
	}

	for (fd = 0; fd < open_max; fd++)
		fdtab[fd] = fcntl(fd, F_GETFD);

	if ((dirp = opendir(".")) == NULL)
	{
		err = errno;
		(void)sprintf(rptbuf,"opendir(\".\") failed - errno %d (%s)",
			err, errname(err));
		pipe_report(DELETION,rptbuf);
		DBUG_VOID_RETURN;
	}

#ifdef TESTBED2
	{
		int flag = fcntl(dirp->dd_fd, F_GETFD);
		(void) fcntl(dirp->dd_fd, F_SETFD, flag ^ FD_CLOEXEC);
	}
#endif

	for (fd = 0; fd < open_max; fd++)
	{
		if (fdtab[fd] == SYSERROR && fcntl(fd, F_GETFD) != SYSERROR)
			break;
	}

	if (fd >= open_max)
	{
		pipe_report(UNTESTED,"opendir() did not open a file descriptor");
		DBUG_VOID_RETURN;
	}
	
	DBUG_PRINTF("cht33", "fd = %d", fd);

	if((str0 = newstr(itoa(tet_thistest))) == (char *)NULL ||
	   (str1 = newstr(itoa(fd))) == (char *)NULL)
        {
		err = errno;
		(void)sprintf(rptbuf,"malloc() failed - errno %d (%s)", err, errname(err));
		pipe_report(DELETION,rptbuf);
		DBUG_VOID_RETURN;
	}
	(void) init_exec(t_prog, str0, str1, "", "", vg_ident, vg_dbug);
#if TEST_XNFS && !defined(DTET1)
	tet_logoff();
#endif
	(void) doexec();

	err = errno;
	(void)sprintf(rptbuf,"%s(%s, ...) failed - errno %d (%s)",
		NAME(), t_prog, err, errname(err));
	pipe_report(DELETION,rptbuf);

	(void) closedir(dirp);

	DBUG_VOID_RETURN;
}

public void
test34()
{
	DBUG_ENTER("test34");

	if (arg_max < 0 && !(arg_max == -1 && arg_err == 0))
	{
		xx_rpt(DELETION);
		in_rpt("sysconf(_SC_ARG_MAX) failed in prep");
		DBUG_VOID_RETURN;
	}

	if(arg_max > PCTS_ARG_MAX || arg_max == -1) {
		xx_rpt(UNSUPPORTED);
		in_rpt("ARG_MAX > PCTS_ARG_MAX or == -1");
		DBUG_VOID_RETURN;
	}

	test_fail = 0;
	test_deleted = 0;

	t_prog = tfail_prog;
	t_path = EXEC_PATH;

	(void) cpsig(ch_toobig, pa_toobig, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

public void
test35()
{
#if TEST_XNFS
	int	rval, fd, err;
#endif
	int	pathok = setupok;

	DBUG_ENTER("test35");

	test_deleted = 0;
	test_fail = 0;

#if TEST_XNFS

	/* User ID mapped */

	if (tet_sync(SP_T35_MKDIR_END, snames, SYNC_TIMEOUT) < 0)
	{
		test_deleted++;
		xx_rpt(DELETION);
		in_rpt("sync to MKDIR_END failed");
	}
	else if ((fd = creat(t35_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		test_deleted++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			t35_file, MODEGEN, err, errname(err));
	}
	else if((void)close(fd), file_copy(t35_prog, t35_file) == SYSERROR)
	{
		test_deleted++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: file_copy() failed");
	}
	else if (tet_sync(SP_T35_SETID_END, snames, SYNC_TIMEOUT) < 0)
	{
		test_deleted++;
		xx_rpt(DELETION);
		in_rpt("sync to SETID_END failed");
	}
	else
	{
		rval = tet_fork(xnfs_prog,NULLFN,WAITTIME * 2,XNFS_OK|XNFS_NOK|XNFS_PATHOK|XNFS_EXEC);

		switch(rval) {
		case XNFS_PATHOK:
			PATH_TRACE;
			break;
		case XNFS_EXEC:
			if(!test_fail++)
				xx_rpt(FAILURE);
			in_rpt("%s(%s, ...) when directory %s has no search permission, executed successfully", NAME(), t35_file, t35_dir);
			break;
		case XNFS_OK:
		case XNFS_NOK:
			test_fail++;
			break;
		}
	}
	(void) tet_sync(SP_T35_UNLINK_START, snames, SYNC_TIMEOUT);

#else /* TEST_XNFS */

	PATH_TRACE;

#endif /* TEST_XNFS */

	PATH_FUNC_RPT(1);

	/* User ID not mapped */

#if TEST_XNFS
	xnfs_doroot_c(t35_pt2, WAITTIME*3, SP_NOSYNC, SP_NOSYNC);
#else
	t35_pt2();
#endif

	DBUG_VOID_RETURN;
}

private void
t35_pt2()
{
	int	rval;
	int	pathok = 0;

	DBUG_ENTER("t35_pt2");

	PATH_FUNC_TRACE;

	rval = tet_fork(cht35,NULLFN,WAITTIME * 2,1|2|4);
	switch(rval) {
	case 2:
		PATH_TRACE;
		break;
	case 4:
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) when directory %s has no search permission, executed successfully", NAME(), t35_file, t35_dir);
		break;
	case 0:
	case 1:
		test_fail++;
		break;
	}

	if(!test_fail && !test_deleted)
		PATH_XS_RPT(2);

	/* Set t35_dir to give sufficient permission to remove it */
	tsetid_mode = MODEGEN;
	tsetid_file = t35_dir;
	(void) tet_fork(exec_setid, NULLFN, WAITTIME, 0);
	(void) unlink(t35_file);
	(void) unlink(tclose_file);
	(void) vxrmdir(t35_dir);
	DBUG_VOID_RETURN;
}

private void
cht35()
{
	int		fd;
	int		rval;
	int		err;
	struct stat	statb;
	int	pathok = 0;

	DBUG_ENTER("cht35");

	/* Set t35_dir to give sufficient permission to remove it */
	if (stat(t35_dir, &statb) == 0)
	{
		tsetid_mode = MODEGEN;
		tsetid_file = t35_dir;
		if(tet_fork(exec_setid, NULLFN, WAITTIME, 0) != 0)
			DBUG_EXIT(0);
		(void) unlink(t35_file);
		(void) vxrmdir(t35_dir);
	}

	if (stat(t35_dir, &statb) != SYSERROR || errno != ENOENT)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not remove directory %s", t35_dir);
		DBUG_EXIT(0);
	}
	else
		PATH_TRACE;
	if (crdir(t35_dir, (mode_t)MODEGEN, (int) getuid(), (int) getgid()) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not creat directory %s - errno %d (%s)", t35_dir, err, errname(err));
		DBUG_EXIT(0);
	}
	else
		PATH_TRACE;
	if ((fd = creat(t35_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t35_file, (long)MODEGEN, err, errname(err));
		DBUG_EXIT(0);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if(file_copy(t35_prog, t35_file) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: file_copy() failed");
		DBUG_EXIT(0);
	}
	else
		PATH_TRACE;

	/* Need to set user and group ID bits to other than vsx0/vsxg0 */
	tsetid_mode = MODESETID;
	tsetid_file = t35_file;
	if(tet_fork(exec_setid, NULLFN, WAITTIME, 0) != 0)
		DBUG_EXIT(0);

	/* Then set t35_dir to deny search permission */
	tsetid_mode = T35_MODE;
	tsetid_file = t35_dir;
	if(tet_fork(exec_setid, NULLFN, WAITTIME, 0) != 0)
		DBUG_EXIT(0);

	t_prog = t35_file;
	t_path = EXEC_PATH;
	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2, P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
	errno = NOERROR;
	do_save();
	if(test_deleted)
		DBUG_EXIT(0);
	rval = doexec();
	err = errno;
	do_cmp();
	if (rval != SYSERROR || err != EACCES)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) when directory %s has no search permission, did not return expected values", NAME(), t35_file, t35_dir);
		valrpt(SYSERROR, rval, EACCES, err);
		DBUG_EXIT(0);
	}
	else if (test_deleted || test_fail)
		DBUG_EXIT(0);
	else if(pathok == 4)
		DBUG_EXIT(2);
	else
		DBUG_EXIT(1);
}

private void
vxrmdir(file)
char	*file;
{
	(void) rmdir(file);
}

public void
test36()
{
#if TEST_XNFS
	int	rval, fd, err;
#endif
	int	pathok = setupok;

	DBUG_ENTER("test36");

	test_fail = 0;
	test_deleted = 0;

#if TEST_XNFS

	/* User ID mapped */

	if ((fd = creat(t36_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		test_deleted++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#o) failed - errno %d (%s)",
			t36_file, MODEGEN, err, errname(err));
	}
	else if(file_copy(t36_prog, t36_file) == SYSERROR) {
		test_deleted++;
		xx_rpt(DELETION);
		in_rpt("deletion reason: file_copy() failed");
	}
	else if (tet_sync(SP_T36_SETID_END, snames, SYNC_TIMEOUT) < 0)
	{
		test_deleted++;
		xx_rpt(DELETION);
		in_rpt("sync to SETID_END failed");
	}
	else
	{
		rval = tet_fork(xnfs_prog,NULLFN,WAITTIME * 2,XNFS_OK|XNFS_NOK|XNFS_PATHOK|XNFS_EXEC);

		switch(rval) {
		case XNFS_PATHOK:
			PATH_TRACE;
			break;
		case XNFS_EXEC:
			if(!test_fail++)
				xx_rpt(FAILURE);
			in_rpt("%s(%s, ...) when there is no read or execute permission on %s, executed successfully", NAME(), t36_file, t36_file);
			break;
		case XNFS_OK:
		case XNFS_NOK:
			test_fail++;
			break;
		}
	}
	(void) close(fd);
	(void) unlink(t36_file);

#else /* TEST_XNFS */

	PATH_TRACE;

#endif /* TEST_XNFS */

	PATH_FUNC_RPT(1);

	/* User ID not mapped */

#if TEST_XNFS
	xnfs_doroot_c(t36_pt2, WAITTIME*3, SP_NOSYNC, SP_NOSYNC);
#else
	t36_pt2();
#endif

	DBUG_VOID_RETURN;
}

private void
t36_pt2()
{
	int	rval;
	int	pathok = 0;

	DBUG_ENTER("t36_pt2");

	PATH_FUNC_TRACE;

	rval = tet_fork(cht36,NULLFN,WAITTIME * 2,1|2|4);
	switch(rval) {
	case 2:
		PATH_TRACE;
		break;
	case 4:
		if(!test_fail++)
			xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) when there is no execute permission on %s, executed successfully", NAME(), t36_file, t36_file);
		break;
	case 0:
	case 1:
		test_fail++;
		break;
	}

	if(!test_fail && !test_deleted)
		PATH_XS_RPT(2);

	(void)unlink(t36_file);
	DBUG_VOID_RETURN;
}

private void
cht36()
{
	int	fd;
	int	rval;
	int	err;
	int	pathok = 0;


	DBUG_ENTER("cht36");

	
	if (unlink(t36_file) == SYSERROR && errno != ENOENT)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: cannot remove %s", t36_file);
		DBUG_EXIT(0);
	}
	else
		PATH_TRACE;
	if ((fd = creat(t36_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t36_file, (long)MODEGEN, err, errname(err));
		DBUG_EXIT(0);
	}
	else
		PATH_TRACE;
	(void) close(fd);

	if(file_copy(t36_prog, t36_file) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: file_copy() failed");
		DBUG_EXIT(0);
	}
	else
		PATH_TRACE;

	/* Need to set user and group ID bits to other than vsx0/vsxg0 */
	tsetid_mode = T36_MODE;
	tsetid_file = t36_file;
	if(tet_fork(exec_setid, NULLFN, WAITTIME, 0) != 0)
		DBUG_EXIT(0);

	t_prog = t36_file;
	t_path = EXEC_PATH;
	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2, P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
	errno = NOERROR;
	do_save();
	if(test_deleted)
		DBUG_EXIT(0);
	rval = doexec();
	err = errno;
	do_cmp();
	if (rval != SYSERROR || err != EACCES)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) when there is no execute permission on %s, did not return expected values", NAME(), t36_file, t36_file);
		valrpt(SYSERROR, rval, EACCES, err);
	}
	else if (test_deleted || test_fail)
		DBUG_EXIT(0);
	else if(pathok == 3)
		DBUG_EXIT(2);
	else
		DBUG_EXIT(1);
}

public void
test37()
{
	char	**longname;
	int	fail = 0;
	int	pathok = setupok;

	DBUG_ENTER("test37");

	test_deleted = 0;
	globok = 0;
	longname = pathtoolng(t37_prog);
	if(longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	t_prog = longname[0];
	if (cppair(ch1t37, NULLFN, WAITTIME, E_EXEC) == E_EXEC)
		fail++;
	else
		PATH_TRACE;

	if ((t_prog = longname[1]) != NULL)
	{
		switch (cppair(ch2t37, NULLFN, WAITTIME, E_EXEC|NOK))
		{
		case E_EXEC:
			PATH_TRACE;
			break;
		case NOK:
			fail++;
			break;
		default:
			fail++;
			xx_rpt(FAILURE);
			in_rpt("%s(%s, ...) when %s prefixed by %d char path",
				NAME(), t37_prog, t37_prog,
				(int)(strlen(t_prog)-strlen(t37_prog)));
			in_rpt("did not return expected values");
			valrpt(SYSERROR, SUCCEED, ENAMETOOLONG, NOERROR);
		}
	}
	else
		PATH_TRACE;

	if (fail == 0 )
		PATH_XS_RPT(3);

	clntoolng();

	DBUG_VOID_RETURN;
}

private void
ch1t37()
{
	int	rval1, err1;

	DBUG_ENTER("ch1t37");

	t_path = EXEC_PATH;
	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2,
		P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
	errno = NOERROR;
	rval1 = doexec();

	err1 = errno;
	xx_rpt(FAILURE);
	in_rpt("%s(%s, ...) when %s prefixed by %d char path", NAME(),
		t37_prog, t37_prog, (int)(strlen(t_prog)-strlen(t37_prog)));
	in_rpt("did not return expected values");
	valrpt(SUCCEED, rval1, NOERROR, err1);
	DBUG_EXIT(E_EXEC);
}

private void
ch2t37()
{
	int	rval1, err1;

	DBUG_ENTER("ch2t37");

	t_path = EXEC_PATH;
	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2,
		P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
	errno = NOERROR;
	test_deleted = 0;
	test_fail = 0;
	do_save();
	if(test_deleted)
		DBUG_EXIT(NOK);
	rval1 = doexec();

	if ((err1 = errno) != ENAMETOOLONG || rval1 != SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) when %s prefixed by %d char path", NAME(),
			t37_prog, t37_prog,
			(int)(strlen(t_prog)-strlen(t37_prog)));
		in_rpt("did not return expected values");
		valrpt(SYSERROR, rval1, ENAMETOOLONG, err1);
		DBUG_EXIT(NOK);
	}
	do_cmp();
	if ( test_deleted || test_fail )
		DBUG_EXIT(NOK);

	DBUG_EXIT(E_EXEC);
}

#if TEST_XNFS

public void
test38()
{
	DBUG_ENTER("test38");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */
	
public void
test38()
{
	char	**longname;
	int	fail = 0;
	int	pathok = setupok;

	DBUG_ENTER("test38");

	test_deleted = 0;
	globok = 0;
	longname = nametoolng();
	if(longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(file_copy(t38_prog, longname[0]) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: file_copy() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t_prog = longname[0];
	globok = 0;
	if (cppair(ch1t38, NULLFN, WAITTIME, E_EXEC) == E_EXEC)
		fail++;
	else
		PATH_TRACE;

	if ((t_prog = longname[1]) != NULL)
	{
		globok = 0;
		switch (cppair(ch2t38, NULLFN, WAITTIME, E_EXEC|NOK))
		{
		case E_EXEC:
			PATH_TRACE;
			break;
		case NOK:
			fail++;
			break;
		default:
			fail++;
			xx_rpt(FAILURE);
			in_rpt("%s(, ...) with %d char filename", NAME(),
				(int)strlen(t_prog));
			in_rpt("did not return expected values");
			valrpt(SYSERROR, SUCCEED, ENAMETOOLONG, NOERROR);
		}
	}
	else
		PATH_TRACE;

	if (fail == 0 && test_deleted == 0)
		PATH_XS_RPT(4);

	(void) unlink(longname[0]);
	clntoolng();

	DBUG_VOID_RETURN;
}

private void
ch1t38()
{
	int	rval1, err1;

	DBUG_ENTER("ch1t38");

	t_path = EXEC_PATH;
	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2,
		P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
	errno = NOERROR;
	rval1 = doexec();

	err1 = errno;
	xx_rpt(FAILURE);
	in_rpt("%s(, ...) with %d char filename", NAME(), (int)strlen(t_prog));
	in_rpt("did not return expected values");
	valrpt(SUCCEED, rval1, NOERROR, err1);
	DBUG_EXIT(E_EXEC);
}

private void
ch2t38()
{
	int	rval1, err1;

	DBUG_ENTER("ch2t38");

	test_fail = 0;
	t_path = EXEC_PATH;
	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2,
		P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
	errno = NOERROR;
	do_save();
	if(test_deleted)
		DBUG_EXIT(NOK);
	rval1 = doexec();
	err1 = errno;
	do_cmp();

	if (err1 != ENAMETOOLONG || rval1 != SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(, ...) with %d char filename", NAME(),
			(int)strlen(t_prog));
		in_rpt("did not return expected values");
		valrpt(SYSERROR, rval1, ENAMETOOLONG, err1);
		DBUG_EXIT(NOK);
	}

	if ( test_fail )
		DBUG_EXIT(NOK);

	DBUG_EXIT(E_EXEC);
}

#endif /* TEST_XNFS */

public void
test39()
{
	DBUG_ENTER("test39");
	(void)cppair(cht39,NULLFN,WAITTIME,0);
	DBUG_VOID_RETURN;
}

private void
cht39()
{
	int	rval1;
	int	rval2;
	int	err1 = 0;
	int	err2;
	int	fail = 0;
	int	pathok = setupok;

	DBUG_ENTER("cht39");
	test_deleted = 0;
	test_fail = 0;
	/* Note: t39_dir is never created as a directory */
	if (unlink(t39_1_file) == SYSERROR && errno != ENOENT ||
		unlink(t39_dir) == SYSERROR && errno != ENOENT && (err1 = 1))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: cannot remove %s", err1 ? t39_dir : t39_1_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t_prog = t39_1_file;
	t_path = EXEC_PATH;
	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2, P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
	errno = NOERROR;
	do_save();
	if(test_deleted)
		DBUG_VOID_RETURN;
	rval1 = doexec();
	err1 = errno;
	do_cmp();
	if (err1  != ENOENT || rval1 != SYSERROR)
		fail |= 01;
	else
		PATH_TRACE;

	t_prog = t39_2_file;
	t_path = EXEC_PATH;
	errno = NOERROR;
	do_save();
	if(test_deleted)
		DBUG_VOID_RETURN;
	rval2 = doexec();
	err2  = errno;
	do_cmp();
	if (err2  != ENOENT || rval2 != SYSERROR)
		fail |= 02;
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);
		if (fail & 01)
		{
			in_rpt("%s(%s, ...) when %s did not exist did not return expected values", NAME(), t39_1_file, t39_1_file);
			valrpt(SYSERROR, rval1, ENOENT, err1);
		}
		if (fail & 02)
		{
			in_rpt("%s(%s, ...) when %s did not exist did not return expected values", NAME(), t39_2_file, t39_dir);
			valrpt(SYSERROR, rval2, ENOENT, err2);
		}
	}
	else if ( !test_deleted && !test_fail )
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}

	DBUG_VOID_RETURN;
}

public void
test40()
{
	DBUG_ENTER("test40");
	(void)cppair(cht40,NULLFN,WAITTIME,0);
	DBUG_VOID_RETURN;
}

private void
cht40()
{
	int	rval1;
	int	err1 = 0;
	int	fail = 0;
	int	pathok = setupok;

	DBUG_ENTER("cht40");

	t_prog = t40_file;
	t_path = EXEC_PATH;
	test_deleted = 0;
	test_fail = 0;
	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2, P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
	errno = NOERROR;
	do_save();
	if(test_deleted)
		DBUG_VOID_RETURN;
	rval1 = doexec();
	err1 = errno;
	do_cmp();
	if (err1  != ENOENT || rval1 != SYSERROR)
		fail |= 01;
	else
		PATH_TRACE;

	if (fail)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(\"%s\", ...) did not return expected values", NAME(), t40_file);
		valrpt(SYSERROR, rval1, ENOENT, err1);
	}
	else if ( !test_deleted && !test_fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	DBUG_VOID_RETURN;
}

public void
test41()
{
	DBUG_ENTER("test41");

#if TEST_XNFS
	xnfs_doroot_c(cht41, WAITTIME, SP_NOSYNC, SP_NOSYNC);
#else
	(void) cppair(cht41, NULLFN, WAITTIME, 0);
#endif

	DBUG_VOID_RETURN;
}

private void
cht41()
{
	int	fd;
	int	rval;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("cht41");
	test_deleted = 0;
	test_fail = 0;
	if ((fd = creat(t41_1_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t41_1_file, (long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) close(fd);

	/* Need to set user and group ID bits to other than vsx0/vsxg0 */
	tsetid_mode = MODESETID;
	tsetid_file = t41_1_file;
	if(tet_fork(exec_setid, NULLFN, WAITTIME, 0) != 0)
	{
		(void)unlink(t41_1_file);
		DBUG_VOID_RETURN;
	}

	t_prog = t41_2_file;
	t_path = EXEC_PATH;
	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2, P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
	do_save();
	if(test_deleted)
	{
		(void)unlink(t41_1_file);
		DBUG_VOID_RETURN;
	}
	errno = NOERROR;
	rval = doexec();
	err = errno;
	do_cmp();
	if (rval != SYSERROR || err != ENOTDIR)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) when %s is a file, did not return expected values", NAME(), t41_2_file, t41_1_file);
		valrpt(SYSERROR, rval, ENOTDIR, err);
	}
	else if ( !test_deleted && !test_fail )
	{
		PATH_TRACE;
		PATH_XS_RPT(2);
	}

	(void)unlink(t41_1_file);

	DBUG_VOID_RETURN;
}

public void
test42()
{
	DBUG_ENTER("test42");

#if TEST_XNFS
	xnfs_doroot_c(cht42, WAITTIME, SP_NOSYNC, SP_NOSYNC);
#else
	(void) cppair(cht42, NULLFN, WAITTIME, 0);
#endif

	DBUG_VOID_RETURN;
}

private void
cht42()
{
	int	fd;
	int	rval;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("cht42");

	if(iwhich != I_EXECLE && iwhich != I_EXECVE &&
	   iwhich != I_EXECL  && iwhich != I_EXECV ) {
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if (unlink(t42_file) == SYSERROR && errno != ENOENT)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: cannot remove %s", t42_file);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ((fd = creat(t42_file, MODEGEN)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: creat(%s, %#lo) failed - errno %d (%s)",
			t42_file, (long)MODEGEN, err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	test_deleted = 0;
	test_fail = 0;
	(void) close(fd);

	tsetid_mode = MODESETID;
	tsetid_file = t42_file;
	/* Need to set user and group ID bits to other than vsx0/vsxg0 */
	if(tet_fork(exec_setid, NULLFN, WAITTIME, 0) != 0)
	{
		(void)unlink(t42_file);
		DBUG_VOID_RETURN;
	}

	t_prog = t42_file;
	t_path = EXEC_PATH;
	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2, P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
	do_save();
	if(test_deleted)
	{
		(void)unlink(t42_file);
		DBUG_VOID_RETURN;
	}
	errno = NOERROR;
	rval = doexec();
	err = errno;
	do_cmp();
	if (rval != SYSERROR || err != ENOEXEC)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) when file is empty %s, did not return expected values", NAME(), t42_file, t42_file);
		valrpt(SYSERROR, rval, ENOEXEC, err);
	}
	else if (!test_deleted && !test_fail)
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}

	(void)unlink(t42_file);

	DBUG_VOID_RETURN;
}

/*
 * Test that when an attempt to exec a file of a non-executable type
 * is made it fails with errno EACCES
 */
public void
test43()
{

	DBUG_ENTER("test43");

	test_fail = 0;

	t_prog = tet_getvar(VSX_NONEXEC_FILE);
	if (t_prog == NULL || t_prog[0] == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter %s is not set", VSX_NONEXEC_FILE);
		DBUG_VOID_RETURN;
	}
	else if (strncmp(t_prog, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter %s is set to \"unsup\"", VSX_NONEXEC_FILE);
		DBUG_VOID_RETURN;
	}

	/*
	 * It is essential to check for execute permission because the
	 * lack of that alone would cause an EACCES error.
	 */
	if (checkxperm(t_prog) == -1)
		DBUG_VOID_RETURN;

	t_path = EXEC_PATH;

	if (pipe(res_pipe) == SYSERROR) {
	int 	err = errno;
		if (test_fail++ == 0)
			xx_rpt(DELETION);
		in_rpt("pipe failed errno %s (%d)", errname(err), err);
		DBUG_VOID_RETURN;
	}

	/*
	 * Mask out all possible errors here as we
	 * need to in general fail rather than delete if the child
	 * has a bad exit val.
	 */
	(void) cppair(ch43, pa43, WAITTIME, ~0);

	DBUG_VOID_RETURN;

}

private void
ch43()
{
	int 	rval;
	int 	err;
	char 	dummy;
	int	pathok = setupok;

	DBUG_ENTER("ch43");

	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2, P1_ARG3, P1_ARG4, vg_ident, vg_dbug);

	errno = NOERROR;

	(void) close(res_pipe[0]);
	/*
	 * Set the close-on-exec flags on stdout and the pipe so that if
	 * the exec does happen it cannot scribble on the report file
	 * or back up to the parent. Not interested in stderr either
	 */
	(void) fcntl(1, F_SETFD, FD_CLOEXEC|fcntl(1, F_GETFD));
	(void) fcntl(2, F_SETFD, FD_CLOEXEC|fcntl(2, F_GETFD));
	(void) fcntl(res_pipe[1], F_SETFD,
		FD_CLOEXEC|fcntl(res_pipe[1], F_GETFD));

	test_deleted = 0;
	test_fail = 0;
	do_save();
	if(test_deleted) {
		(void) write(res_pipe[1], &dummy, sizeof(dummy));
		DBUG_VOID_RETURN;
	}
	errno = NOERROR;
	rval = doexec();
	err = errno;
	do_cmp();

	DBUG_PRINTF("ch43", "returned from exec of %s", t_prog);

	if (rval != SYSERROR || err != EACCES) {
		test_fail++;
		xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) did not return expected value", NAME(), t_prog);
		valrpt(SYSERROR, rval, EACCES, err);

	} else if ( !test_deleted && !test_fail ) {
		PATH_TRACE;
		xs_rpt();
	}

	(void) write(res_pipe[1], &dummy, sizeof(dummy));
	DBUG_VOID_RETURN;
}

private void
pa43()
{
	char 	dummy;
	int	pathok = 0;

	DBUG_ENTER("pa43");

	(void) close(res_pipe[1]);

	/*
	 * This mearly waits for the child to say that it survived the
	 * exec.  Since the close-on-exec flags is on the other end of
	 * the pipe then if the exec does in fact happen then we suffer
	 * from a broken pipe or time out here.
	 */
	if (read(res_pipe[0], &dummy, sizeof(dummy)) != sizeof(dummy)) {
		if (test_fail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("exec unexpectedly succeeded or other problem in child process");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

public void
test44()
{
	char	*longname;
	int	pathok = setupok;

	DBUG_ENTER("test44");

	globok = 0;
	longname = xnfs_nametoolng(1, MK_SPNO(0));
	if(longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	t_prog = longname;
	t_path = EXEC_PATH;

	if(tet_fork(cht44, NULLFN, WAITTIME * 2, OK|NOK) == OK)
		PATH_XS_RPT(1);

	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

private void
cht44()
{
	int	err, rval;

	DBUG_ENTER("cht44");

	rval = doexec();

	err = errno;
	if (err != ENAMETOOLONG || rval != SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(, ...) with %d char filename", NAME(),
			(int)strlen(t_prog));
		in_rpt("did not return expected values");
		valrpt(SYSERROR, rval, ENAMETOOLONG, err);
		DBUG_EXIT(NOK);
	}
	else
		DBUG_EXIT(OK);
}

#else /* TEST_XNFS */

public void
test44()
{
	DBUG_ENTER("test44");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

public void
test45()
{
	char	*longname;
	int	pathok = setupok;

	DBUG_ENTER("test45");

	globok = 0;
	longname = xnfs_nametoolng(2, MK_SPNO(0));
	if(longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	t_prog = longname;
	t_path = EXEC_PATH;

	if(tet_fork(cht45, NULLFN, WAITTIME * 2, OK|NOK) == OK)
		PATH_XS_RPT(1);

	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

private void
cht45()
{
	int	rval, err;

	DBUG_ENTER("cht45");

	rval = doexec();

	err = errno;
	if (err != ENAMETOOLONG || rval != SYSERROR)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(, ...) with %d char filename", NAME(),
			(int)strlen(t_prog));
		in_rpt("did not return expected values");
		valrpt(SYSERROR, rval, ENAMETOOLONG, err);
		DBUG_EXIT(NOK);
	}
	else
		DBUG_EXIT(OK);
}

#else /* TEST_XNFS */

public void
test45()
{
	DBUG_ENTER("test45");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

public void
test46()
{
	char	*longname;
	int	pathok = setupok;

	DBUG_ENTER("test46");

	globok = 0;
	longname = xnfs_nametoolng(3, MK_SPNO(0));
	if(longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	t_prog = longname;
	t_path = EXEC_PATH;

	if(tet_fork(cht46, NULLFN, WAITTIME * 2, OK|NOK) == OK)
		PATH_XS_RPT(1);

	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

private void
cht46()
{
	int	err;

	DBUG_ENTER("cht46");

	(void) doexec();

	err = errno;
	if (err == ENAMETOOLONG)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(, ...) with %d char filename", NAME(),
			(int)strlen(t_prog));
		in_rpt("set errno to ENAMETOOLONG");
		DBUG_EXIT(NOK);
	}
	else
		DBUG_EXIT(OK);
}

#else /* TEST_XNFS */

public void
test46()
{
	DBUG_ENTER("test46");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

public void
test47()
{
	char	*longname;
	int	pathok = setupok;

	DBUG_ENTER("test47");

	globok = 0;
	longname = xnfs_nametoolng(4, MK_SPNO(0));
	if(longname == NULL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	t_prog = longname;
	t_path = EXEC_PATH;

	if(tet_fork(cht47, NULLFN, WAITTIME * 2, OK|NOK) == OK)
		PATH_XS_RPT(1);

	(void) unlink(longname);
	clntoolng();

	DBUG_VOID_RETURN;
}

private void
cht47()
{
	int	err;

	DBUG_ENTER("cht47");

	(void) doexec();

	err = errno;
	if (err == ENAMETOOLONG)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(, ...) with %d char filename", NAME(),
			(int)strlen(t_prog));
		in_rpt("set errno to ENAMETOOLONG");
		DBUG_EXIT(NOK);
	}
	else
		DBUG_EXIT(OK);
}

#else /* TEST_XNFS */

public void
test47()
{
	DBUG_ENTER("test47");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

public void
test48()
{
	char	*setid_supp;

	DBUG_ENTER("test48");

	setid_supp = tet_getvar(VSX_SET_ID_MODES_SUPP);
	if ( setid_supp ) {
		if ( (setid_supp[0] == 'y' || setid_supp[0] == 'Y' ) ) {
			xnfs_doroot_c(testuid, WAITTIME*2, SP_NOSYNC, SP_NOSYNC);
			DBUG_VOID_RETURN;
		}
		if ( (setid_supp[0] == 'n' || setid_supp[0] == 'N' ) ) {
			xx_rpt(UNSUPPORTED);
			DBUG_VOID_RETURN;
		}
		/* else fall thru */
	}

	xx_rpt(DELETION);
	in_rpt("Parameter %s is not set to 'y', 'Y', 'n' or 'N'", 
		VSX_SET_ID_MODES_SUPP);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

public void
test48()
{
	DBUG_ENTER("test48");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

public void
test49()
{
	int	rval;
	char	*setid_supp;

	DBUG_ENTER("test49");

	test_deleted = 0;

	setid_supp = tet_getvar(VSX_SET_ID_MODES_SUPP);
	if ( setid_supp ) {
		if ( (setid_supp[0] == 'y' || setid_supp[0] == 'Y' ) ) {
			if (tet_sync(SP_T49_SETID_END, snames, SYNC_TIMEOUT) < 0)
			{
				xx_rpt(DELETION);
				in_rpt("sync to SETID_END failed");
			}
			else if (pipe_open())
			{
				rval = cppair(cht49, NULLFN, WAITTIME, RPTED);
				if (rval == RPTED)
					pipe_parent();
				else if (rval != -1)
				{
					xx_rpt(DELETION);
					in_rpt("child gave unexpected exit code %d",
						rval);
				}
				pipe_close();
			}

			DBUG_VOID_RETURN;
		}
		if ( (setid_supp[0] == 'n' || setid_supp[0] == 'N' ) ) {
			xx_rpt(UNSUPPORTED);
			DBUG_VOID_RETURN;
		}
		/* else fall thru */
	}

	xx_rpt(DELETION);
	in_rpt("Parameter %s is not set to 'y', 'Y', 'n' or 'N'", 
		VSX_SET_ID_MODES_SUPP);
	DBUG_VOID_RETURN;
}

private void
cht49()
{
	char	*ruid, *euid, *rgid, *egid;
	int	err;
	char	rptbuf[256];

	DBUG_ENTER("cht49");

	pipe_child();

	ruid = newstr(itoa((int)getuid()));
	euid = newstr(itoa((int)geteuid()));
	rgid = newstr(itoa((int)getgid()));
	egid = newstr(itoa((int)getegid()));
	if (ruid == NULL || euid == NULL || rgid == NULL || egid == NULL)
	{
		pipe_report(DELETION, "malloc() failed");
		DBUG_VOID_RETURN;
	}

	t_prog = t49_prog;
	(void) init_exec(t_prog, itoa(tet_thistest), ruid, euid, rgid, egid,
		vg_ident);

#if TEST_XNFS && !defined(DTET1)
	tet_logoff();
#endif
	(void) doexec();
	err = errno;
	(void) sprintf(rptbuf, "%s(%s, ...) failed - errno %d (%s)",
		NAME(), t_prog, err, errname(err));
	pipe_report(DELETION, rptbuf);

	DBUG_EXIT(RPTED);
}

#else /* TEST_XNFS */

public void
test49()
{
	DBUG_ENTER("test49");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */

#if TEST_XNFS

public void
test50()
{
	DBUG_ENTER("test50");
	(void)cppair(cht50,NULLFN,WAITTIME,0);
	DBUG_VOID_RETURN;
}

private void
cht50()
{
	int	pathok = setupok;

	DBUG_ENTER("cht50");

	errno = NOERROR;
	test_fail = 0;
	test_deleted = 0;

	t_prog = t50_prog;
	t_path = EXEC_PATH;

	globok = 0;
	if(cpsig(ch_fildes, pa_exec, 2*WAITTIME, 0) == SYSERROR) 
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (tet_sync(SP_T50_CHECK_DATA_START, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CHECK_DATA_START failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if (tet_sync(SP_T50_CHECK_DATA_END, snames, SYNC_TIMEOUT) < 0)
	{
		xx_rpt(DELETION);
		in_rpt("sync to CHECK_DATA_END failed");
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	(void) unlink(topen_file);
	(void) unlink(tclose_file);

	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

public void
test50()
{
	DBUG_ENTER("test50");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */


private int
checkxperm(file)
char	*file;
{
struct	stat	sbuf;

	DBUG_ENTER("checkxperm");

	if (stat(file, &sbuf) == SYSERROR) {
		if (test_fail++ == 0)
			xx_rpt(DELETION);
		in_rpt("cannot stat %s", file);
	}
#ifndef TEST_BED
	if (S_ISREG(sbuf.st_mode)) {
		if (test_fail++ == 0)
			xx_rpt(DELETION);
		in_rpt("%s is a regular file", file);
	}
#endif
	if (eaccess(file, X_OK) == SYSERROR) {
		if (test_fail++ == 0)
			xx_rpt(DELETION);
		in_rpt("Execute permission for %s not set", file);
	}

	DBUG_RETURN(test_fail? -1: 0);

}

private void
pa_toobig()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_toobig");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitpid() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_MISC) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) did not return as expected", NAME(), t_prog);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_toobig()
{
	int	err;
	int	ret;
	int	pathok = setupok;

	DBUG_ENTER("ch_toobig");

	test_deleted = 0;
	test_fail = 0;
	toobig_exec();
	do_save();
	if(test_deleted)
		DBUG_EXIT(OK);
	ret = doexec();
	err = errno;
	do_cmp();
	if(ret != SYSERROR || err != E2BIG) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) did not return expected value", 
							NAME(), t_prog);
		valrpt(SYSERROR, ret, E2BIG, err);
	} else if (!test_deleted && !test_fail)
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
exec_setid()
{
	int	err;
	char	*teststr;
	char	*intstr1;
	char	*intstr2;
	char	*intstr3;
	char	*args[8];

	DBUG_ENTER("exec_setid");

	if ((teststr = newstr(tsetid_file)) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((intstr1 = newstr(itoa(vguid1))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((intstr2 = newstr(itoa(vggid1))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((intstr3 = newstr(itoa(tsetid_mode))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	
	/* Can't use execle() - null vg_dbug stops environ being passed */

	args[0] = t_setid;
	args[1] = teststr;
	args[2] = intstr1;
	args[3] = intstr2;
	args[4] = intstr3;
	args[5] = vg_ident;
	args[6] = vg_dbug;
	args[7] = (char *) NULL;

	(void) tet_exec(execpath(t_setid), args, environ);
	err = errno;
	vsxerror("tet_exec(%s,..) failed errno %d", execpath(t_setid), err);
	DBUG_EXIT(E_EXEC);

	DBUG_VOID_RETURN;
}

private void
toobig_exec()
{
	/*
	 * Try to exec so much data as to cause an error.
	 */
	private	char	*p = (char *) NULL;
	private	char	**pp;
	int	i;
	size_t	size2;
	char	*big2;
	char	*str;
			 
	DBUG_ENTER("toobig_exec");

	p_argv[0] = vg_ident;
	p_argv[1] = newstr(itoa(tet_thistest));
	p_argv[2] = newstr(ltoa(arg_max));
	p_argv[3] = newstr(ltoa(arg_max));/* Unused - any non NULL value is OK*/
	size2 = arg_max + 1;
	big2 = (char *)malloc(size2);
	if (big2 == NULL)
	{
		vsxerror("malloc() failed");
		DBUG_EXIT(A_NOMEM);
	}
	for(str = big2, i = 0; i < size2; str++, i++) {
		if(i & 01) {
			*str = '1';
		} else {
			*str = '0';
		}
	}
	big2[size2 - 1] = '\0';
	p_argv[4] = big2;
	p_argv[5] = vg_ident;
	p_argv[6] = vg_dbug;
	p_argv[7] = (char *) NULL;
	p_argc = 7;

	pp = &p;
	if (iwhich != I_EXECLE && iwhich != I_EXECVE)
		environ = pp;

	DBUG_VOID_RETURN;
}
