/*
*	SCCS:	@(#)	POSIX.os/procprim/exec/exec_setid.c	Rel 4.4.1     (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/exec - exec_setid Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		exec_setid - used by T.exec in several tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.exec to set user and group ID's and file mode for files.
	  usage:
		exec_setid file uid gid mode vg_ident vg_dbug

METHODS:
ASSUMES:  exec_setid must be setuid root.
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, Unisoft Ltd.
DATE CREATED:	Wed Jun 10 13:12:13 BST 1992
MODIFICATIONS:

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<errno.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<sys/times.h>
#include	<exec.h>
#include <tet_api.h>
#include	<setprv.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif


extern      void    set_pname();

extern	int	vguid0;
extern	int	vguid1;
extern	int	vguid2;
extern	int	vggid0;
extern	int	vggid1;
extern	int	vggid2;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	int	uid, gid, file_mode;
	int	err;
	char	*file_name;

	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"exec_setid" : (argv[0]));
	if (argc == SETID_NARG+2)
	{
		if (argv[SETID_NARG+1] != (char *) NULL)
			vg_dbug = argv[SETID_NARG+1];
		argc--;
	}
	if (argc == SETID_NARG+1)
	{
		if (argv[SETID_NARG] != (char *) NULL)
			vg_ident = argv[SETID_NARG];
		argc--;
	}
	if (argc != SETID_NARG)
	{
		xx_rpt(DELETION);
		in_rpt("exec_setid: argument count wrong - expected %d, actual %d", SETID_NARG, argc);
		return A_MISC;
	}
	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if (setprv(PRV_CHOWN) != 0) {
		xx_rpt(DELETION);
		in_rpt("exec_setid: setprv(PRV_CHOWN) failed");
		return A_TSETDIR;
	}

	if (setprv(PRV_SETID) != 0) {
		xx_rpt(DELETION);
		in_rpt("exec_setid: setprv(PRV_SETID) failed");
		return A_TSETDIR;
	}

	if(uids()!=0) {
		xx_rpt(DELETION);
		in_rpt("exec_setid: uids() failed");
		return A_TSETDIR;
	}

	file_name = argv[1];
	uid       = atoi(argv[2]);
	gid       = atoi(argv[3]);
	file_mode = atoi(argv[4]);

	if(uid != vguid0 && uid != vguid1 && uid != vguid2) {
		xx_rpt(DELETION);
		in_rpt("exec_setid: user ID (%d) was not VSX_UID0, VSX_UID1 or VSX_UID2", uid);
		return A_TSETDIR;
	}

	if(gid != vggid0 && gid != vggid1 && gid != vggid2) {
		xx_rpt(DELETION);
		in_rpt("exec_setid: group ID (%d) was not VSX_UID0, VSX_UID1 or VSX_UID2", gid);
		return A_TSETDIR;
	}

	/* Make the owner and mode correct for the file exec'ed */
	if(chown(file_name, (uid_t)uid, (gid_t)gid) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("exec_setid: chown failed - errno %d", err);
		return A_TSETDIR;
	}

	/* Set mode so file is set-user ID */
	if(chmod(file_name, (mode_t)file_mode) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("exec_setid: chmod() failed - errno %d", err);
		return A_CHMODFAIL;
	}

	return 0;
}
