/*
*      SCCS:  @(#)  POSIX.os/procprim/exec/exec_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/exec - exec_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		exec_su - used by T.exec in several tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.exec to test user and group ID's after exec() calls.
	  This file exec's another in which the reporting is conducted.
	  This may be set-user or set-group ID depending on the test itself.
	  usage:
		exec_su testnum iwhich startdir vg_ident vg_dbug

	  where testnum is the number of the test within the testset,
	  iwhich indicates which exec() function to use,
	  and startdir is the location of the original programs to exec.
METHODS:
ASSUMES:  exec_su must be setuid root.
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, Unisoft Ltd.
DATE CREATED:	Tue Apr 25 14:03:01 BST 1989
MODIFICATIONS:
		DPJ Cater, UniSoft Ltd.  Tue Jul 11 18:13:54 BST 1989
		Added do_access() test (test 32).

		Geoff Clare, 15 Nov 1989
			New do_access() argument

		David Sawyer
		Mon Aug 20 12:30:33 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.
			Changed main() to tet_main().
			Eliminated vg_file

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 13 May 1991
			Removed p_argv[8] from calls in do_exec() so
			as not to pass a null envp to execle().

		Peter Skan, 12 Sept 1991
			Set/verify CHOWN and SETID privileges

		Neil Moses, 29 October 1992
			Add new tests 48 and 49

		Dave Cater, 10 December 1992
			Alter vsxerror() to pipe reporting for test32.
			
		John-Paul, 8 January 1993
			Add sync point argument to do_access() calls.
		
		Geoff Clare, 12 Feb 1993
			Remove test 49 stuff (now done on server)

		Geoff Clare, 7 Jan 1994
			Inserted tet_logoff() calls before do_exec() and
			do_t32() calls.

		Geoff Clare, 20 Sept 1995
			Missing argument to vsxerror().

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<string.h>
#include	<errno.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<sys/times.h>
#include	<exec.h>
#include	<tet_api.h>
#include	<setprv.h>
#include	<xnfs.h>
#include	"exec_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MAXARG		10
#define	GEN_MODE	0755
#define	GID_MODE	02755
#define	UID_MODE	04755
#define	UGID_MODE	06755
#define	SU_UID		0
#define	NAME()		(i_name[iwhich])

extern	void	cln_access();
extern	void	set_pname();
extern	char	*newstr();
extern	char	*itoa();
extern	void	pipe_xx_rpt();
extern	void	pipe_end_rpt();
extern	void	pipe_in_rpt();
extern	void	pipe_xs_rpt();

extern	int	vguid1;
extern	int	vguid2;
extern	int	vggid1;
extern	int	vggid2;
extern	char	**environ;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private int	testnum;
private int	iwhich;
private int	test_fail;
private char	*p_argv[MAXARG];
private	char	*t_prog = "../exec_uids";
private	char	*t_path;
private	char	t32_prog[] =		"exec_tok";
private	char	t32_buf[_POSIX_PATH_MAX];
private	char	*i_name[6] = 		{ "execl",
					  "execle",
					  "execlp",
					  "execv",
					  "execve",
					  "execvp"
					};

private	void	su_t25();
private	void	su_t26();
private	void	su_t27_48();
private	void	su_t28();
private	void	su_t29();
private	void	su_t32();
private	void	subtest();
private	void	init_uids();
private	int	do_exec();

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"exec_su" : (argv[0]));
	if (argc == SU_NARG+2)
	{
		if (argv[SU_NARG+1] != (char *) NULL)
			vg_dbug = argv[SU_NARG+1];
		argc--;
	}
	if (argc == SU_NARG+1)
	{
		if (argv[SU_NARG] != (char *) NULL)
			vg_ident = argv[SU_NARG];
		argc--;
	}
	if (argc != SU_NARG)
	{
		vsxerror("argument count wrong");
		return A_MISC;
	}
	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if (setprv(PRV_CHOWN) != 0) {
		vsxerror("setprv(PRV_CHOWN) failed");
		return A_TSETDIR;
	}

	if (setprv(PRV_SETID) != 0) {
		vsxerror("setprv(PRV_SETID) failed");
		return A_TSETDIR;
	}

	if(uids()!=0)
	{
		pipe_xx_rpt(DELETION);
		pipe_end_rpt();
		return RPTED;
	}

	/* 
	 * Original location of programs to exec - 
	 * Strictly, this is only needed when testing XNFS.
	 * Also, only used in test32.
	 */
	(void) strcat(strcat(strcpy(t32_buf, argv[3]), "/"), t32_prog);

	subtest(argv[1], argv[2]);

	return RPTED;
}

private void
subtest(teststr, interfacestr)
char	*teststr;
char	*interfacestr;
{
	int	ret;
	int	err;

	DBUG_ENTER("subtest");

	testnum = atoi(teststr);

	if (testnum != 32)
	{
		/* Make the owner and mode correct for the file exec'ed */
		if(chown(t_prog, (uid_t)vguid2, (gid_t)vggid2) < 0) {
			err = errno;
			vsxerror("chown of \"%s\" failed - errno %d",
				t_prog, err);
			DBUG_TET_EXIT(A_TSETDIR);
		}
	}

	iwhich = atoi(interfacestr);

	switch(testnum) {
	case 25:
		su_t25();
		break;
	case 26:
		su_t26();
		break;
	case 27:
	case 48:
		su_t27_48();
		break;
	case 28:
		su_t28();
		break;
	case 29:
		su_t29();
		break;
	case 32:
		su_t32();

		DBUG_VOID_RETURN;

		/* NOTREACHED */
		break;
	default:
		vsxerror("internal error - unknown test %d", testnum);
		DBUG_TET_EXIT(A_TSETDIR);

		/* NOTREACHED */
		break;
	}

	init_uids(teststr);
#if TEST_XNFS && !defined(DTET1)
	tet_logoff();
#endif
	errno = NOERROR;
	ret = do_exec();
	err = errno;

	vsxerror("%s(%s,..) returned %d errno %d", NAME(), t_prog, ret, err);
	DBUG_TET_EXIT(A_EXECFAIL);

	DBUG_VOID_RETURN;
}

private void
init_uids(arg1)
char	*arg1;
{
	DBUG_ENTER("init_uids");

	p_argv[0] = t_prog;
	p_argv[1] = arg1;
	p_argv[2] = newstr(itoa(vguid1));
	p_argv[3] = newstr(itoa(vguid2));
	p_argv[4] = newstr(itoa(vggid1));
	p_argv[5] = newstr(itoa(vggid2));
	if(p_argv[2] == (char *)NULL ||
	   p_argv[3] == (char *)NULL ||
	   p_argv[4] == (char *)NULL ||
	   p_argv[5] == (char *)NULL) {
		vsxerror("external error - Out of memory");
		DBUG_TET_EXIT(A_NOMEM);
	}
	p_argv[6] = vg_ident;
	p_argv[7] = vg_dbug;
	p_argv[8] = (char *) NULL;

	DBUG_VOID_RETURN;
}

private int
do_exec()
{
	int	ret;

	DBUG_ENTER("do_exec");

	switch (iwhich)
	{
	case I_EXECL:
		ret = execl(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6],
			p_argv[7], (char *) NULL);
		break;
	case I_EXECLE:
		ret = execle(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6],
			p_argv[7], (char *) NULL, environ);
		break;
	case I_EXECLP:
		ret = execlp(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6],
			p_argv[7], (char *) NULL);
		break;
	case I_EXECV:
		ret = execv(t_prog, p_argv);
		break;
	case I_EXECVE:
		ret = execve(t_prog, p_argv, environ);
		break;
	case I_EXECVP:
		ret = execvp(t_prog, p_argv);
		break;
	}

	DBUG_RETURN(ret);
}

private void
su_t25()
{
	DBUG_ENTER("su_t25");

	/* Set mode so file is not set-user ID */
	if(chmod(t_prog, (mode_t)GEN_MODE) < 0) {
		vsxerror("chmod() failed - errno %d", errno);
		DBUG_TET_EXIT(A_CHMODFAIL);
	}

	/* Set real and effective user id by calling setuid() */
	if(setuid((uid_t)vguid1) < 0) {
		vsxerror("setuid() failed - errno %d", errno);
		DBUG_TET_EXIT(A_UIDFAIL);
	}

	/* Check effective user ID */
	if(geteuid() != vguid1) {
		vsxerror("Failure setting effective user ID");
		DBUG_TET_EXIT(A_UIDFAIL);
	}

	DBUG_VOID_RETURN;
}

private void
su_t26()
{
	DBUG_ENTER("su_t26");

	/* Set mode so file is not set-group ID */
	if(chmod(t_prog, (mode_t)GEN_MODE) < 0) {
		vsxerror("chmod() failed - errno %d", errno);
		DBUG_TET_EXIT(A_CHMODFAIL);
	}

	/* Set real and effective group id by calling setgid() */
	if(setgid((gid_t)vggid1) < 0) {
		vsxerror("setgid() failed - errno %d", errno);
		DBUG_TET_EXIT(A_UIDFAIL);
	}

	/* Check effective group ID */
	if(getegid() != vggid1) {
		vsxerror("Failure setting effective group ID");
		DBUG_TET_EXIT(A_UIDFAIL);
	}

	DBUG_VOID_RETURN;
}

private void
su_t27_48()
{
	DBUG_ENTER("su_t27_48");

	/* Set mode so file is set-user ID */
	if(chmod(t_prog, (mode_t)UID_MODE) < 0) {
		vsxerror("chmod() failed - errno %d", errno);
		DBUG_TET_EXIT(A_CHMODFAIL);
	}

	/* Set real and effective user id by calling setuid() */
	if(setuid((uid_t)vguid1) < 0) {
		vsxerror("setuid() failed - errno %d", errno);
		DBUG_TET_EXIT(A_UIDFAIL);
	}

	/* Check effective user ID */
	if(geteuid() != vguid1) {
		vsxerror("Failure setting effective user ID");
		DBUG_TET_EXIT(A_UIDFAIL);
	}

	DBUG_VOID_RETURN;
}

private void
su_t28()
{
	DBUG_ENTER("su_t28");

	/* Set mode so file is set-group ID */
	if(chmod(t_prog, (mode_t)GID_MODE) < 0) {
		vsxerror("chmod() failed - errno %d", errno);
		DBUG_TET_EXIT(A_CHMODFAIL);
	}

	/* Set real and effective group id by calling setgid() */
	if(setgid((gid_t)vggid1) < 0) {
		vsxerror("setgid() failed - errno %d", errno);
		DBUG_TET_EXIT(A_UIDFAIL);
	}

	/* Check effective group ID */
	if(getegid() != vggid1) {
		vsxerror("Failure setting effective group ID");
		DBUG_TET_EXIT(A_UIDFAIL);
	}

	DBUG_VOID_RETURN;
}

private void
su_t29()
{
	DBUG_ENTER("su_t29");

	/* Set mode so file is set-user ID  and set-group ID */
	if(chmod(t_prog, (mode_t)UGID_MODE) < 0) {
		vsxerror("chmod() failed - errno %d", errno);
		DBUG_TET_EXIT(A_CHMODFAIL);
	}

	/* SET GROUP THINGS FIRST SO USER PRIVILEGE IS NOT LOST */

	/* Set real and effective group id by calling setgid() */
	if(setgid((gid_t)vggid1) < 0) {
		vsxerror("setgid() failed - errno %d", errno);
		DBUG_TET_EXIT(A_UIDFAIL);
	}

	/* Check effective group ID */
	if(getegid() != vggid1) {
		vsxerror("Failure setting effective group ID");
		DBUG_TET_EXIT(A_UIDFAIL);
	}

	/* Set real and effective user id by calling setuid() */
	if(setuid((uid_t)vguid1) < 0) {
		vsxerror("setuid() failed - errno %d", errno);
		DBUG_TET_EXIT(A_UIDFAIL);
	}

	/* Check effective user ID */
	if(geteuid() != vguid1) {
		vsxerror("Failure setting effective user ID");
		DBUG_TET_EXIT(A_UIDFAIL);
	}

	DBUG_VOID_RETURN;
}

public void
init_t32(arg1, arg2, arg3, arg4, arg5, arg6, arg7)
char	*arg1;
char	*arg2;
char	*arg3;
char	*arg4;
char	*arg5;
char	*arg6;
char	*arg7;
{
	DBUG_ENTER("init_t32");

	p_argv[0] = arg1;
	p_argv[1] = arg2;
	p_argv[2] = arg3;
	p_argv[3] = arg4;
	p_argv[4] = arg5;
	
	if(arg6 == (char *)NULL)
		p_argv[5] = "";
	else
		p_argv[5] = arg6;
	if(arg7 == (char *)NULL)
		p_argv[6] = "";
	else
		p_argv[6] = arg7;

	p_argv[7] = (char *) NULL;

	DBUG_VOID_RETURN;
}

public int
do_t32()
{
	int	ret;

	DBUG_ENTER("do_t32");

	switch (iwhich)
	{
	case I_EXECL:
		ret = execl(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			(char *) NULL);
		break;
	case I_EXECLE:
		ret = execle(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			(char *) NULL, environ);
		break;
	case I_EXECLP:
		ret = execlp(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			(char *) NULL);
		break;
	case I_EXECV:
		ret = execv(t_prog, p_argv);
		break;
	case I_EXECVE:
		ret = execve(t_prog, p_argv, environ);
		break;
	case I_EXECVP:
		ret = execvp(t_prog, p_argv);
		break;
	}

	DBUG_RETURN(ret);
}

private void
ch_t32func()
{
	int	rval;
	int	err;
	char	rbuf[256];

	DBUG_ENTER("t32_chfunc");

	init_t32(t_prog, newstr(itoa(testnum)), P1_ARG2, P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
#if TEST_XNFS && !defined(DTET1)
	tet_logoff();
#endif
	errno = NOERROR;
	rval = do_t32();
	if (rval == SYSERROR && errno == EACCES)
	{
		err = errno;
		pipe_xx_rpt(FAILURE);
		(void)sprintf(rbuf,"%s(%s, ...) when file has access permission", NAME(), t_prog);
		pipe_in_rpt(rbuf);
		(void)sprintf(rbuf,"returned %d with errno set to %d", rval, err);
		pipe_in_rpt(rbuf);
	} else {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf,"%s(%s, ...) when file has access permission", NAME(), t_prog);
		pipe_in_rpt(rbuf);
		(void)sprintf(rbuf,"returned with errno set to %d", err);
		pipe_in_rpt(rbuf);
	}
	pipe_end_rpt();
	DBUG_TET_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private int
exec_copy(from, to)
char	*from;
char	*to;
{
	/*
	 * Copy executable file from "from" to "to"
	 * without changing file permissions.
	 *
	 * Assumes write permission to "to" is available.
	 */

	FILE	*fromptr;
	FILE	*toptr;
	int	c;
	int	err;
	char	rbuf[256];

	if((fromptr = fopen(from, "r")) == (FILE *)NULL) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf, "fopen(%s, \"r\") failed - errno %d", 
							from, errno);
		pipe_in_rpt(rbuf);
		return(SYSERROR);
	}
	if((toptr = fopen(to, "w")) == (FILE *)NULL) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf, "fopen(%s, \"w\") failed - errno %d", 
							to, errno);
		pipe_in_rpt(rbuf);
		return(SYSERROR);
	}
	while((c = getc(fromptr)) != EOF)
		if(putc(c, toptr) == EOF)
			break;
	err = errno;
	if(ferror(fromptr)) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf, "error reading from file %s, errno %d", 
							from, err);
		pipe_in_rpt(rbuf);
		return(SYSERROR);
	}
	if(ferror(toptr)) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf, "error reading from file %s, errno %d", 
							from, err);
		pipe_in_rpt(rbuf);
		return(SYSERROR);
	}
	(void) fclose(fromptr);
	(void) fclose(toptr);

	return(SUCCEED);
}

private int
t32_func(file, desc)
char	*file;
char	**desc;
{
	int	pathok = 0;

	DBUG_ENTER("t32_func");

	/* Copy executable file to "file" without changing file permissions */
	if(exec_copy(t32_buf, file) == SYSERROR) {
		test_fail++;
	} else {
		PATH_TRACE;

		/* In a child process, execution of file should succeed */
		test_fail = 0;
		t_prog = file;
		t_path = EXEC_PATH;
		if(cppair(ch_t32func, NULLFN, WAITTIME, 0xff) != OK) 
			test_fail++;
		else
			PATH_TRACE;
	}

	if(test_fail != 0) {
		for( ; *desc != NULL; desc++) 
			pipe_in_rpt(*desc);
	}	

	PATH_FUNC_RPT(2);

	DBUG_RETURN(test_fail);
}

private void
su_t32()
{
	DBUG_ENTER("su_t32");

	/*
	 * Test positive access using library function do_access().
	 */

	/*
	 * Give execute permission for path as test requires.
	 * Give write permission for path so executable file can be copied in.
	 * Give execute permission for path prefix to give search permission.
	 */

	globok = 0;
	if(do_access('f', (mode_t)(S_IXUSR|S_IWUSR), (mode_t)S_IXUSR, t32_func, 5*WAITTIME, SP_T32_DO_ACCESS) == 0)
	{
		pipe_xs_rpt();
		pipe_end_rpt();
	}

	cln_access();

	DBUG_VOID_RETURN;
}
