/*
*      SCCS:  @(#)  POSIX.os/procprim/exec/exec_tgen.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/exec - exec_tgen Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		exec_tgen.c - used in lots of exec() tests.

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
SYNOPSIS:
DESCRIPTION:
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Mon Apr 17 16:17:10 BST 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG

		Dave Cater, UniSoft Ltd.
		Tue Oct 31 14:00:47 GMT 1989
			Corrected test2() to test initial value of argc
			Corrected test5() to call getenv()
			Corrected an error message in test14()

		Geoff Clare, 30 May 1990
			Allow for unlimited ARG_MAX

		Geoff Clare, 26 June 1990
			Add test that directory streams are closed across exec

		David Sawyer
		Mon Aug 20 12:31:25 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Changed main to tet_main.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Stuart Boutell, 7 Nov 1990
			Changed tet_main back to main.
			Declared tet_pname, tet_thistest, tet_cleanup.

		Stuart Boutell,21 Nov 1990
			Changed all reporting to use pipes.
			Added pipe_end_rpt, pipe_in_rpt, pipe_xs_rpt,
			pipe_path_xs_rpt, pipe_xx_rpt.
			Removed all references to the tet_api

		Stuart Boutell,22 Nov 1990
			Move pipe_ functions to pipelib.c	
		
		Geoff Clare, 24 June 1992
			Changes to avoid writing to string literals.
		
		Geoff Clare, 24 Sept 1992
			Fix buggy code in subtest5() that happened to work
			on most systems because getenv("var=value") works
			the same as getenv("var").

		Dave Cater, 10 December 1992
			After XNFS changes, restore the vital exit(OK)
		
		Geoff Clare, 4 Jan 1994
			When testing XNFS redefine DBUG_EXIT so it doesn't
			use tet_exit().

		Geoff Clare, 18 Aug 1994
			Use vgetenv() instead of getenv() in subtest13().

		Geoff Clare, 30 Oct 1996
			Don't call pipe_end_rpt() for test 30.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<exitcodes.h>
#include	<report.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<ctype.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<sys/times.h>
#include	<exec.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif


#if TEST_XNFS
#undef DBUG_EXIT
#define DBUG_EXIT(a1) (DBUG_LEAVE, exit(a1))
#endif

#define	WAITTIME		(5 * SPEEDFACTOR)
#define	SLASH			"/"
#define	UMASK			022
#define PATH_OK			02

extern	char	*newstr();
extern	char	*errname();
extern	char	*signame();
extern	void	set_pname();
extern	char	*pr_char();
extern	char	*vgetenv();

extern	char	**environ;
extern	int	sig_array[];

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	int	test_fail = 0;
private	int	test_deleted = 0;
private	int	tim_fail = 0;
private	char	rptbuf[256];
private	sigset_t sig_empty;
private	int	sig_recvd;
private	int	nsigs;
private	int	testno;
private	struct	tres t30res;

private	void	subtest2();
private	void	subtest3();
private	void	subtest4();
private	void	subtest5();
private	void	subtest13();
private	void	subtest14();
private	void	subtest16();
private	void	subtest17();
private	void	subtest20();
private	void	subtest21();
private	void	t21p1();
private	void	t21p2();
private	void	testsigs();
private	void	subtest30();
private	void	subtest33();
private	void	after_exec();
extern	void	pipe_xx_rpt();
extern	void	pipe_xs_rpt();
extern	void	pipe_in_rpt();
extern	void	pipe_end_rpt();
extern	void	pipe_path_xs_rpt();
private	int	sig_init();
private	void	sig_catch();
private void	prepare_test(); 	/* made private, was extern ? */

int main(argc, argv)
	int	argc;
	char	*argv[];
{
	int	sav_argc;

	/* DO THIS FIRST - get alarm time left */
	t30res.tm_to_alarm = alarm(0);

	/* DO THIS NEXT - get process times - child time should be unaltered  */
	if(times(&t30res.timb) == (clock_t)SYSERROR)
		tim_fail++;
		
	/* Save value of argc for tests 2 and 3 - argc should be P1_NARG + 2 */
	sav_argc = argc;

	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"exec_tgen" : (argv[0]));

	/* argc should be P1_NARG + 2, but try to continue even if it isn't */
	if (argc == P1_NARG+2)
	{
		if (argv[P1_NARG+1] != (char *) NULL)
			vg_dbug = argv[P1_NARG+1];
		argc--;
	}
	if (argc == P1_NARG+1)
	{
		if (argv[P1_NARG] != (char *) NULL)
			vg_ident = argv[P1_NARG];
		argc--;
	}
	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if(argc >= 2) 
		testno = atoi(argv[1]);
	else {
		vsxerror("internal error - argc is %d - should be at least %d",
								argc, 2);
		return A_TSETDIR;
	}

	switch (testno)
	{
	case 2:
		subtest2(sav_argc);
		break;

	case 3:
		subtest3(sav_argc, argv);
		break;

	case 4:
		subtest4();
		break;

	case 5:
		subtest5();
		break;

	case 13:
		subtest13();
		break;

	case 14:
		subtest14();
		break;

	case 16:
		subtest16(argc, argv);
		break;

	case 17:
		subtest17(argc, argv);
		break;

	case 20:
		subtest20(argc, argv);
		break;

	case 21:
		subtest21(argc, argv);
		break;

	case 22:
		testsigs(SIG_DFL);
		break;

	case 23:
		testsigs(SIG_IGN);
		break;

	case 24:
		testsigs(SIG_DFL);
		break;

	case 30:
		subtest30(argc, argv);
		break;

	case 33:
		subtest33(atoi(argv[2]));
		break;

	default:
		vsxerror("internal error - unknown test %d", testno);
		return A_TSETDIR;

		/* NOTREACHED */
		break;
	}

	if (testno != 30) /* test 30 does not use pipe*rpt */
		pipe_end_rpt();
	exit(OK);

	/* NOTREACHED */
	return 0;
}


private void
subtest2(argc)
int	argc;
{

	DBUG_ENTER("subtest2");

	if(argc != P1_NARG + 2) {
		pipe_xx_rpt(FAILURE);
		(void)sprintf(rptbuf,"argc set incorrectly by exec: expected %d, got %d", P1_NARG+2, argc);
		pipe_in_rpt(rptbuf);
	}
	else
	{
		pipe_xs_rpt();
	}

	DBUG_VOID_RETURN;
}

private void
subtest3(argc, argv)
int	argc;
char	*argv[];
{
	int	fail = 0;
	int	pathok = 0;

	DBUG_ENTER("subtest3");

	if(argc != P1_NARG + 2) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"deletion reason - argc incorrect: expected %d, got %d",
							 P1_NARG+2, argc);
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(argv[argc] != (char *)NULL)
		fail |= 020;

	if(fail)
	{
		pipe_xx_rpt(FAILURE);
		if(fail & 020)
			pipe_in_rpt("argv is not NULL terminated as expected");
	}
	else
		pipe_path_xs_rpt(1,pathok);

	DBUG_VOID_RETURN;
}

private void
subtest4()
{
	int	fail = 0;
	int	i;
	char	**envp = environ;
	int	pathok = 0;

	DBUG_ENTER("subtest4");
	if(!(envp[0] != (char *) NULL && strcmp(envp[0], EXEC_PATH) == 0
		&& envp[1] != (char *) NULL && strcmp(envp[1], P1_TSET) == 0
		&& envp[2] != (char *) NULL && strcmp(envp[2], P1_FILE) == 0
		&& envp[3] != (char *) NULL && strcmp(envp[3], P1_TENV1) == 0
		&& envp[4] != (char *) NULL && strcmp(envp[4], P1_LOWER) == 0
		&& envp[5] != (char *) NULL && strcmp(envp[5], P1_UPPER) == 0
		&& envp[6] != (char *) NULL && strcmp(envp[6], P1_PORT) == 0
		&& envp[7] == NULL))
			fail = 2;
	else
		PATH_TRACE;

	if(fail)
	{
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("\"extern char **environ\" is incorrect");
		pipe_in_rpt("expected:");
		(void)sprintf(rptbuf,"\t`%s'", EXEC_PATH);
		pipe_in_rpt(rptbuf);
		(void)sprintf(rptbuf,"\t`%s'", P1_TSET);
		pipe_in_rpt(rptbuf);
		(void)sprintf(rptbuf,"\t`%s'", P1_FILE);
		pipe_in_rpt(rptbuf);
		(void)sprintf(rptbuf,"\t`%s'", P1_TENV1);
		pipe_in_rpt(rptbuf);
		(void)sprintf(rptbuf,"\t`%s'", P1_LOWER);
		pipe_in_rpt(rptbuf);
		(void)sprintf(rptbuf,"\t`%s'", P1_UPPER);
		pipe_in_rpt(rptbuf);
		(void)sprintf(rptbuf,"\t`%s'", P1_PORT);
		pipe_in_rpt(rptbuf);
		pipe_in_rpt("got:");
		for (i = 0 ; envp[i] != (char *) NULL ; i++)
		{
			(void)sprintf(rptbuf,"\t`%s'", envp[i]);
			pipe_in_rpt(rptbuf);
		}
	}
	else
		pipe_path_xs_rpt(1,pathok);

	DBUG_VOID_RETURN;
}

private void
subtest5()
{
	struct	envtab {
		char	*e_name;
		char	*e_string;
		char	*e_val;
	};

	static	struct	envtab envt[] = {
		EXEC_PATH,	EXEC_PATH,	(char *)NULL,
		P1_TSET,	P1_TSET,	(char *)NULL,
		P1_FILE,	P1_FILE,	(char *)NULL,
		P1_TENV1,	P1_TENV1,	(char *)NULL,
		P1_LOWER,	P1_LOWER,	(char *)NULL,
		P1_UPPER,	P1_UPPER,	(char *)NULL,
		P1_PORT,	P1_PORT,	(char *)NULL
	};
	char	*str;
	char	*str2;
	int	fail = 0;
	int	i;
	int	loop = 0;
	int	pathok = 0;

	DBUG_ENTER("subtest5");


	for(i = 0; i < sizeof(envt)/sizeof(struct envtab); i++) {

		/* Use first part of environment string to pass to getenv() */
		str = strchr((str2 = newstr(envt[i].e_name)), '=');
		*str = '\0';
		envt[i].e_val = getenv(str2);
		if(envt[i].e_val == (char *)NULL)
		{
			fail = 1;
			continue;
		}
		else
			PATH_TRACE;
		/* Compare result with second part of environment string */
		str = strchr(envt[i].e_string, '=');
		if(strcmp(++str, envt[i].e_val))
			fail = 1;
		else
			PATH_TRACE;

		loop++;
	}

	if(fail)
	{
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("environment passed by exec has been changed");
		pipe_in_rpt("expected:");
		for(i = 0; i < sizeof(envt)/sizeof(struct envtab); i++)
		{
			(void)sprintf(rptbuf, "\t`%s'", envt[i].e_string);
			pipe_in_rpt(rptbuf);
		}
		pipe_in_rpt("got:");
		for(i = 0; i < sizeof(envt)/sizeof(struct envtab); i++)
		{
			if(envt[i].e_val == (char *)NULL)
				continue;
			(void)sprintf(rptbuf,"\t`%s=%s'", envt[i].e_name, envt[i].e_val);
			pipe_in_rpt(rptbuf);
		}
	}
	else
		pipe_path_xs_rpt(2*loop,pathok);

	DBUG_VOID_RETURN;
}

private void
subtest13()
{
	int	fail = 0;
	char	*lower;
	char	*upper;
	int	pathok = 0;

	DBUG_ENTER("subtest13");

	/* vgetenv() is used here because getenv() may return a pointer
	   to an internal static buffer */
	lower = vgetenv(environ, LOWER_ENV);
	upper = vgetenv(environ, UPPER_ENV);

	if(!lower || !upper) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"Test environment variables %s and %s not found",
							LOWER_ENV, UPPER_ENV);
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(strcmp(lower, LOWER_STR))
		fail |= 01;
	else
		PATH_TRACE;
	if(strcmp(upper, UPPER_STR))
		fail |= 02;
	else
		PATH_TRACE;

	if(fail) {
		pipe_xx_rpt(FAILURE);
		if(fail & 01) {
			(void)sprintf(rptbuf,"Lower case letters \"%s\" in environment",
								LOWER_STR);
			pipe_in_rpt(rptbuf);
			(void)sprintf(rptbuf,"altered to         \"%s\"", lower);
			pipe_in_rpt(rptbuf);
		}
		if(fail & 02) {
			(void)sprintf(rptbuf,"Upper case letters \"%s\" in environment",
								UPPER_STR);
			pipe_in_rpt(rptbuf);
			(void)sprintf(rptbuf,"altered to         \"%s\"", upper);
			pipe_in_rpt(rptbuf);
		}
	}
	else
		pipe_path_xs_rpt(3,pathok);

	DBUG_VOID_RETURN;
}

private void
subtest14()
{
	int	fail = 0;
	char	*port;
	int	pathok = 0;

	DBUG_ENTER("subtest14");

	port = getenv(PORT_ENV);

	if(!port) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"Test environment variable %s not found",
							PORT_ENV);
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(strcmp(port, PORT_STR))
		fail |= 01;
	else
		PATH_TRACE;

	if(fail) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("Portable filename character set in environment");
		(void)sprintf(rptbuf,"\"%s\"", PORT_STR);
		pipe_in_rpt(rptbuf);
		pipe_in_rpt("altered to");
		(void)sprintf(rptbuf,"\"%s\"", port);
		pipe_in_rpt(rptbuf);
	}
	else
		pipe_path_xs_rpt(2,pathok);

	DBUG_VOID_RETURN;
}

private void
subtest16(argc, argv)
int	argc;
char	*argv[];
{
	int	fail = 0;
	int	ptr_max;
	long	arg_max;
	long	i;
	char	*p1;
	int	pathok = 0;

	DBUG_ENTER("subtest16");

	if(argc != P1_NARG) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"argc should be %d, actual value %d", P1_NARG, argc);
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	arg_max = atol(argv[2]);
	ptr_max = atoi(argv[3]);

	/*
	 * Subtract one from (arg_max - ptr_max) due to NULL termination.
	 * This area of data is initialised in big_exec() in exec.c
	 */
	for(p1 = argv[4], i = 0; i < arg_max - ptr_max - 1; p1++, i++) {
		if (i==0)
			PATH_TRACE;
		if(i & 01) {
			if(*p1 != '1')
				fail |= 01;
		} else {
			if(*p1 != '0')
				fail |= 01;
		}
		if(fail & 01)
			break;
	}

	if(fail) {
		pipe_xx_rpt(FAILURE);
		(void)sprintf(rptbuf,"ARG_MAX (%ld) bytes of data not accepted", arg_max);
		pipe_in_rpt(rptbuf);
		(void)sprintf(rptbuf,"Error at byte %d ('%s')", p1 - argv[4], pr_char(*p1));
		pipe_in_rpt(rptbuf);
	}
	else
		pipe_path_xs_rpt(2,pathok);

	DBUG_VOID_RETURN;
}

private void
subtest17(argc, argv)
int	argc;
char	*argv[];
{
	int	fail = 0;
	int	pathok = 0;

	DBUG_ENTER("subtest17");
	if(argc >= 3 && strcmp(argv[2], P1_ARG2) != 0)
		fail |= 02;
	else
		PATH_TRACE;
	if(argc >= 4 && strcmp(argv[3], P1_ARG3) != 0)
		fail |= 04;
	else
		PATH_TRACE;
	if(argc >= 5 && strcmp(argv[4], P1_ARG4) != 0)
		fail |= 010;
	else
		PATH_TRACE;

	if(fail)
	{
		pipe_xx_rpt(FAILURE);
		if(fail & 02)
		{
			(void)sprintf(rptbuf,"arg 1 set incorrectly by exec: expected \"%s\", got \"%s\"", P1_ARG2, argv[1]);
			pipe_in_rpt(rptbuf);
		}
		if(fail & 04)
		{
			(void)sprintf(rptbuf,"arg 2 set incorrectly by exec: expected \"%s\", got \"%s\"", P1_ARG3, argv[2]);
			pipe_in_rpt(rptbuf);
		}
		if(fail & 010)
		{
			(void)sprintf(rptbuf,"arg 3 set incorrectly by exec: expected \"%s\", got \"%s\"", P1_ARG4, argv[3]);
			pipe_in_rpt(rptbuf);
		}
	}
	else
		pipe_path_xs_rpt(3,pathok);

	DBUG_VOID_RETURN;
}

private void
subtest20(argc, argv)
int	argc;
char	*argv[];
{
	int	err;
	int	fail = 0;
	int	fd1;		/* should be still open file descriptor */
	int	fd2;		/* should be closed on exec */
	struct	stat stbuf;
	int	pathok = 0;

	DBUG_ENTER("subtest20");

	if(argc != P1_NARG) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"argc should be %d, actual value %d", P1_NARG, argc);
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	fd1 = atoi(argv[2]);
	fd2 = atoi(argv[3]);

	errno = NOERROR;
	if(fstat(fd1, &stbuf) != 0) 
		fail |= 02;
	else
		PATH_TRACE;
	if(fstat(fd2, &stbuf) != SYSERROR)
		fail |= 04;
	else if(errno != EBADF) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"fstat() set errno to unexpected value %d (%s)", err,
							errname(err));
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(fail)
	{
		pipe_xx_rpt(FAILURE);
		if(fail & 02)
		{
			(void)sprintf(rptbuf,"file descriptor %d was not open after exec", 
									fd1);
			pipe_in_rpt(rptbuf);
		}
		if(fail & 04)
		{
			(void)sprintf(rptbuf,"file descriptor %d was open after exec", 
									fd2);
			pipe_in_rpt(rptbuf);
		}
	}
	else
		pipe_path_xs_rpt(3,pathok);

	DBUG_VOID_RETURN;
}

private void
subtest21(argc, argv)
int	argc;
char	*argv[];
{
	int	t_case;
	int	pathok = 0;

	DBUG_ENTER("subtest21");

	if(argc != P1_NARG) {
		if(!test_fail)
			pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"argc should be %d, actual value %d", P1_NARG, argc);
		pipe_in_rpt(rptbuf);
		pipe_end_rpt();
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	/* Case 1, do mode checks; Case 2, do locking tests. */
	t_case = atoi(argv[4]);

	switch(t_case) {
	case 1:
		globok = 0;
		t21p1(argv);
		PATH_FUNC_TRACE;
		break;
	case 2:
		t21p2(argv);
		PATH_TRACE; /* record fact that we past this point */
		break;
	default:
		if(!test_fail)
			pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"argv[4] should be \"1\" or \"2\" for test %d", testno);
		pipe_in_rpt(rptbuf);
		pipe_end_rpt();
		DBUG_EXIT(A_TSETDIR);
	}

	/* Exit to tell the parent what happened */
	if(test_fail)
		DBUG_EXIT(A_MISC);

	if (pathok ==2)
	{
		if (t_case == 1)
			DBUG_EXIT(PATH_OK);
	}
	else if(t_case == 2)
		DBUG_PRINTF("test21","Unexpected path trace value %d, expected 2.", pathok);

	DBUG_EXIT(OK);
}

private void
t21p1(argv)
char	**argv;
{
	int	err;
	int	fail = 0;
	int	fd1;		/* should be still open file descriptor */
	int	smode;
	int	fmode;
	off_t	offset;
	int	pathok = 0;

	DBUG_ENTER("t21p1");

	fd1 = atoi(argv[2]);

	errno = NOERROR;
	if((offset = lseek(fd1, (off_t)0, SEEK_CUR)) == (off_t)SYSERROR) {
		err = errno;
		if(!test_fail)
			pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"lseek() failed - errno %d (%s)", err, errname(err));
		pipe_in_rpt(rptbuf);
		pipe_end_rpt();
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	if(offset != FD_OFF)
		fail |= 02;
	else
		PATH_TRACE;
	if((smode = fcntl(fd1, F_GETFL, 0)) == SYSERROR) {
		err = errno;
		if(!test_fail)
			pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"fcntl(F_GETFL) failed - errno %d (%s)", 
						err, errname(err));
		pipe_in_rpt(rptbuf);
		pipe_end_rpt();
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;
	if(!(smode & O_APPEND))
		fail |= 04;
	else
		PATH_TRACE;
	if(!(smode & O_NONBLOCK))
		fail |= 010;
	else
		PATH_TRACE;
	if((fmode = (smode & O_ACCMODE)) != FD_MODE)
		fail |= 020;
	else
		PATH_TRACE;

	if(fail)
	{
		if(!test_fail);
			pipe_xx_rpt(FAILURE);
		if(fail & 02) {
			pipe_in_rpt("file offset altered by exec:");
			(void)sprintf(rptbuf,"Expected value %d: Actual value %d", FD_OFF, 
								offset);
			pipe_in_rpt(rptbuf);
		}
		if(fail & 14)
			pipe_in_rpt("file status flags altered by exec:");
		if(fail & 04) 
			pipe_in_rpt("Expected value O_APPEND not set");
		if(fail & 010) 
			pipe_in_rpt("Expected value O_NONBLOCK not set");
		if(fail & 020) {
			pipe_in_rpt("file modes altered by exec:");
			(void)sprintf(rptbuf,"Expected value %o: Actual value %o", FD_MODE, 
								fmode);
			pipe_in_rpt(rptbuf);
		}
		test_fail++;
	}

	PATH_FUNC_RPT(6);

	DBUG_VOID_RETURN;
}

private void
t21p2(argv)
char	**argv;
{
	int	fd1;
	int	fd2;
	struct stat stbuf;
	int	fail = 0;
	int	err;

/* lack of synchronisation between parent and child makes tracing difficult */

	DBUG_ENTER("t21p2");

	fd1 = atoi(argv[2]);
	fd2 = atoi(argv[3]);

	errno = NOERROR;
	if(fstat(fd1, &stbuf) != 0) 
		fail |= 02;
	if(fstat(fd2, &stbuf) != SYSERROR)
		fail |= 04;
	else if(errno != EBADF) {
		err = errno;
		if(!test_fail)
			pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"fstat() set errno to unexpected value %d (%s)", err,
							errname(err));
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;
	}

	if(fail)
	{
		if(!test_fail)
			pipe_xx_rpt(DELETION);
		if(fail & 02)
		{
			(void)sprintf(rptbuf,"file descriptor %d was not open after exec", 
									fd1);
			pipe_in_rpt(rptbuf);
		}
		if(fail & 04)
		{
			(void)sprintf(rptbuf,"file descriptor %d was open after exec", 
									fd2);
			pipe_in_rpt(rptbuf);
		}
		DBUG_VOID_RETURN;
	}

	/* Sleep for a while whilst parent makes locking attempt */
	(void) sleep(WAITTIME);

	DBUG_VOID_RETURN;
}

private void
testsigs(arg)
void	(*arg)();
{
	struct	sigaction act;
	char	*sg_name;
	int	sg_num;
	char	*exp;
	char	*obs;
	int	pathok = 0;

	DBUG_ENTER("testsigs");

	if(arg == SIG_IGN)
		exp = "SIG_IGN";
	else if(arg == SIG_DFL)
		exp = "SIG_DFL";
	else
		exp = "signal set to be caught";

	sg_num = SIGQUIT;
	sg_name = signame(sg_num);

	if(sigaction(sg_num, (struct sigaction *)NULL, &act) < 0) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"deletion reason - sigaction() failed - signal %d (%s)",
					sg_num, sg_name);
		pipe_in_rpt(rptbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(act.sa_handler == SIG_IGN)
		obs = "SIG_IGN";
	else if(act.sa_handler == SIG_DFL)
		obs = "SIG_DFL";
	else
		obs = "signal set to be caught";

	if(act.sa_handler != arg) {
		pipe_xx_rpt(FAILURE);
		(void)sprintf(rptbuf,"signal action wrongly set in new process - signal %d (%s)",
					sg_num, sg_name);
		pipe_in_rpt(rptbuf);
		(void)sprintf(rptbuf,"Expected action %s; Observed action: %s", exp, obs);
		pipe_in_rpt(rptbuf);
	}
	else
		pipe_path_xs_rpt(1,pathok);

	DBUG_VOID_RETURN;
}

private void
subtest30(argc, argv)
int	argc;
char	**argv;
{
	int	err;
	int	ret;
	int	res_pipe;
	int	fail = 0;

	DBUG_ENTER("subtest30");

	if(argc != P1_NARG) {
		vsxerror("argc incorrect: expected %d, got %d",
							 P1_NARG, argc);
		DBUG_EXIT(A_TSETDIR);
	}

	res_pipe = atoi(argv[2]);

	prepare_test();

	/* Obtain test results after exec() */
	after_exec();

	/* Write back to parent the details of the test results */
	errno = NOERROR;
	ret = write(res_pipe, (char *)&t30res, sizeof(t30res));
	err = errno;
	if(ret != sizeof(t30res)) {
		if(ret == SYSERROR) {
			vsxerror("write failed, errno %d", err);
			fail++;
		} else {
			vsxerror("write returned wrong value");
			vsxerror("Expected value %d; Observed value %d", 
							sizeof(t30res), ret);
			fail++;
		}
	}

	if(test_deleted)
		DBUG_EXIT(A_MISC);

	if(fail)
		DBUG_EXIT(A_WRITEFAIL);

	DBUG_VOID_RETURN;
}

private void
subtest33(fd)
int	fd;
{
	int	err;
	struct	stat stbuf;

	DBUG_ENTER("subtest33");

	DBUG_PRINTF("test33", "fd = %d", fd);

	if (fstat(fd, &stbuf) != SYSERROR)
	{
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("file descriptor opened by opendir() was not closed after exec");
	}
	else if (errno != EBADF)
	{
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"fstat() set errno to unexpected value %d (%s)", err,
							errname(err));
		pipe_in_rpt(rptbuf);
	}
	else
		pipe_xs_rpt();

	DBUG_VOID_RETURN;
}

private void
after_exec()
{
	int	i;
	int	err;
	int	res_ngrps;
	gid_t	*res_grps;
	int	max_groups;
	sigset_t set;
	int pathok = 0;

	DBUG_ENTER("after_exec");

	t30res.pathok = 0;

	if(tim_fail) {
		pipe_xx_rpt(DELETION);
		pipe_in_rpt("deletion reason - times() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t30res.pid  = getpid();
	t30res.ppid = getppid();
	t30res.pgrp = getpgrp();
	t30res.uid = getuid();
	t30res.gid = getgid();

	if((max_groups = sysconf(_SC_NGROUPS_MAX)) == SYSERROR) {
		pipe_xx_rpt(DELETION);
		pipe_in_rpt("deletion reason - sysconf(_SC_NGROUPS_MAX) failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	res_grps = (gid_t *)calloc((size_t)max_groups, sizeof(gid_t));
	if (res_grps == NULL)
	{
		pipe_xx_rpt(DELETION);
		pipe_in_rpt("deletion reason - calloc() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((res_ngrps = getgroups(max_groups, res_grps)) == SYSERROR) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"deletion reason - getgroups() failed errno %d", err);
		pipe_in_rpt(rptbuf);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t30res.ngrps = res_ngrps;
	for(i = 0; i < max_groups && i < MAXGRP; i++)
		t30res.grps[i] = res_grps[i];

	if(getcwd(t30res.cwdir, sizeof(t30res.cwdir)) == (char *)NULL) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"deletion reason - getcwd() failed errno", err);
		pipe_in_rpt(rptbuf);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chdir(SLASH) == SYSERROR) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"deletion reason - chdir() failed errno", err);
		pipe_in_rpt(rptbuf);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(getcwd(t30res.rootdir, sizeof(t30res.rootdir)) == (char *)NULL) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"deletion reason - getcwd() failed errno", err);
		pipe_in_rpt(rptbuf);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chdir(t30res.cwdir) == SYSERROR) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"deletion reason - chdir() failed errno %d", err);
		pipe_in_rpt(rptbuf);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	t30res.umsk = umask((mode_t)UMASK);

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		pipe_xx_rpt(DELETION);
		pipe_in_rpt("deletion reason - sigaction() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Call sigprocmask() to obtain signal mask */
	(void) sigemptyset(&t30res.sigmask);

	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGHUP);
	(void) sigaddset(&set, SIGTTIN);

	if(sigprocmask(SIG_UNBLOCK, &set, &t30res.sigmask) != 0) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rptbuf,"deletion reason - sigprocmask() failed errno %d", err);
		pipe_in_rpt(rptbuf);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* sigprocmask() should also have unblocked this pending signal */
	if(sig_recvd == SIGHUP) 
		t30res.sigpend = 1;

	t30res.pathok = (pathok == 10 ? 1 : 0);

	DBUG_VOID_RETURN;
}

private void
prepare_test()
{
	int	ret;
	int	i;

	DBUG_ENTER("subtest");

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		nsigs = VNOJSIG;
	} else {
		nsigs = VNUMSIG;
	}

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

