/*
*      SCCS:  @(#)  POSIX.os/procprim/exec/exec_uids.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/exec - exec_uids Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		exec_uids - used by T.exec in several tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.exec to test user and group ID's after exec() calls.
	  This may be set-user or set-group ID depending on the test itself.
	  usage:
		exec_uids testno ruid euid rgid egid vg_ident vg_dbug

	  testno is the number of the test within the testset,
	  ruid is real user ID before exec,
	  euid is owner of file before exec,
	  rgid is real group ID before exec,
	  egid is group of file before exec.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, Unisoft Ltd.
DATE CREATED:	Tue Apr 25 14:14:06 BST 1989
MODIFICATIONS:
		Dave Cater, Unisoft Ltd.
		Wed Nov 01 09:27:33 GMT 1989
		Check real ID is not altered in tests 27-28

		David Sawyer
		Mon Aug 20 12:32:18 BST 1990
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file.

		Stuart Boutell, 7 Nov 1990

		Stuart Boutell, 21 Nov 1990
			Removed all tet_ references. Adapted to pipe
			reporting.

		Neil Moses, 30 October 1992
			New tests for XNFS 48 and 49

		Geoff Clare, 4 Jan 1994
			Change DBUG_EXIT() to exit().

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<errno.h>
#include	<signal.h>
#include	<sys/times.h>
#include	<exec.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif


extern	void	set_pname();
extern	void	pipe_xx_rpt();
extern	void	pipe_end_rpt();
extern	void	pipe_in_rpt();
extern	void	pipe_xs_rpt();
extern	void	pipe_path_xs_rpt();

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	void	do_t25();
private	void	do_t26();
private	void	do_t27();
#if TEST_XNFS
private	void	do_t48();
private	void	do_t49();
#endif
private	void	do_t28();
private	void	do_t29();
private	void	subtest();

public	int	testno;
private	uid_t	ruid;
private	uid_t	euid;
private	gid_t	rgid;
private	gid_t	egid;
private	int	test_fail;
private	char	rbuf[256];

main(argc, argv)

	int	argc;
	char	**argv;
{
	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"exec_uids" : (argv[0]));

	if (argc == UID_NARG+2)
	{
		if (argv[UID_NARG+1] != (char *) NULL)
			vg_dbug = argv[UID_NARG+1];
		argc--;
	}
	if (argc == UID_NARG+1)
	{
		if (argv[UID_NARG] != (char *) NULL)
			vg_ident = argv[UID_NARG];
		argc--;
	}
	if (argc != UID_NARG)
	{
		vsxerror("argument count wrong");
		exit(A_MISC);
	}
	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testno = atoi(argv[1]);
	ruid = atoi(argv[2]);
	euid = atoi(argv[3]);
	rgid = atoi(argv[4]);
	egid = atoi(argv[5]);

	test_fail = 0;

	subtest();

	pipe_end_rpt();

	exit(RPTED);

        /* NOTREACHED */
        return(0);
}

private void
subtest()
{
	DBUG_ENTER("subtest");

	switch(testno) {
	case 25:
		do_t25();
		break;
	case 26:
		do_t26();
		break;
	case 27:
		do_t27();
		break;
	case 28:
		do_t28();
		break;
	case 29:
		do_t29();
		break;
#if TEST_XNFS
	case 48:
		do_t48();
		break;
	case 49:
		do_t49();
		break;
#endif
	default:
		vsxerror("internal error - unknown test %d", testno);
		DBUG_LEAVE; exit(A_TSETDIR);
		/* NOTREACHED */
		break;
	}
	pipe_end_rpt();

	DBUG_VOID_RETURN;
}

private void
do_t25()
{
	uid_t	uid;

	DBUG_ENTER("do_t25");

	/* Expect effective user ID to be same as real - parent did setuid() */
	if((uid = geteuid()) != ruid) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("Effective user ID incorrect after exec:");
		(void)sprintf(rbuf,"Expected value %d: Observed value: %d", ruid, uid);
		pipe_in_rpt(rbuf);
	} else
	{
		pipe_xs_rpt();
	}
	
	DBUG_VOID_RETURN;
}

private void
do_t26()
{
	gid_t	gid;

	DBUG_ENTER("do_t26");

	/* Expect effective group ID to be same as real - parent did setgid() */
	if((gid = getegid()) != rgid) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("Effective user ID incorrect after exec:");
		(void)sprintf(rbuf,"Expected value %d: Observed value: %d", rgid, gid);
		pipe_in_rpt(rbuf);
	} else
	{
		pipe_xs_rpt();
	}

	DBUG_VOID_RETURN;
}

private void
do_t27()
{
	uid_t	real_uid, uid;
	int	pathok = 0;

	DBUG_ENTER("do_t27");

	/* Check that real user ID has not been changed */
	if((real_uid = getuid()) != ruid) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("Real user ID incorrect after exec:");
		(void)sprintf(rbuf,"Expected value %d: Observed value: %d", ruid, real_uid);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((uid = geteuid()) != euid) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("Effective user ID incorrect after exec:");
		(void)sprintf(rbuf,"Expected value %d: Observed value: %d", euid, uid);
		pipe_in_rpt(rbuf);
	} else
	{
		PATH_TRACE;
		pipe_path_xs_rpt(2,pathok);
	}

	DBUG_VOID_RETURN;
}

private void
do_t28()
{
	gid_t	real_gid, gid;
	int	pathok = 0;

	DBUG_ENTER("do_t28");

	/* Check that real group ID has not been changed */
	if((real_gid = getgid()) != rgid) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("Real group ID incorrect after exec:");
		(void)sprintf(rbuf,"Expected value %d: Observed value: %d", rgid, real_gid);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((gid = getegid()) != egid) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("Effective group ID incorrect after exec:");
		(void)sprintf(rbuf,"Expected value %d: Observed value: %d", egid, gid);
	} else
	{
		PATH_TRACE;
		pipe_path_xs_rpt(2,pathok);
	}

	DBUG_VOID_RETURN;
}

private void
do_t29()
{
	int	err;
	uid_t	uid;
	gid_t	gid;
	int	pathok = 0;

	DBUG_ENTER("do_t29");

	/* If expected real and effective ID's match, test is set up wrongly */
	if((euid = geteuid()) == (ruid = getuid())) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf,"Effective user ID and real user ID both %d", euid);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((egid = getegid()) == (rgid = getgid())) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf,"Effective group ID and real group ID both %d", egid);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Do setuid() to change effective user ID back to real value */
	errno = NOERROR;
	if(setuid(ruid) != 0) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf,"deletion reason - setuid() failed - errno %d", err);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(geteuid() != ruid) {
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf,"deletion reason - failed to set effective user ID to real user ID");
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Do setgid() to change effective group ID back to real value */
	errno = NOERROR;
	if(setgid(rgid) != 0) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void)sprintf(rbuf,"deletion reason - setgid() failed - errno %d", err);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(getegid() != rgid) {
		pipe_xx_rpt(DELETION);
		pipe_in_rpt("deletion reason - failed to set effective group ID to real group ID");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Test that saved effective user ID can be used by setuid() */
	errno = NOERROR;
	if(setuid(euid) != 0) {
		err = errno;
		if(!test_fail);
			pipe_xx_rpt(FAILURE);
		(void)sprintf(rbuf,"setuid() failed - errno %d", err);
		pipe_in_rpt(rbuf);
		test_fail++;
	} else if((uid = geteuid()) != euid) {
		if(!test_fail)
			pipe_xx_rpt(FAILURE);
		pipe_in_rpt("setuid(saved-user ID) set effective user ID wrongly");
		(void)sprintf(rbuf,"Expected value %d; Observed value %d", euid, uid);
		pipe_in_rpt(rbuf);
		test_fail++;
	}
	else
		PATH_TRACE;

	/* Test that saved effective group ID can be used by setgid() */
	errno = NOERROR;
	if(setgid(egid) != 0) {
		err = errno;
		if(!test_fail);
			pipe_xx_rpt(FAILURE);
		(void)sprintf(rbuf,"setgid() failed - errno %d", err);
		pipe_in_rpt(rbuf);
		test_fail++;
	} else if((gid = getegid()) != egid) {
		if(!test_fail)
			pipe_xx_rpt(FAILURE);
		pipe_in_rpt("setgid(saved-group ID) set effective group ID wrongly");
		(void)sprintf(rbuf,"Expected value %d; Observed value %d", egid, gid);
		pipe_in_rpt(rbuf);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		pipe_path_xs_rpt(8,pathok);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
do_t48()
{
	uid_t	real_uid, uid;
	int	pathok = 0;

	DBUG_ENTER("do_t48");

	/* Check that real user ID has not been changed */
	if((real_uid = getuid()) != ruid) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("Real user ID incorrect after exec:");
		(void)sprintf(rbuf,"Expected value %d: Observed value: %d", ruid, real_uid);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((uid = geteuid()) != euid) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("Effective user ID incorrect after exec:");
		(void)sprintf(rbuf,"Expected value %d: Observed value: %d", euid, uid);
		pipe_in_rpt(rbuf);
	} else
	{
		PATH_TRACE;
		pipe_path_xs_rpt(2,pathok);
	}

	DBUG_VOID_RETURN;
}

private void
do_t49()
{
	uid_t	real_uid, uid;
	int	pathok = 0;

	DBUG_ENTER("do_t49");

	/* Check that real user ID has not been changed */
	if((real_uid = getuid()) != ruid) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("Real user ID incorrect after exec:");
		(void)sprintf(rbuf,"Expected value %d: Observed value: %d", ruid, real_uid);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Expect effective user ID to be same as real - parent did setuid() */
	if((uid = geteuid()) != ruid) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("Effective user ID incorrect after exec:");
		(void)sprintf(rbuf,"Expected value %d: Observed value: %d", ruid, uid);
		pipe_in_rpt(rbuf);
	} else
	{
		PATH_TRACE;
		pipe_path_xs_rpt(2,pathok);
	}

	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */
