/*
*      SCCS:  @(#)  POSIX.os/procprim/exec/xnfs_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/exec - xnfs_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		xnfs_su - used by T.exec in XNFS tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.exec to setup a remote file system with setUID set to 
	  true.
METHODS:
ASSUMES:  xnfs_su must be setuid root.
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Neil Moses, Unisoft Ltd.
DATE CREATED:	Mon Oct 26 15:40:00 GMT 1992
MODIFICATIONS:
                Dave Cater, 3 December 1992
			Corrections to XNFS start and cleanup code

		Dave Cater, 10 December 1992
			Altered to use exec_tp4 in both tests 35 and 36.

		Geoff Clare, 10 Feb 1993
			Revised use of XNFS DEFAULT/ROOT/SPEC file systems.

		Geoff Clare, 7 March 1994
			Minor reporting change in test36.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<errno.h>
#include	<vsx_signal.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<sys/times.h>
#include	<exec.h>
#include <tet_api.h>
#include	<setprv.h>
#include	<xnfs.h>
#include	"exec_h.h"
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEGEN		(S_IRWXU|S_IRWXG|S_IRWXO)
#define	MAXARG		8
#define	NAME()		(i_name[iwhich])

extern	char	**environ;
extern	char	*t_prog;
extern	char	*t_path;
extern	void	do_save();
extern	void	do_cmp();
extern	int	doexec();
extern	char	*errname();
extern	char	*newstr();
extern	char	*itoa();
extern	void	set_pname();
extern	void	valrpt();
extern	void	init_exec();
extern	int	file_copy();

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
public  int     test_deleted;
public  int     test_fail;
public  sigset_t sig_empty;
public	int	sig_recvd;
public  int     iwhich;         /* what are we? (execl, execvp,... ) */
public	int	p_argc;
public	char	*p_argv[MAXARG];
public	char	**p_envp;
public	char	*i_name[6] = 		{ "execl",
					  "execle",
					  "execlp",
					  "execv",
					  "execve",
					  "execvp"
					};

private	int	tfail = 0;
private	int	testno;
private	char	*prog_list[] =	PROG_LIST;
private	char	t35_file[] =	FILE35;
private char    t35_dir[] =     DIR35;
private	char	t36_file[] =	FILE36;

private	void	do_setup();
private	void	do_t35();
private	void	do_t36();

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	int	pathok = 0;

	tfail = 0;

	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"xnfs_su" : (argv[0]));

	if (argc < 4 || argc > 6)
	{
		in_rpt("argument count wrong");
		tet_exit(XNFS_NOK);
	}
	else
		PATH_TRACE;

	testno = atoi(argv[2]);
	iwhich = atoi(argv[3]);
	if (argc > 4)
		vg_ident = argv[4];
	if (argc > 5)
		vg_dbug = argv[5];

	if (vg_dbug != NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	switch(testno)
	{
		case 0:
			do_setup(argv[1]);
			tet_exit(XNFS_OK);
			break;
		case 35:
			globok = 0;
			do_t35();
			PATH_FUNC_TRACE;
			break;
		case 36:
			globok = 0;
			do_t36();
			PATH_FUNC_TRACE;
			break;
		default:
			in_rpt(
		"Unknown test number passed to xnfs_su");
			tet_exit(XNFS_NOK);
	}

	if (test_deleted || test_fail)
		tet_exit(XNFS_NOK);
	else if(pathok == 2)
		tet_exit(XNFS_PATHOK);
	else
		tet_exit(XNFS_OK);
}

private void
do_setup(epath)
char	*epath;
{
	char	buf[_POSIX_PATH_MAX];
	int	i;

	DBUG_ENTER("do_setup");

	for(i = 0; prog_list[i] != NULL; i++)
	{
		(void) strcat(strcat(strcpy(buf, epath), "/"), prog_list[i]);
		if(file_copy(buf, prog_list[i]) == SYSERROR)
		{
			in_rpt("file_copy(\"%s\", \"%s\") failed in setup", buf,
				prog_list[i]);
			DBUG_TET_EXIT(XNFS_NOK);
		}
	}

	DBUG_VOID_RETURN;
}

private void
do_t35()
{
	int	rval, err;
	int	pathok = 0;

	DBUG_ENTER("do_t35");

	test_deleted = 0;
	test_fail = 0;

	if((p_envp = (char **) malloc(sizeof(char *))) == (char **) NULL) {
		in_rpt("malloc failed in xnfs_su");
		DBUG_TET_EXIT(XNFS_NOK);
	}

	t_prog = t35_file;
	t_path = EXEC_PATH;
	(void) init_exec(t_prog, newstr(itoa(testno)), P1_ARG2, P1_ARG3, P1_ARG4, vg_ident, vg_dbug);

	errno = NOERROR;
	do_save();
	if(test_deleted)
		DBUG_TET_EXIT(XNFS_NOK);
	rval = doexec();
	err = errno;
	do_cmp();
	if (rval != SYSERROR || err != EACCES)
	{
		test_fail++;
		in_rpt("%s(%s, ...) when directory %s has no search permission, did not return expected values", NAME(), t35_file, t35_dir);
		valrpt(SYSERROR, rval, EACCES, err);
	}
	else
	{
		PATH_TRACE;
		PATH_FUNC_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
do_t36()
{
	int	rval, err;
	int	pathok = 0;

	DBUG_ENTER("do_t36");

	test_deleted = 0;
	test_fail = 0;

	if((p_envp = (char **) malloc(sizeof(char *))) == (char **) NULL) {
		in_rpt("malloc failed in xnfs_su");
		DBUG_TET_EXIT(XNFS_NOK);
	}

	t_prog = t36_file;
	t_path = EXEC_PATH;
	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), P1_ARG2, P1_ARG3, P1_ARG4, vg_ident, vg_dbug);
	errno = NOERROR;
	do_save();
	if(test_deleted)
		DBUG_TET_EXIT(XNFS_NOK);
	rval = doexec();
	err = errno;
	do_cmp();
	if (rval != SYSERROR || err != EACCES)
	{
		test_fail++;
		in_rpt("%s(%s, ...) when there is no read or execute permission on %s, did not return expected values", NAME(), t36_file, t36_file);
		valrpt(SYSERROR, rval, EACCES, err);
	}
	else if ( !test_deleted && !test_fail )
	{
		PATH_TRACE;
		PATH_FUNC_RPT(1);
	}

	DBUG_VOID_RETURN;
}

