/*
*      SCCS:  @(#)POSIX.os/procprim/exec_X/exec_X.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/exec_X - T.exec_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.exec_X

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the exec() system call, X/OPEN extensions.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Tue Apr 11 10:26:43 BST 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG
		
		Geoff Clare, 8 Feb 1990
			Don't assume initial nice value is 0
		Keith Holder, 22 Jun 1990
			run test4 in a sub-process

		David Sawyer
		Mon Aug 20 13:40:54 BST 1990
		Added resilience code.

		Geoff Clare, 12 Oct 1990
			Guard against NULL arguments in init_exec().

		Stuart Boutell, 12 Nov 1990
			uids() return value check added
			moved signal initialisation to signal_init()

		Stuart Boutell, 4 Apr 1991
			Modified test 4 to ensure the test reports if the
			exec call succeeds.
		
		Geoff Clare, 25 June 1991
			VSX_PURE_FILE="unsup" for unsupported result
		
		Geoff Clare, 5 May 1992
			Renamed do_exec to call_exec (conflicted with genlib)

		Neil Moses, 12 Oct 1992
			XNFS enhancements

		Geoff Clare, 15 Feb 1993
			Revised use of XNFS DEFAULT/ROOT/SPEC file systems.

		Geoff Clare, 29 March 1994
			Guard against NULL arguments in test 4.

		Geoff Clare, 20 June 1994
			Change X_NICE to X_FSIZE in message in before_exec().

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<pcts_max.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<sys/times.h>
#include	<sys/wait.h>
#include	<stdio.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<string.h>
#include	<ulimit.h>
#include	<exec.h>
#include	<synch.h>
#include <tet_api.h>
#include	<xnfs.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif


#define	NO_TESTS	4		/* number of tests in testset */
#define	VSX_PURE_FILE	"VSX_PURE_FILE"
#define	MAXARG		9
#define	NAME()		(i_name[interface])
#define	WAITTIME	(5 * SPEEDFACTOR)
#define	MODEGEN		(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)

extern	void		vsx_sigs();
extern	void		valrpt();
extern	void		xx_all();
extern	char		*itoa();
extern	char		*newstr();
extern	char		*status_child();
extern	char		*errname();
extern	char		*execpath();
extern	void		pipe_close();
extern	void		pipe_parent();
extern	void		pipe_child();
extern	int		pipe_open();


extern	char	*vg_dbug;
extern	char	*vg_ident;
extern	char	**environ;
extern	int	sig_array[];
extern	int	ex_code;	/* defined in wait_child.c */
#if TEST_XNFS
extern	char		startdir[];
private	char	*prog_list[] = {
	"exec_tgn_X",
	"exec_tsh_X",
	NULL
};
#endif /* TEST_XNFS */

int	no_tests = NO_TESTS;

private int	p_argc;
private char	*p_argv[MAXARG];
private char	**p_envp;
private char	*rsn_s = 		"setup() failed to malloc memory";
private int	interface;
private char	*i_name[6] = 		{ "execl",
					  "execle",
					  "execlp",
					  "execv",
					  "execve",
					  "execvp"
					};

private	char	*t_prog;
private	char	*t_path;
private	char	tgen_prog[] =		"../exec_tgn_X";
private	char	tsh_prog[] =		"../exec_tsh_X";
#if	!TEST_XNFS
private	char	tpure_file[] =		"./exec_t.pure";
private	synch	*t4_sync;
#endif /* TEST_XNFS */
private	int	test_fail;
private	int	test_deleted;
private	int	niceval = -1;
private	int	nsigs;
public	int	setupok = -1;

private	void		test1();
private	void		pa1and2();
private	void		test2();
private	void		test3();
private	void		pa3();
private	void		test4();
#if !TEST_XNFS
private	void		chtest4();
private	void		patest4();
#endif /* !TEST_XNFS */
private	void		init_exec();
private	void		ch_exec();
private	void		pipe_ch_exec();
private	void		before_exec();
private	void		prepare_tests();
#if	TEST_XNFS
private	void		clean_tests();
private	int		file_copy();
#endif
private	void		setup();
private	int		s_init();
private	void		signal_init();


public	void	(*setfunc) () = prepare_tests;
#if	TEST_XNFS
public	void	(*clnfunc) () = clean_tests;
#else
public	void	(*clnfunc) () = NULL;
#endif
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4 , 4,
	NULL, 0
};

private	void
prepare_tests()
{
#if	TEST_XNFS
	int	i;
	char	buf[_POSIX_PATH_MAX];
	char	to_file[20];
#endif
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

#if	TEST_XNFS

	for(i = 0; prog_list[i] != NULL; i++)
	{
		if(i == 0)
			PATH_TRACE;

		(void) strcat(strcat(strcpy(buf, startdir), "/"), prog_list[i]);
		(void) strcat(strcpy(to_file, "../"), prog_list[i]);
		(void) unlink(to_file);
		ret = creat(to_file, MODEGEN);
		if (ret < 0)
		{
			in_rpt("creat(\"%s\", 0x%lx) failed in setup",
				to_file, (unsigned long)MODEGEN);
			xx_all(DELETION, "creat() failed in setup function");
			DBUG_VOID_RETURN;
		}
		(void) close(ret);
		if(file_copy(buf, to_file) == SYSERROR)
		{
			in_rpt("file_copy(\"%s\", \"%s\") failed in setup", buf,
				to_file);
			xx_all(DELETION, "file_copy() failed in setup function");
			DBUG_VOID_RETURN;
		}
	}


#else /* TEST_XNFS */

	PATH_TRACE;

#endif /* TEST_XNFS */

	/* guard against NULL vg_dbug being taken as arg list terminator */
	if (vg_dbug == NULL)
		vg_dbug = "-D";

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		nsigs = VNOJSIG;
	} else {
		nsigs = VNUMSIG;
	}


	setup();

	PATH_SETUP_RPT(2);

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private	void
clean_tests()
{
	int i;

	DBUG_ENTER("clean_tests");

	(void) chdir("..");
	for(i = 0; prog_list[i] != NULL; i++)
		(void) unlink(prog_list[i]);

	DBUG_VOID_RETURN;
}

private	int
file_copy(from, to)
	char	*from;
	char	*to;
{
	struct	stat stbuf;
	int	err;
	char	buf[_POSIX_PATH_MAX * 2 + 30];

	if (strlen(from) + strlen(to) > 2 * _POSIX_PATH_MAX)
	{
		vsxerror("path names too long in file_copy()");
		return(SYSERROR);
	}

	/* generate the string "cp from to" */
	(void) strcat(strcat(strcat(strcat(strcpy(buf, "cp "), from), " "),
		to), "> /dev/null 2>&1");
	DBUG_PRINTF("file_copy", "%s", buf);

	(void) unlink(to);
	
	if (stat(from, &stbuf) == SYSERROR)
	{
		err = errno;
		vsxerror("stat failed, errno %d", err);
		return (SYSERROR);
	}
	if (system(buf) != SUCCEED)
	{
		err = errno;
		(void) unlink(to);
		vsxerror("system(\"%s\") failed", buf);
		return (SYSERROR);
	}
	return (SUCCEED);
}

#endif /* TEST_XNFS */

private void
signal_init()
{
	int	i;
	sigset_t sig_empty;
	DBUG_ENTER("signal_init");

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) s_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);
	DBUG_VOID_RETURN;
}

private	void
setup()
{
	int	i;
	char	*s;
	char	*p;
	static	char	delreason[80];

	if((s = strrchr(vg_ident, '/')) != (char *)NULL)
		s+=3;
	else
		s = vg_ident+2;
	if (strcmp(s, "execl_X" ) == 0)
		interface = I_EXECL;
	else if (strcmp(s, "execle_X") == 0)
		interface = I_EXECLE;
	else if (strcmp(s, "execlp_X") == 0)
		interface = I_EXECLP;
	else if (strcmp(s, "execv_X" ) == 0)
		interface = I_EXECV;
	else if (strcmp(s, "execve_X") == 0)
		interface = I_EXECVE;
	else if (strcmp(s, "execvp_X") == 0)
		interface = I_EXECVP;
	else
	{
		(void) strcpy(delreason, "testset id not set correctly");
		vsxerror("%s: is %s, should be exec{l,le,lp,v,ve,vp}_X",
			delreason, vg_ident);
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i, delreason);
		return;
	}
	/*
	 *	Setup the environment path for exec[vl]p
	 */
	if (interface == I_EXECVP || interface == I_EXECLP)
	{
		if ((s = tet_getvar("VSX_PATH")) == (char *) NULL)
			s = "::/bin:/usr/bin";
		if ((p = (char *)malloc((size_t) (strlen(s) + strlen(EXEC_PATH) + 1)))
							== (char *) NULL)
		{
			for (i = 1; i <= no_tests; i++)
				(void) tet_delete(i, rsn_s);
			return;
		}
		(void) strcat(strcpy(p, EXEC_PATH), s);
		if (vputenv(&environ, p) == SYSERROR)
		{
			(void) strcpy(delreason, "could not set environment path");
			vsxerror("%s to %s", delreason, p);
			for (i = 1; i <= no_tests; i++)
				(void) tet_delete(i, delreason);
			return;
		}
	}

	if((p_envp = (char **) malloc(sizeof(char *))) == (char **) NULL) {
		for (i = 1; i <= no_tests; i++)
			(void) tet_delete(i, rsn_s);
		return;
	}
	p_envp[0] = (char *) NULL;
}

private int
s_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
test1()
{
	DBUG_ENTER("test1");
	signal_init();

	test_fail = 0;
	test_deleted = 0;

	t_prog = tgen_prog;
	t_path = EXEC_PATH;

	if(pipe_open()!=0)
		(void) cpsig(pipe_ch_exec, pa1and2, WAITTIME, 0);
	pipe_close();

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	signal_init();
	test_fail = 0;
	test_deleted = 0;

	t_prog = tgen_prog;
	t_path = EXEC_PATH;

	if(pipe_open()!=0)
		(void) cpsig(pipe_ch_exec, pa1and2, WAITTIME, 0);
	pipe_close();

	DBUG_VOID_RETURN;
}

private	void
pipe_ch_exec()
{
	DBUG_ENTER("pipe_ch_exec");
	pipe_child();
	ch_exec();
	DBUG_VOID_RETURN;
}
	

private int
call_exec()
{
	int	ret;

	DBUG_ENTER("call_exec");

	switch (interface)
	{
	case I_EXECL:
		ret = execl(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			p_argv[7], (char *) NULL);
		break;
	case I_EXECLE:
		ret = execle(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			p_argv[7], (char *) NULL, p_envp);
		break;
	case I_EXECLP:
		ret = execlp(t_prog, p_argv[0], p_argv[1], p_argv[2],
			p_argv[3], p_argv[4], p_argv[5], p_argv[6], 
			p_argv[7], (char *) NULL);
		break;
	case I_EXECV:
		ret = execv(t_prog, p_argv);
		break;
	case I_EXECVE:
		ret = execve(t_prog, p_argv, p_envp);
		break;
	case I_EXECVP:
		ret = execvp(t_prog, p_argv);
		break;
	}

	DBUG_RETURN(ret);
}

private void
pa1and2()
{
	int	err;
	int	stat_child;
	int	pathok = setupok;

	DBUG_ENTER("pa1and2");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitpid() failed - errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_EXECFAIL) {
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	pipe_parent();

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
before_exec()
{
	int	err;
	long	ulmt;

	DBUG_ENTER("before_exec");

	if(ulimit(UL_SETFSIZE, X_FSIZE) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - ulimit() failed - errno %d", err);
		DBUG_VOID_RETURN;
	}
	if((ulmt = ulimit(UL_GETFSIZE, (long)0)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - ulimit() failed - errno %d", err);
		DBUG_VOID_RETURN;
	}
	if(ulmt != X_FSIZE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - ulimit() failed to set file size limit to %ld",
									X_FSIZE);
		in_rpt("instead ulimit() set file size limit to %ld", ulmt);
		DBUG_VOID_RETURN;
	}

	if(nice(X_NICE) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - nice() failed - errno %d", err);
		DBUG_VOID_RETURN;
	}
	if((niceval = nice(0)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - nice() failed - errno %d", err);
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private void
ch_exec()
{
	int	err;

	DBUG_ENTER("ch_exec");

	/* Set up limit values prior to exec() */
	before_exec();

	(void) init_exec(t_prog, newstr(itoa(tet_thistest)), newstr(itoa(niceval)),
		P1_ARG3, P1_ARG4, P1_ARG5, vg_ident, vg_dbug);

	(void) call_exec();
	err = errno;
	xx_rpt(DELETION);
	in_rpt("%s(%s, ...) failed - errno %d (%s)",
		NAME(), t_prog, err, errname(err));
	DBUG_EXIT(A_EXECFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
init_exec(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8)
char	*arg1;
char	*arg2;
char	*arg3;
char	*arg4;
char	*arg5;
char	*arg6;
char	*arg7;
char	*arg8;
{
	int i;

	DBUG_ENTER("init_exec");

	p_argv[0] = arg1;
	p_argv[1] = arg2;
	p_argv[2] = arg3;
	p_argv[3] = arg4;
	p_argv[4] = arg5;
	p_argv[5] = arg6;
	p_argv[6] = arg7;
	p_argv[7] = arg8;
	
	/* Guard against NULL arguments */
	for (i = 0; i < 8; i++)
		if(p_argv[i] == NULL)
			p_argv[i] = "";

	p_argv[8] = (char *) NULL;
	p_argc = 8;

	p_envp[0] = (char *) NULL;
	(void) vputenv(&p_envp, t_path);
	(void) vputenv(&p_envp, P1_TSET);
	(void) vputenv(&p_envp, P1_FILE);
	(void) vputenv(&p_envp, P1_TENV1);
	(void) vputenv(&p_envp, P1_LOWER);
	(void) vputenv(&p_envp, P1_UPPER);
	(void) vputenv(&p_envp, P1_PORT);
	if (interface != I_EXECLE && interface != I_EXECVE)
		environ = p_envp;

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");
	signal_init();

	if(interface != I_EXECLP && interface != I_EXECVP) {
		xx_rpt(NOTINUSE);
		DBUG_VOID_RETURN;
	}

	test_fail = 0;
	test_deleted = 0;

	t_prog = tsh_prog;
	t_path = EXEC_PATH;

	(void) cpsig(ch_exec, pa3, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
pa3()
{
	int	err;
	int	stat_child;
	int	pathok = setupok;

	DBUG_ENTER("pa3");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitpid() failed - errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT) {
		xx_rpt(FAILURE);
		in_rpt("Following %s(%s), command interpreter did not exit", NAME(), t_prog);
		in_rpt("observed action: %s", status_child(stat_child));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(ex_code == A_EXECFAIL) {
		DBUG_VOID_RETURN;
	} else if(ex_code != OK) {
		xx_rpt(FAILURE);
		in_rpt("Following %s(%s), command interpreter gave wrong exit code", NAME(), t_prog);
		in_rpt("Value in command file: %d; Observed value: %d", 
								OK, ex_code);
	} else 
	{
		PATH_TRACE;
		PATH_XS_RPT(3);
	}

	globok = 1; 	/* cpsig expects globok to be set */

	DBUG_VOID_RETURN;
}

#if TEST_XNFS

private void
test4()
{
	DBUG_ENTER("test4");
	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN;
}

#else /* TEST_XNFS */

private void
test4()
{
	DBUG_ENTER("test4");
	signal_init();
	if( (t4_sync = opensync())  == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("opensync() failed");
		DBUG_VOID_RETURN;
	}
	(void) cppair(chtest4, patest4, WAITTIME*3, 0);
	(void) closesync(t4_sync);
	(void) unlink(tpure_file);

	DBUG_VOID_RETURN;
}

private void
chtest4()
{
	int	rval;
	int	err;
	int	sendval = 1;
	int	fd;
	char	*pure;
	int	pathok = setupok;

	DBUG_ENTER("chtest4");
	
	pure = tet_getvar(VSX_PURE_FILE);
	if ((pure == NULL) || (*pure == '\0'))
	{
		xx_rpt(DELETION);
		in_rpt("parameter %s is not set", VSX_PURE_FILE);
		(void) sendsync(t4_sync[0][1],(char *)&sendval, sizeof(sendval),
				WAITTIME);
		DBUG_VOID_RETURN;
	}
	else if (strncmp(pure, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("parameter %s is set to \"unsup\"", VSX_PURE_FILE);
		(void) sendsync(t4_sync[0][1],(char *)&sendval, sizeof(sendval),
				WAITTIME);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	globok = 0;
	if (pure_copy(pure, tpure_file) != SUCCEED)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: could not copy %s to %s", pure, tpure_file);
		if(sendsync(t4_sync[0][1],(char *)&sendval, sizeof(sendval),
				WAITTIME) != sizeof(sendval))
		{
			in_rpt("sendsync() failed");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if ((fd = open(tpure_file, O_WRONLY)) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("open(%s, O_WRONLY) failed - errno %d (%s)",
			tpure_file, err, errname(err));
		(void) unlink(tpure_file);
		if(sendsync(t4_sync[0][1],(char *)&sendval, sizeof(sendval),
				WAITTIME) != sizeof(sendval))
		{
			in_rpt("sendsync() failed");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
#ifdef TESTBED2
	(void) close(fd);
#endif
	p_argv[0] = vg_ident;
	p_argv[1] = (char *) NULL;
	p_envp[0] = (char *) NULL;
	(void) vputenv(&environ, EXEC_PATH);
	errno = NOERROR;
	t_prog = tpure_file;

	switch (interface)
	{
	case I_EXECL:  rval = execl (t_prog, p_argv[0], (char *) NULL); break;
	case I_EXECLE: rval = execle(t_prog, p_argv[0], (char *) NULL, p_envp); break;
	case I_EXECLP: rval = execlp(t_prog, p_argv[0], (char *) NULL); break;
	case I_EXECV:  rval = execv (t_prog, p_argv); break;
	case I_EXECVE: rval = execve(t_prog, p_argv, p_envp); break;
	case I_EXECVP: rval = execvp(t_prog, p_argv); break;
	}
	err  = errno;

#ifdef TESTBED
	err = ENOSPC;
#endif

	if (rval != SYSERROR || err != ETXTBSY)
	{
		xx_rpt(FAILURE);
		in_rpt("%s(%s, ...) on a busy pure executable did not return expected values",
			NAME(), tpure_file);
		valrpt(SYSERROR, rval, ETXTBSY, err);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4);
	}

	if( sendsync(t4_sync[0][1],(char *)&sendval,
			sizeof(sendval), WAITTIME) !=sizeof(sendval))
	{
		xx_rpt(DELETION);
		in_rpt("sendsync() failed");
	}
	(void) close(fd);

	DBUG_VOID_RETURN;
}

private void
patest4()
{
	int	res;
	int	status;
	int	err;
	DBUG_ENTER("patest4");

	errno = NOERROR;

	if ( ( res = waitsync(t4_sync[0][0],(char *)&status, 
				sizeof(status), WAITTIME*2)) != sizeof(status) )
	{
		err = errno;
		DBUG_PRINTF("patest4","res: %d, errno: %d (%s)", res, err,
				errname(err));
		if (alrm_flag)
		{
			xx_rpt(WARNING);
			in_rpt("%s(%s ,..) call succeeded (waitsync timed out)",
				NAME(), tpure_file);
		}
		else
		{
			xx_rpt(DELETION);
			in_rpt("waitsync() failed - errno %d (%s)",
				err, errname(err));
		}
	}
	DBUG_VOID_RETURN;
}

#endif /* TEST_XNFS */
