/*
*      SCCS:  @(#)  POSIX.os/procprim/exec_X/exec_tgn_X.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/exec_X - exec_tgn_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		exec_tgn_X.c

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
SYNOPSIS:
DESCRIPTION:
	Used by T.exec_X in tests 1 and 2 of exec() with X/OPEN extensions.
METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Fri Apr 28 16:59:06 BST 1989
MODIFICATIONS:
		Geoff Clare, 8 Feb 1990
			Pass expected nice value on command line

		David Sawyer
		Mon Aug 20 13:41:08 BST 1990
		Added resilience code.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file


************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<exitcodes.h>
#include	<report.h>
#include	<signal.h>
#include	<errno.h>
#include	<ctype.h>
#include	<sys/stat.h>
#include	<sys/times.h>
#include	<ulimit.h>
#include	<exec.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef execl
#undef execle
#undef execlp
#undef execv
#undef execve
#undef execvp
#endif


extern	void	set_pname();
extern	void	pipe_xx_rpt();
extern	void	pipe_in_rpt();
extern	void	pipe_end_rpt();
extern	void	pipe_path_xs_rpt();

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	int	testnum;
private	int	niceval;
private	char	rbuf[256];

private	void	test1();
private	void	test2();

int	main(argc, argv)
	int	argc;
	char	*argv[];
{
	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"exec_tgen" : (argv[0]));
	if (argc == P1_NARG+2)
	{
		if (argv[P1_NARG+1] != (char *) NULL)
			vg_dbug = argv[P1_NARG+1];
		argc--;
	}
	if (argc == P1_NARG+1)
	{
		if (argv[P1_NARG] != (char *) NULL)
			vg_ident = argv[P1_NARG];
		argc--;
	}
	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if(argc >= 3) {
		testnum = atoi(argv[1]);
		niceval = atoi(argv[2]);
	}
	else {
		vsxerror("internal error - argc is %d - should be at least %d",
								argc, 3);
		exit(A_TSETDIR);
	}

	switch (testnum)
	{
	case 1:
		test1();
		break;

	case 2:
		test2();
		break;
	default:
		vsxerror("internal error - unknown test %d", testnum);
		exit(A_TSETDIR);

		/* NOTREACHED */
		break;
	}

	pipe_end_rpt();
	exit(OK);

	/* NOTREACHED */
	return 0;
}


private void
test1()
{
	int	err;
	long	ulmt;
	int	pathok = 0;

	DBUG_ENTER("test1");

	if((ulmt = ulimit(UL_GETFSIZE, (long)0)) == SYSERROR) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void) sprintf(rbuf, "deletion reason - ulimit() failed - errno %d", err);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(ulmt != X_FSIZE) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("file size limit (in blocks) set incorrectly by exec:");
		(void) sprintf(rbuf,"expected value %ld; observed value %ld", X_FSIZE, ulmt);
		pipe_in_rpt(rbuf);
	}
	else
	{
		PATH_TRACE;
		pipe_path_xs_rpt(2, pathok);
	}

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	err;
	int	newval;
	int	pathok = 0;

	DBUG_ENTER("test2");

	if((newval = nice(0)) == SYSERROR) {
		err = errno;
		pipe_xx_rpt(DELETION);
		(void) sprintf(rbuf, "deletion reason - nice() failed - errno %d", err);
		pipe_in_rpt(rbuf);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(newval != niceval) {
		pipe_xx_rpt(FAILURE);
		pipe_in_rpt("nice value set incorrectly by exec:");
		(void) sprintf(rbuf, "expected value %d; observed value %d", niceval, newval);
		pipe_in_rpt(rbuf);
	}
	else
	{
		PATH_TRACE;
		pipe_path_xs_rpt(2, pathok);
	}

	DBUG_VOID_RETURN;
}
