/*
*      SCCS:  @(#)POSIX.os/procprim/fork/fork.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/fork - T.fork Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fork

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the fork() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Mon Apr  3 10:13:44 GMT 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Thu Sep 14 12:59:02 BST 1989
		test5() - add waste_time() call (for very fast machines)

		Geoff Clare, 15 Sep 1989
			Write c_cc[]'s as 1 char, not sizeof(cc_t)

		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG
		
		Geoff Clare, 19 Oct 1989
			Check calloc() return values.

		Dave Cater, 1 Nov 1989
			test5() - times() return value is clock_t not long
			test20() - close pipe file descriptors
			test20() - ensure read() does not block for ever.
		
		Geoff Clare, 31 May 1990
			Allow for unlimited OPEN_MAX and CHILD_MAX
		
		Geoff Clare, 8 June 1990
			UNSUPPORTED result if termios_prep() returns 1
		
		Geoff Clare, 26 June 1990
			Alter test19 strategy to ensure independent execution

		David Sawyer
		Mon Aug 13 15:33:38 BST 1990
		Replaced execle() calls with tet_exec().
		Added resilience code.

		Geoff Clare, 12 Oct 1990
			Guard against NULL vg_dbug.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.
			Removed tet_child declaration.
			Removed incorrect (int)casts where nec.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called

		Geoff Clare, 20 July 1992
			Change sav_child from int to pid_t.
			Cast pid_t to long for printing.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<pcts_max.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<sys/wait.h>
#include	<wait_child.h>
#include	<sys/times.h>
#include	<signal.h>
#include	<dirent.h>
#include	<termios.h>
#include	<fcntl.h>
#include	<string.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fork
#endif


#define	NO_TESTS		21	/* number of tests in testset */
#define	WAITTIME		(5 * SPEEDFACTOR)
#define	READ			0
#define	WRITE			1
#define	CREATMODE		0666
#define	UMASK			022
#define	BUFLEN			128
#define	PATH_LEN		_POSIX_PATH_MAX
#define	DOT			"."
#define	SLASH			"/"
#define	PARENT			"parent"
#define	CHILD			"child"
#define	ZERORET			0
#define	COUNT_UT	(1000 / SPEEDFACTOR)	/* count for user time wasting */
#define	COUNT_ST	(100 / SPEEDFACTOR)	/* count for system time wasting */

typedef	struct	{
		int	ofile;		/* NOERROR iff open file descriptor */
		int	fd;		/* file descriptor */
		dev_t	dev;		/* fd's device */
		ino_t	ino;		/* fd's inode */
		int	mode;		/* fd's mode */
		off_t	pos;		/* fd's file offset */
	} dim_t;	/* ie Device/Inode/Mode type... */

extern	void		vsx_sigs();
extern	void		setsigs();
extern	void		valrpt();
extern	char		*errname();
extern	char		*newstr();
extern	char		*itoa();
extern	char		*status_child();

extern	char		*vg_ident;
extern	char		*vg_dbug;
extern	char		**environ;
extern	int		vguid1;
extern	int		vguid2;
extern	int		sig_array[];
extern	int		ex_code;	/* defined in wait_child.c */
extern	int		sg_code;	/* defined in wait_child.c */

int	no_tests = NO_TESTS;

private char		*prog_su  = "../fork_su";
private char		*prog_18  = "../fork_t18";
private char		*prog_21  = "../fork_t21";
private char 		*args[5];
private	char		*open_file = "fork-ofile";
private	dim_t		*pa_fds;
private	dim_t		*ch_fds;
private	struct sigaction *pa_sigs;
private	struct sigaction *ch_sigs;
private	int		chld_max;
private	int		open_max;
private	int		open_fd;
private	int		pipe_fd[2];
private	int		msg_len;
private	char		msg_buf[] = "0123456789ABCDEF";
private	int		test_fail;
private	int		test_deleted;
private	sigset_t	sig_empty;
private	int		nsigs;
private	int		sig_recvd;
private	int		job_control;
private	int		saveids;
private	char		buffer[BUFLEN];
private	int		tty_fildes, loop_fildes;
private	struct termios	tty_termios, loop_termios,
			*loop_termios_p = &loop_termios,
			*tty_termios_p = &tty_termios;

private	void		test1();
private	void		test2();
private	void		test3();
private	void		ch_t3();
private	void		test4();
private	void		test5();
private	void		test6();
private	void		test7();
private	void		ch_t7();
private	void		test8();
private	void		ch_t8();
private	void		test9();
private	void		ch_t9();
private	void		test10();
private	void		ch_t10();
private	void		test11();
private	void		ch_t11();
private	void		test12();
private	void		ch_t12();
private	void		test13();
private	void		test14();
private	void		test15();
private	void		ch_t15();
private	void		test16();
private	void		ch_t16();
private	void		test17();
private	void		ch_t17();
private	void		test18();
private	void		ch_t18();
private	void		pa_t18();
private	void		test19();
private	void		ch_t19();
private	void		test20();
private	void		test21();
private	void		ch_t21();
private	void		wait_for_child();
private	void		wait_read();
private	void		save_fds();
private	void		save_sigs();
private	void		cmp_sigs();
private	void		cmp_mask();
private	void		cmp_grps();
private	void		prepare_tests();
private	void		prep_fds();
private	int		waste_time();
private	void		exec_suproc();
private	void		sig_catch();
private	int		sig_init();
private	int		setcmp();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	  test2, 2,
	  test3, 3,
	  test4, 4,
	  test5, 5,
	 test6, 6,
	  test7, 7,
	  test8, 8,
	  test9, 9,
	  test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	NULL, 0
};

private	void
prepare_tests()
{
	int	i;	/* Counter */
	int	ret;	/* Used for most return values */

	DBUG_ENTER("prepare_tests");

	test_deleted = 0;

	/* guard against NULL vg_dbug being taken as arg list terminator */
	if (vg_dbug == NULL)
		vg_dbug = "-D";

	vsx_sigs();

	if(uids()!=0)
	{
		for(i=1; i <= no_tests; i++)
		{
			(void) tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		job_control = 0;
		nsigs = VNOJSIG;
	} else {
		job_control = 1;
		nsigs = VNUMSIG;
	}

	/* Obtain max. number of open files */
	errno = 0;
	open_max = sysconf(_SC_OPEN_MAX);
	if(open_max <= 0) {
		if (open_max == -1 && errno == 0)
			open_max = PCTS_OPEN_MAX;
		else {
			for(i = 1; i <= NO_TESTS; i++) {
				(void) tet_delete(i, "sysconf(_SC_OPEN_MAX) failed");
			}
			DBUG_VOID_RETURN;
		}
	}

	/* Allocate arrays for signal descriptions */
	pa_sigs = (struct sigaction *)calloc((size_t)nsigs, 
						sizeof(struct sigaction));
	ch_sigs = (struct sigaction *)calloc((size_t)nsigs, 
						sizeof(struct sigaction));

	/* Allocate arrays for file descriptions */
	pa_fds = (dim_t *)calloc((size_t)open_max, sizeof(dim_t));
	ch_fds = (dim_t *)calloc((size_t)open_max, sizeof(dim_t));

	if (pa_sigs == NULL || ch_sigs == NULL ||
		pa_fds == NULL || ch_fds == NULL)
	{
		for(i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "calloc() failed in preparation");
		DBUG_VOID_RETURN;
	}

	/* Set up open files and pipes */
	prep_fds();

	if(test_deleted)
		DBUG_VOID_RETURN;

	/* check if save set UIDS are in the standard */
	saveids = sysconf(_SC_SAVED_IDS);

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

private void
interrupt()
{
	int	ret;
	int	err;
	unsigned char chr;

	DBUG_ENTER("interrupt");

        /* for child initialisation */
        (void) sleep(WAITTIME/2);

        sig_recvd = 0;

	DBUG_PRINTF("interrupt", "c_cc[VINTR] = 0%o",
		(int)tty_termios_p->c_cc[VINTR]);

        chr = tty_termios_p->c_cc[VINTR];

        SET_TIMEOUT(WAITTIME/2)

        ret = write(loop_fildes, (char *)&chr, 1);
	err = errno;
	if(ret == 1)
		(void) pause();

        CLEAR_ALARM

	if(ret != 1) {
		xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		test_fail++;
		DBUG_VOID_RETURN;
	}

        if(sig_recvd != SIGINT) {
		xx_rpt(FAILURE);
		in_rpt("process did not receive SIGINT when expected");
		test_fail++;
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private	int
waste_time()
{
	int		count;
	struct	stat	statbuf;
	int		dummy = 0;

	DBUG_ENTER("waste_time");
	for (count = 0; count <= 1000*COUNT_UT; count++)
		dummy = dummy + count / (dummy + 1) * (dummy % 3);

	for (count = 0; count <= 1000*COUNT_ST; count++)
		if (stat("/", &statbuf) == SYSERROR)
		{
			if (!test_fail++)
				xx_rpt(DELETION);
			in_rpt("deletion reason: stat('/') failed");
			DBUG_RETURN(SYSERROR);
		}

	DBUG_RETURN(OK);
}

private void
wait_for_child(waittime)
int	waittime;
{
	int	err;
	int	stat_child;

	DBUG_ENTER("wait_for_child");

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(waittime)) == CHLD_ERR) {
		err = errno;
		if(!test_deleted && !test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		if(!test_deleted && !test_fail)
			xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		if(stat_child == CHLD_LIVE ||
		   stat_child == CHLD_STOP)
			(void) killchild(tet_child, WAITTIME);
		test_deleted++;
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private void
test1()
{
	pid_t	ppid;
	int	pathok = 0;

	DBUG_ENTER("test1");

	/*
	 * Test that child process ID is different from parent process ID.
	 */

	test_fail = 0;
	test_deleted = 0;

	ppid = getpid();

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		if(getpid() == ppid) {
			xx_rpt(FAILURE);
			in_rpt("child process ID is not different from parent process ID");
		} else
		{
			PATH_TRACE;
			xs_rpt();
		}

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */
		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}
	tet_setblock();
	DBUG_VOID_RETURN;
}

private void
test2()
{
	pid_t	ppid;
	pid_t	cpid;
	int	pathok = 0;

	DBUG_ENTER("test2");

	/*
	 * Test that child's parent process ID is same as parent's process ID.
	 */

	test_fail = 0;
	test_deleted = 0;

	ppid = getpid();

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */
		tet_setcontext();
		if((cpid = getppid()) != ppid) {
			xx_rpt(FAILURE);
			in_rpt("child's parent process ID is different from parent's process ID");
			in_rpt("PPID in child: %ld; Parent's PID: %ld",
				(long)cpid, (long)ppid);
		} else
		{
			PATH_TRACE;
			xs_rpt();
		}

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */
		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}
	tet_setblock();
	DBUG_VOID_RETURN;
}

private void
test3()
{
	int	i;
	dim_t	*pa_dptr;

	DBUG_ENTER("test3");

	/*
	 * Test that child's open file descriptors refer to the same open 
	 * file description as the corresponding file descriptor in parent.
	 */

	test_fail = 0;
	test_deleted = 0;

	pa_dptr = pa_fds;

	for(i = 0; i < open_max; i++, pa_dptr++) {

		pa_dptr->fd = i;
		save_fds(pa_dptr);
	}

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		ch_t3();
		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}

	tet_setblock();
	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	dim_t	*pa_dptr;
	dim_t	*ch_dptr;
	char	*str;
	char	*ostr = "has open file description";
	char	*nostr = "is not open";
	int	i;
	int	err;
	int	ret;
	int	flag;
	int	pipe_err = 0;
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	/*
	 * Check each open file description in the child process.
	 */
	pa_dptr = pa_fds;
	ch_dptr = ch_fds;

	for(i = 0; i < open_max; i++, ch_dptr++, pa_dptr++) {

		ch_dptr->fd = i;
		save_fds(ch_dptr);

		/* Check the open files in child match parent's */
		if(ch_dptr->ofile != pa_dptr->ofile) {
			if(!test_fail) 
				xx_rpt(FAILURE);
			str = (ch_dptr->ofile == NOERROR) ? ostr : nostr;
			in_rpt("file descriptor %d %s in child", i, str);
			str = (pa_dptr->ofile == NOERROR) ? ostr : nostr;
			in_rpt("file descriptor %d %s in parent", i, str);
			test_fail++;
			continue;
		}
		else
			PATH_TRACE;

		if(ch_dptr->ofile != NOERROR)	/* parent will be the same */
		{
			PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE;
			PATH_TRACE; PATH_TRACE;
			continue;
		}
		else
			PATH_TRACE;

		/* Check open file descriptions correspond */
		if(ch_dptr->mode != pa_dptr->mode) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("modes of file descriptor %d differ", i);
			in_rpt("Child value: %d; Parent value: %d", 
						ch_dptr->mode, pa_dptr->mode);
			test_fail++;
		}
		else
			PATH_TRACE;
		if(ch_dptr->ino != pa_dptr->ino) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("inodes of file descriptor %d differ", i);
			in_rpt("Child value: %d; Parent value: %d", 
						ch_dptr->ino, pa_dptr->ino);
			test_fail++;
		}
		else
			PATH_TRACE;
		if(ch_dptr->dev != pa_dptr->dev) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("device ID of file descriptor %d differ", i);
			in_rpt("Child value: %d; Parent value: %d", 
						ch_dptr->dev, pa_dptr->dev);
			test_fail++;
		}
		else
			PATH_TRACE;

		/*
		 * Check file positioning in test fd's only.
		 * Report output streams likely to have been written to.
		 */
	
		if(ch_dptr->fd != open_fd &&
		   ch_dptr->fd != pipe_fd[READ] &&
		   ch_dptr->fd != pipe_fd[WRITE]) 
		{
			PATH_TRACE; PATH_TRACE;
			continue;
		}
		else
			PATH_TRACE;
	
		if(ch_dptr->pos != pa_dptr->pos) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("file offsets of file descriptor %d differ", i);
			in_rpt("Child value: %d; Parent value: %d", 
						ch_dptr->pos, pa_dptr->pos);
			test_fail++;
		}
		else
			PATH_TRACE;
	}

	/* Check we can read from file what was already written */
	(void) lseek(open_fd, 0L, SEEK_SET);
	errno = NOERROR;
	if((ret = read(open_fd, buffer, sizeof(buffer)))  ==
			SYSERROR || ret != msg_len) {
		err = errno;
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("open file descriptor: read() returned wrong value");
		valrpt(msg_len, ret, NOERROR, err);
		test_fail++;
	}
	else
		PATH_TRACE;

	/* Check file control modes */
	if((flag = fcntl(pipe_fd[READ], F_GETFL, 0)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - fcntl() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!(flag & O_NONBLOCK))
		pipe_err++;

	/* Set it whatever happened */
	if(fcntl(pipe_fd[READ], F_SETFL, flag | O_NONBLOCK) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - fcntl() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(pipe_err) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("pipe file descriptor: file control mode O_NONBLOCK altered by fork()");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check we can read from pipe what was already written */
	errno = NOERROR;
	if((ret = read(pipe_fd[READ], buffer, sizeof(buffer)))  ==
			SYSERROR || ret != msg_len) {
		err = errno;
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("pipe file descriptor: read() returned wrong value");
		valrpt(msg_len, ret, NOERROR, err);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(5+(7*open_max));

	DBUG_VOID_RETURN;
}

private void
prep_fds()
{
	int	i;
	int	ret;
	int	flag;

	DBUG_ENTER("prep_fds");

	msg_len = (int) strlen(msg_buf);
	if((open_fd = creat(open_file, CREATMODE)) == SYSERROR) {
		for(i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "creat() failed in prep_fds()");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	(void) close(open_fd);

	/* Reopen needed to get correct open mode */
	if((open_fd = open(open_file, O_RDWR)) == SYSERROR) {
		for(i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "open() failed in prep_fds()");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	if((ret = write(open_fd, msg_buf, (unsigned)msg_len)) == 
						SYSERROR || ret != msg_len) {
		for(i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "write() failed in prep_fds()");
		test_deleted++;
		DBUG_VOID_RETURN;
	}

	/* Doesn't really matter if it fails */
	(void) unlink(open_file);

	if(pipe(pipe_fd) == SYSERROR) {
		for(i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "pipe() failed in prep_fds()");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	if((ret = write(pipe_fd[WRITE], msg_buf, (unsigned)msg_len))  ==
						SYSERROR || ret != msg_len) {
		for(i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "write() failed in prep_fds()");
		test_deleted++;
		DBUG_VOID_RETURN;
	}

	flag = fcntl(pipe_fd[READ], F_GETFL, 0);
	if(fcntl(pipe_fd[READ], F_SETFL, flag | O_NONBLOCK) == SYSERROR) {
		for(i = 1; i <= NO_TESTS; i++)
			(void) tet_delete(i, "fcntl() failed in prep_fds()");
		test_deleted++;
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private void
save_fds(dptr)
dim_t	*dptr;
{
	struct	stat statb;

	/* 
	 * fill in file description parameters
	 */

	dptr->ofile = NOERROR;
	if(fstat(dptr->fd, &statb) == SYSERROR) {
		dptr->ofile = errno;
		return;
	}
	dptr->ino = statb.st_ino;
	dptr->dev = statb.st_dev;
	dptr->mode = statb.st_mode;
	dptr->pos = lseek(dptr->fd, 0L, SEEK_CUR);

	return;
}

private void
test4()
{
	DIR	*dirp;
	int	err;
	int	dircnt;
	int	pathok = 0;

	DBUG_ENTER("test4");

	/*
	 * Test that child process has private copies of directory streams.
	 */

	test_fail = 0;
	test_deleted = 0;

	if((dirp = opendir(DOT)) == (DIR *)NULL) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - opendir() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	dircnt = 0;
	while(readdir(dirp) != (struct dirent *)NULL)
		dircnt++;
	rewinddir(dirp);
	if(dircnt <= 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - test directory is empty");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		while(--dircnt >= 0)
			if(readdir(dirp) == (struct dirent *)NULL)
				test_fail++;

		if(readdir(dirp) != (struct dirent *)NULL)
			test_fail++;

		if(test_fail) {
			xx_rpt(FAILURE);
			in_rpt("child did not have private copy of parent's directory streams");
		}
		else
		{
			PATH_TRACE;
			PATH_XS_RPT(3);
		}

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}

	tet_setblock();
	DBUG_VOID_RETURN;
}

private void
test5()
{
	struct	tms ptb;
	struct	tms tb;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test5");

	/*
	 * Test that child process's values of process time are OK.
	 */

	/* Waste some time to make sure parent's utime, stime are not zero */
	if(waste_time() == SYSERROR)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	test_fail = 0;
	test_deleted = 0;

	if(times(&ptb) == (clock_t) SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: times() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(ptb.tms_utime == 0 || ptb.tms_stime == 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: tms_utime and tms_stime are not");
		in_rpt("both non-zero before fork()");
		in_rpt("\tutime: %ld; stime: %ld",
				ptb.tms_utime, ptb.tms_stime);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		if(times(&tb) == (clock_t) SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: times() failed errno %d", err);
		}
		else
		{
			PATH_TRACE;
			/* 
			 * tms_utime, tms_stime:
			 * Can't check that they are zero because of the gap
			 * since we forked. Check they are less than parent.
			 */
			if( (tb.tms_utime >= ptb.tms_utime) ||
			    (tb.tms_stime >= ptb.tms_stime) ||
			    (tb.tms_cutime != (time_t)0) ||
			    (tb.tms_cstime != (time_t)0) ) {
				xx_rpt(FAILURE);
				in_rpt("Child's process times:");
				in_rpt("\tutime: %ld; stime: %ld",
					tb.tms_utime, tb.tms_stime);
				in_rpt("\tcutime: %ld; cstime: %ld",
					tb.tms_cutime, tb.tms_cstime);
			} else {
				PATH_TRACE;
				PATH_XS_RPT(5);
			}
		}

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}

	tet_setblock();
	DBUG_VOID_RETURN;
}

private void
test6()
{
	struct	flock flock_fcntl;
	int	err;
	pid_t	sav_child;
	int	pathok = 0;

	DBUG_ENTER("test6");

	/*
	 * Test that locks are not inherited by child processes.
	 */

	test_fail = 0;
	test_deleted = 0;

	/* Make a lock on an open file */
	flock_fcntl.l_type = F_WRLCK;
	flock_fcntl.l_whence = SEEK_SET;
	flock_fcntl.l_start = (off_t)0;
	flock_fcntl.l_len = sizeof(int);
	flock_fcntl.l_pid = (pid_t)0;
	if(fcntl(open_fd, F_SETLK, &flock_fcntl) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		/* Sleep long enough for another locking attempt */
		(void) sleep(WAITTIME/2);

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		PATH_TRACE;
		break;
	}
	tet_setblock();

	/* Release the file lock created earlier */
	flock_fcntl.l_type = F_UNLCK;
	flock_fcntl.l_whence = SEEK_SET;
	flock_fcntl.l_start = (off_t)0;
	flock_fcntl.l_len = sizeof(int);
	flock_fcntl.l_pid = (pid_t)0;
	if(fcntl(open_fd, F_SETLK, &flock_fcntl) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fcntl() failed errno %d", err);
		test_deleted++;
		wait_for_child(WAITTIME);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Try to lock the file in another child process - should succeed */

	sav_child = tet_child;

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		flock_fcntl.l_type = F_WRLCK;
		flock_fcntl.l_whence = SEEK_SET;
		flock_fcntl.l_start = (off_t)0;
		flock_fcntl.l_len = sizeof(int);
		flock_fcntl.l_pid = (pid_t)0;
		if(fcntl(open_fd, F_SETLK, &flock_fcntl) == SYSERROR) {
			xx_rpt(FAILURE);
			in_rpt("lock created by parent process was inherited by child");
		} else 
		{
			PATH_TRACE;
			PATH_XS_RPT(4);
		}

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		wait_for_child(WAITTIME);

		/* ... and the one before ... */
		tet_child = sav_child;

		tet_setcontext();
		wait_for_child(WAITTIME);

		break;
	}
	tet_setblock();

	DBUG_VOID_RETURN;
}

private void
test7()
{
	DBUG_ENTER("test7");

	/*
	 * Test that pending alarms are cleared for child process.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t7, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t7");

	(void) alarm(WAITTIME);

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		if(alarm(WAITTIME) != 0) {
			xx_rpt(FAILURE);
			in_rpt("pending alarms not cleared in child process");
		} else
		{
			PATH_TRACE;
			xs_rpt();
		}

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		(void) alarm(0);

		wait_for_child(WAITTIME);
		break;
	}

	tet_setblock();
	DBUG_VOID_RETURN;
}

private void
test8()
{
	DBUG_ENTER("test8");

	/*
	 * Test that the set of pending signals is initialised to
	 * the empty set in the child.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t8, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	sigset_t set;
	int	ret;
	int	err;
	int	stat_child;
	pid_t	myself;
	int	pathok = 0;

	DBUG_ENTER("ch_t8");

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Call sigprocmask() to block signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGHUP);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Send the signal to this process */
	sig_recvd = 0;
	myself = getpid();
	ret = kill(myself, SIGHUP);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(sig_recvd) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() did not block signal");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		/* Call sigprocmask() to unblock signal */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, SIGHUP);
		if(sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL) != 0)
			DBUG_EXIT(A_TSETDIR);

		if(sig_recvd == SIGHUP) 
			DBUG_EXIT(A_MISC);
			
		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		PATH_TRACE;
		break;
	}

	tet_setblock();
	/* Call sigprocmask() to unblock signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGHUP);
	if((ret = sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL)) != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returned %d instead of 0", ret);
		test_deleted++;
		wait_for_child(WAITTIME);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check signal is received when unblocked */
	if(sig_recvd != SIGHUP) {
		xx_rpt(DELETION);
		in_rpt("signal %d was not pending before fork()", SIGPIPE);
		test_deleted++;
		wait_for_child(WAITTIME);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check child did not receive signal when unblocked. */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT) {
		xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		if(stat_child == CHLD_LIVE ||
		   stat_child == CHLD_STOP)
			(void) killchild(tet_child, WAITTIME);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(ex_code == A_TSETDIR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(ex_code == A_MISC) {
		xx_rpt(FAILURE);
		in_rpt("signal was unexpectedly caught by child process");
		in_rpt("set of pending signals was not initialised to the empty set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(11);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	DBUG_ENTER("test9");

	/*
	 * Test that current root and working directory are inherited.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t9, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	char	ch_cwd[PATH_LEN];
	char	pa_cwd[PATH_LEN];
	char	ch_root[PATH_LEN];
	char	pa_root[PATH_LEN];
	int	root_fail = 0;
	int	cwd_fail = 0;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch_t9");

	if(getcwd(pa_cwd, sizeof(pa_cwd)) == (char *)NULL) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - getcwd() failed errno", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chdir(SLASH) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - chdir() failed errno", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(getcwd(pa_root, sizeof(pa_root)) == (char *)NULL) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - getcwd() failed errno", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(chdir(pa_cwd) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - chdir() failed errno", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		if(getcwd(ch_cwd, sizeof(ch_cwd)) == (char *)NULL) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - getcwd() failed errno", err);
			DBUG_EXIT(OK);
		}
		else
			PATH_TRACE;

		if(chdir(SLASH) == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - chdir() failed errno", err);
			DBUG_EXIT(OK);
		}
		else
			PATH_TRACE;

		if(getcwd(ch_root, sizeof(ch_root)) == (char *)NULL) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - getcwd() failed errno", err);
			DBUG_EXIT(OK);
		}
		else
			PATH_TRACE;

		if(strncmp(ch_cwd, pa_cwd, PATH_LEN))
			cwd_fail++;
		if(strncmp(ch_root, pa_root, PATH_LEN))
			root_fail++;

		if(root_fail || cwd_fail)
			xx_rpt(FAILURE);
		if(cwd_fail) {
			in_rpt("current working directory in child not inherited after fork()");
			in_rpt("Child current working directory: %s", ch_cwd);
			in_rpt("Parent current working directory: %s", pa_cwd);
		}
		if(root_fail) {
			in_rpt("root directory in child not inherited after fork()");
			in_rpt("Child root directory: %s", ch_root);
			in_rpt("Parent root directory: %s", pa_root);
		}

		if(root_fail || cwd_fail)
			DBUG_EXIT(OK);

		PATH_XS_RPT(7);

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}

	tet_setblock();
	DBUG_VOID_RETURN;
}

private void
test10()
{
	DBUG_ENTER("test10");

	/*
	 * Test that signal settings for the child are the same as parent.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t10, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t10");

	/* Initialise the SIGHUP action to be caught */
	/* Initialise the SIGQUIT action to SIG_DFL  */
	/* Initialise the SIGPIPE action to SIG_IGN  */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0 ||
	   sig_init(sig_array[VSIGQUIT], &sig_empty, SIG_DFL, 0) < 0 ||
	   sig_init(sig_array[VSIGPIPE], &sig_empty, SIG_IGN, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	save_sigs(pa_sigs);

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
	
		globok = 0;
		cmp_sigs(pa_sigs, ch_sigs);
		PATH_FUNC_TRACE;

		if(test_fail || test_deleted)
			DBUG_EXIT(OK);

		PATH_XS_RPT(2);

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}
	tet_setblock();

	DBUG_VOID_RETURN;
}

private void
save_sigs(sigptr)
struct sigaction *sigptr;
{
	int	i;

	for(i = 1; i < nsigs; i++, sigptr++)
		(void) sigaction(sig_array[i], (struct sigaction *)NULL, 
								sigptr);
}

private void
cmp_sigs(sigptr1, sigptr2)
struct sigaction *sigptr1;
struct sigaction *sigptr2;
{
	int	i;
	int	sg_num;
	int	pathok = 0;

	DBUG_ENTER("cmp_sigs");

	for(i = 1; i < nsigs; i++, sigptr1++, sigptr2++) {

		if (i==1)
			PATH_TRACE;

		sg_num = sig_array[i];
	
		/* Obtain actions in child process */
		if(sigaction(sg_num, (struct sigaction *)NULL, sigptr2) != 0)
			continue;

		/* Compare with saved actions in parent process */
		if(sigptr1->sa_handler != sigptr2->sa_handler) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("signal handling action different for signal %d",
								sg_num);
			test_fail++;
		}
		if(setcmp(&sigptr1->sa_mask, &sigptr2->sa_mask) != 0) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("signal mask different for signal %d",
								sg_num);
			test_fail++;
		}
		if(sigptr1->sa_flags != sigptr2->sa_flags) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("signal flags different for signal %d",
								sg_num);
			test_fail++;
		}
	}

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
	
}

private void
test11()
{
	DBUG_ENTER("test11");

	/*
	 * Test that real and effective user and group ID are the same
	 * in child as for the parent.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t11, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	DBUG_ENTER("ch_t11");

	DBUG_PRINTF("ch_t11", "calling exec_suproc");

	exec_suproc(SYSERROR);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	DBUG_ENTER("test12");

	/* If POSIX_SAVED_IDS not defined, untested */
	if(saveids == SYSERROR) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}

	/*
	 * Test that saved set-user ID and saved set-group ID are same 
	 * in child as for the parent.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t12, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	DBUG_ENTER("ch_t12");

	DBUG_PRINTF("ch_t12", "calling exec_suproc");

	exec_suproc(saveids);

	DBUG_VOID_RETURN;
}

private void
exec_suproc(save_ids)
int	save_ids;
{
	char	*savestr;

	DBUG_ENTER("exec_suproc");

	if ((savestr = newstr(itoa(save_ids))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}

	/* Can't use execle() - null vg_dbug stops environ being passed */

	args[0] = prog_su;
	args[1] = savestr;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = (char *) NULL;

	(void) tet_exec(prog_su, args, environ);

	vsxerror("exec(%s,..) failed errno %d", prog_su, errno);
	DBUG_EXIT(A_EXECFAIL);

	DBUG_VOID_RETURN;
}

private void
test13()
{
	int	err;
	int	pa_ngrps;
	int	ch_ngrps;
	gid_t	*pa_grps;
	gid_t	*ch_grps;
	int	max_groups;
	int	pathok = 0;

	DBUG_ENTER("test13");

	/*
	 * Test that supplementary groups for child are the same as for parent
	 */

	test_fail = 0;
	test_deleted = 0;

	if((max_groups = sysconf(_SC_NGROUPS_MAX)) == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sysconf(_SC_NGROUPS_MAX) failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Allocate arrays for supplementary groups */
	pa_grps  = (gid_t *)calloc((size_t)max_groups, sizeof(gid_t));
	ch_grps = (gid_t *)calloc((size_t)max_groups, sizeof(gid_t));

	if (pa_grps == NULL || ch_grps == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason - calloc() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((pa_ngrps = getgroups(max_groups, pa_grps)) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - getgroups() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		if((ch_ngrps = getgroups(max_groups, ch_grps)) == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - getgroups() failed after ");
			in_rpt("fork, errno %d", err);
			DBUG_EXIT(OK);
		}
		else
			PATH_TRACE;

		if(ch_ngrps != pa_ngrps) {
			xx_rpt(FAILURE);
			in_rpt("number of supplementary groups differs");
			in_rpt("child: %d; parent: %d", ch_ngrps, pa_ngrps);
			DBUG_EXIT(OK);
		}
		else
			PATH_TRACE;

		cmp_grps(pa_ngrps, pa_grps, ch_grps);

		if(!test_fail)
			PATH_XS_RPT(5);

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}
	tet_setblock();

	DBUG_VOID_RETURN;
}

private void
cmp_grps(num, pagrps, chgrps)
int	num;
gid_t	*pagrps;
gid_t	*chgrps;
{
	gid_t	*p_grptr;
	gid_t	*c_grptr;
	int	i;
	int	j;
	int	gotgroup;

	/* DBUG_PRINTF statements makes path tracing unneeded */

	/* Check each parent group is in the child's group list. */
	p_grptr = pagrps;
	for(i = 0; i < num; i++) {
		DBUG_PRINTF("cmp_grps", "parent group %d", (int)*p_grptr);
		gotgroup = 0;
		c_grptr = chgrps;
		for(j = 0; j < num; j++) {
			if(*c_grptr++ == *p_grptr)
				gotgroup++;
		}
		if(!gotgroup) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("child does not have supplementary group %d",
								(int)*p_grptr);
			test_fail++;
		}
		p_grptr++;
	}

	/* Check each child group is in the parent's group list. */
	c_grptr = chgrps;
	for(i = 0; i < num; i++) {
		DBUG_PRINTF("cmp_grps", "child group %d", (int)*c_grptr);
		gotgroup = 0;
		p_grptr = pagrps;
		for(j = 0; j < num; j++) {
			if(*p_grptr++ == *c_grptr)
				gotgroup++;
		}
		if(!gotgroup) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("child has extra supplementary group %d",
								(int)*c_grptr);
			test_fail++;
		}
		c_grptr++;
	}
}

private void
test14()
{
	pid_t	ppgrp;
	pid_t	cpgrp;
	int	pathok = 0;

	DBUG_ENTER("test14");

	/*
	 * Test that _child's process group ID is same as parent's.
	 */

	test_fail = 0;
	test_deleted = 0;

	ppgrp = getpgrp();

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		if((cpgrp = getpgrp()) != ppgrp) {
			xx_rpt(FAILURE);
			in_rpt("child's process group ID is different from parent's");
			in_rpt("PGID in child: %ld; PGID in parent: %ld",
				(long)cpgrp, (long)ppgrp);
		} else
		{
			PATH_TRACE;
			xs_rpt();
		}

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}
	tet_setblock();

	DBUG_VOID_RETURN;
}

private void
test15()
{
	DBUG_ENTER("test15");

	/*
	 * Test that child's controlling terminal is same as parent's.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t15, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t15()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("ch_t15");

	/* Acquire controlling terminal */

	switch (termios_prep(&tty_fildes, tty_termios_p,
			&loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE;
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_VOID_RETURN;
	default:
		xx_rpt(DELETION);
		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY");
		DBUG_VOID_RETURN;
	}

	/* Set signal action for terminal interrupt signal to be caught */
	if(sig_init(sig_array[VSIGINT], &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		if(sig_init(sig_array[VSIGINT], &sig_empty, SIG_DFL, 0) < 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigaction() failed");
			DBUG_EXIT(A_MISC);
		}

		/* Sleep during which terminal interrupt should arrive */
		(void) sleep(3*WAITTIME/2);

		/* Should not get here */
		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */
		tet_setblock();
		PATH_TRACE;
		break;
	}
	tet_setblock();

	/* In the parent */

	/* Send the interrupt signal by writing to the controlling terminal */
	interrupt();

	/* Wait for child which should have been terminated by SIGINT */
	errno = NOERROR;
	if((stat_child = wait_child(2*WAITTIME)) == CHLD_ERR) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_MISC)
		test_fail++;	/* initialisation failure already reported */
	else
		PATH_TRACE;

	if(test_fail)
		DBUG_VOID_RETURN;

	if(stat_child != CHLD_ABRT || sg_code != SIGINT) {
		xx_rpt(FAILURE);
		in_rpt("child has different controlling terminal from parent");
		in_rpt("expected action: signal %d terminated process", SIGINT);
		in_rpt("observed action: %s", status_child(stat_child));
	} else
	{
		PATH_TRACE;
		PATH_XS_RPT(6);
	}

	DBUG_VOID_RETURN;
}

private void
test16()
{
	DBUG_ENTER("test16");

	/*
	 * Test that child's process creation mask is same as parent's.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t16, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t16()
{
	mode_t	cmask;
	mode_t	pmask;
	int	pathok = 0;

	DBUG_ENTER("ch_t16");

	pmask = umask((mode_t)UMASK);
	if((pmask = umask((mode_t)UMASK)) != (mode_t)UMASK) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - umask() did not return old mask");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		if((cmask = umask((mode_t)UMASK)) != (mode_t)UMASK) {
			xx_rpt(FAILURE);
			in_rpt("child's file mode creation mask is different from parent's");
			in_rpt("mask in child: 0%o; mask in parent: 0%o", 
								cmask, pmask);
		} else
		{
			PATH_TRACE;
			PATH_XS_RPT(2);
		}

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}
	tet_setblock();

	DBUG_VOID_RETURN;
}

private void
test17()
{
	DBUG_ENTER("test17");

	/*
	 * Test that child's process signal mask is same as parent's.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t17, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t17()
{
	int		ret;
	sigset_t	ch_set;
	sigset_t	pa_set;
	int		pathok = 0;

	DBUG_ENTER("ch_t17");

	(void) sigemptyset(&pa_set);
	(void) sigaddset(&pa_set, SIGTERM);
	(void) sigaddset(&pa_set, SIGINT);
	(void) sigaddset(&pa_set, SIGTTIN);
	(void) sigaddset(&pa_set, SIGTTOU);
	ret = sigprocmask(SIG_SETMASK, &pa_set, (sigset_t *)NULL);
	if(ret != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returns incorrect value %d", ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	ret = sigprocmask(SIG_SETMASK, (sigset_t *)NULL, &pa_set);
	if(ret != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returns incorrect value %d", ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		ret = sigprocmask(SIG_SETMASK, (sigset_t *)NULL, &ch_set);
		if(ret != 0) {
			xx_rpt(DELETION);
			in_rpt("sigprocmask() returns incorrect value %d", ret);
			DBUG_EXIT(OK);
		}
		else
			PATH_TRACE;

		globok = 0;
		cmp_mask(&pa_set, &ch_set);
		PATH_FUNC_TRACE;

		if(test_fail)
			DBUG_EXIT(OK);

		PATH_XS_RPT(4);

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}

	tet_setblock();
	DBUG_VOID_RETURN;
}

private void
cmp_mask(aset, acmpset)
sigset_t	*aset;		/* parent signal mask */
sigset_t	*acmpset;	/* child signal mask  */
{
	int	i;
	int	signum;
	char	*name;
	int	pathok = 0;

	DBUG_ENTER("cmp_mask");

	for(i = 1; i < nsigs; i++) {
		if (i==1)
			PATH_TRACE;
		signum = sig_array[i];
		name = sig_name[i];
		if(sigismember(acmpset, signum)) {
			if(!sigismember(aset, signum)) {
				if(!test_fail)
					xx_rpt(FAILURE);
				in_rpt("child signal mask includes extra signal %d %s",
								signum, name);
				test_fail++;
			}
		} else {
			if(sigismember(aset, signum)) {
				if(!test_fail)
					xx_rpt(FAILURE);
				in_rpt("child signal mask omits signal %d %s",
								signum, name);
				test_fail++;
			}
		}
	}

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test18()
{
	int	pathok = 0;

	DBUG_ENTER("test18");

	/* If POSIX_JOB_CONTROL not defined, untested */
	/* Note this really is UNTESTED not UNSUPPORTED */
	if(!job_control) {
		xx_rpt(UNTESTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test that child's process session is same as parent's.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		ch_t18();
		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		globok = 0;
		pa_t18();
		PATH_FUNC_TRACE;
		break;
	}
	tet_setblock();

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t18()
{
	char	*suid;

	DBUG_ENTER("ch_t18");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	
	/* Can't use execle() - null vg_dbug stops environ being passed */

	args[0] = prog_18;
	args[1] = suid;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = (char *) NULL;

	(void) tet_exec(prog_18, args, environ);

	vsxerror("exec(%s,..) failed errno %d", prog_18, errno);
	DBUG_EXIT(A_EXECFAIL);

	DBUG_VOID_RETURN;
}

private void
pa_t18()
{
	int	stat_child;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t18");

	/* Wait for child and expect it to be stopped */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if(stat_child != CHLD_STOP) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - child process not stopped");
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to send the SIGKILL signal - expect error */
	errno = NOERROR;
	ret = kill(tet_child, SIGKILL);
	err = errno;

	if(ret != SYSERROR || err != EPERM) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill(pid, SIGKILL) returned un-expected value");
		valrpt(SYSERROR, ret, EPERM, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to send the SIGCONT signal - expect success */
	errno = NOERROR;
	ret = kill(tet_child, SIGCONT);
	err = errno;

	if(ret == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("kill(pid, SIGCONT) failed");
		valrpt(ZERORET, ret, NOERROR, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if(stat_child != CHLD_EXIT || ex_code != SIGCONT) {
		xx_rpt(FAILURE);
		in_rpt("kill(pid, SIGCONT) did not cause delivery of SIGCONT signal");
		in_rpt("expected action: process exitted - exit code %d", SIGCONT);
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test19()
{
	int	pa_pid[2];
	int	ch_pid[2];
	int	ret, err;
	pid_t	pid;
	int	pathok = 0;

	DBUG_ENTER("test19");

	/*
	 * Test that parent and child can execute independently after fork().
	 * Use two-way communication via pipes.
	 */

	test_fail = 0;
	test_deleted = 0;

	if(pipe(pa_pid) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(pipe(ch_pid) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fflush(stdout);
	(void) fflush(stderr);

	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case (pid_t)0:		/* child */

		tet_setcontext();
		setsigs(SIG_DFL);

		ch_t19(ch_pid, pa_pid);

		DBUG_EXIT(OK);
		/* NOTREACHED */
		break;

	default:	/* parent */
		tet_setblock();

#ifdef TESTBED2
(void) sleep(2*WAITTIME);
#endif
		/* write parent process ID to pipe */
		pid = getpid();

		if((ret = write(pa_pid[WRITE], (char *)&pid,
				sizeof(pid_t))) != sizeof(pid_t))
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("write() returned %d errno %d", ret, err);
		}
		else
			PATH_TRACE;

		/* read child process ID from pipe */
		wait_read(CHILD, ch_pid);

		wait_for_child(2*WAITTIME);

		if(test_deleted || test_fail)
			DBUG_VOID_RETURN;

		PATH_XS_RPT(3);

		break;
	}
	tet_setblock();

	(void) close(ch_pid[READ]);
	(void) close(ch_pid[WRITE]);
	(void) close(pa_pid[READ]);
	(void) close(pa_pid[WRITE]);

	DBUG_VOID_RETURN;
}

private void
ch_t19(ch_pid, pa_pid)
int	ch_pid[2];
int	pa_pid[2];
{
	pid_t	pid;
	int	err;
	int	ret;

	DBUG_ENTER("ch_t19");

	/* read parent process ID from pipe */
	wait_read(PARENT, pa_pid);

	if(test_deleted || test_fail)
		DBUG_VOID_RETURN;

	/* write our process ID to the pipe */
	pid = getpid();

	if((ret = write(ch_pid[WRITE], (char *)&pid,
			sizeof(pid_t))) != sizeof(pid_t))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("write() returned %d errno %d", ret, err);
	}

	DBUG_VOID_RETURN;
}

private void
wait_read(proc, pip)
char	*proc;
int	pip[2];
{
	pid_t	pid;
	int	ret;
	int	err;

	SET_TIMEOUT(WAITTIME)
	ret = read(pip[READ], (char *)&pid, sizeof(pid_t));
	CLEAR_ALARM
	if(ret == SYSERROR) {
		err = errno;
		if(err == EINTR && alrm_flag) {
			if(!test_fail && !test_deleted)
				xx_rpt(FAILURE);
			in_rpt("%s process did not continue executing after fork()", proc);
			test_fail++;
		} else {
			if(!test_fail && !test_deleted)
				xx_rpt(DELETION);
			in_rpt("deletion reason - read() returned wrong value");
			valrpt(sizeof(pid_t), ret, NOERROR, err);
			test_deleted++;
		}
	} else {
		if(ret != sizeof(pid_t)) {
			if(!test_fail && !test_deleted)
				xx_rpt(DELETION);
			in_rpt("deletion reason - read() returned wrong value");
			valrpt(sizeof(pid_t), ret, NOERROR, err);
			test_deleted++;
		}
	}
}

private void
test20()
{
	int	p_pid[2];
	pid_t	pid;
	int	err;
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("test20");

	/*
	 * Test that on successful completion, (pid_t)0 is returned to the
	 * child process and the child process ID is returned to the parent.
	 */

	test_fail = 0;
	test_deleted = 0;

	if(pipe(p_pid) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case (pid_t)0:		/* child */

		tet_setcontext();
		pid = getpid();

		/* write our process ID to the pipe */
		if((ret = write(p_pid[WRITE], (char *)&pid, sizeof(pid_t))) ==
					SYSERROR || ret != sizeof(pid_t)) {
			err = errno;
			vsxerror("write() returned %d errno %d", ret, err);
			DBUG_EXIT(A_WRITEFAIL);
		}

		(void) sleep(WAITTIME/2);

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		/* read child process ID from the pipe */
		SET_TIMEOUT(WAITTIME)
		ret = read(p_pid[READ], (char *)&pid, sizeof(pid_t));
		CLEAR_ALARM
		if(ret == SYSERROR || ret != sizeof(pid_t)) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - read() returned %d errno %d",
								ret, err);
			test_deleted++;
		}
		else
			PATH_TRACE;

		wait_for_child(WAITTIME);

		if(test_deleted)
			DBUG_VOID_RETURN;

		if(pid != tet_child) {
			xx_rpt(FAILURE);
			in_rpt("fork() returned %ld; child process ID: %ld",
							(long)tet_child, (long)pid);
		} else
		{
			PATH_TRACE;
			PATH_XS_RPT(3);
		}

		break;
	}
	tet_setblock();

	(void) close(p_pid[READ]);
	(void) close(p_pid[WRITE]);

	DBUG_VOID_RETURN;
}

private void
test21()
{
	int	err;

	DBUG_ENTER("test21");

	errno = 0;
	chld_max = sysconf(_SC_CHILD_MAX);
	if(chld_max > PCTS_CHILD_MAX || chld_max == -1 && errno == 0) {
		xx_rpt(UNSUPPORTED);
		in_rpt("CHILD_MAX > PCTS_CHILD_MAX or == -1");
		DBUG_VOID_RETURN;
	}
	if(chld_max < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sysconf(_SC_CHILD_MAX) failed errno %d", err);
		DBUG_VOID_RETURN;
	}

	/*
	 * Test that (pid_t) -1 is returned by fork() when total number 
	 * of processes available to user is exceeded.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t21, NULLFN, 100*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t21()
{
	char	*uidstr;

	DBUG_ENTER("ch_t21");

	if ((uidstr = newstr(itoa(vguid2))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}

	/* Can't use execle() - null vg_dbug stops environ being passed */

	args[0] = prog_21;
	args[1] = uidstr;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = (char *) NULL;

	(void) tet_exec(prog_21, args, environ);

	vsxerror("exec(%s,..) failed errno %d", prog_21, errno);
	DBUG_EXIT(A_EXECFAIL);

	DBUG_VOID_RETURN;
}

private int
setcmp(mask1, mask2)
sigset_t *mask1, *mask2;
{
	int sig, is1, is2;

	for (sig = 1; sig < nsigs; sig++)
	{
		is1 = sigismember(mask1, sig_array[sig]);
		is2 = sigismember(mask2, sig_array[sig]);
		if ((is1 && !is2) || (!is1 && is2))
			return 1;
	}

	return 0;
}
