/*
*      SCCS:  @(#)  POSIX.os/procprim/fork/fork_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/fork - fork_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		fork_su - used by T.fork in several tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.fork to test the delivery of signals. 
	  This file exec's another to either send or receive a signal 
	  using real/effective user id as specified below.
	  usage:
		fork_su saveids vg_ident vg_dbug

	  where saveids indicates whether POSIX_SAVED_IDS are to be set.
METHODS:
ASSUMES:  fork_su must be setuid root.
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, Unisoft Ltd.
DATE CREATED:	Thu Apr  6 15:30:09 GMT 1989
MODIFICATIONS:

		David Sawyer
		Mon Aug 13 15:33:58 BST 1990
		Modified argument checking for main and changed execle()
		call to tet_exec().
		Added resilience code.

		Geoff Clare, 15 Oct 1990
			Set vg_env to environ.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Peter Skan, 12 Sept 1991
			Set/verify CHOWN privilege
			Correct comments on setuid and setgid calls

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<setprv.h>
#include	<exitcodes.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fork
#endif


#define	SU_MODE		(S_ISUID|S_ISGID|S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)
#define	SU_UID		0

extern	char	**environ;

extern	void	set_pname();

extern	int	vguid0;
extern	int	vguid1;
extern	int	vggid0;
extern	int	vggid1;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private char	*prog_uids = "../fork_uids";
private char	*args[5];

private	void	subtest();

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{

	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"fork_su" : (argv[0]));
	
	switch(argc)
	{
	case 4:
		vg_dbug = argv[3];
		/* fall through */
	case 3:
		vg_ident = argv[2];
		break;
	default:
		vsxerror("argument count wrong");
		exit(A_MISC);
	}
	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if (setprv(PRV_CHOWN) != 0) {
		vsxerror("setprv(PRV_CHOWN) failed");
		exit(A_TSETDIR);
	}


	if(uids()!=0)
	{
		vsxerror("uids() failed");
		exit(A_UIDFAIL);
	}

	subtest(argv[1]);
	exit(OK);

	/* NOTREACHED */

	return(0);
}

private void
subtest(savestr)
char	*savestr;
{
	char	*prog;
	int	rgid = vggid0;
	int	egid = vggid1;
	int	ruid = vguid0;
	int	euid = vguid1;
	int	pathok = 0;

	DBUG_ENTER("subtest");

	prog = prog_uids;

	/* Make the owner and mode correct for the file exec'ed */
	if(chown(prog, (uid_t)euid, (gid_t)egid) < 0) {
		vsxerror("external error %d errno %d", A_TSETDIR, errno);
		DBUG_EXIT(A_TSETDIR);
	}
	if(chmod(prog, (mode_t)SU_MODE) < 0) {
		vsxerror("external error %d errno %d", A_CHMODFAIL, errno);
		DBUG_EXIT(A_CHMODFAIL);
	}

	/* Make the effective user id correct by calling setuid() */
	if(setuid((uid_t)ruid) < 0) {
		vsxerror("external error %d errno %d", A_UIDFAIL, errno);
		DBUG_EXIT(A_UIDFAIL);
	}
	/* Make the effective group id correct by calling setgid() */
	if(setgid((gid_t)rgid) < 0) {
		vsxerror("external error %d errno %d", A_UIDFAIL, errno);
		DBUG_EXIT(A_UIDFAIL);
	}

	PATH_TRACE; /* this is just so that we know we've been here */

	/* Can't use execle() - null vg_dbug stops environ being passed */

	args[0] = prog;
	args[1] = savestr;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = (char *) NULL;

	(void) tet_exec(prog, args, environ);

	vsxerror("external error %d errno %d", A_EXECFAIL, errno);
	DBUG_EXIT(A_EXECFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}
