/*
*      SCCS:  @(#)  POSIX.os/procprim/fork/fork_t21.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/fork - fork_t21 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		fork_t21 - used by T.fork in test21()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.fork to user and group ID'S after fork().
	  usage:
		fork_t21 uid vg_ident vg_dbug

	  where uid indicates real user ID for test.

METHODS:
ASSUMES:  fork_t21 must be setuid root.
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, Unisoft Ltd.
DATE CREATED:	Tue Apr 11 09:15:47 BST 1989
MODIFICATIONS:
		Geoff Clare, 17 July 1989
			Use setsid() instead of setpgid() to set new
			process group, as setpgid() may not be supported.
		
		Geoff Clare, 31 May 1990
			Allow for unlimited CHILD_MAX

		David Sawyer
		Mon Aug 13 15:36:26 BST 1990
		Modified argument checking for main().
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.
			Eliminated tet_child declaration.
			Eliminated vg_file

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<pcts_max.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<sys/wait.h>
#include	<errno.h>
#include	<vsx_signal.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fork
#endif


#define	WAITTIME		(5 * SPEEDFACTOR)

extern	char	**environ;

extern	void	set_pname();
extern	void	valrpt();
extern	char	*errname();

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	sigset_t	sig_empty;
private	int		chld_max;

private	void	subtest21();
private	int	sig_init();

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	int	err;

	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"fork_t21" : (argv[0]));

	switch(argc)
	{
	case 4:
		vg_dbug = argv[3];
		/* fall through */
	case 3:
		vg_ident = argv[2];
		break;
	default:
		vsxerror("argument count wrong");
		exit(A_MISC);
	}

	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	errno = 0;
	chld_max = sysconf(_SC_CHILD_MAX);
	if(chld_max > PCTS_CHILD_MAX || chld_max == -1 && errno == 0) {
		xx_rpt(UNSUPPORTED);
		in_rpt("CHILD_MAX > PCTS_CHILD_MAX or == -1");
		exit(OK);
	}
	if(chld_max < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("sysconf(_SC_CHILD_MAX) failed errno %d", err);
		exit(OK);
	}

	if(setprv(PRV_SETID) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: setprv(PRV_SETID) failed");
		exit(OK);
	}

	subtest21(atoi(argv[1]));

	exit(OK);

	/* NOTREACHED */

	return(0);
}

private void
subtest21(uid)
int	uid;
{
	int	count;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("subtest21");

	/* Make ourself into a new user */
	if(setuid((uid_t)uid) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - setuid() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Set process group to make killing the child processes easier */
	if(setsid() < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - setsid() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sig_init(SIGUSR1, &sig_empty, SIG_IGN, 0);

	(void) fflush(stdout);
	(void) fflush(stderr);

	errno = NOERROR;

	/*
	 * Try to fork() CHILD_MAX+1 processes. 
	 * Failure at CHILD_MAX or CHILD_MAX+1 is allowed, as it
	 * is unclear whether the parent which has performed the
	 * setuid() call is counted.
	 */
	for(count = 1; count <= chld_max+1; count++) {
		if (count==1)
			PATH_TRACE;
		DBUG_PRINTF("subtest21", "count is %d", count);
		switch(tet_child = fork()) {

		case SYSERROR:

			/* Break out of switch and check error values */
			break;

		case 0:		/* child */

			tet_setcontext();
			(void) sig_init(SIGUSR1, &sig_empty, SIG_DFL, 0);

			/* Sleep until processes are killed off */
			(void) sleep(100*WAITTIME);

			DBUG_EXIT(OK);

			/* NOTREACHED */
			break;

		default:	/* parent */

			tet_setblock();
			/* Go on to make another child */
			continue;

			/* NOTREACHED */
			break;
		}

		tet_setblock();
		/* Got the SYSERROR case in the above switch */
		break;
	}
	err = errno;

	/* Sleep to let child processes all initialise themselves */
	(void) sleep(WAITTIME/2);

	/* Kill off the child processes */
	(void) kill((pid_t)0, SIGUSR1);
	while(wait((int *)NULL) != SYSERROR) 
		;

	if(count > chld_max+1) {
		xx_rpt(FAILURE);
		in_rpt("fork() created more than CHILD_MAX+1 (%d) processes for user ID %d",
							 chld_max+1, uid);
		in_rpt("fork() created %d processes", count);
		in_rpt("(incuding the parent which did a setuid(%d) call)",
			uid);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(count < chld_max) {
		xx_rpt(FAILURE);
		in_rpt("fork() did not allow creation of CHILD_MAX (%d) processes for user ID %d", 
							chld_max, uid);
		in_rpt("fork() returned error when %d processes created", count);
		in_rpt("fork() returned %ld and set errno to %d", (long)tet_child, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(tet_child != SYSERROR || err != EAGAIN) {
		xx_rpt(FAILURE);
		in_rpt("fork() did not return expected value when CHILD_MAX+1 (%d) processes created", chld_max+1);
		if (tet_child != SYSERROR)
			in_rpt("expected return value %d, observed %ld",
				SYSERROR, (long)tet_child);
		else if (err != EAGAIN)
			in_rpt("expected errno value %d (EAGAIN), observed %ld (%s)",
				EAGAIN, err, errname(err));
	} else 
	{
		PATH_TRACE;
		PATH_XS_RPT(6);
	}

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}
