/*
*      SCCS:  @(#)POSIX.os/procprim/fork_X/fork_X.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/fork_X - T.fork_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.fork_X

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the fork() system call, X/OPEN extensions.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Mon Apr  3 10:13:44 GMT 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG
		
		Geoff Clare, 16 Nov 1989
			catval was declared incorrectly

		David Sawyer
		Mon Aug 13 15:59:40 BST 1990
		Added a rmdir() call to cln_tests().
		Added resilience code.

		Stuart Boutell, 7 Nov 1990
			tet_child redeclaration removed.

		Geoff Clare, 16 Nov 1989
			Use NL_CAT_LOCALE in catopen call.

                John-Paul, 29 July 1992
                        Mode parameter in call to crdir() changed from
			'int' to 'mode_t' type.
		
************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<nl_types.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<sys/wait.h>
#include	<wait_child.h>
#include	<sys/times.h>
#include	<signal.h>
#include	<string.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef fork
#endif


#if !(TEST_XPG > 3)
#undef NL_CAT_LOCALE
#define NL_CAT_LOCALE 0
#endif

#define	NO_TESTS		1	/* number of tests in testset */
#define	WAITTIME		(5 * SPEEDFACTOR)
#define	NUMCATDS		1

extern	void		vsx_sigs();
extern	char		*status_child();

extern	int		sig_array[];
extern	int		ex_code;	/* defined in wait_child.c */

int	no_tests = NO_TESTS;

private int	setupok;
private char	*defmsg = "descriptor returned is invalid";
private char	*msgfile = "./msgfile_t";
private	char	*catname = "dir1/test.c_t";
private char	*identmsg[NUMCATDS] = {
				"dir1/test.c_t",
			};

private	void		test1();
private	void		wait_for_child();
private	void		prepare_tests();
private	void		prep_cats();
private	void		cln_tests();
private	int		sig_init();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private	void
prepare_tests()
{
	int	i;	/* Counter */
	int	ret;	/* Used for most return values */
	int	nsigs;
	sigset_t sig_empty;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		nsigs = VNOJSIG;
	} else {
		nsigs = VNUMSIG;
	}

	/* Set up message catalogues */
	prep_cats();

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if (i==1)
			PATH_TRACE;
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);

	if (pathok == 2)
		setupok = 0;
	else
		setupok = -1;

	DBUG_VOID_RETURN;
}

private void
prep_cats()
{
	int	i;
	int	prepfail = 0;
	char	cmdstr[60];
	char	msgstr[50];
	FILE	*msg_fd;

	DBUG_ENTER("prep_cats");

	if (crdir("./dir1", (mode_t)0777, (int) geteuid(), (int) getegid()) == SYSERROR)
	{
		for (i=1; i<=NO_TESTS; i++) 
		{
			(void) tet_delete(i, "Could not create directory dir1");
		}
		DBUG_VOID_RETURN;
	}

	for (i=0; i<NUMCATDS; i++)
	{
		if ((msg_fd = fopen(msgfile, "w")) == (FILE *) NULL)
			prepfail = 4;

		if (prepfail)
			break;

		(void) strcpy(msgstr, "$set 1\n1 ");
		(void) strcat(msgstr, identmsg[i]);
		(void) strcat(msgstr, "\n");
		if (fputs(msgstr, msg_fd) == EOF)
			prepfail = 5;

		(void) fclose(msg_fd);
		(void) unlink(identmsg[i]);

		if (prepfail)
			break;

		(void) strcpy(cmdstr, "gencat ");
		(void) strcat(cmdstr, identmsg[i]);
		(void) strcat(cmdstr, " ");
		(void) strcat(cmdstr, msgfile);

		if (system(cmdstr) != 0)
			prepfail = 6;
	}

	if (prepfail)
	{
		for (i=1; i<=NO_TESTS; i++)
		{
			switch(prepfail)
			{
			case 4:
				(void) tet_delete(i, "Could not create message text file");
				break;
			case 5:
				(void) tet_delete(i, "Could not write messages to message text file");
				break;
			case 6:
				(void) tet_delete(i, "gencat command returned non-zero exit status");
				break;
			}
		}
	}


	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	int	i;

	DBUG_ENTER("cln_tests");

	for (i=0; i<NUMCATDS; i++)
	{
		(void) unlink(identmsg[i]);
	}

	(void) unlink(msgfile);
	(void) rmdir("./dir1");

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
wait_for_child(waittime)
int	waittime;
{
	int	err;
	int	stat_child;

	DBUG_ENTER("wait_for_child");

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(waittime)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		DBUG_VOID_RETURN;
	}

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		if(stat_child == CHLD_LIVE ||
		   stat_child == CHLD_STOP)
			(void) killchild(tet_child, WAITTIME);
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	err;
	nl_catd	catval;
	char	*s;
	int	pathok = setupok;

	DBUG_ENTER("test1");

	/*
	 * Test that child has it's own copy of message catalogues.
	 */

	if((catval = catopen(catname, NL_CAT_LOCALE)) == (nl_catd)SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - catopen() failed - errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	s = catgets(catval, 1, 1, defmsg);
	if(strcmp(s, identmsg[0])) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - catgets() returned \"%s\"", s);
		in_rpt("instead of \"%s\"", identmsg[0]);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		s = catgets(catval, 1, 1, defmsg);
		if(strcmp(s, identmsg[0])) {
			xx_rpt(FAILURE);
			in_rpt("catgets() returned \"%s\" after fork()", s);
			in_rpt("instead of \"%s\"", identmsg[0]);
		} else
		{
			PATH_TRACE;
			PATH_XS_RPT(3);
		}

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		wait_for_child(WAITTIME);
		break;
	}

	tet_setblock();
	DBUG_VOID_RETURN;
}
