/*
*      SCCS:  @(#)POSIX.os/procprim/kill/kill.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/kill - T.kill Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.kill

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the kill() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Wed Mar 29 13:29:06 GMT 1989
MODIFICATIONS:
		Geoff Clare, 17 July 1989
			If setpgid() is not supported, use setsid() to
			create background process group.
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG
		
		Geoff Clare, 8 Nov 1989
			Make sure stop signals are not tested on orphaned
			process group.

		Keith Holder, 29 June 1990
			Modified test 2 to run the tests even if
			job control is not supported.
			Added two new tests (16 & 17) .

		David Sawyer
		Tue Aug 14 11:13:14 BST 1990
		Replaced execle() call with tet_exec().
		Added resilience code.

		Geoff Clare, 12 Oct 1990
			Guard against NULL vg_dbug.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.
			Changed vg_sjmp to internal handler

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called

		Stuart Boutell, 30 Apr 1991
			Changed cpsig() return code check from FAIL to
		SYSERROR

		Stuart Boutell, 14 May 1992
			Add test to ensure that ESRCH isn't given on
		a kill to a process group where the last process
		hasn't been waited for.

		Geoff Clare, 20 July 1992
			Change exec_suproc arg and some variables from
			int to pid_t.

		Geoff Clare, 14 Sept 1992
			Allow errors other than ESRCH in "ESRCH isn't
			given" test.

		Geoff Clare, 23 June 1994
			Ensure sigsetjmp() is used in line with constraints.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<sys/wait.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<setjmp.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef kill
#endif


#define	WAITTIME		(5 * SPEEDFACTOR)
#define	EXTERR			"external error"
#define	NCHILDREN		5
#define	NO_TESTS		18		/* number of tests in testset */
#define	NEGATIVE		-1
#define	ZERO			0
#define	SU_UID			0
#define	ZERORET			0

extern	void		vsx_sigs();
extern	void		valrpt();
extern	char		*itoa();
extern	char		*ltoa();
extern	char		*newstr();
extern	char		*status_child();
extern	char		*errname();
extern	void		setsigs();

extern	char		*vg_ident;
extern	char		*vg_dbug;
extern	char	**environ;
extern	int		vguid0;
extern	int		vguid1;
extern	int		vguid2;
extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sg_code;	/* defined in wait_child.c */
extern	int		ex_code;	/* defined in wait_child.c */

int	no_tests = NO_TESTS;

public  char		*prog_su  = "../kill_su";
private char		*prog_12 = "../kill_t12";
private	void		(*sig_arg)();
private	int		sig_index;
private	int		sig_err;
private	sigset_t	sig_empty;
private	int		nsigs;
private	int		saveids;
private	int		test_case;
private	int		test_fail;
private	int		test_deleted;
private	int		sig_recvd;
private	int		job_control;
private	int		setupok;
private	int		jmpsignum;
private	sigjmp_buf	sjmp;

private	void		signal_init();
private	void		test1();
private	void		test2();
private	void		test3();
private	void		do_sigs();
private	void		ch_tsigs();
private	void		pa_tsigs();
private	void		test4();
private	void		ch_t4();
private	void		ch_t4recv();
private	void		ch_t4send();
private	void		test5();
private	void		ch_t5();
private	void		ch_t5recv();
private	void		ch_t5send();
private	void		test6();
private	void		ch_t6();
private	void		ch_t6recv();
private	void		ch_t6send();
private	void		test7();
private	void		ch_t7();
private	void		ch_t7recv();
private	void		ch_t7send();
private	void		pa_suproc();
private	void		exec_suproc();
private	void		test8();
private	void		ch_t8();
private	void		ch_t8recv();
private	void		ch_t8send();
private	void		test9();
private	void		test10();
private	void		ch_t9and10();
private	void		pa_t9and10();
private	void		test11();
private	void		ch_t11();
private	void		test12();
private	void		ch_t12();
private	void		pa_t12();
private	void		test13();
private	void		ch_t13();
private	void		pa_t13();
private	void		test14();
private	void		ch_t14();
private	void		pa_t14();
private	void		test15();
private	void		ch_t15();
private	void		test16();
private	void		ch_t16();
private	void		test17();
private	void		ch_t17();
private	void		ch_15to17recv();
private	void		ch_15to17send();
private	void		test18();
private	void		ch_t18();
private	void		pa_t18();
private	void		prepare_tests();
private	void		clean_tests();
private	void		sig_catch();
private	int		sig_init();
private	void		sjmpfunc();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	  test2, 2,
	  test3, 3,
	  test4, 4,
	  test5, 5,
	 test6, 6,
	  test7, 7,
	  test8, 8,
	  test9, 9,
	  test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	NULL, 0
};

private	void
prepare_tests()
{
	int	i;	/* Counter */
	int	ret;	/* Used for most return values */
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	/* guard against NULL vg_dbug being taken as arg list terminator */
	if (vg_dbug == NULL)
		vg_dbug = "-D";

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	if(uids()!=0)
	{
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		job_control = 0;
		nsigs = VNOJSIG;
	} else {
		job_control = 1;
		nsigs = VNUMSIG;
	}

	/* check if save set UIDS are in the standard */
	saveids = sysconf(_SC_SAVED_IDS);


	if (pathok == 2)
		setupok = 0;
	else
		setupok = -1;

	DBUG_VOID_RETURN;
}

private void
signal_init()
{
	int i;
	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);
}


private	void
clean_tests()
{
	DBUG_ENTER("clean_tests");

	(void) unlink("core");

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = setupok;

	DBUG_ENTER("test1");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	do_sigs(TYPE_ABRT);
	PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	pathok = setupok;

	DBUG_ENTER("test2");

	(void) signal_init();
	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	do_sigs(TYPE_IGNR);
	PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	do_sigs(TYPE_STOP);
	PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	do_sigs(TYPE_CONT);
	PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int	pathok = setupok;

	DBUG_ENTER("test3");
	(void) signal_init();

	test_fail = 0;
	test_deleted = 0;

	sig_index = VNULLSIG;
	sig_arg = SIG_DFL;

	globok = 0;
	if(cpsig(ch_tsigs, pa_tsigs, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
do_sigs(arg)
int	arg;	/* what default action type to test */
{
	int	i;
	int	sg_num;
	char *	sg_name;
	int	pathok = 0;

	DBUG_ENTER("do_sigs");

	/*
	 * Test action requested for all signals that have default action
	 * type which matches "arg".
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		sg_num = sig_array[i];
		sg_name = sig_name[i];

		/* check default action for this signal */
		if(sig_type[sg_num] != arg)
		{
			PATH_TRACE;
			continue;
		}

		DBUG_PRINTF("do_sigs", "signal %s", sg_name);

		sig_index = i;
		sig_arg = SIG_DFL;

		globok = 0;
		if(cpsig(ch_tsigs, pa_tsigs, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		else
			PATH_FUNC_TRACE;

		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	PATH_FUNC_RPT(nsigs-1);

	DBUG_VOID_RETURN;
}

private void
ch_tsigs()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_tsigs");

	/* If the signal will stop the process, make sure
	   it's not in an orphaned process group */

	if(sig_type[sg_num] == TYPE_STOP)
		(void) setpgid((pid_t)0, (pid_t)0);

	/* Set the required action if we can */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	sig_recvd = 0;

	/* Pause and expect signal to be received */
	(void) pause();

	/* Exit code is signal number we received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_tsigs()
{
	int	ret;
	int	err;
	int	stat_child;
	char *	str_expect;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	sg_type = sig_type[sg_num];
	char	exp_buf[BUFSIZ];
	int	jcs = 0;	/* job control signal */
	int	pathok = 0;

	DBUG_ENTER("pa_tsigs");

	sig_err = 0;

	/* Wait for child and expect it to be alive */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME/2)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - child terminated unexpectedly");
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to send the signal */
	errno = NOERROR;
	ret = kill(tet_child, sg_num);
	err = errno;

	if ( sg_type == TYPE_STOP || sg_type == TYPE_CONT )
		jcs++;

	if(ret != ZERORET) {
		/* Only for test 2.
		 * If it's a job control signal and
		 * we don't support job control, it can
		 * fail with an EINVAL [sic].
		 */
		if ( tet_thistest == 2 && jcs && !job_control  && err == EINVAL ) {
			(void)killchild(tet_child,WAITTIME/5); /* zap child! */
			PATH_FUNC_RPT(2);
			DBUG_VOID_RETURN;
		}

		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("kill(pid, %s) did not return expected value", sg_name);
		valrpt(ZERORET, ret, NOERROR, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Switch on what default action is expected. */
	switch(sg_type) {
	case TYPE_ABRT:	/* child aborts on receiving signal */
		(void) sprintf(exp_buf, "signal %d terminated child", sg_num);
		str_expect = exp_buf;
		if(stat_child != CHLD_ABRT)
			sig_err++;
		if(sg_code != sg_num)
			sig_err++;
		PATH_TRACE;
		break;
	case TYPE_IGNR:	/* child ignores so exits normally */
	case TYPE_CONT:	/* here, child ignores SIGCONT */
		str_expect = "child ignored signal";
		if(stat_child != CHLD_LIVE)
			sig_err++;
		PATH_TRACE;
		break;
	case TYPE_STOP:	/* child process stops */
		str_expect = "signal stopped child";
		if(stat_child != CHLD_STOP)
			sig_err++;
		PATH_TRACE;
		break;
	}

	if(sig_err) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caused wrong action", sg_name);
		in_rpt("expected action: %s", str_expect);
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	(void) cpsig(ch_t4, pa_suproc, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	DBUG_ENTER("ch_t4");

	(void) cpsig(ch_t4recv, ch_t4send, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4recv()
{
	DBUG_ENTER("ch_t4recv");

	exec_suproc(tet_child, vguid0, vguid2, SYSERROR, ZERORET,tet_thistest);

	DBUG_VOID_RETURN;
}

private void
ch_t4send()
{
	DBUG_ENTER("ch_t4send");

	exec_suproc(tet_child, vguid0, vguid1, SYSERROR, ZERORET,tet_thistest);

	DBUG_VOID_RETURN;
}

private void
test5()
{
	DBUG_ENTER("test5");

	(void) cpsig(ch_t5, pa_suproc, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	DBUG_ENTER("ch_t5");

	(void) cpsig(ch_t5recv, ch_t5send, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5recv()
{
	DBUG_ENTER("ch_t5recv");

	exec_suproc(tet_child, vguid1, vguid2, SYSERROR, ZERORET,tet_thistest);

	DBUG_VOID_RETURN;
}

private void
ch_t5send()
{
	DBUG_ENTER("ch_t5send");

	exec_suproc(tet_child, vguid0, vguid1, SYSERROR, ZERORET,tet_thistest);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	DBUG_ENTER("test6");

	(void) cpsig(ch_t6, pa_suproc, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	DBUG_ENTER("ch_t6");

	(void) cpsig(ch_t6recv, ch_t6send, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6recv()
{
	DBUG_ENTER("ch_t6recv");

	exec_suproc(tet_child, vguid2, vguid0, saveids, ZERORET,tet_thistest);

	DBUG_VOID_RETURN;
}

private void
ch_t6send()
{
	DBUG_ENTER("ch_t6send");

	exec_suproc(tet_child, vguid0, vguid1, saveids, ZERORET,tet_thistest);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	DBUG_ENTER("test7");

	(void) cpsig(ch_t7, pa_suproc, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	DBUG_ENTER("ch_t7");

	(void) cpsig(ch_t7recv, ch_t7send, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t7recv()
{
	DBUG_ENTER("ch_t7recv");

	exec_suproc(tet_child, vguid2, vguid1, saveids, ZERORET,tet_thistest);

	DBUG_VOID_RETURN;
}

private void
ch_t7send()
{
	DBUG_ENTER("ch_t7send");

	exec_suproc(tet_child, vguid0, vguid1, saveids, ZERORET,tet_thistest);

	DBUG_VOID_RETURN;
}

private void
pa_suproc()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_suproc");

	/* Wait for the sending child */
	errno = NOERROR;
	if((stat_child = wait_child(3*WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected child status");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);	/* cpsig expects globok to be set */

	DBUG_VOID_RETURN;
}

private void
exec_suproc(ch_pid, real_uid, eff_uid, save_ids, result,testno)
pid_t	ch_pid;
int	real_uid;
int	eff_uid;
int	save_ids;
int	result;
int	testno;
{
	char	*chldstr;
	char	*ruid;
	char	*euid;
	char	*savestr;
	char	*resultstr;
	char	*testnostr;
	char	*args[10];

	DBUG_ENTER("exec_suproc");

	if ((chldstr = newstr(ltoa((long)ch_pid))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((ruid = newstr(itoa(real_uid))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((euid = newstr(itoa(eff_uid))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((savestr = newstr(itoa(save_ids))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	if ((resultstr = newstr(itoa(result))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}

	if ((testnostr = newstr(itoa(testno))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}

	DBUG_PRINTF("exec_suproc", "ch_pid is %d", ch_pid);

	/* Can't use execle() - null vg_dbug stops environ being passed */

	args[0] = prog_su;
	args[1] = chldstr;
	args[2] = ruid;
	args[3] = euid;
	args[4] = savestr;
	args[5] = resultstr;
	args[6] = testnostr;
	args[7] = vg_ident;
	args[8] = vg_dbug;
	args[9] = (char *) NULL;

	(void) tet_exec(prog_su, args, environ);

	vsxerror("tet_exec(%s,..) failed errno %d", prog_su, errno);
	DBUG_EXIT(A_EXECFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
test8()
{
	DBUG_ENTER("test8");

	(void) cpsig(ch_t8, pa_suproc, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	DBUG_ENTER("ch_t8");

	(void) cpsig(ch_t8recv, ch_t8send, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8recv()
{
	DBUG_ENTER("ch_t8recv");

	exec_suproc(tet_child, vguid0, vguid2, SYSERROR, ZERORET,tet_thistest);

	DBUG_VOID_RETURN;
}

private void
ch_t8send()
{
	DBUG_ENTER("ch_t8send");

	exec_suproc(tet_child, SU_UID, SU_UID, SYSERROR, ZERORET,tet_thistest);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	DBUG_ENTER("test9");

	test_case = ZERO;

	(void) cpsig(ch_t9and10, pa_t9and10, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
test10()
{
	DBUG_ENTER("test10");

	test_case = NEGATIVE;

	(void) cpsig(ch_t9and10, pa_t9and10, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t9and10()
{
	char	i, j;
	int	tm_out;
	int	chld_sig;
	int	chld_exit;
	int	chld_stop;
	pid_t	chld_pid;
	pid_t	children[NCHILDREN];
	int	waitchild[NCHILDREN];
	int	rval, err;
	int	status;
	int	pathok = setupok;

	DBUG_ENTER("test9and10");
	errno = 0;
	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		if (errno != ENOSYS)
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("setpgid(0, 0) failed - errno %d (%s)",
				err, errname(err));
			DBUG_EXIT(A_MISC);
		}
		else
			PATH_TRACE;
		(void) setsid();
	}
	else
		PATH_TRACE;

	for (i = 0 ; i < NCHILDREN ; i++)
	{
		(void) fflush(stdout);
		(void) fflush(stderr);
		PATH_TRACE;
		switch (children[i] = fork())
		{
		case SYSERROR:
			vsxerror("external error %d", A_FORKFAIL);
			DBUG_EXIT(A_FORKFAIL);

			/* NOTREACHED */
			break;

		case 0:		/* child */
			tet_setcontext();
			if (DBUG_SETJMP(sjmp) != 0)
			{
				xx_rpt(DELETION);
				in_rpt("deletion reason: signal %d received",
					jmpsignum);
				DBUG_EXIT(A_MISC);
			}
			setsigs(sjmpfunc);
			(void) sig_init(SIGUSR1, &sig_empty, SIG_DFL, 0);
			(void) pause();
			DBUG_EXIT(A_MISC);

			/* NOTREACHED */
			break;

		/* parent does nothing yet */
		}
		tet_setblock();
	}
	/* parent of NCHILDREN */
	tet_setblock();

	/* Wait for children to get paused */
	(void) sleep(WAITTIME);
	for (i = 0 ; i < NCHILDREN ; i++)
	{
		if (i==0)
			PATH_TRACE;
		waitchild[i] = 0;
	}
	sig_recvd = 0;
	(void) sig_init(SIGUSR1, &sig_empty, sig_catch, 0);
	switch(test_case) {
	case NEGATIVE:
		DBUG_PRINTF("ch_t9and10", "kill(-getpid(), ...)");
		(void) kill(-getpid(), sig_array[VSIGUSR1]);
		break;
	case ZERO:
	default:
		DBUG_PRINTF("ch_t9and10", "kill(0, ...)");
		(void) kill((pid_t)0, sig_array[VSIGUSR1]);
		break;
	}
	for (i = 0 ; i < NCHILDREN ; i++)
	{
		if (i==0)
			PATH_TRACE;
		waitchild[i] = 0;
	}
	tm_out = chld_stop = chld_exit = chld_sig = 0;

	/* Wait for all except the first and expect signal to be delivered */
	for (i = 0 ; i < NCHILDREN; i++)
	{
		/* just do one trace per loop to avoid verbose output */
		PATH_TRACE;
		SET_TIMEOUT(WAITTIME);
		chld_pid = wait(&status);
		CLEAR_ALARM;
		if(chld_pid == SYSERROR) {
			if(errno == EINTR && alrm_flag) {
				tm_out++;
			} else {
				vsxerror("external error %d", A_WAITFAIL);
				DBUG_EXIT(A_WAITFAIL);
			}
		} else {
			for(j = 0; j < NCHILDREN; j++)
				if(children[j] == chld_pid)
					break;
			if(j == NCHILDREN || waitchild[j]++) {
				vsxerror("external error %d", A_WRCHILD);
				DBUG_EXIT(A_WRCHILD);
			}
			if(WIFSIGNALED(status)) {
				if(WTERMSIG(status) != SIGUSR1)
					chld_sig++;
			} else if(WIFSTOPPED(status)) {
				chld_stop++;
			} else if(WIFEXITED(status)) {
				chld_exit++;
			}
		}
		DBUG_PRINTF("ch_t9and10", "chld_stop %d chld_exit %d chld_sig %d tm_out %d ", chld_stop, chld_exit, chld_sig, tm_out);
	}

	if (chld_sig || chld_exit || chld_stop || tm_out || sig_recvd != SIGUSR1)
	{
		xx_rpt(FAILURE);
		if (sig_recvd != SIGUSR1)
			in_rpt("parent did not receive signal");
		if (chld_exit)
			in_rpt("child ignored signal and exitted");
		if (chld_stop)
			in_rpt("child stopped by signal");
		if (chld_sig)
			in_rpt("wrong signal killed child");
		if (tm_out)
		{
			in_rpt("signal did not kill child");
			for (i = 0 ; i < NCHILDREN ; i++)
			DBUG_PRINTF("ch_t9and10", "kill_child %ld", (long)children[i]);
				if ((rval = killchild(children[i],
						WAITTIME)) != SUCCEED)
				{
					vsxerror("killchild() returned %d",
						rval);
					DBUG_EXIT(A_KILLFAIL);
				}
		}
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(4+(NCHILDREN*2));
	}

	DBUG_VOID_RETURN;
}

private void
pa_t9and10()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t9and10");

	/* Wait for child */
	errno = NOERROR;
	if(wait_child((NCHILDREN+2)*WAITTIME) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1); /* cpsig expects globok to be set in parent */

	DBUG_VOID_RETURN;
}

private void
test11()
{
	DBUG_ENTER("test11");

	/*
	 * Test that kill() delivers signal before returning.
	 */

	(void) cppair(ch_t11, NULLFN, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	int	err;
	int	ret;
	int	pathok = setupok;

	DBUG_ENTER("ch_t11");

	/* Call sigaction() to initialise signal handling function */
	if(sig_init(SIGUSR1, &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig_recvd = 0;

	ret = kill(getpid(), SIGUSR1);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(sig_recvd != SIGUSR1) {
		xx_rpt(FAILURE);
		in_rpt("kill() returned before un-blocked signal was delivered");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	int	pathok = setupok;

	DBUG_ENTER("test12");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test that kill(..., SIGCONT) sends SIGCONT signal even when 
	 * process has a different user ID.
	 */

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(ch_t12, pa_t12, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	char	*suid;
	char	*args[5];

	DBUG_ENTER("ch_t12");

	if ((suid = newstr(itoa(vguid1))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}
	args[0] = prog_12;
	args[1] = suid;
	args[2] = vg_ident;
	args[3] = vg_dbug;
	args[4] = (char *) NULL;
	(void) tet_exec(prog_12, args, environ );
	vsxerror("tet_exec(%s,..) failed errno %d", prog_12, errno);
	DBUG_EXIT(A_EXECFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t12()
{
	int	stat_child;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t12");

	/* Wait for child and expect it to be stopped */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if(stat_child != CHLD_STOP) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - child process not stopped");
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to send the SIGKILL signal - expect error */
	errno = NOERROR;
	ret = kill(tet_child, SIGKILL);
	err = errno;

	if(ret != SYSERROR || err != EPERM) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill(pid, SIGKILL) returned un-expected value");
		valrpt(SYSERROR, ret, EPERM, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to send the SIGCONT signal - expect success */
	errno = NOERROR;
	ret = kill(tet_child, SIGCONT);
	err = errno;

	if(ret != ZERORET) {
		xx_rpt(FAILURE);
		in_rpt("kill(pid, SIGCONT) failed");
		valrpt(ZERORET, ret, NOERROR, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if(stat_child != CHLD_EXIT || ex_code != SIGCONT) {
		xx_rpt(FAILURE);
		in_rpt("kill(pid, SIGCONT) did not cause delivery of SIGCONT signal");
		in_rpt("expected action: process exitted - exit code %d", SIGCONT);
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(6);

	DBUG_VOID_RETURN;
}

private void
test13()
{

	DBUG_ENTER("test13");
	/* 
	 * Test that ESRCH isn't given on a kill to a process
	 * group where the last process hasn't been waited for.
	 */

#if TEST_XPG > 3 || !defined(_XOPEN_SOURCE)
	(void) cppair(ch_t13, pa_t13, WAITTIME, 0);
#else
	xx_rpt(NOTINUSE);
#endif

	DBUG_VOID_RETURN;
}

private void
ch_t13()
{
	DBUG_ENTER("ch_t13");
	
	/* Become session leader, and exit */
	(void) setsid();
	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t13()
{
	int	fail = 0;
	int	ret;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("pa_t13");

	/* Allow time for the child to exit - mustn't wait() for it */
	(void) sleep( WAITTIME/2 );

	/* kill process group of child */
	errno = NOERROR;
	ret = kill(-tet_child, 0);
	err = errno;

	if(ret != ZERORET && err == ESRCH) {
		xx_rpt(FAILURE);
		in_rpt("kill(-child, 0) failed with ESRCH");
		in_rpt("expected success or a different error condition");
		fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!fail) 
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test14()
{
	int	pathok = setupok;

	DBUG_ENTER("test14");

	/*
	 * Test that kill fails with invalid signal number.
	 */
	(void) signal_init();

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(ch_t14, pa_t14, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t14()
{
	DBUG_ENTER("ch_t14");

	/* Pause and expect to receive no signal */
	(void) pause();

	DBUG_VOID_RETURN;
}

private void
pa_t14()
{
	int	stat_child;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t14");

	DBUG_PRINTF("pa_t14", "child is %ld signal number %d", (long)tet_child, sig_array[VSIGINVAL]);

	/* Try to send the invalid signal */
	errno = NOERROR;
	ret = kill(tet_child, sig_array[VSIGINVAL]);
	err = errno;

	/* Wait for child and expect it to be alive */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if(stat_child != CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - child terminated expectedly");
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(ret != SYSERROR || err != EINVAL) {
		xx_rpt(FAILURE);
		in_rpt("kill using invalid signal number did not return expected value");
		valrpt(SYSERROR, ret, EINVAL, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);	/* cpsig expects globok to be set */

	DBUG_VOID_RETURN;
}

private void
test15()
{
	DBUG_ENTER("test15");

	(void) cpsig(ch_t15, pa_suproc, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t15()
{
	DBUG_ENTER("ch_t15");

	(void) cpsig(ch_15to17recv, ch_15to17send, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_15to17recv()
{
	DBUG_ENTER("ch_15to17recv");

	exec_suproc(tet_child, vguid0, vguid2, saveids, SYSERROR,tet_thistest);

	DBUG_VOID_RETURN;
}

private void
ch_15to17send()
{
	DBUG_ENTER("ch_15to17send");

	exec_suproc(tet_child, vguid1, vguid1, saveids, SYSERROR,tet_thistest);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	DBUG_ENTER("test16");

	/* If POSIX_JOB_CONTROL not defined, untested */
	/* If POSIX_SAVE_IDS not defined, untested */
	if ( job_control == 0 || saveids == SYSERROR ) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	(void) cpsig(ch_t16, pa_suproc, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t16()
{
	DBUG_ENTER("ch_t16");

	(void) cpsig(ch_15to17recv, ch_15to17send, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
test17()
{
	DBUG_ENTER("test17");

	/* If POSIX_JOB_CONTROL not defined, untested */
	/* If POSIX_SAVE_IDS is defined, untested */
	if ( job_control == 0 || saveids != SYSERROR ) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	(void) cpsig(ch_t17, pa_suproc, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t17()
{
	DBUG_ENTER("ch_t17");

	(void) cpsig(ch_15to17recv, ch_15to17send, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
test18()
{
	int	pathok = setupok;

	DBUG_ENTER("test18");

	/*
	 * Test that kill fails with non-existant process.
	 */

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(ch_t18, pa_t18, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
ch_t18()
{
	DBUG_ENTER("ch_t18");

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t18()
{
	int	stat_child;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t18");

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - timed out waiting for child process");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to send the signal to the process that's just disappeared */
	errno = NOERROR;
	ret = kill(tet_child, SIGKILL);
	err = errno;

	if(ret != SYSERROR || err != ESRCH) {
		xx_rpt(FAILURE);
		in_rpt("kill on non-existant process did not return expected value");
		valrpt(SYSERROR, ret, ESRCH, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3); 	/* cpsig expects globok to be set */

	DBUG_VOID_RETURN;
}

private void
sjmpfunc(sig)
int sig;
{
	DBUG_ENTER("sjmpfunc");
	jmpsignum = sig;
	DBUG_LONGJMP(sjmp, 1);
	DBUG_VOID_RETURN;
}
