/*
*      SCCS:  @(#)  POSIX.os/procprim/kill/kill_recv.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/kill - kill_recv Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		kill_recv - used by T.kill in several tests.

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by kill_su to test successful sending of signals via kill()
	  usage:
		kill_recv savestr testnostr vg_ident vg_dbug
	  where savestr indicates whether POSIX_SAVED_IDS to be used.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, Unisoft Ltd.
DATE CREATED:	Fri Mar 31 10:53:31 GMT 1989
MODIFICATIONS:

		Keith Holder, 29 June 1990
		Added privilege checks.
		Pass testno thru to test.

		David Sawyer
		Tue Aug 14 11:14:58 BST 1990
		Restructured argument checking in main().
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

************************************************************************/
#include	 <std.h>
#include	 <sys/types.h>
#include	 <unistd.h>
#include	 <stdlib.h>
#include	 <dbug.h>
#include	 <errno.h>
#include	 <report.h>
#include	 <exitcodes.h>
#include	 <setprv.h>
#include <tet_api.h>
#include	 <sysdep.h>

#ifdef UNDEF_MACROS
#undef kill
#endif


#define	WAITTIME	(5 * SPEEDFACTOR)

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	void	set_pname();

private	void	set_uids();
private	void	subrecv();
private	int	testno;

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{

	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"kill_recv" : (argv[0]));

	switch(argc)
	{
	case 5:
		vg_dbug = argv[4];
		/* fall through */
	case 4:
		vg_ident = argv[3];
		break;
	default:
		vsxerror("argument count wrong");
		exit(A_MISC);
	}

	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testno = atoi(argv[2]);

	set_uids(atoi(argv[1]));

	subrecv();

	exit(OK);

	/* NOTREACHED */

	return(0);
}

private void
set_uids(saveids)
int	saveids;
{
	DBUG_ENTER("set_uids");

	/*
	 * If saved set UIDS required, set effective uid back to the real, 
	 * leaving the saved set UID hopefully different from either of 
	 * the real UID or effective UID.
	 */

	if(saveids == SYSERROR)
		DBUG_VOID_RETURN;

	DBUG_PRINTF("set_uids", "ruid is %d", getuid());
	DBUG_PRINTF("set_uids", "euid is %d", geteuid());


	if ( setuid(getuid()) < 0 ) {
		DBUG_EXIT(A_UIDFAIL);
	}

	DBUG_VOID_RETURN;
}

private void
subrecv()
{
	DBUG_ENTER("subrecv");

	DBUG_PRINTF("subrecv", "ruid is %d", getuid());
	DBUG_PRINTF("subrecv", "euid is %d", geteuid());

	if ( testno == 16 || testno == 17 )
		(void)setsid();

	(void) sleep(WAITTIME);

	DBUG_VOID_RETURN;
}
