/*
*      SCCS:  @(#)  POSIX.os/procprim/kill/kill_send.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/kill - kill_send Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		kill_send - used by T.kill in several tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.kill to test the delivery of signals by a process
	  with appropriate permissions. 
	  usage:
		kill_send child result testno vg_ident vg_dbug

	  where child is the child process that is to be killed,
	  and result says whether kill() should succeed or fail.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, Unisoft Ltd.
DATE CREATED:	Thu Mar 30 16:54:32 GMT 1989
MODIFICATIONS:
		Keith Holder, 28 Jun 1990
		Attempt to send all signals except SIGCONT
		in test 14.
		For tests 15 and 16 send SIGCONT instead of SIGKILL.

		David Sawyer
		Tue Aug 14 11:17:09 BST 1990
		Restructured argument checking in main().
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env,
			removed tet_child declaration.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Geoff Clare, 20 July 1992
			Cast pid_t to long for printing.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<errno.h>
#include	<setprv.h>
#include	<vsx_signal.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef kill
#endif


#define	WAITTIME		(5 * SPEEDFACTOR)
#define	ZERORET			0

extern	void	set_pname();
extern	void	valrpt();
extern	char	*status_child();
extern	char	*signame();

extern	int	sg_code;	/* defined in wait_child.c */
extern	int	ex_code;	/* defined in wait_child.c */
extern	int	sig_array[];

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	int	test_fail = 0;

private	void	subsend();
private	void	success_test();
private	void	fail_test();
private	void	fail_test15();
private	int	testno;

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{

	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"kill_send" : (argv[0]));

	switch(argc)
	{
	case 6:
		vg_dbug = argv[5];
		/* fall through */
	case 5:
		vg_ident = argv[4];
		break;
	default:
		vsxerror("argument count wrong");
		exit(A_MISC);
	}

	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	testno = atoi(argv[3]);

	if ( testno == 8 ) {
		if ( setprv(PRV_KILL) == -1 ) {
			exit(A_MISC);
		}
	}

	subsend(atoi(argv[1]), atoi(argv[2]));

	exit(OK);

	/* NOTREACHED */

	return(0);
}

private void
subsend(chuid, result)
int	chuid;
int	result;
{
	int	stat_child;
	int	err;

	DBUG_ENTER("subsend");

	DBUG_PRINTF("subsend", "child is %d", chuid);
	DBUG_PRINTF("subsend", "ruid is %d", getuid());
	DBUG_PRINTF("subsend", "euid is %d", geteuid());
	DBUG_PRINTF("subsend", "testno is %d", testno);


	tet_child = chuid;		/* Need this up for waitpid() etc. */

	/* Wait for child and expect it to be alive */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME/2)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		DBUG_VOID_RETURN;
	}

	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - child terminated unexpectedly");
		in_rpt("observed action: %s", status_child(stat_child));
		DBUG_VOID_RETURN;
	}

	if ( testno != 15 ) { /* yuck!! */
		if(result != SYSERROR)
			success_test();
		else
			fail_test();
	}
	else {
		fail_test15();
	}

	DBUG_VOID_RETURN;
}
 
private void
success_test()
{
	int	stat_child;
	int	ret;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("success_test");
	DBUG_PRINTF("success_test:", "testno is %d", testno);

	/* Try to send the signal */
	errno = NOERROR;
	ret = kill(tet_child, SIGKILL);
	err = errno;

	if(ret == SYSERROR)
		test_fail++;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(test_fail) {
		xx_rpt(FAILURE);
		in_rpt("kill() did not return expected value");
		valrpt(ZERORET, ret, NOERROR, err);
	}
	else
		PATH_TRACE;
	if(stat_child != CHLD_ABRT || sg_code != SIGKILL) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("child not terminated by signal SIGKILL");
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
fail_test()
{
	int	stat_child;
	int	err;
	int	kret;
	int	kerr;
	int	pathok = 0;

	DBUG_ENTER("fail_test");
	DBUG_PRINTF("fail_test:", "testno is %d", testno);

	/* Try to send the signal */
	errno = NOERROR;
	if ( testno == 16 || testno == 17 ) {
		kret = kill(tet_child, SIGCONT);
	} else {
		kret = kill(tet_child, SIGKILL);
	}
	kerr = errno;

	if(kret != SYSERROR || kerr != EPERM)
		test_fail++;
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(test_fail) {
		xx_rpt(FAILURE);
		in_rpt("kill() did not return expected value");
		valrpt(SYSERROR, kret, EPERM, kerr);
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("child did not exit as expected");
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}


private void
fail_test15()
{
	int	stat_child;
	int	err;
	int	kret;
	int	kerr;
	int	i;
	int	pathok = 0;

	DBUG_ENTER("fail_test15");
	DBUG_PRINTF("fail_test15:", "testno is %d", testno);

	for ( i=1; i < VNUMSIG; i++ ) {
		int sg_num = sig_array[i];


		if ( i == VSIGCONT )
			continue;

		errno = NOERROR;
		/* Try to send the signal */
		DBUG_PRINTF("test15:","sending signal %d (%s) to pid %ld",
			sg_num,signame(sg_num),(long)tet_child);
		kret = kill(tet_child, sg_num );
		kerr = errno;

		if(kret != SYSERROR || kerr != EPERM) {
			if(!test_fail) {
				xx_rpt(FAILURE);
			}
			test_fail++;
			in_rpt("kill(child,%d) (%s) did not return expected value",
			sg_num,signame(sg_num));
			valrpt(SYSERROR, kret, EPERM, kerr);
		}
		else
			PATH_TRACE;

	}
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("child did not exit as expected");
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(((VNUMSIG-2)+2));

	DBUG_VOID_RETURN;
}
