/*
*      SCCS:  @(#)  POSIX.os/procprim/kill/kill_su.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/kill - kill_su Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		kill_su - used by T.kill in several tests

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.kill to test the delivery of signals. 
	  This file exec's another to either send or receive a signal 
	  using real/effective user id as specified below.
	  usage:
		kill_su child ruid euid saveids result testno vg_ident vg_dbug

	  where child is either zero for receiver or process number to 
	  send signal to,
	  ruid is the real user id that is to be used in the test,
	  euid is the effective user id that is to be used in the test,
	  saveids indicates whether POSIX_SAVED_IDS are to be set,
	  and result indicates whether kill() should succeed or fail.
METHODS:
ASSUMES:  kill_su must be setuid root.
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, Unisoft Ltd.
DATE CREATED:	Thu Mar 30 15:00:48 GMT 1989
MODIFICATIONS:

		Keith Holder, 29 June 1990
		Pass testno thru to test.
		Added privilege checks.

		David Sawyer
		Tue Aug 14 11:18:39 BST 1990
		Replaced execle() calls with tet_exec().
		Restructured argument checking in main().

		Geoff Clare, 15 Oct 1990
			Set vg_env to environ.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Peter Skan, 11 Sept 1991
			Added setprv(PRV_ASSIGN)
			Assign privileges to exec'ed progs using prv_assign()

		Geoff Clare, 21 Jan 1992
			Fix spurious chown group argument.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<sys/stat.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<errno.h>
#include	<setprv.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef kill
#endif


#define	SU_UID		0
#define	SU_MODE		(S_ISUID|S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)

extern	char	**environ;

extern	void	set_pname();

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;
extern	char	**environ;

public	char	*prog_send = "../kill_send";
public	char	*prog_recv = "../kill_recv";

private	void	subtest();

private	int	privileges[] = {
	PRV_KILL,
	0
};

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{

	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"kill_su" : (argv[0]));
	switch(argc)
	{
	case 9:
		vg_dbug = argv[8];
		/* fall through */
	case 8:
		vg_ident = argv[7];
		break;
	default:
		vsxerror("argument count wrong");
		exit(A_MISC);
	}

	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}


	subtest(argv[1], atoi(argv[2]), atoi(argv[3]), argv[4], argv[5],
		argv[6]);

	exit(OK);

	/* NOTREACHED */

	return(0);
}

private void
subtest(chldstr, ruid, euid, savestr, resultstr,testnostr)
char	*chldstr;
int	ruid;
int	euid;
char	*savestr;
char	*resultstr;
char	*testnostr;
{
	int	chld;
	char	*prog;
	char	*args[7];

	DBUG_ENTER("subtest");

	chld = atoi(chldstr);
	prog = (chld == 0) ? prog_recv : prog_send;
	DBUG_PRINTF("subtest","prog is %s", prog);

	if ( setprv(PRV_CHOWN) == -1 || setprv(PRV_SETID) == -1 ||
	     setprv(PRV_ASSIGN) == -1 ) {
		DBUG_EXIT(A_MISC);
	}

	/* Make the owner and mode correct for the file exec'ed */
	if(chown(prog, (uid_t)euid, getgid()) < 0) {
		DBUG_EXIT(A_TSETDIR);
	}

	if(chmod(prog, (mode_t)SU_MODE) < 0) {
		DBUG_EXIT(A_CHMODFAIL);
	}

	if (euid == SU_UID) {
		/* Assign privileges to the file exec'ed */
		if (prv_assign(prog, privileges) != 0) {
			DBUG_EXIT(A_MISC);
		}
	}

	/* Make the real user id correct by calling setuid() */
	if(setuid((uid_t)ruid) < 0) {
		DBUG_EXIT(A_UIDFAIL);
	}

	/* 
	 * If child is 0 exec a receiving process.
	 * Otherwise exec a sending process.
	 */

	if(chld == 0) {
		args[0] = prog;
		args[1] = savestr;
		args[2] = testnostr;
		args[3] = vg_ident;
		args[4] = vg_dbug;
		args[5] = (char *) NULL;	

		(void) tet_exec(prog, args, environ);
	} else {
		args[0] = prog;
		args[1] = chldstr;
		args[2] = resultstr;
		args[3] = testnostr;
		args[4] = vg_ident;
		args[5] = vg_dbug;
		args[6] = (char *) NULL;	

		(void) tet_exec(prog, args, environ);
	}

	vsxerror("tet_exec(%s,..) failed errno %d", prog, errno);
	DBUG_EXIT(A_EXECFAIL);

	DBUG_VOID_RETURN;
}
