/*
*      SCCS:  @(#)  POSIX.os/procprim/kill/kill_t12.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/kill - kill_t12 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		kill_t12 - used by T.kill in test12()

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.kill to test the receipt of SIGCONT signal by any 
	  process in the same session irrespective of its user ID.
	  usage:
		kill_t12 uid vg_ident vg_dbug

	  where uid is the user id number that is to be used in the test.
METHODS:
ASSUMES:  kill_t12 must be setuid root.
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Wed Mar 29 12:54:20 GMT 1989
MODIFICATIONS:
		Keith Holder, 29 June 1990
		Added privilege check.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

************************************************************************/
#include	 <std.h>
#include	 <sys/types.h>
#include	 <unistd.h>
#include	 <stdlib.h>
#include	 <dbug.h>
#include	 <setprv.h>
#include	 <exitcodes.h>
#include	 <signal.h>
#include <tet_api.h>
#include	 <sysdep.h>

#ifdef UNDEF_MACROS
#undef kill
#endif



public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

extern	void	set_pname();

private	int	sig_recvd;

private	int	sig_init();
private	void	sig_catch();
private	void	subtest12();

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"kill_t12" : (argv[0]));
	if (argc == 4)
	{
		if (argv[3] != (char *) NULL)
			vg_dbug = argv[3];
		argc--;
	}
	if (argc == 3)
	{
		if (argv[2] != (char *) NULL)
			vg_ident = argv[2];
		argc--;
	}
	if (argc != 2)
	{
		vsxerror("argument count wrong");
		exit(A_MISC);
	}
	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if ( setprv(PRV_SETID) == -1 ) {
		exit(A_UIDFAIL);
	}

	subtest12(atoi(argv[1]));

	exit(OK);

	/* NOTREACHED */

	return(0);
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

private void
subtest12(uid)

	int	uid;
{
	sigset_t	sig_empty;

	DBUG_ENTER("subtest12");

	(void) setuid((uid_t)uid);

	/* Call sigaction() to initialise signal handling for SIGCONT */
	(void) sigemptyset(&sig_empty);
	(void) sig_init(SIGCONT, &sig_empty, sig_catch, 0);

	sig_recvd = 0;

	/* Send ourself the stop signal */
	(void) kill(getpid(), SIGSTOP);

	/* Exit with signal number caught - should be SIGCONT */
	DBUG_EXIT(sig_recvd);

	DBUG_VOID_RETURN;
}
