/*
*      SCCS:  @(#)POSIX.os/procprim/kill_X/kill_X.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/kill_X - T.kill_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.kill_X

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the kill() system call, X/OPEN extensions.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu May  4 12:02:32 BST 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG

		David Sawyer
		Wed Aug 15 09:33:26 BST 1990
		Changed execle call to tet_exec.

		Stuart Boutell, 5 Nov 1990
			Changed vg_env to environ.
			WAITTIME in cppair changed from *4 -> *16

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<exitcodes.h>
#include	<dbug.h>
#include	<report.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef kill
#endif


#define	WAITTIME		(5 * SPEEDFACTOR)
#define	NO_TESTS		1		/* number of tests in testset */

extern	void		vsx_sigs();
extern	char		*itoa();
extern	char		*newstr();

extern	char		*vg_ident;
extern	char		*vg_dbug;
extern	char		**environ;
extern	int		sig_array[];

int	no_tests = NO_TESTS;

private char		*prog_su  = "../kill_su_X";

private	void		test1();
private	void		ch_t1();
private	void		exec_suproc();
private	void		prepare_test();
private	int		sig_init();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private	void
prepare_test()
{
	int	i;	/* Counter */
	int	ret;	/* Used for most return values */
	int	nsigs;
	sigset_t sig_empty;

	DBUG_ENTER("prepare_test");

	vsx_sigs();

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		nsigs = VNOJSIG;
	} else {
		nsigs = VNUMSIG;
	}

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
test1()
{
	DBUG_ENTER("test1");

	prepare_test();
	(void) cppair(ch_t1, NULLFN, 16*WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	DBUG_ENTER("ch_t1");

	exec_suproc();

	DBUG_VOID_RETURN;
}

private void
exec_suproc()
{
	char 	*args[4];

	DBUG_ENTER("exec_suproc");

	/* Can't use execle() - null vg_dbug stops environ being passed */

	args[0] = prog_su;
	args[1] = vg_ident;
	args[2] = vg_dbug;
	args[3] = (char *) NULL;

	(void) tet_exec(prog_su, args, environ);

	vsxerror("exec(%s,..) failed errno %d", prog_su, errno);
	DBUG_EXIT(A_EXECFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}
