/*
*      SCCS:  @(#)  POSIX.os/procprim/kill_X/kill_su_X.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/kill_X - kill_su_X Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		kill_su_X - used by T.kill_X in test 1.

PROJECT:	VSX (X/OPEN Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.kill_X to test kill(-1, ...)
	  usage:
		kill_su_X vg_ident vg_dbug

METHODS:
ASSUMES:  kill_su_X must be setuid root.
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, Unisoft Ltd.
DATE CREATED:	Thu May  4 12:01:21 BST 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG

		David Sawyer
		Wed Aug 15 09:34:53 BST 1990
		Restructured argument checking for main.
		Added resilience code.

		Stuart Boutell, 5 Nov 1990
			Eliminated vg_env.

		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Stuart Boutell, 13 Dec 1990
			DBUG_EXIT(OK) added to p1,p2,p3

		Peter Skan, 27 Sept 1991
			Added setprv(PRV_SETID)

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<errno.h>
#include <tet_api.h>
#include	<setprv.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef kill
#endif


#define	WAITTIME		(5 * SPEEDFACTOR)
#define	SU_UID		0
#define	READ		0
#define	WRITE		1

extern	char	*status_child();
extern	void	set_pname();
extern	void	vsx_sigs();

extern	int	vguid2;
extern	int	ex_code;

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	sigset_t sig_empty;
private	int	sig_recvd;
private	int	test_fail = 0;
private	int	test_deleted = 0;
private	int	p1_pipe[2];
private	int	p2_pipe[2];
private	int	p3_pipe[2];
private int	setupok;

private void	pa_p1();
private void	pa_p2();
private void	pa_p3();
private void	p1();
private void	p2();
private void	p3();
private	int	wait_for();
private	int	wait_exit();
private	int	write_err();
private	void	prepare_tests();
private	void	subtest();
private	int	sig_init();
private	void	sig_catch();

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"kill_su_X" : (argv[0]));
	switch(argc)
	{
	case 3:
		vg_dbug = argv[2];
		/* fall through */
	case 2:
		vg_ident = argv[1];
		break;
	default:
		vsxerror("argument count wrong");
		exit(A_MISC);
	}

	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	if (setprv(PRV_SETID) != 0) {
		vsxerror("setprv(PRV_SETID) failed");
		exit(A_TSETDIR);
	}

	prepare_tests();
	subtest();
	exit(OK);

	/* NOTREACHED */

	return(0);
}

private	void
prepare_tests()
{
	int	i;	/* Counter */
	int	ret;	/* Used for most return values */
	int	nsigs;
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	if(uids()!=0)
	{
		xx_rpt(DELETION);
		DBUG_EXIT(OK);
	}

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		nsigs = VNOJSIG;
	} else {
		nsigs = VNUMSIG;
	}

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if (i==1)
			PATH_TRACE;
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	if (pathok == 2)
		setupok = 0;
	else
		setupok = -1;

	(void) sigemptyset(&sig_empty);

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

private void
subtest()
{
	int	err;

	DBUG_ENTER("subtest");

	/*
	 * Fork a process p1, which in turn forks p2, which in turn forks p3.
	 * p1 alters real and effective user ID to ruid but stays in same 
	 * process group as the parent.
	 * p2 creates a new session and calls kill(-1, SIGUSR1).
	 * p3 creates a new session.
	 * p1, p2 and p3 should all receive signal SIGUSR1. 
	 * Parent (still running as super-user) should not receive signal.
	 * Parent checks all signals were received.
	 */


	/* Initialise signal action */
	if(sig_init(SIGUSR1, &sig_empty, sig_catch, 0) != 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed - errno %d", err);
		DBUG_VOID_RETURN;
	}

	/* Open three pipes */
	if(pipe(p1_pipe) == SYSERROR ||
	   pipe(p2_pipe) == SYSERROR ||
	   pipe(p3_pipe) == SYSERROR ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed - errno %d", err);
		DBUG_VOID_RETURN;
	}

	(void) cpsig(p1, pa_p1, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
p1()
{
	int	err;

	DBUG_ENTER("p1");

	/* Set real and effective user ID to ruid */
	if(setuid((uid_t)vguid2) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - setuid() failed - errno %d", err);
		DBUG_EXIT(A_MISC);
	}

	/*
	 * Do extra checks on user ID since if this test is conducted as
	 * super-user, it might kill system processes.
	 */

	if(getuid() != vguid2) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - real user ID %d should be %d", 
						getuid(), vguid2);
		DBUG_EXIT(A_MISC);
	}

	if(geteuid() != vguid2) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - effective user ID %d should be %d", 
						geteuid(), vguid2);
		DBUG_EXIT(A_MISC);
	}
	 
	(void) cpsig(p2, pa_p2, WAITTIME, 0);

	DBUG_EXIT(OK);
	/*NOTREACHED*/	
	DBUG_VOID_RETURN;
}

private void
p2()
{
	int	err;

	DBUG_ENTER("p2");

	/* Create a new session */
	if(setsid() == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - setsid() failed - errno %d", err);
		DBUG_EXIT(A_MISC);
	}

	(void) cpsig(p3, pa_p3, WAITTIME, 0);
	DBUG_EXIT(OK);
	/*NOTREACHED*/	
	DBUG_VOID_RETURN;
}

private void
p3()
{
	int	ret;
	int	err;

	DBUG_ENTER("p3");


	/* Create a new session */
	if(setsid() == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - setsid() failed - errno %d", err);
		DBUG_EXIT(A_MISC);
	}

	sig_recvd = 0;

	/* Sleep for long enough to be interrupted by the signal */
	(void) sleep(5*WAITTIME/2);

	/* Write to the pipe the signal information */
	errno = NOERROR;
	ret = write(p3_pipe[WRITE], (char *)&sig_recvd, sizeof(int));
	err = errno;

	/* Check for write failure */
	if(write_err(ret, err, "child of process calling kill()") == FAIL) 
		DBUG_EXIT(A_MISC);
	DBUG_EXIT(OK);
	/*NOTREACHED*/
	DBUG_VOID_RETURN;
}

private void
pa_p3()
{
	int	ret;
	int	err;

	DBUG_ENTER("pa_p3");

	/* Check for initialisation failure */
	if(wait_for(WAITTIME/2, "child of process calling kill()") == FAIL)
		DBUG_EXIT(A_MISC);

	sig_recvd = 0;

	/* Sleep for long enough to let other initialisation complete */
	(void) sleep(3*WAITTIME/2);

	/* Send the test signal */
	if(kill((pid_t)-1, SIGUSR1) == SYSERROR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed - errno %d", err);
		test_deleted++;
	}

	/* Write to the pipe the signal information */
	errno = NOERROR;
	ret = write(p2_pipe[WRITE], (char *)&sig_recvd, sizeof(int));
	err = errno;

	/* Check for child failure */
	if(wait_exit(WAITTIME, "child of process calling kill()") == FAIL) 
		DBUG_EXIT(A_MISC);

	if(test_deleted)
		DBUG_EXIT(A_MISC);

	/* Check for write failure */
	if(write_err(ret, err, "process calling kill()") == FAIL) 
		DBUG_EXIT(A_MISC);

	globok = 1;	/* cpsig expects globok to be set */

	DBUG_VOID_RETURN;
}

private void
pa_p2()
{
	int	ret;
	int	err;

	DBUG_ENTER("pa_p2");

	/* Check for initialisation failure */
	if(wait_for(WAITTIME, "process calling kill()") == FAIL)
		DBUG_EXIT(A_MISC);

	sig_recvd = 0;

	/* Sleep for long enough to be interrupted by the signal */
	(void) sleep(3*WAITTIME/2);

	/* Write to the pipe the signal information */
	errno = NOERROR;
	ret = write(p1_pipe[WRITE], (char *)&sig_recvd, sizeof(int));
	err = errno;

	/* Check for child failure */
	if(wait_exit(WAITTIME, "process calling kill()") == FAIL) 
		DBUG_EXIT(A_MISC);

	/* Check for write failure */
	if(write_err(ret, err, "parent of process calling kill()") == FAIL) 
		DBUG_EXIT(A_MISC);

	globok = 1;	/* cpsig expects globok to be set */

	DBUG_VOID_RETURN;
}

private void
pa_p1()
{
	int	p1_recv;
	int	p2_recv;
	int	p3_recv;
	int	ret1;
	int	ret2;
	int	ret3;
	int	err1;
	int	err2;
	int	err3;
	int	pathok = setupok;

	DBUG_ENTER("pa_p1");

	/* Check for initialisation failure */
	if(wait_for(3*WAITTIME/2, "parent of process calling kill()") == FAIL) 
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig_recvd = 0;

	/* Sleep for long enough to be interrupted by the signal */
	(void) sleep(WAITTIME);

	/* Try to read from each pipe to obtain signal information */
	SET_TIMEOUT(WAITTIME/2)
	errno = NOERROR;
	ret1 = read(p1_pipe[READ], (char *)&p1_recv, sizeof(int));
	err1 = errno;
	CLEAR_ALARM
	SET_TIMEOUT(WAITTIME/2)
	errno = NOERROR;
	ret2 = read(p2_pipe[READ], (char *)&p2_recv, sizeof(int));
	err2 = errno;
	CLEAR_ALARM
	SET_TIMEOUT(WAITTIME/2)
	errno = NOERROR;
	ret3 = read(p3_pipe[READ], (char *)&p3_recv, sizeof(int));
	err3 = errno;
	CLEAR_ALARM

	/* Check for child failure */
	if(wait_exit(WAITTIME, "parent of process calling kill()") == FAIL) 
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the results of reading pipes */
	if(ret1 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - 1. read() failed - errno %d", err1);
		DBUG_VOID_RETURN;
	} else if(ret1 != sizeof(int)) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - 1. read() returned %d instead of %d",
							ret1, sizeof(int));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(ret2 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - 2. read() failed - errno %d", err2);
		DBUG_VOID_RETURN;
	} else if(ret2 != sizeof(int)) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - 2. read() returned %d instead of %d",
							ret2, sizeof(int));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(ret3 == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - 3. read() failed - errno %d", err3);
		DBUG_VOID_RETURN;
	} else if(ret3 != sizeof(int)) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - 3. read() returned %d instead of %d",
							ret3, sizeof(int));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the signal was received by all expected processes */
	if(p1_recv != SIGUSR1) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("parent of process calling kill() did not receive signal %d", SIGUSR1);
		if(p1_recv)
			in_rpt("instead received signal %d", p1_recv);
		test_fail++;
	}
	else
		PATH_TRACE;
	if(p2_recv != SIGUSR1) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("process calling kill() did not receive signal %d", SIGUSR1);
		if(p2_recv)
			in_rpt("instead received signal %d", p2_recv);
		test_fail++;
	}
	else
		PATH_TRACE;
	if(p3_recv != SIGUSR1) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("child of process calling kill() did not receive signal %d", SIGUSR1);
		if(p3_recv)
			in_rpt("instead received signal %d", p3_recv);
		test_fail++;
	}
	else
		PATH_TRACE;
	if(sig_recvd == SIGUSR1) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("process with wrong user ID received signal %d", SIGUSR1);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!test_fail)
	{
		PATH_XS_RPT(9);
	} else
	{
		xx_rpt(NOTINUSE);
	}

	globok = 1;	/* cpsig expects globok to be set */

	DBUG_VOID_RETURN;
}

private int
wait_for(waittime, process)
int	waittime;
char	*process;
{
	int	err;
	int	stat_child;

	DBUG_ENTER("wait_for");

	/* Wait for a while and check that initialisation was all done */
	errno = NOERROR;
	if((stat_child = wait_child(waittime)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waiting for %s", process);
		in_rpt("waitpid() failed - errno %d", err);
		DBUG_RETURN(FAIL);
	}

	/* Expect process to be still alive */
	if(stat_child == CHLD_EXIT) {
		/* There was an initialisation failure */
		DBUG_RETURN(FAIL);
	}
	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - waiting for %s", process);
		in_rpt("expected action: process still active");
		in_rpt("observed action %s", status_child(stat_child));
		DBUG_RETURN(FAIL);
	}

	DBUG_RETURN(SUCCEED);
}

private int
wait_exit(waittime, process)
int	waittime;
char	*process;
{
	int	err;
	int	stat_child;

	DBUG_ENTER("wait_exit");

	/* Wait for the child and ensure test was not already deleted */
	errno = NOERROR;
	if((stat_child = wait_child(waittime)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waiting for %s", process);
		in_rpt("waitpid() failed - errno %d", err);
		DBUG_RETURN(FAIL);
	}

	/* Expect process to exit with exit code OK */
	if(stat_child == CHLD_EXIT && ex_code != OK) {
		/* Test was already deleted */
		DBUG_RETURN(FAIL);
	}
	if(stat_child != CHLD_EXIT) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - waiting for %s", process);
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action %s", status_child(stat_child));
		DBUG_RETURN(FAIL);
	}

	DBUG_RETURN(SUCCEED);
}

private int
write_err(ret, err, process)
int	ret;
int	err;
char	*process;
{
	DBUG_ENTER("write_err");

	if(ret == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - write in %s", process);
		in_rpt("write() failed - errno %d", err);
		DBUG_RETURN(FAIL);
	} else if(ret != sizeof(int)) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - write in %s", process);
		in_rpt("write() returned %d instead of %d", ret, sizeof(int));
		DBUG_RETURN(FAIL);
	}

	DBUG_RETURN(SUCCEED);
}
