/*
*      SCCS:  @(#)POSIX.os/procprim/pause/pause.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/pause - T.pause Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.pause

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the pause() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Wed Mar  8 09:23:45 GMT 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG

		Peter Lee, Unisoft Ltd.,
		Fri Aug 10 13:03:58 BST 1990
		Added resilience code.

		Stuart Boutell, 30 Apr 1991
			Changed cpsig() return code check from FAIL to
			SYSERROR

		Geoff Clare, 22 Nov 1994
			Added declaration of sig_init().

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef pause
#endif


#define	NO_TESTS	4		/* number of tests in testset */
#define	WAITTIME	(3*SPEEDFACTOR)
#define	EXTERR		"external error"

extern	void		vsx_sigs();
extern	void		valrpt();

extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sig_type[];
extern	int		ex_code;

int	no_tests = NO_TESTS;

private	void		(*sig_arg)();
private	int		test_fail;
private	int		test_deleted;
private	int		sig_index;
private	int		sig_err;
private	sigset_t	sig_empty;
private int		sig_recvd;
private int		nsigs;
private int		job_control;

private	void		test1();
private	void		test2();
private	void		test3();
private	void		test4();
private	void		do_1();
private	void		ch_t1();
private	void		pa_t1();
private	void		ch_t2();
private	void		pa_t2();
private	void		ch_t3();
private	void		pa_t3();
private	void		ch_t4();
private	void		pa_t4();
private	void		prepare_tests();
private	void		sig_catch();
private int		sig_init();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	NULL, 0
};

private	void
prepare_tests()
{
	int	ret;		/* Used for most return values */
	int	i;		/* counter */

	DBUG_ENTER("prepare_tests");

	vsx_sigs();

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		job_control = 0;
		nsigs = VNOJSIG;
	} else {
		job_control = 1;
		nsigs = VNUMSIG;
	}

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */
	int	ret;

	DBUG_ENTER("sig_init");

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	ret = sigaction(signum, &act, (struct sigaction *)NULL);

	if(sigfunc == SIG_DFL)
		DBUG_PRINTF("sig_init", "DEFAULT sigaction returns %d for signal %d", ret, signum);
	else if (sigfunc == SIG_IGN)
		DBUG_PRINTF("sig_init", "IGNORE sigaction returns %d for signal %d", ret, signum);
	else
		DBUG_PRINTF("sig_init", "CATCH sigaction returns %d for signal %d", ret, signum);

	DBUG_RETURN(ret);
}

private void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = 0;

	DBUG_ENTER("test1");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	do_1(SIG_DFL);
	PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	do_1(sig_catch);
	PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
do_1(arg)
void	(*arg)();	/* One of SIG_DFL or signal catching function */
{
	int	i;
	int	pathok = 0;
	int	loop1;

	DBUG_ENTER("do_1");

	/*
	 * Test pause().
	 * If arg is SIG_DFL, do this for all signals which terminate.
	 * If arg is signal catcher, do this for all catchable signals. 
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1, loop1 = 0 ; i < nsigs ; i++, loop1++)
	{
		if(sig_array[i] == VNOSIG)
		{
			PATH_TRACE; PATH_TRACE;
			continue;
		}

		if(arg != SIG_DFL)
			/* Can't catch these signals */
			if(i == VSIGKILL || i == VSIGSTOP)
			{
				PATH_TRACE; PATH_TRACE;
				continue;
			}

		if(arg == SIG_DFL)
			/* Do this for all signals which terminate process */
			if(sig_type[sig_array[i]] != TYPE_ABRT)
			{
				PATH_TRACE; PATH_TRACE;
				continue;
			}

		sig_index = i;
		sig_arg = arg;

		globok = 0;
		if(cpsig(ch_t1, pa_t1, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		PATH_FUNC_TRACE;

		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	PATH_FUNC_RPT(2 * loop1);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t1");

	/* Initialise required signal action */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	sig_recvd = 0;

	/* Suspend the process */
	(void) pause();

	/* If pause() returns, exit with number of signal received */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t1");

	sig_err = 0;

	/* Wait - should time out if child really is suspended */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child != CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("pause() did not suspend process");
		/* set deleted flag - no point in carrying on */
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the un-blocked signal */
	if(kill(tet_child, sg_num) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the process is no longer suspended */
	if(stat_child != CHLD_EXIT && stat_child != CHLD_ABRT) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("pause() not interrupted by un-blocked signal %d (%s)", 
							sg_num, sg_name);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	i;
	int	pathok = 0;
	int	loop1;

	DBUG_ENTER("test2");

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Test that pause() does not return when signal terminates 
	 * process.
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1, loop1 = 0 ; i < nsigs ; i++, loop1++)
	{
		if(sig_array[i] == VNOSIG)
		{
			PATH_TRACE; PATH_TRACE;
			continue;
		}

		/* Do this for all signals which terminate process */
		if(sig_type[sig_array[i]] != TYPE_ABRT)
		{
			PATH_TRACE; PATH_TRACE;
			continue;
		}

		sig_index = i;

		globok = 0;
		if(cpsig(ch_t2, pa_t2, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		PATH_FUNC_TRACE;

		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	if(!test_fail)
		PATH_XS_RPT(2 * loop1);

	DBUG_VOID_RETURN;
}


private void
ch_t2()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("pa_t2");

	/* Initialise required signal action */
	(void) sig_init(sg_num, &sig_empty, SIG_DFL, 0);

	/* Suspend the process */
	(void) pause();

	/* Failure. Parent checks exit code */
	DBUG_EXIT(A_MISC);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	int	err;
	int	stat_child;
	int	sg_num   = sig_array[sig_index];
	char *	sg_name  = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t2");

	/* Wait - should time out if child really is suspended */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - pause() did not suspend process");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the signal */
	if(kill(tet_child, sg_num) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_MISC) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("pause() returned when signal %d (%s) generated",
					sg_num, sg_name);
		in_rpt("expected process to terminate on delivery of signal");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	int	i;
	int	pathok = 0;
	int	loop1;

	DBUG_ENTER("test3");

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Test that pause() returns after signal catching function executes.
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1, loop1 = 0 ; i < nsigs ; i++, loop1++)
	{
		if(sig_array[i] == VNOSIG)
		{
			PATH_TRACE; PATH_TRACE; 
			continue;
		}

		/* Can't catch or ignore these signals */
		if(i == VSIGKILL || i == VSIGSTOP)
		{
			PATH_TRACE; PATH_TRACE;
			continue;
		}

		sig_index = i;

		globok = 0;
		if(cpsig(ch_t3, pa_t3, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		PATH_FUNC_TRACE;

		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	if(!test_fail)
		PATH_XS_RPT(2 * loop1);

	DBUG_VOID_RETURN;
}
		
private void
ch_t3()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t3");

	/* Initialise required signal action */
	(void) sig_init(sg_num, &sig_empty, sig_catch, 0);

	/* Suspend the process */
	(void) pause();

	/* If pause() returns, exit with number of signal received */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("ch_t3");

	/* Wait - should time out if child really is suspended */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - pause() did not suspend process");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the signal */
	if(kill(tet_child, sg_num) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != sg_num) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("pause() did not return when signal %d (%s) generated",
					sg_num, sg_name);
		in_rpt("expected process to catch signal and pause() to return");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	/*
	 * Test that pause(), when a signal is caught and control returned,
	 * returns a value of -1 and errno is set to EINTR.
	 * Do this in the child process. The child process also does all the 
	 * reporting for this test.
	 */

	sig_index = VSIGHUP;

	(void) cppair(ch_t4, pa_t4, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	sg_num  = sig_array[sig_index];
	int	err;
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("ch_t4");

	/* Initialise required signal action */
	(void) sig_init(sg_num, &sig_empty, sig_catch, 0);

	errno = NOERROR;
	ret = pause();
	if(ret != SYSERROR || errno != EINTR) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("pause() returned wrong value when interrupted");
		valrpt(SYSERROR, ret, EINTR, err);
	} else 
		PATH_XS_RPT(0);

	DBUG_VOID_RETURN;
}

private void
pa_t4()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("pa_t4");

	/* Sleep to give the child time to suspend */
	(void) sleep(WAITTIME/2);

	/* Send un-blocked signal to interrupt pause() */
	(void) kill(tet_child, sg_num);

	globok = 1; /* must set globok for cppair */

	DBUG_VOID_RETURN;
}
