/*
*      SCCS:  @(#)POSIX.os/procprim/sigaction/sigaction.c	Rel 4.4.4     (07/19/99) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/sigaction - T.sigaction Rel 4.4.4 (07/19/99)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.sigaction

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the sigaction() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Wed Mar  1 11:31:40 GMT 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG

		Geoff Clare, 8 Nov 1989
			Make sure stop signals are not tested on orphaned
			process group.
		
		Geoff Clare, 23 Nov 1989
			Test which uses signal() is unsupported if
			ANSI_SIGNAL_FUNC is 0
		
		Geoff Clare, 6 Apr 1990
			Allow setting SIGKILL and SIGSTOP to SIG_DFL
			either to succeed or to give EINVAL.

		Peter Lee, Unisoft Ltd.,
		Mon Aug 13 10:06:30 BST 1990
		Added resilience code.

		David Sawyer
		Tue Aug 21 11:32:02 BST 1990
		Appended resilience code.

		Stuart Boutell, 7 Nov 1990
			tet_child redeclaration removed.

		Stuart Boutell, 12 Apr 1991
			Added memset() code to fill structure members
			before calling sigaction. Note that sig_init()
			performs a fill_struct() call.
			Added new tests for SIG_DFL with SIGKILL & SIGSTOP.

		Stuart Boutell, 30 Apr 1991
			Changed cpsig() return code check from FAIL to
		SYSERROR

		Geoff Clare, 30 Apr 1992
			Removed sigaction(sig, NULL, NULL) test.

		Geoff Clare, 14 May 1993
			Simplify gch_t7().

		Geoff Clare, 1 Nov 1994
			Add (unused) argument to sig_alter().

		Geoff Clare, 6 Nov 1998
			Fix omits/includes mix-up in cmp_mask().

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<setjmp.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigaction
#endif


#define	NO_TESTS		17		/* number of tests in testset */
#define	WAITTIME		(5*SPEEDFACTOR)
#define	EXTERR			"external error"

#undef  memset
extern	void *		memset();

extern	char *		sig_child();
extern	char *		signame();
extern	char *		errname();
extern	void		vsx_sigs();
extern	void		valrpt();
extern	int		cppair();

extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sig_type[];
extern	int		ex_code;
extern	int		sg_code;

int	no_tests = NO_TESTS;

private	void		(*sig_arg)();
private	int		sig_index;
private pid_t		parent;
private pid_t		myself;
private	sigset_t	sig_empty;
private int		sig_recvd;
private int		sig_new;
private int		sig_err;
private int		test_fail;
private int		test_deleted;
private int		test_arg;
private int		nsigs;
private int		job_control;
private int		setupok;

private	void		test1();
private	void		test2();
private	void		test3();
private	void		test4();
private	void		test5();
private	void		test6();
private	void		test7();
private	void		test8();
private	void		test9();
private	void		test10();
private	void		test11();
private	void		test12();
private	void		test13();
private	void		test14();
private	void		test15();
private	void		test16();
private	void		test17();
private	void		ch_t1();
private	void		ch_t2();
private	void		ch_t3();
private	void		ch_t5();
private	void		do_6();
private	void		ch_t6();
private	void		pa_t6();
private	void		ch_t7();
private	void		gch_t7();
private	void		ch_t8();
private	void		ch_t9();
private	void		pa_t9();
private	void		ch_t10();
private	void		pa_t10();
private	void		ch_t11();
private	void		ch_t12();
private	void		ch_t13();
private	void		ch_t14();
private	void		ch_t16and17();
private	void		pa_t16and17();
private	void		do_sig_catch();
private	void		prepare_tests();
private	void		clean_tests();
private	void		sig_catch();
private	void		sig_alter();
private	void		sig1_catch();
private	void		sig2_catch();
private	void		sig3_catch();
private	void		sig_level2();
private	void		sig_gchild();
private	void		sig_recur();
private	int		sig_init();
private	void		fill_struct();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	NULL, 0
};

private	void
prepare_tests()
{
	int	ret;		/* Used for most return values */
	int	i;		/* counter */
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		job_control = 0;
		nsigs = VNOJSIG;
	} else {
		job_control = 1;
		nsigs = VNUMSIG;
	}

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);

	PATH_SETUP_RPT(1);

	DBUG_VOID_RETURN;
}

private	void
clean_tests()
{
	DBUG_ENTER("clean_tests");

	(void) unlink("core");

	DBUG_VOID_RETURN;
}

private int
cmp_mask(aset, acmpset)
sigset_t	*aset;		/* signal mask */
sigset_t	*acmpset;	/* signal mask  for comparison */
{
	int	fail = 0;
	int	i;
	int	signum;
	char	*name;
	int	pathok = 0;

	DBUG_ENTER("cmp_mask");

	globok = 0;

	for(i = 1; i < nsigs; i++) {
		signum = sig_array[i];
		name = sig_name[i];
		if(sigismember(acmpset, signum)) {
			if(!sigismember(aset, signum)) {
				if(!test_fail)
					xx_rpt(FAILURE);
				in_rpt("signal mask omits signal %d %s",
								signum, name);
				test_fail++;
				fail++;
			}
			else 
				PATH_TRACE;
		} else {
			if(sigismember(aset, signum)) {
				if(!test_fail)
					xx_rpt(FAILURE);
				in_rpt("signal mask includes signal %d %s",
								signum, name);
				test_fail++;
				fail++;
			}
			else
				PATH_TRACE;
		}
	}
	if(fail) {
		DBUG_RETURN(FAIL);
	}

	PATH_FUNC_RPT(nsigs - 1);
	
	DBUG_RETURN(SUCCEED);
}

private sigset_t *
sig_mask(signum)
int	signum;		/*  a signal to put in mask */
{
	private	sigset_t set;		/* signal mask */

	/* Return address of signal mask */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, signum);

	return(&set);
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	fill_struct(&act);	/* Fill sigaction structure */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

/* ARGSUSED */
private void
sig_alter(sig)
int sig;
{
	DBUG_ENTER("sig_alter");

	sig_new++;

	DBUG_PRINTF("sig_alter", "sig_new %d", sig_new);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	int	err;
	int	ret;		/* Used for most return values */
	struct sigaction oact;	/* signal action structure */
	int	pathok = setupok;

	DBUG_ENTER("ch_t1");

	/* Call sigaction() to initialise signal handling function */
	if(sig_init(SIGCHLD, sig_mask(SIGINT), sig_catch, SA_NOCLDSTOP) < 0) {
		xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Clear sigaction structure for return value */
	globok = 0;
	fill_struct(&oact);	/* Fill the sigaction structure with data */
	PATH_FUNC_TRACE;
	oact.sa_handler = SIG_IGN;
	oact.sa_flags = ~SA_NOCLDSTOP;
	(void) sigemptyset(&(oact.sa_mask));

	/* Call sigaction() */
	ret = sigaction(SIGCHLD, (struct sigaction *)NULL, &oact);
	if(ret != 0) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() returns incorrect value %d", ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	test_fail = 0;

	/* Check sigaction structure returned in oact */
	if(oact.sa_handler != (void(*)())sig_catch) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() returns sa_handler field incorrectly");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!(oact.sa_flags & SA_NOCLDSTOP)) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigaction() returns sa_flags field incorrectly");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(sigismember(&(oact.sa_mask), SIGINT) != 1) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigaction() returns sa_mask field incorrectly");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(test_fail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the signal handling is not altered */
	sig_recvd = 0;
	myself = getpid();
	ret = kill(myself, SIGCHLD);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!sig_recvd) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() has incorrectly altered signal handling");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(9);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	(void) cppair(ch_t2, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	int	err;
	int	ret;		/* Used for most return values */
	struct sigaction act;	/* signal action structure */
	int	pathok = setupok;

	DBUG_ENTER("ch_t2");

	/* Call sigaction() to initialise signal handling function */
	if(sig_init(SIGCHLD, sig_mask(SIGINT), sig_catch, SA_NOCLDSTOP) < 0) {
		xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Clear sigaction structure for new action */
	globok = 0;
	fill_struct(&act);	/* Fill the sigaction structure with data */
	PATH_FUNC_TRACE;
	act.sa_handler = SIG_IGN;
	act.sa_flags = 0;
	(void) sigemptyset(&(act.sa_mask));

	/* Call sigaction() */
	ret = sigaction(SIGCHLD, &act, (struct sigaction *)NULL);
	if(ret != 0) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() returns incorrect value %d", ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the signal handling is altered to ignore signal */
	sig_recvd = 0;
	myself = getpid();
	ret = kill(myself, SIGCHLD);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(sig_recvd) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() has not altered signal handling");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	(void) cppair(ch_t3, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int	err;
	int	ret;		/* Used for most return values */
	struct sigaction act;	/* signal action structure */
	struct sigaction oact;	/* signal action structure */
	int	pathok = setupok;

	DBUG_ENTER("ch_t3");

	/* Call sigaction() to initialise signal handling function */
	if(sig_init(SIGCHLD, sig_mask(SIGINT), sig_catch, SA_NOCLDSTOP) < 0) {
		xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Clear sigaction structure for return value */
	globok = 0;
	fill_struct(&oact);	/* Fill the sigaction structure with data */
	PATH_FUNC_TRACE;
	oact.sa_handler = SIG_IGN;
	oact.sa_flags = ~SA_NOCLDSTOP;
	(void) sigemptyset(&(oact.sa_mask));

	/* Clear sigaction structure for new action */
	globok = 0;
	fill_struct(&act);	/* Fill the sigaction structure with data */
	PATH_FUNC_TRACE;
	act.sa_handler = SIG_IGN;
	act.sa_flags = 0;
	(void) sigemptyset(&(act.sa_mask));

	/* Call sigaction() */
	ret = sigaction(SIGCHLD, &act, &oact);
	if(ret != 0) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() returns incorrect value %d", ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	test_fail = 0;

	/* Check sigaction structure returned in oact */
	if(oact.sa_handler != (void(*)())sig_catch) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() returns sa_handler field incorrectly");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!(oact.sa_flags & SA_NOCLDSTOP)) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigaction() returns sa_flags field incorrectly");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(sigismember(&(oact.sa_mask), SIGINT) != 1) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigaction() returns sa_mask field incorrectly");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(test_fail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the signal handling is altered to ignore signal */
	sig_recvd = 0;
	myself = getpid();
	ret = kill(myself, SIGCHLD);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(sig_recvd) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() has incorrectly altered signal handling");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(10);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	xx_rpt(NOTINUSE);

	DBUG_VOID_RETURN;
}


private void
test5()
{
	DBUG_ENTER("test5");

	(void) cppair(ch_t5, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	pathok = setupok;

	DBUG_ENTER("ch_t5");

	test_fail = 0;

	/* Call sigaction() to initialise signal handling function */
	if(sig_init(SIGCHLD, sig_mask(SIGINT), SIG_DFL, SA_NOCLDSTOP) < 0) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() setting sa_handler to SIG_DFL");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(sig_init(SIGCHLD, sig_mask(SIGINT), SIG_IGN, SA_NOCLDSTOP) < 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigaction() setting sa_handler to SIG_IGN");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(sig_init(SIGCHLD, sig_mask(SIGINT), sig_catch, SA_NOCLDSTOP) < 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigaction() setting sa_handler to signal catcher");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	int	pathok = setupok;

	DBUG_ENTER("test6");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	do_6(SIG_DFL);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	do_6(SIG_IGN);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	do_6(sig_catch);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
do_6(arg)
void	(*arg)();	/* One of SIG_IGN, SIG_DFL, signal catching function */
{
	char	*args;
	int	i, err;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("do_6");

	globok = 0;

	if(arg == SIG_IGN)
		args = "SIG_IGN";
	else if (arg == SIG_DFL)
		args = "SIG_DFL";
	else
		args = "signal catching function";

	/*
	 * Test action requested for all signals that are defined.
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		/* Can't catch or ignore these signals */
		if(arg != SIG_DFL)
			if(i == VSIGKILL || i == VSIGSTOP)
			{
				PATH_TRACE; PATH_TRACE; PATH_TRACE; 
				PATH_TRACE; PATH_TRACE;
				continue;
			}

		/* Can't ignore this signal either */
		if(arg == SIG_IGN)
			if(i == VSIGCONT)
			{
				PATH_TRACE; PATH_TRACE; PATH_TRACE; 
				PATH_TRACE; PATH_TRACE;
				continue;
			}

		sg_num = sig_array[i];
		sg_name = sig_name[i];

		if(sg_num == VNOSIG)
		{
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("signal %s not defined", sg_name);
			test_fail++;
			continue;
		}
		else
			PATH_TRACE;

		DBUG_PRINTF("do_6", "signal %s", sg_name);

	 	/* 
		 * Set default action via sigaction before forking, so if
		 * it fails we can give up now rather than later.
		 */
		if(sig_init(sg_num, &sig_empty, arg, 0) < 0)
		{
			if (!(arg == SIG_DFL && (sg_num == SIGKILL ||
				sg_num == SIGSTOP) && errno == EINVAL))
			{
				err = errno;
				if(!test_fail)
					xx_rpt(FAILURE);
				in_rpt("sigaction() failed setting action to %s", args);
				in_rpt("for signal %d (%s) - errno %d (%s)",
					sg_num, sg_name, err, errname(err));
				test_fail++;
				continue;
			}
			else
				PATH_TRACE;
		}
		else
			PATH_TRACE;

		/*
		 * Set default for SIGCHLD - prevent interruptions.
		 * Must be before fork in case child exits and signals us 
		 * immediately.
		 */
		if(sig_init(SIGCHLD, &sig_empty, SIG_DFL, 0) < 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigaction() failed");
			test_deleted++;
		}
		else
			PATH_TRACE;

		sig_index = i;
		sig_arg = arg;

		if(!test_deleted) {
			globok = 0;
			if(cpsig(ch_t6, pa_t6, WAITTIME, 0) == SYSERROR)
				test_deleted++;
			PATH_FUNC_TRACE;
		}

		/* set signal action back to default */
		(void) sig_init(sg_num, &sig_empty, SIG_DFL, 0); 

		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	PATH_FUNC_RPT((nsigs-1)*5);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t6");

	/* If the signal will stop the process, make sure
	   it's not in an orphaned process group */

	if(sig_type[sg_num] == TYPE_STOP)
		(void) setpgid((pid_t)0, (pid_t)0);

	/* Have to do this again  -  see actions before fork */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	sig_recvd = 0;

	myself = getpid();

	(void) kill(myself, sg_num);

	/* Exit code is signal number we received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t6()
{
	int	err;
	int	stat_child;
	char *	str_expect;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	sg_type = sig_type[sg_num];
	char	exp_buf[BUFSIZ];
	int	pathok = 0;

	DBUG_ENTER("pa_t6");

	sig_err = 0;
	globok = 0;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* 
	 * Switch on what the argument demanded.
	 */
	if(sig_arg == SIG_DFL) {
		/* Switch on what default action is expected. */
		switch(sg_type) {
		case TYPE_ABRT:	/* child aborts on receiving signal */
			(void) sprintf(exp_buf, "signal %d terminated process", sg_num);
			str_expect = exp_buf;
			if(stat_child != CHLD_ABRT)
				sig_err++;
			else
				PATH_TRACE;
			if(sg_code != sg_num)
				sig_err++;
			else
				PATH_TRACE;
			break;
		case TYPE_IGNR:	/* child ignores so exits normally */
		case TYPE_CONT:	/* here, child ignores SIGCONT */
			str_expect = "process ignored signal";
			if(stat_child != CHLD_EXIT)
				sig_err++;
			else
				PATH_TRACE;
			if(ex_code != 0)
				sig_err++;
			else
				PATH_TRACE;
			break;
		case TYPE_STOP:	/* child process stops */
			PATH_TRACE;
			str_expect = "signal stopped process";
			if(stat_child != CHLD_STOP)
				sig_err++;
			else
				PATH_TRACE;
			break;
		}
	} else if(sig_arg == SIG_IGN) {
		str_expect = "process ignored signal";
		if(stat_child != CHLD_EXIT)
			sig_err++;
		else
			PATH_TRACE;
		if(ex_code != 0)
			sig_err++;
		else
			PATH_TRACE;
	} else {
		(void) sprintf(exp_buf, "process caught signal %d", sg_num);
		str_expect = exp_buf;
		if(stat_child != CHLD_EXIT)
			sig_err++;
		else
			PATH_TRACE;
		if(ex_code != sg_num)
			sig_err++;
		else
			PATH_TRACE;
	}

	if(sig_err) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caused wrong action", sg_name);
		in_rpt("expected action: %s", str_expect);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test7()
{
	DBUG_ENTER("test7");

	(void) cppair(ch_t7, NULLFN, 5*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t7()
{
	int	pathok = setupok;

	DBUG_ENTER("ch_t7");

	test_fail = 0;
	parent = getpid();

	/* First test is that signal mask is correct in signal catcher */
	test_arg = 1;
	globok = 0;
	do_sig_catch();
	if(test_fail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Second test is that sigprocmask() can change mask */
	test_arg = 2;
	globok = 0;
	do_sig_catch();
	if(test_fail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Third test involves sigsupend() - so child must give us a signal */
	test_arg = 3;
	globok = 0;
	(void) cppair(gch_t7, do_sig_catch, 4*WAITTIME, 0);
	if(test_fail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
do_sig_catch()
{
	int	err;
	sigset_t set;		/* blocked signal set */
	int	ret;		/* Used for most return values */
	struct sigaction act;	/* signal action structure */
	int	arg = test_arg;	/* method 1, 2 or 3 */
	int	pathok = 0;

	DBUG_ENTER("do_sig_catch");

	globok = 0;

	/* Set an action for SIGPIPE - used with test_arg = 3 */
	if(sig_init(SIGPIPE, &sig_empty, sig_level2, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Set up mask for basic signals blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTTIN);
	(void) sigaddset(&set, SIGINT);
	(void) sigaddset(&set, SIGKILL);
	ret = sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL);
	if(ret != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returns incorrect value %d", ret);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	fill_struct(&act);	/* Fill the sigaction structure with data */
	PATH_FUNC_TRACE;
	/* Call sigaction to create overlapping blocked signal mask */
	switch(arg) {
	case 1:
		act.sa_handler = sig1_catch;
		PATH_TRACE;
		break;
	case 2:
		act.sa_handler = sig2_catch;
		PATH_TRACE;
		break;
	case 3:
		act.sa_handler = sig3_catch;
		PATH_TRACE;
		break;
	}
	act.sa_flags = 0;
	(void) sigemptyset(&(act.sa_mask));
	(void) sigaddset(&(act.sa_mask), SIGTTOU);
	(void) sigaddset(&(act.sa_mask), SIGINT);
	(void) sigaddset(&(act.sa_mask), SIGSTOP);
	(void) sigaction(SIGTERM, &act, (struct sigaction *)NULL);

	/* The mask is checked in the signal catching function */
	sig_err = 0;
	sig_recvd = 0;
	myself = getpid();
	ret = kill(myself, SIGTERM);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!sig_recvd) {
		xx_rpt(DELETION);
		in_rpt("signal not caught");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(sig_err) 
		test_fail++;
	else
		PATH_TRACE;

	PATH_FUNC_RPT(7);

	DBUG_VOID_RETURN;
}

private void
gch_t7()
{
	/* 
	 * The grandchild is only used to send a signal to it's parent 
	 * when it's parent has called sigsuspend.
	 * When it's parent is about to suspend, it will send us a signal
	 * telling us it is ready. We sleep for 2*WAITTIME whereas it's parent
	 * sleeps for WAITTIME, so the grandchild should be sleeping when the
	 * signal comes.
	 * If this never comes, the child stays simply exits without sending
	 * the signal.
	 */
	DBUG_ENTER("gch_t7");

	/* Set an action for SIGPIPE - signal that tells us to continue */
	if(sig_init(SIGPIPE, &sig_empty, sig_gchild, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		DBUG_VOID_RETURN;
	}

	(void) sleep(2*WAITTIME);

	DBUG_VOID_RETURN;
}

private void
sig_gchild()
{
	DBUG_ENTER("sig_gchild");

	/* Grandchild function gch_t7() will come here on catching SIGPIPE */

	(void) sleep(WAITTIME);
	(void) kill(parent, SIGPIPE);

	DBUG_VOID_RETURN;
}

private void
sig1_catch()
{
	sigset_t set;		/* blocked signal set */
	sigset_t cmpset;	/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	pathok = 0;

	DBUG_ENTER("sig1_catch");

	sig_recvd++;

	DBUG_PRINTF("sig1_catch", "sig_recvd %d", sig_recvd);

	/* 
	 * Check the mask contains signal caught, signals currently blocked
	 * and signals masked via sigaction()
	 * and that SIGSTOP and SIGKILL are not in the mask.
	 */
	ret = sigprocmask(SIG_SETMASK, (sigset_t *)NULL, &set);
	if(ret != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returns incorrect value %d", ret);
		sig_err++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sigemptyset(&cmpset);
	(void) sigaddset(&cmpset, SIGTERM);
	(void) sigaddset(&cmpset, SIGINT);
	(void) sigaddset(&cmpset, SIGTTIN);
	(void) sigaddset(&cmpset, SIGTTOU);
	if(cmp_mask(&set, &cmpset) == FAIL) {
		in_rpt("new signal mask does not contain signal caught,");
		in_rpt("signals currently blocked, and specified by sigaction()");
		sig_err++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
sig2_catch()
{
	sigset_t set;		/* blocked signal set */
	sigset_t cmpset;	/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	pathok = 0;

	DBUG_ENTER("sig2_catch");

	globok = 0;

	sig_recvd++;

	DBUG_PRINTF("sig2_catch", "sig_recvd %d", sig_recvd);

	/* Check the signal mask can be altered by sigprocmask() */
	/* Set up new mask for basic signals blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTSTP);
	(void) sigaddset(&set, SIGHUP);
	ret = sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL);
	if(ret != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returns incorrect value %d", ret);
		sig_err++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	ret = sigprocmask(SIG_SETMASK, (sigset_t *)NULL, &set);
	if(ret != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returns incorrect value %d", ret);
		sig_err++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sigemptyset(&cmpset);
	(void) sigaddset(&cmpset, SIGHUP);
	(void) sigaddset(&cmpset, SIGTSTP);
	if(cmp_mask(&set, &cmpset) == FAIL) {
		in_rpt("signal mask incorrectly altered by sigprocmask()");
		sig_err++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
sig3_catch()
{
	sigset_t set;		/* blocked signal set */

	DBUG_ENTER("sig3_catch");

	/* Check the signal mask can be altered by sigsuspend() */
	/* This is checked in a second signal catching function sig_level2 */

	/* Set up new mask for basic signals blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTSTP);
	(void) sigaddset(&set, SIGHUP);

	/* Tell child to signal (us after a short delay) */
	(void) sleep(WAITTIME);

	(void) kill(tet_child, SIGPIPE);

	(void) sigsuspend(&set);

	DBUG_VOID_RETURN;
}

private void
sig_level2()
{
	sigset_t set;		/* blocked signal set */
	sigset_t cmpset;	/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	pathok = 0;

	DBUG_ENTER("sig_level2");

	globok = 0;

	sig_recvd++;

	/* Check the signal mask can be altered by sigsuspend() */
	ret = sigprocmask(SIG_SETMASK, (sigset_t *)NULL, &set);
	if(ret != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returns incorrect value %d", ret);
		sig_err++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the blocked signal mask includes SIGPIPE but not SIGTERM */
	(void) sigemptyset(&cmpset);
	(void) sigaddset(&cmpset, SIGHUP);
	(void) sigaddset(&cmpset, SIGTSTP);
	(void) sigaddset(&cmpset, SIGPIPE);
	if(cmp_mask(&set, &cmpset) == FAIL) {
		in_rpt("signal mask incorrectly altered by sigsuspend()");
		sig_err++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	DBUG_ENTER("test8");

	(void) cppair(ch_t8, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t8()
{
	sigset_t set;		/* blocked signal set */
	sigset_t cmpset;	/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	pathok = setupok;

	DBUG_ENTER("ch_t8");

	test_fail = 0;
	parent = getpid();

	test_arg = 1;
	globok = 0;
	do_sig_catch();
	if(test_fail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* 
	 * Check the mask contains original information and NOT the stuff
	 * masked out in the signal handling function.
	 */
	ret = sigprocmask(SIG_SETMASK, (sigset_t *)NULL, &set);
	if(ret != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returns incorrect value %d", ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) sigemptyset(&cmpset);
	(void) sigaddset(&cmpset, SIGINT);
	(void) sigaddset(&cmpset, SIGTTIN);
	if(cmp_mask(&set, &cmpset) == FAIL) {
		in_rpt("original signal mask not restored after signal");
		in_rpt("handling function returned");
		sig_err++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(3);
	
	DBUG_VOID_RETURN;
}

private void
test9()
{
	DBUG_ENTER("test9");

	(void) cpsig(ch_t9, pa_t9, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
pa_t9()
{
	int	err;
	int	ret;		/* Used for most return values */
	int	pathok = setupok;

	DBUG_ENTER("pa_t9");

	test_fail = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Set signal catching function for SIGCHLD */
	/* Call sigaction() for SIGCHLD with SA_NOCLDSTOP bit clear */
	if(sig_init(SIGCHLD, sig_mask(SIGINT), sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Send SIGSTOP to child process */
	sig_recvd = 0;
	ret = kill(tet_child, SIGSTOP);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		test_fail++;
	}
	else
		PATH_TRACE;

	(void) sleep(WAITTIME);

	/* Check we received signal */
	if(!test_fail && sig_recvd != SIGCHLD) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() called with SA_NOCLDSTOP clear did not");
		in_rpt("generate SIGCHLD signal when child process is stopped");
		if(sig_recvd)
			in_rpt("signal %d (%s) was generated", 
						sig_recvd, sig_name[sig_recvd]);

		test_fail++;
	}
	else
		PATH_TRACE;

	/* Set default for SIGCHLD so we get no more interruptions */
	if(sig_init(SIGCHLD, sig_mask(SIGINT), SIG_DFL, 0) < 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(5);

	globok = 1; /* set for cppair */

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	/*
	 * Child process which should receive a stop signal
	 * This process should then get tidied up by cppair()
	 */
	DBUG_ENTER("ch_t9");

	(void) sleep(WAITTIME);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	DBUG_ENTER("test10");

	(void) cpsig(ch_t10, pa_t10, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
pa_t10()
{
	int	err;
	int	ret;		/* Used for most return values */
	int	pathok = setupok;

	DBUG_ENTER("pa_t10");

	test_fail = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Set signal catching function for SIGCHLD */
	/* Call sigaction() for SIGCHLD with SA_NOCLDSTOP bit set */
	if(sig_init(SIGCHLD, sig_mask(SIGINT), sig_catch, SA_NOCLDSTOP) < 0) {
		xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Send SIGSTOP to child process */
	sig_recvd = 0;
	ret = kill(tet_child, SIGSTOP);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		test_fail++;
	}
	else
		PATH_TRACE;

	(void) sleep(WAITTIME);

	/* Check we received no signal */
	if(!test_fail && sig_recvd == SIGCHLD) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() called with SA_NOCLDSTOP set generated");
		in_rpt("SIGCHLD signal when child process stopped");
		test_fail++;
	}
	else
		PATH_TRACE;

	/* Set default for SIGCHLD so we get no more interruptions */
	if(sig_init(SIGCHLD, sig_mask(SIGINT), SIG_DFL, 0) < 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(5);

	globok = 1; /* set for cppair */

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	/*
	 * Child process which should receive a SIGSTOP signal, then a SIGCONT
	 */
	DBUG_ENTER("ch_t10");

	(void) sleep(WAITTIME);

	DBUG_VOID_RETURN;
}

private void
test11()
{
	DBUG_ENTER("test11");

	(void) cppair(ch_t11, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	/*
	 * This test proves that the action installed for signal SIGCHLD 
	 * is unaltered by receipt of the signal, ie. the signal can be
	 * re-sent within the signal catching function.
	 *
	 * It's philosophically very hard to test that the action installed is
	 * unaltered by any other sequence of events because this is such a
	 * wide open category.
	 */
	int	err;
	int	ret;		/* Used for most return values */
	int	pathok = setupok;

	DBUG_ENTER("ch_t11");

	test_fail = 0;

	myself = getpid();	/* set this before sig_recur() is involved */

	/* Call sigaction() to initialise signal handling function */
	if(sig_init(SIGCHLD, sig_mask(SIGINT), sig_recur, SA_NOCLDSTOP) < 0) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() failed initialising signal action");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check the signal handling is not altered */
	sig_recvd = 0;
	ret = kill(myself, SIGCHLD);
	if(test_fail)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(sig_recvd < 3) {
		xx_rpt(FAILURE);
		in_rpt("installed action is altered within signal handling");
		in_rpt("function");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
sig_recur()
{
	int	err;
	sigset_t set;		/* blocked signal set */
	int	ret;		/* Used for most return values */

	DBUG_ENTER("sig_recur");

	sig_recvd++;

	DBUG_PRINTF("sig1_catch", "sig_recvd %d", sig_recvd);

	if(sig_recvd >= 3)
		DBUG_VOID_RETURN;

	(void) sigemptyset(&set);
	ret = sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL);
	if(ret != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returns incorrect value %d", ret);
		sig_err++;
		DBUG_VOID_RETURN;
	}

	ret = kill(myself, SIGCHLD);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		sig_err++;
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

private void
test12()
{
	DBUG_ENTER("test12");

#if defined(_XOPEN_SOURCE) || TEST_ANSI || ANSI_SIGNAL_FUNC
	(void) cppair(ch_t12, NULLFN, WAITTIME, 0);
#else
	xx_rpt(UNSUPPORTED);
	in_rpt("ANSI signal() not supported");
#endif

	DBUG_VOID_RETURN;
}

#if defined(_XOPEN_SOURCE) || TEST_ANSI || ANSI_SIGNAL_FUNC
private void
ch_t12()
{
	int	err;
	int	ret;		/* Used for most return values */
	struct sigaction act;	/* signal action structure */
	struct sigaction oact;	/* signal action structure */
	int	pathok = setupok;

	DBUG_ENTER("ch_t12");

	/* Call signal(2) to install signal handling function */
	if(signal(SIGCHLD, sig_catch) == SIG_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - signal() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Call sigaction to alter action and save old action */
	globok = 0;
	fill_struct(&act);
	PATH_FUNC_TRACE;
	act.sa_handler = sig_alter;
	act.sa_flags = 0;
	(void) sigemptyset(&(act.sa_mask));
	ret = sigaction(SIGCHLD, &act, &oact);
	if(ret < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the signal handling is altered to new catching function */
	sig_new = 0;
	myself = getpid();
	ret = kill(myself, SIGCHLD);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(!sig_new) {
		xx_rpt(DELETION);
		in_rpt("sigaction() did not alter signal handling");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Re-establish the original signal handling function */
	ret = sigaction(SIGCHLD, &oact, (struct sigaction *)NULL);
	if(ret < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the signal handling is altered to old catching function */
	sig_new = 0;
	sig_recvd = 0;
	ret = kill(myself, SIGCHLD);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(sig_new || !sig_recvd) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() did not re-establish signal handling");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(8);

	DBUG_VOID_RETURN;
}
#endif

private void
test13()
{
	DBUG_ENTER("test13");

	(void) cppair(ch_t13, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t13()
{
	int	ret;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t13");

	errno = NOERROR;
	ret = sig_init(sig_array[VSIGKILL], &sig_empty, SIG_DFL, 0);
	err = errno;

#ifdef	TESTBED
	ret = -1;
	err = EINVAL;
#endif

	if(!(ret == 0 || (ret == -1 && err == EINVAL))) {
		xx_rpt(FAILURE);
		in_rpt("sigaction(SIGKILL) returned wrong value when attempting to set action to SIG_DFL");
		in_rpt("Expecting return value of 0, or -1 and errno set to EINVAL");
		in_rpt("Return value: %d", ret);
		in_rpt("Error number: %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test14()
{
	DBUG_ENTER("test14");

	(void) cppair(ch_t14, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t14()
{
	int	ret;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("ch_t14");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	ret = sig_init(sig_array[VSIGSTOP], &sig_empty, SIG_DFL, 0);
	err = errno;

#ifdef	TESTBED
	ret = -1;
	err = EINVAL;
#endif

	if(!(ret == 0 || (ret == -1 && err == EINVAL))) {
		xx_rpt(FAILURE);
		in_rpt("sigaction(SIGSTOP) returned wrong value when attempting to set action to SIG_DFL");
		in_rpt("Expecting return value of 0, or -1 and errno set to EINVAL");
		in_rpt("Return value: %d", ret);
		in_rpt("Error number: %d (%s)", err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test15()
{
	int	ret;
	int	err;
	int	pathok = setupok;

	DBUG_ENTER("test15");

	errno = NOERROR;
	ret = sig_init(sig_array[VSIGINVAL], &sig_empty, SIG_DFL, 0);
	err = errno;

	if(ret != SYSERROR || err != EINVAL) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() returned wrong value with invalid signal number");
		valrpt(SYSERROR, ret, EINVAL, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	int	pathok = setupok;

	DBUG_ENTER("test16");

	test_fail = 0;
	test_deleted = 0;

	sig_index = VSIGKILL;
	sig_arg = SIG_IGN;

	globok = 0;
	if(cpsig(ch_t16and17, pa_t16and17, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig_index = VSIGKILL;
	sig_arg = sig_catch;

	globok = 0;
	if(cpsig(ch_t16and17, pa_t16and17, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test17()
{
	int	pathok = setupok;

	DBUG_ENTER("test17");

	test_fail = 0;
	test_deleted = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig_index = VSIGSTOP;
	sig_arg = SIG_IGN;

	globok = 0;
	if(cpsig(ch_t16and17, pa_t16and17, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	sig_index = VSIGSTOP;
	sig_arg = sig_catch;

	globok = 0;
	if(cpsig(ch_t16and17, pa_t16and17, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch_t16and17()
{
	/*
	 * Call sigaction and expect it to fail with errno set to EINVAL
	 *
	 * Return an exit code if this did not happen.
	 *
	 * Send the signal to this process, parent checks that default action
	 * is observed.
	 */
	int	ret;
	int	err;
	char *	args;
	int	sg_num = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];

	DBUG_ENTER("ch_t16and17");

	if(sig_arg == SIG_IGN)
		args = "SIG_IGN";
	else if (sig_arg == SIG_DFL)
		args = "SIG_DFL";
	else
		args = "signal catching function";

	errno = NOERROR;
	ret = sig_init(sg_num, &sig_empty, sig_arg, 0);
	err = errno;

	if(ret != SYSERROR || err != EINVAL) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigaction() returned wrong value setting action to %s", 
									args);
		in_rpt("for signal %d (%s)", sg_num, sg_name);
		valrpt(SYSERROR, ret, EINVAL, err);
		DBUG_EXIT(A_SIGFAIL);
	}

	sig_recvd = 0;

	(void) kill(getpid(), sg_num);

	/* Exit code is signal number we received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t16and17()
{
	int	err;
	int	stat_child;
	char *	str_expect;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	sg_type = sig_type[sg_num];
	char	exp_buf[BUFSIZ];
	int	pathok = 0;

	DBUG_ENTER("pa_t16and17");

	sig_err = 0;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(DELETION);
		err = errno;
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check return value of sigaction() */
	if(stat_child == CHLD_EXIT && ex_code == A_SIGFAIL) {
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Switch on what default action is expected. */
	switch(sg_type) {
	case TYPE_ABRT:	/* child aborts on receiving signal */
		(void) sprintf(exp_buf, "signal %d terminated process", sg_num);
		str_expect = exp_buf;
		if(stat_child != CHLD_ABRT)
			sig_err++;
		else
			PATH_TRACE;
		if(sg_code != sg_num)
			sig_err++;
		else
			PATH_TRACE;
		break;
	case TYPE_IGNR:	/* child ignores so exits normally */
	case TYPE_CONT:	/* here, child ignores SIGCONT */
		str_expect = "process ignored signal";
		if(stat_child != CHLD_EXIT)
			sig_err++;
		else
			PATH_TRACE;
		if(ex_code != 0)
			sig_err++;
		else
			PATH_TRACE;
		break;
	case TYPE_STOP:	/* child process stops */
		PATH_TRACE;
		str_expect = "signal stopped process";
		if(stat_child != CHLD_STOP)
			sig_err++;
		else
			PATH_TRACE;
		break;
	}

	if(sig_err) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %d (%s) caused wrong action after sigaction()",
							sg_num, sg_name);
		in_rpt("expected action: %s", str_expect);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);	/* set globok for cpsig */

	DBUG_VOID_RETURN;
}

private void
fill_struct(s_to_fill)
struct	sigaction	*s_to_fill;
{
	int	pathok = 0;
	DBUG_ENTER("fill_struct");

	/* Use memset() to fill the sigaction structure with data (S's) */
	DBUG_PRINTF("fill_struct","Filling %d bytes",  (int) sizeof(*s_to_fill));
	
	(void)  memset((void *) s_to_fill, 'S',  sizeof(*s_to_fill));
	PATH_TRACE;

	PATH_FUNC_RPT(1);
	DBUG_VOID_RETURN;
}
