/*
*      SCCS:  @(#)POSIX.os/procprim/sigaddset/sigaddset.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/sigaddset - T.sigaddset Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.sigaddset

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the sigaddset() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Wed Mar  1 11:31:40 GMT 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG

		Peter Lee, Unisoft Ltd.,
		Fri Aug 10 13:46:35 BST 1990
		Added resilience code.

		Geoff Clare, 27 June 1991
			VSX_SIGSET_EINVAL="N" for untested result

		Neil Moses, 13 May 1992
			For test 2, check that signal set is left 
			unchanged on failure.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<dbug.h>
#include	<report.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigaddset
#endif


#define	NO_TESTS		2		/* number of tests in testset */

extern	void		vsx_sigs();
extern	void		valrpt();

extern	int		sig_array[];

int	no_tests = NO_TESTS;

private	int	nsigs;	/* How many signals */
private	int	nfails;
private int	setupok;

private	void		test1();
private	void		test2();
private	void		prepare_tests();

private	void		save_sigs();
private	int		cmp_sigs();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private	void
prepare_tests()
{
	int	ret;	/* Used for most return values */
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1)
		nsigs = VNOJSIG;
	else
		nsigs = VNUMSIG;

	PATH_SETUP_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	i;	/* Loop index */
	int	ret;	/* Used for most return values */
	sigset_t  set;	/* Signal set */
	int	pathok = setupok;

	DBUG_ENTER("test1");

	/* Call sigemptyset() */
	(void) sigemptyset(&set);
	
	/* Check that sigaddset() adds each signal in the last */
	for(i = 1; i < nsigs; i++) {
		if(sig_array[i] == VNOSIG)
		{
			PATH_TRACE; PATH_TRACE;
			continue;
		}
		ret = sigaddset(&set, sig_array[i]);
		if(ret != 0) {
			xx_rpt(FAILURE);
			in_rpt("sigaddset() returns %d instead of 0", ret);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		ret = sigismember(&set, sig_array[i]);
		if(ret != 1) {
			xx_rpt(FAILURE);
			in_rpt("sigismember() returned %d instead of 1", ret);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	PATH_XS_RPT(2 * (nsigs-1));

	DBUG_VOID_RETURN;
}

private void
test2()
{
	char	*cp;
	int	ret;	/* Used for most return values */
	int	err;	/* saved value of errno */
	int	sigset1[NSIG];
	int	sigset2[NSIG];
	sigset_t  set;	/* Signal set */
	int	pathok = setupok;

	DBUG_ENTER("test2");

	nfails = 0;

	/* don't test if EINVAL not implemented for sigsetops */
	cp = tet_getvar("VSX_SIGSET_EINVAL");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_SIGSET_EINVAL is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'n' || *cp == 'N')
	{
		xx_rpt(UNTESTED);
		in_rpt("parameter VSX_SIGSET_EINVAL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Call sigemptyset() purely to initialise */
	(void) sigemptyset(&set);

	/* Save signal set before calling sigaddset */
	save_sigs(sigset1, &set);

	errno = NOERROR;
	ret = sigaddset(&set, sig_array[VSIGINVAL]);
	if(ret != SYSERROR || errno != EINVAL) 
	{
		err = errno;
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("sigaddset() returns wrong value using invalid signal");
		valrpt(SYSERROR, ret, EINVAL, err);
		DBUG_VOID_RETURN;
	} 
	else
		PATH_TRACE;

	/* Save signal set after calling sigaddset */
	save_sigs(sigset2, &set);

	/* Check that signal set is unchanged */
	if(cmp_sigs(sigset1, sigset2))
	{
		if(!nfails++)
			xx_rpt(FAILURE);
		in_rpt("sigaddset() on failure changed the signals set");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!nfails)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
save_sigs(buf, set)
int		*buf;
sigset_t  	*set;
{
	int	sig, i;

	DBUG_ENTER("save_sigs");

	for(i = 0, sig = 1; sig < NSIG; sig++, i++)
	{
		if(sigismember(set, sig) == 1)
			*(buf + i) = sig;
		else
			*(buf + i) = 0;
	}

	DBUG_VOID_RETURN;
}

private int
cmp_sigs(buf1, buf2)
int	*buf1;
int  	*buf2;
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("cmp_sigs");

	for(i = 0; i < (NSIG - 1); i++)
	{
		if(i == 0)
			PATH_TRACE;

		if(*(buf1 + i) != *(buf2 + i))
			DBUG_RETURN(1);

		PATH_TRACE;
	}

	PATH_FUNC_RPT(NSIG);

	DBUG_RETURN(0);
}
