/*
*      SCCS:  @(#)POSIX.os/procprim/signal/sigcon1.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.signal

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for signal concepts

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu Mar 16 12:43:44 GMT 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Thu Aug 10 12:06:28 BST 1989
		  Test child has terminated in test2() before reading pipe,
		  rather than after.

		Geoff Clare, 8 Nov 1989
			Make sure stop signals are not tested on orphaned
			process group (unless specified in the test).

		Anthony Lee, 15 August 1990
		Added resilience code.

		Stuart Boutell, 7 Nov 1990
			tet_child redeclaration removed.

		Stuart Boutell, 12  Nov 1990
			vsignal(SIGCONT, SIG_DFL) added in test 6,8

		Stuart Boutell, 15 Jan 1990
			cpsig return code check changed to SYSERROR from FAIL

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<fcntl.h>
#include	<dbug.h>
#include	<report.h>
#include	<wait_child.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<signal.h>
#include	<termios.h>
#include	<vsx_signal.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<sys/wait.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigconcept
#endif


#define	WRITETIME		(SPEEDFACTOR)
#define	WAITTIME		(5*SPEEDFACTOR)
#define	TESTTIME		(7*SPEEDFACTOR)
#define	EXTERR			"external error"
#define	READ			0
#define	WRITE			1
#define MODEANY			(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	CREATMODE		0666
#define	ZERORET			0
#define	WAIT			1
#define	WAITPID			2

extern	void		(*vsignal())();
extern	char		*sig_child();
extern	void		vsx_sigs();
extern	void		valrpt();
extern	int		cppair();
extern	char		*signame();

extern	char		*vg_ident;
extern	char		*vg_dbug;
extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sig_type[];
extern	int		ex_code;
extern	int		sg_code;

extern	void		(*sig_arg)();
extern	int		sig_index;
extern	sigset_t	sig_empty;
extern int		sig_recvd;
extern int		sig_err;
extern int		test_fail;
extern int		test_deleted;
extern int		nsigs;
extern int		job_control;

private	int		pipe_fd[2];
private	int		fd_fcntl;
private	struct flock	lock_info;
private	char		*t_lock     = "./signal-t.3lock";
private	int		test_case;

public	void		test1();
public	void		test2();
public	void		test3();
public	void		test4();
public	void		test5();
public	void		test6();
public	void		test7();
public	void		test8();
public	void		test9();
private	void		do_default();
private	void		do_cont();
private	void		ch_t2();
private	void		pa_t2();
private	void		ch_t3();
private	void		pa_t3();
private	void		ch_t4();
private	void		pa_t4();
private	void		pa_t5();
private	void		ch_t6();
private	void		pa_t6();
private	void		pa_t8();
private	void		t_init();
private	void		t_cln();

extern	void		sig_catch();
extern	void		ch_tdefault();
extern	void		pa_tdefault();

public void
test1()
{
	int	pathok = 0;
	DBUG_ENTER("test1");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	do_default(TYPE_ABRT);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

public void
test2()
{
	int	i;
	int	sg_num;
	int	pathok = 0;

	DBUG_ENTER("test2");

	/*
	 * Test that signals whose action is to terminate process 
	 * do terminate process.
	 */

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Test for all signals that have default action
	 * type which terminate the process.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		sg_num = sig_array[i];

		/* check default action for this signal */
		if(sig_type[sg_num] != TYPE_ABRT)
		{
			PATH_TRACE; PATH_TRACE;  PATH_TRACE; 
			continue;
		}

		sig_index = i;
		sig_arg = SIG_DFL;

		globok = 0;
		t_init();
		PATH_FUNC_TRACE;
		
		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE; 

		globok = 0; 
		if(cpsig(ch_t2, pa_t2, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;

		t_cln();
		
		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT((nsigs-1)*3); 

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	char	buffer[1];
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t2");

	/* Set the required action if we can */
	(void) sig_init(sg_num, &sig_empty, SIG_DFL, 0);

	sig_recvd = 0;

	/* Send ourself the signal */
	(void) kill(getpid(), sg_num);

	/* Write to the pipe which parent will read */
	(void) write(pipe_fd[WRITE], buffer, sizeof(char));

	/* Exit code is signal number we received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	char	buffer[1];
	int	ret;
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t2");

	/* Wait for the child and expect it to have exitted. */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != sg_num) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("signal %d (%s) produced wrong action", sg_num, sg_name);
		in_rpt("expected action: signal %d terminated process", sg_num);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to read from the pipe and expect read() to time out */
	SET_TIMEOUT(WAITTIME/2)
	errno = NOERROR;
	ret = read(pipe_fd[READ], buffer, sizeof(char));
	err = errno;
	CLEAR_ALARM
	if(ret != SYSERROR || err != EINTR) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("read() from pipe returned wrong value");
		in_rpt("signal %d (%s) did not terminate execution of process", 
							sg_num, sg_name);
		valrpt(SYSERROR, ret, EINTR, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	PATH_FUNC_RPT(3); 

	DBUG_VOID_RETURN;
}

public void
test3()
{
	int	i;
	int	sg_num;
	int	pathok = 0;

	DBUG_ENTER("test3");

	/*
	 * Test that abnormal termination closes all file descriptors.
	 */

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Test for all signals that have default action
	 * type which causes abnormal termination.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		sg_num = sig_array[i];

		/* check default action for this signal */
		if(sig_type[sg_num] != TYPE_ABRT)
		{
			PATH_TRACE; PATH_TRACE;
			continue;
		}

		sig_index = i;
		sig_arg = SIG_DFL;

		globok = 0;
		t_init();
		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		globok = 0; 
		if(cpsig(ch_t3, pa_t3, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;

		t_cln();
		
		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT((nsigs-1)*2);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t3");

	/* Set a lock for the parent to detect, then sleep */
	lock_info.l_type = F_WRLCK;
	lock_info.l_whence = SEEK_SET;
	lock_info.l_start = (off_t)0;
	lock_info.l_len = (off_t)1;
	(void)fcntl(fd_fcntl, F_SETLK, &lock_info);

	/* Set the required action if we can */
	(void) sig_init(sg_num, &sig_empty, SIG_DFL, 0);

	sig_recvd = 0;

	/* Sleep during which time parent examines file descriptors */
	(void) sleep(WAITTIME);

	(void) kill(getpid(), sg_num);

	/* Exit code is signal number we received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	char	buffer[1];
	int	stat_child;
	int	ret;
	int	err;
	int	sg_num  = sig_array[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t3");

	/* The child locks a file and sleeps for WAITTIME */
	(void) sleep(WAITTIME/2);

	/* Obtain information on lock file */
	lock_info.l_type = F_WRLCK;
	lock_info.l_whence = SEEK_SET;
	lock_info.l_start = (off_t)0;
	lock_info.l_len = (off_t)1;
	ret = fcntl(fd_fcntl, F_GETLK, &lock_info);
	if(ret == SYSERROR)
	{
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - fcntl() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check file really is locked */
	if(lock_info.l_pid != tet_child || lock_info.l_type != F_WRLCK) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - file descriptor not locked by child process");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != sg_num) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("expected action: signal %d terminated process", sg_num);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Initialise the SIGPIPE action */
	if(sig_init(sig_array[VSIGPIPE], &sig_empty, sig_catch, 0) < 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Close down the read side of the pipe */
	ret = close(pipe_fd[READ]);
	if(ret == SYSERROR) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - close() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Obtain information on lock file */
	lock_info.l_type = F_WRLCK;
	lock_info.l_whence = SEEK_SET;
	lock_info.l_start = (off_t)0;
	lock_info.l_len = (off_t)1;
	ret = fcntl(fd_fcntl, F_GETLK, &lock_info);
	if(ret == SYSERROR) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - fcntl() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the file is no longer locked by the child */
	if(lock_info.l_type != F_UNLCK) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("file descriptor still locked after signal received");
		test_fail++;
	}
	else
		PATH_TRACE;

	/* Try to write to the write side of the pipe */
	sig_recvd = 0;
	ret = write(pipe_fd[WRITE], buffer, sizeof(char));
	err = errno;
	if(ret != SYSERROR || err != EPIPE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("pipe file descriptor still open after signal received");
		in_rpt("write() to pipe in parent process did not return expected value");
		valrpt(SYSERROR, ret, EPIPE, err);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(sig_recvd != SIGPIPE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal SIGPIPE not received when attempting to write to pipe");
		in_rpt("which is not open for reading by any process");
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(10); 
	DBUG_VOID_RETURN;
}

public void
test4()
{
	int	i;
	int	sg_num;
	int	pathok = 0;

	DBUG_ENTER("test4");

	/*
	 * Test that termination signal is saved for waitpid() or wait().
	 * This test is very similar to test1 except the child process
	 * forces the parent to wait for a while.
	 */

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Test for all signals that have default action
	 * type which causes abnormal termination.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		sg_num = sig_array[i];

		/* check default action for this signal */
		if(sig_type[sg_num] != TYPE_ABRT)
		{
			PATH_TRACE; PATH_TRACE; 
			continue;
		}

		sig_index = i;
		sig_arg = SIG_DFL;

		test_case = WAIT;

		globok = 0; 
		if(cpsig(ch_t4, pa_t4, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;
			
		if(test_deleted)
			DBUG_VOID_RETURN;

		test_case = WAITPID;

		globok = 0; 
		if(cpsig(ch_t4, pa_t4, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;
	
		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT(2*(nsigs-1)); 

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t4");

	/* Set the required action if we can */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	sig_recvd = 0;

	/* Sleep a while to force the parent to wait */
	(void) sleep(WAITTIME/2);

	(void) kill(getpid(), sg_num);

	/* Exit code is signal number we received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t4()
{
	int	stat_child;
	int	ret;
	int	err;
	int	status;
	int	sg_num  = sig_array[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t4");

	/* Wait for child and expect it to exit after WAITTIME/2  */
	errno = NOERROR;
	SET_TIMEOUT(WAITTIME)
	switch(test_case) {
	case WAITPID:
		ret = waitpid(tet_child, &status, WUNTRACED);
		PATH_TRACE; 
		break;
	case WAIT:
		ret = wait(&status);
		PATH_TRACE; 
		break;
	default:
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - test_case not known");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	CLEAR_ALARM
	err = errno;
	if(ret == SYSERROR)
	{
		if(err == EINTR && alrm_flag)
		{
			stat_child = CHLD_LIVE;
		}
		else
		{
			stat_child = CHLD_ERR;
		}
	}
	else if(WIFEXITED(status))
	{
		stat_child = CHLD_EXIT;
		ex_code = WEXITSTATUS(status);
	}
	else if(WIFSIGNALED(status))
	{
		stat_child = CHLD_ABRT;
		sg_code = WTERMSIG(status);
	}
	else if(WIFSTOPPED(status))
	{	
		stat_child = CHLD_STOP;
	}
	else
	{
		/* SHOULD NOT GET HERE - waitpid() WRONGLY IMPLEMENTED */
		stat_child = CHLD_ERR;
	}

	if(stat_child == CHLD_ERR) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != sg_num) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal number not made available for parent process");
		in_rpt("expected action: signal %d terminated process");
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3); 

	DBUG_VOID_RETURN;
}

public void
test5()
{
	int	i;
	int	sg_num;
	int	pathok = 0;

	DBUG_ENTER("test5");

	/*
	 * Test that termination signal is saved for waitpid() or wait().
	 * This test is very similar to test1 except the parent process
	 * waits a while before waiting for the child process.
	 */

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Test for all signals that have default action
	 * type which causes abnormal termination.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		sg_num = sig_array[i];

		/* check default action for this signal */
		if(sig_type[sg_num] != TYPE_ABRT)
		{
			PATH_TRACE; PATH_TRACE; 
			continue;
		}

		sig_index = i;
		sig_arg = SIG_DFL;

		test_case = WAIT;

		globok = 0; 
		if(cpsig(ch_tdefault, pa_t5, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		test_case = WAITPID;

		globok = 0; 
		if(cpsig(ch_tdefault, pa_t5, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		else
			PATH_FUNC_TRACE;
	
		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT((nsigs-1)*2); 

	DBUG_VOID_RETURN;
}

private void
pa_t5()
{
	int	stat_child;
	int	ret;
	int	err;
	int	status;
	int	sg_num  = sig_array[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t5");

	/* Sleep whilst child terminates abnormally */
	(void) sleep(WAITTIME/2);

	/* Wait for child and expect it to have exitted */
	errno = NOERROR;
	SET_TIMEOUT(WAITTIME)
	switch(test_case) {
	case WAITPID:
		ret = waitpid(tet_child, &status, WUNTRACED);
		PATH_TRACE; 
		break;
	case WAIT:
		ret = wait(&status);
		PATH_TRACE; 
		break;
	default:
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - test_case not known");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	CLEAR_ALARM
	err = errno;
	if(ret == SYSERROR) {
		if(err == EINTR && alrm_flag) {
			stat_child = CHLD_LIVE;
		} else {
			stat_child = CHLD_ERR;
		}
	} else if(WIFEXITED(status)) {
		stat_child = CHLD_EXIT;
		ex_code = WEXITSTATUS(status);
	} else if(WIFSIGNALED(status)) {
		stat_child = CHLD_ABRT;
		sg_code = WTERMSIG(status);
	} else if(WIFSTOPPED(status)) {
		stat_child = CHLD_STOP;
	} else {
		/* SHOULD NOT GET HERE - waitpid() WRONGLY IMPLEMENTED */
		stat_child = CHLD_ERR;
	}

	if(stat_child == CHLD_ERR) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != sg_num) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal number not saved for parent process");
		in_rpt("expected action: signal %d terminated process");
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3); 
	DBUG_VOID_RETURN;
}

public void
test6()
{
	int	i;
	int	sg_num;
	int	pathok = 0;

	DBUG_ENTER("test6");
	/*
	 * Test that stop signal suspends process execution.
	 */

	test_fail = 0;
	test_deleted = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test for all signals that have default action
	 * type which stop the process.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		sg_num = sig_array[i];

		/* check default action for this signal */
		if(sig_type[sg_num] != TYPE_STOP)
		{
			PATH_TRACE; PATH_TRACE;
			continue;
		}

		sig_index = i;
		sig_arg = SIG_DFL;

		globok = 0;
		t_init();
		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		PATH_FUNC_TRACE;

		(void) vsignal(SIGCONT, SIG_DFL);
		globok = 0; 
		if(cpsig(ch_t6, pa_t6, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;

		t_cln();
		
		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT(1+ ((nsigs-1)*2) );

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	char	buffer[1];
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t6");

	/* make sure the process is not in an orphaned process group */
	(void) setpgid((pid_t)0, (pid_t)0);

	/* Set the required action if we can */
	(void) sig_init(sg_num, &sig_empty, SIG_DFL, 0);

	sig_recvd = 0;

	/* Send ourself the stop signal */
	(void) kill(getpid(), sg_num);

	/* Write to the pipe which parent will read */
	(void) write(pipe_fd[WRITE], buffer, sizeof(char));

	/* Exit code is signal number we received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t6()
{
	char	buffer[1];
	int	ret;
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t6");

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Expect the SIGSTOP signal to have stopped the child process */
	if(stat_child != CHLD_STOP) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - signal %s caused wrong action", sg_name);
		in_rpt("expected action: signal stopped process");
		in_rpt("observed action: %s", sig_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to read from the pipe and expect read() to time out */
	SET_TIMEOUT(WAITTIME/2)
	errno = NOERROR;
	ret = read(pipe_fd[READ], buffer, sizeof(char));
	err = errno;
	CLEAR_ALARM
	if(ret != SYSERROR || err != EINTR) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("read() from pipe returned wrong value");
		in_rpt("signal %d (%s) did not suspend execution of process", 
							sg_num, sg_name);
		valrpt(SYSERROR, ret, EINTR, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Now switch over to SIGCONT and try to continue the stopped process */
	sig_index = VSIGCONT;

	sg_num  = sig_array[sig_index];
	sg_name = sig_name[sig_index];

	/* Send the SIGCONT signal */
	if(kill(tet_child, sg_num) < 0 ) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Read from the pipe again and expect success */
	SET_TIMEOUT(WAITTIME/2)
	errno = NOERROR;
	ret = read(pipe_fd[READ], buffer, sizeof(char));
	err = errno;
	CLEAR_ALARM
	if(ret != sizeof(char) || err != NOERROR) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("read() from pipe returned %d instead of %d", 
							ret, sizeof(char));
		in_rpt("signal %d (%s) did not restart suspended process", 
							sg_num, sg_name);
		valrpt(sizeof(char), ret, NOERROR, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait for the child and expect it to have exitted. */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s did not cause process to continue", sg_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	PATH_FUNC_RPT(7); 

	DBUG_VOID_RETURN;
}

public void
test7()
{
	int	pathok = 0;
	DBUG_ENTER("test7");

	test_fail = 0;
	test_deleted = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	do_default(TYPE_IGNR);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

public void
test8()
{
	int	pathok = 0;
	DBUG_ENTER("test8");

	test_fail = 0;
	test_deleted = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Test that SIGCONT is ignored when process not stopped */

	globok = 0;
	do_default(TYPE_CONT);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	/* Test that SIGCONT restarts process which is stopped */

	(void) vsignal(SIGCONT, SIG_DFL);
	globok = 0;
	do_cont();
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}

public void
test9()
{
	int	pathok = 0;
	DBUG_ENTER("test9");

	test_fail = 0;
	test_deleted = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	do_default(TYPE_STOP);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
do_default(arg)
int	arg;	/* what default action type to test */
{
	int	i;
	int	sg_num;
	char *	sg_name;
	int	pathok = 0;

	DBUG_ENTER("do_default");

	/*
	 * Test action requested for all signals that have default action
	 * type which matches "arg".
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		sg_num = sig_array[i];
		sg_name = sig_name[i];

		/* check default action for this signal */
		if(sig_type[sg_num] != arg)
		{
			PATH_TRACE; 
			continue;
		}

		DBUG_PRINTF("do_default", "signal %s", sg_name);

		sig_index = i;

		sig_arg = SIG_DFL;

		globok = 0; 
		if(cpsig(ch_tdefault, pa_tdefault, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;
			
		if(test_deleted)
			DBUG_VOID_RETURN;
	}
	PATH_FUNC_RPT(nsigs-1); 
	DBUG_VOID_RETURN;
}

private void
do_cont()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("do_cont");

	/*
	 * Test action of SIGCONT when process is stopped.
	 * First of all set up a stopped process using ch_tdefault.
	 * Then send SIGCONT signal in parent process using pa_t8.
	 */
	i = VSIGSTOP;

	sig_index = i;
	sig_arg = SIG_DFL;

	globok = 0; 
	if(cpsig(ch_tdefault, pa_t8, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
pa_t8()
{
	int	err;
	int	stat_child;
	char *	str_expect;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t8");

	sig_err = 0;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Expect the SIGSTOP signal to have stopped the child process */
	str_expect = "signal stopped process";
	if(stat_child != CHLD_STOP)
		sig_err++;

	if(sig_err) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - signal %s caused wrong action", sg_name);
		in_rpt("expected action: %s", str_expect);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Now switch over to SIGCONT and try to continue the stopped process */
	sig_index = VSIGCONT;

	sg_num  = sig_array[sig_index];
	sg_name = sig_name[sig_index];

	/* Send the SIGCONT signal */
	if(kill(tet_child, sg_num) < 0 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait for the child and expect it to have exitted. */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s did not cause process to continue", sg_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5); 

	DBUG_VOID_RETURN;
}

private void
t_init()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("t_init");

	/* Create and open a lock file */
	if((fd_fcntl = creat(t_lock, CREATMODE)) < 0) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: creat() failed - errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Open a pipe before forking */
	if(pipe(pipe_fd) < 0) {
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	PATH_FUNC_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
t_cln()
{
	DBUG_ENTER("t_cln");

	(void) close(pipe_fd[READ]);
	(void) close(pipe_fd[WRITE]);
	(void) close(fd_fcntl);
	(void) unlink(t_lock);

	DBUG_VOID_RETURN;
}
