/*
*      SCCS:  @(#)POSIX.os/procprim/signal/sigcon2.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.signal

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for signal concepts

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu Mar 16 12:43:44 GMT 1989
MODIFICATIONS:
		Geoff Clare, 22 Mar 1990
			Ensure child in test10 is not in an
			orphaned process group.

		Geoff Clare, 24 Apr 1990
			Ensure second child in test10 is not in an
			orphaned process group.

		Anthony Lee, 15 August 1990
		Added resilience code.

		Stuart Boutell, 7 Nov 1990
			tet_child redeclaration removed.


		Stuart Boutell, 4 Dec 1990
			Added prep_sigs() call to test 10;

		Stuart Boutell, 15 Jan 1990
			Change cpsig return code check to SYSERROR from FAIL

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<wait_child.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<sys/wait.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigconcept
#endif


#define	WRITETIME		(SPEEDFACTOR)
#define	WAITTIME		(5*SPEEDFACTOR)
#define	TESTTIME		(7*SPEEDFACTOR)
#define	EXTERR			"external error"
#define	READ			0
#define	WRITE			1
#define MODEANY			(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	CREATMODE		0666
#define	ZERORET			0
#define	WAIT			1
#define	WAITPID			2
#define	TEST_END		0
#define	TEST_HANG		1
#define	TEST_WAIT		2

/* Actions just for test 10 */
#define	TEST_UNBLOCK	1
#define	TEST_IGNORE	2

extern	char		*sig_child();
extern	void		vsx_sigs();
extern	void		valrpt();
extern	int		cppair();
extern	char		*signame();

extern	char		*vg_ident;
extern	char		*vg_dbug;
extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sig_type[];
extern	int		ex_code;
extern	int		sg_code;

extern	void		prep_sigs();
extern	void		(*sig_arg)();
extern	int		sig_index;
extern	sigset_t	sig_empty;
extern int		sig_recvd;
extern int		sig_err;
extern int		test_fail;
extern int		test_deleted;
extern int		nsigs;
extern int		job_control;

private int		sig_new;
private int		test_arg;

public	void		test10();
public	void		test11();
public	void		test12();
public	void		test13();
public	void		test14();
public	void		test15();
public	void		test16();
private	void		do_10catch();
private	void		ch_t10catch();
private	void		pa_t10catch();
private	void		do_10default();
private	void		ch_t10default();
private	void		pa_t10default();
private	void		do_11();
private	void		ch_t11();
private	void		pa_t11();
private	void		ch_t12();
private	void		ch_t13();
private	void		pa_t13();
private	void		ch_t14();
private	void		ch_t15();
private	void		pa_t15();
private	void		ch_t16unblock();
private	void		pa_t16unblock();
private	void		ch_t16stop();
private	void		pa_t16stop();
private	void		sig_stop();
private	void		sig_alter();

extern	int		sigset_ck();
extern	void		sig_catch();

public void
test10()
{
	int	pathok = 0;
	DBUG_ENTER("test10");

	prep_sigs();

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	do_10catch(TEST_UNBLOCK);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	do_10catch(TEST_IGNORE);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	do_10default(TEST_UNBLOCK);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	do_10default(TEST_IGNORE);
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
do_10catch(arg)
int	arg;		/* Either TEST_UNBLOCK or TEST_IGNORE */
			/* action if any to be taken before unblocking signal */
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("do_10catch");

	/*
	 * Test that signals which are blocked give the required action
	 * when unblocked. Initially signals are set up to be caught, 
	 * done in function sig_ck() where the signal is also generated.
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		/* Can't catch these signals */
		if(i == VSIGKILL || i == VSIGSTOP)
		{
			PATH_TRACE; 
			continue;
		}

		sig_index = i;
		test_arg = arg;

		globok = 0; 
		if(cpsig(ch_t10catch, pa_t10catch, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;

		if(test_deleted)
			DBUG_VOID_RETURN;
	}
	PATH_FUNC_RPT(nsigs-1); 
	DBUG_VOID_RETURN;
}

private void
ch_t10catch()
{
	sigset_t set;
	int	ret;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];

	DBUG_ENTER("ch_t10catch");

	/* make sure the process is not in an orphaned process group */
	(void) setpgid((pid_t)0, (pid_t)0);

	/* Call sigaction() to initialise signal handling function */
	if(sig_init(sg_num, &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed - signal %d (%s)",
						sg_num, sg_name);
		DBUG_EXIT(A_TSETDIR);
	}

	/* Call sigprocmask() to block the signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_EXIT(A_TSETDIR);
	}
	
	/* Check the signal is blocked */
	sig_recvd = 0;
	ret = kill(getpid(), sg_num);
	if(ret < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		DBUG_EXIT(A_TSETDIR);
	}
	if(sig_recvd == sg_num) {
		xx_rpt(DELETION);
		in_rpt("signal %d (%s) is unblocked, should be blocked", 
						sg_num, sg_name);
		DBUG_EXIT(A_TSETDIR);
	}

	sig_recvd = 0;

	if(test_arg == TEST_IGNORE) {
		/* ignore the signal before unblocking. */
		if(sig_init(sg_num, &sig_empty, SIG_IGN, 0) < 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigaction() failed to set");
			in_rpt("action SIG_IGN for signal %d (%s)",
							sg_num, sg_name);
			DBUG_EXIT(A_TSETDIR);
		}
	}

	/* Call sigprocmask() to unblock signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if((ret = sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL)) != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returned %d instead of 0", ret);
		DBUG_EXIT(A_TSETDIR);
	}

	/*
	 * Check signal is received.
	 */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t10catch()
{
	int	err;
	int	stat_child;
	char *	str_expect;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	char	exp_buf[BUFSIZ];
	int	pathok = 0;

	DBUG_ENTER("pa_t10catch");

	sig_err = 0;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/*
	 * Switch on what action was taken before unblocking the signal.
	 */
	if(test_arg == TEST_IGNORE)
	{
		(void) sprintf(exp_buf, "child ignored signal %d whilst blocked", sg_num);
		str_expect = exp_buf;
		if(stat_child != CHLD_EXIT)
		{
			sig_err++;
		}
		else
		{
			if(ex_code == A_TSETDIR)
				test_deleted++;
			if(ex_code != 0)
				sig_err++;
		}
	}
	else
	{
		(void) sprintf(exp_buf, "child caught signal %d after unblocking signal", sg_num);
		str_expect = exp_buf;
		if(stat_child != CHLD_EXIT)
		{
			sig_err++;
		}
		else
		{
			if(ex_code == A_TSETDIR)
				test_deleted++;
			if(ex_code != sg_num)
				sig_err++;
		}
	}

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(sig_err) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caused wrong action", sg_name);
		in_rpt("expected action: %s", str_expect);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
do_10default(arg)
int	arg;		/* Either TEST_UNBLOCK or TEST_IGNORE */
			/* action if any to be taken before unblocking signal */
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("do_10default");

	/*
	 * Test that signals which are blocked give the required action
	 * when unblocked. Initially signals are set up for default action.
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		/* Can't block these signals */
		if(i == VSIGKILL || i == VSIGSTOP)
		{
			PATH_TRACE; PATH_TRACE; 
			continue;
		}

		/* Can't use this since it's action alters in sleep() */
		if(i == VSIGALRM)
		{
			PATH_TRACE; PATH_TRACE; 
			continue;
		}

		/* Can't use this since it can't be ignored */
		if(i == VSIGCONT)
		{
			PATH_TRACE; PATH_TRACE; 
			continue;
		}

		sig_index = i;
		test_arg = arg;

		globok = 0; 
		if(cpsig(ch_t10default, pa_t10default, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;

		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	PATH_FUNC_RPT((nsigs-1)*2); 
	DBUG_VOID_RETURN;
}

private void
ch_t10default()
{
	sigset_t set;
	int	ret;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];

	DBUG_ENTER("ch_t10default");

	/* make sure the process is not in an orphaned process group */
	(void) setpgid((pid_t)0, (pid_t)0);

	/* Set the default action if possible */
	(void) sig_init(sg_num, &sig_empty, SIG_DFL, 0);

	/* Call sigprocmask() to block the signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_EXIT(A_TSETDIR);
	}
	
	/* Check the signal is blocked by sending it. */
	sig_recvd = 0;
	ret = kill(getpid(), sg_num);
	if(ret < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		DBUG_EXIT(A_TSETDIR);
	}

	/* Parent should wait for WAITTIME and timeout */
	(void) sleep(2 * WAITTIME);

	if(test_arg == TEST_IGNORE) {
		/* ignore the signal before unblocking. */
		if(sig_init(sg_num, &sig_empty, SIG_IGN, 0) < 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigaction() failed to set");
			in_rpt("action SIG_IGN for signal %d (%s)",
							sg_num, sg_name);
			DBUG_EXIT(A_TSETDIR);
		}
	}

	/* Call sigprocmask() to unblock signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if((ret = sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL)) != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returned %d instead of 0", ret);
		DBUG_EXIT(A_TSETDIR);
	}

	/*
	 * Check if a signal is received and caught. This should not occur.
	 */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t10default()
{
	int	err;
	int	stat_child;
	char *	str_expect;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	sg_type  = sig_type[sg_num];
	char	exp_buf[BUFSIZ];
	int	pathok = 0;

	DBUG_ENTER("pa_t10default");

	sig_err = 0;

	/* Wait for child and expect timeout if signal is really blocked */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - signal %d (%s) not blocked", 
							sg_num, sg_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Wait again long enough for child to unblock signal */
	errno = NOERROR;
	if((stat_child = wait_child(2 * WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Switch on what action was taken before unblocking the signal.
	 */
	if(test_arg == TEST_IGNORE)
	{
		(void) sprintf(exp_buf, "child ignored signal %d whilst blocked", sg_num);
		str_expect = exp_buf;
		if(stat_child != CHLD_EXIT)
			sig_err++;
		else
		{
			if(ex_code == A_TSETDIR)
				test_deleted++;
			if(ex_code != 0)
				sig_err++;
		}
	}
	else
	{
		if(stat_child == CHLD_EXIT)
		{
			if(ex_code == A_TSETDIR)
				test_deleted++;
		}
		switch(sg_type) 
		{
		case TYPE_ABRT:	/* child aborts on receiving signal */
			(void) sprintf(exp_buf, "signal %d terminated process after being unblocked", sg_num);
			str_expect = exp_buf;
			if(stat_child != CHLD_ABRT)
				sig_err++;
			if(sg_code != sg_num)
				sig_err++;
			break;
		case TYPE_IGNR:	/* child ignores so exits normally */
		case TYPE_CONT:	/* here, child ignores SIGCONT */
			(void) sprintf(exp_buf, "process ignored signal %d after being unblocked", sg_num);
			str_expect = exp_buf;
			if(stat_child != CHLD_EXIT)
				sig_err++;
			if(ex_code != 0)
				sig_err++;
			break;
		case TYPE_STOP:	/* child process stops */
			(void) sprintf(exp_buf, "signal %d stopped process after being unblocked", sg_num);
			str_expect = exp_buf;
			if(stat_child != CHLD_STOP)
				sig_err++;
			break;
		}
	}

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(sig_err) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caused wrong action", sg_name);
		in_rpt("expected action: %s", str_expect);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);  

	DBUG_VOID_RETURN;
}

public void
test11()
{
	int	pathok = 0;
	DBUG_ENTER("test11");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	do_11(TEST_UNBLOCK);
	PATH_FUNC_TRACE;
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	do_11(TEST_IGNORE);
	PATH_FUNC_TRACE;
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
do_11(arg)
int	arg;		/* Either TEST_UNBLOCK or TEST_IGNORE */
			/* action if any to be taken before unblocking signal */
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("do_11");

	/*
	 * Test that signals which are blocked give the required action
	 * when unblocked. Initially signals are set up to be ignored. 
	 * It is unspecified whether ignored blocked signals remain pending
	 * after generation or are discarded immediately.
	 * Either way, signal shoudl never be received.
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		/* Can't ignore these signals */
		if(i == VSIGKILL || i == VSIGSTOP)
		{
			PATH_TRACE; 
			continue;
		}

		sig_index = i;
		test_arg = arg;

		globok = 0; 
		if(cpsig(ch_t11, pa_t11, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;

		if(test_deleted)
			DBUG_VOID_RETURN;
	}
	PATH_FUNC_RPT(nsigs-1); 
	DBUG_VOID_RETURN;
}

private void
ch_t11()
{
	sigset_t set;
	int	ret;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];

	DBUG_ENTER("ch_t11");

	/* Call sigaction() to initialise signal handling function */
	(void) sig_init(sg_num, &sig_empty, SIG_IGN, 0);

	/* Call sigprocmask() to block the signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_EXIT(A_TSETDIR);
	}
	
	/* Check the signal is blocked */
	sig_recvd = 0;
	ret = kill(getpid(), sg_num);
	if(ret < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		DBUG_EXIT(A_TSETDIR);
	}
	if(sig_recvd == sg_num) {
		xx_rpt(DELETION);
		in_rpt("signal %d (%s) is unblocked, should be blocked", 
						sg_num, sg_name);
		DBUG_EXIT(A_TSETDIR);
	}

	sig_recvd = 0;

	if(test_arg == TEST_IGNORE) {
		/* ignore the signal before unblocking. */
		(void) sig_init(sg_num, &sig_empty, SIG_IGN, 0);
	}

	/* Call sigprocmask() to unblock signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if((ret = sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL)) != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returned %d instead of 0", ret);
		DBUG_EXIT(A_TSETDIR);
	}

	/*
	 * Check no signal is received.
	 */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t11()
{
	int	err;
	int	stat_child;
	char *	str_expect;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	char	exp_buf[BUFSIZ];
	int	pathok = 0;

	DBUG_ENTER("pa_t11");

	sig_err = 0;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/*
	 * Switch on what action was taken before unblocking the signal.
	 */
	(void) sprintf(exp_buf, "process ignored signal %d", sg_num);
	str_expect = exp_buf;
	if(stat_child != CHLD_EXIT)
		sig_err++;
	else {
		if(ex_code == A_TSETDIR)
			test_deleted++;
		if(ex_code != 0)
			sig_err++;
	}

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(sig_err) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caused wrong action", sg_name);
		in_rpt("expected action: %s", str_expect);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3); 
	DBUG_VOID_RETURN;
}

public void
test12()
{
	DBUG_ENTER("test12");

	(void) cppair(ch_t12, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t12()
{
	sigset_t set;
	int	ret;
	int	sg_num  = sig_array[VSIGHUP];
	char *	sg_name = sig_name[VSIGHUP];
	int	pathok = 0;

	DBUG_ENTER("ch_t12");

	/* Call sigaction() to initialise signal handling function */
	if(sig_init(sg_num, &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed - signal %d (%s)",
						sg_num, sg_name);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Call sigprocmask() to block the signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Check the signal is blocked */
	sig_recvd = 0;
	sig_new = 0;
	ret = kill(getpid(), sg_num);
	if(ret < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if(sig_recvd == sg_num) {
		xx_rpt(DELETION);
		in_rpt("signal %d (%s) is unblocked, should be blocked", 
						sg_num, sg_name);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Alter signal action to new catching function */
	if(sig_init(sg_num, &sig_empty, sig_alter, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed - signal %d (%s)",
						sg_num, sg_name);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Call sigprocmask() to unblock signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if((ret = sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL)) != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returned %d instead of 0", ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Check signal is received in new signal catching function.
	 */
	if(sig_recvd == sg_num || sig_new != sg_num) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() failed to alter signal action");
		in_rpt("while signal %d (%s) pending", sg_num, sg_name);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(7); 

	DBUG_VOID_RETURN;
}

public void
test13()
{
	int	pathok = 0;
	DBUG_ENTER("test13");

	test_fail = 0;
	test_deleted = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if(cpsig(ch_t13, pa_t13, 4*WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
ch_t13()
{
	int	err;
	sigset_t set;
	int	ret;

	DBUG_ENTER("ch_t13");

	/* Call sigaction() to initialise signal handling for SIGCONT */
	(void) sig_init(SIGCONT, &sig_empty, sig_catch, 0);

	/* Call sigprocmask() to block SIGCONT */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGCONT);
	(void) sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL);

	/* Send SIGCONT signal to this process */
	sig_recvd = 0;
	if(kill(getpid(), SIGCONT) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		DBUG_EXIT(A_TSETDIR);
	}

	/* Check that SIGCONT is still pending after generation */
	if(sig_recvd == SIGCONT) {
		xx_rpt(DELETION);
		in_rpt("signal %d (%s) did not remain pending after generation",
					sig_recvd, signame(sig_recvd));
		in_rpt("but was delivered");
		DBUG_EXIT(A_TSETDIR);
	}
	if((ret = sigpending(&set)) < 0 ) {
		xx_rpt(DELETION);
		in_rpt("sigpending() returned %d instead of 0", ret);
		DBUG_EXIT(A_TSETDIR);
	}
	if(!sigismember(&set, SIGCONT)) {
		xx_rpt(DELETION);
		in_rpt("signal %d (%s) did not remain pending after generation",
					SIGCONT, signame(SIGCONT));
		in_rpt("but was discarded");
		DBUG_EXIT(A_TSETDIR);
	}

	/* Call sigaction() to initialise signal handling for SIGTSTP */
	(void) sig_init(SIGTSTP, &sig_empty, sig_stop, 0);

	/* Send signal to stop this process */
	if(kill(getpid(), SIGTSTP) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		DBUG_EXIT(A_TSETDIR);
	}

	/* Should not be reached because the process exits in sig_stop() */
	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t13()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_t13");

	sig_err = 0;

	/* Wait should report child is stopped */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT)
	{
		if(ex_code == A_MISC)
		{
			PATH_TRACE; 
			test_fail++;
		}
		else if(ex_code == A_TSETDIR)
		{
			PATH_TRACE; 
			test_deleted++;
		}
		else
			PATH_TRACE;
	} else if(stat_child == CHLD_ABRT) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected signal %d (%s) received",
						 sg_code, signame(sg_code));
		test_deleted++;
	} else if(stat_child == CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: child process timed out unexpectedly");
		test_deleted++;
	} else if(stat_child == CHLD_STOP) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: child process stopped unexpectedly");
		test_deleted++;
	} else
		PATH_TRACE; 

	PATH_FUNC_RPT(2); 
	DBUG_VOID_RETURN;
}

public void
test14()
{
	DBUG_ENTER("test14");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}

	(void) cppair(ch_t14, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t14()
{
	int	err;
	sigset_t set;		/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	pathok = 0;

	DBUG_ENTER("ch_t14");

	/* Call sigprocmask() to block some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGTSTP);
	(void) sigaddset(&set, SIGTTIN);
	(void) sigaddset(&set, SIGTTOU);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check these signals are indeed blocked */
	if(sigset_ck(&set, DELETION) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send ourself the SIGCONT signal */
	if(kill(getpid(), SIGCONT) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig_recvd = 0;

	/* Call sigprocmask() to unblock signals */
	if((ret = sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL)) != 0) {
		xx_rpt(FAILURE);
		in_rpt("sigprocmask() returned %d instead of 0", ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Check all pending signals are discarded.
	 * This relies on the fact that signals have been generated in 
	 * sigset_ck which when delivered are caught
	 */
	if(sig_recvd == SIGTSTP ||
	   sig_recvd == SIGTTIN ||
	   sig_recvd == SIGTTOU ) {
		xx_rpt(FAILURE);
		in_rpt("pending signal %d (%s) not discarded by SIGCONT signal",
					sig_recvd, signame(sig_recvd));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(5); 

	DBUG_VOID_RETURN;
}

public void
test15()
{
	DBUG_ENTER("test15");


	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}

	(void) cpsig(ch_t15, pa_t15, 4*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t15()
{
	sigset_t	set;

	DBUG_ENTER("ch_t15");

	/* Call sigaction() to initialise signal handling for SIGCONT */
	(void) sig_init(SIGCONT, &sig_empty, sig_catch, 0);

	/* Call sigprocmask() to block SIGCONT */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGCONT);
	(void) sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL);

	(void) sleep(2 * WAITTIME);

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t15()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_t15");

	/*
	 * Test that when SIGCONT is generated for a process that is stopped, 
	 * the process continues even if SIGCONT is blocked or ignored.
	 */
	sig_err = 0;

	/* Sleep to give child time to initialise */
	(void) sleep(WAITTIME);

	/* Send the stop signal */
	if(kill(tet_child, SIGSTOP) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait should report child is stopped */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is stopped */
	if(stat_child != CHLD_STOP) {
		xx_rpt(FAILURE);
		in_rpt("SIGSTOP did not stop process");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the SIGCONT signal */
	if(kill(tet_child, SIGCONT) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(2 * WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the process is no longer suspended */
	if(stat_child != CHLD_EXIT && stat_child != CHLD_ABRT) {
		xx_rpt(FAILURE);
		in_rpt("SIGCONT did not cause stopped process to resume");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(6);
	globok = 1;			/* Needed for cpsig */

	DBUG_VOID_RETURN;
}

public void
test16()
{
	int	pathok = 0;
	DBUG_ENTER("test16");

	test_fail = 0;
	test_deleted = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if(cpsig(ch_t16unblock, pa_t16unblock, 4*WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if(cpsig(ch_t16stop, pa_t16stop, 4*WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(5); 

	DBUG_VOID_RETURN;
}

private void
ch_t16unblock()
{
	int	err;
	sigset_t set;
	int	ret;

	DBUG_ENTER("ch_t16unblock");

	/* Call sigaction() to initialise signal handling for SIGCONT */
	(void) sig_init(SIGCONT, &sig_empty, sig_catch, 0);

	/* Call sigprocmask() to block SIGCONT */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGCONT);
	(void) sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL);

	/* Send signal to stop this process */
	sig_recvd = 0;
	if(kill(getpid(), SIGSTOP) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		DBUG_EXIT(A_TSETDIR);
	}

	/* Check that SIGCONT is still pending after generation */
	if(sig_recvd == SIGCONT) {
		xx_rpt(DELETION);
		in_rpt("signal %d (%s) did not remain pending after generation",
					sig_recvd, signame(sig_recvd));
		in_rpt("but was delivered");
		DBUG_EXIT(A_TSETDIR);
	}
	if((ret = sigpending(&set)) < 0 ) {
		xx_rpt(DELETION);
		in_rpt("sigpending() returned %d instead of 0", ret);
		DBUG_EXIT(A_TSETDIR);
	}
	if(!sigismember(&set, SIGCONT)) {
		xx_rpt(DELETION);
		in_rpt("signal %d (%s) did not remain pending after generation",
					SIGCONT, signame(SIGCONT));
		in_rpt("but was discarded");
		DBUG_EXIT(A_TSETDIR);
	}

	/* Call sigprocmask() to unblock signals */
	(void) sigemptyset(&set);
	if((ret = sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL)) != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returned %d instead of 0", ret);
		DBUG_EXIT(A_TSETDIR);
	}

	/* Check that SIGCONT is now received */
	if(sig_recvd != SIGCONT) {
		if(!test_fail);
			xx_rpt(FAILURE);
		/* Check if it's still pending */
		if((ret = sigpending(&set)) < 0 ) {
			xx_rpt(DELETION);
			in_rpt("sigpending() returned %d instead of 0", ret);
			DBUG_EXIT(A_TSETDIR);
		}
		if(sigismember(&set, SIGCONT))
			in_rpt("signal SIGCONT remained pending after being unblocked");
		else
			in_rpt("signal SIGCONT discarded after being unblocked");
		DBUG_EXIT(A_MISC);
	}

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t16unblock()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_t16unblock");

	sig_err = 0;

	/* Wait should report child is stopped */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT)
	{
		if(ex_code == A_MISC)
		{
			test_fail++;
			PATH_TRACE;
			PATH_FUNC_RPT(2);  /* Not deleted yet */
			DBUG_VOID_RETURN;
		}
		else if(ex_code == A_TSETDIR)
		{
			test_deleted++;
			PATH_TRACE; 
			PATH_FUNC_RPT(2);  /* Not deleted yet */
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else if(stat_child == CHLD_ABRT)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected signal %d (%s) received",
						 sg_code, signame(sg_code));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else if(stat_child == CHLD_LIVE)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: child ignored signal SIGSTOP");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the SIGCONT signal */
	if(kill(tet_child, SIGCONT) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(2 * WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT)
	{
		if(ex_code == A_MISC)
		{
			PATH_TRACE; 
			test_fail++;
		}
		else if(ex_code == A_TSETDIR)
		{
			PATH_TRACE; 
			test_deleted++;
		}
		else
			PATH_TRACE;
	}
	else if(stat_child == CHLD_ABRT)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected signal %d (%s) received",
						 sg_code, signame(sg_code));
		test_deleted++;
	}
	else if(stat_child == CHLD_LIVE)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: child process timed out unexpectedly");
		test_deleted++;
	}
	else if(stat_child == CHLD_STOP)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: SIGCONT did not continue process");
		test_deleted++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5); 

	DBUG_VOID_RETURN;
}

private void
ch_t16stop()
{
	int	err;
	sigset_t set;
	int	ret;

	DBUG_ENTER("ch_t16stop");

	/* Call sigaction() to initialise signal handling for SIGCONT */
	(void) sig_init(SIGCONT, &sig_empty, sig_catch, 0);

	/* Call sigprocmask() to block SIGCONT */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGCONT);
	(void) sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL);

	/* Send signal to stop this process */
	sig_recvd = 0;
	if(kill(getpid(), SIGSTOP) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		DBUG_EXIT(A_TSETDIR);
	}

	/* Check that SIGCONT is still pending after generation */
	if(sig_recvd == SIGCONT) {
		xx_rpt(DELETION);
		in_rpt("signal %d (%s) did not remain pending after generation",
					sig_recvd, signame(sig_recvd));
		in_rpt("but was delivered");
		DBUG_EXIT(A_TSETDIR);
	}
	if((ret = sigpending(&set)) < 0 ) {
		xx_rpt(DELETION);
		in_rpt("sigpending() returned %d instead of 0", ret);
		DBUG_EXIT(A_TSETDIR);
	}
	if(!sigismember(&set, SIGCONT)) {
		xx_rpt(DELETION);
		in_rpt("signal %d (%s) did not remain pending after generation",
					SIGCONT, signame(SIGCONT));
		in_rpt("but was discarded");
		DBUG_EXIT(A_TSETDIR);
	}

	/* Call sigaction() to initialise signal handling for SIGTSTP */
	(void) sig_init(SIGTSTP, &sig_empty, sig_stop, 0);

	/* Send signal to stop this process */
	if(kill((pid_t)getpid(), SIGTSTP) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		DBUG_EXIT(A_TSETDIR);
	}

	/* Should not be reached because the process is stopped */
	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t16stop()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_t16stop");

	sig_err = 0;

	/* Wait should report child is stopped */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT)
	{
		if(ex_code == A_MISC)
		{
			test_fail++;
			PATH_TRACE; 
			PATH_FUNC_RPT(2); 
			DBUG_VOID_RETURN;
		}
		else if(ex_code == A_TSETDIR)
		{
			test_deleted++;
			PATH_TRACE; 
			PATH_FUNC_RPT(2); 
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else if(stat_child == CHLD_ABRT)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected signal %d (%s) received",
						 sg_code, signame(sg_code));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else if(stat_child == CHLD_LIVE)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: child ignored signal SIGSTOP");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the SIGCONT signal */
	if(kill(tet_child, SIGCONT) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(2 * WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT)
	{
		if(ex_code == A_MISC)
		{
			PATH_TRACE; 
			test_fail++;
		}
		else if(ex_code == A_TSETDIR)
		{
			PATH_TRACE; 
			test_deleted++;
		}
		else
			PATH_TRACE;
	}
	else if(stat_child == CHLD_ABRT)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected signal %d (%s) received",
						 sg_code, signame(sg_code));
		test_deleted++;
	}
	else if(stat_child == CHLD_LIVE)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: child process timed out unexpectedly");
		test_deleted++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5); 
	DBUG_VOID_RETURN;
}


private void
sig_stop()
{
	int	ret;
	sigset_t set;

	DBUG_ENTER("sig_catch");

	/*
	 * Signal catching function for SIGTSTP.
	 * Check in here that SIGCONT signal is no longer pending.
	 */
	 
	/* Check that SIGCONT is not received */
	if(sig_recvd == SIGCONT) {
		if(!test_fail);
			xx_rpt(FAILURE);
		in_rpt("signal SIGCONT remained pending after stop signal");
		in_rpt("and was delivered");
		DBUG_EXIT(A_MISC);
	}

	/* Check it's no longer pending */
	if((ret = sigpending(&set)) < 0 ) {
		xx_rpt(DELETION);
		in_rpt("sigpending() returned %d instead of 0", ret);
		DBUG_EXIT(A_TSETDIR);
	}

	if(sigismember(&set, SIGCONT)) {
		if(!test_fail);
			xx_rpt(FAILURE);
		in_rpt("signal SIGCONT remained pending after stop signal");
		DBUG_EXIT(A_MISC);
	}

	DBUG_EXIT(OK);

	DBUG_VOID_RETURN;
}

private void
sig_alter(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_alter");

	sig_new = sg_num;

	DBUG_PRINTF("sig_alter", "sig_new %d", sig_new);

	DBUG_VOID_RETURN;
}
