/*
*      SCCS:  @(#)POSIX.os/procprim/signal/sigcon3.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.signal

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for signal concepts

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu Mar 16 12:43:44 GMT 1989
MODIFICATIONS:	Dave Cater, UniSoft Ltd.
		Thu Aug 10 12:08:04 BST 1989
		  test18 - Ensure child sleeps longer than parent to avoid
		  undesired extra SIGCHLD signal.
		  test19, test20, test21, make initialisation phase shorter
		  test20, Don't bother trying to catch SIGKILL.
		  test21, Correct a comment regarding SIG_DFL.
		  test24, restructured - had used SIG_DFL instead of SIG_IGN!!
		  test27, initialise data (even though its static)
		
		Geoff Clare, 14 Sep 1989
			Change pause() to sleep() in ch_t21

		Dave Cater, 19 Sep 1989
			Only fail if child is stopped (in pa_t21)
		
		Geoff Clare, 12 Oct 1989
			Test 3 types of orphaned process group in test21
		
		Geoff Clare, 2 Apr 1990
			Use sigpending() in ch_tpend().
		Keith Holder, 27 Jun 1990
			Deleted tests 23 & 24.
			Renumbered remainder.

		Anthony Lee, 15 August 1990
		Added resilience code.

		Stuart Boutell, 2 Nov 1990
			changed execv to tet_exec; added char **environ

		Stuart Boutell, 7 Nov 1990
			tet_child redeclaration removed.

		Stuart Boutell, 12 Nov 1990
			(void) vsignal(SIGCONT, SIG_DFL) added in test19

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called

		Stuart Boutell, 15 Jan 1990
			Changed cpsig return code check from FAIL to SYSERROR
		
		Geoff Clare, 20 May 1992
			Add check to test21 that signal is discarded.
		
		Geoff Clare, 12 Nov 1993
			Check for tty_prep() returning 1 in test21.

		Geoff Clare, 21 Dec 1995
			Make sig_rcvd volatile.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<wait_child.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<termios.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<sys/wait.h>
#include	<stdio.h>
#include	<errno.h>
#include	<synch.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigconcept
#endif


#if !__STDC__
#define volatile
#endif

#define	WRITETIME		(SPEEDFACTOR)
#define	WAITTIME		(5*SPEEDFACTOR)
#define	TESTTIME		(7*SPEEDFACTOR)
#define	EXTERR			"external error"
#define	NMR_ARG_VARS		5
#define	READ			0
#define	WRITE			1
#define MODEANY			(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	CREATMODE		0666
#define	ZERORET			0
#define	WAIT			1
#define	WAITPID			2
#define	TEST_END		0
#define	TEST_HANG		1
#define	TEST_WAIT		2

extern	void		(*vsignal())();
extern	char		*sig_child();
extern	void		vsx_sigs();
extern	void		setsigs();
extern	void		valrpt();
extern	int		cppair();
extern	char		*signame();
extern	char		*errname();

extern	char		**environ;
extern	char		*vg_ident;
extern	char		*vg_dbug;
extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sig_type[];
extern	int		ex_code;
extern	int		sg_code;

extern	void		(*sig_arg)();
extern	int		sig_index;
extern	sigset_t	sig_empty;
extern volatile int	sig_recvd;
extern int		sig_err;
extern int		test_fail;
extern int		test_deleted;
extern int		nsigs;
extern int		job_control;

private	char		*args[NMR_ARG_VARS];
private	char		*testcase;
private synch		*synpid, *synres;
private synch		*synpath;

public	void		test17();
public	void		test18();
public	void		test19();
public	void		test20();
public	void		test21();
public	void		test22();
public	void		test23();
public	void		test24();
public	void		test25();
public	void		test26();
public	void		test27();
public	void		test28();
public	void		test29();
private	void		ch_t17();
private	void		ch_t18();
private	void		pa_t18();
private	void		ch_t19();
private	void		pa_t19();
private	void		ch_t20();
private	void		pa_t20();
private	void		ch1_t21();
private	void		ch2_t21();
private	void		ch3_t21();
private	void		pa_t21();
private	void		gch_t21();
private	void		ch_tpend();
private	void		pa_tpend();
private	void		ch_t23();
private	void		pa_t23();
private	void		do_25to26();
private	void		ch_t25to26();
private	void		pa_t25to26();
private	void		ch_t27();
private	void		pa_t27();

extern	sigset_t	*sig_mask();
extern	void		sig_catch();
extern	void		ch_tdefault();
extern	void		pa_tdefault();

private struct rvals {
	int	data;
	int	pathval;
}rval;

public void
test17()
{
	DBUG_ENTER("test17");
	/*
	 * Test that all signals are set to either SIG_DFL or SIG_IGN on
	 * entry to routine main().
	 * This is done by exec() in a child process where all the 
	 * reporting is done.
	 */

	/* set up environment for tet_exec call */

	args[0] = "signal_t17";
	args[1] = vg_ident;
	args[2] = vg_dbug;
	args[3] = (char *) NULL;

	(void) cppair(ch_t17, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t17()
{
	int	pathok = 0;
	DBUG_ENTER("ch_t17");

	/*
	 * Set some signals to various actions to test the assertion.
	 */
	if(sig_init(SIGHUP, &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed - signal SIGHUP");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(sig_init(SIGQUIT, &sig_empty, SIG_IGN, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed - signal SIGQUIT");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	if(sig_init(SIGINT, &sig_empty, SIG_DFL, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed - signal SIGINT");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	DBUG_PRINTF("test17","Expected pathok = 3: Observed %d",pathok );

	(void) tet_exec("../signal_t17", args, environ);

	DBUG_EXIT(A_EXECFAIL);

	DBUG_VOID_RETURN;
}

public void
test18()
{
	DBUG_ENTER("test18");

	(void) cpsig(ch_t18, pa_t18, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
pa_t18()
{
	int	err;
	int	ret;		/* Used for most return values */
	int	pathok = 0;

	DBUG_ENTER("pa_t18");

	test_fail = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Set signal catching function for SIGCHLD */
	/* Call sigaction() for SIGCHLD with SA_NOCLDSTOP bit clear */
	if(sig_init(SIGCHLD, sig_mask(SIGINT), sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	/* Send SIGSTOP to child process */
	sig_recvd = 0;
	ret = kill(tet_child, SIGSTOP);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		test_fail++;
	}
	else
		PATH_TRACE;

	(void) sleep(WAITTIME/2);

	/* Check we received signal */
	if(!test_fail && sig_recvd != SIGCHLD) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() called with SA_NOCLDSTOP clear did not");
		in_rpt("generate SIGCHLD signal when child process is stopped");
		if(sig_recvd)
			in_rpt("signal %d (%s) was generated", 
						sig_recvd, sig_name[sig_recvd]);

		test_fail++;
	}
	else
		PATH_TRACE;

	/* Set default for SIGCHLD so we get no more interruptions */
	if(sig_init(SIGCHLD, sig_mask(SIGINT), SIG_DFL, 0) < 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(5);
	globok = 1;

	DBUG_VOID_RETURN;
}

private void
ch_t18()
{
	/*
	 * Child process which should receive a stop signal
	 * This process should then get tidied up by cppair()
	 */
	DBUG_ENTER("ch_t18");

	(void) sleep(WAITTIME);

	DBUG_VOID_RETURN;
}

public void
test19()
{
	int	i;
	int	sg_num;
	int	pathok = 0;

	DBUG_ENTER("test19");

	/*
	 * Test that signals sent to a stopped process are not delivered
	 * until the process is continued.
	 */

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	test_fail = 0;
	test_deleted = 0;

	(void)	vsignal(SIGCONT, SIG_DFL);
	
	for(i = 1; i < nsigs; i++) {

		sg_num = sig_array[i];

		if(i == VSIGKILL || i == VSIGSTOP)
		{
			PATH_TRACE; 
			continue;
		}

		if(sg_num == VNOSIG)
		{
			PATH_TRACE; 
			continue;
		}
		DBUG_PRINTF("test19", "signal %d", sg_num);

		/*
		 * Signals tested are those that by default terminate process
		 * or are ignored - excepting SIGKILL.
		 */
		if(sig_type[sg_num] != TYPE_ABRT &&
		   sig_type[sg_num] != TYPE_IGNR)
		   {
		   	PATH_TRACE; 
			continue;
		   }

		sig_index = i;

		globok = 0; 
		if(cpsig(ch_t19, pa_t19, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;

		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT(nsigs); 

	DBUG_VOID_RETURN;
}


private void
ch_t19()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t19");

	/* Set the relevant signal to be caught */
	(void) sig_init(sg_num, &sig_empty, sig_catch, 0);

	sig_recvd = 0;

	/* Pause and wait for SIGSTOP and other signals */
	(void) pause();

	/* Exit with number of signal received */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t19()
{
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t19");

	sig_err = 0;

	/* Sleep to give the child time to initialise */
	(void) sleep(WAITTIME/2);

	/* Send the stop signal */
	if(kill(tet_child, SIGSTOP) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the test signal */
	if(kill(tet_child, sg_num) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
						EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait should report child is stopped */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_STOP) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal delivered before stopped process was continued");
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the SIGCONT signal */
	if(kill(tet_child, SIGCONT) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(2 * WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the process is no longer suspended */
	if(stat_child != CHLD_EXIT || ex_code != sg_num) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %d (%s) not delivered after process continued", 
							sg_num, sg_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(7); 

	DBUG_VOID_RETURN;
}

public void
test20()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test20");


	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test that SIGKILL terminates a stopped process.
	 */

	test_fail = 0;
	test_deleted = 0;

	i = VSIGKILL;

	sig_index = i;

	globok = 0; 
	if(cpsig(ch_t20, pa_t20, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
ch_t20()
{
	DBUG_ENTER("ch_t20");

	/* No reason to try to alter default action for SIGKILL */

	sig_recvd = 0;

	/* Sleep and wait for SIGSTOP and other signals */
	(void) sleep(WAITTIME);

	/* Exit with number of signal received */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t20()
{
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t20");

	sig_err = 0;

	/* Sleep to give the child time to initialise */
	(void) sleep(WAITTIME/2);

	/* Send the stop signal */
	if(kill(tet_child, SIGSTOP) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait should report child is stopped */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_STOP) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal delivered before stopped process was continued");
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the test signal */
	if(kill(tet_child, sg_num) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
						EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the process terminated */
	if(stat_child != CHLD_ABRT || sg_code != sg_num) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %d (%s) did not terminate stopped process",
							sg_num, sg_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	PATH_FUNC_RPT(6); 

	DBUG_VOID_RETURN;
}

public void
test21()
{
	int	i;
	int	err;
	int	sg_num;
	int	pathok = 0;
	int 	ret_val;

	DBUG_ENTER("test21");

	/*
	 * Test that signals SIGTSTP, SIGTTIN, SIGTTOU
	 * do not cause processes in orphaned process groups to stop.
	 */

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	test_fail = 0;
	test_deleted = 0;

	if ((synpid = opensync()) == NULL || (synres = opensync()) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("opensync() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	DBUG_PRINTF("test21","nsigs is %d",nsigs);
	for(i = 1; i < nsigs; i++)
	{
		DBUG_PRINTF("test21","i is %d",i);
		sg_num = sig_array[i];

		if(i == VSIGKILL || i == VSIGSTOP)
		{
			PATH_TRACE; PATH_TRACE; PATH_TRACE; 
			PATH_TRACE; PATH_TRACE; PATH_TRACE; 
			continue;
		}

		if(sg_num == VNOSIG)
		{
			PATH_TRACE; PATH_TRACE; PATH_TRACE; 
			PATH_TRACE; PATH_TRACE; PATH_TRACE; 
			continue;
		}

		DBUG_PRINTF("test21", "signal %d", sg_num);

		/*
		 * Signals tested are those that by default stop process
		 * - excepting SIGSTOP.
		 */
		if(sig_type[sg_num] != TYPE_STOP)
		{
			PATH_TRACE; PATH_TRACE; PATH_TRACE; 
			PATH_TRACE; PATH_TRACE; PATH_TRACE; 
			continue;
		}

		sig_index = i;

		testcase = "for process in session leader's process group";

		ret_val = cppair(ch1_t21, NULLFN, 2 * WAITTIME, 0);
		if(ret_val != OK)
			test_deleted++;
		else if (waitsync(synres[0][0], (char *)&rval, sizeof(rval),
			WAITTIME) != sizeof(rval))
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: waitsync() failed, errno %d",
				err);
			test_deleted++;
		}
		else
		{
			err = rval.data;		/* Get err out of the parcel */
			if (err == 1)
				test_deleted++;
			else if (err)
				test_fail++;
			else if (rval.pathval == 1)
				PATH_TRACE; 
			else
				DBUG_PRINTF("test21","rval.pathval is %d",rval.pathval);
		}

		if(test_deleted)
		{
			break;
		}
		else
			PATH_TRACE;

		testcase = "for process in process group with no leader";

		ret_val = cppair(ch2_t21, NULLFN, 2 * WAITTIME, 0);
		if(ret_val != OK)
			test_deleted++;
		else if (waitsync(synres[0][0], (char *)&rval, sizeof(rval),
			WAITTIME) != sizeof(rval))
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: waitsync() failed, errno %d",
				err);
			test_deleted++;
		}
		else
		{
			err = rval.data;		/* Get err out of the parcel */
			if (err == 1)
				test_deleted++;
			else if (err)
				test_fail++;
			else if (rval.pathval == 1)
				PATH_TRACE; 
			else
				DBUG_PRINTF("test21","rval.pathval is %d",rval.pathval);
		}

		if(test_deleted)
		{
			break;
		}
		else
			PATH_TRACE;

		testcase = "for process in process group where leader's parent has exited";

		ret_val = cppair(ch3_t21, NULLFN, 2 * WAITTIME, 0);
		if(ret_val != OK)
			test_deleted++;
		else if (waitsync(synres[0][0], (char *)&rval, sizeof(rval),
			WAITTIME) != sizeof(rval))
		{
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: waitsync() failed, errno %d",
				err);
			test_deleted++;
		}
		else
		{
			err = rval.data;
			if (err == 1)	
				test_deleted++;
			else if (err)
				test_fail++;
			else if (rval.pathval == 1)
				PATH_TRACE; 
		}

		if(test_deleted)
		{
			break;
		}
		else
			PATH_TRACE;
	}

	if(!test_fail && !test_deleted)
		PATH_XS_RPT(((nsigs-1)*6)+2); 

	(void) closesync(synpid);
	(void) closesync(synres);

	DBUG_VOID_RETURN;
}

private void
pa_t21()
{
	int	err;
	int	stat_child;
	pid_t	pid;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t21");

	sig_err = 0;

	/* Get process ID from child to send signal to */
	if (waitsync(synpid[0][0], (char *)&rval, sizeof(rval), WAITTIME)
		!= sizeof(rval))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitsync() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		pid = rval.data;
		DBUG_PRINTF("pa_t21","pid is %d",pid);
		if (rval.pathval == 1)
		{
			PATH_TRACE;
		}
		else
			DBUG_PRINTF("pa_t21","rval.pathval is %d",rval.pathval);
	}

	/* Send the test signal */
	(void) sleep(2);
	if(kill(pid, sg_num) < 0)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: kill() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait should not report child has stopped */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_STOP)
	{
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s", testcase);
		in_rpt("signal %d (%s) produced wrong action for process",
							sg_num, sg_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4); 
	DBUG_VOID_RETURN;
}

private void
gch_t21()
{
	int	err;
	pid_t	pid;
	int	pathok = 0;

	DBUG_ENTER("gch_t21");

	PATH_TRACE; /* historical artifact */

	/* Set all signals to SIG_DFL */
	setsigs(SIG_DFL);

	/* Send process ID to parent */
	pid = getpid();
	rval.data = pid;
	rval.pathval = pathok == 1 ? 1 : 0;	/* Pass back path checking */
	if (sendsync(synpid[0][1], (char *)&rval, sizeof(rval), WAITTIME)
		!= sizeof(rval))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: sendsync() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}

	/* wait for stop signals */
	(void) sleep(WAITTIME/2);

	/* Unorphan self and make sure stop signal doesn't then arrive */
	(void) setpgid((pid_t)0, (pid_t)0);
	(void) sleep(1+WAITTIME/4);

	DBUG_VOID_RETURN;
}


private void
ch1_t21()
{
	int	err;
	int	tty_fildes;
	struct termios tty_termios;
	int	pathok = 0;

	DBUG_ENTER("ch1_t21");

	/* Put this process in an orphaned process group */
	/* case 1: process is in session leader's process group */

	if(setsid() == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: setsid() failed, errno %d", 
			err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(tty_prep(&tty_fildes, &tty_termios))
	{
	case 0:
		PATH_TRACE;
		break;
	case 1:
		/* terminal interface not supported */
		/* test can still be done but is less rigorous */
		PATH_TRACE;
		break;
	default:
		xx_rpt(DELETION);
		in_rpt("deletion reason: tty_prep() failed");
		DBUG_VOID_RETURN;
	}

	globok = 0; 
	if(cpsig(gch_t21, pa_t21, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	err = 0;
	if (test_deleted)
		err = 1;
	else if (test_fail)
		err = 2;

	/* Send result to grandparent */
	rval.data = err;			/* Have to package it for delivery */
	rval.pathval = (pathok == 3 ? 1 : 0);	/* 1 to indicate path ok */
	if (sendsync(synres[0][1], (char *)&rval, sizeof(rval), WAITTIME)
		!= sizeof(rval))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: sendsync() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	
	DBUG_VOID_RETURN;
}

private void
ch2_t21()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch2_t21");

	/* Put this process in an orphaned process group */
	/* case 2: process group leader has terminated */

	if(setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: setpgid(0,0) failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	switch (fork())
	{
	case SYSERROR :
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fork() failed, errno %d", err);
		DBUG_VOID_RETURN;
		/* NOTREACHED */
		break;

	case 0 :
		tet_setcontext();
		break;

	default :
		tet_setblock();
		DBUG_VOID_RETURN;
		/* NOTREACHED */
		break;
	}

	globok = 0; 
	if(cpsig(gch_t21, pa_t21, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	err = 0;
	if (test_deleted)
		err = 1;
	else if (test_fail)
		err = 2;

	/* Send result to grandparent */
	rval.data = err;			/* Have to package it for delivery */
	rval.pathval = (pathok == 2 ? 1 : 0);	/* 1 to indicate path ok */
	if (sendsync(synres[0][1], (char *)&rval, sizeof(rval), WAITTIME)
		!= sizeof(rval))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: sendsync() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	DBUG_VOID_RETURN;
}

private void
ch3_t21()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("ch3_t21");

	/* Put this process in an orphaned process group */
	/* case 3: process group leader's parent has terminated */

	switch (fork())
	{
	case SYSERROR :
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: fork() failed, errno %d", err);
		DBUG_VOID_RETURN;
		/* NOTREACHED */
		break;

	case 0 :
		tet_setcontext();
		break;

	default :
		tet_setblock();
		DBUG_VOID_RETURN;
		/* NOTREACHED */
		break;
	}

	if(setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: setpgid(0,0) failed, errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if(cpsig(gch_t21, pa_t21, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	err = 0;
	if (test_deleted)
		err = 1;
	else if (test_fail)
		err = 2;

	/* Send result to grandparent */
	rval.data = (int) err;
	rval.pathval = (pathok == 2 ? 1 : 0);
	if (sendsync(synres[0][1], (char *)&rval, sizeof(rval), WAITTIME)
		!= sizeof(rval))
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: sendsync() failed, errno %d", err);
		DBUG_VOID_RETURN;
	}

	DBUG_VOID_RETURN;
}

public void
test22()
{
	int	pathok = 0;
	DBUG_ENTER("test22");

	test_fail = 0;
	test_deleted = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig_index = VSIGCHLD;
	sig_arg = SIG_DFL;

	/*
	 * Borrowing this pipe to send back pathchecking info
	 */
	
	if ((synpath = opensync()) == NULL)
	{
		xx_rpt(DELETION);
		in_rpt("opensync() failed");
		DBUG_VOID_RETURN;
	}

	globok = 0; 
	if(cpsig(ch_tpend, pa_tpend, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE; 

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(2); 

	(void) closesync(synpath);
	DBUG_VOID_RETURN;
}

private void
ch_tpend()
{
	sigset_t set;
	int	ret;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("ch_tpend");

	/*
	 * Test that setting signal action to sig_arg when signal is
	 * pending causes the pending signal to be discarded.
	 */

	/* Call sigaction() to initialise signal handling function */
	if(sig_init(sg_num, &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed - signal %d (%s)",
						sg_num, sg_name);
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	/* Call sigprocmask() to block the signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;
	
	/* Check the signal is blocked */
	sig_recvd = 0;
	ret = kill(getpid(), sg_num);
	if(ret < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;
		
	if(sig_recvd == sg_num) {
		xx_rpt(DELETION);
		in_rpt("signal %d (%s) is unblocked, should be blocked", 
						sg_num, sg_name);
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	/* Check the signal is pending */
	(void) sigemptyset(&set);
	if (sigpending(&set) != 0) {
		xx_rpt(DELETION);
		in_rpt("sigpending() failed - errno %d (%s)", 
			errno, errname(errno));
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;
		
	if (!sigismember(&set, sg_num)) {
		xx_rpt(DELETION);
		in_rpt("signal %d (%s) not pending after blocking and sending",
						sg_num, sg_name);
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	/* Call sigaction() to set action to sig_arg */
	if(sig_init(sg_num, &sig_empty, sig_arg, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("sigaction() failed - signal %d (%s)", sg_num, sg_name);
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	/* Check the signal is no longer pending */
	(void) sigemptyset(&set);
	if (sigpending(&set) != 0) {
		xx_rpt(DELETION);
		in_rpt("sigpending() failed - errno %d (%s)", 
			errno, errname(errno));
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;
	if (sigismember(&set, sg_num)) {
		xx_rpt(FAILURE);
		in_rpt("signal %d (%s) still pending after sigaction()",
						sg_num, sg_name);
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	/* Call sigprocmask() to unblock signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if((ret = sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL)) != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returned %d instead of 0", ret);
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE; 

	/* Exit code is signal number we received if any */
	rval.data = (int)NULL;			/* Nothing here */
	rval.pathval = (pathok == 10 ? 1 : 0);
	if (sendsync(synpath[0][1], (char *)&rval, sizeof(rval), WAITTIME)
		!= sizeof(rval))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: sendsync() failed, errno %d", errno);
		DBUG_VOID_RETURN;
	}

	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_tpend()
{
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_tpend");

	if (waitsync(synpath[0][0], (char *)&rval, sizeof(rval), WAITTIME)
		!= sizeof(rval))
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitsync() failed, errno %d", errno);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		if (rval.pathval == 1)
		{
			PATH_TRACE;
		}
		else
			DBUG_PRINTF("pa_tpend","rval.pathval is %d",rval.pathval);
	}
	
	/* Wait should report child has exitted */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE)
	{
		err = errno;
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_TSETDIR)
	{
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("pending signal %d (%s) produced wrong action when unblocked",
							sg_num, sg_name);
		in_rpt("expected action: process ignored signal");
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4); 
	DBUG_VOID_RETURN;
}

public void
test23()
{
	int	i;
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("test23");

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Test action SIG_IGN for all signals except SIGKILL and SIGSTOP
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		sg_name = sig_name[i];

		/* Can't ignore these signals */
		if(i == VSIGKILL || i == VSIGSTOP)
		{
			PATH_TRACE; 
			continue;
		}

		DBUG_PRINTF("test23", "signal %s", sg_name);

		sig_index = i;
		sig_arg = SIG_IGN;

		globok = 0; 
		if(cpsig(ch_t23, pa_t23, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;

		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT(nsigs-1); 

	DBUG_VOID_RETURN;
}

private void
ch_t23()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t23");

	/* Set the required action if we can */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	sig_recvd = 0;

	(void) kill(getpid(), sg_num);

	/* Exit code is signal number we received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t23()
{
	int	err;
	int	stat_child;
	char *	str_expect;
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t23");

	sig_err = 0;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Switch on what default action is expected. */
	str_expect = "process ignored signal";
	if(stat_child != CHLD_EXIT)
		sig_err++;
	if(ex_code != 0)
		sig_err++;

	if(sig_err) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caused wrong action", sg_name);
		in_rpt("expected action: %s", str_expect);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 
	DBUG_VOID_RETURN;
}

public void
test24()
{
	int	i;
	int	sg_num;
	int	pathok = 0;

	DBUG_ENTER("test24");

	/*
	 * Test that setting action to SIG_IGN for a signal that is 
	 * pending causes the pending signal to be discarded.
	 */

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Borrowing this pipe to send back pathchecking info
	 */
	
	if ((synpath = opensync()) == NULL )
	{
		xx_rpt(DELETION);
		in_rpt("opensync() failed");
		DBUG_VOID_RETURN;
	}

	for(i = 1; i < nsigs; i++)
	{
		sg_num = sig_array[i];

		/* Can't catch these signals */
		if(i == VSIGKILL || i == VSIGSTOP)
		{
			PATH_TRACE; 
			continue;
		}

		/* Can't block this signal either ... */
		if(i == VSIGCONT)
		{
			PATH_TRACE; 
			continue;
		}

		if(sg_num == VNOSIG)
		{
			PATH_TRACE; 
			continue;
		}
		DBUG_PRINTF("test24", "signal %d", sg_num);

		sig_index = i;
		sig_arg = SIG_IGN;

		globok = 0; 
		if(cpsig(ch_tpend, pa_tpend, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE; 
			
		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT(nsigs-1); 

	DBUG_VOID_RETURN;
}

public void
test25()
{
	int	pathok = 0;
	DBUG_ENTER("test25");

	/*
	 * Test standard signals can be caught.
	 */

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	do_25to26(TYPE_ABRT);
	PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

public void
test26()
{
	int	pathok = 0;
	DBUG_ENTER("test26");

	/*
	 * Test POSIX job control signals can be caught.
	 */

	test_fail = 0;
	test_deleted = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	do_25to26(TYPE_IGNR);
	PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	do_25to26(TYPE_STOP);
	PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	do_25to26(TYPE_CONT);
	PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
do_25to26(arg)
int	arg;	/* what default action type to test */
{
	int	i;
	int	sg_num;
	char *	sg_name;
	int	pathok = 0;

	DBUG_ENTER("do_25to26");

	/*
	 * Test action for all signals that have default action
	 * type which matches "arg".
	 * This test shows they can also be caught in a signal catching 
	 * function and the signal number is passed as the argument.
	 *
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		sg_num = sig_array[i];
		sg_name = sig_name[i];

		/* Can't catch these signals */
		if(i == VSIGKILL || i == VSIGSTOP)
		{
			PATH_TRACE;
			continue;
		}

		/* check default action for this signal */
		if(sig_type[sg_num] != arg)
		{
			PATH_TRACE;
			continue;
		}

		DBUG_PRINTF("do_25to26", "signal %s", sg_name);

		sig_index = i;
		sig_arg = sig_catch;


		globok = 0; 
		if(cpsig(ch_t25to26, pa_t25to26, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;
			
		if(test_deleted)
			DBUG_VOID_RETURN;
	}
	PATH_FUNC_RPT(nsigs-1); 
	DBUG_VOID_RETURN;
}

private void
ch_t25to26()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t25to26");

	/* Set the required action if we can */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	sig_recvd = 0;

	(void) kill(getpid(), sg_num);

	/* Exit code is signal number we received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t25to26()
{
	int	err;
	int	stat_child;
	char *	str_expect;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	char	exp_buf[BUFSIZ];
	int	pathok = 0;

	DBUG_ENTER("pa_t25to26");

	sig_err = 0;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Switch on what default action is expected. */
	(void) sprintf(exp_buf, "process caught signal %d", sg_num);
	str_expect = exp_buf;
	if(stat_child != CHLD_EXIT)
		sig_err++;
	if(ex_code != sg_num)
		sig_err++;

	if(sig_err) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caused wrong action", sg_name);
		in_rpt("expected action: %s", str_expect);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_VOID_RETURN;
}

public void
test27()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test27");

	/*
	 * Test that after returning from a signal catching function,
	 * execution resumes from the point at which it was interrupted.
	 *
	 * This test shows they can also be caught in a signal catching 
	 * function and the signal number is passed as the argument.
	 *
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */

	i = VSIGHUP;

	sig_index = i;
	sig_arg = sig_catch;

	globok = 0; 
	if(cpsig(ch_t27, pa_t27, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
ch_t27()
{
	int	sg_num  = sig_array[sig_index];
	static	int	i, j;

	DBUG_ENTER("ch_t27");

	/* Set the required action if we can */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	sig_recvd = 0;

	i = 0;
	j = 0;

	while(!sig_recvd)
		i++, j++;

	if(i != j)
		DBUG_EXIT(A_MISC);

	/* Exit code is signal number we received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t27()
{
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t27");

	sig_err = 0;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: child process not executing");
		in_rpt("observed action: %s", sig_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the signal */
	if(kill(tet_child, sg_num) < 0 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait for the child and expect it to have exitted. */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_MISC) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("execution did not resume from point of interruption");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	if(stat_child != CHLD_EXIT)
		sig_err++;
	if(ex_code != sg_num)
		sig_err++;

	if(sig_err) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - signal %s not caught", sg_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(6); 

	DBUG_VOID_RETURN;
}

public void
test28()
{
	int	i;
	int	sg_num;
	char *	sg_name;
	int	pathok = 0;

	DBUG_ENTER("test28");

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Test action for signal SIGKILL can't be set to be signal catching.
	 * Do this both by trying to set signal catching using sigaction()
	 * and by ensuring the action taken on delivery is still the default.
	 * Use the tests for default signal handling !
	 *
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */

	i = VSIGKILL;

	sg_num = sig_array[i];
	sg_name = sig_name[i];

	sig_index = i;
	sig_arg = sig_catch;

	/* Attempt to set the action */
	if(sig_init(sg_num, &sig_empty, sig_arg, 0) == 0) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() succeeded setting action to catch");
		in_rpt("signal %d (%s)", sg_num, sg_name);
		(void) sig_init(sg_num, &sig_empty, SIG_DFL, 0);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if(cpsig(ch_tdefault, pa_tdefault, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

public void
test29()
{
	int	i;
	int	sg_num;
	char *	sg_name;
	int	pathok = 0;

	DBUG_ENTER("test29");

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Test action for signal SIGSTOP can't be set to be signal catching.
	 * Do this both by trying to set signal catching using sigaction()
	 * and by ensuring the action taken on delivery is still the default.
	 * Use the tests for default signal handling !
	 *
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	i = VSIGSTOP;

	sg_num = sig_array[i];
	sg_name = sig_name[i];

	sig_index = i;
	sig_arg = sig_catch;

	/* Attempt to set the action */
	if(sig_init(sg_num, &sig_empty, sig_arg, 0) == 0) {
		xx_rpt(FAILURE);
		in_rpt("sigaction() succeeded setting action to catch");
		in_rpt("signal %d (%s)", sg_num, sg_name);
		(void) sig_init(sg_num, &sig_empty, SIG_DFL, 0);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if(cpsig(ch_tdefault, pa_tdefault, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}
