/*
*      SCCS:  @(#)POSIX.os/procprim/signal/sigcon4.c	Rel 4.4.3     (07/20/98) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.signal

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for signal concepts

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu Mar 16 12:43:44 GMT 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Thu Aug 10 13:31:20 BST 1989
		  test30, check entry was made 3 times as well as exit twice
		  test32, test33, remove unused function calls
		  test34, test35, Set SIGSTOP to SIG_DFL (if allowed).
		Dave Cater, UniSoft Ltd.
		Wed Sep 13 18:18:44 BST 1989
		  init_tfuncs(), cln_tfuncs() more efficient, only take
		  specific action for function being tested
		Dave Cater, UniSoft Ltd.
		Wed Sep 20 16:20:54 BST 1989
		  Allow re-entrance of sleep to be terminated by original 
		  alarm value (i.e. sooner than requested).
		
		Geoff Clare, 9 Oct 1989
			Don't want a controlling terminal in tty_setup()

		Geoff Clare, 8 Nov 1989
			Make sure stop signals are not tested on orphaned
			process group (unless specified in test).
		
		Geoff Clare, 16 Jan 1990
			Empty pipe completely in pa_write().
		
		Geoff Clare, 22 Mar 1990
			Improve efficiency of pipe filling strategy.
		
		Geoff Clare, 24 Apr 1990
			Turn tty output back on after interrupted tcdrain().
		
		Geoff Clare, 21 May 1990
			Open both sides of loopback to avoid possible
			EIO errors during setup.
		
		Geoff Clare, 5 June 1990
			Don't use setpgid() in tty_setup() if no job control.
			Change reporting in tty_setup().

		Keith Holder, 27 Jun 1990
			Renumbered tests "-2"

		Anthony Lee, 16 August 1990
		Added resilience code.	
		
		Stuart Boutell, 7 Nov 1990
			tet_child redeclaration removed.

		Stuart Boutell, 4 Dec 1990
			pipe_empty and pipe_full declarare to -1;
			Added code to ensure stdin still open after
			test30

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called
			Added prep_sigs() call to tests 34,35,36,37

		Stuart Boutell, 15 Jan 1990
			Change cpsig return code check to SYSERROR from FAIL
		
		Geoff Clare, 24 June 1991
			Call tty_setup() from init_tfuncs() instead of
			t_tcdrain().  Add tty cleanup code to cln_tfuncs().

		Geoff Clare, 23 Jan 1992
			Change order of operations in pa_t34and35 to
			prevent race condition.

		Geoff Clare, 9 June 1992
			Skip tcdrain() tests if no terminal output queue.

		Geoff Clare, 2 Dec 1993
			Use terms_prep() instead of tty_prep() and loop_prep()
			so pseudo-terms will be connected properly.

		Geoff Clare, 9 Oct 1997
			Switch access modes in t_open() and pa_open() to
			avoid the possibility of an ENXIO in pa_open().

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<fcntl.h>
#include	<dbug.h>
#include	<report.h>
#include	<wait_child.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<sys/wait.h>
#include	<stdio.h>
#include	<errno.h>
#include	<termios.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigconcept
#endif


#define	WRITETIME		(SPEEDFACTOR)
#define	WAITTIME		(5*SPEEDFACTOR)
#define	TESTTIME		(7*SPEEDFACTOR)
#define	EXTERR			"external error"
#define	READ			0
#define	WRITE			1
#define MODEANY			(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	CREATMODE		0666
#define	ZERORET			0
#define	WAIT			1
#define	WAITPID			2
#define	TEST_END		0
#define	TEST_HANG		1
#define	TEST_WAIT		2
#define	T_DUMMY			0
#define	T_TCDRAIN		1
#define	T_FCNTL			2
#define	T_SLEEP			3
#define	T_PAUSE			4
#define	T_SIGSUSP		5
#define	T_READ			6
#define	T_WRITE			7
#define	T_OPEN			8
#define	T_WAIT			9
#define	T_WAITPID		10
#define	TYPE_ABORT		1
#define	TYPE_REENTER		2
#define	TYPE_STOP		3
#define	TYPE_IGNORE		4
#define PATH_NOK		16

extern	int		no_tests;
extern	int		tet_thistest;
extern	char		*sig_child();
extern	void		vsx_sigs();
extern	void		valrpt();
extern	int		cppair();
extern	char		*signame();
extern	void		prep_sigs();

extern	char		*vg_ident;
extern	char		*vg_dbug;
extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sig_type[];
extern	int		ex_code;
extern	int		sg_code;

extern	void		(*sig_arg)();
extern	int		sig_index;
extern	sigset_t	sig_empty;
extern int		sig_recvd;
extern int		sig_err;
extern int		test_fail;
extern int		test_deleted;
extern int		nsigs;
extern int		job_control;

private	int 		tty_fildes, loop_fildes;
private struct termios	tty_termios, *tty_termios_p = &tty_termios,
			loop_termios, *loop_termios_p = &loop_termios;
private int		sig_exit;
private int		sig_cont;
private int		sig_errno;
private int		sig_ret;
private	int		test_case;
private	int		test_ret;
private	int		pipe_empty[2] = { -1, -1 };
private	int		pipe_full[2] = { -1, -1 };
private	int		fd_fcntl;
private	struct flock	lock_info;
private	char		*t30_fifo    = "./sig-t.30fifo";
private	char		*t30_lock    = "./sig-t.30lock";
private	char		*ensure_file = ".";
private	void		(*test_fn)();
private	void		(*pa_fn)();
private	int		fd_rdfifo;
private	char		*test_name;
private	int		test_type;
private	int		test_num;
private	sigset_t	sig_suspend;
private int		sig_entry;
private int		local_child;
private	int		gti_supp;

public	void		test30();
public	void		test31();
public	void		test32();
public	void		test33();
public	void		test34();
public	void		test35();
public	void		test36();
public	void		test37();
private	void		ch_tkill();
private	void		ch_tprocmask();
private	void		pa_tkillprocmask();
private	void		init_tfuncs();
private	void		cln_tfuncs();
private	void		ch_t30();
private	void		ch_fork();
private	void		pa_t30();
private	void		pa_t31();
private	void		do_34and35();
private	void		ch_t34and35();
private	void		pa_t34and35();
private	void		ch_t36();
private	void		pa_t36();
private	void		ch_t37();
private	void		pa_t37();
private	void		t_tcdrain();
private	void		t_fcntl();
private	void		t_sleep();
private	void		t_pause();
private	void		t_sigsusp();
private	void		t_read();
private	void		t_write();
private	void		t_open();
private	void		t_wait();
private	void		t_waitpid();
private	void		t_myfunc();
private	void		pa_fcntl();
private	void		pa_dummy();
private	void		pa_read();
private	void		pa_write();
private	void		pa_open();
private	void		sig_reenter();
private	void		sig_proc_reenter();
private	void		sig_kill_reenter();

extern	void		sig_catch();

struct test_struct {
	void	(*testfuncs)();
	void	(*pafuncs)();
	char	*testnames;
	int	testnum;
	int	testtypes;
	int	testreturns;
};

struct test_struct myfunc = {
 t_myfunc, 	pa_dummy, "loop test",	  T_DUMMY, 	TEST_END,  ZERORET
};

struct test_struct test_array[] = {
 { t_tcdrain, 	pa_dummy, "tcdrain()", 	  T_TCDRAIN,	TEST_HANG, SYSERROR },
 { t_fcntl, 	pa_fcntl, "fcntl()", 	  T_FCNTL,	TEST_END,  SYSERROR },
 { t_sleep, 	pa_dummy, "sleep()", 	  T_SLEEP,	TEST_END , ZERORET  },
 { t_pause, 	pa_dummy, "pause()", 	  T_PAUSE,	TEST_HANG, SYSERROR },
 { t_sigsusp,	pa_dummy, "sigsuspend()", T_SIGSUSP,	TEST_HANG, SYSERROR },
 { t_read, 	pa_read,  "read()", 	  T_READ,	TEST_END,  SYSERROR },
 { t_write, 	pa_write, "write()", 	  T_WRITE,	TEST_END,  SYSERROR },
 { t_open, 	pa_open,  "open()", 	  T_OPEN,	TEST_END,  SYSERROR },
 { t_wait, 	pa_dummy, "wait()", 	  T_WAIT,	TEST_WAIT, SYSERROR },
 { t_waitpid, 	pa_dummy, "waitpid()", 	  T_WAITPID,	TEST_WAIT, SYSERROR }
};

private int
tty_setup()
{
	char	*msg = NULL;
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("tty_setup");

	/* Open and initialize terminal file */
	/* Don't use termios_prep(): it calls setsid() */

	SET_TIMEOUT(WAITTIME)

	switch (terms_prep(O_RDWR|O_NOCTTY, &tty_fildes, tty_termios_p,
			   O_RDWR, &loop_fildes, loop_termios_p))
	{
	case 0:
		break;
	case 1:
		gti_supp = 0;
		break;
	default:
		if (alrm_flag > 0)
			msg = "open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP timed out";
		else
			msg = "could not open/initialise VSX_TERMIOS_TTY and VSX_TERMIOS_LOOP";
	}

	CLEAR_ALARM

	if (msg != NULL)
	{
		xx_rpt(DELETION);
		in_rpt("%s", msg);
		DBUG_RETURN(-1);
	}
	else if (!gti_supp)
	{
		/* VSX_TERMIOS_TTY/LOOP is set to "unsup" */
		PATH_TRACE;
		PATH_FUNC_RPT(1); 
		DBUG_RETURN(1);
	}
	else
		PATH_TRACE;

	if (tcflow(tty_fildes, TCOOFF) == SYSERROR) 
	{
		xx_rpt(DELETION);
		in_rpt("tcflow(TCOOFF) failed in preparation");
		DBUG_RETURN(-1);
	}
	else
		PATH_TRACE;

	SET_TIMEOUT(WRITETIME)

	(void) write(tty_fildes, buffer, sizeof(char));

	CLEAR_ALARM
	PATH_FUNC_RPT(2); 

	DBUG_RETURN(0);
}

private void
ch_tprocmask()
{
	sigset_t set;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("ch_tprocmask");

	/* Call sigaction() to initialise signal handling function */
	if(sig_init(sg_num, &sig_empty, sig_arg, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed - signal %d (%s)",
						sg_num, sg_name);
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	/* Call sigprocmask() to block the signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	sig_entry = sig_exit = 0;

	/* Call re-entrant function */

	globok = 0;
	sig_proc_reenter();
	PATH_FUNC_TRACE;

	if(sig_exit != 2 || sig_entry != 3) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigprocmask() not re-entrant");
		DBUG_EXIT(A_MISC);
	}
	else
		PATH_TRACE;

	if (pathok != 4)
	{
		DBUG_EXIT(PATH_NOK);
	}
	else		
		DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
sig_proc_reenter()
{
	sigset_t set;
	int	ret;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("sig_proc_reenter");

	sig_entry++;

	if(sig_entry > 2) {
		globok = 1; 
		DBUG_VOID_RETURN;
	}

	/* Check the signal is blocked */
	ret = kill(getpid(), sg_num);
	if(ret < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	if(sig_exit > 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - blocked signal %d (%s) received",
						sg_num, sg_name);
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	/* Call sigprocmask() to unblock signal - leading to reentry */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if((ret = sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL)) != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigprocmask() returned %d instead of 0", ret);
		DBUG_EXIT(A_MISC);
	}
	else
		PATH_TRACE;

	/* Increment counter on returning from sigprocmask() */
	sig_exit++;

	DBUG_PRINTF("sig_proc_reenter()", "sig_exit is %d", sig_exit);

	PATH_FUNC_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
ch_tkill()
{
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("ch_tkill");

	/* Call sigaction() to initialise signal handling function */
	if(sig_init(sg_num, &sig_empty, sig_arg, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed - signal %d (%s)",
						sg_num, sg_name);
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	sig_entry = sig_exit = 0;

	/* Call re-entrant function */

	globok = 0;
	sig_kill_reenter();
	PATH_FUNC_TRACE;

	if(sig_exit != 2) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("kill() not re-entrant");
		DBUG_EXIT(A_MISC);
	}
	else
		PATH_TRACE;

	if (pathok != 3)
		DBUG_EXIT(PATH_NOK);
		
	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
sig_kill_reenter()
{
	sigset_t set;
	int	ret;
	int	sg_num  = sig_array[sig_index];
	int	pathok = 0;

	DBUG_ENTER("sig_kill_reenter");

	sig_entry++;

	if(sig_entry > 2) {
		globok = 1;
		DBUG_VOID_RETURN;
	}

	/* Call sigprocmask() to unblock signal before trying kill() again */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if((ret = sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL)) != 0) {
		xx_rpt(DELETION);
		in_rpt("sigprocmask() returned %d instead of 0", ret);
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	if(sig_exit > 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() returned before being re-entered");
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	/* Send the signal */
	ret = kill(getpid(), sg_num);
	if(ret < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	/* Increment counter on returning from kill() */
	sig_exit++;

	DBUG_PRINTF("sig_kill_reenter()", "sig_exit is %d", sig_exit);
	PATH_FUNC_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
pa_tkillprocmask()
{
	int	err;
	int	stat_child;
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_tkillprocmask");

	sig_err = 0;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	if(stat_child != CHLD_EXIT)
		sig_err++;
	else {
		if(ex_code == A_TSETDIR) {
			test_deleted++;
			PATH_FUNC_RPT(1); 
			DBUG_VOID_RETURN;
		}
		if(ex_code == A_MISC) {
			test_fail++;
			PATH_FUNC_RPT(1); 
			DBUG_VOID_RETURN;
		}
		if(ex_code == PATH_NOK)
			ex_code = OK;
		else
			PATH_TRACE; 
	}

	if(sig_err) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caused wrong action", sg_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
init_tfuncs()
{
	char	*cp;
	char	buffer[512];
	int	statw;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("init_tfuncs");

	if(test_num == T_READ) {
		/* Make an empty pipe for read() tests */
		if(pipe(pipe_empty) < 0) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - pipe() failed, errno %d", err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if(test_num == T_WRITE) {
		/* Make a full pipe for write() tests */
		if(pipe(pipe_full) < 0) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason - pipe() failed, errno %d", err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		/* Set non-blocking mode for write file descriptor */
		statw = fcntl(pipe_full[WRITE], F_GETFL, 0);
		(void) fcntl(pipe_full[WRITE], F_SETFL, statw | O_NONBLOCK);

		/* Fill up the pipe */
		while(write(pipe_full[WRITE], buffer, sizeof(buffer)) > 0)
			;
		while(write(pipe_full[WRITE], buffer, 1) > 0)
			;

		/* Set blocking mode for write file descriptor */
		statw = fcntl(pipe_full[WRITE], F_GETFL, 0);
		(void) fcntl(pipe_full[WRITE], F_SETFL, statw & ~O_NONBLOCK);
	}
	else
		PATH_TRACE;

	if(test_num == T_OPEN) {
		/* Make a fifo for open() tests */
		if(mkfifo(t30_fifo, MODEANY) == SYSERROR) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: mkfifo(%s) failed - errno %d",
				t30_fifo, err);
			(void) unlink(t30_fifo);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if(test_num == T_FCNTL) {
		/* Create and lock a file for fcntl() tests */
		if((fd_fcntl = creat(t30_lock, CREATMODE)) < 0) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: creat() failed - errno %d", err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		lock_info.l_type = F_WRLCK;
		lock_info.l_whence = SEEK_SET;
		lock_info.l_start = (off_t)0;
		lock_info.l_len = sizeof(int);
		lock_info.l_pid = (pid_t)0;
		if(fcntl(fd_fcntl, F_SETLK, &lock_info) < 0) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: fcntl() failed - errno %d", err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;PATH_TRACE; 
	}

	if(test_num == T_SIGSUSP) {
		/* Make a signal set for sigsuspend() tests */
		(void) sigemptyset(&sig_suspend);
	}

	if(test_num == T_TCDRAIN)
	{
		/* Initialise terminal loopback */

		cp = tet_getvar("VSX_TERMIOS_BUFFERED");
		if (cp == NULL || *cp == '\0')
		{
			xx_rpt(DELETION);
			in_rpt("parameter VSX_TERMIOS_BUFFERED is not set");
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else if (*cp == 'N' || *cp == 'n')
		{
			PATH_TRACE;
			gti_supp = 0;
		}
		else
		{
			PATH_TRACE;
			gti_supp = 1;
		}

		if (gti_supp)
		{
			globok = 0;
			switch (tty_setup())
			{
			case 0:
				PATH_FUNC_TRACE;
				break;
			case 1:
				gti_supp = 0;
				PATH_FUNC_TRACE;
				break;
			default:
				test_deleted++;
				DBUG_VOID_RETURN;
			}
		}
		else
			PATH_TRACE;
	}
	else
	{
		PATH_TRACE;PATH_TRACE; 
	}

	PATH_FUNC_RPT(7); 
	DBUG_VOID_RETURN;
}

private void
cln_tfuncs()
{
	struct	stat	ensure_buf;
	DBUG_ENTER("cln_tfuncs");

	if(test_num == T_READ) {
		/* Tidy up empty pipe */
		(void) close(pipe_empty[READ]);
		(void) close(pipe_empty[WRITE]);
	}

	if(test_num == T_WRITE) {
		/* Tidy up full pipe */
		(void) close(pipe_full[READ]);
		(void) close(pipe_full[WRITE]);
	}

	if(test_num == T_OPEN) {
		/* Tidy up and remove fifo */
		(void) close(fd_rdfifo);
		(void) unlink(t30_fifo);
	}

	if(test_num == T_FCNTL) {
		/* Tidy up and remove lock file */
		lock_info.l_type = F_UNLCK;
		lock_info.l_whence = SEEK_SET;
		lock_info.l_start = (off_t)0;
		lock_info.l_len = sizeof(int);
		lock_info.l_pid = (pid_t)0;
		(void) fcntl(fd_fcntl, F_SETLK, &lock_info);
		(void) close(fd_fcntl);
		(void) unlink(t30_lock);
	}

	if(test_num == T_TCDRAIN && gti_supp) {
		/* Restart output and close terminals */

		(void) tcflow(tty_fildes, TCOON);
		(void) close(tty_fildes);
		(void) close(loop_fildes);
	}

	if((fstat(fileno(stdin), &ensure_buf) == SYSERROR) &&
		(freopen(ensure_file, "r",stdin) == (FILE *)NULL))
	{
		int i;
		char	message[256];
		(void) sprintf(message,"Failed to ensure stdin stream left open after test%d",tet_thistest);
		for(i=1; i<=no_tests; i++)
			tet_delete(i,message);
	}

	DBUG_VOID_RETURN;
}

public void
test30()
{
	int	nfuncs;
	int	i;
	int	j;
	int	pathok = 0;

	DBUG_ENTER("test30");

	/*
	 * Test that functions are re-entrant with respect to signals.
	 *
	 * The signal is caught and the signal catching function sig_reenter()
	 * is called. This calls the test function (*test_fn)() again.
	 *
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */

	test_fail = 0;
	test_deleted = 0;
	test_case = TYPE_REENTER;

	/* kill() and sigprocmask() have to be done specially. */

	i = VSIGHUP;

	sig_index = i;

	sig_arg = sig_kill_reenter;

	globok = 0; 
	if(cpsig(ch_tkill, pa_tkillprocmask, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	sig_arg = sig_proc_reenter;

	globok = 0; 
	if(cpsig(ch_tprocmask, pa_tkillprocmask, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;
		
	if(test_deleted)
		DBUG_VOID_RETURN;

	/* Test the rest of the functions in a loop */

	cln_tfuncs();	/* In case last test was aborted */

	sig_arg = sig_reenter;
	sig_cont = SIGCONT;

	/* Number of test functions to test for signal effects */
	nfuncs = sizeof(test_array) / sizeof(struct test_struct);

	DBUG_PRINTF("test30","nfuncs is %d",nfuncs);
	for(j = 0; j < nfuncs; j++) {

		test_fn = test_array[j].testfuncs;
		test_name = test_array[j].testnames;
		test_num = test_array[j].testnum;
		test_type = test_array[j].testtypes;
		pa_fn = test_array[j].pafuncs;
		test_ret = test_array[j].testreturns;

		globok = 0;
		init_tfuncs();
		PATH_FUNC_TRACE;

		if(test_deleted)
			DBUG_VOID_RETURN;

		if (test_num != T_TCDRAIN || gti_supp)
		{
			globok = 0; 
			if(cpsig(ch_t30, pa_t30, WAITTIME, 0) == SYSERROR)
			{
				test_deleted++;
			}
			else
				PATH_FUNC_TRACE;
		}
		else
			PATH_TRACE;

		cln_tfuncs();

		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT((nfuncs*2)+2); 

	DBUG_VOID_RETURN;
}

private void
ch_t30()
{
	int	sg_num  = sig_array[sig_index];
	int	pathok = 0;

	DBUG_ENTER("ch_t30");

	/* Initialise required signal action for test signal */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	sig_recvd = 0;

	/* Make a child if this test waits for child */
	if(test_type == TEST_WAIT)
		/* ensure wait() never returns but is re-entered repeatedly */
		ch_fork(3 * TESTTIME);

	/* Call the test function */

	globok = 0;
	(*test_fn)();
	PATH_FUNC_TRACE;

	/* If function returns, exit to indicate error */
	if(test_type == TEST_WAIT)
		(void) kill((pid_t)local_child, SIGKILL);
	xx_rpt(DELETION);
	in_rpt("deletion reason - %s returned %d errno %d", test_name, sig_ret,
								sig_errno);
	DBUG_EXIT(A_TSETDIR);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t30()
{
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t30");

	sig_err = 0;

	/* Wait - should time out if child really is suspended */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_TSETDIR) {
			PATH_FUNC_RPT(1); 
			test_deleted++;
			DBUG_VOID_RETURN;
		} else if(ex_code == A_MISC) {
			PATH_FUNC_RPT(1); 
			test_fail++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("test function %s did not suspend process", test_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the test signal */
	if(kill(tet_child, sg_num) < 0 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
						EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_TSETDIR) {
			PATH_FUNC_RPT(5); 
			test_deleted++;
			DBUG_VOID_RETURN;
		} else if(ex_code == A_MISC) {
			PATH_FUNC_RPT(5); 
			test_fail++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE; 
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("test function %s returned unexpectedly after re-entry",
								test_name);
		in_rpt("from signal handler - signal %d (%s)", sg_num, sg_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the test signal again to halt the test */
	if(kill(tet_child, sg_num) < 0 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
						EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait again - ensure child caught signal */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: child process timed out");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure that in the child process, the signal was caught */
	if(stat_child != CHLD_EXIT || ex_code != sg_num) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("test function %s not re-entered via signal catching function", test_name);
		in_rpt("expected action: process caught signal %d", sg_num);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(11); 

	DBUG_VOID_RETURN;
}

private void
ch_fork(waittime)
int	waittime;
{
	int	err;

	DBUG_ENTER("ch_fork");

	/*
	 * Make a child using fork() for use in wait() testing.
	 *
	 * This function only returns in the parent side of the fork().
	 */

	local_child = fork();
	switch(local_child) {
	case SYSERROR:
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed, errno %d", err);
		DBUG_EXIT(A_TSETDIR);
	case 0:	
		tet_setcontext();
		/* Sleep for long enough that we can re-enter wait() */
		(void) sleep((unsigned)waittime);

		/* Should not get here - should get killed off */
		DBUG_EXIT(OK);
	default:
		tet_setblock();
		break;
	}

	DBUG_VOID_RETURN;
}

public void
test31()
{
	int	nfuncs;
	int	i;
	int	j;
	int	pathok = 0;

	DBUG_ENTER("test31");

	/*
	 * Test that functions can be terminated by signals.
	 *
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */

	test_fail = 0;
	test_deleted = 0;
	test_case = TYPE_ABORT;

	cln_tfuncs();	/* In case last test was aborted */

	i = VSIGHUP;

	sig_index = i;
	sig_arg = SIG_DFL;
	sig_cont = SIGCONT;

	/* Number of test functions to test for signal effects */
	nfuncs = sizeof(test_array) / sizeof(struct test_struct);
	DBUG_PRINTF("test31","nfuncs is %d",nfuncs);
	for(j = 0; j < nfuncs; j++) {

		test_fn = test_array[j].testfuncs;
		test_name = test_array[j].testnames;
		test_num = test_array[j].testnum;
		test_type = test_array[j].testtypes;
		pa_fn = test_array[j].pafuncs;
		test_ret = test_array[j].testreturns;

		globok = 0;
		init_tfuncs();
		PATH_FUNC_TRACE;

		if(test_deleted)
			DBUG_VOID_RETURN;

		if (test_num != T_TCDRAIN || gti_supp)
		{
			globok = 0; 
			if(cpsig(ch_t30, pa_t31, WAITTIME, 0) == SYSERROR)
			{
				test_deleted++;
			}
			else
				PATH_FUNC_TRACE;
		}
		else
			PATH_TRACE;
				
		cln_tfuncs();

		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT(nfuncs*2); 

	DBUG_VOID_RETURN;
}

private void
pa_t31()
{
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t31");

	sig_err = 0;

	/* Wait - should time out if child really is suspended */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child == CHLD_EXIT)
	{
		if(ex_code == A_TSETDIR)
		{
			PATH_TRACE; 
			PATH_FUNC_RPT(2); 
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else if(ex_code == A_MISC)
		{
			PATH_TRACE; 
			PATH_FUNC_RPT(2); 
			test_fail++;
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("test function %s did not suspend process", test_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the test signal */
	if(kill(tet_child, sg_num) < 0 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
						EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is terminated */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_TSETDIR) {
			PATH_TRACE; 
			PATH_FUNC_RPT(6); 
			test_deleted++;
			DBUG_VOID_RETURN;
		} else if(ex_code == A_MISC) {
			PATH_TRACE; 
			PATH_FUNC_RPT(6); 
			test_fail++;
			DBUG_VOID_RETURN;
		}
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != sg_num) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("test function %s not terminated by signal %d (%s)",
						test_name, sg_num, sg_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(7); 
	DBUG_VOID_RETURN;
}

public void
test32()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test32");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test that functions can be stopped and resumed by signals. 
	 *
	 * This test tests only a home grown function at present.
	 */

	test_fail = 0;
	test_deleted = 0;
	test_case = TYPE_STOP;

	i = VSIGSTOP;

	sig_index = i;
	sig_arg = SIG_DFL;
	sig_cont = SIGCONT;

	test_fn = myfunc.testfuncs;
	test_name = myfunc.testnames;
	test_num = myfunc.testnum;
	test_type = myfunc.testtypes;
	pa_fn = myfunc.pafuncs;
	test_ret = myfunc.testreturns;

	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0; 
	if(cpsig(ch_t34and35, pa_t34and35, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

public void
test33()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test33");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test that functions can be stopped and killed by signals. 
	 *
	 * This test tests only a home grown function at present.
	 */

	test_fail = 0;
	test_deleted = 0;
	test_case = TYPE_STOP;

	i = VSIGSTOP;

	sig_index = i;
	sig_arg = SIG_DFL;
	sig_cont = SIGKILL;

	test_fn = myfunc.testfuncs;
	test_name = myfunc.testnames;
	test_num = myfunc.testnum;
	test_type = myfunc.testtypes;
	pa_fn = myfunc.pafuncs;
	test_ret = myfunc.testreturns;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(cpsig(ch_t34and35, pa_t34and35, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE; 

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

public void
test34()
{
	int	pathok = 0;
	DBUG_ENTER("test34");

	prep_sigs();  /* Set up default signal handlers */

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test that functions can be stopped and resumed by signals 
	 * without the interruption or resumption being reported by the 
	 * function. This should happen if signals take default action.
	 */

	test_fail = 0;
	test_deleted = 0;
	test_case = TYPE_STOP;

	globok = 0;
	do_34and35(SIG_DFL);
	PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

public void
test35()
{
	int	pathok = 0;
	DBUG_ENTER("test35");

	prep_sigs();

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test that functions can be stopped and resumed by signals 
	 * and the interruption or resumption is reported by the 
	 * function. This should happen if signals are caught.
	 */

	test_fail = 0;
	test_deleted = 0;
	test_case = TYPE_STOP;

	globok = 0;
	do_34and35(sig_catch);
	PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
do_34and35(arg)
void	(*arg)();	/* One of SIG_IGN, SIG_DFL, signal catching function */
{
	int	nfuncs;
	int	i;
	int	j;
	int	pathok = 0;

	DBUG_ENTER("do_34and35");

	/*
	 * Test signal action for stop and SIGCONT signals on other functions.
	 *
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */

	cln_tfuncs();	/* In case last test was aborted */

	i = VSIGTSTP;	/* Choose a stop signal that can be caught */

	sig_index = i;
	sig_arg = arg;
	sig_cont = SIGCONT;

	/* Number of test functions to test for signal effects */
	nfuncs = sizeof(test_array) / sizeof(struct test_struct);

	for(j = 0; j < nfuncs; j++) {

		test_fn = test_array[j].testfuncs;
		test_name = test_array[j].testnames;
		test_num = test_array[j].testnum;
		test_type = test_array[j].testtypes;
		pa_fn = test_array[j].pafuncs;
		test_ret = test_array[j].testreturns;

		globok = 0;
		init_tfuncs();
		PATH_FUNC_TRACE;
		
		if(test_deleted)
			DBUG_VOID_RETURN;

		if (test_num != T_TCDRAIN || gti_supp)
		{
			globok = 0; 
			if(cpsig(ch_t34and35, pa_t34and35, WAITTIME, 0) == SYSERROR)
			{
				test_deleted++;
			}
			else
				PATH_FUNC_TRACE;
		}
		else
			PATH_TRACE;

		cln_tfuncs();

		if(test_deleted)
			DBUG_VOID_RETURN;
	}
	PATH_FUNC_RPT(nfuncs*2); 

	DBUG_VOID_RETURN;
}

private void
ch_t34and35()
{
	int	sg_num  = sig_array[sig_index];
	int	pathok = 0;
 	DBUG_ENTER("ch_t34and35");

	/* make sure the process is not in an orphaned process group */
	(void) setpgid((pid_t)0, (pid_t)0);

	/* Initialise required signal action for sig_cont signal */
	(void) sig_init(sig_cont, &sig_empty, sig_arg, 0);

	/* Initialise required signal action for test signal */
	(void) sig_init(sg_num, &sig_empty, SIG_DFL, 0);

	sig_recvd = 0;

	/* Make a child if this test waits for child */
	if(test_type == TEST_WAIT)
		/* Carefully try to wake process when parent has had SIGCONT */
		ch_fork(TESTTIME + WAITTIME);

	/* Initialise so as to force error if variables not reset */
	if(sig_arg == SIG_DFL) {
		sig_ret = test_ret;
		sig_errno = EINTR;
	} else {
		sig_ret = test_ret + 1;
		sig_errno = NOERROR;
	}

	/* Call the test function */
	(*test_fn)();

	if(test_type == TEST_WAIT)
		(void) kill((pid_t)local_child, SIGKILL);

	if(test_ret == ZERORET) {
		/*
		 * No error defined for these interrupted calls. 
		 * Make sure none is reported!
		 */
		if(sig_arg == SIG_DFL) {
			sig_ret = test_ret + 1;
			sig_errno = NOERROR;
		} else {
			sig_ret = test_ret;
			sig_errno = EINTR;
		}
	}

	/* If signal not caught but function interrupted, indicate error */
	if(sig_arg == SIG_DFL && (sig_ret == test_ret && sig_errno == EINTR)) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("function reported interruption by stopping or SIGCONT signal");
		in_rpt("%s returned %d errno %d", 
						test_name, sig_ret, sig_errno);
		DBUG_EXIT(A_MISC);
	}
	else
		PATH_TRACE;

	/* If signals caught but function gave no error, indicate error */
	if(sig_arg != SIG_DFL && (sig_ret != test_ret || sig_errno != EINTR)) {
		if(!test_fail);
			xx_rpt(FAILURE);
		in_rpt("function reported no interruption by stopping or SIGCONT signal");
		in_rpt("%s returned %d errno %d, should return %d errno %d", 
				test_name, sig_ret, sig_errno, test_ret, EINTR);
		DBUG_EXIT(A_MISC);
	}
	else
		PATH_TRACE;

	/* Otherwise exit with any signal code */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t34and35()
{
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t34and35");

	sig_err = 0;

	/* Wait - should time out if child really is suspended */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_TSETDIR) {
			PATH_FUNC_RPT(1); 
			test_deleted++;
			DBUG_VOID_RETURN;
		} else if(ex_code == A_MISC) {
			PATH_FUNC_RPT(1); 
			test_fail++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - %s did not suspend process", 
								test_name);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the test signal which should stop the process */
	if(kill(tet_child, sg_num) < 0 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
						EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Sleep to prove child really is stopped */
	(void) sleep(WAITTIME);

	/* Ensure the child is still there i.e. it really did stop */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_TSETDIR) {
			PATH_FUNC_RPT(6); 
			test_deleted++;
			DBUG_VOID_RETURN;
		} else if(ex_code == A_MISC) {
			PATH_FUNC_RPT(6); 
			test_fail++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_STOP) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s not stopped by signal %d (%s)",
						test_name, sg_num, sg_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Function to continue suspended process when SIGCONT is received */

	globok = 0;
	(*pa_fn)();
	PATH_FUNC_TRACE;

	/* Send the SIGCONT or SIGKILL signal to resume the test */
	if(kill(tet_child, sig_cont) < 0 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
						EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Wait again - ensure child function reports no interrupt */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_STOP) {
		xx_rpt(DELETION);
		in_rpt("process failed to continue after generating signal %d", 
							sig_cont);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		if(sig_cont == SIGKILL) {
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("signal %d (%s) failed to terminate %s",
					sig_cont, signame(sig_cont), test_name);
			test_fail++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		/* This is correct action for some tests eg pause() */
		if(sig_arg == SIG_DFL && test_type == TEST_HANG)
		{
			PATH_FUNC_RPT(12); 
			DBUG_VOID_RETURN;
		}
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s failed to return within expected time interval", 
								test_name);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Child exitted with a message */
	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_TSETDIR) {
			PATH_FUNC_RPT(12); 
			test_deleted++;
			DBUG_VOID_RETURN;
		} else if(ex_code == A_MISC) {
			PATH_FUNC_RPT(12); 
			test_fail++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	/* The SIGKILL signal should have killed the process */
	if(sig_cont == SIGKILL) {
		if(stat_child == CHLD_ABRT && sg_code == sig_cont)
		{
			PATH_FUNC_RPT(13); 
			DBUG_VOID_RETURN;
		}
		
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %d (%s) failed to terminate %s",
				sig_cont, signame(sig_cont), test_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	/* Functions which hang should have already led to CHLD_LIVE */
	if(sig_arg == SIG_DFL && test_type == TEST_HANG) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s returned unexpectedly after sending signal %d (%s)", 
					test_name, sig_cont, signame(sig_cont));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Other functions should have exitted OK */
	if(stat_child == CHLD_EXIT)
	{
		PATH_FUNC_RPT(15); 
		DBUG_VOID_RETURN;
	}

	/* Only possibility is termination due to a signal */
	if(!test_fail)
		xx_rpt(FAILURE);
	in_rpt("%s did not return as expected", test_name);
	in_rpt("observed action: %s", sig_child(stat_child));
	test_fail++;

	DBUG_VOID_RETURN;
}

public void
test36()
{
	int	nfuncs;
	int	i;
	int	j;
	int	pathok = 0;

	DBUG_ENTER("test36");

	prep_sigs();

	/*
	 * Test that functions are not affected by ignored signals.
	 *
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */

	test_fail = 0;
	test_deleted = 0;
	test_case = TYPE_IGNORE;

	cln_tfuncs();	/* In case last test was aborted */

	i = VSIGHUP;

	sig_index = i;
	sig_arg = SIG_IGN;
	sig_cont = SIGCONT;

	/* Number of test functions to test for signal effects */
	nfuncs = sizeof(test_array) / sizeof(struct test_struct);

	for(j = 0; j < nfuncs; j++) {

		test_fn = test_array[j].testfuncs;
		test_name = test_array[j].testnames;
		test_num = test_array[j].testnum;
		test_type = test_array[j].testtypes;
		pa_fn = test_array[j].pafuncs;
		test_ret = test_array[j].testreturns;

		globok = 0;
		init_tfuncs();
		PATH_FUNC_TRACE;

		if(test_deleted)
			DBUG_VOID_RETURN;

		if (test_num != T_TCDRAIN || gti_supp)
		{
			globok = 0; 
			if(cpsig(ch_t36, pa_t36, WAITTIME, 0) == SYSERROR)
			{
				test_deleted++;
			}
			else
				PATH_FUNC_TRACE;
		}
		else
			PATH_TRACE;

		cln_tfuncs();

		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT(nfuncs*2); 

	DBUG_VOID_RETURN;
}

private void
ch_t36()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t36");

	/* Initialise required signal action for test signal */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	sig_recvd = 0;

	/* Make a child if this test waits for child */
	if(test_type == TEST_WAIT)
		ch_fork(TESTTIME);

	/* Call the test function */
	(*test_fn)();

	if(test_type == TEST_WAIT)
		(void) kill((pid_t)local_child, SIGKILL);

	/* Exit with signal received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t36()
{
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t36");

	sig_err = 0;

	/* Wait - should time out if child really is suspended */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_TSETDIR) {
			PATH_FUNC_RPT(1); 
			test_deleted++;
			DBUG_VOID_RETURN;
		} else if(ex_code == A_MISC) {
			PATH_FUNC_RPT(1); 
			test_fail++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("test function %s did not suspend process", test_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the test signal */
	if(kill(tet_child, sg_num) < 0 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
						EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Function to continue suspended process straight away */

	globok = 0;
	(*pa_fn)();
	PATH_FUNC_TRACE;
	
	/* Wait - ensure child ignored signal */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		/* This is correct action for some tests eg pause() */
		if(test_type == TEST_HANG)
		{
			PATH_FUNC_RPT(6); 
			DBUG_VOID_RETURN;
		}
		xx_rpt(DELETION);
		in_rpt("deletion reason: child process timed out");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_TSETDIR) {
			PATH_FUNC_RPT(7); 
			test_deleted++;
			DBUG_VOID_RETURN;
		} else if(ex_code == A_MISC) {
			PATH_FUNC_RPT(7); 
			test_fail++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	/* Functions which hang should have already led to CHLD_LIVE */
	if(test_type == TEST_HANG) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s returned unexpectedly after sending ignored signal %d (%s)", 
						test_name, sg_num, sg_name);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure that in the child process, the signal was ignored */
	if(stat_child != CHLD_EXIT || ex_code != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("ignored signal %d (%s) caused wrong action in %s", 
						sg_num, sg_name, test_name);
		in_rpt("expected action: process ignored signal %d", sg_num);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(10); 
	DBUG_VOID_RETURN;
}

public void
test37()
{
	int	nfuncs;
	int	i;
	int	j;
	int	pathok = 0;
	
	DBUG_ENTER("test37");

	prep_sigs();

	/*
	 * Test that functions are not affected by blocked signals until they
	 * are delivered.
	 *
	 * The signal is caught when it is unblocked after the test function
	 * has returned.
	 *
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */

	test_fail = 0;
	test_deleted = 0;
	test_case = TYPE_IGNORE;

	cln_tfuncs();	/* In case last test was aborted */

	i = VSIGHUP;

	sig_index = i;
	sig_arg = sig_catch;
	sig_cont = SIGCONT;

	/* Number of test functions to test for signal effects */
	nfuncs = sizeof(test_array) / sizeof(struct test_struct);

	for(j = 0; j < nfuncs; j++) {

		test_fn = test_array[j].testfuncs;
		test_name = test_array[j].testnames;
		test_num = test_array[j].testnum;
		test_type = test_array[j].testtypes;
		pa_fn = test_array[j].pafuncs;
		test_ret = test_array[j].testreturns;

		globok = 0;
		init_tfuncs();
		PATH_FUNC_TRACE;

		if(test_deleted)
			DBUG_VOID_RETURN;

		if (test_num != T_TCDRAIN || gti_supp)
		{
			globok = 0; 
			if(cpsig(ch_t37, pa_t37, WAITTIME, 0) == SYSERROR)
			{
				test_deleted++;
			}
			else
				PATH_FUNC_TRACE;
		}
		else
			PATH_TRACE;

		cln_tfuncs();

		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT(nfuncs * 2); 

	DBUG_VOID_RETURN;
}

private void
ch_t37()
{
	sigset_t set;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];

	DBUG_ENTER("ch_t37");

	/* Initialise required signal action for test signal */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	/* Call sigprocmask() to block the signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_EXIT(A_TSETDIR);
	}

	/* Make sure the signal is still blocked in sigsuspend() tests */
	(void) sigaddset(&sig_suspend, sg_num);

	sig_recvd = 0;

	/* Make a child if this test waits for child */
	if(test_type == TEST_WAIT)
		/* Ensure wait() returns after blocked signal received */
		ch_fork(TESTTIME);

	/* Call the test function */
	(*test_fn)();

	if(sig_recvd) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("blocked signal %d (%s) received in %s", 
						sg_num, sg_name, test_name);
		ex_code = A_MISC;
	} else {
		/* Unblock the signal to prove it is received */
		(void) sigemptyset(&set);
		(void) sigaddset(&set, sg_num);
		(void) sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL);

		ex_code = sig_recvd;
	}

	if(test_type == TEST_WAIT)
		(void) kill((pid_t)local_child, SIGKILL);

	/* Exit with signal received if any */
	DBUG_EXIT(ex_code);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t37()
{
	int	err;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t37");

	sig_err = 0;

	/* Wait - should time out if child really is suspended */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_TSETDIR) {
			PATH_FUNC_RPT(1); 
			test_deleted++;
			DBUG_VOID_RETURN;
		} else if(ex_code == A_MISC) {
			PATH_FUNC_RPT(1); 
			test_fail++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("test function %s did not suspend process", test_name);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the test signal */
	if(kill(tet_child, sg_num) < 0 ) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
						EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Function to continue suspended process straight away */

	globok = 0;
	(*pa_fn)();
	PATH_FUNC_TRACE;
	
	/* Wait - ensure child caught signal */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_LIVE) {
		/* This is correct action for some tests eg pause() */
		if(test_type == TEST_HANG)
		{
			PATH_FUNC_RPT(6); 
			DBUG_VOID_RETURN;
		}
		xx_rpt(DELETION);
		in_rpt("deletion reason: child process timed out");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_TSETDIR) {
			PATH_FUNC_RPT(7); 
			test_deleted++;
			DBUG_VOID_RETURN;
		} else if(ex_code == A_MISC) {
			PATH_FUNC_RPT(7); 
			test_fail++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	/* Functions which hang should have already led to CHLD_LIVE */
	if(test_type == TEST_HANG) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s returned unexpectedly after sending blocked signal %d (%s)", 
						test_name, sg_num, sg_name);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure that in the child process, the signal was caught */
	if(stat_child != CHLD_EXIT || ex_code != sg_num) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("blocked signal %d (%s) caused wrong action in %s", 
						sg_num, sg_name, test_name);
		in_rpt("expected action: process caught signal %d", sg_num);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(10); 

	DBUG_VOID_RETURN;
}

private void
t_tcdrain()
{
	int	pathok = 0;
	DBUG_ENTER("t_tcdrain");

	/* Create a new background process group, so can be stopped by signal */
	/* Only necessary if job control is supported */

	if (job_control && setpgid((pid_t)0, (pid_t)0) == SYSERROR)
	{
		xx_rpt(DELETION);
		in_rpt("setpgid(0, 0) failed in preparation");
		DBUG_EXIT(A_MISC);
	}
	else
		PATH_TRACE;

	sig_ret = tcdrain(tty_fildes);
	sig_errno = errno;

	PATH_FUNC_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
t_fcntl()
{
	DBUG_ENTER("t_fcntl");

	sig_ret = fcntl(fd_fcntl, F_SETLKW, &lock_info);
	sig_errno = errno;

	globok = 1;

	DBUG_VOID_RETURN;
}

private void
t_sleep()
{
	/*
	 * It is acceptable for sleep(), when it is re-entered, so
	 * be interrupted by the previously set sleep(). Thus it may
	 * not sleep for the requested time period. This is what we 
	 * would expect if sleep() is implemented using alarm() and pause()
	 * primitives - the previously set alarm time must be respected in
	 * case the application (rather than another sleep() ) set it.
	 */

	static 	int sleep_entry = 0;
	time_t	t1, t2;
	unsigned sleep_ret;
	int	sleep_err;

	DBUG_ENTER("t_sleep");

	if(test_case == TYPE_REENTER) {
		sleep_entry++;

		if(sleep_entry > 1) {
			t1 = time((time_t *)NULL);
			sleep_ret = sleep(TESTTIME);
			sleep_err = errno;
			t2 = time((time_t *)NULL);

			/* 
			 * Based on the above comment, the alarm might occur
			 * after (TESTTIME - WAITTIME) , i.e. 2*SPEEDFACTOR
			 * (because parent signals child after WAITTIME)
			 * If we slept for at least half the expected time, 
			 * sleep again for the unslept time , otherwise count it
			 * as an error.
			 */
			if(t2 - t1 > SPEEDFACTOR) {
				sleep_ret = sleep((unsigned)(TESTTIME - (t2 - t1)));
				sleep_err = errno;
			}

		} else {
			sleep_ret = sleep(TESTTIME);
			sleep_err = errno;
		}
		sig_ret = sleep_ret;
		sig_errno = sleep_err;
	} else {
		sig_ret = sleep(TESTTIME);
		sig_errno = errno;
	}

	globok = 1; 
	
	DBUG_VOID_RETURN;
}

private void
t_pause()
{
	DBUG_ENTER("t_pause");

	sig_ret = pause();
	sig_errno = errno;

	globok = 1;

	DBUG_VOID_RETURN;
}

private void
t_sigsusp()
{
	DBUG_ENTER("t_sigsusp");

	sig_ret = sigsuspend(&sig_suspend);
	sig_errno = errno;
	globok = 1;
	DBUG_VOID_RETURN;
}

private void
t_read()
{
	char	buffer[1];

	DBUG_ENTER("t_read");

	sig_ret = read(pipe_empty[READ], buffer, sizeof(char));
	sig_errno = errno;
	globok = 1;

	DBUG_VOID_RETURN;
}

private void
t_write()
{
	char	buffer[1];

	DBUG_ENTER("t_write");

	sig_ret = write(pipe_full[WRITE], buffer, sizeof(char));
	sig_errno = errno;
	globok = 1;

	DBUG_VOID_RETURN;
}

private void
t_open()
{
	DBUG_ENTER("t_open");

	sig_ret = open(t30_fifo, O_WRONLY);
	sig_errno = errno;
	globok = 1;

	DBUG_VOID_RETURN;
}

private void
t_wait()
{
	int	status;

	DBUG_ENTER("t_wait");

	sig_ret = wait(&status);
	sig_errno = errno;
	globok = 1;

	DBUG_PRINTF("t_wait", "wait returns %d", sig_ret);

	DBUG_VOID_RETURN;
}

private void
t_waitpid()
{
	int	status;

	DBUG_ENTER("t_waitpid");

	sig_ret = waitpid((pid_t)local_child, &status, WUNTRACED);
	sig_errno = errno;
	globok = 1;

	DBUG_VOID_RETURN;
}

private void
t_myfunc()
{
	time_t	tm_start;

	DBUG_ENTER("t_myfunc");

	tm_start = time((time_t *)NULL);
	for( ;; ) {
		if(time((time_t *)NULL) > tm_start + TESTTIME)
			break;
	}
	globok = 1;
	DBUG_VOID_RETURN;
}

private void
pa_dummy()
{
	/* Nothing needed to make us continue processing */
	globok = 1; 	/* Well, actually need this for path checking */
}

private void
pa_fcntl()
{
	DBUG_ENTER("pa_fcntl");

	lock_info.l_type = F_UNLCK;
	lock_info.l_whence = SEEK_SET;
	lock_info.l_start = (off_t)0;
	lock_info.l_len = sizeof(int);
	lock_info.l_pid = (pid_t)0;
	(void) fcntl(fd_fcntl, F_SETLK, &lock_info);
	globok = 1;
	DBUG_VOID_RETURN;
}

private void
pa_read()
{
	char	buffer[1];
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_read");

	/* Write to the empty pipe */
	if(write(pipe_empty[WRITE], buffer, sizeof(char)) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - write() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
pa_write()
{
	char	buffer[512];
	int	ret, err;
	int	pathok = 0;

	DBUG_ENTER("pa_write");

	/* Set non-blocking mode on reading side of pipe */
	ret = fcntl(pipe_full[READ], F_GETFL);
	if (ret == SYSERROR ||
		fcntl(pipe_full[READ], F_SETFL, ret|O_NONBLOCK) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - fcntl() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Empty pipe completely to ensure write() will unblock */
	while ((ret = read(pipe_full[READ], buffer, sizeof(buffer))) > 0)
		;
	if (ret < 0 && errno != EAGAIN)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - read() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
pa_open()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_open");

	/* Open the fifo for reading */
	fd_rdfifo = open(t30_fifo, O_RDONLY|O_NONBLOCK);
	if(fd_rdfifo < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - open() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	PATH_FUNC_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
sig_reenter(sg_num)
int	sg_num;
{
	private int entry = 0;
	private	sigset_t set;		/* signal mask */

	DBUG_ENTER("sig_reenter");

	entry++;

	DBUG_PRINTF("sig_reenter", "sg_num %d", sg_num);

	/* Second time we enter - exit with signal code last set */
	if(entry > 1) {
		if(test_type == TEST_WAIT)
			(void) kill((pid_t)local_child, SIGKILL);
		DBUG_EXIT(sig_recvd);
	}

	sig_recvd = sg_num;

	/* Call sigprocmask() to unblock signal */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	(void) sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL);

	/* Call the test function */
	(*test_fn)();

	/* If function returns, exit to indicate error */
	if(test_type == TEST_WAIT)
		(void) kill((pid_t)local_child, SIGKILL);
	if(!test_fail)
		xx_rpt(FAILURE);
	in_rpt("%s returned instead of blocking after re-entry", test_name);
	in_rpt("function returned %d errno %d", sig_ret, sig_errno);
	DBUG_EXIT(A_MISC);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}
