/*
*      SCCS:  @(#)POSIX.os/procprim/signal/sigcon5.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		T.signal

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for signal concepts

AUTHOR:		Neil Moses, UniSoft Ltd.
DATE CREATED:	Tue May 19 14:26:47 BST 1992
MODIFICATIONS:

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<fcntl.h>
#include	<dbug.h>
#include	<report.h>
#include	<wait_child.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<signal.h>
#include	<setjmp.h>
#include	<vsx_signal.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<sys/wait.h>
#include	<stdio.h>
#include	<errno.h>
#include	<termios.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigconcept
#endif


#define	WAITTIME		(5*SPEEDFACTOR)
#define	READ			0
#define	WRITE			1
#define	TEST_END		0
#define	TEST_HANG		1
#define	TEST_WAIT		2
#define PATH_NOK		16
#define	TEST_SIGNAL		3
#define	TEST_SIGACTION		4
#define	TEST_SETPGID		5
#define	TEST_KILL		6
#define	TEST_PPID		7
#define	TEST_SUCCESS		1
#define	TEST_FAIL		2
#define NAP			4
#define NOK			02
#define NO			0
#define YES			1

extern	int		no_tests;
extern	char		*status_child();
extern	char		*errname();
extern  void            setsigs();
extern	int		cppair();

extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sig_type[];
extern	int		ex_code;
extern	int		sg_code;

extern	sigset_t	sig_empty;
extern int		sig_recvd;
extern int		test_fail;
extern int		test_deleted;
extern int		job_control;

private	int 		tty_fildes, loop_fildes;
private struct termios	tty_termios, *tty_termios_p = &tty_termios,
			loop_termios, *loop_termios_p = &loop_termios;
private int		sig_cont;
private	int		fg_fd[2];
private	int		bg_fd[2];
private	void		ch_sleep();
private	void		sig_resume();
private	void		interrupt();
private	void		ch_stop();
private	int		ctty();
private	int		testfail;
private	sigjmp_buf	sjmp;

public	void		test38();
public	void		test39();
public	void		test40();
public	void		test41();
public	void		test42();
private	void		ch_t38();
private	void		pa_t38();
private	void		gch_t38();
private	void		pgch_t38();
private	void		ch_t39();
private	void		pa_t39();
private	void		ch_t40();
private	void		pa_t40();
private	void		pgch_t40();
private	void		ch_t40fg();
private	void		ch_t40bg();
private	int		ch_t41();
private	void		pa_t41();
private	void		ch_t42();
private	void		pa_t42();
private	void		pgch_t42();
private	void		sjmpfunc();

extern	void		sig_catch();

public void
test38()
{
	DBUG_ENTER("test38");

	/*
	 * Test that when parent terminates from a signal it
	 * causes children to acquire new parent process ID.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cpsig(ch_t38, pa_t38, WAITTIME, 0);
	
	DBUG_VOID_RETURN;
}

private void
ch_t38()
{
	DBUG_ENTER("ch_t38");

	/*
	 * Turn into a grandchild and it's parent.
	 * The parent terminates after a sleep during which time 
	 * grandchild obtains parent process ID.
	 * The grandchild obtains new parent process ID after it's parent
	 * calls kill(getpid(), SIGKILL), expecting it to have altered.
	 */

	(void) cppair(gch_t38, pgch_t38, WAITTIME, 0);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch_t38()
{
	DBUG_ENTER("pgch_t38");

	/* Exit after a short pause long enough for child to call getppid() */
	(void) sleep(WAITTIME/2);

	(void) kill(getpid(), SIGKILL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
gch_t38()
{
	pid_t	ppid;
	pid_t	ppid_old;
	int	pathok = 0;

	DBUG_ENTER("gch_t38");

	ppid_old = getppid();

	/* Sleep, during which time the parent calls kill() */
	(void) sleep(WAITTIME);

	ppid = getppid();

	if(ppid == ppid_old) {
		xx_rpt(FAILURE);
		in_rpt("parent process ID not altered after parent called kill()");
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t38()
{
	int	stat_child;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("pa_t38");

	/* Wait for child and expect it to have terminated */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != SIGKILL) {
		xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: process terminated - signal %d", SIGKILL);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Sleep further to give grandchild time to report */
	(void) sleep(WAITTIME);

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

public void
test39()
{
	DBUG_ENTER("test39");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}

	/*
	 * Test that kill() causes SIGCHLD to be sent to parent process.
	 */

	test_fail = 0;
	test_deleted = 0;

	(void) cpsig(ch_t39, pa_t39, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t39()
{
	/*
	 * Child process which terminates.
	 * This should cause parent to receive signal SIGCHLD.
	 */
	DBUG_ENTER("ch_t39");

	(void) sleep(WAITTIME/2);

	(void) kill(getpid(), SIGKILL);

	DBUG_VOID_RETURN;
}

private void
pa_t39()
{
	int	pathok = 0;

	DBUG_ENTER("pa_t39");

	/* Set signal catching function for SIGCHLD */
	if(sig_init(SIGCHLD, &sig_empty, sig_catch, 0) < 0) {
		xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
		
	sig_recvd = 0;

	(void) sleep(WAITTIME);

	/* Check we received signal */
	if(!test_fail && sig_recvd != SIGCHLD) {
		xx_rpt(FAILURE);
		in_rpt("SIGCHLD signal not generated when child exitted");
		if(sig_recvd)
			in_rpt("instead signal %d (%s) was generated", 
						sig_recvd, sig_name[sig_recvd]);

		test_fail++;
	}
	else
		PATH_TRACE;

	/* Set default for SIGCHLD so we get no more interruptions */
	if(sig_init(SIGCHLD, &sig_empty, SIG_DFL, 0) < 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed initialising signal action");
		test_fail++;
	}
	else
		PATH_TRACE;

	/* Wait for child process so we don't create a zombie */
	(void) wait_child(WAITTIME);

	if(!test_fail)
		PATH_XS_RPT(3);

	globok = 1; /* globok must be set for cppair */

	DBUG_VOID_RETURN;
}

public void
test40()
{
	char	*cp;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test40");

	/*
	 * Test that kill() sends SIGHUP to foreground processes.
	 */

	cp = tet_getvar("VSX_TERMIOS_TTY");
	if (cp != NULL && strncmp(cp, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	if(pipe(fg_fd) < 0 ||
	   pipe(bg_fd) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);
	(void) cpsig(ch_t40, pa_t40, WAITTIME, 0);

	(void) close(fg_fd[READ]);
	(void) close(fg_fd[WRITE]);
	(void) close(bg_fd[READ]);
	(void) close(bg_fd[WRITE]);

	DBUG_VOID_RETURN;
}

private void
ch_t40()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t40");

	PATH_FUNC_TRACE;

	/*
	 * Turn into two grandchildren and parent.
	 * The parent terminates using kill(getpid(), SIGKILL) after a sleep.
	 * The grandchildren report to the grandparent on what happens.
	 */

	/* Set up a new session */
	if(setsid() == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("setsid() failed in child");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	/* Set up a controlling terminal for this session */
	/* Note: VSX_TERMIOS_TTY=unsup has already been checked for */
	if(tty_prep(&tty_fildes, tty_termios_p) != 0) {
		xx_rpt(DELETION);
		in_rpt("open/initialise failed on VSX_TERMIOS_TTY");
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	/* Produce a process in foreground process group */
	switch(fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		PATH_FUNC_RPT(3);
		ch_t40fg();

		/* NOTREACHED */
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		PATH_TRACE;
		break;
	}
	tet_setblock();
	/* Produce a process in background process group */
	switch(fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		PATH_FUNC_RPT(4);
		ch_t40bg();

		/* NOTREACHED */
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		break;
	}
	tet_setblock();
	pgch_t40();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch_t40()
{
	DBUG_ENTER("pgch_t40");

	/* Exit after a short pause long enough to call sigaction() */
	(void) sleep(WAITTIME/2);

	(void) kill(getpid(), SIGKILL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
ch_t40fg()
{
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("ch_t40fg");

	PATH_FUNC_TRACE;

	if(DBUG_SETJMP(sjmp) != 0) {
		buffer[0] = TEST_SIGNAL;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;
	setsigs(sjmpfunc);

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	sig_recvd = 0;

	/* Sleep, during which time the parent calls kill() */
	(void) sleep(WAITTIME);

	/* Expect signal SIGHUP to have been received */
	if(sig_recvd != SIGHUP) {
		buffer[0] = TEST_FAIL;
	} else {
		PATH_TRACE;
		buffer[0] = TEST_SUCCESS;
	}

	/* Write down pipe to grandparent success code */
	(void) write(fg_fd[WRITE], buffer, sizeof(char));

	/* write path count success flag to grandparent */
	PATH_FUNC_RPT(4);
	buffer[0] = (char) globok;
	(void) write(fg_fd[WRITE], buffer, sizeof(char));
	
	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
ch_t40bg()
{
	char	buffer[1];
	pid_t	ppid;
	pid_t	ppid_old;
	int	pathok = 0;

	DBUG_ENTER("ch_t40bg");

	PATH_FUNC_TRACE;

	if(DBUG_SETJMP(sjmp) != 0) {
		buffer[0] = TEST_SIGNAL;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;
	setsigs(sjmpfunc);

	/* Obtain parent process ID */
	ppid_old = getppid();

	/* Create a new (background) process group for this process */
	errno = 0;
	if(setpgid((pid_t)0, (pid_t)0) < 0) {
		if (errno != ENOSYS) {
			buffer[0] = TEST_SETPGID;
			(void) write(bg_fd[WRITE], buffer, sizeof(char));
			DBUG_EXIT(OK);
		}
		else
			PATH_TRACE;
		(void) setsid();
	}
	else
		PATH_TRACE;

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	sig_recvd = 0;

	/* Sleep, during which time the parent calls kill() */
	(void) sleep(WAITTIME);

	/* Obtain new parent process ID */
	ppid = getppid();

	if(ppid == ppid_old) {
		buffer[0] = TEST_PPID;
	} else {
		/* Expect no SIGHUP signal to have been received */
		if(sig_recvd == SIGHUP)
			buffer[0] = TEST_FAIL;
		else
		{
			PATH_TRACE;
			buffer[0] = TEST_SUCCESS;
		}
	}

	/* Write down pipe to grandparent success code */
	(void) write(bg_fd[WRITE], buffer, sizeof(char));

	/* write path count success flag to grandparent */
	PATH_FUNC_RPT(5);
	buffer[0] = (char) globok;
	(void) write(bg_fd[WRITE], buffer, sizeof(char));

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t40()
{
	int	stat_child;
	int	ret;
	int	err;
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("pa_t40");

	/* Wait for child and expect it to have terminated */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != SIGKILL) {
		xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: process terminated - signal %d", SIGKILL);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Read from pipes to determine success of grandchildren */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_SETPGID:
		xx_rpt(DELETION);
		in_rpt("deletion reason - setpgid() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling kill() did not send signal SIGHUP to foreground process group");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	/* read path check success flag from ch_t40fg */
	globok = 0;
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = (int) buffer[0];
		PATH_FUNC_TRACE;
	}


	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_SETPGID:
		xx_rpt(DELETION);
		in_rpt("deletion reason - setpgid() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_PPID:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("parent process ID not reassigned in background process group");
		test_fail++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling kill() sent signal SIGHUP to background process group");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	/* read path check success flag from ch_t40bg */
	globok = 0;
	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = (int) buffer[0];
		PATH_FUNC_TRACE;
	}

	if(test_fail || test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(9);

	globok = 1; /* globok must be set for cpsig */

	DBUG_VOID_RETURN;
}

public void
test41()
{
	/* When session leader exits, terminal can be reacquired as ctty */
	char	*cp;
	int	pathok = 0;
	int	ret;

	DBUG_ENTER("test41");

	testfail = 0;

	cp = tet_getvar("VSX_TERMIOS_TTY");
	if (cp != NULL && strncmp(cp, "unsup", (size_t)5) == 0)
	{
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY is set to \"unsup\"");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	ret = cpsig((void(*)())ch_t41, pa_t41, NAP + WAITTIME, NOK|PATH_NOK);
	PATH_FUNC_TRACE;

       	if (ret == NOK)
        {
                DBUG_VOID_RETURN;
        }
        else if (ret == PATH_NOK)
                ret = OK;
        else
                PATH_TRACE;

	/* Second call "re-acquires" ctty */
	globok = 0;
	ret = cpsig((void(*)())ch_t41, pa_t41, NAP + WAITTIME, NOK|PATH_NOK);
	PATH_FUNC_TRACE;

       	if (ret == NOK)
        {
                DBUG_VOID_RETURN;
        }
        else if (ret == PATH_NOK)
                ret = OK;
        else
                PATH_TRACE;

	PATH_XS_RPT(5);
}

private void
pa_t41()
{
	int	err, stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_t41");

	errno = NOERROR;
	if((stat_child = wait_child(NAP + WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != SIGKILL) {
		xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: process terminated - signal %d", SIGKILL);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private int
ch_t41()
{
	int	pathok = 0;
	DBUG_ENTER("ch_t41");

	globok = 0; 
	if (ctty() != 0)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	globok = 0;
	interrupt(YES);
	PATH_FUNC_TRACE;

	if(pathok != 2)
		DBUG_EXIT(PATH_NOK);
	else
		(void) kill(getpid(), SIGKILL);

	/* NOTREACHED */
	xx_rpt(DELETION);
	in_rpt("kill(getpid(), SIGKILL) failed");
	DBUG_EXIT(NOK);
}

public void
test42()
{
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test42");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Test that kill() sends SIGHUP and SIGCONT if process is stopped
	 * when a process group is orphaned (by leader exiting).
	 */

	test_fail = 0;
	test_deleted = 0;

	errno = NOERROR;
	if(pipe(fg_fd) < 0 ||
	   pipe(bg_fd) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - pipe() failed errno %d", err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);
	(void) cpsig(ch_t42, pa_t42, WAITTIME, 0);

	(void) close(fg_fd[READ]);
	(void) close(fg_fd[WRITE]);
	(void) close(bg_fd[READ]);
	(void) close(bg_fd[WRITE]);

	DBUG_VOID_RETURN;
}

private void
ch_t42()
{
	int	pathok = 0;

	DBUG_ENTER("ch_t42");

	/*
	 * Do everything that follows in background process group.
	 */

	errno = 0;

	PATH_FUNC_TRACE;
	
	if(setpgid((pid_t)0, (pid_t)0) < 0) {
		if (errno != ENOSYS) {
			vsxerror("setpgid(0, 0) failed - errno %d (%s)",
				errno, errname(errno));
			DBUG_EXIT(A_MISC);
		}
		else
			PATH_TRACE;
		(void) setsid();
	}
	else
		PATH_TRACE;

	/*
	 * Creat process group with one stopped process and one
	 * sleeping process.  Then orphan it by terminating this process
	 * (the group leader).
	 * The grandchildren report to the grandparent on what happens.
	 */

	/* Produce two processes, stop one via SIGSTOP */
	switch(tet_child = fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		PATH_FUNC_RPT(2);
		ch_stop();
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		PATH_TRACE;
		break;
	}
	tet_setblock();
	switch(fork())
	{
	case SYSERROR:
		vsxerror("external error %d", A_FORKFAIL);
		DBUG_EXIT(A_FORKFAIL);

		/* NOTREACHED */
		break;

	case 0:		/* child */
		tet_setcontext();
		PATH_FUNC_RPT(3);
		ch_sleep();
		break;

	default:
		tet_setblock();
		/* parent does nothing yet */
		break;
	}
	tet_setblock();

	pgch_t42();

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch_t42()
{
	DBUG_ENTER("pgch_t42");

	/* Terminate after a short pause long enough to initialise children */
	(void) sleep(WAITTIME/2);

	/* This is the call that is actually tested */
	(void) kill(getpid(), SIGKILL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t42()
{
	int	stat_child;
	int	ret;
	int	err;
	pid_t	pid;
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("pa_t42");

	/* Wait for child and expect it to have terminated */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason - waitpid() failed errno %d", err);
		test_deleted++;
	}
	else if(stat_child != CHLD_ABRT || sg_code != SIGKILL) {
		xx_rpt(DELETION);
		in_rpt("child process terminated incorrectly");
		in_rpt("expected action: process terminated - signal %d", SIGKILL);
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
	}
	else
		PATH_TRACE;

	/* Read from pipe to determine PID of stopped grandchild */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], (char *)&pid, sizeof(pid_t));
	CLEAR_ALARM
	if(ret != sizeof(pid_t)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(pid_t));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Read from pipes to determine success of grandchildren */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		(void) killchild(pid, WAITTIME);
		test_deleted++;
	}
	else
		PATH_TRACE;

	if (test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_KILL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling kill() failed to send SIGHUP and SIGCONT signals");
		in_rpt("to process in orphaned process group which is stopped");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	/* Read from pipes to determine path count success of grandchildren */
	SET_TIMEOUT(WAITTIME)
	ret = read(fg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = (int) buffer[0];
		PATH_FUNC_TRACE;
	}


	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	switch(buffer[0]) {
	case TEST_KILL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		test_deleted++;
		break;
	case TEST_SIGNAL:
		xx_rpt(DELETION);
		in_rpt("deletion reason - unexpected signal received");
		test_deleted++;
		break;
	case TEST_SIGACTION:
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		test_deleted++;
		break;
	case TEST_FAIL:
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("calling kill() failed to send SIGHUP and SIGCONT signals");
		in_rpt("to process in orphaned process group which is not stopped");
		test_fail++;
		break;
	case TEST_SUCCESS:
		PATH_TRACE;
		break;
	}

	/* read from pipe to determine path count success of grandchild */
	globok = 0;
	SET_TIMEOUT(WAITTIME)
	ret = read(bg_fd[READ], buffer, sizeof(char));
	CLEAR_ALARM
	if(ret != sizeof(char)) {
		xx_rpt(DELETION);
		in_rpt("read from pipe returned %d instead of %d", 
							ret, sizeof(char));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
	{
		globok = (int) buffer[0];
		PATH_FUNC_TRACE;
	}

	if(test_fail || test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(10);

	globok = 1; /* globok must be set for cpsig */

	DBUG_VOID_RETURN;
}

private	void
sjmpfunc(sig)
int	sig;
{
	DBUG_ENTER("sjmpfunc");
	DBUG_LONGJMP(sjmp, sig+1);
	DBUG_VOID_RETURN;
}

private void
ch_sleep()
{
	char	buffer[1];
	int	pathok = 0;

	DBUG_ENTER("ch_sleep");

	PATH_FUNC_TRACE;

	if(DBUG_SETJMP(sjmp) != 0) {
		buffer[0] = TEST_SIGNAL;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;
	setsigs(sjmpfunc);

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	/* Initialise the SIGCONT action */
	if(sig_init(sig_array[VSIGCONT], &sig_empty, sig_resume, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(bg_fd[WRITE], buffer, sizeof(char));
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	sig_recvd = 0;

	/* Sleep, during which time the parent calls kill() */
	(void) sleep(WAITTIME);

	/* Allow time for second signal to arrive, if necessary */
	if(sig_recvd != SIGHUP || sig_cont != SIGCONT)
		(void) sleep(SPEEDFACTOR);

	/* Expect signal SIGHUP and SIGCONT to have been received */
	if(sig_recvd != SIGHUP || sig_cont != SIGCONT) {
		buffer[0] = TEST_FAIL;
	} else {
		PATH_TRACE;
		buffer[0] = TEST_SUCCESS;
	}

	/* Write down pipe to grandparent success code */
	(void) write(bg_fd[WRITE], buffer, sizeof(char));

	/* write path success flag to grandparent */
	PATH_FUNC_RPT(5);
	buffer[0] = (char) globok;
	(void) write(bg_fd[WRITE], buffer, sizeof(char));

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
interrupt(toggle)
int toggle;                     /* YES == expecting SIGKILL      */
{                               /* NO == not expecting SIGINT   */
	/* send SIGINT to self and children */

	int err, ret;
	struct sigaction sig;
	unsigned char chr;
	int	pathok = 0;

	DBUG_ENTER("interrupt");

	sig.sa_handler = sig_catch;
	sig.sa_flags = 0;
	(void) sigemptyset(&sig.sa_mask);
	if (sigaction(SIGINT, &sig, NULLSA) == SYSERROR)
	{
		err = errno;
		if (test_fail++ == 0)
			xx_rpt(DELETION);
		in_rpt("sigaction() failed when setting catching function for SIGINT - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        /* for child setup, if any - allow for this in cppair() timeouts */
        (void) sleep(NAP);

        sig_recvd= 0;

	DBUG_PRINTF("interrupt", "c_cc[VINTR] = 0%o",
		(int)tty_termios_p->c_cc[VINTR]);

        chr = tty_termios_p->c_cc[VINTR];

        SET_TIMEOUT(WAITTIME/2)

        ret = write(loop_fildes, (char *)&chr, 1);
	err = errno;
	if (ret == 1)
		(void) pause();

        CLEAR_ALARM

	if (ret != 1)
	{
		if (test_fail++ == 0)
			xx_rpt(DELETION);
		in_rpt("write() to VSX_TERMIOS_LOOP failed - errno %d (%s)",
			err, errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

        if (toggle == YES && sig_recvd != SIGINT)
	{
		if (test_fail++ == 0)
			xx_rpt(FAILURE);
		in_rpt("process did not receive SIGINT when expected");
	}
	else
		PATH_TRACE;

	if (toggle == NO && sig_recvd == SIGINT)
	{
		if (test_fail++ == 0)
		{
			xx_rpt(FAILURE);
		}
		in_rpt("process received SIGINT when not expected");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
ch_stop()
{
	char	buffer[1];
	pid_t	pid = getpid();
	int	pathok = 0;

	DBUG_ENTER("ch_stop");

	PATH_FUNC_TRACE;

	/* Write down pipe to grandparent process ID */
	(void) write(fg_fd[WRITE], (char *)&pid, sizeof(pid_t));

	if(DBUG_SETJMP(sjmp) != 0) {
		buffer[0] = TEST_SIGNAL;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;
	setsigs(sjmpfunc);

	/* Initialise the SIGHUP action */
	if(sig_init(sig_array[VSIGHUP], &sig_empty, sig_catch, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	/* Initialise the SIGCONT action */
	if(sig_init(sig_array[VSIGCONT], &sig_empty, sig_resume, 0) < 0) {
		buffer[0] = TEST_SIGACTION;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	sig_recvd = 0;

	/* Stop ourself */
	if(kill(pid, SIGSTOP) < 0) {
		buffer[0] = TEST_KILL;
		(void) write(fg_fd[WRITE], buffer, sizeof(char));
		DBUG_EXIT(OK);
	}
	else
		PATH_TRACE;

	/* Allow time for second signal to arrive, if necessary */
	if(sig_recvd != SIGHUP || sig_cont != SIGCONT)
		(void) sleep(SPEEDFACTOR);

	/* Expect signal SIGHUP and SIGCONT to have been received */
	if(sig_recvd != SIGHUP || sig_cont != SIGCONT) {
		buffer[0] = TEST_FAIL;
	} else {
		PATH_TRACE;
		buffer[0] = TEST_SUCCESS;
	}

	/* Write down pipe to grandparent success code */
	(void) write(fg_fd[WRITE], buffer, sizeof(char));

	/* write path check success flag to grandparent */
	PATH_FUNC_RPT(6);
	buffer[0] = (char) globok;
	(void) write(fg_fd[WRITE], buffer, sizeof(char));

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private int
ctty()
{
	/* acquire controlling terminal */

	int	pathok = 0;
	DBUG_ENTER("ctty");

	switch (termios_prep(&tty_fildes, tty_termios_p,
			&loop_fildes, loop_termios_p))
	{
	case 0:
		PATH_TRACE; 
		break;
	case 1:
		xx_rpt(UNSUPPORTED);
		in_rpt("VSX_TERMIOS_TTY/LOOP is set to \"unsup\"");
		DBUG_RETURN(-1);
	default:
		if (testfail++ == 0)
			xx_rpt(DELETION);

		if (alrm_flag > 0)
			in_rpt("open/initialise VSX_TERMIOS_TTY/LOOP timed out");
		else
			in_rpt("could not open/initialise VSX_TERMIOS_TTY/LOOP");

		DBUG_RETURN(-1);
	}
	PATH_FUNC_RPT(1); 

	DBUG_RETURN(0);
}

private void
sig_resume(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_resume");

	sig_cont = sg_num;

	DBUG_PRINTF("sig_resume", "sig_cont %d", sig_cont);

	DBUG_VOID_RETURN;
}

