/*
*      SCCS:  @(#)POSIX.os/procprim/signal/sigconcept.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/signal - T.signal Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.signal

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for signal concepts

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Thu Mar 16 12:43:44 GMT 1989
MODIFICATIONS:
		Geoff Clare, 3 Aug 1989
			Do setsid() to protect vtest from signals
			sent to process group.
		
		Geoff Clare, 14 Sep 1989
			The setsid() causes other problems, so remove
			it and make vtest take care of itself

		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG
		
		Geoff Clare, 13 Oct 1989
			Add cleanup routine

		Geoff Clare, 8 Nov 1989
			Make sure stop signals are not tested on orphaned
			process group (unless specified in test).

		Keith Holder, 27 Jun 1990
			Deleted two tests.

		Stuart Boutell, 7 Nov 1990
			tet_child redeclaration removed.


		Stuart Boutell, 4 Dec 1990
			Signal initialisation moved out of prepare_test
			into prep_sigs function.

		Neil Moses, 14 May 1992
			Added new tests 38, 39, 40, 41, 42.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<wait_child.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<time.h>
#include	<sys/stat.h>
#include	<sys/wait.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigconcept
#endif


#define	NO_TESTS		42		/* number of tests in testset */
#define	WRITETIME		(SPEEDFACTOR)
#define	WAITTIME		(5*SPEEDFACTOR)
#define	TESTTIME		(7*SPEEDFACTOR)
#define	EXTERR			"external error"
#define	READ			0
#define	WRITE			1
#define MODEANY			(mode_t)(S_IRWXU|S_IRWXG|S_IRWXO)
#define	CREATMODE		0666
#define	ZERORET			0
#define	WAIT			1
#define	WAITPID			2
#define	TEST_END		0
#define	TEST_HANG		1
#define	TEST_WAIT		2

/* Actions just for test 10 */
#define	TEST_UNBLOCK	1
#define	TEST_IGNORE	2

extern	char		*sig_child();
extern	void		vsx_sigs();
extern	void		valrpt();
extern	int		cppair();
extern	char		*signame();

extern	char		*vg_ident;
extern	char		*vg_dbug;
extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sig_type[];
extern	int		ex_code;
extern	int		sg_code;

public	int	no_tests = NO_TESTS;

public int		job_control;
public int		sig_recvd;
public	void		(*sig_arg)();
public	int		sig_index;
public	sigset_t	sig_empty;
public int		sig_err;
public int		test_fail;
public int		test_deleted;
public int		nsigs;

extern	void		test1();
extern	void		test2();
extern	void		test3();
extern	void		test4();
extern	void		test5();
extern	void		test6();
extern	void		test7();
extern	void		test8();
extern	void		test9();
extern	void		test10();
extern	void		test11();
extern	void		test12();
extern	void		test13();
extern	void		test14();
extern	void		test15();
extern	void		test16();
extern	void		test17();
extern	void		test18();
extern	void		test19();
extern	void		test20();
extern	void		test21();
extern	void		test22();
extern	void		test23();
extern	void		test24();
extern	void		test25();
extern	void		test26();
extern	void		test27();
extern	void		test28();
extern	void		test29();
extern	void		test30();
extern	void		test31();
extern	void		test32();
extern	void		test33();
extern	void		test34();
extern	void		test35();
extern	void		test36();
extern	void		test37();
extern	void		test38();
extern	void		test39();
extern	void		test40();
extern	void		test41();
extern	void		test42();

public	int		sig_init();
public	sigset_t	*sig_mask();
public	int		sigset_ck();
public	void		sig_catch();
public	void		ch_tdefault();
public	void		pa_tdefault();
public	void		prep_sigs();

private	void		prepare_tests();
private	void		clean_tests();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = clean_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	  test2, 2,
	  test3, 3,
	  test4, 4,
	  test5, 5,
	 test6, 6,
	  test7, 7,
	  test8, 8,
	  test9, 9,
	  test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	 test29, 29,
	 test30, 30,
	 test31, 31,
	 test32, 32,
	 test33, 33,
	 test34, 34,
	 test35, 35,
	 test36, 36,
	 test37, 37,
	 test38, 38,
	 test39, 39,
	 test40, 40,
	 test41, 41,
	 test42, 42,
	NULL, 0
};

private	void
prepare_tests()
{
	int	ret;		/* Used for most return values */

	DBUG_ENTER("prepare_tests");

	vsx_sigs();

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		job_control = 0;
		nsigs = VNOJSIG;
	} else {
		job_control = 1;
		nsigs = VNUMSIG;
	}

	(void) sigemptyset(&sig_empty);

	prep_sigs();
	DBUG_VOID_RETURN;
}

public void
prep_sigs()
{
	int	i;
	DBUG_ENTER("prep_sigs");
	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);

	DBUG_VOID_RETURN;
}

private	void
clean_tests()
{
	DBUG_ENTER("clean_tests");

	(void) unlink("core");

	DBUG_VOID_RETURN;
}

public sigset_t *
sig_mask(signum)
int	signum;		/*  a signal to put in mask */
{
	private	sigset_t set;		/* signal mask */

	/* Return address of signal mask */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, signum);

	return(&set);
}

public int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

public void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

private int
sig_ck(signum, name, act, deletion)
int	signum;		/* check whether this signal is blocked/unblocked */
char	*name;		/* name of that signal */
int	act;		/* if SIG_BLOCK, should be blocked */
			/* if SIG_UNBLOCK, should be unblocked */
int	deletion;	/* report deletion, else failure */
{
	/*
	 * Check that a signal is unblocked or not by trying to catch it.
	 * This will not work for SIGKILL or SIGSTOP. It will terminate 
	 * the process.
	 *
	 * This function has the side effect of setting the signal action for
	 * the signal concerned to be caught in sig_catch().
	 */
	int	ret;		/* Used for most return values */

	DBUG_ENTER("sig_ck");

	/* Call sigaction() to initialise signal handling function */
	(void) sig_init(signum, &sig_empty, sig_catch, 0);

	sig_recvd = 0;
	ret = kill(getpid(), signum);
	if(ret < 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		DBUG_RETURN(FAIL);
	}

	if(act == SIG_UNBLOCK && !sig_recvd) {
		xx_rpt(deletion);
		in_rpt("signal %s is blocked, should be unblocked", name);
		DBUG_RETURN(FAIL);
	}
	if(act == SIG_BLOCK && (sig_recvd == signum)) {
		xx_rpt(deletion);
		in_rpt("signal %s is unblocked, should be blocked", name);
		DBUG_RETURN(FAIL);
	}
	if(act == SIG_BLOCK && sig_recvd) {
		xx_rpt(DELETION);
		in_rpt("signal %s unexpectedly delivered when signal %s", 
						signame(sig_recvd), name);
		in_rpt("is generated");
		DBUG_RETURN(FAIL);
	}
	
	DBUG_RETURN(SUCCEED);
}

public int
sigset_ck(sigs, flag)
sigset_t	*sigs;	/* mask of signals that should be blocked */
int		flag;	/* whether failure here requires test deletion */
{
	/*
	 * Check whether signals are blocked/unblocked as per "sigs". 
	 * This ignores SIGKILL or SIGSTOP.
	 *
	 * This function has the side effect of setting the signal action for
	 * all other signals concerned to be caught in sig_catch().
	 */
	int	i;	/* counter for signal array */
	int	signum;	/* signal number */
	char	*name;	/* signal name */

	DBUG_ENTER("sigset_ck");

	for(i = 1; i < nsigs; i++) {
		signum = sig_array[i];
		name   = sig_name[i];

		if(i == VSIGKILL || i == VSIGSTOP)
			continue;

		if(signum == VNOSIG)
			continue;

		if(sigismember(sigs, signum)) {
			if(sig_ck(signum, name, SIG_BLOCK, flag) == FAIL)
				DBUG_RETURN(FAIL);
		} else {
			if(sig_ck(signum, name, SIG_UNBLOCK, flag) == FAIL)
				DBUG_RETURN(FAIL);
		}
	}
	DBUG_RETURN(SUCCEED);
}

public void
ch_tdefault()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_tdefault");

	/* If the signal will stop the process, make sure
	   it's not in an orphaned process group */

	if(sig_type[sg_num] == TYPE_STOP)
		(void) setpgid((pid_t)0, (pid_t)0);

	/* Set the required action if we can */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	sig_recvd = 0;

	(void) kill(getpid(), sg_num);

	/* Exit code is signal number we received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

public void
pa_tdefault()
{
	int	err;
	int	stat_child;
	char *	str_expect;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	sg_type = sig_type[sg_num];
	char	exp_buf[BUFSIZ];
	int	pathok = 0;

	DBUG_ENTER("pa_tdefault");

	sig_err = 0;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Switch on what default action is expected. */
	switch(sg_type) {
	case TYPE_ABRT:	/* child aborts on receiving signal */
		(void) sprintf(exp_buf, "signal %d terminated process", sg_num);
		str_expect = exp_buf;
		if(stat_child != CHLD_ABRT)
			sig_err++;
		if(sg_code != sg_num)
			sig_err++;
		break;
	case TYPE_IGNR:	/* child ignores so exits normally */
	case TYPE_CONT:	/* here, child ignores SIGCONT */
		str_expect = "process ignored signal";
		if(stat_child != CHLD_EXIT)
			sig_err++;
		if(ex_code != 0)
			sig_err++;
		break;
	case TYPE_STOP:	/* child process stops */
		str_expect = "signal stopped process";
		if(stat_child != CHLD_STOP)
			sig_err++;
		break;
	}

	if(sig_err) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caused wrong action", sg_name);
		in_rpt("expected action: %s", str_expect);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;
	PATH_FUNC_RPT(2); 

	DBUG_VOID_RETURN;
}
