/*
*      SCCS:  @(#)  POSIX.os/procprim/signal/signal_t17.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/signal - signal_t17 Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		signal_t17 - test 17 for signal concepts.

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

MODULE CONTAINS:
		tet_main()

SYNOPSIS:
		int main(argc, argv)
			int argc;
			char **argv;

DESCRIPTION:
	  Exec'd by T.signal to test signals are set to SIG_IGN or SIG_DFL
	  on entry to main().

	  usage:	signal_t17 vg_ident vg_dbug

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Dave Cater, Unisoft Ltd.
DATE CREATED:	Tue Mar 21 16:58:35 GMT 1989
MODIFICATIONS:	
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG
		
		Geoff Clare, 19 Oct 1989
			Check calloc() return value.

		Keith Holder, 25 Jun 1990
			Only check for specific signals.

		Anthony Lee, 16 August 1990
			Added resilience code.
	
		Stuart Boutell, 6 Nov 1990
			Eliminated vg_file
	
************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigconcept
#endif


extern	void	set_pname();

extern	char 	*signame();

public	char	*vg_ident =	"<no ident>";
public	char	*vg_dbug =	(char *) NULL;

private	int	test_fail;
private	int	test_deleted;

private	void	subtest17();
private	void	check_action();

public	int
tet_main(argc, argv)

	int	argc;
	char	**argv;
{
 
	set_pname(argc < 1 || argv[0] == (char *) NULL ?
		"signal17" : (argv[0]));

	if (argc == 3)
	{
		if (argv[2] != (char *) NULL)
			vg_dbug = argv[2];
		argc--;
	}
	if (argc == 2)
	{
		if (argv[1] != (char *) NULL)
			vg_ident = argv[1];
		argc--;
	}
	if (argc != 1)
	{
		vsxerror("argument count wrong");
		exit(A_MISC);
	}
	if (vg_dbug != (char *) NULL)
	{
		DBUG_PROCESS(vg_ident);
		DBUG_PUSH(vg_dbug+2);
	}

	subtest17();

	exit(OK);

	/*NOTREACHED*/
	return 0;
}


private void
subtest17()
{
	int	pathok = 0;
	DBUG_ENTER("test17");
	
	test_fail = 0;
	test_deleted = 0;

/*
 * The choice of signal numbers must match
 * those in the main test setup.
 */

	globok = 0;
	check_action(SIGHUP,SIG_DFL);
	PATH_FUNC_TRACE;

	globok = 0;
	check_action(SIGQUIT,SIG_IGN);
	PATH_FUNC_TRACE;

	globok = 0;
	check_action(SIGINT,SIG_DFL);
	PATH_FUNC_TRACE;

	if(!test_fail && !test_deleted) {
		PATH_XS_RPT(3); 
	}

	DBUG_VOID_RETURN;
}

private void
check_action(signo,action)
int	signo;
void	(*action)();
{
	struct sigaction act;
	char	*sg_name ;
	int	pathok = 0;
	
	DBUG_ENTER("check_action");

	sg_name = signame(signo);

	if(sigaction(signo, (struct sigaction *)NULL, &act) < 0) {
		if ( !test_deleted ) {
			xx_rpt(DELETION);
		}
		test_deleted++;
		in_rpt("deletion reason - sigaction() failed - signal %d (%s)",
					signo, sg_name);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if ( act.sa_handler != action ) {
		if ( ! test_fail ) {
			xx_rpt(FAILURE);
		}
		test_fail++;
		in_rpt("signal %d (%s) action not set to %s in main()",signo,
		sg_name,(action == SIG_DFL) ? "SIG_DFL":"SIG_IGN");
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);
	
	DBUG_VOID_RETURN;
}
