/*
*      SCCS:  @(#)POSIX.os/procprim/sigfillset/sigfillset.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/sigfillset - T.sigfillset Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.sigfillset

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the sigfillset() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Wed Mar  1 11:31:40 GMT 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG

		Anthony Lee, 14 August 1990
		Added resilience code.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<stdio.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigfillset
#endif


#define	NO_TESTS		1		/* number of tests in testset */

extern	void		vsx_sigs();

extern	int		sig_array[];
extern	char *		sig_name[];

int	no_tests = NO_TESTS;

private	int	nsigs;	/* How many signals */

private	void		test1();
private	void		prepare_tests();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private	void
prepare_tests()
{
	int	ret;	/* Used for most return values */

	DBUG_ENTER("prepare_tests");

	vsx_sigs();

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1)
		nsigs = VNOJSIG;
	else
		nsigs = VNUMSIG;

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	i;	/* Loop index */
	int	ret;	/* Used for most return values */
	sigset_t  set;	/* Signal set */
	int	pathok = 0;

	DBUG_ENTER("test1");

	/* Call sigfillset() */
	ret = sigfillset(&set);
	
	/* check return value is zero */
	if(ret != 0) {
		xx_rpt(FAILURE);
		in_rpt("sigfillset did not return 0");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check that the signal set is filled (includes all signals) */
	DBUG_PRINTF("test1","nsigs is %d",nsigs);
	for(i = 1; i < nsigs; i++) {
		if(sig_array[i] == VNOSIG)
		{
			PATH_TRACE; 
			continue;
		}
		ret = sigismember(&set, sig_array[i]);
		if(ret != 1) {
			xx_rpt(FAILURE);
			in_rpt("signal %s not included by sigfillset()", 
								sig_name[i]);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	PATH_XS_RPT(nsigs);
	DBUG_VOID_RETURN; 
}

