/*
*      SCCS:  @(#)POSIX.os/procprim/sigismembe/sigismembe.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/sigismembe - T.sigismembe Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.sigismembe

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the sigismember() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Wed Mar  1 11:31:40 GMT 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG

		Anthony Lee, 14 August 1990
		Added resilience code.

		Geoff Clare, 27 June 1991
			VSX_SIGSET_EINVAL="N" for untested result

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigismember
#endif


#define	NO_TESTS		2		/* number of tests in testset */

extern	void		vsx_sigs();
extern	void		valrpt();

extern	int		sig_array[];

int	no_tests = NO_TESTS;

private	int	nsigs;	/* How many signals */

private	void		test1();
private	void		test2();
private	void		prepare_tests();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private	void
prepare_tests()
{
	int	ret;	/* Used for most return values */

	DBUG_ENTER("prepare_tests");

	vsx_sigs();

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1)
		nsigs = VNOJSIG;
	else
		nsigs = VNUMSIG;

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	i;	/* Loop index */
	int	ret;	/* Used for most return values */
	sigset_t  set;	/* Signal set */
	int	pathok = 0;

	DBUG_ENTER("test1");

	/* Call sigemptyset() */
	(void) sigemptyset(&set);
	
	/* Check that the sigismember() determines whether signal is included */
	for(i = 1; i < nsigs; i++) {
		if(sig_array[i] == VNOSIG)
		{
			PATH_TRACE; PATH_TRACE; PATH_TRACE; 
			continue;
		}
		ret = sigismember(&set, sig_array[i]);
		if(ret != 0) {
			xx_rpt(FAILURE);
			in_rpt("sigismember() returned %d instead of 0", ret);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		ret = sigaddset(&set, sig_array[i]);
		if(ret != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigaddset() failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;

		ret = sigismember(&set, sig_array[i]);
		if(ret != 1) {
			xx_rpt(FAILURE);
			in_rpt("sigismember() returned %d instead of 1", ret);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	PATH_XS_RPT(3*(nsigs-1)); 

	DBUG_VOID_RETURN;
}

private void
test2()
{
	char	*cp;
	int	ret;	/* Used for most return values */
	int	err;	/* saved value of errno */
	sigset_t  set;	/* Signal set */
	int	pathok = 0;

	DBUG_ENTER("test2");

	/* Call sigemptyset() purely to initialise */
	(void) sigemptyset(&set);

	/* don't test if EINVAL not implemented for sigsetops */
	cp = tet_getvar("VSX_SIGSET_EINVAL");
	if (cp == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_SIGSET_EINVAL is not set");
		DBUG_VOID_RETURN;
	}
	else if (*cp == 'n' || *cp == 'N')
	{
		xx_rpt(UNTESTED);
		in_rpt("parameter VSX_SIGSET_EINVAL is set to 'N'");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	ret = sigismember(&set, sig_array[VSIGINVAL]);

	if(ret != SYSERROR || errno != EINVAL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sigismember() returns wrong value using invalid signal");
		valrpt(SYSERROR, ret, EINVAL, err);
		DBUG_VOID_RETURN;
	} else
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}
