/*
*      SCCS:  @(#)POSIX.os/procprim/sigpending/sigpending.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/sigpending - T.sigpending Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.sigpending

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the sigpending() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Tue Mar  7 16:15:49 GMT 1989
MODIFICATIONS:
		Geoff Clare, 23 Aug 1989
			Set signals to be caught instead of SIG_DFL
		
		Geoff Clare, 11 Sep 1989
			... but do it in the child, so parent does not
			get SIGCHLD and report deletion.

		Anthony Lee, 14 August 1990
		Added resilience code.

		Stuart Boutell, 7 Nov 1990
			external sig_func replaced with internal code.

		Geoff Clare, 23 June 1994
			Ensure sigsetjmp() is used in line with constraints.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<setjmp.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigpending
#endif


#define	NO_TESTS		1		/* number of tests in testset */
#define	WAITTIME		(SPEEDFACTOR)

extern	void		vsx_sigs();
extern	int		cppair();
extern	char *		signame();

extern	int		sig_array[];
extern	char *		sig_name[];

int	no_tests = NO_TESTS;

private int		nsigs;
private int		jmpsignum;
private	sigjmp_buf	sjmp;

private	void		sig_func();
private	void		test1();
private	void		ch_t1();
private	void		prepare_tests();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private	void
prepare_tests()
{
	int	ret;		/* Used for most return values */

	DBUG_ENTER("prepare_tests");

	vsx_sigs();

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		nsigs = VNOJSIG;
	} else {
		nsigs = VNUMSIG;
	}

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
test1()
{
	DBUG_ENTER("test1");

	(void) cppair(ch_t1, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	sigset_t set;		/* blocked signal set */
	sigset_t oset;		/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	i;		/* counter for signal array */
	int	signum;		/* signal number */
	char	*name;		/* signal name */
	pid_t	myself;		/* process number of this process */
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	(void) sigemptyset(&set);
	DBUG_PRINTF("test1","nsigs is %d",nsigs);

	if(DBUG_SETJMP(sjmp) != 0)
	{
		xx_rpt(DELETION);
		in_rpt("deletion reason: signal %d (%s) received",
				jmpsignum, signame(jmpsignum));
		DBUG_VOID_RETURN;
	}

	for(i = 1; i < nsigs; i++)
	{
		PATH_TRACE; 
		(void) sig_init(sig_array[i], &set, sig_func, 0);
	}

	myself = getpid();

	/* Block all the signals that we can and send them to ourself */
	DBUG_PRINTF("test1","nsigs is %d",nsigs);
	(void) sigemptyset(&set);
	for(i = 1; i < nsigs; i++) {
		signum = sig_array[i];
		name   = sig_name[i];

		/* 
		 * Ignore signals that can't be caught
		 */
		if(i == VSIGKILL || i == VSIGSTOP)
		{
			PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
			continue;
		}

		/*
		 * Ignore SIGCONT since it will be ignored rather than blocked.
		 */
		if(i == VSIGCONT)
		{
			PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
			continue;
		}

		if(signum == VNOSIG)
		{
			PATH_TRACE; PATH_TRACE; PATH_TRACE; PATH_TRACE; 
			continue;
		}

		(void) sigaddset(&set, signum);
		if(sigprocmask(SIG_BLOCK, &set, (sigset_t *)NULL) != 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - sigprocmask() failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
			
		if(kill(myself, signum) < 0) {
			xx_rpt(DELETION);
			in_rpt("deletion reason - kill() failed");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
			
		(void) sigemptyset(&oset);
		if((ret = sigpending(&oset)) != 0) {
			xx_rpt(FAILURE);
			in_rpt("sigpending() returned %d instead of 0", ret);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
			
		if(!sigismember(&oset, signum)) {
			xx_rpt(FAILURE);
			in_rpt("signal %s not set in mask by sigpending()", 
									name);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	PATH_XS_RPT((nsigs-1)+((nsigs-1)*4));

	DBUG_VOID_RETURN;
}

private void
sig_func(sig)
int	sig;
{
	DBUG_ENTER("sig_func");
	jmpsignum = sig;
	DBUG_LONGJMP(sjmp, 1);
	DBUG_VOID_RETURN;
}

