/*
*      SCCS:  @(#)POSIX.os/procprim/sigprocmas/sigprocmas.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/sigprocmas - T.sigprocmas Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.sigprocmas

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the sigprocmask() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Tue Mar  7 16:15:49 GMT 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG

		Anthony Lee, 15 August 1990
			Added resilience code.

		Stuart Boutell, 30 Apr 1991
			Changed cpsig() return code check from FAIL to
			SYSERROR

		Geoff Clare, 22 Nov 1994
			Added declarations of sig_init() and do_7and8().

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigprocmask
#endif


#define	NO_TESTS		10		/* number of tests in testset */
#define	WAITTIME		(SPEEDFACTOR)
#define	EXTERR			"external error"

extern	void		vsx_sigs();
extern	void		valrpt();
extern	char		*signame();
extern	char		*sig_child();
extern	int		cppair();

extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sig_type[];
extern	int		ex_code;
extern	int		sg_code;

int	no_tests = NO_TESTS;

private	void		(*sig_arg)();
private	int		sig_index;
private	sigset_t	sig_empty;
private int		sig_recvd;
private int		nsigs;
private int		job_control;
private int		sig_err;
private int		test_fail;
private int		test_deleted;
private int		setupok;

private	void		test1();
private	void		test2();
private	void		test3();
private	void		test4();
private	void		test5();
private	void		test6();
private	void		test7();
private	void		test8();
private	void		test9();
private	void		test10();
private	void		ch_t1();
private	void		ch_t2();
private	void		ch_t3();
private	void		ch_t4();
private	void		do_t4();
private	void		ch_t5();
private	void		do_t5p1();
private	void		do_t5p2();
private	void		do_t5p3();
private	void		ch_t6();
private	void		do_t6();
private	void		ch_t7and8();
private	void		pa_t7and8();
private int		do_7and8();
private	void		ch_t9();
private	void		ch_t10();
private	void		prepare_tests();
private	void		sig_catch();
private int		sig_init();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	NULL, 0
};

private	void
prepare_tests()
{
	int	ret;		/* Used for most return values */
	int	i;		/* counter */
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		job_control = 0;
		nsigs = VNOJSIG;
	} else {
		job_control = 1;
		nsigs = VNUMSIG;
	}

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		PATH_TRACE; 
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);
	PATH_SETUP_RPT(nsigs);

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

private int
sig_ck(signum, name, act, deletion)
int	signum;		/* check whether this signal is blocked/unblocked */
char	*name;		/* name of that signal */
int	act;		/* if SIG_BLOCK, should be blocked */
			/* if SIG_UNBLOCK, should be unblocked */
int	deletion;	/* report deletion, else failure */
{
	/*
	 * Check that a signal is unblocked or not by trying to catch it.
	 * This will not work for SIGKILL or SIGSTOP.
	 *
	 * This function has the side effect of setting the signal action for
	 * the signal concerned to be caught in sig_catch()
	 *
	 * This function will also leave 
	 * pending signals if they are blocked on entry to this function.
	 *
	 * ASSUMPTIONS: test_fail is set if FAILURE/DELETION has already been
	 * reported, otherwise FAILURE/DELETION may be safely 
	 * reported in this function.
	 */
	int	ret;		/* Used for most return values */
	pid_t	myself;		/* process number of this process */
	int	pathok = 0;

	DBUG_ENTER("sig_ck");

	DBUG_PRINTF("sig_ck", "signal %s", name);

	/* Call sigaction() to initialise signal handling function */
	if(sig_init(signum, &sig_empty, sig_catch, 0) < 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - sigaction() failed");
		DBUG_RETURN(FAIL);
	}
	else
		PATH_TRACE;

	sig_recvd = 0;
	myself = getpid();
	ret = kill(myself, signum);
	if(ret < 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - kill() failed");
		DBUG_RETURN(FAIL);
	}
	else
		PATH_TRACE;

	if(act == SIG_UNBLOCK && !sig_recvd) {
		if(!test_fail)
			xx_rpt(deletion ? DELETION : FAILURE);
		in_rpt("signal %s is blocked, should be unblocked", name);
		DBUG_RETURN(FAIL);
	}
	else
		PATH_TRACE;

	if(act == SIG_BLOCK && (sig_recvd == signum)) {
		if(!test_fail)
			xx_rpt(deletion ? DELETION : FAILURE);
		in_rpt("signal %s is unblocked, should be blocked", name);
		DBUG_RETURN(FAIL);
	}
	else
		PATH_TRACE;

	if(act == SIG_BLOCK && sig_recvd) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("signal %s unexpectedly delivered when signal %s", 
						signame(sig_recvd), name);
		in_rpt("is generated");
		DBUG_RETURN(FAIL);
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);
	
	DBUG_RETURN(SUCCEED);
}

private int
sigset_ck(sigs, flag)
sigset_t	*sigs;	/* mask of signals that should be blocked */
int		flag;	/* whether failure here requires test deletion */
{
	/*
	 * Check whether signals are blocked/unblocked as per "sigs". 
	 * This ignores SIGKILL or SIGSTOP.
	 *
	 * This function has the side effect of setting the signal action for
	 * all other signals concerned to be caught in sig_catch().
	 */
	int	i;	/* counter for signal array */
	int	signum;	/* signal number */
	char	*name;	/* signal name */
	int	pathok = 0;

	DBUG_ENTER("sigset_ck");

	for(i = 1; i < nsigs; i++) {
		PATH_TRACE; 
		signum = sig_array[i];
		name   = sig_name[i];

		if(i == VSIGKILL || i == VSIGSTOP)
			continue;

		if(signum == VNOSIG)
			continue;

		if(sigismember(sigs, signum)) {
			if(sig_ck(signum, name, SIG_BLOCK, flag) == FAIL)
				DBUG_RETURN(FAIL);
		} else {
			if(sig_ck(signum, name, SIG_UNBLOCK, flag) == FAIL)
				DBUG_RETURN(FAIL);
		}
	}
	PATH_FUNC_RPT(nsigs-1); 
	DBUG_RETURN(SUCCEED);
}

private void
test1()
{
	DBUG_ENTER("test1");

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t1, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	sigset_t set;		/* blocked signal set */
	sigset_t oset;		/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	pathok = setupok; 

	DBUG_ENTER("ch_t1");

	/* Call sigprocmask() to block some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check these signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, DELETION) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask() to block some more signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGINT);
	if((ret = sigprocmask(SIG_BLOCK, &set, &oset)) != 0) {
		xx_rpt(FAILURE);
		in_rpt("sigprocmask(SIG_BLOCK, ,) returned %d instead of 0",
								ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the new and original signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGINT);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, FAILURE) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask(SIG_SETMASK, &oset, ) using the previous mask */
	if(sigprocmask(SIG_SETMASK, &oset, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check the original signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, FAILURE) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t2, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t2()
{
	sigset_t set;		/* blocked signal set */
	sigset_t oset;		/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	pathok = setupok; 

	DBUG_ENTER("ch_t2");

	/* Call sigprocmask() to block some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check these signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, DELETION) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask() to unblock some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGINT);
	if((ret = sigprocmask(SIG_UNBLOCK, &set, &oset)) != 0) {
		xx_rpt(FAILURE);
		in_rpt("sigprocmask(SIG_UNBLOCK, ,) returned %d instead of 0",
								ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check only the correct signals have been unblocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, FAILURE) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;
		

	/* Call sigprocmask(SIG_SETMASK, &oset, ) using the previous mask */
	if(sigprocmask(SIG_SETMASK, &oset, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check the original signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, FAILURE) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t3, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t3()
{
	sigset_t set;		/* blocked signal set */
	sigset_t oset;		/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	pathok = setupok; 

	DBUG_ENTER("ch_t3");

	/* Call sigprocmask() to block some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check these signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, DELETION) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask() to set new signal mask */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGINT);
	if((ret = sigprocmask(SIG_SETMASK, &set, &oset)) != 0) {
		xx_rpt(FAILURE);
		in_rpt("sigprocmask(SIG_SETMASK, ,) returned %d instead of 0",
								ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check only the new signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGINT);
	if(sigset_ck(&set, FAILURE) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask(SIG_SETMASK, &oset, ) using the previous mask */
	if(sigprocmask(SIG_SETMASK, &oset, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check the original signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, FAILURE) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t4, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t4()
{
	int	pathok = setupok; 
	DBUG_ENTER("ch_t4");

	globok = 0;
	do_t4(SIG_BLOCK);
	PATH_FUNC_TRACE;
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	do_t4(SIG_UNBLOCK);
	PATH_FUNC_TRACE;
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	do_t4(SIG_SETMASK);
	PATH_FUNC_TRACE;
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
do_t4(how)
int	how;	/* 1st argument of sigprocmask() */
{
	int	err;
	int	ret;
	sigset_t set;		/* blocked signal set */
	sigset_t oset;		/* blocked signal set */
	int	pathok = 0;

	DBUG_ENTER("do_t4");

	/* Call sigprocmask() to block some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check these signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, DELETION) == FAIL) {
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask() to set oset and leave signals unaltered */
	(void) sigemptyset(&oset);
	errno = NOERROR;
	if((ret = sigprocmask(how, (sigset_t *)NULL, &oset)) != 0) {
		err = errno;
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigprocmask() did not return expected value 0");
		in_rpt("instead returned %d with errno set to %d", ret, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check signals are still blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, FAILURE) == FAIL) {
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask() to block some more signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGINT);
	if(sigprocmask(SIG_BLOCK, &set, (sigset_t *)NULL) != 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the new and original signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGINT);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, DELETION) == FAIL) {
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask(SIG_SETMASK, &oset, ) using the previous mask */
	if(sigprocmask(SIG_SETMASK, &oset, (sigset_t *)NULL) != 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check the original signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, FAILURE) == FAIL) {
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	PATH_FUNC_RPT(8); 
	DBUG_VOID_RETURN;
}

private void
test5()
{
	DBUG_ENTER("test5");

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t5, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t5()
{
	int	pathok = setupok;
	DBUG_ENTER("ch_t5");

	globok = 0;
	do_t5p1();
	PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	do_t5p2();
	PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	do_t5p3();
	PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
do_t5p1()
{
	sigset_t set;		/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	pathok = 0;

	DBUG_ENTER("do_t5p1");

	/* Call sigprocmask() to block some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check these signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, DELETION) == FAIL) {
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask() to block some more signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGINT);
	if((ret = sigprocmask(SIG_BLOCK, &set, (sigset_t *)NULL)) != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigprocmask(SIG_BLOCK, ,) returned %d instead of 0",
								ret);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the new and original signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGINT);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, FAILURE) == FAIL) {
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 		
	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
do_t5p2()
{
	sigset_t set;		/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	pathok = 0;

	DBUG_ENTER("do_t5p2");

	/* Call sigprocmask() to block some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check these signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, DELETION) == FAIL) {
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask() to unblock some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGINT);
	if((ret = sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL)) != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigprocmask(SIG_UNBLOCK, ,) returned %d instead of 0",
								ret);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check only the correct signals have been unblocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, FAILURE) == FAIL) {
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 
	PATH_FUNC_RPT(4); 
	DBUG_VOID_RETURN;
}

private void
do_t5p3()
{
	sigset_t set;		/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	pathok = 0;

	DBUG_ENTER("do_t5p3");

	/* Call sigprocmask() to block some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check these signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, DELETION) == FAIL) {
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask() to set new signal mask */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGINT);
	if((ret = sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL)) != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigprocmask(SIG_SETMASK, ,) returned %d instead of 0",
								ret);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check only the new signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGINT);
	if(sigset_ck(&set, FAILURE) == FAIL) {
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 
	PATH_FUNC_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
test6()
{
	DBUG_ENTER("test6");

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t6, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	int	pathok = setupok; 
	DBUG_ENTER("ch_t6");

	globok = 0;
	do_t6(SIG_BLOCK);
	PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	do_t6(SIG_UNBLOCK);
	PATH_FUNC_TRACE;
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	do_t6(SIG_SETMASK);
	PATH_FUNC_TRACE;
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(6); 

	DBUG_VOID_RETURN;
}

private void
do_t6(how)
int	how;	/* 1st argument of sigprocmask() */
{
	int	err;
	int	ret;
	sigset_t set;		/* blocked signal set */
	int	pathok = 0;

	DBUG_ENTER("do_t6");

	/* Call sigprocmask() to block some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check these signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, DELETION) == FAIL) {
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask() to leave signal mask unaltered */
	errno = NOERROR;
	if((ret = sigprocmask(how, (sigset_t *)NULL, (sigset_t *)NULL)) != 0) {
		err = errno;
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigprocmask() did not return expected value 0");
		in_rpt("instead returned %d with errno set to %d", ret, err);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check signals are still blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, FAILURE) == FAIL) {
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 
	PATH_FUNC_RPT(4); 

	DBUG_VOID_RETURN;
}

private void
test7()
{

	int	pathok = setupok;
	DBUG_ENTER("test7");

	test_fail = 0;
	test_deleted = 0;

	globok = 0; 
	if(do_7and8(VSIGKILL) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test8()
{
	int	pathok = setupok; 
	DBUG_ENTER("test8");

	test_fail = 0;
	test_deleted = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0; 
	if(do_7and8(VSIGSTOP) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private int
do_7and8(i)
int	i;		/* index in sig_array of signal to test */
{
	/*
	 * Test that the given signal is not blocked from delivery 
	 * when set in the argument to sigprocmask() and that the 
	 * sigprocmask() function succeeds.
	 */
	sigset_t set;		/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	sg_num;
	int	pathok = 0;

	DBUG_ENTER("do_7and8");

	test_fail = 0;
	test_deleted = 0;

	sig_index = i;
	sig_arg = SIG_DFL;
	sg_num = sig_array[sig_index];

	if(sg_num == VNOSIG) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - signal (%s) not available", 
							sig_name[i]);
		DBUG_RETURN(FAIL);
	}
	else
		PATH_TRACE;

	/* Check the call to sigprocmask() succeeds */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	if((ret = sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL)) != 0) {
		xx_rpt(FAILURE);
		in_rpt("sigprocmask() returned %d instead of 0", ret);
		DBUG_RETURN(FAIL);
	}
	else
		PATH_TRACE;
	
	if(cpsig(ch_t7and8, pa_t7and8, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE; 

	if(test_fail || test_deleted)
	{
		DBUG_RETURN(FAIL);
	}
	else
		PATH_TRACE;
	PATH_FUNC_RPT(4); 

	DBUG_RETURN(SUCCEED);
}

private void
ch_t7and8()
{
	pid_t	myself;		/* process number of this process */
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t7and8");

	/* Have to do this again  -  see actions before fork */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	sig_recvd = 0;

	myself = getpid();

	(void) kill(myself, sg_num);

	/* Exit code is signal number we received if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t7and8()
{
	int	err;
	int	stat_child;
	char *	str_expect;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	sg_type = sig_type[sg_num];
	char	exp_buf[BUFSIZ];
	int	pathok = 0;

	DBUG_ENTER("pa_t7and8");

	sig_err = 0;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR ||
	    stat_child == CHLD_LIVE) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* 
	 * Switch on what the argument demanded.
	 */
	if(sig_arg == SIG_DFL)
	{
		/* Switch on what default action is expected. */
		switch(sg_type)
		{
		case TYPE_ABRT:	/* child aborts on receiving signal */
			(void) sprintf(exp_buf, "signal %d terminated process", sg_num);
			str_expect = exp_buf;
			if(stat_child != CHLD_ABRT)
				sig_err++;
			if(sg_code != sg_num)
				sig_err++;
			break;
		case TYPE_IGNR:	/* child ignores so exits normally */
		case TYPE_CONT:	/* here, child ignores SIGCONT */
			str_expect = "process ignored signal";
			if(stat_child != CHLD_EXIT)
				sig_err++;
			if(ex_code != 0)
				sig_err++;
			break;
		case TYPE_STOP:	/* child process stops */
			str_expect = "signal stopped process";
			if(stat_child != CHLD_STOP)
				sig_err++;
			break;
		}
	}
	else if(sig_arg == SIG_IGN)
	{
		str_expect = "process ignored signal";
		if(stat_child != CHLD_EXIT)
			sig_err++;
		if(ex_code != 0)
			sig_err++;
	}
	else
	{
		(void) sprintf(exp_buf, "process caught signal %d", sg_num);
		str_expect = exp_buf;
		if(stat_child != CHLD_EXIT)
			sig_err++;
		if(ex_code != sg_num)
			sig_err++;
	}

	if(sig_err)
	{
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caused wrong action", sg_name);
		in_rpt("expected action: %s", str_expect);
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
test9()
{
	DBUG_ENTER("test9");

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t9, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t9()
{
	sigset_t set;		/* blocked signal set */
	int	ret;		/* Used for most return values */
	int	pathok = setupok; 

	DBUG_ENTER("ch_t9");

	/* Call sigprocmask() to block some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check these signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, DELETION) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig_recvd = 0;

	/* Call sigprocmask() to unblock signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if((ret = sigprocmask(SIG_UNBLOCK, &set, (sigset_t *)NULL)) != 0) {
		xx_rpt(FAILURE);
		in_rpt("sigprocmask() returned %d instead of 0", ret);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/*
	 * Check at least one of them is received.
	 * This relies on the fact that signals have been generated in 
	 * sigset_ck which when delivered are caught
	 */
	if(sig_recvd != SIGPIPE && sig_recvd != SIGHUP) {
		xx_rpt(FAILURE);
		in_rpt("sigprocmask() returned before any unblocked signal");
		in_rpt("was delivered");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	DBUG_ENTER("test10");

	test_fail = 0;
	test_deleted = 0;

	(void) cppair(ch_t10, NULLFN, 3*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t10()
{
	sigset_t set;		/* blocked signal set */
	sigset_t oset;		/* blocked signal set */
	int	err;		/* Save value of errno */
	int	ret;		/* Used for most return values */
	int	i;		/* Counter for invalid argument */
	int	pathok = 0;
	int 	loop = 0;

	DBUG_ENTER("ch_t10");

	/* Call sigprocmask() to block some signals */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL) != 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigprocmask() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	/* Check these signals are indeed blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, DELETION) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

	/* Call sigprocmask() which will fail */
	for(i = 0; i == SIG_SETMASK || i == SIG_BLOCK || i == SIG_UNBLOCK; i++)
	{
		PATH_TRACE;
		loop++;
	}
	errno = NOERROR;
	ret = sigprocmask(i, &set, &oset);
	if(ret != SYSERROR || errno != EINVAL) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sigprocmask() gave wrong value with invalid argument");
		valrpt(SYSERROR, ret, EINVAL, err);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the original signals are still blocked */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, SIGPIPE);
	(void) sigaddset(&set, SIGHUP);
	if(sigset_ck(&set, FAILURE) == FAIL)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE; 

		PATH_XS_RPT(4+loop); 

	DBUG_VOID_RETURN;
}
