/*
*      SCCS:  @(#)POSIX.os/procprim/sigsuspend/sigsuspend.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/sigsuspend - T.sigsuspend Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.sigsuspend

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the sigsuspend() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Wed Mar  8 09:23:45 GMT 1989
MODIFICATIONS:
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG

		Anthony Lee, 14 August 1990
		Added resilience code.

		Stuart Boutell, 26 Nov 1990
			Moved signal initialisiation to test start
			as necessary.

		Stuart Boutell, 30 Apr 1991
			Changed cpsig() return code check from FAIL to
			SYSERROR
		
		Geoff Clare, 19 Sep 1991
			Deal with signals arriving after sigsuspend()
			returns in ch_t1().

		Geoff Clare, 22 Nov 1994
			Added declaration of sig_init().

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sigsuspend
#endif


#define	NO_TESTS		6		/* number of tests in testset */
#define	WAITTIME		(3*SPEEDFACTOR)
#define	EXTERR			"external error"

extern	void		vsx_sigs();
extern	void		valrpt();
extern	char *		sig_child();
extern	char *		status_child();
extern	int		cppair();

extern	int		sig_array[];
extern	char *		sig_name[];
extern	int		sig_type[];
extern	int		ex_code;
extern	int		sg_code;

int	no_tests = NO_TESTS;

private	void		(*sig_arg)();
private	int		test_fail;
private	int		test_deleted;
private	int		sig_index;
private	sigset_t	sig_empty;
private int		sig_recvd;
private int		chk_recvd;
private int		chk_save;
private int		nsigs;
private int		job_control;

private	void		test1();
private	void		test2();
private	void		test3();
private	void		test4();
private	void		test5();
private	void		test6();
private	void		do_1();
private	void		ch_t1();
private	void		pa_t1();
private	void		ch_t2();
private	void		pa_t2();
private	void		ch_t3();
private	void		pa_t3();
private	void		ch_t4and5();
private	void		pa_t4and5();
private	void		ch_t6();
private	void		pa_t6();
private	void		prepare_test();
private	void		sig_catch();
private void		cln_tests();
private int		sig_init();

public	void	(*setfunc) () = NULL;
public	void	(*clnfunc) () = cln_tests;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	NULL, 0
};

private	void
prepare_test()
{
	int	ret;		/* Used for most return values */
	int	i;		/* counter */
	int	pathok = 0;

	DBUG_ENTER("prepare_test");

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		job_control = 0;
		nsigs = VNOJSIG;
	} else {
		job_control = 1;
		nsigs = VNUMSIG;
	}

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++)
	{
		PATH_TRACE;
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);
	PATH_FUNC_RPT(nsigs);
	
	DBUG_VOID_RETURN;
}

private void
cln_tests()
{
	DBUG_ENTER("cln_tests");

	(void) unlink("core");

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */
	int	ret;

	DBUG_ENTER("sig_init");

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	ret = sigaction(signum, &act, (struct sigaction *)NULL);
	if(sigfunc == SIG_DFL)
		DBUG_PRINTF("sig_init", "DEFAULT sigaction returns %d for signal %d", ret, signum);
	else if (sigfunc == SIG_IGN)
		DBUG_PRINTF("sig_init", "IGNORE sigaction returns %d for signal %d", ret, signum);
	else
		DBUG_PRINTF("sig_init", "CATCH sigaction returns %d for signal %d", ret, signum);

	DBUG_RETURN(ret);
}

private void
sig_catch(sg_num)
int	sg_num;
{
	sig_recvd = sg_num;
	chk_save = chk_recvd;
}

private void
sig_check(sg_num)
int	sg_num;
{
	chk_recvd = sg_num;
}

private void
test1()
{
	int	pathok = 0;
	DBUG_ENTER("test1");

	globok = 0;
	prepare_test();
	PATH_FUNC_TRACE;

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	do_1(SIG_DFL);
	PATH_FUNC_TRACE;
	
	if(test_deleted)
		DBUG_VOID_RETURN;

	globok = 0;
	do_1(sig_catch);
	PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(3); 

	DBUG_VOID_RETURN;
}

private void
do_1(arg)
void	(*arg)();	/* One of SIG_DFL or signal catching function */
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("do_1");

	/*
	 * Test sigsuspend().
	 * If arg is SIG_DFL, do this for all signals which terminate.
	 * If arg is signal catcher, do this for all catchable signals. 
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		if(sig_array[i] == VNOSIG)
		{
			PATH_TRACE; 
			continue;
		}

		if(arg != SIG_DFL)
			/* Can't catch these signals */
			if(i == VSIGKILL || i == VSIGSTOP)
			{
				PATH_TRACE; 
				continue;
			}


		if(arg == SIG_DFL)
			/* Do this for all signals which terminate process */
			if(sig_type[sig_array[i]] != TYPE_ABRT)
			{
				PATH_TRACE; 
				continue;
			}

		sig_index = i;
		sig_arg = arg;

		if(cpsig(ch_t1, pa_t1, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		else
			PATH_FUNC_TRACE;

		if(test_deleted)
			DBUG_VOID_RETURN;
	}
	PATH_FUNC_RPT(nsigs-1); 


	DBUG_VOID_RETURN;
}

private void
ch_t1()
{
	sigset_t set;		/* blocked signal set */
	int	sg_num  = sig_array[sig_index];
	int	j;
	int	fail = 0;
	int	pathok = 0;

	DBUG_ENTER("ch_t1");

	/* Prepare to block all signals but one in sigsuspend() */
	(void) sigemptyset(&set);
	for(j = 1; j < nsigs; j++) {
		if (j == 1)
			PATH_TRACE;

		if(j == sig_index) 
			continue;
		if(j == VSIGKILL || j == VSIGSTOP)
			continue;
		/* Might continue process even if blocked ... */
		if(j == VSIGCONT)
			continue;
		if(sig_array[j] == VNOSIG)
			continue;
		(void) sigaddset(&set, sig_array[j]);

		/* catch them as well, so they don't terminate the
		   process after sigsuspend() returns */
		if (sig_arg != SIG_DFL)
			(void) sig_init(sig_array[j], &sig_empty, sig_check, 0);
	}

	/* Initialise required signal action */
	(void) sig_init(sg_num, &sig_empty, sig_arg, 0);

	sig_recvd = 0;
	chk_recvd = 0;
	chk_save = 0;

	/* Suspend the process blocking all but one signal */
	(void) sigsuspend(&set);

	/* If sigsuspend() returns, all the unblocked signals now arrive,
	   so block them to stop them interfering with remaining processing,
	   then check if any unexpected signals were received, and exit
	   indicating if the expected signal was received. */

	(void) sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL);

	if (chk_save != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("Signal %d not blocked by sigsuspend(mask)", chk_save);
		in_rpt("when mask contained all signals except %d", sg_num);
		fail = 1;
	}
	else
		PATH_TRACE;

	if (sig_recvd != sg_num)
	{
		if (fail == 0)
			xx_rpt(FAILURE);
		in_rpt("Signal %d not received during sigsuspend(mask)",
			sg_num);
		in_rpt("when mask contained all signals except %d", sg_num);
		fail = 1;
	}
	else
		PATH_TRACE;

	if (pathok != 3)
		DBUG_EXIT(2);

	DBUG_EXIT(fail);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t1()
{
	int	err;
	int	j;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	pathok = 0;

	DBUG_LENTER("pa_t1",2);		/* Too much output */

	/* Wait - should time out if child really is suspended */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child != CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigsuspend() did not suspend process");
		/* set deleted flag - no point in carrying on */
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try every blockable signal but the un-blocked one */
	for(j = 1; j < nsigs; j++) {
		if(j == sig_index)
		{
			PATH_TRACE;
			continue;
		}
		
		if(j == VSIGKILL || j == VSIGSTOP)
		{
			PATH_TRACE;
			continue;
		}

		/* Might continue process even if blocked ... */
		if(j == VSIGCONT)
		{
			PATH_TRACE;
			continue;
		}

		if(sig_array[j] == VNOSIG)
		{
			PATH_TRACE;
			continue;
		}

		if(kill(tet_child, sig_array[j]) < 0) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child != CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigsuspend() did not block all signals in argument sigmask");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the un-blocked signal */
	if(kill(tet_child, sg_num) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Check the process is no longer suspended */
	if(sig_arg == SIG_DFL)
	{
		if (stat_child != CHLD_ABRT)
		{
			if(!test_fail)
				xx_rpt(FAILURE);
			in_rpt("sigsuspend() not interrupted by un-blocked signal %d (%s)", 
				sg_num, sg_name);
			test_fail++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else if(stat_child != CHLD_EXIT)
	{
		xx_rpt(DELETION);
		in_rpt("child did not exit as expected");
		in_rpt("status: %s", sig_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else if (ex_code != 0 && ex_code != 2)
		test_fail++; /* reported by child */
	else if (ex_code == 0)
		PATH_TRACE;

	PATH_FUNC_RPT((nsigs-1)+7); 

	DBUG_VOID_RETURN;
}

private void
test2()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test2");

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Test that sigsuspend() does not return when signal terminates 
	 * process.
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		if(sig_array[i] == VNOSIG)
		{
			PATH_TRACE; 
			continue;
		}

		/* Do this for all signals which terminate process */
		if(sig_type[sig_array[i]] != TYPE_ABRT)
		{
			PATH_TRACE; 
			continue;
		}

		sig_index = i;

		globok = 0; 
		if(cpsig(ch_t2, pa_t2, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;

		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT((nsigs-1)); 

	DBUG_VOID_RETURN;
}


private void
ch_t2()
{
	sigset_t set;		/* blocked signal set */
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t2");

	/* Prepare to unblock all signals in sigsuspend() */
	(void) sigemptyset(&set);

	/* Initialise required signal action */
	(void) sig_init(sg_num, &sig_empty, SIG_DFL, 0);

	/* Suspend the process */
	(void) sigsuspend(&set);

	/* Failure. Parent checks exit code */
	DBUG_EXIT(A_MISC);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t2()
{
	int	err;
	int	stat_child;
	int	sg_num   = sig_array[sig_index];
	char *	sg_name  = sig_name[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t2");

	/* Wait - should time out if child really is suspended */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigsuspend() did not suspend process");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the signal */
	if(kill(tet_child, sg_num) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_MISC) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sigsuspend() returned when signal %d (%s) generated",
					sg_num, sg_name);
		in_rpt("expected process to terminate on delivery of signal");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	PATH_FUNC_RPT(5); 
	DBUG_VOID_RETURN;
}

private void
test3()
{
	int	i;
	int	pathok = 0;

	DBUG_ENTER("test3");

	test_fail = 0;
	test_deleted = 0;

	/*
	 * Test that signal mask is restored when sigsuspend() returns.
	 * Do this in the child process.The parent works out what happens
	 * based on the child exit status, etc.
	 */
	for (i = 1 ; i < nsigs ; i++)
	{
		if(sig_array[i] == VNOSIG)
		{
			PATH_TRACE; 
			continue;
		}

		/* Can't catch or ignore these signals */
		if(i == VSIGKILL || i == VSIGSTOP)
		{
			PATH_TRACE; 
			continue;
		}

		sig_index = i;

		globok = 0; 
		if(cpsig(ch_t3, pa_t3, WAITTIME, 0) == SYSERROR)
		{
			test_deleted++;
		}
		else
			PATH_FUNC_TRACE;

		if(test_deleted)
			DBUG_VOID_RETURN;
	}

	if(!test_fail)
		PATH_XS_RPT(nsigs-1); 

	DBUG_VOID_RETURN;
}
		
private void
ch_t3()
{
	sigset_t set;		/* blocked signal set */
	int	sg_num  = sig_array[sig_index];
	int	j;

	DBUG_ENTER("ch_t3");

	/* Prepare to block all blockable signals using sigprocmask() */
	(void) sigemptyset(&set);
	for(j = 1; j < nsigs; j++) {
		if(j == VSIGKILL || j == VSIGSTOP)
			continue;
		/* Might continue process even if blocked ... */
		if(j == VSIGCONT)
			continue;
		if(sig_array[j] == VNOSIG)
			continue;
		(void) sigaddset(&set, sig_array[j]);
	}
	/* Set signal mask to block all signals */
	(void) sigprocmask(SIG_SETMASK, &set, (sigset_t *)NULL);

	/* Initialise required signal action */
	(void) sig_init(sg_num, &sig_empty, sig_catch, 0);

	/* Suspend the process - no signals blocked */
	(void) sigemptyset(&set);
	(void) sigsuspend(&set);

	sig_recvd = 0;

	/* Pause - all blockable signals should be blocked */
	(void) pause();

	/* Should not reach here */
	/* If sigsuspend() returns, exit with number of signal received */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t3()
{
	int	err;
	int	j;
	int	stat_child;
	int	sg_num  = sig_array[sig_index];
	int	pathok = 0;

	DBUG_ENTER("pa_t3");

	/* Wait - should time out if child really is suspended */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - sigsuspend() did not suspend process");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send un-blocked signal to interrupt sigsuspend()*/
	if(kill(tet_child, sg_num) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: sigsuspend() did not return");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Sleep to give the child time to pause */
	(void) sleep(WAITTIME);

	/* Try every blockable signal */
	for(j = 1; j < nsigs; j++) {
		if(j == VSIGKILL || j == VSIGSTOP)
		{
			PATH_TRACE; 
			continue;
		}
		/* Might continue process even if blocked ... */
		if(j == VSIGCONT)
		{
			PATH_TRACE; 
			continue;
		}
		if(sig_array[j] == VNOSIG)
		{
			PATH_TRACE; 
			continue;
		}
		if(kill(tet_child, sig_array[j]) < 0) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
			test_deleted++;
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal mask not restored when sigsuspend() returns");
		in_rpt("all blockable signals should be blocked");
		in_rpt("expected action: process still active"); 
		in_rpt("observed action: %s", sig_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

		PATH_FUNC_RPT(nsigs+6);
		DBUG_VOID_RETURN;
}

private void
test4()
{
	/*
	 * Test that the given signal is not blocked from delivery 
	 * when set in the argument to sigsuspend() and that the 
	 * sigsuspend() function succeeds.
	 */
	int	pathok = 0;

	DBUG_ENTER("test4");

	test_fail = 0;
	test_deleted = 0;

	sig_index = VSIGKILL;

	globok = 0; 
	if(cpsig(ch_t4and5, pa_t4and5, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(1); 

	DBUG_VOID_RETURN;
}

private void
test5()
{
	/*
	 * Test that the given signal is not blocked from delivery 
	 * when set in the argument to sigsuspend() and that the 
	 * sigsuspend() function succeeds.
	 */

	int	pathok = 0;
	DBUG_ENTER("test5");

	test_fail = 0;
	test_deleted = 0;

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	sig_index = VSIGSTOP;

	globok = 0; 
	if(cpsig(ch_t4and5, pa_t4and5, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
	}
	else
		PATH_FUNC_TRACE;
		
	if(test_deleted)
		DBUG_VOID_RETURN;

	if(!test_fail)
		PATH_XS_RPT(2); 

	DBUG_VOID_RETURN;
}

private void
ch_t4and5()
{
	sigset_t set;		/* blocked signal set */
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("ch_t4and5");

	/* If call to sigsuspend() fails, parent will realise */
	(void) sigemptyset(&set);
	(void) sigaddset(&set, sg_num);
	(void) sigsuspend(&set);
	
	/* should not get here */
	DBUG_EXIT(A_MISC);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_t4and5()
{
	int	err;
	int	fail = 0;
	int	stat_child;
	char *	str_expect;
	int	sg_num  = sig_array[sig_index];
	char *	sg_name = sig_name[sig_index];
	int	sg_type = sig_type[sg_num];
	int	pathok = 0;

	DBUG_ENTER("pa_t4and5");

	/* Wait - should time out if child really is suspended */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Ensure the child is still there */
	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("sigsuspend() did not suspend process");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Send the signal to terminate process */
	if(kill(tet_child, sg_num) < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - kill() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Switch on what default action is expected. */
	switch(sg_type) {
	case TYPE_ABRT:	/* child aborts on receiving signal */
		str_expect = "signal terminated process";
		if(stat_child != CHLD_ABRT)
			fail++;
		else
			PATH_TRACE;
		break;
	case TYPE_IGNR:	/* child ignores so exits normally */
	case TYPE_CONT:	/* here, child ignores SIGCONT */
		str_expect = "process ignored signal";
		if(stat_child != CHLD_EXIT)
			fail++;
		else
			PATH_TRACE; 
		break;
	case TYPE_STOP:	/* child process stops */
		str_expect = "signal stopped process";
		if(stat_child != CHLD_STOP)
			fail++;
		else
			PATH_TRACE; 
		break;

	default:
		PATH_TRACE;
		break;
	}

	if(fail) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("signal %s caused wrong action", sg_name);
		in_rpt("expected action: %s", str_expect);
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(6); 
	
	DBUG_VOID_RETURN;
}
	
private void
test6()
{
	DBUG_ENTER("test6");

	/*
	 * Test that sigsuspend(), when a signal is caught and control returned,
	 * returns a value of -1 and errno is set to EINTR.
	 * Do this in the child process. The child process also does all the 
	 * reporting for this test.
	 */

	sig_index = VSIGHUP;

	(void) cppair(ch_t6, pa_t6, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch_t6()
{
	sigset_t set;		/* blocked signal set */
	int	sg_num  = sig_array[sig_index];
	int	err;
	int	ret;
	int	pathok = 0;

	DBUG_ENTER("ch_t6");

	/* Initialise required signal action */
	(void) sig_init(sg_num, &sig_empty, sig_catch, 0);

	/* Suspend the process - no signals blocked */
	(void) sigemptyset(&set);
	errno = NOERROR;
	ret = sigsuspend(&set);
	if(ret != SYSERROR || errno != EINTR) {
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("sigsuspend() returned wrong value when interrupted");
		valrpt(SYSERROR, ret, EINTR, err);
	}
	else
	{
		PATH_TRACE; 
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
pa_t6()
{
	int	sg_num  = sig_array[sig_index];

	DBUG_ENTER("pa_t6");

	/* Sleep to give the child time to suspend */
	(void) sleep(WAITTIME/2);

	/* Send un-blocked signal to interrupt sigsuspend() */
	(void) kill(tet_child, sg_num);

	globok = 1; /* must set globok for cppair */

	DBUG_VOID_RETURN;
}
