/*
*      SCCS:  @(#)  POSIX.os/procprim/sleep/sleep.c	Rel 4.4.1	    (06/09/97)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/sleep - T.sleep Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN */
#endif	/* lint */

/***********************************************************************

NAME:		T.sleep

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the sleep() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Alex Flynn, UniSoft Ltd.
DATE CREATED:	Mon Nov  3 15:50:24 GMT 1986
MODIFICATIONS:	Andrew Twigger 16 June 87
		Removed	abandon(), testid()
		Modified xx_rpt(), xs_rpt()

		Dave Cater 15 March 1989
			upgraded for XPG 3 to comply with POSIX assertions 

		Dave Cater 22 December 1989
			Ensure part 2 detects failure if sleep does not return
			immediately after signal handler has been entered.
		
		Geoff Clare, 22 Mar 1990
			Use SIG_ERR provided in vlib if not defined

		Peter Lee, Unisoft Ltd.,
		Fri Aug 10 15:40:52 BST 1990
		Added resilience code.

		Stuart Boutell, 30 Apr 1991
			Changed cpsig() return code check from FAIL to
		SYSERROR

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<errno.h>
#include	<dbug.h>
#include	<wait_child.h>
#include	<exitcodes.h>
#include	<signal.h>
#include	<time.h>
#include	<report.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef sleep
#endif


#define 	NO_TESTS	1		/* number of tests in testset */
#define		EXTERR		"external error"
#define		SLEEPTIME	(5 * SPEEDFACTOR)
#define		WAITTIME	(3 * SLEEPTIME) + (5 * SPEEDFACTOR)
#define		VSX_AL_ACCURACY	"VSX_AL_ACCURACY"
#define		DEFAULT		5	/* default value for above */
#define		PATH_NOK	3

#ifndef SIG_ERR
extern	void	SIG_ERR();
#endif

extern	char	*errname();
extern	void	(*vsignal())();

extern	int	ex_code;
extern	int	sg_code;

int	no_tests = NO_TESTS;

private	int	globvar = 0;
private	int	test_fail;
private	int	test_deleted;
private	int	accuracy;
private	void	prepare_tests();
private void	test1();
private	void	ch_p1();
private	void	pa_p1();
private	void	ch_p2();
private	void	pa_p2();
private	void	ch_p3();
private	void	pa_p3();
private	void 	onint1();
private	void	onint2();
private long	longabs();

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	NULL, 0
};

private void
prepare_tests()
{
	char	*cp;

	DBUG_ENTER("prepare_tests");

	if ( (cp = tet_getvar(VSX_AL_ACCURACY)) == (char *) NULL )
		accuracy = DEFAULT;
	else
		accuracy = longabs((long)atoi(cp));

	DBUG_VOID_RETURN;
}

private void
test1()
{
	int	pathok = 0;

	DBUG_ENTER("test1");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(ch_p1, pa_p1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if(cpsig(ch_p2, pa_p2, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;

	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	globok = 0;
	if(cpsig(ch_p3, pa_p3, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	else
		PATH_FUNC_TRACE;
	
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(6);

	DBUG_VOID_RETURN;
}

private void
ch_p1()
{
	int	err;
	time_t	svdtime;
	time_t	curtime;
	unsigned	unslept;
	int	pathok = 0;

	DBUG_ENTER("ch_p1");

	/*
	 * First part of the test.
	 * Check that sleep() returns zero when specified number of seconds 
	 * have elapsed, give or take the factor for accuracy.
	 */

	errno = NOERROR;
	if (time(&svdtime) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("time failed with errno %d (%s)", err,
					errname(err));
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	unslept = sleep(SLEEPTIME);

	if (time(&curtime) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("time failed with errno %d (%s)", err,
					errname(err));
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	if (unslept != 0 ||
		longabs((long)(svdtime + SLEEPTIME - curtime)) > accuracy)
	{
		if(!test_fail)
			xx_rpt(FAILURE);
		if (unslept != 0)
			in_rpt("sleep returned %d instead of 0", 
							unslept);
		if (longabs((long)(svdtime + SLEEPTIME - curtime)) > accuracy)
		{
			in_rpt("slept for %d seconds", SLEEPTIME);
			in_rpt("time before sleep: %d", svdtime);
			in_rpt("time after sleep: %d", curtime);
		}
		test_fail++;
	}
	else
		PATH_TRACE;

	if(test_fail)
	{
		DBUG_EXIT(A_MISC);
	}
	else
		PATH_TRACE;

	if (pathok != 4)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_p1()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa_p1");

	/* Wait for the child and work out if success or failure */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_MISC)
			test_fail++;
		else if(ex_code == A_TSETDIR)
			test_deleted++;
		else if (ex_code != PATH_NOK)
			PATH_TRACE;
	} else if(stat_child == CHLD_ABRT) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected signal %d received",
								 sg_code);
		test_deleted++;
	} else {
		xx_rpt(DELETION);
		in_rpt("deletion reason: part 1 timed out after %d seconds", 
								WAITTIME);
		test_deleted++;
	}

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private	void
ch_p2()
{
	int	err;
	time_t	svdtime;
	time_t	curtime;
	unsigned	unslept;
	int	pathok = 0;

	DBUG_ENTER("ch_p2");

	/*
	 * Second part of the test.
	 * Check that signal delivered before sleep has completed enters
	 * signal catching function and sleep returns the unslept time.
	 */

	errno = NOERROR;
	if (vsignal(SIGUSR1, onint2) == SIG_ERR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: vsignal failed with errno %d(%s)"
					, err, errname(err));
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if (time(&svdtime) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("time failed with errno %d (%s)", err,
					errname(err));
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	unslept = sleep(3*SLEEPTIME);

	if (time(&curtime) == SYSERROR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("time failed with errno %d (%s)", err,
					errname(err));
		DBUG_EXIT(A_TSETDIR);
	}
	else
		PATH_TRACE;

	if ( longabs((long) (svdtime + 3*SLEEPTIME - curtime - unslept)) > accuracy )
	{
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sleep for %d seconds - sent SIGUSR1 after %d",
						 3*SLEEPTIME, SLEEPTIME);
		in_rpt("time before sleep: %d", svdtime);
		in_rpt("time after sleep: %d", curtime);
		in_rpt("unslept time returned by sleep: %d", unslept);
		test_fail++;
	}
	else if(longabs((long)(svdtime + SLEEPTIME - curtime)) > accuracy)
	{
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sleep for %d seconds - sent SIGUSR1 after %d",
						 3*SLEEPTIME, SLEEPTIME);
		in_rpt("time before sleep: %d", svdtime);
		in_rpt("time after sleep: %d", curtime);
		in_rpt("sleep returned after %d seconds instead of %d seconds",
						 curtime - svdtime, SLEEPTIME);
		test_fail++;
	}
	else
		PATH_TRACE;

	if(globvar == 0)
	{
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sleep for %d seconds - sent SIGUSR1 after %d",
						 3*SLEEPTIME, SLEEPTIME);
		in_rpt("signal catching function not entered");
		test_fail++;
	}
	else
		PATH_TRACE;

	if(test_fail)
	{
		DBUG_EXIT(A_MISC);
	}
	else
		PATH_TRACE;

	if (pathok != 6)
		DBUG_EXIT(PATH_NOK);

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_p2()
{
	int	err;
	int	stat_child;
	int	pathok = 0;
	
	DBUG_ENTER("pa_p2");

	errno = NOERROR;
	if (vsignal(SIGALRM, onint1) == SIG_ERR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: vsignal failed with errno %d(%s)"
					, err, errname(err));
		(void) kill(tet_child, SIGKILL);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	(void) alarm(SLEEPTIME);

	errno = NOERROR;
	if (pause() != SYSERROR || errno != EINTR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: pause failed with errno %d(%s)"
					, err, errname(err));
		(void) kill(tet_child, SIGKILL);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) kill(tet_child, SIGUSR1);

	/* Wait for the child and work out if success or failure */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT) {
		if(ex_code == A_MISC)
			test_fail++;
		else if(ex_code == A_TSETDIR)
			test_deleted++;
		else if (ex_code != PATH_NOK)
			PATH_TRACE;
	} else if(stat_child == CHLD_ABRT) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: unexpected signal %d received",
								 sg_code);
		test_deleted++;
	} else {
		xx_rpt(DELETION);
		in_rpt("deletion reason: part 2 timed out after %d seconds", 
								WAITTIME);
		test_deleted++;
	}

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private	void
ch_p3()
{
	int	err;

	DBUG_ENTER("ch_p3");

	/*
	 * Third part of the test.
	 * Check that signal delivered before sleep has completed 
	 * terminates process and sleep does not return.
	 */
	errno = NOERROR;
	if (vsignal(SIGUSR1, SIG_DFL) == SIG_ERR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: vsignal failed with errno %d(%s)"
					, err, errname(err));
		DBUG_EXIT(A_TSETDIR);
	}

	(void) sleep(2*SLEEPTIME);

	/* Should not get here if test successful */
	DBUG_EXIT(A_MISC);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa_p3()
{
	int	err;
	int	stat_child;
	int	pathok = 0;
	
	DBUG_ENTER("pa_p3");

	errno = NOERROR;
	if (vsignal(SIGALRM, onint1) == SIG_ERR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: vsignal failed with errno %d(%s)"
					, err, errname(err));
		(void) kill(tet_child, SIGKILL);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;
	
	(void) alarm(SLEEPTIME);

	errno = NOERROR;
	if (pause() != SYSERROR || errno != EINTR)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: pause failed with errno %d(%s)"
					, err, errname(err));
		(void) kill(tet_child, SIGKILL);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	(void) kill(tet_child, SIGUSR1);

	/* Wait for the child and expect it to be terminated with signal */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
							EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_MISC) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("sleep() returned when signal SIGUSR1 generated");
		in_rpt("expected process to terminate on delivery of signal");
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child == CHLD_EXIT && ex_code == A_TSETDIR) 
		test_deleted++;
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

private void
onint1()
{
	DBUG_ENTER("onint1");
	DBUG_VOID_RETURN;
}


private void
onint2()
{
	DBUG_ENTER("onint2");
	globvar++;
	DBUG_VOID_RETURN;
}

private long
longabs(x)
long	x;
{
	if (x < 0)
		return(-x);
	else
		return(x);
}
