/*
*      SCCS:  @(#)POSIX.os/procprim/waitpid/waitpid.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/procprim/waitpid - T.waitpid Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.waitpid

PROJECT:	VSX (X/OPEN Issue 3 Validation Suite)

DESCRIPTION:	Testset for the waitpid() system call.

AUTHOR:		Dave Cater, UniSoft Ltd.
DATE CREATED:	Tue Apr 11 10:26:43 BST 1989
MODIFICATIONS:
		Geoff Clare, 17 July 1989
			If setpgid() is not supported, use setsid() to
			create new process groups instead.
		Dave Cater, UniSoft Ltd.
		Mon Sep 18 14:37:06 BST 1989
			Corrected VNOSIG to VNOJSIG
		
		Geoff Clare, 11 Oct 1989
			Increase wait time in test7 and test17

		Dave Cater, 1 Nov 1989
			Improve test1 part 1 to ensure waitpid() really waits.
			Correct message in test1 part 2 
			Correct comment in test15
			Remove main_ret from test11
			test12 now tests that ONLY low order 8 bits are obtained
		
		Geoff Clare, 27 Apr 1990
			In the test for WNOHANG only the WUNTRACED case
			is conditional on job control

		Keith Holder, 25 Jun 1990
			Added 3 new deviance tests.

		Geoff Clare, 27 June 1990
			Delete test that orphaned children get new parent pid

		David Sawyer
		Wed Aug 15 13:41:32 BST 1990
		Added resilience code.

		Stuart Boutell, 2 Nov 1990
			changed execl for tet_exec ; added environ reference

		Stuart Boutell, 7 Nov 1990
			tet_child cleaned up.

		Stuart Boutell, 12 Nov 1990
			uids() return value check added
			Changed DBUG_VOID_RETURN to DBUG_EXIT(OK) at end
			of ch1p1; Extended test15,16 WAITTIME to WAITTIME*2

		Stuart Boutell, 4 Dec 1990
			Added tet_setcontext() and tet_setblock() calls
			where fork() is called

		Stuart Boutell, 30 Apr 1991
			Changed cpsig() return code check from FAIL to
			SYSERROR

		Calum Laing,	12 Jun 1992
			Added code to ensure wait status of children was
			still available to subsequent waitpid() calls in
			tests 18, 19, 20, 21, 23 and 24.

		Geoff Clare, 20 July 1992
			Changed some int variables to pid_t.

		Geoff Clare, 20 Jan 1995
			Increased some cppair() timeouts.

************************************************************************/
#include	<std.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<dbug.h>
#include	<report.h>
#include	<exitcodes.h>
#include	<timeout.h>
#include	<wait_child.h>
#include	<signal.h>
#include	<vsx_signal.h>
#include	<sys/stat.h>
#include	<sys/wait.h>
#include	<time.h>
#include	<stdio.h>
#include	<errno.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef waitpid
#endif


#define	WAITTIME		(5 * SPEEDFACTOR)
#define	EXTERR			"external error"
#define	NCHILDREN		5
#define	NO_TESTS		24		/* number of tests in testset */
#define	NEGATIVE		-1
#define	ZERO			0
#define	BUF_LEN			80
#define	BYTEMASK		0xff
#define	TWOBYTEMASK		0xffff

extern	void		vsx_sigs();
extern	void		valrpt();
extern	char		*itoa();
extern	char		*newstr();
extern	char		*status_child();

extern	char		**environ;

extern	int		sig_array[];

int	no_tests = NO_TESTS;

private char		*prog_exec  = "../wait_exec";
private	int		sig_err;
private	sigset_t	sig_empty;
private	int		nsigs;
private	int		test_case;
private	int		test_fail;
private	int		test_deleted;
private	int		sig_recvd;
private	int		job_control;
private	int		main_ret;
private	int		test_exit;
private	int		global_opts;
private	pid_t		global_pid;
private	int		setupok;

private	void		test1();
private	void		ch1p1();
private	void		pa1p1();
private	void		ch1p2();
private	void		pgch1p2();
private	void		pa1p2();
private	void		ch1p3();
private	void		gch1p3();
private	void		pgch1p3();
private	void		pa1p3();
private	void		test2();
private	void		ch2();
private	void		pa2();
private	void		test3();
private	void		ch3();
private	void		pa3p1();
private	void		pa3p2();
private	void		test4();
private	void		ch4();
private	void		test5();
private	void		test6();
private	void		ch5and6();
private	void		test7();
private	void		ch7();
private	void		pa7p1();
private	void		test8();
private	void		ch8();
private	void		pa8p1();
private	void		pa8p2();
private	void		test9();
private	void		ch9();
private	void		pa9();
private	void		test10();
private	void		ch10();
private	void		pa10();
private	void		test11();
private	void		ch11();
private	void		pa11();
private	void		test12();
private	void		ch12();
private	void		pa12();
private	void		test13();
private	void		ch13();
private	void		pa13();
private	void		test14();
private	void		ch14();
private	void		pa14();
private	void		test15();
private	void		ch15();
private	void		pa15();
private	void		test16();
private	void		ch16();
private	void		pa16();
private	void		test17();
private	void		ch17();
private	void		pa17();
private	void		test18();
private	void		ch18();
private	void		pa18p1();
private	void		pa18p2();
private	void		test19();
private	void		ch19();
private	void		pa19();
private	void		test20();
private	void		ch20();
private	void		pa20();
private	void		test21();
private	void		ch21();
private	void		pa21();
private	void		test22();
private	void		ch22();
private	void		test23();
private	void		t_23();
private	void		pa23();
private	void		test24();
private	void		t_24();
private	void		chlongsleep();
private	void		ch_exit();
private	void		chexit();
private	void		chkilled();
private	void		chstop();
private	void		prepare_tests();
private	void		sig_catch();
private	int		sig_init();
private	int		wait_child();

/* Goes with our private wait_child() at end of waitpid.c */
public	int	ex_code;
public	int	sg_code;

public	void	(*setfunc) () = prepare_tests;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	  test2, 2,
	  test3, 3,
	  test4, 4,
	  test5, 5,
	 test6, 6,
	  test7, 7,
	  test8, 8,
	  test9, 9,
	  test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	NULL, 0
};

private	void
prepare_tests()
{
	int	i;	/* Counter */
	int	ret;	/* Used for most return values */
	int	pathok = 0;

	DBUG_ENTER("prepare_tests");

	globok = 0;
	vsx_sigs();
	PATH_FUNC_TRACE;


	if(uids()!=0)
	{
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* check what signals are in the standard */
	ret = sysconf(_SC_JOB_CONTROL);
	if(ret == -1) {
		job_control = 0;
		nsigs = VNOJSIG;
	} else {
		job_control = 1;
		nsigs = VNUMSIG;
	}

	(void) sigemptyset(&sig_empty);

	for(i = 1; i < nsigs; i++) {
		if (i==1)
			PATH_TRACE;
		if(i == VSIGINT)	/* Still allow user to interrupt */
			continue;
		(void) sig_init(sig_array[i], &sig_empty, SIG_DFL, 0);
	}

	(void) sigemptyset(&sig_empty);

	PATH_SETUP_RPT(3);

	DBUG_VOID_RETURN;
}

private int
sig_init(signum, sigmask, sigfunc, sigflags)
int	signum;		/* signal to act for */
sigset_t *sigmask;	/* signals to mask out */
void (*sigfunc)();	/* one of SIG_IGN, SIG_DFL or s signal catcher */
int	sigflags;	/* flags attached to signal action */
{
	struct sigaction act;	/* signal action structure */

	/* 
	 * Call sigaction() to initialise signal handling function.
	 * Return the value returned by sigaction()
	 */
	act.sa_flags   = sigflags;
	act.sa_mask    = *sigmask;
	act.sa_handler = sigfunc;
	return(sigaction(signum, &act, (struct sigaction *)NULL));
}

private void
sig_catch(sg_num)
int	sg_num;
{
	DBUG_ENTER("sig_catch");

	sig_recvd = sg_num;

	DBUG_PRINTF("sig_catch", "sig_recvd %d", sig_recvd);

	DBUG_VOID_RETURN;
}

private void
chlongsleep()
{
	DBUG_ENTER("chlongsleep");

	/* Sleep for a while so waitpid() gets interrupted */
	(void) sleep(WAITTIME);

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
ch_exit()
{
	DBUG_ENTER("ch_exit");

	/* Set up a new process group with just this process in it */
	errno = 0;
	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR && errno == ENOSYS)
		(void) setsid();

	/* Exit straight away */
	_exit(test_exit);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
chexit()
{
	DBUG_ENTER("chexit");

	/* Set up a new process group with just this process in it */
	errno = 0;
	if (setpgid((pid_t)0, (pid_t)0) == SYSERROR && errno == ENOSYS)
		(void) setsid();

	/* Exit straight away */
	exit(test_exit);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
chkilled()
{
	DBUG_ENTER("chkilled");

	/* Kill this process straight away */
	(void) kill(getpid(), SIGKILL);

	/* NOTREACHED */
	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
chstop()
{
	DBUG_ENTER("chstop");

	/* Stop this process by sending a SIGSTOP signal */
	(void) kill(getpid(), SIGSTOP);

	/* cpsig() will send SIGKILL to tidy this process up */

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
chexec()
{
	char	*args[3];
	char	*retstr;

	DBUG_ENTER("chexec");

	if ((retstr = newstr(itoa(main_ret))) == (char *) NULL)
	{
		vsxerror("external error %d", A_NOMEM);
		DBUG_EXIT(A_NOMEM);
	}

	args[0] = prog_exec;
	args[1] = retstr;
	args[2] = (char *)NULL;
	(void) tet_exec(prog_exec, args, environ);
	vsxerror("tet_exec(%s,...) failed errno %d", prog_exec, errno);
	DBUG_EXIT(A_EXECFAIL);

	DBUG_VOID_RETURN;
}

/*
 * A private version of wait_child() which uses wait() not waitpid().
 * for the testing of waitpid().
 */

private
int wait_child(waittime)
int	waittime;
{
	int	stat_child;	/* child process status */
	int	status;		/* wait return status */
	pid_t	rval;		/* return value of waitpid */

	errno = NOERROR;

	for( ;; ) {
		SET_TIMEOUT(waittime)
		rval = wait(&status);
		CLEAR_ALARM
		DBUG_PRINTF("wait_child()", "rval %ld", (long)rval);
		if(rval == SYSERROR)
			break;
		if(rval == tet_child)
			break;
	}

	if(rval == SYSERROR) {
		if(errno == EINTR && alrm_flag) {
			stat_child = CHLD_LIVE;
		} else {
			stat_child = CHLD_ERR;
		}
	} else if(WIFEXITED(status)) {
		stat_child = CHLD_EXIT;
		ex_code = WEXITSTATUS(status);
	} else if(WIFSIGNALED(status)) {
		stat_child = CHLD_ABRT;
		sg_code = WTERMSIG(status);
	} else if(WIFSTOPPED(status)) {
		stat_child = CHLD_STOP;
	} else {
		/* SHOULD NOT GET HERE - waitpid() WRONGLY IMPLEMENTED */
		/* errno will be set to NOERROR if this is the case */
		stat_child = CHLD_ERR;
	}

	return(stat_child);
}

private void
test1()
{
	int	pathok = setupok;

	DBUG_ENTER("test1");

	/*
	 * Test that waitpid() suspends execution of a process until:
	 * Part 1) information is available from a terminated child process.
	 * Part 2) delivery of a signal which is caught.
	 * Part 3) delivery of a signal which terminates the process.
	 */

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(ch1p1, pa1p1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if(cpsig(ch1p2, pa1p2, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if(cpsig(ch1p3, pa1p3, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
ch1p1()
{
	int	status;

	DBUG_ENTER("ch1p1");

	(void) fflush(stdout);
	(void) fflush(stderr);
	switch(tet_child = fork()) {

	case SYSERROR:

		xx_rpt(DELETION);
		in_rpt("deletion reason - fork() failed");
		DBUG_VOID_RETURN;

	case 0:		/* child */

		tet_setcontext();
		(void) sleep(WAITTIME);

		DBUG_EXIT(OK);

		/* NOTREACHED */
		break;

	default:	/* parent */

		tet_setblock();
		/* Set a timeout in case the wait never returns */
		SET_TIMEOUT(2*WAITTIME)

		/* THIS IS THE CALL THAT IS TESTED IN TEST1 PART 1 */
		(void) waitpid(-1, (int *)&status, 0);

		CLEAR_ALARM

		break;
	}

	tet_setblock();
	DBUG_EXIT(OK);
}

private void
pa1p1()
{
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa1p1");

	sig_err = 0;

	/* Wait for process executing waitpid() 
		- expect it to exit when it's child terminates */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME/2)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("waitpid() did not suspend process execution");
		in_rpt("expected action: process still active");
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != OK) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("waitpid() did not return when child terminated");
		in_rpt("expected action: process exitted - exit code %d", OK);
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
ch1p2()
{
	DBUG_ENTER("ch1p2");

	/* Set the required action if we can */
	errno = NOERROR;
	if(sig_init(SIGUSR1, &sig_empty, sig_catch, 0) < 0) {
		vsxerror("sig_init() failed, errno %d", errno);
		DBUG_EXIT(A_NOSIG);
	}

	(void) cpsig(chlongsleep, pgch1p2, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
pgch1p2()
{
	int	status;

	DBUG_ENTER("pgch1p2");

	sig_recvd = 0;

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME)

	/* THIS IS THE CALL THAT IS TESTED IN TEST1 PART 2 */
	(void) waitpid(-1, (int *)&status, 0);

	CLEAR_ALARM

	globok = 1; /* cpsig expects globok to be set */

	/* Exit with the number of signal caught if any */
	DBUG_EXIT(sig_recvd);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pa1p2()
{
	int	ret;
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa1p2");

	sig_err = 0;

	/* Wait for process executing waitpid() - ensure it's alive */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME/2)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitpid() returned unexpectedly");
		in_rpt("expected action: process still active");
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to send the signal */
	errno = NOERROR;
	ret = kill(tet_child, SIGUSR1);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("kill() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Expect child to exit with the number of signal received */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_EXIT || ex_code != SIGUSR1) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("process failed to catch signal during waitpid()");
		in_rpt("expected action: process exitted - exit code %d", 
								SIGUSR1);
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5);

	DBUG_VOID_RETURN;
}

private void
ch1p3()
{
	DBUG_ENTER("ch1p3");

	/* Set the required action if we can */
	errno = NOERROR;
	if(sig_init(SIGUSR1, &sig_empty, SIG_DFL, 0) < 0) {
		vsxerror("sig_init() failed, errno %d", errno);
		DBUG_EXIT(A_NOSIG);
	}

	(void) cpsig(gch1p3, pgch1p3, WAITTIME, 0);

	vsxerror("waitpid() returned unexpectedly");
	DBUG_EXIT(A_WAITFAIL);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
gch1p3()
{
	DBUG_ENTER("gch1p3");

	/* Sleep for a while so waitpid() gets the signal */
	(void) sleep(WAITTIME);

	DBUG_EXIT(OK);

	/* NOTREACHED */
	DBUG_VOID_RETURN;
}

private void
pgch1p3()
{
	int	status;

	DBUG_ENTER("pgch1p3");

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME)

	/* THIS IS THE CALL THAT IS TESTED IN TEST1 PART 3 */
	(void) waitpid(-1, (int *)&status, 0);

	CLEAR_ALARM

	DBUG_VOID_RETURN;
}

private void
pa1p3()
{
	int	ret;
	int	err;
	int	stat_child;
	int	pathok = 0;

	DBUG_ENTER("pa1p3");

	sig_err = 0;

	/* Wait for process executing waitpid() - ensure it's alive */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME/2)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_LIVE) {
		xx_rpt(DELETION);
		in_rpt("deletion reason: waitpid() returned unexpectedly");
		in_rpt("expected action: process still active");
		in_rpt("observed action: %s", status_child(stat_child));
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Try to send the signal */
	errno = NOERROR;
	ret = kill(tet_child, SIGUSR1);
	if(ret < 0) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("kill() failed, errno %d", err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Expect child to abort due to the signal */
	errno = NOERROR;
	if((stat_child = wait_child(WAITTIME)) == CHLD_ERR) {
		err = errno;
		xx_rpt(DELETION);
		in_rpt("deletion reason: %s - waitpid() failed, errno %d", 
								EXTERR, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(stat_child != CHLD_ABRT || sg_code != SIGUSR1) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("process not terminated by signal during waitpid()");
		in_rpt("expected action: signal %d terminated process", 
								SIGUSR1);
		in_rpt("observed action: %s", status_child(stat_child));
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(5); /* cpsig expects globok to be set */

	DBUG_VOID_RETURN;
}

private void
test2()
{
	DBUG_ENTER("test2");

	/*
	 * Test that waitpid() returns immediately if prior information is 
	 * available.
	 */

	(void) cppair(ch2, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch2()
{
	int	pathok = setupok;

	DBUG_ENTER("ch2");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(chexit, pa2, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa2()
{
	int	t_fail = 0;
	int	status;
	time_t	time_before;
	time_t	time_after;
	int	pathok = 0;

	DBUG_ENTER("pa2");

	/* Sleep for long enough for child to give status information */
	(void) sleep(WAITTIME/2);

	if(time(&time_before) == (time_t)SYSERROR)
		t_fail++;

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)

	(void) waitpid(-1, (int *)&status, 0);

	CLEAR_ALARM

	/* Check the time is not much beyond that found before waitpid() */
	if(time(&time_after) == (time_t)SYSERROR)
		t_fail++;

	if(t_fail) {
		xx_rpt(DELETION);
		in_rpt("time() failed");
		test_deleted++;
	} else if(time_after > time_before + (time_t)1) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("waitpid() did not return immediately"); 
		in_rpt("time before waitpid(): %ld; time after waitpid(): %ld", 
					(long)time_before, (long)time_after);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test3()
{
	DBUG_ENTER("test3");

	/*
	 * Test that waitpid() returns the process ID of the child if it
	 * returned because child status information is available.
	 */

	(void) cppair(ch3, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch3()
{
	int	pathok = setupok;

	DBUG_ENTER("ch3");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(chexit, pa3p1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if(cpsig(chexit, pa3p2, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
pa3p1()
{
	int	status;
	int	err;
	pid_t	ret;
	char	*str = "waitpid( , stat_loc, )";
	int	pathok = 0;

	DBUG_ENTER("pa3p1");

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)

	errno = NOERROR;
	ret = waitpid(-1, (int *)&status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret == SYSERROR) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld and set errno %d", (long)ret, err);
		test_fail++;
	} else if(ret != tet_child) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld ; child process ID %ld", 
						(long)ret, (long)tet_child);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa3p2()
{
	pid_t	ret;
	int	err;
	char	*str = "waitpid( , (int *)NULL, )";
	int	pathok = 0;

	DBUG_ENTER("pa3p2");

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)

	errno = NOERROR;
	ret = waitpid((pid_t)-1, (int *)NULL, 0);
	err = errno;

	CLEAR_ALARM

	if(ret == SYSERROR) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld and set errno %d", (long)ret, err);
		test_fail++;
	} else if(ret != tet_child) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld ; child process ID %ld", 
						(long)ret, (long)tet_child);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test4()
{
	DBUG_ENTER("test4");

	/*
	 * Test that waitpid(pid, , ) returns the process ID of the child when 
	 * the process "pid" terminates.
	 */

	(void) cppair(ch4, NULLFN, 2*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch4()
{
	int	fork_fail = 0;
	int	status;
	pid_t	ret;
	int	err;
	int	i;
	int	children[NCHILDREN];
	pid_t	pid;
	int	pathok = setupok;

	DBUG_ENTER("ch4");

	test_fail = 0;
	test_deleted = 0;

	/* Produce a number of children which exit at different times */
	for (i = 0 ; i < NCHILDREN ; i++)
	{
		PATH_TRACE; /* trace number of times round loop */
		(void) fflush(stdout);
		(void) fflush(stderr);
		switch (children[i] = fork())
		{
		case SYSERROR:
			fork_fail++;
			break;

		case 0:		/* child */

			tet_setcontext();
			(void) sleep((unsigned)(i * SPEEDFACTOR));
			DBUG_EXIT(OK);

			/* NOTREACHED */
			break;

		/* parent does nothing yet */
		}
		tet_setblock();
	}
	/* parent of NCHILDREN */

	tet_setblock();
	/* Choose a child to wait for near the middle of the list */
	pid = children[NCHILDREN/2];

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(NCHILDREN * SPEEDFACTOR)

	errno = NOERROR;
	ret = waitpid(pid, (int *)&status, 0);
	err = errno;

	CLEAR_ALARM

	/* Wait for rest of children */
	while( wait((int *)&status) != SYSERROR)
		;

	if(fork_fail) {
		xx_rpt(DELETION);
		in_rpt("fork() failed");
	} else if(ret == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("waitpid(pid, stat_loc, options) did not return pid"); 
		in_rpt("instead returned %ld and set errno %d", (long)ret, err);
	} else if(ret != pid) {
		xx_rpt(FAILURE);
		in_rpt("waitpid(pid, stat_loc, options) did not return pid (process ID)"); 
		in_rpt("instead returned %ld ; process ID %ld", (long)ret, (long)pid);
	} else
	{
		PATH_TRACE;
		PATH_XS_RPT(1+NCHILDREN);
	}

	DBUG_VOID_RETURN;
}

private void
test5()
{
	DBUG_ENTER("test5");

	/*
	 * Test that waitpid(0, , ) returns when a process with process group ID
	 * equal to that of the calling process terminates.
	 */

	test_case = ZERO;

	(void) cppair(ch5and6, NULLFN, 2*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
test6()
{
	DBUG_ENTER("test6");

	/*
	 * Test that waitpid(-pid, , ) returns when a process with process 
	 * group ID equal to "pid" terminates.
	 */

	test_case = NEGATIVE;

	(void) cppair(ch5and6, NULLFN, 2*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch5and6()
{
	char	*zerostr = "waitpid(0, stat_loc, options)";
	char	*negstr  = "waitpid(-pid, stat_loc, options)";
	char	*str;
	int	fork_fail = 0;
	int	status;
	pid_t	ret;
	int	err;
	int	i;
	int	children[NCHILDREN];
	int	pathok = setupok;

	DBUG_ENTER("ch5and6");

	test_fail = 0;
	test_deleted = 0;

	/* Put process in new process group - needed for test6 */
	errno = NOERROR;
	if(setpgid((pid_t)0, (pid_t)0) < 0) {
		if (errno != ENOSYS) {
			err = errno;
			xx_rpt(DELETION);
			in_rpt("deletion reason: setpgid() failed, errno %d", err);
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
		(void) setsid();
	}
	else
		PATH_TRACE;

	/* Produce a number of children which exit at different times */
	for (i = 0 ; i < NCHILDREN ; i++)
	{
		PATH_TRACE; /* trace number of times round loop */
		(void) fflush(stdout);
		(void) fflush(stderr);
		switch (children[i] = fork())
		{
		case SYSERROR:
			fork_fail++;
			break;

		case 0:		/* child */

			tet_setcontext();
			/* Put the first half of them in new process group */
			if(i < NCHILDREN/2) 
			{
				errno = 0;
				if (setpgid((pid_t)0, (pid_t)0) == SYSERROR &&
						errno == ENOSYS)
					(void) setsid();
			}

			(void) sleep((unsigned)(i * SPEEDFACTOR));
			DBUG_EXIT(OK);

			/* NOTREACHED */
			break;

		/* parent does nothing yet */
		}
		tet_setblock();
	}
	/* parent of NCHILDREN */

	tet_setblock();
	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(NCHILDREN * SPEEDFACTOR)

	switch(test_case) {
	case NEGATIVE:
		str = negstr;
		DBUG_PRINTF("ch5and6", "%s", str);
		errno = NOERROR;
		ret = waitpid(-getpid(), (int *)&status, 0);
		PATH_TRACE;
		break;
	case ZERO:
	default:
		str = zerostr;
		DBUG_PRINTF("ch5and6", "%s", str);
		errno = NOERROR;
		ret = waitpid((pid_t)0, (int *)&status, 0);
		PATH_TRACE;
		break;
	}
	err = errno;

	CLEAR_ALARM

	/* Wait for rest of children */
	while( wait((int *)&status) != SYSERROR)
		;

	if(fork_fail) {
		xx_rpt(DELETION);
		in_rpt("fork() failed");
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Make sure the returned process was not in first half */
	for(i = 0; i < NCHILDREN; i++)
	{
		if (i==0)
			PATH_TRACE;
		if(ret == children[i])
			break;
	}

	if(ret == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("%s did not return process in process group", str); 
		in_rpt("instead returned %ld and set errno %d", (long)ret, err);
	} else if(i < NCHILDREN/2) {
		xx_rpt(FAILURE);
		in_rpt("%s returned process in wrong process group", str);
	} else if(i >= NCHILDREN) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - ");
		in_rpt("%s returned unknown child %ld", str, (long)ret);
	} else
	{
		PATH_TRACE;
		PATH_XS_RPT(5+NCHILDREN);
	}

	DBUG_VOID_RETURN;
}

private void
test7()
{
	DBUG_ENTER("test7");

	/*
	 * Test that neither waitpid(, , WNOHANG) or 
	 * waitpid(, , WNOHANG|WUNTRACED)  suspends execution
	 * if no information is immediately available.
	 */

	(void) cppair(ch7, NULLFN, (2 * WAITTIME), 0);

	DBUG_VOID_RETURN;
}

private void
ch7()
{
	int	pathok = setupok;

	DBUG_ENTER("ch7");

	test_fail = 0;
	test_deleted = 0;

	global_opts = WNOHANG;

	globok = 0;
	if(cpsig(chlongsleep, pa7p1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (job_control)
	{
		global_opts = WNOHANG|WUNTRACED;

		globok = 0;
		if(cpsig(chlongsleep, pa7p1, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;
	}
	else
		PATH_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
pa7p1()
{
	int	err;
	pid_t	ret;
	int	status;
	int	t_fail = 0;
	time_t	time_before;
	time_t	time_after;
	char	buf[BUF_LEN];
	char	*str = buf;
	int	pathok = 0;

	DBUG_ENTER("pa7p1");

	(void) sprintf(buf, "waitpid( , ,%d)", global_opts);

	/* Sleep for long enough for child to reach sleep() */
	(void) sleep(WAITTIME/2);

	if(time(&time_before) == (time_t)SYSERROR)
		t_fail++;

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)
	errno = NOERROR;
	ret = waitpid(tet_child, (int *)&status, global_opts);
	err = errno;
	CLEAR_ALARM

	/* Check the time is not much beyond that found before waitpid() */
	if(time(&time_after) == (time_t)SYSERROR)
		t_fail++;

	if(t_fail) {
		if(!test_fail)
			xx_rpt(DELETION);
		in_rpt("time() failed");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(time_after > time_before + (time_t)1) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not return immediately", str); 
		in_rpt("time before waitpid(): %ld; time after waitpid(): %ld", 
					(long)time_before, (long)time_after);
		test_fail++;
	} else if(ret == SYSERROR) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not return zero as expected", str);
		in_rpt("instead returned %ld and set errno to %d", (long)ret, err);
		test_fail++;
	} else if(ret != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not return zero as expected", str);
		in_rpt("instead returned %ld ; child process ID %ld", 
						(long)ret, (long)tet_child);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test8()
{
	DBUG_ENTER("test8");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}

	/*
	 * Test that both waitpid(, , WUNTRACED) and 
	 * waitpid(, , WNOHANG|WUNTRACED) reports the status of
	 * stopped child process that have not been waited for as well as
	 * those that have exitted.
	 */

	(void) cppair(ch8, NULLFN, 6*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch8()
{
	int	pathok = setupok;

	DBUG_ENTER("ch8");

	test_fail = 0;
	test_deleted = 0;

	global_opts = WUNTRACED;

	globok = 0;
	if(cpsig(chstop, pa8p1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if(cpsig(chexit, pa8p2, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	global_opts = WNOHANG|WUNTRACED;

	globok = 0;
	if(cpsig(chstop, pa8p1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if(cpsig(chexit, pa8p2, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(4);

	DBUG_VOID_RETURN;
}

private void
pa8p1()
{
	int	err;
	pid_t	ret;
	int	status;
	char	buf[BUF_LEN];
	char	*str = buf;
	int	pathok = 0;

	DBUG_ENTER("pa8p1");

	(void) sprintf(buf, "waitpid( , ,%d)", global_opts);

	/* Sleep for long enough for child to give status information */
	(void) sleep(WAITTIME/2);

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)
	errno = NOERROR;
	ret = waitpid(tet_child, (int *)&status, global_opts);
	err = errno;
	CLEAR_ALARM

	if(ret == SYSERROR) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not report on stopped process", str); 
		in_rpt("instead returned %ld and set errno to %d", (long)ret, err);
		test_fail++;
	} else if(ret != tet_child) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not report on stopped process", str); 
		in_rpt("instead returned %ld ; stopped process ID %ld", 
						(long)ret, (long)tet_child);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa8p2()
{
	int	err;
	pid_t	ret;
	int	status;
	char	buf[BUF_LEN];
	char	*str = buf;
	int	pathok = 0;

	DBUG_ENTER("pa8p2");

	(void) sprintf(buf, "waitpid( , ,%d)", global_opts);

	/* Sleep for long enough for child to give status information */
	(void) sleep(WAITTIME/2);

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)
	errno = NOERROR;
	ret = waitpid(tet_child, (int *)&status, global_opts);
	err = errno;
	CLEAR_ALARM

	if(ret == SYSERROR) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not report on exitted process", str); 
		in_rpt("instead returned %ld and set errno to %d", (long)ret, err);
		test_fail++;
	} else if(ret != tet_child) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not report on exitted process", str); 
		in_rpt("instead returned %ld ; exitted process ID %ld", 
						(long)ret, (long)tet_child);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test9()
{
	DBUG_ENTER("test9");

	/*
	 * Test that waitpid(-1, stat_loc, 0) stores status information in 
	 * *stat_loc when stat_loc is not NULL.
	 */

	(void) cppair(ch9, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch9()
{
	int	pathok = setupok;

	DBUG_ENTER("ch9");

	test_fail = 0;
	test_deleted = 0;

	main_ret = A_MISC;	/* value to be returned from main() */
	test_exit = A_MISC;	/* value to be passed to (_)exit() */

	globok = 0;
	if(cpsig(ch_exit, pa9, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if(cpsig(chexit, pa9, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if(cpsig(chexec, pa9, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
pa9()
{
	int	status;
	pid_t	ret;
	int	pathok = 0;

	DBUG_ENTER("pa9");

	status = 0;	/* ensure it is altered */

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)
	ret = waitpid(-1, (int *)&status, 0);
	CLEAR_ALARM

	if(ret != tet_child) {
		xx_rpt(DELETION);
		in_rpt("waitpid() did not return child process ID"); 
		in_rpt("instead returned %ld", (long)ret);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(status == 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("*stat_loc evaluated to zero");
		in_rpt("status information not stored in *stat_loc");
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test10()
{
	DBUG_ENTER("test10");

	/*
	 * Test that waitpid(-1, stat_loc, 0) stores zero in 
	 * *stat_loc when stat_loc is not NULL and a child terminates 
	 * returning 0 from main() or passed to exit() or _exit().
	 */

	(void) cppair(ch10, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch10()
{
	int	pathok = setupok;

	DBUG_ENTER("ch10");

	test_fail = 0;
	test_deleted = 0;

	main_ret = 0;	/* value to be returned from main() */
	test_exit = 0;	/* value to be passed to (_)exit() */

	globok = 0;
	if(cpsig(ch_exit, pa10, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if(cpsig(chexit, pa10, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	globok = 0;
	if(cpsig(chexec, pa10, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(3);

	DBUG_VOID_RETURN;
}

private void
pa10()
{
	int	status;
	pid_t	ret;
	int	pathok = 0;

	DBUG_ENTER("pa10");

	status = 1;	/* ensure it is altered */

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)
	ret = waitpid(-1, (int *)&status, 0);
	CLEAR_ALARM

	if(ret != tet_child) {
		xx_rpt(DELETION);
		in_rpt("waitpid() did not return child process ID"); 
		in_rpt("instead returned %ld", (long)ret);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(status != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("*stat_loc did not evaluate to zero");
		in_rpt("instead *stat_loc evaluated to %d", status);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test11()
{
	DBUG_ENTER("test11");

	/*
	 * Test that waitpid(-1, stat_loc, 0) sets *stat_loc so that
	 * WIFEXITED(*stat_loc) evaluates to non-zero value,
	 * WIFSIGNALED(*stat_loc) evaluates to zero,
	 * WIFSTOPPED(*stat_loc) evaluates to zero,
	 * when a child terminates normally.
	 */

	(void) cppair(ch11, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch11()
{
	int	pathok = setupok;

	DBUG_ENTER("ch11");

	/*
	 * Test for a zero and non-zero exit code.
	 */
	test_fail = 0;
	test_deleted = 0;

	test_exit = A_MISC;	/* value to be passed to (_)exit() */

	globok = 0;
	if(cpsig(chexit, pa11, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	test_exit = 0;	/* value to be passed to (_)exit() */

	globok = 0;
	if(cpsig(chexit, pa11, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
pa11()
{
	int	status;
	int	val;
	pid_t	ret;
	int	pathok = 0;

	DBUG_ENTER("pa11");

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)
	ret = waitpid(-1, (int *)&status, 0);
	CLEAR_ALARM

	if(ret != tet_child) {
		xx_rpt(DELETION);
		in_rpt("waitpid() did not return child process ID"); 
		in_rpt("instead returned %ld", (long)ret);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(WIFEXITED(status) == 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("WIFEXITED() did not return non-zero value");
		test_fail++;
	}
	else
		PATH_TRACE;

	if((val = WIFSIGNALED(status)) != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("WIFSIGNALED() returned non-zero value %d", val);
		test_fail++;
	}
	else
		PATH_TRACE;

	if((val = WIFSTOPPED(status)) != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("WIFSTOPPED() returned non-zero value %d", val);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test12()
{
	DBUG_ENTER("test12");

	/*
	 * Test that waitpid(-1, stat_loc, 0) sets *stat_loc so that
	 * when WIFEXITED(*stat_loc) evaluates to non-zero value,
	 * WEXITSTATUS(*stat_loc) evaluates to the low order 8 bits of the 
	 * value returned from main() or passed to exit() or _exit().
	 */

	(void) cppair(ch12, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch12()
{
	int	i;
	int	loop = 0;
	int	pathok = setupok;

	DBUG_ENTER("ch12");

	test_fail = 0;
	test_deleted = 0;

	for(i = 0; test_exit < TWOBYTEMASK; i++) {

		if(i == 0)
			test_exit = 0;
		else if(i == 1) 
			test_exit = 1;
		else
			test_exit = (2*test_exit) | test_exit;
	
		main_ret = test_exit;		/* to be returned from main() */

		globok = 0;
		if(cpsig(ch_exit, pa12, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		globok = 0;
		if(cpsig(chexit, pa12, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		globok = 0;
		if(cpsig(chexec, pa12, WAITTIME, 0) == SYSERROR)
			test_deleted++;
		if(test_deleted)
		{
			DBUG_VOID_RETURN;
		}
		else
			PATH_FUNC_TRACE;

		loop++;
	}

	if(!test_fail)
		PATH_XS_RPT(3*loop);

	DBUG_VOID_RETURN;
}

private void
pa12()
{
	int	status;
	int	val;
	pid_t	ret;
	int	pathok = 0;

	DBUG_ENTER("pa12");

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)
	ret = waitpid(-1, (int *)&status, 0);
	CLEAR_ALARM

	if(ret != tet_child) {
		xx_rpt(DELETION);
		in_rpt("waitpid() did not return child process ID"); 
		in_rpt("instead returned %ld", (long)ret);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(WIFEXITED(status) == 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - WIFEXITED() evaluated to zero");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((val = WEXITSTATUS(status)) != (test_exit&BYTEMASK)) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("WEXITSTATUS() did not evaluate to %d", 
							test_exit&BYTEMASK);
		in_rpt("instead WEXITSTATUS() evaluated to %d", val);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test13()
{
	DBUG_ENTER("test13");

	/*
	 * Test that waitpid(-1, stat_loc, 0) sets *stat_loc so that
	 * WIFEXITED(*stat_loc) evaluates to zero,
	 * WIFSIGNALED(*stat_loc) evaluates to a non-zero value,
	 * WIFSTOPPED(*stat_loc) evaluates to zero,
	 * when a child terminates due to receipt of an uncaught signal.
	 */

	(void) cppair(ch13, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch13()
{
	int	pathok = setupok;

	DBUG_ENTER("ch13");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(chkilled, pa13, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa13()
{
	int	status;
	int	val;
	pid_t	ret;
	int	pathok = 0;

	DBUG_ENTER("pa13");

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)
	ret = waitpid(-1, (int *)&status, 0);
	CLEAR_ALARM

	if(ret != tet_child) {
		xx_rpt(DELETION);
		in_rpt("waitpid() did not return child process ID"); 
		in_rpt("instead returned %ld", (long)ret);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(WIFSIGNALED(status) == 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("WIFSIGNALED() did not return non-zero value");
		test_fail++;
	}
	else
		PATH_TRACE;

	if((val = WIFEXITED(status)) != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("WIFEXITED() returned non-zero value %d", val);
		test_fail++;
	}
	else
		PATH_TRACE;

	if((val = WIFSTOPPED(status)) != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("WIFSTOPPED() returned non-zero value %d", val);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test14()
{
	DBUG_ENTER("test14");

	/*
	 * Test that waitpid(-1, stat_loc, 0) sets *stat_loc so that
	 * when WIFSIGNALED(*stat_loc) evaluates to non-zero value,
	 * WTERMSIG(*stat_loc) evaluates to the signal that caused 
	 * the child process to terminate.
	 */

	(void) cppair(ch14, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch14()
{
	int	pathok = setupok;

	DBUG_ENTER("ch14");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(chkilled, pa14, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa14()
{
	int	status;
	int	val;
	pid_t	ret;
	int	pathok = 0;

	DBUG_ENTER("pa14");

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)
	ret = waitpid(-1, (int *)&status, 0);
	CLEAR_ALARM

	if(ret != tet_child) {
		xx_rpt(DELETION);
		in_rpt("waitpid() did not return child process ID"); 
		in_rpt("instead returned %ld", (long)ret);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(WIFSIGNALED(status) == 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - WIFSIGNALED() evaluated to zero");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((val = WTERMSIG(status)) != SIGKILL) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("WTERMSIG() did not evaluate to %d", SIGKILL);
		in_rpt("instead WTERMSIG() evaluated to %d", val);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);

	DBUG_VOID_RETURN;
}

private void
test15()
{
	DBUG_ENTER("test15");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}

	/*
	 * Test that waitpid(-1, stat_loc, 0) sets *stat_loc so that
	 * WIFEXITED(*stat_loc) evaluates to zero,
	 * WIFSIGNALED(*stat_loc) evaluates to zero,
	 * WIFSTOPPED(*stat_loc) evaluates to a non-zero value,
	 * when a child is stopped due to receipt of a signal.
	 */

	(void) cppair(ch15, NULLFN, WAITTIME*2, 0);

	DBUG_VOID_RETURN;
}

private void
ch15()
{
	int	pathok = setupok;

	DBUG_ENTER("ch15");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(chstop, pa15, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa15()
{
	int	status;
	int	val;
	pid_t	ret;
	int	pathok = 0;

	DBUG_ENTER("pa15");

	/* Sleep for long enough for child to give status information */
	(void) sleep(WAITTIME/2);

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)
	ret = waitpid(-1, (int *)&status, WNOHANG|WUNTRACED);
	CLEAR_ALARM

	if(ret != tet_child) {
		xx_rpt(DELETION);
		in_rpt("waitpid() did not return child process ID"); 
		in_rpt("instead returned %ld", (long)ret);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(WIFSTOPPED(status) == 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("WIFSTOPPED() did not return non-zero value");
		test_fail++;
	}
	else
		PATH_TRACE;

	if((val = WIFEXITED(status)) != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("WIFEXITED() returned non-zero value %d", val);
		test_fail++;
	}
	else
		PATH_TRACE;

	if((val = WIFSIGNALED(status)) != 0) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("WIFSIGNALED() returned non-zero value %d", val);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test16()
{
	DBUG_ENTER("test16");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}

	/*
	 * Test that waitpid(-1, stat_loc, 0) sets *stat_loc so that
	 * when WIFSTOPPED(*stat_loc) evaluates to non-zero value,
	 * WSTOPSIG(*stat_loc) evaluates to the signal that caused 
	 * the child process to stop.
	 */

	(void) cppair(ch16, NULLFN, WAITTIME*2, 0);

	DBUG_VOID_RETURN;
}

private void
ch16()
{
	int	pathok = setupok;

	DBUG_ENTER("ch16");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(chstop, pa16, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa16()
{
	int	status;
	int	val;
	pid_t	ret;
	int	pathok = 0;

	DBUG_ENTER("pa16");

	/* Sleep for long enough for child to give status information */
	(void) sleep(WAITTIME/2);

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)
	ret = waitpid(-1, (int *)&status, WNOHANG|WUNTRACED);
	CLEAR_ALARM

	if(ret != tet_child) {
		xx_rpt(DELETION);
		in_rpt("waitpid() did not return child process ID"); 
		in_rpt("instead returned %ld", (long)ret);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(WIFSTOPPED(status) == 0) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - WIFSTOPPED() evaluated to zero");
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if((val = WSTOPSIG(status)) != SIGSTOP) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("WSTOPSIG() did not evaluate to %d", SIGSTOP);
		in_rpt("instead WSTOPSIG() evaluated to %d", val);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(3);
	DBUG_VOID_RETURN;
}

private void
test17()
{
	DBUG_ENTER("test17");

	/* If POSIX_JOB_CONTROL not defined, untested */
	if(!job_control) {
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN;
	}

	/*
	 * Test that waitpid() without WUNTRACED and WNOHANG 
	 * does not return when there are stopped 
	 * child processes that have not yet been waited for.
	 */

	(void) cppair(ch17, NULLFN, (2 * WAITTIME), 0);

	DBUG_VOID_RETURN;
}

private void
ch17()
{
	int	pathok = setupok;

	DBUG_ENTER("ch17");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(chstop, pa17, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa17()
{
	int	status;
	int	err;
	pid_t	ret;
	char	*str = "waitpid(-1, *stat_loc, 0)";
	int	pathok = 0;

	DBUG_ENTER("pa17");

	/* Sleep for long enough for child to give status information */
	(void) sleep(WAITTIME/2);

	/* Set a timeout and expect it to interrupt the wait() */
	SET_TIMEOUT(WAITTIME/2)

	errno = NOERROR;
	ret = waitpid(-1, (int *)&status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret != SYSERROR || err != EINTR) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s returned when child process stopped", str);
		in_rpt("WIFEXITED() evaluates to %d; WIFSIGNALED() evaluates to %d",
					WIFEXITED(status), WIFSIGNALED(status));
		valrpt(SYSERROR, (int)ret, EINTR, err);
		test_fail++;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
test18()
{
	DBUG_ENTER("test18");

	/*
	 * Test that waitpid() returns -1 and sets errno to ECHILD
	 * when the process and the process group specified by pid do 
	 * not exist.
	 */

	(void) cppair(ch18, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch18()
{
	int	pathok = setupok;

	DBUG_ENTER("ch18");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(chexit, pa18p1, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa18p1()
{
	int	pathok = 0;

	DBUG_ENTER("pa18p1");

	global_pid = tet_child;

	globok = 0;
        if(cpsig(chexit, pa18p2, WAITTIME, 0) == SYSERROR)
	{
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	PATH_FUNC_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa18p2()
{
	int	status;
	int	err;
	pid_t	ret;
	pid_t	pid;
	pid_t	pid2;
	char	*str = "waitpid()";
	int	pathok = 0;

	DBUG_ENTER("pa18p2");

	pid = global_pid;

	pid2 = tet_child;

	DBUG_PRINTF("test18", "pid,  pid2 %d %d", pid,  pid2);

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)

	/* THIS IS NOT THE CALL THAT IS TESTED IN TEST18 */
	errno = NOERROR;
	ret = waitpid(pid, (int *)&status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret == SYSERROR) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - %s did not return child process ID", str); 
		in_rpt("instead returned %ld and set errno %d", (long)ret, err);
		test_deleted++;
		DBUG_VOID_RETURN;
	} else if(ret != pid) {
		xx_rpt(DELETION);
		in_rpt("deletion reason - %s did not return child process ID", str); 
		in_rpt("instead returned %ld ; child process ID %ld",
			(long)ret, (long)pid);
		test_deleted++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)

	errno = NOERROR;
	ret = waitpid(pid, (int *)&status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret != SYSERROR || err != ECHILD) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not return expected value for non-existant process", str); 
		valrpt(SYSERROR, (int)ret, ECHILD, err);
		test_fail++;
	}
	else
		PATH_TRACE;

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)

	errno = NOERROR;
	ret = waitpid(-pid, (int *)&status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret != SYSERROR || err != ECHILD) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not return expected value for non-existant process group", str); 
		valrpt(SYSERROR, (int)ret, ECHILD, err);
		test_fail++;
	}
	else
		PATH_TRACE;

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)

	errno = NOERROR;
	ret = waitpid(pid2, (int *)&status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld and set errno %d", (long)ret, err);
		test_fail++;
		DBUG_VOID_RETURN;
	} else if(ret != pid2) {
		xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld ; child process ID %ld",
			(long)ret, (long)pid2);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(4);

	DBUG_VOID_RETURN;
}

private void
test19()
{
	DBUG_ENTER("test19");

	/*
	 * Test that waitpid() returns -1 and sets errno to ECHILD
	 * when the process waited for is not a child of the calling process.
	 */

	(void) cppair(ch19, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch19()
{
	int	pathok = setupok;

	DBUG_ENTER("ch19");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(chexit, pa19, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa19()
{
	int	status;
        pid_t	chld;
	int	err;
	pid_t	ret;
	pid_t	pid;
	char	*str = "waitpid()";
	int	pathok = 0;

	DBUG_ENTER("pa19");

	pid = getpid();

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)

	errno = NOERROR;
	ret = waitpid(pid, (int *)&status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret != SYSERROR || err != ECHILD) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not return expected value waiting for non-child process", str); 
		valrpt(SYSERROR, (int)ret, ECHILD, err);
		test_fail++;
	}
	else
		PATH_TRACE;

	/*
	 * Now reap the child, ensuring sufficient time.
	 */

	chld = tet_child;
	SET_TIMEOUT(2*WAITTIME)

	errno = NOERROR;
	ret = waitpid(chld, &status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld and set errno %d", (long)ret, err);
		test_fail++;
		DBUG_VOID_RETURN;
	} else if(ret != chld) {
		xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld ; child process ID %ld",
			(long)ret, (long)chld);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test20()
{
	DBUG_ENTER("test20");

	/*
	 * Test that waitpid() returns -1 and sets errno to EINTR
	 * when waitpid() is interrupted by a signal.
	 */

	(void) cppair(ch20, NULLFN, 2*WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch20()
{
	int	pathok = setupok;

	DBUG_ENTER("ch20");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(chlongsleep, pa20, 2*WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa20()
{
	int	status;
	int	err;
	pid_t	ret;
	pid_t	pid;
	char	*str = "waitpid()";
	int	pathok = 0;

	DBUG_ENTER("pa20");

	pid = tet_child;

	/* Set a timeout for less than child's execution time */
	SET_TIMEOUT(WAITTIME/2)

	errno = NOERROR;
	ret = waitpid(pid, (int *)&status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret != SYSERROR || err != EINTR) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s when interrupted did not return expected value", str); 
		valrpt(SYSERROR, (int)ret, EINTR, err);
		test_fail++;
	}
	else
		PATH_TRACE;

	/*
	 * Now reap the child, ensuring sufficient time.
	 */

	SET_TIMEOUT(WAITTIME)

	errno = NOERROR;
	ret = waitpid(pid, &status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld and set errno %d", (long)ret, err);
		test_fail++;
		DBUG_VOID_RETURN;
	} else if(ret != tet_child) {
		xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld ; child process ID %ld",
			(long)ret, (long)tet_child);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test21()
{
	DBUG_ENTER("test21");

	/*
	 * Test that waitpid() returns -1 and sets errno to EINVAL
	 * when the options argument is invalid.
	 */

	(void) cppair(ch21, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}

private void
ch21()
{
	int	pathok = setupok;

	DBUG_ENTER("ch21");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if(cpsig(chexit, pa21, WAITTIME, 0) == SYSERROR)
		test_deleted++;
	if(test_deleted)
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if(!test_fail)
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa21()
{
	int	status;
	int	err;
	pid_t	ret;
	pid_t	pid;
	char	*str = "waitpid()";
	int	pathok = 0;

	DBUG_ENTER("pa21");

	pid = tet_child;

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)

	errno = NOERROR;
	ret = waitpid(pid, (int *)&status, ~(WNOHANG|WUNTRACED));
	err = errno;

	CLEAR_ALARM

	if(ret != SYSERROR || err != EINVAL) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s with invalid options argument did not return expected value", str); 
		valrpt(SYSERROR, (int)ret, EINVAL, err);
		test_fail++;
	}
	else
		PATH_TRACE;

	/*
	 * Now reap the child, ensuring sufficient time.
	 */

	SET_TIMEOUT(WAITTIME)

	errno = NOERROR;
	ret = waitpid(pid, &status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld and set errno %d", (long)ret, err);
		test_fail++;
		DBUG_VOID_RETURN;
	} else if(ret != tet_child) {
		xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld ; child process ID %ld",
			(long)ret, (long)tet_child);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test22()
{
	DBUG_ENTER("test22");

	/*
	 * Test that waitpid() returns -1 and sets errno to ECHILD
	 * when the pid is equal to -1 and the calling process does
	 * not have any child processes.
	 */

	(void) cppair(ch22, NULLFN, WAITTIME, 0);

	DBUG_VOID_RETURN;
}


private void
ch22()
{
	int	status;
	int	err;
	pid_t	ret;
	pid_t	pid;
	char	*str = "waitpid()";
	int	pathok = 0;

	DBUG_ENTER("ch22");

	pid = -1;

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)

	errno = NOERROR;
	ret = waitpid(pid, (int *)&status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret != SYSERROR || err != ECHILD) {
		xx_rpt(FAILURE);
		in_rpt("%s with pid set to -1 did not return expected value", str); 
		valrpt(SYSERROR, (int)ret, EINVAL, err);
	}
	else
	{
		PATH_TRACE;
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}


private void
test23()
{
	DBUG_ENTER("test23");

	/*
	 * Test that waitpid() returns -1 and sets errno to ECHILD
	 * when the pid is equal to 0 and the calling process does
	 * not have any child processes in its process group.
	 */

	(void) cppair(t_23, NULLFN, (2 * WAITTIME), 0);

	DBUG_VOID_RETURN;
}

private	void
t_23()
{
	int	pathok = setupok;

	DBUG_ENTER("t_23");

	test_fail = 0;
	test_deleted = 0;

	globok = 0;
	if ( cpsig(chexit,pa23,WAITTIME,0) == SYSERROR)
		test_deleted++;
	if ( test_deleted )
	{
		DBUG_VOID_RETURN;
	}
	else
		PATH_FUNC_TRACE;

	if (! test_fail )
		PATH_XS_RPT(1);

	DBUG_VOID_RETURN;
}

private void
pa23()
{
	int	status;
	int	err;
	pid_t	ret;
        pid_t   pid;
	char	*str = "waitpid()";
	int	pathok = 0;

	DBUG_ENTER("pa23");

        pid = tet_child;

	(void)sleep(WAITTIME/2);

	/* Set a timeout in case the wait never returns */
	SET_TIMEOUT(WAITTIME/2)

	errno = NOERROR;
	ret = waitpid(0, (int *)&status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret != SYSERROR || err != ECHILD) {
		if(!test_fail)
			xx_rpt(FAILURE);
		in_rpt("%s did not return expected value waiting for no child in process group", str); 
		valrpt(SYSERROR, (int)ret, ECHILD, err);
		test_fail++;
	}
	else
		PATH_TRACE;

	/*
	 * Now reap the child, ensuring sufficient time.
	 */

	SET_TIMEOUT(WAITTIME)

	errno = NOERROR;
	ret = waitpid(pid, &status, 0);
	err = errno;

	CLEAR_ALARM

	if(ret == SYSERROR) {
		xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld and set errno %d", (long)ret, err);
		test_fail++;
		DBUG_VOID_RETURN;
	} else if(ret != tet_child) {
		xx_rpt(FAILURE);
		in_rpt("%s did not return child process ID", str); 
		in_rpt("instead returned %ld ; child process ID %ld",
			(long)ret, (long)tet_child);
		test_fail++;
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_FUNC_RPT(2);

	DBUG_VOID_RETURN;
}


private void
test24()
{
	DBUG_ENTER("test24");

	/*
	 * Test that waitpid() returns -1 and sets errno to ECHILD
	 * when the pid is < -1  and the parent does not have any
	 * child processes in abs(pid)'s process group.
	 */

	(void) cppair(t_24, NULLFN, (2 * WAITTIME), 0);

	DBUG_VOID_RETURN;
}

private void
t_24()
{
	int status;
	pid_t chld;
	pid_t ret;
	int err;
	char *str = "waitpid";
	int	pathok = setupok;

	DBUG_ENTER("t_24");

	switch( chld = fork() ) {

		case SYSERROR:
			xx_rpt(DELETION);
			in_rpt("deletion reason - fork() failed");
			DBUG_VOID_RETURN;

		case 0: /* child */
			/* child stays in same process group as parent */
			tet_setcontext();
			(void) sleep(WAITTIME);
			DBUG_EXIT(OK);

		default: /* parent */
			tet_setblock();
			SET_TIMEOUT(WAITTIME) 

			err = NOERROR;
			/* wait for process in child's process group */
			ret = waitpid(-(chld),&status,0);
			err = errno;

			CLEAR_ALARM

			if ( ret != SYSERROR || err != ECHILD ) {
				xx_rpt(FAILURE);
				in_rpt("%s did not return expected value for non-existent child in process group",str);
				valrpt(SYSERROR,(int)ret,ECHILD,err);

			}
			else
			{
				PATH_TRACE;
			}

			/*
			 * Now reap the child, ensuring sufficient time.
			 */

			SET_TIMEOUT(2*WAITTIME)

			errno = NOERROR;
			ret = waitpid(chld, &status, 0);
			err = errno;

			CLEAR_ALARM

			if(ret == SYSERROR) {
				xx_rpt(FAILURE);
				in_rpt("%s did not return child process ID", str); 
				in_rpt("instead returned %ld and set errno %d", (long)ret, err);
				test_fail++;
				DBUG_VOID_RETURN;
			} else if(ret != chld) {
				xx_rpt(FAILURE);
				in_rpt("%s did not return child process ID", str); 
				in_rpt("instead returned %ld ; child process ID %ld",
					(long)ret, (long)chld);
				test_fail++;
				DBUG_VOID_RETURN;
			}
			else
				PATH_TRACE;

			PATH_XS_RPT(2);
		}

		tet_setblock();
		DBUG_VOID_RETURN;
}
