/*
*      SCCS:  @(#)  POSIX.os/sysdb/getgrgid/getgrgid.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/sysdb/getgrgid - T.getgrgid Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.getgrgid

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the getgrgid() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Mon Feb 13 18:15:49 GMT 1989
MODIFICATIONS:	
		Stuart Boutell, 12 Nov 1990
			uids() return value check added

		Geoff Clare, 9 Feb 1995
			Allow VSX_UNUSED_GID=unsup

************************************************************************/
#include	<std.h>
#include	<string.h>
#include	<errno.h>
#include	<limits.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<grp.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getgrgid
#endif


#define NO_TESTS		2		/* number of tests in testset */

extern	int		uids();
extern	char		*errname();

extern	int		vggid0;


int	no_tests = NO_TESTS;

private	struct	group	*tb_getgrgid();
private void	test1();
private void	test2();
private	void	setup();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private	void
setup()
{
	DBUG_ENTER("setup");
	if(uids()!=0)
	{
		int	i;
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
	}

	DBUG_VOID_RETURN;
}


private void
test1()
{
	struct group	*rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	errno = 0;
	if((rval = tb_getgrgid((gid_t) vggid0)) == (struct group *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("getgrgid has failed %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(rval->gr_gid != (gid_t) vggid0)
	{
		xx_rpt(FAILURE);
		in_rpt("getgrgid has returned the incorrect group - expected %d, got %ld",
			vggid0, (long)rval->gr_gid);
		in_rpt("name of group found %s",rval->gr_name);
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private void
test2()
{
	char	*cp;
	gid_t	unusedgid;
	struct group	*rval;
	int	pathok = 0;

	DBUG_ENTER("test2");

	if ((cp = tet_getvar("VSX_UNUSED_GID")) == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_UNUSED_GID is not set");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(cp, "unsup", (size_t)5) == 0)
	{
		/* All unused group ID's are invalid, so use an invalid ID */

		if ((cp = tet_getvar("VSX_INVALID_GID")) == NULL || *cp == '\0')
		{
			xx_rpt(DELETION);
			in_rpt("parameter VSX_INVALID_GID is not set");
			DBUG_VOID_RETURN;
		}
		else if (strncmp(cp, "unsup", (size_t)5) == 0)
		{
			/* Contradictictory parameter settings */
			xx_rpt(DELETION);
			in_rpt("parameters VSX_UNUSED_GID and VSX_INVALID_GID cannot both be \"unsup\"");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	unusedgid = atol(cp);

	errno = 0;
	if((rval = tb_getgrgid(unusedgid)) != (struct group *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("getgrgid(%ld) did not return NULL as expected",
			(long)unusedgid);
		in_rpt("ID of group found %ld", (long)rval->gr_gid);
		in_rpt("name of group found %s",rval->gr_name);
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private	struct	group *
tb_getgrgid(gid)
gid_t	gid;
{
	struct	group	*rval;

	errno = 0;
	rval = getgrgid(gid);
#ifdef	TESTBED
	errno++;
	rval = -1;
#endif

	return(rval);
}
