/*
*      SCCS:  @(#)  POSIX.os/sysdb/getpwnam/getpwnam.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/sysdb/getpwnam - T.getpwnam Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.getpwnam

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the getpwnam() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Mon Feb 13 18:15:49 GMT 1989
MODIFICATIONS:	
		Geoff Clare, 2 Nov 1990
			Change erroneous malloc()+strlen() to newstr().

		Stuart Boutell, 12 Nov 1990
			uids() return value check added

************************************************************************/
#include	<std.h>
#include	<errno.h>
#include	<limits.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<pwd.h>
#include	<stdlib.h>
#include	<string.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getpwnam
#endif


#define NO_TESTS		2	/* number of tests in testset */
#define	INVALID_NAME	"nouser"	/* default invalid passwd name */

extern	int		uids();
extern	char		*errname();
extern	char		*newstr();

extern	int		vguid0;


int	no_tests = NO_TESTS;

private	struct	passwd	*tb_getpwuid();
private	struct	passwd	*tb_getpwnam();
private void	test1();
private void	test2();
private	void	setup();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private	void
setup()
{
	DBUG_ENTER("setup");
	if(uids()!=0)
	{
		int i;
		for(i=1; i <= no_tests; i++)
		{
			(void) tet_delete(i, "uids() failed");
		}
	}

	DBUG_VOID_RETURN;
}


private void
test1()
{
	struct passwd	*rval;
	char	*name;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	errno = 0;
	if((rval = tb_getpwuid((uid_t) vguid0)) == (struct passwd *) NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("getpwuid has failed %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(rval->pw_uid != (uid_t) vguid0)
	{
		xx_rpt(DELETION);
		in_rpt("getpwuid has returned the incorrect user - expected %d, got %d",(uid_t)vguid0,rval->pw_uid);
		in_rpt("name of user found %s",rval->pw_name);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if ((name = newstr(rval->pw_name)) == NULL)
	{
		err = errno;
		xx_rpt(DELETION);
		in_rpt("malloc has failed %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	errno = 0;
	if((rval = tb_getpwnam(name)) == (struct passwd *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("getpwnam has failed %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(strcmp(name , rval->pw_name) != 0)
	{
		xx_rpt(FAILURE);
		in_rpt("getpwnam has returned the wrong name");
		in_rpt("expected \"%s\" but got \"%s\" ",name,rval->pw_name);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(5);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	char	*cp;
	char	*invalid_name;
	struct passwd	*rval;
	int	pathok = 0;

	DBUG_ENTER("test2");

	if ((cp = tet_getvar("VSX_INVALID_PNAME")) != (char *)NULL)
		invalid_name = cp;
	else
		invalid_name = INVALID_NAME;

	errno = 0;
	if((rval = tb_getpwnam(invalid_name)) != (struct passwd *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("getpwuid has found a non-existent user");
		in_rpt("name of user found is %s",rval->pw_name);
		in_rpt("ID of user found is %d", (int)rval->pw_uid);
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_TRACE;
		xs_rpt();
	}

	DBUG_VOID_RETURN;
}


private	struct	passwd *
tb_getpwuid(gid)
uid_t	gid;
{
	struct	passwd	*rval;

	errno = 0;
	rval = getpwuid(gid);
#ifdef	TESTBED
	errno++;
	rval = -1;
#endif

	return(rval);
}


private	struct	passwd *
tb_getpwnam(p)
char	*p;
{
	struct	passwd	*rval;

	errno = 0;
	rval = getpwnam(p);
#ifdef	TESTBED
	errno++;
	rval = -1;
#endif

	return(rval);
}

