/*
*      SCCS:  @(#)  POSIX.os/sysdb/getpwuid/getpwuid.c	Rel 4.4.1     (06/09/97) 
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

#ifndef lint
#define MAIN
#ifdef	MAIN
static	char sccsid[] = "@(#)POSIX.os/sysdb/getpwuid - T.getpwuid Rel 4.4.1 (06/09/97)";
char	*copyright[] = {
		"(C) Copyright 1991 X/Open Company Limited",
		"All Rights Reserved."
};
#endif	/* MAIN	*/
#endif  /* lint	*/

/***********************************************************************

NAME:		T.getpwuid

PROJECT:	VSX (X/OPEN Validation Suite)

DESCRIPTION:
	Testset for the getpwuid() system call.

METHODS:
ASSUMES:
CONDITIONAL COMPILE PARAMS:
SIDE EFFECTS:
RELATED DOCUMENTS:
SOURCE:

AUTHOR:		Joseph Nave, UniSoft Ltd.
DATE CREATED:	Mon Feb 13 18:15:49 GMT 1989
MODIFICATIONS:	
		Stuart Boutell, 12 Nov 1990
			uids() return value check added
		
		Geoff Clare, 9 Feb 1995
			Allow VSX_UNUSED_UID=unsup

************************************************************************/
#include	<std.h>
#include	<string.h>
#include	<errno.h>
#include	<limits.h>
#include	<dbug.h>
#include	<report.h>
#include	<sys/types.h>
#include	<pwd.h>
#include	<stdlib.h>
#include <tet_api.h>
#include	<sysdep.h>

#ifdef UNDEF_MACROS
#undef getpwuid
#endif


#define NO_TESTS		2		/* number of tests in testset */

extern	int		uids();
extern	char		*errname();

extern	int		vguid0;


int	no_tests = NO_TESTS;

private	struct	passwd	*tb_getpwuid();
private void	test1();
private void	test2();
private	void	setup();

public	void	(*setfunc) () = setup;
public	void	(*clnfunc) () = NULL;
public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	NULL, 0
};

private	void
setup()
{
	DBUG_ENTER("setup");
	if(uids()!=0)
	{
		int i;
		for(i=1; i <= no_tests; i++)
		{
			tet_delete(i, "uids() failed");
		}
	}

	DBUG_VOID_RETURN;
}


private void
test1()
{
	struct passwd	*rval;
	int	err;
	int	pathok = 0;

	DBUG_ENTER("test1");

	errno = 0;
	if((rval = tb_getpwuid((uid_t) vguid0)) == (struct passwd *) NULL)
	{
		err = errno;
		xx_rpt(FAILURE);
		in_rpt("getpwuid has failed %d(%s)",err,errname(err));
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	if(rval->pw_uid != (uid_t) vguid0)
	{
		xx_rpt(FAILURE);
		in_rpt("getpwuid has returned the incorrect user - expected %d, got %ld",
			vguid0, (long)rval->pw_uid);
		in_rpt("name of user found %s",rval->pw_name);
		DBUG_VOID_RETURN;
	}
	else
		PATH_TRACE;

	PATH_XS_RPT(2);

	DBUG_VOID_RETURN;
}

private void
test2()
{
	char	*cp;
	uid_t	unuseduid;
	struct passwd	*rval;
	int	pathok = 0;

	DBUG_ENTER("test2");

	if ((cp = tet_getvar("VSX_UNUSED_UID")) == NULL || *cp == '\0')
	{
		xx_rpt(DELETION);
		in_rpt("parameter VSX_UNUSED_UID is not set");
		DBUG_VOID_RETURN;
	}
	else if (strncmp(cp, "unsup", (size_t)5) == 0)
	{
		/* All unused user ID's are invalid, so use an invalid ID */

		if ((cp = tet_getvar("VSX_INVALID_UID")) == NULL || *cp == '\0')
		{
			xx_rpt(DELETION);
			in_rpt("parameter VSX_INVALID_UID is not set");
			DBUG_VOID_RETURN;
		}
		else if (strncmp(cp, "unsup", (size_t)5) == 0)
		{
			/* Contradictictory parameter settings */
			xx_rpt(DELETION);
			in_rpt("parameters VSX_UNUSED_UID and VSX_INVALID_UID cannot both be \"unsup\"");
			DBUG_VOID_RETURN;
		}
		else
			PATH_TRACE;
	}
	else
		PATH_TRACE;

	unuseduid = atol(cp);

	errno = 0;
	if((rval = tb_getpwuid(unuseduid)) != (struct passwd *) NULL)
	{
		xx_rpt(FAILURE);
		in_rpt("getpwuid(%ld) did not return NULL as expected",
			(long)unuseduid);
		in_rpt("ID of user found %ld", (long)rval->pw_uid);
		in_rpt("name of user found %s",rval->pw_name);
		DBUG_VOID_RETURN;
	}
	else
	{
		PATH_XS_RPT(1);
	}

	DBUG_VOID_RETURN;
}

private	struct	passwd *
tb_getpwuid(uid)
uid_t	uid;
{
	struct	passwd	*rval;

	errno = 0;
	rval = getpwuid(uid);
#ifdef	TESTBED
	errno++;
	rval = -1;
#endif

	return(rval);
}
