
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/asctime_r
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)asctime_r.c	1.14
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	2

#ifdef  UNDEF_MACROS
#undef  asctime_r
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );

void	*t1_t1( void *arg );
void	*t2_t1( void *arg );

#endif

void
startup()
{
	vsth_startup("asctime_r");
}

void
cleanup()
{
	vsth_cleanup("asctime_r");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("concurrent calls by multiple threads to asctime_r(tm, buf)");
	tet_infoline("convert the broken-down time in the structure referenced by tm into");
	tet_infoline("a string placed in the buffer referenced by buf and return buf");
	tet_infoline("if successful.");

	tet_infoline("Posix Ref: Component ASCTIME_R");
	tet_infoline("Assertion 9945-1:1996 8.3.4.2-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_NTHREADS	8

VSTH_TSET		T1_tset[T1_NTHREADS];

int			T1_flag;
pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
		return;

	T1_flag = T1_NTHREADS;

	vsth_weave( &T1_tset[0], T1_NTHREADS, t1_t1, &T1_mx, &T1_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

/* FROM VSX4.3.5:tset/ANSI.os/time/asctime */
static char	T1_fmt[] = "ccc ccc dd dd:dd:dd dddd";

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	int		i;
	char		*p;
	char		buf[26];
	time_t		whenever;
	struct tm	*ptm;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	memset( (void *) buf, 0, 64 );
	whenever	= 530454574L;
	ptm		= localtime( &whenever );

	rval = vsth_bwait( &T1_cv, &T1_mx, &T1_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	p = asctime_r( ptm, buf );
	if ( p != buf )
	{
		++nfails;
		tet_printf("asctime_r returned %p, expected %p", p, buf);
	}

	for ( i = 0; T1_fmt[i]; ++i )
	{
		switch(T1_fmt[i])
		{
		case 'c':
			if ( ! isalpha(buf[i]) )
				++nfails;
			break;
		case 'd':
			if ( ! isdigit(buf[i]) )
				++nfails;
			break;
		default:
			if ( buf[i] != T1_fmt[i] )
				++nfails;
			break;
		}
	}
	
	if ( nfails )
	{
		tet_printf("asctime_( localtime(530454574L), buf ) was %s", p);
		tet_printf("expected string of the form Sun Sep 16 01:03:52 1973");
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("a call to asctime_r(tm, buf) shall return NULL on failure.");

	tet_infoline("Posix Ref: Component ASCTIME_R");
	tet_infoline("Assertion 9945-1:1996 8.3.4.3-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	/* POSIX 1996, page 221, 8.3.4.4: no specified error conditions
	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );
	*/

	tet_infoline("No portable test");
	result = TET_UNTESTED;

#else

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T2_NTHREADS	8

int			T2_flag;
pthread_mutex_t		T2_mx;
pthread_cond_t		T2_cv;
VSTH_TSET		T2_tset[T2_NTHREADS];

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
		return;

	T2_flag = T2_NTHREADS;

	vsth_weave( &T2_tset[0], T2_NTHREADS, t2_t1, &T2_mx, &T2_cv,
				SCHED_RR, priomin, 0, errors, fails );
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	struct tm	tm;
	char		*p;
	char		buf[64];

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	tm.tm_hour	= -1;
	tm.tm_mday	= -1;
	tm.tm_min	= -1;
	tm.tm_sec	= -1;
	tm.tm_wday	= 99;
	tm.tm_yday	= 999;
	tm.tm_year	= -1;

	memset( (void *) buf, 0, 64 );

	rval = vsth_bwait( &T2_cv, &T2_mx, &T2_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	p = asctime_r( &tm, buf );
	if ( p != NULL )
	{
		++nfails;
		tet_printf("Expected asctime_r to return NULL, got %p", p);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	NULL,	0	}
};

