
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/ctime_r
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)ctime_r.c	1.15
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	2

#ifdef  UNDEF_MACROS
#undef  ctime_r
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();

void	test1();
void	test2();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );

void	*t1_t1( void *arg );
void	*t2_t1( void *arg );

#endif

void
startup()
{
	vsth_startup("ctime_r");
}

void
cleanup()
{
	vsth_cleanup("ctime_r");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("concurrent calls by multiple threads to ctime_r(clock, buf)");
	tet_infoline("convert the calendar time referenced by clock to a string");
	tet_infoline("representation of the corresponding local time, store the string");
	tet_infoline("in the location referenced by buf, and return buf if successful.");

	tet_infoline("Posix Ref: Component CTIME_R");
	tet_infoline("Assertion 9945-1:1996 8.3.5.2-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");
#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_NTHREADS	8

int			T1_flag;
pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;
time_t			T1_whenever;
char			T1_expected[64];
VSTH_TSET		T1_tset[T1_NTHREADS];

/* From VSX4.3.5:tset/ANSI.os/time/ctime */
void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;
	char		*p;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
		return;

	T1_flag		= T1_NTHREADS;
	T1_whenever	= 567869055L;

	p = asctime_r(localtime(&T1_whenever), T1_expected);
	if ( p != T1_expected )
	{
		++*errors;
		tet_infoline("asctime_r unexpectedly failed");
		return;
	}

	vsth_weave( &T1_tset[0], T1_NTHREADS, t1_t1, &T1_mx, &T1_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	int		i;
	char		*p;
	char		buf[64];

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T1_cv, &T1_mx, &T1_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
		return;
	}

	p = ctime_r( &T1_whenever, buf );
	if ( p )
	{
		if ( strcmp( p, T1_expected ) )
		{
			++nfails;
			tet_printf("ctime_r returned %s, expected %s", p, T1_expected);
		}
	}
	else
	{
		++nfails;
		tet_infoline("ctime_r unexpectedly failed");
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;
	time_t		whenever;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when an error is encountered, a NULL pointer is returned.");

	tet_infoline("Posix Ref: Component CTIME_R");
	tet_infoline("Assertion 9945-1:1996 8.3.5.3-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	/* POSIX 1996, page 222, 8.3.5.4: no error conditions defined 
	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );
	*/

	tet_infoline("No error conditions are defined");
	result = TET_UNTESTED;

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");
#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T2_NTHREADS	8

int			T2_flag;
pthread_mutex_t		T2_mx;
pthread_cond_t		T2_cv;
VSTH_TSET		T2_tset[T2_NTHREADS];

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;
	char		*p;

	T2_flag = T2_NTHREADS;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
		return;

	vsth_weave( &T2_tset[0], T2_NTHREADS, t2_t1, &T2_mx, &T2_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	char		*p;
	char		buf[64];
	time_t		clock;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;
	clock	= -1;

	rval = vsth_bwait( &T2_cv, &T2_mx, &T2_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	p = ctime_r( (const time_t *) &clock, buf );
	if ( p != NULL )
	{
		++nfails;
		tet_printf("ctime_r returned %s", p);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	NULL,	0	}
};


