
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/ftrylockfile
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)ftrylockfile.c	1.16
*
*	History
*		Changed pthread_detach to vsth_pthread_detach to allow
*		TET API to only reap residual threads.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	3

#ifdef  UNDEF_MACROS
#undef  ftrylockfile
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );

void	*t1_t1( void *arg );
void	*t2_t1( void *arg );
void	*t2_t2( void *arg );
void	*t3_t1( void *arg );

#endif

void
startup()
{
	vsth_startup("ftrylockfile");
}

void
cleanup()
{
	vsth_cleanup("ftrylockfile");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the file pointer referenced by the file argument is not locked");
	tet_infoline("and no thread is blocked waiting for a lock,");
	tet_infoline("then a call to ftrylockfile(file) acquires a lock on the file pointer,");
	tet_infoline("sets the lock count to 1 and returns 0 if successful.");

	tet_infoline("Posix Ref: Component FTRYLOCKFILE");
	tet_infoline("Assertion 9945-1:1996 8.2.6.2-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_FILENAME	"./t1_file"

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;
	FILE		*fp;

	fp = fopen( T1_FILENAME, "w" );
	if ( fp == NULL )
	{
		++*errors;
		tet_printf("fopen unexpectedly failed, errno == %d", errno);
		return;
	}

	rval = ftrylockfile( fp );
	if ( rval != 0 )
	{
		vsth_surprise("ftrylockfile", 0, rval, fails);
		(void) fclose(fp);
		(void) remove(T1_FILENAME);
		return;
	}

	rval = vsth_pthread_create( &tid, NULL, t1_t1, (void *) fp );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		funlockfile( fp );
		(void) fclose( fp );
		(void) remove( T1_FILENAME );
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	funlockfile( fp );

	(void) fclose( fp );
	(void) remove( T1_FILENAME );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	FILE		*fp;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;
	fp	= (FILE *) arg;

	/* we have no other way of determining whether we have a lock ... */
	rval = ftrylockfile( fp );
	if ( rval == 0 )
	{
		++nfails;
		tet_infoline("ftrylockfile unexpectedly succeeded");
		(void) funlockfile( fp );
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the file pointer referenced by the file argument is locked by");
	tet_infoline("the calling thread,");
	tet_infoline("then a call to ftrylockfile(file) increments the lock count on the");
	tet_infoline("file pointer and returns 0 if successful.");

	tet_infoline("Posix Ref: Component FTRYLOCKFILE");
	tet_infoline("Assertion 9945-1:1996 8.2.6.2-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T2_FILENAME	"./t2_file"

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;
	FILE		*fp;

	fp = fopen( T2_FILENAME, "w" );
	if ( fp == NULL )
	{
		tet_printf("fopen unexpectedly failed, errno == %d", errno);
		return;
	}

	rval = ftrylockfile( fp );
	if ( rval != 0 )
	{
		vsth_surprise("ftrylockfile", 0, rval, fails);
		(void) fclose(fp);
		(void) remove(T2_FILENAME);
		return;
	}

	rval = ftrylockfile( fp );
	if ( rval != 0 )
	{
		vsth_surprise("ftrylockfile", 0, rval, fails);
		(void) funlockfile( fp );
		(void) fclose(fp);
		(void) remove(T2_FILENAME);
		return;
	}

	rval = vsth_pthread_create( &tid, NULL, t2_t1, (void *) fp );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		funlockfile( fp );
		(void) fclose(fp);
		(void) remove(T2_FILENAME);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	funlockfile( fp );

	rval = vsth_pthread_create( &tid, NULL, t2_t1, (void *) fp );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		funlockfile( fp );
		(void) fclose(fp);
		(void) remove(T2_FILENAME);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	funlockfile( fp );

	rval = vsth_pthread_create( &tid, NULL, t2_t2, (void *) fp );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		funlockfile( fp );
		(void) fclose(fp);
		(void) remove(T2_FILENAME);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	(void) fclose( fp );
	(void) remove( T2_FILENAME );
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	FILE		*fp;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;
	fp	= (FILE *) arg;

	rval = ftrylockfile( fp );
	if ( rval == 0 )
	{
		++nfails;
		tet_infoline("ftrylockfile unexpectedly succeeded");
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void *
t2_t2( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	FILE		*fp;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;
	fp	= (FILE *) arg;

	rval = ftrylockfile( fp );
	if ( rval != 0 )
	{
		vsth_surprise("ftrylockfile", 0, rval, &nfails);
	}
	else
	{
		(void) funlockfile( fp );
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when a lock on the file pointer referenced by the file argument");
	tet_infoline("can not be obtained,");
	tet_infoline("then a call to ftrylockfile(file) returns non-zero.");

	tet_infoline("Posix Ref: Component FTRYLOCKFILE");
	tet_infoline("Assertion 9945-1:1996 8.2.6.2-3(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T3_FILENAME	"./t3_file"

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;
	FILE		*fp;

	fp = fopen( T3_FILENAME, "w" );
	if ( fp == NULL )
	{
		tet_printf("fopen unexpectedly failed, errno == %d", errno);
		return;
	}

	rval = ftrylockfile( fp );
	if ( rval != 0 )
	{
		vsth_surprise("ftrylockfile", 0, rval, fails);
		(void) fclose(fp);
		(void) remove(T3_FILENAME);
		return;
	}

	rval = vsth_pthread_create( &tid, NULL, t3_t1, (void *) fp );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		funlockfile( fp );
		(void) fclose(fp);
		(void) remove(T3_FILENAME);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	funlockfile( fp );
	(void) fclose( fp );
	(void) remove( T3_FILENAME );
}

void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	FILE		*fp;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;
	fp	= (FILE *) arg;

	rval = ftrylockfile( fp );
	if ( rval == 0 )
	{
		++nfails;
		tet_infoline("ftrylockfile unexpectedly succeeded");
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	NULL,	0	}
};

