
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/funlockfile
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)funlockfile.c	1.16
*
*	History
*		Changed pthread_detach to vsth_pthread_detach to allow
*		TET API to only reap residual threads.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	1

#ifdef  UNDEF_MACROS
#undef  funlockfile
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	*t1_t1( void *arg );
void	*t1_t2( void *arg );

#endif

void
startup()
{
	vsth_startup("funlockfile");
}

void
cleanup()
{
	vsth_cleanup("funlockfile");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the file pointer referenced by the file argument is locked by");
	tet_infoline("the calling thread,");
	tet_infoline("then a call to funlockfile(file) decrements the lock count on the");
	tet_infoline("file and releases the lock if the lock count becomes 0.");

	tet_infoline("Posix Ref: Component FUNLOCKFILE Assertion 9945-1:1996 8.2.6.2-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_FILENAME	"./t1_file"
#define	T1_TRIES	16

/* as in flockfile, test 3 */
void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		cnt;
	pthread_t	tid;
	void		*tval;
	FILE		*fp;

	fp = fopen( T1_FILENAME, "w" );
	if ( fp == NULL )
	{
		tet_printf("fopen unexpectedly failed, errno == %d", errno);
		return;
	}

	/* one extra lock */
	for ( cnt = 0; cnt <= T1_TRIES; ++cnt )
	{
		flockfile( fp );
	}

	for ( cnt = 0; cnt < T1_TRIES; ++cnt )
	{
		funlockfile( fp );
	}

	rval = vsth_pthread_create( &tid, NULL, t1_t1, (void *) fp );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		(void) fclose( fp );
		(void) remove( T1_FILENAME );
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
		else if ( tval )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}
	
	funlockfile( fp );

	rval = vsth_pthread_create( &tid, NULL, t1_t2, (void *) fp );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		(void) fclose( fp );
		(void) remove( T1_FILENAME );
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
		else if ( tval )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}
	
	(void) fclose( fp );
	(void) remove( T1_FILENAME );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	FILE		*fp;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;
	fp	= (FILE *) arg;

	rval = ftrylockfile( fp );
	if ( rval == 0 )
	{
		++nfails;
		tet_infoline("ftrylockfile unexpectedly succeeded");
		funlockfile( fp );
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void *
t1_t2( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	FILE		*fp;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;
	fp	= (FILE *) arg;

	rval = ftrylockfile( fp );
	if ( rval == 0 )
	{
		funlockfile( fp );
	}
	else
	{
		++nfails;
		tet_infoline("ftrylockfile unexpectedly failed");
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	NULL,	0	}
};

