
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/getc_unlocked
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)getc_unlocked.c	1.16
*
*	History
*		Changed pthread_detach to vsth_pthread_detach to allow
*		TET API to only reap residual threads.
*
*		Add calls to flockfile(), funlockfile() around the
*		calls to getc_unlocked(). TSD5TH.00069
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	3

#ifdef  UNDEF_MACROS
#undef  getc_unlocked
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );

void	*t1_t1( void *arg );
void	*t2_t1( void *arg );
void	*t3_t1( void *arg );

#endif

void
startup()
{
	vsth_startup("getc_unlocked");
}

void
cleanup()
{
	vsth_cleanup("getc_unlocked");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the input stream referenced by the stream argument is not at");
	tet_infoline("end-of-file,");
	tet_infoline("then a call to getc_unlocked(stream) returns the next character");
	tet_infoline("from the stream if successful.");

	tet_infoline("Posix Ref: Component GETC_UNLOCKED");
	tet_infoline("Assertion 9945-1:1996 8.2.7.2, ANSI/ISO 9899-1990 7.9.7.5-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_T_NTHREADS	8
#define	T1_FILENAME	"./t1_file"
#define	T1_STRING	"0123456789"

int			T1_flag;
pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;
VSTH_TSET		T1_tset[T1_T_NTHREADS];

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		fd;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
		return;

	fd = creat( T1_FILENAME, S_IRWXU | S_IRWXG );
	if ( fd < 0 )
	{
		++*errors;
		tet_printf("creat unexpectedly failed, errno == %d", errno);
		return;
	}

	rval = write( fd, T1_STRING, strlen(T1_STRING) );
	if ( rval != strlen(T1_STRING) )
	{
		++*errors;
		tet_printf("write unexpectedly failed, expected %d, got %d",
				strlen(T1_STRING), rval);
		(void) close(fd);
		(void) remove(T1_FILENAME);
		return;
	}

	rval = close(fd);
	if ( rval != 0 )
	{
		vsth_surprise("close", 0, rval, errors);
		(void) remove(T1_FILENAME);
		return;
	}

	T1_flag = T1_T_NTHREADS;

	vsth_weave( &T1_tset[0], T1_T_NTHREADS, t1_t1, &T1_mx, &T1_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	int		cnt;
	int		len;
	FILE		*fp;
	char		*buf;
	char		*p;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T1_cv, &T1_mx, &T1_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	fp = fopen( T1_FILENAME, "r" );
	if ( fp != 0 )
	{
		flockfile(fp); /* TSD5TH.00069*/
		len = strlen( T1_STRING );
		buf = (char *) malloc( (len + 1) * sizeof(char) );
		if ( buf != NULL )
		{
			memset(buf, 0, len + 1);
			for ( cnt = 0, p = buf; cnt < len; ++cnt, ++p )
			{
				*p = getc_unlocked( fp );
			}
			*p = '\0';

			if ( strcmp( T1_STRING, buf ))
			{
				++nfails;
				tet_printf("Expected %s, got %s", T1_STRING, buf);
			}

			(void) free(buf);
		}
		else
		{
			++nerrs;
			tet_printf("malloc unexpectedly failed, errno == %d", errno);
		}

		funlockfile(fp); /* TSD5TH.00069 */
		(void) fclose(fp);
	}
	else
	{
		++nerrs;
		tet_printf("fopen unexpectedly failed, errno == %d", errno);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the input stream referenced by the stream argument is at");
	tet_infoline("end-of_file, then a call to getc_unlocked(stream) sets the end-of-file");
	tet_infoline("indicator for the stream and returns EOF.");

	tet_infoline("Posix Ref: Component GETC_UNLOCKED");
	tet_infoline("Assertion 9945-1:1996 8.2.7.2, ANSI/ISO 9899-1990 7.9.7.5-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T2_FILENAME	"./t2_file"
#define	T2_T_NTHREADS	8

int			T2_flag;
pthread_mutex_t		T2_mx;
pthread_cond_t		T2_cv;
VSTH_TSET		T2_tset[T2_T_NTHREADS];

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	int		fd;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors );
	if ( *errors )
		return;

	fd = creat( T2_FILENAME, S_IRWXU | S_IRWXG );
	if ( fd < 0 )
	{
		++*errors;
		tet_printf("open unexpectedly failed, errno == %d", errno);
		return;
	}

	rval = close(fd);
	if ( rval != 0 )
	{
		vsth_surprise("close", 0, rval, errors);
		(void) remove(T2_FILENAME);
		return;
	}

	T2_flag = T2_T_NTHREADS;

	vsth_weave( &T2_tset[0], T2_T_NTHREADS, t2_t1, &T2_mx, &T2_cv,
			SCHED_RR, priomin, 0, errors, fails );

	(void) remove(T2_FILENAME);
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	FILE		*fp;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T2_cv, &T2_mx, &T2_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	fp = fopen(T2_FILENAME, "r");
	if ( fp != NULL )
	{
		flockfile(fp);
		rval = getc_unlocked( fp );
		if ( rval != EOF )
		{
			++nfails;
			vsth_surprise("getc_unlocked", EOF, rval, &nfails);
		}

		rval = feof(fp);
		if ( rval == 0 )
		{
			vsth_surprise("feof", 1, rval, &nfails);
		}

		funlockfile(fp);
		(void) fclose(fp);
	}
	else
	{
		++nerrs;
		tet_printf("fopen unexpectedly failed, errno == %d", errno);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when a read error occurs,");
	tet_infoline("then a call to getc_unlocked(stream) sets the error indicator for the");
	tet_infoline("stream and returns EOF.");

	tet_infoline("Posix Ref: Component GETC_UNLOCKED");
	tet_infoline("Assertion 9945-1:1996 8.2.7.2, ANSI/ISO 9899-1990 7.9.7.5-3(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

int			T3_fd[2];
FILE			*T3_fp;

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = pipe( T3_fd );
	if ( rval != 0 )
	{
		vsth_surprise("pipe", 0, rval, errors);
		return;
	}

	rval = fcntl( T3_fd[0], F_SETFL, O_NONBLOCK );
	if ( rval != 0 )
	{
		vsth_surprise("fcntl", 0, rval, errors);
		(void) close(T3_fd[1]);
		(void) close(T3_fd[0]);
		return;
	}

	T3_fp = fdopen( T3_fd[0], "r" );
	if ( T3_fp == NULL )
	{
		vsth_surprise("fdopen", 0, (int) T3_fp, errors);
		(void) close(T3_fd[1]);
		(void) close(T3_fd[0]);
		return;
	}

	rval = vsth_pthread_create( &tid, NULL, t3_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		(void) close(T3_fd[1]);
		(void) close(T3_fd[0]);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	(void) fclose(T3_fp);
	(void) close(T3_fd[1]);
	(void) close(T3_fd[0]);
}

void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	FILE		*fp;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;


	flockfile(T3_fp);
	rval = getc_unlocked( T3_fp );
	if ( rval != EOF )
	{
		++nfails;
		tet_printf("Expected EOF, got %d", rval);
	}

	if ( ! ferror(T3_fp) )
	{
		++nfails;
		tet_infoline("The error indicator for the stream is not set");
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	funlockfile(T3_fp);
	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	NULL,	0	}
};


