
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/getchar_unlocked
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)getchar_unlocked.c	1.16
*
*	History:
*	Moved tests 1 and 2 to a separate executable
*
*	Changed pthread_detach to vsth_pthread_detach to allow
*	TET API to only reap residual threads.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	3

#ifdef  UNDEF_MACROS
#undef  getchar_unlocked
#endif

extern char	**environ;

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );

void	t1_c1();
void	t2_c1();
void	*t3_t1(void *arg);

#endif

void
startup()
{
	vsth_startup("getchar_unlocked");
}

void
cleanup()
{
	vsth_cleanup("getchar_unlocked");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the stdin stream is not at end-of-file,");
	tet_infoline("then a call to getchar_unlocked() returns the next character from");
	tet_infoline("stdin if successful.");

	tet_infoline("Posix Ref: Component GETCHAR_UNLOCKED");
	tet_infoline("Assertion 9945-1:1996 8.2.7.2, ANSI/ISO 9899-1990 7.9.7.6-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	TE_PATH		"getchar_unlocked_exectests"
void
t1_f1( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork( t1_c1, NULL, VSTH_TIMEOUT, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
}

void
t1_c1()
{
	char		*argv[2];

	argv[0] = execpath(TE_PATH);
	argv[1] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the stdin stream is at end-of-file,");
	tet_infoline("then a call to getchar_unlocked() sets the end-of-file indicator for");
	tet_infoline("stdin and returns EOF.");

	tet_infoline("Posix Ref: Component GETCHAR_UNLOCKED");
	tet_infoline("Assertion 9945-1:1996 8.2.7.2, ANSI/ISO 9899-1990 7.9.7.6-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void
t2_f1( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork( t2_c1, NULL, VSTH_TIMEOUT, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
}

void
t2_c1()
{
	char		*argv[2];

	argv[0] = execpath(TE_PATH);
	argv[1] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

#endif

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when a read error occurs,");
	tet_infoline("then a call to getchar_unlocked() sets the error indicator for stdin");
	tet_infoline("and returns EOF.");

	tet_infoline("Posix Ref: Component GETCHAR_UNLOCKED");
	tet_infoline("Assertion 9945-1:1996 8.2.7.2, ANSI/ISO 9899-1990 7.9.7.6-3(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

int			T3_fd[2];

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	(void) close(STDIN_FILENO);

	rval = pipe( T3_fd );
	if ( rval != 0 )
	{
		vsth_surprise("pipe", 0, rval, errors);
		return;
	}

	rval = fcntl( T3_fd[0], F_SETFL, O_NONBLOCK );
	if ( rval != 0 )
	{
		vsth_surprise("fcntl", 0, rval, errors);
		(void) close(T3_fd[1]);
		(void) close(T3_fd[0]);
		return;
	}

	rval = vsth_pthread_create( &tid, NULL, t3_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		(void) close(T3_fd[1]);
		(void) close(T3_fd[0]);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	(void) close(T3_fd[1]);
	(void) close(T3_fd[0]);
}

void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	FILE		*fp;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;


	rval = getchar_unlocked();
	if ( rval != EOF )
	{
		++nfails;
		tet_printf("Expected EOF, got %d", rval);
	}

	if ( ! ferror(stdin) )
	{
		++nfails;
		tet_infoline("The error indicator for the stream is not set");
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	NULL,	0	}
};

