
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/getchar_unlocked_exectests
*
*	The Open Group, VSTH Threads test suite
*	@(#)getchar_unlocked_exectests.c	1.2
*
*	History:
*	Created separate executable to avoid calls to non-asynch safe functions.
*
*	Changed pthread_detach to vsth_pthread_detach to allow
*	TET API to only reap residual threads.
*
*	Add calls to flockfile(), funlockfile() around
*	getchar_unlocked() , TSD5TH.00069
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#ifdef  UNDEF_MACROS
#undef  getchar_unlocked
#endif

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

int	tet_nosigreset = 0;

void	t1_c1();
void	*t1_t1( void *arg );
void	t2_c1();
void	*t2_t1( void *arg );
#endif

int
tet_main( int argc, char **argv)
{

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	if (tet_thistest == 1)
		t1_c1();

	if (tet_thistest == 2)
		t2_c1();
#endif

	exit(1);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_FILENAME	"./t1_file"

/* quote from "Lord of Chaos" by Robert Jordan */
#define	T1_STRING	"The wheel weaves as the wheel will."

void
t1_c1()
{
	int		rval;
	int		fd;
	pthread_t	tid;
	int		nerrs;
	int		nfails;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;

	fd = creat( T1_FILENAME, S_IRWXU | S_IRWXG );
	if ( fd >= 0 )
	{
		rval = write( fd, T1_STRING, strlen(T1_STRING) );
		if ( rval == strlen(T1_STRING) )
		{
			rval = close(fd);
			if ( rval == 0 )
			{
				rval = vsth_pthread_create( &tid, NULL, t1_t1, NULL );
				if ( rval == 0 )
				{
					rval = vsth_pthread_join( tid, &tval );
					if ( rval == 0 )
					{
						if ( tval == (void *) 1 )
						{
							vsth_surprise("Thread exitval", 0, (int) tval, &nerrs);
						}
						else if ( tval == (void *) 2 )
						{
							vsth_surprise("Thread exitval", 0, (int) tval, &nerrs);
						}
					}
					else
					{
						(void) pthread_cancel( tid );
						(void) vsth_pthread_detach( tid );
						vsth_surprise("vsth_pthread_join", 0, rval, &nerrs);
					}
				}
				else
				{
					vsth_surprise("vsth_pthread_create", 0, rval, &nerrs);
				}
			}
			else
			{
				vsth_surprise("close", 0, rval, &nerrs);
			}
		}
		else
		{
			vsth_surprise("write", strlen(T1_STRING), rval, &nerrs);
		}
	}
	else
	{
		vsth_surprise("creat", 0, rval, &nerrs);
	}

	(void) remove(T1_FILENAME);

	rval = 1;
	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;
	exit(rval);
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	int		cnt;
	int		len;
	FILE		*fp;
	char		*buf;
	char		*p;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	fp = freopen( T1_FILENAME, "r", stdin );
	if ( fp != NULL )
	{
		flockfile(fp); /* TSD5TH.00069 */
		len = strlen( T1_STRING );
		buf = (char *) malloc( (len + 1) * sizeof(char) );
		if ( buf != NULL )
		{
			memset( (void *) buf, 0, len + 1 );

			p = buf;
			for ( cnt = 0; cnt < len; ++cnt )
			{
				*p++ = getchar_unlocked();
			}

			if ( strcmp( T1_STRING, buf ))
			{
				++nfails;
				tet_printf("Expected %s, got %s", T1_STRING, buf);
			}

			(void) free(buf);
		}
		else
		{
			++nerrs;
			tet_printf("malloc unexpectedly failed, errno == %d", errno);
		}

		funlockfile(fp); /* TSD5TH.00069 */
		(void) fclose(fp);
	}
	else
	{
		++nerrs;
		tet_printf("freopen unexpectedly failed, errno == %d", errno);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#define	T2_FILENAME	"./t2_file"
#define	T2_T_NTHREADS	8

int			T2_flag;
FILE			*T2_fp;
pthread_mutex_t		T2_mx;
pthread_cond_t		T2_cv;
VSTH_TSET		T2_tset[T2_T_NTHREADS];

void
t2_c1()
{
	int		rval;
	int		nerrs;
	int		nfails;
	int		fd;
	int		priomin;

	nerrs	= 0;
	nfails	= 0;

	vsth_getprio( SCHED_RR, &priomin, NULL, &nerrs );
	if ( nerrs )
		return;

	/* create an empty file */
	fd = creat( T2_FILENAME, S_IRWXU | S_IRWXG );
	if ( fd < 0 )
	{
		tet_printf("open unexpectedly failed, errno == %d", errno);
		exit(1);
	}

	rval = close(fd);
	if ( rval != 0 )
	{
		tet_printf("close unexpectedly failed, errno == %d", errno);
		(void) remove(T2_FILENAME);
		exit(1);
	}

	T2_fp = freopen( T2_FILENAME, "r", stdin );
	if ( T2_fp == NULL )
	{
		tet_printf("freopen unexpectedly failed, errno == %d", errno);
		(void) remove(T2_FILENAME);
		exit(1);
	}


	T2_flag = T2_T_NTHREADS;

	vsth_weave( &T2_tset[0], T2_T_NTHREADS, t2_t1, &T2_mx, &T2_cv,
			SCHED_RR, priomin, 0, &nerrs, &nfails );

	(void) fclose(T2_fp);
	(void) remove(T2_FILENAME);

	rval = 1;

	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;

	exit(rval);
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	FILE		*fp;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T2_cv, &T2_mx, &T2_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	flockfile(stdin); /* TSD5TH.00069*/

	rval = getchar_unlocked();
	if ( rval != EOF )
	{
		++nfails;
		vsth_surprise("getc_unlocked", EOF, rval, &nfails);
	}

	rval = feof(stdin);
	if ( rval == 0 )
	{
		++nerrs;
		tet_infoline("The end of file indicator for stream was not set");
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	funlockfile(stdin); /* TSD5TH.00069*/
	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif
