
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/gmtime_r
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)gmtime_r.c	1.14
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<limits.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	2

#ifdef  UNDEF_MACROS
#undef  gmtime_r
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );

void	*t1_t1( void *arg );
void	*t2_t1( void *arg );

#endif

void
startup()
{
	vsth_startup("gmtime_r");
}

void
cleanup()
{
	vsth_cleanup("gmtime_r");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("concurrent calls by multiple threads to gmtime_r(clock, result)");
	tet_infoline("convert the calendar time referenced by clock into a broken-down");
	tet_infoline("time expressed as UTC, store this time in the structure");
	tet_infoline("referenced by result and return result if successful.");

	tet_infoline("Posix Ref: Component GMTIME_R");
	tet_infoline("Assertion 9945-1:1996 8.3.6.2-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_NTHREADS	8

int			T1_flag;
pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;
time_t			T1_ticks;
VSTH_TSET		T1_tset[T1_NTHREADS];

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
		return;

	T1_flag		= T1_NTHREADS;
	T1_ticks	= 530454574L;

	vsth_weave( &T1_tset[0], T1_NTHREADS, t1_t1, &T1_mx, &T1_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	struct tm	*ptm;
	struct tm	tm;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	ptm = gmtime_r( &T1_ticks, &tm );

	/* expected values from VSX4.3.5 tset/ANSI.os/time/gmtime */
	if ( ptm == &tm )
	{
		if ( tm.tm_sec != 34 )
		{
			++nfails;
			tet_printf("Expected tm_sec of 34, got %d", tm.tm_sec);
		}

		if ( tm.tm_min != 29 )
		{
			++nfails;
			tet_printf("Expected tm_min of 29, got %d", tm.tm_min);
		}

		if ( tm.tm_hour != 12 )
		{
			++nfails;
			tet_printf("Expected tm_hour of 12, got %d", tm.tm_hour);
		}

		if ( tm.tm_mday != 23 )
		{
			++nfails;
			tet_printf("Expected tm_mday of 23, got %d", tm.tm_mday);
		}

		if ( tm.tm_year != 86 )
		{
			++nfails;
			tet_printf("Expected tm_year of 86, got %d", tm.tm_year);
		}

		if ( tm.tm_wday != 4 )
		{
			++nfails;
			tet_printf("Expected tm_wday of 4, got %d", tm.tm_wday);
		}

		if ( tm.tm_yday != 295 )
		{
			++nfails;
			tet_printf("Expected tm_yday of 295, got %d", tm.tm_yday);
		}

		if ( tm.tm_isdst != 0 )
		{
			++nfails;
			tet_printf("Expected tm_isdst of 0, got %d", tm.tm_isdst);
		}
	}
	else
	{
		++nfails;
		tet_infoline("gmtime_r unexpectedly failed");
	}

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when an error is detected or UTC is not available,");
	tet_infoline("then concurrent calls by multiple threads to");
	tet_infoline("gmtime_r(clock, result) return NULL");

	tet_infoline("Posix Ref: Component GMTIME_R");
	tet_infoline("Assertion 9945-1:1996 8.3.6.3-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	/* POSIX 1996l, page 222, 8.3.6.4: no defined error conditions
	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );
	*/

	tet_infoline("No portable test");
	result = TET_UNTESTED;

#else

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T2_NTHREADS	8

int			T2_flag;
pthread_mutex_t		T2_mx;
pthread_cond_t		T2_cv;
VSTH_TSET		T2_tset[T2_NTHREADS];

/******	There are no reliable ways of causing gmtime_r to err. We try taking
	gmtime_r our of range with an argument of INT_MAX. If it succeeds,
	we report an unresolved and dump the data to the journal for later
	inspection. If we could rely on a time_t being a signed value, we
	could try range errors on the negative side, but no such luck.	******/

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
		return;

	T2_flag		= T2_NTHREADS;

	vsth_weave( &T2_tset[0], T2_NTHREADS, t2_t1, &T2_mx, &T2_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	struct tm	*ptm;
	struct tm	tm;
	time_t		ticks;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	ticks	= INT_MAX;
	ptm	= gmtime_r( &ticks, &tm );

	/* expected values from VSX4.3.5 tset/ANSI.os/time/gmtime */
	if ( ptm != NULL )
	{
		++nerrs;
		tet_infoline("gmtime_r unexpectedly succeeded");

		tet_printf("tm.tm_sec\t%d", tm.tm_sec);
		tet_printf("tm.tm_min\t%d", tm.tm_min);
		tet_printf("tm.tm_hour\t%d", tm.tm_hour);
		tet_printf("tm.tm_mday\t%d", tm.tm_mday);
		tet_printf("tm.tm_mon\t%d", tm.tm_mon);
		tet_printf("tm.tm_year\t%d", tm.tm_year);
		tet_printf("tm.tm_wday\t%d", tm.tm_wday);
		tet_printf("tm.tm_yday\t%d", tm.tm_yday);
		tet_printf("tm.tm_isdst\t%d", tm.tm_isdst);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	NULL,	0	}
};

