
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/localtime_r
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)localtime_r.c	1.14
*
*	History:
*	Moved test 1 to a separate executable.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	1

#ifdef  UNDEF_MACROS
#undef  localtime_r
#endif

extern	char	**environ;

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();

void	t1_f1( int *errors, int *fails );
void	t1_c1();

void
startup()
{
	vsth_startup("localtime_r");
}

void
cleanup()
{
	vsth_cleanup("localtime_r");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("concurrent calls by multiple threads to localtime_r(clock, result)");
	tet_infoline("convert the calendar time referenced by clock into a broken-down");
	tet_infoline("time expressed as local time, store this time in the structure");
	tet_infoline("referenced by result and return result if successful.");

	tet_infoline("Posix Ref: Component LOCALTIME_R");
	tet_infoline("Assertion 9945-1:1996 8.3.7.2-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	TE_PATH		"localtime_r_exectests"

void
t1_f1( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork(t1_c1, NULL, VSTH_TIMEOUT, 0 | 1 | 2);
	if ( rval == 2 )
	{
		++*fails;
	}
	else if ( rval )
	{
		++*errors;
	}
}

void
t1_c1()
{
	char		*argv[2];

	argv[0] = execpath(TE_PATH);
	argv[1] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	NULL,	0	}
};

