
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/localtime_r_exectests
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)localtime_r_exectests.c	1.1
*
*	History:
*	Created separate executable to avoid calls to non-asynch safe functions.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#ifdef  UNDEF_MACROS
#undef  localtime_r
#endif

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_c1();
void	*t1_t1( void *arg );

#endif

int	tet_nosigreset = 0;

int
tet_main(int argc, char **argv )
{

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_c1();

#endif

	exit(1);
}

extern	char **environ;

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_NTHREADS	8

int			T1_flag;
pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;
time_t			T1_ticks;
VSTH_TSET		T1_tset[T1_NTHREADS];

void
t1_c1()
{
	int	rval;
	int	nerrs;
	int	nfails;
	int	priomin;
	char	*tz;

	nerrs	= 0;
	nfails	= 0;
	tz	= "LOC-3";

	vsth_getprio( SCHED_RR, &priomin, NULL, &nerrs);
	if ( nerrs )
		return;

	T1_flag		= T1_NTHREADS;
	T1_ticks	= 567869055L;

	rval = vputenv(&environ, "TZ=LOC-3");
	if ( rval )
	{
		++nerrs;
		tet_infoline("vputenv failed");
	}

	tz = getenv("TZ");
	if (( ! tz ) || strcmp(tz, "LOC-3"))
	{
		++nerrs;
		tet_infoline("Unable to set TZ");
	}

	if ( nerrs == 0 )
	{
		vsth_weave( &T1_tset[0], T1_NTHREADS, t1_t1, &T1_mx, &T1_cv,
				SCHED_RR, priomin, 0, &nerrs, &nfails );
	}

	rval = 1;
	if ( nfails )
	{
		rval = 2;
	}
	else if (( nerrs == 0 ) && ( nfails == 0 ))
	{
		rval = 0;
	}

	exit(rval);
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	struct tm	*ptm;
	struct tm	tm;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	ptm = localtime_r( &T1_ticks, &tm );

	/* expected values from VSX4.3.5 tset/ANSI.os/time/localtime:test1 */
	if ( ptm == &tm )
	{
		if ( tm.tm_sec != 15 )
		{
			++nfails;
			tet_printf("Expected tm_sec of 34, got %d", tm.tm_sec);
		}

		if ( tm.tm_min != 24 )
		{
			++nfails;
			tet_printf("Expected tm_min of 29, got %d", tm.tm_min);
		}

		if ( tm.tm_hour != 16 )
		{
			++nfails;
			tet_printf("Expected tm_hour of 12, got %d", tm.tm_hour);
		}

		if ( tm.tm_mday != 30 )
		{
			++nfails;
			tet_printf("Expected tm_mday of 23, got %d", tm.tm_mday);
		}

		if ( tm.tm_mon != 11 )
		{
			++nfails;
			tet_printf("Expected tm_mon of 11, got %d", tm.tm_mon);
		}

		if ( tm.tm_year != 87 )
		{
			++nfails;
			tet_printf("Expected tm_year of 86, got %d", tm.tm_year);
		}

		if ( tm.tm_wday != 3 )
		{
			++nfails;
			tet_printf("Expected tm_wday of 4, got %d", tm.tm_wday);
		}

		if ( tm.tm_yday != 363 )
		{
			++nfails;
			tet_printf("Expected tm_yday of 295, got %d", tm.tm_yday);
		}

		if ( tm.tm_isdst != 0 )
		{
			++nfails;
			tet_printf("Expected tm_isdst of 0, got %d", tm.tm_isdst);
		}
	}
	else
	{
		++nfails;
		tet_infoline("localtime_r unexpectedly failed");
	}

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif
