
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/putc_unlocked
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)putc_unlocked.c	1.12
*
*	History:
*	Moved tests 1, 2 and 3 to a separate executable.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	3

#ifdef  UNDEF_MACROS
#undef  putc_unlocked
#endif

extern	char	**environ;

extern	char	*itoa(int);

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t2_f2( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );

void	t1_c1();
void	t2_c1();
void	t2_c2();
void	t3_c1();

#endif

void
startup()
{
	vsth_startup("putc_unlocked");
}

void
cleanup()
{
	vsth_cleanup("putc_unlocked");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the file referenced by stream supports positioning requests,");
	tet_infoline("then a call to putc_unlocked(c, stream) writes the character specified");
	tet_infoline("by c, converted to an unsigned char, to the output stream referenced");
	tet_infoline("by stream at the position indicated by the associated file position");
	tet_infoline("indicator for the stream, advances the indicator appropriately and");
	tet_infoline("returns c if successful.");

	tet_infoline("Posix Ref: Component PUTC_UNLOCKED");
	tet_infoline("Assertion 9945-1:1996 8.2.7.2, ANSI/ISO 9899-1990 7.9.7.8-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_FILENAME	"./t1_file"
#define TE_PATH		"putc_unlocked_exectests"

/* quote taken from "Learning How to Learn" by Idries Shah */
#define	T1_STRING	"Nobody attains to the degree of Truth until a thousand sincere people have pronounced him an unbeliever."

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		fd;
	pthread_t	tid;
	void		*tval;
	int		len;
	char		*buf;

	rval = tet_fork( t1_c1, NULL, VSTH_TIMEOUT / 2, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		++*fails;
	}
	else if ( rval )
	{
		++*errors;
	}

	fd = open(T1_FILENAME, O_RDONLY );
	if ( fd < 0 )
	{
		++*errors;
		tet_printf("open unexpectedly failed, errno == %d", errno);
		return;
	}

	len = strlen(T1_STRING);
	buf = (char *) malloc( (len + 1) * sizeof(char));
	if ( buf == NULL )
	{
		++*errors;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
		(void) close(fd);
		(void) remove(T1_FILENAME);
		return;
	}

	memset(buf, 0, len + 1);

	rval = read( fd, buf, len );
	if ( rval != len )
	{
		++*errors;
		tet_printf("read unexpectedly failed, expected %d, got %d", len, rval);
		(void) close(fd);
		(void) remove(T1_FILENAME);
		(void) free(buf);
		return;
	}

	rval = close(fd);
	if ( rval != 0 )
	{
		vsth_surprise("close", 0, rval, errors);
		(void) remove(T1_FILENAME);
		(void) free(buf);
		return;
	}

	if ( strcmp(buf, T1_STRING) )
	{
		++*fails;
		tet_printf("Expected to read %s", T1_STRING);
		tet_printf("Read %s", buf);
	}

	(void) free(buf);
	(void) remove(T1_FILENAME);
}

void
t1_c1()
{
	char		*argv[2];

	argv[0] = execpath(TE_PATH);
	argv[1] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the file referenced by stream does not support positioning");
	tet_infoline("requests or the stream was opened in append mode,");
	tet_infoline("then a call to putc_unlocked(c, stream) appends the character");
	tet_infoline("specified by c, converted to an unsigned char, to the output stream");
	tet_infoline("referenced by stream and returns c if successful.");

	tet_infoline("Posix Ref: Component PUTC_UNLOCKED");
	tet_infoline("Assertion 9945-1:1996 8.2.7.2, ANSI/ISO 9899-1990 7.9.7.8-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t2_f1( &errors, &fails );
	t2_f2( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

/* from "The Art of Worldly Wisdom" by the seventeen century Spanish Jesuit Balthasar Gracian */
#define	T2_STRING	"Do good a little at a time, but often."

int		T2_fd[2];

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	int		fd;
	pthread_t	tid;
	void		*tval;
	int		len;
	char		*buf;

	rval = pipe(T2_fd);
	if ( rval != 0 )
	{
		vsth_surprise("pipe", 0, rval, errors);
		(void) close(T2_fd[0]);
		(void) close(T2_fd[1]);
		return;
	}

	rval = fcntl( T2_fd[0], F_SETFL, O_NONBLOCK );
	if ( rval != 0 )
	{
		vsth_surprise("fcntl", 0, rval, errors);
		(void) close(T2_fd[0]);
		(void) close(T2_fd[1]);
		return;
	}

	rval = tet_fork( t2_c1, NULL, VSTH_TIMEOUT / 2, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		++*fails;
	}
	else if ( rval )
	{
		++*errors;
	}

	len = strlen(T2_STRING);
	buf = (char *) malloc( (len + 1) * sizeof(char));
	if ( buf == NULL )
	{
		++*errors;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
		(void) close(T2_fd[0]);
		(void) close(T2_fd[1]);
		return;
	}

	memset(buf, 0, len + 1);

	rval = read( T2_fd[0], buf, len );
	if ( rval != len )
	{
		++*errors;
		tet_printf("read unexpectedly failed, expected %d, got %d", len, rval);
		(void) close(T2_fd[0]);
		(void) close(T2_fd[1]);
		(void) free(buf);
		return;
	}

	if ( strcmp(buf, T2_STRING) )
	{
		++*fails;
		tet_printf("Expected to read %s", T2_STRING);
		tet_printf("Read %s", buf);
	}

	(void) close(T2_fd[0]);
	(void) close(T2_fd[1]);
	(void) free(buf);
}

void
t2_c1()
{
	char		*argv[4];
	char		ascii_fd0[20];
	char		ascii_fd1[20];

	argv[0] = execpath(TE_PATH);
	argv[1] = "1";
	argv[2] = strcpy(ascii_fd0, itoa(T2_fd[0]));
	argv[3] = strcpy(ascii_fd1, itoa(T2_fd[1]));
	argv[4] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

#define	T2_FILENAME	"./t2_file"

/* quote from Blaise Pascal taken from "A History of Pi" by Petr Beckmann */
#define	T2_STRING1	"I have made this letter long because I lacked the time to make it short."

/* quote from "The Essential Rumi", translated by Coleman Barks with John Moyne */
#define	T2_STRING2	"Who comes to a spring thirsty and sees a moon reflected in it ?"

/******	Check append mode								******/
void
t2_f2( int *errors, int *fails )
{
	int		rval;
	int		fd;
	pthread_t	tid;
	void		*tval;
	int		len;
	char		*buf;

	fd = creat(T2_FILENAME, S_IRWXU | S_IRWXG );
	if ( fd < 0 )
	{
		++*errors;
		tet_printf("creat unexpectedly failed, errno == %d", errno);
		return;
	}

	len = strlen(T2_STRING1);
	rval = write( fd, T2_STRING1, len );
	if ( rval != len )
	{
		vsth_surprise("write", len, rval, errors);
		(void) close(fd);
		(void) remove(T2_FILENAME);
		return;
	}

	rval = close(fd);
	if ( rval != 0 )
	{
		vsth_surprise("close", 0, rval, errors);
		(void) remove(T2_FILENAME);
		return;
	}

	rval = tet_fork( t2_c2, NULL, VSTH_TIMEOUT / 2, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		++*fails;
	}
	else if ( rval )
	{
		++*errors;
	}

	fd = open(T2_FILENAME, O_RDONLY );
	if ( fd < 0 )
	{
		++*errors;
		tet_printf("open unexpectedly failed, errno == %d", errno);
		(void) remove(T2_FILENAME);
		return;
	}

	len = strlen(T2_STRING1);
	rval = lseek( fd, len, SEEK_SET );
	if ( rval != len )
	{
		++*errors;
		tet_printf("lseek unexpectedly failed, errno == %d", errno);
		(void) close(fd);
		(void) remove(T2_FILENAME);
		return;
	}
		
	len = strlen(T2_STRING2);
	buf = (char *) malloc( (len + 1) * sizeof(char));
	if ( buf == NULL )
	{
		++*errors;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
		(void) close(fd);
		(void) remove(T2_FILENAME);
		return;
	}

	memset(buf, 0, len + 1);

	rval = read( fd, buf, len );
	if ( rval != len )
	{
		++*errors;
		tet_printf("read unexpectedly failed, expected %d, got %d", len, rval);
		(void) close(fd);
		(void) remove(T2_FILENAME);
		(void) free(buf);
		return;
	}

	rval = close(fd);
	if ( rval != 0 )
	{
		vsth_surprise("close", 0, rval, errors);
		(void) remove(T2_FILENAME);
		(void) free(buf);
		return;
	}

	if ( strcmp(buf, T2_STRING2) )
	{
		++*fails;
		tet_printf("Expected to read %s", T2_STRING);
		tet_printf("Read %s", buf);
	}

	(void) free(buf);
	(void) remove(T2_FILENAME);
}

void
t2_c2()
{
	char		*argv[2];

	argv[0] = execpath(TE_PATH);
	argv[1] = "2";
	argv[2] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

#endif

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when a write-error occurs,");
	tet_infoline("then a call to putc_unlocked(c, stream) shall set the error indicator");
	tet_infoline("for stream and return EOF.");

	tet_infoline("Posix Ref: Component PUTC_UNLOCKED");
	tet_infoline("Assertion 9945-1:1996 8.2.7.2, ANSI/ISO 9899-1990 7.9.7.8-3(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T3_FILENAME	"./t3_fifo"

int			T3_sigflag;
sigset_t		T3_oset;
struct sigaction	T3_osa;

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	int		fd;
	pthread_t	tid;
	void		*tval;
	int		len;
	char		*buf;

	rval = mkfifo( T3_FILENAME, S_IRWXU | S_IRWXG );
	if ( rval != 0 )
	{
		vsth_surprise("mkfifo", 0, rval, errors);
		return;
	}

	rval = tet_fork( t3_c1, NULL, VSTH_TIMEOUT / 2, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		++*fails;
	}
	else if ( rval )
	{
		++*errors;
	}

	(void) remove(T3_FILENAME);
}

void
t3_c1()
{
	char		*argv[2];

	argv[0] = execpath(TE_PATH);
	argv[1] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	NULL,	0	}
};


