
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/putchar_unlocked_exectests
*
*	The Open Group, VSTH Threads test suite
*	@(#)putchar_unlocked_exectests.c	1.2
*
*	History:
*	Created separate executable to avoid calls to non-asynch safe functions.
*	Changed pthread_detach to vsth_pthread_detach to allow
*	TET API to only reap residual threads.
*
*	Add calls to flockfile and funlockfile around the function under test
*	TSD5TH.00069.
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#ifdef  UNDEF_MACROS
#undef  putchar_unlocked
#endif

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	*t1_t1( void *arg );
void	t1_c1();
void	*t2_t1( void *arg );
void	t2_c1();
void	*t2_t2( void *arg );
void	t2_c2();
void	t3_c1();
void	*t3_t1( void *arg );
void	t3_sigpipe( int sig );
int	t3_ssigs();
int	t3_rsigs();

int		T2_fd[2];

#endif

int	tet_nosigreset = 0;

int
tet_main( int argc, char **argv)
{
	int	T2_subtest;

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	if (tet_thistest == 1)
		t1_c1();

	if (tet_thistest == 2)
	{
		if (argc >= 2)
		{
			T2_subtest = atoi(argv[1]);
			switch (T2_subtest) {
				case 1:
					if (argc < 4)
					{
						tet_printf("Wrong number of arguments to exec of test2_1");
						exit(1);
					}
					T2_fd[0] = atoi(argv[2]);
					T2_fd[1] = atoi(argv[3]);
					t2_c1();
					break;
				case 2:
					t2_c2();
					break;
				default:
					tet_printf("Wrong subtest number passed in arguments to tet_exec()");
					break;
			}
		}
		else
			tet_printf("Wrong number of arguments to exec of test2_1");
	}

	if (tet_thistest == 3)
		t3_c1();

#endif

	exit(1);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_FILENAME	"./t1_file"

/* quote taken from "The Prophet" by Kahlil Gibran */
#define	T1_STRING	"He who wears his morality as his best garment were better naked."

void
t1_c1()
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = vsth_pthread_create( &tid, NULL, t1_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, NULL);
		exit(1);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		exit( (int) tval );
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		exit(1);
	}
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	int		len;
	int		cnt;
	FILE		*fp;
	char		*p;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	len	= strlen(T1_STRING);
	p	= T1_STRING + len;
	fp	= freopen(T1_FILENAME, "w", stdout);

	setbuf( stdout, NULL );
	flockfile(stdout);

	/* write the file backwards */
	for ( cnt = 0; cnt < len; ++cnt )
	{
		rval = fseek( stdout, len - 1 - cnt, SEEK_SET );
		if ( rval != 0 )
		{
			vsth_surprise("fseek", 0, rval, &nerrs);
		}

		rval = putchar_unlocked( (int) *--p );
		if ( rval != *p )
		{
			vsth_surprise("putchar_unlocked", (int) *p, rval, &nfails);
		}
	}

	funlockfile(stdout);
	(void) fclose(fp);

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

/* quote taken from a poem by William Blake */
#define	T2_STRING	"To see a World in a Grain of Sand, And a Heaven in a Wild Flower."

void
t2_c1()
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = vsth_pthread_create( &tid, NULL, t2_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, NULL);
		exit(1);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		exit( (int) tval );
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		exit(1);
	}
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	int		len;
	int		cnt;
	char		*p;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	p	= T2_STRING;
	len	= strlen(T2_STRING);

	rval = close(STDOUT_FILENO);
	if ( rval == 0 )
	{
		rval = dup( T2_fd[1] );
		if ( rval == STDOUT_FILENO )
		{
			flockfile(stdout);
			for ( cnt = 0; cnt < len; ++cnt )
			{
				rval = putchar_unlocked( (int) *p );
				if ( rval != *p )
				{
					vsth_surprise("putchar_unlocked", (int) *p, rval, &nfails);
				}
				++p;
			}
			funlockfile(stdout);
		}
		else
		{
			vsth_surprise("dup", STDOUT_FILENO, rval, &nerrs);
		}
	}
	else
	{
		vsth_surprise("close", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#define	T2_FILENAME	"./t2_file"

/* quote from Blaise Pascal taken from "A History of Pi" by Petr Beckmann */
#define	T2_STRING1	"I have made this letter long because I lacked the time to make it short."

/* quote from "The Essential Rumi", translated by Coleman Barks with John Moyne */
#define	T2_STRING2	"Who comes to a spring thirsty and sees a moon reflected in it ?"

void
t2_c2()
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = vsth_pthread_create( &tid, NULL, t2_t2, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, NULL);
		exit(1);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		exit( (int) tval );
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		exit(1);
	}
}

void *
t2_t2( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	int		len;
	int		cnt;
	FILE		*fp;
	char		*p;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	len	= strlen(T2_STRING2);
	p	= T2_STRING2;
	fp	= freopen(T2_FILENAME, "a", stdout);
	if ( fp != NULL )
	{
		setbuf( stdout, NULL );
	
		flockfile(stdout);
		for ( cnt = 0; cnt < len; ++cnt )
		{
			rval = putchar_unlocked( (int) *p );
			if ( rval != *p )
			{
				vsth_surprise("putchar_unlocked", (int) *p, rval, &nfails);
			}
			++p;
		}
		funlockfile(stdout);
		(void) fclose(stdout);
	}
	else
	{
		++nerrs;
		tet_printf("fopen unexpectedly failed, errno == %d", errno);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#define	T3_FILENAME	"./t3_fifo"

int			T3_sigflag;
sigset_t		T3_oset;
struct sigaction	T3_osa;

void
t3_c1()
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	T3_sigflag = 0;

	rval = t3_ssigs();
	if ( rval != 0 )
	{
		(void) t3_rsigs();
		return;
	}

	rval = vsth_pthread_create( &tid, NULL, t3_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, NULL);
		exit(1);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		exit( (int) tval );
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		exit(1);
	}

	if ( T3_sigflag != 1 )
	{
		vsth_surprise("T3_sigflag", 1, T3_sigflag, NULL);
		exit(1);
	}

	(void) t3_rsigs();
}

void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	int		rfd;
	FILE		*fp;
	sigset_t	set;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGPIPE );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_UNBLOCK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rfd = open(T3_FILENAME, O_RDONLY | O_NONBLOCK );
	if ( rfd >= 0 )
	{
		fp = freopen( T3_FILENAME, "w", stdout );
		if ( fp != NULL )
		{
			setbuf( fp, NULL );

			rval = close( rfd );
			if ( rval == 0 )
			{
				flockfile(stdout);
				rval = putchar_unlocked('X');
				if ( rval != EOF )
				{
					vsth_surprise("putchar_unlocked",
							EOF,
							rval,
							&nfails);
				}
		
				rval = ferror(stdout);
				if ( rval == 0 )
				{
					vsth_surprise("ferror",
							1,
							rval,
							&nfails);
				}
				funlockfile(stdout);
			}
			else
			{
				vsth_surprise("close", 0, rval, &nerrs);
			}

			(void) fclose(fp);
		}
		else
		{
			++nerrs;
			tet_printf("freopen unexpectedly failed, errno == %d", errno);
			(void) close(rfd);
		}
	}
	else
	{
		++nerrs;
		tet_printf("open unexpectedly failed, errno == %d", errno);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

int
t3_ssigs()
{
	int			rval;
	int			nerrs;
	struct sigaction	sa;

	nerrs	= 0;

	rval = sigfillset( &sa.sa_mask );
	if ( rval == 0 )
	{
		rval = pthread_sigmask( SIG_SETMASK, &sa.sa_mask, &T3_oset );
		if ( rval == 0 )
		{
			sa.sa_flags	= 0;
			sa.sa_handler	= t3_sigpipe;
			rval = sigaction( SIGPIPE, &sa, &T3_osa );
			if ( rval != 0 )
			{
				vsth_surprise("sigaction", 0, rval, &nerrs);
			}
		}
		else
		{
			vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
		}
	}
	else
	{
		vsth_surprise("sigfillset", 0, rval, &nerrs);
	}

	return(nerrs);
}

int
t3_rsigs()
{
	int	rval;

	rval = sigaction( SIGPIPE, &T3_osa, NULL );
	if ( rval == 0 )
	{
		rval = pthread_sigmask( SIG_SETMASK, &T3_oset, NULL );
	}

	return(rval);
}

void
t3_sigpipe( int sig )
{
	++T3_sigflag;
}

#endif
