
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/rand_r
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)rand_r.c	1.11
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<limits.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	3

#ifdef  UNDEF_MACROS
#undef  rand_r
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );

void	*t1_t1( void *arg );
void	*t3_t1( void *arg );

#endif

void
startup()
{
	vsth_startup("rand_r");
}

void
cleanup()
{
	vsth_cleanup("rand_r");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("concurrent calls by multiple threads to rand_r(seed) return");
	tet_infoline("a pseudo-random integer in the range 0 to RAND_MAX.");

	tet_infoline("Posix Ref: Component RAND_R");
	tet_infoline("Assertion 9945-1:1996 8.3.8.2-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_T_NTHREADS	8
#define	T1_TRIES	100

int			T1_flag;
pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;
VSTH_TSET		T1_tset[T1_T_NTHREADS];

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors );
	if ( *errors )
		return;

	T1_flag = T1_T_NTHREADS;

	vsth_weave( &T1_tset[0], T1_T_NTHREADS, t1_t1, &T1_mx, &T1_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	int		cnt;
	int		flag;
	unsigned int	seed;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;
	seed	= time(NULL) % getpid();

	rval = vsth_bwait( &T1_cv, &T1_mx, &T1_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	flag = 0;
	for ( cnt = 0; cnt < T1_TRIES; ++cnt )
	{
		rval = rand_r( &seed );
		if (( rval < 0 ) || ( rval > RAND_MAX ))
		{
			++nfails;
			if ( ! flag )
			{
				tet_printf("rand_r returned %d", rval);
				flag = 1;
			}
		}
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("RAND_MAX shall be at least 32767.");

	tet_infoline("Posix Ref: Component RAND_R");
	tet_infoline("Assertion 9945-1:1996 8.3.8.2-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void
t2_f1( int *errors, int *fails )
{
	if ( RAND_MAX < 32767 )
	{
		++*fails;
		tet_printf("RAND_MAX is %d", RAND_MAX);
	}
}

#endif

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the object referenced by seed is not modified between successive");
	tet_infoline("calls,");
	tet_infoline("then calls to rand_r(seed) shall return the same sequence of numbers.");

	tet_infoline("Posix Ref: Component RAND_R");
	tet_infoline("Assertion 9945-1:1996 8.3.8.2-3(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T3_T_NTHREADS	8
#define	T3_TRIES	100

int			T3_flag;
pthread_mutex_t		T3_mx;
pthread_cond_t		T3_cv;
VSTH_TSET		T3_tset[T3_T_NTHREADS];
int			T3_results[T3_T_NTHREADS][T3_TRIES];

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors );
	if ( *errors )
		return;

	T3_flag = T3_T_NTHREADS;
	vsth_weave( &T3_tset[0], T3_T_NTHREADS, t3_t1, &T3_mx, &T3_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	int		cnt;
	int		tidnum;
	int		*presult;
	unsigned int	seed;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tidnum	= (int) arg;
	presult = &T3_results[tidnum][0];
	tval	= (void *) 1;

	seed = getpid();

	rval = vsth_bwait( &T3_cv, &T3_mx, &T3_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	for ( cnt = 0; cnt < T3_TRIES; ++cnt );
	{
		*presult++ = rand_r( &seed );
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
t3_eval( int *errors, int *fails )
{
	int	cnt;
	int	flag;
	int	tidnum;
	int	result;

	flag = 0;
	for ( cnt = 0; cnt < T3_TRIES; ++cnt )
	{
		/* use thread 0 as a reference value */
		result = T3_results[0][cnt];
		for ( tidnum = 1; tidnum < T3_T_NTHREADS; ++tidnum )
		{
			/* check other thread's rand_r value */
			if ( T3_results[tidnum][cnt] != result )
			{
				++*fails;
				if ( ! flag )
				{
					tet_printf("Thread %d, sequence %d, value %d, expected %d",
							tidnum, cnt, T3_results[tidnum][cnt], result);
					flag = 1;
				}
			}
		}
	}
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	NULL,	0	}
};

