
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/setlocale
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)setlocale.c	1.12
*
*	History
*		Changed pthread_detach to vsth_pthread_detach to allow
*		TET API to only reap residual threads.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<locale.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	1

#ifdef  UNDEF_MACROS
#undef  setlocale
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();

void	t1_f1( int *errors, int *fails );
void	*t1_t1( void *arg );

void
startup()
{
	vsth_startup("setlocale");
}

void
cleanup()
{
	vsth_cleanup("setlocale");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A successful call to setlocale(category, locale) by a thread changes");
	tet_infoline("the locale for all threads in the same process as the calling thread.");

	tet_infoline("Posix Ref: Component SETLOCALE Assertion 9945-1:1996 8.1.2.2-1(A)");

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int			T1_flag;
pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;
char			*T1_loc;

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;
	char		*oldloc;

	T1_flag = 0;

	oldloc = setlocale( LC_ALL, NULL);
	T1_loc = setlocale( LC_ALL, "POSIX" );

	if ( T1_loc == NULL )
	{
		++*errors;
		tet_infoline("setlocale unexpectedly failed");
		return;
	}

	rval = pthread_cond_init( &T1_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		return;
	}

	rval = pthread_mutex_init( &T1_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		(void) pthread_cond_destroy( &T1_cv );
		return;
	}

	rval = pthread_mutex_lock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
		(void) pthread_cond_destroy( &T1_cv );
		(void) pthread_mutex_destroy( &T1_mx );
		return;
	}

	rval = vsth_pthread_create( &tid, NULL, t1_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		(void) pthread_cond_destroy( &T1_cv );
		(void) pthread_mutex_destroy( &T1_mx );
		return;
	}

	while ( T1_flag != 1 )
	{
		rval = pthread_cond_wait( &T1_cv, &T1_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
			(void) pthread_cancel( tid );
			(void) vsth_pthread_detach( tid );
			(void) pthread_cond_destroy( &T1_cv );
			(void) pthread_mutex_destroy( &T1_mx );
			return;
		}
	}

	T1_loc = setlocale( LC_ALL, "C" );

	if ( T1_loc == NULL )
	{	
		++*errors;
		tet_infoline("setlocale unexpectedly failed");
	}

	++T1_flag;

	rval = pthread_mutex_unlock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
	}

	rval = pthread_cond_signal( &T1_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, errors);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	(void) setlocale( LC_ALL, oldloc );
	(void) pthread_mutex_destroy( &T1_mx );
	(void) pthread_cond_destroy( &T1_cv );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	char		*loc;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	loc = setlocale( LC_ALL, NULL );
	if ( strcmp( loc, T1_loc ) )
	{
		++nfails;
		tet_printf("setlocale returned an unexpected locale string %s", loc);
	}

	rval = pthread_mutex_lock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	++T1_flag;

	rval = pthread_cond_signal( &T1_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	while ( T1_flag != 2 )
	{
		rval = pthread_cond_wait( &T1_cv, &T1_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
		}
	}

	rval = pthread_mutex_unlock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
	}

	loc = setlocale( LC_ALL, NULL );
	if ( strcmp( loc, T1_loc ) )
	{
		++nfails;
		tet_printf("setlocale returned an unexpected locale string %s", loc);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}


void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	NULL,	0	}
};

