
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/strtok_r
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)strtok_r.c	1.11
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	3

#ifdef  UNDEF_MACROS
#undef  strtok_r
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );

void	*t1_t1( void *arg );
void	*t2_t1( void *arg );
void	*t3_t1( void *arg );

#endif

void
startup()
{
	vsth_startup("strtok_r");
}

void
cleanup()
{
	vsth_cleanup("strtok_r");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when s points to a null-terminated string, and sep points to a");
	tet_infoline("null-terminated string of token separator characters,");
	tet_infoline("then concurrent calls by multiple threads to strtok_r(s, sep, lasts)");
	tet_infoline("shall write a null character into s immediately following the first");
	tet_infoline("token, update the pointer referenced by lasts, and shall return");
	tet_infoline("a pointer to the first character of the first token to the");
	tet_infoline("calling thread.");

	tet_infoline("Posix Ref: Component STRTOK_R");
	tet_infoline("Assertion 9945-1:1996 8.3.3.2-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_T_NTHREADS	8

/* quote from "The Prophet" by Kahlil Gibran */
#define	T1_STRING	"People of Orphalese, beauty is life when life unveils her holy face"
#define	T1_TOKEN1	"People"
#define	T1_SEPSTR	" ,"

int			T1_flag;
pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;
VSTH_TSET		T1_tset[T1_T_NTHREADS];

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors );
	if ( *errors )
		return;

	T1_flag = T1_T_NTHREADS;

	vsth_weave( &T1_tset[0], T1_T_NTHREADS, t1_t1, &T1_mx, &T1_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	char		*sepstr;
	char		*lasts;
	char		*p;
	char		*val;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T1_cv, &T1_mx, &T1_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	p = (char *) malloc( (strlen(T1_STRING) + 1 ) * sizeof(char));
	if ( p != NULL )
	{
		strcpy( p, T1_STRING );
		sepstr = T1_SEPSTR;

		val = strtok_r( p, sepstr, &lasts );

		if ( strcmp( p, T1_TOKEN1 ))
		{
			++nfails;
			tet_printf("Unexpected string %s", p);
		}

		if ( lasts != ( p + strlen(T1_TOKEN1) + 1 ))
		{
			++nfails;
			tet_printf("Expected lasts of %p, but got %p",
				p + strlen(T1_TOKEN1) + 1, lasts );
		}

		if ( val != p )
		{
			++nfails;
			tet_printf("Expected return value of %p, got %p", p, val);
		}

		(void) free(p);
	}
	else
	{
		++nerrs;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when s is the NULL pointer, sep points to a null-terminated");
	tet_infoline("string of token separator characters, lasts is unchanged from");
	tet_infoline("the first call to strtok_r() by the calling thread and tokens remain,");
	tet_infoline("then concurrent calls by multiple threads to strtok_r(s, sep, lasts)");
	tet_infoline("shall write a null character into s immediately following the next");
	tet_infoline("successive token, update the pointer referenced by lasts, and");
	tet_infoline("shall return a pointer to the first character of the new token to");
	tet_infoline("the calling thread.");

	tet_infoline("Posix Ref: Component STRTOK_R");
	tet_infoline("Assertion 9945-1:1996 8.3.3.2-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T2_T_NTHREADS		8

/* quote from "The Prophet" by Kahlil Gibran */
#define	T2_STRING		"Your	joy is your sorrow unmasked"
#define	T2_TOKEN2_OFFSET	5
#define	T2_TOKEN2		"joy"
#define	T2_SEPSTR		" \t"

int			T2_flag;
pthread_mutex_t		T2_mx;
pthread_cond_t		T2_cv;
VSTH_TSET		T2_tset[T2_T_NTHREADS];

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors );
	if ( *errors )
		return;

	T2_flag = T2_T_NTHREADS;

	vsth_weave( &T2_tset[0], T2_T_NTHREADS, t2_t1, &T2_mx, &T2_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	char		*sepstr;
	char		*lasts;
	char		*p;
	char		*val;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T2_cv, &T2_mx, &T2_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	p = (char *) malloc( (strlen(T2_STRING) + 1 ) * sizeof(char));
	if ( p != NULL )
	{
		strcpy( p, T2_STRING );
		sepstr	= T2_SEPSTR;
		val	= strtok_r( p, sepstr, &lasts );
		val	= strtok_r( NULL, sepstr, &lasts );

		if ( strcmp( p + T2_TOKEN2_OFFSET, T2_TOKEN2 ))
		{
			++nfails;
			tet_printf("Unexpected string %s", p + T2_TOKEN2_OFFSET);
		}

		if ( lasts != ( p + T2_TOKEN2_OFFSET + strlen(T2_TOKEN2) + 1 ))
		{
			++nfails;
			tet_printf("Expected lasts of %p, but got %p",
				p + T2_TOKEN2_OFFSET + strlen(T2_TOKEN2) + 1, lasts );
		}

		if ( val != ( p + T2_TOKEN2_OFFSET ) )
		{
			++nfails;
			tet_printf("Expected strtok_r to return %p, got %p", p + T2_TOKEN2_OFFSET, val);
		}

		(void) free(p);
	}
	else
	{
		++nerrs;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when no tokens remain in the string pointed to by s,");
	tet_infoline("then concurrent calls by multiple threads to strtok_r(s, sep, lasts)");
	tet_infoline("shall return NULL.");

	tet_infoline("Posix Ref: Component STRTOK_R");
	tet_infoline("Assertion 9945-1:1996 8.3.3.2-3(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T3_T_NTHREADS	8

/* quote from "The Prophet" by Kahlil Gibran */
#define	T3_STRING	"Your pain is the breaking of the shell\tthat encloses your understanding"
#define	T3_SEPSTR	"\t"

int			T3_flag;
pthread_mutex_t		T3_mx;
pthread_cond_t		T3_cv;
VSTH_TSET		T3_tset[T3_T_NTHREADS];

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors );
	if ( *errors )
		return;

	T3_flag = T3_T_NTHREADS;

	vsth_weave( &T3_tset[0], T3_T_NTHREADS, t3_t1, &T3_mx, &T3_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	char		*sepstr;
	char		*lasts;
	char		*p;
	char		*val;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T3_cv, &T3_mx, &T3_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	p = (char *) malloc( (strlen(T3_STRING) + 1 ) * sizeof(char));
	if ( p != NULL )
	{
		strcpy( p, T3_STRING );
		sepstr = T3_SEPSTR;

		(void) strtok_r( p, sepstr, &lasts );
		(void) strtok_r( NULL, sepstr, &lasts );
		val = strtok_r( NULL, sepstr, &lasts );
		if ( val != NULL )
		{
			++nfails;
			tet_printf("Unexpected return value %p", val);
		}

		(void) free(p);
	}
	else
	{
		++nerrs;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	NULL,	0	}
};

