
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/readdir_r
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)readdir_r.c	1.12
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<dirent.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	5

#ifdef  UNDEF_MACROS
#undef  readdir_r
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();
void	test5();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );
void	t4_f1( int *errors, int *fails );
void	t5_f1( int *errors, int *fails );

void	*t1_t1( void *arg );
void	*t2_t1( void *arg );
void	*t3_t1( void *arg );
void	*t4_t1( void *arg );
void	*t5_t1( void *arg );

typedef struct testentry {
	char	*name;
	ino_t	inode;
} TE, *PTE;

/* Quote taken from Kahil Gibran's book "The Prophet" */
static TE	Testentry[] = {
	{	"People",	0	},
	{	"of",		0	},
	{	"Orphalese",	0	},
	{	"the",		0	},
	{	"wind",		0	},
	{	"bids",		0	},
	{	"me",		0	},
	{	"leave",	0	},
	{	"you",		0	},
	{	".",		0	},
	{	"..",		0	},
	{	NULL,		0	}
};

static int	Nte = sizeof(Testentry) / sizeof(TE);

static int	mktf( char *dname );
static int	clntf( char *dname );
static int	goodentryP( int *map, char *name, ino_t inode );

#endif

void
startup()
{
	vsth_startup("readdir_r");
}

void
cleanup()
{
	vsth_cleanup("readdir_r");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the directory stream referenced by dirp is not at the end of");
	tet_infoline("the directory stream,");
	tet_infoline("then concurrent calls by multiple threads to");
	tet_infoline("readdir_r(dirp, entry, result) shall store information about the");
	tet_infoline("directory entry at the current position in the directory stream");
	tet_infoline("referenced by dirp in the location referenced by entry,");
	tet_infoline("store the value entry in the location referenced by result,");
	tet_infoline("position the directory stream at the next directory entry");
	tet_infoline("and return 0 if successful.");

	tet_infoline("Posix Ref: Component READDIR_R");
	tet_infoline("Assertion 9945-1:1996 5.1.2.2-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_NTHREADS	8
#define	T1_DIRNAME	"./t1_dir"

int			T1_flag;
long			T1_namemax;
pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;
VSTH_TSET		T1_tset[T1_NTHREADS];

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
	{
		return;
	}

	T1_namemax = pathconf( ".", _PC_NAME_MAX );
	if ( T1_namemax == -1 )
	{
		++*errors;
		tet_printf("pathconf unexpectedly failed, errno == %d", errno);
		return;
	}

	rval = mktf( T1_DIRNAME );
	if ( rval != 0 )
	{
		(void) clntf( T1_DIRNAME );
		return;
	}

	T1_flag = T1_NTHREADS;

	vsth_weave( &T1_tset[0], T1_NTHREADS, t1_t1, &T1_mx, &T1_cv,
			SCHED_RR, priomin, 0, errors, fails );

	(void) clntf( T1_DIRNAME );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	int		cnt;
	int		breakflag;
	int		*visits;
	void		*tval;
	struct dirent	*pentry;
	struct dirent	*result;
	DIR		*dirp;
	PTE		pte;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	visits = (int *) malloc( Nte * sizeof(int) );
	if ( visits != NULL  )
	{
		for ( cnt = 0; cnt < Nte; ++cnt )
			visits[cnt] = 0;
	}
	else
	{
		++nerrs;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
	}

	pentry = (struct dirent *) malloc((sizeof(struct dirent) + T1_namemax + 1) * sizeof(char));
	if ( pentry == NULL )
	{
		++nerrs;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
	}

	dirp = opendir( T1_DIRNAME );
	if ( dirp == NULL )
	{
		++nerrs;
		tet_printf("opendir unexpectedly failed, errno == %d", errno);
	}

	rval = vsth_bwait( &T1_cv, &T1_mx, &T1_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	if ( nerrs == 0 )
	{
		while (( rval = readdir_r( dirp, pentry, &result )) == 0 )
		{
			if ( result == pentry )
			{
				rval = goodentryP( visits, pentry->d_name, pentry->d_ino );
				if ( rval != 0 )
				{
					++nfails;
				}
			}
			else if ( result == NULL )
			{
				break;
			}
			else
			{
				++nfails;
				tet_printf("Expected result of %p, got %p", pentry, result);
			}
		}
	}

	if (( nerrs == 0 ) && ( nfails == 0 ))
	{
		/* make sure every entry was visited once */
		for ( cnt = 0, pte = &Testentry[0]; pte->name; ++cnt, ++pte )
		{
			if (( ! strcmp(pte->name, ".")) || ( ! strcmp( pte->name, ".." )))
				continue;
	
			if ( visits[cnt] != 1 )
			{
				++nfails;
	
				if ( visits[cnt] == 0 )
				{
					tet_printf("readdir_r did not read an entry for %s", pte->name);
				}
				else
				{
					tet_printf("readdir_r read multiple entries for %s", pte->name);
				}
			}
		}
	}

	if ( pentry )
	{
		(void) free(pentry);
	}

	if ( visits )
	{
		(void) free(visits);
	}

	if ( dirp )
	{
		(void) closedir(dirp);
	}

	if ( nfails )
	{
		tval = (void *) 2;
	}
	else if (( nerrs == 0 ) && ( nfails == 0 ))
	{
		tval = NULL;
	}

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the directory stream referenced by dirp is at the end of the");
	tet_infoline("directory stream,");
	tet_infoline("then concurrent calls by multiple threads to");
	tet_infoline("readdir_r(dirp, entry, result) shall store the value NULL in result.");

	tet_infoline("Posix Ref: Component READDIR_R");
	tet_infoline("Assertion 9945-1:1996 5.1.2.2-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T2_NTHREADS	8
#define	T2_DIRNAME	"./t2_dir"

int			T2_flag;
long			T2_namemax;
pthread_mutex_t		T2_mx;
pthread_cond_t		T2_cv;
VSTH_TSET		T2_tset[T2_NTHREADS];

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
	{
		return;
	}

	T2_namemax = pathconf( ".", _PC_NAME_MAX );
	if ( T2_namemax == -1 )
	{
		++*errors;
		tet_printf("pathconf unexpectedly failed, errno == %d", errno);
		return;
	}

	rval = mktf( T2_DIRNAME );
	if ( rval != 0 )
	{
		(void) clntf( T2_DIRNAME );
		return;
	}

	T2_flag = T2_NTHREADS;

	vsth_weave( &T2_tset[0], T2_NTHREADS, t2_t1, &T2_mx, &T2_cv,
			SCHED_RR, priomin, 0, errors, fails );

	(void) clntf( T2_DIRNAME );
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	int		cnt;
	int		breakflag;
	void		*tval;
	DIR		*dirp;
	struct dirent	*pentry;
	struct dirent	*result;
	PTE		pte;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	pentry = (struct dirent *) malloc((sizeof(struct dirent) + T2_namemax + 1) * sizeof(char));
	if ( pentry == NULL )
	{
		++nerrs;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
	}

	dirp = opendir( T2_DIRNAME );
	if ( dirp == NULL )
	{
		++nerrs;
		tet_printf("opendir unexpectedly failed, errno == %d", errno);
	}

	rval = vsth_bwait( &T2_cv, &T2_mx, &T2_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	if ( nerrs == 0 )
	{
		/* read each directory entry */
		for ( cnt = 0; cnt < Nte; ++cnt )
		{
			rval = readdir_r( dirp, pentry, &result );
			if ( rval != 0 )
			{
				vsth_surprise("readdir_r", 0, rval, &nerrs);
			}
		}
	}

	if ( nerrs == 0 )
	{
		rval = readdir_r( dirp, pentry, &result );
		if ( rval == 0 )
		{
			if ( result != NULL )
			{
				++nfails;
				tet_printf("Expected NULL result, got %p", NULL, result);
			}
		}
		else
		{
			vsth_surprise("readdir_r", 0, rval, &nerrs);
		}
	}

	if ( pentry )
	{
		(void) free(pentry);
	}

	if ( dirp )
	{
		(void) closedir(dirp);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("a call to readdir_r(dirp, entry, result) shall not return directory");
	tet_infoline("entries containing empty names.");

	tet_infoline("Posix Ref: Component READDIR_R");
	tet_infoline("Assertion 9945-1:1996 5.1.2.2-3(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T3_NTHREADS	8
#define	T3_DIRNAME	"./t3_dir"

int			T3_flag;
long			T3_namemax;
pthread_mutex_t		T3_mx;
pthread_cond_t		T3_cv;
VSTH_TSET		T3_tset[T3_NTHREADS];

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
	{
		return;
	}

	T3_namemax = pathconf( ".", _PC_NAME_MAX );
	if ( T3_namemax == -1 )
	{
		++*errors;
		tet_printf("pathconf unexpectedly failed, errno == %d", errno);
		return;
	}

	rval = mktf( T3_DIRNAME );
	if ( rval != 0 )
	{
		(void) clntf( T3_DIRNAME );
		return;
	}

	T3_flag = T3_NTHREADS;

	vsth_weave( &T3_tset[0], T3_NTHREADS, t3_t1, &T3_mx, &T3_cv,
			SCHED_RR, priomin, 0, errors, fails );

	(void) clntf( T3_DIRNAME );
}

void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	int		cnt;
	int		breakflag;
	void		*tval;
	struct dirent	*pentry;
	struct dirent	*result;
	DIR		*dirp;
	PTE		pte;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	pentry = (struct dirent *) malloc((sizeof(struct dirent) + T3_namemax + 1) * sizeof(char));
	if ( pentry == NULL )
	{
		++nerrs;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
	}

	dirp = opendir( T3_DIRNAME );
	if ( dirp == NULL )
	{
		++nerrs;
		tet_printf("opendir unexpectedly failed, errno == %d", errno);
	}

	rval = vsth_bwait( &T3_cv, &T3_mx, &T3_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	if ( nerrs == 0 )
	{
		while (( rval = readdir_r( dirp, pentry, &result )) == 0 )
		{
			if ( result == NULL )
			{
				break;
			}

			else if ( result == pentry )
			{
				if ( ! strcmp( pentry->d_name, "" ))
				{
					++nfails;
					tet_infoline("readdir_r unexpectedly returned an empty entry");
				}
			}
			else
			{
				++nerrs;
				tet_printf("Expected result == %p, got %p", pentry, result);
			}
		}
	}

	if ( pentry )
	{
		(void) free(pentry);
	}

	if ( dirp )
	{
		(void) closedir(dirp);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("concurrent calls by multiple threads to readdir_r(dirp, entry, result)");
	tet_infoline("shall mark for update the st_atime field of the directory each time");
	tet_infoline("the directory is actually read.");

	tet_infoline("Posix Ref: Component READDIR_R");
	tet_infoline("Assertion 9945-1:1996 5.1.2.2-4(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t4_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T4_NTHREADS	8
#define	T4_DIRNAME	"./t4_dir"

int			T4_flag;
long			T4_namemax;
pthread_mutex_t		T4_mx;
pthread_cond_t		T4_cv;
VSTH_TSET		T4_tset[T4_NTHREADS];

void
t4_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
	{
		return;
	}

	T4_namemax = pathconf( ".", _PC_NAME_MAX );
	if ( T4_namemax == -1 )
	{
		++*errors;
		tet_printf("pathconf unexpectedly failed, errno == %d", errno);
		return;
	}

	rval = mktf( T4_DIRNAME );
	if ( rval != 0 )
	{
		(void) clntf( T4_DIRNAME );
		return;
	}

	T4_flag = T4_NTHREADS;

	vsth_weave( &T4_tset[0], T4_NTHREADS, t4_t1, &T4_mx, &T4_cv,
			SCHED_RR, priomin, 0, errors, fails );

	(void) clntf( T4_DIRNAME );
}

void *
t4_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	int		cnt;
	int		breakflag;
	void		*tval;
	time_t		starttime;
	time_t		now;
	time_t		endtime;
	struct dirent	*pentry;
	struct dirent	*result;
	struct stat	sb;
	DIR		*dirp;
	PTE		pte;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	pentry = (struct dirent *) malloc((sizeof(struct dirent) + T4_namemax + 1) * sizeof(char));
	if ( pentry == NULL )
	{
		++nerrs;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
	}

	dirp = opendir( T4_DIRNAME );
	if ( dirp == NULL )
	{
		++nerrs;
		tet_printf("opendir unexpectedly failed, errno == %d", errno);
	}

	rval = vsth_bwait( &T4_cv, &T4_mx, &T4_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	if ( nerrs == 0 )
	{
		/* kill some time between the open and the read */
		endtime = time(NULL) + 2;
		while ( time(NULL) < endtime )
		{
			vsth_sched_yield();
		}

		rval = stat( T4_DIRNAME, &sb );
		if ( rval == 0 )
		{
			starttime = sb.st_atime;
		}
		else
		{
			vsth_surprise("stat", 0, rval, &nerrs);
		}

		rval = readdir_r( dirp, pentry, &result );
		if ( rval != 0 )
		{
			vsth_surprise("readdir_r", 0, rval, &nerrs);
		}

		rval = closedir( dirp );
		if ( rval != 0 )
		{
			vsth_surprise("closedir", 0, rval, &nerrs);
		}

		rval = stat( T4_DIRNAME, &sb );
		if ( rval == 0 )
		{
			endtime = sb.st_atime;
		}
		else
		{
			vsth_surprise("stat", 0, rval, &nerrs);
		}

		if (( sb.st_atime < starttime ) || ( endtime > sb.st_atime ))
		{
			++nfails;
			tet_printf("Expected st_atime between %ld and %ld, got %ld",
					starttime, endtime, sb.st_atime);
		}
	}

	if ( pentry )
	{
		(void) free(pentry);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test5()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when the implementation detects that the directory stream referenced");
	tet_infoline("by dirp does not refer to an open directory stream,");
	tet_infoline("then a call to readdir_r(dirp, entry, result) shall fail and");
	tet_infoline("return EBADF.");

	tet_infoline("Posix Ref: Component READDIR_R");
	tet_infoline("Assertion 9945-1:1996 5.1.2.3, 9945-1:1996 5.1.2.4-5(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	p = tet_getvar("VSTH_READDIR_R_EBADF");
	if ( p && *p )
	{
		if (( *p == 'Y' ) || ( *p == 'y' ))
		{
			/* TSD5TH.00055, TSD5TH.00060 
			t5_f1( &errors, &fails );
			result = VSTH_RESULT( errors, fails );
			*/
			tet_infoline("no portable test method");
			result = TET_UNTESTED;
		}
		else
		{
			tet_printf("VSTH_READDIR_R_EBADF is set to %s", p);
			result = TET_UNSUPPORTED;
		}
	}
	else
	{
		vsth_badconfig("VSTH_READDIR_R_EBADF", &errors);
		result = VSTH_RESULT( errors, fails );
	}

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T5_NTHREADS	8

int			T5_flag;
long			T5_namemax;
pthread_mutex_t		T5_mx;
pthread_cond_t		T5_cv;
VSTH_TSET		T5_tset[T5_NTHREADS];

void
t5_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
	{
		return;
	}

	T5_namemax = pathconf( ".", _PC_NAME_MAX );
	if ( T5_namemax == -1 )
	{
		++*errors;
		tet_printf("pathconf unexpectedly failed, errno == %d", errno);
		return;
	}

	T5_flag = T5_NTHREADS;

	vsth_weave( &T5_tset[0], T5_NTHREADS, t5_t1, &T5_mx, &T5_cv,
			SCHED_RR, priomin, 0, errors, fails );

}

void *
t5_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	int		cnt;
	int		breakflag;
	void		*tval;
	struct dirent	*pentry;
	struct dirent	*result;
	DIR		*dirp;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	pentry = (struct dirent *) malloc((sizeof(struct dirent) + T5_namemax + 1) * sizeof(char));
	if ( pentry == NULL )
	{
		++nerrs;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
	}

	dirp = opendir( "." );
	if ( dirp != NULL )
	{
		rval = closedir( dirp );
		if ( rval != 0 )
		{
			vsth_surprise("closedir", 0, rval, &nerrs);
		}
	}
	else
	{
		++nerrs;
		tet_printf("opendir unexpectedly failed, errno == %d", errno);
	}

	rval = vsth_bwait( &T5_cv, &T5_mx, &T5_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	if ( nerrs == 0 )
	{
		rval = readdir_r( dirp, pentry, &result );
		if ( rval != EBADF )
		{
			vsth_surprise("readdir_r", EBADF, rval, &nfails);
		}
	}

	if ( pentry )
	{
		(void) free(pentry);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

/* Create a test directory heirarchy */
int
mktf( char *dname )
{
	int		rval;
	int		nerrs;
	int		fd;
	char		*p;
	PTE		pte;
	struct stat	sb;

	nerrs = 0;

	rval = mkdir( dname, S_IRWXU | S_IRWXG | S_IRWXG );
	if ( rval == 0 )
	{
		rval = chdir( dname );
		if ( rval == 0 )
		{
			for ( pte = &Testentry[0]; pte->name; ++pte )
			{
				if (( ! strcmp( pte->name, "." )) || (! strcmp( pte->name, ".." )))
					continue;

				fd = open( pte->name, O_RDWR | O_CREAT, S_IRWXU | S_IRWXG );
				if ( fd >= 0 )
				{
					rval = write( fd, pte->name, strlen(pte->name) );
					if ( rval != strlen(pte->name))
					{
						vsth_surprise("write", strlen(pte->name), rval, &nerrs);
					}
	
					rval = fstat( fd, &sb );
					if ( rval == 0 )
					{
						pte->inode = sb.st_ino;
					}
					else
					{
						vsth_surprise("stat", 0, rval, &nerrs);
					}

					rval = close(fd);
					if ( rval != 0 )
					{
						vsth_surprise("close", 0, rval, &nerrs);
					}
				}
				else
				{
					++nerrs;
					tet_printf("open unexpectedly failed, errno == %d", errno);
				}
			}

			rval = chdir("..");
			if ( rval != 0 )
			{
				vsth_surprise("chdir", 0, rval, &nerrs);
			}
		}
		else
		{
			vsth_surprise("chdir", 0, rval, &nerrs);
		}
	}
	else
	{
		vsth_surprise("mkdir", 0, rval, &nerrs);
	}

	return(nerrs);
}

static int
clntf( char *dname )
{
	int	rval;
	int	nerrs;
	PTE	pte;

	nerrs = 0;

	rval = chdir( dname );
	if ( rval == 0 )
	{
		pte = &Testentry[0];
		for ( pte = &Testentry[0]; pte->name; ++pte )
		{
			if (( ! strcmp( pte->name, "." )) || ( ! strcmp( pte->name, ".." )))
				continue;

			rval = remove(pte->name);
			if ( rval != 0 )
			{
				vsth_surprise("remove", 0, rval, &nerrs);
			}
		}

		rval = chdir("..");
		if ( rval == 0 )
		{
			rval = rmdir( dname );
			if ( rval != 0 )
			{
				vsth_surprise("rmdir", 0, rval, &nerrs);
			}
		}
		else
		{
			vsth_surprise("chdir", 0, rval, &nerrs);
		}
	}
	else
	{
		vsth_surprise("chdir", 0, rval, &nerrs);
	}

	return(nerrs);
}

static int
goodentryP( int *map, char *name, ino_t inode )
{
	int			cnt;
	int			rval;
	int			nerrs;
	struct testentry	*pte;

	cnt	= 0;
	nerrs	= 0;
	pte	= &Testentry[0];

	while ( pte->name )
	{
		if ( ! strcmp( pte->name, name ) )
		{
			/* have we been here before ? */
			if ( map[cnt] )
			{

				++nerrs;
				tet_printf("Entry %s has already been visited", pte->name);
			}
			else
			{
				map[cnt] = 1;
			}

			if ( strcmp( name, "." ) && strcmp( name, ".." ) )
			{
				if ( pte->inode != inode )
				{
					++nerrs;
					tet_printf("Expected inode %d, got %d", pte->inode, inode);
				}
			}
			break;
		}

		++cnt;
		++pte;
	}

	if ( ! pte->name )
	{
		++nerrs;
		tet_printf("readdir_r returned an invalid name %s", name);
	}

	return(nerrs);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	test5,	5	},
	{	NULL,	0	}
};

