
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/getlogin_r
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)getlogin_r.c	1.12
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	2

#ifdef  UNDEF_MACROS
#undef  getlogin_r
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	VSTH_LOGIN_NAME		"vsx0"

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	*t1_t1( void *arg );
void	*t2_t1( void *arg );

#endif

void
startup()
{
	vsth_startup("getlogin_r");
}

void
cleanup()
{
	vsth_cleanup("getlogin_r");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("concurrent calls by multiple threads to getlogin_r(name, namesize)");
	tet_infoline("store the name associated by the login activity with the");
	tet_infoline("controlling terminal of the current process in the character");
	tet_infoline("array referenced by name and return 0 if successful.");

	tet_infoline("Posix Ref: Component GETLOGIN_R");
	tet_infoline("Assertion 9945-1:1996 4.2.4.2-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_T_NTHREADS	8

VSTH_TSET		T1_tset[T1_T_NTHREADS];

int			T1_flag;
long			T1_lnm;
pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
		return;

	T1_lnm = sysconf(_SC_LOGIN_NAME_MAX);
	if ( T1_lnm == -1 )
	{
		++*errors;
		tet_infoline("sysconf unexpectedly failed");
	}

	T1_flag = T1_T_NTHREADS;

	vsth_weave( &T1_tset[0], T1_T_NTHREADS, t1_t1, &T1_mx, &T1_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	char		*buf;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T1_cv, &T1_mx, &T1_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	buf = (char *) malloc( (T1_lnm + 1) * sizeof(char) );
	if ( buf )
	{
		rval = getlogin_r( buf, T1_lnm );
		if ( rval == 0 )
		{
			if ( strcmp( buf, VSTH_LOGIN_NAME ) )
			{
				++nfails;
				tet_printf("Unexpected login name %s, should be %s",
						buf, VSTH_LOGIN_NAME );
			}
		}
		else
		{
			vsth_surprise("getlogin_r", 0, rval, &nfails);
		}

		(void) free(buf);
	}
	else
	{
		++nerrs;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined");
	tet_infoline("and the implementation detects that the value of namesize is");
	tet_infoline("smaller than the length of the string to be stored in name:");
	tet_infoline("a call to getlogin_r(name, namesize) shall fail and return ERANGE.");

	tet_infoline("Posix Ref: Component GETLOGIN_R");
	tet_infoline("Assertion 9945-1:1996 4.2.4.4-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	p = tet_getvar("VSTH_GETLOGIN_R_ERANGE");
	if ( p && *p )
	{
		if (( *p == 'Y' ) || ( *p == 'y' ))
		{
			t2_f1( &errors, &fails );
			result = VSTH_RESULT( errors, fails );
		}
		else
		{
			tet_printf("VSTH_GETLOGIN_R_ERANGE is set to %s", p);
			result = TET_UNSUPPORTED;
		}
	}
	else
	{
		vsth_badconfig("VSTH_GETLOGIN_R_ERANGE", &errors);
		result = VSTH_RESULT( errors, fails );
	}

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T2_T_NTHREADS	8

VSTH_TSET		T2_tset[T2_T_NTHREADS];

int			T2_flag;
long			T2_lnm;
pthread_mutex_t		T2_mx;
pthread_cond_t		T2_cv;

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
		return;

	T2_lnm = sysconf(_SC_LOGIN_NAME_MAX);
	if ( T2_lnm == -1 )
	{
		++*errors;
		tet_printf("sysconf unexpectedly failed, errno == %d", errno);
		return;
	}

	T2_flag = T2_T_NTHREADS;

	vsth_weave( &T2_tset[0], T2_T_NTHREADS, t2_t1, &T2_mx, &T2_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	char		*buf;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T2_cv, &T2_mx, &T2_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	buf = (char *) malloc( (T2_lnm + 1) * sizeof(char) );
	if ( buf )
	{
		rval = getlogin_r( buf, 2 );
		if ( rval != ERANGE )
		{
			vsth_surprise("getlogin_r", ERANGE, rval, &nfails);
		}

		(void) free(buf);
	}
	else
	{
		++nerrs;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	NULL,	0	}
};

