
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/sysconf
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)sysconf.c	1.16
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<limits.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	12

#ifdef  UNDEF_MACROS
#undef  sysconf
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();
void	test5();
void	test6();
void	test7();
void	test8();
void	test9();
void	test10();
void	test11();
void	test12();

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );
void	t4_f1( int *errors, int *fails );
void	t5_f1( int *errors, int *fails );
void	t6_f1( int *errors, int *fails );
void	t7_f1( int *errors, int *fails );
void	t8_f1( int *errors, int *fails );
void	t9_f1( int *errors, int *fails );
void	t10_f1( int *errors, int *fails );
void	t11_f1( int *errors, int *fails );
void	t12_f1( int *errors, int *fails );

void
startup()
{
	vsth_startup("sysconf");
}

void
cleanup()
{
	vsth_cleanup("sysconf");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to sysconf(_SC_THREAD_DESTRUCTOR_ITERATIONS) returns a");
	tet_infoline("value for the system variable PTHREAD_DESTRUCTOR_ITERATIONS");
	tet_infoline("which is not more restrictive than the value");
	tet_infoline("_POSIX_THREAD_DESTRUCTOR_ITERATIONS.");

	tet_infoline("Posix Ref: Component SYSCONF");
	tet_infoline("Assertion 9945-1:1996 4.8.1.2, 9945-1:1996 4.8.1.3-1(A)");

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t1_f1( int *errors, int *fails )
{
	long		lval;

	errno = 0;

	lval = sysconf(_SC_THREAD_DESTRUCTOR_ITERATIONS);
	if ( lval < _POSIX_THREAD_DESTRUCTOR_ITERATIONS )
	{
		if (( lval == -1 ) && ( errno == 0 ))
		{
			tet_infoline("The implementation imposes no deterministic limit");
		}
		else
		{
			++*fails;
			tet_printf("Unexpected value for _SC_THREAD_DESTRUCTOR_ITERATIONS %ld", lval );
		}
	}
}

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to sysconf(_SC_THREAD_KEYS_MAX) returns a value");
	tet_infoline("for the system variable PTHREAD_KEYS_MAX which is not more");
	tet_infoline("restrictive than the value _POSIX_THREAD_KEYS_MAX.");

	tet_infoline("Posix Ref: Component SYSCONF");
	tet_infoline("Assertion 9945-1:1996 4.8.1.2-2(A)");

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t2_f1( int *errors, int *fails )
{
	long		lval;

	errno = 0;

	lval = sysconf(_SC_THREAD_KEYS_MAX);
	if ( lval < _POSIX_THREAD_KEYS_MAX )
	{
		if (( lval == -1 ) && ( errno == 0 ))
		{
			tet_infoline("The implementation imposes no deterministic limit");
		}
		else
		{
			++*fails;
			tet_printf("Unexpected value for _SC_THREAD_KEYS_MAX %ld", lval );
		}
	}
}

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to sysconf(_SC_THREAD_STACK_MIN) returns a value");
	tet_infoline("for the system variable PTHREAD_STACK_MIN which is not more");
	tet_infoline("restrictive than the value 0.");

	tet_infoline("Posix Ref: Component SYSCONF");
	tet_infoline("Assertion 9945-1:1996 4.8.1.2-3(A)");

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t3_f1( int *errors, int *fails )
{
	long		lval;

	lval = sysconf(_SC_THREAD_STACK_MIN);
	if ( lval < 0 )
	{
		++*fails;
		tet_printf("Unexpected value for _SC_THREAD_STACK_MIN %ld", lval );
	}
}

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to sysconf(_SC_THREAD_THREADS_MAX) returns a value");
	tet_infoline("for the system variable PTHREAD_THREADS_MAX which is not");
	tet_infoline("more restrictive than the value _POSIX_THREAD_THREADS_MAX.");

	tet_infoline("Posix Ref: Component SYSCONF");
	tet_infoline("Assertion 9945-1:1996 4.8.1.2-4(A)");

	t4_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t4_f1( int *errors, int *fails )
{
	long		lval;

	errno = 0;

	lval = sysconf(_SC_THREAD_THREADS_MAX);
	if ( lval < _POSIX_THREAD_THREADS_MAX )
	{
		if (( lval == -1 ) && ( errno == 0 ))
		{
			tet_infoline("The implementation imposes no deterministic limit");
		}
		else
		{
			++*fails;
			tet_printf("Unexpected value for _SC_THREAD_THREADS_MAX %ld", lval );
		}
	}
}

void
test5()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to sysconf(_SC_THREADS) returns the value");
	tet_infoline("of the system variable _POSIX_THREADS.");

	tet_infoline("Posix Ref: Component SYSCONF");
	tet_infoline("Assertion 9945-1:1996 4.8.1.2-5(A)");

	t5_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t5_f1( int *errors, int *fails )
{
	long		lval;

	lval = sysconf(_SC_THREADS);
	if ( lval == -1 )
	{
		++*fails;
		tet_printf("Unexpected value for _SC_THREADS %ld", lval );
	}
}

void
test6()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to sysconf(_SC_THREAD_ATTR_STACKADDR) returns the value");
	tet_infoline("of the system variable _POSIX_THREAD_ATTR_STACKADDR.");

	tet_infoline("Posix Ref: Component SYSCONF");
	tet_infoline("Assertion 9945-1:1996 4.8.1.2-6(A)");

	t6_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t6_f1( int *errors, int *fails )
{
	long		lval;

	lval = sysconf(_SC_THREAD_ATTR_STACKADDR);

#ifdef	_POSIX_THREAD_ATTR_STACKADDR
	if ( lval == -1 )
#else
	if ( lval != -1 )
#endif
	{
		++*fails;
		tet_printf("Unexpected value for _SC_THREAD_ATTR_STACKADDR %ld", lval );
	}
}

void
test7()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to sysconf(_SC_THREAD_ATTR_STACKSIZE) returns the value");
	tet_infoline("of the system variable _POSIX_THREAD_ATTR_STACKSIZE.");

	tet_infoline("Posix Ref: Component SYSCONF");
	tet_infoline("Assertion 9945-1:1996 4.8.1.2-7(A)");

	t7_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t7_f1( int *errors, int *fails )
{
	long		lval;

	lval = sysconf(_SC_THREAD_ATTR_STACKSIZE);

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE
	if ( lval == -1 )
#else
	if ( lval != -1 )
#endif
	{
		++*fails;
		tet_printf("Unexpected value for _SC_THREAD_ATTR_STACKSIZE %ld", lval );
	}
}

void
test8()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to sysconf(_SC_THREAD_PRIORITY_SCHEDULING) returns the value");
	tet_infoline("of the system variable _POSIX_THREAD_PRIORITY_SCHEDULING.");

	tet_infoline("Posix Ref: Component SYSCONF");
	tet_infoline("Assertion 9945-1:1996 4.8.1.2-8(A)");

	t8_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t8_f1( int *errors, int *fails )
{
	long		lval;

	lval = sysconf(_SC_THREAD_PRIORITY_SCHEDULING);

#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING
	if ( lval == -1 )
#else
	if ( lval != -1 )
#endif
	{
		++*fails;
		tet_printf("Unexpected value for _SC_THREAD_ATTR_PRIORITY_SCHEDULING %ld",
				lval );
	}
}

void
test9()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to sysconf(_SC_THREAD_PRIO_INHERIT) returns the value");
	tet_infoline("of the system variable _POSIX_THREAD_PRIO_INHERIT.");

	tet_infoline("Posix Ref: Component SYSCONF");
	tet_infoline("Assertion 9945-1:1996 4.8.1.2-9(A)");

	t9_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t9_f1( int *errors, int *fails )
{
	long		lval;

	lval = sysconf(_SC_THREAD_PRIO_INHERIT);

#ifdef	_POSIX_THREAD_PRIO_INHERIT
	if ( lval == -1 )
#else
	if ( lval != -1 )
#endif
	{
		++*fails;
		tet_printf("Unexpected value for _SC_THREAD_PRIO_INHERIT %ld",
				lval );
	}
}

void
test10()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to sysconf(_SC_THREAD_PRIO_PROTECT) returns the value");
	tet_infoline("of the system variable _POSIX_THREAD_PRIO_PROTECT.");

	tet_infoline("Posix Ref: Component SYSCONF");
	tet_infoline("Assertion 9945-1:1996 4.8.1.2-10(A)");

	t10_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t10_f1( int *errors, int *fails )
{
	long		lval;

	lval = sysconf(_SC_THREAD_PRIO_PROTECT);

#ifdef	_POSIX_THREAD_PRIO_PROTECT
	if ( lval == -1 )
#else
	if ( lval != -1 )
#endif
	{
		++*fails;
		tet_printf("Unexpected value for _SC_THREAD_PRIO_PROTECT %ld",
				lval );
	}
}

void
test11()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to sysconf(_SC_THREAD_PROCESS_SHARED) returns the value");
	tet_infoline("of the system variable _POSIX_THREAD_PROCESS_SHARED.");

	tet_infoline("Posix Ref: Component SYSCONF");
	tet_infoline("Assertion 9945-1:1996 4.8.1.2-11(A)");

	t11_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t11_f1( int *errors, int *fails )
{
	long		lval;

	lval = sysconf(_SC_THREAD_PROCESS_SHARED);

#ifdef	_POSIX_THREAD_PROCESS_SHARED
	if ( lval == -1 )
#else
	if ( lval != -1 )
#endif
	{
		++*fails;
		tet_printf("Unexpected value for _SC_THREAD_PROCESS_SHARED %ld",
				lval );
	}

}

void
test12()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to sysconf(_SC_THREAD_SAFE_FUNCTIONS) returns the value");
	tet_infoline("of the system variable _POSIX_THREAD_SAFE_FUNCTIONS.");

	tet_infoline("Posix Ref: Component SYSCONF");
	tet_infoline("Assertion 9945-1:1996 4.8.1.2-12(A)");

	t12_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t12_f1( int *errors, int *fails )
{
	long		lval;

	lval = sysconf(_SC_THREAD_SAFE_FUNCTIONS);

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS
	if ( lval == -1 )
#else
	if ( lval != -1 )
#endif
	{
		++*fails;
		tet_printf("Unexpected value for _SC_THREAD_SAFE_FUNCTIONS %ld",
				lval );
	}

}

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	test5,	5	},
	{	test6,	6	},
	{	test7,	7	},
	{	test8,	8	},
	{	test9,	9	},
	{	test10,	10	},
	{	test11,	11	},
	{	test12,	12	},
	{	NULL,	0	}
};

