
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/ttyname_r
*
*	The Open Group, VSTH Threads test suite
*	@(#)ttyname_r.c	1.13
*
*	HISTORY
*	TSD5TH.00073, test 4, make this conditional based on
*	the parameter VSTH_TTYNAME_R_ERANGE
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	4

#ifdef  UNDEF_MACROS
#undef  ttyname_r
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );
void	t4_f1( int *errors, int *fails );

void	*t1_t1( void *arg );
void	*t2_t1( void *arg );
void	*t3_t1( void *arg );
void	*t4_t1( void *arg );

#endif

void
startup()
{
	vsth_startup("ttyname_r");
}

void
cleanup()
{
	vsth_cleanup("ttyname_r");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("concurrent calls by multiple threads to");
	tet_infoline("ttyname_r(fildes, name, namesize)");
	tet_infoline("shall store the null terminated pathname of the terminal associated");
	tet_infoline("with the file descriptor fildes in the character array referenced by");
	tet_infoline("name and return 0 if successful.");

	tet_infoline("Posix Ref: Component TTYNAME_R");
	tet_infoline("Assertion 9945-1:1996 4.7.2.2-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_T_NTHREADS	8

int			T1_tty_name_max;
int			T1_flag;
char			*T1_vsx_ttyname;
pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;
VSTH_TSET		T1_tset[T1_T_NTHREADS];

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors );
	if ( *errors )
		return;

	T1_tty_name_max = sysconf( _SC_TTY_NAME_MAX );
	if ( T1_tty_name_max == -1 )
	{
		++*errors;
		tet_printf("sysconf unexpectedly failed");
		return;
	}

	T1_vsx_ttyname = tet_getvar("VSX_TTYNAME");
	if (( ! T1_vsx_ttyname ) || ( ! *T1_vsx_ttyname ))
	{
		vsth_badconfig("VSX_TTYNAME", errors);
		return;
	}

	T1_flag = T1_T_NTHREADS;

	vsth_weave( &T1_tset[0], T1_T_NTHREADS, t1_t1, &T1_mx, &T1_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	char		*buf;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T1_cv, &T1_mx, &T1_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	buf = (char *) malloc( (T1_tty_name_max + 1) * sizeof(char) );
	if ( buf != NULL )
	{
		rval = ttyname_r( STDIN_FILENO, buf, T1_tty_name_max );
		if ( rval == 0 )
		{
			if ( strcmp( T1_vsx_ttyname, buf ) )
			{
				++nfails;
				tet_printf("Expected ttyname %s, got %s", T1_vsx_ttyname, buf);
			}
		}
		else
		{
			vsth_surprise("ttyname_r", 0, rval, &nfails);
		}

		(void) free(buf);
	}
	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the implementation detects that fildes is not a valid file");
	tet_infoline("descriptor: a call to ttyname_r(fildes, name, namesize) shall fail");
	tet_infoline("and return EBADF.");

	tet_infoline("Posix Ref: Component TTYNAME_R");
	tet_infoline("Assertion 9945-1:1996 4.7.2.4-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T2_T_NTHREADS	8
#define	T2_FILENAME	"./t2_file"

int			T2_tty_name_max;
int			T2_flag;
pthread_mutex_t		T2_mx;
pthread_cond_t		T2_cv;
VSTH_TSET		T2_tset[T2_T_NTHREADS];

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors );
	if ( *errors )
		return;

	T2_tty_name_max = sysconf( _SC_TTY_NAME_MAX );
	if ( T2_tty_name_max == -1 )
	{
		++*errors;
		tet_printf("sysconf unexpectedly failed");
		return;
	}

	T2_flag = T2_T_NTHREADS;

	vsth_weave( &T2_tset[0], T2_T_NTHREADS, t2_t1, &T2_mx, &T2_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	int		fd;
	char		*buf;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T2_cv, &T2_mx, &T2_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	buf = (char *) malloc( (T2_tty_name_max + 1) * sizeof(char) );
	if ( buf != NULL )
	{
		fd = open( T2_FILENAME, O_RDWR | O_CREAT, S_IRWXU | S_IRWXG );
		if ( fd >= 0 )
		{
			rval = close( fd );
			if ( rval == 0 )
			{
				rval = ttyname_r( fd, buf, T2_tty_name_max );
				if ( rval != EBADF )
				{
					vsth_surprise("ttyname_r", EBADF, rval, &nfails);
				}
			}
			else
			{
				vsth_surprise("close", 0, rval, &nerrs);
			}

			(void) remove(T2_FILENAME);
		}
		else
		{
			++nerrs;
			tet_printf("open unexpectedly failed, errno == %d", errno);
		}

		(void) free(buf);
	}
	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the implementation detects that fildes does not refer to a tty:");
	tet_infoline("a call to ttyname_r(fildes, name, namesize) shall fail and");
	tet_infoline("return ENOTTY.");

	tet_infoline("Posix Ref: Component TTYNAME_R");
	tet_infoline("Assertion 9945-1:1996 4.7.2.4-3(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T3_T_NTHREADS	8
#define	T3_FILENAME	"./t3_file"

int			T3_tty_name_max;
int			T3_flag;
pthread_mutex_t		T3_mx;
pthread_cond_t		T3_cv;
VSTH_TSET		T3_tset[T3_T_NTHREADS];

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors );
	if ( *errors )
		return;

	T3_tty_name_max = sysconf( _SC_TTY_NAME_MAX );
	if ( T3_tty_name_max == -1 )
	{
		++*errors;
		tet_printf("sysconf unexpectedly failed");
		return;
	}

	T3_flag = T3_T_NTHREADS;

	vsth_weave( &T3_tset[0], T3_T_NTHREADS, t3_t1, &T3_mx, &T3_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		fd;
	int		nfails;
	void		*tval;
	char		*buf;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T3_cv, &T3_mx, &T3_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	buf = (char *) malloc( (T3_tty_name_max + 1) * sizeof(char) );
	if ( buf != NULL )
	{
		fd = open( T3_FILENAME, O_RDWR | O_CREAT, S_IRWXU | S_IRWXG );
		if ( fd >= 0 )
		{
			rval = ttyname_r( fd, buf, T3_tty_name_max );
			if ( rval != ENOTTY )
			{
				vsth_surprise("ttyname_r", ENOTTY, rval, &nfails);
			}

			(void) close(fd);
			(void) remove(T3_FILENAME);
		}

		(void) free(buf);
	}
	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;
#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS
	char		*p;
#endif

	errors		= 0;
	fails		= 0;

	tet_infoline("If the implementation detects that the value of namesize is smaller");
	tet_infoline("than the length of the string to be returned, including the");
	tet_infoline("terminating null character:");
	tet_infoline("a call to ttyname_r(fildes, name, namesize) shall fail and");
	tet_infoline("return ERANGE.");

	tet_infoline("Posix Ref: Component TTYNAME_R");
	tet_infoline("Assertion 9945-1:1996 4.7.2.4-4(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	/* TSD5TH.00073 */
	p = tet_getvar("VSTH_TTYNAME_R_ERANGE");
	if ( p && *p )
	{
		if ((*p == 'Y') || (*p == 'y'))
		{
			t4_f1( &errors, &fails );
			result = VSTH_RESULT( errors, fails );
		}
		else
		{
			tet_printf("VSTH_TTYNAME_R_ERANGE is set to %s", p);
			result = TET_UNSUPPORTED;
		}
	}
	else
	{
		vsth_badconfig("VSTH_TTYNAME_R_ERANGE", &errors);
		result = VSTH_RESULT( errors, fails );
	}


#else

	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T4_T_NTHREADS	8
#define	T4_FILENAME	"./t4_file"

int			T4_tty_name_max;
int			T4_flag;
pthread_mutex_t		T4_mx;
pthread_cond_t		T4_cv;
VSTH_TSET		T4_tset[T4_T_NTHREADS];

void
t4_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors );
	if ( *errors )
		return;

	T4_tty_name_max = sysconf( _SC_TTY_NAME_MAX );
	if ( T4_tty_name_max == -1 )
	{
		++*errors;
		tet_printf("sysconf unexpectedly failed");
		return;
	}

	T4_flag = T4_T_NTHREADS;

	vsth_weave( &T4_tset[0], T4_T_NTHREADS, t4_t1, &T4_mx, &T4_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t4_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	char		*buf;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T4_cv, &T4_mx, &T4_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	buf = (char *) malloc( (T4_tty_name_max + 1) * sizeof(char) );
	if ( buf != NULL )
	{
		rval = ttyname_r( STDIN_FILENO, buf, 1 );
		if ( rval != ERANGE )
		{
			vsth_surprise("ttyname_r", ERANGE, rval, &nfails);
		}

		(void) free(buf);
	}
	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	NULL,	0	}
};

