
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_atfork
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_atfork.c	1.18
*
*	History:
*	Moved test 2 to a separate executable.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	7

#define	T2_PATH		"pthread_atfork_exectests"

#ifdef  UNDEF_MACROS
#undef  pthread_atfork
#endif

int	no_tests =	NO_TESTS;

extern	char	**environ;
extern	char	*itoa();

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();
void	test5();
void	test6();
void	test7();

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );
void	t4_f1( int *errors, int *fails );
void	t5_f1( int *errors, int *fails );

void	t1_c1();
void	t1_prepare();
void	t1_parent();
void	t1_child();
void	t2_c1();
void	t3_c1();
void	t3_parent1();
void	t3_parent2();
void	t4_c1();
void	t4_child1();
void	t4_child2();
void	t4_c1();
void	t4_parent1();
void	t4_parent2();
void	t5_prepare1();
void	t5_prepare2();
void	t5_c1();

void
startup()
{
	vsth_startup("pthread_atfork");
}

void
cleanup()
{
	vsth_cleanup("pthread_atfork");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_atfork(prepare, parent, child) declares fork");
	tet_infoline("handlers to be called before and after fork(), in the context");
	tet_infoline("of the thread that called fork() and returns 0 upon success.");
	tet_infoline("The prepare fork handler is called before fork() processing");
	tet_infoline("commences. The parent fork handler is called after fork");
	tet_infoline("processing completes in the parent process.");
	tet_infoline("The child fork handler is called after fork() processing");
	tet_infoline("completes in the child process.");

	tet_infoline("Posix Ref: Component PTHREAD_ATFORK");
	tet_infoline("Assertion 9945-1:1996 3.1.3.2, 9945-1:1996 3.1.3.3-1(A)");

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

/******	Register fork handlers, fork and ensure that they were executed.
	Unfortunately, we have no portable way of discerning the precise
	timing of execution of the fork handlers. Ideas ?		******/

int	T1_prepareFlag;
int	T1_parentFlag;
int	T1_childFlag;

void
t1_f1( int *errors, int *fails )
{
	int	rval;

	T1_prepareFlag	= 0;
	T1_parentFlag	= 0;
	T1_childFlag	= 0;

	rval = pthread_atfork( t1_prepare, t1_parent, t1_child );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_atfork", 0, rval, fails);
		return;
	}

	rval = tet_fork( t1_c1, NULL, VSTH_TIMEOUT / 2, 0 | 1 | 2 | 3 | 4 );

	switch (rval) {
		case 0:
			break;
		case 1:
			vsth_surprise("tet_fork", 0, rval, errors);
			break;
		case 2:
			tet_infoline("Prepare routine not called in child");
			vsth_surprise("tet_fork", 0, rval, fails);
			break;
		case 3:
			tet_infoline("Parent routine was called in child");
			vsth_surprise("tet_fork", 0, rval, fails);
			break;
		case 4:
			tet_infoline("Child routine not called in child");
			vsth_surprise("tet_fork", 0, rval, fails);
			break;
		default:
			vsth_surprise("tet_fork", 0, rval, errors);
			break;
	}

	if ( T1_prepareFlag != 1 )
	{
		vsth_surprise("T1_prepareFlag", 1, T1_prepareFlag, fails);
	}

	if ( T1_parentFlag != 1 )
	{
		vsth_surprise("T1_parentFlag", 1, T1_parentFlag, fails);
	}

	if ( T1_childFlag != 0 )
	{
		vsth_surprise("T1_childFlag", 0, T1_childFlag, fails);
	}
}

void
t1_c1()
{
	int	rval;
	int	nerrs;
	int	nfails;

	nerrs	= 0;
	nfails	= 0;
	rval	= 1;

	if ( T1_prepareFlag != 1 )
	{
		_exit(2);
	}

	if ( T1_parentFlag != 0 )
	{
		_exit(3);
	}

	if ( T1_childFlag != 1 )
	{
		_exit(4);
	}

	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;

	_exit(rval);
}

void
t1_prepare()
{
	++T1_prepareFlag;
}

void
t1_parent()
{
	++T1_parentFlag;
}

void
t1_child()
{
	++T1_childFlag;
}

int	T2_flag;
int	T2_prepareFlag;
int	T2_parentFlag;
int	T2_childFlag;

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When prepare, parent or child are NULL,");
	tet_infoline("then a call to pthread_atfork(prepare, parent, child)");
	tet_infoline("shall not install fork handlers for the corresponding");
	tet_infoline("phase of fork processing.");

	tet_infoline("Posix Ref: Component PTHREAD_ATFORK");
	tet_infoline("Assertion 9945-1:1996 3.1.3.2-2(A)");

	T2_flag = 0;
	t2_f1( &errors, &fails );

	T2_flag = 1;
	t2_f1( &errors, &fails );

	T2_flag = 2;
	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t2_f1( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork( t2_c1, NULL, VSTH_TIMEOUT / 2, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
}

void
t2_c1()
{
	char		*argv[3];
	char		ascii_T2Flag[20];

	argv[0] = execpath(T2_PATH);
	argv[1] = strcpy(ascii_T2Flag, itoa(T2_flag));
	argv[2] = NULL;

	(void) tet_exec(execpath(T2_PATH), argv, environ);

	_exit(1);
}

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("The parent fork handlers are called in the order in which they");
	tet_infoline("were established by calls to pthread_atfork().");

	tet_infoline("Posix Ref: Component PTHREAD_ATFORK");
	tet_infoline("Assertion 9945-1:1996 3.1.3.2-3(A)");

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int	T3_flag;

void
t3_f1( int *errors, int *fails )
{
	int	rval;

	T3_flag	= 0;

	rval = pthread_atfork( NULL, t3_parent1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_atfork", 0, rval, errors);
	}

	rval = pthread_atfork( NULL, t3_parent2, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_atfork", 0, rval, errors);
	}

	rval = tet_fork( t3_c1, NULL, VSTH_TIMEOUT / 2, 0 );
	if ( rval == 1 )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
	else if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}

	if ( T3_flag != 2 )
	{
		vsth_surprise("T3_flag", 2, T3_flag, fails);
	}
}

void
t3_c1()
{
	_exit(0);
}

void
t3_parent1()
{
	if ( T3_flag == 0 )
		T3_flag = 1;
}

void
t3_parent2()
{
	if ( T3_flag == 1 )
		T3_flag = 2;
}

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("The child fork handlers are called in the order in which they");
	tet_infoline("were established by calls to pthread_atfork().");

	tet_infoline("Posix Ref: Component PTHREAD_ATFORK");
	tet_infoline("Assertion 9945-1:1996 3.1.3.2-4(A)");

	t4_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int	T4_flag;

void
t4_f1( int *errors, int *fails )
{
	int	rval;

	T4_flag	= 0;

	rval = pthread_atfork( NULL, NULL, t4_child1 );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_atfork", 0, rval, errors);
	}

	rval = pthread_atfork( NULL, NULL, t4_child2 );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_atfork", 0, rval, errors);
	}

	rval = tet_fork( t4_c1, NULL, VSTH_TIMEOUT / 2, 0 | 1 | 2 );
	if ( rval == 1 )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
	else if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
}

void
t4_c1()
{
	if ( T4_flag != 2 )
	{
		_exit(2);
	}
	else
	{
		_exit(0);
	}
}

void
t4_child1()
{
	if ( T4_flag == 0 )
		T4_flag = 1;
}

void
t4_child2()
{
	if ( T4_flag == 1 )
		T4_flag = 2;
}

void
test5()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("The prepare fork handlers are called in the opposite order to");
	tet_infoline("which they were established by calls to pthread_atfork().");

	tet_infoline("Posix Ref: Component PTHREAD_ATFORK");
	tet_infoline("Assertion 9945-1:1996 3.1.3.2-5(A)");

	t5_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int	T5_flag;

void
t5_f1( int *errors, int *fails )
{
	int	rval;

	T5_flag	= 0;

	rval = pthread_atfork( t5_prepare1, NULL, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_atfork", 0, rval, errors);
	}

	rval = pthread_atfork( t5_prepare2, NULL, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_atfork", 0, rval, errors);
	}

	rval = tet_fork( t5_c1, NULL, VSTH_TIMEOUT / 2, 0 );
	if ( rval == 1 )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
		return;
	}
	else if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
		return;
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
		return;
	}
	if ( T5_flag != 1 )
	{
		vsth_surprise("T5_flag", 2, T5_flag, fails);
	}
}

void
t5_c1()
{
	_exit(0);
}

void
t5_prepare1()
{
	if ( T5_flag == 2 )
		T5_flag = 1;
}

void
t5_prepare2()
{
	if ( T5_flag == 0 )
		T5_flag = 2;
}

void
test6()
{

	tet_infoline("When insufficient table space exists to record the");
	tet_infoline("fork handler addresses,");
	tet_infoline("then a call to pthread_atfork() function shall fail");
	tet_infoline("and return ENOMEM.");

	tet_infoline("Posix Ref: Component PTHREAD_ATFORK");
	tet_infoline("Assertion 9945-1:1996 3.1.3.4-6(A)");

	tet_infoline("No portable test");
	tet_result(TET_UNTESTED);
}

int		t7_tfunc();

VSTH_RT		T7_et = {
	NULL,	t7_tfunc,	NULL
};

void
test7()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_atfork() shall not return a value of EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_ATFORK");
	tet_infoline("Assertion 9945-1:1996  2.4-7(A)");

	vsth_et( &T7_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int
t7_tfunc()
{
	int	val;
	int	rval;

	rval = 1;

	val = pthread_atfork( NULL, NULL, NULL );
	if ( val != EINTR )
		rval = 0;

	return(rval);
}

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	test5,	5	},
	{	test6,	6	},
	{	test7,	7	},
	{	NULL,	0	}
};


