
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_atfork_exectests
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_atfork_exectests.c	1.1
*
*	History:
*	Created separate executable to avoid calls to non-asynch safe functions.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#ifdef  UNDEF_MACROS
#undef  pthread_atfork
#endif

void	t2_c1();
void	t2_c2();
void	t2_prepare();
void	t2_parent();
void	t2_child();

int	T2_flag;
int	T2_prepareFlag;
int	T2_parentFlag;
int	T2_childFlag;


int	tet_nosigreset = 0;

int
tet_main( int argc, char **argv)
{

	if (argc >= 1)
	{
		T2_flag = atoi(argv[1]);
		t2_c1();
	}
	else
		tet_printf("Incorrect number of arguments to exectests");

	exit(1);
}

void
t2_c1()
{
	int	rval;
	int	nerrs;
	int	nfails;

	nerrs	= 0;
	nfails	= 0;

	T2_prepareFlag	= 0;
	T2_parentFlag	= 0;
	T2_childFlag	= 0;

	if ( T2_flag == 0 )
	{
		rval = pthread_atfork( NULL, t2_parent, t2_child );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_atfork", 0, rval, NULL);
			exit(1);
		}
	}

	else if ( T2_flag == 1 )
	{
		rval = pthread_atfork( t2_prepare, NULL, t2_child );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_atfork", 0, rval, NULL);
			exit(1);
		}
	}

	else if ( T2_flag == 2 )
	{
		rval = pthread_atfork( t2_prepare, t2_parent, NULL );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_atfork", 0, rval, NULL);
			exit(1);
		}
	}

	rval = tet_fork( t2_c2, NULL, VSTH_TIMEOUT / 2, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, &nfails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, &nerrs);
	}

	if ( T2_flag == 0 )
	{
		if ( T2_prepareFlag != 0 )
		{
			vsth_surprise("Case 0P: T2_prepareFlag", 0, T2_prepareFlag, &nfails);
		}
	
		if ( T2_parentFlag != 1 )
		{
			vsth_surprise("Case 0P: T2_parentFlag", 1, T2_parentFlag, &nfails);
		}
	
		if ( T2_childFlag != 0 )
		{
			vsth_surprise("Case 0P: T2_childFlag", 0, T2_childFlag, &nfails);
		}
	}
	else if ( T2_flag == 1 )
	{
		if ( T2_prepareFlag != 1 )
		{
			vsth_surprise("Case 1P: T2_prepareFlag", 1, T2_prepareFlag, &nfails);
		}
	
		if ( T2_parentFlag != 0 )
		{
			vsth_surprise("Case 1P: T2_parentFlag", 0, T2_parentFlag, &nfails);
		}
	
		if ( T2_childFlag != 0 )
		{
			vsth_surprise("Case 1P: T2_childFlag", 0, T2_childFlag, &nfails);
		}
	}
	else if ( T2_flag == 2 )
	{
		if ( T2_prepareFlag != 1 )
		{
			vsth_surprise("Case 2P: T2_prepareFlag", 1, T2_prepareFlag, &nfails);
		}
	
		if ( T2_parentFlag != 1 )
		{
			vsth_surprise("Case 2P: T2_parentFlag", 1, T2_parentFlag, &nfails);
		}
	
		if ( T2_childFlag != 0 )
		{
			vsth_surprise("Case 2P: T2_childFlag", 0, T2_childFlag, &nfails);
		}
	}

	rval = 1;

	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;

	exit(rval);
}

void
t2_c2()
{
	int	rval;
	int	nerrs;
	int	nfails;

	nerrs	= 0;
	nfails	= 0;

	if ( T2_flag == 0 )
	{
		if ( T2_prepareFlag != 0 )
		{
			vsth_surprise("Case 0C: T2_prepareFlag", 0, T2_prepareFlag, &nfails);
		}
	
		if ( T2_parentFlag != 0 )
		{
			vsth_surprise("Case 0C: T2_parentFlag", 0, T2_parentFlag, &nfails);
		}
	
		if ( T2_childFlag != 1 )
		{
			vsth_surprise("Case 0C: T2_childFlag", 1, T2_childFlag, &nfails);
		}
	}
	else if ( T2_flag == 1 )
	{
		/* inherited */
		if ( T2_prepareFlag != 1 )
		{
			vsth_surprise("Case 1C: T2_prepareFlag", 0, T2_prepareFlag, &nfails);
		}
	
		if ( T2_parentFlag != 0 )
		{
			vsth_surprise("Case 1C: T2_parentFlag", 0, T2_parentFlag, &nfails);
		}
	
		if ( T2_childFlag != 1 )
		{
			vsth_surprise("Case 1C: T2_childFlag", 1, T2_childFlag, &nfails);
		}
	}
	else if ( T2_flag == 2 )
	{
		/* inherited */
		if ( T2_prepareFlag != 1 )
		{
			vsth_surprise("Case 2C: T2_prepareFlag", 0, T2_prepareFlag, &nfails);
		}
	
		if ( T2_parentFlag != 0 )
		{
			vsth_surprise("Case 2C: T2_parentFlag", 0, T2_parentFlag, &nfails);
		}
	
		if ( T2_childFlag != 0 )
		{
			vsth_surprise("Case 2C: T2_childFlag", 0, T2_childFlag, &nfails);
		}
	}

	rval = 1;

	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;

	exit(rval);
}

void
t2_prepare()
{
	++T2_prepareFlag;
}

void
t2_parent()
{
	++T2_parentFlag;
}

void
t2_child()
{
	++T2_childFlag;
}

