
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_kill
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_kill.c	1.15
*
*	History
*		Changed pthread_detach to vsth_pthread_detach to allow
*		TET API to only reap residual threads.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	5

#ifdef  UNDEF_MACROS
#undef  pthread_kill
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();
void	test5();

void	t1_f1( int *errors, int *fails );
void	*t1_t1( void *arg );
void	t1_sigterm( int sig );
void	t2_f1( int *errors, int *fails );
void	*t2_t1( void *arg );
void	t3_f1( int *errors, int *fails );
void	t4_f1( int val, int *errors, int *fails );
void	*t4_t1( void *arg );


void
startup()
{
	vsth_startup("pthread_kill");
}

void
cleanup()
{
	vsth_cleanup("pthread_kill");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_kill(thread, sig) requests that the signal");
	tet_infoline("specified by the sig argument be delivered to the thread");
	tet_infoline("specified by the thread argument and returns 0 if successful.");

	tet_infoline("Posix Ref: Component PTHREAD_KILL");
	tet_infoline("Assertion 9945-1:1996 3.3.10.2-1(A)");

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int		T1_flag;
int		T1_sigflag;
pthread_cond_t	T1_cv;
pthread_mutex_t	T1_mx;

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	sigset_t	set;
	sigset_t	oset;
	pthread_t	tid;
	void		*tval;

	T1_flag		= 0;
	T1_sigflag	= 0;

	rval = pthread_cond_init( &T1_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		return;
	}

	rval = pthread_mutex_init( &T1_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		(void) pthread_cond_destroy( &T1_cv );
		return;
	}

	/* temporarily block all signals in this thread */
	sigfillset( &set );
	rval = pthread_sigmask(SIG_BLOCK, &set, &oset);
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
		(void) pthread_sigmask( SIG_BLOCK, &oset, NULL );
		(void) pthread_cond_destroy( &T1_cv );
		(void) pthread_mutex_destroy( &T1_mx );
		return;
	}

	rval = pthread_mutex_lock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, errors);
		(void) pthread_sigmask( SIG_BLOCK, &oset, NULL );
		(void) pthread_cond_destroy( &T1_cv );
		(void) pthread_mutex_destroy( &T1_mx );
		return;
	}

	rval = vsth_pthread_create( &tid, NULL, t1_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		(void) pthread_sigmask( SIG_BLOCK, &oset, NULL );
		(void) pthread_cond_destroy( &T1_cv );
		(void) pthread_mutex_destroy( &T1_mx );
		return;
	}

	/* synchronize with receiving thread */
	while ( T1_flag != 1 )
	{
		rval = pthread_cond_wait( &T1_cv, &T1_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
		}
	}
	
	/* other thread is in the pthread_cond_wait now */

	rval = pthread_kill( tid, SIGTERM );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_kill", 0, rval, errors);
		(void) pthread_sigmask( SIG_BLOCK, &oset, NULL );
		(void) pthread_cond_destroy( &T1_cv );
		(void) pthread_mutex_destroy( &T1_mx );
		return;
	}

	++T1_flag;

	rval = pthread_cond_signal( &T1_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, errors);
		(void) pthread_sigmask( SIG_BLOCK, &oset, NULL );
		(void) pthread_cond_destroy( &T1_cv );
		(void) pthread_mutex_destroy( &T1_mx );
		return;
	}

	rval = pthread_mutex_unlock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
		(void) pthread_sigmask( SIG_BLOCK, &oset, NULL );
		(void) pthread_cond_destroy( &T1_cv );
		(void) pthread_mutex_destroy( &T1_mx );
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exit value", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exit value", 0, (int) tval, fails);
		}
	}
	else
	{
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}
	if ( T1_sigflag != 1 )
	{
		vsth_surprise("T1_sigflag", 1, T1_sigflag, fails);
	}

	(void) pthread_mutex_destroy( &T1_mx );
	(void) pthread_cond_destroy( &T1_cv );

	(void) pthread_sigmask( SIG_BLOCK, &oset, NULL );
}

void *
t1_t1( void *arg )
{
	int			nerrs;
	int			nfails;
	int			rval;
	void			*tval;
	struct sigaction	sa;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	sigfillset( &sa.sa_mask );
	sa.sa_handler	= t1_sigterm;
	sa.sa_flags	= 0;

	rval = sigaction( SIGTERM, &sa, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, &nerrs);
	}

	sigemptyset( &sa.sa_mask );
	sigaddset( &sa.sa_mask, SIGTERM );
	rval = pthread_sigmask( SIG_UNBLOCK, &sa.sa_mask, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = pthread_mutex_lock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	++T1_flag;

	rval = pthread_cond_signal( &T1_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	while ( T1_flag != 2 )
	{
		rval = pthread_cond_wait( &T1_cv, &T1_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
		}
	}

	rval = pthread_mutex_unlock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
t1_sigterm( int sig )
{
	++T1_sigflag;
}


int		T2_flag;
pthread_cond_t	T2_cv;
pthread_mutex_t	T2_mx;

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the thread specified by the thread argument exists,");
	tet_infoline("then a call to pthread_kill(thread, 0) shall return 0 without sending");
	tet_infoline("a signal to the target thread.");

	tet_infoline("Posix Ref: Component PTHREAD_KILL");
	tet_infoline("Assertion 9945-1:1996 3.3.10.2-2(A)");

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	T2_flag		= 0;

	rval = pthread_cond_init( &T2_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		return;
	}

	rval = pthread_mutex_init( &T2_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		(void) pthread_cond_destroy( &T2_cv );
		return;
	}

	rval = pthread_mutex_lock( &T2_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, errors);
		(void) pthread_cond_destroy( &T2_cv );
		(void) pthread_mutex_destroy( &T2_mx );
		return;
	}

	rval = vsth_pthread_create( &tid, NULL, t2_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		(void) pthread_cond_destroy( &T2_cv );
		(void) pthread_mutex_destroy( &T2_mx );
		return;
	}

	/* synchronize with receiving thread */
	while ( T2_flag != 1 )
	{
		rval = pthread_cond_wait( &T2_cv, &T2_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
		}
	}
	
	rval = pthread_kill( tid, 0 );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_kill", 0, rval, fails);
	}

	++T2_flag;

	rval = pthread_cond_signal( &T2_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, errors);
	}

	rval = pthread_mutex_unlock( &T2_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	(void) pthread_mutex_destroy( &T2_mx );
	(void) pthread_cond_destroy( &T2_cv );
}

void *
t2_t1( void *arg )
{
	int		nerrs;
	int		nfails;
	int		rval;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_mutex_lock( &T2_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	++T2_flag;

	rval = pthread_cond_signal( &T2_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	while ( T2_flag != 2 )
	{
		rval = pthread_cond_wait( &T2_cv, &T2_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
		}
	}

	rval = pthread_mutex_unlock( &T2_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}


void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When no thread could be found corresponding to that specified by the");
	tet_infoline("thread argument,");
	tet_infoline("then a call to pthread_kill(thread, sig) shall fail and return ESRCH.");

	tet_infoline("Posix Ref: Component PTHREAD_KILL");
	tet_infoline("Assertion 9945-1:1996 3.3.10.4-3(A)");

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = vsth_pthread_create( &tid, NULL, vsth_null_thread, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	rval = pthread_kill( tid, 0 );
	if ( rval != ESRCH )
	{
		vsth_surprise("pthread_kill", ESRCH, rval, fails);
	}
}

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the value of the sig argument is an invalid or");
	tet_infoline("unsupported signal number,");
	tet_infoline("then a call to pthread_kill(thread, sig) shall fail");
	tet_infoline("and return EINVAL.");

	tet_infoline("Posix Ref: Component PTHREAD_KILL");
	tet_infoline("Assertion 9945-1:1996 3.3.10.4-4(A)");

	p = tet_getvar("VSTH_INVALID_SIG");
	if ( p && *p )
	{
		t4_f1( atoi(p), &errors, &fails );
	}
	else
	{
		vsth_badconfig("VSTH_INVALID_SIG", &errors);
	}

	result = VSTH_RESULT( errors, fails );
	tet_result(result);
}

void
t4_f1( int val, int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = vsth_pthread_create( &tid, NULL, t4_t1, (void *) val);
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exit value", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exit value", 0, (int) tval, fails);
		}
	}
	else
	{
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}
}

void *
t4_t1( void *arg )
{
	int	rval;
	int	nerrs;
	int	nfails;
	void	*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_kill(pthread_self(), (int) arg);
	if ( rval != EINVAL )
	{
		vsth_surprise("pthread_kill", EINVAL, rval, &nfails);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

int	t5_sfunc();
int	t5_tfunc();

pthread_t	T5_tid;

VSTH_RT		T5_et = {
	t5_sfunc,	t5_tfunc,	NULL
};

void
test5()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_kill() shall not return EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_KILL");
	tet_infoline("Assertion 9945-1:1996 2.4-5(A)");

	vsth_et( &T5_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int
t5_sfunc()
{
	T5_tid = pthread_self();
	return(0);
}

int
t5_tfunc()
{
	int	val;
	int	rval;

	rval = 1;
	val = pthread_kill( T5_tid, 0 );
	if ( val != EINTR )
		rval = 0;
	return(rval);
}



void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	test5,	5	},
	{	NULL,	0	}
};


