
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_sigmask
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_sigmask.c	1.18
*
*	History:
*	Moved tests 1 and 4 to a separate executable.
*	Changed pthread_detach to vsth_pthread_detach to allow
*	TET API to only reap residual threads.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	12

#define	TE_PATH		"pthread_sigmask_exectests"

#ifdef  UNDEF_MACROS
#undef  pthread_sigmask
#endif

int	no_tests =	NO_TESTS;

extern	char	**environ;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();
void	test5();
void	test6();
void	test7();
void	test8();
void	test9();
void	test10();
void	test11();
void	test12();

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );
void	t4_f1( int *errors, int *fails );
void	t5_f1( int *errors, int *fails );
void	t6_f1( int *errors, int *fails );
void	t7_f1( int *errors, int *fails );
void	t8_f1( int *errors, int *fails );
void	t9_f1( int *errors, int *fails );
void	t10_f1( int *errors, int *fails );
void	t11_f1( int *errors, int *fails );

void	t1_c1();
void	*t1_t1( void *arg );
void	*t2_t1( void *arg );
void	t3_c1();
void	t4_sigquit( int sig );
void	t4_c1();
void	*t5_t1( void *arg );
void	*t6_t1( void *arg );
void	*t7_t1( void *arg );
void	*t8_t1( void *arg );
void	*t9_t1( void *arg );
void	*t10_t1( void *arg );

static void	psigset( sigset_t *set );

void
startup()
{
	vsth_startup("pthread_sigmask");
}

void
cleanup()
{
	vsth_cleanup("pthread_sigmask");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors	= 0;
	fails	= 0;

	tet_infoline("Each thread has a signal mask that defines the set of signals");
	tet_infoline("currently blocked from delivery to it.");

	tet_infoline("Posix Ref: Component PTHREAD_SIGMASK");
	tet_infoline("Assertion 9945-1:1996 3.3.5.2, 9945-1:1996 3.3.1.2-1(A)");

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t1_f1( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork( t1_c1, NULL, VSTH_TIMEOUT, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
}

/******	Block SIGTERM, SIGQUIT and SIGINT and install handlers. Create a
	thread, wait for it to block, then signal it.			******/


void
t1_c1()
{
	char		*argv[2];

	argv[0] = execpath(TE_PATH);
	argv[1] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("The signal mask of a thread created by a call to");
	tet_infoline("pthread_create() is that of the calling thread.");

	tet_infoline("Posix Ref: Component PTHREAD_SIGMASK");
	tet_infoline("Assertion 9945-1:1996 3.3.5.1, 9945-1:1996 3.3.1.2-2(A)");

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

sigset_t	T2_set;

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;
	sigset_t	set;

	rval = pthread_sigmask( SIG_SETMASK, NULL, &set );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
		return;
	}

	rval = vsth_pthread_create( &tid, NULL, t2_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	if ( memcmp( (void *) &T2_set, (void *) &set, sizeof(sigset_t) ) )
	{
		++*fails;
	}
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_sigmask( SIG_SETMASK, NULL, &T2_set );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("The signal mask of a thread created by a call to fork()");
	tet_infoline("is that of the thread which called fork().");

	tet_infoline("Posix Ref: Component PTHREAD_SIGMASK");
	tet_infoline("Assertion 9945-1:1996 3.3.5.1, 9945-1:1996 3.3.1.2-3(A)");

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

sigset_t	T3_set1;
sigset_t	T3_set2;

void
t3_f1( int *errors, int *fails )
{
	int		rval;

	rval = pthread_sigmask( SIG_SETMASK, NULL, &T3_set1 );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
	}

	rval = tet_fork( t3_c1, NULL, VSTH_TIMEOUT, 0 | 1 | 2 | 3 );

	switch (rval)	{

		case 0:
			break;
		case 1:
			vsth_surprise("tet_fork", 0, rval, errors);
			break;
		case 2:
			tet_infoline("Thread signal sets differ after fork()");
			vsth_surprise("tet_fork", 0, rval, fails);
			break;
		case 3:
			tet_infoline("Call to sigprocmask() failed in child");
			vsth_surprise("tet_fork", 0, rval, errors);
			break;
		default:
			vsth_surprise("tet_fork", 0, rval, errors);
			break;
	}
}

void
t3_c1()
{
	int		rval;
	int		nerrs;
	int		nfails;
	pthread_t	tid;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	
	rval = sigprocmask( SIG_SETMASK, NULL, &T3_set2 );
	if ( rval != 0 )
	{
		_exit(3);
	}

	if ( memcmp( &T3_set1, &T3_set2, sizeof(sigset_t) ))
	{
		++nfails;
	}

	rval = 1;

	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;

	_exit(rval);
}

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When there are any pending unblocked signals,");
	tet_infoline("then at least one of those signals shall be delivered before");
	tet_infoline("a call to the pthread_sigmask() function returns.");

	tet_infoline("Posix Ref: Component PTHREAD_SIGMASK");
	tet_infoline("Assertion 9945-1:1996 3.3.5.2-4(A)");

	t4_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

/******	Block all signals. Install a handler for SIGQUIT. Spin a thread.
	Wait for the thread to announce. Signal the thread with SIGQUIT.
	The thread unblocks SIGQUIT. Check that the signal was delivered
	upon return from pthread_sigmask.				******/

void
t4_f1( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork( t4_c1, NULL, VSTH_TIMEOUT, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
}


void
t4_c1()
{
	char		*argv[2];

	argv[0] = execpath(TE_PATH);
	argv[1] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	_exit(1);
}

void
test5()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the argument how is SIG_BLOCK,");
	tet_infoline("then a call to pthread_sigmask(how, set, oset) shall set the");
	tet_infoline("signal mask of the calling thread to the union of the current");
	tet_infoline("set and the signal set pointed to by the argument set and");
	tet_infoline("return 0 if successful.");

	tet_infoline("Posix Ref: Component PTHREAD_SIGMASK");
	tet_infoline("Assertion 9945-1:1996 3.3.5.2-5(A)");

	t5_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t5_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = vsth_pthread_create( &tid, NULL, t5_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}
}

void *
t5_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	sigset_t	set;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_SETMASK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGINT );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_BLOCK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_SETMASK, NULL, &set);
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = sigismember( &set, SIGINT );
	if ( rval == 0 )
	{
		tet_infoline("SIGINT is unexpectedly not blocked");
		++nfails;
	}

	rval = sigismember( &set, SIGQUIT );
	if ( rval )
	{
		tet_infoline("SIGQUIT is unexpectedly blocked");
		++nfails;
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
test6()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the argument how is SIG_UNBLOCK,");
	tet_infoline("then a call to pthread_sigmask(how, set, oset) shall set the");
	tet_infoline("signal mask of the calling thread to the intersection of the");
	tet_infoline("current set and the compliment of the  set pointed to by the");
	tet_infoline("argument set and return 0 if successful");

	tet_infoline("Posix Ref: Component PTHREAD_SIGMASK");
	tet_infoline("Assertion 9945-1:1996 3.3.5.2-6(A)");

	t6_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t6_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = vsth_pthread_create( &tid, NULL, t6_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}
}

void *
t6_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	sigset_t	set;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigfillset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_SETMASK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGINT );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_UNBLOCK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_SETMASK, NULL, &set);
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = sigismember( &set, SIGINT );
	if ( rval != 0  )
	{
		tet_infoline("SIGINT is unexpectedly blocked");
		++nfails;
	}

	rval = sigismember( &set, SIGTERM );
	if ( rval == 0 )
	{
		tet_infoline("SIGTERM is unexpectedly unblocked");
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
test7()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the argument how is SIG_SETMASK,");
	tet_infoline("then a call to pthread_sigmask(how, set, oset) shall set the");
	tet_infoline("signal mask of the calling thread to the set pointed to by the");
	tet_infoline("argument set and return 0 if successful.");

	tet_infoline("Posix Ref: Component PTHREAD_SIGMASK");
	tet_infoline("Assertion 9945-1:1996 3.3.5.2-7(A)");

	t7_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t7_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = vsth_pthread_create( &tid, NULL, t7_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}
}

void *
t7_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	sigset_t	set;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGPIPE );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_SETMASK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_SETMASK, NULL, &set);
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = sigismember( &set, SIGPIPE );
	if ( rval == 0  )
	{
		tet_infoline("SIGPIPE is unexpectedly unblocked");
		++nfails;
	}

	rval = sigismember( &set, SIGTERM );
	if ( rval != 0 )
	{
		tet_infoline("SIGTERM is unexpecedly blocked");
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
test8()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the argument oset is not NULL,");
	tet_infoline("then a call to pthread_sigmask(how, set, oset) shall store the");
	tet_infoline("previous signal mask of the calling thread in the location");
	tet_infoline("specified by oset and return 0 if successful.");

	tet_infoline("Posix Ref: Component PTHREAD_SIGMASK");
	tet_infoline("Assertion 9945-1:1996 3.3.5.2-8(A)");

	t8_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t8_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = vsth_pthread_create( &tid, NULL, t8_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}
}

void *
t8_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	sigset_t	set;
	sigset_t	newset;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGPIPE );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_SETMASK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_SETMASK, NULL, &newset);
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	if ( memcmp( (void *) &set, (void *) &newset, sizeof(sigset_t) ))
	{
		tet_infoline("Signal sets do not match");
		psigset( &set );
		psigset( &newset );
		++nfails;
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
test9()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the argument set is NULL,");
	tet_infoline("then a call to pthread_sigmask(how, NULL, oset) shall not change");
	tet_infoline("the signal mask of the calling thread and shall return 0");
	tet_infoline("if successful.");

	tet_infoline("Posix Ref: Component PTHREAD_SIGMASK");
	tet_infoline("Assertion 9945-1:1996 3.3.5.2-9(A)");

	t9_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t9_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = vsth_pthread_create( &tid, NULL, t9_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}
}

/* self-referential systems again ... */
void *
t9_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	sigset_t	set;
	sigset_t	oldset;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, &nerrs);
	}

	rval = sigfillset( &oldset );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_SETMASK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_SETMASK, NULL, &oldset );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nfails);
	}

	if ( memcmp( (void *) &set, (void *) &oldset, sizeof(sigset_t) ))
	{
		tet_infoline("Signal sets do not match");
		psigset( &set );
		psigset( &oldset );
		++nfails;
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
test10()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When a thread attempts to add the signal SIGKILL or SIGSTOP to");
	tet_infoline("the set of signals to be blocked,");
	tet_infoline("then a call to pthread_sigmask(how, set, oset) shall not return");
	tet_infoline("an error.");

	tet_infoline("Posix Ref: Component PTHREAD_SIGMASK");
	tet_infoline("Assertion 9945-1:1996 3.3.5.2-10(A)");

	t10_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t10_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = vsth_pthread_create( &tid, NULL, t10_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}
}

void *
t10_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	sigset_t	set;
	sigset_t	newset;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGKILL );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_BLOCK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nfails);
	}

	rval = sigaddset( &set, SIGSTOP );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nfails);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
test11()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the argument how is not one of SIG_BLOCK, SIG_UNBLOCK or");
	tet_infoline("SIG_SETMASK, then a call to pthread_sigmask(how, set, oset)");
	tet_infoline("shall return EINVAL");

	tet_infoline("Posix Ref: Component PTHREAD_SIGMASK");
	tet_infoline("Assertion 9945-1:1996 3.3.5.4-11(A)");

	t11_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t11_f1( int *errors, int *fails )
{
	int		rval;
	sigset_t	set;
	sigset_t	oset;

	rval = sigemptyset(&set);
	if ( rval == 0 )
	{

		rval = pthread_sigmask( 2 * ( SIG_BLOCK + SIG_UNBLOCK + SIG_SETMASK ),
					&set, &oset );
		if ( rval != EINVAL )
		{
			vsth_surprise("pthread_sigmask", EINVAL, rval, fails);
			if ( rval == 0 )
			{
				(void) pthread_sigmask(SIG_SETMASK, &oset, NULL);
			}
		}
	}
	else
	{
		vsth_surprise("sigemptyset", 0, rval, errors);
	}
}

int		t12_tfunc();
VSTH_RT		T12_et = {
	NULL,	t12_tfunc,	NULL
};

void
test12()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_sigmask() shall not return EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_SIGMASK");
	tet_infoline("Assertion 9945-1:1996 2.4-12(A)");

	vsth_et( &T12_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int
t12_tfunc()
{
	int		val;
	int		rval;
	sigset_t	oset;

	rval = 1;

	val = pthread_sigmask( SIG_SETMASK, NULL, &oset );
	if ( val != EINTR )
		rval = 0;

	return(rval);
}

static void
psigset( sigset_t *pset )
{
	int		cnt;
	char		*p;
	char		*buf;
	unsigned char	*pc;

	pc	= (unsigned char *) pset;
	cnt	= sizeof( sigset_t );
	buf 	= malloc(( cnt * sizeof(char) * 2 ) + 1);
	if ( buf == NULL )
		return;

	p	= buf;
	while ( cnt-- )
	{
		sprintf(p, "%.2x", *pc);
		++pc;
		p += 2;
	}

	tet_infoline(buf);
	(void) free(buf);
}

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	test5,	5	},
	{	test6,	6	},
	{	test7,	7	},
	{	test8,	8	},
	{	test9,	9	},
	{	test10,	10	},
	{	test11,	11	},
	{	test12,	12	},
	{	NULL,	0	}
};


