
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_sigmask_exectests
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_sigmask_exectests.c	1.1
*
*	History:
*	Created separate executable to avoid calls to non-asynch safe functions.
*	Changed pthread_detach to vsth_pthread_detach to allow
*	TET API to only reap residual threads.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#ifdef  UNDEF_MACROS
#undef  pthread_sigmask
#endif

void	t1_sigterm( int sig );
void	t1_sigquit( int sig );
void	t1_sigint( int sig );
int	t1_ssigs();
int	t1_rsigs();
void	t1_c1();
void	*t1_t1( void *arg );
void	t4_sigquit( int sig );
void	t4_c1();
void	*t4_t1( void *arg );
int	t4_ssigs();
int	t4_rsigs();


int	tet_nosigreset = 0;

int
tet_main( int argc, char **argv)
{

	switch (tet_thistest) {
		case 1: t1_c1();
			break;
		case 4: t4_c1();
			break;
		default:
			tet_printf("Invalid test in test exec list - %d", tet_thistest);
			break;
	}

	exit(1);
}

/******	Block SIGTERM, SIGQUIT and SIGINT and install handlers. Create a
	thread, wait for it to block, then signal it.			******/

sigset_t		T1_savedset;
struct sigaction	T1_termsa;
struct sigaction	T1_quitsa;
struct sigaction	T1_intsa;

pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;

int			T1_flag;
int			T1_termflag;
int			T1_quitflag;
int			T1_intflag;

void
t1_c1()
{
	int		rval;
	int		nerrs;
	int		nfails;
	pthread_t	tid;
	void		*tval;

	nerrs		= 0;
	nfails		= 0;

	T1_flag		= 0;
	T1_termflag	= 0;
	T1_quitflag	= 0;
	T1_intflag	= 0;

	rval = pthread_mutex_init( &T1_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, &nerrs);
		exit(1);
	}

	rval = pthread_cond_init( &T1_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, &nerrs);
		(void) pthread_mutex_destroy( &T1_mx );
		exit(1);
	}

	rval = t1_ssigs();
	if ( rval != 0 )
	{
		tet_infoline("Failed to set signal handlers");
		(void) pthread_cond_destroy( &T1_cv );
		(void) pthread_mutex_destroy( &T1_mx );
		exit(1);
	}

	rval = pthread_mutex_lock( &T1_mx );
	if ( rval != 0 )
	{
		tet_printf("pthread_mutex_lock failed, errno == %d", rval);
		(void) pthread_cond_destroy( &T1_cv );
		(void) pthread_mutex_destroy( &T1_mx );
		t1_rsigs();
		exit(1);
	}
	
	rval = vsth_pthread_create( &tid, NULL, t1_t1, NULL );
	if ( rval != 0 )
	{
		tet_printf("Unable to create a thread, errno == %d", rval);
		(void) pthread_cond_destroy( &T1_cv );
		(void) pthread_mutex_destroy( &T1_mx );
		t1_rsigs();
		exit(1);
	}

	while ( T1_flag != 1 )
	{
		rval = pthread_cond_wait( &T1_cv, &T1_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
		}
	}

	rval = pthread_kill( tid, SIGTERM );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_kill", 0, rval, &nerrs);
	}

	rval = pthread_kill( tid, SIGQUIT );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_kill", 0, rval, &nerrs);
	}

	rval = pthread_kill( tid, SIGINT );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_kill", 0, rval, &nerrs);
	}

	T1_flag = 0;

	rval = pthread_cond_signal( &T1_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	rval = pthread_mutex_unlock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise(	"Thread exitval", 0, (int) tval, &nerrs );
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise(	"Thread exitval", 0, (int) tval, &nfails );
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, &nerrs);
	}

	/* check that signals were blocked appropriately in t1_t1 */
	if ( T1_termflag != 1 )
	{
		vsth_surprise("T1_termflag", 1, T1_termflag, &nfails);
	}

	if ( T1_quitflag != 0 )
	{
		vsth_surprise("T1_quitflag", 0, T1_quitflag, &nfails);
	}

	if ( T1_intflag != 0 )
	{
		vsth_surprise("T1_intflag", 0, T1_intflag, &nfails);
	}

	rval = t1_rsigs();
	if ( rval != 0 )
	{
		vsth_surprise("t1_rsigs", 0, rval, &nerrs);
	}

	(void) pthread_mutex_destroy( &T1_mx );
	(void) pthread_cond_destroy( &T1_cv );

	rval = 1;

	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;

	exit(rval);
}

/* Block all signals. Install handlers for SIGTERM, SIGINT, and SIGQUIT */

void
t1_sigterm( int sig )
{
	++T1_termflag;
}

void
t1_sigquit( int sig )
{
	++T1_quitflag;
}

void
t1_sigint( int sig )
{
	++T1_intflag;
}

int
t1_ssigs()
{
	int			rval;
	int			nerrs;
	int			nfails;
	sigset_t		set;
	struct sigaction	sa;

	nerrs	= 0;
	nfails	= 0;

	rval = sigfillset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, &nerrs);
	}

	rval = sigdelset( &set, SIGTERM );
	if ( rval != 0 )
	{
		vsth_surprise("sigdelset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_BLOCK, &set, &T1_savedset );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nfails);
	}

	rval = sigfillset( &sa.sa_mask );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, &nerrs);
	}

	sa.sa_flags	= 0;
	sa.sa_handler	= t1_sigquit;

	rval = sigaction( SIGQUIT, &sa, &T1_quitsa );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, &nerrs);
	}

	sa.sa_handler	= t1_sigint;

	rval = sigaction( SIGINT,  &sa, &T1_intsa );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, &nerrs);
	}

	sa.sa_handler	= t1_sigterm;

	rval = sigaction( SIGTERM, &sa, &T1_termsa );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, &nerrs);
	}

	rval = 1;

	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;

	return(rval);
}

/* Restore signal handling */
int
t1_rsigs()
{
	int	rval;
	int	nerrs;
	int	nfails;

	nerrs	= 0;
	nfails	= 0;

	rval = pthread_sigmask( SIG_SETMASK, &T1_savedset, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = sigaction( SIGTERM, &T1_termsa, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, &nerrs);
	}

	rval = sigaction( SIGQUIT, &T1_quitsa, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, &nerrs);
	}

	rval = sigaction( SIGINT,  &T1_intsa,  NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, &nerrs);
	}

	rval = 1;

	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;

	return(rval);
}

/* Unblock SIGTERM. Announce and then block while t1_c1() signals	*/

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	sigset_t	set;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	/* the sigemptyset way assumes that masks are inherited */
	rval = sigfillset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, &nerrs);
	}

	rval = sigdelset( &set, SIGTERM );
	if ( rval != 0 )
	{
		vsth_surprise("sigdelset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_BLOCK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nfails);
	}

	rval = pthread_mutex_lock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	++T1_flag;

	rval = pthread_cond_signal( &T1_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	/* t1_c1 signals us while we're blocked in the wait */
	while ( T1_flag != 0 )
	{
		rval = pthread_cond_wait( &T1_cv, &T1_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, &nfails);
		}
	}

	rval = pthread_mutex_unlock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}


/******	Block all signals. Install a handler for SIGQUIT. Spin a thread.
	Wait for the thread to announce. Signal the thread with SIGQUIT.
	The thread unblocks SIGQUIT. Check that the signal was delivered
	upon return from pthread_sigmask.				******/

sigset_t		T4_savedset;
struct sigaction	T4_quitsa;

pthread_mutex_t		T4_mx;
pthread_cond_t		T4_cv;

int			T4_flag;
int			T4_quitflag;

void
t4_c1()
{
	int		rval;
	int		nerrs;
	int		nfails;
	pthread_t	tid;
	void		*tval;

	nerrs		= 0;
	nfails		= 0;

	T4_flag		= 0;
	T4_quitflag	= 0;

	rval = pthread_mutex_init( &T4_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, &nerrs);
		exit(1);
	}

	rval = pthread_cond_init( &T4_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, &nerrs);
		(void) pthread_mutex_destroy( &T4_mx );
		exit(1);
	}

	rval = t4_ssigs();
	if ( rval != 0 )
	{
		tet_infoline("Failed to set signal handlers");
		(void) pthread_cond_destroy( &T4_cv );
		(void) pthread_mutex_destroy( &T4_mx );
		exit(1);
	}

	rval = pthread_mutex_lock( &T4_mx );
	if ( rval != 0 )
	{
		tet_printf("pthread_mutex_lock failed, errno == %d", rval);
		(void) pthread_cond_destroy( &T4_cv );
		(void) pthread_mutex_destroy( &T4_mx );
		t4_rsigs();
		exit(1);
	}
	
	rval = vsth_pthread_create( &tid, NULL, t4_t1, NULL );
	if ( rval != 0 )
	{
		tet_printf("Unable to create a thread, errno == %d", rval);
		(void) pthread_cond_destroy( &T4_cv );
		(void) pthread_mutex_destroy( &T4_mx );
		t4_rsigs();
		exit(1);
	}

	while ( T4_flag != 1 )
	{
		rval = pthread_cond_wait( &T4_cv, &T4_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
		}
	}

	rval = pthread_kill( tid, SIGQUIT );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_kill", 0, rval, &nerrs);
	}

	T4_flag = 0;

	rval = pthread_cond_signal( &T4_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	rval = pthread_mutex_unlock( &T4_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise(	"Thread exitval", 0, (int) tval, &nerrs );
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise(	"Thread exitval", 0, (int) tval, &nfails );
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, &nerrs);
	}

	rval = t4_rsigs();
	if ( rval != 0 )
	{
		vsth_surprise("t4_rsigs", 0, rval, &nerrs);
	}

	(void) pthread_mutex_destroy( &T4_mx );
	(void) pthread_cond_destroy( &T4_cv );

	rval = 1;

	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;

	exit(rval);
}

void
t4_sigquit( int sig )
{
	++T4_quitflag;
}

int
t4_ssigs()
{
	int			rval;
	int			nerrs;
	int			nfails;
	sigset_t		set;
	struct sigaction	sa;

	nerrs	= 0;
	nfails	= 0;

	rval = sigfillset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_BLOCK, &set, &T4_savedset );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nfails);
	}

	rval = sigfillset( &sa.sa_mask );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, &nerrs);
	}

	sa.sa_flags	= 0;
	sa.sa_handler	= t4_sigquit;

	rval = sigaction( SIGQUIT, &sa, &T4_quitsa );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, &nerrs);
	}

	rval = 1;

	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;

	return(rval);
}

/* Restore signal handling */
int
t4_rsigs()
{
	int	rval;
	int	nerrs;
	int	nfails;

	nerrs	= 0;
	nfails	= 0;

	rval = pthread_sigmask( SIG_SETMASK, &T4_savedset, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = sigaction( SIGQUIT, &T4_quitsa, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, &nerrs);
	}

	rval = 1;

	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;

	return(rval);
}

/******	Block all signals, wait for a SIGTERM in pthread_cond_wait. Unblock
	SIGTERM and expect to have received the signal.				******/

void *
t4_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	sigset_t	set;
	sigset_t	pending_set;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigfillset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, &nerrs);
	}

	/* we should have inherited a good sigmask, but just in case */
	rval = pthread_sigmask( SIG_BLOCK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = sigdelset( &set, SIGQUIT );
	if ( rval != 0 )
	{
		vsth_surprise("sigdelset", 0, rval, &nerrs);
	}

	rval = pthread_mutex_lock( &T4_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	++T4_flag;

	rval = pthread_cond_signal( &T4_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	/* t4_c1 signals us while we're blocked in the wait */
	while ( T4_flag != 0 )
	{
		rval = pthread_cond_wait( &T4_cv, &T4_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, &nfails);
		}
	}

	rval = pthread_mutex_unlock( &T4_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
	}

	rval = sigpending( &pending_set );
	if ( rval != 0 )
	{
		vsth_surprise("sigpending", 0, rval, &nerrs);
	}

	rval = sigismember( &pending_set, SIGQUIT );
	if ( rval == 0 )
	{
		++nerrs;
		tet_infoline("SIGQUIT is not pending");
	}

	if ( T4_quitflag != 0 )
	{
		vsth_surprise("T4_quitflag", 0, T4_quitflag, &nfails);
	}

	rval = pthread_sigmask( SIG_SETMASK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nfails);
	}

	if ( T4_quitflag != 1 )
	{
		vsth_surprise("T4_quitflag", 1, T4_quitflag, &nfails);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}
