
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/sigwait
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)sigwait.c	1.43
*
*	History
*		Changed pthread_detach to vsth_pthread_detach to allow
*		TET API to only reap residual threads.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	<sys/wait.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	8

/* TSD5TH.00053: added for test 2*/

#define TEST2_COUNTER  10000000

#ifdef  UNDEF_MACROS
#undef  sigwait
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();
void	test5();
void	test6();
void	test7();
void	test8();

void	t1_f1( int *errors, int *fails );
void	t1_sigpipe( int sig );
void	t1_ssigs( int *errors, int *fails );
void	t1_rsigs( int *errors, int *fails );
void	*t1_t1( void *arg );

static int 	t2_autoinc ( int *arg );
void	t2_f1( int *errors, int *fails );
void	t2_sigalrm( int sig );
void	t2_ssigs( int *errors, int *fails );
void	t2_rsigs( int *errors, int *fails );
void	*t2_t1( void *arg );

void	t3_f1( int flag, int *errors, int *fails );
void	*t3_t1( void *arg );
void	*t3_t2( void *arg );
void	*t3_t3( void *arg );
void	t3_sigint( int sig );
void	t3_ssigs( int *errors, int *fails );
void	t3_rsigs( int *errors,int *fails );

void	t5_f1( int *errors, int *fails );
void	t5_f2( int *errors, int *fails );
void	t5_c1();
void	t5_c2();
void	t5_p1( pid_t pid, int *errors, int *fails );
void	t5_p2( pid_t pid, int *errors, int *fails );
void	*t5_t1( void *arg );
void	*t5_t2( void *arg );
void	*t5_t2( void *arg );
void	t5_sigcont(int);
void	t5_sigchld(int);

void	t6_f1( int *errors, int *fails );
int	t6_precheck();
void	t6_postcheck( int sig, int *errors, int *fails );
void	t6_sh( int sig );
void	*t6_t1( void *arg );
void	t6_ssigs( int *errors, int *fails );
void	t6_rsigs( int *errors, int *fails );

#ifdef	_POSIX_REALTIME_SIGNALS

void	t7_f1( int *errors, int *fails );
int	t7_check( int sig, int *errors, int *fails );
void	t7_sh( int sig, siginfo_t *info, void *context );
void	t7_ssigs( int *errors, int *fails );
void	t7_rsigs( int *errors, int *fails );
void	*t7_t1( void *arg );

#endif

void	t8_f1( int *errors, int *fails );
void	*t8_t1( void *arg );

void
startup()
{
	vsth_startup("sigwait");
}

void
cleanup()
{
	vsth_cleanup("sigwait");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("Signals that are generated by some action attributable to a");
	tet_infoline("particular thread shall be generated for the thread that caused");
	tet_infoline("the signal.");

	tet_infoline("Posix Ref: Component SIGWAIT");
	tet_infoline("Assertion 9945-1:1996 3.3.8.2, 9945-1:1996 3.3.1.2-1(A)");

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

/******	Install a handler for SIGPIPE, create a thread. The thread creates a fifo,
	opens it for writing, attempts a write, and generates a SIGPIPE. The signal
	handler stores its tid and we check that the correct thread received the
	signal. 								******/

#define	T1_FIFONAME	"./t1_fifo"

pthread_t		T1_tid;
sigset_t		T1_oset;
struct sigaction	T1_osa;

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	t1_ssigs( errors, fails );

	rval = vsth_pthread_create( &tid, NULL, t1_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		t1_rsigs( errors, fails );
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == NULL )
		{
			if ( ! pthread_equal( T1_tid, tid ) )
			{
				tet_infoline("The signal was not delivered to the generating thread");
				++*fails;
			}
		}
		else if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	t1_rsigs( errors, fails );
}

void
t1_sigpipe( int sig )
{
	T1_tid = pthread_self();
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	int		fd1;
	int		fd2;
	sigset_t	set;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &set );
	if ( rval == 0 )
	{
		rval = sigaddset( &set, SIGPIPE );
		if ( rval == 0 )
		{
			rval = pthread_sigmask( SIG_UNBLOCK, &set, NULL );
			if ( rval == 0 )
			{

				(void) remove(T1_FIFONAME);

				rval = mkfifo(T1_FIFONAME, S_IRWXU | S_IRWXG);
				if ( rval == 0 )
				{
					fd1 = open( T1_FIFONAME, O_RDONLY | O_NONBLOCK );
					if ( fd1 >= 0 )
					{
						fd2 = open( T1_FIFONAME, O_WRONLY );
						if ( fd2 >= 0 )
						{
							char	c;

							c = 'X';

							/* write with no reader generates SIGPIPE */
							(void) close(fd1);
							(void) write(fd2, (void *) &c, 1);
							(void) close(fd2);
						}
						else
						{
							++nerrs;
							tet_printf("open O_WRONLY failed, errno == %d", errno);
							(void) close(fd1);
						}
					}
					else
					{
						++nerrs;
						tet_printf("open O_RDONLY failed, errno == %d", errno);
					}

					(void) remove(T1_FIFONAME);
				}
				else
				{
					vsth_surprise("mkfifo", 0, rval, &nerrs);
				}
			}
			else
			{
				vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
			}
		}
		else
		{
			vsth_surprise("sigaddset", 0, rval, &nerrs);
		}
	}
	else
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
t1_ssigs( int *errors, int *fails )
{
	int			rval;
	sigset_t		set;
	struct sigaction	sa;

	sa.sa_handler	= t1_sigpipe;
	sa.sa_flags	= 0;

	rval = sigfillset( &sa.sa_mask );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, errors);
	}

	rval = pthread_sigmask( SIG_BLOCK, &sa.sa_mask, &T1_oset );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
	}

	rval = sigaction( SIGPIPE, &sa, &T1_osa );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}
}

void
t1_rsigs( int *errors, int *fails )
{
	int		rval;

	rval = pthread_sigmask( SIG_SETMASK, &T1_oset, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
	}

	rval = sigaction( SIGPIPE, &T1_osa, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}
}

int		T2_errors;
int		T2_fails;

void
test2()
{
	int		result;
	int		rval;

	T2_errors	= 0;
	T2_fails	= 0;

	tet_infoline("Signals generated for the process shall be delivered to exactly");
	tet_infoline("one of the threads waiting in a sigwait() call for the signal or");
	tet_infoline("not blocking the signal.");

	tet_infoline("Posix Ref: Component SIGWAIT");
	tet_infoline("Assertion 9945-1:1996 3.3.8.2, 9945-1:1996 3.3.1.2-2(A)");

	t2_f1( &T2_errors, &T2_fails );
	result = VSTH_RESULT( T2_errors, T2_fails );

	tet_result(result);
}

/* a small routine called to use up some cpu time */
int
t2_autoinc ( int *arg )
{
        return(++*arg);
}                     

int			T2_flag;
int			T2_sig;
int			T2_sigflag1;
int			T2_sigflag2;
pthread_mutex_t		T2_mx;
pthread_cond_t		T2_cv;
sigset_t		T2_oset;
struct sigaction	T2_osa;

/******	Block SIGALRM in the main thread and install a signal handler. Create two
	threads. One thread unblocks SIGALRM, the other thread issues a sigwait()
	on SIGALRM. Send a SIGALRM to the process. Check the number of signals
	received and the identity of the receiving thread.			******/
void
t2_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid1;
	pthread_t	tid2;
	void		*tval;
	int		i,j;	/* counter variables to force a time delay */

	T2_flag		= 0;
	T2_sig		= 0;
	T2_sigflag1	= 0;
	T2_sigflag2	= 0;

	t2_ssigs( errors, fails );

	rval = pthread_mutex_init( &T2_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		t2_rsigs( errors, fails );
		return;
	}

	rval = pthread_cond_init( &T2_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		(void) pthread_mutex_destroy( &T2_mx );
		t2_rsigs( errors, fails );
		return;
	}

	rval = pthread_mutex_lock( &T2_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, errors);
		(void) pthread_mutex_destroy( &T2_mx );
		(void) pthread_cond_destroy( &T2_cv );
		t2_rsigs( errors, fails );
		return;
	}

	rval = vsth_pthread_create( &tid1, NULL, t2_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		(void) pthread_mutex_destroy( &T2_mx );
		(void) pthread_cond_destroy( &T2_cv );
		t2_rsigs( errors, fails );
		return;
	}

	while ( T2_flag != 1 )
	{
		rval = pthread_cond_wait( &T2_cv, &T2_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
		}
	}

	rval = vsth_pthread_create( &tid2, NULL, t2_t1, (void *) 1 );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		(void) pthread_cancel(tid1);
		(void) vsth_pthread_detach(tid1);
		(void) pthread_mutex_destroy( &T2_mx );
		(void) pthread_cond_destroy( &T2_cv );
		t2_rsigs( errors, fails );
		return;
	}

	while ( T2_flag != 2 )
	{
		rval = pthread_cond_wait( &T2_cv, &T2_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
		}
	}

	/******	the main thread has SIGALRM blocked
		thread 1 is in a sigwait with SIGALRM blocked,
		thread 2 is in a pthread_cond_wait() with SIGALRM unblocked	******/

	rval = kill( getpid(), SIGALRM );
	if ( rval != 0 )
	{
		vsth_surprise("kill", 0, rval, errors);
	}

	++T2_flag;

	/* TSD5TH.00053: use some CPU time in order for the allow time for
	   the signal to be delivered in the other thread, used in 
	   preference over sleep() since that may have a side effect
	   on the result. */

	i = 0; j=0;
	while ( i  < TEST2_COUNTER )
			 i = t2_autoinc( &j);


	rval = pthread_cond_signal( &T2_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, errors);
	}

	rval = pthread_mutex_unlock( &T2_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
	}

	rval = vsth_pthread_join( tid2, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
		else if ( tval )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
	}

	if ( ! T2_sigflag1 )
	{
		/* tid1 is still blocked in the sigwait so cancel it */
		(void) pthread_cancel(tid1);
		(void) vsth_pthread_detach(tid1);
	}
	else
	{
		rval = vsth_pthread_join( tid1, &tval );
		if ( rval == 0 )
		{
			if ( tval == (void *) 2 )
			{
				vsth_surprise("Thread exitval", 0, (int) tval, fails);
			}
			else if ( tval )
			{
				vsth_surprise("Thread exitval", 0, (int) tval, errors);
			}
		}
	}

	if ( T2_sigflag1 && T2_sigflag2 )
	{
		++*fails;
		tet_infoline("The signal was delivered to multiple threads");
	}

	else if (( T2_sigflag1 == 0 ) && ( T2_sigflag2 == 0 ))
	{
		++*fails;
		tet_infoline("The signal was not delivered");
	}

	(void) pthread_mutex_destroy( &T2_mx );
	(void) pthread_cond_destroy( &T2_cv );

	t2_rsigs( errors, fails );
}

void
t2_sigalrm( int sig )
{

	++T2_sigflag2;
}

void *
t2_t1( void *arg )
{
	int		rval;
	sigset_t	set;

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &T2_errors);
	}

	rval = sigaddset( &set, SIGALRM );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &T2_errors);
	}

	if ( arg == NULL )
	{
		rval = pthread_mutex_lock( &T2_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_mutex_lock", 0, rval, &T2_errors);
		}

		++T2_flag;

		rval = pthread_cond_signal( &T2_cv );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_signal", 0, rval, &T2_errors);
		}

		rval = pthread_mutex_unlock( &T2_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_mutex_unlock", 0, rval, &T2_errors);
		}

		rval = sigwait( &set, &T2_sig );
		if ( rval == 0 )
		{
			if ( T2_sig == SIGALRM )
			{
				++T2_sigflag1;
			}
		}
		else
		{
			vsth_surprise("sigwait", 0, rval, &T2_errors);
		}

	}
	else 
	{
		rval = pthread_mutex_lock( &T2_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_mutex_lock", 0, rval, &T2_errors);
		}

		++T2_flag;

		rval = pthread_sigmask( SIG_UNBLOCK, &set, NULL );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_sigmask", 0, rval, &T2_errors);
		}

		rval = pthread_cond_signal( &T2_cv );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_signal", 0, rval, &T2_errors);
		}

		while ( T2_flag != 3 )
		{
			rval = pthread_cond_wait( &T2_cv, &T2_mx );
			if ( rval != 0 )
			{
				vsth_surprise("pthread_cond_wait", 0, rval, &T2_errors);
			}
		}

		rval = pthread_mutex_unlock( &T2_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_mutex_unlock", 0, rval, &T2_errors);
		}
	}

	/* we're cancelled in this test, so tval isn't used */
	pthread_exit(NULL);

	/* NOTREACHED */
	return(NULL);
}

void
t2_ssigs( int *errors, int *fails )
{
	int			rval;
	sigset_t		set;
	struct sigaction	sa;

	sa.sa_handler	= t2_sigalrm;
	sa.sa_flags	= 0;

	rval = sigfillset( &sa.sa_mask );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, errors);
	}

	rval = pthread_sigmask( SIG_SETMASK, &sa.sa_mask, &T2_oset );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
	}

	rval = sigaction( SIGALRM, &sa, &T2_osa );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}
}

void
t2_rsigs( int *errors, int *fails )
{
	int		rval;

	rval = pthread_sigmask( SIG_SETMASK, &T2_oset, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
	}

	rval = sigaction( SIGALRM, &T2_osa, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}
}

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the action associated with a blocked signal is anything other");
	tet_infoline("than to ignore the signal, and that signal is generated for a");
	tet_infoline("thread, then the signal shall remain pending until either it");
	tet_infoline("is accepted, it is unblocked, or the action associated with it");
	tet_infoline("is set to ignore the signal.");

	tet_infoline("Posix Ref: Component SIGWAIT");
	tet_infoline("Assertion 9945-1:1996 3.3.8.2, 9945-1:1996 3.3.1.2-3(A)");

	t3_f1( 0, &errors, &fails );
	t3_f1( 1, &errors, &fails );
	t3_f1( 2, &errors, &fails );

	/* need to add the SIG_DFL case - revisit */
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

/******	If flag == 0, test the accepted case.
	If flag == 1, test the unblocked case.
	If flag == 2, test the ignore case.					******/

int			T3_flag;
int			T3_nsigs;
int			T3_sig;
sigset_t		T3_oset;
struct sigaction	T3_osa;
pthread_mutex_t		T3_mx;
pthread_cond_t		T3_cv;

void
t3_f1( int flag, int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	T3_flag		= 0;
	T3_nsigs	= 0;
	T3_sig		= 0;

	if ( flag == 0 )
		tet_infoline("Testing pending until accepted");
	else if ( flag == 1 )
		tet_infoline("Testing pending until unblocked");
	else if ( flag == 2 )
		tet_infoline("Testing pending until ignored");

	t3_ssigs( errors, fails );
	if ( *errors )
		return;

	rval = pthread_mutex_init( &T3_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		t3_rsigs( errors, fails );
		return;
	}

	rval = pthread_cond_init( &T3_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		(void) pthread_mutex_destroy( &T3_mx );
		t3_rsigs( errors, fails );
		return;
	}

	rval = pthread_mutex_lock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, errors);
		(void) pthread_mutex_destroy( &T3_mx );
		(void) pthread_cond_destroy( &T3_cv );
		t3_rsigs( errors, fails );
		return;
	}

	if ( flag == 0 )
	{
		rval = vsth_pthread_create( &tid, NULL, t3_t1, NULL );
		if ( rval != 0 )
		{
			vsth_surprise("vsth_pthread_create", 0, rval, errors);
			(void) pthread_mutex_destroy( &T3_mx );
			(void) pthread_cond_destroy( &T3_cv );
			t3_rsigs( errors, fails );
			return;
		}
	}
	else if ( flag == 1 )
	{
		rval = vsth_pthread_create( &tid, NULL, t3_t2, NULL );
		if ( rval != 0 )
		{
			vsth_surprise("vsth_pthread_create", 0, rval, errors);
			(void) pthread_mutex_destroy( &T3_mx );
			(void) pthread_cond_destroy( &T3_cv );
			t3_rsigs( errors, fails );
			return;
		}
	}
	else
	{
		rval = vsth_pthread_create( &tid, NULL, t3_t3, NULL );
		if ( rval != 0 )
		{
			vsth_surprise("vsth_pthread_create", 0, rval, errors);
			(void) pthread_mutex_destroy( &T3_mx );
			(void) pthread_cond_destroy( &T3_cv );
			t3_rsigs( errors, fails );
			return;
		}
	}

	while ( ! T3_flag )
	{
		rval = pthread_cond_wait( &T3_cv, &T3_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
		}
	}

	rval = pthread_kill( tid, SIGINT );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_kill", 0, rval, errors);
	}

	++T3_flag;

	rval = pthread_cond_signal( &T3_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, errors);
	}

	rval = pthread_mutex_unlock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
		else if ( tval )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	(void) pthread_mutex_destroy( &T3_mx );
	(void) pthread_cond_destroy( &T3_cv );
	t3_rsigs( errors, fails );
}

/******	Test pending till signal acceptance					******/
void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	sigset_t	set;
	sigset_t	pending_set;


	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGINT );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &nerrs);
	}

	rval = pthread_mutex_lock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	++T3_flag;

	rval = pthread_cond_signal( &T3_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	while ( T3_flag != 2 )
	{
		/* we're hit with a SIGINT while blocked in this wait */
		rval = pthread_cond_wait( &T3_cv, &T3_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
		}
	}

	rval = pthread_mutex_unlock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
	}

	rval = sigpending( &pending_set );
	if ( rval == 0 )
	{
		rval = sigismember( &pending_set, SIGINT );
		if ( rval )
		{
			rval = sigwait( &set, &T3_sig );
			if ( rval == 0 )
			{
				rval = sigpending( &pending_set );
				if ( rval == 0 )
				{
					rval = sigismember( &pending_set, SIGINT );
					if ( rval )
					{
						tet_infoline("SIGINT is still pending");
						++nfails;
					}
				}
				else
				{
					vsth_surprise("sigpending", 0, rval, &nerrs);
				}
			}
			else
			{
				vsth_surprise("sigwait", 0, rval, &nfails);
			}
		}
		else
		{
			tet_infoline("SIGINT is not pending");
			++nerrs;
		}
	}
	else
	{
		vsth_surprise("sigpending", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

/******	Test pending till unblocked						******/
void *
t3_t2( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	sigset_t	set;
	sigset_t	pending_set;


	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGINT );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &nerrs);
	}

	rval = pthread_mutex_lock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	++T3_flag;

	rval = pthread_cond_signal( &T3_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	/* we're signaled while in the wait */
	rval = pthread_cond_wait( &T3_cv, &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
	}

	rval = pthread_mutex_unlock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
	}

	rval = sigpending( &pending_set );
	if ( rval == 0 )
	{
		rval = sigismember( &pending_set, SIGINT );
		if ( rval )
		{
			rval = pthread_sigmask( SIG_UNBLOCK, &set, NULL );
			if ( rval == 0 )
			{
				rval = sigpending( &pending_set );
				if ( rval == 0 )
				{
					rval = sigismember( &pending_set, SIGINT );
					if ( rval )
					{
						tet_infoline("SIGINT is still pending");
						++nfails;
					}
				}
				else
				{
					vsth_surprise("sigpending", 0, rval, &nerrs);
				}
			}
			else
			{
				vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
			}
		}
		else
		{
			tet_infoline("SIGINT is not pending");
			++nerrs;
		}
	}
	else
	{
		vsth_surprise("sigpending", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

/******	Test pending till ignored						******/
void *
t3_t3( void *arg )
{
	int			rval;
	int			nerrs;
	int			nfails;
	void			*tval;
	sigset_t		set;
	sigset_t		pending_set;
	struct sigaction	sa;


	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGINT );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &nerrs);
	}

	rval = pthread_mutex_lock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	++T3_flag;

	rval = pthread_cond_signal( &T3_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	/* we're signaled while in the wait */
	rval = pthread_cond_wait( &T3_cv, &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
	}

	rval = pthread_mutex_unlock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
	}

	rval = sigpending( &pending_set );
	if ( rval == 0 )
	{
		rval = sigismember( &pending_set, SIGINT );
		if ( rval )
		{
			sa.sa_handler	= SIG_IGN;
			sa.sa_flags	= 0;
			(void) sigfillset( &sa.sa_mask );

			rval = sigaction( SIGINT, &sa, NULL );
			if ( rval == 0 )
			{
				rval = sigpending( &pending_set );
				if ( rval == 0 )
				{
					rval = sigismember( &pending_set, SIGINT );
					if ( rval )
					{
						tet_infoline("SIGINT is still pending");
						++nfails;
					}
				}
				else
				{
					vsth_surprise("sigpending", 0, rval, &nerrs);
				}
			}
			else
			{
				vsth_surprise("sigaction", 0, rval, &nerrs);
			}
		}
		else
		{
			tet_infoline("SIGINT is not pending");
			++nerrs;
		}
	}
	else
	{
		vsth_surprise("sigpending", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
t3_ssigs( int *errors, int *fails )
{
	int			rval;
	sigset_t		set;
	struct sigaction	sa;

	sa.sa_flags	= 0;
	sa.sa_handler	= t3_sigint;

	rval = sigfillset( &sa.sa_mask );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, errors);
	}

	rval = pthread_sigmask( SIG_SETMASK, &sa.sa_mask, &T3_oset );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
	}

	rval = sigaction( SIGINT, &sa, &T3_osa );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}
}

void
t3_sigint( int sig )
{
	++T3_nsigs;
}

void
t3_rsigs( int *errors, int *fails )
{
	int		rval;

	rval = pthread_sigmask( SIG_SETMASK, &T3_oset, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
	}

	rval = sigaction( SIGINT, &T3_osa, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}
}

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When there are no threads in a call to sigwait() selecting that");
	tet_infoline("signal, and all threads within the process block delivery of the");
	tet_infoline("signal, then the signal shall remain pending on the process until");
	tet_infoline("either a thread calls sigwait() selecting that signal, a thread");
	tet_infoline("unblocks delivery of the signal, or the action associated with");
	tet_infoline("the signal is set to ignore the signal.");

	tet_infoline("Posix Ref: Component SIGWAIT");
	tet_infoline("Assertion 9945-1:1996 3.3.8.2, 9945-1:1996 3.3.1.2-4(A)");

	tet_infoline("Assertion 4 is a duplicate of assertion 3");
	tet_result(TET_UNTESTED);
}

void
test5()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When a signal is delivered to a thread and the action of");
	tet_infoline("that signal specifies termination, stop, or continue,");
	tet_infoline("then the entire process shall be terminated, stopped or");
	tet_infoline("continued respectively.");

	tet_infoline("Posix Ref: Component SIGWAIT");
	tet_infoline("Assertion 9945-1:1996 3.3.8.2, 9945-1:1996 3.3.1.2-5(A)");

	t5_f1( &errors, &fails );

#ifdef	_POSIX_JOB_CONTROL

	t5_f2( &errors, &fails );

#endif

	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

/******	Test the process termination case.
	Fork a child. Wait on it. The child sets the default behavior for SIGTERM
	then creates a thread. The thread sends itself a SIGTERM. The parent
	checks that the process has exited appropriately.			******/

void
t5_f1( int *errors, int *fails )
{
	int			rval;
	pid_t			pid;

	tet_infoline("Testing process termination");

	pid = fork();
	if ( pid == 0 )
	{
		t5_c1();
	}
	else if ( pid > 0 )
	{
		t5_p1( pid, errors, fails );
	}
	else
	{
		++*errors;
		tet_printf("fork unexpectedly failed, errno == %d", errno);
	}
} 

void
t5_c1()
{
	int			rval;
	pthread_t		tid;
	void			*tval;
	struct sigaction	sa;

	/* set SIGTERM action to SIG_DFL */

	rval = sigfillset( &sa.sa_mask );
	if ( rval != 0 )
	{
		tet_printf("sigfillset failed, errno == %d", errno);
		exit(1);
	}

	sa.sa_flags	= 0;
	sa.sa_handler	= SIG_DFL;

	rval = sigaction( SIGTERM, &sa, NULL );
	if ( rval != 0 )
	{
		tet_printf("sigaction failed, errno == %d", errno);
		exit(1);
	}

	rval = vsth_pthread_create( &tid, NULL, t5_t1, NULL );
	if ( rval != 0 )
	{
		tet_printf("vsth_pthread_create failed, rval == %d", rval);
		exit(1);
	}

	/* pause(); */
	(void) vsth_pthread_join(tid, &tval);
	exit(2);
}

void
t5_p1( pid_t pid, int *errors, int *fails )
{

	int	rval;
	pid_t	wval;
	int	status;
	int	signum;

	/* we expect the child process to show a WIFSIGNALED status */

	wval = waitpid( pid, &status, 0 );
	if ( wval == pid )
	{
		if ( WIFEXITED(status) )
		{
			++*fails;
			tet_printf("Process exited with a status of %d", WEXITSTATUS(status));
		}
		else if (  WIFSIGNALED(status) )
		{
			signum = WTERMSIG(status);
	
			if ( signum != SIGTERM )
			{
				++*fails;
				tet_printf("Process was terminated with unexpected signal %d", signum);
			}
		}
		else
		{
			++*fails;
			tet_printf("Unexpected process termination, status == %d", status);
			(void) kill(pid, SIGKILL);
		}
	}
	else
	{
		++*errors;
		tet_printf("waitpid unexpectedly failed, errno == %d", errno);
		(void) kill(pid, SIGKILL);
	}
}

#ifdef	_POSIX_JOB_CONTROL

/******	Test the SIGSTOP/SIGCONT case. Fork. The child process creates a thread.
	The thread sends itself a SIGSTOP. The parent obtains the child process
	status and then continues the thread with a SIGCONT. The child joins the
	thread and exits. The parent again obtains the child process status. 	******/


int			T5_flag;
int			T5_chldflag;
int			T5_contflag;
pthread_t		T5_tid;
struct sigaction	T5_osa_cont;
struct sigaction	T5_osa_chld;
pthread_mutex_t		T5_mx	= PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t		T5_cv	= PTHREAD_COND_INITIALIZER;

void
t5_f2( int *errors, int *fails )
{
	int			rval;
	pid_t			pid;
	struct sigaction	sa;

	tet_infoline("Testing suspend and continue");

	rval = sigfillset( &sa.sa_mask );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, errors);
		return;
	}

	sa.sa_flags	= SA_NOCLDSTOP;
	sa.sa_handler	= t5_sigchld;

	rval = sigaction( SIGCHLD, &sa, &T5_osa_chld );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction SIGCONT", 0, rval, errors);
		return;
	}

	sa.sa_handler	= t5_sigcont;
	rval = sigaction( SIGCONT, &sa, &T5_osa_cont );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction SIGCONT", 0, rval, errors);
		return;
	}

	pid = fork();
	if ( pid == 0 )
	{
		t5_c2();
	}
	else if ( pid > 0 )
	{
		t5_p2( pid, errors, fails );
	}
	else
	{
		vsth_surprise("fork", 0, rval, errors);
	}

	rval = sigaction( SIGCHLD, &T5_osa_chld, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction SIGCHLD", 0, rval, errors);
	}

	rval = sigaction( SIGCHLD, &T5_osa_cont, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction SIGCHLD", 0, rval, errors);
	}
} 

void
t5_p2( pid_t pid, int *errors, int *fails )
{

	int	rval;
	pid_t	wval;
	int	status;

	wval = waitpid( pid, &status, WUNTRACED );
	if ( wval == pid )
	{
		if ( WIFSTOPPED(status) )
		{
			rval = kill( pid, SIGCONT );
			if ( rval == 0 )
			{
				while ( wval = waitpid( pid, &status, 0 ) )
				{
					if ( wval == pid )
					{
						if ( ! (WIFEXITED(status) && ( WEXITSTATUS(status) == 0 )))
						{
							++*fails;
							tet_printf("Unexpected process exit status 0x%x", status);
						}
						break;
					}
					else if (( wval == (pid_t) -1 ) && ( errno == EINTR ))
					{
						continue;
					}
					else
					{
						++*errors;
						tet_printf("second waitpid unexpectedly failed, errno == %d", errno);
						(void) kill( pid, SIGKILL );
						break;
					}
				}
			}
			else
			{
				++*errors;
				tet_printf("kill unexpectely failed, errno == %d", errno);
				(void) kill(pid, SIGKILL);
			}
		}
		else
		{
			++*errors;
			tet_printf("Unexpected process exit status 0x%x", status);
			(void) kill(pid, SIGKILL);
		}
	}
	else
	{
		++*fails;
		tet_printf("first waitpid unexpectedly failed, errno == %d", errno);
		(void) kill(pid, SIGKILL);
		return;
	}
}

void
t5_c2()
{
	int		rval;
	void		*tval;

	T5_flag = 0;

	rval = pthread_mutex_lock( &T5_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, NULL);
		exit(1);
	}

	rval = vsth_pthread_create( &T5_tid, NULL, t5_t2, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, NULL);
		exit(1);
	}

	while ( ! T5_flag )
	{
		rval = pthread_cond_wait( &T5_cv, &T5_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, NULL);
			(void) pthread_cancel( T5_tid );
			(void) vsth_pthread_detach( T5_tid );
			exit(1);
		}
	}

	rval = pthread_mutex_unlock( &T5_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, NULL);
	}

	rval = vsth_pthread_join( T5_tid, &tval );
	if ( rval == 0 )
	{
		exit((int) tval);
	}
	else
	{
		(void) pthread_cancel( T5_tid );
		(void) vsth_pthread_detach( T5_tid );
		vsth_surprise("vsth_pthread_join", 0, rval, NULL);
		exit(1);
	}
}

#endif	/* _POSIX_JOB_CONTROL */

void *
t5_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_kill( pthread_self(), SIGTERM );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_kill", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#ifdef	_POSIX_JOB_CONTROL

void *
t5_t2( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_mutex_lock( &T5_mx );
	if ( rval == 0 )
	{

		++T5_flag;
	
		rval = pthread_cond_signal( &T5_cv );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
		}
	
		rval = pthread_kill( pthread_self(), SIGSTOP );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_kill", 0, rval, &nerrs);
		}
	
		rval = pthread_mutex_unlock( &T5_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
		}
	}
	else
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
t5_sigcont( int sig )
{
	++T5_contflag;
}

void
t5_sigchld( int sig )
{
	++T5_chldflag;
}

#endif	/* _POSIX_JOB_CONTROL */

void
test6()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the signals specified by set are blocked from delivery");
	tet_infoline("to the calling thread,");
	tet_infoline("then a call to sigwait(set, sig) selects a pending signal from");
	tet_infoline("set, atomically clears it from the set of pending signals for");
	tet_infoline("the calling thread, stores that signal number in the location");
	tet_infoline("referenced by sig and returns 0 if successful.");

	tet_infoline("Posix Ref: Component SIGWAIT");
	tet_infoline("Assertion 9945-1:1996 3.3.8.2-6(A)");

	t6_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int			T6_flag;
int			T6_nsigs;
int			T6_sig;
sigset_t		T6_oset;
struct sigaction	T6_int_osa;
struct sigaction	T6_pipe_osa;
pthread_mutex_t		T6_mx;
pthread_cond_t		T6_cv;

/******	Block and set up a handler for SIGINT and SIGPIPE. Create a thread.
	The thread unblocks both signals, signals us, then goes into a
	pthread_cond_wait. We signal the thread with both signals. We then
	continue the thread and join it. The thread checks pending signals,
	performs a sigwait, then checks pending and delivered signals and
	exits with an appropriate status code.					******/

void
t6_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	T6_flag		= 0;
	T6_nsigs	= 0;
	T6_sig		= 0;

	/* set up a handler for SIGINT and for SIGPIPE */
	t6_ssigs( errors, fails );
	if ( *errors )
		return;

	rval = pthread_mutex_init( &T6_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		t6_rsigs( errors, fails );
		return;
	}

	rval = pthread_cond_init( &T6_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		(void) pthread_mutex_destroy( &T6_mx );
		t6_rsigs( errors, fails );
		return;
	}

	rval = pthread_mutex_lock( &T6_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, errors);
		(void) pthread_mutex_destroy( &T6_mx );
		(void) pthread_cond_destroy( &T6_cv );
		t6_rsigs( errors, fails );
		return;
	}

	rval = vsth_pthread_create( &tid, NULL, t6_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		(void) pthread_mutex_destroy( &T6_mx );
		(void) pthread_cond_destroy( &T6_cv );
		t6_rsigs( errors, fails );
		return;
	}

	while ( ! T6_flag )
	{
		rval = pthread_cond_wait( &T6_cv, &T6_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
		}
	}

	rval = pthread_kill( tid, SIGINT );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_kill", 0, rval, errors);
	}

	rval = pthread_kill( tid, SIGPIPE );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_kill", 0, rval, errors);
	}

	++T6_flag;

	rval = pthread_cond_signal( &T6_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, errors);
	}

	rval = pthread_mutex_unlock( &T6_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
		else if ( tval )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	(void) pthread_mutex_destroy( &T6_mx );
	(void) pthread_cond_destroy( &T6_cv );
	t6_rsigs( errors, fails );
}

void *
t6_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	sigset_t	set;
	sigset_t	pending_set;


	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGINT );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGPIPE );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_BLOCK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = pthread_mutex_lock( &T6_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	++T6_flag;

	rval = pthread_cond_signal( &T6_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	while ( T6_flag != 2 )
	{
		/* we're hit with a SIGINT and a SIGPIPE while blocked here */
		rval = pthread_cond_wait( &T6_cv, &T6_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
		}
	}

	rval = pthread_mutex_unlock( &T6_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
	}

	rval = t6_precheck();
	if ( rval == 0 )
	{
		rval = sigwait( &set, &T6_sig );
		if ( rval == 0 )
		{
			t6_postcheck( T6_sig, &nerrs, &nfails );
		}
		else
		{
			vsth_surprise("sigwait", 0, rval, &nfails);
		}
	}
	else
	{
		++nerrs;
		tet_infoline("No signals pending");
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

int
t6_precheck()
{
	int		rval;
	int		npending;
	sigset_t	pending_set;

	npending = 0;

	rval = sigpending( &pending_set );
	if ( rval == 0 )
	{
		if ( sigismember( &pending_set, SIGINT ) )
		{
			++npending;
		}
		else
		{
			tet_infoline("SIGINT is not pending");
		}

		if ( sigismember( &pending_set, SIGPIPE ) )
		{
			++npending;
		}
		else
		{
			tet_infoline("SIGPIPE is not pending");
		}
	}
	else
	{
		tet_printf("sigpending failed, errno == %d", errno);
	}

	return(npending == 2 ? 0 : 1);
}

void
t6_postcheck( int sig, int *errors, int *fails )
{

	int		rval;
	int		anysig;
	int		pending_sig;
	sigset_t	pending_set;

	pending_sig = 0;

	rval = sigpending( &pending_set );
	if ( rval != 0 )
	{
		vsth_surprise("sigpending", 0, rval, errors);
		return;
	}

	if ( sig == SIGINT )
	{
		pending_sig = SIGPIPE;
	}
	else if ( sig == SIGPIPE )
	{
		pending_sig = SIGINT;
	}
	else
	{
		++*errors;
		tet_printf("Unexpected signal %d", sig);
	}

	/* check that the received signal is no longer pending */
	if ( sigismember( &pending_set, sig ) )
	{
		++*fails;
		tet_printf("Signal %d is pending", sig);
	}

	/* check that the other signal is still pending */
	if ( ! sigismember( &pending_set, pending_sig ) )
	{
		++*fails;
		tet_printf("Signal %d is not pending", pending_sig);
	}

	/* get rid of the other signal */
	(void) sigemptyset( &pending_set );
	(void) sigaddset( &pending_set, pending_sig );
	(void) sigwait( &pending_set, &anysig );
}

void
t6_ssigs( int *errors, int *fails )
{
	int			rval;
	sigset_t		set;
	struct sigaction	sa;

	sa.sa_handler	= t6_sh;
	sa.sa_flags	= 0;

	rval = sigfillset( &sa.sa_mask );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, errors);
	}

	rval = pthread_sigmask( SIG_SETMASK, &sa.sa_mask, &T6_oset );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
	}

	rval = sigaction( SIGINT, &sa, &T6_int_osa );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}

	rval = sigaction( SIGPIPE, &sa, &T6_pipe_osa );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}
}

void
t6_rsigs( int *errors, int *fails )
{
	int		rval;

	rval = pthread_sigmask( SIG_SETMASK, &T6_oset, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
	}

	rval = sigaction( SIGINT, &T6_int_osa, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}

	rval = sigaction( SIGPIPE, &T6_pipe_osa, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}
}

void
t6_sh( int sig )
{
	++T6_nsigs;
}

void
test7()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the implementation supports queued signals:");
	tet_infoline("when there are multiple signals queued for the signal number");
	tet_infoline("selected, then the first such queued signal shall cause a return");
	tet_infoline("from sigwait() and the remainder shall remain queued.");

	tet_infoline("Posix Ref: Component SIGWAIT");
	tet_infoline("Assertion 9945-1:1996 3.3.8.2-7(C)");

#ifdef	_POSIX_REALTIME_SIGNALS

	t7_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_REALTIME_SIGNALS is not defined");
	result = TET_UNSUPPORTED;

#endif

	tet_result(result);
}

#ifdef	_POSIX_REALTIME_SIGNALS

int			T7_flag;
int			T7_nsigs;
int			T7_sig;
sigset_t		T7_oset;
struct sigaction	T7_int_osa;
struct sigaction	T7_pipe_osa;
pthread_mutex_t		T7_mx;
pthread_cond_t		T7_cv;

/******	Block and set up a handler for SIGINT and SIGPIPE. Create a thread.
	The thread blocks both signals, signals us, then goes into a
	pthread_cond_wait. We signal the thread with both signals. We then
	continue the thread and join it. The thread checks pending signals,
	performs a sigwait, then checks pending and delivered signals and
	exits with an appropriate status code.					******/

void
t7_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;
	union sigval	sval;

	T7_flag		= 0;
	T7_nsigs	= 0;
	T7_sig		= 0;

	t7_ssigs( errors, fails );
	if ( *errors )
		return;

	rval = pthread_mutex_init( &T7_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		t7_rsigs( errors, fails );
		return;
	}

	rval = pthread_cond_init( &T7_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		(void) pthread_mutex_destroy( &T7_mx );
		t7_rsigs( errors, fails );
		return;
	}

	rval = pthread_mutex_lock( &T7_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, errors);
		(void) pthread_mutex_destroy( &T7_mx );
		(void) pthread_cond_destroy( &T7_cv );
		t7_rsigs( errors, fails );
		return;
	}

	rval = vsth_pthread_create( &tid, NULL, t7_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		(void) pthread_mutex_destroy( &T7_mx );
		(void) pthread_cond_destroy( &T7_cv );
		t7_rsigs( errors, fails );
		return;
	}

	while ( T7_flag != 1 )
	{
		rval = pthread_cond_wait( &T7_cv, &T7_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
			(void) pthread_cancel( tid );
			(void) vsth_pthread_detach( tid );
			(void) pthread_mutex_destroy( &T7_mx );
			(void) pthread_cond_destroy( &T7_cv );
			t7_rsigs( errors, fails );
			return;
		}
	}

	/* queue two SIGINT signals */
	sval.sival_ptr = NULL;

	rval = sigqueue( getpid(), SIGINT, sval );
	if ( rval != 0 )
	{
		++*errors;
		tet_printf("sigqueue unexpectedly failed, errno == %d", errno);
	}

	rval = sigqueue( getpid(), SIGPIPE, sval );
	if ( rval != 0 )
	{
		++*errors;
		tet_printf("sigqueue unexpectedly failed, errno == %d", errno);
	}

	++T7_flag;

	rval = pthread_cond_signal( &T7_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, errors);
	}

	rval = pthread_mutex_unlock( &T7_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	(void) pthread_mutex_destroy( &T7_mx );
	(void) pthread_cond_destroy( &T7_cv );
	t7_rsigs( errors, fails );
}

void *
t7_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	int		sig;
	void		*tval;
	sigset_t	set;
	sigset_t	pending_set;


	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGINT );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &nerrs);
	}

	rval = sigaddset( &set, SIGPIPE );
	if ( rval != 0 )
	{
		vsth_surprise("sigaddset", 0, rval, &nerrs);
	}

	rval = pthread_mutex_lock( &T7_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	++T7_flag;

	rval = pthread_cond_signal( &T7_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	while ( T7_flag != 2 )
	{
		/* SIGINT and SIGPIPE are queued while we're blocked */
		rval = pthread_cond_wait( &T7_cv, &T7_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
		}
	}

	rval = pthread_mutex_unlock( &T7_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
	}

	rval = t7_check( SIGINT, &nerrs, &nfails );
	if ( rval == 0 )
	{
		rval = sigwait( &set, &T7_sig );
		if ( rval == 0 )
		{
			if ( T7_sig == SIGINT )
			{
				rval = t7_check( SIGPIPE, &nerrs, &nfails );
				if ( rval )
				{
					++nfails;
					tet_infoline("SIGPIPE is not pending");
				}
				else
				{
					/* toss the SIGPIPE */
					(void) sigwait( &set, &sig );
				}
			}
			else
			{
				++nfails;
				tet_infoline("sigwait didn't return SIGINT");
			}
		}
		else
		{
			vsth_surprise("sigwait", 0, rval, &nfails);
		}
	}
	else
	{
		tet_infoline("No signals pending");
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

/* Make sure that we have signal sig pending */

int
t7_check( int sig, int *errors, int *fails )
{
	int		rval;
	int		npending;
	sigset_t	pending_set;

	npending = 0;

	rval = sigpending( &pending_set );
	if ( rval == 0 )
	{
		if ( sigismember( &pending_set, sig ) )
		{
			++npending;
		}
		else
		{
			tet_printf("signal %d is not pending, sig");
		}
	}
	else
	{
		tet_printf("sigpending failed, errno == %d", errno);
	}

	return(npending ? 0 : 1);
}

void
t7_ssigs( int *errors, int *fails )
{
	int			rval;
	sigset_t		set;
	struct sigaction	sa;

	sa.sa_sigaction	= t7_sh;
	sa.sa_flags	= SA_SIGINFO;

	rval = sigfillset( &sa.sa_mask );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, errors);
	}

	rval = pthread_sigmask( SIG_SETMASK, &sa.sa_mask, &T7_oset );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
	}

	rval = sigaction( SIGINT, &sa, &T7_int_osa );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}

	rval = sigaction( SIGPIPE, &sa, &T7_pipe_osa );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}
}

void
t7_rsigs( int *errors, int *fails )
{
	int		rval;

	rval = pthread_sigmask( SIG_SETMASK, &T7_oset, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
	}

	rval = sigaction( SIGINT, &T7_int_osa, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}

	rval = sigaction( SIGPIPE, &T7_pipe_osa, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("sigaction", 0, rval, errors);
	}
}

void
t7_sh( int sig, siginfo_t *info, void *context )
{
	++T7_nsigs;
}

#endif

void
test8()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	int		sig;
	char		*p;
	sigset_t	set;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the implementation detects that the argument set");
	tet_infoline("contains an invalid or unsupported signal number:");
	tet_infoline("a call to sigwait(set, sig) shall fail and return EINVAL.");

	tet_infoline("Posix Ref: Component SIGWAIT");
	tet_infoline("Assertion 9945-1:1996 3.3.8.4-8(C)");

	p = tet_getvar("VSTH_SIG_EINVAL");
	if ( p && *p )
	{
		if (( *p == 'Y' ) || ( *p == 'y' ))
		{
			p = tet_getvar("VSTH_INVALID_SIG");
			if ( p && *p )
			{
				rval = sigemptyset( &set );
				if ( rval == 0 )
				{
					rval = sigaddset( &set, atoi(p) );
					if ( rval == 0 )
					{
						rval = sigwait( &set, &sig );
						if ( rval != EINVAL )
						{
							vsth_surprise("sigwait", EINVAL, rval, &fails);
							result = VSTH_RESULT( errors, fails );
						}
					}
					else if (( rval == -1 ) && ( errno == EINVAL ))
					{
						tet_infoline("sigaddset detects invalid signals");
						result = TET_UNTESTED;
					}
					else
					{
						vsth_surprise("sigaddset", 0, rval, &errors);
						result = VSTH_RESULT( errors, fails );
					}
				}
				else
				{
					vsth_surprise("sigemptyset", 0, rval, &errors);
					result = VSTH_RESULT( errors, fails );
				}
			}
			else
			{
				tet_printf("VSTH_SIG_EINVAL is set to %s", p);
				vsth_badconfig("VSTH_INVALID_SIG", &errors);
				result = VSTH_RESULT( errors, fails );
			}
		}
		else
		{
			tet_printf("VSTH_SIG_EINVAL is set to %s", p);
			result = TET_UNSUPPORTED;
		}
	}
	else
	{
		vsth_badconfig("VSTH_SIG_EINVAL", &errors);
		result = VSTH_RESULT( errors, fails );
	}

	tet_result(result);
}

void
t8_f1( int *errors, int *fails )
{
	int		rval;
	int		sig;
	sigset_t	set;
	char		*p;

	p = tet_getvar("VSTH_INVALID_SIG");
	if ( p && *p )
	{
		rval = sigemptyset( &set );
		if ( rval != 0 )
		{
			vsth_surprise("sigemptyset", 0, rval, errors);
			return;
		}

		/******	there IS the problem of adding an invalid signal to
			a sigset_t. Some implementations don't allow this	******/

		rval = sigaddset( &set, atoi(p) );
		if ( rval == 0 )
		{
			rval = sigwait( &set, &sig );
			if ( rval != EINVAL )
			{
				vsth_surprise("sigwait", EINVAL, rval, fails);
			}
		}
		else if ( rval != EINVAL )
		{
			vsth_surprise("sigaddset", 0, rval, errors);
			return;
		}
	}
	else
	{
		vsth_badconfig("VSTH_INVALID_SIG", errors);
	}
}

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	test5,	5	},
	{	test6,	6	},
	{	test7,	7	},
	{	test8,	8	},
	{	NULL,	0	}
};


