
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/getgrgid_r
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)getgrgid_r.c	1.17
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<grp.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	3

#ifdef  UNDEF_MACROS
#undef  getgrgid_r
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

void	t1_f1( int *errors, int *fails );
void	t2_f1( char *p, int *errors, int *fails );
void	t3_f1( int *errors, int *fails );

void	*t1_t1( void *arg );
void	*t2_t1( void *arg );
void	*t3_t1( void *arg );

#endif

void
startup()
{
	vsth_startup("getgrgid_r");
}

void
cleanup()
{
	vsth_cleanup("getgrgid_r");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when gid is a valid group id from the system group database,");
	tet_infoline("then concurrent calls by multiple threads to");
	tet_infoline("getgrgid_r(gid, grp, buffer, bufsize, result) shall update the");
	tet_infoline("group structure referenced by grp with information about the group");
	tet_infoline("specified by gid, store a pointer to that structure at the");
	tet_infoline("location referenced by result, and return 0 if successful.");

	tet_infoline("Posix Ref: Component GETGRGID_R");
	tet_infoline("Assertion 9945-1:1996 9.2.1.2-1(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}


#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T1_NTHREADS	8

int		T1_flag;
long		T1_bufsize;
gid_t		T1_gid;
pthread_mutex_t	T1_mx;
pthread_cond_t	T1_cv;
VSTH_TSET	T1_tset[T1_NTHREADS];

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;
	char		*p;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
		return;

	p = tet_getvar("VSX_GID0");
	if (( !p ) || ( ! *p ))
	{
		vsth_badconfig("VSX_GID0", errors);
		return;
	}

	T1_gid = (gid_t) atoi(p);

	T1_bufsize = sysconf( _SC_GETGR_R_SIZE_MAX );
	if ( T1_bufsize == -1 )
	{
		++*errors;
		tet_infoline("sysconf unexpectedly failed");
		return;
	}

	T1_flag = T1_NTHREADS;

	vsth_weave( &T1_tset[0], T1_NTHREADS, t1_t1, &T1_mx, &T1_cv,
				SCHED_RR, priomin, 0, errors, fails );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	char		*buf;
	struct group	grp;
	struct group	*result;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T1_cv, &T1_mx, &T1_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	buf = (char *) malloc( T1_bufsize * sizeof(char) );
	if ( buf )
	{
		rval = getgrgid_r( T1_gid, &grp, buf, T1_bufsize, &result );
		if ( rval == 0 )
		{
			if (	( ! result->gr_name)	||
				strcmp( "vsxg0", result->gr_name ))
			{
				++nfails;
				tet_printf("Expected group name of vsxg0, got %s",
						result->gr_name);
			}
	
			if ( T1_gid != result->gr_gid )
			{
				++nfails;
				tet_printf("Expected group id of %d, got %d",
						T1_gid, result->gr_gid);
			}
	
			if ( result != &grp )
			{
				++nfails;
				tet_printf("Expected result %p, got %p", &grp, result);
			}
		}
		else
		{
			vsth_surprise("getgrgid_r", 0, rval, &nfails);
		}

		(void) free( buf );
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when gid is not a valid group id from the system group database,");
	tet_infoline("then concurrent calls by multiple threads to");
	tet_infoline("getgrgid_r(gid, grp, buffer, bufsize, result) shall store a NULL");
	tet_infoline("pointer in the location referenced by result and return 0.");

	tet_infoline("Posix Ref: Component GETGRGID_R");
	tet_infoline("Assertion 9945-1:1996 9.2.1.2-2(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	p = tet_getvar("VSX_INVALID_GID");
	if ( p && *p )
	{
		if ( strcmp(p, "unsup") )
		{
			t2_f1( p, &errors, &fails );
			result = VSTH_RESULT( errors, fails );
		}
		else
		{
			tet_infoline("The implementation considers gid_t values to be valid");
			result = TET_UNSUPPORTED;
		}
	}
	else 
	{
		vsth_badconfig("VSX_INVALID_GID", &errors);
		result = VSTH_RESULT( errors, fails );
	}

#else

#if	( TEST_XPG > 4 )
	tet_infoline("_POSIX_THREAD_SAFE_FUNCTIONS is not defined");
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T2_NTHREADS	8

gid_t			T2_gid;
int			T2_flag;
pthread_mutex_t		T2_mx;
pthread_cond_t		T2_cv;
VSTH_TSET		T2_tset[T2_NTHREADS];
long			T2_bufsize;

void
t2_f1( char *gid, int *errors, int *fails )
{
	int		rval;
	int		priomin;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors );
	if ( *errors )
		return;

	T2_gid		= (gid_t) atoi(gid);
	T2_flag		= T2_NTHREADS;
	T2_bufsize	= sysconf( _SC_GETGR_R_SIZE_MAX );
	if ( T2_bufsize == -1 )
	{
		++*errors;
		tet_infoline("sysconf unexpectedly failed");
		return;
	}

	vsth_weave( &T2_tset[0], T2_NTHREADS, t2_t1, &T2_mx, &T2_cv,
			SCHED_RR, priomin, 0, errors, fails );
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	char		*buf;
	struct group	grp;
	struct group	*result;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = vsth_bwait( &T2_cv, &T2_mx, &T2_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	buf = (char *) malloc( T2_bufsize * sizeof(char) );
	if ( buf )
	{

		rval = getgrgid_r( T2_gid, &grp, buf, T2_bufsize, &result );
		if ( rval == 0 )
		{
			if ( result != NULL )
			{
				++nfails;
				tet_printf("Expected NULL result, got %p", result);
			}
		}
		else
		{
			vsth_surprise("getgrgid_r", 0, rval, &nfails);
		}

		(void) free(buf);
	}
	else
	{
		++nerrs;
		tet_printf("malloc unexpectedly failed, errno == %d", errno);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_SAFE_FUNCTIONS is defined:");
	tet_infoline("when insufficient storage was supplied via buffer and bufsize to");
	tet_infoline("contain the data referenced by gid,");
	tet_infoline("then concurrent calls by multiple threads to");
	tet_infoline("getgrgid_r(gid, grp, buffer, bufsize, result) shall store a NULL");
	tet_infoline("pointer in the location referenced by result and return ERANGE.");

	tet_infoline("Posix Ref: Component GETGRGID_R");
	tet_infoline("Assertion 9945-1:1996 9.2.1.4-3(C)");

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS

#define	T3_NTHREADS	8

int		T3_flag;
long		T3_bufsize;
gid_t		T3_gid;
pthread_mutex_t	T3_mx;
pthread_cond_t	T3_cv;
VSTH_TSET	T3_tset[T3_NTHREADS];

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	int		priomin;
	char		*p;

	vsth_getprio( SCHED_RR, &priomin, NULL, errors);
	if ( *errors )
		return;

	p = tet_getvar("VSX_GID0");
	if (( ! p ) || ( ! *p ))
	{
		vsth_badconfig("VSX_GID0", errors);
		return;
	}

	T3_gid = (gid_t) atoi(p);

	T3_bufsize = sysconf( _SC_GETGR_R_SIZE_MAX );
	if ( T3_bufsize == -1 )
	{
		++*errors;
		tet_infoline("sysconf unexpectedly failed");
		return;
	}

	T3_flag = T3_NTHREADS;

	vsth_weave( &T3_tset[0], T3_NTHREADS, t3_t1, &T3_mx, &T3_cv,
				SCHED_RR, priomin, 0, errors, fails );
}

void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	char		*buf;
	struct group	grp;
	struct group	*result;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 3;

	rval = vsth_bwait( &T3_cv, &T3_mx, &T3_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	buf = (char *) malloc( T3_bufsize * sizeof(char) );
	if ( buf )
	{
		rval = getgrgid_r( T3_gid, &grp, buf, 0, &result );
		if ( rval == ERANGE )
		{
			if ( result != NULL )
			{
				++nfails;
				tet_printf("Expected NULL result, got %p", result);
			}
		}
		else
		{
			++nfails;
			tet_printf("Expected getgrgid_r to return ERANGE, got %d", rval);
		}

		(void) free( buf );
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	NULL,	0	}
};

