
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_create
*
*	The Open Group, VSTH Threads test suite
*	@(#)pthread_create.c	1.18
*
*	History:
*	Moved all tests to a separate executable.
*
*	Assert that required privilege is set in the startup() function
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"setprv.h"
#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	15

#ifdef  UNDEF_MACROS
#undef  pthread_create
#endif

extern char	**environ;
extern char	*itoa( int );

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();
void	test5();
void	test6();
void	test7();
void	test8();
void	test9();
void	test10();
void	test11();
void	test12();
void	test13();
void	test14();

void
startup()
{
	int i;


	/* assert privilege */

        if (setprv(PRV_SETTHRSCHED) != 0)
        {
                for (i=1; i<=no_tests; i++)
                {
                        tet_delete(i, "setprv(PRV_SETTHRSCHED) failed");
                }
                return;
        }

        if (setprv(PRV_GETTHRSCHED) != 0)
        {
                for (i=1; i<=no_tests; i++)
                {
                        tet_delete(i, "setprv(PRV_GETTHRSCHED) failed");
                }
                return;
        }
	vsth_startup("pthread_create");
}

void
cleanup()
{
	vsth_cleanup("pthread_create");
}

void	test_f1( int *errors, int *fails );
void	test_f4( int *errors, int *fails );

void	test_c1();
void	test_c4();

int	t4_pass = 0;

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("The cancellation state of all newly created threads,");
	tet_infoline("including the thread in which main() is first invoked,");
	tet_infoline("is PTHREAD_CANCEL_ENABLE.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 18.1.1-1(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

#define	TE_PATH		"pthread_create_exectests"

void
test_f1( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork( test_c1, NULL, VSTH_TIMEOUT, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
}

void
test_c1()
{
	char		*argv[2];

	argv[0] = execpath(TE_PATH);
	argv[1] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("The cancellation type of all newly created threads,");
	tet_infoline("including the thread in which main() is first invoked,");
	tet_infoline("is PTHREAD_CANCEL_DEFERRED.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 18.1.1-2(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("Upon thread creation, the value NULL shall be associated");
	tet_infoline("with all defined keys in the new thread.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 17.1.1.2-3(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_create(thread, attr, start_routine, arg)");
	tet_infoline("creates a new thread with the attributes specified by attr");
	tet_infoline("within the process containing the calling thread, stores");
	tet_infoline("the thread identifier of the newly created thread in the");
	tet_infoline("location referenced by thread, causes the newly created");
	tet_infoline("thread to execute the routine referenced by start_routine");
	tet_infoline("with arg as its sole argument, and returns 0 if successful.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 16.2.2.2-4(A)");

	t4_pass = 1;
	test_f4( &errors, &fails );

#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING

	/* testing scheduling related attributes */
	t4_pass = 2;
	test_f4( &errors, &fails );

#else

	tet_infoline("Unable to test thread scheduling parameters");
	tet_infoline("_POSIX_THREAD_PRIORITY_SCHEDULING is not defined");

#endif

	t4_pass = 3;
	test_f4( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test_f4( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork( test_c4, NULL, VSTH_TIMEOUT, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
}

void
test_c4()
{
	char		*argv[3];
	char		ascii_T4[20];

	argv[0] = execpath(TE_PATH);
	argv[1] = strcpy(ascii_T4, itoa(t4_pass));
	argv[2] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

void
test5()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the attr argument is NULL,");
	tet_infoline("then a call to pthread_create(thread, attr, start_routine, arg)");
	tet_infoline("creates a thread using the default attributes.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 16.2.2.2-5(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test6()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("Modification of the thread attributes object referenced by attr");
	tet_infoline("after a successful call to");
	tet_infoline("pthread_create(thread, attr, start_routine, arg) does not");
	tet_infoline("modify attributes of the newly created thread.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 16.2.2.2-6(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test7()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the routine referenced by start_routine exits,");
	tet_infoline("then the effect shall be as if there was an implicit call to");
	tet_infoline("pthread_exit() using the return value of start_routine");
	tet_infoline("as the exit status.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 16.2.2.2-7(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test8()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("The signal mask of a newly created thread is that of the");
	tet_infoline("calling thread.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 16.2.2.2-8(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test9()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("The set of signals pending for a newly created thread shall");
	tet_infoline("be empty.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 16.2.2.2-9(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test10()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the detachstate attribute of the thread attribute object");
	tet_infoline("referenced by attr is set to PTHREAD_CREATE_JOINABLE,");
	tet_infoline("then a call to pthread_create(thread, attr, start_routine, arg)");
	tet_infoline("shall cause the new thread to be created in the joinable state.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2-10(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test11()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the detachstate attribute of the thread attribute object");
	tet_infoline("referenced by attr is set to PTHREAD_CREATE_DETACHED,");
	tet_infoline("then a call to pthread_create(thread, attr, start_routine, arg)");
	tet_infoline("shall cause the new thread to be created in the detached state.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2-11(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test12()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("The cancelability state and type of any newly created threads");
	tet_infoline("shall be PTHREAD_CANCEL_ENABLE and PTHREAD_CANCEL_DEFERRED");
	tet_infoline("respectively.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 18.2.2.2-12(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test13()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_PRIORITY_SCHEDULING");
	tet_infoline("is defined:");
	tet_infoline("when the inheritsched attribute of the thread attribute object");
	tet_infoline("referenced by attr has the value PTHREAD_INHERIT_SCHED,");
	tet_infoline("then any thread created by a call to");
	tet_infoline("pthread_create(thread, attr, start_routine, arg) inherits its");
	tet_infoline("scheduling policy and associated attributes from the");
	tet_infoline("calling thread and the scheduling attributes in the attr");
	tet_infoline("argument are ignored.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 13.5.1.2-13(C)");

#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_PRIORITY_SCHEDULING is not defined");
#if	defined(_XOPEN_REALTIME_THREADS) && (_XOPEN_REALTIME_THREADS != -1) && ( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

void
test14()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_PRIORITY_SCHEDULING");
	tet_infoline("is defined:");
	tet_infoline("when the inheritsched attribute of the thread attribute object");
	tet_infoline("referenced by attr has the value PTHREAD_EXPLICIT_SCHED,");
	tet_infoline("then any thread created by a call to");
	tet_infoline("pthread_create(thread, attr, start_routine, arg) has its");
	tet_infoline("scheduling policy and associated attributes set to the");
	tet_infoline("corresponding values from the attributes object.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 13.5.1.2-14(C)");

#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_PRIORITY_SCHEDULING is not defined");
#if	defined(_XOPEN_REALTIME_THREADS) && (_XOPEN_REALTIME_THREADS != -1) && ( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

void
test15()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_create() shall not return EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_CREATE");
	tet_infoline("Assertion 9945-1:1996 2.4-15(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	test5,	5	},
	{	test6,	6	},
	{	test7,	7	},
	{	test8,	8	},
	{	test9,	9	},
	{	test10,	10	},
	{	test11,	11	},
	{	test12,	12	},
	{	test13,	13	},
	{	test14,	14	},
	{	test15,	15	},
	{	NULL,	0	}
};

