
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_create
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_create_exectests.c	1.2
*
*	History:
*	Moved all tests to a separate executable to avoid calls to non-asynch
*	safe functions.  Avoid use of tet_pthread_create(), tet_pthread_join()
*	and tet_pthread_detach().
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#ifdef  UNDEF_MACROS
#undef  pthread_create
#endif

int	tet_nosigreset = 0;

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );
void	t4_f1( int *errors, int *fails );
#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING
void	t4_f2( int *errors, int *fails );
#endif
void	t4_f3( int *errors, int *fails );
void	t5_f1( int *errors, int *fails );
void	t6_f1( int *errors, int *fails );
void	t7_f1( int *errors, int *fails );
void	t8_f1( int *errors, int *fails );
void	t9_f1( int *errors, int *fails );
void	t10_f1( int *errors, int *fails );
void	t11_f1( int *errors, int *fails );
void	t12_f1( int *errors, int *fails );
#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING
void	t13_f1( int *errors, int *fails );
void	t14_f1( int *errors, int *fails );
#endif
void	t15_f1( int *errors, int *fails );

void	*t1_t1( void *arg );
void	*t2_t1( void *arg );
void	*t3_t1( void *arg );
#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING
void	*t4_t2( void *arg );
#endif
void	*t6_t1( void *arg );
void	*t7_t1( void *arg );
void	*t8_t1( void *arg );
void	*t9_t1( void *arg );
void	*t9_t2( void *arg );
int	t9_ssigs();
int	t9_rsigs();
void	t9_sigterm( int sig );
void	*t12_t1( void *arg );
#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING
void	*t13_t1( void *arg );
void	*t14_t1( void *arg );
#endif

int
tet_main( int argc, char **argv)
{
	int	errors;
	int	fails;
	int	rval;
	int	t4_subtest;

	errors = 0;
	fails = 0;

	switch (tet_thistest) {
		case 1:		t1_f1( &errors, &fails );
				break;
		case 2:		t2_f1( &errors, &fails );
				break;
		case 3:		t3_f1( &errors, &fails );
				break;
		case 4:		if (argc >= 2)
				{
					t4_subtest = atoi(argv[1]);
					switch (t4_subtest) {
					case 1:
						t4_f1( &errors, &fails );
						break;
#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING
					case 2:
						t4_f2( &errors, &fails );
						break;
#endif
					case 3:
						t4_f3( &errors, &fails );
						break;
					default:
						tet_printf("Wrong subtest number for test 4");
						errors++;
						break;
					}
				}
				else
				{
					tet_printf("Wrong number of arguments to tet_exec()");
					errors++;
				}

				break;
		case 5:		t5_f1( &errors, &fails );
				break;
		case 6:		t6_f1( &errors, &fails );
				break;
		case 7:		t7_f1( &errors, &fails );
				break;
		case 8:		t8_f1( &errors, &fails );
				break;
		case 9:		t9_f1( &errors, &fails );
				break;
		case 10:	t10_f1( &errors, &fails );
				break;
		case 11:	t11_f1( &errors, &fails );
				break;
		case 12:	t12_f1( &errors, &fails );
				break;
#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING
		case 13:	t13_f1( &errors, &fails );
				break;
		case 14:	t14_f1( &errors, &fails );
				break;
#endif
		case 15:	t15_f1( &errors, &fails );
				break;
		default:
				tet_printf("Invalid test number");
				errors++;
				break;
	}

	rval = 1;
	if ( fails )
		rval = 2;
	if (( errors == 0) && (fails == 0))
		rval = 0;

	exit(rval);
}

int			T1_flag;
pthread_mutex_t		T1_mx;
pthread_cond_t		T1_cv;

/******	Create a thread, synchronize, then cancel it. Check that the thread
	was cancelled.							******/
void
t1_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	T1_flag = 0;

	rval = pthread_mutex_init( &T1_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		return;
	}

	rval = pthread_cond_init( &T1_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		(void) pthread_mutex_destroy( &T1_mx );
		return;
	}

	rval = pthread_mutex_lock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, errors);
		(void) pthread_mutex_destroy( &T1_mx );
		(void) pthread_cond_destroy( &T1_cv );
		return;
	}

	rval = pthread_create( &tid, NULL, t1_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		(void) pthread_mutex_destroy( &T1_mx );
		(void) pthread_cond_destroy( &T1_cv );
		return;
	}

	while ( ! T1_flag )
	{
		rval = pthread_cond_wait( &T1_cv, &T1_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
		}
	}

	rval = pthread_cancel( tid );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cancel", 0, rval, errors);
	}

	rval= pthread_mutex_unlock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval != (void *) PTHREAD_CANCELED )
		{
			vsth_surprise(	"Thread exitval",
					(int) PTHREAD_CANCELED,
					(int) tval,
					fails );
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, errors);
	}

	(void) pthread_mutex_destroy( &T1_mx );
	(void) pthread_cond_destroy( &T1_cv );
}

void *
t1_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_mutex_lock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	++T1_flag;

	rval = pthread_cond_signal( &T1_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	/* we get cancelled while blocked in this wait */
	rval = pthread_cond_wait( &T1_cv, &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
	}

	rval = pthread_mutex_unlock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;
	int		state;

	rval = pthread_create( &tid, NULL, t2_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval != (void *) PTHREAD_CANCELED )
		{
			vsth_surprise(	"Thread exitval",
					(int) PTHREAD_CANCELED,
					(int) tval,
					errors );
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, errors);
	}

	rval = pthread_setcanceltype( PTHREAD_CANCEL_ASYNCHRONOUS, &state );
	if ( rval == 0 )
	{
		if ( state != PTHREAD_CANCEL_DEFERRED )
		{
			vsth_surprise("state", PTHREAD_CANCEL_DEFERRED, state, fails);
		}
	}
	else
	{
		vsth_surprise("pthread_setcanceltype", 0, rval, errors);
	}
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	int		state;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_setcanceltype( PTHREAD_CANCEL_ASYNCHRONOUS, &state );
	if ( rval == 0 )
	{
		if ( state != PTHREAD_CANCEL_DEFERRED )
		{
			vsth_surprise("state", PTHREAD_CANCEL_DEFERRED, state, &nfails);
		}
	}
	else
	{
		vsth_surprise("pthread_setcanceltype", 0, rval, &nerrs);
	}

	rval = pthread_cancel( pthread_self() );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cancel", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

pthread_key_t		T3_key;

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = pthread_key_create( &T3_key, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_key_create", 0, rval, errors);
		return;
	}

	rval = pthread_setspecific( T3_key, (void *) t3_f1 );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_setspecific", 0, rval, errors);
		(void) pthread_key_delete( T3_key );
		return;
	}

	rval = pthread_create( &tid, NULL, t3_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		(void) pthread_key_delete( T3_key);
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, errors);
	}

	(void) pthread_key_delete( T3_key );
}

void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*val;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	val = pthread_getspecific( T3_key );
	if ( val != NULL )
	{
		++nfails;
		tet_printf("Unexpected TSD %p", val );
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

/******	test basic thread creation, return value, and argument handling	******/
void
t4_f1( int *errors, int *fails )
{
	int		rval;
	pthread_attr_t	attr;
	pthread_t	tid;
	void		*tval;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_create( &tid, &attr, vsth_null_thread, (void *) 123);
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, fails);
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval != (void *) 123 )
		{
			vsth_surprise("Thread exitval", 123, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, errors);
	}

	(void) pthread_attr_destroy( &attr );
}

#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING

int		T4_prio;

/******	Test the scheduling parameters					******/
void
t4_f2( int *errors, int *fails )
{
	int			rval;
	int			policy;
	pthread_t		tid;
	void			*tval;
	pthread_attr_t		attr;
	struct sched_param	params;

	vsth_getprio( SCHED_RR, &T4_prio, NULL, errors);
	if ( *errors )
		return;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	params.sched_priority = T4_prio;
	rval = pthread_attr_setschedparam( &attr, &params );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setschedparam", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_attr_setschedpolicy( &attr, SCHED_RR );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setschedpolicy", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_attr_setinheritsched( &attr, PTHREAD_EXPLICIT_SCHED );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setinheritsched", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_create( &tid, &attr, t4_t2, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, errors);
	}

	(void) pthread_attr_destroy( &attr );
}

void *
t4_t2( void *arg )
{
	int			rval;
	int			nerrs;
	int			nfails;
	int			policy;
	struct sched_param	params;
	void			*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_getschedparam( pthread_self(), &policy, &params );
	if ( rval == 0 )
	{
		if ( policy != SCHED_RR )
		{
			vsth_surprise(	"Scheduling policy",
					SCHED_RR,
					policy,
					&nfails );
		}
		if ( params.sched_priority != T4_prio )
		{
			vsth_surprise(	"Scheduling priority",
					T4_prio,
					params.sched_priority,
					&nfails );
		}
	}
	else
	{
		vsth_surprise("pthread_getschedparam", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

/******	Check the detachstate attribute					******/

void
t4_f3( int *errors, int *fails )
{
	int			rval;
	int			policy;
	pthread_t		tid;
	void			*tval;
	pthread_attr_t		attr;
	struct sched_param	params;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_DETACHED );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setdetachstate", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_create( &tid, &attr, vsth_null_thread, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		++*fails;
		tet_infoline("pthread_join unexpectedly succeeded");
	}

	(void) pthread_attr_destroy( &attr );
}

/******	The default attributes that are scheduling related, such as
	policy, priority, scope, and inheritsched are implementation defined.
	There are no portable tests for the stackaddr and stacksize attribute.
	So ... we test the detachstate attribute.			******/

void
t5_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = pthread_create( &tid, NULL, vsth_null_thread, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval != 0 )
	{
		++*fails;
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		tet_infoline("Default detachstate is not PTHREAD_CREATE_JOINABLE");
	}
}

/******	Create a joinable thread. Synchronize. Change the attribute to
	PTHREAD_CREATE_DETACHED. Continue and join the thread.		******/

int		T6_flag;
pthread_mutex_t	T6_mx;
pthread_cond_t	T6_cv;

void
t6_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	pthread_attr_t	attr;
	void		*tval;

	T6_flag = 0;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_JOINABLE );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setdetachstate", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_cond_init( &T6_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_mutex_init( &T6_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		(void) pthread_cond_destroy( &T6_cv );
		return;
	}

	rval = pthread_mutex_lock( &T6_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		(void) pthread_cond_destroy( &T6_cv );
		(void) pthread_mutex_destroy( &T6_mx );
		return;
	}


	rval = pthread_create( &tid, &attr, t6_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		(void) pthread_cond_destroy( &T6_cv );
		(void) pthread_mutex_destroy( &T6_mx );
		return;
	}

	while ( ! T6_flag )
	{
		rval = pthread_cond_wait( &T6_cv, &T6_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
		}
	}

	rval = pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_DETACHED );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setdetachstate", 0, rval, errors);
	}

	rval = pthread_cond_signal( &T6_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, errors);
	}

	rval = pthread_mutex_unlock( &T6_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
	}

	/* join should fail if the thread's detachstate is changed */
	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, fails);
	}

	(void) pthread_attr_destroy( &attr );
	(void) pthread_cond_destroy( &T6_cv );
	(void) pthread_mutex_destroy( &T6_mx );
}

void *
t6_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_mutex_lock( &T6_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	++T6_flag;

	rval = pthread_cond_signal( &T6_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, &nerrs);
	}

	rval = pthread_cond_wait( &T6_cv, &T6_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
	}

	rval = pthread_mutex_unlock( &T6_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
t7_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = pthread_create( &tid, NULL, t7_t1, (void *) t7_t1 );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval != (void *) t7_t1 )
		{
			++*fails;
			tet_printf("Unexpected thread status %p", tval);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, errors);
	}
}

void *
t7_t1( void *arg )
{
	return(arg);
}

sigset_t	T8_set;

void
t8_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = pthread_sigmask( SIG_SETMASK, NULL, &T8_set );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, errors);
		return;
	}

	rval = pthread_create( &tid, NULL, t8_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, errors);
	}
}

void *
t8_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	sigset_t	set;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_sigmask( SIG_SETMASK, NULL, &set );
	if ( rval == 0 )
	{
		if ( memcmp( &T8_set, &set, sizeof(sigset_t) ))
		{
			++nfails;
			tet_infoline("Unexpected signal mask");
		}
	}
	else
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

/******	Create a thread. The thread blocks SIGTERM then sends itself a SIGTERM.
	The thread then creates a second thread. This thread checks
	pending signals with sigpending().				******/

int		T9_sigflag;

void
t9_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	T9_sigflag = 0;

	rval = t9_ssigs();
	if ( rval != 0 )
	{
		++*errors;
		tet_infoline("Unable to initialize signal handling");
		return;
	}

	rval = pthread_create( &tid, NULL, t9_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		(void) t9_rsigs();
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, errors);
	}

	rval = t9_rsigs();
	if ( rval != 0 )
	{
		++*errors;
		tet_infoline("Unable to restore signal handling");
	}
}

void *
t9_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	sigset_t	set;
	pthread_t	tid;
	void		*tval;
	void		*tval1;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigfillset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigfillset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_SETMASK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	rval = pthread_kill( pthread_self(), SIGTERM );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_kill", 0, rval, &nerrs);
	}

	/* we now have a pending SIGTERM */
	rval = pthread_create( &tid, NULL, t9_t2, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, &nerrs);
	}

	rval = pthread_join( tid, &tval1 );
	if ( rval == 0 )
	{
		if ( tval1 == (void *) 1 )
		{
			vsth_surprise("Thread 2 exitval", 0, (int) tval1, &nerrs);
		}
		else if ( tval1 == (void *) 2 )
		{
			vsth_surprise("Thread 2 exitval", 0, (int) tval1, &nfails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, &nerrs);
	}

	/* Allow delivery of the SIGTERM */
	rval = sigemptyset( &set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = pthread_sigmask( SIG_SETMASK, &set, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_sigmask", 0, rval, &nerrs);
	}

	if ( T9_sigflag != 1 )
	{
		vsth_surprise("T9_sigflag", 1, T9_sigflag, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void *
t9_t2( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;
	sigset_t	empty_set;
	sigset_t	pending_set;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = sigemptyset( &empty_set );
	if ( rval != 0 )
	{
		vsth_surprise("sigemptyset", 0, rval, &nerrs);
	}

	rval = sigpending( &pending_set );
	if ( rval != 0 )
	{
		vsth_surprise("sigpending", 0, rval, &nerrs);
	}

	if ( memcmp( &empty_set, &pending_set, sizeof(sigset_t) ) )
	{
		tet_infoline("Pending signals exist for thread 2");
		++nfails;
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

sigset_t		T9_oset;
struct sigaction	T9_osa;

int
t9_ssigs()
{
	int			rval;
	sigset_t		set;
	struct sigaction	sa;

	rval = sigfillset( &sa.sa_mask );
	if ( rval == 0 )
	{
		rval = pthread_sigmask( SIG_SETMASK, &sa.sa_mask, &T9_oset );
		sa.sa_handler	= t9_sigterm;
		sa.sa_flags	= 0;
		rval = sigaction( SIGTERM, &sa, &T9_osa );
	}
	return(rval);
}

void
t9_sigterm( int sig )
{
	++T9_sigflag;
}

int
t9_rsigs()
{
	int	rval;

	rval = pthread_sigmask( SIG_SETMASK, &T9_oset, NULL );
	if ( rval == 0 )
	{
		rval = sigaction( SIGTERM, &T9_osa, NULL );
	}

	return(rval);
}

void
t10_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	pthread_attr_t	attr;
	void		*tval;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_JOINABLE );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setdetachstate", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_create( &tid, &attr, vsth_null_thread, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval != 0 )
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, fails);
	}

	(void) pthread_attr_destroy( &attr );
}

void
t11_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	pthread_attr_t	attr;
	void		*tval;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_DETACHED );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setdetachstate", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_create( &tid, &attr, vsth_null_thread, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		++*fails;
		tet_infoline("pthread_join unexpectedly succeeded");
	}

	(void) pthread_attr_destroy( &attr );
}

void
t12_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = pthread_create( &tid, NULL, t12_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, errors);
	}
}

void *
t12_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	int		cancelstate;
	int		canceltype;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_setcancelstate( NULL, &cancelstate );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_setcancelstate", 0, rval, &nerrs);
	}

	rval = pthread_setcanceltype( NULL, &canceltype );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_setcanceltype", 0, rval, &nerrs);
	}

	if ( cancelstate != PTHREAD_CANCEL_ENABLE )
	{
		++nfails;
		tet_printf("Unexpected cancelstate %d", cancelstate);
	}

	if ( canceltype != PTHREAD_CANCEL_DEFERRED )
	{
		++nfails;
		tet_printf("Unexpected canceltype %d", canceltype);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING

int			T13_policy;
struct sched_param	T13_params;

void
t13_f1( int *errors, int *fails )
{
	int			rval;
	int			newpolicy;
	struct sched_param	params;
	pthread_t		tid;
	void			*tval;
	pthread_attr_t		attr;

	rval = pthread_getschedparam( pthread_self(), &T13_policy, &T13_params );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_getschedparam", 0, rval, errors);
		return;
	}

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	newpolicy = SCHED_RR;
	if ( T13_policy == SCHED_RR )
		newpolicy = SCHED_FIFO;

	vsth_getprio( newpolicy, &params.sched_priority, NULL, errors );
	if ( *errors )
		return;

	rval = pthread_attr_setschedpolicy( &attr, newpolicy );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setschedpolicy", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	if (T13_params.sched_priority == params.sched_priority )
		++params.sched_priority;

	rval = pthread_attr_setschedparam( &attr, &params );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setschedparam", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_attr_setinheritsched( &attr, PTHREAD_INHERIT_SCHED );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setinheritsched", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_create( &tid, &attr, t13_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, errors);
	}

	(void) pthread_attr_destroy( &attr );
}

void *
t13_t1( void *arg )
{
	int			rval;
	int			nerrs;
	int			nfails;
	int			policy;
	void			*tval;
	struct sched_param	params;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_getschedparam( pthread_self(), &policy, &params );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_getschedparam", 0, rval, &nerrs);
	}

	if ( policy != T13_policy )
	{
		vsth_surprise(	"Scheduling policy",
				T13_policy,
				policy,
				&nfails );
	}

	if ( params.sched_priority != T13_params.sched_priority )
	{
		vsth_surprise(	"Scheduling priority",
				T13_params.sched_priority,
				params.sched_priority,
				&nfails );
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING

int			T14_policy;
struct sched_param	T14_params;

void
t14_f1( int *errors, int *fails )
{
	int			rval;
	int			policy;
	struct sched_param	params;
	pthread_t		tid;
	void			*tval;
	pthread_attr_t		attr;

	rval = pthread_getschedparam( pthread_self(), &policy, &params );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_getschedparam", 0, rval, errors);
		return;
	}

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	T14_policy = SCHED_RR;
	if ( policy == SCHED_RR )
		T14_policy = SCHED_FIFO;

	rval = pthread_attr_setschedpolicy( &attr, T14_policy );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setschedpolicy", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	vsth_getprio( T14_policy, &T14_params.sched_priority, NULL, errors);
	if ( *errors )
	{
		(void) pthread_attr_destroy(&attr);
		return;
	}

	if (T14_params.sched_priority == params.sched_priority )
		++T14_params.sched_priority;

	rval = pthread_attr_setschedparam( &attr, &T14_params );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setschedparam", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_attr_setinheritsched( &attr, PTHREAD_EXPLICIT_SCHED );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setinheritsched", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_create( &tid, &attr, t14_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, errors);
	}

	(void) pthread_attr_destroy( &attr );
}

void *
t14_t1( void *arg )
{
	int			rval;
	int			nerrs;
	int			nfails;
	void			*tval;
	int			policy;
	struct sched_param	params;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_getschedparam( pthread_self(), &policy, &params );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_getschedparam", 0, rval, &nerrs);
	}

	if ( policy != T14_policy )
	{
		vsth_surprise(	"Scheduling policy",
				T14_policy,
				policy,
				&nfails );
	}

	if ( params.sched_priority != T14_params.sched_priority )
	{
		vsth_surprise(	"Scheduling priority",
				T14_params.sched_priority,
				params.sched_priority,
				&nfails );
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

#endif

int		t15_tfunc();
int		t15_efunc();

pthread_t	T15_tid;

VSTH_RT		T15_et = {
	NULL,	t15_tfunc,	t15_efunc
};

void
t15_f1( int *errors, int *fails)
{
	int		result;
	int		rval;

	vsth_et( &T15_et, errors, fails );
}

int
t15_tfunc()
{
	int	val;
	int	rval;

	rval	= 1;

	val = pthread_create( &T15_tid, NULL, vsth_null_thread, NULL );
	if ( val != EINTR )
		rval = 0 ;

	return(rval);
}

int
t15_efunc()
{
	void	*tval;

	(void) pthread_join( T15_tid, &tval );
	return(0);
}
