
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_detach
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_detach.c	1.14
*
*	History:
*	Moved tests to a separate executable.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	5

#ifdef  UNDEF_MACROS
#undef  pthread_detach
#endif

extern	char	**environ;

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();
void	test5();
void	test_f1( int *errors, int *fails );
void	test_c1();

void
startup()
{
	vsth_startup("pthread_detach");
}

void
cleanup()
{
	vsth_cleanup("pthread_detach");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_detach(thread) indicates to the implementation");
	tet_infoline("that storage for the thread specified by thread can be reclaimed when");
	tet_infoline("that thread terminates and returns 0 if successful.");

	tet_infoline("Posix Ref: Component PTHREAD_DETACH");
	tet_infoline("Assertion 9945-1:1996 16.2.4.2-1(A)");

	tet_infoline("No portable test for reclaimation of storage");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

#define	TE_PATH		"pthread_detach_exectests"

void
test_f1( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork( test_c1, NULL, VSTH_TIMEOUT, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
}

void
test_c1()
{
	char		*argv[2];

	argv[0] = execpath(TE_PATH);
	argv[1] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_detach(thread) shall not cause the");
	tet_infoline("thread specified by thread to terminate.");

	tet_infoline("Posix Ref: Component PTHREAD_DETACH");
	tet_infoline("Assertion 9945-1:1996 16.2.4.2-2(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the implementation detects that the value specified by the thread");
	tet_infoline("parameter in a call to pthread_detach(thread) does not refer to a");
	tet_infoline("thread that can be joined:");
	tet_infoline("a call to pthread_detach(thread) shall fail and return EINVAL.");

	tet_infoline("Posix Ref: Component PTHREAD_DETACH");
	tet_infoline("Assertion 9945-1:1996 16.2.4.4-3(C)");

	p = tet_getvar("VSTH_DETACH_EINVAL");
	if ( p && *p )
	{
		if (( *p == 'Y' ) || ( *p == 'y' ))
		{
			test_f1( &errors, &fails );
			result = VSTH_RESULT( errors, fails );
		}
		else
		{
			tet_printf("VSTH_DETACH_EINVAL is set to %s", p);
			result = TET_UNSUPPORTED;
		}
	}
	else
	{
		vsth_badconfig("VSTH_DETACH_EINVAL", &errors);
		result = VSTH_RESULT( errors, fails );
	}

	tet_result(result);
}

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the implementation detects that the thread specified by the thread");
	tet_infoline("parameter in a call to pthread_detach(thread) can not be found:");
	tet_infoline("a call to pthread_detach(thread) shall fail and return ESRCH.");

	tet_infoline("Posix Ref: Component PTHREAD_DETACH");
	tet_infoline("Assertion 9945-1:1996 16.2.4.4-4(C)");

	p = tet_getvar("VSTH_TID_ESRCH");
	if ( p && *p )
	{
		if (( *p == 'Y' ) || ( *p == 'y' ))
		{
			test_f1( &errors, &fails );
			result = VSTH_RESULT( errors, fails );
		}
		else
		{
			tet_printf("VSTH_TID_ESRCH is set to %s", p);
			result = TET_UNSUPPORTED;
		}
	}
	else
	{
		vsth_badconfig("VSTH_TID_ESRCH", &errors);
		result = VSTH_RESULT( errors, fails );
	}

	result = VSTH_RESULT( errors, fails );
	tet_result(result);

}

void
test5()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_detach() shall not return EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_DETACH");
	tet_infoline("Assertion 9945-1:1996 2.4-5(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	test5,	5	},
	{	NULL,	0	}
};


