
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_detach
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_detach_exectests.c	1.1
*
*	History:
*	Moved tests to a separate executable to avoid calls to non-asych safe
*	functions.  Avoid calls to tet_pthread_create(), tet_pthread_join() and
*	tet_pthread_detach().
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#ifdef  UNDEF_MACROS
#undef  pthread_detach
#endif

int	tet_nosigreset = 0;

void	t1_f1( int *errors, int *fails );
void	*t1_t1( void *arg );

void	t2_f1( int *errors, int *fails );
void	*t2_t1( void *arg );

void	t3_f1( int *errors, int *fails );
void	*t3_t1( void *arg );

void	t4_f1( int *errors, int *fails );
void	*t4_t1( void *arg );

void	t5_f1( int *errors, int *fails );

int
tet_main( int argc, char **argv)
{
	int	errors;
	int	fails;
	int	rval;

	errors = 0;
	fails = 0;

	switch (tet_thistest) {
		case 1:		t1_f1( &errors, &fails );
				break;
		case 2:		t2_f1( &errors, &fails );
				break;
		case 3:		t3_f1( &errors, &fails );
				break;
		case 4:		t4_f1( &errors, &fails );
				break;
		case 5:		t5_f1( &errors, &fails );
				break;
		default:
				tet_printf("Invalid test number");
				errors++;
				break;
	}

	rval = 1;
	if ( fails )
		rval = 2;
	if (( errors == 0) && (fails == 0))
		rval = 0;

	exit(rval);
}

int		T1_flag;
pthread_cond_t	T1_cv;
pthread_mutex_t	T1_mx;

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	int		breakflag;
	pthread_t	tid;
	void		*tval;

	T1_flag = 0;

	rval = pthread_cond_init( &T1_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		return;
	}

	rval = pthread_mutex_init( &T1_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		(void) pthread_cond_destroy( &T1_cv );
		return;
	}

	rval = pthread_mutex_lock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, errors);
		(void) pthread_cond_destroy( &T1_cv );
		(void) pthread_mutex_destroy( &T1_mx );
		return;
	}

	rval = pthread_create( &tid, NULL, t1_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		return;
	}

	/******	Wait for t1_t1 to announce, so there is no issue of an
		ESRCH from pthread_detach(), then detach and signal it		******/

	while ( T1_flag != 1 )
	{
		rval = pthread_cond_wait( &T1_cv, &T1_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
		}
	}

	++T1_flag;
	rval = pthread_detach( tid );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_detach", 0, rval, fails);
	}

	rval = pthread_cond_signal( &T1_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, errors);
	}

	rval = pthread_mutex_unlock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
	}

	(void) pthread_cond_destroy( &T1_cv );
	(void) pthread_mutex_destroy( &T1_mx );
}

void *
t1_t1( void *arg )
{
	int	rval;
	int	errors;
	void	*tval;

	rval	= 0;
	errors	= 0;
	tval	= (void *) 1;

	rval = pthread_mutex_lock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, NULL);
		++errors;
	}

	++T1_flag;
	rval = pthread_cond_signal( &T1_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, NULL);
		++errors;
	}

	while ( T1_flag != 2 )
	{
		rval = pthread_cond_wait( &T1_cv, &T1_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, NULL);
			++errors;
		}
	}

	rval = pthread_mutex_unlock( &T1_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, NULL);
		++errors;
	}

	if ( errors == 0 )
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

int		T2_flag;
pthread_mutex_t	T2_mx;
pthread_cond_t	T2_cv;

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	int		breakflag;
	pthread_t	tid;
	void		*tval;

	T2_flag = 0;

	rval = pthread_cond_init( &T2_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		return;
	}

	rval = pthread_mutex_init( &T2_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		(void) pthread_cond_destroy( &T2_cv );
		return;
	}

	rval = pthread_mutex_lock( &T2_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, errors);
		(void) pthread_cond_destroy( &T2_cv );
		(void) pthread_mutex_destroy( &T2_mx );
		return;
	}

	rval = pthread_create( &tid, NULL, t2_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		return;
	}

	/******	Wait for t2_t1 to announce, so there is no issue of an
		ESRCH from pthread_detach(), then detach and signal it		******/

	while ( T2_flag != 1 )
	{
		rval = pthread_cond_wait( &T2_cv, &T2_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
		}
	}

	rval = pthread_detach( tid );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_detach", 0, rval, errors);
	}

	rval = pthread_kill( tid, 0 );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_kill", 0, rval, fails );
	}

	++T2_flag;
	rval = pthread_cond_signal( &T2_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, errors);
	}

	rval = pthread_mutex_unlock( &T2_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
	}

	(void) pthread_cond_destroy( &T2_cv );
	(void) pthread_mutex_destroy( &T2_mx );
}

void *
t2_t1( void *arg )
{
	int	rval;
	int	errors;
	void	*tval;

	rval	= 0;
	errors	= 0;
	tval	= (void *) 1;

	rval = pthread_mutex_lock( &T2_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, NULL);
		++errors;
	}

	++T2_flag;
	rval = pthread_cond_signal( &T2_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, NULL);
		++errors;
	}

	while ( T2_flag != 2 )
	{
		rval = pthread_cond_wait( &T2_cv, &T2_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, NULL);
			++errors;
		}
	}

	rval = pthread_mutex_unlock( &T2_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, NULL);
		++errors;
	}

	if ( errors == 0 )
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

int		T3_flag;
pthread_mutex_t	T3_mx;
pthread_cond_t	T3_cv;

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	int		breakflag;
	pthread_attr_t	attr;
	pthread_t	tid;
	void		*tval;

	T3_flag = 0;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_DETACHED );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setdetachstate", 0, rval, errors);
		return;
	}

	rval = pthread_cond_init( &T3_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		return;
	}

	rval = pthread_mutex_init( &T3_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		(void) pthread_cond_destroy( &T3_cv );
		return;
	}

	rval = pthread_mutex_lock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, errors);
		(void) pthread_cond_destroy( &T3_cv );
		(void) pthread_mutex_destroy( &T3_mx );
		return;
	}

	rval = pthread_create( &tid, &attr, t3_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		return;
	}

	/******	Wait for t3_t1 to announce, so there is no issue of an
		ESRCH from pthread_detach(), then detach and signal it		******/

	while ( T3_flag != 1 )
	{
		rval = pthread_cond_wait( &T3_cv, &T3_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, errors);
		}
	}

	rval = pthread_detach( tid );
	if ( rval != EINVAL )
	{
		vsth_surprise("pthread_detach", EINVAL, rval, fails);
	}

	++T3_flag;
	rval = pthread_cond_signal( &T3_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, errors);
	}

	rval = pthread_mutex_unlock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, errors);
	}

	(void) pthread_cond_destroy( &T3_cv );
	(void) pthread_mutex_destroy( &T3_mx );
	(void) pthread_attr_destroy( &attr );
}

void *
t3_t1( void *arg )
{
	int	rval;
	int	errors;
	void	*tval;

	rval	= 0;
	errors	= 0;
	tval	= (void *) 1;

	rval = pthread_mutex_lock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, NULL);
		++errors;
	}

	++T3_flag;
	rval = pthread_cond_signal( &T3_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_signal", 0, rval, NULL);
		++errors;
	}

	while ( T3_flag != 2 )
	{
		rval = pthread_cond_wait( &T3_cv, &T3_mx );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, NULL);
			++errors;
		}
	}

	rval = pthread_mutex_unlock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_unlock", 0, rval, NULL);
		++errors;
	}

	if ( errors == 0 )
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
t4_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	tid;
	void		*tval;

	rval = pthread_create( &tid, NULL, vsth_null_thread, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}

		rval = pthread_detach( tid );
		if ( rval != ESRCH )
		{
			vsth_surprise("pthread_detach", ESRCH, rval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) pthread_detach( tid );
		vsth_surprise("pthread_join", 0, rval, errors);
	}
}

pthread_t	T5_tid;
int		t5_sfunc();
int		t5_tfunc();
int		t5_efunc();
void 		*t5_t1( void * arg );

VSTH_RT T5_et = {
	t5_sfunc,	t5_tfunc,	NULL
};

void
t5_f1( int *errors, int *fails )
{
	int		result;
	int		rval;

	vsth_et( &T5_et, errors, fails );
}

int
t5_sfunc()
{
	int		val;
	int		rval;

	rval = pthread_create( &T5_tid, NULL, t5_t1, NULL );

	return(rval);
}

int
t5_tfunc()
{
	int	val;
	int	rval;

	rval = 1;
	val = pthread_detach( T5_tid );
	if ( val != EINTR )
		rval = 0;
	return(rval);
}

void *
t5_t1( void * arg )
{
	pthread_exit(NULL);

	/* NOTREACHED */
	return(NULL);
}
