
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_equal
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_equal.c	1.11
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	3

#ifdef  UNDEF_MACROS
#undef  pthread_equal
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );

void
startup()
{
	vsth_startup("pthread_equal");
}

void
cleanup()
{
	vsth_cleanup("pthread_equal");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	pthread_t	mytid;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_equal(t1, t2) shall return a nonzero");
	tet_infoline("value if t1 and t2 are equal.");

	tet_infoline("Posix Ref: Component PTHREAD_EQUAL");
	tet_infoline("Assertion 9945-1:1996 16.2.7.2-1(A)");

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	pthread_t	mytid;

	mytid = pthread_self();

	rval = pthread_equal( mytid, mytid );
	if ( rval == 0 )
	{
		vsth_surprise("pthread_equal", 0, rval, fails);
	}
}

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_equal(t1, t2) shall return a zero");
	tet_infoline("value if t1 and t2 are not equal.");

	tet_infoline("Posix Ref: Component PTHREAD_EQUAL");
	tet_infoline("Assertion 9945-1:1996 16.2.7.2-2(A)");

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	void		*tval;
	pthread_t	tid;
	pthread_t	mytid;

	mytid = pthread_self();

	rval = vsth_pthread_create( &tid, NULL, vsth_null_thread, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval != NULL )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
	}
	else
	{
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	rval = pthread_equal( mytid, tid );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_equal", 0, rval, fails);
	}

}

int		t3_sfunc();
int		t3_tfunc();
pthread_t	T3_tid;

VSTH_RT		T3_et = {
	t3_sfunc,	t3_tfunc,	NULL
};

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_equal() shall not return EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_EQUAL");
	tet_infoline("Assertion 9945-1:1996 2.4-3(A)");

	vsth_et( &T3_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int
t3_sfunc()
{
	T3_tid = pthread_self();
	return(0);
}

int
t3_tfunc()
{
	int		val;
	int		rval;
	pthread_t	mytid;

	rval = 1;

	val = pthread_equal( T3_tid, T3_tid );
	if ( val != EINTR )
		rval = 0;

	return(rval);
}

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	NULL,	0	}
};


