
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_exit
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_exit.c	1.21
*
*	History:
*	Changed pthread_join to vsth_pthread_join to allow TET
*	API to only reap residual threads.
*	
*	Changed pthread_detach to vsth_pthread_detach to allow
*	TET API to only reap residual threads.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"

#define	NO_TESTS	4

#ifdef  UNDEF_MACROS
#undef  pthread_exit
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();

void	t1_f1( int *errors, int *fails );
void	t1_ch1( void *arg );
void	*t1_t1( void *arg );

void	t2_f1( int *errors, int *fails );
void	t2_eval( int *errors, int *fails );
void	*t2_t1( void *arg );
void	t2_ch1( void *arg );
void	t2_ch2( void *arg );
void	t2_df1( void *arg );

void	t3_f1( int *errors, int *fails );
void	t3_eval( int *errors, int *fails );
void	*t3_t1( void *arg );
void	t3_ch1( void *arg );
void	t3_ch2( void *arg );
void	t3_df1( void *arg );
void	t4_f1( int *errors, int *fails );
void	t4_f2();


void
startup()
{
	vsth_startup("pthread_exit");
}

void
cleanup()
{
	vsth_cleanup("pthread_exit");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("Cancellation cleanup handlers are invoked when a thread calls");
	tet_infoline("pthread_exit().");

	tet_infoline("Posix Ref: Component PTHREAD_EXIT");
	tet_infoline("Assertion 9945-1:1996 18.1.3-1(A)");

	t1_f1( &errors, &fails );

	result = VSTH_RESULT( errors, fails );
	tet_result(result);
}

int		T1_flag;

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	void		*tval;
	pthread_t	tid;

	T1_flag = 0;

	rval = vsth_pthread_create( &tid, NULL, t1_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		return;
	}
	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval != NULL )
		{
			vsth_surprise("thread return value", 0, (int) tval, errors);
		}
	}
	else
	{
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}
	if ( T1_flag != 1 )
	{
		vsth_surprise("T1_flag", 1, T1_flag, fails);
	}
}

void *
t1_t1( void *arg )
{
	pthread_cleanup_push( t1_ch1, NULL );
	pthread_exit(NULL);
	pthread_cleanup_pop( 0 );

	/* NOTREACHED */
	return(NULL);
}

void
t1_ch1( void *arg )
{
	T1_flag = 1;
}

int	T2_errors;
int	T2_fails;

void
test2()
{
	int	result;
	int	rval;

	T2_errors	= 0;
	T2_fails	= 0;

	tet_infoline("A successful call to pthread_exit(value_ptr) performs the");
	tet_infoline("following actions in order:");
	tet_infoline("a) each cancellation cleanup handler registered for the exiting");
	tet_infoline("thread is called in last-registered, first-called order.");
	tet_infoline("b) each thread-specific data destructor function registered for");
	tet_infoline("the exiting thread is called in unspecified order.");
	tet_infoline("c) terminates the calling thread");
	tet_infoline("d) makes value_ptr available to any successful join with the");
	tet_infoline("terminating thread.");

	tet_infoline("Posix Ref: Component PTHREAD_EXIT");
	tet_infoline("Assertion 9945-1:1996 16.2.5.2-2(A)");

	t2_f1  ( &T2_errors, &T2_fails );
	t2_eval( &T2_errors, &T2_fails );

	result = VSTH_RESULT( T2_errors, T2_fails );

	tet_result(result);
}

int		T2_cflag;
int		T2_dflag;
pthread_key_t	T2_key1;
pthread_key_t	T2_key2;

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	void		*tval;
	pthread_t	tid;

	T2_cflag	= 0;
	T2_dflag	= 0;

	rval = vsth_pthread_create( &tid, NULL, t2_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}

	(void) pthread_key_delete(T2_key1);
	(void) pthread_key_delete(T2_key2);
}

void
t2_eval( int *errors, int *fails )
{

	if ( T2_cflag != 2 )
	{
		vsth_surprise("T2_cflag", 2, T2_cflag, fails);
	}

	if ( T2_dflag != 2 )
	{
		vsth_surprise("T2_dflag", 2, T2_dflag, fails);
	}
}

void *
t2_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	pthread_cleanup_push( t2_ch1, NULL );
	pthread_cleanup_push( t2_ch2, NULL );

	rval = pthread_key_create( &T2_key1, t2_df1 );
	if ( rval == 0 )
	{
		rval = pthread_setspecific( T2_key1, (void *) &T2_key1 );
		if ( rval == 0 )
		{
			rval = pthread_key_create( &T2_key2, t2_df1 );
			if ( rval == 0 )
			{
				rval = pthread_setspecific( T2_key2, (void *) &T2_key2 );
				if ( rval != 0 )
				{
					vsth_surprise("pthread_setspecific", 0, rval, &nerrs);
				}
			}
			else
			{
				vsth_surprise("pthread_key_create", 0, rval, &nerrs);
			}
		}
		else
		{
			vsth_surprise("pthread_setspecific", 0, rval, &nerrs);
		}
	}
	else
	{
		vsth_surprise("pthread_key_create", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	(void) pthread_exit( tval );

	/* NOTREACHED */

	pthread_cleanup_pop( 0 );
	pthread_cleanup_pop( 0 );

	return(tval);
}

void
t2_ch1( void *arg )
{
	if ( T2_dflag != 0 )
	{
		tet_infoline("TSD destructors were called before cleanup functions");
		++T2_fails;
	}

	if ( T2_cflag == 1 )
	{
		++T2_cflag;
	}
	else
	{
		tet_infoline("Cleanup handlers were called in the wrong order");
		++T2_fails;
	}
}

void
t2_ch2( void *arg )
{
	if ( T2_dflag != 0 )
	{
		tet_infoline("TSD destructors were called before cleanup functions");
		++T2_fails;
	}

	if ( T2_cflag == 0 )
	{
		++T2_cflag;
	}
	else
	{
		tet_infoline("Cleanup handlers were called in the wrong order");
		++T2_fails;
	}
}

void
t2_df1( void *arg )
{
	int		rval;
	pthread_key_t	*pk;

	pk = arg;

	if ( pk != NULL )
	{
		rval = pthread_setspecific((pthread_key_t) *pk, NULL);
		if ( rval != 0 )
		{
			vsth_surprise("pthread_setspecific", 0, rval, &T2_errors);
		}
	}
	++T2_dflag;
}

int		T3_errors;
int		T3_fails;

void
test3()
{
	int		result;
	int		rval;

	T3_errors	= 0;
	T3_fails	= 0;

	tet_infoline("An implicit call to pthread_exit() is made when a thread other");
	tet_infoline("than the thread in which main() was first invoked returns from");
	tet_infoline("the start routine that was used to create it.");

	tet_infoline("Posix Ref: Component PTHREAD_EXIT");
	tet_infoline("Assertion 9945-1:1996 16.2.5.2-3(A)");

	t3_f1  ( &T3_errors, &T3_fails );
	t3_eval( &T3_errors, &T3_fails );
	result = VSTH_RESULT( T3_errors, T3_fails );

	tet_result( result );
}


/******	This is a repeat of test 2
	except the thread does a return() instead of a pthread_exit()	******/

int		T3_cflag;
int		T3_dflag;

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	void		*tval;
	pthread_t	tid;

	T3_cflag	= 0;
	T3_dflag	= 0;

	rval = vsth_pthread_create( &tid, NULL, t3_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 1 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, errors);
		}
		else if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exitval", 0, (int) tval, fails);
		}
	}
	else
	{
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
	}
}

void
t3_eval( int *errors, int *fails )
{

	if ( T3_cflag != 2 )
	{
		vsth_surprise("T3_cflag", 2, T3_cflag, fails);
	}

	if ( T3_dflag != 2 )
	{
		vsth_surprise("T3_dflag", 2, T3_dflag, fails);
	}
}

pthread_key_t	T3_key1;
pthread_key_t	T3_key2;

void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	pthread_cleanup_push( t3_ch1, NULL );
	pthread_cleanup_push( t3_ch2, NULL );

	rval = pthread_key_create( &T3_key1, t3_df1 );
	if ( rval == 0 )
	{
		rval = pthread_setspecific( T3_key1, (void *) &T3_key1 );
		if ( rval == 0 )
		{
			rval = pthread_key_create( &T3_key2, t3_df1 );
			if ( rval == 0 )
			{
				rval = pthread_setspecific( T3_key2, (void *) &T3_key2 );
				if ( rval != 0 )
				{
					vsth_surprise("pthread_setspecific", 0, rval, &nerrs);
				}
			}
			else
			{
				vsth_surprise("pthread_key_create", 0, rval, &nerrs);
			}
		}
		else
		{
			vsth_surprise("pthread_setspecific", 0, rval, &nerrs);
		}
	}
	else
	{
		vsth_surprise("pthread_key_create", 0, rval, &nerrs);
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;


	return(tval);

	/* NOTREACHED */

	pthread_cleanup_pop( 0 );
	pthread_cleanup_pop( 0 );

	return(tval);
}

void
t3_ch1( void *arg )
{
	if ( T3_dflag != 0 )
	{
		tet_infoline("TSD destructors were called before cleanup functions");
		++T3_fails;
	}

	if ( T3_cflag == 1 )
	{
		++T3_cflag;
	}
	else
	{
		tet_infoline("Cleanup handlers were called in the wrong order");
		++T3_fails;
	}
}

void
t3_ch2( void *arg )
{
	if ( T3_dflag != 0 )
	{
		tet_infoline("TSD destructors were called before cleanup functions");
		++T3_fails;
	}

	if ( T3_cflag == 0 )
	{
		++T3_cflag;
	}
	else
	{
		tet_infoline("Cleanup handlers were called in the wrong order");
		++T3_fails;
	}
}

void
t3_df1( void *arg )
{
	int		rval;
	pthread_key_t	*pk;

	pk = (pthread_key_t *) arg;

	if ( pk != NULL )
	{
		rval = pthread_setspecific((pthread_key_t) *pk, NULL);
		if ( rval != 0 )
		{
			vsth_surprise("pthread_setspecific", 0, rval, &T3_errors);
		}
	}

	++T3_dflag;
}

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("The process shall exit with an exit status of 0 after the last thread");
	tet_infoline("has been terminated. The behavior shall be as if the implementation");
	tet_infoline("called exit() with a zero argument at the time of thread termination.");

	tet_infoline("Posix Ref: Component PTHREAD_EXIT");
	tet_infoline("Assertion 9945-1:1996 16.2.5.2-4(A)");

	t4_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t4_f1( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork( t4_f2, NULL, VSTH_TIMEOUT, 0 | 1 | 2 );
	if ( rval )
	{
		if ( rval == 2 )
		{
			vsth_surprise("tet_fork", 0, rval, fails);
		}
		else
		{
			vsth_surprise("tet_fork", 0, rval, errors);
		}
	}
}

#define	T4_PROG	"./pthread_exit_4"

void
t4_f2()
{
	char	*argv[2];
	char	*envp[1];

	argv[0]	= execpath(T4_PROG);
	argv[1]	= NULL;
	envp[0] = NULL;

	(void) tet_exec( execpath(T4_PROG), &argv[0], envp );
}

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	NULL,	0	}
};

