
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_join
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_join.c	1.24
*
*	History:
*	Moved all tests to a separate executable.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	7

#define TE_PATH		"pthread_join_exectests"

#ifdef  UNDEF_MACROS
#undef  pthread_join
#endif

extern	char	**environ;
extern	char	*itoa( int );

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();
void	test5();
void	test6();
void	test7();

void	test_f1( int *errors, int *fails );
void	test_c1();
void	test_f6( int *errors, int *fails );
void	test_c6();
int	t6_pass;

void
startup()
{
	vsth_startup("pthread_join");
}

void
cleanup()
{
	vsth_cleanup("pthread_join");
}

int	T1_flag;

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the target thread was created with a detachstate of");
	tet_infoline("PTHREAD_CREATE_JOINABLE and the thread has not terminated,");
	tet_infoline("then a call to pthread_join(thread, value_ptr) suspends execution");
	tet_infoline("of the calling thread until the target thread terminates");
	tet_infoline("and returns 0 if successful.");

	tet_infoline("Posix Ref: Component PTHREAD_JOIN");
	tet_infoline("Assertion 9945-1:1996 16.2.3.2-1(A)");

	test_f1( &errors, &fails );

	result = VSTH_RESULT( errors, fails );
	tet_result(result);

}

void
test_f1( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork( test_c1, NULL, VSTH_TIMEOUT, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
}

void
test_c1()
{
	char		*argv[2];

	argv[0] = execpath(TE_PATH);
	argv[1] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When value_ptr is not NULL,");
	tet_infoline("then a successful call to pthread_join(thread, value_ptr) causes");
	tet_infoline("the value passed to pthread_exit() by the target thread to be");
	tet_infoline("stored in the location referenced by value_ptr.");

	tet_infoline("Posix Ref: Component PTHREAD_JOIN");
	tet_infoline("Assertion 9945-1:1996 16.2.3.2-2(A)");

	test_f1( &errors, &fails );

	result = VSTH_RESULT( errors, fails );
	tet_result(result);

}

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the calling thread is cancelled,");
	tet_infoline("then a call to pthread_join(thread, value_ptr) shall not detach the");
	tet_infoline("target thread.");

	tet_infoline("Posix Ref: Component PTHREAD_JOIN");
	tet_infoline("Assertion 9945-1:1996 16.2.3.2-3(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the value specified by thread does not refer to a thread");
	tet_infoline("that can be joined,");
	tet_infoline("then a call to pthread_join(thread, value_ptr) shall fail");
	tet_infoline("and return EINVAL.");

	tet_infoline("Posix Ref: Component PTHREAD_JOIN");
	tet_infoline("Assertion 9945-1:1996 16.2.3.4-4(A)");

	/******	Test with a thread created with the PTHREAD_CREATE_DETACHED
		detachstate attribute						******/

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test5()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the thread referenced by the thread argument does not exist,");
	tet_infoline("then a call to pthread_join(thread, value_ptr) shall fail");
	tet_infoline("and return ESRCH.");

	tet_infoline("Posix Ref: Component PTHREAD_JOIN");
	tet_infoline("Assertion 9945-1:1996 16.2.3.4-5(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
test6()
{
	int		errors;
	int		fails;
	int		unsup;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;
	unsup		= 0;

	tet_infoline("If the implementation detects a deadlock or the value of thread");
	tet_infoline("specifies the calling thread:");
	tet_infoline("a call to pthread_join(thread, value_ptr) shall fail and");
	tet_infoline("return EDEADLK.");

	tet_infoline("Posix Ref: Component PTHREAD_JOIN");
	tet_infoline("Assertion 9945-1:1996 16.2.3.4-6(C)");

	p = tet_getvar("VSTH_SELF_EDEADLK_DETECTED");
	if ( p && *p )
	{
		if ((*p == 'Y') || (*p == 'y'))
		{
			/* The pthread_self() case */
			t6_pass = 1;
			test_f6( &errors, &fails );
		}
		else
		{
			++unsup;
			tet_printf("VSTH_SELF_EDEADLK is set to %s", p);
		}
	}
	else
	{
		vsth_badconfig("VSTH_SELF_EDEADLK", &errors);
	}

	p = tet_getvar("VSTH_REFERENTIAL_EDEADLK_DETECTED");
	if ( p && *p )
	{
		if (( *p == 'Y' ) || ( *p == 'y' ))
		{
			/* The deadly embrace case */
			t6_pass = 1;
			test_f6( &errors, &fails );

			/* The deadly ring case */
			t6_pass = 1;
			test_f6( &errors, &fails );

		}
		else
		{
			++unsup;
			tet_printf("VSTH_REFERENTIAL_EDEADLK_DETECTED is set to %s", p);
		}
	}
	else
	{
		vsth_badconfig("VSTH_REFERENTIAL_EDEADLK_EDEADLK", &errors);
	}

	if ( unsup == 2 )
		result = TET_UNSUPPORTED;
	else
		result = VSTH_RESULT( errors, fails );

	result = VSTH_RESULT( errors, fails );
	tet_result(result);
}

void
test_f6( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork( test_c6, NULL, VSTH_TIMEOUT, 0 | 1 | 2 );
	if ( rval == 2 )
	{
		vsth_surprise("tet_fork", 0, rval, fails);
	}
	else if ( rval )
	{
		vsth_surprise("tet_fork", 0, rval, errors);
	}
}

void
test_c6()
{
	char		*argv[3];
	char		ascii_T6[20];

	argv[0] = execpath(TE_PATH);
	argv[1] = strcpy(ascii_T6, itoa(t6_pass));
	argv[2] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

void
test7()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_join() shall not return EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_JOIN");
	tet_infoline("Assertion 9945-1:1996 2.4-7(A)");

	test_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	test5,	5	},
	{	test6,	6	},
	{	test7,	7	},
	{	NULL,	0	}
};


