
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_join_exectests
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_join_exectests.c	1.1
*
*	History:
*	Created separate executable to avoid calls to non-asynch safe functions.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#ifdef  UNDEF_MACROS
#undef  pthread_join
#endif

int	tet_nosigreset = 0;

void	t1_f1( int *errors, int *fails );
void	*t1_t1( void *arg );
void	t1_ch1( void *arg );
void	t2_f1( int *errors, int *fails );
void	t3_f1( int *errors, int *fails );
void	t3_eval( int *errors, int *fails );
void	*t3_t1( void *arg );
void	t4_f1( int *errors, int *fails );
void	t5_f1( int *errors, int *fails );
void	t6_f1( int *errors, int *fails );
void	t6_f2();
void	t6_f3();
int	t6_gen(void * (*start_func)(void *), int nthreads);
void	*t6_t1( void *arg );
void	*t6_t2( void *arg );
void	*t6_t3( void *arg );
void	t7_f1( int *errors, int *fails );

int
tet_main( int argc, char **argv)
{
	int	errors;
	int	fails;
	int	rval;
	int	t6_subtest;

	errors = 0;
	fails = 0;

	switch (tet_thistest) {
		case 1:		t1_f1( &errors, &fails );
				break;
		case 2:		t2_f1( &errors, &fails );
				break;
		case 3:		t3_f1( &errors, &fails );
				break;
		case 4:		t4_f1( &errors, &fails );
				break;
		case 5:		t5_f1( &errors, &fails );
				break;
		case 6:		if (argc >= 2)
				{
					t6_subtest = atoi(argv[1]);
					switch (t6_subtest) {
					case 1:
						t6_f1( &errors, &fails );
						break;
					case 2:
						t6_f2();
						break;
					case 3:
						t6_f3();
						break;
					default:
						tet_printf("Wrong subtest number for test 4");
						errors++;
						break;
					}
				}
				else
				{
					tet_printf("Wrong number of arguments to tet_exec()");
					errors++;
				}

				break;
		case 7:		t7_f1( &errors, &fails );
				break;
		default:
				tet_printf("Invalid test number");
				errors++;
				break;
	}

	rval = 1;
	if ( fails )
		rval = 2;
	if (( errors == 0) && (fails == 0))
		rval = 0;

	exit(rval);
}

int	T1_flag;

void
t1_f1( int *errors, int *fails )
{
	int		rval;
	void		*tval;
	pthread_t	tid;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_JOINABLE );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setdetachstate", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	T1_flag = 0;

	rval = pthread_create( &tid, &attr, t1_t1, (void *) 1 );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( T1_flag != 1 )
		{
			tet_infoline("pthread_join returned before the thread terminated");
			++*fails;
		}
	}
	else
	{
		vsth_surprise("pthread_join", 0, rval, fails);
	}

	(void) pthread_attr_destroy( &attr );
}

void *
t1_t1( void *arg )
{
	int	breakflag;
	time_t	now;
	time_t	endtime;

	pthread_cleanup_push( t1_ch1, NULL );

	endtime = time(NULL) + (VSTH_TIMEOUT / 2);

	/* hang out for a while */

	breakflag = 0;

	while ( ! breakflag )
	{
		now = time(NULL);
		if ( now > endtime )
		{
			breakflag = 1;
		}
		else
		{
			(void) vsth_sched_yield();
		}
	}

	pthread_exit(NULL);
	pthread_cleanup_pop( 0 );

	/* NOTREACHED */
	return(NULL);
}

void
t1_ch1( void *arg )
{
	T1_flag = 1;
}

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	void		*tval;
	pthread_t	tid;

	tval = NULL;

	rval = pthread_create( &tid, NULL, vsth_null_thread, (void *) 1 );
	if ( rval == 0 )
	{
		rval = pthread_join( tid, &tval );
		if ( rval == 0 )
		{
			if ( tval != (void *) 1 )
			{
				tet_printf("Unexpected stored value %p", tval);
				++*fails;
			}
		}
		else
		{
			(void) pthread_cancel( tid );
			(void) pthread_detach( tid );
			vsth_surprise("pthread_join", 0, rval, errors);
		}
	}
	else
	{
		vsth_surprise("pthread_create", 0, rval, errors);
	}
}

#define	T3_NTHREADS	2

int		T3_flag;
pthread_mutex_t	T3_mx;
pthread_cond_t	T3_cv;

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	int		prio;
	int		cnt;
	pthread_t	tid1;
	pthread_t	tid2;
	void		*tval;

	vsth_getprio( SCHED_FIFO, &prio, NULL, errors);
	if ( *errors )
		return;

	rval = pthread_cond_init( &T3_cv, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		return;
	}

	rval = pthread_mutex_init( &T3_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		(void) pthread_cond_destroy( &T3_cv );
		return;
	}

	T3_flag = T3_NTHREADS;

	rval = pthread_create( &tid1, NULL, t3_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
	}

	rval = pthread_create( &tid2, NULL, t3_t1, (void *) tid1 );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
	}

	rval = pthread_join( tid2, &tval );
	if ( rval == 0 )
	{
		if ( tval != PTHREAD_CANCELED )
		{
			++*errors;
			tet_infoline("Thread 2 was not cancelled");
		}
	}
	else
	{
		vsth_surprise("vsth_pthread_join", 0, rval, fails);
	}

	rval = pthread_join( tid1, &tval );
	if ( rval == 0 )
	{
		if ( tval == (void *) 2 )
		{
			vsth_surprise("Thread exit value", 0, (int) tval, fails);
		}
		else if ( tval )
		{
			vsth_surprise("Thread exit value", 0, (int) tval, errors);
		}
	}
	else
	{
		vsth_surprise("pthread_join", 0, rval, fails );
	}

	(void) pthread_cond_destroy( &T3_cv );
	(void) pthread_mutex_destroy( &T3_mx );
}

void *
t3_t1( void *arg )
{
	int	rval;
	int	nerrs;
	int	nfails;
	void	*joinval;
	void	*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	/* Sync up threads 1 and 2 */
	rval = vsth_bwait( &T3_cv, &T3_mx, &T3_flag );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	if ( arg )
	{
		rval = pthread_cancel( pthread_self() );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cancel", 0, rval, &nerrs);
		}

		pthread_join( (pthread_t) arg, &joinval );

		++nfails;
	}

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit( tval );

	/* NOTREACHED */
	return(tval);
}

void
t4_f1( int *errors, int *fails)
{
	int		rval;
	void		*tval;
	pthread_t	tid;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval == 0 )
	{
		rval = pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_DETACHED );
		if ( rval == 0 )
		{
			rval = pthread_create( &tid, &attr, vsth_null_thread, NULL );
			if ( rval == 0 )
			{
				rval = pthread_join( tid, &tval );
				if ( rval != EINVAL )
				{
					vsth_surprise("pthread_join", EINVAL, rval, fails);
				}
			}
			else
			{
				vsth_surprise("pthread_create", 0, rval, errors);
			}
		}
		else
		{
			vsth_surprise("pthread_attr_setdetachstate", 0, rval, errors);
		}

		(void) pthread_attr_destroy( &attr );
	}
	else
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
	}

}

/******	Create a thread. Join it. Try joining it again. Expect an ESRCH return
	value from the second pthread_join().					******/
void
t5_f1( int *errors, int *fails )
{
	int		rval;
	void		*tval;
	pthread_t	tid;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_JOINABLE );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setdetachstate", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_create( &tid, &attr, vsth_null_thread, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval != NULL )
		{
			tet_printf("Unexpected thread exit value %d", (long) tval);
			++*errors;
		}
	}
	else
	{
		vsth_surprise("pthread_join", 0, rval, errors);
	}

	rval = pthread_join( tid, &tval );
	if ( rval != ESRCH )
	{
		vsth_surprise("pthread_join", EINVAL, 0, fails);
	}

	(void) pthread_attr_destroy( &attr );
}

#define	T6_NTHREADS	3

int		T6_flag;
void		*T6_joinvals[T6_NTHREADS];
pthread_cond_t	T6_cv;
pthread_mutex_t	T6_mx;
VSTH_TSET	T6_tset[T6_NTHREADS];

void
t6_f1( int *errors, int *fails )
{
	int		rval;
	int		nerrs;
	int		nfails;
	pthread_t	tid;
	void		*tval;

	tet_infoline("Testing the pthread_self() deadlock case");

	nerrs	= 0;
	nfails	= 0;

	rval = pthread_create( &tid, NULL, t6_t1, NULL );
	if ( rval == 0 )
	{
		rval = pthread_join( tid, &tval );
		if ( rval == 0 )
		{
			if ( (int) tval != EDEADLK )
			{
				tet_printf("Unexpected thread exit value %d", (int) tval);
				++fails;
			}
		}
		else
		{
			(void) pthread_cancel( tid );
			(void) pthread_detach( tid );
			vsth_surprise("pthread_join", 0, rval, errors);
		}
	}
	else
	{
		vsth_surprise("pthread_create", 0, rval, errors);
	}

}

/******	Join with self						******/

void *
t6_t1( void *arg )
{
	int	rval;

	rval = pthread_join( pthread_self(), NULL );
	pthread_exit((void *) rval);

	/* NOTREACHED */
	return((void *) rval);
}

void
t6_f2()
{
	tet_infoline("Testing the deadly embrace case");

	exit(t6_gen(t6_t2, 2));
}

void
t6_f3()
{
	tet_infoline("Testing the deadly ring case");

	exit(t6_gen(t6_t3, 3));
}

int
t6_gen(void * (*start_func)(void *), int nthreads)
{
	int		rval;
	int		nerrs;
	int		nfails;
	int		cnt;
	int		prio;
	time_t		endtime;
	VSTH_PTSET	pts;

	nerrs	= 0;
	nfails	= 0;

	vsth_getprio(SCHED_FIFO, &prio, NULL, &nerrs);
	if ( nerrs )
		return(1);

	T6_flag = nthreads;

	pts = &T6_tset[0];
	for ( cnt = 0; cnt < nthreads; ++cnt )
	{
		pts->policy		= SCHED_FIFO;
		pts->priority		= prio;
		pts->scope		= PTHREAD_SCOPE_PROCESS;
		pts->detachstate	= PTHREAD_CREATE_JOINABLE;
		pts->func		= start_func;
		pts->arg		= (void *) cnt;
		pts->exitval		= (void *) 1;
		pts->status		= -1;
		T6_joinvals[cnt]	= (void *) -1;

		++pts;
	}

	rval = vsth_ts_create( &T6_tset[0], nthreads, VSTH_NOSYNC, NULL, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_ts_create", 0, rval, NULL);
		vsth_ts_cancel( &T6_tset[0], nthreads, 1 );
		(void) pthread_mutex_destroy( &T6_mx );
		(void) pthread_cond_destroy( &T6_cv );
		return(1);
	}

	endtime = time(NULL) + ( VSTH_TIMEOUT / 4 );
	while ( time(NULL) < endtime )
	{
		(void) vsth_sched_yield();
	}

	for ( cnt = 0; cnt < nthreads; ++cnt )
	{
		if ( T6_joinvals[cnt] != (void *) EDEADLK )
		{
			++nfails;
			tet_printf("Join of thread %d did not return EDEADLK", cnt );
		}

		(void) pthread_cancel( T6_tset[cnt].tid );
		(void) pthread_detach( T6_tset[cnt].tid );
	}

	rval = 1;

	if ( nfails )
		rval = 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		rval = 0;

	return(rval);
}

/****** Two threads join with each other			******/
void *
t6_t2( void *arg )
{
	int	rval;
	int	nerrs;
	int	target;
	void	*tval;

	nerrs = 0;

	/* sync the threads */
	rval = vsth_bwait( &T6_cv, &T6_mx, &T6_flag );
	if ( rval == 0 )
	{
		/* join with each other */

		target = ( arg == NULL ? 1 : 0 );
		rval = pthread_join( T6_tset[target].tid, &T6_joinvals[target] );
	}
	else
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	if ( nerrs )
	{
		tval = (void *) 1;
	}
	else
	{
		tval = (void *) rval;
	}

	pthread_exit(tval);

	/* NOTREACHED */
	return((void *) rval);
}

/******	Three threads join in a ring				******/

void *
t6_t3( void *arg )
{
	int	rval;
	int	nerrs;
	int	target;
	void	*tval;

	nerrs	= 0;

	/* sync up */
	rval = vsth_bwait( &T6_cv, &T6_mx, &T6_flag );
	if ( rval == 0 )
	{
		/* join with the next thread in the ring */
		target = ( ((int) arg) + 1 ) % 3 ;
		rval = pthread_join( T6_tset[target].tid, &T6_joinvals[target] );
	}
	else
	{
		vsth_surprise("vsth_bwait", 0, rval, &nerrs);
	}

	if ( nerrs )
	{
		tval = (void *) 1;
	}
	else
	{
		tval = (void *) rval;
	}

	pthread_exit(tval);

	/* NOTREACHED */
	return((void *) rval);
}

int		t7_sfunc();
int		t7_tfunc();
int		t7_efunc();

pthread_t	T7_tid;

VSTH_RT		T7_et = {
	t7_sfunc,	t7_tfunc,	t7_efunc
};

void
t7_f1( int *errors, int *fails )
{
	int		result;
	int		rval;

	vsth_et( &T7_et, errors, fails );
}

int
t7_sfunc()
{
	int		rval;

	rval = pthread_create( &T7_tid, NULL, vsth_null_thread, NULL );

	return(rval);
}

int
t7_tfunc()
{
	int	val;
	int	rval;
	void	*tval;

	rval = 1;

	val = pthread_join( T7_tid, &tval );
	if ( val != EINTR )
		rval = 0;

	return(rval);
}

int
t7_efunc()
{
	/* just in case */
	(void) pthread_detach( T7_tid );
	return(0);
}
