
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_once
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_once.c	1.13
*
*	History
*		Changed pthread_detach to vsth_pthread_detach to allow
*		TET API to only reap residual threads.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	4

#ifdef  UNDEF_MACROS
#undef  pthread_once
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();

void	t1_f1( int *errors, int *fails );
void	t1_o1();
void	t2_f1( int *errors, int *fails );
void	t2_o1();
void	t3_f1( int *errors, int *fails );
void	*t3_t1( void *arg );
void	t3_o1();
void	t3_c1( void *arg );


void
startup()
{
	vsth_startup("pthread_once");
}

void
cleanup()
{
	vsth_cleanup("pthread_once");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When it is the first call with a given once_control in a thread and");
	tet_infoline("once_control has been initialized to PTHREAD_ONCE_INIT,");
	tet_infoline("then a call to pthread_once(once_control, init_routine) shall call");
	tet_infoline("init_routine with no arguments and return 0 if successful.");

	tet_infoline("Posix Ref: Component PTHREAD_ONCE");
	tet_infoline("Assertion 9945-1:1996 16.2.8.2-1(A)");

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int		T1_flag = 0;
pthread_once_t	T1_once = PTHREAD_ONCE_INIT;

void
t1_f1( int *errors, int *fails )
{
	int		rval;

	rval = pthread_once( &T1_once, t1_o1 );
	if ( T1_flag != 1 )
	{
		vsth_surprise("T1_flag", 1, T1_flag, fails);
	}
}

void
t1_o1()
{
	++T1_flag;
}

int		T2_flag;
pthread_once_t	T2_once = PTHREAD_ONCE_INIT;

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When it is not the first call with a given once_control in a thread and");
	tet_infoline("once_control was previously initialized to PTHREAD_ONCE_INIT,");
	tet_infoline("then a call to pthread_once(once_control, init_routine) shall not call");
	tet_infoline("the init_routine and shall return 0 if successful.");

	tet_infoline("Posix Ref: Component PTHREAD_ONCE");
	tet_infoline("Assertion 9945-1:1996 16.2.8.2-2(A)");

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t2_f1( int *errors, int *fails )
{
	int	rval;

	T2_flag = 0;

	rval = pthread_once( &T2_once, t2_o1 );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_once", 0, rval, errors);
		return;
	}

	rval = pthread_once( &T2_once, t2_o1 );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_once", 0, rval, fails);
	}
	if ( T2_flag != 1 )
	{
		vsth_surprise("T2_flag", 1, T2_flag, fails);
	}
}

void
t2_o1()
{
	++T2_flag;
}

int		T3_flag;
int		T3_errors;
int		T3_fails;
pthread_once_t	T3_once = PTHREAD_ONCE_INIT;

void
test3()
{
	int		result;
	int		rval;

	T3_errors	= 0;
	T3_fails	= 0;

	tet_infoline("When the routine referenced by the argument init_routine is a");
	tet_infoline("cancellation point and is canceled,");
	tet_infoline("then the effect on once_control of a call to");
	tet_infoline("pthread_once(once_control, init_routine) shall be as if pthread_once()");
	tet_infoline("was never called.");

	tet_infoline("Posix Ref: Component PTHREAD_ONCE");
	tet_infoline("Assertion 9945-1:1996 16.2.8.2-3(A)");

	t3_f1( &T3_errors, &T3_fails );
	result = VSTH_RESULT( T3_errors, T3_fails );

	tet_result(result);
}

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	void		*tval;
	pthread_t	tid;

	T3_flag		= 0;

	rval = vsth_pthread_create( &tid, NULL, t3_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( rval == 0 )
	{
		if ( tval != (void *) PTHREAD_CANCELED )
		{
			vsth_surprise(	"Thread exitval",
					(int) PTHREAD_CANCELED,
					(int) tval,
					errors );
		}
	}
	else
	{
		vsth_surprise("vsth_pthread_join", 0, rval, errors);
		(void) pthread_cancel( tid );
		(void) vsth_pthread_detach( tid );
	}

	if ( T3_flag != 1 )
	{
		vsth_surprise("T3_flag", 1, T3_flag, fails);
	}
}

void *
t3_t1( void *arg )
{
	int		rval;
	void		*tval;
	pthread_t	mytid;

	tval	= (void *) 1;
	mytid	= pthread_self();

	pthread_cleanup_push( t3_c1, NULL );

	rval = pthread_cancel( mytid );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cancel", 0, rval, &T3_errors);
	}

	/******	we should get cancelled in the pthread_testcancel, but the once
		control should remain fresh				******/

	rval = pthread_once( &T3_once, pthread_testcancel );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_once", 0, rval, &T3_errors);
	}

	pthread_cleanup_pop( 0 );

	pthread_exit((void *) 1);

	/* NOTREACHED */
	return((void *) 1);
}

void
t3_c1( void *arg )
{
	int	rval;

	rval = pthread_once( &T3_once, t3_o1 );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_once", 0, rval, &T3_errors);
	}
}

void
t3_o1()
{
	++T3_flag;
}

int		T4_flag;

int		t4_sfunc();
int		t4_tfunc();

VSTH_RT		T4_et = {
	t4_sfunc,	t4_tfunc,	NULL
};

int		T4_flag;
pthread_once_t	T4_once = PTHREAD_ONCE_INIT;

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_once() shall not return EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_ONCE");
	tet_infoline("Assertion 9945-1:1996 2.4-4(A)");

	vsth_et( &T4_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t4_o1()
{
	++T4_flag;
}

int
t4_sfunc()
{
	T4_flag = 0;
	return(0);
}

int
t4_tfunc()
{
	int	val;
	int	rval;

	rval = 1;

	val = pthread_once( &T4_once, t4_o1 );
	if ( val != EINTR )
		rval = 0;

	return(rval);
}

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	NULL,	0	}
};


