
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_self
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_self.c	1.11
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	2

#ifdef  UNDEF_MACROS
#undef  pthread_self
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();

void	t1_f1( int *errors, int *fails );
void	*t1_t1( void *arg );

void
startup()
{
	vsth_startup("pthread_self");
}

void
cleanup()
{
	vsth_cleanup("pthread_self");
}

pthread_t	T1_tid;

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_self() returns the thread identifier");
	tet_infoline("of the calling thread");

	tet_infoline("Posix Ref: Component PTHREAD_SELF");
	tet_infoline("Assertion 9945-1:1996 16.2.6.2-1(A)");

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t1_f1( int *errors, int *fails )
{
	int	rval;
	void	*tval;

	rval = vsth_pthread_create( &T1_tid, NULL, t1_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_create", 0, rval, errors);
		return;
	}

	rval = vsth_pthread_join( T1_tid, &tval );
	if ( rval == 0 )
	{
		if ( tval != NULL )
		{
			tet_infoline("Thread ids differed");
			++*fails;
		}
	}
	else
	{
		vsth_surprise("pthread_join", 0, rval, errors);
	}
}

void *
t1_t1( void *arg )
{
	void	*tval;

	tval = (void *) 1;

	if ( pthread_equal( T1_tid, pthread_self() ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}


int	t2_tfunc();

VSTH_RT	T2_et = {
	NULL,	t2_tfunc,	NULL
};

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_self() shall not return EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_SELF");
	tet_infoline("Assertion 9945-1:1996 2.4-2(A)");

	vsth_et( &T2_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result( result );
}

int
t2_tfunc()
{
	int		rval;
	pthread_t	val;

	rval = 1;

	val = pthread_self();
	if ( (int) val != EINTR )
		rval = 0;

	return(rval);
}

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	NULL,	0	}
};


