
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_attr_destroy
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_attr_destroy.c	1.13
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	2

#ifdef  UNDEF_MACROS
#undef  pthread_attr_destroy
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();


void
startup()
{
	vsth_startup("pthread_attr_destroy");
}

void
cleanup()
{
	vsth_cleanup("pthread_attr_destroy");
}

void
test1()
{

	int		errors;
	int		fails;
	int		result;
	int		rval;

	pthread_attr_t	attr;

	errors		= 0;
	fails		= 0;
	result		= TET_UNRESOLVED;


	tet_infoline("A call to pthread_attr_destroy(attr) destroys the thread attribute");
	tet_infoline("object specified by attr and returns 0 on successful completion.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_DESTROY");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2, 9945-1:1996 16.2.1.3-1(A)");

	/*******
		there isn't a really good way to test this because we have no
		portable way to tell that the attribute object has been destroyed.
		No EINVAL errors are mandated for an uninitialized attr in
		pthread_create() or pthread_attr_[sg]*() functions.	*******/

	rval = pthread_attr_init( &attr );
	if ( rval == 0 )
	{
		rval = pthread_attr_destroy( &attr );
		if ( rval == 0 )
		{
			rval = pthread_attr_init( &attr );
			if ( rval == 0 )
			{
				rval = pthread_attr_destroy( &attr );
				if ( rval != 0 )
				{
					vsth_surprise("pthread_attr_destroy", 0, rval, &fails);
				}
			}
		}
		else
		{
			vsth_surprise("pthread_attr_destroy", 0, rval, &fails);
		}
	}
	else
	{
		vsth_surprise("pthread_attr_init", 0, rval, &errors);
	}

	result = VSTH_RESULT( errors, fails );
	tet_result(result);
}

int		t2_sfunc();
int		t2_tfunc();

pthread_attr_t	T2_attr;

VSTH_RT		T2_et = {
	t2_sfunc,	t2_tfunc,	NULL
};

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_attr_destroy() shall not return a value of EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_DESTROY");
	tet_infoline("Assertion 9945-1:1996 2.4-2(A)");

	vsth_et( &T2_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int
t2_sfunc()
{
	int	rval;

	rval = pthread_attr_init( &T2_attr );

	return( rval );
}

int
t2_tfunc()
{
	int	val;
	int	rval;

	rval = 1;
	val = pthread_attr_destroy( &T2_attr );
	if ( val != EINTR )
		rval = 0;

	return(rval);
}



void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	NULL,	0	}
};


