
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_attr_getdetachstate
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_attr_getdetachstate.c	1.12
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	2

#ifdef  UNDEF_MACROS
#undef  pthread_attr_getdetachstate
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();

void	t1_f1( pthread_attr_t *attr, int val, int *errors, int *fails );


void
startup()
{
	vsth_startup("pthread_attr_getdetachstate");
}

void
cleanup()
{
	vsth_cleanup("pthread_attr_getdetachstate");
}

void
test1()
{


	int		cnt;
	int		errors;
	int		fails;
	int		result;
	int		rval;

	pthread_attr_t	attr;
	int		setv;
	int		getv;

	errors		= 0;
	fails		= 0;
	result		= TET_UNRESOLVED;


	tet_infoline("A call to pthread_attr_getdetachstate(attr, detachstate)");
	tet_infoline("stores the value of the detachstate attribute of the");
	tet_infoline("attribute object specified by attr in detachstate");

	tet_infoline("and returns 0 if successful.");
	tet_infoline("The stored value shall be either PTHREAD_CREATE_DETACHED");
	tet_infoline("or PTHREAD_CREATE_JOINABLE.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_GETDETACHSTATE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2,9945-1:1996 16.2.1.3-1(A)");

	rval = pthread_attr_init( &attr );
	if ( rval == 0 )
	{
		t1_f1( &attr, PTHREAD_CREATE_DETACHED, &errors, &fails );
		t1_f1( &attr, PTHREAD_CREATE_JOINABLE, &errors, &fails );
		(void) pthread_attr_destroy( &attr );
	}
	else
	{
		vsth_surprise("pthread_attr_init rval", 0, rval, &errors);
	}

	result = VSTH_RESULT( errors, fails );

	tet_result(result);

}

void
t1_f1( pthread_attr_t *attr, int val, int *errors, int *fails )
{
	int	rval;
	int	getv;

	rval = pthread_attr_setdetachstate( attr, val );
	if (rval == 0)
	{
		rval = pthread_attr_getdetachstate( attr, &getv );
		if (rval != 0)
		{
			vsth_surprise("pthread_attr_getdetachstate rval", 0, rval, fails);
		}
		else
		{
			if ( getv != val )
			{
				vsth_surprise( NULL, val, getv, fails );
			}
		}
	}
	else
	{
		vsth_surprise("pthread_attr_setdetachstate rval", 0, rval, errors);
	}
}

int		t2_sfunc();
int		t2_tfunc();

pthread_attr_t	T2_attr;

VSTH_RT		T2_et = {
	t2_sfunc,	t2_tfunc,	NULL
};

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_attr_getdetachstate() shall not");
	tet_infoline("return a value of EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_GETDETACHSTATE");
	tet_infoline("Assertion 9945-1:1996 2.4-2(A)");

	vsth_et( &T2_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int
t2_sfunc()
{
	int	rval;

	rval = pthread_attr_init( &T2_attr );
	if ( rval == 0 )
	{
		rval = pthread_attr_setdetachstate( &T2_attr, PTHREAD_CREATE_JOINABLE );
	}

	return(rval);
}

int
t2_tfunc()
{
	int	val;
	int	rval;
	int	detachstate;

	rval = 1;

	val = pthread_attr_getdetachstate( &T2_attr, &detachstate );
	if ( val != EINTR )
		rval = 0;

	return(rval);
}

int
t2_efunc()
{
	(void) pthread_attr_destroy( &T2_attr );
	return(0);
}



void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	NULL,	0	}
};


