
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_attr_getguardsize
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_attr_getguardsize.c	1.14
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<sched.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/mman.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	4

#ifdef  UNDEF_MACROS
#undef  pthread_attr_getguardsize
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();

void	t1_f1( size_t size, int *errors, int *fails );
void	t3_f1( int *errors, int *fails );
void	t4_f1( int *errors, int *fails );

void
startup()
{
	vsth_startup("pthread_attr_getguardsize");
}

void
cleanup()
{
	vsth_cleanup("pthread_attr_getguardsize");
}

void
test1()
{

	int		errors;
	int		fails;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_attr_getguardsize(attr, guardsize)");
	tet_infoline("stores the value of the guardsize attribute");
	tet_infoline("of the attribute object specified by attr in guardsize");
	tet_infoline("and returns 0 if successful.");
	tet_infoline("The stored value shall be that specified by the");
	tet_infoline("corresponding call to the pthread_attr_setguardsize()");
	tet_infoline("function for attr.");

	tet_infoline("XCAE ref: Component PTHREAD_ATTR_GETGUARDSIZE");
	tet_infoline("Assertion 1(A)");


	p = tet_getvar("VSTH_VALID_GUARDSIZE");
	if ( p && *p )
	{
		t1_f1( (size_t) atoi(p), &errors, &fails );
	}
	else
	{
		vsth_badconfig("VSTH_VALID_GUARDSIZE", &errors);
	}

	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t1_f1( size_t size, int *errors, int *fails )
{

	int		rval;
	size_t		getv;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}
	rval = pthread_attr_setguardsize( &attr, size );
	if (rval == 0)
	{
		rval = pthread_attr_getguardsize( &attr, &getv );
		if (rval == 0)
		{
			if (getv != size)
			{
				vsth_surprise( NULL, size, getv, fails);
			}
		}
		else
		{
			vsth_surprise("pthread_attr_getguardsize rval", 0, rval, fails);
		}
	}

	(void) pthread_attr_destroy( &attr );
}

int		t2_sfunc();
int		t2_tfunc();
int		t2_efunc();

pthread_attr_t	T2_attr;

VSTH_RT		T2_et = {
	t2_sfunc,	t2_tfunc,	t2_efunc
};

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_attr_getguardsize() shall not");
	tet_infoline("return a value of EINTR.");

	tet_infoline("XCAE ref: Component PTHREAD_ATTR_GETGUARDSIZE");
	tet_infoline("Assertion 2(A)");

	vsth_et( &T2_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);

}

int
t2_sfunc()
{
	int	rval;

	rval = pthread_attr_init( &T2_attr );
	if ( rval == 0 )
	{
		rval = pthread_attr_setguardsize( &T2_attr, 0 );
	}
	return(rval);
}

int
t2_tfunc()
{
	int	val;
	int	rval;
	int	size;

	rval = 1;

	val = pthread_attr_getguardsize( &T2_attr, &size );
	if ( val != EINTR )
		rval = 0;

	return(rval);
}

int
t2_efunc()
{
	(void) pthread_attr_destroy( &T2_attr );
	return(0);
}

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the attribute object specified by attr is invalid,");
	tet_infoline("then a call to pthread_attr_getguardsize(attr, guardsize)");
	tet_infoline("shall fail and return EINVAL");

	tet_infoline("XCAE ref: Component PTHREAD_ATTR_GETGUARDSIZE");
	tet_infoline("Assertion 3(A)");

	p = tet_getvar("VSTH_UPAO_EINVAL");
	if ( p && *p )
	{
		if (( *p == 'Y' ) || ( *p == 'y' ))
		{
			t3_f1( &errors, &fails );
			result = VSTH_RESULT( errors, fails );
		}
		else
		{
			tet_printf("VSTH_UPAO_EINVAL is set to %s", p);
			result = TET_UNSUPPORTED;
		}
	}
	else
	{
		vsth_badconfig("VSTH_UPAO_EINVAL", &errors);
		result = VSTH_RESULT( errors, fails );
	}

	tet_result(result);
}

void
t3_f1( int *errors, int *fails )
{
	int		rval;
	size_t		size;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_destroy( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_destroy", 0, rval, errors);
		return;
	}

	rval = pthread_attr_getguardsize( &attr, &size );
	if ( rval != EINVAL )
	{
		vsth_surprise("pthread_attr_getguardsize", EINVAL, rval, fails);
	}
}


void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the parameter guardsize is invalid,");
	tet_infoline("then a call to pthread_attr_getguardsize(attr, guardsize)");
	tet_infoline("shall fail and return EINVAL");

	tet_infoline("XCAE ref: Component PTHREAD_ATTR_GETGUARDSIZE");
	tet_infoline("Assertion 4(A)");

	/* this assertion is a pthread_attr_setguardsize assertion

	t4_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	*/

	tet_result(TET_NOTINUSE);
}

void
t4_f1( int *errors, int *fails )
{

	int		rval;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}
	rval = pthread_attr_setguardsize( &attr, 0 );
	if ( rval == 0 )
	{
		rval = pthread_attr_getguardsize( &attr, NULL );
		if ( rval != EINVAL )
		{
			vsth_surprise(	"pthread_attr_getguardsize",
					EINVAL,
					rval,
					fails );
		}
	}
	else
	{
		vsth_surprise("pthread_attr_setguardsize", 0, rval, errors);
	}
			
	(void) pthread_attr_destroy( &attr );
}

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	NULL,	0	}
};


