
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_attr_getstacksize
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_attr_getstacksize.c	1.16
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	3

#ifdef  UNDEF_MACROS
#undef  pthread_attr_getstacksize
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

static void	t1_f1( size_t val, int *errors, int *fails );

#else

static void	t2_f1( int *errors, int *fails );

#endif

void
startup()
{
	vsth_startup("pthread_attr_getstacksize");
}

void
cleanup()
{
	vsth_cleanup("pthread_attr_getstacksize");
}

void
test1()
{

	int	errors;
	int	fails;
	int	result;
	int	rval;

	char	*p;
	long	val;

	errors	= 0;
	fails	= 0;

	tet_infoline("If _POSIX_THREAD_ATTR_STACKSIZE is defined:");
	tet_infoline("a call to pthread_attr_getstacksize(attr, stacksize)");
	tet_infoline("stores the value of the stacksize attribute of the");
	tet_infoline("attribute object specified by attr in stacksize");
	tet_infoline("and returns 0 if successful.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_GETSTACKSIZE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2-1(C)");

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

	/* try it with the user configurable value */
	p = tet_getvar("VSTH_VALID_STACKSIZE");
	if ( p && *p )
	{
		t1_f1( (size_t) atoi(p), &errors, &fails );
	}
	else
	{
		vsth_badconfig("VSTH_VALID_STACKSIZE", &errors);
	}

	/* now try it with PTHREAD_STACK_MIN */
	val = sysconf(_SC_THREAD_STACK_MIN);;
	if (val != -1)
	{
		t1_f1( (size_t) val, &errors, &fails );
	}
	else
	{
		++errors;
		tet_printf("sysconf unexpectedly failed, errno == %d", errno);
	}

	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_ATTR_STACKSIZE is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif /* _POSIX_THREAD_ATTR_STACKSIZE */

	tet_result(result);
}

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

void
t1_f1( size_t val, int *errors, int *fails )
{

	int		rval;
	size_t		getv;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setstacksize( &attr, val );
	if (rval == 0)
	{
		rval = pthread_attr_getstacksize( &attr, &getv );
		if (rval == 0)
		{
			if (getv != val)
			{
				vsth_surprise( NULL, val, getv, fails );
			}
		}
		else
		{
			vsth_surprise("pthread_attr_getstacksize rval", 0, rval, fails);
		}
	}
	else
	{
		vsth_surprise("pthread_attr_setstacksize rval", 0, rval, errors);
	}

	(void) pthread_attr_destroy( &attr );
}

#endif	/* _POSIX_THREAD_ATTR_STACKSIZE */

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If _POSIX_THREAD_ATTR_STACKSIZE is not defined:");
	tet_infoline("a call to pthread_attr_getstacksize() shall fail");
	tet_infoline("and return ENOSYS.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_GETSTACKSIZE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.4-2(C)");

#ifndef	_POSIX_THREAD_ATTR_STACKSIZE

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else
	tet_infoline("_POSIX_THREAD_ATTR_STACKSIZE is defined");

	result = TET_UNSUPPORTED;
#endif
	tet_result(result);

}

#ifndef	_POSIX_THREAD_ATTR_STACKSIZE

void
t2_f1( int *errors, int *fails )
{
	int		rval;
	size_t		size;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_getstacksize( &attr, &size );
	if ( rval != ENOSYS )
	{
		vsth_surprise("pthread_attr_getstacksize", ENOSYS, rval, fails);
	}

	(void) pthread_attr_destroy( &attr );
}

#endif

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

int		t3_sfunc();
int		t3_tfunc();
int		t3_efunc();

pthread_attr_t	T3_attr;

VSTH_RT		T3_et = {
	t3_sfunc,	t3_tfunc,	t3_efunc
};

#endif

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If _POSIX_THREAD_ATTR_STACKSIZE is defined:");
	tet_infoline("a call to pthread_attr_getstacksize() shall");
	tet_infoline("not return a value of EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_GETSTACKSIZE");
	tet_infoline("Assertion 9945-1:1996 2.4-3(C)");

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

	vsth_et( &T3_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_ATTR_STACKSIZE is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif


#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

int
t3_sfunc()
{
	int	rval;

	rval = pthread_attr_init( &T3_attr );
	return(rval);
}

int
t3_tfunc()
{
	int	val;
	int	rval;
	size_t	addr;

	rval = 1;

	val = pthread_attr_getstacksize( &T3_attr, &addr );
	if ( val != EINTR )
		rval = 0;
	
	return(rval);
}

int
t3_efunc()
{
	(void) pthread_attr_destroy( &T3_attr );
	return(0);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	NULL,	0	}
};

