
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_attr_init
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_attr_init.c	1.17
*
*	History:
*	Changed pthread_detach to vsth_pthread_detach to allow
*	TET API to only reap residual threads.
*	Changed pthread_join to vsth_pthread_join to allow TET
*	API to only reap residual threads.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	5

#ifdef  UNDEF_MACROS
#undef  pthread_attr_init
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();
void	test5();

void	t1_f1( int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	*t2_t1( void *arg );
void	t3_f1( int *errors, int *fails );
void	*t3_t1( void *arg );

void
startup()
{
	vsth_startup("pthread_attr_init");
}

void
cleanup()
{
	vsth_cleanup("pthread_attr_init");
}

void
test1()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_attr_init(attr) initialises the attributes");
	tet_infoline("object specified by attr with the default value for all the individual");
	tet_infoline("attributes used by a given implementation and returns 0 if successful.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_INIT");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2-1(A)");

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t1_f1( int *errors, int *fails )
{

	int			rval;
	int			val;
	pthread_attr_t		attr;
	size_t			size;
	void			*addr;
	struct sched_param	params;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
	}

	rval = pthread_attr_getdetachstate( &attr, &val );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_getdetachstate", 0, rval, errors);
	}

	if ( val != PTHREAD_CREATE_JOINABLE )
	{
		tet_printf("Unexpected default detachstate %d", val);
		++*fails;
	}

#ifdef	_POSIX_THREAD_PRIORITY_SCHEDULING

	/* default scheduling attributes are implementation defined */
	rval = pthread_attr_getinheritsched( &attr, &val );
	if ( rval == 0 )
	{
		tet_printf("Default inheritsched attribute is %s",
				val == PTHREAD_INHERIT_SCHED ? "PTHREAD_INHERIT_SCHED" : "PTHREAD_EXPLICIT_SCHED" );
	}
	else
	{
		vsth_surprise("pthread_attr_getinheritsched", 0, rval, errors);
	}

	rval = pthread_attr_getschedpolicy( &attr, &val );
	if ( rval == 0 )
	{
		tet_printf("Default scheduling policy is %s", 
				val == SCHED_RR ? "SCHED_RR" : ( val == SCHED_FIFO ? "SCHED_FIFO" : (val == SCHED_OTHER ? "SCHED_OTHER" : "UNKNOWN")));
	}
	else
	{
		vsth_surprise("pthread_attr_getschedpolicy", 0, rval, errors);
	}

	rval = pthread_attr_getscope( &attr, &val );
	if ( rval == 0 )
	{
		tet_printf("Default scheduling scope is %s",
				val == PTHREAD_SCOPE_PROCESS ? "PTHREAD_SCOPE_PROCESS" : (val == PTHREAD_SCOPE_SYSTEM ? "PTHREAD_SCOPE_SYSTEM" : "UNKNOWN"));
	}
	else
	{
		vsth_surprise("pthread_attr_getscope", 0, rval, errors);
	}

#endif

#ifdef	_POSIX_THREAD_ATTR_STACKADDR

	/* this is also implementation defined */
	rval = pthread_attr_getstackaddr( &attr, &addr );
	if ( rval == 0 )
	{
		tet_printf("Default stack address is %p", addr );
	}
	else
	{
		vsth_surprise("pthread_attr_getstackaddr", 0, rval, errors);
	}

#endif

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

	rval = pthread_attr_getstacksize( &attr, &size );
	if ( rval == 0 )
	{
		tet_printf("Default stack size is %d", size );
	}
	else
	{
		vsth_surprise("pthread_attr_getstacksize", 0, rval, errors);
	}

#endif
	/* also implementation defined */
	rval = pthread_attr_getschedparam( &attr, &params );
	if ( rval == 0 )
	{
		tet_printf("Default priority is %d", params.sched_priority );
	}
	else
	{
		vsth_surprise("pthread_attr_getschedparam", 0, rval, errors );
	}

	(void) pthread_attr_destroy( &attr );
}

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A threads attribute object destroyed by a previous call to");
	tet_infoline("pthread_attr_destroy() may be reinitialized by a call to");
	tet_infoline("pthread_attr_init().");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_INIT");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2-2(A)");

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t2_f1( int *errors, int *fails )
{

	int		rval;
	void		*tval;
	pthread_t	tid;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}
	rval = pthread_attr_destroy( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_destroy", 0, rval, errors);
		return;
	}

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, fails);
		return;
	}

	rval = vsth_pthread_create( &tid, &attr, t2_t1, NULL );
	if ( rval == 0 )
	{
		rval = vsth_pthread_join( tid, &tval );
		if ( rval == 0 )
		{
			if ( tval == (void *) 1 )
			{
				vsth_surprise("Thread exitval", 0, (int) tval, errors);
			}
			else if ( tval == (void *) 2 )
			{
				vsth_surprise("Thread exitval", 0, (int) tval, fails);
			}
		}
		else
		{
			(void) pthread_cancel( tid );
			(void) vsth_pthread_detach( tid );
			vsth_surprise("vsth_pthread_join", 0, rval, errors);
		}
	}
}

void *
t2_t1( void *arg )
{
	pthread_exit( NULL );

	/* NOTREACHED */
	return( NULL );
}

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A single attributes object may be used in multiple calls to");
	tet_infoline("pthread_create().");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_INIT");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2-3(A)");

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

#define		T3_NTHREADS	4

int		T3_flag;
pthread_mutex_t	T3_mx;
pthread_cond_t	T3_cv;
pthread_t	T3_tids[T3_NTHREADS];

void
t3_f1( int *errors, int *fails )
{

	int		rval;
	void		*tval;
	int		cnt;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_mutex_init( &T3_mx, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_init", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_cond_init( &T3_cv, NULL);
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_init", 0, rval, errors);
		(void) pthread_mutex_destroy( &T3_mx );
		(void) pthread_attr_destroy( &attr );
		return;
	}

	T3_flag = T3_NTHREADS;

	for ( cnt = 0; cnt < T3_NTHREADS; ++cnt )
	{
		rval = vsth_pthread_create(&T3_tids[cnt], &attr, t3_t1, NULL);
		if ( rval != 0 )
		{
			vsth_surprise("vsth_pthread_create", 0, rval, fails);
		}
	}

	for ( cnt = 0; cnt < T3_NTHREADS; ++cnt )
	{
		rval = vsth_pthread_join(T3_tids[cnt], &tval);
		if ( rval == 0 )
		{
			if ( tval == (void *) 2 )
			{
				vsth_surprise("thread exit value",
							0, (int) tval, fails);
			}
			else if ( tval )
			{
				vsth_surprise("thread exit value",
							0, (int) tval, errors);
			}
		}
		else
		{
			vsth_surprise("vsth_pthread_join", 0, rval, errors);
		}
	}

	(void) pthread_mutex_destroy( &T3_mx );
	(void) pthread_cond_destroy( &T3_cv );
	(void) pthread_attr_destroy( &attr );
}


void *
t3_t1( void *arg )
{
	int		rval;
	int		nerrs;
	int		nfails;
	void		*tval;

	nerrs	= 0;
	nfails	= 0;
	tval	= (void *) 1;

	rval = pthread_mutex_lock( &T3_mx );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_mutex_lock", 0, rval, &nerrs);
	}

	--T3_flag;

	rval = pthread_cond_broadcast( &T3_cv );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_cond_broadcast", 0, rval, &nerrs);
	}

	while ( T3_flag )
	{
		rval = pthread_cond_wait (&T3_cv, &T3_mx);
		if ( rval != 0 )
		{
			vsth_surprise("pthread_cond_wait", 0, rval, &nerrs);
		}
	}

	rval = pthread_mutex_unlock( &T3_mx );

	if ( nfails )
		tval = (void *) 2;
	else if (( nerrs == 0 ) && ( nfails == 0 ))
		tval = NULL;

	pthread_exit(tval);

	/* NOTREACHED */
	return(tval);
}

void
test4()
{

	tet_infoline("If insufficient memory exists to initialize the thread");
	tet_infoline("attributes object specified by attr:");
	tet_infoline("a call to pthread_attr_init(attr) shall fail and return");
	tet_infoline("the value ENOMEM.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_INIT");
	tet_infoline("Assertion 9945-1:1996 16.2.1.4-4(A)");

	tet_infoline("No portable test exists");
	tet_result(TET_UNTESTED);

}

int		t5_tfunc();
int		t5_efunc();

pthread_attr_t	T5_attr;

VSTH_RT		T5_et = {
	NULL,	t5_tfunc,	t5_efunc
};

void
test5()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_attr_init() shall not return a value of EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_INIT");
	tet_infoline("Assertion 9945-1:1996 2.4-5(A)");


	vsth_et( &T5_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int
t5_tfunc()
{
	int	val;
	int	rval;

	rval = 1;

	val = pthread_attr_init( &T5_attr );
	if ( val != EINTR )
		rval = 0;
	
	return(rval);
}

int
t5_efunc()
{
	(void) pthread_attr_destroy( &T5_attr );
	return(0);
}



void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	test5,	5	},
	{	NULL,	0	}
};


