
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_attr_setdetachstate
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_attr_setdetachstate.c	1.13
*
*	History:
*	Moved test 3 to a separate executable.
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	5

#define	TE_PATH		"pthread_attr_setdetachstate_exectests"

#ifdef  UNDEF_MACROS
#undef  pthread_attr_setdetachstate
#endif

int	no_tests =	NO_TESTS;

extern	char	**environ;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();
void	test5();

void	t1_f1( int val, int *errors, int *fails );
void	t2_f1( int detachstate, int *errors, int *fails );
void	*t2_t1( void *arg );
void	t3_f1( int *errors, int *fails );
void	t3_c1();
void	t4_f1( int *errors, int *fails );

void
startup()
{
	vsth_startup("pthread_attr_setdetachstate");
}

void
cleanup()
{
	vsth_cleanup("pthread_attr_setdetachstate");
}

void
test1()
{

	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_attr_setdetachstate(attr, detachstate)");
	tet_infoline("shall set the detachstate attribute in the ");
	tet_infoline("attribute object specified by attr to the value of ");
	tet_infoline("detachstate and return 0 if successful.");
	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETDETACHSTATE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2-1(A)");

	t1_f1( PTHREAD_CREATE_DETACHED, &errors, &fails );
	t1_f1( PTHREAD_CREATE_JOINABLE, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t1_f1( int val, int *errors, int *fails )
{

	int		rval;
	int		getv;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setdetachstate( &attr, val );
	if (rval != 0)
	{
		vsth_surprise("pthread_attr_setdetachstate", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_attr_getdetachstate( &attr, &getv );
	if (rval == 0)
	{
		if (getv != val)
		{
			vsth_surprise( NULL, val, getv, fails );
		}
	}
	else
	{
		vsth_surprise("pthread_attr_getdetachstate rval", 0, rval, errors);
	}

	(void) pthread_attr_destroy( &attr );
}

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When detachstate has the value PTHREAD_CREATE_JOINABLE,");
	tet_infoline("then a call to pthread_attr_setdetachstate(attr, detachstate)");
	tet_infoline("shall cause all new threads created with the attributes object attr");
	tet_infoline("to be in the joinable state.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETDETACHSTATE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2-2(A)");

	t2_f1( PTHREAD_CREATE_JOINABLE, &errors, &fails );

	result = VSTH_RESULT( errors, fails );
	tet_result(result);

}

void
t2_f1( int detachstate, int *errors, int *fails )
{
	int		rval;
	void		*tval;
	pthread_t	tid;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setdetachstate( &attr, detachstate );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setdetachstate", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = vsth_pthread_create( &tid, &attr, t2_t1, NULL );
	if ( rval != 0 )
	{
		vsth_surprise("vsth_pthread_create", 0, rval, errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = vsth_pthread_join( tid, &tval );
	if ( detachstate == PTHREAD_CREATE_JOINABLE )
	{
		if ( rval == 0 )
		{
			if ( tval != NULL )
			{
				vsth_surprise("Thread exitval", 0, (int) tval, errors);
			}
		}
		else
		{
			vsth_surprise("vsth_pthread_join", 0, rval, fails );
		}
	}
	else if ( detachstate == PTHREAD_CREATE_DETACHED )
	{
		if ( rval != EINVAL )
		{
			vsth_surprise("vsth_pthread_join", EINVAL, rval, fails);
		}
	}

	(void) pthread_attr_destroy( &attr );
}

void *
t2_t1( void *arg )
{
	pthread_exit(NULL);

	/* NOTREACHED */
	return(NULL);
}

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When detachstate has the value PTHREAD_CREATE_DETACHED,");
	tet_infoline("then a call to");
	tet_infoline("pthread_attr_setdetachstate(attr, detachstate)");
	tet_infoline("shall cause all new threads created with the attributes");
	tet_infoline("object attr to be in the detached state.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETDETACHSTATE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2-3(A)");

	t3_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t3_f1( int *errors, int *fails )
{
	int	rval;

	rval = tet_fork(t3_c1, NULL, VSTH_TIMEOUT, 0 | 1 | 2 );
	if ( rval )
	{
		if ( rval == 2 )
		{
			vsth_surprise("tet_fork", 0, rval, fails);
		}
		else
		{
			vsth_surprise("tet_fork", 0, rval, errors);
		}
	}
}

void
t3_c1()
{
	char		*argv[2];

	argv[0] = execpath(TE_PATH);
	argv[1] = NULL;

	(void) tet_exec(execpath(TE_PATH), argv, environ);

	exit(1);
}

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the value of detachstate is other than");
	tet_infoline("PTHREAD_CREATE_DETACHED or PTHREAD_CREATE_JOINABLE,");
	tet_infoline("then a call to");
	tet_infoline("pthread_attr_setdetachstate(attr, detachstate) shall");
	tet_infoline("fail and return EINVAL.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETDETACHSTATE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.4-4(A)");

	t4_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

void
t4_f1( int *errors, int *fails )
{
	int		rval;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setdetachstate(
				&attr,
				2 * ( PTHREAD_CREATE_DETACHED + PTHREAD_CREATE_JOINABLE ));
	if ( rval != EINVAL )
	{
		vsth_surprise("pthread_attr_setdetachstate", EINVAL, rval, fails);
	}

	(void) pthread_attr_destroy( &attr );
}


int		t5_sfunc();
int		t5_tfunc();
int		t5_efunc();

pthread_attr_t	T5_attr;

VSTH_RT		T5_et = {
	t5_sfunc,	t5_tfunc,	t5_efunc
};


void
test5()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("pthread_attr_setdetachstate 5");
	tet_infoline("A call to pthread_attr_setdetachstate() shall");
	tet_infoline("not return a value of EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETDETACHSTATE");
	tet_infoline("Assertion 9945-1:1996 2.4-5(A)");

	vsth_et( &T5_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int
t5_sfunc()
{
	int	rval;

	rval = pthread_attr_init( &T5_attr );
	return(rval);
}

int
t5_tfunc()
{
	int	val;
	int	rval;
	void	*addr;

	rval = 1;

	val = pthread_attr_setdetachstate( &T5_attr, PTHREAD_CREATE_JOINABLE );
	if ( val != EINTR )
		rval = 0;
	
	return(rval);
}

int
t5_efunc()
{
	(void) pthread_attr_destroy( &T5_attr );
	return(0);
}




void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	test5,	5	},
	{	NULL,	0	}
};


