
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_attr_setguardsize
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_attr_setguardsize.c	1.20
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	4

#ifdef  UNDEF_MACROS
#undef  pthread_attr_setguardsize
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();

void	t1_f1( int val, int *errors, int *fails );
void	t2_f1( int *errors, int *fails );
void	t3_f1( size_t val, int *errors, int *fails );

void
startup()
{
	vsth_startup("pthread_attr_setguardsize");
}

void
cleanup()
{
	vsth_cleanup("pthread_attr_setguardsize");
}

void
test1()
{

	int		errors;
	int		fails;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_attr_setguardsize(attr, guardsize)");
	tet_infoline("sets the value of the guardsize attribute in the");
	tet_infoline("object specified by attr to guardsize and returns 0");
	tet_infoline("if successful.");

	tet_infoline("XCAE ref: Component PTHREAD_ATTR_SETGUARDSIZE");
	tet_infoline("Assertion 1(A)");

	p = tet_getvar("VSTH_VALID_GUARDSIZE");
	if ( p && *p )
	{
		t1_f1( atoi(p), &errors, &fails );
		t1_f1( 0,   &errors, &fails );
	}
	else
	{
		vsth_badconfig("VSTH_VALID_GUARDSIZE", &errors);
	}

	result = VSTH_RESULT( errors, fails );
	tet_result(result);
}

void
t1_f1( int val, int *errors, int *fails )
{
	int		rval;
	size_t		getv;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setguardsize( &attr, (size_t) val );
	if (rval != 0)
	{
		vsth_surprise("pthread_attr_setguardsize", 0, rval, fails);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_attr_getguardsize( &attr, &getv );
	if (rval == 0)
	{
		if ( getv != (size_t) val )
		{
			vsth_surprise( "Saved guardsize", val, getv, fails);
		}
	}
	else
	{
		vsth_surprise( "pthread_attr_getguardsize rval", 0, rval, errors );
	}

	(void) pthread_attr_destroy( &attr );
}


void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the parameter attr specifies an invalid attributes");
	tet_infoline("object, then a call to");
	tet_infoline("pthread_attr_setguardsize(attr, guardsize) shall fail");
	tet_infoline("and return EINVAL.");

	tet_infoline("XCAE ref: Component PTHREAD_ATTR_SETGUARDSIZE");
	tet_infoline("Assertion 2(A)");

	p = tet_getvar("VSTH_UPAO_EINVAL");
	if ( p && *p )
	{
		if (( *p == 'Y' ) || ( *p == 'y' ))
		{
			t2_f1( &errors, &fails );
			result = VSTH_RESULT( errors, fails );
		}
		else
		{
			tet_printf("VSTH_UPAO_EINVAL is set to %s", p);
			result = TET_UNSUPPORTED;
		}
	}
	else
	{
		vsth_badconfig("VSTH_UPAO_EINVAL", &errors);
		result = VSTH_RESULT( errors, fails );
	}

	tet_result(result);
}

void
t2_f1( int *errors, int *fails )
{

	int		rval;
	size_t		size;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_destroy( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_destroy", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setguardsize( &attr, 0 );
	if ( rval != EINVAL )
	{
		vsth_surprise("pthread_attr_setguardsize", EINVAL, rval, fails);
	}
}

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;

	tet_infoline("When the parameter guardsize contains an invalid value,");
	tet_infoline("then a call to");
	tet_infoline("pthread_attr_setguardsize(attr, guardsize) shall fail");
	tet_infoline("and return EINVAL.");

	tet_infoline("XCAE ref: Component PTHREAD_ATTR_SETGUARDSIZE");
	tet_infoline("Assertion 3(A)");

	p = tet_getvar("VSTH_INVALID_GUARDSIZE");
	if ( p && *p )
	{
		if (( *p == 'N' ) || ( *p == 'n' ))
		{
			tet_infoline("The implementation considers all guardsize values valid");
			result = TET_UNSUPPORTED;
		}
		else
		{
			t3_f1( (size_t) atol(p), &errors, &fails );
			result = VSTH_RESULT( errors, fails );
		}
	}
	else
	{
		vsth_badconfig("VSTH_INVALID_GUARDSIZE", &errors);
		result = VSTH_RESULT( errors, fails );
	}

	tet_result(result);
}

void
t3_f1( size_t val, int *errors, int *fails )
{

	int		rval;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setguardsize( &attr, val );
	if ( rval != EINVAL )
	{
		vsth_surprise("pthread_attr_setguardsize", EINVAL, rval, fails);
	}

	(void) pthread_attr_destroy( &attr );
}


int		t4_sfunc();
int		t4_tfunc();
int		t4_efunc();

pthread_attr_t	T4_attr;

VSTH_RT		T4_et = {
	t4_sfunc,	t4_tfunc,	t4_efunc
};

void
test4()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("A call to pthread_attr_setguardsize() shall not");
	tet_infoline("return a value of EINTR.");

	tet_infoline("XCAE ref: Component PTHREAD_ATTR_SETGUARDSIZE");
	tet_infoline("Assertion 4(A)");

	vsth_et( &T4_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

	tet_result(result);
}

int
t4_sfunc()
{
	int	rval;

	rval = pthread_attr_init( &T4_attr );
	return(rval);
}

int
t4_tfunc()
{
	int	val;
	int	rval;

	rval = 1;

	val = pthread_attr_setguardsize( &T4_attr, 0 );
	if ( val != EINTR )
		rval = 0;
	
	return(rval);
}

int
t4_efunc()
{
	(void) pthread_attr_destroy( &T4_attr );
	return(0);
}




void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	NULL,	0	}
};


