
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_attr_setstackaddr
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_attr_setstackaddr.c	1.16
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	3

#ifdef  UNDEF_MACROS
#undef  pthread_attr_setstackaddr
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();

#ifdef	_POSIX_THREAD_ATTR_STACKADDR

void	t1_f1( int val, int *errors, int *fails );

#endif

void
startup()
{
	vsth_startup("pthread_attr_setstackaddr");
}

void
cleanup()
{
	vsth_cleanup("pthread_attr_setstackaddr");
}

void
test1()
{

	int		errors;
	int		fails;
	int		result;
	int		rval;

	char		*p;

	errors		= 0;
	fails		= 0;
	result		= TET_UNRESOLVED;

	tet_infoline("If the feature test macro _POSIX_THREAD_ATTR_STACKADDR");
	tet_infoline("is defined:");
	tet_infoline("a call to pthread_attr_setstackaddr(attr, stackaddr)");
	tet_infoline("sets the value of the stackaddr attribute of the");
	tet_infoline("attributes object specified by attr to stackaddr");
	tet_infoline("and returns 0 on success.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETSTACKADDR");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2-1(C)");

#ifdef	_POSIX_THREAD_ATTR_STACKADDR

	p = tet_getvar("VSTH_VALID_STACKSIZE");
	if ( p  && *p )
	{
		t1_f1( atoi(p), &errors, &fails );
	}
	else
	{
		vsth_badconfig("VSTH_VALID_STACKSIZE", &errors);
	}
	result = VSTH_RESULT( errors, fails );

#else /* _POSIX_THREAD_ATTR_STACKADDR */


	tet_infoline("_POSIX_THREAD_ATTR_STACKADDR is not defined");
#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif


#endif /* _POSIX_THREAD_ATTR_STACKADDR */

	tet_result(result);
}

#ifdef	_POSIX_THREAD_ATTR_STACKADDR

void
t1_f1( int val, int *errors, int *fails )
{
	int		rval;
	void		*setv;
	void		*getv;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	setv = (void *) malloc(val * sizeof(char));
	if ( setv == NULL )
	{
		tet_infoline("Malloc failed");
		(void) pthread_attr_destroy( &attr );
		return;
	}

	rval = pthread_attr_setstackaddr( &attr, setv );
	if (rval == 0)
	{
		rval = pthread_attr_getstackaddr( &attr, &getv );
		if (rval == 0)
		{
			if (getv != setv)
			{
				vsth_surprise( NULL, (int) setv, (int) getv, fails );
			}
		}
		else
		{
			vsth_surprise("pthread_attr_getstackaddr rval", 0, rval, errors);
		}
	}
	else
	{
		vsth_surprise("pthread_attr_setstackaddr rval", 0, rval, fails);
	}

	(void) pthread_attr_destroy( &attr );
}

#endif	/* _POSIX_THREAD_ATTR_STACKADDR */


#ifdef	_POSIX_THREAD_ATTR_STACKADDR

int		t2_sfunc();
int		t2_tfunc();
int		t2_efunc();

pthread_attr_t	T2_attr;
void		*T2_addr;

VSTH_RT		T2_et = {
	t2_sfunc,	t2_tfunc,	t2_efunc
};

#endif

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_ATTR_STACKADDR");
	tet_infoline("is defined:");
	tet_infoline("a call to pthread_attr_setstackaddr() shall not return");
	tet_infoline("a value of EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETSTACKADDR");
	tet_infoline("Assertion 9945-1:1996 2.4-2(C)");

#ifdef	_POSIX_THREAD_ATTR_STACKADDR

	vsth_et( &T2_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else


	tet_infoline("_POSIX_THREAD_ATTR_STACKADDR is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif


#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_ATTR_STACKADDR

int
t2_sfunc()
{
	int	rval;
	char	*p;

	rval = 1;

	p = tet_getvar("VSTH_VALID_STACKSIZE");
	if ( p && *p )
	{
		T2_addr = (void *) malloc( atoi(p) * sizeof(char) );
		if ( T2_addr != NULL )
		{
			rval = pthread_attr_init( &T2_attr );
		}
	}
	else
	{
		vsth_badconfig("VSTH_VALID_STACKSIZE", NULL);
	}

	return(rval);
}

int
t2_tfunc()
{
	int	val;
	int	rval;
	void	*addr;

	rval = 1;

	val = pthread_attr_setstackaddr( &T2_attr, T2_addr );
	if ( val != EINTR )
		rval = 0;
	
	return(rval);
}

int
t2_efunc()
{
	(void) pthread_attr_destroy( &T2_attr );
	if ( T2_addr != NULL )
		(void) free( T2_addr );
	return(0);
}

#endif

void
test3()
{

	int		errors;
	int		fails;
	int		result;
	int		rval;

	pthread_attr_t	attr;

	errors		= 0;
	fails		= 0;
	result		= TET_UNRESOLVED;


	tet_infoline("If the feature test macro _POSIX_THREAD_ATTR_STACKADDR");
	tet_infoline("is not defined:");
	tet_infoline("a call to pthread_attr_setstackaddr() shall fail");
	tet_infoline("and return ENOSYS.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETSTACKADDR");
	tet_infoline("Assertion 9945-1:1996 16.2.1.4-3(C)");


#ifndef	_POSIX_THREAD_ATTR_STACKADDR

#if	( TEST_XPG > 4 )

	tet_infoline("_POSIX_THREAD_ATTR_STACKADDR is not defined");
	result = TET_UNRESOLVED;

#else

	rval = pthread_attr_init( &attr );
	if ( rval == 0 )
	{
		char	*p;

		p = tet_getvar("VSTH_VALID_STACKSIZE");
		if ( p && *p )
		{
			size_t	val;
			void	*addr;

			val = (size_t) atoi(p);

			addr = (void *) malloc( val * sizeof(char));
			if ( addr != NULL )
			{
				rval = pthread_attr_setstackaddr( &attr, addr );
				if ( rval != ENOSYS )
				{
					vsth_surprise( NULL, ENOSYS, rval, &fails);
				}
				(void) free(addr);
			}
			else
			{
				tet_printf("malloc( %d ) failed %d", val, rval);
				++errors;
			}

			result = VSTH_RESULT( errors, fails );
		}
		else
		{
			result = TET_UNRESOLVED;
			vsth_badconfig("VSTH_VALID_STACKSIZE", &errors);
		}
		(void) pthread_attr_destroy( &attr );
	}
	else
	{
		vsth_surprise("pthread_attr_init rval", 0, rval, &errors);
		result = VSTH_RESULT( errors, fails );
	}

#endif	/* TEST_XPG > 4 */

#else	/* _POSIX_THREAD_ATTR_STACKADDR */

	tet_infoline("_POSIX_THREAD_ATTR_STACKADDR is defined");
	result = TET_UNSUPPORTED;

#endif	/* _POSIX_THREAD_ATTR_STACKADDR */

	tet_result(result);
}



void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	NULL,	0	}
};


