
/*************************************************************************
*
* Copyright (c) 1997 by The Open Group
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif (R) and UNIX (R) are registered trademarks and 'IT DialTone' (TM),
* 'The Open Group' (TM) and the 'X Device' (TM) are trademarks of
* The Open Group.
*
*
*	testset for tset/PTHR.os/all/pthread_attr_setstacksize
*
*	Written by Mount Bonnell Inc. Austin, Texas, USA
*	@(#)pthread_attr_setstacksize.c	1.17
*
*
*************************************************************************/

#if	( TEST_XPG <= 4 )
#include	<sys/types.h>	/* needed for pthread types in POSIX96 mode */
#endif
#include	<ctype.h>
#include	<errno.h>
#include	<pthread.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<time.h>
#include	<unistd.h>

#include	"tet_api.h"
#include	"vsth/vsth.h"
#include	"vsth/vsth_fw.h"

#define	NO_TESTS	5

#ifdef  UNDEF_MACROS
#undef  pthread_attr_setstacksize
#endif

int	no_tests =	NO_TESTS;

void	startup();
void	cleanup();
void	test1();
void	test2();
void	test3();
void	test4();
void	test5();

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

static void	t1_f1( int *errors, int *fails );
static void	t2_f1( int *errors, int *fails );
static void	t3_f1( size_t size, int *errors, int *fails );
static void	t4_f1( int *errors, int *fails );

#else

void	t4_f1( int *errors, int *fails );

#endif

void
startup()
{
	vsth_startup("pthread_attr_setstacksize");
}

void
cleanup()
{
	vsth_cleanup("pthread_attr_setstacksize");
}

void
test1()
{

	int	errors;
	int	fails;
	int	result;
	int	rval;

	errors	= 0;
	fails	= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_ATTR_STACKSIZE");
	tet_infoline("is defined:");
	tet_infoline("a call to pthread_attr_setstacksize(attr, stacksize)");
	tet_infoline("sets the value of the stacksize attribute of the");
	tet_infoline("attributes object specified by attr to stacksize");
	tet_infoline("and returns 0 on success.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETSTACKSIZE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.2-1(C)");

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

	t1_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else /* _POSIX_THREAD_ATTR_STACKSIZE */

	tet_infoline("_POSIX_THREAD_ATTR_STACKSIZE is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif	/* ( TEST_XPG > 4 ) */

#endif /* _POSIX_THREAD_ATTR_STACKSIZE */

	tet_result(result);
}

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

void
t1_f1( int *errors, int *fails )
{

	char		*p;
	int		rval;
	size_t		size;
	pthread_attr_t	attr;

	p = tet_getvar("VSTH_VALID_STACKSIZE");
	if (( ! p ) || ( ! *p ))
	{
		vsth_badconfig("VSTH_VALID_STACKSIZE", errors);
		return;
	}

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	/* try it first with the user configurable value */
	size = atoi(p);
	rval = pthread_attr_setstacksize( &attr, size );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_setstacksize", 0, rval, fails);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	size = sysconf(_SC_THREAD_STACK_MIN);;
	if ( size != -1 )
	{
		rval = pthread_attr_setstacksize( &attr, size );
		if ( rval != 0 )
		{
			vsth_surprise("pthread_attr_setstacksize", 0, rval, fails);
		}
	}
	else
	{
		vsth_surprise("sysconf(_SC_THREAD_STACK_MIN) rval", 0, rval, errors);
	}

	(void) pthread_attr_destroy( &attr );
}

#endif	/* _POSIX_THREAD_ATTR_STACKSIZE */

void
test2()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_ATTR_STACKSIZE");
	tet_infoline("is defined:");
	tet_infoline("when the value of stacksize is less than");
	tet_infoline("PTHREAD_STACK_MIN, then a call to");
	tet_infoline("pthread_attr_setstacksize(attr, stacksize) shall fail");
	tet_infoline("and return EINVAL.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETSTACKSIZE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.4-2(C)");

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

	t2_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_ATTR_STACKSIZE is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}


#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

void
t2_f1( int *errors, int *fails )
{

	int		rval;
	long		stackmin;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	stackmin = sysconf( _SC_THREAD_STACK_MIN );
	if ( stackmin == -1 )
	{
		tet_infoline("sysconf(_SC_THREAD_STACK_MIN) unexpectedly failed");
		(void) pthread_attr_destroy( &attr );
		return;
	}
	rval = pthread_attr_setstacksize( &attr, stackmin - 1 );
	if ( rval != EINVAL )
	{
		vsth_surprise("pthread_attr_setstacksize", EINVAL, rval, fails);
	}

	(void) pthread_attr_destroy( &attr );
}

#endif

void
test3()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;
	char		*p;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_ATTR_STACKSIZE");
	tet_infoline("is defined:");
	tet_infoline("when the value of stacksize exceeds an implementation");
	tet_infoline("defined limit, then a call to");
	tet_infoline("pthread_attr_setstacksize(attr, stacksize) shall fail");
	tet_infoline("and return EINVAL.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETSTACKSIZE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.4-3(C)");

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

	p = tet_getvar("VSTH_INVALID_STACKSIZE");
	if ( p && *p )
	{
		t3_f1( (size_t) atol(p), &errors, &fails );
	}
	else
	{
		vsth_badconfig("VSTH_INVALID_STACKSIZE", &errors);
	}

	result = VSTH_RESULT( errors, fails );

#else

	tet_infoline("_POSIX_THREAD_ATTR_STACKSIZE is not defined");

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif	/* _POSIX_THREAD_ATTR_STACKSIZE */

	tet_result(result);
}

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

void
t3_f1( size_t size, int *errors, int *fails )
{
	int		rval;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}

	rval = pthread_attr_setstacksize( &attr, size );
	if ( rval != EINVAL )
	{
		vsth_surprise("pthread_attr_setstacksize", EINVAL, rval, fails);
	}

	(void) pthread_attr_destroy( &attr );
}

#endif

void
test4()
{

	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_ATTR_STACKSIZE");
	tet_infoline("is not defined:");
	tet_infoline("a call to pthread_attr_setstacksize() shall fail");
	tet_infoline("and return ENOSYS.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETSTACKSIZE");
	tet_infoline("Assertion 9945-1:1996 16.2.1.4-4(C)");

#ifndef	_POSIX_THREAD_ATTR_STACKSIZE

#if	( TEST_XPG > 4 )
	tet_infoline("_POSIX_THREAD_ATTR_STACKSIZE is not defined");
	result = TET_UNRESOLVED;
#else

	t4_f1( &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#endif

#else
	tet_infoline("_POSIX_THREAD_ATTR_STACKSIZE is defined");
	result = TET_UNSUPPORTED;
#endif

	tet_result(result);
}

#ifndef	_POSIX_THREAD_ATTR_STACKSIZE

void
t4_f1( int *errors, int *fails )
{
	int		rval;
	size_t		size;
	char		*p;
	pthread_attr_t	attr;

	rval = pthread_attr_init( &attr );
	if ( rval != 0 )
	{
		vsth_surprise("pthread_attr_init", 0, rval, errors);
		return;
	}
	p = tet_getvar("VSTH_VALID_STACKSIZE");
	if (( ! p ) || ( ! *p ))
	{
		vsth_badconfig("VSTH_VALID_STACKSIZE", errors);
		(void) pthread_attr_destroy( &attr );
		return;
	}

	size = (size_t) atoi(p);
	rval = pthread_attr_setstacksize( &attr, size );
	if ( rval != ENOSYS )
	{
		vsth_surprise("pthread_attr_setstacksize", ENOSYS, rval, fails);
	}

	(void) pthread_attr_destroy( &attr );
}

#endif	/* ! _POSIX_THREAD_ATTR_STACKSIZE */

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

int		t5_sfunc();
int		t5_tfunc();
int		t5_efunc();

size_t		T5_size;
pthread_attr_t	T5_attr;

VSTH_RT		T5_et = {
	t5_sfunc,	t5_tfunc,	t5_efunc
};

#endif

void
test5()
{
	int		errors;
	int		fails;
	int		result;
	int		rval;

	errors		= 0;
	fails		= 0;

	tet_infoline("If the feature test macro _POSIX_THREAD_ATTR_STACKSIZE");
	tet_infoline("is defined:");
	tet_infoline("a call to pthread_attr_setstacksize() shall not return");
	tet_infoline("a value of EINTR.");

	tet_infoline("Posix Ref: Component PTHREAD_ATTR_SETSTACKSIZE");
	tet_infoline("Assertion 9945-1:1996 2.4-5(C)");

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

	vsth_et( &T5_et, &errors, &fails );
	result = VSTH_RESULT( errors, fails );

#else

#if	( TEST_XPG > 4 )
	result = TET_UNRESOLVED;
#else
	result = TET_UNSUPPORTED;
#endif

#endif

	tet_result(result);
}

#ifdef	_POSIX_THREAD_ATTR_STACKSIZE

int
t5_sfunc()
{
	int	rval;
	char	*p;

	rval = 1;

	p = tet_getvar("VSTH_VALID_STACKSIZE");
	if ( p  &&  *p )
	{
		T5_size = (size_t) atoi(p);
		rval = pthread_attr_init( &T5_attr );
	}

	return(rval);
}

int
t5_tfunc()
{
	int	val;
	int	rval;
	void	*addr;

	rval = 1;

	val = pthread_attr_setstacksize( &T5_attr, T5_size );
	if ( val != EINTR )
		rval = 0;
	
	return(rval);
}

int
t5_efunc()
{
	(void) pthread_attr_destroy( &T5_attr );
	return(0);
}

#endif

void	(*setfunc)() = startup;
void	(*clnfunc)() = cleanup;

struct tet_testlist tet_testlist[] = {
	{	test1,	1	},
	{	test2,	2	},
	{	test3,	3	},
	{	test4,	4	},
	{	test5,	5	},
	{	NULL,	0	}
};

