/*
* Copyright (c) 1997 X/Open Company Ltd., trading as The Open Group
*
* All rights reserved. No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* Motif, OSF/1 and UNIX are registered trademarks and X/Open,
* the "X Device" and The Open Group are trademarks of The Open Group.
*
*
* Project: LSB-OS
*
* File: tset/LSB.os/aio/lio_listio/lio_listio.c
*
* Modifications:
* $Log: lio_listio.c,v $
* Revision 1.2  2001/10/02 13:25:13  ajosey
* clean up header block comments
*
* Revision 1.1  2001/05/27 06:59:13  cyeoh
* Adds files for lsb-os test suite
*
*
*/

#include	<vsrt/vsrt.h>
#include	<aio.h>
#include	<termios.h>
#include	<limits.h>
#include	<setprv.h>

#define 	NO_TESTS	45	/* number of tests in testset */

#ifdef UNDEF_MACROS
#undef lio_listio
#endif

#define		WAITTIME	(5 * SPEEDFACTOR)

extern	char	*errname();

int	no_tests = NO_TESTS;

private	int	tb_lio_listio();
private void	test1();
private void	test2();
private void	test3();
private void	test4();
private void	test5();
private void	test6();
private void	test7();
private void	test8();
private void	test9();
private void	test10();
private void	test11();
private void	test12();
private void	test13();
private void	test14();
private void	test15();
private void	test16();
private void	test17();
private void	test18();
private void	test19();
private void	test20();
private void	test21();
private void	test22();
private void	test23();
private void	test24();
private void	test25();
private void	test26();
private void	test27();
private void	test28();
private void	test29();
private void	test30();
private void	test31();
private void	test32();
private void	test33();
private void	test34();
private void	test35();
private void	test36();
private void	test37();
private void	test38();
private void	test39();
private void	test40();
private void	test41();
private void	test42();
private void	test43();
private void	test44();
private void	test45();
private int dotest1read();
private int dotest1write();
private int dotest2read();
private int dotest2write();
private int dotest4read();
private int dotest4write();
private int dotest5read();
private int dotest5write();
private int dotest6read();
private int dotest6write();
private int dotest9read();
private int dotest9write();
private int dotest10read();
private int dotest10write();
private int dotest15read();
private int dotest17read();
private int dotest18read();
private int dotest24write();
private int dotest25write();
private int dotest27write();
private int dotest28write();
private int dotest31();
private int dotest32();
private int dotest33();
private int dotest34();
private int dotest35read();
private int dotest35write();
private int dotest36();
private int dotest37();
private int dotest38();
private int dotest39();
private int dotest40();
private int dotest41();
private int dotest42();
private int dotest44read();
private int dotest44write();

int	supports_lio_listio;
int	supports_aio_cancel;
int	supports_aio_error;
int	supports_aio_return;

/*determine if a type of file is to be tried. set in startup*/
/*pipe support is determined dynamically*/
int	vsrt_aio_test_with_tty = 0;
int	vsrt_aio_test_with_file = 0;
int	vsrt_aio_test_with_pty = 0;

/*each piece of a test increments on of these.  must be zeroed at test start*/
int	vsrt_aio_test_pass;
int	vsrt_aio_test_unsup;
int	vsrt_aio_test_fail;

/*instances of things to test on from tetexec, loaded in startup*/
char	vsrt_aio_tty1[128];
char	vsrt_aio_tty2[128];
char	vsrt_aio_ptym[128];
char	vsrt_aio_testfile[4096];

int	vsrt_relax_write_order = 0;

void atstart(void);
void atstart(void)
{
	char	*str;

	vsrt_check_support("AIO_LIO_LISTIO", &supports_lio_listio);
	vsrt_check_support("AIO_ERROR", &supports_aio_error);
	vsrt_check_support("AIO_RETURN", &supports_aio_return);
	vsrt_check_support("AIO_CANCEL", &supports_aio_cancel);

	/*get config info and determine what sorts of files to test with*/
	str = tet_getvar("VSRT_RELAX_WRITE_ORDER");
	if ((str != NULL) && (strlen(str) != 0))
		if ((str[0] == 'y') || (str[0] == 'Y'))
			vsrt_relax_write_order = 1;

	str = tet_getvar("VSRT_FILE_ASYNC_IO");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_file = 1;
		strcpy(vsrt_aio_testfile, str);
	}

	str = tet_getvar("VSRT_TERMIOS_TTY");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_tty = 1;
		strcpy(vsrt_aio_tty1, str);
		str = tet_getvar("VSRT_TERMIOS_LOOP");
		if ((str != NULL) && (strlen(str) != 0))
			strcpy(vsrt_aio_tty2, str);
		else
			vsrt_aio_test_with_tty = 0;
	}

	str = tet_getvar("VSRT_MASTER_PTY");
	if ((str != NULL) && (strlen(str) != 0)) {
		vsrt_aio_test_with_pty = 1;
		strcpy(vsrt_aio_ptym, str);
	}
}

void atend(void);
void atend(void)
{
}

public	void	(*setfunc) () = atstart;
public	void	(*clnfunc) () = atend;

public	struct tet_testlist tet_testlist[] = {
	 test1, 1,
	 test2, 2,
	 test3, 3,
	 test4, 4,
	 test5, 5,
	 test6, 6,
	 test7, 7,
	 test8, 8,
	 test9, 9,
	 test10, 10,
	 test11, 11,
	 test12, 12,
	 test13, 13,
	 test14, 14,
	 test15, 15,
	 test16, 16,
	 test17, 17,
	 test18, 18,
	 test19, 19,
	 test20, 20,
	 test21, 21,
	 test22, 22,
	 test23, 23,
	 test24, 24,
	 test25, 25,
	 test26, 26,
	 test27, 27,
	 test28, 28,
	 test29, 29,
	 test30, 30,
	 test31, 31,
	 test32, 32,
	 test33, 33,
	 test34, 34,
	 test35, 35,
	 test36, 36,
	 test37, 37,
	 test38, 38,
	 test39, 39,
	 test40, 40,
	 test41, 41,
	 test42, 42,
	 test43, 43,
	 test44, 44,
	 test45, 45,
	NULL, 0
};

long	aio_max = 0;
long	aio_sysmax = 0;

int	got_sig = 0;

private void sleeploop(unsigned int secs)
{
	/* Sleep for the specified number of seconds by calling sleep()
	   in a loop until it returns 0 as the unslept time.
	   This is used where one or more signals are expected to
	   arrive during the sleep time. */

	while (secs != 0)
		secs = sleep(secs);
}

private	void	handler5();
private	void	handler5(signo)
int	signo;
{
	got_sig++;
}

private void rt_handler7();
private void rt_handler7(signo, info, context)
int	signo;
siginfo_t	*info;
void	*context;
{
	got_sig++;
}

unsigned char got_data[100];
unsigned char	got_requests[101];


/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementation
supports the lio_listio() function as described in System
Interfaces and Headers, Issue 5:
	A successful call to lio_listio() when mode is
	LIO_WAIT shall initiate the nent asynchronous I/O
	requests in list, wait until all I/O is complete,
	return 0, and not generate a signal on completion.
*/

private void test1()
{
	int	terrors;

	DBUG_ENTER("test1");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t1", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest1read);
	vsrt_aio_test_allbutfile_write(dotest1write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest1read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	time_t	tstart, tend;
	pid_t	pid2;
	int	status;
	struct	sigevent	sig;
	int	c;

	DBUG_ENTER("dotest1read");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t1read", "set up for aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_READ;
		plist[i] = &paiocbs[i];
	}
	sig.sigev_notify = SIGEV_NONE;
	sig.sigev_signo = SIGUSR1;

	PATH_TRACE;
	DBUG_PRINTF("t1read", "fork a child which will initiate the reads");
	VSRT_FORK(pid2);
		got_sig = 0;
		if (vsrt_signal(SIGUSR1, handler5) == -1)
			exit(1);

		DBUG_PRINTF("t1read", "lio_listio");
		(void)time(&tstart);
		if (tb_lio_listio(LIO_WAIT, plist, aio_max, &sig) == -1) {
			vsrt_sysfail("lio_listio");
			exit(1);
		}
		(void)time(&tend);

		PATH_TRACE;
		DBUG_PRINTF("t1read", "check it waited");
		if ((tend - tstart) < 3) {
			in_rpt("Expected lio_listio to wait at least 3 seconds, waited %d", tend - tstart);
			xx_rpt(FAILURE);
			exit(1);
		}
		PATH_TRACE;
		DBUG_PRINTF("t1read", "wait for the reads to complete");
		for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
			for (i = 0; i < aio_max; i++) {
				if (aio_error(&paiocbs[i]) == EINPROGRESS) {
					sleep(2);
					break;
				}
			}
			if (i == aio_max)
				break;
		}

		if (j == VSRT_AIO_ITERATIONS) {
			for (i = 0; i < aio_max; i++) {
				if (aio_error(&paiocbs[i]) == EINPROGRESS)
					in_rpt("request %d is still in progress", i);
			}
			(void)aio_cancel(readfd, NULL);
			exit(1);
		}

		if (got_sig != 0) {
			in_rpt("Expected no signals to be generated, received %d", got_sig);
			xx_rpt(FAILURE);
			errors++;
		}

		PATH_TRACE;
		DBUG_PRINTF("t1read", "check the results");
		memset(got_data, 0xff, 50);
		for (i = aio_max-1; i >= 0; i--) {
			retval2 = aio_error(&paiocbs[i]);
			if (retval2 == -1) {
				err = errno;
				in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			if (retval2 != 0) {
				in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
				errors++;
				continue;
			}
			retval = aio_return(&paiocbs[i]);
			if (retval == -1) {
				err = errno;
				in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			memset(tbuf, 0, sizeof(tbuf));
			sprintf(tbuf, "message %3d\n", i);
			if (retval != strlen(tbuf)) {
				in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
				errors++;
				continue;
			}
			if (sscanf((char *)paiocbs[i].aio_buf, "message %3d\n", &c) != 1) {
		
				in_rpt("Could not match returned data for request %d to \"message xxx \\n\", received \"%s\"", i, paiocbs[i].aio_buf);
				errors++;
			}
			else {
				if ((c < 0) || (c >= aio_max)) {
					in_rpt("Message number returned in request %d is %d, expected between 0 and %d", i, c, aio_max-1);
					errors++;
				} else
					if (got_data[c] != 0xff) {
						got_data[c]++;
						in_rpt("Message %d received %d times", c, got_data[c]);
						errors++;
					} else
						got_data[c]= 1;
			}
		}
		for (i = aio_max-1; i >= 0; i--) {
			if (got_data[i] == 0xff) {
				in_rpt("Message number %d not read by any request", i);
				errors++;
			}
		}

	if (errors != 0)
		exit(1);

	VSRT_EFORK;

	PATH_TRACE;
	DBUG_PRINTF("t1read", "write some data after a delay");
	sleep(5);
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
	}

	status = vsrt_wait_for(pid2, 60);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest1write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;
	int	status;
	pid_t	pid2;
	time_t	tstart, tend;
	struct	sigevent	sig;
	int	fillcnt, readcnt;
	char	*null;
	FILE	*readfile;
	int	c;

	DBUG_ENTER("dotest1write");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t1write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	fillcnt = 0;
	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if ((retval = write(writefd, bbuf, sizeof(bbuf))) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
		else
			fillcnt += retval;
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	DBUG_PRINTF("t1write", "set up some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[i] = &paiocbs[i];
	}

	sig.sigev_notify = SIGEV_NONE;
	sig.sigev_signo = SIGUSR1;

	PATH_TRACE;
	DBUG_PRINTF("t1write", "fork a child which will initiate the writes");
	VSRT_FORK(pid2);

		got_sig = 0;

		if (vsrt_signal(SIGUSR1, handler5) == -1)
			exit(1);

		PATH_TRACE;
		DBUG_PRINTF("t1write", "lio_listio");
		(void)time(&tstart);
		if (tb_lio_listio(LIO_WAIT, plist, aio_max, &sig) == -1) {
			vsrt_sysfail("lio_listio");
			exit(1);
		}

		(void)time(&tend);

		PATH_TRACE;
		DBUG_PRINTF("t1write", "check it waited");
		if ((tend - tstart) < 3) {
			in_rpt("Expected lio_listio to wait at least 3 seconds, waited %d", tend - tstart);
			xx_rpt(FAILURE);
			exit(1);
		}

		PATH_TRACE;

		DBUG_PRINTF("t1write", "wait for writes to complete");
		for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
			for (i = 0; i < aio_max; i++) {
				if (aio_error(&paiocbs[i]) == EINPROGRESS) {
					sleep(2);
					break;
				}
			}
			if (i == aio_max)
				break;
		}
	
		if (j == VSRT_AIO_ITERATIONS) {
			for (i = 0; i < aio_max; i++) {
				if (aio_error(&paiocbs[i]) == EINPROGRESS)
					in_rpt("request %d is still in progress", i);
			}
			(void)aio_cancel(writefd, NULL);
			exit(1);
		}

		PATH_TRACE;
		DBUG_PRINTF("t1write", "check the results");
		for (i = aio_max-1; i >= 0; i--) {
			retval2 = aio_error(&paiocbs[i]);
			if (retval2 == -1) {
				err = errno;
				in_rpt("aio_error failed for write #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			if (retval2 != 0) {
				in_rpt("Error status for write %d is %d (%s)", i, retval2, errname(retval2));
				errors++;
				continue;
			}
			retval = aio_return(&paiocbs[i]);
			if (retval == -1) {
				err = errno;
				in_rpt("aio_return failed for write #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			if (retval != 12) {
				in_rpt("aio_return returned %d for write #%d, expected 12", retval, i);
				errors++;
			}
		}

		if (got_sig != 0) {
			in_rpt("Expected no signals to be generated, received %d", got_sig);
			xx_rpt(FAILURE);
			errors++;
		}

		if (errors != 0)
			exit(1);

	VSRT_EFORK;

	PATH_TRACE;
	DBUG_PRINTF("t1write", "delay");
	sleep(5);

	PATH_TRACE;
	DBUG_PRINTF("t1write", "read all the blocking data so the lio_listio can proceed");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
	}
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl #1");
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		DBUG_RETURN(-1);
	}

	if ((null = malloc(fillcnt)) == NULL) {
		vsrt_sysfail("malloc null");
		DBUG_RETURN(-1);
	}

	readcnt = 0;
	(void)lseek(readfd, 0, SEEK_SET);
	while (readcnt != fillcnt) {
		retval = read(readfd, null, fillcnt-readcnt);
		if (retval == -1) {
			if (errno == EAGAIN) {
				sleep(1);
				continue;
			}
			vsrt_sysfail("read");
			free(null);
			DBUG_RETURN(-1);
		}
		readcnt += retval;
	}

	PATH_TRACE;
	DBUG_PRINTF("t1write", "completed reading data");

	free(null);

	status = vsrt_wait_for(pid2, 60);
	if (status == -1)
		errors++;
	else if (!WIFEXITED(status)) {
		if (WIFSIGNALED(status) != 0)
			in_rpt("Child process exited due to signal %d",  WTERMSIG(status));
		else
			in_rpt("Child process exited abnormally with exit status %d",  WEXITSTATUS(status));
		xx_rpt(FAILURE);
		errors++;
	} else
		errors += WEXITSTATUS(status);

	free(plist);
	free(bufs);
	free(paiocbs);

	if (errors != 0)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t1write", "now read the lio_listio data");
	readfile = fdopen(readfd, "r");
	if (readfile == NULL) {
		vsrt_sysfail("fdopen");
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t1write", "check the results");
	for (i = 0; i < aio_max; i++) {
		if (fgets(tbuf, sizeof(tbuf), readfile) == NULL) {
			err = errno;
			in_rpt("For message %d fgets() failed, errno = %d (%s)", i, err, errname(err));
			errors++;
			break;
		}
		if (strlen(tbuf) != 12) {
			in_rpt("For message %d expected to read %d bytes, received %d", i, strlen(&bufs[i*sizeof(tbuf)]), strlen(tbuf));
			errors++;
		}
		if (vsrt_relax_write_order == 0) {
			sprintf(bbuf, "message %3d\n", i);
			if (strcmp(bbuf, tbuf) != 0) {
				in_rpt("For message %d, expected returned data \"%s\", received \"%s\"", i, bbuf, tbuf);
				errors++;
			}
		} else {
			if (sscanf(tbuf, "message %3d\n", &c) != 1) {
		
				in_rpt("Could not match piece %d of data from file to \"message xxx \\n\", received \"%s\"", i, tbuf);
				errors++;
			}
			else {
				if ((c < 0) || (c >= aio_max)) {
					in_rpt("Message number in piece %d of data from file is %d, expected between 0 and %d", i, c, aio_max-1);
					errors++;
				} else
					got_data[c] = 0;
			}
		}
	}

	if (vsrt_relax_write_order == 1) {
		for (i = 0; i < aio_max; i++) {
			if (got_data[i] == 0xff) {
				in_rpt("Data from request %d not found in file", i);
				errors++;
			}
		}
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}


/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_SYNCHRONOUS_IO are
defined or the implementation supports the lio_listio()
function as described in System Interfaces and Headers,
Issue 5:
	A successful call to lio_listio() when mode is
	LIO_NOWAIT shall initiate or queue the nent asynchro-
	nous I/O requests in list, and return 0 immediately.
*/

private void test2()
{
	int	terrors;

	DBUG_ENTER("test2");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t2", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest2read);
	vsrt_aio_test_allbutfile_write(dotest2write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest2read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	time_t	tstart, tend;
	struct	sigevent	sig;
	int	c;

	DBUG_ENTER("dotest2read");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t2read", "set up for aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_READ;
		plist[i] = &paiocbs[i];
	}
	sig.sigev_notify = SIGEV_NONE;
	sig.sigev_signo = SIGUSR1;

	got_sig = 0;
	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t2read", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, &sig) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	(void)time(&tend);
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t2read", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t2read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t2read", "wait for the reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	if (got_sig != 0) {
		in_rpt("Expected no signals to be generated, received %d", got_sig);
		xx_rpt(FAILURE);
		errors++;
	}

	PATH_TRACE;
	DBUG_PRINTF("t2read", "check the results");
	memset(got_data, 0xff, 50);
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (retval != strlen(tbuf)) {
			in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
			errors++;
			continue;
		}
		if (sscanf((char *)paiocbs[i].aio_buf, "message %3d\n", &c) != 1) {
		
			in_rpt("Could not match returned data for request %d to \"message xxx \\n\", received \"%s\"", i, paiocbs[i].aio_buf);
			errors++;
		}
		else {
			if ((c < 0) || (c >= aio_max)) {
				in_rpt("Message number returned in request %d is %d, expected between 0 and %d", i, c, aio_max-1);
				errors++;
			} else
				if (got_data[c] != 0xff) {
					got_data[c]++;
					in_rpt("Message %d received %d times", c, got_data[c]);
					errors++;
				} else
					got_data[c]= 1;
		}
	}
	for (i = aio_max-1; i >= 0; i--) {
		if (got_data[i] == 0xff) {
			in_rpt("Message number %d not read by any request", i);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest2write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;
	time_t	tstart, tend;
	struct	sigevent	sig;
	int	fillcnt, readcnt;
	char	*null;
	FILE	*readfile;
	int	c;

	DBUG_ENTER("dotest2write");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t2write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	fillcnt = 0;
	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if ((retval = write(writefd, bbuf, sizeof(bbuf))) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
		else
			fillcnt += retval;
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	DBUG_PRINTF("t2write", "set up some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[i] = &paiocbs[i];
	}

	sig.sigev_notify = SIGEV_NONE;
	sig.sigev_signo = SIGUSR1;
	got_sig = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t2write", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, &sig) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	(void)time(&tend);

	PATH_TRACE;
	DBUG_PRINTF("t2write", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;

	DBUG_PRINTF("t2write", "read all the blocking data so the writes can proceed");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
		sleep(4);
	}
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl #1");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	if ((null = malloc(fillcnt)) == NULL) {
		vsrt_sysfail("malloc null");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	readcnt = 0;
	(void)lseek(readfd, 0, SEEK_SET);
	while (readcnt != fillcnt) {
		retval = read(readfd, null, fillcnt-readcnt);
		if (retval == -1) {
			if (errno == EAGAIN) {
				sleep(1);
				continue;
			}
			vsrt_sysfail("read");
			(void)aio_cancel(writefd, NULL);
			free(null);
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
		readcnt += retval;
	}

	free(null);

	PATH_TRACE;
	DBUG_PRINTF("t2write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t2write", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for write %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval != 12) {
			in_rpt("aio_return returned %d for write #%d, expected 12", retval, i);
			errors++;
		}
	}

	if (got_sig != 0) {
		in_rpt("Expected no signals to be generated, received %d", got_sig);
		xx_rpt(FAILURE);
		errors++;
	}

	free(plist);
	free(bufs);
	free(paiocbs);

	if (errors != 0)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t2write", "now read the lio_listio data ");
	readfile = fdopen(readfd, "r");
	if (readfile == NULL) {
		vsrt_sysfail("fdopen");
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t2write", "check the results");
	memset(got_data, 0xff, 50);
	for (i = 0; i < aio_max; i++) {
		if (fgets(tbuf, sizeof(tbuf), readfile) == NULL) {
			err = errno;
			in_rpt("For message %d fgets() failed, errno = %d (%s)", i, err, errname(err));
			errors++;
			break;
		}
		if (strlen(tbuf) != 12) {
			in_rpt("For message %d expected to read %d bytes, received %d", i, strlen(&bufs[i*sizeof(tbuf)]), strlen(tbuf));
			errors++;
		}
		if (vsrt_relax_write_order == 0) {
			sprintf(bbuf, "message %3d\n", i);
			if (strcmp(bbuf, tbuf) != 0) {
				in_rpt("For message %d, expected returned data \"%s\", received \"%s\"", i, bbuf, tbuf);
				errors++;
			}
		} else {
			if (sscanf(tbuf, "message %3d\n", &c) != 1) {
		
				in_rpt("Could not match piece %d of data from file to \"message xxx \\n\", received \"%s\"", i, tbuf);
				errors++;
			}
			else {
				if ((c < 0) || (c >= aio_max)) {
					in_rpt("Message number in piece %d of data from file is %d, expected between 0 and %d", i, c, aio_max-1);
					errors++;
				} else
					if (got_data[c] != 0xff) {
						got_data[c]++;
						in_rpt("Message %d received %d times", c, got_data[c]);
						errors++;
					} else
						got_data[c]= 1;
			}
		}
	}
	if (vsrt_relax_write_order == 1) {
		for (i = 0; i < aio_max; i++) {
			if (got_data[i] == 0xff) {
				in_rpt("Data from request %d not found in file", i);
				errors++;
			}
		}
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}


/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_SYNCHRONOUS_IO are
defined or the implementation supports the lio_listio()
function as described in System Interfaces and Headers,
Issue 5:
	On a call to lio_listio() when mode is LIO_NOWAIT and
	sig->sigev_notify is SIGEV_NONE no asynchronous
	notification shall be delivered when the all the
	operations complete.
*/

private void test3()
{
	int	terrors;

	DBUG_ENTER("test3");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t3", "using %d requests", aio_max);

	/*just rerun test 2 as this is covered therein*/
	vsrt_aio_test_allbutfile(dotest2read);
	vsrt_aio_test_allbutfile_write(dotest2write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when mode is LIO_NOWAIT and
	sig is NULL no asynchronous notification shall be
	delivered when the all the operations complete.
*/

private void test4()
{
	int	terrors;

	DBUG_ENTER("test4");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t4", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest4read);
	vsrt_aio_test_allbutfile_write(dotest4write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest4read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	time_t	tstart, tend;
	struct	sigevent	sig;
	int	c;

	DBUG_ENTER("dotest4read");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t4read", "set up for aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_READ;
		plist[i] = &paiocbs[i];
	}
	sig.sigev_notify = SIGEV_NONE;
	sig.sigev_signo = SIGUSR1;

	got_sig = 0;
	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t4read", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (lio_listio(LIO_NOWAIT, plist, aio_max, NULL) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	(void)time(&tend);
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t4read", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t4read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t4read", "wait for the reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	if (got_sig != 0) {
		in_rpt("Expected no signals to be generated, received %d", got_sig);
		xx_rpt(FAILURE);
		errors++;
	}

	PATH_TRACE;
	DBUG_PRINTF("t4read", "check the results");
	memset(got_data, 0xff, 50);
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (retval != strlen(tbuf)) {
			in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
			errors++;
			continue;
		}
		if (sscanf((char *)paiocbs[i].aio_buf, "message %3d\n", &c) != 1) {
	
			in_rpt("Could not match returned data for request %d to \"message xxx \\n\", received \"%s\"", i, paiocbs[i].aio_buf);
			errors++;
		}
		else {
			if ((c < 0) || (c >= aio_max)) {
				in_rpt("Message number returned in request %d is %d, expected between 0 and %d", i, c, aio_max-1);
				errors++;
			} else
				if (got_data[c] != 0xff) {
					got_data[c]++;
					in_rpt("Message %d received %d times", c, got_data[c]);
					errors++;
				} else
					got_data[c]= 1;
		}
	}
	for (i = aio_max-1; i >= 0; i--) {
		if (got_data[i] == 0xff) {
			in_rpt("Data from request %d not found in file", i);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest4write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;
	time_t	tstart, tend;
	struct	sigevent	sig;
	int	fillcnt, readcnt;
	char	*null;
	FILE	*readfile;
	int	c;

	DBUG_ENTER("dotest4write");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t4write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	fillcnt = 0;
	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if ((retval = write(writefd, bbuf, sizeof(bbuf))) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
		else
			fillcnt += retval;
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t4write", "set up some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[i] = &paiocbs[i];
	}

	sig.sigev_notify = SIGEV_NONE;
	sig.sigev_signo = SIGUSR1;


	got_sig = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t4write", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, NULL) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	(void)time(&tend);

	PATH_TRACE;
	DBUG_PRINTF("t4write", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		xx_rpt(FAILURE);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;

	DBUG_PRINTF("t4write", "read all the blocking data so the writes can proceed");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
		sleep(4);
	}
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl #1");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	if ((null = malloc(fillcnt)) == NULL) {
		vsrt_sysfail("malloc null");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	readcnt = 0;
	(void)lseek(readfd, 0, SEEK_SET);
	while (readcnt != fillcnt) {
		retval = read(readfd, null, fillcnt-readcnt);
		if (retval == -1) {
			if (errno == EAGAIN) {
				sleep(1);
				continue;
			}
			vsrt_sysfail("read");
			(void)aio_cancel(writefd, NULL);
			free(null);
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
		readcnt += retval;
	}

	free(null);

	PATH_TRACE;
	DBUG_PRINTF("t4write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t4write", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for write %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval != 12) {
			in_rpt("aio_return returned %d for write #%d, expected 12", retval, i);
			errors++;
		}
	}

	if (got_sig != 0) {
		in_rpt("Expected no signals to be generated, received %d", got_sig);
		xx_rpt(FAILURE);
		errors++;
	}

	free(plist);
	free(bufs);
	free(paiocbs);

	if (errors != 0)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t4write", "now read the lio_listio data ");
	readfile = fdopen(readfd, "r");
	if (readfile == NULL) {
		vsrt_sysfail("fdopen");
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t4write", "check the results");
	memset(got_data, 0xff, 50);
	for (i = 0; i < aio_max; i++) {
		if (fgets(tbuf, sizeof(tbuf), readfile) == NULL) {
			err = errno;
			in_rpt("For message %d fgets() failed, errno = %d (%s)", i, err, errname(err));
			errors++;
			break;
		}
		if (strlen(tbuf) != 12) {
			in_rpt("For message %d expected to read %d bytes, received %d", i, strlen(&bufs[i*sizeof(tbuf)]), strlen(tbuf));
			errors++;
		}
		if (vsrt_relax_write_order == 0) {
			sprintf(bbuf, "message %3d\n", i);
			if (strcmp(bbuf, tbuf) != 0) {
				in_rpt("For message %d, expected returned data \"%s\", received \"%s\"", i, bbuf, tbuf);
				errors++;
			}
		} else {
			if (sscanf(tbuf, "message %3d\n", &c) != 1) {
		
				in_rpt("Could not match piece %d of data from file to \"message xxx \\n\", received \"%s\"", i, tbuf);
				errors++;
			}
			else {
				if ((c < 0) || (c >= aio_max)) {
					in_rpt("Message number in piece %d of data from file is %d, expected between 0 and %d", i, c, aio_max-1);
					errors++;
				} else
					if (got_data[c] != 0xff) {
						got_data[c]++;
						in_rpt("Message %d received %d times", c, got_data[c]);
						errors++;
					} else
						got_data[c]= 1;
			}
		}
	}
	if (vsrt_relax_write_order == 1) {
		for (i = 0; i < aio_max; i++) {
			if (got_data[i] == 0xff) {
				in_rpt("Data from request %d not found in file", i);
				errors++;
			}
		}
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when mode is LIO_NOWAIT and
	sig->sigev_notify is SIGEV_SIGNAL the signal speci-
	fied in sig->sigev_signo shall be generated for the
	process when the all the operations complete.
*/

private void test5()
{
	int	terrors;

	DBUG_ENTER("test5");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t5", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest5read);
	vsrt_aio_test_allbutfile_write(dotest5write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest5read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	time_t	tstart, tend;
	struct	sigevent	sig;
	int	c;

	DBUG_ENTER("dotest5read");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t5read", "set up for aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*12];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_READ;
		plist[i] = &paiocbs[i];
	}
	sig.sigev_notify = SIGEV_SIGNAL;
	sig.sigev_signo = SIGUSR1;

	got_sig = 0;
	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t5read", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, &sig) == -1) {
		vsrt_sysfail("lio_listio");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	(void)time(&tend);
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t5read", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	if (got_sig != 0) {
		in_rpt("Signal was generated before any operations had completed");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t5read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t5read", "wait for the reads to complete");
	sleep(2);
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				if (got_sig != 0) {
					in_rpt("Signal was generated but operation %d is in progress", i);
					(void)aio_cancel(readfd, NULL);
					free(plist);
					free(paiocbs);
					free(bufs);
					DBUG_RETURN(-1);
				}
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	if (got_sig == 0) {
		in_rpt("All operations completed but no signal was generated");
		xx_rpt(FAILURE);
		errors++;
	}

	PATH_TRACE;
	DBUG_PRINTF("t5read", "check the results");
	memset(got_data, 0xff, 50);
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (retval != strlen(tbuf)) {
			in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
			errors++;
			continue;
		}
		if (sscanf((char *)paiocbs[i].aio_buf, "message %3d\n", &c) != 1) {
			in_rpt("Could not match returned data for request %d to \"message xxx \\n\", received \"%s\"", i, paiocbs[i].aio_buf);
			errors++;
		}
		else {
			if ((c < 0) || (c >= aio_max)) {
				in_rpt("Message number returned in request %d is %d, expected between 0 and %d", i, c, aio_max-1);
				errors++;
			} else
				if (got_data[c] != 0xff) {
					got_data[c]++;
					in_rpt("Message %d received %d times", c, got_data[c]);
					errors++;
				} else
					got_data[c]= 1;
		}
	}
	for (i = aio_max-1; i >= 0; i--) {
		if (got_data[i] == 0xff) {
			in_rpt("Message number %d not read by any request", i);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest5write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;
	time_t	tstart, tend;
	struct	sigevent	sig;
	int	fillcnt, readcnt;
	char	*null;
	FILE	*readfile;
	int	c;

	DBUG_ENTER("dotest5write");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t5write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	fillcnt = 0;
	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if ((retval = write(writefd, bbuf, sizeof(bbuf))) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
		else
			fillcnt += retval;
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t5write", "set up some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[i] = &paiocbs[i];
	}

	sig.sigev_notify = SIGEV_SIGNAL;
	sig.sigev_signo = SIGUSR1;

	got_sig = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t5write", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, &sig) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	(void)time(&tend);

	PATH_TRACE;
	DBUG_PRINTF("t5write", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	if (got_sig != 0) {
		in_rpt("Signal was generated before any operations had completed");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	PATH_TRACE;

	DBUG_PRINTF("t5write", "read all the blocking data so the writes can proceed");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
		sleep(4);
	}
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl #1");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	if ((null = malloc(fillcnt)) == NULL) {
		vsrt_sysfail("malloc null");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	readcnt = 0;
	(void)lseek(readfd, 0, SEEK_SET);
	while (readcnt != fillcnt) {
		retval = read(readfd, null, fillcnt-readcnt);
		if (retval == -1) {
			if (errno == EAGAIN) {
				sleep(1);
				continue;
			}
			vsrt_sysfail("read");
			(void)aio_cancel(writefd, NULL);
			free(null);
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
		readcnt += retval;
	}

	free(null);

	PATH_TRACE;
	DBUG_PRINTF("t5write", "wait for writes to complete");
	sleep(2);
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				if (got_sig != 0) {
					in_rpt("Signal was generated but operation %d is in progress", i);
					xx_rpt(FAILURE);
					(void)aio_cancel(writefd, NULL);
					free(plist);
					free(paiocbs);
					free(bufs);
					DBUG_RETURN(-1);
				}
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	if (got_sig == 0) {
		in_rpt("All operations completed but no signal was generated");
		xx_rpt(FAILURE);
		errors++;
	}

	PATH_TRACE;
	DBUG_PRINTF("t5write", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for write %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval != 12) {
			in_rpt("aio_return returned %d for write #%d, expected 12", retval, i);
			errors++;
		}
	}

	free(plist);
	free(bufs);
	free(paiocbs);

	if (errors != 0)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t5write", "now read the lio_listio data ");
	readfile = fdopen(readfd, "r");
	if (readfile == NULL) {
		vsrt_sysfail("fdopen");
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t5write", "check the results");
	memset(got_data, 0xff, 50);
	for (i = 0; i < aio_max; i++) {
		if (fgets(tbuf, sizeof(tbuf), readfile) == NULL) {
			err = errno;
			in_rpt("For message %d fgets() failed, errno = %d (%s)", i, err, errname(err));
			errors++;
			break;
		}
		if (strlen(tbuf) != 12) {
			in_rpt("For message %d expected to read %d bytes, received %d", i, strlen(&bufs[i*sizeof(tbuf)]), strlen(tbuf));
			errors++;
		}
		if (vsrt_relax_write_order == 0) {
			sprintf(bbuf, "message %3d\n", i);
			if (strcmp(bbuf, tbuf) != 0) {
				in_rpt("For message %d, expected returned data \"%s\", received \"%s\"", i, bbuf, tbuf);
				errors++;
			}
		} else {
			if (sscanf(tbuf, "message %3d\n", &c) != 1) {
		
				in_rpt("Could not match piece %d of data from file to \"message xxx \\n\", received \"%s\"", i, tbuf);
				errors++;
			}
			else {
				if ((c < 0) || (c >= aio_max)) {
					in_rpt("Message number in piece %d of data from file is %d, expected between 0 and %d", i, c, aio_max-1);
					errors++;
				} else
					if (got_data[c] != 0xff) {
						got_data[c]++;
						in_rpt("Message %d received %d times", c, got_data[c]);
						errors++;
					} else
						got_data[c]= 1;
			}
		}
	}
	if (vsrt_relax_write_order == 1) {
		for (i = 0; i < aio_max; i++) {
			if (got_data[i] == 0xff) {
				in_rpt("Data from request %d not found in file", i);
				errors++;
			}
		}
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when mode is LIO_NOWAIT and
	sig->sigev_notify is SIGEV_SIGNAL and the SA_SIGINFO
	flag is set for the signal specified in sig-
	>sigev_signo, the signal shall be queued for the pro-
	cess when it is generated and the value specified in
	sig->sigev_value shall be in the si_value component
	of the generated signal.
*/

private void test6()
{
	int	terrors;

	DBUG_ENTER("test6");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t6", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest6read);
	vsrt_aio_test_allbutfile_write(dotest6write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest6read(readfd, writefd)
int	readfd;
int	writefd;
{
	struct	timespec	timeout;
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	int	errors;
	time_t	tstart, tend;
	struct	sigevent	sig;
	sigset_t	set;
	siginfo_t	info;

	DBUG_ENTER("dotest6read");

	errors = 0;

	got_sig = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);

	DBUG_PRINTF("t6read", "block the signal to cause queuing");
	sigemptyset(&set);
	if (sigaddset(&set, SIGRTMIN) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}
	if (sigprocmask(SIG_BLOCK, &set, NULL) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t6read", "set up for aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*12];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_READ;
		plist[i] = &paiocbs[i];
	}
	sig.sigev_notify = SIGEV_SIGNAL;
	sig.sigev_signo = SIGRTMIN;
	sig.sigev_value.sival_int = aio_max;

	PATH_TRACE;
	DBUG_PRINTF("t6read", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, &sig) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	(void)time(&tend);
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t6read", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t6read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t6read", "wait for the reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(2);
	PATH_TRACE;
	DBUG_PRINTF("t6read", "verify signal generation and delivery");
	/*test for aio_max for the reads and another for the lio_listio*/
	/*its si_value should be aio_max*/
	timeout.tv_sec = 1;
	timeout.tv_nsec = 0;
	memset(got_requests, 0xff, 51);
	for (i = 0; i < aio_max+1; i++) {
		if (sigtimedwait(&set, &info, &timeout) == -1) {
			err = errno;
			in_rpt("sigtimedwait failed on signal #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
		else {
			/*save these as we may be getting the requests back
			out of order*/
			if (info.si_value.sival_int <= aio_max)
				got_requests[info.si_value.sival_int] = info.si_value.sival_int;
			else {
				in_rpt("si_value for signal %d = %d, maximum value in requests is %d", i, info.si_value.sival_int, aio_max);
				errors++;
			}
			if (info.si_signo != SIGRTMIN) {
				in_rpt("Expected si_signo for signal %d = %d, is %d", i, SIGRTMIN, info.si_signo);
				errors++;
			}
			if (info.si_code != SI_ASYNCIO) {
				in_rpt("Expected si_code for signal %d = %d, is %d", i, SI_ASYNCIO, info.si_code);
				errors++;
			}
		}
	}
	for (i = 0; i < aio_max+1; i++) {
		if (got_requests[i] == 0xff) {
			in_rpt("No signal received for request %d", i);
			errors++;
			continue;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest6write(readfd, writefd)
int	readfd;
int	writefd;
{
	struct	timespec	timeout;
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	errors, origflags;
	time_t	tstart, tend;
	struct	sigevent	sig;
	int	fillcnt, readcnt;
	char	*null;
	sigset_t	set;
	siginfo_t	info;

	DBUG_ENTER("dotest6write");

	errors = 0;

	got_sig = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);
	DBUG_PRINTF("t6write", "block the signal to cause queuing");
	sigemptyset(&set);
	if (sigaddset(&set, SIGRTMIN) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}
	if (sigprocmask(SIG_BLOCK, &set, NULL) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t6write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	fillcnt = 0;
	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if ((retval = write(writefd, bbuf, sizeof(bbuf))) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
		else
			fillcnt += retval;
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t6write", "set up some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[i] = &paiocbs[i];
	}

	sig.sigev_notify = SIGEV_SIGNAL;
	sig.sigev_signo = SIGRTMIN;
	sig.sigev_value.sival_int= aio_max;

	PATH_TRACE;
	DBUG_PRINTF("t6write", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, &sig) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	(void)time(&tend);

	PATH_TRACE;
	DBUG_PRINTF("t6write", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t6write", "read all the blocking data so the writes can proceed");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
		sleep(4);
	}
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl #1");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	if ((null = malloc(fillcnt)) == NULL) {
		vsrt_sysfail("malloc null");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	readcnt = 0;
	(void)lseek(readfd, 0, SEEK_SET);
	while (readcnt != fillcnt) {
		retval = read(readfd, null, fillcnt-readcnt);
		if (retval == -1) {
			if (errno == EAGAIN) {
				sleep(1);
				continue;
			}
			vsrt_sysfail("read");
			(void)aio_cancel(writefd, NULL);
			free(null);
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
		readcnt += retval;
	}

	free(null);

	PATH_TRACE;
	DBUG_PRINTF("t6write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	sleep(2);
	PATH_TRACE;
	DBUG_PRINTF("t6write", "verify signal generation and delivery");
	/*aio_max for the writes and another for the lio_listio*/
	/*its si_value should be aio_max*/
	timeout.tv_sec = 1;
	timeout.tv_nsec = 0;
	memset(got_requests, 0xff, 51);
	for (i = 0; i < aio_max+1; i++) {
		if (sigtimedwait(&set, &info, &timeout) == -1) {
			err = errno;
			in_rpt("sigtimedwait failed on signal #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
		else {
			/*save these as we may be getting the requests back
			out of order*/
			if (info.si_value.sival_int <= aio_max)
				got_requests[info.si_value.sival_int] = info.si_value.sival_int;
			else {
				in_rpt("si_value for signal %d = %d, maximum value in requests is %d", i, info.si_value.sival_int, aio_max);
				errors++;
			}
			if (info.si_signo != SIGRTMIN) {
				in_rpt("Expected si_signo for signal %d = %d, is %d", i, SIGRTMIN, info.si_signo);
				errors++;
			}
			if (info.si_code != SI_ASYNCIO) {
				in_rpt("Expected si_code for signal %d = %d, is %d", i, SI_ASYNCIO, info.si_code);
				errors++;
			}
		}
	}
	for (i = 0; i < aio_max+1; i++) {
		if (got_requests[i] == 0xff) {
			in_rpt("No signal received for request %d", i);
			errors++;
			continue;
		}
	}

	free(plist);
	free(bufs);
	free(paiocbs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_REALTIME_SIGNALS
are defined or the implementation supports the
lio_listio() function as described in System Interfaces
and Headers, Issue 5:
	On a call to lio_listio() when mode is LIO_NOWAIT and
	sig->sigev_notify is SIGEV_THREAD the function in
	sig->sigev_notify_function shall be called with the
	value of sig->sigev_value as a parameter when all the
	operations complete, executed in an environment as if
	it were the start_routine for a newly created thread
	with thread attributes specified by sig-
	>sigev_notify_attributes.
*/

private void test7()
{
	DBUG_ENTER("test7");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_REALTIME_SIGNALS
are defined or the implementation supports the
lio_listio() function as described in System Interfaces
and Headers, Issue 5:
	On a call to lio_listio() when mode is LIO_NOWAIT,
	sig->sigev_notify is SIGEV_THREAD, and sig-
	>sigev_notify_attributes is NULL, the function in
	sig->sigev_notify_function shall be called with the
	value of sig->sigev_value as a parameter when all the
	operations complete, executed in an environment as if
	it were the start_routine for a newly created thread
	with the detachstate attribute set to
	PTHREAD_CREATE_DETACHED.
*/

private void test8()
{
	DBUG_ENTER("test8");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_REALTIME_SIGNALS
are defined or the implementation supports the
lio_listio() function as described in System Interfaces
and Headers, Issue 5:
	A call to lio_listio() shall ignore NULL entries in list.
*/

private void test9()
{
	int	terrors;

	DBUG_ENTER("test9");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 26)
		aio_max = 26;
	DBUG_PRINTF("t9", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest9read);
	vsrt_aio_test_allbutfile_write(dotest9write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest9read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	time_t	tstart, tend;
	struct	sigevent	sig;
	int	expect_sig;
	int	c;

	DBUG_ENTER("dotest9read");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t9read", "set up for aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*12];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_READ;
		plist[i] = &paiocbs[i];
	}
	sig.sigev_notify = SIGEV_NONE;
	sig.sigev_signo = SIGRTMIN;

	got_sig = 0;
	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t9read", "zap some entries");
	expect_sig = aio_max;
	for (i = 1; i < aio_max; i+=2) {
		plist[i] = NULL;
		expect_sig--;
	}

	PATH_TRACE;
	DBUG_PRINTF("t9read", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, &sig) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	(void)time(&tend);
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t9read", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t9read", "write some data");
	for (i = 0; i < aio_max; i+=2) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t9read", "wait for the reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i+=2) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleeploop(4);

	PATH_TRACE;
	DBUG_PRINTF("t9read", "check the results");
	if (got_sig != expect_sig) {
		in_rpt("Expected %d signals, received %d", expect_sig, got_sig);
		errors++;
	}

	memset(got_data, 0xff, 50);
	for (i = aio_max-1; i >= 0; i--) {
		if (plist[i] != NULL) {
			retval2 = aio_error(&paiocbs[i]);
			if (retval2 == -1) {
				err = errno;
				in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			if (retval2 != 0) {
				in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
				errors++;
				continue;
			}
			retval = aio_return(&paiocbs[i]);
			if (retval == -1) {
				err = errno;
				in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			memset(tbuf, 0, sizeof(tbuf));
			sprintf(tbuf, "message %3d\n", i);
			if (retval != strlen(tbuf)) {
				in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
				errors++;
				continue;
			}
			if (sscanf((char *)paiocbs[i].aio_buf, "message %3d\n", &c) != 1) {
				in_rpt("Could not match returned data for request %d to \"message xxx \\n\", received \"%s\"", i, paiocbs[i].aio_buf);
				errors++;
			}
			else {
				if ((c < 0) || (c >= aio_max)) {
					in_rpt("Message number returned in request %d is %d, expected between 0 and %d", i, c, aio_max-1);
					errors++;
				} else
					if (got_data[c] != 0xff) {
						got_data[c]++;
						in_rpt("Message %d received %d times", c, got_data[c]);
						errors++;
					} else
						got_data[c]= 1;
			}
		}
		else
			got_data[i] = 0;
	}
	for (i = aio_max-1; i >= 0; i--) {
		if (got_data[i] == 0xff) {
			in_rpt("Message number %d not read by any request", i);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest9write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;
	time_t	tstart, tend;
	struct	sigevent	sig;
	int	fillcnt, readcnt;
	char	*null;
	FILE	*readfile;
	int	expect_sig;
	int	c;

	DBUG_ENTER("dotest9write");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t9write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	fillcnt = 0;
	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if ((retval = write(writefd, bbuf, sizeof(bbuf))) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
		else
			fillcnt += retval;
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t9write", "set up some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[i] = &paiocbs[i];
	}

	PATH_TRACE;
	DBUG_PRINTF("t9write", "zap some entries");
	expect_sig = aio_max;
	for (i = 1; i < aio_max; i+=2) {
		plist[i] = NULL;
		expect_sig--;
	}

	sig.sigev_notify = SIGEV_NONE;
	sig.sigev_signo = SIGRTMIN;

	got_sig = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);

	DBUG_PRINTF("t9write", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, &sig) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	(void)time(&tend);
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t9write", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t9write", "read all the blocking data so the writes can proceed");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
		sleep(4);
	}
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl #1");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	if ((null = malloc(fillcnt)) == NULL) {
		vsrt_sysfail("malloc null");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	readcnt = 0;
	(void)lseek(readfd, 0, SEEK_SET);
	while (readcnt != fillcnt) {
		retval = read(readfd, null, fillcnt-readcnt);
		if (retval == -1) {
			if (errno == EAGAIN) {
				sleep(1);
				continue;
			}
			vsrt_sysfail("read");
			(void)aio_cancel(writefd, NULL);
			free(null);
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
		readcnt += retval;
	}

	free(null);

	PATH_TRACE;
	DBUG_PRINTF("t9write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i+=2) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	sleeploop(4);

	PATH_TRACE;
	DBUG_PRINTF("t9write", "check the results");
	if (got_sig != expect_sig) {
		in_rpt("Expected %d signals, received %d", expect_sig, got_sig);
		errors++;
	}

	for (i = aio_max-1; i >= 0; i--) {
		if (plist[i] != NULL) {
			retval2 = aio_error(&paiocbs[i]);
			if (retval2 == -1) {
				err = errno;
				in_rpt("aio_error failed for write #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			if (retval2 != 0) {
				in_rpt("Error status for write %d is %d (%s)", i, retval2, errname(retval2));
				errors++;
				continue;
			}
			retval = aio_return(&paiocbs[i]);
			if (retval == -1) {
				err = errno;
				in_rpt("aio_return failed for write #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			if (retval != 12) {
				in_rpt("aio_return returned %d for write #%d, expected 12", retval, i);
				errors++;
			}
		}
	}

	free(plist);
	free(bufs);
	free(paiocbs);

	if (errors != 0)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t9write", "now read the lio_listio data ");
	readfile = fdopen(readfd, "r");
	if (readfile == NULL) {
		vsrt_sysfail("fdopen");
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t9write", "check the results");
	memset(got_data, 0xff, 50);
	for (i = 0; i < aio_max; i+=2) {
		if (fgets(tbuf, sizeof(tbuf), readfile) == NULL) {
			err = errno;
			in_rpt("For message %d fgets() failed, errno = %d (%s)", i, err, errname(err));
			errors++;
			break;
		}
		if (strlen(tbuf) != 12) {
			in_rpt("For message %d expected to read %d bytes, received %d", i, strlen(&bufs[i*sizeof(tbuf)]), strlen(tbuf));
			errors++;
		}
		if (vsrt_relax_write_order == 0) {
			sprintf(bbuf, "message %3d\n", i);
			if (strcmp(bbuf, tbuf) != 0) {
				in_rpt("For message %d, expected returned data \"%s\", received \"%s\"", i, bbuf, tbuf);
				errors++;
			}
		} else {
			if (sscanf(tbuf, "message %3d\n", &c) != 1) {
		
				in_rpt("Could not match piece %d of data from file to \"message xxx \\n\", received \"%s\"", i, tbuf);
				errors++;
			}
			else {
				if ((c < 0) || (c >= aio_max)) {
					in_rpt("Message number in piece %d of data from file is %d, expected between 0 and %d", i, c, aio_max-1);
					errors++;
				} else
					if (got_data[c] != 0xff) {
						got_data[c]++;
						in_rpt("Message %d received %d times", c, got_data[c]);
						errors++;
					} else
						got_data[c]= 1;
			}
		}
	}
	if (vsrt_relax_write_order == 1) {
		for (i = 0; i < aio_max; i+=2) {
			if (got_data[i] == 0xff) {
				in_rpt("Data from request %d not found in file", i);
				errors++;
			}
		}
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	A call to lio_listio() shall ignore list entries for
	which the aio_lio_opcode member is LIST_NOP.
*/

private void test10()
{
	int	terrors;

	DBUG_ENTER("test10");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 26)
		aio_max = 26;
	DBUG_PRINTF("t10", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest10read);
	vsrt_aio_test_allbutfile_write(dotest10write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest10read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	time_t	tstart, tend;
	struct	sigevent	sig;
	int	expect_sig;
	int	c;

	DBUG_ENTER("dotest10read");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t10read", "set up for aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*12];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_READ;
		plist[i] = &paiocbs[i];
	}
	sig.sigev_notify = SIGEV_NONE;
	sig.sigev_signo = SIGRTMIN;

	got_sig = 0;
	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t10read", "zap some entries");
	expect_sig = aio_max;
	for (i = 1; i < aio_max; i+=2) {
		paiocbs[i].aio_lio_opcode = LIO_NOP;
		expect_sig--;
	}

	PATH_TRACE;
	DBUG_PRINTF("t10read", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, &sig) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	(void)time(&tend);
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t10read", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t10read", "write some data");
	for (i = 0; i < aio_max; i+=2) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t10read", "wait for the reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i+=2) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleeploop(4);

	PATH_TRACE;
	DBUG_PRINTF("t10read", "check the results");
	if (got_sig != expect_sig) {
		in_rpt("Expected %d signals, received %d", expect_sig, got_sig);
		errors++;
	}

	memset(got_data, 0xff, 50);
	for (i = aio_max-1; i >= 0; i--) {
		if (paiocbs[i].aio_lio_opcode != LIO_NOP) {
			retval2 = aio_error(&paiocbs[i]);
			if (retval2 == -1) {
				err = errno;
				in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			if (retval2 != 0) {
				in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
				errors++;
				continue;
			}
			retval = aio_return(&paiocbs[i]);
			if (retval == -1) {
				err = errno;
				in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			memset(tbuf, 0, sizeof(tbuf));
			sprintf(tbuf, "message %3d\n", i);
			if (retval != strlen(tbuf)) {
				in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
				errors++;
				continue;
			}
			if (sscanf((char *)paiocbs[i].aio_buf, "message %3d\n", &c) != 1) {
		
				in_rpt("Could not match returned data for request %d to \"message xxx \\n\", received \"%s\"", i, paiocbs[i].aio_buf);
				errors++;
			}
			else {
				if ((c < 0) || (c >= aio_max)) {
					in_rpt("Message number returned in request %d is %d, expected between 0 and %d", i, c, aio_max-1);
					errors++;
				} else
					if (got_data[c] != 0xff) {
						got_data[c]++;
						in_rpt("Message %d received %d times", c, got_data[c]);
						errors++;
					} else
						got_data[c]= 1;
			}
		}
		else
			got_data[i] = 0;
	}
	for (i = aio_max-1; i >= 0; i--) {
		if (got_data[i] == 0xff) {
			in_rpt("Message number %d not read by any request", i);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest10write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;
	struct	sigevent	sig;
	int	fillcnt, readcnt;
	char	*null;
	FILE	*readfile;
	int	expect_sig;
	int	c;

	DBUG_ENTER("dotest10write");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t10write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	fillcnt = 0;
	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if ((retval = write(writefd, bbuf, sizeof(bbuf))) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
		else
			fillcnt += retval;
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	DBUG_PRINTF("t10write", "set up some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[i] = &paiocbs[i];
	}

	PATH_TRACE;
	DBUG_PRINTF("t10write", "zap some entries");
	expect_sig = aio_max;
	for (i = 1; i < aio_max; i+=2) {
		paiocbs[i].aio_lio_opcode = LIO_NOP;
		expect_sig--;
	}

	sig.sigev_notify = SIGEV_NONE;
	sig.sigev_signo = SIGRTMIN;

	got_sig = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t10read", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, &sig) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;

	DBUG_PRINTF("t10write", "read all the blocking data so the writes can proceed");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
		sleep(4);
	}
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl #1");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	if ((null = malloc(fillcnt)) == NULL) {
		vsrt_sysfail("malloc null");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	readcnt = 0;
	(void)lseek(readfd, 0, SEEK_SET);
	while (readcnt != fillcnt) {
		retval = read(readfd, null, fillcnt-readcnt);
		if (retval == -1) {
			if (errno == EAGAIN) {
				sleep(1);
				continue;
			}
			vsrt_sysfail("read");
			(void)aio_cancel(writefd, NULL);
			free(null);
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
		readcnt += retval;
	}

	free(null);

	PATH_TRACE;
	DBUG_PRINTF("t10write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i+=2) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	sleeploop(4);

	PATH_TRACE;
	DBUG_PRINTF("t10write", "check the results");
	if (got_sig != expect_sig) {
		in_rpt("Expected %d signals, received %d", expect_sig, got_sig);
		errors++;
	}

	for (i = aio_max-1; i >= 0; i--) {
		if (paiocbs[i].aio_lio_opcode != LIO_NOP) {
			retval2 = aio_error(&paiocbs[i]);
			if (retval2 == -1) {
				err = errno;
				in_rpt("aio_error failed for write #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			if (retval2 != 0) {
				in_rpt("Error status for write %d is %d (%s)", i, retval2, errname(retval2));
				errors++;
				continue;
			}
			retval = aio_return(&paiocbs[i]);
			if (retval == -1) {
				err = errno;
				in_rpt("aio_return failed for write #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			if (retval != 12) {
				in_rpt("aio_return returned %d for write #%d, expected 12", retval, i);
				errors++;
			}
		}
	}

	free(plist);
	free(bufs);
	free(paiocbs);

	if (errors != 0)
		DBUG_RETURN(-1);

	PATH_TRACE;
	DBUG_PRINTF("t10write", "now read the lio_listio data ");
	readfile = fdopen(readfd, "r");
	if (readfile == NULL) {
		vsrt_sysfail("fdopen");
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t10write", "check the results");
	memset(got_data, 0xff, 50);
	for (i = 0; i < aio_max; i+=2) {
		if (fgets(tbuf, sizeof(tbuf), readfile) == NULL) {
			err = errno;
			in_rpt("For message %d fgets() failed, errno = %d (%s)", i, err, errname(err));
			errors++;
			break;
		}
		if (strlen(tbuf) != 12) {
			in_rpt("For message %d expected to read %d bytes, received %d", i, strlen(&bufs[i*sizeof(tbuf)]), strlen(tbuf));
			errors++;
		}
		if (vsrt_relax_write_order == 0) {
			sprintf(bbuf, "message %3d\n", i);
			if (strcmp(bbuf, tbuf) != 0) {
				in_rpt("For message %d, expected returned data \"%s\", received \"%s\"", i, bbuf, tbuf);
				errors++;
			}
		} else {
			if (sscanf(tbuf, "message %3d\n", &c) != 1) {
		
				in_rpt("Could not match piece %d of data from file to \"message xxx \\n\", received \"%s\"", i, tbuf);
				errors++;
			}
			else {
				if ((c < 0) || (c >= aio_max)) {
					in_rpt("Message number in piece %d of data from file is %d, expected between 0 and %d", i, c, aio_max-1);
					errors++;
				} else
					if (got_data[c] != 0xff) {
						got_data[c]++;
						in_rpt("Message %d received %d times", c, got_data[c]);
						errors++;
					} else
						got_data[c]= 1;
			}
		}
	}
	if (vsrt_relax_write_order == 1) {
		for (i = 0; i < aio_max; i+=2) {
			if (got_data[i] == 0xff) {
				in_rpt("Data from request %d not found in file", i);
				errors++;
			}
		}
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	The addresses of the aiocb structures passed to
	lio_listio() in list shall be usable as arguments to
	aio_error() and aio_return() in order to determine
	the error and return status, respectively, of an
	operation while it is proceeding.
*/

private void test11()
{
	int	terrors;

	DBUG_ENTER("test11");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t11", "using %d requests", aio_max);

	/*just rerun test 2 as this covered therein*/
	vsrt_aio_test_allbutfile(dotest2read);
	vsrt_aio_test_allbutfile_write(dotest2write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	When list[n].aio_lio_opcode is LIO_READ a call to
	lio_listio() shall initiate or queue a request on
	behalf of the calling process to read list[n]-
	>aio_nbytes from the file associated with list[n]-
	>aio_fildes into the buffer pointed to by list[n]-
	>aio_buf.
*/

private void test12()
{
	int	terrors;

	DBUG_ENTER("test12");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t12", "using %d requests", aio_max);

	/*just rerun test 2 as this covered therein*/
	vsrt_aio_test_allbutfile(dotest2read);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO  and _POSIX_PRIORITIZED_IO are defined or
the implementation supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	A read request generated by a call to lio_listio()
	from a process whose scheduling policy is SCHED_FIFO
	or SCHED_RR shall be queued in priority order at a
	priority equal to the scheduling priority of the pro-
	cess minus list[n]->aio_reqprio for values of
	list[n]->aio_reqprio which lower the operation prior-
	ity, in the range 0 to {AIO_PRIO_DELTA_MAX}.
*/

private void test13()
{
	DBUG_ENTER("test13");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif
#ifndef _POSIX_PRIORITIZED_IO
	in_rpt("_POSIX_PRIORITIZED_IO not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN
#else

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
#endif
}

/*
If _POSIX_ASYNCHRONOUS_IO  and _POSIX_PRIORITIZED_IO are defined or
the implementation supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5 and the implementation supports
prioritized I/O on character special files:
	Read requests generated by calls to lio_listio() from
	processes whose scheduling policy is SCHED_FIFO or
	SCHED_RR with the same priority to a character spe-
	cial file shall be processed by the underlying device
	in FIFO order.
*/

private void test14()
{
	DBUG_ENTER("test14");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif
#ifndef _POSIX_PRIORITIZED_IO
	in_rpt("_POSIX_PRIORITIZED_IO not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN
#else

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
#endif
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	Unless list[n]->aio_fildes is associated with a dev-
	ice which is incapable of seeking a request generated
	by a call to lio_listio() when list[n]-
	>aio_lio_opcode is LIO_READ shall take place at the
	absolute position in the file given by list[n]-
	>aio_offset.
*/

private void test15()
{
	int	terrors;

	DBUG_ENTER("test15");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t15", "using %d requests", aio_max);

	vsrt_aio_test_file(dotest15read);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest15read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	errors;
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;

	DBUG_ENTER("dotest15read");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	DBUG_PRINTF("t15read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*sizeof(tbuf)+1))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t15read", "kick off some aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*12];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_READ;
		plist[i] = &paiocbs[i];
	}

	got_sig = 0;

	PATH_TRACE;
	DBUG_PRINTF("t15read", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, NULL) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t15read", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t15read", "check the results");
	for (i = aio_max-1; i >= 0; i--) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval != strlen(tbuf)) {
			in_rpt("Expected request %d to read %d bytes, received %d", i, strlen(tbuf), retval);
			errors++;
			continue;
		}
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (strncmp((char *)paiocbs[i].aio_buf, tbuf, strlen(tbuf)) != 0) {
			bufs[strlen(tbuf)*(i+1)] = 0;
			in_rpt("For request %d, expected returned data \"%s\", received \"%s\"", i, tbuf, paiocbs[i].aio_buf);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}


/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n]-
	>aio_lio_opcode is LIO_READ and list[n]-
	>aio_sigevent.sigev_notify is SIGEV_NONE no
	asynchronous notification shall be delivered when the
	operation completes.
*/

private void test16()
{
	int	terrors;

	DBUG_ENTER("test16");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t16", "using %d requests", aio_max);

	/*just rerun test 2 as this covered therein*/
	vsrt_aio_test_allbutfile(dotest2read);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n]-
	>aio_lio_opcode is LIO_READ and list[n]-
	>aio_sigevent.sigev_notify is SIGEV_SIGNAL the signal
	specified in list[n]->aio_sigevent.sigev_signo shall
	be generated for the process when the operation com-
	pletes.
*/

private void test17()
{
	int	terrors;

	DBUG_ENTER("test17");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t17", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest17read);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest17read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	time_t	tstart, tend;
	sigset_t	set;

	DBUG_ENTER("dotest17read");

	errors = 0;

	got_sig = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t17read", "set up for aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*12];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_READ;
		plist[i] = &paiocbs[i];
	}

	PATH_TRACE;
	DBUG_PRINTF("t17read", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, NULL) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	(void)time(&tend);
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t17read", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t17read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t17read", "wait for the reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleeploop(4);

	PATH_TRACE;
	DBUG_PRINTF("t17read", "verify signal generation");
	if (got_sig != aio_max) {
		in_rpt("Expected %d signals, received %d", aio_max, got_sig);
		errors++;
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n]-
	>aio_lio_opcode is LIO_READ and list[n]-
	>aio_sigevent.sigev_notify is SIGEV_SIGNAL and the
	SA_SIGINFO flag is set for the signal specified in
	list[n]->aio_sigevent.sigev_signo, the signal shall
	be queued for the process when it is generated and
	the value specified in list[n]->aio_sigevent.sigev_value
	shall be in the si_value component of the generated signal.
*/

private void test18()
{
	int	terrors;

	DBUG_ENTER("test18");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t18", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile(dotest18read);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest18read(readfd, writefd)
int	readfd;
int	writefd;
{
	struct	timespec	timeout;
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	int	errors;
	time_t	tstart, tend;
	struct	sigevent	sig;
	sigset_t	set;
	siginfo_t	info;

	DBUG_ENTER("dotest18read");

	errors = 0;

	got_sig = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);

	DBUG_PRINTF("t18read", "block the signal to cause queuing");
	sigemptyset(&set);
	if (sigaddset(&set, SIGRTMIN) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}
	if (sigprocmask(SIG_BLOCK, &set, NULL) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t18read", "set up for aio reads");
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*12];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_READ;
		plist[i] = &paiocbs[i];
	}
	sig.sigev_notify = SIGEV_NONE;
	sig.sigev_signo = SIGRTMIN;

	PATH_TRACE;
	DBUG_PRINTF("t18read", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, &sig) == -1) {
		vsrt_sysfail("lio_listio");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	(void)time(&tend);
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t18read", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t18read", "write some data");
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "message %3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;
	DBUG_PRINTF("t18read", "wait for the reads to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	sleep(2);
	PATH_TRACE;
	DBUG_PRINTF("t18read", "verify signal generation and delivery");
	timeout.tv_sec = 1;
	timeout.tv_nsec = 0;
	memset(got_data, 0xff, 50);
	for (i = 0; i < aio_max; i++) {
		if (sigtimedwait(&set, &info, &timeout) == -1) {
			err = errno;
			in_rpt("sigtimedwait failed on signal #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
		else {
			/*save these as we may be getting the requests back
			out of order*/
			if (info.si_value.sival_int < aio_max)
				got_requests[info.si_value.sival_int] = info.si_value.sival_int;
			else {
				in_rpt("si_value for signal %d = %d, maximum value in requests is %d", i, info.si_value.sival_int, aio_max);
				errors++;
			}
			if (info.si_signo != SIGRTMIN) {
				in_rpt("Expected si_signo for signal %d = %d, is %d", i, SIGRTMIN, info.si_signo);
				errors++;
			}
			if (info.si_code != SI_ASYNCIO) {
				in_rpt("Expected si_code for signal %d = %d, is %d", i, SI_ASYNCIO, info.si_code);
				errors++;
			}
		}
	}
	for (i = 0; i < aio_max; i++) {
		if (got_requests[i] == 0xff) {
			in_rpt("No signal received for request %d", i);
			errors++;
			continue;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n]-
	>aio_lio_opcode is LIO_READ and list[n]-
	>aio_sigevent.sigev_notify is SIGEV_THREAD the func-
	tion in list[n]->aio_sigevent.sigev_notify_function
	shall be called with the value of list[n]-
	>aio_sigevent.sigev_value as a parameter when the
	operation completes, executed in an environment as if
	it were the start_routine for a newly created thread
	with thread attributes specified by list[n]-
	>aio_sigevent.sigev_notify_attributes.
*/

private void test19()
{
	DBUG_ENTER("test19");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n]-
	>aio_lio_opcode is LIO_READ, list[n]-
	>aio_sigevent.sigev_notify is SIGEV_THREAD, and
	list[n]->aio_sigevent.sigev_notify_attributes is
	NULL, the function in list[n]-
	>aio_sigevent.sigev_notify_function shall be called
	with the value of list[n]->aio_sigevent.sigev_value
	as a parameter when the operation completes, executed
	in an environment as if it were the start_routine for
	a newly created thread with the detachstate attribute
	set to PTHREAD_CREATE_DETACHED.
*/

private void test20()
{
	DBUG_ENTER("test20");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	When list[n].aio_lio_opcode is LIO_WRITE a call to
	lio_listio() shall initiate or queue a request on
	behalf of the calling process to write list[n]-
	>aio_nbytes from the file associated with list[n]-
	>aio_fildes from the buffer pointed to by list[n]-
	>aio_buf.
*/

private void test21()
{
	int	terrors;

	DBUG_ENTER("test21");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t22", "using %d requests", aio_max);

	/*just rerun test 2 as this covered therein*/
	vsrt_aio_test_allbutfile_write(dotest2write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_PRIORITIZED_IO are defined or
the implementation supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	A write request generated by a call to lio_listio()
	from a process whose scheduling policy is SCHED_FIFO
	or SCHED_RR shall be queued in priority order at a
	priority equal to the scheduling priority of the pro-
	cess minus list[n]->aio_reqprio for values of
	list[n]->aio_reqprio which lower the operation prior-
	ity, in the range 0 to {AIO_PRIO_DELTA_MAX}.
*/

private void test22()
{
	DBUG_ENTER("test22");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif
#ifndef _POSIX_PRIORITIZED_IO
	in_rpt("_POSIX_PRIORITIZED_IO not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN
#else

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
#endif
}

/*
If _POSIX_ASYNCHRONOUS_IO and _POSIX_PRIORITIZED_IO are defined
or the implementation supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	Write requests generated by calls to lio_listio()
	from processes whose scheduling policy is SCHED_FIFO
	or SCHED_RR with the same priority to a character
	special file shall be processed by the underlying
	device in FIFO order.
*/

private void test23()
{
	DBUG_ENTER("test23");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif
#ifndef _POSIX_PRIORITIZED_IO
	in_rpt("_POSIX_PRIORITIZED_IO not defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN
#else

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
#endif
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	Unless the O_APPEND flag is set for list[n]-
	aio_fildes or that file is associated with a device
	which is incapable of seeking a request generated by
	a call to lio_listio() when list[n].aio_lio_opcode is
	LIO_WRITE shall take place at the absolute position
	in the file given by list[n]->aio_offset.
*/

private void test24()
{
	int	terrors;

	DBUG_ENTER("test24");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;
	terrors = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t24", "using %d requests", aio_max);

	vsrt_aio_test_file(dotest24write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest24write(readfd, writefd)
int	writefd;
int	readfd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	FILE	*readfile;
	struct	sigevent	sig;

	DBUG_ENTER("dotest24write");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	memset(tbuf, 0, sizeof(tbuf));

	PATH_TRACE;

	DBUG_PRINTF("t24write", "kick off some aio writes");
	for (i = aio_max-1; i >= 0; i--) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[i] = &paiocbs[i];
	}

	sig.sigev_notify = SIGEV_NONE;
	sig.sigev_signo = SIGUSR1;

	PATH_TRACE;
	DBUG_PRINTF("t24write", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, &sig) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t24write", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	readfile = fdopen(readfd, "r");
	if (readfile == NULL) {
		vsrt_sysfail("fdopen");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	if (fseek(readfile, 0, SEEK_SET) != 0) {
		vsrt_sysfail("fseek");
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t24write", "check the results");
	for (i = 0; i < aio_max; i++) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != 0) {
			in_rpt("Error status for write %d is %d (%s)", i, retval2, errname(retval2));
			errors++;
			continue;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval == -1) {
			err = errno;
			in_rpt("aio_return failed for write #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (fgets(tbuf, sizeof(tbuf), readfile) == NULL) {
			err = errno;
			in_rpt("For message %d fgets() failed, errno = %d (%s)", i, err, errname(err));
			errors++;
			break;
		}
		if (strlen(tbuf) != strlen(&bufs[i*sizeof(tbuf)])) {
			in_rpt("For message %d expected to read %d bytes, received %d", i, strlen(&bufs[i*sizeof(tbuf)]), strlen(tbuf));
			errors++;
		}
		if (strcmp(&bufs[i*sizeof(tbuf)], tbuf) != 0) {
			in_rpt("For message %d, expected returned data \"%s\", received \"%s\"", i, &bufs[sizeof(tbuf)*i], tbuf);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	If the O_APPEND flag is set for list[n]->aio_fildes
	write requests generated by calls to lio_listio()
	shall append to the file.
*/

private void test25()
{
	int	terrors;

	DBUG_ENTER("test25");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 2)
		aio_max = 2;
	DBUG_PRINTF("t25", "using %d requests", aio_max);

	vsrt_aio_test_file(dotest25write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest25write(readfd, writefd)
int	writefd;
int	readfd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	int	errors;
	FILE	*readfile;
	int	origflags;
	int	c;
	char	bbuf[1024];

	DBUG_ENTER("dotest25write");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	memset(bufs, 0, aio_max*sizeof(tbuf));

	memset(tbuf, 0, sizeof(tbuf));

	PATH_TRACE;

	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_APPEND) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t25write", "kick off some aio writes");
	for (i = 0; i < aio_max ; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = 0;
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[0] = &paiocbs[i];

		if (lseek(readfd, 0, SEEK_SET) != 0) {
			vsrt_sysfail("lseek");
			(void)aio_cancel(writefd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
		VSRT_SET_ALARM_WITH_ERROR(5)
		if (tb_lio_listio(LIO_NOWAIT, plist, 1, NULL) == -1) {
			vsrt_sysfail("lio_listio");
			(void)aio_cancel(writefd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
		VSRT_CLEAR_ALARM_WITH_ERROR
	}

	PATH_TRACE;
	DBUG_PRINTF("t25write", "wait for them to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	readfile = fdopen(readfd, "r");
	if (readfile == NULL) {
		vsrt_sysfail("fdopen");
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	if (fseek(readfile, 0, SEEK_SET) != 0) {
		vsrt_sysfail("fseek");
		free(plist);
		free(paiocbs);
		free(bufs);
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t25write", "check the results");
	memset(got_data, 0xff, 50);
	for (i = 0; i < aio_max; i++) {
		if (fgets(tbuf, sizeof(tbuf), readfile) == NULL) {
			err = errno;
			in_rpt("For file read %d fgets() failed, errno = %d (%s)", i, err, errname(err));
			errors++;
			break;
		}
		if (strlen(tbuf) != strlen(&bufs[i*sizeof(tbuf)])) {
			in_rpt("For file read %d expected to read %d bytes, received %d", i, strlen(&bufs[i*sizeof(tbuf)]), strlen(tbuf));
			errors++;
		}
		if (vsrt_relax_write_order == 0) {
			sprintf(bbuf, "message %3d\n", i);
			if (strcmp(bbuf, tbuf) != 0) {
				in_rpt("For message %d, expected returned data \"%s\", received \"%s\"", i, bbuf, tbuf);
				errors++;
			}
		} else {
			if (sscanf(tbuf, "message %3d\n", &c) != 1) {
		
				in_rpt("Could not match piece %d of data from file to \"message xxx \\n\", received \"%s\"", i, tbuf);
				errors++;
			}
			else {
				if ((c < 0) || (c >= aio_max)) {
					in_rpt("Message number in piece %d of data from file is %d, expected between 0 and %d", i, c, aio_max-1);
					errors++;
				} else
					if (got_data[c] != 0xff) {
						got_data[c]++;
						in_rpt("Message %d received %d times", c, got_data[c]);
						errors++;
					} else
						got_data[c]= 1;
			}
		}
	}
	if (vsrt_relax_write_order == 1) {
		for (i = 0; i < aio_max; i++) {
			if (got_data[i] == 0xff) {
				in_rpt("Data from request %d not found in file", i);
				errors++;
			}
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n].aio_lio_opcode
	is LIO_WRITE and list[n]->aio_sigevent.sigev_notify
	is SIGEV_NONE no asynchronous notification shall be
	delivered when the operation completes.
*/

private void test26()
{
	int	terrors;

	DBUG_ENTER("test26");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t26", "using %d requests", aio_max);

	/*just rerun test 2 as this covered therein*/
	vsrt_aio_test_allbutfile_write(dotest2write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n].aio_lio_opcode
	is LIO_WRITE and list[n]->aio_sigevent.sigev_notify
	is SIGEV_SIGNAL the signal specified in list[n]-
	>aio_sigevent.sigev_signo shall be generated for the
	process when the operation completes.
*/

private void test27()
{
	int	terrors;

	DBUG_ENTER("test27");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t27", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile_write(dotest27write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest27write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	ssize_t	retval;
	int	errors, origflags;
	int	fillcnt, readcnt;
	char	*null;

	DBUG_ENTER("dotest27write");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t27write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	fillcnt = 0;
	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if ((retval = write(writefd, bbuf, sizeof(bbuf))) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
		else
			fillcnt += retval;
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	DBUG_PRINTF("t27write", "set up some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[i] = &paiocbs[i];
	}

	got_sig = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);

	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, NULL) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;

	DBUG_PRINTF("t27write", "read all the blocking data so the writes can proceed");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
		sleep(4);
	}
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl #1");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	if ((null = malloc(fillcnt)) == NULL) {
		vsrt_sysfail("malloc null");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	readcnt = 0;
	(void)lseek(readfd, 0, SEEK_SET);
	while (readcnt != fillcnt) {
		retval = read(readfd, null, fillcnt-readcnt);
		if (retval == -1) {
			if (errno == EAGAIN) {
				sleep(1);
				continue;
			}
			vsrt_sysfail("read");
			(void)aio_cancel(writefd, NULL);
			free(null);
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
		readcnt += retval;
	}

	free(null);

	PATH_TRACE;
	DBUG_PRINTF("t27write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	sleeploop(4);

	if (got_sig != aio_max) {
		in_rpt("Expected %d signals to be generated, received %d", aio_max, got_sig);
		xx_rpt(FAILURE);
		errors++;
	}

	free(plist);
	free(bufs);
	free(paiocbs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n].aio_lio_opcode
	is LIO_WRITE and list[n]->aio_sigevent.sigev_notify
	is SIGEV_SIGNAL and the SA_SIGINFO flag is set for
	the signal specified in list[n]->aio_sigevent.sigev_signo,
	the signal shall be queued for the process when it is
	generated and the value specified in
	list[n]->aio_sigevent.sigev_value shall
	be in the si_value component of the generated signal.
*/

private void test28()
{
	int	terrors;

	DBUG_ENTER("test28");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	if (aio_max > 25)
		aio_max = 25;
	DBUG_PRINTF("t28", "using %d requests", aio_max);

	vsrt_aio_test_allbutfile_write(dotest28write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest28write(readfd, writefd)
int	readfd;
int	writefd;
{
	struct	timespec	timeout;
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	errors, origflags;
	time_t	tstart, tend;
	int	fillcnt, readcnt;
	char	*null;
	sigset_t	set;
	siginfo_t	info;

	DBUG_ENTER("dotest28write");

	errors = 0;

	got_sig = 0;

	if (vsrt_rt_signal(SIGRTMIN, rt_handler7) == -1)
		DBUG_RETURN(-1);

	DBUG_PRINTF("t28write", "block the signal to cause queuing");
	sigemptyset(&set);
	if (sigaddset(&set, SIGRTMIN) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}
	if (sigprocmask(SIG_BLOCK, &set, NULL) == -1) {
		vsrt_sysfail("sigaddset");
		DBUG_RETURN(-1);
	}

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	DBUG_PRINTF("t28write", "try and fill up the data object by writing a lot");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	fillcnt = 0;
	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if ((retval = write(writefd, bbuf, sizeof(bbuf))) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
		else
			fillcnt += retval;
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	DBUG_PRINTF("t28write", "set up some aio writes");
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_SIGNAL;
		paiocbs[i].aio_sigevent.sigev_signo = SIGRTMIN;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[i] = &paiocbs[i];
	}

	PATH_TRACE;
	DBUG_PRINTF("t28write", "lio_listio");
	VSRT_SET_ALARM_WITH_ERROR(5)
	(void)time(&tstart);
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, NULL) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	(void)time(&tend);
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;
	DBUG_PRINTF("t28write", "check it did not wait");
	if ((tend - tstart) > 2) {
		in_rpt("Expected lio_listio to not wait, waited %d seconds", tend - tstart);
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;
	DBUG_PRINTF("t28write", "read all the blocking data so the writes can proceed");
	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
		sleep(4);
	}
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl #1");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	if ((null = malloc(fillcnt)) == NULL) {
		vsrt_sysfail("malloc null");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	readcnt = 0;
	(void)lseek(readfd, 0, SEEK_SET);
	while (readcnt != fillcnt) {
		retval = read(readfd, null, fillcnt-readcnt);
		if (retval == -1) {
			if (errno == EAGAIN) {
				sleep(1);
				continue;
			}
			vsrt_sysfail("read");
			(void)aio_cancel(writefd, NULL);
			free(null);
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
		readcnt += retval;
	}

	free(null);

	PATH_TRACE;
	DBUG_PRINTF("t28write", "wait for writes to complete");
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	sleep(2);
	PATH_TRACE;
	DBUG_PRINTF("t28write", "verify signal generation and delivery");
	memset(got_requests, 0xff, 50);
	timeout.tv_sec = 1;
	timeout.tv_nsec = 0;
	for (i = 0; i < aio_max; i++) {
		if (sigtimedwait(&set, &info, &timeout) == -1) {
			err = errno;
			in_rpt("sigtimedwait failed on signal #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
		}
		else {
			/*save these as we may be getting the requests back
			out of order*/
			if (info.si_value.sival_int < aio_max)
				got_requests[info.si_value.sival_int] = info.si_value.sival_int;
			else {
				in_rpt("si_value for signal %d = %d, maximum value in requests is %d", i, info.si_value.sival_int, aio_max);
				errors++;
			}
			if (info.si_signo != SIGRTMIN) {
				in_rpt("Expected si_signo for signal %d = %d, is %d", i, SIGRTMIN, info.si_signo);
				errors++;
			}
			if (info.si_code != SI_ASYNCIO) {
				in_rpt("Expected si_code for signal %d = %d, is %d", i, SI_ASYNCIO, info.si_code);
				errors++;
			}
		}
	}
	for (i = 0; i < aio_max; i++) {
		if (got_requests[i] == 0xff) {
			in_rpt("No signal received for request %d", i);
			errors++;
			continue;
		}
	}

	free(plist);
	free(bufs);
	free(paiocbs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n].aio_lio_opcode
	is LIO_WRITE and list[n]->aio_sigevent.sigev_notify
	is SIGEV_THREAD the function in list[n]-
	>aio_sigevent.sigev_notify_function shall be called
	with the value of list[n]->aio_sigevent.sigev_value
	as a parameter when the operation completes, executed
	in an environment as if it were the start_routine for
	a newly created thread with thread attributes speci-
	fied by list[n]->aio_sigevent.sigev_notify_attributes.
*/

private void test29()
{
	DBUG_ENTER("test29");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n].aio_lio_opcode
	is LIO_WRITE, list[n]->aio_sigevent.sigev_notify is
	SIGEV_THREAD, and list[n]->aio_sigevent.sigev_notify_attributes
	is NULL, the function in list[n]->aio_sigevent.sigev_notify_function
	shall be called with the value of list[n]->aio_sigevent.sigev_value
	as a parameter when the operation completes, executed
	in an environment as if it were the start_routine for
	a newly created thread with the detachstate attribute
	set to PTHREAD_CREATE_DETACHED.
*/

private void test30()
{
	DBUG_ENTER("test30");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	xx_rpt(UNTESTED);

	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	A call to lio_listio() shall return -1 and set errno
	to EAGAIN when the number of entries indicated by
	nent would cause the systemwide limit AIO_MAX to be
	exceeded.
*/

private void test31()
{
	int	terrors;

	DBUG_ENTER("test31");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		DBUG_VOID_RETURN
	}

	errno = 0;
	if ((aio_sysmax = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			xx_rpt(UNSUPPORTED);
		} else
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
		DBUG_VOID_RETURN
	}

	vsrt_aio_test_allbutfile(dotest31);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest31(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i;
	char	tbuf[32];
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	errors;
	long	nsent;
	int	tosend;

	DBUG_ENTER("dotest31");

	errors = 0;

	if ((paiocbs = malloc((aio_sysmax+1)*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc((aio_sysmax+1)*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_sysmax+1)*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	got_sig = 0;
	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	nsent = 0;
	while (nsent < aio_sysmax) {
		if ((aio_sysmax - nsent) > aio_max)
			tosend = aio_max;
		else
			tosend = aio_sysmax - nsent;
			
		for (i = 0; i < tosend; i++) {
			paiocbs[nsent+i].aio_fildes = readfd;
			paiocbs[nsent+i].aio_buf = &bufs[(nsent+i)*12];
			paiocbs[nsent+i].aio_offset = (nsent+i)*12;
			paiocbs[nsent+i].aio_nbytes = 12;
			paiocbs[nsent+i].aio_sigevent.sigev_notify = SIGEV_NONE;
			paiocbs[nsent+i].aio_sigevent.sigev_signo = SIGUSR1;
			paiocbs[nsent+i].aio_sigevent.sigev_value.sival_int = (nsent+i);
			paiocbs[nsent+i].aio_reqprio = 0;
			paiocbs[nsent+i].aio_lio_opcode = LIO_READ;
			plist[nsent+i] = &paiocbs[nsent+i];
		}

		VSRT_SET_ALARM_WITH_ERROR(5)
		if (tb_lio_listio(LIO_NOWAIT, &plist[nsent], tosend, NULL) == -1) {
			if (errno == EAGAIN) {
				in_rpt("Exhausted system resources sending %d requests with %d already in progress. AIO_MAX = %d", tosend, nsent, aio_sysmax);
				(void)aio_cancel(readfd, NULL);
				free(plist);
				free(paiocbs);
				free(bufs);
				DBUG_RETURN(1);
			} else {
				vsrt_sysfail("lio_listio");
				(void)aio_cancel(readfd, NULL);
				free(plist);
				free(paiocbs);
				free(bufs);
				DBUG_RETURN(-1);
			}
		}
		VSRT_CLEAR_ALARM_WITH_ERROR

		nsent += tosend;
	}

	paiocbs[aio_sysmax].aio_fildes = readfd;
	paiocbs[aio_sysmax].aio_buf = &bufs[aio_sysmax*12];
	paiocbs[aio_sysmax].aio_offset = 0;
	paiocbs[aio_sysmax].aio_nbytes = 12;
	paiocbs[aio_sysmax].aio_sigevent.sigev_notify = SIGEV_NONE;
	paiocbs[aio_sysmax].aio_sigevent.sigev_signo = SIGUSR1;
	paiocbs[aio_sysmax].aio_sigevent.sigev_value.sival_int = aio_sysmax;
	paiocbs[aio_sysmax].aio_reqprio = 0;
	paiocbs[aio_sysmax].aio_lio_opcode = LIO_READ;
	plist[aio_sysmax] = &paiocbs[aio_sysmax];

	VSRT_SET_ALARM_WITH_ERROR(5)

	if (tb_lio_listio(LIO_NOWAIT, &plist[aio_sysmax], 1, NULL) != -1) {
		in_rpt("lio_listio for AIO_MAXth request succeeded");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (errno != EAGAIN) {
		vsrt_sysfail("lio_listio for AIO_MAXth request");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	(void)aio_cancel(readfd, NULL);
	free(plist);
	free(paiocbs);
	free(bufs);

	DBUG_RETURN(0);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	A call to lio_listio() shall return -1 and set errno
	to EINVAL when mode is not a proper value.
*/

private void test32()
{
	int	terrors;

	DBUG_ENTER("test32");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	vsrt_aio_test_all(dotest32);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest32(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	myaiocb;
	struct	aiocb	*plist[1];
	int	errors;

	DBUG_ENTER("dotest32");

	errors = 0;

	PATH_TRACE;

	myaiocb.aio_fildes = readfd;
	myaiocb.aio_buf = tbuf;
	myaiocb.aio_offset = 0;
	myaiocb.aio_nbytes = 1;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGRTMIN;
	myaiocb.aio_sigevent.sigev_value.sival_int = 1;
	myaiocb.aio_reqprio = 0;
	myaiocb.aio_lio_opcode = LIO_READ;
	plist[0] = &myaiocb;

	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(~(LIO_NOWAIT|LIO_WAIT), plist, 1, NULL) != -1) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	if (errno != EINVAL) {
		vsrt_sysfail("lio_listio");
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	DBUG_RETURN(0);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	A call to lio_listio() shall return -1 and set errno
	to EINVAL when the value of nent is greater than
	AIO_LISTIO_MAX.
*/

private void test33()
{
	int	terrors;

	DBUG_ENTER("test33");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max == -1)
		aio_max = 2;
	DBUG_PRINTF("t33", "using %d requests", aio_max);

	vsrt_aio_test_all_write(dotest33);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest33(readfd, writefd)
int	readfd;
int	writefd;
{
	int	pathok = 0;
	struct	aiocb	**plist;
	int	errors;

	DBUG_ENTER("dotest33");

	errors = 0;

	if ((plist = malloc((aio_max+1)*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		DBUG_RETURN(-1);
	}
	memset(plist, 0, (aio_max+1)*sizeof(struct aiocb *));

	PATH_TRACE;

	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_NOWAIT, &plist, aio_max+1, NULL) != -1) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		free(plist);
		DBUG_RETURN(-1);
	}
	if (errno != EINVAL) {
		vsrt_sysfail("lio_listio");
		free(plist);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	free(plist);
	DBUG_RETURN(0);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	A call to lio_listio() when mode is LIO_WAIT shall
	return -1 and set errno to EINTR when the operation
	is interrupted by a signal.
*/

private void test34()
{
	int	terrors;

	DBUG_ENTER("test34");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	vsrt_aio_test_allbutfile(dotest34);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest34(readfd, writefd)
int	readfd;
int	writefd;
{
	char	tbuf[32];
	struct	aiocb	myaiocb;
	struct	aiocb	*plist[1];
	int	errors;

	DBUG_ENTER("dotest34");

	errors = 0;

	myaiocb.aio_fildes = readfd;
	myaiocb.aio_buf = tbuf;
	myaiocb.aio_offset = 0;
	myaiocb.aio_nbytes = 1;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGRTMIN;
	myaiocb.aio_reqprio = 0;
	myaiocb.aio_lio_opcode = LIO_READ;
	plist[0] = &myaiocb;

	VSRT_SET_ALARM_WITH_ERROR(3)
	if (tb_lio_listio(LIO_WAIT, plist, 1, NULL) != -1) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}
	if (errno != EINTR) {
		vsrt_sysfail("lio_listio");
		DBUG_RETURN(-1);
	}
	VSRT_EXPECT_ALARM_WITH_ERROR

	DBUG_RETURN(0);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	A call to lio_listio() shall return -1 and set errno
	to EIO when one of more of the individual I/O opera-
	tions fail.
*/

private void test35()
{
	int	terrors;

	DBUG_ENTER("test35");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	if ((aio_max = sysconf(_SC_AIO_LISTIO_MAX)) == -1) {
		vsrt_sysfail("sysconf(_SC_AIO_LISTIO_MAX)");
		terrors++;
	}

	/*determine the number of requests to do, keep it reasonable*/
	aio_max = 2;

	vsrt_aio_test_allbutfile(dotest35read);
	vsrt_aio_test_allbutfile_write(dotest35write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest35read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	int	origflags;

	DBUG_ENTER("dotest35read");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	/*set O_ONBLOCK so reads will fail*/
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	/*set up for aio reads*/
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*12];
		paiocbs[i].aio_offset = i*12;
		paiocbs[i].aio_nbytes = 12;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_READ;
		plist[i] = &paiocbs[i];
	}

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_WAIT, plist, aio_max, NULL) == 0) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		errors++;
	} else
		if (errno != EIO) {
			vsrt_sysfail("lio_listio");
			errors++;
		}
	VSRT_CLEAR_ALARM_WITH_ERROR


	/*wait for the reads to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		xx_rpt(FAILURE);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	/*check the results*/
	for (i = 0; i < aio_max; i++) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != EAGAIN) {
			in_rpt("Error status for read %d is %d (%s), expected EAGAIN", i, retval2, errname(retval2));
			errors++;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval != -1) {
			err = errno;
			in_rpt("aio_return returned %d for read #%d, expected -1", retval, i);
			errors++;
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest35write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;

	DBUG_ENTER("dotest35write");

	errors = 0;

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	/*try and fill up the data object by writing a lot*/
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if ((retval = write(writefd, bbuf, sizeof(bbuf))) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	/*O_NONBLOCK still set so writes will fail*/

	/*set up some aio writes*/
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[i] = &paiocbs[i];
	}

	got_sig = 0;
	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_WAIT, plist, aio_max, NULL) == 0) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		errors++;
	} else
		if (errno != EIO) {
			vsrt_sysfail("lio_listio");
			errors++;
		}
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;

	/*wait for writes to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if (aio_error(&paiocbs[i]) == EINPROGRESS)
				in_rpt("request %d is still in progress", i);
		}
		xx_rpt(FAILURE);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	/*check the results*/
	for (i = 0; i < aio_max; i++) {
		retval2 = aio_error(&paiocbs[i]);
		if (retval2 == -1) {
			err = errno;
			in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
			errors++;
			continue;
		}
		if (retval2 != EAGAIN) {
			in_rpt("Error status for read %d is %d (%s), expected EAGAIN", i, retval2, errname(retval2));
			errors++;
		}
		retval = aio_return(&paiocbs[i]);
		if (retval != -1) {
			err = errno;
			in_rpt("aio_return returned %d for read #%d, expected -1", retval, i);
			errors++;
		}
	}

	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
	}
	free(plist);
	free(bufs);
	free(paiocbs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n]-
	>aio_lio_opcode is LIO_READ and list[n]->aio_fildes
	is not a valid file descriptor open for reading the
	return status of the asynchronous operation shall be
	set to -1 and its error status shall be set to EBADF.
*/

private void test36()
{
	int	terrors, retval;

	DBUG_ENTER("test36");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	retval = vsrt_forkatest(dotest36, 0, 0);
	if (retval == -1)
		vsrt_aio_test_fail++;
	if (retval == 1)
		vsrt_aio_test_unsup++;
	if (retval == 0)
		vsrt_aio_test_pass++;

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest36(readfd, writefd)
int	readfd;
int	writefd;
{
	int	j;
	char	tbuf[32];
	struct	aiocb	myaiocb;
	struct	aiocb	*plist[1];
	ssize_t	retval;
	int	retval2;
	int	errors;
	int	fd;
	int	err;

	DBUG_ENTER("dotest36");

	errors = 0;

	if ((fd = open("vsrt_lio_listio", O_RDWR|O_CREAT, VSRT_PROT_ALL)) == -1) {
		vsrt_sysfail("open");
		DBUG_RETURN(-1);
	}

	(void)close(fd);
	(void)unlink("vsrt_lio_listio");

	myaiocb.aio_fildes = fd+500;
	myaiocb.aio_buf = tbuf;
	myaiocb.aio_offset = 0;
	myaiocb.aio_nbytes = 1;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGRTMIN;
	myaiocb.aio_reqprio = 0;
	myaiocb.aio_lio_opcode = LIO_READ;
	plist[0] = &myaiocb;

	VSRT_SET_ALARM_WITH_ERROR(3)
	if (tb_lio_listio(LIO_WAIT, plist, 1, NULL) != -1) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		errors++;
	} else
		if (errno != EIO) {
			vsrt_sysfail("lio_listio");
			errors++;
		}
	VSRT_CLEAR_ALARM_WITH_ERROR

	/*wait for the read to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if (aio_error(&myaiocb) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("request is still in progress");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	/*check the results*/
	retval2 = aio_error(&myaiocb);
	if (retval2 == -1) {
		err = errno;
		in_rpt("aio_error failed for read, errno = %d (%s)", err, errname(err));
		errors++;
	} else
		if (retval2 != EBADF) {
			in_rpt("Error status for read is %d (%s), expected EBADF", retval2, errname(retval2));
			errors++;
		}
	retval = aio_return(&myaiocb);
	if (retval != -1) {
		err = errno;
		in_rpt("aio_return returned %d for read, expected -1", retval);
		errors++;
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}


/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n]-
	>aio_lio_opcode is LIO_READ and the offset implied by
	list[n]->aio_offset would be invalid the return
	status of the asynchronous operation shall be set to
	-1 and its error status shall be set to EINVAL.
*/

private void test37()
{
	int	terrors;

	DBUG_ENTER("test37");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	vsrt_aio_test_file(dotest37);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest37(readfd, writefd)
int	readfd;
int	writefd;
{
	int	j;
	char	tbuf[32];
	struct	aiocb	myaiocb;
	struct	aiocb	*plist[1];
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	int	origflags;

	DBUG_ENTER("dotest37");

	errors = 0;

	/*set O_ONBLOCK so reads won't hang up*/
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		DBUG_RETURN(-1);
	}

	myaiocb.aio_fildes = readfd;
	myaiocb.aio_buf = tbuf;
	/*invalid offset*/
	myaiocb.aio_offset = -1;
	myaiocb.aio_nbytes = 1;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGRTMIN;
	myaiocb.aio_reqprio = 0;
	myaiocb.aio_lio_opcode = LIO_READ;
	plist[0] = &myaiocb;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_WAIT, plist, 1, NULL) == 0) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		errors++;
	} else
		if (errno != EIO) {
			vsrt_sysfail("lio_listio");
			errors++;
		}
	VSRT_CLEAR_ALARM_WITH_ERROR

	/*wait for the read to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if (aio_error(&myaiocb) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("request is still in progress");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	/*check the results*/
	retval2 = aio_error(&myaiocb);
	if (retval2 == -1) {
		err = errno;
		in_rpt("aio_error failed for read, errno = %d (%s)", err, errname(err));
		errors++;
	} else
		if (retval2 != EINVAL) {
			in_rpt("Error status for read is %d (%s), expected EINVAL", retval2, errname(retval2));
			errors++;
		}
	retval = aio_return(&myaiocb);
	if (retval != -1) {
		err = errno;
		in_rpt("aio_return returned %d for read, expected -1", retval);
		errors++;
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n]-
	>aio_lio_opcode is LIO_READ and list[n]->aio_reqprio
	is not a valid value the return status of the asyn-
	chronous operation shall be set to -1 and its error
	status shall be set to EINVAL.
*/

private void test38()
{
	int	terrors;

	DBUG_ENTER("test38");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	vsrt_aio_test_all(dotest38);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest38(readfd, writefd)
int	readfd;
int	writefd;
{
	int	j;
	char	tbuf[32];
	struct	aiocb	myaiocb;
	struct	aiocb	*plist[1];
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	int	origflags;

	DBUG_ENTER("dotest38");

	errors = 0;

	/*set O_ONBLOCK so reads won't hang up*/
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		DBUG_RETURN(-1);
	}

	myaiocb.aio_fildes = readfd;
	myaiocb.aio_buf = tbuf;
	myaiocb.aio_offset = 0;
	myaiocb.aio_nbytes = 1;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGRTMIN;
	/*invalid priority*/
	myaiocb.aio_reqprio = -1;
	myaiocb.aio_lio_opcode = LIO_READ;
	plist[0] = &myaiocb;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_WAIT, plist, 1, NULL) == 0) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		errors++;
	} else
		if (errno != EIO) {
			vsrt_sysfail("lio_listio");
			errors++;
		}
	VSRT_CLEAR_ALARM_WITH_ERROR

	/*wait for the read to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if (aio_error(&myaiocb) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("request is still in progress");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	/*check the results*/
	retval2 = aio_error(&myaiocb);
	if (retval2 == -1) {
		err = errno;
		in_rpt("aio_error failed for read, errno = %d (%s)", err, errname(err));
		errors++;
	} else
		if (retval2 != EINVAL) {
			in_rpt("Error status for read is %d (%s), expected EINVAL", retval2, errname(retval2));
			errors++;
		}
	retval = aio_return(&myaiocb);
	if (retval != -1) {
		err = errno;
		in_rpt("aio_return returned %d for read, expected -1", retval);
		errors++;
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

size_t	aio_nbytes_invalid;

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	When list[n]->aio_lio_opcode is LIO_READ and
	list[n]->aio_nbytes is not a valid value the return
	status of the asynchronous operation shall be set to
	-1 and its error status shall be set to EINVAL.
*/

private void test39()
{
	int	terrors;
	char	*str;

	DBUG_ENTER("test39");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	/*user needs to supply an invalid value, if there is one*/
	str = tet_getvar("VSRT_INVALID_AIO_NBYTES_READ");
	if ((str != NULL) && (strlen(str) != 0))
		aio_nbytes_invalid = (size_t)atol(str);
	else {
		in_rpt("No VSRT_INVALID_AIO_NBYTES_READ value defined in tetexec.cfg");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}

	vsrt_aio_test_all(dotest39);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest39(readfd, writefd)
int	readfd;
int	writefd;
{
	int	j;
	char	tbuf[32];
	struct	aiocb	myaiocb;
	struct	aiocb	*plist[1];
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	int	origflags;

	DBUG_ENTER("dotest39");

	errors = 0;

	/*set O_ONBLOCK so reads won't hang up*/
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		DBUG_RETURN(-1);
	}

	myaiocb.aio_fildes = readfd;
	myaiocb.aio_buf = tbuf;
	myaiocb.aio_offset = 0;
	myaiocb.aio_nbytes = aio_nbytes_invalid;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGRTMIN;
	myaiocb.aio_reqprio = 0;
	myaiocb.aio_lio_opcode = LIO_READ;
	plist[0] = &myaiocb;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_WAIT, plist, 1, NULL) == 0) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		errors++;
	} else
		if (errno != EIO) {
			vsrt_sysfail("lio_listio");
			errors++;
		}
	VSRT_CLEAR_ALARM_WITH_ERROR

	/*wait for the read to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if (aio_error(&myaiocb) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("request is still in progress");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	/*check the results*/
	retval2 = aio_error(&myaiocb);
	if (retval2 == -1) {
		err = errno;
		in_rpt("aio_error failed for read, errno = %d (%s)", err, errname(err));
		errors++;
	} else
		if (retval2 != EINVAL) {
			in_rpt("With aio_nbytes = %ld, error status for read is %d (%s), expected EINVAL", (long)aio_nbytes_invalid, retval2, errname(retval2));
			errors++;
		}
	retval = aio_return(&myaiocb);
	if (retval != -1) {
		err = errno;
		in_rpt("With aio_nbytes = %ld, aio_return returned %d for read, expected -1", (long)aio_nbytes_invalid, retval);
		errors++;
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n].aio_lio_opcode
	is LIO_WRITE and list[n]->aio_fildes is not a valid
	file descriptor open for writing the return status of
	the asynchronous operation shall be set to -1 and its
	error status shall be set to EBADF.
*/

private void test40()
{
	int	terrors, retval;

	DBUG_ENTER("test40");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	retval = vsrt_forkatest(dotest40, 0, 0);
	if (retval == -1)
		vsrt_aio_test_fail++;
	if (retval == 1)
		vsrt_aio_test_unsup++;
	if (retval == 0)
		vsrt_aio_test_pass++;

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest40(readfd, writefd)
int	readfd;
int	writefd;
{
	int	j;
	char	tbuf[32];
	struct	aiocb	myaiocb;
	struct	aiocb	*plist[1];
	ssize_t	retval;
	int	retval2;
	int	errors;
	int	fd;
	int	err;

	DBUG_ENTER("dotest40");

	errors = 0;

	if ((fd = open("vsrt_lio_listio", O_RDWR|O_CREAT, VSRT_PROT_ALL)) == -1) {
		vsrt_sysfail("open");
		DBUG_RETURN(-1);
	}

	(void)close(fd);
	(void)unlink("vsrt_lio_listio");

	myaiocb.aio_fildes = fd+500;
	myaiocb.aio_buf = tbuf;
	myaiocb.aio_offset = 0;
	myaiocb.aio_nbytes = 1;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGRTMIN;
	myaiocb.aio_reqprio = 0;
	myaiocb.aio_lio_opcode = LIO_WRITE;
	plist[0] = &myaiocb;

	VSRT_SET_ALARM_WITH_ERROR(3)
	if (tb_lio_listio(LIO_WAIT, plist, 1, NULL) != -1) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		errors++;
	} else
		if (errno != EIO) {
			vsrt_sysfail("lio_listio");
			errors++;
		}
	VSRT_CLEAR_ALARM_WITH_ERROR

	/*wait for the write to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if (aio_error(&myaiocb) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("request is still in progress");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	/*check the results*/
	retval2 = aio_error(&myaiocb);
	if (retval2 == -1) {
		err = errno;
		in_rpt("aio_error failed for write, errno = %d (%s)", err, errname(err));
		errors++;
	} else
		if (retval2 != EBADF) {
			in_rpt("Error status for write is %d (%s), expected EBADF", retval2, errname(retval2));
			errors++;
		}
	retval = aio_return(&myaiocb);
	if (retval != -1) {
		err = errno;
		in_rpt("aio_return returned %d for write, expected -1", retval);
		errors++;
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n].aio_lio_opcode
	is LIO_WRITE and the offset implied by list[n]-
	>aio_offset would be invalid the return status of the
	asynchronous operation shall be set to -1 and its
	error status shall be set to EINVAL.
*/

private void test41()
{
	int	terrors;

	DBUG_ENTER("test41");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	vsrt_aio_test_file(dotest41);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest41(readfd, writefd)
int	readfd;
int	writefd;
{
	int	j;
	char	tbuf[32];
	struct	aiocb	myaiocb;
	struct	aiocb	*plist[1];
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;

	DBUG_ENTER("dotest41");

	errors = 0;

	myaiocb.aio_fildes = readfd;
	myaiocb.aio_buf = tbuf;
	/*invalid offset*/
	myaiocb.aio_offset = -1;
	myaiocb.aio_nbytes = 1;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGRTMIN;
	myaiocb.aio_reqprio = 0;
	myaiocb.aio_lio_opcode = LIO_WRITE;
	plist[0] = &myaiocb;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_WAIT, plist, 1, NULL) == 0) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		errors++;
	} else
		if (errno != EIO) {
			vsrt_sysfail("lio_listio");
			errors++;
		}
	VSRT_CLEAR_ALARM_WITH_ERROR

	/*wait for the write to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if (aio_error(&myaiocb) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("request is still in progress");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	/*check the results*/
	retval2 = aio_error(&myaiocb);
	if (retval2 == -1) {
		err = errno;
		in_rpt("aio_error failed for write, errno = %d (%s)", err, errname(err));
		errors++;
	} else
		if (retval2 != EINVAL) {
			in_rpt("Error status for write is %d (%s), expected EINVAL", retval2, errname(retval2));
			errors++;
		}
	retval = aio_return(&myaiocb);
	if (retval != -1) {
		err = errno;
		in_rpt("aio_return returned %d for write, expected -1", retval);
		errors++;
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementa-
tion supports the lio_listio() function as described in
System Interfaces and Headers, Issue 5:
	On a call to lio_listio() when list[n].aio_lio_opcode
	is LIO_WRITE and list[n]->aio_reqprio is not a valid
	value the return status of the asynchronous operation
	shall be set to -1 and its error status shall be set
	to EINVAL.
*/

private void test42()
{
	int	terrors;

	DBUG_ENTER("test42");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	vsrt_aio_test_all(dotest42);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}

private int dotest42(readfd, writefd)
int	readfd;
int	writefd;
{
	int	j;
	char	tbuf[32];
	struct	aiocb	myaiocb;
	struct	aiocb	*plist[1];
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;

	DBUG_ENTER("dotest42");

	errors = 0;

	myaiocb.aio_fildes = readfd;
	myaiocb.aio_buf = tbuf;
	myaiocb.aio_offset = 0;
	myaiocb.aio_nbytes = 1;
	myaiocb.aio_sigevent.sigev_notify = SIGEV_NONE;
	myaiocb.aio_sigevent.sigev_signo = SIGRTMIN;
	/*invalid priority*/
	myaiocb.aio_reqprio = -1;
	myaiocb.aio_lio_opcode = LIO_WRITE;
	plist[0] = &myaiocb;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_WAIT, plist, 1, NULL) == 0) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		errors++;
	} else
		if (errno != EIO) {
			vsrt_sysfail("lio_listio");
			errors++;
		}
	VSRT_CLEAR_ALARM_WITH_ERROR

	/*wait for the write to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if (aio_error(&myaiocb) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("request is still in progress");
		xx_rpt(FAILURE);
		DBUG_RETURN(-1);
	}

	/*check the results*/
	retval2 = aio_error(&myaiocb);
	if (retval2 == -1) {
		err = errno;
		in_rpt("aio_error failed for write, errno = %d (%s)", err, errname(err));
		errors++;
	} else
		if (retval2 != EINVAL) {
			in_rpt("Error status for write is %d (%s), expected EINVAL", retval2, errname(retval2));
			errors++;
		}
	retval = aio_return(&myaiocb);
	if (retval != -1) {
		err = errno;
		in_rpt("aio_return returned %d for write, expected -1", retval);
		errors++;
	}

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private void test43()
{
	DBUG_ENTER("test43");

	xx_rpt(NOTINUSE);
	DBUG_VOID_RETURN
}

/*
If _POSIX_ASYNCHRONOUS_IO is defined or the implementation supports
the lio_listio() function as described in System Interfaces and
Headers, Issue 5:
	On a call to lio_listio() when a requested operation is canceled by
	a call to aio_cancel() the return status of the asynchronous opera-
	tion shall be set to -1 and its error status shall be set to ECANCELED.
*/

private void test44()
{
	int	terrors;

	DBUG_ENTER("test44");

#ifndef _POSIX_ASYNCHRONOUS_IO
	if (!supports_lio_listio) {
		in_rpt("lio_listio not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_return) {
		in_rpt("aio_return not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_error) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (!supports_aio_cancel) {
		in_rpt("aio_error not supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
#endif

	terrors = 0;
	vsrt_aio_test_fail = vsrt_aio_test_pass = vsrt_aio_test_unsup = 0;

	errno = 0;
	if ((aio_max = sysconf(_SC_AIO_MAX)) == -1) {
		if (errno == 0) {
			in_rpt("INFO: No limit on AIO_MAX");
			aio_max = LONG_MAX;
		} else {
			vsrt_sysfail("sysconf(_SC_AIO_MAX)");
			DBUG_VOID_RETURN
		}
	}

	/*determine the number of requests to do, keep it reasonable*/
	if (aio_max > 25)
		aio_max = 25;

	vsrt_aio_test_allbutfile(dotest44read);
	vsrt_aio_test_allbutfile_write(dotest44write);

	if ((vsrt_aio_test_fail != 0) || (terrors != 0)) {
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (vsrt_aio_test_pass != 0) {
		xs_rpt();
		DBUG_VOID_RETURN
	}
	xx_rpt(UNSUPPORTED);

	DBUG_VOID_RETURN
}


private int dotest44read(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors;
	int	cancelno;

	DBUG_ENTER("dotest44read");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc((aio_max*2)+1)) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	/*kick off some aio reads*/
	for (i = 0; i < aio_max; i++) {
		paiocbs[i].aio_fildes = readfd;
		paiocbs[i].aio_buf = &bufs[i*2];
		paiocbs[i].aio_offset = i*2;
		paiocbs[i].aio_nbytes = 2;
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_READ;
		plist[i] = &paiocbs[i];
	}

	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, NULL) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	sleep(2);
	/*ensure they are not complete (as no data is available yet)*/
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}

	if (errors != 0) {
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	if (aio_max < 2)
		cancelno = 0;
	else
		cancelno = aio_max-2;

	retval2 = aio_cancel(readfd, &paiocbs[cancelno]);
	/*may not be cancelled, this is OK*/
	if (retval2 == AIO_NOTCANCELED) {
		in_rpt("aio_cancel returned AIO_NOTCANCELED for read request");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(1);
	}
	if (retval2 == AIO_ALLDONE) {
		in_rpt("aio_cancel returned AIO_ALLDONE for read request");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 == -1) {
		vsrt_sysfail("aio_cancel for read request");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 != AIO_CANCELED) {
		in_rpt("aio_cancel returned %d for read request, expected AIO_CANCELED", retval2);
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	/*wait for the cancelled one to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if (aio_error(&paiocbs[cancelno]) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("cancelled read request still in progress");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	/*check its error and return status*/
	retval2 = aio_error(&paiocbs[cancelno]);
	if (retval2 == -1) {
		err = errno;
		in_rpt("aio_error failed for canceled read request, errno = %d (%s)", err, errname(err));
		errors++;
	} else
		if (retval2 != ECANCELED) {
			in_rpt("Error status for canceled read request is %d (%s), expected ECANCELED", retval2, errname(retval2));
			errors++;
		}
	retval = aio_return(&paiocbs[cancelno]);
	if (retval != -1) {
		err = errno;
		in_rpt("aio_return returned %d for canceled read request, expected -1", retval);
		errors++;
	}

	/*write some data*/
	for (i = 0; i < aio_max; i++) {
		memset(tbuf, 0, sizeof(tbuf));
		sprintf(tbuf, "%3d\n", i);
		if (write(writefd, tbuf, strlen(tbuf)) == -1) {
			vsrt_sysfail("write");
			(void)aio_cancel(readfd, NULL);
			free(plist);
			free(paiocbs);
			free(bufs);
			DBUG_RETURN(-1);
		}
	}

	PATH_TRACE;

	/*wait for the others to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if ((i != cancelno) && (aio_error(&paiocbs[i]) == EINPROGRESS)) {
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if ((i != cancelno) && (aio_error(&paiocbs[i]) == EINPROGRESS))
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	/*check the results of the others*/
	for (i = aio_max-1; i >= 0; i--) {
		if (i != cancelno) {
			retval2 = aio_error(&paiocbs[i]);
			if (retval2 == -1) {
				err = errno;
				in_rpt("aio_error failed for read #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
			if (retval2 != 0) {
				in_rpt("Error status for read %d is %d (%s)", i, retval2, errname(retval2));
				errors++;
				continue;
			}
			retval = aio_return(&paiocbs[i]);
			if (retval == -1) {
				err = errno;
				in_rpt("aio_return failed for read #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}

private int dotest44write(readfd, writefd)
int	readfd;
int	writefd;
{
	int	i, j;
	char	tbuf[32];
	char	bbuf[1024];
	int	pathok = 0;
	struct	aiocb	*paiocbs;
	struct	aiocb	**plist;
	char	*bufs;
	int	err;
	ssize_t	retval;
	int	retval2;
	int	errors, origflags;
	int	cancelno;

	DBUG_ENTER("dotest44write");

	errors = 0;

	if (vsrt_signal(SIGUSR1, handler5) == -1)
		DBUG_RETURN(-1);

	if ((paiocbs = malloc(aio_max*sizeof(struct aiocb))) == NULL) {
		vsrt_sysfail("malloc aiocbs");
		DBUG_RETURN(-1);
	}

	if ((plist = malloc(aio_max*sizeof(struct aiocb *))) == NULL) {
		vsrt_sysfail("malloc plist");
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if ((bufs = malloc(aio_max*sizeof(tbuf))) == NULL) {
		vsrt_sysfail("malloc bufs");
		free(plist);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	/*try and fill up the data object by writing a lot*/
	if (isatty(readfd)) {
		if (tcflow(readfd, TCIOFF) == -1) {
			vsrt_sysfail("tcflow(TCIOFF)");
			free(plist);
			free(bufs);
			free(paiocbs);
			DBUG_RETURN(-1);
		}
	}
	if ((origflags = fcntl(writefd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(writefd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #2");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	bbuf[sizeof(bbuf)-1] = '\n';
	for (i = 0; i < 1024; i++) {
		if (write(writefd, bbuf, sizeof(bbuf)) == -1) {
			if (errno == EAGAIN)
				break;
			vsrt_sysfail("write");
			DBUG_RETURN(-1);
		}
	}

	/*unsupported as cannot fill*/
	if (i == 1024) {
		in_rpt("Cannot fill");
		DBUG_RETURN(1);	
	}

	PATH_TRACE;

	if (fcntl(writefd, F_SETFL, origflags) == -1) {
		vsrt_sysfail("fcntl #3");
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}

	/*kick off some aio writes*/
	for (i = 0; i < aio_max; i++) {
		sprintf(&bufs[i*sizeof(tbuf)], "message %3d\n", i);
		paiocbs[i].aio_fildes = writefd;
		paiocbs[i].aio_buf = &bufs[i*sizeof(tbuf)];
		paiocbs[i].aio_offset = i*strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_nbytes = strlen(&bufs[i*sizeof(tbuf)]);
		paiocbs[i].aio_sigevent.sigev_notify = SIGEV_NONE;
		paiocbs[i].aio_sigevent.sigev_signo = SIGUSR1;
		paiocbs[i].aio_sigevent.sigev_value.sival_int = i;
		paiocbs[i].aio_reqprio = 0;
		paiocbs[i].aio_lio_opcode = LIO_WRITE;
		plist[i] = &paiocbs[i];
	}

	VSRT_SET_ALARM_WITH_ERROR(5)
	if (tb_lio_listio(LIO_NOWAIT, plist, aio_max, NULL) == -1) {
		vsrt_sysfail("lio_listio");
		(void)aio_cancel(readfd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	VSRT_CLEAR_ALARM_WITH_ERROR

	PATH_TRACE;

	/*ensure they are not complete (should block)*/
	for (i = 0; i < aio_max; i++) {
		if ((retval2 = aio_error(&paiocbs[i])) != EINPROGRESS) {
			in_rpt("request %d is not in progress, its error status is %d (%s)", i, retval2, errname(retval2));
			errors++;
		}
	}
	if (errors != 0) {
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	/*cancel one*/
	if (aio_max < 2)
		cancelno = 0;
	else
		cancelno = aio_max-2;

	retval2 = aio_cancel(writefd, &paiocbs[cancelno]);
	/*may not be cancelled, this is OK (UNSUPPORTED)*/
	if (retval2 == AIO_NOTCANCELED) {
		in_rpt("aio_cancel returned AIO_NOTCANCELED for write request");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		DBUG_RETURN(1);
	}
	if (retval2 == AIO_ALLDONE) {
		in_rpt("aio_cancel returned AIO_ALLDONE for write request");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 == -1) {
		vsrt_sysfail("aio_cancel for write request");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}
	if (retval2 != AIO_CANCELED) {
		in_rpt("aio_cancel returned %d for write request, expected AIO_CANCELED", retval2);
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	PATH_TRACE;

	/*wait for the cancelled one to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		if (aio_error(&paiocbs[cancelno]) == EINPROGRESS)
			sleep(2);
		else
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		in_rpt("cancelled write request still in progress");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	/*check its error and return status*/
	retval2 = aio_error(&paiocbs[cancelno]);
	if (retval2 == -1) {
		err = errno;
		in_rpt("aio_error failed for canceled write request, errno = %d (%s)", err, errname(err));
		errors++;
	} else
		if (retval2 != ECANCELED) {
			in_rpt("Error status for canceled write request is %d (%s), expected ECANCELED", retval2, errname(retval2));
			errors++;
		}
	retval = aio_return(&paiocbs[cancelno]);
	if (retval != -1) {
		err = errno;
		in_rpt("aio_return returned %d for canceled write request, expected -1", retval);
		errors++;
	}

	PATH_TRACE;

	if (isatty(readfd)) {
		if (tcflow(readfd, TCION) == -1)
			vsrt_sysfail("tcflow(TCION)");
		sleep(4);
	}
	if ((origflags = fcntl(readfd, F_GETFL)) == -1) {
		vsrt_sysfail("fcntl #3");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	if (fcntl(readfd, F_SETFL, origflags|O_NONBLOCK) == -1) {
		vsrt_sysfail("fcntl #4");
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(bufs);
		free(paiocbs);
		DBUG_RETURN(-1);
	}
	/* Read some data, so the blocked writes can unblock.
	   This reads as much as is available now, but more
	   may arrive later. */
	while (read(readfd, bbuf, sizeof(bbuf)) > 0)
		;

	PATH_TRACE;

	/*wait for writes to complete*/
	for (j = 0; j < VSRT_AIO_ITERATIONS; j++) {
		for (i = 0; i < aio_max; i++) {
			if ((i != cancelno) && (aio_error(&paiocbs[i]) == EINPROGRESS)) {
				/* One of the writes is still in progress.
				   Read more data, in case it has blocked. */
				while (read(readfd, bbuf, sizeof(bbuf)) > 0)
					;
				sleep(2);
				break;
			}
		}
		if (i == aio_max)
			break;
	}

	if (j == VSRT_AIO_ITERATIONS) {
		for (i = 0; i < aio_max; i++) {
			if ((i != cancelno) && (aio_error(&paiocbs[i]) == EINPROGRESS))
				in_rpt("request %d is still in progress", i);
		}
		(void)aio_cancel(writefd, NULL);
		free(plist);
		free(paiocbs);
		free(bufs);
		DBUG_RETURN(-1);
	}

	/*check the results*/
	for (i = aio_max-1; i >= 0; i--) {
		if (i != cancelno) {
			retval2 = aio_error(&paiocbs[i]);
			if (retval2 == -1) {
				err = errno;
				in_rpt("aio_error failed for write #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
			} else
				if (retval2 != 0) {
					in_rpt("Error status for write %d is %d (%s)", i, retval2, errname(retval2));
					errors++;
					continue;
				}
			retval = aio_return(&paiocbs[i]);
			if (retval == -1) {
				err = errno;
				in_rpt("aio_return failed for write #%d, errno = %d (%s)", i, err, errname(err));
				errors++;
				continue;
			}
		}
	}

	free(plist);
	free(paiocbs);
	free(bufs);

	if (errors == 0)
		DBUG_RETURN(0);
	else
		DBUG_RETURN(-1);
}


/*
If _POSIX_ASYNCHRONOUS_IO is not defined and the imple-
mentation does not support the lio_listio() function as
described in System Interfaces and Headers, Issue 5:
	A call to lio_listio() shall return -1 and set errno
	to ENOSYS.
*/

private void test45()
{
	DBUG_ENTER("test45");

#ifdef _POSIX_ASYNCHRONOUS_IO
	in_rpt("_POSIX_ASYNCHRONOUS_IO defined");
	xx_rpt(UNSUPPORTED);
	DBUG_VOID_RETURN
#else
	if (supports_lio_listio) {
		in_rpt("lio_listio supported");
		xx_rpt(UNSUPPORTED);
		DBUG_VOID_RETURN
	}
	if (tb_lio_listio(0, NULL, 0, NULL) != -1) {
		in_rpt("lio_listio succeeded");
		xx_rpt(FAILURE);
		DBUG_VOID_RETURN
	}
	if (errno != ENOSYS) {
		vsrt_sysfail("lio_listio");
		DBUG_VOID_RETURN
	}

	xs_rpt();

	DBUG_VOID_RETURN
#endif
}


private	int tb_lio_listio(mode, list, nent, sig)
int	mode;
struct aiocb	*list[];
int	nent;
struct sigevent	*sig;
{
	int 	retval;

	retval = lio_listio(mode, (struct aiocb *const *)list, nent, sig);
#ifdef	TESTBED
	errno++;
	return -1;
#else
	return retval;
#endif
}
